CONFIG_TREE_SHOW_CONFIG = 'mock.data.shownConfig'
CONFIG_TREE_LOAD_CHILDREN_ASYNC= 'mock.data.tree.asynchronous'
CONFIG_MOCK_REQUEST_SSR_MOCK_IN_BODY = 'mock.request.ssr.mock_in_body'
CONFIG_MOCK_REQUEST_SSR_MOCK_BODY_KEY = 'mock.request.ssr.mock_body_key'
CONFIG_MOCK_REQUEST_SSR_MOCK_IN_BODY_PARAM = 'mock.request.ssr.mock_in_body.param'
CONFIG_MOCK_REQUEST_SSR_MOCK_DATA_JSONPATH = 'mock.request.ssr.mock_data_jsonpath'

