(function(e){var n={};function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{enumerable:!0,get:r})},t.r=function(e){"undefined"!==typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},t.t=function(e,n){if(1&n&&(e=t(e)),8&n)return e;if(4&n&&"object"===typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(t.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&n&&"string"!=typeof e)for(var i in e)t.d(r,i,function(n){return e[n]}.bind(null,i));return r},t.n=function(e){var n=e&&e.__esModule?function(){return e["default"]}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,n){return Object.prototype.hasOwnProperty.call(e,n)},t.p="/ui/static/",t(t.s="d0c7")})({"30db":function(e,n,t){"use strict";(function(e,r){var i;t.d(n,"a",function(){return y}),t.d(n,"d",function(){return E}),t.d(n,"b",function(){return T}),t.d(n,"c",function(){return S}),t.d(n,"e",function(){return x});const a="en";let o=!1,s=!1,c=!1,l=!1,u=!1,d=!1,p=!1,f=void 0,m=a,_=void 0,g=void 0;const y="object"===typeof self?self:"object"===typeof e?e:{};let h=void 0;"undefined"!==typeof y.vscode&&"undefined"!==typeof y.vscode.process?h=y.vscode.process:"undefined"!==typeof r&&(h=r);const v="string"===typeof(null===(i=null===h||void 0===h?void 0:h.versions)||void 0===i?void 0:i.electron)&&"renderer"===h.type;if("object"!==typeof navigator||v)if("object"===typeof h){o="win32"===h.platform,s="darwin"===h.platform,c="linux"===h.platform,l=c&&!!Object({NODE_ENV:"production",BASE_URL:"/ui/static/"})["SNAP"]&&!!Object({NODE_ENV:"production",BASE_URL:"/ui/static/"})["SNAP_REVISION"],f=a,m=a;const e=Object({NODE_ENV:"production",BASE_URL:"/ui/static/"})["VSCODE_NLS_CONFIG"];if(e)try{const n=JSON.parse(e),t=n.availableLanguages["*"];f=n.locale,m=t||a,_=n._translationsConfigFile}catch(L){}u=!0}else console.error("Unable to resolve platform.");else g=navigator.userAgent,o=g.indexOf("Windows")>=0,s=g.indexOf("Macintosh")>=0,p=(g.indexOf("Macintosh")>=0||g.indexOf("iPad")>=0||g.indexOf("iPhone")>=0)&&!!navigator.maxTouchPoints&&navigator.maxTouchPoints>0,c=g.indexOf("Linux")>=0,d=!0,f=navigator.language,m=f;let b=0;s?b=1:o?b=3:c&&(b=2);const E=o,T=s,S=d,x=function(){if(y.setImmediate)return y.setImmediate.bind(y);if("function"===typeof y.postMessage&&!y.importScripts){let e=[];y.addEventListener("message",n=>{if(n.data&&n.data.vscodeSetImmediateId)for(let t=0,r=e.length;t<r;t++){const r=e[t];if(r.id===n.data.vscodeSetImmediateId)return e.splice(t,1),void r.callback()}});let n=0;return t=>{const r=++n;e.push({id:r,callback:t}),y.postMessage({vscodeSetImmediateId:r},"*")}}if("function"===typeof(null===h||void 0===h?void 0:h.nextTick))return h.nextTick.bind(h);const e=Promise.resolve();return n=>e.then(n)}()}).call(this,t("c8ba"),t("f28c"))},"973d":function(e,n,t){"use strict";t.r(n),function(e,r,i){t.d(n,"createClassifier",function(){return f}),t.d(n,"createLanguageService",function(){return m}),t.d(n,"displayPartsToString",function(){return _}),t.d(n,"EndOfLineState",function(){return g}),t.d(n,"flattenDiagnosticMessageText",function(){return y}),t.d(n,"IndentStyle",function(){return h}),t.d(n,"ScriptKind",function(){return v}),t.d(n,"ScriptTarget",function(){return b}),t.d(n,"TokenClass",function(){return E});
/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
var a=function(e,n,t){if(t||2===arguments.length)for(var r,i=0,a=n.length;i<a;i++)!r&&i in n||(r||(r=Array.prototype.slice.call(n,0,i)),r[i]=n[i]);return e.concat(r||Array.prototype.slice.call(n))},o=function(){return o=Object.assign||function(e){for(var n,t=1,r=arguments.length;t<r;t++)for(var i in n=arguments[t],n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i]);return e},o.apply(this,arguments)},s=function(e,n){return Object.defineProperty?Object.defineProperty(e,"raw",{value:n}):e.raw=n,e},c=function(e,n){var t,r,i,a,o={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return a={next:s(0),throw:s(1),return:s(2)},"function"===typeof Symbol&&(a[Symbol.iterator]=function(){return this}),a;function s(e){return function(n){return c([e,n])}}function c(a){if(t)throw new TypeError("Generator is already executing.");while(o)try{if(t=1,r&&(i=2&a[0]?r["return"]:a[0]?r["throw"]||((i=r["return"])&&i.call(r),0):r.next)&&!(i=i.call(r,a[1])).done)return i;switch(r=0,i&&(a=[2&a[0],i.value]),a[0]){case 0:case 1:i=a;break;case 4:return o.label++,{value:a[1],done:!1};case 5:o.label++,r=a[1],a=[0];continue;case 7:a=o.ops.pop(),o.trys.pop();continue;default:if(i=o.trys,!(i=i.length>0&&i[i.length-1])&&(6===a[0]||2===a[0])){o=0;continue}if(3===a[0]&&(!i||a[1]>i[0]&&a[1]<i[3])){o.label=a[1];break}if(6===a[0]&&o.label<i[1]){o.label=i[1],i=a;break}if(i&&o.label<i[2]){o.label=i[2],o.ops.push(a);break}i[2]&&o.ops.pop(),o.trys.pop();continue}a=n.call(e,o)}catch(s){a=[6,s],r=0}finally{t=i=0}if(5&a[0])throw a[1];return{value:a[0]?a[1]:void 0,done:!0}}},l=function(e,n){var t={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&n.indexOf(r)<0&&(t[r]=e[r]);if(null!=e&&"function"===typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)n.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(t[r[i]]=e[r[i]])}return t},u=function(){var e=function(n,t){return e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,n){e.__proto__=n}||function(e,n){for(var t in n)Object.prototype.hasOwnProperty.call(n,t)&&(e[t]=n[t])},e(n,t)};return function(n,t){if("function"!==typeof t&&null!==t)throw new TypeError("Class extends value "+String(t)+" is not a constructor or null");function r(){this.constructor=n}e(n,t),n.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}}();(function(e){function n(){var e={};return e.prev=e,{head:e,tail:e,size:0}}function t(e,n){return{key:e,value:n,next:void 0,prev:void 0}}function r(e,n){return e===n||e!==e&&n!==n}function i(e){var n=e.prev;if(!n||n===e)throw new Error("Illegal state");return n}function a(e){while(e){var n=!e.prev;if(e=e.next,!n)return e}}function o(e,n){for(var t=e.tail;t!==e.head;t=i(t))if(r(t.key,n))return t}function s(e,n,r){var i=o(e,n);if(!i){var a=t(n,r);return a.prev=e.tail,e.tail.next=a,e.tail=a,e.size++,a}i.value=r}function c(e,n){for(var t=e.tail;t!==e.head;t=i(t)){if(void 0===t.prev)throw new Error("Illegal state");if(r(t.key,n)){if(t.next)t.next.prev=t.prev;else{if(e.tail!==t)throw new Error("Illegal state");e.tail=t.prev}return t.prev.next=t.next,t.next=t.prev,t.prev=void 0,e.size--,t}}}function l(e){var n=e.tail;while(n!==e.head){var t=i(n);n.next=e.head,n.prev=void 0,n=t}e.head.next=void 0,e.tail=e.head,e.size=0}function u(e,n){var t=e.head;while(t)t=a(t),t&&n(t.value,t.key)}function d(e,n){if(e)for(var t=e.next();!t.done;t=e.next())n(t.value)}function p(e,n){return{current:e.head,selector:n}}function f(e){return e.current=a(e.current),e.current?{value:e.selector(e.current.key,e.current.value),done:!1}:{value:void 0,done:!0}}(function(e){function t(e){var t=function(){function e(e,n){this._data=p(e,n)}return e.prototype.next=function(){return f(this._data)},e}();return function(){function r(t){var r=this;this._mapData=n(),d(e(t),function(e){var n=e[0],t=e[1];return r.set(n,t)})}return Object.defineProperty(r.prototype,"size",{get:function(){return this._mapData.size},enumerable:!1,configurable:!0}),r.prototype.get=function(e){var n;return null===(n=o(this._mapData,e))||void 0===n?void 0:n.value},r.prototype.set=function(e,n){return s(this._mapData,e,n),this},r.prototype.has=function(e){return!!o(this._mapData,e)},r.prototype.delete=function(e){return!!c(this._mapData,e)},r.prototype.clear=function(){l(this._mapData)},r.prototype.keys=function(){return new t(this._mapData,function(e,n){return e})},r.prototype.values=function(){return new t(this._mapData,function(e,n){return n})},r.prototype.entries=function(){return new t(this._mapData,function(e,n){return[e,n]})},r.prototype.forEach=function(e){u(this._mapData,e)},r}()}function r(e){var t=function(){function e(e,n){this._data=p(e,n)}return e.prototype.next=function(){return f(this._data)},e}();return function(){function r(t){var r=this;this._mapData=n(),d(e(t),function(e){return r.add(e)})}return Object.defineProperty(r.prototype,"size",{get:function(){return this._mapData.size},enumerable:!1,configurable:!0}),r.prototype.add=function(e){return s(this._mapData,e,e),this},r.prototype.has=function(e){return!!o(this._mapData,e)},r.prototype.delete=function(e){return!!c(this._mapData,e)},r.prototype.clear=function(){l(this._mapData)},r.prototype.keys=function(){return new t(this._mapData,function(e,n){return e})},r.prototype.values=function(){return new t(this._mapData,function(e,n){return n})},r.prototype.entries=function(){return new t(this._mapData,function(e,n){return[e,n]})},r.prototype.forEach=function(e){u(this._mapData,e)},r}()}e.createMapShim=t,e.createSetShim=r})(e.ShimCollections||(e.ShimCollections={}))})(d||(d={})),function(e){e.versionMajorMinor="4.4",e.version="4.4.4",function(e){e[e["LessThan"]=-1]="LessThan",e[e["EqualTo"]=0]="EqualTo",e[e["GreaterThan"]=1]="GreaterThan"}(e.Comparison||(e.Comparison={})),function(e){function n(){return"undefined"!==typeof Map&&"entries"in Map.prototype&&1===new Map([[0,0]]).size?Map:void 0}function t(){return"undefined"!==typeof Set&&"entries"in Set.prototype&&1===new Set([0]).size?Set:void 0}e.tryGetNativeMap=n,e.tryGetNativeSet=t}(e.NativeCollections||(e.NativeCollections={}))}(d||(d={})),function(e){function n(n,r,i){var a,o=null!==(a=e.NativeCollections[r]())&&void 0!==a?a:null===e.ShimCollections||void 0===e.ShimCollections?void 0:e.ShimCollections[i](t);if(o)return o;throw new Error("TypeScript requires an environment that provides a compatible native "+n+" implementation.")}function t(n){if(n){if(tn(n))return ye(n);if(n instanceof e.Map)return n.entries();if(n instanceof e.Set)return n.values();throw new Error("Iteration not supported.")}}function r(){return new e.Map}function i(n){var t=new e.Map;for(var r in n)Re.call(n,r)&&t.set(r,n[r]);return t}function o(e){return e?e.length:0}function s(e,n){if(e)for(var t=0;t<e.length;t++){var r=n(e[t],t);if(r)return r}}function c(e,n){if(e)for(var t=e.length-1;t>=0;t--){var r=n(e[t],t);if(r)return r}}function l(e,n){if(void 0!==e)for(var t=0;t<e.length;t++){var r=n(e[t],t);if(void 0!==r)return r}}function u(e,n){while(1){var t=e.next();if(t.done)return;var r=n(t.value);if(void 0!==r)return r}}function d(e,n,t){var r=t;if(e)for(var i=e.next(),a=0;!i.done;i=e.next(),a++)r=n(r,i.value,a);return r}function p(n,t,r){var i=[];e.Debug.assertEqual(n.length,t.length);for(var a=0;a<n.length;a++)i.push(r(n[a],t[a],a));return i}function f(n,t){e.Debug.assertEqual(n.length,t.length);var r=0;return{next:function(){return r===n.length?{value:void 0,done:!0}:(r++,{value:[n[r-1],t[r-1]],done:!1})}}}function m(n,t){e.Debug.assert(n.length===t.length);for(var r=new e.Map,i=0;i<n.length;++i)r.set(n[i],t[i]);return r}function _(e,n){if(e.length<=1)return e;for(var t=[],r=0,i=e.length;r<i;r++)r&&t.push(n),t.push(e[r]);return t}function g(e,n){if(e)for(var t=0;t<e.length;t++)if(!n(e[t],t))return!1;return!0}function y(e,n){for(var t=0;t<e.length;t++){var r=e[t];if(n(r,t))return r}}function h(e,n){for(var t=e.length-1;t>=0;t--){var r=e[t];if(n(r,t))return r}}function v(e,n,t){for(var r=t||0;r<e.length;r++)if(n(e[r],r))return r;return-1}function b(e,n,t){for(var r=void 0===t?e.length-1:t;r>=0;r--)if(n(e[r],r))return r;return-1}function E(n,t){for(var r=0;r<n.length;r++){var i=t(n[r],r);if(i)return i}return e.Debug.fail()}function T(e,n,t){if(void 0===t&&(t=En),e)for(var r=0,i=e;r<i.length;r++){var a=i[r];if(t(a,n))return!0}return!1}function S(e,n,t){return void 0===t&&(t=En),e.length===n.length&&e.every(function(e,r){return t(e,n[r])})}function x(e,n,t){for(var r=t||0;r<e.length;r++)if(T(n,e.charCodeAt(r)))return r;return-1}function L(e,n){var t=0;if(e)for(var r=0;r<e.length;r++){var i=e[r];n(i,r)&&t++}return t}function D(e,n){if(e){var t=e.length,r=0;while(r<t&&n(e[r]))r++;if(r<t){var i=e.slice(0,r);r++;while(r<t){var a=e[r];n(a)&&i.push(a),r++}return i}}return e}function A(e,n){for(var t=0,r=0;r<e.length;r++)n(e[r],r,e)&&(e[t]=e[r],t++);e.length=t}function C(e){e.length=0}function I(e,n){var t;if(e){t=[];for(var r=0;r<e.length;r++)t.push(n(e[r],r))}return t}function N(e,n){return{next:function(){var t=e.next();return t.done?t:{value:n(t.value),done:!1}}}}function k(e,n){if(e)for(var t=0;t<e.length;t++){var r=e[t],i=n(r,t);if(r!==i){var a=e.slice(0,t);for(a.push(i),t++;t<e.length;t++)a.push(n(e[t],t));return a}}return e}function R(e){for(var n=[],t=0,r=e;t<r.length;t++){var i=r[t];i&&(tn(i)?pe(n,i):n.push(i))}return n}function O(n,t){var r;if(n)for(var i=0;i<n.length;i++){var a=t(n[i],i);a&&(r=tn(a)?pe(r,a):le(r,a))}return r||e.emptyArray}function w(e,n){var t=[];if(e)for(var r=0;r<e.length;r++){var i=n(e[r],r);i&&(tn(i)?pe(t,i):t.push(i))}return t}function P(n,t){var r=n.next();if(r.done)return e.emptyIterator;var i=a(r.value);return{next:function(){while(1){var e=i.next();if(!e.done)return e;var t=n.next();if(t.done)return t;i=a(t.value)}}};function a(n){var r=t(n);return void 0===r?e.emptyIterator:tn(r)?ye(r):r}}function M(e,n){var t;if(e)for(var r=0;r<e.length;r++){var i=e[r],a=n(i,r);(t||i!==a||tn(a))&&(t||(t=e.slice(0,r)),tn(a)?pe(t,a):t.push(a))}return t||e}function F(e,n){for(var t=[],r=0;r<e.length;r++){var i=n(e[r],r);if(void 0===i)return;t.push(i)}return t}function G(e,n){var t=[];if(e)for(var r=0;r<e.length;r++){var i=n(e[r],r);void 0!==i&&t.push(i)}return t}function B(e,n){return{next:function(){while(1){var t=e.next();if(t.done)return t;var r=n(t.value);if(void 0!==r)return{value:r,done:!1}}}}}function U(n,t){if(n){var r=new e.Map;return n.forEach(function(e,n){var i=t(n,e);if(void 0!==i){var a=i[0],o=i[1];void 0!==a&&void 0!==o&&r.set(a,o)}}),r}}function V(n,t){if(n){var r=new e.Set;return n.forEach(function(e){var n=t(e);void 0!==n&&r.add(n)}),r}}function j(e,n,t){if(e.has(n))return e.get(n);var r=t();return e.set(n,r),r}function K(e,n){return!e.has(n)&&(e.add(n),!0)}function H(e){var n=!1;return{next:function(){var t=n;return n=!0,t?{value:void 0,done:!0}:{value:e,done:!1}}}}function W(e,n,t){var r;if(e){r=[];var i=e.length,a=void 0,o=void 0,s=0,c=0;while(s<i){while(c<i){var l=e[c];if(o=n(l,c),0===c)a=o;else if(o!==a)break;c++}if(s<c){var u=t(e.slice(s,c),a,s,c);u&&r.push(u),s=c}a=o,c++}}return r}function z(n,t){if(n){var r=new e.Map;return n.forEach(function(e,n){var i=t(n,e),a=i[0],o=i[1];r.set(a,o)}),r}}function J(e,n){if(e){if(!n)return e.length>0;for(var t=0,r=e;t<r.length;t++){var i=r[t];if(n(i))return!0}}return!1}function q(e,n,t){for(var r,i=0;i<e.length;i++)n(e[i])?r=void 0===r?i:r:void 0!==r&&(t(r,i),r=void 0);void 0!==r&&t(r,e.length)}function X(e,n){return J(n)?J(e)?a(a([],e,!0),n,!0):n:e}function Y(e,n){return n}function Q(e){return e.map(Y)}function Z(e,n,t){var r=Q(e);_e(e,r,t);for(var i=e[r[0]],a=[r[0]],o=1;o<r.length;o++){var s=r[o],c=e[s];n(i,c)||(a.push(s),i=c)}return a.sort(),a.map(function(n){return e[n]})}function $(e,n){for(var t=[],r=0,i=e;r<i.length;r++){var a=i[r];fe(t,a,n)}return t}function ee(e,n,t){return 0===e.length?[]:1===e.length?e.slice():t?Z(e,n,t):$(e,n)}function ne(n,t){if(0===n.length)return e.emptyArray;for(var r=n[0],i=[r],a=1;a<n.length;a++){var o=n[a];switch(t(o,r)){case!0:case 0:continue;case-1:return e.Debug.fail("Array is unsorted.")}i.push(r=o)}return i}function te(e,n,t){if(0!==e.length){var r=Ie(e,n,fn,t);r<0&&e.splice(~r,0,n)}else e.push(n)}function re(e,n,t){return ne(ge(e,n),t||n||In)}function ie(e,n){if(e.length<2)return!0;for(var t=e[0],r=0,i=e.slice(1);r<i.length;r++){var a=i[r];if(1===n(t,a))return!1;t=a}return!0}function ae(e,n,t){if(void 0===t&&(t=En),!e||!n)return e===n;if(e.length!==n.length)return!1;for(var r=0;r<e.length;r++)if(!t(e[r],n[r],r))return!1;return!0}function oe(e){var n;if(e)for(var t=0;t<e.length;t++){var r=e[t];!n&&r||(n||(n=e.slice(0,t)),r&&n.push(r))}return n||e}function se(n,t,r){if(!t||!n||0===t.length||0===n.length)return t;var i=[];e:for(var a=0,o=0;o<t.length;o++){o>0&&e.Debug.assertGreaterThanOrEqual(r(t[o],t[o-1]),0);n:for(var s=a;a<n.length;a++)switch(a>s&&e.Debug.assertGreaterThanOrEqual(r(n[a],n[a-1]),0),r(t[o],n[a])){case-1:i.push(t[o]);continue e;case 0:continue e;case 1:continue n}}return i}function ce(e,n){for(var t=0,r=0,i=e;r<i.length;r++){var a=i[r];t+=a[n]}return t}function le(e,n){return void 0===n?e:void 0===e?[n]:(e.push(n),e)}function ue(e,n){return void 0===e?n:void 0===n?e:tn(e)?tn(n)?X(e,n):le(e,n):tn(n)?le(n,e):[e,n]}function de(e,n){return n<0?e.length+n:n}function pe(e,n,t,r){if(void 0===n||0===n.length)return e;if(void 0===e)return n.slice(t,r);t=void 0===t?0:de(n,t),r=void 0===r?n.length:de(n,r);for(var i=t;i<r&&i<n.length;i++)void 0!==n[i]&&e.push(n[i]);return e}function fe(e,n,t){return!T(e,n,t)&&(e.push(n),!0)}function me(e,n,t){return e?(fe(e,n,t),e):[n]}function _e(e,n,t){n.sort(function(n,r){return t(e[n],e[r])||Ln(n,r)})}function ge(e,n){return 0===e.length?e:e.slice().sort(n)}function ye(e){var n=0;return{next:function(){return n===e.length?{value:void 0,done:!0}:(n++,{value:e[n-1],done:!1})}}}function he(e){var n=e.length;return{next:function(){return 0===n?{value:void 0,done:!0}:(n--,{value:e[n],done:!1})}}}function ve(e,n){var t=Q(e);return _e(e,t,n),t.map(function(n){return e[n]})}function be(e,n,t,r){while(t<r){if(e[t]!==n[t])return!1;t++}return!0}function Ee(e,n){if(e&&(n=de(e,n),n<e.length))return e[n]}function Te(e){return 0===e.length?void 0:e[0]}function Se(n){return e.Debug.assert(0!==n.length),n[0]}function xe(e){return 0===e.length?void 0:e[e.length-1]}function Le(n){return e.Debug.assert(0!==n.length),n[n.length-1]}function De(e){return e&&1===e.length?e[0]:void 0}function Ae(e){return e&&1===e.length?e[0]:e}function Ce(e,n,t){var r=e.slice(0);return r[n]=t,r}function Ie(e,n,t,r,i){return Ne(e,t(n),t,r,i)}function Ne(e,n,t,r,i){if(!J(e))return-1;var a=i||0,o=e.length-1;while(a<=o){var s=a+(o-a>>1),c=t(e[s],s);switch(r(c,n)){case-1:a=s+1;break;case 0:return s;case 1:o=s-1;break}}return~a}function ke(e,n,t,r,i){if(e&&e.length>0){var a=e.length;if(a>0){var o=void 0===r||r<0?0:r,s=void 0===i||o+i>a-1?a-1:o+i,c=void 0;arguments.length<=2?(c=e[o],o++):c=t;while(o<=s)c=n(c,e[o],o),o++;return c}}return t}e.Map=n("Map","tryGetNativeMap","createMapShim"),e.Set=n("Set","tryGetNativeSet","createSetShim"),e.getIterator=t,e.emptyArray=[],e.emptyMap=new e.Map,e.emptySet=new e.Set,e.createMap=r,e.createMapFromTemplate=i,e.length=o,e.forEach=s,e.forEachRight=c,e.firstDefined=l,e.firstDefinedIterator=u,e.reduceLeftIterator=d,e.zipWith=p,e.zipToIterator=f,e.zipToMap=m,e.intersperse=_,e.every=g,e.find=y,e.findLast=h,e.findIndex=v,e.findLastIndex=b,e.findMap=E,e.contains=T,e.arraysEqual=S,e.indexOfAnyCharCode=x,e.countWhere=L,e.filter=D,e.filterMutate=A,e.clear=C,e.map=I,e.mapIterator=N,e.sameMap=k,e.flatten=R,e.flatMap=O,e.flatMapToMutable=w,e.flatMapIterator=P,e.sameFlatMap=M,e.mapAllOrFail=F,e.mapDefined=G,e.mapDefinedIterator=B,e.mapDefinedEntries=U,e.mapDefinedValues=V,e.getOrUpdate=j,e.tryAddToSet=K,e.emptyIterator={next:function(){return{value:void 0,done:!0}}},e.singleIterator=H,e.spanMap=W,e.mapEntries=z,e.some=J,e.getRangesWhere=q,e.concatenate=X,e.indicesOf=Q,e.deduplicate=ee,e.insertSorted=te,e.sortAndDeduplicate=re,e.arrayIsSorted=ie,e.arrayIsEqualTo=ae,e.compact=oe,e.relativeComplement=se,e.sum=ce,e.append=le,e.combine=ue,e.addRange=pe,e.pushIfUnique=fe,e.appendIfUnique=me,e.sort=ge,e.arrayIterator=ye,e.arrayReverseIterator=he,e.stableSort=ve,e.rangeEquals=be,e.elementAt=Ee,e.firstOrUndefined=Te,e.first=Se,e.lastOrUndefined=xe,e.last=Le,e.singleOrUndefined=De,e.singleOrMany=Ae,e.replaceElement=Ce,e.binarySearch=Ie,e.binarySearchKey=Ne,e.reduceLeft=ke;var Re=Object.prototype.hasOwnProperty;function Oe(e,n){return Re.call(e,n)}function we(e,n){return Re.call(e,n)?e[n]:void 0}function Pe(e){var n=[];for(var t in e)Re.call(e,t)&&n.push(t);return n}function Me(e){var n=[];do{for(var t=Object.getOwnPropertyNames(e),r=0,i=t;r<i.length;r++){var a=i[r];fe(n,a)}}while(e=Object.getPrototypeOf(e));return n}function Fe(e){var n=[];for(var t in e)Re.call(e,t)&&n.push(e[t]);return n}e.hasProperty=Oe,e.getProperty=we,e.getOwnKeys=Pe,e.getAllKeys=Me,e.getOwnValues=Fe;var Ge=Object.entries||function(e){for(var n=Pe(e),t=Array(n.length),r=0;r<n.length;r++)t[r]=[n[r],e[n[r]]];return t};function Be(e){return e?Ge(e):[]}function Ue(e,n){for(var t=new Array(e),r=0;r<e;r++)t[r]=n(r);return t}function Ve(e,n){for(var t=[],r=e.next();!r.done;r=e.next())t.push(n?n(r.value):r.value);return t}function je(e){for(var n=[],t=1;t<arguments.length;t++)n[t-1]=arguments[t];for(var r=0,i=n;r<i.length;r++){var a=i[r];if(void 0!==a)for(var o in a)Oe(a,o)&&(e[o]=a[o])}return e}function Ke(e,n,t){if(void 0===t&&(t=En),e===n)return!0;if(!e||!n)return!1;for(var r in e)if(Re.call(e,r)){if(!Re.call(n,r))return!1;if(!t(e[r],n[r]))return!1}for(var r in n)if(Re.call(n,r)&&!Re.call(e,r))return!1;return!0}function He(n,t,r){void 0===r&&(r=fn);for(var i=new e.Map,a=0,o=n;a<o.length;a++){var s=o[a],c=t(s);void 0!==c&&i.set(c,r(s))}return i}function We(e,n,t){void 0===t&&(t=fn);for(var r=[],i=0,a=e;i<a.length;i++){var o=a[i];r[n(o)]=t(o)}return r}function ze(e,n,t){void 0===t&&(t=fn);for(var r=Ze(),i=0,a=e;i<a.length;i++){var o=a[i];r.add(n(o),t(o))}return r}function Je(e,n,t){return void 0===t&&(t=fn),Ve(ze(e,n).values(),t)}function qe(e){var n={};for(var t in e)Re.call(e,t)&&(n[t]=e[t]);return n}function Xe(e,n){var t={};for(var r in n)Re.call(n,r)&&(t[r]=n[r]);for(var r in e)Re.call(e,r)&&(t[r]=e[r]);return t}function Ye(e,n){for(var t in n)Re.call(n,t)&&(e[t]=n[t])}function Qe(e,n){return n?n.bind(e):void 0}function Ze(){var n=new e.Map;return n.add=$e,n.remove=en,n}function $e(e,n){var t=this.get(e);return t?t.push(n):this.set(e,t=[n]),t}function en(e,n){var t=this.get(e);t&&(Xn(t,n),t.length||this.delete(e))}function nn(){return Ze()}function tn(e){return Array.isArray?Array.isArray(e):e instanceof Array}function rn(e){return tn(e)?e:[e]}function an(e){return"string"===typeof e}function on(e){return"number"===typeof e}function sn(e,n){return void 0!==e&&n(e)?e:void 0}function cn(n,t){return void 0!==n&&t(n)?n:e.Debug.fail("Invalid cast. The supplied value "+n+" did not pass the test '"+e.Debug.getFunctionName(t)+"'.")}function ln(e){}function un(){return!1}function dn(){return!0}function pn(){}function fn(e){return e}function mn(e){return e.toLowerCase()}e.getEntries=Be,e.arrayOf=Ue,e.arrayFrom=Ve,e.assign=je,e.equalOwnProperties=Ke,e.arrayToMap=He,e.arrayToNumericMap=We,e.arrayToMultiMap=ze,e.group=Je,e.clone=qe,e.extend=Xe,e.copyProperties=Ye,e.maybeBind=Qe,e.createMultiMap=Ze,e.createUnderscoreEscapedMultiMap=nn,e.isArray=tn,e.toArray=rn,e.isString=an,e.isNumber=on,e.tryCast=sn,e.cast=cn,e.noop=ln,e.returnFalse=un,e.returnTrue=dn,e.returnUndefined=pn,e.identity=fn,e.toLowerCase=mn;var _n=/[^\u0130\u0131\u00DFa-z0-9\\/:\-_\. ]+/g;function gn(e){return _n.test(e)?e.replace(_n,mn):e}function yn(){throw new Error("Not implemented")}function hn(e){var n;return function(){return e&&(n=e(),e=void 0),n}}function vn(n){var t=new e.Map;return function(e){var r=typeof e+":"+e,i=t.get(r);return void 0!==i||t.has(r)||(i=n(e),t.set(r,i)),i}}function bn(e,n,t,r,i){if(i){for(var a=[],o=0;o<arguments.length;o++)a[o]=arguments[o];return function(e){return ke(a,function(e,n){return n(e)},e)}}return r?function(i){return r(t(n(e(i))))}:t?function(r){return t(n(e(r)))}:n?function(t){return n(e(t))}:e?function(n){return e(n)}:function(e){return e}}function En(e,n){return e===n}function Tn(e,n){return e===n||void 0!==e&&void 0!==n&&e.toUpperCase()===n.toUpperCase()}function Sn(e,n){return En(e,n)}function xn(e,n){return e===n?0:void 0===e?-1:void 0===n?1:e<n?-1:1}function Ln(e,n){return xn(e,n)}function Dn(e,n){return Ln(null===e||void 0===e?void 0:e.start,null===n||void 0===n?void 0:n.start)||Ln(null===e||void 0===e?void 0:e.length,null===n||void 0===n?void 0:n.length)}function An(e,n,t){return-1===t(e,n)?e:n}function Cn(e,n){return e===n?0:void 0===e?-1:void 0===n?1:(e=e.toUpperCase(),n=n.toUpperCase(),e<n?-1:e>n?1:0)}function In(e,n){return xn(e,n)}function Nn(e){return e?Cn:In}e.toFileNameLowerCase=gn,e.notImplemented=yn,e.memoize=hn,e.memoizeOne=vn,e.compose=bn,function(e){e[e["None"]=0]="None",e[e["Normal"]=1]="Normal",e[e["Aggressive"]=2]="Aggressive",e[e["VeryAggressive"]=3]="VeryAggressive"}(e.AssertionLevel||(e.AssertionLevel={})),e.equateValues=En,e.equateStringsCaseInsensitive=Tn,e.equateStringsCaseSensitive=Sn,e.compareValues=Ln,e.compareTextSpans=Dn,e.min=An,e.compareStringsCaseInsensitive=Cn,e.compareStringsCaseSensitive=In,e.getStringComparer=Nn;var kn,Rn,On=function(){var e,n,t=s();return c;function r(e,n,t){if(e===n)return 0;if(void 0===e)return-1;if(void 0===n)return 1;var r=t(e,n);return r<0?-1:r>0?1:0}function i(e){var n=new Intl.Collator(e,{usage:"sort",sensitivity:"variant"}).compare;return function(e,t){return r(e,t,n)}}function a(e){return void 0!==e?o():function(e,t){return r(e,t,n)};function n(e,n){return e.localeCompare(n)}}function o(){return function(n,t){return r(n,t,e)};function e(e,t){return n(e.toUpperCase(),t.toUpperCase())||n(e,t)}function n(e,n){return e<n?-1:e>n?1:0}}function s(){return"object"===typeof Intl&&"function"===typeof Intl.Collator?i:"function"===typeof String.prototype.localeCompare&&"function"===typeof String.prototype.toLocaleUpperCase&&"a".localeCompare("B")<0?a:o}function c(r){return void 0===r?e||(e=t(r)):"en-US"===r?n||(n=t(r)):t(r)}}();function wn(){return Rn}function Pn(e){Rn!==e&&(Rn=e,kn=void 0)}function Mn(e,n){var t=kn||(kn=On(Rn));return t(e,n)}function Fn(e,n,t,r){return e===n?0:void 0===e?-1:void 0===n?1:r(e[t],n[t])}function Gn(e,n){return Ln(e?1:0,n?1:0)}function Bn(n,t,r){for(var i,a=Math.min(2,Math.floor(.34*n.length)),o=Math.floor(.4*n.length)+1,s=0,c=t;s<c.length;s++){var l=c[s],u=r(l);if(void 0!==u&&Math.abs(u.length-n.length)<=a){if(u===n)continue;if(u.length<3&&u.toLowerCase()!==n.toLowerCase())continue;var d=Un(n,u,o-.1);if(void 0===d)continue;e.Debug.assert(d<o),o=d,i=l}}return i}function Un(e,n,t){for(var r=new Array(n.length+1),i=new Array(n.length+1),a=t+.01,o=0;o<=n.length;o++)r[o]=o;for(o=1;o<=e.length;o++){var s=e.charCodeAt(o-1),c=Math.ceil(o>t?o-t:1),l=Math.floor(n.length>t+o?t+o:n.length);i[0]=o;for(var u=o,d=1;d<c;d++)i[d]=a;for(d=c;d<=l;d++){var p=e[o-1].toLowerCase()===n[d-1].toLowerCase()?r[d-1]+.1:r[d-1]+2,f=s===n.charCodeAt(d-1)?r[d-1]:Math.min(r[d]+1,i[d-1]+1,p);i[d]=f,u=Math.min(u,f)}for(d=l+1;d<=n.length;d++)i[d]=a;if(u>t)return;var m=r;r=i,i=m}var _=r[n.length];return _>t?void 0:_}function Vn(e,n){var t=e.length-n.length;return t>=0&&e.indexOf(n,t)===t}function jn(e,n){return Vn(e,n)?e.slice(0,e.length-n.length):e}function Kn(e,n){return Vn(e,n)?e.slice(0,e.length-n.length):void 0}function Hn(e,n){return-1!==e.indexOf(n)}function Wn(e){for(var n=e.length,t=n-1;t>0;t--){var r=e.charCodeAt(t);if(r>=48&&r<=57)do{--t,r=e.charCodeAt(t)}while(t>0&&r>=48&&r<=57);else{if(!(t>4)||110!==r&&78!==r)break;if(--t,r=e.charCodeAt(t),105!==r&&73!==r)break;if(--t,r=e.charCodeAt(t),109!==r&&77!==r)break;--t,r=e.charCodeAt(t)}if(45!==r&&46!==r)break;n=t}return n===e.length?e:e.slice(0,n)}function zn(e,n){for(var t=0;t<e.length;t++)if(e[t]===n)return Jn(e,t),!0;return!1}function Jn(e,n){for(var t=n;t<e.length-1;t++)e[t]=e[t+1];e.pop()}function qn(e,n){e[n]=e[e.length-1],e.pop()}function Xn(e,n){return Yn(e,function(e){return e===n})}function Yn(e,n){for(var t=0;t<e.length;t++)if(n(e[t]))return qn(e,t),!0;return!1}function Qn(e){return e?fn:gn}function Zn(e){var n=e.prefix,t=e.suffix;return n+"*"+t}function $n(n,t){return e.Debug.assert(it(n,t)),t.substring(n.prefix.length,t.length-n.suffix.length)}function et(e,n,t){for(var r,i=-1,a=0,o=e;a<o.length;a++){var s=o[a],c=n(s);it(c,t)&&c.prefix.length>i&&(i=c.prefix.length,r=s)}return r}function nt(e,n){return 0===e.lastIndexOf(n,0)}function tt(e,n){return nt(e,n)?e.substr(n.length):e}function rt(e,n,t){return void 0===t&&(t=fn),nt(t(e),t(n))?e.substring(n.length):void 0}function it(e,n){var t=e.prefix,r=e.suffix;return n.length>=t.length+r.length&&nt(n,t)&&Vn(n,r)}function at(e,n){return function(t){return e(t)&&n(t)}}function ot(){for(var e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];return function(){for(var n=[],t=0;t<arguments.length;t++)n[t]=arguments[t];for(var r=0,i=e;r<i.length;r++){var a=i[r];if(a.apply(void 0,n))return!0}return!1}}function st(e){return function(){for(var n=[],t=0;t<arguments.length;t++)n[t]=arguments[t];return!e.apply(void 0,n)}}function ct(e){}function lt(e){return void 0===e?void 0:[e]}function ut(e,n,t,r,i,a){a=a||ln;var o=0,s=0,c=e.length,l=n.length,u=!1;while(o<c&&s<l){var d=e[o],p=n[s],f=t(d,p);-1===f?(r(d),o++,u=!0):1===f?(i(p),s++,u=!0):(a(p,d),o++,s++)}while(o<c)r(e[o++]),u=!0;while(s<l)i(n[s++]),u=!0;return u}function dt(e,n){for(var t=Array(e),r=0;r<e;r++)t[r]=n(r);return t}function pt(e){var n=[];return ft(e,n,void 0,0),n}function ft(e,n,t,r){for(var i=0,a=e[r];i<a.length;i++){var o=a[i],s=void 0;t?(s=t.slice(),s.push(o)):s=[o],r===e.length-1?n.push(s):ft(e,n,s,r+1)}}function mt(e,n,t){return void 0===t&&(t=" "),n<=e.length?e:t.repeat(n-e.length)+e}function _t(e,n,t){return void 0===t&&(t=" "),n<=e.length?e:e+t.repeat(n-e.length)}function gt(e,n){var t=e.length,r=0;while(r<t&&n(e[r]))r++;return e.slice(0,r)}function yt(n){var t=n.length-1;while(t>=0){if(!e.isWhiteSpaceLike(n.charCodeAt(t)))break;t--}return n.slice(0,t+1)}e.getUILocale=wn,e.setUILocale=Pn,e.compareStringsCaseSensitiveUI=Mn,e.compareProperties=Fn,e.compareBooleans=Gn,e.getSpellingSuggestion=Bn,e.endsWith=Vn,e.removeSuffix=jn,e.tryRemoveSuffix=Kn,e.stringContains=Hn,e.removeMinAndVersionNumbers=Wn,e.orderedRemoveItem=zn,e.orderedRemoveItemAt=Jn,e.unorderedRemoveItemAt=qn,e.unorderedRemoveItem=Xn,e.createGetCanonicalFileName=Qn,e.patternText=Zn,e.matchedText=$n,e.findBestPatternMatch=et,e.startsWith=nt,e.removePrefix=tt,e.tryRemovePrefix=rt,e.and=at,e.or=ot,e.not=st,e.assertType=ct,e.singleElementArray=lt,e.enumerateInsertsAndDeletes=ut,e.fill=dt,e.cartesianProduct=pt,e.padLeft=mt,e.padRight=_t,e.takeWhile=gt,e.trimString=String.prototype.trim?function(e){return e.trim()}:function(n){return e.trimStringEnd(e.trimStringStart(n))},e.trimStringEnd=String.prototype.trimEnd?function(e){return e.trimEnd()}:yt,e.trimStringStart=String.prototype.trimStart?function(e){return e.trimStart()}:function(e){return e.replace(/^\s+/g,"")}}(d||(d={})),function(e){var n;(function(e){e[e["Off"]=0]="Off",e[e["Error"]=1]="Error",e[e["Warning"]=2]="Warning",e[e["Info"]=3]="Info",e[e["Verbose"]=4]="Verbose"})(n=e.LogLevel||(e.LogLevel={})),function(t){var r,i=0;function a(){return null!==r&&void 0!==r?r:r=new e.Version(e.version)}function o(e){return t.currentLogLevel<=e}function s(e,n){t.loggingHost&&o(e)&&t.loggingHost.log(e,n)}function c(e){s(n.Info,e)}t.currentLogLevel=n.Warning,t.isDebugging=!1,t.getTypeScriptVersion=a,t.shouldLog=o,t.log=c,function(e){function t(e){s(n.Error,e)}function r(e){s(n.Warning,e)}function i(e){s(n.Info,e)}function a(e){s(n.Verbose,e)}e.error=t,e.warn=r,e.log=i,e.trace=a}(c=t.log||(t.log={}));var l={};function u(){return i}function d(n){var r=i;if(i=n,n>r)for(var a=0,o=e.getOwnKeys(l);a<o.length;a++){var s=o[a],c=l[s];void 0!==c&&t[s]!==c.assertion&&n>=c.level&&(t[s]=c,l[s]=void 0)}}function p(e){return i>=e}function f(n,r){return!!p(n)||(l[r]={level:n,assertion:t[r]},t[r]=e.noop,!1)}function m(e,n){var t=new Error(e?"Debug Failure. "+e:"Debug Failure.");throw Error.captureStackTrace&&Error.captureStackTrace(t,n||m),t}function _(e,n,t){return m((n||"Unexpected node.")+"\r\nNode "+F(e.kind)+" was unexpected.",t||_)}function g(e,n,t,r){e||(n=n?"False expression: "+n:"False expression.",t&&(n+="\r\nVerbose Debug Information: "+("string"===typeof t?t:t())),m(n,r||g))}function y(e,n,t,r,i){if(e!==n){var a=t?r?t+" "+r:t:"";m("Expected "+e+" === "+n+". "+a,i||y)}}function h(e,n,t,r){e>=n&&m("Expected "+e+" < "+n+". "+(t||""),r||h)}function v(e,n,t){e>n&&m("Expected "+e+" <= "+n,t||v)}function b(e,n,t){e<n&&m("Expected "+e+" >= "+n,t||b)}function E(e,n,t){void 0!==e&&null!==e||m(n,t||E)}function T(e,n,t){return E(e,n,t||T),e}function S(e,n,t){for(var r=0,i=e;r<i.length;r++){var a=i[r];E(a,n,t||S)}}function x(e,n,t){return S(e,n,t||x),e}function L(n,t,r){void 0===t&&(t="Illegal value:");var i="object"===typeof n&&e.hasProperty(n,"kind")&&e.hasProperty(n,"pos")&&F?"SyntaxKind: "+F(n.kind):JSON.stringify(n);return m(t+" "+i,r||L)}function D(n,t,r,i){f(1,"assertEachNode")&&g(void 0===t||e.every(n,t),r||"Unexpected node.",function(){return"Node array did not pass test '"+O(t)+"'."},i||D)}function A(e,n,t,r){f(1,"assertNode")&&g(void 0!==e&&(void 0===n||n(e)),t||"Unexpected node.",function(){return"Node "+F(null===e||void 0===e?void 0:e.kind)+" did not pass test '"+O(n)+"'."},r||A)}function C(e,n,t,r){f(1,"assertNotNode")&&g(void 0===e||void 0===n||!n(e),t||"Unexpected node.",function(){return"Node "+F(e.kind)+" should not have passed test '"+O(n)+"'."},r||C)}function I(e,n,t,r){f(1,"assertOptionalNode")&&g(void 0===n||void 0===e||n(e),t||"Unexpected node.",function(){return"Node "+F(null===e||void 0===e?void 0:e.kind)+" did not pass test '"+O(n)+"'."},r||I)}function N(e,n,t,r){f(1,"assertOptionalToken")&&g(void 0===n||void 0===e||e.kind===n,t||"Unexpected node.",function(){return"Node "+F(null===e||void 0===e?void 0:e.kind)+" was not a '"+F(n)+"' token."},r||N)}function k(e,n,t){f(1,"assertMissingNode")&&g(void 0===e,n||"Unexpected node.",function(){return"Node "+F(e.kind)+" was unexpected'."},t||k)}function R(e){}function O(e){if("function"!==typeof e)return"";if(e.hasOwnProperty("name"))return e.name;var n=Function.prototype.toString.call(e),t=/^function\s+([\w\$]+)\s*\(/.exec(n);return t?t[1]:""}function w(n){return"{ name: "+e.unescapeLeadingUnderscores(n.escapedName)+"; flags: "+j(n.flags)+"; declarations: "+e.map(n.declarations,function(e){return F(e.kind)})+" }"}function P(e,n,t){void 0===e&&(e=0);var r=M(n);if(0===e)return r.length>0&&0===r[0][0]?r[0][1]:"0";if(t){for(var i="",a=e,o=0,s=r;o<s.length;o++){var c=s[o],l=c[0],u=c[1];if(l>e)break;0!==l&&l&e&&(i=i+(i?"|":"")+u,a&=~l)}if(0===a)return i}else for(var d=0,p=r;d<p.length;d++){var f=p[d];l=f[0],u=f[1];if(l===e)return u}return e.toString()}function M(n){var t=[];for(var r in n){var i=n[r];"number"===typeof i&&t.push([i,r])}return e.stableSort(t,function(n,t){return e.compareValues(n[0],t[0])})}function F(n){return P(n,e.SyntaxKind,!1)}function G(n){return P(n,e.NodeFlags,!0)}function B(n){return P(n,e.ModifierFlags,!0)}function U(n){return P(n,e.TransformFlags,!0)}function V(n){return P(n,e.EmitFlags,!0)}function j(n){return P(n,e.SymbolFlags,!0)}function K(n){return P(n,e.TypeFlags,!0)}function H(n){return P(n,e.SignatureFlags,!0)}function W(n){return P(n,e.ObjectFlags,!0)}function z(n){return P(n,e.FlowFlags,!0)}t.getAssertionLevel=u,t.setAssertionLevel=d,t.shouldAssert=p,t.fail=m,t.failBadSyntaxKind=_,t.assert=g,t.assertEqual=y,t.assertLessThan=h,t.assertLessThanOrEqual=v,t.assertGreaterThanOrEqual=b,t.assertIsDefined=E,t.checkDefined=T,t.assertDefined=T,t.assertEachIsDefined=S,t.checkEachDefined=x,t.assertEachDefined=x,t.assertNever=L,t.assertEachNode=D,t.assertNode=A,t.assertNotNode=C,t.assertOptionalNode=I,t.assertOptionalToken=N,t.assertMissingNode=k,t.type=R,t.getFunctionName=O,t.formatSymbol=w,t.formatEnum=P,t.formatSyntaxKind=F,t.formatNodeFlags=G,t.formatModifierFlags=B,t.formatTransformFlags=U,t.formatEmitFlags=V,t.formatSymbolFlags=j,t.formatTypeFlags=K,t.formatSignatureFlags=H,t.formatObjectFlags=W,t.formatFlowFlags=z;var J,q,X,Y=!1;function Q(){if(ie(),!J)throw new Error("Debugging helpers could not be loaded.");return J}function Z(e){return console.log($(e))}function $(e){return Q().formatControlFlowGraph(e)}function ee(n){"__debugFlowFlags"in n||Object.defineProperties(n,{__tsDebuggerDisplay:{value:function(){var e=2&this.flags?"FlowStart":4&this.flags?"FlowBranchLabel":8&this.flags?"FlowLoopLabel":16&this.flags?"FlowAssignment":32&this.flags?"FlowTrueCondition":64&this.flags?"FlowFalseCondition":128&this.flags?"FlowSwitchClause":256&this.flags?"FlowArrayMutation":512&this.flags?"FlowCall":1024&this.flags?"FlowReduceLabel":1&this.flags?"FlowUnreachable":"UnknownFlow",n=-2048&this.flags;return e+(n?" ("+z(n)+")":"")}},__debugFlowFlags:{get:function(){return P(this.flags,e.FlowFlags,!0)}},__debugToString:{value:function(){return $(this)}}})}function ne(e){Y&&("function"===typeof Object.setPrototypeOf?(q||(q=Object.create(Object.prototype),ee(q)),Object.setPrototypeOf(e,q)):ee(e))}function te(e){"__tsDebuggerDisplay"in e||Object.defineProperties(e,{__tsDebuggerDisplay:{value:function(e){return e=String(e).replace(/(?:,[\s\w\d_]+:[^,]+)+\]$/,"]"),"NodeArray "+e}}})}function re(e){Y&&("function"===typeof Object.setPrototypeOf?(X||(X=Object.create(Array.prototype),te(X)),Object.setPrototypeOf(e,X)):te(e))}function ie(){if(!Y){var n,t;Object.defineProperties(e.objectAllocator.getSymbolConstructor().prototype,{__tsDebuggerDisplay:{value:function(){var n=33554432&this.flags?"TransientSymbol":"Symbol",t=-33554433&this.flags;return n+" '"+e.symbolName(this)+"'"+(t?" ("+j(t)+")":"")}},__debugFlags:{get:function(){return j(this.flags)}}}),Object.defineProperties(e.objectAllocator.getTypeConstructor().prototype,{__tsDebuggerDisplay:{value:function(){var n=98304&this.flags?"NullableType":384&this.flags?"LiteralType "+JSON.stringify(this.value):2048&this.flags?"LiteralType "+(this.value.negative?"-":"")+this.value.base10Value+"n":8192&this.flags?"UniqueESSymbolType":32&this.flags?"EnumType":67359327&this.flags?"IntrinsicType "+this.intrinsicName:1048576&this.flags?"UnionType":2097152&this.flags?"IntersectionType":4194304&this.flags?"IndexType":8388608&this.flags?"IndexedAccessType":16777216&this.flags?"ConditionalType":33554432&this.flags?"SubstitutionType":262144&this.flags?"TypeParameter":524288&this.flags?3&this.objectFlags?"InterfaceType":4&this.objectFlags?"TypeReference":8&this.objectFlags?"TupleType":16&this.objectFlags?"AnonymousType":32&this.objectFlags?"MappedType":1024&this.objectFlags?"ReverseMappedType":256&this.objectFlags?"EvolvingArrayType":"ObjectType":"Type",t=524288&this.flags?-1344&this.objectFlags:0;return n+(this.symbol?" '"+e.symbolName(this.symbol)+"'":"")+(t?" ("+W(t)+")":"")}},__debugFlags:{get:function(){return K(this.flags)}},__debugObjectFlags:{get:function(){return 524288&this.flags?W(this.objectFlags):""}},__debugTypeToString:{value:function(){var e=c(),n=null===e||void 0===e?void 0:e.get(this);return void 0===n&&(n=this.checker.typeToString(this),null===e||void 0===e||e.set(this,n)),n}}}),Object.defineProperties(e.objectAllocator.getSignatureConstructor().prototype,{__debugFlags:{get:function(){return H(this.flags)}},__debugSignatureToString:{value:function(){var e;return null===(e=this.checker)||void 0===e?void 0:e.signatureToString(this)}}});for(var r=[e.objectAllocator.getNodeConstructor(),e.objectAllocator.getIdentifierConstructor(),e.objectAllocator.getTokenConstructor(),e.objectAllocator.getSourceFileConstructor()],i=0,a=r;i<a.length;i++){var o=a[i];o.prototype.hasOwnProperty("__debugKind")||Object.defineProperties(o.prototype,{__tsDebuggerDisplay:{value:function(){var n=e.isGeneratedIdentifier(this)?"GeneratedIdentifier":e.isIdentifier(this)?"Identifier '"+e.idText(this)+"'":e.isPrivateIdentifier(this)?"PrivateIdentifier '"+e.idText(this)+"'":e.isStringLiteral(this)?"StringLiteral "+JSON.stringify(this.text.length<10?this.text:this.text.slice(10)+"..."):e.isNumericLiteral(this)?"NumericLiteral "+this.text:e.isBigIntLiteral(this)?"BigIntLiteral "+this.text+"n":e.isTypeParameterDeclaration(this)?"TypeParameterDeclaration":e.isParameter(this)?"ParameterDeclaration":e.isConstructorDeclaration(this)?"ConstructorDeclaration":e.isGetAccessorDeclaration(this)?"GetAccessorDeclaration":e.isSetAccessorDeclaration(this)?"SetAccessorDeclaration":e.isCallSignatureDeclaration(this)?"CallSignatureDeclaration":e.isConstructSignatureDeclaration(this)?"ConstructSignatureDeclaration":e.isIndexSignatureDeclaration(this)?"IndexSignatureDeclaration":e.isTypePredicateNode(this)?"TypePredicateNode":e.isTypeReferenceNode(this)?"TypeReferenceNode":e.isFunctionTypeNode(this)?"FunctionTypeNode":e.isConstructorTypeNode(this)?"ConstructorTypeNode":e.isTypeQueryNode(this)?"TypeQueryNode":e.isTypeLiteralNode(this)?"TypeLiteralNode":e.isArrayTypeNode(this)?"ArrayTypeNode":e.isTupleTypeNode(this)?"TupleTypeNode":e.isOptionalTypeNode(this)?"OptionalTypeNode":e.isRestTypeNode(this)?"RestTypeNode":e.isUnionTypeNode(this)?"UnionTypeNode":e.isIntersectionTypeNode(this)?"IntersectionTypeNode":e.isConditionalTypeNode(this)?"ConditionalTypeNode":e.isInferTypeNode(this)?"InferTypeNode":e.isParenthesizedTypeNode(this)?"ParenthesizedTypeNode":e.isThisTypeNode(this)?"ThisTypeNode":e.isTypeOperatorNode(this)?"TypeOperatorNode":e.isIndexedAccessTypeNode(this)?"IndexedAccessTypeNode":e.isMappedTypeNode(this)?"MappedTypeNode":e.isLiteralTypeNode(this)?"LiteralTypeNode":e.isNamedTupleMember(this)?"NamedTupleMember":e.isImportTypeNode(this)?"ImportTypeNode":F(this.kind);return n+(this.flags?" ("+G(this.flags)+")":"")}},__debugKind:{get:function(){return F(this.kind)}},__debugNodeFlags:{get:function(){return G(this.flags)}},__debugModifierFlags:{get:function(){return B(e.getEffectiveModifierFlagsNoCache(this))}},__debugTransformFlags:{get:function(){return U(this.transformFlags)}},__debugIsParseTreeNode:{get:function(){return e.isParseTreeNode(this)}},__debugEmitFlags:{get:function(){return V(e.getEmitFlags(this))}},__debugGetText:{value:function(n){if(e.nodeIsSynthesized(this))return"";var t=l(),r=null===t||void 0===t?void 0:t.get(this);if(void 0===r){var i=e.getParseTreeNode(this),a=i&&e.getSourceFileOfNode(i);r=a?e.getSourceTextOfNodeFromSourceFile(a,i,n):"",null===t||void 0===t||t.set(this,r)}return r}}})}try{if(e.sys&&e.sys.require){e.getDirectoryPath(e.resolvePath(e.sys.getExecutingFilePath()));var s=void 0;s.error||(s.module.init(e),J=s.module)}}catch(u){}Y=!0}function c(){return void 0===n&&"function"===typeof WeakMap&&(n=new WeakMap),n}function l(){return void 0===t&&"function"===typeof WeakMap&&(t=new WeakMap),t}}function ae(n,t,r,i,a){var o=t?"DeprecationError: ":"DeprecationWarning: ";return o+="'"+n+"' ",o+=i?"has been deprecated since v"+i:"is deprecated",o+=t?" and can no longer be used.":r?" and will no longer be usable after v"+r+".":".",o+=a?" "+e.formatStringFromArgs(a,[n],0):"",o}function oe(e,n,t,r){var i=ae(e,!0,n,t,r);return function(){throw new TypeError(i)}}function se(e,n,t,r){var i=!1;return function(){i||(c.warn(ae(e,!1,n,t,r)),i=!0)}}function ce(n,t){var r,i;void 0===t&&(t={});var o="string"===typeof t.typeScriptVersion?new e.Version(t.typeScriptVersion):null!==(r=t.typeScriptVersion)&&void 0!==r?r:a(),s="string"===typeof t.errorAfter?new e.Version(t.errorAfter):t.errorAfter,c="string"===typeof t.warnAfter?new e.Version(t.warnAfter):t.warnAfter,l="string"===typeof t.since?new e.Version(t.since):null!==(i=t.since)&&void 0!==i?i:c,u=t.error||s&&o.compareTo(s)<=0,d=!c||o.compareTo(c)>=0;return u?oe(n,s,l,t.message):d?se(n,s,l,t.message):e.noop}function le(e,n){return function(){return e(),n.apply(this,arguments)}}function ue(e,n){var t=ce(O(e),n);return le(t,e)}t.printControlFlowGraph=Z,t.formatControlFlowGraph=$,t.attachFlowNodeDebugInfo=ne,t.attachNodeArrayDebugInfo=re,t.enableDebugInfo=ie,t.deprecate=ue}(e.Debug||(e.Debug={}))}(d||(d={})),function(e){var n=/^(0|[1-9]\d*)(?:\.(0|[1-9]\d*)(?:\.(0|[1-9]\d*)(?:\-([a-z0-9-.]+))?(?:\+([a-z0-9-.]+))?)?)?$/i,t=/^(?:0|[1-9]\d*|[a-z-][a-z0-9-]*)(?:\.(?:0|[1-9]\d*|[a-z-][a-z0-9-]*))*$/i,r=/^[a-z0-9-]+(?:\.[a-z0-9-]+)*$/i,i=/^(0|[1-9]\d*)$/,a=function(){function n(n,i,a,s,c){if(void 0===i&&(i=0),void 0===a&&(a=0),void 0===s&&(s=""),void 0===c&&(c=""),"string"===typeof n){var l=e.Debug.checkDefined(o(n),"Invalid version");n=l.major,i=l.minor,a=l.patch,s=l.prerelease,c=l.build}e.Debug.assert(n>=0,"Invalid argument: major"),e.Debug.assert(i>=0,"Invalid argument: minor"),e.Debug.assert(a>=0,"Invalid argument: patch"),e.Debug.assert(!s||t.test(s),"Invalid argument: prerelease"),e.Debug.assert(!c||r.test(c),"Invalid argument: build"),this.major=n,this.minor=i,this.patch=a,this.prerelease=s?s.split("."):e.emptyArray,this.build=c?c.split("."):e.emptyArray}return n.tryParse=function(e){var t=o(e);if(t){var r=t.major,i=t.minor,a=t.patch,s=t.prerelease,c=t.build;return new n(r,i,a,s,c)}},n.prototype.compareTo=function(n){return this===n?0:void 0===n?1:e.compareValues(this.major,n.major)||e.compareValues(this.minor,n.minor)||e.compareValues(this.patch,n.patch)||s(this.prerelease,n.prerelease)},n.prototype.increment=function(t){switch(t){case"major":return new n(this.major+1,0,0);case"minor":return new n(this.major,this.minor+1,0);case"patch":return new n(this.major,this.minor,this.patch+1);default:return e.Debug.assertNever(t)}},n.prototype.toString=function(){var n=this.major+"."+this.minor+"."+this.patch;return e.some(this.prerelease)&&(n+="-"+this.prerelease.join(".")),e.some(this.build)&&(n+="+"+this.build.join(".")),n},n.zero=new n(0,0,0),n}();function o(e){var i=n.exec(e);if(i){var a=i[1],o=i[2],s=void 0===o?"0":o,c=i[3],l=void 0===c?"0":c,u=i[4],d=void 0===u?"":u,p=i[5],f=void 0===p?"":p;if((!d||t.test(d))&&(!f||r.test(f)))return{major:parseInt(a,10),minor:parseInt(s,10),patch:parseInt(l,10),prerelease:d,build:f}}}function s(n,t){if(n===t)return 0;if(0===n.length)return 0===t.length?0:1;if(0===t.length)return-1;for(var r=Math.min(n.length,t.length),a=0;a<r;a++){var o=n[a],s=t[a];if(o!==s){var c=i.test(o),l=i.test(s);if(c||l){if(c!==l)return c?-1:1;var u=e.compareValues(+o,+s);if(u)return u}else{u=e.compareStringsCaseSensitive(o,s);if(u)return u}}}return e.compareValues(n.length,t.length)}e.Version=a;var c=function(){function n(n){this._alternatives=n?e.Debug.checkDefined(m(n),"Invalid range spec."):e.emptyArray}return n.tryParse=function(e){var t=m(e);if(t){var r=new n("");return r._alternatives=t,r}},n.prototype.test=function(e){return"string"===typeof e&&(e=new a(e)),b(e,this._alternatives)},n.prototype.toString=function(){return S(this._alternatives)},n}();e.VersionRange=c;var l=/\|\|/g,u=/\s+/g,d=/^([xX*0]|[1-9]\d*)(?:\.([xX*0]|[1-9]\d*)(?:\.([xX*0]|[1-9]\d*)(?:-([a-z0-9-.]+))?(?:\+([a-z0-9-.]+))?)?)?$/i,p=/^\s*([a-z0-9-+.*]+)\s+-\s+([a-z0-9-+.*]+)\s*$/i,f=/^(~|\^|<|<=|>|>=|=)?\s*([a-z0-9-+.*]+)$/i;function m(n){for(var t=[],r=0,i=e.trimString(n).split(l);r<i.length;r++){var a=i[r];if(a){var o=[];a=e.trimString(a);var s=p.exec(a);if(s){if(!g(s[1],s[2],o))return}else for(var c=0,d=a.split(u);c<d.length;c++){var m=d[c],_=f.exec(e.trimString(m));if(!_||!y(_[1],_[2],o))return}t.push(o)}}return t}function _(e){var n=d.exec(e);if(n){var t=n[1],r=n[2],i=void 0===r?"*":r,o=n[3],s=void 0===o?"*":o,c=n[4],l=n[5],u=new a(h(t)?0:parseInt(t,10),h(t)||h(i)?0:parseInt(i,10),h(t)||h(i)||h(s)?0:parseInt(s,10),c,l);return{version:u,major:t,minor:i,patch:s}}}function g(e,n,t){var r=_(e);if(!r)return!1;var i=_(n);return!!i&&(h(r.major)||t.push(v(">=",r.version)),h(i.major)||t.push(h(i.minor)?v("<",i.version.increment("major")):h(i.patch)?v("<",i.version.increment("minor")):v("<=",i.version)),!0)}function y(e,n,t){var r=_(n);if(!r)return!1;var i=r.version,o=r.major,s=r.minor,c=r.patch;if(h(o))"<"!==e&&">"!==e||t.push(v("<",a.zero));else switch(e){case"~":t.push(v(">=",i)),t.push(v("<",i.increment(h(s)?"major":"minor")));break;case"^":t.push(v(">=",i)),t.push(v("<",i.increment(i.major>0||h(s)?"major":i.minor>0||h(c)?"minor":"patch")));break;case"<":case">=":t.push(v(e,i));break;case"<=":case">":t.push(h(s)?v("<="===e?"<":">=",i.increment("major")):h(c)?v("<="===e?"<":">=",i.increment("minor")):v(e,i));break;case"=":case void 0:h(s)||h(c)?(t.push(v(">=",i)),t.push(v("<",i.increment(h(s)?"major":"minor")))):t.push(v("=",i));break;default:return!1}return!0}function h(e){return"*"===e||"x"===e||"X"===e}function v(e,n){return{operator:e,operand:n}}function b(e,n){if(0===n.length)return!0;for(var t=0,r=n;t<r.length;t++){var i=r[t];if(E(e,i))return!0}return!1}function E(e,n){for(var t=0,r=n;t<r.length;t++){var i=r[t];if(!T(e,i.operator,i.operand))return!1}return!0}function T(n,t,r){var i=n.compareTo(r);switch(t){case"<":return i<0;case"<=":return i<=0;case">":return i>0;case">=":return i>=0;case"=":return 0===i;default:return e.Debug.assertNever(t)}}function S(n){return e.map(n,x).join(" || ")||"*"}function x(n){return e.map(n,L).join(" ")}function L(e){return""+e.operator+e.operand}}(d||(d={})),function(n){function t(e,n){return"object"===typeof e&&"number"===typeof e.timeOrigin&&"function"===typeof e.mark&&"function"===typeof e.measure&&"function"===typeof e.now&&"function"===typeof n}function r(){if("object"===typeof performance&&"function"===typeof PerformanceObserver&&t(performance,PerformanceObserver))return{shouldWriteNativeEvents:!0,performance:performance,PerformanceObserver:PerformanceObserver}}function a(){if("undefined"!==typeof e&&e.nextTick&&!e.browser&&"object"===typeof i)try{var r,a={},o=a.performance,s=a.PerformanceObserver;if(t(o,s)){r=o;var c=new n.Version(e.versions.node),l=new n.VersionRange("<12.16.3 || 13 <13.13");return l.test(c)&&(r={get timeOrigin(){return o.timeOrigin},now:function(){return o.now()},mark:function(e){return o.mark(e)},measure:function(e,n,t){void 0===n&&(n="nodeStart"),void 0===t&&(t="__performance.measure-fix__",o.mark(t)),o.measure(e,n,t),"__performance.measure-fix__"===t&&o.clearMarks("__performance.measure-fix__")}}),{shouldWriteNativeEvents:!1,performance:r,PerformanceObserver:s}}}catch(u){}}var o=r()||a(),s=null===o||void 0===o?void 0:o.performance;function c(){return o}n.tryGetNativePerformanceHooks=c,n.timestamp=s?function(){return s.now()}:Date.now?Date.now:function(){return+new Date}}(d||(d={})),function(e){(function(n){var t,r;function i(e,t,r,i){return e?a(t,r,i):n.nullTimer}function a(n,t,r){var i=0;return{enter:a,exit:o};function a(){1===++i&&d(t)}function o(){0===--i?(d(r),p(n,t,r)):i<0&&e.Debug.fail("enter/exit count does not match.")}}n.createTimerIf=i,n.createTimer=a,n.nullTimer={enter:e.noop,exit:e.noop};var o=!1,s=e.timestamp(),c=new e.Map,l=new e.Map,u=new e.Map;function d(n){var t;if(o){var i=null!==(t=l.get(n))&&void 0!==t?t:0;l.set(n,i+1),c.set(n,e.timestamp()),null===r||void 0===r||r.mark(n)}}function p(n,t,i){var a,l;if(o){var d=null!==(a=void 0!==i?c.get(i):void 0)&&void 0!==a?a:e.timestamp(),p=null!==(l=void 0!==t?c.get(t):void 0)&&void 0!==l?l:s,f=u.get(n)||0;u.set(n,f+(d-p)),null===r||void 0===r||r.measure(n,t,i)}}function f(e){return l.get(e)||0}function m(e){return u.get(e)||0}function _(e){u.forEach(function(n,t){return e(t,n)})}function g(){return o}function y(n){var i;return void 0===n&&(n=e.sys),o||(o=!0,t||(t=e.tryGetNativePerformanceHooks()),t&&(s=t.performance.timeOrigin,(t.shouldWriteNativeEvents||(null===(i=null===n||void 0===n?void 0:n.cpuProfilingEnabled)||void 0===i?void 0:i.call(n))||(null===n||void 0===n?void 0:n.debugMode))&&(r=t.performance))),!0}function h(){o&&(c.clear(),l.clear(),u.clear(),r=void 0,o=!1)}n.mark=d,n.measure=p,n.getCount=f,n.getDuration=m,n.forEachMeasure=_,n.isEnabled=g,n.enable=y,n.disable=h})(e.performance||(e.performance={}))}(d||(d={})),function(e){var n,t,r={logEvent:e.noop,logErrEvent:e.noop,logPerfEvent:e.noop,logInfoEvent:e.noop,logStartCommand:e.noop,logStopCommand:e.noop,logStartUpdateProgram:e.noop,logStopUpdateProgram:e.noop,logStartUpdateGraph:e.noop,logStopUpdateGraph:e.noop,logStartResolveModule:e.noop,logStopResolveModule:e.noop,logStartParseSourceFile:e.noop,logStopParseSourceFile:e.noop,logStartReadFile:e.noop,logStopReadFile:e.noop,logStartBindFile:e.noop,logStopBindFile:e.noop,logStartScheduledOperation:e.noop,logStopScheduledOperation:e.noop};try{n=Object({NODE_ENV:"production",BASE_URL:"/ui/static/"}).TS_ETW_MODULE_PATH;t=void 0}catch(i){t=void 0}e.perfLogger=t&&t.logEvent?t:r}(d||(d={})),function(n){var t;(function(t){var r,i,a,s=0,c=0,l=[],u=[];function d(d,p,f){if(n.Debug.assert(!n.tracing,"Tracing already started"),void 0===r)try{r=void 0}catch(h){throw new Error("tracing requires having fs\n(original error: "+(h.message||h)+")")}i=d,l.length=0,void 0===a&&(a=n.combinePaths(p,"legend.json")),r.existsSync(p)||r.mkdirSync(p,{recursive:!0});var m="build"===i?"."+e.pid+"-"+ ++s:"server"===i?"."+e.pid:"",_=n.combinePaths(p,"trace"+m+".json"),g=n.combinePaths(p,"types"+m+".json");u.push({configFilePath:f,tracePath:_,typesPath:g}),c=r.openSync(_,"w"),n.tracing=t;var y={cat:"__metadata",ph:"M",ts:1e3*n.timestamp(),pid:1,tid:1};r.writeSync(c,"[\n"+[o({name:"process_name",args:{name:"tsc"}},y),o({name:"thread_name",args:{name:"Main"}},y),o(o({name:"TracingStartedInBrowser"},y),{cat:"disabled-by-default-devtools.timeline"})].map(function(e){return JSON.stringify(e)}).join(",\n"))}function p(){n.Debug.assert(n.tracing,"Tracing is not in progress"),n.Debug.assert(!!l.length===("server"!==i)),r.writeSync(c,"\n]\n"),r.closeSync(c),n.tracing=void 0,l.length?S(l):u[u.length-1].typesPath=void 0}function f(e){"server"!==i&&l.push(e)}function m(e,n,t){E("I",e,n,t,'"s":"g"')}t.startTracing=d,t.stopTracing=p,t.recordType=f,function(e){e["Parse"]="parse",e["Program"]="program",e["Bind"]="bind",e["Check"]="check",e["CheckTypes"]="checkTypes",e["Emit"]="emit",e["Session"]="session"}(t.Phase||(t.Phase={})),t.instant=m;var _=[];function g(e,t,r,i){void 0===i&&(i=!1),i&&E("B",e,t,r),_.push({phase:e,name:t,args:r,time:1e3*n.timestamp(),separateBeginAndEnd:i})}function y(){n.Debug.assert(_.length>0),b(_.length-1,1e3*n.timestamp()),_.length--}function h(){for(var e=1e3*n.timestamp(),t=_.length-1;t>=0;t--)b(t,e);_.length=0}t.push=g,t.pop=y,t.popAll=h;var v=1e4;function b(e,n){var t=_[e],r=t.phase,i=t.name,a=t.args,o=t.time,s=t.separateBeginAndEnd;s?E("E",r,i,a,void 0,n):v-o%v<=n-o&&E("X",r,i,a,'"dur":'+(n-o),o)}function E(e,t,a,o,s,l){void 0===l&&(l=1e3*n.timestamp()),"server"===i&&"checkTypes"===t||(n.performance.mark("beginTracing"),r.writeSync(c,',\n{"pid":1,"tid":1,"ph":"'+e+'","cat":"'+t+'","ts":'+l+',"name":"'+a+'"'),s&&r.writeSync(c,","+s),o&&r.writeSync(c,',"args":'+JSON.stringify(o)),r.writeSync(c,"}"),n.performance.mark("endTracing"),n.performance.measure("Tracing","beginTracing","endTracing"))}function T(e){var t=n.getSourceFileOfNode(e);return t?{path:t.path,start:r(n.getLineAndCharacterOfPosition(t,e.pos)),end:r(n.getLineAndCharacterOfPosition(t,e.end))}:void 0;function r(e){return{line:e.line+1,character:e.character+1}}}function S(e){var t,i,a,s,c,l,d,p,f,m,_,g,y,h,v,b,E,S,x,L,D,A;n.performance.mark("beginDumpTypes");var C=u[u.length-1].typesPath,I=r.openSync(C,"w"),N=new n.Map;r.writeSync(I,"[");for(var k=e.length,R=0;R<k;R++){var O=e[R],w=O.objectFlags,P=null!==(t=O.aliasSymbol)&&void 0!==t?t:O.symbol,M=void 0;if(16&w|2944&O.flags)try{M=null===(i=O.checker)||void 0===i?void 0:i.typeToString(O)}catch(Z){M=void 0}var F={};if(8388608&O.flags){var G=O;F={indexedAccessObjectType:null===(a=G.objectType)||void 0===a?void 0:a.id,indexedAccessIndexType:null===(s=G.indexType)||void 0===s?void 0:s.id}}var B={};if(4&w){var U=O;B={instantiatedType:null===(c=U.target)||void 0===c?void 0:c.id,typeArguments:null===(l=U.resolvedTypeArguments)||void 0===l?void 0:l.map(function(e){return e.id}),referenceLocation:T(U.node)}}var V={};if(16777216&O.flags){var j=O;V={conditionalCheckType:null===(d=j.checkType)||void 0===d?void 0:d.id,conditionalExtendsType:null===(p=j.extendsType)||void 0===p?void 0:p.id,conditionalTrueType:null!==(m=null===(f=j.resolvedTrueType)||void 0===f?void 0:f.id)&&void 0!==m?m:-1,conditionalFalseType:null!==(g=null===(_=j.resolvedFalseType)||void 0===_?void 0:_.id)&&void 0!==g?g:-1}}var K={};if(33554432&O.flags){var H=O;K={substitutionBaseType:null===(y=H.baseType)||void 0===y?void 0:y.id,substituteType:null===(h=H.substitute)||void 0===h?void 0:h.id}}var W={};if(1024&w){var z=O;W={reverseMappedSourceType:null===(v=z.source)||void 0===v?void 0:v.id,reverseMappedMappedType:null===(b=z.mappedType)||void 0===b?void 0:b.id,reverseMappedConstraintType:null===(E=z.constraintType)||void 0===E?void 0:E.id}}var J={};if(256&w){var q=O;J={evolvingArrayElementType:q.elementType.id,evolvingArrayFinalType:null===(S=q.finalArrayType)||void 0===S?void 0:S.id}}var X=void 0,Y=O.checker.getRecursionIdentity(O);Y&&(X=N.get(Y),X||(X=N.size,N.set(Y,X)));var Q=o(o(o(o(o(o(o({id:O.id,intrinsicName:O.intrinsicName,symbolName:(null===P||void 0===P?void 0:P.escapedName)&&n.unescapeLeadingUnderscores(P.escapedName),recursionId:X,isTuple:!!(8&w)||void 0,unionTypes:1048576&O.flags?null===(x=O.types)||void 0===x?void 0:x.map(function(e){return e.id}):void 0,intersectionTypes:2097152&O.flags?O.types.map(function(e){return e.id}):void 0,aliasTypeArguments:null===(L=O.aliasTypeArguments)||void 0===L?void 0:L.map(function(e){return e.id}),keyofType:4194304&O.flags?null===(D=O.type)||void 0===D?void 0:D.id:void 0},F),B),V),K),W),J),{destructuringPattern:T(O.pattern),firstDeclaration:T(null===(A=null===P||void 0===P?void 0:P.declarations)||void 0===A?void 0:A[0]),flags:n.Debug.formatTypeFlags(O.flags).split("|"),display:M});r.writeSync(I,JSON.stringify(Q)),R<k-1&&r.writeSync(I,",\n")}r.writeSync(I,"]\n"),r.closeSync(I),n.performance.mark("endDumpTypes"),n.performance.measure("Dump types","beginDumpTypes","endDumpTypes")}function x(){a&&r.writeFileSync(a,JSON.stringify(u))}t.dumpLegend=x})(t||(t={})),n.startTracing=t.startTracing,n.dumpTracingLegend=t.dumpLegend}(d||(d={})),function(e){(function(e){e[e["Unknown"]=0]="Unknown",e[e["EndOfFileToken"]=1]="EndOfFileToken",e[e["SingleLineCommentTrivia"]=2]="SingleLineCommentTrivia",e[e["MultiLineCommentTrivia"]=3]="MultiLineCommentTrivia",e[e["NewLineTrivia"]=4]="NewLineTrivia",e[e["WhitespaceTrivia"]=5]="WhitespaceTrivia",e[e["ShebangTrivia"]=6]="ShebangTrivia",e[e["ConflictMarkerTrivia"]=7]="ConflictMarkerTrivia",e[e["NumericLiteral"]=8]="NumericLiteral",e[e["BigIntLiteral"]=9]="BigIntLiteral",e[e["StringLiteral"]=10]="StringLiteral",e[e["JsxText"]=11]="JsxText",e[e["JsxTextAllWhiteSpaces"]=12]="JsxTextAllWhiteSpaces",e[e["RegularExpressionLiteral"]=13]="RegularExpressionLiteral",e[e["NoSubstitutionTemplateLiteral"]=14]="NoSubstitutionTemplateLiteral",e[e["TemplateHead"]=15]="TemplateHead",e[e["TemplateMiddle"]=16]="TemplateMiddle",e[e["TemplateTail"]=17]="TemplateTail",e[e["OpenBraceToken"]=18]="OpenBraceToken",e[e["CloseBraceToken"]=19]="CloseBraceToken",e[e["OpenParenToken"]=20]="OpenParenToken",e[e["CloseParenToken"]=21]="CloseParenToken",e[e["OpenBracketToken"]=22]="OpenBracketToken",e[e["CloseBracketToken"]=23]="CloseBracketToken",e[e["DotToken"]=24]="DotToken",e[e["DotDotDotToken"]=25]="DotDotDotToken",e[e["SemicolonToken"]=26]="SemicolonToken",e[e["CommaToken"]=27]="CommaToken",e[e["QuestionDotToken"]=28]="QuestionDotToken",e[e["LessThanToken"]=29]="LessThanToken",e[e["LessThanSlashToken"]=30]="LessThanSlashToken",e[e["GreaterThanToken"]=31]="GreaterThanToken",e[e["LessThanEqualsToken"]=32]="LessThanEqualsToken",e[e["GreaterThanEqualsToken"]=33]="GreaterThanEqualsToken",e[e["EqualsEqualsToken"]=34]="EqualsEqualsToken",e[e["ExclamationEqualsToken"]=35]="ExclamationEqualsToken",e[e["EqualsEqualsEqualsToken"]=36]="EqualsEqualsEqualsToken",e[e["ExclamationEqualsEqualsToken"]=37]="ExclamationEqualsEqualsToken",e[e["EqualsGreaterThanToken"]=38]="EqualsGreaterThanToken",e[e["PlusToken"]=39]="PlusToken",e[e["MinusToken"]=40]="MinusToken",e[e["AsteriskToken"]=41]="AsteriskToken",e[e["AsteriskAsteriskToken"]=42]="AsteriskAsteriskToken",e[e["SlashToken"]=43]="SlashToken",e[e["PercentToken"]=44]="PercentToken",e[e["PlusPlusToken"]=45]="PlusPlusToken",e[e["MinusMinusToken"]=46]="MinusMinusToken",e[e["LessThanLessThanToken"]=47]="LessThanLessThanToken",e[e["GreaterThanGreaterThanToken"]=48]="GreaterThanGreaterThanToken",e[e["GreaterThanGreaterThanGreaterThanToken"]=49]="GreaterThanGreaterThanGreaterThanToken",e[e["AmpersandToken"]=50]="AmpersandToken",e[e["BarToken"]=51]="BarToken",e[e["CaretToken"]=52]="CaretToken",e[e["ExclamationToken"]=53]="ExclamationToken",e[e["TildeToken"]=54]="TildeToken",e[e["AmpersandAmpersandToken"]=55]="AmpersandAmpersandToken",e[e["BarBarToken"]=56]="BarBarToken",e[e["QuestionToken"]=57]="QuestionToken",e[e["ColonToken"]=58]="ColonToken",e[e["AtToken"]=59]="AtToken",e[e["QuestionQuestionToken"]=60]="QuestionQuestionToken",e[e["BacktickToken"]=61]="BacktickToken",e[e["HashToken"]=62]="HashToken",e[e["EqualsToken"]=63]="EqualsToken",e[e["PlusEqualsToken"]=64]="PlusEqualsToken",e[e["MinusEqualsToken"]=65]="MinusEqualsToken",e[e["AsteriskEqualsToken"]=66]="AsteriskEqualsToken",e[e["AsteriskAsteriskEqualsToken"]=67]="AsteriskAsteriskEqualsToken",e[e["SlashEqualsToken"]=68]="SlashEqualsToken",e[e["PercentEqualsToken"]=69]="PercentEqualsToken",e[e["LessThanLessThanEqualsToken"]=70]="LessThanLessThanEqualsToken",e[e["GreaterThanGreaterThanEqualsToken"]=71]="GreaterThanGreaterThanEqualsToken",e[e["GreaterThanGreaterThanGreaterThanEqualsToken"]=72]="GreaterThanGreaterThanGreaterThanEqualsToken",e[e["AmpersandEqualsToken"]=73]="AmpersandEqualsToken",e[e["BarEqualsToken"]=74]="BarEqualsToken",e[e["BarBarEqualsToken"]=75]="BarBarEqualsToken",e[e["AmpersandAmpersandEqualsToken"]=76]="AmpersandAmpersandEqualsToken",e[e["QuestionQuestionEqualsToken"]=77]="QuestionQuestionEqualsToken",e[e["CaretEqualsToken"]=78]="CaretEqualsToken",e[e["Identifier"]=79]="Identifier",e[e["PrivateIdentifier"]=80]="PrivateIdentifier",e[e["BreakKeyword"]=81]="BreakKeyword",e[e["CaseKeyword"]=82]="CaseKeyword",e[e["CatchKeyword"]=83]="CatchKeyword",e[e["ClassKeyword"]=84]="ClassKeyword",e[e["ConstKeyword"]=85]="ConstKeyword",e[e["ContinueKeyword"]=86]="ContinueKeyword",e[e["DebuggerKeyword"]=87]="DebuggerKeyword",e[e["DefaultKeyword"]=88]="DefaultKeyword",e[e["DeleteKeyword"]=89]="DeleteKeyword",e[e["DoKeyword"]=90]="DoKeyword",e[e["ElseKeyword"]=91]="ElseKeyword",e[e["EnumKeyword"]=92]="EnumKeyword",e[e["ExportKeyword"]=93]="ExportKeyword",e[e["ExtendsKeyword"]=94]="ExtendsKeyword",e[e["FalseKeyword"]=95]="FalseKeyword",e[e["FinallyKeyword"]=96]="FinallyKeyword",e[e["ForKeyword"]=97]="ForKeyword",e[e["FunctionKeyword"]=98]="FunctionKeyword",e[e["IfKeyword"]=99]="IfKeyword",e[e["ImportKeyword"]=100]="ImportKeyword",e[e["InKeyword"]=101]="InKeyword",e[e["InstanceOfKeyword"]=102]="InstanceOfKeyword",e[e["NewKeyword"]=103]="NewKeyword",e[e["NullKeyword"]=104]="NullKeyword",e[e["ReturnKeyword"]=105]="ReturnKeyword",e[e["SuperKeyword"]=106]="SuperKeyword",e[e["SwitchKeyword"]=107]="SwitchKeyword",e[e["ThisKeyword"]=108]="ThisKeyword",e[e["ThrowKeyword"]=109]="ThrowKeyword",e[e["TrueKeyword"]=110]="TrueKeyword",e[e["TryKeyword"]=111]="TryKeyword",e[e["TypeOfKeyword"]=112]="TypeOfKeyword",e[e["VarKeyword"]=113]="VarKeyword",e[e["VoidKeyword"]=114]="VoidKeyword",e[e["WhileKeyword"]=115]="WhileKeyword",e[e["WithKeyword"]=116]="WithKeyword",e[e["ImplementsKeyword"]=117]="ImplementsKeyword",e[e["InterfaceKeyword"]=118]="InterfaceKeyword",e[e["LetKeyword"]=119]="LetKeyword",e[e["PackageKeyword"]=120]="PackageKeyword",e[e["PrivateKeyword"]=121]="PrivateKeyword",e[e["ProtectedKeyword"]=122]="ProtectedKeyword",e[e["PublicKeyword"]=123]="PublicKeyword",e[e["StaticKeyword"]=124]="StaticKeyword",e[e["YieldKeyword"]=125]="YieldKeyword",e[e["AbstractKeyword"]=126]="AbstractKeyword",e[e["AsKeyword"]=127]="AsKeyword",e[e["AssertsKeyword"]=128]="AssertsKeyword",e[e["AnyKeyword"]=129]="AnyKeyword",e[e["AsyncKeyword"]=130]="AsyncKeyword",e[e["AwaitKeyword"]=131]="AwaitKeyword",e[e["BooleanKeyword"]=132]="BooleanKeyword",e[e["ConstructorKeyword"]=133]="ConstructorKeyword",e[e["DeclareKeyword"]=134]="DeclareKeyword",e[e["GetKeyword"]=135]="GetKeyword",e[e["InferKeyword"]=136]="InferKeyword",e[e["IntrinsicKeyword"]=137]="IntrinsicKeyword",e[e["IsKeyword"]=138]="IsKeyword",e[e["KeyOfKeyword"]=139]="KeyOfKeyword",e[e["ModuleKeyword"]=140]="ModuleKeyword",e[e["NamespaceKeyword"]=141]="NamespaceKeyword",e[e["NeverKeyword"]=142]="NeverKeyword",e[e["ReadonlyKeyword"]=143]="ReadonlyKeyword",e[e["RequireKeyword"]=144]="RequireKeyword",e[e["NumberKeyword"]=145]="NumberKeyword",e[e["ObjectKeyword"]=146]="ObjectKeyword",e[e["SetKeyword"]=147]="SetKeyword",e[e["StringKeyword"]=148]="StringKeyword",e[e["SymbolKeyword"]=149]="SymbolKeyword",e[e["TypeKeyword"]=150]="TypeKeyword",e[e["UndefinedKeyword"]=151]="UndefinedKeyword",e[e["UniqueKeyword"]=152]="UniqueKeyword",e[e["UnknownKeyword"]=153]="UnknownKeyword",e[e["FromKeyword"]=154]="FromKeyword",e[e["GlobalKeyword"]=155]="GlobalKeyword",e[e["BigIntKeyword"]=156]="BigIntKeyword",e[e["OverrideKeyword"]=157]="OverrideKeyword",e[e["OfKeyword"]=158]="OfKeyword",e[e["QualifiedName"]=159]="QualifiedName",e[e["ComputedPropertyName"]=160]="ComputedPropertyName",e[e["TypeParameter"]=161]="TypeParameter",e[e["Parameter"]=162]="Parameter",e[e["Decorator"]=163]="Decorator",e[e["PropertySignature"]=164]="PropertySignature",e[e["PropertyDeclaration"]=165]="PropertyDeclaration",e[e["MethodSignature"]=166]="MethodSignature",e[e["MethodDeclaration"]=167]="MethodDeclaration",e[e["ClassStaticBlockDeclaration"]=168]="ClassStaticBlockDeclaration",e[e["Constructor"]=169]="Constructor",e[e["GetAccessor"]=170]="GetAccessor",e[e["SetAccessor"]=171]="SetAccessor",e[e["CallSignature"]=172]="CallSignature",e[e["ConstructSignature"]=173]="ConstructSignature",e[e["IndexSignature"]=174]="IndexSignature",e[e["TypePredicate"]=175]="TypePredicate",e[e["TypeReference"]=176]="TypeReference",e[e["FunctionType"]=177]="FunctionType",e[e["ConstructorType"]=178]="ConstructorType",e[e["TypeQuery"]=179]="TypeQuery",e[e["TypeLiteral"]=180]="TypeLiteral",e[e["ArrayType"]=181]="ArrayType",e[e["TupleType"]=182]="TupleType",e[e["OptionalType"]=183]="OptionalType",e[e["RestType"]=184]="RestType",e[e["UnionType"]=185]="UnionType",e[e["IntersectionType"]=186]="IntersectionType",e[e["ConditionalType"]=187]="ConditionalType",e[e["InferType"]=188]="InferType",e[e["ParenthesizedType"]=189]="ParenthesizedType",e[e["ThisType"]=190]="ThisType",e[e["TypeOperator"]=191]="TypeOperator",e[e["IndexedAccessType"]=192]="IndexedAccessType",e[e["MappedType"]=193]="MappedType",e[e["LiteralType"]=194]="LiteralType",e[e["NamedTupleMember"]=195]="NamedTupleMember",e[e["TemplateLiteralType"]=196]="TemplateLiteralType",e[e["TemplateLiteralTypeSpan"]=197]="TemplateLiteralTypeSpan",e[e["ImportType"]=198]="ImportType",e[e["ObjectBindingPattern"]=199]="ObjectBindingPattern",e[e["ArrayBindingPattern"]=200]="ArrayBindingPattern",e[e["BindingElement"]=201]="BindingElement",e[e["ArrayLiteralExpression"]=202]="ArrayLiteralExpression",e[e["ObjectLiteralExpression"]=203]="ObjectLiteralExpression",e[e["PropertyAccessExpression"]=204]="PropertyAccessExpression",e[e["ElementAccessExpression"]=205]="ElementAccessExpression",e[e["CallExpression"]=206]="CallExpression",e[e["NewExpression"]=207]="NewExpression",e[e["TaggedTemplateExpression"]=208]="TaggedTemplateExpression",e[e["TypeAssertionExpression"]=209]="TypeAssertionExpression",e[e["ParenthesizedExpression"]=210]="ParenthesizedExpression",e[e["FunctionExpression"]=211]="FunctionExpression",e[e["ArrowFunction"]=212]="ArrowFunction",e[e["DeleteExpression"]=213]="DeleteExpression",e[e["TypeOfExpression"]=214]="TypeOfExpression",e[e["VoidExpression"]=215]="VoidExpression",e[e["AwaitExpression"]=216]="AwaitExpression",e[e["PrefixUnaryExpression"]=217]="PrefixUnaryExpression",e[e["PostfixUnaryExpression"]=218]="PostfixUnaryExpression",e[e["BinaryExpression"]=219]="BinaryExpression",e[e["ConditionalExpression"]=220]="ConditionalExpression",e[e["TemplateExpression"]=221]="TemplateExpression",e[e["YieldExpression"]=222]="YieldExpression",e[e["SpreadElement"]=223]="SpreadElement",e[e["ClassExpression"]=224]="ClassExpression",e[e["OmittedExpression"]=225]="OmittedExpression",e[e["ExpressionWithTypeArguments"]=226]="ExpressionWithTypeArguments",e[e["AsExpression"]=227]="AsExpression",e[e["NonNullExpression"]=228]="NonNullExpression",e[e["MetaProperty"]=229]="MetaProperty",e[e["SyntheticExpression"]=230]="SyntheticExpression",e[e["TemplateSpan"]=231]="TemplateSpan",e[e["SemicolonClassElement"]=232]="SemicolonClassElement",e[e["Block"]=233]="Block",e[e["EmptyStatement"]=234]="EmptyStatement",e[e["VariableStatement"]=235]="VariableStatement",e[e["ExpressionStatement"]=236]="ExpressionStatement",e[e["IfStatement"]=237]="IfStatement",e[e["DoStatement"]=238]="DoStatement",e[e["WhileStatement"]=239]="WhileStatement",e[e["ForStatement"]=240]="ForStatement",e[e["ForInStatement"]=241]="ForInStatement",e[e["ForOfStatement"]=242]="ForOfStatement",e[e["ContinueStatement"]=243]="ContinueStatement",e[e["BreakStatement"]=244]="BreakStatement",e[e["ReturnStatement"]=245]="ReturnStatement",e[e["WithStatement"]=246]="WithStatement",e[e["SwitchStatement"]=247]="SwitchStatement",e[e["LabeledStatement"]=248]="LabeledStatement",e[e["ThrowStatement"]=249]="ThrowStatement",e[e["TryStatement"]=250]="TryStatement",e[e["DebuggerStatement"]=251]="DebuggerStatement",e[e["VariableDeclaration"]=252]="VariableDeclaration",e[e["VariableDeclarationList"]=253]="VariableDeclarationList",e[e["FunctionDeclaration"]=254]="FunctionDeclaration",e[e["ClassDeclaration"]=255]="ClassDeclaration",e[e["InterfaceDeclaration"]=256]="InterfaceDeclaration",e[e["TypeAliasDeclaration"]=257]="TypeAliasDeclaration",e[e["EnumDeclaration"]=258]="EnumDeclaration",e[e["ModuleDeclaration"]=259]="ModuleDeclaration",e[e["ModuleBlock"]=260]="ModuleBlock",e[e["CaseBlock"]=261]="CaseBlock",e[e["NamespaceExportDeclaration"]=262]="NamespaceExportDeclaration",e[e["ImportEqualsDeclaration"]=263]="ImportEqualsDeclaration",e[e["ImportDeclaration"]=264]="ImportDeclaration",e[e["ImportClause"]=265]="ImportClause",e[e["NamespaceImport"]=266]="NamespaceImport",e[e["NamedImports"]=267]="NamedImports",e[e["ImportSpecifier"]=268]="ImportSpecifier",e[e["ExportAssignment"]=269]="ExportAssignment",e[e["ExportDeclaration"]=270]="ExportDeclaration",e[e["NamedExports"]=271]="NamedExports",e[e["NamespaceExport"]=272]="NamespaceExport",e[e["ExportSpecifier"]=273]="ExportSpecifier",e[e["MissingDeclaration"]=274]="MissingDeclaration",e[e["ExternalModuleReference"]=275]="ExternalModuleReference",e[e["JsxElement"]=276]="JsxElement",e[e["JsxSelfClosingElement"]=277]="JsxSelfClosingElement",e[e["JsxOpeningElement"]=278]="JsxOpeningElement",e[e["JsxClosingElement"]=279]="JsxClosingElement",e[e["JsxFragment"]=280]="JsxFragment",e[e["JsxOpeningFragment"]=281]="JsxOpeningFragment",e[e["JsxClosingFragment"]=282]="JsxClosingFragment",e[e["JsxAttribute"]=283]="JsxAttribute",e[e["JsxAttributes"]=284]="JsxAttributes",e[e["JsxSpreadAttribute"]=285]="JsxSpreadAttribute",e[e["JsxExpression"]=286]="JsxExpression",e[e["CaseClause"]=287]="CaseClause",e[e["DefaultClause"]=288]="DefaultClause",e[e["HeritageClause"]=289]="HeritageClause",e[e["CatchClause"]=290]="CatchClause",e[e["PropertyAssignment"]=291]="PropertyAssignment",e[e["ShorthandPropertyAssignment"]=292]="ShorthandPropertyAssignment",e[e["SpreadAssignment"]=293]="SpreadAssignment",e[e["EnumMember"]=294]="EnumMember",e[e["UnparsedPrologue"]=295]="UnparsedPrologue",e[e["UnparsedPrepend"]=296]="UnparsedPrepend",e[e["UnparsedText"]=297]="UnparsedText",e[e["UnparsedInternalText"]=298]="UnparsedInternalText",e[e["UnparsedSyntheticReference"]=299]="UnparsedSyntheticReference",e[e["SourceFile"]=300]="SourceFile",e[e["Bundle"]=301]="Bundle",e[e["UnparsedSource"]=302]="UnparsedSource",e[e["InputFiles"]=303]="InputFiles",e[e["JSDocTypeExpression"]=304]="JSDocTypeExpression",e[e["JSDocNameReference"]=305]="JSDocNameReference",e[e["JSDocMemberName"]=306]="JSDocMemberName",e[e["JSDocAllType"]=307]="JSDocAllType",e[e["JSDocUnknownType"]=308]="JSDocUnknownType",e[e["JSDocNullableType"]=309]="JSDocNullableType",e[e["JSDocNonNullableType"]=310]="JSDocNonNullableType",e[e["JSDocOptionalType"]=311]="JSDocOptionalType",e[e["JSDocFunctionType"]=312]="JSDocFunctionType",e[e["JSDocVariadicType"]=313]="JSDocVariadicType",e[e["JSDocNamepathType"]=314]="JSDocNamepathType",e[e["JSDocComment"]=315]="JSDocComment",e[e["JSDocText"]=316]="JSDocText",e[e["JSDocTypeLiteral"]=317]="JSDocTypeLiteral",e[e["JSDocSignature"]=318]="JSDocSignature",e[e["JSDocLink"]=319]="JSDocLink",e[e["JSDocLinkCode"]=320]="JSDocLinkCode",e[e["JSDocLinkPlain"]=321]="JSDocLinkPlain",e[e["JSDocTag"]=322]="JSDocTag",e[e["JSDocAugmentsTag"]=323]="JSDocAugmentsTag",e[e["JSDocImplementsTag"]=324]="JSDocImplementsTag",e[e["JSDocAuthorTag"]=325]="JSDocAuthorTag",e[e["JSDocDeprecatedTag"]=326]="JSDocDeprecatedTag",e[e["JSDocClassTag"]=327]="JSDocClassTag",e[e["JSDocPublicTag"]=328]="JSDocPublicTag",e[e["JSDocPrivateTag"]=329]="JSDocPrivateTag",e[e["JSDocProtectedTag"]=330]="JSDocProtectedTag",e[e["JSDocReadonlyTag"]=331]="JSDocReadonlyTag",e[e["JSDocOverrideTag"]=332]="JSDocOverrideTag",e[e["JSDocCallbackTag"]=333]="JSDocCallbackTag",e[e["JSDocEnumTag"]=334]="JSDocEnumTag",e[e["JSDocParameterTag"]=335]="JSDocParameterTag",e[e["JSDocReturnTag"]=336]="JSDocReturnTag",e[e["JSDocThisTag"]=337]="JSDocThisTag",e[e["JSDocTypeTag"]=338]="JSDocTypeTag",e[e["JSDocTemplateTag"]=339]="JSDocTemplateTag",e[e["JSDocTypedefTag"]=340]="JSDocTypedefTag",e[e["JSDocSeeTag"]=341]="JSDocSeeTag",e[e["JSDocPropertyTag"]=342]="JSDocPropertyTag",e[e["SyntaxList"]=343]="SyntaxList",e[e["NotEmittedStatement"]=344]="NotEmittedStatement",e[e["PartiallyEmittedExpression"]=345]="PartiallyEmittedExpression",e[e["CommaListExpression"]=346]="CommaListExpression",e[e["MergeDeclarationMarker"]=347]="MergeDeclarationMarker",e[e["EndOfDeclarationMarker"]=348]="EndOfDeclarationMarker",e[e["SyntheticReferenceExpression"]=349]="SyntheticReferenceExpression",e[e["Count"]=350]="Count",e[e["FirstAssignment"]=63]="FirstAssignment",e[e["LastAssignment"]=78]="LastAssignment",e[e["FirstCompoundAssignment"]=64]="FirstCompoundAssignment",e[e["LastCompoundAssignment"]=78]="LastCompoundAssignment",e[e["FirstReservedWord"]=81]="FirstReservedWord",e[e["LastReservedWord"]=116]="LastReservedWord",e[e["FirstKeyword"]=81]="FirstKeyword",e[e["LastKeyword"]=158]="LastKeyword",e[e["FirstFutureReservedWord"]=117]="FirstFutureReservedWord",e[e["LastFutureReservedWord"]=125]="LastFutureReservedWord",e[e["FirstTypeNode"]=175]="FirstTypeNode",e[e["LastTypeNode"]=198]="LastTypeNode",e[e["FirstPunctuation"]=18]="FirstPunctuation",e[e["LastPunctuation"]=78]="LastPunctuation",e[e["FirstToken"]=0]="FirstToken",e[e["LastToken"]=158]="LastToken",e[e["FirstTriviaToken"]=2]="FirstTriviaToken",e[e["LastTriviaToken"]=7]="LastTriviaToken",e[e["FirstLiteralToken"]=8]="FirstLiteralToken",e[e["LastLiteralToken"]=14]="LastLiteralToken",e[e["FirstTemplateToken"]=14]="FirstTemplateToken",e[e["LastTemplateToken"]=17]="LastTemplateToken",e[e["FirstBinaryOperator"]=29]="FirstBinaryOperator",e[e["LastBinaryOperator"]=78]="LastBinaryOperator",e[e["FirstStatement"]=235]="FirstStatement",e[e["LastStatement"]=251]="LastStatement",e[e["FirstNode"]=159]="FirstNode",e[e["FirstJSDocNode"]=304]="FirstJSDocNode",e[e["LastJSDocNode"]=342]="LastJSDocNode",e[e["FirstJSDocTagNode"]=322]="FirstJSDocTagNode",e[e["LastJSDocTagNode"]=342]="LastJSDocTagNode",e[e["FirstContextualKeyword"]=126]="FirstContextualKeyword",e[e["LastContextualKeyword"]=158]="LastContextualKeyword"})(e.SyntaxKind||(e.SyntaxKind={})),function(e){e[e["None"]=0]="None",e[e["Let"]=1]="Let",e[e["Const"]=2]="Const",e[e["NestedNamespace"]=4]="NestedNamespace",e[e["Synthesized"]=8]="Synthesized",e[e["Namespace"]=16]="Namespace",e[e["OptionalChain"]=32]="OptionalChain",e[e["ExportContext"]=64]="ExportContext",e[e["ContainsThis"]=128]="ContainsThis",e[e["HasImplicitReturn"]=256]="HasImplicitReturn",e[e["HasExplicitReturn"]=512]="HasExplicitReturn",e[e["GlobalAugmentation"]=1024]="GlobalAugmentation",e[e["HasAsyncFunctions"]=2048]="HasAsyncFunctions",e[e["DisallowInContext"]=4096]="DisallowInContext",e[e["YieldContext"]=8192]="YieldContext",e[e["DecoratorContext"]=16384]="DecoratorContext",e[e["AwaitContext"]=32768]="AwaitContext",e[e["ThisNodeHasError"]=65536]="ThisNodeHasError",e[e["JavaScriptFile"]=131072]="JavaScriptFile",e[e["ThisNodeOrAnySubNodesHasError"]=262144]="ThisNodeOrAnySubNodesHasError",e[e["HasAggregatedChildData"]=524288]="HasAggregatedChildData",e[e["PossiblyContainsDynamicImport"]=1048576]="PossiblyContainsDynamicImport",e[e["PossiblyContainsImportMeta"]=2097152]="PossiblyContainsImportMeta",e[e["JSDoc"]=4194304]="JSDoc",e[e["Ambient"]=8388608]="Ambient",e[e["InWithStatement"]=16777216]="InWithStatement",e[e["JsonFile"]=33554432]="JsonFile",e[e["TypeCached"]=67108864]="TypeCached",e[e["Deprecated"]=134217728]="Deprecated",e[e["BlockScoped"]=3]="BlockScoped",e[e["ReachabilityCheckFlags"]=768]="ReachabilityCheckFlags",e[e["ReachabilityAndEmitFlags"]=2816]="ReachabilityAndEmitFlags",e[e["ContextFlags"]=25358336]="ContextFlags",e[e["TypeExcludesFlags"]=40960]="TypeExcludesFlags",e[e["PermanentlySetIncrementalFlags"]=3145728]="PermanentlySetIncrementalFlags"}(e.NodeFlags||(e.NodeFlags={})),function(e){e[e["None"]=0]="None",e[e["Export"]=1]="Export",e[e["Ambient"]=2]="Ambient",e[e["Public"]=4]="Public",e[e["Private"]=8]="Private",e[e["Protected"]=16]="Protected",e[e["Static"]=32]="Static",e[e["Readonly"]=64]="Readonly",e[e["Abstract"]=128]="Abstract",e[e["Async"]=256]="Async",e[e["Default"]=512]="Default",e[e["Const"]=2048]="Const",e[e["HasComputedJSDocModifiers"]=4096]="HasComputedJSDocModifiers",e[e["Deprecated"]=8192]="Deprecated",e[e["Override"]=16384]="Override",e[e["HasComputedFlags"]=536870912]="HasComputedFlags",e[e["AccessibilityModifier"]=28]="AccessibilityModifier",e[e["ParameterPropertyModifier"]=16476]="ParameterPropertyModifier",e[e["NonPublicAccessibilityModifier"]=24]="NonPublicAccessibilityModifier",e[e["TypeScriptModifier"]=18654]="TypeScriptModifier",e[e["ExportDefault"]=513]="ExportDefault",e[e["All"]=27647]="All"}(e.ModifierFlags||(e.ModifierFlags={})),function(e){e[e["None"]=0]="None",e[e["IntrinsicNamedElement"]=1]="IntrinsicNamedElement",e[e["IntrinsicIndexedElement"]=2]="IntrinsicIndexedElement",e[e["IntrinsicElement"]=3]="IntrinsicElement"}(e.JsxFlags||(e.JsxFlags={})),function(e){e[e["Succeeded"]=1]="Succeeded",e[e["Failed"]=2]="Failed",e[e["Reported"]=4]="Reported",e[e["ReportsUnmeasurable"]=8]="ReportsUnmeasurable",e[e["ReportsUnreliable"]=16]="ReportsUnreliable",e[e["ReportsMask"]=24]="ReportsMask"}(e.RelationComparisonResult||(e.RelationComparisonResult={})),function(e){e[e["None"]=0]="None",e[e["Auto"]=1]="Auto",e[e["Loop"]=2]="Loop",e[e["Unique"]=3]="Unique",e[e["Node"]=4]="Node",e[e["KindMask"]=7]="KindMask",e[e["ReservedInNestedScopes"]=8]="ReservedInNestedScopes",e[e["Optimistic"]=16]="Optimistic",e[e["FileLevel"]=32]="FileLevel",e[e["AllowNameSubstitution"]=64]="AllowNameSubstitution"}(e.GeneratedIdentifierFlags||(e.GeneratedIdentifierFlags={})),function(e){e[e["None"]=0]="None",e[e["PrecedingLineBreak"]=1]="PrecedingLineBreak",e[e["PrecedingJSDocComment"]=2]="PrecedingJSDocComment",e[e["Unterminated"]=4]="Unterminated",e[e["ExtendedUnicodeEscape"]=8]="ExtendedUnicodeEscape",e[e["Scientific"]=16]="Scientific",e[e["Octal"]=32]="Octal",e[e["HexSpecifier"]=64]="HexSpecifier",e[e["BinarySpecifier"]=128]="BinarySpecifier",e[e["OctalSpecifier"]=256]="OctalSpecifier",e[e["ContainsSeparator"]=512]="ContainsSeparator",e[e["UnicodeEscape"]=1024]="UnicodeEscape",e[e["ContainsInvalidEscape"]=2048]="ContainsInvalidEscape",e[e["BinaryOrOctalSpecifier"]=384]="BinaryOrOctalSpecifier",e[e["NumericLiteralFlags"]=1008]="NumericLiteralFlags",e[e["TemplateLiteralLikeFlags"]=2048]="TemplateLiteralLikeFlags"}(e.TokenFlags||(e.TokenFlags={})),function(e){e[e["Unreachable"]=1]="Unreachable",e[e["Start"]=2]="Start",e[e["BranchLabel"]=4]="BranchLabel",e[e["LoopLabel"]=8]="LoopLabel",e[e["Assignment"]=16]="Assignment",e[e["TrueCondition"]=32]="TrueCondition",e[e["FalseCondition"]=64]="FalseCondition",e[e["SwitchClause"]=128]="SwitchClause",e[e["ArrayMutation"]=256]="ArrayMutation",e[e["Call"]=512]="Call",e[e["ReduceLabel"]=1024]="ReduceLabel",e[e["Referenced"]=2048]="Referenced",e[e["Shared"]=4096]="Shared",e[e["Label"]=12]="Label",e[e["Condition"]=96]="Condition"}(e.FlowFlags||(e.FlowFlags={})),function(e){e[e["ExpectError"]=0]="ExpectError",e[e["Ignore"]=1]="Ignore"}(e.CommentDirectiveType||(e.CommentDirectiveType={}));var n,t=function(){function e(){}return e}();function r(e,t){void 0===t&&(t=!0);var r=n[e.category];return t?r.toLowerCase():r}e.OperationCanceledException=t,function(e){e[e["RootFile"]=0]="RootFile",e[e["SourceFromProjectReference"]=1]="SourceFromProjectReference",e[e["OutputFromProjectReference"]=2]="OutputFromProjectReference",e[e["Import"]=3]="Import",e[e["ReferenceFile"]=4]="ReferenceFile",e[e["TypeReferenceDirective"]=5]="TypeReferenceDirective",e[e["LibFile"]=6]="LibFile",e[e["LibReferenceDirective"]=7]="LibReferenceDirective",e[e["AutomaticTypeDirectiveFile"]=8]="AutomaticTypeDirectiveFile"}(e.FileIncludeKind||(e.FileIncludeKind={})),function(e){e[e["FilePreprocessingReferencedDiagnostic"]=0]="FilePreprocessingReferencedDiagnostic",e[e["FilePreprocessingFileExplainingDiagnostic"]=1]="FilePreprocessingFileExplainingDiagnostic"}(e.FilePreprocessingDiagnosticsKind||(e.FilePreprocessingDiagnosticsKind={})),function(e){e[e["Not"]=0]="Not",e[e["SafeModules"]=1]="SafeModules",e[e["Completely"]=2]="Completely"}(e.StructureIsReused||(e.StructureIsReused={})),function(e){e[e["Success"]=0]="Success",e[e["DiagnosticsPresent_OutputsSkipped"]=1]="DiagnosticsPresent_OutputsSkipped",e[e["DiagnosticsPresent_OutputsGenerated"]=2]="DiagnosticsPresent_OutputsGenerated",e[e["InvalidProject_OutputsSkipped"]=3]="InvalidProject_OutputsSkipped",e[e["ProjectReferenceCycle_OutputsSkipped"]=4]="ProjectReferenceCycle_OutputsSkipped",e[e["ProjectReferenceCycle_OutputsSkupped"]=4]="ProjectReferenceCycle_OutputsSkupped"}(e.ExitStatus||(e.ExitStatus={})),function(e){e[e["None"]=0]="None",e[e["Literal"]=1]="Literal",e[e["Subtype"]=2]="Subtype"}(e.UnionReduction||(e.UnionReduction={})),function(e){e[e["None"]=0]="None",e[e["Signature"]=1]="Signature",e[e["NoConstraints"]=2]="NoConstraints",e[e["Completions"]=4]="Completions",e[e["SkipBindingPatterns"]=8]="SkipBindingPatterns"}(e.ContextFlags||(e.ContextFlags={})),function(e){e[e["None"]=0]="None",e[e["NoTruncation"]=1]="NoTruncation",e[e["WriteArrayAsGenericType"]=2]="WriteArrayAsGenericType",e[e["GenerateNamesForShadowedTypeParams"]=4]="GenerateNamesForShadowedTypeParams",e[e["UseStructuralFallback"]=8]="UseStructuralFallback",e[e["ForbidIndexedAccessSymbolReferences"]=16]="ForbidIndexedAccessSymbolReferences",e[e["WriteTypeArgumentsOfSignature"]=32]="WriteTypeArgumentsOfSignature",e[e["UseFullyQualifiedType"]=64]="UseFullyQualifiedType",e[e["UseOnlyExternalAliasing"]=128]="UseOnlyExternalAliasing",e[e["SuppressAnyReturnType"]=256]="SuppressAnyReturnType",e[e["WriteTypeParametersInQualifiedName"]=512]="WriteTypeParametersInQualifiedName",e[e["MultilineObjectLiterals"]=1024]="MultilineObjectLiterals",e[e["WriteClassExpressionAsTypeLiteral"]=2048]="WriteClassExpressionAsTypeLiteral",e[e["UseTypeOfFunction"]=4096]="UseTypeOfFunction",e[e["OmitParameterModifiers"]=8192]="OmitParameterModifiers",e[e["UseAliasDefinedOutsideCurrentScope"]=16384]="UseAliasDefinedOutsideCurrentScope",e[e["UseSingleQuotesForStringLiteralType"]=268435456]="UseSingleQuotesForStringLiteralType",e[e["NoTypeReduction"]=536870912]="NoTypeReduction",e[e["NoUndefinedOptionalParameterType"]=1073741824]="NoUndefinedOptionalParameterType",e[e["AllowThisInObjectLiteral"]=32768]="AllowThisInObjectLiteral",e[e["AllowQualifiedNameInPlaceOfIdentifier"]=65536]="AllowQualifiedNameInPlaceOfIdentifier",e[e["AllowQualifedNameInPlaceOfIdentifier"]=65536]="AllowQualifedNameInPlaceOfIdentifier",e[e["AllowAnonymousIdentifier"]=131072]="AllowAnonymousIdentifier",e[e["AllowEmptyUnionOrIntersection"]=262144]="AllowEmptyUnionOrIntersection",e[e["AllowEmptyTuple"]=524288]="AllowEmptyTuple",e[e["AllowUniqueESSymbolType"]=1048576]="AllowUniqueESSymbolType",e[e["AllowEmptyIndexInfoType"]=2097152]="AllowEmptyIndexInfoType",e[e["AllowNodeModulesRelativePaths"]=67108864]="AllowNodeModulesRelativePaths",e[e["DoNotIncludeSymbolChain"]=134217728]="DoNotIncludeSymbolChain",e[e["IgnoreErrors"]=70221824]="IgnoreErrors",e[e["InObjectTypeLiteral"]=4194304]="InObjectTypeLiteral",e[e["InTypeAlias"]=8388608]="InTypeAlias",e[e["InInitialEntityName"]=16777216]="InInitialEntityName"}(e.NodeBuilderFlags||(e.NodeBuilderFlags={})),function(e){e[e["None"]=0]="None",e[e["NoTruncation"]=1]="NoTruncation",e[e["WriteArrayAsGenericType"]=2]="WriteArrayAsGenericType",e[e["UseStructuralFallback"]=8]="UseStructuralFallback",e[e["WriteTypeArgumentsOfSignature"]=32]="WriteTypeArgumentsOfSignature",e[e["UseFullyQualifiedType"]=64]="UseFullyQualifiedType",e[e["SuppressAnyReturnType"]=256]="SuppressAnyReturnType",e[e["MultilineObjectLiterals"]=1024]="MultilineObjectLiterals",e[e["WriteClassExpressionAsTypeLiteral"]=2048]="WriteClassExpressionAsTypeLiteral",e[e["UseTypeOfFunction"]=4096]="UseTypeOfFunction",e[e["OmitParameterModifiers"]=8192]="OmitParameterModifiers",e[e["UseAliasDefinedOutsideCurrentScope"]=16384]="UseAliasDefinedOutsideCurrentScope",e[e["UseSingleQuotesForStringLiteralType"]=268435456]="UseSingleQuotesForStringLiteralType",e[e["NoTypeReduction"]=536870912]="NoTypeReduction",e[e["AllowUniqueESSymbolType"]=1048576]="AllowUniqueESSymbolType",e[e["AddUndefined"]=131072]="AddUndefined",e[e["WriteArrowStyleSignature"]=262144]="WriteArrowStyleSignature",e[e["InArrayType"]=524288]="InArrayType",e[e["InElementType"]=2097152]="InElementType",e[e["InFirstTypeArgument"]=4194304]="InFirstTypeArgument",e[e["InTypeAlias"]=8388608]="InTypeAlias",e[e["WriteOwnNameForAnyLike"]=0]="WriteOwnNameForAnyLike",e[e["NodeBuilderFlagsMask"]=814775659]="NodeBuilderFlagsMask"}(e.TypeFormatFlags||(e.TypeFormatFlags={})),function(e){e[e["None"]=0]="None",e[e["WriteTypeParametersOrArguments"]=1]="WriteTypeParametersOrArguments",e[e["UseOnlyExternalAliasing"]=2]="UseOnlyExternalAliasing",e[e["AllowAnyNodeKind"]=4]="AllowAnyNodeKind",e[e["UseAliasDefinedOutsideCurrentScope"]=8]="UseAliasDefinedOutsideCurrentScope",e[e["DoNotIncludeSymbolChain"]=16]="DoNotIncludeSymbolChain"}(e.SymbolFormatFlags||(e.SymbolFormatFlags={})),function(e){e[e["Accessible"]=0]="Accessible",e[e["NotAccessible"]=1]="NotAccessible",e[e["CannotBeNamed"]=2]="CannotBeNamed"}(e.SymbolAccessibility||(e.SymbolAccessibility={})),function(e){e[e["UnionOrIntersection"]=0]="UnionOrIntersection",e[e["Spread"]=1]="Spread"}(e.SyntheticSymbolKind||(e.SyntheticSymbolKind={})),function(e){e[e["This"]=0]="This",e[e["Identifier"]=1]="Identifier",e[e["AssertsThis"]=2]="AssertsThis",e[e["AssertsIdentifier"]=3]="AssertsIdentifier"}(e.TypePredicateKind||(e.TypePredicateKind={})),function(e){e[e["Unknown"]=0]="Unknown",e[e["TypeWithConstructSignatureAndValue"]=1]="TypeWithConstructSignatureAndValue",e[e["VoidNullableOrNeverType"]=2]="VoidNullableOrNeverType",e[e["NumberLikeType"]=3]="NumberLikeType",e[e["BigIntLikeType"]=4]="BigIntLikeType",e[e["StringLikeType"]=5]="StringLikeType",e[e["BooleanType"]=6]="BooleanType",e[e["ArrayLikeType"]=7]="ArrayLikeType",e[e["ESSymbolType"]=8]="ESSymbolType",e[e["Promise"]=9]="Promise",e[e["TypeWithCallSignature"]=10]="TypeWithCallSignature",e[e["ObjectType"]=11]="ObjectType"}(e.TypeReferenceSerializationKind||(e.TypeReferenceSerializationKind={})),function(e){e[e["None"]=0]="None",e[e["FunctionScopedVariable"]=1]="FunctionScopedVariable",e[e["BlockScopedVariable"]=2]="BlockScopedVariable",e[e["Property"]=4]="Property",e[e["EnumMember"]=8]="EnumMember",e[e["Function"]=16]="Function",e[e["Class"]=32]="Class",e[e["Interface"]=64]="Interface",e[e["ConstEnum"]=128]="ConstEnum",e[e["RegularEnum"]=256]="RegularEnum",e[e["ValueModule"]=512]="ValueModule",e[e["NamespaceModule"]=1024]="NamespaceModule",e[e["TypeLiteral"]=2048]="TypeLiteral",e[e["ObjectLiteral"]=4096]="ObjectLiteral",e[e["Method"]=8192]="Method",e[e["Constructor"]=16384]="Constructor",e[e["GetAccessor"]=32768]="GetAccessor",e[e["SetAccessor"]=65536]="SetAccessor",e[e["Signature"]=131072]="Signature",e[e["TypeParameter"]=262144]="TypeParameter",e[e["TypeAlias"]=524288]="TypeAlias",e[e["ExportValue"]=1048576]="ExportValue",e[e["Alias"]=2097152]="Alias",e[e["Prototype"]=4194304]="Prototype",e[e["ExportStar"]=8388608]="ExportStar",e[e["Optional"]=16777216]="Optional",e[e["Transient"]=33554432]="Transient",e[e["Assignment"]=67108864]="Assignment",e[e["ModuleExports"]=134217728]="ModuleExports",e[e["All"]=67108863]="All",e[e["Enum"]=384]="Enum",e[e["Variable"]=3]="Variable",e[e["Value"]=111551]="Value",e[e["Type"]=788968]="Type",e[e["Namespace"]=1920]="Namespace",e[e["Module"]=1536]="Module",e[e["Accessor"]=98304]="Accessor",e[e["FunctionScopedVariableExcludes"]=111550]="FunctionScopedVariableExcludes",e[e["BlockScopedVariableExcludes"]=111551]="BlockScopedVariableExcludes",e[e["ParameterExcludes"]=111551]="ParameterExcludes",e[e["PropertyExcludes"]=0]="PropertyExcludes",e[e["EnumMemberExcludes"]=900095]="EnumMemberExcludes",e[e["FunctionExcludes"]=110991]="FunctionExcludes",e[e["ClassExcludes"]=899503]="ClassExcludes",e[e["InterfaceExcludes"]=788872]="InterfaceExcludes",e[e["RegularEnumExcludes"]=899327]="RegularEnumExcludes",e[e["ConstEnumExcludes"]=899967]="ConstEnumExcludes",e[e["ValueModuleExcludes"]=110735]="ValueModuleExcludes",e[e["NamespaceModuleExcludes"]=0]="NamespaceModuleExcludes",e[e["MethodExcludes"]=103359]="MethodExcludes",e[e["GetAccessorExcludes"]=46015]="GetAccessorExcludes",e[e["SetAccessorExcludes"]=78783]="SetAccessorExcludes",e[e["TypeParameterExcludes"]=526824]="TypeParameterExcludes",e[e["TypeAliasExcludes"]=788968]="TypeAliasExcludes",e[e["AliasExcludes"]=2097152]="AliasExcludes",e[e["ModuleMember"]=2623475]="ModuleMember",e[e["ExportHasLocal"]=944]="ExportHasLocal",e[e["BlockScoped"]=418]="BlockScoped",e[e["PropertyOrAccessor"]=98308]="PropertyOrAccessor",e[e["ClassMember"]=106500]="ClassMember",e[e["ExportSupportsDefaultModifier"]=112]="ExportSupportsDefaultModifier",e[e["ExportDoesNotSupportDefaultModifier"]=-113]="ExportDoesNotSupportDefaultModifier",e[e["Classifiable"]=2885600]="Classifiable",e[e["LateBindingContainer"]=6256]="LateBindingContainer"}(e.SymbolFlags||(e.SymbolFlags={})),function(e){e[e["Numeric"]=0]="Numeric",e[e["Literal"]=1]="Literal"}(e.EnumKind||(e.EnumKind={})),function(e){e[e["Instantiated"]=1]="Instantiated",e[e["SyntheticProperty"]=2]="SyntheticProperty",e[e["SyntheticMethod"]=4]="SyntheticMethod",e[e["Readonly"]=8]="Readonly",e[e["ReadPartial"]=16]="ReadPartial",e[e["WritePartial"]=32]="WritePartial",e[e["HasNonUniformType"]=64]="HasNonUniformType",e[e["HasLiteralType"]=128]="HasLiteralType",e[e["ContainsPublic"]=256]="ContainsPublic",e[e["ContainsProtected"]=512]="ContainsProtected",e[e["ContainsPrivate"]=1024]="ContainsPrivate",e[e["ContainsStatic"]=2048]="ContainsStatic",e[e["Late"]=4096]="Late",e[e["ReverseMapped"]=8192]="ReverseMapped",e[e["OptionalParameter"]=16384]="OptionalParameter",e[e["RestParameter"]=32768]="RestParameter",e[e["DeferredType"]=65536]="DeferredType",e[e["HasNeverType"]=131072]="HasNeverType",e[e["Mapped"]=262144]="Mapped",e[e["StripOptional"]=524288]="StripOptional",e[e["Synthetic"]=6]="Synthetic",e[e["Discriminant"]=192]="Discriminant",e[e["Partial"]=48]="Partial"}(e.CheckFlags||(e.CheckFlags={})),function(e){e["Call"]="__call",e["Constructor"]="__constructor",e["New"]="__new",e["Index"]="__index",e["ExportStar"]="__export",e["Global"]="__global",e["Missing"]="__missing",e["Type"]="__type",e["Object"]="__object",e["JSXAttributes"]="__jsxAttributes",e["Class"]="__class",e["Function"]="__function",e["Computed"]="__computed",e["Resolving"]="__resolving__",e["ExportEquals"]="export=",e["Default"]="default",e["This"]="this"}(e.InternalSymbolName||(e.InternalSymbolName={})),function(e){e[e["TypeChecked"]=1]="TypeChecked",e[e["LexicalThis"]=2]="LexicalThis",e[e["CaptureThis"]=4]="CaptureThis",e[e["CaptureNewTarget"]=8]="CaptureNewTarget",e[e["SuperInstance"]=256]="SuperInstance",e[e["SuperStatic"]=512]="SuperStatic",e[e["ContextChecked"]=1024]="ContextChecked",e[e["AsyncMethodWithSuper"]=2048]="AsyncMethodWithSuper",e[e["AsyncMethodWithSuperBinding"]=4096]="AsyncMethodWithSuperBinding",e[e["CaptureArguments"]=8192]="CaptureArguments",e[e["EnumValuesComputed"]=16384]="EnumValuesComputed",e[e["LexicalModuleMergesWithClass"]=32768]="LexicalModuleMergesWithClass",e[e["LoopWithCapturedBlockScopedBinding"]=65536]="LoopWithCapturedBlockScopedBinding",e[e["ContainsCapturedBlockScopeBinding"]=131072]="ContainsCapturedBlockScopeBinding",e[e["CapturedBlockScopedBinding"]=262144]="CapturedBlockScopedBinding",e[e["BlockScopedBindingInLoop"]=524288]="BlockScopedBindingInLoop",e[e["ClassWithBodyScopedClassBinding"]=1048576]="ClassWithBodyScopedClassBinding",e[e["BodyScopedClassBinding"]=2097152]="BodyScopedClassBinding",e[e["NeedsLoopOutParameter"]=4194304]="NeedsLoopOutParameter",e[e["AssignmentsMarked"]=8388608]="AssignmentsMarked",e[e["ClassWithConstructorReference"]=16777216]="ClassWithConstructorReference",e[e["ConstructorReferenceInClass"]=33554432]="ConstructorReferenceInClass",e[e["ContainsClassWithPrivateIdentifiers"]=67108864]="ContainsClassWithPrivateIdentifiers",e[e["ContainsSuperPropertyInStaticInitializer"]=134217728]="ContainsSuperPropertyInStaticInitializer"}(e.NodeCheckFlags||(e.NodeCheckFlags={})),function(e){e[e["Any"]=1]="Any",e[e["Unknown"]=2]="Unknown",e[e["String"]=4]="String",e[e["Number"]=8]="Number",e[e["Boolean"]=16]="Boolean",e[e["Enum"]=32]="Enum",e[e["BigInt"]=64]="BigInt",e[e["StringLiteral"]=128]="StringLiteral",e[e["NumberLiteral"]=256]="NumberLiteral",e[e["BooleanLiteral"]=512]="BooleanLiteral",e[e["EnumLiteral"]=1024]="EnumLiteral",e[e["BigIntLiteral"]=2048]="BigIntLiteral",e[e["ESSymbol"]=4096]="ESSymbol",e[e["UniqueESSymbol"]=8192]="UniqueESSymbol",e[e["Void"]=16384]="Void",e[e["Undefined"]=32768]="Undefined",e[e["Null"]=65536]="Null",e[e["Never"]=131072]="Never",e[e["TypeParameter"]=262144]="TypeParameter",e[e["Object"]=524288]="Object",e[e["Union"]=1048576]="Union",e[e["Intersection"]=2097152]="Intersection",e[e["Index"]=4194304]="Index",e[e["IndexedAccess"]=8388608]="IndexedAccess",e[e["Conditional"]=16777216]="Conditional",e[e["Substitution"]=33554432]="Substitution",e[e["NonPrimitive"]=67108864]="NonPrimitive",e[e["TemplateLiteral"]=134217728]="TemplateLiteral",e[e["StringMapping"]=268435456]="StringMapping",e[e["AnyOrUnknown"]=3]="AnyOrUnknown",e[e["Nullable"]=98304]="Nullable",e[e["Literal"]=2944]="Literal",e[e["Unit"]=109440]="Unit",e[e["StringOrNumberLiteral"]=384]="StringOrNumberLiteral",e[e["StringOrNumberLiteralOrUnique"]=8576]="StringOrNumberLiteralOrUnique",e[e["DefinitelyFalsy"]=117632]="DefinitelyFalsy",e[e["PossiblyFalsy"]=117724]="PossiblyFalsy",e[e["Intrinsic"]=67359327]="Intrinsic",e[e["Primitive"]=131068]="Primitive",e[e["StringLike"]=402653316]="StringLike",e[e["NumberLike"]=296]="NumberLike",e[e["BigIntLike"]=2112]="BigIntLike",e[e["BooleanLike"]=528]="BooleanLike",e[e["EnumLike"]=1056]="EnumLike",e[e["ESSymbolLike"]=12288]="ESSymbolLike",e[e["VoidLike"]=49152]="VoidLike",e[e["DisjointDomains"]=469892092]="DisjointDomains",e[e["UnionOrIntersection"]=3145728]="UnionOrIntersection",e[e["StructuredType"]=3670016]="StructuredType",e[e["TypeVariable"]=8650752]="TypeVariable",e[e["InstantiableNonPrimitive"]=58982400]="InstantiableNonPrimitive",e[e["InstantiablePrimitive"]=406847488]="InstantiablePrimitive",e[e["Instantiable"]=465829888]="Instantiable",e[e["StructuredOrInstantiable"]=469499904]="StructuredOrInstantiable",e[e["ObjectFlagsType"]=3899393]="ObjectFlagsType",e[e["Simplifiable"]=25165824]="Simplifiable",e[e["Singleton"]=67358815]="Singleton",e[e["Narrowable"]=536624127]="Narrowable",e[e["NotPrimitiveUnion"]=468598819]="NotPrimitiveUnion",e[e["IncludesMask"]=205258751]="IncludesMask",e[e["IncludesStructuredOrInstantiable"]=262144]="IncludesStructuredOrInstantiable",e[e["IncludesNonWideningType"]=4194304]="IncludesNonWideningType",e[e["IncludesWildcard"]=8388608]="IncludesWildcard",e[e["IncludesEmptyObject"]=16777216]="IncludesEmptyObject"}(e.TypeFlags||(e.TypeFlags={})),function(e){e[e["Class"]=1]="Class",e[e["Interface"]=2]="Interface",e[e["Reference"]=4]="Reference",e[e["Tuple"]=8]="Tuple",e[e["Anonymous"]=16]="Anonymous",e[e["Mapped"]=32]="Mapped",e[e["Instantiated"]=64]="Instantiated",e[e["ObjectLiteral"]=128]="ObjectLiteral",e[e["EvolvingArray"]=256]="EvolvingArray",e[e["ObjectLiteralPatternWithComputedProperties"]=512]="ObjectLiteralPatternWithComputedProperties",e[e["ReverseMapped"]=1024]="ReverseMapped",e[e["JsxAttributes"]=2048]="JsxAttributes",e[e["MarkerType"]=4096]="MarkerType",e[e["JSLiteral"]=8192]="JSLiteral",e[e["FreshLiteral"]=16384]="FreshLiteral",e[e["ArrayLiteral"]=32768]="ArrayLiteral",e[e["PrimitiveUnion"]=65536]="PrimitiveUnion",e[e["ContainsWideningType"]=131072]="ContainsWideningType",e[e["ContainsObjectOrArrayLiteral"]=262144]="ContainsObjectOrArrayLiteral",e[e["NonInferrableType"]=524288]="NonInferrableType",e[e["CouldContainTypeVariablesComputed"]=1048576]="CouldContainTypeVariablesComputed",e[e["CouldContainTypeVariables"]=2097152]="CouldContainTypeVariables",e[e["ClassOrInterface"]=3]="ClassOrInterface",e[e["RequiresWidening"]=393216]="RequiresWidening",e[e["PropagatingFlags"]=917504]="PropagatingFlags",e[e["ObjectTypeKindMask"]=1343]="ObjectTypeKindMask",e[e["ContainsSpread"]=4194304]="ContainsSpread",e[e["ObjectRestType"]=8388608]="ObjectRestType",e[e["IsClassInstanceClone"]=16777216]="IsClassInstanceClone",e[e["IdenticalBaseTypeCalculated"]=33554432]="IdenticalBaseTypeCalculated",e[e["IdenticalBaseTypeExists"]=67108864]="IdenticalBaseTypeExists",e[e["IsGenericTypeComputed"]=4194304]="IsGenericTypeComputed",e[e["IsGenericObjectType"]=8388608]="IsGenericObjectType",e[e["IsGenericIndexType"]=16777216]="IsGenericIndexType",e[e["IsGenericType"]=25165824]="IsGenericType",e[e["ContainsIntersections"]=33554432]="ContainsIntersections",e[e["IsNeverIntersectionComputed"]=33554432]="IsNeverIntersectionComputed",e[e["IsNeverIntersection"]=67108864]="IsNeverIntersection"}(e.ObjectFlags||(e.ObjectFlags={})),function(e){e[e["Invariant"]=0]="Invariant",e[e["Covariant"]=1]="Covariant",e[e["Contravariant"]=2]="Contravariant",e[e["Bivariant"]=3]="Bivariant",e[e["Independent"]=4]="Independent",e[e["VarianceMask"]=7]="VarianceMask",e[e["Unmeasurable"]=8]="Unmeasurable",e[e["Unreliable"]=16]="Unreliable",e[e["AllowsStructuralFallback"]=24]="AllowsStructuralFallback"}(e.VarianceFlags||(e.VarianceFlags={})),function(e){e[e["Required"]=1]="Required",e[e["Optional"]=2]="Optional",e[e["Rest"]=4]="Rest",e[e["Variadic"]=8]="Variadic",e[e["Fixed"]=3]="Fixed",e[e["Variable"]=12]="Variable",e[e["NonRequired"]=14]="NonRequired",e[e["NonRest"]=11]="NonRest"}(e.ElementFlags||(e.ElementFlags={})),function(e){e[e["None"]=0]="None",e[e["IncludeUndefined"]=1]="IncludeUndefined",e[e["NoIndexSignatures"]=2]="NoIndexSignatures",e[e["Writing"]=4]="Writing",e[e["CacheSymbol"]=8]="CacheSymbol",e[e["NoTupleBoundsCheck"]=16]="NoTupleBoundsCheck",e[e["ExpressionPosition"]=32]="ExpressionPosition",e[e["ReportDeprecated"]=64]="ReportDeprecated",e[e["SuppressNoImplicitAnyError"]=128]="SuppressNoImplicitAnyError",e[e["Contextual"]=256]="Contextual",e[e["Persistent"]=1]="Persistent"}(e.AccessFlags||(e.AccessFlags={})),function(e){e[e["Component"]=0]="Component",e[e["Function"]=1]="Function",e[e["Mixed"]=2]="Mixed"}(e.JsxReferenceKind||(e.JsxReferenceKind={})),function(e){e[e["Call"]=0]="Call",e[e["Construct"]=1]="Construct"}(e.SignatureKind||(e.SignatureKind={})),function(e){e[e["None"]=0]="None",e[e["HasRestParameter"]=1]="HasRestParameter",e[e["HasLiteralTypes"]=2]="HasLiteralTypes",e[e["Abstract"]=4]="Abstract",e[e["IsInnerCallChain"]=8]="IsInnerCallChain",e[e["IsOuterCallChain"]=16]="IsOuterCallChain",e[e["IsUntypedSignatureInJSFile"]=32]="IsUntypedSignatureInJSFile",e[e["PropagatingFlags"]=39]="PropagatingFlags",e[e["CallChainFlags"]=24]="CallChainFlags"}(e.SignatureFlags||(e.SignatureFlags={})),function(e){e[e["String"]=0]="String",e[e["Number"]=1]="Number"}(e.IndexKind||(e.IndexKind={})),function(e){e[e["Simple"]=0]="Simple",e[e["Array"]=1]="Array",e[e["Function"]=2]="Function",e[e["Composite"]=3]="Composite",e[e["Merged"]=4]="Merged"}(e.TypeMapKind||(e.TypeMapKind={})),function(e){e[e["NakedTypeVariable"]=1]="NakedTypeVariable",e[e["SpeculativeTuple"]=2]="SpeculativeTuple",e[e["SubstituteSource"]=4]="SubstituteSource",e[e["HomomorphicMappedType"]=8]="HomomorphicMappedType",e[e["PartialHomomorphicMappedType"]=16]="PartialHomomorphicMappedType",e[e["MappedTypeConstraint"]=32]="MappedTypeConstraint",e[e["ContravariantConditional"]=64]="ContravariantConditional",e[e["ReturnType"]=128]="ReturnType",e[e["LiteralKeyof"]=256]="LiteralKeyof",e[e["NoConstraints"]=512]="NoConstraints",e[e["AlwaysStrict"]=1024]="AlwaysStrict",e[e["MaxValue"]=2048]="MaxValue",e[e["PriorityImpliesCombination"]=416]="PriorityImpliesCombination",e[e["Circularity"]=-1]="Circularity"}(e.InferencePriority||(e.InferencePriority={})),function(e){e[e["None"]=0]="None",e[e["NoDefault"]=1]="NoDefault",e[e["AnyDefault"]=2]="AnyDefault",e[e["SkippedGenericFunction"]=4]="SkippedGenericFunction"}(e.InferenceFlags||(e.InferenceFlags={})),function(e){e[e["False"]=0]="False",e[e["Unknown"]=1]="Unknown",e[e["Maybe"]=3]="Maybe",e[e["True"]=-1]="True"}(e.Ternary||(e.Ternary={})),function(e){e[e["None"]=0]="None",e[e["ExportsProperty"]=1]="ExportsProperty",e[e["ModuleExports"]=2]="ModuleExports",e[e["PrototypeProperty"]=3]="PrototypeProperty",e[e["ThisProperty"]=4]="ThisProperty",e[e["Property"]=5]="Property",e[e["Prototype"]=6]="Prototype",e[e["ObjectDefinePropertyValue"]=7]="ObjectDefinePropertyValue",e[e["ObjectDefinePropertyExports"]=8]="ObjectDefinePropertyExports",e[e["ObjectDefinePrototypeProperty"]=9]="ObjectDefinePrototypeProperty"}(e.AssignmentDeclarationKind||(e.AssignmentDeclarationKind={})),function(e){e[e["Warning"]=0]="Warning",e[e["Error"]=1]="Error",e[e["Suggestion"]=2]="Suggestion",e[e["Message"]=3]="Message"}(n=e.DiagnosticCategory||(e.DiagnosticCategory={})),e.diagnosticCategoryName=r,function(e){e[e["Classic"]=1]="Classic",e[e["NodeJs"]=2]="NodeJs"}(e.ModuleResolutionKind||(e.ModuleResolutionKind={})),function(e){e[e["FixedPollingInterval"]=0]="FixedPollingInterval",e[e["PriorityPollingInterval"]=1]="PriorityPollingInterval",e[e["DynamicPriorityPolling"]=2]="DynamicPriorityPolling",e[e["FixedChunkSizePolling"]=3]="FixedChunkSizePolling",e[e["UseFsEvents"]=4]="UseFsEvents",e[e["UseFsEventsOnParentDirectory"]=5]="UseFsEventsOnParentDirectory"}(e.WatchFileKind||(e.WatchFileKind={})),function(e){e[e["UseFsEvents"]=0]="UseFsEvents",e[e["FixedPollingInterval"]=1]="FixedPollingInterval",e[e["DynamicPriorityPolling"]=2]="DynamicPriorityPolling",e[e["FixedChunkSizePolling"]=3]="FixedChunkSizePolling"}(e.WatchDirectoryKind||(e.WatchDirectoryKind={})),function(e){e[e["FixedInterval"]=0]="FixedInterval",e[e["PriorityInterval"]=1]="PriorityInterval",e[e["DynamicPriority"]=2]="DynamicPriority",e[e["FixedChunkSize"]=3]="FixedChunkSize"}(e.PollingWatchKind||(e.PollingWatchKind={})),function(e){e[e["None"]=0]="None",e[e["CommonJS"]=1]="CommonJS",e[e["AMD"]=2]="AMD",e[e["UMD"]=3]="UMD",e[e["System"]=4]="System",e[e["ES2015"]=5]="ES2015",e[e["ES2020"]=6]="ES2020",e[e["ESNext"]=99]="ESNext"}(e.ModuleKind||(e.ModuleKind={})),function(e){e[e["None"]=0]="None",e[e["Preserve"]=1]="Preserve",e[e["React"]=2]="React",e[e["ReactNative"]=3]="ReactNative",e[e["ReactJSX"]=4]="ReactJSX",e[e["ReactJSXDev"]=5]="ReactJSXDev"}(e.JsxEmit||(e.JsxEmit={})),function(e){e[e["Remove"]=0]="Remove",e[e["Preserve"]=1]="Preserve",e[e["Error"]=2]="Error"}(e.ImportsNotUsedAsValues||(e.ImportsNotUsedAsValues={})),function(e){e[e["CarriageReturnLineFeed"]=0]="CarriageReturnLineFeed",e[e["LineFeed"]=1]="LineFeed"}(e.NewLineKind||(e.NewLineKind={})),function(e){e[e["Unknown"]=0]="Unknown",e[e["JS"]=1]="JS",e[e["JSX"]=2]="JSX",e[e["TS"]=3]="TS",e[e["TSX"]=4]="TSX",e[e["External"]=5]="External",e[e["JSON"]=6]="JSON",e[e["Deferred"]=7]="Deferred"}(e.ScriptKind||(e.ScriptKind={})),function(e){e[e["ES3"]=0]="ES3",e[e["ES5"]=1]="ES5",e[e["ES2015"]=2]="ES2015",e[e["ES2016"]=3]="ES2016",e[e["ES2017"]=4]="ES2017",e[e["ES2018"]=5]="ES2018",e[e["ES2019"]=6]="ES2019",e[e["ES2020"]=7]="ES2020",e[e["ES2021"]=8]="ES2021",e[e["ESNext"]=99]="ESNext",e[e["JSON"]=100]="JSON",e[e["Latest"]=99]="Latest"}(e.ScriptTarget||(e.ScriptTarget={})),function(e){e[e["Standard"]=0]="Standard",e[e["JSX"]=1]="JSX"}(e.LanguageVariant||(e.LanguageVariant={})),function(e){e[e["None"]=0]="None",e[e["Recursive"]=1]="Recursive"}(e.WatchDirectoryFlags||(e.WatchDirectoryFlags={})),function(e){e[e["nullCharacter"]=0]="nullCharacter",e[e["maxAsciiCharacter"]=127]="maxAsciiCharacter",e[e["lineFeed"]=10]="lineFeed",e[e["carriageReturn"]=13]="carriageReturn",e[e["lineSeparator"]=8232]="lineSeparator",e[e["paragraphSeparator"]=8233]="paragraphSeparator",e[e["nextLine"]=133]="nextLine",e[e["space"]=32]="space",e[e["nonBreakingSpace"]=160]="nonBreakingSpace",e[e["enQuad"]=8192]="enQuad",e[e["emQuad"]=8193]="emQuad",e[e["enSpace"]=8194]="enSpace",e[e["emSpace"]=8195]="emSpace",e[e["threePerEmSpace"]=8196]="threePerEmSpace",e[e["fourPerEmSpace"]=8197]="fourPerEmSpace",e[e["sixPerEmSpace"]=8198]="sixPerEmSpace",e[e["figureSpace"]=8199]="figureSpace",e[e["punctuationSpace"]=8200]="punctuationSpace",e[e["thinSpace"]=8201]="thinSpace",e[e["hairSpace"]=8202]="hairSpace",e[e["zeroWidthSpace"]=8203]="zeroWidthSpace",e[e["narrowNoBreakSpace"]=8239]="narrowNoBreakSpace",e[e["ideographicSpace"]=12288]="ideographicSpace",e[e["mathematicalSpace"]=8287]="mathematicalSpace",e[e["ogham"]=5760]="ogham",e[e["_"]=95]="_",e[e["$"]=36]="$",e[e["_0"]=48]="_0",e[e["_1"]=49]="_1",e[e["_2"]=50]="_2",e[e["_3"]=51]="_3",e[e["_4"]=52]="_4",e[e["_5"]=53]="_5",e[e["_6"]=54]="_6",e[e["_7"]=55]="_7",e[e["_8"]=56]="_8",e[e["_9"]=57]="_9",e[e["a"]=97]="a",e[e["b"]=98]="b",e[e["c"]=99]="c",e[e["d"]=100]="d",e[e["e"]=101]="e",e[e["f"]=102]="f",e[e["g"]=103]="g",e[e["h"]=104]="h",e[e["i"]=105]="i",e[e["j"]=106]="j",e[e["k"]=107]="k",e[e["l"]=108]="l",e[e["m"]=109]="m",e[e["n"]=110]="n",e[e["o"]=111]="o",e[e["p"]=112]="p",e[e["q"]=113]="q",e[e["r"]=114]="r",e[e["s"]=115]="s",e[e["t"]=116]="t",e[e["u"]=117]="u",e[e["v"]=118]="v",e[e["w"]=119]="w",e[e["x"]=120]="x",e[e["y"]=121]="y",e[e["z"]=122]="z",e[e["A"]=65]="A",e[e["B"]=66]="B",e[e["C"]=67]="C",e[e["D"]=68]="D",e[e["E"]=69]="E",e[e["F"]=70]="F",e[e["G"]=71]="G",e[e["H"]=72]="H",e[e["I"]=73]="I",e[e["J"]=74]="J",e[e["K"]=75]="K",e[e["L"]=76]="L",e[e["M"]=77]="M",e[e["N"]=78]="N",e[e["O"]=79]="O",e[e["P"]=80]="P",e[e["Q"]=81]="Q",e[e["R"]=82]="R",e[e["S"]=83]="S",e[e["T"]=84]="T",e[e["U"]=85]="U",e[e["V"]=86]="V",e[e["W"]=87]="W",e[e["X"]=88]="X",e[e["Y"]=89]="Y",e[e["Z"]=90]="Z",e[e["ampersand"]=38]="ampersand",e[e["asterisk"]=42]="asterisk",e[e["at"]=64]="at",e[e["backslash"]=92]="backslash",e[e["backtick"]=96]="backtick",e[e["bar"]=124]="bar",e[e["caret"]=94]="caret",e[e["closeBrace"]=125]="closeBrace",e[e["closeBracket"]=93]="closeBracket",e[e["closeParen"]=41]="closeParen",e[e["colon"]=58]="colon",e[e["comma"]=44]="comma",e[e["dot"]=46]="dot",e[e["doubleQuote"]=34]="doubleQuote",e[e["equals"]=61]="equals",e[e["exclamation"]=33]="exclamation",e[e["greaterThan"]=62]="greaterThan",e[e["hash"]=35]="hash",e[e["lessThan"]=60]="lessThan",e[e["minus"]=45]="minus",e[e["openBrace"]=123]="openBrace",e[e["openBracket"]=91]="openBracket",e[e["openParen"]=40]="openParen",e[e["percent"]=37]="percent",e[e["plus"]=43]="plus",e[e["question"]=63]="question",e[e["semicolon"]=59]="semicolon",e[e["singleQuote"]=39]="singleQuote",e[e["slash"]=47]="slash",e[e["tilde"]=126]="tilde",e[e["backspace"]=8]="backspace",e[e["formFeed"]=12]="formFeed",e[e["byteOrderMark"]=65279]="byteOrderMark",e[e["tab"]=9]="tab",e[e["verticalTab"]=11]="verticalTab"}(e.CharacterCodes||(e.CharacterCodes={})),function(e){e["Ts"]=".ts",e["Tsx"]=".tsx",e["Dts"]=".d.ts",e["Js"]=".js",e["Jsx"]=".jsx",e["Json"]=".json",e["TsBuildInfo"]=".tsbuildinfo"}(e.Extension||(e.Extension={})),function(e){e[e["None"]=0]="None",e[e["ContainsTypeScript"]=1]="ContainsTypeScript",e[e["ContainsJsx"]=2]="ContainsJsx",e[e["ContainsESNext"]=4]="ContainsESNext",e[e["ContainsES2021"]=8]="ContainsES2021",e[e["ContainsES2020"]=16]="ContainsES2020",e[e["ContainsES2019"]=32]="ContainsES2019",e[e["ContainsES2018"]=64]="ContainsES2018",e[e["ContainsES2017"]=128]="ContainsES2017",e[e["ContainsES2016"]=256]="ContainsES2016",e[e["ContainsES2015"]=512]="ContainsES2015",e[e["ContainsGenerator"]=1024]="ContainsGenerator",e[e["ContainsDestructuringAssignment"]=2048]="ContainsDestructuringAssignment",e[e["ContainsTypeScriptClassSyntax"]=4096]="ContainsTypeScriptClassSyntax",e[e["ContainsLexicalThis"]=8192]="ContainsLexicalThis",e[e["ContainsRestOrSpread"]=16384]="ContainsRestOrSpread",e[e["ContainsObjectRestOrSpread"]=32768]="ContainsObjectRestOrSpread",e[e["ContainsComputedPropertyName"]=65536]="ContainsComputedPropertyName",e[e["ContainsBlockScopedBinding"]=131072]="ContainsBlockScopedBinding",e[e["ContainsBindingPattern"]=262144]="ContainsBindingPattern",e[e["ContainsYield"]=524288]="ContainsYield",e[e["ContainsAwait"]=1048576]="ContainsAwait",e[e["ContainsHoistedDeclarationOrCompletion"]=2097152]="ContainsHoistedDeclarationOrCompletion",e[e["ContainsDynamicImport"]=4194304]="ContainsDynamicImport",e[e["ContainsClassFields"]=8388608]="ContainsClassFields",e[e["ContainsPossibleTopLevelAwait"]=16777216]="ContainsPossibleTopLevelAwait",e[e["ContainsLexicalSuper"]=33554432]="ContainsLexicalSuper",e[e["ContainsUpdateExpressionForIdentifier"]=67108864]="ContainsUpdateExpressionForIdentifier",e[e["HasComputedFlags"]=536870912]="HasComputedFlags",e[e["AssertTypeScript"]=1]="AssertTypeScript",e[e["AssertJsx"]=2]="AssertJsx",e[e["AssertESNext"]=4]="AssertESNext",e[e["AssertES2021"]=8]="AssertES2021",e[e["AssertES2020"]=16]="AssertES2020",e[e["AssertES2019"]=32]="AssertES2019",e[e["AssertES2018"]=64]="AssertES2018",e[e["AssertES2017"]=128]="AssertES2017",e[e["AssertES2016"]=256]="AssertES2016",e[e["AssertES2015"]=512]="AssertES2015",e[e["AssertGenerator"]=1024]="AssertGenerator",e[e["AssertDestructuringAssignment"]=2048]="AssertDestructuringAssignment",e[e["OuterExpressionExcludes"]=536870912]="OuterExpressionExcludes",e[e["PropertyAccessExcludes"]=536870912]="PropertyAccessExcludes",e[e["NodeExcludes"]=536870912]="NodeExcludes",e[e["ArrowFunctionExcludes"]=557748224]="ArrowFunctionExcludes",e[e["FunctionExcludes"]=591310848]="FunctionExcludes",e[e["ConstructorExcludes"]=591306752]="ConstructorExcludes",e[e["MethodOrAccessorExcludes"]=574529536]="MethodOrAccessorExcludes",e[e["PropertyExcludes"]=570433536]="PropertyExcludes",e[e["ClassExcludes"]=536940544]="ClassExcludes",e[e["ModuleExcludes"]=589443072]="ModuleExcludes",e[e["TypeExcludes"]=-2]="TypeExcludes",e[e["ObjectLiteralExcludes"]=536973312]="ObjectLiteralExcludes",e[e["ArrayLiteralOrCallOrNewExcludes"]=536887296]="ArrayLiteralOrCallOrNewExcludes",e[e["VariableDeclarationListExcludes"]=537165824]="VariableDeclarationListExcludes",e[e["ParameterExcludes"]=536870912]="ParameterExcludes",e[e["CatchClauseExcludes"]=536903680]="CatchClauseExcludes",e[e["BindingPatternExcludes"]=536887296]="BindingPatternExcludes",e[e["ContainsLexicalThisOrSuper"]=33562624]="ContainsLexicalThisOrSuper",e[e["PropertyNamePropagatingFlags"]=33562624]="PropertyNamePropagatingFlags"}(e.TransformFlags||(e.TransformFlags={})),function(e){e[e["None"]=0]="None",e[e["SingleLine"]=1]="SingleLine",e[e["AdviseOnEmitNode"]=2]="AdviseOnEmitNode",e[e["NoSubstitution"]=4]="NoSubstitution",e[e["CapturesThis"]=8]="CapturesThis",e[e["NoLeadingSourceMap"]=16]="NoLeadingSourceMap",e[e["NoTrailingSourceMap"]=32]="NoTrailingSourceMap",e[e["NoSourceMap"]=48]="NoSourceMap",e[e["NoNestedSourceMaps"]=64]="NoNestedSourceMaps",e[e["NoTokenLeadingSourceMaps"]=128]="NoTokenLeadingSourceMaps",e[e["NoTokenTrailingSourceMaps"]=256]="NoTokenTrailingSourceMaps",e[e["NoTokenSourceMaps"]=384]="NoTokenSourceMaps",e[e["NoLeadingComments"]=512]="NoLeadingComments",e[e["NoTrailingComments"]=1024]="NoTrailingComments",e[e["NoComments"]=1536]="NoComments",e[e["NoNestedComments"]=2048]="NoNestedComments",e[e["HelperName"]=4096]="HelperName",e[e["ExportName"]=8192]="ExportName",e[e["LocalName"]=16384]="LocalName",e[e["InternalName"]=32768]="InternalName",e[e["Indented"]=65536]="Indented",e[e["NoIndentation"]=131072]="NoIndentation",e[e["AsyncFunctionBody"]=262144]="AsyncFunctionBody",e[e["ReuseTempVariableScope"]=524288]="ReuseTempVariableScope",e[e["CustomPrologue"]=1048576]="CustomPrologue",e[e["NoHoisting"]=2097152]="NoHoisting",e[e["HasEndOfDeclarationMarker"]=4194304]="HasEndOfDeclarationMarker",e[e["Iterator"]=8388608]="Iterator",e[e["NoAsciiEscaping"]=16777216]="NoAsciiEscaping",e[e["TypeScriptClassWrapper"]=33554432]="TypeScriptClassWrapper",e[e["NeverApplyImportHelper"]=67108864]="NeverApplyImportHelper",e[e["IgnoreSourceNewlines"]=134217728]="IgnoreSourceNewlines",e[e["Immutable"]=268435456]="Immutable",e[e["IndirectCall"]=536870912]="IndirectCall"}(e.EmitFlags||(e.EmitFlags={})),function(e){e[e["Extends"]=1]="Extends",e[e["Assign"]=2]="Assign",e[e["Rest"]=4]="Rest",e[e["Decorate"]=8]="Decorate",e[e["Metadata"]=16]="Metadata",e[e["Param"]=32]="Param",e[e["Awaiter"]=64]="Awaiter",e[e["Generator"]=128]="Generator",e[e["Values"]=256]="Values",e[e["Read"]=512]="Read",e[e["SpreadArray"]=1024]="SpreadArray",e[e["Await"]=2048]="Await",e[e["AsyncGenerator"]=4096]="AsyncGenerator",e[e["AsyncDelegator"]=8192]="AsyncDelegator",e[e["AsyncValues"]=16384]="AsyncValues",e[e["ExportStar"]=32768]="ExportStar",e[e["ImportStar"]=65536]="ImportStar",e[e["ImportDefault"]=131072]="ImportDefault",e[e["MakeTemplateObject"]=262144]="MakeTemplateObject",e[e["ClassPrivateFieldGet"]=524288]="ClassPrivateFieldGet",e[e["ClassPrivateFieldSet"]=1048576]="ClassPrivateFieldSet",e[e["CreateBinding"]=2097152]="CreateBinding",e[e["FirstEmitHelper"]=1]="FirstEmitHelper",e[e["LastEmitHelper"]=2097152]="LastEmitHelper",e[e["ForOfIncludes"]=256]="ForOfIncludes",e[e["ForAwaitOfIncludes"]=16384]="ForAwaitOfIncludes",e[e["AsyncGeneratorIncludes"]=6144]="AsyncGeneratorIncludes",e[e["AsyncDelegatorIncludes"]=26624]="AsyncDelegatorIncludes",e[e["SpreadIncludes"]=1536]="SpreadIncludes"}(e.ExternalEmitHelpers||(e.ExternalEmitHelpers={})),function(e){e[e["SourceFile"]=0]="SourceFile",e[e["Expression"]=1]="Expression",e[e["IdentifierName"]=2]="IdentifierName",e[e["MappedTypeParameter"]=3]="MappedTypeParameter",e[e["Unspecified"]=4]="Unspecified",e[e["EmbeddedStatement"]=5]="EmbeddedStatement",e[e["JsxAttributeValue"]=6]="JsxAttributeValue"}(e.EmitHint||(e.EmitHint={})),function(e){e[e["Parentheses"]=1]="Parentheses",e[e["TypeAssertions"]=2]="TypeAssertions",e[e["NonNullAssertions"]=4]="NonNullAssertions",e[e["PartiallyEmittedExpressions"]=8]="PartiallyEmittedExpressions",e[e["Assertions"]=6]="Assertions",e[e["All"]=15]="All"}(e.OuterExpressionKinds||(e.OuterExpressionKinds={})),function(e){e[e["None"]=0]="None",e[e["InParameters"]=1]="InParameters",e[e["VariablesHoistedInParameters"]=2]="VariablesHoistedInParameters"}(e.LexicalEnvironmentFlags||(e.LexicalEnvironmentFlags={})),function(e){e["Prologue"]="prologue",e["EmitHelpers"]="emitHelpers",e["NoDefaultLib"]="no-default-lib",e["Reference"]="reference",e["Type"]="type",e["Lib"]="lib",e["Prepend"]="prepend",e["Text"]="text",e["Internal"]="internal"}(e.BundleFileSectionKind||(e.BundleFileSectionKind={})),function(e){e[e["None"]=0]="None",e[e["SingleLine"]=0]="SingleLine",e[e["MultiLine"]=1]="MultiLine",e[e["PreserveLines"]=2]="PreserveLines",e[e["LinesMask"]=3]="LinesMask",e[e["NotDelimited"]=0]="NotDelimited",e[e["BarDelimited"]=4]="BarDelimited",e[e["AmpersandDelimited"]=8]="AmpersandDelimited",e[e["CommaDelimited"]=16]="CommaDelimited",e[e["AsteriskDelimited"]=32]="AsteriskDelimited",e[e["DelimitersMask"]=60]="DelimitersMask",e[e["AllowTrailingComma"]=64]="AllowTrailingComma",e[e["Indented"]=128]="Indented",e[e["SpaceBetweenBraces"]=256]="SpaceBetweenBraces",e[e["SpaceBetweenSiblings"]=512]="SpaceBetweenSiblings",e[e["Braces"]=1024]="Braces",e[e["Parenthesis"]=2048]="Parenthesis",e[e["AngleBrackets"]=4096]="AngleBrackets",e[e["SquareBrackets"]=8192]="SquareBrackets",e[e["BracketsMask"]=15360]="BracketsMask",e[e["OptionalIfUndefined"]=16384]="OptionalIfUndefined",e[e["OptionalIfEmpty"]=32768]="OptionalIfEmpty",e[e["Optional"]=49152]="Optional",e[e["PreferNewLine"]=65536]="PreferNewLine",e[e["NoTrailingNewLine"]=131072]="NoTrailingNewLine",e[e["NoInterveningComments"]=262144]="NoInterveningComments",e[e["NoSpaceIfEmpty"]=524288]="NoSpaceIfEmpty",e[e["SingleElement"]=1048576]="SingleElement",e[e["SpaceAfterList"]=2097152]="SpaceAfterList",e[e["Modifiers"]=262656]="Modifiers",e[e["HeritageClauses"]=512]="HeritageClauses",e[e["SingleLineTypeLiteralMembers"]=768]="SingleLineTypeLiteralMembers",e[e["MultiLineTypeLiteralMembers"]=32897]="MultiLineTypeLiteralMembers",e[e["SingleLineTupleTypeElements"]=528]="SingleLineTupleTypeElements",e[e["MultiLineTupleTypeElements"]=657]="MultiLineTupleTypeElements",e[e["UnionTypeConstituents"]=516]="UnionTypeConstituents",e[e["IntersectionTypeConstituents"]=520]="IntersectionTypeConstituents",e[e["ObjectBindingPatternElements"]=525136]="ObjectBindingPatternElements",e[e["ArrayBindingPatternElements"]=524880]="ArrayBindingPatternElements",e[e["ObjectLiteralExpressionProperties"]=526226]="ObjectLiteralExpressionProperties",e[e["ArrayLiteralExpressionElements"]=8914]="ArrayLiteralExpressionElements",e[e["CommaListElements"]=528]="CommaListElements",e[e["CallExpressionArguments"]=2576]="CallExpressionArguments",e[e["NewExpressionArguments"]=18960]="NewExpressionArguments",e[e["TemplateExpressionSpans"]=262144]="TemplateExpressionSpans",e[e["SingleLineBlockStatements"]=768]="SingleLineBlockStatements",e[e["MultiLineBlockStatements"]=129]="MultiLineBlockStatements",e[e["VariableDeclarationList"]=528]="VariableDeclarationList",e[e["SingleLineFunctionBodyStatements"]=768]="SingleLineFunctionBodyStatements",e[e["MultiLineFunctionBodyStatements"]=1]="MultiLineFunctionBodyStatements",e[e["ClassHeritageClauses"]=0]="ClassHeritageClauses",e[e["ClassMembers"]=129]="ClassMembers",e[e["InterfaceMembers"]=129]="InterfaceMembers",e[e["EnumMembers"]=145]="EnumMembers",e[e["CaseBlockClauses"]=129]="CaseBlockClauses",e[e["NamedImportsOrExportsElements"]=525136]="NamedImportsOrExportsElements",e[e["JsxElementOrFragmentChildren"]=262144]="JsxElementOrFragmentChildren",e[e["JsxElementAttributes"]=262656]="JsxElementAttributes",e[e["CaseOrDefaultClauseStatements"]=163969]="CaseOrDefaultClauseStatements",e[e["HeritageClauseTypes"]=528]="HeritageClauseTypes",e[e["SourceFileStatements"]=131073]="SourceFileStatements",e[e["Decorators"]=2146305]="Decorators",e[e["TypeArguments"]=53776]="TypeArguments",e[e["TypeParameters"]=53776]="TypeParameters",e[e["Parameters"]=2576]="Parameters",e[e["IndexSignatureParameters"]=8848]="IndexSignatureParameters",e[e["JSDocComment"]=33]="JSDocComment"}(e.ListFormat||(e.ListFormat={})),function(e){e[e["None"]=0]="None",e[e["TripleSlashXML"]=1]="TripleSlashXML",e[e["SingleLine"]=2]="SingleLine",e[e["MultiLine"]=4]="MultiLine",e[e["All"]=7]="All",e[e["Default"]=7]="Default"}(e.PragmaKindFlags||(e.PragmaKindFlags={})),e.commentPragmas={reference:{args:[{name:"types",optional:!0,captureSpan:!0},{name:"lib",optional:!0,captureSpan:!0},{name:"path",optional:!0,captureSpan:!0},{name:"no-default-lib",optional:!0}],kind:1},"amd-dependency":{args:[{name:"path"},{name:"name",optional:!0}],kind:1},"amd-module":{args:[{name:"name"}],kind:1},"ts-check":{kind:2},"ts-nocheck":{kind:2},jsx:{args:[{name:"factory"}],kind:4},jsxfrag:{args:[{name:"factory"}],kind:4},jsximportsource:{args:[{name:"factory"}],kind:4},jsxruntime:{args:[{name:"factory"}],kind:4}}}(d||(d={})),function(e){e.directorySeparator="/",e.altDirectorySeparator="\\";var n="://",t=/\\/g;function r(e){return 47===e||92===e}function i(e){return y(e)<0}function o(e){return y(e)>0}function s(e){var n=y(e);return n>0&&n===e.length}function c(e){return 0!==y(e)}function l(e){return/^\.\.?($|[\\/])/.test(e)}function u(e){return!c(e)&&!l(e)}function d(n){return e.stringContains(b(n),".")}function p(n,t){return n.length>t.length&&e.endsWith(n,t)}function f(e,n){for(var t=0,r=n;t<r.length;t++){var i=r[t];if(p(e,i))return!0}return!1}function m(e){return e.length>0&&r(e.charCodeAt(e.length-1))}function _(e){return e>=97&&e<=122||e>=65&&e<=90}function g(e,n){var t=e.charCodeAt(n);if(58===t)return n+1;if(37===t&&51===e.charCodeAt(n+1)){var r=e.charCodeAt(n+2);if(97===r||65===r)return n+3}return-1}function y(t){if(!t)return 0;var r=t.charCodeAt(0);if(47===r||92===r){if(t.charCodeAt(1)!==r)return 1;var i=t.indexOf(47===r?e.directorySeparator:e.altDirectorySeparator,2);return i<0?t.length:i+1}if(_(r)&&58===t.charCodeAt(1)){var a=t.charCodeAt(2);if(47===a||92===a)return 3;if(2===t.length)return 2}var o=t.indexOf(n);if(-1!==o){var s=o+n.length,c=t.indexOf(e.directorySeparator,s);if(-1!==c){var l=t.slice(0,o),u=t.slice(s,c);if("file"===l&&(""===u||"localhost"===u)&&_(t.charCodeAt(c+1))){var d=g(t,c+2);if(-1!==d){if(47===t.charCodeAt(d))return~(d+1);if(d===t.length)return~d}}return~(c+1)}return~t.length}return 0}function h(e){var n=y(e);return n<0?~n:n}function v(n){n=A(n);var t=h(n);return t===n.length?n:(n=G(n),n.slice(0,Math.max(t,n.lastIndexOf(e.directorySeparator))))}function b(n,t,r){n=A(n);var i=h(n);if(i===n.length)return"";n=G(n);var a=n.slice(Math.max(h(n),n.lastIndexOf(e.directorySeparator)+1)),o=void 0!==t&&void 0!==r?S(a,t,r):void 0;return o?a.slice(0,a.length-o.length):a}function E(n,t,r){if(e.startsWith(t,".")||(t="."+t),n.length>=t.length&&46===n.charCodeAt(n.length-t.length)){var i=n.slice(n.length-t.length);if(r(i,t))return i}}function T(e,n,t){if("string"===typeof n)return E(e,n,t)||"";for(var r=0,i=n;r<i.length;r++){var a=i[r],o=E(e,a,t);if(o)return o}return""}function S(n,t,r){if(t)return T(G(n),t,r?e.equateStringsCaseInsensitive:e.equateStringsCaseSensitive);var i=b(n),a=i.lastIndexOf(".");return a>=0?i.substring(a):""}function x(n,t){var r=n.substring(0,t),i=n.substring(t).split(e.directorySeparator);return i.length&&!e.lastOrUndefined(i)&&i.pop(),a([r],i,!0)}function L(e,n){return void 0===n&&(n=""),e=I(n,e),x(e,h(e))}function D(n){if(0===n.length)return"";var t=n[0]&&B(n[0]);return t+n.slice(1).join(e.directorySeparator)}function A(n){var r=n.indexOf("\\");return-1===r?n:(t.lastIndex=r,n.replace(t,e.directorySeparator))}function C(n){if(!e.some(n))return[];for(var t=[n[0]],r=1;r<n.length;r++){var i=n[r];if(i&&"."!==i){if(".."===i)if(t.length>1){if(".."!==t[t.length-1]){t.pop();continue}}else if(t[0])continue;t.push(i)}}return t}function I(e){for(var n=[],t=1;t<arguments.length;t++)n[t-1]=arguments[t];e&&(e=A(e));for(var r=0,i=n;r<i.length;r++){var a=i[r];a&&(a=A(a),e=e&&0===h(a)?B(e)+a:a)}return e}function N(n){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];return O(e.some(t)?I.apply(void 0,a([n],t,!1)):A(n))}function k(e,n){return C(L(e,n))}function R(e,n){return D(k(e,n))}function O(e){if(e=A(e),!j.test(e))return e;var n=e.replace(/\/\.\//g,"/").replace(/^\.\//,"");if(n!==e&&(e=n,!j.test(e)))return e;var t=D(C(L(e)));return t&&m(e)?B(t):t}function w(n){return 0===n.length?"":n.slice(1).join(e.directorySeparator)}function P(e,n){return w(k(e,n))}function M(e,n,t){var r=o(e)?O(e):R(e,n);return t(r)}function F(n){n=A(n);var t=C(L(n)),r=t[0],i=t.slice(1);if(i.length){var a=r+i.join(e.directorySeparator);return{path:m(n)?B(a):a,parts:i}}return{path:r,parts:i}}function G(e){return m(e)?e.substr(0,e.length-1):e}function B(n){return m(n)?n:n+e.directorySeparator}function U(e){return c(e)||l(e)?e:"./"+e}function V(n,t,r,i){var a=void 0!==r&&void 0!==i?S(n,r,i):S(n);return a?n.slice(0,n.length-a.length)+(e.startsWith(t,".")?t:"."+t):n}e.isAnyDirectorySeparator=r,e.isUrl=i,e.isRootedDiskPath=o,e.isDiskPathRoot=s,e.pathIsAbsolute=c,e.pathIsRelative=l,e.pathIsBareSpecifier=u,e.hasExtension=d,e.fileExtensionIs=p,e.fileExtensionIsOneOf=f,e.hasTrailingDirectorySeparator=m,e.getRootLength=h,e.getDirectoryPath=v,e.getBaseFileName=b,e.getAnyExtensionFromPath=S,e.getPathComponents=L,e.getPathFromPathComponents=D,e.normalizeSlashes=A,e.reducePathComponents=C,e.combinePaths=I,e.resolvePath=N,e.getNormalizedPathComponents=k,e.getNormalizedAbsolutePath=R,e.normalizePath=O,e.getNormalizedAbsolutePathWithoutRoot=P,e.toPath=M,e.normalizePathAndParts=F,e.removeTrailingDirectorySeparator=G,e.ensureTrailingDirectorySeparator=B,e.ensurePathIsNonModuleName=U,e.changeAnyExtension=V;var j=/(?:\/\/)|(?:^|\/)\.\.?(?:$|\/)/;function K(n,t,r){if(n===t)return 0;if(void 0===n)return-1;if(void 0===t)return 1;var i=n.substring(0,h(n)),a=t.substring(0,h(t)),o=e.compareStringsCaseInsensitive(i,a);if(0!==o)return o;var s=n.substring(i.length),c=t.substring(a.length);if(!j.test(s)&&!j.test(c))return r(s,c);for(var l=C(L(n)),u=C(L(t)),d=Math.min(l.length,u.length),p=1;p<d;p++){var f=r(l[p],u[p]);if(0!==f)return f}return e.compareValues(l.length,u.length)}function H(n,t){return K(n,t,e.compareStringsCaseSensitive)}function W(n,t){return K(n,t,e.compareStringsCaseInsensitive)}function z(n,t,r,i){return"string"===typeof r?(n=I(r,n),t=I(r,t)):"boolean"===typeof r&&(i=r),K(n,t,e.getStringComparer(i))}function J(n,t,r,i){if("string"===typeof r?(n=I(r,n),t=I(r,t)):"boolean"===typeof r&&(i=r),void 0===n||void 0===t)return!1;if(n===t)return!0;var a=C(L(n)),o=C(L(t));if(o.length<a.length)return!1;for(var s=i?e.equateStringsCaseInsensitive:e.equateStringsCaseSensitive,c=0;c<a.length;c++){var l=0===c?e.equateStringsCaseInsensitive:s;if(!l(a[c],o[c]))return!1}return!0}function q(n,t,r){var i=r(n),a=r(t);return e.startsWith(i,a+"/")||e.startsWith(i,a+"\\")}function X(n,t,r,i){var o,s=C(L(n)),c=C(L(t));for(o=0;o<s.length&&o<c.length;o++){var l=i(s[o]),u=i(c[o]),d=0===o?e.equateStringsCaseInsensitive:r;if(!d(l,u))break}if(0===o)return c;for(var p=c.slice(o),f=[];o<s.length;o++)f.push("..");return a(a([""],f,!0),p,!0)}function Y(n,t,r){e.Debug.assert(h(n)>0===h(t)>0,"Paths must either both be absolute or both be relative");var i="function"===typeof r?r:e.identity,a="boolean"===typeof r&&r,o=X(n,t,a?e.equateStringsCaseInsensitive:e.equateStringsCaseSensitive,i);return D(o)}function Q(e,n,t){return o(e)?$(n,e,n,t,!1):e}function Z(e,n,t){return U(Y(v(e),n,t))}function $(n,t,r,i,a){var s=X(N(r,n),N(r,t),e.equateStringsCaseSensitive,i),c=s[0];if(a&&o(c)){var l=c.charAt(0)===e.directorySeparator?"file://":"file:///";s[0]=l+c}return D(s)}function ee(e,n){while(1){var t=n(e);if(void 0!==t)return t;var r=v(e);if(r===e)return;e=r}}function ne(n){return e.endsWith(n,"/node_modules")}e.comparePathsCaseSensitive=H,e.comparePathsCaseInsensitive=W,e.comparePaths=z,e.containsPath=J,e.startsWithDirectory=q,e.getPathComponentsRelativeTo=X,e.getRelativePathFromDirectory=Y,e.convertToRelativePath=Q,e.getRelativePathFromFile=Z,e.getRelativePathToDirectoryOrUrl=$,e.forEachAncestorDirectory=ee,e.isNodeModulesDirectory=ne}(d||(d={})),function(n){function t(e){for(var n=5381,t=0;t<e.length;t++)n=(n<<5)+n+e.charCodeAt(t);return n.toString()}function r(){Error.stackTraceLimit<100&&(Error.stackTraceLimit=100)}var i,a;function s(e,t){return e.getModifiedTime(t)||n.missingFileModifiedTime}function c(e){var n;return n={},n[a.Low]=e.Low,n[a.Medium]=e.Medium,n[a.High]=e.High,n}n.generateDjb2Hash=t,n.setStackTraceLimit=r,function(e){e[e["Created"]=0]="Created",e[e["Changed"]=1]="Changed",e[e["Deleted"]=2]="Deleted"}(i=n.FileWatcherEventKind||(n.FileWatcherEventKind={})),function(e){e[e["High"]=2e3]="High",e[e["Medium"]=500]="Medium",e[e["Low"]=250]="Low"}(a=n.PollingInterval||(n.PollingInterval={})),n.missingFileModifiedTime=new Date(0),n.getModifiedTime=s;var l={Low:32,Medium:64,High:256},u=c(l);function d(e){if(e.getEnvironmentVariable){var t=s("TSC_WATCH_POLLINGINTERVAL",a);u=d("TSC_WATCH_POLLINGCHUNKSIZE",l)||u,n.unchangedPollThresholds=d("TSC_WATCH_UNCHANGEDPOLLTHRESHOLDS",l)||n.unchangedPollThresholds}function r(n,t){return e.getEnvironmentVariable(n+"_"+t.toUpperCase())}function i(e){var n;return t("Low"),t("Medium"),t("High"),n;function t(t){var i=r(e,t);i&&((n||(n={}))[t]=Number(i))}}function s(e,n){var t=i(e);return!!t&&(r("Low"),r("Medium"),r("High"),!0);function r(e){n[e]=t[e]||n[e]}}function d(e,n){var r=i(e);return(t||r)&&c(r?o(o({},n),r):n)}}function p(e,n,t,r,i){for(var a=t,o=n.length;r&&o;u(),o--){var c=n[t];if(c)if(c.isClosed)n[t]=void 0;else{r--;var l=y(c,s(e,c.fileName));c.isClosed?n[t]=void 0:(null===i||void 0===i||i(c,t,l),n[t]&&(a<t&&(n[a]=c,n[t]=void 0),a++))}}return t;function u(){t++,t===n.length&&(a<t&&(n.length=a),t=0,a=0)}}function f(e){var t=[],r=[],i=d(a.Low),o=d(a.Medium),c=d(a.High);return l;function l(r,i,a){var o={fileName:r,callback:i,unchangedPolls:0,mtime:s(e,r)};return t.push(o),y(o,a),{close:function(){o.isClosed=!0,n.unorderedRemoveItem(t,o)}}}function d(e){var n=[];return n.pollingInterval=e,n.pollIndex=0,n.pollScheduled=!1,n}function f(e){e.pollIndex=_(e,e.pollingInterval,e.pollIndex,u[e.pollingInterval]),e.length?b(e.pollingInterval):(n.Debug.assert(0===e.pollIndex),e.pollScheduled=!1)}function m(e){_(r,a.Low,0,r.length),f(e),!e.pollScheduled&&r.length&&b(a.Low)}function _(t,i,o,s){return p(e,t,o,s,c);function c(e,o,s){s?(e.unchangedPolls=0,t!==r&&(t[o]=void 0,h(e))):e.unchangedPolls!==n.unchangedPollThresholds[i]?e.unchangedPolls++:t===r?(e.unchangedPolls=1,t[o]=void 0,y(e,a.Low)):i!==a.High&&(e.unchangedPolls++,t[o]=void 0,y(e,i===a.Low?a.Medium:a.High))}}function g(e){switch(e){case a.Low:return i;case a.Medium:return o;case a.High:return c}}function y(e,n){g(n).push(e),v(n)}function h(e){r.push(e),v(a.Low)}function v(e){g(e).pollScheduled||b(e)}function b(n){g(n).pollScheduled=e.setTimeout(n===a.Low?m:f,n,g(n))}}function m(e,t){var r=n.createMultiMap(),o=new n.Map,s=n.createGetCanonicalFileName(t);return c;function c(e,t,i,a){var c=s(e);r.add(c,t);var u=n.getDirectoryPath(c)||".",d=o.get(u)||l(n.getDirectoryPath(e)||".",u,a);return d.referenceCount++,{close:function(){1===d.referenceCount?(d.close(),o.delete(u)):d.referenceCount--,r.remove(c,t)}}}function l(t,c,l){var u=e(t,1,function(e,a){if(n.isString(a)){var o=n.getNormalizedAbsolutePath(a,t),c=o&&r.get(s(o));if(c)for(var l=0,u=c;l<u.length;l++){var d=u[l];d(o,i.Changed)}}},!1,a.Medium,l);return u.referenceCount=0,o.set(c,u),u}}function _(e){var t,r=[],i=0;return o;function o(t,i){var a={fileName:t,callback:i,mtime:s(e,t)};return r.push(a),l(),{close:function(){a.isClosed=!0,n.unorderedRemoveItem(r,a)}}}function c(){t=void 0,i=p(e,r,i,u[a.Low]),l()}function l(){r.length&&!t&&(t=e.setTimeout(c,a.High))}}function g(e,t){var r=new n.Map,i=n.createMultiMap(),a=n.createGetCanonicalFileName(t);return function(t,o,s,c){var l=a(t),u=r.get(l);return u?u.refCount++:r.set(l,{watcher:e(t,function(e,t){return n.forEach(i.get(l),function(n){return n(e,t)})},s,c),refCount:1}),i.add(l,o),{close:function(){var e=n.Debug.checkDefined(r.get(l));i.remove(l,o),e.refCount--,e.refCount||(r.delete(l),n.closeFileWatcherOf(e))}}}}function y(e,n){var t=e.mtime.getTime(),r=n.getTime();return t!==r&&(e.mtime=n,e.callback(e.fileName,h(t,r)),!0)}function h(e,n){return 0===e?i.Created:0===n?i.Deleted:i.Changed}function v(e){n.sysLog=e}function b(e){var t,r=e.watchDirectory,i=e.useCaseSensitiveFileNames,a=e.getCurrentDirectory,o=e.getAccessibleSortedChildDirectories,s=e.directoryExists,c=e.realpath,l=e.setTimeout,u=e.clearTimeout,d=new n.Map,p=n.createMultiMap(),f=new n.Map,m=n.getStringComparer(!i),_=n.createGetCanonicalFileName(i);return function(e,n,t,i){return t?g(e,i,n):r(e,n,t,i)};function g(e,t,i){var a=_(e),o=d.get(a);o?o.refCount++:(o={watcher:r(e,function(n){x(n,t)||((null===t||void 0===t?void 0:t.synchronousWatchDirectory)?(y(a,n),T(e,a,t)):h(e,a,n,t))},!1,t),refCount:1,childWatches:n.emptyArray},d.set(a,o),T(e,a,t));var s=i&&{dirName:e,callback:i};return s&&p.add(a,s),{dirName:e,close:function(){var e=n.Debug.checkDefined(d.get(a));s&&p.remove(a,s),e.refCount--,e.refCount||(d.delete(a),n.closeFileWatcherOf(e),e.childWatches.forEach(n.closeFileWatcher))}}}function y(e,t,r){var i,a;n.isString(t)?i=t:a=t,p.forEach(function(t,o){var s;if((!a||!0!==a.get(o))&&(o===e||n.startsWith(e,o)&&e[o.length]===n.directorySeparator))if(a)if(r){var c=a.get(o);c?(s=c).push.apply(s,r):a.set(o,r.slice())}else a.set(o,!0);else t.forEach(function(e){var n=e.callback;return n(i)})})}function h(e,n,t,r){var i=d.get(n);i&&s(e)?v(e,n,t,r):(y(n,t),E(i))}function v(e,n,r,i){var a=f.get(n);a?a.fileNames.push(r):f.set(n,{dirName:e,options:i,fileNames:[r]}),t&&(u(t),t=void 0),t=l(b,1e3)}function b(){t=void 0,n.sysLog("sysLog:: onTimerToUpdateChildWatches:: "+f.size);var e=n.timestamp(),r=new n.Map;while(!t&&f.size){var i=f.entries().next();n.Debug.assert(!i.done);var a=i.value,o=a[0],s=a[1],c=s.dirName,l=s.options,u=s.fileNames;f.delete(o);var d=T(c,o,l);y(o,r,d?void 0:u)}n.sysLog("sysLog:: invokingWatchers:: Elapsed:: "+(n.timestamp()-e)+"ms:: "+f.size),p.forEach(function(e,t){var i=r.get(t);i&&e.forEach(function(e){var t=e.callback,r=e.dirName;n.isArray(i)?i.forEach(t):t(r)})});var m=n.timestamp()-e;n.sysLog("sysLog:: Elapsed:: "+m+"ms:: onTimerToUpdateChildWatches:: "+f.size+" "+t)}function E(e){if(e){var t=e.childWatches;e.childWatches=n.emptyArray;for(var r=0,i=t;r<i.length;r++){var a=i[r];a.close(),E(d.get(_(a.dirName)))}}}function T(e,t,r){var i,a=d.get(t);if(!a)return!1;var l=n.enumerateInsertsAndDeletes(s(e)?n.mapDefined(o(e),function(t){var i=n.getNormalizedAbsolutePath(t,e);return x(i,r)||0!==m(i,n.normalizePath(c(i)))?void 0:i}):n.emptyArray,a.childWatches,function(e,n){return m(e,n.dirName)},u,n.closeFileWatcher,p);return a.childWatches=i||n.emptyArray,l;function u(e){var n=g(e,r);p(n)}function p(e){(i||(i=[])).push(e)}}function x(e,t){return n.some(n.ignoredPaths,function(n){return L(e,n)})||S(e,t,i,a)}function L(e,t){return!!n.stringContains(e,t)||!i&&n.stringContains(_(e),t)}}function E(e){return function(n,t){return e(t===i.Changed?"change":"rename","")}}function T(e,n,t){return function(r){n(e,"rename"===r?t(e)?i.Created:i.Deleted:i.Changed)}}function S(e,t,r,i){return((null===t||void 0===t?void 0:t.excludeDirectories)||(null===t||void 0===t?void 0:t.excludeFiles))&&(n.matchesExclude(e,null===t||void 0===t?void 0:t.excludeFiles,r,i())||n.matchesExclude(e,null===t||void 0===t?void 0:t.excludeDirectories,r,i()))}function x(e,t,r,i,a){return function(o,s){if("rename"===o){var c=s?n.normalizePath(n.combinePaths(e,s)):e;s&&S(c,r,i,a)||t(c)}}}function L(e){var t,r,i,o,s=e.pollingWatchFile,c=e.getModifiedTime,l=e.setTimeout,u=e.clearTimeout,d=e.fsWatch,p=e.fileExists,g=e.useCaseSensitiveFileNames,y=e.getCurrentDirectory,h=e.fsSupportsRecursiveFsWatch,v=e.directoryExists,E=e.getAccessibleSortedChildDirectories,S=e.realpath,L=e.tscWatchFile,D=e.useNonPollingWatchers,A=e.tscWatchDirectory,C=e.defaultWatchFileKind;return{watchFile:I,watchDirectory:w};function I(e,t,r,o){o=R(o,D);var c=n.Debug.checkDefined(o.watchFile);switch(c){case n.WatchFileKind.FixedPollingInterval:return s(e,t,a.Low,void 0);case n.WatchFileKind.PriorityPollingInterval:return s(e,t,r,void 0);case n.WatchFileKind.DynamicPriorityPolling:return N()(e,t,r,void 0);case n.WatchFileKind.FixedChunkSizePolling:return k()(e,t,void 0,void 0);case n.WatchFileKind.UseFsEvents:return d(e,0,T(e,t,p),!1,r,n.getFallbackOptions(o));case n.WatchFileKind.UseFsEventsOnParentDirectory:return i||(i=m(d,g)),i(e,t,r,n.getFallbackOptions(o));default:n.Debug.assertNever(c)}}function N(){return t||(t=f({getModifiedTime:c,setTimeout:l}))}function k(){return r||(r=_({getModifiedTime:c,setTimeout:l}))}function R(e,t){if(e&&void 0!==e.watchFile)return e;switch(L){case"PriorityPollingInterval":return{watchFile:n.WatchFileKind.PriorityPollingInterval};case"DynamicPriorityPolling":return{watchFile:n.WatchFileKind.DynamicPriorityPolling};case"UseFsEvents":return O(n.WatchFileKind.UseFsEvents,n.PollingWatchKind.PriorityInterval,e);case"UseFsEventsWithFallbackDynamicPolling":return O(n.WatchFileKind.UseFsEvents,n.PollingWatchKind.DynamicPriority,e);case"UseFsEventsOnParentDirectory":t=!0;default:return t?O(n.WatchFileKind.UseFsEventsOnParentDirectory,n.PollingWatchKind.PriorityInterval,e):{watchFile:(null===C||void 0===C?void 0:C())||n.WatchFileKind.FixedPollingInterval}}}function O(e,n,t){var r=null===t||void 0===t?void 0:t.fallbackPolling;return{watchFile:e,fallbackPolling:void 0===r?n:r}}function w(e,t,r,i){return h?d(e,1,x(e,t,i,g,y),r,a.Medium,n.getFallbackOptions(i)):(o||(o=b({useCaseSensitiveFileNames:g,getCurrentDirectory:y,directoryExists:v,getAccessibleSortedChildDirectories:E,watchDirectory:P,realpath:S,setTimeout:l,clearTimeout:u})),o(e,t,r,i))}function P(e,t,r,i){n.Debug.assert(!r);var o=M(i),c=n.Debug.checkDefined(o.watchDirectory);switch(c){case n.WatchDirectoryKind.FixedPollingInterval:return s(e,function(){return t(e)},a.Medium,void 0);case n.WatchDirectoryKind.DynamicPriorityPolling:return N()(e,function(){return t(e)},a.Medium,void 0);case n.WatchDirectoryKind.FixedChunkSizePolling:return k()(e,function(){return t(e)},void 0,void 0);case n.WatchDirectoryKind.UseFsEvents:return d(e,1,x(e,t,i,g,y),r,a.Medium,n.getFallbackOptions(o));default:n.Debug.assertNever(c)}}function M(e){if(e&&void 0!==e.watchDirectory)return e;switch(A){case"RecursiveDirectoryUsingFsWatchFile":return{watchDirectory:n.WatchDirectoryKind.FixedPollingInterval};case"RecursiveDirectoryUsingDynamicPriorityPolling":return{watchDirectory:n.WatchDirectoryKind.DynamicPriorityPolling};default:var t=null===e||void 0===e?void 0:e.fallbackPolling;return{watchDirectory:n.WatchDirectoryKind.UseFsEvents,fallbackPolling:void 0!==t?t:void 0}}}}function D(e){var t=e.writeFile;e.writeFile=function(r,i,a){return n.writeFileEnsuringDirectories(r,i,!!a,function(n,r,i){return t.call(e,n,r,i)},function(n){return e.createDirectory(n)},function(n){return e.directoryExists(n)})}}function A(){if("undefined"!==typeof e){var n=e.version;if(n){var t=n.indexOf(".");if(-1!==t)return parseInt(n.substring(1,t))}}}n.unchangedPollThresholds=c(l),n.setCustomPollingValues=d,n.createDynamicPriorityPollingWatchFile=f,n.createSingleFileWatcherPerName=g,n.onWatchedFileStat=y,n.getFileWatcherEventKind=h,n.ignoredPaths=["/node_modules/.","/.git","/.#"],n.sysLog=n.noop,n.setSysLog=v,n.createDirectoryWatcherSupportingRecursive=b,function(e){e[e["File"]=0]="File",e[e["Directory"]=1]="Directory"}(n.FileSystemEntryKind||(n.FileSystemEntryKind={})),n.createFileWatcherCallback=E,n.createSystemWatchFunctions=L,n.patchWriteFileEnsuringDirectory=D,n.getNodeMajorVersion=A,n.sys=void 0,n.sys&&n.sys.getEnvironmentVariable&&(d(n.sys),n.Debug.setAssertionLevel(/^development$/i.test(n.sys.getEnvironmentVariable("NODE_ENV"))?1:0)),n.sys&&n.sys.debugMode&&(n.Debug.isDebugging=!0)}(d||(d={})),function(e){function n(e,n,t,r,i,a,o){return{code:e,category:n,key:t,message:r,reportsUnnecessary:i,elidedInCompatabilityPyramid:a,reportsDeprecated:o}}e.Diagnostics={Unterminated_string_literal:n(1002,e.DiagnosticCategory.Error,"Unterminated_string_literal_1002","Unterminated string literal."),Identifier_expected:n(1003,e.DiagnosticCategory.Error,"Identifier_expected_1003","Identifier expected."),_0_expected:n(1005,e.DiagnosticCategory.Error,"_0_expected_1005","'{0}' expected."),A_file_cannot_have_a_reference_to_itself:n(1006,e.DiagnosticCategory.Error,"A_file_cannot_have_a_reference_to_itself_1006","A file cannot have a reference to itself."),The_parser_expected_to_find_a_to_match_the_token_here:n(1007,e.DiagnosticCategory.Error,"The_parser_expected_to_find_a_to_match_the_token_here_1007","The parser expected to find a '}' to match the '{' token here."),Trailing_comma_not_allowed:n(1009,e.DiagnosticCategory.Error,"Trailing_comma_not_allowed_1009","Trailing comma not allowed."),Asterisk_Slash_expected:n(1010,e.DiagnosticCategory.Error,"Asterisk_Slash_expected_1010","'*/' expected."),An_element_access_expression_should_take_an_argument:n(1011,e.DiagnosticCategory.Error,"An_element_access_expression_should_take_an_argument_1011","An element access expression should take an argument."),Unexpected_token:n(1012,e.DiagnosticCategory.Error,"Unexpected_token_1012","Unexpected token."),A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma:n(1013,e.DiagnosticCategory.Error,"A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma_1013","A rest parameter or binding pattern may not have a trailing comma."),A_rest_parameter_must_be_last_in_a_parameter_list:n(1014,e.DiagnosticCategory.Error,"A_rest_parameter_must_be_last_in_a_parameter_list_1014","A rest parameter must be last in a parameter list."),Parameter_cannot_have_question_mark_and_initializer:n(1015,e.DiagnosticCategory.Error,"Parameter_cannot_have_question_mark_and_initializer_1015","Parameter cannot have question mark and initializer."),A_required_parameter_cannot_follow_an_optional_parameter:n(1016,e.DiagnosticCategory.Error,"A_required_parameter_cannot_follow_an_optional_parameter_1016","A required parameter cannot follow an optional parameter."),An_index_signature_cannot_have_a_rest_parameter:n(1017,e.DiagnosticCategory.Error,"An_index_signature_cannot_have_a_rest_parameter_1017","An index signature cannot have a rest parameter."),An_index_signature_parameter_cannot_have_an_accessibility_modifier:n(1018,e.DiagnosticCategory.Error,"An_index_signature_parameter_cannot_have_an_accessibility_modifier_1018","An index signature parameter cannot have an accessibility modifier."),An_index_signature_parameter_cannot_have_a_question_mark:n(1019,e.DiagnosticCategory.Error,"An_index_signature_parameter_cannot_have_a_question_mark_1019","An index signature parameter cannot have a question mark."),An_index_signature_parameter_cannot_have_an_initializer:n(1020,e.DiagnosticCategory.Error,"An_index_signature_parameter_cannot_have_an_initializer_1020","An index signature parameter cannot have an initializer."),An_index_signature_must_have_a_type_annotation:n(1021,e.DiagnosticCategory.Error,"An_index_signature_must_have_a_type_annotation_1021","An index signature must have a type annotation."),An_index_signature_parameter_must_have_a_type_annotation:n(1022,e.DiagnosticCategory.Error,"An_index_signature_parameter_must_have_a_type_annotation_1022","An index signature parameter must have a type annotation."),readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature:n(1024,e.DiagnosticCategory.Error,"readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature_1024","'readonly' modifier can only appear on a property declaration or index signature."),An_index_signature_cannot_have_a_trailing_comma:n(1025,e.DiagnosticCategory.Error,"An_index_signature_cannot_have_a_trailing_comma_1025","An index signature cannot have a trailing comma."),Accessibility_modifier_already_seen:n(1028,e.DiagnosticCategory.Error,"Accessibility_modifier_already_seen_1028","Accessibility modifier already seen."),_0_modifier_must_precede_1_modifier:n(1029,e.DiagnosticCategory.Error,"_0_modifier_must_precede_1_modifier_1029","'{0}' modifier must precede '{1}' modifier."),_0_modifier_already_seen:n(1030,e.DiagnosticCategory.Error,"_0_modifier_already_seen_1030","'{0}' modifier already seen."),_0_modifier_cannot_appear_on_class_elements_of_this_kind:n(1031,e.DiagnosticCategory.Error,"_0_modifier_cannot_appear_on_class_elements_of_this_kind_1031","'{0}' modifier cannot appear on class elements of this kind."),super_must_be_followed_by_an_argument_list_or_member_access:n(1034,e.DiagnosticCategory.Error,"super_must_be_followed_by_an_argument_list_or_member_access_1034","'super' must be followed by an argument list or member access."),Only_ambient_modules_can_use_quoted_names:n(1035,e.DiagnosticCategory.Error,"Only_ambient_modules_can_use_quoted_names_1035","Only ambient modules can use quoted names."),Statements_are_not_allowed_in_ambient_contexts:n(1036,e.DiagnosticCategory.Error,"Statements_are_not_allowed_in_ambient_contexts_1036","Statements are not allowed in ambient contexts."),A_declare_modifier_cannot_be_used_in_an_already_ambient_context:n(1038,e.DiagnosticCategory.Error,"A_declare_modifier_cannot_be_used_in_an_already_ambient_context_1038","A 'declare' modifier cannot be used in an already ambient context."),Initializers_are_not_allowed_in_ambient_contexts:n(1039,e.DiagnosticCategory.Error,"Initializers_are_not_allowed_in_ambient_contexts_1039","Initializers are not allowed in ambient contexts."),_0_modifier_cannot_be_used_in_an_ambient_context:n(1040,e.DiagnosticCategory.Error,"_0_modifier_cannot_be_used_in_an_ambient_context_1040","'{0}' modifier cannot be used in an ambient context."),_0_modifier_cannot_be_used_with_a_class_declaration:n(1041,e.DiagnosticCategory.Error,"_0_modifier_cannot_be_used_with_a_class_declaration_1041","'{0}' modifier cannot be used with a class declaration."),_0_modifier_cannot_be_used_here:n(1042,e.DiagnosticCategory.Error,"_0_modifier_cannot_be_used_here_1042","'{0}' modifier cannot be used here."),_0_modifier_cannot_appear_on_a_data_property:n(1043,e.DiagnosticCategory.Error,"_0_modifier_cannot_appear_on_a_data_property_1043","'{0}' modifier cannot appear on a data property."),_0_modifier_cannot_appear_on_a_module_or_namespace_element:n(1044,e.DiagnosticCategory.Error,"_0_modifier_cannot_appear_on_a_module_or_namespace_element_1044","'{0}' modifier cannot appear on a module or namespace element."),A_0_modifier_cannot_be_used_with_an_interface_declaration:n(1045,e.DiagnosticCategory.Error,"A_0_modifier_cannot_be_used_with_an_interface_declaration_1045","A '{0}' modifier cannot be used with an interface declaration."),Top_level_declarations_in_d_ts_files_must_start_with_either_a_declare_or_export_modifier:n(1046,e.DiagnosticCategory.Error,"Top_level_declarations_in_d_ts_files_must_start_with_either_a_declare_or_export_modifier_1046","Top-level declarations in .d.ts files must start with either a 'declare' or 'export' modifier."),A_rest_parameter_cannot_be_optional:n(1047,e.DiagnosticCategory.Error,"A_rest_parameter_cannot_be_optional_1047","A rest parameter cannot be optional."),A_rest_parameter_cannot_have_an_initializer:n(1048,e.DiagnosticCategory.Error,"A_rest_parameter_cannot_have_an_initializer_1048","A rest parameter cannot have an initializer."),A_set_accessor_must_have_exactly_one_parameter:n(1049,e.DiagnosticCategory.Error,"A_set_accessor_must_have_exactly_one_parameter_1049","A 'set' accessor must have exactly one parameter."),A_set_accessor_cannot_have_an_optional_parameter:n(1051,e.DiagnosticCategory.Error,"A_set_accessor_cannot_have_an_optional_parameter_1051","A 'set' accessor cannot have an optional parameter."),A_set_accessor_parameter_cannot_have_an_initializer:n(1052,e.DiagnosticCategory.Error,"A_set_accessor_parameter_cannot_have_an_initializer_1052","A 'set' accessor parameter cannot have an initializer."),A_set_accessor_cannot_have_rest_parameter:n(1053,e.DiagnosticCategory.Error,"A_set_accessor_cannot_have_rest_parameter_1053","A 'set' accessor cannot have rest parameter."),A_get_accessor_cannot_have_parameters:n(1054,e.DiagnosticCategory.Error,"A_get_accessor_cannot_have_parameters_1054","A 'get' accessor cannot have parameters."),Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value:n(1055,e.DiagnosticCategory.Error,"Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Prom_1055","Type '{0}' is not a valid async function return type in ES5/ES3 because it does not refer to a Promise-compatible constructor value."),Accessors_are_only_available_when_targeting_ECMAScript_5_and_higher:n(1056,e.DiagnosticCategory.Error,"Accessors_are_only_available_when_targeting_ECMAScript_5_and_higher_1056","Accessors are only available when targeting ECMAScript 5 and higher."),An_async_function_or_method_must_have_a_valid_awaitable_return_type:n(1057,e.DiagnosticCategory.Error,"An_async_function_or_method_must_have_a_valid_awaitable_return_type_1057","An async function or method must have a valid awaitable return type."),The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:n(1058,e.DiagnosticCategory.Error,"The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_t_1058","The return type of an async function must either be a valid promise or must not contain a callable 'then' member."),A_promise_must_have_a_then_method:n(1059,e.DiagnosticCategory.Error,"A_promise_must_have_a_then_method_1059","A promise must have a 'then' method."),The_first_parameter_of_the_then_method_of_a_promise_must_be_a_callback:n(1060,e.DiagnosticCategory.Error,"The_first_parameter_of_the_then_method_of_a_promise_must_be_a_callback_1060","The first parameter of the 'then' method of a promise must be a callback."),Enum_member_must_have_initializer:n(1061,e.DiagnosticCategory.Error,"Enum_member_must_have_initializer_1061","Enum member must have initializer."),Type_is_referenced_directly_or_indirectly_in_the_fulfillment_callback_of_its_own_then_method:n(1062,e.DiagnosticCategory.Error,"Type_is_referenced_directly_or_indirectly_in_the_fulfillment_callback_of_its_own_then_method_1062","Type is referenced directly or indirectly in the fulfillment callback of its own 'then' method."),An_export_assignment_cannot_be_used_in_a_namespace:n(1063,e.DiagnosticCategory.Error,"An_export_assignment_cannot_be_used_in_a_namespace_1063","An export assignment cannot be used in a namespace."),The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type_Did_you_mean_to_write_Promise_0:n(1064,e.DiagnosticCategory.Error,"The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type_Did_you_mean_to_wri_1064","The return type of an async function or method must be the global Promise<T> type. Did you mean to write 'Promise<{0}>'?"),In_ambient_enum_declarations_member_initializer_must_be_constant_expression:n(1066,e.DiagnosticCategory.Error,"In_ambient_enum_declarations_member_initializer_must_be_constant_expression_1066","In ambient enum declarations member initializer must be constant expression."),Unexpected_token_A_constructor_method_accessor_or_property_was_expected:n(1068,e.DiagnosticCategory.Error,"Unexpected_token_A_constructor_method_accessor_or_property_was_expected_1068","Unexpected token. A constructor, method, accessor, or property was expected."),Unexpected_token_A_type_parameter_name_was_expected_without_curly_braces:n(1069,e.DiagnosticCategory.Error,"Unexpected_token_A_type_parameter_name_was_expected_without_curly_braces_1069","Unexpected token. A type parameter name was expected without curly braces."),_0_modifier_cannot_appear_on_a_type_member:n(1070,e.DiagnosticCategory.Error,"_0_modifier_cannot_appear_on_a_type_member_1070","'{0}' modifier cannot appear on a type member."),_0_modifier_cannot_appear_on_an_index_signature:n(1071,e.DiagnosticCategory.Error,"_0_modifier_cannot_appear_on_an_index_signature_1071","'{0}' modifier cannot appear on an index signature."),A_0_modifier_cannot_be_used_with_an_import_declaration:n(1079,e.DiagnosticCategory.Error,"A_0_modifier_cannot_be_used_with_an_import_declaration_1079","A '{0}' modifier cannot be used with an import declaration."),Invalid_reference_directive_syntax:n(1084,e.DiagnosticCategory.Error,"Invalid_reference_directive_syntax_1084","Invalid 'reference' directive syntax."),Octal_literals_are_not_available_when_targeting_ECMAScript_5_and_higher_Use_the_syntax_0:n(1085,e.DiagnosticCategory.Error,"Octal_literals_are_not_available_when_targeting_ECMAScript_5_and_higher_Use_the_syntax_0_1085","Octal literals are not available when targeting ECMAScript 5 and higher. Use the syntax '{0}'."),_0_modifier_cannot_appear_on_a_constructor_declaration:n(1089,e.DiagnosticCategory.Error,"_0_modifier_cannot_appear_on_a_constructor_declaration_1089","'{0}' modifier cannot appear on a constructor declaration."),_0_modifier_cannot_appear_on_a_parameter:n(1090,e.DiagnosticCategory.Error,"_0_modifier_cannot_appear_on_a_parameter_1090","'{0}' modifier cannot appear on a parameter."),Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement:n(1091,e.DiagnosticCategory.Error,"Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement_1091","Only a single variable declaration is allowed in a 'for...in' statement."),Type_parameters_cannot_appear_on_a_constructor_declaration:n(1092,e.DiagnosticCategory.Error,"Type_parameters_cannot_appear_on_a_constructor_declaration_1092","Type parameters cannot appear on a constructor declaration."),Type_annotation_cannot_appear_on_a_constructor_declaration:n(1093,e.DiagnosticCategory.Error,"Type_annotation_cannot_appear_on_a_constructor_declaration_1093","Type annotation cannot appear on a constructor declaration."),An_accessor_cannot_have_type_parameters:n(1094,e.DiagnosticCategory.Error,"An_accessor_cannot_have_type_parameters_1094","An accessor cannot have type parameters."),A_set_accessor_cannot_have_a_return_type_annotation:n(1095,e.DiagnosticCategory.Error,"A_set_accessor_cannot_have_a_return_type_annotation_1095","A 'set' accessor cannot have a return type annotation."),An_index_signature_must_have_exactly_one_parameter:n(1096,e.DiagnosticCategory.Error,"An_index_signature_must_have_exactly_one_parameter_1096","An index signature must have exactly one parameter."),_0_list_cannot_be_empty:n(1097,e.DiagnosticCategory.Error,"_0_list_cannot_be_empty_1097","'{0}' list cannot be empty."),Type_parameter_list_cannot_be_empty:n(1098,e.DiagnosticCategory.Error,"Type_parameter_list_cannot_be_empty_1098","Type parameter list cannot be empty."),Type_argument_list_cannot_be_empty:n(1099,e.DiagnosticCategory.Error,"Type_argument_list_cannot_be_empty_1099","Type argument list cannot be empty."),Invalid_use_of_0_in_strict_mode:n(1100,e.DiagnosticCategory.Error,"Invalid_use_of_0_in_strict_mode_1100","Invalid use of '{0}' in strict mode."),with_statements_are_not_allowed_in_strict_mode:n(1101,e.DiagnosticCategory.Error,"with_statements_are_not_allowed_in_strict_mode_1101","'with' statements are not allowed in strict mode."),delete_cannot_be_called_on_an_identifier_in_strict_mode:n(1102,e.DiagnosticCategory.Error,"delete_cannot_be_called_on_an_identifier_in_strict_mode_1102","'delete' cannot be called on an identifier in strict mode."),for_await_loops_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules:n(1103,e.DiagnosticCategory.Error,"for_await_loops_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules_1103","'for await' loops are only allowed within async functions and at the top levels of modules."),A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement:n(1104,e.DiagnosticCategory.Error,"A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement_1104","A 'continue' statement can only be used within an enclosing iteration statement."),A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement:n(1105,e.DiagnosticCategory.Error,"A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement_1105","A 'break' statement can only be used within an enclosing iteration or switch statement."),The_left_hand_side_of_a_for_of_statement_may_not_be_async:n(1106,e.DiagnosticCategory.Error,"The_left_hand_side_of_a_for_of_statement_may_not_be_async_1106","The left-hand side of a 'for...of' statement may not be 'async'."),Jump_target_cannot_cross_function_boundary:n(1107,e.DiagnosticCategory.Error,"Jump_target_cannot_cross_function_boundary_1107","Jump target cannot cross function boundary."),A_return_statement_can_only_be_used_within_a_function_body:n(1108,e.DiagnosticCategory.Error,"A_return_statement_can_only_be_used_within_a_function_body_1108","A 'return' statement can only be used within a function body."),Expression_expected:n(1109,e.DiagnosticCategory.Error,"Expression_expected_1109","Expression expected."),Type_expected:n(1110,e.DiagnosticCategory.Error,"Type_expected_1110","Type expected."),A_default_clause_cannot_appear_more_than_once_in_a_switch_statement:n(1113,e.DiagnosticCategory.Error,"A_default_clause_cannot_appear_more_than_once_in_a_switch_statement_1113","A 'default' clause cannot appear more than once in a 'switch' statement."),Duplicate_label_0:n(1114,e.DiagnosticCategory.Error,"Duplicate_label_0_1114","Duplicate label '{0}'."),A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement:n(1115,e.DiagnosticCategory.Error,"A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement_1115","A 'continue' statement can only jump to a label of an enclosing iteration statement."),A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement:n(1116,e.DiagnosticCategory.Error,"A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement_1116","A 'break' statement can only jump to a label of an enclosing statement."),An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode:n(1117,e.DiagnosticCategory.Error,"An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode_1117","An object literal cannot have multiple properties with the same name in strict mode."),An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name:n(1118,e.DiagnosticCategory.Error,"An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name_1118","An object literal cannot have multiple get/set accessors with the same name."),An_object_literal_cannot_have_property_and_accessor_with_the_same_name:n(1119,e.DiagnosticCategory.Error,"An_object_literal_cannot_have_property_and_accessor_with_the_same_name_1119","An object literal cannot have property and accessor with the same name."),An_export_assignment_cannot_have_modifiers:n(1120,e.DiagnosticCategory.Error,"An_export_assignment_cannot_have_modifiers_1120","An export assignment cannot have modifiers."),Octal_literals_are_not_allowed_in_strict_mode:n(1121,e.DiagnosticCategory.Error,"Octal_literals_are_not_allowed_in_strict_mode_1121","Octal literals are not allowed in strict mode."),Variable_declaration_list_cannot_be_empty:n(1123,e.DiagnosticCategory.Error,"Variable_declaration_list_cannot_be_empty_1123","Variable declaration list cannot be empty."),Digit_expected:n(1124,e.DiagnosticCategory.Error,"Digit_expected_1124","Digit expected."),Hexadecimal_digit_expected:n(1125,e.DiagnosticCategory.Error,"Hexadecimal_digit_expected_1125","Hexadecimal digit expected."),Unexpected_end_of_text:n(1126,e.DiagnosticCategory.Error,"Unexpected_end_of_text_1126","Unexpected end of text."),Invalid_character:n(1127,e.DiagnosticCategory.Error,"Invalid_character_1127","Invalid character."),Declaration_or_statement_expected:n(1128,e.DiagnosticCategory.Error,"Declaration_or_statement_expected_1128","Declaration or statement expected."),Statement_expected:n(1129,e.DiagnosticCategory.Error,"Statement_expected_1129","Statement expected."),case_or_default_expected:n(1130,e.DiagnosticCategory.Error,"case_or_default_expected_1130","'case' or 'default' expected."),Property_or_signature_expected:n(1131,e.DiagnosticCategory.Error,"Property_or_signature_expected_1131","Property or signature expected."),Enum_member_expected:n(1132,e.DiagnosticCategory.Error,"Enum_member_expected_1132","Enum member expected."),Variable_declaration_expected:n(1134,e.DiagnosticCategory.Error,"Variable_declaration_expected_1134","Variable declaration expected."),Argument_expression_expected:n(1135,e.DiagnosticCategory.Error,"Argument_expression_expected_1135","Argument expression expected."),Property_assignment_expected:n(1136,e.DiagnosticCategory.Error,"Property_assignment_expected_1136","Property assignment expected."),Expression_or_comma_expected:n(1137,e.DiagnosticCategory.Error,"Expression_or_comma_expected_1137","Expression or comma expected."),Parameter_declaration_expected:n(1138,e.DiagnosticCategory.Error,"Parameter_declaration_expected_1138","Parameter declaration expected."),Type_parameter_declaration_expected:n(1139,e.DiagnosticCategory.Error,"Type_parameter_declaration_expected_1139","Type parameter declaration expected."),Type_argument_expected:n(1140,e.DiagnosticCategory.Error,"Type_argument_expected_1140","Type argument expected."),String_literal_expected:n(1141,e.DiagnosticCategory.Error,"String_literal_expected_1141","String literal expected."),Line_break_not_permitted_here:n(1142,e.DiagnosticCategory.Error,"Line_break_not_permitted_here_1142","Line break not permitted here."),or_expected:n(1144,e.DiagnosticCategory.Error,"or_expected_1144","'{' or ';' expected."),Declaration_expected:n(1146,e.DiagnosticCategory.Error,"Declaration_expected_1146","Declaration expected."),Import_declarations_in_a_namespace_cannot_reference_a_module:n(1147,e.DiagnosticCategory.Error,"Import_declarations_in_a_namespace_cannot_reference_a_module_1147","Import declarations in a namespace cannot reference a module."),Cannot_use_imports_exports_or_module_augmentations_when_module_is_none:n(1148,e.DiagnosticCategory.Error,"Cannot_use_imports_exports_or_module_augmentations_when_module_is_none_1148","Cannot use imports, exports, or module augmentations when '--module' is 'none'."),File_name_0_differs_from_already_included_file_name_1_only_in_casing:n(1149,e.DiagnosticCategory.Error,"File_name_0_differs_from_already_included_file_name_1_only_in_casing_1149","File name '{0}' differs from already included file name '{1}' only in casing."),const_declarations_must_be_initialized:n(1155,e.DiagnosticCategory.Error,"const_declarations_must_be_initialized_1155","'const' declarations must be initialized."),const_declarations_can_only_be_declared_inside_a_block:n(1156,e.DiagnosticCategory.Error,"const_declarations_can_only_be_declared_inside_a_block_1156","'const' declarations can only be declared inside a block."),let_declarations_can_only_be_declared_inside_a_block:n(1157,e.DiagnosticCategory.Error,"let_declarations_can_only_be_declared_inside_a_block_1157","'let' declarations can only be declared inside a block."),Unterminated_template_literal:n(1160,e.DiagnosticCategory.Error,"Unterminated_template_literal_1160","Unterminated template literal."),Unterminated_regular_expression_literal:n(1161,e.DiagnosticCategory.Error,"Unterminated_regular_expression_literal_1161","Unterminated regular expression literal."),An_object_member_cannot_be_declared_optional:n(1162,e.DiagnosticCategory.Error,"An_object_member_cannot_be_declared_optional_1162","An object member cannot be declared optional."),A_yield_expression_is_only_allowed_in_a_generator_body:n(1163,e.DiagnosticCategory.Error,"A_yield_expression_is_only_allowed_in_a_generator_body_1163","A 'yield' expression is only allowed in a generator body."),Computed_property_names_are_not_allowed_in_enums:n(1164,e.DiagnosticCategory.Error,"Computed_property_names_are_not_allowed_in_enums_1164","Computed property names are not allowed in enums."),A_computed_property_name_in_an_ambient_context_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type:n(1165,e.DiagnosticCategory.Error,"A_computed_property_name_in_an_ambient_context_must_refer_to_an_expression_whose_type_is_a_literal_t_1165","A computed property name in an ambient context must refer to an expression whose type is a literal type or a 'unique symbol' type."),A_computed_property_name_in_a_class_property_declaration_must_have_a_simple_literal_type_or_a_unique_symbol_type:n(1166,e.DiagnosticCategory.Error,"A_computed_property_name_in_a_class_property_declaration_must_have_a_simple_literal_type_or_a_unique_1166","A computed property name in a class property declaration must have a simple literal type or a 'unique symbol' type."),A_computed_property_name_in_a_method_overload_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type:n(1168,e.DiagnosticCategory.Error,"A_computed_property_name_in_a_method_overload_must_refer_to_an_expression_whose_type_is_a_literal_ty_1168","A computed property name in a method overload must refer to an expression whose type is a literal type or a 'unique symbol' type."),A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type:n(1169,e.DiagnosticCategory.Error,"A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_1169","A computed property name in an interface must refer to an expression whose type is a literal type or a 'unique symbol' type."),A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type:n(1170,e.DiagnosticCategory.Error,"A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type__1170","A computed property name in a type literal must refer to an expression whose type is a literal type or a 'unique symbol' type."),A_comma_expression_is_not_allowed_in_a_computed_property_name:n(1171,e.DiagnosticCategory.Error,"A_comma_expression_is_not_allowed_in_a_computed_property_name_1171","A comma expression is not allowed in a computed property name."),extends_clause_already_seen:n(1172,e.DiagnosticCategory.Error,"extends_clause_already_seen_1172","'extends' clause already seen."),extends_clause_must_precede_implements_clause:n(1173,e.DiagnosticCategory.Error,"extends_clause_must_precede_implements_clause_1173","'extends' clause must precede 'implements' clause."),Classes_can_only_extend_a_single_class:n(1174,e.DiagnosticCategory.Error,"Classes_can_only_extend_a_single_class_1174","Classes can only extend a single class."),implements_clause_already_seen:n(1175,e.DiagnosticCategory.Error,"implements_clause_already_seen_1175","'implements' clause already seen."),Interface_declaration_cannot_have_implements_clause:n(1176,e.DiagnosticCategory.Error,"Interface_declaration_cannot_have_implements_clause_1176","Interface declaration cannot have 'implements' clause."),Binary_digit_expected:n(1177,e.DiagnosticCategory.Error,"Binary_digit_expected_1177","Binary digit expected."),Octal_digit_expected:n(1178,e.DiagnosticCategory.Error,"Octal_digit_expected_1178","Octal digit expected."),Unexpected_token_expected:n(1179,e.DiagnosticCategory.Error,"Unexpected_token_expected_1179","Unexpected token. '{' expected."),Property_destructuring_pattern_expected:n(1180,e.DiagnosticCategory.Error,"Property_destructuring_pattern_expected_1180","Property destructuring pattern expected."),Array_element_destructuring_pattern_expected:n(1181,e.DiagnosticCategory.Error,"Array_element_destructuring_pattern_expected_1181","Array element destructuring pattern expected."),A_destructuring_declaration_must_have_an_initializer:n(1182,e.DiagnosticCategory.Error,"A_destructuring_declaration_must_have_an_initializer_1182","A destructuring declaration must have an initializer."),An_implementation_cannot_be_declared_in_ambient_contexts:n(1183,e.DiagnosticCategory.Error,"An_implementation_cannot_be_declared_in_ambient_contexts_1183","An implementation cannot be declared in ambient contexts."),Modifiers_cannot_appear_here:n(1184,e.DiagnosticCategory.Error,"Modifiers_cannot_appear_here_1184","Modifiers cannot appear here."),Merge_conflict_marker_encountered:n(1185,e.DiagnosticCategory.Error,"Merge_conflict_marker_encountered_1185","Merge conflict marker encountered."),A_rest_element_cannot_have_an_initializer:n(1186,e.DiagnosticCategory.Error,"A_rest_element_cannot_have_an_initializer_1186","A rest element cannot have an initializer."),A_parameter_property_may_not_be_declared_using_a_binding_pattern:n(1187,e.DiagnosticCategory.Error,"A_parameter_property_may_not_be_declared_using_a_binding_pattern_1187","A parameter property may not be declared using a binding pattern."),Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement:n(1188,e.DiagnosticCategory.Error,"Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement_1188","Only a single variable declaration is allowed in a 'for...of' statement."),The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer:n(1189,e.DiagnosticCategory.Error,"The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer_1189","The variable declaration of a 'for...in' statement cannot have an initializer."),The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer:n(1190,e.DiagnosticCategory.Error,"The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer_1190","The variable declaration of a 'for...of' statement cannot have an initializer."),An_import_declaration_cannot_have_modifiers:n(1191,e.DiagnosticCategory.Error,"An_import_declaration_cannot_have_modifiers_1191","An import declaration cannot have modifiers."),Module_0_has_no_default_export:n(1192,e.DiagnosticCategory.Error,"Module_0_has_no_default_export_1192","Module '{0}' has no default export."),An_export_declaration_cannot_have_modifiers:n(1193,e.DiagnosticCategory.Error,"An_export_declaration_cannot_have_modifiers_1193","An export declaration cannot have modifiers."),Export_declarations_are_not_permitted_in_a_namespace:n(1194,e.DiagnosticCategory.Error,"Export_declarations_are_not_permitted_in_a_namespace_1194","Export declarations are not permitted in a namespace."),export_Asterisk_does_not_re_export_a_default:n(1195,e.DiagnosticCategory.Error,"export_Asterisk_does_not_re_export_a_default_1195","'export *' does not re-export a default."),Catch_clause_variable_type_annotation_must_be_any_or_unknown_if_specified:n(1196,e.DiagnosticCategory.Error,"Catch_clause_variable_type_annotation_must_be_any_or_unknown_if_specified_1196","Catch clause variable type annotation must be 'any' or 'unknown' if specified."),Catch_clause_variable_cannot_have_an_initializer:n(1197,e.DiagnosticCategory.Error,"Catch_clause_variable_cannot_have_an_initializer_1197","Catch clause variable cannot have an initializer."),An_extended_Unicode_escape_value_must_be_between_0x0_and_0x10FFFF_inclusive:n(1198,e.DiagnosticCategory.Error,"An_extended_Unicode_escape_value_must_be_between_0x0_and_0x10FFFF_inclusive_1198","An extended Unicode escape value must be between 0x0 and 0x10FFFF inclusive."),Unterminated_Unicode_escape_sequence:n(1199,e.DiagnosticCategory.Error,"Unterminated_Unicode_escape_sequence_1199","Unterminated Unicode escape sequence."),Line_terminator_not_permitted_before_arrow:n(1200,e.DiagnosticCategory.Error,"Line_terminator_not_permitted_before_arrow_1200","Line terminator not permitted before arrow."),Import_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_import_Asterisk_as_ns_from_mod_import_a_from_mod_import_d_from_mod_or_another_module_format_instead:n(1202,e.DiagnosticCategory.Error,"Import_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_import_Asterisk_as_1202","Import assignment cannot be used when targeting ECMAScript modules. Consider using 'import * as ns from \"mod\"', 'import {a} from \"mod\"', 'import d from \"mod\"', or another module format instead."),Export_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_export_default_or_another_module_format_instead:n(1203,e.DiagnosticCategory.Error,"Export_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_export_default_or__1203","Export assignment cannot be used when targeting ECMAScript modules. Consider using 'export default' or another module format instead."),Re_exporting_a_type_when_the_isolatedModules_flag_is_provided_requires_using_export_type:n(1205,e.DiagnosticCategory.Error,"Re_exporting_a_type_when_the_isolatedModules_flag_is_provided_requires_using_export_type_1205","Re-exporting a type when the '--isolatedModules' flag is provided requires using 'export type'."),Decorators_are_not_valid_here:n(1206,e.DiagnosticCategory.Error,"Decorators_are_not_valid_here_1206","Decorators are not valid here."),Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name:n(1207,e.DiagnosticCategory.Error,"Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name_1207","Decorators cannot be applied to multiple get/set accessors of the same name."),_0_cannot_be_compiled_under_isolatedModules_because_it_is_considered_a_global_script_file_Add_an_import_export_or_an_empty_export_statement_to_make_it_a_module:n(1208,e.DiagnosticCategory.Error,"_0_cannot_be_compiled_under_isolatedModules_because_it_is_considered_a_global_script_file_Add_an_imp_1208","'{0}' cannot be compiled under '--isolatedModules' because it is considered a global script file. Add an import, export, or an empty 'export {}' statement to make it a module."),Code_contained_in_a_class_is_evaluated_in_JavaScript_s_strict_mode_which_does_not_allow_this_use_of_0_For_more_information_see_https_Colon_Slash_Slashdeveloper_mozilla_org_Slashen_US_Slashdocs_SlashWeb_SlashJavaScript_SlashReference_SlashStrict_mode:n(1210,e.DiagnosticCategory.Error,"Code_contained_in_a_class_is_evaluated_in_JavaScript_s_strict_mode_which_does_not_allow_this_use_of__1210","Code contained in a class is evaluated in JavaScript's strict mode which does not allow this use of '{0}'. For more information, see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Strict_mode."),A_class_declaration_without_the_default_modifier_must_have_a_name:n(1211,e.DiagnosticCategory.Error,"A_class_declaration_without_the_default_modifier_must_have_a_name_1211","A class declaration without the 'default' modifier must have a name."),Identifier_expected_0_is_a_reserved_word_in_strict_mode:n(1212,e.DiagnosticCategory.Error,"Identifier_expected_0_is_a_reserved_word_in_strict_mode_1212","Identifier expected. '{0}' is a reserved word in strict mode."),Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_strict_mode:n(1213,e.DiagnosticCategory.Error,"Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_stric_1213","Identifier expected. '{0}' is a reserved word in strict mode. Class definitions are automatically in strict mode."),Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode:n(1214,e.DiagnosticCategory.Error,"Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode_1214","Identifier expected. '{0}' is a reserved word in strict mode. Modules are automatically in strict mode."),Invalid_use_of_0_Modules_are_automatically_in_strict_mode:n(1215,e.DiagnosticCategory.Error,"Invalid_use_of_0_Modules_are_automatically_in_strict_mode_1215","Invalid use of '{0}'. Modules are automatically in strict mode."),Identifier_expected_esModule_is_reserved_as_an_exported_marker_when_transforming_ECMAScript_modules:n(1216,e.DiagnosticCategory.Error,"Identifier_expected_esModule_is_reserved_as_an_exported_marker_when_transforming_ECMAScript_modules_1216","Identifier expected. '__esModule' is reserved as an exported marker when transforming ECMAScript modules."),Export_assignment_is_not_supported_when_module_flag_is_system:n(1218,e.DiagnosticCategory.Error,"Export_assignment_is_not_supported_when_module_flag_is_system_1218","Export assignment is not supported when '--module' flag is 'system'."),Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_in_your_tsconfig_or_jsconfig_to_remove_this_warning:n(1219,e.DiagnosticCategory.Error,"Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_t_1219","Experimental support for decorators is a feature that is subject to change in a future release. Set the 'experimentalDecorators' option in your 'tsconfig' or 'jsconfig' to remove this warning."),Generators_are_only_available_when_targeting_ECMAScript_2015_or_higher:n(1220,e.DiagnosticCategory.Error,"Generators_are_only_available_when_targeting_ECMAScript_2015_or_higher_1220","Generators are only available when targeting ECMAScript 2015 or higher."),Generators_are_not_allowed_in_an_ambient_context:n(1221,e.DiagnosticCategory.Error,"Generators_are_not_allowed_in_an_ambient_context_1221","Generators are not allowed in an ambient context."),An_overload_signature_cannot_be_declared_as_a_generator:n(1222,e.DiagnosticCategory.Error,"An_overload_signature_cannot_be_declared_as_a_generator_1222","An overload signature cannot be declared as a generator."),_0_tag_already_specified:n(1223,e.DiagnosticCategory.Error,"_0_tag_already_specified_1223","'{0}' tag already specified."),Signature_0_must_be_a_type_predicate:n(1224,e.DiagnosticCategory.Error,"Signature_0_must_be_a_type_predicate_1224","Signature '{0}' must be a type predicate."),Cannot_find_parameter_0:n(1225,e.DiagnosticCategory.Error,"Cannot_find_parameter_0_1225","Cannot find parameter '{0}'."),Type_predicate_0_is_not_assignable_to_1:n(1226,e.DiagnosticCategory.Error,"Type_predicate_0_is_not_assignable_to_1_1226","Type predicate '{0}' is not assignable to '{1}'."),Parameter_0_is_not_in_the_same_position_as_parameter_1:n(1227,e.DiagnosticCategory.Error,"Parameter_0_is_not_in_the_same_position_as_parameter_1_1227","Parameter '{0}' is not in the same position as parameter '{1}'."),A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods:n(1228,e.DiagnosticCategory.Error,"A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods_1228","A type predicate is only allowed in return type position for functions and methods."),A_type_predicate_cannot_reference_a_rest_parameter:n(1229,e.DiagnosticCategory.Error,"A_type_predicate_cannot_reference_a_rest_parameter_1229","A type predicate cannot reference a rest parameter."),A_type_predicate_cannot_reference_element_0_in_a_binding_pattern:n(1230,e.DiagnosticCategory.Error,"A_type_predicate_cannot_reference_element_0_in_a_binding_pattern_1230","A type predicate cannot reference element '{0}' in a binding pattern."),An_export_assignment_must_be_at_the_top_level_of_a_file_or_module_declaration:n(1231,e.DiagnosticCategory.Error,"An_export_assignment_must_be_at_the_top_level_of_a_file_or_module_declaration_1231","An export assignment must be at the top level of a file or module declaration."),An_import_declaration_can_only_be_used_in_a_namespace_or_module:n(1232,e.DiagnosticCategory.Error,"An_import_declaration_can_only_be_used_in_a_namespace_or_module_1232","An import declaration can only be used in a namespace or module."),An_export_declaration_can_only_be_used_in_a_module:n(1233,e.DiagnosticCategory.Error,"An_export_declaration_can_only_be_used_in_a_module_1233","An export declaration can only be used in a module."),An_ambient_module_declaration_is_only_allowed_at_the_top_level_in_a_file:n(1234,e.DiagnosticCategory.Error,"An_ambient_module_declaration_is_only_allowed_at_the_top_level_in_a_file_1234","An ambient module declaration is only allowed at the top level in a file."),A_namespace_declaration_is_only_allowed_in_a_namespace_or_module:n(1235,e.DiagnosticCategory.Error,"A_namespace_declaration_is_only_allowed_in_a_namespace_or_module_1235","A namespace declaration is only allowed in a namespace or module."),The_return_type_of_a_property_decorator_function_must_be_either_void_or_any:n(1236,e.DiagnosticCategory.Error,"The_return_type_of_a_property_decorator_function_must_be_either_void_or_any_1236","The return type of a property decorator function must be either 'void' or 'any'."),The_return_type_of_a_parameter_decorator_function_must_be_either_void_or_any:n(1237,e.DiagnosticCategory.Error,"The_return_type_of_a_parameter_decorator_function_must_be_either_void_or_any_1237","The return type of a parameter decorator function must be either 'void' or 'any'."),Unable_to_resolve_signature_of_class_decorator_when_called_as_an_expression:n(1238,e.DiagnosticCategory.Error,"Unable_to_resolve_signature_of_class_decorator_when_called_as_an_expression_1238","Unable to resolve signature of class decorator when called as an expression."),Unable_to_resolve_signature_of_parameter_decorator_when_called_as_an_expression:n(1239,e.DiagnosticCategory.Error,"Unable_to_resolve_signature_of_parameter_decorator_when_called_as_an_expression_1239","Unable to resolve signature of parameter decorator when called as an expression."),Unable_to_resolve_signature_of_property_decorator_when_called_as_an_expression:n(1240,e.DiagnosticCategory.Error,"Unable_to_resolve_signature_of_property_decorator_when_called_as_an_expression_1240","Unable to resolve signature of property decorator when called as an expression."),Unable_to_resolve_signature_of_method_decorator_when_called_as_an_expression:n(1241,e.DiagnosticCategory.Error,"Unable_to_resolve_signature_of_method_decorator_when_called_as_an_expression_1241","Unable to resolve signature of method decorator when called as an expression."),abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration:n(1242,e.DiagnosticCategory.Error,"abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration_1242","'abstract' modifier can only appear on a class, method, or property declaration."),_0_modifier_cannot_be_used_with_1_modifier:n(1243,e.DiagnosticCategory.Error,"_0_modifier_cannot_be_used_with_1_modifier_1243","'{0}' modifier cannot be used with '{1}' modifier."),Abstract_methods_can_only_appear_within_an_abstract_class:n(1244,e.DiagnosticCategory.Error,"Abstract_methods_can_only_appear_within_an_abstract_class_1244","Abstract methods can only appear within an abstract class."),Method_0_cannot_have_an_implementation_because_it_is_marked_abstract:n(1245,e.DiagnosticCategory.Error,"Method_0_cannot_have_an_implementation_because_it_is_marked_abstract_1245","Method '{0}' cannot have an implementation because it is marked abstract."),An_interface_property_cannot_have_an_initializer:n(1246,e.DiagnosticCategory.Error,"An_interface_property_cannot_have_an_initializer_1246","An interface property cannot have an initializer."),A_type_literal_property_cannot_have_an_initializer:n(1247,e.DiagnosticCategory.Error,"A_type_literal_property_cannot_have_an_initializer_1247","A type literal property cannot have an initializer."),A_class_member_cannot_have_the_0_keyword:n(1248,e.DiagnosticCategory.Error,"A_class_member_cannot_have_the_0_keyword_1248","A class member cannot have the '{0}' keyword."),A_decorator_can_only_decorate_a_method_implementation_not_an_overload:n(1249,e.DiagnosticCategory.Error,"A_decorator_can_only_decorate_a_method_implementation_not_an_overload_1249","A decorator can only decorate a method implementation, not an overload."),Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5:n(1250,e.DiagnosticCategory.Error,"Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_1250","Function declarations are not allowed inside blocks in strict mode when targeting 'ES3' or 'ES5'."),Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_definitions_are_automatically_in_strict_mode:n(1251,e.DiagnosticCategory.Error,"Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_d_1251","Function declarations are not allowed inside blocks in strict mode when targeting 'ES3' or 'ES5'. Class definitions are automatically in strict mode."),Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_are_automatically_in_strict_mode:n(1252,e.DiagnosticCategory.Error,"Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_1252","Function declarations are not allowed inside blocks in strict mode when targeting 'ES3' or 'ES5'. Modules are automatically in strict mode."),_0_tag_cannot_be_used_independently_as_a_top_level_JSDoc_tag:n(1253,e.DiagnosticCategory.Error,"_0_tag_cannot_be_used_independently_as_a_top_level_JSDoc_tag_1253","'{0}' tag cannot be used independently as a top level JSDoc tag."),A_const_initializer_in_an_ambient_context_must_be_a_string_or_numeric_literal_or_literal_enum_reference:n(1254,e.DiagnosticCategory.Error,"A_const_initializer_in_an_ambient_context_must_be_a_string_or_numeric_literal_or_literal_enum_refere_1254","A 'const' initializer in an ambient context must be a string or numeric literal or literal enum reference."),A_definite_assignment_assertion_is_not_permitted_in_this_context:n(1255,e.DiagnosticCategory.Error,"A_definite_assignment_assertion_is_not_permitted_in_this_context_1255","A definite assignment assertion '!' is not permitted in this context."),A_required_element_cannot_follow_an_optional_element:n(1257,e.DiagnosticCategory.Error,"A_required_element_cannot_follow_an_optional_element_1257","A required element cannot follow an optional element."),A_default_export_must_be_at_the_top_level_of_a_file_or_module_declaration:n(1258,e.DiagnosticCategory.Error,"A_default_export_must_be_at_the_top_level_of_a_file_or_module_declaration_1258","A default export must be at the top level of a file or module declaration."),Module_0_can_only_be_default_imported_using_the_1_flag:n(1259,e.DiagnosticCategory.Error,"Module_0_can_only_be_default_imported_using_the_1_flag_1259","Module '{0}' can only be default-imported using the '{1}' flag"),Keywords_cannot_contain_escape_characters:n(1260,e.DiagnosticCategory.Error,"Keywords_cannot_contain_escape_characters_1260","Keywords cannot contain escape characters."),Already_included_file_name_0_differs_from_file_name_1_only_in_casing:n(1261,e.DiagnosticCategory.Error,"Already_included_file_name_0_differs_from_file_name_1_only_in_casing_1261","Already included file name '{0}' differs from file name '{1}' only in casing."),Identifier_expected_0_is_a_reserved_word_at_the_top_level_of_a_module:n(1262,e.DiagnosticCategory.Error,"Identifier_expected_0_is_a_reserved_word_at_the_top_level_of_a_module_1262","Identifier expected. '{0}' is a reserved word at the top-level of a module."),Declarations_with_initializers_cannot_also_have_definite_assignment_assertions:n(1263,e.DiagnosticCategory.Error,"Declarations_with_initializers_cannot_also_have_definite_assignment_assertions_1263","Declarations with initializers cannot also have definite assignment assertions."),Declarations_with_definite_assignment_assertions_must_also_have_type_annotations:n(1264,e.DiagnosticCategory.Error,"Declarations_with_definite_assignment_assertions_must_also_have_type_annotations_1264","Declarations with definite assignment assertions must also have type annotations."),A_rest_element_cannot_follow_another_rest_element:n(1265,e.DiagnosticCategory.Error,"A_rest_element_cannot_follow_another_rest_element_1265","A rest element cannot follow another rest element."),An_optional_element_cannot_follow_a_rest_element:n(1266,e.DiagnosticCategory.Error,"An_optional_element_cannot_follow_a_rest_element_1266","An optional element cannot follow a rest element."),Property_0_cannot_have_an_initializer_because_it_is_marked_abstract:n(1267,e.DiagnosticCategory.Error,"Property_0_cannot_have_an_initializer_because_it_is_marked_abstract_1267","Property '{0}' cannot have an initializer because it is marked abstract."),An_index_signature_parameter_type_must_be_string_number_symbol_or_a_template_literal_type:n(1268,e.DiagnosticCategory.Error,"An_index_signature_parameter_type_must_be_string_number_symbol_or_a_template_literal_type_1268","An index signature parameter type must be 'string', 'number', 'symbol', or a template literal type."),with_statements_are_not_allowed_in_an_async_function_block:n(1300,e.DiagnosticCategory.Error,"with_statements_are_not_allowed_in_an_async_function_block_1300","'with' statements are not allowed in an async function block."),await_expressions_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules:n(1308,e.DiagnosticCategory.Error,"await_expressions_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules_1308","'await' expressions are only allowed within async functions and at the top levels of modules."),Did_you_mean_to_use_a_Colon_An_can_only_follow_a_property_name_when_the_containing_object_literal_is_part_of_a_destructuring_pattern:n(1312,e.DiagnosticCategory.Error,"Did_you_mean_to_use_a_Colon_An_can_only_follow_a_property_name_when_the_containing_object_literal_is_1312","Did you mean to use a ':'? An '=' can only follow a property name when the containing object literal is part of a destructuring pattern."),The_body_of_an_if_statement_cannot_be_the_empty_statement:n(1313,e.DiagnosticCategory.Error,"The_body_of_an_if_statement_cannot_be_the_empty_statement_1313","The body of an 'if' statement cannot be the empty statement."),Global_module_exports_may_only_appear_in_module_files:n(1314,e.DiagnosticCategory.Error,"Global_module_exports_may_only_appear_in_module_files_1314","Global module exports may only appear in module files."),Global_module_exports_may_only_appear_in_declaration_files:n(1315,e.DiagnosticCategory.Error,"Global_module_exports_may_only_appear_in_declaration_files_1315","Global module exports may only appear in declaration files."),Global_module_exports_may_only_appear_at_top_level:n(1316,e.DiagnosticCategory.Error,"Global_module_exports_may_only_appear_at_top_level_1316","Global module exports may only appear at top level."),A_parameter_property_cannot_be_declared_using_a_rest_parameter:n(1317,e.DiagnosticCategory.Error,"A_parameter_property_cannot_be_declared_using_a_rest_parameter_1317","A parameter property cannot be declared using a rest parameter."),An_abstract_accessor_cannot_have_an_implementation:n(1318,e.DiagnosticCategory.Error,"An_abstract_accessor_cannot_have_an_implementation_1318","An abstract accessor cannot have an implementation."),A_default_export_can_only_be_used_in_an_ECMAScript_style_module:n(1319,e.DiagnosticCategory.Error,"A_default_export_can_only_be_used_in_an_ECMAScript_style_module_1319","A default export can only be used in an ECMAScript-style module."),Type_of_await_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:n(1320,e.DiagnosticCategory.Error,"Type_of_await_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member_1320","Type of 'await' operand must either be a valid promise or must not contain a callable 'then' member."),Type_of_yield_operand_in_an_async_generator_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:n(1321,e.DiagnosticCategory.Error,"Type_of_yield_operand_in_an_async_generator_must_either_be_a_valid_promise_or_must_not_contain_a_cal_1321","Type of 'yield' operand in an async generator must either be a valid promise or must not contain a callable 'then' member."),Type_of_iterated_elements_of_a_yield_Asterisk_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:n(1322,e.DiagnosticCategory.Error,"Type_of_iterated_elements_of_a_yield_Asterisk_operand_must_either_be_a_valid_promise_or_must_not_con_1322","Type of iterated elements of a 'yield*' operand must either be a valid promise or must not contain a callable 'then' member."),Dynamic_imports_are_only_supported_when_the_module_flag_is_set_to_es2020_esnext_commonjs_amd_system_or_umd:n(1323,e.DiagnosticCategory.Error,"Dynamic_imports_are_only_supported_when_the_module_flag_is_set_to_es2020_esnext_commonjs_amd_system__1323","Dynamic imports are only supported when the '--module' flag is set to 'es2020', 'esnext', 'commonjs', 'amd', 'system', or 'umd'."),Dynamic_import_must_have_one_specifier_as_an_argument:n(1324,e.DiagnosticCategory.Error,"Dynamic_import_must_have_one_specifier_as_an_argument_1324","Dynamic import must have one specifier as an argument."),Specifier_of_dynamic_import_cannot_be_spread_element:n(1325,e.DiagnosticCategory.Error,"Specifier_of_dynamic_import_cannot_be_spread_element_1325","Specifier of dynamic import cannot be spread element."),Dynamic_import_cannot_have_type_arguments:n(1326,e.DiagnosticCategory.Error,"Dynamic_import_cannot_have_type_arguments_1326","Dynamic import cannot have type arguments."),String_literal_with_double_quotes_expected:n(1327,e.DiagnosticCategory.Error,"String_literal_with_double_quotes_expected_1327","String literal with double quotes expected."),Property_value_can_only_be_string_literal_numeric_literal_true_false_null_object_literal_or_array_literal:n(1328,e.DiagnosticCategory.Error,"Property_value_can_only_be_string_literal_numeric_literal_true_false_null_object_literal_or_array_li_1328","Property value can only be string literal, numeric literal, 'true', 'false', 'null', object literal or array literal."),_0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write_0:n(1329,e.DiagnosticCategory.Error,"_0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write__1329","'{0}' accepts too few arguments to be used as a decorator here. Did you mean to call it first and write '@{0}()'?"),A_property_of_an_interface_or_type_literal_whose_type_is_a_unique_symbol_type_must_be_readonly:n(1330,e.DiagnosticCategory.Error,"A_property_of_an_interface_or_type_literal_whose_type_is_a_unique_symbol_type_must_be_readonly_1330","A property of an interface or type literal whose type is a 'unique symbol' type must be 'readonly'."),A_property_of_a_class_whose_type_is_a_unique_symbol_type_must_be_both_static_and_readonly:n(1331,e.DiagnosticCategory.Error,"A_property_of_a_class_whose_type_is_a_unique_symbol_type_must_be_both_static_and_readonly_1331","A property of a class whose type is a 'unique symbol' type must be both 'static' and 'readonly'."),A_variable_whose_type_is_a_unique_symbol_type_must_be_const:n(1332,e.DiagnosticCategory.Error,"A_variable_whose_type_is_a_unique_symbol_type_must_be_const_1332","A variable whose type is a 'unique symbol' type must be 'const'."),unique_symbol_types_may_not_be_used_on_a_variable_declaration_with_a_binding_name:n(1333,e.DiagnosticCategory.Error,"unique_symbol_types_may_not_be_used_on_a_variable_declaration_with_a_binding_name_1333","'unique symbol' types may not be used on a variable declaration with a binding name."),unique_symbol_types_are_only_allowed_on_variables_in_a_variable_statement:n(1334,e.DiagnosticCategory.Error,"unique_symbol_types_are_only_allowed_on_variables_in_a_variable_statement_1334","'unique symbol' types are only allowed on variables in a variable statement."),unique_symbol_types_are_not_allowed_here:n(1335,e.DiagnosticCategory.Error,"unique_symbol_types_are_not_allowed_here_1335","'unique symbol' types are not allowed here."),An_index_signature_parameter_type_cannot_be_a_literal_type_or_generic_type_Consider_using_a_mapped_object_type_instead:n(1337,e.DiagnosticCategory.Error,"An_index_signature_parameter_type_cannot_be_a_literal_type_or_generic_type_Consider_using_a_mapped_o_1337","An index signature parameter type cannot be a literal type or generic type. Consider using a mapped object type instead."),infer_declarations_are_only_permitted_in_the_extends_clause_of_a_conditional_type:n(1338,e.DiagnosticCategory.Error,"infer_declarations_are_only_permitted_in_the_extends_clause_of_a_conditional_type_1338","'infer' declarations are only permitted in the 'extends' clause of a conditional type."),Module_0_does_not_refer_to_a_value_but_is_used_as_a_value_here:n(1339,e.DiagnosticCategory.Error,"Module_0_does_not_refer_to_a_value_but_is_used_as_a_value_here_1339","Module '{0}' does not refer to a value, but is used as a value here."),Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0:n(1340,e.DiagnosticCategory.Error,"Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0_1340","Module '{0}' does not refer to a type, but is used as a type here. Did you mean 'typeof import('{0}')'?"),Type_arguments_cannot_be_used_here:n(1342,e.DiagnosticCategory.Error,"Type_arguments_cannot_be_used_here_1342","Type arguments cannot be used here."),The_import_meta_meta_property_is_only_allowed_when_the_module_option_is_es2020_esnext_or_system:n(1343,e.DiagnosticCategory.Error,"The_import_meta_meta_property_is_only_allowed_when_the_module_option_is_es2020_esnext_or_system_1343","The 'import.meta' meta-property is only allowed when the '--module' option is 'es2020', 'esnext', or 'system'."),A_label_is_not_allowed_here:n(1344,e.DiagnosticCategory.Error,"A_label_is_not_allowed_here_1344","'A label is not allowed here."),An_expression_of_type_void_cannot_be_tested_for_truthiness:n(1345,e.DiagnosticCategory.Error,"An_expression_of_type_void_cannot_be_tested_for_truthiness_1345","An expression of type 'void' cannot be tested for truthiness."),This_parameter_is_not_allowed_with_use_strict_directive:n(1346,e.DiagnosticCategory.Error,"This_parameter_is_not_allowed_with_use_strict_directive_1346","This parameter is not allowed with 'use strict' directive."),use_strict_directive_cannot_be_used_with_non_simple_parameter_list:n(1347,e.DiagnosticCategory.Error,"use_strict_directive_cannot_be_used_with_non_simple_parameter_list_1347","'use strict' directive cannot be used with non-simple parameter list."),Non_simple_parameter_declared_here:n(1348,e.DiagnosticCategory.Error,"Non_simple_parameter_declared_here_1348","Non-simple parameter declared here."),use_strict_directive_used_here:n(1349,e.DiagnosticCategory.Error,"use_strict_directive_used_here_1349","'use strict' directive used here."),Print_the_final_configuration_instead_of_building:n(1350,e.DiagnosticCategory.Message,"Print_the_final_configuration_instead_of_building_1350","Print the final configuration instead of building."),An_identifier_or_keyword_cannot_immediately_follow_a_numeric_literal:n(1351,e.DiagnosticCategory.Error,"An_identifier_or_keyword_cannot_immediately_follow_a_numeric_literal_1351","An identifier or keyword cannot immediately follow a numeric literal."),A_bigint_literal_cannot_use_exponential_notation:n(1352,e.DiagnosticCategory.Error,"A_bigint_literal_cannot_use_exponential_notation_1352","A bigint literal cannot use exponential notation."),A_bigint_literal_must_be_an_integer:n(1353,e.DiagnosticCategory.Error,"A_bigint_literal_must_be_an_integer_1353","A bigint literal must be an integer."),readonly_type_modifier_is_only_permitted_on_array_and_tuple_literal_types:n(1354,e.DiagnosticCategory.Error,"readonly_type_modifier_is_only_permitted_on_array_and_tuple_literal_types_1354","'readonly' type modifier is only permitted on array and tuple literal types."),A_const_assertions_can_only_be_applied_to_references_to_enum_members_or_string_number_boolean_array_or_object_literals:n(1355,e.DiagnosticCategory.Error,"A_const_assertions_can_only_be_applied_to_references_to_enum_members_or_string_number_boolean_array__1355","A 'const' assertions can only be applied to references to enum members, or string, number, boolean, array, or object literals."),Did_you_mean_to_mark_this_function_as_async:n(1356,e.DiagnosticCategory.Error,"Did_you_mean_to_mark_this_function_as_async_1356","Did you mean to mark this function as 'async'?"),An_enum_member_name_must_be_followed_by_a_or:n(1357,e.DiagnosticCategory.Error,"An_enum_member_name_must_be_followed_by_a_or_1357","An enum member name must be followed by a ',', '=', or '}'."),Tagged_template_expressions_are_not_permitted_in_an_optional_chain:n(1358,e.DiagnosticCategory.Error,"Tagged_template_expressions_are_not_permitted_in_an_optional_chain_1358","Tagged template expressions are not permitted in an optional chain."),Identifier_expected_0_is_a_reserved_word_that_cannot_be_used_here:n(1359,e.DiagnosticCategory.Error,"Identifier_expected_0_is_a_reserved_word_that_cannot_be_used_here_1359","Identifier expected. '{0}' is a reserved word that cannot be used here."),Did_you_mean_to_parenthesize_this_function_type:n(1360,e.DiagnosticCategory.Error,"Did_you_mean_to_parenthesize_this_function_type_1360","Did you mean to parenthesize this function type?"),_0_cannot_be_used_as_a_value_because_it_was_imported_using_import_type:n(1361,e.DiagnosticCategory.Error,"_0_cannot_be_used_as_a_value_because_it_was_imported_using_import_type_1361","'{0}' cannot be used as a value because it was imported using 'import type'."),_0_cannot_be_used_as_a_value_because_it_was_exported_using_export_type:n(1362,e.DiagnosticCategory.Error,"_0_cannot_be_used_as_a_value_because_it_was_exported_using_export_type_1362","'{0}' cannot be used as a value because it was exported using 'export type'."),A_type_only_import_can_specify_a_default_import_or_named_bindings_but_not_both:n(1363,e.DiagnosticCategory.Error,"A_type_only_import_can_specify_a_default_import_or_named_bindings_but_not_both_1363","A type-only import can specify a default import or named bindings, but not both."),Convert_to_type_only_export:n(1364,e.DiagnosticCategory.Message,"Convert_to_type_only_export_1364","Convert to type-only export"),Convert_all_re_exported_types_to_type_only_exports:n(1365,e.DiagnosticCategory.Message,"Convert_all_re_exported_types_to_type_only_exports_1365","Convert all re-exported types to type-only exports"),Split_into_two_separate_import_declarations:n(1366,e.DiagnosticCategory.Message,"Split_into_two_separate_import_declarations_1366","Split into two separate import declarations"),Split_all_invalid_type_only_imports:n(1367,e.DiagnosticCategory.Message,"Split_all_invalid_type_only_imports_1367","Split all invalid type-only imports"),Did_you_mean_0:n(1369,e.DiagnosticCategory.Message,"Did_you_mean_0_1369","Did you mean '{0}'?"),This_import_is_never_used_as_a_value_and_must_use_import_type_because_importsNotUsedAsValues_is_set_to_error:n(1371,e.DiagnosticCategory.Error,"This_import_is_never_used_as_a_value_and_must_use_import_type_because_importsNotUsedAsValues_is_set__1371","This import is never used as a value and must use 'import type' because 'importsNotUsedAsValues' is set to 'error'."),Convert_to_type_only_import:n(1373,e.DiagnosticCategory.Message,"Convert_to_type_only_import_1373","Convert to type-only import"),Convert_all_imports_not_used_as_a_value_to_type_only_imports:n(1374,e.DiagnosticCategory.Message,"Convert_all_imports_not_used_as_a_value_to_type_only_imports_1374","Convert all imports not used as a value to type-only imports"),await_expressions_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_has_no_imports_or_exports_Consider_adding_an_empty_export_to_make_this_file_a_module:n(1375,e.DiagnosticCategory.Error,"await_expressions_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_fi_1375","'await' expressions are only allowed at the top level of a file when that file is a module, but this file has no imports or exports. Consider adding an empty 'export {}' to make this file a module."),_0_was_imported_here:n(1376,e.DiagnosticCategory.Message,"_0_was_imported_here_1376","'{0}' was imported here."),_0_was_exported_here:n(1377,e.DiagnosticCategory.Message,"_0_was_exported_here_1377","'{0}' was exported here."),Top_level_await_expressions_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_target_option_is_set_to_es2017_or_higher:n(1378,e.DiagnosticCategory.Error,"Top_level_await_expressions_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_t_1378","Top-level 'await' expressions are only allowed when the 'module' option is set to 'esnext' or 'system', and the 'target' option is set to 'es2017' or higher."),An_import_alias_cannot_reference_a_declaration_that_was_exported_using_export_type:n(1379,e.DiagnosticCategory.Error,"An_import_alias_cannot_reference_a_declaration_that_was_exported_using_export_type_1379","An import alias cannot reference a declaration that was exported using 'export type'."),An_import_alias_cannot_reference_a_declaration_that_was_imported_using_import_type:n(1380,e.DiagnosticCategory.Error,"An_import_alias_cannot_reference_a_declaration_that_was_imported_using_import_type_1380","An import alias cannot reference a declaration that was imported using 'import type'."),Unexpected_token_Did_you_mean_or_rbrace:n(1381,e.DiagnosticCategory.Error,"Unexpected_token_Did_you_mean_or_rbrace_1381","Unexpected token. Did you mean `{'}'}` or `&rbrace;`?"),Unexpected_token_Did_you_mean_or_gt:n(1382,e.DiagnosticCategory.Error,"Unexpected_token_Did_you_mean_or_gt_1382","Unexpected token. Did you mean `{'>'}` or `&gt;`?"),Only_named_exports_may_use_export_type:n(1383,e.DiagnosticCategory.Error,"Only_named_exports_may_use_export_type_1383","Only named exports may use 'export type'."),A_new_expression_with_type_arguments_must_always_be_followed_by_a_parenthesized_argument_list:n(1384,e.DiagnosticCategory.Error,"A_new_expression_with_type_arguments_must_always_be_followed_by_a_parenthesized_argument_list_1384","A 'new' expression with type arguments must always be followed by a parenthesized argument list."),Function_type_notation_must_be_parenthesized_when_used_in_a_union_type:n(1385,e.DiagnosticCategory.Error,"Function_type_notation_must_be_parenthesized_when_used_in_a_union_type_1385","Function type notation must be parenthesized when used in a union type."),Constructor_type_notation_must_be_parenthesized_when_used_in_a_union_type:n(1386,e.DiagnosticCategory.Error,"Constructor_type_notation_must_be_parenthesized_when_used_in_a_union_type_1386","Constructor type notation must be parenthesized when used in a union type."),Function_type_notation_must_be_parenthesized_when_used_in_an_intersection_type:n(1387,e.DiagnosticCategory.Error,"Function_type_notation_must_be_parenthesized_when_used_in_an_intersection_type_1387","Function type notation must be parenthesized when used in an intersection type."),Constructor_type_notation_must_be_parenthesized_when_used_in_an_intersection_type:n(1388,e.DiagnosticCategory.Error,"Constructor_type_notation_must_be_parenthesized_when_used_in_an_intersection_type_1388","Constructor type notation must be parenthesized when used in an intersection type."),_0_is_not_allowed_as_a_variable_declaration_name:n(1389,e.DiagnosticCategory.Error,"_0_is_not_allowed_as_a_variable_declaration_name_1389","'{0}' is not allowed as a variable declaration name."),Provides_a_root_package_name_when_using_outFile_with_declarations:n(1390,e.DiagnosticCategory.Message,"Provides_a_root_package_name_when_using_outFile_with_declarations_1390","Provides a root package name when using outFile with declarations."),The_bundledPackageName_option_must_be_provided_when_using_outFile_and_node_module_resolution_with_declaration_emit:n(1391,e.DiagnosticCategory.Error,"The_bundledPackageName_option_must_be_provided_when_using_outFile_and_node_module_resolution_with_de_1391","The 'bundledPackageName' option must be provided when using outFile and node module resolution with declaration emit."),An_import_alias_cannot_use_import_type:n(1392,e.DiagnosticCategory.Error,"An_import_alias_cannot_use_import_type_1392","An import alias cannot use 'import type'"),Imported_via_0_from_file_1:n(1393,e.DiagnosticCategory.Message,"Imported_via_0_from_file_1_1393","Imported via {0} from file '{1}'"),Imported_via_0_from_file_1_with_packageId_2:n(1394,e.DiagnosticCategory.Message,"Imported_via_0_from_file_1_with_packageId_2_1394","Imported via {0} from file '{1}' with packageId '{2}'"),Imported_via_0_from_file_1_to_import_importHelpers_as_specified_in_compilerOptions:n(1395,e.DiagnosticCategory.Message,"Imported_via_0_from_file_1_to_import_importHelpers_as_specified_in_compilerOptions_1395","Imported via {0} from file '{1}' to import 'importHelpers' as specified in compilerOptions"),Imported_via_0_from_file_1_with_packageId_2_to_import_importHelpers_as_specified_in_compilerOptions:n(1396,e.DiagnosticCategory.Message,"Imported_via_0_from_file_1_with_packageId_2_to_import_importHelpers_as_specified_in_compilerOptions_1396","Imported via {0} from file '{1}' with packageId '{2}' to import 'importHelpers' as specified in compilerOptions"),Imported_via_0_from_file_1_to_import_jsx_and_jsxs_factory_functions:n(1397,e.DiagnosticCategory.Message,"Imported_via_0_from_file_1_to_import_jsx_and_jsxs_factory_functions_1397","Imported via {0} from file '{1}' to import 'jsx' and 'jsxs' factory functions"),Imported_via_0_from_file_1_with_packageId_2_to_import_jsx_and_jsxs_factory_functions:n(1398,e.DiagnosticCategory.Message,"Imported_via_0_from_file_1_with_packageId_2_to_import_jsx_and_jsxs_factory_functions_1398","Imported via {0} from file '{1}' with packageId '{2}' to import 'jsx' and 'jsxs' factory functions"),File_is_included_via_import_here:n(1399,e.DiagnosticCategory.Message,"File_is_included_via_import_here_1399","File is included via import here."),Referenced_via_0_from_file_1:n(1400,e.DiagnosticCategory.Message,"Referenced_via_0_from_file_1_1400","Referenced via '{0}' from file '{1}'"),File_is_included_via_reference_here:n(1401,e.DiagnosticCategory.Message,"File_is_included_via_reference_here_1401","File is included via reference here."),Type_library_referenced_via_0_from_file_1:n(1402,e.DiagnosticCategory.Message,"Type_library_referenced_via_0_from_file_1_1402","Type library referenced via '{0}' from file '{1}'"),Type_library_referenced_via_0_from_file_1_with_packageId_2:n(1403,e.DiagnosticCategory.Message,"Type_library_referenced_via_0_from_file_1_with_packageId_2_1403","Type library referenced via '{0}' from file '{1}' with packageId '{2}'"),File_is_included_via_type_library_reference_here:n(1404,e.DiagnosticCategory.Message,"File_is_included_via_type_library_reference_here_1404","File is included via type library reference here."),Library_referenced_via_0_from_file_1:n(1405,e.DiagnosticCategory.Message,"Library_referenced_via_0_from_file_1_1405","Library referenced via '{0}' from file '{1}'"),File_is_included_via_library_reference_here:n(1406,e.DiagnosticCategory.Message,"File_is_included_via_library_reference_here_1406","File is included via library reference here."),Matched_by_include_pattern_0_in_1:n(1407,e.DiagnosticCategory.Message,"Matched_by_include_pattern_0_in_1_1407","Matched by include pattern '{0}' in '{1}'"),File_is_matched_by_include_pattern_specified_here:n(1408,e.DiagnosticCategory.Message,"File_is_matched_by_include_pattern_specified_here_1408","File is matched by include pattern specified here."),Part_of_files_list_in_tsconfig_json:n(1409,e.DiagnosticCategory.Message,"Part_of_files_list_in_tsconfig_json_1409","Part of 'files' list in tsconfig.json"),File_is_matched_by_files_list_specified_here:n(1410,e.DiagnosticCategory.Message,"File_is_matched_by_files_list_specified_here_1410","File is matched by 'files' list specified here."),Output_from_referenced_project_0_included_because_1_specified:n(1411,e.DiagnosticCategory.Message,"Output_from_referenced_project_0_included_because_1_specified_1411","Output from referenced project '{0}' included because '{1}' specified"),Output_from_referenced_project_0_included_because_module_is_specified_as_none:n(1412,e.DiagnosticCategory.Message,"Output_from_referenced_project_0_included_because_module_is_specified_as_none_1412","Output from referenced project '{0}' included because '--module' is specified as 'none'"),File_is_output_from_referenced_project_specified_here:n(1413,e.DiagnosticCategory.Message,"File_is_output_from_referenced_project_specified_here_1413","File is output from referenced project specified here."),Source_from_referenced_project_0_included_because_1_specified:n(1414,e.DiagnosticCategory.Message,"Source_from_referenced_project_0_included_because_1_specified_1414","Source from referenced project '{0}' included because '{1}' specified"),Source_from_referenced_project_0_included_because_module_is_specified_as_none:n(1415,e.DiagnosticCategory.Message,"Source_from_referenced_project_0_included_because_module_is_specified_as_none_1415","Source from referenced project '{0}' included because '--module' is specified as 'none'"),File_is_source_from_referenced_project_specified_here:n(1416,e.DiagnosticCategory.Message,"File_is_source_from_referenced_project_specified_here_1416","File is source from referenced project specified here."),Entry_point_of_type_library_0_specified_in_compilerOptions:n(1417,e.DiagnosticCategory.Message,"Entry_point_of_type_library_0_specified_in_compilerOptions_1417","Entry point of type library '{0}' specified in compilerOptions"),Entry_point_of_type_library_0_specified_in_compilerOptions_with_packageId_1:n(1418,e.DiagnosticCategory.Message,"Entry_point_of_type_library_0_specified_in_compilerOptions_with_packageId_1_1418","Entry point of type library '{0}' specified in compilerOptions with packageId '{1}'"),File_is_entry_point_of_type_library_specified_here:n(1419,e.DiagnosticCategory.Message,"File_is_entry_point_of_type_library_specified_here_1419","File is entry point of type library specified here."),Entry_point_for_implicit_type_library_0:n(1420,e.DiagnosticCategory.Message,"Entry_point_for_implicit_type_library_0_1420","Entry point for implicit type library '{0}'"),Entry_point_for_implicit_type_library_0_with_packageId_1:n(1421,e.DiagnosticCategory.Message,"Entry_point_for_implicit_type_library_0_with_packageId_1_1421","Entry point for implicit type library '{0}' with packageId '{1}'"),Library_0_specified_in_compilerOptions:n(1422,e.DiagnosticCategory.Message,"Library_0_specified_in_compilerOptions_1422","Library '{0}' specified in compilerOptions"),File_is_library_specified_here:n(1423,e.DiagnosticCategory.Message,"File_is_library_specified_here_1423","File is library specified here."),Default_library:n(1424,e.DiagnosticCategory.Message,"Default_library_1424","Default library"),Default_library_for_target_0:n(1425,e.DiagnosticCategory.Message,"Default_library_for_target_0_1425","Default library for target '{0}'"),File_is_default_library_for_target_specified_here:n(1426,e.DiagnosticCategory.Message,"File_is_default_library_for_target_specified_here_1426","File is default library for target specified here."),Root_file_specified_for_compilation:n(1427,e.DiagnosticCategory.Message,"Root_file_specified_for_compilation_1427","Root file specified for compilation"),File_is_output_of_project_reference_source_0:n(1428,e.DiagnosticCategory.Message,"File_is_output_of_project_reference_source_0_1428","File is output of project reference source '{0}'"),File_redirects_to_file_0:n(1429,e.DiagnosticCategory.Message,"File_redirects_to_file_0_1429","File redirects to file '{0}'"),The_file_is_in_the_program_because_Colon:n(1430,e.DiagnosticCategory.Message,"The_file_is_in_the_program_because_Colon_1430","The file is in the program because:"),for_await_loops_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_has_no_imports_or_exports_Consider_adding_an_empty_export_to_make_this_file_a_module:n(1431,e.DiagnosticCategory.Error,"for_await_loops_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_1431","'for await' loops are only allowed at the top level of a file when that file is a module, but this file has no imports or exports. Consider adding an empty 'export {}' to make this file a module."),Top_level_for_await_loops_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_target_option_is_set_to_es2017_or_higher:n(1432,e.DiagnosticCategory.Error,"Top_level_for_await_loops_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_1432","Top-level 'for await' loops are only allowed when the 'module' option is set to 'esnext' or 'system', and the 'target' option is set to 'es2017' or higher."),Decorators_may_not_be_applied_to_this_parameters:n(1433,e.DiagnosticCategory.Error,"Decorators_may_not_be_applied_to_this_parameters_1433","Decorators may not be applied to 'this' parameters."),Unexpected_keyword_or_identifier:n(1434,e.DiagnosticCategory.Error,"Unexpected_keyword_or_identifier_1434","Unexpected keyword or identifier."),Unknown_keyword_or_identifier_Did_you_mean_0:n(1435,e.DiagnosticCategory.Error,"Unknown_keyword_or_identifier_Did_you_mean_0_1435","Unknown keyword or identifier. Did you mean '{0}'?"),Decorators_must_precede_the_name_and_all_keywords_of_property_declarations:n(1436,e.DiagnosticCategory.Error,"Decorators_must_precede_the_name_and_all_keywords_of_property_declarations_1436","Decorators must precede the name and all keywords of property declarations."),Namespace_must_be_given_a_name:n(1437,e.DiagnosticCategory.Error,"Namespace_must_be_given_a_name_1437","Namespace must be given a name."),Interface_must_be_given_a_name:n(1438,e.DiagnosticCategory.Error,"Interface_must_be_given_a_name_1438","Interface must be given a name."),Type_alias_must_be_given_a_name:n(1439,e.DiagnosticCategory.Error,"Type_alias_must_be_given_a_name_1439","Type alias must be given a name."),Variable_declaration_not_allowed_at_this_location:n(1440,e.DiagnosticCategory.Error,"Variable_declaration_not_allowed_at_this_location_1440","Variable declaration not allowed at this location."),Cannot_start_a_function_call_in_a_type_annotation:n(1441,e.DiagnosticCategory.Error,"Cannot_start_a_function_call_in_a_type_annotation_1441","Cannot start a function call in a type annotation."),Expected_for_property_initializer:n(1442,e.DiagnosticCategory.Error,"Expected_for_property_initializer_1442","Expected '=' for property initializer."),Module_declaration_names_may_only_use_or_quoted_strings:n(1443,e.DiagnosticCategory.Error,"Module_declaration_names_may_only_use_or_quoted_strings_1443","Module declaration names may only use ' or \" quoted strings."),The_types_of_0_are_incompatible_between_these_types:n(2200,e.DiagnosticCategory.Error,"The_types_of_0_are_incompatible_between_these_types_2200","The types of '{0}' are incompatible between these types."),The_types_returned_by_0_are_incompatible_between_these_types:n(2201,e.DiagnosticCategory.Error,"The_types_returned_by_0_are_incompatible_between_these_types_2201","The types returned by '{0}' are incompatible between these types."),Call_signature_return_types_0_and_1_are_incompatible:n(2202,e.DiagnosticCategory.Error,"Call_signature_return_types_0_and_1_are_incompatible_2202","Call signature return types '{0}' and '{1}' are incompatible.",void 0,!0),Construct_signature_return_types_0_and_1_are_incompatible:n(2203,e.DiagnosticCategory.Error,"Construct_signature_return_types_0_and_1_are_incompatible_2203","Construct signature return types '{0}' and '{1}' are incompatible.",void 0,!0),Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1:n(2204,e.DiagnosticCategory.Error,"Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1_2204","Call signatures with no arguments have incompatible return types '{0}' and '{1}'.",void 0,!0),Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1:n(2205,e.DiagnosticCategory.Error,"Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1_2205","Construct signatures with no arguments have incompatible return types '{0}' and '{1}'.",void 0,!0),Duplicate_identifier_0:n(2300,e.DiagnosticCategory.Error,"Duplicate_identifier_0_2300","Duplicate identifier '{0}'."),Initializer_of_instance_member_variable_0_cannot_reference_identifier_1_declared_in_the_constructor:n(2301,e.DiagnosticCategory.Error,"Initializer_of_instance_member_variable_0_cannot_reference_identifier_1_declared_in_the_constructor_2301","Initializer of instance member variable '{0}' cannot reference identifier '{1}' declared in the constructor."),Static_members_cannot_reference_class_type_parameters:n(2302,e.DiagnosticCategory.Error,"Static_members_cannot_reference_class_type_parameters_2302","Static members cannot reference class type parameters."),Circular_definition_of_import_alias_0:n(2303,e.DiagnosticCategory.Error,"Circular_definition_of_import_alias_0_2303","Circular definition of import alias '{0}'."),Cannot_find_name_0:n(2304,e.DiagnosticCategory.Error,"Cannot_find_name_0_2304","Cannot find name '{0}'."),Module_0_has_no_exported_member_1:n(2305,e.DiagnosticCategory.Error,"Module_0_has_no_exported_member_1_2305","Module '{0}' has no exported member '{1}'."),File_0_is_not_a_module:n(2306,e.DiagnosticCategory.Error,"File_0_is_not_a_module_2306","File '{0}' is not a module."),Cannot_find_module_0_or_its_corresponding_type_declarations:n(2307,e.DiagnosticCategory.Error,"Cannot_find_module_0_or_its_corresponding_type_declarations_2307","Cannot find module '{0}' or its corresponding type declarations."),Module_0_has_already_exported_a_member_named_1_Consider_explicitly_re_exporting_to_resolve_the_ambiguity:n(2308,e.DiagnosticCategory.Error,"Module_0_has_already_exported_a_member_named_1_Consider_explicitly_re_exporting_to_resolve_the_ambig_2308","Module {0} has already exported a member named '{1}'. Consider explicitly re-exporting to resolve the ambiguity."),An_export_assignment_cannot_be_used_in_a_module_with_other_exported_elements:n(2309,e.DiagnosticCategory.Error,"An_export_assignment_cannot_be_used_in_a_module_with_other_exported_elements_2309","An export assignment cannot be used in a module with other exported elements."),Type_0_recursively_references_itself_as_a_base_type:n(2310,e.DiagnosticCategory.Error,"Type_0_recursively_references_itself_as_a_base_type_2310","Type '{0}' recursively references itself as a base type."),A_class_may_only_extend_another_class:n(2311,e.DiagnosticCategory.Error,"A_class_may_only_extend_another_class_2311","A class may only extend another class."),An_interface_can_only_extend_an_object_type_or_intersection_of_object_types_with_statically_known_members:n(2312,e.DiagnosticCategory.Error,"An_interface_can_only_extend_an_object_type_or_intersection_of_object_types_with_statically_known_me_2312","An interface can only extend an object type or intersection of object types with statically known members."),Type_parameter_0_has_a_circular_constraint:n(2313,e.DiagnosticCategory.Error,"Type_parameter_0_has_a_circular_constraint_2313","Type parameter '{0}' has a circular constraint."),Generic_type_0_requires_1_type_argument_s:n(2314,e.DiagnosticCategory.Error,"Generic_type_0_requires_1_type_argument_s_2314","Generic type '{0}' requires {1} type argument(s)."),Type_0_is_not_generic:n(2315,e.DiagnosticCategory.Error,"Type_0_is_not_generic_2315","Type '{0}' is not generic."),Global_type_0_must_be_a_class_or_interface_type:n(2316,e.DiagnosticCategory.Error,"Global_type_0_must_be_a_class_or_interface_type_2316","Global type '{0}' must be a class or interface type."),Global_type_0_must_have_1_type_parameter_s:n(2317,e.DiagnosticCategory.Error,"Global_type_0_must_have_1_type_parameter_s_2317","Global type '{0}' must have {1} type parameter(s)."),Cannot_find_global_type_0:n(2318,e.DiagnosticCategory.Error,"Cannot_find_global_type_0_2318","Cannot find global type '{0}'."),Named_property_0_of_types_1_and_2_are_not_identical:n(2319,e.DiagnosticCategory.Error,"Named_property_0_of_types_1_and_2_are_not_identical_2319","Named property '{0}' of types '{1}' and '{2}' are not identical."),Interface_0_cannot_simultaneously_extend_types_1_and_2:n(2320,e.DiagnosticCategory.Error,"Interface_0_cannot_simultaneously_extend_types_1_and_2_2320","Interface '{0}' cannot simultaneously extend types '{1}' and '{2}'."),Excessive_stack_depth_comparing_types_0_and_1:n(2321,e.DiagnosticCategory.Error,"Excessive_stack_depth_comparing_types_0_and_1_2321","Excessive stack depth comparing types '{0}' and '{1}'."),Type_0_is_not_assignable_to_type_1:n(2322,e.DiagnosticCategory.Error,"Type_0_is_not_assignable_to_type_1_2322","Type '{0}' is not assignable to type '{1}'."),Cannot_redeclare_exported_variable_0:n(2323,e.DiagnosticCategory.Error,"Cannot_redeclare_exported_variable_0_2323","Cannot redeclare exported variable '{0}'."),Property_0_is_missing_in_type_1:n(2324,e.DiagnosticCategory.Error,"Property_0_is_missing_in_type_1_2324","Property '{0}' is missing in type '{1}'."),Property_0_is_private_in_type_1_but_not_in_type_2:n(2325,e.DiagnosticCategory.Error,"Property_0_is_private_in_type_1_but_not_in_type_2_2325","Property '{0}' is private in type '{1}' but not in type '{2}'."),Types_of_property_0_are_incompatible:n(2326,e.DiagnosticCategory.Error,"Types_of_property_0_are_incompatible_2326","Types of property '{0}' are incompatible."),Property_0_is_optional_in_type_1_but_required_in_type_2:n(2327,e.DiagnosticCategory.Error,"Property_0_is_optional_in_type_1_but_required_in_type_2_2327","Property '{0}' is optional in type '{1}' but required in type '{2}'."),Types_of_parameters_0_and_1_are_incompatible:n(2328,e.DiagnosticCategory.Error,"Types_of_parameters_0_and_1_are_incompatible_2328","Types of parameters '{0}' and '{1}' are incompatible."),Index_signature_for_type_0_is_missing_in_type_1:n(2329,e.DiagnosticCategory.Error,"Index_signature_for_type_0_is_missing_in_type_1_2329","Index signature for type '{0}' is missing in type '{1}'."),_0_and_1_index_signatures_are_incompatible:n(2330,e.DiagnosticCategory.Error,"_0_and_1_index_signatures_are_incompatible_2330","'{0}' and '{1}' index signatures are incompatible."),this_cannot_be_referenced_in_a_module_or_namespace_body:n(2331,e.DiagnosticCategory.Error,"this_cannot_be_referenced_in_a_module_or_namespace_body_2331","'this' cannot be referenced in a module or namespace body."),this_cannot_be_referenced_in_current_location:n(2332,e.DiagnosticCategory.Error,"this_cannot_be_referenced_in_current_location_2332","'this' cannot be referenced in current location."),this_cannot_be_referenced_in_constructor_arguments:n(2333,e.DiagnosticCategory.Error,"this_cannot_be_referenced_in_constructor_arguments_2333","'this' cannot be referenced in constructor arguments."),this_cannot_be_referenced_in_a_static_property_initializer:n(2334,e.DiagnosticCategory.Error,"this_cannot_be_referenced_in_a_static_property_initializer_2334","'this' cannot be referenced in a static property initializer."),super_can_only_be_referenced_in_a_derived_class:n(2335,e.DiagnosticCategory.Error,"super_can_only_be_referenced_in_a_derived_class_2335","'super' can only be referenced in a derived class."),super_cannot_be_referenced_in_constructor_arguments:n(2336,e.DiagnosticCategory.Error,"super_cannot_be_referenced_in_constructor_arguments_2336","'super' cannot be referenced in constructor arguments."),Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors:n(2337,e.DiagnosticCategory.Error,"Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors_2337","Super calls are not permitted outside constructors or in nested functions inside constructors."),super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_derived_class:n(2338,e.DiagnosticCategory.Error,"super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_der_2338","'super' property access is permitted only in a constructor, member function, or member accessor of a derived class."),Property_0_does_not_exist_on_type_1:n(2339,e.DiagnosticCategory.Error,"Property_0_does_not_exist_on_type_1_2339","Property '{0}' does not exist on type '{1}'."),Only_public_and_protected_methods_of_the_base_class_are_accessible_via_the_super_keyword:n(2340,e.DiagnosticCategory.Error,"Only_public_and_protected_methods_of_the_base_class_are_accessible_via_the_super_keyword_2340","Only public and protected methods of the base class are accessible via the 'super' keyword."),Property_0_is_private_and_only_accessible_within_class_1:n(2341,e.DiagnosticCategory.Error,"Property_0_is_private_and_only_accessible_within_class_1_2341","Property '{0}' is private and only accessible within class '{1}'."),An_index_expression_argument_must_be_of_type_string_number_symbol_or_any:n(2342,e.DiagnosticCategory.Error,"An_index_expression_argument_must_be_of_type_string_number_symbol_or_any_2342","An index expression argument must be of type 'string', 'number', 'symbol', or 'any'."),This_syntax_requires_an_imported_helper_named_1_which_does_not_exist_in_0_Consider_upgrading_your_version_of_0:n(2343,e.DiagnosticCategory.Error,"This_syntax_requires_an_imported_helper_named_1_which_does_not_exist_in_0_Consider_upgrading_your_ve_2343","This syntax requires an imported helper named '{1}' which does not exist in '{0}'. Consider upgrading your version of '{0}'."),Type_0_does_not_satisfy_the_constraint_1:n(2344,e.DiagnosticCategory.Error,"Type_0_does_not_satisfy_the_constraint_1_2344","Type '{0}' does not satisfy the constraint '{1}'."),Argument_of_type_0_is_not_assignable_to_parameter_of_type_1:n(2345,e.DiagnosticCategory.Error,"Argument_of_type_0_is_not_assignable_to_parameter_of_type_1_2345","Argument of type '{0}' is not assignable to parameter of type '{1}'."),Call_target_does_not_contain_any_signatures:n(2346,e.DiagnosticCategory.Error,"Call_target_does_not_contain_any_signatures_2346","Call target does not contain any signatures."),Untyped_function_calls_may_not_accept_type_arguments:n(2347,e.DiagnosticCategory.Error,"Untyped_function_calls_may_not_accept_type_arguments_2347","Untyped function calls may not accept type arguments."),Value_of_type_0_is_not_callable_Did_you_mean_to_include_new:n(2348,e.DiagnosticCategory.Error,"Value_of_type_0_is_not_callable_Did_you_mean_to_include_new_2348","Value of type '{0}' is not callable. Did you mean to include 'new'?"),This_expression_is_not_callable:n(2349,e.DiagnosticCategory.Error,"This_expression_is_not_callable_2349","This expression is not callable."),Only_a_void_function_can_be_called_with_the_new_keyword:n(2350,e.DiagnosticCategory.Error,"Only_a_void_function_can_be_called_with_the_new_keyword_2350","Only a void function can be called with the 'new' keyword."),This_expression_is_not_constructable:n(2351,e.DiagnosticCategory.Error,"This_expression_is_not_constructable_2351","This expression is not constructable."),Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the_other_If_this_was_intentional_convert_the_expression_to_unknown_first:n(2352,e.DiagnosticCategory.Error,"Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the__2352","Conversion of type '{0}' to type '{1}' may be a mistake because neither type sufficiently overlaps with the other. If this was intentional, convert the expression to 'unknown' first."),Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1:n(2353,e.DiagnosticCategory.Error,"Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1_2353","Object literal may only specify known properties, and '{0}' does not exist in type '{1}'."),This_syntax_requires_an_imported_helper_but_module_0_cannot_be_found:n(2354,e.DiagnosticCategory.Error,"This_syntax_requires_an_imported_helper_but_module_0_cannot_be_found_2354","This syntax requires an imported helper but module '{0}' cannot be found."),A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value:n(2355,e.DiagnosticCategory.Error,"A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value_2355","A function whose declared type is neither 'void' nor 'any' must return a value."),An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type:n(2356,e.DiagnosticCategory.Error,"An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type_2356","An arithmetic operand must be of type 'any', 'number', 'bigint' or an enum type."),The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access:n(2357,e.DiagnosticCategory.Error,"The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access_2357","The operand of an increment or decrement operator must be a variable or a property access."),The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_parameter:n(2358,e.DiagnosticCategory.Error,"The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_paramete_2358","The left-hand side of an 'instanceof' expression must be of type 'any', an object type or a type parameter."),The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_Function_interface_type:n(2359,e.DiagnosticCategory.Error,"The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_F_2359","The right-hand side of an 'instanceof' expression must be of type 'any' or of a type assignable to the 'Function' interface type."),The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol:n(2360,e.DiagnosticCategory.Error,"The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol_2360","The left-hand side of an 'in' expression must be of type 'any', 'string', 'number', or 'symbol'."),The_right_hand_side_of_an_in_expression_must_not_be_a_primitive:n(2361,e.DiagnosticCategory.Error,"The_right_hand_side_of_an_in_expression_must_not_be_a_primitive_2361","The right-hand side of an 'in' expression must not be a primitive."),The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type:n(2362,e.DiagnosticCategory.Error,"The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type_2362","The left-hand side of an arithmetic operation must be of type 'any', 'number', 'bigint' or an enum type."),The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type:n(2363,e.DiagnosticCategory.Error,"The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type_2363","The right-hand side of an arithmetic operation must be of type 'any', 'number', 'bigint' or an enum type."),The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access:n(2364,e.DiagnosticCategory.Error,"The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access_2364","The left-hand side of an assignment expression must be a variable or a property access."),Operator_0_cannot_be_applied_to_types_1_and_2:n(2365,e.DiagnosticCategory.Error,"Operator_0_cannot_be_applied_to_types_1_and_2_2365","Operator '{0}' cannot be applied to types '{1}' and '{2}'."),Function_lacks_ending_return_statement_and_return_type_does_not_include_undefined:n(2366,e.DiagnosticCategory.Error,"Function_lacks_ending_return_statement_and_return_type_does_not_include_undefined_2366","Function lacks ending return statement and return type does not include 'undefined'."),This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap:n(2367,e.DiagnosticCategory.Error,"This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap_2367","This condition will always return '{0}' since the types '{1}' and '{2}' have no overlap."),Type_parameter_name_cannot_be_0:n(2368,e.DiagnosticCategory.Error,"Type_parameter_name_cannot_be_0_2368","Type parameter name cannot be '{0}'."),A_parameter_property_is_only_allowed_in_a_constructor_implementation:n(2369,e.DiagnosticCategory.Error,"A_parameter_property_is_only_allowed_in_a_constructor_implementation_2369","A parameter property is only allowed in a constructor implementation."),A_rest_parameter_must_be_of_an_array_type:n(2370,e.DiagnosticCategory.Error,"A_rest_parameter_must_be_of_an_array_type_2370","A rest parameter must be of an array type."),A_parameter_initializer_is_only_allowed_in_a_function_or_constructor_implementation:n(2371,e.DiagnosticCategory.Error,"A_parameter_initializer_is_only_allowed_in_a_function_or_constructor_implementation_2371","A parameter initializer is only allowed in a function or constructor implementation."),Parameter_0_cannot_reference_itself:n(2372,e.DiagnosticCategory.Error,"Parameter_0_cannot_reference_itself_2372","Parameter '{0}' cannot reference itself."),Parameter_0_cannot_reference_identifier_1_declared_after_it:n(2373,e.DiagnosticCategory.Error,"Parameter_0_cannot_reference_identifier_1_declared_after_it_2373","Parameter '{0}' cannot reference identifier '{1}' declared after it."),Duplicate_index_signature_for_type_0:n(2374,e.DiagnosticCategory.Error,"Duplicate_index_signature_for_type_0_2374","Duplicate index signature for type '{0}'."),A_super_call_must_be_the_first_statement_in_the_constructor_when_a_class_contains_initialized_properties_parameter_properties_or_private_identifiers:n(2376,e.DiagnosticCategory.Error,"A_super_call_must_be_the_first_statement_in_the_constructor_when_a_class_contains_initialized_proper_2376","A 'super' call must be the first statement in the constructor when a class contains initialized properties, parameter properties, or private identifiers."),Constructors_for_derived_classes_must_contain_a_super_call:n(2377,e.DiagnosticCategory.Error,"Constructors_for_derived_classes_must_contain_a_super_call_2377","Constructors for derived classes must contain a 'super' call."),A_get_accessor_must_return_a_value:n(2378,e.DiagnosticCategory.Error,"A_get_accessor_must_return_a_value_2378","A 'get' accessor must return a value."),The_return_type_of_a_get_accessor_must_be_assignable_to_its_set_accessor_type:n(2380,e.DiagnosticCategory.Error,"The_return_type_of_a_get_accessor_must_be_assignable_to_its_set_accessor_type_2380","The return type of a 'get' accessor must be assignable to its 'set' accessor type"),A_signature_with_an_implementation_cannot_use_a_string_literal_type:n(2381,e.DiagnosticCategory.Error,"A_signature_with_an_implementation_cannot_use_a_string_literal_type_2381","A signature with an implementation cannot use a string literal type."),Specialized_overload_signature_is_not_assignable_to_any_non_specialized_signature:n(2382,e.DiagnosticCategory.Error,"Specialized_overload_signature_is_not_assignable_to_any_non_specialized_signature_2382","Specialized overload signature is not assignable to any non-specialized signature."),Overload_signatures_must_all_be_exported_or_non_exported:n(2383,e.DiagnosticCategory.Error,"Overload_signatures_must_all_be_exported_or_non_exported_2383","Overload signatures must all be exported or non-exported."),Overload_signatures_must_all_be_ambient_or_non_ambient:n(2384,e.DiagnosticCategory.Error,"Overload_signatures_must_all_be_ambient_or_non_ambient_2384","Overload signatures must all be ambient or non-ambient."),Overload_signatures_must_all_be_public_private_or_protected:n(2385,e.DiagnosticCategory.Error,"Overload_signatures_must_all_be_public_private_or_protected_2385","Overload signatures must all be public, private or protected."),Overload_signatures_must_all_be_optional_or_required:n(2386,e.DiagnosticCategory.Error,"Overload_signatures_must_all_be_optional_or_required_2386","Overload signatures must all be optional or required."),Function_overload_must_be_static:n(2387,e.DiagnosticCategory.Error,"Function_overload_must_be_static_2387","Function overload must be static."),Function_overload_must_not_be_static:n(2388,e.DiagnosticCategory.Error,"Function_overload_must_not_be_static_2388","Function overload must not be static."),Function_implementation_name_must_be_0:n(2389,e.DiagnosticCategory.Error,"Function_implementation_name_must_be_0_2389","Function implementation name must be '{0}'."),Constructor_implementation_is_missing:n(2390,e.DiagnosticCategory.Error,"Constructor_implementation_is_missing_2390","Constructor implementation is missing."),Function_implementation_is_missing_or_not_immediately_following_the_declaration:n(2391,e.DiagnosticCategory.Error,"Function_implementation_is_missing_or_not_immediately_following_the_declaration_2391","Function implementation is missing or not immediately following the declaration."),Multiple_constructor_implementations_are_not_allowed:n(2392,e.DiagnosticCategory.Error,"Multiple_constructor_implementations_are_not_allowed_2392","Multiple constructor implementations are not allowed."),Duplicate_function_implementation:n(2393,e.DiagnosticCategory.Error,"Duplicate_function_implementation_2393","Duplicate function implementation."),This_overload_signature_is_not_compatible_with_its_implementation_signature:n(2394,e.DiagnosticCategory.Error,"This_overload_signature_is_not_compatible_with_its_implementation_signature_2394","This overload signature is not compatible with its implementation signature."),Individual_declarations_in_merged_declaration_0_must_be_all_exported_or_all_local:n(2395,e.DiagnosticCategory.Error,"Individual_declarations_in_merged_declaration_0_must_be_all_exported_or_all_local_2395","Individual declarations in merged declaration '{0}' must be all exported or all local."),Duplicate_identifier_arguments_Compiler_uses_arguments_to_initialize_rest_parameters:n(2396,e.DiagnosticCategory.Error,"Duplicate_identifier_arguments_Compiler_uses_arguments_to_initialize_rest_parameters_2396","Duplicate identifier 'arguments'. Compiler uses 'arguments' to initialize rest parameters."),Declaration_name_conflicts_with_built_in_global_identifier_0:n(2397,e.DiagnosticCategory.Error,"Declaration_name_conflicts_with_built_in_global_identifier_0_2397","Declaration name conflicts with built-in global identifier '{0}'."),constructor_cannot_be_used_as_a_parameter_property_name:n(2398,e.DiagnosticCategory.Error,"constructor_cannot_be_used_as_a_parameter_property_name_2398","'constructor' cannot be used as a parameter property name."),Duplicate_identifier_this_Compiler_uses_variable_declaration_this_to_capture_this_reference:n(2399,e.DiagnosticCategory.Error,"Duplicate_identifier_this_Compiler_uses_variable_declaration_this_to_capture_this_reference_2399","Duplicate identifier '_this'. Compiler uses variable declaration '_this' to capture 'this' reference."),Expression_resolves_to_variable_declaration_this_that_compiler_uses_to_capture_this_reference:n(2400,e.DiagnosticCategory.Error,"Expression_resolves_to_variable_declaration_this_that_compiler_uses_to_capture_this_reference_2400","Expression resolves to variable declaration '_this' that compiler uses to capture 'this' reference."),Duplicate_identifier_super_Compiler_uses_super_to_capture_base_class_reference:n(2401,e.DiagnosticCategory.Error,"Duplicate_identifier_super_Compiler_uses_super_to_capture_base_class_reference_2401","Duplicate identifier '_super'. Compiler uses '_super' to capture base class reference."),Expression_resolves_to_super_that_compiler_uses_to_capture_base_class_reference:n(2402,e.DiagnosticCategory.Error,"Expression_resolves_to_super_that_compiler_uses_to_capture_base_class_reference_2402","Expression resolves to '_super' that compiler uses to capture base class reference."),Subsequent_variable_declarations_must_have_the_same_type_Variable_0_must_be_of_type_1_but_here_has_type_2:n(2403,e.DiagnosticCategory.Error,"Subsequent_variable_declarations_must_have_the_same_type_Variable_0_must_be_of_type_1_but_here_has_t_2403","Subsequent variable declarations must have the same type.  Variable '{0}' must be of type '{1}', but here has type '{2}'."),The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation:n(2404,e.DiagnosticCategory.Error,"The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation_2404","The left-hand side of a 'for...in' statement cannot use a type annotation."),The_left_hand_side_of_a_for_in_statement_must_be_of_type_string_or_any:n(2405,e.DiagnosticCategory.Error,"The_left_hand_side_of_a_for_in_statement_must_be_of_type_string_or_any_2405","The left-hand side of a 'for...in' statement must be of type 'string' or 'any'."),The_left_hand_side_of_a_for_in_statement_must_be_a_variable_or_a_property_access:n(2406,e.DiagnosticCategory.Error,"The_left_hand_side_of_a_for_in_statement_must_be_a_variable_or_a_property_access_2406","The left-hand side of a 'for...in' statement must be a variable or a property access."),The_right_hand_side_of_a_for_in_statement_must_be_of_type_any_an_object_type_or_a_type_parameter_but_here_has_type_0:n(2407,e.DiagnosticCategory.Error,"The_right_hand_side_of_a_for_in_statement_must_be_of_type_any_an_object_type_or_a_type_parameter_but_2407","The right-hand side of a 'for...in' statement must be of type 'any', an object type or a type parameter, but here has type '{0}'."),Setters_cannot_return_a_value:n(2408,e.DiagnosticCategory.Error,"Setters_cannot_return_a_value_2408","Setters cannot return a value."),Return_type_of_constructor_signature_must_be_assignable_to_the_instance_type_of_the_class:n(2409,e.DiagnosticCategory.Error,"Return_type_of_constructor_signature_must_be_assignable_to_the_instance_type_of_the_class_2409","Return type of constructor signature must be assignable to the instance type of the class."),The_with_statement_is_not_supported_All_symbols_in_a_with_block_will_have_type_any:n(2410,e.DiagnosticCategory.Error,"The_with_statement_is_not_supported_All_symbols_in_a_with_block_will_have_type_any_2410","The 'with' statement is not supported. All symbols in a 'with' block will have type 'any'."),Property_0_of_type_1_is_not_assignable_to_2_index_type_3:n(2411,e.DiagnosticCategory.Error,"Property_0_of_type_1_is_not_assignable_to_2_index_type_3_2411","Property '{0}' of type '{1}' is not assignable to '{2}' index type '{3}'."),_0_index_type_1_is_not_assignable_to_2_index_type_3:n(2413,e.DiagnosticCategory.Error,"_0_index_type_1_is_not_assignable_to_2_index_type_3_2413","'{0}' index type '{1}' is not assignable to '{2}' index type '{3}'."),Class_name_cannot_be_0:n(2414,e.DiagnosticCategory.Error,"Class_name_cannot_be_0_2414","Class name cannot be '{0}'."),Class_0_incorrectly_extends_base_class_1:n(2415,e.DiagnosticCategory.Error,"Class_0_incorrectly_extends_base_class_1_2415","Class '{0}' incorrectly extends base class '{1}'."),Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2:n(2416,e.DiagnosticCategory.Error,"Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2_2416","Property '{0}' in type '{1}' is not assignable to the same property in base type '{2}'."),Class_static_side_0_incorrectly_extends_base_class_static_side_1:n(2417,e.DiagnosticCategory.Error,"Class_static_side_0_incorrectly_extends_base_class_static_side_1_2417","Class static side '{0}' incorrectly extends base class static side '{1}'."),Type_of_computed_property_s_value_is_0_which_is_not_assignable_to_type_1:n(2418,e.DiagnosticCategory.Error,"Type_of_computed_property_s_value_is_0_which_is_not_assignable_to_type_1_2418","Type of computed property's value is '{0}', which is not assignable to type '{1}'."),Types_of_construct_signatures_are_incompatible:n(2419,e.DiagnosticCategory.Error,"Types_of_construct_signatures_are_incompatible_2419","Types of construct signatures are incompatible."),Class_0_incorrectly_implements_interface_1:n(2420,e.DiagnosticCategory.Error,"Class_0_incorrectly_implements_interface_1_2420","Class '{0}' incorrectly implements interface '{1}'."),A_class_can_only_implement_an_object_type_or_intersection_of_object_types_with_statically_known_members:n(2422,e.DiagnosticCategory.Error,"A_class_can_only_implement_an_object_type_or_intersection_of_object_types_with_statically_known_memb_2422","A class can only implement an object type or intersection of object types with statically known members."),Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_accessor:n(2423,e.DiagnosticCategory.Error,"Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_access_2423","Class '{0}' defines instance member function '{1}', but extended class '{2}' defines it as instance member accessor."),Class_0_defines_instance_member_property_1_but_extended_class_2_defines_it_as_instance_member_function:n(2425,e.DiagnosticCategory.Error,"Class_0_defines_instance_member_property_1_but_extended_class_2_defines_it_as_instance_member_functi_2425","Class '{0}' defines instance member property '{1}', but extended class '{2}' defines it as instance member function."),Class_0_defines_instance_member_accessor_1_but_extended_class_2_defines_it_as_instance_member_function:n(2426,e.DiagnosticCategory.Error,"Class_0_defines_instance_member_accessor_1_but_extended_class_2_defines_it_as_instance_member_functi_2426","Class '{0}' defines instance member accessor '{1}', but extended class '{2}' defines it as instance member function."),Interface_name_cannot_be_0:n(2427,e.DiagnosticCategory.Error,"Interface_name_cannot_be_0_2427","Interface name cannot be '{0}'."),All_declarations_of_0_must_have_identical_type_parameters:n(2428,e.DiagnosticCategory.Error,"All_declarations_of_0_must_have_identical_type_parameters_2428","All declarations of '{0}' must have identical type parameters."),Interface_0_incorrectly_extends_interface_1:n(2430,e.DiagnosticCategory.Error,"Interface_0_incorrectly_extends_interface_1_2430","Interface '{0}' incorrectly extends interface '{1}'."),Enum_name_cannot_be_0:n(2431,e.DiagnosticCategory.Error,"Enum_name_cannot_be_0_2431","Enum name cannot be '{0}'."),In_an_enum_with_multiple_declarations_only_one_declaration_can_omit_an_initializer_for_its_first_enum_element:n(2432,e.DiagnosticCategory.Error,"In_an_enum_with_multiple_declarations_only_one_declaration_can_omit_an_initializer_for_its_first_enu_2432","In an enum with multiple declarations, only one declaration can omit an initializer for its first enum element."),A_namespace_declaration_cannot_be_in_a_different_file_from_a_class_or_function_with_which_it_is_merged:n(2433,e.DiagnosticCategory.Error,"A_namespace_declaration_cannot_be_in_a_different_file_from_a_class_or_function_with_which_it_is_merg_2433","A namespace declaration cannot be in a different file from a class or function with which it is merged."),A_namespace_declaration_cannot_be_located_prior_to_a_class_or_function_with_which_it_is_merged:n(2434,e.DiagnosticCategory.Error,"A_namespace_declaration_cannot_be_located_prior_to_a_class_or_function_with_which_it_is_merged_2434","A namespace declaration cannot be located prior to a class or function with which it is merged."),Ambient_modules_cannot_be_nested_in_other_modules_or_namespaces:n(2435,e.DiagnosticCategory.Error,"Ambient_modules_cannot_be_nested_in_other_modules_or_namespaces_2435","Ambient modules cannot be nested in other modules or namespaces."),Ambient_module_declaration_cannot_specify_relative_module_name:n(2436,e.DiagnosticCategory.Error,"Ambient_module_declaration_cannot_specify_relative_module_name_2436","Ambient module declaration cannot specify relative module name."),Module_0_is_hidden_by_a_local_declaration_with_the_same_name:n(2437,e.DiagnosticCategory.Error,"Module_0_is_hidden_by_a_local_declaration_with_the_same_name_2437","Module '{0}' is hidden by a local declaration with the same name."),Import_name_cannot_be_0:n(2438,e.DiagnosticCategory.Error,"Import_name_cannot_be_0_2438","Import name cannot be '{0}'."),Import_or_export_declaration_in_an_ambient_module_declaration_cannot_reference_module_through_relative_module_name:n(2439,e.DiagnosticCategory.Error,"Import_or_export_declaration_in_an_ambient_module_declaration_cannot_reference_module_through_relati_2439","Import or export declaration in an ambient module declaration cannot reference module through relative module name."),Import_declaration_conflicts_with_local_declaration_of_0:n(2440,e.DiagnosticCategory.Error,"Import_declaration_conflicts_with_local_declaration_of_0_2440","Import declaration conflicts with local declaration of '{0}'."),Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module:n(2441,e.DiagnosticCategory.Error,"Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_2441","Duplicate identifier '{0}'. Compiler reserves name '{1}' in top level scope of a module."),Types_have_separate_declarations_of_a_private_property_0:n(2442,e.DiagnosticCategory.Error,"Types_have_separate_declarations_of_a_private_property_0_2442","Types have separate declarations of a private property '{0}'."),Property_0_is_protected_but_type_1_is_not_a_class_derived_from_2:n(2443,e.DiagnosticCategory.Error,"Property_0_is_protected_but_type_1_is_not_a_class_derived_from_2_2443","Property '{0}' is protected but type '{1}' is not a class derived from '{2}'."),Property_0_is_protected_in_type_1_but_public_in_type_2:n(2444,e.DiagnosticCategory.Error,"Property_0_is_protected_in_type_1_but_public_in_type_2_2444","Property '{0}' is protected in type '{1}' but public in type '{2}'."),Property_0_is_protected_and_only_accessible_within_class_1_and_its_subclasses:n(2445,e.DiagnosticCategory.Error,"Property_0_is_protected_and_only_accessible_within_class_1_and_its_subclasses_2445","Property '{0}' is protected and only accessible within class '{1}' and its subclasses."),Property_0_is_protected_and_only_accessible_through_an_instance_of_class_1_This_is_an_instance_of_class_2:n(2446,e.DiagnosticCategory.Error,"Property_0_is_protected_and_only_accessible_through_an_instance_of_class_1_This_is_an_instance_of_cl_2446","Property '{0}' is protected and only accessible through an instance of class '{1}'. This is an instance of class '{2}'."),The_0_operator_is_not_allowed_for_boolean_types_Consider_using_1_instead:n(2447,e.DiagnosticCategory.Error,"The_0_operator_is_not_allowed_for_boolean_types_Consider_using_1_instead_2447","The '{0}' operator is not allowed for boolean types. Consider using '{1}' instead."),Block_scoped_variable_0_used_before_its_declaration:n(2448,e.DiagnosticCategory.Error,"Block_scoped_variable_0_used_before_its_declaration_2448","Block-scoped variable '{0}' used before its declaration."),Class_0_used_before_its_declaration:n(2449,e.DiagnosticCategory.Error,"Class_0_used_before_its_declaration_2449","Class '{0}' used before its declaration."),Enum_0_used_before_its_declaration:n(2450,e.DiagnosticCategory.Error,"Enum_0_used_before_its_declaration_2450","Enum '{0}' used before its declaration."),Cannot_redeclare_block_scoped_variable_0:n(2451,e.DiagnosticCategory.Error,"Cannot_redeclare_block_scoped_variable_0_2451","Cannot redeclare block-scoped variable '{0}'."),An_enum_member_cannot_have_a_numeric_name:n(2452,e.DiagnosticCategory.Error,"An_enum_member_cannot_have_a_numeric_name_2452","An enum member cannot have a numeric name."),The_type_argument_for_type_parameter_0_cannot_be_inferred_from_the_usage_Consider_specifying_the_type_arguments_explicitly:n(2453,e.DiagnosticCategory.Error,"The_type_argument_for_type_parameter_0_cannot_be_inferred_from_the_usage_Consider_specifying_the_typ_2453","The type argument for type parameter '{0}' cannot be inferred from the usage. Consider specifying the type arguments explicitly."),Variable_0_is_used_before_being_assigned:n(2454,e.DiagnosticCategory.Error,"Variable_0_is_used_before_being_assigned_2454","Variable '{0}' is used before being assigned."),Type_argument_candidate_1_is_not_a_valid_type_argument_because_it_is_not_a_supertype_of_candidate_0:n(2455,e.DiagnosticCategory.Error,"Type_argument_candidate_1_is_not_a_valid_type_argument_because_it_is_not_a_supertype_of_candidate_0_2455","Type argument candidate '{1}' is not a valid type argument because it is not a supertype of candidate '{0}'."),Type_alias_0_circularly_references_itself:n(2456,e.DiagnosticCategory.Error,"Type_alias_0_circularly_references_itself_2456","Type alias '{0}' circularly references itself."),Type_alias_name_cannot_be_0:n(2457,e.DiagnosticCategory.Error,"Type_alias_name_cannot_be_0_2457","Type alias name cannot be '{0}'."),An_AMD_module_cannot_have_multiple_name_assignments:n(2458,e.DiagnosticCategory.Error,"An_AMD_module_cannot_have_multiple_name_assignments_2458","An AMD module cannot have multiple name assignments."),Module_0_declares_1_locally_but_it_is_not_exported:n(2459,e.DiagnosticCategory.Error,"Module_0_declares_1_locally_but_it_is_not_exported_2459","Module '{0}' declares '{1}' locally, but it is not exported."),Module_0_declares_1_locally_but_it_is_exported_as_2:n(2460,e.DiagnosticCategory.Error,"Module_0_declares_1_locally_but_it_is_exported_as_2_2460","Module '{0}' declares '{1}' locally, but it is exported as '{2}'."),Type_0_is_not_an_array_type:n(2461,e.DiagnosticCategory.Error,"Type_0_is_not_an_array_type_2461","Type '{0}' is not an array type."),A_rest_element_must_be_last_in_a_destructuring_pattern:n(2462,e.DiagnosticCategory.Error,"A_rest_element_must_be_last_in_a_destructuring_pattern_2462","A rest element must be last in a destructuring pattern."),A_binding_pattern_parameter_cannot_be_optional_in_an_implementation_signature:n(2463,e.DiagnosticCategory.Error,"A_binding_pattern_parameter_cannot_be_optional_in_an_implementation_signature_2463","A binding pattern parameter cannot be optional in an implementation signature."),A_computed_property_name_must_be_of_type_string_number_symbol_or_any:n(2464,e.DiagnosticCategory.Error,"A_computed_property_name_must_be_of_type_string_number_symbol_or_any_2464","A computed property name must be of type 'string', 'number', 'symbol', or 'any'."),this_cannot_be_referenced_in_a_computed_property_name:n(2465,e.DiagnosticCategory.Error,"this_cannot_be_referenced_in_a_computed_property_name_2465","'this' cannot be referenced in a computed property name."),super_cannot_be_referenced_in_a_computed_property_name:n(2466,e.DiagnosticCategory.Error,"super_cannot_be_referenced_in_a_computed_property_name_2466","'super' cannot be referenced in a computed property name."),A_computed_property_name_cannot_reference_a_type_parameter_from_its_containing_type:n(2467,e.DiagnosticCategory.Error,"A_computed_property_name_cannot_reference_a_type_parameter_from_its_containing_type_2467","A computed property name cannot reference a type parameter from its containing type."),Cannot_find_global_value_0:n(2468,e.DiagnosticCategory.Error,"Cannot_find_global_value_0_2468","Cannot find global value '{0}'."),The_0_operator_cannot_be_applied_to_type_symbol:n(2469,e.DiagnosticCategory.Error,"The_0_operator_cannot_be_applied_to_type_symbol_2469","The '{0}' operator cannot be applied to type 'symbol'."),Symbol_reference_does_not_refer_to_the_global_Symbol_constructor_object:n(2470,e.DiagnosticCategory.Error,"Symbol_reference_does_not_refer_to_the_global_Symbol_constructor_object_2470","'Symbol' reference does not refer to the global Symbol constructor object."),A_computed_property_name_of_the_form_0_must_be_of_type_symbol:n(2471,e.DiagnosticCategory.Error,"A_computed_property_name_of_the_form_0_must_be_of_type_symbol_2471","A computed property name of the form '{0}' must be of type 'symbol'."),Spread_operator_in_new_expressions_is_only_available_when_targeting_ECMAScript_5_and_higher:n(2472,e.DiagnosticCategory.Error,"Spread_operator_in_new_expressions_is_only_available_when_targeting_ECMAScript_5_and_higher_2472","Spread operator in 'new' expressions is only available when targeting ECMAScript 5 and higher."),Enum_declarations_must_all_be_const_or_non_const:n(2473,e.DiagnosticCategory.Error,"Enum_declarations_must_all_be_const_or_non_const_2473","Enum declarations must all be const or non-const."),const_enum_member_initializers_can_only_contain_literal_values_and_other_computed_enum_values:n(2474,e.DiagnosticCategory.Error,"const_enum_member_initializers_can_only_contain_literal_values_and_other_computed_enum_values_2474","const enum member initializers can only contain literal values and other computed enum values."),const_enums_can_only_be_used_in_property_or_index_access_expressions_or_the_right_hand_side_of_an_import_declaration_or_export_assignment_or_type_query:n(2475,e.DiagnosticCategory.Error,"const_enums_can_only_be_used_in_property_or_index_access_expressions_or_the_right_hand_side_of_an_im_2475","'const' enums can only be used in property or index access expressions or the right hand side of an import declaration or export assignment or type query."),A_const_enum_member_can_only_be_accessed_using_a_string_literal:n(2476,e.DiagnosticCategory.Error,"A_const_enum_member_can_only_be_accessed_using_a_string_literal_2476","A const enum member can only be accessed using a string literal."),const_enum_member_initializer_was_evaluated_to_a_non_finite_value:n(2477,e.DiagnosticCategory.Error,"const_enum_member_initializer_was_evaluated_to_a_non_finite_value_2477","'const' enum member initializer was evaluated to a non-finite value."),const_enum_member_initializer_was_evaluated_to_disallowed_value_NaN:n(2478,e.DiagnosticCategory.Error,"const_enum_member_initializer_was_evaluated_to_disallowed_value_NaN_2478","'const' enum member initializer was evaluated to disallowed value 'NaN'."),Property_0_does_not_exist_on_const_enum_1:n(2479,e.DiagnosticCategory.Error,"Property_0_does_not_exist_on_const_enum_1_2479","Property '{0}' does not exist on 'const' enum '{1}'."),let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations:n(2480,e.DiagnosticCategory.Error,"let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations_2480","'let' is not allowed to be used as a name in 'let' or 'const' declarations."),Cannot_initialize_outer_scoped_variable_0_in_the_same_scope_as_block_scoped_declaration_1:n(2481,e.DiagnosticCategory.Error,"Cannot_initialize_outer_scoped_variable_0_in_the_same_scope_as_block_scoped_declaration_1_2481","Cannot initialize outer scoped variable '{0}' in the same scope as block scoped declaration '{1}'."),The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation:n(2483,e.DiagnosticCategory.Error,"The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation_2483","The left-hand side of a 'for...of' statement cannot use a type annotation."),Export_declaration_conflicts_with_exported_declaration_of_0:n(2484,e.DiagnosticCategory.Error,"Export_declaration_conflicts_with_exported_declaration_of_0_2484","Export declaration conflicts with exported declaration of '{0}'."),The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access:n(2487,e.DiagnosticCategory.Error,"The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access_2487","The left-hand side of a 'for...of' statement must be a variable or a property access."),Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator:n(2488,e.DiagnosticCategory.Error,"Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator_2488","Type '{0}' must have a '[Symbol.iterator]()' method that returns an iterator."),An_iterator_must_have_a_next_method:n(2489,e.DiagnosticCategory.Error,"An_iterator_must_have_a_next_method_2489","An iterator must have a 'next()' method."),The_type_returned_by_the_0_method_of_an_iterator_must_have_a_value_property:n(2490,e.DiagnosticCategory.Error,"The_type_returned_by_the_0_method_of_an_iterator_must_have_a_value_property_2490","The type returned by the '{0}()' method of an iterator must have a 'value' property."),The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern:n(2491,e.DiagnosticCategory.Error,"The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern_2491","The left-hand side of a 'for...in' statement cannot be a destructuring pattern."),Cannot_redeclare_identifier_0_in_catch_clause:n(2492,e.DiagnosticCategory.Error,"Cannot_redeclare_identifier_0_in_catch_clause_2492","Cannot redeclare identifier '{0}' in catch clause."),Tuple_type_0_of_length_1_has_no_element_at_index_2:n(2493,e.DiagnosticCategory.Error,"Tuple_type_0_of_length_1_has_no_element_at_index_2_2493","Tuple type '{0}' of length '{1}' has no element at index '{2}'."),Using_a_string_in_a_for_of_statement_is_only_supported_in_ECMAScript_5_and_higher:n(2494,e.DiagnosticCategory.Error,"Using_a_string_in_a_for_of_statement_is_only_supported_in_ECMAScript_5_and_higher_2494","Using a string in a 'for...of' statement is only supported in ECMAScript 5 and higher."),Type_0_is_not_an_array_type_or_a_string_type:n(2495,e.DiagnosticCategory.Error,"Type_0_is_not_an_array_type_or_a_string_type_2495","Type '{0}' is not an array type or a string type."),The_arguments_object_cannot_be_referenced_in_an_arrow_function_in_ES3_and_ES5_Consider_using_a_standard_function_expression:n(2496,e.DiagnosticCategory.Error,"The_arguments_object_cannot_be_referenced_in_an_arrow_function_in_ES3_and_ES5_Consider_using_a_stand_2496","The 'arguments' object cannot be referenced in an arrow function in ES3 and ES5. Consider using a standard function expression."),This_module_can_only_be_referenced_with_ECMAScript_imports_Slashexports_by_turning_on_the_0_flag_and_referencing_its_default_export:n(2497,e.DiagnosticCategory.Error,"This_module_can_only_be_referenced_with_ECMAScript_imports_Slashexports_by_turning_on_the_0_flag_and_2497","This module can only be referenced with ECMAScript imports/exports by turning on the '{0}' flag and referencing its default export."),Module_0_uses_export_and_cannot_be_used_with_export_Asterisk:n(2498,e.DiagnosticCategory.Error,"Module_0_uses_export_and_cannot_be_used_with_export_Asterisk_2498","Module '{0}' uses 'export =' and cannot be used with 'export *'."),An_interface_can_only_extend_an_identifier_Slashqualified_name_with_optional_type_arguments:n(2499,e.DiagnosticCategory.Error,"An_interface_can_only_extend_an_identifier_Slashqualified_name_with_optional_type_arguments_2499","An interface can only extend an identifier/qualified-name with optional type arguments."),A_class_can_only_implement_an_identifier_Slashqualified_name_with_optional_type_arguments:n(2500,e.DiagnosticCategory.Error,"A_class_can_only_implement_an_identifier_Slashqualified_name_with_optional_type_arguments_2500","A class can only implement an identifier/qualified-name with optional type arguments."),A_rest_element_cannot_contain_a_binding_pattern:n(2501,e.DiagnosticCategory.Error,"A_rest_element_cannot_contain_a_binding_pattern_2501","A rest element cannot contain a binding pattern."),_0_is_referenced_directly_or_indirectly_in_its_own_type_annotation:n(2502,e.DiagnosticCategory.Error,"_0_is_referenced_directly_or_indirectly_in_its_own_type_annotation_2502","'{0}' is referenced directly or indirectly in its own type annotation."),Cannot_find_namespace_0:n(2503,e.DiagnosticCategory.Error,"Cannot_find_namespace_0_2503","Cannot find namespace '{0}'."),Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator:n(2504,e.DiagnosticCategory.Error,"Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator_2504","Type '{0}' must have a '[Symbol.asyncIterator]()' method that returns an async iterator."),A_generator_cannot_have_a_void_type_annotation:n(2505,e.DiagnosticCategory.Error,"A_generator_cannot_have_a_void_type_annotation_2505","A generator cannot have a 'void' type annotation."),_0_is_referenced_directly_or_indirectly_in_its_own_base_expression:n(2506,e.DiagnosticCategory.Error,"_0_is_referenced_directly_or_indirectly_in_its_own_base_expression_2506","'{0}' is referenced directly or indirectly in its own base expression."),Type_0_is_not_a_constructor_function_type:n(2507,e.DiagnosticCategory.Error,"Type_0_is_not_a_constructor_function_type_2507","Type '{0}' is not a constructor function type."),No_base_constructor_has_the_specified_number_of_type_arguments:n(2508,e.DiagnosticCategory.Error,"No_base_constructor_has_the_specified_number_of_type_arguments_2508","No base constructor has the specified number of type arguments."),Base_constructor_return_type_0_is_not_an_object_type_or_intersection_of_object_types_with_statically_known_members:n(2509,e.DiagnosticCategory.Error,"Base_constructor_return_type_0_is_not_an_object_type_or_intersection_of_object_types_with_statically_2509","Base constructor return type '{0}' is not an object type or intersection of object types with statically known members."),Base_constructors_must_all_have_the_same_return_type:n(2510,e.DiagnosticCategory.Error,"Base_constructors_must_all_have_the_same_return_type_2510","Base constructors must all have the same return type."),Cannot_create_an_instance_of_an_abstract_class:n(2511,e.DiagnosticCategory.Error,"Cannot_create_an_instance_of_an_abstract_class_2511","Cannot create an instance of an abstract class."),Overload_signatures_must_all_be_abstract_or_non_abstract:n(2512,e.DiagnosticCategory.Error,"Overload_signatures_must_all_be_abstract_or_non_abstract_2512","Overload signatures must all be abstract or non-abstract."),Abstract_method_0_in_class_1_cannot_be_accessed_via_super_expression:n(2513,e.DiagnosticCategory.Error,"Abstract_method_0_in_class_1_cannot_be_accessed_via_super_expression_2513","Abstract method '{0}' in class '{1}' cannot be accessed via super expression."),Classes_containing_abstract_methods_must_be_marked_abstract:n(2514,e.DiagnosticCategory.Error,"Classes_containing_abstract_methods_must_be_marked_abstract_2514","Classes containing abstract methods must be marked abstract."),Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2:n(2515,e.DiagnosticCategory.Error,"Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2_2515","Non-abstract class '{0}' does not implement inherited abstract member '{1}' from class '{2}'."),All_declarations_of_an_abstract_method_must_be_consecutive:n(2516,e.DiagnosticCategory.Error,"All_declarations_of_an_abstract_method_must_be_consecutive_2516","All declarations of an abstract method must be consecutive."),Cannot_assign_an_abstract_constructor_type_to_a_non_abstract_constructor_type:n(2517,e.DiagnosticCategory.Error,"Cannot_assign_an_abstract_constructor_type_to_a_non_abstract_constructor_type_2517","Cannot assign an abstract constructor type to a non-abstract constructor type."),A_this_based_type_guard_is_not_compatible_with_a_parameter_based_type_guard:n(2518,e.DiagnosticCategory.Error,"A_this_based_type_guard_is_not_compatible_with_a_parameter_based_type_guard_2518","A 'this'-based type guard is not compatible with a parameter-based type guard."),An_async_iterator_must_have_a_next_method:n(2519,e.DiagnosticCategory.Error,"An_async_iterator_must_have_a_next_method_2519","An async iterator must have a 'next()' method."),Duplicate_identifier_0_Compiler_uses_declaration_1_to_support_async_functions:n(2520,e.DiagnosticCategory.Error,"Duplicate_identifier_0_Compiler_uses_declaration_1_to_support_async_functions_2520","Duplicate identifier '{0}'. Compiler uses declaration '{1}' to support async functions."),Expression_resolves_to_variable_declaration_0_that_compiler_uses_to_support_async_functions:n(2521,e.DiagnosticCategory.Error,"Expression_resolves_to_variable_declaration_0_that_compiler_uses_to_support_async_functions_2521","Expression resolves to variable declaration '{0}' that compiler uses to support async functions."),The_arguments_object_cannot_be_referenced_in_an_async_function_or_method_in_ES3_and_ES5_Consider_using_a_standard_function_or_method:n(2522,e.DiagnosticCategory.Error,"The_arguments_object_cannot_be_referenced_in_an_async_function_or_method_in_ES3_and_ES5_Consider_usi_2522","The 'arguments' object cannot be referenced in an async function or method in ES3 and ES5. Consider using a standard function or method."),yield_expressions_cannot_be_used_in_a_parameter_initializer:n(2523,e.DiagnosticCategory.Error,"yield_expressions_cannot_be_used_in_a_parameter_initializer_2523","'yield' expressions cannot be used in a parameter initializer."),await_expressions_cannot_be_used_in_a_parameter_initializer:n(2524,e.DiagnosticCategory.Error,"await_expressions_cannot_be_used_in_a_parameter_initializer_2524","'await' expressions cannot be used in a parameter initializer."),Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value:n(2525,e.DiagnosticCategory.Error,"Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value_2525","Initializer provides no value for this binding element and the binding element has no default value."),A_this_type_is_available_only_in_a_non_static_member_of_a_class_or_interface:n(2526,e.DiagnosticCategory.Error,"A_this_type_is_available_only_in_a_non_static_member_of_a_class_or_interface_2526","A 'this' type is available only in a non-static member of a class or interface."),The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary:n(2527,e.DiagnosticCategory.Error,"The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary_2527","The inferred type of '{0}' references an inaccessible '{1}' type. A type annotation is necessary."),A_module_cannot_have_multiple_default_exports:n(2528,e.DiagnosticCategory.Error,"A_module_cannot_have_multiple_default_exports_2528","A module cannot have multiple default exports."),Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_containing_async_functions:n(2529,e.DiagnosticCategory.Error,"Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_containing_async_func_2529","Duplicate identifier '{0}'. Compiler reserves name '{1}' in top level scope of a module containing async functions."),Property_0_is_incompatible_with_index_signature:n(2530,e.DiagnosticCategory.Error,"Property_0_is_incompatible_with_index_signature_2530","Property '{0}' is incompatible with index signature."),Object_is_possibly_null:n(2531,e.DiagnosticCategory.Error,"Object_is_possibly_null_2531","Object is possibly 'null'."),Object_is_possibly_undefined:n(2532,e.DiagnosticCategory.Error,"Object_is_possibly_undefined_2532","Object is possibly 'undefined'."),Object_is_possibly_null_or_undefined:n(2533,e.DiagnosticCategory.Error,"Object_is_possibly_null_or_undefined_2533","Object is possibly 'null' or 'undefined'."),A_function_returning_never_cannot_have_a_reachable_end_point:n(2534,e.DiagnosticCategory.Error,"A_function_returning_never_cannot_have_a_reachable_end_point_2534","A function returning 'never' cannot have a reachable end point."),Enum_type_0_has_members_with_initializers_that_are_not_literals:n(2535,e.DiagnosticCategory.Error,"Enum_type_0_has_members_with_initializers_that_are_not_literals_2535","Enum type '{0}' has members with initializers that are not literals."),Type_0_cannot_be_used_to_index_type_1:n(2536,e.DiagnosticCategory.Error,"Type_0_cannot_be_used_to_index_type_1_2536","Type '{0}' cannot be used to index type '{1}'."),Type_0_has_no_matching_index_signature_for_type_1:n(2537,e.DiagnosticCategory.Error,"Type_0_has_no_matching_index_signature_for_type_1_2537","Type '{0}' has no matching index signature for type '{1}'."),Type_0_cannot_be_used_as_an_index_type:n(2538,e.DiagnosticCategory.Error,"Type_0_cannot_be_used_as_an_index_type_2538","Type '{0}' cannot be used as an index type."),Cannot_assign_to_0_because_it_is_not_a_variable:n(2539,e.DiagnosticCategory.Error,"Cannot_assign_to_0_because_it_is_not_a_variable_2539","Cannot assign to '{0}' because it is not a variable."),Cannot_assign_to_0_because_it_is_a_read_only_property:n(2540,e.DiagnosticCategory.Error,"Cannot_assign_to_0_because_it_is_a_read_only_property_2540","Cannot assign to '{0}' because it is a read-only property."),The_target_of_an_assignment_must_be_a_variable_or_a_property_access:n(2541,e.DiagnosticCategory.Error,"The_target_of_an_assignment_must_be_a_variable_or_a_property_access_2541","The target of an assignment must be a variable or a property access."),Index_signature_in_type_0_only_permits_reading:n(2542,e.DiagnosticCategory.Error,"Index_signature_in_type_0_only_permits_reading_2542","Index signature in type '{0}' only permits reading."),Duplicate_identifier_newTarget_Compiler_uses_variable_declaration_newTarget_to_capture_new_target_meta_property_reference:n(2543,e.DiagnosticCategory.Error,"Duplicate_identifier_newTarget_Compiler_uses_variable_declaration_newTarget_to_capture_new_target_me_2543","Duplicate identifier '_newTarget'. Compiler uses variable declaration '_newTarget' to capture 'new.target' meta-property reference."),Expression_resolves_to_variable_declaration_newTarget_that_compiler_uses_to_capture_new_target_meta_property_reference:n(2544,e.DiagnosticCategory.Error,"Expression_resolves_to_variable_declaration_newTarget_that_compiler_uses_to_capture_new_target_meta__2544","Expression resolves to variable declaration '_newTarget' that compiler uses to capture 'new.target' meta-property reference."),A_mixin_class_must_have_a_constructor_with_a_single_rest_parameter_of_type_any:n(2545,e.DiagnosticCategory.Error,"A_mixin_class_must_have_a_constructor_with_a_single_rest_parameter_of_type_any_2545","A mixin class must have a constructor with a single rest parameter of type 'any[]'."),The_type_returned_by_the_0_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value_property:n(2547,e.DiagnosticCategory.Error,"The_type_returned_by_the_0_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value_pro_2547","The type returned by the '{0}()' method of an async iterator must be a promise for a type with a 'value' property."),Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator:n(2548,e.DiagnosticCategory.Error,"Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator_2548","Type '{0}' is not an array type or does not have a '[Symbol.iterator]()' method that returns an iterator."),Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator:n(2549,e.DiagnosticCategory.Error,"Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns__2549","Type '{0}' is not an array type or a string type or does not have a '[Symbol.iterator]()' method that returns an iterator."),Property_0_does_not_exist_on_type_1_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_2_or_later:n(2550,e.DiagnosticCategory.Error,"Property_0_does_not_exist_on_type_1_Do_you_need_to_change_your_target_library_Try_changing_the_lib_c_2550","Property '{0}' does not exist on type '{1}'. Do you need to change your target library? Try changing the 'lib' compiler option to '{2}' or later."),Property_0_does_not_exist_on_type_1_Did_you_mean_2:n(2551,e.DiagnosticCategory.Error,"Property_0_does_not_exist_on_type_1_Did_you_mean_2_2551","Property '{0}' does not exist on type '{1}'. Did you mean '{2}'?"),Cannot_find_name_0_Did_you_mean_1:n(2552,e.DiagnosticCategory.Error,"Cannot_find_name_0_Did_you_mean_1_2552","Cannot find name '{0}'. Did you mean '{1}'?"),Computed_values_are_not_permitted_in_an_enum_with_string_valued_members:n(2553,e.DiagnosticCategory.Error,"Computed_values_are_not_permitted_in_an_enum_with_string_valued_members_2553","Computed values are not permitted in an enum with string valued members."),Expected_0_arguments_but_got_1:n(2554,e.DiagnosticCategory.Error,"Expected_0_arguments_but_got_1_2554","Expected {0} arguments, but got {1}."),Expected_at_least_0_arguments_but_got_1:n(2555,e.DiagnosticCategory.Error,"Expected_at_least_0_arguments_but_got_1_2555","Expected at least {0} arguments, but got {1}."),A_spread_argument_must_either_have_a_tuple_type_or_be_passed_to_a_rest_parameter:n(2556,e.DiagnosticCategory.Error,"A_spread_argument_must_either_have_a_tuple_type_or_be_passed_to_a_rest_parameter_2556","A spread argument must either have a tuple type or be passed to a rest parameter."),Expected_0_type_arguments_but_got_1:n(2558,e.DiagnosticCategory.Error,"Expected_0_type_arguments_but_got_1_2558","Expected {0} type arguments, but got {1}."),Type_0_has_no_properties_in_common_with_type_1:n(2559,e.DiagnosticCategory.Error,"Type_0_has_no_properties_in_common_with_type_1_2559","Type '{0}' has no properties in common with type '{1}'."),Value_of_type_0_has_no_properties_in_common_with_type_1_Did_you_mean_to_call_it:n(2560,e.DiagnosticCategory.Error,"Value_of_type_0_has_no_properties_in_common_with_type_1_Did_you_mean_to_call_it_2560","Value of type '{0}' has no properties in common with type '{1}'. Did you mean to call it?"),Object_literal_may_only_specify_known_properties_but_0_does_not_exist_in_type_1_Did_you_mean_to_write_2:n(2561,e.DiagnosticCategory.Error,"Object_literal_may_only_specify_known_properties_but_0_does_not_exist_in_type_1_Did_you_mean_to_writ_2561","Object literal may only specify known properties, but '{0}' does not exist in type '{1}'. Did you mean to write '{2}'?"),Base_class_expressions_cannot_reference_class_type_parameters:n(2562,e.DiagnosticCategory.Error,"Base_class_expressions_cannot_reference_class_type_parameters_2562","Base class expressions cannot reference class type parameters."),The_containing_function_or_module_body_is_too_large_for_control_flow_analysis:n(2563,e.DiagnosticCategory.Error,"The_containing_function_or_module_body_is_too_large_for_control_flow_analysis_2563","The containing function or module body is too large for control flow analysis."),Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor:n(2564,e.DiagnosticCategory.Error,"Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor_2564","Property '{0}' has no initializer and is not definitely assigned in the constructor."),Property_0_is_used_before_being_assigned:n(2565,e.DiagnosticCategory.Error,"Property_0_is_used_before_being_assigned_2565","Property '{0}' is used before being assigned."),A_rest_element_cannot_have_a_property_name:n(2566,e.DiagnosticCategory.Error,"A_rest_element_cannot_have_a_property_name_2566","A rest element cannot have a property name."),Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations:n(2567,e.DiagnosticCategory.Error,"Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations_2567","Enum declarations can only merge with namespace or other enum declarations."),Property_0_may_not_exist_on_type_1_Did_you_mean_2:n(2568,e.DiagnosticCategory.Error,"Property_0_may_not_exist_on_type_1_Did_you_mean_2_2568","Property '{0}' may not exist on type '{1}'. Did you mean '{2}'?"),Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators:n(2569,e.DiagnosticCategory.Error,"Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterati_2569","Type '{0}' is not an array type or a string type. Use compiler option '--downlevelIteration' to allow iterating of iterators."),Could_not_find_name_0_Did_you_mean_1:n(2570,e.DiagnosticCategory.Error,"Could_not_find_name_0_Did_you_mean_1_2570","Could not find name '{0}'. Did you mean '{1}'?"),Object_is_of_type_unknown:n(2571,e.DiagnosticCategory.Error,"Object_is_of_type_unknown_2571","Object is of type 'unknown'."),Rest_signatures_are_incompatible:n(2572,e.DiagnosticCategory.Error,"Rest_signatures_are_incompatible_2572","Rest signatures are incompatible."),Property_0_is_incompatible_with_rest_element_type:n(2573,e.DiagnosticCategory.Error,"Property_0_is_incompatible_with_rest_element_type_2573","Property '{0}' is incompatible with rest element type."),A_rest_element_type_must_be_an_array_type:n(2574,e.DiagnosticCategory.Error,"A_rest_element_type_must_be_an_array_type_2574","A rest element type must be an array type."),No_overload_expects_0_arguments_but_overloads_do_exist_that_expect_either_1_or_2_arguments:n(2575,e.DiagnosticCategory.Error,"No_overload_expects_0_arguments_but_overloads_do_exist_that_expect_either_1_or_2_arguments_2575","No overload expects {0} arguments, but overloads do exist that expect either {1} or {2} arguments."),Property_0_does_not_exist_on_type_1_Did_you_mean_to_access_the_static_member_2_instead:n(2576,e.DiagnosticCategory.Error,"Property_0_does_not_exist_on_type_1_Did_you_mean_to_access_the_static_member_2_instead_2576","Property '{0}' does not exist on type '{1}'. Did you mean to access the static member '{2}' instead?"),Return_type_annotation_circularly_references_itself:n(2577,e.DiagnosticCategory.Error,"Return_type_annotation_circularly_references_itself_2577","Return type annotation circularly references itself."),Unused_ts_expect_error_directive:n(2578,e.DiagnosticCategory.Error,"Unused_ts_expect_error_directive_2578","Unused '@ts-expect-error' directive."),Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_save_dev_types_Slashnode:n(2580,e.DiagnosticCategory.Error,"Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_save_dev_types_Slashno_2580","Cannot find name '{0}'. Do you need to install type definitions for node? Try `npm i --save-dev @types/node`."),Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_save_dev_types_Slashjquery:n(2581,e.DiagnosticCategory.Error,"Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_save_dev_types_Slash_2581","Cannot find name '{0}'. Do you need to install type definitions for jQuery? Try `npm i --save-dev @types/jquery`."),Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_save_dev_types_Slashjest_or_npm_i_save_dev_types_Slashmocha:n(2582,e.DiagnosticCategory.Error,"Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_save_dev_type_2582","Cannot find name '{0}'. Do you need to install type definitions for a test runner? Try `npm i --save-dev @types/jest` or `npm i --save-dev @types/mocha`."),Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_1_or_later:n(2583,e.DiagnosticCategory.Error,"Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_2583","Cannot find name '{0}'. Do you need to change your target library? Try changing the 'lib' compiler option to '{1}' or later."),Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_include_dom:n(2584,e.DiagnosticCategory.Error,"Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_2584","Cannot find name '{0}'. Do you need to change your target library? Try changing the 'lib' compiler option to include 'dom'."),_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later:n(2585,e.DiagnosticCategory.Error,"_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Do_you_need_to_change_your_target_library_2585","'{0}' only refers to a type, but is being used as a value here. Do you need to change your target library? Try changing the 'lib' compiler option to es2015 or later."),Enum_type_0_circularly_references_itself:n(2586,e.DiagnosticCategory.Error,"Enum_type_0_circularly_references_itself_2586","Enum type '{0}' circularly references itself."),JSDoc_type_0_circularly_references_itself:n(2587,e.DiagnosticCategory.Error,"JSDoc_type_0_circularly_references_itself_2587","JSDoc type '{0}' circularly references itself."),Cannot_assign_to_0_because_it_is_a_constant:n(2588,e.DiagnosticCategory.Error,"Cannot_assign_to_0_because_it_is_a_constant_2588","Cannot assign to '{0}' because it is a constant."),Type_instantiation_is_excessively_deep_and_possibly_infinite:n(2589,e.DiagnosticCategory.Error,"Type_instantiation_is_excessively_deep_and_possibly_infinite_2589","Type instantiation is excessively deep and possibly infinite."),Expression_produces_a_union_type_that_is_too_complex_to_represent:n(2590,e.DiagnosticCategory.Error,"Expression_produces_a_union_type_that_is_too_complex_to_represent_2590","Expression produces a union type that is too complex to represent."),Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_save_dev_types_Slashnode_and_then_add_node_to_the_types_field_in_your_tsconfig:n(2591,e.DiagnosticCategory.Error,"Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_save_dev_types_Slashno_2591","Cannot find name '{0}'. Do you need to install type definitions for node? Try `npm i --save-dev @types/node` and then add 'node' to the types field in your tsconfig."),Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_save_dev_types_Slashjquery_and_then_add_jquery_to_the_types_field_in_your_tsconfig:n(2592,e.DiagnosticCategory.Error,"Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_save_dev_types_Slash_2592","Cannot find name '{0}'. Do you need to install type definitions for jQuery? Try `npm i --save-dev @types/jquery` and then add 'jquery' to the types field in your tsconfig."),Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_save_dev_types_Slashjest_or_npm_i_save_dev_types_Slashmocha_and_then_add_jest_or_mocha_to_the_types_field_in_your_tsconfig:n(2593,e.DiagnosticCategory.Error,"Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_save_dev_type_2593","Cannot find name '{0}'. Do you need to install type definitions for a test runner? Try `npm i --save-dev @types/jest` or `npm i --save-dev @types/mocha` and then add 'jest' or 'mocha' to the types field in your tsconfig."),This_module_is_declared_with_using_export_and_can_only_be_used_with_a_default_import_when_using_the_0_flag:n(2594,e.DiagnosticCategory.Error,"This_module_is_declared_with_using_export_and_can_only_be_used_with_a_default_import_when_using_the__2594","This module is declared with using 'export =', and can only be used with a default import when using the '{0}' flag."),_0_can_only_be_imported_by_using_a_default_import:n(2595,e.DiagnosticCategory.Error,"_0_can_only_be_imported_by_using_a_default_import_2595","'{0}' can only be imported by using a default import."),_0_can_only_be_imported_by_turning_on_the_esModuleInterop_flag_and_using_a_default_import:n(2596,e.DiagnosticCategory.Error,"_0_can_only_be_imported_by_turning_on_the_esModuleInterop_flag_and_using_a_default_import_2596","'{0}' can only be imported by turning on the 'esModuleInterop' flag and using a default import."),_0_can_only_be_imported_by_using_a_require_call_or_by_using_a_default_import:n(2597,e.DiagnosticCategory.Error,"_0_can_only_be_imported_by_using_a_require_call_or_by_using_a_default_import_2597","'{0}' can only be imported by using a 'require' call or by using a default import."),_0_can_only_be_imported_by_using_a_require_call_or_by_turning_on_the_esModuleInterop_flag_and_using_a_default_import:n(2598,e.DiagnosticCategory.Error,"_0_can_only_be_imported_by_using_a_require_call_or_by_turning_on_the_esModuleInterop_flag_and_using__2598","'{0}' can only be imported by using a 'require' call or by turning on the 'esModuleInterop' flag and using a default import."),JSX_element_attributes_type_0_may_not_be_a_union_type:n(2600,e.DiagnosticCategory.Error,"JSX_element_attributes_type_0_may_not_be_a_union_type_2600","JSX element attributes type '{0}' may not be a union type."),The_return_type_of_a_JSX_element_constructor_must_return_an_object_type:n(2601,e.DiagnosticCategory.Error,"The_return_type_of_a_JSX_element_constructor_must_return_an_object_type_2601","The return type of a JSX element constructor must return an object type."),JSX_element_implicitly_has_type_any_because_the_global_type_JSX_Element_does_not_exist:n(2602,e.DiagnosticCategory.Error,"JSX_element_implicitly_has_type_any_because_the_global_type_JSX_Element_does_not_exist_2602","JSX element implicitly has type 'any' because the global type 'JSX.Element' does not exist."),Property_0_in_type_1_is_not_assignable_to_type_2:n(2603,e.DiagnosticCategory.Error,"Property_0_in_type_1_is_not_assignable_to_type_2_2603","Property '{0}' in type '{1}' is not assignable to type '{2}'."),JSX_element_type_0_does_not_have_any_construct_or_call_signatures:n(2604,e.DiagnosticCategory.Error,"JSX_element_type_0_does_not_have_any_construct_or_call_signatures_2604","JSX element type '{0}' does not have any construct or call signatures."),JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements:n(2605,e.DiagnosticCategory.Error,"JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements_2605","JSX element type '{0}' is not a constructor function for JSX elements."),Property_0_of_JSX_spread_attribute_is_not_assignable_to_target_property:n(2606,e.DiagnosticCategory.Error,"Property_0_of_JSX_spread_attribute_is_not_assignable_to_target_property_2606","Property '{0}' of JSX spread attribute is not assignable to target property."),JSX_element_class_does_not_support_attributes_because_it_does_not_have_a_0_property:n(2607,e.DiagnosticCategory.Error,"JSX_element_class_does_not_support_attributes_because_it_does_not_have_a_0_property_2607","JSX element class does not support attributes because it does not have a '{0}' property."),The_global_type_JSX_0_may_not_have_more_than_one_property:n(2608,e.DiagnosticCategory.Error,"The_global_type_JSX_0_may_not_have_more_than_one_property_2608","The global type 'JSX.{0}' may not have more than one property."),JSX_spread_child_must_be_an_array_type:n(2609,e.DiagnosticCategory.Error,"JSX_spread_child_must_be_an_array_type_2609","JSX spread child must be an array type."),_0_is_defined_as_an_accessor_in_class_1_but_is_overridden_here_in_2_as_an_instance_property:n(2610,e.DiagnosticCategory.Error,"_0_is_defined_as_an_accessor_in_class_1_but_is_overridden_here_in_2_as_an_instance_property_2610","'{0}' is defined as an accessor in class '{1}', but is overridden here in '{2}' as an instance property."),_0_is_defined_as_a_property_in_class_1_but_is_overridden_here_in_2_as_an_accessor:n(2611,e.DiagnosticCategory.Error,"_0_is_defined_as_a_property_in_class_1_but_is_overridden_here_in_2_as_an_accessor_2611","'{0}' is defined as a property in class '{1}', but is overridden here in '{2}' as an accessor."),Property_0_will_overwrite_the_base_property_in_1_If_this_is_intentional_add_an_initializer_Otherwise_add_a_declare_modifier_or_remove_the_redundant_declaration:n(2612,e.DiagnosticCategory.Error,"Property_0_will_overwrite_the_base_property_in_1_If_this_is_intentional_add_an_initializer_Otherwise_2612","Property '{0}' will overwrite the base property in '{1}'. If this is intentional, add an initializer. Otherwise, add a 'declare' modifier or remove the redundant declaration."),Module_0_has_no_default_export_Did_you_mean_to_use_import_1_from_0_instead:n(2613,e.DiagnosticCategory.Error,"Module_0_has_no_default_export_Did_you_mean_to_use_import_1_from_0_instead_2613","Module '{0}' has no default export. Did you mean to use 'import { {1} } from {0}' instead?"),Module_0_has_no_exported_member_1_Did_you_mean_to_use_import_1_from_0_instead:n(2614,e.DiagnosticCategory.Error,"Module_0_has_no_exported_member_1_Did_you_mean_to_use_import_1_from_0_instead_2614","Module '{0}' has no exported member '{1}'. Did you mean to use 'import {1} from {0}' instead?"),Type_of_property_0_circularly_references_itself_in_mapped_type_1:n(2615,e.DiagnosticCategory.Error,"Type_of_property_0_circularly_references_itself_in_mapped_type_1_2615","Type of property '{0}' circularly references itself in mapped type '{1}'."),_0_can_only_be_imported_by_using_import_1_require_2_or_a_default_import:n(2616,e.DiagnosticCategory.Error,"_0_can_only_be_imported_by_using_import_1_require_2_or_a_default_import_2616","'{0}' can only be imported by using 'import {1} = require({2})' or a default import."),_0_can_only_be_imported_by_using_import_1_require_2_or_by_turning_on_the_esModuleInterop_flag_and_using_a_default_import:n(2617,e.DiagnosticCategory.Error,"_0_can_only_be_imported_by_using_import_1_require_2_or_by_turning_on_the_esModuleInterop_flag_and_us_2617","'{0}' can only be imported by using 'import {1} = require({2})' or by turning on the 'esModuleInterop' flag and using a default import."),Source_has_0_element_s_but_target_requires_1:n(2618,e.DiagnosticCategory.Error,"Source_has_0_element_s_but_target_requires_1_2618","Source has {0} element(s) but target requires {1}."),Source_has_0_element_s_but_target_allows_only_1:n(2619,e.DiagnosticCategory.Error,"Source_has_0_element_s_but_target_allows_only_1_2619","Source has {0} element(s) but target allows only {1}."),Target_requires_0_element_s_but_source_may_have_fewer:n(2620,e.DiagnosticCategory.Error,"Target_requires_0_element_s_but_source_may_have_fewer_2620","Target requires {0} element(s) but source may have fewer."),Target_allows_only_0_element_s_but_source_may_have_more:n(2621,e.DiagnosticCategory.Error,"Target_allows_only_0_element_s_but_source_may_have_more_2621","Target allows only {0} element(s) but source may have more."),Source_provides_no_match_for_required_element_at_position_0_in_target:n(2623,e.DiagnosticCategory.Error,"Source_provides_no_match_for_required_element_at_position_0_in_target_2623","Source provides no match for required element at position {0} in target."),Source_provides_no_match_for_variadic_element_at_position_0_in_target:n(2624,e.DiagnosticCategory.Error,"Source_provides_no_match_for_variadic_element_at_position_0_in_target_2624","Source provides no match for variadic element at position {0} in target."),Variadic_element_at_position_0_in_source_does_not_match_element_at_position_1_in_target:n(2625,e.DiagnosticCategory.Error,"Variadic_element_at_position_0_in_source_does_not_match_element_at_position_1_in_target_2625","Variadic element at position {0} in source does not match element at position {1} in target."),Type_at_position_0_in_source_is_not_compatible_with_type_at_position_1_in_target:n(2626,e.DiagnosticCategory.Error,"Type_at_position_0_in_source_is_not_compatible_with_type_at_position_1_in_target_2626","Type at position {0} in source is not compatible with type at position {1} in target."),Type_at_positions_0_through_1_in_source_is_not_compatible_with_type_at_position_2_in_target:n(2627,e.DiagnosticCategory.Error,"Type_at_positions_0_through_1_in_source_is_not_compatible_with_type_at_position_2_in_target_2627","Type at positions {0} through {1} in source is not compatible with type at position {2} in target."),Cannot_assign_to_0_because_it_is_an_enum:n(2628,e.DiagnosticCategory.Error,"Cannot_assign_to_0_because_it_is_an_enum_2628","Cannot assign to '{0}' because it is an enum."),Cannot_assign_to_0_because_it_is_a_class:n(2629,e.DiagnosticCategory.Error,"Cannot_assign_to_0_because_it_is_a_class_2629","Cannot assign to '{0}' because it is a class."),Cannot_assign_to_0_because_it_is_a_function:n(2630,e.DiagnosticCategory.Error,"Cannot_assign_to_0_because_it_is_a_function_2630","Cannot assign to '{0}' because it is a function."),Cannot_assign_to_0_because_it_is_a_namespace:n(2631,e.DiagnosticCategory.Error,"Cannot_assign_to_0_because_it_is_a_namespace_2631","Cannot assign to '{0}' because it is a namespace."),Cannot_assign_to_0_because_it_is_an_import:n(2632,e.DiagnosticCategory.Error,"Cannot_assign_to_0_because_it_is_an_import_2632","Cannot assign to '{0}' because it is an import."),JSX_property_access_expressions_cannot_include_JSX_namespace_names:n(2633,e.DiagnosticCategory.Error,"JSX_property_access_expressions_cannot_include_JSX_namespace_names_2633","JSX property access expressions cannot include JSX namespace names"),_0_index_signatures_are_incompatible:n(2634,e.DiagnosticCategory.Error,"_0_index_signatures_are_incompatible_2634","'{0}' index signatures are incompatible."),Cannot_augment_module_0_with_value_exports_because_it_resolves_to_a_non_module_entity:n(2649,e.DiagnosticCategory.Error,"Cannot_augment_module_0_with_value_exports_because_it_resolves_to_a_non_module_entity_2649","Cannot augment module '{0}' with value exports because it resolves to a non-module entity."),A_member_initializer_in_a_enum_declaration_cannot_reference_members_declared_after_it_including_members_defined_in_other_enums:n(2651,e.DiagnosticCategory.Error,"A_member_initializer_in_a_enum_declaration_cannot_reference_members_declared_after_it_including_memb_2651","A member initializer in a enum declaration cannot reference members declared after it, including members defined in other enums."),Merged_declaration_0_cannot_include_a_default_export_declaration_Consider_adding_a_separate_export_default_0_declaration_instead:n(2652,e.DiagnosticCategory.Error,"Merged_declaration_0_cannot_include_a_default_export_declaration_Consider_adding_a_separate_export_d_2652","Merged declaration '{0}' cannot include a default export declaration. Consider adding a separate 'export default {0}' declaration instead."),Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1:n(2653,e.DiagnosticCategory.Error,"Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1_2653","Non-abstract class expression does not implement inherited abstract member '{0}' from class '{1}'."),Exported_external_package_typings_file_cannot_contain_tripleslash_references_Please_contact_the_package_author_to_update_the_package_definition:n(2654,e.DiagnosticCategory.Error,"Exported_external_package_typings_file_cannot_contain_tripleslash_references_Please_contact_the_pack_2654","Exported external package typings file cannot contain tripleslash references. Please contact the package author to update the package definition."),Exported_external_package_typings_file_0_is_not_a_module_Please_contact_the_package_author_to_update_the_package_definition:n(2656,e.DiagnosticCategory.Error,"Exported_external_package_typings_file_0_is_not_a_module_Please_contact_the_package_author_to_update_2656","Exported external package typings file '{0}' is not a module. Please contact the package author to update the package definition."),JSX_expressions_must_have_one_parent_element:n(2657,e.DiagnosticCategory.Error,"JSX_expressions_must_have_one_parent_element_2657","JSX expressions must have one parent element."),Type_0_provides_no_match_for_the_signature_1:n(2658,e.DiagnosticCategory.Error,"Type_0_provides_no_match_for_the_signature_1_2658","Type '{0}' provides no match for the signature '{1}'."),super_is_only_allowed_in_members_of_object_literal_expressions_when_option_target_is_ES2015_or_higher:n(2659,e.DiagnosticCategory.Error,"super_is_only_allowed_in_members_of_object_literal_expressions_when_option_target_is_ES2015_or_highe_2659","'super' is only allowed in members of object literal expressions when option 'target' is 'ES2015' or higher."),super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions:n(2660,e.DiagnosticCategory.Error,"super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions_2660","'super' can only be referenced in members of derived classes or object literal expressions."),Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module:n(2661,e.DiagnosticCategory.Error,"Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module_2661","Cannot export '{0}'. Only local declarations can be exported from a module."),Cannot_find_name_0_Did_you_mean_the_static_member_1_0:n(2662,e.DiagnosticCategory.Error,"Cannot_find_name_0_Did_you_mean_the_static_member_1_0_2662","Cannot find name '{0}'. Did you mean the static member '{1}.{0}'?"),Cannot_find_name_0_Did_you_mean_the_instance_member_this_0:n(2663,e.DiagnosticCategory.Error,"Cannot_find_name_0_Did_you_mean_the_instance_member_this_0_2663","Cannot find name '{0}'. Did you mean the instance member 'this.{0}'?"),Invalid_module_name_in_augmentation_module_0_cannot_be_found:n(2664,e.DiagnosticCategory.Error,"Invalid_module_name_in_augmentation_module_0_cannot_be_found_2664","Invalid module name in augmentation, module '{0}' cannot be found."),Invalid_module_name_in_augmentation_Module_0_resolves_to_an_untyped_module_at_1_which_cannot_be_augmented:n(2665,e.DiagnosticCategory.Error,"Invalid_module_name_in_augmentation_Module_0_resolves_to_an_untyped_module_at_1_which_cannot_be_augm_2665","Invalid module name in augmentation. Module '{0}' resolves to an untyped module at '{1}', which cannot be augmented."),Exports_and_export_assignments_are_not_permitted_in_module_augmentations:n(2666,e.DiagnosticCategory.Error,"Exports_and_export_assignments_are_not_permitted_in_module_augmentations_2666","Exports and export assignments are not permitted in module augmentations."),Imports_are_not_permitted_in_module_augmentations_Consider_moving_them_to_the_enclosing_external_module:n(2667,e.DiagnosticCategory.Error,"Imports_are_not_permitted_in_module_augmentations_Consider_moving_them_to_the_enclosing_external_mod_2667","Imports are not permitted in module augmentations. Consider moving them to the enclosing external module."),export_modifier_cannot_be_applied_to_ambient_modules_and_module_augmentations_since_they_are_always_visible:n(2668,e.DiagnosticCategory.Error,"export_modifier_cannot_be_applied_to_ambient_modules_and_module_augmentations_since_they_are_always__2668","'export' modifier cannot be applied to ambient modules and module augmentations since they are always visible."),Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_declarations:n(2669,e.DiagnosticCategory.Error,"Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_2669","Augmentations for the global scope can only be directly nested in external modules or ambient module declarations."),Augmentations_for_the_global_scope_should_have_declare_modifier_unless_they_appear_in_already_ambient_context:n(2670,e.DiagnosticCategory.Error,"Augmentations_for_the_global_scope_should_have_declare_modifier_unless_they_appear_in_already_ambien_2670","Augmentations for the global scope should have 'declare' modifier unless they appear in already ambient context."),Cannot_augment_module_0_because_it_resolves_to_a_non_module_entity:n(2671,e.DiagnosticCategory.Error,"Cannot_augment_module_0_because_it_resolves_to_a_non_module_entity_2671","Cannot augment module '{0}' because it resolves to a non-module entity."),Cannot_assign_a_0_constructor_type_to_a_1_constructor_type:n(2672,e.DiagnosticCategory.Error,"Cannot_assign_a_0_constructor_type_to_a_1_constructor_type_2672","Cannot assign a '{0}' constructor type to a '{1}' constructor type."),Constructor_of_class_0_is_private_and_only_accessible_within_the_class_declaration:n(2673,e.DiagnosticCategory.Error,"Constructor_of_class_0_is_private_and_only_accessible_within_the_class_declaration_2673","Constructor of class '{0}' is private and only accessible within the class declaration."),Constructor_of_class_0_is_protected_and_only_accessible_within_the_class_declaration:n(2674,e.DiagnosticCategory.Error,"Constructor_of_class_0_is_protected_and_only_accessible_within_the_class_declaration_2674","Constructor of class '{0}' is protected and only accessible within the class declaration."),Cannot_extend_a_class_0_Class_constructor_is_marked_as_private:n(2675,e.DiagnosticCategory.Error,"Cannot_extend_a_class_0_Class_constructor_is_marked_as_private_2675","Cannot extend a class '{0}'. Class constructor is marked as private."),Accessors_must_both_be_abstract_or_non_abstract:n(2676,e.DiagnosticCategory.Error,"Accessors_must_both_be_abstract_or_non_abstract_2676","Accessors must both be abstract or non-abstract."),A_type_predicate_s_type_must_be_assignable_to_its_parameter_s_type:n(2677,e.DiagnosticCategory.Error,"A_type_predicate_s_type_must_be_assignable_to_its_parameter_s_type_2677","A type predicate's type must be assignable to its parameter's type."),Type_0_is_not_comparable_to_type_1:n(2678,e.DiagnosticCategory.Error,"Type_0_is_not_comparable_to_type_1_2678","Type '{0}' is not comparable to type '{1}'."),A_function_that_is_called_with_the_new_keyword_cannot_have_a_this_type_that_is_void:n(2679,e.DiagnosticCategory.Error,"A_function_that_is_called_with_the_new_keyword_cannot_have_a_this_type_that_is_void_2679","A function that is called with the 'new' keyword cannot have a 'this' type that is 'void'."),A_0_parameter_must_be_the_first_parameter:n(2680,e.DiagnosticCategory.Error,"A_0_parameter_must_be_the_first_parameter_2680","A '{0}' parameter must be the first parameter."),A_constructor_cannot_have_a_this_parameter:n(2681,e.DiagnosticCategory.Error,"A_constructor_cannot_have_a_this_parameter_2681","A constructor cannot have a 'this' parameter."),get_and_set_accessor_must_have_the_same_this_type:n(2682,e.DiagnosticCategory.Error,"get_and_set_accessor_must_have_the_same_this_type_2682","'get' and 'set' accessor must have the same 'this' type."),this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation:n(2683,e.DiagnosticCategory.Error,"this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_2683","'this' implicitly has type 'any' because it does not have a type annotation."),The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1:n(2684,e.DiagnosticCategory.Error,"The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1_2684","The 'this' context of type '{0}' is not assignable to method's 'this' of type '{1}'."),The_this_types_of_each_signature_are_incompatible:n(2685,e.DiagnosticCategory.Error,"The_this_types_of_each_signature_are_incompatible_2685","The 'this' types of each signature are incompatible."),_0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead:n(2686,e.DiagnosticCategory.Error,"_0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead_2686","'{0}' refers to a UMD global, but the current file is a module. Consider adding an import instead."),All_declarations_of_0_must_have_identical_modifiers:n(2687,e.DiagnosticCategory.Error,"All_declarations_of_0_must_have_identical_modifiers_2687","All declarations of '{0}' must have identical modifiers."),Cannot_find_type_definition_file_for_0:n(2688,e.DiagnosticCategory.Error,"Cannot_find_type_definition_file_for_0_2688","Cannot find type definition file for '{0}'."),Cannot_extend_an_interface_0_Did_you_mean_implements:n(2689,e.DiagnosticCategory.Error,"Cannot_extend_an_interface_0_Did_you_mean_implements_2689","Cannot extend an interface '{0}'. Did you mean 'implements'?"),_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Did_you_mean_to_use_1_in_0:n(2690,e.DiagnosticCategory.Error,"_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Did_you_mean_to_use_1_in_0_2690","'{0}' only refers to a type, but is being used as a value here. Did you mean to use '{1} in {0}'?"),An_import_path_cannot_end_with_a_0_extension_Consider_importing_1_instead:n(2691,e.DiagnosticCategory.Error,"An_import_path_cannot_end_with_a_0_extension_Consider_importing_1_instead_2691","An import path cannot end with a '{0}' extension. Consider importing '{1}' instead."),_0_is_a_primitive_but_1_is_a_wrapper_object_Prefer_using_0_when_possible:n(2692,e.DiagnosticCategory.Error,"_0_is_a_primitive_but_1_is_a_wrapper_object_Prefer_using_0_when_possible_2692","'{0}' is a primitive, but '{1}' is a wrapper object. Prefer using '{0}' when possible."),_0_only_refers_to_a_type_but_is_being_used_as_a_value_here:n(2693,e.DiagnosticCategory.Error,"_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_2693","'{0}' only refers to a type, but is being used as a value here."),Namespace_0_has_no_exported_member_1:n(2694,e.DiagnosticCategory.Error,"Namespace_0_has_no_exported_member_1_2694","Namespace '{0}' has no exported member '{1}'."),Left_side_of_comma_operator_is_unused_and_has_no_side_effects:n(2695,e.DiagnosticCategory.Error,"Left_side_of_comma_operator_is_unused_and_has_no_side_effects_2695","Left side of comma operator is unused and has no side effects.",!0),The_Object_type_is_assignable_to_very_few_other_types_Did_you_mean_to_use_the_any_type_instead:n(2696,e.DiagnosticCategory.Error,"The_Object_type_is_assignable_to_very_few_other_types_Did_you_mean_to_use_the_any_type_instead_2696","The 'Object' type is assignable to very few other types. Did you mean to use the 'any' type instead?"),An_async_function_or_method_must_return_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option:n(2697,e.DiagnosticCategory.Error,"An_async_function_or_method_must_return_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_in_2697","An async function or method must return a 'Promise'. Make sure you have a declaration for 'Promise' or include 'ES2015' in your '--lib' option."),Spread_types_may_only_be_created_from_object_types:n(2698,e.DiagnosticCategory.Error,"Spread_types_may_only_be_created_from_object_types_2698","Spread types may only be created from object types."),Static_property_0_conflicts_with_built_in_property_Function_0_of_constructor_function_1:n(2699,e.DiagnosticCategory.Error,"Static_property_0_conflicts_with_built_in_property_Function_0_of_constructor_function_1_2699","Static property '{0}' conflicts with built-in property 'Function.{0}' of constructor function '{1}'."),Rest_types_may_only_be_created_from_object_types:n(2700,e.DiagnosticCategory.Error,"Rest_types_may_only_be_created_from_object_types_2700","Rest types may only be created from object types."),The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access:n(2701,e.DiagnosticCategory.Error,"The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access_2701","The target of an object rest assignment must be a variable or a property access."),_0_only_refers_to_a_type_but_is_being_used_as_a_namespace_here:n(2702,e.DiagnosticCategory.Error,"_0_only_refers_to_a_type_but_is_being_used_as_a_namespace_here_2702","'{0}' only refers to a type, but is being used as a namespace here."),The_operand_of_a_delete_operator_must_be_a_property_reference:n(2703,e.DiagnosticCategory.Error,"The_operand_of_a_delete_operator_must_be_a_property_reference_2703","The operand of a 'delete' operator must be a property reference."),The_operand_of_a_delete_operator_cannot_be_a_read_only_property:n(2704,e.DiagnosticCategory.Error,"The_operand_of_a_delete_operator_cannot_be_a_read_only_property_2704","The operand of a 'delete' operator cannot be a read-only property."),An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option:n(2705,e.DiagnosticCategory.Error,"An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_de_2705","An async function or method in ES5/ES3 requires the 'Promise' constructor.  Make sure you have a declaration for the 'Promise' constructor or include 'ES2015' in your '--lib' option."),Required_type_parameters_may_not_follow_optional_type_parameters:n(2706,e.DiagnosticCategory.Error,"Required_type_parameters_may_not_follow_optional_type_parameters_2706","Required type parameters may not follow optional type parameters."),Generic_type_0_requires_between_1_and_2_type_arguments:n(2707,e.DiagnosticCategory.Error,"Generic_type_0_requires_between_1_and_2_type_arguments_2707","Generic type '{0}' requires between {1} and {2} type arguments."),Cannot_use_namespace_0_as_a_value:n(2708,e.DiagnosticCategory.Error,"Cannot_use_namespace_0_as_a_value_2708","Cannot use namespace '{0}' as a value."),Cannot_use_namespace_0_as_a_type:n(2709,e.DiagnosticCategory.Error,"Cannot_use_namespace_0_as_a_type_2709","Cannot use namespace '{0}' as a type."),_0_are_specified_twice_The_attribute_named_0_will_be_overwritten:n(2710,e.DiagnosticCategory.Error,"_0_are_specified_twice_The_attribute_named_0_will_be_overwritten_2710","'{0}' are specified twice. The attribute named '{0}' will be overwritten."),A_dynamic_import_call_returns_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option:n(2711,e.DiagnosticCategory.Error,"A_dynamic_import_call_returns_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES20_2711","A dynamic import call returns a 'Promise'. Make sure you have a declaration for 'Promise' or include 'ES2015' in your '--lib' option."),A_dynamic_import_call_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option:n(2712,e.DiagnosticCategory.Error,"A_dynamic_import_call_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declarat_2712","A dynamic import call in ES5/ES3 requires the 'Promise' constructor.  Make sure you have a declaration for the 'Promise' constructor or include 'ES2015' in your '--lib' option."),Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_property_1_in_0_with_0_1:n(2713,e.DiagnosticCategory.Error,"Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_p_2713","Cannot access '{0}.{1}' because '{0}' is a type, but not a namespace. Did you mean to retrieve the type of the property '{1}' in '{0}' with '{0}[\"{1}\"]'?"),The_expression_of_an_export_assignment_must_be_an_identifier_or_qualified_name_in_an_ambient_context:n(2714,e.DiagnosticCategory.Error,"The_expression_of_an_export_assignment_must_be_an_identifier_or_qualified_name_in_an_ambient_context_2714","The expression of an export assignment must be an identifier or qualified name in an ambient context."),Abstract_property_0_in_class_1_cannot_be_accessed_in_the_constructor:n(2715,e.DiagnosticCategory.Error,"Abstract_property_0_in_class_1_cannot_be_accessed_in_the_constructor_2715","Abstract property '{0}' in class '{1}' cannot be accessed in the constructor."),Type_parameter_0_has_a_circular_default:n(2716,e.DiagnosticCategory.Error,"Type_parameter_0_has_a_circular_default_2716","Type parameter '{0}' has a circular default."),Subsequent_property_declarations_must_have_the_same_type_Property_0_must_be_of_type_1_but_here_has_type_2:n(2717,e.DiagnosticCategory.Error,"Subsequent_property_declarations_must_have_the_same_type_Property_0_must_be_of_type_1_but_here_has_t_2717","Subsequent property declarations must have the same type.  Property '{0}' must be of type '{1}', but here has type '{2}'."),Duplicate_property_0:n(2718,e.DiagnosticCategory.Error,"Duplicate_property_0_2718","Duplicate property '{0}'."),Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated:n(2719,e.DiagnosticCategory.Error,"Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated_2719","Type '{0}' is not assignable to type '{1}'. Two different types with this name exist, but they are unrelated."),Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass:n(2720,e.DiagnosticCategory.Error,"Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclas_2720","Class '{0}' incorrectly implements class '{1}'. Did you mean to extend '{1}' and inherit its members as a subclass?"),Cannot_invoke_an_object_which_is_possibly_null:n(2721,e.DiagnosticCategory.Error,"Cannot_invoke_an_object_which_is_possibly_null_2721","Cannot invoke an object which is possibly 'null'."),Cannot_invoke_an_object_which_is_possibly_undefined:n(2722,e.DiagnosticCategory.Error,"Cannot_invoke_an_object_which_is_possibly_undefined_2722","Cannot invoke an object which is possibly 'undefined'."),Cannot_invoke_an_object_which_is_possibly_null_or_undefined:n(2723,e.DiagnosticCategory.Error,"Cannot_invoke_an_object_which_is_possibly_null_or_undefined_2723","Cannot invoke an object which is possibly 'null' or 'undefined'."),_0_has_no_exported_member_named_1_Did_you_mean_2:n(2724,e.DiagnosticCategory.Error,"_0_has_no_exported_member_named_1_Did_you_mean_2_2724","'{0}' has no exported member named '{1}'. Did you mean '{2}'?"),Class_name_cannot_be_Object_when_targeting_ES5_with_module_0:n(2725,e.DiagnosticCategory.Error,"Class_name_cannot_be_Object_when_targeting_ES5_with_module_0_2725","Class name cannot be 'Object' when targeting ES5 with module {0}."),Cannot_find_lib_definition_for_0:n(2726,e.DiagnosticCategory.Error,"Cannot_find_lib_definition_for_0_2726","Cannot find lib definition for '{0}'."),Cannot_find_lib_definition_for_0_Did_you_mean_1:n(2727,e.DiagnosticCategory.Error,"Cannot_find_lib_definition_for_0_Did_you_mean_1_2727","Cannot find lib definition for '{0}'. Did you mean '{1}'?"),_0_is_declared_here:n(2728,e.DiagnosticCategory.Message,"_0_is_declared_here_2728","'{0}' is declared here."),Property_0_is_used_before_its_initialization:n(2729,e.DiagnosticCategory.Error,"Property_0_is_used_before_its_initialization_2729","Property '{0}' is used before its initialization."),An_arrow_function_cannot_have_a_this_parameter:n(2730,e.DiagnosticCategory.Error,"An_arrow_function_cannot_have_a_this_parameter_2730","An arrow function cannot have a 'this' parameter."),Implicit_conversion_of_a_symbol_to_a_string_will_fail_at_runtime_Consider_wrapping_this_expression_in_String:n(2731,e.DiagnosticCategory.Error,"Implicit_conversion_of_a_symbol_to_a_string_will_fail_at_runtime_Consider_wrapping_this_expression_i_2731","Implicit conversion of a 'symbol' to a 'string' will fail at runtime. Consider wrapping this expression in 'String(...)'."),Cannot_find_module_0_Consider_using_resolveJsonModule_to_import_module_with_json_extension:n(2732,e.DiagnosticCategory.Error,"Cannot_find_module_0_Consider_using_resolveJsonModule_to_import_module_with_json_extension_2732","Cannot find module '{0}'. Consider using '--resolveJsonModule' to import module with '.json' extension."),Property_0_was_also_declared_here:n(2733,e.DiagnosticCategory.Error,"Property_0_was_also_declared_here_2733","Property '{0}' was also declared here."),Are_you_missing_a_semicolon:n(2734,e.DiagnosticCategory.Error,"Are_you_missing_a_semicolon_2734","Are you missing a semicolon?"),Did_you_mean_for_0_to_be_constrained_to_type_new_args_Colon_any_1:n(2735,e.DiagnosticCategory.Error,"Did_you_mean_for_0_to_be_constrained_to_type_new_args_Colon_any_1_2735","Did you mean for '{0}' to be constrained to type 'new (...args: any[]) => {1}'?"),Operator_0_cannot_be_applied_to_type_1:n(2736,e.DiagnosticCategory.Error,"Operator_0_cannot_be_applied_to_type_1_2736","Operator '{0}' cannot be applied to type '{1}'."),BigInt_literals_are_not_available_when_targeting_lower_than_ES2020:n(2737,e.DiagnosticCategory.Error,"BigInt_literals_are_not_available_when_targeting_lower_than_ES2020_2737","BigInt literals are not available when targeting lower than ES2020."),An_outer_value_of_this_is_shadowed_by_this_container:n(2738,e.DiagnosticCategory.Message,"An_outer_value_of_this_is_shadowed_by_this_container_2738","An outer value of 'this' is shadowed by this container."),Type_0_is_missing_the_following_properties_from_type_1_Colon_2:n(2739,e.DiagnosticCategory.Error,"Type_0_is_missing_the_following_properties_from_type_1_Colon_2_2739","Type '{0}' is missing the following properties from type '{1}': {2}"),Type_0_is_missing_the_following_properties_from_type_1_Colon_2_and_3_more:n(2740,e.DiagnosticCategory.Error,"Type_0_is_missing_the_following_properties_from_type_1_Colon_2_and_3_more_2740","Type '{0}' is missing the following properties from type '{1}': {2}, and {3} more."),Property_0_is_missing_in_type_1_but_required_in_type_2:n(2741,e.DiagnosticCategory.Error,"Property_0_is_missing_in_type_1_but_required_in_type_2_2741","Property '{0}' is missing in type '{1}' but required in type '{2}'."),The_inferred_type_of_0_cannot_be_named_without_a_reference_to_1_This_is_likely_not_portable_A_type_annotation_is_necessary:n(2742,e.DiagnosticCategory.Error,"The_inferred_type_of_0_cannot_be_named_without_a_reference_to_1_This_is_likely_not_portable_A_type_a_2742","The inferred type of '{0}' cannot be named without a reference to '{1}'. This is likely not portable. A type annotation is necessary."),No_overload_expects_0_type_arguments_but_overloads_do_exist_that_expect_either_1_or_2_type_arguments:n(2743,e.DiagnosticCategory.Error,"No_overload_expects_0_type_arguments_but_overloads_do_exist_that_expect_either_1_or_2_type_arguments_2743","No overload expects {0} type arguments, but overloads do exist that expect either {1} or {2} type arguments."),Type_parameter_defaults_can_only_reference_previously_declared_type_parameters:n(2744,e.DiagnosticCategory.Error,"Type_parameter_defaults_can_only_reference_previously_declared_type_parameters_2744","Type parameter defaults can only reference previously declared type parameters."),This_JSX_tag_s_0_prop_expects_type_1_which_requires_multiple_children_but_only_a_single_child_was_provided:n(2745,e.DiagnosticCategory.Error,"This_JSX_tag_s_0_prop_expects_type_1_which_requires_multiple_children_but_only_a_single_child_was_pr_2745","This JSX tag's '{0}' prop expects type '{1}' which requires multiple children, but only a single child was provided."),This_JSX_tag_s_0_prop_expects_a_single_child_of_type_1_but_multiple_children_were_provided:n(2746,e.DiagnosticCategory.Error,"This_JSX_tag_s_0_prop_expects_a_single_child_of_type_1_but_multiple_children_were_provided_2746","This JSX tag's '{0}' prop expects a single child of type '{1}', but multiple children were provided."),_0_components_don_t_accept_text_as_child_elements_Text_in_JSX_has_the_type_string_but_the_expected_type_of_1_is_2:n(2747,e.DiagnosticCategory.Error,"_0_components_don_t_accept_text_as_child_elements_Text_in_JSX_has_the_type_string_but_the_expected_t_2747","'{0}' components don't accept text as child elements. Text in JSX has the type 'string', but the expected type of '{1}' is '{2}'."),Cannot_access_ambient_const_enums_when_the_isolatedModules_flag_is_provided:n(2748,e.DiagnosticCategory.Error,"Cannot_access_ambient_const_enums_when_the_isolatedModules_flag_is_provided_2748","Cannot access ambient const enums when the '--isolatedModules' flag is provided."),_0_refers_to_a_value_but_is_being_used_as_a_type_here_Did_you_mean_typeof_0:n(2749,e.DiagnosticCategory.Error,"_0_refers_to_a_value_but_is_being_used_as_a_type_here_Did_you_mean_typeof_0_2749","'{0}' refers to a value, but is being used as a type here. Did you mean 'typeof {0}'?"),The_implementation_signature_is_declared_here:n(2750,e.DiagnosticCategory.Error,"The_implementation_signature_is_declared_here_2750","The implementation signature is declared here."),Circularity_originates_in_type_at_this_location:n(2751,e.DiagnosticCategory.Error,"Circularity_originates_in_type_at_this_location_2751","Circularity originates in type at this location."),The_first_export_default_is_here:n(2752,e.DiagnosticCategory.Error,"The_first_export_default_is_here_2752","The first export default is here."),Another_export_default_is_here:n(2753,e.DiagnosticCategory.Error,"Another_export_default_is_here_2753","Another export default is here."),super_may_not_use_type_arguments:n(2754,e.DiagnosticCategory.Error,"super_may_not_use_type_arguments_2754","'super' may not use type arguments."),No_constituent_of_type_0_is_callable:n(2755,e.DiagnosticCategory.Error,"No_constituent_of_type_0_is_callable_2755","No constituent of type '{0}' is callable."),Not_all_constituents_of_type_0_are_callable:n(2756,e.DiagnosticCategory.Error,"Not_all_constituents_of_type_0_are_callable_2756","Not all constituents of type '{0}' are callable."),Type_0_has_no_call_signatures:n(2757,e.DiagnosticCategory.Error,"Type_0_has_no_call_signatures_2757","Type '{0}' has no call signatures."),Each_member_of_the_union_type_0_has_signatures_but_none_of_those_signatures_are_compatible_with_each_other:n(2758,e.DiagnosticCategory.Error,"Each_member_of_the_union_type_0_has_signatures_but_none_of_those_signatures_are_compatible_with_each_2758","Each member of the union type '{0}' has signatures, but none of those signatures are compatible with each other."),No_constituent_of_type_0_is_constructable:n(2759,e.DiagnosticCategory.Error,"No_constituent_of_type_0_is_constructable_2759","No constituent of type '{0}' is constructable."),Not_all_constituents_of_type_0_are_constructable:n(2760,e.DiagnosticCategory.Error,"Not_all_constituents_of_type_0_are_constructable_2760","Not all constituents of type '{0}' are constructable."),Type_0_has_no_construct_signatures:n(2761,e.DiagnosticCategory.Error,"Type_0_has_no_construct_signatures_2761","Type '{0}' has no construct signatures."),Each_member_of_the_union_type_0_has_construct_signatures_but_none_of_those_signatures_are_compatible_with_each_other:n(2762,e.DiagnosticCategory.Error,"Each_member_of_the_union_type_0_has_construct_signatures_but_none_of_those_signatures_are_compatible_2762","Each member of the union type '{0}' has construct signatures, but none of those signatures are compatible with each other."),Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_for_of_will_always_send_0:n(2763,e.DiagnosticCategory.Error,"Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_for_of_will_always_s_2763","Cannot iterate value because the 'next' method of its iterator expects type '{1}', but for-of will always send '{0}'."),Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_spread_will_always_send_0:n(2764,e.DiagnosticCategory.Error,"Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_spread_will_al_2764","Cannot iterate value because the 'next' method of its iterator expects type '{1}', but array spread will always send '{0}'."),Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_destructuring_will_always_send_0:n(2765,e.DiagnosticCategory.Error,"Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_destructuring__2765","Cannot iterate value because the 'next' method of its iterator expects type '{1}', but array destructuring will always send '{0}'."),Cannot_delegate_iteration_to_value_because_the_next_method_of_its_iterator_expects_type_1_but_the_containing_generator_will_always_send_0:n(2766,e.DiagnosticCategory.Error,"Cannot_delegate_iteration_to_value_because_the_next_method_of_its_iterator_expects_type_1_but_the_co_2766","Cannot delegate iteration to value because the 'next' method of its iterator expects type '{1}', but the containing generator will always send '{0}'."),The_0_property_of_an_iterator_must_be_a_method:n(2767,e.DiagnosticCategory.Error,"The_0_property_of_an_iterator_must_be_a_method_2767","The '{0}' property of an iterator must be a method."),The_0_property_of_an_async_iterator_must_be_a_method:n(2768,e.DiagnosticCategory.Error,"The_0_property_of_an_async_iterator_must_be_a_method_2768","The '{0}' property of an async iterator must be a method."),No_overload_matches_this_call:n(2769,e.DiagnosticCategory.Error,"No_overload_matches_this_call_2769","No overload matches this call."),The_last_overload_gave_the_following_error:n(2770,e.DiagnosticCategory.Error,"The_last_overload_gave_the_following_error_2770","The last overload gave the following error."),The_last_overload_is_declared_here:n(2771,e.DiagnosticCategory.Error,"The_last_overload_is_declared_here_2771","The last overload is declared here."),Overload_0_of_1_2_gave_the_following_error:n(2772,e.DiagnosticCategory.Error,"Overload_0_of_1_2_gave_the_following_error_2772","Overload {0} of {1}, '{2}', gave the following error."),Did_you_forget_to_use_await:n(2773,e.DiagnosticCategory.Error,"Did_you_forget_to_use_await_2773","Did you forget to use 'await'?"),This_condition_will_always_return_true_since_this_function_is_always_defined_Did_you_mean_to_call_it_instead:n(2774,e.DiagnosticCategory.Error,"This_condition_will_always_return_true_since_this_function_is_always_defined_Did_you_mean_to_call_it_2774","This condition will always return true since this function is always defined. Did you mean to call it instead?"),Assertions_require_every_name_in_the_call_target_to_be_declared_with_an_explicit_type_annotation:n(2775,e.DiagnosticCategory.Error,"Assertions_require_every_name_in_the_call_target_to_be_declared_with_an_explicit_type_annotation_2775","Assertions require every name in the call target to be declared with an explicit type annotation."),Assertions_require_the_call_target_to_be_an_identifier_or_qualified_name:n(2776,e.DiagnosticCategory.Error,"Assertions_require_the_call_target_to_be_an_identifier_or_qualified_name_2776","Assertions require the call target to be an identifier or qualified name."),The_operand_of_an_increment_or_decrement_operator_may_not_be_an_optional_property_access:n(2777,e.DiagnosticCategory.Error,"The_operand_of_an_increment_or_decrement_operator_may_not_be_an_optional_property_access_2777","The operand of an increment or decrement operator may not be an optional property access."),The_target_of_an_object_rest_assignment_may_not_be_an_optional_property_access:n(2778,e.DiagnosticCategory.Error,"The_target_of_an_object_rest_assignment_may_not_be_an_optional_property_access_2778","The target of an object rest assignment may not be an optional property access."),The_left_hand_side_of_an_assignment_expression_may_not_be_an_optional_property_access:n(2779,e.DiagnosticCategory.Error,"The_left_hand_side_of_an_assignment_expression_may_not_be_an_optional_property_access_2779","The left-hand side of an assignment expression may not be an optional property access."),The_left_hand_side_of_a_for_in_statement_may_not_be_an_optional_property_access:n(2780,e.DiagnosticCategory.Error,"The_left_hand_side_of_a_for_in_statement_may_not_be_an_optional_property_access_2780","The left-hand side of a 'for...in' statement may not be an optional property access."),The_left_hand_side_of_a_for_of_statement_may_not_be_an_optional_property_access:n(2781,e.DiagnosticCategory.Error,"The_left_hand_side_of_a_for_of_statement_may_not_be_an_optional_property_access_2781","The left-hand side of a 'for...of' statement may not be an optional property access."),_0_needs_an_explicit_type_annotation:n(2782,e.DiagnosticCategory.Message,"_0_needs_an_explicit_type_annotation_2782","'{0}' needs an explicit type annotation."),_0_is_specified_more_than_once_so_this_usage_will_be_overwritten:n(2783,e.DiagnosticCategory.Error,"_0_is_specified_more_than_once_so_this_usage_will_be_overwritten_2783","'{0}' is specified more than once, so this usage will be overwritten."),get_and_set_accessors_cannot_declare_this_parameters:n(2784,e.DiagnosticCategory.Error,"get_and_set_accessors_cannot_declare_this_parameters_2784","'get' and 'set' accessors cannot declare 'this' parameters."),This_spread_always_overwrites_this_property:n(2785,e.DiagnosticCategory.Error,"This_spread_always_overwrites_this_property_2785","This spread always overwrites this property."),_0_cannot_be_used_as_a_JSX_component:n(2786,e.DiagnosticCategory.Error,"_0_cannot_be_used_as_a_JSX_component_2786","'{0}' cannot be used as a JSX component."),Its_return_type_0_is_not_a_valid_JSX_element:n(2787,e.DiagnosticCategory.Error,"Its_return_type_0_is_not_a_valid_JSX_element_2787","Its return type '{0}' is not a valid JSX element."),Its_instance_type_0_is_not_a_valid_JSX_element:n(2788,e.DiagnosticCategory.Error,"Its_instance_type_0_is_not_a_valid_JSX_element_2788","Its instance type '{0}' is not a valid JSX element."),Its_element_type_0_is_not_a_valid_JSX_element:n(2789,e.DiagnosticCategory.Error,"Its_element_type_0_is_not_a_valid_JSX_element_2789","Its element type '{0}' is not a valid JSX element."),The_operand_of_a_delete_operator_must_be_optional:n(2790,e.DiagnosticCategory.Error,"The_operand_of_a_delete_operator_must_be_optional_2790","The operand of a 'delete' operator must be optional."),Exponentiation_cannot_be_performed_on_bigint_values_unless_the_target_option_is_set_to_es2016_or_later:n(2791,e.DiagnosticCategory.Error,"Exponentiation_cannot_be_performed_on_bigint_values_unless_the_target_option_is_set_to_es2016_or_lat_2791","Exponentiation cannot be performed on 'bigint' values unless the 'target' option is set to 'es2016' or later."),Cannot_find_module_0_Did_you_mean_to_set_the_moduleResolution_option_to_node_or_to_add_aliases_to_the_paths_option:n(2792,e.DiagnosticCategory.Error,"Cannot_find_module_0_Did_you_mean_to_set_the_moduleResolution_option_to_node_or_to_add_aliases_to_th_2792","Cannot find module '{0}'. Did you mean to set the 'moduleResolution' option to 'node', or to add aliases to the 'paths' option?"),The_call_would_have_succeeded_against_this_implementation_but_implementation_signatures_of_overloads_are_not_externally_visible:n(2793,e.DiagnosticCategory.Error,"The_call_would_have_succeeded_against_this_implementation_but_implementation_signatures_of_overloads_2793","The call would have succeeded against this implementation, but implementation signatures of overloads are not externally visible."),Expected_0_arguments_but_got_1_Did_you_forget_to_include_void_in_your_type_argument_to_Promise:n(2794,e.DiagnosticCategory.Error,"Expected_0_arguments_but_got_1_Did_you_forget_to_include_void_in_your_type_argument_to_Promise_2794","Expected {0} arguments, but got {1}. Did you forget to include 'void' in your type argument to 'Promise'?"),The_intrinsic_keyword_can_only_be_used_to_declare_compiler_provided_intrinsic_types:n(2795,e.DiagnosticCategory.Error,"The_intrinsic_keyword_can_only_be_used_to_declare_compiler_provided_intrinsic_types_2795","The 'intrinsic' keyword can only be used to declare compiler provided intrinsic types."),It_is_likely_that_you_are_missing_a_comma_to_separate_these_two_template_expressions_They_form_a_tagged_template_expression_which_cannot_be_invoked:n(2796,e.DiagnosticCategory.Error,"It_is_likely_that_you_are_missing_a_comma_to_separate_these_two_template_expressions_They_form_a_tag_2796","It is likely that you are missing a comma to separate these two template expressions. They form a tagged template expression which cannot be invoked."),A_mixin_class_that_extends_from_a_type_variable_containing_an_abstract_construct_signature_must_also_be_declared_abstract:n(2797,e.DiagnosticCategory.Error,"A_mixin_class_that_extends_from_a_type_variable_containing_an_abstract_construct_signature_must_also_2797","A mixin class that extends from a type variable containing an abstract construct signature must also be declared 'abstract'."),The_declaration_was_marked_as_deprecated_here:n(2798,e.DiagnosticCategory.Error,"The_declaration_was_marked_as_deprecated_here_2798","The declaration was marked as deprecated here."),Type_produces_a_tuple_type_that_is_too_large_to_represent:n(2799,e.DiagnosticCategory.Error,"Type_produces_a_tuple_type_that_is_too_large_to_represent_2799","Type produces a tuple type that is too large to represent."),Expression_produces_a_tuple_type_that_is_too_large_to_represent:n(2800,e.DiagnosticCategory.Error,"Expression_produces_a_tuple_type_that_is_too_large_to_represent_2800","Expression produces a tuple type that is too large to represent."),This_condition_will_always_return_true_since_this_0_is_always_defined:n(2801,e.DiagnosticCategory.Error,"This_condition_will_always_return_true_since_this_0_is_always_defined_2801","This condition will always return true since this '{0}' is always defined."),Type_0_can_only_be_iterated_through_when_using_the_downlevelIteration_flag_or_with_a_target_of_es2015_or_higher:n(2802,e.DiagnosticCategory.Error,"Type_0_can_only_be_iterated_through_when_using_the_downlevelIteration_flag_or_with_a_target_of_es201_2802","Type '{0}' can only be iterated through when using the '--downlevelIteration' flag or with a '--target' of 'es2015' or higher."),Cannot_assign_to_private_method_0_Private_methods_are_not_writable:n(2803,e.DiagnosticCategory.Error,"Cannot_assign_to_private_method_0_Private_methods_are_not_writable_2803","Cannot assign to private method '{0}'. Private methods are not writable."),Duplicate_identifier_0_Static_and_instance_elements_cannot_share_the_same_private_name:n(2804,e.DiagnosticCategory.Error,"Duplicate_identifier_0_Static_and_instance_elements_cannot_share_the_same_private_name_2804","Duplicate identifier '{0}'. Static and instance elements cannot share the same private name."),Static_fields_with_private_names_can_t_have_initializers_when_the_useDefineForClassFields_flag_is_not_specified_with_a_target_of_esnext_Consider_adding_the_useDefineForClassFields_flag:n(2805,e.DiagnosticCategory.Error,"Static_fields_with_private_names_can_t_have_initializers_when_the_useDefineForClassFields_flag_is_no_2805","Static fields with private names can't have initializers when the '--useDefineForClassFields' flag is not specified with a '--target' of 'esnext'. Consider adding the '--useDefineForClassFields' flag."),Private_accessor_was_defined_without_a_getter:n(2806,e.DiagnosticCategory.Error,"Private_accessor_was_defined_without_a_getter_2806","Private accessor was defined without a getter."),This_syntax_requires_an_imported_helper_named_1_with_2_parameters_which_is_not_compatible_with_the_one_in_0_Consider_upgrading_your_version_of_0:n(2807,e.DiagnosticCategory.Error,"This_syntax_requires_an_imported_helper_named_1_with_2_parameters_which_is_not_compatible_with_the_o_2807","This syntax requires an imported helper named '{1}' with {2} parameters, which is not compatible with the one in '{0}'. Consider upgrading your version of '{0}'."),A_get_accessor_must_be_at_least_as_accessible_as_the_setter:n(2808,e.DiagnosticCategory.Error,"A_get_accessor_must_be_at_least_as_accessible_as_the_setter_2808","A get accessor must be at least as accessible as the setter"),Declaration_or_statement_expected_This_follows_a_block_of_statements_so_if_you_intended_to_write_a_destructuring_assignment_you_might_need_to_wrap_the_the_whole_assignment_in_parentheses:n(2809,e.DiagnosticCategory.Error,"Declaration_or_statement_expected_This_follows_a_block_of_statements_so_if_you_intended_to_write_a_d_2809","Declaration or statement expected. This '=' follows a block of statements, so if you intended to write a destructuring assignment, you might need to wrap the the whole assignment in parentheses."),Property_0_may_not_be_used_in_a_static_property_s_initializer_in_the_same_class_when_target_is_esnext_and_useDefineForClassFields_is_false:n(2810,e.DiagnosticCategory.Error,"Property_0_may_not_be_used_in_a_static_property_s_initializer_in_the_same_class_when_target_is_esnex_2810","Property '{0}' may not be used in a static property's initializer in the same class when 'target' is 'esnext' and 'useDefineForClassFields' is 'false'."),Initializer_for_property_0:n(2811,e.DiagnosticCategory.Error,"Initializer_for_property_0_2811","Initializer for property '{0}'"),Property_0_does_not_exist_on_type_1_Try_changing_the_lib_compiler_option_to_include_dom:n(2812,e.DiagnosticCategory.Error,"Property_0_does_not_exist_on_type_1_Try_changing_the_lib_compiler_option_to_include_dom_2812","Property '{0}' does not exist on type '{1}'. Try changing the 'lib' compiler option to include 'dom'."),Class_declaration_cannot_implement_overload_list_for_0:n(2813,e.DiagnosticCategory.Error,"Class_declaration_cannot_implement_overload_list_for_0_2813","Class declaration cannot implement overload list for '{0}'."),Function_with_bodies_can_only_merge_with_classes_that_are_ambient:n(2814,e.DiagnosticCategory.Error,"Function_with_bodies_can_only_merge_with_classes_that_are_ambient_2814","Function with bodies can only merge with classes that are ambient."),arguments_cannot_be_referenced_in_property_initializers:n(2815,e.DiagnosticCategory.Error,"arguments_cannot_be_referenced_in_property_initializers_2815","'arguments' cannot be referenced in property initializers."),Cannot_use_this_in_a_static_property_initializer_of_a_decorated_class:n(2816,e.DiagnosticCategory.Error,"Cannot_use_this_in_a_static_property_initializer_of_a_decorated_class_2816","Cannot use 'this' in a static property initializer of a decorated class."),Property_0_has_no_initializer_and_is_not_definitely_assigned_in_a_class_static_block:n(2817,e.DiagnosticCategory.Error,"Property_0_has_no_initializer_and_is_not_definitely_assigned_in_a_class_static_block_2817","Property '{0}' has no initializer and is not definitely assigned in a class static block."),Duplicate_identifier_0_Compiler_reserves_name_1_when_emitting_super_references_in_static_initializers:n(2818,e.DiagnosticCategory.Error,"Duplicate_identifier_0_Compiler_reserves_name_1_when_emitting_super_references_in_static_initializer_2818","Duplicate identifier '{0}'. Compiler reserves name '{1}' when emitting 'super' references in static initializers."),Namespace_name_cannot_be_0:n(2819,e.DiagnosticCategory.Error,"Namespace_name_cannot_be_0_2819","Namespace name cannot be '{0}'."),Import_declaration_0_is_using_private_name_1:n(4e3,e.DiagnosticCategory.Error,"Import_declaration_0_is_using_private_name_1_4000","Import declaration '{0}' is using private name '{1}'."),Type_parameter_0_of_exported_class_has_or_is_using_private_name_1:n(4002,e.DiagnosticCategory.Error,"Type_parameter_0_of_exported_class_has_or_is_using_private_name_1_4002","Type parameter '{0}' of exported class has or is using private name '{1}'."),Type_parameter_0_of_exported_interface_has_or_is_using_private_name_1:n(4004,e.DiagnosticCategory.Error,"Type_parameter_0_of_exported_interface_has_or_is_using_private_name_1_4004","Type parameter '{0}' of exported interface has or is using private name '{1}'."),Type_parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1:n(4006,e.DiagnosticCategory.Error,"Type_parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1_4006","Type parameter '{0}' of constructor signature from exported interface has or is using private name '{1}'."),Type_parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1:n(4008,e.DiagnosticCategory.Error,"Type_parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1_4008","Type parameter '{0}' of call signature from exported interface has or is using private name '{1}'."),Type_parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1:n(4010,e.DiagnosticCategory.Error,"Type_parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1_4010","Type parameter '{0}' of public static method from exported class has or is using private name '{1}'."),Type_parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1:n(4012,e.DiagnosticCategory.Error,"Type_parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1_4012","Type parameter '{0}' of public method from exported class has or is using private name '{1}'."),Type_parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1:n(4014,e.DiagnosticCategory.Error,"Type_parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1_4014","Type parameter '{0}' of method from exported interface has or is using private name '{1}'."),Type_parameter_0_of_exported_function_has_or_is_using_private_name_1:n(4016,e.DiagnosticCategory.Error,"Type_parameter_0_of_exported_function_has_or_is_using_private_name_1_4016","Type parameter '{0}' of exported function has or is using private name '{1}'."),Implements_clause_of_exported_class_0_has_or_is_using_private_name_1:n(4019,e.DiagnosticCategory.Error,"Implements_clause_of_exported_class_0_has_or_is_using_private_name_1_4019","Implements clause of exported class '{0}' has or is using private name '{1}'."),extends_clause_of_exported_class_0_has_or_is_using_private_name_1:n(4020,e.DiagnosticCategory.Error,"extends_clause_of_exported_class_0_has_or_is_using_private_name_1_4020","'extends' clause of exported class '{0}' has or is using private name '{1}'."),extends_clause_of_exported_class_has_or_is_using_private_name_0:n(4021,e.DiagnosticCategory.Error,"extends_clause_of_exported_class_has_or_is_using_private_name_0_4021","'extends' clause of exported class has or is using private name '{0}'."),extends_clause_of_exported_interface_0_has_or_is_using_private_name_1:n(4022,e.DiagnosticCategory.Error,"extends_clause_of_exported_interface_0_has_or_is_using_private_name_1_4022","'extends' clause of exported interface '{0}' has or is using private name '{1}'."),Exported_variable_0_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4023,e.DiagnosticCategory.Error,"Exported_variable_0_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4023","Exported variable '{0}' has or is using name '{1}' from external module {2} but cannot be named."),Exported_variable_0_has_or_is_using_name_1_from_private_module_2:n(4024,e.DiagnosticCategory.Error,"Exported_variable_0_has_or_is_using_name_1_from_private_module_2_4024","Exported variable '{0}' has or is using name '{1}' from private module '{2}'."),Exported_variable_0_has_or_is_using_private_name_1:n(4025,e.DiagnosticCategory.Error,"Exported_variable_0_has_or_is_using_private_name_1_4025","Exported variable '{0}' has or is using private name '{1}'."),Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4026,e.DiagnosticCategory.Error,"Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot__4026","Public static property '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named."),Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:n(4027,e.DiagnosticCategory.Error,"Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4027","Public static property '{0}' of exported class has or is using name '{1}' from private module '{2}'."),Public_static_property_0_of_exported_class_has_or_is_using_private_name_1:n(4028,e.DiagnosticCategory.Error,"Public_static_property_0_of_exported_class_has_or_is_using_private_name_1_4028","Public static property '{0}' of exported class has or is using private name '{1}'."),Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4029,e.DiagnosticCategory.Error,"Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_name_4029","Public property '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named."),Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:n(4030,e.DiagnosticCategory.Error,"Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4030","Public property '{0}' of exported class has or is using name '{1}' from private module '{2}'."),Public_property_0_of_exported_class_has_or_is_using_private_name_1:n(4031,e.DiagnosticCategory.Error,"Public_property_0_of_exported_class_has_or_is_using_private_name_1_4031","Public property '{0}' of exported class has or is using private name '{1}'."),Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4032,e.DiagnosticCategory.Error,"Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2_4032","Property '{0}' of exported interface has or is using name '{1}' from private module '{2}'."),Property_0_of_exported_interface_has_or_is_using_private_name_1:n(4033,e.DiagnosticCategory.Error,"Property_0_of_exported_interface_has_or_is_using_private_name_1_4033","Property '{0}' of exported interface has or is using private name '{1}'."),Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4034,e.DiagnosticCategory.Error,"Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_name_1_from_private_mod_4034","Parameter type of public static setter '{0}' from exported class has or is using name '{1}' from private module '{2}'."),Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_private_name_1:n(4035,e.DiagnosticCategory.Error,"Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_private_name_1_4035","Parameter type of public static setter '{0}' from exported class has or is using private name '{1}'."),Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4036,e.DiagnosticCategory.Error,"Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2_4036","Parameter type of public setter '{0}' from exported class has or is using name '{1}' from private module '{2}'."),Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_private_name_1:n(4037,e.DiagnosticCategory.Error,"Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_private_name_1_4037","Parameter type of public setter '{0}' from exported class has or is using private name '{1}'."),Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4038,e.DiagnosticCategory.Error,"Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_external_modul_4038","Return type of public static getter '{0}' from exported class has or is using name '{1}' from external module {2} but cannot be named."),Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4039,e.DiagnosticCategory.Error,"Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_4039","Return type of public static getter '{0}' from exported class has or is using name '{1}' from private module '{2}'."),Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_private_name_1:n(4040,e.DiagnosticCategory.Error,"Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_private_name_1_4040","Return type of public static getter '{0}' from exported class has or is using private name '{1}'."),Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4041,e.DiagnosticCategory.Error,"Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_4041","Return type of public getter '{0}' from exported class has or is using name '{1}' from external module {2} but cannot be named."),Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4042,e.DiagnosticCategory.Error,"Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2_4042","Return type of public getter '{0}' from exported class has or is using name '{1}' from private module '{2}'."),Return_type_of_public_getter_0_from_exported_class_has_or_is_using_private_name_1:n(4043,e.DiagnosticCategory.Error,"Return_type_of_public_getter_0_from_exported_class_has_or_is_using_private_name_1_4043","Return type of public getter '{0}' from exported class has or is using private name '{1}'."),Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:n(4044,e.DiagnosticCategory.Error,"Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_name_0_from_private_mod_4044","Return type of constructor signature from exported interface has or is using name '{0}' from private module '{1}'."),Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_0:n(4045,e.DiagnosticCategory.Error,"Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_0_4045","Return type of constructor signature from exported interface has or is using private name '{0}'."),Return_type_of_call_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:n(4046,e.DiagnosticCategory.Error,"Return_type_of_call_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1_4046","Return type of call signature from exported interface has or is using name '{0}' from private module '{1}'."),Return_type_of_call_signature_from_exported_interface_has_or_is_using_private_name_0:n(4047,e.DiagnosticCategory.Error,"Return_type_of_call_signature_from_exported_interface_has_or_is_using_private_name_0_4047","Return type of call signature from exported interface has or is using private name '{0}'."),Return_type_of_index_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:n(4048,e.DiagnosticCategory.Error,"Return_type_of_index_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1_4048","Return type of index signature from exported interface has or is using name '{0}' from private module '{1}'."),Return_type_of_index_signature_from_exported_interface_has_or_is_using_private_name_0:n(4049,e.DiagnosticCategory.Error,"Return_type_of_index_signature_from_exported_interface_has_or_is_using_private_name_0_4049","Return type of index signature from exported interface has or is using private name '{0}'."),Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:n(4050,e.DiagnosticCategory.Error,"Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_external_module__4050","Return type of public static method from exported class has or is using name '{0}' from external module {1} but cannot be named."),Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_private_module_1:n(4051,e.DiagnosticCategory.Error,"Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_private_module_1_4051","Return type of public static method from exported class has or is using name '{0}' from private module '{1}'."),Return_type_of_public_static_method_from_exported_class_has_or_is_using_private_name_0:n(4052,e.DiagnosticCategory.Error,"Return_type_of_public_static_method_from_exported_class_has_or_is_using_private_name_0_4052","Return type of public static method from exported class has or is using private name '{0}'."),Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:n(4053,e.DiagnosticCategory.Error,"Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_c_4053","Return type of public method from exported class has or is using name '{0}' from external module {1} but cannot be named."),Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_private_module_1:n(4054,e.DiagnosticCategory.Error,"Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_private_module_1_4054","Return type of public method from exported class has or is using name '{0}' from private module '{1}'."),Return_type_of_public_method_from_exported_class_has_or_is_using_private_name_0:n(4055,e.DiagnosticCategory.Error,"Return_type_of_public_method_from_exported_class_has_or_is_using_private_name_0_4055","Return type of public method from exported class has or is using private name '{0}'."),Return_type_of_method_from_exported_interface_has_or_is_using_name_0_from_private_module_1:n(4056,e.DiagnosticCategory.Error,"Return_type_of_method_from_exported_interface_has_or_is_using_name_0_from_private_module_1_4056","Return type of method from exported interface has or is using name '{0}' from private module '{1}'."),Return_type_of_method_from_exported_interface_has_or_is_using_private_name_0:n(4057,e.DiagnosticCategory.Error,"Return_type_of_method_from_exported_interface_has_or_is_using_private_name_0_4057","Return type of method from exported interface has or is using private name '{0}'."),Return_type_of_exported_function_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:n(4058,e.DiagnosticCategory.Error,"Return_type_of_exported_function_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named_4058","Return type of exported function has or is using name '{0}' from external module {1} but cannot be named."),Return_type_of_exported_function_has_or_is_using_name_0_from_private_module_1:n(4059,e.DiagnosticCategory.Error,"Return_type_of_exported_function_has_or_is_using_name_0_from_private_module_1_4059","Return type of exported function has or is using name '{0}' from private module '{1}'."),Return_type_of_exported_function_has_or_is_using_private_name_0:n(4060,e.DiagnosticCategory.Error,"Return_type_of_exported_function_has_or_is_using_private_name_0_4060","Return type of exported function has or is using private name '{0}'."),Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4061,e.DiagnosticCategory.Error,"Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_can_4061","Parameter '{0}' of constructor from exported class has or is using name '{1}' from external module {2} but cannot be named."),Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4062,e.DiagnosticCategory.Error,"Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_private_module_2_4062","Parameter '{0}' of constructor from exported class has or is using name '{1}' from private module '{2}'."),Parameter_0_of_constructor_from_exported_class_has_or_is_using_private_name_1:n(4063,e.DiagnosticCategory.Error,"Parameter_0_of_constructor_from_exported_class_has_or_is_using_private_name_1_4063","Parameter '{0}' of constructor from exported class has or is using private name '{1}'."),Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4064,e.DiagnosticCategory.Error,"Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_name_1_from_private_mod_4064","Parameter '{0}' of constructor signature from exported interface has or is using name '{1}' from private module '{2}'."),Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1:n(4065,e.DiagnosticCategory.Error,"Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1_4065","Parameter '{0}' of constructor signature from exported interface has or is using private name '{1}'."),Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4066,e.DiagnosticCategory.Error,"Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2_4066","Parameter '{0}' of call signature from exported interface has or is using name '{1}' from private module '{2}'."),Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1:n(4067,e.DiagnosticCategory.Error,"Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1_4067","Parameter '{0}' of call signature from exported interface has or is using private name '{1}'."),Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4068,e.DiagnosticCategory.Error,"Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_external_module__4068","Parameter '{0}' of public static method from exported class has or is using name '{1}' from external module {2} but cannot be named."),Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4069,e.DiagnosticCategory.Error,"Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_private_module_2_4069","Parameter '{0}' of public static method from exported class has or is using name '{1}' from private module '{2}'."),Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1:n(4070,e.DiagnosticCategory.Error,"Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1_4070","Parameter '{0}' of public static method from exported class has or is using private name '{1}'."),Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4071,e.DiagnosticCategory.Error,"Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_c_4071","Parameter '{0}' of public method from exported class has or is using name '{1}' from external module {2} but cannot be named."),Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_private_module_2:n(4072,e.DiagnosticCategory.Error,"Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_private_module_2_4072","Parameter '{0}' of public method from exported class has or is using name '{1}' from private module '{2}'."),Parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1:n(4073,e.DiagnosticCategory.Error,"Parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1_4073","Parameter '{0}' of public method from exported class has or is using private name '{1}'."),Parameter_0_of_method_from_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4074,e.DiagnosticCategory.Error,"Parameter_0_of_method_from_exported_interface_has_or_is_using_name_1_from_private_module_2_4074","Parameter '{0}' of method from exported interface has or is using name '{1}' from private module '{2}'."),Parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1:n(4075,e.DiagnosticCategory.Error,"Parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1_4075","Parameter '{0}' of method from exported interface has or is using private name '{1}'."),Parameter_0_of_exported_function_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4076,e.DiagnosticCategory.Error,"Parameter_0_of_exported_function_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4076","Parameter '{0}' of exported function has or is using name '{1}' from external module {2} but cannot be named."),Parameter_0_of_exported_function_has_or_is_using_name_1_from_private_module_2:n(4077,e.DiagnosticCategory.Error,"Parameter_0_of_exported_function_has_or_is_using_name_1_from_private_module_2_4077","Parameter '{0}' of exported function has or is using name '{1}' from private module '{2}'."),Parameter_0_of_exported_function_has_or_is_using_private_name_1:n(4078,e.DiagnosticCategory.Error,"Parameter_0_of_exported_function_has_or_is_using_private_name_1_4078","Parameter '{0}' of exported function has or is using private name '{1}'."),Exported_type_alias_0_has_or_is_using_private_name_1:n(4081,e.DiagnosticCategory.Error,"Exported_type_alias_0_has_or_is_using_private_name_1_4081","Exported type alias '{0}' has or is using private name '{1}'."),Default_export_of_the_module_has_or_is_using_private_name_0:n(4082,e.DiagnosticCategory.Error,"Default_export_of_the_module_has_or_is_using_private_name_0_4082","Default export of the module has or is using private name '{0}'."),Type_parameter_0_of_exported_type_alias_has_or_is_using_private_name_1:n(4083,e.DiagnosticCategory.Error,"Type_parameter_0_of_exported_type_alias_has_or_is_using_private_name_1_4083","Type parameter '{0}' of exported type alias has or is using private name '{1}'."),Exported_type_alias_0_has_or_is_using_private_name_1_from_module_2:n(4084,e.DiagnosticCategory.Error,"Exported_type_alias_0_has_or_is_using_private_name_1_from_module_2_4084","Exported type alias '{0}' has or is using private name '{1}' from module {2}."),Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_library_to_resolve_the_conflict:n(4090,e.DiagnosticCategory.Error,"Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_librar_4090","Conflicting definitions for '{0}' found at '{1}' and '{2}'. Consider installing a specific version of this library to resolve the conflict."),Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4091,e.DiagnosticCategory.Error,"Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2_4091","Parameter '{0}' of index signature from exported interface has or is using name '{1}' from private module '{2}'."),Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_private_name_1:n(4092,e.DiagnosticCategory.Error,"Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_private_name_1_4092","Parameter '{0}' of index signature from exported interface has or is using private name '{1}'."),Property_0_of_exported_class_expression_may_not_be_private_or_protected:n(4094,e.DiagnosticCategory.Error,"Property_0_of_exported_class_expression_may_not_be_private_or_protected_4094","Property '{0}' of exported class expression may not be private or protected."),Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4095,e.DiagnosticCategory.Error,"Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_4095","Public static method '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named."),Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:n(4096,e.DiagnosticCategory.Error,"Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4096","Public static method '{0}' of exported class has or is using name '{1}' from private module '{2}'."),Public_static_method_0_of_exported_class_has_or_is_using_private_name_1:n(4097,e.DiagnosticCategory.Error,"Public_static_method_0_of_exported_class_has_or_is_using_private_name_1_4097","Public static method '{0}' of exported class has or is using private name '{1}'."),Public_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4098,e.DiagnosticCategory.Error,"Public_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4098","Public method '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named."),Public_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:n(4099,e.DiagnosticCategory.Error,"Public_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4099","Public method '{0}' of exported class has or is using name '{1}' from private module '{2}'."),Public_method_0_of_exported_class_has_or_is_using_private_name_1:n(4100,e.DiagnosticCategory.Error,"Public_method_0_of_exported_class_has_or_is_using_private_name_1_4100","Public method '{0}' of exported class has or is using private name '{1}'."),Method_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:n(4101,e.DiagnosticCategory.Error,"Method_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2_4101","Method '{0}' of exported interface has or is using name '{1}' from private module '{2}'."),Method_0_of_exported_interface_has_or_is_using_private_name_1:n(4102,e.DiagnosticCategory.Error,"Method_0_of_exported_interface_has_or_is_using_private_name_1_4102","Method '{0}' of exported interface has or is using private name '{1}'."),Type_parameter_0_of_exported_mapped_object_type_is_using_private_name_1:n(4103,e.DiagnosticCategory.Error,"Type_parameter_0_of_exported_mapped_object_type_is_using_private_name_1_4103","Type parameter '{0}' of exported mapped object type is using private name '{1}'."),The_type_0_is_readonly_and_cannot_be_assigned_to_the_mutable_type_1:n(4104,e.DiagnosticCategory.Error,"The_type_0_is_readonly_and_cannot_be_assigned_to_the_mutable_type_1_4104","The type '{0}' is 'readonly' and cannot be assigned to the mutable type '{1}'."),Private_or_protected_member_0_cannot_be_accessed_on_a_type_parameter:n(4105,e.DiagnosticCategory.Error,"Private_or_protected_member_0_cannot_be_accessed_on_a_type_parameter_4105","Private or protected member '{0}' cannot be accessed on a type parameter."),Parameter_0_of_accessor_has_or_is_using_private_name_1:n(4106,e.DiagnosticCategory.Error,"Parameter_0_of_accessor_has_or_is_using_private_name_1_4106","Parameter '{0}' of accessor has or is using private name '{1}'."),Parameter_0_of_accessor_has_or_is_using_name_1_from_private_module_2:n(4107,e.DiagnosticCategory.Error,"Parameter_0_of_accessor_has_or_is_using_name_1_from_private_module_2_4107","Parameter '{0}' of accessor has or is using name '{1}' from private module '{2}'."),Parameter_0_of_accessor_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:n(4108,e.DiagnosticCategory.Error,"Parameter_0_of_accessor_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4108","Parameter '{0}' of accessor has or is using name '{1}' from external module '{2}' but cannot be named."),Type_arguments_for_0_circularly_reference_themselves:n(4109,e.DiagnosticCategory.Error,"Type_arguments_for_0_circularly_reference_themselves_4109","Type arguments for '{0}' circularly reference themselves."),Tuple_type_arguments_circularly_reference_themselves:n(4110,e.DiagnosticCategory.Error,"Tuple_type_arguments_circularly_reference_themselves_4110","Tuple type arguments circularly reference themselves."),Property_0_comes_from_an_index_signature_so_it_must_be_accessed_with_0:n(4111,e.DiagnosticCategory.Error,"Property_0_comes_from_an_index_signature_so_it_must_be_accessed_with_0_4111","Property '{0}' comes from an index signature, so it must be accessed with ['{0}']."),This_member_cannot_have_an_override_modifier_because_its_containing_class_0_does_not_extend_another_class:n(4112,e.DiagnosticCategory.Error,"This_member_cannot_have_an_override_modifier_because_its_containing_class_0_does_not_extend_another__4112","This member cannot have an 'override' modifier because its containing class '{0}' does not extend another class."),This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0:n(4113,e.DiagnosticCategory.Error,"This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0_4113","This member cannot have an 'override' modifier because it is not declared in the base class '{0}'."),This_member_must_have_an_override_modifier_because_it_overrides_a_member_in_the_base_class_0:n(4114,e.DiagnosticCategory.Error,"This_member_must_have_an_override_modifier_because_it_overrides_a_member_in_the_base_class_0_4114","This member must have an 'override' modifier because it overrides a member in the base class '{0}'."),This_parameter_property_must_have_an_override_modifier_because_it_overrides_a_member_in_base_class_0:n(4115,e.DiagnosticCategory.Error,"This_parameter_property_must_have_an_override_modifier_because_it_overrides_a_member_in_base_class_0_4115","This parameter property must have an 'override' modifier because it overrides a member in base class '{0}'."),This_member_must_have_an_override_modifier_because_it_overrides_an_abstract_method_that_is_declared_in_the_base_class_0:n(4116,e.DiagnosticCategory.Error,"This_member_must_have_an_override_modifier_because_it_overrides_an_abstract_method_that_is_declared__4116","This member must have an 'override' modifier because it overrides an abstract method that is declared in the base class '{0}'."),This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0_Did_you_mean_1:n(4117,e.DiagnosticCategory.Error,"This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0_Did_you__4117","This member cannot have an 'override' modifier because it is not declared in the base class '{0}'. Did you mean '{1}'?"),The_type_of_this_node_cannot_be_serialized_because_its_property_0_cannot_be_serialized:n(4118,e.DiagnosticCategory.Error,"The_type_of_this_node_cannot_be_serialized_because_its_property_0_cannot_be_serialized_4118","The type of this node cannot be serialized because its property '{0}' cannot be serialized."),The_current_host_does_not_support_the_0_option:n(5001,e.DiagnosticCategory.Error,"The_current_host_does_not_support_the_0_option_5001","The current host does not support the '{0}' option."),Cannot_find_the_common_subdirectory_path_for_the_input_files:n(5009,e.DiagnosticCategory.Error,"Cannot_find_the_common_subdirectory_path_for_the_input_files_5009","Cannot find the common subdirectory path for the input files."),File_specification_cannot_end_in_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0:n(5010,e.DiagnosticCategory.Error,"File_specification_cannot_end_in_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0_5010","File specification cannot end in a recursive directory wildcard ('**'): '{0}'."),Cannot_read_file_0_Colon_1:n(5012,e.DiagnosticCategory.Error,"Cannot_read_file_0_Colon_1_5012","Cannot read file '{0}': {1}."),Failed_to_parse_file_0_Colon_1:n(5014,e.DiagnosticCategory.Error,"Failed_to_parse_file_0_Colon_1_5014","Failed to parse file '{0}': {1}."),Unknown_compiler_option_0:n(5023,e.DiagnosticCategory.Error,"Unknown_compiler_option_0_5023","Unknown compiler option '{0}'."),Compiler_option_0_requires_a_value_of_type_1:n(5024,e.DiagnosticCategory.Error,"Compiler_option_0_requires_a_value_of_type_1_5024","Compiler option '{0}' requires a value of type {1}."),Unknown_compiler_option_0_Did_you_mean_1:n(5025,e.DiagnosticCategory.Error,"Unknown_compiler_option_0_Did_you_mean_1_5025","Unknown compiler option '{0}'. Did you mean '{1}'?"),Could_not_write_file_0_Colon_1:n(5033,e.DiagnosticCategory.Error,"Could_not_write_file_0_Colon_1_5033","Could not write file '{0}': {1}."),Option_project_cannot_be_mixed_with_source_files_on_a_command_line:n(5042,e.DiagnosticCategory.Error,"Option_project_cannot_be_mixed_with_source_files_on_a_command_line_5042","Option 'project' cannot be mixed with source files on a command line."),Option_isolatedModules_can_only_be_used_when_either_option_module_is_provided_or_option_target_is_ES2015_or_higher:n(5047,e.DiagnosticCategory.Error,"Option_isolatedModules_can_only_be_used_when_either_option_module_is_provided_or_option_target_is_ES_5047","Option 'isolatedModules' can only be used when either option '--module' is provided or option 'target' is 'ES2015' or higher."),Option_0_cannot_be_specified_when_option_target_is_ES3:n(5048,e.DiagnosticCategory.Error,"Option_0_cannot_be_specified_when_option_target_is_ES3_5048","Option '{0}' cannot be specified when option 'target' is 'ES3'."),Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided:n(5051,e.DiagnosticCategory.Error,"Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided_5051","Option '{0} can only be used when either option '--inlineSourceMap' or option '--sourceMap' is provided."),Option_0_cannot_be_specified_without_specifying_option_1:n(5052,e.DiagnosticCategory.Error,"Option_0_cannot_be_specified_without_specifying_option_1_5052","Option '{0}' cannot be specified without specifying option '{1}'."),Option_0_cannot_be_specified_with_option_1:n(5053,e.DiagnosticCategory.Error,"Option_0_cannot_be_specified_with_option_1_5053","Option '{0}' cannot be specified with option '{1}'."),A_tsconfig_json_file_is_already_defined_at_Colon_0:n(5054,e.DiagnosticCategory.Error,"A_tsconfig_json_file_is_already_defined_at_Colon_0_5054","A 'tsconfig.json' file is already defined at: '{0}'."),Cannot_write_file_0_because_it_would_overwrite_input_file:n(5055,e.DiagnosticCategory.Error,"Cannot_write_file_0_because_it_would_overwrite_input_file_5055","Cannot write file '{0}' because it would overwrite input file."),Cannot_write_file_0_because_it_would_be_overwritten_by_multiple_input_files:n(5056,e.DiagnosticCategory.Error,"Cannot_write_file_0_because_it_would_be_overwritten_by_multiple_input_files_5056","Cannot write file '{0}' because it would be overwritten by multiple input files."),Cannot_find_a_tsconfig_json_file_at_the_specified_directory_Colon_0:n(5057,e.DiagnosticCategory.Error,"Cannot_find_a_tsconfig_json_file_at_the_specified_directory_Colon_0_5057","Cannot find a tsconfig.json file at the specified directory: '{0}'."),The_specified_path_does_not_exist_Colon_0:n(5058,e.DiagnosticCategory.Error,"The_specified_path_does_not_exist_Colon_0_5058","The specified path does not exist: '{0}'."),Invalid_value_for_reactNamespace_0_is_not_a_valid_identifier:n(5059,e.DiagnosticCategory.Error,"Invalid_value_for_reactNamespace_0_is_not_a_valid_identifier_5059","Invalid value for '--reactNamespace'. '{0}' is not a valid identifier."),Pattern_0_can_have_at_most_one_Asterisk_character:n(5061,e.DiagnosticCategory.Error,"Pattern_0_can_have_at_most_one_Asterisk_character_5061","Pattern '{0}' can have at most one '*' character."),Substitution_0_in_pattern_1_can_have_at_most_one_Asterisk_character:n(5062,e.DiagnosticCategory.Error,"Substitution_0_in_pattern_1_can_have_at_most_one_Asterisk_character_5062","Substitution '{0}' in pattern '{1}' can have at most one '*' character."),Substitutions_for_pattern_0_should_be_an_array:n(5063,e.DiagnosticCategory.Error,"Substitutions_for_pattern_0_should_be_an_array_5063","Substitutions for pattern '{0}' should be an array."),Substitution_0_for_pattern_1_has_incorrect_type_expected_string_got_2:n(5064,e.DiagnosticCategory.Error,"Substitution_0_for_pattern_1_has_incorrect_type_expected_string_got_2_5064","Substitution '{0}' for pattern '{1}' has incorrect type, expected 'string', got '{2}'."),File_specification_cannot_contain_a_parent_directory_that_appears_after_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0:n(5065,e.DiagnosticCategory.Error,"File_specification_cannot_contain_a_parent_directory_that_appears_after_a_recursive_directory_wildca_5065","File specification cannot contain a parent directory ('..') that appears after a recursive directory wildcard ('**'): '{0}'."),Substitutions_for_pattern_0_shouldn_t_be_an_empty_array:n(5066,e.DiagnosticCategory.Error,"Substitutions_for_pattern_0_shouldn_t_be_an_empty_array_5066","Substitutions for pattern '{0}' shouldn't be an empty array."),Invalid_value_for_jsxFactory_0_is_not_a_valid_identifier_or_qualified_name:n(5067,e.DiagnosticCategory.Error,"Invalid_value_for_jsxFactory_0_is_not_a_valid_identifier_or_qualified_name_5067","Invalid value for 'jsxFactory'. '{0}' is not a valid identifier or qualified-name."),Adding_a_tsconfig_json_file_will_help_organize_projects_that_contain_both_TypeScript_and_JavaScript_files_Learn_more_at_https_Colon_Slash_Slashaka_ms_Slashtsconfig:n(5068,e.DiagnosticCategory.Error,"Adding_a_tsconfig_json_file_will_help_organize_projects_that_contain_both_TypeScript_and_JavaScript__5068","Adding a tsconfig.json file will help organize projects that contain both TypeScript and JavaScript files. Learn more at https://aka.ms/tsconfig."),Option_0_cannot_be_specified_without_specifying_option_1_or_option_2:n(5069,e.DiagnosticCategory.Error,"Option_0_cannot_be_specified_without_specifying_option_1_or_option_2_5069","Option '{0}' cannot be specified without specifying option '{1}' or option '{2}'."),Option_resolveJsonModule_cannot_be_specified_without_node_module_resolution_strategy:n(5070,e.DiagnosticCategory.Error,"Option_resolveJsonModule_cannot_be_specified_without_node_module_resolution_strategy_5070","Option '--resolveJsonModule' cannot be specified without 'node' module resolution strategy."),Option_resolveJsonModule_can_only_be_specified_when_module_code_generation_is_commonjs_amd_es2015_or_esNext:n(5071,e.DiagnosticCategory.Error,"Option_resolveJsonModule_can_only_be_specified_when_module_code_generation_is_commonjs_amd_es2015_or_5071","Option '--resolveJsonModule' can only be specified when module code generation is 'commonjs', 'amd', 'es2015' or 'esNext'."),Unknown_build_option_0:n(5072,e.DiagnosticCategory.Error,"Unknown_build_option_0_5072","Unknown build option '{0}'."),Build_option_0_requires_a_value_of_type_1:n(5073,e.DiagnosticCategory.Error,"Build_option_0_requires_a_value_of_type_1_5073","Build option '{0}' requires a value of type {1}."),Option_incremental_can_only_be_specified_using_tsconfig_emitting_to_single_file_or_when_option_tsBuildInfoFile_is_specified:n(5074,e.DiagnosticCategory.Error,"Option_incremental_can_only_be_specified_using_tsconfig_emitting_to_single_file_or_when_option_tsBui_5074","Option '--incremental' can only be specified using tsconfig, emitting to single file or when option '--tsBuildInfoFile' is specified."),_0_is_assignable_to_the_constraint_of_type_1_but_1_could_be_instantiated_with_a_different_subtype_of_constraint_2:n(5075,e.DiagnosticCategory.Error,"_0_is_assignable_to_the_constraint_of_type_1_but_1_could_be_instantiated_with_a_different_subtype_of_5075","'{0}' is assignable to the constraint of type '{1}', but '{1}' could be instantiated with a different subtype of constraint '{2}'."),_0_and_1_operations_cannot_be_mixed_without_parentheses:n(5076,e.DiagnosticCategory.Error,"_0_and_1_operations_cannot_be_mixed_without_parentheses_5076","'{0}' and '{1}' operations cannot be mixed without parentheses."),Unknown_build_option_0_Did_you_mean_1:n(5077,e.DiagnosticCategory.Error,"Unknown_build_option_0_Did_you_mean_1_5077","Unknown build option '{0}'. Did you mean '{1}'?"),Unknown_watch_option_0:n(5078,e.DiagnosticCategory.Error,"Unknown_watch_option_0_5078","Unknown watch option '{0}'."),Unknown_watch_option_0_Did_you_mean_1:n(5079,e.DiagnosticCategory.Error,"Unknown_watch_option_0_Did_you_mean_1_5079","Unknown watch option '{0}'. Did you mean '{1}'?"),Watch_option_0_requires_a_value_of_type_1:n(5080,e.DiagnosticCategory.Error,"Watch_option_0_requires_a_value_of_type_1_5080","Watch option '{0}' requires a value of type {1}."),Cannot_find_a_tsconfig_json_file_at_the_current_directory_Colon_0:n(5081,e.DiagnosticCategory.Error,"Cannot_find_a_tsconfig_json_file_at_the_current_directory_Colon_0_5081","Cannot find a tsconfig.json file at the current directory: {0}."),_0_could_be_instantiated_with_an_arbitrary_type_which_could_be_unrelated_to_1:n(5082,e.DiagnosticCategory.Error,"_0_could_be_instantiated_with_an_arbitrary_type_which_could_be_unrelated_to_1_5082","'{0}' could be instantiated with an arbitrary type which could be unrelated to '{1}'."),Cannot_read_file_0:n(5083,e.DiagnosticCategory.Error,"Cannot_read_file_0_5083","Cannot read file '{0}'."),Tuple_members_must_all_have_names_or_all_not_have_names:n(5084,e.DiagnosticCategory.Error,"Tuple_members_must_all_have_names_or_all_not_have_names_5084","Tuple members must all have names or all not have names."),A_tuple_member_cannot_be_both_optional_and_rest:n(5085,e.DiagnosticCategory.Error,"A_tuple_member_cannot_be_both_optional_and_rest_5085","A tuple member cannot be both optional and rest."),A_labeled_tuple_element_is_declared_as_optional_with_a_question_mark_after_the_name_and_before_the_colon_rather_than_after_the_type:n(5086,e.DiagnosticCategory.Error,"A_labeled_tuple_element_is_declared_as_optional_with_a_question_mark_after_the_name_and_before_the_c_5086","A labeled tuple element is declared as optional with a question mark after the name and before the colon, rather than after the type."),A_labeled_tuple_element_is_declared_as_rest_with_a_before_the_name_rather_than_before_the_type:n(5087,e.DiagnosticCategory.Error,"A_labeled_tuple_element_is_declared_as_rest_with_a_before_the_name_rather_than_before_the_type_5087","A labeled tuple element is declared as rest with a '...' before the name, rather than before the type."),The_inferred_type_of_0_references_a_type_with_a_cyclic_structure_which_cannot_be_trivially_serialized_A_type_annotation_is_necessary:n(5088,e.DiagnosticCategory.Error,"The_inferred_type_of_0_references_a_type_with_a_cyclic_structure_which_cannot_be_trivially_serialize_5088","The inferred type of '{0}' references a type with a cyclic structure which cannot be trivially serialized. A type annotation is necessary."),Option_0_cannot_be_specified_when_option_jsx_is_1:n(5089,e.DiagnosticCategory.Error,"Option_0_cannot_be_specified_when_option_jsx_is_1_5089","Option '{0}' cannot be specified when option 'jsx' is '{1}'."),Non_relative_paths_are_not_allowed_when_baseUrl_is_not_set_Did_you_forget_a_leading_Slash:n(5090,e.DiagnosticCategory.Error,"Non_relative_paths_are_not_allowed_when_baseUrl_is_not_set_Did_you_forget_a_leading_Slash_5090","Non-relative paths are not allowed when 'baseUrl' is not set. Did you forget a leading './'?"),Option_preserveConstEnums_cannot_be_disabled_when_isolatedModules_is_enabled:n(5091,e.DiagnosticCategory.Error,"Option_preserveConstEnums_cannot_be_disabled_when_isolatedModules_is_enabled_5091","Option 'preserveConstEnums' cannot be disabled when 'isolatedModules' is enabled."),The_root_value_of_a_0_file_must_be_an_object:n(5092,e.DiagnosticCategory.Error,"The_root_value_of_a_0_file_must_be_an_object_5092","The root value of a '{0}' file must be an object."),Compiler_option_0_may_only_be_used_with_build:n(5093,e.DiagnosticCategory.Error,"Compiler_option_0_may_only_be_used_with_build_5093","Compiler option '--{0}' may only be used with '--build'."),Compiler_option_0_may_not_be_used_with_build:n(5094,e.DiagnosticCategory.Error,"Compiler_option_0_may_not_be_used_with_build_5094","Compiler option '--{0}' may not be used with '--build'."),Generates_a_sourcemap_for_each_corresponding_d_ts_file:n(6e3,e.DiagnosticCategory.Message,"Generates_a_sourcemap_for_each_corresponding_d_ts_file_6000","Generates a sourcemap for each corresponding '.d.ts' file."),Concatenate_and_emit_output_to_single_file:n(6001,e.DiagnosticCategory.Message,"Concatenate_and_emit_output_to_single_file_6001","Concatenate and emit output to single file."),Generates_corresponding_d_ts_file:n(6002,e.DiagnosticCategory.Message,"Generates_corresponding_d_ts_file_6002","Generates corresponding '.d.ts' file."),Specify_the_location_where_debugger_should_locate_map_files_instead_of_generated_locations:n(6655,e.DiagnosticCategory.Message,"Specify_the_location_where_debugger_should_locate_map_files_instead_of_generated_locations_6655","Specify the location where debugger should locate map files instead of generated locations."),Specify_the_location_where_debugger_should_locate_TypeScript_files_instead_of_source_locations:n(6004,e.DiagnosticCategory.Message,"Specify_the_location_where_debugger_should_locate_TypeScript_files_instead_of_source_locations_6004","Specify the location where debugger should locate TypeScript files instead of source locations."),Watch_input_files:n(6005,e.DiagnosticCategory.Message,"Watch_input_files_6005","Watch input files."),Redirect_output_structure_to_the_directory:n(6006,e.DiagnosticCategory.Message,"Redirect_output_structure_to_the_directory_6006","Redirect output structure to the directory."),Do_not_erase_const_enum_declarations_in_generated_code:n(6007,e.DiagnosticCategory.Message,"Do_not_erase_const_enum_declarations_in_generated_code_6007","Do not erase const enum declarations in generated code."),Do_not_emit_outputs_if_any_errors_were_reported:n(6008,e.DiagnosticCategory.Message,"Do_not_emit_outputs_if_any_errors_were_reported_6008","Do not emit outputs if any errors were reported."),Do_not_emit_comments_to_output:n(6009,e.DiagnosticCategory.Message,"Do_not_emit_comments_to_output_6009","Do not emit comments to output."),Do_not_emit_outputs:n(6010,e.DiagnosticCategory.Message,"Do_not_emit_outputs_6010","Do not emit outputs."),Allow_default_imports_from_modules_with_no_default_export_This_does_not_affect_code_emit_just_typechecking:n(6011,e.DiagnosticCategory.Message,"Allow_default_imports_from_modules_with_no_default_export_This_does_not_affect_code_emit_just_typech_6011","Allow default imports from modules with no default export. This does not affect code emit, just typechecking."),Skip_type_checking_of_declaration_files:n(6012,e.DiagnosticCategory.Message,"Skip_type_checking_of_declaration_files_6012","Skip type checking of declaration files."),Do_not_resolve_the_real_path_of_symlinks:n(6013,e.DiagnosticCategory.Message,"Do_not_resolve_the_real_path_of_symlinks_6013","Do not resolve the real path of symlinks."),Only_emit_d_ts_declaration_files:n(6014,e.DiagnosticCategory.Message,"Only_emit_d_ts_declaration_files_6014","Only emit '.d.ts' declaration files."),Specify_ECMAScript_target_version:n(6015,e.DiagnosticCategory.Message,"Specify_ECMAScript_target_version_6015","Specify ECMAScript target version."),Specify_module_code_generation:n(6016,e.DiagnosticCategory.Message,"Specify_module_code_generation_6016","Specify module code generation."),Print_this_message:n(6017,e.DiagnosticCategory.Message,"Print_this_message_6017","Print this message."),Print_the_compiler_s_version:n(6019,e.DiagnosticCategory.Message,"Print_the_compiler_s_version_6019","Print the compiler's version."),Compile_the_project_given_the_path_to_its_configuration_file_or_to_a_folder_with_a_tsconfig_json:n(6020,e.DiagnosticCategory.Message,"Compile_the_project_given_the_path_to_its_configuration_file_or_to_a_folder_with_a_tsconfig_json_6020","Compile the project given the path to its configuration file, or to a folder with a 'tsconfig.json'."),Syntax_Colon_0:n(6023,e.DiagnosticCategory.Message,"Syntax_Colon_0_6023","Syntax: {0}"),options:n(6024,e.DiagnosticCategory.Message,"options_6024","options"),file:n(6025,e.DiagnosticCategory.Message,"file_6025","file"),Examples_Colon_0:n(6026,e.DiagnosticCategory.Message,"Examples_Colon_0_6026","Examples: {0}"),Options_Colon:n(6027,e.DiagnosticCategory.Message,"Options_Colon_6027","Options:"),Version_0:n(6029,e.DiagnosticCategory.Message,"Version_0_6029","Version {0}"),Insert_command_line_options_and_files_from_a_file:n(6030,e.DiagnosticCategory.Message,"Insert_command_line_options_and_files_from_a_file_6030","Insert command line options and files from a file."),Starting_compilation_in_watch_mode:n(6031,e.DiagnosticCategory.Message,"Starting_compilation_in_watch_mode_6031","Starting compilation in watch mode..."),File_change_detected_Starting_incremental_compilation:n(6032,e.DiagnosticCategory.Message,"File_change_detected_Starting_incremental_compilation_6032","File change detected. Starting incremental compilation..."),KIND:n(6034,e.DiagnosticCategory.Message,"KIND_6034","KIND"),FILE:n(6035,e.DiagnosticCategory.Message,"FILE_6035","FILE"),VERSION:n(6036,e.DiagnosticCategory.Message,"VERSION_6036","VERSION"),LOCATION:n(6037,e.DiagnosticCategory.Message,"LOCATION_6037","LOCATION"),DIRECTORY:n(6038,e.DiagnosticCategory.Message,"DIRECTORY_6038","DIRECTORY"),STRATEGY:n(6039,e.DiagnosticCategory.Message,"STRATEGY_6039","STRATEGY"),FILE_OR_DIRECTORY:n(6040,e.DiagnosticCategory.Message,"FILE_OR_DIRECTORY_6040","FILE OR DIRECTORY"),Generates_corresponding_map_file:n(6043,e.DiagnosticCategory.Message,"Generates_corresponding_map_file_6043","Generates corresponding '.map' file."),Compiler_option_0_expects_an_argument:n(6044,e.DiagnosticCategory.Error,"Compiler_option_0_expects_an_argument_6044","Compiler option '{0}' expects an argument."),Unterminated_quoted_string_in_response_file_0:n(6045,e.DiagnosticCategory.Error,"Unterminated_quoted_string_in_response_file_0_6045","Unterminated quoted string in response file '{0}'."),Argument_for_0_option_must_be_Colon_1:n(6046,e.DiagnosticCategory.Error,"Argument_for_0_option_must_be_Colon_1_6046","Argument for '{0}' option must be: {1}."),Locale_must_be_of_the_form_language_or_language_territory_For_example_0_or_1:n(6048,e.DiagnosticCategory.Error,"Locale_must_be_of_the_form_language_or_language_territory_For_example_0_or_1_6048","Locale must be of the form <language> or <language>-<territory>. For example '{0}' or '{1}'."),Unsupported_locale_0:n(6049,e.DiagnosticCategory.Error,"Unsupported_locale_0_6049","Unsupported locale '{0}'."),Unable_to_open_file_0:n(6050,e.DiagnosticCategory.Error,"Unable_to_open_file_0_6050","Unable to open file '{0}'."),Corrupted_locale_file_0:n(6051,e.DiagnosticCategory.Error,"Corrupted_locale_file_0_6051","Corrupted locale file {0}."),Raise_error_on_expressions_and_declarations_with_an_implied_any_type:n(6052,e.DiagnosticCategory.Message,"Raise_error_on_expressions_and_declarations_with_an_implied_any_type_6052","Raise error on expressions and declarations with an implied 'any' type."),File_0_not_found:n(6053,e.DiagnosticCategory.Error,"File_0_not_found_6053","File '{0}' not found."),File_0_has_an_unsupported_extension_The_only_supported_extensions_are_1:n(6054,e.DiagnosticCategory.Error,"File_0_has_an_unsupported_extension_The_only_supported_extensions_are_1_6054","File '{0}' has an unsupported extension. The only supported extensions are {1}."),Suppress_noImplicitAny_errors_for_indexing_objects_lacking_index_signatures:n(6055,e.DiagnosticCategory.Message,"Suppress_noImplicitAny_errors_for_indexing_objects_lacking_index_signatures_6055","Suppress noImplicitAny errors for indexing objects lacking index signatures."),Do_not_emit_declarations_for_code_that_has_an_internal_annotation:n(6056,e.DiagnosticCategory.Message,"Do_not_emit_declarations_for_code_that_has_an_internal_annotation_6056","Do not emit declarations for code that has an '@internal' annotation."),Specify_the_root_directory_of_input_files_Use_to_control_the_output_directory_structure_with_outDir:n(6058,e.DiagnosticCategory.Message,"Specify_the_root_directory_of_input_files_Use_to_control_the_output_directory_structure_with_outDir_6058","Specify the root directory of input files. Use to control the output directory structure with --outDir."),File_0_is_not_under_rootDir_1_rootDir_is_expected_to_contain_all_source_files:n(6059,e.DiagnosticCategory.Error,"File_0_is_not_under_rootDir_1_rootDir_is_expected_to_contain_all_source_files_6059","File '{0}' is not under 'rootDir' '{1}'. 'rootDir' is expected to contain all source files."),Specify_the_end_of_line_sequence_to_be_used_when_emitting_files_Colon_CRLF_dos_or_LF_unix:n(6060,e.DiagnosticCategory.Message,"Specify_the_end_of_line_sequence_to_be_used_when_emitting_files_Colon_CRLF_dos_or_LF_unix_6060","Specify the end of line sequence to be used when emitting files: 'CRLF' (dos) or 'LF' (unix)."),NEWLINE:n(6061,e.DiagnosticCategory.Message,"NEWLINE_6061","NEWLINE"),Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_null_on_command_line:n(6064,e.DiagnosticCategory.Error,"Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_null_on_command_line_6064","Option '{0}' can only be specified in 'tsconfig.json' file or set to 'null' on command line."),Enables_experimental_support_for_ES7_decorators:n(6065,e.DiagnosticCategory.Message,"Enables_experimental_support_for_ES7_decorators_6065","Enables experimental support for ES7 decorators."),Enables_experimental_support_for_emitting_type_metadata_for_decorators:n(6066,e.DiagnosticCategory.Message,"Enables_experimental_support_for_emitting_type_metadata_for_decorators_6066","Enables experimental support for emitting type metadata for decorators."),Enables_experimental_support_for_ES7_async_functions:n(6068,e.DiagnosticCategory.Message,"Enables_experimental_support_for_ES7_async_functions_6068","Enables experimental support for ES7 async functions."),Specify_module_resolution_strategy_Colon_node_Node_js_or_classic_TypeScript_pre_1_6:n(6069,e.DiagnosticCategory.Message,"Specify_module_resolution_strategy_Colon_node_Node_js_or_classic_TypeScript_pre_1_6_6069","Specify module resolution strategy: 'node' (Node.js) or 'classic' (TypeScript pre-1.6)."),Initializes_a_TypeScript_project_and_creates_a_tsconfig_json_file:n(6070,e.DiagnosticCategory.Message,"Initializes_a_TypeScript_project_and_creates_a_tsconfig_json_file_6070","Initializes a TypeScript project and creates a tsconfig.json file."),Successfully_created_a_tsconfig_json_file:n(6071,e.DiagnosticCategory.Message,"Successfully_created_a_tsconfig_json_file_6071","Successfully created a tsconfig.json file."),Suppress_excess_property_checks_for_object_literals:n(6072,e.DiagnosticCategory.Message,"Suppress_excess_property_checks_for_object_literals_6072","Suppress excess property checks for object literals."),Stylize_errors_and_messages_using_color_and_context_experimental:n(6073,e.DiagnosticCategory.Message,"Stylize_errors_and_messages_using_color_and_context_experimental_6073","Stylize errors and messages using color and context (experimental)."),Do_not_report_errors_on_unused_labels:n(6074,e.DiagnosticCategory.Message,"Do_not_report_errors_on_unused_labels_6074","Do not report errors on unused labels."),Report_error_when_not_all_code_paths_in_function_return_a_value:n(6075,e.DiagnosticCategory.Message,"Report_error_when_not_all_code_paths_in_function_return_a_value_6075","Report error when not all code paths in function return a value."),Report_errors_for_fallthrough_cases_in_switch_statement:n(6076,e.DiagnosticCategory.Message,"Report_errors_for_fallthrough_cases_in_switch_statement_6076","Report errors for fallthrough cases in switch statement."),Do_not_report_errors_on_unreachable_code:n(6077,e.DiagnosticCategory.Message,"Do_not_report_errors_on_unreachable_code_6077","Do not report errors on unreachable code."),Disallow_inconsistently_cased_references_to_the_same_file:n(6078,e.DiagnosticCategory.Message,"Disallow_inconsistently_cased_references_to_the_same_file_6078","Disallow inconsistently-cased references to the same file."),Specify_library_files_to_be_included_in_the_compilation:n(6079,e.DiagnosticCategory.Message,"Specify_library_files_to_be_included_in_the_compilation_6079","Specify library files to be included in the compilation."),Specify_JSX_code_generation:n(6080,e.DiagnosticCategory.Message,"Specify_JSX_code_generation_6080","Specify JSX code generation."),File_0_has_an_unsupported_extension_so_skipping_it:n(6081,e.DiagnosticCategory.Message,"File_0_has_an_unsupported_extension_so_skipping_it_6081","File '{0}' has an unsupported extension, so skipping it."),Only_amd_and_system_modules_are_supported_alongside_0:n(6082,e.DiagnosticCategory.Error,"Only_amd_and_system_modules_are_supported_alongside_0_6082","Only 'amd' and 'system' modules are supported alongside --{0}."),Base_directory_to_resolve_non_absolute_module_names:n(6083,e.DiagnosticCategory.Message,"Base_directory_to_resolve_non_absolute_module_names_6083","Base directory to resolve non-absolute module names."),Deprecated_Use_jsxFactory_instead_Specify_the_object_invoked_for_createElement_when_targeting_react_JSX_emit:n(6084,e.DiagnosticCategory.Message,"Deprecated_Use_jsxFactory_instead_Specify_the_object_invoked_for_createElement_when_targeting_react__6084","[Deprecated] Use '--jsxFactory' instead. Specify the object invoked for createElement when targeting 'react' JSX emit"),Enable_tracing_of_the_name_resolution_process:n(6085,e.DiagnosticCategory.Message,"Enable_tracing_of_the_name_resolution_process_6085","Enable tracing of the name resolution process."),Resolving_module_0_from_1:n(6086,e.DiagnosticCategory.Message,"Resolving_module_0_from_1_6086","======== Resolving module '{0}' from '{1}'. ========"),Explicitly_specified_module_resolution_kind_Colon_0:n(6087,e.DiagnosticCategory.Message,"Explicitly_specified_module_resolution_kind_Colon_0_6087","Explicitly specified module resolution kind: '{0}'."),Module_resolution_kind_is_not_specified_using_0:n(6088,e.DiagnosticCategory.Message,"Module_resolution_kind_is_not_specified_using_0_6088","Module resolution kind is not specified, using '{0}'."),Module_name_0_was_successfully_resolved_to_1:n(6089,e.DiagnosticCategory.Message,"Module_name_0_was_successfully_resolved_to_1_6089","======== Module name '{0}' was successfully resolved to '{1}'. ========"),Module_name_0_was_not_resolved:n(6090,e.DiagnosticCategory.Message,"Module_name_0_was_not_resolved_6090","======== Module name '{0}' was not resolved. ========"),paths_option_is_specified_looking_for_a_pattern_to_match_module_name_0:n(6091,e.DiagnosticCategory.Message,"paths_option_is_specified_looking_for_a_pattern_to_match_module_name_0_6091","'paths' option is specified, looking for a pattern to match module name '{0}'."),Module_name_0_matched_pattern_1:n(6092,e.DiagnosticCategory.Message,"Module_name_0_matched_pattern_1_6092","Module name '{0}', matched pattern '{1}'."),Trying_substitution_0_candidate_module_location_Colon_1:n(6093,e.DiagnosticCategory.Message,"Trying_substitution_0_candidate_module_location_Colon_1_6093","Trying substitution '{0}', candidate module location: '{1}'."),Resolving_module_name_0_relative_to_base_url_1_2:n(6094,e.DiagnosticCategory.Message,"Resolving_module_name_0_relative_to_base_url_1_2_6094","Resolving module name '{0}' relative to base url '{1}' - '{2}'."),Loading_module_as_file_Slash_folder_candidate_module_location_0_target_file_type_1:n(6095,e.DiagnosticCategory.Message,"Loading_module_as_file_Slash_folder_candidate_module_location_0_target_file_type_1_6095","Loading module as file / folder, candidate module location '{0}', target file type '{1}'."),File_0_does_not_exist:n(6096,e.DiagnosticCategory.Message,"File_0_does_not_exist_6096","File '{0}' does not exist."),File_0_exist_use_it_as_a_name_resolution_result:n(6097,e.DiagnosticCategory.Message,"File_0_exist_use_it_as_a_name_resolution_result_6097","File '{0}' exist - use it as a name resolution result."),Loading_module_0_from_node_modules_folder_target_file_type_1:n(6098,e.DiagnosticCategory.Message,"Loading_module_0_from_node_modules_folder_target_file_type_1_6098","Loading module '{0}' from 'node_modules' folder, target file type '{1}'."),Found_package_json_at_0:n(6099,e.DiagnosticCategory.Message,"Found_package_json_at_0_6099","Found 'package.json' at '{0}'."),package_json_does_not_have_a_0_field:n(6100,e.DiagnosticCategory.Message,"package_json_does_not_have_a_0_field_6100","'package.json' does not have a '{0}' field."),package_json_has_0_field_1_that_references_2:n(6101,e.DiagnosticCategory.Message,"package_json_has_0_field_1_that_references_2_6101","'package.json' has '{0}' field '{1}' that references '{2}'."),Allow_javascript_files_to_be_compiled:n(6102,e.DiagnosticCategory.Message,"Allow_javascript_files_to_be_compiled_6102","Allow javascript files to be compiled."),Option_0_should_have_array_of_strings_as_a_value:n(6103,e.DiagnosticCategory.Error,"Option_0_should_have_array_of_strings_as_a_value_6103","Option '{0}' should have array of strings as a value."),Checking_if_0_is_the_longest_matching_prefix_for_1_2:n(6104,e.DiagnosticCategory.Message,"Checking_if_0_is_the_longest_matching_prefix_for_1_2_6104","Checking if '{0}' is the longest matching prefix for '{1}' - '{2}'."),Expected_type_of_0_field_in_package_json_to_be_1_got_2:n(6105,e.DiagnosticCategory.Message,"Expected_type_of_0_field_in_package_json_to_be_1_got_2_6105","Expected type of '{0}' field in 'package.json' to be '{1}', got '{2}'."),baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1:n(6106,e.DiagnosticCategory.Message,"baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1_6106","'baseUrl' option is set to '{0}', using this value to resolve non-relative module name '{1}'."),rootDirs_option_is_set_using_it_to_resolve_relative_module_name_0:n(6107,e.DiagnosticCategory.Message,"rootDirs_option_is_set_using_it_to_resolve_relative_module_name_0_6107","'rootDirs' option is set, using it to resolve relative module name '{0}'."),Longest_matching_prefix_for_0_is_1:n(6108,e.DiagnosticCategory.Message,"Longest_matching_prefix_for_0_is_1_6108","Longest matching prefix for '{0}' is '{1}'."),Loading_0_from_the_root_dir_1_candidate_location_2:n(6109,e.DiagnosticCategory.Message,"Loading_0_from_the_root_dir_1_candidate_location_2_6109","Loading '{0}' from the root dir '{1}', candidate location '{2}'."),Trying_other_entries_in_rootDirs:n(6110,e.DiagnosticCategory.Message,"Trying_other_entries_in_rootDirs_6110","Trying other entries in 'rootDirs'."),Module_resolution_using_rootDirs_has_failed:n(6111,e.DiagnosticCategory.Message,"Module_resolution_using_rootDirs_has_failed_6111","Module resolution using 'rootDirs' has failed."),Do_not_emit_use_strict_directives_in_module_output:n(6112,e.DiagnosticCategory.Message,"Do_not_emit_use_strict_directives_in_module_output_6112","Do not emit 'use strict' directives in module output."),Enable_strict_null_checks:n(6113,e.DiagnosticCategory.Message,"Enable_strict_null_checks_6113","Enable strict null checks."),Unknown_option_excludes_Did_you_mean_exclude:n(6114,e.DiagnosticCategory.Error,"Unknown_option_excludes_Did_you_mean_exclude_6114","Unknown option 'excludes'. Did you mean 'exclude'?"),Raise_error_on_this_expressions_with_an_implied_any_type:n(6115,e.DiagnosticCategory.Message,"Raise_error_on_this_expressions_with_an_implied_any_type_6115","Raise error on 'this' expressions with an implied 'any' type."),Resolving_type_reference_directive_0_containing_file_1_root_directory_2:n(6116,e.DiagnosticCategory.Message,"Resolving_type_reference_directive_0_containing_file_1_root_directory_2_6116","======== Resolving type reference directive '{0}', containing file '{1}', root directory '{2}'. ========"),Resolving_using_primary_search_paths:n(6117,e.DiagnosticCategory.Message,"Resolving_using_primary_search_paths_6117","Resolving using primary search paths..."),Resolving_from_node_modules_folder:n(6118,e.DiagnosticCategory.Message,"Resolving_from_node_modules_folder_6118","Resolving from node_modules folder..."),Type_reference_directive_0_was_successfully_resolved_to_1_primary_Colon_2:n(6119,e.DiagnosticCategory.Message,"Type_reference_directive_0_was_successfully_resolved_to_1_primary_Colon_2_6119","======== Type reference directive '{0}' was successfully resolved to '{1}', primary: {2}. ========"),Type_reference_directive_0_was_not_resolved:n(6120,e.DiagnosticCategory.Message,"Type_reference_directive_0_was_not_resolved_6120","======== Type reference directive '{0}' was not resolved. ========"),Resolving_with_primary_search_path_0:n(6121,e.DiagnosticCategory.Message,"Resolving_with_primary_search_path_0_6121","Resolving with primary search path '{0}'."),Root_directory_cannot_be_determined_skipping_primary_search_paths:n(6122,e.DiagnosticCategory.Message,"Root_directory_cannot_be_determined_skipping_primary_search_paths_6122","Root directory cannot be determined, skipping primary search paths."),Resolving_type_reference_directive_0_containing_file_1_root_directory_not_set:n(6123,e.DiagnosticCategory.Message,"Resolving_type_reference_directive_0_containing_file_1_root_directory_not_set_6123","======== Resolving type reference directive '{0}', containing file '{1}', root directory not set. ========"),Type_declaration_files_to_be_included_in_compilation:n(6124,e.DiagnosticCategory.Message,"Type_declaration_files_to_be_included_in_compilation_6124","Type declaration files to be included in compilation."),Looking_up_in_node_modules_folder_initial_location_0:n(6125,e.DiagnosticCategory.Message,"Looking_up_in_node_modules_folder_initial_location_0_6125","Looking up in 'node_modules' folder, initial location '{0}'."),Containing_file_is_not_specified_and_root_directory_cannot_be_determined_skipping_lookup_in_node_modules_folder:n(6126,e.DiagnosticCategory.Message,"Containing_file_is_not_specified_and_root_directory_cannot_be_determined_skipping_lookup_in_node_mod_6126","Containing file is not specified and root directory cannot be determined, skipping lookup in 'node_modules' folder."),Resolving_type_reference_directive_0_containing_file_not_set_root_directory_1:n(6127,e.DiagnosticCategory.Message,"Resolving_type_reference_directive_0_containing_file_not_set_root_directory_1_6127","======== Resolving type reference directive '{0}', containing file not set, root directory '{1}'. ========"),Resolving_type_reference_directive_0_containing_file_not_set_root_directory_not_set:n(6128,e.DiagnosticCategory.Message,"Resolving_type_reference_directive_0_containing_file_not_set_root_directory_not_set_6128","======== Resolving type reference directive '{0}', containing file not set, root directory not set. ========"),Resolving_real_path_for_0_result_1:n(6130,e.DiagnosticCategory.Message,"Resolving_real_path_for_0_result_1_6130","Resolving real path for '{0}', result '{1}'."),Cannot_compile_modules_using_option_0_unless_the_module_flag_is_amd_or_system:n(6131,e.DiagnosticCategory.Error,"Cannot_compile_modules_using_option_0_unless_the_module_flag_is_amd_or_system_6131","Cannot compile modules using option '{0}' unless the '--module' flag is 'amd' or 'system'."),File_name_0_has_a_1_extension_stripping_it:n(6132,e.DiagnosticCategory.Message,"File_name_0_has_a_1_extension_stripping_it_6132","File name '{0}' has a '{1}' extension - stripping it."),_0_is_declared_but_its_value_is_never_read:n(6133,e.DiagnosticCategory.Error,"_0_is_declared_but_its_value_is_never_read_6133","'{0}' is declared but its value is never read.",!0),Report_errors_on_unused_locals:n(6134,e.DiagnosticCategory.Message,"Report_errors_on_unused_locals_6134","Report errors on unused locals."),Report_errors_on_unused_parameters:n(6135,e.DiagnosticCategory.Message,"Report_errors_on_unused_parameters_6135","Report errors on unused parameters."),The_maximum_dependency_depth_to_search_under_node_modules_and_load_JavaScript_files:n(6136,e.DiagnosticCategory.Message,"The_maximum_dependency_depth_to_search_under_node_modules_and_load_JavaScript_files_6136","The maximum dependency depth to search under node_modules and load JavaScript files."),Cannot_import_type_declaration_files_Consider_importing_0_instead_of_1:n(6137,e.DiagnosticCategory.Error,"Cannot_import_type_declaration_files_Consider_importing_0_instead_of_1_6137","Cannot import type declaration files. Consider importing '{0}' instead of '{1}'."),Property_0_is_declared_but_its_value_is_never_read:n(6138,e.DiagnosticCategory.Error,"Property_0_is_declared_but_its_value_is_never_read_6138","Property '{0}' is declared but its value is never read.",!0),Import_emit_helpers_from_tslib:n(6139,e.DiagnosticCategory.Message,"Import_emit_helpers_from_tslib_6139","Import emit helpers from 'tslib'."),Auto_discovery_for_typings_is_enabled_in_project_0_Running_extra_resolution_pass_for_module_1_using_cache_location_2:n(6140,e.DiagnosticCategory.Error,"Auto_discovery_for_typings_is_enabled_in_project_0_Running_extra_resolution_pass_for_module_1_using__6140","Auto discovery for typings is enabled in project '{0}'. Running extra resolution pass for module '{1}' using cache location '{2}'."),Parse_in_strict_mode_and_emit_use_strict_for_each_source_file:n(6141,e.DiagnosticCategory.Message,"Parse_in_strict_mode_and_emit_use_strict_for_each_source_file_6141",'Parse in strict mode and emit "use strict" for each source file.'),Module_0_was_resolved_to_1_but_jsx_is_not_set:n(6142,e.DiagnosticCategory.Error,"Module_0_was_resolved_to_1_but_jsx_is_not_set_6142","Module '{0}' was resolved to '{1}', but '--jsx' is not set."),Module_0_was_resolved_as_locally_declared_ambient_module_in_file_1:n(6144,e.DiagnosticCategory.Message,"Module_0_was_resolved_as_locally_declared_ambient_module_in_file_1_6144","Module '{0}' was resolved as locally declared ambient module in file '{1}'."),Module_0_was_resolved_as_ambient_module_declared_in_1_since_this_file_was_not_modified:n(6145,e.DiagnosticCategory.Message,"Module_0_was_resolved_as_ambient_module_declared_in_1_since_this_file_was_not_modified_6145","Module '{0}' was resolved as ambient module declared in '{1}' since this file was not modified."),Specify_the_JSX_factory_function_to_use_when_targeting_react_JSX_emit_e_g_React_createElement_or_h:n(6146,e.DiagnosticCategory.Message,"Specify_the_JSX_factory_function_to_use_when_targeting_react_JSX_emit_e_g_React_createElement_or_h_6146","Specify the JSX factory function to use when targeting 'react' JSX emit, e.g. 'React.createElement' or 'h'."),Resolution_for_module_0_was_found_in_cache_from_location_1:n(6147,e.DiagnosticCategory.Message,"Resolution_for_module_0_was_found_in_cache_from_location_1_6147","Resolution for module '{0}' was found in cache from location '{1}'."),Directory_0_does_not_exist_skipping_all_lookups_in_it:n(6148,e.DiagnosticCategory.Message,"Directory_0_does_not_exist_skipping_all_lookups_in_it_6148","Directory '{0}' does not exist, skipping all lookups in it."),Show_diagnostic_information:n(6149,e.DiagnosticCategory.Message,"Show_diagnostic_information_6149","Show diagnostic information."),Show_verbose_diagnostic_information:n(6150,e.DiagnosticCategory.Message,"Show_verbose_diagnostic_information_6150","Show verbose diagnostic information."),Emit_a_single_file_with_source_maps_instead_of_having_a_separate_file:n(6151,e.DiagnosticCategory.Message,"Emit_a_single_file_with_source_maps_instead_of_having_a_separate_file_6151","Emit a single file with source maps instead of having a separate file."),Emit_the_source_alongside_the_sourcemaps_within_a_single_file_requires_inlineSourceMap_or_sourceMap_to_be_set:n(6152,e.DiagnosticCategory.Message,"Emit_the_source_alongside_the_sourcemaps_within_a_single_file_requires_inlineSourceMap_or_sourceMap__6152","Emit the source alongside the sourcemaps within a single file; requires '--inlineSourceMap' or '--sourceMap' to be set."),Transpile_each_file_as_a_separate_module_similar_to_ts_transpileModule:n(6153,e.DiagnosticCategory.Message,"Transpile_each_file_as_a_separate_module_similar_to_ts_transpileModule_6153","Transpile each file as a separate module (similar to 'ts.transpileModule')."),Print_names_of_generated_files_part_of_the_compilation:n(6154,e.DiagnosticCategory.Message,"Print_names_of_generated_files_part_of_the_compilation_6154","Print names of generated files part of the compilation."),Print_names_of_files_part_of_the_compilation:n(6155,e.DiagnosticCategory.Message,"Print_names_of_files_part_of_the_compilation_6155","Print names of files part of the compilation."),The_locale_used_when_displaying_messages_to_the_user_e_g_en_us:n(6156,e.DiagnosticCategory.Message,"The_locale_used_when_displaying_messages_to_the_user_e_g_en_us_6156","The locale used when displaying messages to the user (e.g. 'en-us')"),Do_not_generate_custom_helper_functions_like_extends_in_compiled_output:n(6157,e.DiagnosticCategory.Message,"Do_not_generate_custom_helper_functions_like_extends_in_compiled_output_6157","Do not generate custom helper functions like '__extends' in compiled output."),Do_not_include_the_default_library_file_lib_d_ts:n(6158,e.DiagnosticCategory.Message,"Do_not_include_the_default_library_file_lib_d_ts_6158","Do not include the default library file (lib.d.ts)."),Do_not_add_triple_slash_references_or_imported_modules_to_the_list_of_compiled_files:n(6159,e.DiagnosticCategory.Message,"Do_not_add_triple_slash_references_or_imported_modules_to_the_list_of_compiled_files_6159","Do not add triple-slash references or imported modules to the list of compiled files."),Deprecated_Use_skipLibCheck_instead_Skip_type_checking_of_default_library_declaration_files:n(6160,e.DiagnosticCategory.Message,"Deprecated_Use_skipLibCheck_instead_Skip_type_checking_of_default_library_declaration_files_6160","[Deprecated] Use '--skipLibCheck' instead. Skip type checking of default library declaration files."),List_of_folders_to_include_type_definitions_from:n(6161,e.DiagnosticCategory.Message,"List_of_folders_to_include_type_definitions_from_6161","List of folders to include type definitions from."),Disable_size_limitations_on_JavaScript_projects:n(6162,e.DiagnosticCategory.Message,"Disable_size_limitations_on_JavaScript_projects_6162","Disable size limitations on JavaScript projects."),The_character_set_of_the_input_files:n(6163,e.DiagnosticCategory.Message,"The_character_set_of_the_input_files_6163","The character set of the input files."),Emit_a_UTF_8_Byte_Order_Mark_BOM_in_the_beginning_of_output_files:n(6622,e.DiagnosticCategory.Message,"Emit_a_UTF_8_Byte_Order_Mark_BOM_in_the_beginning_of_output_files_6622","Emit a UTF-8 Byte Order Mark (BOM) in the beginning of output files."),Do_not_truncate_error_messages:n(6165,e.DiagnosticCategory.Message,"Do_not_truncate_error_messages_6165","Do not truncate error messages."),Output_directory_for_generated_declaration_files:n(6166,e.DiagnosticCategory.Message,"Output_directory_for_generated_declaration_files_6166","Output directory for generated declaration files."),A_series_of_entries_which_re_map_imports_to_lookup_locations_relative_to_the_baseUrl:n(6167,e.DiagnosticCategory.Message,"A_series_of_entries_which_re_map_imports_to_lookup_locations_relative_to_the_baseUrl_6167","A series of entries which re-map imports to lookup locations relative to the 'baseUrl'."),List_of_root_folders_whose_combined_content_represents_the_structure_of_the_project_at_runtime:n(6168,e.DiagnosticCategory.Message,"List_of_root_folders_whose_combined_content_represents_the_structure_of_the_project_at_runtime_6168","List of root folders whose combined content represents the structure of the project at runtime."),Show_all_compiler_options:n(6169,e.DiagnosticCategory.Message,"Show_all_compiler_options_6169","Show all compiler options."),Deprecated_Use_outFile_instead_Concatenate_and_emit_output_to_single_file:n(6170,e.DiagnosticCategory.Message,"Deprecated_Use_outFile_instead_Concatenate_and_emit_output_to_single_file_6170","[Deprecated] Use '--outFile' instead. Concatenate and emit output to single file"),Command_line_Options:n(6171,e.DiagnosticCategory.Message,"Command_line_Options_6171","Command-line Options"),Provide_full_support_for_iterables_in_for_of_spread_and_destructuring_when_targeting_ES5_or_ES3:n(6179,e.DiagnosticCategory.Message,"Provide_full_support_for_iterables_in_for_of_spread_and_destructuring_when_targeting_ES5_or_ES3_6179","Provide full support for iterables in 'for-of', spread, and destructuring when targeting 'ES5' or 'ES3'."),Enable_all_strict_type_checking_options:n(6180,e.DiagnosticCategory.Message,"Enable_all_strict_type_checking_options_6180","Enable all strict type-checking options."),List_of_language_service_plugins:n(6181,e.DiagnosticCategory.Message,"List_of_language_service_plugins_6181","List of language service plugins."),Scoped_package_detected_looking_in_0:n(6182,e.DiagnosticCategory.Message,"Scoped_package_detected_looking_in_0_6182","Scoped package detected, looking in '{0}'"),Reusing_resolution_of_module_0_from_1_of_old_program_it_was_successfully_resolved_to_2:n(6183,e.DiagnosticCategory.Message,"Reusing_resolution_of_module_0_from_1_of_old_program_it_was_successfully_resolved_to_2_6183","Reusing resolution of module '{0}' from '{1}' of old program, it was successfully resolved to '{2}'."),Reusing_resolution_of_module_0_from_1_of_old_program_it_was_successfully_resolved_to_2_with_Package_ID_3:n(6184,e.DiagnosticCategory.Message,"Reusing_resolution_of_module_0_from_1_of_old_program_it_was_successfully_resolved_to_2_with_Package__6184","Reusing resolution of module '{0}' from '{1}' of old program, it was successfully resolved to '{2}' with Package ID '{3}'."),Enable_strict_checking_of_function_types:n(6186,e.DiagnosticCategory.Message,"Enable_strict_checking_of_function_types_6186","Enable strict checking of function types."),Enable_strict_checking_of_property_initialization_in_classes:n(6187,e.DiagnosticCategory.Message,"Enable_strict_checking_of_property_initialization_in_classes_6187","Enable strict checking of property initialization in classes."),Numeric_separators_are_not_allowed_here:n(6188,e.DiagnosticCategory.Error,"Numeric_separators_are_not_allowed_here_6188","Numeric separators are not allowed here."),Multiple_consecutive_numeric_separators_are_not_permitted:n(6189,e.DiagnosticCategory.Error,"Multiple_consecutive_numeric_separators_are_not_permitted_6189","Multiple consecutive numeric separators are not permitted."),Whether_to_keep_outdated_console_output_in_watch_mode_instead_of_clearing_the_screen:n(6191,e.DiagnosticCategory.Message,"Whether_to_keep_outdated_console_output_in_watch_mode_instead_of_clearing_the_screen_6191","Whether to keep outdated console output in watch mode instead of clearing the screen."),All_imports_in_import_declaration_are_unused:n(6192,e.DiagnosticCategory.Error,"All_imports_in_import_declaration_are_unused_6192","All imports in import declaration are unused.",!0),Found_1_error_Watching_for_file_changes:n(6193,e.DiagnosticCategory.Message,"Found_1_error_Watching_for_file_changes_6193","Found 1 error. Watching for file changes."),Found_0_errors_Watching_for_file_changes:n(6194,e.DiagnosticCategory.Message,"Found_0_errors_Watching_for_file_changes_6194","Found {0} errors. Watching for file changes."),Resolve_keyof_to_string_valued_property_names_only_no_numbers_or_symbols:n(6195,e.DiagnosticCategory.Message,"Resolve_keyof_to_string_valued_property_names_only_no_numbers_or_symbols_6195","Resolve 'keyof' to string valued property names only (no numbers or symbols)."),_0_is_declared_but_never_used:n(6196,e.DiagnosticCategory.Error,"_0_is_declared_but_never_used_6196","'{0}' is declared but never used.",!0),Include_modules_imported_with_json_extension:n(6197,e.DiagnosticCategory.Message,"Include_modules_imported_with_json_extension_6197","Include modules imported with '.json' extension"),All_destructured_elements_are_unused:n(6198,e.DiagnosticCategory.Error,"All_destructured_elements_are_unused_6198","All destructured elements are unused.",!0),All_variables_are_unused:n(6199,e.DiagnosticCategory.Error,"All_variables_are_unused_6199","All variables are unused.",!0),Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0:n(6200,e.DiagnosticCategory.Error,"Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0_6200","Definitions of the following identifiers conflict with those in another file: {0}"),Conflicts_are_in_this_file:n(6201,e.DiagnosticCategory.Message,"Conflicts_are_in_this_file_6201","Conflicts are in this file."),Project_references_may_not_form_a_circular_graph_Cycle_detected_Colon_0:n(6202,e.DiagnosticCategory.Error,"Project_references_may_not_form_a_circular_graph_Cycle_detected_Colon_0_6202","Project references may not form a circular graph. Cycle detected: {0}"),_0_was_also_declared_here:n(6203,e.DiagnosticCategory.Message,"_0_was_also_declared_here_6203","'{0}' was also declared here."),and_here:n(6204,e.DiagnosticCategory.Message,"and_here_6204","and here."),All_type_parameters_are_unused:n(6205,e.DiagnosticCategory.Error,"All_type_parameters_are_unused_6205","All type parameters are unused."),package_json_has_a_typesVersions_field_with_version_specific_path_mappings:n(6206,e.DiagnosticCategory.Message,"package_json_has_a_typesVersions_field_with_version_specific_path_mappings_6206","'package.json' has a 'typesVersions' field with version-specific path mappings."),package_json_does_not_have_a_typesVersions_entry_that_matches_version_0:n(6207,e.DiagnosticCategory.Message,"package_json_does_not_have_a_typesVersions_entry_that_matches_version_0_6207","'package.json' does not have a 'typesVersions' entry that matches version '{0}'."),package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_match_module_name_2:n(6208,e.DiagnosticCategory.Message,"package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_ma_6208","'package.json' has a 'typesVersions' entry '{0}' that matches compiler version '{1}', looking for a pattern to match module name '{2}'."),package_json_has_a_typesVersions_entry_0_that_is_not_a_valid_semver_range:n(6209,e.DiagnosticCategory.Message,"package_json_has_a_typesVersions_entry_0_that_is_not_a_valid_semver_range_6209","'package.json' has a 'typesVersions' entry '{0}' that is not a valid semver range."),An_argument_for_0_was_not_provided:n(6210,e.DiagnosticCategory.Message,"An_argument_for_0_was_not_provided_6210","An argument for '{0}' was not provided."),An_argument_matching_this_binding_pattern_was_not_provided:n(6211,e.DiagnosticCategory.Message,"An_argument_matching_this_binding_pattern_was_not_provided_6211","An argument matching this binding pattern was not provided."),Did_you_mean_to_call_this_expression:n(6212,e.DiagnosticCategory.Message,"Did_you_mean_to_call_this_expression_6212","Did you mean to call this expression?"),Did_you_mean_to_use_new_with_this_expression:n(6213,e.DiagnosticCategory.Message,"Did_you_mean_to_use_new_with_this_expression_6213","Did you mean to use 'new' with this expression?"),Enable_strict_bind_call_and_apply_methods_on_functions:n(6214,e.DiagnosticCategory.Message,"Enable_strict_bind_call_and_apply_methods_on_functions_6214","Enable strict 'bind', 'call', and 'apply' methods on functions."),Using_compiler_options_of_project_reference_redirect_0:n(6215,e.DiagnosticCategory.Message,"Using_compiler_options_of_project_reference_redirect_0_6215","Using compiler options of project reference redirect '{0}'."),Found_1_error:n(6216,e.DiagnosticCategory.Message,"Found_1_error_6216","Found 1 error."),Found_0_errors:n(6217,e.DiagnosticCategory.Message,"Found_0_errors_6217","Found {0} errors."),Module_name_0_was_successfully_resolved_to_1_with_Package_ID_2:n(6218,e.DiagnosticCategory.Message,"Module_name_0_was_successfully_resolved_to_1_with_Package_ID_2_6218","======== Module name '{0}' was successfully resolved to '{1}' with Package ID '{2}'. ========"),Type_reference_directive_0_was_successfully_resolved_to_1_with_Package_ID_2_primary_Colon_3:n(6219,e.DiagnosticCategory.Message,"Type_reference_directive_0_was_successfully_resolved_to_1_with_Package_ID_2_primary_Colon_3_6219","======== Type reference directive '{0}' was successfully resolved to '{1}' with Package ID '{2}', primary: {3}. ========"),package_json_had_a_falsy_0_field:n(6220,e.DiagnosticCategory.Message,"package_json_had_a_falsy_0_field_6220","'package.json' had a falsy '{0}' field."),Disable_use_of_source_files_instead_of_declaration_files_from_referenced_projects:n(6221,e.DiagnosticCategory.Message,"Disable_use_of_source_files_instead_of_declaration_files_from_referenced_projects_6221","Disable use of source files instead of declaration files from referenced projects."),Emit_class_fields_with_Define_instead_of_Set:n(6222,e.DiagnosticCategory.Message,"Emit_class_fields_with_Define_instead_of_Set_6222","Emit class fields with Define instead of Set."),Generates_a_CPU_profile:n(6223,e.DiagnosticCategory.Message,"Generates_a_CPU_profile_6223","Generates a CPU profile."),Disable_solution_searching_for_this_project:n(6224,e.DiagnosticCategory.Message,"Disable_solution_searching_for_this_project_6224","Disable solution searching for this project."),Specify_strategy_for_watching_file_Colon_FixedPollingInterval_default_PriorityPollingInterval_DynamicPriorityPolling_FixedChunkSizePolling_UseFsEvents_UseFsEventsOnParentDirectory:n(6225,e.DiagnosticCategory.Message,"Specify_strategy_for_watching_file_Colon_FixedPollingInterval_default_PriorityPollingInterval_Dynami_6225","Specify strategy for watching file: 'FixedPollingInterval' (default), 'PriorityPollingInterval', 'DynamicPriorityPolling', 'FixedChunkSizePolling', 'UseFsEvents', 'UseFsEventsOnParentDirectory'."),Specify_strategy_for_watching_directory_on_platforms_that_don_t_support_recursive_watching_natively_Colon_UseFsEvents_default_FixedPollingInterval_DynamicPriorityPolling_FixedChunkSizePolling:n(6226,e.DiagnosticCategory.Message,"Specify_strategy_for_watching_directory_on_platforms_that_don_t_support_recursive_watching_natively__6226","Specify strategy for watching directory on platforms that don't support recursive watching natively: 'UseFsEvents' (default), 'FixedPollingInterval', 'DynamicPriorityPolling', 'FixedChunkSizePolling'."),Specify_strategy_for_creating_a_polling_watch_when_it_fails_to_create_using_file_system_events_Colon_FixedInterval_default_PriorityInterval_DynamicPriority_FixedChunkSize:n(6227,e.DiagnosticCategory.Message,"Specify_strategy_for_creating_a_polling_watch_when_it_fails_to_create_using_file_system_events_Colon_6227","Specify strategy for creating a polling watch when it fails to create using file system events: 'FixedInterval' (default), 'PriorityInterval', 'DynamicPriority', 'FixedChunkSize'."),Tag_0_expects_at_least_1_arguments_but_the_JSX_factory_2_provides_at_most_3:n(6229,e.DiagnosticCategory.Error,"Tag_0_expects_at_least_1_arguments_but_the_JSX_factory_2_provides_at_most_3_6229","Tag '{0}' expects at least '{1}' arguments, but the JSX factory '{2}' provides at most '{3}'."),Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_false_or_null_on_command_line:n(6230,e.DiagnosticCategory.Error,"Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_false_or_null_on_command_line_6230","Option '{0}' can only be specified in 'tsconfig.json' file or set to 'false' or 'null' on command line."),Could_not_resolve_the_path_0_with_the_extensions_Colon_1:n(6231,e.DiagnosticCategory.Error,"Could_not_resolve_the_path_0_with_the_extensions_Colon_1_6231","Could not resolve the path '{0}' with the extensions: {1}."),Declaration_augments_declaration_in_another_file_This_cannot_be_serialized:n(6232,e.DiagnosticCategory.Error,"Declaration_augments_declaration_in_another_file_This_cannot_be_serialized_6232","Declaration augments declaration in another file. This cannot be serialized."),This_is_the_declaration_being_augmented_Consider_moving_the_augmenting_declaration_into_the_same_file:n(6233,e.DiagnosticCategory.Error,"This_is_the_declaration_being_augmented_Consider_moving_the_augmenting_declaration_into_the_same_fil_6233","This is the declaration being augmented. Consider moving the augmenting declaration into the same file."),This_expression_is_not_callable_because_it_is_a_get_accessor_Did_you_mean_to_use_it_without:n(6234,e.DiagnosticCategory.Error,"This_expression_is_not_callable_because_it_is_a_get_accessor_Did_you_mean_to_use_it_without_6234","This expression is not callable because it is a 'get' accessor. Did you mean to use it without '()'?"),Disable_loading_referenced_projects:n(6235,e.DiagnosticCategory.Message,"Disable_loading_referenced_projects_6235","Disable loading referenced projects."),Arguments_for_the_rest_parameter_0_were_not_provided:n(6236,e.DiagnosticCategory.Error,"Arguments_for_the_rest_parameter_0_were_not_provided_6236","Arguments for the rest parameter '{0}' were not provided."),Generates_an_event_trace_and_a_list_of_types:n(6237,e.DiagnosticCategory.Message,"Generates_an_event_trace_and_a_list_of_types_6237","Generates an event trace and a list of types."),Specify_the_module_specifier_to_be_used_to_import_the_jsx_and_jsxs_factory_functions_from_eg_react:n(6238,e.DiagnosticCategory.Error,"Specify_the_module_specifier_to_be_used_to_import_the_jsx_and_jsxs_factory_functions_from_eg_react_6238","Specify the module specifier to be used to import the 'jsx' and 'jsxs' factory functions from. eg, react"),File_0_exists_according_to_earlier_cached_lookups:n(6239,e.DiagnosticCategory.Message,"File_0_exists_according_to_earlier_cached_lookups_6239","File '{0}' exists according to earlier cached lookups."),File_0_does_not_exist_according_to_earlier_cached_lookups:n(6240,e.DiagnosticCategory.Message,"File_0_does_not_exist_according_to_earlier_cached_lookups_6240","File '{0}' does not exist according to earlier cached lookups."),Resolution_for_type_reference_directive_0_was_found_in_cache_from_location_1:n(6241,e.DiagnosticCategory.Message,"Resolution_for_type_reference_directive_0_was_found_in_cache_from_location_1_6241","Resolution for type reference directive '{0}' was found in cache from location '{1}'."),Resolving_type_reference_directive_0_containing_file_1:n(6242,e.DiagnosticCategory.Message,"Resolving_type_reference_directive_0_containing_file_1_6242","======== Resolving type reference directive '{0}', containing file '{1}'. ========"),Interpret_optional_property_types_as_written_rather_than_adding_undefined:n(6243,e.DiagnosticCategory.Message,"Interpret_optional_property_types_as_written_rather_than_adding_undefined_6243","Interpret optional property types as written, rather than adding 'undefined'."),Modules:n(6244,e.DiagnosticCategory.Message,"Modules_6244","Modules"),File_Management:n(6245,e.DiagnosticCategory.Message,"File_Management_6245","File Management"),Emit:n(6246,e.DiagnosticCategory.Message,"Emit_6246","Emit"),JavaScript_Support:n(6247,e.DiagnosticCategory.Message,"JavaScript_Support_6247","JavaScript Support"),Type_Checking:n(6248,e.DiagnosticCategory.Message,"Type_Checking_6248","Type Checking"),Editor_Support:n(6249,e.DiagnosticCategory.Message,"Editor_Support_6249","Editor Support"),Watch_and_Build_Modes:n(6250,e.DiagnosticCategory.Message,"Watch_and_Build_Modes_6250","Watch and Build Modes"),Compiler_Diagnostics:n(6251,e.DiagnosticCategory.Message,"Compiler_Diagnostics_6251","Compiler Diagnostics"),Interop_Constraints:n(6252,e.DiagnosticCategory.Message,"Interop_Constraints_6252","Interop Constraints"),Backwards_Compatibility:n(6253,e.DiagnosticCategory.Message,"Backwards_Compatibility_6253","Backwards Compatibility"),Language_and_Environment:n(6254,e.DiagnosticCategory.Message,"Language_and_Environment_6254","Language and Environment"),Projects:n(6255,e.DiagnosticCategory.Message,"Projects_6255","Projects"),Output_Formatting:n(6256,e.DiagnosticCategory.Message,"Output_Formatting_6256","Output Formatting"),Completeness:n(6257,e.DiagnosticCategory.Message,"Completeness_6257","Completeness"),Projects_to_reference:n(6300,e.DiagnosticCategory.Message,"Projects_to_reference_6300","Projects to reference"),Enable_project_compilation:n(6302,e.DiagnosticCategory.Message,"Enable_project_compilation_6302","Enable project compilation"),Composite_projects_may_not_disable_declaration_emit:n(6304,e.DiagnosticCategory.Error,"Composite_projects_may_not_disable_declaration_emit_6304","Composite projects may not disable declaration emit."),Output_file_0_has_not_been_built_from_source_file_1:n(6305,e.DiagnosticCategory.Error,"Output_file_0_has_not_been_built_from_source_file_1_6305","Output file '{0}' has not been built from source file '{1}'."),Referenced_project_0_must_have_setting_composite_Colon_true:n(6306,e.DiagnosticCategory.Error,"Referenced_project_0_must_have_setting_composite_Colon_true_6306","Referenced project '{0}' must have setting \"composite\": true."),File_0_is_not_listed_within_the_file_list_of_project_1_Projects_must_list_all_files_or_use_an_include_pattern:n(6307,e.DiagnosticCategory.Error,"File_0_is_not_listed_within_the_file_list_of_project_1_Projects_must_list_all_files_or_use_an_includ_6307","File '{0}' is not listed within the file list of project '{1}'. Projects must list all files or use an 'include' pattern."),Cannot_prepend_project_0_because_it_does_not_have_outFile_set:n(6308,e.DiagnosticCategory.Error,"Cannot_prepend_project_0_because_it_does_not_have_outFile_set_6308","Cannot prepend project '{0}' because it does not have 'outFile' set"),Output_file_0_from_project_1_does_not_exist:n(6309,e.DiagnosticCategory.Error,"Output_file_0_from_project_1_does_not_exist_6309","Output file '{0}' from project '{1}' does not exist"),Referenced_project_0_may_not_disable_emit:n(6310,e.DiagnosticCategory.Error,"Referenced_project_0_may_not_disable_emit_6310","Referenced project '{0}' may not disable emit."),Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2:n(6350,e.DiagnosticCategory.Message,"Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2_6350","Project '{0}' is out of date because oldest output '{1}' is older than newest input '{2}'"),Project_0_is_up_to_date_because_newest_input_1_is_older_than_oldest_output_2:n(6351,e.DiagnosticCategory.Message,"Project_0_is_up_to_date_because_newest_input_1_is_older_than_oldest_output_2_6351","Project '{0}' is up to date because newest input '{1}' is older than oldest output '{2}'"),Project_0_is_out_of_date_because_output_file_1_does_not_exist:n(6352,e.DiagnosticCategory.Message,"Project_0_is_out_of_date_because_output_file_1_does_not_exist_6352","Project '{0}' is out of date because output file '{1}' does not exist"),Project_0_is_out_of_date_because_its_dependency_1_is_out_of_date:n(6353,e.DiagnosticCategory.Message,"Project_0_is_out_of_date_because_its_dependency_1_is_out_of_date_6353","Project '{0}' is out of date because its dependency '{1}' is out of date"),Project_0_is_up_to_date_with_d_ts_files_from_its_dependencies:n(6354,e.DiagnosticCategory.Message,"Project_0_is_up_to_date_with_d_ts_files_from_its_dependencies_6354","Project '{0}' is up to date with .d.ts files from its dependencies"),Projects_in_this_build_Colon_0:n(6355,e.DiagnosticCategory.Message,"Projects_in_this_build_Colon_0_6355","Projects in this build: {0}"),A_non_dry_build_would_delete_the_following_files_Colon_0:n(6356,e.DiagnosticCategory.Message,"A_non_dry_build_would_delete_the_following_files_Colon_0_6356","A non-dry build would delete the following files: {0}"),A_non_dry_build_would_build_project_0:n(6357,e.DiagnosticCategory.Message,"A_non_dry_build_would_build_project_0_6357","A non-dry build would build project '{0}'"),Building_project_0:n(6358,e.DiagnosticCategory.Message,"Building_project_0_6358","Building project '{0}'..."),Updating_output_timestamps_of_project_0:n(6359,e.DiagnosticCategory.Message,"Updating_output_timestamps_of_project_0_6359","Updating output timestamps of project '{0}'..."),delete_this_Project_0_is_up_to_date_because_it_was_previously_built:n(6360,e.DiagnosticCategory.Message,"delete_this_Project_0_is_up_to_date_because_it_was_previously_built_6360","delete this - Project '{0}' is up to date because it was previously built"),Project_0_is_up_to_date:n(6361,e.DiagnosticCategory.Message,"Project_0_is_up_to_date_6361","Project '{0}' is up to date"),Skipping_build_of_project_0_because_its_dependency_1_has_errors:n(6362,e.DiagnosticCategory.Message,"Skipping_build_of_project_0_because_its_dependency_1_has_errors_6362","Skipping build of project '{0}' because its dependency '{1}' has errors"),Project_0_can_t_be_built_because_its_dependency_1_has_errors:n(6363,e.DiagnosticCategory.Message,"Project_0_can_t_be_built_because_its_dependency_1_has_errors_6363","Project '{0}' can't be built because its dependency '{1}' has errors"),Build_one_or_more_projects_and_their_dependencies_if_out_of_date:n(6364,e.DiagnosticCategory.Message,"Build_one_or_more_projects_and_their_dependencies_if_out_of_date_6364","Build one or more projects and their dependencies, if out of date"),Delete_the_outputs_of_all_projects:n(6365,e.DiagnosticCategory.Message,"Delete_the_outputs_of_all_projects_6365","Delete the outputs of all projects"),Show_what_would_be_built_or_deleted_if_specified_with_clean:n(6367,e.DiagnosticCategory.Message,"Show_what_would_be_built_or_deleted_if_specified_with_clean_6367","Show what would be built (or deleted, if specified with '--clean')"),Option_build_must_be_the_first_command_line_argument:n(6369,e.DiagnosticCategory.Error,"Option_build_must_be_the_first_command_line_argument_6369","Option '--build' must be the first command line argument."),Options_0_and_1_cannot_be_combined:n(6370,e.DiagnosticCategory.Error,"Options_0_and_1_cannot_be_combined_6370","Options '{0}' and '{1}' cannot be combined."),Updating_unchanged_output_timestamps_of_project_0:n(6371,e.DiagnosticCategory.Message,"Updating_unchanged_output_timestamps_of_project_0_6371","Updating unchanged output timestamps of project '{0}'..."),Project_0_is_out_of_date_because_output_of_its_dependency_1_has_changed:n(6372,e.DiagnosticCategory.Message,"Project_0_is_out_of_date_because_output_of_its_dependency_1_has_changed_6372","Project '{0}' is out of date because output of its dependency '{1}' has changed"),Updating_output_of_project_0:n(6373,e.DiagnosticCategory.Message,"Updating_output_of_project_0_6373","Updating output of project '{0}'..."),A_non_dry_build_would_update_timestamps_for_output_of_project_0:n(6374,e.DiagnosticCategory.Message,"A_non_dry_build_would_update_timestamps_for_output_of_project_0_6374","A non-dry build would update timestamps for output of project '{0}'"),A_non_dry_build_would_update_output_of_project_0:n(6375,e.DiagnosticCategory.Message,"A_non_dry_build_would_update_output_of_project_0_6375","A non-dry build would update output of project '{0}'"),Cannot_update_output_of_project_0_because_there_was_error_reading_file_1:n(6376,e.DiagnosticCategory.Message,"Cannot_update_output_of_project_0_because_there_was_error_reading_file_1_6376","Cannot update output of project '{0}' because there was error reading file '{1}'"),Cannot_write_file_0_because_it_will_overwrite_tsbuildinfo_file_generated_by_referenced_project_1:n(6377,e.DiagnosticCategory.Error,"Cannot_write_file_0_because_it_will_overwrite_tsbuildinfo_file_generated_by_referenced_project_1_6377","Cannot write file '{0}' because it will overwrite '.tsbuildinfo' file generated by referenced project '{1}'"),Enable_incremental_compilation:n(6378,e.DiagnosticCategory.Message,"Enable_incremental_compilation_6378","Enable incremental compilation"),Composite_projects_may_not_disable_incremental_compilation:n(6379,e.DiagnosticCategory.Error,"Composite_projects_may_not_disable_incremental_compilation_6379","Composite projects may not disable incremental compilation."),Specify_file_to_store_incremental_compilation_information:n(6380,e.DiagnosticCategory.Message,"Specify_file_to_store_incremental_compilation_information_6380","Specify file to store incremental compilation information"),Project_0_is_out_of_date_because_output_for_it_was_generated_with_version_1_that_differs_with_current_version_2:n(6381,e.DiagnosticCategory.Message,"Project_0_is_out_of_date_because_output_for_it_was_generated_with_version_1_that_differs_with_curren_6381","Project '{0}' is out of date because output for it was generated with version '{1}' that differs with current version '{2}'"),Skipping_build_of_project_0_because_its_dependency_1_was_not_built:n(6382,e.DiagnosticCategory.Message,"Skipping_build_of_project_0_because_its_dependency_1_was_not_built_6382","Skipping build of project '{0}' because its dependency '{1}' was not built"),Project_0_can_t_be_built_because_its_dependency_1_was_not_built:n(6383,e.DiagnosticCategory.Message,"Project_0_can_t_be_built_because_its_dependency_1_was_not_built_6383","Project '{0}' can't be built because its dependency '{1}' was not built"),Have_recompiles_in_incremental_and_watch_assume_that_changes_within_a_file_will_only_affect_files_directly_depending_on_it:n(6384,e.DiagnosticCategory.Message,"Have_recompiles_in_incremental_and_watch_assume_that_changes_within_a_file_will_only_affect_files_di_6384","Have recompiles in '--incremental' and '--watch' assume that changes within a file will only affect files directly depending on it."),_0_is_deprecated:n(6385,e.DiagnosticCategory.Suggestion,"_0_is_deprecated_6385","'{0}' is deprecated.",void 0,void 0,!0),Performance_timings_for_diagnostics_or_extendedDiagnostics_are_not_available_in_this_session_A_native_implementation_of_the_Web_Performance_API_could_not_be_found:n(6386,e.DiagnosticCategory.Message,"Performance_timings_for_diagnostics_or_extendedDiagnostics_are_not_available_in_this_session_A_nativ_6386","Performance timings for '--diagnostics' or '--extendedDiagnostics' are not available in this session. A native implementation of the Web Performance API could not be found."),The_signature_0_of_1_is_deprecated:n(6387,e.DiagnosticCategory.Suggestion,"The_signature_0_of_1_is_deprecated_6387","The signature '{0}' of '{1}' is deprecated.",void 0,void 0,!0),Project_0_is_being_forcibly_rebuilt:n(6388,e.DiagnosticCategory.Message,"Project_0_is_being_forcibly_rebuilt_6388","Project '{0}' is being forcibly rebuilt"),Reusing_resolution_of_module_0_from_1_of_old_program_it_was_not_resolved:n(6389,e.DiagnosticCategory.Message,"Reusing_resolution_of_module_0_from_1_of_old_program_it_was_not_resolved_6389","Reusing resolution of module '{0}' from '{1}' of old program, it was not resolved."),Reusing_resolution_of_type_reference_directive_0_from_1_of_old_program_it_was_successfully_resolved_to_2:n(6390,e.DiagnosticCategory.Message,"Reusing_resolution_of_type_reference_directive_0_from_1_of_old_program_it_was_successfully_resolved__6390","Reusing resolution of type reference directive '{0}' from '{1}' of old program, it was successfully resolved to '{2}'."),Reusing_resolution_of_type_reference_directive_0_from_1_of_old_program_it_was_successfully_resolved_to_2_with_Package_ID_3:n(6391,e.DiagnosticCategory.Message,"Reusing_resolution_of_type_reference_directive_0_from_1_of_old_program_it_was_successfully_resolved__6391","Reusing resolution of type reference directive '{0}' from '{1}' of old program, it was successfully resolved to '{2}' with Package ID '{3}'."),Reusing_resolution_of_type_reference_directive_0_from_1_of_old_program_it_was_not_resolved:n(6392,e.DiagnosticCategory.Message,"Reusing_resolution_of_type_reference_directive_0_from_1_of_old_program_it_was_not_resolved_6392","Reusing resolution of type reference directive '{0}' from '{1}' of old program, it was not resolved."),Reusing_resolution_of_module_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_3:n(6393,e.DiagnosticCategory.Message,"Reusing_resolution_of_module_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_6393","Reusing resolution of module '{0}' from '{1}' found in cache from location '{2}', it was successfully resolved to '{3}'."),Reusing_resolution_of_module_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_3_with_Package_ID_4:n(6394,e.DiagnosticCategory.Message,"Reusing_resolution_of_module_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_6394","Reusing resolution of module '{0}' from '{1}' found in cache from location '{2}', it was successfully resolved to '{3}' with Package ID '{4}'."),Reusing_resolution_of_module_0_from_1_found_in_cache_from_location_2_it_was_not_resolved:n(6395,e.DiagnosticCategory.Message,"Reusing_resolution_of_module_0_from_1_found_in_cache_from_location_2_it_was_not_resolved_6395","Reusing resolution of module '{0}' from '{1}' found in cache from location '{2}', it was not resolved."),Reusing_resolution_of_type_reference_directive_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_3:n(6396,e.DiagnosticCategory.Message,"Reusing_resolution_of_type_reference_directive_0_from_1_found_in_cache_from_location_2_it_was_succes_6396","Reusing resolution of type reference directive '{0}' from '{1}' found in cache from location '{2}', it was successfully resolved to '{3}'."),Reusing_resolution_of_type_reference_directive_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_3_with_Package_ID_4:n(6397,e.DiagnosticCategory.Message,"Reusing_resolution_of_type_reference_directive_0_from_1_found_in_cache_from_location_2_it_was_succes_6397","Reusing resolution of type reference directive '{0}' from '{1}' found in cache from location '{2}', it was successfully resolved to '{3}' with Package ID '{4}'."),Reusing_resolution_of_type_reference_directive_0_from_1_found_in_cache_from_location_2_it_was_not_resolved:n(6398,e.DiagnosticCategory.Message,"Reusing_resolution_of_type_reference_directive_0_from_1_found_in_cache_from_location_2_it_was_not_re_6398","Reusing resolution of type reference directive '{0}' from '{1}' found in cache from location '{2}', it was not resolved."),The_expected_type_comes_from_property_0_which_is_declared_here_on_type_1:n(6500,e.DiagnosticCategory.Message,"The_expected_type_comes_from_property_0_which_is_declared_here_on_type_1_6500","The expected type comes from property '{0}' which is declared here on type '{1}'"),The_expected_type_comes_from_this_index_signature:n(6501,e.DiagnosticCategory.Message,"The_expected_type_comes_from_this_index_signature_6501","The expected type comes from this index signature."),The_expected_type_comes_from_the_return_type_of_this_signature:n(6502,e.DiagnosticCategory.Message,"The_expected_type_comes_from_the_return_type_of_this_signature_6502","The expected type comes from the return type of this signature."),Print_names_of_files_that_are_part_of_the_compilation_and_then_stop_processing:n(6503,e.DiagnosticCategory.Message,"Print_names_of_files_that_are_part_of_the_compilation_and_then_stop_processing_6503","Print names of files that are part of the compilation and then stop processing."),File_0_is_a_JavaScript_file_Did_you_mean_to_enable_the_allowJs_option:n(6504,e.DiagnosticCategory.Error,"File_0_is_a_JavaScript_file_Did_you_mean_to_enable_the_allowJs_option_6504","File '{0}' is a JavaScript file. Did you mean to enable the 'allowJs' option?"),Print_names_of_files_and_the_reason_they_are_part_of_the_compilation:n(6505,e.DiagnosticCategory.Message,"Print_names_of_files_and_the_reason_they_are_part_of_the_compilation_6505","Print names of files and the reason they are part of the compilation."),Consider_adding_a_declare_modifier_to_this_class:n(6506,e.DiagnosticCategory.Message,"Consider_adding_a_declare_modifier_to_this_class_6506","Consider adding a 'declare' modifier to this class."),Allow_JavaScript_files_to_be_a_part_of_your_program_Use_the_checkJS_option_to_get_errors_from_these_files:n(6600,e.DiagnosticCategory.Message,"Allow_JavaScript_files_to_be_a_part_of_your_program_Use_the_checkJS_option_to_get_errors_from_these__6600","Allow JavaScript files to be a part of your program. Use the `checkJS` option to get errors from these files."),Allow_import_x_from_y_when_a_module_doesn_t_have_a_default_export:n(6601,e.DiagnosticCategory.Message,"Allow_import_x_from_y_when_a_module_doesn_t_have_a_default_export_6601","Allow 'import x from y' when a module doesn't have a default export."),Allow_accessing_UMD_globals_from_modules:n(6602,e.DiagnosticCategory.Message,"Allow_accessing_UMD_globals_from_modules_6602","Allow accessing UMD globals from modules."),Disable_error_reporting_for_unreachable_code:n(6603,e.DiagnosticCategory.Message,"Disable_error_reporting_for_unreachable_code_6603","Disable error reporting for unreachable code."),Disable_error_reporting_for_unused_labels:n(6604,e.DiagnosticCategory.Message,"Disable_error_reporting_for_unused_labels_6604","Disable error reporting for unused labels."),Ensure_use_strict_is_always_emitted:n(6605,e.DiagnosticCategory.Message,"Ensure_use_strict_is_always_emitted_6605","Ensure 'use strict' is always emitted."),Have_recompiles_in_projects_that_use_incremental_and_watch_mode_assume_that_changes_within_a_file_will_only_affect_files_directly_depending_on_it:n(6606,e.DiagnosticCategory.Message,"Have_recompiles_in_projects_that_use_incremental_and_watch_mode_assume_that_changes_within_a_file_wi_6606","Have recompiles in projects that use `incremental` and `watch` mode assume that changes within a file will only affect files directly depending on it."),Specify_the_base_directory_to_resolve_non_relative_module_names:n(6607,e.DiagnosticCategory.Message,"Specify_the_base_directory_to_resolve_non_relative_module_names_6607","Specify the base directory to resolve non-relative module names."),No_longer_supported_In_early_versions_manually_set_the_text_encoding_for_reading_files:n(6608,e.DiagnosticCategory.Message,"No_longer_supported_In_early_versions_manually_set_the_text_encoding_for_reading_files_6608","No longer supported. In early versions, manually set the text encoding for reading files."),Enable_error_reporting_in_type_checked_JavaScript_files:n(6609,e.DiagnosticCategory.Message,"Enable_error_reporting_in_type_checked_JavaScript_files_6609","Enable error reporting in type-checked JavaScript files."),Enable_constraints_that_allow_a_TypeScript_project_to_be_used_with_project_references:n(6611,e.DiagnosticCategory.Message,"Enable_constraints_that_allow_a_TypeScript_project_to_be_used_with_project_references_6611","Enable constraints that allow a TypeScript project to be used with project references."),Generate_d_ts_files_from_TypeScript_and_JavaScript_files_in_your_project:n(6612,e.DiagnosticCategory.Message,"Generate_d_ts_files_from_TypeScript_and_JavaScript_files_in_your_project_6612","Generate .d.ts files from TypeScript and JavaScript files in your project."),Specify_the_output_directory_for_generated_declaration_files:n(6613,e.DiagnosticCategory.Message,"Specify_the_output_directory_for_generated_declaration_files_6613","Specify the output directory for generated declaration files."),Create_sourcemaps_for_d_ts_files:n(6614,e.DiagnosticCategory.Message,"Create_sourcemaps_for_d_ts_files_6614","Create sourcemaps for d.ts files."),Output_compiler_performance_information_after_building:n(6615,e.DiagnosticCategory.Message,"Output_compiler_performance_information_after_building_6615","Output compiler performance information after building."),Disables_inference_for_type_acquisition_by_looking_at_filenames_in_a_project:n(6616,e.DiagnosticCategory.Message,"Disables_inference_for_type_acquisition_by_looking_at_filenames_in_a_project_6616","Disables inference for type acquisition by looking at filenames in a project."),Reduce_the_number_of_projects_loaded_automatically_by_TypeScript:n(6617,e.DiagnosticCategory.Message,"Reduce_the_number_of_projects_loaded_automatically_by_TypeScript_6617","Reduce the number of projects loaded automatically by TypeScript."),Remove_the_20mb_cap_on_total_source_code_size_for_JavaScript_files_in_the_TypeScript_language_server:n(6618,e.DiagnosticCategory.Message,"Remove_the_20mb_cap_on_total_source_code_size_for_JavaScript_files_in_the_TypeScript_language_server_6618","Remove the 20mb cap on total source code size for JavaScript files in the TypeScript language server."),Opt_a_project_out_of_multi_project_reference_checking_when_editing:n(6619,e.DiagnosticCategory.Message,"Opt_a_project_out_of_multi_project_reference_checking_when_editing_6619","Opt a project out of multi-project reference checking when editing."),Disable_preferring_source_files_instead_of_declaration_files_when_referencing_composite_projects:n(6620,e.DiagnosticCategory.Message,"Disable_preferring_source_files_instead_of_declaration_files_when_referencing_composite_projects_6620","Disable preferring source files instead of declaration files when referencing composite projects"),Emit_more_compliant_but_verbose_and_less_performant_JavaScript_for_iteration:n(6621,e.DiagnosticCategory.Message,"Emit_more_compliant_but_verbose_and_less_performant_JavaScript_for_iteration_6621","Emit more compliant, but verbose and less performant JavaScript for iteration."),Only_output_d_ts_files_and_not_JavaScript_files:n(6623,e.DiagnosticCategory.Message,"Only_output_d_ts_files_and_not_JavaScript_files_6623","Only output d.ts files and not JavaScript files."),Emit_design_type_metadata_for_decorated_declarations_in_source_files:n(6624,e.DiagnosticCategory.Message,"Emit_design_type_metadata_for_decorated_declarations_in_source_files_6624","Emit design-type metadata for decorated declarations in source files."),Disable_the_type_acquisition_for_JavaScript_projects:n(6625,e.DiagnosticCategory.Message,"Disable_the_type_acquisition_for_JavaScript_projects_6625","Disable the type acquisition for JavaScript projects"),Emit_additional_JavaScript_to_ease_support_for_importing_CommonJS_modules_This_enables_allowSyntheticDefaultImports_for_type_compatibility:n(6626,e.DiagnosticCategory.Message,"Emit_additional_JavaScript_to_ease_support_for_importing_CommonJS_modules_This_enables_allowSyntheti_6626","Emit additional JavaScript to ease support for importing CommonJS modules. This enables `allowSyntheticDefaultImports` for type compatibility."),Filters_results_from_the_include_option:n(6627,e.DiagnosticCategory.Message,"Filters_results_from_the_include_option_6627","Filters results from the `include` option."),Remove_a_list_of_directories_from_the_watch_process:n(6628,e.DiagnosticCategory.Message,"Remove_a_list_of_directories_from_the_watch_process_6628","Remove a list of directories from the watch process."),Remove_a_list_of_files_from_the_watch_mode_s_processing:n(6629,e.DiagnosticCategory.Message,"Remove_a_list_of_files_from_the_watch_mode_s_processing_6629","Remove a list of files from the watch mode's processing."),Enable_experimental_support_for_TC39_stage_2_draft_decorators:n(6630,e.DiagnosticCategory.Message,"Enable_experimental_support_for_TC39_stage_2_draft_decorators_6630","Enable experimental support for TC39 stage 2 draft decorators."),Print_files_read_during_the_compilation_including_why_it_was_included:n(6631,e.DiagnosticCategory.Message,"Print_files_read_during_the_compilation_including_why_it_was_included_6631","Print files read during the compilation including why it was included."),Output_more_detailed_compiler_performance_information_after_building:n(6632,e.DiagnosticCategory.Message,"Output_more_detailed_compiler_performance_information_after_building_6632","Output more detailed compiler performance information after building."),Specify_one_or_more_path_or_node_module_references_to_base_configuration_files_from_which_settings_are_inherited:n(6633,e.DiagnosticCategory.Message,"Specify_one_or_more_path_or_node_module_references_to_base_configuration_files_from_which_settings_a_6633","Specify one or more path or node module references to base configuration files from which settings are inherited."),Specify_what_approach_the_watcher_should_use_if_the_system_runs_out_of_native_file_watchers:n(6634,e.DiagnosticCategory.Message,"Specify_what_approach_the_watcher_should_use_if_the_system_runs_out_of_native_file_watchers_6634","Specify what approach the watcher should use if the system runs out of native file watchers."),Include_a_list_of_files_This_does_not_support_glob_patterns_as_opposed_to_include:n(6635,e.DiagnosticCategory.Message,"Include_a_list_of_files_This_does_not_support_glob_patterns_as_opposed_to_include_6635","Include a list of files. This does not support glob patterns, as opposed to `include`."),Build_all_projects_including_those_that_appear_to_be_up_to_date:n(6636,e.DiagnosticCategory.Message,"Build_all_projects_including_those_that_appear_to_be_up_to_date_6636","Build all projects, including those that appear to be up to date"),Ensure_that_casing_is_correct_in_imports:n(6637,e.DiagnosticCategory.Message,"Ensure_that_casing_is_correct_in_imports_6637","Ensure that casing is correct in imports."),Emit_a_v8_CPU_profile_of_the_compiler_run_for_debugging:n(6638,e.DiagnosticCategory.Message,"Emit_a_v8_CPU_profile_of_the_compiler_run_for_debugging_6638","Emit a v8 CPU profile of the compiler run for debugging."),Allow_importing_helper_functions_from_tslib_once_per_project_instead_of_including_them_per_file:n(6639,e.DiagnosticCategory.Message,"Allow_importing_helper_functions_from_tslib_once_per_project_instead_of_including_them_per_file_6639","Allow importing helper functions from tslib once per project, instead of including them per-file."),Specify_a_list_of_glob_patterns_that_match_files_to_be_included_in_compilation:n(6641,e.DiagnosticCategory.Message,"Specify_a_list_of_glob_patterns_that_match_files_to_be_included_in_compilation_6641","Specify a list of glob patterns that match files to be included in compilation."),Save_tsbuildinfo_files_to_allow_for_incremental_compilation_of_projects:n(6642,e.DiagnosticCategory.Message,"Save_tsbuildinfo_files_to_allow_for_incremental_compilation_of_projects_6642","Save .tsbuildinfo files to allow for incremental compilation of projects."),Include_sourcemap_files_inside_the_emitted_JavaScript:n(6643,e.DiagnosticCategory.Message,"Include_sourcemap_files_inside_the_emitted_JavaScript_6643","Include sourcemap files inside the emitted JavaScript."),Include_source_code_in_the_sourcemaps_inside_the_emitted_JavaScript:n(6644,e.DiagnosticCategory.Message,"Include_source_code_in_the_sourcemaps_inside_the_emitted_JavaScript_6644","Include source code in the sourcemaps inside the emitted JavaScript."),Ensure_that_each_file_can_be_safely_transpiled_without_relying_on_other_imports:n(6645,e.DiagnosticCategory.Message,"Ensure_that_each_file_can_be_safely_transpiled_without_relying_on_other_imports_6645","Ensure that each file can be safely transpiled without relying on other imports."),Specify_what_JSX_code_is_generated:n(6646,e.DiagnosticCategory.Message,"Specify_what_JSX_code_is_generated_6646","Specify what JSX code is generated."),Specify_the_JSX_factory_function_used_when_targeting_React_JSX_emit_e_g_React_createElement_or_h:n(6647,e.DiagnosticCategory.Message,"Specify_the_JSX_factory_function_used_when_targeting_React_JSX_emit_e_g_React_createElement_or_h_6647","Specify the JSX factory function used when targeting React JSX emit, e.g. 'React.createElement' or 'h'"),Specify_the_JSX_Fragment_reference_used_for_fragments_when_targeting_React_JSX_emit_e_g_React_Fragment_or_Fragment:n(6648,e.DiagnosticCategory.Message,"Specify_the_JSX_Fragment_reference_used_for_fragments_when_targeting_React_JSX_emit_e_g_React_Fragme_6648","Specify the JSX Fragment reference used for fragments when targeting React JSX emit e.g. 'React.Fragment' or 'Fragment'."),Specify_module_specifier_used_to_import_the_JSX_factory_functions_when_using_jsx_Colon_react_jsx_Asterisk:n(6649,e.DiagnosticCategory.Message,"Specify_module_specifier_used_to_import_the_JSX_factory_functions_when_using_jsx_Colon_react_jsx_Ast_6649","Specify module specifier used to import the JSX factory functions when using `jsx: react-jsx*`.`"),Make_keyof_only_return_strings_instead_of_string_numbers_or_symbols_Legacy_option:n(6650,e.DiagnosticCategory.Message,"Make_keyof_only_return_strings_instead_of_string_numbers_or_symbols_Legacy_option_6650","Make keyof only return strings instead of string, numbers or symbols. Legacy option."),Specify_a_set_of_bundled_library_declaration_files_that_describe_the_target_runtime_environment:n(6651,e.DiagnosticCategory.Message,"Specify_a_set_of_bundled_library_declaration_files_that_describe_the_target_runtime_environment_6651","Specify a set of bundled library declaration files that describe the target runtime environment."),Print_the_names_of_emitted_files_after_a_compilation:n(6652,e.DiagnosticCategory.Message,"Print_the_names_of_emitted_files_after_a_compilation_6652","Print the names of emitted files after a compilation."),Print_all_of_the_files_read_during_the_compilation:n(6653,e.DiagnosticCategory.Message,"Print_all_of_the_files_read_during_the_compilation_6653","Print all of the files read during the compilation."),Set_the_language_of_the_messaging_from_TypeScript_This_does_not_affect_emit:n(6654,e.DiagnosticCategory.Message,"Set_the_language_of_the_messaging_from_TypeScript_This_does_not_affect_emit_6654","Set the language of the messaging from TypeScript. This does not affect emit."),Specify_the_maximum_folder_depth_used_for_checking_JavaScript_files_from_node_modules_Only_applicable_with_allowJs:n(6656,e.DiagnosticCategory.Message,"Specify_the_maximum_folder_depth_used_for_checking_JavaScript_files_from_node_modules_Only_applicabl_6656","Specify the maximum folder depth used for checking JavaScript files from `node_modules`. Only applicable with `allowJs`."),Specify_what_module_code_is_generated:n(6657,e.DiagnosticCategory.Message,"Specify_what_module_code_is_generated_6657","Specify what module code is generated."),Specify_how_TypeScript_looks_up_a_file_from_a_given_module_specifier:n(6658,e.DiagnosticCategory.Message,"Specify_how_TypeScript_looks_up_a_file_from_a_given_module_specifier_6658","Specify how TypeScript looks up a file from a given module specifier."),Set_the_newline_character_for_emitting_files:n(6659,e.DiagnosticCategory.Message,"Set_the_newline_character_for_emitting_files_6659","Set the newline character for emitting files."),Disable_emitting_files_from_a_compilation:n(6660,e.DiagnosticCategory.Message,"Disable_emitting_files_from_a_compilation_6660","Disable emitting files from a compilation."),Disable_generating_custom_helper_functions_like_extends_in_compiled_output:n(6661,e.DiagnosticCategory.Message,"Disable_generating_custom_helper_functions_like_extends_in_compiled_output_6661","Disable generating custom helper functions like `__extends` in compiled output."),Disable_emitting_files_if_any_type_checking_errors_are_reported:n(6662,e.DiagnosticCategory.Message,"Disable_emitting_files_if_any_type_checking_errors_are_reported_6662","Disable emitting files if any type checking errors are reported."),Disable_truncating_types_in_error_messages:n(6663,e.DiagnosticCategory.Message,"Disable_truncating_types_in_error_messages_6663","Disable truncating types in error messages."),Enable_error_reporting_for_fallthrough_cases_in_switch_statements:n(6664,e.DiagnosticCategory.Message,"Enable_error_reporting_for_fallthrough_cases_in_switch_statements_6664","Enable error reporting for fallthrough cases in switch statements."),Enable_error_reporting_for_expressions_and_declarations_with_an_implied_any_type:n(6665,e.DiagnosticCategory.Message,"Enable_error_reporting_for_expressions_and_declarations_with_an_implied_any_type_6665","Enable error reporting for expressions and declarations with an implied `any` type.."),Ensure_overriding_members_in_derived_classes_are_marked_with_an_override_modifier:n(6666,e.DiagnosticCategory.Message,"Ensure_overriding_members_in_derived_classes_are_marked_with_an_override_modifier_6666","Ensure overriding members in derived classes are marked with an override modifier."),Enable_error_reporting_for_codepaths_that_do_not_explicitly_return_in_a_function:n(6667,e.DiagnosticCategory.Message,"Enable_error_reporting_for_codepaths_that_do_not_explicitly_return_in_a_function_6667","Enable error reporting for codepaths that do not explicitly return in a function."),Enable_error_reporting_when_this_is_given_the_type_any:n(6668,e.DiagnosticCategory.Message,"Enable_error_reporting_when_this_is_given_the_type_any_6668","Enable error reporting when `this` is given the type `any`."),Disable_adding_use_strict_directives_in_emitted_JavaScript_files:n(6669,e.DiagnosticCategory.Message,"Disable_adding_use_strict_directives_in_emitted_JavaScript_files_6669","Disable adding 'use strict' directives in emitted JavaScript files."),Disable_including_any_library_files_including_the_default_lib_d_ts:n(6670,e.DiagnosticCategory.Message,"Disable_including_any_library_files_including_the_default_lib_d_ts_6670","Disable including any library files, including the default lib.d.ts."),Enforces_using_indexed_accessors_for_keys_declared_using_an_indexed_type:n(6671,e.DiagnosticCategory.Message,"Enforces_using_indexed_accessors_for_keys_declared_using_an_indexed_type_6671","Enforces using indexed accessors for keys declared using an indexed type"),Disallow_import_s_require_s_or_reference_s_from_expanding_the_number_of_files_TypeScript_should_add_to_a_project:n(6672,e.DiagnosticCategory.Message,"Disallow_import_s_require_s_or_reference_s_from_expanding_the_number_of_files_TypeScript_should_add__6672","Disallow `import`s, `require`s or `<reference>`s from expanding the number of files TypeScript should add to a project."),Disable_strict_checking_of_generic_signatures_in_function_types:n(6673,e.DiagnosticCategory.Message,"Disable_strict_checking_of_generic_signatures_in_function_types_6673","Disable strict checking of generic signatures in function types."),Add_undefined_to_a_type_when_accessed_using_an_index:n(6674,e.DiagnosticCategory.Message,"Add_undefined_to_a_type_when_accessed_using_an_index_6674","Add `undefined` to a type when accessed using an index."),Enable_error_reporting_when_a_local_variables_aren_t_read:n(6675,e.DiagnosticCategory.Message,"Enable_error_reporting_when_a_local_variables_aren_t_read_6675","Enable error reporting when a local variables aren't read."),Raise_an_error_when_a_function_parameter_isn_t_read:n(6676,e.DiagnosticCategory.Message,"Raise_an_error_when_a_function_parameter_isn_t_read_6676","Raise an error when a function parameter isn't read"),Deprecated_setting_Use_outFile_instead:n(6677,e.DiagnosticCategory.Message,"Deprecated_setting_Use_outFile_instead_6677","Deprecated setting. Use `outFile` instead."),Specify_an_output_folder_for_all_emitted_files:n(6678,e.DiagnosticCategory.Message,"Specify_an_output_folder_for_all_emitted_files_6678","Specify an output folder for all emitted files."),Specify_a_file_that_bundles_all_outputs_into_one_JavaScript_file_If_declaration_is_true_also_designates_a_file_that_bundles_all_d_ts_output:n(6679,e.DiagnosticCategory.Message,"Specify_a_file_that_bundles_all_outputs_into_one_JavaScript_file_If_declaration_is_true_also_designa_6679","Specify a file that bundles all outputs into one JavaScript file. If `declaration` is true, also designates a file that bundles all .d.ts output."),Specify_a_set_of_entries_that_re_map_imports_to_additional_lookup_locations:n(6680,e.DiagnosticCategory.Message,"Specify_a_set_of_entries_that_re_map_imports_to_additional_lookup_locations_6680","Specify a set of entries that re-map imports to additional lookup locations."),Specify_a_list_of_language_service_plugins_to_include:n(6681,e.DiagnosticCategory.Message,"Specify_a_list_of_language_service_plugins_to_include_6681","Specify a list of language service plugins to include."),Disable_erasing_const_enum_declarations_in_generated_code:n(6682,e.DiagnosticCategory.Message,"Disable_erasing_const_enum_declarations_in_generated_code_6682","Disable erasing `const enum` declarations in generated code."),Disable_resolving_symlinks_to_their_realpath_This_correlates_to_the_same_flag_in_node:n(6683,e.DiagnosticCategory.Message,"Disable_resolving_symlinks_to_their_realpath_This_correlates_to_the_same_flag_in_node_6683","Disable resolving symlinks to their realpath. This correlates to the same flag in node."),Disable_wiping_the_console_in_watch_mode:n(6684,e.DiagnosticCategory.Message,"Disable_wiping_the_console_in_watch_mode_6684","Disable wiping the console in watch mode"),Enable_color_and_formatting_in_TypeScript_s_output_to_make_compiler_errors_easier_to_read:n(6685,e.DiagnosticCategory.Message,"Enable_color_and_formatting_in_TypeScript_s_output_to_make_compiler_errors_easier_to_read_6685","Enable color and formatting in TypeScript's output to make compiler errors easier to read"),Specify_the_object_invoked_for_createElement_This_only_applies_when_targeting_react_JSX_emit:n(6686,e.DiagnosticCategory.Message,"Specify_the_object_invoked_for_createElement_This_only_applies_when_targeting_react_JSX_emit_6686","Specify the object invoked for `createElement`. This only applies when targeting `react` JSX emit."),Specify_an_array_of_objects_that_specify_paths_for_projects_Used_in_project_references:n(6687,e.DiagnosticCategory.Message,"Specify_an_array_of_objects_that_specify_paths_for_projects_Used_in_project_references_6687","Specify an array of objects that specify paths for projects. Used in project references."),Disable_emitting_comments:n(6688,e.DiagnosticCategory.Message,"Disable_emitting_comments_6688","Disable emitting comments."),Enable_importing_json_files:n(6689,e.DiagnosticCategory.Message,"Enable_importing_json_files_6689","Enable importing .json files"),Specify_the_root_folder_within_your_source_files:n(6690,e.DiagnosticCategory.Message,"Specify_the_root_folder_within_your_source_files_6690","Specify the root folder within your source files."),Allow_multiple_folders_to_be_treated_as_one_when_resolving_modules:n(6691,e.DiagnosticCategory.Message,"Allow_multiple_folders_to_be_treated_as_one_when_resolving_modules_6691","Allow multiple folders to be treated as one when resolving modules."),Skip_type_checking_d_ts_files_that_are_included_with_TypeScript:n(6692,e.DiagnosticCategory.Message,"Skip_type_checking_d_ts_files_that_are_included_with_TypeScript_6692","Skip type checking .d.ts files that are included with TypeScript."),Skip_type_checking_all_d_ts_files:n(6693,e.DiagnosticCategory.Message,"Skip_type_checking_all_d_ts_files_6693","Skip type checking all .d.ts files."),Create_source_map_files_for_emitted_JavaScript_files:n(6694,e.DiagnosticCategory.Message,"Create_source_map_files_for_emitted_JavaScript_files_6694","Create source map files for emitted JavaScript files."),Specify_the_root_path_for_debuggers_to_find_the_reference_source_code:n(6695,e.DiagnosticCategory.Message,"Specify_the_root_path_for_debuggers_to_find_the_reference_source_code_6695","Specify the root path for debuggers to find the reference source code."),Check_that_the_arguments_for_bind_call_and_apply_methods_match_the_original_function:n(6697,e.DiagnosticCategory.Message,"Check_that_the_arguments_for_bind_call_and_apply_methods_match_the_original_function_6697","Check that the arguments for `bind`, `call`, and `apply` methods match the original function."),When_assigning_functions_check_to_ensure_parameters_and_the_return_values_are_subtype_compatible:n(6698,e.DiagnosticCategory.Message,"When_assigning_functions_check_to_ensure_parameters_and_the_return_values_are_subtype_compatible_6698","When assigning functions, check to ensure parameters and the return values are subtype-compatible."),When_type_checking_take_into_account_null_and_undefined:n(6699,e.DiagnosticCategory.Message,"When_type_checking_take_into_account_null_and_undefined_6699","When type checking, take into account `null` and `undefined`."),Check_for_class_properties_that_are_declared_but_not_set_in_the_constructor:n(6700,e.DiagnosticCategory.Message,"Check_for_class_properties_that_are_declared_but_not_set_in_the_constructor_6700","Check for class properties that are declared but not set in the constructor."),Disable_emitting_declarations_that_have_internal_in_their_JSDoc_comments:n(6701,e.DiagnosticCategory.Message,"Disable_emitting_declarations_that_have_internal_in_their_JSDoc_comments_6701","Disable emitting declarations that have `@internal` in their JSDoc comments."),Disable_reporting_of_excess_property_errors_during_the_creation_of_object_literals:n(6702,e.DiagnosticCategory.Message,"Disable_reporting_of_excess_property_errors_during_the_creation_of_object_literals_6702","Disable reporting of excess property errors during the creation of object literals."),Suppress_noImplicitAny_errors_when_indexing_objects_that_lack_index_signatures:n(6703,e.DiagnosticCategory.Message,"Suppress_noImplicitAny_errors_when_indexing_objects_that_lack_index_signatures_6703","Suppress `noImplicitAny` errors when indexing objects that lack index signatures."),Synchronously_call_callbacks_and_update_the_state_of_directory_watchers_on_platforms_that_don_t_support_recursive_watching_natively:n(6704,e.DiagnosticCategory.Message,"Synchronously_call_callbacks_and_update_the_state_of_directory_watchers_on_platforms_that_don_t_supp_6704","Synchronously call callbacks and update the state of directory watchers on platforms that don`t support recursive watching natively."),Set_the_JavaScript_language_version_for_emitted_JavaScript_and_include_compatible_library_declarations:n(6705,e.DiagnosticCategory.Message,"Set_the_JavaScript_language_version_for_emitted_JavaScript_and_include_compatible_library_declaratio_6705","Set the JavaScript language version for emitted JavaScript and include compatible library declarations."),Log_paths_used_during_the_moduleResolution_process:n(6706,e.DiagnosticCategory.Message,"Log_paths_used_during_the_moduleResolution_process_6706","Log paths used during the `moduleResolution` process."),Specify_the_folder_for_tsbuildinfo_incremental_compilation_files:n(6707,e.DiagnosticCategory.Message,"Specify_the_folder_for_tsbuildinfo_incremental_compilation_files_6707","Specify the folder for .tsbuildinfo incremental compilation files."),Specify_options_for_automatic_acquisition_of_declaration_files:n(6709,e.DiagnosticCategory.Message,"Specify_options_for_automatic_acquisition_of_declaration_files_6709","Specify options for automatic acquisition of declaration files."),Specify_multiple_folders_that_act_like_Slashnode_modules_Slash_types:n(6710,e.DiagnosticCategory.Message,"Specify_multiple_folders_that_act_like_Slashnode_modules_Slash_types_6710","Specify multiple folders that act like `./node_modules/@types`."),Specify_type_package_names_to_be_included_without_being_referenced_in_a_source_file:n(6711,e.DiagnosticCategory.Message,"Specify_type_package_names_to_be_included_without_being_referenced_in_a_source_file_6711","Specify type package names to be included without being referenced in a source file."),Emit_ECMAScript_standard_compliant_class_fields:n(6712,e.DiagnosticCategory.Message,"Emit_ECMAScript_standard_compliant_class_fields_6712","Emit ECMAScript-standard-compliant class fields."),Enable_verbose_logging:n(6713,e.DiagnosticCategory.Message,"Enable_verbose_logging_6713","Enable verbose logging"),Specify_how_directories_are_watched_on_systems_that_lack_recursive_file_watching_functionality:n(6714,e.DiagnosticCategory.Message,"Specify_how_directories_are_watched_on_systems_that_lack_recursive_file_watching_functionality_6714","Specify how directories are watched on systems that lack recursive file-watching functionality."),Specify_how_the_TypeScript_watch_mode_works:n(6715,e.DiagnosticCategory.Message,"Specify_how_the_TypeScript_watch_mode_works_6715","Specify how the TypeScript watch mode works."),Include_undefined_in_index_signature_results:n(6716,e.DiagnosticCategory.Message,"Include_undefined_in_index_signature_results_6716","Include 'undefined' in index signature results"),Require_undeclared_properties_from_index_signatures_to_use_element_accesses:n(6717,e.DiagnosticCategory.Message,"Require_undeclared_properties_from_index_signatures_to_use_element_accesses_6717","Require undeclared properties from index signatures to use element accesses."),Specify_emit_Slashchecking_behavior_for_imports_that_are_only_used_for_types:n(6718,e.DiagnosticCategory.Message,"Specify_emit_Slashchecking_behavior_for_imports_that_are_only_used_for_types_6718","Specify emit/checking behavior for imports that are only used for types"),Type_catch_clause_variables_as_unknown_instead_of_any:n(6803,e.DiagnosticCategory.Message,"Type_catch_clause_variables_as_unknown_instead_of_any_6803","Type catch clause variables as 'unknown' instead of 'any'."),one_of_Colon:n(6900,e.DiagnosticCategory.Message,"one_of_Colon_6900","one of:"),one_or_more_Colon:n(6901,e.DiagnosticCategory.Message,"one_or_more_Colon_6901","one or more:"),type_Colon:n(6902,e.DiagnosticCategory.Message,"type_Colon_6902","type:"),default_Colon:n(6903,e.DiagnosticCategory.Message,"default_Colon_6903","default:"),module_system_or_esModuleInterop:n(6904,e.DiagnosticCategory.Message,"module_system_or_esModuleInterop_6904",'module === "system" or esModuleInterop'),false_unless_strict_is_set:n(6905,e.DiagnosticCategory.Message,"false_unless_strict_is_set_6905","`false`, unless `strict` is set"),false_unless_composite_is_set:n(6906,e.DiagnosticCategory.Message,"false_unless_composite_is_set_6906","`false`, unless `composite` is set"),node_modules_bower_components_jspm_packages_plus_the_value_of_outDir_if_one_is_specified:n(6907,e.DiagnosticCategory.Message,"node_modules_bower_components_jspm_packages_plus_the_value_of_outDir_if_one_is_specified_6907",'`["node_modules", "bower_components", "jspm_packages"]`, plus the value of `outDir` if one is specified.'),if_files_is_specified_otherwise_Asterisk_Asterisk_Slash_Asterisk:n(6908,e.DiagnosticCategory.Message,"if_files_is_specified_otherwise_Asterisk_Asterisk_Slash_Asterisk_6908",'`[]` if `files` is specified, otherwise `["**/*"]`'),true_if_composite_false_otherwise:n(6909,e.DiagnosticCategory.Message,"true_if_composite_false_otherwise_6909","`true` if `composite`, `false` otherwise"),module_AMD_or_UMD_or_System_or_ES6_then_Classic_Otherwise_Node:n(69010,e.DiagnosticCategory.Message,"module_AMD_or_UMD_or_System_or_ES6_then_Classic_Otherwise_Node_69010","module === `AMD` or `UMD` or `System` or `ES6`, then `Classic`, Otherwise `Node`"),Computed_from_the_list_of_input_files:n(6911,e.DiagnosticCategory.Message,"Computed_from_the_list_of_input_files_6911","Computed from the list of input files"),Platform_specific:n(6912,e.DiagnosticCategory.Message,"Platform_specific_6912","Platform specific"),You_can_learn_about_all_of_the_compiler_options_at_0:n(6913,e.DiagnosticCategory.Message,"You_can_learn_about_all_of_the_compiler_options_at_0_6913","You can learn about all of the compiler options at {0}"),Including_watch_w_will_start_watching_the_current_project_for_the_file_changes_Once_set_you_can_config_watch_mode_with_Colon:n(6914,e.DiagnosticCategory.Message,"Including_watch_w_will_start_watching_the_current_project_for_the_file_changes_Once_set_you_can_conf_6914","Including --watch, -w will start watching the current project for the file changes. Once set, you can config watch mode with:"),Using_build_b_will_make_tsc_behave_more_like_a_build_orchestrator_than_a_compiler_This_is_used_to_trigger_building_composite_projects_which_you_can_learn_more_about_at_0:n(6915,e.DiagnosticCategory.Message,"Using_build_b_will_make_tsc_behave_more_like_a_build_orchestrator_than_a_compiler_This_is_used_to_tr_6915","Using --build, -b will make tsc behave more like a build orchestrator than a compiler. This is used to trigger building composite projects which you can learn more about at {0}"),COMMON_COMMANDS:n(6916,e.DiagnosticCategory.Message,"COMMON_COMMANDS_6916","COMMON COMMANDS"),ALL_COMPILER_OPTIONS:n(6917,e.DiagnosticCategory.Message,"ALL_COMPILER_OPTIONS_6917","ALL COMPILER OPTIONS"),WATCH_OPTIONS:n(6918,e.DiagnosticCategory.Message,"WATCH_OPTIONS_6918","WATCH OPTIONS"),BUILD_OPTIONS:n(6919,e.DiagnosticCategory.Message,"BUILD_OPTIONS_6919","BUILD OPTIONS"),COMMON_COMPILER_OPTIONS:n(6920,e.DiagnosticCategory.Message,"COMMON_COMPILER_OPTIONS_6920","COMMON COMPILER OPTIONS"),COMMAND_LINE_FLAGS:n(6921,e.DiagnosticCategory.Message,"COMMAND_LINE_FLAGS_6921","COMMAND LINE FLAGS"),tsc_Colon_The_TypeScript_Compiler:n(6922,e.DiagnosticCategory.Message,"tsc_Colon_The_TypeScript_Compiler_6922","tsc: The TypeScript Compiler"),Compiles_the_current_project_tsconfig_json_in_the_working_directory:n(6923,e.DiagnosticCategory.Message,"Compiles_the_current_project_tsconfig_json_in_the_working_directory_6923","Compiles the current project (tsconfig.json in the working directory.)"),Ignoring_tsconfig_json_compiles_the_specified_files_with_default_compiler_options:n(6924,e.DiagnosticCategory.Message,"Ignoring_tsconfig_json_compiles_the_specified_files_with_default_compiler_options_6924","Ignoring tsconfig.json, compiles the specified files with default compiler options."),Build_a_composite_project_in_the_working_directory:n(6925,e.DiagnosticCategory.Message,"Build_a_composite_project_in_the_working_directory_6925","Build a composite project in the working directory."),Creates_a_tsconfig_json_with_the_recommended_settings_in_the_working_directory:n(6926,e.DiagnosticCategory.Message,"Creates_a_tsconfig_json_with_the_recommended_settings_in_the_working_directory_6926","Creates a tsconfig.json with the recommended settings in the working directory."),Compiles_the_TypeScript_project_located_at_the_specified_path:n(6927,e.DiagnosticCategory.Message,"Compiles_the_TypeScript_project_located_at_the_specified_path_6927","Compiles the TypeScript project located at the specified path."),An_expanded_version_of_this_information_showing_all_possible_compiler_options:n(6928,e.DiagnosticCategory.Message,"An_expanded_version_of_this_information_showing_all_possible_compiler_options_6928","An expanded version of this information, showing all possible compiler options"),Compiles_the_current_project_with_additional_settings:n(6929,e.DiagnosticCategory.Message,"Compiles_the_current_project_with_additional_settings_6929","Compiles the current project, with additional settings."),Variable_0_implicitly_has_an_1_type:n(7005,e.DiagnosticCategory.Error,"Variable_0_implicitly_has_an_1_type_7005","Variable '{0}' implicitly has an '{1}' type."),Parameter_0_implicitly_has_an_1_type:n(7006,e.DiagnosticCategory.Error,"Parameter_0_implicitly_has_an_1_type_7006","Parameter '{0}' implicitly has an '{1}' type."),Member_0_implicitly_has_an_1_type:n(7008,e.DiagnosticCategory.Error,"Member_0_implicitly_has_an_1_type_7008","Member '{0}' implicitly has an '{1}' type."),new_expression_whose_target_lacks_a_construct_signature_implicitly_has_an_any_type:n(7009,e.DiagnosticCategory.Error,"new_expression_whose_target_lacks_a_construct_signature_implicitly_has_an_any_type_7009","'new' expression, whose target lacks a construct signature, implicitly has an 'any' type."),_0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type:n(7010,e.DiagnosticCategory.Error,"_0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type_7010","'{0}', which lacks return-type annotation, implicitly has an '{1}' return type."),Function_expression_which_lacks_return_type_annotation_implicitly_has_an_0_return_type:n(7011,e.DiagnosticCategory.Error,"Function_expression_which_lacks_return_type_annotation_implicitly_has_an_0_return_type_7011","Function expression, which lacks return-type annotation, implicitly has an '{0}' return type."),Construct_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type:n(7013,e.DiagnosticCategory.Error,"Construct_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type_7013","Construct signature, which lacks return-type annotation, implicitly has an 'any' return type."),Function_type_which_lacks_return_type_annotation_implicitly_has_an_0_return_type:n(7014,e.DiagnosticCategory.Error,"Function_type_which_lacks_return_type_annotation_implicitly_has_an_0_return_type_7014","Function type, which lacks return-type annotation, implicitly has an '{0}' return type."),Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number:n(7015,e.DiagnosticCategory.Error,"Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number_7015","Element implicitly has an 'any' type because index expression is not of type 'number'."),Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type:n(7016,e.DiagnosticCategory.Error,"Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type_7016","Could not find a declaration file for module '{0}'. '{1}' implicitly has an 'any' type."),Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature:n(7017,e.DiagnosticCategory.Error,"Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature_7017","Element implicitly has an 'any' type because type '{0}' has no index signature."),Object_literal_s_property_0_implicitly_has_an_1_type:n(7018,e.DiagnosticCategory.Error,"Object_literal_s_property_0_implicitly_has_an_1_type_7018","Object literal's property '{0}' implicitly has an '{1}' type."),Rest_parameter_0_implicitly_has_an_any_type:n(7019,e.DiagnosticCategory.Error,"Rest_parameter_0_implicitly_has_an_any_type_7019","Rest parameter '{0}' implicitly has an 'any[]' type."),Call_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type:n(7020,e.DiagnosticCategory.Error,"Call_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type_7020","Call signature, which lacks return-type annotation, implicitly has an 'any' return type."),_0_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_and_is_referenced_directly_or_indirectly_in_its_own_initializer:n(7022,e.DiagnosticCategory.Error,"_0_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_and_is_referenced_directly_or__7022","'{0}' implicitly has type 'any' because it does not have a type annotation and is referenced directly or indirectly in its own initializer."),_0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions:n(7023,e.DiagnosticCategory.Error,"_0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_reference_7023","'{0}' implicitly has return type 'any' because it does not have a return type annotation and is referenced directly or indirectly in one of its return expressions."),Function_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions:n(7024,e.DiagnosticCategory.Error,"Function_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_ref_7024","Function implicitly has return type 'any' because it does not have a return type annotation and is referenced directly or indirectly in one of its return expressions."),Generator_implicitly_has_yield_type_0_because_it_does_not_yield_any_values_Consider_supplying_a_return_type_annotation:n(7025,e.DiagnosticCategory.Error,"Generator_implicitly_has_yield_type_0_because_it_does_not_yield_any_values_Consider_supplying_a_retu_7025","Generator implicitly has yield type '{0}' because it does not yield any values. Consider supplying a return type annotation."),JSX_element_implicitly_has_type_any_because_no_interface_JSX_0_exists:n(7026,e.DiagnosticCategory.Error,"JSX_element_implicitly_has_type_any_because_no_interface_JSX_0_exists_7026","JSX element implicitly has type 'any' because no interface 'JSX.{0}' exists."),Unreachable_code_detected:n(7027,e.DiagnosticCategory.Error,"Unreachable_code_detected_7027","Unreachable code detected.",!0),Unused_label:n(7028,e.DiagnosticCategory.Error,"Unused_label_7028","Unused label.",!0),Fallthrough_case_in_switch:n(7029,e.DiagnosticCategory.Error,"Fallthrough_case_in_switch_7029","Fallthrough case in switch."),Not_all_code_paths_return_a_value:n(7030,e.DiagnosticCategory.Error,"Not_all_code_paths_return_a_value_7030","Not all code paths return a value."),Binding_element_0_implicitly_has_an_1_type:n(7031,e.DiagnosticCategory.Error,"Binding_element_0_implicitly_has_an_1_type_7031","Binding element '{0}' implicitly has an '{1}' type."),Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation:n(7032,e.DiagnosticCategory.Error,"Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation_7032","Property '{0}' implicitly has type 'any', because its set accessor lacks a parameter type annotation."),Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation:n(7033,e.DiagnosticCategory.Error,"Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation_7033","Property '{0}' implicitly has type 'any', because its get accessor lacks a return type annotation."),Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined:n(7034,e.DiagnosticCategory.Error,"Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined_7034","Variable '{0}' implicitly has type '{1}' in some locations where its type cannot be determined."),Try_npm_i_save_dev_types_Slash_1_if_it_exists_or_add_a_new_declaration_d_ts_file_containing_declare_module_0:n(7035,e.DiagnosticCategory.Error,"Try_npm_i_save_dev_types_Slash_1_if_it_exists_or_add_a_new_declaration_d_ts_file_containing_declare__7035","Try `npm i --save-dev @types/{1}` if it exists or add a new declaration (.d.ts) file containing `declare module '{0}';`"),Dynamic_import_s_specifier_must_be_of_type_string_but_here_has_type_0:n(7036,e.DiagnosticCategory.Error,"Dynamic_import_s_specifier_must_be_of_type_string_but_here_has_type_0_7036","Dynamic import's specifier must be of type 'string', but here has type '{0}'."),Enables_emit_interoperability_between_CommonJS_and_ES_Modules_via_creation_of_namespace_objects_for_all_imports_Implies_allowSyntheticDefaultImports:n(7037,e.DiagnosticCategory.Message,"Enables_emit_interoperability_between_CommonJS_and_ES_Modules_via_creation_of_namespace_objects_for__7037","Enables emit interoperability between CommonJS and ES Modules via creation of namespace objects for all imports. Implies 'allowSyntheticDefaultImports'."),Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cause_a_failure_at_runtime_Consider_using_a_default_import_or_import_require_here_instead:n(7038,e.DiagnosticCategory.Message,"Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cau_7038","Type originates at this import. A namespace-style import cannot be called or constructed, and will cause a failure at runtime. Consider using a default import or import require here instead."),Mapped_object_type_implicitly_has_an_any_template_type:n(7039,e.DiagnosticCategory.Error,"Mapped_object_type_implicitly_has_an_any_template_type_7039","Mapped object type implicitly has an 'any' template type."),If_the_0_package_actually_exposes_this_module_consider_sending_a_pull_request_to_amend_https_Colon_Slash_Slashgithub_com_SlashDefinitelyTyped_SlashDefinitelyTyped_Slashtree_Slashmaster_Slashtypes_Slash_1:n(7040,e.DiagnosticCategory.Error,"If_the_0_package_actually_exposes_this_module_consider_sending_a_pull_request_to_amend_https_Colon_S_7040","If the '{0}' package actually exposes this module, consider sending a pull request to amend 'https://github.com/DefinitelyTyped/DefinitelyTyped/tree/master/types/{1}'"),The_containing_arrow_function_captures_the_global_value_of_this:n(7041,e.DiagnosticCategory.Error,"The_containing_arrow_function_captures_the_global_value_of_this_7041","The containing arrow function captures the global value of 'this'."),Module_0_was_resolved_to_1_but_resolveJsonModule_is_not_used:n(7042,e.DiagnosticCategory.Error,"Module_0_was_resolved_to_1_but_resolveJsonModule_is_not_used_7042","Module '{0}' was resolved to '{1}', but '--resolveJsonModule' is not used."),Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage:n(7043,e.DiagnosticCategory.Suggestion,"Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage_7043","Variable '{0}' implicitly has an '{1}' type, but a better type may be inferred from usage."),Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage:n(7044,e.DiagnosticCategory.Suggestion,"Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage_7044","Parameter '{0}' implicitly has an '{1}' type, but a better type may be inferred from usage."),Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage:n(7045,e.DiagnosticCategory.Suggestion,"Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage_7045","Member '{0}' implicitly has an '{1}' type, but a better type may be inferred from usage."),Variable_0_implicitly_has_type_1_in_some_locations_but_a_better_type_may_be_inferred_from_usage:n(7046,e.DiagnosticCategory.Suggestion,"Variable_0_implicitly_has_type_1_in_some_locations_but_a_better_type_may_be_inferred_from_usage_7046","Variable '{0}' implicitly has type '{1}' in some locations, but a better type may be inferred from usage."),Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage:n(7047,e.DiagnosticCategory.Suggestion,"Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage_7047","Rest parameter '{0}' implicitly has an 'any[]' type, but a better type may be inferred from usage."),Property_0_implicitly_has_type_any_but_a_better_type_for_its_get_accessor_may_be_inferred_from_usage:n(7048,e.DiagnosticCategory.Suggestion,"Property_0_implicitly_has_type_any_but_a_better_type_for_its_get_accessor_may_be_inferred_from_usage_7048","Property '{0}' implicitly has type 'any', but a better type for its get accessor may be inferred from usage."),Property_0_implicitly_has_type_any_but_a_better_type_for_its_set_accessor_may_be_inferred_from_usage:n(7049,e.DiagnosticCategory.Suggestion,"Property_0_implicitly_has_type_any_but_a_better_type_for_its_set_accessor_may_be_inferred_from_usage_7049","Property '{0}' implicitly has type 'any', but a better type for its set accessor may be inferred from usage."),_0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage:n(7050,e.DiagnosticCategory.Suggestion,"_0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage_7050","'{0}' implicitly has an '{1}' return type, but a better type may be inferred from usage."),Parameter_has_a_name_but_no_type_Did_you_mean_0_Colon_1:n(7051,e.DiagnosticCategory.Error,"Parameter_has_a_name_but_no_type_Did_you_mean_0_Colon_1_7051","Parameter has a name but no type. Did you mean '{0}: {1}'?"),Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature_Did_you_mean_to_call_1:n(7052,e.DiagnosticCategory.Error,"Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature_Did_you_mean_to_call_1_7052","Element implicitly has an 'any' type because type '{0}' has no index signature. Did you mean to call '{1}'?"),Element_implicitly_has_an_any_type_because_expression_of_type_0_can_t_be_used_to_index_type_1:n(7053,e.DiagnosticCategory.Error,"Element_implicitly_has_an_any_type_because_expression_of_type_0_can_t_be_used_to_index_type_1_7053","Element implicitly has an 'any' type because expression of type '{0}' can't be used to index type '{1}'."),No_index_signature_with_a_parameter_of_type_0_was_found_on_type_1:n(7054,e.DiagnosticCategory.Error,"No_index_signature_with_a_parameter_of_type_0_was_found_on_type_1_7054","No index signature with a parameter of type '{0}' was found on type '{1}'."),_0_which_lacks_return_type_annotation_implicitly_has_an_1_yield_type:n(7055,e.DiagnosticCategory.Error,"_0_which_lacks_return_type_annotation_implicitly_has_an_1_yield_type_7055","'{0}', which lacks return-type annotation, implicitly has an '{1}' yield type."),The_inferred_type_of_this_node_exceeds_the_maximum_length_the_compiler_will_serialize_An_explicit_type_annotation_is_needed:n(7056,e.DiagnosticCategory.Error,"The_inferred_type_of_this_node_exceeds_the_maximum_length_the_compiler_will_serialize_An_explicit_ty_7056","The inferred type of this node exceeds the maximum length the compiler will serialize. An explicit type annotation is needed."),yield_expression_implicitly_results_in_an_any_type_because_its_containing_generator_lacks_a_return_type_annotation:n(7057,e.DiagnosticCategory.Error,"yield_expression_implicitly_results_in_an_any_type_because_its_containing_generator_lacks_a_return_t_7057","'yield' expression implicitly results in an 'any' type because its containing generator lacks a return-type annotation."),You_cannot_rename_this_element:n(8e3,e.DiagnosticCategory.Error,"You_cannot_rename_this_element_8000","You cannot rename this element."),You_cannot_rename_elements_that_are_defined_in_the_standard_TypeScript_library:n(8001,e.DiagnosticCategory.Error,"You_cannot_rename_elements_that_are_defined_in_the_standard_TypeScript_library_8001","You cannot rename elements that are defined in the standard TypeScript library."),import_can_only_be_used_in_TypeScript_files:n(8002,e.DiagnosticCategory.Error,"import_can_only_be_used_in_TypeScript_files_8002","'import ... =' can only be used in TypeScript files."),export_can_only_be_used_in_TypeScript_files:n(8003,e.DiagnosticCategory.Error,"export_can_only_be_used_in_TypeScript_files_8003","'export =' can only be used in TypeScript files."),Type_parameter_declarations_can_only_be_used_in_TypeScript_files:n(8004,e.DiagnosticCategory.Error,"Type_parameter_declarations_can_only_be_used_in_TypeScript_files_8004","Type parameter declarations can only be used in TypeScript files."),implements_clauses_can_only_be_used_in_TypeScript_files:n(8005,e.DiagnosticCategory.Error,"implements_clauses_can_only_be_used_in_TypeScript_files_8005","'implements' clauses can only be used in TypeScript files."),_0_declarations_can_only_be_used_in_TypeScript_files:n(8006,e.DiagnosticCategory.Error,"_0_declarations_can_only_be_used_in_TypeScript_files_8006","'{0}' declarations can only be used in TypeScript files."),Type_aliases_can_only_be_used_in_TypeScript_files:n(8008,e.DiagnosticCategory.Error,"Type_aliases_can_only_be_used_in_TypeScript_files_8008","Type aliases can only be used in TypeScript files."),The_0_modifier_can_only_be_used_in_TypeScript_files:n(8009,e.DiagnosticCategory.Error,"The_0_modifier_can_only_be_used_in_TypeScript_files_8009","The '{0}' modifier can only be used in TypeScript files."),Type_annotations_can_only_be_used_in_TypeScript_files:n(8010,e.DiagnosticCategory.Error,"Type_annotations_can_only_be_used_in_TypeScript_files_8010","Type annotations can only be used in TypeScript files."),Type_arguments_can_only_be_used_in_TypeScript_files:n(8011,e.DiagnosticCategory.Error,"Type_arguments_can_only_be_used_in_TypeScript_files_8011","Type arguments can only be used in TypeScript files."),Parameter_modifiers_can_only_be_used_in_TypeScript_files:n(8012,e.DiagnosticCategory.Error,"Parameter_modifiers_can_only_be_used_in_TypeScript_files_8012","Parameter modifiers can only be used in TypeScript files."),Non_null_assertions_can_only_be_used_in_TypeScript_files:n(8013,e.DiagnosticCategory.Error,"Non_null_assertions_can_only_be_used_in_TypeScript_files_8013","Non-null assertions can only be used in TypeScript files."),Type_assertion_expressions_can_only_be_used_in_TypeScript_files:n(8016,e.DiagnosticCategory.Error,"Type_assertion_expressions_can_only_be_used_in_TypeScript_files_8016","Type assertion expressions can only be used in TypeScript files."),Octal_literal_types_must_use_ES2015_syntax_Use_the_syntax_0:n(8017,e.DiagnosticCategory.Error,"Octal_literal_types_must_use_ES2015_syntax_Use_the_syntax_0_8017","Octal literal types must use ES2015 syntax. Use the syntax '{0}'."),Octal_literals_are_not_allowed_in_enums_members_initializer_Use_the_syntax_0:n(8018,e.DiagnosticCategory.Error,"Octal_literals_are_not_allowed_in_enums_members_initializer_Use_the_syntax_0_8018","Octal literals are not allowed in enums members initializer. Use the syntax '{0}'."),Report_errors_in_js_files:n(8019,e.DiagnosticCategory.Message,"Report_errors_in_js_files_8019","Report errors in .js files."),JSDoc_types_can_only_be_used_inside_documentation_comments:n(8020,e.DiagnosticCategory.Error,"JSDoc_types_can_only_be_used_inside_documentation_comments_8020","JSDoc types can only be used inside documentation comments."),JSDoc_typedef_tag_should_either_have_a_type_annotation_or_be_followed_by_property_or_member_tags:n(8021,e.DiagnosticCategory.Error,"JSDoc_typedef_tag_should_either_have_a_type_annotation_or_be_followed_by_property_or_member_tags_8021","JSDoc '@typedef' tag should either have a type annotation or be followed by '@property' or '@member' tags."),JSDoc_0_is_not_attached_to_a_class:n(8022,e.DiagnosticCategory.Error,"JSDoc_0_is_not_attached_to_a_class_8022","JSDoc '@{0}' is not attached to a class."),JSDoc_0_1_does_not_match_the_extends_2_clause:n(8023,e.DiagnosticCategory.Error,"JSDoc_0_1_does_not_match_the_extends_2_clause_8023","JSDoc '@{0} {1}' does not match the 'extends {2}' clause."),JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name:n(8024,e.DiagnosticCategory.Error,"JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_8024","JSDoc '@param' tag has name '{0}', but there is no parameter with that name."),Class_declarations_cannot_have_more_than_one_augments_or_extends_tag:n(8025,e.DiagnosticCategory.Error,"Class_declarations_cannot_have_more_than_one_augments_or_extends_tag_8025","Class declarations cannot have more than one '@augments' or '@extends' tag."),Expected_0_type_arguments_provide_these_with_an_extends_tag:n(8026,e.DiagnosticCategory.Error,"Expected_0_type_arguments_provide_these_with_an_extends_tag_8026","Expected {0} type arguments; provide these with an '@extends' tag."),Expected_0_1_type_arguments_provide_these_with_an_extends_tag:n(8027,e.DiagnosticCategory.Error,"Expected_0_1_type_arguments_provide_these_with_an_extends_tag_8027","Expected {0}-{1} type arguments; provide these with an '@extends' tag."),JSDoc_may_only_appear_in_the_last_parameter_of_a_signature:n(8028,e.DiagnosticCategory.Error,"JSDoc_may_only_appear_in_the_last_parameter_of_a_signature_8028","JSDoc '...' may only appear in the last parameter of a signature."),JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_It_would_match_arguments_if_it_had_an_array_type:n(8029,e.DiagnosticCategory.Error,"JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_It_would_match_arguments_if_it_h_8029","JSDoc '@param' tag has name '{0}', but there is no parameter with that name. It would match 'arguments' if it had an array type."),The_type_of_a_function_declaration_must_match_the_function_s_signature:n(8030,e.DiagnosticCategory.Error,"The_type_of_a_function_declaration_must_match_the_function_s_signature_8030","The type of a function declaration must match the function's signature."),You_cannot_rename_a_module_via_a_global_import:n(8031,e.DiagnosticCategory.Error,"You_cannot_rename_a_module_via_a_global_import_8031","You cannot rename a module via a global import."),Qualified_name_0_is_not_allowed_without_a_leading_param_object_1:n(8032,e.DiagnosticCategory.Error,"Qualified_name_0_is_not_allowed_without_a_leading_param_object_1_8032","Qualified name '{0}' is not allowed without a leading '@param {object} {1}'."),A_JSDoc_typedef_comment_may_not_contain_multiple_type_tags:n(8033,e.DiagnosticCategory.Error,"A_JSDoc_typedef_comment_may_not_contain_multiple_type_tags_8033","A JSDoc '@typedef' comment may not contain multiple '@type' tags."),The_tag_was_first_specified_here:n(8034,e.DiagnosticCategory.Error,"The_tag_was_first_specified_here_8034","The tag was first specified here."),Only_identifiers_Slashqualified_names_with_optional_type_arguments_are_currently_supported_in_a_class_extends_clause:n(9002,e.DiagnosticCategory.Error,"Only_identifiers_Slashqualified_names_with_optional_type_arguments_are_currently_supported_in_a_clas_9002","Only identifiers/qualified-names with optional type arguments are currently supported in a class 'extends' clause."),class_expressions_are_not_currently_supported:n(9003,e.DiagnosticCategory.Error,"class_expressions_are_not_currently_supported_9003","'class' expressions are not currently supported."),Language_service_is_disabled:n(9004,e.DiagnosticCategory.Error,"Language_service_is_disabled_9004","Language service is disabled."),Declaration_emit_for_this_file_requires_using_private_name_0_An_explicit_type_annotation_may_unblock_declaration_emit:n(9005,e.DiagnosticCategory.Error,"Declaration_emit_for_this_file_requires_using_private_name_0_An_explicit_type_annotation_may_unblock_9005","Declaration emit for this file requires using private name '{0}'. An explicit type annotation may unblock declaration emit."),Declaration_emit_for_this_file_requires_using_private_name_0_from_module_1_An_explicit_type_annotation_may_unblock_declaration_emit:n(9006,e.DiagnosticCategory.Error,"Declaration_emit_for_this_file_requires_using_private_name_0_from_module_1_An_explicit_type_annotati_9006","Declaration emit for this file requires using private name '{0}' from module '{1}'. An explicit type annotation may unblock declaration emit."),JSX_attributes_must_only_be_assigned_a_non_empty_expression:n(17e3,e.DiagnosticCategory.Error,"JSX_attributes_must_only_be_assigned_a_non_empty_expression_17000","JSX attributes must only be assigned a non-empty 'expression'."),JSX_elements_cannot_have_multiple_attributes_with_the_same_name:n(17001,e.DiagnosticCategory.Error,"JSX_elements_cannot_have_multiple_attributes_with_the_same_name_17001","JSX elements cannot have multiple attributes with the same name."),Expected_corresponding_JSX_closing_tag_for_0:n(17002,e.DiagnosticCategory.Error,"Expected_corresponding_JSX_closing_tag_for_0_17002","Expected corresponding JSX closing tag for '{0}'."),JSX_attribute_expected:n(17003,e.DiagnosticCategory.Error,"JSX_attribute_expected_17003","JSX attribute expected."),Cannot_use_JSX_unless_the_jsx_flag_is_provided:n(17004,e.DiagnosticCategory.Error,"Cannot_use_JSX_unless_the_jsx_flag_is_provided_17004","Cannot use JSX unless the '--jsx' flag is provided."),A_constructor_cannot_contain_a_super_call_when_its_class_extends_null:n(17005,e.DiagnosticCategory.Error,"A_constructor_cannot_contain_a_super_call_when_its_class_extends_null_17005","A constructor cannot contain a 'super' call when its class extends 'null'."),An_unary_expression_with_the_0_operator_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses:n(17006,e.DiagnosticCategory.Error,"An_unary_expression_with_the_0_operator_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_ex_17006","An unary expression with the '{0}' operator is not allowed in the left-hand side of an exponentiation expression. Consider enclosing the expression in parentheses."),A_type_assertion_expression_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses:n(17007,e.DiagnosticCategory.Error,"A_type_assertion_expression_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Con_17007","A type assertion expression is not allowed in the left-hand side of an exponentiation expression. Consider enclosing the expression in parentheses."),JSX_element_0_has_no_corresponding_closing_tag:n(17008,e.DiagnosticCategory.Error,"JSX_element_0_has_no_corresponding_closing_tag_17008","JSX element '{0}' has no corresponding closing tag."),super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class:n(17009,e.DiagnosticCategory.Error,"super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class_17009","'super' must be called before accessing 'this' in the constructor of a derived class."),Unknown_type_acquisition_option_0:n(17010,e.DiagnosticCategory.Error,"Unknown_type_acquisition_option_0_17010","Unknown type acquisition option '{0}'."),super_must_be_called_before_accessing_a_property_of_super_in_the_constructor_of_a_derived_class:n(17011,e.DiagnosticCategory.Error,"super_must_be_called_before_accessing_a_property_of_super_in_the_constructor_of_a_derived_class_17011","'super' must be called before accessing a property of 'super' in the constructor of a derived class."),_0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2:n(17012,e.DiagnosticCategory.Error,"_0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2_17012","'{0}' is not a valid meta-property for keyword '{1}'. Did you mean '{2}'?"),Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constructor:n(17013,e.DiagnosticCategory.Error,"Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constru_17013","Meta-property '{0}' is only allowed in the body of a function declaration, function expression, or constructor."),JSX_fragment_has_no_corresponding_closing_tag:n(17014,e.DiagnosticCategory.Error,"JSX_fragment_has_no_corresponding_closing_tag_17014","JSX fragment has no corresponding closing tag."),Expected_corresponding_closing_tag_for_JSX_fragment:n(17015,e.DiagnosticCategory.Error,"Expected_corresponding_closing_tag_for_JSX_fragment_17015","Expected corresponding closing tag for JSX fragment."),The_jsxFragmentFactory_compiler_option_must_be_provided_to_use_JSX_fragments_with_the_jsxFactory_compiler_option:n(17016,e.DiagnosticCategory.Error,"The_jsxFragmentFactory_compiler_option_must_be_provided_to_use_JSX_fragments_with_the_jsxFactory_com_17016","The 'jsxFragmentFactory' compiler option must be provided to use JSX fragments with the 'jsxFactory' compiler option."),An_jsxFrag_pragma_is_required_when_using_an_jsx_pragma_with_JSX_fragments:n(17017,e.DiagnosticCategory.Error,"An_jsxFrag_pragma_is_required_when_using_an_jsx_pragma_with_JSX_fragments_17017","An @jsxFrag pragma is required when using an @jsx pragma with JSX fragments."),Unknown_type_acquisition_option_0_Did_you_mean_1:n(17018,e.DiagnosticCategory.Error,"Unknown_type_acquisition_option_0_Did_you_mean_1_17018","Unknown type acquisition option '{0}'. Did you mean '{1}'?"),Circularity_detected_while_resolving_configuration_Colon_0:n(18e3,e.DiagnosticCategory.Error,"Circularity_detected_while_resolving_configuration_Colon_0_18000","Circularity detected while resolving configuration: {0}"),A_path_in_an_extends_option_must_be_relative_or_rooted_but_0_is_not:n(18001,e.DiagnosticCategory.Error,"A_path_in_an_extends_option_must_be_relative_or_rooted_but_0_is_not_18001","A path in an 'extends' option must be relative or rooted, but '{0}' is not."),The_files_list_in_config_file_0_is_empty:n(18002,e.DiagnosticCategory.Error,"The_files_list_in_config_file_0_is_empty_18002","The 'files' list in config file '{0}' is empty."),No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2:n(18003,e.DiagnosticCategory.Error,"No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2_18003","No inputs were found in config file '{0}'. Specified 'include' paths were '{1}' and 'exclude' paths were '{2}'."),File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module:n(80001,e.DiagnosticCategory.Suggestion,"File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module_80001","File is a CommonJS module; it may be converted to an ES6 module."),This_constructor_function_may_be_converted_to_a_class_declaration:n(80002,e.DiagnosticCategory.Suggestion,"This_constructor_function_may_be_converted_to_a_class_declaration_80002","This constructor function may be converted to a class declaration."),Import_may_be_converted_to_a_default_import:n(80003,e.DiagnosticCategory.Suggestion,"Import_may_be_converted_to_a_default_import_80003","Import may be converted to a default import."),JSDoc_types_may_be_moved_to_TypeScript_types:n(80004,e.DiagnosticCategory.Suggestion,"JSDoc_types_may_be_moved_to_TypeScript_types_80004","JSDoc types may be moved to TypeScript types."),require_call_may_be_converted_to_an_import:n(80005,e.DiagnosticCategory.Suggestion,"require_call_may_be_converted_to_an_import_80005","'require' call may be converted to an import."),This_may_be_converted_to_an_async_function:n(80006,e.DiagnosticCategory.Suggestion,"This_may_be_converted_to_an_async_function_80006","This may be converted to an async function."),await_has_no_effect_on_the_type_of_this_expression:n(80007,e.DiagnosticCategory.Suggestion,"await_has_no_effect_on_the_type_of_this_expression_80007","'await' has no effect on the type of this expression."),Numeric_literals_with_absolute_values_equal_to_2_53_or_greater_are_too_large_to_be_represented_accurately_as_integers:n(80008,e.DiagnosticCategory.Suggestion,"Numeric_literals_with_absolute_values_equal_to_2_53_or_greater_are_too_large_to_be_represented_accur_80008","Numeric literals with absolute values equal to 2^53 or greater are too large to be represented accurately as integers."),Add_missing_super_call:n(90001,e.DiagnosticCategory.Message,"Add_missing_super_call_90001","Add missing 'super()' call"),Make_super_call_the_first_statement_in_the_constructor:n(90002,e.DiagnosticCategory.Message,"Make_super_call_the_first_statement_in_the_constructor_90002","Make 'super()' call the first statement in the constructor"),Change_extends_to_implements:n(90003,e.DiagnosticCategory.Message,"Change_extends_to_implements_90003","Change 'extends' to 'implements'"),Remove_unused_declaration_for_Colon_0:n(90004,e.DiagnosticCategory.Message,"Remove_unused_declaration_for_Colon_0_90004","Remove unused declaration for: '{0}'"),Remove_import_from_0:n(90005,e.DiagnosticCategory.Message,"Remove_import_from_0_90005","Remove import from '{0}'"),Implement_interface_0:n(90006,e.DiagnosticCategory.Message,"Implement_interface_0_90006","Implement interface '{0}'"),Implement_inherited_abstract_class:n(90007,e.DiagnosticCategory.Message,"Implement_inherited_abstract_class_90007","Implement inherited abstract class"),Add_0_to_unresolved_variable:n(90008,e.DiagnosticCategory.Message,"Add_0_to_unresolved_variable_90008","Add '{0}.' to unresolved variable"),Remove_variable_statement:n(90010,e.DiagnosticCategory.Message,"Remove_variable_statement_90010","Remove variable statement"),Remove_template_tag:n(90011,e.DiagnosticCategory.Message,"Remove_template_tag_90011","Remove template tag"),Remove_type_parameters:n(90012,e.DiagnosticCategory.Message,"Remove_type_parameters_90012","Remove type parameters"),Import_0_from_module_1:n(90013,e.DiagnosticCategory.Message,"Import_0_from_module_1_90013","Import '{0}' from module \"{1}\""),Change_0_to_1:n(90014,e.DiagnosticCategory.Message,"Change_0_to_1_90014","Change '{0}' to '{1}'"),Add_0_to_existing_import_declaration_from_1:n(90015,e.DiagnosticCategory.Message,"Add_0_to_existing_import_declaration_from_1_90015","Add '{0}' to existing import declaration from \"{1}\""),Declare_property_0:n(90016,e.DiagnosticCategory.Message,"Declare_property_0_90016","Declare property '{0}'"),Add_index_signature_for_property_0:n(90017,e.DiagnosticCategory.Message,"Add_index_signature_for_property_0_90017","Add index signature for property '{0}'"),Disable_checking_for_this_file:n(90018,e.DiagnosticCategory.Message,"Disable_checking_for_this_file_90018","Disable checking for this file"),Ignore_this_error_message:n(90019,e.DiagnosticCategory.Message,"Ignore_this_error_message_90019","Ignore this error message"),Initialize_property_0_in_the_constructor:n(90020,e.DiagnosticCategory.Message,"Initialize_property_0_in_the_constructor_90020","Initialize property '{0}' in the constructor"),Initialize_static_property_0:n(90021,e.DiagnosticCategory.Message,"Initialize_static_property_0_90021","Initialize static property '{0}'"),Change_spelling_to_0:n(90022,e.DiagnosticCategory.Message,"Change_spelling_to_0_90022","Change spelling to '{0}'"),Declare_method_0:n(90023,e.DiagnosticCategory.Message,"Declare_method_0_90023","Declare method '{0}'"),Declare_static_method_0:n(90024,e.DiagnosticCategory.Message,"Declare_static_method_0_90024","Declare static method '{0}'"),Prefix_0_with_an_underscore:n(90025,e.DiagnosticCategory.Message,"Prefix_0_with_an_underscore_90025","Prefix '{0}' with an underscore"),Rewrite_as_the_indexed_access_type_0:n(90026,e.DiagnosticCategory.Message,"Rewrite_as_the_indexed_access_type_0_90026","Rewrite as the indexed access type '{0}'"),Declare_static_property_0:n(90027,e.DiagnosticCategory.Message,"Declare_static_property_0_90027","Declare static property '{0}'"),Call_decorator_expression:n(90028,e.DiagnosticCategory.Message,"Call_decorator_expression_90028","Call decorator expression"),Add_async_modifier_to_containing_function:n(90029,e.DiagnosticCategory.Message,"Add_async_modifier_to_containing_function_90029","Add async modifier to containing function"),Replace_infer_0_with_unknown:n(90030,e.DiagnosticCategory.Message,"Replace_infer_0_with_unknown_90030","Replace 'infer {0}' with 'unknown'"),Replace_all_unused_infer_with_unknown:n(90031,e.DiagnosticCategory.Message,"Replace_all_unused_infer_with_unknown_90031","Replace all unused 'infer' with 'unknown'"),Import_default_0_from_module_1:n(90032,e.DiagnosticCategory.Message,"Import_default_0_from_module_1_90032","Import default '{0}' from module \"{1}\""),Add_default_import_0_to_existing_import_declaration_from_1:n(90033,e.DiagnosticCategory.Message,"Add_default_import_0_to_existing_import_declaration_from_1_90033","Add default import '{0}' to existing import declaration from \"{1}\""),Add_parameter_name:n(90034,e.DiagnosticCategory.Message,"Add_parameter_name_90034","Add parameter name"),Declare_private_property_0:n(90035,e.DiagnosticCategory.Message,"Declare_private_property_0_90035","Declare private property '{0}'"),Replace_0_with_Promise_1:n(90036,e.DiagnosticCategory.Message,"Replace_0_with_Promise_1_90036","Replace '{0}' with 'Promise<{1}>'"),Fix_all_incorrect_return_type_of_an_async_functions:n(90037,e.DiagnosticCategory.Message,"Fix_all_incorrect_return_type_of_an_async_functions_90037","Fix all incorrect return type of an async functions"),Declare_private_method_0:n(90038,e.DiagnosticCategory.Message,"Declare_private_method_0_90038","Declare private method '{0}'"),Remove_unused_destructuring_declaration:n(90039,e.DiagnosticCategory.Message,"Remove_unused_destructuring_declaration_90039","Remove unused destructuring declaration"),Remove_unused_declarations_for_Colon_0:n(90041,e.DiagnosticCategory.Message,"Remove_unused_declarations_for_Colon_0_90041","Remove unused declarations for: '{0}'"),Declare_a_private_field_named_0:n(90053,e.DiagnosticCategory.Message,"Declare_a_private_field_named_0_90053","Declare a private field named '{0}'."),Convert_function_to_an_ES2015_class:n(95001,e.DiagnosticCategory.Message,"Convert_function_to_an_ES2015_class_95001","Convert function to an ES2015 class"),Convert_function_0_to_class:n(95002,e.DiagnosticCategory.Message,"Convert_function_0_to_class_95002","Convert function '{0}' to class"),Convert_0_to_1_in_0:n(95003,e.DiagnosticCategory.Message,"Convert_0_to_1_in_0_95003","Convert '{0}' to '{1} in {0}'"),Extract_to_0_in_1:n(95004,e.DiagnosticCategory.Message,"Extract_to_0_in_1_95004","Extract to {0} in {1}"),Extract_function:n(95005,e.DiagnosticCategory.Message,"Extract_function_95005","Extract function"),Extract_constant:n(95006,e.DiagnosticCategory.Message,"Extract_constant_95006","Extract constant"),Extract_to_0_in_enclosing_scope:n(95007,e.DiagnosticCategory.Message,"Extract_to_0_in_enclosing_scope_95007","Extract to {0} in enclosing scope"),Extract_to_0_in_1_scope:n(95008,e.DiagnosticCategory.Message,"Extract_to_0_in_1_scope_95008","Extract to {0} in {1} scope"),Annotate_with_type_from_JSDoc:n(95009,e.DiagnosticCategory.Message,"Annotate_with_type_from_JSDoc_95009","Annotate with type from JSDoc"),Annotate_with_types_from_JSDoc:n(95010,e.DiagnosticCategory.Message,"Annotate_with_types_from_JSDoc_95010","Annotate with types from JSDoc"),Infer_type_of_0_from_usage:n(95011,e.DiagnosticCategory.Message,"Infer_type_of_0_from_usage_95011","Infer type of '{0}' from usage"),Infer_parameter_types_from_usage:n(95012,e.DiagnosticCategory.Message,"Infer_parameter_types_from_usage_95012","Infer parameter types from usage"),Convert_to_default_import:n(95013,e.DiagnosticCategory.Message,"Convert_to_default_import_95013","Convert to default import"),Install_0:n(95014,e.DiagnosticCategory.Message,"Install_0_95014","Install '{0}'"),Replace_import_with_0:n(95015,e.DiagnosticCategory.Message,"Replace_import_with_0_95015","Replace import with '{0}'."),Use_synthetic_default_member:n(95016,e.DiagnosticCategory.Message,"Use_synthetic_default_member_95016","Use synthetic 'default' member."),Convert_to_ES6_module:n(95017,e.DiagnosticCategory.Message,"Convert_to_ES6_module_95017","Convert to ES6 module"),Add_undefined_type_to_property_0:n(95018,e.DiagnosticCategory.Message,"Add_undefined_type_to_property_0_95018","Add 'undefined' type to property '{0}'"),Add_initializer_to_property_0:n(95019,e.DiagnosticCategory.Message,"Add_initializer_to_property_0_95019","Add initializer to property '{0}'"),Add_definite_assignment_assertion_to_property_0:n(95020,e.DiagnosticCategory.Message,"Add_definite_assignment_assertion_to_property_0_95020","Add definite assignment assertion to property '{0}'"),Convert_all_type_literals_to_mapped_type:n(95021,e.DiagnosticCategory.Message,"Convert_all_type_literals_to_mapped_type_95021","Convert all type literals to mapped type"),Add_all_missing_members:n(95022,e.DiagnosticCategory.Message,"Add_all_missing_members_95022","Add all missing members"),Infer_all_types_from_usage:n(95023,e.DiagnosticCategory.Message,"Infer_all_types_from_usage_95023","Infer all types from usage"),Delete_all_unused_declarations:n(95024,e.DiagnosticCategory.Message,"Delete_all_unused_declarations_95024","Delete all unused declarations"),Prefix_all_unused_declarations_with_where_possible:n(95025,e.DiagnosticCategory.Message,"Prefix_all_unused_declarations_with_where_possible_95025","Prefix all unused declarations with '_' where possible"),Fix_all_detected_spelling_errors:n(95026,e.DiagnosticCategory.Message,"Fix_all_detected_spelling_errors_95026","Fix all detected spelling errors"),Add_initializers_to_all_uninitialized_properties:n(95027,e.DiagnosticCategory.Message,"Add_initializers_to_all_uninitialized_properties_95027","Add initializers to all uninitialized properties"),Add_definite_assignment_assertions_to_all_uninitialized_properties:n(95028,e.DiagnosticCategory.Message,"Add_definite_assignment_assertions_to_all_uninitialized_properties_95028","Add definite assignment assertions to all uninitialized properties"),Add_undefined_type_to_all_uninitialized_properties:n(95029,e.DiagnosticCategory.Message,"Add_undefined_type_to_all_uninitialized_properties_95029","Add undefined type to all uninitialized properties"),Change_all_jsdoc_style_types_to_TypeScript:n(95030,e.DiagnosticCategory.Message,"Change_all_jsdoc_style_types_to_TypeScript_95030","Change all jsdoc-style types to TypeScript"),Change_all_jsdoc_style_types_to_TypeScript_and_add_undefined_to_nullable_types:n(95031,e.DiagnosticCategory.Message,"Change_all_jsdoc_style_types_to_TypeScript_and_add_undefined_to_nullable_types_95031","Change all jsdoc-style types to TypeScript (and add '| undefined' to nullable types)"),Implement_all_unimplemented_interfaces:n(95032,e.DiagnosticCategory.Message,"Implement_all_unimplemented_interfaces_95032","Implement all unimplemented interfaces"),Install_all_missing_types_packages:n(95033,e.DiagnosticCategory.Message,"Install_all_missing_types_packages_95033","Install all missing types packages"),Rewrite_all_as_indexed_access_types:n(95034,e.DiagnosticCategory.Message,"Rewrite_all_as_indexed_access_types_95034","Rewrite all as indexed access types"),Convert_all_to_default_imports:n(95035,e.DiagnosticCategory.Message,"Convert_all_to_default_imports_95035","Convert all to default imports"),Make_all_super_calls_the_first_statement_in_their_constructor:n(95036,e.DiagnosticCategory.Message,"Make_all_super_calls_the_first_statement_in_their_constructor_95036","Make all 'super()' calls the first statement in their constructor"),Add_qualifier_to_all_unresolved_variables_matching_a_member_name:n(95037,e.DiagnosticCategory.Message,"Add_qualifier_to_all_unresolved_variables_matching_a_member_name_95037","Add qualifier to all unresolved variables matching a member name"),Change_all_extended_interfaces_to_implements:n(95038,e.DiagnosticCategory.Message,"Change_all_extended_interfaces_to_implements_95038","Change all extended interfaces to 'implements'"),Add_all_missing_super_calls:n(95039,e.DiagnosticCategory.Message,"Add_all_missing_super_calls_95039","Add all missing super calls"),Implement_all_inherited_abstract_classes:n(95040,e.DiagnosticCategory.Message,"Implement_all_inherited_abstract_classes_95040","Implement all inherited abstract classes"),Add_all_missing_async_modifiers:n(95041,e.DiagnosticCategory.Message,"Add_all_missing_async_modifiers_95041","Add all missing 'async' modifiers"),Add_ts_ignore_to_all_error_messages:n(95042,e.DiagnosticCategory.Message,"Add_ts_ignore_to_all_error_messages_95042","Add '@ts-ignore' to all error messages"),Annotate_everything_with_types_from_JSDoc:n(95043,e.DiagnosticCategory.Message,"Annotate_everything_with_types_from_JSDoc_95043","Annotate everything with types from JSDoc"),Add_to_all_uncalled_decorators:n(95044,e.DiagnosticCategory.Message,"Add_to_all_uncalled_decorators_95044","Add '()' to all uncalled decorators"),Convert_all_constructor_functions_to_classes:n(95045,e.DiagnosticCategory.Message,"Convert_all_constructor_functions_to_classes_95045","Convert all constructor functions to classes"),Generate_get_and_set_accessors:n(95046,e.DiagnosticCategory.Message,"Generate_get_and_set_accessors_95046","Generate 'get' and 'set' accessors"),Convert_require_to_import:n(95047,e.DiagnosticCategory.Message,"Convert_require_to_import_95047","Convert 'require' to 'import'"),Convert_all_require_to_import:n(95048,e.DiagnosticCategory.Message,"Convert_all_require_to_import_95048","Convert all 'require' to 'import'"),Move_to_a_new_file:n(95049,e.DiagnosticCategory.Message,"Move_to_a_new_file_95049","Move to a new file"),Remove_unreachable_code:n(95050,e.DiagnosticCategory.Message,"Remove_unreachable_code_95050","Remove unreachable code"),Remove_all_unreachable_code:n(95051,e.DiagnosticCategory.Message,"Remove_all_unreachable_code_95051","Remove all unreachable code"),Add_missing_typeof:n(95052,e.DiagnosticCategory.Message,"Add_missing_typeof_95052","Add missing 'typeof'"),Remove_unused_label:n(95053,e.DiagnosticCategory.Message,"Remove_unused_label_95053","Remove unused label"),Remove_all_unused_labels:n(95054,e.DiagnosticCategory.Message,"Remove_all_unused_labels_95054","Remove all unused labels"),Convert_0_to_mapped_object_type:n(95055,e.DiagnosticCategory.Message,"Convert_0_to_mapped_object_type_95055","Convert '{0}' to mapped object type"),Convert_namespace_import_to_named_imports:n(95056,e.DiagnosticCategory.Message,"Convert_namespace_import_to_named_imports_95056","Convert namespace import to named imports"),Convert_named_imports_to_namespace_import:n(95057,e.DiagnosticCategory.Message,"Convert_named_imports_to_namespace_import_95057","Convert named imports to namespace import"),Add_or_remove_braces_in_an_arrow_function:n(95058,e.DiagnosticCategory.Message,"Add_or_remove_braces_in_an_arrow_function_95058","Add or remove braces in an arrow function"),Add_braces_to_arrow_function:n(95059,e.DiagnosticCategory.Message,"Add_braces_to_arrow_function_95059","Add braces to arrow function"),Remove_braces_from_arrow_function:n(95060,e.DiagnosticCategory.Message,"Remove_braces_from_arrow_function_95060","Remove braces from arrow function"),Convert_default_export_to_named_export:n(95061,e.DiagnosticCategory.Message,"Convert_default_export_to_named_export_95061","Convert default export to named export"),Convert_named_export_to_default_export:n(95062,e.DiagnosticCategory.Message,"Convert_named_export_to_default_export_95062","Convert named export to default export"),Add_missing_enum_member_0:n(95063,e.DiagnosticCategory.Message,"Add_missing_enum_member_0_95063","Add missing enum member '{0}'"),Add_all_missing_imports:n(95064,e.DiagnosticCategory.Message,"Add_all_missing_imports_95064","Add all missing imports"),Convert_to_async_function:n(95065,e.DiagnosticCategory.Message,"Convert_to_async_function_95065","Convert to async function"),Convert_all_to_async_functions:n(95066,e.DiagnosticCategory.Message,"Convert_all_to_async_functions_95066","Convert all to async functions"),Add_missing_call_parentheses:n(95067,e.DiagnosticCategory.Message,"Add_missing_call_parentheses_95067","Add missing call parentheses"),Add_all_missing_call_parentheses:n(95068,e.DiagnosticCategory.Message,"Add_all_missing_call_parentheses_95068","Add all missing call parentheses"),Add_unknown_conversion_for_non_overlapping_types:n(95069,e.DiagnosticCategory.Message,"Add_unknown_conversion_for_non_overlapping_types_95069","Add 'unknown' conversion for non-overlapping types"),Add_unknown_to_all_conversions_of_non_overlapping_types:n(95070,e.DiagnosticCategory.Message,"Add_unknown_to_all_conversions_of_non_overlapping_types_95070","Add 'unknown' to all conversions of non-overlapping types"),Add_missing_new_operator_to_call:n(95071,e.DiagnosticCategory.Message,"Add_missing_new_operator_to_call_95071","Add missing 'new' operator to call"),Add_missing_new_operator_to_all_calls:n(95072,e.DiagnosticCategory.Message,"Add_missing_new_operator_to_all_calls_95072","Add missing 'new' operator to all calls"),Add_names_to_all_parameters_without_names:n(95073,e.DiagnosticCategory.Message,"Add_names_to_all_parameters_without_names_95073","Add names to all parameters without names"),Enable_the_experimentalDecorators_option_in_your_configuration_file:n(95074,e.DiagnosticCategory.Message,"Enable_the_experimentalDecorators_option_in_your_configuration_file_95074","Enable the 'experimentalDecorators' option in your configuration file"),Convert_parameters_to_destructured_object:n(95075,e.DiagnosticCategory.Message,"Convert_parameters_to_destructured_object_95075","Convert parameters to destructured object"),Extract_type:n(95077,e.DiagnosticCategory.Message,"Extract_type_95077","Extract type"),Extract_to_type_alias:n(95078,e.DiagnosticCategory.Message,"Extract_to_type_alias_95078","Extract to type alias"),Extract_to_typedef:n(95079,e.DiagnosticCategory.Message,"Extract_to_typedef_95079","Extract to typedef"),Infer_this_type_of_0_from_usage:n(95080,e.DiagnosticCategory.Message,"Infer_this_type_of_0_from_usage_95080","Infer 'this' type of '{0}' from usage"),Add_const_to_unresolved_variable:n(95081,e.DiagnosticCategory.Message,"Add_const_to_unresolved_variable_95081","Add 'const' to unresolved variable"),Add_const_to_all_unresolved_variables:n(95082,e.DiagnosticCategory.Message,"Add_const_to_all_unresolved_variables_95082","Add 'const' to all unresolved variables"),Add_await:n(95083,e.DiagnosticCategory.Message,"Add_await_95083","Add 'await'"),Add_await_to_initializer_for_0:n(95084,e.DiagnosticCategory.Message,"Add_await_to_initializer_for_0_95084","Add 'await' to initializer for '{0}'"),Fix_all_expressions_possibly_missing_await:n(95085,e.DiagnosticCategory.Message,"Fix_all_expressions_possibly_missing_await_95085","Fix all expressions possibly missing 'await'"),Remove_unnecessary_await:n(95086,e.DiagnosticCategory.Message,"Remove_unnecessary_await_95086","Remove unnecessary 'await'"),Remove_all_unnecessary_uses_of_await:n(95087,e.DiagnosticCategory.Message,"Remove_all_unnecessary_uses_of_await_95087","Remove all unnecessary uses of 'await'"),Enable_the_jsx_flag_in_your_configuration_file:n(95088,e.DiagnosticCategory.Message,"Enable_the_jsx_flag_in_your_configuration_file_95088","Enable the '--jsx' flag in your configuration file"),Add_await_to_initializers:n(95089,e.DiagnosticCategory.Message,"Add_await_to_initializers_95089","Add 'await' to initializers"),Extract_to_interface:n(95090,e.DiagnosticCategory.Message,"Extract_to_interface_95090","Extract to interface"),Convert_to_a_bigint_numeric_literal:n(95091,e.DiagnosticCategory.Message,"Convert_to_a_bigint_numeric_literal_95091","Convert to a bigint numeric literal"),Convert_all_to_bigint_numeric_literals:n(95092,e.DiagnosticCategory.Message,"Convert_all_to_bigint_numeric_literals_95092","Convert all to bigint numeric literals"),Convert_const_to_let:n(95093,e.DiagnosticCategory.Message,"Convert_const_to_let_95093","Convert 'const' to 'let'"),Prefix_with_declare:n(95094,e.DiagnosticCategory.Message,"Prefix_with_declare_95094","Prefix with 'declare'"),Prefix_all_incorrect_property_declarations_with_declare:n(95095,e.DiagnosticCategory.Message,"Prefix_all_incorrect_property_declarations_with_declare_95095","Prefix all incorrect property declarations with 'declare'"),Convert_to_template_string:n(95096,e.DiagnosticCategory.Message,"Convert_to_template_string_95096","Convert to template string"),Add_export_to_make_this_file_into_a_module:n(95097,e.DiagnosticCategory.Message,"Add_export_to_make_this_file_into_a_module_95097","Add 'export {}' to make this file into a module"),Set_the_target_option_in_your_configuration_file_to_0:n(95098,e.DiagnosticCategory.Message,"Set_the_target_option_in_your_configuration_file_to_0_95098","Set the 'target' option in your configuration file to '{0}'"),Set_the_module_option_in_your_configuration_file_to_0:n(95099,e.DiagnosticCategory.Message,"Set_the_module_option_in_your_configuration_file_to_0_95099","Set the 'module' option in your configuration file to '{0}'"),Convert_invalid_character_to_its_html_entity_code:n(95100,e.DiagnosticCategory.Message,"Convert_invalid_character_to_its_html_entity_code_95100","Convert invalid character to its html entity code"),Convert_all_invalid_characters_to_HTML_entity_code:n(95101,e.DiagnosticCategory.Message,"Convert_all_invalid_characters_to_HTML_entity_code_95101","Convert all invalid characters to HTML entity code"),Add_class_tag:n(95102,e.DiagnosticCategory.Message,"Add_class_tag_95102","Add '@class' tag"),Add_this_tag:n(95103,e.DiagnosticCategory.Message,"Add_this_tag_95103","Add '@this' tag"),Add_this_parameter:n(95104,e.DiagnosticCategory.Message,"Add_this_parameter_95104","Add 'this' parameter."),Convert_function_expression_0_to_arrow_function:n(95105,e.DiagnosticCategory.Message,"Convert_function_expression_0_to_arrow_function_95105","Convert function expression '{0}' to arrow function"),Convert_function_declaration_0_to_arrow_function:n(95106,e.DiagnosticCategory.Message,"Convert_function_declaration_0_to_arrow_function_95106","Convert function declaration '{0}' to arrow function"),Fix_all_implicit_this_errors:n(95107,e.DiagnosticCategory.Message,"Fix_all_implicit_this_errors_95107","Fix all implicit-'this' errors"),Wrap_invalid_character_in_an_expression_container:n(95108,e.DiagnosticCategory.Message,"Wrap_invalid_character_in_an_expression_container_95108","Wrap invalid character in an expression container"),Wrap_all_invalid_characters_in_an_expression_container:n(95109,e.DiagnosticCategory.Message,"Wrap_all_invalid_characters_in_an_expression_container_95109","Wrap all invalid characters in an expression container"),Visit_https_Colon_Slash_Slashaka_ms_Slashtsconfig_json_to_read_more_about_this_file:n(95110,e.DiagnosticCategory.Message,"Visit_https_Colon_Slash_Slashaka_ms_Slashtsconfig_json_to_read_more_about_this_file_95110","Visit https://aka.ms/tsconfig.json to read more about this file"),Add_a_return_statement:n(95111,e.DiagnosticCategory.Message,"Add_a_return_statement_95111","Add a return statement"),Remove_braces_from_arrow_function_body:n(95112,e.DiagnosticCategory.Message,"Remove_braces_from_arrow_function_body_95112","Remove braces from arrow function body"),Wrap_the_following_body_with_parentheses_which_should_be_an_object_literal:n(95113,e.DiagnosticCategory.Message,"Wrap_the_following_body_with_parentheses_which_should_be_an_object_literal_95113","Wrap the following body with parentheses which should be an object literal"),Add_all_missing_return_statement:n(95114,e.DiagnosticCategory.Message,"Add_all_missing_return_statement_95114","Add all missing return statement"),Remove_braces_from_all_arrow_function_bodies_with_relevant_issues:n(95115,e.DiagnosticCategory.Message,"Remove_braces_from_all_arrow_function_bodies_with_relevant_issues_95115","Remove braces from all arrow function bodies with relevant issues"),Wrap_all_object_literal_with_parentheses:n(95116,e.DiagnosticCategory.Message,"Wrap_all_object_literal_with_parentheses_95116","Wrap all object literal with parentheses"),Move_labeled_tuple_element_modifiers_to_labels:n(95117,e.DiagnosticCategory.Message,"Move_labeled_tuple_element_modifiers_to_labels_95117","Move labeled tuple element modifiers to labels"),Convert_overload_list_to_single_signature:n(95118,e.DiagnosticCategory.Message,"Convert_overload_list_to_single_signature_95118","Convert overload list to single signature"),Generate_get_and_set_accessors_for_all_overriding_properties:n(95119,e.DiagnosticCategory.Message,"Generate_get_and_set_accessors_for_all_overriding_properties_95119","Generate 'get' and 'set' accessors for all overriding properties"),Wrap_in_JSX_fragment:n(95120,e.DiagnosticCategory.Message,"Wrap_in_JSX_fragment_95120","Wrap in JSX fragment"),Wrap_all_unparented_JSX_in_JSX_fragment:n(95121,e.DiagnosticCategory.Message,"Wrap_all_unparented_JSX_in_JSX_fragment_95121","Wrap all unparented JSX in JSX fragment"),Convert_arrow_function_or_function_expression:n(95122,e.DiagnosticCategory.Message,"Convert_arrow_function_or_function_expression_95122","Convert arrow function or function expression"),Convert_to_anonymous_function:n(95123,e.DiagnosticCategory.Message,"Convert_to_anonymous_function_95123","Convert to anonymous function"),Convert_to_named_function:n(95124,e.DiagnosticCategory.Message,"Convert_to_named_function_95124","Convert to named function"),Convert_to_arrow_function:n(95125,e.DiagnosticCategory.Message,"Convert_to_arrow_function_95125","Convert to arrow function"),Remove_parentheses:n(95126,e.DiagnosticCategory.Message,"Remove_parentheses_95126","Remove parentheses"),Could_not_find_a_containing_arrow_function:n(95127,e.DiagnosticCategory.Message,"Could_not_find_a_containing_arrow_function_95127","Could not find a containing arrow function"),Containing_function_is_not_an_arrow_function:n(95128,e.DiagnosticCategory.Message,"Containing_function_is_not_an_arrow_function_95128","Containing function is not an arrow function"),Could_not_find_export_statement:n(95129,e.DiagnosticCategory.Message,"Could_not_find_export_statement_95129","Could not find export statement"),This_file_already_has_a_default_export:n(95130,e.DiagnosticCategory.Message,"This_file_already_has_a_default_export_95130","This file already has a default export"),Could_not_find_import_clause:n(95131,e.DiagnosticCategory.Message,"Could_not_find_import_clause_95131","Could not find import clause"),Could_not_find_namespace_import_or_named_imports:n(95132,e.DiagnosticCategory.Message,"Could_not_find_namespace_import_or_named_imports_95132","Could not find namespace import or named imports"),Selection_is_not_a_valid_type_node:n(95133,e.DiagnosticCategory.Message,"Selection_is_not_a_valid_type_node_95133","Selection is not a valid type node"),No_type_could_be_extracted_from_this_type_node:n(95134,e.DiagnosticCategory.Message,"No_type_could_be_extracted_from_this_type_node_95134","No type could be extracted from this type node"),Could_not_find_property_for_which_to_generate_accessor:n(95135,e.DiagnosticCategory.Message,"Could_not_find_property_for_which_to_generate_accessor_95135","Could not find property for which to generate accessor"),Name_is_not_valid:n(95136,e.DiagnosticCategory.Message,"Name_is_not_valid_95136","Name is not valid"),Can_only_convert_property_with_modifier:n(95137,e.DiagnosticCategory.Message,"Can_only_convert_property_with_modifier_95137","Can only convert property with modifier"),Switch_each_misused_0_to_1:n(95138,e.DiagnosticCategory.Message,"Switch_each_misused_0_to_1_95138","Switch each misused '{0}' to '{1}'"),Convert_to_optional_chain_expression:n(95139,e.DiagnosticCategory.Message,"Convert_to_optional_chain_expression_95139","Convert to optional chain expression"),Could_not_find_convertible_access_expression:n(95140,e.DiagnosticCategory.Message,"Could_not_find_convertible_access_expression_95140","Could not find convertible access expression"),Could_not_find_matching_access_expressions:n(95141,e.DiagnosticCategory.Message,"Could_not_find_matching_access_expressions_95141","Could not find matching access expressions"),Can_only_convert_logical_AND_access_chains:n(95142,e.DiagnosticCategory.Message,"Can_only_convert_logical_AND_access_chains_95142","Can only convert logical AND access chains"),Add_void_to_Promise_resolved_without_a_value:n(95143,e.DiagnosticCategory.Message,"Add_void_to_Promise_resolved_without_a_value_95143","Add 'void' to Promise resolved without a value"),Add_void_to_all_Promises_resolved_without_a_value:n(95144,e.DiagnosticCategory.Message,"Add_void_to_all_Promises_resolved_without_a_value_95144","Add 'void' to all Promises resolved without a value"),Use_element_access_for_0:n(95145,e.DiagnosticCategory.Message,"Use_element_access_for_0_95145","Use element access for '{0}'"),Use_element_access_for_all_undeclared_properties:n(95146,e.DiagnosticCategory.Message,"Use_element_access_for_all_undeclared_properties_95146","Use element access for all undeclared properties."),Delete_all_unused_imports:n(95147,e.DiagnosticCategory.Message,"Delete_all_unused_imports_95147","Delete all unused imports"),Infer_function_return_type:n(95148,e.DiagnosticCategory.Message,"Infer_function_return_type_95148","Infer function return type"),Return_type_must_be_inferred_from_a_function:n(95149,e.DiagnosticCategory.Message,"Return_type_must_be_inferred_from_a_function_95149","Return type must be inferred from a function"),Could_not_determine_function_return_type:n(95150,e.DiagnosticCategory.Message,"Could_not_determine_function_return_type_95150","Could not determine function return type"),Could_not_convert_to_arrow_function:n(95151,e.DiagnosticCategory.Message,"Could_not_convert_to_arrow_function_95151","Could not convert to arrow function"),Could_not_convert_to_named_function:n(95152,e.DiagnosticCategory.Message,"Could_not_convert_to_named_function_95152","Could not convert to named function"),Could_not_convert_to_anonymous_function:n(95153,e.DiagnosticCategory.Message,"Could_not_convert_to_anonymous_function_95153","Could not convert to anonymous function"),Can_only_convert_string_concatenation:n(95154,e.DiagnosticCategory.Message,"Can_only_convert_string_concatenation_95154","Can only convert string concatenation"),Selection_is_not_a_valid_statement_or_statements:n(95155,e.DiagnosticCategory.Message,"Selection_is_not_a_valid_statement_or_statements_95155","Selection is not a valid statement or statements"),Add_missing_function_declaration_0:n(95156,e.DiagnosticCategory.Message,"Add_missing_function_declaration_0_95156","Add missing function declaration '{0}'"),Add_all_missing_function_declarations:n(95157,e.DiagnosticCategory.Message,"Add_all_missing_function_declarations_95157","Add all missing function declarations"),Method_not_implemented:n(95158,e.DiagnosticCategory.Message,"Method_not_implemented_95158","Method not implemented."),Function_not_implemented:n(95159,e.DiagnosticCategory.Message,"Function_not_implemented_95159","Function not implemented."),Add_override_modifier:n(95160,e.DiagnosticCategory.Message,"Add_override_modifier_95160","Add 'override' modifier"),Remove_override_modifier:n(95161,e.DiagnosticCategory.Message,"Remove_override_modifier_95161","Remove 'override' modifier"),Add_all_missing_override_modifiers:n(95162,e.DiagnosticCategory.Message,"Add_all_missing_override_modifiers_95162","Add all missing 'override' modifiers"),Remove_all_unnecessary_override_modifiers:n(95163,e.DiagnosticCategory.Message,"Remove_all_unnecessary_override_modifiers_95163","Remove all unnecessary 'override' modifiers"),Can_only_convert_named_export:n(95164,e.DiagnosticCategory.Message,"Can_only_convert_named_export_95164","Can only convert named export"),Add_missing_properties:n(95165,e.DiagnosticCategory.Message,"Add_missing_properties_95165","Add missing properties"),Add_all_missing_properties:n(95166,e.DiagnosticCategory.Message,"Add_all_missing_properties_95166","Add all missing properties"),Add_missing_attributes:n(95167,e.DiagnosticCategory.Message,"Add_missing_attributes_95167","Add missing attributes"),Add_all_missing_attributes:n(95168,e.DiagnosticCategory.Message,"Add_all_missing_attributes_95168","Add all missing attributes"),No_value_exists_in_scope_for_the_shorthand_property_0_Either_declare_one_or_provide_an_initializer:n(18004,e.DiagnosticCategory.Error,"No_value_exists_in_scope_for_the_shorthand_property_0_Either_declare_one_or_provide_an_initializer_18004","No value exists in scope for the shorthand property '{0}'. Either declare one or provide an initializer."),Classes_may_not_have_a_field_named_constructor:n(18006,e.DiagnosticCategory.Error,"Classes_may_not_have_a_field_named_constructor_18006","Classes may not have a field named 'constructor'."),JSX_expressions_may_not_use_the_comma_operator_Did_you_mean_to_write_an_array:n(18007,e.DiagnosticCategory.Error,"JSX_expressions_may_not_use_the_comma_operator_Did_you_mean_to_write_an_array_18007","JSX expressions may not use the comma operator. Did you mean to write an array?"),Private_identifiers_cannot_be_used_as_parameters:n(18009,e.DiagnosticCategory.Error,"Private_identifiers_cannot_be_used_as_parameters_18009","Private identifiers cannot be used as parameters."),An_accessibility_modifier_cannot_be_used_with_a_private_identifier:n(18010,e.DiagnosticCategory.Error,"An_accessibility_modifier_cannot_be_used_with_a_private_identifier_18010","An accessibility modifier cannot be used with a private identifier."),The_operand_of_a_delete_operator_cannot_be_a_private_identifier:n(18011,e.DiagnosticCategory.Error,"The_operand_of_a_delete_operator_cannot_be_a_private_identifier_18011","The operand of a 'delete' operator cannot be a private identifier."),constructor_is_a_reserved_word:n(18012,e.DiagnosticCategory.Error,"constructor_is_a_reserved_word_18012","'#constructor' is a reserved word."),Property_0_is_not_accessible_outside_class_1_because_it_has_a_private_identifier:n(18013,e.DiagnosticCategory.Error,"Property_0_is_not_accessible_outside_class_1_because_it_has_a_private_identifier_18013","Property '{0}' is not accessible outside class '{1}' because it has a private identifier."),The_property_0_cannot_be_accessed_on_type_1_within_this_class_because_it_is_shadowed_by_another_private_identifier_with_the_same_spelling:n(18014,e.DiagnosticCategory.Error,"The_property_0_cannot_be_accessed_on_type_1_within_this_class_because_it_is_shadowed_by_another_priv_18014","The property '{0}' cannot be accessed on type '{1}' within this class because it is shadowed by another private identifier with the same spelling."),Property_0_in_type_1_refers_to_a_different_member_that_cannot_be_accessed_from_within_type_2:n(18015,e.DiagnosticCategory.Error,"Property_0_in_type_1_refers_to_a_different_member_that_cannot_be_accessed_from_within_type_2_18015","Property '{0}' in type '{1}' refers to a different member that cannot be accessed from within type '{2}'."),Private_identifiers_are_not_allowed_outside_class_bodies:n(18016,e.DiagnosticCategory.Error,"Private_identifiers_are_not_allowed_outside_class_bodies_18016","Private identifiers are not allowed outside class bodies."),The_shadowing_declaration_of_0_is_defined_here:n(18017,e.DiagnosticCategory.Error,"The_shadowing_declaration_of_0_is_defined_here_18017","The shadowing declaration of '{0}' is defined here"),The_declaration_of_0_that_you_probably_intended_to_use_is_defined_here:n(18018,e.DiagnosticCategory.Error,"The_declaration_of_0_that_you_probably_intended_to_use_is_defined_here_18018","The declaration of '{0}' that you probably intended to use is defined here"),_0_modifier_cannot_be_used_with_a_private_identifier:n(18019,e.DiagnosticCategory.Error,"_0_modifier_cannot_be_used_with_a_private_identifier_18019","'{0}' modifier cannot be used with a private identifier."),An_enum_member_cannot_be_named_with_a_private_identifier:n(18024,e.DiagnosticCategory.Error,"An_enum_member_cannot_be_named_with_a_private_identifier_18024","An enum member cannot be named with a private identifier."),can_only_be_used_at_the_start_of_a_file:n(18026,e.DiagnosticCategory.Error,"can_only_be_used_at_the_start_of_a_file_18026","'#!' can only be used at the start of a file."),Compiler_reserves_name_0_when_emitting_private_identifier_downlevel:n(18027,e.DiagnosticCategory.Error,"Compiler_reserves_name_0_when_emitting_private_identifier_downlevel_18027","Compiler reserves name '{0}' when emitting private identifier downlevel."),Private_identifiers_are_only_available_when_targeting_ECMAScript_2015_and_higher:n(18028,e.DiagnosticCategory.Error,"Private_identifiers_are_only_available_when_targeting_ECMAScript_2015_and_higher_18028","Private identifiers are only available when targeting ECMAScript 2015 and higher."),Private_identifiers_are_not_allowed_in_variable_declarations:n(18029,e.DiagnosticCategory.Error,"Private_identifiers_are_not_allowed_in_variable_declarations_18029","Private identifiers are not allowed in variable declarations."),An_optional_chain_cannot_contain_private_identifiers:n(18030,e.DiagnosticCategory.Error,"An_optional_chain_cannot_contain_private_identifiers_18030","An optional chain cannot contain private identifiers."),The_intersection_0_was_reduced_to_never_because_property_1_has_conflicting_types_in_some_constituents:n(18031,e.DiagnosticCategory.Error,"The_intersection_0_was_reduced_to_never_because_property_1_has_conflicting_types_in_some_constituent_18031","The intersection '{0}' was reduced to 'never' because property '{1}' has conflicting types in some constituents."),The_intersection_0_was_reduced_to_never_because_property_1_exists_in_multiple_constituents_and_is_private_in_some:n(18032,e.DiagnosticCategory.Error,"The_intersection_0_was_reduced_to_never_because_property_1_exists_in_multiple_constituents_and_is_pr_18032","The intersection '{0}' was reduced to 'never' because property '{1}' exists in multiple constituents and is private in some."),Only_numeric_enums_can_have_computed_members_but_this_expression_has_type_0_If_you_do_not_need_exhaustiveness_checks_consider_using_an_object_literal_instead:n(18033,e.DiagnosticCategory.Error,"Only_numeric_enums_can_have_computed_members_but_this_expression_has_type_0_If_you_do_not_need_exhau_18033","Only numeric enums can have computed members, but this expression has type '{0}'. If you do not need exhaustiveness checks, consider using an object literal instead."),Specify_the_JSX_fragment_factory_function_to_use_when_targeting_react_JSX_emit_with_jsxFactory_compiler_option_is_specified_e_g_Fragment:n(18034,e.DiagnosticCategory.Message,"Specify_the_JSX_fragment_factory_function_to_use_when_targeting_react_JSX_emit_with_jsxFactory_compi_18034","Specify the JSX fragment factory function to use when targeting 'react' JSX emit with 'jsxFactory' compiler option is specified, e.g. 'Fragment'."),Invalid_value_for_jsxFragmentFactory_0_is_not_a_valid_identifier_or_qualified_name:n(18035,e.DiagnosticCategory.Error,"Invalid_value_for_jsxFragmentFactory_0_is_not_a_valid_identifier_or_qualified_name_18035","Invalid value for 'jsxFragmentFactory'. '{0}' is not a valid identifier or qualified-name."),Class_decorators_can_t_be_used_with_static_private_identifier_Consider_removing_the_experimental_decorator:n(18036,e.DiagnosticCategory.Error,"Class_decorators_can_t_be_used_with_static_private_identifier_Consider_removing_the_experimental_dec_18036","Class decorators can't be used with static private identifier. Consider removing the experimental decorator."),Await_expression_cannot_be_used_inside_a_class_static_block:n(18037,e.DiagnosticCategory.Error,"Await_expression_cannot_be_used_inside_a_class_static_block_18037","Await expression cannot be used inside a class static block."),For_await_loops_cannot_be_used_inside_a_class_static_block:n(18038,e.DiagnosticCategory.Error,"For_await_loops_cannot_be_used_inside_a_class_static_block_18038","'For await' loops cannot be used inside a class static block."),Invalid_use_of_0_It_cannot_be_used_inside_a_class_static_block:n(18039,e.DiagnosticCategory.Error,"Invalid_use_of_0_It_cannot_be_used_inside_a_class_static_block_18039","Invalid use of '{0}'. It cannot be used inside a class static block."),A_return_statement_cannot_be_used_inside_a_class_static_block:n(18041,e.DiagnosticCategory.Error,"A_return_statement_cannot_be_used_inside_a_class_static_block_18041","A 'return' statement cannot be used inside a class static block.")}}(d||(d={})),function(e){var n;function t(e){return e>=79}function r(e){return 31===e||t(e)}e.tokenIsIdentifierOrKeyword=t,e.tokenIsIdentifierOrKeywordOrGreaterThan=r,e.textToKeywordObj=(n={abstract:126,any:129,as:127,asserts:128,bigint:156,boolean:132,break:81,case:82,catch:83,class:84,continue:86,const:85},n["constructor"]=133,n.debugger=87,n.declare=134,n.default=88,n.delete=89,n.do=90,n.else=91,n.enum=92,n.export=93,n.extends=94,n.false=95,n.finally=96,n.for=97,n.from=154,n.function=98,n.get=135,n.if=99,n.implements=117,n.import=100,n.in=101,n.infer=136,n.instanceof=102,n.interface=118,n.intrinsic=137,n.is=138,n.keyof=139,n.let=119,n.module=140,n.namespace=141,n.never=142,n.new=103,n.null=104,n.number=145,n.object=146,n.package=120,n.private=121,n.protected=122,n.public=123,n.override=157,n.readonly=143,n.require=144,n.global=155,n.return=105,n.set=147,n.static=124,n.string=148,n.super=106,n.switch=107,n.symbol=149,n.this=108,n.throw=109,n.true=110,n.try=111,n.type=150,n.typeof=112,n.undefined=151,n.unique=152,n.unknown=153,n.var=113,n.void=114,n.while=115,n.with=116,n.yield=125,n.async=130,n.await=131,n.of=158,n);var i=new e.Map(e.getEntries(e.textToKeywordObj)),a=new e.Map(e.getEntries(o(o({},e.textToKeywordObj),{"{":18,"}":19,"(":20,")":21,"[":22,"]":23,".":24,"...":25,";":26,",":27,"<":29,">":31,"<=":32,">=":33,"==":34,"!=":35,"===":36,"!==":37,"=>":38,"+":39,"-":40,"**":42,"*":41,"/":43,"%":44,"++":45,"--":46,"<<":47,"</":30,">>":48,">>>":49,"&":50,"|":51,"^":52,"!":53,"~":54,"&&":55,"||":56,"?":57,"??":60,"?.":28,":":58,"=":63,"+=":64,"-=":65,"*=":66,"**=":67,"/=":68,"%=":69,"<<=":70,">>=":71,">>>=":72,"&=":73,"|=":74,"^=":78,"||=":75,"&&=":76,"??=":77,"@":59,"#":62,"`":61}))),s=[170,170,181,181,186,186,192,214,216,246,248,543,546,563,592,685,688,696,699,705,720,721,736,740,750,750,890,890,902,902,904,906,908,908,910,929,931,974,976,983,986,1011,1024,1153,1164,1220,1223,1224,1227,1228,1232,1269,1272,1273,1329,1366,1369,1369,1377,1415,1488,1514,1520,1522,1569,1594,1600,1610,1649,1747,1749,1749,1765,1766,1786,1788,1808,1808,1810,1836,1920,1957,2309,2361,2365,2365,2384,2384,2392,2401,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2524,2525,2527,2529,2544,2545,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2649,2652,2654,2654,2674,2676,2693,2699,2701,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2749,2749,2768,2768,2784,2784,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2870,2873,2877,2877,2908,2909,2911,2913,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,2997,2999,3001,3077,3084,3086,3088,3090,3112,3114,3123,3125,3129,3168,3169,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3294,3294,3296,3297,3333,3340,3342,3344,3346,3368,3370,3385,3424,3425,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3585,3632,3634,3635,3648,3654,3713,3714,3716,3716,3719,3720,3722,3722,3725,3725,3732,3735,3737,3743,3745,3747,3749,3749,3751,3751,3754,3755,3757,3760,3762,3763,3773,3773,3776,3780,3782,3782,3804,3805,3840,3840,3904,3911,3913,3946,3976,3979,4096,4129,4131,4135,4137,4138,4176,4181,4256,4293,4304,4342,4352,4441,4447,4514,4520,4601,4608,4614,4616,4678,4680,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4742,4744,4744,4746,4749,4752,4782,4784,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4814,4816,4822,4824,4846,4848,4878,4880,4880,4882,4885,4888,4894,4896,4934,4936,4954,5024,5108,5121,5740,5743,5750,5761,5786,5792,5866,6016,6067,6176,6263,6272,6312,7680,7835,7840,7929,7936,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8319,8319,8450,8450,8455,8455,8458,8467,8469,8469,8473,8477,8484,8484,8486,8486,8488,8488,8490,8493,8495,8497,8499,8505,8544,8579,12293,12295,12321,12329,12337,12341,12344,12346,12353,12436,12445,12446,12449,12538,12540,12542,12549,12588,12593,12686,12704,12727,13312,19893,19968,40869,40960,42124,44032,55203,63744,64045,64256,64262,64275,64279,64285,64285,64287,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65136,65138,65140,65140,65142,65276,65313,65338,65345,65370,65382,65470,65474,65479,65482,65487,65490,65495,65498,65500],c=[170,170,181,181,186,186,192,214,216,246,248,543,546,563,592,685,688,696,699,705,720,721,736,740,750,750,768,846,864,866,890,890,902,902,904,906,908,908,910,929,931,974,976,983,986,1011,1024,1153,1155,1158,1164,1220,1223,1224,1227,1228,1232,1269,1272,1273,1329,1366,1369,1369,1377,1415,1425,1441,1443,1465,1467,1469,1471,1471,1473,1474,1476,1476,1488,1514,1520,1522,1569,1594,1600,1621,1632,1641,1648,1747,1749,1756,1759,1768,1770,1773,1776,1788,1808,1836,1840,1866,1920,1968,2305,2307,2309,2361,2364,2381,2384,2388,2392,2403,2406,2415,2433,2435,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2492,2492,2494,2500,2503,2504,2507,2509,2519,2519,2524,2525,2527,2531,2534,2545,2562,2562,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2620,2620,2622,2626,2631,2632,2635,2637,2649,2652,2654,2654,2662,2676,2689,2691,2693,2699,2701,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2748,2757,2759,2761,2763,2765,2768,2768,2784,2784,2790,2799,2817,2819,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2870,2873,2876,2883,2887,2888,2891,2893,2902,2903,2908,2909,2911,2913,2918,2927,2946,2947,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,2997,2999,3001,3006,3010,3014,3016,3018,3021,3031,3031,3047,3055,3073,3075,3077,3084,3086,3088,3090,3112,3114,3123,3125,3129,3134,3140,3142,3144,3146,3149,3157,3158,3168,3169,3174,3183,3202,3203,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3262,3268,3270,3272,3274,3277,3285,3286,3294,3294,3296,3297,3302,3311,3330,3331,3333,3340,3342,3344,3346,3368,3370,3385,3390,3395,3398,3400,3402,3405,3415,3415,3424,3425,3430,3439,3458,3459,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3530,3530,3535,3540,3542,3542,3544,3551,3570,3571,3585,3642,3648,3662,3664,3673,3713,3714,3716,3716,3719,3720,3722,3722,3725,3725,3732,3735,3737,3743,3745,3747,3749,3749,3751,3751,3754,3755,3757,3769,3771,3773,3776,3780,3782,3782,3784,3789,3792,3801,3804,3805,3840,3840,3864,3865,3872,3881,3893,3893,3895,3895,3897,3897,3902,3911,3913,3946,3953,3972,3974,3979,3984,3991,3993,4028,4038,4038,4096,4129,4131,4135,4137,4138,4140,4146,4150,4153,4160,4169,4176,4185,4256,4293,4304,4342,4352,4441,4447,4514,4520,4601,4608,4614,4616,4678,4680,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4742,4744,4744,4746,4749,4752,4782,4784,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4814,4816,4822,4824,4846,4848,4878,4880,4880,4882,4885,4888,4894,4896,4934,4936,4954,4969,4977,5024,5108,5121,5740,5743,5750,5761,5786,5792,5866,6016,6099,6112,6121,6160,6169,6176,6263,6272,6313,7680,7835,7840,7929,7936,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8255,8256,8319,8319,8400,8412,8417,8417,8450,8450,8455,8455,8458,8467,8469,8469,8473,8477,8484,8484,8486,8486,8488,8488,8490,8493,8495,8497,8499,8505,8544,8579,12293,12295,12321,12335,12337,12341,12344,12346,12353,12436,12441,12442,12445,12446,12449,12542,12549,12588,12593,12686,12704,12727,13312,19893,19968,40869,40960,42124,44032,55203,63744,64045,64256,64262,64275,64279,64285,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65056,65059,65075,65076,65101,65103,65136,65138,65140,65140,65142,65276,65296,65305,65313,65338,65343,65343,65345,65370,65381,65470,65474,65479,65482,65487,65490,65495,65498,65500],l=[170,170,181,181,186,186,192,214,216,246,248,705,710,721,736,740,748,748,750,750,880,884,886,887,890,893,902,902,904,906,908,908,910,929,931,1013,1015,1153,1162,1319,1329,1366,1369,1369,1377,1415,1488,1514,1520,1522,1568,1610,1646,1647,1649,1747,1749,1749,1765,1766,1774,1775,1786,1788,1791,1791,1808,1808,1810,1839,1869,1957,1969,1969,1994,2026,2036,2037,2042,2042,2048,2069,2074,2074,2084,2084,2088,2088,2112,2136,2208,2208,2210,2220,2308,2361,2365,2365,2384,2384,2392,2401,2417,2423,2425,2431,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2493,2493,2510,2510,2524,2525,2527,2529,2544,2545,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2649,2652,2654,2654,2674,2676,2693,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2749,2749,2768,2768,2784,2785,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2869,2873,2877,2877,2908,2909,2911,2913,2929,2929,2947,2947,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,3001,3024,3024,3077,3084,3086,3088,3090,3112,3114,3123,3125,3129,3133,3133,3160,3161,3168,3169,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3261,3261,3294,3294,3296,3297,3313,3314,3333,3340,3342,3344,3346,3386,3389,3389,3406,3406,3424,3425,3450,3455,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3585,3632,3634,3635,3648,3654,3713,3714,3716,3716,3719,3720,3722,3722,3725,3725,3732,3735,3737,3743,3745,3747,3749,3749,3751,3751,3754,3755,3757,3760,3762,3763,3773,3773,3776,3780,3782,3782,3804,3807,3840,3840,3904,3911,3913,3948,3976,3980,4096,4138,4159,4159,4176,4181,4186,4189,4193,4193,4197,4198,4206,4208,4213,4225,4238,4238,4256,4293,4295,4295,4301,4301,4304,4346,4348,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4744,4746,4749,4752,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4822,4824,4880,4882,4885,4888,4954,4992,5007,5024,5108,5121,5740,5743,5759,5761,5786,5792,5866,5870,5872,5888,5900,5902,5905,5920,5937,5952,5969,5984,5996,5998,6e3,6016,6067,6103,6103,6108,6108,6176,6263,6272,6312,6314,6314,6320,6389,6400,6428,6480,6509,6512,6516,6528,6571,6593,6599,6656,6678,6688,6740,6823,6823,6917,6963,6981,6987,7043,7072,7086,7087,7098,7141,7168,7203,7245,7247,7258,7293,7401,7404,7406,7409,7413,7414,7424,7615,7680,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8305,8305,8319,8319,8336,8348,8450,8450,8455,8455,8458,8467,8469,8469,8473,8477,8484,8484,8486,8486,8488,8488,8490,8493,8495,8505,8508,8511,8517,8521,8526,8526,8544,8584,11264,11310,11312,11358,11360,11492,11499,11502,11506,11507,11520,11557,11559,11559,11565,11565,11568,11623,11631,11631,11648,11670,11680,11686,11688,11694,11696,11702,11704,11710,11712,11718,11720,11726,11728,11734,11736,11742,11823,11823,12293,12295,12321,12329,12337,12341,12344,12348,12353,12438,12445,12447,12449,12538,12540,12543,12549,12589,12593,12686,12704,12730,12784,12799,13312,19893,19968,40908,40960,42124,42192,42237,42240,42508,42512,42527,42538,42539,42560,42606,42623,42647,42656,42735,42775,42783,42786,42888,42891,42894,42896,42899,42912,42922,43e3,43009,43011,43013,43015,43018,43020,43042,43072,43123,43138,43187,43250,43255,43259,43259,43274,43301,43312,43334,43360,43388,43396,43442,43471,43471,43520,43560,43584,43586,43588,43595,43616,43638,43642,43642,43648,43695,43697,43697,43701,43702,43705,43709,43712,43712,43714,43714,43739,43741,43744,43754,43762,43764,43777,43782,43785,43790,43793,43798,43808,43814,43816,43822,43968,44002,44032,55203,55216,55238,55243,55291,63744,64109,64112,64217,64256,64262,64275,64279,64285,64285,64287,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65136,65140,65142,65276,65313,65338,65345,65370,65382,65470,65474,65479,65482,65487,65490,65495,65498,65500],u=[170,170,181,181,186,186,192,214,216,246,248,705,710,721,736,740,748,748,750,750,768,884,886,887,890,893,902,902,904,906,908,908,910,929,931,1013,1015,1153,1155,1159,1162,1319,1329,1366,1369,1369,1377,1415,1425,1469,1471,1471,1473,1474,1476,1477,1479,1479,1488,1514,1520,1522,1552,1562,1568,1641,1646,1747,1749,1756,1759,1768,1770,1788,1791,1791,1808,1866,1869,1969,1984,2037,2042,2042,2048,2093,2112,2139,2208,2208,2210,2220,2276,2302,2304,2403,2406,2415,2417,2423,2425,2431,2433,2435,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2492,2500,2503,2504,2507,2510,2519,2519,2524,2525,2527,2531,2534,2545,2561,2563,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2620,2620,2622,2626,2631,2632,2635,2637,2641,2641,2649,2652,2654,2654,2662,2677,2689,2691,2693,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2748,2757,2759,2761,2763,2765,2768,2768,2784,2787,2790,2799,2817,2819,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2869,2873,2876,2884,2887,2888,2891,2893,2902,2903,2908,2909,2911,2915,2918,2927,2929,2929,2946,2947,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,3001,3006,3010,3014,3016,3018,3021,3024,3024,3031,3031,3046,3055,3073,3075,3077,3084,3086,3088,3090,3112,3114,3123,3125,3129,3133,3140,3142,3144,3146,3149,3157,3158,3160,3161,3168,3171,3174,3183,3202,3203,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3260,3268,3270,3272,3274,3277,3285,3286,3294,3294,3296,3299,3302,3311,3313,3314,3330,3331,3333,3340,3342,3344,3346,3386,3389,3396,3398,3400,3402,3406,3415,3415,3424,3427,3430,3439,3450,3455,3458,3459,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3530,3530,3535,3540,3542,3542,3544,3551,3570,3571,3585,3642,3648,3662,3664,3673,3713,3714,3716,3716,3719,3720,3722,3722,3725,3725,3732,3735,3737,3743,3745,3747,3749,3749,3751,3751,3754,3755,3757,3769,3771,3773,3776,3780,3782,3782,3784,3789,3792,3801,3804,3807,3840,3840,3864,3865,3872,3881,3893,3893,3895,3895,3897,3897,3902,3911,3913,3948,3953,3972,3974,3991,3993,4028,4038,4038,4096,4169,4176,4253,4256,4293,4295,4295,4301,4301,4304,4346,4348,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4744,4746,4749,4752,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4822,4824,4880,4882,4885,4888,4954,4957,4959,4992,5007,5024,5108,5121,5740,5743,5759,5761,5786,5792,5866,5870,5872,5888,5900,5902,5908,5920,5940,5952,5971,5984,5996,5998,6e3,6002,6003,6016,6099,6103,6103,6108,6109,6112,6121,6155,6157,6160,6169,6176,6263,6272,6314,6320,6389,6400,6428,6432,6443,6448,6459,6470,6509,6512,6516,6528,6571,6576,6601,6608,6617,6656,6683,6688,6750,6752,6780,6783,6793,6800,6809,6823,6823,6912,6987,6992,7001,7019,7027,7040,7155,7168,7223,7232,7241,7245,7293,7376,7378,7380,7414,7424,7654,7676,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8204,8205,8255,8256,8276,8276,8305,8305,8319,8319,8336,8348,8400,8412,8417,8417,8421,8432,8450,8450,8455,8455,8458,8467,8469,8469,8473,8477,8484,8484,8486,8486,8488,8488,8490,8493,8495,8505,8508,8511,8517,8521,8526,8526,8544,8584,11264,11310,11312,11358,11360,11492,11499,11507,11520,11557,11559,11559,11565,11565,11568,11623,11631,11631,11647,11670,11680,11686,11688,11694,11696,11702,11704,11710,11712,11718,11720,11726,11728,11734,11736,11742,11744,11775,11823,11823,12293,12295,12321,12335,12337,12341,12344,12348,12353,12438,12441,12442,12445,12447,12449,12538,12540,12543,12549,12589,12593,12686,12704,12730,12784,12799,13312,19893,19968,40908,40960,42124,42192,42237,42240,42508,42512,42539,42560,42607,42612,42621,42623,42647,42655,42737,42775,42783,42786,42888,42891,42894,42896,42899,42912,42922,43e3,43047,43072,43123,43136,43204,43216,43225,43232,43255,43259,43259,43264,43309,43312,43347,43360,43388,43392,43456,43471,43481,43520,43574,43584,43597,43600,43609,43616,43638,43642,43643,43648,43714,43739,43741,43744,43759,43762,43766,43777,43782,43785,43790,43793,43798,43808,43814,43816,43822,43968,44010,44012,44013,44016,44025,44032,55203,55216,55238,55243,55291,63744,64109,64112,64217,64256,64262,64275,64279,64285,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65024,65039,65056,65062,65075,65076,65101,65103,65136,65140,65142,65276,65296,65305,65313,65338,65343,65343,65345,65370,65382,65470,65474,65479,65482,65487,65490,65495,65498,65500],d=[65,90,97,122,170,170,181,181,186,186,192,214,216,246,248,705,710,721,736,740,748,748,750,750,880,884,886,887,890,893,895,895,902,902,904,906,908,908,910,929,931,1013,1015,1153,1162,1327,1329,1366,1369,1369,1376,1416,1488,1514,1519,1522,1568,1610,1646,1647,1649,1747,1749,1749,1765,1766,1774,1775,1786,1788,1791,1791,1808,1808,1810,1839,1869,1957,1969,1969,1994,2026,2036,2037,2042,2042,2048,2069,2074,2074,2084,2084,2088,2088,2112,2136,2144,2154,2208,2228,2230,2237,2308,2361,2365,2365,2384,2384,2392,2401,2417,2432,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2493,2493,2510,2510,2524,2525,2527,2529,2544,2545,2556,2556,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2649,2652,2654,2654,2674,2676,2693,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2749,2749,2768,2768,2784,2785,2809,2809,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2869,2873,2877,2877,2908,2909,2911,2913,2929,2929,2947,2947,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,3001,3024,3024,3077,3084,3086,3088,3090,3112,3114,3129,3133,3133,3160,3162,3168,3169,3200,3200,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3261,3261,3294,3294,3296,3297,3313,3314,3333,3340,3342,3344,3346,3386,3389,3389,3406,3406,3412,3414,3423,3425,3450,3455,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3585,3632,3634,3635,3648,3654,3713,3714,3716,3716,3718,3722,3724,3747,3749,3749,3751,3760,3762,3763,3773,3773,3776,3780,3782,3782,3804,3807,3840,3840,3904,3911,3913,3948,3976,3980,4096,4138,4159,4159,4176,4181,4186,4189,4193,4193,4197,4198,4206,4208,4213,4225,4238,4238,4256,4293,4295,4295,4301,4301,4304,4346,4348,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4744,4746,4749,4752,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4822,4824,4880,4882,4885,4888,4954,4992,5007,5024,5109,5112,5117,5121,5740,5743,5759,5761,5786,5792,5866,5870,5880,5888,5900,5902,5905,5920,5937,5952,5969,5984,5996,5998,6e3,6016,6067,6103,6103,6108,6108,6176,6264,6272,6312,6314,6314,6320,6389,6400,6430,6480,6509,6512,6516,6528,6571,6576,6601,6656,6678,6688,6740,6823,6823,6917,6963,6981,6987,7043,7072,7086,7087,7098,7141,7168,7203,7245,7247,7258,7293,7296,7304,7312,7354,7357,7359,7401,7404,7406,7411,7413,7414,7418,7418,7424,7615,7680,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8305,8305,8319,8319,8336,8348,8450,8450,8455,8455,8458,8467,8469,8469,8472,8477,8484,8484,8486,8486,8488,8488,8490,8505,8508,8511,8517,8521,8526,8526,8544,8584,11264,11310,11312,11358,11360,11492,11499,11502,11506,11507,11520,11557,11559,11559,11565,11565,11568,11623,11631,11631,11648,11670,11680,11686,11688,11694,11696,11702,11704,11710,11712,11718,11720,11726,11728,11734,11736,11742,12293,12295,12321,12329,12337,12341,12344,12348,12353,12438,12443,12447,12449,12538,12540,12543,12549,12591,12593,12686,12704,12730,12784,12799,13312,19893,19968,40943,40960,42124,42192,42237,42240,42508,42512,42527,42538,42539,42560,42606,42623,42653,42656,42735,42775,42783,42786,42888,42891,42943,42946,42950,42999,43009,43011,43013,43015,43018,43020,43042,43072,43123,43138,43187,43250,43255,43259,43259,43261,43262,43274,43301,43312,43334,43360,43388,43396,43442,43471,43471,43488,43492,43494,43503,43514,43518,43520,43560,43584,43586,43588,43595,43616,43638,43642,43642,43646,43695,43697,43697,43701,43702,43705,43709,43712,43712,43714,43714,43739,43741,43744,43754,43762,43764,43777,43782,43785,43790,43793,43798,43808,43814,43816,43822,43824,43866,43868,43879,43888,44002,44032,55203,55216,55238,55243,55291,63744,64109,64112,64217,64256,64262,64275,64279,64285,64285,64287,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65136,65140,65142,65276,65313,65338,65345,65370,65382,65470,65474,65479,65482,65487,65490,65495,65498,65500,65536,65547,65549,65574,65576,65594,65596,65597,65599,65613,65616,65629,65664,65786,65856,65908,66176,66204,66208,66256,66304,66335,66349,66378,66384,66421,66432,66461,66464,66499,66504,66511,66513,66517,66560,66717,66736,66771,66776,66811,66816,66855,66864,66915,67072,67382,67392,67413,67424,67431,67584,67589,67592,67592,67594,67637,67639,67640,67644,67644,67647,67669,67680,67702,67712,67742,67808,67826,67828,67829,67840,67861,67872,67897,67968,68023,68030,68031,68096,68096,68112,68115,68117,68119,68121,68149,68192,68220,68224,68252,68288,68295,68297,68324,68352,68405,68416,68437,68448,68466,68480,68497,68608,68680,68736,68786,68800,68850,68864,68899,69376,69404,69415,69415,69424,69445,69600,69622,69635,69687,69763,69807,69840,69864,69891,69926,69956,69956,69968,70002,70006,70006,70019,70066,70081,70084,70106,70106,70108,70108,70144,70161,70163,70187,70272,70278,70280,70280,70282,70285,70287,70301,70303,70312,70320,70366,70405,70412,70415,70416,70419,70440,70442,70448,70450,70451,70453,70457,70461,70461,70480,70480,70493,70497,70656,70708,70727,70730,70751,70751,70784,70831,70852,70853,70855,70855,71040,71086,71128,71131,71168,71215,71236,71236,71296,71338,71352,71352,71424,71450,71680,71723,71840,71903,71935,71935,72096,72103,72106,72144,72161,72161,72163,72163,72192,72192,72203,72242,72250,72250,72272,72272,72284,72329,72349,72349,72384,72440,72704,72712,72714,72750,72768,72768,72818,72847,72960,72966,72968,72969,72971,73008,73030,73030,73056,73061,73063,73064,73066,73097,73112,73112,73440,73458,73728,74649,74752,74862,74880,75075,77824,78894,82944,83526,92160,92728,92736,92766,92880,92909,92928,92975,92992,92995,93027,93047,93053,93071,93760,93823,93952,94026,94032,94032,94099,94111,94176,94177,94179,94179,94208,100343,100352,101106,110592,110878,110928,110930,110948,110951,110960,111355,113664,113770,113776,113788,113792,113800,113808,113817,119808,119892,119894,119964,119966,119967,119970,119970,119973,119974,119977,119980,119982,119993,119995,119995,119997,120003,120005,120069,120071,120074,120077,120084,120086,120092,120094,120121,120123,120126,120128,120132,120134,120134,120138,120144,120146,120485,120488,120512,120514,120538,120540,120570,120572,120596,120598,120628,120630,120654,120656,120686,120688,120712,120714,120744,120746,120770,120772,120779,123136,123180,123191,123197,123214,123214,123584,123627,124928,125124,125184,125251,125259,125259,126464,126467,126469,126495,126497,126498,126500,126500,126503,126503,126505,126514,126516,126519,126521,126521,126523,126523,126530,126530,126535,126535,126537,126537,126539,126539,126541,126543,126545,126546,126548,126548,126551,126551,126553,126553,126555,126555,126557,126557,126559,126559,126561,126562,126564,126564,126567,126570,126572,126578,126580,126583,126585,126588,126590,126590,126592,126601,126603,126619,126625,126627,126629,126633,126635,126651,131072,173782,173824,177972,177984,178205,178208,183969,183984,191456,194560,195101],p=[48,57,65,90,95,95,97,122,170,170,181,181,183,183,186,186,192,214,216,246,248,705,710,721,736,740,748,748,750,750,768,884,886,887,890,893,895,895,902,906,908,908,910,929,931,1013,1015,1153,1155,1159,1162,1327,1329,1366,1369,1369,1376,1416,1425,1469,1471,1471,1473,1474,1476,1477,1479,1479,1488,1514,1519,1522,1552,1562,1568,1641,1646,1747,1749,1756,1759,1768,1770,1788,1791,1791,1808,1866,1869,1969,1984,2037,2042,2042,2045,2045,2048,2093,2112,2139,2144,2154,2208,2228,2230,2237,2259,2273,2275,2403,2406,2415,2417,2435,2437,2444,2447,2448,2451,2472,2474,2480,2482,2482,2486,2489,2492,2500,2503,2504,2507,2510,2519,2519,2524,2525,2527,2531,2534,2545,2556,2556,2558,2558,2561,2563,2565,2570,2575,2576,2579,2600,2602,2608,2610,2611,2613,2614,2616,2617,2620,2620,2622,2626,2631,2632,2635,2637,2641,2641,2649,2652,2654,2654,2662,2677,2689,2691,2693,2701,2703,2705,2707,2728,2730,2736,2738,2739,2741,2745,2748,2757,2759,2761,2763,2765,2768,2768,2784,2787,2790,2799,2809,2815,2817,2819,2821,2828,2831,2832,2835,2856,2858,2864,2866,2867,2869,2873,2876,2884,2887,2888,2891,2893,2902,2903,2908,2909,2911,2915,2918,2927,2929,2929,2946,2947,2949,2954,2958,2960,2962,2965,2969,2970,2972,2972,2974,2975,2979,2980,2984,2986,2990,3001,3006,3010,3014,3016,3018,3021,3024,3024,3031,3031,3046,3055,3072,3084,3086,3088,3090,3112,3114,3129,3133,3140,3142,3144,3146,3149,3157,3158,3160,3162,3168,3171,3174,3183,3200,3203,3205,3212,3214,3216,3218,3240,3242,3251,3253,3257,3260,3268,3270,3272,3274,3277,3285,3286,3294,3294,3296,3299,3302,3311,3313,3314,3328,3331,3333,3340,3342,3344,3346,3396,3398,3400,3402,3406,3412,3415,3423,3427,3430,3439,3450,3455,3458,3459,3461,3478,3482,3505,3507,3515,3517,3517,3520,3526,3530,3530,3535,3540,3542,3542,3544,3551,3558,3567,3570,3571,3585,3642,3648,3662,3664,3673,3713,3714,3716,3716,3718,3722,3724,3747,3749,3749,3751,3773,3776,3780,3782,3782,3784,3789,3792,3801,3804,3807,3840,3840,3864,3865,3872,3881,3893,3893,3895,3895,3897,3897,3902,3911,3913,3948,3953,3972,3974,3991,3993,4028,4038,4038,4096,4169,4176,4253,4256,4293,4295,4295,4301,4301,4304,4346,4348,4680,4682,4685,4688,4694,4696,4696,4698,4701,4704,4744,4746,4749,4752,4784,4786,4789,4792,4798,4800,4800,4802,4805,4808,4822,4824,4880,4882,4885,4888,4954,4957,4959,4969,4977,4992,5007,5024,5109,5112,5117,5121,5740,5743,5759,5761,5786,5792,5866,5870,5880,5888,5900,5902,5908,5920,5940,5952,5971,5984,5996,5998,6e3,6002,6003,6016,6099,6103,6103,6108,6109,6112,6121,6155,6157,6160,6169,6176,6264,6272,6314,6320,6389,6400,6430,6432,6443,6448,6459,6470,6509,6512,6516,6528,6571,6576,6601,6608,6618,6656,6683,6688,6750,6752,6780,6783,6793,6800,6809,6823,6823,6832,6845,6912,6987,6992,7001,7019,7027,7040,7155,7168,7223,7232,7241,7245,7293,7296,7304,7312,7354,7357,7359,7376,7378,7380,7418,7424,7673,7675,7957,7960,7965,7968,8005,8008,8013,8016,8023,8025,8025,8027,8027,8029,8029,8031,8061,8064,8116,8118,8124,8126,8126,8130,8132,8134,8140,8144,8147,8150,8155,8160,8172,8178,8180,8182,8188,8255,8256,8276,8276,8305,8305,8319,8319,8336,8348,8400,8412,8417,8417,8421,8432,8450,8450,8455,8455,8458,8467,8469,8469,8472,8477,8484,8484,8486,8486,8488,8488,8490,8505,8508,8511,8517,8521,8526,8526,8544,8584,11264,11310,11312,11358,11360,11492,11499,11507,11520,11557,11559,11559,11565,11565,11568,11623,11631,11631,11647,11670,11680,11686,11688,11694,11696,11702,11704,11710,11712,11718,11720,11726,11728,11734,11736,11742,11744,11775,12293,12295,12321,12335,12337,12341,12344,12348,12353,12438,12441,12447,12449,12538,12540,12543,12549,12591,12593,12686,12704,12730,12784,12799,13312,19893,19968,40943,40960,42124,42192,42237,42240,42508,42512,42539,42560,42607,42612,42621,42623,42737,42775,42783,42786,42888,42891,42943,42946,42950,42999,43047,43072,43123,43136,43205,43216,43225,43232,43255,43259,43259,43261,43309,43312,43347,43360,43388,43392,43456,43471,43481,43488,43518,43520,43574,43584,43597,43600,43609,43616,43638,43642,43714,43739,43741,43744,43759,43762,43766,43777,43782,43785,43790,43793,43798,43808,43814,43816,43822,43824,43866,43868,43879,43888,44010,44012,44013,44016,44025,44032,55203,55216,55238,55243,55291,63744,64109,64112,64217,64256,64262,64275,64279,64285,64296,64298,64310,64312,64316,64318,64318,64320,64321,64323,64324,64326,64433,64467,64829,64848,64911,64914,64967,65008,65019,65024,65039,65056,65071,65075,65076,65101,65103,65136,65140,65142,65276,65296,65305,65313,65338,65343,65343,65345,65370,65382,65470,65474,65479,65482,65487,65490,65495,65498,65500,65536,65547,65549,65574,65576,65594,65596,65597,65599,65613,65616,65629,65664,65786,65856,65908,66045,66045,66176,66204,66208,66256,66272,66272,66304,66335,66349,66378,66384,66426,66432,66461,66464,66499,66504,66511,66513,66517,66560,66717,66720,66729,66736,66771,66776,66811,66816,66855,66864,66915,67072,67382,67392,67413,67424,67431,67584,67589,67592,67592,67594,67637,67639,67640,67644,67644,67647,67669,67680,67702,67712,67742,67808,67826,67828,67829,67840,67861,67872,67897,67968,68023,68030,68031,68096,68099,68101,68102,68108,68115,68117,68119,68121,68149,68152,68154,68159,68159,68192,68220,68224,68252,68288,68295,68297,68326,68352,68405,68416,68437,68448,68466,68480,68497,68608,68680,68736,68786,68800,68850,68864,68903,68912,68921,69376,69404,69415,69415,69424,69456,69600,69622,69632,69702,69734,69743,69759,69818,69840,69864,69872,69881,69888,69940,69942,69951,69956,69958,69968,70003,70006,70006,70016,70084,70089,70092,70096,70106,70108,70108,70144,70161,70163,70199,70206,70206,70272,70278,70280,70280,70282,70285,70287,70301,70303,70312,70320,70378,70384,70393,70400,70403,70405,70412,70415,70416,70419,70440,70442,70448,70450,70451,70453,70457,70459,70468,70471,70472,70475,70477,70480,70480,70487,70487,70493,70499,70502,70508,70512,70516,70656,70730,70736,70745,70750,70751,70784,70853,70855,70855,70864,70873,71040,71093,71096,71104,71128,71133,71168,71232,71236,71236,71248,71257,71296,71352,71360,71369,71424,71450,71453,71467,71472,71481,71680,71738,71840,71913,71935,71935,72096,72103,72106,72151,72154,72161,72163,72164,72192,72254,72263,72263,72272,72345,72349,72349,72384,72440,72704,72712,72714,72758,72760,72768,72784,72793,72818,72847,72850,72871,72873,72886,72960,72966,72968,72969,72971,73014,73018,73018,73020,73021,73023,73031,73040,73049,73056,73061,73063,73064,73066,73102,73104,73105,73107,73112,73120,73129,73440,73462,73728,74649,74752,74862,74880,75075,77824,78894,82944,83526,92160,92728,92736,92766,92768,92777,92880,92909,92912,92916,92928,92982,92992,92995,93008,93017,93027,93047,93053,93071,93760,93823,93952,94026,94031,94087,94095,94111,94176,94177,94179,94179,94208,100343,100352,101106,110592,110878,110928,110930,110948,110951,110960,111355,113664,113770,113776,113788,113792,113800,113808,113817,113821,113822,119141,119145,119149,119154,119163,119170,119173,119179,119210,119213,119362,119364,119808,119892,119894,119964,119966,119967,119970,119970,119973,119974,119977,119980,119982,119993,119995,119995,119997,120003,120005,120069,120071,120074,120077,120084,120086,120092,120094,120121,120123,120126,120128,120132,120134,120134,120138,120144,120146,120485,120488,120512,120514,120538,120540,120570,120572,120596,120598,120628,120630,120654,120656,120686,120688,120712,120714,120744,120746,120770,120772,120779,120782,120831,121344,121398,121403,121452,121461,121461,121476,121476,121499,121503,121505,121519,122880,122886,122888,122904,122907,122913,122915,122916,122918,122922,123136,123180,123184,123197,123200,123209,123214,123214,123584,123641,124928,125124,125136,125142,125184,125259,125264,125273,126464,126467,126469,126495,126497,126498,126500,126500,126503,126503,126505,126514,126516,126519,126521,126521,126523,126523,126530,126530,126535,126535,126537,126537,126539,126539,126541,126543,126545,126546,126548,126548,126551,126551,126553,126553,126555,126555,126557,126557,126559,126559,126561,126562,126564,126564,126567,126570,126572,126578,126580,126583,126585,126588,126590,126590,126592,126601,126603,126619,126625,126627,126629,126633,126635,126651,131072,173782,173824,177972,177984,178205,178208,183969,183984,191456,194560,195101,917760,917999],f=/^\/\/\/?\s*@(ts-expect-error|ts-ignore)/,m=/^(?:\/|\*)*\s*@(ts-expect-error|ts-ignore)/;function _(e,n){if(e<n[0])return!1;var t,r=0,i=n.length;while(r+1<i){if(t=r+(i-r)/2,t-=t%2,n[t]<=e&&e<=n[t+1])return!0;e<n[t]?i=t:r=t+2}return!1}function g(e,n){return _(e,n>=2?d:1===n?l:s)}function y(e,n){return _(e,n>=2?p:1===n?u:c)}function h(e){var n=[];return e.forEach(function(e,t){n[e]=t}),n}e.isUnicodeIdentifierStart=g;var v=h(a);function b(e){return v[e]}function E(e){return a.get(e)}function T(e){var n=new Array,t=0,r=0;while(t<e.length){var i=e.charCodeAt(t);switch(t++,i){case 13:10===e.charCodeAt(t)&&t++;case 10:n.push(r),r=t;break;default:i>127&&R(i)&&(n.push(r),r=t);break}}return n.push(r),n}function S(e,n,t,r){return e.getPositionOfLineAndCharacter?e.getPositionOfLineAndCharacter(n,t,r):x(L(e),n,t,e.text,r)}function x(n,t,r,i,a){(t<0||t>=n.length)&&(a?t=t<0?0:t>=n.length?n.length-1:t:e.Debug.fail("Bad line number. Line: "+t+", lineStarts.length: "+n.length+" , line map is correct? "+(void 0!==i?e.arraysEqual(n,T(i)):"unknown")));var o=n[t]+r;return a?o>n[t+1]?n[t+1]:"string"===typeof i&&o>i.length?i.length:o:(t<n.length-1?e.Debug.assert(o<n[t+1]):void 0!==i&&e.Debug.assert(o<=i.length),o)}function L(e){return e.lineMap||(e.lineMap=T(e.text))}function D(e,n){var t=A(e,n);return{line:t,character:n-e[t]}}function A(n,t,r){var i=e.binarySearch(n,t,e.identity,e.compareValues,r);return i<0&&(i=~i-1,e.Debug.assert(-1!==i,"position cannot precede the beginning of the file")),i}function C(e,n,t){if(n===t)return 0;var r=L(e),i=Math.min(n,t),a=i===t,o=a?n:t,s=A(r,i),c=A(r,o,s);return a?s-c:c-s}function I(e,n){return D(L(e),n)}function N(e){return k(e)||R(e)}function k(e){return 32===e||9===e||11===e||12===e||160===e||133===e||5760===e||e>=8192&&e<=8203||8239===e||8287===e||12288===e||65279===e}function R(e){return 10===e||13===e||8232===e||8233===e}function O(e){return e>=48&&e<=57}function w(e){return O(e)||e>=65&&e<=70||e>=97&&e<=102}function P(e){return e<=1114111}function M(e){return e>=48&&e<=55}function F(e,n){var t=e.charCodeAt(n);switch(t){case 13:case 10:case 9:case 11:case 12:case 32:case 47:case 60:case 124:case 61:case 62:return!0;case 35:return 0===n;default:return t>127}}function G(n,t,r,i,a){if(e.positionIsSynthesized(t))return t;var o=!1;while(1){var s=n.charCodeAt(t);switch(s){case 13:10===n.charCodeAt(t+1)&&t++;case 10:if(t++,r)return t;o=!!a;continue;case 9:case 11:case 12:case 32:t++;continue;case 47:if(i)break;if(47===n.charCodeAt(t+1)){t+=2;while(t<n.length){if(R(n.charCodeAt(t)))break;t++}o=!1;continue}if(42===n.charCodeAt(t+1)){t+=2;while(t<n.length){if(42===n.charCodeAt(t)&&47===n.charCodeAt(t+1)){t+=2;break}t++}o=!1;continue}break;case 60:case 124:case 61:case 62:if(U(n,t)){t=V(n,t),o=!1;continue}break;case 35:if(0===t&&K(n,t)){t=H(n,t),o=!1;continue}break;case 42:if(o){t++,o=!1;continue}break;default:if(s>127&&N(s)){t++;continue}break}return t}}e.tokenToString=b,e.stringToToken=E,e.computeLineStarts=T,e.getPositionOfLineAndCharacter=S,e.computePositionOfLineAndCharacter=x,e.getLineStarts=L,e.computeLineAndCharacterOfPosition=D,e.computeLineOfPosition=A,e.getLinesBetweenPositions=C,e.getLineAndCharacterOfPosition=I,e.isWhiteSpaceLike=N,e.isWhiteSpaceSingleLine=k,e.isLineBreak=R,e.isOctalDigit=M,e.couldStartTrivia=F,e.skipTrivia=G;var B="<<<<<<<".length;function U(n,t){if(e.Debug.assert(t>=0),0===t||R(n.charCodeAt(t-1))){var r=n.charCodeAt(t);if(t+B<n.length){for(var i=0;i<B;i++)if(n.charCodeAt(t+i)!==r)return!1;return 61===r||32===n.charCodeAt(t+B)}}return!1}function V(n,t,r){r&&r(e.Diagnostics.Merge_conflict_marker_encountered,t,B);var i=n.charCodeAt(t),a=n.length;if(60===i||62===i)while(t<a&&!R(n.charCodeAt(t)))t++;else{e.Debug.assert(124===i||61===i);while(t<a){var o=n.charCodeAt(t);if((61===o||62===o)&&o!==i&&U(n,t))break;t++}}return t}var j=/^#!.*/;function K(n,t){return e.Debug.assert(0===t),j.test(n)}function H(e,n){var t=j.exec(e)[0];return n+=t.length,n}function W(e,n,t,r,i,a,o){var s,c,l,u,d=!1,p=r,f=o;if(0===t){p=!0;var m=$(n);m&&(t=m.length)}e:while(t>=0&&t<n.length){var _=n.charCodeAt(t);switch(_){case 13:10===n.charCodeAt(t+1)&&t++;case 10:if(t++,r)break e;p=!0,d&&(u=!0);continue;case 9:case 11:case 12:case 32:t++;continue;case 47:var g=n.charCodeAt(t+1),y=!1;if(47===g||42===g){var h=47===g?2:3,v=t;if(t+=2,47===g)while(t<n.length){if(R(n.charCodeAt(t))){y=!0;break}t++}else while(t<n.length){if(42===n.charCodeAt(t)&&47===n.charCodeAt(t+1)){t+=2;break}t++}if(p){if(d&&(f=i(s,c,l,u,a,f),!e&&f))return f;s=v,c=t,l=h,u=y,d=!0}continue}break e;default:if(_>127&&N(_)){d&&R(_)&&(u=!0),t++;continue}break e}}return d&&(f=i(s,c,l,u,a,f)),f}function z(e,n,t,r){return W(!1,e,n,!1,t,r)}function J(e,n,t,r){return W(!1,e,n,!0,t,r)}function q(e,n,t,r,i){return W(!0,e,n,!1,t,r,i)}function X(e,n,t,r,i){return W(!0,e,n,!0,t,r,i)}function Y(e,n,t,r,i,a){return a||(a=[]),a.push({kind:t,pos:e,end:n,hasTrailingNewLine:r}),a}function Q(e,n){return q(e,n,Y,void 0,void 0)}function Z(e,n){return X(e,n,Y,void 0,void 0)}function $(e){var n=j.exec(e);if(n)return n[0]}function ee(e,n){return e>=65&&e<=90||e>=97&&e<=122||36===e||95===e||e>127&&g(e,n)}function ne(e,n,t){return e>=65&&e<=90||e>=97&&e<=122||e>=48&&e<=57||36===e||95===e||1===t&&(45===e||58===e)||e>127&&y(e,n)}function te(e,n,t){var r=ie(e,0);if(!ee(r,n))return!1;for(var i=ae(r);i<e.length;i+=ae(r))if(!ne(r=ie(e,i),n,t))return!1;return!0}function re(n,r,a,o,s,c,l){void 0===a&&(a=0);var u,d,p,_,g,y,h,v,b=o,E=0;Ie(b,c,l);var T={getStartPos:function(){return p},getTextPos:function(){return u},getToken:function(){return g},getTokenPos:function(){return _},getTokenText:function(){return b.substring(_,u)},getTokenValue:function(){return y},hasUnicodeEscape:function(){return 0!==(1024&h)},hasExtendedUnicodeEscape:function(){return 0!==(8&h)},hasPrecedingLineBreak:function(){return 0!==(1&h)},hasPrecedingJSDocComment:function(){return 0!==(2&h)},isIdentifier:function(){return 79===g||g>116},isReservedWord:function(){return g>=81&&g<=116},isUnterminated:function(){return 0!==(4&h)},getCommentDirectives:function(){return v},getNumericLiteralFlags:function(){return 1008&h},getTokenFlags:function(){return h},reScanGreaterToken:oe,reScanAsteriskEqualsToken:se,reScanSlashToken:le,reScanTemplateToken:pe,reScanTemplateHeadOrNoSubstitutionTemplate:fe,scanJsxIdentifier:ve,scanJsxAttributeValue:be,reScanJsxAttributeValue:Ee,reScanJsxToken:me,reScanLessThanToken:_e,reScanHashToken:ge,reScanQuestionToken:ye,reScanInvalidIdentifier:te,scanJsxToken:he,scanJsDocToken:Te,scan:$,getText:Ae,clearCommentDirectives:Ce,setText:Ie,setScriptTarget:ke,setLanguageVariant:Re,setOnError:Ne,setTextPos:Oe,setInJSDocType:we,tryScan:De,lookAhead:Le,scanRange:xe};return e.Debug.isDebugging&&Object.defineProperty(T,"__debugShowCurrentPositionInText",{get:function(){var e=T.getText();return e.slice(0,T.getStartPos())+"║"+e.slice(T.getStartPos())}}),T;function S(e,n,t){if(void 0===n&&(n=u),s){var r=u;u=n,s(e,t||0),u=r}}function x(){var n=u,t=!1,r=!1,i="";while(1){var a=b.charCodeAt(u);if(95!==a){if(!O(a))break;t=!0,r=!1,u++}else h|=512,t?(t=!1,r=!0,i+=b.substring(n,u)):S(r?e.Diagnostics.Multiple_consecutive_numeric_separators_are_not_permitted:e.Diagnostics.Numeric_separators_are_not_allowed_here,u,1),u++,n=u}return 95===b.charCodeAt(u-1)&&S(e.Diagnostics.Numeric_separators_are_not_allowed_here,u-1,1),i+b.substring(n,u)}function L(){var n,t,r=u,i=x();46===b.charCodeAt(u)&&(u++,n=x());var a,o=u;if(69===b.charCodeAt(u)||101===b.charCodeAt(u)){u++,h|=16,43!==b.charCodeAt(u)&&45!==b.charCodeAt(u)||u++;var s=u,c=x();c?(t=b.substring(o,s)+c,o=u):S(e.Diagnostics.Digit_expected)}if(512&h?(a=i,n&&(a+="."+n),t&&(a+=t)):a=b.substring(r,o),void 0!==n||16&h)return D(r,void 0===n&&!!(16&h)),{type:8,value:""+ +a};y=a;var l=Z();return D(r),{type:l,value:y}}function D(t,r){if(ee(ie(b,u),n)){var i=u,a=X().length;1===a&&"n"===b[i]?S(r?e.Diagnostics.A_bigint_literal_cannot_use_exponential_notation:e.Diagnostics.A_bigint_literal_must_be_an_integer,t,i-t+1):(S(e.Diagnostics.An_identifier_or_keyword_cannot_immediately_follow_a_numeric_literal,i,a),u=i)}}function A(){var e=u;while(M(b.charCodeAt(u)))u++;return+b.substring(e,u)}function C(e,n){var t=F(e,!1,n);return t?parseInt(t,16):-1}function I(e,n){return F(e,!0,n)}function F(n,t,r){var i=[],a=!1,o=!1;while(i.length<n||t){var s=b.charCodeAt(u);if(r&&95===s)h|=512,a?(a=!1,o=!0):S(o?e.Diagnostics.Multiple_consecutive_numeric_separators_are_not_permitted:e.Diagnostics.Numeric_separators_are_not_allowed_here,u,1),u++;else{if(a=r,s>=65&&s<=70)s+=32;else if(!(s>=48&&s<=57||s>=97&&s<=102))break;i.push(s),u++,o=!1}}return i.length<n&&(i=[]),95===b.charCodeAt(u-1)&&S(e.Diagnostics.Numeric_separators_are_not_allowed_here,u-1,1),String.fromCharCode.apply(String,i)}function G(n){void 0===n&&(n=!1);var t=b.charCodeAt(u);u++;var r="",i=u;while(1){if(u>=d){r+=b.substring(i,u),h|=4,S(e.Diagnostics.Unterminated_string_literal);break}var a=b.charCodeAt(u);if(a===t){r+=b.substring(i,u),u++;break}if(92!==a||n){if(R(a)&&!n){r+=b.substring(i,u),h|=4,S(e.Diagnostics.Unterminated_string_literal);break}u++}else r+=b.substring(i,u),r+=j(),i=u}return r}function B(n){var t=96===b.charCodeAt(u);u++;var r,i=u,a="";while(1){if(u>=d){a+=b.substring(i,u),h|=4,S(e.Diagnostics.Unterminated_template_literal),r=t?14:17;break}var o=b.charCodeAt(u);if(96===o){a+=b.substring(i,u),u++,r=t?14:17;break}if(36===o&&u+1<d&&123===b.charCodeAt(u+1)){a+=b.substring(i,u),u+=2,r=t?15:16;break}92!==o?13!==o?u++:(a+=b.substring(i,u),u++,u<d&&10===b.charCodeAt(u)&&u++,a+="\n",i=u):(a+=b.substring(i,u),a+=j(n),i=u)}return e.Debug.assert(void 0!==r),y=a,r}function j(n){var t=u;if(u++,u>=d)return S(e.Diagnostics.Unexpected_end_of_text),"";var r=b.charCodeAt(u);switch(u++,r){case 48:return n&&u<d&&O(b.charCodeAt(u))?(u++,h|=2048,b.substring(t,u)):"\0";case 98:return"\b";case 116:return"\t";case 110:return"\n";case 118:return"\v";case 102:return"\f";case 114:return"\r";case 39:return"'";case 34:return'"';case 117:if(n)for(var i=u;i<u+4;i++)if(i<d&&!w(b.charCodeAt(i))&&123!==b.charCodeAt(i))return u=i,h|=2048,b.substring(t,u);if(u<d&&123===b.charCodeAt(u)){if(u++,n&&!w(b.charCodeAt(u)))return h|=2048,b.substring(t,u);if(n){var a=u,o=I(1,!1),s=o?parseInt(o,16):-1;if(!P(s)||125!==b.charCodeAt(u))return h|=2048,b.substring(t,u);u=a}return h|=8,z()}return h|=1024,W(4);case 120:if(n){if(!w(b.charCodeAt(u)))return h|=2048,b.substring(t,u);if(!w(b.charCodeAt(u+1)))return u++,h|=2048,b.substring(t,u)}return W(2);case 13:u<d&&10===b.charCodeAt(u)&&u++;case 10:case 8232:case 8233:return"";default:return String.fromCharCode(r)}}function W(n){var t=C(n,!1);return t>=0?String.fromCharCode(t):(S(e.Diagnostics.Hexadecimal_digit_expected),"")}function z(){var n=I(1,!1),t=n?parseInt(n,16):-1,r=!1;return t<0?(S(e.Diagnostics.Hexadecimal_digit_expected),r=!0):t>1114111&&(S(e.Diagnostics.An_extended_Unicode_escape_value_must_be_between_0x0_and_0x10FFFF_inclusive),r=!0),u>=d?(S(e.Diagnostics.Unexpected_end_of_text),r=!0):125===b.charCodeAt(u)?u++:(S(e.Diagnostics.Unterminated_Unicode_escape_sequence),r=!0),r?"":ce(t)}function J(){if(u+5<d&&117===b.charCodeAt(u+1)){var e=u;u+=2;var n=C(4,!1);return u=e,n}return-1}function q(){if(n>=2&&117===ie(b,u+1)&&123===ie(b,u+2)){var e=u;u+=3;var t=I(1,!1),r=t?parseInt(t,16):-1;return u=e,r}return-1}function X(){var e="",t=u;while(u<d){var r=ie(b,u);if(ne(r,n))u+=ae(r);else{if(92!==r)break;if(r=q(),r>=0&&ne(r,n)){u+=3,h|=8,e+=z(),t=u;continue}if(r=J(),!(r>=0&&ne(r,n)))break;h|=1024,e+=b.substring(t,u),e+=ce(r),u+=6,t=u}}return e+=b.substring(t,u),e}function Y(){var e=y.length;if(e>=2&&e<=12){var n=y.charCodeAt(0);if(n>=97&&n<=122){var t=i.get(y);if(void 0!==t)return g=t}}return g=79}function Q(n){var t="",r=!1,i=!1;while(1){var a=b.charCodeAt(u);if(95!==a){if(r=!0,!O(a)||a-48>=n)break;t+=b[u],u++,i=!1}else h|=512,r?(r=!1,i=!0):S(i?e.Diagnostics.Multiple_consecutive_numeric_separators_are_not_permitted:e.Diagnostics.Numeric_separators_are_not_allowed_here,u,1),u++}return 95===b.charCodeAt(u-1)&&S(e.Diagnostics.Numeric_separators_are_not_allowed_here,u-1,1),t}function Z(){if(110===b.charCodeAt(u))return y+="n",384&h&&(y=e.parsePseudoBigInt(y)+"n"),u++,9;var n=128&h?parseInt(y.slice(2),2):256&h?parseInt(y.slice(2),8):+y;return y=""+n,8}function $(){var t;p=u,h=0;var i=!1;while(1){if(_=u,u>=d)return g=1;var o=ie(b,u);if(35===o&&0===u&&K(b,u)){if(u=H(b,u),r)continue;return g=6}switch(o){case 10:case 13:if(h|=1,r){u++;continue}return 13===o&&u+1<d&&10===b.charCodeAt(u+1)?u+=2:u++,g=4;case 9:case 11:case 12:case 32:case 160:case 5760:case 8192:case 8193:case 8194:case 8195:case 8196:case 8197:case 8198:case 8199:case 8200:case 8201:case 8202:case 8203:case 8239:case 8287:case 12288:case 65279:if(r){u++;continue}while(u<d&&k(b.charCodeAt(u)))u++;return g=5;case 33:return 61===b.charCodeAt(u+1)?61===b.charCodeAt(u+2)?(u+=3,g=37):(u+=2,g=35):(u++,g=53);case 34:case 39:return y=G(),g=10;case 96:return g=B(!1);case 37:return 61===b.charCodeAt(u+1)?(u+=2,g=69):(u++,g=44);case 38:return 38===b.charCodeAt(u+1)?61===b.charCodeAt(u+2)?(u+=3,g=76):(u+=2,g=55):61===b.charCodeAt(u+1)?(u+=2,g=73):(u++,g=50);case 40:return u++,g=20;case 41:return u++,g=21;case 42:if(61===b.charCodeAt(u+1))return u+=2,g=66;if(42===b.charCodeAt(u+1))return 61===b.charCodeAt(u+2)?(u+=3,g=67):(u+=2,g=42);if(u++,E&&!i&&1&h){i=!0;continue}return g=41;case 43:return 43===b.charCodeAt(u+1)?(u+=2,g=45):61===b.charCodeAt(u+1)?(u+=2,g=64):(u++,g=39);case 44:return u++,g=27;case 45:return 45===b.charCodeAt(u+1)?(u+=2,g=46):61===b.charCodeAt(u+1)?(u+=2,g=65):(u++,g=40);case 46:return O(b.charCodeAt(u+1))?(y=L().value,g=8):46===b.charCodeAt(u+1)&&46===b.charCodeAt(u+2)?(u+=3,g=25):(u++,g=24);case 47:if(47===b.charCodeAt(u+1)){u+=2;while(u<d){if(R(b.charCodeAt(u)))break;u++}if(v=ue(v,b.slice(_,u),f,_),r)continue;return g=2}if(42===b.charCodeAt(u+1)){u+=2,42===b.charCodeAt(u)&&47!==b.charCodeAt(u+1)&&(h|=2);var s=!1,c=_;while(u<d){var l=b.charCodeAt(u);if(42===l&&47===b.charCodeAt(u+1)){u+=2,s=!0;break}u++,R(l)&&(c=u,h|=1)}if(v=ue(v,b.slice(c,u),m,c),s||S(e.Diagnostics.Asterisk_Slash_expected),r)continue;return s||(h|=4),g=3}return 61===b.charCodeAt(u+1)?(u+=2,g=68):(u++,g=43);case 48:if(u+2<d&&(88===b.charCodeAt(u+1)||120===b.charCodeAt(u+1)))return u+=2,y=I(1,!0),y||(S(e.Diagnostics.Hexadecimal_digit_expected),y="0"),y="0x"+y,h|=64,g=Z();if(u+2<d&&(66===b.charCodeAt(u+1)||98===b.charCodeAt(u+1)))return u+=2,y=Q(2),y||(S(e.Diagnostics.Binary_digit_expected),y="0"),y="0b"+y,h|=128,g=Z();if(u+2<d&&(79===b.charCodeAt(u+1)||111===b.charCodeAt(u+1)))return u+=2,y=Q(8),y||(S(e.Diagnostics.Octal_digit_expected),y="0"),y="0o"+y,h|=256,g=Z();if(u+1<d&&M(b.charCodeAt(u+1)))return y=""+A(),h|=32,g=8;case 49:case 50:case 51:case 52:case 53:case 54:case 55:case 56:case 57:return t=L(),g=t.type,y=t.value,g;case 58:return u++,g=58;case 59:return u++,g=26;case 60:if(U(b,u)){if(u=V(b,u,S),r)continue;return g=7}return 60===b.charCodeAt(u+1)?61===b.charCodeAt(u+2)?(u+=3,g=70):(u+=2,g=47):61===b.charCodeAt(u+1)?(u+=2,g=32):1===a&&47===b.charCodeAt(u+1)&&42!==b.charCodeAt(u+2)?(u+=2,g=30):(u++,g=29);case 61:if(U(b,u)){if(u=V(b,u,S),r)continue;return g=7}return 61===b.charCodeAt(u+1)?61===b.charCodeAt(u+2)?(u+=3,g=36):(u+=2,g=34):62===b.charCodeAt(u+1)?(u+=2,g=38):(u++,g=63);case 62:if(U(b,u)){if(u=V(b,u,S),r)continue;return g=7}return u++,g=31;case 63:return 46!==b.charCodeAt(u+1)||O(b.charCodeAt(u+2))?63===b.charCodeAt(u+1)?61===b.charCodeAt(u+2)?(u+=3,g=77):(u+=2,g=60):(u++,g=57):(u+=2,g=28);case 91:return u++,g=22;case 93:return u++,g=23;case 94:return 61===b.charCodeAt(u+1)?(u+=2,g=78):(u++,g=52);case 123:return u++,g=18;case 124:if(U(b,u)){if(u=V(b,u,S),r)continue;return g=7}return 124===b.charCodeAt(u+1)?61===b.charCodeAt(u+2)?(u+=3,g=75):(u+=2,g=56):61===b.charCodeAt(u+1)?(u+=2,g=74):(u++,g=51);case 125:return u++,g=19;case 126:return u++,g=54;case 64:return u++,g=59;case 92:var T=q();if(T>=0&&ee(T,n))return u+=3,h|=8,y=z()+X(),g=Y();var x=J();return x>=0&&ee(x,n)?(u+=6,h|=1024,y=String.fromCharCode(x)+X(),g=Y()):(S(e.Diagnostics.Invalid_character),u++,g=0);case 35:return 0!==u&&"!"===b[u+1]?(S(e.Diagnostics.can_only_be_used_at_the_start_of_a_file),u++,g=0):(ee(ie(b,u+1),n)?(u++,re(ie(b,u),n)):(y=String.fromCharCode(ie(b,u)),S(e.Diagnostics.Invalid_character,u++,ae(o))),g=80);default:var D=re(o,n);if(D)return g=D;if(k(o)){u+=ae(o);continue}if(R(o)){h|=1,u+=ae(o);continue}var C=ae(o);return S(e.Diagnostics.Invalid_character,u,C),u+=C,g=0}}}function te(){e.Debug.assert(0===g,"'reScanInvalidIdentifier' should only be called when the current token is 'SyntaxKind.Unknown'."),u=_=p,h=0;var n=ie(b,u),t=re(n,99);return t?g=t:(u+=ae(n),g)}function re(e,n){var t=e;if(ee(t,n)){u+=ae(t);while(u<d&&ne(t=ie(b,u),n))u+=ae(t);return y=b.substring(_,u),92===t&&(y+=X()),Y()}}function oe(){if(31===g){if(62===b.charCodeAt(u))return 62===b.charCodeAt(u+1)?61===b.charCodeAt(u+2)?(u+=3,g=72):(u+=2,g=49):61===b.charCodeAt(u+1)?(u+=2,g=71):(u++,g=48);if(61===b.charCodeAt(u))return u++,g=33}return g}function se(){return e.Debug.assert(66===g,"'reScanAsteriskEqualsToken' should only be called on a '*='"),u=_+1,g=63}function le(){if(43===g||68===g){var t=_+1,r=!1,i=!1;while(1){if(t>=d){h|=4,S(e.Diagnostics.Unterminated_regular_expression_literal);break}var a=b.charCodeAt(t);if(R(a)){h|=4,S(e.Diagnostics.Unterminated_regular_expression_literal);break}if(r)r=!1;else{if(47===a&&!i){t++;break}91===a?i=!0:92===a?r=!0:93===a&&(i=!1)}t++}while(t<d&&ne(b.charCodeAt(t),n))t++;u=t,y=b.substring(_,u),g=13}return g}function ue(n,t,r,i){var a=de(e.trimStringStart(t),r);return void 0===a?n:e.append(n,{range:{pos:i,end:u},type:a})}function de(e,n){var t=n.exec(e);if(t)switch(t[1]){case"ts-expect-error":return 0;case"ts-ignore":return 1}}function pe(n){return e.Debug.assert(19===g,"'reScanTemplateToken' should only be called on a '}'"),u=_,g=B(n)}function fe(){return u=_,g=B(!0)}function me(e){return void 0===e&&(e=!0),u=_=p,g=he(e)}function _e(){return 47===g?(u=_+1,g=29):g}function ge(){return 80===g?(u=_+1,g=62):g}function ye(){return e.Debug.assert(60===g,"'reScanQuestionToken' should only be called on a '??'"),u=_+1,g=57}function he(n){if(void 0===n&&(n=!0),p=_=u,u>=d)return g=1;var t=b.charCodeAt(u);if(60===t)return 47===b.charCodeAt(u+1)?(u+=2,g=30):(u++,g=29);if(123===t)return u++,g=18;var r=0;while(u<d){if(t=b.charCodeAt(u),123===t)break;if(60===t){if(U(b,u))return u=V(b,u,S),g=7;break}if(62===t&&S(e.Diagnostics.Unexpected_token_Did_you_mean_or_gt,u,1),125===t&&S(e.Diagnostics.Unexpected_token_Did_you_mean_or_rbrace,u,1),R(t)&&0===r)r=-1;else{if(!n&&R(t)&&r>0)break;N(t)||(r=u)}u++}return y=b.substring(p,u),-1===r?12:11}function ve(){if(t(g)){var e=!1;while(u<d){var n=b.charCodeAt(u);if(45!==n)if(58!==n||e){var r=u;if(y+=X(),u===r)break}else y+=":",u++,e=!0,g=79;else y+="-",u++}":"===y.slice(-1)&&(y=y.slice(0,-1),u--)}return g}function be(){switch(p=u,b.charCodeAt(u)){case 34:case 39:return y=G(!0),g=10;default:return $()}}function Ee(){return u=_=p,be()}function Te(){if(p=_=u,h=0,u>=d)return g=1;var e=ie(b,u);switch(u+=ae(e),e){case 9:case 11:case 12:case 32:while(u<d&&k(b.charCodeAt(u)))u++;return g=5;case 64:return g=59;case 13:10===b.charCodeAt(u)&&u++;case 10:return h|=1,g=4;case 42:return g=41;case 123:return g=18;case 125:return g=19;case 91:return g=22;case 93:return g=23;case 60:return g=29;case 62:return g=31;case 61:return g=63;case 44:return g=27;case 46:return g=24;case 96:return g=61;case 35:return g=62;case 92:u--;var t=q();if(t>=0&&ee(t,n))return u+=3,h|=8,y=z()+X(),g=Y();var r=J();return r>=0&&ee(r,n)?(u+=6,h|=1024,y=String.fromCharCode(r)+X(),g=Y()):(u++,g=0)}if(ee(e,n)){var i=e;while(u<d&&ne(i=ie(b,u),n)||45===b.charCodeAt(u))u+=ae(i);return y=b.substring(_,u),92===i&&(y+=X()),g=Y()}return g=0}function Se(e,n){var t=u,r=p,i=_,a=g,o=y,s=h,c=e();return c&&!n||(u=t,p=r,_=i,g=a,y=o,h=s),c}function xe(e,n,t){var r=d,i=u,a=p,o=_,s=g,c=y,l=h,f=v;Ie(b,e,n);var m=t();return d=r,u=i,p=a,_=o,g=s,y=c,h=l,v=f,m}function Le(e){return Se(e,!0)}function De(e){return Se(e,!1)}function Ae(){return b}function Ce(){v=void 0}function Ie(e,n,t){b=e||"",d=void 0===t?b.length:n+t,Oe(n||0)}function Ne(e){s=e}function ke(e){n=e}function Re(e){a=e}function Oe(n){e.Debug.assert(n>=0),u=n,p=n,_=n,g=0,y=void 0,h=0}function we(e){E+=e?1:-1}}e.isShebangTrivia=K,e.scanShebangTrivia=H,e.forEachLeadingCommentRange=z,e.forEachTrailingCommentRange=J,e.reduceEachLeadingCommentRange=q,e.reduceEachTrailingCommentRange=X,e.getLeadingCommentRanges=Q,e.getTrailingCommentRanges=Z,e.getShebang=$,e.isIdentifierStart=ee,e.isIdentifierPart=ne,e.isIdentifierText=te,e.createScanner=re;var ie=String.prototype.codePointAt?function(e,n){return e.codePointAt(n)}:function(e,n){var t=e.length;if(!(n<0||n>=t)){var r=e.charCodeAt(n);if(r>=55296&&r<=56319&&t>n+1){var i=e.charCodeAt(n+1);if(i>=56320&&i<=57343)return 1024*(r-55296)+i-56320+65536}return r}};function ae(e){return e>=65536?2:1}function oe(n){if(e.Debug.assert(0<=n&&n<=1114111),n<=65535)return String.fromCharCode(n);var t=Math.floor((n-65536)/1024)+55296,r=(n-65536)%1024+56320;return String.fromCharCode(t,r)}var se=String.fromCodePoint?function(e){return String.fromCodePoint(e)}:oe;function ce(e){return se(e)}e.utf16EncodeAsString=ce}(d||(d={})),function(e){function n(n){return e.pathIsRelative(n)||e.isRootedDiskPath(n)}function t(n){return e.sortAndDeduplicate(n,e.compareDiagnostics)}function r(e){switch(e.target){case 99:return"lib.esnext.full.d.ts";case 8:return"lib.es2021.full.d.ts";case 7:return"lib.es2020.full.d.ts";case 6:return"lib.es2019.full.d.ts";case 5:return"lib.es2018.full.d.ts";case 4:return"lib.es2017.full.d.ts";case 3:return"lib.es2016.full.d.ts";case 2:return"lib.es6.d.ts";default:return"lib.d.ts"}}function i(e){return e.start+e.length}function a(e){return 0===e.length}function o(e,n){return n>=e.start&&n<i(e)}function s(e,n){return n>=e.pos&&n<=e.end}function c(e,n){return n.start>=e.start&&i(n)<=i(e)}function l(e,n){return void 0!==u(e,n)}function u(e,n){var t=_(e,n);return t&&0===t.length?void 0:t}function d(e,n){return f(e.start,e.length,n.start,n.length)}function p(e,n,t){return f(e.start,e.length,n,t)}function f(e,n,t,r){var i=e+n,a=t+r;return t<=i&&a>=e}function m(e,n){return n<=i(e)&&n>=e.start}function _(e,n){var t=Math.max(e.start,n.start),r=Math.min(i(e),i(n));return t<=r?y(t,r):void 0}function g(e,n){if(e<0)throw new Error("start < 0");if(n<0)throw new Error("length < 0");return{start:e,length:n}}function y(e,n){return g(e,n-e)}function h(e){return g(e.span.start,e.newLength)}function v(e){return a(e.span)&&0===e.newLength}function b(e,n){if(n<0)throw new Error("newLength < 0");return{span:e,newLength:n}}function E(n){if(0===n.length)return e.unchangedTextChangeRange;if(1===n.length)return n[0];for(var t=n[0],r=t.span.start,a=i(t.span),o=r+t.newLength,s=1;s<n.length;s++){var c=n[s],l=r,u=a,d=o,p=c.span.start,f=i(c.span),m=p+c.newLength;r=Math.min(l,p),a=Math.max(u,u+(f-d)),o=Math.max(m,m+(d-f))}return b(y(r,a),o-r)}function T(e){if(e&&161===e.kind)for(var n=e;n;n=n.parent)if(vn(n)||An(n)||256===n.kind)return n}function S(n,t){return e.hasSyntacticModifier(n,16476)&&169===t.kind}function x(n){return!!Pn(n)&&e.every(n.elements,L)}function L(n){return!!e.isOmittedExpression(n)||x(n.name)}function D(n){var t=n.parent;while(e.isBindingElement(t.parent))t=t.parent.parent;return t.parent}function A(n,t){e.isBindingElement(n)&&(n=D(n));var r=t(n);return 252===n.kind&&(n=n.parent),n&&253===n.kind&&(r|=t(n),n=n.parent),n&&235===n.kind&&(r|=t(n)),r}function C(n){return A(n,e.getEffectiveModifierFlags)}function I(n){return A(n,e.getEffectiveModifierFlagsAlwaysIncludeJSDoc)}function N(e){return A(e,function(e){return e.flags})}function k(n,t,r){var i=n.toLowerCase(),a=/^([a-z]+)([_\-]([a-z]+))?$/.exec(i);if(a){var o=a[1],s=a[3];e.contains(e.supportedLocaleDirectories,i)&&!c(o,s,r)&&c(o,void 0,r),e.setUILocale(n)}else r&&r.push(e.createCompilerDiagnostic(e.Diagnostics.Locale_must_be_of_the_form_language_or_language_territory_For_example_0_or_1,"en","ja-jp"));function c(n,r,i){var a=e.normalizePath(t.getExecutingFilePath()),o=e.getDirectoryPath(a),s=e.combinePaths(o,n);if(r&&(s=s+"-"+r),s=t.resolvePath(e.combinePaths(s,"diagnosticMessages.generated.json")),!t.fileExists(s))return!1;var c="";try{c=t.readFile(s)}catch(l){return i&&i.push(e.createCompilerDiagnostic(e.Diagnostics.Unable_to_open_file_0,s)),!1}try{e.setLocalizedDiagnosticMessages(JSON.parse(c))}catch(u){return i&&i.push(e.createCompilerDiagnostic(e.Diagnostics.Corrupted_locale_file_0,s)),!1}return!0}}function R(e,n){if(e)while(void 0!==e.original)e=e.original;return!n||n(e)?e:void 0}function O(e,n){while(e){var t=n(e);if("quit"===t)return;if(t)return e;e=e.parent}}function w(e){return 0===(8&e.flags)}function P(e,n){if(void 0===e||w(e))return e;e=e.original;while(e){if(w(e))return!n||n(e)?e:void 0;e=e.original}}function M(e){return e.length>=2&&95===e.charCodeAt(0)&&95===e.charCodeAt(1)?"_"+e:e}function F(e){var n=e;return n.length>=3&&95===n.charCodeAt(0)&&95===n.charCodeAt(1)&&95===n.charCodeAt(2)?n.substr(1):n}function G(e){return F(e.escapedText)}function B(e){return e.valueDeclaration&&un(e.valueDeclaration)?G(e.valueDeclaration.name):F(e.escapedName)}function U(n){var t=n.parent.parent;if(t){if(bt(t))return V(t);switch(t.kind){case 235:if(t.declarationList&&t.declarationList.declarations[0])return V(t.declarationList.declarations[0]);break;case 236:var r=t.expression;switch(219===r.kind&&63===r.operatorToken.kind&&(r=r.left),r.kind){case 204:return r.name;case 205:var i=r.argumentExpression;if(e.isIdentifier(i))return i}break;case 210:return V(t.expression);case 248:if(bt(t.statement)||$n(t.statement))return V(t.statement);break}}}function V(n){var t=z(n);return t&&e.isIdentifier(t)?t:void 0}function j(n,t){return!(!H(n)||!e.isIdentifier(n.name)||G(n.name)!==G(t))||!(!e.isVariableStatement(n)||!e.some(n.declarationList.declarations,function(e){return j(e,t)}))}function K(e){return e.name||U(e)}function H(e){return!!e.name}function W(n){switch(n.kind){case 79:return n;case 342:case 335:var t=n.name;if(159===t.kind)return t.right;break;case 206:case 219:var r=n;switch(e.getAssignmentDeclarationKind(r)){case 1:case 4:case 5:case 3:return e.getElementOrPropertyAccessArgumentExpressionOrName(r.left);case 7:case 8:case 9:return r.arguments[1];default:return}case 340:return K(n);case 334:return U(n);case 269:var i=n.expression;return e.isIdentifier(i)?i:void 0;case 205:var a=n;if(e.isBindableStaticElementAccessExpression(a))return a.argumentExpression}return n.name}function z(n){if(void 0!==n)return W(n)||(e.isFunctionExpression(n)||e.isArrowFunction(n)||e.isClassExpression(n)?J(n):void 0)}function J(n){if(n.parent){if(e.isPropertyAssignment(n.parent)||e.isBindingElement(n.parent))return n.parent.name;if(e.isBinaryExpression(n.parent)&&n===n.parent.right){if(e.isIdentifier(n.parent.left))return n.parent.left;if(e.isAccessExpression(n.parent.left))return e.getElementOrPropertyAccessArgumentExpressionOrName(n.parent.left)}else if(e.isVariableDeclaration(n.parent)&&e.isIdentifier(n.parent.name))return n.parent.name}}function q(n,t){if(n.name){if(e.isIdentifier(n.name)){var r=n.name.escapedText;return Te(n.parent,t).filter(function(n){return e.isJSDocParameterTag(n)&&e.isIdentifier(n.name)&&n.name.escapedText===r})}var i=n.parent.parameters.indexOf(n);e.Debug.assert(i>-1,"Parameters should always be in their parents' parameter list");var a=Te(n.parent,t).filter(e.isJSDocParameterTag);if(i<a.length)return[a[i]]}return e.emptyArray}function X(e){return q(e,!1)}function Y(e){return q(e,!0)}function Q(n,t){var r=n.name.escapedText;return Te(n.parent,t).filter(function(n){return e.isJSDocTemplateTag(n)&&n.typeParameters.some(function(e){return e.name.escapedText===r})})}function Z(e){return Q(e,!1)}function $(e){return Q(e,!0)}function ee(n){return!!Le(n,e.isJSDocParameterTag)}function ne(n){return Le(n,e.isJSDocAugmentsTag)}function te(n){return De(n,e.isJSDocImplementsTag)}function re(n){return Le(n,e.isJSDocClassTag)}function ie(n){return Le(n,e.isJSDocPublicTag)}function ae(n){return Le(n,e.isJSDocPublicTag,!0)}function oe(n){return Le(n,e.isJSDocPrivateTag)}function se(n){return Le(n,e.isJSDocPrivateTag,!0)}function ce(n){return Le(n,e.isJSDocProtectedTag)}function le(n){return Le(n,e.isJSDocProtectedTag,!0)}function ue(n){return Le(n,e.isJSDocReadonlyTag)}function de(n){return Le(n,e.isJSDocReadonlyTag,!0)}function pe(n){return Le(n,e.isJSDocOverrideTag,!0)}function fe(n){return Le(n,e.isJSDocDeprecatedTag)}function me(n){return Le(n,e.isJSDocDeprecatedTag,!0)}function _e(n){return Le(n,e.isJSDocEnumTag)}function ge(n){return Le(n,e.isJSDocThisTag)}function ye(n){return Le(n,e.isJSDocReturnTag)}function he(n){return Le(n,e.isJSDocTemplateTag)}function ve(n){var t=Le(n,e.isJSDocTypeTag);if(t&&t.typeExpression&&t.typeExpression.type)return t}function be(n){var t=Le(n,e.isJSDocTypeTag);return!t&&e.isParameter(n)&&(t=e.find(X(n),function(e){return!!e.typeExpression})),t&&t.typeExpression&&t.typeExpression.type}function Ee(n){var t=ye(n);if(t&&t.typeExpression)return t.typeExpression.type;var r=ve(n);if(r&&r.typeExpression){var i=r.typeExpression.type;if(e.isTypeLiteralNode(i)){var a=e.find(i.members,e.isCallSignatureDeclaration);return a&&a.type}if(e.isFunctionTypeNode(i)||e.isJSDocFunctionType(i))return i.type}}function Te(n,t){var r=n.jsDocCache;if(void 0===r||t){var i=e.getJSDocCommentsAndTags(n,t);e.Debug.assert(i.length<2||i[0]!==i[1]),r=e.flatMap(i,function(n){return e.isJSDoc(n)?n.tags:n}),t||(n.jsDocCache=r)}return r}function Se(e){return Te(e,!1)}function xe(e){return Te(e,!0)}function Le(n,t,r){return e.find(Te(n,r),t)}function De(e,n){return Se(e).filter(n)}function Ae(e,n){return Se(e).filter(function(e){return e.kind===n})}function Ce(n){return"string"===typeof n?n:null===n||void 0===n?void 0:n.map(function(n){return 316===n.kind?n.text:"{@link "+(n.name?e.entityNameToString(n.name)+" ":"")+n.text+"}"}).join("")}function Ie(n){if(e.isJSDocSignature(n))return e.emptyArray;if(e.isJSDocTypeAlias(n))return e.Debug.assert(315===n.parent.kind),e.flatMap(n.parent.tags,function(n){return e.isJSDocTemplateTag(n)?n.typeParameters:void 0});if(n.typeParameters)return n.typeParameters;if(e.isInJSFile(n)){var t=e.getJSDocTypeParameterDeclarations(n);if(t.length)return t;var r=be(n);if(r&&e.isFunctionTypeNode(r)&&r.typeParameters)return r.typeParameters}return e.emptyArray}function Ne(n){return n.constraint?n.constraint:e.isJSDocTemplateTag(n.parent)&&n===n.parent.typeParameters[0]?n.parent.constraint:void 0}function ke(e){return 79===e.kind||80===e.kind}function Re(e){return 171===e.kind||170===e.kind}function Oe(n){return e.isPropertyAccessExpression(n)&&!!(32&n.flags)}function we(n){return e.isElementAccessExpression(n)&&!!(32&n.flags)}function Pe(n){return e.isCallExpression(n)&&!!(32&n.flags)}function Me(e){var n=e.kind;return!!(32&e.flags)&&(204===n||205===n||206===n||228===n)}function Fe(n){return Me(n)&&!e.isNonNullExpression(n)&&!!n.questionDotToken}function Ge(e){return Fe(e.parent)&&e.parent.expression===e}function Be(e){return!Me(e.parent)||Fe(e.parent)||e!==e.parent.expression}function Ue(e){return 219===e.kind&&60===e.operatorToken.kind}function Ve(n){return e.isTypeReferenceNode(n)&&e.isIdentifier(n.typeName)&&"const"===n.typeName.escapedText&&!n.typeArguments}function je(n){return e.skipOuterExpressions(n,8)}function Ke(n){return e.isNonNullExpression(n)&&!!(32&n.flags)}function He(e){return 244===e.kind||243===e.kind}function We(e){return 272===e.kind||271===e.kind}function ze(e){switch(e.kind){case 297:case 298:return!0;default:return!1}}function Je(e){return ze(e)||295===e.kind||299===e.kind}function qe(e){return 342===e.kind||335===e.kind}function Xe(e){return Ye(e.kind)}function Ye(e){return e>=159}function Qe(e){return e>=0&&e<=158}function Ze(e){return Qe(e.kind)}function $e(e){return e.hasOwnProperty("pos")&&e.hasOwnProperty("end")}function en(e){return 8<=e&&e<=14}function nn(e){return en(e.kind)}function tn(e){return 14<=e&&e<=17}function rn(e){return tn(e.kind)}function an(e){var n=e.kind;return 16===n||17===n}function on(n){return e.isImportSpecifier(n)||e.isExportSpecifier(n)}function sn(e){switch(e.kind){case 268:case 273:return e.parent.parent.isTypeOnly;case 266:return e.parent.isTypeOnly;case 265:case 263:return e.isTypeOnly;default:return!1}}function cn(e){return 10===e.kind||tn(e.kind)}function ln(n){return e.isIdentifier(n)&&(7&n.autoGenerateFlags)>0}function un(n){return(e.isPropertyDeclaration(n)||In(n))&&e.isPrivateIdentifier(n.name)}function dn(n){return e.isPropertyAccessExpression(n)&&e.isPrivateIdentifier(n.name)}function pn(e){switch(e){case 126:case 130:case 85:case 134:case 88:case 93:case 123:case 121:case 122:case 143:case 124:case 157:return!0}return!1}function fn(n){return!!(16476&e.modifierToFlag(n))}function mn(e){return fn(e)||124===e||157===e}function _n(e){return pn(e.kind)}function gn(e){var n=e.kind;return 159===n||79===n}function yn(e){var n=e.kind;return 79===n||80===n||10===n||8===n||160===n}function hn(e){var n=e.kind;return 79===n||199===n||200===n}function vn(e){return!!e&&xn(e.kind)}function bn(n){return!!n&&(xn(n.kind)||e.isClassStaticBlockDeclaration(n))}function En(e){return e&&Sn(e.kind)}function Tn(e){return 110===e.kind||95===e.kind}function Sn(e){switch(e){case 254:case 167:case 169:case 170:case 171:case 211:case 212:return!0;default:return!1}}function xn(e){switch(e){case 166:case 172:case 318:case 173:case 174:case 177:case 312:case 178:return!0;default:return Sn(e)}}function Ln(n){return e.isSourceFile(n)||e.isModuleBlock(n)||e.isBlock(n)&&vn(n.parent)}function Dn(e){var n=e.kind;return 169===n||165===n||167===n||170===n||171===n||174===n||168===n||232===n}function An(e){return e&&(255===e.kind||224===e.kind)}function Cn(e){return e&&(170===e.kind||171===e.kind)}function In(e){switch(e.kind){case 167:case 170:case 171:return!0;default:return!1}}function Nn(e){var n=e.kind;return 173===n||172===n||164===n||166===n||174===n}function kn(e){return Nn(e)||Dn(e)}function Rn(e){var n=e.kind;return 291===n||292===n||293===n||167===n||170===n||171===n}function On(n){return e.isTypeNodeKind(n.kind)}function wn(e){switch(e.kind){case 177:case 178:return!0}return!1}function Pn(e){if(e){var n=e.kind;return 200===n||199===n}return!1}function Mn(e){var n=e.kind;return 202===n||203===n}function Fn(e){var n=e.kind;return 201===n||225===n}function Gn(e){switch(e.kind){case 252:case 162:case 201:return!0}return!1}function Bn(e){return Un(e)||jn(e)}function Un(e){switch(e.kind){case 199:case 203:return!0}return!1}function Vn(e){switch(e.kind){case 201:case 291:case 292:case 293:return!0}return!1}function jn(e){switch(e.kind){case 200:case 202:return!0}return!1}function Kn(e){var n=e.kind;return 204===n||159===n||198===n}function Hn(e){var n=e.kind;return 204===n||159===n}function Wn(e){switch(e.kind){case 278:case 277:case 206:case 207:case 208:case 163:return!0;default:return!1}}function zn(e){return 206===e.kind||207===e.kind}function Jn(e){var n=e.kind;return 221===n||14===n}function qn(e){return Xn(je(e).kind)}function Xn(e){switch(e){case 204:case 205:case 207:case 206:case 276:case 277:case 280:case 208:case 202:case 210:case 203:case 224:case 211:case 79:case 13:case 8:case 9:case 10:case 14:case 221:case 95:case 104:case 108:case 110:case 106:case 228:case 229:case 100:return!0;default:return!1}}function Yn(e){return Qn(je(e).kind)}function Qn(e){switch(e){case 217:case 218:case 213:case 214:case 215:case 216:case 209:return!0;default:return Xn(e)}}function Zn(e){switch(e.kind){case 218:return!0;case 217:return 45===e.operator||46===e.operator;default:return!1}}function $n(e){return et(je(e).kind)}function et(e){switch(e){case 220:case 222:case 212:case 219:case 223:case 227:case 225:case 346:case 345:return!0;default:return Qn(e)}}function nt(e){var n=e.kind;return 209===n||227===n}function tt(n){return e.isNotEmittedStatement(n)||e.isPartiallyEmittedExpression(n)}function rt(e,n){switch(e.kind){case 240:case 241:case 242:case 238:case 239:return!0;case 248:return n&&rt(e.statement,n)}return!1}function it(n){return e.isExportAssignment(n)||e.isExportDeclaration(n)}function at(n){return e.some(n,it)}function ot(n){return!e.isAnyImportOrReExport(n)&&!e.isExportAssignment(n)&&!e.hasSyntacticModifier(n,1)&&!e.isAmbientModule(n)}function st(n){return e.isAnyImportOrReExport(n)||e.isExportAssignment(n)||e.hasSyntacticModifier(n,1)}function ct(e){return 241===e.kind||242===e.kind}function lt(n){return e.isBlock(n)||$n(n)}function ut(n){return e.isBlock(n)}function dt(n){return e.isVariableDeclarationList(n)||$n(n)}function pt(e){var n=e.kind;return 260===n||259===n||79===n}function ft(e){var n=e.kind;return 260===n||259===n}function mt(e){var n=e.kind;return 79===n||259===n}function _t(e){var n=e.kind;return 267===n||266===n}function gt(e){return 259===e.kind||258===e.kind}function yt(e){return 212===e||201===e||255===e||224===e||168===e||169===e||258===e||294===e||273===e||254===e||211===e||170===e||265===e||263===e||268===e||256===e||283===e||167===e||166===e||259===e||262===e||266===e||272===e||162===e||291===e||165===e||164===e||171===e||292===e||257===e||161===e||252===e||340===e||333===e||342===e}function ht(e){return 254===e||274===e||255===e||256===e||257===e||258===e||259===e||264===e||263===e||270===e||269===e||262===e}function vt(e){return 244===e||243===e||251===e||238===e||236===e||234===e||241===e||242===e||240===e||237===e||248===e||245===e||247===e||249===e||250===e||235===e||239===e||246===e||344===e||348===e||347===e}function bt(n){return 161===n.kind?n.parent&&339!==n.parent.kind||e.isInJSFile(n):yt(n.kind)}function Et(e){return ht(e.kind)}function Tt(e){return vt(e.kind)}function St(e){var n=e.kind;return vt(n)||ht(n)||xt(e)}function xt(n){return 233===n.kind&&((void 0===n.parent||250!==n.parent.kind&&290!==n.parent.kind)&&!e.isFunctionBlock(n))}function Lt(e){var n=e.kind;return vt(n)||ht(n)||233===n}function Dt(e){var n=e.kind;return 275===n||159===n||79===n}function At(e){var n=e.kind;return 108===n||79===n||204===n}function Ct(e){var n=e.kind;return 276===n||286===n||277===n||11===n||280===n}function It(e){var n=e.kind;return 283===n||285===n}function Nt(e){var n=e.kind;return 10===n||286===n}function kt(e){var n=e.kind;return 278===n||277===n}function Rt(e){var n=e.kind;return 287===n||288===n}function Ot(e){return e.kind>=304&&e.kind<=342}function wt(n){return 315===n.kind||314===n.kind||316===n.kind||Jt(n)||Pt(n)||e.isJSDocTypeLiteral(n)||e.isJSDocSignature(n)}function Pt(e){return e.kind>=322&&e.kind<=342}function Mt(e){return 171===e.kind}function Ft(e){return 170===e.kind}function Gt(e){var n=e.jsDoc;return!!n&&n.length>0}function Bt(e){return!!e.type}function Ut(e){return!!e.initializer}function Vt(e){switch(e.kind){case 252:case 162:case 201:case 164:case 165:case 291:case 294:return!0;default:return!1}}function jt(e){return 283===e.kind||285===e.kind||Rn(e)}function Kt(e){return 176===e.kind||226===e.kind}e.isExternalModuleNameRelative=n,e.sortAndDeduplicateDiagnostics=t,e.getDefaultLibFileName=r,e.textSpanEnd=i,e.textSpanIsEmpty=a,e.textSpanContainsPosition=o,e.textRangeContainsPositionInclusive=s,e.textSpanContainsTextSpan=c,e.textSpanOverlapsWith=l,e.textSpanOverlap=u,e.textSpanIntersectsWithTextSpan=d,e.textSpanIntersectsWith=p,e.decodedTextSpanIntersectsWith=f,e.textSpanIntersectsWithPosition=m,e.textSpanIntersection=_,e.createTextSpan=g,e.createTextSpanFromBounds=y,e.textChangeRangeNewSpan=h,e.textChangeRangeIsUnchanged=v,e.createTextChangeRange=b,e.unchangedTextChangeRange=b(g(0,0),0),e.collapseTextChangeRangesAcrossMultipleVersions=E,e.getTypeParameterOwner=T,e.isParameterPropertyDeclaration=S,e.isEmptyBindingPattern=x,e.isEmptyBindingElement=L,e.walkUpBindingElementsAndPatterns=D,e.getCombinedModifierFlags=C,e.getCombinedNodeFlagsAlwaysIncludeJSDoc=I,e.getCombinedNodeFlags=N,e.supportedLocaleDirectories=["cs","de","es","fr","it","ja","ko","pl","pt-br","ru","tr","zh-cn","zh-tw"],e.validateLocaleAndSetLanguage=k,e.getOriginalNode=R,e.findAncestor=O,e.isParseTreeNode=w,e.getParseTreeNode=P,e.escapeLeadingUnderscores=M,e.unescapeLeadingUnderscores=F,e.idText=G,e.symbolName=B,e.nodeHasName=j,e.getNameOfJSDocTypedef=K,e.isNamedDeclaration=H,e.getNonAssignedNameOfDeclaration=W,e.getNameOfDeclaration=z,e.getAssignedName=J,e.getJSDocParameterTags=X,e.getJSDocParameterTagsNoCache=Y,e.getJSDocTypeParameterTags=Z,e.getJSDocTypeParameterTagsNoCache=$,e.hasJSDocParameterTags=ee,e.getJSDocAugmentsTag=ne,e.getJSDocImplementsTags=te,e.getJSDocClassTag=re,e.getJSDocPublicTag=ie,e.getJSDocPublicTagNoCache=ae,e.getJSDocPrivateTag=oe,e.getJSDocPrivateTagNoCache=se,e.getJSDocProtectedTag=ce,e.getJSDocProtectedTagNoCache=le,e.getJSDocReadonlyTag=ue,e.getJSDocReadonlyTagNoCache=de,e.getJSDocOverrideTagNoCache=pe,e.getJSDocDeprecatedTag=fe,e.getJSDocDeprecatedTagNoCache=me,e.getJSDocEnumTag=_e,e.getJSDocThisTag=ge,e.getJSDocReturnTag=ye,e.getJSDocTemplateTag=he,e.getJSDocTypeTag=ve,e.getJSDocType=be,e.getJSDocReturnType=Ee,e.getJSDocTags=Se,e.getJSDocTagsNoCache=xe,e.getAllJSDocTags=De,e.getAllJSDocTagsOfKind=Ae,e.getTextOfJSDocComment=Ce,e.getEffectiveTypeParameterDeclarations=Ie,e.getEffectiveConstraintOfTypeParameter=Ne,e.isMemberName=ke,e.isGetOrSetAccessorDeclaration=Re,e.isPropertyAccessChain=Oe,e.isElementAccessChain=we,e.isCallChain=Pe,e.isOptionalChain=Me,e.isOptionalChainRoot=Fe,e.isExpressionOfOptionalChainRoot=Ge,e.isOutermostOptionalChain=Be,e.isNullishCoalesce=Ue,e.isConstTypeReference=Ve,e.skipPartiallyEmittedExpressions=je,e.isNonNullChain=Ke,e.isBreakOrContinueStatement=He,e.isNamedExportBindings=We,e.isUnparsedTextLike=ze,e.isUnparsedNode=Je,e.isJSDocPropertyLikeTag=qe,e.isNode=Xe,e.isNodeKind=Ye,e.isTokenKind=Qe,e.isToken=Ze,e.isNodeArray=$e,e.isLiteralKind=en,e.isLiteralExpression=nn,e.isTemplateLiteralKind=tn,e.isTemplateLiteralToken=rn,e.isTemplateMiddleOrTemplateTail=an,e.isImportOrExportSpecifier=on,e.isTypeOnlyImportOrExportDeclaration=sn,e.isStringTextContainingNode=cn,e.isGeneratedIdentifier=ln,e.isPrivateIdentifierClassElementDeclaration=un,e.isPrivateIdentifierPropertyAccessExpression=dn,e.isModifierKind=pn,e.isParameterPropertyModifier=fn,e.isClassMemberModifier=mn,e.isModifier=_n,e.isEntityName=gn,e.isPropertyName=yn,e.isBindingName=hn,e.isFunctionLike=vn,e.isFunctionLikeOrClassStaticBlockDeclaration=bn,e.isFunctionLikeDeclaration=En,e.isBooleanLiteral=Tn,e.isFunctionLikeKind=xn,e.isFunctionOrModuleBlock=Ln,e.isClassElement=Dn,e.isClassLike=An,e.isAccessor=Cn,e.isMethodOrAccessor=In,e.isTypeElement=Nn,e.isClassOrTypeElement=kn,e.isObjectLiteralElementLike=Rn,e.isTypeNode=On,e.isFunctionOrConstructorTypeNode=wn,e.isBindingPattern=Pn,e.isAssignmentPattern=Mn,e.isArrayBindingElement=Fn,e.isDeclarationBindingElement=Gn,e.isBindingOrAssignmentPattern=Bn,e.isObjectBindingOrAssignmentPattern=Un,e.isObjectBindingOrAssignmentElement=Vn,e.isArrayBindingOrAssignmentPattern=jn,e.isPropertyAccessOrQualifiedNameOrImportTypeNode=Kn,e.isPropertyAccessOrQualifiedName=Hn,e.isCallLikeExpression=Wn,e.isCallOrNewExpression=zn,e.isTemplateLiteral=Jn,e.isLeftHandSideExpression=qn,e.isUnaryExpression=Yn,e.isUnaryExpressionWithWrite=Zn,e.isExpression=$n,e.isAssertionExpression=nt,e.isNotEmittedOrPartiallyEmittedNode=tt,e.isIterationStatement=rt,e.isScopeMarker=it,e.hasScopeMarker=at,e.needsScopeMarker=ot,e.isExternalModuleIndicator=st,e.isForInOrOfStatement=ct,e.isConciseBody=lt,e.isFunctionBody=ut,e.isForInitializer=dt,e.isModuleBody=pt,e.isNamespaceBody=ft,e.isJSDocNamespaceBody=mt,e.isNamedImportBindings=_t,e.isModuleOrEnumDeclaration=gt,e.isDeclaration=bt,e.isDeclarationStatement=Et,e.isStatementButNotDeclaration=Tt,e.isStatement=St,e.isStatementOrBlock=Lt,e.isModuleReference=Dt,e.isJsxTagNameExpression=At,e.isJsxChild=Ct,e.isJsxAttributeLike=It,e.isStringLiteralOrJsxExpression=Nt,e.isJsxOpeningLikeElement=kt,e.isCaseOrDefaultClause=Rt,e.isJSDocNode=Ot,e.isJSDocCommentContainingNode=wt,e.isJSDocTag=Pt,e.isSetAccessor=Mt,e.isGetAccessor=Ft,e.hasJSDocNodes=Gt,e.hasType=Bt,e.hasInitializer=Ut,e.hasOnlyExpressionInitializer=Vt,e.isObjectLiteralElement=jt,e.isTypeReferenceType=Kt;var Ht=1073741823;function Wt(n){for(var t=Ht,r=0,i=n;r<i.length;r++){var a=i[r];if(a.length){for(var o=0;o<a.length&&o<t;o++)if(!e.isWhiteSpaceLike(a.charCodeAt(o)))break;if(o<t&&(t=o),0===t)return 0}}return t===Ht?void 0:t}function zt(e){return 10===e.kind||14===e.kind}function Jt(e){return 319===e.kind||320===e.kind||321===e.kind}e.guessIndentation=Wt,e.isStringLiteralLike=zt,e.isJSDocLinkLike=Jt}(d||(d={})),function(e){function n(e,n){var t=e.declarations;if(t)for(var r=0,i=t;r<i.length;r++){var a=i[r];if(a.kind===n)return a}}function t(){return new e.Map}function r(e){return!!e&&!!e.size}function i(n){var t=new e.Map;if(n)for(var r=0,i=n;r<i.length;r++){var a=i[r];t.set(a.escapedName,a)}return t}function s(e){return 0!==(33554432&e.flags)}e.resolvingEmptyArray=[],e.externalHelpersModuleNameText="tslib",e.defaultMaximumTruncationLength=160,e.noTruncationMaximumTruncationLength=1e6,e.getDeclarationOfKind=n,e.createUnderscoreEscapedMap=t,e.hasEntries=r,e.createSymbolTable=i,e.isTransientSymbol=s;var c=l();function l(){var n="",t=function(e){return n+=e};return{getText:function(){return n},write:t,rawWrite:t,writeKeyword:t,writeOperator:t,writePunctuation:t,writeSpace:t,writeStringLiteral:t,writeLiteral:t,writeParameter:t,writeProperty:t,writeSymbol:function(e,n){return t(e)},writeTrailingSemicolon:t,writeComment:t,getTextPos:function(){return n.length},getLine:function(){return 0},getColumn:function(){return 0},getIndent:function(){return 0},isAtStartOfLine:function(){return!1},hasTrailingComment:function(){return!1},hasTrailingWhitespace:function(){return!!n.length&&e.isWhiteSpaceLike(n.charCodeAt(n.length-1))},writeLine:function(){return n+=" "},increaseIndent:e.noop,decreaseIndent:e.noop,clear:function(){return n=""},trackSymbol:function(){return!1},reportInaccessibleThisError:e.noop,reportInaccessibleUniqueSymbolError:e.noop,reportPrivateInBaseOfClassExpression:e.noop}}function u(e,n){return e.configFilePath!==n.configFilePath||d(e,n)}function d(n,t){return f(n,t,e.moduleResolutionOptionDeclarations)}function p(n,t){return f(n,t,e.optionsAffectingProgramStructure)}function f(e,n,t){return e!==n&&t.some(function(t){return!uu(Zc(e,t),Zc(n,t))})}function m(n,t){while(1){var r=t(n);if("quit"===r)return;if(void 0!==r)return r;if(e.isSourceFile(n))return;n=n.parent}}function _(e,n){for(var t=e.entries(),r=t.next();!r.done;r=t.next()){var i=r.value,a=i[0],o=i[1],s=n(o,a);if(s)return s}}function g(e,n){for(var t=e.keys(),r=t.next();!r.done;r=t.next()){var i=n(r.value);if(i)return i}}function y(e,n){e.forEach(function(e,t){n.set(t,e)})}function h(e){var n=c.getText();try{return e(c),c.getText()}finally{c.clear(),c.writeKeyword(n)}}function v(e){return e.end-e.pos}function b(e,n){return e&&e.resolvedModules&&e.resolvedModules.get(n)}function E(n,t,r){n.resolvedModules||(n.resolvedModules=new e.Map),n.resolvedModules.set(t,r)}function T(n,t,r){n.resolvedTypeReferenceDirectiveNames||(n.resolvedTypeReferenceDirectiveNames=new e.Map),n.resolvedTypeReferenceDirectiveNames.set(t,r)}function S(e,n){return e.path===n.path&&!e.prepend===!n.prepend&&!e.circular===!n.circular}function x(e,n){return e.isExternalLibraryImport===n.isExternalLibraryImport&&e.extension===n.extension&&e.resolvedFileName===n.resolvedFileName&&e.originalPath===n.originalPath&&L(e.packageId,n.packageId)}function L(e,n){return e===n||!!e&&!!n&&e.name===n.name&&e.subModuleName===n.subModuleName&&e.version===n.version}function D(e){var n=e.name,t=e.subModuleName,r=e.version,i=t?n+"/"+t:n;return i+"@"+r}function A(e,n){return e.resolvedFileName===n.resolvedFileName&&e.primary===n.primary&&e.originalPath===n.originalPath}function C(n,t,r,i){e.Debug.assert(n.length===t.length);for(var a=0;a<n.length;a++){var o=t[a],s=r&&r.get(n[a]),c=s?!o||!i(s,o):o;if(c)return!0}return!1}function I(e){return N(e),0!==(262144&e.flags)}function N(n){if(!(524288&n.flags)){var t=0!==(65536&n.flags)||e.forEachChild(n,I);t&&(n.flags|=262144),n.flags|=524288}}function k(e){while(e&&300!==e.kind)e=e.parent;return e}function R(e){return k(e.valueDeclaration||Se(e))}function O(e){switch(e.kind){case 233:case 261:case 240:case 241:case 242:return!0}return!1}function w(n,t){return e.Debug.assert(n>=0),e.getLineStarts(t)[n]}function P(n){var t=k(n),r=e.getLineAndCharacterOfPosition(t,n.pos);return t.fileName+"("+(r.line+1)+","+(r.character+1)+")"}function M(n,t){e.Debug.assert(n>=0);var r=e.getLineStarts(t),i=n,a=t.text;if(i+1===r.length)return a.length-1;var o=r[i],s=r[i+1]-1;e.Debug.assert(e.isLineBreak(a.charCodeAt(s)));while(o<=s&&e.isLineBreak(a.charCodeAt(s)))s--;return s}function F(e,n,t){return!(t&&t(n))&&!e.identifiers.has(n)}function G(e){return void 0===e||e.pos===e.end&&e.pos>=0&&1!==e.kind}function B(e){return!G(e)}function U(e,n,t){if(void 0===n||0===n.length)return e;for(var r=0;r<e.length;++r)if(!t(e[r]))break;return e.splice.apply(e,a([r,0],n,!1)),e}function V(e,n,t){if(void 0===n)return e;for(var r=0;r<e.length;++r)if(!t(e[r]))break;return e.splice(r,0,n),e}function j(e){return cn(e)||!!(1048576&ae(e))}function K(e,n){return U(e,n,cn)}function H(e,n){return U(e,n,j)}function W(e,n){return V(e,n,cn)}function z(e,n){return V(e,n,j)}function J(n,t,r){if(47===n.charCodeAt(t+1)&&t+2<r&&47===n.charCodeAt(t+2)){var i=n.substring(t,r);return!!(e.fullTripleSlashReferencePathRegEx.test(i)||e.fullTripleSlashAMDReferencePathRegEx.test(i)||_n.test(i)||gn.test(i))}return!1}function q(e,n){return 42===e.charCodeAt(n+1)&&33===e.charCodeAt(n+2)}function X(n,t){var r=new e.Map(t.map(function(t){return[""+e.getLineAndCharacterOfPosition(n,t.range.end).line,t]})),i=new e.Map;return{getUnusedExpectations:a,markUsed:o};function a(){return e.arrayFrom(r.entries()).filter(function(e){var n=e[0],t=e[1];return 0===t.type&&!i.get(n)}).map(function(e){e[0];var n=e[1];return n})}function o(e){return!!r.has(""+e)&&(i.set(""+e,!0),!0)}}function Y(n,t,r){return G(n)?n.pos:e.isJSDocNode(n)||11===n.kind?e.skipTrivia((t||k(n)).text,n.pos,!1,!0):r&&e.hasJSDocNodes(n)?Y(n.jsDoc[0],t):343===n.kind&&n._children.length>0?Y(n._children[0],t,r):e.skipTrivia((t||k(n)).text,n.pos,!1,!1,Et(n))}function Q(n,t){return G(n)||!n.decorators?Y(n,t):e.skipTrivia((t||k(n)).text,n.decorators.end)}function Z(e,n,t){return void 0===t&&(t=!1),ne(e.text,n,t)}function $(n){return!!e.findAncestor(n,e.isJSDocTypeExpression)}function ee(n){return!!(e.isExportDeclaration(n)&&n.exportClause&&e.isNamespaceExport(n.exportClause)&&"default"===n.exportClause.name.escapedText)}function ne(n,t,r){if(void 0===r&&(r=!1),G(t))return"";var i=n.substring(r?t.pos:e.skipTrivia(n,t.pos),t.end);return $(t)&&(i=i.split(/\r\n|\n|\r/).map(function(n){return e.trimStringStart(n.replace(/^\s*\*/,""))}).join("\n")),i}function te(e,n){return void 0===n&&(n=!1),Z(k(e),e,n)}function re(e){return e.pos}function ie(n,t){return e.binarySearch(n,t,re,e.compareValues)}function ae(e){var n=e.emitNode;return n&&n.flags||0}function oe(){return{es2015:{Array:["find","findIndex","fill","copyWithin","entries","keys","values"],RegExp:["flags","sticky","unicode"],Reflect:["apply","construct","defineProperty","deleteProperty","get"," getOwnPropertyDescriptor","getPrototypeOf","has","isExtensible","ownKeys","preventExtensions","set","setPrototypeOf"],ArrayConstructor:["from","of"],ObjectConstructor:["assign","getOwnPropertySymbols","keys","is","setPrototypeOf"],NumberConstructor:["isFinite","isInteger","isNaN","isSafeInteger","parseFloat","parseInt"],Math:["clz32","imul","sign","log10","log2","log1p","expm1","cosh","sinh","tanh","acosh","asinh","atanh","hypot","trunc","fround","cbrt"],Map:["entries","keys","values"],Set:["entries","keys","values"],Promise:e.emptyArray,PromiseConstructor:["all","race","reject","resolve"],Symbol:["for","keyFor"],WeakMap:["entries","keys","values"],WeakSet:["entries","keys","values"],Iterator:e.emptyArray,AsyncIterator:e.emptyArray,String:["codePointAt","includes","endsWith","normalize","repeat","startsWith","anchor","big","blink","bold","fixed","fontcolor","fontsize","italics","link","small","strike","sub","sup"],StringConstructor:["fromCodePoint","raw"]},es2016:{Array:["includes"]},es2017:{Atomics:e.emptyArray,SharedArrayBuffer:e.emptyArray,String:["padStart","padEnd"],ObjectConstructor:["values","entries","getOwnPropertyDescriptors"],DateTimeFormat:["formatToParts"]},es2018:{Promise:["finally"],RegExpMatchArray:["groups"],RegExpExecArray:["groups"],RegExp:["dotAll"],Intl:["PluralRules"],AsyncIterable:e.emptyArray,AsyncIterableIterator:e.emptyArray,AsyncGenerator:e.emptyArray,AsyncGeneratorFunction:e.emptyArray},es2019:{Array:["flat","flatMap"],ObjectConstructor:["fromEntries"],String:["trimStart","trimEnd","trimLeft","trimRight"],Symbol:["description"]},es2020:{BigInt:e.emptyArray,BigInt64Array:e.emptyArray,BigUint64Array:e.emptyArray,PromiseConstructor:["allSettled"],SymbolConstructor:["matchAll"],String:["matchAll"],DataView:["setBigInt64","setBigUint64","getBigInt64","getBigUint64"],RelativeTimeFormat:["format","formatToParts","resolvedOptions"]},es2021:{PromiseConstructor:["any"],String:["replaceAll"]},esnext:{NumberFormat:["formatToParts"]}}}function se(n,t,r){var i;if(ce(n,r))return Z(t,n);switch(n.kind){case 10:var a=2&r?sa:1&r||16777216&ae(n)?$i:na;return n.singleQuote?"'"+a(n.text,39)+"'":'"'+a(n.text,34)+'"';case 14:case 15:case 16:case 17:a=1&r||16777216&ae(n)?$i:na;var o=null!==(i=n.rawText)&&void 0!==i?i:Wi(a(n.text,96));switch(n.kind){case 14:return"`"+o+"`";case 15:return"`"+o+"${";case 16:return"}"+o+"${";case 17:return"}"+o+"`"}break;case 8:case 9:return n.text;case 13:return 4&r&&n.isUnterminated?n.text+(92===n.text.charCodeAt(n.text.length-1)?" /":"/"):n.text}return e.Debug.fail("Literal kind '"+n.kind+"' not accounted for.")}function ce(n,t){return!(wi(n)||!n.parent||4&t&&n.isUnterminated)&&(e.isNumericLiteral(n)&&512&n.numericLiteralFlags?!!(8&t):!e.isBigIntLiteral(n))}function le(n){return e.isString(n)?'"'+na(n)+'"':""+n}function ue(n){return e.getBaseFileName(n).replace(/^(\d)/,"_$1").replace(/\W/g,"_")}function de(n){return 0!==(3&e.getCombinedNodeFlags(n))||pe(n)}function pe(e){var n=Ri(e);return 252===n.kind&&290===n.parent.kind}function fe(n){return e.isModuleDeclaration(n)&&(10===n.name.kind||be(n))}function me(n){return e.isModuleDeclaration(n)&&10===n.name.kind}function _e(n){return e.isModuleDeclaration(n)&&e.isStringLiteral(n.name)}function ge(n){return e.isModuleDeclaration(n)||e.isIdentifier(n)}function ye(e){return he(e.valueDeclaration)}function he(e){return!!e&&259===e.kind&&!e.body}function ve(n){return 300===n.kind||259===n.kind||e.isFunctionLikeOrClassStaticBlockDeclaration(n)}function be(e){return!!(1024&e.flags)}function Ee(e){return fe(e)&&Te(e)}function Te(n){switch(n.parent.kind){case 300:return e.isExternalModule(n.parent);case 260:return fe(n.parent.parent)&&e.isSourceFile(n.parent.parent.parent)&&!e.isExternalModule(n.parent.parent.parent)}return!1}function Se(n){var t;return null===(t=n.declarations)||void 0===t?void 0:t.find(function(n){return!Ee(n)&&!(e.isModuleDeclaration(n)&&be(n))})}function xe(n,t){return e.isExternalModule(n)||t.isolatedModules||Fc(t)===e.ModuleKind.CommonJS&&!!n.commonJsModuleIndicator}function Le(n,t){switch(n.scriptKind){case 1:case 3:case 2:case 4:break;default:return!1}return!n.isDeclarationFile&&(!!Jc(t,"alwaysStrict")||(!!e.startsWithUseStrict(n.statements)||!(!e.isExternalModule(n)&&!t.isolatedModules)&&(Fc(t)>=e.ModuleKind.ES2015||!t.noImplicitUseStrict)))}function De(n,t){switch(n.kind){case 300:case 261:case 290:case 259:case 240:case 241:case 242:case 169:case 167:case 170:case 171:case 254:case 211:case 212:case 165:case 168:return!0;case 233:return!e.isFunctionLikeOrClassStaticBlockDeclaration(t)}return!1}function Ae(n){switch(n.kind){case 333:case 340:case 318:return!0;default:return e.assertType(n),Ce(n)}}function Ce(n){switch(n.kind){case 172:case 173:case 166:case 174:case 177:case 178:case 312:case 255:case 224:case 256:case 257:case 339:case 254:case 167:case 169:case 170:case 171:case 211:case 212:return!0;default:return e.assertType(n),!1}}function Ie(e){switch(e.kind){case 264:case 263:return!0;default:return!1}}function Ne(e){switch(e.kind){case 264:case 263:case 235:case 255:case 254:case 259:case 257:case 256:case 258:return!0;default:return!1}}function ke(n){return Re(n)||e.isModuleDeclaration(n)||e.isImportTypeNode(n)||an(n)}function Re(n){return Ie(n)||e.isExportDeclaration(n)}function Oe(n){return e.findAncestor(n.parent,function(e){return De(e,e.parent)})}function we(e,n){var t=Oe(e);while(t)n(t),t=Oe(t)}function Pe(e){return e&&0!==v(e)?te(e):"(Missing)"}function Me(e){return e.declaration?Pe(e.declaration.parameters[0].name):void 0}function Fe(e){return 160===e.kind&&!yi(e.expression)}function Ge(n){switch(n.kind){case 79:case 80:return n.escapedText;case 10:case 8:case 14:return e.escapeLeadingUnderscores(n.text);case 160:return yi(n.expression)?e.escapeLeadingUnderscores(n.expression.text):e.Debug.fail("Text of property name cannot be read from non-literal-valued ComputedPropertyNames");default:return e.Debug.assertNever(n)}}function Be(n){switch(n.kind){case 108:return"this";case 80:case 79:return 0===v(n)?e.idText(n):te(n);case 159:return Be(n.left)+"."+Be(n.right);case 204:return e.isIdentifier(n.name)||e.isPrivateIdentifier(n.name)?Be(n.expression)+"."+Be(n.name):e.Debug.assertNever(n.name);case 306:return Be(n.left)+Be(n.right);default:return e.Debug.assertNever(n)}}function Ue(e,n,t,r,i,a){var o=k(e);return je(o,e,n,t,r,i,a)}function Ve(n,t,r,i,a,o,s){var c=e.skipTrivia(n.text,t.pos);return xc(n,c,t.end-c,r,i,a,o,s)}function je(e,n,t,r,i,a,o){var s=Ye(e,n);return xc(e,s.start,s.length,t,r,i,a,o)}function Ke(e,n,t){var r=k(e),i=Ye(r,e);return We(r,i.start,i.length,n,t)}function He(n,t,r){e.Debug.assertGreaterThanOrEqual(t,0),e.Debug.assertGreaterThanOrEqual(r,0),n&&(e.Debug.assertLessThanOrEqual(t,n.text.length),e.Debug.assertLessThanOrEqual(t+r,n.text.length))}function We(e,n,t,r,i){return He(e,n,t),{file:e,start:n,length:t,code:r.code,category:r.category,messageText:r.next?r:r.messageText,relatedInformation:i}}function ze(e,n,t){return{file:e,start:0,length:0,code:n.code,category:n.category,messageText:n.next?n:n.messageText,relatedInformation:t}}function Je(e,n,t){return{file:e,start:n.pos,length:n.end-n.pos,code:t.code,category:t.category,messageText:t.message}}function qe(n,t){var r=e.createScanner(n.languageVersion,!0,n.languageVariant,n.text,void 0,t);r.scan();var i=r.getTokenPos();return e.createTextSpanFromBounds(i,r.getTextPos())}function Xe(n,t){var r=e.skipTrivia(n.text,t.pos);if(t.body&&233===t.body.kind){var i=e.getLineAndCharacterOfPosition(n,t.body.pos).line,a=e.getLineAndCharacterOfPosition(n,t.body.end).line;if(i<a)return e.createTextSpan(r,M(i,n)-r+1)}return e.createTextSpanFromBounds(r,t.end)}function Ye(n,t){var r=t;switch(t.kind){case 300:var i=e.skipTrivia(n.text,0,!1);return i===n.text.length?e.createTextSpan(0,0):qe(n,i);case 252:case 201:case 255:case 224:case 256:case 259:case 258:case 294:case 254:case 211:case 167:case 170:case 171:case 257:case 165:case 164:case 266:r=t.name;break;case 212:return Xe(n,t);case 287:case 288:var a=e.skipTrivia(n.text,t.pos),o=t.statements.length>0?t.statements[0].pos:t.end;return e.createTextSpanFromBounds(a,o)}if(void 0===r)return qe(n,t.pos);e.Debug.assert(!e.isJSDoc(r));var s=G(r),c=s||e.isJsxText(t)?r.pos:e.skipTrivia(n.text,r.pos);return s?(e.Debug.assert(c===r.pos,"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809"),e.Debug.assert(c===r.end,"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809")):(e.Debug.assert(c>=r.pos,"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809"),e.Debug.assert(c<=r.end,"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809")),e.createTextSpanFromBounds(c,r.end)}function Qe(e){return void 0!==(e.externalModuleIndicator||e.commonJsModuleIndicator)}function Ze(e){return 6===e.scriptKind}function $e(n){return!!(2048&e.getCombinedModifierFlags(n))}function en(n){return!(!(64&e.getCombinedModifierFlags(n))||e.isParameterPropertyDeclaration(n,n.parent))}function nn(n){return!!(2&e.getCombinedNodeFlags(n))}function tn(n){return!!(1&e.getCombinedNodeFlags(n))}function rn(e){return 206===e.kind&&106===e.expression.kind}function an(e){return 206===e.kind&&100===e.expression.kind}function on(n){return e.isMetaProperty(n)&&100===n.keywordToken&&"meta"===n.name.escapedText}function sn(n){return e.isImportTypeNode(n)&&e.isLiteralTypeNode(n.argument)&&e.isStringLiteral(n.argument.literal)}function cn(e){return 236===e.kind&&10===e.expression.kind}function ln(e){return!!(1048576&ae(e))}function un(n){return ln(n)&&e.isFunctionDeclaration(n)}function dn(n){return e.isIdentifier(n.name)&&!n.initializer}function pn(n){return ln(n)&&e.isVariableStatement(n)&&e.every(n.declarationList.declarations,dn)}function fn(n,t){return 11!==n.kind?e.getLeadingCommentRanges(t.text,n.pos):void 0}function mn(n,t){var r=162===n.kind||161===n.kind||211===n.kind||212===n.kind||210===n.kind||252===n.kind?e.concatenate(e.getTrailingCommentRanges(t,n.pos),e.getLeadingCommentRanges(t,n.pos)):e.getLeadingCommentRanges(t,n.pos);return e.filter(r,function(e){return 42===t.charCodeAt(e.pos+1)&&42===t.charCodeAt(e.pos+2)&&47!==t.charCodeAt(e.pos+3)})}e.changesAffectModuleResolution=u,e.optionsHaveModuleResolutionChanges=d,e.changesAffectingProgramStructure=p,e.optionsHaveChanges=f,e.forEachAncestor=m,e.forEachEntry=_,e.forEachKey=g,e.copyEntries=y,e.usingSingleLineStringWriter=h,e.getFullWidth=v,e.getResolvedModule=b,e.setResolvedModule=E,e.setResolvedTypeReferenceDirective=T,e.projectReferenceIsEqualTo=S,e.moduleResolutionIsEqualTo=x,e.packageIdToString=D,e.typeDirectiveIsEqualTo=A,e.hasChangesInResolutions=C,e.containsParseError=I,e.getSourceFileOfNode=k,e.getSourceFileOfModule=R,e.isStatementWithLocals=O,e.getStartPositionOfLine=w,e.nodePosToString=P,e.getEndLinePosition=M,e.isFileLevelUniqueName=F,e.nodeIsMissing=G,e.nodeIsPresent=B,e.insertStatementsAfterStandardPrologue=K,e.insertStatementsAfterCustomPrologue=H,e.insertStatementAfterStandardPrologue=W,e.insertStatementAfterCustomPrologue=z,e.isRecognizedTripleSlashComment=J,e.isPinnedComment=q,e.createCommentDirectivesMap=X,e.getTokenPosOfNode=Y,e.getNonDecoratorTokenPosOfNode=Q,e.getSourceTextOfNodeFromSourceFile=Z,e.isExportNamespaceAsDefaultDeclaration=ee,e.getTextOfNodeFromSourceText=ne,e.getTextOfNode=te,e.indexOfNode=ie,e.getEmitFlags=ae,e.getScriptTargetFeatures=oe,function(e){e[e["None"]=0]="None",e[e["NeverAsciiEscape"]=1]="NeverAsciiEscape",e[e["JsxAttributeEscape"]=2]="JsxAttributeEscape",e[e["TerminateUnterminatedLiterals"]=4]="TerminateUnterminatedLiterals",e[e["AllowNumericSeparator"]=8]="AllowNumericSeparator"}(e.GetLiteralTextFlags||(e.GetLiteralTextFlags={})),e.getLiteralText=se,e.getTextOfConstantValue=le,e.makeIdentifierFromModuleName=ue,e.isBlockOrCatchScoped=de,e.isCatchClauseVariableDeclarationOrBindingElement=pe,e.isAmbientModule=fe,e.isModuleWithStringLiteralName=me,e.isNonGlobalAmbientModule=_e,e.isEffectiveModuleDeclaration=ge,e.isShorthandAmbientModuleSymbol=ye,e.isBlockScopedContainerTopLevel=ve,e.isGlobalScopeAugmentation=be,e.isExternalModuleAugmentation=Ee,e.isModuleAugmentationExternal=Te,e.getNonAugmentationDeclaration=Se,e.isEffectiveExternalModule=xe,e.isEffectiveStrictModeSourceFile=Le,e.isBlockScope=De,e.isDeclarationWithTypeParameters=Ae,e.isDeclarationWithTypeParameterChildren=Ce,e.isAnyImportSyntax=Ie,e.isLateVisibilityPaintedStatement=Ne,e.hasPossibleExternalModuleReference=ke,e.isAnyImportOrReExport=Re,e.getEnclosingBlockScopeContainer=Oe,e.forEachEnclosingBlockScopeContainer=we,e.declarationNameToString=Pe,e.getNameFromIndexInfo=Me,e.isComputedNonLiteralName=Fe,e.getTextOfPropertyName=Ge,e.entityNameToString=Be,e.createDiagnosticForNode=Ue,e.createDiagnosticForNodeArray=Ve,e.createDiagnosticForNodeInSourceFile=je,e.createDiagnosticForNodeFromMessageChain=Ke,e.createFileDiagnosticFromMessageChain=We,e.createDiagnosticForFileFromMessageChain=ze,e.createDiagnosticForRange=Je,e.getSpanOfTokenAtPosition=qe,e.getErrorSpanForNode=Ye,e.isExternalOrCommonJsModule=Qe,e.isJsonSourceFile=Ze,e.isEnumConst=$e,e.isDeclarationReadonly=en,e.isVarConst=nn,e.isLet=tn,e.isSuperCall=rn,e.isImportCall=an,e.isImportMeta=on,e.isLiteralImportTypeNode=sn,e.isPrologueDirective=cn,e.isCustomPrologue=ln,e.isHoistedFunction=un,e.isHoistedVariableStatement=pn,e.getLeadingCommentRangesOfNode=fn,e.getJSDocCommentRanges=mn,e.fullTripleSlashReferencePathRegEx=/^(\/\/\/\s*<reference\s+path\s*=\s*)(('[^']*')|("[^"]*")).*?\/>/;var _n=/^(\/\/\/\s*<reference\s+types\s*=\s*)(('[^']*')|("[^"]*")).*?\/>/;e.fullTripleSlashAMDReferencePathRegEx=/^(\/\/\/\s*<amd-dependency\s+path\s*=\s*)(('[^']*')|("[^"]*")).*?\/>/;var gn=/^(\/\/\/\s*<reference\s+no-default-lib\s*=\s*)(('[^']*')|("[^"]*"))\s*\/>/;function yn(n){if(175<=n.kind&&n.kind<=198)return!0;switch(n.kind){case 129:case 153:case 145:case 156:case 148:case 132:case 149:case 146:case 151:case 142:return!0;case 114:return 215!==n.parent.kind;case 226:return!Fo(n);case 161:return 193===n.parent.kind||188===n.parent.kind;case 79:159===n.parent.kind&&n.parent.right===n?n=n.parent:204===n.parent.kind&&n.parent.name===n&&(n=n.parent),e.Debug.assert(79===n.kind||159===n.kind||204===n.kind,"'node' was expected to be a qualified name, identifier or property access in 'isPartOfTypeNode'.");case 159:case 204:case 108:var t=n.parent;if(179===t.kind)return!1;if(198===t.kind)return!t.isTypeOf;if(175<=t.kind&&t.kind<=198)return!0;switch(t.kind){case 226:return!Fo(t);case 161:return n===t.constraint;case 339:return n===t.constraint;case 165:case 164:case 162:case 252:return n===t.type;case 254:case 211:case 212:case 169:case 167:case 166:case 170:case 171:return n===t.type;case 172:case 173:case 174:return n===t.type;case 209:return n===t.type;case 206:case 207:return e.contains(t.typeArguments,n);case 208:return!1}}return!1}function hn(e,n){while(e){if(e.kind===n)return!0;e=e.parent}return!1}function vn(n,t){return r(n);function r(n){switch(n.kind){case 245:return t(n);case 261:case 233:case 237:case 238:case 239:case 240:case 241:case 242:case 246:case 247:case 287:case 288:case 248:case 250:case 290:return e.forEachChild(n,r)}}}function bn(n,t){return r(n);function r(n){switch(n.kind){case 222:t(n);var i=n.expression;return void(i&&r(i));case 258:case 256:case 259:case 257:return;default:if(e.isFunctionLike(n)){if(n.name&&160===n.name.kind)return void r(n.name.expression)}else yn(n)||e.forEachChild(n,r)}}}function En(n){return n&&181===n.kind?n.elementType:n&&176===n.kind?e.singleOrUndefined(n.typeArguments):void 0}function Tn(e){switch(e.kind){case 256:case 255:case 224:case 180:return e.members;case 203:return e.properties}}function Sn(e){if(e)switch(e.kind){case 201:case 294:case 162:case 291:case 165:case 164:case 292:case 252:return!0}return!1}function xn(n){return Sn(n)||e.isAccessor(n)}function Ln(e){return 253===e.parent.kind&&235===e.parent.parent.kind}function Dn(n){return e.isVariableDeclaration(n)?nn(n)&&e.isIdentifier(n.name)&&Ln(n):e.isPropertyDeclaration(n)?_o(n)&&uo(n):e.isPropertySignature(n)&&_o(n)}function An(e){switch(e.kind){case 167:case 166:case 169:case 170:case 171:case 254:case 211:return!0}return!1}function Cn(e,n){while(1){if(n&&n(e),248!==e.statement.kind)return e.statement;e=e.statement}}function In(n){return n&&233===n.kind&&e.isFunctionLike(n.parent)}function Nn(e){return e&&167===e.kind&&203===e.parent.kind}function kn(e){return 167===e.kind&&(203===e.parent.kind||224===e.parent.kind)}function Rn(e){return e&&1===e.kind}function On(e){return e&&0===e.kind}function wn(e,n,t){return e.properties.filter(function(e){if(291===e.kind){var r=Ge(e.name);return n===r||!!t&&t===r}return!1})}function Pn(n,t,r){return e.firstDefined(wn(n,t),function(n){return e.isArrayLiteralExpression(n.initializer)?e.find(n.initializer.elements,function(n){return e.isStringLiteral(n)&&n.text===r}):void 0})}function Mn(n){if(n&&n.statements.length){var t=n.statements[0].expression;return e.tryCast(t,e.isObjectLiteralExpression)}}function Fn(n,t,r){return e.firstDefined(Gn(n,t),function(n){return e.isArrayLiteralExpression(n.initializer)?e.find(n.initializer.elements,function(n){return e.isStringLiteral(n)&&n.text===r}):void 0})}function Gn(n,t){var r=Mn(n);return r?wn(r,t):e.emptyArray}function Bn(n){return e.findAncestor(n.parent,e.isFunctionLike)}function Un(n){return e.findAncestor(n.parent,e.isFunctionLikeDeclaration)}function Vn(n){return e.findAncestor(n.parent,e.isClassLike)}function jn(n){return e.findAncestor(n.parent,function(n){return e.isClassLike(n)||e.isFunctionLike(n)?"quit":e.isClassStaticBlockDeclaration(n)})}function Kn(n){return e.findAncestor(n.parent,e.isFunctionLikeOrClassStaticBlockDeclaration)}function Hn(n,t){e.Debug.assert(300!==n.kind);while(1){if(n=n.parent,!n)return e.Debug.fail();switch(n.kind){case 160:if(e.isClassLike(n.parent.parent))return n;n=n.parent;break;case 163:162===n.parent.kind&&e.isClassElement(n.parent.parent)?n=n.parent.parent:e.isClassElement(n.parent)&&(n=n.parent);break;case 212:if(!t)continue;case 254:case 211:case 259:case 168:case 165:case 164:case 167:case 166:case 169:case 170:case 171:case 172:case 173:case 174:case 258:case 300:return n}}}function Wn(n){e.isIdentifier(n)&&(e.isClassDeclaration(n.parent)||e.isFunctionDeclaration(n.parent))&&n.parent.name===n&&(n=n.parent);var t=Hn(n,!0);return e.isSourceFile(t)}function zn(e){var n=Hn(e,!1);if(n)switch(n.kind){case 169:case 254:case 211:return n}}function Jn(n,t){while(1){if(n=n.parent,!n)return n;switch(n.kind){case 160:n=n.parent;break;case 254:case 211:case 212:if(!t)continue;case 165:case 164:case 167:case 166:case 169:case 170:case 171:case 168:return n;case 163:162===n.parent.kind&&e.isClassElement(n.parent.parent)?n=n.parent.parent:e.isClassElement(n.parent)&&(n=n.parent);break}}}function qn(e){if(211===e.kind||212===e.kind){var n=e,t=e.parent;while(210===t.kind)n=t,t=t.parent;if(206===t.kind&&t.expression===n)return t}}function Xn(e){return 106===e.kind||Yn(e)}function Yn(e){var n=e.kind;return(204===n||205===n)&&106===e.expression.kind}function Qn(e){var n=e.kind;return(204===n||205===n)&&108===e.expression.kind}function Zn(n){var t;return!!n&&e.isVariableDeclaration(n)&&108===(null===(t=n.initializer)||void 0===t?void 0:t.kind)}function $n(n){return!!n&&(e.isShorthandPropertyAssignment(n)||e.isPropertyAssignment(n))&&e.isBinaryExpression(n.parent.parent)&&63===n.parent.parent.operatorToken.kind&&108===n.parent.parent.right.kind}function et(e){switch(e.kind){case 176:return e.typeName;case 226:return Go(e.expression)?e.expression:void 0;case 79:case 159:return e}}function nt(e){switch(e.kind){case 208:return e.tag;case 278:case 277:return e.tagName;default:return e.expression}}function tt(n,t,r){if(e.isNamedDeclaration(n)&&e.isPrivateIdentifier(n.name))return!1;switch(n.kind){case 255:return!0;case 165:return 255===t.kind;case 170:case 171:case 167:return void 0!==n.body&&255===t.kind;case 162:return void 0!==t.body&&(169===t.kind||167===t.kind||171===t.kind)&&255===r.kind}return!1}function rt(e,n,t){return void 0!==e.decorators&&tt(e,n,t)}function it(e,n,t){return rt(e,n,t)||at(e,n)}function at(n,t){switch(n.kind){case 255:return e.some(n.members,function(e){return it(e,n,t)});case 167:case 171:case 169:return e.some(n.parameters,function(e){return rt(e,n,t)});default:return!1}}function ot(e){if(rt(e))return!0;var n=Ma(e);return!!n&&at(n,e)}function st(e){var n=e.parent;return(278===n.kind||277===n.kind||279===n.kind)&&n.tagName===e}function ct(n){switch(n.kind){case 106:case 104:case 110:case 95:case 13:case 202:case 203:case 204:case 205:case 206:case 207:case 208:case 227:case 209:case 228:case 210:case 211:case 224:case 212:case 215:case 213:case 214:case 217:case 218:case 219:case 220:case 223:case 221:case 225:case 276:case 277:case 280:case 222:case 216:case 229:return!0;case 159:while(159===n.parent.kind)n=n.parent;return 179===n.parent.kind||e.isJSDocLinkLike(n.parent)||e.isJSDocNameReference(n.parent)||e.isJSDocMemberName(n.parent)||st(n);case 306:while(e.isJSDocMemberName(n.parent))n=n.parent;return 179===n.parent.kind||e.isJSDocLinkLike(n.parent)||e.isJSDocNameReference(n.parent)||e.isJSDocMemberName(n.parent)||st(n);case 79:if(179===n.parent.kind||e.isJSDocLinkLike(n.parent)||e.isJSDocNameReference(n.parent)||e.isJSDocMemberName(n.parent)||st(n))return!0;case 8:case 9:case 10:case 14:case 108:return lt(n);default:return!1}}function lt(e){var n=e.parent;switch(n.kind){case 252:case 162:case 165:case 164:case 294:case 291:case 201:return n.initializer===e;case 236:case 237:case 238:case 239:case 245:case 246:case 247:case 287:case 249:return n.expression===e;case 240:var t=n;return t.initializer===e&&253!==t.initializer.kind||t.condition===e||t.incrementor===e;case 241:case 242:var r=n;return r.initializer===e&&253!==r.initializer.kind||r.expression===e;case 209:case 227:return e===n.expression;case 231:return e===n.expression;case 160:return e===n.expression;case 163:case 286:case 285:case 293:return!0;case 226:return n.expression===e&&Fo(n);case 292:return n.objectAssignmentInitializer===e;default:return ct(n)}}function ut(e){while(159===e.kind||79===e.kind)e=e.parent;return 179===e.kind}function dt(n){return e.isNamespaceExport(n)&&!!n.parent.moduleSpecifier}function pt(e){return 263===e.kind&&275===e.moduleReference.kind}function ft(n){return e.Debug.assert(pt(n)),n.moduleReference.expression}function mt(e){return xt(e)&&sc(e.initializer).arguments[0]}function _t(e){return 263===e.kind&&275!==e.moduleReference.kind}function gt(e){return ht(e)}function yt(e){return!ht(e)}function ht(e){return!!e&&!!(131072&e.flags)}function vt(e){return!!e&&!!(33554432&e.flags)}function bt(e){return!Ze(e)}function Et(e){return!!e&&!!(4194304&e.flags)}function Tt(n){return e.isTypeReferenceNode(n)&&e.isIdentifier(n.typeName)&&"Object"===n.typeName.escapedText&&n.typeArguments&&2===n.typeArguments.length&&(148===n.typeArguments[0].kind||145===n.typeArguments[0].kind)}function St(n,t){if(206!==n.kind)return!1;var r=n,i=r.expression,a=r.arguments;if(79!==i.kind||"require"!==i.escapedText)return!1;if(1!==a.length)return!1;var o=a[0];return!t||e.isStringLiteralLike(o)}function xt(n){return 201===n.kind&&(n=n.parent.parent),e.isVariableDeclaration(n)&&!!n.initializer&&St(sc(n.initializer),!0)}function Lt(n){return e.isVariableStatement(n)&&n.declarationList.declarations.length>0&&e.every(n.declarationList.declarations,function(e){return xt(e)})}function Dt(e){return 39===e||34===e}function At(e,n){return 34===Z(n,e).charCodeAt(0)}function Ct(n){return e.isBinaryExpression(n)||rc(n)||e.isIdentifier(n)||e.isCallExpression(n)}function It(n){return ht(n)&&n.initializer&&e.isBinaryExpression(n.initializer)&&(56===n.initializer.operatorToken.kind||60===n.initializer.operatorToken.kind)&&n.name&&Go(n.name)&&Ft(n.name,n.initializer.left)?n.initializer.right:n.initializer}function Nt(e){var n=It(e);return n&&Ot(n,Ko(e.name))}function kt(n,t){return e.forEach(n.properties,function(n){return e.isPropertyAssignment(n)&&e.isIdentifier(n.name)&&"value"===n.name.escapedText&&n.initializer&&Ot(n.initializer,t)})}function Rt(n){if(n&&n.parent&&e.isBinaryExpression(n.parent)&&63===n.parent.operatorToken.kind){var t=Ko(n.parent.left);return Ot(n.parent.right,t)||wt(n.parent.left,n.parent.right,t)}if(n&&e.isCallExpression(n)&&Kt(n)){var r=kt(n.arguments[2],"prototype"===n.arguments[1].text);if(r)return r}}function Ot(n,t){if(e.isCallExpression(n)){var r=Vr(n.expression);return 211===r.kind||212===r.kind?n:void 0}return 211===n.kind||224===n.kind||212===n.kind?n:e.isObjectLiteralExpression(n)&&(0===n.properties.length||t)?n:void 0}function wt(n,t,r){var i=e.isBinaryExpression(t)&&(56===t.operatorToken.kind||60===t.operatorToken.kind)&&Ot(t.right,r);if(i&&Ft(n,t.left))return i}function Pt(n){var t=e.isVariableDeclaration(n.parent)?n.parent.name:e.isBinaryExpression(n.parent)&&63===n.parent.operatorToken.kind?n.parent.left:void 0;return t&&Ot(n.right,Ko(t))&&Go(t)&&Ft(t,n.left)}function Mt(n){if(e.isBinaryExpression(n.parent)){var t=56!==n.parent.operatorToken.kind&&60!==n.parent.operatorToken.kind||!e.isBinaryExpression(n.parent.parent)?n.parent:n.parent.parent;if(63===t.operatorToken.kind&&e.isIdentifier(t.left))return t.left}else if(e.isVariableDeclaration(n.parent))return n.parent.name}function Ft(n,t){if(Ti(n)&&Ti(t))return Si(n)===Si(t);if(e.isIdentifier(n)&&Ht(t)&&(108===t.expression.kind||e.isIdentifier(t.expression)&&("window"===t.expression.escapedText||"self"===t.expression.escapedText||"global"===t.expression.escapedText))){var r=Xt(t);return e.isPrivateIdentifier(r)&&e.Debug.fail("Unexpected PrivateIdentifier in name expression with literal-like access."),Ft(n,r)}return!(!Ht(n)||!Ht(t))&&($t(n)===$t(t)&&Ft(n.expression,t.expression))}function Gt(e){while(wo(e,!0))e=e.right;return e}function Bt(n){return e.isIdentifier(n)&&"exports"===n.escapedText}function Ut(n){return e.isIdentifier(n)&&"module"===n.escapedText}function Vt(n){return(e.isPropertyAccessExpression(n)||Wt(n))&&Ut(n.expression)&&"exports"===$t(n)}function jt(e){var n=Yt(e);return 5===n||ht(e)?n:0}function Kt(n){return 3===e.length(n.arguments)&&e.isPropertyAccessExpression(n.expression)&&e.isIdentifier(n.expression.expression)&&"Object"===e.idText(n.expression.expression)&&"defineProperty"===e.idText(n.expression.name)&&yi(n.arguments[1])&&qt(n.arguments[0],!0)}function Ht(n){return e.isPropertyAccessExpression(n)||Wt(n)}function Wt(n){return e.isElementAccessExpression(n)&&yi(n.argumentExpression)}function zt(n,t){return e.isPropertyAccessExpression(n)&&(!t&&108===n.expression.kind||e.isIdentifier(n.name)&&qt(n.expression,!0))||Jt(n,t)}function Jt(e,n){return Wt(e)&&(!n&&108===e.expression.kind||Go(e.expression)||zt(e.expression,!0))}function qt(e,n){return Go(e)||zt(e,n)}function Xt(n){return e.isPropertyAccessExpression(n)?n.name:n.argumentExpression}function Yt(n){if(e.isCallExpression(n)){if(!Kt(n))return 0;var t=n.arguments[0];return Bt(t)||Vt(t)?8:zt(t)&&"prototype"===$t(t)?9:7}return 63!==n.operatorToken.kind||!rc(n.left)||Qt(Gt(n))?0:qt(n.left.expression,!0)&&"prototype"===$t(n.left)&&e.isObjectLiteralExpression(nr(n))?6:er(n.left)}function Qt(n){return e.isVoidExpression(n)&&e.isNumericLiteral(n.expression)&&"0"===n.expression.text}function Zt(n){if(e.isPropertyAccessExpression(n))return n.name;var t=Vr(n.argumentExpression);return e.isNumericLiteral(t)||e.isStringLiteralLike(t)?t:n}function $t(n){var t=Zt(n);if(t){if(e.isIdentifier(t))return t.escapedText;if(e.isStringLiteralLike(t)||e.isNumericLiteral(t))return e.escapeLeadingUnderscores(t.text)}}function er(n){if(108===n.expression.kind)return 4;if(Vt(n))return 2;if(qt(n.expression,!0)){if(Ko(n.expression))return 3;var t=n;while(!e.isIdentifier(t.expression))t=t.expression;var r=t.expression;if(("exports"===r.escapedText||"module"===r.escapedText&&"exports"===$t(t))&&zt(n))return 1;if(qt(n,!0)||e.isElementAccessExpression(n)&&bi(n))return 5}return 0}function nr(n){while(e.isBinaryExpression(n.right))n=n.right;return n.right}function tr(n){return e.isBinaryExpression(n)&&3===jt(n)}function rr(n){return ht(n)&&n.parent&&236===n.parent.kind&&(!e.isElementAccessExpression(n)||Wt(n))&&!!e.getJSDocTypeTag(n.parent)}function ir(e,n){var t=e.valueDeclaration;(!t||(!(8388608&n.flags)||8388608&t.flags)&&Ct(t)&&!Ct(n)||t.kind!==n.kind&&ge(t))&&(e.valueDeclaration=n)}function ar(n){if(!n||!n.valueDeclaration)return!1;var t=n.valueDeclaration;return 254===t.kind||e.isVariableDeclaration(t)&&t.initializer&&e.isFunctionLike(t.initializer)}function or(n){var t,r,i;switch(n.kind){case 252:return n.initializer.arguments[0].text;case 264:return null===(t=e.tryCast(n.moduleSpecifier,e.isStringLiteralLike))||void 0===t?void 0:t.text;case 263:return null===(i=e.tryCast(null===(r=e.tryCast(n.moduleReference,e.isExternalModuleReference))||void 0===r?void 0:r.expression,e.isStringLiteralLike))||void 0===i?void 0:i.text;default:e.Debug.assertNever(n)}}function sr(n){return cr(n)||e.Debug.failBadSyntaxKind(n.parent)}function cr(n){switch(n.parent.kind){case 264:case 270:return n.parent;case 275:return n.parent.parent;case 206:return an(n.parent)||St(n.parent,!1)?n.parent:void 0;case 194:return e.Debug.assert(e.isStringLiteral(n)),e.tryCast(n.parent.parent,e.isImportTypeNode);default:return}}function lr(n){switch(n.kind){case 264:case 270:return n.moduleSpecifier;case 263:return 275===n.moduleReference.kind?n.moduleReference.expression:void 0;case 198:return sn(n)?n.argument.literal:void 0;case 206:return n.arguments[0];case 259:return 10===n.name.kind?n.name:void 0;default:return e.Debug.assertNever(n)}}function ur(n){switch(n.kind){case 264:return n.importClause&&e.tryCast(n.importClause.namedBindings,e.isNamespaceImport);case 263:return n;case 270:return n.exportClause&&e.tryCast(n.exportClause,e.isNamespaceExport);default:return e.Debug.assertNever(n)}}function dr(e){return 264===e.kind&&!!e.importClause&&!!e.importClause.name}function pr(n,t){if(n.name){var r=t(n);if(r)return r}if(n.namedBindings){r=e.isNamespaceImport(n.namedBindings)?t(n.namedBindings):e.forEach(n.namedBindings.elements,t);if(r)return r}}function fr(e){if(e)switch(e.kind){case 162:case 167:case 166:case 292:case 291:case 165:case 164:return void 0!==e.questionToken}return!1}function mr(n){var t=e.isJSDocFunctionType(n)?e.firstOrUndefined(n.parameters):void 0,r=e.tryCast(t&&t.name,e.isIdentifier);return!!r&&"new"===r.escapedText}function _r(e){return 340===e.kind||333===e.kind||334===e.kind}function gr(n){return _r(n)||e.isTypeAliasDeclaration(n)}function yr(n){return e.isExpressionStatement(n)&&e.isBinaryExpression(n.expression)&&63===n.expression.operatorToken.kind?Gt(n.expression):void 0}function hr(n){return e.isExpressionStatement(n)&&e.isBinaryExpression(n.expression)&&0!==jt(n.expression)&&e.isBinaryExpression(n.expression.right)&&(56===n.expression.right.operatorToken.kind||60===n.expression.right.operatorToken.kind)?n.expression.right.right:void 0}function vr(e){switch(e.kind){case 235:var n=br(e);return n&&n.initializer;case 165:return e.initializer;case 291:return e.initializer}}function br(n){return e.isVariableStatement(n)?e.firstOrUndefined(n.declarationList.declarations):void 0}function Er(n){return e.isModuleDeclaration(n)&&n.body&&259===n.body.kind?n.body:void 0}function Tr(n,t){var r;Sn(n)&&e.hasInitializer(n)&&e.hasJSDocNodes(n.initializer)&&(r=e.append(r,e.last(n.initializer.jsDoc)));var i=n;while(i&&i.parent){if(e.hasJSDocNodes(i)&&(r=e.append(r,e.last(i.jsDoc))),162===i.kind){r=e.addRange(r,(t?e.getJSDocParameterTagsNoCache:e.getJSDocParameterTags)(i));break}if(161===i.kind){r=e.addRange(r,(t?e.getJSDocTypeParameterTagsNoCache:e.getJSDocTypeParameterTags)(i));break}i=Sr(i)}return r||e.emptyArray}function Sr(n){var t=n.parent;return 291===t.kind||269===t.kind||165===t.kind||236===t.kind&&204===n.kind||245===t.kind||Er(t)||e.isBinaryExpression(n)&&63===n.operatorToken.kind?t:t.parent&&(br(t.parent)===n||e.isBinaryExpression(t)&&63===t.operatorToken.kind)?t.parent:t.parent&&t.parent.parent&&(br(t.parent.parent)||vr(t.parent.parent)===n||hr(t.parent.parent))?t.parent.parent:void 0}function xr(n){if(n.symbol)return n.symbol;if(e.isIdentifier(n.name)){var t=n.name.escapedText,r=Lr(n);if(r){var i=e.find(r.parameters,function(e){return 79===e.name.kind&&e.name.escapedText===t});return i&&i.symbol}}}function Lr(n){var t=Dr(n);return t&&e.isFunctionLike(t)?t:void 0}function Dr(e){var n=Ar(e);if(n)return hr(n)||yr(n)||vr(n)||br(n)||Er(n)||n}function Ar(n){var t=Cr(n);if(t){var r=t.parent;return r&&r.jsDoc&&t===e.lastOrUndefined(r.jsDoc)?r:void 0}}function Cr(n){return e.findAncestor(n.parent,e.isJSDoc)}function Ir(n){var t=n.name.escapedText,r=n.parent.parent.parent.typeParameters;return r&&e.find(r,function(e){return e.name.escapedText===t})}function Nr(n){var t=e.lastOrUndefined(n.parameters);return!!t&&kr(t)}function kr(n){var t=e.isJSDocParameterTag(n)?n.typeExpression&&n.typeExpression.type:n.type;return void 0!==n.dotDotDotToken||!!t&&313===t.kind}function Rr(e){return!!e.typeArguments}function Or(e){var n=e.parent;while(1){switch(n.kind){case 219:var t=n.operatorToken.kind;return ko(t)&&n.left===e?63===t||Io(t)?1:2:0;case 217:case 218:var r=n.operator;return 45===r||46===r?2:0;case 241:case 242:return n.initializer===e?1:0;case 210:case 202:case 223:case 228:e=n;break;case 293:e=n.parent;break;case 292:if(n.name!==e)return 0;e=n.parent;break;case 291:if(n.name===e)return 0;e=n.parent;break;default:return 0}n=e.parent}}function wr(e){return 0!==Or(e)}function Pr(e){switch(e.kind){case 233:case 235:case 246:case 237:case 247:case 261:case 287:case 288:case 248:case 240:case 241:case 242:case 238:case 239:case 250:case 290:return!0}return!1}function Mr(n){return e.isFunctionExpression(n)||e.isArrowFunction(n)||e.isMethodOrAccessor(n)||e.isFunctionDeclaration(n)||e.isConstructorDeclaration(n)}function Fr(e,n){while(e&&e.kind===n)e=e.parent;return e}function Gr(e){return Fr(e,189)}function Br(e){return Fr(e,210)}function Ur(e){var n;while(e&&189===e.kind)n=e,e=e.parent;return[n,e]}function Vr(n){return e.skipOuterExpressions(n,1)}function jr(e){return(204===e.kind||205===e.kind)&&(e=Br(e.parent),e&&213===e.kind)}function Kr(e,n){while(e){if(e===n)return!0;e=e.parent}return!1}function Hr(n){return!e.isSourceFile(n)&&!e.isBindingPattern(n)&&e.isDeclaration(n.parent)&&n.parent.name===n}function Wr(n){var t=n.parent;switch(n.kind){case 10:case 14:case 8:if(e.isComputedPropertyName(t))return t.parent;case 79:if(e.isDeclaration(t))return t.name===n?t:void 0;if(e.isQualifiedName(t)){var r=t.parent;return e.isJSDocParameterTag(r)&&r.name===t?r:void 0}var i=t.parent;return e.isBinaryExpression(i)&&0!==jt(i)&&(i.left.symbol||i.symbol)&&e.getNameOfDeclaration(i)===n?i:void 0;case 80:return e.isDeclaration(t)&&t.name===n?t:void 0;default:return}}function zr(n){return yi(n)&&160===n.parent.kind&&e.isDeclaration(n.parent.parent)}function Jr(e){var n=e.parent;switch(n.kind){case 165:case 164:case 167:case 166:case 170:case 171:case 294:case 291:case 204:return n.name===e;case 159:return n.right===e;case 201:case 268:return n.propertyName===e;case 273:case 283:return!0}return!1}function qr(n){return 263===n.kind||262===n.kind||265===n.kind&&!!n.name||266===n.kind||272===n.kind||268===n.kind||273===n.kind||269===n.kind&&Qr(n)||e.isBinaryExpression(n)&&2===jt(n)&&Qr(n)||e.isPropertyAccessExpression(n)&&e.isBinaryExpression(n.parent)&&n.parent.left===n&&63===n.parent.operatorToken.kind&&Yr(n.parent.right)||292===n.kind||291===n.kind&&Yr(n.initializer)}function Xr(e){switch(e.parent.kind){case 265:case 268:case 266:case 273:case 269:case 263:return e.parent;case 159:do{e=e.parent}while(159===e.parent.kind);return Xr(e)}}function Yr(n){return Go(n)||e.isClassExpression(n)}function Qr(e){var n=Zr(e);return Yr(n)}function Zr(n){return e.isExportAssignment(n)?n.expression:n.right}function $r(e){return 292===e.kind?e.name:291===e.kind?e.initializer:e.parent.right}function ei(n){var t=ni(n);if(t&&ht(n)){var r=e.getJSDocAugmentsTag(n);if(r)return r.class}return t}function ni(e){var n=ai(e.heritageClauses,94);return n&&n.types.length>0?n.types[0]:void 0}function ti(n){if(ht(n))return e.getJSDocImplementsTags(n).map(function(e){return e.class});var t=ai(n.heritageClauses,117);return null===t||void 0===t?void 0:t.types}function ri(n){return e.isInterfaceDeclaration(n)?ii(n)||e.emptyArray:e.isClassLike(n)&&e.concatenate(e.singleElementArray(ei(n)),ti(n))||e.emptyArray}function ii(e){var n=ai(e.heritageClauses,94);return n?n.types:void 0}function ai(e,n){if(e)for(var t=0,r=e;t<r.length;t++){var i=r[t];if(i.token===n)return i}}function oi(e,n){while(e){if(e.kind===n)return e;e=e.parent}}function si(e){return 81<=e&&e<=158}function ci(e){return 126<=e&&e<=158}function li(e){return si(e)&&!ci(e)}function ui(e){return 117<=e&&e<=125}function di(n){var t=e.stringToToken(n);return void 0!==t&&li(t)}function pi(n){var t=e.stringToToken(n);return void 0!==t&&si(t)}function fi(e){var n=e.originalKeywordKind;return!!n&&!ci(n)}function mi(e){return 2<=e&&e<=7}function _i(e){if(!e)return 4;var n=0;switch(e.kind){case 254:case 211:case 167:e.asteriskToken&&(n|=1);case 212:co(e,256)&&(n|=2);break}return e.body||(n|=4),n}function gi(e){switch(e.kind){case 254:case 211:case 212:case 167:return void 0!==e.body&&void 0===e.asteriskToken&&co(e,256)}return!1}function yi(n){return e.isStringLiteralLike(n)||e.isNumericLiteral(n)}function hi(n){return e.isPrefixUnaryExpression(n)&&(39===n.operator||40===n.operator)&&e.isNumericLiteral(n.operand)}function vi(n){var t=e.getNameOfDeclaration(n);return!!t&&bi(t)}function bi(n){if(160!==n.kind&&205!==n.kind)return!1;var t=e.isElementAccessExpression(n)?Vr(n.argumentExpression):n.expression;return!yi(t)&&!hi(t)}function Ei(n){switch(n.kind){case 79:case 80:return n.escapedText;case 10:case 8:return e.escapeLeadingUnderscores(n.text);case 160:var t=n.expression;return yi(t)?e.escapeLeadingUnderscores(t.text):hi(t)?40===t.operator?e.tokenToString(t.operator)+t.operand.text:t.operand.text:void 0;default:return e.Debug.assertNever(n)}}function Ti(e){switch(e.kind){case 79:case 10:case 14:case 8:return!0;default:return!1}}function Si(n){return e.isMemberName(n)?e.idText(n):n.text}function xi(n){return e.isMemberName(n)?n.escapedText:e.escapeLeadingUnderscores(n.text)}function Li(n){return"__@"+e.getSymbolId(n)+"@"+n.escapedName}function Di(n,t){return"__#"+e.getSymbolId(n)+"@"+t}function Ai(n){return e.startsWith(n.escapedName,"__@")}function Ci(n){return e.startsWith(n.escapedName,"__#")}function Ii(e){return 79===e.kind&&"Symbol"===e.escapedText}function Ni(e){return"push"===e.escapedText||"unshift"===e.escapedText}function ki(e){var n=Ri(e);return 162===n.kind}function Ri(e){while(201===e.kind)e=e.parent.parent;return e}function Oi(e){var n=e.kind;return 169===n||211===n||254===n||212===n||167===n||170===n||171===n||259===n||300===n}function wi(e){return Yl(e.pos)||Yl(e.end)}function Pi(n){return e.getParseTreeNode(n,e.isSourceFile)||n}function Mi(e){var n=Bi(e),t=207===e.kind&&void 0!==e.arguments;return Fi(e.kind,n,t)}function Fi(e,n,t){switch(e){case 207:return t?0:1;case 217:case 214:case 215:case 213:case 216:case 220:case 222:return 1;case 219:switch(n){case 42:case 63:case 64:case 65:case 67:case 66:case 68:case 69:case 70:case 71:case 72:case 73:case 78:case 74:case 75:case 76:case 77:return 1}}return 0}function Gi(e){var n=Bi(e),t=207===e.kind&&void 0!==e.arguments;return Ui(e.kind,n,t)}function Bi(e){return 219===e.kind?e.operatorToken.kind:217===e.kind||218===e.kind?e.operator:e.kind}function Ui(e,n,t){switch(e){case 346:return 0;case 223:return 1;case 222:return 2;case 220:return 4;case 219:switch(n){case 27:return 0;case 63:case 64:case 65:case 67:case 66:case 68:case 69:case 70:case 71:case 72:case 73:case 78:case 74:case 75:case 76:case 77:return 3;default:return Vi(n)}case 209:case 228:case 217:case 214:case 215:case 213:case 216:return 16;case 218:return 17;case 206:return 18;case 207:return t?19:18;case 208:case 204:case 205:case 229:return 19;case 227:return 11;case 108:case 106:case 79:case 104:case 110:case 95:case 8:case 9:case 10:case 202:case 203:case 211:case 212:case 224:case 13:case 14:case 221:case 210:case 225:case 276:case 277:case 280:return 20;default:return-1}}function Vi(e){switch(e){case 60:return 4;case 56:return 5;case 55:return 6;case 51:return 7;case 52:return 8;case 50:return 9;case 34:case 35:case 36:case 37:return 10;case 29:case 31:case 32:case 33:case 102:case 101:case 127:return 11;case 47:case 48:case 49:return 12;case 39:case 40:return 13;case 41:case 43:case 44:return 14;case 42:return 15}return-1}function ji(n){return e.filter(n,function(e){switch(e.kind){case 286:return!!e.expression;case 11:return!e.containsOnlyTriviaWhiteSpaces;default:return!0}})}function Ki(){var n=[],t=[],r=new e.Map,i=!1;return{add:o,lookup:a,getGlobalDiagnostics:s,getDiagnostics:c};function a(t){var i;if(i=t.file?r.get(t.file.fileName):n,i){var a=e.binarySearch(i,t,e.identity,Rc);return a>=0?i[a]:void 0}}function o(a){var o;a.file?(o=r.get(a.file.fileName),o||(o=[],r.set(a.file.fileName,o),e.insertSorted(t,a.file.fileName,e.compareStringsCaseSensitive))):(i&&(i=!1,n=n.slice()),o=n),e.insertSorted(o,a,kc)}function s(){return i=!0,n}function c(i){if(i)return r.get(i)||[];var a=e.flatMapToMutable(t,function(e){return r.get(e)});return n.length?(a.unshift.apply(a,n),a):a}}e.isPartOfTypeNode=yn,e.isChildOfNodeWithKind=hn,e.forEachReturnStatement=vn,e.forEachYieldExpression=bn,e.getRestParameterElementType=En,e.getMembersOfDeclaration=Tn,e.isVariableLike=Sn,e.isVariableLikeOrAccessor=xn,e.isVariableDeclarationInVariableStatement=Ln,e.isValidESSymbolDeclaration=Dn,e.introducesArgumentsExoticObject=An,e.unwrapInnermostStatementOfLabel=Cn,e.isFunctionBlock=In,e.isObjectLiteralMethod=Nn,e.isObjectLiteralOrClassExpressionMethod=kn,e.isIdentifierTypePredicate=Rn,e.isThisTypePredicate=On,e.getPropertyAssignment=wn,e.getPropertyArrayElementValue=Pn,e.getTsConfigObjectLiteralExpression=Mn,e.getTsConfigPropArrayElementValue=Fn,e.getTsConfigPropArray=Gn,e.getContainingFunction=Bn,e.getContainingFunctionDeclaration=Un,e.getContainingClass=Vn,e.getContainingClassStaticBlock=jn,e.getContainingFunctionOrClassStaticBlock=Kn,e.getThisContainer=Hn,e.isInTopLevelContext=Wn,e.getNewTargetContainer=zn,e.getSuperContainer=Jn,e.getImmediatelyInvokedFunctionExpression=qn,e.isSuperOrSuperProperty=Xn,e.isSuperProperty=Yn,e.isThisProperty=Qn,e.isThisInitializedDeclaration=Zn,e.isThisInitializedObjectBindingExpression=$n,e.getEntityNameFromTypeNode=et,e.getInvokedExpression=nt,e.nodeCanBeDecorated=tt,e.nodeIsDecorated=rt,e.nodeOrChildIsDecorated=it,e.childIsDecorated=at,e.classOrConstructorParameterIsDecorated=ot,e.isJSXTagName=st,e.isExpressionNode=ct,e.isInExpressionContext=lt,e.isPartOfTypeQuery=ut,e.isNamespaceReexportDeclaration=dt,e.isExternalModuleImportEqualsDeclaration=pt,e.getExternalModuleImportEqualsDeclarationExpression=ft,e.getExternalModuleRequireArgument=mt,e.isInternalModuleImportEqualsDeclaration=_t,e.isSourceFileJS=gt,e.isSourceFileNotJS=yt,e.isInJSFile=ht,e.isInJsonFile=vt,e.isSourceFileNotJson=bt,e.isInJSDoc=Et,e.isJSDocIndexSignature=Tt,e.isRequireCall=St,e.isRequireVariableDeclaration=xt,e.isRequireVariableStatement=Lt,e.isSingleOrDoubleQuote=Dt,e.isStringDoubleQuoted=At,e.isAssignmentDeclaration=Ct,e.getEffectiveInitializer=It,e.getDeclaredExpandoInitializer=Nt,e.getAssignedExpandoInitializer=Rt,e.getExpandoInitializer=Ot,e.isDefaultedExpandoInitializer=Pt,e.getNameOfExpando=Mt,e.isSameEntityName=Ft,e.getRightMostAssignedExpression=Gt,e.isExportsIdentifier=Bt,e.isModuleIdentifier=Ut,e.isModuleExportsAccessExpression=Vt,e.getAssignmentDeclarationKind=jt,e.isBindableObjectDefinePropertyCall=Kt,e.isLiteralLikeAccess=Ht,e.isLiteralLikeElementAccess=Wt,e.isBindableStaticAccessExpression=zt,e.isBindableStaticElementAccessExpression=Jt,e.isBindableStaticNameExpression=qt,e.getNameOrArgument=Xt,e.getElementOrPropertyAccessArgumentExpressionOrName=Zt,e.getElementOrPropertyAccessName=$t,e.getAssignmentDeclarationPropertyAccessKind=er,e.getInitializerOfBinaryExpression=nr,e.isPrototypePropertyAssignment=tr,e.isSpecialPropertyDeclaration=rr,e.setValueDeclaration=ir,e.isFunctionSymbol=ar,e.tryGetModuleSpecifierFromDeclaration=or,e.importFromModuleSpecifier=sr,e.tryGetImportFromModuleSpecifier=cr,e.getExternalModuleName=lr,e.getNamespaceDeclarationNode=ur,e.isDefaultImport=dr,e.forEachImportClauseDeclaration=pr,e.hasQuestionToken=fr,e.isJSDocConstructSignature=mr,e.isJSDocTypeAlias=_r,e.isTypeAlias=gr,e.getSingleInitializerOfVariableStatementOrPropertyDeclaration=vr,e.getSingleVariableOfVariableStatement=br,e.getJSDocCommentsAndTags=Tr,e.getNextJSDocCommentLocation=Sr,e.getParameterSymbolFromJSDoc=xr,e.getHostSignatureFromJSDoc=Lr,e.getEffectiveJSDocHost=Dr,e.getJSDocHost=Ar,e.getJSDocRoot=Cr,e.getTypeParameterFromJsDoc=Ir,e.hasRestParameter=Nr,e.isRestParameter=kr,e.hasTypeArguments=Rr,function(e){e[e["None"]=0]="None",e[e["Definite"]=1]="Definite",e[e["Compound"]=2]="Compound"}(e.AssignmentKind||(e.AssignmentKind={})),e.getAssignmentTargetKind=Or,e.isAssignmentTarget=wr,e.isNodeWithPossibleHoistedDeclaration=Pr,e.isValueSignatureDeclaration=Mr,e.walkUpParenthesizedTypes=Gr,e.walkUpParenthesizedExpressions=Br,e.walkUpParenthesizedTypesAndGetParentAndChild=Ur,e.skipParentheses=Vr,e.isDeleteTarget=jr,e.isNodeDescendantOf=Kr,e.isDeclarationName=Hr,e.getDeclarationFromName=Wr,e.isLiteralComputedPropertyDeclarationName=zr,e.isIdentifierName=Jr,e.isAliasSymbolDeclaration=qr,e.getAliasDeclarationFromName=Xr,e.isAliasableExpression=Yr,e.exportAssignmentIsAlias=Qr,e.getExportAssignmentExpression=Zr,e.getPropertyAssignmentAliasLikeExpression=$r,e.getEffectiveBaseTypeNode=ei,e.getClassExtendsHeritageElement=ni,e.getEffectiveImplementsTypeNodes=ti,e.getAllSuperTypeNodes=ri,e.getInterfaceBaseTypeNodes=ii,e.getHeritageClause=ai,e.getAncestor=oi,e.isKeyword=si,e.isContextualKeyword=ci,e.isNonContextualKeyword=li,e.isFutureReservedKeyword=ui,e.isStringANonContextualKeyword=di,e.isStringAKeyword=pi,e.isIdentifierANonContextualKeyword=fi,e.isTrivia=mi,function(e){e[e["Normal"]=0]="Normal",e[e["Generator"]=1]="Generator",e[e["Async"]=2]="Async",e[e["Invalid"]=4]="Invalid",e[e["AsyncGenerator"]=3]="AsyncGenerator"}(e.FunctionFlags||(e.FunctionFlags={})),e.getFunctionFlags=_i,e.isAsyncFunction=gi,e.isStringOrNumericLiteralLike=yi,e.isSignedNumericLiteral=hi,e.hasDynamicName=vi,e.isDynamicName=bi,e.getPropertyNameForPropertyNameNode=Ei,e.isPropertyNameLiteral=Ti,e.getTextOfIdentifierOrLiteral=Si,e.getEscapedTextOfIdentifierOrLiteral=xi,e.getPropertyNameForUniqueESSymbol=Li,e.getSymbolNameForPrivateIdentifier=Di,e.isKnownSymbol=Ai,e.isPrivateIdentifierSymbol=Ci,e.isESSymbolIdentifier=Ii,e.isPushOrUnshiftIdentifier=Ni,e.isParameterDeclaration=ki,e.getRootDeclaration=Ri,e.nodeStartsNewLexicalEnvironment=Oi,e.nodeIsSynthesized=wi,e.getOriginalSourceFile=Pi,function(e){e[e["Left"]=0]="Left",e[e["Right"]=1]="Right"}(e.Associativity||(e.Associativity={})),e.getExpressionAssociativity=Mi,e.getOperatorAssociativity=Fi,e.getExpressionPrecedence=Gi,e.getOperator=Bi,function(e){e[e["Comma"]=0]="Comma",e[e["Spread"]=1]="Spread",e[e["Yield"]=2]="Yield",e[e["Assignment"]=3]="Assignment",e[e["Conditional"]=4]="Conditional",e[e["Coalesce"]=4]="Coalesce",e[e["LogicalOR"]=5]="LogicalOR",e[e["LogicalAND"]=6]="LogicalAND",e[e["BitwiseOR"]=7]="BitwiseOR",e[e["BitwiseXOR"]=8]="BitwiseXOR",e[e["BitwiseAND"]=9]="BitwiseAND",e[e["Equality"]=10]="Equality",e[e["Relational"]=11]="Relational",e[e["Shift"]=12]="Shift",e[e["Additive"]=13]="Additive",e[e["Multiplicative"]=14]="Multiplicative",e[e["Exponentiation"]=15]="Exponentiation",e[e["Unary"]=16]="Unary",e[e["Update"]=17]="Update",e[e["LeftHandSide"]=18]="LeftHandSide",e[e["Member"]=19]="Member",e[e["Primary"]=20]="Primary",e[e["Highest"]=20]="Highest",e[e["Lowest"]=0]="Lowest",e[e["Invalid"]=-1]="Invalid"}(e.OperatorPrecedence||(e.OperatorPrecedence={})),e.getOperatorPrecedence=Ui,e.getBinaryOperatorPrecedence=Vi,e.getSemanticJsxChildren=ji,e.createDiagnosticCollection=Ki;var Hi=/\$\{/g;function Wi(e){return e.replace(Hi,"\\${")}function zi(n){return n&&!!(e.isNoSubstitutionTemplateLiteral(n)?n.templateFlags:n.head.templateFlags||e.some(n.templateSpans,function(e){return!!e.literal.templateFlags}))}e.hasInvalidEscape=zi;var Ji=/[\\\"\u0000-\u001f\t\v\f\b\r\n\u2028\u2029\u0085]/g,qi=/[\\\'\u0000-\u001f\t\v\f\b\r\n\u2028\u2029\u0085]/g,Xi=/\r\n|[\\\`\u0000-\u001f\t\v\f\b\r\u2028\u2029\u0085]/g,Yi=new e.Map(e.getEntries({"\t":"\\t","\v":"\\v","\f":"\\f","\b":"\\b","\r":"\\r","\n":"\\n","\\":"\\\\",'"':'\\"',"'":"\\'","`":"\\`","\u2028":"\\u2028","\u2029":"\\u2029","":"\\u0085","\r\n":"\\r\\n"}));function Qi(e){var n=e.toString(16).toUpperCase(),t=("0000"+n).slice(-4);return"\\u"+t}function Zi(e,n,t){if(0===e.charCodeAt(0)){var r=t.charCodeAt(n+e.length);return r>=48&&r<=57?"\\x00":"\\0"}return Yi.get(e)||Qi(e.charCodeAt(0))}function $i(e,n){var t=96===n?Xi:39===n?qi:Ji;return e.replace(t,Zi)}e.escapeString=$i;var ea=/[^\u0000-\u007F]/g;function na(e,n){return e=$i(e,n),ea.test(e)?e.replace(ea,function(e){return Qi(e.charCodeAt(0))}):e}e.escapeNonAsciiString=na;var ta=/[\"\u0000-\u001f\u2028\u2029\u0085]/g,ra=/[\'\u0000-\u001f\u2028\u2029\u0085]/g,ia=new e.Map(e.getEntries({'"':"&quot;","'":"&apos;"}));function aa(e){var n=e.toString(16).toUpperCase();return"&#x"+n+";"}function oa(e){return 0===e.charCodeAt(0)?"&#0;":ia.get(e)||aa(e.charCodeAt(0))}function sa(e,n){var t=39===n?ra:ta;return e.replace(t,oa)}function ca(e){var n=e.length;return n>=2&&e.charCodeAt(0)===e.charCodeAt(n-1)&&la(e.charCodeAt(0))?e.substring(1,n-1):e}function la(e){return 39===e||34===e||96===e}function ua(n){var t=n.charCodeAt(0);return t>=97&&t<=122||e.stringContains(n,"-")||e.stringContains(n,":")}e.escapeJsxAttributeString=sa,e.stripQuotes=ca,e.isIntrinsicJsxName=ua;var da=["","    "];function pa(e){for(var n=da[1],t=da.length;t<=e;t++)da.push(da[t-1]+n);return da[e]}function fa(){return da[1].length}function ma(n){var t,r,i,a,o,s=!1;function c(n){var r=e.computeLineStarts(n);r.length>1?(a=a+r.length-1,o=t.length-n.length+e.last(r),i=o-t.length===0):i=!1}function l(e){e&&e.length&&(i&&(e=pa(r)+e,i=!1),t+=e,c(e))}function u(e){e&&(s=!1),l(e)}function d(e){e&&(s=!0),l(e)}function p(){t="",r=0,i=!0,a=0,o=0,s=!1}function f(e){void 0!==e&&(t+=e,c(e),s=!1)}function m(e){e&&e.length&&u(e)}function _(e){i&&!e||(t+=n,a++,o=t.length,i=!0,s=!1)}function g(){return i?t.length:t.length+n.length}return p(),{write:u,rawWrite:f,writeLiteral:m,writeLine:_,increaseIndent:function(){r++},decreaseIndent:function(){r--},getIndent:function(){return r},getTextPos:function(){return t.length},getLine:function(){return a},getColumn:function(){return i?r*fa():t.length-o},getText:function(){return t},isAtStartOfLine:function(){return i},hasTrailingComment:function(){return s},hasTrailingWhitespace:function(){return!!t.length&&e.isWhiteSpaceLike(t.charCodeAt(t.length-1))},clear:p,reportInaccessibleThisError:e.noop,reportPrivateInBaseOfClassExpression:e.noop,reportInaccessibleUniqueSymbolError:e.noop,trackSymbol:function(){return!1},writeKeyword:u,writeOperator:u,writeParameter:u,writeProperty:u,writePunctuation:u,writeSpace:u,writeStringLiteral:u,writeSymbol:function(e,n){return u(e)},writeTrailingSemicolon:u,writeComment:d,getTextPosWithWriteLine:g}}function _a(e){var n=!1;function t(){n&&(e.writeTrailingSemicolon(";"),n=!1)}return o(o({},e),{writeTrailingSemicolon:function(){n=!0},writeLiteral:function(n){t(),e.writeLiteral(n)},writeStringLiteral:function(n){t(),e.writeStringLiteral(n)},writeSymbol:function(n,r){t(),e.writeSymbol(n,r)},writePunctuation:function(n){t(),e.writePunctuation(n)},writeKeyword:function(n){t(),e.writeKeyword(n)},writeOperator:function(n){t(),e.writeOperator(n)},writeParameter:function(n){t(),e.writeParameter(n)},writeSpace:function(n){t(),e.writeSpace(n)},writeProperty:function(n){t(),e.writeProperty(n)},writeComment:function(n){t(),e.writeComment(n)},writeLine:function(){t(),e.writeLine()},increaseIndent:function(){t(),e.increaseIndent()},decreaseIndent:function(){t(),e.decreaseIndent()}})}function ga(e){return!!e.useCaseSensitiveFileNames&&e.useCaseSensitiveFileNames()}function ya(n){return e.createGetCanonicalFileName(ga(n))}function ha(e,n,t){return n.moduleName||Ea(e,n.fileName,t&&t.fileName)}function va(n,t){return n.getCanonicalFileName(e.getNormalizedAbsolutePath(t,n.getCurrentDirectory()))}function ba(n,t,r){var i=t.getExternalModuleFileFromDeclaration(r);if(i&&!i.isDeclarationFile){var a=lr(r);if(!a||!e.isStringLiteralLike(a)||e.pathIsRelative(a.text)||-1!==va(n,i.path).indexOf(va(n,e.ensureTrailingDirectorySeparator(n.getCommonSourceDirectory()))))return ha(n,i)}}function Ea(n,t,r){var i=function(e){return n.getCanonicalFileName(e)},a=e.toPath(r?e.getDirectoryPath(r):n.getCommonSourceDirectory(),n.getCurrentDirectory(),i),o=e.getNormalizedAbsolutePath(t,n.getCurrentDirectory()),s=e.getRelativePathToDirectoryOrUrl(a,o,a,i,!1),c=Hl(s);return r?e.ensurePathIsNonModuleName(c):c}function Ta(e,n,t){var r,i=n.getCompilerOptions();return r=i.outDir?Hl(Ia(e,n,i.outDir)):Hl(e),r+t}function Sa(e,n){return xa(e,n.getCompilerOptions(),n.getCurrentDirectory(),n.getCommonSourceDirectory(),function(e){return n.getCanonicalFileName(e)})}function xa(e,n,t,r,i){var a=n.declarationDir||n.outDir,o=a?Na(e,a,t,r,i):e;return Hl(o)+".d.ts"}function La(e){return e.outFile||e.out}function Da(n,t){var r,i;if(n.paths)return null!==(r=n.baseUrl)&&void 0!==r?r:e.Debug.checkDefined(n.pathsBasePath||(null===(i=t.getCurrentDirectory)||void 0===i?void 0:i.call(t)),"Encountered 'paths' without a 'baseUrl', config file, or host 'getCurrentDirectory'.")}function Aa(n,t,r){var i=n.getCompilerOptions();if(La(i)){var a=Fc(i),o=i.emitDeclarationOnly||a===e.ModuleKind.AMD||a===e.ModuleKind.System;return e.filter(n.getSourceFiles(),function(t){return(o||!e.isExternalModule(t))&&Ca(t,n,r)})}var s=void 0===t?n.getSourceFiles():[t];return e.filter(s,function(e){return Ca(e,n,r)})}function Ca(e,n,t){var r=n.getCompilerOptions();return!(r.noEmitForJsFiles&&gt(e))&&!e.isDeclarationFile&&!n.isSourceFileFromExternalLibrary(e)&&(t||!(Ze(e)&&n.getResolvedProjectReferenceToRedirect(e.fileName))&&!n.isSourceOfProjectReferenceRedirect(e.fileName))}function Ia(e,n,t){return Na(e,t,n.getCurrentDirectory(),n.getCommonSourceDirectory(),function(e){return n.getCanonicalFileName(e)})}function Na(n,t,r,i,a){var o=e.getNormalizedAbsolutePath(n,r),s=0===a(o).indexOf(a(i));return o=s?o.substring(i.length):o,e.combinePaths(t,o)}function ka(n,t,r,i,a,o){n.writeFile(r,i,a,function(n){t.add(Dc(e.Diagnostics.Could_not_write_file_0_Colon_1,r,n))},o)}function Ra(n,t,r){if(n.length>e.getRootLength(n)&&!r(n)){var i=e.getDirectoryPath(n);Ra(i,t,r),t(n)}}function Oa(n,t,r,i,a,o){try{i(n,t,r)}catch(s){Ra(e.getDirectoryPath(e.normalizePath(n)),a,o),i(n,t,r)}}function wa(n,t){var r=e.getLineStarts(n);return e.computeLineOfPosition(r,t)}function Pa(n,t){return e.computeLineOfPosition(n,t)}function Ma(n){return e.find(n.members,function(n){return e.isConstructorDeclaration(n)&&B(n.body)})}function Fa(e){if(e&&e.parameters.length>0){var n=2===e.parameters.length&&Ua(e.parameters[0]);return e.parameters[n?1:0]}}function Ga(e){var n=Fa(e);return n&&n.type}function Ba(n){if(n.parameters.length&&!e.isJSDocSignature(n)){var t=n.parameters[0];if(Ua(t))return t}}function Ua(e){return Va(e.name)}function Va(e){return!!e&&79===e.kind&&Ka(e)}function ja(n){if(!Va(n))return!1;while(e.isQualifiedName(n.parent)&&n.parent.left===n)n=n.parent;return 179===n.parent.kind}function Ka(e){return 108===e.originalKeywordKind}function Ha(n,t){var r,i,a,o;return vi(t)?(r=t,170===t.kind?a=t:171===t.kind?o=t:e.Debug.fail("Accessor has wrong kind")):e.forEach(n,function(n){if(e.isAccessor(n)&&lo(n)===lo(t)){var s=Ei(n.name),c=Ei(t.name);s===c&&(r?i||(i=n):r=n,170!==n.kind||a||(a=n),171!==n.kind||o||(o=n))}}),{firstAccessor:r,secondAccessor:i,getAccessor:a,setAccessor:o}}function Wa(n){if(ht(n)||!e.isFunctionDeclaration(n)){var t=n.type;return t||!ht(n)?t:e.isJSDocPropertyLikeTag(n)?n.typeExpression&&n.typeExpression.type:e.getJSDocType(n)}}function za(e){return e.type}function Ja(n){return e.isJSDocSignature(n)?n.type&&n.type.typeExpression&&n.type.typeExpression.type:n.type||(ht(n)?e.getJSDocReturnType(n):void 0)}function qa(n){return e.flatMap(e.getJSDocTags(n),function(e){return Xa(e)?e.typeParameters:void 0})}function Xa(n){return e.isJSDocTemplateTag(n)&&!(315===n.parent.kind&&n.parent.tags.some(_r))}function Ya(e){var n=Fa(e);return n&&Wa(n)}function Qa(e,n,t,r){Za(e,n,t.pos,r)}function Za(e,n,t,r){r&&r.length&&t!==r[0].pos&&Pa(e,t)!==Pa(e,r[0].pos)&&n.writeLine()}function $a(e,n,t,r){t!==r&&Pa(e,t)!==Pa(e,r)&&n.writeLine()}function eo(e,n,t,r,i,a,o,s){if(r&&r.length>0){i&&t.writeSpace(" ");for(var c=!1,l=0,u=r;l<u.length;l++){var d=u[l];c&&(t.writeSpace(" "),c=!1),s(e,n,t,d.pos,d.end,o),d.hasTrailingNewLine?t.writeLine():c=!0}c&&a&&t.writeSpace(" ")}}function no(n,t,r,i,a,o,s){var c,l;if(s?0===a.pos&&(c=e.filter(e.getLeadingCommentRanges(n,a.pos),h)):c=e.getLeadingCommentRanges(n,a.pos),c){for(var u=[],d=void 0,p=0,f=c;p<f.length;p++){var m=f[p];if(d){var _=Pa(t,d.end),g=Pa(t,m.pos);if(g>=_+2)break}u.push(m),d=m}if(u.length){_=Pa(t,e.last(u).end);var y=Pa(t,e.skipTrivia(n,a.pos));y>=_+2&&(Qa(t,r,a,c),eo(n,t,r,u,!1,!0,o,i),l={nodePos:a.pos,detachedCommentEndPos:e.last(u).end})}}return l;function h(e){return q(n,e.pos)}}function to(n,t,r,i,a,o){if(42===n.charCodeAt(i+1))for(var s=e.computeLineAndCharacterOfPosition(t,i),c=t.length,l=void 0,u=i,d=s.line;u<a;d++){var p=d+1===c?n.length+1:t[d+1];if(u!==i){void 0===l&&(l=io(n,t[s.line],i));var f=r.getIndent()*fa(),m=f-l+io(n,u,p);if(m>0){var _=m%fa(),g=pa((m-_)/fa());r.rawWrite(g);while(_)r.rawWrite(" "),_--}else r.rawWrite("")}ro(n,a,r,o,u,p),u=p}else r.writeComment(n.substring(i,a))}function ro(n,t,r,i,a,o){var s=Math.min(t,o-1),c=e.trimString(n.substring(a,s));c?(r.writeComment(c),s!==t&&r.writeLine()):r.rawWrite(i)}function io(n,t,r){for(var i=0;t<r&&e.isWhiteSpaceSingleLine(n.charCodeAt(t));t++)9===n.charCodeAt(t)?i+=fa()-i%fa():i++;return i}function ao(e){return 0!==vo(e)}function oo(e){return 0!==Eo(e)}function so(e,n){return!!go(e,n)}function co(e,n){return!!yo(e,n)}function lo(n){return e.isClassElement(n)&&uo(n)||e.isClassStaticBlockDeclaration(n)}function uo(e){return co(e,32)}function po(e){return so(e,16384)}function fo(e){return co(e,128)}function mo(e){return co(e,2)}function _o(e){return so(e,64)}function go(e,n){return vo(e)&n}function yo(e,n){return Eo(e)&n}function ho(e,n,t){return e.kind>=0&&e.kind<=158?0:(536870912&e.modifierFlagsCache||(e.modifierFlagsCache=536870912|xo(e)),!n||4096&e.modifierFlagsCache||!t&&!ht(e)||!e.parent||(e.modifierFlagsCache|=4096|To(e)),-536875009&e.modifierFlagsCache)}function vo(e){return ho(e,!0)}function bo(e){return ho(e,!0,!0)}function Eo(e){return ho(e,!1)}function To(n){var t=0;return n.parent&&!e.isParameter(n)&&(ht(n)&&(e.getJSDocPublicTagNoCache(n)&&(t|=4),e.getJSDocPrivateTagNoCache(n)&&(t|=8),e.getJSDocProtectedTagNoCache(n)&&(t|=16),e.getJSDocReadonlyTagNoCache(n)&&(t|=64),e.getJSDocOverrideTagNoCache(n)&&(t|=16384)),e.getJSDocDeprecatedTagNoCache(n)&&(t|=8192)),t}function So(e){return xo(e)|To(e)}function xo(e){var n=Lo(e.modifiers);return(4&e.flags||79===e.kind&&e.isInJSDocNamespace)&&(n|=1),n}function Lo(e){var n=0;if(e)for(var t=0,r=e;t<r.length;t++){var i=r[t];n|=Do(i.kind)}return n}function Do(e){switch(e){case 124:return 32;case 123:return 4;case 122:return 16;case 121:return 8;case 126:return 128;case 93:return 1;case 134:return 2;case 85:return 2048;case 88:return 512;case 130:return 256;case 143:return 64;case 157:return 16384}return 0}function Ao(n){return n?e.factory.createNodeArray(e.factory.createModifiersFromModifierFlags(n)):void 0}function Co(e){return 56===e||55===e||53===e}function Io(e){return 75===e||76===e||77===e}function No(e){return Io(e.operatorToken.kind)}function ko(e){return e>=63&&e<=78}function Ro(e){var n=Oo(e);return n&&!n.isImplements?n.class:void 0}function Oo(n){return e.isExpressionWithTypeArguments(n)&&e.isHeritageClause(n.parent)&&e.isClassLike(n.parent.parent)?{class:n.parent.parent,isImplements:117===n.parent.token}:void 0}function wo(n,t){return e.isBinaryExpression(n)&&(t?63===n.operatorToken.kind:ko(n.operatorToken.kind))&&e.isLeftHandSideExpression(n.left)}function Po(e){return wo(e.parent)&&e.parent.left===e}function Mo(e){if(wo(e,!0)){var n=e.left.kind;return 203===n||202===n}return!1}function Fo(e){return void 0!==Ro(e)}function Go(e){return 79===e.kind||Vo(e)}function Bo(e){switch(e.kind){case 79:return e;case 159:do{e=e.left}while(79!==e.kind);return e;case 204:do{e=e.expression}while(79!==e.kind);return e}}function Uo(e){return 79===e.kind||108===e.kind||106===e.kind||229===e.kind||204===e.kind&&Uo(e.expression)||210===e.kind&&Uo(e.expression)}function Vo(n){return e.isPropertyAccessExpression(n)&&e.isIdentifier(n.name)&&Go(n.expression)}function jo(n){if(e.isPropertyAccessExpression(n)){var t=jo(n.expression);if(void 0!==t)return t+"."+Be(n.name)}else if(e.isElementAccessExpression(n)){t=jo(n.expression);if(void 0!==t&&e.isPropertyName(n.argumentExpression))return t+"."+Ei(n.argumentExpression)}else if(e.isIdentifier(n))return e.unescapeLeadingUnderscores(n.escapedText)}function Ko(e){return zt(e)&&"prototype"===$t(e)}function Ho(e){return 159===e.parent.kind&&e.parent.right===e||204===e.parent.kind&&e.parent.name===e}function Wo(n){return e.isQualifiedName(n.parent)&&n.parent.right===n||e.isPropertyAccessExpression(n.parent)&&n.parent.name===n||e.isJSDocMemberName(n.parent)&&n.parent.right===n}function zo(e){return 203===e.kind&&0===e.properties.length}function Jo(e){return 202===e.kind&&0===e.elements.length}function qo(e){if(Xo(e)&&e.declarations)for(var n=0,t=e.declarations;n<t.length;n++){var r=t[n];if(r.localSymbol)return r.localSymbol}}function Xo(n){return n&&e.length(n.declarations)>0&&co(n.declarations[0],512)}function Yo(n){return e.find(e.supportedTSExtensionsForExtractExtension,function(t){return e.fileExtensionIs(n,t)})}function Qo(n){for(var t=[],r=n.length,i=0;i<r;i++){var a=n.charCodeAt(i);a<128?t.push(a):a<2048?(t.push(a>>6|192),t.push(63&a|128)):a<65536?(t.push(a>>12|224),t.push(a>>6&63|128),t.push(63&a|128)):a<131072?(t.push(a>>18|240),t.push(a>>12&63|128),t.push(a>>6&63|128),t.push(63&a|128)):e.Debug.assert(!1,"Unexpected code point")}return t}e.getIndentString=pa,e.getIndentSize=fa,e.createTextWriter=ma,e.getTrailingSemicolonDeferringWriter=_a,e.hostUsesCaseSensitiveFileNames=ga,e.hostGetCanonicalFileName=ya,e.getResolvedExternalModuleName=ha,e.getExternalModuleNameFromDeclaration=ba,e.getExternalModuleNameFromPath=Ea,e.getOwnEmitOutputFilePath=Ta,e.getDeclarationEmitOutputFilePath=Sa,e.getDeclarationEmitOutputFilePathWorker=xa,e.outFile=La,e.getPathsBasePath=Da,e.getSourceFilesToEmit=Aa,e.sourceFileMayBeEmitted=Ca,e.getSourceFilePathInNewDir=Ia,e.getSourceFilePathInNewDirWorker=Na,e.writeFile=ka,e.writeFileEnsuringDirectories=Oa,e.getLineOfLocalPosition=wa,e.getLineOfLocalPositionFromLineMap=Pa,e.getFirstConstructorWithBody=Ma,e.getSetAccessorValueParameter=Fa,e.getSetAccessorTypeAnnotationNode=Ga,e.getThisParameter=Ba,e.parameterIsThisKeyword=Ua,e.isThisIdentifier=Va,e.isThisInTypeQuery=ja,e.identifierIsThisKeyword=Ka,e.getAllAccessorDeclarations=Ha,e.getEffectiveTypeAnnotationNode=Wa,e.getTypeAnnotationNode=za,e.getEffectiveReturnTypeNode=Ja,e.getJSDocTypeParameterDeclarations=qa,e.getEffectiveSetAccessorTypeAnnotationNode=Ya,e.emitNewLineBeforeLeadingComments=Qa,e.emitNewLineBeforeLeadingCommentsOfPosition=Za,e.emitNewLineBeforeLeadingCommentOfPosition=$a,e.emitComments=eo,e.emitDetachedComments=no,e.writeCommentRange=to,e.hasEffectiveModifiers=ao,e.hasSyntacticModifiers=oo,e.hasEffectiveModifier=so,e.hasSyntacticModifier=co,e.isStatic=lo,e.hasStaticModifier=uo,e.hasOverrideModifier=po,e.hasAbstractModifier=fo,e.hasAmbientModifier=mo,e.hasEffectiveReadonlyModifier=_o,e.getSelectedEffectiveModifierFlags=go,e.getSelectedSyntacticModifierFlags=yo,e.getEffectiveModifierFlags=vo,e.getEffectiveModifierFlagsAlwaysIncludeJSDoc=bo,e.getSyntacticModifierFlags=Eo,e.getEffectiveModifierFlagsNoCache=So,e.getSyntacticModifierFlagsNoCache=xo,e.modifiersToFlags=Lo,e.modifierToFlag=Do,e.createModifiers=Ao,e.isLogicalOperator=Co,e.isLogicalOrCoalescingAssignmentOperator=Io,e.isLogicalOrCoalescingAssignmentExpression=No,e.isAssignmentOperator=ko,e.tryGetClassExtendingExpressionWithTypeArguments=Ro,e.tryGetClassImplementingOrExtendingExpressionWithTypeArguments=Oo,e.isAssignmentExpression=wo,e.isLeftHandSideOfAssignment=Po,e.isDestructuringAssignment=Mo,e.isExpressionWithTypeArgumentsInClassExtendsClause=Fo,e.isEntityNameExpression=Go,e.getFirstIdentifier=Bo,e.isDottedName=Uo,e.isPropertyAccessEntityNameExpression=Vo,e.tryGetPropertyAccessOrIdentifierToString=jo,e.isPrototypeAccess=Ko,e.isRightSideOfQualifiedNameOrPropertyAccess=Ho,e.isRightSideOfQualifiedNameOrPropertyAccessOrJSDocMemberName=Wo,e.isEmptyObjectLiteral=zo,e.isEmptyArrayLiteral=Jo,e.getLocalSymbolForExportDefault=qo,e.tryExtractTSExtension=Yo;var Zo="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";function $o(e){var n,t,r,i,a="",o=Qo(e),s=0,c=o.length;while(s<c)n=o[s]>>2,t=(3&o[s])<<4|o[s+1]>>4,r=(15&o[s+1])<<2|o[s+2]>>6,i=63&o[s+2],s+1>=c?r=i=64:s+2>=c&&(i=64),a+=Zo.charAt(n)+Zo.charAt(t)+Zo.charAt(r)+Zo.charAt(i),s+=3;return a}function es(e){var n="",t=0,r=e.length;while(t<r){var i=e[t];if(i<128)n+=String.fromCharCode(i),t++;else if(192===(192&i)){var a=63&i;t++;var o=e[t];while(128===(192&o))a=a<<6|63&o,t++,o=e[t];n+=String.fromCharCode(a)}else n+=String.fromCharCode(i),t++}return n}function ns(e,n){return e&&e.base64encode?e.base64encode(n):$o(n)}function ts(e,n){if(e&&e.base64decode)return e.base64decode(n);var t=n.length,r=[],i=0;while(i<t){if(n.charCodeAt(i)===Zo.charCodeAt(64))break;var a=Zo.indexOf(n[i]),o=Zo.indexOf(n[i+1]),s=Zo.indexOf(n[i+2]),c=Zo.indexOf(n[i+3]),l=(63&a)<<2|o>>4&3,u=(15&o)<<4|s>>2&15,d=(3&s)<<6|63&c;0===u&&0!==s?r.push(l):0===d&&0!==c?r.push(l,u):r.push(l,u,d),i+=4}return es(r)}function rs(n,t){try{var r=t.readFile(n);if(!r)return{};var i=e.parseConfigFileTextToJson(n,r);return i.error?{}:i.config}catch(a){return{}}}function is(e,n){return!n.directoryExists||n.directoryExists(e)}e.convertToBase64=$o,e.base64encode=ns,e.base64decode=ts,e.readJson=rs,e.directoryProbablyExists=is;var as,os="\r\n",ss="\n";function cs(n,t){switch(n.newLine){case 0:return os;case 1:return ss}return t?t():e.sys?e.sys.newLine:os}function ls(n,t){return void 0===t&&(t=n),e.Debug.assert(t>=n||-1===t),{pos:n,end:t}}function us(e,n){return ls(e.pos,n)}function ds(e,n){return ls(n,e.end)}function ps(e){return e.decorators&&e.decorators.length>0?ds(e,e.decorators.end):e}function fs(e){return e.modifiers&&e.modifiers.length>0?ds(e,e.modifiers.end):ps(e)}function ms(e){return e.pos===e.end}function _s(n,t){return ls(n,n+e.tokenToString(t).length)}function gs(e,n){return vs(e,e,n)}function ys(e,n,t){return xs(Ls(e,t,!1),Ls(n,t,!1),t)}function hs(e,n,t){return xs(e.end,n.end,t)}function vs(e,n,t){return xs(Ls(e,t,!1),n.end,t)}function bs(e,n,t){return xs(e.end,Ls(n,t,!1),t)}function Es(n,t,r,i){var a=Ls(t,r,i);return e.getLinesBetweenPositions(r,n.end,a)}function Ts(n,t,r){return e.getLinesBetweenPositions(r,n.end,t.end)}function Ss(e,n){return!xs(e.pos,e.end,n)}function xs(n,t,r){return 0===e.getLinesBetweenPositions(r,n,t)}function Ls(n,t,r){return Yl(n.pos)?-1:e.skipTrivia(t.text,n.pos,!1,r)}function Ds(n,t,r,i){var a=e.skipTrivia(r.text,n,!1,i),o=Cs(a,t,r);return e.getLinesBetweenPositions(r,null!==o&&void 0!==o?o:t,a)}function As(n,t,r,i){var a=e.skipTrivia(r.text,n,!1,i);return e.getLinesBetweenPositions(r,n,Math.min(t,a))}function Cs(n,t,r){void 0===t&&(t=0);while(n-- >t)if(!e.isWhiteSpaceLike(r.text.charCodeAt(n)))return n}function Is(n){var t=e.getParseTreeNode(n);if(t)switch(t.parent.kind){case 258:case 259:return t===t.parent.name}return!1}function Ns(n){return e.filter(n.declarations,ks)}function ks(e){return void 0!==e.initializer}function Rs(e){return e.watch&&e.hasOwnProperty("watch")}function Os(e){e.close()}function ws(e){return 33554432&e.flags?e.checkFlags:0}function Ps(n,t){if(void 0===t&&(t=!1),n.valueDeclaration){var r=t&&n.declarations&&e.find(n.declarations,function(e){return 171===e.kind})||n.valueDeclaration,i=e.getCombinedModifierFlags(r);return n.parent&&32&n.parent.flags?i:-29&i}if(6&ws(n)){var a=n.checkFlags,o=1024&a?8:256&a?4:16,s=2048&a?32:0;return o|s}return 4194304&n.flags?36:0}function Ms(e,n){return 2097152&e.flags?n.getAliasedSymbol(e):e}function Fs(e){return e.exportSymbol?e.exportSymbol.flags|e.flags:e.flags}function Gs(e){return 1===Us(e)}function Bs(e){return 0!==Us(e)}function Us(e){var n=e.parent;if(!n)return 0;switch(n.kind){case 210:return Us(n);case 218:case 217:var t=n.operator;return 45===t||46===t?s():0;case 219:var r=n,i=r.left,a=r.operatorToken;return i===e&&ko(a.kind)?63===a.kind?1:s():0;case 204:return n.name!==e?0:Us(n);case 291:var o=Us(n.parent);return e===n.name?Vs(o):o;case 292:return e===n.objectAssignmentInitializer?0:Us(n.parent);case 202:return Us(n);default:return 0}function s(){return n.parent&&236===Br(n.parent).kind?1:2}}function Vs(n){switch(n){case 0:return 1;case 1:return 0;case 2:return 2;default:return e.Debug.assertNever(n)}}function js(e,n){if(!e||!n||Object.keys(e).length!==Object.keys(n).length)return!1;for(var t in e)if("object"===typeof e[t]){if(!js(e[t],n[t]))return!1}else if("function"!==typeof e[t]&&e[t]!==n[t])return!1;return!0}function Ks(e,n){e.forEach(n),e.clear()}function Hs(e,n,t){var r=t.onDeleteValue,i=t.onExistingValue;e.forEach(function(t,a){var o=n.get(a);void 0===o?(e.delete(a),r(t,a)):i&&i(t,o,a)})}function Ws(e,n,t){Hs(e,n,t);var r=t.createNewValue;n.forEach(function(n,t){e.has(t)||e.set(t,r(t,n))})}function zs(e){if(32&e.flags){var n=Js(e);return!!n&&co(n,128)}return!1}function Js(n){var t;return null===(t=n.declarations)||void 0===t?void 0:t.find(e.isClassLike)}function qs(e){return 3899393&e.flags?e.objectFlags:0}function Xs(e,n){return 0!==n.getSignaturesOfType(e,0).length||0!==n.getSignaturesOfType(e,1).length}function Ys(n,t){return!!e.forEachAncestorDirectory(n,function(e){return!!t(e)||void 0})}function Qs(n){return!!n&&!!n.declarations&&!!n.declarations[0]&&e.isNamespaceExportDeclaration(n.declarations[0])}function Zs(n){var t=n.moduleSpecifier;return e.isStringLiteral(t)?t.text:te(t)}function $s(n){var t;return e.forEachChild(n,function(e){B(e)&&(t=e)},function(e){for(var n=e.length-1;n>=0;n--)if(B(e[n])){t=e[n];break}}),t}function ec(e,n,t){return void 0===t&&(t=!0),!e.has(n)&&(e.set(n,t),!0)}function nc(n){return e.isClassLike(n)||e.isInterfaceDeclaration(n)||e.isTypeLiteralNode(n)}function tc(e){return e>=175&&e<=198||129===e||153===e||145===e||156===e||146===e||132===e||148===e||149===e||114===e||151===e||142===e||226===e||307===e||308===e||309===e||310===e||311===e||312===e||313===e}function rc(e){return 204===e.kind||205===e.kind}function ic(n){return 204===n.kind?n.name:(e.Debug.assert(205===n.kind),n.argumentExpression)}function ac(e){switch(e.kind){case"text":case"internal":return!0;default:return!1}}function oc(e){return 267===e.kind||271===e.kind}function sc(e){while(rc(e))e=e.expression;return e}function cc(e,n){while(1){switch(e.kind){case 218:e=e.operand;continue;case 219:e=e.left;continue;case 220:e=e.condition;continue;case 208:e=e.tag;continue;case 206:if(n)return e;case 227:case 205:case 204:case 228:case 345:e=e.expression;continue}return e}}function lc(e,n){this.flags=e,this.escapedName=n,this.declarations=void 0,this.valueDeclaration=void 0,this.id=void 0,this.mergeId=void 0,this.parent=void 0}function uc(n,t){this.flags=t,(e.Debug.isDebugging||e.tracing)&&(this.checker=n)}function dc(n,t){this.flags=t,e.Debug.isDebugging&&(this.checker=n)}function pc(e,n,t){this.pos=n,this.end=t,this.kind=e,this.id=0,this.flags=0,this.modifierFlagsCache=0,this.transformFlags=0,this.parent=void 0,this.original=void 0}function fc(e,n,t){this.pos=n,this.end=t,this.kind=e,this.id=0,this.flags=0,this.transformFlags=0,this.parent=void 0}function mc(e,n,t){this.pos=n,this.end=t,this.kind=e,this.id=0,this.flags=0,this.transformFlags=0,this.parent=void 0,this.original=void 0,this.flowNode=void 0}function _c(e,n,t){this.fileName=e,this.text=n,this.skipTrivia=t||function(e){return e}}function gc(n){e.objectAllocator=n}function yc(n,t,r){return void 0===r&&(r=0),n.replace(/{(\d+)}/g,function(n,i){return""+e.Debug.checkDefined(t[+i+r])})}function hc(n){e.localizedDiagnosticMessages=n}function vc(n){return e.localizedDiagnosticMessages&&e.localizedDiagnosticMessages[n.key]||n.message}function bc(e,n,t,r){He(void 0,n,t);var i=vc(r);return arguments.length>4&&(i=yc(i,arguments,4)),{file:void 0,start:n,length:t,messageText:i,category:r.category,code:r.code,reportsUnnecessary:r.reportsUnnecessary,fileName:e}}function Ec(e){return void 0===e.file&&void 0!==e.start&&void 0!==e.length&&"string"===typeof e.fileName}function Tc(n,t){var r=t.fileName||"",i=t.text.length;e.Debug.assertEqual(n.fileName,r),e.Debug.assertLessThanOrEqual(n.start,i),e.Debug.assertLessThanOrEqual(n.start+n.length,i);var a={file:t,start:n.start,length:n.length,messageText:n.messageText,category:n.category,code:n.code,reportsUnnecessary:n.reportsUnnecessary};if(n.relatedInformation){a.relatedInformation=[];for(var o=0,s=n.relatedInformation;o<s.length;o++){var c=s[o];Ec(c)&&c.fileName===r?(e.Debug.assertLessThanOrEqual(c.start,i),e.Debug.assertLessThanOrEqual(c.start+c.length,i),a.relatedInformation.push(Tc(c,t))):a.relatedInformation.push(c)}}return a}function Sc(e,n){for(var t=[],r=0,i=e;r<i.length;r++){var a=i[r];t.push(Tc(a,n))}return t}function xc(e,n,t,r){He(e,n,t);var i=vc(r);return arguments.length>4&&(i=yc(i,arguments,4)),{file:e,start:n,length:t,messageText:i,category:r.category,code:r.code,reportsUnnecessary:r.reportsUnnecessary,reportsDeprecated:r.reportsDeprecated}}function Lc(e,n){var t=vc(n);return arguments.length>2&&(t=yc(t,arguments,2)),t}function Dc(e){var n=vc(e);return arguments.length>1&&(n=yc(n,arguments,1)),{file:void 0,start:void 0,length:void 0,messageText:n,category:e.category,code:e.code,reportsUnnecessary:e.reportsUnnecessary,reportsDeprecated:e.reportsDeprecated}}function Ac(e,n){return{file:void 0,start:void 0,length:void 0,code:e.code,category:e.category,messageText:e.next?e:e.messageText,relatedInformation:n}}function Cc(e,n){var t=vc(n);return arguments.length>2&&(t=yc(t,arguments,2)),{messageText:t,category:n.category,code:n.code,next:void 0===e||Array.isArray(e)?e:[e]}}function Ic(e,n){var t=e;while(t.next)t=t.next[0];t.next=[n]}function Nc(e){return e.file?e.file.path:void 0}function kc(e,n){return Rc(e,n)||Oc(e,n)||0}function Rc(n,t){return e.compareStringsCaseSensitive(Nc(n),Nc(t))||e.compareValues(n.start,t.start)||e.compareValues(n.length,t.length)||e.compareValues(n.code,t.code)||wc(n.messageText,t.messageText)||0}function Oc(n,t){return n.relatedInformation||t.relatedInformation?n.relatedInformation&&t.relatedInformation?e.compareValues(n.relatedInformation.length,t.relatedInformation.length)||e.forEach(n.relatedInformation,function(e,n){var r=t.relatedInformation[n];return kc(e,r)})||0:n.relatedInformation?-1:1:0}function wc(n,t){if("string"===typeof n&&"string"===typeof t)return e.compareStringsCaseSensitive(n,t);if("string"===typeof n)return-1;if("string"===typeof t)return 1;var r=e.compareStringsCaseSensitive(n.messageText,t.messageText);if(r)return r;if(!n.next&&!t.next)return 0;if(!n.next)return-1;if(!t.next)return 1;for(var i=Math.min(n.next.length,t.next.length),a=0;a<i;a++)if(r=wc(n.next[a],t.next[a]),r)return r;return n.next.length<t.next.length?-1:n.next.length>t.next.length?1:0}function Pc(e){return 4===e||2===e||1===e||6===e?1:0}function Mc(e){return e.target||0}function Fc(n){return"number"===typeof n.module?n.module:Mc(n)>=2?e.ModuleKind.ES2015:e.ModuleKind.CommonJS}function Gc(n){var t=n.moduleResolution;return void 0===t&&(t=Fc(n)===e.ModuleKind.CommonJS?e.ModuleResolutionKind.NodeJs:e.ModuleResolutionKind.Classic),t}function Bc(n){switch(Fc(n)){case e.ModuleKind.CommonJS:case e.ModuleKind.AMD:case e.ModuleKind.ES2015:case e.ModuleKind.ES2020:case e.ModuleKind.ESNext:return!0;default:return!1}}function Uc(e){return!1===e.allowUnreachableCode}function Vc(e){return!1===e.allowUnusedLabels}function jc(e){return!(!Hc(e)||!e.declarationMap)}function Kc(n){var t=Fc(n);return void 0!==n.allowSyntheticDefaultImports?n.allowSyntheticDefaultImports:n.esModuleInterop||t===e.ModuleKind.System}function Hc(e){return!(!e.declaration&&!e.composite)}function Wc(e){return!(!e.preserveConstEnums&&!e.isolatedModules)}function zc(e){return!(!e.incremental&&!e.composite)}function Jc(e,n){return void 0===e[n]?!!e.strict:!!e[n]}function qc(e){return void 0===e.allowJs?!!e.checkJs:e.allowJs}function Xc(e){return void 0===e.useDefineForClassFields?99===e.target:e.useDefineForClassFields}function Yc(n,t){return f(t,n,e.semanticDiagnosticsOptionDeclarations)}function Qc(n,t){return f(t,n,e.affectsEmitOptionDeclarations)}function Zc(e,n){return n.strictFlag?Jc(e,n.name):e[n.name]}function $c(e){var n=e.jsx;return 2===n||4===n||5===n}function el(n,t){var r=null===t||void 0===t?void 0:t.pragmas.get("jsximportsource"),i=e.isArray(r)?r[r.length-1]:r;return 4===n.jsx||5===n.jsx||n.jsxImportSource||i?(null===i||void 0===i?void 0:i.arguments.factory)||n.jsxImportSource||"react":void 0}function nl(e,n){return e?e+"/"+(5===n.jsx?"jsx-dev-runtime":"jsx-runtime"):void 0}function tl(e){for(var n=!1,t=0;t<e.length;t++)if(42===e.charCodeAt(t)){if(n)return!1;n=!0}return!0}function rl(n,t){var r,i,a,o=!1;return{getSymlinkedFiles:function(){return a},getSymlinkedDirectories:function(){return r},getSymlinkedDirectoriesByRealpath:function(){return i},setSymlinkedFile:function(n,t){return(a||(a=new e.Map)).set(n,t)},setSymlinkedDirectory:function(a,o){var s=e.toPath(a,n,t);ku(s)||(s=e.ensureTrailingDirectorySeparator(s),!1===o||(null===r||void 0===r?void 0:r.has(s))||(i||(i=e.createMultiMap())).add(e.ensureTrailingDirectorySeparator(o.realPath),a),(r||(r=new e.Map)).set(s,o))},setSymlinkedDirectoryFromSymlinkedFile:function(r,i){this.setSymlinkedFile(e.toPath(r,n,t),i);var a=il(i,r,n,t)||e.emptyArray,o=a[0],s=a[1];o&&s&&this.setSymlinkedDirectory(s,{real:o,realPath:e.toPath(o,n,t)})},setSymlinksFromResolutions:function(n,t){var r,i=this;e.Debug.assert(!o),o=!0;for(var a=0,c=n;a<c.length;a++){var l=c[a];null===(r=l.resolvedModules)||void 0===r||r.forEach(function(e){return s(i,e)})}null===t||void 0===t||t.forEach(function(e){return s(i,e)})},hasProcessedResolutions:function(){return o}};function s(r,i){if(i&&i.originalPath&&i.resolvedFileName){var a=i.resolvedFileName,o=i.originalPath;r.setSymlinkedFile(e.toPath(o,n,t),a);var s=il(a,o,n,t)||e.emptyArray,c=s[0],l=s[1];c&&l&&r.setSymlinkedDirectory(l,{real:c,realPath:e.toPath(c,n,t)})}}}function il(n,t,r,i){var a=e.getPathComponents(e.getNormalizedAbsolutePath(n,r)),o=e.getPathComponents(e.getNormalizedAbsolutePath(t,r)),s=!1;while(!al(a[a.length-2],i)&&!al(o[o.length-2],i)&&i(a[a.length-1])===i(o[o.length-1]))a.pop(),o.pop(),s=!0;return s?[e.getPathFromPathComponents(a),e.getPathFromPathComponents(o)]:void 0}function al(n,t){return"node_modules"===t(n)||e.startsWith(n,"@")}function ol(n){return e.isAnyDirectorySeparator(n.charCodeAt(0))?n.slice(1):void 0}function sl(n,t,r){var i=e.tryRemovePrefix(n,t,r);return void 0===i?void 0:ol(i)}e.getNewLineCharacter=cs,e.createRange=ls,e.moveRangeEnd=us,e.moveRangePos=ds,e.moveRangePastDecorators=ps,e.moveRangePastModifiers=fs,e.isCollapsedRange=ms,e.createTokenRange=_s,e.rangeIsOnSingleLine=gs,e.rangeStartPositionsAreOnSameLine=ys,e.rangeEndPositionsAreOnSameLine=hs,e.rangeStartIsOnSameLineAsRangeEnd=vs,e.rangeEndIsOnSameLineAsRangeStart=bs,e.getLinesBetweenRangeEndAndRangeStart=Es,e.getLinesBetweenRangeEndPositions=Ts,e.isNodeArrayMultiLine=Ss,e.positionsAreOnSameLine=xs,e.getStartPositionOfRange=Ls,e.getLinesBetweenPositionAndPrecedingNonWhitespaceCharacter=Ds,e.getLinesBetweenPositionAndNextNonWhitespaceCharacter=As,e.isDeclarationNameOfEnumOrNamespace=Is,e.getInitializedVariables=Ns,e.isWatchSet=Rs,e.closeFileWatcher=Os,e.getCheckFlags=ws,e.getDeclarationModifierFlagsFromSymbol=Ps,e.skipAlias=Ms,e.getCombinedLocalAndExportSymbolFlags=Fs,e.isWriteOnlyAccess=Gs,e.isWriteAccess=Bs,function(e){e[e["Read"]=0]="Read",e[e["Write"]=1]="Write",e[e["ReadWrite"]=2]="ReadWrite"}(as||(as={})),e.compareDataObjects=js,e.clearMap=Ks,e.mutateMapSkippingNewValues=Hs,e.mutateMap=Ws,e.isAbstractConstructorSymbol=zs,e.getClassLikeDeclarationOfSymbol=Js,e.getObjectFlags=qs,e.typeHasCallOrConstructSignatures=Xs,e.forSomeAncestorDirectory=Ys,e.isUMDExportSymbol=Qs,e.showModuleSpecifier=Zs,e.getLastChild=$s,e.addToSeen=ec,e.isObjectTypeDeclaration=nc,e.isTypeNodeKind=tc,e.isAccessExpression=rc,e.getNameOfAccessExpression=ic,e.isBundleFileTextLike=ac,e.isNamedImportsOrExports=oc,e.getLeftmostAccessExpression=sc,e.getLeftmostExpression=cc,e.objectAllocator={getNodeConstructor:function(){return pc},getTokenConstructor:function(){return fc},getIdentifierConstructor:function(){return mc},getPrivateIdentifierConstructor:function(){return pc},getSourceFileConstructor:function(){return pc},getSymbolConstructor:function(){return lc},getTypeConstructor:function(){return uc},getSignatureConstructor:function(){return dc},getSourceMapSourceConstructor:function(){return _c}},e.setObjectAllocator=gc,e.formatStringFromArgs=yc,e.setLocalizedDiagnosticMessages=hc,e.getLocaleSpecificMessage=vc,e.createDetachedDiagnostic=bc,e.attachFileToDiagnostics=Sc,e.createFileDiagnostic=xc,e.formatMessage=Lc,e.createCompilerDiagnostic=Dc,e.createCompilerDiagnosticFromMessageChain=Ac,e.chainDiagnosticMessages=Cc,e.concatenateDiagnosticMessageChains=Ic,e.compareDiagnostics=kc,e.compareDiagnosticsSkipRelatedInformation=Rc,e.getLanguageVariant=Pc,e.getEmitScriptTarget=Mc,e.getEmitModuleKind=Fc,e.getEmitModuleResolutionKind=Gc,e.hasJsonModuleEmitEnabled=Bc,e.unreachableCodeIsError=Uc,e.unusedLabelIsError=Vc,e.getAreDeclarationMapsEnabled=jc,e.getAllowSyntheticDefaultImports=Kc,e.getEmitDeclarations=Hc,e.shouldPreserveConstEnums=Wc,e.isIncrementalCompilation=zc,e.getStrictOptionValue=Jc,e.getAllowJSCompilerOption=qc,e.getUseDefineForClassFields=Xc,e.compilerOptionsAffectSemanticDiagnostics=Yc,e.compilerOptionsAffectEmit=Qc,e.getCompilerOptionValue=Zc,e.getJSXTransformEnabled=$c,e.getJSXImplicitImportBase=el,e.getJSXRuntimeImport=nl,e.hasZeroOrOneAsteriskCharacter=tl,e.createSymlinkCache=rl,e.tryRemoveDirectoryPrefix=sl;var cl=/[^\w\s\/]/g;function ll(e){return e.replace(cl,ul)}function ul(e){return"\\"+e}e.regExpEscape=ll;var dl=[42,63];e.commonPackageFolders=["node_modules","bower_components","jspm_packages"];var pl="(?!("+e.commonPackageFolders.join("|")+")(/|$))",fl={singleAsteriskRegexFragment:"([^./]|(\\.(?!min\\.js$))?)*",doubleAsteriskRegexFragment:"(/"+pl+"[^/.][^/]*)*?",replaceWildcardCharacter:function(e){return Tl(e,fl.singleAsteriskRegexFragment)}},ml={singleAsteriskRegexFragment:"[^/]*",doubleAsteriskRegexFragment:"(/"+pl+"[^/.][^/]*)*?",replaceWildcardCharacter:function(e){return Tl(e,ml.singleAsteriskRegexFragment)}},_l={singleAsteriskRegexFragment:"[^/]*",doubleAsteriskRegexFragment:"(/.+?)?",replaceWildcardCharacter:function(e){return Tl(e,_l.singleAsteriskRegexFragment)}},gl={files:fl,directories:ml,exclude:_l};function yl(e,n,t){var r=hl(e,n,t);if(r&&r.length){var i=r.map(function(e){return"("+e+")"}).join("|"),a="exclude"===t?"($|/)":"$";return"^("+i+")"+a}}function hl(n,t,r){if(void 0!==n&&0!==n.length)return e.flatMap(n,function(e){return e&&El(e,t,r,gl[r])})}function vl(e){return!/[.*?]/.test(e)}function bl(e,n,t){var r=e&&El(e,n,t,gl[t]);return r&&"^("+r+")"+("exclude"===t?"($|/)":"$")}function El(n,t,r,i){var a=i.singleAsteriskRegexFragment,o=i.doubleAsteriskRegexFragment,s=i.replaceWildcardCharacter,c="",l=!1,u=e.getNormalizedPathComponents(n,t),d=e.last(u);if("exclude"===r||"**"!==d){u[0]=e.removeTrailingDirectorySeparator(u[0]),vl(d)&&u.push("**","*");for(var p=0,f=0,m=u;f<m.length;f++){var _=m[f];if("**"===_)c+=o;else if("directories"===r&&(c+="(",p++),l&&(c+=e.directorySeparator),"exclude"!==r){var g="";42===_.charCodeAt(0)?(g+="([^./]"+a+")?",_=_.substr(1)):63===_.charCodeAt(0)&&(g+="[^./]",_=_.substr(1)),g+=_.replace(cl,s),g!==_&&(c+=pl),c+=g}else c+=_.replace(cl,s);l=!0}while(p>0)c+=")?",p--;return c}}function Tl(e,n){return"*"===e?n:"?"===e?"[^/]":"\\"+e}function Sl(n,t,r,i,a){n=e.normalizePath(n),a=e.normalizePath(a);var o=e.combinePaths(a,n);return{includeFilePatterns:e.map(hl(r,o,"files"),function(e){return"^"+e+"$"}),includeFilePattern:yl(r,o,"files"),includeDirectoryPattern:yl(r,o,"directories"),excludePattern:yl(t,o,"exclude"),basePaths:Dl(n,r,i)}}function xl(e,n){return new RegExp(e,n?"":"i")}function Ll(n,t,r,i,a,o,s,c,l,u){n=e.normalizePath(n),o=e.normalizePath(o);for(var d=Sl(n,r,i,a,o),p=d.includeFilePatterns&&d.includeFilePatterns.map(function(e){return xl(e,a)}),f=d.includeDirectoryPattern&&xl(d.includeDirectoryPattern,a),m=d.excludePattern&&xl(d.excludePattern,a),_=p?p.map(function(){return[]}):[[]],g=new e.Map,y=e.createGetCanonicalFileName(a),h=0,v=d.basePaths;h<v.length;h++){var b=v[h];u(b)&&E(b,e.combinePaths(o,b),s)}return e.flatten(_);function E(n,r,i){var a=y(l(r));if(!g.has(a)){g.set(a,!0);for(var o=c(n),s=o.files,u=o.directories,d=function(i){var a=e.combinePaths(n,i),o=e.combinePaths(r,i);if(t&&!e.fileExtensionIsOneOf(a,t))return"continue";if(m&&m.test(o))return"continue";if(p){var s=e.findIndex(p,function(e){return e.test(o)});-1!==s&&_[s].push(a)}else _[0].push(a)},h=0,v=e.sort(s,e.compareStringsCaseSensitive);h<v.length;h++){var b=v[h];d(b)}if(void 0===i||(i--,0!==i))for(var T=0,S=e.sort(u,e.compareStringsCaseSensitive);T<S.length;T++){b=S[T];var x=e.combinePaths(n,b),L=e.combinePaths(r,b);f&&!f.test(L)||m&&m.test(L)||E(x,L,i)}}}}function Dl(n,t,r){var i=[n];if(t){for(var a=[],o=0,s=t;o<s.length;o++){var c=s[o],l=e.isRootedDiskPath(c)?c:e.normalizePath(e.combinePaths(n,c));a.push(Al(l))}a.sort(e.getStringComparer(!r));for(var u=function(t){e.every(i,function(i){return!e.containsPath(i,t,n,!r)})&&i.push(t)},d=0,p=a;d<p.length;d++){var f=p[d];u(f)}}return i}function Al(n){var t=e.indexOfAnyCharCode(n,dl);return t<0?e.hasExtension(n)?e.removeTrailingDirectorySeparator(e.getDirectoryPath(n)):n:n.substring(0,n.lastIndexOf(e.directorySeparator,t))}function Cl(e,n){return n||Il(e)||3}function Il(e){var n=e.substr(e.lastIndexOf("."));switch(n.toLowerCase()){case".js":return 1;case".jsx":return 2;case".ts":return 3;case".tsx":return 4;case".json":return 6;default:return 0}}e.getRegularExpressionForWildcard=yl,e.getRegularExpressionsForWildcards=hl,e.isImplicitGlob=vl,e.getPatternFromSpec=bl,e.getFileMatcherPatterns=Sl,e.getRegexFromPattern=xl,e.matchFiles=Ll,e.ensureScriptKind=Cl,e.getScriptKindFromFileName=Il,e.supportedTSExtensions=[".ts",".tsx",".d.ts"],e.supportedTSExtensionsWithJson=[".ts",".tsx",".d.ts",".json"],e.supportedTSExtensionsForExtractExtension=[".d.ts",".ts",".tsx"],e.supportedJSExtensions=[".js",".jsx"],e.supportedJSAndJsonExtensions=[".js",".jsx",".json"];var Nl=a(a([],e.supportedTSExtensions,!0),e.supportedJSExtensions,!0),kl=a(a(a([],e.supportedTSExtensions,!0),e.supportedJSExtensions,!0),[".json"],!1);function Rl(n,t){var r=n&&qc(n);if(!t||0===t.length)return r?Nl:e.supportedTSExtensions;var i=a(a([],r?Nl:e.supportedTSExtensions,!0),e.mapDefined(t,function(e){return 7===e.scriptKind||r&&wl(e.scriptKind)?e.extension:void 0}),!0);return e.deduplicate(i,e.equateStringsCaseSensitive,e.compareStringsCaseSensitive)}function Ol(n,t){return n&&n.resolveJsonModule?t===Nl?kl:t===e.supportedTSExtensions?e.supportedTSExtensionsWithJson:a(a([],t,!0),[".json"],!1):t}function wl(e){return 1===e||2===e}function Pl(n){return e.some(e.supportedJSExtensions,function(t){return e.fileExtensionIs(n,t)})}function Ml(n){return e.some(e.supportedTSExtensions,function(t){return e.fileExtensionIs(n,t)})}function Fl(n,t,r){if(!n)return!1;for(var i=Rl(t,r),a=0,o=Ol(t,i);a<o.length;a++){var s=o[a];if(e.fileExtensionIs(n,s))return!0}return!1}function Gl(e){var n=e.match(/\//g);return n?n.length:0}function Bl(n,t){return e.compareValues(Gl(n),Gl(t))}function Ul(n,t){for(var r=t.length-1;r>=0;r--)if(e.fileExtensionIs(n,t[r]))return Vl(r,t);return 0}function Vl(e,n){return e<2?0:e<n.length?2:n.length}function jl(e,n){return e<2?2:n.length}e.getSupportedExtensions=Rl,e.getSuppoertedExtensionsWithJsonIfResolveJsonModule=Ol,e.hasJSFileExtension=Pl,e.hasTSFileExtension=Ml,e.isSupportedSourceFileName=Fl,e.compareNumberOfDirectorySeparators=Bl,function(e){e[e["TypeScriptFiles"]=0]="TypeScriptFiles",e[e["DeclarationAndJavaScriptFiles"]=2]="DeclarationAndJavaScriptFiles",e[e["Highest"]=0]="Highest",e[e["Lowest"]=2]="Lowest"}(e.ExtensionPriority||(e.ExtensionPriority={})),e.getExtensionPriority=Ul,e.adjustExtensionPriority=Vl,e.getNextLowestExtensionPriority=jl;var Kl=[".d.ts",".ts",".js",".tsx",".jsx",".json"];function Hl(e){for(var n=0,t=Kl;n<t.length;n++){var r=t[n],i=Wl(e,r);if(void 0!==i)return i}return e}function Wl(n,t){return e.fileExtensionIs(n,t)?zl(n,t):void 0}function zl(e,n){return e.substring(0,e.length-n.length)}function Jl(n,t){return e.changeAnyExtension(n,t,Kl,!1)}function ql(e){var n=e.indexOf("*");return-1===n?e:-1!==e.indexOf("*",n+1)?void 0:{prefix:e.substr(0,n),suffix:e.substr(n+1)}}function Xl(n){return e.mapDefined(e.getOwnKeys(n),function(e){return ql(e)})}function Yl(e){return!(e>=0)}function Ql(e){return".ts"===e||".tsx"===e||".d.ts"===e}function Zl(e){return Ql(e)||".json"===e}function $l(n){var t=nu(n);return void 0!==t?t:e.Debug.fail("File "+n+" has unknown extension.")}function eu(e){return void 0!==nu(e)}function nu(n){return e.find(Kl,function(t){return e.fileExtensionIs(n,t)})}function tu(e,n){return e.checkJsDirective?e.checkJsDirective.enabled:n.checkJs}function ru(n,t){for(var r=[],i=0,a=n;i<a.length;i++){var o=a[i];if(o===t)return t;e.isString(o)||r.push(o)}return e.findBestPatternMatch(r,function(e){return e},t)}function iu(n,t){var r=n.indexOf(t);return e.Debug.assert(-1!==r),n.slice(r)}function au(n){for(var t,r=[],i=1;i<arguments.length;i++)r[i-1]=arguments[i];return r.length?(n.relatedInformation||(n.relatedInformation=[]),e.Debug.assert(n.relatedInformation!==e.emptyArray,"Diagnostic had empty array singleton for related info, but is still being constructed!"),(t=n.relatedInformation).push.apply(t,r),n):n}function ou(n,t){e.Debug.assert(0!==n.length);for(var r=t(n[0]),i=r,a=1;a<n.length;a++){var o=t(n[a]);o<r?r=o:o>i&&(i=o)}return{min:r,max:i}}function su(e){return{pos:Y(e),end:e.end}}function cu(n,t){var r=t.pos-1,i=e.skipTrivia(n.text,t.end)+1;return{pos:r,end:i}}function lu(e,n,t){return n.skipLibCheck&&e.isDeclarationFile||n.skipDefaultLibCheck&&e.hasNoDefaultLib||t.isSourceOfProjectReferenceRedirect(e.fileName)}function uu(n,t){return n===t||"object"===typeof n&&null!==n&&"object"===typeof t&&null!==t&&e.equalOwnProperties(n,t,uu)}function du(e){var n;switch(e.charCodeAt(1)){case 98:case 66:n=1;break;case 111:case 79:n=3;break;case 120:case 88:n=4;break;default:var t=e.length-1,r=0;while(48===e.charCodeAt(r))r++;return e.slice(r,t)||"0"}for(var i=2,a=e.length-1,o=(a-i)*n,s=new Uint16Array((o>>>4)+(15&o?1:0)),c=a-1,l=0;c>=i;c--,l+=n){var u=l>>>4,d=e.charCodeAt(c),p=d<=57?d-48:10+d-(d<=70?65:97),f=p<<(15&l);s[u]|=f;var m=f>>>16;m&&(s[u+1]|=m)}var _="",g=s.length-1,y=!0;while(y){var h=0;y=!1;for(u=g;u>=0;u--){var v=h<<16|s[u],b=v/10|0;s[u]=b,h=v-10*b,b&&!y&&(g=u,y=!0)}_=h+_}return _}function pu(e){var n=e.negative,t=e.base10Value;return(n&&"0"!==t?"-":"")+t}function fu(e){return!!(8388608&e.flags)||ut(e)||yu(e)||gu(e)||!(ct(e)||_u(e))}function mu(e){return 273===e.kind}function _u(n){return e.isIdentifier(n)&&e.isShorthandPropertyAssignment(n.parent)&&n.parent.name===n}function gu(e){while(79===e.kind||204===e.kind)e=e.parent;if(160!==e.kind)return!1;if(co(e.parent,128))return!0;var n=e.parent.parent.kind;return 256===n||180===n}function yu(n){if(79!==n.kind)return!1;var t=e.findAncestor(n.parent,function(e){switch(e.kind){case 289:return!0;case 204:case 226:return!1;default:return"quit"}});return 117===(null===t||void 0===t?void 0:t.token)||256===(null===t||void 0===t?void 0:t.parent.kind)}function hu(n){return e.isTypeReferenceNode(n)&&e.isIdentifier(n.typeName)}function vu(n,t){if(void 0===t&&(t=e.equateValues),n.length<2)return!0;for(var r=n[0],i=1,a=n.length;i<a;i++){var o=n[i];if(!t(r,o))return!1}return!0}function bu(e,n){return e.pos=n,e}function Eu(e,n){return e.end=n,e}function Tu(e,n,t){return Eu(bu(e,n),t)}function Su(e,n,t){return Tu(e,n,n+t)}function xu(e,n){return e&&(e.flags=n),e}function Lu(e,n){return e&&n&&(e.parent=n),e}function Du(e,n){if(e)for(var t=0,r=e;t<r.length;t++){var i=r[t];Lu(i,n)}return e}function Au(n,t){return n?(e.forEachChildRecursively(n,e.isJSDocNode(n)?r:a),n):n;function r(e,n){if(t&&e.parent===n)return"skip";Lu(e,n)}function i(n){if(e.hasJSDocNodes(n))for(var t=0,i=n.jsDoc;t<i.length;t++){var a=i[t];r(a,n),e.forEachChildRecursively(a,r)}}function a(e,n){return r(e,n)||i(e)}}function Cu(n){return!e.isOmittedExpression(n)}function Iu(n){return e.isArrayLiteralExpression(n)&&e.every(n.elements,Cu)}function Nu(n){e.Debug.assertIsDefined(n.parent);while(1){var t=n.parent;if(e.isParenthesizedExpression(t))n=t;else{if(e.isExpressionStatement(t)||e.isVoidExpression(t)||e.isForStatement(t)&&(t.initializer===n||t.incrementor===n))return!0;if(e.isCommaListExpression(t)){if(n!==e.last(t.elements))return!0;n=t}else{if(!e.isBinaryExpression(t)||27!==t.operatorToken.kind)return!1;if(n===t.left)return!0;n=t}}}}function ku(n){return e.some(e.ignoredPaths,function(t){return e.stringContains(n,t)})}function Ru(n){if(n.parent){switch(n.kind){case 161:var t=n.parent;return 188===t.kind?void 0:t.typeParameters;case 162:return n.parent.parameters;case 197:return n.parent.templateSpans;case 231:return n.parent.templateSpans;case 163:return n.parent.decorators;case 289:return n.parent.heritageClauses}var r=n.parent;if(e.isJSDocTag(n))return e.isJSDocTypeLiteral(n.parent)?void 0:n.parent.tags;switch(r.kind){case 180:case 256:return e.isTypeElement(n)?r.members:void 0;case 185:case 186:return r.types;case 182:case 202:case 346:case 267:case 271:return r.elements;case 203:case 284:return r.properties;case 206:case 207:return e.isTypeNode(n)?r.typeArguments:r.expression===n?void 0:r.arguments;case 276:case 280:return e.isJsxChild(n)?r.children:void 0;case 278:case 277:return e.isTypeNode(n)?r.typeArguments:void 0;case 233:case 287:case 288:case 260:return r.statements;case 261:return r.clauses;case 255:case 224:return e.isClassElement(n)?r.members:void 0;case 258:return e.isEnumMember(n)?r.members:void 0;case 300:return r.statements}}}function Ou(n){if(!n.typeParameters){if(e.some(n.parameters,function(e){return!Wa(e)}))return!0;if(212!==n.kind){var t=e.firstOrUndefined(n.parameters);if(!t||!Ua(t))return!0}}return!1}function wu(e){return"Infinity"===e||"-Infinity"===e||"NaN"===e}e.removeFileExtension=Hl,e.tryRemoveExtension=Wl,e.removeExtension=zl,e.changeExtension=Jl,e.tryParsePattern=ql,e.tryParsePatterns=Xl,e.positionIsSynthesized=Yl,e.extensionIsTS=Ql,e.resolutionExtensionIsTSOrJson=Zl,e.extensionFromPath=$l,e.isAnySupportedFileExtension=eu,e.tryGetExtensionFromPath=nu,e.isCheckJsEnabledForFile=tu,e.emptyFileSystemEntries={files:e.emptyArray,directories:e.emptyArray},e.matchPatternOrExact=ru,e.sliceAfter=iu,e.addRelatedInfo=au,e.minAndMax=ou,e.rangeOfNode=su,e.rangeOfTypeParameters=cu,e.skipTypeChecking=lu,e.isJsonEqual=uu,e.parsePseudoBigInt=du,e.pseudoBigIntToString=pu,e.isValidTypeOnlyAliasUseSite=fu,e.typeOnlyDeclarationIsExport=mu,e.isIdentifierTypeReference=hu,e.arrayIsHomogeneous=vu,e.setTextRangePos=bu,e.setTextRangeEnd=Eu,e.setTextRangePosEnd=Tu,e.setTextRangePosWidth=Su,e.setNodeFlags=xu,e.setParent=Lu,e.setEachParent=Du,e.setParentRecursive=Au,e.isPackedArrayLiteral=Iu,e.expressionResultIsUnused=Nu,e.containsIgnoredPath=ku,e.getContainingNodeArray=Ru,e.hasContextSensitiveParameters=Ou,e.isInfinityOrNaNString=wu}(d||(d={})),function(e){function n(){var n,t,r,i,a;return{createBaseSourceFileNode:o,createBaseIdentifierNode:s,createBasePrivateIdentifierNode:c,createBaseTokenNode:l,createBaseNode:u};function o(n){return new(a||(a=e.objectAllocator.getSourceFileConstructor()))(n,-1,-1)}function s(n){return new(r||(r=e.objectAllocator.getIdentifierConstructor()))(n,-1,-1)}function c(n){return new(i||(i=e.objectAllocator.getPrivateIdentifierConstructor()))(n,-1,-1)}function l(n){return new(t||(t=e.objectAllocator.getTokenConstructor()))(n,-1,-1)}function u(t){return new(n||(n=e.objectAllocator.getNodeConstructor()))(t,-1,-1)}}e.createBaseNodeFactory=n}(d||(d={})),function(e){function n(n){var t,r;return{getParenthesizeLeftSideOfBinaryForOperator:i,getParenthesizeRightSideOfBinaryForOperator:a,parenthesizeLeftSideOfBinary:u,parenthesizeRightSideOfBinary:d,parenthesizeExpressionOfComputedPropertyName:p,parenthesizeConditionOfConditionalExpression:f,parenthesizeBranchOfConditionalExpression:m,parenthesizeExpressionOfExportDefault:_,parenthesizeExpressionOfNew:g,parenthesizeLeftSideOfAccess:y,parenthesizeOperandOfPostfixUnary:h,parenthesizeOperandOfPrefixUnary:v,parenthesizeExpressionsOfCommaDelimitedList:b,parenthesizeExpressionForDisallowedComma:E,parenthesizeExpressionOfExpressionStatement:T,parenthesizeConciseBodyOfArrowFunction:S,parenthesizeMemberOfConditionalType:x,parenthesizeMemberOfElementType:L,parenthesizeElementTypeOfArrayType:D,parenthesizeConstituentTypesOfUnionOrIntersectionType:A,parenthesizeTypeArguments:I};function i(n){t||(t=new e.Map);var r=t.get(n);return r||(r=function(e){return u(n,e)},t.set(n,r)),r}function a(n){r||(r=new e.Map);var t=r.get(n);return t||(t=function(e){return d(n,void 0,e)},r.set(n,t)),t}function o(n,t,r,i){var a=e.getOperatorPrecedence(219,n),o=e.getOperatorAssociativity(219,n),l=e.skipPartiallyEmittedExpressions(t);if(!r&&212===t.kind&&a>3)return!0;var u=e.getExpressionPrecedence(l);switch(e.compareValues(u,a)){case-1:return!(!r&&1===o&&222===t.kind);case 1:return!1;case 0:if(r)return 1===o;if(e.isBinaryExpression(l)&&l.operatorToken.kind===n){if(s(n))return!1;if(39===n){var d=i?c(i):0;if(e.isLiteralKind(d)&&d===c(l))return!1}}var p=e.getExpressionAssociativity(l);return 0===p}}function s(e){return 41===e||51===e||50===e||52===e}function c(n){if(n=e.skipPartiallyEmittedExpressions(n),e.isLiteralKind(n.kind))return n.kind;if(219===n.kind&&39===n.operatorToken.kind){if(void 0!==n.cachedLiteralKind)return n.cachedLiteralKind;var t=c(n.left),r=e.isLiteralKind(t)&&t===c(n.right)?t:0;return n.cachedLiteralKind=r,r}return 0}function l(t,r,i,a){var s=e.skipPartiallyEmittedExpressions(r);return 210===s.kind?r:o(t,r,i,a)?n.createParenthesizedExpression(r):r}function u(e,n){return l(e,n,!0)}function d(e,n,t){return l(e,t,!1,n)}function p(t){return e.isCommaSequence(t)?n.createParenthesizedExpression(t):t}function f(t){var r=e.getOperatorPrecedence(220,57),i=e.skipPartiallyEmittedExpressions(t),a=e.getExpressionPrecedence(i);return 1!==e.compareValues(a,r)?n.createParenthesizedExpression(t):t}function m(t){var r=e.skipPartiallyEmittedExpressions(t);return e.isCommaSequence(r)?n.createParenthesizedExpression(t):t}function _(t){var r=e.skipPartiallyEmittedExpressions(t),i=e.isCommaSequence(r);if(!i)switch(e.getLeftmostExpression(r,!1).kind){case 224:case 211:i=!0}return i?n.createParenthesizedExpression(t):t}function g(t){var r=e.getLeftmostExpression(t,!0);switch(r.kind){case 206:return n.createParenthesizedExpression(t);case 207:return r.arguments?t:n.createParenthesizedExpression(t)}return y(t)}function y(t){var r=e.skipPartiallyEmittedExpressions(t);return e.isLeftHandSideExpression(r)&&(207!==r.kind||r.arguments)?t:e.setTextRange(n.createParenthesizedExpression(t),t)}function h(t){return e.isLeftHandSideExpression(t)?t:e.setTextRange(n.createParenthesizedExpression(t),t)}function v(t){return e.isUnaryExpression(t)?t:e.setTextRange(n.createParenthesizedExpression(t),t)}function b(t){var r=e.sameMap(t,E);return e.setTextRange(n.createNodeArray(r,t.hasTrailingComma),t)}function E(t){var r=e.skipPartiallyEmittedExpressions(t),i=e.getExpressionPrecedence(r),a=e.getOperatorPrecedence(219,27);return i>a?t:e.setTextRange(n.createParenthesizedExpression(t),t)}function T(t){var r=e.skipPartiallyEmittedExpressions(t);if(e.isCallExpression(r)){var i=r.expression,a=e.skipPartiallyEmittedExpressions(i).kind;if(211===a||212===a){var o=n.updateCallExpression(r,e.setTextRange(n.createParenthesizedExpression(i),i),r.typeArguments,r.arguments);return n.restoreOuterExpressions(t,o,8)}}var s=e.getLeftmostExpression(r,!1).kind;return 203===s||211===s?e.setTextRange(n.createParenthesizedExpression(t),t):t}function S(t){return e.isBlock(t)||!e.isCommaSequence(t)&&203!==e.getLeftmostExpression(t,!1).kind?t:e.setTextRange(n.createParenthesizedExpression(t),t)}function x(e){return 187===e.kind?n.createParenthesizedType(e):e}function L(e){switch(e.kind){case 185:case 186:case 177:case 178:return n.createParenthesizedType(e)}return x(e)}function D(e){switch(e.kind){case 179:case 191:case 188:return n.createParenthesizedType(e)}return L(e)}function A(t){return n.createNodeArray(e.sameMap(t,L))}function C(t,r){return 0===r&&e.isFunctionOrConstructorTypeNode(t)&&t.typeParameters?n.createParenthesizedType(t):t}function I(t){if(e.some(t))return n.createNodeArray(e.sameMap(t,C))}}e.createParenthesizerRules=n,e.nullParenthesizerRules={getParenthesizeLeftSideOfBinaryForOperator:function(n){return e.identity},getParenthesizeRightSideOfBinaryForOperator:function(n){return e.identity},parenthesizeLeftSideOfBinary:function(e,n){return n},parenthesizeRightSideOfBinary:function(e,n,t){return t},parenthesizeExpressionOfComputedPropertyName:e.identity,parenthesizeConditionOfConditionalExpression:e.identity,parenthesizeBranchOfConditionalExpression:e.identity,parenthesizeExpressionOfExportDefault:e.identity,parenthesizeExpressionOfNew:function(n){return e.cast(n,e.isLeftHandSideExpression)},parenthesizeLeftSideOfAccess:function(n){return e.cast(n,e.isLeftHandSideExpression)},parenthesizeOperandOfPostfixUnary:function(n){return e.cast(n,e.isLeftHandSideExpression)},parenthesizeOperandOfPrefixUnary:function(n){return e.cast(n,e.isUnaryExpression)},parenthesizeExpressionsOfCommaDelimitedList:function(n){return e.cast(n,e.isNodeArray)},parenthesizeExpressionForDisallowedComma:e.identity,parenthesizeExpressionOfExpressionStatement:e.identity,parenthesizeConciseBodyOfArrowFunction:e.identity,parenthesizeMemberOfConditionalType:e.identity,parenthesizeMemberOfElementType:e.identity,parenthesizeElementTypeOfArrayType:e.identity,parenthesizeConstituentTypesOfUnionOrIntersectionType:function(n){return e.cast(n,e.isNodeArray)},parenthesizeTypeArguments:function(n){return n&&e.cast(n,e.isNodeArray)}}}(d||(d={})),function(e){function n(n){return{convertToFunctionBlock:t,convertToFunctionExpression:r,convertToArrayAssignmentElement:i,convertToObjectAssignmentElement:a,convertToAssignmentPattern:o,convertToObjectAssignmentPattern:s,convertToArrayAssignmentPattern:c,convertToAssignmentElementTarget:l};function t(t,r){if(e.isBlock(t))return t;var i=n.createReturnStatement(t);e.setTextRange(i,t);var a=n.createBlock([i],r);return e.setTextRange(a,t),a}function r(t){if(!t.body)return e.Debug.fail("Cannot convert a FunctionDeclaration without a body");var r=n.createFunctionExpression(t.modifiers,t.asteriskToken,t.name,t.typeParameters,t.parameters,t.type,t.body);return e.setOriginalNode(r,t),e.setTextRange(r,t),e.getStartsOnNewLine(t)&&e.setStartsOnNewLine(r,!0),r}function i(t){if(e.isBindingElement(t)){if(t.dotDotDotToken)return e.Debug.assertNode(t.name,e.isIdentifier),e.setOriginalNode(e.setTextRange(n.createSpreadElement(t.name),t),t);var r=l(t.name);return t.initializer?e.setOriginalNode(e.setTextRange(n.createAssignment(r,t.initializer),t),t):r}return e.cast(t,e.isExpression)}function a(t){if(e.isBindingElement(t)){if(t.dotDotDotToken)return e.Debug.assertNode(t.name,e.isIdentifier),e.setOriginalNode(e.setTextRange(n.createSpreadAssignment(t.name),t),t);if(t.propertyName){var r=l(t.name);return e.setOriginalNode(e.setTextRange(n.createPropertyAssignment(t.propertyName,t.initializer?n.createAssignment(r,t.initializer):r),t),t)}return e.Debug.assertNode(t.name,e.isIdentifier),e.setOriginalNode(e.setTextRange(n.createShorthandPropertyAssignment(t.name,t.initializer),t),t)}return e.cast(t,e.isObjectLiteralElementLike)}function o(e){switch(e.kind){case 200:case 202:return c(e);case 199:case 203:return s(e)}}function s(t){return e.isObjectBindingPattern(t)?e.setOriginalNode(e.setTextRange(n.createObjectLiteralExpression(e.map(t.elements,a)),t),t):e.cast(t,e.isObjectLiteralExpression)}function c(t){return e.isArrayBindingPattern(t)?e.setOriginalNode(e.setTextRange(n.createArrayLiteralExpression(e.map(t.elements,i)),t),t):e.cast(t,e.isArrayLiteralExpression)}function l(n){return e.isBindingPattern(n)?o(n):e.cast(n,e.isExpression)}}e.createNodeConverters=n,e.nullNodeConverters={convertToFunctionBlock:e.notImplemented,convertToFunctionExpression:e.notImplemented,convertToArrayAssignmentElement:e.notImplemented,convertToObjectAssignmentElement:e.notImplemented,convertToAssignmentPattern:e.notImplemented,convertToObjectAssignmentPattern:e.notImplemented,convertToArrayAssignmentPattern:e.notImplemented,convertToAssignmentElementTarget:e.notImplemented}}(d||(d={})),function(e){var n,t=0;function r(n,r){var c=8&n?i:o,d=e.memoize(function(){return 1&n?e.nullParenthesizerRules:e.createParenthesizerRules(A)}),_=e.memoize(function(){return 2&n?e.nullNodeConverters:e.createNodeConverters(A)}),g=e.memoizeOne(function(e){return function(n,t){return Bt(n,e,t)}}),y=e.memoizeOne(function(e){return function(n){return Pt(e,n)}}),h=e.memoizeOne(function(e){return function(n){return Ft(n,e)}}),v=e.memoizeOne(function(e){return function(){return Ji(e)}}),b=e.memoizeOne(function(e){return function(n){return qi(e,n)}}),E=e.memoizeOne(function(e){return function(n,t){return Xi(e,n,t)}}),T=e.memoizeOne(function(e){return function(n,t){return Ra(e,n,t)}}),S=e.memoizeOne(function(e){return function(n,t,r){return Oa(e,n,t,r)}}),x=e.memoizeOne(function(e){return function(n,t,r){return wa(e,n,t,r)}}),D=e.memoizeOne(function(e){return function(n,t,r,i){return Pa(e,n,t,r,i)}}),A={get parenthesizer(){return d()},get converters(){return _()},createNodeArray:C,createNumericLiteral:j,createBigIntLiteral:K,createStringLiteral:W,createStringLiteralFromNode:z,createRegularExpressionLiteral:J,createLiteralLikeNode:q,createIdentifier:Q,updateIdentifier:Z,createTempVariable:$,createLoopVariable:ee,createUniqueName:ne,getGeneratedNameForNode:te,createPrivateIdentifier:re,createToken:ae,createSuper:oe,createThis:se,createNull:ce,createTrue:le,createFalse:ue,createModifier:de,createModifiersFromModifierFlags:pe,createQualifiedName:fe,updateQualifiedName:me,createComputedPropertyName:_e,updateComputedPropertyName:ge,createTypeParameterDeclaration:ye,updateTypeParameterDeclaration:he,createParameterDeclaration:ve,updateParameterDeclaration:be,createDecorator:Ee,updateDecorator:Te,createPropertySignature:Se,updatePropertySignature:xe,createPropertyDeclaration:Le,updatePropertyDeclaration:De,createMethodSignature:Ae,updateMethodSignature:Ce,createMethodDeclaration:Ie,updateMethodDeclaration:Ne,createConstructorDeclaration:Oe,updateConstructorDeclaration:we,createGetAccessorDeclaration:Pe,updateGetAccessorDeclaration:Me,createSetAccessorDeclaration:Fe,updateSetAccessorDeclaration:Ge,createCallSignature:Be,updateCallSignature:Ue,createConstructSignature:Ve,updateConstructSignature:je,createIndexSignature:Ke,updateIndexSignature:He,createClassStaticBlockDeclaration:ke,updateClassStaticBlockDeclaration:Re,createTemplateLiteralTypeSpan:We,updateTemplateLiteralTypeSpan:ze,createKeywordTypeNode:Je,createTypePredicateNode:qe,updateTypePredicateNode:Xe,createTypeReferenceNode:Ye,updateTypeReferenceNode:Qe,createFunctionTypeNode:Ze,updateFunctionTypeNode:$e,createConstructorTypeNode:en,updateConstructorTypeNode:rn,createTypeQueryNode:sn,updateTypeQueryNode:cn,createTypeLiteralNode:ln,updateTypeLiteralNode:un,createArrayTypeNode:dn,updateArrayTypeNode:pn,createTupleTypeNode:fn,updateTupleTypeNode:mn,createNamedTupleMember:_n,updateNamedTupleMember:gn,createOptionalTypeNode:yn,updateOptionalTypeNode:hn,createRestTypeNode:vn,updateRestTypeNode:bn,createUnionTypeNode:Sn,updateUnionTypeNode:xn,createIntersectionTypeNode:Ln,updateIntersectionTypeNode:Dn,createConditionalTypeNode:An,updateConditionalTypeNode:Cn,createInferTypeNode:In,updateInferTypeNode:Nn,createImportTypeNode:On,updateImportTypeNode:wn,createParenthesizedType:Pn,updateParenthesizedType:Mn,createThisTypeNode:Fn,createTypeOperatorNode:Gn,updateTypeOperatorNode:Bn,createIndexedAccessTypeNode:Un,updateIndexedAccessTypeNode:Vn,createMappedTypeNode:jn,updateMappedTypeNode:Kn,createLiteralTypeNode:Hn,updateLiteralTypeNode:Wn,createTemplateLiteralType:kn,updateTemplateLiteralType:Rn,createObjectBindingPattern:zn,updateObjectBindingPattern:Jn,createArrayBindingPattern:qn,updateArrayBindingPattern:Xn,createBindingElement:Yn,updateBindingElement:Qn,createArrayLiteralExpression:$n,updateArrayLiteralExpression:et,createObjectLiteralExpression:nt,updateObjectLiteralExpression:tt,createPropertyAccessExpression:4&n?function(n,t){return e.setEmitFlags(rt(n,t),131072)}:rt,updatePropertyAccessExpression:it,createPropertyAccessChain:4&n?function(n,t,r){return e.setEmitFlags(at(n,t,r),131072)}:at,updatePropertyAccessChain:ot,createElementAccessExpression:st,updateElementAccessExpression:ct,createElementAccessChain:lt,updateElementAccessChain:ut,createCallExpression:dt,updateCallExpression:pt,createCallChain:ft,updateCallChain:mt,createNewExpression:_t,updateNewExpression:gt,createTaggedTemplateExpression:yt,updateTaggedTemplateExpression:ht,createTypeAssertion:vt,updateTypeAssertion:bt,createParenthesizedExpression:Et,updateParenthesizedExpression:Tt,createFunctionExpression:St,updateFunctionExpression:xt,createArrowFunction:Lt,updateArrowFunction:Dt,createDeleteExpression:At,updateDeleteExpression:Ct,createTypeOfExpression:It,updateTypeOfExpression:Nt,createVoidExpression:kt,updateVoidExpression:Rt,createAwaitExpression:Ot,updateAwaitExpression:wt,createPrefixUnaryExpression:Pt,updatePrefixUnaryExpression:Mt,createPostfixUnaryExpression:Ft,updatePostfixUnaryExpression:Gt,createBinaryExpression:Bt,updateBinaryExpression:Vt,createConditionalExpression:jt,updateConditionalExpression:Kt,createTemplateExpression:Ht,updateTemplateExpression:Wt,createTemplateHead:qt,createTemplateMiddle:Xt,createTemplateTail:Yt,createNoSubstitutionTemplateLiteral:Qt,createTemplateLiteralLikeNode:Jt,createYieldExpression:Zt,updateYieldExpression:$t,createSpreadElement:er,updateSpreadElement:nr,createClassExpression:tr,updateClassExpression:rr,createOmittedExpression:ir,createExpressionWithTypeArguments:ar,updateExpressionWithTypeArguments:or,createAsExpression:sr,updateAsExpression:cr,createNonNullExpression:lr,updateNonNullExpression:ur,createNonNullChain:dr,updateNonNullChain:pr,createMetaProperty:fr,updateMetaProperty:mr,createTemplateSpan:_r,updateTemplateSpan:gr,createSemicolonClassElement:yr,createBlock:hr,updateBlock:vr,createVariableStatement:br,updateVariableStatement:Er,createEmptyStatement:Tr,createExpressionStatement:Sr,updateExpressionStatement:xr,createIfStatement:Lr,updateIfStatement:Dr,createDoStatement:Ar,updateDoStatement:Cr,createWhileStatement:Ir,updateWhileStatement:Nr,createForStatement:kr,updateForStatement:Rr,createForInStatement:Or,updateForInStatement:wr,createForOfStatement:Pr,updateForOfStatement:Mr,createContinueStatement:Fr,updateContinueStatement:Gr,createBreakStatement:Br,updateBreakStatement:Ur,createReturnStatement:Vr,updateReturnStatement:jr,createWithStatement:Kr,updateWithStatement:Hr,createSwitchStatement:Wr,updateSwitchStatement:zr,createLabeledStatement:Jr,updateLabeledStatement:qr,createThrowStatement:Xr,updateThrowStatement:Yr,createTryStatement:Qr,updateTryStatement:Zr,createDebuggerStatement:$r,createVariableDeclaration:ei,updateVariableDeclaration:ni,createVariableDeclarationList:ti,updateVariableDeclarationList:ri,createFunctionDeclaration:ii,updateFunctionDeclaration:ai,createClassDeclaration:oi,updateClassDeclaration:si,createInterfaceDeclaration:ci,updateInterfaceDeclaration:li,createTypeAliasDeclaration:ui,updateTypeAliasDeclaration:di,createEnumDeclaration:pi,updateEnumDeclaration:fi,createModuleDeclaration:mi,updateModuleDeclaration:_i,createModuleBlock:gi,updateModuleBlock:yi,createCaseBlock:hi,updateCaseBlock:vi,createNamespaceExportDeclaration:bi,updateNamespaceExportDeclaration:Ei,createImportEqualsDeclaration:Ti,updateImportEqualsDeclaration:Si,createImportDeclaration:xi,updateImportDeclaration:Li,createImportClause:Di,updateImportClause:Ai,createNamespaceImport:Ci,updateNamespaceImport:Ii,createNamespaceExport:Ni,updateNamespaceExport:ki,createNamedImports:Ri,updateNamedImports:Oi,createImportSpecifier:wi,updateImportSpecifier:Pi,createExportAssignment:Mi,updateExportAssignment:Fi,createExportDeclaration:Gi,updateExportDeclaration:Bi,createNamedExports:Ui,updateNamedExports:Vi,createExportSpecifier:ji,updateExportSpecifier:Ki,createMissingDeclaration:Hi,createExternalModuleReference:Wi,updateExternalModuleReference:zi,get createJSDocAllType(){return v(307)},get createJSDocUnknownType(){return v(308)},get createJSDocNonNullableType(){return b(310)},get updateJSDocNonNullableType(){return E(310)},get createJSDocNullableType(){return b(309)},get updateJSDocNullableType(){return E(309)},get createJSDocOptionalType(){return b(311)},get updateJSDocOptionalType(){return E(311)},get createJSDocVariadicType(){return b(313)},get updateJSDocVariadicType(){return E(313)},get createJSDocNamepathType(){return b(314)},get updateJSDocNamepathType(){return E(314)},createJSDocFunctionType:Yi,updateJSDocFunctionType:Qi,createJSDocTypeLiteral:Zi,updateJSDocTypeLiteral:$i,createJSDocTypeExpression:ea,updateJSDocTypeExpression:na,createJSDocSignature:ta,updateJSDocSignature:ra,createJSDocTemplateTag:oa,updateJSDocTemplateTag:sa,createJSDocTypedefTag:ca,updateJSDocTypedefTag:la,createJSDocParameterTag:ua,updateJSDocParameterTag:da,createJSDocPropertyTag:pa,updateJSDocPropertyTag:fa,createJSDocCallbackTag:ma,updateJSDocCallbackTag:_a,createJSDocAugmentsTag:ga,updateJSDocAugmentsTag:ya,createJSDocImplementsTag:ha,updateJSDocImplementsTag:ka,createJSDocSeeTag:va,updateJSDocSeeTag:ba,createJSDocNameReference:Ea,updateJSDocNameReference:Ta,createJSDocMemberName:Sa,updateJSDocMemberName:xa,createJSDocLink:La,updateJSDocLink:Da,createJSDocLinkCode:Aa,updateJSDocLinkCode:Ca,createJSDocLinkPlain:Ia,updateJSDocLinkPlain:Na,get createJSDocTypeTag(){return x(338)},get updateJSDocTypeTag(){return D(338)},get createJSDocReturnTag(){return x(336)},get updateJSDocReturnTag(){return D(336)},get createJSDocThisTag(){return x(337)},get updateJSDocThisTag(){return D(337)},get createJSDocEnumTag(){return x(334)},get updateJSDocEnumTag(){return D(334)},get createJSDocAuthorTag(){return T(325)},get updateJSDocAuthorTag(){return S(325)},get createJSDocClassTag(){return T(327)},get updateJSDocClassTag(){return S(327)},get createJSDocPublicTag(){return T(328)},get updateJSDocPublicTag(){return S(328)},get createJSDocPrivateTag(){return T(329)},get updateJSDocPrivateTag(){return S(329)},get createJSDocProtectedTag(){return T(330)},get updateJSDocProtectedTag(){return S(330)},get createJSDocReadonlyTag(){return T(331)},get updateJSDocReadonlyTag(){return S(331)},get createJSDocOverrideTag(){return T(332)},get updateJSDocOverrideTag(){return S(332)},get createJSDocDeprecatedTag(){return T(326)},get updateJSDocDeprecatedTag(){return S(326)},createJSDocUnknownTag:Ma,updateJSDocUnknownTag:Fa,createJSDocText:Ga,updateJSDocText:Ba,createJSDocComment:Ua,updateJSDocComment:Va,createJsxElement:ja,updateJsxElement:Ka,createJsxSelfClosingElement:Ha,updateJsxSelfClosingElement:Wa,createJsxOpeningElement:za,updateJsxOpeningElement:Ja,createJsxClosingElement:qa,updateJsxClosingElement:Xa,createJsxFragment:Ya,createJsxText:Za,updateJsxText:$a,createJsxOpeningFragment:eo,createJsxJsxClosingFragment:no,updateJsxFragment:Qa,createJsxAttribute:to,updateJsxAttribute:ro,createJsxAttributes:io,updateJsxAttributes:ao,createJsxSpreadAttribute:oo,updateJsxSpreadAttribute:so,createJsxExpression:co,updateJsxExpression:lo,createCaseClause:uo,updateCaseClause:po,createDefaultClause:fo,updateDefaultClause:mo,createHeritageClause:_o,updateHeritageClause:go,createCatchClause:yo,updateCatchClause:ho,createPropertyAssignment:vo,updatePropertyAssignment:Eo,createShorthandPropertyAssignment:To,updateShorthandPropertyAssignment:xo,createSpreadAssignment:Lo,updateSpreadAssignment:Do,createEnumMember:Ao,updateEnumMember:Co,createSourceFile:Io,updateSourceFile:ko,createBundle:Ro,updateBundle:Oo,createUnparsedSource:wo,createUnparsedPrologue:Mo,createUnparsedPrepend:Fo,createUnparsedTextLike:Go,createUnparsedSyntheticReference:Bo,createInputFiles:Uo,createSyntheticExpression:Vo,createSyntaxList:jo,createNotEmittedStatement:Ko,createPartiallyEmittedExpression:Ho,updatePartiallyEmittedExpression:Wo,createCommaListExpression:Jo,updateCommaListExpression:qo,createEndOfDeclarationMarker:Xo,createMergeDeclarationMarker:Yo,createSyntheticReferenceExpression:Qo,updateSyntheticReferenceExpression:Zo,cloneNode:$o,get createComma(){return g(27)},get createAssignment(){return g(63)},get createLogicalOr(){return g(56)},get createLogicalAnd(){return g(55)},get createBitwiseOr(){return g(51)},get createBitwiseXor(){return g(52)},get createBitwiseAnd(){return g(50)},get createStrictEquality(){return g(36)},get createStrictInequality(){return g(37)},get createEquality(){return g(34)},get createInequality(){return g(35)},get createLessThan(){return g(29)},get createLessThanEquals(){return g(32)},get createGreaterThan(){return g(31)},get createGreaterThanEquals(){return g(33)},get createLeftShift(){return g(47)},get createRightShift(){return g(48)},get createUnsignedRightShift(){return g(49)},get createAdd(){return g(39)},get createSubtract(){return g(40)},get createMultiply(){return g(41)},get createDivide(){return g(43)},get createModulo(){return g(44)},get createExponent(){return g(42)},get createPrefixPlus(){return y(39)},get createPrefixMinus(){return y(40)},get createPrefixIncrement(){return y(45)},get createPrefixDecrement(){return y(46)},get createBitwiseNot(){return y(54)},get createLogicalNot(){return y(53)},get createPostfixIncrement(){return h(45)},get createPostfixDecrement(){return h(46)},createImmediatelyInvokedFunctionExpression:es,createImmediatelyInvokedArrowFunction:ns,createVoidZero:ts,createExportDefault:rs,createExternalModuleExport:is,createTypeCheck:as,createMethodCall:os,createGlobalMethodCall:us,createFunctionBindCall:ss,createFunctionCallCall:cs,createFunctionApplyCall:ls,createArraySliceCall:ds,createArrayConcatCall:ps,createObjectDefinePropertyCall:fs,createReflectGetCall:ms,createReflectSetCall:_s,createPropertyDescriptor:ys,createCallBinding:Ss,createAssignmentTargetWrapper:xs,inlineExpressions:Ls,getInternalName:As,getLocalName:Cs,getExportName:Is,getDeclarationName:Ns,getNamespaceMemberName:ks,getExternalModuleOrNamespaceExportName:Rs,restoreOuterExpressions:bs,restoreEnclosingLabel:Es,createUseStrictPrologue:Ps,copyPrologue:Os,copyStandardPrologue:Ms,copyCustomPrologue:Fs,ensureUseStrict:Gs,liftToBlock:Bs,mergeLexicalEnvironment:Vs,updateModifiers:js};return A;function C(n,t){if(void 0===n||n===e.emptyArray)n=[];else if(e.isNodeArray(n)){if(void 0===t||n.hasTrailingComma===t)return void 0===n.transformFlags&&m(n),e.Debug.attachNodeArrayDebugInfo(n),n;var r=n.slice();return r.pos=n.pos,r.end=n.end,r.hasTrailingComma=t,r.transformFlags=n.transformFlags,e.Debug.attachNodeArrayDebugInfo(r),r}var i=n.length,a=i>=1&&i<=4?n.slice():n;return e.setTextRangePosEnd(a,-1,-1),a.hasTrailingComma=!!t,m(a),e.Debug.attachNodeArrayDebugInfo(a),a}function I(e){return r.createBaseNode(e)}function N(e,n,t){var r=I(e);return r.decorators=Ks(n),r.modifiers=Ks(t),r.transformFlags|=f(r.decorators)|f(r.modifiers),r.symbol=void 0,r.localSymbol=void 0,r.locals=void 0,r.nextContainer=void 0,r}function k(n,t,r,i){var a=N(n,t,r);if(i=Hs(i),a.name=i,i)switch(a.kind){case 167:case 170:case 171:case 165:case 291:if(e.isIdentifier(i)){a.transformFlags|=u(i);break}default:a.transformFlags|=p(i);break}return a}function R(e,n,t,r,i){var a=k(e,n,t,r);return a.typeParameters=Ks(i),a.transformFlags|=f(a.typeParameters),i&&(a.transformFlags|=1),a}function O(e,n,t,r,i,a,o){var s=R(e,n,t,r,i);return s.parameters=C(a),s.type=o,s.transformFlags|=f(s.parameters)|p(s.type),o&&(s.transformFlags|=1),s}function w(e,n){return n.typeArguments&&(e.typeArguments=n.typeArguments),c(e,n)}function P(e,n,t,r,i,a,o,s){var c=O(e,n,t,r,i,a,o);return c.body=s,c.transformFlags|=-16777217&p(c.body),s||(c.transformFlags|=1),c}function M(e,n){return n.exclamationToken&&(e.exclamationToken=n.exclamationToken),n.typeArguments&&(e.typeArguments=n.typeArguments),w(e,n)}function F(e,n,t,r,i,a){var o=R(e,n,t,r,i);return o.heritageClauses=Ks(a),o.transformFlags|=f(o.heritageClauses),o}function G(e,n,t,r,i,a,o){var s=F(e,n,t,r,i,a);return s.members=C(o),s.transformFlags|=f(s.members),s}function B(e,n,t,r,i){var a=k(e,n,t,r);return a.initializer=i,a.transformFlags|=p(a.initializer),a}function U(e,n,t,r,i,a){var o=B(e,n,t,r,a);return o.type=i,o.transformFlags|=p(i),i&&(o.transformFlags|=1),o}function V(e,n){var t=ie(e);return t.text=n,t}function j(e,n){void 0===n&&(n=0);var t=V(8,"number"===typeof e?e+"":e);return t.numericLiteralFlags=n,384&n&&(t.transformFlags|=512),t}function K(n){var t=V(9,"string"===typeof n?n:e.pseudoBigIntToString(n)+"n");return t.transformFlags|=4,t}function H(e,n){var t=V(10,e);return t.singleQuote=n,t}function W(e,n,t){var r=H(e,n);return r.hasExtendedUnicodeEscape=t,t&&(r.transformFlags|=512),r}function z(n){var t=H(e.getTextOfIdentifierOrLiteral(n),void 0);return t.textSourceNode=n,t}function J(e){var n=V(13,e);return n}function q(e,n){switch(e){case 8:return j(n,0);case 9:return K(n);case 10:return W(n,void 0);case 11:return Za(n,!1);case 12:return Za(n,!0);case 13:return J(n);case 14:return Jt(e,n,void 0,0)}}function X(n,t){void 0===t&&n&&(t=e.stringToToken(n)),79===t&&(t=void 0);var i=r.createBaseIdentifierNode(79);return i.originalKeywordKind=t,i.escapedText=e.escapeLeadingUnderscores(n),i}function Y(e,n){var r=X(e,void 0);return r.autoGenerateFlags=n,r.autoGenerateId=t,t++,r}function Q(e,n,t){var r=X(e,t);return n&&(r.typeArguments=C(n)),131===r.originalKeywordKind&&(r.transformFlags|=16777216),r}function Z(n,t){return n.typeArguments!==t?c(Q(e.idText(n),t),n):n}function $(e,n){var t=1;n&&(t|=8);var r=Y("",t);return e&&e(r),r}function ee(e){var n=2;return e&&(n|=8),Y("",n)}function ne(n,t){return void 0===t&&(t=0),e.Debug.assert(!(7&t),"Argument out of range: flags"),e.Debug.assert(32!==(48&t),"GeneratedIdentifierFlags.FileLevel cannot be set without also setting GeneratedIdentifierFlags.Optimistic"),Y(n,3|t)}function te(n,t){void 0===t&&(t=0),e.Debug.assert(!(7&t),"Argument out of range: flags");var r=Y(n&&e.isIdentifier(n)?e.idText(n):"",4|t);return r.original=n,r}function re(n){e.startsWith(n,"#")||e.Debug.fail("First character of private identifier must be #: "+n);var t=r.createBasePrivateIdentifierNode(80);return t.escapedText=e.escapeLeadingUnderscores(n),t.transformFlags|=8388608,t}function ie(e){return r.createBaseTokenNode(e)}function ae(n){e.Debug.assert(n>=0&&n<=158,"Invalid token"),e.Debug.assert(n<=14||n>=17,"Invalid token. Use 'createTemplateLiteralLikeNode' to create template literals."),e.Debug.assert(n<=8||n>=14,"Invalid token. Use 'createLiteralLikeNode' to create literals."),e.Debug.assert(79!==n,"Invalid token. Use 'createIdentifier' to create identifiers");var t=ie(n),r=0;switch(n){case 130:r=192;break;case 123:case 121:case 122:case 143:case 126:case 134:case 85:case 129:case 145:case 156:case 142:case 146:case 157:case 148:case 132:case 149:case 114:case 153:case 151:r=1;break;case 106:r=33554944;break;case 124:r=512;break;case 108:r=8192;break}return r&&(t.transformFlags|=r),t}function oe(){return ae(106)}function se(){return ae(108)}function ce(){return ae(104)}function le(){return ae(110)}function ue(){return ae(95)}function de(e){return ae(e)}function pe(e){var n=[];return 1&e&&n.push(de(93)),2&e&&n.push(de(134)),512&e&&n.push(de(88)),2048&e&&n.push(de(85)),4&e&&n.push(de(123)),8&e&&n.push(de(121)),16&e&&n.push(de(122)),128&e&&n.push(de(126)),32&e&&n.push(de(124)),16384&e&&n.push(de(157)),64&e&&n.push(de(143)),256&e&&n.push(de(130)),n}function fe(e,n){var t=I(159);return t.left=e,t.right=Hs(n),t.transformFlags|=p(t.left)|u(t.right),t}function me(e,n,t){return e.left!==n||e.right!==t?c(fe(n,t),e):e}function _e(e){var n=I(160);return n.expression=d().parenthesizeExpressionOfComputedPropertyName(e),n.transformFlags|=66048|p(n.expression),n}function ge(e,n){return e.expression!==n?c(_e(n),e):e}function ye(e,n,t){var r=k(161,void 0,void 0,e);return r.constraint=n,r.default=t,r.transformFlags=1,r}function he(e,n,t,r){return e.name!==n||e.constraint!==t||e.default!==r?c(ye(n,t,r),e):e}function ve(n,t,r,i,a,o,s){var c=U(162,n,t,i,o,s&&d().parenthesizeExpressionForDisallowedComma(s));return c.dotDotDotToken=r,c.questionToken=a,e.isThisIdentifier(c.name)?c.transformFlags=1:(c.transformFlags|=p(c.dotDotDotToken)|p(c.questionToken),a&&(c.transformFlags|=1),16476&e.modifiersToFlags(c.modifiers)&&(c.transformFlags|=4096),(s||r)&&(c.transformFlags|=512)),c}function be(e,n,t,r,i,a,o,s){return e.decorators!==n||e.modifiers!==t||e.dotDotDotToken!==r||e.name!==i||e.questionToken!==a||e.type!==o||e.initializer!==s?c(ve(n,t,r,i,a,o,s),e):e}function Ee(e){var n=I(163);return n.expression=d().parenthesizeLeftSideOfAccess(e),n.transformFlags|=4097|p(n.expression),n}function Te(e,n){return e.expression!==n?c(Ee(n),e):e}function Se(e,n,t,r){var i=k(164,void 0,e,n);return i.type=r,i.questionToken=t,i.transformFlags=1,i}function xe(e,n,t,r,i){return e.modifiers!==n||e.name!==t||e.questionToken!==r||e.type!==i?c(Se(n,t,r,i),e):e}function Le(n,t,r,i,a,o){var s=U(165,n,t,r,a,o);return s.questionToken=i&&e.isQuestionToken(i)?i:void 0,s.exclamationToken=i&&e.isExclamationToken(i)?i:void 0,s.transformFlags|=p(s.questionToken)|p(s.exclamationToken)|8388608,(e.isComputedPropertyName(s.name)||e.hasStaticModifier(s)&&s.initializer)&&(s.transformFlags|=4096),(i||2&e.modifiersToFlags(s.modifiers))&&(s.transformFlags|=1),s}function De(n,t,r,i,a,o,s){return n.decorators!==t||n.modifiers!==r||n.name!==i||n.questionToken!==(void 0!==a&&e.isQuestionToken(a)?a:void 0)||n.exclamationToken!==(void 0!==a&&e.isExclamationToken(a)?a:void 0)||n.type!==o||n.initializer!==s?c(Le(t,r,i,a,o,s),n):n}function Ae(e,n,t,r,i,a){var o=O(166,void 0,e,n,r,i,a);return o.questionToken=t,o.transformFlags=1,o}function Ce(e,n,t,r,i,a,o){return e.modifiers!==n||e.name!==t||e.questionToken!==r||e.typeParameters!==i||e.parameters!==a||e.type!==o?w(Ae(n,t,r,i,a,o),e):e}function Ie(n,t,r,i,a,o,s,c,l){var u=P(167,n,t,i,o,s,c,l);return u.asteriskToken=r,u.questionToken=a,u.transformFlags|=p(u.asteriskToken)|p(u.questionToken)|512,a&&(u.transformFlags|=1),256&e.modifiersToFlags(u.modifiers)?u.transformFlags|=r?64:128:r&&(u.transformFlags|=1024),u}function Ne(e,n,t,r,i,a,o,s,c,l){return e.decorators!==n||e.modifiers!==t||e.asteriskToken!==r||e.name!==i||e.questionToken!==a||e.typeParameters!==o||e.parameters!==s||e.type!==c||e.body!==l?M(Ie(n,t,r,i,a,o,s,c,l),e):e}function ke(e,n,t){var r=R(168,e,n,void 0,void 0);return r.body=t,r.transformFlags=8388608|p(t),r}function Re(e,n,t,r){return e.decorators!==n||e.modifier!==t||e.body!==r?c(ke(n,t,r),e):e}function Oe(e,n,t,r){var i=P(169,e,n,void 0,void 0,t,void 0,r);return i.transformFlags|=512,i}function we(e,n,t,r,i){return e.decorators!==n||e.modifiers!==t||e.parameters!==r||e.body!==i?M(Oe(n,t,r,i),e):e}function Pe(e,n,t,r,i,a){return P(170,e,n,t,void 0,r,i,a)}function Me(e,n,t,r,i,a,o){return e.decorators!==n||e.modifiers!==t||e.name!==r||e.parameters!==i||e.type!==a||e.body!==o?M(Pe(n,t,r,i,a,o),e):e}function Fe(e,n,t,r,i){return P(171,e,n,t,void 0,r,void 0,i)}function Ge(e,n,t,r,i,a){return e.decorators!==n||e.modifiers!==t||e.name!==r||e.parameters!==i||e.body!==a?M(Fe(n,t,r,i,a),e):e}function Be(e,n,t){var r=O(172,void 0,void 0,void 0,e,n,t);return r.transformFlags=1,r}function Ue(e,n,t,r){return e.typeParameters!==n||e.parameters!==t||e.type!==r?w(Be(n,t,r),e):e}function Ve(e,n,t){var r=O(173,void 0,void 0,void 0,e,n,t);return r.transformFlags=1,r}function je(e,n,t,r){return e.typeParameters!==n||e.parameters!==t||e.type!==r?w(Ve(n,t,r),e):e}function Ke(e,n,t,r){var i=O(174,e,n,void 0,void 0,t,r);return i.transformFlags=1,i}function He(e,n,t,r,i){return e.parameters!==r||e.type!==i||e.decorators!==n||e.modifiers!==t?w(Ke(n,t,r,i),e):e}function We(e,n){var t=I(197);return t.type=e,t.literal=n,t.transformFlags=1,t}function ze(e,n,t){return e.type!==n||e.literal!==t?c(We(n,t),e):e}function Je(e){return ae(e)}function qe(e,n,t){var r=I(175);return r.assertsModifier=e,r.parameterName=Hs(n),r.type=t,r.transformFlags=1,r}function Xe(e,n,t,r){return e.assertsModifier!==n||e.parameterName!==t||e.type!==r?c(qe(n,t,r),e):e}function Ye(e,n){var t=I(176);return t.typeName=Hs(e),t.typeArguments=n&&d().parenthesizeTypeArguments(C(n)),t.transformFlags=1,t}function Qe(e,n,t){return e.typeName!==n||e.typeArguments!==t?c(Ye(n,t),e):e}function Ze(e,n,t){var r=O(177,void 0,void 0,void 0,e,n,t);return r.transformFlags=1,r}function $e(e,n,t,r){return e.typeParameters!==n||e.parameters!==t||e.type!==r?w(Ze(n,t,r),e):e}function en(){for(var n=[],t=0;t<arguments.length;t++)n[t]=arguments[t];return 4===n.length?nn.apply(void 0,n):3===n.length?tn.apply(void 0,n):e.Debug.fail("Incorrect number of arguments specified.")}function nn(e,n,t,r){var i=O(178,void 0,e,void 0,n,t,r);return i.transformFlags=1,i}function tn(e,n,t){return nn(void 0,e,n,t)}function rn(){for(var n=[],t=0;t<arguments.length;t++)n[t]=arguments[t];return 5===n.length?an.apply(void 0,n):4===n.length?on.apply(void 0,n):e.Debug.fail("Incorrect number of arguments specified.")}function an(e,n,t,r,i){return e.modifiers!==n||e.typeParameters!==t||e.parameters!==r||e.type!==i?w(en(n,t,r,i),e):e}function on(e,n,t,r){return an(e,e.modifiers,n,t,r)}function sn(e){var n=I(179);return n.exprName=e,n.transformFlags=1,n}function cn(e,n){return e.exprName!==n?c(sn(n),e):e}function ln(e){var n=I(180);return n.members=C(e),n.transformFlags=1,n}function un(e,n){return e.members!==n?c(ln(n),e):e}function dn(e){var n=I(181);return n.elementType=d().parenthesizeElementTypeOfArrayType(e),n.transformFlags=1,n}function pn(e,n){return e.elementType!==n?c(dn(n),e):e}function fn(e){var n=I(182);return n.elements=C(e),n.transformFlags=1,n}function mn(e,n){return e.elements!==n?c(fn(n),e):e}function _n(e,n,t,r){var i=I(195);return i.dotDotDotToken=e,i.name=n,i.questionToken=t,i.type=r,i.transformFlags=1,i}function gn(e,n,t,r,i){return e.dotDotDotToken!==n||e.name!==t||e.questionToken!==r||e.type!==i?c(_n(n,t,r,i),e):e}function yn(e){var n=I(183);return n.type=d().parenthesizeElementTypeOfArrayType(e),n.transformFlags=1,n}function hn(e,n){return e.type!==n?c(yn(n),e):e}function vn(e){var n=I(184);return n.type=e,n.transformFlags=1,n}function bn(e,n){return e.type!==n?c(vn(n),e):e}function En(e,n){var t=I(e);return t.types=d().parenthesizeConstituentTypesOfUnionOrIntersectionType(n),t.transformFlags=1,t}function Tn(e,n){return e.types!==n?c(En(e.kind,n),e):e}function Sn(e){return En(185,e)}function xn(e,n){return Tn(e,n)}function Ln(e){return En(186,e)}function Dn(e,n){return Tn(e,n)}function An(e,n,t,r){var i=I(187);return i.checkType=d().parenthesizeMemberOfConditionalType(e),i.extendsType=d().parenthesizeMemberOfConditionalType(n),i.trueType=t,i.falseType=r,i.transformFlags=1,i}function Cn(e,n,t,r,i){return e.checkType!==n||e.extendsType!==t||e.trueType!==r||e.falseType!==i?c(An(n,t,r,i),e):e}function In(e){var n=I(188);return n.typeParameter=e,n.transformFlags=1,n}function Nn(e,n){return e.typeParameter!==n?c(In(n),e):e}function kn(e,n){var t=I(196);return t.head=e,t.templateSpans=C(n),t.transformFlags=1,t}function Rn(e,n,t){return e.head!==n||e.templateSpans!==t?c(kn(n,t),e):e}function On(e,n,t,r){void 0===r&&(r=!1);var i=I(198);return i.argument=e,i.qualifier=n,i.typeArguments=t&&d().parenthesizeTypeArguments(t),i.isTypeOf=r,i.transformFlags=1,i}function wn(e,n,t,r,i){return void 0===i&&(i=e.isTypeOf),e.argument!==n||e.qualifier!==t||e.typeArguments!==r||e.isTypeOf!==i?c(On(n,t,r,i),e):e}function Pn(e){var n=I(189);return n.type=e,n.transformFlags=1,n}function Mn(e,n){return e.type!==n?c(Pn(n),e):e}function Fn(){var e=I(190);return e.transformFlags=1,e}function Gn(e,n){var t=I(191);return t.operator=e,t.type=d().parenthesizeMemberOfElementType(n),t.transformFlags=1,t}function Bn(e,n){return e.type!==n?c(Gn(e.operator,n),e):e}function Un(e,n){var t=I(192);return t.objectType=d().parenthesizeMemberOfElementType(e),t.indexType=n,t.transformFlags=1,t}function Vn(e,n,t){return e.objectType!==n||e.indexType!==t?c(Un(n,t),e):e}function jn(e,n,t,r,i){var a=I(193);return a.readonlyToken=e,a.typeParameter=n,a.nameType=t,a.questionToken=r,a.type=i,a.transformFlags=1,a}function Kn(e,n,t,r,i,a){return e.readonlyToken!==n||e.typeParameter!==t||e.nameType!==r||e.questionToken!==i||e.type!==a?c(jn(n,t,r,i,a),e):e}function Hn(e){var n=I(194);return n.literal=e,n.transformFlags=1,n}function Wn(e,n){return e.literal!==n?c(Hn(n),e):e}function zn(e){var n=I(199);return n.elements=C(e),n.transformFlags|=262656|f(n.elements),16384&n.transformFlags&&(n.transformFlags|=32832),n}function Jn(e,n){return e.elements!==n?c(zn(n),e):e}function qn(e){var n=I(200);return n.elements=C(e),n.transformFlags|=262656|f(n.elements),n}function Xn(e,n){return e.elements!==n?c(qn(n),e):e}function Yn(n,t,r,i){var a=B(201,void 0,void 0,r,i&&d().parenthesizeExpressionForDisallowedComma(i));return a.propertyName=Hs(t),a.dotDotDotToken=n,a.transformFlags|=512|p(a.dotDotDotToken),a.propertyName&&(a.transformFlags|=e.isIdentifier(a.propertyName)?u(a.propertyName):p(a.propertyName)),n&&(a.transformFlags|=16384),a}function Qn(e,n,t,r,i){return e.propertyName!==t||e.dotDotDotToken!==n||e.name!==r||e.initializer!==i?c(Yn(n,t,r,i),e):e}function Zn(e){var n=I(e);return n}function $n(n,t){var r=Zn(202),i=n&&e.lastOrUndefined(n),a=C(n,!(!i||!e.isOmittedExpression(i))||void 0);return r.elements=d().parenthesizeExpressionsOfCommaDelimitedList(a),r.multiLine=t,r.transformFlags|=f(r.elements),r}function et(e,n){return e.elements!==n?c($n(n,e.multiLine),e):e}function nt(e,n){var t=Zn(203);return t.properties=C(e),t.multiLine=n,t.transformFlags|=f(t.properties),t}function tt(e,n){return e.properties!==n?c(nt(n,e.multiLine),e):e}function rt(n,t){var r=Zn(204);return r.expression=d().parenthesizeLeftSideOfAccess(n),r.name=Hs(t),r.transformFlags=p(r.expression)|(e.isIdentifier(r.name)?u(r.name):p(r.name)),e.isSuperKeyword(n)&&(r.transformFlags|=192),r}function it(n,t,r){return e.isPropertyAccessChain(n)?ot(n,t,n.questionDotToken,e.cast(r,e.isIdentifier)):n.expression!==t||n.name!==r?c(rt(t,r),n):n}function at(n,t,r){var i=Zn(204);return i.flags|=32,i.expression=d().parenthesizeLeftSideOfAccess(n),i.questionDotToken=t,i.name=Hs(r),i.transformFlags|=16|p(i.expression)|p(i.questionDotToken)|(e.isIdentifier(i.name)?u(i.name):p(i.name)),i}function ot(n,t,r,i){return e.Debug.assert(!!(32&n.flags),"Cannot update a PropertyAccessExpression using updatePropertyAccessChain. Use updatePropertyAccess instead."),n.expression!==t||n.questionDotToken!==r||n.name!==i?c(at(t,r,i),n):n}function st(n,t){var r=Zn(205);return r.expression=d().parenthesizeLeftSideOfAccess(n),r.argumentExpression=Ws(t),r.transformFlags|=p(r.expression)|p(r.argumentExpression),e.isSuperKeyword(n)&&(r.transformFlags|=192),r}function ct(n,t,r){return e.isElementAccessChain(n)?ut(n,t,n.questionDotToken,r):n.expression!==t||n.argumentExpression!==r?c(st(t,r),n):n}function lt(e,n,t){var r=Zn(205);return r.flags|=32,r.expression=d().parenthesizeLeftSideOfAccess(e),r.questionDotToken=n,r.argumentExpression=Ws(t),r.transformFlags|=p(r.expression)|p(r.questionDotToken)|p(r.argumentExpression)|16,r}function ut(n,t,r,i){return e.Debug.assert(!!(32&n.flags),"Cannot update a ElementAccessExpression using updateElementAccessChain. Use updateElementAccess instead."),n.expression!==t||n.questionDotToken!==r||n.argumentExpression!==i?c(lt(t,r,i),n):n}function dt(n,t,r){var i=Zn(206);return i.expression=d().parenthesizeLeftSideOfAccess(n),i.typeArguments=Ks(t),i.arguments=d().parenthesizeExpressionsOfCommaDelimitedList(C(r)),i.transformFlags|=p(i.expression)|f(i.typeArguments)|f(i.arguments),i.typeArguments&&(i.transformFlags|=1),e.isImportKeyword(i.expression)?i.transformFlags|=4194304:e.isSuperProperty(i.expression)&&(i.transformFlags|=8192),i}function pt(n,t,r,i){return e.isCallChain(n)?mt(n,t,n.questionDotToken,r,i):n.expression!==t||n.typeArguments!==r||n.arguments!==i?c(dt(t,r,i),n):n}function ft(n,t,r,i){var a=Zn(206);return a.flags|=32,a.expression=d().parenthesizeLeftSideOfAccess(n),a.questionDotToken=t,a.typeArguments=Ks(r),a.arguments=d().parenthesizeExpressionsOfCommaDelimitedList(C(i)),a.transformFlags|=p(a.expression)|p(a.questionDotToken)|f(a.typeArguments)|f(a.arguments)|16,a.typeArguments&&(a.transformFlags|=1),e.isSuperProperty(a.expression)&&(a.transformFlags|=8192),a}function mt(n,t,r,i,a){return e.Debug.assert(!!(32&n.flags),"Cannot update a CallExpression using updateCallChain. Use updateCall instead."),n.expression!==t||n.questionDotToken!==r||n.typeArguments!==i||n.arguments!==a?c(ft(t,r,i,a),n):n}function _t(e,n,t){var r=Zn(207);return r.expression=d().parenthesizeExpressionOfNew(e),r.typeArguments=Ks(n),r.arguments=t?d().parenthesizeExpressionsOfCommaDelimitedList(t):void 0,r.transformFlags|=p(r.expression)|f(r.typeArguments)|f(r.arguments)|16,r.typeArguments&&(r.transformFlags|=1),r}function gt(e,n,t,r){return e.expression!==n||e.typeArguments!==t||e.arguments!==r?c(_t(n,t,r),e):e}function yt(n,t,r){var i=Zn(208);return i.tag=d().parenthesizeLeftSideOfAccess(n),i.typeArguments=Ks(t),i.template=r,i.transformFlags|=p(i.tag)|f(i.typeArguments)|p(i.template)|512,i.typeArguments&&(i.transformFlags|=1),e.hasInvalidEscape(i.template)&&(i.transformFlags|=64),i}function ht(e,n,t,r){return e.tag!==n||e.typeArguments!==t||e.template!==r?c(yt(n,t,r),e):e}function vt(e,n){var t=Zn(209);return t.expression=d().parenthesizeOperandOfPrefixUnary(n),t.type=e,t.transformFlags|=p(t.expression)|p(t.type)|1,t}function bt(e,n,t){return e.type!==n||e.expression!==t?c(vt(n,t),e):e}function Et(e){var n=Zn(210);return n.expression=e,n.transformFlags=p(n.expression),n}function Tt(e,n){return e.expression!==n?c(Et(n),e):e}function St(n,t,r,i,a,o,s){var c=P(211,void 0,n,r,i,a,o,s);return c.asteriskToken=t,c.transformFlags|=p(c.asteriskToken),c.typeParameters&&(c.transformFlags|=1),256&e.modifiersToFlags(c.modifiers)?c.asteriskToken?c.transformFlags|=64:c.transformFlags|=128:c.asteriskToken&&(c.transformFlags|=1024),c}function xt(e,n,t,r,i,a,o,s){return e.name!==r||e.modifiers!==n||e.asteriskToken!==t||e.typeParameters!==i||e.parameters!==a||e.type!==o||e.body!==s?M(St(n,t,r,i,a,o,s),e):e}function Lt(n,t,r,i,a,o){var s=P(212,void 0,n,void 0,t,r,i,d().parenthesizeConciseBodyOfArrowFunction(o));return s.equalsGreaterThanToken=null!==a&&void 0!==a?a:ae(38),s.transformFlags|=512|p(s.equalsGreaterThanToken),256&e.modifiersToFlags(s.modifiers)&&(s.transformFlags|=8320),s}function Dt(e,n,t,r,i,a,o){return e.modifiers!==n||e.typeParameters!==t||e.parameters!==r||e.type!==i||e.equalsGreaterThanToken!==a||e.body!==o?M(Lt(n,t,r,i,a,o),e):e}function At(e){var n=Zn(213);return n.expression=d().parenthesizeOperandOfPrefixUnary(e),n.transformFlags|=p(n.expression),n}function Ct(e,n){return e.expression!==n?c(At(n),e):e}function It(e){var n=Zn(214);return n.expression=d().parenthesizeOperandOfPrefixUnary(e),n.transformFlags|=p(n.expression),n}function Nt(e,n){return e.expression!==n?c(It(n),e):e}function kt(e){var n=Zn(215);return n.expression=d().parenthesizeOperandOfPrefixUnary(e),n.transformFlags|=p(n.expression),n}function Rt(e,n){return e.expression!==n?c(kt(n),e):e}function Ot(e){var n=Zn(216);return n.expression=d().parenthesizeOperandOfPrefixUnary(e),n.transformFlags|=1048768|p(n.expression),n}function wt(e,n){return e.expression!==n?c(Ot(n),e):e}function Pt(n,t){var r=Zn(217);return r.operator=n,r.operand=d().parenthesizeOperandOfPrefixUnary(t),r.transformFlags|=p(r.operand),45!==n&&46!==n||!e.isIdentifier(r.operand)||e.isGeneratedIdentifier(r.operand)||e.isLocalName(r.operand)||(r.transformFlags|=67108864),r}function Mt(e,n){return e.operand!==n?c(Pt(e.operator,n),e):e}function Ft(n,t){var r=Zn(218);return r.operator=t,r.operand=d().parenthesizeOperandOfPostfixUnary(n),r.transformFlags|=p(r.operand),!e.isIdentifier(r.operand)||e.isGeneratedIdentifier(r.operand)||e.isLocalName(r.operand)||(r.transformFlags|=67108864),r}function Gt(e,n){return e.operand!==n?c(Ft(n,e.operator),e):e}function Bt(n,t,r){var i=Zn(219),a=zs(t),o=a.kind;return i.left=d().parenthesizeLeftSideOfBinary(o,n),i.operatorToken=a,i.right=d().parenthesizeRightSideOfBinary(o,i.left,r),i.transformFlags|=p(i.left)|p(i.operatorToken)|p(i.right),60===o?i.transformFlags|=16:63===o?e.isObjectLiteralExpression(i.left)?i.transformFlags|=2624|Ut(i.left):e.isArrayLiteralExpression(i.left)&&(i.transformFlags|=2560|Ut(i.left)):42===o||67===o?i.transformFlags|=256:e.isLogicalOrCoalescingAssignmentOperator(o)&&(i.transformFlags|=8),i}function Ut(n){if(32768&n.transformFlags)return 32768;if(64&n.transformFlags)for(var t=0,r=e.getElementsOfBindingOrAssignmentPattern(n);t<r.length;t++){var i=r[t],a=e.getTargetOfBindingOrAssignmentElement(i);if(a&&e.isAssignmentPattern(a)){if(32768&a.transformFlags)return 32768;if(64&a.transformFlags){var o=Ut(a);if(o)return o}}}return 0}function Vt(e,n,t,r){return e.left!==n||e.operatorToken!==t||e.right!==r?c(Bt(n,t,r),e):e}function jt(e,n,t,r,i){var a=Zn(220);return a.condition=d().parenthesizeConditionOfConditionalExpression(e),a.questionToken=null!==n&&void 0!==n?n:ae(57),a.whenTrue=d().parenthesizeBranchOfConditionalExpression(t),a.colonToken=null!==r&&void 0!==r?r:ae(58),a.whenFalse=d().parenthesizeBranchOfConditionalExpression(i),a.transformFlags|=p(a.condition)|p(a.questionToken)|p(a.whenTrue)|p(a.colonToken)|p(a.whenFalse),a}function Kt(e,n,t,r,i,a){return e.condition!==n||e.questionToken!==t||e.whenTrue!==r||e.colonToken!==i||e.whenFalse!==a?c(jt(n,t,r,i,a),e):e}function Ht(e,n){var t=Zn(221);return t.head=e,t.templateSpans=C(n),t.transformFlags|=p(t.head)|f(t.templateSpans)|512,t}function Wt(e,n,t){return e.head!==n||e.templateSpans!==t?c(Ht(n,t),e):e}function zt(n,t,r,i){void 0===i&&(i=0),e.Debug.assert(!(-2049&i),"Unsupported template flags.");var a=void 0;if(void 0!==r&&r!==t&&(a=l(n,r),"object"===typeof a))return e.Debug.fail("Invalid raw text");if(void 0===t){if(void 0===a)return e.Debug.fail("Arguments 'text' and 'rawText' may not both be undefined.");t=a}else void 0!==a&&e.Debug.assert(t===a,"Expected argument 'text' to be the normalized (i.e. 'cooked') version of argument 'rawText'.");return Jt(n,t,r,i)}function Jt(e,n,t,r){var i=ie(e);return i.text=n,i.rawText=t,i.templateFlags=2048&r,i.transformFlags|=512,i.templateFlags&&(i.transformFlags|=64),i}function qt(e,n,t){return zt(15,e,n,t)}function Xt(e,n,t){return zt(16,e,n,t)}function Yt(e,n,t){return zt(17,e,n,t)}function Qt(e,n,t){return zt(14,e,n,t)}function Zt(n,t){e.Debug.assert(!n||!!t,"A `YieldExpression` with an asteriskToken must have an expression.");var r=Zn(222);return r.expression=t&&d().parenthesizeExpressionForDisallowedComma(t),r.asteriskToken=n,r.transformFlags|=524864|(p(r.expression)|p(r.asteriskToken)),r}function $t(e,n,t){return e.expression!==t||e.asteriskToken!==n?c(Zt(n,t),e):e}function er(e){var n=Zn(223);return n.expression=d().parenthesizeExpressionForDisallowedComma(e),n.transformFlags|=16896|p(n.expression),n}function nr(e,n){return e.expression!==n?c(er(n),e):e}function tr(e,n,t,r,i,a){var o=G(224,e,n,t,r,i,a);return o.transformFlags|=512,o}function rr(e,n,t,r,i,a,o){return e.decorators!==n||e.modifiers!==t||e.name!==r||e.typeParameters!==i||e.heritageClauses!==a||e.members!==o?c(tr(n,t,r,i,a,o),e):e}function ir(){return Zn(225)}function ar(e,n){var t=I(226);return t.expression=d().parenthesizeLeftSideOfAccess(e),t.typeArguments=n&&d().parenthesizeTypeArguments(n),t.transformFlags|=p(t.expression)|f(t.typeArguments)|512,t}function or(e,n,t){return e.expression!==n||e.typeArguments!==t?c(ar(n,t),e):e}function sr(e,n){var t=Zn(227);return t.expression=e,t.type=n,t.transformFlags|=p(t.expression)|p(t.type)|1,t}function cr(e,n,t){return e.expression!==n||e.type!==t?c(sr(n,t),e):e}function lr(e){var n=Zn(228);return n.expression=d().parenthesizeLeftSideOfAccess(e),n.transformFlags|=1|p(n.expression),n}function ur(n,t){return e.isNonNullChain(n)?pr(n,t):n.expression!==t?c(lr(t),n):n}function dr(e){var n=Zn(228);return n.flags|=32,n.expression=d().parenthesizeLeftSideOfAccess(e),n.transformFlags|=1|p(n.expression),n}function pr(n,t){return e.Debug.assert(!!(32&n.flags),"Cannot update a NonNullExpression using updateNonNullChain. Use updateNonNullExpression instead."),n.expression!==t?c(dr(t),n):n}function fr(n,t){var r=Zn(229);switch(r.keywordToken=n,r.name=t,r.transformFlags|=p(r.name),n){case 103:r.transformFlags|=512;break;case 100:r.transformFlags|=4;break;default:return e.Debug.assertNever(n)}return r}function mr(e,n){return e.name!==n?c(fr(e.keywordToken,n),e):e}function _r(e,n){var t=I(231);return t.expression=e,t.literal=n,t.transformFlags|=p(t.expression)|p(t.literal)|512,t}function gr(e,n,t){return e.expression!==n||e.literal!==t?c(_r(n,t),e):e}function yr(){var e=I(232);return e.transformFlags|=512,e}function hr(e,n){var t=I(233);return t.statements=C(e),t.multiLine=n,t.transformFlags|=f(t.statements),t}function vr(e,n){return e.statements!==n?c(hr(n,e.multiLine),e):e}function br(n,t){var r=N(235,void 0,n);return r.declarationList=e.isArray(t)?ti(t):t,r.transformFlags|=p(r.declarationList),2&e.modifiersToFlags(r.modifiers)&&(r.transformFlags=1),r}function Er(e,n,t){return e.modifiers!==n||e.declarationList!==t?c(br(n,t),e):e}function Tr(){return I(234)}function Sr(e){var n=I(236);return n.expression=d().parenthesizeExpressionOfExpressionStatement(e),n.transformFlags|=p(n.expression),n}function xr(e,n){return e.expression!==n?c(Sr(n),e):e}function Lr(e,n,t){var r=I(237);return r.expression=e,r.thenStatement=Js(n),r.elseStatement=Js(t),r.transformFlags|=p(r.expression)|p(r.thenStatement)|p(r.elseStatement),r}function Dr(e,n,t,r){return e.expression!==n||e.thenStatement!==t||e.elseStatement!==r?c(Lr(n,t,r),e):e}function Ar(e,n){var t=I(238);return t.statement=Js(e),t.expression=n,t.transformFlags|=p(t.statement)|p(t.expression),t}function Cr(e,n,t){return e.statement!==n||e.expression!==t?c(Ar(n,t),e):e}function Ir(e,n){var t=I(239);return t.expression=e,t.statement=Js(n),t.transformFlags|=p(t.expression)|p(t.statement),t}function Nr(e,n,t){return e.expression!==n||e.statement!==t?c(Ir(n,t),e):e}function kr(e,n,t,r){var i=I(240);return i.initializer=e,i.condition=n,i.incrementor=t,i.statement=Js(r),i.transformFlags|=p(i.initializer)|p(i.condition)|p(i.incrementor)|p(i.statement),i}function Rr(e,n,t,r,i){return e.initializer!==n||e.condition!==t||e.incrementor!==r||e.statement!==i?c(kr(n,t,r,i),e):e}function Or(e,n,t){var r=I(241);return r.initializer=e,r.expression=n,r.statement=Js(t),r.transformFlags|=p(r.initializer)|p(r.expression)|p(r.statement),r}function wr(e,n,t,r){return e.initializer!==n||e.expression!==t||e.statement!==r?c(Or(n,t,r),e):e}function Pr(e,n,t,r){var i=I(242);return i.awaitModifier=e,i.initializer=n,i.expression=d().parenthesizeExpressionForDisallowedComma(t),i.statement=Js(r),i.transformFlags|=p(i.awaitModifier)|p(i.initializer)|p(i.expression)|p(i.statement)|512,e&&(i.transformFlags|=64),i}function Mr(e,n,t,r,i){return e.awaitModifier!==n||e.initializer!==t||e.expression!==r||e.statement!==i?c(Pr(n,t,r,i),e):e}function Fr(e){var n=I(243);return n.label=Hs(e),n.transformFlags|=2097152|p(n.label),n}function Gr(e,n){return e.label!==n?c(Fr(n),e):e}function Br(e){var n=I(244);return n.label=Hs(e),n.transformFlags|=2097152|p(n.label),n}function Ur(e,n){return e.label!==n?c(Br(n),e):e}function Vr(e){var n=I(245);return n.expression=e,n.transformFlags|=2097216|p(n.expression),n}function jr(e,n){return e.expression!==n?c(Vr(n),e):e}function Kr(e,n){var t=I(246);return t.expression=e,t.statement=Js(n),t.transformFlags|=p(t.expression)|p(t.statement),t}function Hr(e,n,t){return e.expression!==n||e.statement!==t?c(Kr(n,t),e):e}function Wr(e,n){var t=I(247);return t.expression=d().parenthesizeExpressionForDisallowedComma(e),t.caseBlock=n,t.transformFlags|=p(t.expression)|p(t.caseBlock),t}function zr(e,n,t){return e.expression!==n||e.caseBlock!==t?c(Wr(n,t),e):e}function Jr(e,n){var t=I(248);return t.label=Hs(e),t.statement=Js(n),t.transformFlags|=p(t.label)|p(t.statement),t}function qr(e,n,t){return e.label!==n||e.statement!==t?c(Jr(n,t),e):e}function Xr(e){var n=I(249);return n.expression=e,n.transformFlags|=p(n.expression),n}function Yr(e,n){return e.expression!==n?c(Xr(n),e):e}function Qr(e,n,t){var r=I(250);return r.tryBlock=e,r.catchClause=n,r.finallyBlock=t,r.transformFlags|=p(r.tryBlock)|p(r.catchClause)|p(r.finallyBlock),r}function Zr(e,n,t,r){return e.tryBlock!==n||e.catchClause!==t||e.finallyBlock!==r?c(Qr(n,t,r),e):e}function $r(){return I(251)}function ei(e,n,t,r){var i=U(252,void 0,void 0,e,t,r&&d().parenthesizeExpressionForDisallowedComma(r));return i.exclamationToken=n,i.transformFlags|=p(i.exclamationToken),n&&(i.transformFlags|=1),i}function ni(e,n,t,r,i){return e.name!==n||e.type!==r||e.exclamationToken!==t||e.initializer!==i?c(ei(n,t,r,i),e):e}function ti(e,n){void 0===n&&(n=0);var t=I(253);return t.flags|=3&n,t.declarations=C(e),t.transformFlags|=2097152|f(t.declarations),3&n&&(t.transformFlags|=131584),t}function ri(e,n){return e.declarations!==n?c(ti(n,e.flags),e):e}function ii(n,t,r,i,a,o,s,c){var l=P(254,n,t,i,a,o,s,c);return l.asteriskToken=r,!l.body||2&e.modifiersToFlags(l.modifiers)?l.transformFlags=1:(l.transformFlags|=2097152|p(l.asteriskToken),256&e.modifiersToFlags(l.modifiers)?l.asteriskToken?l.transformFlags|=64:l.transformFlags|=128:l.asteriskToken&&(l.transformFlags|=1024)),l}function ai(e,n,t,r,i,a,o,s,c){return e.decorators!==n||e.modifiers!==t||e.asteriskToken!==r||e.name!==i||e.typeParameters!==a||e.parameters!==o||e.type!==s||e.body!==c?M(ii(n,t,r,i,a,o,s,c),e):e}function oi(n,t,r,i,a,o){var s=G(255,n,t,r,i,a,o);return 2&e.modifiersToFlags(s.modifiers)?s.transformFlags=1:(s.transformFlags|=512,4096&s.transformFlags&&(s.transformFlags|=1)),s}function si(e,n,t,r,i,a,o){return e.decorators!==n||e.modifiers!==t||e.name!==r||e.typeParameters!==i||e.heritageClauses!==a||e.members!==o?c(oi(n,t,r,i,a,o),e):e}function ci(e,n,t,r,i,a){var o=F(256,e,n,t,r,i);return o.members=C(a),o.transformFlags=1,o}function li(e,n,t,r,i,a,o){return e.decorators!==n||e.modifiers!==t||e.name!==r||e.typeParameters!==i||e.heritageClauses!==a||e.members!==o?c(ci(n,t,r,i,a,o),e):e}function ui(e,n,t,r,i){var a=R(257,e,n,t,r);return a.type=i,a.transformFlags=1,a}function di(e,n,t,r,i,a){return e.decorators!==n||e.modifiers!==t||e.name!==r||e.typeParameters!==i||e.type!==a?c(ui(n,t,r,i,a),e):e}function pi(e,n,t,r){var i=k(258,e,n,t);return i.members=C(r),i.transformFlags|=1|f(i.members),i.transformFlags&=-16777217,i}function fi(e,n,t,r,i){return e.decorators!==n||e.modifiers!==t||e.name!==r||e.members!==i?c(pi(n,t,r,i),e):e}function mi(n,t,r,i,a){void 0===a&&(a=0);var o=N(259,n,t);return o.flags|=1044&a,o.name=r,o.body=i,2&e.modifiersToFlags(o.modifiers)?o.transformFlags=1:o.transformFlags|=p(o.name)|p(o.body)|1,o.transformFlags&=-16777217,o}function _i(e,n,t,r,i){return e.decorators!==n||e.modifiers!==t||e.name!==r||e.body!==i?c(mi(n,t,r,i,e.flags),e):e}function gi(e){var n=I(260);return n.statements=C(e),n.transformFlags|=f(n.statements),n}function yi(e,n){return e.statements!==n?c(gi(n),e):e}function hi(e){var n=I(261);return n.clauses=C(e),n.transformFlags|=f(n.clauses),n}function vi(e,n){return e.clauses!==n?c(hi(n),e):e}function bi(e){var n=k(262,void 0,void 0,e);return n.transformFlags=1,n}function Ei(e,n){return e.name!==n?c(bi(n),e):e}function Ti(n,t,r,i,a){var o=k(263,n,t,i);return o.isTypeOnly=r,o.moduleReference=a,o.transformFlags|=p(o.moduleReference),e.isExternalModuleReference(o.moduleReference)||(o.transformFlags|=1),o.transformFlags&=-16777217,o}function Si(e,n,t,r,i,a){return e.decorators!==n||e.modifiers!==t||e.isTypeOnly!==r||e.name!==i||e.moduleReference!==a?c(Ti(n,t,r,i,a),e):e}function xi(e,n,t,r){var i=N(264,e,n);return i.importClause=t,i.moduleSpecifier=r,i.transformFlags|=p(i.importClause)|p(i.moduleSpecifier),i.transformFlags&=-16777217,i}function Li(e,n,t,r,i){return e.decorators!==n||e.modifiers!==t||e.importClause!==r||e.moduleSpecifier!==i?c(xi(n,t,r,i),e):e}function Di(e,n,t){var r=I(265);return r.isTypeOnly=e,r.name=n,r.namedBindings=t,r.transformFlags|=p(r.name)|p(r.namedBindings),e&&(r.transformFlags|=1),r.transformFlags&=-16777217,r}function Ai(e,n,t,r){return e.isTypeOnly!==n||e.name!==t||e.namedBindings!==r?c(Di(n,t,r),e):e}function Ci(e){var n=I(266);return n.name=e,n.transformFlags|=p(n.name),n.transformFlags&=-16777217,n}function Ii(e,n){return e.name!==n?c(Ci(n),e):e}function Ni(e){var n=I(272);return n.name=e,n.transformFlags|=4|p(n.name),n.transformFlags&=-16777217,n}function ki(e,n){return e.name!==n?c(Ni(n),e):e}function Ri(e){var n=I(267);return n.elements=C(e),n.transformFlags|=f(n.elements),n.transformFlags&=-16777217,n}function Oi(e,n){return e.elements!==n?c(Ri(n),e):e}function wi(e,n){var t=I(268);return t.propertyName=e,t.name=n,t.transformFlags|=p(t.propertyName)|p(t.name),t.transformFlags&=-16777217,t}function Pi(e,n,t){return e.propertyName!==n||e.name!==t?c(wi(n,t),e):e}function Mi(e,n,t,r){var i=N(269,e,n);return i.isExportEquals=t,i.expression=t?d().parenthesizeRightSideOfBinary(63,void 0,r):d().parenthesizeExpressionOfExportDefault(r),i.transformFlags|=p(i.expression),i.transformFlags&=-16777217,i}function Fi(e,n,t,r){return e.decorators!==n||e.modifiers!==t||e.expression!==r?c(Mi(n,t,e.isExportEquals,r),e):e}function Gi(e,n,t,r,i){var a=N(270,e,n);return a.isTypeOnly=t,a.exportClause=r,a.moduleSpecifier=i,a.transformFlags|=p(a.exportClause)|p(a.moduleSpecifier),a.transformFlags&=-16777217,a}function Bi(e,n,t,r,i,a){return e.decorators!==n||e.modifiers!==t||e.isTypeOnly!==r||e.exportClause!==i||e.moduleSpecifier!==a?c(Gi(n,t,r,i,a),e):e}function Ui(e){var n=I(271);return n.elements=C(e),n.transformFlags|=f(n.elements),n.transformFlags&=-16777217,n}function Vi(e,n){return e.elements!==n?c(Ui(n),e):e}function ji(e,n){var t=I(273);return t.propertyName=Hs(e),t.name=Hs(n),t.transformFlags|=p(t.propertyName)|p(t.name),t.transformFlags&=-16777217,t}function Ki(e,n,t){return e.propertyName!==n||e.name!==t?c(ji(n,t),e):e}function Hi(){var e=N(274,void 0,void 0);return e}function Wi(e){var n=I(275);return n.expression=e,n.transformFlags|=p(n.expression),n.transformFlags&=-16777217,n}function zi(e,n){return e.expression!==n?c(Wi(n),e):e}function Ji(e){return I(e)}function qi(e,n){var t=I(e);return t.type=n,t}function Xi(e,n,t){return n.type!==t?c(qi(e,t),n):n}function Yi(e,n){var t=O(312,void 0,void 0,void 0,void 0,e,n);return t}function Qi(e,n,t){return e.parameters!==n||e.type!==t?c(Yi(n,t),e):e}function Zi(e,n){void 0===n&&(n=!1);var t=I(317);return t.jsDocPropertyTags=Ks(e),t.isArrayType=n,t}function $i(e,n,t){return e.jsDocPropertyTags!==n||e.isArrayType!==t?c(Zi(n,t),e):e}function ea(e){var n=I(304);return n.type=e,n}function na(e,n){return e.type!==n?c(ea(n),e):e}function ta(e,n,t){var r=I(318);return r.typeParameters=Ks(e),r.parameters=C(n),r.type=t,r}function ra(e,n,t,r){return e.typeParameters!==n||e.parameters!==t||e.type!==r?c(ta(n,t,r),e):e}function ia(n){var t=s(n.kind);return n.tagName.escapedText===e.escapeLeadingUnderscores(t)?n.tagName:Q(t)}function aa(e,n,t){var r=I(e);return r.tagName=n,r.comment=t,r}function oa(e,n,t,r){var i=aa(339,null!==e&&void 0!==e?e:Q("template"),r);return i.constraint=n,i.typeParameters=C(t),i}function sa(e,n,t,r,i){return void 0===n&&(n=ia(e)),e.tagName!==n||e.constraint!==t||e.typeParameters!==r||e.comment!==i?c(oa(n,t,r,i),e):e}function ca(n,t,r,i){var a=aa(340,null!==n&&void 0!==n?n:Q("typedef"),i);return a.typeExpression=t,a.fullName=r,a.name=e.getJSDocTypeAliasName(r),a}function la(e,n,t,r,i){return void 0===n&&(n=ia(e)),e.tagName!==n||e.typeExpression!==t||e.fullName!==r||e.comment!==i?c(ca(n,t,r,i),e):e}function ua(e,n,t,r,i,a){var o=aa(335,null!==e&&void 0!==e?e:Q("param"),a);return o.typeExpression=r,o.name=n,o.isNameFirst=!!i,o.isBracketed=t,o}function da(e,n,t,r,i,a,o){return void 0===n&&(n=ia(e)),e.tagName!==n||e.name!==t||e.isBracketed!==r||e.typeExpression!==i||e.isNameFirst!==a||e.comment!==o?c(ua(n,t,r,i,a,o),e):e}function pa(e,n,t,r,i,a){var o=aa(342,null!==e&&void 0!==e?e:Q("prop"),a);return o.typeExpression=r,o.name=n,o.isNameFirst=!!i,o.isBracketed=t,o}function fa(e,n,t,r,i,a,o){return void 0===n&&(n=ia(e)),e.tagName!==n||e.name!==t||e.isBracketed!==r||e.typeExpression!==i||e.isNameFirst!==a||e.comment!==o?c(pa(n,t,r,i,a,o),e):e}function ma(n,t,r,i){var a=aa(333,null!==n&&void 0!==n?n:Q("callback"),i);return a.typeExpression=t,a.fullName=r,a.name=e.getJSDocTypeAliasName(r),a}function _a(e,n,t,r,i){return void 0===n&&(n=ia(e)),e.tagName!==n||e.typeExpression!==t||e.fullName!==r||e.comment!==i?c(ma(n,t,r,i),e):e}function ga(e,n,t){var r=aa(323,null!==e&&void 0!==e?e:Q("augments"),t);return r.class=n,r}function ya(e,n,t,r){return void 0===n&&(n=ia(e)),e.tagName!==n||e.class!==t||e.comment!==r?c(ga(n,t,r),e):e}function ha(e,n,t){var r=aa(324,null!==e&&void 0!==e?e:Q("implements"),t);return r.class=n,r}function va(e,n,t){var r=aa(341,null!==e&&void 0!==e?e:Q("see"),t);return r.name=n,r}function ba(e,n,t,r){return e.tagName!==n||e.name!==t||e.comment!==r?c(va(n,t,r),e):e}function Ea(e){var n=I(305);return n.name=e,n}function Ta(e,n){return e.name!==n?c(Ea(n),e):e}function Sa(e,n){var t=I(306);return t.left=e,t.right=n,t.transformFlags|=p(t.left)|p(t.right),t}function xa(e,n,t){return e.left!==n||e.right!==t?c(Sa(n,t),e):e}function La(e,n){var t=I(319);return t.name=e,t.text=n,t}function Da(e,n,t){return e.name!==n?c(La(n,t),e):e}function Aa(e,n){var t=I(320);return t.name=e,t.text=n,t}function Ca(e,n,t){return e.name!==n?c(Aa(n,t),e):e}function Ia(e,n){var t=I(321);return t.name=e,t.text=n,t}function Na(e,n,t){return e.name!==n?c(Ia(n,t),e):e}function ka(e,n,t,r){return void 0===n&&(n=ia(e)),e.tagName!==n||e.class!==t||e.comment!==r?c(ha(n,t,r),e):e}function Ra(e,n,t){var r=aa(e,null!==n&&void 0!==n?n:Q(s(e)),t);return r}function Oa(e,n,t,r){return void 0===t&&(t=ia(n)),n.tagName!==t||n.comment!==r?c(Ra(e,t,r),n):n}function wa(e,n,t,r){var i=aa(e,null!==n&&void 0!==n?n:Q(s(e)),r);return i.typeExpression=t,i}function Pa(e,n,t,r,i){return void 0===t&&(t=ia(n)),n.tagName!==t||n.typeExpression!==r||n.comment!==i?c(wa(e,t,r,i),n):n}function Ma(e,n){var t=aa(322,e,n);return t}function Fa(e,n,t){return e.tagName!==n||e.comment!==t?c(Ma(n,t),e):e}function Ga(e){var n=I(316);return n.text=e,n}function Ba(e,n){return e.text!==n?c(Ga(n),e):e}function Ua(e,n){var t=I(315);return t.comment=e,t.tags=Ks(n),t}function Va(e,n,t){return e.comment!==n||e.tags!==t?c(Ua(n,t),e):e}function ja(e,n,t){var r=I(276);return r.openingElement=e,r.children=C(n),r.closingElement=t,r.transformFlags|=p(r.openingElement)|f(r.children)|p(r.closingElement)|2,r}function Ka(e,n,t,r){return e.openingElement!==n||e.children!==t||e.closingElement!==r?c(ja(n,t,r),e):e}function Ha(e,n,t){var r=I(277);return r.tagName=e,r.typeArguments=Ks(n),r.attributes=t,r.transformFlags|=p(r.tagName)|f(r.typeArguments)|p(r.attributes)|2,r.typeArguments&&(r.transformFlags|=1),r}function Wa(e,n,t,r){return e.tagName!==n||e.typeArguments!==t||e.attributes!==r?c(Ha(n,t,r),e):e}function za(e,n,t){var r=I(278);return r.tagName=e,r.typeArguments=Ks(n),r.attributes=t,r.transformFlags|=p(r.tagName)|f(r.typeArguments)|p(r.attributes)|2,n&&(r.transformFlags|=1),r}function Ja(e,n,t,r){return e.tagName!==n||e.typeArguments!==t||e.attributes!==r?c(za(n,t,r),e):e}function qa(e){var n=I(279);return n.tagName=e,n.transformFlags|=2|p(n.tagName),n}function Xa(e,n){return e.tagName!==n?c(qa(n),e):e}function Ya(e,n,t){var r=I(280);return r.openingFragment=e,r.children=C(n),r.closingFragment=t,r.transformFlags|=p(r.openingFragment)|f(r.children)|p(r.closingFragment)|2,r}function Qa(e,n,t,r){return e.openingFragment!==n||e.children!==t||e.closingFragment!==r?c(Ya(n,t,r),e):e}function Za(e,n){var t=I(11);return t.text=e,t.containsOnlyTriviaWhiteSpaces=!!n,t.transformFlags|=2,t}function $a(e,n,t){return e.text!==n||e.containsOnlyTriviaWhiteSpaces!==t?c(Za(n,t),e):e}function eo(){var e=I(281);return e.transformFlags|=2,e}function no(){var e=I(282);return e.transformFlags|=2,e}function to(e,n){var t=I(283);return t.name=e,t.initializer=n,t.transformFlags|=p(t.name)|p(t.initializer)|2,t}function ro(e,n,t){return e.name!==n||e.initializer!==t?c(to(n,t),e):e}function io(e){var n=I(284);return n.properties=C(e),n.transformFlags|=2|f(n.properties),n}function ao(e,n){return e.properties!==n?c(io(n),e):e}function oo(e){var n=I(285);return n.expression=e,n.transformFlags|=2|p(n.expression),n}function so(e,n){return e.expression!==n?c(oo(n),e):e}function co(e,n){var t=I(286);return t.dotDotDotToken=e,t.expression=n,t.transformFlags|=p(t.dotDotDotToken)|p(t.expression)|2,t}function lo(e,n){return e.expression!==n?c(co(e.dotDotDotToken,n),e):e}function uo(e,n){var t=I(287);return t.expression=d().parenthesizeExpressionForDisallowedComma(e),t.statements=C(n),t.transformFlags|=p(t.expression)|f(t.statements),t}function po(e,n,t){return e.expression!==n||e.statements!==t?c(uo(n,t),e):e}function fo(e){var n=I(288);return n.statements=C(e),n.transformFlags=f(n.statements),n}function mo(e,n){return e.statements!==n?c(fo(n),e):e}function _o(n,t){var r=I(289);switch(r.token=n,r.types=C(t),r.transformFlags|=f(r.types),n){case 94:r.transformFlags|=512;break;case 117:r.transformFlags|=1;break;default:return e.Debug.assertNever(n)}return r}function go(e,n){return e.types!==n?c(_o(e.token,n),e):e}function yo(n,t){var r=I(290);return n=e.isString(n)?ei(n,void 0,void 0,void 0):n,r.variableDeclaration=n,r.block=t,r.transformFlags|=p(r.variableDeclaration)|p(r.block),n||(r.transformFlags|=32),r}function ho(e,n,t){return e.variableDeclaration!==n||e.block!==t?c(yo(n,t),e):e}function vo(e,n){var t=k(291,void 0,void 0,e);return t.initializer=d().parenthesizeExpressionForDisallowedComma(n),t.transformFlags|=p(t.name)|p(t.initializer),t}function bo(e,n){return n.decorators&&(e.decorators=n.decorators),n.modifiers&&(e.modifiers=n.modifiers),n.questionToken&&(e.questionToken=n.questionToken),n.exclamationToken&&(e.exclamationToken=n.exclamationToken),c(e,n)}function Eo(e,n,t){return e.name!==n||e.initializer!==t?bo(vo(n,t),e):e}function To(e,n){var t=k(292,void 0,void 0,e);return t.objectAssignmentInitializer=n&&d().parenthesizeExpressionForDisallowedComma(n),t.transformFlags|=512|p(t.objectAssignmentInitializer),t}function So(e,n){return n.decorators&&(e.decorators=n.decorators),n.modifiers&&(e.modifiers=n.modifiers),n.equalsToken&&(e.equalsToken=n.equalsToken),n.questionToken&&(e.questionToken=n.questionToken),n.exclamationToken&&(e.exclamationToken=n.exclamationToken),c(e,n)}function xo(e,n,t){return e.name!==n||e.objectAssignmentInitializer!==t?So(To(n,t),e):e}function Lo(e){var n=I(293);return n.expression=d().parenthesizeExpressionForDisallowedComma(e),n.transformFlags|=32832|p(n.expression),n}function Do(e,n){return e.expression!==n?c(Lo(n),e):e}function Ao(e,n){var t=I(294);return t.name=Hs(e),t.initializer=n&&d().parenthesizeExpressionForDisallowedComma(n),t.transformFlags|=p(t.name)|p(t.initializer)|1,t}function Co(e,n,t){return e.name!==n||e.initializer!==t?c(Ao(n,t),e):e}function Io(e,n,t){var i=r.createBaseSourceFileNode(300);return i.statements=C(e),i.endOfFileToken=n,i.flags|=t,i.fileName="",i.text="",i.languageVersion=0,i.languageVariant=0,i.scriptKind=0,i.isDeclarationFile=!1,i.hasNoDefaultLib=!1,i.transformFlags|=f(i.statements)|p(i.endOfFileToken),i}function No(n,t,i,a,o,s,c){var l=r.createBaseSourceFileNode(300);for(var u in n)"emitNode"!==u&&!e.hasProperty(l,u)&&e.hasProperty(n,u)&&(l[u]=n[u]);return l.flags|=n.flags,l.statements=C(t),l.endOfFileToken=n.endOfFileToken,l.isDeclarationFile=i,l.referencedFiles=a,l.typeReferenceDirectives=o,l.hasNoDefaultLib=s,l.libReferenceDirectives=c,l.transformFlags=f(l.statements)|p(l.endOfFileToken),l}function ko(e,n,t,r,i,a,o){return void 0===t&&(t=e.isDeclarationFile),void 0===r&&(r=e.referencedFiles),void 0===i&&(i=e.typeReferenceDirectives),void 0===a&&(a=e.hasNoDefaultLib),void 0===o&&(o=e.libReferenceDirectives),e.statements!==n||e.isDeclarationFile!==t||e.referencedFiles!==r||e.typeReferenceDirectives!==i||e.hasNoDefaultLib!==a||e.libReferenceDirectives!==o?c(No(e,n,t,r,i,a,o),e):e}function Ro(n,t){void 0===t&&(t=e.emptyArray);var r=I(301);return r.prepends=t,r.sourceFiles=n,r}function Oo(n,t,r){return void 0===r&&(r=e.emptyArray),n.sourceFiles!==t||n.prepends!==r?c(Ro(t,r),n):n}function wo(n,t,r){var i=I(302);return i.prologues=n,i.syntheticReferences=t,i.texts=r,i.fileName="",i.text="",i.referencedFiles=e.emptyArray,i.libReferenceDirectives=e.emptyArray,i.getLineAndCharacterOfPosition=function(n){return e.getLineAndCharacterOfPosition(i,n)},i}function Po(e,n){var t=I(e);return t.data=n,t}function Mo(e){return Po(295,e)}function Fo(e,n){var t=Po(296,e);return t.texts=n,t}function Go(e,n){return Po(n?298:297,e)}function Bo(e){var n=I(299);return n.data=e.data,n.section=e,n}function Uo(){var e=I(303);return e.javascriptText="",e.declarationText="",e}function Vo(e,n,t){void 0===n&&(n=!1);var r=I(230);return r.type=e,r.isSpread=n,r.tupleNameSource=t,r}function jo(e){var n=I(343);return n._children=e,n}function Ko(n){var t=I(344);return t.original=n,e.setTextRange(t,n),t}function Ho(n,t){var r=I(345);return r.expression=n,r.original=t,r.transformFlags|=1|p(r.expression),e.setTextRange(r,t),r}function Wo(e,n){return e.expression!==n?c(Ho(n,e.original),e):e}function zo(n){if(e.nodeIsSynthesized(n)&&!e.isParseTreeNode(n)&&!n.original&&!n.emitNode&&!n.id){if(e.isCommaListExpression(n))return n.elements;if(e.isBinaryExpression(n)&&e.isCommaToken(n.operatorToken))return[n.left,n.right]}return n}function Jo(n){var t=I(346);return t.elements=C(e.sameFlatMap(n,zo)),t.transformFlags|=f(t.elements),t}function qo(e,n){return e.elements!==n?c(Jo(n),e):e}function Xo(e){var n=I(348);return n.emitNode={},n.original=e,n}function Yo(e){var n=I(347);return n.emitNode={},n.original=e,n}function Qo(e,n){var t=I(349);return t.expression=e,t.thisArg=n,t.transformFlags|=p(t.expression)|p(t.thisArg),t}function Zo(e,n,t){return e.expression!==n||e.thisArg!==t?c(Qo(n,t),e):e}function $o(n){if(void 0===n)return n;var t=e.isSourceFile(n)?r.createBaseSourceFileNode(300):e.isIdentifier(n)?r.createBaseIdentifierNode(79):e.isPrivateIdentifier(n)?r.createBasePrivateIdentifierNode(80):e.isNodeKind(n.kind)?r.createBaseNode(n.kind):r.createBaseTokenNode(n.kind);for(var i in t.flags|=-9&n.flags,t.transformFlags=n.transformFlags,L(t,n),n)!t.hasOwnProperty(i)&&n.hasOwnProperty(i)&&(t[i]=n[i]);return t}function es(e,n,t){return dt(St(void 0,void 0,void 0,void 0,n?[n]:[],void 0,hr(e,!0)),void 0,t?[t]:[])}function ns(e,n,t){return dt(Lt(void 0,void 0,n?[n]:[],void 0,void 0,hr(e,!0)),void 0,t?[t]:[])}function ts(){return kt(j("0"))}function rs(e){return Mi(void 0,void 0,!1,e)}function is(e){return Gi(void 0,void 0,!1,Ui([ji(void 0,e)]))}function as(e,n){return"undefined"===n?A.createStrictEquality(e,ts()):A.createStrictEquality(It(e),W(n))}function os(n,t,r){return e.isCallChain(n)?ft(at(n,void 0,t),void 0,void 0,r):dt(rt(n,t),void 0,r)}function ss(e,n,t){return os(e,"bind",a([n],t,!0))}function cs(e,n,t){return os(e,"call",a([n],t,!0))}function ls(e,n,t){return os(e,"apply",[n,t])}function us(e,n,t){return os(Q(e),n,t)}function ds(e,n){return os(e,"slice",void 0===n?[]:[Ws(n)])}function ps(e,n){return os(e,"concat",n)}function fs(e,n,t){return us("Object","defineProperty",[e,Ws(n),t])}function ms(e,n,t){return us("Reflect","get",t?[e,n,t]:[e,n])}function _s(e,n,t,r){return us("Reflect","set",r?[e,n,t,r]:[e,n,t])}function gs(e,n,t){return!!t&&(e.push(vo(n,t)),!0)}function ys(n,t){var r=[];gs(r,"enumerable",Ws(n.enumerable)),gs(r,"configurable",Ws(n.configurable));var i=gs(r,"writable",Ws(n.writable));i=gs(r,"value",n.value)||i;var a=gs(r,"get",n.get);return a=gs(r,"set",n.set)||a,e.Debug.assert(!(i&&a),"A PropertyDescriptor may not be both an accessor descriptor and a data descriptor."),nt(r,!t)}function hs(e,n){switch(e.kind){case 210:return Tt(e,n);case 209:return bt(e,e.type,n);case 227:return cr(e,n,e.type);case 228:return ur(e,n);case 345:return Wo(e,n)}}function vs(n){return e.isParenthesizedExpression(n)&&e.nodeIsSynthesized(n)&&e.nodeIsSynthesized(e.getSourceMapRange(n))&&e.nodeIsSynthesized(e.getCommentRange(n))&&!e.some(e.getSyntheticLeadingComments(n))&&!e.some(e.getSyntheticTrailingComments(n))}function bs(n,t,r){return void 0===r&&(r=15),n&&e.isOuterExpression(n,r)&&!vs(n)?hs(n,bs(n.expression,t)):t}function Es(n,t,r){if(!t)return n;var i=qr(t,t.label,e.isLabeledStatement(t.statement)?Es(n,t.statement):n);return r&&r(t),i}function Ts(n,t){var r=e.skipParentheses(n);switch(r.kind){case 79:return t;case 108:case 8:case 9:case 10:return!1;case 202:var i=r.elements;return 0!==i.length;case 203:return r.properties.length>0;default:return!0}}function Ss(n,t,r,i){void 0===i&&(i=!1);var a,o,s=e.skipOuterExpressions(n,15);return e.isSuperProperty(s)?(a=se(),o=s):e.isSuperKeyword(s)?(a=se(),o=void 0!==r&&r<2?e.setTextRange(Q("_super"),s):s):4096&e.getEmitFlags(s)?(a=ts(),o=d().parenthesizeLeftSideOfAccess(s)):e.isPropertyAccessExpression(s)?Ts(s.expression,i)?(a=$(t),o=rt(e.setTextRange(A.createAssignment(a,s.expression),s.expression),s.name),e.setTextRange(o,s)):(a=s.expression,o=s):e.isElementAccessExpression(s)?Ts(s.expression,i)?(a=$(t),o=st(e.setTextRange(A.createAssignment(a,s.expression),s.expression),s.argumentExpression),e.setTextRange(o,s)):(a=s.expression,o=s):(a=ts(),o=d().parenthesizeLeftSideOfAccess(n)),{target:o,thisArg:a}}function xs(e,n){return rt(Et(nt([Fe(void 0,void 0,"value",[ve(void 0,void 0,void 0,e,void 0,void 0,void 0)],hr([Sr(n)]))])),"value")}function Ls(n){return n.length>10?Jo(n):e.reduceLeft(n,A.createComma)}function Ds(n,t,r,i){void 0===i&&(i=0);var a=e.getNameOfDeclaration(n);if(a&&e.isIdentifier(a)&&!e.isGeneratedIdentifier(a)){var o=e.setParent(e.setTextRange($o(a),a),a.parent);return i|=e.getEmitFlags(a),r||(i|=48),t||(i|=1536),i&&e.setEmitFlags(o,i),o}return te(n)}function As(e,n,t){return Ds(e,n,t,49152)}function Cs(e,n,t){return Ds(e,n,t,16384)}function Is(e,n,t){return Ds(e,n,t,8192)}function Ns(e,n,t){return Ds(e,n,t)}function ks(n,t,r,i){var a=rt(n,e.nodeIsSynthesized(t)?t:$o(t));e.setTextRange(a,t);var o=0;return i||(o|=48),r||(o|=1536),o&&e.setEmitFlags(a,o),a}function Rs(n,t,r,i){return n&&e.hasSyntacticModifier(t,1)?ks(n,Ds(t),r,i):Is(t,r,i)}function Os(e,n,t,r){var i=Ms(e,n,t);return Fs(e,n,i,r)}function ws(n){return e.isStringLiteral(n.expression)&&"use strict"===n.expression.text}function Ps(){return e.startOnNewLine(Sr(W("use strict")))}function Ms(n,t,r){e.Debug.assert(0===t.length,"Prologue directives should be at the first statement in the target statements array");var i=!1,a=0,o=n.length;while(a<o){var s=n[a];if(!e.isPrologueDirective(s))break;ws(s)&&(i=!0),t.push(s),a++}return r&&!i&&t.push(Ps()),a}function Fs(n,t,r,i,a){void 0===a&&(a=e.returnTrue);var o=n.length;while(void 0!==r&&r<o){var s=n[r];if(!(1048576&e.getEmitFlags(s)&&a(s)))break;e.append(t,i?e.visitNode(s,i,e.isStatement):s),r++}return r}function Gs(n){var t=e.findUseStrictPrologue(n);return t?n:e.setTextRange(C(a([Ps()],n,!0)),n)}function Bs(n){return e.Debug.assert(e.every(n,e.isStatementOrBlock),"Cannot lift nodes to a Block."),e.singleOrUndefined(n)||hr(n)}function Us(e,n,t){var r=t;while(r<e.length&&n(e[r]))r++;return r}function Vs(n,t){if(!e.some(t))return n;var r=Us(n,e.isPrologueDirective,0),i=Us(n,e.isHoistedFunction,r),o=Us(n,e.isHoistedVariableStatement,i),s=Us(t,e.isPrologueDirective,0),c=Us(t,e.isHoistedFunction,s),l=Us(t,e.isHoistedVariableStatement,c),u=Us(t,e.isCustomPrologue,l);e.Debug.assert(u===t.length,"Expected declarations to be valid standard or custom prologues");var d=e.isNodeArray(n)?n.slice():n;if(u>l&&d.splice.apply(d,a([o,0],t.slice(l,u),!1)),l>c&&d.splice.apply(d,a([i,0],t.slice(c,l),!1)),c>s&&d.splice.apply(d,a([r,0],t.slice(s,c),!1)),s>0)if(0===r)d.splice.apply(d,a([0,0],t.slice(0,s),!1));else{for(var p=new e.Map,f=0;f<r;f++){var m=n[f];p.set(m.expression.text,!0)}for(f=s-1;f>=0;f--){var _=t[f];p.has(_.expression.text)||d.unshift(_)}}return e.isNodeArray(n)?e.setTextRange(C(d,n.hasTrailingComma),n):n}function js(n,t){var r;return"number"===typeof t&&(t=pe(t)),e.isParameter(n)?be(n,n.decorators,t,n.dotDotDotToken,n.name,n.questionToken,n.type,n.initializer):e.isPropertySignature(n)?xe(n,t,n.name,n.questionToken,n.type):e.isPropertyDeclaration(n)?De(n,n.decorators,t,n.name,null!==(r=n.questionToken)&&void 0!==r?r:n.exclamationToken,n.type,n.initializer):e.isMethodSignature(n)?Ce(n,t,n.name,n.questionToken,n.typeParameters,n.parameters,n.type):e.isMethodDeclaration(n)?Ne(n,n.decorators,t,n.asteriskToken,n.name,n.questionToken,n.typeParameters,n.parameters,n.type,n.body):e.isConstructorDeclaration(n)?we(n,n.decorators,t,n.parameters,n.body):e.isGetAccessorDeclaration(n)?Me(n,n.decorators,t,n.name,n.parameters,n.type,n.body):e.isSetAccessorDeclaration(n)?Ge(n,n.decorators,t,n.name,n.parameters,n.body):e.isIndexSignatureDeclaration(n)?He(n,n.decorators,t,n.parameters,n.type):e.isFunctionExpression(n)?xt(n,t,n.asteriskToken,n.name,n.typeParameters,n.parameters,n.type,n.body):e.isArrowFunction(n)?Dt(n,t,n.typeParameters,n.parameters,n.type,n.equalsGreaterThanToken,n.body):e.isClassExpression(n)?rr(n,n.decorators,t,n.name,n.typeParameters,n.heritageClauses,n.members):e.isVariableStatement(n)?Er(n,t,n.declarationList):e.isFunctionDeclaration(n)?ai(n,n.decorators,t,n.asteriskToken,n.name,n.typeParameters,n.parameters,n.type,n.body):e.isClassDeclaration(n)?si(n,n.decorators,t,n.name,n.typeParameters,n.heritageClauses,n.members):e.isInterfaceDeclaration(n)?li(n,n.decorators,t,n.name,n.typeParameters,n.heritageClauses,n.members):e.isTypeAliasDeclaration(n)?di(n,n.decorators,t,n.name,n.typeParameters,n.type):e.isEnumDeclaration(n)?fi(n,n.decorators,t,n.name,n.members):e.isModuleDeclaration(n)?_i(n,n.decorators,t,n.name,n.body):e.isImportEqualsDeclaration(n)?Si(n,n.decorators,t,n.isTypeOnly,n.name,n.moduleReference):e.isImportDeclaration(n)?Li(n,n.decorators,t,n.importClause,n.moduleSpecifier):e.isExportAssignment(n)?Fi(n,n.decorators,t,n.expression):e.isExportDeclaration(n)?Bi(n,n.decorators,t,n.isTypeOnly,n.exportClause,n.moduleSpecifier):e.Debug.assertNever(n)}function Ks(e){return e?C(e):void 0}function Hs(e){return"string"===typeof e?Q(e):e}function Ws(e){return"string"===typeof e?W(e):"number"===typeof e?j(e):"boolean"===typeof e?e?le():ue():e}function zs(e){return"number"===typeof e?ae(e):e}function Js(n){return n&&e.isNotEmittedStatement(n)?e.setTextRange(L(Tr(),n),n):n}}function i(n,t){return n!==t&&e.setTextRange(n,t),n}function o(n,t){return n!==t&&(L(n,t),e.setTextRange(n,t)),n}function s(n){switch(n){case 338:return"type";case 336:return"returns";case 337:return"this";case 334:return"enum";case 325:return"author";case 327:return"class";case 328:return"public";case 329:return"private";case 330:return"protected";case 331:return"readonly";case 332:return"override";case 339:return"template";case 340:return"typedef";case 335:return"param";case 342:return"prop";case 333:return"callback";case 323:return"augments";case 324:return"implements";default:return e.Debug.fail("Unsupported kind: "+e.Debug.formatSyntaxKind(n))}}(function(e){e[e["None"]=0]="None",e[e["NoParenthesizerRules"]=1]="NoParenthesizerRules",e[e["NoNodeConverters"]=2]="NoNodeConverters",e[e["NoIndentationOnFreshPropertyAccess"]=4]="NoIndentationOnFreshPropertyAccess",e[e["NoOriginalNode"]=8]="NoOriginalNode"})(e.NodeFactoryFlags||(e.NodeFactoryFlags={})),e.createNodeFactory=r;var c={};function l(t,r){switch(n||(n=e.createScanner(99,!1,0)),t){case 14:n.setText("`"+r+"`");break;case 15:n.setText("`"+r+"${");break;case 16:n.setText("}"+r+"${");break;case 17:n.setText("}"+r+"`");break}var i,a=n.scan();if(19===a&&(a=n.reScanTemplateToken(!1)),n.isUnterminated())return n.setText(void 0),c;switch(a){case 14:case 15:case 16:case 17:i=n.getTokenValue();break}return void 0===i||1!==n.scan()?(n.setText(void 0),c):(n.setText(void 0),i)}function u(e){return-16777217&p(e)}function d(e,n){return n|33562624&e.transformFlags}function p(n){if(!n)return 0;var t=n.transformFlags&~_(n.kind);return e.isNamedDeclaration(n)&&e.isPropertyName(n.name)?d(n.name,t):t}function f(e){return e?e.transformFlags:0}function m(e){for(var n=0,t=0,r=e;t<r.length;t++){var i=r[t];n|=p(i)}e.transformFlags=n}function _(e){if(e>=175&&e<=198)return-2;switch(e){case 206:case 207:case 202:return 536887296;case 259:return 589443072;case 162:return 536870912;case 212:return 557748224;case 211:case 254:return 591310848;case 253:return 537165824;case 255:case 224:return 536940544;case 169:return 591306752;case 165:return 570433536;case 167:case 170:case 171:return 574529536;case 129:case 145:case 156:case 142:case 148:case 146:case 132:case 149:case 114:case 161:case 164:case 166:case 172:case 173:case 174:case 256:case 257:return-2;case 203:return 536973312;case 290:return 536903680;case 199:case 200:return 536887296;case 209:case 227:case 345:case 210:case 106:return 536870912;case 204:case 205:return 536870912;default:return 536870912}}e.getTransformFlagsSubtreeExclusions=_;var g=e.createBaseNodeFactory();function y(e){return e.flags|=8,e}var h,v={createBaseSourceFileNode:function(e){return y(g.createBaseSourceFileNode(e))},createBaseIdentifierNode:function(e){return y(g.createBaseIdentifierNode(e))},createBasePrivateIdentifierNode:function(e){return y(g.createBasePrivateIdentifierNode(e))},createBaseTokenNode:function(e){return y(g.createBaseTokenNode(e))},createBaseNode:function(e){return y(g.createBaseNode(e))}};function b(n,t,r){var i,a,o,s,c,l,u,d,p,f;e.isString(n)?(o="",s=n,c=n.length,l=t,u=r):(e.Debug.assert("js"===t||"dts"===t),o=("js"===t?n.javascriptPath:n.declarationPath)||"",l="js"===t?n.javascriptMapPath:n.declarationMapPath,d=function(){return"js"===t?n.javascriptText:n.declarationText},p=function(){return"js"===t?n.javascriptMapText:n.declarationMapText},c=function(){return d().length},n.buildInfo&&n.buildInfo.bundle&&(e.Debug.assert(void 0===r||"boolean"===typeof r),i=r,a="js"===t?n.buildInfo.bundle.js:n.buildInfo.bundle.dts,f=n.oldFileOfCurrentEmit));var m=f?T(e.Debug.assertDefined(a)):E(a,i,c);return m.fileName=o,m.sourceMapPath=l,m.oldFileOfCurrentEmit=f,d&&p?(Object.defineProperty(m,"text",{get:d}),Object.defineProperty(m,"sourceMapText",{get:p})):(e.Debug.assert(!f),m.text=null!==s&&void 0!==s?s:"",m.sourceMapText=u),m}function E(n,t,r){for(var i,a,o,s,c,l,u,d,p=0,f=n?n.sections:e.emptyArray;p<f.length;p++){var m=f[p];switch(m.kind){case"prologue":i=e.append(i,e.setTextRange(e.factory.createUnparsedPrologue(m.data),m));break;case"emitHelpers":a=e.append(a,e.getAllUnscopedEmitHelpers().get(m.data));break;case"no-default-lib":d=!0;break;case"reference":o=e.append(o,{pos:-1,end:-1,fileName:m.data});break;case"type":s=e.append(s,m.data);break;case"lib":c=e.append(c,{pos:-1,end:-1,fileName:m.data});break;case"prepend":for(var _=void 0,g=0,y=m.texts;g<y.length;g++){var h=y[g];t&&"internal"===h.kind||(_=e.append(_,e.setTextRange(e.factory.createUnparsedTextLike(h.data,"internal"===h.kind),h)))}l=e.addRange(l,_),u=e.append(u,e.factory.createUnparsedPrepend(m.data,null!==_&&void 0!==_?_:e.emptyArray));break;case"internal":if(t){u||(u=[]);break}case"text":u=e.append(u,e.setTextRange(e.factory.createUnparsedTextLike(m.data,"internal"===m.kind),m));break;default:e.Debug.assertNever(m)}}if(!u){var v=e.factory.createUnparsedTextLike(void 0,!1);e.setTextRangePosWidth(v,0,"function"===typeof r?r():r),u=[v]}var b=e.parseNodeFactory.createUnparsedSource(null!==i&&void 0!==i?i:e.emptyArray,void 0,u);return e.setEachParent(i,b),e.setEachParent(u,b),e.setEachParent(l,b),b.hasNoDefaultLib=d,b.helpers=a,b.referencedFiles=o||e.emptyArray,b.typeReferenceDirectives=s,b.libReferenceDirectives=c||e.emptyArray,b}function T(n){for(var t,r,i=0,a=n.sections;i<a.length;i++){var o=a[i];switch(o.kind){case"internal":case"text":t=e.append(t,e.setTextRange(e.factory.createUnparsedTextLike(o.data,"internal"===o.kind),o));break;case"no-default-lib":case"reference":case"type":case"lib":r=e.append(r,e.setTextRange(e.factory.createUnparsedSyntheticReference(o),o));break;case"prologue":case"emitHelpers":case"prepend":break;default:e.Debug.assertNever(o)}}var s=e.factory.createUnparsedSource(e.emptyArray,r,null!==t&&void 0!==t?t:e.emptyArray);return e.setEachParent(r,s),e.setEachParent(t,s),s.helpers=e.map(n.sources&&n.sources.helpers,function(n){return e.getAllUnscopedEmitHelpers().get(n)}),s}function S(n,t,r,i,a,o,s,c,l,u,d){var p=e.parseNodeFactory.createInputFiles();if(e.isString(n))p.javascriptText=n,p.javascriptMapPath=r,p.javascriptMapText=i,p.declarationText=t,p.declarationMapPath=a,p.declarationMapText=o,p.javascriptPath=s,p.declarationPath=c,p.buildInfoPath=l,p.buildInfo=u,p.oldFileOfCurrentEmit=d;else{var f,m=new e.Map,_=function(e){if(void 0!==e){var t=m.get(e);return void 0===t&&(t=n(e),m.set(e,void 0!==t&&t)),!1!==t?t:void 0}},g=function(e){var n=_(e);return void 0!==n?n:"/* Input file "+e+" was missing */\r\n"},y=function(n){if(void 0===f){var t=n();f=void 0!==t&&e.getBuildInfo(t)}return f||void 0};p.javascriptPath=t,p.javascriptMapPath=r,p.declarationPath=e.Debug.assertDefined(i),p.declarationMapPath=a,p.buildInfoPath=o,Object.defineProperties(p,{javascriptText:{get:function(){return g(t)}},javascriptMapText:{get:function(){return _(r)}},declarationText:{get:function(){return g(e.Debug.assertDefined(i))}},declarationMapText:{get:function(){return _(a)}},buildInfo:{get:function(){return y(function(){return _(o)})}}})}return p}function x(n,t,r){return new(h||(h=e.objectAllocator.getSourceMapSourceConstructor()))(n,t,r)}function L(e,n){if(e.original=n,n){var t=n.emitNode;t&&(e.emitNode=D(t,e.emitNode))}return e}function D(n,t){var r=n.flags,i=n.leadingComments,a=n.trailingComments,o=n.commentRange,s=n.sourceMapRange,c=n.tokenSourceMapRanges,l=n.constantValue,u=n.helpers,d=n.startsOnNewLine;if(t||(t={}),i&&(t.leadingComments=e.addRange(i.slice(),t.leadingComments)),a&&(t.trailingComments=e.addRange(a.slice(),t.trailingComments)),r&&(t.flags=-268435457&r),o&&(t.commentRange=o),s&&(t.sourceMapRange=s),c&&(t.tokenSourceMapRanges=A(c,t.tokenSourceMapRanges)),void 0!==l&&(t.constantValue=l),u)for(var p=0,f=u;p<f.length;p++){var m=f[p];t.helpers=e.appendIfUnique(t.helpers,m)}return void 0!==d&&(t.startsOnNewLine=d),t}function A(e,n){for(var t in n||(n=[]),e)n[t]=e[t];return n}e.factory=r(4,v),e.createUnparsedSourceFile=b,e.createInputFiles=S,e.createSourceMapSource=x,e.setOriginalNode=L}(d||(d={})),function(e){function n(t){var r;if(t.emitNode)e.Debug.assert(!(268435456&t.emitNode.flags),"Invalid attempt to mutate an immutable node.");else{if(e.isParseTreeNode(t)){if(300===t.kind)return t.emitNode={annotatedNodes:[t]};var i=null!==(r=e.getSourceFileOfNode(e.getParseTreeNode(e.getSourceFileOfNode(t))))&&void 0!==r?r:e.Debug.fail("Could not determine parsed source file.");n(i).annotatedNodes.push(t)}t.emitNode={}}return t.emitNode}function t(n){var t,r,i=null===(r=null===(t=e.getSourceFileOfNode(e.getParseTreeNode(n)))||void 0===t?void 0:t.emitNode)||void 0===r?void 0:r.annotatedNodes;if(i)for(var a=0,o=i;a<o.length;a++){var s=o[a];s.emitNode=void 0}}function r(e){var t=n(e);return t.flags|=1536,t.leadingComments=void 0,t.trailingComments=void 0,e}function i(e,t){return n(e).flags=t,e}function a(e,t){var r=n(e);return r.flags=r.flags|t,e}function o(e){var n,t;return null!==(t=null===(n=e.emitNode)||void 0===n?void 0:n.sourceMapRange)&&void 0!==t?t:e}function s(e,t){return n(e).sourceMapRange=t,e}function c(e,n){var t,r;return null===(r=null===(t=e.emitNode)||void 0===t?void 0:t.tokenSourceMapRanges)||void 0===r?void 0:r[n]}function l(e,t,r){var i,a=n(e),o=null!==(i=a.tokenSourceMapRanges)&&void 0!==i?i:a.tokenSourceMapRanges=[];return o[t]=r,e}function u(e){var n;return null===(n=e.emitNode)||void 0===n?void 0:n.startsOnNewLine}function d(e,t){return n(e).startsOnNewLine=t,e}function p(e){var n,t;return null!==(t=null===(n=e.emitNode)||void 0===n?void 0:n.commentRange)&&void 0!==t?t:e}function f(e,t){return n(e).commentRange=t,e}function m(e){var n;return null===(n=e.emitNode)||void 0===n?void 0:n.leadingComments}function _(e,t){return n(e).leadingComments=t,e}function g(n,t,r,i){return _(n,e.append(m(n),{kind:t,pos:-1,end:-1,hasTrailingNewLine:i,text:r}))}function y(e){var n;return null===(n=e.emitNode)||void 0===n?void 0:n.trailingComments}function h(e,t){return n(e).trailingComments=t,e}function v(n,t,r,i){return h(n,e.append(y(n),{kind:t,pos:-1,end:-1,hasTrailingNewLine:i,text:r}))}function b(e,t){_(e,m(t)),h(e,y(t));var r=n(t);return r.leadingComments=void 0,r.trailingComments=void 0,e}function E(e){var n;return null===(n=e.emitNode)||void 0===n?void 0:n.constantValue}function T(e,t){var r=n(e);return r.constantValue=t,e}function S(t,r){var i=n(t);return i.helpers=e.append(i.helpers,r),t}function x(t,r){if(e.some(r))for(var i=n(t),a=0,o=r;a<o.length;a++){var s=o[a];i.helpers=e.appendIfUnique(i.helpers,s)}return t}function L(n,t){var r,i=null===(r=n.emitNode)||void 0===r?void 0:r.helpers;return!!i&&e.orderedRemoveItem(i,t)}function D(e){var n;return null===(n=e.emitNode)||void 0===n?void 0:n.helpers}function A(t,r,i){var a=t.emitNode,o=a&&a.helpers;if(e.some(o)){for(var s=n(r),c=0,l=0;l<o.length;l++){var u=o[l];i(u)?(c++,s.helpers=e.appendIfUnique(s.helpers,u)):c>0&&(o[l-c]=u)}c>0&&(o.length-=c)}}function C(e){return n(e).flags|=134217728,e}e.getOrCreateEmitNode=n,e.disposeEmitNodes=t,e.removeAllComments=r,e.setEmitFlags=i,e.addEmitFlags=a,e.getSourceMapRange=o,e.setSourceMapRange=s,e.getTokenSourceMapRange=c,e.setTokenSourceMapRange=l,e.getStartsOnNewLine=u,e.setStartsOnNewLine=d,e.getCommentRange=p,e.setCommentRange=f,e.getSyntheticLeadingComments=m,e.setSyntheticLeadingComments=_,e.addSyntheticLeadingComment=g,e.getSyntheticTrailingComments=y,e.setSyntheticTrailingComments=h,e.addSyntheticTrailingComment=v,e.moveSyntheticComments=b,e.getConstantValue=E,e.setConstantValue=T,e.addEmitHelper=S,e.addEmitHelpers=x,e.removeEmitHelper=L,e.getEmitHelpers=D,e.moveEmitHelpers=A,e.ignoreSourceNewlines=C}(d||(d={})),function(e){function n(n){var t=n.factory,r=e.memoize(function(){return e.setEmitFlags(t.createTrue(),268435456)}),i=e.memoize(function(){return e.setEmitFlags(t.createFalse(),268435456)});return{getUnscopedHelperName:o,createDecorateHelper:s,createMetadataHelper:c,createParamHelper:l,createAssignHelper:u,createAwaitHelper:d,createAsyncGeneratorHelper:p,createAsyncDelegatorHelper:f,createAsyncValuesHelper:m,createRestHelper:_,createAwaiterHelper:g,createExtendsHelper:y,createTemplateObjectHelper:h,createSpreadArrayHelper:v,createValuesHelper:b,createReadHelper:E,createGeneratorHelper:T,createCreateBindingHelper:S,createImportStarHelper:x,createImportStarCallbackHelper:L,createImportDefaultHelper:D,createExportStarHelper:A,createClassPrivateFieldGetHelper:C,createClassPrivateFieldSetHelper:I};function o(n){return e.setEmitFlags(t.createIdentifier(n),4098)}function s(r,i,a,s){n.requestEmitHelper(e.decorateHelper);var c=[];return c.push(t.createArrayLiteralExpression(r,!0)),c.push(i),a&&(c.push(a),s&&c.push(s)),t.createCallExpression(o("__decorate"),void 0,c)}function c(r,i){return n.requestEmitHelper(e.metadataHelper),t.createCallExpression(o("__metadata"),void 0,[t.createStringLiteral(r),i])}function l(r,i,a){return n.requestEmitHelper(e.paramHelper),e.setTextRange(t.createCallExpression(o("__param"),void 0,[t.createNumericLiteral(i+""),r]),a)}function u(r){return n.getCompilerOptions().target>=2?t.createCallExpression(t.createPropertyAccessExpression(t.createIdentifier("Object"),"assign"),void 0,r):(n.requestEmitHelper(e.assignHelper),t.createCallExpression(o("__assign"),void 0,r))}function d(r){return n.requestEmitHelper(e.awaitHelper),t.createCallExpression(o("__await"),void 0,[r])}function p(r,i){return n.requestEmitHelper(e.awaitHelper),n.requestEmitHelper(e.asyncGeneratorHelper),(r.emitNode||(r.emitNode={})).flags|=786432,t.createCallExpression(o("__asyncGenerator"),void 0,[i?t.createThis():t.createVoidZero(),t.createIdentifier("arguments"),r])}function f(r){return n.requestEmitHelper(e.awaitHelper),n.requestEmitHelper(e.asyncDelegator),t.createCallExpression(o("__asyncDelegator"),void 0,[r])}function m(r){return n.requestEmitHelper(e.asyncValues),t.createCallExpression(o("__asyncValues"),void 0,[r])}function _(r,i,a,s){n.requestEmitHelper(e.restHelper);for(var c=[],l=0,u=0;u<i.length-1;u++){var d=e.getPropertyNameOfBindingOrAssignmentElement(i[u]);if(d)if(e.isComputedPropertyName(d)){e.Debug.assertIsDefined(a,"Encountered computed property name but 'computedTempVariables' argument was not provided.");var p=a[l];l++,c.push(t.createConditionalExpression(t.createTypeCheck(p,"symbol"),void 0,p,void 0,t.createAdd(p,t.createStringLiteral(""))))}else c.push(t.createStringLiteralFromNode(d))}return t.createCallExpression(o("__rest"),void 0,[r,e.setTextRange(t.createArrayLiteralExpression(c),s)])}function g(r,i,a,s){n.requestEmitHelper(e.awaiterHelper);var c=t.createFunctionExpression(void 0,t.createToken(41),void 0,void 0,[],void 0,s);return(c.emitNode||(c.emitNode={})).flags|=786432,t.createCallExpression(o("__awaiter"),void 0,[r?t.createThis():t.createVoidZero(),i?t.createIdentifier("arguments"):t.createVoidZero(),a?e.createExpressionFromEntityName(t,a):t.createVoidZero(),c])}function y(r){return n.requestEmitHelper(e.extendsHelper),t.createCallExpression(o("__extends"),void 0,[r,t.createUniqueName("_super",48)])}function h(r,i){return n.requestEmitHelper(e.templateObjectHelper),t.createCallExpression(o("__makeTemplateObject"),void 0,[r,i])}function v(a,s,c){return n.requestEmitHelper(e.spreadArrayHelper),t.createCallExpression(o("__spreadArray"),void 0,[a,s,c?r():i()])}function b(r){return n.requestEmitHelper(e.valuesHelper),t.createCallExpression(o("__values"),void 0,[r])}function E(r,i){return n.requestEmitHelper(e.readHelper),t.createCallExpression(o("__read"),void 0,void 0!==i?[r,t.createNumericLiteral(i+"")]:[r])}function T(r){return n.requestEmitHelper(e.generatorHelper),t.createCallExpression(o("__generator"),void 0,[t.createThis(),r])}function S(r,i,s){return n.requestEmitHelper(e.createBindingHelper),t.createCallExpression(o("__createBinding"),void 0,a([t.createIdentifier("exports"),r,i],s?[s]:[],!0))}function x(r){return n.requestEmitHelper(e.importStarHelper),t.createCallExpression(o("__importStar"),void 0,[r])}function L(){return n.requestEmitHelper(e.importStarHelper),o("__importStar")}function D(r){return n.requestEmitHelper(e.importDefaultHelper),t.createCallExpression(o("__importDefault"),void 0,[r])}function A(r,i){return void 0===i&&(i=t.createIdentifier("exports")),n.requestEmitHelper(e.exportStarHelper),n.requestEmitHelper(e.createBindingHelper),t.createCallExpression(o("__exportStar"),void 0,[r,i])}function C(r,i,a,s){var c;return n.requestEmitHelper(e.classPrivateFieldGetHelper),c=s?[r,i,t.createStringLiteral(a),s]:[r,i,t.createStringLiteral(a)],t.createCallExpression(o("__classPrivateFieldGet"),void 0,c)}function I(r,i,a,s,c){var l;return n.requestEmitHelper(e.classPrivateFieldSetHelper),l=c?[r,i,a,t.createStringLiteral(s),c]:[r,i,a,t.createStringLiteral(s)],t.createCallExpression(o("__classPrivateFieldSet"),void 0,l)}}function t(n,t){return n===t?0:n.priority===t.priority?0:void 0===n.priority?1:void 0===t.priority?-1:e.compareValues(n.priority,t.priority)}function r(e){for(var n=[],t=1;t<arguments.length;t++)n[t-1]=arguments[t];return function(t){for(var r="",i=0;i<n.length;i++)r+=e[i],r+=t(n[i]);return r+=e[e.length-1],r}}var i;function o(){return i||(i=e.arrayToMap([e.decorateHelper,e.metadataHelper,e.paramHelper,e.assignHelper,e.awaitHelper,e.asyncGeneratorHelper,e.asyncDelegator,e.asyncValues,e.restHelper,e.awaiterHelper,e.extendsHelper,e.templateObjectHelper,e.spreadArrayHelper,e.valuesHelper,e.readHelper,e.generatorHelper,e.importStarHelper,e.importDefaultHelper,e.exportStarHelper,e.classPrivateFieldGetHelper,e.classPrivateFieldSetHelper,e.createBindingHelper,e.setModuleDefaultHelper],function(e){return e.name}))}function c(n,t){return e.isCallExpression(n)&&e.isIdentifier(n.expression)&&0!==(4096&e.getEmitFlags(n.expression))&&n.expression.escapedText===t}e.createEmitHelperFactory=n,e.compareEmitHelpers=t,e.helperString=r,e.decorateHelper={name:"typescript:decorate",importName:"__decorate",scoped:!1,priority:2,text:'\n            var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {\n                var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;\n                if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);\n                else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;\n                return c > 3 && r && Object.defineProperty(target, key, r), r;\n            };'},e.metadataHelper={name:"typescript:metadata",importName:"__metadata",scoped:!1,priority:3,text:'\n            var __metadata = (this && this.__metadata) || function (k, v) {\n                if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);\n            };'},e.paramHelper={name:"typescript:param",importName:"__param",scoped:!1,priority:4,text:"\n            var __param = (this && this.__param) || function (paramIndex, decorator) {\n                return function (target, key) { decorator(target, key, paramIndex); }\n            };"},e.assignHelper={name:"typescript:assign",importName:"__assign",scoped:!1,priority:1,text:"\n            var __assign = (this && this.__assign) || function () {\n                __assign = Object.assign || function(t) {\n                    for (var s, i = 1, n = arguments.length; i < n; i++) {\n                        s = arguments[i];\n                        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))\n                            t[p] = s[p];\n                    }\n                    return t;\n                };\n                return __assign.apply(this, arguments);\n            };"},e.awaitHelper={name:"typescript:await",importName:"__await",scoped:!1,text:"\n            var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }"},e.asyncGeneratorHelper={name:"typescript:asyncGenerator",importName:"__asyncGenerator",scoped:!1,dependencies:[e.awaitHelper],text:'\n            var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {\n                if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");\n                var g = generator.apply(thisArg, _arguments || []), i, q = [];\n                return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;\n                function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }\n                function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }\n                function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }\n                function fulfill(value) { resume("next", value); }\n                function reject(value) { resume("throw", value); }\n                function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }\n            };'},e.asyncDelegator={name:"typescript:asyncDelegator",importName:"__asyncDelegator",scoped:!1,dependencies:[e.awaitHelper],text:'\n            var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {\n                var i, p;\n                return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;\n                function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }\n            };'},e.asyncValues={name:"typescript:asyncValues",importName:"__asyncValues",scoped:!1,text:'\n            var __asyncValues = (this && this.__asyncValues) || function (o) {\n                if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");\n                var m = o[Symbol.asyncIterator], i;\n                return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);\n                function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }\n                function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }\n            };'},e.restHelper={name:"typescript:rest",importName:"__rest",scoped:!1,text:'\n            var __rest = (this && this.__rest) || function (s, e) {\n                var t = {};\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)\n                    t[p] = s[p];\n                if (s != null && typeof Object.getOwnPropertySymbols === "function")\n                    for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {\n                        if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))\n                            t[p[i]] = s[p[i]];\n                    }\n                return t;\n            };'},e.awaiterHelper={name:"typescript:awaiter",importName:"__awaiter",scoped:!1,priority:5,text:'\n            var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {\n                function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }\n                return new (P || (P = Promise))(function (resolve, reject) {\n                    function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }\n                    function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }\n                    function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }\n                    step((generator = generator.apply(thisArg, _arguments || [])).next());\n                });\n            };'},e.extendsHelper={name:"typescript:extends",importName:"__extends",scoped:!1,priority:0,text:'\n            var __extends = (this && this.__extends) || (function () {\n                var extendStatics = function (d, b) {\n                    extendStatics = Object.setPrototypeOf ||\n                        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\n                        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };\n                    return extendStatics(d, b);\n                };\n\n                return function (d, b) {\n                    if (typeof b !== "function" && b !== null)\n                        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");\n                    extendStatics(d, b);\n                    function __() { this.constructor = d; }\n                    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\n                };\n            })();'},e.templateObjectHelper={name:"typescript:makeTemplateObject",importName:"__makeTemplateObject",scoped:!1,priority:0,text:'\n            var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {\n                if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }\n                return cooked;\n            };'},e.readHelper={name:"typescript:read",importName:"__read",scoped:!1,text:'\n            var __read = (this && this.__read) || function (o, n) {\n                var m = typeof Symbol === "function" && o[Symbol.iterator];\n                if (!m) return o;\n                var i = m.call(o), r, ar = [], e;\n                try {\n                    while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);\n                }\n                catch (error) { e = { error: error }; }\n                finally {\n                    try {\n                        if (r && !r.done && (m = i["return"])) m.call(i);\n                    }\n                    finally { if (e) throw e.error; }\n                }\n                return ar;\n            };'},e.spreadArrayHelper={name:"typescript:spreadArray",importName:"__spreadArray",scoped:!1,text:"\n            var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {\n                if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {\n                    if (ar || !(i in from)) {\n                        if (!ar) ar = Array.prototype.slice.call(from, 0, i);\n                        ar[i] = from[i];\n                    }\n                }\n                return to.concat(ar || Array.prototype.slice.call(from));\n            };"},e.valuesHelper={name:"typescript:values",importName:"__values",scoped:!1,text:'\n            var __values = (this && this.__values) || function(o) {\n                var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;\n                if (m) return m.call(o);\n                if (o && typeof o.length === "number") return {\n                    next: function () {\n                        if (o && i >= o.length) o = void 0;\n                        return { value: o && o[i++], done: !o };\n                    }\n                };\n                throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");\n            };'},e.generatorHelper={name:"typescript:generator",importName:"__generator",scoped:!1,priority:6,text:'\n            var __generator = (this && this.__generator) || function (thisArg, body) {\n                var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;\n                return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;\n                function verb(n) { return function (v) { return step([n, v]); }; }\n                function step(op) {\n                    if (f) throw new TypeError("Generator is already executing.");\n                    while (_) try {\n                        if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;\n                        if (y = 0, t) op = [op[0] & 2, t.value];\n                        switch (op[0]) {\n                            case 0: case 1: t = op; break;\n                            case 4: _.label++; return { value: op[1], done: false };\n                            case 5: _.label++; y = op[1]; op = [0]; continue;\n                            case 7: op = _.ops.pop(); _.trys.pop(); continue;\n                            default:\n                                if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }\n                                if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }\n                                if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }\n                                if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }\n                                if (t[2]) _.ops.pop();\n                                _.trys.pop(); continue;\n                        }\n                        op = body.call(thisArg, _);\n                    } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }\n                    if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };\n                }\n            };'},e.createBindingHelper={name:"typescript:commonjscreatebinding",importName:"__createBinding",scoped:!1,priority:1,text:"\n            var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {\n                if (k2 === undefined) k2 = k;\n                Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });\n            }) : (function(o, m, k, k2) {\n                if (k2 === undefined) k2 = k;\n                o[k2] = m[k];\n            }));"},e.setModuleDefaultHelper={name:"typescript:commonjscreatevalue",importName:"__setModuleDefault",scoped:!1,priority:1,text:'\n            var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {\n                Object.defineProperty(o, "default", { enumerable: true, value: v });\n            }) : function(o, v) {\n                o["default"] = v;\n            });'},e.importStarHelper={name:"typescript:commonjsimportstar",importName:"__importStar",scoped:!1,dependencies:[e.createBindingHelper,e.setModuleDefaultHelper],priority:2,text:'\n            var __importStar = (this && this.__importStar) || function (mod) {\n                if (mod && mod.__esModule) return mod;\n                var result = {};\n                if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);\n                __setModuleDefault(result, mod);\n                return result;\n            };'},e.importDefaultHelper={name:"typescript:commonjsimportdefault",importName:"__importDefault",scoped:!1,text:'\n            var __importDefault = (this && this.__importDefault) || function (mod) {\n                return (mod && mod.__esModule) ? mod : { "default": mod };\n            };'},e.exportStarHelper={name:"typescript:export-star",importName:"__exportStar",scoped:!1,dependencies:[e.createBindingHelper],priority:2,text:'\n            var __exportStar = (this && this.__exportStar) || function(m, exports) {\n                for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);\n            };'},e.classPrivateFieldGetHelper={name:"typescript:classPrivateFieldGet",importName:"__classPrivateFieldGet",scoped:!1,text:'\n            var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {\n                if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");\n                if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");\n                return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);\n            };'},e.classPrivateFieldSetHelper={name:"typescript:classPrivateFieldSet",importName:"__classPrivateFieldSet",scoped:!1,text:'\n            var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {\n                if (kind === "m") throw new TypeError("Private method is not writable");\n                if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");\n                if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");\n                return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;\n            };'},e.getAllUnscopedEmitHelpers=o,e.asyncSuperHelper={name:"typescript:async-super",scoped:!0,text:r(s(["\n            const "," = name => super[name];"],["\n            const "," = name => super[name];"]),"_superIndex")},e.advancedAsyncSuperHelper={name:"typescript:advanced-async-super",scoped:!0,text:r(s(["\n            const "," = (function (geti, seti) {\n                const cache = Object.create(null);\n                return name => cache[name] || (cache[name] = { get value() { return geti(name); }, set value(v) { seti(name, v); } });\n            })(name => super[name], (name, value) => super[name] = value);"],["\n            const "," = (function (geti, seti) {\n                const cache = Object.create(null);\n                return name => cache[name] || (cache[name] = { get value() { return geti(name); }, set value(v) { seti(name, v); } });\n            })(name => super[name], (name, value) => super[name] = value);"]),"_superIndex")},e.isCallToHelper=c}(d||(d={})),function(e){function n(e){return 8===e.kind}function t(e){return 9===e.kind}function r(e){return 10===e.kind}function i(e){return 11===e.kind}function a(e){return 13===e.kind}function o(e){return 14===e.kind}function s(e){return 15===e.kind}function c(e){return 16===e.kind}function l(e){return 17===e.kind}function u(e){return 25===e.kind}function d(e){return 27===e.kind}function p(e){return 39===e.kind}function f(e){return 40===e.kind}function m(e){return 41===e.kind}function _(e){return 53===e.kind}function g(e){return 57===e.kind}function y(e){return 58===e.kind}function h(e){return 28===e.kind}function v(e){return 38===e.kind}function b(e){return 79===e.kind}function E(e){return 80===e.kind}function T(e){return 93===e.kind}function S(e){return 130===e.kind}function x(e){return 128===e.kind}function L(e){return 131===e.kind}function D(e){return 143===e.kind}function A(e){return 124===e.kind}function C(e){return 126===e.kind}function I(e){return 106===e.kind}function N(e){return 100===e.kind}function k(e){return 159===e.kind}function R(e){return 160===e.kind}function O(e){return 161===e.kind}function w(e){return 162===e.kind}function P(e){return 163===e.kind}function M(e){return 164===e.kind}function F(e){return 165===e.kind}function G(e){return 166===e.kind}function B(e){return 167===e.kind}function U(e){return 168===e.kind}function V(e){return 169===e.kind}function j(e){return 170===e.kind}function K(e){return 171===e.kind}function H(e){return 172===e.kind}function W(e){return 173===e.kind}function z(e){return 174===e.kind}function J(e){return 175===e.kind}function q(e){return 176===e.kind}function X(e){return 177===e.kind}function Y(e){return 178===e.kind}function Q(e){return 179===e.kind}function Z(e){return 180===e.kind}function $(e){return 181===e.kind}function ee(e){return 182===e.kind}function ne(e){return 195===e.kind}function te(e){return 183===e.kind}function re(e){return 184===e.kind}function ie(e){return 185===e.kind}function ae(e){return 186===e.kind}function oe(e){return 187===e.kind}function se(e){return 188===e.kind}function ce(e){return 189===e.kind}function le(e){return 190===e.kind}function ue(e){return 191===e.kind}function de(e){return 192===e.kind}function pe(e){return 193===e.kind}function fe(e){return 194===e.kind}function me(e){return 198===e.kind}function _e(e){return 197===e.kind}function ge(e){return 196===e.kind}function ye(e){return 199===e.kind}function he(e){return 200===e.kind}function ve(e){return 201===e.kind}function be(e){return 202===e.kind}function Ee(e){return 203===e.kind}function Te(e){return 204===e.kind}function Se(e){return 205===e.kind}function xe(e){return 206===e.kind}function Le(e){return 207===e.kind}function De(e){return 208===e.kind}function Ae(e){return 209===e.kind}function Ce(e){return 210===e.kind}function Ie(e){return 211===e.kind}function Ne(e){return 212===e.kind}function ke(e){return 213===e.kind}function Re(e){return 214===e.kind}function Oe(e){return 215===e.kind}function we(e){return 216===e.kind}function Pe(e){return 217===e.kind}function Me(e){return 218===e.kind}function Fe(e){return 219===e.kind}function Ge(e){return 220===e.kind}function Be(e){return 221===e.kind}function Ue(e){return 222===e.kind}function Ve(e){return 223===e.kind}function je(e){return 224===e.kind}function Ke(e){return 225===e.kind}function He(e){return 226===e.kind}function We(e){return 227===e.kind}function ze(e){return 228===e.kind}function Je(e){return 229===e.kind}function qe(e){return 230===e.kind}function Xe(e){return 345===e.kind}function Ye(e){return 346===e.kind}function Qe(e){return 231===e.kind}function Ze(e){return 232===e.kind}function $e(e){return 233===e.kind}function en(e){return 235===e.kind}function nn(e){return 234===e.kind}function tn(e){return 236===e.kind}function rn(e){return 237===e.kind}function an(e){return 238===e.kind}function on(e){return 239===e.kind}function sn(e){return 240===e.kind}function cn(e){return 241===e.kind}function ln(e){return 242===e.kind}function un(e){return 243===e.kind}function dn(e){return 244===e.kind}function pn(e){return 245===e.kind}function fn(e){return 246===e.kind}function mn(e){return 247===e.kind}function _n(e){return 248===e.kind}function gn(e){return 249===e.kind}function yn(e){return 250===e.kind}function hn(e){return 251===e.kind}function vn(e){return 252===e.kind}function bn(e){return 253===e.kind}function En(e){return 254===e.kind}function Tn(e){return 255===e.kind}function Sn(e){return 256===e.kind}function xn(e){return 257===e.kind}function Ln(e){return 258===e.kind}function Dn(e){return 259===e.kind}function An(e){return 260===e.kind}function Cn(e){return 261===e.kind}function In(e){return 262===e.kind}function Nn(e){return 263===e.kind}function kn(e){return 264===e.kind}function Rn(e){return 265===e.kind}function On(e){return 266===e.kind}function wn(e){return 272===e.kind}function Pn(e){return 267===e.kind}function Mn(e){return 268===e.kind}function Fn(e){return 269===e.kind}function Gn(e){return 270===e.kind}function Bn(e){return 271===e.kind}function Un(e){return 273===e.kind}function Vn(e){return 274===e.kind}function jn(e){return 344===e.kind}function Kn(e){return 349===e.kind}function Hn(e){return 347===e.kind}function Wn(e){return 348===e.kind}function zn(e){return 275===e.kind}function Jn(e){return 276===e.kind}function qn(e){return 277===e.kind}function Xn(e){return 278===e.kind}function Yn(e){return 279===e.kind}function Qn(e){return 280===e.kind}function Zn(e){return 281===e.kind}function $n(e){return 282===e.kind}function et(e){return 283===e.kind}function nt(e){return 284===e.kind}function tt(e){return 285===e.kind}function rt(e){return 286===e.kind}function it(e){return 287===e.kind}function at(e){return 288===e.kind}function ot(e){return 289===e.kind}function st(e){return 290===e.kind}function ct(e){return 291===e.kind}function lt(e){return 292===e.kind}function ut(e){return 293===e.kind}function dt(e){return 294===e.kind}function pt(e){return 296===e.kind}function ft(e){return 300===e.kind}function mt(e){return 301===e.kind}function _t(e){return 302===e.kind}function gt(e){return 304===e.kind}function yt(e){return 305===e.kind}function ht(e){return 306===e.kind}function vt(e){return 319===e.kind}function bt(e){return 320===e.kind}function Et(e){return 321===e.kind}function Tt(e){return 307===e.kind}function St(e){return 308===e.kind}function xt(e){return 309===e.kind}function Lt(e){return 310===e.kind}function Dt(e){return 311===e.kind}function At(e){return 312===e.kind}function Ct(e){return 313===e.kind}function It(e){return 314===e.kind}function Nt(e){return 315===e.kind}function kt(e){return 317===e.kind}function Rt(e){return 318===e.kind}function Ot(e){return 323===e.kind}function wt(e){return 325===e.kind}function Pt(e){return 327===e.kind}function Mt(e){return 333===e.kind}function Ft(e){return 328===e.kind}function Gt(e){return 329===e.kind}function Bt(e){return 330===e.kind}function Ut(e){return 331===e.kind}function Vt(e){return 332===e.kind}function jt(e){return 326===e.kind}function Kt(e){return 341===e.kind}function Ht(e){return 334===e.kind}function Wt(e){return 335===e.kind}function zt(e){return 336===e.kind}function Jt(e){return 337===e.kind}function qt(e){return 338===e.kind}function Xt(e){return 339===e.kind}function Yt(e){return 340===e.kind}function Qt(e){return 322===e.kind}function Zt(e){return 342===e.kind}function $t(e){return 324===e.kind}function er(e){return 343===e.kind}e.isNumericLiteral=n,e.isBigIntLiteral=t,e.isStringLiteral=r,e.isJsxText=i,e.isRegularExpressionLiteral=a,e.isNoSubstitutionTemplateLiteral=o,e.isTemplateHead=s,e.isTemplateMiddle=c,e.isTemplateTail=l,e.isDotDotDotToken=u,e.isCommaToken=d,e.isPlusToken=p,e.isMinusToken=f,e.isAsteriskToken=m,e.isExclamationToken=_,e.isQuestionToken=g,e.isColonToken=y,e.isQuestionDotToken=h,e.isEqualsGreaterThanToken=v,e.isIdentifier=b,e.isPrivateIdentifier=E,e.isExportModifier=T,e.isAsyncModifier=S,e.isAssertsKeyword=x,e.isAwaitKeyword=L,e.isReadonlyKeyword=D,e.isStaticModifier=A,e.isAbstractModifier=C,e.isSuperKeyword=I,e.isImportKeyword=N,e.isQualifiedName=k,e.isComputedPropertyName=R,e.isTypeParameterDeclaration=O,e.isParameter=w,e.isDecorator=P,e.isPropertySignature=M,e.isPropertyDeclaration=F,e.isMethodSignature=G,e.isMethodDeclaration=B,e.isClassStaticBlockDeclaration=U,e.isConstructorDeclaration=V,e.isGetAccessorDeclaration=j,e.isSetAccessorDeclaration=K,e.isCallSignatureDeclaration=H,e.isConstructSignatureDeclaration=W,e.isIndexSignatureDeclaration=z,e.isTypePredicateNode=J,e.isTypeReferenceNode=q,e.isFunctionTypeNode=X,e.isConstructorTypeNode=Y,e.isTypeQueryNode=Q,e.isTypeLiteralNode=Z,e.isArrayTypeNode=$,e.isTupleTypeNode=ee,e.isNamedTupleMember=ne,e.isOptionalTypeNode=te,e.isRestTypeNode=re,e.isUnionTypeNode=ie,e.isIntersectionTypeNode=ae,e.isConditionalTypeNode=oe,e.isInferTypeNode=se,e.isParenthesizedTypeNode=ce,e.isThisTypeNode=le,e.isTypeOperatorNode=ue,e.isIndexedAccessTypeNode=de,e.isMappedTypeNode=pe,e.isLiteralTypeNode=fe,e.isImportTypeNode=me,e.isTemplateLiteralTypeSpan=_e,e.isTemplateLiteralTypeNode=ge,e.isObjectBindingPattern=ye,e.isArrayBindingPattern=he,e.isBindingElement=ve,e.isArrayLiteralExpression=be,e.isObjectLiteralExpression=Ee,e.isPropertyAccessExpression=Te,e.isElementAccessExpression=Se,e.isCallExpression=xe,e.isNewExpression=Le,e.isTaggedTemplateExpression=De,e.isTypeAssertionExpression=Ae,e.isParenthesizedExpression=Ce,e.isFunctionExpression=Ie,e.isArrowFunction=Ne,e.isDeleteExpression=ke,e.isTypeOfExpression=Re,e.isVoidExpression=Oe,e.isAwaitExpression=we,e.isPrefixUnaryExpression=Pe,e.isPostfixUnaryExpression=Me,e.isBinaryExpression=Fe,e.isConditionalExpression=Ge,e.isTemplateExpression=Be,e.isYieldExpression=Ue,e.isSpreadElement=Ve,e.isClassExpression=je,e.isOmittedExpression=Ke,e.isExpressionWithTypeArguments=He,e.isAsExpression=We,e.isNonNullExpression=ze,e.isMetaProperty=Je,e.isSyntheticExpression=qe,e.isPartiallyEmittedExpression=Xe,e.isCommaListExpression=Ye,e.isTemplateSpan=Qe,e.isSemicolonClassElement=Ze,e.isBlock=$e,e.isVariableStatement=en,e.isEmptyStatement=nn,e.isExpressionStatement=tn,e.isIfStatement=rn,e.isDoStatement=an,e.isWhileStatement=on,e.isForStatement=sn,e.isForInStatement=cn,e.isForOfStatement=ln,e.isContinueStatement=un,e.isBreakStatement=dn,e.isReturnStatement=pn,e.isWithStatement=fn,e.isSwitchStatement=mn,e.isLabeledStatement=_n,e.isThrowStatement=gn,e.isTryStatement=yn,e.isDebuggerStatement=hn,e.isVariableDeclaration=vn,e.isVariableDeclarationList=bn,e.isFunctionDeclaration=En,e.isClassDeclaration=Tn,e.isInterfaceDeclaration=Sn,e.isTypeAliasDeclaration=xn,e.isEnumDeclaration=Ln,e.isModuleDeclaration=Dn,e.isModuleBlock=An,e.isCaseBlock=Cn,e.isNamespaceExportDeclaration=In,e.isImportEqualsDeclaration=Nn,e.isImportDeclaration=kn,e.isImportClause=Rn,e.isNamespaceImport=On,e.isNamespaceExport=wn,e.isNamedImports=Pn,e.isImportSpecifier=Mn,e.isExportAssignment=Fn,e.isExportDeclaration=Gn,e.isNamedExports=Bn,e.isExportSpecifier=Un,e.isMissingDeclaration=Vn,e.isNotEmittedStatement=jn,e.isSyntheticReference=Kn,e.isMergeDeclarationMarker=Hn,e.isEndOfDeclarationMarker=Wn,e.isExternalModuleReference=zn,e.isJsxElement=Jn,e.isJsxSelfClosingElement=qn,e.isJsxOpeningElement=Xn,e.isJsxClosingElement=Yn,e.isJsxFragment=Qn,e.isJsxOpeningFragment=Zn,e.isJsxClosingFragment=$n,e.isJsxAttribute=et,e.isJsxAttributes=nt,e.isJsxSpreadAttribute=tt,e.isJsxExpression=rt,e.isCaseClause=it,e.isDefaultClause=at,e.isHeritageClause=ot,e.isCatchClause=st,e.isPropertyAssignment=ct,e.isShorthandPropertyAssignment=lt,e.isSpreadAssignment=ut,e.isEnumMember=dt,e.isUnparsedPrepend=pt,e.isSourceFile=ft,e.isBundle=mt,e.isUnparsedSource=_t,e.isJSDocTypeExpression=gt,e.isJSDocNameReference=yt,e.isJSDocMemberName=ht,e.isJSDocLink=vt,e.isJSDocLinkCode=bt,e.isJSDocLinkPlain=Et,e.isJSDocAllType=Tt,e.isJSDocUnknownType=St,e.isJSDocNullableType=xt,e.isJSDocNonNullableType=Lt,e.isJSDocOptionalType=Dt,e.isJSDocFunctionType=At,e.isJSDocVariadicType=Ct,e.isJSDocNamepathType=It,e.isJSDoc=Nt,e.isJSDocTypeLiteral=kt,e.isJSDocSignature=Rt,e.isJSDocAugmentsTag=Ot,e.isJSDocAuthorTag=wt,e.isJSDocClassTag=Pt,e.isJSDocCallbackTag=Mt,e.isJSDocPublicTag=Ft,e.isJSDocPrivateTag=Gt,e.isJSDocProtectedTag=Bt,e.isJSDocReadonlyTag=Ut,e.isJSDocOverrideTag=Vt,e.isJSDocDeprecatedTag=jt,e.isJSDocSeeTag=Kt,e.isJSDocEnumTag=Ht,e.isJSDocParameterTag=Wt,e.isJSDocReturnTag=zt,e.isJSDocThisTag=Jt,e.isJSDocTypeTag=qt,e.isJSDocTemplateTag=Xt,e.isJSDocTypedefTag=Yt,e.isJSDocUnknownTag=Qt,e.isJSDocPropertyTag=Zt,e.isJSDocImplementsTag=$t,e.isSyntaxList=er}(d||(d={})),function(e){function n(e){return e.createExportDeclaration(void 0,void 0,!1,e.createNamedExports([]),void 0)}function t(n,t,r,i){if(e.isComputedPropertyName(r))return e.setTextRange(n.createElementAccessExpression(t,r.expression),i);var a=e.setTextRange(e.isMemberName(r)?n.createPropertyAccessExpression(t,r):n.createElementAccessExpression(t,r),r);return e.getOrCreateEmitNode(a).flags|=64,a}function r(n,t){var r=e.parseNodeFactory.createIdentifier(n||"React");return e.setParent(r,e.getParseTreeNode(t)),r}function i(n,t,a){if(e.isQualifiedName(t)){var o=i(n,t.left,a),s=n.createIdentifier(e.idText(t.right));return s.escapedText=t.right.escapedText,n.createPropertyAccessExpression(o,s)}return r(e.idText(t),a)}function o(e,n,t,a){return n?i(e,n,a):e.createPropertyAccessExpression(r(t,a),"createElement")}function s(e,n,t,a){return n?i(e,n,a):e.createPropertyAccessExpression(r(t,a),"Fragment")}function c(n,t,r,i,a,o){var s=[r];if(i&&s.push(i),a&&a.length>0)if(i||s.push(n.createNull()),a.length>1)for(var c=0,l=a;c<l.length;c++){var u=l[c];N(u),s.push(u)}else s.push(a[0]);return e.setTextRange(n.createCallExpression(t,void 0,s),o)}function l(n,t,r,i,a,c,l){var u=s(n,r,i,c),d=[u,n.createNull()];if(a&&a.length>0)if(a.length>1)for(var p=0,f=a;p<f.length;p++){var m=f[p];N(m),d.push(m)}else d.push(a[0]);return e.setTextRange(n.createCallExpression(o(n,t,i,c),void 0,d),l)}function u(n,t,r){if(e.isVariableDeclarationList(t)){var i=e.first(t.declarations),a=n.updateVariableDeclaration(i,i.name,void 0,void 0,r);return e.setTextRange(n.createVariableStatement(void 0,n.updateVariableDeclarationList(t,[a])),t)}var o=e.setTextRange(n.createAssignment(t,r),t);return e.setTextRange(n.createExpressionStatement(o),t)}function d(n,t,r){return e.isBlock(t)?n.updateBlock(t,e.setTextRange(n.createNodeArray(a([r],t.statements,!0)),t.statements)):n.createBlock(n.createNodeArray([t,r]),!0)}function p(n,t){if(e.isQualifiedName(t)){var r=p(n,t.left),i=e.setParent(e.setTextRange(n.cloneNode(t.right),t.right),t.right.parent);return e.setTextRange(n.createPropertyAccessExpression(r,i),t)}return e.setParent(e.setTextRange(n.cloneNode(t),t),t.parent)}function f(n,t){return e.isIdentifier(t)?n.createStringLiteralFromNode(t):e.isComputedPropertyName(t)?e.setParent(e.setTextRange(n.cloneNode(t.expression),t.expression),t.expression.parent):e.setParent(e.setTextRange(n.cloneNode(t),t),t.parent)}function m(n,t,r,i,a){var o=e.getAllAccessorDeclarations(t,r),s=o.firstAccessor,c=o.getAccessor,l=o.setAccessor;if(r===s)return e.setTextRange(n.createObjectDefinePropertyCall(i,f(n,r.name),n.createPropertyDescriptor({enumerable:n.createFalse(),configurable:!0,get:c&&e.setTextRange(e.setOriginalNode(n.createFunctionExpression(c.modifiers,void 0,void 0,void 0,c.parameters,void 0,c.body),c),c),set:l&&e.setTextRange(e.setOriginalNode(n.createFunctionExpression(l.modifiers,void 0,void 0,void 0,l.parameters,void 0,l.body),l),l)},!a)),s)}function _(n,r,i){return e.setOriginalNode(e.setTextRange(n.createAssignment(t(n,i,r.name,r.name),r.initializer),r),r)}function g(n,r,i){return e.setOriginalNode(e.setTextRange(n.createAssignment(t(n,i,r.name,r.name),n.cloneNode(r.name)),r),r)}function y(n,r,i){return e.setOriginalNode(e.setTextRange(n.createAssignment(t(n,i,r.name,r.name),e.setOriginalNode(e.setTextRange(n.createFunctionExpression(r.modifiers,r.asteriskToken,void 0,void 0,r.parameters,void 0,r.body),r),r)),r),r)}function h(n,t,r,i){switch(r.name&&e.isPrivateIdentifier(r.name)&&e.Debug.failBadSyntaxKind(r.name,"Private identifiers are not allowed in object literals."),r.kind){case 170:case 171:return m(n,t.properties,r,i,!!t.multiLine);case 291:return _(n,r,i);case 292:return g(n,r,i);case 167:return y(n,r,i)}}function v(n,t,r,i,a){var o=t.operator;e.Debug.assert(45===o||46===o,"Expected 'node' to be a pre- or post-increment or pre- or post-decrement expression");var s=n.createTempVariable(i);r=n.createAssignment(s,r),e.setTextRange(r,t.operand);var c=e.isPrefixUnaryExpression(t)?n.createPrefixUnaryExpression(o,s):n.createPostfixUnaryExpression(s,o);return e.setTextRange(c,t),a&&(c=n.createAssignment(a,c),e.setTextRange(c,t)),r=n.createComma(r,c),e.setTextRange(r,t),e.isPostfixUnaryExpression(t)&&(r=n.createComma(r,s),e.setTextRange(r,t)),r}function b(n){return 0!==(32768&e.getEmitFlags(n))}function E(n){return 0!==(16384&e.getEmitFlags(n))}function T(n){return 0!==(8192&e.getEmitFlags(n))}function S(n){return e.isStringLiteral(n.expression)&&"use strict"===n.expression.text}function x(n){for(var t=0,r=n;t<r.length;t++){var i=r[t];if(!e.isPrologueDirective(i))break;if(S(i))return i}}function L(n){var t=e.firstOrUndefined(n);return void 0!==t&&e.isPrologueDirective(t)&&S(t)}function D(e){return 219===e.kind&&27===e.operatorToken.kind||346===e.kind}function A(e,n){switch(void 0===n&&(n=15),e.kind){case 210:return 0!==(1&n);case 209:case 227:return 0!==(2&n);case 228:return 0!==(4&n);case 345:return 0!==(8&n)}return!1}function C(e,n){void 0===n&&(n=15);while(A(e,n))e=e.expression;return e}function I(e){return C(e,6)}function N(n){return e.setStartsOnNewLine(n,!0)}function k(n){var t=e.getOriginalNode(n,e.isSourceFile),r=t&&t.emitNode;return r&&r.externalHelpersModuleName}function R(n){var t=e.getOriginalNode(n,e.isSourceFile),r=t&&t.emitNode;return!!r&&(!!r.externalHelpersModuleName||!!r.externalHelpers)}function O(n,t,r,i,a,o,s){if(i.importHelpers&&e.isEffectiveExternalModule(r,i)){var c=void 0,l=e.getEmitModuleKind(i);if(l>=e.ModuleKind.ES2015&&l<=e.ModuleKind.ESNext){var u=e.getEmitHelpers(r);if(u){for(var d=[],p=0,f=u;p<f.length;p++){var m=f[p];if(!m.scoped){var _=m.importName;_&&e.pushIfUnique(d,_)}}if(e.some(d)){d.sort(e.compareStringsCaseSensitive),c=n.createNamedImports(e.map(d,function(i){return e.isFileLevelUniqueName(r,i)?n.createImportSpecifier(void 0,n.createIdentifier(i)):n.createImportSpecifier(n.createIdentifier(i),t.getUnscopedHelperName(i))}));var g=e.getOriginalNode(r,e.isSourceFile),y=e.getOrCreateEmitNode(g);y.externalHelpers=!0}}}else{var h=w(n,r,i,a,o||s);h&&(c=n.createNamespaceImport(h))}if(c){var v=n.createImportDeclaration(void 0,void 0,n.createImportClause(!1,void 0,c),n.createStringLiteral(e.externalHelpersModuleNameText));return e.addEmitFlags(v,67108864),v}}}function w(n,t,r,i,a){if(r.importHelpers&&e.isEffectiveExternalModule(t,r)){var o=k(t);if(o)return o;var s=e.getEmitModuleKind(r),c=(i||r.esModuleInterop&&a)&&s!==e.ModuleKind.System&&s<e.ModuleKind.ES2015;if(!c){var l=e.getEmitHelpers(t);if(l)for(var u=0,d=l;u<d.length;u++){var p=d[u];if(!p.scoped){c=!0;break}}}if(c){var f=e.getOriginalNode(t,e.isSourceFile),m=e.getOrCreateEmitNode(f);return m.externalHelpersModuleName||(m.externalHelpersModuleName=n.createUniqueName(e.externalHelpersModuleNameText))}}}function P(n,t,r){var i=e.getNamespaceDeclarationNode(t);if(i&&!e.isDefaultImport(t)&&!e.isExportNamespaceAsDefaultDeclaration(t)){var a=i.name;return e.isGeneratedIdentifier(a)?a:n.createIdentifier(e.getSourceTextOfNodeFromSourceFile(r,a)||e.idText(a))}return 264===t.kind&&t.importClause?n.getGeneratedNameForNode(t):270===t.kind&&t.moduleSpecifier?n.getGeneratedNameForNode(t):void 0}function M(n,t,r,i,a,o){var s=e.getExternalModuleName(t);if(s&&e.isStringLiteral(s))return B(t,i,n,a,o)||F(n,s,r)||n.cloneNode(s)}function F(e,n,t){var r=t.renamedDependencies&&t.renamedDependencies.get(n.text);return r?e.createStringLiteral(r):void 0}function G(n,t,r,i){if(t)return t.moduleName?n.createStringLiteral(t.moduleName):!t.isDeclarationFile&&e.outFile(i)?n.createStringLiteral(e.getExternalModuleNameFromPath(r,t.fileName)):void 0}function B(e,n,t,r,i){return G(t,r.getExternalModuleFileFromDeclaration(e),n,i)}function U(n){if(e.isDeclarationBindingElement(n))return n.initializer;if(e.isPropertyAssignment(n)){var t=n.initializer;return e.isAssignmentExpression(t,!0)?t.right:void 0}return e.isShorthandPropertyAssignment(n)?n.objectAssignmentInitializer:e.isAssignmentExpression(n,!0)?n.right:e.isSpreadElement(n)?U(n.expression):void 0}function V(n){if(e.isDeclarationBindingElement(n))return n.name;if(!e.isObjectLiteralElementLike(n))return e.isAssignmentExpression(n,!0)?V(n.left):e.isSpreadElement(n)?V(n.expression):n;switch(n.kind){case 291:return V(n.initializer);case 292:return n.name;case 293:return V(n.expression)}}function j(e){switch(e.kind){case 162:case 201:return e.dotDotDotToken;case 223:case 293:return e}}function K(n){var t=H(n);return e.Debug.assert(!!t||e.isSpreadAssignment(n),"Invalid property name for binding element."),t}function H(n){switch(n.kind){case 201:if(n.propertyName){var t=n.propertyName;return e.isPrivateIdentifier(t)?e.Debug.failBadSyntaxKind(t):e.isComputedPropertyName(t)&&W(t.expression)?t.expression:t}break;case 291:if(n.name){t=n.name;return e.isPrivateIdentifier(t)?e.Debug.failBadSyntaxKind(t):e.isComputedPropertyName(t)&&W(t.expression)?t.expression:t}break;case 293:return n.name&&e.isPrivateIdentifier(n.name)?e.Debug.failBadSyntaxKind(n.name):n.name}var r=V(n);if(r&&e.isPropertyName(r))return r}function W(e){var n=e.kind;return 10===n||8===n}function z(e){switch(e.kind){case 199:case 200:case 202:return e.elements;case 203:return e.properties}}function J(n){if(n){var t=n;while(1){if(e.isIdentifier(t)||!t.body)return e.isIdentifier(t)?t:t.name;t=t.body}}}function q(e){var n=e.kind;return 162===n||164===n||165===n||166===n||167===n||169===n||170===n||171===n||174===n||211===n||212===n||224===n||235===n||254===n||255===n||256===n||257===n||258===n||259===n||263===n||264===n||269===n||270===n}function X(n){var t=n.kind;return 104===t||110===t||95===t||e.isLiteralExpression(n)||e.isPrefixUnaryExpression(n)}function Y(e){return 42===e}function Q(e){return 41===e||43===e||44===e}function Z(e){return Y(e)||Q(e)}function $(e){return 39===e||40===e}function ee(e){return $(e)||Z(e)}function ne(e){return 47===e||48===e||49===e}function te(e){return ne(e)||ee(e)}function re(e){return 29===e||32===e||31===e||33===e||102===e||101===e}function ie(e){return re(e)||te(e)}function ae(e){return 34===e||36===e||35===e||37===e}function oe(e){return ae(e)||ie(e)}function se(e){return 50===e||51===e||52===e}function ce(e){return se(e)||oe(e)}function le(e){return 55===e||56===e}function ue(e){return le(e)||ce(e)}function de(n){return 60===n||ue(n)||e.isAssignmentOperator(n)}function pe(e){return de(e)||27===e}function fe(e){return pe(e.kind)}var me;e.createEmptyExports=n,e.createMemberAccessForPropertyName=t,e.createJsxFactoryExpression=o,e.createExpressionForJsxElement=c,e.createExpressionForJsxFragment=l,e.createForOfBindingStatement=u,e.insertLeadingStatement=d,e.createExpressionFromEntityName=p,e.createExpressionForPropertyName=f,e.createExpressionForObjectLiteralElementLike=h,e.expandPreOrPostfixIncrementOrDecrementExpression=v,e.isInternalName=b,e.isLocalName=E,e.isExportName=T,e.findUseStrictPrologue=x,e.startsWithUseStrict=L,e.isCommaSequence=D,e.isOuterExpression=A,e.skipOuterExpressions=C,e.skipAssertions=I,e.startOnNewLine=N,e.getExternalHelpersModuleName=k,e.hasRecordedExternalHelpers=R,e.createExternalHelpersImportDeclarationIfNeeded=O,e.getOrCreateExternalHelpersModuleNameIfNeeded=w,e.getLocalNameForExternalImport=P,e.getExternalModuleNameLiteral=M,e.tryGetModuleNameFromFile=G,e.getInitializerOfBindingOrAssignmentElement=U,e.getTargetOfBindingOrAssignmentElement=V,e.getRestIndicatorOfBindingOrAssignmentElement=j,e.getPropertyNameOfBindingOrAssignmentElement=K,e.tryGetPropertyNameOfBindingOrAssignmentElement=H,e.getElementsOfBindingOrAssignmentPattern=z,e.getJSDocTypeAliasName=J,e.canHaveModifiers=q,e.isTypeNodeOrTypeParameterDeclaration=e.or(e.isTypeNode,e.isTypeParameterDeclaration),e.isQuestionOrExclamationToken=e.or(e.isQuestionToken,e.isExclamationToken),e.isIdentifierOrThisTypeNode=e.or(e.isIdentifier,e.isThisTypeNode),e.isReadonlyKeywordOrPlusOrMinusToken=e.or(e.isReadonlyKeyword,e.isPlusToken,e.isMinusToken),e.isQuestionOrPlusOrMinusToken=e.or(e.isQuestionToken,e.isPlusToken,e.isMinusToken),e.isModuleName=e.or(e.isIdentifier,e.isStringLiteral),e.isLiteralTypeLikeExpression=X,e.isBinaryOperatorToken=fe,function(n){function t(n,r,i,a,o,s,l){var u=r>0?o[r-1]:void 0;return e.Debug.assertEqual(i[r],t),o[r]=n.onEnter(a[r],u,l),i[r]=c(n,t),r}function r(n,t,i,a,o,s,d){e.Debug.assertEqual(i[t],r),e.Debug.assertIsDefined(n.onLeft),i[t]=c(n,r);var p=n.onLeft(a[t].left,o[t],a[t]);return p?(u(t,a,p),l(t,i,a,o,p)):t}function i(n,t,r,a,o,s,l){return e.Debug.assertEqual(r[t],i),e.Debug.assertIsDefined(n.onOperator),r[t]=c(n,i),n.onOperator(a[t].operatorToken,o[t],a[t]),t}function a(n,t,r,i,o,s,d){e.Debug.assertEqual(r[t],a),e.Debug.assertIsDefined(n.onRight),r[t]=c(n,a);var p=n.onRight(i[t].right,o[t],i[t]);return p?(u(t,i,p),l(t,r,i,o,p)):t}function o(n,t,r,i,a,s,l){e.Debug.assertEqual(r[t],o),r[t]=c(n,o);var u=n.onExit(i[t],a[t]);if(t>0){if(t--,n.foldState){var d=r[t]===o?"right":"left";a[t]=n.foldState(a[t],u,d)}}else s.value=u;return t}function s(n,t,r,i,a,o,c){return e.Debug.assertEqual(r[t],s),t}function c(n,c){switch(c){case t:if(n.onLeft)return r;case r:if(n.onOperator)return i;case i:if(n.onRight)return a;case a:return o;case o:return s;case s:return s;default:e.Debug.fail("Invalid state")}}function l(e,n,r,i,a){return e++,n[e]=t,r[e]=a,i[e]=void 0,e}function u(n,t,r){if(e.Debug.shouldAssert(2))while(n>=0)e.Debug.assert(t[n]!==r,"Circular traversal detected."),n--}n.enter=t,n.left=r,n.operator=i,n.right=a,n.exit=o,n.done=s,n.nextState=c}(me||(me={}));var _e=function(){function e(e,n,t,r,i,a){this.onEnter=e,this.onLeft=n,this.onOperator=t,this.onRight=r,this.onExit=i,this.foldState=a}return e}();function ge(n,t,r,i,a,o){var s=new _e(n,t,r,i,a,o);return c;function c(n,t){var r={value:void 0},i=[me.enter],a=[n],o=[void 0],c=0;while(i[c]!==me.done)c=i[c](s,c,i,a,o,r,t);return e.Debug.assertEqual(c,0),r.value}}e.createBinaryExpressionTrampoline=ge}(d||(d={})),function(e){function n(n,t){return t?e.setTextRangePosEnd(n,t.pos,t.end):n}e.setTextRange=n}(d||(d={})),function(e){var n,t,r,i,o,s,c,l,u;function d(e,n){return n&&e(n)}function p(e,n,t){if(t){if(n)return n(t);for(var r=0,i=t;r<i.length;r++){var a=i[r],o=e(a);if(o)return o}}}function f(e,n){return 42===e.charCodeAt(n+1)&&42===e.charCodeAt(n+2)&&47!==e.charCodeAt(n+3)}function m(n,t,r){if(n&&!(n.kind<=158))switch(n.kind){case 159:return d(t,n.left)||d(t,n.right);case 161:return d(t,n.name)||d(t,n.constraint)||d(t,n.default)||d(t,n.expression);case 292:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||d(t,n.questionToken)||d(t,n.exclamationToken)||d(t,n.equalsToken)||d(t,n.objectAssignmentInitializer);case 293:return d(t,n.expression);case 162:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.dotDotDotToken)||d(t,n.name)||d(t,n.questionToken)||d(t,n.type)||d(t,n.initializer);case 165:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||d(t,n.questionToken)||d(t,n.exclamationToken)||d(t,n.type)||d(t,n.initializer);case 164:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||d(t,n.questionToken)||d(t,n.type)||d(t,n.initializer);case 291:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||d(t,n.questionToken)||d(t,n.initializer);case 252:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||d(t,n.exclamationToken)||d(t,n.type)||d(t,n.initializer);case 201:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.dotDotDotToken)||d(t,n.propertyName)||d(t,n.name)||d(t,n.initializer);case 177:case 178:case 172:case 173:case 174:return p(t,r,n.decorators)||p(t,r,n.modifiers)||p(t,r,n.typeParameters)||p(t,r,n.parameters)||d(t,n.type);case 167:case 166:case 169:case 170:case 171:case 211:case 254:case 212:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.asteriskToken)||d(t,n.name)||d(t,n.questionToken)||d(t,n.exclamationToken)||p(t,r,n.typeParameters)||p(t,r,n.parameters)||d(t,n.type)||d(t,n.equalsGreaterThanToken)||d(t,n.body);case 168:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.body);case 176:return d(t,n.typeName)||p(t,r,n.typeArguments);case 175:return d(t,n.assertsModifier)||d(t,n.parameterName)||d(t,n.type);case 179:return d(t,n.exprName);case 180:return p(t,r,n.members);case 181:return d(t,n.elementType);case 182:return p(t,r,n.elements);case 185:case 186:return p(t,r,n.types);case 187:return d(t,n.checkType)||d(t,n.extendsType)||d(t,n.trueType)||d(t,n.falseType);case 188:return d(t,n.typeParameter);case 198:return d(t,n.argument)||d(t,n.qualifier)||p(t,r,n.typeArguments);case 189:case 191:return d(t,n.type);case 192:return d(t,n.objectType)||d(t,n.indexType);case 193:return d(t,n.readonlyToken)||d(t,n.typeParameter)||d(t,n.nameType)||d(t,n.questionToken)||d(t,n.type);case 194:return d(t,n.literal);case 195:return d(t,n.dotDotDotToken)||d(t,n.name)||d(t,n.questionToken)||d(t,n.type);case 199:case 200:return p(t,r,n.elements);case 202:return p(t,r,n.elements);case 203:return p(t,r,n.properties);case 204:return d(t,n.expression)||d(t,n.questionDotToken)||d(t,n.name);case 205:return d(t,n.expression)||d(t,n.questionDotToken)||d(t,n.argumentExpression);case 206:case 207:return d(t,n.expression)||d(t,n.questionDotToken)||p(t,r,n.typeArguments)||p(t,r,n.arguments);case 208:return d(t,n.tag)||d(t,n.questionDotToken)||p(t,r,n.typeArguments)||d(t,n.template);case 209:return d(t,n.type)||d(t,n.expression);case 210:return d(t,n.expression);case 213:return d(t,n.expression);case 214:return d(t,n.expression);case 215:return d(t,n.expression);case 217:return d(t,n.operand);case 222:return d(t,n.asteriskToken)||d(t,n.expression);case 216:return d(t,n.expression);case 218:return d(t,n.operand);case 219:return d(t,n.left)||d(t,n.operatorToken)||d(t,n.right);case 227:return d(t,n.expression)||d(t,n.type);case 228:return d(t,n.expression);case 229:return d(t,n.name);case 220:return d(t,n.condition)||d(t,n.questionToken)||d(t,n.whenTrue)||d(t,n.colonToken)||d(t,n.whenFalse);case 223:return d(t,n.expression);case 233:case 260:return p(t,r,n.statements);case 300:return p(t,r,n.statements)||d(t,n.endOfFileToken);case 235:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.declarationList);case 253:return p(t,r,n.declarations);case 236:return d(t,n.expression);case 237:return d(t,n.expression)||d(t,n.thenStatement)||d(t,n.elseStatement);case 238:return d(t,n.statement)||d(t,n.expression);case 239:return d(t,n.expression)||d(t,n.statement);case 240:return d(t,n.initializer)||d(t,n.condition)||d(t,n.incrementor)||d(t,n.statement);case 241:return d(t,n.initializer)||d(t,n.expression)||d(t,n.statement);case 242:return d(t,n.awaitModifier)||d(t,n.initializer)||d(t,n.expression)||d(t,n.statement);case 243:case 244:return d(t,n.label);case 245:return d(t,n.expression);case 246:return d(t,n.expression)||d(t,n.statement);case 247:return d(t,n.expression)||d(t,n.caseBlock);case 261:return p(t,r,n.clauses);case 287:return d(t,n.expression)||p(t,r,n.statements);case 288:return p(t,r,n.statements);case 248:return d(t,n.label)||d(t,n.statement);case 249:return d(t,n.expression);case 250:return d(t,n.tryBlock)||d(t,n.catchClause)||d(t,n.finallyBlock);case 290:return d(t,n.variableDeclaration)||d(t,n.block);case 163:return d(t,n.expression);case 255:case 224:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||p(t,r,n.typeParameters)||p(t,r,n.heritageClauses)||p(t,r,n.members);case 256:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||p(t,r,n.typeParameters)||p(t,r,n.heritageClauses)||p(t,r,n.members);case 257:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||p(t,r,n.typeParameters)||d(t,n.type);case 258:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||p(t,r,n.members);case 294:return d(t,n.name)||d(t,n.initializer);case 259:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||d(t,n.body);case 263:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.name)||d(t,n.moduleReference);case 264:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.importClause)||d(t,n.moduleSpecifier);case 265:return d(t,n.name)||d(t,n.namedBindings);case 262:return d(t,n.name);case 266:return d(t,n.name);case 272:return d(t,n.name);case 267:case 271:return p(t,r,n.elements);case 270:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.exportClause)||d(t,n.moduleSpecifier);case 268:case 273:return d(t,n.propertyName)||d(t,n.name);case 269:return p(t,r,n.decorators)||p(t,r,n.modifiers)||d(t,n.expression);case 221:return d(t,n.head)||p(t,r,n.templateSpans);case 231:return d(t,n.expression)||d(t,n.literal);case 196:return d(t,n.head)||p(t,r,n.templateSpans);case 197:return d(t,n.type)||d(t,n.literal);case 160:return d(t,n.expression);case 289:return p(t,r,n.types);case 226:return d(t,n.expression)||p(t,r,n.typeArguments);case 275:return d(t,n.expression);case 274:return p(t,r,n.decorators);case 346:return p(t,r,n.elements);case 276:return d(t,n.openingElement)||p(t,r,n.children)||d(t,n.closingElement);case 280:return d(t,n.openingFragment)||p(t,r,n.children)||d(t,n.closingFragment);case 277:case 278:return d(t,n.tagName)||p(t,r,n.typeArguments)||d(t,n.attributes);case 284:return p(t,r,n.properties);case 283:return d(t,n.name)||d(t,n.initializer);case 285:return d(t,n.expression);case 286:return d(t,n.dotDotDotToken)||d(t,n.expression);case 279:return d(t,n.tagName);case 183:case 184:case 304:case 310:case 309:case 311:case 313:return d(t,n.type);case 312:return p(t,r,n.parameters)||d(t,n.type);case 315:return("string"===typeof n.comment?void 0:p(t,r,n.comment))||p(t,r,n.tags);case 341:return d(t,n.tagName)||d(t,n.name)||("string"===typeof n.comment?void 0:p(t,r,n.comment));case 305:return d(t,n.name);case 306:return d(t,n.left)||d(t,n.right);case 335:case 342:return d(t,n.tagName)||(n.isNameFirst?d(t,n.name)||d(t,n.typeExpression)||("string"===typeof n.comment?void 0:p(t,r,n.comment)):d(t,n.typeExpression)||d(t,n.name)||("string"===typeof n.comment?void 0:p(t,r,n.comment)));case 325:return d(t,n.tagName)||("string"===typeof n.comment?void 0:p(t,r,n.comment));case 324:return d(t,n.tagName)||d(t,n.class)||("string"===typeof n.comment?void 0:p(t,r,n.comment));case 323:return d(t,n.tagName)||d(t,n.class)||("string"===typeof n.comment?void 0:p(t,r,n.comment));case 339:return d(t,n.tagName)||d(t,n.constraint)||p(t,r,n.typeParameters)||("string"===typeof n.comment?void 0:p(t,r,n.comment));case 340:return d(t,n.tagName)||(n.typeExpression&&304===n.typeExpression.kind?d(t,n.typeExpression)||d(t,n.fullName)||("string"===typeof n.comment?void 0:p(t,r,n.comment)):d(t,n.fullName)||d(t,n.typeExpression)||("string"===typeof n.comment?void 0:p(t,r,n.comment)));case 333:return d(t,n.tagName)||d(t,n.fullName)||d(t,n.typeExpression)||("string"===typeof n.comment?void 0:p(t,r,n.comment));case 336:case 338:case 337:case 334:return d(t,n.tagName)||d(t,n.typeExpression)||("string"===typeof n.comment?void 0:p(t,r,n.comment));case 318:return e.forEach(n.typeParameters,t)||e.forEach(n.parameters,t)||d(t,n.type);case 319:case 320:case 321:return d(t,n.name);case 317:return e.forEach(n.jsDocPropertyTags,t);case 322:case 327:case 328:case 329:case 330:case 331:case 326:return d(t,n.tagName)||("string"===typeof n.comment?void 0:p(t,r,n.comment));case 345:return d(t,n.expression)}}function _(n,t,r){var i=g(n),a=[];while(a.length<i.length)a.push(n);while(0!==i.length){var o=i.pop(),s=a.pop();if(e.isArray(o)){if(r){var c=r(o,s);if(c){if("skip"===c)continue;return c}}for(var l=o.length-1;l>=0;--l)i.push(o[l]),a.push(s)}else{c=t(o,s);if(c){if("skip"===c)continue;return c}if(o.kind>=159)for(var u=0,d=g(o);u<d.length;u++){var p=d[u];i.push(p),a.push(o)}}}}function g(e){var n=[];return m(e,t,t),n;function t(e){n.unshift(e)}}function y(n,t,r,i,a){var o;return void 0===i&&(i=!1),null===e.tracing||void 0===e.tracing||e.tracing.push("parse","createSourceFile",{path:n},!0),e.performance.mark("beforeParse"),e.perfLogger.logStartParseSourceFile(n),o=100===r?l.parseSourceFile(n,t,r,void 0,i,6):l.parseSourceFile(n,t,r,void 0,i,a),e.perfLogger.logStopParseSourceFile(),e.performance.mark("afterParse"),e.performance.measure("Parse","beforeParse","afterParse"),null===e.tracing||void 0===e.tracing||e.tracing.pop(),o}function h(e,n){return l.parseIsolatedEntityName(e,n)}function v(e,n){return l.parseJsonText(e,n)}function b(e){return void 0!==e.externalModuleIndicator}function E(e,n,t,r){void 0===r&&(r=!1);var i=u.updateSourceFile(e,n,t,r);return i.flags|=3145728&e.flags,i}function T(e,n,t){var r=l.JSDocParser.parseIsolatedJSDocComment(e,n,t);return r&&r.jsDoc&&l.fixupParentReferences(r.jsDoc),r}function S(e,n,t){return l.JSDocParser.parseJSDocTypeExpressionForTests(e,n,t)}function x(n){return e.fileExtensionIs(n,".d.ts")}function L(n,t){for(var r=[],i=0,a=e.getLeadingCommentRanges(t,0)||e.emptyArray;i<a.length;i++){var o=a[i],s=t.substring(o.pos,o.end);k(r,o,s)}n.pragmas=new e.Map;for(var c=0,l=r;c<l.length;c++){var u=l[c];if(n.pragmas.has(u.name)){var d=n.pragmas.get(u.name);d instanceof Array?d.push(u.args):n.pragmas.set(u.name,[d,u.args])}else n.pragmas.set(u.name,u.args)}}function D(n,t){n.checkJsDirective=void 0,n.referencedFiles=[],n.typeReferenceDirectives=[],n.libReferenceDirectives=[],n.amdDependencies=[],n.hasNoDefaultLib=!1,n.pragmas.forEach(function(r,i){switch(i){case"reference":var a=n.referencedFiles,o=n.typeReferenceDirectives,s=n.libReferenceDirectives;e.forEach(e.toArray(r),function(r){var i=r.arguments,c=i.types,l=i.lib,u=i.path;r.arguments["no-default-lib"]?n.hasNoDefaultLib=!0:c?o.push({pos:c.pos,end:c.end,fileName:c.value}):l?s.push({pos:l.pos,end:l.end,fileName:l.value}):u?a.push({pos:u.pos,end:u.end,fileName:u.value}):t(r.range.pos,r.range.end-r.range.pos,e.Diagnostics.Invalid_reference_directive_syntax)});break;case"amd-dependency":n.amdDependencies=e.map(e.toArray(r),function(e){return{name:e.arguments.name,path:e.arguments.path}});break;case"amd-module":if(r instanceof Array)for(var c=0,l=r;c<l.length;c++){var u=l[c];n.moduleName&&t(u.range.pos,u.range.end-u.range.pos,e.Diagnostics.An_AMD_module_cannot_have_multiple_name_assignments),n.moduleName=u.arguments.name}else n.moduleName=r.arguments.name;break;case"ts-nocheck":case"ts-check":e.forEach(e.toArray(r),function(e){(!n.checkJsDirective||e.range.pos>n.checkJsDirective.pos)&&(n.checkJsDirective={enabled:"ts-check"===i,end:e.range.end,pos:e.range.pos})});break;case"jsx":case"jsxfrag":case"jsximportsource":case"jsxruntime":return;default:e.Debug.fail("Unhandled pragma kind")}})}(function(e){e[e["None"]=0]="None",e[e["Yield"]=1]="Yield",e[e["Await"]=2]="Await",e[e["Type"]=4]="Type",e[e["IgnoreMissingOpenBrace"]=16]="IgnoreMissingOpenBrace",e[e["JSDoc"]=32]="JSDoc"})(n||(n={})),function(e){e[e["TryParse"]=0]="TryParse",e[e["Lookahead"]=1]="Lookahead",e[e["Reparse"]=2]="Reparse"}(t||(t={})),e.parseBaseNodeFactory={createBaseSourceFileNode:function(n){return new(c||(c=e.objectAllocator.getSourceFileConstructor()))(n,-1,-1)},createBaseIdentifierNode:function(n){return new(o||(o=e.objectAllocator.getIdentifierConstructor()))(n,-1,-1)},createBasePrivateIdentifierNode:function(n){return new(s||(s=e.objectAllocator.getPrivateIdentifierConstructor()))(n,-1,-1)},createBaseTokenNode:function(n){return new(i||(i=e.objectAllocator.getTokenConstructor()))(n,-1,-1)},createBaseNode:function(n){return new(r||(r=e.objectAllocator.getNodeConstructor()))(n,-1,-1)}},e.parseNodeFactory=e.createNodeFactory(1,e.parseBaseNodeFactory),e.isJSDocLikeText=f,e.forEachChild=m,e.forEachChildRecursively=_,e.createSourceFile=y,e.parseIsolatedEntityName=h,e.parseJsonText=v,e.isExternalModule=b,e.updateSourceFile=E,e.parseIsolatedJSDocComment=T,e.parseJSDocTypeExpressionForTests=S,function(n){var t,r,i,o,s,c=e.createScanner(99,!0),l=20480;function d(e){return C++,e}var p,_,g,y,h,v,E,T,S,A,C,I,N,k,R,O,P,M={createBaseSourceFileNode:function(e){return d(new s(e,0,0))},createBaseIdentifierNode:function(e){return d(new i(e,0,0))},createBasePrivateIdentifierNode:function(e){return d(new o(e,0,0))},createBaseTokenNode:function(e){return d(new r(e,0,0))},createBaseNode:function(e){return d(new t(e,0,0))}},F=e.createNodeFactory(11,M),G=!0,B=!1;function U(n,t,r,i,a,o){var s;if(void 0===a&&(a=!1),o=e.ensureScriptKind(n,o),6===o){var c=j(n,t,r,i,a);return e.convertToObjectWorker(c,null===(s=c.statements[0])||void 0===s?void 0:s.expression,c.parseDiagnostics,!1,void 0,void 0),c.referencedFiles=e.emptyArray,c.typeReferenceDirectives=e.emptyArray,c.libReferenceDirectives=e.emptyArray,c.amdDependencies=e.emptyArray,c.hasNoDefaultLib=!1,c.pragmas=e.emptyMap,c}K(n,t,r,i,o);var l=W(r,a,o);return H(),l}function V(e,n){K("",e,n,void 0,1),Ce();var t=ut(!0),r=1===Le()&&!E.length;return H(),r?t:void 0}function j(n,t,r,i,a){void 0===r&&(r=2),void 0===a&&(a=!1),K(n,t,r,i,6),_=P,Ce();var o,s,c=Se();if(1===Le())o=dn([],c,c),s=on();else{var l=void 0;while(1!==Le()){var u=void 0;switch(Le()){case 22:u=oa();break;case 110:case 95:case 104:u=on();break;case 40:u=Ue(function(){return 8===Ce()&&58!==Ce()})?yi():ca();break;case 8:case 10:if(Ue(function(){return 58!==Ce()})){u=bt();break}default:u=ca();break}l&&e.isArray(l)?l.push(u):l?l=[l,u]:(l=u,1!==Le()&&he(e.Diagnostics.Unexpected_token))}var d=e.isArray(l)?pn(F.createArrayLiteralExpression(l),c):e.Debug.checkDefined(l),p=F.createExpressionStatement(d);pn(p,c),o=dn([p],c),s=rn(1,e.Diagnostics.Unexpected_token)}var f=Q(n,2,6,!1,o,s,_);a&&Y(f),f.nodeCount=C,f.identifierCount=k,f.identifiers=I,f.parseDiagnostics=e.attachFileToDiagnostics(E,f),T&&(f.jsDocDiagnostics=e.attachFileToDiagnostics(T,f));var m=f;return H(),m}function K(n,a,l,u,d){switch(t=e.objectAllocator.getNodeConstructor(),r=e.objectAllocator.getTokenConstructor(),i=e.objectAllocator.getIdentifierConstructor(),o=e.objectAllocator.getPrivateIdentifierConstructor(),s=e.objectAllocator.getSourceFileConstructor(),p=e.normalizePath(n),g=a,y=l,S=u,h=d,v=e.getLanguageVariant(d),E=[],R=0,I=new e.Map,N=new e.Map,k=0,C=0,_=0,G=!0,h){case 1:case 2:P=131072;break;case 6:P=33685504;break;default:P=0;break}B=!1,c.setText(g),c.setOnError(Te),c.setScriptTarget(y),c.setLanguageVariant(v)}function H(){c.clearCommentDirectives(),c.setText(""),c.setOnError(void 0),g=void 0,y=void 0,S=void 0,h=void 0,v=void 0,_=0,E=void 0,T=void 0,R=0,I=void 0,O=void 0,G=!0}function W(n,t,r){var i=x(p);i&&(P|=8388608),_=P,Ce();var a=Hn(0,Va);e.Debug.assert(1===Le());var o=q(on()),s=Q(p,n,r,i,a,o,_);return L(s,g),D(s,l),s.commentDirectives=c.getCommentDirectives(),s.nodeCount=C,s.identifierCount=k,s.identifiers=I,s.parseDiagnostics=e.attachFileToDiagnostics(E,s),T&&(s.jsDocDiagnostics=e.attachFileToDiagnostics(T,s)),t&&Y(s),s;function l(n,t,r){E.push(e.createDetachedDiagnostic(p,n,t,r))}}function z(e,n){return n?q(e):e}n.parseSourceFile=U,n.parseIsolatedEntityName=V,n.parseJsonText=j;var J=!1;function q(n){e.Debug.assert(!n.jsDoc);var t=e.mapDefined(e.getJSDocCommentRanges(n,g),function(e){return Je.parseJSDocComment(n,e.pos,e.end-e.pos)});return t.length&&(n.jsDoc=t),J&&(J=!1,n.flags|=134217728),n}function X(n){var t=S,r=u.createSyntaxCursor(n);S={currentNode:g};var i=[],a=E;E=[];var o=0,s=m(n.statements,0),l=function(){var t=n.statements[o],r=n.statements[s];e.addRange(i,n.statements,o,s),o=_(n.statements,s);var l=e.findIndex(a,function(e){return e.start>=t.pos}),u=l>=0?e.findIndex(a,function(e){return e.start>=r.pos},l):-1;l>=0&&e.addRange(E,a,l,u>=0?u:void 0),Be(function(){var e=P;P|=32768,c.setTextPos(r.pos),Ce();while(1!==Le()){var t=c.getStartPos(),a=Wn(0,Va);if(i.push(a),t===c.getStartPos()&&Ce(),o>=0){var s=n.statements[o];if(a.end===s.pos)break;a.end>s.pos&&(o=_(n.statements,o+1))}}P=e},2),s=o>=0?m(n.statements,o):-1};while(-1!==s)l();if(o>=0){var d=n.statements[o];e.addRange(i,n.statements,o);var p=e.findIndex(a,function(e){return e.start>=d.pos});p>=0&&e.addRange(E,a,p)}return S=t,F.updateSourceFile(n,e.setTextRange(F.createNodeArray(i),n.statements));function f(e){return!(32768&e.flags)&&!!(16777216&e.transformFlags)}function m(e,n){for(var t=n;t<e.length;t++)if(f(e[t]))return t;return-1}function _(e,n){for(var t=n;t<e.length;t++)if(!f(e[t]))return t;return-1}function g(e){var n=r.currentNode(e);return G&&n&&f(n)&&(n.intersectsChange=!0),n}}function Y(n){e.setParentRecursive(n,!0)}function Q(n,t,r,i,a,o,s){var c=F.createSourceFile(a,o,s);return e.setTextRangePosWidth(c,0,g.length),ls(c),!i&&b(c)&&16777216&c.transformFlags&&(c=X(c)),c.text=g,c.bindDiagnostics=[],c.bindSuggestionDiagnostics=void 0,c.languageVersion=t,c.fileName=n,c.languageVariant=e.getLanguageVariant(r),c.isDeclarationFile=i,c.scriptKind=r,c}function Z(e,n){e?P|=n:P&=~n}function $(e){Z(e,4096)}function ee(e){Z(e,8192)}function ne(e){Z(e,16384)}function te(e){Z(e,32768)}function re(e,n){var t=e&P;if(t){Z(!1,t);var r=n();return Z(!0,t),r}return n()}function ie(e,n){var t=e&~P;if(t){Z(!0,t);var r=n();return Z(!1,t),r}return n()}function ae(e){return re(4096,e)}function oe(e){return ie(4096,e)}function se(e){return ie(8192,e)}function ce(e){return ie(16384,e)}function le(e){return ie(32768,e)}function ue(e){return re(32768,e)}function de(e){return ie(40960,e)}function pe(e){return re(40960,e)}function fe(e){return 0!==(P&e)}function me(){return fe(8192)}function _e(){return fe(4096)}function ge(){return fe(16384)}function ye(){return fe(32768)}function he(e,n){be(c.getTokenPos(),c.getTextPos(),e,n)}function ve(n,t,r,i){var a=e.lastOrUndefined(E);a&&n===a.start||E.push(e.createDetachedDiagnostic(p,n,t,r,i)),B=!0}function be(e,n,t,r){ve(e,n-e,t,r)}function Ee(e,n,t){be(e.pos,e.end,n,t)}function Te(e,n){ve(c.getTextPos(),n,e)}function Se(){return c.getStartPos()}function xe(){return c.hasPrecedingJSDocComment()}function Le(){return A}function De(){return A=c.scan()}function Ae(e){return Ce(),e()}function Ce(){return e.isKeyword(A)&&(c.hasUnicodeEscape()||c.hasExtendedUnicodeEscape())&&be(c.getTokenPos(),c.getTextPos(),e.Diagnostics.Keywords_cannot_contain_escape_characters),De()}function Ie(){return A=c.scanJsDocToken()}function Ne(){return A=c.reScanGreaterToken()}function ke(){return A=c.reScanSlashToken()}function Re(e){return A=c.reScanTemplateToken(e)}function Oe(){return A=c.reScanTemplateHeadOrNoSubstitutionTemplate()}function we(){return A=c.reScanLessThanToken()}function Pe(){return A=c.reScanHashToken()}function Me(){return A=c.scanJsxIdentifier()}function Fe(){return A=c.scanJsxToken()}function Ge(){return A=c.scanJsxAttributeValue()}function Be(n,t){var r=A,i=E.length,a=B,o=P,s=0!==t?c.lookAhead(n):c.tryScan(n);return e.Debug.assert(o===P),s&&0===t||(A=r,2!==t&&(E.length=i),B=a),s}function Ue(e){return Be(e,1)}function Ve(e){return Be(e,0)}function je(){return 79===Le()||Le()>116}function Ke(){return 79===Le()||(125!==Le()||!me())&&((131!==Le()||!ye())&&Le()>116)}function He(n,t,r){return void 0===r&&(r=!0),Le()===n?(r&&Ce(),!0):(t?he(t):he(e.Diagnostics._0_expected,e.tokenToString(n)),!1)}n.fixupParentReferences=Y;var We,ze,Je,qe=Object.keys(e.textToKeywordObj).filter(function(e){return e.length>2});function Xe(n){var t;if(e.isTaggedTemplateExpression(n))be(e.skipTrivia(g,n.template.pos),n.template.end,e.Diagnostics.Module_declaration_names_may_only_use_or_quoted_strings);else{var r=e.isIdentifier(n)?e.idText(n):void 0;if(r&&e.isIdentifierText(r,y)){var i=e.skipTrivia(g,n.pos);switch(r){case"const":case"let":case"var":return void be(i,n.end,e.Diagnostics.Variable_declaration_not_allowed_at_this_location);case"declare":return;case"interface":return void Ye(e.Diagnostics.Interface_name_cannot_be_0,e.Diagnostics.Interface_must_be_given_a_name,18);case"is":return void be(i,c.getTextPos(),e.Diagnostics.A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods);case"module":case"namespace":return void Ye(e.Diagnostics.Namespace_name_cannot_be_0,e.Diagnostics.Namespace_must_be_given_a_name,18);case"type":return void Ye(e.Diagnostics.Type_alias_name_cannot_be_0,e.Diagnostics.Type_alias_must_be_given_a_name,63)}var a=null!==(t=e.getSpellingSuggestion(r,qe,function(e){return e}))&&void 0!==t?t:Qe(r);a?be(i,n.end,e.Diagnostics.Unknown_keyword_or_identifier_Did_you_mean_0,a):0!==Le()&&be(i,n.end,e.Diagnostics.Unexpected_keyword_or_identifier)}else he(e.Diagnostics._0_expected,e.tokenToString(26))}}function Ye(n,t,r){Le()===r?he(t):he(n,e.tokenToString(Le()))}function Qe(n){for(var t=0,r=qe;t<r.length;t++){var i=r[t];if(n.length>i.length+2&&e.startsWith(n,i))return i+" "+n.slice(i.length)}}function Ze(n,t,r){if(59!==Le()||c.hasPrecedingLineBreak())return 20===Le()?(he(e.Diagnostics.Cannot_start_a_function_call_in_a_type_annotation),void Ce()):void(!t||cn()?ln()||(r?18===Le()&&he(e.Diagnostics._0_expected,e.tokenToString(26)):Xe(n)):r?he(e.Diagnostics._0_expected,e.tokenToString(26)):he(e.Diagnostics.Expected_for_property_initializer));he(e.Diagnostics.Decorators_must_precede_the_name_and_all_keywords_of_property_declarations)}function $e(n){return Le()===n?(Ie(),!0):(he(e.Diagnostics._0_expected,e.tokenToString(n)),!1)}function en(e){return Le()===e&&(Ce(),!0)}function nn(e){if(Le()===e)return on()}function tn(e){if(Le()===e)return sn()}function rn(n,t,r){return nn(n)||fn(n,!1,t||e.Diagnostics._0_expected,r||e.tokenToString(n))}function an(n){return tn(n)||fn(n,!1,e.Diagnostics._0_expected,e.tokenToString(n))}function on(){var e=Se(),n=Le();return Ce(),pn(F.createToken(n),e)}function sn(){var e=Se(),n=Le();return Ie(),pn(F.createToken(n),e)}function cn(){return 26===Le()||(19===Le()||1===Le()||c.hasPrecedingLineBreak())}function ln(){return!!cn()&&(26===Le()&&Ce(),!0)}function un(){return ln()||He(26)}function dn(n,t,r,i){var a=F.createNodeArray(n,i);return e.setTextRangePosEnd(a,t,null!==r&&void 0!==r?r:c.getStartPos()),a}function pn(n,t,r){return e.setTextRangePosEnd(n,t,null!==r&&void 0!==r?r:c.getStartPos()),P&&(n.flags|=P),B&&(B=!1,n.flags|=65536),n}function fn(n,t,r,i){t?ve(c.getStartPos(),0,r,i):r&&he(r,i);var a=Se(),o=79===n?F.createIdentifier("",void 0,void 0):e.isTemplateLiteralKind(n)?F.createTemplateLiteralLikeNode(n,"","",void 0):8===n?F.createNumericLiteral("",void 0):10===n?F.createStringLiteral("",void 0):274===n?F.createMissingDeclaration():F.createToken(n);return pn(o,a)}function mn(e){var n=I.get(e);return void 0===n&&I.set(e,n=e),n}function _n(n,t,r){if(n){k++;var i=Se(),a=Le(),o=mn(c.getTokenValue());return De(),pn(F.createIdentifier(o,void 0,a),i)}if(80===Le())return he(r||e.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies),_n(!0);if(0===Le()&&c.tryScan(function(){return 79===c.reScanInvalidIdentifier()}))return _n(!0);k++;var s=1===Le(),l=c.isReservedWord(),u=c.getTokenText(),d=l?e.Diagnostics.Identifier_expected_0_is_a_reserved_word_that_cannot_be_used_here:e.Diagnostics.Identifier_expected;return fn(79,s,t||d,u)}function gn(e){return _n(je(),void 0,e)}function yn(e,n){return _n(Ke(),e,n)}function hn(n){return _n(e.tokenIsIdentifierOrKeyword(Le()),n)}function vn(){return e.tokenIsIdentifierOrKeyword(Le())||10===Le()||8===Le()}function bn(e){if(10===Le()||8===Le()){var n=bt();return n.text=mn(n.text),n}return e&&22===Le()?Tn():80===Le()?xn():hn()}function En(){return bn(!0)}function Tn(){var e=Se();He(22);var n=ae(Xr);return He(23),pn(F.createComputedPropertyName(n),e)}function Sn(e){var n=N.get(e);return void 0===n&&N.set(e,n=e),n}function xn(){var e=Se(),n=F.createPrivateIdentifier(Sn(c.getTokenText()));return Ce(),pn(n,e)}function Ln(e){return Le()===e&&Ve(An)}function Dn(){return Ce(),!c.hasPrecedingLineBreak()&&kn()}function An(){switch(Le()){case 85:return 92===Ce();case 93:return Ce(),88===Le()?Ue(Rn):150===Le()?Ue(In):Cn();case 88:return Rn();case 124:return Dn();case 135:case 147:return Ce(),kn();default:return Dn()}}function Cn(){return 41!==Le()&&127!==Le()&&18!==Le()&&kn()}function In(){return Ce(),Cn()}function Nn(){return e.isModifierKind(Le())&&Ve(An)}function kn(){return 22===Le()||18===Le()||41===Le()||25===Le()||vn()}function Rn(){return Ce(),84===Le()||98===Le()||118===Le()||126===Le()&&Ue(Oa)||130===Le()&&Ue(wa)}function On(n,t){var r=zn(n);if(r)return!0;switch(n){case 0:case 1:case 3:return!(26===Le()&&t)&&Ga();case 2:return 82===Le()||88===Le();case 4:return Ue(nr);case 5:return Ue(fo)||26===Le()&&!t;case 6:return 22===Le()||vn();case 12:switch(Le()){case 22:case 41:case 25:case 24:return!0;default:return vn()}case 18:return vn();case 9:return 22===Le()||25===Le()||vn();case 7:return 18===Le()?Ue(wn):t?Ke()&&!Gn():zr()&&!Gn();case 8:return Za();case 10:return 27===Le()||25===Le()||Za();case 19:return Ke();case 15:switch(Le()){case 27:case 24:return!0}case 11:return 25===Le()||Jr();case 16:return Ut(!1);case 17:return Ut(!0);case 20:case 21:return 27===Le()||Lr();case 22:return Ro();case 23:return e.tokenIsIdentifierOrKeyword(Le());case 13:return e.tokenIsIdentifierOrKeyword(Le())||18===Le();case 14:return!0}return e.Debug.fail("Non-exhaustive case in 'isListElement'.")}function wn(){if(e.Debug.assert(18===Le()),19===Ce()){var n=Ce();return 27===n||18===n||94===n||117===n}return!0}function Pn(){return Ce(),Ke()}function Mn(){return Ce(),e.tokenIsIdentifierOrKeyword(Le())}function Fn(){return Ce(),e.tokenIsIdentifierOrKeywordOrGreaterThan(Le())}function Gn(){return(117===Le()||94===Le())&&Ue(Bn)}function Bn(){return Ce(),Jr()}function Un(){return Ce(),Lr()}function Vn(e){if(1===Le())return!0;switch(e){case 1:case 2:case 4:case 5:case 6:case 12:case 9:case 23:return 19===Le();case 3:return 19===Le()||82===Le()||88===Le();case 7:return 18===Le()||94===Le()||117===Le();case 8:return jn();case 19:return 31===Le()||20===Le()||18===Le()||94===Le()||117===Le();case 11:return 21===Le()||26===Le();case 15:case 21:case 10:return 23===Le();case 17:case 16:case 18:return 21===Le()||23===Le();case 20:return 27!==Le();case 22:return 18===Le()||19===Le();case 13:return 31===Le()||43===Le();case 14:return 29===Le()&&Ue(Wo);default:return!1}}function jn(){return!!cn()||(!!pi(Le())||38===Le())}function Kn(){for(var e=0;e<24;e++)if(R&1<<e&&(On(e,!0)||Vn(e)))return!0;return!1}function Hn(e,n){var t=R;R|=1<<e;var r=[],i=Se();while(!Vn(e))if(On(e,!1))r.push(Wn(e,n));else if(rt(e))break;return R=t,dn(r,i)}function Wn(e,n){var t=zn(e);return t?Jn(t):n()}function zn(n){if(S&&qn(n)&&!B){var t=S.currentNode(c.getStartPos());if(!(e.nodeIsMissing(t)||t.intersectsChange||e.containsParseError(t))){var r=25358336&t.flags;if(r===P&&Xn(t,n))return t.jsDocCache&&(t.jsDocCache=void 0),t}}}function Jn(e){return c.setTextPos(e.end),Ce(),e}function qn(e){switch(e){case 5:case 2:case 0:case 1:case 3:case 6:case 4:case 8:case 17:case 16:return!0}return!1}function Xn(e,n){switch(n){case 5:return Yn(e);case 2:return Qn(e);case 0:case 1:case 3:return Zn(e);case 6:return $n(e);case 4:return et(e);case 8:return nt(e);case 17:case 16:return tt(e)}return!1}function Yn(e){if(e)switch(e.kind){case 169:case 174:case 170:case 171:case 165:case 232:return!0;case 167:var n=e,t=79===n.name.kind&&133===n.name.originalKeywordKind;return!t}return!1}function Qn(e){if(e)switch(e.kind){case 287:case 288:return!0}return!1}function Zn(e){if(e)switch(e.kind){case 254:case 235:case 233:case 237:case 236:case 249:case 245:case 247:case 244:case 243:case 241:case 242:case 240:case 239:case 246:case 234:case 250:case 248:case 238:case 251:case 264:case 263:case 270:case 269:case 259:case 255:case 256:case 258:case 257:return!0}return!1}function $n(e){return 294===e.kind}function et(e){if(e)switch(e.kind){case 173:case 166:case 174:case 164:case 172:return!0}return!1}function nt(e){if(252!==e.kind)return!1;var n=e;return void 0===n.initializer}function tt(e){if(162!==e.kind)return!1;var n=e;return void 0===n.initializer}function rt(e){return it(e),!!Kn()||(Ce(),!1)}function it(n){switch(n){case 0:return 88===Le()?he(e.Diagnostics._0_expected,e.tokenToString(93)):he(e.Diagnostics.Declaration_or_statement_expected);case 1:return he(e.Diagnostics.Declaration_or_statement_expected);case 2:return he(e.Diagnostics.case_or_default_expected);case 3:return he(e.Diagnostics.Statement_expected);case 18:case 4:return he(e.Diagnostics.Property_or_signature_expected);case 5:return he(e.Diagnostics.Unexpected_token_A_constructor_method_accessor_or_property_was_expected);case 6:return he(e.Diagnostics.Enum_member_expected);case 7:return he(e.Diagnostics.Expression_expected);case 8:return e.isKeyword(Le())?he(e.Diagnostics._0_is_not_allowed_as_a_variable_declaration_name,e.tokenToString(Le())):he(e.Diagnostics.Variable_declaration_expected);case 9:return he(e.Diagnostics.Property_destructuring_pattern_expected);case 10:return he(e.Diagnostics.Array_element_destructuring_pattern_expected);case 11:return he(e.Diagnostics.Argument_expression_expected);case 12:return he(e.Diagnostics.Property_assignment_expected);case 15:return he(e.Diagnostics.Expression_or_comma_expected);case 17:return he(e.Diagnostics.Parameter_declaration_expected);case 16:return he(e.Diagnostics.Parameter_declaration_expected);case 19:return he(e.Diagnostics.Type_parameter_declaration_expected);case 20:return he(e.Diagnostics.Type_argument_expected);case 21:return he(e.Diagnostics.Type_expected);case 22:return he(e.Diagnostics.Unexpected_token_expected);case 23:return he(e.Diagnostics.Identifier_expected);case 13:return he(e.Diagnostics.Identifier_expected);case 14:return he(e.Diagnostics.Identifier_expected);default:return[void 0]}}function at(e,n,t){var r=R;R|=1<<e;var i=[],a=Se(),o=-1;while(1)if(On(e,!1)){var s=c.getStartPos();if(i.push(Wn(e,n)),o=c.getTokenPos(),en(27))continue;if(o=-1,Vn(e))break;He(27,ot(e)),t&&26===Le()&&!c.hasPrecedingLineBreak()&&Ce(),s===c.getStartPos()&&Ce()}else{if(Vn(e))break;if(rt(e))break}return R=r,dn(i,a,void 0,o>=0)}function ot(n){return 6===n?e.Diagnostics.An_enum_member_name_must_be_followed_by_a_or:void 0}function st(){var e=dn([],Se());return e.isMissingList=!0,e}function ct(e){return!!e.isMissingList}function lt(e,n,t,r){if(He(t)){var i=at(e,n);return He(r),i}return st()}function ut(e,n){var t=Se(),r=e?hn(n):yn(n),i=Se();while(en(24)){if(29===Le()){r.jsdocDotPos=i;break}i=Se(),r=pn(F.createQualifiedName(r,pt(e,!1)),t)}return r}function dt(e,n){return pn(F.createQualifiedName(e,n),e.pos)}function pt(n,t){if(c.hasPrecedingLineBreak()&&e.tokenIsIdentifierOrKeyword(Le())){var r=Ue(Ra);if(r)return fn(79,!0,e.Diagnostics.Identifier_expected)}if(80===Le()){var i=xn();return t?i:fn(79,!0,e.Diagnostics.Identifier_expected)}return n?hn():yn()}function ft(e){var n,t=Se(),r=[];do{n=vt(e),r.push(n)}while(16===n.literal.kind);return dn(r,t)}function mt(e){var n=Se();return pn(F.createTemplateExpression(Et(e),ft(e)),n)}function _t(){var e=Se();return pn(F.createTemplateLiteralType(Et(!1),gt()),e)}function gt(){var e,n=Se(),t=[];do{e=yt(),t.push(e)}while(16===e.literal.kind);return dn(t,n)}function yt(){var e=Se();return pn(F.createTemplateLiteralTypeSpan(Kr(),ht(!1)),e)}function ht(n){return 19===Le()?(Re(n),Tt()):rn(17,e.Diagnostics._0_expected,e.tokenToString(19))}function vt(e){var n=Se();return pn(F.createTemplateSpan(ae(Xr),ht(e)),n)}function bt(){return xt(Le())}function Et(n){n&&Oe();var t=xt(Le());return e.Debug.assert(15===t.kind,"Template head has wrong token kind"),t}function Tt(){var n=xt(Le());return e.Debug.assert(16===n.kind||17===n.kind,"Template fragment has wrong token kind"),n}function St(e){var n=14===e||17===e,t=c.getTokenText();return t.substring(1,t.length-(c.isUnterminated()?0:n?1:2))}function xt(n){var t=Se(),r=e.isTemplateLiteralKind(n)?F.createTemplateLiteralLikeNode(n,c.getTokenValue(),St(n),2048&c.getTokenFlags()):8===n?F.createNumericLiteral(c.getTokenValue(),c.getNumericLiteralFlags()):10===n?F.createStringLiteral(c.getTokenValue(),void 0,c.hasExtendedUnicodeEscape()):e.isLiteralKind(n)?F.createLiteralLikeNode(n,c.getTokenValue()):e.Debug.fail();return c.hasExtendedUnicodeEscape()&&(r.hasExtendedUnicodeEscape=!0),c.isUnterminated()&&(r.isUnterminated=!0),Ce(),pn(r,t)}function Lt(){return ut(!0,e.Diagnostics.Type_expected)}function Dt(){if(!c.hasPrecedingLineBreak()&&29===we())return lt(20,Kr,29,31)}function At(){var e=Se();return pn(F.createTypeReferenceNode(Lt(),Dt()),e)}function Ct(n){switch(n.kind){case 176:return e.nodeIsMissing(n.typeName);case 177:case 178:var t=n,r=t.parameters,i=t.type;return ct(r)||Ct(i);case 189:return Ct(n.type);default:return!1}}function It(e){return Ce(),pn(F.createTypePredicateNode(void 0,e,Kr()),e.pos)}function Nt(){var e=Se();return Ce(),pn(F.createThisTypeNode(),e)}function kt(){var e=Se();return Ce(),pn(F.createJSDocAllType(),e)}function Rt(){var e=Se();return Ce(),pn(F.createJSDocNonNullableType(xr()),e)}function Ot(){var e=Se();return Ce(),27===Le()||19===Le()||21===Le()||31===Le()||63===Le()||51===Le()?pn(F.createJSDocUnknownType(),e):pn(F.createJSDocNullableType(Kr()),e)}function wt(){var e=Se(),n=xe();if(Ue(Ko)){Ce();var t=qt(36),r=Wt(58,!1);return z(pn(F.createJSDocFunctionType(t,r),e),n)}return pn(F.createTypeReferenceNode(hn(),void 0),e)}function Pt(){var e,n=Se();return 108!==Le()&&103!==Le()||(e=hn(),He(58)),pn(F.createParameterDeclaration(void 0,void 0,void 0,e,void 0,Mt(),void 0),n)}function Mt(){c.setInJSDocType(!0);var e=Se();if(en(140)){var n=F.createJSDocNamepathType(void 0);e:while(1)switch(Le()){case 19:case 1:case 27:case 5:break e;default:Ie()}return c.setInJSDocType(!1),pn(n,e)}var t=en(25),r=Ur();return c.setInJSDocType(!1),t&&(r=pn(F.createJSDocVariadicType(r),e)),63===Le()?(Ce(),pn(F.createJSDocOptionalType(r),e)):r}function Ft(){var e=Se();return He(112),pn(F.createTypeQueryNode(ut(!0)),e)}function Gt(){var e,n,t=Se(),r=yn();en(94)&&(Lr()||!Jr()?e=Kr():n=Si());var i=en(63)?Kr():void 0,a=F.createTypeParameterDeclaration(r,e,i);return a.expression=n,pn(a,t)}function Bt(){if(29===Le())return lt(19,Gt,29,31)}function Ut(n){return 25===Le()||Za()||e.isModifierKind(Le())||59===Le()||Lr(!n)}function Vt(n){var t=$a(e.Diagnostics.Private_identifiers_cannot_be_used_as_parameters);return 0===e.getFullWidth(t)&&!e.some(n)&&e.isModifierKind(Le())&&Ce(),t}function jt(){return Ht(!0)}function Kt(){return Ht(!1)}function Ht(n){var t=Se(),r=xe(),i=n?le(ho):ho();if(108===Le()){var a=F.createParameterDeclaration(i,void 0,void 0,_n(!0),void 0,Wr(),void 0);return i&&Ee(i[0],e.Diagnostics.Decorators_may_not_be_applied_to_this_parameters),z(pn(a,t),r)}var o=G;G=!1;var s=bo(),c=z(pn(F.createParameterDeclaration(i,s,nn(25),Vt(s),nn(57),Wr(),Yr()),t),r);return G=o,c}function Wt(e,n){if(zt(e,n))return Ur()}function zt(n,t){return 38===n?(He(n),!0):!!en(58)||!(!t||38!==Le())&&(he(e.Diagnostics._0_expected,e.tokenToString(58)),Ce(),!0)}function Jt(e){var n=me(),t=ye();ee(!!(1&e)),te(!!(2&e));var r=32&e?at(17,Pt):at(16,t?jt:Kt);return ee(n),te(t),r}function qt(e){if(!He(20))return st();var n=Jt(e);return He(21),n}function Xt(){en(27)||un()}function Yt(e){var n=Se(),t=xe();173===e&&He(103);var r=Bt(),i=qt(4),a=Wt(58,!0);Xt();var o=172===e?F.createCallSignature(r,i,a):F.createConstructSignature(r,i,a);return z(pn(o,n),t)}function Qt(){return 22===Le()&&Ue(Zt)}function Zt(){if(Ce(),25===Le()||23===Le())return!0;if(e.isModifierKind(Le())){if(Ce(),Ke())return!0}else{if(!Ke())return!1;Ce()}return 58===Le()||27===Le()||57===Le()&&(Ce(),58===Le()||27===Le()||23===Le())}function $t(e,n,t,r){var i=lt(16,Kt,22,23),a=Wr();Xt();var o=F.createIndexSignature(t,r,i,a);return z(pn(o,e),n)}function er(e,n,t){var r,i=En(),a=nn(57);if(20===Le()||29===Le()){var o=Bt(),s=qt(4),c=Wt(58,!0);r=F.createMethodSignature(t,i,a,o,s,c)}else{c=Wr();r=F.createPropertySignature(t,i,a,c),63===Le()&&(r.initializer=Yr())}return Xt(),z(pn(r,e),n)}function nr(){if(20===Le()||29===Le()||135===Le()||147===Le())return!0;var n=!1;while(e.isModifierKind(Le()))n=!0,Ce();return 22===Le()||(vn()&&(n=!0,Ce()),!!n&&(20===Le()||29===Le()||57===Le()||58===Le()||27===Le()||cn()))}function tr(){if(20===Le()||29===Le())return Yt(172);if(103===Le()&&Ue(rr))return Yt(173);var e=Se(),n=xe(),t=bo();return Ln(135)?po(e,n,void 0,t,170):Ln(147)?po(e,n,void 0,t,171):Qt()?$t(e,n,void 0,t):er(e,n,t)}function rr(){return Ce(),20===Le()||29===Le()}function ir(){return 24===Ce()}function ar(){switch(Ce()){case 20:case 29:case 24:return!0}return!1}function or(){var e=Se();return pn(F.createTypeLiteralNode(sr()),e)}function sr(){var e;return He(18)?(e=Hn(4,tr),He(19)):e=st(),e}function cr(){return Ce(),39===Le()||40===Le()?143===Ce():(143===Le()&&Ce(),22===Le()&&Pn()&&101===Ce())}function lr(){var e=Se(),n=hn();He(101);var t=Kr();return pn(F.createTypeParameterDeclaration(n,t,void 0),e)}function ur(){var e,n=Se();He(18),143!==Le()&&39!==Le()&&40!==Le()||(e=on(),143!==e.kind&&He(143)),He(22);var t,r=lr(),i=en(127)?Kr():void 0;He(23),57!==Le()&&39!==Le()&&40!==Le()||(t=on(),57!==t.kind&&He(57));var a=Wr();return un(),He(19),pn(F.createMappedTypeNode(e,r,i,t,a),n)}function dr(){var n=Se();if(en(25))return pn(F.createRestTypeNode(Kr()),n);var t=Kr();if(e.isJSDocNullableType(t)&&t.pos===t.type.pos){var r=F.createOptionalTypeNode(t.type);return e.setTextRange(r,t),r.flags=t.flags,r}return t}function pr(){return 58===Ce()||57===Le()&&58===Ce()}function fr(){return 25===Le()?e.tokenIsIdentifierOrKeyword(Ce())&&pr():e.tokenIsIdentifierOrKeyword(Le())&&pr()}function mr(){if(Ue(fr)){var e=Se(),n=xe(),t=nn(25),r=hn(),i=nn(57);He(58);var a=dr(),o=F.createNamedTupleMember(t,r,i,a);return z(pn(o,e),n)}return dr()}function _r(){var e=Se();return pn(F.createTupleTypeNode(lt(21,mr,22,23)),e)}function gr(){var e=Se();He(20);var n=Kr();return He(21),pn(F.createParenthesizedType(n),e)}function yr(){var e;if(126===Le()){var n=Se();Ce();var t=pn(F.createToken(126),n);e=dn([t],n)}return e}function hr(){var e=Se(),n=xe(),t=yr(),r=en(103),i=Bt(),a=qt(4),o=Wt(38,!1),s=r?F.createConstructorTypeNode(t,i,a,o):F.createFunctionTypeNode(i,a,o);return r||(s.modifiers=t),z(pn(s,e),n)}function vr(){var e=on();return 24===Le()?void 0:e}function br(e){var n=Se();e&&Ce();var t=110===Le()||95===Le()||104===Le()?on():xt(Le());return e&&(t=pn(F.createPrefixUnaryExpression(40,t),n)),pn(F.createLiteralTypeNode(t),n)}function Er(){return Ce(),100===Le()}function Tr(){_|=1048576;var e=Se(),n=en(112);He(100),He(20);var t=Kr();He(21);var r=en(24)?Lt():void 0,i=Dt();return pn(F.createImportTypeNode(t,r,i,n),e)}function Sr(){return Ce(),8===Le()||9===Le()}function xr(){switch(Le()){case 129:case 153:case 148:case 145:case 156:case 149:case 132:case 151:case 142:case 146:return Ve(vr)||At();case 66:c.reScanAsteriskEqualsToken();case 41:return kt();case 60:c.reScanQuestionToken();case 57:return Ot();case 98:return wt();case 53:return Rt();case 14:case 10:case 8:case 9:case 110:case 95:case 104:return br();case 40:return Ue(Sr)?br(!0):At();case 114:return on();case 108:var e=Nt();return 138!==Le()||c.hasPrecedingLineBreak()?e:It(e);case 112:return Ue(Er)?Tr():Ft();case 18:return Ue(cr)?ur():or();case 22:return _r();case 20:return gr();case 100:return Tr();case 128:return Ue(Ra)?jr():At();case 15:return _t();default:return At()}}function Lr(e){switch(Le()){case 129:case 153:case 148:case 145:case 156:case 132:case 143:case 149:case 152:case 114:case 151:case 104:case 108:case 112:case 142:case 18:case 22:case 29:case 51:case 50:case 103:case 10:case 8:case 9:case 110:case 95:case 146:case 41:case 57:case 53:case 25:case 136:case 100:case 128:case 14:case 15:return!0;case 98:return!e;case 40:return!e&&Ue(Sr);case 20:return!e&&Ue(Dr);default:return Ke()}}function Dr(){return Ce(),21===Le()||Ut(!1)||Lr()}function Ar(){var e=Se(),n=xr();while(!c.hasPrecedingLineBreak())switch(Le()){case 53:Ce(),n=pn(F.createJSDocNonNullableType(n),e);break;case 57:if(Ue(Un))return n;Ce(),n=pn(F.createJSDocNullableType(n),e);break;case 22:if(He(22),Lr()){var t=Kr();He(23),n=pn(F.createIndexedAccessTypeNode(n,t),e)}else He(23),n=pn(F.createArrayTypeNode(n),e);break;default:return n}return n}function Cr(e){var n=Se();return He(e),pn(F.createTypeOperatorNode(e,kr()),n)}function Ir(){var e=Se();return pn(F.createTypeParameterDeclaration(yn(),void 0,void 0),e)}function Nr(){var e=Se();return He(136),pn(F.createInferTypeNode(Ir()),e)}function kr(){var e=Le();switch(e){case 139:case 152:case 143:return Cr(e);case 136:return Nr()}return Ar()}function Rr(n){if(Fr()){var t=hr(),r=void 0;return r=e.isFunctionTypeNode(t)?n?e.Diagnostics.Function_type_notation_must_be_parenthesized_when_used_in_a_union_type:e.Diagnostics.Function_type_notation_must_be_parenthesized_when_used_in_an_intersection_type:n?e.Diagnostics.Constructor_type_notation_must_be_parenthesized_when_used_in_a_union_type:e.Diagnostics.Constructor_type_notation_must_be_parenthesized_when_used_in_an_intersection_type,Ee(t,r),t}}function Or(e,n,t){var r=Se(),i=51===e,a=en(e),o=a&&Rr(i)||n();if(Le()===e||a){var s=[o];while(en(e))s.push(Rr(i)||n());o=pn(t(dn(s,r)),r)}return o}function wr(){return Or(50,kr,F.createIntersectionTypeNode)}function Pr(){return Or(51,wr,F.createUnionTypeNode)}function Mr(){return Ce(),103===Le()}function Fr(){return 29===Le()||(!(20!==Le()||!Ue(Br))||(103===Le()||126===Le()&&Ue(Mr)))}function Gr(){if(e.isModifierKind(Le())&&bo(),Ke()||108===Le())return Ce(),!0;if(22===Le()||18===Le()){var n=E.length;return $a(),n===E.length}return!1}function Br(){if(Ce(),21===Le()||25===Le())return!0;if(Gr()){if(58===Le()||27===Le()||57===Le()||63===Le())return!0;if(21===Le()&&(Ce(),38===Le()))return!0}return!1}function Ur(){var e=Se(),n=Ke()&&Ve(Vr),t=Kr();return n?pn(F.createTypePredicateNode(void 0,n,t),e):t}function Vr(){var e=yn();if(138===Le()&&!c.hasPrecedingLineBreak())return Ce(),e}function jr(){var e=Se(),n=rn(128),t=108===Le()?Nt():yn(),r=en(138)?Kr():void 0;return pn(F.createTypePredicateNode(n,t,r),e)}function Kr(){return re(40960,Hr)}function Hr(e){if(Fr())return hr();var n=Se(),t=Pr();if(!e&&!c.hasPrecedingLineBreak()&&en(94)){var r=Hr(!0);He(57);var i=Hr();He(58);var a=Hr();return pn(F.createConditionalTypeNode(t,r,i,a),n)}return t}function Wr(){return en(58)?Kr():void 0}function zr(){switch(Le()){case 108:case 106:case 104:case 110:case 95:case 8:case 9:case 10:case 14:case 15:case 20:case 22:case 18:case 98:case 84:case 103:case 43:case 68:case 79:return!0;case 100:return Ue(ar);default:return Ke()}}function Jr(){if(zr())return!0;switch(Le()){case 39:case 40:case 54:case 53:case 89:case 112:case 114:case 45:case 46:case 29:case 131:case 125:case 80:return!0;default:return!!mi()||Ke()}}function qr(){return 18!==Le()&&98!==Le()&&84!==Le()&&59!==Le()&&Jr()}function Xr(){var e=ge();e&&ne(!1);var n,t=Se(),r=Qr();while(n=nn(27))r=_i(r,n,Qr(),t);return e&&ne(!0),r}function Yr(){return en(63)?Qr():void 0}function Qr(){if(Zr())return ei();var n=ti()||oi();if(n)return n;var t=Se(),r=di(0);return 79===r.kind&&38===Le()?ni(t,r,void 0):e.isLeftHandSideExpression(r)&&e.isAssignmentOperator(Ne())?_i(r,on(),Qr(),t):ui(r,t)}function Zr(){return 125===Le()&&(!!me()||Ue(Pa))}function $r(){return Ce(),!c.hasPrecedingLineBreak()&&Ke()}function ei(){var e=Se();return Ce(),c.hasPrecedingLineBreak()||41!==Le()&&!Jr()?pn(F.createYieldExpression(void 0,void 0),e):pn(F.createYieldExpression(nn(41),Qr()),e)}function ni(n,t,r){e.Debug.assert(38===Le(),"parseSimpleArrowFunctionExpression should only have been called if we had a =>");var i=F.createParameterDeclaration(void 0,void 0,void 0,t,void 0,void 0,void 0);pn(i,t.pos);var a=dn([i],i.pos,i.end),o=rn(38),s=li(!!r),c=F.createArrowFunction(r,void 0,a,void 0,o,s);return q(pn(c,n))}function ti(){var e=ri();if(0!==e)return 1===e?ci(!0):Ve(ai)}function ri(){return 20===Le()||29===Le()||130===Le()?Ue(ii):38===Le()?1:0}function ii(){if(130===Le()){if(Ce(),c.hasPrecedingLineBreak())return 0;if(20!==Le()&&29!==Le())return 0}var n=Le(),t=Ce();if(20===n){if(21===t){var r=Ce();switch(r){case 38:case 58:case 18:return 1;default:return 0}}if(22===t||18===t)return 2;if(25===t)return 1;if(e.isModifierKind(t)&&130!==t&&Ue(Pn))return 1;if(!Ke()&&108!==t)return 0;switch(Ce()){case 58:return 1;case 57:return Ce(),58===Le()||27===Le()||63===Le()||21===Le()?1:0;case 27:case 63:case 21:return 2}return 0}if(e.Debug.assert(29===n),!Ke())return 0;if(1===v){var i=Ue(function(){var e=Ce();if(94===e){var n=Ce();switch(n){case 63:case 31:return!1;default:return!0}}else if(27===e)return!0;return!1});return i?1:0}return 2}function ai(){var n=c.getTokenPos();if(null===O||void 0===O||!O.has(n)){var t=ci(!1);return t||(O||(O=new e.Set)).add(n),t}}function oi(){if(130===Le()&&1===Ue(si)){var e=Se(),n=Eo(),t=di(0);return ni(e,t,n)}}function si(){if(130===Le()){if(Ce(),c.hasPrecedingLineBreak()||38===Le())return 0;var e=di(0);if(!c.hasPrecedingLineBreak()&&79===e.kind&&38===Le())return 1}return 0}function ci(n){var t,r=Se(),i=xe(),a=Eo(),o=e.some(a,e.isAsyncModifier)?2:0,s=Bt();if(He(20)){if(t=Jt(o),!He(21)&&!n)return}else{if(!n)return;t=st()}var c=Wt(58,!1);if(!c||n||!Ct(c)){var l=c&&e.isJSDocFunctionType(c);if(n||38===Le()||!l&&18===Le()){var u=Le(),d=rn(38),p=38===u||18===u?li(e.some(a,e.isAsyncModifier)):yn(),f=F.createArrowFunction(a,s,t,c,d,p);return z(pn(f,r),i)}}}function li(e){if(18===Le())return fa(e?2:0);if(26!==Le()&&98!==Le()&&84!==Le()&&Ga()&&!qr())return fa(16|(e?2:0));var n=G;G=!1;var t=e?le(Qr):ue(Qr);return G=n,t}function ui(n,t){var r,i=nn(57);return i?pn(F.createConditionalExpression(n,i,re(l,Qr),r=rn(58),e.nodeIsPresent(r)?Qr():fn(79,!1,e.Diagnostics._0_expected,e.tokenToString(58))),t):n}function di(e){var n=Se(),t=Si();return fi(e,t,n)}function pi(e){return 101===e||158===e}function fi(n,t,r){while(1){Ne();var i=e.getBinaryOperatorPrecedence(Le()),a=42===Le()?i>=n:i>n;if(!a)break;if(101===Le()&&_e())break;if(127===Le()){if(c.hasPrecedingLineBreak())break;Ce(),t=gi(t,Kr())}else t=_i(t,on(),di(i),r)}return t}function mi(){return(!_e()||101!==Le())&&e.getBinaryOperatorPrecedence(Le())>0}function _i(e,n,t,r){return pn(F.createBinaryExpression(e,n,t),r)}function gi(e,n){return pn(F.createAsExpression(e,n),e.pos)}function yi(){var e=Se();return pn(F.createPrefixUnaryExpression(Le(),Ae(xi)),e)}function hi(){var e=Se();return pn(F.createDeleteExpression(Ae(xi)),e)}function vi(){var e=Se();return pn(F.createTypeOfExpression(Ae(xi)),e)}function bi(){var e=Se();return pn(F.createVoidExpression(Ae(xi)),e)}function Ei(){return 131===Le()&&(!!ye()||Ue(Pa))}function Ti(){var e=Se();return pn(F.createAwaitExpression(Ae(xi)),e)}function Si(){if(Li()){var n=Se(),t=Di();return 42===Le()?fi(e.getBinaryOperatorPrecedence(Le()),t,n):t}var r=Le(),i=xi();if(42===Le()){n=e.skipTrivia(g,i.pos);var a=i.end;209===i.kind?be(n,a,e.Diagnostics.A_type_assertion_expression_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses):be(n,a,e.Diagnostics.An_unary_expression_with_the_0_operator_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses,e.tokenToString(r))}return i}function xi(){switch(Le()){case 39:case 40:case 54:case 53:return yi();case 89:return hi();case 112:return vi();case 114:return bi();case 29:return ji();case 131:if(Ei())return Ti();default:return Di()}}function Li(){switch(Le()){case 39:case 40:case 54:case 53:case 89:case 112:case 114:case 131:return!1;case 29:if(1!==v)return!1;default:return!0}}function Di(){if(45===Le()||46===Le()){var n=Se();return pn(F.createPrefixUnaryExpression(Le(),Ae(Ai)),n)}if(1===v&&29===Le()&&Ue(Fn))return Ni(!0);var t=Ai();if(e.Debug.assert(e.isLeftHandSideExpression(t)),(45===Le()||46===Le())&&!c.hasPrecedingLineBreak()){var r=Le();return Ce(),pn(F.createPostfixUnaryExpression(t,r),t.pos)}return t}function Ai(){var e,n=Se();return 100===Le()?Ue(rr)?(_|=1048576,e=on()):Ue(ir)?(Ce(),Ce(),e=pn(F.createMetaProperty(100,hn()),n),_|=2097152):e=Ci():e=106===Le()?Ii():Ci(),Qi(n,e)}function Ci(){var e=Se(),n=na();return qi(e,n,!0)}function Ii(){var n=Se(),t=on();if(29===Le()){var r=Se(),i=Ve($i);void 0!==i&&be(r,Se(),e.Diagnostics.super_may_not_use_type_arguments)}return 20===Le()||24===Le()||22===Le()?t:(rn(24,e.Diagnostics.super_must_be_followed_by_an_argument_list_or_member_access),pn(F.createPropertyAccessExpression(t,pt(!0,!0)),n))}function Ni(n,t,r){var i,o=Se(),s=Pi(n);if(278===s.kind){var c=Oi(s),l=void 0,u=c[c.length-1];if(276===(null===u||void 0===u?void 0:u.kind)&&!w(u.openingElement.tagName,u.closingElement.tagName)&&w(s.tagName,u.closingElement.tagName)){var d=u.openingElement.end,p=pn(F.createJsxElement(u.openingElement,dn([],d,d),pn(F.createJsxClosingElement(pn(F.createIdentifier(""),d,d)),d,d)),u.openingElement.pos,d);c=dn(a(a([],c.slice(0,c.length-1),!0),[p],!1),c.pos,d),l=u.closingElement}else l=Ui(s,n),w(s.tagName,l.tagName)||(r&&e.isJsxOpeningElement(r)&&w(l.tagName,r.tagName)?Ee(s.tagName,e.Diagnostics.JSX_element_0_has_no_corresponding_closing_tag,e.getTextOfNodeFromSourceText(g,s.tagName)):Ee(l.tagName,e.Diagnostics.Expected_corresponding_JSX_closing_tag_for_0,e.getTextOfNodeFromSourceText(g,s.tagName)));i=pn(F.createJsxElement(s,c,l),o)}else 281===s.kind?i=pn(F.createJsxFragment(s,Oi(s),Vi(n)),o):(e.Debug.assert(277===s.kind),i=s);if(n&&29===Le()){var f="undefined"===typeof t?i.pos:t,m=Ve(function(){return Ni(!0,f)});if(m){var _=fn(27,!1);return e.setTextRangePosWidth(_,m.pos,0),be(e.skipTrivia(g,f),m.end,e.Diagnostics.JSX_expressions_must_have_one_parent_element),pn(F.createBinaryExpression(i,_,m),o)}}return i}function ki(){var e=Se(),n=F.createJsxText(c.getTokenValue(),12===A);return A=c.scanJsxToken(),pn(n,e)}function Ri(n,t){switch(t){case 1:if(e.isJsxOpeningFragment(n))Ee(n,e.Diagnostics.JSX_fragment_has_no_corresponding_closing_tag);else{var r=n.tagName,i=e.skipTrivia(g,r.pos);be(i,r.end,e.Diagnostics.JSX_element_0_has_no_corresponding_closing_tag,e.getTextOfNodeFromSourceText(g,n.tagName))}return;case 30:case 7:return;case 11:case 12:return ki();case 18:return Fi(!1);case 29:return Ni(!1,void 0,n);default:return e.Debug.assertNever(t)}}function Oi(n){var t=[],r=Se(),i=R;R|=16384;while(1){var a=Ri(n,A=c.reScanJsxToken());if(!a)break;if(t.push(a),e.isJsxOpeningElement(n)&&276===(null===a||void 0===a?void 0:a.kind)&&!w(a.openingElement.tagName,a.closingElement.tagName)&&w(n.tagName,a.closingElement.tagName))break}return R=i,dn(t,r)}function wi(){var e=Se();return pn(F.createJsxAttributes(Hn(13,Gi)),e)}function Pi(e){var n=Se();if(He(29),31===Le())return Fe(),pn(F.createJsxOpeningFragment(),n);var t,r=Mi(),i=0===(131072&P)?ko():void 0,a=wi();return 31===Le()?(Fe(),t=F.createJsxOpeningElement(r,i,a)):(He(43),He(31,void 0,!1)&&(e?Ce():Fe()),t=F.createJsxSelfClosingElement(r,i,a)),pn(t,n)}function Mi(){var e=Se();Me();var n=108===Le()?on():hn();while(en(24))n=pn(F.createPropertyAccessExpression(n,pt(!0,!1)),e);return n}function Fi(e){var n,t,r=Se();if(He(18))return 19!==Le()&&(n=nn(25),t=Xr()),e?He(19):He(19,void 0,!1)&&Fe(),pn(F.createJsxExpression(n,t),r)}function Gi(){if(18===Le())return Bi();Me();var e=Se();return pn(F.createJsxAttribute(hn(),63!==Le()?void 0:10===Ge()?bt():Fi(!0)),e)}function Bi(){var e=Se();He(18),He(25);var n=Xr();return He(19),pn(F.createJsxSpreadAttribute(n),e)}function Ui(e,n){var t=Se();He(30);var r=Mi();return He(31,void 0,!1)&&(n||!w(e.tagName,r)?Ce():Fe()),pn(F.createJsxClosingElement(r),t)}function Vi(n){var t=Se();return He(30),e.tokenIsIdentifierOrKeyword(Le())&&Ee(Mi(),e.Diagnostics.Expected_corresponding_closing_tag_for_JSX_fragment),He(31,void 0,!1)&&(n?Ce():Fe()),pn(F.createJsxJsxClosingFragment(),t)}function ji(){var e=Se();He(29);var n=Kr();He(31);var t=xi();return pn(F.createTypeAssertion(n,t),e)}function Ki(){return Ce(),e.tokenIsIdentifierOrKeyword(Le())||22===Le()||Xi()}function Hi(){return 28===Le()&&Ue(Ki)}function Wi(n){if(32&n.flags)return!0;if(e.isNonNullExpression(n)){var t=n.expression;while(e.isNonNullExpression(t)&&!(32&t.flags))t=t.expression;if(32&t.flags){while(e.isNonNullExpression(n))n.flags|=32,n=n.expression;return!0}}return!1}function zi(n,t,r){var i=pt(!0,!0),a=r||Wi(t),o=a?F.createPropertyAccessChain(t,r,i):F.createPropertyAccessExpression(t,i);return a&&e.isPrivateIdentifier(o.name)&&Ee(o.name,e.Diagnostics.An_optional_chain_cannot_contain_private_identifiers),pn(o,n)}function Ji(n,t,r){var i;if(23===Le())i=fn(79,!0,e.Diagnostics.An_element_access_expression_should_take_an_argument);else{var a=ae(Xr);e.isStringOrNumericLiteralLike(a)&&(a.text=mn(a.text)),i=a}He(23);var o=r||Wi(t)?F.createElementAccessChain(t,r,i):F.createElementAccessExpression(t,i);return pn(o,n)}function qi(n,t,r){while(1){var i=void 0,a=!1;if(r&&Hi()?(i=rn(28),a=e.tokenIsIdentifierOrKeyword(Le())):a=en(24),a)t=zi(n,t,i);else if(i||53!==Le()||c.hasPrecedingLineBreak())if(!i&&ge()||!en(22)){if(!Xi())return t;t=Yi(n,t,i,void 0)}else t=Ji(n,t,i);else Ce(),t=pn(F.createNonNullExpression(t),n)}}function Xi(){return 14===Le()||15===Le()}function Yi(e,n,t,r){var i=F.createTaggedTemplateExpression(n,r,14===Le()?(Oe(),bt()):mt(!0));return(t||32&n.flags)&&(i.flags|=32),i.questionDotToken=t,pn(i,e)}function Qi(n,t){while(1){t=qi(n,t,!0);var r=nn(28);if(0!==(131072&P)||29!==Le()&&47!==Le()){if(20===Le()){a=Zi(),o=r||Wi(t)?F.createCallChain(t,r,void 0,a):F.createCallExpression(t,void 0,a);t=pn(o,n);continue}}else{var i=Ve($i);if(i){if(Xi()){t=Yi(n,t,r,i);continue}var a=Zi(),o=r||Wi(t)?F.createCallChain(t,r,i,a):F.createCallExpression(t,i,a);t=pn(o,n);continue}}if(r){var s=fn(79,!1,e.Diagnostics.Identifier_expected);t=pn(F.createPropertyAccessChain(t,r,s),n)}break}return t}function Zi(){He(20);var e=at(11,aa);return He(21),e}function $i(){if(0===(131072&P)&&29===we()){Ce();var e=at(20,Kr);if(He(31))return e&&ea()?e:void 0}}function ea(){switch(Le()){case 20:case 14:case 15:case 24:case 21:case 23:case 58:case 26:case 57:case 34:case 36:case 35:case 37:case 55:case 56:case 60:case 52:case 50:case 51:case 19:case 1:return!0;case 27:case 18:default:return!1}}function na(){switch(Le()){case 8:case 9:case 10:case 14:return bt();case 108:case 106:case 104:case 110:case 95:return on();case 20:return ta();case 22:return oa();case 18:return ca();case 130:if(!Ue(wa))break;return la();case 84:return So();case 98:return la();case 103:return da();case 43:case 68:if(13===ke())return bt();break;case 15:return mt(!1)}return yn(e.Diagnostics.Expression_expected)}function ta(){var e=Se(),n=xe();He(20);var t=ae(Xr);return He(21),z(pn(F.createParenthesizedExpression(t),e),n)}function ra(){var e=Se();He(25);var n=Qr();return pn(F.createSpreadElement(n),e)}function ia(){return 25===Le()?ra():27===Le()?pn(F.createOmittedExpression(),Se()):Qr()}function aa(){return re(l,ia)}function oa(){var e=Se();He(22);var n=c.hasPrecedingLineBreak(),t=at(15,ia);return He(23),pn(F.createArrayLiteralExpression(t,n),e)}function sa(){var e=Se(),n=xe();if(nn(25)){var t=Qr();return z(pn(F.createSpreadAssignment(t),e),n)}var r=ho(),i=bo();if(Ln(135))return po(e,n,r,i,170);if(Ln(147))return po(e,n,r,i,171);var a,o=nn(41),s=Ke(),c=En(),l=nn(57),u=nn(53);if(o||20===Le()||29===Le())return co(e,n,r,i,o,c,l,u);var d=s&&58!==Le();if(d){var p=nn(63),f=p?ae(Qr):void 0;a=F.createShorthandPropertyAssignment(c,f),a.equalsToken=p}else{He(58);var m=ae(Qr);a=F.createPropertyAssignment(c,m)}return a.decorators=r,a.modifiers=i,a.questionToken=l,a.exclamationToken=u,z(pn(a,e),n)}function ca(){var n=Se(),t=c.getTokenPos();He(18);var r=c.hasPrecedingLineBreak(),i=at(12,sa,!0);if(!He(19)){var a=e.lastOrUndefined(E);a&&a.code===e.Diagnostics._0_expected.code&&e.addRelatedInfo(a,e.createDetachedDiagnostic(p,t,1,e.Diagnostics.The_parser_expected_to_find_a_to_match_the_token_here))}return pn(F.createObjectLiteralExpression(i,r),n)}function la(){var n=ge();ne(!1);var t=Se(),r=xe(),i=bo();He(98);var a=nn(41),o=a?1:0,s=e.some(i,e.isAsyncModifier)?2:0,c=o&&s?de(ua):o?se(ua):s?le(ua):ua(),l=Bt(),u=qt(o|s),d=Wt(58,!1),p=fa(o|s);ne(n);var f=F.createFunctionExpression(i,a,c,l,u,d,p);return z(pn(f,t),r)}function ua(){return je()?gn():void 0}function da(){var n=Se();if(He(103),en(24)){var t=hn();return pn(F.createMetaProperty(103,t),n)}var r,i,a=Se(),o=na();while(1){o=qi(a,o,!1),r=Ve($i),Xi()&&(e.Debug.assert(!!r,"Expected a type argument list; all plain tagged template starts should be consumed in 'parseMemberExpressionRest'"),o=Yi(a,o,void 0,r),r=void 0);break}return 20===Le()?i=Zi():r&&be(n,c.getStartPos(),e.Diagnostics.A_new_expression_with_type_arguments_must_always_be_followed_by_a_parenthesized_argument_list),pn(F.createNewExpression(o,r,i),n)}function pa(n,t){var r=Se(),i=xe(),a=c.getTokenPos();if(He(18,t)||n){var o=c.hasPrecedingLineBreak(),s=Hn(1,Va);if(!He(19)){var l=e.lastOrUndefined(E);l&&l.code===e.Diagnostics._0_expected.code&&e.addRelatedInfo(l,e.createDetachedDiagnostic(p,a,1,e.Diagnostics.The_parser_expected_to_find_a_to_match_the_token_here))}var u=z(pn(F.createBlock(s,o),r),i);return 63===Le()&&(he(e.Diagnostics.Declaration_or_statement_expected_This_follows_a_block_of_statements_so_if_you_intended_to_write_a_destructuring_assignment_you_might_need_to_wrap_the_the_whole_assignment_in_parentheses),Ce()),u}s=st();return z(pn(F.createBlock(s,void 0),r),i)}function fa(e,n){var t=me();ee(!!(1&e));var r=ye();te(!!(2&e));var i=G;G=!1;var a=ge();a&&ne(!1);var o=pa(!!(16&e),n);return a&&ne(!0),G=i,ee(t),te(r),o}function ma(){var e=Se(),n=xe();return He(26),z(pn(F.createEmptyStatement(),e),n)}function _a(){var e=Se(),n=xe();He(99),He(20);var t=ae(Xr);He(21);var r=Va(),i=en(91)?Va():void 0;return z(pn(F.createIfStatement(t,r,i),e),n)}function ga(){var e=Se(),n=xe();He(90);var t=Va();He(115),He(20);var r=ae(Xr);return He(21),en(26),z(pn(F.createDoStatement(t,r),e),n)}function ya(){var e=Se(),n=xe();He(115),He(20);var t=ae(Xr);He(21);var r=Va();return z(pn(F.createWhileStatement(t,r),e),n)}function ha(){var e=Se(),n=xe();He(97);var t,r,i=nn(131);if(He(20),26!==Le()&&(t=113===Le()||119===Le()||85===Le()?to(!0):oe(Xr)),i?He(158):en(158)){var a=ae(Qr);He(21),r=F.createForOfStatement(i,t,a,Va())}else if(en(101)){a=ae(Xr);He(21),r=F.createForInStatement(t,a,Va())}else{He(26);var o=26!==Le()&&21!==Le()?ae(Xr):void 0;He(26);var s=21!==Le()?ae(Xr):void 0;He(21),r=F.createForStatement(t,o,s,Va())}return z(pn(r,e),n)}function va(e){var n=Se(),t=xe();He(244===e?81:86);var r=cn()?void 0:yn();un();var i=244===e?F.createBreakStatement(r):F.createContinueStatement(r);return z(pn(i,n),t)}function ba(){var e=Se(),n=xe();He(105);var t=cn()?void 0:ae(Xr);return un(),z(pn(F.createReturnStatement(t),e),n)}function Ea(){var e=Se(),n=xe();He(116),He(20);var t=ae(Xr);He(21);var r=ie(16777216,Va);return z(pn(F.createWithStatement(t,r),e),n)}function Ta(){var e=Se();He(82);var n=ae(Xr);He(58);var t=Hn(3,Va);return pn(F.createCaseClause(n,t),e)}function Sa(){var e=Se();He(88),He(58);var n=Hn(3,Va);return pn(F.createDefaultClause(n),e)}function xa(){return 82===Le()?Ta():Sa()}function La(){var e=Se();He(18);var n=Hn(2,xa);return He(19),pn(F.createCaseBlock(n),e)}function Da(){var e=Se(),n=xe();He(107),He(20);var t=ae(Xr);He(21);var r=La();return z(pn(F.createSwitchStatement(t,r),e),n)}function Aa(){var e=Se(),n=xe();He(109);var t=c.hasPrecedingLineBreak()?void 0:ae(Xr);return void 0===t&&(k++,t=pn(F.createIdentifier(""),Se())),ln()||Xe(t),z(pn(F.createThrowStatement(t),e),n)}function Ca(){var e=Se(),n=xe();He(111);var t,r=pa(!1),i=83===Le()?Ia():void 0;return i&&96!==Le()||(He(96),t=pa(!1)),z(pn(F.createTryStatement(r,i,t),e),n)}function Ia(){var e,n=Se();He(83),en(20)?(e=no(),He(21)):e=void 0;var t=pa(!1);return pn(F.createCatchClause(e,t),n)}function Na(){var e=Se(),n=xe();return He(87),un(),z(pn(F.createDebuggerStatement(),e),n)}function ka(){var n,t=Se(),r=xe(),i=20===Le(),a=ae(Xr);return e.isIdentifier(a)&&en(58)?n=F.createLabeledStatement(a,Va()):(ln()||Xe(a),n=F.createExpressionStatement(a),i&&(r=!1)),z(pn(n,t),r)}function Ra(){return Ce(),e.tokenIsIdentifierOrKeyword(Le())&&!c.hasPrecedingLineBreak()}function Oa(){return Ce(),84===Le()&&!c.hasPrecedingLineBreak()}function wa(){return Ce(),98===Le()&&!c.hasPrecedingLineBreak()}function Pa(){return Ce(),(e.tokenIsIdentifierOrKeyword(Le())||8===Le()||9===Le()||10===Le())&&!c.hasPrecedingLineBreak()}function Ma(){while(1)switch(Le()){case 113:case 119:case 85:case 98:case 84:case 92:return!0;case 118:case 150:return $r();case 140:case 141:return za();case 126:case 130:case 134:case 121:case 122:case 123:case 143:if(Ce(),c.hasPrecedingLineBreak())return!1;continue;case 155:return Ce(),18===Le()||79===Le()||93===Le();case 100:return Ce(),10===Le()||41===Le()||18===Le()||e.tokenIsIdentifierOrKeyword(Le());case 93:var n=Ce();if(150===n&&(n=Ue(Ce)),63===n||41===n||18===n||88===n||127===n)return!0;continue;case 124:Ce();continue;default:return!1}}function Fa(){return Ue(Ma)}function Ga(){switch(Le()){case 59:case 26:case 18:case 113:case 119:case 98:case 84:case 92:case 99:case 90:case 115:case 97:case 86:case 81:case 105:case 116:case 107:case 109:case 111:case 87:case 83:case 96:return!0;case 100:return Fa()||Ue(ar);case 85:case 93:return Fa();case 130:case 134:case 118:case 140:case 141:case 150:case 155:return!0;case 123:case 121:case 122:case 124:case 143:return Fa()||!Ue(Ra);default:return Jr()}}function Ba(){return Ce(),je()||18===Le()||22===Le()}function Ua(){return Ue(Ba)}function Va(){switch(Le()){case 26:return ma();case 18:return pa(!1);case 113:return io(Se(),xe(),void 0,void 0);case 119:if(Ua())return io(Se(),xe(),void 0,void 0);break;case 98:return ao(Se(),xe(),void 0,void 0);case 84:return xo(Se(),xe(),void 0,void 0);case 99:return _a();case 90:return ga();case 115:return ya();case 97:return ha();case 86:return va(243);case 81:return va(244);case 105:return ba();case 116:return Ea();case 107:return Da();case 109:return Aa();case 111:case 83:case 96:return Ca();case 87:return Na();case 59:return Ka();case 130:case 118:case 150:case 140:case 141:case 134:case 85:case 92:case 93:case 100:case 121:case 122:case 123:case 126:case 124:case 143:case 155:if(Fa())return Ka();break}return ka()}function ja(e){return 134===e.kind}function Ka(){var n=e.some(Ue(function(){return ho(),bo()}),ja);if(n){var t=Ha();if(t)return t}var r=Se(),i=xe(),a=ho(),o=bo();if(n){for(var s=0,c=o;s<c.length;s++){var l=c[s];l.flags|=8388608}return ie(8388608,function(){return Wa(r,i,a,o)})}return Wa(r,i,a,o)}function Ha(){return ie(8388608,function(){var e=zn(R);if(e)return Jn(e)})}function Wa(n,t,r,i){switch(Le()){case 113:case 119:case 85:return io(n,t,r,i);case 98:return ao(n,t,r,i);case 84:return xo(n,t,r,i);case 118:return wo(n,t,r,i);case 150:return Po(n,t,r,i);case 92:return Fo(n,t,r,i);case 155:case 140:case 141:return Vo(n,t,r,i);case 100:return Jo(n,t,r,i);case 93:switch(Ce(),Le()){case 88:case 63:return cs(n,t,r,i);case 127:return zo(n,t,r,i);default:return ss(n,t,r,i)}default:if(r||i){var a=fn(274,!0,e.Diagnostics.Declaration_expected);return e.setTextRangePos(a,n),a.decorators=r,a.modifiers=i,a}return}}function za(){return Ce(),!c.hasPrecedingLineBreak()&&(Ke()||10===Le())}function Ja(e,n){if(18===Le()||!cn())return fa(e,n);un()}function qa(){var e=Se();if(27===Le())return pn(F.createOmittedExpression(),e);var n=nn(25),t=$a(),r=Yr();return pn(F.createBindingElement(n,void 0,t,r),e)}function Xa(){var e,n=Se(),t=nn(25),r=je(),i=En();r&&58!==Le()?(e=i,i=void 0):(He(58),e=$a());var a=Yr();return pn(F.createBindingElement(t,i,e,a),n)}function Ya(){var e=Se();He(18);var n=at(9,Xa);return He(19),pn(F.createObjectBindingPattern(n),e)}function Qa(){var e=Se();He(22);var n=at(10,qa);return He(23),pn(F.createArrayBindingPattern(n),e)}function Za(){return 18===Le()||22===Le()||80===Le()||je()}function $a(e){return 22===Le()?Qa():18===Le()?Ya():gn(e)}function eo(){return no(!0)}function no(n){var t,r=Se(),i=xe(),a=$a(e.Diagnostics.Private_identifiers_are_not_allowed_in_variable_declarations);n&&79===a.kind&&53===Le()&&!c.hasPrecedingLineBreak()&&(t=on());var o=Wr(),s=pi(Le())?void 0:Yr(),l=F.createVariableDeclaration(a,t,o,s);return z(pn(l,r),i)}function to(n){var t,r=Se(),i=0;switch(Le()){case 113:break;case 119:i|=1;break;case 85:i|=2;break;default:e.Debug.fail()}if(Ce(),158===Le()&&Ue(ro))t=st();else{var a=_e();$(n),t=at(8,n?no:eo),$(a)}return pn(F.createVariableDeclarationList(t,i),r)}function ro(){return Pn()&&21===Ce()}function io(e,n,t,r){var i=to(!1);un();var a=F.createVariableStatement(r,i);return a.decorators=t,z(pn(a,e),n)}function ao(n,t,r,i){var a=ye(),o=e.modifiersToFlags(i);He(98);var s=nn(41),c=512&o?ua():gn(),l=s?1:0,u=256&o?2:0,d=Bt();1&o&&te(!0);var p=qt(l|u),f=Wt(58,!1),m=Ja(l|u,e.Diagnostics.or_expected);te(a);var _=F.createFunctionDeclaration(r,i,s,c,d,p,f,m);return z(pn(_,n),t)}function oo(){return 133===Le()?He(133):10===Le()&&20===Ue(Ce)?Ve(function(){var e=bt();return"constructor"===e.text?e:void 0}):void 0}function so(n,t,r,i){return Ve(function(){if(oo()){var a=Bt(),o=qt(0),s=Wt(58,!1),c=Ja(0,e.Diagnostics.or_expected),l=F.createConstructorDeclaration(r,i,o,c);return l.typeParameters=a,l.type=s,z(pn(l,n),t)}})}function co(n,t,r,i,a,o,s,c,l){var u=a?1:0,d=e.some(i,e.isAsyncModifier)?2:0,p=Bt(),f=qt(u|d),m=Wt(58,!1),_=Ja(u|d,l),g=F.createMethodDeclaration(r,i,a,o,s,p,f,m,_);return g.exclamationToken=c,z(pn(g,n),t)}function lo(e,n,t,r,i,a){var o=a||c.hasPrecedingLineBreak()?void 0:nn(53),s=Wr(),l=re(45056,Yr);Ze(i,s,l);var u=F.createPropertyDeclaration(t,r,i,a||o,s,l);return z(pn(u,e),n)}function uo(n,t,r,i){var a=nn(41),o=En(),s=nn(57);return a||20===Le()||29===Le()?co(n,t,r,i,a,o,s,void 0,e.Diagnostics.or_expected):lo(n,t,r,i,o,s)}function po(e,n,t,r,i){var a=En(),o=Bt(),s=qt(0),c=Wt(58,!1),l=Ja(0),u=170===i?F.createGetAccessorDeclaration(t,r,a,s,c,l):F.createSetAccessorDeclaration(t,r,a,s,l);return u.typeParameters=o,c&&171===u.kind&&(u.type=c),z(pn(u,e),n)}function fo(){var n;if(59===Le())return!0;while(e.isModifierKind(Le())){if(n=Le(),e.isClassMemberModifier(n))return!0;Ce()}if(41===Le())return!0;if(vn()&&(n=Le(),Ce()),22===Le())return!0;if(void 0!==n){if(!e.isKeyword(n)||147===n||135===n)return!0;switch(Le()){case 20:case 29:case 53:case 58:case 63:case 57:return!0;default:return cn()}}return!1}function mo(e,n,t,r){rn(124);var i=_o();return z(pn(F.createClassStaticBlockDeclaration(t,r,i),e),n)}function _o(){var e=me(),n=ye();ee(!1),te(!0);var t=pa(!1);return ee(e),te(n),t}function go(){if(ye()&&131===Le()){var n=Se(),t=yn(e.Diagnostics.Expression_expected);Ce();var r=qi(n,t,!0);return Qi(n,r)}return Ai()}function yo(){var e=Se();if(en(59)){var n=ce(go);return pn(F.createDecorator(n),e)}}function ho(){var n,t,r=Se();while(t=yo())n=e.append(n,t);return n&&dn(n,r)}function vo(e,n){var t=Se(),r=Le();if(85===Le()&&e){if(!Ve(Dn))return}else{if(n&&124===Le()&&Ue(Ho))return;if(!Nn())return}return pn(F.createToken(r),t)}function bo(n,t){var r,i,a=Se();while(i=vo(n,t))r=e.append(r,i);return r&&dn(r,a)}function Eo(){var e;if(130===Le()){var n=Se();Ce();var t=pn(F.createToken(130),n);e=dn([t],n)}return e}function To(){var n=Se();if(26===Le())return Ce(),pn(F.createSemicolonClassElement(),n);var t=xe(),r=ho(),i=bo(!0,!0);if(124===Le()&&Ue(Ho))return mo(n,t,r,i);if(Ln(135))return po(n,t,r,i,170);if(Ln(147))return po(n,t,r,i,171);if(133===Le()||10===Le()){var a=so(n,t,r,i);if(a)return a}if(Qt())return $t(n,t,r,i);if(e.tokenIsIdentifierOrKeyword(Le())||10===Le()||8===Le()||41===Le()||22===Le()){var o=e.some(i,ja);if(o){for(var s=0,c=i;s<c.length;s++){var l=c[s];l.flags|=8388608}return ie(8388608,function(){return uo(n,t,r,i)})}return uo(n,t,r,i)}if(r||i){var u=fn(79,!0,e.Diagnostics.Declaration_expected);return lo(n,t,r,i,u,void 0)}return e.Debug.fail("Should not have attempted to parse class member declaration.")}function So(){return Lo(Se(),xe(),void 0,void 0,224)}function xo(e,n,t,r){return Lo(e,n,t,r,255)}function Lo(n,t,r,i,a){var o=ye();He(84);var s=Do(),c=Bt();e.some(i,e.isExportModifier)&&te(!0);var l,u=Co();He(18)?(l=Oo(),He(19)):l=st(),te(o);var d=255===a?F.createClassDeclaration(r,i,s,c,u,l):F.createClassExpression(r,i,s,c,u,l);return z(pn(d,n),t)}function Do(){return je()&&!Ao()?_n(je()):void 0}function Ao(){return 117===Le()&&Ue(Mn)}function Co(){if(Ro())return Hn(22,Io)}function Io(){var n=Se(),t=Le();e.Debug.assert(94===t||117===t),Ce();var r=at(7,No);return pn(F.createHeritageClause(t,r),n)}function No(){var e=Se(),n=Ai(),t=ko();return pn(F.createExpressionWithTypeArguments(n,t),e)}function ko(){return 29===Le()?lt(20,Kr,29,31):void 0}function Ro(){return 94===Le()||117===Le()}function Oo(){return Hn(5,To)}function wo(e,n,t,r){He(118);var i=yn(),a=Bt(),o=Co(),s=sr(),c=F.createInterfaceDeclaration(t,r,i,a,o,s);return z(pn(c,e),n)}function Po(e,n,t,r){He(150);var i=yn(),a=Bt();He(63);var o=137===Le()&&Ve(vr)||Kr();un();var s=F.createTypeAliasDeclaration(t,r,i,a,o);return z(pn(s,e),n)}function Mo(){var e=Se(),n=xe(),t=En(),r=ae(Yr);return z(pn(F.createEnumMember(t,r),e),n)}function Fo(e,n,t,r){He(92);var i,a=yn();He(18)?(i=pe(function(){return at(6,Mo)}),He(19)):i=st();var o=F.createEnumDeclaration(t,r,a,i);return z(pn(o,e),n)}function Go(){var e,n=Se();return He(18)?(e=Hn(1,Va),He(19)):e=st(),pn(F.createModuleBlock(e),n)}function Bo(e,n,t,r,i){var a=16&i,o=yn(),s=en(24)?Bo(Se(),!1,void 0,void 0,4|a):Go(),c=F.createModuleDeclaration(t,r,o,s,i);return z(pn(c,e),n)}function Uo(e,n,t,r){var i,a,o=0;155===Le()?(i=yn(),o|=1024):(i=bt(),i.text=mn(i.text)),18===Le()?a=Go():un();var s=F.createModuleDeclaration(t,r,i,a,o);return z(pn(s,e),n)}function Vo(e,n,t,r){var i=0;if(155===Le())return Uo(e,n,t,r);if(en(141))i|=16;else if(He(140),10===Le())return Uo(e,n,t,r);return Bo(e,n,t,r,i)}function jo(){return 144===Le()&&Ue(Ko)}function Ko(){return 20===Ce()}function Ho(){return 18===Ce()}function Wo(){return 43===Ce()}function zo(e,n,t,r){He(127),He(141);var i=yn();un();var a=F.createNamespaceExportDeclaration(i);return a.decorators=t,a.modifiers=r,z(pn(a,e),n)}function Jo(e,n,t,r){He(100);var i,a=c.getStartPos();Ke()&&(i=yn());var o,s=!1;if(154!==Le()&&"type"===(null===i||void 0===i?void 0:i.escapedText)&&(Ke()||qo())&&(s=!0,i=Ke()?yn():void 0),i&&!Xo())return Yo(e,n,t,r,i,s);(i||41===Le()||18===Le())&&(o=Qo(i,a,s),He(154));var l=es();un();var u=F.createImportDeclaration(t,r,o,l);return z(pn(u,e),n)}function qo(){return 41===Le()||18===Le()}function Xo(){return 27===Le()||154===Le()}function Yo(e,n,t,r,i,a){He(63);var o=Zo();un();var s=F.createImportEqualsDeclaration(t,r,a,i,o),c=z(pn(s,e),n);return c}function Qo(e,n,t){var r;return e&&!en(27)||(r=41===Le()?ns():ts(267)),pn(F.createImportClause(t,e,r),n)}function Zo(){return jo()?$o():ut(!1)}function $o(){var e=Se();He(144),He(20);var n=es();return He(21),pn(F.createExternalModuleReference(n),e)}function es(){if(10===Le()){var e=bt();return e.text=mn(e.text),e}return Xr()}function ns(){var e=Se();He(41),He(127);var n=yn();return pn(F.createNamespaceImport(n),e)}function ts(e){var n=Se(),t=267===e?F.createNamedImports(lt(23,is,18,19)):F.createNamedExports(lt(23,rs,18,19));return pn(t,n)}function rs(){return as(273)}function is(){return as(268)}function as(n){var t,r,i=Se(),a=e.isKeyword(Le())&&!Ke(),o=c.getTokenPos(),s=c.getTextPos(),l=hn();127===Le()?(t=l,He(127),a=e.isKeyword(Le())&&!Ke(),o=c.getTokenPos(),s=c.getTextPos(),r=hn()):r=l,268===n&&a&&be(o,s,e.Diagnostics.Identifier_expected);var u=268===n?F.createImportSpecifier(t,r):F.createExportSpecifier(t,r);return pn(u,i)}function os(e){return pn(F.createNamespaceExport(hn()),e)}function ss(e,n,t,r){var i,a,o=ye();te(!0);var s=en(150),l=Se();en(41)?(en(127)&&(i=os(l)),He(154),a=es()):(i=ts(271),(154===Le()||10===Le()&&!c.hasPrecedingLineBreak())&&(He(154),a=es())),un(),te(o);var u=F.createExportDeclaration(t,r,s,i,a);return z(pn(u,e),n)}function cs(e,n,t,r){var i,a=ye();te(!0),en(63)?i=!0:He(88);var o=Qr();un(),te(a);var s=F.createExportAssignment(t,r,i,o);return z(pn(s,e),n)}function ls(n){n.externalModuleIndicator=e.forEach(n.statements,us)||ds(n)}function us(n){return fs(n,93)||e.isImportEqualsDeclaration(n)&&e.isExternalModuleReference(n.moduleReference)||e.isImportDeclaration(n)||e.isExportAssignment(n)||e.isExportDeclaration(n)?n:void 0}function ds(e){return 2097152&e.flags?ps(e):void 0}function ps(e){return ms(e)?e:m(e,ps)}function fs(n,t){return e.some(n.modifiers,function(e){return e.kind===t})}function ms(n){return e.isMetaProperty(n)&&100===n.keywordToken&&"meta"===n.name.escapedText}(function(e){e[e["SourceElements"]=0]="SourceElements",e[e["BlockStatements"]=1]="BlockStatements",e[e["SwitchClauses"]=2]="SwitchClauses",e[e["SwitchClauseStatements"]=3]="SwitchClauseStatements",e[e["TypeMembers"]=4]="TypeMembers",e[e["ClassMembers"]=5]="ClassMembers",e[e["EnumMembers"]=6]="EnumMembers",e[e["HeritageClauseElement"]=7]="HeritageClauseElement",e[e["VariableDeclarations"]=8]="VariableDeclarations",e[e["ObjectBindingElements"]=9]="ObjectBindingElements",e[e["ArrayBindingElements"]=10]="ArrayBindingElements",e[e["ArgumentExpressions"]=11]="ArgumentExpressions",e[e["ObjectLiteralMembers"]=12]="ObjectLiteralMembers",e[e["JsxAttributes"]=13]="JsxAttributes",e[e["JsxChildren"]=14]="JsxChildren",e[e["ArrayLiteralMembers"]=15]="ArrayLiteralMembers",e[e["Parameters"]=16]="Parameters",e[e["JSDocParameters"]=17]="JSDocParameters",e[e["RestProperties"]=18]="RestProperties",e[e["TypeParameters"]=19]="TypeParameters",e[e["TypeArguments"]=20]="TypeArguments",e[e["TupleElementTypes"]=21]="TupleElementTypes",e[e["HeritageClauses"]=22]="HeritageClauses",e[e["ImportOrExportSpecifiers"]=23]="ImportOrExportSpecifiers",e[e["Count"]=24]="Count"})(We||(We={})),function(e){e[e["False"]=0]="False",e[e["True"]=1]="True",e[e["Unknown"]=2]="Unknown"}(ze||(ze={})),function(n){function t(n,t,i){K("file.js",n,99,void 0,1),c.setText(n,t,i),A=c.scan();var a=r(),o=Q("file.js",99,1,!1,[],F.createToken(1),0),s=e.attachFileToDiagnostics(E,o);return T&&(o.jsDocDiagnostics=e.attachFileToDiagnostics(T,o)),H(),a?{jsDocTypeExpression:a,diagnostics:s}:void 0}function r(e){var n=Se(),t=(e?en:He)(18),r=ie(4194304,Mt);e&&!t||$e(19);var i=F.createJSDocTypeExpression(r);return Y(i),pn(i,n)}function i(){var e=Se(),n=en(18),t=Se(),r=ut(!1);while(80===Le())Pe(),Ie(),r=pn(F.createJSDocMemberName(r,yn()),t);n&&$e(19);var i=F.createJSDocNameReference(r);return Y(i),pn(i,e)}function a(n,t,r){K("",n,99,void 0,1);var i=ie(4194304,function(){return u(t,r)}),a={languageVariant:0,text:n},o=e.attachFileToDiagnostics(E,a);return H(),i?{jsDoc:i,diagnostics:o}:void 0}function o(n,t,r){var i=A,a=E.length,o=B,s=ie(4194304,function(){return u(t,r)});return e.setParent(s,n),131072&P&&(T||(T=[]),T.push.apply(T,E)),A=i,E.length=a,B=o,s}var s,l;function u(n,t){void 0===n&&(n=0);var a=g,o=void 0===t?a.length:n+t;if(t=o-n,e.Debug.assert(n>=0),e.Debug.assert(n<=o),e.Debug.assert(o<=a.length),f(a,n)){var s,l,u,d,m,_=[],y=[];return c.scanRange(n+3,t-5,function(){var t,r=1,i=n-(a.lastIndexOf("\n",n)+1)+4;function p(e){t||(t=i),_.push(e),i+=e.length}Ie();while(le(5));le(4)&&(r=0,i=0);e:while(1){switch(Le()){case 59:0===r||1===r?(v(_),m||(m=Se()),R(x(i)),r=0,t=void 0):p(c.getTokenText());break;case 4:_.push(c.getTokenText()),r=0,i=0;break;case 41:var f=c.getTokenText();1===r||2===r?(r=2,p(f)):(r=1,i+=f.length);break;case 5:var g=c.getTokenText();2===r?_.push(g):void 0!==t&&i+g.length>t&&_.push(g.slice(t-i)),i+=g.length;break;case 1:break e;case 18:r=2;var b=c.getStartPos(),E=c.getTextPos()-1,T=C(E);if(T){d||h(_),y.push(pn(F.createJSDocText(_.join("")),null!==d&&void 0!==d?d:n,b)),y.push(T),_=[],d=c.getTextPos();break}default:r=2,p(c.getTokenText());break}Ie()}v(_),y.length&&_.length&&y.push(pn(F.createJSDocText(_.join("")),null!==d&&void 0!==d?d:n,m)),y.length&&s&&e.Debug.assertIsDefined(m,"having parsed tags implies that the end of the comment span should be set");var S=s&&dn(s,l,u);return pn(F.createJSDocComment(y.length?dn(y,n,m):_.length?_.join(""):void 0,S),n,o)})}function h(e){while(e.length&&("\n"===e[0]||"\r"===e[0]))e.shift()}function v(e){while(e.length&&""===e[e.length-1].trim())e.pop()}function b(){while(1){if(Ie(),1===Le())return!0;if(5!==Le()&&4!==Le())return!1}}function T(){if(5!==Le()&&4!==Le()||!Ue(b))while(5===Le()||4===Le())Ie()}function S(){if((5===Le()||4===Le())&&Ue(b))return"";var e=c.hasPrecedingLineBreak(),n=!1,t="";while(e&&41===Le()||5===Le()||4===Le())t+=c.getTokenText(),4===Le()?(e=!0,n=!0,t=""):41===Le()&&(e=!1),Ie();return n?t:""}function x(n){e.Debug.assert(59===Le());var t=c.getTokenPos();Ie();var r,i=de(void 0),a=S();switch(i.escapedText){case"author":r=j(t,i,n,a);break;case"implements":r=H(t,i,n,a);break;case"augments":case"extends":r=W(t,i,n,a);break;case"class":case"constructor":r=X(t,F.createJSDocClassTag,i,n,a);break;case"public":r=X(t,F.createJSDocPublicTag,i,n,a);break;case"private":r=X(t,F.createJSDocPrivateTag,i,n,a);break;case"protected":r=X(t,F.createJSDocProtectedTag,i,n,a);break;case"readonly":r=X(t,F.createJSDocReadonlyTag,i,n,a);break;case"override":r=X(t,F.createJSDocOverrideTag,i,n,a);break;case"deprecated":J=!0,r=X(t,F.createJSDocDeprecatedTag,i,n,a);break;case"this":r=Y(t,i,n,a);break;case"enum":r=Q(t,i,n,a);break;case"arg":case"argument":case"param":return M(t,i,2,n);case"return":case"returns":r=B(t,i,n,a);break;case"template":r=ce(t,i,n,a);break;case"type":r=U(t,i,n,a);break;case"typedef":r=Z(t,i,n,a);break;case"callback":r=ne(t,i,n,a);break;case"see":r=V(t,i,n,a);break;default:r=N(t,i,n,a);break}return r}function L(e,n,t,r){return r||(t+=n-e),D(t,r.slice(t))}function D(e,n){var t,r,i=Se(),a=[],o=[],s=0,l=!0;function u(n){r||(r=e),a.push(n),e+=n.length}void 0!==n&&(""!==n&&u(n),s=1);var d=Le();e:while(1){switch(d){case 4:s=0,a.push(c.getTokenText()),e=0;break;case 59:if(3===s||2===s&&(!l||Ue(A))){a.push(c.getTokenText());break}c.setTextPos(c.getTextPos()-1);case 1:break e;case 5:if(2===s||3===s)u(c.getTokenText());else{var p=c.getTokenText();void 0!==r&&e+p.length>r&&a.push(p.slice(r-e)),e+=p.length}break;case 18:s=2;var f=c.getStartPos(),m=c.getTextPos()-1,_=C(m);_?(o.push(pn(F.createJSDocText(a.join("")),null!==t&&void 0!==t?t:i,f)),o.push(_),a=[],t=c.getTextPos()):u(c.getTokenText());break;case 61:s=3===s?2:3,u(c.getTokenText());break;case 41:if(0===s){s=1,e+=1;break}default:3!==s&&(s=2),u(c.getTokenText());break}l=5===Le(),d=Ie()}return h(a),v(a),o.length?(a.length&&o.push(pn(F.createJSDocText(a.join("")),null!==t&&void 0!==t?t:i)),dn(o,i,c.getTextPos())):a.length?a.join(""):void 0}function A(){var e=Ie();return 5===e||4===e}function C(n){var t=Ve(I);if(t){Ie(),T();var r=Se(),i=e.tokenIsIdentifierOrKeyword(Le())?ut(!0):void 0;if(i)while(80===Le())Pe(),Ie(),i=pn(F.createJSDocMemberName(i,yn()),r);var a=[];while(19!==Le()&&4!==Le()&&1!==Le())a.push(c.getTokenText()),Ie();var o="link"===t?F.createJSDocLink:"linkcode"===t?F.createJSDocLinkCode:F.createJSDocLinkPlain;return pn(o(i,a.join("")),n,c.getTextPos())}}function I(){if(S(),18===Le()&&59===Ie()&&e.tokenIsIdentifierOrKeyword(Ie())){var n=c.getTokenValue();if("link"===n||"linkcode"===n||"linkplain"===n)return n}}function N(e,n,t,r){return pn(F.createJSDocUnknownTag(n,L(e,Se(),t,r)),e)}function R(e){e&&(s?s.push(e):(s=[e],l=e.pos),u=e.end)}function O(){return S(),18===Le()?r():void 0}function w(){var e=le(22);e&&T();var n=le(61),t=ue();return n&&an(61),e&&(T(),nn(63)&&Xr(),He(23)),{name:t,isBracketed:e}}function P(n){switch(n.kind){case 146:return!0;case 181:return P(n.elementType);default:return e.isTypeReferenceNode(n)&&e.isIdentifier(n.typeName)&&"Object"===n.typeName.escapedText&&!n.typeArguments}}function M(e,n,t,r){var i=O(),a=!i;S();var o=w(),s=o.name,c=o.isBracketed,l=S();a&&!Ue(I)&&(i=O());var u=L(e,Se(),r,l),d=4!==t&&G(i,s,t,r);d&&(i=d,a=!0);var p=1===t?F.createJSDocPropertyTag(n,s,c,i,a,u):F.createJSDocParameterTag(n,s,c,i,a,u);return pn(p,e)}function G(n,t,r,i){if(n&&P(n.type)){var a=Se(),o=void 0,s=void 0;while(o=Ve(function(){return ie(r,i,t)}))335!==o.kind&&342!==o.kind||(s=e.append(s,o));if(s){var c=pn(F.createJSDocTypeLiteral(s,181===n.type.kind),a);return pn(F.createJSDocTypeExpression(c),a)}}}function B(n,t,r,i){e.some(s,e.isJSDocReturnTag)&&be(t.pos,c.getTokenPos(),e.Diagnostics._0_tag_already_specified,t.escapedText);var a=O();return pn(F.createJSDocReturnTag(t,a,L(n,Se(),r,i)),n)}function U(n,t,i,a){e.some(s,e.isJSDocTypeTag)&&be(t.pos,c.getTokenPos(),e.Diagnostics._0_tag_already_specified,t.escapedText);var o=r(!0),l=void 0!==i&&void 0!==a?L(n,Se(),i,a):void 0;return pn(F.createJSDocTypeTag(t,o,l),n)}function V(n,t,r,a){var o=Ue(function(){return 59===Ie()&&e.tokenIsIdentifierOrKeyword(Ie())&&"link"===c.getTokenValue()}),s=o?void 0:i(),l=void 0!==r&&void 0!==a?L(n,Se(),r,a):void 0;return pn(F.createJSDocSeeTag(t,s,l),n)}function j(n,t,r,i){var a=Se(),o=K(),s=c.getStartPos(),l=L(n,s,r,i);l||(s=c.getStartPos());var u="string"!==typeof l?dn(e.concatenate([pn(o,a,s)],l),a):o.text+l;return pn(F.createJSDocAuthorTag(t,u),n)}function K(){var e=[],n=!1,t=c.getToken();while(1!==t&&4!==t){if(29===t)n=!0;else{if(59===t&&!n)break;if(31===t&&n){e.push(c.getTokenText()),c.setTextPos(c.getTokenPos()+1);break}}e.push(c.getTokenText()),t=Ie()}return F.createJSDocText(e.join(""))}function H(e,n,t,r){var i=z();return pn(F.createJSDocImplementsTag(n,i,L(e,Se(),t,r)),e)}function W(e,n,t,r){var i=z();return pn(F.createJSDocAugmentsTag(n,i,L(e,Se(),t,r)),e)}function z(){var e=en(18),n=Se(),t=q(),r=ko(),i=F.createExpressionWithTypeArguments(t,r),a=pn(i,n);return e&&He(19),a}function q(){var e=Se(),n=de();while(en(24)){var t=de();n=pn(F.createPropertyAccessExpression(n,t),e)}return n}function X(e,n,t,r,i){return pn(n(t,L(e,Se(),r,i)),e)}function Y(e,n,t,i){var a=r(!0);return T(),pn(F.createJSDocThisTag(n,a,L(e,Se(),t,i)),e)}function Q(e,n,t,i){var a=r(!0);return T(),pn(F.createJSDocEnumTag(n,a,L(e,Se(),t,i)),e)}function Z(n,t,r,i){var a,o=O();S();var s=$();T();var c,l=D(r);if(!o||P(o.type)){var u=void 0,d=void 0,f=void 0,m=!1;while(u=Ve(function(){return re(r)}))if(m=!0,338===u.kind){if(d){he(e.Diagnostics.A_JSDoc_typedef_comment_may_not_contain_multiple_type_tags);var _=e.lastOrUndefined(E);_&&e.addRelatedInfo(_,e.createDetachedDiagnostic(p,0,0,e.Diagnostics.The_tag_was_first_specified_here));break}d=u}else f=e.append(f,u);if(m){var g=o&&181===o.type.kind,y=F.createJSDocTypeLiteral(f,g);o=d&&d.typeExpression&&!P(d.typeExpression.type)?d.typeExpression:pn(y,n),c=o.end}}c=c||void 0!==l?Se():(null!==(a=null!==s&&void 0!==s?s:o)&&void 0!==a?a:t).end,l||(l=L(n,c,r,i));var h=F.createJSDocTypedefTag(t,o,s,l);return pn(h,n,c)}function $(n){var t=c.getTokenPos();if(e.tokenIsIdentifierOrKeyword(Le())){var r=de();if(en(24)){var i=$(!0),a=F.createModuleDeclaration(void 0,void 0,r,i,n?4:void 0);return pn(a,t)}return n&&(r.isInJSDocNamespace=!0),r}}function ee(n){var t,r,i=Se();while(t=Ve(function(){return ie(4,n)}))r=e.append(r,t);return dn(r||[],i)}function ne(e,n,t,r){var i=$();T();var a=D(t),o=ee(t),s=Ve(function(){if(le(59)){var e=x(t);if(e&&336===e.kind)return e}}),c=pn(F.createJSDocSignature(void 0,o,s),e);return a||(a=L(e,Se(),t,r)),pn(F.createJSDocCallbackTag(n,c,i,a),e)}function te(n,t){while(!e.isIdentifier(n)||!e.isIdentifier(t)){if(e.isIdentifier(n)||e.isIdentifier(t)||n.right.escapedText!==t.right.escapedText)return!1;n=n.left,t=t.left}return n.escapedText===t.escapedText}function re(e){return ie(1,e)}function ie(n,t,r){var i=!0,a=!1;while(1)switch(Ie()){case 59:if(i){var o=ae(n,t);return!(o&&(335===o.kind||342===o.kind)&&4!==n&&r&&(e.isIdentifier(o.name)||!te(r,o.name.left)))&&o}a=!1;break;case 4:i=!0,a=!1;break;case 41:a&&(i=!1),a=!0;break;case 79:i=!1;break;case 1:return!1}}function ae(n,t){e.Debug.assert(59===Le());var r=c.getStartPos();Ie();var i,a=de();switch(T(),a.escapedText){case"type":return 1===n&&U(r,a);case"prop":case"property":i=1;break;case"arg":case"argument":case"param":i=6;break;default:return!1}return!!(n&i)&&M(r,a,n,t)}function oe(){var n=Se(),t=de(e.Diagnostics.Unexpected_token_A_type_parameter_name_was_expected_without_curly_braces);if(!e.nodeIsMissing(t))return pn(F.createTypeParameterDeclaration(t,void 0,void 0),n)}function se(){var e=Se(),n=[];do{T();var t=oe();void 0!==t&&n.push(t),S()}while(le(27));return dn(n,e)}function ce(e,n,t,i){var a=18===Le()?r():void 0,o=se();return pn(F.createJSDocTemplateTag(n,a,o,L(e,Se(),t,i)),e)}function le(e){return Le()===e&&(Ie(),!0)}function ue(){var e=de();en(22)&&He(23);while(en(24)){var n=de();en(22)&&He(23),e=dt(e,n)}return e}function de(n){if(!e.tokenIsIdentifierOrKeyword(Le()))return fn(79,!n,n||e.Diagnostics.Identifier_expected);k++;var t=c.getTokenPos(),r=c.getTextPos(),i=Le(),a=mn(c.getTokenValue()),o=pn(F.createIdentifier(a,void 0,i),t,r);return Ie(),o}}n.parseJSDocTypeExpressionForTests=t,n.parseJSDocTypeExpression=r,n.parseJSDocNameReference=i,n.parseIsolatedJSDocComment=a,n.parseJSDocComment=o,function(e){e[e["BeginningOfLine"]=0]="BeginningOfLine",e[e["SawAsterisk"]=1]="SawAsterisk",e[e["SavingComments"]=2]="SavingComments",e[e["SavingBackticks"]=3]="SavingBackticks"}(s||(s={})),function(e){e[e["Property"]=1]="Property",e[e["Parameter"]=2]="Parameter",e[e["CallbackParameter"]=4]="CallbackParameter"}(l||(l={}))}(Je=n.JSDocParser||(n.JSDocParser={}))}(l||(l={})),function(n){function t(n,t,i,a){if(a=a||e.Debug.shouldAssert(2),p(n,t,i,a),e.textChangeRangeIsUnchanged(i))return n;if(0===n.statements.length)return l.parseSourceFile(n.fileName,t,n.languageVersion,void 0,!0,n.scriptKind);var o=n;e.Debug.assert(!o.hasBeenIncrementallyParsed),o.hasBeenIncrementallyParsed=!0,l.fixupParentReferences(o);var s=n.text,d=f(n),m=u(n,i);p(n,t,m,a),e.Debug.assert(m.span.start<=i.span.start),e.Debug.assert(e.textSpanEnd(m.span)===e.textSpanEnd(i.span)),e.Debug.assert(e.textSpanEnd(e.textChangeRangeNewSpan(m))===e.textSpanEnd(e.textChangeRangeNewSpan(i)));var _=e.textChangeRangeNewSpan(m).length-m.span.length;c(o,m.span.start,e.textSpanEnd(m.span),e.textSpanEnd(e.textChangeRangeNewSpan(m)),_,s,t,a);var g=l.parseSourceFile(n.fileName,t,n.languageVersion,d,!0,n.scriptKind);return g.commentDirectives=r(n.commentDirectives,g.commentDirectives,m.span.start,e.textSpanEnd(m.span),_,s,t,a),g}function r(n,t,r,i,a,o,s,c){if(!n)return t;for(var l,u=!1,d=0,p=n;d<p.length;d++){var f=p[d],m=f.range,_=f.type;if(m.end<r)l=e.append(l,f);else if(m.pos>i){y();var g={range:{pos:m.pos+a,end:m.end+a},type:_};l=e.append(l,g),c&&e.Debug.assert(o.substring(m.pos,m.end)===s.substring(g.range.pos,g.range.end))}}return y(),l;function y(){u||(u=!0,l?t&&l.push.apply(l,t):l=t)}}function i(n,t,r,i,o,c){return void(t?u(n):l(n));function l(n){var t="";if(c&&a(n)&&(t=i.substring(n.pos,n.end)),n._children&&(n._children=void 0),e.setTextRangePosEnd(n,n.pos+r,n.end+r),c&&a(n)&&e.Debug.assert(t===o.substring(n.pos,n.end)),m(n,l,u),e.hasJSDocNodes(n))for(var d=0,p=n.jsDoc;d<p.length;d++){var f=p[d];l(f)}s(n,c)}function u(n){n._children=void 0,e.setTextRangePosEnd(n,n.pos+r,n.end+r);for(var t=0,i=n;t<i.length;t++){var a=i[t];l(a)}}}function a(e){switch(e.kind){case 10:case 8:case 79:return!0}return!1}function o(n,t,r,i,a){e.Debug.assert(n.end>=t,"Adjusting an element that was entirely before the change range"),e.Debug.assert(n.pos<=r,"Adjusting an element that was entirely after the change range"),e.Debug.assert(n.pos<=n.end);var o=Math.min(n.pos,i),s=n.end>=r?n.end+a:Math.min(n.end,i);e.Debug.assert(o<=s),n.parent&&(e.Debug.assertGreaterThanOrEqual(o,n.parent.pos),e.Debug.assertLessThanOrEqual(s,n.parent.end)),e.setTextRangePosEnd(n,o,s)}function s(n,t){if(t){var r=n.pos,i=function(n){e.Debug.assert(n.pos>=r),r=n.end};if(e.hasJSDocNodes(n))for(var a=0,o=n.jsDoc;a<o.length;a++){var s=o[a];i(s)}m(n,i),e.Debug.assert(r<=n.end)}}function c(n,t,r,a,c,l,u,d){return void p(n);function p(n){if(e.Debug.assert(n.pos<=n.end),n.pos>r)i(n,!1,c,l,u,d);else{var _=n.end;if(_>=t){if(n.intersectsChange=!0,n._children=void 0,o(n,t,r,a,c),m(n,p,f),e.hasJSDocNodes(n))for(var g=0,y=n.jsDoc;g<y.length;g++){var h=y[g];p(h)}s(n,d)}else e.Debug.assert(_<t)}}function f(n){if(e.Debug.assert(n.pos<=n.end),n.pos>r)i(n,!0,c,l,u,d);else{var s=n.end;if(s>=t){n.intersectsChange=!0,n._children=void 0,o(n,t,r,a,c);for(var f=0,m=n;f<m.length;f++){var _=m[f];p(_)}}else e.Debug.assert(s<t)}}}function u(n,t){for(var r=1,i=t.span.start,a=0;i>0&&a<=r;a++){var o=d(n,i);e.Debug.assert(o.pos<=i);var s=o.pos;i=Math.max(0,s-1)}var c=e.createTextSpanFromBounds(i,e.textSpanEnd(t.span)),l=t.newLength+(t.span.start-i);return e.createTextChangeRange(c,l)}function d(n,t){var r,i=n;if(m(n,s),r){var a=o(r);a.pos>i.pos&&(i=a)}return i;function o(n){while(1){var t=e.getLastChild(n);if(!t)return n;n=t}}function s(n){if(!e.nodeIsMissing(n))return n.pos<=t?(n.pos>=i.pos&&(i=n),t<n.end?(m(n,s),!0):(e.Debug.assert(n.end<=t),void(r=n))):(e.Debug.assert(n.pos>t),!0)}}function p(n,t,r,i){var a=n.text;if(r&&(e.Debug.assert(a.length-r.span.length+r.newLength===t.length),i||e.Debug.shouldAssert(3))){var o=a.substr(0,r.span.start),s=t.substr(0,r.span.start);e.Debug.assert(o===s);var c=a.substring(e.textSpanEnd(r.span),a.length),l=t.substring(e.textSpanEnd(e.textChangeRangeNewSpan(r)),t.length);e.Debug.assert(c===l)}}function f(n){var t=n.statements,r=0;e.Debug.assert(r<t.length);var i=t[r],a=-1;return{currentNode:function(n){return n!==a&&(i&&i.end===n&&r<t.length-1&&(r++,i=t[r]),i&&i.pos===n||o(n)),a=n,e.Debug.assert(!i||i.pos===n),i}};function o(e){return t=void 0,r=-1,i=void 0,void m(n,a,o);function a(n){return e>=n.pos&&e<n.end&&(m(n,a,o),!0)}function o(n){if(e>=n.pos&&e<n.end)for(var s=0;s<n.length;s++){var c=n[s];if(c){if(c.pos===e)return t=n,r=s,i=c,!0;if(c.pos<e&&e<c.end)return m(c,a,o),!0}}return!1}}}var _;n.updateSourceFile=t,n.createSyntaxCursor=f,function(e){e[e["Value"]=-1]="Value"}(_||(_={}))}(u||(u={})),e.isDeclarationFileName=x,e.processCommentPragmas=L,e.processPragmasIntoFields=D;var A=new e.Map;function C(e){if(A.has(e))return A.get(e);var n=new RegExp("(\\s"+e+"\\s*=\\s*)(?:(?:'([^']*)')|(?:\"([^\"]*)\"))","im");return A.set(e,n),n}var I=/^\/\/\/\s*<(\S+)\s.*?\/>/im,N=/^\/\/\/?\s*@(\S+)\s*(.*)\s*$/im;function k(n,t,r){var i=2===t.kind&&I.exec(r);if(i){var a=i[1].toLowerCase(),o=e.commentPragmas[a];if(!o||!(1&o.kind))return;if(o.args){for(var s={},c=0,l=o.args;c<l.length;c++){var u=l[c],d=C(u.name),p=d.exec(r);if(!p&&!u.optional)return;if(p){var f=p[2]||p[3];if(u.captureSpan){var m=t.pos+p.index+p[1].length+1;s[u.name]={value:f,pos:m,end:m+f.length}}else s[u.name]=f}}n.push({name:a,args:{arguments:s,range:t}})}else n.push({name:a,args:{arguments:{},range:t}})}else{var _=2===t.kind&&N.exec(r);if(_)return R(n,t,2,_);if(3===t.kind){var g=/@(\S+)(\s+.*)?$/gim,y=void 0;while(y=g.exec(r))R(n,t,4,y)}}}function R(n,t,r,i){if(i){var a=i[1].toLowerCase(),o=e.commentPragmas[a];if(o&&o.kind&r){var s=i[2],c=O(o,s);"fail"!==c&&n.push({name:a,args:{arguments:c,range:t}})}}}function O(n,t){if(!t)return{};if(!n.args)return{};for(var r=e.trimString(t).split(/\s+/),i={},a=0;a<n.args.length;a++){var o=n.args[a];if(!r[a]&&!o.optional)return"fail";if(o.captureSpan)return e.Debug.fail("Capture spans not yet implemented for non-xml pragmas");i[o.name]=r[a]}return i}function w(e,n){return e.kind===n.kind&&(79===e.kind?e.escapedText===n.escapedText:108===e.kind||e.name.escapedText===n.name.escapedText&&w(e.expression,n.expression))}e.tagNamesAreEquivalent=w}(d||(d={})),function(e){e.compileOnSaveCommandLineOption={name:"compileOnSave",type:"boolean"};var n=new e.Map(e.getEntries({preserve:1,"react-native":3,react:2,"react-jsx":4,"react-jsxdev":5}));e.inverseJsxOptionMap=new e.Map(e.arrayFrom(e.mapIterator(n.entries(),function(e){var n=e[0],t=e[1];return[""+t,n]})));var t=[["es5","lib.es5.d.ts"],["es6","lib.es2015.d.ts"],["es2015","lib.es2015.d.ts"],["es7","lib.es2016.d.ts"],["es2016","lib.es2016.d.ts"],["es2017","lib.es2017.d.ts"],["es2018","lib.es2018.d.ts"],["es2019","lib.es2019.d.ts"],["es2020","lib.es2020.d.ts"],["es2021","lib.es2021.d.ts"],["esnext","lib.esnext.d.ts"],["dom","lib.dom.d.ts"],["dom.iterable","lib.dom.iterable.d.ts"],["webworker","lib.webworker.d.ts"],["webworker.importscripts","lib.webworker.importscripts.d.ts"],["webworker.iterable","lib.webworker.iterable.d.ts"],["scripthost","lib.scripthost.d.ts"],["es2015.core","lib.es2015.core.d.ts"],["es2015.collection","lib.es2015.collection.d.ts"],["es2015.generator","lib.es2015.generator.d.ts"],["es2015.iterable","lib.es2015.iterable.d.ts"],["es2015.promise","lib.es2015.promise.d.ts"],["es2015.proxy","lib.es2015.proxy.d.ts"],["es2015.reflect","lib.es2015.reflect.d.ts"],["es2015.symbol","lib.es2015.symbol.d.ts"],["es2015.symbol.wellknown","lib.es2015.symbol.wellknown.d.ts"],["es2016.array.include","lib.es2016.array.include.d.ts"],["es2017.object","lib.es2017.object.d.ts"],["es2017.sharedmemory","lib.es2017.sharedmemory.d.ts"],["es2017.string","lib.es2017.string.d.ts"],["es2017.intl","lib.es2017.intl.d.ts"],["es2017.typedarrays","lib.es2017.typedarrays.d.ts"],["es2018.asyncgenerator","lib.es2018.asyncgenerator.d.ts"],["es2018.asynciterable","lib.es2018.asynciterable.d.ts"],["es2018.intl","lib.es2018.intl.d.ts"],["es2018.promise","lib.es2018.promise.d.ts"],["es2018.regexp","lib.es2018.regexp.d.ts"],["es2019.array","lib.es2019.array.d.ts"],["es2019.object","lib.es2019.object.d.ts"],["es2019.string","lib.es2019.string.d.ts"],["es2019.symbol","lib.es2019.symbol.d.ts"],["es2020.bigint","lib.es2020.bigint.d.ts"],["es2020.promise","lib.es2020.promise.d.ts"],["es2020.sharedmemory","lib.es2020.sharedmemory.d.ts"],["es2020.string","lib.es2020.string.d.ts"],["es2020.symbol.wellknown","lib.es2020.symbol.wellknown.d.ts"],["es2020.intl","lib.es2020.intl.d.ts"],["es2021.promise","lib.es2021.promise.d.ts"],["es2021.string","lib.es2021.string.d.ts"],["es2021.weakref","lib.es2021.weakref.d.ts"],["esnext.array","lib.es2019.array.d.ts"],["esnext.symbol","lib.es2019.symbol.d.ts"],["esnext.asynciterable","lib.es2018.asynciterable.d.ts"],["esnext.intl","lib.esnext.intl.d.ts"],["esnext.bigint","lib.es2020.bigint.d.ts"],["esnext.string","lib.es2021.string.d.ts"],["esnext.promise","lib.es2021.promise.d.ts"],["esnext.weakref","lib.es2021.weakref.d.ts"]];e.libs=t.map(function(e){return e[0]}),e.libMap=new e.Map(t),e.optionsForWatch=[{name:"watchFile",type:new e.Map(e.getEntries({fixedpollinginterval:e.WatchFileKind.FixedPollingInterval,prioritypollinginterval:e.WatchFileKind.PriorityPollingInterval,dynamicprioritypolling:e.WatchFileKind.DynamicPriorityPolling,fixedchunksizepolling:e.WatchFileKind.FixedChunkSizePolling,usefsevents:e.WatchFileKind.UseFsEvents,usefseventsonparentdirectory:e.WatchFileKind.UseFsEventsOnParentDirectory})),category:e.Diagnostics.Watch_and_Build_Modes,description:e.Diagnostics.Specify_how_the_TypeScript_watch_mode_works},{name:"watchDirectory",type:new e.Map(e.getEntries({usefsevents:e.WatchDirectoryKind.UseFsEvents,fixedpollinginterval:e.WatchDirectoryKind.FixedPollingInterval,dynamicprioritypolling:e.WatchDirectoryKind.DynamicPriorityPolling,fixedchunksizepolling:e.WatchDirectoryKind.FixedChunkSizePolling})),category:e.Diagnostics.Watch_and_Build_Modes,description:e.Diagnostics.Specify_how_directories_are_watched_on_systems_that_lack_recursive_file_watching_functionality},{name:"fallbackPolling",type:new e.Map(e.getEntries({fixedinterval:e.PollingWatchKind.FixedInterval,priorityinterval:e.PollingWatchKind.PriorityInterval,dynamicpriority:e.PollingWatchKind.DynamicPriority,fixedchunksize:e.PollingWatchKind.FixedChunkSize})),category:e.Diagnostics.Watch_and_Build_Modes,description:e.Diagnostics.Specify_what_approach_the_watcher_should_use_if_the_system_runs_out_of_native_file_watchers},{name:"synchronousWatchDirectory",type:"boolean",category:e.Diagnostics.Watch_and_Build_Modes,description:e.Diagnostics.Synchronously_call_callbacks_and_update_the_state_of_directory_watchers_on_platforms_that_don_t_support_recursive_watching_natively},{name:"excludeDirectories",type:"list",element:{name:"excludeDirectory",type:"string",isFilePath:!0,extraValidation:Xe},category:e.Diagnostics.Watch_and_Build_Modes,description:e.Diagnostics.Remove_a_list_of_directories_from_the_watch_process},{name:"excludeFiles",type:"list",element:{name:"excludeFile",type:"string",isFilePath:!0,extraValidation:Xe},category:e.Diagnostics.Watch_and_Build_Modes,description:e.Diagnostics.Remove_a_list_of_files_from_the_watch_mode_s_processing}],e.commonOptionsWithBuild=[{name:"help",shortName:"h",type:"boolean",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Print_this_message},{name:"help",shortName:"?",type:"boolean"},{name:"watch",shortName:"w",type:"boolean",showInSimplifiedHelpView:!0,isCommandLineOnly:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Watch_input_files},{name:"preserveWatchOutput",type:"boolean",showInSimplifiedHelpView:!1,category:e.Diagnostics.Output_Formatting,description:e.Diagnostics.Disable_wiping_the_console_in_watch_mode,defaultValueDescription:"n/a"},{name:"listFiles",type:"boolean",category:e.Diagnostics.Compiler_Diagnostics,description:e.Diagnostics.Print_all_of_the_files_read_during_the_compilation,defaultValueDescription:"false"},{name:"explainFiles",type:"boolean",category:e.Diagnostics.Compiler_Diagnostics,description:e.Diagnostics.Print_files_read_during_the_compilation_including_why_it_was_included},{name:"listEmittedFiles",type:"boolean",category:e.Diagnostics.Compiler_Diagnostics,description:e.Diagnostics.Print_the_names_of_emitted_files_after_a_compilation,defaultValueDescription:"false"},{name:"pretty",type:"boolean",showInSimplifiedHelpView:!0,category:e.Diagnostics.Output_Formatting,description:e.Diagnostics.Enable_color_and_formatting_in_TypeScript_s_output_to_make_compiler_errors_easier_to_read,defaultValueDescription:"true"},{name:"traceResolution",type:"boolean",category:e.Diagnostics.Compiler_Diagnostics,description:e.Diagnostics.Log_paths_used_during_the_moduleResolution_process,defaultValueDescription:"false"},{name:"diagnostics",type:"boolean",category:e.Diagnostics.Compiler_Diagnostics,description:e.Diagnostics.Output_compiler_performance_information_after_building,defaultValueDescription:"false"},{name:"extendedDiagnostics",type:"boolean",category:e.Diagnostics.Compiler_Diagnostics,description:e.Diagnostics.Output_more_detailed_compiler_performance_information_after_building,defaultValueDescription:"false"},{name:"generateCpuProfile",type:"string",isFilePath:!0,paramType:e.Diagnostics.FILE_OR_DIRECTORY,category:e.Diagnostics.Compiler_Diagnostics,description:e.Diagnostics.Emit_a_v8_CPU_profile_of_the_compiler_run_for_debugging,defaultValueDescription:"profile.cpuprofile"},{name:"generateTrace",type:"string",isFilePath:!0,isCommandLineOnly:!0,paramType:e.Diagnostics.DIRECTORY,category:e.Diagnostics.Compiler_Diagnostics,description:e.Diagnostics.Generates_an_event_trace_and_a_list_of_types},{name:"incremental",shortName:"i",type:"boolean",category:e.Diagnostics.Projects,description:e.Diagnostics.Enable_incremental_compilation,transpileOptionValue:void 0,defaultValueDescription:e.Diagnostics.false_unless_composite_is_set},{name:"assumeChangesOnlyAffectDirectDependencies",type:"boolean",affectsSemanticDiagnostics:!0,affectsEmit:!0,category:e.Diagnostics.Watch_and_Build_Modes,description:e.Diagnostics.Have_recompiles_in_projects_that_use_incremental_and_watch_mode_assume_that_changes_within_a_file_will_only_affect_files_directly_depending_on_it},{name:"locale",type:"string",category:e.Diagnostics.Command_line_Options,isCommandLineOnly:!0,description:e.Diagnostics.Set_the_language_of_the_messaging_from_TypeScript_This_does_not_affect_emit,defaultValueDescription:e.Diagnostics.Platform_specific}],e.targetOptionDeclaration={name:"target",shortName:"t",type:new e.Map(e.getEntries({es3:0,es5:1,es6:2,es2015:2,es2016:3,es2017:4,es2018:5,es2019:6,es2020:7,es2021:8,esnext:99})),affectsSourceFile:!0,affectsModuleResolution:!0,affectsEmit:!0,paramType:e.Diagnostics.VERSION,showInSimplifiedHelpView:!0,category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Set_the_JavaScript_language_version_for_emitted_JavaScript_and_include_compatible_library_declarations,defaultValueDescription:"ES3"};var r,i=[{name:"all",type:"boolean",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Show_all_compiler_options},{name:"version",shortName:"v",type:"boolean",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Print_the_compiler_s_version},{name:"init",type:"boolean",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Initializes_a_TypeScript_project_and_creates_a_tsconfig_json_file},{name:"project",shortName:"p",type:"string",isFilePath:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,paramType:e.Diagnostics.FILE_OR_DIRECTORY,description:e.Diagnostics.Compile_the_project_given_the_path_to_its_configuration_file_or_to_a_folder_with_a_tsconfig_json},{name:"build",type:"boolean",shortName:"b",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Build_one_or_more_projects_and_their_dependencies_if_out_of_date},{name:"showConfig",type:"boolean",showInSimplifiedHelpView:!0,category:e.Diagnostics.Command_line_Options,isCommandLineOnly:!0,description:e.Diagnostics.Print_the_final_configuration_instead_of_building},{name:"listFilesOnly",type:"boolean",category:e.Diagnostics.Command_line_Options,affectsSemanticDiagnostics:!0,affectsEmit:!0,isCommandLineOnly:!0,description:e.Diagnostics.Print_names_of_files_that_are_part_of_the_compilation_and_then_stop_processing},e.targetOptionDeclaration,{name:"module",shortName:"m",type:new e.Map(e.getEntries({none:e.ModuleKind.None,commonjs:e.ModuleKind.CommonJS,amd:e.ModuleKind.AMD,system:e.ModuleKind.System,umd:e.ModuleKind.UMD,es6:e.ModuleKind.ES2015,es2015:e.ModuleKind.ES2015,es2020:e.ModuleKind.ES2020,esnext:e.ModuleKind.ESNext})),affectsModuleResolution:!0,affectsEmit:!0,paramType:e.Diagnostics.KIND,showInSimplifiedHelpView:!0,category:e.Diagnostics.Modules,description:e.Diagnostics.Specify_what_module_code_is_generated},{name:"lib",type:"list",element:{name:"lib",type:e.libMap},affectsProgramStructure:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Specify_a_set_of_bundled_library_declaration_files_that_describe_the_target_runtime_environment,transpileOptionValue:void 0},{name:"allowJs",type:"boolean",affectsModuleResolution:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.JavaScript_Support,description:e.Diagnostics.Allow_JavaScript_files_to_be_a_part_of_your_program_Use_the_checkJS_option_to_get_errors_from_these_files,defaultValueDescription:"false"},{name:"checkJs",type:"boolean",showInSimplifiedHelpView:!0,category:e.Diagnostics.JavaScript_Support,description:e.Diagnostics.Enable_error_reporting_in_type_checked_JavaScript_files,defaultValueDescription:"false"},{name:"jsx",type:n,affectsSourceFile:!0,affectsEmit:!0,affectsModuleResolution:!0,paramType:e.Diagnostics.KIND,showInSimplifiedHelpView:!0,category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Specify_what_JSX_code_is_generated,defaultValueDescription:"undefined"},{name:"declaration",shortName:"d",type:"boolean",affectsEmit:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Emit,transpileOptionValue:void 0,description:e.Diagnostics.Generate_d_ts_files_from_TypeScript_and_JavaScript_files_in_your_project,defaultValueDescription:e.Diagnostics.false_unless_composite_is_set},{name:"declarationMap",type:"boolean",affectsEmit:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Emit,transpileOptionValue:void 0,defaultValueDescription:"false",description:e.Diagnostics.Create_sourcemaps_for_d_ts_files},{name:"emitDeclarationOnly",type:"boolean",affectsEmit:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Only_output_d_ts_files_and_not_JavaScript_files,transpileOptionValue:void 0,defaultValueDescription:"false"},{name:"sourceMap",type:"boolean",affectsEmit:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Emit,defaultValueDescription:"false",description:e.Diagnostics.Create_source_map_files_for_emitted_JavaScript_files},{name:"outFile",type:"string",affectsEmit:!0,isFilePath:!0,paramType:e.Diagnostics.FILE,showInSimplifiedHelpView:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Specify_a_file_that_bundles_all_outputs_into_one_JavaScript_file_If_declaration_is_true_also_designates_a_file_that_bundles_all_d_ts_output,transpileOptionValue:void 0,defaultValueDescription:"n/a"},{name:"outDir",type:"string",affectsEmit:!0,isFilePath:!0,paramType:e.Diagnostics.DIRECTORY,showInSimplifiedHelpView:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Specify_an_output_folder_for_all_emitted_files,defaultValueDescription:"n/a"},{name:"rootDir",type:"string",affectsEmit:!0,isFilePath:!0,paramType:e.Diagnostics.LOCATION,category:e.Diagnostics.Modules,description:e.Diagnostics.Specify_the_root_folder_within_your_source_files,defaultValueDescription:e.Diagnostics.Computed_from_the_list_of_input_files},{name:"composite",type:"boolean",affectsEmit:!0,isTSConfigOnly:!0,category:e.Diagnostics.Projects,transpileOptionValue:void 0,defaultValueDescription:"false",description:e.Diagnostics.Enable_constraints_that_allow_a_TypeScript_project_to_be_used_with_project_references},{name:"tsBuildInfoFile",type:"string",affectsEmit:!0,isFilePath:!0,paramType:e.Diagnostics.FILE,category:e.Diagnostics.Projects,transpileOptionValue:void 0,defaultValueDescription:".tsbuildinfo",description:e.Diagnostics.Specify_the_folder_for_tsbuildinfo_incremental_compilation_files},{name:"removeComments",type:"boolean",affectsEmit:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Emit,defaultValueDescription:"false",description:e.Diagnostics.Disable_emitting_comments},{name:"noEmit",type:"boolean",showInSimplifiedHelpView:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Disable_emitting_files_from_a_compilation,transpileOptionValue:void 0,defaultValueDescription:"false"},{name:"importHelpers",type:"boolean",affectsEmit:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Allow_importing_helper_functions_from_tslib_once_per_project_instead_of_including_them_per_file,defaultValueDescription:"false"},{name:"importsNotUsedAsValues",type:new e.Map(e.getEntries({remove:0,preserve:1,error:2})),affectsEmit:!0,affectsSemanticDiagnostics:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Specify_emit_Slashchecking_behavior_for_imports_that_are_only_used_for_types},{name:"downlevelIteration",type:"boolean",affectsEmit:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Emit_more_compliant_but_verbose_and_less_performant_JavaScript_for_iteration,defaultValueDescription:"false"},{name:"isolatedModules",type:"boolean",category:e.Diagnostics.Interop_Constraints,description:e.Diagnostics.Ensure_that_each_file_can_be_safely_transpiled_without_relying_on_other_imports,transpileOptionValue:!0,defaultValueDescription:"false"},{name:"strict",type:"boolean",showInSimplifiedHelpView:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Enable_all_strict_type_checking_options,defaultValueDescription:"false"},{name:"noImplicitAny",type:"boolean",affectsSemanticDiagnostics:!0,strictFlag:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Enable_error_reporting_for_expressions_and_declarations_with_an_implied_any_type,defaultValueDescription:e.Diagnostics.false_unless_strict_is_set},{name:"strictNullChecks",type:"boolean",affectsSemanticDiagnostics:!0,strictFlag:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.When_type_checking_take_into_account_null_and_undefined,defaultValueDescription:e.Diagnostics.false_unless_strict_is_set},{name:"strictFunctionTypes",type:"boolean",strictFlag:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.When_assigning_functions_check_to_ensure_parameters_and_the_return_values_are_subtype_compatible,defaultValueDescription:e.Diagnostics.false_unless_strict_is_set},{name:"strictBindCallApply",type:"boolean",strictFlag:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Check_that_the_arguments_for_bind_call_and_apply_methods_match_the_original_function,defaultValueDescription:e.Diagnostics.false_unless_strict_is_set},{name:"strictPropertyInitialization",type:"boolean",affectsSemanticDiagnostics:!0,strictFlag:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Check_for_class_properties_that_are_declared_but_not_set_in_the_constructor,defaultValueDescription:e.Diagnostics.false_unless_strict_is_set},{name:"noImplicitThis",type:"boolean",affectsSemanticDiagnostics:!0,strictFlag:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Enable_error_reporting_when_this_is_given_the_type_any,defaultValueDescription:e.Diagnostics.false_unless_strict_is_set},{name:"useUnknownInCatchVariables",type:"boolean",affectsSemanticDiagnostics:!0,strictFlag:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Type_catch_clause_variables_as_unknown_instead_of_any},{name:"alwaysStrict",type:"boolean",affectsSourceFile:!0,strictFlag:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Ensure_use_strict_is_always_emitted,defaultValueDescription:e.Diagnostics.false_unless_strict_is_set},{name:"noUnusedLocals",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Enable_error_reporting_when_a_local_variables_aren_t_read,defaultValueDescription:"false"},{name:"noUnusedParameters",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Raise_an_error_when_a_function_parameter_isn_t_read,defaultValueDescription:"false"},{name:"exactOptionalPropertyTypes",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Interpret_optional_property_types_as_written_rather_than_adding_undefined},{name:"noImplicitReturns",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Enable_error_reporting_for_codepaths_that_do_not_explicitly_return_in_a_function,defaultValueDescription:"false"},{name:"noFallthroughCasesInSwitch",type:"boolean",affectsBindDiagnostics:!0,affectsSemanticDiagnostics:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Enable_error_reporting_for_fallthrough_cases_in_switch_statements},{name:"noUncheckedIndexedAccess",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Include_undefined_in_index_signature_results},{name:"noImplicitOverride",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Ensure_overriding_members_in_derived_classes_are_marked_with_an_override_modifier},{name:"noPropertyAccessFromIndexSignature",type:"boolean",showInSimplifiedHelpView:!1,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Enforces_using_indexed_accessors_for_keys_declared_using_an_indexed_type,defaultValueDescription:"false"},{name:"moduleResolution",type:new e.Map(e.getEntries({node:e.ModuleResolutionKind.NodeJs,classic:e.ModuleResolutionKind.Classic})),affectsModuleResolution:!0,paramType:e.Diagnostics.STRATEGY,category:e.Diagnostics.Modules,description:e.Diagnostics.Specify_how_TypeScript_looks_up_a_file_from_a_given_module_specifier,defaultValueDescription:e.Diagnostics.module_AMD_or_UMD_or_System_or_ES6_then_Classic_Otherwise_Node},{name:"baseUrl",type:"string",affectsModuleResolution:!0,isFilePath:!0,category:e.Diagnostics.Modules,description:e.Diagnostics.Specify_the_base_directory_to_resolve_non_relative_module_names},{name:"paths",type:"object",affectsModuleResolution:!0,isTSConfigOnly:!0,category:e.Diagnostics.Modules,description:e.Diagnostics.Specify_a_set_of_entries_that_re_map_imports_to_additional_lookup_locations,transpileOptionValue:void 0},{name:"rootDirs",type:"list",isTSConfigOnly:!0,element:{name:"rootDirs",type:"string",isFilePath:!0},affectsModuleResolution:!0,category:e.Diagnostics.Modules,description:e.Diagnostics.Allow_multiple_folders_to_be_treated_as_one_when_resolving_modules,transpileOptionValue:void 0,defaultValueDescription:e.Diagnostics.Computed_from_the_list_of_input_files},{name:"typeRoots",type:"list",element:{name:"typeRoots",type:"string",isFilePath:!0},affectsModuleResolution:!0,category:e.Diagnostics.Modules,description:e.Diagnostics.Specify_multiple_folders_that_act_like_Slashnode_modules_Slash_types},{name:"types",type:"list",element:{name:"types",type:"string"},affectsProgramStructure:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Modules,description:e.Diagnostics.Specify_type_package_names_to_be_included_without_being_referenced_in_a_source_file,transpileOptionValue:void 0},{name:"allowSyntheticDefaultImports",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Interop_Constraints,description:e.Diagnostics.Allow_import_x_from_y_when_a_module_doesn_t_have_a_default_export,defaultValueDescription:e.Diagnostics.module_system_or_esModuleInterop},{name:"esModuleInterop",type:"boolean",affectsSemanticDiagnostics:!0,affectsEmit:!0,showInSimplifiedHelpView:!0,category:e.Diagnostics.Interop_Constraints,description:e.Diagnostics.Emit_additional_JavaScript_to_ease_support_for_importing_CommonJS_modules_This_enables_allowSyntheticDefaultImports_for_type_compatibility,defaultValueDescription:"false"},{name:"preserveSymlinks",type:"boolean",category:e.Diagnostics.Interop_Constraints,description:e.Diagnostics.Disable_resolving_symlinks_to_their_realpath_This_correlates_to_the_same_flag_in_node,defaultValueDescription:"n/a"},{name:"allowUmdGlobalAccess",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Modules,description:e.Diagnostics.Allow_accessing_UMD_globals_from_modules,defaultValueDescription:"false"},{name:"sourceRoot",type:"string",affectsEmit:!0,paramType:e.Diagnostics.LOCATION,category:e.Diagnostics.Emit,description:e.Diagnostics.Specify_the_root_path_for_debuggers_to_find_the_reference_source_code},{name:"mapRoot",type:"string",affectsEmit:!0,paramType:e.Diagnostics.LOCATION,category:e.Diagnostics.Emit,description:e.Diagnostics.Specify_the_location_where_debugger_should_locate_map_files_instead_of_generated_locations},{name:"inlineSourceMap",type:"boolean",affectsEmit:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Include_sourcemap_files_inside_the_emitted_JavaScript,defaultValueDescription:"false"},{name:"inlineSources",type:"boolean",affectsEmit:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Include_source_code_in_the_sourcemaps_inside_the_emitted_JavaScript,defaultValueDescription:"false"},{name:"experimentalDecorators",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Enable_experimental_support_for_TC39_stage_2_draft_decorators},{name:"emitDecoratorMetadata",type:"boolean",affectsSemanticDiagnostics:!0,affectsEmit:!0,category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Emit_design_type_metadata_for_decorated_declarations_in_source_files},{name:"jsxFactory",type:"string",category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Specify_the_JSX_factory_function_used_when_targeting_React_JSX_emit_e_g_React_createElement_or_h,defaultValueDescription:"`React.createElement`"},{name:"jsxFragmentFactory",type:"string",category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Specify_the_JSX_Fragment_reference_used_for_fragments_when_targeting_React_JSX_emit_e_g_React_Fragment_or_Fragment},{name:"jsxImportSource",type:"string",affectsSemanticDiagnostics:!0,affectsEmit:!0,affectsModuleResolution:!0,category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Specify_module_specifier_used_to_import_the_JSX_factory_functions_when_using_jsx_Colon_react_jsx_Asterisk,defaultValueDescription:"react"},{name:"resolveJsonModule",type:"boolean",affectsModuleResolution:!0,category:e.Diagnostics.Modules,description:e.Diagnostics.Enable_importing_json_files,defaultValueDescription:"false"},{name:"out",type:"string",affectsEmit:!0,isFilePath:!1,category:e.Diagnostics.Backwards_Compatibility,paramType:e.Diagnostics.FILE,transpileOptionValue:void 0,defaultValueDescription:"n/a",description:e.Diagnostics.Deprecated_setting_Use_outFile_instead},{name:"reactNamespace",type:"string",affectsEmit:!0,category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Specify_the_object_invoked_for_createElement_This_only_applies_when_targeting_react_JSX_emit,defaultValueDescription:"`React`"},{name:"skipDefaultLibCheck",type:"boolean",category:e.Diagnostics.Completeness,description:e.Diagnostics.Skip_type_checking_d_ts_files_that_are_included_with_TypeScript,defaultValueDescription:"false"},{name:"charset",type:"string",category:e.Diagnostics.Backwards_Compatibility,description:e.Diagnostics.No_longer_supported_In_early_versions_manually_set_the_text_encoding_for_reading_files,defaultValueDescription:"utf8"},{name:"emitBOM",type:"boolean",affectsEmit:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Emit_a_UTF_8_Byte_Order_Mark_BOM_in_the_beginning_of_output_files,defaultValueDescription:"false"},{name:"newLine",type:new e.Map(e.getEntries({crlf:0,lf:1})),affectsEmit:!0,paramType:e.Diagnostics.NEWLINE,category:e.Diagnostics.Emit,description:e.Diagnostics.Set_the_newline_character_for_emitting_files,defaultValueDescription:e.Diagnostics.Platform_specific},{name:"noErrorTruncation",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Output_Formatting,description:e.Diagnostics.Disable_truncating_types_in_error_messages,defaultValueDescription:"false"},{name:"noLib",type:"boolean",category:e.Diagnostics.Language_and_Environment,affectsProgramStructure:!0,description:e.Diagnostics.Disable_including_any_library_files_including_the_default_lib_d_ts,transpileOptionValue:!0,defaultValueDescription:"false"},{name:"noResolve",type:"boolean",affectsModuleResolution:!0,category:e.Diagnostics.Modules,description:e.Diagnostics.Disallow_import_s_require_s_or_reference_s_from_expanding_the_number_of_files_TypeScript_should_add_to_a_project,transpileOptionValue:!0,defaultValueDescription:"false"},{name:"stripInternal",type:"boolean",affectsEmit:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Disable_emitting_declarations_that_have_internal_in_their_JSDoc_comments},{name:"disableSizeLimit",type:"boolean",affectsProgramStructure:!0,category:e.Diagnostics.Editor_Support,description:e.Diagnostics.Remove_the_20mb_cap_on_total_source_code_size_for_JavaScript_files_in_the_TypeScript_language_server,defaultValueDescription:"false"},{name:"disableSourceOfProjectReferenceRedirect",type:"boolean",isTSConfigOnly:!0,category:e.Diagnostics.Projects,description:e.Diagnostics.Disable_preferring_source_files_instead_of_declaration_files_when_referencing_composite_projects},{name:"disableSolutionSearching",type:"boolean",isTSConfigOnly:!0,category:e.Diagnostics.Projects,description:e.Diagnostics.Opt_a_project_out_of_multi_project_reference_checking_when_editing},{name:"disableReferencedProjectLoad",type:"boolean",isTSConfigOnly:!0,category:e.Diagnostics.Projects,description:e.Diagnostics.Reduce_the_number_of_projects_loaded_automatically_by_TypeScript},{name:"noImplicitUseStrict",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Backwards_Compatibility,description:e.Diagnostics.Disable_adding_use_strict_directives_in_emitted_JavaScript_files,defaultValueDescription:"false"},{name:"noEmitHelpers",type:"boolean",affectsEmit:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Disable_generating_custom_helper_functions_like_extends_in_compiled_output,defaultValueDescription:"false"},{name:"noEmitOnError",type:"boolean",affectsEmit:!0,category:e.Diagnostics.Emit,transpileOptionValue:void 0,description:e.Diagnostics.Disable_emitting_files_if_any_type_checking_errors_are_reported,defaultValueDescription:"false"},{name:"preserveConstEnums",type:"boolean",affectsEmit:!0,category:e.Diagnostics.Emit,description:e.Diagnostics.Disable_erasing_const_enum_declarations_in_generated_code,defaultValueDescription:"n/a"},{name:"declarationDir",type:"string",affectsEmit:!0,isFilePath:!0,paramType:e.Diagnostics.DIRECTORY,category:e.Diagnostics.Emit,transpileOptionValue:void 0,description:e.Diagnostics.Specify_the_output_directory_for_generated_declaration_files,defaultValueDescription:"n/a"},{name:"skipLibCheck",type:"boolean",category:e.Diagnostics.Completeness,description:e.Diagnostics.Skip_type_checking_all_d_ts_files,defaultValueDescription:"false"},{name:"allowUnusedLabels",type:"boolean",affectsBindDiagnostics:!0,affectsSemanticDiagnostics:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Disable_error_reporting_for_unused_labels,defaultValueDescription:"undefined"},{name:"allowUnreachableCode",type:"boolean",affectsBindDiagnostics:!0,affectsSemanticDiagnostics:!0,category:e.Diagnostics.Type_Checking,description:e.Diagnostics.Disable_error_reporting_for_unreachable_code,defaultValueDescription:"undefined"},{name:"suppressExcessPropertyErrors",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Backwards_Compatibility,description:e.Diagnostics.Disable_reporting_of_excess_property_errors_during_the_creation_of_object_literals,defaultValueDescription:"false"},{name:"suppressImplicitAnyIndexErrors",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Backwards_Compatibility,description:e.Diagnostics.Suppress_noImplicitAny_errors_when_indexing_objects_that_lack_index_signatures,defaultValueDescription:"false"},{name:"forceConsistentCasingInFileNames",type:"boolean",affectsModuleResolution:!0,category:e.Diagnostics.Interop_Constraints,description:e.Diagnostics.Ensure_that_casing_is_correct_in_imports,defaultValueDescription:"false"},{name:"maxNodeModuleJsDepth",type:"number",affectsModuleResolution:!0,category:e.Diagnostics.JavaScript_Support,description:e.Diagnostics.Specify_the_maximum_folder_depth_used_for_checking_JavaScript_files_from_node_modules_Only_applicable_with_allowJs,defaultValueDescription:"0"},{name:"noStrictGenericChecks",type:"boolean",affectsSemanticDiagnostics:!0,category:e.Diagnostics.Backwards_Compatibility,description:e.Diagnostics.Disable_strict_checking_of_generic_signatures_in_function_types,defaultValueDescription:"false"},{name:"useDefineForClassFields",type:"boolean",affectsSemanticDiagnostics:!0,affectsEmit:!0,category:e.Diagnostics.Language_and_Environment,description:e.Diagnostics.Emit_ECMAScript_standard_compliant_class_fields,defaultValueDescription:"false"},{name:"keyofStringsOnly",type:"boolean",category:e.Diagnostics.Backwards_Compatibility,description:e.Diagnostics.Make_keyof_only_return_strings_instead_of_string_numbers_or_symbols_Legacy_option,defaultValueDescription:"false"},{name:"plugins",type:"list",isTSConfigOnly:!0,element:{name:"plugin",type:"object"},description:e.Diagnostics.List_of_language_service_plugins,category:e.Diagnostics.Editor_Support}];function s(n){var t=new e.Map,r=new e.Map;return e.forEach(n,function(e){t.set(e.name.toLowerCase(),e),e.shortName&&r.set(e.shortName,e.name)}),{optionsNameMap:t,shortOptionNames:r}}function c(){return r||(r=s(e.optionDeclarations))}e.optionDeclarations=a(a([],e.commonOptionsWithBuild,!0),i,!0),e.semanticDiagnosticsOptionDeclarations=e.optionDeclarations.filter(function(e){return!!e.affectsSemanticDiagnostics}),e.affectsEmitOptionDeclarations=e.optionDeclarations.filter(function(e){return!!e.affectsEmit}),e.moduleResolutionOptionDeclarations=e.optionDeclarations.filter(function(e){return!!e.affectsModuleResolution}),e.sourceFileAffectingCompilerOptions=e.optionDeclarations.filter(function(e){return!!e.affectsSourceFile||!!e.affectsModuleResolution||!!e.affectsBindDiagnostics}),e.optionsAffectingProgramStructure=e.optionDeclarations.filter(function(e){return!!e.affectsProgramStructure}),e.transpileOptionValueCompilerOptions=e.optionDeclarations.filter(function(n){return e.hasProperty(n,"transpileOptionValue")}),e.optionsForBuild=[{name:"verbose",shortName:"v",category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Enable_verbose_logging,type:"boolean"},{name:"dry",shortName:"d",category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Show_what_would_be_built_or_deleted_if_specified_with_clean,type:"boolean"},{name:"force",shortName:"f",category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Build_all_projects_including_those_that_appear_to_be_up_to_date,type:"boolean"},{name:"clean",category:e.Diagnostics.Command_line_Options,description:e.Diagnostics.Delete_the_outputs_of_all_projects,type:"boolean"}],e.buildOpts=a(a([],e.commonOptionsWithBuild,!0),e.optionsForBuild,!0),e.typeAcquisitionDeclarations=[{name:"enableAutoDiscovery",type:"boolean"},{name:"enable",type:"boolean"},{name:"include",type:"list",element:{name:"include",type:"string"}},{name:"exclude",type:"list",element:{name:"exclude",type:"string"}},{name:"disableFilenameBasedTypeAcquisition",type:"boolean"}],e.createOptionNameMap=s,e.getOptionsNameMap=c;var l,u={diagnostic:e.Diagnostics.Compiler_option_0_may_only_be_used_with_build,getOptionsNameMap:S};function d(e){return e&&void 0!==e.enableAutoDiscovery&&void 0===e.enable?{enable:e.enableAutoDiscovery,include:e.include||[],exclude:e.exclude||[]}:e}function p(n){return f(n,e.createCompilerDiagnostic)}function f(n,t){var r=e.arrayFrom(n.type.keys()).map(function(e){return"'"+e+"'"}).join(", ");return t(e.Diagnostics.Argument_for_0_option_must_be_Colon_1,"--"+n.name,r)}function m(n,t,r){return Be(n,e.trimString(t||""),r)}function _(n,t,r){if(void 0===t&&(t=""),t=e.trimString(t),!e.startsWith(t,"-")){if(""===t)return[];var i=t.split(",");switch(n.element.type){case"number":return e.mapDefined(i,function(e){return Ge(n.element,parseInt(e),r)});case"string":return e.mapDefined(i,function(e){return Ge(n.element,e||"",r)});default:return e.mapDefined(i,function(e){return m(n.element,e,r)})}}}function g(e){return e.name}function y(n,t,r,i){var a;if(null===(a=t.alternateMode)||void 0===a?void 0:a.getOptionsNameMap().optionsNameMap.has(n.toLowerCase()))return r(t.alternateMode.diagnostic,n);var o=e.getSpellingSuggestion(n,t.optionDeclarations,g);return o?r(t.unknownDidYouMeanDiagnostic,i||n,o.name):r(t.unknownOptionDiagnostic,i||n)}function h(n,t,r){var i,a={},o=[],s=[];return c(t),{options:a,watchOptions:i,fileNames:o,errors:s};function c(t){var r=0;while(r<t.length){var c=t[r];if(r++,64===c.charCodeAt(0))l(c.slice(1));else if(45===c.charCodeAt(0)){var u=c.slice(45===c.charCodeAt(1)?2:1),d=T(n.getOptionsNameMap,u,!0);if(d)r=v(t,r,n,d,a,s);else{var p=T(V.getOptionsNameMap,u,!0);p?r=v(t,r,V,p,i||(i={}),s):s.push(y(u,n,e.createCompilerDiagnostic,c))}}else o.push(c)}}function l(n){var t=R(n,r||function(n){return e.sys.readFile(n)});if(e.isString(t)){var i=[],a=0;while(1){while(a<t.length&&t.charCodeAt(a)<=32)a++;if(a>=t.length)break;var o=a;if(34===t.charCodeAt(o)){a++;while(a<t.length&&34!==t.charCodeAt(a))a++;a<t.length?(i.push(t.substring(o+1,a)),a++):s.push(e.createCompilerDiagnostic(e.Diagnostics.Unterminated_quoted_string_in_response_file_0,n))}else{while(t.charCodeAt(a)>32)a++;i.push(t.substring(o,a))}}c(i)}else s.push(t)}}function v(n,t,r,i,a,o){if(i.isTSConfigOnly){var s=n[t];"null"===s?(a[i.name]=void 0,t++):"boolean"===i.type?"false"===s?(a[i.name]=Ge(i,!1,o),t++):("true"===s&&t++,o.push(e.createCompilerDiagnostic(e.Diagnostics.Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_false_or_null_on_command_line,i.name))):(o.push(e.createCompilerDiagnostic(e.Diagnostics.Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_null_on_command_line,i.name)),s&&!e.startsWith(s,"-")&&t++)}else if(n[t]||"boolean"===i.type||o.push(e.createCompilerDiagnostic(r.optionTypeMismatchDiagnostic,i.name,X(i))),"null"!==n[t])switch(i.type){case"number":a[i.name]=Ge(i,parseInt(n[t]),o),t++;break;case"boolean":s=n[t];a[i.name]=Ge(i,"false"!==s,o),"false"!==s&&"true"!==s||t++;break;case"string":a[i.name]=Ge(i,n[t]||"",o),t++;break;case"list":var c=_(i,n[t],o);a[i.name]=c||[],c&&t++;break;default:a[i.name]=m(i,n[t],o),t++;break}else a[i.name]=void 0,t++;return t}function b(n,t){return h(e.compilerOptionsDidYouMeanDiagnostics,n,t)}function E(e,n){return T(c,e,n)}function T(e,n,t){void 0===t&&(t=!1),n=n.toLowerCase();var r=e(),i=r.optionsNameMap,a=r.shortOptionNames;if(t){var o=a.get(n);void 0!==o&&(n=o)}return i.get(n)}function S(){return l||(l=s(e.buildOpts))}e.defaultInitCompilerOptions={module:e.ModuleKind.CommonJS,target:1,strict:!0,esModuleInterop:!0,forceConsistentCasingInFileNames:!0,skipLibCheck:!0},e.convertEnableAutoDiscoveryToEnable=d,e.createCompilerDiagnosticForInvalidCustomType=p,e.parseCustomTypeOption=m,e.parseListTypeOption=_,e.parseCommandLineWorker=h,e.compilerOptionsDidYouMeanDiagnostics={alternateMode:u,getOptionsNameMap:c,optionDeclarations:e.optionDeclarations,unknownOptionDiagnostic:e.Diagnostics.Unknown_compiler_option_0,unknownDidYouMeanDiagnostic:e.Diagnostics.Unknown_compiler_option_0_Did_you_mean_1,optionTypeMismatchDiagnostic:e.Diagnostics.Compiler_option_0_expects_an_argument},e.parseCommandLine=b,e.getOptionFromName=E;var x={diagnostic:e.Diagnostics.Compiler_option_0_may_not_be_used_with_build,getOptionsNameMap:c},L={alternateMode:x,getOptionsNameMap:S,optionDeclarations:e.buildOpts,unknownOptionDiagnostic:e.Diagnostics.Unknown_build_option_0,unknownDidYouMeanDiagnostic:e.Diagnostics.Unknown_build_option_0_Did_you_mean_1,optionTypeMismatchDiagnostic:e.Diagnostics.Build_option_0_requires_a_value_of_type_1};function D(n){var t=h(L,n),r=t.options,i=t.watchOptions,a=t.fileNames,o=t.errors,s=r;return 0===a.length&&a.push("."),s.clean&&s.force&&o.push(e.createCompilerDiagnostic(e.Diagnostics.Options_0_and_1_cannot_be_combined,"clean","force")),s.clean&&s.verbose&&o.push(e.createCompilerDiagnostic(e.Diagnostics.Options_0_and_1_cannot_be_combined,"clean","verbose")),s.clean&&s.watch&&o.push(e.createCompilerDiagnostic(e.Diagnostics.Options_0_and_1_cannot_be_combined,"clean","watch")),s.watch&&s.dry&&o.push(e.createCompilerDiagnostic(e.Diagnostics.Options_0_and_1_cannot_be_combined,"watch","dry")),{buildOptions:s,watchOptions:i,projects:a,errors:o}}function A(n){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];var i=e.createCompilerDiagnostic.apply(void 0,arguments);return i.messageText}function C(n,t,r,i,a,o){var s=R(n,function(e){return r.readFile(e)});if(e.isString(s)){var c=e.parseJsonText(n,s),l=r.getCurrentDirectory();return c.path=e.toPath(n,l,e.createGetCanonicalFileName(r.useCaseSensitiveFileNames)),c.resolvedPath=c.path,c.originalFileName=c.fileName,ue(c,r,e.getNormalizedAbsolutePath(e.getDirectoryPath(n),l),t,e.getNormalizedAbsolutePath(n,l),void 0,o,i,a)}r.onUnRecoverableConfigFileDiagnostic(s)}function I(n,t){var r=R(n,t);return e.isString(r)?N(n,r):{config:{},error:r}}function N(n,t){var r=e.parseJsonText(n,t);return{config:z(r,r.parseDiagnostics,!1,void 0),error:r.parseDiagnostics.length?r.parseDiagnostics[0]:void 0}}function k(n,t){var r=R(n,t);return e.isString(r)?e.parseJsonText(n,r):{fileName:n,parseDiagnostics:[r]}}function R(n,t){var r;try{r=t(n)}catch(i){return e.createCompilerDiagnostic(e.Diagnostics.Cannot_read_file_0_Colon_1,n,i.message)}return void 0===r?e.createCompilerDiagnostic(e.Diagnostics.Cannot_read_file_0,n):r}function O(n){return e.arrayToMap(n,g)}e.parseBuildCommand=D,e.getDiagnosticText=A,e.getParsedCommandLineOfConfigFile=C,e.readConfigFile=I,e.parseConfigFileTextToJson=N,e.readJsonConfigFile=k,e.tryReadFile=R;var w,P={optionDeclarations:e.typeAcquisitionDeclarations,unknownOptionDiagnostic:e.Diagnostics.Unknown_type_acquisition_option_0,unknownDidYouMeanDiagnostic:e.Diagnostics.Unknown_type_acquisition_option_0_Did_you_mean_1};function M(){return w||(w=s(e.optionsForWatch))}var F,G,B,U,V={getOptionsNameMap:M,optionDeclarations:e.optionsForWatch,unknownOptionDiagnostic:e.Diagnostics.Unknown_watch_option_0,unknownDidYouMeanDiagnostic:e.Diagnostics.Unknown_watch_option_0_Did_you_mean_1,optionTypeMismatchDiagnostic:e.Diagnostics.Watch_option_0_requires_a_value_of_type_1};function j(){return F||(F=O(e.optionDeclarations))}function K(){return G||(G=O(e.optionsForWatch))}function H(){return B||(B=O(e.typeAcquisitionDeclarations))}function W(){return void 0===U&&(U={name:void 0,type:"object",elementOptions:O([{name:"compilerOptions",type:"object",elementOptions:j(),extraKeyDiagnostics:e.compilerOptionsDidYouMeanDiagnostics},{name:"watchOptions",type:"object",elementOptions:K(),extraKeyDiagnostics:V},{name:"typingOptions",type:"object",elementOptions:H(),extraKeyDiagnostics:P},{name:"typeAcquisition",type:"object",elementOptions:H(),extraKeyDiagnostics:P},{name:"extends",type:"string",category:e.Diagnostics.File_Management},{name:"references",type:"list",element:{name:"references",type:"object"},category:e.Diagnostics.Projects},{name:"files",type:"list",element:{name:"files",type:"string"},category:e.Diagnostics.File_Management},{name:"include",type:"list",element:{name:"include",type:"string"},category:e.Diagnostics.File_Management,defaultValueDescription:e.Diagnostics.if_files_is_specified_otherwise_Asterisk_Asterisk_Slash_Asterisk},{name:"exclude",type:"list",element:{name:"exclude",type:"string"},category:e.Diagnostics.File_Management,defaultValueDescription:e.Diagnostics.node_modules_bower_components_jspm_packages_plus_the_value_of_outDir_if_one_is_specified},e.compileOnSaveCommandLineOption])}),U}function z(n,t,r,i){var a,o=null===(a=n.statements[0])||void 0===a?void 0:a.expression,s=r?W():void 0;if(o&&203!==o.kind){if(t.push(e.createDiagnosticForNodeInSourceFile(n,o,e.Diagnostics.The_root_value_of_a_0_file_must_be_an_object,"jsconfig.json"===e.getBaseFileName(n.fileName)?"jsconfig.json":"tsconfig.json")),e.isArrayLiteralExpression(o)){var c=e.find(o.elements,e.isObjectLiteralExpression);if(c)return q(n,c,t,!0,s,i)}return{}}return q(n,o,t,!0,s,i)}function J(e,n){var t;return q(e,null===(t=e.statements[0])||void 0===t?void 0:t.expression,n,!0,void 0,void 0)}function q(n,t,r,i,o,s){return t?d(t,o):i?{}:void 0;function c(e){return o&&o.elementOptions===e}function l(t,a,o,l){for(var u=i?{}:void 0,f=function(t){if(291!==t.kind)return r.push(e.createDiagnosticForNodeInSourceFile(n,t,e.Diagnostics.Property_assignment_expected)),"continue";t.questionToken&&r.push(e.createDiagnosticForNodeInSourceFile(n,t.questionToken,e.Diagnostics.The_0_modifier_can_only_be_used_in_TypeScript_files,"?")),p(t.name)||r.push(e.createDiagnosticForNodeInSourceFile(n,t.name,e.Diagnostics.String_literal_with_double_quotes_expected));var f=e.isComputedNonLiteralName(t.name)?void 0:e.getTextOfPropertyName(t.name),m=f&&e.unescapeLeadingUnderscores(f),_=m&&a?a.get(m):void 0;m&&o&&!_&&(a?r.push(y(m,o,function(r,i,a){return e.createDiagnosticForNodeInSourceFile(n,t.name,r,i,a)})):r.push(e.createDiagnosticForNodeInSourceFile(n,t.name,o.unknownOptionDiagnostic,m)));var g=d(t.initializer,_);if("undefined"!==typeof m&&(i&&(u[m]=g),s&&(l||c(a)))){var h=Y(_,g);l?h&&s.onSetValidOptionKeyValueInParent(l,_,g):c(a)&&(h?s.onSetValidOptionKeyValueInRoot(m,t.name,g,t.initializer):_||s.onSetUnknownOptionKeyValueInRoot(m,t.name,g,t.initializer))}},m=0,_=t.properties;m<_.length;m++){var g=_[m];f(g)}return u}function u(n,t){if(i)return e.filter(n.map(function(e){return d(e,t)}),function(e){return void 0!==e});n.forEach(function(e){return d(e,t)})}function d(t,i){var o;switch(t.kind){case 110:return v(i&&"boolean"!==i.type),h(!0);case 95:return v(i&&"boolean"!==i.type),h(!1);case 104:return v(i&&"extends"===i.name),h(null);case 10:p(t)||r.push(e.createDiagnosticForNodeInSourceFile(n,t,e.Diagnostics.String_literal_with_double_quotes_expected)),v(i&&e.isString(i.type)&&"string"!==i.type);var s=t.text;if(i&&!e.isString(i.type)){var c=i;c.type.has(s.toLowerCase())||(r.push(f(c,function(r,i,a){return e.createDiagnosticForNodeInSourceFile(n,t,r,i,a)})),o=!0)}return h(s);case 8:return v(i&&"number"!==i.type),h(Number(t.text));case 217:if(40!==t.operator||8!==t.operand.kind)break;return v(i&&"number"!==i.type),h(-Number(t.operand.text));case 203:v(i&&"object"!==i.type);var d=t;if(i){var m=i,_=m.elementOptions,g=m.extraKeyDiagnostics,y=m.name;return h(l(d,_,g,y))}return h(l(d,void 0,void 0,void 0));case 202:return v(i&&"list"!==i.type),h(u(t.elements,i&&i.element))}return void(i?v(!0):r.push(e.createDiagnosticForNodeInSourceFile(n,t,e.Diagnostics.Property_value_can_only_be_string_literal_numeric_literal_true_false_null_object_literal_or_array_literal)));function h(s){var c;if(!o){var l=null===(c=null===i||void 0===i?void 0:i.extraValidation)||void 0===c?void 0:c.call(i,s);if(l)return void r.push(e.createDiagnosticForNodeInSourceFile.apply(void 0,a([n,t],l,!1)))}return s}function v(a){a&&(r.push(e.createDiagnosticForNodeInSourceFile(n,t,e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,i.name,X(i))),o=!0)}}function p(t){return e.isStringLiteral(t)&&e.isStringDoubleQuoted(t,n)}}function X(n){return"list"===n.type?"Array":e.isString(n.type)?n.type:"string"}function Y(n,t){if(n){if(pe(t))return!0;if("list"===n.type)return e.isArray(t);var r=e.isString(n.type)?n.type:"string";return typeof t===r}return!1}function Q(n,t,r){var i,a,s,c=e.createGetCanonicalFileName(r.useCaseSensitiveFileNames),l=e.map(e.filter(n.fileNames,(null===(a=null===(i=n.options.configFile)||void 0===i?void 0:i.configFileSpecs)||void 0===a?void 0:a.validatedIncludeSpecs)?ee(t,n.options.configFile.configFileSpecs.validatedIncludeSpecs,n.options.configFile.configFileSpecs.validatedExcludeSpecs,r):e.returnTrue),function(n){return e.getRelativePathFromFile(e.getNormalizedAbsolutePath(t,r.getCurrentDirectory()),e.getNormalizedAbsolutePath(n,r.getCurrentDirectory()),c)}),u=re(n.options,{configFilePath:e.getNormalizedAbsolutePath(t,r.getCurrentDirectory()),useCaseSensitiveFileNames:r.useCaseSensitiveFileNames}),d=n.watchOptions&&ie(n.watchOptions),p=o(o({compilerOptions:o(o({},Z(u)),{showConfig:void 0,configFile:void 0,configFilePath:void 0,help:void 0,init:void 0,listFiles:void 0,listEmittedFiles:void 0,project:void 0,build:void 0,version:void 0}),watchOptions:d&&Z(d),references:e.map(n.projectReferences,function(e){return o(o({},e),{path:e.originalPath?e.originalPath:"",originalPath:void 0})}),files:e.length(l)?l:void 0},(null===(s=n.options.configFile)||void 0===s?void 0:s.configFileSpecs)?{include:$(n.options.configFile.configFileSpecs.validatedIncludeSpecs),exclude:n.options.configFile.configFileSpecs.validatedExcludeSpecs}:{}),{compileOnSave:!!n.compileOnSave||void 0});return p}function Z(n){return o({},e.arrayFrom(n.entries()).reduce(function(e,n){var t;return o(o({},e),(t={},t[n[0]]=n[1],t))},{}))}function $(n){if(e.length(n)){if(1!==e.length(n))return n;if("**/*"!==n[0])return n}}function ee(n,t,r,i){if(!t)return e.returnTrue;var a=e.getFileMatcherPatterns(n,r,t,i.useCaseSensitiveFileNames,i.getCurrentDirectory()),o=a.excludePattern&&e.getRegexFromPattern(a.excludePattern,i.useCaseSensitiveFileNames),s=a.includeFilePattern&&e.getRegexFromPattern(a.includeFilePattern,i.useCaseSensitiveFileNames);return s?o?function(e){return!(s.test(e)&&!o.test(e))}:function(e){return!s.test(e)}:o?function(e){return o.test(e)}:e.returnTrue}function ne(e){return"string"===e.type||"number"===e.type||"boolean"===e.type||"object"===e.type?void 0:"list"===e.type?ne(e.element):e.type}function te(n,t){return e.forEachEntry(t,function(e,t){if(e===n)return t})}function re(e,n){return ae(e,c(),n)}function ie(e){return ae(e,M())}function ae(n,t,r){var i=t.optionsNameMap,a=new e.Map,o=r&&e.createGetCanonicalFileName(r.useCaseSensitiveFileNames),s=function(t){if(e.hasProperty(n,t)){if(i.has(t)&&(i.get(t).category===e.Diagnostics.Command_line_Options||i.get(t).category===e.Diagnostics.Output_Formatting))return"continue";var s=n[t],c=i.get(t.toLowerCase());if(c){var l=ne(c);l?"list"===c.type?a.set(t,s.map(function(e){return te(e,l)})):a.set(t,te(s,l)):r&&c.isFilePath?a.set(t,e.getRelativePathFromFile(r.configFilePath,e.getNormalizedAbsolutePath(s,e.getDirectoryPath(r.configFilePath)),o)):a.set(t,s)}}};for(var c in n)s(c);return a}function oe(n,t,r){var i=e.extend(n,e.defaultInitCompilerOptions),a=re(i);return l();function o(n){switch(n.type){case"number":return 1;case"boolean":return!0;case"string":return n.isFilePath?"./":"";case"list":return[];case"object":return{};default:var t=n.type.keys().next();return t.done?e.Debug.fail("Expected 'option.type' to have entries."):t.value}}function s(e){return Array(e+1).join(" ")}function c(n){var t=n.category,r=n.name,i=n.isCommandLineOnly,o=[e.Diagnostics.Command_line_Options,e.Diagnostics.Editor_Support,e.Diagnostics.Compiler_Diagnostics,e.Diagnostics.Backwards_Compatibility,e.Diagnostics.Watch_and_Build_Modes,e.Diagnostics.Output_Formatting];return!i&&void 0!==t&&(!o.includes(t)||a.has(r))}function l(){for(var n=e.createMultiMap(),i=0,l=e.optionDeclarations;i<l.length;i++){var u=l[i],d=u.category;c(u)&&n.add(e.getLocaleSpecificMessage(d),u)}var p=0,f=0,m=[];n.forEach(function(n,t){0!==m.length&&m.push({value:""}),m.push({value:"/* "+t+" */"});for(var r=0,i=n;r<i.length;r++){var s=i[r],c=void 0;c=a.has(s.name)?'"'+s.name+'": '+JSON.stringify(a.get(s.name))+((f+=1)===a.size?"":","):'// "'+s.name+'": '+JSON.stringify(o(s))+",",m.push({value:c,description:"/* "+(s.description&&e.getLocaleSpecificMessage(s.description)||s.name)+" */"}),p=Math.max(c.length,p)}});var _=s(2),g=[];g.push("{"),g.push(_+'"compilerOptions": {'),g.push(""+_+_+"/* "+e.getLocaleSpecificMessage(e.Diagnostics.Visit_https_Colon_Slash_Slashaka_ms_Slashtsconfig_json_to_read_more_about_this_file)+" */"),g.push("");for(var y=0,h=m;y<h.length;y++){var v=h[y],b=v.value,E=v.description,T=void 0===E?"":E;g.push(b&&""+_+_+b+(T&&s(p-b.length+2)+T))}if(t.length){g.push(_+"},"),g.push(_+'"files": [');for(var S=0;S<t.length;S++)g.push(""+_+_+JSON.stringify(t[S])+(S===t.length-1?"":","));g.push(_+"]")}else g.push(_+"}");return g.push("}"),g.join(r)+r}}function se(n,t){var r={},i=c().optionsNameMap;for(var a in n)e.hasProperty(n,a)&&(r[a]=ce(i.get(a.toLowerCase()),n[a],t));return r.configFilePath&&(r.configFilePath=t(r.configFilePath)),r}function ce(e,n,t){if(e&&!pe(n))if("list"===e.type){var r=n;if(e.element.isFilePath&&r.length)return r.map(t)}else if(e.isFilePath)return t(n);return n}function le(e,n,t,r,i,a,o,s,c){return me(e,void 0,n,t,r,c,i,a,o,s)}function ue(e,n,t,r,i,a,o,s,c){return me(void 0,e,n,t,r,c,i,a,o,s)}function de(e,n){n&&Object.defineProperty(e,"configFile",{enumerable:!1,writable:!1,value:n})}function pe(e){return void 0===e||null===e}function fe(n,t){return e.getDirectoryPath(e.getNormalizedAbsolutePath(n,t))}function me(n,t,r,i,a,o,s,c,l,u){void 0===a&&(a={}),void 0===c&&(c=[]),void 0===l&&(l=[]),e.Debug.assert(void 0===n&&void 0!==t||void 0!==n&&void 0===t);var d=[],p=Ee(n,t,r,i,s,c,d,u),f=p.raw,m=e.extend(a,p.options||{}),_=o&&p.watchOptions?e.extend(o,p.watchOptions):p.watchOptions||o;m.configFilePath=s&&e.normalizeSlashes(s);var g=h();t&&(t.configFileSpecs=g),de(m,t);var y=e.normalizePath(s?fe(s,i):i);return{options:m,watchOptions:_,fileNames:v(y),projectReferences:b(y),typeAcquisition:p.typeAcquisition||ke(),raw:f,errors:d,wildcardDirectories:Ye(g,y,r.useCaseSensitiveFileNames),compileOnSave:!!f.compileOnSave};function h(){var n=S("references",function(e){return"object"===typeof e},"object"),r=E(T("files"));if(r){var i="no-prop"===n||e.isArray(n)&&0===n.length,a=e.hasProperty(f,"extends");if(0===r.length&&i&&!a)if(t){var o=s||"tsconfig.json",c=e.Diagnostics.The_files_list_in_config_file_0_is_empty,l=e.firstDefined(e.getTsConfigPropArray(t,"files"),function(e){return e.initializer}),u=l?e.createDiagnosticForNodeInSourceFile(t,l,c,o):e.createCompilerDiagnostic(c,o);d.push(u)}else x(e.Diagnostics.The_files_list_in_config_file_0_is_empty,s||"tsconfig.json")}var p,m,_=E(T("include")),g=T("exclude"),y=E(g);if("no-prop"===g&&f.compilerOptions){var h=f.compilerOptions.outDir,v=f.compilerOptions.declarationDir;(h||v)&&(y=[h,v].filter(function(e){return!!e}))}return void 0===r&&void 0===_&&(_=["**/*"]),_&&(p=qe(_,d,!0,t,"include")),y&&(m=qe(y,d,!1,t,"exclude")),{filesSpecs:r,includeSpecs:_,excludeSpecs:y,validatedFilesSpec:e.filter(r,e.isString),validatedIncludeSpecs:p,validatedExcludeSpecs:m,pathPatterns:void 0}}function v(e){var n=Ke(g,e,m,r,l);return ye(n,he(f),c)&&d.push(ge(g,s)),n}function b(n){var t,r=S("references",function(e){return"object"===typeof e},"object");if(e.isArray(r))for(var i=0,a=r;i<a.length;i++){var o=a[i];"string"!==typeof o.path?x(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,"reference.path","string"):(t||(t=[])).push({path:e.getNormalizedAbsolutePath(o.path,n),originalPath:o.path,prepend:o.prepend,circular:o.circular})}return t}function E(n){return e.isArray(n)?n:void 0}function T(n){return S(n,e.isString,"string")}function S(n,r,i){if(e.hasProperty(f,n)&&!pe(f[n])){if(e.isArray(f[n])){var a=f[n];return t||e.every(a,r)||d.push(e.createCompilerDiagnostic(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,n,i)),a}return x(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,n,"Array"),"not-array"}return"no-prop"}function x(n,r,i){t||d.push(e.createCompilerDiagnostic(n,r,i))}}function _e(n){return n.code===e.Diagnostics.No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2.code}function ge(n,t){var r=n.includeSpecs,i=n.excludeSpecs;return e.createCompilerDiagnostic(e.Diagnostics.No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2,t||"tsconfig.json",JSON.stringify(r||[]),JSON.stringify(i||[]))}function ye(e,n,t){return 0===e.length&&n&&(!t||0===t.length)}function he(n){return!e.hasProperty(n,"files")&&!e.hasProperty(n,"references")}function ve(n,t,r,i,a){var o=i.length;return ye(n,a)?i.push(ge(r,t)):e.filterMutate(i,function(e){return!_e(e)}),o!==i.length}function be(e){return!!e.options}function Ee(n,t,r,i,o,s,c,l){var u;i=e.normalizeSlashes(i);var d=e.getNormalizedAbsolutePath(o||"",i);if(s.indexOf(d)>=0)return c.push(e.createCompilerDiagnostic(e.Diagnostics.Circularity_detected_while_resolving_configuration_Colon_0,a(a([],s,!0),[d],!1).join(" -> "))),{raw:n||J(t,c)};var p=n?Te(n,r,i,o,c):Se(t,r,i,o,c);if((null===(u=p.options)||void 0===u?void 0:u.paths)&&(p.options.pathsBasePath=i),p.extendedConfigPath){s=s.concat([d]);var f=Le(t,p.extendedConfigPath,r,s,c,l);if(f&&be(f)){var m,_=f.raw,g=p.raw,y=function(n){!g[n]&&_[n]&&(g[n]=e.map(_[n],function(n){return e.isRootedDiskPath(n)?n:e.combinePaths(m||(m=e.convertToRelativePath(e.getDirectoryPath(p.extendedConfigPath),i,e.createGetCanonicalFileName(r.useCaseSensitiveFileNames))),n)}))};y("include"),y("exclude"),y("files"),void 0===g.compileOnSave&&(g.compileOnSave=_.compileOnSave),p.options=e.assign({},f.options,p.options),p.watchOptions=p.watchOptions&&f.watchOptions?e.assign({},f.watchOptions,p.watchOptions):p.watchOptions||f.watchOptions}}return p}function Te(n,t,r,i,a){e.hasProperty(n,"excludes")&&a.push(e.createCompilerDiagnostic(e.Diagnostics.Unknown_option_excludes_Did_you_mean_exclude));var o,s=Ne(n.compilerOptions,r,a,i),c=Re(n.typeAcquisition||n.typingOptions,r,a,i),l=Oe(n.watchOptions,r,a);if(n.compileOnSave=De(n,r,a),n.extends)if(e.isString(n.extends)){var u=i?fe(i,r):r;o=xe(n.extends,t,u,a,e.createCompilerDiagnostic)}else a.push(e.createCompilerDiagnostic(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,"extends","string"));return{raw:n,options:s,watchOptions:l,typeAcquisition:c,extendedConfigPath:o}}function Se(n,t,r,i,a){var o,s,c,l,u=Ie(i),d={onSetValidOptionKeyValueInParent:function(n,t,a){var l;switch(n){case"compilerOptions":l=u;break;case"watchOptions":l=c||(c={});break;case"typeAcquisition":l=o||(o=ke(i));break;case"typingOptions":l=s||(s=ke(i));break;default:e.Debug.fail("Unknown option")}l[t.name]=Me(t,r,a)},onSetValidOptionKeyValueInRoot:function(o,s,c,u){switch(o){case"extends":var d=i?fe(i,r):r;return void(l=xe(c,t,d,a,function(t,r){return e.createDiagnosticForNodeInSourceFile(n,u,t,r)}))}},onSetUnknownOptionKeyValueInRoot:function(t,r,i,o){"excludes"===t&&a.push(e.createDiagnosticForNodeInSourceFile(n,r,e.Diagnostics.Unknown_option_excludes_Did_you_mean_exclude))}},p=z(n,a,!0,d);return o||(o=s?void 0!==s.enableAutoDiscovery?{enable:s.enableAutoDiscovery,include:s.include,exclude:s.exclude}:s:ke(i)),{raw:p,options:u,watchOptions:c,typeAcquisition:o,extendedConfigPath:l}}function xe(n,t,r,i,a){if(n=e.normalizeSlashes(n),e.isRootedDiskPath(n)||e.startsWith(n,"./")||e.startsWith(n,"../")){var o=e.getNormalizedAbsolutePath(n,r);return t.fileExists(o)||e.endsWith(o,".json")||(o+=".json",t.fileExists(o))?o:void i.push(a(e.Diagnostics.File_0_not_found,n))}var s=e.nodeModuleNameResolver(n,e.combinePaths(r,"tsconfig.json"),{moduleResolution:e.ModuleResolutionKind.NodeJs},t,void 0,void 0,!0);if(s.resolvedModule)return s.resolvedModule.resolvedFileName;i.push(a(e.Diagnostics.File_0_not_found,n))}function Le(n,t,r,i,a,o){var s,c,l,u,d=r.useCaseSensitiveFileNames?t:e.toFileNameLowerCase(t);if(o&&(c=o.get(d))?(l=c.extendedResult,u=c.extendedConfig):(l=k(t,function(e){return r.readFile(e)}),l.parseDiagnostics.length||(u=Ee(void 0,l,r,e.getDirectoryPath(t),e.getBaseFileName(t),i,a,o)),o&&o.set(d,{extendedResult:l,extendedConfig:u})),n&&(n.extendedSourceFiles=[l.fileName],l.extendedSourceFiles&&(s=n.extendedSourceFiles).push.apply(s,l.extendedSourceFiles)),!l.parseDiagnostics.length)return u;a.push.apply(a,l.parseDiagnostics)}function De(n,t,r){if(!e.hasProperty(n,e.compileOnSaveCommandLineOption.name))return!1;var i=Pe(e.compileOnSaveCommandLineOption,n.compileOnSave,t,r);return"boolean"===typeof i&&i}function Ae(e,n,t){var r=[],i=Ne(e,n,r,t);return{options:i,errors:r}}function Ce(e,n,t){var r=[],i=Re(e,n,r,t);return{options:i,errors:r}}function Ie(n){var t=n&&"jsconfig.json"===e.getBaseFileName(n)?{allowJs:!0,maxNodeModuleJsDepth:2,allowSyntheticDefaultImports:!0,skipLibCheck:!0,noEmit:!0}:{};return t}function Ne(n,t,r,i){var a=Ie(i);return we(j(),n,t,a,e.compilerOptionsDidYouMeanDiagnostics,r),i&&(a.configFilePath=e.normalizeSlashes(i)),a}function ke(n){return{enable:!!n&&"jsconfig.json"===e.getBaseFileName(n),include:[],exclude:[]}}function Re(e,n,t,r){var i=ke(r),a=d(e);return we(H(),a,n,i,P,t),i}function Oe(e,n,t){return we(K(),e,n,void 0,V,t)}function we(n,t,r,i,a,o){if(t){for(var s in t){var c=n.get(s);c?(i||(i={}))[c.name]=Pe(c,t[s],r,o):o.push(y(s,a,e.createCompilerDiagnostic))}return i}}function Pe(n,t,r,i){if(Y(n,t)){var a=n.type;if("list"===a&&e.isArray(t))return Ue(n,t,r,i);if(!e.isString(a))return Be(n,t,i);var o=Ge(n,t,i);return pe(o)?o:Fe(n,r,o)}i.push(e.createCompilerDiagnostic(e.Diagnostics.Compiler_option_0_requires_a_value_of_type_1,n.name,X(n)))}function Me(n,t,r){if(!pe(r)){if("list"===n.type){var i=n;return i.element.isFilePath||!e.isString(i.element.type)?e.filter(e.map(r,function(e){return Me(i.element,t,e)}),function(e){return!!e}):r}return e.isString(n.type)?Fe(n,t,r):n.type.get(e.isString(r)?r.toLowerCase():r)}}function Fe(n,t,r){return n.isFilePath&&(r=e.getNormalizedAbsolutePath(r,t),""===r&&(r=".")),r}function Ge(n,t,r){var i;if(!pe(t)){var a=null===(i=n.extraValidation)||void 0===i?void 0:i.call(n,t);if(!a)return t;r.push(e.createCompilerDiagnostic.apply(void 0,a))}}function Be(e,n,t){if(!pe(n)){var r=n.toLowerCase(),i=e.type.get(r);if(void 0!==i)return Ge(e,i,t);t.push(p(e))}}function Ue(n,t,r,i){return e.filter(e.map(t,function(e){return Pe(n.element,e,r,i)}),function(e){return!!e})}e.convertToObject=J,e.convertToObjectWorker=q,e.convertToTSConfig=Q,e.generateTSConfig=oe,e.convertToOptionsWithAbsolutePaths=se,e.parseJsonConfigFileContent=le,e.parseJsonSourceFileConfigFileContent=ue,e.setConfigFileInOptions=de,e.canJsonReportNoInputFiles=he,e.updateErrorForNoInputFiles=ve,e.convertCompilerOptionsFromJson=Ae,e.convertTypeAcquisitionFromJson=Ce,e.convertJsonOption=Pe;var Ve=/(^|\/)\*\*\/?$/,je=/^[^*?]*(?=\/[^/]*[*?])/;function Ke(n,t,r,i,a){void 0===a&&(a=e.emptyArray),t=e.normalizePath(t);var o,s=e.createGetCanonicalFileName(i.useCaseSensitiveFileNames),c=new e.Map,l=new e.Map,u=new e.Map,d=n.validatedFilesSpec,p=n.validatedIncludeSpecs,f=n.validatedExcludeSpecs,m=e.getSupportedExtensions(r,a),_=e.getSuppoertedExtensionsWithJsonIfResolveJsonModule(r,m);if(d)for(var g=0,y=d;g<y.length;g++){var h=y[g],v=e.getNormalizedAbsolutePath(h,t);c.set(s(v),v)}if(p&&p.length>0)for(var b=function(n){if(e.fileExtensionIs(n,".json")){if(!o){var r=p.filter(function(n){return e.endsWith(n,".json")}),a=e.map(e.getRegularExpressionsForWildcards(r,t,"files"),function(e){return"^"+e+"$"});o=a?a.map(function(n){return e.getRegexFromPattern(n,i.useCaseSensitiveFileNames)}):e.emptyArray}var d=e.findIndex(o,function(e){return e.test(n)});if(-1!==d){var f=s(n);c.has(f)||u.has(f)||u.set(f,n)}return"continue"}if(Ze(n,c,l,m,s))return"continue";$e(n,l,m,s);var _=s(n);c.has(_)||l.has(_)||l.set(_,n)},E=0,T=i.readDirectory(t,_,f,p,void 0);E<T.length;E++){v=T[E];b(v)}var S=e.arrayFrom(c.values()),x=e.arrayFrom(l.values());return S.concat(x,e.arrayFrom(u.values()))}function He(n,t,r,i,a){var o=t.validatedFilesSpec,s=t.validatedIncludeSpecs,c=t.validatedExcludeSpecs;if(!e.length(s)||!e.length(c))return!1;r=e.normalizePath(r);var l=e.createGetCanonicalFileName(i);if(o)for(var u=0,d=o;u<d.length;u++){var p=d[u];if(l(e.getNormalizedAbsolutePath(p,r))===n)return!1}return Je(n,c,i,a,r)}function We(n){var t=e.startsWith(n,"**/")?0:n.indexOf("/**/");if(-1===t)return!1;var r=e.endsWith(n,"/..")?n.length:n.lastIndexOf("/../");return r>t}function ze(n,t,r,i){return Je(n,e.filter(t,function(e){return!We(e)}),r,i)}function Je(n,t,r,i,a){var o=e.getRegularExpressionForWildcard(t,e.combinePaths(e.normalizePath(i),a),"exclude"),s=o&&e.getRegexFromPattern(o,r);return!!s&&(!!s.test(n)||!e.hasExtension(n)&&s.test(e.ensureTrailingDirectorySeparator(n)))}function qe(n,t,r,i,a){return n.filter(function(n){if(!e.isString(n))return!1;var i=Xe(n,r);return void 0!==i&&t.push(o.apply(void 0,i)),void 0===i});function o(n,t){var r=e.getTsConfigPropArrayElementValue(i,a,t);return r?e.createDiagnosticForNodeInSourceFile(i,r,n,t):e.createCompilerDiagnostic(n,t)}}function Xe(n,t){return t&&Ve.test(n)?[e.Diagnostics.File_specification_cannot_end_in_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0,n]:We(n)?[e.Diagnostics.File_specification_cannot_contain_a_parent_directory_that_appears_after_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0,n]:void 0}function Ye(n,t,r){var i=n.validatedIncludeSpecs,a=n.validatedExcludeSpecs,o=e.getRegularExpressionForWildcard(a,t,"exclude"),s=o&&new RegExp(o,r?"":"i"),c={};if(void 0!==i){for(var l=[],u=0,d=i;u<d.length;u++){var p=d[u],f=e.normalizePath(e.combinePaths(t,p));if(!s||!s.test(f)){var m=Qe(f,r);if(m){var _=m.key,g=m.flags,y=c[_];(void 0===y||y<g)&&(c[_]=g,1===g&&l.push(_))}}}for(var _ in c)if(e.hasProperty(c,_))for(var h=0,v=l;h<v.length;h++){var b=v[h];_!==b&&e.containsPath(b,_,t,!r)&&delete c[_]}}return c}function Qe(n,t){var r=je.exec(n);if(r){var i=n.indexOf("?"),a=n.indexOf("*"),o=n.lastIndexOf(e.directorySeparator);return{key:t?r[0]:e.toFileNameLowerCase(r[0]),flags:-1!==i&&i<o||-1!==a&&a<o?1:0}}if(e.isImplicitGlob(n))return{key:t?n:e.toFileNameLowerCase(n),flags:1}}function Ze(n,t,r,i,a){for(var o=e.getExtensionPriority(n,i),s=e.adjustExtensionPriority(o,i),c=0;c<s;c++){var l=i[c],u=a(e.changeExtension(n,l));if(t.has(u)||r.has(u))return!0}return!1}function $e(n,t,r,i){for(var a=e.getExtensionPriority(n,r),o=e.getNextLowestExtensionPriority(a,r),s=o;s<r.length;s++){var c=r[s],l=i(e.changeExtension(n,c));t.delete(l)}}function en(e){var n={};for(var t in e)if(e.hasOwnProperty(t)){var r=E(t);void 0!==r&&(n[t]=nn(e[t],r))}return n}function nn(n,t){switch(t.type){case"object":return"";case"string":return"";case"number":return"number"===typeof n?n:"";case"boolean":return"boolean"===typeof n?n:"";case"list":var r=t.element;return e.isArray(n)?n.map(function(e){return nn(e,r)}):"";default:return e.forEachEntry(t.type,function(e,t){if(e===n)return t})}}e.getFileNamesFromConfigSpecs=Ke,e.isExcludedFile=He,e.matchesExclude=ze,e.convertCompilerOptionsForTelemetry=en}(d||(d={})),function(e){function n(n){n.trace(e.formatMessage.apply(void 0,arguments))}function t(e,n){return!!e.traceResolution&&void 0!==n.trace}function r(n,t){var r;if(t&&n){var i=n.packageJsonContent;"string"===typeof i.name&&"string"===typeof i.version&&(r={name:i.name,subModuleName:t.path.slice(n.packageDirectory.length+e.directorySeparator.length),version:i.version})}return t&&{path:t.path,extension:t.ext,packageId:r}}function i(e){return r(void 0,e)}function s(n){if(n)return e.Debug.assert(void 0===n.packageId),{path:n.path,ext:n.extension}}var c,l;function u(n){if(n)return e.Debug.assert(e.extensionIsTS(n.extension)),{fileName:n.path,packageId:n.packageId}}function d(e,n,t,r){var i;return r?((i=r.failedLookupLocations).push.apply(i,t),r):{resolvedModule:e&&{resolvedFileName:e.path,originalPath:!0===e.originalPath?void 0:e.originalPath,extension:e.extension,isExternalLibraryImport:n,packageId:e.packageId},failedLookupLocations:t}}function p(t,r,i,a){if(e.hasProperty(t,r)){var o=t[r];if(typeof o===i&&null!==o)return o;a.traceEnabled&&n(a.host,e.Diagnostics.Expected_type_of_0_field_in_package_json_to_be_1_got_2,r,i,null===o?"null":typeof o)}else a.traceEnabled&&n(a.host,e.Diagnostics.package_json_does_not_have_a_0_field,r)}function f(t,r,i,a){var o=p(t,r,"string",a);if(void 0!==o){if(o){var s=e.normalizePath(e.combinePaths(i,o));return a.traceEnabled&&n(a.host,e.Diagnostics.package_json_has_0_field_1_that_references_2,r,o,s),s}a.traceEnabled&&n(a.host,e.Diagnostics.package_json_had_a_falsy_0_field,r)}}function m(e,n,t){return f(e,"typings",n,t)||f(e,"types",n,t)}function _(e,n,t){return f(e,"tsconfig",n,t)}function g(e,n,t){return f(e,"main",n,t)}function y(t,r){var i=p(t,"typesVersions","object",r);if(void 0!==i)return r.traceEnabled&&n(r.host,e.Diagnostics.package_json_has_a_typesVersions_field_with_version_specific_path_mappings),i}function h(t,r){var i=y(t,r);if(void 0!==i){if(r.traceEnabled)for(var a in i)e.hasProperty(i,a)&&!e.VersionRange.tryParse(a)&&n(r.host,e.Diagnostics.package_json_has_a_typesVersions_entry_0_that_is_not_a_valid_semver_range,a);var o=v(i);if(o){var s=o.version,c=o.paths;if("object"===typeof c)return o;r.traceEnabled&&n(r.host,e.Diagnostics.Expected_type_of_0_field_in_package_json_to_be_1_got_2,"typesVersions['"+s+"']","object",typeof c)}else r.traceEnabled&&n(r.host,e.Diagnostics.package_json_does_not_have_a_typesVersions_entry_that_matches_version_0,e.versionMajorMinor)}}function v(n){for(var t in l||(l=new e.Version(e.version)),n)if(e.hasProperty(n,t)){var r=e.VersionRange.tryParse(t);if(void 0!==r&&r.test(l))return{version:t,paths:n[t]}}}function b(n,t){return n.typeRoots?n.typeRoots:(n.configFilePath?r=e.getDirectoryPath(n.configFilePath):t.getCurrentDirectory&&(r=t.getCurrentDirectory()),void 0!==r?E(r,t):void 0);var r}function E(n,t){return t.directoryExists?(e.forEachAncestorDirectory(e.normalizePath(n),function(n){var i=e.combinePaths(n,T);t.directoryExists(i)&&(r||(r=[])).push(i)}),r):[e.combinePaths(n,T)];var r}e.trace=n,e.isTraceEnabled=t,function(e){e[e["TypeScript"]=0]="TypeScript",e[e["JavaScript"]=1]="JavaScript",e[e["Json"]=2]="Json",e[e["TSConfig"]=3]="TSConfig",e[e["DtsOnly"]=4]="DtsOnly"}(c||(c={})),e.getPackageJsonTypesVersionsPaths=v,e.getEffectiveTypeRoots=b;var T=e.combinePaths("node_modules","@types");function S(r,i,a,o,s,l){var d=t(a,o);s&&(a=s.commandLine.options);var p=i?e.getDirectoryPath(i):void 0,f=p?l&&l.getOrCreateCacheForDirectory(p,s):void 0,m=f&&f.get(r);if(m)return d&&(n(o,e.Diagnostics.Resolving_type_reference_directive_0_containing_file_1,r,i),s&&n(o,e.Diagnostics.Using_compiler_options_of_project_reference_redirect_0,s.sourceFile.fileName),n(o,e.Diagnostics.Resolution_for_type_reference_directive_0_was_found_in_cache_from_location_1,r,p),L(m)),m;var _=b(a,o);d&&(void 0===i?void 0===_?n(o,e.Diagnostics.Resolving_type_reference_directive_0_containing_file_not_set_root_directory_not_set,r):n(o,e.Diagnostics.Resolving_type_reference_directive_0_containing_file_not_set_root_directory_1,r,_):void 0===_?n(o,e.Diagnostics.Resolving_type_reference_directive_0_containing_file_1_root_directory_not_set,r,i):n(o,e.Diagnostics.Resolving_type_reference_directive_0_containing_file_1_root_directory_2,r,i,_),s&&n(o,e.Diagnostics.Using_compiler_options_of_project_reference_redirect_0,s.sourceFile.fileName));var g,y=[],h={compilerOptions:a,host:o,traceEnabled:d,failedLookupLocations:y,packageJsonInfoCache:l},v=D(),E=!0;if(v||(v=A(),E=!1),v){var T=v.fileName,S=v.packageId,x=a.preserveSymlinks?T:J(T,o,d);g={primary:E,resolvedFileName:x,originalPath:T===x?void 0:T,packageId:S,isExternalLibraryImport:X(T)}}return m={resolvedTypeReferenceDirective:g,failedLookupLocations:y},null===f||void 0===f||f.set(r,m),d&&L(m),m;function L(t){var i;(null===(i=t.resolvedTypeReferenceDirective)||void 0===i?void 0:i.resolvedFileName)?t.resolvedTypeReferenceDirective.packageId?n(o,e.Diagnostics.Type_reference_directive_0_was_successfully_resolved_to_1_with_Package_ID_2_primary_Colon_3,r,t.resolvedTypeReferenceDirective.resolvedFileName,e.packageIdToString(t.resolvedTypeReferenceDirective.packageId),t.resolvedTypeReferenceDirective.primary):n(o,e.Diagnostics.Type_reference_directive_0_was_successfully_resolved_to_1_primary_Colon_2,r,t.resolvedTypeReferenceDirective.resolvedFileName,t.resolvedTypeReferenceDirective.primary):n(o,e.Diagnostics.Type_reference_directive_0_was_not_resolved,r)}function D(){if(_&&_.length)return d&&n(o,e.Diagnostics.Resolving_with_primary_search_path_0,_.join(", ")),e.firstDefined(_,function(t){var i=e.combinePaths(t,r),a=e.getDirectoryPath(i),s=e.directoryProbablyExists(a,o);return!s&&d&&n(o,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,a),u(te(c.DtsOnly,i,!s,h))});d&&n(o,e.Diagnostics.Root_directory_cannot_be_determined_skipping_primary_search_paths)}function A(){var t=i&&e.getDirectoryPath(i);if(void 0!==t){var a;if(d&&n(o,e.Diagnostics.Looking_up_in_node_modules_folder_initial_location_0,t),e.isExternalModuleNameRelative(r)){var s=e.normalizePathAndParts(e.combinePaths(t,r)).path;a=q(c.DtsOnly,s,!1,h,!0)}else{var l=ce(c.DtsOnly,r,t,h,void 0,void 0);a=l&&l.value}return u(a)}d&&n(o,e.Diagnostics.Containing_file_is_not_specified_and_root_directory_cannot_be_determined_skipping_lookup_in_node_modules_folder)}}function x(n,t){if(n.types)return n.types;var r=[];if(t.directoryExists&&t.getDirectories){var i=b(n,t);if(i)for(var a=0,o=i;a<o.length;a++){var s=o[a];if(t.directoryExists(s))for(var c=0,l=t.getDirectories(s);c<l.length;c++){var u=l[c],d=e.normalizePath(u),p=e.combinePaths(s,d,"package.json"),f=t.fileExists(p)&&null===e.readJson(p,t).typings;if(!f){var m=e.getBaseFileName(d);46!==m.charCodeAt(0)&&r.push(m)}}}}return r}function L(n){var t=new e.Map,r=new e.Map;return{getOwnMap:i,redirectsMap:r,getOrCreateMapOfCacheRedirects:s,clear:c,setOwnOptions:a,setOwnMap:o};function i(){return t}function a(e){n=e}function o(e){t=e}function s(i){if(!i)return t;var a=i.sourceFile.path,o=r.get(a);return o||(o=!n||e.optionsHaveModuleResolutionChanges(n,i.commandLine.options)?new e.Map:t,r.set(a,o)),o}function c(){t.clear(),r.clear()}}function D(n,t){var r;return{getPackageJsonInfo:i,setPackageJsonInfo:a,clear:o,entries:s};function i(i){return null===r||void 0===r?void 0:r.get(e.toPath(i,n,t))}function a(i,a){(r||(r=new e.Map)).set(e.toPath(i,n,t),a)}function o(){r=void 0}function s(){var n=null===r||void 0===r?void 0:r.entries();return n?e.arrayFrom(n):[]}}function A(e,n,t,r){var i=e.getOrCreateMapOfCacheRedirects(n),a=i.get(t);return a||(a=r(),i.set(t,a)),a}function C(n,t,r){if(n.configFile){if(0===t.redirectsMap.size)e.Debug.assert(!r||0===r.redirectsMap.size),e.Debug.assert(0===t.getOwnMap().size),e.Debug.assert(!r||0===r.getOwnMap().size),t.redirectsMap.set(n.configFile.path,t.getOwnMap()),null===r||void 0===r||r.redirectsMap.set(n.configFile.path,r.getOwnMap());else{e.Debug.assert(!r||r.redirectsMap.size>0);var i={sourceFile:n.configFile,commandLine:{options:n}};t.setOwnMap(t.getOrCreateMapOfCacheRedirects(i)),null===r||void 0===r||r.setOwnMap(r.getOrCreateMapOfCacheRedirects(i))}t.setOwnOptions(n),null===r||void 0===r||r.setOwnOptions(n)}}function I(n,t,r){return{getOrCreateCacheForDirectory:o,clear:i,update:a};function i(){r.clear()}function a(e){C(e,r)}function o(i,a){var o=e.toPath(i,n,t);return A(r,a,o,function(){return new e.Map})}}function N(n,t,r,i,a){var s=I(n,t,i||(i=L(r)));a||(a=L(r));var c=D(n,t);return o(o(o({},c),s),{getOrCreateCacheForModuleName:d,clear:l,update:u,getPackageJsonInfoCache:function(){return c}});function l(){s.clear(),a.clear(),c.clear()}function u(e){C(e,i,a)}function d(n,t){return e.Debug.assert(!e.isExternalModuleNameRelative(n)),A(a,t,n,p)}function p(){var r=new e.Map;return{get:i,set:a};function i(i){return r.get(e.toPath(i,n,t))}function a(i,a){var s=e.toPath(i,n,t);if(!r.has(s)){r.set(s,a);var c=a.resolvedModule&&(a.resolvedModule.originalPath||a.resolvedModule.resolvedFileName),l=c&&o(s,c),u=s;while(u!==l){var d=e.getDirectoryPath(u);if(d===u||r.has(d))break;r.set(d,a),u=d}}}function o(r,i){var a=e.toPath(e.getDirectoryPath(i),n,t),o=0,s=Math.min(r.length,a.length);while(o<s&&r.charCodeAt(o)===a.charCodeAt(o))o++;if(o===r.length&&(a.length===o||a[o]===e.directorySeparator))return r;var c=e.getRootLength(r);if(!(o<c)){var l=r.lastIndexOf(e.directorySeparator,o-1);if(-1!==l)return r.substr(0,Math.max(l,c))}}}}function k(e,n,t,r,i){var a=I(e,n,i||(i=L(t)));return r||(r=D(e,n)),o(o(o({},r),a),{clear:s});function s(){a.clear(),r.clear()}}function R(n,t,r){var i=e.getDirectoryPath(t),a=r&&r.getOrCreateCacheForDirectory(i);return a&&a.get(n)}function O(r,i,a,o,s,c){var l=t(a,o);c&&(a=c.commandLine.options),l&&(n(o,e.Diagnostics.Resolving_module_0_from_1,r,i),c&&n(o,e.Diagnostics.Using_compiler_options_of_project_reference_redirect_0,c.sourceFile.fileName));var u=e.getDirectoryPath(i),d=s&&s.getOrCreateCacheForDirectory(u,c),p=d&&d.get(r);if(p)l&&n(o,e.Diagnostics.Resolution_for_module_0_was_found_in_cache_from_location_1,r,u);else{var f=a.moduleResolution;switch(void 0===f?(f=e.getEmitModuleKind(a)===e.ModuleKind.CommonJS?e.ModuleResolutionKind.NodeJs:e.ModuleResolutionKind.Classic,l&&n(o,e.Diagnostics.Module_resolution_kind_is_not_specified_using_0,e.ModuleResolutionKind[f])):l&&n(o,e.Diagnostics.Explicitly_specified_module_resolution_kind_Colon_0,e.ModuleResolutionKind[f]),e.perfLogger.logStartResolveModule(r),f){case e.ModuleResolutionKind.NodeJs:p=W(r,i,a,o,s,c);break;case e.ModuleResolutionKind.Classic:p=Ee(r,i,a,o,s,c);break;default:return e.Debug.fail("Unexpected moduleResolution: "+f)}p&&p.resolvedModule&&e.perfLogger.logInfoEvent('Module "'+r+'" resolved to "'+p.resolvedModule.resolvedFileName+'"'),e.perfLogger.logStopResolveModule(p&&p.resolvedModule?""+p.resolvedModule.resolvedFileName:"null"),d&&(d.set(r,p),e.isExternalModuleNameRelative(r)||s.getOrCreateCacheForModuleName(r,c).set(u,p))}return l&&(p.resolvedModule?p.resolvedModule.packageId?n(o,e.Diagnostics.Module_name_0_was_successfully_resolved_to_1_with_Package_ID_2,r,p.resolvedModule.resolvedFileName,e.packageIdToString(p.resolvedModule.packageId)):n(o,e.Diagnostics.Module_name_0_was_successfully_resolved_to_1,r,p.resolvedModule.resolvedFileName):n(o,e.Diagnostics.Module_name_0_was_not_resolved,r)),p}function w(n,t,r,i,a){var o=P(n,t,i,a);return o?o.value:e.isExternalModuleNameRelative(t)?M(n,t,r,i,a):F(n,t,i,a)}function P(t,r,i,a){var o,s=a.compilerOptions,c=s.baseUrl,l=s.paths,u=s.configFile;if(l&&!e.pathIsRelative(r)){a.traceEnabled&&(c&&n(a.host,e.Diagnostics.baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1,c,r),n(a.host,e.Diagnostics.paths_option_is_specified_looking_for_a_pattern_to_match_module_name_0,r));var d=e.getPathsBasePath(a.compilerOptions,a.host),p=(null===u||void 0===u?void 0:u.configFileSpecs)?(o=u.configFileSpecs).pathPatterns||(o.pathPatterns=e.tryParsePatterns(l)):void 0;return fe(t,r,d,l,p,i,!1,a)}}function M(t,r,i,a,o){if(o.compilerOptions.rootDirs){o.traceEnabled&&n(o.host,e.Diagnostics.rootDirs_option_is_set_using_it_to_resolve_relative_module_name_0,r);for(var s,c,l=e.normalizePath(e.combinePaths(i,r)),u=0,d=o.compilerOptions.rootDirs;u<d.length;u++){var p=d[u],f=e.normalizePath(p);e.endsWith(f,e.directorySeparator)||(f+=e.directorySeparator);var m=e.startsWith(l,f)&&(void 0===c||c.length<f.length);o.traceEnabled&&n(o.host,e.Diagnostics.Checking_if_0_is_the_longest_matching_prefix_for_1_2,f,l,m),m&&(c=f,s=p)}if(c){o.traceEnabled&&n(o.host,e.Diagnostics.Longest_matching_prefix_for_0_is_1,l,c);var _=l.substr(c.length);o.traceEnabled&&n(o.host,e.Diagnostics.Loading_0_from_the_root_dir_1_candidate_location_2,_,c,l);var g=a(t,l,!e.directoryProbablyExists(i,o.host),o);if(g)return g;o.traceEnabled&&n(o.host,e.Diagnostics.Trying_other_entries_in_rootDirs);for(var y=0,h=o.compilerOptions.rootDirs;y<h.length;y++){p=h[y];if(p!==s){var v=e.combinePaths(e.normalizePath(p),_);o.traceEnabled&&n(o.host,e.Diagnostics.Loading_0_from_the_root_dir_1_candidate_location_2,_,p,v);var b=e.getDirectoryPath(v),E=a(t,v,!e.directoryProbablyExists(b,o.host),o);if(E)return E}}o.traceEnabled&&n(o.host,e.Diagnostics.Module_resolution_using_rootDirs_has_failed)}}}function F(t,r,i,a){var o=a.compilerOptions.baseUrl;if(o){a.traceEnabled&&n(a.host,e.Diagnostics.baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1,o,r);var s=e.normalizePath(e.combinePaths(o,r));return a.traceEnabled&&n(a.host,e.Diagnostics.Resolving_module_name_0_relative_to_base_url_1_2,r,o,s),i(t,s,!e.directoryProbablyExists(e.getDirectoryPath(s),a.host),a)}}function G(e,n,t){var r=H(e,n,t),i=r.resolvedModule,a=r.failedLookupLocations;if(!i)throw new Error("Could not resolve JS module '"+e+"' starting at '"+n+"'. Looked in: "+a.join(", "));return i.resolvedFileName}function B(e,n,t){var r=H(e,n,t).resolvedModule;return r&&r.resolvedFileName}e.resolveTypeReferenceDirective=S,e.getAutomaticTypeDirectiveNames=x,e.createCacheWithRedirects=L,e.createModuleResolutionCache=N,e.createTypeReferenceDirectiveResolutionCache=k,e.resolveModuleNameFromCache=R,e.resolveModuleName=O,e.resolveJSModule=G,e.tryResolveJSModule=B;var U=[c.JavaScript],V=[c.TypeScript,c.JavaScript],j=a(a([],V,!0),[c.Json],!1),K=[c.TSConfig];function H(n,t,r){return z(n,t,{moduleResolution:e.ModuleResolutionKind.NodeJs,allowJs:!0},r,void 0,U,void 0)}function W(n,t,r,i,a,o,s){return z(n,e.getDirectoryPath(t),r,i,a,s?K:r.resolveJsonModule?j:V,o)}function z(r,i,a,s,l,u,p){var f,m,_=t(a,s),g=[],y={compilerOptions:a,host:s,traceEnabled:_,failedLookupLocations:g,packageJsonInfoCache:l},h=e.forEach(u,function(e){return v(e)});return d(null===(f=null===h||void 0===h?void 0:h.value)||void 0===f?void 0:f.resolved,null===(m=null===h||void 0===h?void 0:h.value)||void 0===m?void 0:m.isExternalLibraryImport,g,y.resultFromCache);function v(t){var u=function(e,n,t,r){return q(e,n,t,r,!0)},d=w(t,r,i,u,y);if(d)return Se({resolved:d,isExternalLibraryImport:X(d.path)});if(e.isExternalModuleNameRelative(r)){var f=e.normalizePathAndParts(e.combinePaths(i,r)),m=f.path,g=f.parts,h=q(t,m,!1,y,!0);return h&&Se({resolved:h,isExternalLibraryImport:e.contains(g,"node_modules")})}_&&n(s,e.Diagnostics.Loading_module_0_from_node_modules_folder_target_file_type_1,r,c[t]);var v=ce(t,r,i,y,l,p);if(v){var b=v.value;if(!a.preserveSymlinks&&b&&!b.originalPath){var E=J(b.path,s,_),T=E===b.path?void 0:b.path;b=o(o({},b),{path:E,originalPath:T})}return{value:b&&{resolved:b,isExternalLibraryImport:!0}}}}}function J(t,r,i){if(!r.realpath)return t;var a=e.normalizePath(r.realpath(t));return i&&n(r,e.Diagnostics.Resolving_real_path_for_0_result_1,t,a),e.Debug.assert(r.fileExists(a),t+" linked to nonexistent file "+a),a}function q(t,i,a,o,s){if(o.traceEnabled&&n(o.host,e.Diagnostics.Loading_module_as_file_Slash_folder_candidate_module_location_0_target_file_type_1,i,c[t]),!e.hasTrailingDirectorySeparator(i)){if(!a){var l=e.getDirectoryPath(i);e.directoryProbablyExists(l,o.host)||(o.traceEnabled&&n(o.host,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,l),a=!0)}var u=$(t,i,a,o);if(u){var d=s?Y(u.path):void 0,p=d?re(d,!1,o):void 0;return r(p,u)}}if(!a){var f=e.directoryProbablyExists(i,o.host);f||(o.traceEnabled&&n(o.host,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,i),a=!0)}return te(t,i,a,o,s)}function X(n){return e.stringContains(n,e.nodeModulesPathPart)}function Y(n){var t=e.normalizePath(n),r=t.lastIndexOf(e.nodeModulesPathPart);if(-1!==r){var i=r+e.nodeModulesPathPart.length,a=Q(t,i);return 64===t.charCodeAt(i)&&(a=Q(t,a)),t.slice(0,a)}}function Q(n,t){var r=n.indexOf(e.directorySeparator,t+1);return-1===r?t:r}function Z(e,n,t,r){return i($(e,n,t,r))}function $(t,r,i,a){if(t===c.Json||t===c.TSConfig){var o=e.tryRemoveExtension(r,".json");return void 0===o&&t===c.Json?void 0:ee(o||r,t,i,a)}var s=ee(r,t,i,a);if(s)return s;if(e.hasJSFileExtension(r)){var l=e.removeFileExtension(r);if(a.traceEnabled){var u=r.substring(l.length);n(a.host,e.Diagnostics.File_name_0_has_a_1_extension_stripping_it,r,u)}return ee(l,t,i,a)}}function ee(n,t,r,i){if(!r){var a=e.getDirectoryPath(n);a&&(r=!e.directoryProbablyExists(a,i.host))}switch(t){case c.DtsOnly:return o(".d.ts");case c.TypeScript:return o(".ts")||o(".tsx")||o(".d.ts");case c.JavaScript:return o(".js")||o(".jsx");case c.TSConfig:case c.Json:return o(".json")}function o(e){var t=ne(n+e,r,i);return void 0===t?void 0:{path:t,ext:e}}}function ne(t,r,i){if(!r){if(i.host.fileExists(t))return i.traceEnabled&&n(i.host,e.Diagnostics.File_0_exist_use_it_as_a_name_resolution_result,t),t;i.traceEnabled&&n(i.host,e.Diagnostics.File_0_does_not_exist,t)}i.failedLookupLocations.push(t)}function te(e,n,t,i,a){void 0===a&&(a=!0);var o=a?re(n,t,i):void 0,s=o&&o.packageJsonContent,c=o&&o.versionPaths;return r(o,ie(e,n,t,i,s,c))}function re(t,r,i){var a,o,s,c=i.host,l=i.traceEnabled,u=e.combinePaths(t,"package.json");if(r)i.failedLookupLocations.push(u);else{var d=null===(a=i.packageJsonInfoCache)||void 0===a?void 0:a.getPackageJsonInfo(u);if(void 0!==d)return"boolean"!==typeof d?(l&&n(c,e.Diagnostics.File_0_exists_according_to_earlier_cached_lookups,u),d):(d&&l&&n(c,e.Diagnostics.File_0_does_not_exist_according_to_earlier_cached_lookups,u),void i.failedLookupLocations.push(u));var p=e.directoryProbablyExists(t,c);if(p&&c.fileExists(u)){var f=e.readJson(u,c);l&&n(c,e.Diagnostics.Found_package_json_at_0,u);var m=h(f,i),_={packageDirectory:t,packageJsonContent:f,versionPaths:m};return null===(o=i.packageJsonInfoCache)||void 0===o||o.setPackageJsonInfo(u,_),_}p&&l&&n(c,e.Diagnostics.File_0_does_not_exist,u),null===(s=i.packageJsonInfoCache)||void 0===s||s.setPackageJsonInfo(u,p),i.failedLookupLocations.push(u)}}function ie(t,r,a,o,l,u){var d;if(l)switch(t){case c.JavaScript:case c.Json:d=g(l,r,o);break;case c.TypeScript:d=m(l,r,o)||g(l,r,o);break;case c.DtsOnly:d=m(l,r,o);break;case c.TSConfig:d=_(l,r,o);break;default:return e.Debug.assertNever(t)}var p=function(t,r,a,o){var s=ne(r,a,o);if(s){var l=ae(t,s);if(l)return i(l);o.traceEnabled&&n(o.host,e.Diagnostics.File_0_has_an_unsupported_extension_so_skipping_it,s)}var u=t===c.DtsOnly?c.TypeScript:t;return q(u,r,a,o,!1)},f=d?!e.directoryProbablyExists(e.getDirectoryPath(d),o.host):void 0,y=a||!e.directoryProbablyExists(r,o.host),h=e.combinePaths(r,t===c.TSConfig?"tsconfig":"index");if(u&&(!d||e.containsPath(r,d))){var v=e.getRelativePathFromDirectory(r,d||h,!1);o.traceEnabled&&n(o.host,e.Diagnostics.package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_match_module_name_2,u.version,e.version,v);var b=fe(t,v,r,u.paths,void 0,p,f||y,o);if(b)return s(b.value)}var E=d&&s(p(t,d,f,o));return E||$(t,h,y,o)}function ae(n,t){var r=e.tryGetExtensionFromPath(t);return void 0!==r&&oe(n,r)?{path:t,ext:r}:void 0}function oe(e,n){switch(e){case c.JavaScript:return".js"===n||".jsx"===n;case c.TSConfig:case c.Json:return".json"===n;case c.TypeScript:return".ts"===n||".tsx"===n||".d.ts"===n;case c.DtsOnly:return".d.ts"===n}}function se(n){var t=n.indexOf(e.directorySeparator);return"@"===n[0]&&(t=n.indexOf(e.directorySeparator,t+1)),-1===t?{packageName:n,rest:""}:{packageName:n.slice(0,t),rest:n.slice(t+1)}}function ce(e,n,t,r,i,a){return ue(e,n,t,r,!1,i,a)}function le(e,n,t){return ue(c.DtsOnly,e,n,t,!0,void 0,void 0)}function ue(n,t,r,i,a,o,s){var c=o&&o.getOrCreateCacheForModuleName(t,s);return e.forEachAncestorDirectory(e.normalizeSlashes(r),function(r){if("node_modules"!==e.getBaseFileName(r)){var o=be(c,t,r,i);return o||Se(de(n,t,r,i,a))}})}function de(t,r,i,a,o){var s=e.combinePaths(i,"node_modules"),l=e.directoryProbablyExists(s,a.host);!l&&a.traceEnabled&&n(a.host,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,s);var u=o?void 0:pe(t,r,s,l,a);if(u)return u;if(t===c.TypeScript||t===c.DtsOnly){var d=e.combinePaths(s,"@types"),p=l;return l&&!e.directoryProbablyExists(d,a.host)&&(a.traceEnabled&&n(a.host,e.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it,d),p=!1),pe(c.DtsOnly,_e(r,a),d,p,a)}}function pe(t,a,o,s,c){var l=e.normalizePath(e.combinePaths(o,a)),u=re(l,!s,c);if(u){var d=$(t,l,!s,c);if(d)return i(d);var p=ie(t,l,!s,c,u.packageJsonContent,u.versionPaths);return r(u,p)}var f=function(e,n,t,i){var a=$(e,n,t,i)||ie(e,n,t,i,u&&u.packageJsonContent,u&&u.versionPaths);return r(u,a)},m=se(a),_=m.packageName,g=m.rest;if(""!==g){var y=e.combinePaths(o,_);if(u=re(y,!s,c),u&&u.versionPaths){c.traceEnabled&&n(c.host,e.Diagnostics.package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_match_module_name_2,u.versionPaths.version,e.version,g);var h=s&&e.directoryProbablyExists(y,c.host),v=fe(t,g,y,u.versionPaths.paths,void 0,f,!h,c);if(v)return v.value}}return f(t,l,!s,c)}function fe(t,r,a,o,s,c,l,u){s||(s=e.tryParsePatterns(o));var d=e.matchPatternOrExact(s,r);if(d){var p=e.isString(d)?void 0:e.matchedText(d,r),f=e.isString(d)?d:e.patternText(d);u.traceEnabled&&n(u.host,e.Diagnostics.Module_name_0_matched_pattern_1,r,f);var m=e.forEach(o[f],function(r){var o=p?r.replace("*",p):r,s=e.normalizePath(e.combinePaths(a,o));u.traceEnabled&&n(u.host,e.Diagnostics.Trying_substitution_0_candidate_module_location_Colon_1,r,o);var d=e.tryGetExtensionFromPath(r);if(void 0!==d){var f=ne(s,l,u);if(void 0!==f)return i({path:f,ext:d})}return c(t,s,l||!e.directoryProbablyExists(e.getDirectoryPath(s),u.host),u)});return{value:m}}}e.nodeModuleNameResolver=W,e.nodeModulesPathPart="/node_modules/",e.pathContainsNodeModules=X,e.parseNodeModuleFromPath=Y,e.parsePackageName=se;var me="__";function _e(t,r){var i=ye(t);return r.traceEnabled&&i!==t&&n(r.host,e.Diagnostics.Scoped_package_detected_looking_in_0,i),i}function ge(e){return"@types/"+ye(e)}function ye(n){if(e.startsWith(n,"@")){var t=n.replace(e.directorySeparator,me);if(t!==n)return t.slice(1)}return n}function he(n){var t=e.removePrefix(n,"@types/");return t!==n?ve(t):n}function ve(n){return e.stringContains(n,me)?"@"+n.replace(me,e.directorySeparator):n}function be(t,r,i,a){var o=t&&t.get(i);if(o)return a.traceEnabled&&n(a.host,e.Diagnostics.Resolution_for_module_0_was_found_in_cache_from_location_1,r,i),a.resultFromCache=o,{value:o.resolvedModule&&{path:o.resolvedModule.resolvedFileName,originalPath:o.resolvedModule.originalPath||!0,extension:o.resolvedModule.extension,packageId:o.resolvedModule.packageId}}}function Ee(n,r,i,a,o,s){var l=t(i,a),u=[],p={compilerOptions:i,host:a,traceEnabled:l,failedLookupLocations:u,packageJsonInfoCache:o},f=e.getDirectoryPath(r),m=_(c.TypeScript)||_(c.JavaScript);return d(m&&m.value,!1,u,p.resultFromCache);function _(t){var r=w(t,n,f,Z,p);if(r)return{value:r};if(e.isExternalModuleNameRelative(n)){var i=e.normalizePath(e.combinePaths(f,n));return Se(Z(t,i,!1,p))}var a=o&&o.getOrCreateCacheForModuleName(n,s),l=e.forEachAncestorDirectory(f,function(r){var i=be(a,n,r,p);if(i)return i;var o=e.normalizePath(e.combinePaths(r,n));return Se(Z(t,o,!1,p))});return l||(t===c.TypeScript?le(n,f,p):void 0)}}function Te(r,i,a,o,s,l){var u=t(a,o);u&&n(o,e.Diagnostics.Auto_discovery_for_typings_is_enabled_in_project_0_Running_extra_resolution_pass_for_module_1_using_cache_location_2,i,r,s);var p=[],f={compilerOptions:a,host:o,traceEnabled:u,failedLookupLocations:p,packageJsonInfoCache:l},m=de(c.DtsOnly,r,s,f,!1);return d(m,!0,p,f.resultFromCache)}function Se(e){return void 0!==e?{value:e}:void 0}e.getTypesPackageName=ge,e.mangleScopedPackageName=ye,e.getPackageNameFromTypesPackageName=he,e.unmangleScopedPackageName=ve,e.classicNameResolver=Ee,e.loadModuleFromGlobalCache=Te}(d||(d={})),function(e){var n;function t(n,t){return n.body&&!n.body.parent&&(e.setParent(n.body,n),e.setParentRecursive(n.body,!1)),n.body?r(n.body,t):1}function r(n,t){void 0===t&&(t=new e.Map);var r=e.getNodeId(n);if(t.has(r))return t.get(r)||0;t.set(r,void 0);var a=i(n,t);return t.set(r,a),a}function i(n,i){switch(n.kind){case 256:case 257:return 0;case 258:if(e.isEnumConst(n))return 2;break;case 264:case 263:if(!e.hasSyntacticModifier(n,1))return 0;break;case 270:var a=n;if(!a.moduleSpecifier&&a.exportClause&&271===a.exportClause.kind){for(var o=0,c=0,l=a.exportClause.elements;c<l.length;c++){var u=l[c],d=s(u,i);if(d>o&&(o=d),1===o)return o}return o}break;case 260:var p=0;return e.forEachChild(n,function(n){var t=r(n,i);switch(t){case 0:return;case 2:return void(p=2);case 1:return p=1,!0;default:e.Debug.assertNever(t)}}),p;case 259:return t(n,i);case 79:if(n.isInJSDocNamespace)return 0}return 1}function s(n,t){var i=n.propertyName||n.name,a=n.parent;while(a){if(e.isBlock(a)||e.isModuleBlock(a)||e.isSourceFile(a)){for(var o=a.statements,s=void 0,c=0,l=o;c<l.length;c++){var u=l[c];if(e.nodeHasName(u,i)){u.parent||(e.setParent(u,a),e.setParentRecursive(u,!1));var d=r(u,t);if((void 0===s||d>s)&&(s=d),1===s)return s}}if(void 0!==s)return s}a=a.parent}return 1}function c(n){return e.Debug.attachFlowNodeDebugInfo(n),n}(function(e){e[e["NonInstantiated"]=0]="NonInstantiated",e[e["Instantiated"]=1]="Instantiated",e[e["ConstEnumOnly"]=2]="ConstEnumOnly"})(e.ModuleInstanceState||(e.ModuleInstanceState={})),e.getModuleInstanceState=t,function(e){e[e["None"]=0]="None",e[e["IsContainer"]=1]="IsContainer",e[e["IsBlockScopedContainer"]=2]="IsBlockScopedContainer",e[e["IsControlFlowContainer"]=4]="IsControlFlowContainer",e[e["IsFunctionLike"]=8]="IsFunctionLike",e[e["IsFunctionExpression"]=16]="IsFunctionExpression",e[e["HasLocals"]=32]="HasLocals",e[e["IsInterface"]=64]="IsInterface",e[e["IsObjectLiteralOrClassExpressionMethod"]=128]="IsObjectLiteralOrClassExpressionMethod"}(n||(n={}));var l=d();function u(n,t){null===e.tracing||void 0===e.tracing||e.tracing.push("bind","bindSourceFile",{path:n.path},!0),e.performance.mark("beforeBind"),e.perfLogger.logStartBindFile(""+n.fileName),l(n,t),e.perfLogger.logStopBindFile(),e.performance.mark("afterBind"),e.performance.measure("Bind","beforeBind","afterBind"),null===e.tracing||void 0===e.tracing||e.tracing.pop()}function d(){var n,r,i,s,l,u,d,f,m,y,h,v,b,E,T,S,x,L,D,A,C,I,N,k,R=!1,O=0,w={flags:1},P={flags:1},M=We();function F(t,r,i,a,o){return e.createDiagnosticForNodeInSourceFile(e.getSourceFileOfNode(t)||n,t,r,i,a,o)}function G(t,a){n=t,r=a,i=e.getEmitScriptTarget(r),I=B(n,a),k=new e.Set,O=0,N=e.objectAllocator.getSymbolConstructor(),e.Debug.attachFlowNodeDebugInfo(w),e.Debug.attachFlowNodeDebugInfo(P),n.locals||(Hn(n),n.symbolCount=O,n.classifiableNames=k,Tn()),n=void 0,r=void 0,i=void 0,s=void 0,l=void 0,u=void 0,d=void 0,f=void 0,m=void 0,y=!1,h=void 0,v=void 0,b=void 0,E=void 0,T=void 0,S=void 0,x=void 0,D=void 0,A=!1,R=!1,C=0}return G;function B(n,t){return!(!e.getStrictOptionValue(t,"alwaysStrict")||n.isDeclarationFile)||!!n.externalModuleIndicator}function U(e,n){return O++,new N(e,n)}function V(n,t,r){n.flags|=r,t.symbol=n,n.declarations=e.appendIfUnique(n.declarations,t),1955&r&&!n.exports&&(n.exports=e.createSymbolTable()),6240&r&&!n.members&&(n.members=e.createSymbolTable()),n.constEnumOnlyModule&&304&n.flags&&(n.constEnumOnlyModule=!1),111551&r&&e.setValueDeclaration(n,t)}function j(n){if(269===n.kind)return n.isExportEquals?"export=":"default";var t=e.getNameOfDeclaration(n);if(t){if(e.isAmbientModule(n)){var r=e.getTextOfIdentifierOrLiteral(t);return e.isGlobalScopeAugmentation(n)?"__global":'"'+r+'"'}if(160===t.kind){var i=t.expression;if(e.isStringOrNumericLiteralLike(i))return e.escapeLeadingUnderscores(i.text);if(e.isSignedNumericLiteral(i))return e.tokenToString(i.operator)+i.operand.text;e.Debug.fail("Only computed properties with literal names have declaration names")}if(e.isPrivateIdentifier(t)){var a=e.getContainingClass(n);if(!a)return;var o=a.symbol;return e.getSymbolNameForPrivateIdentifier(o,t.escapedText)}return e.isPropertyNameLiteral(t)?e.getEscapedTextOfIdentifierOrLiteral(t):void 0}switch(n.kind){case 169:return"__constructor";case 177:case 172:case 318:return"__call";case 178:case 173:return"__new";case 174:return"__index";case 270:return"__export";case 300:return"export=";case 219:if(2===e.getAssignmentDeclarationKind(n))return"export=";e.Debug.fail("Unknown binary declaration kind");break;case 312:return e.isJSDocConstructSignature(n)?"__new":"__call";case 162:e.Debug.assert(312===n.parent.kind,"Impossible parameter parent kind",function(){return"parent is: "+(e.SyntaxKind?e.SyntaxKind[n.parent.kind]:n.parent.kind)+", expected JSDocFunctionType"});var s=n.parent,c=s.parameters.indexOf(n);return"arg"+c}}function K(n){return e.isNamedDeclaration(n)?e.declarationNameToString(n.name):e.unescapeLeadingUnderscores(e.Debug.checkDefined(j(n)))}function H(t,r,i,o,s,c,l){e.Debug.assert(l||!e.hasDynamicName(i));var u,d=e.hasSyntacticModifier(i,512)||e.isExportSpecifier(i)&&"default"===i.name.escapedText,p=l?"__computed":d&&r?"default":j(i);if(void 0===p)u=U(0,"__missing");else if(u=t.get(p),2885600&o&&k.add(p),u){if(c&&!u.isReplaceableByMethod)return u;if(u.flags&s)if(u.isReplaceableByMethod)t.set(p,u=U(0,p));else if(!(3&o&&67108864&u.flags)){e.isNamedDeclaration(i)&&e.setParent(i.name,i);var f=2&u.flags?e.Diagnostics.Cannot_redeclare_block_scoped_variable_0:e.Diagnostics.Duplicate_identifier_0,m=!0;(384&u.flags||384&o)&&(f=e.Diagnostics.Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations,m=!1);var _=!1;e.length(u.declarations)&&(d?(f=e.Diagnostics.A_module_cannot_have_multiple_default_exports,m=!1,_=!0):u.declarations&&u.declarations.length&&269===i.kind&&!i.isExportEquals&&(f=e.Diagnostics.A_module_cannot_have_multiple_default_exports,m=!1,_=!0));var g=[];e.isTypeAliasDeclaration(i)&&e.nodeIsMissing(i.type)&&e.hasSyntacticModifier(i,1)&&2887656&u.flags&&g.push(F(i,e.Diagnostics.Did_you_mean_0,"export type { "+e.unescapeLeadingUnderscores(i.name.escapedText)+" }"));var y=e.getNameOfDeclaration(i)||i;e.forEach(u.declarations,function(t,r){var i=e.getNameOfDeclaration(t)||t,a=F(i,f,m?K(t):void 0);n.bindDiagnostics.push(_?e.addRelatedInfo(a,F(y,0===r?e.Diagnostics.Another_export_default_is_here:e.Diagnostics.and_here)):a),_&&g.push(F(i,e.Diagnostics.The_first_export_default_is_here))});var h=F(y,f,m?K(i):void 0);n.bindDiagnostics.push(e.addRelatedInfo.apply(void 0,a([h],g,!1))),u=U(0,p)}}else t.set(p,u=U(0,p)),c&&(u.isReplaceableByMethod=!0);return V(u,i,o),u.parent?e.Debug.assert(u.parent===r,"Existing symbol parent should match new one"):u.parent=r,u}function W(n,t,r){var i=!!(1&e.getCombinedModifierFlags(n))||z(n);if(2097152&t)return 273===n.kind||263===n.kind&&i?H(l.symbol.exports,l.symbol,n,t,r):H(l.locals,void 0,n,t,r);if(e.isJSDocTypeAlias(n)&&e.Debug.assert(e.isInJSFile(n)),!e.isAmbientModule(n)&&(i||64&l.flags)){if(!l.locals||e.hasSyntacticModifier(n,512)&&!j(n))return H(l.symbol.exports,l.symbol,n,t,r);var a=111551&t?1048576:0,o=H(l.locals,void 0,n,a,r);return o.exportSymbol=H(l.symbol.exports,l.symbol,n,t,r),n.localSymbol=o,o}return H(l.locals,void 0,n,t,r)}function z(n){if(n.parent&&e.isModuleDeclaration(n)&&(n=n.parent),!e.isJSDocTypeAlias(n))return!1;if(!e.isJSDocEnumTag(n)&&n.fullName)return!0;var t=e.getNameOfDeclaration(n);return!!t&&(!(!e.isPropertyAccessEntityNameExpression(t.parent)||!bt(t.parent))||!!(e.isDeclaration(t.parent)&&1&e.getCombinedModifierFlags(t.parent)))}function J(n,t){var r=l,i=u,a=d;if(1&t?(212!==n.kind&&(u=l),l=d=n,32&t&&(l.locals=e.createSymbolTable()),cn(l)):2&t&&(d=n,d.locals=void 0),4&t){var o=h,s=v,p=b,f=E,m=x,_=D,g=A,T=16&t&&!e.hasSyntacticModifier(n,256)&&!n.asteriskToken&&!!e.getImmediatelyInvokedFunctionExpression(n);T||(h=c({flags:2}),144&t&&(h.node=n)),E=T||169===n.kind||168===n.kind||e.isInJSFile(n)&&(254===n.kind||211===n.kind)?ae():void 0,x=void 0,v=void 0,b=void 0,D=void 0,A=!1,Q(n),n.flags&=-2817,!(1&h.flags)&&8&t&&e.nodeIsPresent(n.body)&&(n.flags|=256,A&&(n.flags|=512),n.endFlowNode=h),300===n.kind&&(n.flags|=C,n.endFlowNode=h),E&&(le(E,h),h=me(E),(169===n.kind||168===n.kind||e.isInJSFile(n)&&(254===n.kind||211===n.kind))&&(n.returnFlowNode=h)),T||(h=o),v=s,b=p,E=f,x=m,D=_,A=g}else 64&t?(y=!1,Q(n),n.flags=y?128|n.flags:-129&n.flags):Q(n);l=r,u=i,d=a}function q(e){X(e,function(e){return 254===e.kind?Hn(e):void 0}),X(e,function(e){return 254!==e.kind?Hn(e):void 0})}function X(n,t){void 0===t&&(t=Hn),void 0!==n&&e.forEach(n,t)}function Y(n){e.forEachChild(n,Hn,X)}function Q(n){var t=R;if(R=!1,Ft(n))return Y(n),Wn(n),void(R=t);switch(n.kind>=235&&n.kind<=251&&!r.allowUnreachableCode&&(n.flowNode=h),n.kind){case 239:Se(n);break;case 238:xe(n);break;case 240:Le(n);break;case 241:case 242:De(n);break;case 237:Ae(n);break;case 245:case 249:Ce(n);break;case 244:case 243:ke(n);break;case 250:Re(n);break;case 247:Oe(n);break;case 261:we(n);break;case 287:Pe(n);break;case 236:Me(n);break;case 248:Ge(n);break;case 217:je(n);break;case 218:Ke(n);break;case 219:if(e.isDestructuringAssignment(n))return R=t,void He(n);M(n);break;case 213:ze(n);break;case 220:Je(n);break;case 252:Xe(n);break;case 204:case 205:an(n);break;case 206:on(n);break;case 228:rn(n);break;case 340:case 333:case 334:Qe(n);break;case 300:q(n.statements),Hn(n.endOfFileToken);break;case 233:case 260:q(n.statements);break;case 201:Ye(n);break;case 203:case 202:case 291:case 223:R=t;default:Y(n);break}Wn(n),R=t}function Z(e){switch(e.kind){case 79:case 80:case 108:case 204:case 205:return ee(e);case 206:return ne(e);case 210:case 228:return Z(e.expression);case 219:return re(e);case 217:return 53===e.operator&&Z(e.operand);case 214:return Z(e.expression)}return!1}function $(n){return e.isDottedName(n)||(e.isPropertyAccessExpression(n)||e.isNonNullExpression(n)||e.isParenthesizedExpression(n))&&$(n.expression)||e.isBinaryExpression(n)&&27===n.operatorToken.kind&&$(n.right)||e.isElementAccessExpression(n)&&e.isStringOrNumericLiteralLike(n.argumentExpression)&&$(n.expression)||e.isAssignmentExpression(n)&&$(n.left)}function ee(n){return $(n)||e.isOptionalChain(n)&&ee(n.expression)}function ne(e){if(e.arguments)for(var n=0,t=e.arguments;n<t.length;n++){var r=t[n];if(ee(r))return!0}return!(204!==e.expression.kind||!ee(e.expression.expression))}function te(n,t){return e.isTypeOfExpression(n)&&ie(n.expression)&&e.isStringLiteralLike(t)}function re(e){switch(e.operatorToken.kind){case 63:case 75:case 76:case 77:return ee(e.left);case 34:case 35:case 36:case 37:return ie(e.left)||ie(e.right)||te(e.right,e.left)||te(e.left,e.right);case 102:return ie(e.left);case 101:return Z(e.right);case 27:return Z(e.right)}return!1}function ie(e){switch(e.kind){case 210:return ie(e.expression);case 219:switch(e.operatorToken.kind){case 63:return ie(e.left);case 27:return ie(e.right)}}return ee(e)}function ae(){return c({flags:4,antecedents:void 0})}function oe(){return c({flags:8,antecedents:void 0})}function se(e,n,t){return c({flags:1024,target:e,antecedents:n,antecedent:t})}function ce(e){e.flags|=2048&e.flags?4096:2048}function le(n,t){1&t.flags||e.contains(n.antecedents,t)||((n.antecedents||(n.antecedents=[])).push(t),ce(t))}function ue(n,t,r){return 1&t.flags?t:r?!(110===r.kind&&64&n||95===r.kind&&32&n)||e.isExpressionOfOptionalChainRoot(r)||e.isNullishCoalesce(r.parent)?Z(r)?(ce(t),c({flags:n,antecedent:t,node:r})):t:w:32&n?t:w}function de(e,n,t,r){return ce(e),c({flags:128,antecedent:e,switchStatement:n,clauseStart:t,clauseEnd:r})}function pe(e,n,t){ce(n);var r=c({flags:e,antecedent:n,node:t});return x&&le(x,r),r}function fe(e,n){return ce(e),c({flags:512,antecedent:e,node:n})}function me(e){var n=e.antecedents;return n?1===n.length?n[0]:e:w}function _e(e){var n=e.parent;switch(n.kind){case 237:case 239:case 238:return n.expression===e;case 240:case 220:return n.condition===e}return!1}function ge(e){while(1)if(210===e.kind)e=e.expression;else{if(217!==e.kind||53!==e.operator)return 219===e.kind&&(55===e.operatorToken.kind||56===e.operatorToken.kind||60===e.operatorToken.kind);e=e.operand}}function ye(n){return n=e.skipParentheses(n),e.isBinaryExpression(n)&&e.isLogicalOrCoalescingAssignmentOperator(n.operatorToken.kind)}function he(n){while(e.isParenthesizedExpression(n.parent)||e.isPrefixUnaryExpression(n.parent)&&53===n.parent.operator)n=n.parent;return!_e(n)&&!ye(n.parent)&&!ge(n.parent)&&!(e.isOptionalChain(n.parent)&&n.parent.expression===n)}function ve(e,n,t,r){var i=T,a=S;T=t,S=r,e(n),T=i,S=a}function be(n,t,r){ve(Hn,n,t,r),n&&(ye(n)||ge(n)||e.isOptionalChain(n)&&e.isOutermostOptionalChain(n))||(le(t,ue(32,h,n)),le(r,ue(64,h,n)))}function Ee(e,n,t){var r=v,i=b;v=n,b=t,Hn(e),v=r,b=i}function Te(e,n){var t=D;while(t&&248===e.parent.kind)t.continueTarget=n,t=t.next,e=e.parent;return n}function Se(e){var n=Te(e,oe()),t=ae(),r=ae();le(n,h),h=n,be(e.expression,t,r),h=me(t),Ee(e.statement,r,n),le(n,h),h=me(r)}function xe(e){var n=oe(),t=Te(e,ae()),r=ae();le(n,h),h=n,Ee(e.statement,r,t),le(t,h),h=me(t),be(e.expression,n,r),h=me(r)}function Le(e){var n=Te(e,oe()),t=ae(),r=ae();Hn(e.initializer),le(n,h),h=n,be(e.condition,t,r),h=me(t),Ee(e.statement,r,n),Hn(e.incrementor),le(n,h),h=me(r)}function De(e){var n=Te(e,oe()),t=ae();Hn(e.expression),le(n,h),h=n,242===e.kind&&Hn(e.awaitModifier),le(t,h),Hn(e.initializer),253!==e.initializer.kind&&Ue(e.initializer),Ee(e.statement,t,n),le(n,h),h=me(t)}function Ae(e){var n=ae(),t=ae(),r=ae();be(e.expression,n,t),h=me(n),Hn(e.thenStatement),le(r,h),h=me(t),Hn(e.elseStatement),le(r,h),h=me(r)}function Ce(e){Hn(e.expression),245===e.kind&&(A=!0,E&&le(E,h)),h=w}function Ie(e){for(var n=D;n;n=n.next)if(n.name===e)return n}function Ne(e,n,t){var r=244===e.kind?n:t;r&&(le(r,h),h=w)}function ke(e){if(Hn(e.label),e.label){var n=Ie(e.label.escapedText);n&&(n.referenced=!0,Ne(e,n.breakTarget,n.continueTarget))}else Ne(e,v,b)}function Re(n){var t=E,r=x,i=ae(),a=ae(),o=ae();if(n.finallyBlock&&(E=a),le(o,h),x=o,Hn(n.tryBlock),le(i,h),n.catchClause&&(h=me(o),o=ae(),le(o,h),x=o,Hn(n.catchClause),le(i,h)),E=t,x=r,n.finallyBlock){var s=ae();s.antecedents=e.concatenate(e.concatenate(i.antecedents,o.antecedents),a.antecedents),h=s,Hn(n.finallyBlock),1&h.flags?h=w:(E&&a.antecedents&&le(E,se(s,a.antecedents,h)),x&&o.antecedents&&le(x,se(s,o.antecedents,h)),h=i.antecedents?se(s,i.antecedents,h):w)}else h=me(i)}function Oe(n){var t=ae();Hn(n.expression);var r=v,i=L;v=t,L=h,Hn(n.caseBlock),le(t,h);var a=e.forEach(n.caseBlock.clauses,function(e){return 288===e.kind});n.possiblyExhaustive=!a&&!t.antecedents,a||le(t,de(L,n,0,0)),v=r,L=i,h=me(t)}function we(e){for(var n=e.clauses,t=Z(e.parent.expression),i=w,a=0;a<n.length;a++){var o=a;while(!n[a].statements.length&&a+1<n.length)Hn(n[a]),a++;var s=ae();le(s,t?de(L,e.parent,o,a+1):L),le(s,i),h=me(s);var c=n[a];Hn(c),i=h,1&h.flags||a===n.length-1||!r.noFallthroughCasesInSwitch||(c.fallthroughFlowNode=h)}}function Pe(e){var n=h;h=L,Hn(e.expression),h=n,X(e.statements)}function Me(e){Hn(e.expression),Fe(e.expression)}function Fe(n){if(206===n.kind){var t=n;106!==t.expression.kind&&e.isDottedName(t.expression)&&(h=fe(h,t))}}function Ge(n){var t=ae();D={next:D,name:n.label.escapedText,breakTarget:t,continueTarget:void 0,referenced:!1},Hn(n.label),Hn(n.statement),D.referenced||r.allowUnusedLabels||Vn(e.unusedLabelIsError(r),n.label,e.Diagnostics.Unused_label),D=D.next,le(t,h),h=me(t)}function Be(e){219===e.kind&&63===e.operatorToken.kind?Ue(e.left):Ue(e)}function Ue(e){if($(e))h=pe(16,h,e);else if(202===e.kind)for(var n=0,t=e.elements;n<t.length;n++){var r=t[n];223===r.kind?Ue(r.expression):Be(r)}else if(203===e.kind)for(var i=0,a=e.properties;i<a.length;i++){var o=a[i];291===o.kind?Be(o.initializer):292===o.kind?Ue(o.name):293===o.kind&&Ue(o.expression)}}function Ve(n,t,r){var i=ae();55===n.operatorToken.kind||76===n.operatorToken.kind?be(n.left,i,r):be(n.left,t,i),h=me(i),Hn(n.operatorToken),e.isLogicalOrCoalescingAssignmentOperator(n.operatorToken.kind)?(ve(Hn,n.right,t,r),Ue(n.left),le(t,ue(32,h,n)),le(r,ue(64,h,n))):be(n.right,t,r)}function je(e){if(53===e.operator){var n=T;T=S,S=n,Y(e),S=T,T=n}else Y(e),45!==e.operator&&46!==e.operator||Ue(e.operand)}function Ke(e){Y(e),45!==e.operator&&46!==e.operator||Ue(e.operand)}function He(e){R?(R=!1,Hn(e.operatorToken),Hn(e.right),R=!0,Hn(e.left)):(R=!0,Hn(e.left),R=!1,Hn(e.operatorToken),Hn(e.right)),Ue(e.left)}function We(){return e.createBinaryExpressionTrampoline(n,t,r,i,a,void 0);function n(n,t){if(t){t.stackIndex++,e.setParent(n,s);var r=I;qn(n);var i=s;s=n,t.skip=!1,t.inStrictModeStack[t.stackIndex]=r,t.parentStack[t.stackIndex]=i}else t={stackIndex:0,skip:!1,inStrictModeStack:[void 0],parentStack:[void 0]};var a=n.operatorToken.kind;if(55===a||56===a||60===a||e.isLogicalOrCoalescingAssignmentOperator(a)){if(he(n)){var o=ae();Ve(n,o,o),h=me(o)}else Ve(n,T,S);t.skip=!0}return t}function t(e,n,t){if(!n.skip)return o(e)}function r(e,n,t){n.skip||(27===e.kind&&Fe(t.left),Hn(e))}function i(e,n,t){if(!n.skip)return o(e)}function a(n,t){if(!t.skip){var r=n.operatorToken.kind;if(e.isAssignmentOperator(r)&&!e.isAssignmentTarget(n)&&(Ue(n.left),63===r&&205===n.left.kind)){var i=n.left;ie(i.expression)&&(h=pe(256,h,n))}}var a=t.inStrictModeStack[t.stackIndex],o=t.parentStack[t.stackIndex];void 0!==a&&(I=a),void 0!==o&&(s=o),t.skip=!1,t.stackIndex--}function o(n){if(n&&e.isBinaryExpression(n)&&!e.isDestructuringAssignment(n))return n;Hn(n)}}function ze(e){Y(e),204===e.expression.kind&&Ue(e.expression)}function Je(e){var n=ae(),t=ae(),r=ae();be(e.condition,n,t),h=me(n),Hn(e.questionToken),Hn(e.whenTrue),le(r,h),h=me(t),Hn(e.colonToken),Hn(e.whenFalse),le(r,h),h=me(r)}function qe(n){var t=e.isOmittedExpression(n)?void 0:n.name;if(e.isBindingPattern(t))for(var r=0,i=t.elements;r<i.length;r++){var a=i[r];qe(a)}else h=pe(16,h,n)}function Xe(n){Y(n),(n.initializer||e.isForInOrOfStatement(n.parent.parent))&&qe(n)}function Ye(n){e.isBindingPattern(n.name)?(X(n.decorators),X(n.modifiers),Hn(n.dotDotDotToken),Hn(n.propertyName),Hn(n.initializer),Hn(n.name)):Y(n)}function Qe(n){Hn(n.tagName),334!==n.kind&&n.fullName&&(e.setParent(n.fullName,n),e.setParentRecursive(n.fullName,!1)),"string"!==typeof n.comment&&X(n.comment)}function Ze(n){Y(n);var t=e.getHostSignatureFromJSDoc(n);t&&167!==t.kind&&V(t.symbol,t,32)}function $e(n,t,r){ve(Hn,n,t,r),e.isOptionalChain(n)&&!e.isOutermostOptionalChain(n)||(le(t,ue(32,h,n)),le(r,ue(64,h,n)))}function en(e){switch(e.kind){case 204:Hn(e.questionDotToken),Hn(e.name);break;case 205:Hn(e.questionDotToken),Hn(e.argumentExpression);break;case 206:Hn(e.questionDotToken),X(e.typeArguments),X(e.arguments);break}}function nn(n,t,r){var i=e.isOptionalChainRoot(n)?ae():void 0;$e(n.expression,i||t,r),i&&(h=me(i)),ve(en,n,t,r),e.isOutermostOptionalChain(n)&&(le(t,ue(32,h,n)),le(r,ue(64,h,n)))}function tn(e){if(he(e)){var n=ae();nn(e,n,n),h=me(n)}else nn(e,T,S)}function rn(n){e.isOptionalChain(n)?tn(n):Y(n)}function an(n){e.isOptionalChain(n)?tn(n):Y(n)}function on(n){if(e.isOptionalChain(n))tn(n);else{var t=e.skipParentheses(n.expression);211===t.kind||212===t.kind?(X(n.typeArguments),X(n.arguments),Hn(n.expression)):(Y(n),106===n.expression.kind&&(h=fe(h,n)))}if(204===n.expression.kind){var r=n.expression;e.isIdentifier(r.name)&&ie(r.expression)&&e.isPushOrUnshiftIdentifier(r.name)&&(h=pe(256,h,n))}}function sn(n){switch(n.kind){case 224:case 255:case 258:case 203:case 180:case 317:case 284:return 1;case 256:return 65;case 259:case 257:case 193:return 33;case 300:return 37;case 167:if(e.isObjectLiteralOrClassExpressionMethod(n))return 173;case 169:case 254:case 166:case 170:case 171:case 172:case 318:case 312:case 177:case 173:case 174:case 178:case 168:return 45;case 211:case 212:return 61;case 260:return 4;case 165:return n.initializer?4:0;case 290:case 240:case 241:case 242:case 261:return 2;case 233:return e.isFunctionLike(n.parent)||e.isClassStaticBlockDeclaration(n.parent)?0:2}return 0}function cn(e){f&&(f.nextContainer=e),f=e}function ln(e,n,t){switch(l.kind){case 259:return W(e,n,t);case 300:return dn(e,n,t);case 224:case 255:return un(e,n,t);case 258:return H(l.symbol.exports,l.symbol,e,n,t);case 180:case 317:case 203:case 256:case 284:return H(l.symbol.members,l.symbol,e,n,t);case 177:case 178:case 172:case 173:case 318:case 174:case 167:case 166:case 169:case 170:case 171:case 254:case 211:case 212:case 312:case 340:case 333:case 168:case 257:case 193:return H(l.locals,void 0,e,n,t)}}function un(n,t,r){return e.isStatic(n)?H(l.symbol.exports,l.symbol,n,t,r):H(l.symbol.members,l.symbol,n,t,r)}function dn(t,r,i){return e.isExternalModule(n)?W(t,r,i):H(n.locals,void 0,t,r,i)}function pn(n){var t=e.isSourceFile(n)?n:e.tryCast(n.body,e.isModuleBlock);return!!t&&t.statements.some(function(n){return e.isExportDeclaration(n)||e.isExportAssignment(n)})}function fn(e){8388608&e.flags&&!pn(e)?e.flags|=64:e.flags&=-65}function mn(t){if(fn(t),e.isAmbientModule(t))if(e.hasSyntacticModifier(t,1)&&Un(t,e.Diagnostics.export_modifier_cannot_be_applied_to_ambient_modules_and_module_augmentations_since_they_are_always_visible),e.isModuleAugmentationExternal(t))_n(t);else{var r=void 0;if(10===t.name.kind){var i=t.name.text;r=e.tryParsePattern(i),void 0===r&&Un(t.name,e.Diagnostics.Pattern_0_can_have_at_most_one_Asterisk_character,i)}var a=ln(t,512,110735);n.patternAmbientModules=e.append(n.patternAmbientModules,r&&!e.isString(r)?{pattern:r,symbol:a}:void 0)}else{var o=_n(t);if(0!==o){a=t.symbol;a.constEnumOnlyModule=!(304&a.flags)&&2===o&&!1!==a.constEnumOnlyModule}}}function _n(e){var n=t(e),r=0!==n;return ln(e,r?512:1024,r?110735:0),n}function gn(n){var t=U(131072,j(n));V(t,n,131072);var r=U(2048,"__type");V(r,n,2048),r.members=e.createSymbolTable(),r.members.set(t.escapedName,t)}function yn(t){var r;if(function(e){e[e["Property"]=1]="Property",e[e["Accessor"]=2]="Accessor"}(r||(r={})),I&&!e.isAssignmentTarget(t))for(var i=new e.Map,a=0,o=t.properties;a<o.length;a++){var s=o[a];if(293!==s.kind&&79===s.name.kind){var c=s.name,l=291===s.kind||292===s.kind||167===s.kind?1:2,u=i.get(c.escapedText);if(u){if(1===l&&1===u){var d=e.getErrorSpanForNode(n,c);n.bindDiagnostics.push(e.createFileDiagnostic(n,d.start,d.length,e.Diagnostics.An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode))}}else i.set(c.escapedText,l)}}return bn(t,4096,"__object")}function hn(e){return bn(e,4096,"__jsxAttributes")}function vn(e,n,t){return ln(e,n,t)}function bn(e,n,t){var r=U(n,t);return 106508&n&&(r.parent=l.symbol),V(r,e,n),r}function En(n,t,r){switch(d.kind){case 259:W(n,t,r);break;case 300:if(e.isExternalOrCommonJsModule(l)){W(n,t,r);break}default:d.locals||(d.locals=e.createSymbolTable(),cn(d)),H(d.locals,void 0,n,t,r)}}function Tn(){if(m){for(var t=l,r=f,i=d,a=s,o=h,u=0,p=m;u<p.length;u++){var g=p[u],y=g.parent.parent;l=e.findAncestor(y.parent,function(e){return!!(1&sn(e))})||n,d=e.getEnclosingBlockScopeContainer(y)||n,h=c({flags:2}),s=g,Hn(g.typeExpression);var v=e.getNameOfDeclaration(g);if((e.isJSDocEnumTag(g)||!g.fullName)&&v&&e.isPropertyAccessEntityNameExpression(v.parent)){var b=bt(v.parent);if(b){ht(n.symbol,v.parent,b,!!e.findAncestor(v,function(n){return e.isPropertyAccessExpression(n)&&"prototype"===n.name.escapedText}),!1);var E=l;switch(e.getAssignmentDeclarationPropertyAccessKind(v.parent)){case 1:case 2:l=e.isExternalOrCommonJsModule(n)?n:void 0;break;case 4:l=v.parent.expression;break;case 3:l=v.parent.expression.name;break;case 5:l=_(n,v.parent.expression)?n:e.isPropertyAccessExpression(v.parent.expression)?v.parent.expression.name:v.parent.expression;break;case 0:return e.Debug.fail("Shouldn't have detected typedef or enum on non-assignment declaration")}l&&W(g,524288,788968),l=E}}else e.isJSDocEnumTag(g)||!g.fullName||79===g.fullName.kind?(s=g.parent,En(g,524288,788968)):Hn(g.fullName)}l=t,f=r,d=i,s=a,h=o}}function Sn(t){n.parseDiagnostics.length||8388608&t.flags||4194304&t.flags||e.isIdentifierName(t)||(I&&t.originalKeywordKind>=117&&t.originalKeywordKind<=125?n.bindDiagnostics.push(F(t,xn(t),e.declarationNameToString(t))):131===t.originalKeywordKind?e.isExternalModule(n)&&e.isInTopLevelContext(t)?n.bindDiagnostics.push(F(t,e.Diagnostics.Identifier_expected_0_is_a_reserved_word_at_the_top_level_of_a_module,e.declarationNameToString(t))):32768&t.flags&&n.bindDiagnostics.push(F(t,e.Diagnostics.Identifier_expected_0_is_a_reserved_word_that_cannot_be_used_here,e.declarationNameToString(t))):125===t.originalKeywordKind&&8192&t.flags&&n.bindDiagnostics.push(F(t,e.Diagnostics.Identifier_expected_0_is_a_reserved_word_that_cannot_be_used_here,e.declarationNameToString(t))))}function xn(t){return e.getContainingClass(t)?e.Diagnostics.Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_strict_mode:n.externalModuleIndicator?e.Diagnostics.Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode:e.Diagnostics.Identifier_expected_0_is_a_reserved_word_in_strict_mode}function Ln(t){"#constructor"===t.escapedText&&(n.parseDiagnostics.length||n.bindDiagnostics.push(F(t,e.Diagnostics.constructor_is_a_reserved_word,e.declarationNameToString(t))))}function Dn(n){I&&e.isLeftHandSideExpression(n.left)&&e.isAssignmentOperator(n.operatorToken.kind)&&Nn(n,n.left)}function An(e){I&&e.variableDeclaration&&Nn(e,e.variableDeclaration.name)}function Cn(t){if(I&&79===t.expression.kind){var r=e.getErrorSpanForNode(n,t.expression);n.bindDiagnostics.push(e.createFileDiagnostic(n,r.start,r.length,e.Diagnostics.delete_cannot_be_called_on_an_identifier_in_strict_mode))}}function In(n){return e.isIdentifier(n)&&("eval"===n.escapedText||"arguments"===n.escapedText)}function Nn(t,r){if(r&&79===r.kind){var i=r;if(In(i)){var a=e.getErrorSpanForNode(n,r);n.bindDiagnostics.push(e.createFileDiagnostic(n,a.start,a.length,kn(t),e.idText(i)))}}}function kn(t){return e.getContainingClass(t)?e.Diagnostics.Code_contained_in_a_class_is_evaluated_in_JavaScript_s_strict_mode_which_does_not_allow_this_use_of_0_For_more_information_see_https_Colon_Slash_Slashdeveloper_mozilla_org_Slashen_US_Slashdocs_SlashWeb_SlashJavaScript_SlashReference_SlashStrict_mode:n.externalModuleIndicator?e.Diagnostics.Invalid_use_of_0_Modules_are_automatically_in_strict_mode:e.Diagnostics.Invalid_use_of_0_in_strict_mode}function Rn(e){I&&Nn(e,e.name)}function On(t){return e.getContainingClass(t)?e.Diagnostics.Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_definitions_are_automatically_in_strict_mode:n.externalModuleIndicator?e.Diagnostics.Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_are_automatically_in_strict_mode:e.Diagnostics.Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5}function wn(t){if(i<2&&300!==d.kind&&259!==d.kind&&!e.isFunctionLikeOrClassStaticBlockDeclaration(d)){var r=e.getErrorSpanForNode(n,t);n.bindDiagnostics.push(e.createFileDiagnostic(n,r.start,r.length,On(t)))}}function Pn(t){I&&32&t.numericLiteralFlags&&n.bindDiagnostics.push(F(t,e.Diagnostics.Octal_literals_are_not_allowed_in_strict_mode))}function Mn(e){I&&Nn(e,e.operand)}function Fn(e){I&&(45!==e.operator&&46!==e.operator||Nn(e,e.operand))}function Gn(n){I&&Un(n,e.Diagnostics.with_statements_are_not_allowed_in_strict_mode)}function Bn(n){I&&r.target>=2&&(e.isDeclarationStatement(n.statement)||e.isVariableStatement(n.statement))&&Un(n.label,e.Diagnostics.A_label_is_not_allowed_here)}function Un(t,r,i,a,o){var s=e.getSpanOfTokenAtPosition(n,t.pos);n.bindDiagnostics.push(e.createFileDiagnostic(n,s.start,s.length,r,i,a,o))}function Vn(e,n,t){jn(e,n,n,t)}function jn(t,r,i,a){Kn(t,{pos:e.getTokenPosOfNode(r,n),end:i.end},a)}function Kn(t,r,i){var a=e.createFileDiagnostic(n,r.pos,r.end-r.pos,i);t?n.bindDiagnostics.push(a):n.bindSuggestionDiagnostics=e.append(n.bindSuggestionDiagnostics,o(o({},a),{category:e.DiagnosticCategory.Suggestion}))}function Hn(n){if(n){e.setParent(n,s);var t=I;if(qn(n),n.kind>158){var r=s;s=n;var i=sn(n);0===i?Q(n):J(n,i),s=r}else{r=s;1===n.kind&&(s=n),Wn(n),s=r}I=t}}function Wn(n){if(e.hasJSDocNodes(n))if(e.isInJSFile(n))for(var t=0,r=n.jsDoc;t<r.length;t++){var i=r[t];Hn(i)}else for(var a=0,o=n.jsDoc;a<o.length;a++){i=o[a];e.setParent(i,n),e.setParentRecursive(i,!1)}}function zn(n){if(!I)for(var t=0,r=n;t<r.length;t++){var i=r[t];if(!e.isPrologueDirective(i))return;if(Jn(i))return void(I=!0)}}function Jn(t){var r=e.getSourceTextOfNodeFromSourceFile(n,t.expression);return'"use strict"'===r||"'use strict'"===r}function qn(t){switch(t.kind){case 79:if(t.isInJSDocNamespace){var r=t.parent;while(r&&!e.isJSDocTypeAlias(r))r=r.parent;En(r,524288,788968);break}case 108:return h&&(e.isExpression(t)||292===s.kind)&&(t.flowNode=h),Sn(t);case 159:h&&e.isPartOfTypeQuery(t)&&(t.flowNode=h);break;case 229:case 106:t.flowNode=h;break;case 80:return Ln(t);case 204:case 205:var i=t;h&&$(i)&&(i.flowNode=h),e.isSpecialPropertyDeclaration(i)&&dt(i),e.isInJSFile(i)&&n.commonJsModuleIndicator&&e.isModuleExportsAccessExpression(i)&&!g(d,"module")&&H(n.locals,void 0,i.expression,134217729,111550);break;case 219:var a=e.getAssignmentDeclarationKind(t);switch(a){case 1:at(t);break;case 2:ot(t);break;case 3:mt(t.left,t);break;case 6:pt(t);break;case 4:ct(t);break;case 5:var o=t.left.expression;if(e.isInJSFile(t)&&e.isIdentifier(o)){var c=g(d,o.escapedText);if(e.isThisInitializedDeclaration(null===c||void 0===c?void 0:c.valueDeclaration)){ct(t);break}}gt(t);break;case 0:break;default:e.Debug.fail("Unknown binary expression special property assignment kind")}return Dn(t);case 290:return An(t);case 213:return Cn(t);case 8:return Pn(t);case 218:return Mn(t);case 217:return Fn(t);case 246:return Gn(t);case 248:return Bn(t);case 190:return void(y=!0);case 175:break;case 161:return Pt(t);case 162:return Nt(t);case 252:return It(t);case 201:return t.flowNode=h,It(t);case 165:case 164:return Xn(t);case 291:case 292:return Ot(t,4,0);case 294:return Ot(t,8,900095);case 172:case 173:case 174:return ln(t,131072,0);case 167:case 166:return Ot(t,8192|(t.questionToken?16777216:0),e.isObjectLiteralMethod(t)?0:103359);case 254:return kt(t);case 169:return ln(t,16384,0);case 170:return Ot(t,32768,46015);case 171:return Ot(t,65536,78783);case 177:case 312:case 318:case 178:return gn(t);case 180:case 317:case 193:return Yn(t);case 327:return Ze(t);case 203:return yn(t);case 211:case 212:return Rt(t);case 206:var l=e.getAssignmentDeclarationKind(t);switch(l){case 7:return _t(t);case 8:return it(t);case 9:return ft(t);case 0:break;default:return e.Debug.fail("Unknown call expression assignment declaration kind")}e.isInJSFile(t)&&Dt(t);break;case 224:case 255:return I=!0,At(t);case 256:return En(t,64,788872);case 257:return En(t,524288,788968);case 258:return Ct(t);case 259:return mn(t);case 284:return hn(t);case 283:return vn(t,4,0);case 263:case 266:case 268:case 273:return ln(t,2097152,2097152);case 262:return et(t);case 265:return tt(t);case 270:return nt(t);case 269:return $n(t);case 300:return zn(t.statements),Qn();case 233:if(!e.isFunctionLikeOrClassStaticBlockDeclaration(t.parent))return;case 260:return zn(t.statements);case 335:if(318===t.parent.kind)return Nt(t);if(317!==t.parent.kind)break;case 342:var u=t,p=u.isBracketed||u.typeExpression&&311===u.typeExpression.type.kind?16777220:4;return ln(u,p,0);case 340:case 333:case 334:return(m||(m=[])).push(t)}}function Xn(e){return Ot(e,4|(e.questionToken?16777216:0),0)}function Yn(e){return bn(e,2048,"__type")}function Qn(){if(fn(n),e.isExternalModule(n))Zn();else if(e.isJsonSourceFile(n)){Zn();var t=n.symbol;H(n.symbol.exports,n.symbol,n,4,67108863),n.symbol=t}}function Zn(){bn(n,512,'"'+e.removeFileExtension(n.fileName)+'"')}function $n(n){if(l.symbol&&l.symbol.exports){var t=e.exportAssignmentIsAlias(n)?2097152:4,r=H(l.symbol.exports,l.symbol,n,t,67108863);n.isExportEquals&&e.setValueDeclaration(r,n)}else bn(n,111551,j(n))}function et(t){t.modifiers&&t.modifiers.length&&n.bindDiagnostics.push(F(t,e.Diagnostics.Modifiers_cannot_appear_here));var r=e.isSourceFile(t.parent)?e.isExternalModule(t.parent)?t.parent.isDeclarationFile?void 0:e.Diagnostics.Global_module_exports_may_only_appear_in_declaration_files:e.Diagnostics.Global_module_exports_may_only_appear_in_module_files:e.Diagnostics.Global_module_exports_may_only_appear_at_top_level;r?n.bindDiagnostics.push(F(t,r)):(n.symbol.globalExports=n.symbol.globalExports||e.createSymbolTable(),H(n.symbol.globalExports,n.symbol,t,2097152,2097152))}function nt(n){l.symbol&&l.symbol.exports?n.exportClause?e.isNamespaceExport(n.exportClause)&&(e.setParent(n.exportClause,n),H(l.symbol.exports,l.symbol,n.exportClause,2097152,2097152)):H(l.symbol.exports,l.symbol,n,8388608,0):bn(n,8388608,j(n))}function tt(e){e.name&&ln(e,2097152,2097152)}function rt(e){return!n.externalModuleIndicator&&(n.commonJsModuleIndicator||(n.commonJsModuleIndicator=e,Zn()),!0)}function it(e){if(rt(e)){var n=Lt(e.arguments[0],void 0,function(e,n){return n&&V(n,e,67110400),n});if(n){var t=1048580;H(n.exports,n,e,t,0)}}}function at(n){if(rt(n)){var t=Lt(n.left.expression,void 0,function(e,n){return n&&V(n,e,67110400),n});if(t){var r=e.isAliasableExpression(n.right)&&(e.isExportsIdentifier(n.left.expression)||e.isModuleExportsAccessExpression(n.left.expression)),i=r?2097152:1048580;e.setParent(n.left,n),H(t.exports,t,n.left,i,0)}}}function ot(t){if(rt(t)){var r=e.getRightMostAssignedExpression(t.right);if(!(e.isEmptyObjectLiteral(r)||l===n&&_(n,r)))if(e.isObjectLiteralExpression(r)&&e.every(r.properties,e.isShorthandPropertyAssignment))e.forEach(r.properties,st);else{var i=e.exportAssignmentIsAlias(t)?2097152:1049092,a=H(n.symbol.exports,n.symbol,t,67108864|i,0);e.setValueDeclaration(a,t)}}}function st(e){H(n.symbol.exports,n.symbol,e,69206016,0)}function ct(n){e.Debug.assert(e.isInJSFile(n));var t=e.isBinaryExpression(n)&&e.isPropertyAccessExpression(n.left)&&e.isPrivateIdentifier(n.left.name)||e.isPropertyAccessExpression(n)&&e.isPrivateIdentifier(n.name);if(!t){var r=e.getThisContainer(n,!1);switch(r.kind){case 254:case 211:var i=r.symbol;if(e.isBinaryExpression(r.parent)&&63===r.parent.operatorToken.kind){var a=r.parent.left;e.isBindableStaticAccessExpression(a)&&e.isPrototypeAccess(a.expression)&&(i=xt(a.expression.expression,u))}i&&i.valueDeclaration&&(i.members=i.members||e.createSymbolTable(),e.hasDynamicName(n)?lt(n,i,i.members):H(i.members,i,n,67108868,0),V(i,i.valueDeclaration,32));break;case 169:case 165:case 167:case 170:case 171:var o=r.parent,s=e.isStatic(r)?o.symbol.exports:o.symbol.members;e.hasDynamicName(n)?lt(n,o.symbol,s):H(s,o.symbol,n,67108868,0,!0);break;case 300:if(e.hasDynamicName(n))break;r.commonJsModuleIndicator?H(r.symbol.exports,r.symbol,n,1048580,0):ln(n,1,111550);break;default:e.Debug.failBadSyntaxKind(r)}}}function lt(e,n,t){H(t,n,e,4,0,!0,!0),ut(e,n)}function ut(n,t){t&&(t.assignmentDeclarationMembers||(t.assignmentDeclarationMembers=new e.Map)).set(e.getNodeId(n),n)}function dt(n){108===n.expression.kind?ct(n):e.isBindableStaticAccessExpression(n)&&300===n.parent.parent.kind&&(e.isPrototypeAccess(n.expression)?mt(n,n.parent):yt(n))}function pt(n){e.setParent(n.left,n),e.setParent(n.right,n),Et(n.left.expression,n.left,!1,!0)}function ft(e){var n=xt(e.arguments[0].expression);n&&n.valueDeclaration&&V(n,n.valueDeclaration,32),vt(e,n,!0)}function mt(n,t){var r=n.expression,i=r.expression;e.setParent(i,r),e.setParent(r,n),e.setParent(n,t),Et(i,n,!0,!0)}function _t(e){var n=xt(e.arguments[0]),t=300===e.parent.parent.kind;n=ht(n,e.arguments[0],t,!1,!1),vt(e,n,!1)}function gt(t){var r,i=xt(t.left.expression,l)||xt(t.left.expression,d);if(e.isInJSFile(t)||e.isFunctionSymbol(i)){var a=e.getLeftmostAccessExpression(t.left);if(!(e.isIdentifier(a)&&2097152&(null===(r=g(l,a.escapedText))||void 0===r?void 0:r.flags)))if(e.setParent(t.left,t),e.setParent(t.right,t),e.isIdentifier(t.left.expression)&&l===n&&_(n,t.left.expression))at(t);else if(e.hasDynamicName(t)){bn(t,67108868,"__computed");var o=ht(i,t.left.expression,bt(t.left),!1,!1);ut(t,o)}else yt(e.cast(t.left,e.isBindableStaticNameExpression))}}function yt(n){e.Debug.assert(!e.isIdentifier(n)),e.setParent(n.expression,n),Et(n.expression,n,!1,!1)}function ht(t,r,i,a,o){if(2097152&(null===t||void 0===t?void 0:t.flags))return t;if(i&&!a){var s=67110400,c=110735;t=Lt(r,t,function(t,r,i){if(r)return V(r,t,s),r;var a=i?i.exports:n.jsGlobalAugmentations||(n.jsGlobalAugmentations=e.createSymbolTable());return H(a,i,t,s,c)})}return o&&t&&t.valueDeclaration&&V(t,t.valueDeclaration,32),t}function vt(n,t,r){if(t&&Tt(t)){var i=r?t.members||(t.members=e.createSymbolTable()):t.exports||(t.exports=e.createSymbolTable()),a=0,o=0;e.isFunctionLikeDeclaration(e.getAssignedExpandoInitializer(n))?(a=8192,o=103359):e.isCallExpression(n)&&e.isBindableObjectDefinePropertyCall(n)&&(e.some(n.arguments[2].properties,function(n){var t=e.getNameOfDeclaration(n);return!!t&&e.isIdentifier(t)&&"set"===e.idText(t)})&&(a|=65540,o|=78783),e.some(n.arguments[2].properties,function(n){var t=e.getNameOfDeclaration(n);return!!t&&e.isIdentifier(t)&&"get"===e.idText(t)})&&(a|=32772,o|=46015)),0===a&&(a=4,o=0),H(i,t,n,67108864|a,-67108865&o)}}function bt(n){return e.isBinaryExpression(n.parent)?300===St(n.parent).parent.kind:300===n.parent.parent.kind}function Et(e,n,t,r){var i=xt(e,l)||xt(e,d),a=bt(n);i=ht(i,n.expression,a,t,r),vt(n,i,t)}function Tt(n){if(1072&n.flags)return!0;var t=n.valueDeclaration;if(t&&e.isCallExpression(t))return!!e.getAssignedExpandoInitializer(t);var r=t?e.isVariableDeclaration(t)?t.initializer:e.isBinaryExpression(t)?t.right:e.isPropertyAccessExpression(t)&&e.isBinaryExpression(t.parent)?t.parent.right:void 0:void 0;if(r=r&&e.getRightMostAssignedExpression(r),r){var i=e.isPrototypeAccess(e.isVariableDeclaration(t)?t.name:e.isBinaryExpression(t)?t.left:t);return!!e.getExpandoInitializer(!e.isBinaryExpression(r)||56!==r.operatorToken.kind&&60!==r.operatorToken.kind?r:r.right,i)}return!1}function St(n){while(e.isBinaryExpression(n.parent))n=n.parent;return n.parent}function xt(n,t){if(void 0===t&&(t=l),e.isIdentifier(n))return g(t,n.escapedText);var r=xt(n.expression);return r&&r.exports&&r.exports.get(e.getElementOrPropertyAccessName(n))}function Lt(t,r,i){if(_(n,t))return n.symbol;if(e.isIdentifier(t))return i(t,xt(t),r);var a=Lt(t.expression,r,i),o=e.getNameOrArgument(t);return e.isPrivateIdentifier(o)&&e.Debug.fail("unexpected PrivateIdentifier"),i(o,a&&a.exports&&a.exports.get(e.getElementOrPropertyAccessName(t)),a)}function Dt(t){!n.commonJsModuleIndicator&&e.isRequireCall(t,!1)&&rt(t)}function At(t){if(255===t.kind)En(t,32,899503);else{var r=t.name?t.name.escapedText:"__class";bn(t,32,r),t.name&&k.add(t.name.escapedText)}var i=t.symbol,a=U(4194308,"prototype"),o=i.exports.get(a.escapedName);o&&(t.name&&e.setParent(t.name,t),n.bindDiagnostics.push(F(o.declarations[0],e.Diagnostics.Duplicate_identifier_0,e.symbolName(a)))),i.exports.set(a.escapedName,a),a.parent=i}function Ct(n){return e.isEnumConst(n)?En(n,128,899967):En(n,256,899327)}function It(n){I&&Nn(n,n.name),e.isBindingPattern(n.name)||(e.isInJSFile(n)&&e.isRequireVariableDeclaration(n)&&!e.getJSDocTypeTag(n)?ln(n,2097152,2097152):e.isBlockOrCatchScoped(n)?En(n,2,111551):e.isParameterDeclaration(n)?ln(n,1,111551):ln(n,1,111550))}function Nt(n){if((335!==n.kind||318===l.kind)&&(!I||8388608&n.flags||Nn(n,n.name),e.isBindingPattern(n.name)?bn(n,1,"__"+n.parent.parameters.indexOf(n)):ln(n,1,111551),e.isParameterPropertyDeclaration(n,n.parent))){var t=n.parent.parent;H(t.symbol.members,t.symbol,n,4|(n.questionToken?16777216:0),0)}}function kt(t){n.isDeclarationFile||8388608&t.flags||e.isAsyncFunction(t)&&(C|=2048),Rn(t),I?(wn(t),En(t,16,110991)):ln(t,16,110991)}function Rt(t){n.isDeclarationFile||8388608&t.flags||e.isAsyncFunction(t)&&(C|=2048),h&&(t.flowNode=h),Rn(t);var r=t.name?t.name.escapedText:"__function";return bn(t,16,r)}function Ot(t,r,i){return n.isDeclarationFile||8388608&t.flags||!e.isAsyncFunction(t)||(C|=2048),h&&e.isObjectLiteralOrClassExpressionMethod(t)&&(t.flowNode=h),e.hasDynamicName(t)?bn(t,r,"__computed"):ln(t,r,i)}function wt(n){var t=e.findAncestor(n,function(n){return n.parent&&e.isConditionalTypeNode(n.parent)&&n.parent.extendsType===n});return t&&t.parent}function Pt(n){if(e.isJSDocTemplateTag(n.parent)){var t=e.find(n.parent.parent.tags,e.isJSDocTypeAlias)||e.getHostSignatureFromJSDoc(n.parent);t?(t.locals||(t.locals=e.createSymbolTable()),H(t.locals,void 0,n,262144,526824)):ln(n,262144,526824)}else if(188===n.parent.kind){var r=wt(n.parent);r?(r.locals||(r.locals=e.createSymbolTable()),H(r.locals,void 0,n,262144,526824)):bn(n,262144,j(n))}else ln(n,262144,526824)}function Mt(n){var i=t(n);return 1===i||2===i&&e.shouldPreserveConstEnums(r)}function Ft(n){if(!(1&h.flags))return!1;if(h===w){var t=e.isStatementButNotDeclaration(n)&&234!==n.kind||255===n.kind||259===n.kind&&Mt(n);if(t&&(h=P,!r.allowUnreachableCode)){var i=e.unreachableCodeIsError(r)&&!(8388608&n.flags)&&(!e.isVariableStatement(n)||!!(3&e.getCombinedNodeFlags(n.declarationList))||n.declarationList.declarations.some(function(e){return!!e.initializer}));p(n,function(n,t){return jn(i,n,t,e.Diagnostics.Unreachable_code_detected)})}}return!0}}function p(n,t){if(e.isStatement(n)&&f(n)&&e.isBlock(n.parent)){var r=n.parent.statements,i=e.sliceAfter(r,n);e.getRangesWhere(i,f,function(e,n){return t(i[e],i[n-1])})}else t(n,n)}function f(n){return!e.isFunctionDeclaration(n)&&!m(n)&&!e.isEnumDeclaration(n)&&!(e.isVariableStatement(n)&&!(3&e.getCombinedNodeFlags(n))&&n.declarationList.declarations.some(function(e){return!e.initializer}))}function m(n){switch(n.kind){case 256:case 257:return!0;case 259:return 1!==t(n);case 258:return e.hasSyntacticModifier(n,2048);default:return!1}}function _(n,t){var r=0,i=[t];while(i.length&&r<100){if(r++,t=i.shift(),e.isExportsIdentifier(t)||e.isModuleExportsAccessExpression(t))return!0;if(e.isIdentifier(t)){var a=g(n,t.escapedText);if(a&&a.valueDeclaration&&e.isVariableDeclaration(a.valueDeclaration)&&a.valueDeclaration.initializer){var o=a.valueDeclaration.initializer;i.push(o),e.isAssignmentExpression(o,!0)&&(i.push(o.left),i.push(o.right))}}}return!1}function g(n,t){var r=n.locals&&n.locals.get(t);return r?r.exportSymbol||r:e.isSourceFile(n)&&n.jsGlobalAugmentations&&n.jsGlobalAugmentations.has(t)?n.jsGlobalAugmentations.get(t):n.symbol&&n.symbol.exports&&n.symbol.exports.get(t)}e.bindSourceFile=u,e.isExportsOrModuleExportsOrAlias=_}(d||(d={})),function(e){function n(n,t,r,i,a,o,s,c,l,u){return d;function d(d){void 0===d&&(d=function(){return!0});var p=[],f=[];return{walkType:function(n){try{return m(n),{visitedTypes:e.getOwnValues(p),visitedSymbols:e.getOwnValues(f)}}finally{e.clear(p),e.clear(f)}},walkSymbol:function(n){try{return x(n),{visitedTypes:e.getOwnValues(p),visitedSymbols:e.getOwnValues(f)}}finally{e.clear(p),e.clear(f)}}};function m(e){if(e&&!p[e.id]){p[e.id]=e;var n=x(e.symbol);if(!n){if(524288&e.flags){var t=e,r=t.objectFlags;4&r&&_(e),32&r&&b(e),3&r&&T(e),24&r&&S(t)}262144&e.flags&&g(e),3145728&e.flags&&y(e),4194304&e.flags&&h(e),8388608&e.flags&&v(e)}}}function _(n){m(n.target),e.forEach(u(n),m)}function g(e){m(c(e))}function y(n){e.forEach(n.types,m)}function h(e){m(e.type)}function v(e){m(e.objectType),m(e.indexType),m(e.constraint)}function b(e){m(e.typeParameter),m(e.constraintType),m(e.templateType),m(e.modifiersType)}function E(i){var a=t(i);a&&m(a.type),e.forEach(i.typeParameters,m);for(var o=0,s=i.parameters;o<s.length;o++){var c=s[o];x(c)}m(n(i)),m(r(i))}function T(n){S(n),e.forEach(n.typeParameters,m),e.forEach(i(n),m),m(n.thisType)}function S(e){for(var n=a(e),t=0,r=n.indexInfos;t<r.length;t++){var i=r[t];m(i.keyType),m(i.type)}for(var o=0,s=n.callSignatures;o<s.length;o++){var c=s[o];E(c)}for(var l=0,u=n.constructSignatures;l<u.length;l++){c=u[l];E(c)}for(var d=0,p=n.properties;d<p.length;d++){var f=p[d];x(f)}}function x(n){if(!n)return!1;var t=e.getSymbolId(n);if(f[t])return!1;if(f[t]=n,!d(n))return!0;var r=o(n);return m(r),n.exports&&n.exports.forEach(x),e.forEach(n.declarations,function(e){if(e.type&&179===e.type.kind){var n=e.type,t=s(l(n.exprName));x(t)}}),!1}}}e.createGetSymbolWalker=n}(d||(d={})),function(e){var n,t,r,i,s=/^".+"$/,l="(anonymous)",u=1,d=1,p=1,f=1;(function(e){e[e["AllowsSyncIterablesFlag"]=1]="AllowsSyncIterablesFlag",e[e["AllowsAsyncIterablesFlag"]=2]="AllowsAsyncIterablesFlag",e[e["AllowsStringInputFlag"]=4]="AllowsStringInputFlag",e[e["ForOfFlag"]=8]="ForOfFlag",e[e["YieldStarFlag"]=16]="YieldStarFlag",e[e["SpreadFlag"]=32]="SpreadFlag",e[e["DestructuringFlag"]=64]="DestructuringFlag",e[e["PossiblyOutOfBounds"]=128]="PossiblyOutOfBounds",e[e["Element"]=1]="Element",e[e["Spread"]=33]="Spread",e[e["Destructuring"]=65]="Destructuring",e[e["ForOf"]=13]="ForOf",e[e["ForAwaitOf"]=15]="ForAwaitOf",e[e["YieldStar"]=17]="YieldStar",e[e["AsyncYieldStar"]=19]="AsyncYieldStar",e[e["GeneratorReturnType"]=1]="GeneratorReturnType",e[e["AsyncGeneratorReturnType"]=2]="AsyncGeneratorReturnType"})(n||(n={})),function(e){e[e["Yield"]=0]="Yield",e[e["Return"]=1]="Return",e[e["Next"]=2]="Next"}(t||(t={})),function(e){e[e["Normal"]=0]="Normal",e[e["FunctionReturn"]=1]="FunctionReturn",e[e["GeneratorNext"]=2]="GeneratorNext",e[e["GeneratorYield"]=3]="GeneratorYield"}(r||(r={})),function(e){e[e["None"]=0]="None",e[e["TypeofEQString"]=1]="TypeofEQString",e[e["TypeofEQNumber"]=2]="TypeofEQNumber",e[e["TypeofEQBigInt"]=4]="TypeofEQBigInt",e[e["TypeofEQBoolean"]=8]="TypeofEQBoolean",e[e["TypeofEQSymbol"]=16]="TypeofEQSymbol",e[e["TypeofEQObject"]=32]="TypeofEQObject",e[e["TypeofEQFunction"]=64]="TypeofEQFunction",e[e["TypeofEQHostObject"]=128]="TypeofEQHostObject",e[e["TypeofNEString"]=256]="TypeofNEString",e[e["TypeofNENumber"]=512]="TypeofNENumber",e[e["TypeofNEBigInt"]=1024]="TypeofNEBigInt",e[e["TypeofNEBoolean"]=2048]="TypeofNEBoolean",e[e["TypeofNESymbol"]=4096]="TypeofNESymbol",e[e["TypeofNEObject"]=8192]="TypeofNEObject",e[e["TypeofNEFunction"]=16384]="TypeofNEFunction",e[e["TypeofNEHostObject"]=32768]="TypeofNEHostObject",e[e["EQUndefined"]=65536]="EQUndefined",e[e["EQNull"]=131072]="EQNull",e[e["EQUndefinedOrNull"]=262144]="EQUndefinedOrNull",e[e["NEUndefined"]=524288]="NEUndefined",e[e["NENull"]=1048576]="NENull",e[e["NEUndefinedOrNull"]=2097152]="NEUndefinedOrNull",e[e["Truthy"]=4194304]="Truthy",e[e["Falsy"]=8388608]="Falsy",e[e["All"]=16777215]="All",e[e["BaseStringStrictFacts"]=3735041]="BaseStringStrictFacts",e[e["BaseStringFacts"]=12582401]="BaseStringFacts",e[e["StringStrictFacts"]=16317953]="StringStrictFacts",e[e["StringFacts"]=16776705]="StringFacts",e[e["EmptyStringStrictFacts"]=12123649]="EmptyStringStrictFacts",e[e["EmptyStringFacts"]=12582401]="EmptyStringFacts",e[e["NonEmptyStringStrictFacts"]=7929345]="NonEmptyStringStrictFacts",e[e["NonEmptyStringFacts"]=16776705]="NonEmptyStringFacts",e[e["BaseNumberStrictFacts"]=3734786]="BaseNumberStrictFacts",e[e["BaseNumberFacts"]=12582146]="BaseNumberFacts",e[e["NumberStrictFacts"]=16317698]="NumberStrictFacts",e[e["NumberFacts"]=16776450]="NumberFacts",e[e["ZeroNumberStrictFacts"]=12123394]="ZeroNumberStrictFacts",e[e["ZeroNumberFacts"]=12582146]="ZeroNumberFacts",e[e["NonZeroNumberStrictFacts"]=7929090]="NonZeroNumberStrictFacts",e[e["NonZeroNumberFacts"]=16776450]="NonZeroNumberFacts",e[e["BaseBigIntStrictFacts"]=3734276]="BaseBigIntStrictFacts",e[e["BaseBigIntFacts"]=12581636]="BaseBigIntFacts",e[e["BigIntStrictFacts"]=16317188]="BigIntStrictFacts",e[e["BigIntFacts"]=16775940]="BigIntFacts",e[e["ZeroBigIntStrictFacts"]=12122884]="ZeroBigIntStrictFacts",e[e["ZeroBigIntFacts"]=12581636]="ZeroBigIntFacts",e[e["NonZeroBigIntStrictFacts"]=7928580]="NonZeroBigIntStrictFacts",e[e["NonZeroBigIntFacts"]=16775940]="NonZeroBigIntFacts",e[e["BaseBooleanStrictFacts"]=3733256]="BaseBooleanStrictFacts",e[e["BaseBooleanFacts"]=12580616]="BaseBooleanFacts",e[e["BooleanStrictFacts"]=16316168]="BooleanStrictFacts",e[e["BooleanFacts"]=16774920]="BooleanFacts",e[e["FalseStrictFacts"]=12121864]="FalseStrictFacts",e[e["FalseFacts"]=12580616]="FalseFacts",e[e["TrueStrictFacts"]=7927560]="TrueStrictFacts",e[e["TrueFacts"]=16774920]="TrueFacts",e[e["SymbolStrictFacts"]=7925520]="SymbolStrictFacts",e[e["SymbolFacts"]=16772880]="SymbolFacts",e[e["ObjectStrictFacts"]=7888800]="ObjectStrictFacts",e[e["ObjectFacts"]=16736160]="ObjectFacts",e[e["FunctionStrictFacts"]=7880640]="FunctionStrictFacts",e[e["FunctionFacts"]=16728e3]="FunctionFacts",e[e["UndefinedFacts"]=9830144]="UndefinedFacts",e[e["NullFacts"]=9363232]="NullFacts",e[e["EmptyObjectStrictFacts"]=16318463]="EmptyObjectStrictFacts",e[e["AllTypeofNE"]=556800]="AllTypeofNE",e[e["EmptyObjectFacts"]=16777215]="EmptyObjectFacts"}(i||(i={}));var m,_,g,y,h,v,b,E,T=new e.Map(e.getEntries({string:1,number:2,bigint:4,boolean:8,symbol:16,undefined:65536,object:32,function:64})),S=new e.Map(e.getEntries({string:256,number:512,bigint:1024,boolean:2048,symbol:4096,undefined:524288,object:8192,function:16384}));(function(e){e[e["Type"]=0]="Type",e[e["ResolvedBaseConstructorType"]=1]="ResolvedBaseConstructorType",e[e["DeclaredType"]=2]="DeclaredType",e[e["ResolvedReturnType"]=3]="ResolvedReturnType",e[e["ImmediateBaseConstraint"]=4]="ImmediateBaseConstraint",e[e["EnumTagType"]=5]="EnumTagType",e[e["ResolvedTypeArguments"]=6]="ResolvedTypeArguments",e[e["ResolvedBaseTypes"]=7]="ResolvedBaseTypes"})(m||(m={})),function(e){e[e["Normal"]=0]="Normal",e[e["Contextual"]=1]="Contextual",e[e["Inferential"]=2]="Inferential",e[e["SkipContextSensitive"]=4]="SkipContextSensitive",e[e["SkipGenericFunctions"]=8]="SkipGenericFunctions",e[e["IsForSignatureHelp"]=16]="IsForSignatureHelp"}(_||(_={})),function(e){e[e["BivariantCallback"]=1]="BivariantCallback",e[e["StrictCallback"]=2]="StrictCallback",e[e["IgnoreReturnTypes"]=4]="IgnoreReturnTypes",e[e["StrictArity"]=8]="StrictArity",e[e["Callback"]=3]="Callback"}(g||(g={})),function(e){e[e["None"]=0]="None",e[e["Source"]=1]="Source",e[e["Target"]=2]="Target",e[e["PropertyCheck"]=4]="PropertyCheck",e[e["UnionIntersectionCheck"]=8]="UnionIntersectionCheck",e[e["InPropertyCheck"]=16]="InPropertyCheck"}(y||(y={})),function(e){e[e["IncludeReadonly"]=1]="IncludeReadonly",e[e["ExcludeReadonly"]=2]="ExcludeReadonly",e[e["IncludeOptional"]=4]="IncludeOptional",e[e["ExcludeOptional"]=8]="ExcludeOptional"}(h||(h={})),function(e){e[e["None"]=0]="None",e[e["Source"]=1]="Source",e[e["Target"]=2]="Target",e[e["Both"]=3]="Both"}(v||(v={})),function(e){e["resolvedExports"]="resolvedExports",e["resolvedMembers"]="resolvedMembers"}(b||(b={})),function(e){e[e["Local"]=0]="Local",e[e["Parameter"]=1]="Parameter"}(E||(E={}));var x,L,D,A,C=e.and(G,F);(function(e){e[e["GetAccessor"]=1]="GetAccessor",e[e["SetAccessor"]=2]="SetAccessor",e[e["PropertyAssignment"]=4]="PropertyAssignment",e[e["Method"]=8]="Method",e[e["PrivateStatic"]=16]="PrivateStatic",e[e["GetOrSetAccessor"]=3]="GetOrSetAccessor",e[e["PropertyAssignmentOrMethod"]=12]="PropertyAssignmentOrMethod"})(x||(x={})),function(e){e[e["None"]=0]="None",e[e["ExportValue"]=1]="ExportValue",e[e["ExportType"]=2]="ExportType",e[e["ExportNamespace"]=4]="ExportNamespace"}(L||(L={})),function(e){e[e["None"]=0]="None",e[e["StrongArityForUntypedJS"]=1]="StrongArityForUntypedJS",e[e["VoidIsNonOptional"]=2]="VoidIsNonOptional"}(D||(D={})),function(e){e[e["Uppercase"]=0]="Uppercase",e[e["Lowercase"]=1]="Lowercase",e[e["Capitalize"]=2]="Capitalize",e[e["Uncapitalize"]=3]="Uncapitalize"}(A||(A={}));var I,N=new e.Map(e.getEntries({Uppercase:0,Lowercase:1,Capitalize:2,Uncapitalize:3}));function k(){}function R(){this.flags=0}function O(e){return e.id||(e.id=d,d++),e.id}function w(e){return e.id||(e.id=u,u++),e.id}function P(n,t){var r=e.getModuleInstanceState(n);return 1===r||t&&2===r}function M(n,t){var r,i,u,d,m=e.memoize(function(){var t=new e.Set;return n.getSourceFiles().forEach(function(n){n.resolvedModules&&e.forEachEntry(n.resolvedModules,function(e){e&&e.packageId&&t.add(e.packageId.name)})}),t}),_=e.objectAllocator.getSymbolConstructor(),g=e.objectAllocator.getTypeConstructor(),y=e.objectAllocator.getSignatureConstructor(),h=0,v=0,b=0,E=0,x=0,L=0,D=e.createSymbolTable(),A=[1],M=n.getCompilerOptions(),F=e.getEmitScriptTarget(M),K=e.getEmitModuleKind(M),H=e.getUseDefineForClassFields(M),W=e.getAllowSyntheticDefaultImports(M),z=e.getStrictOptionValue(M,"strictNullChecks"),J=e.getStrictOptionValue(M,"strictFunctionTypes"),q=e.getStrictOptionValue(M,"strictBindCallApply"),X=e.getStrictOptionValue(M,"strictPropertyInitialization"),Y=e.getStrictOptionValue(M,"noImplicitAny"),Q=e.getStrictOptionValue(M,"noImplicitThis"),Z=e.getStrictOptionValue(M,"useUnknownInCatchVariables"),$=!!M.keyofStringsOnly,ee=M.suppressExcessPropertyErrors?0:16384,ne=M.exactOptionalPropertyTypes,te=vA(),re=DO(),ie=To(),ae=e.createSymbolTable(),oe=Or(4,"undefined");oe.declarations=[];var se=Or(1536,"globalThis",8);se.exports=ae,se.declarations=[],ae.set(se.escapedName,se);var ce,le=Or(4,"arguments"),ue=Or(4,"require"),de={getNodeCount:function(){return e.sum(n.getSourceFiles(),"nodeCount")},getIdentifierCount:function(){return e.sum(n.getSourceFiles(),"identifierCount")},getSymbolCount:function(){return e.sum(n.getSourceFiles(),"symbolCount")+v},getTypeCount:function(){return h},getInstantiationCount:function(){return E},getRelationCacheSizes:function(){return{assignable:gr.size,identity:hr.size,subtype:mr.size,strictSubtype:_r.size}},isUndefinedSymbol:function(e){return e===oe},isArgumentsSymbol:function(e){return e===le},isUnknownSymbol:function(e){return e===Ae},getMergedSymbol:Aa,getDiagnostics:uR,getGlobalDiagnostics:pR,getRecursionIdentity:_y,getUnmatchedProperties:$h,getTypeOfSymbolAtLocation:function(n,t){var r=e.getParseTreeNode(t);return r?tE(n,r):Re},getSymbolsOfParameterPropertyDeclaration:function(n,t){var r=e.getParseTreeNode(n,e.isParameter);return void 0===r?e.Debug.fail("Cannot get symbols of a synthetic parameter that cannot be resolved to a parse-tree node."):qr(r,e.escapeLeadingUnderscores(t))},getDeclaredTypeOfSymbol:Dc,getPropertiesOfType:Hl,getPropertyOfType:function(n,t){return Lu(n,e.escapeLeadingUnderscores(t))},getPrivateIdentifierPropertyOfType:function(n,t,r){var i=e.getParseTreeNode(r);if(i){var a=e.escapeLeadingUnderscores(t),o=qS(a,i);return o?XS(n,o):void 0}},getTypeOfPropertyOfType:function(n,t){return Bo(n,e.escapeLeadingUnderscores(t))},getIndexInfoOfType:function(e,n){return Ou(e,0===n?je:Ke)},getIndexInfosOfType:Ru,getSignaturesOfType:Au,getIndexTypeOfType:function(e,n){return wu(e,0===n?je:Ke)},getBaseTypes:lc,getBaseTypeOfLiteralType:Vy,getWidenedType:Ch,getTypeFromTypeNode:function(n){var t=e.getParseTreeNode(n,e.isTypeNode);return t?u_(t):Re},getParameterType:pD,getParameterIdentifierNameAtPosition:cD,getPromisedTypeOfPromise:JC,getAwaitedType:function(e){return YC(e)},getReturnTypeOfSignature:ad,isNullableType:FS,getNullableType:oh,getNonNullableType:lh,getNonOptionalType:dh,getTypeArguments:Md,typeToTypeNode:ie.typeToTypeNode,indexInfoToIndexSignatureDeclaration:ie.indexInfoToIndexSignatureDeclaration,signatureToSignatureDeclaration:ie.signatureToSignatureDeclaration,symbolToEntityName:ie.symbolToEntityName,symbolToExpression:ie.symbolToExpression,symbolToTypeParameterDeclarations:ie.symbolToTypeParameterDeclarations,symbolToParameterDeclaration:ie.symbolToParameterDeclaration,typeParameterToDeclaration:ie.typeParameterToDeclaration,getSymbolsInScope:function(n,t){var r=e.getParseTreeNode(n);return r?mR(r,t):[]},getSymbolAtLocation:function(n){var t=e.getParseTreeNode(n);return t?CR(t,!0):void 0},getIndexInfosAtLocation:function(n){var t=e.getParseTreeNode(n);return t?IR(t):void 0},getShorthandAssignmentValueSymbol:function(n){var t=e.getParseTreeNode(n);return t?NR(t):void 0},getExportSpecifierLocalTargetSymbol:function(n){var t=e.getParseTreeNode(n,e.isExportSpecifier);return t?kR(t):void 0},getExportSymbolOfSymbol:function(e){return Aa(e.exportSymbol||e)},getTypeAtLocation:function(n){var t=e.getParseTreeNode(n);return t?RR(t):Re},getTypeOfAssignmentPattern:function(n){var t=e.getParseTreeNode(n,e.isAssignmentPattern);return t&&OR(t)||Re},getPropertySymbolOfDestructuringAssignment:function(n){var t=e.getParseTreeNode(n,e.isIdentifier);return t?wR(t):void 0},signatureToString:function(n,t,r,i){return _o(n,e.getParseTreeNode(t),r,i)},typeToString:function(n,t,r){return go(n,e.getParseTreeNode(t),r)},symbolToString:function(n,t,r,i){return mo(n,e.getParseTreeNode(t),r,i)},typePredicateToString:function(n,t,r){return So(n,e.getParseTreeNode(t),r)},writeSignature:function(n,t,r,i,a){return _o(n,e.getParseTreeNode(t),r,i,a)},writeType:function(n,t,r,i){return go(n,e.getParseTreeNode(t),r,i)},writeSymbol:function(n,t,r,i,a){return mo(n,e.getParseTreeNode(t),r,i,a)},writeTypePredicate:function(n,t,r,i){return So(n,e.getParseTreeNode(t),r,i)},getAugmentedPropertiesOfType:GR,getRootSymbols:UR,getSymbolOfExpando:UL,getContextualType:function(n,t){var r=e.getParseTreeNode(n,e.isExpression);if(r){var i=e.findAncestor(r,e.isCallLikeExpression),a=i&&Wr(i).resolvedSignature;if(4&t&&i){var o=r;do{Wr(o).skipDirectInference=!0,o=o.parent}while(o&&o!==i);Wr(i).resolvedSignature=void 0}var s=vT(r,t);if(4&t&&i){o=r;do{Wr(o).skipDirectInference=void 0,o=o.parent}while(o&&o!==i);Wr(i).resolvedSignature=a}return s}},getContextualTypeForObjectLiteralElement:function(n){var t=e.getParseTreeNode(n,e.isObjectLiteralElementLike);return t?sT(t):void 0},getContextualTypeForArgumentAtIndex:function(n,t){var r=e.getParseTreeNode(n,e.isCallLikeExpression);return r&&QE(r,t)},getContextualTypeForJsxAttribute:function(n){var t=e.getParseTreeNode(n,e.isJsxAttributeLike);return t&&pT(t)},isContextSensitive:$_,getTypeOfPropertyOfContextualType:aT,getFullyQualifiedName:ta,getResolvedSignature:function(e,n,t){return pe(e,n,t,0)},getResolvedSignatureForSignatureHelp:function(e,n,t){return pe(e,n,t,16)},getExpandedParameters:il,hasEffectiveRestParameter:yD,containsArgumentsReference:Zu,getConstantValue:function(n){var t=e.getParseTreeNode(n,dO);return t?pO(t):void 0},isValidPropertyAccess:function(n,t){var r=e.getParseTreeNode(n,e.isPropertyAccessOrQualifiedNameOrImportTypeNode);return!!r&&Tx(r,e.escapeLeadingUnderscores(t))},isValidPropertyAccessForCompletions:function(n,t,r){var i=e.getParseTreeNode(n,e.isPropertyAccessExpression);return!!i&&Sx(i,t,r)},getSignatureFromDeclaration:function(n){var t=e.getParseTreeNode(n,e.isFunctionLike);return t?qu(t):void 0},isImplementationOfOverload:function(n){var t=e.getParseTreeNode(n,e.isFunctionLike);return t?rO(t):void 0},getImmediateAliasedSymbol:XT,getAliasedSymbol:Ji,getEmitResolver:Tr,getExportsOfModule:ya,getExportsAndPropertiesOfModule:ha,forEachExportAndPropertyOfModule:va,getSymbolWalker:e.createGetSymbolWalker(cd,td,ad,lc,Ul,Ws,xv,ql,e.getFirstIdentifier,Md),getAmbientModules:qw,getJsxIntrinsicTagNamesAt:LS,isOptionalParameter:function(n){var t=e.getParseTreeNode(n,e.isParameter);return!!t&&ju(t)},tryGetMemberInModuleExports:function(n,t){return ba(e.escapeLeadingUnderscores(n),t)},tryGetMemberInModuleExportsAndProperties:function(n,t){return Ea(e.escapeLeadingUnderscores(n),t)},tryFindAmbientModule:function(e){return Vu(e,!0)},tryFindAmbientModuleWithoutAugmentations:function(e){return Vu(e,!1)},getApparentType:mu,getUnionType:ff,isTypeAssignableTo:lg,createAnonymousType:Xa,createSignature:$c,createSymbol:Or,createIndexInfo:Td,getAnyType:function(){return Ie},getStringType:function(){return je},getNumberType:function(){return Ke},createPromiseType:AD,createArrayType:Mp,getElementTypeOfArrayType:Cy,getBooleanType:function(){return Xe},getFalseType:function(e){return e?We:ze},getTrueType:function(e){return e?Je:qe},getVoidType:function(){return Qe},getUndefinedType:function(){return Me},getNullType:function(){return Ue},getESSymbolType:function(){return Ye},getNeverType:function(){return Ze},getOptionalType:function(){return Ge},isSymbolAccessible:oo,isArrayType:Ly,isTupleType:Jy,isArrayLikeType:Iy,isTypeInvalidDueToUnionDiscriminant:Wl,getAllPossiblePropertiesOfTypes:zl,getSuggestedSymbolForNonexistentProperty:dx,getSuggestionForNonexistentProperty:fx,getSuggestedSymbolForNonexistentJSXAttribute:px,getSuggestedSymbolForNonexistentSymbol:function(n,t,r){return mx(n,e.escapeLeadingUnderscores(t),r)},getSuggestionForNonexistentSymbol:function(n,t,r){return _x(n,e.escapeLeadingUnderscores(t),r)},getSuggestedSymbolForNonexistentModule:gx,getSuggestionForNonexistentExport:yx,getSuggestedSymbolForNonexistentClassMember:ux,getBaseConstraintOfType:ru,getDefaultFromTypeParameter:function(e){return e&&262144&e.flags?lu(e):void 0},resolveName:function(n,t,r,i){return Qr(t,e.escapeLeadingUnderscores(n),r,void 0,void 0,!1,i)},getJsxNamespace:function(n){return e.unescapeLeadingUnderscores(Er(n))},getJsxFragmentFactory:function(n){var t=LO(n);return t&&e.unescapeLeadingUnderscores(e.getFirstIdentifier(t).escapedText)},getAccessibleSymbolChain:$a,getTypePredicateOfSignature:td,resolveExternalModuleName:function(n){var t=e.getParseTreeNode(n,e.isExpression);return t&&ca(t,t,!0)},resolveExternalModuleSymbol:fa,tryGetThisTypeAt:function(n,t){var r=e.getParseTreeNode(n);return r&&IE(r,t)},getTypeArgumentConstraint:function(n){var t=e.getParseTreeNode(n,e.isTypeNode);return t&&DC(t)},getSuggestionDiagnostics:function(t,i){var a,s=e.getParseTreeNode(t,e.isSourceFile)||e.Debug.fail("Could not determine parsed source file.");if(e.skipTypeChecking(s,M,n))return e.emptyArray;try{return r=i,oR(s),e.Debug.assert(!!(1&Wr(s).flags)),a=e.addRange(a,dr.getDiagnostics(s.fileName)),vI(cR(s),function(n,t,r){e.containsParseError(n)||sR(t,!!(8388608&n.flags))||(a||(a=[])).push(o(o({},r),{category:e.DiagnosticCategory.Suggestion}))}),a||e.emptyArray}finally{r=void 0}},runWithCancellationToken:function(e,n){try{return r=e,n(de)}finally{r=void 0}},getLocalTypeParametersOfClassOrInterfaceOrTypeAlias:$s,isDeclarationVisible:ko};function pe(n,t,r,i){var a=e.getParseTreeNode(n,e.isCallLikeExpression);ce=r;var o=a?ML(a,t,i):void 0;return ce=void 0,o}var fe=new e.Map,me=new e.Map,_e=new e.Map,ge=new e.Map,ye=new e.Map,he=new e.Map,ve=new e.Map,be=new e.Map,Ee=new e.Map,Te=new e.Map,Se=new e.Map,xe=new e.Map,Le=[],De=new e.Map,Ae=Or(4,"unknown"),Ce=Or(0,"__resolving__"),Ie=Va(1,"any"),Ne=Va(1,"any"),ke=Va(1,"any"),Re=Va(1,"error"),Oe=Va(1,"any",131072),we=Va(1,"intrinsic"),Pe=Va(2,"unknown"),Me=Va(32768,"undefined"),Fe=z?Me:Va(32768,"undefined",131072),Ge=Va(32768,"undefined"),Be=ne?Va(32768,"undefined"):Me,Ue=Va(65536,"null"),Ve=z?Ue:Va(65536,"null",131072),je=Va(4,"string"),Ke=Va(8,"number"),He=Va(64,"bigint"),We=Va(512,"false"),ze=Va(512,"false"),Je=Va(512,"true"),qe=Va(512,"true");Je.regularType=qe,Je.freshType=Je,qe.regularType=qe,qe.freshType=Je,We.regularType=ze,We.freshType=We,ze.regularType=ze,ze.freshType=We;var Xe=ff([ze,qe]),Ye=Va(4096,"symbol"),Qe=Va(16384,"void"),Ze=Va(131072,"never"),$e=Va(131072,"never"),en=Va(131072,"never",524288),nn=Va(131072,"never"),tn=Va(131072,"never"),rn=Va(67108864,"object"),an=ff([je,Ke]),on=ff([je,Ke,Ye]),sn=$?je:on,cn=ff([Ke,He]),ln=ff([je,Ke,Xe,He,Ue,Me]),un=b_(function(e){return 262144&e.flags?C_(e):e}),dn=b_(function(e){return 262144&e.flags?ke:e}),pn=Xa(void 0,D,e.emptyArray,e.emptyArray,e.emptyArray),fn=Xa(void 0,D,e.emptyArray,e.emptyArray,e.emptyArray);fn.objectFlags|=2048;var mn=Or(2048,"__type");mn.members=e.createSymbolTable();var _n=Xa(mn,D,e.emptyArray,e.emptyArray,e.emptyArray),gn=Xa(void 0,D,e.emptyArray,e.emptyArray,e.emptyArray);gn.instantiations=new e.Map;var yn=Xa(void 0,D,e.emptyArray,e.emptyArray,e.emptyArray);yn.objectFlags|=524288;var hn=Xa(void 0,D,e.emptyArray,e.emptyArray,e.emptyArray),vn=Xa(void 0,D,e.emptyArray,e.emptyArray,e.emptyArray),bn=Xa(void 0,D,e.emptyArray,e.emptyArray,e.emptyArray),En=Ha(),Tn=Ha();Tn.constraint=En;var Sn,xn,Ln,Dn,An,Cn,In,Nn,kn,Rn,On,wn,Pn,Mn,Fn,Gn,Bn,Un,Vn,jn,Kn,Hn,Wn,zn,Jn,qn,Xn,Yn,Qn,Zn,$n,et,nt,tt,rt,it,at,ot,st,ct,lt,ut,dt,pt,ft,mt,_t,gt,yt,ht=Ha(),vt=Wu(1,"<<unresolved>>",0,Ie),bt=$c(void 0,void 0,void 0,e.emptyArray,Ie,void 0,0,0),Et=$c(void 0,void 0,void 0,e.emptyArray,Re,void 0,0,0),Tt=$c(void 0,void 0,void 0,e.emptyArray,Ie,void 0,0,0),St=$c(void 0,void 0,void 0,e.emptyArray,$e,void 0,0,0),xt=Td(Ke,je,!0),Lt=new e.Map,Dt={get yieldType(){return e.Debug.fail("Not supported")},get returnType(){return e.Debug.fail("Not supported")},get nextType(){return e.Debug.fail("Not supported")}},At=yN(Ie,Ie,Ie),Ct=yN(Ie,Ie,Pe),It=yN(Ze,Ie,Me),Nt={iterableCacheKey:"iterationTypesOfAsyncIterable",iteratorCacheKey:"iterationTypesOfAsyncIterator",iteratorSymbolName:"asyncIterator",getGlobalIteratorType:bp,getGlobalIterableType:vp,getGlobalIterableIteratorType:Ep,getGlobalGeneratorType:Tp,resolveIterationType:YC,mustHaveANextMethodDiagnostic:e.Diagnostics.An_async_iterator_must_have_a_next_method,mustBeAMethodDiagnostic:e.Diagnostics.The_0_property_of_an_async_iterator_must_be_a_method,mustHaveAValueDiagnostic:e.Diagnostics.The_type_returned_by_the_0_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value_property},kt={iterableCacheKey:"iterationTypesOfIterable",iteratorCacheKey:"iterationTypesOfIterator",iteratorSymbolName:"iterator",getGlobalIteratorType:xp,getGlobalIterableType:Sp,getGlobalIterableIteratorType:Lp,getGlobalGeneratorType:Dp,resolveIterationType:function(e,n){return e},mustHaveANextMethodDiagnostic:e.Diagnostics.An_iterator_must_have_a_next_method,mustBeAMethodDiagnostic:e.Diagnostics.The_0_property_of_an_iterator_must_be_a_method,mustHaveAValueDiagnostic:e.Diagnostics.The_type_returned_by_the_0_method_of_an_iterator_must_have_a_value_property},Rt=new e.Map,Ot=!1,wt=new e.Map,Pt=0,Mt=0,Ft=0,Gt=!1,Bt=0,Ut=Zm(""),Vt=$m(0),jt=e_({negative:!1,base10Value:"0"}),Kt=[],Ht=[],Wt=[],zt=0,Jt=10,qt=[],Xt=[],Yt=[],Qt=[],Zt=[],$t=[],er=[],nr=[],tr=[],rr=[],ir=[],ar=[],or=[],sr=[],cr=[],lr=[],ur=e.createDiagnosticCollection(),dr=e.createDiagnosticCollection(),pr=new e.Map(e.getEntries({string:je,number:Ke,bigint:He,boolean:Xe,symbol:Ye,undefined:Me})),fr=Ka(),mr=new e.Map,_r=new e.Map,gr=new e.Map,yr=new e.Map,hr=new e.Map,vr=new e.Map,br=e.createSymbolTable();return br.set(oe.escapedName,oe),CO(),de;function Er(n){if(n){var t=e.getSourceFileOfNode(n);if(t)if(e.isJsxOpeningFragment(n)){if(t.localJsxFragmentNamespace)return t.localJsxFragmentNamespace;var r=t.pragmas.get("jsxfrag");if(r){var i=e.isArray(r)?r[0]:r;if(t.localJsxFragmentFactory=e.parseIsolatedEntityName(i.arguments.factory,F),e.visitNode(t.localJsxFragmentFactory,s),t.localJsxFragmentFactory)return t.localJsxFragmentNamespace=e.getFirstIdentifier(t.localJsxFragmentFactory).escapedText}var a=LO(n);if(a)return t.localJsxFragmentFactory=a,t.localJsxFragmentNamespace=e.getFirstIdentifier(a).escapedText}else{if(t.localJsxNamespace)return t.localJsxNamespace;var o=t.pragmas.get("jsx");if(o){i=e.isArray(o)?o[0]:o;if(t.localJsxFactory=e.parseIsolatedEntityName(i.arguments.factory,F),e.visitNode(t.localJsxFactory,s),t.localJsxFactory)return t.localJsxNamespace=e.getFirstIdentifier(t.localJsxFactory).escapedText}}}return _t||(_t="React",M.jsxFactory?(gt=e.parseIsolatedEntityName(M.jsxFactory,F),e.visitNode(gt,s),gt&&(_t=e.getFirstIdentifier(gt).escapedText)):M.reactNamespace&&(_t=e.escapeLeadingUnderscores(M.reactNamespace))),gt||(gt=e.factory.createQualifiedName(e.factory.createIdentifier(e.unescapeLeadingUnderscores(_t)),"createElement")),_t;function s(n){return e.setTextRangePosEnd(n,-1,-1),e.visitEachChild(n,s,e.nullTransformationContext)}}function Tr(e,n){return uR(e,n),re}function Sr(n,t,r,i,a,o){var s=n?e.createDiagnosticForNode(n,t,r,i,a,o):e.createCompilerDiagnostic(t,r,i,a,o),c=ur.lookup(s);return c||(ur.add(s),s)}function xr(e,n,t,r,i,a,o){var s=Dr(n,t,r,i,a,o);return s.skippedOn=e,s}function Lr(n,t,r,i,a,o){return n?e.createDiagnosticForNode(n,t,r,i,a,o):e.createCompilerDiagnostic(t,r,i,a,o)}function Dr(e,n,t,r,i,a){var o=Lr(e,n,t,r,i,a);return ur.add(o),o}function Ar(n,t){n?ur.add(t):dr.add(o(o({},t),{category:e.DiagnosticCategory.Suggestion}))}function Cr(n,t,r,i,a,o,s){if(t.pos<0||t.end<0){if(!n)return;var c=e.getSourceFileOfNode(t);Ar(n,"message"in r?e.createFileDiagnostic(c,0,0,r,i,a,o,s):e.createDiagnosticForFileFromMessageChain(c,r))}else Ar(n,"message"in r?e.createDiagnosticForNode(t,r,i,a,o,s):e.createDiagnosticForNodeFromMessageChain(t,r))}function Ir(n,t,r,i,a,o,s){var c=Dr(n,r,i,a,o,s);if(t){var l=e.createDiagnosticForNode(n,e.Diagnostics.Did_you_forget_to_use_await);e.addRelatedInfo(c,l)}return c}function Nr(n,t){var r=Array.isArray(n)?e.forEach(n,e.getJSDocDeprecatedTag):e.getJSDocDeprecatedTag(n);return r&&e.addRelatedInfo(t,e.createDiagnosticForNode(r,e.Diagnostics.The_declaration_was_marked_as_deprecated_here)),dr.add(t),t}function kr(n,t,r){var i=e.createDiagnosticForNode(n,e.Diagnostics._0_is_deprecated,r);return Nr(t,i)}function Rr(n,t,r,i){var a=r?e.createDiagnosticForNode(n,e.Diagnostics.The_signature_0_of_1_is_deprecated,i,r):e.createDiagnosticForNode(n,e.Diagnostics._0_is_deprecated,i);return Nr(t,a)}function Or(e,n,t){v++;var r=new _(33554432|e,n);return r.checkFlags=t||0,r}function wr(e){var n=0;return 2&e&&(n|=111551),1&e&&(n|=111550),4&e&&(n|=0),8&e&&(n|=900095),16&e&&(n|=110991),32&e&&(n|=899503),64&e&&(n|=788872),256&e&&(n|=899327),128&e&&(n|=899967),512&e&&(n|=110735),8192&e&&(n|=103359),32768&e&&(n|=46015),65536&e&&(n|=78783),262144&e&&(n|=526824),524288&e&&(n|=788968),2097152&e&&(n|=2097152),n}function Pr(e,n){n.mergeId||(n.mergeId=p,p++),qt[n.mergeId]=e}function Mr(n){var t=Or(n.flags,n.escapedName);return t.declarations=n.declarations?n.declarations.slice():[],t.parent=n.parent,n.valueDeclaration&&(t.valueDeclaration=n.valueDeclaration),n.constEnumOnlyModule&&(t.constEnumOnlyModule=!0),n.members&&(t.members=new e.Map(n.members)),n.exports&&(t.exports=new e.Map(n.exports)),Pr(t,n),t}function Fr(n,t,r){if(void 0===r&&(r=!1),!(n.flags&wr(t.flags))||67108864&(t.flags|n.flags)){if(t===n)return n;if(!(33554432&n.flags)){var i=zi(n);if(i===Ae)return t;n=Mr(i)}512&t.flags&&512&n.flags&&n.constEnumOnlyModule&&!t.constEnumOnlyModule&&(n.constEnumOnlyModule=!1),n.flags|=t.flags,t.valueDeclaration&&e.setValueDeclaration(n,t.valueDeclaration),e.addRange(n.declarations,t.declarations),t.members&&(n.members||(n.members=e.createSymbolTable()),Vr(n.members,t.members,r)),t.exports&&(n.exports||(n.exports=e.createSymbolTable()),Vr(n.exports,t.exports,r)),r||Pr(n,t)}else if(1024&n.flags)n!==se&&Dr(t.declarations&&e.getNameOfDeclaration(t.declarations[0]),e.Diagnostics.Cannot_augment_module_0_with_value_exports_because_it_resolves_to_a_non_module_entity,mo(n));else{var a=!!(384&n.flags||384&t.flags),o=!!(2&n.flags||2&t.flags),s=a?e.Diagnostics.Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations:o?e.Diagnostics.Cannot_redeclare_block_scoped_variable_0:e.Diagnostics.Duplicate_identifier_0,c=t.declarations&&e.getSourceFileOfNode(t.declarations[0]),l=n.declarations&&e.getSourceFileOfNode(n.declarations[0]),u=mo(t);if(c&&l&&Sn&&!a&&c!==l){var d=-1===e.comparePaths(c.path,l.path)?c:l,p=d===c?l:c,f=e.getOrUpdate(Sn,d.path+"|"+p.path,function(){return{firstFile:d,secondFile:p,conflictingSymbols:new e.Map}}),m=e.getOrUpdate(f.conflictingSymbols,u,function(){return{isBlockScoped:o,firstFileLocations:[],secondFileLocations:[]}});_(m.firstFileLocations,t),_(m.secondFileLocations,n)}else Gr(t,s,u,n),Gr(n,s,u,t)}return n;function _(n,t){if(t.declarations)for(var r=0,i=t.declarations;r<i.length;r++){var a=i[r];e.pushIfUnique(n,a)}}}function Gr(n,t,r,i){e.forEach(n.declarations,function(e){Br(e,t,r,i.declarations)})}function Br(n,t,r,i){for(var a=(e.getExpandoInitializer(n,!1)?e.getNameOfExpando(n):e.getNameOfDeclaration(n))||n,o=Sr(a,t,r),s=function(n){var t=(e.getExpandoInitializer(n,!1)?e.getNameOfExpando(n):e.getNameOfDeclaration(n))||n;if(t===a)return"continue";o.relatedInformation=o.relatedInformation||[];var i=e.createDiagnosticForNode(t,e.Diagnostics._0_was_also_declared_here,r),s=e.createDiagnosticForNode(t,e.Diagnostics.and_here);if(e.length(o.relatedInformation)>=5||e.some(o.relatedInformation,function(n){return 0===e.compareDiagnostics(n,s)||0===e.compareDiagnostics(n,i)}))return"continue";e.addRelatedInfo(o,e.length(o.relatedInformation)?s:i)},c=0,l=i||e.emptyArray;c<l.length;c++){var u=l[c];s(u)}}function Ur(n,t){if(!(null===n||void 0===n?void 0:n.size))return t;if(!(null===t||void 0===t?void 0:t.size))return n;var r=e.createSymbolTable();return Vr(r,n),Vr(r,t),r}function Vr(e,n,t){void 0===t&&(t=!1),n.forEach(function(n,r){var i=e.get(r);e.set(r,i?Fr(i,n,t):n)})}function jr(n){var t,r,i,a=n.parent;if((null===(t=a.symbol.declarations)||void 0===t?void 0:t[0])===a)if(e.isGlobalScopeAugmentation(a))Vr(ae,a.symbol.exports);else{var o=8388608&n.parent.parent.flags?void 0:e.Diagnostics.Invalid_module_name_in_augmentation_module_0_cannot_be_found,s=la(n,n,o,!0);if(!s)return;if(s=fa(s),1920&s.flags)if(e.some(Ln,function(e){return s===e.symbol})){var c=Fr(a.symbol,s,!0);Dn||(Dn=new e.Map),Dn.set(n.text,c)}else{if((null===(r=s.exports)||void 0===r?void 0:r.get("__export"))&&(null===(i=a.symbol.exports)||void 0===i?void 0:i.size))for(var l=zc(s,"resolvedExports"),u=0,d=e.arrayFrom(a.symbol.exports.entries());u<d.length;u++){var p=d[u],f=p[0],m=p[1];l.has(f)&&!s.exports.has(f)&&Fr(l.get(f),m)}Fr(s,a.symbol)}else Dr(n,e.Diagnostics.Cannot_augment_module_0_because_it_resolves_to_a_non_module_entity,n.text)}else e.Debug.assert(a.symbol.declarations.length>1)}function Kr(n,t,r){function i(n,t){return function(r){return ur.add(e.createDiagnosticForNode(r,t,n))}}t.forEach(function(t,a){var o=n.get(a);o?e.forEach(o.declarations,i(e.unescapeLeadingUnderscores(a),r)):n.set(a,t)})}function Hr(e){if(33554432&e.flags)return e;var n=w(e);return Xt[n]||(Xt[n]=new k)}function Wr(e){var n=O(e);return Yt[n]||(Yt[n]=new R)}function zr(n){return 300===n.kind&&!e.isExternalOrCommonJsModule(n)}function Jr(n,t,r){if(r){var i=Aa(n.get(t));if(i){if(e.Debug.assert(0===(1&e.getCheckFlags(i)),"Should never get an instantiated symbol here."),i.flags&r)return i;if(2097152&i.flags){var a=Ji(i);if(a===Ae||a.flags&r)return i}}}}function qr(n,t){var r=n.parent,i=n.parent.parent,a=Jr(r.locals,t,111551),o=Jr(Jc(i.symbol),t,111551);return a&&o?[a,o]:e.Debug.fail("There should exist two symbols, one as property declaration and one as parameter declaration")}function Xr(t,r){var i=e.getSourceFileOfNode(t),a=e.getSourceFileOfNode(r),o=e.getEnclosingBlockScopeContainer(t);if(i!==a){if(K&&(i.externalModuleIndicator||a.externalModuleIndicator)||!e.outFile(M)||Lv(r)||8388608&t.flags)return!0;if(d(r,t))return!0;var s=n.getSourceFiles();return s.indexOf(i)<=s.indexOf(a)}if(t.pos<=r.pos&&(!e.isPropertyDeclaration(t)||!e.isThisProperty(r.parent)||t.initializer||t.exclamationToken)){if(201===t.kind){var c=e.getAncestor(r,201);return c?e.findAncestor(c,e.isBindingElement)!==e.findAncestor(t,e.isBindingElement)||t.pos<c.pos:Xr(e.getAncestor(t,252),r)}return 252===t.kind?!u(t,r):e.isClassDeclaration(t)?!e.findAncestor(r,function(n){return e.isComputedPropertyName(n)&&n.parent.parent===t}):e.isPropertyDeclaration(t)?!p(t,r,!1):!e.isParameterPropertyDeclaration(t,t.parent)||!(99===M.target&&H&&e.getContainingClass(t)===e.getContainingClass(r)&&d(r,t))}return!!(273===r.parent.kind||269===r.parent.kind&&r.parent.isExportEquals)||(!(269!==r.kind||!r.isExportEquals)||(!!(4194304&r.flags||Lv(r)||l())||!!d(r,t)&&(99!==M.target||!H||!e.getContainingClass(t)||!e.isPropertyDeclaration(t)&&!e.isParameterPropertyDeclaration(t,t.parent)||!p(t,r,!0))));function l(){return!!e.findAncestor(r,function(n){return e.isInterfaceDeclaration(n)||e.isTypeAliasDeclaration(n)})}function u(n,t){switch(n.parent.parent.kind){case 235:case 240:case 242:if(gi(t,n,o))return!0;break}var r=n.parent.parent;return e.isForInOrOfStatement(r)&&gi(t,r.expression,o)}function d(n,t){return!!e.findAncestor(n,function(r){if(r===o)return"quit";if(e.isFunctionLike(r))return!0;if(e.isClassStaticBlockDeclaration(r))return t.pos<n.pos;var i=e.tryCast(r.parent,e.isPropertyDeclaration);if(i){var a=i.initializer===r;if(a)if(e.isStatic(r.parent)){if(167===t.kind)return!0;if(e.isPropertyDeclaration(t)&&e.getContainingClass(n)===e.getContainingClass(t)){var s=t.name;if(e.isIdentifier(s)||e.isPrivateIdentifier(s)){var c=Ws(Ca(t)),l=e.filter(t.parent.members,e.isClassStaticBlockDeclaration);if(vk(s,c,l,t.parent.pos,r.pos))return!0}}}else{var u=165===t.kind&&!e.isStatic(t);if(!u||e.getContainingClass(n)!==e.getContainingClass(t))return!0}}return!1})}function p(n,t,r){if(t.end>n.end)return!1;var i=e.findAncestor(t,function(t){if(t===n)return"quit";switch(t.kind){case 212:return!0;case 165:return!r||!(e.isPropertyDeclaration(n)&&t.parent===n.parent||e.isParameterPropertyDeclaration(n,n.parent)&&t.parent===n.parent.parent)||"quit";case 233:switch(t.parent.kind){case 170:case 167:case 171:return!0;default:return!1}default:return!1}});return void 0===i}}function Yr(n,t,r){var i=e.getEmitScriptTarget(M),a=t;if(e.isParameter(r)&&a.body&&n.valueDeclaration&&n.valueDeclaration.pos>=a.body.pos&&n.valueDeclaration.end<=a.body.end&&i>=2){var o=Wr(a);return void 0===o.declarationRequiresScopeChange&&(o.declarationRequiresScopeChange=e.forEach(a.parameters,s)||!1),!o.declarationRequiresScopeChange}return!1;function s(e){return c(e.name)||!!e.initializer&&c(e.initializer)}function c(n){switch(n.kind){case 212:case 211:case 254:case 169:return!1;case 167:case 170:case 171:case 291:return c(n.name);case 165:return e.hasStaticModifier(n)?i<99||!H:c(n.name);default:return e.isNullishCoalesce(n)||e.isOptionalChain(n)?i<7:e.isBindingElement(n)&&n.dotDotDotToken&&e.isObjectBindingPattern(n.parent)?i<4:!e.isTypeNode(n)&&(e.forEachChild(n,c)||!1)}}}function Qr(e,n,t,r,i,a,o,s){return void 0===o&&(o=!1),Zr(e,n,t,r,i,a,o,Jr,s)}function Zr(n,t,r,i,a,o,s,c,l){var u,d,p,f,m,_,g,y=n,h=!1,v=n,b=!1;e:while(n){if(n.locals&&!zr(n)&&(d=c(n.locals,t,r))){var E=!0;if(e.isFunctionLike(n)&&p&&p!==n.body?(r&d.flags&788968&&315!==p.kind&&(E=!!(262144&d.flags)&&(p===n.type||162===p.kind||161===p.kind)),r&d.flags&3&&(Yr(d,n,p)?E=!1:1&d.flags&&(E=162===p.kind||p===n.type&&!!e.findAncestor(d.valueDeclaration,e.isParameter)))):187===n.kind&&(E=p===n.trueType),E)break e;d=void 0}switch(h=h||ei(n,p),n.kind){case 300:if(!e.isExternalOrCommonJsModule(n))break;b=!0;case 259:var T=Ca(n).exports||D;if(300===n.kind||e.isModuleDeclaration(n)&&8388608&n.flags&&!e.isGlobalScopeAugmentation(n)){if(d=T.get("default")){var S=e.getLocalSymbolForExportDefault(d);if(S&&d.flags&r&&S.escapedName===t)break e;d=void 0}var x=T.get(t);if(x&&2097152===x.flags&&(e.getDeclarationOfKind(x,273)||e.getDeclarationOfKind(x,272)))break}if("default"!==t&&(d=c(T,t,2623475&r))){if(!e.isSourceFile(n)||!n.commonJsModuleIndicator||(null===(u=d.declarations)||void 0===u?void 0:u.some(e.isJSDocTypeAlias)))break e;d=void 0}break;case 258:if(d=c(Ca(n).exports,t,8&r))break e;break;case 165:if(!e.isStatic(n)){var L=Ga(n.parent);L&&L.locals&&c(L.locals,t,111551&r)&&(m=n)}break;case 255:case 224:case 256:if(d=c(Ca(n).members||D,t,788968&r)){if(!ri(d,n)){d=void 0;break}if(p&&e.isStatic(p))return void Dr(v,e.Diagnostics.Static_members_cannot_reference_class_type_parameters);break e}if(224===n.kind&&32&r){var A=n.name;if(A&&t===A.escapedText){d=n.symbol;break e}}break;case 226:if(p===n.expression&&94===n.parent.token){var C=n.parent.parent;if(e.isClassLike(C)&&(d=c(Ca(C).members,t,788968&r)))return void(i&&Dr(v,e.Diagnostics.Base_class_expressions_cannot_reference_class_type_parameters))}break;case 160:if(g=n.parent.parent,(e.isClassLike(g)||256===g.kind)&&(d=c(Ca(g).members,t,788968&r)))return void Dr(v,e.Diagnostics.A_computed_property_name_cannot_reference_a_type_parameter_from_its_containing_type);break;case 212:if(M.target>=2)break;case 167:case 169:case 170:case 171:case 254:if(3&r&&"arguments"===t){d=le;break e}break;case 211:if(3&r&&"arguments"===t){d=le;break e}if(16&r){var I=n.name;if(I&&t===I.escapedText){d=n.symbol;break e}}break;case 163:n.parent&&162===n.parent.kind&&(n=n.parent),n.parent&&(e.isClassElement(n.parent)||255===n.parent.kind)&&(n=n.parent);break;case 340:case 333:case 334:var N=e.getJSDocRoot(n);N&&(n=N.parent);break;case 162:p&&(p===n.initializer||p===n.name&&e.isBindingPattern(p))&&(_||(_=n));break;case 201:p&&(p===n.initializer||p===n.name&&e.isBindingPattern(p))&&e.isParameterDeclaration(n)&&!_&&(_=n);break;case 188:if(262144&r){var k=n.typeParameter.name;if(k&&t===k.escapedText){d=n.typeParameter.symbol;break e}}break}ni(n)&&(f=n),p=n,n=n.parent}if(!o||!d||f&&d===f.symbol||(d.isReferenced|=r),!d){if(p&&(e.Debug.assert(300===p.kind),p.commonJsModuleIndicator&&"exports"===t&&r&p.symbol.flags))return p.symbol;s||(d=c(ae,t,r))}if(!d&&y&&e.isInJSFile(y)&&y.parent&&e.isRequireCall(y.parent,!1))return ue;if(d){if(i){if(m&&(99!==M.target||!H)){var R=m.name;return void Dr(v,e.Diagnostics.Initializer_of_instance_member_variable_0_cannot_reference_identifier_1_declared_in_the_constructor,e.declarationNameToString(R),ti(a))}if(v&&(2&r||(32&r||384&r)&&111551===(111551&r))){var O=Ma(d);(2&O.flags||32&O.flags||384&O.flags)&&_i(O,v)}if(d&&b&&111551===(111551&r)&&!(4194304&y.flags)){var w=Aa(d);e.length(w.declarations)&&e.every(w.declarations,function(n){return e.isNamespaceExportDeclaration(n)||e.isSourceFile(n)&&!!n.symbol.globalExports})&&Cr(!M.allowUmdGlobalAccess,v,e.Diagnostics._0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead,e.unescapeLeadingUnderscores(t))}if(d&&_&&!h&&111551===(111551&r)){var P=Aa(qc(d));N=e.getRootDeclaration(_);P===Ca(_)?Dr(v,e.Diagnostics.Parameter_0_cannot_reference_itself,e.declarationNameToString(_.name)):P.valueDeclaration&&P.valueDeclaration.pos>_.pos&&N.parent.locals&&c(N.parent.locals,P.escapedName,r)===P&&Dr(v,e.Diagnostics.Parameter_0_cannot_reference_identifier_1_declared_after_it,e.declarationNameToString(_.name),e.declarationNameToString(v))}d&&v&&111551&r&&2097152&d.flags&&$r(d,t,v)}return d}if(i&&(!v||!ii(v,t,a)&&!ai(v)&&!si(v,t,r)&&!ui(v,t)&&!di(v,t,r)&&!mi(v,t,r)&&!ci(v,t,r))){var F=void 0;if(l&&zt<Jt){F=mx(y,t,r);var G=(null===F||void 0===F?void 0:F.valueDeclaration)&&e.isAmbientModule(F.valueDeclaration)&&e.isGlobalScopeAugmentation(F.valueDeclaration);if(G&&(F=void 0),F){var B=mo(F),U=$S(y,F,!1),V=U?e.Diagnostics.Could_not_find_name_0_Did_you_mean_1:e.Diagnostics.Cannot_find_name_0_Did_you_mean_1,j=Lr(v,V,ti(a),B);Ar(!U,j),F.valueDeclaration&&e.addRelatedInfo(j,e.createDiagnosticForNode(F.valueDeclaration,e.Diagnostics._0_is_declared_here,B))}}if(!F&&a){var K=cx(a);K?Dr(v,i,ti(a),K):Dr(v,i,ti(a))}zt++}}function $r(n,t,r){if(!e.isValidTypeOnlyAliasUseSite(r)){var i=Qi(n);if(i){var a=e.typeOnlyDeclarationIsExport(i),o=a?e.Diagnostics._0_cannot_be_used_as_a_value_because_it_was_exported_using_export_type:e.Diagnostics._0_cannot_be_used_as_a_value_because_it_was_imported_using_import_type,s=a?e.Diagnostics._0_was_exported_here:e.Diagnostics._0_was_imported_here,c=e.unescapeLeadingUnderscores(t);e.addRelatedInfo(Dr(r,o,c),e.createDiagnosticForNode(i,s,c))}}}function ei(n,t){return 212!==n.kind&&211!==n.kind?e.isTypeQueryNode(n)||(e.isFunctionLikeDeclaration(n)||165===n.kind&&!e.isStatic(n))&&(!t||t!==n.name):(!t||t!==n.name)&&(!(!n.asteriskToken&&!e.hasSyntacticModifier(n,256))||!e.getImmediatelyInvokedFunctionExpression(n))}function ni(e){switch(e.kind){case 254:case 255:case 256:case 258:case 257:case 259:return!0;default:return!1}}function ti(n){return e.isString(n)?e.unescapeLeadingUnderscores(n):e.declarationNameToString(n)}function ri(n,t){if(n.declarations)for(var r=0,i=n.declarations;r<i.length;r++){var a=i[r];if(161===a.kind){var o=e.isJSDocTemplateTag(a.parent)?e.getJSDocHost(a.parent):a.parent;if(o===t)return!(e.isJSDocTemplateTag(a.parent)&&e.find(a.parent.parent.tags,e.isJSDocTypeAlias))}}return!1}function ii(n,t,r){if(!e.isIdentifier(n)||n.escapedText!==t||yR(n)||Lv(n))return!1;var i=e.getThisContainer(n,!1),a=i;while(a){if(e.isClassLike(a.parent)){var o=Ca(a.parent);if(!o)break;var s=Ws(o);if(Lu(s,t))return Dr(n,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0,ti(r),mo(o)),!0;if(a===i&&!e.isStatic(a)){var c=Dc(o).thisType;if(Lu(c,t))return Dr(n,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0,ti(r)),!0}}a=a.parent}return!1}function ai(n){var t=oi(n);return!(!t||!ra(t,64,!0))&&(Dr(n,e.Diagnostics.Cannot_extend_an_interface_0_Did_you_mean_implements,e.getTextOfNode(t)),!0)}function oi(n){switch(n.kind){case 79:case 204:return n.parent?oi(n.parent):void 0;case 226:if(e.isEntityNameExpression(n.expression))return n.expression;default:return}}function si(n,t,r){var i=1920|(e.isInJSFile(n)?111551:0);if(r===i){var a=zi(Qr(n,t,788968&~i,void 0,void 0,!1)),o=n.parent;if(a){if(e.isQualifiedName(o)){e.Debug.assert(o.left===n,"Should only be resolving left side of qualified name as a namespace");var s=o.right.escapedText,c=Lu(Dc(a),s);if(c)return Dr(o,e.Diagnostics.Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_property_1_in_0_with_0_1,e.unescapeLeadingUnderscores(t),e.unescapeLeadingUnderscores(s)),!0}return Dr(n,e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_namespace_here,e.unescapeLeadingUnderscores(t)),!0}}return!1}function ci(n,t,r){if(788584&r){var i=zi(Qr(n,t,111127,void 0,void 0,!1));if(i&&!(1920&i.flags))return Dr(n,e.Diagnostics._0_refers_to_a_value_but_is_being_used_as_a_type_here_Did_you_mean_typeof_0,e.unescapeLeadingUnderscores(t)),!0}return!1}function li(e){return"any"===e||"string"===e||"number"===e||"boolean"===e||"never"===e||"unknown"===e}function ui(n,t){return!(!li(t)||273!==n.parent.kind)&&(Dr(n,e.Diagnostics.Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module,t),!0)}function di(n,t,r){if(111551&r){if(li(t))return Dr(n,e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here,e.unescapeLeadingUnderscores(t)),!0;var i=zi(Qr(n,t,788544,void 0,void 0,!1));if(i&&!(1024&i.flags)){var a=e.unescapeLeadingUnderscores(t);return fi(t)?Dr(n,e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later,a):pi(n,i)?Dr(n,e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Did_you_mean_to_use_1_in_0,a,"K"===a?"P":"K"):Dr(n,e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here,a),!0}}return!1}function pi(n,t){var r=e.findAncestor(n.parent,function(n){return!e.isComputedPropertyName(n)&&!e.isPropertySignature(n)&&(e.isTypeLiteralNode(n)||"quit")});if(r&&1===r.members.length){var i=Dc(t);return!!(1048576&i.flags)&&oA(i,384,!0)}return!1}function fi(e){switch(e){case"Promise":case"Symbol":case"Map":case"WeakMap":case"Set":case"WeakSet":return!0}return!1}function mi(n,t,r){if(111127&r){var i=zi(Qr(n,t,1024,void 0,void 0,!1));if(i)return Dr(n,e.Diagnostics.Cannot_use_namespace_0_as_a_value,e.unescapeLeadingUnderscores(t)),!0}else if(788544&r){i=zi(Qr(n,t,1536,void 0,void 0,!1));if(i)return Dr(n,e.Diagnostics.Cannot_use_namespace_0_as_a_type,e.unescapeLeadingUnderscores(t)),!0}return!1}function _i(n,t){var r;if(e.Debug.assert(!!(2&n.flags||32&n.flags||384&n.flags)),!(67108881&n.flags&&32&n.flags)){var i=null===(r=n.declarations)||void 0===r?void 0:r.find(function(n){return e.isBlockOrCatchScoped(n)||e.isClassLike(n)||258===n.kind});if(void 0===i)return e.Debug.fail("checkResolvedBlockScopedVariable could not find block-scoped declaration");if(!(8388608&i.flags)&&!Xr(i,t)){var a=void 0,o=e.declarationNameToString(e.getNameOfDeclaration(i));2&n.flags?a=Dr(t,e.Diagnostics.Block_scoped_variable_0_used_before_its_declaration,o):32&n.flags?a=Dr(t,e.Diagnostics.Class_0_used_before_its_declaration,o):256&n.flags?a=Dr(t,e.Diagnostics.Enum_0_used_before_its_declaration,o):(e.Debug.assert(!!(128&n.flags)),e.shouldPreserveConstEnums(M)&&(a=Dr(t,e.Diagnostics.Enum_0_used_before_its_declaration,o))),a&&e.addRelatedInfo(a,e.createDiagnosticForNode(i,e.Diagnostics._0_is_declared_here,o))}}}function gi(n,t,r){return!!t&&!!e.findAncestor(n,function(n){return n===r||e.isFunctionLike(n)?"quit":n===t})}function yi(e){switch(e.kind){case 263:return e;case 265:return e.parent;case 266:return e.parent.parent;case 268:return e.parent.parent.parent;default:return}}function hi(n){return n.declarations&&e.findLast(n.declarations,vi)}function vi(n){return 263===n.kind||262===n.kind||265===n.kind&&!!n.name||266===n.kind||272===n.kind||268===n.kind||273===n.kind||269===n.kind&&e.exportAssignmentIsAlias(n)||e.isBinaryExpression(n)&&2===e.getAssignmentDeclarationKind(n)&&e.exportAssignmentIsAlias(n)||e.isAccessExpression(n)&&e.isBinaryExpression(n.parent)&&n.parent.left===n&&63===n.parent.operatorToken.kind&&bi(n.parent.right)||292===n.kind||291===n.kind&&bi(n.initializer)||e.isRequireVariableDeclaration(n)}function bi(n){return e.isAliasableExpression(n)||e.isFunctionExpression(n)&&FL(n)}function Ei(n,t){var r=Fi(n);if(r){var i=e.getLeftmostAccessExpression(r.expression).arguments[0];return e.isIdentifier(r.name)?zi(Lu(ed(i),r.name.escapedText)):void 0}if(e.isVariableDeclaration(n)||275===n.moduleReference.kind){var a=ca(n,e.getExternalModuleRequireArgument(n)||e.getExternalModuleImportEqualsDeclarationExpression(n)),o=fa(a);return Xi(n,a,o,!1),o}var s=na(n.moduleReference,t);return Ti(n,s),s}function Ti(n,t){if(Xi(n,void 0,t,!1)&&!n.isTypeOnly){var r=Qi(Ca(n)),i=e.typeOnlyDeclarationIsExport(r),a=i?e.Diagnostics.An_import_alias_cannot_reference_a_declaration_that_was_exported_using_export_type:e.Diagnostics.An_import_alias_cannot_reference_a_declaration_that_was_imported_using_import_type,o=i?e.Diagnostics._0_was_exported_here:e.Diagnostics._0_was_imported_here,s=e.unescapeLeadingUnderscores(r.name.escapedText);e.addRelatedInfo(Dr(n.moduleReference,a),e.createDiagnosticForNode(r,o,s))}}function Si(e,n,t,r){var i=e.exports.get("export="),a=i?Lu(Ws(i),n):e.exports.get(n),o=zi(a,r);return Xi(t,a,o,!1),o}function xi(n){return e.isExportAssignment(n)&&!n.isExportEquals||e.hasSyntacticModifier(n,512)||e.isExportSpecifier(n)}function Li(n,t,r){if(!W)return!1;if(!n||n.isDeclarationFile){var i=Si(t,"default",void 0,!0);return(!i||!e.some(i.declarations,xi))&&!Si(t,e.escapeLeadingUnderscores("__esModule"),void 0,r)}return e.isSourceFileJS(n)?!n.externalModuleIndicator&&!Si(t,e.escapeLeadingUnderscores("__esModule"),void 0,r):ga(t)}function Di(n,t){var r,i=ca(n,n.parent.moduleSpecifier);if(i){var a=void 0;a=e.isShorthandAmbientModuleSymbol(i)?i:Si(i,"default",n,t);var o=null===(r=i.declarations)||void 0===r?void 0:r.find(e.isSourceFile),s=Li(o,i,t);if(a||s){if(s){var c=fa(i,t)||zi(i,t);return Xi(n,i,c,!1),c}}else if(ga(i)){var l=K>=e.ModuleKind.ES2015?"allowSyntheticDefaultImports":"esModuleInterop",u=i.exports.get("export="),d=u.valueDeclaration,p=Dr(n.name,e.Diagnostics.Module_0_can_only_be_default_imported_using_the_1_flag,mo(i),l);d&&e.addRelatedInfo(p,e.createDiagnosticForNode(d,e.Diagnostics.This_module_is_declared_with_using_export_and_can_only_be_used_with_a_default_import_when_using_the_0_flag,l))}else Ai(i,n);return Xi(n,a,void 0,!1),a}}function Ai(n,t){var r,i,a;if(null===(r=n.exports)||void 0===r?void 0:r.has(t.symbol.escapedName))Dr(t.name,e.Diagnostics.Module_0_has_no_default_export_Did_you_mean_to_use_import_1_from_0_instead,mo(n),mo(t.symbol));else{var o=Dr(t.name,e.Diagnostics.Module_0_has_no_default_export,mo(n)),s=null===(i=n.exports)||void 0===i?void 0:i.get("__export");if(s){var c=null===(a=s.declarations)||void 0===a?void 0:a.find(function(n){var t,r;return!!(e.isExportDeclaration(n)&&n.moduleSpecifier&&(null===(r=null===(t=ca(n,n.moduleSpecifier))||void 0===t?void 0:t.exports)||void 0===r?void 0:r.has("default")))});c&&e.addRelatedInfo(o,e.createDiagnosticForNode(c,e.Diagnostics.export_Asterisk_does_not_re_export_a_default))}}}function Ci(e,n){var t=e.parent.parent.moduleSpecifier,r=ca(e,t),i=_a(r,t,n,!1);return Xi(e,r,i,!1),i}function Ii(e,n){var t=e.parent.moduleSpecifier,r=t&&ca(e,t),i=t&&_a(r,t,n,!1);return Xi(e,r,i,!1),i}function Ni(n,t){if(n===Ae&&t===Ae)return Ae;if(790504&n.flags)return n;var r=Or(n.flags|t.flags,n.escapedName);return r.declarations=e.deduplicate(e.concatenate(n.declarations,t.declarations),e.equateValues),r.parent=n.parent||t.parent,n.valueDeclaration&&(r.valueDeclaration=n.valueDeclaration),t.members&&(r.members=new e.Map(t.members)),n.exports&&(r.exports=new e.Map(n.exports)),r}function ki(e,n,t,r){if(1536&e.flags){var i=Sa(e).get(n.escapedText),a=zi(i,r);return Xi(t,i,a,!1),a}}function Ri(e,n){if(3&e.flags){var t=e.valueDeclaration.type;if(t)return zi(Lu(u_(t),n))}}function Oi(n,t,r){var i,a;void 0===r&&(r=!1);var o=e.getExternalModuleRequireArgument(n)||n.moduleSpecifier,s=ca(n,o),c=!e.isPropertyAccessExpression(t)&&t.propertyName||t.name;if(e.isIdentifier(c)){var l="default"===c.escapedText&&!(!M.allowSyntheticDefaultImports&&!M.esModuleInterop),u=_a(s,o,!1,l);if(u&&c.escapedText){if(e.isShorthandAmbientModuleSymbol(s))return s;var d=void 0;d=s&&s.exports&&s.exports.get("export=")?Lu(Ws(u),c.escapedText,!0):Ri(u,c.escapedText),d=zi(d,r);var p=ki(u,c,t,r);if(void 0===p&&"default"===c.escapedText){var f=null===(i=s.declarations)||void 0===i?void 0:i.find(e.isSourceFile);Li(f,s,r)&&(p=fa(s,r)||zi(s,r))}var m=p&&d&&p!==d?Ni(d,p):p||d;if(!m){var _=ta(s,n),g=e.declarationNameToString(c),y=gx(c,u);if(void 0!==y){var h=mo(y),v=Dr(c,e.Diagnostics._0_has_no_exported_member_named_1_Did_you_mean_2,_,g,h);y.valueDeclaration&&e.addRelatedInfo(v,e.createDiagnosticForNode(y.valueDeclaration,e.Diagnostics._0_is_declared_here,h))}else(null===(a=s.exports)||void 0===a?void 0:a.has("default"))?Dr(c,e.Diagnostics.Module_0_has_no_exported_member_1_Did_you_mean_to_use_import_1_from_0_instead,_,g):wi(n,c,g,s,_)}return m}}}function wi(n,t,r,i,o){var s,c,l=null===(c=null===(s=i.valueDeclaration)||void 0===s?void 0:s.locals)||void 0===c?void 0:c.get(t.escapedText),u=i.exports;if(l){var d=null===u||void 0===u?void 0:u.get("export=");if(d)Pa(d,l)?Pi(n,t,r,o):Dr(t,e.Diagnostics.Module_0_has_no_exported_member_1,o,r);else{var p=u?e.find(Bu(u),function(e){return!!Pa(e,l)}):void 0,f=p?Dr(t,e.Diagnostics.Module_0_declares_1_locally_but_it_is_exported_as_2,o,r,mo(p)):Dr(t,e.Diagnostics.Module_0_declares_1_locally_but_it_is_not_exported,o,r);l.declarations&&e.addRelatedInfo.apply(void 0,a([f],e.map(l.declarations,function(n,t){return e.createDiagnosticForNode(n,0===t?e.Diagnostics._0_is_declared_here:e.Diagnostics.and_here,r)}),!1))}}else Dr(t,e.Diagnostics.Module_0_has_no_exported_member_1,o,r)}function Pi(n,t,r,i){if(K>=e.ModuleKind.ES2015){var a=M.esModuleInterop?e.Diagnostics._0_can_only_be_imported_by_using_a_default_import:e.Diagnostics._0_can_only_be_imported_by_turning_on_the_esModuleInterop_flag_and_using_a_default_import;Dr(t,a,r)}else if(e.isInJSFile(n)){a=M.esModuleInterop?e.Diagnostics._0_can_only_be_imported_by_using_a_require_call_or_by_using_a_default_import:e.Diagnostics._0_can_only_be_imported_by_using_a_require_call_or_by_turning_on_the_esModuleInterop_flag_and_using_a_default_import;Dr(t,a,r)}else{a=M.esModuleInterop?e.Diagnostics._0_can_only_be_imported_by_using_import_1_require_2_or_a_default_import:e.Diagnostics._0_can_only_be_imported_by_using_import_1_require_2_or_by_turning_on_the_esModuleInterop_flag_and_using_a_default_import;Dr(t,a,r,r,i)}}function Mi(n,t){var r=e.isBindingElement(n)?e.getRootDeclaration(n):n.parent.parent.parent,i=Fi(r),a=Oi(r,i||n,t),o=n.propertyName||n.name;return i&&a&&e.isIdentifier(o)?zi(Lu(Ws(a),o.escapedText),t):(Xi(n,void 0,a,!1),a)}function Fi(n){if(e.isVariableDeclaration(n)&&n.initializer&&e.isPropertyAccessExpression(n.initializer))return n.initializer}function Gi(e,n){var t=fa(e.parent.symbol,n);return Xi(e,void 0,t,!1),t}function Bi(e,n,t){var r=e.parent.parent.moduleSpecifier?Oi(e.parent.parent,e,t):ra(e.propertyName||e.name,n,!1,t);return Xi(e,void 0,r,!1),r}function Ui(n,t){var r=e.isExportAssignment(n)?n.expression:n.right,i=Vi(r,t);return Xi(n,void 0,i,!1),i}function Vi(n,t){if(e.isClassExpression(n))return kA(n).symbol;if(e.isEntityName(n)||e.isEntityNameExpression(n)){var r=ra(n,901119,!0,t);return r||(kA(n),Wr(n).resolvedSymbol)}}function ji(e,n){var t=e.initializer;return Vi(t,n)}function Ki(n,t){if(e.isBinaryExpression(n.parent)&&n.parent.left===n&&63===n.parent.operatorToken.kind)return Vi(n.parent.right,t)}function Hi(n,t){switch(void 0===t&&(t=!1),n.kind){case 263:case 252:return Ei(n,t);case 265:return Di(n,t);case 266:return Ci(n,t);case 272:return Ii(n,t);case 268:case 201:return Mi(n,t);case 273:return Bi(n,901119,t);case 269:case 219:return Ui(n,t);case 262:return Gi(n,t);case 292:return ra(n.name,901119,!0,t);case 291:return ji(n,t);case 205:case 204:return Ki(n,t);default:return e.Debug.fail()}}function Wi(e,n){return void 0===n&&(n=901119),!!e&&(2097152===(e.flags&(2097152|n))||!!(2097152&e.flags&&67108864&e.flags))}function zi(e,n){return!n&&Wi(e)?Ji(e):e}function Ji(n){e.Debug.assert(0!==(2097152&n.flags),"Should only get Alias here.");var t=Hr(n);if(t.target)t.target===Ce&&(t.target=Ae);else{t.target=Ce;var r=hi(n);if(!r)return e.Debug.fail();var i=Hi(r);t.target===Ce?t.target=i||Ae:Dr(r,e.Diagnostics.Circular_definition_of_import_alias_0,mo(n))}return t.target}function qi(e){var n=Hr(e);if(n.target!==Ce)return Ji(e)}function Xi(n,t,r,i){if(!n||e.isPropertyAccessExpression(n))return!1;var a=Ca(n);if(e.isTypeOnlyImportOrExportDeclaration(n)){var o=Hr(a);return o.typeOnlyDeclaration=n,!0}var s=Hr(a);return Yi(s,t,i)||Yi(s,r,i)}function Yi(n,t,r){var i,a,o;if(t&&(void 0===n.typeOnlyDeclaration||r&&!1===n.typeOnlyDeclaration)){var s=null!==(a=null===(i=t.exports)||void 0===i?void 0:i.get("export="))&&void 0!==a?a:t,c=s.declarations&&e.find(s.declarations,e.isTypeOnlyImportOrExportDeclaration);n.typeOnlyDeclaration=null!==(o=null!==c&&void 0!==c?c:Hr(s).typeOnlyDeclaration)&&void 0!==o&&o}return!!n.typeOnlyDeclaration}function Qi(e){if(2097152&e.flags){var n=Hr(e);return n.typeOnlyDeclaration||void 0}}function Zi(e){var n=Ca(e),t=Ji(n);if(t){var r=t===Ae||111551&t.flags&&!nO(t)&&!Qi(n);r&&$i(n)}}function $i(n){var t=Hr(n);if(!t.referenced){t.referenced=!0;var r=hi(n);if(!r)return e.Debug.fail();if(e.isInternalModuleImportEqualsDeclaration(r)){var i=zi(n);(i===Ae||111551&i.flags)&&kA(r.moduleReference)}}}function ea(e){var n=Hr(e);n.constEnumReferenced||(n.constEnumReferenced=!0)}function na(n,t){return 79===n.kind&&e.isRightSideOfQualifiedNameOrPropertyAccess(n)&&(n=n.parent),79===n.kind||159===n.parent.kind?ra(n,1920,!1,t):(e.Debug.assert(263===n.parent.kind),ra(n,901119,!1,t))}function ta(e,n){return e.parent?ta(e.parent,n)+"."+mo(e):mo(e,n,void 0,20)}function ra(n,t,r,i,a){if(!e.nodeIsMissing(n)){var o,s=1920|(e.isInJSFile(n)?111551&t:0);if(79===n.kind){var c=t===s||e.nodeIsSynthesized(n)?e.Diagnostics.Cannot_find_namespace_0:Sv(e.getFirstIdentifier(n)),l=e.isInJSFile(n)&&!e.nodeIsSynthesized(n)?ia(n,t):void 0;if(o=Aa(Qr(a||n,n.escapedText,t,r||l?void 0:c,n,!0)),!o)return Aa(l)}else{if(159!==n.kind&&204!==n.kind)throw e.Debug.assertNever(n,"Unknown entity name kind.");var u=159===n.kind?n.left:n.expression,d=159===n.kind?n.right:n.name,p=ra(u,s,r,!1,a);if(!p||e.nodeIsMissing(d))return;if(p===Ae)return p;if(p.valueDeclaration&&e.isInJSFile(p.valueDeclaration)&&e.isVariableDeclaration(p.valueDeclaration)&&p.valueDeclaration.initializer&&qL(p.valueDeclaration.initializer)){var f=p.valueDeclaration.initializer.arguments[0],m=ca(f,f);if(m){var _=fa(m);_&&(p=_)}}if(o=Aa(Jr(Sa(p),d.escapedText,t)),!o){if(!r){var g=ta(p),y=e.declarationNameToString(d),h=gx(d,p);h?Dr(d,e.Diagnostics._0_has_no_exported_member_named_1_Did_you_mean_2,g,y,mo(h)):Dr(d,e.Diagnostics.Namespace_0_has_no_exported_member_1,g,y)}return}}return e.Debug.assert(0===(1&e.getCheckFlags(o)),"Should never get an instantiated symbol here."),!e.nodeIsSynthesized(n)&&e.isEntityName(n)&&(2097152&o.flags||269===n.parent.kind)&&Xi(e.getAliasDeclarationFromName(n),o,void 0,!0),o.flags&t||i?o:Ji(o)}}function ia(e,n){if(Yd(e.parent)){var t=aa(e.parent);if(t)return Qr(t,e.escapedText,n,void 0,e,!0)}}function aa(n){var t=e.findAncestor(n,function(n){return e.isJSDocNode(n)||4194304&n.flags?e.isJSDocTypeAlias(n):"quit"});if(!t){var r=e.getJSDocHost(n);if(r&&e.isExpressionStatement(r)&&e.isBinaryExpression(r.expression)&&3===e.getAssignmentDeclarationKind(r.expression)){var i=Ca(r.expression.left);if(i)return oa(i)}if(r&&(e.isObjectLiteralMethod(r)||e.isPropertyAssignment(r))&&e.isBinaryExpression(r.parent.parent)&&6===e.getAssignmentDeclarationKind(r.parent.parent)){i=Ca(r.parent.parent.left);if(i)return oa(i)}var a=e.getEffectiveJSDocHost(n);if(a&&e.isFunctionLike(a)){i=Ca(a);return i&&i.valueDeclaration}}}function oa(n){var t=n.parent.valueDeclaration;if(t){var r=e.isAssignmentDeclaration(t)?e.getAssignedExpandoInitializer(t):e.hasOnlyExpressionInitializer(t)?e.getDeclaredExpandoInitializer(t):void 0;return r||t}}function sa(n){var t=n.valueDeclaration;if(t&&e.isInJSFile(t)&&!(524288&n.flags)&&!e.getExpandoInitializer(t,!1)){var r=e.isVariableDeclaration(t)?e.getDeclaredExpandoInitializer(t):e.getAssignedExpandoInitializer(t);if(r){var i=Ca(r);if(i)return GL(i,n)}}}function ca(n,t,r){var i=e.getEmitModuleResolutionKind(M)===e.ModuleResolutionKind.Classic,a=i?e.Diagnostics.Cannot_find_module_0_Did_you_mean_to_set_the_moduleResolution_option_to_node_or_to_add_aliases_to_the_paths_option:e.Diagnostics.Cannot_find_module_0_or_its_corresponding_type_declarations;return la(n,t,r?void 0:a)}function la(n,t,r,i){return void 0===i&&(i=!1),e.isStringLiteralLike(t)?ua(n,t.text,r,t,i):void 0}function ua(t,r,i,a,o){if(void 0===o&&(o=!1),e.startsWith(r,"@types/")){var s=e.Diagnostics.Cannot_import_type_declaration_files_Consider_importing_0_instead_of_1,c=e.removePrefix(r,"@types/");Dr(a,s,c,r)}var l=Vu(r,!0);if(l)return l;var u=e.getSourceFileOfNode(t),d=e.getResolvedModule(u,r),p=d&&e.getResolutionDiagnostic(M,d),f=d&&!p&&n.getSourceFile(d.resolvedFileName);if(f)return f.symbol?(d.isExternalLibraryImport&&!e.resolutionExtensionIsTSOrJson(d.extension)&&da(!1,a,d,r),Aa(f.symbol)):void(i&&Dr(a,e.Diagnostics.File_0_is_not_a_module,f.fileName));if(Ln){var m=e.findBestPatternMatch(Ln,function(e){return e.pattern},r);if(m){var _=Dn&&Dn.get(r);return Aa(_?_:m.symbol)}}if(d&&!e.resolutionExtensionIsTSOrJson(d.extension)&&void 0===p||p===e.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type)if(o){s=e.Diagnostics.Invalid_module_name_in_augmentation_Module_0_resolves_to_an_untyped_module_at_1_which_cannot_be_augmented;Dr(a,s,r,d.resolvedFileName)}else da(Y&&!!i,a,d,r);else if(i){if(d){var g=n.getProjectReferenceRedirect(d.resolvedFileName);if(g)return void Dr(a,e.Diagnostics.Output_file_0_has_not_been_built_from_source_file_1,g,d.resolvedFileName)}if(p)Dr(a,p,r,d.resolvedFileName);else{var y=e.tryExtractTSExtension(r);if(y){s=e.Diagnostics.An_import_path_cannot_end_with_a_0_extension_Consider_importing_1_instead;var h=e.removeExtension(r,y),v=h,b=e.getEmitModuleKind(M);b>=e.ModuleKind.ES2015&&(v+=".js"),Dr(a,s,y,v)}else!M.resolveJsonModule&&e.fileExtensionIs(r,".json")&&e.getEmitModuleResolutionKind(M)===e.ModuleResolutionKind.NodeJs&&e.hasJsonModuleEmitEnabled(M)?Dr(a,e.Diagnostics.Cannot_find_module_0_Consider_using_resolveJsonModule_to_import_module_with_json_extension,r):Dr(a,i,r)}}}function da(n,t,r,i){var a=r.packageId,o=r.resolvedFileName,s=!e.isExternalModuleNameRelative(i)&&a?pa(a.name)?e.chainDiagnosticMessages(void 0,e.Diagnostics.If_the_0_package_actually_exposes_this_module_consider_sending_a_pull_request_to_amend_https_Colon_Slash_Slashgithub_com_SlashDefinitelyTyped_SlashDefinitelyTyped_Slashtree_Slashmaster_Slashtypes_Slash_1,a.name,e.mangleScopedPackageName(a.name)):e.chainDiagnosticMessages(void 0,e.Diagnostics.Try_npm_i_save_dev_types_Slash_1_if_it_exists_or_add_a_new_declaration_d_ts_file_containing_declare_module_0,i,e.mangleScopedPackageName(a.name)):void 0;Cr(n,t,e.chainDiagnosticMessages(s,e.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type,i,o))}function pa(n){return m().has(e.getTypesPackageName(n))}function fa(e,n){if(null===e||void 0===e?void 0:e.exports){var t=zi(e.exports.get("export="),n),r=ma(Aa(t),Aa(e));return Aa(r)||e}}function ma(n,t){if(!n||n===Ae||n===t||1===t.exports.size||2097152&n.flags)return n;var r=Hr(n);if(r.cjsExportMerged)return r.cjsExportMerged;var i=33554432&n.flags?n:Mr(n);return i.flags=512|i.flags,void 0===i.exports&&(i.exports=e.createSymbolTable()),t.exports.forEach(function(e,n){"export="!==n&&i.exports.set(n,i.exports.has(n)?Fr(i.exports.get(n),e):e)}),Hr(i).cjsExportMerged=i,r.cjsExportMerged=i}function _a(n,t,r,i){var a=fa(n,r);if(!r&&a){if(!i&&!(1539&a.flags)&&!e.getDeclarationOfKind(a,300)){var o=K>=e.ModuleKind.ES2015?"allowSyntheticDefaultImports":"esModuleInterop";return Dr(t,e.Diagnostics.This_module_can_only_be_referenced_with_ECMAScript_imports_Slashexports_by_turning_on_the_0_flag_and_referencing_its_default_export,o),a}if(M.esModuleInterop){var s=t.parent;if(e.isImportDeclaration(s)&&e.getNamespaceDeclarationNode(s)||e.isImportCall(s)){var c=Ws(a),l=Du(c,0);if(l&&l.length||(l=Du(c,1)),l&&l.length){var u=JL(c,a,n),d=Or(a.flags,a.escapedName);d.declarations=a.declarations?a.declarations.slice():[],d.parent=a.parent,d.target=a,d.originatingImport=s,a.valueDeclaration&&(d.valueDeclaration=a.valueDeclaration),a.constEnumOnlyModule&&(d.constEnumOnlyModule=!0),a.members&&(d.members=new e.Map(a.members)),a.exports&&(d.exports=new e.Map(a.exports));var p=Ul(u);return d.type=Xa(d,p.members,e.emptyArray,e.emptyArray,p.indexInfos),d}}}}return a}function ga(e){return void 0!==e.exports.get("export=")}function ya(e){return Bu(xa(e))}function ha(n){var t=ya(n),r=fa(n);if(r!==n){var i=Ws(r);Ta(i)&&e.addRange(t,Hl(i))}return t}function va(e,n){var t=xa(e);t.forEach(function(e,t){Wa(t)||n(e,t)});var r=fa(e);if(r!==e){var i=Ws(r);Ta(i)&&Hl(i).forEach(function(e){n(e,e.escapedName)})}}function ba(e,n){var t=xa(n);if(t)return t.get(e)}function Ea(e,n){var t=ba(e,n);if(t)return t;var r=fa(n);if(r!==n){var i=Ws(r);return Ta(i)?Lu(i,e):void 0}}function Ta(n){return!(131068&n.flags||1&e.getObjectFlags(n)||Ly(n)||Jy(n))}function Sa(e){return 6256&e.flags?zc(e,"resolvedExports"):1536&e.flags?xa(e):e.exports||D}function xa(e){var n=Hr(e);return n.resolvedExports||(n.resolvedExports=Da(e))}function La(n,t,r,i){t&&t.forEach(function(t,a){if("default"!==a){var o=n.get(a);if(o){if(r&&i&&o&&zi(o)!==zi(t)){var s=r.get(a);s.exportsWithDuplicate?s.exportsWithDuplicate.push(i):s.exportsWithDuplicate=[i]}}else n.set(a,t),r&&i&&r.set(a,{specifierText:e.getTextOfNode(i.moduleSpecifier)})}})}function Da(n){var t=[];return n=fa(n),r(n)||D;function r(n){if(n&&n.exports&&e.pushIfUnique(t,n)){var i=new e.Map(n.exports),a=n.exports.get("__export");if(a){var o=e.createSymbolTable(),s=new e.Map;if(a.declarations)for(var c=0,l=a.declarations;c<l.length;c++){var u=l[c],d=ca(u,u.moduleSpecifier),p=r(d);La(o,p,s,u)}s.forEach(function(n,t){var r=n.exportsWithDuplicate;if("export="!==t&&r&&r.length&&!i.has(t))for(var a=0,o=r;a<o.length;a++){var c=o[a];ur.add(e.createDiagnosticForNode(c,e.Diagnostics.Module_0_has_already_exported_a_member_named_1_Consider_explicitly_re_exporting_to_resolve_the_ambiguity,s.get(t).specifierText,e.unescapeLeadingUnderscores(t)))}}),La(i,o)}return i}}}function Aa(e){var n;return e&&e.mergeId&&(n=qt[e.mergeId])?n:e}function Ca(e){return Aa(e.symbol&&qc(e.symbol))}function Ia(e){return Aa(e.parent&&qc(e.parent))}function Na(t,r){var i,a=e.getSourceFileOfNode(r),o=O(a),s=Hr(t);if(s.extendedContainersByFile&&(i=s.extendedContainersByFile.get(o)))return i;if(a&&a.imports){for(var c=0,l=a.imports;c<l.length;c++){var u=l[c];if(!e.nodeIsSynthesized(u)){var d=ca(r,u,!0);if(d){var p=wa(d,t);p&&(i=e.append(i,d))}}}if(e.length(i))return(s.extendedContainersByFile||(s.extendedContainersByFile=new e.Map)).set(o,i),i}if(s.extendedContainers)return s.extendedContainers;for(var f=n.getSourceFiles(),m=0,_=f;m<_.length;m++){var g=_[m];if(e.isExternalModule(g)){var y=Ca(g);p=wa(y,t);p&&(i=e.append(i,y))}}return s.extendedContainers=i||e.emptyArray}function ka(n,t,r){var i=Ia(n);if(i&&!(262144&n.flags)){var o=e.mapDefined(i.declarations,p),s=t&&Na(n,t),c=Ra(i,r);if(t&&i.flags&Za(r)&&$a(i,t,1920,!1))return e.append(e.concatenate(e.concatenate([i],o),s),c);var l=!(i.flags&Za(r))&&788968&i.flags&&524288&Dc(i).flags&&111551===r?Qa(t,function(n){return e.forEachEntry(n,function(e){if(e.flags&Za(r)&&Ws(e)===Dc(i))return e})}):void 0,u=a(a(l?[l]:[],o,!0),[i],!1);return u=e.append(u,c),u=e.addRange(u,s),u}var d=e.mapDefined(n.declarations,function(n){return!e.isAmbientModule(n)&&n.parent&&uo(n.parent)?Ca(n.parent):e.isClassExpression(n)&&e.isBinaryExpression(n.parent)&&63===n.parent.operatorToken.kind&&e.isAccessExpression(n.parent.left)&&e.isEntityNameExpression(n.parent.left.expression)?e.isModuleExportsAccessExpression(n.parent.left)||e.isExportsIdentifier(n.parent.left.expression)?Ca(e.getSourceFileOfNode(n)):(kA(n.parent.left.expression),Wr(n.parent.left.expression).resolvedSymbol):void 0});if(e.length(d))return e.mapDefined(d,function(e){return wa(e,n)?e:void 0});function p(e){return i&&Oa(e,i)}}function Ra(n,t){var r=!!e.length(n.declarations)&&e.first(n.declarations);if(111551&t&&r&&r.parent&&e.isVariableDeclaration(r.parent)&&(e.isObjectLiteralExpression(r)&&r===r.parent.initializer||e.isTypeLiteralNode(r)&&r===r.parent.type))return Ca(r.parent)}function Oa(e,n){var t=co(e),r=t&&t.exports&&t.exports.get("export=");return r&&Pa(r,n)?t:void 0}function wa(n,t){if(n===Ia(t))return t;var r=n.exports&&n.exports.get("export=");if(r&&Pa(r,t))return n;var i=Sa(n),a=i.get(t.escapedName);return a&&Pa(a,t)?a:e.forEachEntry(i,function(e){if(Pa(e,t))return e})}function Pa(e,n){if(Aa(zi(Aa(e)))===Aa(zi(Aa(n))))return e}function Ma(e){return Aa(e&&0!==(1048576&e.flags)?e.exportSymbol:e)}function Fa(e){return!!(111551&e.flags||2097152&e.flags&&111551&Ji(e).flags&&!Qi(e))}function Ga(n){for(var t=n.members,r=0,i=t;r<i.length;r++){var a=i[r];if(169===a.kind&&e.nodeIsPresent(a.body))return a}}function Ba(n){var r=new g(de,n);return h++,r.id=h,t&&(null===e.tracing||void 0===e.tracing||e.tracing.recordType(r)),r}function Ua(e){return new g(de,e)}function Va(e,n,t){void 0===t&&(t=0);var r=Ba(e);return r.intrinsicName=n,r.objectFlags=t,r}function ja(e,n){var t=Ba(524288);return t.objectFlags=e,t.symbol=n,t.members=void 0,t.properties=void 0,t.callSignatures=void 0,t.constructSignatures=void 0,t.indexInfos=void 0,t}function Ka(){return ff(e.arrayFrom(T.keys(),Zm))}function Ha(e){var n=Ba(262144);return e&&(n.symbol=e),n}function Wa(e){return 95===e.charCodeAt(0)&&95===e.charCodeAt(1)&&95!==e.charCodeAt(2)&&64!==e.charCodeAt(2)&&35!==e.charCodeAt(2)}function za(n){var t;return n.forEach(function(e,n){!Wa(n)&&Fa(e)&&(t||(t=[])).push(e)}),t||e.emptyArray}function Ja(n){var t=za(n),r=Ed(n);return r?e.concatenate(t,[r]):t}function qa(n,t,r,i,a){var o=n;return o.members=t,o.properties=e.emptyArray,o.callSignatures=r,o.constructSignatures=i,o.indexInfos=a,t!==D&&(o.properties=za(t)),o}function Xa(e,n,t,r,i){return qa(ja(16,e),n,t,r,i)}function Ya(n){if(0===n.constructSignatures.length)return n;if(n.objectTypeWithoutAbstractConstructSignatures)return n.objectTypeWithoutAbstractConstructSignatures;var t=e.filter(n.constructSignatures,function(e){return!(4&e.flags)});if(n.constructSignatures===t)return n;var r=Xa(n.symbol,n.members,n.callSignatures,e.some(t)?t:e.emptyArray,n.indexInfos);return n.objectTypeWithoutAbstractConstructSignatures=r,r.objectTypeWithoutAbstractConstructSignatures=r,r}function Qa(n,t){for(var r,i=function(n){if(n.locals&&!zr(n)&&(r=t(n.locals,void 0,!0,n)))return{value:r};switch(n.kind){case 300:if(!e.isExternalOrCommonJsModule(n))break;case 259:var i=Ca(n);if(r=t((null===i||void 0===i?void 0:i.exports)||D,void 0,!0,n))return{value:r};break;case 255:case 224:case 256:var a;if((Ca(n).members||D).forEach(function(n,t){788968&n.flags&&(a||(a=e.createSymbolTable())).set(t,n)}),a&&(r=t(a,void 0,!1,n)))return{value:r};break}},a=n;a;a=a.parent){var o=i(a);if("object"===typeof o)return o.value}return t(ae,void 0,!0)}function Za(e){return 111551===e?111551:1920}function $a(n,t,r,i,a){if(void 0===a&&(a=new e.Map),n&&!no(n)){var o=Hr(n),s=o.accessibleChainCache||(o.accessibleChainCache=new e.Map),c=Qa(t,function(e,n,t,r){return r}),l=(i?0:1)+"|"+(c&&O(c))+"|"+r;if(s.has(l))return s.get(l);var u=w(n),d=a.get(u);d||a.set(u,d=[]);var p=Qa(t,f);return s.set(l,p),p}function f(n,t,r){if(e.pushIfUnique(d,n)){var i=g(n,t,r);return d.pop(),i}}function m(e,n){return!eo(e,t,n)||!!$a(e.parent,t,Za(n),i,a)}function _(t,i,a){return(n===(i||t)||Aa(n)===Aa(i||t))&&!e.some(t.declarations,uo)&&(a||m(Aa(t),r))}function g(r,a,o){if(_(r.get(n.escapedName),void 0,a))return[n];var s=e.forEachEntry(r,function(r){if(2097152&r.flags&&"export="!==r.escapedName&&"default"!==r.escapedName&&!(e.isUMDExportSymbol(r)&&t&&e.isExternalModule(e.getSourceFileOfNode(t)))&&(!i||e.some(r.declarations,e.isExternalModuleImportEqualsDeclaration))&&(!o||!e.some(r.declarations,e.isNamespaceReexportDeclaration))&&(a||!e.getDeclarationOfKind(r,273))){var s=Ji(r),c=y(r,s,a);if(c)return c}if(r.escapedName===n.escapedName&&r.exportSymbol&&_(Aa(r.exportSymbol),void 0,a))return[n]});return s||(r===ae?y(se,se,a):void 0)}function y(e,n,t){if(_(e,n,t))return[e];var i=Sa(n),a=i&&f(i,!0);return a&&m(e,Za(r))?[e].concat(a):void 0}}function eo(n,t,r){var i=!1;return Qa(t,function(t){var a=Aa(t.get(n.escapedName));return!!a&&(a===n||(a=2097152&a.flags&&!e.getDeclarationOfKind(a,273)?Ji(a):a,!!(a.flags&r)&&(i=!0,!0)))}),i}function no(e){if(e.declarations&&e.declarations.length){for(var n=0,t=e.declarations;n<t.length;n++){var r=t[n];switch(r.kind){case 165:case 167:case 170:case 171:continue;default:return!1}}return!0}return!1}function to(e,n){var t=so(e,n,788968,!1,!0);return 0===t.accessibility}function ro(e,n){var t=so(e,n,111551,!1,!0);return 0===t.accessibility}function io(e,n,t){var r=so(e,n,t,!1,!1);return 0===r.accessibility}function ao(n,t,r,i,a,o){if(e.length(n)){for(var s,c=!1,l=0,u=n;l<u.length;l++){var d=u[l],p=$a(d,t,i,!1);if(p){s=d;var f=po(p[0],a);if(f)return f}if(o&&e.some(d.declarations,uo)){if(a){c=!0;continue}return{accessibility:0}}var m=ka(d,t,i),_=ao(m,t,r,r===d?Za(i):i,a,o);if(_)return _}return c?{accessibility:0}:s?{accessibility:1,errorSymbolName:mo(r,t,i),errorModuleName:s!==r?mo(s,t,1920):void 0}:void 0}}function oo(e,n,t,r){return so(e,n,t,r,!0)}function so(n,t,r,i,a){if(n&&t){var o=ao([n],t,n,r,i,a);if(o)return o;var s=e.forEach(n.declarations,co);if(s){var c=co(t);if(s!==c)return{accessibility:2,errorSymbolName:mo(n,t,r),errorModuleName:mo(s),errorNode:e.isInJSFile(t)?t:void 0}}return{accessibility:1,errorSymbolName:mo(n,t,r)}}return{accessibility:0}}function co(n){var t=e.findAncestor(n,lo);return t&&Ca(t)}function lo(n){return e.isAmbientModule(n)||300===n.kind&&e.isExternalOrCommonJsModule(n)}function uo(n){return e.isModuleWithStringLiteralName(n)||300===n.kind&&e.isExternalOrCommonJsModule(n)}function po(n,t){var r;if(e.every(e.filter(n.declarations,function(e){return 79!==e.kind}),i))return{accessibility:0,aliasesToMakeVisible:r};function i(t){var r,i;if(!ko(t)){var o=yi(t);return o&&!e.hasSyntacticModifier(o,1)&&ko(o.parent)?a(t,o):e.isVariableDeclaration(t)&&e.isVariableStatement(t.parent.parent)&&!e.hasSyntacticModifier(t.parent.parent,1)&&ko(t.parent.parent.parent)?a(t,t.parent.parent):e.isLateVisibilityPaintedStatement(t)&&!e.hasSyntacticModifier(t,1)&&ko(t.parent)?a(t,t):!!(2097152&n.flags&&e.isBindingElement(t)&&e.isInJSFile(t)&&(null===(r=t.parent)||void 0===r?void 0:r.parent)&&e.isVariableDeclaration(t.parent.parent)&&(null===(i=t.parent.parent.parent)||void 0===i?void 0:i.parent)&&e.isVariableStatement(t.parent.parent.parent.parent)&&!e.hasSyntacticModifier(t.parent.parent.parent.parent,1)&&t.parent.parent.parent.parent.parent&&ko(t.parent.parent.parent.parent.parent))&&a(t,t.parent.parent.parent.parent)}return!0}function a(n,i){return t&&(Wr(n).isVisible=!0,r=e.appendIfUnique(r,i)),!0}}function fo(n,t){var r;r=179===n.parent.kind||e.isExpressionWithTypeArgumentsInClassExtendsClause(n.parent)||160===n.parent.kind?1160127:159===n.kind||204===n.kind||263===n.parent.kind?1920:788968;var i=e.getFirstIdentifier(n),a=Qr(t,i.escapedText,r,void 0,void 0,!1);return a&&262144&a.flags&&788968&r?{accessibility:0}:a&&po(a,!0)||{accessibility:1,errorSymbolName:e.getTextOfNode(i),errorNode:i}}function mo(n,t,r,i,a){void 0===i&&(i=4);var o=70221824;2&i&&(o|=128),1&i&&(o|=512),8&i&&(o|=16384),16&i&&(o|=134217728);var s=4&i?ie.symbolToExpression:ie.symbolToEntityName;return a?c(a).getText():e.usingSingleLineStringWriter(c);function c(i){var a=s(n,r,t,o),c=300===(null===t||void 0===t?void 0:t.kind)?e.createPrinter({removeComments:!0,neverAsciiEscape:!0}):e.createPrinter({removeComments:!0}),l=t&&e.getSourceFileOfNode(t);return c.writeNode(4,a,l,i),i}}function _o(n,t,r,i,a){return void 0===r&&(r=0),a?o(a).getText():e.usingSingleLineStringWriter(o);function o(a){var o;o=262144&r?1===i?178:177:1===i?173:172;var s=ie.signatureToSignatureDeclaration(n,o,t,70222336|bo(r)),c=e.createPrinter({removeComments:!0,omitTrailingSemicolon:!0}),l=t&&e.getSourceFileOfNode(t);return c.writeNode(4,s,l,e.getTrailingSemicolonDeferringWriter(a)),a}}function go(n,t,r,i){void 0===r&&(r=1064960),void 0===i&&(i=e.createTextWriter(""));var a=M.noErrorTruncation||1&r,o=ie.typeToTypeNode(n,t,70221824|bo(r)|(a?1:0),i);if(void 0===o)return e.Debug.fail("should always get typenode");var s={removeComments:!0},c=e.createPrinter(s),l=t&&e.getSourceFileOfNode(t);c.writeNode(4,o,l,i);var u=i.getText(),d=a?2*e.noTruncationMaximumTruncationLength:2*e.defaultMaximumTruncationLength;return d&&u&&u.length>=d?u.substr(0,d-"...".length)+"...":u}function yo(e,n){var t=vo(e.symbol)?go(e,e.symbol.valueDeclaration):go(e),r=vo(n.symbol)?go(n,n.symbol.valueDeclaration):go(n);return t===r&&(t=ho(e),r=ho(n)),[t,r]}function ho(e){return go(e,void 0,64)}function vo(n){return n&&!!n.valueDeclaration&&e.isExpression(n.valueDeclaration)&&!$_(n.valueDeclaration)}function bo(e){return void 0===e&&(e=0),814775659&e}function Eo(n){return!!n.symbol&&!!(32&n.symbol.flags)&&(n===gc(n.symbol)||!!(524288&n.flags)&&!!(16777216&e.getObjectFlags(n)))}function To(){return{typeToTypeNode:function(e,n,r,i){return t(n,r,i,function(n){return l(e,n)})},indexInfoToIndexSignatureDeclaration:function(e,n,r,i){return t(n,r,i,function(n){return _(e,n,void 0)})},signatureToSignatureDeclaration:function(e,n,r,i,a){return t(r,i,a,function(t){return g(e,n,t)})},symbolToEntityName:function(e,n,r,i,a){return t(r,i,a,function(t){return N(e,t,n,!1)})},symbolToExpression:function(e,n,r,i,a){return t(r,i,a,function(t){return k(e,t,n)})},symbolToTypeParameterDeclarations:function(e,n,r,i){return t(n,r,i,function(n){return S(e,n)})},symbolToParameterDeclaration:function(e,n,r,i){return t(n,r,i,function(n){return v(e,n)})},typeParameterToDeclaration:function(e,n,r,i){return t(n,r,i,function(n){return h(e,n)})},symbolTableToDeclarationStatements:function(e,n,r,i,a){return t(n,r,i,function(n){return q(e,n,a)})}};function t(t,r,a,o){var s,c;e.Debug.assert(void 0===t||0===(8&t.flags));var l={enclosingDeclaration:t,flags:r||0,tracker:a&&a.trackSymbol?a:{trackSymbol:function(){return!1},moduleResolverHost:134217728&r?{getCommonSourceDirectory:n.getCommonSourceDirectory?function(){return n.getCommonSourceDirectory()}:function(){return""},getCurrentDirectory:function(){return n.getCurrentDirectory()},getSymlinkCache:e.maybeBind(n,n.getSymlinkCache),useCaseSensitiveFileNames:e.maybeBind(n,n.useCaseSensitiveFileNames),redirectTargetsMap:n.redirectTargetsMap,getProjectReferenceRedirect:function(e){return n.getProjectReferenceRedirect(e)},isSourceOfProjectReferenceRedirect:function(e){return n.isSourceOfProjectReferenceRedirect(e)},fileExists:function(e){return n.fileExists(e)},getFileIncludeReasons:function(){return n.getFileIncludeReasons()}}:void 0},encounteredError:!1,reportedDiagnostic:!1,visitedTypes:void 0,symbolDepth:void 0,inferTypeParameters:void 0,approximateLength:0};l.tracker=i(l,l.tracker);var u=o(l);return l.truncating&&1&l.flags&&(null===(c=null===(s=l.tracker)||void 0===s?void 0:s.reportTruncationError)||void 0===c||c.call(s)),l.encounteredError?void 0:u}function i(e,n){var t=n.trackSymbol;return o(o({},n),{reportCyclicStructureError:r(n.reportCyclicStructureError),reportInaccessibleThisError:r(n.reportInaccessibleThisError),reportInaccessibleUniqueSymbolError:r(n.reportInaccessibleUniqueSymbolError),reportLikelyUnsafeImportRequiredError:r(n.reportLikelyUnsafeImportRequiredError),reportNonlocalAugmentation:r(n.reportNonlocalAugmentation),reportPrivateInBaseOfClassExpression:r(n.reportPrivateInBaseOfClassExpression),reportNonSerializableProperty:r(n.reportNonSerializableProperty),trackSymbol:t&&function(){for(var n=[],r=0;r<arguments.length;r++)n[r]=arguments[r];var i=t.apply(void 0,n);return i&&(e.reportedDiagnostic=!0),i}});function r(n){return n?function(){for(var t=[],r=0;r<arguments.length;r++)t[r]=arguments[r];return e.reportedDiagnostic=!0,n.apply(void 0,t)}:n}}function c(n){return n.truncating?n.truncating:n.truncating=n.approximateLength>(1&n.flags?e.noTruncationMaximumTruncationLength:e.defaultMaximumTruncationLength)}function l(n,t){r&&r.throwIfCancellationRequested&&r.throwIfCancellationRequested();var i=8388608&t.flags;if(t.flags&=-8388609,!n)return 262144&t.flags?(t.approximateLength+=3,e.factory.createKeywordTypeNode(129)):void(t.encounteredError=!0);if(536870912&t.flags||(n=vu(n)),1&n.flags)return t.approximateLength+=3,e.factory.createKeywordTypeNode(n===we?137:129);if(2&n.flags)return e.factory.createKeywordTypeNode(153);if(4&n.flags)return t.approximateLength+=6,e.factory.createKeywordTypeNode(148);if(8&n.flags)return t.approximateLength+=6,e.factory.createKeywordTypeNode(145);if(64&n.flags)return t.approximateLength+=6,e.factory.createKeywordTypeNode(156);if(16&n.flags&&!n.aliasSymbol)return t.approximateLength+=7,e.factory.createKeywordTypeNode(132);if(1024&n.flags&&!(1048576&n.flags)){var a=Ia(n.symbol),o=A(a,t,788968);if(Dc(a)===n)return o;var s=e.symbolName(n.symbol);return e.isIdentifierText(s,0)?V(o,e.factory.createTypeReferenceNode(s,void 0)):e.isImportTypeNode(o)?(o.isTypeOf=!0,e.factory.createIndexedAccessTypeNode(o,e.factory.createLiteralTypeNode(e.factory.createStringLiteral(s)))):e.isTypeReferenceNode(o)?e.factory.createIndexedAccessTypeNode(e.factory.createTypeQueryNode(o.typeName),e.factory.createLiteralTypeNode(e.factory.createStringLiteral(s))):e.Debug.fail("Unhandled type node kind returned from `symbolToTypeNode`.")}if(1056&n.flags)return A(n.symbol,t,788968);if(128&n.flags)return t.approximateLength+=n.value.length+2,e.factory.createLiteralTypeNode(e.setEmitFlags(e.factory.createStringLiteral(n.value,!!(268435456&t.flags)),16777216));if(256&n.flags){var d=n.value;return t.approximateLength+=(""+d).length,e.factory.createLiteralTypeNode(d<0?e.factory.createPrefixUnaryExpression(40,e.factory.createNumericLiteral(-d)):e.factory.createNumericLiteral(d))}if(2048&n.flags)return t.approximateLength+=e.pseudoBigIntToString(n.value).length+1,e.factory.createLiteralTypeNode(e.factory.createBigIntLiteral(n.value));if(512&n.flags)return t.approximateLength+=n.intrinsicName.length,e.factory.createLiteralTypeNode("true"===n.intrinsicName?e.factory.createTrue():e.factory.createFalse());if(8192&n.flags){if(!(1048576&t.flags)){if(ro(n.symbol,t.enclosingDeclaration))return t.approximateLength+=6,A(n.symbol,t,111551);t.tracker.reportInaccessibleUniqueSymbolError&&t.tracker.reportInaccessibleUniqueSymbolError()}return t.approximateLength+=13,e.factory.createTypeOperatorNode(152,e.factory.createKeywordTypeNode(149))}if(16384&n.flags)return t.approximateLength+=4,e.factory.createKeywordTypeNode(114);if(32768&n.flags)return t.approximateLength+=9,e.factory.createKeywordTypeNode(151);if(65536&n.flags)return t.approximateLength+=4,e.factory.createLiteralTypeNode(e.factory.createNull());if(131072&n.flags)return t.approximateLength+=5,e.factory.createKeywordTypeNode(142);if(4096&n.flags)return t.approximateLength+=6,e.factory.createKeywordTypeNode(149);if(67108864&n.flags)return t.approximateLength+=6,e.factory.createKeywordTypeNode(146);if(um(n))return 4194304&t.flags&&(t.encounteredError||32768&t.flags||(t.encounteredError=!0),t.tracker.reportInaccessibleThisError&&t.tracker.reportInaccessibleThisError()),t.approximateLength+=4,e.factory.createThisTypeNode();if(!i&&n.aliasSymbol&&(16384&t.flags||to(n.aliasSymbol,t.enclosingDeclaration))){var m=f(n.aliasTypeArguments,t);return!Wa(n.aliasSymbol.escapedName)||32&n.aliasSymbol.flags?A(n.aliasSymbol,t,788968,m):e.factory.createTypeReferenceNode(e.factory.createIdentifier(""),m)}var h=e.getObjectFlags(n);if(4&h)return e.Debug.assert(!!(524288&n.flags)),n.node?G(n,U):U(n);if(262144&n.flags||3&h){if(262144&n.flags&&e.contains(t.inferTypeParameters,n))return t.approximateLength+=e.symbolName(n.symbol).length+6,e.factory.createInferTypeNode(y(n,t,void 0));if(4&t.flags&&262144&n.flags&&!to(n.symbol,t.enclosingDeclaration)){var v=I(n,t);return t.approximateLength+=e.idText(v).length,e.factory.createTypeReferenceNode(e.factory.createIdentifier(e.idText(v)),void 0)}return n.symbol?A(n.symbol,t,788968):e.factory.createTypeReferenceNode(e.factory.createIdentifier("?"),void 0)}if(1048576&n.flags&&n.origin&&(n=n.origin),3145728&n.flags){var b=1048576&n.flags?xo(n.types):n.types;if(1===e.length(b))return l(b[0],t);var E=f(b,t,!0);return E&&E.length>0?1048576&n.flags?e.factory.createUnionTypeNode(E):e.factory.createIntersectionTypeNode(E):void(t.encounteredError||262144&t.flags||(t.encounteredError=!0))}if(48&h)return e.Debug.assert(!!(524288&n.flags)),F(n);if(4194304&n.flags){var T=n.type;t.approximateLength+=6;var S=l(T,t);return e.factory.createTypeOperatorNode(139,S)}if(134217728&n.flags){var x=n.texts,L=n.types,D=e.factory.createTemplateHead(x[0]),C=e.factory.createNodeArray(e.map(L,function(n,r){return e.factory.createTemplateLiteralTypeSpan(l(n,t),(r<L.length-1?e.factory.createTemplateMiddle:e.factory.createTemplateTail)(x[r+1]))}));return t.approximateLength+=2,e.factory.createTemplateLiteralType(D,C)}if(268435456&n.flags){var N=l(n.type,t);return A(n.symbol,t,788968,[N])}if(8388608&n.flags){var k=l(n.objectType,t);S=l(n.indexType,t);return t.approximateLength+=2,e.factory.createIndexedAccessTypeNode(k,S)}return 16777216&n.flags?G(n,function(e){return R(e)}):33554432&n.flags?l(n.baseType,t):e.Debug.fail("Should be unreachable.");function R(n){var r=l(n.checkType,t),i=t.inferTypeParameters;t.inferTypeParameters=n.root.inferTypeParameters;var a=l(n.extendsType,t);t.inferTypeParameters=i;var o=P(Im(n)),s=P(Nm(n));return t.approximateLength+=15,e.factory.createConditionalTypeNode(r,a,o,s)}function P(e){var n,r,i;return 1048576&e.flags?(null===(n=t.visitedTypes)||void 0===n?void 0:n.has(nf(e)))?(131072&t.flags||(t.encounteredError=!0,null===(i=null===(r=t.tracker)||void 0===r?void 0:r.reportCyclicStructureError)||void 0===i||i.call(r)),u(t)):G(e,function(e){return l(e,t)}):l(e,t)}function M(n){e.Debug.assert(!!(524288&n.flags));var r,i=n.declaration.readonlyToken?e.factory.createToken(n.declaration.readonlyToken.kind):void 0,a=n.declaration.questionToken?e.factory.createToken(n.declaration.questionToken.kind):void 0;r=Ol(n)?e.factory.createTypeOperatorNode(139,l(wl(n),t)):l(Il(n),t);var o=y(Cl(n),t,r),s=n.declaration.nameType?l(Nl(n),t):void 0,c=l(mh(kl(n),!!(4&Pl(n))),t),u=e.factory.createMappedTypeNode(i,o,s,a,c);return t.approximateLength+=10,e.setEmitFlags(u,1)}function F(n){var r,i=n.id,a=n.symbol;if(a){var o=Eo(n)?788968:111551;if(FL(a.valueDeclaration))return A(a,t,o);if(32&a.flags&&!Ms(a)&&!(a.valueDeclaration&&224===a.valueDeclaration.kind&&2048&t.flags)||896&a.flags||c())return A(a,t,o);if(null===(r=t.visitedTypes)||void 0===r?void 0:r.has(i)){var s=Do(n);return s?A(s,t,788968):u(t)}return G(n,B)}return B(n);function c(){var n,r=!!(8192&a.flags)&&e.some(a.declarations,function(n){return e.isStatic(n)}),o=!!(16&a.flags)&&(a.parent||e.forEach(a.declarations,function(e){return 300===e.parent.kind||260===e.parent.kind}));if(r||o)return(!!(4096&t.flags)||(null===(n=t.visitedTypes)||void 0===n?void 0:n.has(i)))&&(!(8&t.flags)||ro(a,t.enclosingDeclaration))}}function G(n,r){var i,a,o=n.id,s=16&e.getObjectFlags(n)&&n.symbol&&32&n.symbol.flags,c=4&e.getObjectFlags(n)&&n.node?"N"+O(n.node):16777216&n.flags?"N"+O(n.root.node):n.symbol?(s?"+":"")+w(n.symbol):void 0;t.visitedTypes||(t.visitedTypes=new e.Set),c&&!t.symbolDepth&&(t.symbolDepth=new e.Map);var l=t.enclosingDeclaration&&Wr(t.enclosingDeclaration),d=nf(n)+"|"+t.flags;l&&(l.serializedTypes||(l.serializedTypes=new e.Map));var p,f=null===(i=null===l||void 0===l?void 0:l.serializedTypes)||void 0===i?void 0:i.get(d);if(f)return f.truncating&&(t.truncating=!0),t.approximateLength+=f.addedLength,y(f);if(c){if(p=t.symbolDepth.get(c)||0,p>10)return u(t);t.symbolDepth.set(c,p+1)}t.visitedTypes.add(o);var m=t.approximateLength,_=r(n),g=t.approximateLength-m;return t.reportedDiagnostic||t.encounteredError||(t.truncating&&(_.truncating=!0),_.addedLength=g,null===(a=null===l||void 0===l?void 0:l.serializedTypes)||void 0===a||a.set(d,_)),t.visitedTypes.delete(o),c&&t.symbolDepth.set(c,p),_;function y(n){return e.nodeIsSynthesized(n)||e.getParseTreeNode(n)!==n?e.setTextRange(e.factory.cloneNode(e.visitEachChild(n,y,e.nullTransformationContext)),n):n}}function B(n){if(Bl(n)||n.containsError)return M(n);var r=Ul(n);if(!r.properties.length&&!r.indexInfos.length){if(!r.callSignatures.length&&!r.constructSignatures.length)return t.approximateLength+=2,e.setEmitFlags(e.factory.createTypeLiteralNode(void 0),1);if(1===r.callSignatures.length&&!r.constructSignatures.length){var i=r.callSignatures[0],a=g(i,177,t);return a}if(1===r.constructSignatures.length&&!r.callSignatures.length){i=r.constructSignatures[0],a=g(i,178,t);return a}}var o=e.filter(r.constructSignatures,function(e){return!!(4&e.flags)});if(e.some(o)){var s=e.map(o,vd),c=r.callSignatures.length+(r.constructSignatures.length-o.length)+r.indexInfos.length+(2048&t.flags?e.countWhere(r.properties,function(e){return!(4194304&e.flags)}):e.length(r.properties));return c&&s.push(Ya(r)),l(Df(s),t)}var u=t.flags;t.flags|=4194304;var d=K(r);t.flags=u;var p=e.factory.createTypeLiteralNode(d);return t.approximateLength+=2,e.setEmitFlags(p,1024&t.flags?0:1),p}function U(n){var r=Md(n);if(n.target===kn||n.target===Rn){if(2&t.flags){var i=l(r[0],t);return e.factory.createTypeReferenceNode(n.target===kn?"Array":"ReadonlyArray",[i])}var a=l(r[0],t),o=e.factory.createArrayTypeNode(a);return n.target===kn?o:e.factory.createTypeOperatorNode(143,o)}if(!(8&n.target.objectFlags)){if(2048&t.flags&&n.symbol.valueDeclaration&&e.isClassLike(n.symbol.valueDeclaration)&&!ro(n.symbol,t.enclosingDeclaration))return F(n);var s=n.target.outerTypeParameters,c=(T=0,void 0);if(s){var u=s.length;while(T<u){var d=T,p=Id(s[T]);do{T++}while(T<u&&Id(s[T])===p);if(!e.rangeEquals(s,r,d,T)){var m=f(r.slice(d,T),t),_=t.flags;t.flags|=16;var g=A(p,t,788968,m);t.flags=_,c=c?V(c,g):g}}}var y=void 0;if(r.length>0){var h=(n.target.typeParameters||e.emptyArray).length;y=f(r.slice(T,h),t)}S=t.flags;t.flags|=16;var v=A(n.symbol,t,788968,y);return t.flags=S,c?V(c,v):v}if(r=e.sameMap(r,function(e,t){return mh(e,!!(2&n.target.elementFlags[t]))}),r.length>0){var b=Fd(n),E=f(r.slice(0,b),t);if(E){if(n.target.labeledElementDeclarations)for(var T=0;T<E.length;T++){var S=n.target.elementFlags[T];E[T]=e.factory.createNamedTupleMember(12&S?e.factory.createToken(25):void 0,e.factory.createIdentifier(e.unescapeLeadingUnderscores(oD(n.target.labeledElementDeclarations[T]))),2&S?e.factory.createToken(57):void 0,4&S?e.factory.createArrayTypeNode(E[T]):E[T])}else for(var T=0;T<Math.min(b,E.length);T++){var S=n.target.elementFlags[T];E[T]=12&S?e.factory.createRestTypeNode(4&S?e.factory.createArrayTypeNode(E[T]):E[T]):2&S?e.factory.createOptionalTypeNode(E[T]):E[T]}var x=e.setEmitFlags(e.factory.createTupleTypeNode(E),1);return n.target.readonly?e.factory.createTypeOperatorNode(143,x):x}}if(t.encounteredError||524288&t.flags){x=e.setEmitFlags(e.factory.createTupleTypeNode([]),1);return n.target.readonly?e.factory.createTypeOperatorNode(143,x):x}t.encounteredError=!0}function V(n,t){if(e.isImportTypeNode(n)){var r=n.typeArguments,i=n.qualifier;i&&(i=e.isIdentifier(i)?e.factory.updateIdentifier(i,r):e.factory.updateQualifiedName(i,i.left,e.factory.updateIdentifier(i.right,r))),r=t.typeArguments;for(var a=j(t),o=0,s=a;o<s.length;o++){var c=s[o];i=i?e.factory.createQualifiedName(i,c):c}return e.factory.updateImportTypeNode(n,n.argument,i,r,n.isTypeOf)}r=n.typeArguments;var l=n.typeName;l=e.isIdentifier(l)?e.factory.updateIdentifier(l,r):e.factory.updateQualifiedName(l,l.left,e.factory.updateIdentifier(l.right,r)),r=t.typeArguments;a=j(t);for(var u=0,d=a;u<d.length;u++){c=d[u];l=e.factory.createQualifiedName(l,c)}return e.factory.updateTypeReferenceNode(n,l,r)}function j(n){var t=n.typeName,r=[];while(!e.isIdentifier(t))r.unshift(t.right),t=t.left;return r.unshift(t),r}function K(n){if(c(t))return[e.factory.createPropertySignature(void 0,"...",void 0,void 0)];for(var r=[],i=0,a=n.callSignatures;i<a.length;i++){var o=a[i];r.push(g(o,172,t))}for(var s=0,l=n.constructSignatures;s<l.length;s++){o=l[s];4&o.flags||r.push(g(o,173,t))}for(var d=0,f=n.indexInfos;d<f.length;d++){var m=f[d];r.push(_(m,t,1024&n.objectFlags?u(t):void 0))}var y=n.properties;if(!y)return r;for(var h=0,v=0,b=y;v<b.length;v++){var E=b[v];if(h++,2048&t.flags){if(4194304&E.flags)continue;24&e.getDeclarationModifierFlagsFromSymbol(E)&&t.tracker.reportPrivateInBaseOfClassExpression&&t.tracker.reportPrivateInBaseOfClassExpression(e.unescapeLeadingUnderscores(E.escapedName))}if(c(t)&&h+2<y.length-1){r.push(e.factory.createPropertySignature(void 0,"... "+(y.length-h)+" more ...",void 0,void 0)),p(y[y.length-1],t,r);break}p(E,t,r)}return r.length?r:void 0}}function u(n){return n.approximateLength+=3,1&n.flags?e.factory.createKeywordTypeNode(129):e.factory.createTypeReferenceNode(e.factory.createIdentifier("..."),void 0)}function d(n,t){var r;return!!(8192&e.getCheckFlags(n))&&(e.contains(t.reverseMappedStack,n)||(null===(r=t.reverseMappedStack)||void 0===r?void 0:r[0])&&!(16&e.getObjectFlags(e.last(t.reverseMappedStack).propertyType)))}function p(n,t,r){var i,a,o=!!(8192&e.getCheckFlags(n)),s=d(n,t)?Ie:zs(n),c=t.enclosingDeclaration;if(t.enclosingDeclaration=void 0,t.tracker.trackSymbol&&4096&e.getCheckFlags(n)&&Bc(n.escapedName))if(n.declarations){var l=e.first(n.declarations);if(Uc(l))if(e.isBinaryExpression(l)){var p=e.getNameOfDeclaration(l);p&&e.isElementAccessExpression(p)&&e.isPropertyAccessEntityNameExpression(p.argumentExpression)&&b(p.argumentExpression,c,t)}else b(l.name.expression,c,t)}else(null===(i=t.tracker)||void 0===i?void 0:i.reportNonSerializableProperty)&&t.tracker.reportNonSerializableProperty(mo(n));t.enclosingDeclaration=n.valueDeclaration||(null===(a=n.declarations)||void 0===a?void 0:a[0])||c;var f=G(n,t);t.enclosingDeclaration=c,t.approximateLength+=e.symbolName(n).length+1;var m=16777216&n.flags?e.factory.createToken(57):void 0;if(8208&n.flags&&!Vl(s).length&&!JD(n))for(var _=Au(Sb(s,function(e){return!(32768&e.flags)}),0),y=0,h=_;y<h.length;y++){var v=h[y],E=g(v,166,t,{name:f,questionToken:m});r.push(L(E))}else{var T=void 0;d(n,t)?T=u(t):(o&&(t.reverseMappedStack||(t.reverseMappedStack=[]),t.reverseMappedStack.push(n)),T=s?H(t,s,n,c):e.factory.createKeywordTypeNode(129),o&&t.reverseMappedStack.pop());var S=JD(n)?[e.factory.createToken(143)]:void 0;S&&(t.approximateLength+=9);var x=e.factory.createPropertySignature(S,f,m,T);r.push(L(x))}function L(t){var r;if(e.some(n.declarations,function(e){return 342===e.kind})){var i=null===(r=n.declarations)||void 0===r?void 0:r.find(function(e){return 342===e.kind}),a=e.getTextOfJSDocComment(i.comment);a&&e.setSyntheticLeadingComments(t,[{kind:3,text:"*\n * "+a.replace(/\n/g,"\n * ")+"\n ",pos:-1,end:-1,hasTrailingNewLine:!0}])}else n.valueDeclaration&&e.setCommentRange(t,n.valueDeclaration);return t}}function f(n,t,r){if(e.some(n)){if(c(t)){if(!r)return[e.factory.createTypeReferenceNode("...",void 0)];if(n.length>2)return[l(n[0],t),e.factory.createTypeReferenceNode("... "+(n.length-2)+" more ...",void 0),l(n[n.length-1],t)]}for(var i=!(64&t.flags),a=i?e.createUnderscoreEscapedMultiMap():void 0,o=[],s=0,u=0,d=n;u<d.length;u++){var p=d[u];if(s++,c(t)&&s+2<n.length-1){o.push(e.factory.createTypeReferenceNode("... "+(n.length-s)+" more ...",void 0));var f=l(n[n.length-1],t);f&&o.push(f);break}t.approximateLength+=2;var _=l(p,t);_&&(o.push(_),a&&e.isIdentifierTypeReference(_)&&a.add(_.typeName.escapedText,[p,o.length-1]))}if(a){var g=t.flags;t.flags|=64,a.forEach(function(n){if(!e.arrayIsHomogeneous(n,function(e,n){var t=e[0],r=n[0];return m(t,r)}))for(var r=0,i=n;r<i.length;r++){var a=i[r],s=a[0],c=a[1];o[c]=l(s,t)}}),t.flags=g}return o}}function m(e,n){return e===n||!!e.symbol&&e.symbol===n.symbol||!!e.aliasSymbol&&e.aliasSymbol===n.aliasSymbol}function _(n,t,r){var i=e.getNameFromIndexInfo(n)||"x",a=l(n.keyType,t),o=e.factory.createParameterDeclaration(void 0,void 0,void 0,i,void 0,a,void 0);return r||(r=l(n.type||Ie,t)),n.type||2097152&t.flags||(t.encounteredError=!0),t.approximateLength+=i.length+4,e.factory.createIndexSignature(void 0,n.isReadonly?[e.factory.createToken(143)]:void 0,[o],r)}function g(n,t,r,i){var a,o,s,c,u,d,p=256&r.flags;p&&(r.flags&=-257),r.approximateLength+=3,32&r.flags&&n.target&&n.mapper&&n.target.typeParameters?d=n.target.typeParameters.map(function(e){return l(z_(e,n.mapper),r)}):u=n.typeParameters&&n.typeParameters.map(function(e){return h(e,r)});var f,m=il(n,!0)[0],_=(e.some(m,function(n){return n!==m[m.length-1]&&!!(32768&e.getCheckFlags(n))})?n.parameters:m).map(function(e){return v(e,r,169===t,null===i||void 0===i?void 0:i.privateSymbolVisitor,null===i||void 0===i?void 0:i.bundledImports)});if(n.thisParameter){var g=v(n.thisParameter,r);_.unshift(g)}var y=td(n);if(y){var b=2===y.kind||3===y.kind?e.factory.createToken(128):void 0,E=1===y.kind||3===y.kind?e.setEmitFlags(e.factory.createIdentifier(y.parameterName),16777216):e.factory.createThisTypeNode(),T=y.type&&l(y.type,r);f=e.factory.createTypePredicateNode(b,E,T)}else{var S=ad(n);!S||p&&Vo(S)?p||(f=e.factory.createKeywordTypeNode(129)):f=W(r,S,n,null===i||void 0===i?void 0:i.privateSymbolVisitor,null===i||void 0===i?void 0:i.bundledImports)}var x=null===i||void 0===i?void 0:i.modifiers;if(178===t&&4&n.flags){var L=e.modifiersToFlags(x);x=e.factory.createModifiersFromModifierFlags(128|L)}var D=172===t?e.factory.createCallSignature(u,_,f):173===t?e.factory.createConstructSignature(u,_,f):166===t?e.factory.createMethodSignature(x,null!==(a=null===i||void 0===i?void 0:i.name)&&void 0!==a?a:e.factory.createIdentifier(""),null===i||void 0===i?void 0:i.questionToken,u,_,f):167===t?e.factory.createMethodDeclaration(void 0,x,void 0,null!==(o=null===i||void 0===i?void 0:i.name)&&void 0!==o?o:e.factory.createIdentifier(""),void 0,u,_,f,void 0):169===t?e.factory.createConstructorDeclaration(void 0,x,_,void 0):170===t?e.factory.createGetAccessorDeclaration(void 0,x,null!==(s=null===i||void 0===i?void 0:i.name)&&void 0!==s?s:e.factory.createIdentifier(""),_,f,void 0):171===t?e.factory.createSetAccessorDeclaration(void 0,x,null!==(c=null===i||void 0===i?void 0:i.name)&&void 0!==c?c:e.factory.createIdentifier(""),_,void 0):174===t?e.factory.createIndexSignature(void 0,x,_,f):312===t?e.factory.createJSDocFunctionType(_,f):177===t?e.factory.createFunctionTypeNode(u,_,null!==f&&void 0!==f?f:e.factory.createTypeReferenceNode(e.factory.createIdentifier(""))):178===t?e.factory.createConstructorTypeNode(x,u,_,null!==f&&void 0!==f?f:e.factory.createTypeReferenceNode(e.factory.createIdentifier(""))):254===t?e.factory.createFunctionDeclaration(void 0,x,void 0,(null===i||void 0===i?void 0:i.name)?e.cast(i.name,e.isIdentifier):e.factory.createIdentifier(""),u,_,f,void 0):211===t?e.factory.createFunctionExpression(x,void 0,(null===i||void 0===i?void 0:i.name)?e.cast(i.name,e.isIdentifier):e.factory.createIdentifier(""),u,_,f,e.factory.createBlock([])):212===t?e.factory.createArrowFunction(x,u,_,f,void 0,e.factory.createBlock([])):e.Debug.assertNever(t);return d&&(D.typeArguments=e.factory.createNodeArray(d)),D}function y(n,t,r){var i=t.flags;t.flags&=-513;var a=I(n,t),o=lu(n),s=o&&l(o,t);return t.flags=i,e.factory.createTypeParameterDeclaration(a,r,s)}function h(e,n,t){void 0===t&&(t=ql(e));var r=t&&l(t,n);return y(e,n,r)}function v(n,t,r,i,a){var o=e.getDeclarationOfKind(n,162);o||e.isTransientSymbol(n)||(o=e.getDeclarationOfKind(n,335));var s=Ws(n);o&&iO(o)&&(s=sh(s)),1073741824&t.flags&&o&&!e.isJSDocParameterTag(o)&&oO(o)&&(s=Jv(s,524288));var c=H(t,s,n,t.enclosingDeclaration,i,a),l=!(8192&t.flags)&&r&&o&&o.modifiers?o.modifiers.map(e.factory.cloneNode):void 0,u=o&&e.isRestParameter(o)||32768&e.getCheckFlags(n),d=u?e.factory.createToken(25):void 0,p=o&&o.name?79===o.name.kind?e.setEmitFlags(e.factory.cloneNode(o.name),16777216):159===o.name.kind?e.setEmitFlags(e.factory.cloneNode(o.name.right),16777216):g(o.name):e.symbolName(n),f=o&&ju(o)||16384&e.getCheckFlags(n),m=f?e.factory.createToken(57):void 0,_=e.factory.createParameterDeclaration(void 0,l,d,p,m,c,void 0);return t.approximateLength+=e.symbolName(n).length+3,_;function g(n){return r(n);function r(n){t.tracker.trackSymbol&&e.isComputedPropertyName(n)&&Gc(n)&&b(n.expression,t.enclosingDeclaration,t);var i=e.visitEachChild(n,r,e.nullTransformationContext,void 0,r);return e.isBindingElement(i)&&(i=e.factory.updateBindingElement(i,i.dotDotDotToken,i.propertyName,i.name,void 0)),e.nodeIsSynthesized(i)||(i=e.factory.cloneNode(i)),e.setEmitFlags(i,16777217)}}}function b(n,t,r){if(r.tracker.trackSymbol){var i=e.getFirstIdentifier(n),a=Qr(i,i.escapedText,1160127,void 0,void 0,!0);a&&r.tracker.trackSymbol(a,t,111551)}}function E(e,n,t,r){return n.tracker.trackSymbol(e,n.enclosingDeclaration,t),T(e,n,t,r)}function T(n,t,r,i){var a,o=262144&n.flags;return o||!(t.enclosingDeclaration||64&t.flags)||134217728&t.flags?a=[n]:(a=e.Debug.checkDefined(s(n,r,!0)),e.Debug.assert(a&&a.length>0)),a;function s(n,r,a){var o,c=$a(n,t.enclosingDeclaration,r,!!(128&t.flags));if(!c||eo(c[0],t.enclosingDeclaration,1===c.length?r:Za(r))){var l=ka(c?c[0]:n,t.enclosingDeclaration,r);if(e.length(l)){o=l.map(function(n){return e.some(n.declarations,uo)?D(n,t):void 0});var u=l.map(function(e,n){return n});u.sort(g);for(var d=u.map(function(e){return l[e]}),p=0,f=d;p<f.length;p++){var m=f[p],_=s(m,Za(r),!1);if(_){if(m.exports&&m.exports.get("export=")&&Pa(m.exports.get("export="),n)){c=_;break}c=_.concat(c||[wa(m,n)||n]);break}}}}if(c)return c;if(a||!(6144&n.flags)){if(!a&&!i&&e.forEach(n.declarations,uo))return;return[n]}function g(n,t){var r=o[n],i=o[t];if(r&&i){var a=e.pathIsRelative(i);return e.pathIsRelative(r)===a?e.moduleSpecifiers.countPathComponents(r)-e.moduleSpecifiers.countPathComponents(i):a?-1:1}return 0}}}function S(n,t){var r,i=pk(n);return 524384&i.flags&&(r=e.factory.createNodeArray(e.map($s(n),function(e){return h(e,t)}))),r}function x(n,t,r){var i;e.Debug.assert(n&&0<=t&&t<n.length);var a=n[t],o=w(a);if(null===(i=r.typeParameterSymbolList)||void 0===i||!i.has(o)){var s;if((r.typeParameterSymbolList||(r.typeParameterSymbolList=new e.Set)).add(o),512&r.flags&&t<n.length-1){var c=a,l=n[t+1];if(1&e.getCheckFlags(l)){var u=ec(2097152&c.flags?Ji(c):c);s=f(e.map(u,function(e){return y_(e,l.mapper)}),r)}else s=S(a,r)}return s}}function L(n){return e.isIndexedAccessTypeNode(n.objectType)?L(n.objectType):n}function D(n,t){var r,i=e.getDeclarationOfKind(n,300);if(!i){var a=e.firstDefined(n.declarations,function(e){return Oa(e,n)});a&&(i=e.getDeclarationOfKind(a,300))}if(i&&void 0!==i.moduleName)return i.moduleName;if(!i){if(t.tracker.trackReferencedAmbientModule){var c=e.filter(n.declarations,e.isAmbientModule);if(e.length(c))for(var l=0,u=c;l<u.length;l++){var d=u[l];t.tracker.trackReferencedAmbientModule(d,n)}}if(s.test(n.escapedName))return n.escapedName.substring(1,n.escapedName.length-1)}if(!t.enclosingDeclaration||!t.tracker.moduleResolverHost)return s.test(n.escapedName)?n.escapedName.substring(1,n.escapedName.length-1):e.getSourceFileOfNode(e.getNonAugmentationDeclaration(n)).fileName;var p=e.getSourceFileOfNode(e.getOriginalNode(t.enclosingDeclaration)),f=Hr(n),m=f.specifierCache&&f.specifierCache.get(p.path);if(!m){var _=!!e.outFile(M),g=t.tracker.moduleResolverHost,y=_?o(o({},M),{baseUrl:g.getCommonSourceDirectory()}):M;m=e.first(e.moduleSpecifiers.getModuleSpecifiers(n,de,y,p,g,{importModuleSpecifierPreference:_?"non-relative":"project-relative",importModuleSpecifierEnding:_?"minimal":void 0})),null!==(r=f.specifierCache)&&void 0!==r||(f.specifierCache=new e.Map),f.specifierCache.set(p.path,m)}return m}function A(n,t,r,i){var a=E(n,t,r,!(16384&t.flags)),o=111551===r;if(e.some(a[0].declarations,uo)){var s=a.length>1?g(a,a.length-1,1):void 0,c=i||x(a,0,t),l=D(a[0],t);!(67108864&t.flags)&&e.getEmitModuleResolutionKind(M)===e.ModuleResolutionKind.NodeJs&&l.indexOf("/node_modules/")>=0&&(t.encounteredError=!0,t.tracker.reportLikelyUnsafeImportRequiredError&&t.tracker.reportLikelyUnsafeImportRequiredError(l));var u=e.factory.createLiteralTypeNode(e.factory.createStringLiteral(l));if(t.tracker.trackExternalModuleSymbolOfImportTypeNode&&t.tracker.trackExternalModuleSymbolOfImportTypeNode(a[0]),t.approximateLength+=l.length+10,!s||e.isEntityName(s)){if(s){var d=e.isIdentifier(s)?s:s.right;d.typeArguments=void 0}return e.factory.createImportTypeNode(u,s,c,o)}var p=L(s),f=p.objectType.typeName;return e.factory.createIndexedAccessTypeNode(e.factory.createImportTypeNode(u,f,c,o),p.indexType)}var m=g(a,a.length-1,0);if(e.isIndexedAccessTypeNode(m))return m;if(o)return e.factory.createTypeQueryNode(m);d=e.isIdentifier(m)?m:m.right;var _=d.typeArguments;return d.typeArguments=void 0,e.factory.createTypeReferenceNode(m,_);function g(n,r,a){var o,s=r===n.length-1?i:x(n,r,t),c=n[r],l=n[r-1];if(0===r)t.flags|=16777216,o=No(c,t),t.approximateLength+=(o?o.length:0)+1,t.flags^=16777216;else if(l&&Sa(l)){var u=Sa(l);e.forEachEntry(u,function(n,t){if(Pa(n,c)&&!Bc(t)&&"export="!==t)return o=e.unescapeLeadingUnderscores(t),!0})}if(o||(o=No(c,t)),t.approximateLength+=o.length+1,!(16&t.flags)&&l&&Jc(l)&&Jc(l).get(c.escapedName)&&Pa(Jc(l).get(c.escapedName),c)){var d=g(n,r-1,a);return e.isIndexedAccessTypeNode(d)?e.factory.createIndexedAccessTypeNode(d,e.factory.createLiteralTypeNode(e.factory.createStringLiteral(o))):e.factory.createIndexedAccessTypeNode(e.factory.createTypeReferenceNode(d,s),e.factory.createLiteralTypeNode(e.factory.createStringLiteral(o)))}var p=e.setEmitFlags(e.factory.createIdentifier(o,s),16777216);if(p.symbol=c,r>a){d=g(n,r-1,a);return e.isEntityName(d)?e.factory.createQualifiedName(d,p):e.Debug.fail("Impossible construct - an export of an indexed access cannot be reachable")}return p}}function C(e,n,t){var r=Qr(n.enclosingDeclaration,e,788968,void 0,e,!1);return!!r&&!(262144&r.flags&&r===t.symbol)}function I(n,t){var r,i;if(4&t.flags&&t.typeParameterNames){var a=t.typeParameterNames.get(nf(n));if(a)return a}var o=N(n.symbol,t,788968,!0);if(!(79&o.kind))return e.factory.createIdentifier("(Missing type parameter)");if(4&t.flags){var s=o.escapedText,c=(null===(r=t.typeParameterNamesByTextNextNameCount)||void 0===r?void 0:r.get(s))||0,l=s;while((null===(i=t.typeParameterNamesByText)||void 0===i?void 0:i.has(l))||C(l,t,n))c++,l=s+"_"+c;l!==s&&(o=e.factory.createIdentifier(l,o.typeArguments)),(t.typeParameterNamesByTextNextNameCount||(t.typeParameterNamesByTextNextNameCount=new e.Map)).set(s,c),(t.typeParameterNames||(t.typeParameterNames=new e.Map)).set(nf(n),o),(t.typeParameterNamesByText||(t.typeParameterNamesByText=new e.Set)).add(s)}return o}function N(n,t,r,i){var a=E(n,t,r);return!i||1===a.length||t.encounteredError||65536&t.flags||(t.encounteredError=!0),o(a,a.length-1);function o(n,r){var i=x(n,r,t),a=n[r];0===r&&(t.flags|=16777216);var s=No(a,t);0===r&&(t.flags^=16777216);var c=e.setEmitFlags(e.factory.createIdentifier(s,i),16777216);return c.symbol=a,r>0?e.factory.createQualifiedName(o(n,r-1),c):c}}function k(n,t,r){var i=E(n,t,r);return a(i,i.length-1);function a(n,r){var i=x(n,r,t),o=n[r];0===r&&(t.flags|=16777216);var s=No(o,t);0===r&&(t.flags^=16777216);var c=s.charCodeAt(0);if(e.isSingleOrDoubleQuote(c)&&e.some(o.declarations,uo))return e.factory.createStringLiteral(D(o,t));var l=35===c?s.length>1&&e.isIdentifierStart(s.charCodeAt(1),F):e.isIdentifierStart(c,F);if(0===r||l){var u=e.setEmitFlags(e.factory.createIdentifier(s,i),16777216);return u.symbol=o,r>0?e.factory.createPropertyAccessExpression(a(n,r-1),u):u}91===c&&(s=s.substring(1,s.length-1),c=s.charCodeAt(0));var d=void 0;return e.isSingleOrDoubleQuote(c)?d=e.factory.createStringLiteral(s.substring(1,s.length-1).replace(/\\./g,function(e){return e.substring(1)}),39===c):""+ +s===s&&(d=e.factory.createNumericLiteral(+s)),d||(d=e.setEmitFlags(e.factory.createIdentifier(s,i),16777216),d.symbol=o),e.factory.createElementAccessExpression(a(n,r-1),d)}}function R(n){var t=e.getNameOfDeclaration(n);return!!t&&e.isStringLiteral(t)}function P(n){var t=e.getNameOfDeclaration(n);return!!(t&&e.isStringLiteral(t)&&(t.singleQuote||!e.nodeIsSynthesized(t)&&e.startsWith(e.getTextOfNode(t,!1),"'")))}function G(n,t){var r=!!e.length(n.declarations)&&e.every(n.declarations,P),i=B(n,t,r);if(i)return i;var a=e.unescapeLeadingUnderscores(n.escapedName),o=!!e.length(n.declarations)&&e.every(n.declarations,R);return U(a,o,r)}function B(n,t,r){var i=Hr(n).nameType;if(i){if(384&i.flags){var a=""+i.value;return e.isIdentifierText(a,M.target)||HT(a)?HT(a)&&e.startsWith(a,"-")?e.factory.createComputedPropertyName(e.factory.createNumericLiteral(+a)):U(a):e.factory.createStringLiteral(a,!!r)}if(8192&i.flags)return e.factory.createComputedPropertyName(k(i.symbol,t,111551))}}function U(n,t,r){return e.isIdentifierText(n,M.target)?e.factory.createIdentifier(n):!t&&HT(n)&&+n>=0?e.factory.createNumericLiteral(+n):e.factory.createStringLiteral(n,!!r)}function V(n){var t=o({},n);return t.typeParameterNames&&(t.typeParameterNames=new e.Map(t.typeParameterNames)),t.typeParameterNamesByText&&(t.typeParameterNamesByText=new e.Set(t.typeParameterNamesByText)),t.typeParameterSymbolList&&(t.typeParameterSymbolList=new e.Set(t.typeParameterSymbolList)),t.tracker=i(t,t.tracker),t}function j(n,t){return n.declarations&&e.find(n.declarations,function(n){return!!e.getEffectiveTypeAnnotationNode(n)&&(!t||!!e.findAncestor(n,function(e){return e===t}))})}function K(n,t){return!(4&e.getObjectFlags(t))||!e.isTypeReferenceNode(n)||e.length(n.typeArguments)>=zu(t.target.typeParameters)}function H(n,t,r,i,a,o){if(t!==Re&&i){var s=j(r,i);if(s&&!e.isFunctionLikeDeclaration(s)&&!e.isGetAccessorDeclaration(s)){var c=e.getEffectiveTypeAnnotationNode(s);if(u_(c)===t&&K(c,t)){var u=J(n,c,a,o);if(u)return u}}}var d=n.flags;8192&t.flags&&t.symbol===r&&(!n.enclosingDeclaration||e.some(r.declarations,function(t){return e.getSourceFileOfNode(t)===e.getSourceFileOfNode(n.enclosingDeclaration)}))&&(n.flags|=1048576);var p=l(t,n);return n.flags=d,p}function W(n,t,r,i,a){if(t!==Re&&n.enclosingDeclaration){var o=r.declaration&&e.getEffectiveReturnTypeNode(r.declaration);if(e.findAncestor(o,function(e){return e===n.enclosingDeclaration})&&o){var s=u_(o),c=262144&s.flags&&s.isThisType?z_(s,r.mapper):s;if(c===t&&K(o,t)){var u=J(n,o,i,a);if(u)return u}}}return l(t,n)}function z(n,t,r){var i,a,o=!1,s=e.getFirstIdentifier(n);if(e.isInJSFile(n)&&(e.isExportsIdentifier(s)||e.isModuleExportsAccessExpression(s.parent)||e.isQualifiedName(s.parent)&&e.isModuleIdentifier(s.parent.left)&&e.isExportsIdentifier(s.parent.right)))return o=!0,{introducesError:o,node:n};var c=ra(s,67108863,!0,!0);if(c&&(0!==oo(c,t.enclosingDeclaration,67108863,!1).accessibility?o=!0:(null===(a=null===(i=t.tracker)||void 0===i?void 0:i.trackSymbol)||void 0===a||a.call(i,c,t.enclosingDeclaration,67108863),null===r||void 0===r||r(c)),e.isIdentifier(n))){var l=262144&c.flags?I(Dc(c),t):e.factory.cloneNode(n);return l.symbol=c,{introducesError:o,node:e.setEmitFlags(e.setOriginalNode(l,n),16777216)}}return{introducesError:o,node:n}}function J(t,i,a,o){r&&r.throwIfCancellationRequested&&r.throwIfCancellationRequested();var s=!1,c=e.getSourceFileOfNode(i),u=e.visitNode(i,d);if(!s)return u===i?e.setTextRange(e.factory.cloneNode(i),i):u;function d(r){if(e.isJSDocAllType(r)||314===r.kind)return e.factory.createKeywordTypeNode(129);if(e.isJSDocUnknownType(r))return e.factory.createKeywordTypeNode(153);if(e.isJSDocNullableType(r))return e.factory.createUnionTypeNode([e.visitNode(r.type,d),e.factory.createLiteralTypeNode(e.factory.createNull())]);if(e.isJSDocOptionalType(r))return e.factory.createUnionTypeNode([e.visitNode(r.type,d),e.factory.createKeywordTypeNode(151)]);if(e.isJSDocNonNullableType(r))return e.visitNode(r.type,d);if(e.isJSDocVariadicType(r))return e.factory.createArrayTypeNode(e.visitNode(r.type,d));if(e.isJSDocTypeLiteral(r))return e.factory.createTypeLiteralNode(e.map(r.jsDocPropertyTags,function(n){var i=e.isIdentifier(n.name)?n.name:n.name.right,a=Bo(u_(r),i.escapedText),o=a&&n.typeExpression&&u_(n.typeExpression.type)!==a?l(a,t):void 0;return e.factory.createPropertySignature(void 0,i,n.isBracketed||n.typeExpression&&e.isJSDocOptionalType(n.typeExpression.type)?e.factory.createToken(57):void 0,o||n.typeExpression&&e.visitNode(n.typeExpression.type,d)||e.factory.createKeywordTypeNode(129))}));if(e.isTypeReferenceNode(r)&&e.isIdentifier(r.typeName)&&""===r.typeName.escapedText)return e.setOriginalNode(e.factory.createKeywordTypeNode(129),r);if((e.isExpressionWithTypeArguments(r)||e.isTypeReferenceNode(r))&&e.isJSDocIndexSignature(r))return e.factory.createTypeLiteralNode([e.factory.createIndexSignature(void 0,void 0,[e.factory.createParameterDeclaration(void 0,void 0,void 0,"x",void 0,e.visitNode(r.typeArguments[0],d))],e.visitNode(r.typeArguments[1],d))]);var i;if(e.isJSDocFunctionType(r))return e.isJSDocConstructSignature(r)?e.factory.createConstructorTypeNode(r.modifiers,e.visitNodes(r.typeParameters,d),e.mapDefined(r.parameters,function(n,t){return n.name&&e.isIdentifier(n.name)&&"new"===n.name.escapedText?void(i=n.type):e.factory.createParameterDeclaration(void 0,void 0,_(n),g(n,t),n.questionToken,e.visitNode(n.type,d),void 0)}),e.visitNode(i||r.type,d)||e.factory.createKeywordTypeNode(129)):e.factory.createFunctionTypeNode(e.visitNodes(r.typeParameters,d),e.map(r.parameters,function(n,t){return e.factory.createParameterDeclaration(void 0,void 0,_(n),g(n,t),n.questionToken,e.visitNode(n.type,d),void 0)}),e.visitNode(r.type,d)||e.factory.createKeywordTypeNode(129));if(e.isTypeReferenceNode(r)&&e.isInJSDoc(r)&&(!K(r,u_(r))||Zd(r)||Ae===Kd(jd(r),788968,!0)))return e.setOriginalNode(l(u_(r),t),r);if(e.isLiteralImportTypeNode(r)){var u=Wr(r).resolvedSymbol;return!e.isInJSDoc(r)||!u||(r.isTypeOf||788968&u.flags)&&e.length(r.typeArguments)>=zu($s(u))?e.factory.updateImportTypeNode(r,e.factory.updateLiteralTypeNode(r.argument,y(r,r.argument.literal)),r.qualifier,e.visitNodes(r.typeArguments,d,e.isTypeNode),r.isTypeOf):e.setOriginalNode(l(u_(r),t),r)}if(e.isEntityName(r)||e.isEntityNameExpression(r)){var p=z(r,t,a),f=p.introducesError,m=p.node;if(s=s||f,m!==r)return m}return c&&e.isTupleTypeNode(r)&&e.getLineAndCharacterOfPosition(c,r.pos).line===e.getLineAndCharacterOfPosition(c,r.end).line&&e.setEmitFlags(r,1),e.visitEachChild(r,d,e.nullTransformationContext);function _(n){return n.dotDotDotToken||(n.type&&e.isJSDocVariadicType(n.type)?e.factory.createToken(25):void 0)}function g(n,t){return n.name&&e.isIdentifier(n.name)&&"this"===n.name.escapedText?"this":_(n)?"args":"arg"+t}function y(r,i){if(o){if(t.tracker&&t.tracker.moduleResolverHost){var a=AO(r);if(a){var s=e.createGetCanonicalFileName(!!n.useCaseSensitiveFileNames),c={getCanonicalFileName:s,getCurrentDirectory:function(){return t.tracker.moduleResolverHost.getCurrentDirectory()},getCommonSourceDirectory:function(){return t.tracker.moduleResolverHost.getCommonSourceDirectory()}},l=e.getResolvedExternalModuleName(c,a);return e.factory.createStringLiteral(l)}}}else if(t.tracker&&t.tracker.trackExternalModuleSymbolOfImportTypeNode){var u=la(i,i,void 0);u&&t.tracker.trackExternalModuleSymbolOfImportTypeNode(u)}return i}}}function q(n,t,r){var s=ue(e.factory.createPropertyDeclaration,167,!0),c=ue(function(n,t,r,i,a){return e.factory.createPropertySignature(t,r,i,a)},166,!1),u=t.enclosingDeclaration,d=[],p=new e.Set,f=[],m=t;t=o(o({},m),{usedSymbolNames:new e.Set(m.usedSymbolNames),remappedSymbolNames:new e.Map,tracker:o(o({},m.tracker),{trackSymbol:function(e,n,r){var i=oo(e,n,r,!1);if(0===i.accessibility){var a=T(e,t,r);4&e.flags||U(a[0])}else if(m.tracker&&m.tracker.trackSymbol)return m.tracker.trackSymbol(e,n,r);return!1}})}),t.tracker=i(t,t.tracker),e.forEachEntry(n,function(n,t){var r=e.unescapeLeadingUnderscores(t);ve(n,r)});var y=!r,v=n.get("export=");return v&&n.size>1&&2097152&v.flags&&(n=e.createSymbolTable(),n.set("export=",v)),O(n),A(d);function b(e){return!!e&&79===e.kind}function E(n){return e.isVariableStatement(n)?e.filter(e.map(n.declarationList.declarations,e.getNameOfDeclaration),b):e.filter([e.getNameOfDeclaration(n)],b)}function S(n){var t=e.find(n,e.isExportAssignment),r=e.findIndex(n,e.isModuleDeclaration),i=-1!==r?n[r]:void 0;if(i&&t&&t.isExportEquals&&e.isIdentifier(t.expression)&&e.isIdentifier(i.name)&&e.idText(i.name)===e.idText(t.expression)&&i.body&&e.isModuleBlock(i.body)){var o=e.filter(n,function(n){return!!(1&e.getEffectiveModifierFlags(n))}),s=i.name,c=i.body;if(e.length(o)&&(i=e.factory.updateModuleDeclaration(i,i.decorators,i.modifiers,i.name,c=e.factory.updateModuleBlock(c,e.factory.createNodeArray(a(a([],i.body.statements,!0),[e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports(e.map(e.flatMap(o,function(e){return E(e)}),function(n){return e.factory.createExportSpecifier(void 0,n)})),void 0)],!1)))),n=a(a(a([],n.slice(0,r),!0),[i],!1),n.slice(r+1),!0)),!e.find(n,function(n){return n!==i&&e.nodeHasName(n,s)})){d=[];var l=!e.some(c.statements,function(n){return e.hasSyntacticModifier(n,1)||e.isExportAssignment(n)||e.isExportDeclaration(n)});e.forEach(c.statements,function(e){W(e,l?1:0)}),n=a(a([],e.filter(n,function(e){return e!==i&&e!==t}),!0),d,!0)}}return n}function x(n){var t=e.filter(n,function(n){return e.isExportDeclaration(n)&&!n.moduleSpecifier&&!!n.exportClause&&e.isNamedExports(n.exportClause)});if(e.length(t)>1){var r=e.filter(n,function(n){return!e.isExportDeclaration(n)||!!n.moduleSpecifier||!n.exportClause});n=a(a([],r,!0),[e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports(e.flatMap(t,function(n){return e.cast(n.exportClause,e.isNamedExports).elements})),void 0)],!1)}var i=e.filter(n,function(n){return e.isExportDeclaration(n)&&!!n.moduleSpecifier&&!!n.exportClause&&e.isNamedExports(n.exportClause)});if(e.length(i)>1){var o=e.group(i,function(n){return e.isStringLiteral(n.moduleSpecifier)?">"+n.moduleSpecifier.text:">"});if(o.length!==i.length)for(var s=function(t){t.length>1&&(n=a(a([],e.filter(n,function(e){return-1===t.indexOf(e)}),!0),[e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports(e.flatMap(t,function(n){return e.cast(n.exportClause,e.isNamedExports).elements})),t[0].moduleSpecifier)],!1))},c=0,l=o;c<l.length;c++){var u=l[c];s(u)}}return n}function L(n){var t=e.findIndex(n,function(n){return e.isExportDeclaration(n)&&!n.moduleSpecifier&&!!n.exportClause&&e.isNamedExports(n.exportClause)});if(t>=0){var r=n[t],i=e.mapDefined(r.exportClause.elements,function(t){if(!t.propertyName){var r=e.indicesOf(n),i=e.filter(r,function(r){return e.nodeHasName(n[r],t.name)});if(e.length(i)&&e.every(i,function(e){return C(n[e])})){for(var a=0,o=i;a<o.length;a++){var s=o[a];n[s]=I(n[s])}return}}return t});e.length(i)?n[t]=e.factory.updateExportDeclaration(r,r.decorators,r.modifiers,r.isTypeOnly,e.factory.updateNamedExports(r.exportClause,i),r.moduleSpecifier):e.orderedRemoveItemAt(n,t)}return n}function A(n){return n=S(n),n=x(n),n=L(n),u&&(e.isSourceFile(u)&&e.isExternalOrCommonJsModule(u)||e.isModuleDeclaration(u))&&(!e.some(n,e.isExternalModuleIndicator)||!e.hasScopeMarker(n)&&e.some(n,e.needsScopeMarker))&&n.push(e.createEmptyExports(e.factory)),n}function C(n){return e.isEnumDeclaration(n)||e.isVariableStatement(n)||e.isFunctionDeclaration(n)||e.isClassDeclaration(n)||e.isModuleDeclaration(n)&&!e.isExternalModuleAugmentation(n)&&!e.isGlobalScopeAugmentation(n)||e.isInterfaceDeclaration(n)||gR(n)}function I(n){var t=-3&(1|e.getEffectiveModifierFlags(n));return e.factory.updateModifiers(n,t)}function R(n){var t=-2&e.getEffectiveModifierFlags(n);return e.factory.updateModifiers(n,t)}function O(n,t,r){t||f.push(new e.Map),n.forEach(function(e){P(e,!1,!!r)}),t||(f[f.length-1].forEach(function(e){P(e,!0,!!r)}),f.pop())}function P(n,r,i){var a=Aa(n);if(!p.has(w(a))){p.add(w(a));var o=!r;if(o||e.length(n.declarations)&&e.some(n.declarations,function(n){return!!e.findAncestor(n,function(e){return e===u})})){var s=t;t=V(t);var c=B(n,r,i);return t.reportedDiagnostic&&(m.reportedDiagnostic=t.reportedDiagnostic),t=s,c}}}function B(n,i,a){var o,s,c=e.unescapeLeadingUnderscores(n.escapedName),l="default"===n.escapedName;if(!i||131072&t.flags||!e.isStringANonContextualKeyword(c)||l){var d=l&&!!(-113&n.flags||16&n.flags&&e.length(Hl(Ws(n))))&&!(2097152&n.flags),p=!d&&!i&&e.isStringANonContextualKeyword(c)&&!l;(d||p)&&(i=!0);var f=(i?0:1)|(l&&!d?512:0),m=1536&n.flags&&7&n.flags&&"export="!==n.escapedName,_=m&&le(Ws(n),n);if((8208&n.flags||_)&&ee(Ws(n),n,ve(n,c),f),524288&n.flags&&q(n,c,f),7&n.flags&&"export="!==n.escapedName&&!(4194304&n.flags)&&!(32&n.flags)&&!_)if(a){var g=ce(n);g&&(p=!1,d=!1)}else{var y=Ws(n),h=ve(n,c);if(16&n.flags||!le(y,n)){var v=2&n.flags?sE(n)?2:1:void 0,b=!d&&4&n.flags?ye(h,n):h,E=n.declarations&&e.find(n.declarations,function(n){return e.isVariableDeclaration(n)});E&&e.isVariableDeclarationList(E.parent)&&1===E.parent.declarations.length&&(E=E.parent.parent);var T=null===(o=n.declarations)||void 0===o?void 0:o.find(e.isPropertyAccessExpression);if(T&&e.isBinaryExpression(T.parent)&&e.isIdentifier(T.parent.right)&&(null===(s=y.symbol)||void 0===s?void 0:s.valueDeclaration)&&e.isSourceFile(y.symbol.valueDeclaration)){var S=h===T.parent.right.escapedText?void 0:T.parent.right;W(e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports([e.factory.createExportSpecifier(S,h)])),0),t.tracker.trackSymbol(y.symbol,t.enclosingDeclaration,111551)}else{var x=e.setTextRange(e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(b,void 0,H(t,y,n,u,U,r))],v)),E);W(x,b!==h?-2&f:f),b===h||i||(W(e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports([e.factory.createExportSpecifier(b,h)])),0),p=!1,d=!1)}}else ee(y,n,h,f)}if(384&n.flags&&$(n,c,f),32&n.flags&&(4&n.flags&&n.valueDeclaration&&e.isBinaryExpression(n.valueDeclaration.parent)&&e.isClassExpression(n.valueDeclaration.parent.right)?oe(n,ve(n,c),f):ae(n,ve(n,c),f)),(1536&n.flags&&(!m||Q(n))||_)&&Z(n,c,f),64&n.flags&&!(32&n.flags)&&X(n,c,f),2097152&n.flags&&oe(n,ve(n,c),f),4&n.flags&&"export="===n.escapedName&&ce(n),8388608&n.flags&&n.declarations)for(var L=0,A=n.declarations;L<A.length;L++){var C=A[L],I=ca(C,C.moduleSpecifier);I&&W(e.factory.createExportDeclaration(void 0,void 0,!1,void 0,e.factory.createStringLiteral(D(I,t))),0)}d?W(e.factory.createExportAssignment(void 0,void 0,!1,e.factory.createIdentifier(ve(n,c))),0):p&&W(e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports([e.factory.createExportSpecifier(ve(n,c),c)])),0)}else t.encounteredError=!0}function U(n){if(!e.some(n.declarations,e.isParameterDeclaration)){e.Debug.assertIsDefined(f[f.length-1]),ye(e.unescapeLeadingUnderscores(n.escapedName),n);var t=!!(2097152&n.flags)&&!e.some(n.declarations,function(n){return!!e.findAncestor(n,e.isExportDeclaration)||e.isNamespaceExport(n)||e.isImportEqualsDeclaration(n)&&!e.isExternalModuleReference(n.moduleReference)});f[t?0:f.length-1].set(w(n),n)}}function K(n){return e.isSourceFile(n)&&(e.isExternalOrCommonJsModule(n)||e.isJsonSourceFile(n))||e.isAmbientModule(n)&&!e.isGlobalScopeAugmentation(n)}function W(n,r){if(e.canHaveModifiers(n)){var i=0,a=t.enclosingDeclaration&&(e.isJSDocTypeAlias(t.enclosingDeclaration)?e.getSourceFileOfNode(t.enclosingDeclaration):t.enclosingDeclaration);1&r&&a&&(K(a)||e.isModuleDeclaration(a))&&C(n)&&(i|=1),!y||1&i||a&&8388608&a.flags||!(e.isEnumDeclaration(n)||e.isVariableStatement(n)||e.isFunctionDeclaration(n)||e.isClassDeclaration(n)||e.isModuleDeclaration(n))||(i|=2),512&r&&(e.isClassDeclaration(n)||e.isInterfaceDeclaration(n)||e.isFunctionDeclaration(n))&&(i|=512),i&&(n=e.factory.updateModifiers(n,i|e.getEffectiveModifierFlags(n)))}d.push(n)}function q(n,i,a){var o,s=yc(n),c=Hr(n).typeParameters,u=e.map(c,function(e){return h(e,t)}),d=null===(o=n.declarations)||void 0===o?void 0:o.find(e.isJSDocTypeAlias),p=e.getTextOfJSDocComment(d?d.comment||d.parent.comment:void 0),f=t.flags;t.flags|=8388608;var m=t.enclosingDeclaration;t.enclosingDeclaration=d;var _=d&&d.typeExpression&&e.isJSDocTypeExpression(d.typeExpression)&&J(t,d.typeExpression.type,U,r)||l(s,t);W(e.setSyntheticLeadingComments(e.factory.createTypeAliasDeclaration(void 0,void 0,ve(n,i),u,_),p?[{kind:3,text:"*\n * "+p.replace(/\n/g,"\n * ")+"\n ",pos:-1,end:-1,hasTrailingNewLine:!0}]:[]),a),t.flags=f,t.enclosingDeclaration=m}function X(n,r,i){var o=gc(n),s=$s(n),c=e.map(s,function(e){return h(e,t)}),l=lc(o),u=e.length(l)?Df(l):void 0,d=e.flatMap(Hl(o),function(e){return de(e,u)}),p=pe(0,o,u,172),f=pe(1,o,u,173),m=fe(o,u),_=e.length(l)?[e.factory.createHeritageClause(94,e.mapDefined(l,function(e){return _e(e,111551)}))]:void 0;W(e.factory.createInterfaceDeclaration(void 0,void 0,ve(n,r),c,_,a(a(a(a([],m,!0),f,!0),p,!0),d,!0)),i)}function Y(n){return n.exports?e.filter(e.arrayFrom(n.exports.values()),re):[]}function Q(n){return e.every(Y(n),function(e){return!(111551&zi(e).flags)})}function Z(n,r,i){var a=Y(n),o=e.arrayToMultiMap(a,function(e){return e.parent&&e.parent===n?"real":"merged"}),s=o.get("real")||e.emptyArray,c=o.get("merged")||e.emptyArray;if(e.length(s)){var l=ve(n,r);te(s,l,i,!!(67108880&n.flags))}if(e.length(c)){var u=e.getSourceFileOfNode(t.enclosingDeclaration),d=(l=ve(n,r),e.factory.createModuleBlock([e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports(e.mapDefined(e.filter(c,function(e){return"export="!==e.escapedName}),function(r){var i,a,o=e.unescapeLeadingUnderscores(r.escapedName),s=ve(r,o),c=r.declarations&&hi(r);if(!u||(c?u===e.getSourceFileOfNode(c):e.some(r.declarations,function(n){return e.getSourceFileOfNode(n)===u}))){var l=c&&Hi(c,!0);U(l||r);var d=l?ve(l,e.unescapeLeadingUnderscores(l.escapedName)):s;return e.factory.createExportSpecifier(o===d?void 0:d,o)}null===(a=null===(i=t.tracker)||void 0===i?void 0:i.reportNonlocalAugmentation)||void 0===a||a.call(i,u,n,r)})))]));W(e.factory.createModuleDeclaration(void 0,void 0,e.factory.createIdentifier(l),d,16),0)}}function $(n,t,r){W(e.factory.createEnumDeclaration(void 0,e.factory.createModifiersFromModifierFlags(cA(n)?2048:0),ve(n,t),e.map(e.filter(Hl(Ws(n)),function(e){return!!(8&e.flags)}),function(n){var t=n.declarations&&n.declarations[0]&&e.isEnumMember(n.declarations[0])?pO(n.declarations[0]):void 0;return e.factory.createEnumMember(e.unescapeLeadingUnderscores(n.escapedName),void 0===t?void 0:"string"===typeof t?e.factory.createStringLiteral(t):e.factory.createNumericLiteral(t))})),r)}function ee(n,i,a,o){for(var s=Au(n,0),c=0,l=s;c<l.length;c++){var u=l[c],d=g(u,254,t,{name:e.factory.createIdentifier(a),privateSymbolVisitor:U,bundledImports:r});W(e.setTextRange(d,ne(u)),o)}if(!(1536&i.flags&&i.exports&&i.exports.size)){var p=e.filter(Hl(n),re);te(p,a,o,!0)}}function ne(n){if(n.declaration&&n.declaration.parent){if(e.isBinaryExpression(n.declaration.parent)&&5===e.getAssignmentDeclarationKind(n.declaration.parent))return n.declaration.parent;if(e.isVariableDeclaration(n.declaration.parent)&&n.declaration.parent.parent)return n.declaration.parent.parent}return n.declaration}function te(n,r,i,a){if(e.length(n)){var s=e.arrayToMultiMap(n,function(n){return!e.length(n.declarations)||e.some(n.declarations,function(n){return e.getSourceFileOfNode(n)===e.getSourceFileOfNode(t.enclosingDeclaration)})?"local":"remote"}),c=s.get("local")||e.emptyArray,l=e.parseNodeFactory.createModuleDeclaration(void 0,void 0,e.factory.createIdentifier(r),e.factory.createModuleBlock([]),16);e.setParent(l,u),l.locals=e.createSymbolTable(n),l.symbol=n[0].parent;var p=d;d=[];var f=y;y=!1;var m=o(o({},t),{enclosingDeclaration:l}),_=t;t=m,O(e.createSymbolTable(c),a,!0),t=_,y=f;var g=d;d=p;var h=e.map(g,function(n){return e.isExportAssignment(n)&&!n.isExportEquals&&e.isIdentifier(n.expression)?e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports([e.factory.createExportSpecifier(n.expression,e.factory.createIdentifier("default"))])):n}),v=e.every(h,function(n){return e.hasSyntacticModifier(n,1)})?e.map(h,R):h;l=e.factory.updateModuleDeclaration(l,l.decorators,l.modifiers,l.name,e.factory.createModuleBlock(v)),W(l,i)}}function re(n){return!!(2887656&n.flags)||!(4194304&n.flags||"prototype"===n.escapedName||n.valueDeclaration&&e.isStatic(n.valueDeclaration)&&e.isClassLike(n.valueDeclaration.parent))}function ie(n){var i=e.mapDefined(n,function(n){var i,a=t.enclosingDeclaration;t.enclosingDeclaration=n;var o=n.expression;if(e.isEntityNameExpression(o)){if(e.isIdentifier(o)&&""===e.idText(o))return c(void 0);var s=void 0;if(i=z(o,t,U),s=i.introducesError,o=i.node,s)return c(void 0)}return c(e.factory.createExpressionWithTypeArguments(o,e.map(n.typeArguments,function(e){return J(t,e,U,r)||l(u_(e),t)})));function c(e){return t.enclosingDeclaration=a,e}});if(i.length===n.length)return i}function ae(n,r,i){var o,c,l=null===(o=n.declarations)||void 0===o?void 0:o.find(e.isClassLike),u=t.enclosingDeclaration;t.enclosingDeclaration=l||u;var d=$s(n),p=e.map(d,function(e){return h(e,t)}),f=gc(n),m=lc(f),_=l&&e.getEffectiveImplementsTypeNodes(l),g=_&&ie(_)||e.mapDefined(sc(f),ge),y=Ws(n),v=!!(null===(c=y.symbol)||void 0===c?void 0:c.valueDeclaration)&&e.isClassLike(y.symbol.valueDeclaration),b=v?oc(y):Ie,E=a(a([],e.length(m)?[e.factory.createHeritageClause(94,e.map(m,function(e){return me(e,b,r)}))]:[],!0),e.length(g)?[e.factory.createHeritageClause(117,g)]:[],!0),T=_k(f,m,Hl(f)),S=e.filter(T,function(n){var t=n.valueDeclaration;return!!t&&!(e.isNamedDeclaration(t)&&e.isPrivateIdentifier(t.name))}),x=e.some(T,function(n){var t=n.valueDeclaration;return!!t&&e.isNamedDeclaration(t)&&e.isPrivateIdentifier(t.name)}),L=x?[e.factory.createPropertyDeclaration(void 0,void 0,e.factory.createPrivateIdentifier("#private"),void 0,void 0,void 0)]:e.emptyArray,D=e.flatMap(S,function(e){return s(e,!1,m[0])}),A=e.flatMap(e.filter(Hl(y),function(e){return!(4194304&e.flags)&&"prototype"!==e.escapedName&&!re(e)}),function(e){return s(e,!0,b)}),C=!v&&!!n.valueDeclaration&&e.isInJSFile(n.valueDeclaration)&&!e.some(Au(y,1)),I=C?[e.factory.createConstructorDeclaration(void 0,e.factory.createModifiersFromModifierFlags(8),[],void 0)]:pe(1,y,b,169),N=fe(f,m[0]);t.enclosingDeclaration=u,W(e.setTextRange(e.factory.createClassDeclaration(void 0,void 0,r,p,E,a(a(a(a(a([],N,!0),A,!0),I,!0),D,!0),L,!0)),n.declarations&&e.filter(n.declarations,function(n){return e.isClassDeclaration(n)||e.isClassExpression(n)})[0]),i)}function oe(n,r,i){var a,o,s,c,l,u=hi(n);if(!u)return e.Debug.fail();var d=Aa(Hi(u,!0));if(d){var p=e.unescapeLeadingUnderscores(d.escapedName);"export="===p&&(M.esModuleInterop||M.allowSyntheticDefaultImports)&&(p="default");var f=ve(d,p);switch(U(d),u.kind){case 201:if(252===(null===(o=null===(a=u.parent)||void 0===a?void 0:a.parent)||void 0===o?void 0:o.kind)){var m=D(d.parent||d,t),_=u.propertyName;W(e.factory.createImportDeclaration(void 0,void 0,e.factory.createImportClause(!1,void 0,e.factory.createNamedImports([e.factory.createImportSpecifier(_&&e.isIdentifier(_)?e.factory.createIdentifier(e.idText(_)):void 0,e.factory.createIdentifier(r))])),e.factory.createStringLiteral(m)),0);break}e.Debug.failBadSyntaxKind((null===(s=u.parent)||void 0===s?void 0:s.parent)||u,"Unhandled binding element grandparent kind in declaration serialization");break;case 292:219===(null===(l=null===(c=u.parent)||void 0===c?void 0:c.parent)||void 0===l?void 0:l.kind)&&se(e.unescapeLeadingUnderscores(n.escapedName),f);break;case 252:if(e.isPropertyAccessExpression(u.initializer)){var g=u.initializer,y=e.factory.createUniqueName(r),h=D(d.parent||d,t);W(e.factory.createImportEqualsDeclaration(void 0,void 0,!1,y,e.factory.createExternalModuleReference(e.factory.createStringLiteral(h))),0),W(e.factory.createImportEqualsDeclaration(void 0,void 0,!1,e.factory.createIdentifier(r),e.factory.createQualifiedName(y,g.name)),i);break}case 263:if("export="===d.escapedName&&e.some(d.declarations,e.isJsonSourceFile)){ce(n);break}var v=!(512&d.flags)&&!e.isVariableDeclaration(u);W(e.factory.createImportEqualsDeclaration(void 0,void 0,!1,e.factory.createIdentifier(r),v?N(d,t,67108863,!1):e.factory.createExternalModuleReference(e.factory.createStringLiteral(D(d,t)))),v?i:0);break;case 262:W(e.factory.createNamespaceExportDeclaration(e.idText(u.name)),0);break;case 265:W(e.factory.createImportDeclaration(void 0,void 0,e.factory.createImportClause(!1,e.factory.createIdentifier(r),void 0),e.factory.createStringLiteral(D(d.parent||d,t))),0);break;case 266:W(e.factory.createImportDeclaration(void 0,void 0,e.factory.createImportClause(!1,void 0,e.factory.createNamespaceImport(e.factory.createIdentifier(r))),e.factory.createStringLiteral(D(d,t))),0);break;case 272:W(e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamespaceExport(e.factory.createIdentifier(r)),e.factory.createStringLiteral(D(d,t))),0);break;case 268:W(e.factory.createImportDeclaration(void 0,void 0,e.factory.createImportClause(!1,void 0,e.factory.createNamedImports([e.factory.createImportSpecifier(r!==p?e.factory.createIdentifier(p):void 0,e.factory.createIdentifier(r))])),e.factory.createStringLiteral(D(d.parent||d,t))),0);break;case 273:var b=u.parent.parent.moduleSpecifier;se(e.unescapeLeadingUnderscores(n.escapedName),b?p:f,b&&e.isStringLiteralLike(b)?e.factory.createStringLiteral(b.text):void 0);break;case 269:ce(n);break;case 219:case 204:case 205:"default"===n.escapedName||"export="===n.escapedName?ce(n):se(r,f);break;default:return e.Debug.failBadSyntaxKind(u,"Unhandled alias declaration kind in symbol serializer!")}}}function se(n,t,r){W(e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports([e.factory.createExportSpecifier(n!==t?t:void 0,n)]),r),0)}function ce(n){if(4194304&n.flags)return!1;var i=e.unescapeLeadingUnderscores(n.escapedName),a="export="===i,o="default"===i,s=a||o,c=n.declarations&&hi(n),l=c&&Hi(c,!0);if(l&&e.length(l.declarations)&&e.some(l.declarations,function(n){return e.getSourceFileOfNode(n)===e.getSourceFileOfNode(u)})){var p=c&&(e.isExportAssignment(c)||e.isBinaryExpression(c)?e.getExportAssignmentExpression(c):e.getPropertyAssignmentAliasLikeExpression(c)),f=p&&e.isEntityNameExpression(p)?Ok(p):void 0,m=f&&ra(f,67108863,!0,!0,u);(m||l)&&U(m||l);var _=t.tracker.trackSymbol;if(t.tracker.trackSymbol=function(){return!1},s)d.push(e.factory.createExportAssignment(void 0,void 0,a,k(l,t,67108863)));else if(f===p&&f)se(i,e.idText(f));else if(p&&e.isClassExpression(p))se(i,ve(l,e.symbolName(l)));else{var g=ye(i,n);W(e.factory.createImportEqualsDeclaration(void 0,void 0,!1,e.factory.createIdentifier(g),N(l,t,67108863,!1)),0),se(i,g)}return t.tracker.trackSymbol=_,!0}g=ye(i,n);var y=Ch(Ws(Aa(n)));if(le(y,n))ee(y,n,g,s?0:1);else{var h=e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(g,void 0,H(t,y,n,u,U,r))],2));W(h,l&&4&l.flags&&"export="===l.escapedName?2:i===g?1:0)}return s?(d.push(e.factory.createExportAssignment(void 0,void 0,a,e.factory.createIdentifier(g))),!0):i!==g&&(se(i,g),!0)}function le(n,r){var i=e.getSourceFileOfNode(t.enclosingDeclaration);return 48&e.getObjectFlags(n)&&!e.length(Ru(n))&&!Eo(n)&&!(!e.length(e.filter(Hl(n),re))&&!e.length(Au(n,0)))&&!e.length(Au(n,1))&&!j(r,u)&&!(n.symbol&&e.some(n.symbol.declarations,function(n){return e.getSourceFileOfNode(n)!==i}))&&!e.some(Hl(n),function(e){return Bc(e.escapedName)})&&!e.some(Hl(n),function(n){return e.some(n.declarations,function(n){return e.getSourceFileOfNode(n)!==i})})&&e.every(Hl(n),function(n){return e.isIdentifierText(e.symbolName(n),F)})}function ue(n,i,a){return function(o,s,c){var l,d,p,f,m,_=e.getDeclarationModifierFlagsFromSymbol(o),y=!!(8&_);if(s&&2887656&o.flags)return[];if(4194304&o.flags||c&&Lu(c,o.escapedName)&&JD(Lu(c,o.escapedName))===JD(o)&&(16777216&o.flags)===(16777216&Lu(c,o.escapedName).flags)&&ig(Ws(o),Bo(c,o.escapedName)))return[];var h=-257&_|(s?32:0),v=G(o,t),b=null===(l=o.declarations)||void 0===l?void 0:l.find(e.or(e.isPropertyDeclaration,e.isAccessor,e.isVariableDeclaration,e.isPropertySignature,e.isBinaryExpression,e.isPropertyAccessExpression));if(98304&o.flags&&a){var E=[];if(65536&o.flags&&E.push(e.setTextRange(e.factory.createSetAccessorDeclaration(void 0,e.factory.createModifiersFromModifierFlags(h),v,[e.factory.createParameterDeclaration(void 0,void 0,void 0,"arg",void 0,y?void 0:H(t,Ws(o),o,u,U,r))],void 0),(null===(d=o.declarations)||void 0===d?void 0:d.find(e.isSetAccessor))||b)),32768&o.flags){var T=8&_;E.push(e.setTextRange(e.factory.createGetAccessorDeclaration(void 0,e.factory.createModifiersFromModifierFlags(h),v,[],T?void 0:H(t,Ws(o),o,u,U,r),void 0),(null===(p=o.declarations)||void 0===p?void 0:p.find(e.isGetAccessor))||b))}return E}if(98311&o.flags)return e.setTextRange(n(void 0,e.factory.createModifiersFromModifierFlags((JD(o)?64:0)|h),v,16777216&o.flags?e.factory.createToken(57):void 0,y?void 0:H(t,Ws(o),o,u,U,r),void 0),(null===(f=o.declarations)||void 0===f?void 0:f.find(e.or(e.isPropertyDeclaration,e.isVariableDeclaration)))||b);if(8208&o.flags){var S=Ws(o),x=Au(S,0);if(8&h)return e.setTextRange(n(void 0,e.factory.createModifiersFromModifierFlags((JD(o)?64:0)|h),v,16777216&o.flags?e.factory.createToken(57):void 0,void 0,void 0),(null===(m=o.declarations)||void 0===m?void 0:m.find(e.isFunctionLikeDeclaration))||x[0]&&x[0].declaration||o.declarations&&o.declarations[0]);for(var L=[],D=0,A=x;D<A.length;D++){var C=A[D],I=g(C,i,t,{name:v,questionToken:16777216&o.flags?e.factory.createToken(57):void 0,modifiers:h?e.factory.createModifiersFromModifierFlags(h):void 0}),N=C.declaration&&e.isPrototypePropertyAssignment(C.declaration.parent)?C.declaration.parent:C.declaration;L.push(e.setTextRange(I,N))}return L}return e.Debug.fail("Unhandled class member kind! "+(o.__debugFlags||o.flags))}}function de(e,n){return c(e,!1,n)}function pe(n,r,i,a){var o=Au(r,n);if(1===n){if(!i&&e.every(o,function(n){return 0===e.length(n.parameters)}))return[];if(i){var s=Au(i,1);if(!e.length(s)&&e.every(o,function(n){return 0===e.length(n.parameters)}))return[];if(s.length===o.length){for(var c=!1,l=0;l<s.length;l++)if(!vy(o[l],s[l],!1,!1,!0,ag)){c=!0;break}if(!c)return[]}}for(var u=0,d=0,p=o;d<p.length;d++){var f=p[d];f.declaration&&(u|=e.getSelectedEffectiveModifierFlags(f.declaration,24))}if(u)return[e.setTextRange(e.factory.createConstructorDeclaration(void 0,e.factory.createModifiersFromModifierFlags(u),[],void 0),o[0].declaration)]}for(var m=[],_=0,y=o;_<y.length;_++){var h=y[_],v=g(h,a,t);m.push(e.setTextRange(v,h.declaration))}return m}function fe(e,n){for(var r=[],i=0,a=Ru(e);i<a.length;i++){var o=a[i];if(n){var s=Ou(n,o.keyType);if(s&&ig(o.type,s.type))continue}r.push(_(o,t,void 0))}return r}function me(n,r,i){var a=_e(n,111551);if(a)return a;var o=ye(i+"_base"),s=e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(o,void 0,l(r,t))],2));return W(s,0),e.factory.createExpressionWithTypeArguments(e.factory.createIdentifier(o),void 0)}function _e(n,r){var i,a;if(n.target&&io(n.target.symbol,u,r)?(i=e.map(Md(n),function(e){return l(e,t)}),a=k(n.target.symbol,t,788968)):n.symbol&&io(n.symbol,u,r)&&(a=k(n.symbol,t,788968)),a)return e.factory.createExpressionWithTypeArguments(a,i)}function ge(n){var r=_e(n,788968);return r||(n.symbol?e.factory.createExpressionWithTypeArguments(k(n.symbol,t,788968),void 0):void 0)}function ye(e,n){var r,i,a=n?w(n):void 0;if(a&&t.remappedSymbolNames.has(a))return t.remappedSymbolNames.get(a);n&&(e=he(n,e));var o=0,s=e;while(null===(r=t.usedSymbolNames)||void 0===r?void 0:r.has(e))o++,e=s+"_"+o;return null===(i=t.usedSymbolNames)||void 0===i||i.add(e),a&&t.remappedSymbolNames.set(a,e),e}function he(n,r){if("default"===r||"__class"===r||"__function"===r){var i=t.flags;t.flags|=16777216;var a=No(n,t);t.flags=i,r=a.length>0&&e.isSingleOrDoubleQuote(a.charCodeAt(0))?e.stripQuotes(a):a}return"default"===r?r="_default":"export="===r&&(r="_exports"),r=e.isIdentifierText(r,F)&&!e.isStringANonContextualKeyword(r)?r:"_"+r.replace(/[^a-zA-Z0-9]/g,"_"),r}function ve(e,n){var r=w(e);return t.remappedSymbolNames.has(r)?t.remappedSymbolNames.get(r):(n=he(e,n),t.remappedSymbolNames.set(r,n),n)}}}function So(n,t,r,i){return void 0===r&&(r=16384),i?a(i).getText():e.usingSingleLineStringWriter(a);function a(i){var a=e.factory.createTypePredicateNode(2===n.kind||3===n.kind?e.factory.createToken(128):void 0,1===n.kind||3===n.kind?e.factory.createIdentifier(n.parameterName):e.factory.createThisTypeNode(),n.type&&ie.typeToTypeNode(n.type,t,70222336|bo(r))),o=e.createPrinter({removeComments:!0}),s=t&&e.getSourceFileOfNode(t);return o.writeNode(4,a,s,i),i}}function xo(e){for(var n=[],t=0,r=0;r<e.length;r++){var i=e[r];if(t|=i.flags,!(98304&i.flags)){if(1536&i.flags){var a=512&i.flags?Xe:Ec(i);if(1048576&a.flags){var o=a.types.length;if(r+o<=e.length&&Ym(e[r+o-1])===Ym(a.types[o-1])){n.push(a),r+=o-1;continue}}}n.push(i)}}return 65536&t&&n.push(Ue),32768&t&&n.push(Me),n||e}function Lo(e){return 8===e?"private":16===e?"protected":"public"}function Do(n){if(n.symbol&&2048&n.symbol.flags&&n.symbol.declarations){var t=e.walkUpParenthesizedTypes(n.symbol.declarations[0].parent);if(257===t.kind)return Ca(t)}}function Ao(n){return n&&n.parent&&260===n.parent.kind&&e.isExternalModuleAugmentation(n.parent.parent)}function Co(n){return 300===n.kind||e.isAmbientModule(n)}function Io(n,t){var r=Hr(n).nameType;if(r){if(384&r.flags){var i=""+r.value;return e.isIdentifierText(i,M.target)||HT(i)?HT(i)&&e.startsWith(i,"-")?"["+i+"]":i:'"'+e.escapeString(i,34)+'"'}if(8192&r.flags)return"["+No(r.symbol,t)+"]"}}function No(n,t){if(t&&"default"===n.escapedName&&!(16384&t.flags)&&(!(16777216&t.flags)||!n.declarations||t.enclosingDeclaration&&e.findAncestor(n.declarations[0],Co)!==e.findAncestor(t.enclosingDeclaration,Co)))return"default";if(n.declarations&&n.declarations.length){var r=e.firstDefined(n.declarations,function(n){return e.getNameOfDeclaration(n)?n:void 0}),i=r&&e.getNameOfDeclaration(r);if(r&&i){if(e.isCallExpression(r)&&e.isBindableObjectDefinePropertyCall(r))return e.symbolName(n);if(e.isComputedPropertyName(i)&&!(4096&e.getCheckFlags(n))){var a=Hr(n).nameType;if(a&&384&a.flags){var o=Io(n,t);if(void 0!==o)return o}}return e.declarationNameToString(i)}if(r||(r=n.declarations[0]),r.parent&&252===r.parent.kind)return e.declarationNameToString(r.parent.name);switch(r.kind){case 224:case 211:case 212:return!t||t.encounteredError||131072&t.flags||(t.encounteredError=!0),224===r.kind?"(Anonymous class)":"(Anonymous function)"}}var s=Io(n,t);return void 0!==s?s:e.symbolName(n)}function ko(n){if(n){var t=Wr(n);return void 0===t.isVisible&&(t.isVisible=!!r()),t.isVisible}return!1;function r(){switch(n.kind){case 333:case 340:case 334:return!!(n.parent&&n.parent.parent&&n.parent.parent.parent&&e.isSourceFile(n.parent.parent.parent));case 201:return ko(n.parent.parent);case 252:if(e.isBindingPattern(n.name)&&!n.name.elements.length)return!1;case 259:case 255:case 256:case 257:case 254:case 258:case 263:if(e.isExternalModuleAugmentation(n))return!0;var t=Fo(n);return 1&e.getCombinedModifierFlags(n)||263!==n.kind&&300!==t.kind&&8388608&t.flags?ko(t):zr(t);case 165:case 164:case 170:case 171:case 167:case 166:if(e.hasEffectiveModifier(n,24))return!1;case 169:case 173:case 172:case 174:case 162:case 260:case 177:case 178:case 180:case 176:case 181:case 182:case 185:case 186:case 189:case 195:return ko(n.parent);case 265:case 266:case 268:return!1;case 161:case 300:case 262:return!0;case 269:return!1;default:return!1}}}function Ro(n,t){var r,i,a;return n.parent&&269===n.parent.kind?r=Qr(n,n.escapedText,2998271,void 0,n,!1):273===n.parent.kind&&(r=Bi(n.parent,2998271)),r&&(a=new e.Set,a.add(w(r)),o(r.declarations)),i;function o(n){e.forEach(n,function(n){var r=yi(n)||n;if(t?Wr(n).isVisible=!0:(i=i||[],e.pushIfUnique(i,r)),e.isInternalModuleImportEqualsDeclaration(n)){var s=n.moduleReference,c=e.getFirstIdentifier(s),l=Qr(n,c.escapedText,901119,void 0,void 0,!1);l&&a&&e.tryAddToSet(a,w(l))&&o(l.declarations)}})}}function Oo(e,n){var t=wo(e,n);if(t>=0){for(var r=Kt.length,i=t;i<r;i++)Ht[i]=!1;return!1}return Kt.push(e),Ht.push(!0),Wt.push(n),!0}function wo(e,n){for(var t=Kt.length-1;t>=0;t--){if(Po(Kt[t],Wt[t]))return-1;if(Kt[t]===e&&Wt[t]===n)return t}return-1}function Po(n,t){switch(t){case 0:return!!Hr(n).type;case 5:return!!Wr(n).resolvedEnumType;case 2:return!!Hr(n).declaredType;case 1:return!!n.resolvedBaseConstructorType;case 3:return!!n.resolvedReturnType;case 4:return!!n.immediateBaseConstraint;case 6:return!!n.resolvedTypeArguments;case 7:return!!n.baseTypesResolved}return e.Debug.assertNever(t)}function Mo(){return Kt.pop(),Wt.pop(),Ht.pop()}function Fo(n){return e.findAncestor(e.getRootDeclaration(n),function(e){switch(e.kind){case 252:case 253:case 268:case 267:case 266:case 265:return!1;default:return!0}}).parent}function Go(n){var t=Dc(Ia(n));return t.typeParameters?Od(t,e.map(t.typeParameters,function(e){return Ie})):t}function Bo(e,n){var t=Lu(e,n);return t?Ws(t):void 0}function Uo(e,n){var t;return Bo(e,n)||(null===(t=Fu(e,n))||void 0===t?void 0:t.type)||Pe}function Vo(e){return e&&0!==(1&e.flags)}function jo(e){var n=Ca(e);return n&&Hr(n).type||ts(e,!1)}function Ko(n,t,r){if(n=Sb(n,function(e){return!(98304&e.flags)}),131072&n.flags)return pn;if(1048576&n.flags)return Db(n,function(e){return Ko(e,t,r)});var i=ff(e.map(t,Ff));if(sm(n)||cm(i)){if(131072&i.flags)return n;var a=kp();return a?Bd(a,[n,i]):Re}for(var o=e.createSymbolTable(),s=0,c=Hl(n);s<c.length;s++){var l=c[s];lg(Gf(l,8576),i)||24&e.getDeclarationModifierFlagsFromSymbol(l)||!Wm(l)||o.set(l.escapedName,zm(l,!1))}var u=Xa(r,o,e.emptyArray,e.emptyArray,Ru(n));return u.objectFlags|=8388608,u}function Ho(e){return!!(465829888&e.flags)&&iA(ru(e)||Pe,32768)}function Wo(e){var n=bb(e,Ho)?Db(e,function(e){return 465829888&e.flags?iu(e):e}):e;return Jv(n,524288)}function zo(e,n){var t=Jo(e);return t?nE(t,n):n}function Jo(n){var t=qo(n);if(t&&t.flowNode){var r=Xo(n);if(r){var i=e.setTextRange(e.parseNodeFactory.createStringLiteral(r),n),a=e.isLeftHandSideExpression(t)?t:e.parseNodeFactory.createParenthesizedExpression(t),o=e.setTextRange(e.parseNodeFactory.createElementAccessExpression(a,i),n);return e.setParent(i,o),e.setParent(o,n),a!==t&&e.setParent(a,o),o.flowNode=t.flowNode,o}}}function qo(e){var n=e.parent.parent;switch(n.kind){case 201:case 291:return Jo(n);case 202:return Jo(e.parent);case 252:return n.initializer;case 219:return n.right}}function Xo(e){var n=e.parent;return 201===e.kind&&199===n.kind?Yo(e.propertyName||e.name):291===e.kind||292===e.kind?Yo(e.name):""+n.elements.indexOf(e)}function Yo(e){var n=Ff(e);return 384&n.flags?""+n.value:void 0}function Qo(n){var t,r=n.parent,i=jo(r.parent);if(!i||Vo(i))return i;if(z&&8388608&n.flags&&e.isParameterDeclaration(n)?i=lh(i):!z||!r.parent.initializer||65536&zv(sb(r.parent.initializer))||(i=Jv(i,524288)),199===r.kind)if(n.dotDotDotToken){if(i=vu(i),2&i.flags||!QT(i))return Dr(n,e.Diagnostics.Rest_types_may_only_be_created_from_object_types),Re;for(var a=[],o=0,s=r.elements;o<s.length;o++){var c=s[o];c.dotDotDotToken||a.push(c.propertyName||c.name)}t=Ko(i,a,n.symbol)}else{var l=n.propertyName||n.name,u=Ff(l),d=vm(i,u,32,l);t=zo(n,d)}else{var p=fN(65|(n.dotDotDotToken?0:128),i,Me,r),f=r.elements.indexOf(n);if(n.dotDotDotToken)t=Eb(i,Jy)?Db(i,function(e){return Yp(e,f)}):Mp(p);else if(Iy(i)){u=$m(f);var m=32|(BT(n)?16:0);d=Em(i,u,m,n.name)||Re;t=zo(n,d)}else t=p}return n.initializer?e.getEffectiveTypeAnnotationNode(e.walkUpBindingElementsAndPatterns(n))?!z||32768&th(OA(n))?t:Wo(t):PA(n,ff([Wo(t),OA(n)],2)):t}function Zo(n){var t=e.getJSDocType(n);if(t)return u_(t)}function $o(n){var t=e.skipParentheses(n);return 104===t.kind||79===t.kind&&xv(t)===oe}function es(n){var t=e.skipParentheses(n);return 202===t.kind&&0===t.elements.length}function ns(e,n,t){return void 0===n&&(n=!1),void 0===t&&(t=!0),z&&t?sh(e,n):e}function ts(n,t){if(e.isVariableDeclaration(n)&&241===n.parent.parent.kind){var r=Uf(GS(eC(n.parent.parent.expression)));return 4456448&r.flags?Vf(r):je}if(e.isVariableDeclaration(n)&&242===n.parent.parent.kind){var i=n.parent.parent;return pN(i)||Ie}if(e.isBindingPattern(n.parent))return Qo(n);var a=e.isPropertyDeclaration(n)||e.isPropertySignature(n),o=t&&(a&&!!n.questionToken||e.isParameter(n)&&(!!n.questionToken||Uu(n))||Hu(n)),s=Ls(n);if(s)return ns(s,a,o);if((Y||e.isInJSFile(n))&&e.isVariableDeclaration(n)&&!e.isBindingPattern(n.name)&&!(1&e.getCombinedModifierFlags(n))&&!(8388608&n.flags)){if(!(2&e.getCombinedNodeFlags(n))&&(!n.initializer||$o(n.initializer)))return Ne;if(n.initializer&&es(n.initializer))return Bn}if(e.isParameter(n)){var c=n.parent;if(171===c.kind&&Vc(c)){var l=e.getDeclarationOfKind(Ca(n.parent),170);if(l){var u=qu(l),d=_w(c);return d&&n===d?(e.Debug.assert(!d.type),Ws(u.thisParameter)):ad(u)}}if(e.isInJSFile(n)){var p=e.getJSDocType(c);if(p&&e.isFunctionTypeNode(p)){var f=qu(p),m=c.parameters.indexOf(n);return n.dotDotDotToken?mD(f,m):pD(f,m)}}var _="this"===n.symbol.escapedName?GE(c):BE(n);if(_)return ns(_,!1,o)}if(e.hasOnlyExpressionInitializer(n)&&n.initializer){if(e.isInJSFile(n)&&!e.isParameter(n)){var g=ds(n,Ca(n),e.getDeclaredExpandoInitializer(n));if(g)return g}_=PA(n,OA(n));return ns(_,a,o)}if(e.isPropertyDeclaration(n)&&(Y||e.isInJSFile(n))){if(e.hasStaticModifier(n)){var y=e.filter(n.parent.members,e.isClassStaticBlockDeclaration);_=y.length?ss(n.symbol,y):2&e.getEffectiveModifierFlags(n)?uy(n.symbol):void 0;return _&&ns(_,!0,o)}var h=Ga(n.parent),_=h?cs(n.symbol,h):2&e.getEffectiveModifierFlags(n)?uy(n.symbol):void 0;return _&&ns(_,!0,o)}return e.isJsxAttribute(n)?Je:e.isBindingPattern(n.name)?bs(n.name,!1,!0):void 0}function rs(n){if(n.valueDeclaration&&e.isBinaryExpression(n.valueDeclaration)){var t=Hr(n);return void 0===t.isConstructorDeclaredProperty&&(t.isConstructorDeclaredProperty=!1,t.isConstructorDeclaredProperty=!!as(n)&&e.every(n.declarations,function(t){return e.isBinaryExpression(t)&&tT(t)&&(205!==t.left.kind||e.isStringOrNumericLiteralLike(t.left.argumentExpression))&&!ps(void 0,t,n,t)})),t.isConstructorDeclaredProperty}return!1}function is(n){var t=n.valueDeclaration;return t&&e.isPropertyDeclaration(t)&&!e.getEffectiveTypeAnnotationNode(t)&&!t.initializer&&(Y||e.isInJSFile(t))}function as(n){if(n.declarations)for(var t=0,r=n.declarations;t<r.length;t++){var i=r[t],a=e.getThisContainer(i,!1);if(a&&(169===a.kind||FL(a)))return a}}function os(n){var t=e.getSourceFileOfNode(n.declarations[0]),r=e.unescapeLeadingUnderscores(n.escapedName),i=n.declarations.every(function(n){return e.isInJSFile(n)&&e.isAccessExpression(n)&&e.isModuleExportsAccessExpression(n.expression)}),a=i?e.factory.createPropertyAccessExpression(e.factory.createPropertyAccessExpression(e.factory.createIdentifier("module"),e.factory.createIdentifier("exports")),r):e.factory.createPropertyAccessExpression(e.factory.createIdentifier("exports"),r);return i&&e.setParent(a.expression.expression,a.expression),e.setParent(a.expression,a),e.setParent(a,t),a.flowNode=t.endFlowNode,nE(a,Ne,Me)}function ss(n,t){for(var r=e.startsWith(n.escapedName,"__#")?e.factory.createPrivateIdentifier(n.escapedName.split("@")[1]):e.unescapeLeadingUnderscores(n.escapedName),i=0,a=t;i<a.length;i++){var o=a[i],s=e.factory.createPropertyAccessExpression(e.factory.createThis(),r);e.setParent(s.expression,s),e.setParent(s,o),s.flowNode=o.returnFlowNode;var c=ls(s,n);if(!Y||c!==Ne&&c!==Bn||Dr(n.valueDeclaration,e.Diagnostics.Member_0_implicitly_has_an_1_type,mo(n),go(c)),!Eb(c,FS))return zI(c)}}function cs(n,t){var r=e.startsWith(n.escapedName,"__#")?e.factory.createPrivateIdentifier(n.escapedName.split("@")[1]):e.unescapeLeadingUnderscores(n.escapedName),i=e.factory.createPropertyAccessExpression(e.factory.createThis(),r);e.setParent(i.expression,i),e.setParent(i,t),i.flowNode=t.returnFlowNode;var a=ls(i,n);return!Y||a!==Ne&&a!==Bn||Dr(n.valueDeclaration,e.Diagnostics.Member_0_implicitly_has_an_1_type,mo(n),go(a)),Eb(a,FS)?void 0:zI(a)}function ls(n,t){var r=(null===t||void 0===t?void 0:t.valueDeclaration)&&(!is(t)||2&e.getEffectiveModifierFlags(t.valueDeclaration))&&uy(t)||Me;return nE(n,Ne,r)}function us(n,t){var r,i=e.getAssignedExpandoInitializer(n.valueDeclaration);if(i){var a=e.getJSDocTypeTag(i);if(a&&a.typeExpression)return u_(a.typeExpression);var o=n.valueDeclaration&&ds(n.valueDeclaration,n,i);return o||jy(kA(i))}var s=!1,c=!1;if(rs(n)&&(r=cs(n,as(n))),!r){var l=void 0;if(n.declarations){for(var u=void 0,d=0,p=n.declarations;d<p.length;d++){var f=p[d],m=e.isBinaryExpression(f)||e.isCallExpression(f)?f:e.isAccessExpression(f)?e.isBinaryExpression(f.parent)?f.parent:f:void 0;if(m){var _=e.isAccessExpression(m)?e.getAssignmentDeclarationPropertyAccessKind(m):e.getAssignmentDeclarationKind(m);(4===_||e.isBinaryExpression(m)&&tT(m,_))&&(_s(m)?s=!0:c=!0),e.isCallExpression(m)||(u=ps(u,m,n,f)),u||(l||(l=[])).push(e.isBinaryExpression(m)||e.isCallExpression(m)?fs(n,t,m,_):Ze)}}r=u}if(!r){if(!e.length(l))return Re;var g=s&&n.declarations?gs(l,n.declarations):void 0;if(c){var y=uy(n);y&&((g||(g=[])).push(y),s=!0)}var h=e.some(g,function(e){return!!(-98305&e.flags)})?g:l;r=ff(h,2)}}var v=Ch(ns(r,!1,c&&!s));return n.valueDeclaration&&Sb(v,function(e){return!!(-98305&e.flags)})===Ze?(kh(n.valueDeclaration,Ie),Ie):v}function ds(n,t,r){var i,a;if(e.isInJSFile(n)&&r&&e.isObjectLiteralExpression(r)&&!r.properties.length){var o=e.createSymbolTable();while(e.isBinaryExpression(n)||e.isPropertyAccessExpression(n)){var s=Ca(n);(null===(i=null===s||void 0===s?void 0:s.exports)||void 0===i?void 0:i.size)&&Vr(o,s.exports),n=e.isBinaryExpression(n)?n.parent:n.parent.parent}var c=Ca(n);(null===(a=null===c||void 0===c?void 0:c.exports)||void 0===a?void 0:a.size)&&Vr(o,c.exports);var l=Xa(t,o,e.emptyArray,e.emptyArray,e.emptyArray);return l.objectFlags|=8192,l}}function ps(n,t,r,i){var a,o=e.getEffectiveTypeAnnotationNode(t.parent);if(o){var s=Ch(u_(o));if(!n)return s;n===Re||s===Re||ig(n,s)||qI(void 0,n,i,s)}if(null===(a=r.parent)||void 0===a?void 0:a.valueDeclaration){var c=e.getEffectiveTypeAnnotationNode(r.parent.valueDeclaration);if(c){var l=Lu(u_(c),r.escapedName);if(l)return zs(l)}}return n}function fs(n,t,r,i){if(e.isCallExpression(r)){if(t)return Ws(t);var a=kA(r.arguments[2]),o=Bo(a,"value");if(o)return o;var s=Bo(a,"get");if(s){var c=Vx(s);if(c)return ad(c)}var l=Bo(a,"set");if(l){var u=Vx(l);if(u)return bD(u)}return Ie}if(ms(r.left,r.right))return Ie;var d=t?Ws(t):jy(kA(r.right));if(524288&d.flags&&2===i&&"export="===n.escapedName){var p=Ul(d),f=e.createSymbolTable();e.copyEntries(p.members,f);var m=f.size;t&&!t.exports&&(t.exports=e.createSymbolTable()),(t||n).exports.forEach(function(n,t){var r,i=f.get(t);if(i&&i!==n)if(111551&n.flags&&111551&i.flags){if(n.valueDeclaration&&i.valueDeclaration&&e.getSourceFileOfNode(n.valueDeclaration)!==e.getSourceFileOfNode(i.valueDeclaration)){var a=e.unescapeLeadingUnderscores(n.escapedName),o=(null===(r=e.tryCast(i.valueDeclaration,e.isNamedDeclaration))||void 0===r?void 0:r.name)||i.valueDeclaration;e.addRelatedInfo(Dr(n.valueDeclaration,e.Diagnostics.Duplicate_identifier_0,a),e.createDiagnosticForNode(o,e.Diagnostics._0_was_also_declared_here,a)),e.addRelatedInfo(Dr(o,e.Diagnostics.Duplicate_identifier_0,a),e.createDiagnosticForNode(n.valueDeclaration,e.Diagnostics._0_was_also_declared_here,a))}var s=Or(n.flags|i.flags,t);s.type=ff([Ws(n),Ws(i)]),s.valueDeclaration=i.valueDeclaration,s.declarations=e.concatenate(i.declarations,n.declarations),f.set(t,s)}else f.set(t,Fr(n,i));else f.set(t,n)});var _=Xa(m!==f.size?void 0:p.symbol,f,p.callSignatures,p.constructSignatures,p.indexInfos);return _.objectFlags|=8192&e.getObjectFlags(d),_.symbol&&32&_.symbol.flags&&d===gc(_.symbol)&&(_.objectFlags|=16777216),_}return Ry(d)?(kh(r,Gn),Gn):d}function ms(n,t){return e.isPropertyAccessExpression(n)&&108===n.expression.kind&&e.forEachChildRecursively(t,function(e){return Av(n,e)})}function _s(n){var t=e.getThisContainer(n,!1);return 169===t.kind||254===t.kind||211===t.kind&&!e.isPrototypePropertyAssignment(t.parent)}function gs(n,t){return e.Debug.assert(n.length===t.length),n.filter(function(n,r){var i=t[r],a=e.isBinaryExpression(i)?i:e.isBinaryExpression(i.parent)?i.parent:void 0;return a&&_s(a)})}function ys(n,t,r){if(n.initializer){var i=e.isBindingPattern(n.name)?bs(n.name,!0,!1):Pe;return ns(PA(n,OA(n,i)))}return e.isBindingPattern(n.name)?bs(n.name,t,r):(r&&!xs(n)&&kh(n,Ie),t?Oe:Ie)}function hs(n,t,r){var i,a=e.createSymbolTable(),o=262272;e.forEach(n.elements,function(e){var n=e.propertyName||e.name;if(e.dotDotDotToken)i=Td(je,Ie,!1);else{var s=Ff(n);if(Fc(s)){var c=Kc(s),l=4|(e.initializer?16777216:0),u=Or(l,c);u.type=ys(e,t,r),u.bindingElement=e,a.set(u.escapedName,u)}else o|=512}});var s=Xa(void 0,a,e.emptyArray,e.emptyArray,i?[i]:e.emptyArray);return s.objectFlags|=o,t&&(s.pattern=n,s.objectFlags|=262144),s}function vs(n,t,r){var i=n.elements,a=e.lastOrUndefined(i),o=a&&201===a.kind&&a.dotDotDotToken?a:void 0;if(0===i.length||1===i.length&&o)return F>=2?Pp(Ie):Gn;var s=e.map(i,function(n){return e.isOmittedExpression(n)?Ie:ys(n,t,r)}),c=e.findLastIndex(i,function(n){return!(n===o||e.isOmittedExpression(n)||BT(n))},i.length-1)+1,l=e.map(i,function(e,n){return e===o?4:n>=c?2:1}),u=Wp(s,l);return t&&(u=wd(u),u.pattern=n,u.objectFlags|=262144),u}function bs(e,n,t){return void 0===n&&(n=!1),void 0===t&&(t=!1),199===e.kind?hs(e,n,t):vs(e,n,t)}function Es(e,n){return Ss(ts(e,!0),e,n)}function Ts(e){var n=Ca(e),t=fp(!1);return t&&n&&n===t}function Ss(n,t,r){return n?(4096&n.flags&&Ts(t.parent)&&(n=i_(t)),r&&Rh(t,n),8192&n.flags&&(e.isBindingElement(t)||!t.type)&&n.symbol!==Ca(t)&&(n=Ye),Ch(n)):(n=e.isParameter(t)&&t.dotDotDotToken?Gn:Ie,r&&(xs(t)||kh(t,n)),n)}function xs(n){var t=e.getRootDeclaration(n),r=162===t.kind?t.parent:t;return jC(r)}function Ls(n){var t=e.getEffectiveTypeAnnotationNode(n);if(t)return u_(t)}function Ds(e){var n=Hr(e);if(!n.type){var t=As(e);n.type||(n.type=t)}return n.type}function As(n){if(4194304&n.flags)return Go(n);if(n===ue)return Ie;if(134217728&n.flags&&n.valueDeclaration){var t=Ca(e.getSourceFileOfNode(n.valueDeclaration)),r=Or(t.flags,"exports");r.declarations=t.declarations?t.declarations.slice():[],r.parent=n,r.target=t,t.valueDeclaration&&(r.valueDeclaration=t.valueDeclaration),t.members&&(r.members=new e.Map(t.members)),t.exports&&(r.exports=new e.Map(t.exports));var i=e.createSymbolTable();return i.set("exports",r),Xa(n,i,e.emptyArray,e.emptyArray,e.emptyArray)}e.Debug.assertIsDefined(n.valueDeclaration);var a,o=n.valueDeclaration;if(e.isCatchClauseVariableDeclarationOrBindingElement(o)){var s=e.getEffectiveTypeAnnotationNode(o);if(void 0===s)return Z?Pe:Ie;var c=RR(s);return Vo(c)||c===Pe?c:Re}if(e.isSourceFile(o)&&e.isJsonSourceFile(o))return o.statements.length?Ch(jy(eC(o.statements[0].expression))):pn;if(!Oo(n,0))return 512&n.flags&&!(67108864&n.flags)?Fs(n):js(n);if(269===o.kind)a=Ss(kA(o.expression),o);else if(e.isBinaryExpression(o)||e.isInJSFile(o)&&(e.isCallExpression(o)||(e.isPropertyAccessExpression(o)||e.isBindableStaticElementAccessExpression(o))&&e.isBinaryExpression(o.parent)))a=us(n);else if(e.isPropertyAccessExpression(o)||e.isElementAccessExpression(o)||e.isIdentifier(o)||e.isStringLiteralLike(o)||e.isNumericLiteral(o)||e.isClassDeclaration(o)||e.isFunctionDeclaration(o)||e.isMethodDeclaration(o)&&!e.isObjectLiteralMethod(o)||e.isMethodSignature(o)||e.isSourceFile(o)){if(9136&n.flags)return Fs(n);a=e.isBinaryExpression(o.parent)?us(n):Ls(o)||Ie}else if(e.isPropertyAssignment(o))a=Ls(o)||BA(o);else if(e.isJsxAttribute(o))a=Ls(o)||aS(o);else if(e.isShorthandPropertyAssignment(o))a=Ls(o)||GA(o.name,0);else if(e.isObjectLiteralMethod(o))a=Ls(o)||UA(o,0);else if(e.isParameter(o)||e.isPropertyDeclaration(o)||e.isPropertySignature(o)||e.isVariableDeclaration(o)||e.isBindingElement(o)||e.isJSDocPropertyLikeTag(o))a=Es(o,!0);else if(e.isEnumDeclaration(o))a=Fs(n);else if(e.isEnumMember(o))a=Bs(n);else{if(!e.isAccessor(o))return e.Debug.fail("Unhandled declaration kind! "+e.Debug.formatSyntaxKind(o.kind)+" for "+e.Debug.formatSymbol(n));a=Ps(n)||e.Debug.fail("Non-write accessor resolution must always produce a type")}return Mo()?a:512&n.flags&&!(67108864&n.flags)?Fs(n):js(n)}function Cs(n){if(n){if(170===n.kind){var t=e.getEffectiveReturnTypeNode(n);return t}var r=e.getEffectiveSetAccessorTypeAnnotationNode(n);return r}}function Is(e){var n=Cs(e);return n&&u_(n)}function Ns(e){var n=_w(e);return n&&n.symbol}function ks(e){return nd(qu(e))}function Rs(n){var t=Hr(n);return t.type||(t.type=ws(n)||e.Debug.fail("Read type of accessor must always produce a type"))}function Os(e){var n=Hr(e);return n.writeType||(n.writeType=ws(e,!0))}function ws(n,t){if(void 0===t&&(t=!1),!Oo(n,0))return Re;var r=Ps(n,t);if(!Mo()&&(r=Ie,Y)){var i=e.getDeclarationOfKind(n,170);Dr(i,e.Diagnostics._0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions,mo(n))}return r}function Ps(n,t){void 0===t&&(t=!1);var r=e.getDeclarationOfKind(n,170),i=e.getDeclarationOfKind(n,171),a=Is(i);if(t&&a)return l(a,n);if(r&&e.isInJSFile(r)){var o=Zo(r);if(o)return l(o,n)}var s=Is(r);if(s)return l(s,n);if(a)return a;if(r&&r.body){var c=kD(r);return l(c,n)}return i?(jC(i)||Cr(Y,i,e.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation,mo(n)),Ie):r?(e.Debug.assert(!!r,"there must exist a getter as we are current checking either setter or getter in this function"),jC(r)||Cr(Y,r,e.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation,mo(n)),Ie):void 0;function l(n,t){if(1&e.getCheckFlags(t)){var r=Hr(t);return z_(n,r.mapper)}return n}}function Ms(n){var t=oc(gc(n));return 8650752&t.flags?t:2097152&t.flags?e.find(t.types,function(e){return!!(8650752&e.flags)}):void 0}function Fs(e){var n=Hr(e),t=n;if(!n.type){var r=e.valueDeclaration&&UL(e.valueDeclaration,!1);if(r){var i=GL(e,r);i&&(e=n=i)}t.type=n.type=Gs(e)}return n.type}function Gs(n){var t=n.valueDeclaration;if(1536&n.flags&&e.isShorthandAmbientModuleSymbol(n))return Ie;if(t&&(219===t.kind||e.isAccessExpression(t)&&219===t.parent.kind))return us(n);if(512&n.flags&&t&&e.isSourceFile(t)&&t.commonJsModuleIndicator){var r=fa(n);if(r!==n){if(!Oo(n,0))return Re;var i=Aa(n.exports.get("export=")),a=us(i,i===r?void 0:r);return Mo()?a:js(n)}}var o=ja(16,n);if(32&n.flags){var s=Ms(n);return s?Df([o,s]):o}return z&&16777216&n.flags?sh(o):o}function Bs(e){var n=Hr(e);return n.type||(n.type=Sc(e))}function Us(e){var n=Hr(e);if(!n.type){var t=Ji(e),r=e.declarations&&Hi(hi(e),!0);n.type=(null===r||void 0===r?void 0:r.declarations)&&Qk(r.declarations)&&e.declarations.length?os(r):Qk(e.declarations)?Ne:111551&t.flags?Ws(t):Re}return n.type}function Vs(e){var n=Hr(e);if(!n.type){if(!Oo(e,0))return n.type=Re;var t=z_(Ws(n.target),n.mapper);Mo()||(t=js(e)),n.type=t}return n.type}function js(n){var t=n.valueDeclaration;return e.getEffectiveTypeAnnotationNode(t)?(Dr(n.valueDeclaration,e.Diagnostics._0_is_referenced_directly_or_indirectly_in_its_own_type_annotation,mo(n)),Re):(Y&&(162!==t.kind||t.initializer)&&Dr(n.valueDeclaration,e.Diagnostics._0_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_and_is_referenced_directly_or_indirectly_in_its_own_initializer,mo(n)),Ie)}function Ks(n){var t=Hr(n);return t.type||(e.Debug.assertIsDefined(t.deferralParent),e.Debug.assertIsDefined(t.deferralConstituents),t.type=1048576&t.deferralParent.flags?ff(t.deferralConstituents):Df(t.deferralConstituents)),t.type}function Hs(e){if(98304&e.flags){var n=Os(e);if(n)return n}return Ws(e)}function Ws(n){var t=e.getCheckFlags(n);return 65536&t?Ks(n):1&t?Vs(n):262144&t?Al(n):8192&t?Qh(n):7&n.flags?Ds(n):9136&n.flags?Fs(n):8&n.flags?Bs(n):98304&n.flags?Rs(n):2097152&n.flags?Us(n):Re}function zs(e){return mh(Ws(e),!!(16777216&e.flags))}function Js(n,t){return void 0!==n&&void 0!==t&&0!==(4&e.getObjectFlags(n))&&n.target===t}function qs(n){return 4&e.getObjectFlags(n)?n.target:n}function Xs(n,t){return r(n);function r(n){if(7&e.getObjectFlags(n)){var i=qs(n);return i===t||e.some(lc(i),r)}return!!(2097152&n.flags)&&e.some(n.types,r)}}function Ys(n,t){for(var r=0,i=t;r<i.length;r++){var a=i[r];n=e.appendIfUnique(n,xc(Ca(a)))}return n}function Qs(n,t){while(1){if(n=n.parent,n&&e.isBinaryExpression(n)){var r=e.getAssignmentDeclarationKind(n);if(6===r||3===r){var i=Ca(n.left);i&&i.parent&&!e.findAncestor(i.parent.valueDeclaration,function(e){return n===e})&&(n=i.parent.valueDeclaration)}}if(!n)return;switch(n.kind){case 255:case 224:case 256:case 172:case 173:case 166:case 177:case 178:case 312:case 254:case 167:case 211:case 212:case 257:case 339:case 340:case 334:case 333:case 193:case 187:var a=Qs(n,t);if(193===n.kind)return e.append(a,xc(Ca(n.typeParameter)));if(187===n.kind)return e.concatenate(a,Rm(n));var o=Ys(a,e.getEffectiveTypeParameterDeclarations(n)),s=t&&(255===n.kind||224===n.kind||256===n.kind||FL(n))&&gc(Ca(n)).thisType;return s?e.append(o,s):o;case 335:var c=e.getParameterSymbolFromJSDoc(n);c&&(n=c.valueDeclaration);break;case 315:a=Qs(n,t);return n.tags?Ys(a,e.flatMap(n.tags,function(n){return e.isJSDocTemplateTag(n)?n.typeParameters:void 0})):a}}}function Zs(n){var t=32&n.flags?n.valueDeclaration:e.getDeclarationOfKind(n,256);return e.Debug.assert(!!t,"Class was missing valueDeclaration -OR- non-class had no interface declarations"),Qs(t)}function $s(n){if(n.declarations){for(var t,r=0,i=n.declarations;r<i.length;r++){var a=i[r];if(256===a.kind||255===a.kind||224===a.kind||FL(a)||e.isTypeAlias(a)){var o=a;t=Ys(t,e.getEffectiveTypeParameterDeclarations(o))}}return t}}function ec(n){return e.concatenate(Zs(n),$s(n))}function nc(e){var n=Au(e,1);if(1===n.length){var t=n[0];if(!t.typeParameters&&1===t.parameters.length&&V(t)){var r=aD(t.parameters[0]);return Vo(r)||Cy(r)===Ie}}return!1}function tc(e){if(Au(e,1).length>0)return!0;if(8650752&e.flags){var n=ru(e);return!!n&&nc(n)}return!1}function rc(n){return e.getEffectiveBaseTypeNode(n.symbol.valueDeclaration)}function ic(n,t,r){var i=e.length(t),a=e.isInJSFile(r);return e.filter(Au(n,1),function(n){return(a||i>=zu(n.typeParameters))&&i<=e.length(n.typeParameters)})}function ac(n,t,r){var i=ic(n,t,r),a=e.map(t,u_);return e.sameMap(i,function(n){return e.some(n.typeParameters)?ud(n,a,e.isInJSFile(r)):n})}function oc(n){if(!n.resolvedBaseConstructorType){var t=n.symbol.valueDeclaration,r=e.getEffectiveBaseTypeNode(t),i=rc(n);if(!i)return n.resolvedBaseConstructorType=Me;if(!Oo(n,1))return Re;var a=eC(i.expression);if(r&&i!==r&&(e.Debug.assert(!r.typeArguments),eC(r.expression)),2621440&a.flags&&Ul(a),!Mo())return Dr(n.symbol.valueDeclaration,e.Diagnostics._0_is_referenced_directly_or_indirectly_in_its_own_base_expression,mo(n.symbol)),n.resolvedBaseConstructorType=Re;if(!(1&a.flags)&&a!==Ve&&!tc(a)){var o=Dr(i.expression,e.Diagnostics.Type_0_is_not_a_constructor_function_type,go(a));if(262144&a.flags){var s=Cd(a),c=Pe;if(s){var l=Au(s,1);l[0]&&(c=ad(l[0]))}a.symbol.declarations&&e.addRelatedInfo(o,e.createDiagnosticForNode(a.symbol.declarations[0],e.Diagnostics.Did_you_mean_for_0_to_be_constrained_to_type_new_args_Colon_any_1,mo(a.symbol),go(c)))}return n.resolvedBaseConstructorType=Re}n.resolvedBaseConstructorType=a}return n.resolvedBaseConstructorType}function sc(n){var t=e.emptyArray;if(n.symbol.declarations)for(var r=0,i=n.symbol.declarations;r<i.length;r++){var a=i[r],o=e.getEffectiveImplementsTypeNodes(a);if(o)for(var s=0,c=o;s<c.length;s++){var l=c[s],u=u_(l);u!==Re&&(t===e.emptyArray?t=[u]:t.push(u))}}return t}function cc(n,t){Dr(n,e.Diagnostics.Type_0_recursively_references_itself_as_a_base_type,go(t,void 0,2))}function lc(n){if(!n.baseTypesResolved){if(Oo(n,7)&&(8&n.objectFlags?n.resolvedBaseTypes=[uc(n)]:96&n.symbol.flags?(32&n.symbol.flags&&dc(n),64&n.symbol.flags&&mc(n)):e.Debug.fail("type must be class or interface"),!Mo()&&n.symbol.declarations))for(var t=0,r=n.symbol.declarations;t<r.length;t++){var i=r[t];255!==i.kind&&256!==i.kind||cc(i,n)}n.baseTypesResolved=!0}return n.resolvedBaseTypes}function uc(n){var t=e.sameMap(n.typeParameters,function(e,t){return 8&n.elementFlags[t]?vm(e,Ke):e});return Mp(ff(t||e.emptyArray),n.readonly)}function dc(n){n.resolvedBaseTypes=e.resolvingEmptyArray;var t=mu(oc(n));if(!(2621441&t.flags))return n.resolvedBaseTypes=e.emptyArray;var r,i=rc(n),a=t.symbol?Dc(t.symbol):void 0;if(t.symbol&&32&t.symbol.flags&&pc(a))r=Gd(i,t.symbol);else if(1&t.flags)r=t;else{var o=ac(t,i.typeArguments,i);if(!o.length)return Dr(i.expression,e.Diagnostics.No_base_constructor_has_the_specified_number_of_type_arguments),n.resolvedBaseTypes=e.emptyArray;r=ad(o[0])}if(r===Re)return n.resolvedBaseTypes=e.emptyArray;var s=vu(r);if(!fc(s)){var c=xu(void 0,r),l=e.chainDiagnosticMessages(c,e.Diagnostics.Base_constructor_return_type_0_is_not_an_object_type_or_intersection_of_object_types_with_statically_known_members,go(s));return ur.add(e.createDiagnosticForNodeFromMessageChain(i.expression,l)),n.resolvedBaseTypes=e.emptyArray}return n===s||Xs(s,n)?(Dr(n.symbol.valueDeclaration,e.Diagnostics.Type_0_recursively_references_itself_as_a_base_type,go(n,void 0,2)),n.resolvedBaseTypes=e.emptyArray):(n.resolvedBaseTypes===e.resolvingEmptyArray&&(n.members=void 0),n.resolvedBaseTypes=[s])}function pc(e){var n=e.outerTypeParameters;if(n){var t=n.length-1,r=Md(e);return n[t].symbol!==r[t].symbol}return!0}function fc(n){if(262144&n.flags){var t=ru(n);if(t)return fc(t)}return!!(67633153&n.flags&&!Bl(n)||2097152&n.flags&&e.every(n.types,fc))}function mc(n){if(n.resolvedBaseTypes=n.resolvedBaseTypes||e.emptyArray,n.symbol.declarations)for(var t=0,r=n.symbol.declarations;t<r.length;t++){var i=r[t];if(256===i.kind&&e.getInterfaceBaseTypeNodes(i))for(var a=0,o=e.getInterfaceBaseTypeNodes(i);a<o.length;a++){var s=o[a],c=vu(u_(s));c!==Re&&(fc(c)?n===c||Xs(c,n)?cc(i,n):n.resolvedBaseTypes===e.emptyArray?n.resolvedBaseTypes=[c]:n.resolvedBaseTypes.push(c):Dr(s,e.Diagnostics.An_interface_can_only_extend_an_object_type_or_intersection_of_object_types_with_statically_known_members))}}}function _c(n){if(!n.declarations)return!0;for(var t=0,r=n.declarations;t<r.length;t++){var i=r[t];if(256===i.kind){if(128&i.flags)return!1;var a=e.getInterfaceBaseTypeNodes(i);if(a)for(var o=0,s=a;o<s.length;o++){var c=s[o];if(e.isEntityNameExpression(c.expression)){var l=ra(c.expression,788968,!0);if(!l||!(64&l.flags)||gc(l).thisType)return!1}}}}return!0}function gc(n){var t=Hr(n),r=t;if(!t.declaredType){var i=32&n.flags?1:2,a=GL(n,n.valueDeclaration&&BL(n.valueDeclaration));a&&(n=t=a);var o=r.declaredType=t.declaredType=ja(i,n),s=Zs(n),c=$s(n);(s||c||1===i||!_c(n))&&(o.objectFlags|=4,o.typeParameters=e.concatenate(s,c),o.outerTypeParameters=s,o.localTypeParameters=c,o.instantiations=new e.Map,o.instantiations.set(Nd(o.typeParameters),o),o.target=o,o.resolvedTypeArguments=o.typeParameters,o.thisType=Ha(n),o.thisType.isThisType=!0,o.thisType.constraint=o)}return t.declaredType}function yc(n){var t,r=Hr(n);if(!r.declaredType){if(!Oo(n,2))return Re;var i=e.Debug.checkDefined(null===(t=n.declarations)||void 0===t?void 0:t.find(e.isTypeAlias),"Type alias symbol with no valid declaration found"),a=e.isJSDocTypeAlias(i)?i.typeExpression:i.type,o=a?u_(a):Re;if(Mo()){var s=$s(n);s&&(r.typeParameters=s,r.instantiations=new e.Map,r.instantiations.set(Nd(s),o))}else o=Re,334===i.kind?Dr(i.typeExpression.type,e.Diagnostics.Type_alias_0_circularly_references_itself,mo(n)):Dr(e.isNamedDeclaration(i)?i.name:i||i,e.Diagnostics.Type_alias_0_circularly_references_itself,mo(n));r.declaredType=o}return r.declaredType}function hc(n){return!!e.isStringLiteralLike(n)||219===n.kind&&(hc(n.left)&&hc(n.right))}function vc(n){var t=n.initializer;if(!t)return!(8388608&n.flags);switch(t.kind){case 10:case 8:case 14:return!0;case 217:return 40===t.operator&&8===t.operand.kind;case 79:return e.nodeIsMissing(t)||!!Ca(n.parent).exports.get(t.escapedText);case 219:return hc(t);default:return!1}}function bc(n){var t=Hr(n);if(void 0!==t.enumKind)return t.enumKind;var r=!1;if(n.declarations)for(var i=0,a=n.declarations;i<a.length;i++){var o=a[i];if(258===o.kind)for(var s=0,c=o.members;s<c.length;s++){var l=c[s];if(l.initializer&&e.isStringLiteralLike(l.initializer))return t.enumKind=1;vc(l)||(r=!0)}}return t.enumKind=r?0:1}function Ec(e){return 1024&e.flags&&!(1048576&e.flags)?Dc(Ia(e.symbol)):e}function Tc(e){var n=Hr(e);if(n.declaredType)return n.declaredType;if(1===bc(e)){b++;var t=[];if(e.declarations)for(var r=0,i=e.declarations;r<i.length;r++){var a=i[r];if(258===a.kind)for(var o=0,s=a.members;o<s.length;o++){var c=s[o],l=uO(c),u=Xm(n_(void 0!==l?l:0,b,Ca(c)));Hr(Ca(c)).declaredType=u,t.push(Ym(u))}}if(t.length){var d=ff(t,1,e,void 0);return 1048576&d.flags&&(d.flags|=1024,d.symbol=e),n.declaredType=d}}var p=Ba(32);return p.symbol=e,n.declaredType=p}function Sc(e){var n=Hr(e);if(!n.declaredType){var t=Tc(Ia(e));n.declaredType||(n.declaredType=t)}return n.declaredType}function xc(e){var n=Hr(e);return n.declaredType||(n.declaredType=Ha(e))}function Lc(e){var n=Hr(e);return n.declaredType||(n.declaredType=Dc(Ji(e)))}function Dc(e){return Ac(e)||Re}function Ac(e){return 96&e.flags?gc(e):524288&e.flags?yc(e):262144&e.flags?xc(e):384&e.flags?Tc(e):8&e.flags?Sc(e):2097152&e.flags?Lc(e):void 0}function Cc(e){switch(e.kind){case 129:case 153:case 148:case 145:case 156:case 132:case 149:case 146:case 114:case 151:case 142:case 194:return!0;case 181:return Cc(e.elementType);case 176:return!e.typeArguments||e.typeArguments.every(Cc)}return!1}function Ic(n){var t=e.getEffectiveConstraintOfTypeParameter(n);return!t||Cc(t)}function Nc(n){var t=e.getEffectiveTypeAnnotationNode(n);return t?Cc(t):!e.hasInitializer(n)}function kc(n){var t=e.getEffectiveReturnTypeNode(n),r=e.getEffectiveTypeParameterDeclarations(n);return(169===n.kind||!!t&&Cc(t))&&n.parameters.every(Nc)&&r.every(Ic)}function Rc(e){if(e.declarations&&1===e.declarations.length){var n=e.declarations[0];if(n)switch(n.kind){case 165:case 164:return Nc(n);case 167:case 166:case 169:case 170:case 171:return kc(n)}}return!1}function Oc(n,t,r){for(var i=e.createSymbolTable(),a=0,o=n;a<o.length;a++){var s=o[a];i.set(s.escapedName,r&&Rc(s)?s:R_(s,t))}return i}function wc(e,n){for(var t=0,r=n;t<r.length;t++){var i=r[t];e.has(i.escapedName)||Pc(i)||e.set(i.escapedName,i)}}function Pc(n){return!!n.valueDeclaration&&e.isPrivateIdentifierClassElementDeclaration(n.valueDeclaration)&&e.isStatic(n.valueDeclaration)}function Mc(n){if(!n.declaredProperties){var t=n.symbol,r=Jc(t);n.declaredProperties=za(r),n.declaredCallSignatures=e.emptyArray,n.declaredConstructSignatures=e.emptyArray,n.declaredIndexInfos=e.emptyArray,n.declaredCallSignatures=$u(r.get("__call")),n.declaredConstructSignatures=$u(r.get("__new")),n.declaredIndexInfos=Sd(t)}return n}function Fc(e){return!!(8576&e.flags)}function Gc(n){if(!e.isComputedPropertyName(n)&&!e.isElementAccessExpression(n))return!1;var t=e.isComputedPropertyName(n)?n.expression:n.argumentExpression;return e.isEntityNameExpression(t)&&Fc(e.isComputedPropertyName(n)?WT(n):kA(t))}function Bc(e){return 95===e.charCodeAt(0)&&95===e.charCodeAt(1)&&64===e.charCodeAt(2)}function Uc(n){var t=e.getNameOfDeclaration(n);return!!t&&Gc(t)}function Vc(n){return!e.hasDynamicName(n)||Uc(n)}function jc(n){return e.isDynamicName(n)&&!Gc(n)}function Kc(n){return 8192&n.flags?n.escapedName:384&n.flags?e.escapeLeadingUnderscores(""+n.value):e.Debug.fail()}function Hc(n,t,r){e.Debug.assert(!!(4096&e.getCheckFlags(n)),"Expected a late-bound symbol."),n.flags|=r,Hr(t.symbol).lateSymbol=n,n.declarations?t.symbol.isReplaceableByMethod||n.declarations.push(t):n.declarations=[t],111551&r&&(n.valueDeclaration&&n.valueDeclaration.kind===t.kind||(n.valueDeclaration=t))}function Wc(n,t,r,i){e.Debug.assert(!!i.symbol,"The member is expected to have a symbol.");var a=Wr(i);if(!a.resolvedSymbol){a.resolvedSymbol=i.symbol;var o=e.isBinaryExpression(i)?i.left:i.name,s=e.isElementAccessExpression(o)?kA(o.argumentExpression):WT(o);if(Fc(s)){var c=Kc(s),l=i.symbol.flags,u=r.get(c);u||r.set(c,u=Or(0,c,4096));var d=t&&t.get(c);if(u.flags&wr(l)||d){var p=d?e.concatenate(d.declarations,u.declarations):u.declarations,f=!(8192&s.flags)&&e.unescapeLeadingUnderscores(c)||e.declarationNameToString(o);e.forEach(p,function(n){return Dr(e.getNameOfDeclaration(n)||n,e.Diagnostics.Property_0_was_also_declared_here,f)}),Dr(o||i,e.Diagnostics.Duplicate_property_0,f),u=Or(0,c,4096)}return u.nameType=s,Hc(u,i,l),u.parent?e.Debug.assert(u.parent===n,"Existing symbol parent should match new one"):u.parent=n,a.resolvedSymbol=u}}return a.resolvedSymbol}function zc(n,t){var r=Hr(n);if(!r[t]){var i="resolvedExports"===t,a=i?1536&n.flags?Da(n):n.exports:n.members;r[t]=a||D;for(var o=e.createSymbolTable(),s=0,c=n.declarations||e.emptyArray;s<c.length;s++){var l=c[s],u=e.getMembersOfDeclaration(l);if(u)for(var d=0,p=u;d<p.length;d++){var f=p[d];i===e.hasStaticModifier(f)&&Uc(f)&&Wc(n,a,o,f)}}var m=n.assignmentDeclarationMembers;if(m)for(var _=e.arrayFrom(m.values()),g=0,y=_;g<y.length;g++){f=y[g];var h=e.getAssignmentDeclarationKind(f),v=3===h||e.isBinaryExpression(f)&&tT(f,h)||9===h||6===h;i===!v&&Uc(f)&&Wc(n,a,o,f)}r[t]=Ur(a,o)||D}return r[t]}function Jc(e){return 6256&e.flags?zc(e,"resolvedMembers"):e.members||D}function qc(n){if(106500&n.flags&&"__computed"===n.escapedName){var t=Hr(n);if(!t.lateSymbol&&e.some(n.declarations,Uc)){var r=Aa(n.parent);e.some(n.declarations,e.hasStaticModifier)?Sa(r):Jc(r)}return t.lateSymbol||(t.lateSymbol=n)}return n}function Xc(n,t,r){if(4&e.getObjectFlags(n)){var i=n.target,a=Md(n);if(e.length(i.typeParameters)===e.length(a)){var o=Od(i,e.concatenate(a,[t||i.thisType]));return r?mu(o):o}}else if(2097152&n.flags){var s=e.sameMap(n.types,function(e){return Xc(e,t,r)});return s!==n.types?Df(s):n}return r?mu(n):n}function Yc(n,t,r,i){var a,o,s,c,l;e.rangeEquals(r,i,0,r.length)?(o=t.symbol?Jc(t.symbol):e.createSymbolTable(t.declaredProperties),s=t.declaredCallSignatures,c=t.declaredConstructSignatures,l=t.declaredIndexInfos):(a=g_(r,i),o=Oc(t.declaredProperties,a,1===r.length),s=m_(t.declaredCallSignatures,a),c=m_(t.declaredConstructSignatures,a),l=__(t.declaredIndexInfos,a));var u=lc(t);if(u.length){t.symbol&&o===Jc(t.symbol)&&(o=e.createSymbolTable(t.declaredProperties)),qa(n,o,s,c,l);for(var d=e.lastOrUndefined(i),p=0,f=u;p<f.length;p++){var m=f[p],_=d?Xc(z_(m,a),d):m;wc(o,Hl(_)),s=e.concatenate(s,Au(_,0)),c=e.concatenate(c,Au(_,1));var g=_!==Ie?Ru(_):[Td(je,Ie,!1)];l=e.concatenate(l,e.filter(g,function(e){return!Cu(l,e.keyType)}))}}qa(n,o,s,c,l)}function Qc(n){Yc(n,Mc(n),e.emptyArray,e.emptyArray)}function Zc(n){var t=Mc(n.target),r=e.concatenate(t.typeParameters,[t.thisType]),i=Md(n),a=i.length===r.length?i:e.concatenate(i,[n]);Yc(n,t,r,a)}function $c(e,n,t,r,i,a,o,s){var c=new y(de,s);return c.declaration=e,c.typeParameters=n,c.parameters=r,c.thisParameter=t,c.resolvedReturnType=i,c.resolvedTypePredicate=a,c.minArgumentCount=o,c.resolvedMinArgumentCount=void 0,c.target=void 0,c.mapper=void 0,c.compositeSignatures=void 0,c.compositeKind=void 0,c}function el(e){var n=$c(e.declaration,e.typeParameters,e.thisParameter,e.parameters,void 0,void 0,e.minArgumentCount,39&e.flags);return n.target=e.target,n.mapper=e.mapper,n.compositeSignatures=e.compositeSignatures,n.compositeKind=e.compositeKind,n}function nl(e,n){var t=el(e);return t.compositeSignatures=n,t.compositeKind=1048576,t.target=void 0,t.mapper=void 0,t}function tl(e,n){if((24&e.flags)===n)return e;e.optionalCallSignatureCache||(e.optionalCallSignatureCache={});var t=8===n?"inner":"outer";return e.optionalCallSignatureCache[t]||(e.optionalCallSignatureCache[t]=rl(e,n))}function rl(n,t){e.Debug.assert(8===t||16===t,"An optional call signature can either be for an inner call chain or an outer call chain, but not both.");var r=el(n);return r.flags|=t,r}function il(n,t){if(V(n)){var r=n.parameters.length-1,i=Ws(n.parameters[r]);if(Jy(i))return[a(i,r)];if(!t&&1048576&i.flags&&e.every(i.types,Jy))return e.map(i.types,function(e){return a(e,r)})}return[n.parameters];function a(t,r){var i=Md(t),a=t.target.labeledElementDeclarations,o=e.map(i,function(e,i){var o=!!a&&oD(a[i]),s=o||sD(n,r+i,t),c=t.target.elementFlags[i],l=12&c?32768:2&c?16384:0,u=Or(1,s,l);return u.type=4&c?Mp(e):e,u});return e.concatenate(n.parameters.slice(0,r),o)}}function al(n){var t=oc(n),r=Au(t,1),i=e.getClassLikeDeclarationOfSymbol(n.symbol),a=!!i&&e.hasSyntacticModifier(i,128);if(0===r.length)return[$c(void 0,n.localTypeParameters,void 0,e.emptyArray,n,void 0,0,a?4:0)];for(var o=rc(n),s=e.isInJSFile(o),c=np(o),l=e.length(c),u=[],d=0,p=r;d<p.length;d++){var f=p[d],m=zu(f.typeParameters),_=e.length(f.typeParameters);if(s||l>=m&&l<=_){var g=_?pd(f,Ju(c,f.typeParameters,m,s)):el(f);g.typeParameters=n.localTypeParameters,g.resolvedReturnType=n,g.flags=a?4|g.flags:-5&g.flags,u.push(g)}}return u}function ol(e,n,t,r,i){for(var a=0,o=e;a<o.length;a++){var s=o[a];if(vy(s,n,t,r,i,t?sg:ag))return s}}function sl(n,t,r){if(t.typeParameters){if(r>0)return;for(var i=1;i<n.length;i++)if(!ol(n[i],t,!1,!1,!1))return;return[t]}var a;for(i=0;i<n.length;i++){var o=i===r?t:ol(n[i],t,!0,!1,!0);if(!o)return;a=e.appendIfUnique(a,o)}return a}function cl(n){for(var t,r,i=0;i<n.length;i++){if(0===n[i].length)return e.emptyArray;n[i].length>1&&(r=void 0===r?i:-1);for(var a=0,o=n[i];a<o.length;a++){var s=o[a];if(!t||!ol(t,s,!1,!1,!0)){var c=sl(n,s,i);if(c){var l=s;if(c.length>1){var u=s.thisParameter,d=e.forEach(c,function(e){return e.thisParameter});if(d){var p=Df(e.mapDefined(c,function(e){return e.thisParameter&&Ws(e.thisParameter)}));u=vh(d,p)}l=nl(s,c),l.thisParameter=u}(t||(t=[])).push(l)}}}}if(!e.length(t)&&-1!==r){for(var f=n[void 0!==r?r:0],m=f.slice(),_=function(n){if(n!==f){var t=n[0];if(e.Debug.assert(!!t,"getUnionSignatures bails early on empty signature lists and should not have empty lists on second pass"),m=t.typeParameters&&e.some(m,function(e){return!!e.typeParameters&&!ll(t.typeParameters,e.typeParameters)})?void 0:e.map(m,function(e){return pl(e,t)}),!m)return"break"}},g=0,y=n;g<y.length;g++){var h=y[g],v=_(h);if("break"===v)break}t=m}return t||e.emptyArray}function ll(n,t){if(e.length(n)!==e.length(t))return!1;if(!n||!t)return!0;for(var r=g_(t,n),i=0;i<n.length;i++){var a=n[i],o=t[i];if(a!==o&&!ig(Cd(a)||Pe,z_(Cd(o)||Pe,r)))return!1}return!0}function ul(e,n,t){if(!e||!n)return e||n;var r=Df([Ws(e),z_(Ws(n),t)]);return vh(e,r)}function dl(e,n,t){for(var r=_D(e),i=_D(n),a=r>=i?e:n,o=a===e?n:e,s=a===e?r:i,c=yD(e)||yD(n),l=c&&!yD(a),u=new Array(s+(l?1:0)),d=0;d<s;d++){var p=fD(a,d);a===n&&(p=z_(p,t));var f=fD(o,d)||Pe;o===n&&(f=z_(f,t));var m=Df([p,f]),_=c&&!l&&d===s-1,g=d>=gD(a)&&d>=gD(o),y=d>=r?void 0:sD(e,d),h=d>=i?void 0:sD(n,d),v=y===h?y:y?h?void 0:y:h,b=Or(1|(g&&!_?16777216:0),v||"arg"+d);b.type=_?Mp(m):m,u[d]=b}if(l){var E=Or(1,"args");E.type=Mp(pD(o,s)),o===n&&(E.type=z_(E.type,t)),u[s]=E}return u}function pl(n,t){var r,i=n.typeParameters||t.typeParameters;n.typeParameters&&t.typeParameters&&(r=g_(t.typeParameters,n.typeParameters));var a=n.declaration,o=dl(n,t,r),s=ul(n.thisParameter,t.thisParameter,r),c=Math.max(n.minArgumentCount,t.minArgumentCount),l=$c(a,i,s,o,void 0,void 0,c,39&(n.flags|t.flags));return l.compositeKind=1048576,l.compositeSignatures=e.concatenate(2097152!==n.compositeKind&&n.compositeSignatures||[n],[t]),r&&(l.mapper=2097152!==n.compositeKind&&n.mapper&&n.compositeSignatures?x_(n.mapper,r):r),l}function fl(n){var t=Ru(n[0]);if(t){for(var r=[],i=function(t){var i=t.keyType;e.every(n,function(e){return!!Ou(e,i)})&&r.push(Td(i,ff(e.map(n,function(e){return wu(e,i)})),e.some(n,function(e){return Ou(e,i).isReadonly})))},a=0,o=t;a<o.length;a++){var s=o[a];i(s)}return r}return e.emptyArray}function ml(n){var t=cl(e.map(n.types,function(e){return e===Cn?[Et]:Au(e,0)})),r=cl(e.map(n.types,function(e){return Au(e,1)})),i=fl(n.types);qa(n,D,t,r,i)}function _l(e,n){return e?n?Df([e,n]):e:n}function gl(n){var t=e.countWhere(n,function(e){return Au(e,1).length>0}),r=e.map(n,nc);if(t>0&&t===e.countWhere(r,function(e){return e})){var i=r.indexOf(!0);r[i]=!1}return r}function yl(e,n,t,r){for(var i=[],a=0;a<n.length;a++)a===r?i.push(e):t[a]&&i.push(ad(Au(n[a],1)[0]));return Df(i)}function hl(n){for(var t,r,i,a=n.types,o=gl(a),s=e.countWhere(o,function(e){return e}),c=function(c){var l=n.types[c];if(!o[c]){var u=Au(l,1);u.length&&s>0&&(u=e.map(u,function(e){var n=el(e);return n.resolvedReturnType=yl(ad(e),a,o,c),n})),r=vl(r,u)}t=vl(t,Au(l,0)),i=e.reduceLeft(Ru(l),function(e,n){return bl(e,n,!1)},i)},l=0;l<a.length;l++)c(l);qa(n,D,t||e.emptyArray,r||e.emptyArray,i||e.emptyArray)}function vl(n,t){for(var r=function(t){n&&!e.every(n,function(e){return!vy(e,t,!1,!1,!1,ag)})||(n=e.append(n,t))},i=0,a=t;i<a.length;i++){var o=a[i];r(o)}return n}function bl(n,t,r){if(n)for(var i=0;i<n.length;i++){var a=n[i];if(a.keyType===t.keyType)return n[i]=Td(a.keyType,r?ff([a.type,t.type]):Df([a.type,t.type]),r?a.isReadonly||t.isReadonly:a.isReadonly&&t.isReadonly),n}return e.append(n,t)}function El(n){var t=Aa(n.symbol);if(n.target){qa(n,D,e.emptyArray,e.emptyArray,e.emptyArray);var r=Oc(Vl(n.target),n.mapper,!1),i=m_(Au(n.target,0),n.mapper),a=m_(Au(n.target,1),n.mapper),o=__(Ru(n.target),n.mapper);qa(n,r,i,a,o)}else if(2048&t.flags){qa(n,D,e.emptyArray,e.emptyArray,e.emptyArray);r=Jc(t),i=$u(r.get("__call")),a=$u(r.get("__new")),o=Sd(t);qa(n,r,i,a,o)}else{r=D,o=void 0;if(t.exports&&(r=Sa(t),t===se)){var s=new e.Map;r.forEach(function(e){418&e.flags||s.set(e.escapedName,e)}),r=s}var c=void 0;if(qa(n,r,e.emptyArray,e.emptyArray,e.emptyArray),32&t.flags){var l=gc(t),u=oc(l);11272192&u.flags?(r=e.createSymbolTable(Ja(r)),wc(r,Hl(u))):u===Ie&&(c=Td(je,Ie,!1))}var d=Ed(r);if(d?o=xd(d):(c&&(o=e.append(o,c)),384&t.flags&&(32&Dc(t).flags||e.some(n.properties,function(e){return!!(296&Ws(e).flags)}))&&(o=e.append(o,xt))),qa(n,r,e.emptyArray,e.emptyArray,o||e.emptyArray),8208&t.flags&&(n.callSignatures=$u(t)),32&t.flags){var p=gc(t);a=t.members?$u(t.members.get("__constructor")):e.emptyArray;16&t.flags&&(a=e.addRange(a.slice(),e.mapDefined(n.callSignatures,function(e){return FL(e.declaration)?$c(e.declaration,e.typeParameters,e.thisParameter,e.parameters,p,void 0,e.minArgumentCount,39&e.flags):void 0}))),a.length||(a=al(p)),n.constructSignatures=a}}}function Tl(e,n,t){return z_(e,g_([n.indexType,n.objectType],[$m(0),Wp([t])]))}function Sl(n){for(var t=Ou(n.source,je),r=Pl(n.mappedType),i=!(1&r),a=4&r?0:16777216,o=t?[Td(je,Zh(t.type,n.mappedType,n.constraintType),i&&t.isReadonly)]:e.emptyArray,s=e.createSymbolTable(),c=0,l=Hl(n.source);c<l.length;c++){var u=l[c],d=8192|(i&&JD(u)?8:0),p=Or(4|u.flags&a,u.escapedName,d);if(p.declarations=u.declarations,p.nameType=Hr(u).nameType,p.propertyType=Ws(u),8388608&n.constraintType.type.flags&&262144&n.constraintType.type.objectType.flags&&262144&n.constraintType.type.indexType.flags){var f=n.constraintType.type.objectType,m=Tl(n.mappedType,n.constraintType.type,f);p.mappedType=m,p.constraintType=Uf(f)}else p.mappedType=n.mappedType,p.constraintType=n.constraintType;s.set(u.escapedName,p)}qa(n,s,e.emptyArray,e.emptyArray,o)}function xl(n){if(4194304&n.flags){var t=mu(n.type);return qy(t)?Qp(t):Uf(t)}if(16777216&n.flags){if(n.root.isDistributive){var r=n.checkType,i=xl(r);if(i!==r)return H_(n,D_(n.root.checkType,i,n.mapper))}return n}return 1048576&n.flags?Db(n,xl):2097152&n.flags?Df(e.sameMap(n.types,xl)):n}function Ll(n){return 4096&e.getCheckFlags(n)}function Dl(n){var t,r=e.createSymbolTable();qa(n,D,e.emptyArray,e.emptyArray,e.emptyArray);var i=Cl(n),a=Il(n),o=Nl(n.target||n),s=kl(n.target||n),c=mu(wl(n)),l=Pl(n),u=$?128:8576;if(Ol(n)){for(var d=0,p=Hl(c);d<p.length;d++){var f=p[d];y(Gf(f,u))}if(1&c.flags)y(je);else for(var m=0,_=Ru(c);m<_.length;m++){var g=_[m];(!$||134217732&g.keyType.flags)&&y(g.keyType)}}else vb(xl(a),y);function y(e){var t=o?z_(o,A_(n.mapper,i,e)):e;vb(t,function(n){return h(e,n)})}function h(e,a){if(Fc(a)){var u=Kc(a),d=r.get(u);if(d)d.nameType=ff([d.nameType,a]),d.keyType=ff([d.keyType,e]);else{var p=Fc(e)?Lu(c,Kc(e)):void 0,f=!!(4&l||!(8&l)&&p&&16777216&p.flags),m=!!(1&l||!(2&l)&&p&&JD(p)),_=z&&!f&&p&&16777216&p.flags,g=p?Ll(p):0,y=Or(4|(f?16777216:0),u,262144|g|(m?8:0)|(_?524288:0));y.mappedType=n,y.nameType=a,y.keyType=e,p&&(y.syntheticOrigin=p,y.declarations=o?void 0:p.declarations),r.set(u,y)}}else if(Ld(a)||33&a.flags){var h=5&a.flags?je:40&a.flags?Ke:a,v=z_(s,A_(n.mapper,i,e)),b=Td(h,v,!!(1&l));t=bl(t,b,!0)}}qa(n,r,e.emptyArray,e.emptyArray,t||e.emptyArray)}function Al(n){if(!n.type){var t=n.mappedType;if(!Oo(n,0))return t.containsError=!0,Re;var r=kl(t.target||t),i=A_(t.mapper,Cl(t),n.keyType),a=z_(r,i),o=z&&16777216&n.flags&&!iA(a,49152)?sh(a,!0):524288&n.checkFlags?gh(a):a;Mo()||(Dr(d,e.Diagnostics.Type_of_property_0_circularly_references_itself_in_mapped_type_1,mo(n),go(t)),o=Re),n.type=o}return n.type}function Cl(e){return e.typeParameter||(e.typeParameter=xc(Ca(e.declaration.typeParameter)))}function Il(e){return e.constraintType||(e.constraintType=ql(Cl(e))||Re)}function Nl(e){return e.declaration.nameType?e.nameType||(e.nameType=z_(u_(e.declaration.nameType),e.mapper)):void 0}function kl(e){return e.templateType||(e.templateType=e.declaration.type?z_(ns(u_(e.declaration.type),!0,!!(4&Pl(e))),e.mapper):Re)}function Rl(n){return e.getEffectiveConstraintOfTypeParameter(n.declaration.typeParameter)}function Ol(e){var n=Rl(e);return 191===n.kind&&139===n.operator}function wl(e){if(!e.modifiersType)if(Ol(e))e.modifiersType=z_(u_(Rl(e).type),e.mapper);else{var n=Sm(e.declaration),t=Il(n),r=t&&262144&t.flags?ql(t):t;e.modifiersType=r&&4194304&r.flags?z_(r.type,e.mapper):Pe}return e.modifiersType}function Pl(e){var n=e.declaration;return(n.readonlyToken?40===n.readonlyToken.kind?2:1:0)|(n.questionToken?40===n.questionToken.kind?8:4:0)}function Ml(e){var n=Pl(e);return 8&n?-1:4&n?1:0}function Fl(e){var n=Ml(e),t=wl(e);return n||(Bl(t)?Ml(t):0)}function Gl(n){return!!(32&e.getObjectFlags(n)&&4&Pl(n))}function Bl(n){return!!(32&e.getObjectFlags(n))&&cm(Il(n))}function Ul(e){return e.members||(524288&e.flags?4&e.objectFlags?Zc(e):3&e.objectFlags?Qc(e):1024&e.objectFlags?Sl(e):16&e.objectFlags?El(e):32&e.objectFlags&&Dl(e):1048576&e.flags?ml(e):2097152&e.flags&&hl(e)),e}function Vl(n){return 524288&n.flags?Ul(n).properties:e.emptyArray}function jl(e,n){if(524288&e.flags){var t=Ul(e),r=t.members.get(n);if(r&&Fa(r))return r}}function Kl(n){if(!n.resolvedProperties){for(var t=e.createSymbolTable(),r=0,i=n.types;r<i.length;r++){for(var a=i[r],o=0,s=Hl(a);o<s.length;o++){var c=s[o];if(!t.has(c.escapedName)){var l=hu(n,c.escapedName);l&&t.set(c.escapedName,l)}}if(1048576&n.flags&&0===Ru(a).length)break}n.resolvedProperties=za(t)}return n.resolvedProperties}function Hl(e){return e=_u(e),3145728&e.flags?Kl(e):Vl(e)}function Wl(e,n){var t=n.properties;return t.some(function(n){var t=n.name&&Ff(n.name),r=t&&Fc(t)?Kc(t):void 0,i=void 0===r?void 0:Bo(e,r);return!!i&&Uy(i)&&!lg(RR(n),i)})}function zl(n){var t=ff(n);if(!(1048576&t.flags))return GR(t);for(var r=e.createSymbolTable(),i=0,a=n;i<a.length;i++)for(var o=a[i],s=0,c=GR(o);s<c.length;s++){var l=c[s].escapedName;if(!r.has(l)){var u=gu(t,l);u&&r.set(l,u)}}return e.arrayFrom(r.values())}function Jl(e){return 262144&e.flags?ql(e):8388608&e.flags?Xl(e):16777216&e.flags?nu(e):ru(e)}function ql(e){return au(e)?Cd(e):void 0}function Xl(e){return au(e)?Ql(e):void 0}function Yl(e){var n=dm(e,!1);return n!==e?n:Jl(e)}function Ql(e){var n=Yl(e.indexType);if(n&&n!==e.indexType){var t=Em(e.objectType,n,e.accessFlags);if(t)return t}var r=Yl(e.objectType);if(r&&r!==e.objectType)return Em(r,e.indexType,e.accessFlags)}function Zl(e){if(!e.resolvedDefaultConstraint){var n=km(e),t=Nm(e);e.resolvedDefaultConstraint=Vo(n)?t:Vo(t)?n:ff([n,t])}return e.resolvedDefaultConstraint}function $l(e){if(e.root.isDistributive&&e.restrictiveInstantiation!==e){var n=dm(e.checkType,!1),t=n===e.checkType?Jl(n):n;if(t&&t!==e.checkType){var r=H_(e,D_(e.root.checkType,t,e.mapper));if(!(131072&r.flags))return r}}}function eu(e){return $l(e)||Zl(e)}function nu(e){return au(e)?eu(e):void 0}function tu(n,t){for(var r,i=!1,a=0,o=n;a<o.length;a++){var s=o[a];if(465829888&s.flags){var c=Jl(s);while(c&&21233664&c.flags)c=Jl(c);c&&(r=e.append(r,c),t&&(r=e.append(r,s)))}else 469892092&s.flags&&(i=!0)}if(r&&(t||i)){if(i)for(var l=0,u=n;l<u.length;l++){s=u[l];469892092&s.flags&&(r=e.append(r,s))}return Df(r)}}function ru(e){if(464781312&e.flags){var n=ou(e);return n!==hn&&n!==vn?n:void 0}return 4194304&e.flags?sn:void 0}function iu(e){return ru(e)||e}function au(e){return ou(e)!==vn}function ou(n){if(n.resolvedBaseConstraint)return n.resolvedBaseConstraint;var t=[];return n.resolvedBaseConstraint=Xc(r(n),n);function r(n){if(!n.immediateBaseConstraint){if(!Oo(n,4))return vn;var r=void 0;if((t.length<10||t.length<50&&!my(n,t,t.length))&&(t.push(n),r=a(dm(n,!1)),t.pop()),!Mo()){if(262144&n.flags){var i=Dd(n);if(i){var o=Dr(i,e.Diagnostics.Type_parameter_0_has_a_circular_constraint,go(n));!d||e.isNodeDescendantOf(i,d)||e.isNodeDescendantOf(d,i)||e.addRelatedInfo(o,e.createDiagnosticForNode(d,e.Diagnostics.Circularity_originates_in_type_at_this_location))}}r=vn}n.immediateBaseConstraint=r||hn}return n.immediateBaseConstraint}function i(e){var n=r(e);return n!==hn&&n!==vn?n:void 0}function a(n){if(262144&n.flags){var t=Cd(n);return n.isThisType||!t?t:i(t)}if(3145728&n.flags){for(var r=n.types,a=[],o=!1,s=0,c=r;s<c.length;s++){var l=c[s],u=i(l);u?(u!==l&&(o=!0),a.push(u)):o=!0}return o?1048576&n.flags&&a.length===r.length?ff(a):2097152&n.flags&&a.length?Df(a):void 0:n}if(4194304&n.flags)return sn;if(134217728&n.flags){r=n.types;var d=e.mapDefined(r,i);return d.length===r.length?Wf(n.texts,d):je}if(268435456&n.flags){t=i(n.type);return t?qf(n.symbol,t):je}if(8388608&n.flags){var p=i(n.objectType),f=i(n.indexType),m=p&&f&&Em(p,f,n.accessFlags);return m&&i(m)}if(16777216&n.flags){t=eu(n);return t&&i(t)}return 33554432&n.flags?i(n.substitute):n}}function su(e){return e.resolvedApparentType||(e.resolvedApparentType=Xc(e,e,!0))}function cu(n){if(n.default)n.default===bn&&(n.default=vn);else if(n.target){var t=cu(n.target);n.default=t?z_(t,n.mapper):hn}else{n.default=bn;var r=n.symbol&&e.forEach(n.symbol.declarations,function(n){return e.isTypeParameterDeclaration(n)&&n.default}),i=r?u_(r):hn;n.default===bn&&(n.default=i)}return n.default}function lu(e){var n=cu(e);return n!==hn&&n!==vn?n:void 0}function uu(e){return cu(e)!==vn}function du(n){return!(!n.symbol||!e.forEach(n.symbol.declarations,function(n){return e.isTypeParameterDeclaration(n)&&n.default}))}function pu(e){return e.resolvedApparentType||(e.resolvedApparentType=fu(e))}function fu(e){var n=M_(e);if(n&&!e.declaration.nameType){var t=ql(n);if(t&&(Ly(t)||Jy(t)))return z_(e,D_(n,t,e.mapper))}return e}function mu(n){var t=465829888&n.flags?ru(n)||Pe:n;return 32&e.getObjectFlags(t)?pu(t):2097152&t.flags?su(t):402653316&t.flags?On:296&t.flags?wn:2112&t.flags?Rp(F>=7):528&t.flags?Pn:12288&t.flags?mp(F>=2):67108864&t.flags?pn:4194304&t.flags?sn:2&t.flags&&!z?pn:t}function _u(e){return vu(mu(vu(e)))}function gu(n,t,r){for(var i,a,o,s,c,l=1048576&n.flags,u=l?0:16777216,d=4,p=0,f=!1,m=0,_=n.types;m<_.length;m++){var g=_[m],y=mu(g);if(!(y===Re||131072&y.flags)){var h=Lu(y,t,r),v=h?e.getDeclarationModifierFlagsFromSymbol(h):0;if(h){if(l?u|=16777216&h.flags:u&=h.flags,o){if(h!==o){var b=(pk(h)||h)===(pk(o)||o);if(b&&-1===yy(o,h,function(e,n){return e===n?-1:0}))f=!!o.parent&&!!e.length($s(o.parent));else{s||(s=new e.Map,s.set(w(o),o));var E=w(h);s.has(E)||s.set(E,h)}}}else o=h;p|=(JD(h)?8:0)|(24&v?0:256)|(16&v?512:0)|(8&v?1024:0)|(32&v?2048:0),RS(h)||(d=2)}else if(l){var T=!Bc(t)&&Fu(y,t);T?(p|=32|(T.isReadonly?8:0),c=e.append(c,Jy(y)?Yy(y)||Me:T.type)):!_v(y)||4194304&e.getObjectFlags(y)?p|=16:(p|=32,c=e.append(c,Me))}}}if(o&&!(l&&(s||48&p)&&1536&p)){if(!s&&!(16&p)&&!c){if(f){var S=vh(o,o.type);return S.parent=null===(a=null===(i=o.valueDeclaration)||void 0===i?void 0:i.symbol)||void 0===a?void 0:a.parent,S.containingType=n,S.mapper=o.mapper,S}return o}for(var x,L,D,A,C=s?e.arrayFrom(s.values()):[o],I=[],N=!1,k=0,R=C;k<R.length;k++){h=R[k];A?h.valueDeclaration&&h.valueDeclaration!==A&&(N=!0):A=h.valueDeclaration,x=e.addRange(x,h.declarations);y=Ws(h);L?y!==L&&(p|=64):(L=y,D=Hr(h).nameType),Uy(y)&&(p|=128),131072&y.flags&&(p|=131072),I.push(y)}e.addRange(I,c);var O=Or(4|u,t,d|p);return O.containingType=n,!N&&A&&(O.valueDeclaration=A,A.symbol.parent&&(O.parent=A.symbol.parent)),O.declarations=x,O.nameType=D,I.length>2?(O.checkFlags|=65536,O.deferralParent=n,O.deferralConstituents=I):O.type=l?ff(I):Df(I),O}}function yu(n,t,r){var i,a,o=(null===(i=n.propertyCacheWithoutObjectFunctionPropertyAugment)||void 0===i?void 0:i.get(t))||!r?null===(a=n.propertyCache)||void 0===a?void 0:a.get(t):void 0;if(!o&&(o=gu(n,t,r),o)){var s=r?n.propertyCacheWithoutObjectFunctionPropertyAugment||(n.propertyCacheWithoutObjectFunctionPropertyAugment=e.createSymbolTable()):n.propertyCache||(n.propertyCache=e.createSymbolTable());s.set(t,o)}return o}function hu(n,t,r){var i=yu(n,t,r);return!i||16&e.getCheckFlags(i)?void 0:i}function vu(n){return 1048576&n.flags&&33554432&n.objectFlags?n.resolvedReducedType||(n.resolvedReducedType=bu(n)):2097152&n.flags?(33554432&n.objectFlags||(n.objectFlags|=33554432|(e.some(Kl(n),Eu)?67108864:0)),67108864&n.objectFlags?Ze:n):n}function bu(n){var t=e.sameMap(n.types,vu);if(t===n.types)return n;var r=ff(t);return 1048576&r.flags&&(r.resolvedReducedType=r),r}function Eu(e){return Tu(e)||Su(e)}function Tu(n){return!(16777216&n.flags)&&192===(131264&e.getCheckFlags(n))&&!!(131072&Ws(n).flags)}function Su(n){return!n.valueDeclaration&&!!(1024&e.getCheckFlags(n))}function xu(n,t){if(2097152&t.flags&&67108864&e.getObjectFlags(t)){var r=e.find(Kl(t),Tu);if(r)return e.chainDiagnosticMessages(n,e.Diagnostics.The_intersection_0_was_reduced_to_never_because_property_1_has_conflicting_types_in_some_constituents,go(t,void 0,536870912),mo(r));var i=e.find(Kl(t),Su);if(i)return e.chainDiagnosticMessages(n,e.Diagnostics.The_intersection_0_was_reduced_to_never_because_property_1_exists_in_multiple_constituents_and_is_private_in_some,go(t,void 0,536870912),mo(i))}return n}function Lu(e,n,t){if(e=_u(e),524288&e.flags){var r=Ul(e),i=r.members.get(n);if(i&&Fa(i))return i;if(t)return;var a=r===yn?Cn:r.callSignatures.length?In:r.constructSignatures.length?Nn:void 0;if(a){var o=jl(a,n);if(o)return o}return jl(An,n)}if(3145728&e.flags)return hu(e,n,t)}function Du(n,t){if(3670016&n.flags){var r=Ul(n);return 0===t?r.callSignatures:r.constructSignatures}return e.emptyArray}function Au(e,n){return Du(_u(e),n)}function Cu(n,t){return e.find(n,function(e){return e.keyType===t})}function Iu(n,t){for(var r,i,a,o=0,s=n;o<s.length;o++){var c=s[o];c.keyType===je?r=c:Nu(t,c.keyType)&&(i?(a||(a=[i])).push(c):i=c)}return a?Td(Pe,Df(e.map(a,function(e){return e.type})),e.reduceLeft(a,function(e,n){return e&&n.isReadonly},!0)):i||(r&&Nu(t,je)?r:void 0)}function Nu(e,n){return lg(e,n)||n===je&&lg(e,Ke)||n===Ke&&!!(128&e.flags)&&HT(e.value)}function ku(n){if(3670016&n.flags){var t=Ul(n);return t.indexInfos}return e.emptyArray}function Ru(e){return ku(_u(e))}function Ou(e,n){return Cu(Ru(e),n)}function wu(e,n){var t;return null===(t=Ou(e,n))||void 0===t?void 0:t.type}function Pu(e,n){return Ru(e).filter(function(e){return Nu(n,e.keyType)})}function Mu(e,n){return Iu(Ru(e),n)}function Fu(n,t){return Mu(n,Bc(t)?Ye:Zm(e.unescapeLeadingUnderscores(t)))}function Gu(n){for(var t,r=0,i=e.getEffectiveTypeParameterDeclarations(n);r<i.length;r++){var a=i[r];t=e.appendIfUnique(t,xc(a.symbol))}return t}function Bu(e){var n=[];return e.forEach(function(e,t){Wa(t)||n.push(e)}),n}function Uu(n){return e.isInJSFile(n)&&(n.type&&311===n.type.kind||e.getJSDocParameterTags(n).some(function(e){var n=e.isBracketed,t=e.typeExpression;return n||!!t&&311===t.type.kind}))}function Vu(n,t){if(!e.isExternalModuleNameRelative(n)){var r=Jr(ae,'"'+n+'"',512);return r&&t?Aa(r):r}}function ju(n){if(e.hasQuestionToken(n)||Hu(n)||Uu(n))return!0;if(n.initializer){var t=qu(n.parent),r=n.parent.parameters.indexOf(n);return e.Debug.assert(r>=0),r>=gD(t,3)}var i=e.getImmediatelyInvokedFunctionExpression(n.parent);return!!i&&(!n.type&&!n.dotDotDotToken&&n.parent.parameters.indexOf(n)>=i.arguments.length)}function Ku(n){return e.isPropertyDeclaration(n)&&n.questionToken}function Hu(n){if(!e.isJSDocPropertyLikeTag(n))return!1;var t=n.isBracketed,r=n.typeExpression;return t||!!r&&311===r.type.kind}function Wu(e,n,t,r){return{kind:e,parameterName:n,parameterIndex:t,type:r}}function zu(e){var n=0;if(e)for(var t=0;t<e.length;t++)du(e[t])||(n=t+1);return n}function Ju(n,t,r,i){var a=e.length(t);if(!a)return[];var o=e.length(n);if(i||o>=r&&o<=a){for(var s=n?n.slice():[],c=o;c<a;c++)s[c]=Re;var l=Ev(i);for(c=o;c<a;c++){var u=lu(t[c]);i&&u&&(ig(u,Pe)||ig(u,pn))&&(u=Ie),s[c]=u?z_(u,g_(t,s)):l}return s.length=t.length,s}return n&&n.slice()}function qu(n){var t=Wr(n);if(!t.resolvedSignature){var r=[],i=0,a=0,o=void 0,s=!1,c=e.getImmediatelyInvokedFunctionExpression(n),l=e.isJSDocConstructSignature(n),u=!c&&e.isInJSFile(n)&&e.isValueSignatureDeclaration(n)&&!e.hasJSDocParameterTags(n)&&!e.getJSDocType(n);u&&(i|=32);for(var d=l?1:0;d<n.parameters.length;d++){var p=n.parameters[d],f=p.symbol,m=e.isJSDocParameterTag(p)?p.typeExpression&&p.typeExpression.type:p.type;if(f&&4&f.flags&&!e.isBindingPattern(p.name)){var _=Qr(p,f.escapedName,111551,void 0,void 0,!1);f=_}0===d&&"this"===f.escapedName?(s=!0,o=p.symbol):r.push(f),m&&194===m.kind&&(i|=2);var g=Hu(p)||p.initializer||p.questionToken||e.isRestParameter(p)||c&&r.length>c.arguments.length&&!m||Uu(p);g||(a=r.length)}if((170===n.kind||171===n.kind)&&Vc(n)&&(!s||!o)){var y=170===n.kind?171:170,h=e.getDeclarationOfKind(Ca(n),y);h&&(o=Ns(h))}var v=169===n.kind?gc(Aa(n.parent.symbol)):void 0,b=v?v.localTypeParameters:Gu(n);(e.hasRestParameter(n)||e.isInJSFile(n)&&Xu(n,r))&&(i|=1),(e.isConstructorTypeNode(n)&&e.hasSyntacticModifier(n,128)||e.isConstructorDeclaration(n)&&e.hasSyntacticModifier(n.parent,128))&&(i|=4),t.resolvedSignature=$c(n,b,o,r,void 0,void 0,a,i)}return t.resolvedSignature}function Xu(n,t){if(e.isJSDocSignature(n)||!Zu(n))return!1;var r=e.lastOrUndefined(n.parameters),i=r?e.getJSDocParameterTags(r):e.getJSDocTags(n).filter(e.isJSDocParameterTag),a=e.firstDefined(i,function(n){return n.typeExpression&&e.isJSDocVariadicType(n.typeExpression.type)?n.typeExpression.type:void 0}),o=Or(3,"args",32768);return o.type=a?Mp(u_(a.type)):Gn,a&&t.pop(),t.push(o),!0}function Yu(n){if(e.isInJSFile(n)&&e.isFunctionLikeDeclaration(n)){var t=e.getJSDocTypeTag(n);return(null===t||void 0===t?void 0:t.typeExpression)&&Vx(u_(t.typeExpression))}}function Qu(e){var n=Yu(e);return n&&ad(n)}function Zu(n){var t=Wr(n);return void 0===t.containsArgumentsReference&&(8192&t.flags?t.containsArgumentsReference=!0:t.containsArgumentsReference=r(n.body)),t.containsArgumentsReference;function r(n){if(!n)return!1;switch(n.kind){case 79:return n.escapedText===le.escapedName&&xv(n)===le;case 165:case 167:case 170:case 171:return 160===n.name.kind&&r(n.name);case 204:case 205:return r(n.expression);default:return!e.nodeStartsNewLexicalEnvironment(n)&&!e.isPartOfTypeNode(n)&&!!e.forEachChild(n,r)}}}function $u(n){if(!n||!n.declarations)return e.emptyArray;for(var t=[],r=0;r<n.declarations.length;r++){var i=n.declarations[r];if(e.isFunctionLike(i)){if(r>0&&i.body){var a=n.declarations[r-1];if(i.parent===a.parent&&i.kind===a.kind&&i.pos===a.end)continue}t.push(qu(i))}}return t}function ed(e){var n=ca(e,e);if(n){var t=fa(n);if(t)return Ws(t)}return Ie}function nd(e){if(e.thisParameter)return Ws(e.thisParameter)}function td(n){if(!n.resolvedTypePredicate){if(n.target){var t=td(n.target);n.resolvedTypePredicate=t?N_(t,n.mapper):vt}else if(n.compositeSignatures)n.resolvedTypePredicate=mf(n.compositeSignatures,n.compositeKind)||vt;else{var r=n.declaration&&e.getEffectiveReturnTypeNode(n.declaration),i=void 0;if(!r&&e.isInJSFile(n.declaration)){var a=Yu(n.declaration);a&&n!==a&&(i=td(a))}n.resolvedTypePredicate=r&&e.isTypePredicateNode(r)?rd(r,n):i||vt}e.Debug.assert(!!n.resolvedTypePredicate)}return n.resolvedTypePredicate===vt?void 0:n.resolvedTypePredicate}function rd(n,t){var r=n.parameterName,i=n.type&&u_(n.type);return 190===r.kind?Wu(n.assertsModifier?2:0,void 0,void 0,i):Wu(n.assertsModifier?3:1,r.escapedText,e.findIndex(t.parameters,function(e){return e.escapedName===r.escapedText}),i)}function id(e,n,t){return 2097152!==n?ff(e,t):Df(e)}function ad(n){if(!n.resolvedReturnType){if(!Oo(n,3))return Re;var t=n.target?z_(ad(n.target),n.mapper):n.compositeSignatures?z_(id(e.map(n.compositeSignatures,ad),n.compositeKind,2),n.mapper):od(n.declaration)||(e.nodeIsMissing(n.declaration.body)?Ie:kD(n.declaration));if(8&n.flags?t=uh(t):16&n.flags&&(t=sh(t)),!Mo()){if(n.declaration){var r=e.getEffectiveReturnTypeNode(n.declaration);if(r)Dr(r,e.Diagnostics.Return_type_annotation_circularly_references_itself);else if(Y){var i=n.declaration,a=e.getNameOfDeclaration(i);a?Dr(a,e.Diagnostics._0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions,e.declarationNameToString(a)):Dr(i,e.Diagnostics.Function_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions)}}t=Ie}n.resolvedReturnType=t}return n.resolvedReturnType}function od(n){if(169===n.kind)return gc(Aa(n.parent.symbol));if(e.isJSDocConstructSignature(n))return u_(n.parameters[0].type);var t=e.getEffectiveReturnTypeNode(n);if(t)return u_(t);if(170===n.kind&&Vc(n)){var r=e.isInJSFile(n)&&Zo(n);if(r)return r;var i=e.getDeclarationOfKind(Ca(n),171),a=Is(i);if(a)return a}return Qu(n)}function sd(e){return!e.resolvedReturnType&&wo(e,3)>=0}function cd(e){return ld(e)||Ie}function ld(e){if(V(e)){var n=Ws(e.parameters[e.parameters.length-1]),t=Jy(n)?Yy(n):n;return t&&wu(t,Ke)}}function ud(e,n,t,r){var i=dd(e,Ju(n,e.typeParameters,zu(e.typeParameters),t));if(r){var a=jx(ad(i));if(a){var o=el(a);o.typeParameters=r;var s=el(i);return s.resolvedReturnType=vd(o),s}}return i}function dd(n,t){var r=n.instantiations||(n.instantiations=new e.Map),i=Nd(t),a=r.get(i);return a||r.set(i,a=pd(n,t)),a}function pd(e,n){return k_(e,fd(e,n),!0)}function fd(e,n){return g_(e.typeParameters,n)}function md(e){return e.typeParameters?e.erasedSignatureCache||(e.erasedSignatureCache=_d(e)):e}function _d(e){return k_(e,T_(e.typeParameters),!0)}function gd(e){return e.typeParameters?e.canonicalSignatureCache||(e.canonicalSignatureCache=yd(e)):e}function yd(n){return ud(n,e.map(n.typeParameters,function(e){return e.target&&!ql(e.target)?e.target:e}),e.isInJSFile(n.declaration))}function hd(n){var t=n.typeParameters;if(t){if(n.baseSignatureCache)return n.baseSignatureCache;for(var r=T_(t),i=g_(t,e.map(t,function(e){return ql(e)||Pe})),a=e.map(t,function(e){return z_(e,i)||Pe}),o=0;o<t.length-1;o++)a=f_(a,i);return a=f_(a,r),n.baseSignatureCache=k_(n,g_(t,a),!0)}return n}function vd(n){if(!n.isolatedSignatureType){var t=n.declaration?n.declaration.kind:0,r=169===t||173===t||178===t,i=ja(16);i.members=D,i.properties=e.emptyArray,i.callSignatures=r?e.emptyArray:[n],i.constructSignatures=r?[n]:e.emptyArray,i.indexInfos=e.emptyArray,n.isolatedSignatureType=i}return n.isolatedSignatureType}function bd(e){return e.members?Ed(e.members):void 0}function Ed(e){return e.get("__index")}function Td(e,n,t,r){return{keyType:e,type:n,isReadonly:t,declaration:r}}function Sd(n){var t=bd(n);return t?xd(t):e.emptyArray}function xd(n){if(n.declarations){for(var t=[],r=function(n){if(1===n.parameters.length){var r=n.parameters[0];r.type&&vb(u_(r.type),function(r){Ld(r)&&!Cu(t,r)&&t.push(Td(r,n.type?u_(n.type):Ie,e.hasEffectiveModifier(n,64),n))})}},i=0,a=n.declarations;i<a.length;i++){var o=a[i];r(o)}return t}return e.emptyArray}function Ld(n){return!!(4108&n.flags)||am(n)||!!(2097152&n.flags)&&!cm(n)&&!sm(n)&&e.some(n.types,Ld)}function Dd(n){return e.mapDefined(e.filter(n.symbol&&n.symbol.declarations,e.isTypeParameterDeclaration),e.getEffectiveConstraintOfTypeParameter)[0]}function Ad(n){var t,r;if(null===(t=n.symbol)||void 0===t?void 0:t.declarations)for(var i=0,a=n.symbol.declarations;i<a.length;i++){var o=a[i];if(188===o.parent.kind){var s=e.walkUpParenthesizedTypesAndGetParentAndChild(o.parent.parent),c=s[0],l=void 0===c?o.parent:c,u=s[1];if(176===u.kind){var d=u,p=xC(d);if(p){var f=d.typeArguments.indexOf(l);if(f<p.length){var m=ql(p[f]);if(m){var _=g_(p,TC(d,p)),g=z_(m,_);g!==n&&(r=e.append(r,g))}}}}else if(162===u.kind&&u.dotDotDotToken||184===u.kind||195===u.kind&&u.dotDotDotToken)r=e.append(r,Mp(Pe));else if(197===u.kind)r=e.append(r,je);else if(161===u.kind&&193===u.parent.kind)r=e.append(r,sn);else if(193===u.kind&&u.type&&e.skipParentheses(u.type)===o.parent&&187===u.parent.kind&&u.parent.extendsType===u&&193===u.parent.checkType.kind&&u.parent.checkType.type){var y=u.parent.checkType,h=u_(y.type);r=e.append(r,z_(h,h_(xc(Ca(y.typeParameter)),y.typeParameter.constraint?u_(y.typeParameter.constraint):sn)))}}}return r&&Df(r)}function Cd(e){if(!e.constraint)if(e.target){var n=ql(e.target);e.constraint=n?z_(n,e.mapper):hn}else{var t=Dd(e);if(t){var r=u_(t);1&r.flags&&r!==Re&&(r=193===t.parent.parent.kind?sn:Pe),e.constraint=r}else e.constraint=Ad(e)||hn}return e.constraint===hn?void 0:e.constraint}function Id(n){var t=e.getDeclarationOfKind(n.symbol,161),r=e.isJSDocTemplateTag(t.parent)?e.getHostSignatureFromJSDoc(t.parent):t.parent;return r&&Ca(r)}function Nd(e){var n="";if(e){var t=e.length,r=0;while(r<t){var i=e[r].id,a=1;while(r+a<t&&e[r+a].id===i+a)a++;n.length&&(n+=","),n+=i,a>1&&(n+=":"+a),r+=a}}return n}function kd(e,n){return e?"@"+w(e)+(n?":"+Nd(n):""):""}function Rd(n,t){for(var r=0,i=0,a=n;i<a.length;i++){var o=a[i];o.flags&t||(r|=e.getObjectFlags(o))}return 917504&r}function Od(e,n){var t=Nd(n),r=e.instantiations.get(t);return r||(r=ja(4,e.symbol),e.instantiations.set(t,r),r.objectFlags|=n?Rd(n,0):0,r.target=e,r.resolvedTypeArguments=n),r}function wd(e){var n=Ba(e.flags);return n.symbol=e.symbol,n.objectFlags=e.objectFlags,n.target=e.target,n.resolvedTypeArguments=e.resolvedTypeArguments,n}function Pd(e,n,t,r,i){if(!r){r=Bm(n);var a=Um(r);i=t?f_(a,t):a}var o=ja(4,e.symbol);return o.target=e,o.node=n,o.mapper=t,o.aliasSymbol=r,o.aliasTypeArguments=i,o}function Md(n){var t,r;if(!n.resolvedTypeArguments){if(!Oo(n,6))return(null===(t=n.target.localTypeParameters)||void 0===t?void 0:t.map(function(){return Re}))||e.emptyArray;var i=n.node,a=i?176===i.kind?e.concatenate(n.target.outerTypeParameters,TC(i,n.target.localTypeParameters)):181===i.kind?[u_(i.elementType)]:e.map(i.elements,u_):e.emptyArray;Mo()?n.resolvedTypeArguments=n.mapper?f_(a,n.mapper):a:(n.resolvedTypeArguments=(null===(r=n.target.localTypeParameters)||void 0===r?void 0:r.map(function(){return Re}))||e.emptyArray,Dr(n.node||d,n.target.symbol?e.Diagnostics.Type_arguments_for_0_circularly_reference_themselves:e.Diagnostics.Tuple_type_arguments_circularly_reference_themselves,n.target.symbol&&mo(n.target.symbol)))}return n.resolvedTypeArguments}function Fd(n){return e.length(n.target.typeParameters)}function Gd(n,t){var r=Dc(Aa(t)),i=r.localTypeParameters;if(i){var a=e.length(n.typeArguments),o=zu(i),s=e.isInJSFile(n),c=!Y&&s;if(!c&&(a<o||a>i.length)){var l=s&&e.isExpressionWithTypeArguments(n)&&!e.isJSDocAugmentsTag(n.parent),u=o===i.length?l?e.Diagnostics.Expected_0_type_arguments_provide_these_with_an_extends_tag:e.Diagnostics.Generic_type_0_requires_1_type_argument_s:l?e.Diagnostics.Expected_0_1_type_arguments_provide_these_with_an_extends_tag:e.Diagnostics.Generic_type_0_requires_between_1_and_2_type_arguments,d=go(r,void 0,2);if(Dr(n,u,d,o,i.length),!s)return Re}if(176===n.kind&&Up(n,e.length(n.typeArguments)!==i.length))return Pd(r,n,void 0);var p=e.concatenate(r.outerTypeParameters,Ju(np(n),i,o,s));return Od(r,p)}return Qd(n,t)?r:Re}function Bd(n,t,r,i){var a=Dc(n);if(a===we&&N.has(n.escapedName)&&t&&1===t.length)return qf(n,t[0]);var o=Hr(n),s=o.typeParameters,c=Nd(t)+kd(r,i),l=o.instantiations.get(c);return l||o.instantiations.set(c,l=J_(a,g_(s,Ju(t,s,zu(s),e.isInJSFile(n.valueDeclaration))),r,i)),l}function Ud(n,t){var r=Dc(t),i=Hr(t).typeParameters;if(i){var a=e.length(n.typeArguments),o=zu(i);if(a<o||a>i.length)return Dr(n,o===i.length?e.Diagnostics.Generic_type_0_requires_1_type_argument_s:e.Diagnostics.Generic_type_0_requires_between_1_and_2_type_arguments,mo(t),o,i.length),Re;var s=Bm(n),c=!s||!Vd(t)&&Vd(s)?void 0:s;return Bd(t,np(n),c,Um(c))}return Qd(n,t)?r:Re}function Vd(n){var t,r=null===(t=n.declarations)||void 0===t?void 0:t.find(e.isTypeAlias);return!(!r||!e.getContainingFunction(r))}function jd(n){switch(n.kind){case 176:return n.typeName;case 226:var t=n.expression;if(e.isEntityNameExpression(t))return t}}function Kd(e,n,t){return e&&ra(e,n,t)||Ae}function Hd(e,n){if(n===Ae)return Re;if(n=sa(n)||n,96&n.flags)return Gd(e,n);if(524288&n.flags)return Ud(e,n);var t=Ac(n);if(t)return Qd(e,n)?Ym(t):Re;if(111551&n.flags&&Yd(e)){var r=Wd(e,n);return r||(Kd(jd(e),788968),Ws(n))}return Re}function Wd(e,n){var t=Wr(e);if(!t.resolvedJSDocType){var r=Ws(n),i=r;if(n.valueDeclaration){var a=198===e.kind&&e.qualifier;r.symbol&&r.symbol!==n&&a&&(i=Hd(e,r.symbol))}t.resolvedJSDocType=i}return t.resolvedJSDocType}function zd(e,n){if(3&n.flags||n===e)return e;var t=nf(e)+">"+nf(n),r=Se.get(t);if(r)return r;var i=Ba(33554432);return i.baseType=e,i.substitute=n,Se.set(t,i),i}function Jd(e){return 182===e.kind&&1===e.elements.length}function qd(e,n,t){return Jd(n)&&Jd(t)?qd(e,n.elements[0],t.elements[0]):xm(u_(n))===e?u_(t):void 0}function Xd(n,t){var r,i=!0;while(t&&!e.isStatement(t)&&315!==t.kind){var a=t.parent;if(162===a.kind&&(i=!i),(i||8650752&n.flags)&&187===a.kind&&t===a.trueType){var o=qd(n,a.checkType,a.extendsType);o&&(r=e.append(r,o))}t=a}return r?zd(n,Df(e.append(r,n))):n}function Yd(e){return!!(4194304&e.flags)&&(176===e.kind||198===e.kind)}function Qd(n,t){return!n.typeArguments||(Dr(n,e.Diagnostics.Type_0_is_not_generic,t?mo(t):n.typeName?e.declarationNameToString(n.typeName):l),!1)}function Zd(n){if(e.isIdentifier(n.typeName)){var t=n.typeArguments;switch(n.typeName.escapedText){case"String":return Qd(n),je;case"Number":return Qd(n),Ke;case"Boolean":return Qd(n),Xe;case"Void":return Qd(n),Qe;case"Undefined":return Qd(n),Me;case"Null":return Qd(n),Ue;case"Function":case"function":return Qd(n),Cn;case"array":return t&&t.length||Y?void 0:Gn;case"promise":return t&&t.length||Y?void 0:AD(Ie);case"Object":if(t&&2===t.length){if(e.isJSDocIndexSignature(n)){var r=u_(t[0]),i=u_(t[1]),a=r===je||r===Ke?[Td(r,i,!1)]:e.emptyArray;return Xa(void 0,D,e.emptyArray,e.emptyArray,a)}return Ie}return Qd(n),Y?void 0:Ie}}}function $d(e){var n=u_(e.type);return z?oh(n,65536):n}function ep(n){var t=Wr(n);if(!t.resolvedType){if(e.isConstTypeReference(n)&&e.isAssertionExpression(n.parent))return t.resolvedSymbol=Ae,t.resolvedType=kA(n.parent.expression);var r=void 0,i=void 0,a=788968;Yd(n)&&(i=Zd(n),i||(r=Kd(jd(n),a,!0),r===Ae?r=Kd(jd(n),111551|a):Kd(jd(n),a),i=Hd(n,r))),i||(r=Kd(jd(n),a),i=Hd(n,r)),t.resolvedSymbol=r,t.resolvedType=i}return t.resolvedType}function np(n){return e.map(n.typeArguments,u_)}function tp(n){var t=Wr(n);if(!t.resolvedType){var r=e.isThisIdentifier(n.exprName)?CE(n.exprName):eC(n.exprName);t.resolvedType=Ym(Ch(r))}return t.resolvedType}function rp(n,t){function r(e){var n=e.declarations;if(n)for(var t=0,r=n;t<r.length;t++){var i=r[t];switch(i.kind){case 255:case 256:case 258:return i}}}if(!n)return t?gn:pn;var i=Dc(n);return 524288&i.flags?e.length(i.typeParameters)!==t?(Dr(r(n),e.Diagnostics.Global_type_0_must_have_1_type_parameter_s,e.symbolName(n),t),t?gn:pn):i:(Dr(r(n),e.Diagnostics.Global_type_0_must_be_a_class_or_interface_type,e.symbolName(n)),t?gn:pn)}function ip(n,t){return op(n,111551,t?e.Diagnostics.Cannot_find_global_value_0:void 0)}function ap(n,t){return op(n,788968,t?e.Diagnostics.Cannot_find_global_type_0:void 0)}function op(e,n,t){return Qr(void 0,e,n,t,e,!1)}function sp(e,n,t){var r=ap(e,t);return r||t?rp(r,n):void 0}function cp(){return Wn||(Wn=sp("TypedPropertyDescriptor",1,!0))||gn}function lp(){return ot||(ot=sp("TemplateStringsArray",0,!0))||pn}function up(){return st||(st=sp("ImportMeta",0,!0))||pn}function dp(){if(!ct){var n=Or(0,"ImportMetaExpression"),t=up(),r=Or(4,"meta",8);r.parent=n,r.type=t;var i=e.createSymbolTable([r]);n.members=i,ct=Xa(n,i,e.emptyArray,e.emptyArray,e.emptyArray)}return ct}function pp(e){return jn||(jn=ip("Symbol",e))}function fp(e){return Kn||(Kn=ap("SymbolConstructor",e))}function mp(e){return Hn||(Hn=sp("Symbol",0,e))||pn}function _p(e){return zn||(zn=sp("Promise",1,e))||gn}function gp(e){return Jn||(Jn=sp("PromiseLike",1,e))||gn}function yp(e){return qn||(qn=ip("Promise",e))}function hp(e){return Xn||(Xn=sp("PromiseConstructorLike",0,e))||pn}function vp(e){return tt||(tt=sp("AsyncIterable",1,e))||gn}function bp(e){return rt||(rt=sp("AsyncIterator",3,e))||gn}function Ep(e){return it||(it=sp("AsyncIterableIterator",1,e))||gn}function Tp(e){return at||(at=sp("AsyncGenerator",3,e))||gn}function Sp(e){return Yn||(Yn=sp("Iterable",1,e))||gn}function xp(e){return Qn||(Qn=sp("Iterator",3,e))||gn}function Lp(e){return Zn||(Zn=sp("IterableIterator",1,e))||gn}function Dp(e){return $n||($n=sp("Generator",3,e))||gn}function Ap(e){return et||(et=sp("IteratorYieldResult",1,e))||gn}function Cp(e){return nt||(nt=sp("IteratorReturnResult",1,e))||gn}function Ip(e,n){void 0===n&&(n=0);var t=op(e,788968,void 0);return t&&rp(t,n)}function Np(){return lt||(lt=op("Extract",524288,e.Diagnostics.Cannot_find_global_type_0))}function kp(){return ut||(ut=op("Omit",524288,e.Diagnostics.Cannot_find_global_type_0))}function Rp(e){return dt||(dt=sp("BigInt",0,e))||pn}function Op(e,n){return e!==gn?Od(e,n):pn}function wp(e){return Op(cp(),[e])}function Pp(e){return Op(Sp(!0),[e])}function Mp(e,n){return Op(n?Rn:kn,[e])}function Fp(e){switch(e.kind){case 183:return 2;case 184:return Gp(e);case 195:return e.questionToken?2:e.dotDotDotToken?Gp(e):1;default:return 1}}function Gp(e){return c_(e.type)?4:8}function Bp(n){var t=Hp(n.parent),r=c_(n);if(r)return t?Rn:kn;var i=e.map(n.elements,Fp),a=e.some(n.elements,function(e){return 195!==e.kind});return zp(i,t,a?void 0:n.elements)}function Up(n,t){return!!Bm(n)||Vp(n)&&(181===n.kind?jp(n.elementType):182===n.kind?e.some(n.elements,jp):t||e.some(n.typeArguments,jp))}function Vp(e){var n=e.parent;switch(n.kind){case 189:case 195:case 176:case 185:case 186:case 192:case 187:case 191:case 181:case 182:return Vp(n);case 257:return!0}return!1}function jp(n){switch(n.kind){case 176:return Yd(n)||!!(524288&Kd(n.typeName,788968).flags);case 179:return!0;case 191:return 152!==n.operator&&jp(n.type);case 189:case 183:case 195:case 311:case 309:case 310:case 304:return jp(n.type);case 184:return 181!==n.type.kind||jp(n.type.elementType);case 185:case 186:return e.some(n.types,jp);case 192:return jp(n.objectType)||jp(n.indexType);case 187:return jp(n.checkType)||jp(n.extendsType)||jp(n.trueType)||jp(n.falseType)}return!1}function Kp(n){var t=Wr(n);if(!t.resolvedType){var r=Bp(n);if(r===gn)t.resolvedType=pn;else if(182===n.kind&&e.some(n.elements,function(e){return!!(8&Fp(e))})||!Up(n)){var i=181===n.kind?[u_(n.elementType)]:e.map(n.elements,u_);t.resolvedType=qp(r,i)}else t.resolvedType=182===n.kind&&0===n.elements.length?r:Pd(r,n,void 0)}return t.resolvedType}function Hp(n){return e.isTypeOperatorNode(n)&&143===n.operator}function Wp(n,t,r,i){void 0===r&&(r=!1);var a=zp(t||e.map(n,function(e){return 1}),r,i);return a===gn?pn:n.length?qp(a,n):a}function zp(n,t,r){if(1===n.length&&4&n[0])return t?Rn:kn;var i=e.map(n,function(e){return 1&e?"#":2&e?"?":4&e?".":"*"}).join()+(t?"R":"")+(r&&r.length?","+e.map(r,O).join(","):""),a=fe.get(i);return a||fe.set(i,a=Jp(n,t,r)),a}function Jp(n,t,r){var i,a=n.length,o=e.countWhere(n,function(e){return!!(9&e)}),s=[],c=0;if(a){i=new Array(a);for(var l=0;l<a;l++){var u=i[l]=Ha(),d=n[l];if(c|=d,!(12&c)){var p=Or(4|(2&d?16777216:0),""+l,t?8:0);p.tupleLabelDeclaration=null===r||void 0===r?void 0:r[l],p.type=u,s.push(p)}}}var f=s.length,m=Or(4,"length");if(12&c)m.type=Ke;else{var _=[];for(l=o;l<=a;l++)_.push($m(l));m.type=ff(_)}s.push(m);var g=ja(12);return g.typeParameters=i,g.outerTypeParameters=void 0,g.localTypeParameters=i,g.instantiations=new e.Map,g.instantiations.set(Nd(g.typeParameters),g),g.target=g,g.resolvedTypeArguments=g.typeParameters,g.thisType=Ha(),g.thisType.isThisType=!0,g.thisType.constraint=g,g.declaredProperties=s,g.declaredCallSignatures=e.emptyArray,g.declaredConstructSignatures=e.emptyArray,g.declaredIndexInfos=e.emptyArray,g.elementFlags=n,g.minLength=o,g.fixedLength=f,g.hasRestElement=!!(12&c),g.combinedFlags=c,g.readonly=t,g.labeledElementDeclarations=r,g}function qp(e,n){return 8&e.objectFlags?Xp(e,n):Od(e,n)}function Xp(n,t){var r,i,a;if(!(14&n.combinedFlags))return Od(n,t);if(8&n.combinedFlags){var o=e.findIndex(t,function(e,t){return!!(8&n.elementFlags[t]&&1179648&e.flags)});if(o>=0)return Cf(e.map(t,function(e,t){return 8&n.elementFlags[t]?e:Pe}))?Db(t[o],function(r){return Xp(n,e.replaceElement(t,o,r))}):Re}for(var s=[],c=[],l=[],u=-1,p=-1,f=-1,m=function(o){var c=t[o],l=n.elementFlags[o];if(8&l)if(58982400&c.flags||Bl(c))h(c,8,null===(r=n.labeledElementDeclarations)||void 0===r?void 0:r[o]);else if(Jy(c)){var u=Md(c);if(u.length+s.length>=1e4)return Dr(d,e.isPartOfTypeNode(d)?e.Diagnostics.Type_produces_a_tuple_type_that_is_too_large_to_represent:e.Diagnostics.Expression_produces_a_tuple_type_that_is_too_large_to_represent),{value:Re};e.forEach(u,function(e,n){var t;return h(e,c.target.elementFlags[n],null===(t=c.target.labeledElementDeclarations)||void 0===t?void 0:t[n])})}else h(Iy(c)&&wu(c,Ke)||Re,4,null===(i=n.labeledElementDeclarations)||void 0===i?void 0:i[o]);else h(c,l,null===(a=n.labeledElementDeclarations)||void 0===a?void 0:a[o])},_=0;_<t.length;_++){var g=m(_);if("object"===typeof g)return g.value}for(_=0;_<u;_++)2&c[_]&&(c[_]=1);p>=0&&p<f&&(s[p]=ff(e.sameMap(s.slice(p,f+1),function(e,n){return 8&c[p+n]?vm(e,Ke):e})),s.splice(p+1,f-p),c.splice(p+1,f-p),null===l||void 0===l||l.splice(p+1,f-p));var y=zp(c,n.readonly,l);return y===gn?pn:c.length?Od(y,s):y;function h(e,n,t){1&n&&(u=c.length),4&n&&p<0&&(p=c.length),6&n&&(f=c.length),s.push(e),c.push(n),l&&t?l.push(t):l=void 0}}function Yp(n,t,r){void 0===r&&(r=0);var i=n.target,a=Fd(n)-r;return t>i.fixedLength?Qy(n)||Wp(e.emptyArray):Wp(Md(n).slice(t,a),i.elementFlags.slice(t,a),!1,i.labeledElementDeclarations&&i.labeledElementDeclarations.slice(t,a))}function Qp(n){return ff(e.append(e.arrayOf(n.target.fixedLength,function(e){return Zm(""+e)}),Uf(n.target.readonly?Rn:kn)))}function Zp(n,t){var r=e.findIndex(n.elementFlags,function(e){return!(e&t)});return r>=0?r:n.elementFlags.length}function $p(n,t){return n.elementFlags.length-e.findLastIndex(n.elementFlags,function(e){return!(e&t)})-1}function ef(e){return ns(u_(e.type),!0)}function nf(e){return e.id}function tf(n,t){return e.binarySearch(n,t,nf,e.compareValues)>=0}function rf(n,t){var r=e.binarySearch(n,t,nf,e.compareValues);return r<0&&(n.splice(~r,0,t),!0)}function af(n,t,r){var i=r.flags;if(1048576&i)return of(n,t|(uf(r)?1048576:0),r.types);if(!(131072&i))if(t|=205258751&i,469499904&i&&(t|=262144),r===ke&&(t|=8388608),!z&&98304&i)131072&e.getObjectFlags(r)||(t|=4194304);else{var a=n.length,o=a&&r.id>n[a-1].id?~a:e.binarySearch(n,r,nf,e.compareValues);o<0&&n.splice(~o,0,r)}return t}function of(e,n,t){for(var r=0,i=t;r<i.length;r++){var a=i[r];n=af(e,n,a)}return n}function sf(n,t){var r=Nd(n),i=xe.get(r);if(i)return i;var a=t&&e.some(n,function(e){return!!(524288&e.flags)&&!Bl(e)&&Fg(Ul(e))}),o=n.length,s=o,c=0;while(s>0){s--;var l=n[s];if(a||469499904&l.flags)for(var u=61603840&l.flags?e.find(Hl(l),function(e){return Fy(Ws(e))}):void 0,p=u&&Ym(Ws(u)),f=0,m=n;f<m.length;f++){var _=m[f];if(l!==_){if(1e5===c){var g=c/(o-s)*o;if(g>1e6)return null===e.tracing||void 0===e.tracing||e.tracing.instant("checkTypes","removeSubtypes_DepthLimit",{typeIds:n.map(function(e){return e.id})}),void Dr(d,e.Diagnostics.Expression_produces_a_union_type_that_is_too_complex_to_represent)}if(c++,u&&61603840&_.flags){var y=Bo(_,u.escapedName);if(y&&Fy(y)&&Ym(y)!==p)continue}if(Kg(l,_,_r)&&(!(1&e.getObjectFlags(qs(l)))||!(1&e.getObjectFlags(qs(_)))||ug(l,_))){e.orderedRemoveItemAt(n,s);break}}}}return xe.set(r,n),n}function cf(n,t,r){var i=n.length;while(i>0){i--;var a=n[i],o=a.flags,s=402653312&o&&4&t||256&o&&8&t||2048&o&&64&t||8192&o&&4096&t||r&&32768&o&&16384&t||Qm(a)&&tf(n,a.regularType);s&&e.orderedRemoveItemAt(n,i)}}function lf(n){var t=e.filter(n,am);if(t.length){var r=n.length,i=function(){r--;var i=n[r];128&i.flags&&e.some(t,function(e){return cg(i,e)})&&e.orderedRemoveItemAt(n,r)};while(r>0)i()}}function uf(e){return!!(1048576&e.flags&&(e.aliasSymbol||e.origin))}function df(n,t){for(var r=0,i=t;r<i.length;r++){var a=i[r];if(1048576&a.flags){var o=a.origin;a.aliasSymbol||o&&!(1048576&o.flags)?e.pushIfUnique(n,a):o&&1048576&o.flags&&df(n,o.types)}}}function pf(e,n){var t=Ua(e);return t.types=n,t}function ff(n,t,r,i,a){if(void 0===t&&(t=1),0===n.length)return Ze;if(1===n.length)return n[0];var o=[],s=of(o,0,n);if(0!==t){if(3&s)return 1&s?8388608&s?ke:Ie:Pe;if(ne&&32768&s){var c=e.binarySearch(o,Be,nf,e.compareValues);c>=0&&tf(o,Me)&&e.orderedRemoveItemAt(o,c)}if((402664320&s||16384&s&&32768&s)&&cf(o,s,!!(2&t)),128&s&&134217728&s&&lf(o),2===t&&(o=sf(o,!!(524288&s)),!o))return Re;if(0===o.length)return 65536&s?4194304&s?Ue:Ve:32768&s?4194304&s?Me:Fe:Ze}if(!a&&1048576&s){var l=[];df(l,n);for(var u=[],d=function(n){e.some(l,function(e){return tf(e.types,n)})||u.push(n)},p=0,f=o;p<f.length;p++){var m=f[p];d(m)}if(!r&&1===l.length&&0===u.length)return l[0];var _=e.reduceLeft(l,function(e,n){return e+n.types.length},0);if(_+u.length===o.length){for(var g=0,y=l;g<y.length;g++){m=y[g];rf(u,m)}a=pf(1048576,u)}}var h=(468598819&s?0:65536)|(2097152&s?33554432:0);return gf(o,h,r,i,a)}function mf(e,n){for(var t,r=[],i=0,a=e;i<a.length;i++){var o=a[i],s=td(o);if(!s||2===s.kind||3===s.kind){if(2097152!==n)continue;return}if(t){if(!_f(t,s))return}else t=s;r.push(s.type)}if(t){var c=id(r,n);return Wu(t.kind,t.parameterName,t.parameterIndex,c)}}function _f(e,n){return e.kind===n.kind&&e.parameterIndex===n.parameterIndex}function gf(e,n,t,r,i){if(0===e.length)return Ze;if(1===e.length)return e[0];var a=i?1048576&i.flags?"|"+Nd(i.types):2097152&i.flags?"&"+Nd(i.types):"#"+i.type.id+"|"+Nd(e):Nd(e),o=a+kd(t,r),s=me.get(o);return s||(s=Ba(1048576),s.objectFlags=n|Rd(e,98304),s.types=e,s.origin=i,s.aliasSymbol=t,s.aliasTypeArguments=r,2===e.length&&512&e[0].flags&&512&e[1].flags&&(s.flags|=16,s.intrinsicName="boolean"),me.set(o,s)),s}function yf(n){var t=Wr(n);if(!t.resolvedType){var r=Bm(n);t.resolvedType=ff(e.map(n.types,u_),1,r,Um(r))}return t.resolvedType}function hf(e,n,t){var r=t.flags;return 2097152&r?vf(e,n,t.types):(Bg(t)?16777216&n||(n|=16777216,e.set(t.id.toString(),t)):(3&r?t===ke&&(n|=8388608):!z&&98304&r||e.has(t.id.toString())||(109440&t.flags&&109440&n&&(n|=67108864),e.set(t.id.toString(),t)),n|=205258751&r),n)}function vf(e,n,t){for(var r=0,i=t;r<i.length;r++){var a=i[r];n=hf(e,n,Ym(a))}return n}function bf(n,t){var r=n.length;while(r>0){r--;var i=n[r],a=4&i.flags&&128&t||8&i.flags&&256&t||64&i.flags&&2048&t||4096&i.flags&&8192&t;a&&e.orderedRemoveItemAt(n,r)}}function Ef(e,n){for(var t=0,r=e;t<r.length;t++){var i=r[t];if(!tf(i.types,n)){var a=128&n.flags?je:256&n.flags?Ke:2048&n.flags?He:8192&n.flags?Ye:void 0;if(!a||!tf(i.types,a))return!1}}return!0}function Tf(n){var t=n.length,r=e.filter(n,function(e){return!!(128&e.flags)});while(t>0){t--;var i=n[t];if(134217728&i.flags)for(var a=0,o=r;a<o.length;a++){var s=o[a];if(cg(s,i)){e.orderedRemoveItemAt(n,t);break}if(am(i))return!0}}return!1}function Sf(n,t){if(e.every(n,function(n){return!!(1048576&n.flags)&&e.some(n.types,function(e){return!!(e.flags&t)})})){for(var r=0;r<n.length;r++)n[r]=Sb(n[r],function(e){return!(e.flags&t)});return!0}return!1}function xf(n){var t,r=e.findIndex(n,function(n){return!!(65536&e.getObjectFlags(n))});if(r<0)return!1;var i=r+1;while(i<n.length){var a=n[i];65536&e.getObjectFlags(a)?((t||(t=[n[r]])).push(a),e.orderedRemoveItemAt(n,i)):i++}if(!t)return!1;for(var o=[],s=[],c=0,l=t;c<l.length;c++)for(var u=l[c],d=0,p=u.types;d<p.length;d++){a=p[d];rf(o,a)&&Ef(t,a)&&rf(s,a)}return n[r]=gf(s,65536),!0}function Lf(e,n,t){var r=Ba(2097152);return r.objectFlags=Rd(e,98304),r.types=e,r.aliasSymbol=n,r.aliasTypeArguments=t,r}function Df(n,t,r){var i=new e.Map,a=vf(i,0,n),o=e.arrayFrom(i.values());if(131072&a)return e.contains(o,$e)?$e:Ze;if(z&&98304&a&&84410368&a||67108864&a&&402783228&a||402653316&a&&67238776&a||296&a&&469891796&a||2112&a&&469889980&a||12288&a&&469879804&a||49152&a&&469842940&a)return Ze;if(134217728&a&&128&a&&Tf(o))return Ze;if(1&a)return 8388608&a?ke:Ie;if(!z&&98304&a)return 32768&a?Me:Ue;if((4&a&&128&a||8&a&&256&a||64&a&&2048&a||4096&a&&8192&a)&&bf(o,a),16777216&a&&524288&a&&e.orderedRemoveItemAt(o,e.findIndex(o,Bg)),0===o.length)return Pe;if(1===o.length)return o[0];var s=Nd(o)+kd(t,r),c=_e.get(s);if(!c){if(1048576&a)if(xf(o))c=Df(o,t,r);else if(Sf(o,32768))c=ff([Df(o),Me],1,t,r);else if(Sf(o,65536))c=ff([Df(o),Ue],1,t,r);else{if(!Cf(o))return Re;var l=If(o),u=e.some(l,function(e){return!!(2097152&e.flags)})?pf(2097152,o):void 0;c=ff(l,1,t,r,u)}else c=Lf(o,t,r);_e.set(s,c)}return c}function Af(n){return e.reduceLeft(n,function(e,n){return 1048576&n.flags?e*n.types.length:131072&n.flags?0:e},1)}function Cf(n){var t=Af(n);return!(t>=1e5)||(null===e.tracing||void 0===e.tracing||e.tracing.instant("checkTypes","checkCrossProductUnion_DepthLimit",{typeIds:n.map(function(e){return e.id}),size:t}),Dr(d,e.Diagnostics.Expression_produces_a_union_type_that_is_too_complex_to_represent),!1)}function If(e){for(var n=Af(e),t=[],r=0;r<n;r++){for(var i=e.slice(),a=r,o=e.length-1;o>=0;o--)if(1048576&e[o].flags){var s=e[o].types,c=s.length;i[o]=s[a%c],a=Math.floor(a/c)}var l=Df(i);131072&l.flags||t.push(l)}return t}function Nf(n){var t=Wr(n);if(!t.resolvedType){var r=Bm(n);t.resolvedType=Df(e.map(n.types,u_),r,Um(r))}return t.resolvedType}function kf(e,n){var t=Ba(4194304);return t.type=e,t.stringsOnly=n,t}function Rf(e){var n=Ua(4194304);return n.type=e,n}function Of(e,n){return n?e.resolvedStringIndexType||(e.resolvedStringIndexType=kf(e,!0)):e.resolvedIndexType||(e.resolvedIndexType=kf(e,!1))}function wf(e,n,t){return z_(e,A_(n.mapper,Cl(n),t))}function Pf(n,t){var r=Sb(Il(n),function(e){return!(t&&5&e.flags)}),i=n.declaration.nameType&&u_(n.declaration.nameType),a=i&&Eb(r,function(e){return!!(131084&e.flags)})&&Hl(mu(wl(n)));return i?ff([Db(r,function(e){return wf(i,n,e)}),Db(ff(e.map(a||e.emptyArray,function(e){return Gf(e,8576)})),function(e){return wf(i,n,e)})]):r}function Mf(n){var t=Cl(n);return r(Nl(n)||t);function r(n){return!!(68157439&n.flags)||(16777216&n.flags?n.root.isDistributive&&n.checkType===t:137363456&n.flags?e.every(n.types,r):8388608&n.flags?r(n.objectType)&&r(n.indexType):33554432&n.flags?r(n.substitute):!!(268435456&n.flags)&&r(n.type))}}function Ff(n){return e.isPrivateIdentifier(n)?Ze:e.isIdentifier(n)?Zm(e.unescapeLeadingUnderscores(n.escapedText)):Ym(e.isComputedPropertyName(n)?WT(n):eC(n))}function Gf(n,t,r){if(r||!(24&e.getDeclarationModifierFlagsFromSymbol(n))){var i=Hr(qc(n)).nameType;if(!i){var a=e.getNameOfDeclaration(n.valueDeclaration);i="default"===n.escapedName?Zm("default"):a&&Ff(a)||(e.isKnownSymbol(n)?void 0:Zm(e.symbolName(n)))}if(i&&i.flags&t)return i}return Ze}function Bf(n,t,r){var i=r&&(7&e.getObjectFlags(n)||n.aliasSymbol)?Rf(n):void 0,a=e.map(Hl(n),function(e){return Gf(e,t)}),o=e.map(Ru(n),function(e){return e!==xt&&e.keyType.flags&t?e.keyType===je&&8&t?an:e.keyType:Ze});return ff(e.concatenate(a,o),1,void 0,void 0,i)}function Uf(n,t,r){return void 0===t&&(t=$),n=vu(n),1048576&n.flags?Df(e.map(n.types,function(e){return Uf(e,t,r)})):2097152&n.flags?ff(e.map(n.types,function(e){return Uf(e,t,r)})):58982400&n.flags||qy(n)||Bl(n)&&!Mf(n)?Of(n,t):32&e.getObjectFlags(n)?Pf(n,r):n===ke?ke:2&n.flags?Ze:131073&n.flags?sn:Bf(n,(r?128:402653316)|(t?0:12584),t===$&&!r)}function Vf(e){if($)return e;var n=Np();return n?Bd(n,[e,je]):je}function jf(e){var n=Vf(Uf(e));return 131072&n.flags?je:n}function Kf(n){var t=Wr(n);if(!t.resolvedType)switch(n.operator){case 139:t.resolvedType=Uf(u_(n.type));break;case 152:t.resolvedType=149===n.type.kind?i_(e.walkUpParenthesizedTypes(n.parent)):Re;break;case 143:t.resolvedType=u_(n.type);break;default:throw e.Debug.assertNever(n.operator)}return t.resolvedType}function Hf(n){var t=Wr(n);return t.resolvedType||(t.resolvedType=Wf(a([n.head.text],e.map(n.templateSpans,function(e){return e.literal.text}),!0),e.map(n.templateSpans,function(e){return u_(e.type)}))),t.resolvedType}function Wf(n,t){var r=e.findIndex(t,function(e){return!!(1179648&e.flags)});if(r>=0)return Cf(t)?Db(t[r],function(i){return Wf(n,e.replaceElement(t,r,i))}):Re;if(e.contains(t,ke))return ke;var i=[],a=[],o=n[0];if(!l(n,t))return je;if(0===i.length)return Zm(o);if(a.push(o),e.every(a,function(e){return""===e})&&e.every(i,function(e){return!!(4&e.flags)}))return je;var s=Nd(i)+"|"+e.map(a,function(e){return e.length}).join(",")+"|"+a.join(""),c=Ee.get(s);return c||Ee.set(s,c=Jf(a,i)),c;function l(e,n){for(var t=0;t<n.length;t++){var r=n[t];if(101248&r.flags)o+=zf(r)||"",o+=e[t+1];else if(134217728&r.flags){if(o+=r.texts[0],!l(r.texts,r.types))return!1;o+=e[t+1]}else{if(!cm(r)&&!im(r))return!1;i.push(r),a.push(o),o=e[t+1]}}return!0}}function zf(n){return 128&n.flags?n.value:256&n.flags?""+n.value:2048&n.flags?e.pseudoBigIntToString(n.value):98816&n.flags?n.intrinsicName:void 0}function Jf(e,n){var t=Ba(134217728);return t.texts=e,t.types=n,t}function qf(e,n){return 1179648&n.flags?Db(n,function(n){return qf(e,n)}):cm(n)?Yf(e,n):128&n.flags?Zm(Xf(e,n.value)):n}function Xf(e,n){switch(N.get(e.escapedName)){case 0:return n.toUpperCase();case 1:return n.toLowerCase();case 2:return n.charAt(0).toUpperCase()+n.slice(1);case 3:return n.charAt(0).toLowerCase()+n.slice(1)}return n}function Yf(e,n){var t=w(e)+","+nf(n),r=Te.get(t);return r||Te.set(t,r=Qf(e,n)),r}function Qf(e,n){var t=Ba(268435456);return t.symbol=e,t.type=n,t}function Zf(e,n,t,r,i){var a=Ba(8388608);return a.objectType=e,a.indexType=n,a.accessFlags=t,a.aliasSymbol=r,a.aliasTypeArguments=i,a}function $f(n){if(Y)return!1;if(8192&e.getObjectFlags(n))return!0;if(1048576&n.flags)return e.every(n.types,$f);if(2097152&n.flags)return e.some(n.types,$f);if(465829888&n.flags){var t=ou(n);return t!==n&&$f(t)}return!1}function em(n,t){return Fc(n)?Kc(n):t&&e.isPropertyName(t)?e.getPropertyNameForPropertyNameNode(t):void 0}function nm(n,t){if(8208&t.flags){var r=e.findAncestor(n.parent,function(n){return!e.isAccessExpression(n)})||n.parent;return e.isCallLikeExpression(r)?e.isCallOrNewExpression(r)&&e.isIdentifier(n)&&Vv(r,n):e.every(t.declarations,function(n){return!e.isFunctionLike(n)||!!(134217728&e.getCombinedNodeFlags(n))})}return!0}function tm(n,t,r,i,a,o){var s,c=a&&205===a.kind?a:void 0,l=a&&e.isPrivateIdentifier(a)?void 0:em(r,a);if(void 0!==l){if(256&o)return aT(t,l)||Ie;var u=Lu(t,l);if(u){if(64&o&&a&&u.declarations&&134217728&kS(u)&&nm(a,u)){var d=null!==(s=null===c||void 0===c?void 0:c.argumentExpression)&&void 0!==s?s:e.isIndexedAccessTypeNode(a)?a.indexType:a;kr(d,u.declarations,l)}if(c){if(bx(u,c,Ex(c.expression,t.symbol)),qD(c,u,e.getAssignmentTargetKind(c)))return void Dr(c.argumentExpression,e.Diagnostics.Cannot_assign_to_0_because_it_is_a_read_only_property,mo(u));if(8&o&&(Wr(a).resolvedSymbol=u),QS(c,u))return Ne}var p=Ws(u);return c&&1!==e.getAssignmentTargetKind(c)?nE(c,p):p}if(Eb(t,Jy)&&HT(l)&&+l>=0){if(a&&Eb(t,function(e){return!e.target.hasRestElement})&&!(16&o)){var f=rm(a);Jy(t)?Dr(f,e.Diagnostics.Tuple_type_0_of_length_1_has_no_element_at_index_2,go(t),Fd(t),e.unescapeLeadingUnderscores(l)):Dr(f,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.unescapeLeadingUnderscores(l),go(t))}return E(Ou(t,Ke)),Db(t,function(e){var n=Yy(e)||Me;return 1&o?ff([n,Me]):n})}}if(!(98304&r.flags)&&aA(r,402665900)){if(131073&t.flags)return t;var m=Mu(t,r)||Ou(t,je);if(m){if(2&o&&m.keyType!==Ke)return void(c&&Dr(c,e.Diagnostics.Type_0_cannot_be_used_to_index_type_1,go(r),go(n)));if(a&&m.keyType===je&&!aA(r,12)){f=rm(a);return Dr(f,e.Diagnostics.Type_0_cannot_be_used_as_an_index_type,go(r)),1&o?ff([m.type,Me]):m.type}return E(m),1&o?ff([m.type,Me]):m.type}if(131072&r.flags)return Ze;if($f(t))return Ie;if(c&&!sA(t)){if(_v(t)){if(Y&&384&r.flags)return ur.add(e.createDiagnosticForNode(c,e.Diagnostics.Property_0_does_not_exist_on_type_1,r.value,go(t))),Me;if(12&r.flags){var _=e.map(t.properties,function(e){return Ws(e)});return ff(e.append(_,Me))}}if(t.symbol===se&&void 0!==l&&se.exports.has(l)&&418&se.exports.get(l).flags)Dr(c,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.unescapeLeadingUnderscores(l),go(t));else if(Y&&!M.suppressImplicitAnyIndexErrors&&!(128&o))if(void 0!==l&&sx(l,t)){var g=go(t);Dr(c,e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_to_access_the_static_member_2_instead,l,g,g+"["+e.getTextOfNode(c.argumentExpression)+"]")}else if(wu(t,Ke))Dr(c.argumentExpression,e.Diagnostics.Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number);else{var y=void 0;if(void 0!==l&&(y=fx(l,t)))void 0!==y&&Dr(c.argumentExpression,e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2,l,go(t),y);else{var h=hx(t,c,r);if(void 0!==h)Dr(c,e.Diagnostics.Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature_Did_you_mean_to_call_1,go(t),h);else{var v=void 0;if(1024&r.flags)v=e.chainDiagnosticMessages(void 0,e.Diagnostics.Property_0_does_not_exist_on_type_1,"["+go(r)+"]",go(t));else if(8192&r.flags){var b=ta(r.symbol,c);v=e.chainDiagnosticMessages(void 0,e.Diagnostics.Property_0_does_not_exist_on_type_1,"["+b+"]",go(t))}else 128&r.flags?v=e.chainDiagnosticMessages(void 0,e.Diagnostics.Property_0_does_not_exist_on_type_1,r.value,go(t)):256&r.flags?v=e.chainDiagnosticMessages(void 0,e.Diagnostics.Property_0_does_not_exist_on_type_1,r.value,go(t)):12&r.flags&&(v=e.chainDiagnosticMessages(void 0,e.Diagnostics.No_index_signature_with_a_parameter_of_type_0_was_found_on_type_1,go(r),go(t)));v=e.chainDiagnosticMessages(v,e.Diagnostics.Element_implicitly_has_an_any_type_because_expression_of_type_0_can_t_be_used_to_index_type_1,go(i),go(t)),ur.add(e.createDiagnosticForNodeFromMessageChain(c,v))}}}return}}if($f(t))return Ie;if(a){f=rm(a);384&r.flags?Dr(f,e.Diagnostics.Property_0_does_not_exist_on_type_1,""+r.value,go(t)):12&r.flags?Dr(f,e.Diagnostics.Type_0_has_no_matching_index_signature_for_type_1,go(t),go(r)):Dr(f,e.Diagnostics.Type_0_cannot_be_used_as_an_index_type,go(r))}return Vo(r)?r:void 0;function E(n){n&&n.isReadonly&&c&&(e.isAssignmentTarget(c)||e.isDeleteTarget(c))&&Dr(c,e.Diagnostics.Index_signature_in_type_0_only_permits_reading,go(t))}}function rm(e){return 205===e.kind?e.argumentExpression:192===e.kind?e.indexType:160===e.kind?e.expression:e}function im(e){return!!(77&e.flags)}function am(n){return!!(134217728&n.flags)&&e.every(n.types,im)}function om(e){return!!lm(e)}function sm(e){return!!(8388608&lm(e))}function cm(e){return!!(16777216&lm(e))}function lm(n){return 3145728&n.flags?(4194304&n.objectFlags||(n.objectFlags|=4194304|e.reduceLeft(n.types,function(e,n){return e|lm(n)},0)),25165824&n.objectFlags):33554432&n.flags?(4194304&n.objectFlags||(n.objectFlags|=4194304|lm(n.substitute)|lm(n.baseType)),25165824&n.objectFlags):(58982400&n.flags||Bl(n)||qy(n)?8388608:0)|(465829888&n.flags&&!am(n)?16777216:0)}function um(e){return!!(262144&e.flags&&e.isThisType)}function dm(e,n){return 8388608&e.flags?mm(e,n):16777216&e.flags?gm(e,n):e}function pm(n,t,r){if(3145728&n.flags){var i=e.map(n.types,function(e){return dm(vm(e,t),r)});return 2097152&n.flags||r?Df(i):ff(i)}}function fm(n,t,r){if(1048576&t.flags){var i=e.map(t.types,function(e){return dm(vm(n,e),r)});return r?Df(i):ff(i)}}function mm(e,n){var t=n?"simplifiedForWriting":"simplifiedForReading";if(e[t])return e[t]===vn?e:e[t];e[t]=vn;var r=dm(e.objectType,n),i=dm(e.indexType,n),a=fm(r,i,n);if(a)return e[t]=a;if(!(465829888&i.flags)){var o=pm(r,i,n);if(o)return e[t]=o}if(qy(r)&&296&i.flags){var s=Zy(r,8&i.flags?0:r.target.fixedLength,0,n);if(s)return e[t]=s}return Bl(r)?e[t]=Db(hm(r,e.indexType),function(e){return dm(e,n)}):e[t]=e}function _m(n){var t=n.root.inferTypeParameters&&g_(n.root.inferTypeParameters,e.map(n.root.inferTypeParameters,function(){return ke})),r=n.checkType,i=n.extendsType;return lg(Q_(r),Q_(z_(i,t)))}function gm(e,n){var t=e.checkType,r=e.extendsType,i=Im(e),a=Nm(e);if(131072&a.flags&&xm(i)===xm(t)){if(1&t.flags||lg(Q_(t),Q_(r)))return dm(i,n);if(ym(t,r))return Ze}else if(131072&i.flags&&xm(a)===xm(t)){if(!(1&t.flags)&&lg(Q_(t),Q_(r)))return Ze;if(1&t.flags||ym(t,r))return dm(a,n)}return e}function ym(e,n){return!!(131072&ff([_l(e,n),Ze]).flags)}function hm(e,n){var t=g_([Cl(e)],[n]),r=x_(e.mapper,t);return z_(kl(e),r)}function vm(e,n,t,r,i,a){return void 0===t&&(t=0),Em(e,n,t,r,i,a)||(r?Re:Pe)}function bm(e,n){return Eb(e,function(e){if(384&e.flags){var t=Kc(e);if(HT(t)){var r=+t;return r>=0&&r<n}}return!1})}function Em(e,n,t,r,i,a){if(void 0===t&&(t=0),e===ke||n===ke)return ke;if(!Ug(e)||98304&n.flags||!aA(n,12)||(n=je),M.noUncheckedIndexedAccess&&32&t&&(t|=1),cm(n)||(r&&192!==r.kind?qy(e)&&!bm(n,e.target.fixedLength):sm(e)&&(!Jy(e)||!bm(n,e.target.fixedLength)))){if(3&e.flags)return e;var o=1&t,s=e.id+","+n.id+","+o+kd(i,a),c=be.get(s);return c||be.set(s,c=Zf(e,n,o,i,a)),c}var l=_u(e);if(1048576&n.flags&&!(16&n.flags)){for(var u=[],d=!1,p=0,f=n.types;p<f.length;p++){var m=f[p],_=tm(e,l,m,n,r,t|(d?128:0));if(_)u.push(_);else{if(!r)return;d=!0}}if(d)return;return 4&t?Df(u,i,a):ff(u,1,i,a)}return tm(e,l,n,n,r,72|t)}function Tm(e){var n=Wr(e);if(!n.resolvedType){var t=u_(e.objectType),r=u_(e.indexType),i=Bm(e),a=vm(t,r,0,e,i,Um(i));n.resolvedType=8388608&a.flags&&a.objectType===t&&a.indexType===r?Xd(a,e):a}return n.resolvedType}function Sm(e){var n=Wr(e);if(!n.resolvedType){var t=ja(32,e.symbol);t.declaration=e,t.aliasSymbol=Bm(e),t.aliasTypeArguments=Um(t.aliasSymbol),n.resolvedType=t,Il(t)}return n.resolvedType}function xm(e){return 33554432&e.flags?e.baseType:8388608&e.flags&&(33554432&e.objectType.flags||33554432&e.indexType.flags)?vm(xm(e.objectType),xm(e.indexType)):e}function Lm(e){return!e.isDistributive&&Dm(e.node.checkType)&&Dm(e.node.extendsType)}function Dm(n){return e.isTupleTypeNode(n)&&1===e.length(n.elements)&&!e.isOptionalTypeNode(n.elements[0])&&!e.isRestTypeNode(n.elements[0])}function Am(e,n){return Lm(e)&&Jy(n)?Md(n)[0]:n}function Cm(n,t,r,i){var a,o;while(1){var s=Lm(n),c=z_(Am(n,xm(n.checkType)),t),l=om(c),u=z_(Am(n,n.extendsType),t);if(c===ke||u===ke)return ke;var d=void 0;if(n.inferTypeParameters){var p=Ph(n.inferTypeParameters,void 0,0);l||dv(p.inferences,c,u,1536),d=t?x_(p.mapper,t):p.mapper}var f=d?z_(Am(n,n.extendsType),d):u;if(!l&&!om(f)){if(!(3&f.flags)&&(1&c.flags&&!s||!lg(Y_(c),Y_(f)))){1&c.flags&&!s&&(o||(o=[])).push(z_(u_(n.node.trueType),d||t));var m=u_(n.node.falseType);if(16777216&m.flags){var _=m.root;if(_.node.parent===n.node&&(!_.isDistributive||_.checkType===n.checkType)){n=_;continue}}a=z_(m,t);break}if(3&f.flags||lg(Q_(c),Q_(f))){a=z_(u_(n.node.trueType),d||t);break}}a=Ba(16777216),a.root=n,a.checkType=z_(n.checkType,t),a.extendsType=z_(n.extendsType,t),a.mapper=t,a.combinedMapper=d,a.aliasSymbol=r||n.aliasSymbol,a.aliasTypeArguments=r?i:f_(n.aliasTypeArguments,t);break}return o?ff(e.append(o,a)):a}function Im(e){return e.resolvedTrueType||(e.resolvedTrueType=z_(u_(e.root.node.trueType),e.mapper))}function Nm(e){return e.resolvedFalseType||(e.resolvedFalseType=z_(u_(e.root.node.falseType),e.mapper))}function km(e){return e.resolvedInferredTrueType||(e.resolvedInferredTrueType=e.combinedMapper?z_(u_(e.root.node.trueType),e.combinedMapper):Im(e))}function Rm(n){var t;return n.locals&&n.locals.forEach(function(n){262144&n.flags&&(t=e.append(t,Dc(n)))}),t}function Om(n){var t=Wr(n);if(!t.resolvedType){var r=u_(n.checkType),i=Bm(n),a=Um(i),o=Qs(n,!0),s=a?o:e.filter(o,function(e){return P_(e,n)}),c={node:n,checkType:r,extendsType:u_(n.extendsType),isDistributive:!!(262144&r.flags),inferTypeParameters:Rm(n),outerTypeParameters:s,instantiations:void 0,aliasSymbol:i,aliasTypeArguments:a};t.resolvedType=Cm(c,void 0),s&&(c.instantiations=new e.Map,c.instantiations.set(Nd(s),t.resolvedType))}return t.resolvedType}function wm(e){var n=Wr(e);return n.resolvedType||(n.resolvedType=xc(Ca(e.typeParameter))),n.resolvedType}function Pm(n){return e.isIdentifier(n)?[n]:e.append(Pm(n.left),n.right)}function Mm(n){var t=Wr(n);if(!t.resolvedType){if(n.isTypeOf&&n.typeArguments)return Dr(n,e.Diagnostics.Type_arguments_cannot_be_used_here),t.resolvedSymbol=Ae,t.resolvedType=Re;if(!e.isLiteralImportTypeNode(n))return Dr(n.argument,e.Diagnostics.String_literal_expected),t.resolvedSymbol=Ae,t.resolvedType=Re;var r=n.isTypeOf?111551:4194304&n.flags?900095:788968,i=ca(n,n.argument.literal);if(!i)return t.resolvedSymbol=Ae,t.resolvedType=Re;var a=fa(i,!1);if(e.nodeIsMissing(n.qualifier))if(a.flags&r)t.resolvedType=Fm(n,t,a,r);else{var o=111551===r?e.Diagnostics.Module_0_does_not_refer_to_a_value_but_is_used_as_a_value_here:e.Diagnostics.Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0;Dr(n,o,n.argument.literal.text),t.resolvedSymbol=Ae,t.resolvedType=Re}else{var s=Pm(n.qualifier),c=a,l=void 0;while(l=s.shift()){var u=s.length?1920:r,d=Aa(zi(c)),p=n.isTypeOf?Lu(Ws(d),l.escapedText):Jr(Sa(d),l.escapedText,u);if(!p)return Dr(l,e.Diagnostics.Namespace_0_has_no_exported_member_1,ta(c),e.declarationNameToString(l)),t.resolvedType=Re;Wr(l).resolvedSymbol=p,Wr(l.parent).resolvedSymbol=p,c=p}t.resolvedType=Fm(n,t,c,r)}}return t.resolvedType}function Fm(e,n,t,r){var i=zi(t);return n.resolvedSymbol=i,111551===r?Ws(t):Hd(e,i)}function Gm(n){var t=Wr(n);if(!t.resolvedType){var r=Bm(n);if(0!==Jc(n.symbol).size||r){var i=ja(16,n.symbol);i.aliasSymbol=r,i.aliasTypeArguments=Um(r),e.isJSDocTypeLiteral(n)&&n.isArrayType&&(i=Mp(i)),t.resolvedType=i}else t.resolvedType=_n}return t.resolvedType}function Bm(n){var t=n.parent;while(e.isParenthesizedTypeNode(t)||e.isJSDocTypeExpression(t)||e.isTypeOperatorNode(t)&&143===t.operator)t=t.parent;return e.isTypeAlias(t)?Ca(t):void 0}function Um(e){return e?$s(e):void 0}function Vm(e){return!!(524288&e.flags)&&!Bl(e)}function jm(e){return Gg(e)||!!(474058748&e.flags)}function Km(n,t){if(!(1048576&n.flags))return n;if(e.every(n.types,jm))return e.find(n.types,Gg)||pn;var r=e.find(n.types,function(e){return!jm(e)});if(!r)return n;var i=e.find(n.types,function(e){return e!==r&&!jm(e)});return i?n:a(r);function a(n){for(var r=e.createSymbolTable(),i=0,a=Hl(n);i<a.length;i++){var o=a[i];if(24&e.getDeclarationModifierFlagsFromSymbol(o));else if(Wm(o)){var s=65536&o.flags&&!(32768&o.flags),c=16777220,l=Or(c,o.escapedName,Ll(o)|(t?8:0));l.type=s?Me:ns(Ws(o),!0),l.declarations=o.declarations,l.nameType=Hr(o).nameType,l.syntheticOrigin=o,r.set(o.escapedName,l)}}var u=Xa(n.symbol,r,e.emptyArray,e.emptyArray,Ru(n));return u.objectFlags|=262272,u}}function Hm(n,t,r,i,a){if(1&n.flags||1&t.flags)return Ie;if(2&n.flags||2&t.flags)return Pe;if(131072&n.flags)return t;if(131072&t.flags)return n;if(n=Km(n,a),1048576&n.flags)return Cf([n,t])?Db(n,function(e){return Hm(e,t,r,i,a)}):Re;if(t=Km(t,a),1048576&t.flags)return Cf([n,t])?Db(t,function(e){return Hm(n,e,r,i,a)}):Re;if(473960444&t.flags)return n;if(sm(n)||sm(t)){if(Gg(n))return t;if(2097152&n.flags){var o=n.types,s=o[o.length-1];if(Vm(s)&&Vm(t))return Df(e.concatenate(o.slice(0,o.length-1),[Hm(s,t,r,i,a)]))}return Df([n,t])}for(var c=e.createSymbolTable(),l=new e.Set,u=n===pn?Ru(t):fl([n,t]),d=0,p=Hl(t);d<p.length;d++){var f=p[d];24&e.getDeclarationModifierFlagsFromSymbol(f)?l.add(f.escapedName):Wm(f)&&c.set(f.escapedName,zm(f,a))}for(var m=0,_=Hl(n);m<_.length;m++){var g=_[m];if(!l.has(g.escapedName)&&Wm(g))if(c.has(g.escapedName)){f=c.get(g.escapedName);var y=Ws(f);if(16777216&f.flags){var h=e.concatenate(g.declarations,f.declarations),v=4|16777216&g.flags,b=Or(v,g.escapedName);b.type=ff([Ws(g),gh(y)]),b.leftSpread=g,b.rightSpread=f,b.declarations=h,b.nameType=Hr(g).nameType,c.set(g.escapedName,b)}}else c.set(g.escapedName,zm(g,a))}var E=Xa(r,c,e.emptyArray,e.emptyArray,e.sameMap(u,function(e){return Jm(e,a)}));return E.objectFlags|=4456576|i,E}function Wm(n){var t;return!e.some(n.declarations,e.isPrivateIdentifierClassElementDeclaration)&&(!(106496&n.flags)||!(null===(t=n.declarations)||void 0===t?void 0:t.some(function(n){return e.isClassLike(n.parent)})))}function zm(e,n){var t=65536&e.flags&&!(32768&e.flags);if(!t&&n===JD(e))return e;var r=4|16777216&e.flags,i=Or(r,e.escapedName,Ll(e)|(n?8:0));return i.type=t?Me:Ws(e),i.declarations=e.declarations,i.nameType=Hr(e).nameType,i.syntheticOrigin=e,i}function Jm(e,n){return e.isReadonly!==n?Td(e.keyType,e.type,n,e.declaration):e}function qm(e,n,t,r){var i=Ba(e);return i.symbol=t,i.value=n,i.regularType=r||i,i}function Xm(e){if(2944&e.flags){if(!e.freshType){var n=qm(e.flags,e.value,e.symbol,e);n.freshType=n,e.freshType=n}return e.freshType}return e}function Ym(e){return 2944&e.flags?e.regularType:1048576&e.flags?e.regularType||(e.regularType=Db(e,Ym)):e}function Qm(e){return!!(2944&e.flags)&&e.freshType===e}function Zm(e){var n;return ge.get(e)||(ge.set(e,n=qm(128,e)),n)}function $m(e){var n;return ye.get(e)||(ye.set(e,n=qm(256,e)),n)}function e_(n){var t,r=e.pseudoBigIntToString(n);return he.get(r)||(he.set(r,t=qm(2048,n)),t)}function n_(e,n,t){var r,i="string"===typeof e?"@":"#",a=n+i+e,o=1024|("string"===typeof e?128:256);return ve.get(a)||(ve.set(a,r=qm(o,e,t)),r)}function t_(e){if(104===e.literal.kind)return Ue;var n=Wr(e);return n.resolvedType||(n.resolvedType=Ym(eC(e.literal))),n.resolvedType}function r_(e){var n=Ba(8192);return n.symbol=e,n.escapedName="__@"+n.symbol.escapedName+"@"+w(n.symbol),n}function i_(n){if(e.isValidESSymbolDeclaration(n)){var t=Ca(n),r=Hr(t);return r.uniqueESSymbolType||(r.uniqueESSymbolType=r_(t))}return Ye}function a_(n){var t=e.getThisContainer(n,!1),r=t&&t.parent;if(r&&(e.isClassLike(r)||256===r.kind)&&!e.isStatic(t)&&(!e.isConstructorDeclaration(t)||e.isNodeDescendantOf(n,t.body)))return gc(Ca(r)).thisType;if(r&&e.isObjectLiteralExpression(r)&&e.isBinaryExpression(r.parent)&&6===e.getAssignmentDeclarationKind(r.parent))return gc(Ca(r.parent.left).parent).thisType;var i=4194304&n.flags?e.getHostSignatureFromJSDoc(n):void 0;return i&&e.isFunctionExpression(i)&&e.isBinaryExpression(i.parent)&&3===e.getAssignmentDeclarationKind(i.parent)?gc(Ca(i.parent.left).parent).thisType:FL(t)&&e.isNodeDescendantOf(n,t.body)?gc(Ca(t)).thisType:(Dr(n,e.Diagnostics.A_this_type_is_available_only_in_a_non_static_member_of_a_class_or_interface),Re)}function o_(e){var n=Wr(e);return n.resolvedType||(n.resolvedType=a_(e)),n.resolvedType}function s_(e){return u_(c_(e.type)||e.type)}function c_(e){switch(e.kind){case 189:return c_(e.type);case 182:if(1===e.elements.length&&(e=e.elements[0],184===e.kind||195===e.kind&&e.dotDotDotToken))return c_(e.type);break;case 181:return e.elementType}}function l_(e){var n=Wr(e);return n.resolvedType||(n.resolvedType=e.dotDotDotToken?s_(e):ns(u_(e.type),!0,!!e.questionToken))}function u_(e){return Xd(d_(e),e)}function d_(e){switch(e.kind){case 129:case 307:case 308:return Ie;case 153:return Pe;case 148:return je;case 145:return Ke;case 156:return He;case 132:return Xe;case 149:return Ye;case 114:return Qe;case 151:return Me;case 104:return Ue;case 142:return Ze;case 146:return 131072&e.flags&&!Y?Ie:rn;case 137:return we;case 190:case 108:return o_(e);case 194:return t_(e);case 176:return ep(e);case 175:return e.assertsModifier?Qe:Xe;case 226:return ep(e);case 179:return tp(e);case 181:case 182:return Kp(e);case 183:return ef(e);case 185:return yf(e);case 186:return Nf(e);case 309:return $d(e);case 311:return ns(u_(e.type));case 195:return l_(e);case 189:case 310:case 304:return u_(e.type);case 184:return s_(e);case 313:return tR(e);case 177:case 178:case 180:case 317:case 312:case 318:return Gm(e);case 191:return Kf(e);case 192:return Tm(e);case 193:return Sm(e);case 187:return Om(e);case 188:return wm(e);case 196:return Hf(e);case 198:return Mm(e);case 79:case 159:case 204:var n=CR(e);return n?Dc(n):Re;default:return Re}}function p_(e,n,t){if(e&&e.length)for(var r=0;r<e.length;r++){var i=e[r],a=t(i,n);if(i!==a){var o=0===r?[]:e.slice(0,r);for(o.push(a),r++;r<e.length;r++)o.push(t(e[r],n));return o}}return e}function f_(e,n){return p_(e,n,z_)}function m_(e,n){return p_(e,n,k_)}function __(e,n){return p_(e,n,Z_)}function g_(e,n){return 1===e.length?h_(e[0],n?n[0]:Ie):v_(e,n)}function y_(e,n){switch(n.kind){case 0:return e===n.source?n.target:e;case 1:for(var t=n.sources,r=n.targets,i=0;i<t.length;i++)if(e===t[i])return r?r[i]:Ie;return e;case 2:return n.func(e);case 3:case 4:var a=y_(e,n.mapper1);return a!==e&&3===n.kind?z_(a,n.mapper2):y_(a,n.mapper2)}}function h_(e,n){return{kind:0,source:e,target:n}}function v_(e,n){return{kind:1,sources:e,targets:n}}function b_(e){return{kind:2,func:e}}function E_(e,n,t){return{kind:e,mapper1:n,mapper2:t}}function T_(e){return g_(e,void 0)}function S_(n,t){return b_(function(r){return e.findIndex(n.inferences,function(e){return e.typeParameter===r})>=t?Pe:r})}function x_(e,n){return e?E_(3,e,n):n}function L_(e,n){return e?E_(4,e,n):n}function D_(e,n,t){return t?E_(4,h_(e,n),t):h_(e,n)}function A_(e,n,t){return e?E_(4,e,h_(n,t)):h_(n,t)}function C_(e){return e.constraint===Pe?e:e.restrictiveInstantiation||(e.restrictiveInstantiation=Ha(e.symbol),e.restrictiveInstantiation.constraint=Pe,e.restrictiveInstantiation)}function I_(e){var n=Ha(e.symbol);return n.target=e,n}function N_(e,n){return Wu(e.kind,e.parameterName,e.parameterIndex,z_(e.type,n))}function k_(n,t,r){var i;if(n.typeParameters&&!r){i=e.map(n.typeParameters,I_),t=x_(g_(n.typeParameters,i),t);for(var a=0,o=i;a<o.length;a++){var s=o[a];s.mapper=t}}var c=$c(n.declaration,i,n.thisParameter&&R_(n.thisParameter,t),p_(n.parameters,t,R_),void 0,void 0,n.minArgumentCount,39&n.flags);return c.target=n,c.mapper=t,c}function R_(n,t){var r=Hr(n);if(r.type&&!Hh(r.type))return n;1&e.getCheckFlags(n)&&(n=r.target,t=x_(r.mapper,t));var i=Or(n.flags,n.escapedName,1|53256&e.getCheckFlags(n));return i.declarations=n.declarations,i.parent=n.parent,i.target=n,i.mapper=t,n.valueDeclaration&&(i.valueDeclaration=n.valueDeclaration),r.nameType&&(i.nameType=r.nameType),i}function O_(n,t,r,i){var a=4&n.objectFlags?n.node:n.symbol.declarations[0],o=Wr(a),s=4&n.objectFlags?o.resolvedType:64&n.objectFlags?n.target:n,c=o.outerTypeParameters;if(!c){var l=Qs(a,!0);if(FL(a)){var u=Gu(a);l=e.addRange(l,u)}c=l||e.emptyArray;var d=4&n.objectFlags?[a]:n.symbol.declarations;c=(4&s.objectFlags||8192&s.symbol.flags||2048&s.symbol.flags)&&!s.aliasTypeArguments?e.filter(c,function(n){return e.some(d,function(e){return P_(n,e)})}):c,o.outerTypeParameters=c}if(c.length){var p=x_(n.mapper,t),f=e.map(c,function(e){return y_(e,p)}),m=r||n.aliasSymbol,_=r?i:f_(n.aliasTypeArguments,t),g=Nd(f)+kd(m,_);s.instantiations||(s.instantiations=new e.Map,s.instantiations.set(Nd(c)+kd(s.aliasSymbol,s.aliasTypeArguments),s));var y=s.instantiations.get(g);if(!y){var h=g_(c,f);y=4&s.objectFlags?Pd(n.target,n.node,h,m,_):32&s.objectFlags?F_(s,h,m,_):K_(s,h,m,_),s.instantiations.set(g,y)}return y}return n}function w_(e){return!(176===e.parent.kind&&e.parent.typeArguments&&e===e.parent.typeName||198===e.parent.kind&&e.parent.typeArguments&&e===e.parent.qualifier)}function P_(n,t){if(n.symbol&&n.symbol.declarations&&1===n.symbol.declarations.length){for(var r=n.symbol.declarations[0].parent,i=t;i!==r;i=i.parent)if(!i||233===i.kind||187===i.kind&&e.forEachChild(i.extendsType,a))return!0;return a(t)}return!0;function a(t){switch(t.kind){case 190:return!!n.isThisType;case 79:return!n.isThisType&&e.isPartOfTypeNode(t)&&w_(t)&&d_(t)===n;case 179:return!0;case 167:case 166:return!t.type&&!!t.body||e.some(t.typeParameters,a)||e.some(t.parameters,a)||!!t.type&&a(t.type)}return!!e.forEachChild(t,a)}}function M_(e){var n=Il(e);if(4194304&n.flags){var t=xm(n.type);if(262144&t.flags)return t}}function F_(e,n,t,r){var i=M_(e);if(i){var a=z_(i,n);if(i!==a)return Ab(vu(a),function(t){if(61603843&t.flags&&t!==ke&&t!==Re){if(!e.declaration.nameType){if(Ly(t))return U_(t,e,D_(i,t,n));if(qy(t))return B_(t,e,i,n);if(Jy(t))return V_(t,e,D_(i,t,n))}return K_(e,D_(i,t,n))}return t},t,r)}return z_(Il(e),n)===ke?ke:K_(e,n,t,r)}function G_(e,n){return!!(1&n)||!(2&n)&&e}function B_(n,t,r,i){var a=n.target.elementFlags,o=e.map(Md(n),function(e,n){var o=8&a[n]?e:4&a[n]?Mp(e):Wp([e],[a[n]]);return F_(t,D_(r,o,i))}),s=G_(n.target.readonly,Pl(t));return Wp(o,e.map(o,function(e){return 8}),s)}function U_(e,n,t){var r=j_(n,Ke,!0,t);return r===Re?Re:Mp(r,G_(Dy(e),Pl(n)))}function V_(n,t,r){var i=n.target.elementFlags,a=e.map(Md(n),function(e,n){return j_(t,Zm(""+n),!!(2&i[n]),r)}),o=Pl(t),s=4&o?e.map(i,function(e){return 1&e?2:e}):8&o?e.map(i,function(e){return 2&e?1:e}):i,c=G_(n.target.readonly,o);return e.contains(a,Re)?Re:Wp(a,s,c,n.target.labeledElementDeclarations)}function j_(e,n,t,r){var i=A_(r,Cl(e),n),a=z_(kl(e.target||e),i),o=Pl(e);return z&&4&o&&!iA(a,49152)?sh(a,!0):z&&8&o&&t?Jv(a,524288):a}function K_(e,n,t,r){var i=ja(64|e.objectFlags,e.symbol);if(32&e.objectFlags){i.declaration=e.declaration;var a=Cl(e),o=I_(a);i.typeParameter=o,n=x_(h_(a,o),n),o.mapper=n}return i.target=e,i.mapper=n,i.aliasSymbol=t||e.aliasSymbol,i.aliasTypeArguments=t?r:f_(e.aliasTypeArguments,n),i}function H_(n,t,r,i){var a=n.root;if(a.outerTypeParameters){var o=e.map(a.outerTypeParameters,function(e){return y_(e,t)}),s=Nd(o)+kd(r,i),c=a.instantiations.get(s);if(!c){var l=g_(a.outerTypeParameters,o);c=W_(a,l,r,i),a.instantiations.set(s,c)}return c}return n}function W_(e,n,t,r){if(e.isDistributive){var i=e.checkType,a=y_(i,n);if(i!==a&&1179648&a.flags)return Ab(a,function(t){return Cm(e,D_(i,t,n))},t,r)}return Cm(e,n,t,r)}function z_(e,n){return e&&n?J_(e,n,void 0,void 0):e}function J_(n,t,r,i){if(!Hh(n))return n;if(50===L||x>=5e6)return null===e.tracing||void 0===e.tracing||e.tracing.instant("checkTypes","instantiateType_DepthLimit",{typeId:n.id,instantiationDepth:L,instantiationCount:x}),Dr(d,e.Diagnostics.Type_instantiation_is_excessively_deep_and_possibly_infinite),Re;E++,x++,L++;var a=q_(n,t,r,i);return L--,a}function q_(e,n,t,r){var i=e.flags;if(262144&i)return y_(e,n);if(524288&i){var a=e.objectFlags;if(52&a){if(4&a&&!e.node){var o=e.resolvedTypeArguments,s=f_(o,n);return s!==o?qp(e.target,s):e}return 1024&a?X_(e,n):O_(e,n,t,r)}return e}if(3145728&i){var c=1048576&e.flags?e.origin:void 0,l=c&&3145728&c.flags?c.types:e.types,u=f_(l,n);if(u===l&&t===e.aliasSymbol)return e;var d=t||e.aliasSymbol,p=t?r:f_(e.aliasTypeArguments,n);return 2097152&i||c&&2097152&c.flags?Df(u,d,p):ff(u,1,d,p)}if(4194304&i)return Uf(z_(e.type,n));if(134217728&i)return Wf(e.texts,f_(e.types,n));if(268435456&i)return qf(e.symbol,z_(e.type,n));if(8388608&i){d=t||e.aliasSymbol,p=t?r:f_(e.aliasTypeArguments,n);return vm(z_(e.objectType,n),z_(e.indexType,n),e.accessFlags,void 0,d,p)}if(16777216&i)return H_(e,x_(e.mapper,n),t,r);if(33554432&i){var f=z_(e.baseType,n);if(8650752&f.flags)return zd(f,z_(e.substitute,n));var m=z_(e.substitute,n);return 3&m.flags||lg(Q_(f),Q_(m))?f:m}return e}function X_(n,t){var r=z_(n.mappedType,t);if(!(32&e.getObjectFlags(r)))return n;var i=z_(n.constraintType,t);if(!(4194304&i.flags))return n;var a=qh(z_(n.source,t),r,i);return a||n}function Y_(e){return 262143&e.flags?e:e.permissiveInstantiation||(e.permissiveInstantiation=z_(e,dn))}function Q_(e){return 262143&e.flags?e:e.restrictiveInstantiation?e.restrictiveInstantiation:(e.restrictiveInstantiation=z_(e,un),e.restrictiveInstantiation.restrictiveInstantiation=e.restrictiveInstantiation,e.restrictiveInstantiation)}function Z_(e,n){return Td(e.keyType,z_(e.type,n),e.isReadonly,e.declaration)}function $_(n){switch(e.Debug.assert(167!==n.kind||e.isObjectLiteralMethod(n)),n.kind){case 211:case 212:case 167:case 254:return eg(n);case 203:return e.some(n.properties,$_);case 202:return e.some(n.elements,$_);case 220:return $_(n.whenTrue)||$_(n.whenFalse);case 219:return(56===n.operatorToken.kind||60===n.operatorToken.kind)&&($_(n.left)||$_(n.right));case 291:return $_(n.initializer);case 210:return $_(n.expression);case 284:return e.some(n.properties,$_)||e.isJsxOpeningElement(n.parent)&&e.some(n.parent.parent.children,$_);case 283:var t=n.initializer;return!!t&&$_(t);case 286:var r=n.expression;return!!r&&$_(r)}return!1}function eg(n){return(!e.isFunctionDeclaration(n)||e.isInJSFile(n)&&!!Zo(n))&&(e.hasContextSensitiveParameters(n)||ng(n))}function ng(n){return!n.typeParameters&&!e.getEffectiveReturnTypeNode(n)&&!!n.body&&233!==n.body.kind&&$_(n.body)}function tg(n){return(e.isInJSFile(n)&&e.isFunctionDeclaration(n)||wT(n)||e.isObjectLiteralMethod(n))&&eg(n)}function rg(n){if(524288&n.flags){var t=Ul(n);if(t.constructSignatures.length||t.callSignatures.length){var r=ja(16,n.symbol);return r.members=t.members,r.properties=t.properties,r.callSignatures=e.emptyArray,r.constructSignatures=e.emptyArray,r.indexInfos=e.emptyArray,r}}else if(2097152&n.flags)return Df(e.map(n.types,rg));return n}function ig(e,n){return Kg(e,n,hr)}function ag(e,n){return Kg(e,n,hr)?-1:0}function og(e,n){return Kg(e,n,gr)?-1:0}function sg(e,n){return Kg(e,n,mr)?-1:0}function cg(e,n){return Kg(e,n,mr)}function lg(e,n){return Kg(e,n,gr)}function ug(n,t){return 1048576&n.flags?e.every(n.types,function(e){return ug(e,t)}):1048576&t.flags?e.some(t.types,function(e){return ug(n,e)}):58982400&n.flags?ug(ru(n)||Pe,t):t===An?!!(67633152&n.flags):t===Cn?!!(524288&n.flags)&&Wv(n):Xs(n,qs(t))||Ly(t)&&!Dy(t)&&ug(n,Rn)}function dg(e,n){return Kg(e,n,yr)}function pg(e,n){return dg(e,n)||dg(n,e)}function fg(e,n,t,r,i,a){return zg(e,n,gr,t,r,i,a)}function mg(e,n,t,r,i,a){return _g(e,n,gr,t,r,i,a,void 0)}function _g(e,n,t,r,i,a,o,s){return!!Kg(e,n,t)||(!r||!yg(i,e,n,t,a,o,s))&&zg(e,n,t,r,a,o,s)}function gg(n){return!!(16777216&n.flags||2097152&n.flags&&e.some(n.types,gg))}function yg(e,n,t,r,i,a,o){if(!e||gg(t))return!1;if(!zg(n,t,r,void 0)&&hg(e,n,t,r,i,a,o))return!0;switch(e.kind){case 286:case 210:return yg(e.expression,n,t,r,i,a,o);case 219:switch(e.operatorToken.kind){case 63:case 27:return yg(e.right,n,t,r,i,a,o)}break;case 203:return Ng(e,n,t,r,a,o);case 202:return Cg(e,n,t,r,a,o);case 284:return Dg(e,n,t,r,a,o);case 212:return vg(e,n,t,r,a,o)}return!1}function hg(n,t,r,i,a,o,s){for(var c=Au(t,0),l=Au(t,1),u=0,d=[l,c];u<d.length;u++){var p=d[u];if(e.some(p,function(e){var n=ad(e);return!(131073&n.flags)&&zg(n,r,i,void 0)})){var f=s||{};fg(t,r,n,a,o,f);var m=f.errors[f.errors.length-1];return e.addRelatedInfo(m,e.createDiagnosticForNode(n,p===l?e.Diagnostics.Did_you_mean_to_use_new_with_this_expression:e.Diagnostics.Did_you_mean_to_call_this_expression)),!0}}return!1}function vg(n,t,r,i,a,o){if(e.isBlock(n.body))return!1;if(e.some(n.parameters,e.hasType))return!1;var s=Vx(t);if(!s)return!1;var c=Au(r,0);if(!e.length(c))return!1;var l=n.body,u=ad(s),d=ff(e.map(c,ad));if(!zg(u,d,i,void 0)){var p=l&&yg(l,u,d,i,void 0,a,o);if(p)return p;var f=o||{};if(zg(u,d,i,l,void 0,a,f),f.errors)return r.symbol&&e.length(r.symbol.declarations)&&e.addRelatedInfo(f.errors[f.errors.length-1],e.createDiagnosticForNode(r.symbol.declarations[0],e.Diagnostics.The_expected_type_comes_from_the_return_type_of_this_signature)),0===(2&e.getFunctionFlags(n))&&!Bo(u,"then")&&zg(AD(u),d,i,void 0)&&e.addRelatedInfo(f.errors[f.errors.length-1],e.createDiagnosticForNode(n,e.Diagnostics.Did_you_mean_to_mark_this_function_as_async)),!0}return!1}function bg(e,n,t){var r=Em(n,t);if(r)return r;if(1048576&n.flags){var i=qg(e,n);if(i)return Em(i,t)}}function Eg(e,n){e.contextualType=n;try{return GA(e,1,n)}finally{e.contextualType=void 0}}function Tg(n,t,r,i,a,o){for(var s=!1,c=n.next();!c.done;c=n.next()){var l=c.value,u=l.errorNode,d=l.innerExpression,p=l.nameType,f=l.errorMessage,m=bg(t,r,p);if(m&&!(8388608&m.flags)){var _=Em(t,p);if(_){var g=em(p,void 0),y=!!(g&&16777216&(Lu(r,g)||Ae).flags),h=!!(g&&16777216&(Lu(t,g)||Ae).flags);if(m=mh(m,y),_=mh(_,y&&h),!zg(_,m,i,void 0)){var v=d&&yg(d,_,m,i,void 0,a,o);if(v)s=!0;else{var b=o||{},E=d?Eg(d,_):_,T=zg(E,m,i,u,f,a,b);if(T&&E!==_&&zg(_,m,i,u,f,a,b),b.errors){var S=b.errors[b.errors.length-1],x=Fc(p)?Kc(p):void 0,L=void 0!==x?Lu(r,x):void 0,D=!1;if(!L){var A=Mu(r,p);A&&A.declaration&&!e.getSourceFileOfNode(A.declaration).hasNoDefaultLib&&(D=!0,e.addRelatedInfo(S,e.createDiagnosticForNode(A.declaration,e.Diagnostics.The_expected_type_comes_from_this_index_signature)))}if(!D&&(L&&e.length(L.declarations)||r.symbol&&e.length(r.symbol.declarations))){var C=L&&e.length(L.declarations)?L.declarations[0]:r.symbol.declarations[0];e.getSourceFileOfNode(C).hasNoDefaultLib||e.addRelatedInfo(S,e.createDiagnosticForNode(C,e.Diagnostics.The_expected_type_comes_from_property_0_which_is_declared_here_on_type_1,!x||8192&p.flags?go(p):e.unescapeLeadingUnderscores(x),go(r)))}}s=!0}}}}}return s}function Sg(n){var t,r,i;return c(this,function(a){switch(a.label){case 0:if(!e.length(n.properties))return[2];t=0,r=n.properties,a.label=1;case 1:return t<r.length?(i=r[t],e.isJsxSpreadAttribute(i)||rS(e.idText(i.name))?[3,3]:[4,{errorNode:i.name,innerExpression:i.initializer,nameType:Zm(e.idText(i.name))}]):[3,4];case 2:a.sent(),a.label=3;case 3:return t++,[3,1];case 4:return[2]}})}function xg(n,t){var r,i,a,o,s;return c(this,function(c){switch(c.label){case 0:if(!e.length(n.children))return[2];r=0,i=0,c.label=1;case 1:return i<n.children.length?(a=n.children[i],o=$m(i-r),s=Lg(a,o,t),s?[4,s]:[3,3]):[3,5];case 2:return c.sent(),[3,4];case 3:r++,c.label=4;case 4:return i++,[3,1];case 5:return[2]}})}function Lg(n,t,r){switch(n.kind){case 286:return{errorNode:n,innerExpression:n.expression,nameType:t};case 11:if(n.containsOnlyTriviaWhiteSpaces)break;return{errorNode:n,innerExpression:void 0,nameType:t,errorMessage:r()};case 276:case 277:case 280:return{errorNode:n,innerExpression:n,nameType:t};default:return e.Debug.assertNever(n,"Found invalid jsx child")}}function Dg(n,t,r,i,a,s){var l,u=Tg(Sg(n),t,r,i,a,s);if(e.isJsxOpeningElement(n.parent)&&e.isJsxElement(n.parent.parent)){var d=n.parent.parent,p=yS(fS(n)),f=void 0===p?"children":e.unescapeLeadingUnderscores(p),m=Zm(f),_=vm(r,m),g=e.getSemanticJsxChildren(d.children);if(!e.length(g))return u;var y=e.length(g)>1,h=Sb(_,wy),v=Sb(_,function(e){return!wy(e)});if(y){if(h!==Ze){var b=Wp(sS(d,0)),E=xg(d,L);u=Tg(E,b,h,i,a,s)||u}else if(!Kg(vm(t,m),_,i)){u=!0;var T=Dr(d.openingElement.tagName,e.Diagnostics.This_JSX_tag_s_0_prop_expects_a_single_child_of_type_1_but_multiple_children_were_provided,f,go(_));s&&s.skipLogging&&(s.errors||(s.errors=[])).push(T)}}else if(v!==Ze){var S=g[0],x=Lg(S,m,L);x&&(u=Tg(function(){return c(this,function(e){switch(e.label){case 0:return[4,x];case 1:return e.sent(),[2]}})}(),t,r,i,a,s)||u)}else if(!Kg(vm(t,m),_,i)){u=!0;T=Dr(d.openingElement.tagName,e.Diagnostics.This_JSX_tag_s_0_prop_expects_type_1_which_requires_multiple_children_but_only_a_single_child_was_provided,f,go(_));s&&s.skipLogging&&(s.errors||(s.errors=[])).push(T)}}return u;function L(){if(!l){var t=e.getTextOfNode(n.parent.tagName),i=yS(fS(n)),a=void 0===i?"children":e.unescapeLeadingUnderscores(i),s=vm(r,Zm(a)),c=e.Diagnostics._0_components_don_t_accept_text_as_child_elements_Text_in_JSX_has_the_type_string_but_the_expected_type_of_1_is_2;l=o(o({},c),{key:"!!ALREADY FORMATTED!!",message:e.formatMessage(void 0,c,t,a,go(s))})}return l}}function Ag(n,t){var r,i,a,o;return c(this,function(s){switch(s.label){case 0:if(r=e.length(n.elements),!r)return[2];i=0,s.label=1;case 1:return i<r?Oy(t)&&!Lu(t,""+i)?[3,3]:(a=n.elements[i],e.isOmittedExpression(a)?[3,3]:(o=$m(i),[4,{errorNode:a,innerExpression:a,nameType:o}])):[3,4];case 2:s.sent(),s.label=3;case 3:return i++,[3,1];case 4:return[2]}})}function Cg(e,n,t,r,i,a){if(131068&t.flags)return!1;if(Oy(n))return Tg(Ag(e,t),n,t,r,i,a);var o=e.contextualType;e.contextualType=t;try{var s=UT(e,1,!0);return e.contextualType=o,!!Oy(s)&&Tg(Ag(e,t),s,t,r,i,a)}finally{e.contextualType=o}}function Ig(n){var t,r,i,a,o;return c(this,function(s){switch(s.label){case 0:if(!e.length(n.properties))return[2];t=0,r=n.properties,s.label=1;case 1:if(!(t<r.length))return[3,8];if(i=r[t],e.isSpreadAssignment(i))return[3,7];if(a=Gf(Ca(i),8576),!a||131072&a.flags)return[3,7];switch(o=i.kind,o){case 171:return[3,2];case 170:return[3,2];case 167:return[3,2];case 292:return[3,2];case 291:return[3,4]}return[3,6];case 2:return[4,{errorNode:i.name,innerExpression:void 0,nameType:a}];case 3:return s.sent(),[3,7];case 4:return[4,{errorNode:i.name,innerExpression:i.initializer,nameType:a,errorMessage:e.isComputedNonLiteralName(i.name)?e.Diagnostics.Type_of_computed_property_s_value_is_0_which_is_not_assignable_to_type_1:void 0}];case 5:return s.sent(),[3,7];case 6:e.Debug.assertNever(i),s.label=7;case 7:return t++,[3,1];case 8:return[2]}})}function Ng(e,n,t,r,i,a){return!(131068&t.flags)&&Tg(Ig(e),n,t,r,i,a)}function kg(e,n,t,r,i){return zg(e,n,yr,t,r,i)}function Rg(e,n,t){return 0!==wg(e,n,t?4:0,!1,void 0,void 0,og,void 0)}function Og(e){return!e.typeParameters&&(!e.thisParameter||Vo(aD(e.thisParameter)))&&1===e.parameters.length&&V(e)&&(aD(e.parameters[0])===Gn||Vo(aD(e.parameters[0])))&&Vo(ad(e))}function wg(n,t,r,i,a,o,s,c){if(n===t)return-1;if(Og(t))return-1;var l=_D(t),u=!yD(t)&&(8&r?yD(n)||_D(n)>l:gD(n)>l);if(u)return 0;n.typeParameters&&n.typeParameters!==t.typeParameters&&(t=gd(t),n=Hx(n,t,void 0,s));var d=_D(n),p=vD(n),f=vD(t);if((p||f)&&z_(p||f,c),p&&f&&d!==l)return 0;var m=t.declaration?t.declaration.kind:0,_=!(3&r)&&J&&167!==m&&166!==m&&169!==m,g=-1,y=nd(n);if(y&&y!==Qe){var h=nd(t);if(h){var v=!_&&s(y,h,!1)||s(h,y,i);if(!v)return i&&a(e.Diagnostics.The_this_types_of_each_signature_are_incompatible),0;g&=v}}for(var b=p||f?Math.min(d,l):Math.max(d,l),E=p||f?b-1:-1,T=0;T<b;T++){var S=T===E?mD(n,T):fD(n,T),x=T===E?mD(t,T):fD(t,T);if(S&&x){var L=3&r?void 0:Vx(lh(S)),D=3&r?void 0:Vx(lh(x)),A=L&&D&&!td(L)&&!td(D)&&(98304&th(S))===(98304&th(x));v=A?wg(D,L,8&r|(_?2:1),i,a,o,s,c):!(3&r)&&!_&&s(S,x,!1)||s(x,S,i);if(v&&8&r&&T>=gD(n)&&T<gD(t)&&s(S,x,!1)&&(v=0),!v)return i&&a(e.Diagnostics.Types_of_parameters_0_and_1_are_incompatible,e.unescapeLeadingUnderscores(sD(n,T)),e.unescapeLeadingUnderscores(sD(t,T))),0;g&=v}}if(!(4&r)){var C=sd(t)?Ie:t.declaration&&FL(t.declaration)?gc(Aa(t.declaration.symbol)):ad(t);if(C===Qe)return g;var I=sd(n)?Ie:n.declaration&&FL(n.declaration)?gc(Aa(n.declaration.symbol)):ad(n),N=td(t);if(N){var k=td(n);if(k)g&=Pg(k,N,i,a,s);else if(e.isIdentifierTypePredicate(N))return i&&a(e.Diagnostics.Signature_0_must_be_a_type_predicate,_o(n)),0}else g&=1&r&&s(C,I,!1)||s(I,C,i),!g&&i&&o&&o(I,C)}return g}function Pg(n,t,r,i,a){if(n.kind!==t.kind)return r&&(i(e.Diagnostics.A_this_based_type_guard_is_not_compatible_with_a_parameter_based_type_guard),i(e.Diagnostics.Type_predicate_0_is_not_assignable_to_1,So(n),So(t))),0;if((1===n.kind||3===n.kind)&&n.parameterIndex!==t.parameterIndex)return r&&(i(e.Diagnostics.Parameter_0_is_not_in_the_same_position_as_parameter_1,n.parameterName,t.parameterName),i(e.Diagnostics.Type_predicate_0_is_not_assignable_to_1,So(n),So(t))),0;var o=n.type===t.type?-1:n.type&&t.type?a(n.type,t.type,r):0;return 0===o&&r&&i(e.Diagnostics.Type_predicate_0_is_not_assignable_to_1,So(n),So(t)),o}function Mg(e,n){var t=md(e),r=md(n),i=ad(t),a=ad(r);return!(a!==Qe&&!Kg(a,i,gr)&&!Kg(i,a,gr))&&Rg(t,r,!0)}function Fg(e){return e!==yn&&0===e.properties.length&&0===e.callSignatures.length&&0===e.constructSignatures.length&&0===e.indexInfos.length}function Gg(n){return 524288&n.flags?!Bl(n)&&Fg(Ul(n)):!!(67108864&n.flags)||(1048576&n.flags?e.some(n.types,Gg):!!(2097152&n.flags)&&e.every(n.types,Gg))}function Bg(n){return!!(16&e.getObjectFlags(n)&&(n.members&&Fg(n)||n.symbol&&2048&n.symbol.flags&&0===Jc(n.symbol).size))}function Ug(n){return 524288&n.flags&&!Bl(n)&&0===Hl(n).length&&1===Ru(n).length&&!!Ou(n,je)||3145728&n.flags&&e.every(n.types,Ug)||!1}function Vg(n,t,r){if(n===t)return!0;var i=w(n)+","+w(t),a=vr.get(i);if(void 0!==a&&(4&a||!(2&a)||!r))return!!(1&a);if(n.escapedName!==t.escapedName||!(256&n.flags)||!(256&t.flags))return vr.set(i,6),!1;for(var o=Ws(t),s=0,c=Hl(Ws(n));s<c.length;s++){var l=c[s];if(8&l.flags){var u=Lu(o,l.escapedName);if(!u||!(8&u.flags))return r?(r(e.Diagnostics.Property_0_is_missing_in_type_1,e.symbolName(l),go(Dc(t),void 0,64)),vr.set(i,6)):vr.set(i,2),!1}}return vr.set(i,1),!0}function jg(e,n,t,r){var i=e.flags,a=n.flags;if(3&a||131072&i||e===ke)return!0;if(131072&a)return!1;if(402653316&i&&4&a)return!0;if(128&i&&1024&i&&128&a&&!(1024&a)&&e.value===n.value)return!0;if(296&i&&8&a)return!0;if(256&i&&1024&i&&256&a&&!(1024&a)&&e.value===n.value)return!0;if(2112&i&&64&a)return!0;if(528&i&&16&a)return!0;if(12288&i&&4096&a)return!0;if(32&i&&32&a&&Vg(e.symbol,n.symbol,r))return!0;if(1024&i&&1024&a){if(1048576&i&&1048576&a&&Vg(e.symbol,n.symbol,r))return!0;if(2944&i&&2944&a&&e.value===n.value&&Vg(Ia(e.symbol),Ia(n.symbol),r))return!0}if(32768&i&&(!z||49152&a))return!0;if(65536&i&&(!z||65536&a))return!0;if(524288&i&&67108864&a)return!0;if(t===gr||t===yr){if(1&i)return!0;if(264&i&&!(1024&i)&&(32&a||t===gr&&256&a&&1024&a))return!0}return!1}function Kg(e,n,t){if(Qm(e)&&(e=e.regularType),Qm(n)&&(n=n.regularType),e===n)return!0;if(t!==hr){if(t===yr&&!(131072&n.flags)&&jg(n,e,t)||jg(e,n,t))return!0}else{if(e.flags!==n.flags)return!1;if(67358815&e.flags)return!0}if(524288&e.flags&&524288&n.flags){var r=t.get(sy(e,n,0,t));if(void 0!==r)return!!(1&r)}return!!(469499904&e.flags||469499904&n.flags)&&zg(e,n,t,void 0)}function Hg(n,t){return 2048&e.getObjectFlags(n)&&rS(t.escapedName)}function Wg(n,t){while(1){var r=Qm(n)?n.regularType:4&e.getObjectFlags(n)&&n.node?Od(n.target,Md(n)):3145728&n.flags?vu(n):33554432&n.flags?t?n.baseType:n.substitute:25165824&n.flags?dm(n,t):n;if(r=Ny(r)||r,r===n)break;n=r}return n}function zg(n,t,r,i,o,s,c){var u,p,f,m,_,g,y=0,h=0,v=0,b=!1,E=0,T=[],S=!1;e.Debug.assert(r!==hr||!i,"no error reporting in identity checking");var x=j(n,t,!!i,o);if(T.length&&P(),b){null===e.tracing||void 0===e.tracing||e.tracing.instant("checkTypes","checkTypeRelatedTo_DepthLimit",{sourceId:n.id,targetId:t.id,depth:h});var L=Dr(i||d,e.Diagnostics.Excessive_stack_depth_comparing_types_0_and_1,go(n),go(t));c&&(c.errors||(c.errors=[])).push(L)}else if(u){if(s){var D=s();D&&(e.concatenateDiagnosticMessageChains(D,u),u=D)}var A=void 0;if(o&&i&&!x&&n.symbol){var C=Hr(n.symbol);if(C.originatingImport&&!e.isImportCall(C.originatingImport)){var N=zg(Ws(C.target),t,r,void 0);if(N){var k=e.createDiagnosticForNode(C.originatingImport,e.Diagnostics.Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cause_a_failure_at_runtime_Consider_using_a_default_import_or_import_require_here_instead);A=e.append(A,k)}}}L=e.createDiagnosticForNodeFromMessageChain(i,u,A);p&&e.addRelatedInfo.apply(void 0,a([L],p,!1)),c&&(c.errors||(c.errors=[])).push(L),c&&c.skipLogging||ur.add(L)}return i&&c&&c.skipLogging&&0===x&&e.Debug.assert(!!c.errors,"missed opportunity to interact with error."),0!==x;function R(e){u=e.errorInfo,g=e.lastSkippedInfo,T=e.incompatibleStack,E=e.overrideNextErrorInfo,p=e.relatedInfo}function O(){return{errorInfo:u,lastSkippedInfo:g,incompatibleStack:T.slice(),overrideNextErrorInfo:E,relatedInfo:p?p.slice():void 0}}function w(e,n,t,r,i){E++,g=void 0,T.push([e,n,t,r,i])}function P(){var n=T;T=[];var t=g;if(g=void 0,1===n.length)return F.apply(void 0,n[0]),void(t&&B.apply(void 0,a([void 0],t,!1)));var r="",i=[];while(n.length){var o=n.pop(),s=o[0],c=o.slice(1);switch(s.code){case e.Diagnostics.Types_of_property_0_are_incompatible.code:0===r.indexOf("new ")&&(r="("+r+")");var l=""+c[0];r=0===r.length?""+l:e.isIdentifierText(l,M.target)?r+"."+l:"["===l[0]&&"]"===l[l.length-1]?""+r+l:r+"["+l+"]";break;case e.Diagnostics.Call_signature_return_types_0_and_1_are_incompatible.code:case e.Diagnostics.Construct_signature_return_types_0_and_1_are_incompatible.code:case e.Diagnostics.Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code:case e.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code:if(0===r.length){var u=s;s.code===e.Diagnostics.Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code?u=e.Diagnostics.Call_signature_return_types_0_and_1_are_incompatible:s.code===e.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code&&(u=e.Diagnostics.Construct_signature_return_types_0_and_1_are_incompatible),i.unshift([u,c[0],c[1]])}else{var d=s.code===e.Diagnostics.Construct_signature_return_types_0_and_1_are_incompatible.code||s.code===e.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code?"new ":"",p=s.code===e.Diagnostics.Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code||s.code===e.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code?"":"...";r=""+d+r+"("+p+")"}break;case e.Diagnostics.Type_at_position_0_in_source_is_not_compatible_with_type_at_position_1_in_target.code:i.unshift([e.Diagnostics.Type_at_position_0_in_source_is_not_compatible_with_type_at_position_1_in_target,c[0],c[1]]);break;case e.Diagnostics.Type_at_positions_0_through_1_in_source_is_not_compatible_with_type_at_position_2_in_target.code:i.unshift([e.Diagnostics.Type_at_positions_0_through_1_in_source_is_not_compatible_with_type_at_position_2_in_target,c[0],c[1],c[2]]);break;default:return e.Debug.fail("Unhandled Diagnostic: "+s.code)}}r?F(")"===r[r.length-1]?e.Diagnostics.The_types_returned_by_0_are_incompatible_between_these_types:e.Diagnostics.The_types_of_0_are_incompatible_between_these_types,r):i.shift();for(var f=0,m=i;f<m.length;f++){var _=m[f],y=(s=_[0],c=_.slice(1),s.elidedInCompatabilityPyramid);s.elidedInCompatabilityPyramid=!1,F.apply(void 0,a([s],c,!1)),s.elidedInCompatabilityPyramid=y}t&&B.apply(void 0,a([void 0],t,!1))}function F(n,t,r,a,o){e.Debug.assert(!!i),T.length&&P(),n.elidedInCompatabilityPyramid||(u=e.chainDiagnosticMessages(u,n,t,r,a,o))}function G(n){e.Debug.assert(!!u),p?p.push(n):p=[n]}function B(n,t,i){T.length&&P();var a=yo(t,i),o=a[0],s=a[1],c=t,l=o;if(Uy(t)&&!Jg(i)&&(c=Vy(t),e.Debug.assert(!lg(c,i),"generalized source shouldn't be assignable"),l=ho(c)),262144&i.flags){var d=ru(i),p=void 0;d&&(lg(c,d)||(p=lg(t,d)))?F(e.Diagnostics._0_is_assignable_to_the_constraint_of_type_1_but_1_could_be_instantiated_with_a_different_subtype_of_constraint_2,p?o:l,s,go(d)):(u=void 0,F(e.Diagnostics._0_could_be_instantiated_with_an_arbitrary_type_which_could_be_unrelated_to_1,s,l))}n||(n=r===yr?e.Diagnostics.Type_0_is_not_comparable_to_type_1:o===s?e.Diagnostics.Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated:e.Diagnostics.Type_0_is_not_assignable_to_type_1),F(n,l,s)}function U(n,t){var r=vo(n.symbol)?go(n,n.symbol.valueDeclaration):go(n),i=vo(t.symbol)?go(t,t.symbol.valueDeclaration):go(t);(On===n&&je===t||wn===n&&Ke===t||Pn===n&&Xe===t||mp(!1)===n&&Ye===t)&&F(e.Diagnostics._0_is_a_primitive_but_1_is_a_wrapper_object_Prefer_using_0_when_possible,i,r)}function V(n,t,r){return Jy(n)?n.target.readonly&&Ay(t)?(r&&F(e.Diagnostics.The_type_0_is_readonly_and_cannot_be_assigned_to_the_mutable_type_1,go(n),go(t)),!1):Jy(t)||Ly(t):Dy(n)&&Ay(t)?(r&&F(e.Diagnostics.The_type_0_is_readonly_and_cannot_be_assigned_to_the_mutable_type_1,go(n),go(t)),!1):!Jy(t)||Ly(n)}function j(n,t,a,o,s){if(void 0===a&&(a=!1),void 0===s&&(s=0),524288&n.flags&&131068&t.flags)return jg(n,t,r,a?F:void 0)?-1:(L(n,t,0,!!(2048&e.getObjectFlags(n))),0);var c=Wg(n,!1),l=Wg(t,!0);if(c===l)return-1;if(r===hr)return H(c,l);if(262144&c.flags&&Jl(c)===l)return-1;if(1048576&l.flags&&524288&c.flags&&l.types.length<=3&&iA(l,98304)){var d=Ib(l,-98305);if(1179648&d.flags||(l=Wg(d,!0)),c===d)return-1}if(r===yr&&!(131072&l.flags)&&jg(l,c,r)||jg(c,l,r,a?F:void 0))return-1;var p=!!(2048&e.getObjectFlags(c)),f=!(2&s)&&_v(c)&&16384&e.getObjectFlags(c);if(f&&J(c,l,a))return a&&B(o,c,t.aliasSymbol?t:l),0;var m=r!==yr&&!(2&s)&&2752508&c.flags&&c!==An&&2621440&l.flags&&Yg(l)&&(Hl(c).length>0||BR(c));if(m&&!Qg(c,l,p)){if(a){var _=go(n.aliasSymbol?n:c),y=go(t.aliasSymbol?t:l),h=Au(c,0),v=Au(c,1);h.length>0&&j(ad(h[0]),l,!1)||v.length>0&&j(ad(v[0]),l,!1)?F(e.Diagnostics.Value_of_type_0_has_no_properties_in_common_with_type_1_Did_you_mean_to_call_it,_,y):F(e.Diagnostics.Type_0_has_no_properties_in_common_with_type_1,_,y)}return 0}K(c,l);var b=0,T=O();if((3145728&c.flags||3145728&l.flags)&&(b=Cb(c)*Cb(l)>=4?ie(c,l,a,8|s):ae(c,l,a,8|s)),b||1048576&c.flags||!(469499904&c.flags||469499904&l.flags)||(b=ie(c,l,a,s))&&R(T),!b&&2359296&c.flags){var x=tu(2097152&c.flags?c.types:[c],!!(1048576&l.flags));x&&(2097152&c.flags||1048576&l.flags)&&Eb(x,function(e){return e!==c})&&(b=j(x,l,!1,void 0,s))&&R(T)}return b&&!S&&(2097152&l.flags&&(f||m)||Vm(l)&&!Ly(l)&&!Jy(l)&&2097152&c.flags&&3670016&mu(c).flags&&!e.some(c.types,function(n){return!!(524288&e.getObjectFlags(n))}))&&(S=!0,b&=ie(c,l,a,4),S=!1),L(c,l,b,p),b;function L(r,s,c,l){if(!c&&a){var d=!!Ny(n),p=!!Ny(t);r=n.aliasSymbol||d?n:r,s=t.aliasSymbol||p?t:s;var f=E>0;if(f&&E--,524288&r.flags&&524288&s.flags){var m=u;V(r,s,a),u!==m&&(f=!!u)}if(524288&r.flags&&131068&s.flags)U(r,s);else if(r.symbol&&524288&r.flags&&An===r)F(e.Diagnostics.The_Object_type_is_assignable_to_very_few_other_types_Did_you_mean_to_use_the_any_type_instead);else if(l&&2097152&s.flags){var _=s.types,y=uS(I.IntrinsicAttributes,i),h=uS(I.IntrinsicClassAttributes,i);if(y!==Re&&h!==Re&&(e.contains(_,y)||e.contains(_,h)))return c}else u=xu(u,t);if(!o&&f)return g=[r,s],c;B(o,r,s)}}}function K(n,t){if(e.tracing&&3145728&n.flags&&3145728&t.flags){var r=n,a=t;if(r.objectFlags&a.objectFlags&65536)return;var o=r.types.length,s=a.types.length;o*s>1e6&&e.tracing.instant("checkTypes","traceUnionsOrIntersectionsTooLarge_DepthLimit",{sourceId:n.id,sourceSize:o,targetId:t.id,targetSize:s,pos:null===i||void 0===i?void 0:i.pos,end:null===i||void 0===i?void 0:i.end})}}function H(e,n){if(e.flags!==n.flags)return 0;if(67358815&e.flags)return-1;if(K(e,n),3145728&e.flags){var t=X(e,n);return t&&(t&=X(n,e)),t}return ie(e,n,!1,0)}function W(n,t){var r=function(n,r){var i;r=mu(r);var a=3145728&r.flags?hu(r,t):jl(r,t),o=a&&Ws(a)||(null===(i=Fu(r,t))||void 0===i?void 0:i.type)||Me;return e.append(n,o)};return ff(e.reduceLeft(n,r,void 0)||e.emptyArray)}function J(n,t,a){var o;if(!IS(t)||!Y&&8192&e.getObjectFlags(t))return!1;var s=!!(2048&e.getObjectFlags(n));if((r===gr||r===yr)&&(yb(An,t)||!s&&Gg(t)))return!1;var c,l=t;1048576&t.flags&&(l=tP(n,t,j)||nP(t),c=1048576&l.flags?l.types:[l]);for(var u=function(t){if(q(t,n.symbol)&&!Hg(n,t)){if(!CS(l,t.escapedName,s)){if(a){var r=Sb(l,IS);if(!i)return{value:e.Debug.fail()};if(e.isJsxAttributes(i)||e.isJsxOpeningLikeElement(i)||e.isJsxOpeningLikeElement(i.parent)){t.valueDeclaration&&e.isJsxAttribute(t.valueDeclaration)&&e.getSourceFileOfNode(i)===e.getSourceFileOfNode(t.valueDeclaration.name)&&(i=t.valueDeclaration.name);var u=mo(t),d=px(u,r),p=d?mo(d):void 0;p?F(e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2,u,go(r),p):F(e.Diagnostics.Property_0_does_not_exist_on_type_1,u,go(r))}else{var f=(null===(o=n.symbol)||void 0===o?void 0:o.declarations)&&e.firstOrUndefined(n.symbol.declarations);p=void 0;if(t.valueDeclaration&&e.findAncestor(t.valueDeclaration,function(e){return e===f})&&e.getSourceFileOfNode(f)===e.getSourceFileOfNode(i)){var m=t.valueDeclaration;e.Debug.assertNode(m,e.isObjectLiteralElementLike),i=m;var _=m.name;e.isIdentifier(_)&&(p=fx(_,r))}void 0!==p?F(e.Diagnostics.Object_literal_may_only_specify_known_properties_but_0_does_not_exist_in_type_1_Did_you_mean_to_write_2,mo(t),go(r),p):F(e.Diagnostics.Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1,mo(t),go(r))}}return{value:!0}}if(c&&!j(Ws(t),W(c,t.escapedName),a))return a&&w(e.Diagnostics.Types_of_property_0_are_incompatible,mo(t)),{value:!0}}},d=0,p=Hl(n);d<p.length;d++){var f=p[d],m=u(f);if("object"===typeof m)return m.value}return!1}function q(e,n){return e.valueDeclaration&&n.valueDeclaration&&e.valueDeclaration.parent===n.valueDeclaration}function X(e,n){for(var t=-1,r=e.types,i=0,a=r;i<a.length;i++){var o=a[i],s=Q(o,n,!1);if(!s)return 0;t&=s}return t}function Q(e,n,t){var r=n.types;if(1048576&n.flags){if(tf(r,e))return-1;var i=Gv(n,e);if(i){var a=j(e,i,!1);if(a)return a}}for(var o=0,s=r;o<s.length;o++){var c=s[o];a=j(e,c,!1);if(a)return a}if(t){var l=qg(e,n,j);j(e,l||r[r.length-1],!0)}return 0}function Z(e,n,t,r){for(var i=-1,a=n.types,o=0,s=a;o<s.length;o++){var c=s[o],l=j(e,c,t,void 0,r);if(!l)return 0;i&=l}return i}function $(e,n,t,r){var i=e.types;if(1048576&e.flags&&tf(i,n))return-1;for(var a=i.length,o=0;o<a;o++){var s=j(i[o],n,t&&o===a-1,void 0,r);if(s)return s}return 0}function ee(e,n){return 1048576&e.flags&&1048576&n.flags&&!(32768&e.types[0].flags)&&32768&n.types[0].flags?Ib(n,-32769):n}function te(e,n,t,r){for(var i=-1,a=e.types,o=ee(e,n),s=0;s<a.length;s++){var c=a[s];if(1048576&o.flags&&a.length>=o.types.length&&a.length%o.types.length===0){var l=j(c,o.types[s%o.types.length],!1,void 0,r);if(l){i&=l;continue}}var u=j(c,n,t,void 0,r);if(!u)return 0;i&=u}return i}function re(n,t,i,a,o){if(void 0===n&&(n=e.emptyArray),void 0===t&&(t=e.emptyArray),void 0===i&&(i=e.emptyArray),n.length!==t.length&&r===hr)return 0;for(var s=n.length<=t.length?n.length:t.length,c=-1,l=0;l<s;l++){var u=l<i.length?i[l]:1,d=7&u;if(4!==d){var p=n[l],f=t[l],m=-1;if(8&u?m=r===hr?j(p,f,!1):ag(p,f):1===d?m=j(p,f,a,void 0,o):2===d?m=j(f,p,a,void 0,o):3===d?(m=j(f,p,!1),m||(m=j(p,f,a,void 0,o))):(m=j(p,f,a,void 0,o),m&&(m&=j(f,p,a,void 0,o))),!m)return 0;c&=m}}return c}function ie(n,t,i,a){if(b)return 0;var o=sy(n,t,a|(S?16:0),r),s=r.get(o);if(void 0!==s&&(!(i&&2&s)||4&s)){if(yt){var c=24&s;8&c&&z_(n,b_(se)),16&c&&z_(n,b_(ce))}return 1&s?-1:0}if(f){for(var l=o.split(",").map(function(n){return n.replace(/-\d+/g,function(n,t){var r=e.length(o.slice(0,t).match(/[-=]/g)||void 0);return"="+r})}).join(","),u=0;u<y;u++)if(o===f[u]||l===f[u])return 3;if(100===h)return b=!0,0}else f=[],m=[],_=[];var d=y;f[y]=o,y++,m[h]=n,_[h]=t,h++;var p,g=v;1&v||!my(n,m,h)||(v|=1),2&v||!my(t,_,h)||(v|=2);var E=0;yt&&(p=yt,yt=function(e){return E|=e?16:8,p(e)}),3===v&&(null===e.tracing||void 0===e.tracing||e.tracing.instant("checkTypes","recursiveTypeRelatedTo_DepthLimit",{sourceId:n.id,sourceIdStack:m.map(function(e){return e.id}),targetId:t.id,targetIdStack:_.map(function(e){return e.id}),depth:h}));var T=3!==v?ae(n,t,i,a):3;if(yt&&(yt=p),v=g,h--,T){if(-1===T||0===h){if(-1===T||3===T)for(u=d;u<y;u++)r.set(f[u],1|E);y=d}}else r.set(o,2|(i?4:0)|E),y=d;return T}function ae(n,t,r,i){null===e.tracing||void 0===e.tracing||e.tracing.push("checkTypes","structuredTypeRelatedTo",{sourceId:n.id,targetId:t.id});var a=oe(n,t,r,i);return null===e.tracing||void 0===e.tracing||e.tracing.pop(),a}function oe(n,t,i,a){if(4&a)return _e(n,t,i,void 0,0);if(8&a){if(1048576&n.flags)return r===yr?$(n,t,i&&!(131068&n.flags),-9&a):te(n,t,i&&!(131068&n.flags),-9&a);if(1048576&t.flags)return Q(Eh(n),t,i&&!(131068&n.flags)&&!(131068&t.flags));if(2097152&t.flags)return Z(Eh(n),t,i,2);if(r===yr&&131068&t.flags){var o=e.sameMap(n.types,iu);if(o!==n.types&&(n=Df(o),!(2097152&n.flags)))return j(n,t,!1)}return $(n,t,!1,1)}var s,c,l=n.flags&t.flags;if(r===hr&&!(524288&l)){if(4194304&l)return j(n.type,t.type,!1);var d=0;return 8388608&l&&(d=j(n.objectType,t.objectType,!1))&&(d&=j(n.indexType,t.indexType,!1))?d:16777216&l&&n.root.isDistributive===t.root.isDistributive&&(d=j(n.checkType,t.checkType,!1))&&(d&=j(n.extendsType,t.extendsType,!1))&&(d&=j(Im(n),Im(t),!1))&&(d&=j(Nm(n),Nm(t),!1))?d:33554432&l?j(n.substitute,t.substitute,!1):0}var p=!1,f=O();if(17301504&n.flags&&n.aliasSymbol&&n.aliasTypeArguments&&n.aliasSymbol===t.aliasSymbol&&!n.aliasTypeArgumentsContainsMarker&&!t.aliasTypeArgumentsContainsMarker){var m=$g(n.aliasSymbol);if(m===e.emptyArray)return 1;var _=de(n.aliasTypeArguments,t.aliasTypeArguments,m,a);if(void 0!==_)return _}if(Xy(n)&&!n.target.readonly&&(s=j(Md(n)[0],t))||Xy(t)&&(t.target.readonly||Ay(ru(n)||n))&&(s=j(n,Md(t)[0])))return s;if(262144&t.flags){if(32&e.getObjectFlags(n)&&!n.declaration.nameType&&j(Uf(t),Il(n))&&!(4&Pl(n))){var g=kl(n),y=vm(t,Cl(n));if(s=j(g,y,i))return s}}else if(4194304&t.flags){var h=t.type;if(4194304&n.flags&&(s=j(h,n.type,!1)))return s;if(Jy(h)){if(s=j(n,Qp(h),i))return s}else{var v=Yl(h);if(v&&-1===j(n,Uf(v,t.stringsOnly),i))return-1}}else if(8388608&t.flags){if(8388608&n.flags){if((s=j(n.objectType,t.objectType,i))&&(s&=j(n.indexType,t.indexType,i)),s)return R(f),s;i&&(c=u)}if(r===gr||r===yr){var b=t.objectType,E=t.indexType,T=ru(b)||b,S=ru(E)||E;if(!sm(T)&&!cm(S)){var x=4|(T!==b?2:0);v=Em(T,S,x);if(v){if(i&&c&&R(f),s=j(n,v,i))return s;i&&c&&u&&(u=se([c])<=se([u])?c:u)}}}i&&(c=void 0)}else if(Bl(t)&&!t.declaration.nameType){var L=kl(t),D=Pl(t);if(!(8&D)){if(8388608&L.flags&&L.objectType===n&&L.indexType===Cl(t))return-1;if(!Bl(n)){var A=Il(t),C=Uf(n,void 0,!0),I=4&D,N=I?_l(A,C):void 0;if(I?!(131072&N.flags):j(A,C)){g=kl(t);var k=Cl(t),w=Ib(g,-98305);if(8388608&w.flags&&w.indexType===k){if(s=j(n,w.objectType,i))return s}else{var P=N?Df([N,k]):k;y=vm(n,P);if(s=j(y,g,i))return s}}c=u,R(f)}}}else if(16777216&t.flags){var M=t,F=!lg(Y_(M.checkType),Y_(M.extendsType)),G=!F&&_m(M),B=void 0,U=xm(M.root.checkType);if(M.root.isDistributive&&262144&U.flags){var V=I_(U);B=D_(U,V,M.mapper),V.mapper=B}var K=void 0;if((F||(K=j(n,B?z_(u_(M.root.node.trueType),B):Im(M),!1)))&&(G||(K=(K||3)&j(n,B?z_(u_(M.root.node.falseType),B):Nm(M),!1))),K)return R(f),K}else if(134217728&t.flags){134217728&n.flags&&z_(n,b_(ce));var H=cv(n,t);if(H&&e.every(H,function(e,n){return sv(e,t.types[n])}))return-1}if(8650752&n.flags){if(!(8388608&n.flags&&8388608&t.flags)){v=Jl(n);if(!v||262144&n.flags&&1&v.flags){if(s=j(pn,Ib(t,-67108865)))return R(f),s}else{if(s=j(v,t,!1,void 0,a))return R(f),s;if(s=j(Xc(v,n),t,i&&!(t.flags&n.flags&262144),void 0,a))return R(f),s}}}else if(4194304&n.flags){if(s=j(sn,t,i))return R(f),s}else if(134217728&n.flags){if(!(134217728&t.flags)){var W=ru(n);v=W&&W!==n?W:je;if(s=j(v,t,i))return R(f),s}}else if(268435456&n.flags)if(268435456&t.flags&&n.symbol===t.symbol){if(s=j(n.type,t.type,i))return R(f),s}else{v=ru(n);if(v&&(s=j(v,t,i)))return R(f),s}else if(16777216&n.flags){if(16777216&t.flags){var z=n.root.inferTypeParameters,J=n.extendsType,q=void 0;if(z){var X=Ph(z,void 0,0,j);dv(X.inferences,t.extendsType,J,1536),J=z_(J,X.mapper),q=X.mapper}if(ig(J,t.extendsType)&&(j(n.checkType,t.checkType)||j(t.checkType,n.checkType))&&((s=j(z_(Im(n),q),Im(t),i))&&(s&=j(Nm(n),Nm(t),i)),s))return R(f),s}else{var Y=$l(n);if(Y&&(s=j(Y,t,i)))return R(f),s}var ee=Zl(n);if(ee&&(s=j(ee,t,i)))return R(f),s}else{if(r!==mr&&r!==_r&&Gl(t)&&Gg(n))return-1;if(Bl(t))return Bl(n)&&(s=le(n,t,i))?(R(f),s):0;var ne=!!(131068&n.flags);if(r!==hr)n=mu(n);else if(Bl(n))return 0;if(4&e.getObjectFlags(n)&&4&e.getObjectFlags(t)&&n.target===t.target&&!Jy(n)&&!(4096&e.getObjectFlags(n)||4096&e.getObjectFlags(t))){m=ny(n.target);if(m===e.emptyArray)return 1;_=de(Md(n),Md(t),m,a);if(void 0!==_)return _}else{if(Dy(t)?Ly(n)||Jy(n):Ly(t)&&Jy(n)&&!n.target.readonly)return r!==hr?j(wu(n,Ke)||Ie,wu(t,Ke)||Ie,i):0;if((r===mr||r===_r)&&Gg(t)&&16384&e.getObjectFlags(t)&&!Gg(n))return 0}if(2621440&n.flags&&524288&t.flags){var ie=i&&u===f.errorInfo&&!ne;if(s=_e(n,t,ie,void 0,a),s&&(s&=ye(n,t,0,ie),s&&(s&=ye(n,t,1,ie),s&&(s&=xe(n,t,ne,ie,a)))),p&&s)u=c||u||f.errorInfo;else if(s)return s}if(2621440&n.flags&&1048576&t.flags){var ae=Ib(t,36175872);if(1048576&ae.flags){var oe=ue(n,ae);if(oe)return oe}}}return 0;function se(n){return n?e.reduceLeft(n,function(e,n){return e+1+se(n.next)},0):0}function de(n,t,r,a){if(s=re(n,t,r,i,a))return s;if(e.some(r,function(e){return!!(24&e)}))return c=void 0,void R(f);var o=t&&ty(t,r);if(p=!o,r!==e.emptyArray&&!o){if(p&&(!i||!e.some(r,function(e){return 0===(7&e)})))return 0;c=u,R(f)}}}function se(e){return!yt||e!==En&&e!==Tn&&e!==ht||yt(!1),e}function ce(e){return!yt||e!==En&&e!==Tn&&e!==ht||yt(!0),e}function le(e,n,t){var i=r===yr||(r===hr?Pl(e)===Pl(n):Fl(e)<=Fl(n));if(i){var a,o=Il(n),s=z_(Il(e),b_(Fl(e)<0?se:ce));if(a=j(o,s,t)){var c=g_([Cl(e)],[Cl(n)]);if(z_(Nl(e),c)===z_(Nl(n),c))return a&j(z_(kl(e),c),kl(n),t)}}return 0}function ue(n,t){var i=Hl(n),a=wv(i,t);if(!a)return 0;for(var o=1,s=0,c=a;s<c.length;s++){var l=c[s];if(o*=Lb(zs(l)),o>25)return null===e.tracing||void 0===e.tracing||e.tracing.instant("checkTypes","typeRelatedToDiscriminatedType_DepthLimit",{sourceId:n.id,targetId:t.id,numCombinations:o}),0}for(var u=new Array(a.length),d=new e.Set,p=0;p<a.length;p++){l=a[p];var f=zs(l);u[p]=1048576&f.flags?f.types:[f],d.add(l.escapedName)}for(var m=e.cartesianProduct(u),_=[],g=function(i){var o=!1;e:for(var s=0,c=t.types;s<c.length;s++){for(var l=c[s],u=function(e){var o=a[e],s=Lu(l,o.escapedName);if(!s)return"continue-outer";if(o===s)return"continue";var c=fe(n,t,o,s,function(n){return i[e]},!1,0,z||r===yr);return c?void 0:"continue-outer"},d=0;d<a.length;d++){var p=u(d);switch(p){case"continue-outer":continue e}}e.pushIfUnique(_,l,e.equateValues),o=!0}if(!o)return{value:0}},y=0,h=m;y<h.length;y++){var v=h[y],b=g(v);if("object"===typeof b)return b.value}for(var E=-1,T=0,S=_;T<S.length;T++){var x=S[T];if(E&=_e(n,x,!1,d,0),E&&(E&=ye(n,x,0,!1),E&&(E&=ye(n,x,1,!1),!E||Jy(n)&&Jy(x)||(E&=xe(n,x,!1,!1,0)))),!E)return E}return E}function de(e,n){if(!n||0===e.length)return e;for(var t,r=0;r<e.length;r++)n.has(e[r].escapedName)?t||(t=e.slice(0,r)):t&&t.push(e[r]);return t||e}function pe(n,t,r,i,a){var o=z&&!!(48&e.getCheckFlags(t)),s=ns(zs(t),!1,o),c=r(n);return j(c,s,i,void 0,a)}function fe(n,t,r,i,a,o,s,c){var l=e.getDeclarationModifierFlagsFromSymbol(r),u=e.getDeclarationModifierFlagsFromSymbol(i);if(8&l||8&u){if(r.valueDeclaration!==i.valueDeclaration)return o&&(8&l&&8&u?F(e.Diagnostics.Types_have_separate_declarations_of_a_private_property_0,mo(i)):F(e.Diagnostics.Property_0_is_private_in_type_1_but_not_in_type_2,mo(i),go(8&l?n:t),go(8&l?t:n))),0}else if(16&u){if(!py(r,i))return o&&F(e.Diagnostics.Property_0_is_protected_but_type_1_is_not_a_class_derived_from_2,mo(i),go(ly(r)||n),go(ly(i)||t)),0}else if(16&l)return o&&F(e.Diagnostics.Property_0_is_protected_in_type_1_but_public_in_type_2,mo(i),go(n),go(t)),0;var d=pe(r,i,a,o,s);return d?c||!(16777216&r.flags)||16777216&i.flags?d:(o&&F(e.Diagnostics.Property_0_is_optional_in_type_1_but_required_in_type_2,mo(i),go(n),go(t)),0):(o&&w(e.Diagnostics.Types_of_property_0_are_incompatible,mo(i)),0)}function me(n,t,r,i){var s=!1;if(r.valueDeclaration&&e.isNamedDeclaration(r.valueDeclaration)&&e.isPrivateIdentifier(r.valueDeclaration.name)&&n.symbol&&32&n.symbol.flags){var c=r.valueDeclaration.name.escapedText,d=e.getSymbolNameForPrivateIdentifier(n.symbol,c);if(d&&Lu(n,d)){var p=e.factory.getDeclarationName(n.symbol.valueDeclaration),f=e.factory.getDeclarationName(t.symbol.valueDeclaration);return void F(e.Diagnostics.Property_0_in_type_1_refers_to_a_different_member_that_cannot_be_accessed_from_within_type_2,ti(c),ti(""===p.escapedText?l:p),ti(""===f.escapedText?l:f))}}var m=e.arrayFrom($h(n,t,i,!1));if((!o||o.code!==e.Diagnostics.Class_0_incorrectly_implements_interface_1.code&&o.code!==e.Diagnostics.Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass.code)&&(s=!0),1===m.length){var _=mo(r);F.apply(void 0,a([e.Diagnostics.Property_0_is_missing_in_type_1_but_required_in_type_2,_],yo(n,t),!1)),e.length(r.declarations)&&G(e.createDiagnosticForNode(r.declarations[0],e.Diagnostics._0_is_declared_here,_)),s&&u&&E++}else V(n,t,!1)&&(m.length>5?F(e.Diagnostics.Type_0_is_missing_the_following_properties_from_type_1_Colon_2_and_3_more,go(n),go(t),e.map(m.slice(0,4),function(e){return mo(e)}).join(", "),m.length-4):F(e.Diagnostics.Type_0_is_missing_the_following_properties_from_type_1_Colon_2,go(n),go(t),e.map(m,function(e){return mo(e)}).join(", ")),s&&u&&E++)}function _e(n,t,i,a,o){if(r===hr)return ge(n,t,a);var s=-1;if(Jy(t)){if(Ly(n)||Jy(n)){if(!t.target.readonly&&(Dy(n)||Jy(n)&&n.target.readonly))return 0;var c=Fd(n),l=Fd(t),u=Jy(n)?4&n.target.combinedFlags:4,d=4&t.target.combinedFlags,p=Jy(n)?n.target.minLength:0,f=t.target.minLength;if(!u&&c<f)return i&&F(e.Diagnostics.Source_has_0_element_s_but_target_requires_1,c,f),0;if(!d&&l<p)return i&&F(e.Diagnostics.Source_has_0_element_s_but_target_allows_only_1,p,l),0;if(!d&&(u||l<c))return i&&(p<f?F(e.Diagnostics.Target_requires_0_element_s_but_source_may_have_fewer,f):F(e.Diagnostics.Target_allows_only_0_element_s_but_source_may_have_more,l)),0;for(var m=Md(n),_=Md(t),g=Math.min(Jy(n)?Zp(n.target,11):0,Zp(t.target,11)),y=Math.min(Jy(n)?$p(n.target,11):0,d?$p(t.target,11):0),h=!!a,v=0;v<l;v++){var b=v<l-y?v:v+c-l,E=Jy(n)&&(v<g||v>=l-y)?n.target.elementFlags[b]:4,T=t.target.elementFlags[v];if(8&T&&!(8&E))return i&&F(e.Diagnostics.Source_provides_no_match_for_variadic_element_at_position_0_in_target,v),0;if(8&E&&!(12&T))return i&&F(e.Diagnostics.Variadic_element_at_position_0_in_source_does_not_match_element_at_position_1_in_target,b,v),0;if(1&T&&!(1&E))return i&&F(e.Diagnostics.Source_provides_no_match_for_required_element_at_position_0_in_target,v),0;if(!(h&&((12&E||12&T)&&(h=!1),h&&(null===a||void 0===a?void 0:a.has(""+v))))){var S=Jy(n)?v<g||v>=l-y?mh(m[b],!!(E&T&2)):Zy(n,g,y)||Ze:m[0],x=_[v],L=8&E&&4&T?Mp(x):mh(x,!!(2&T)),D=j(S,L,i,void 0,o);if(!D)return i&&(l>1||c>1)&&(v<g||v>=l-y||c-g-y===1?w(e.Diagnostics.Type_at_position_0_in_source_is_not_compatible_with_type_at_position_1_in_target,b,v):w(e.Diagnostics.Type_at_positions_0_through_1_in_source_is_not_compatible_with_type_at_position_2_in_target,g,c-y-1,v)),0;s&=D}}return s}if(12&t.target.combinedFlags)return 0}var A=(r===mr||r===_r)&&!_v(n)&&!Ry(n)&&!Jy(n),C=ev(n,t,A,!1);if(C)return i&&me(n,t,C,A),0;if(_v(t))for(var I=0,N=de(Hl(n),a);I<N.length;I++){var k=N[I];if(!jl(t,k.escapedName)){S=Ws(k);if(!(32768&S.flags))return i&&F(e.Diagnostics.Property_0_does_not_exist_on_type_1,mo(k),go(t)),0}}for(var R=Hl(t),O=Jy(n)&&Jy(t),P=0,M=de(R,a);P<M.length;P++){var G=M[P],B=G.escapedName;if(!(4194304&G.flags)&&(!O||HT(B)||"length"===B)){k=Lu(n,B);if(k&&k!==G){D=fe(n,t,k,G,zs,i,o,r===yr);if(!D)return 0;s&=D}}}return s}function ge(e,n,t){if(!(524288&e.flags&&524288&n.flags))return 0;var r=de(Vl(e),t),i=de(Vl(n),t);if(r.length!==i.length)return 0;for(var a=-1,o=0,s=r;o<s.length;o++){var c=s[o],l=jl(n,c.escapedName);if(!l)return 0;var u=yy(c,l,j);if(!u)return 0;a&=u}return a}function ye(n,t,i,a){var o,s;if(r===hr)return Ee(n,t,i);if(t===yn||n===yn)return-1;var c=n.symbol&&FL(n.symbol.valueDeclaration),l=t.symbol&&FL(t.symbol.valueDeclaration),u=Au(n,c&&1===i?0:i),d=Au(t,l&&1===i?0:i);if(1===i&&u.length&&d.length){var p=!!(4&u[0].flags),f=!!(4&d[0].flags);if(p&&!f)return a&&F(e.Diagnostics.Cannot_assign_an_abstract_constructor_type_to_a_non_abstract_constructor_type),0;if(!Ae(u[0],d[0],a))return 0}var m=-1,_=O(),g=1===i?ve:he,y=e.getObjectFlags(n),h=e.getObjectFlags(t);if(64&y&&64&h&&n.symbol===t.symbol)for(var v=0;v<d.length;v++){var b=be(u[v],d[v],!0,a,g(u[v],d[v]));if(!b)return 0;m&=b}else if(1===u.length&&1===d.length){var E=r===yr||!!M.noStrictGenericChecks,T=e.first(u),S=e.first(d);if(m=be(T,S,E,a,g(T,S)),!m&&a&&1===i&&y&h&&(169===(null===(o=S.declaration)||void 0===o?void 0:o.kind)||169===(null===(s=T.declaration)||void 0===s?void 0:s.kind))){var x=function(e){return _o(e,void 0,262144,i)};return F(e.Diagnostics.Type_0_is_not_assignable_to_type_1,x(T),x(S)),F(e.Diagnostics.Types_of_construct_signatures_are_incompatible),m}}else e:for(var L=0,D=d;L<D.length;L++){for(var A=D[L],C=a,I=0,N=u;I<N.length;I++){var k=N[I];b=be(k,A,!0,C,g(k,A));if(b){m&=b,R(_);continue e}C=!1}return C&&F(e.Diagnostics.Type_0_provides_no_match_for_the_signature_1,go(n),_o(A,void 0,void 0,i)),0}return m}function he(n,t){return 0===n.parameters.length&&0===t.parameters.length?function(n,t){return w(e.Diagnostics.Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1,go(n),go(t))}:function(n,t){return w(e.Diagnostics.Call_signature_return_types_0_and_1_are_incompatible,go(n),go(t))}}function ve(n,t){return 0===n.parameters.length&&0===t.parameters.length?function(n,t){return w(e.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1,go(n),go(t))}:function(n,t){return w(e.Diagnostics.Construct_signature_return_types_0_and_1_are_incompatible,go(n),go(t))}}function be(e,n,t,i,a){return wg(t?md(e):e,t?md(n):n,r===_r?8:0,i,F,a,j,b_(ce))}function Ee(e,n,t){var r=Au(e,t),i=Au(n,t);if(r.length!==i.length)return 0;for(var a=-1,o=0;o<r.length;o++){var s=vy(r[o],i[o],!1,!1,!1,j);if(!s)return 0;a&=s}return a}function Te(n,t,r){for(var i=-1,a=t.keyType,o=2097152&n.flags?Kl(n):Vl(n),s=0,c=o;s<c.length;s++){var l=c[s];if(!Hg(n,l)&&Nu(Gf(l,8576),a)){var u=zs(l),d=ne||32768&u.flags||a===Ke||!(16777216&l.flags)?u:Jv(u,524288),p=j(d,t.type,r);if(!p)return r&&F(e.Diagnostics.Property_0_is_incompatible_with_index_signature,mo(l)),0;i&=p}}for(var f=0,m=Ru(n);f<m.length;f++){var _=m[f];if(Nu(_.keyType,a)){p=Se(_,t,r);if(!p)return 0;i&=p}}return i}function Se(n,t,r){var i=j(n.type,t.type,r);return!i&&r&&(n.keyType===t.keyType?F(e.Diagnostics._0_index_signatures_are_incompatible,go(n.keyType)):F(e.Diagnostics._0_and_1_index_signatures_are_incompatible,go(n.keyType),go(t.keyType))),i}function xe(n,t,i,a,o){if(r===hr)return De(n,t);for(var s=Ru(t),c=e.some(s,function(e){return e.keyType===je}),l=-1,u=0,d=s;u<d.length;u++){var p=d[u],f=!i&&c&&1&p.type.flags?-1:Bl(n)&&c?j(kl(n),p.type,a):Le(n,p,a,o);if(!f)return 0;l&=f}return l}function Le(n,t,r,i){var a=Mu(n,t.keyType);return a?Se(a,t,r):1&i||!hh(n)?(r&&F(e.Diagnostics.Index_signature_for_type_0_is_missing_in_type_1,go(t.keyType),go(n)),0):Te(n,t,r)}function De(e,n){var t=Ru(e),r=Ru(n);if(t.length!==r.length)return 0;for(var i=0,a=r;i<a.length;i++){var o=a[i],s=Ou(e,o.keyType);if(!s||!j(s.type,o.type)||s.isReadonly!==o.isReadonly)return 0}return-1}function Ae(n,t,r){if(!n.declaration||!t.declaration)return!0;var i=e.getSelectedEffectiveModifierFlags(n.declaration,24),a=e.getSelectedEffectiveModifierFlags(t.declaration,24);return 8===a||(16===a&&8!==i||(16!==a&&!i||(r&&F(e.Diagnostics.Cannot_assign_a_0_constructor_type_to_a_1_constructor_type,Lo(i),Lo(a)),!1)))}}function Jg(n){if(16&n.flags)return!1;if(3145728&n.flags)return!!e.forEach(n.types,Jg);if(465829888&n.flags){var t=Jl(n);if(t&&t!==n)return Jg(t)}return Fy(n)||!!(134217728&n.flags)}function qg(e,n,t){return void 0===t&&(t=og),tP(e,n,t,!0)||Qw(e,n)||Zw(e,n)||$w(e,n)||eP(e,n)}function Xg(n,t,r,i,a){for(var o=n.types.map(function(e){}),s=0,c=t;s<c.length;s++){var l=c[s],u=l[0],d=l[1],p=yu(n,d);if(!(a&&p&&16&e.getCheckFlags(p)))for(var f=0,m=0,_=n.types;m<_.length;m++){var g=_[m],y=Bo(g,d);y&&r(u(),y)?o[f]=void 0===o[f]||o[f]:o[f]=!1,f++}}var h=o.indexOf(!0);if(-1===h)return i;var v=o.indexOf(!0,h+1);while(-1!==v){if(!ig(n.types[h],n.types[v]))return i;v=o.indexOf(!0,v+1)}return n.types[h]}function Yg(n){if(524288&n.flags){var t=Ul(n);return 0===t.callSignatures.length&&0===t.constructSignatures.length&&0===t.indexInfos.length&&t.properties.length>0&&e.every(t.properties,function(e){return!!(16777216&e.flags)})}return!!(2097152&n.flags)&&e.every(n.types,Yg)}function Qg(e,n,t){for(var r=0,i=Hl(e);r<i.length;r++){var a=i[r];if(CS(n,a.escapedName,t))return!0}return!1}function Zg(n,t,r){var i=Od(n,e.map(n.typeParameters,function(e){return e===t?r:e}));return i.objectFlags|=4096,i}function $g(e){var n=Hr(e);return ey(n.typeParameters,n,function(t,r,i){var a=Bd(e,f_(n.typeParameters,h_(r,i)));return a.aliasTypeArgumentsContainsMarker=!0,a})}function ey(n,t,r){var i,a,o;void 0===n&&(n=e.emptyArray);var s=t.variances;if(!s){null===e.tracing||void 0===e.tracing||e.tracing.push("checkTypes","getVariancesWorker",{arity:n.length,id:null!==(o=null!==(i=t.id)&&void 0!==i?i:null===(a=t.declaredType)||void 0===a?void 0:a.id)&&void 0!==o?o:-1}),t.variances=e.emptyArray,s=[];for(var c=function(e){var n=!1,i=!1,a=yt;yt=function(e){return e?i=!0:n=!0};var o=r(t,e,En),c=r(t,e,Tn),l=(lg(c,o)?1:0)|(lg(o,c)?2:0);3===l&&lg(r(t,e,ht),o)&&(l=4),yt=a,(n||i)&&(n&&(l|=8),i&&(l|=16)),s.push(l)},l=0,u=n;l<u.length;l++){var d=u[l];c(d)}t.variances=s,null===e.tracing||void 0===e.tracing||e.tracing.pop()}return s}function ny(e){return e===kn||e===Rn||8&e.objectFlags?A:ey(e.typeParameters,e,Zg)}function ty(e,n){for(var t=0;t<n.length;t++)if(1===(7&n[t])&&16384&e[t].flags)return!0;return!1}function ry(e){return 262144&e.flags&&!ql(e)}function iy(n){return!!(4&e.getObjectFlags(n))&&!n.node}function ay(n){return iy(n)&&e.some(Md(n),function(e){return!!(262144&e.flags)||ay(e)})}function oy(e,n,t){void 0===t&&(t=0);for(var r=""+e.target.id,i=0,a=Md(e);i<a.length;i++){var o=a[i];if(ry(o)){var s=n.indexOf(o);s<0&&(s=n.length,n.push(o)),r+="="+s}else t<4&&ay(o)?r+="<"+oy(o,n,t+1)+">":r+="-"+o.id}return r}function sy(e,n,t,r){if(r===hr&&e.id>n.id){var i=e;e=n,n=i}var a=t?":"+t:"";if(ay(e)&&ay(n)){var o=[];return oy(e,o)+","+oy(n,o)+a}return e.id+","+n.id+a}function cy(n,t){if(!(6&e.getCheckFlags(n)))return t(n);for(var r=0,i=n.containingType.types;r<i.length;r++){var a=i[r],o=Lu(a,n.escapedName),s=o&&cy(o,t);if(s)return s}}function ly(e){return e.parent&&32&e.parent.flags?Dc(Ia(e)):void 0}function uy(e){var n=ly(e),t=n&&lc(n)[0];return t&&Bo(t,e.escapedName)}function dy(e,n){return cy(e,function(e){var t=ly(e);return!!t&&Xs(t,n)})}function py(n,t){return!cy(t,function(t){return!!(16&e.getDeclarationModifierFlagsFromSymbol(t))&&!dy(n,ly(t))})}function fy(n,t,r){return cy(t,function(t){return!!(16&e.getDeclarationModifierFlagsFromSymbol(t,r))&&!Xs(n,ly(t))})?void 0:n}function my(e,n,t){if(t>=5)for(var r=_y(e),i=0,a=0;a<t;a++)if(_y(n[a])===r&&(i++,i>=5))return!0;return!1}function _y(n){if(524288&n.flags&&!gv(n)){if(e.getObjectFlags(n)&&n.node)return n.node;if(n.symbol&&!(16&e.getObjectFlags(n)&&32&n.symbol.flags))return n.symbol;if(Jy(n))return n.target}if(262144&n.flags)return n.symbol;if(8388608&n.flags){do{n=n.objectType}while(8388608&n.flags);return n}return 16777216&n.flags?n.root:n}function gy(e,n){return 0!==yy(e,n,ag)}function yy(n,t,r){if(n===t)return-1;var i=24&e.getDeclarationModifierFlagsFromSymbol(n),a=24&e.getDeclarationModifierFlagsFromSymbol(t);if(i!==a)return 0;if(i){if(pk(n)!==pk(t))return 0}else if((16777216&n.flags)!==(16777216&t.flags))return 0;return JD(n)!==JD(t)?0:r(Ws(n),Ws(t))}function hy(e,n,t){var r=_D(e),i=_D(n),a=gD(e),o=gD(n),s=yD(e),c=yD(n);return r===i&&a===o&&s===c||!!(t&&a<=o)}function vy(n,t,r,i,a,o){if(n===t)return-1;if(!hy(n,t,r))return 0;if(e.length(n.typeParameters)!==e.length(t.typeParameters))return 0;if(t.typeParameters){for(var s=g_(n.typeParameters,t.typeParameters),c=0;c<t.typeParameters.length;c++){var l=n.typeParameters[c],u=t.typeParameters[c];if(!(l===u||o(z_(Cd(l),s)||Pe,Cd(u)||Pe)&&o(z_(lu(l),s)||Pe,lu(u)||Pe)))return 0}n=k_(n,s,!0)}var d=-1;if(!i){var p=nd(n);if(p){var f=nd(t);if(f){var m=o(p,f);if(!m)return 0;d&=m}}}var _=_D(t);for(c=0;c<_;c++){l=pD(n,c),u=pD(t,c),m=o(u,l);if(!m)return 0;d&=m}if(!a){var g=td(n),y=td(t);d&=g||y?by(g,y,o):o(ad(n),ad(t))}return d}function by(e,n,t){return e&&n&&_f(e,n)?e.type===n.type?-1:e.type&&n.type?t(e.type,n.type):0:0}function Ey(e){for(var n,t=0,r=e;t<r.length;t++){var i=r[t],a=Vy(i);if(n||(n=a),a===i||a!==n)return!1}return!0}function Ty(n){return 1===n.length?n[0]:Ey(n)?ff(n):e.reduceLeft(n,function(e,n){return cg(e,n)?n:e})}function Sy(n){if(!z)return Ty(n);var t=e.filter(n,function(e){return!(98304&e.flags)});return t.length?oh(Ty(t),98304&nh(n)):ff(n,2)}function xy(n){return e.reduceLeft(n,function(e,n){return cg(n,e)?n:e})}function Ly(n){return!!(4&e.getObjectFlags(n))&&(n.target===kn||n.target===Rn)}function Dy(n){return!!(4&e.getObjectFlags(n))&&n.target===Rn}function Ay(e){return Ly(e)&&!Dy(e)||Jy(e)&&!e.target.readonly}function Cy(e){return Ly(e)?Md(e)[0]:void 0}function Iy(e){return Ly(e)||!(98304&e.flags)&&lg(e,Un)}function Ny(n){if(4&e.getObjectFlags(n)&&3&e.getObjectFlags(n.target)){if(33554432&e.getObjectFlags(n))return 67108864&e.getObjectFlags(n)?n.cachedEquivalentBaseType:void 0;n.objectFlags|=33554432;var t=n.target;if(1&e.getObjectFlags(t)){var r=rc(t);if(r&&79!==r.expression.kind&&204!==r.expression.kind)return}var i=lc(t);if(1===i.length&&!Jc(n.symbol).size){var a=e.length(t.typeParameters)?z_(i[0],g_(t.typeParameters,Md(n).slice(0,t.typeParameters.length))):i[0];return e.length(Md(n))>e.length(t.typeParameters)&&(a=Xc(a,e.last(Md(n)))),n.objectFlags|=67108864,n.cachedEquivalentBaseType=a}}}function ky(e){return z?e===nn:e===Fe}function Ry(e){var n=Cy(e);return!!n&&ky(n)}function Oy(e){return Jy(e)||!!Lu(e,"0")}function wy(e){return Iy(e)||Oy(e)}function Py(e,n){var t=Bo(e,""+n);return t||(Eb(e,Jy)?Db(e,function(e){return Yy(e)||Me}):void 0)}function My(e){return!(240512&e.flags)}function Fy(e){return!!(109440&e.flags)}function Gy(n){return 2097152&n.flags?e.some(n.types,Fy):!!(109440&n.flags)}function By(n){return 2097152&n.flags&&e.find(n.types,Fy)||n}function Uy(n){return!!(16&n.flags)||(1048576&n.flags?!!(1024&n.flags)||e.every(n.types,Fy):Fy(n))}function Vy(e){return 1024&e.flags?Ec(e):128&e.flags?je:256&e.flags?Ke:2048&e.flags?He:512&e.flags?Xe:1048576&e.flags?Db(e,Vy):e}function jy(e){return 1024&e.flags&&Qm(e)?Ec(e):128&e.flags&&Qm(e)?je:256&e.flags&&Qm(e)?Ke:2048&e.flags&&Qm(e)?He:512&e.flags&&Qm(e)?Xe:1048576&e.flags?Db(e,jy):e}function Ky(e){return 8192&e.flags?Ye:1048576&e.flags?Db(e,Ky):e}function Hy(e,n){return MA(e,n)||(e=Ky(jy(e))),e}function Wy(e,n,t){if(e&&Fy(e)){var r=n?t?JC(n):n:void 0;e=Hy(e,r)}return e}function zy(e,n,t,r){if(e&&Fy(e)){var i=n?BN(t,n,r):void 0;e=Hy(e,i)}return e}function Jy(n){return!!(4&e.getObjectFlags(n)&&8&n.target.objectFlags)}function qy(e){return Jy(e)&&!!(8&e.target.combinedFlags)}function Xy(e){return qy(e)&&1===e.target.elementFlags.length}function Yy(e){return Zy(e,e.target.fixedLength)}function Qy(e){var n=Yy(e);return n&&Mp(n)}function Zy(e,n,t,r){void 0===t&&(t=0),void 0===r&&(r=!1);var i=Fd(e)-t;if(n<i){for(var a=Md(e),o=[],s=n;s<i;s++){var c=a[s];o.push(8&e.target.elementFlags[s]?vm(c,Ke):c)}return r?Df(o):ff(o)}}function $y(n,t){return Fd(n)===Fd(t)&&e.every(n.target.elementFlags,function(e,n){return(12&e)===(12&t.target.elementFlags[n])})}function eh(e){var n=e.value;return"0"===n.base10Value}function nh(e){for(var n=0,t=0,r=e;t<r.length;t++){var i=r[t];n|=th(i)}return n}function th(e){return 1048576&e.flags?nh(e.types):128&e.flags?""===e.value?128:0:256&e.flags?0===e.value?256:0:2048&e.flags?eh(e)?2048:0:512&e.flags?e===We||e===ze?512:0:117724&e.flags}function rh(e){return 117632&th(e)?Sb(e,function(e){return!(117632&th(e))}):e}function ih(e){return Db(e,ah)}function ah(e){return 4&e.flags?Ut:8&e.flags?Vt:64&e.flags?jt:e===ze||e===We||114691&e.flags||128&e.flags&&""===e.value||256&e.flags&&0===e.value||2048&e.flags&&eh(e)?e:Ze}function oh(e,n){var t=n&~e.flags&98304;return 0===t?e:ff(32768===t?[e,Me]:65536===t?[e,Ue]:[e,Me,Ue])}function sh(n,t){return void 0===t&&(t=!1),e.Debug.assert(z),32768&n.flags?n:ff([n,t?Be:Me])}function ch(e){var n=Jv(e,2097152);return Vn||(Vn=op("NonNullable",524288,void 0)||Ae),Vn!==Ae?Bd(Vn,[n]):n}function lh(e){return z?ch(e):e}function uh(e){return z?ff([e,Ge]):e}function dh(e){return z?xb(e,Ge):e}function ph(n,t,r){return r?e.isOutermostOptionalChain(t)?sh(n):uh(n):n}function fh(n,t){return e.isExpressionOfOptionalChainRoot(t)?lh(n):e.isOptionalChain(t)?dh(n):n}function mh(e,n){return ne&&n?xb(e,Be):e}function _h(e){return ne&&(e===Be||1048576&e.flags&&tf(e.types,Be))}function gh(e){return ne?xb(e,Be):Jv(e,524288)}function yh(e,n){return 0!==(524&e.flags)&&0!==(28&n.flags)}function hh(n){return 2097152&n.flags?e.every(n.types,hh):!(!n.symbol||0===(7040&n.symbol.flags)||BR(n))||!!(1024&e.getObjectFlags(n)&&hh(n.source))}function vh(n,t){var r=Or(n.flags,n.escapedName,8&e.getCheckFlags(n));r.declarations=n.declarations,r.parent=n.parent,r.type=t,r.target=n,n.valueDeclaration&&(r.valueDeclaration=n.valueDeclaration);var i=Hr(n).nameType;return i&&(r.nameType=i),r}function bh(n,t){for(var r=e.createSymbolTable(),i=0,a=Vl(n);i<a.length;i++){var o=a[i],s=Ws(o),c=t(s);r.set(o.escapedName,c===s?o:vh(o,c))}return r}function Eh(n){if(!(_v(n)&&16384&e.getObjectFlags(n)))return n;var t=n.regularType;if(t)return t;var r=n,i=bh(n,Eh),a=Xa(r.symbol,i,r.callSignatures,r.constructSignatures,r.indexInfos);return a.flags=r.flags,a.objectFlags|=-16385&r.objectFlags,n.regularType=a,a}function Th(e,n,t){return{parent:e,propertyName:n,siblings:t,resolvedProperties:void 0}}function Sh(e){if(!e.siblings){for(var n=[],t=0,r=Sh(e.parent);t<r.length;t++){var i=r[t];if(_v(i)){var a=jl(i,e.propertyName);a&&vb(Ws(a),function(e){n.push(e)})}}e.siblings=n}return e.siblings}function xh(n){if(!n.resolvedProperties){for(var t=new e.Map,r=0,i=Sh(n);r<i.length;r++){var a=i[r];if(_v(a)&&!(4194304&e.getObjectFlags(a)))for(var o=0,s=Hl(a);o<s.length;o++){var c=s[o];t.set(c.escapedName,c)}}n.resolvedProperties=e.arrayFrom(t.values())}return n.resolvedProperties}function Lh(e,n){if(!(4&e.flags))return e;var t=Ws(e),r=n&&Th(n,e.escapedName,void 0),i=Ih(t,r);return i===t?e:vh(e,i)}function Dh(e){var n=De.get(e.escapedName);if(n)return n;var t=vh(e,Be);return t.flags|=16777216,De.set(e.escapedName,t),t}function Ah(n,t){for(var r=e.createSymbolTable(),i=0,a=Vl(n);i<a.length;i++){var o=a[i];r.set(o.escapedName,Lh(o,t))}if(t)for(var s=0,c=xh(t);s<c.length;s++){o=c[s];r.has(o.escapedName)||r.set(o.escapedName,Dh(o))}var l=Xa(n.symbol,r,e.emptyArray,e.emptyArray,e.sameMap(Ru(n),function(e){return Td(e.keyType,Ch(e.type),e.isReadonly)}));return l.objectFlags|=532480&e.getObjectFlags(n),l}function Ch(e){return Ih(e,void 0)}function Ih(n,t){if(393216&e.getObjectFlags(n)){if(void 0===t&&n.widened)return n.widened;var r=void 0;if(98305&n.flags)r=Ie;else if(_v(n))r=Ah(n,t);else if(1048576&n.flags){var i=t||Th(void 0,void 0,n.types),a=e.sameMap(n.types,function(e){return 98304&e.flags?e:Ih(e,i)});r=ff(a,e.some(a,Gg)?2:1)}else 2097152&n.flags?r=Df(e.sameMap(n.types,Ch)):(Ly(n)||Jy(n))&&(r=Od(n.target,e.sameMap(Md(n),Ch)));return r&&void 0===t&&(n.widened=r),r||n}return n}function Nh(n){var t=!1;if(131072&e.getObjectFlags(n)){if(1048576&n.flags)if(e.some(n.types,Gg))t=!0;else for(var r=0,i=n.types;r<i.length;r++){var a=i[r];Nh(a)&&(t=!0)}if(Ly(n)||Jy(n))for(var o=0,s=Md(n);o<s.length;o++){a=s[o];Nh(a)&&(t=!0)}if(_v(n))for(var c=0,l=Vl(n);c<l.length;c++){var u=l[c];a=Ws(u);131072&e.getObjectFlags(a)&&(Nh(a)||Dr(u.valueDeclaration,e.Diagnostics.Object_literal_s_property_0_implicitly_has_an_1_type,mo(u),go(Ch(a))),t=!0)}}return t}function kh(n,t,r){var i=go(Ch(t));if(!e.isInJSFile(n)||e.isCheckJsEnabledForFile(e.getSourceFileOfNode(n),M)){var a;switch(n.kind){case 219:case 165:case 164:a=Y?e.Diagnostics.Member_0_implicitly_has_an_1_type:e.Diagnostics.Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage;break;case 162:var o=n;if(e.isIdentifier(o.name)&&(e.isCallSignatureDeclaration(o.parent)||e.isMethodSignature(o.parent)||e.isFunctionTypeNode(o.parent))&&o.parent.parameters.indexOf(o)>-1&&(Qr(o,o.name.escapedText,788968,void 0,o.name.escapedText,!0)||o.name.originalKeywordKind&&e.isTypeNodeKind(o.name.originalKeywordKind))){var s="arg"+o.parent.parameters.indexOf(o);return void Cr(Y,n,e.Diagnostics.Parameter_has_a_name_but_no_type_Did_you_mean_0_Colon_1,s,e.declarationNameToString(o.name))}a=n.dotDotDotToken?Y?e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type:e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage:Y?e.Diagnostics.Parameter_0_implicitly_has_an_1_type:e.Diagnostics.Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage;break;case 201:if(a=e.Diagnostics.Binding_element_0_implicitly_has_an_1_type,!Y)return;break;case 312:return void Dr(n,e.Diagnostics.Function_type_which_lacks_return_type_annotation_implicitly_has_an_0_return_type,i);case 254:case 167:case 166:case 170:case 171:case 211:case 212:if(Y&&!n.name)return void Dr(n,3===r?e.Diagnostics.Generator_implicitly_has_yield_type_0_because_it_does_not_yield_any_values_Consider_supplying_a_return_type_annotation:e.Diagnostics.Function_expression_which_lacks_return_type_annotation_implicitly_has_an_0_return_type,i);a=Y?3===r?e.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_yield_type:e.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type:e.Diagnostics._0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage;break;case 193:return void(Y&&Dr(n,e.Diagnostics.Mapped_object_type_implicitly_has_an_any_template_type));default:a=Y?e.Diagnostics.Variable_0_implicitly_has_an_1_type:e.Diagnostics.Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage}Cr(Y,n,a,e.declarationNameToString(e.getNameOfDeclaration(n)),i)}}function Rh(n,r,i){!(t&&Y&&131072&e.getObjectFlags(r))||i&&PT(n)||Nh(r)||kh(n,r,i)}function Oh(e,n,t){var r=_D(e),i=_D(n),a=hD(e),o=hD(n),s=o?i-1:i,c=a?s:Math.min(r,s),l=nd(e);if(l){var u=nd(n);u&&t(l,u)}for(var d=0;d<c;d++)t(pD(e,d),pD(n,d));o&&t(mD(e,c),o)}function wh(e,n,t){var r=td(e),i=td(n);r&&i&&_f(r,i)&&r.type&&i.type?t(r.type,i.type):t(ad(e),ad(n))}function Ph(e,n,t,r){return Fh(e.map(Uh),n,t,r||og)}function Mh(n,t){return void 0===t&&(t=0),n&&Fh(e.map(n.inferences,Vh),n.signature,n.flags|t,n.compareTypes)}function Fh(e,n,t,r){var i={inferences:e,signature:n,flags:t,compareTypes:r,mapper:b_(function(e){return Gh(i,e,!0)}),nonFixingMapper:b_(function(e){return Gh(i,e,!1)})};return i}function Gh(e,n,t){for(var r=e.inferences,i=0;i<r.length;i++){var a=r[i];if(n===a.typeParameter)return t&&!a.isFixed&&(Bh(r),a.isFixed=!0),bv(e,i)}return n}function Bh(e){for(var n=0,t=e;n<t.length;n++){var r=t[n];r.isFixed||(r.inferredType=void 0)}}function Uh(e){return{typeParameter:e,candidates:void 0,contraCandidates:void 0,inferredType:void 0,priority:void 0,topLevel:!0,isFixed:!1,impliedArity:void 0}}function Vh(e){return{typeParameter:e.typeParameter,candidates:e.candidates&&e.candidates.slice(),contraCandidates:e.contraCandidates&&e.contraCandidates.slice(),inferredType:e.inferredType,priority:e.priority,topLevel:e.topLevel,isFixed:e.isFixed,impliedArity:e.impliedArity}}function jh(n){var t=e.filter(n.inferences,KA);return t.length?Fh(e.map(t,Vh),n.signature,n.flags,n.compareTypes):void 0}function Kh(e){return e&&e.mapper}function Hh(n){var t=e.getObjectFlags(n);if(1048576&t)return!!(2097152&t);var r=!!(465829888&n.flags||524288&n.flags&&!Wh(n)&&(4&t&&(n.node||e.forEach(Md(n),Hh))||16&t&&n.symbol&&14384&n.symbol.flags&&n.symbol.declarations||8389664&t)||3145728&n.flags&&!(1024&n.flags)&&!Wh(n)&&e.some(n.types,Hh));return 3899393&n.flags&&(n.objectFlags|=1048576|(r?2097152:0)),r}function Wh(n){if(n.aliasSymbol&&!n.aliasTypeArguments){var t=e.getDeclarationOfKind(n.aliasSymbol,257);return!(!t||!e.findAncestor(t.parent,function(e){return 300===e.kind||259!==e.kind&&"quit"}))}return!1}function zh(n,t){return!!(n===t||3145728&n.flags&&e.some(n.types,function(e){return zh(e,t)})||16777216&n.flags&&(Im(n)===t||Nm(n)===t))}function Jh(n){var t=e.createSymbolTable();vb(n,function(n){if(128&n.flags){var r=e.escapeLeadingUnderscores(n.value),i=Or(4,r);i.type=Ie,n.symbol&&(i.declarations=n.symbol.declarations,i.valueDeclaration=n.symbol.valueDeclaration),t.set(r,i)}});var r=4&n.flags?[Td(je,pn,!1)]:e.emptyArray;return Xa(void 0,t,e.emptyArray,e.emptyArray,r)}function qh(e,n,t){if(!Ot){var r=e.id+","+n.id+","+t.id;if(Rt.has(r))return Rt.get(r);Ot=!0;var i=Yh(e,n,t);return Ot=!1,Rt.set(r,i),i}}function Xh(n){return!(524288&e.getObjectFlags(n))||_v(n)&&e.some(Hl(n),function(e){return Xh(Ws(e))})||Jy(n)&&e.some(Md(n),Xh)}function Yh(n,t,r){if(Ou(n,je)||0!==Hl(n).length&&Xh(n)){if(Ly(n))return Mp(Zh(Md(n)[0],t,r),Dy(n));if(Jy(n)){var i=e.map(Md(n),function(e){return Zh(e,t,r)}),a=4&Pl(t)?e.sameMap(n.target.elementFlags,function(e){return 2&e?1:e}):n.target.elementFlags;return Wp(i,a,n.target.readonly,n.target.labeledElementDeclarations)}var o=ja(1040,void 0);return o.source=n,o.mappedType=t,o.constraintType=r,o}}function Qh(e){var n=Hr(e);return n.type||(n.type=Zh(e.propertyType,e.mappedType,e.constraintType)),n.type}function Zh(e,n,t){var r=vm(t.type,Cl(n)),i=kl(n),a=Uh(r);return dv([a],e,i),rv(a)||Pe}function $h(n,t,r,i){var a,o,s,l,u,d,p;return c(this,function(c){switch(c.label){case 0:a=Hl(t),o=0,s=a,c.label=1;case 1:return o<s.length?(l=s[o],Pc(l)?[3,5]:!r&&(16777216&l.flags||48&e.getCheckFlags(l))?[3,5]:(u=Lu(n,l.escapedName),u?[3,3]:[4,l])):[3,6];case 2:return c.sent(),[3,5];case 3:return i?(d=Ws(l),109440&d.flags?(p=Ws(u),1&p.flags||Ym(p)===Ym(d)?[3,5]:[4,l]):[3,5]):[3,5];case 4:c.sent(),c.label=5;case 5:return o++,[3,1];case 6:return[2]}})}function ev(e,n,t,r){var i=$h(e,n,t,r).next();if(!i.done)return i.value}function nv(e,n){return!(8&n.target.combinedFlags)&&n.target.minLength>e.target.minLength||!n.target.hasRestElement&&(e.target.hasRestElement||n.target.fixedLength<e.target.fixedLength)}function tv(e,n){return Jy(e)&&Jy(n)?nv(e,n):!!ev(e,n,!1,!0)&&!!ev(n,e,!1,!1)}function rv(e){return e.candidates?ff(e.candidates,2):e.contraCandidates?Df(e.contraCandidates):void 0}function iv(e){return!!Wr(e).skipDirectInference}function av(n){return!(!n.symbol||!e.some(n.symbol.declarations,iv))}function ov(n){var t=e.createScanner(99,!1),r=!0;t.setOnError(function(){return r=!1}),t.setText(n+"n");var i=t.scan();40===i&&(i=t.scan());var a=t.getTokenFlags();return r&&9===i&&t.getTextPos()===n.length+1&&!(512&a)}function sv(e,n){if(e===n||5&n.flags)return!0;if(128&e.flags){var t=e.value;return!!(8&n.flags&&""!==t&&isFinite(+t)||64&n.flags&&""!==t&&ov(t)||98816&n.flags&&t===n.intrinsicName)}if(134217728&e.flags){var r=e.texts;return 2===r.length&&""===r[0]&&""===r[1]&&lg(e.types[0],n)}return lg(e,n)}function cv(n,t){return 128&n.flags?uv([n.value],e.emptyArray,t):134217728&n.flags?e.arraysEqual(n.texts,t.texts)?e.map(n.types,lv):uv(n.texts,n.types,t):void 0}function lv(e){return 402653317&e.flags?e:Wf(["",""],[e])}function uv(e,n,t){var r=e.length-1,i=e[0],o=e[r],s=t.texts,c=s.length-1,l=s[0],u=s[c];if(!(0===r&&i.length<l.length+u.length)&&i.startsWith(l)&&o.endsWith(u)){for(var d=o.slice(0,o.length-u.length),p=[],f=0,m=l.length,_=1;_<c;_++){var g=s[_];if(g.length>0){var y=f,h=m;while(1){if(h=v(y).indexOf(g,h),h>=0)break;if(y++,y===e.length)return;h=0}b(y,h),m+=g.length}else if(m<v(f).length)b(f,m+1);else{if(!(f<r))return;b(f+1,0)}}return b(r,v(r).length),p}function v(n){return n<r?e[n]:d}function b(t,r){var i=t===f?Zm(v(t).slice(m,r)):Wf(a(a([e[f].slice(m)],e.slice(f+1,t),!0),[v(t).slice(0,r)],!1),n.slice(f,t));p.push(i),f=t,m=r}}function dv(n,t,r,i,a){void 0===i&&(i=0),void 0===a&&(a=!1);var o,s,c,l,u=!1,d=2048,p=!0,f=0;function m(t,s){if(Hh(s)){if(t===ke){var c=o;return o=t,m(s,s),void(o=c)}if(t.aliasSymbol&&t.aliasTypeArguments&&t.aliasSymbol===s.aliasSymbol)h(t.aliasTypeArguments,s.aliasTypeArguments,$g(t.aliasSymbol));else if(t===s&&3145728&t.flags)for(var l=0,f=t.types;l<f.length;l++){var v=f[l];m(v,v)}else{if(1048576&s.flags){var E=y(1048576&t.flags?t.types:[t],s.types,pv),S=E[0],A=E[1],C=y(S,A,fv),I=C[0],N=C[1];if(0===N.length)return;if(s=ff(N),0===I.length)return void _(t,s,1);t=ff(I)}else if(2097152&s.flags&&e.some(s.types,function(e){return!!b(e)||Bl(e)&&!!b(M_(e)||Ze)})){if(!(1048576&t.flags)){var k=y(2097152&t.flags?t.types:[t],s.types,ig);I=k[0],N=k[1];if(0===I.length||0===N.length)return;t=Df(I),s=Df(N)}}else 41943040&s.flags&&(s=xm(s));if(8650752&s.flags){if(524288&e.getObjectFlags(t)||t===Oe||t===$e||128&i&&(t===Ne||t===Bn)||av(t))return;var R=b(s);if(R){if(!R.isFixed){if((void 0===R.priority||i<R.priority)&&(R.candidates=void 0,R.contraCandidates=void 0,R.topLevel=!0,R.priority=i),i===R.priority){var O=o||t;a&&!u?e.contains(R.contraCandidates,O)||(R.contraCandidates=e.append(R.contraCandidates,O),Bh(n)):e.contains(R.candidates,O)||(R.candidates=e.append(R.candidates,O),Bh(n))}!(128&i)&&262144&s.flags&&R.topLevel&&!zh(r,s)&&(R.topLevel=!1,Bh(n))}return void(d=Math.min(d,i))}var w=dm(s,!1);if(w!==s)g(t,w,m);else if(8388608&s.flags){var P=dm(s.indexType,!1);if(465829888&P.flags){var M=pm(dm(s.objectType,!1),P,!1);M&&M!==s&&g(t,M,m)}}}if(!(4&e.getObjectFlags(t)&&4&e.getObjectFlags(s)&&(t.target===s.target||Ly(t)&&Ly(s)))||t.node&&s.node)if(4194304&t.flags&&4194304&s.flags)a=!a,m(t.type,s.type),a=!a;else if((Uy(t)||4&t.flags)&&4194304&s.flags){var F=Jh(t);a=!a,_(F,s.type,256),a=!a}else if(8388608&t.flags&&8388608&s.flags)m(t.objectType,s.objectType),m(t.indexType,s.indexType);else if(268435456&t.flags&&268435456&s.flags)t.symbol===s.symbol&&m(t.type,s.type);else if(33554432&t.flags){m(t.baseType,s);var G=i;i|=4,m(t.substitute,s),i=G}else if(16777216&s.flags)g(t,s,x);else if(3145728&s.flags)T(t,s.types,s.flags);else if(1048576&t.flags)for(var B=t.types,U=0,V=B;U<V.length;U++){var j=V[U];m(j,s)}else if(134217728&s.flags)L(t,s);else{if(t=vu(t),!(512&i&&467927040&t.flags)){var K=mu(t);if(K!==t&&p&&!(2621440&K.flags))return p=!1,m(K,s);t=K}2621440&t.flags&&g(t,s,D)}else h(Md(t),Md(s),ny(t.target))}}}function _(e,n,t){var r=i;i|=t,m(e,n),i=r}function g(n,t,r){var i=n.id+","+t.id,a=s&&s.get(i);if(void 0===a){(s||(s=new e.Map)).set(i,-1);var o=d;d=2048;var u=f,p=_y(n),m=_y(t);e.contains(c,p)&&(f|=1),e.contains(l,m)&&(f|=2),3!==f?((c||(c=[])).push(p),(l||(l=[])).push(m),r(n,t),l.pop(),c.pop()):d=-1,f=u,s.set(i,d),d=Math.min(d,o)}else d=Math.min(d,a)}function y(n,t,r){for(var i,a,o=0,s=t;o<s.length;o++)for(var c=s[o],l=0,u=n;l<u.length;l++){var d=u[l];r(d,c)&&(m(d,c),i=e.appendIfUnique(i,d),a=e.appendIfUnique(a,c))}return[i?e.filter(n,function(n){return!e.contains(i,n)}):n,a?e.filter(t,function(n){return!e.contains(a,n)}):t]}function h(e,n,t){for(var r=e.length<n.length?e.length:n.length,i=0;i<r;i++)i<t.length&&2===(7&t[i])?v(e[i],n[i]):m(e[i],n[i])}function v(e,n){J||1024&i?(a=!a,m(e,n),a=!a):m(e,n)}function b(e){if(8650752&e.flags)for(var t=0,r=n;t<r.length;t++){var i=r[t];if(e===i.typeParameter)return i}}function E(n){for(var t,r=0,i=n;r<i.length;r++){var a=i[r],o=2097152&a.flags&&e.find(a.types,function(e){return!!b(e)});if(!o||t&&o!==t)return;t=o}return t}function T(n,t,r){var a=0;if(1048576&r){for(var o=void 0,s=1048576&n.flags?n.types:[n],c=new Array(s.length),l=!1,u=0,p=t;u<p.length;u++){var f=p[u];if(b(f))o=f,a++;else for(var g=0;g<s.length;g++){var y=d;d=2048,m(s[g],f),d===i&&(c[g]=!0),l=l||-1===d,d=Math.min(d,y)}}if(0===a){var h=E(t);return void(h&&_(n,h,1))}if(1===a&&!l){var v=e.flatMap(s,function(e,n){return c[n]?void 0:e});if(v.length)return void m(ff(v),o)}}else for(var T=0,S=t;T<S.length;T++){f=S[T];b(f)?a++:m(n,f)}if(2097152&r?1===a:a>0)for(var x=0,L=t;x<L.length;x++){f=L[x];b(f)&&_(n,f,1)}}function S(n,t,r){if(1048576&r.flags){for(var i=!1,a=0,o=r.types;a<o.length;a++){var s=o[a];i=S(n,t,s)||i}return i}if(4194304&r.flags){var c=b(r.type);if(c&&!c.isFixed&&!av(n)){var l=qh(n,t,r);l&&_(l,c.typeParameter,524288&e.getObjectFlags(n)?16:8)}return!0}if(262144&r.flags){_(Uf(n),r,32);var u=Jl(r);if(u&&S(n,t,u))return!0;var d=e.map(Hl(n),Ws),p=e.map(Ru(n),function(e){return e!==xt?e.type:Ze});return m(ff(e.concatenate(d,p)),kl(t)),!0}return!1}function x(e,n){if(16777216&e.flags)m(e.checkType,n.checkType),m(e.extendsType,n.extendsType),m(Im(e),Im(n)),m(Nm(e),Nm(n));else{var t=i;i|=a?64:0;var r=[Im(n),Nm(n)];T(e,r,n.flags),i=t}}function L(e,n){for(var t=cv(e,n),r=n.types,i=0;i<r.length;i++)m(t?t[i]:Ze,r[i])}function D(n,t){if(4&e.getObjectFlags(n)&&4&e.getObjectFlags(t)&&(n.target===t.target||Ly(n)&&Ly(t)))h(Md(n),Md(t),ny(n.target));else{if(Bl(n)&&Bl(t)){m(Il(n),Il(t)),m(kl(n),kl(t));var r=Nl(n),i=Nl(t);r&&i&&m(r,i)}if(32&e.getObjectFlags(t)&&!t.declaration.nameType){var a=Il(t);if(S(n,t,a))return}if(!tv(n,t)){if(Ly(n)||Jy(n)){if(Jy(t)){var o=Fd(n),s=Fd(t),c=Md(t),l=t.target.elementFlags;if(Jy(n)&&$y(n,t)){for(var u=0;u<s;u++)m(Md(n)[u],c[u]);return}var d=Jy(n)?Math.min(n.target.fixedLength,t.target.fixedLength):0,p=Math.min(Jy(n)?$p(n.target,3):0,t.target.hasRestElement?$p(t.target,3):0);for(u=0;u<d;u++)m(Md(n)[u],c[u]);if(!Jy(n)||o-d-p===1&&4&n.target.elementFlags[d]){var f=Md(n)[d];for(u=d;u<s-p;u++)m(8&l[u]?Mp(f):f,c[u])}else{var g=s-d-p;if(2===g&&l[d]&l[d+1]&8&&Jy(n)){var y=b(c[d]);y&&void 0!==y.impliedArity&&(m(Yp(n,d,p+o-y.impliedArity),c[d]),m(Yp(n,d+y.impliedArity,p),c[d+1]))}else if(1===g&&8&l[d]){var v=2&t.target.elementFlags[s-1],E=Jy(n)?Yp(n,d,p):Mp(Md(n)[0]);_(E,c[d],v?2:0)}else if(1===g&&4&l[d]){f=Jy(n)?Zy(n,d,p):Md(n)[0];f&&m(f,c[d])}}for(u=0;u<p;u++)m(Md(n)[o-u-1],c[s-u-1]);return}if(Ly(t))return void N(n,t)}A(n,t),C(n,t,0),C(n,t,1),N(n,t)}}}function A(e,n){for(var t=Vl(n),r=0,i=t;r<i.length;r++){var a=i[r],o=Lu(e,a.escapedName);o&&m(Ws(o),Ws(a))}}function C(n,t,r){for(var i=Au(n,r),a=Au(t,r),o=i.length,s=a.length,c=o<s?o:s,l=!!(524288&e.getObjectFlags(n)),u=0;u<c;u++)I(hd(i[o-c+u]),md(a[s-c+u]),l)}function I(e,n,t){if(!t){var r=u,i=n.declaration?n.declaration.kind:0;u=u||167===i||166===i||169===i,Oh(e,n,v),u=r}wh(e,n,m)}function N(n,t){var r=e.getObjectFlags(n)&e.getObjectFlags(t)&32?8:0,i=Ru(t);if(hh(n))for(var a=0,o=i;a<o.length;a++){for(var s=o[a],c=[],l=0,u=Hl(n);l<u.length;l++){var d=u[l];if(Nu(Gf(d,8576),s.keyType)){var p=Ws(d);c.push(16777216&d.flags?gh(p):p)}}for(var f=0,m=Ru(n);f<m.length;f++){var g=m[f];Nu(g.keyType,s.keyType)&&c.push(g.type)}c.length&&_(ff(c),s.type,r)}for(var y=0,h=i;y<h.length;y++){s=h[y];var v=Mu(n,s.keyType);v&&_(v.type,s.type,r)}}m(t,r)}function pv(e,n){return ne&&n===Be?e===n:ig(e,n)||!!(4&n.flags&&128&e.flags||8&n.flags&&256&e.flags)}function fv(e,n){return!!(524288&e.flags&&524288&n.flags&&e.symbol&&e.symbol===n.symbol||e.aliasSymbol&&e.aliasTypeArguments&&e.aliasSymbol===n.aliasSymbol)}function mv(e){var n=ql(e);return!!n&&iA(16777216&n.flags?Zl(n):n,406978556)}function _v(n){return!!(128&e.getObjectFlags(n))}function gv(n){return!!(32896&e.getObjectFlags(n))}function yv(n){if(n.length>1){var t=e.filter(n,gv);if(t.length){var r=ff(t,2);return e.concatenate(e.filter(n,function(e){return!gv(e)}),[r])}}return n}function hv(e){return 416&e.priority?Df(e.contraCandidates):xy(e.contraCandidates)}function vv(n,t){var r=yv(n.candidates),i=mv(n.typeParameter),a=!i&&n.topLevel&&(n.isFixed||!zh(ad(t),n.typeParameter)),o=i?e.sameMap(r,Ym):a?e.sameMap(r,jy):r,s=416&n.priority?ff(o,2):Sy(o);return Ch(s)}function bv(e,n){var t=e.inferences[n];if(!t.inferredType){var r=void 0,i=e.signature;if(i){var a=t.candidates?vv(t,i):void 0;if(t.contraCandidates){var o=hv(t);r=!a||131072&a.flags||!cg(a,o)?o:a}else if(a)r=a;else if(1&e.flags)r=$e;else{var s=lu(t.typeParameter);s&&(r=z_(s,L_(S_(e,n),e.nonFixingMapper)))}}else r=rv(t);t.inferredType=r||Ev(!!(2&e.flags));var c=ql(t.typeParameter);if(c){var l=z_(c,e.nonFixingMapper);r&&e.compareTypes(r,Xc(l,r))||(t.inferredType=r=l)}}return t.inferredType}function Ev(e){return e?Ie:Pe}function Tv(e){for(var n=[],t=0;t<e.inferences.length;t++)n.push(bv(e,t));return n}function Sv(n){switch(n.escapedText){case"document":case"console":return e.Diagnostics.Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_include_dom;case"$":return M.types?e.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_save_dev_types_Slashjquery_and_then_add_jquery_to_the_types_field_in_your_tsconfig:e.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_save_dev_types_Slashjquery;case"describe":case"suite":case"it":case"test":return M.types?e.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_save_dev_types_Slashjest_or_npm_i_save_dev_types_Slashmocha_and_then_add_jest_or_mocha_to_the_types_field_in_your_tsconfig:e.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_save_dev_types_Slashjest_or_npm_i_save_dev_types_Slashmocha;case"process":case"require":case"Buffer":case"module":return M.types?e.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_save_dev_types_Slashnode_and_then_add_node_to_the_types_field_in_your_tsconfig:e.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_save_dev_types_Slashnode;case"Map":case"Set":case"Promise":case"Symbol":case"WeakMap":case"WeakSet":case"Iterator":case"AsyncIterator":case"SharedArrayBuffer":case"Atomics":case"AsyncIterable":case"AsyncIterableIterator":case"AsyncGenerator":case"AsyncGeneratorFunction":case"BigInt":case"Reflect":case"BigInt64Array":case"BigUint64Array":return e.Diagnostics.Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_1_or_later;default:return 292===n.parent.kind?e.Diagnostics.No_value_exists_in_scope_for_the_shorthand_property_0_Either_declare_one_or_provide_an_initializer:e.Diagnostics.Cannot_find_name_0}}function xv(n){var t=Wr(n);return t.resolvedSymbol||(t.resolvedSymbol=!e.nodeIsMissing(n)&&Qr(n,n.escapedText,1160127,Sv(n),n,!e.isWriteOnlyAccess(n),!1,!0)||Ae),t.resolvedSymbol}function Lv(n){return!!e.findAncestor(n,function(e){return 179===e.kind||79!==e.kind&&159!==e.kind&&"quit"})}function Dv(e,n,t,r){switch(e.kind){case 79:var i=xv(e);return i!==Ae?(r?O(r):"-1")+"|"+nf(n)+"|"+nf(t)+"|"+w(i):void 0;case 108:return"0|"+(r?O(r):"-1")+"|"+nf(n)+"|"+nf(t);case 228:case 210:return Dv(e.expression,n,t,r);case 159:var a=Dv(e.left,n,t,r);return a&&a+"."+e.right.escapedText;case 204:case 205:var o=Nv(e);if(void 0!==o){var s=Dv(e.expression,n,t,r);return s&&s+"."+o}}}function Av(n,t){switch(t.kind){case 210:case 228:return Av(n,t.expression);case 219:return e.isAssignmentExpression(t)&&Av(n,t.left)||e.isBinaryExpression(t)&&27===t.operatorToken.kind&&Av(n,t.right)}switch(n.kind){case 229:return 229===t.kind&&n.keywordToken===t.keywordToken&&n.name.escapedText===t.name.escapedText;case 79:case 80:return e.isThisInTypeQuery(n)?108===t.kind:79===t.kind&&xv(n)===xv(t)||(252===t.kind||201===t.kind)&&Ma(xv(n))===Ca(t);case 108:return 108===t.kind;case 106:return 106===t.kind;case 228:case 210:return Av(n.expression,t);case 204:case 205:return e.isAccessExpression(t)&&Nv(n)===Nv(t)&&Av(n.expression,t.expression);case 159:return e.isAccessExpression(t)&&n.right.escapedText===Nv(t)&&Av(n.left,t.expression);case 219:return e.isBinaryExpression(n)&&27===n.operatorToken.kind&&Av(n.right,t)}return!1}function Cv(e,n){return Av(e,n)||219===n.kind&&55===n.operatorToken.kind&&(Cv(e,n.left)||Cv(e,n.right))}function Iv(n){if(e.isAccessExpression(n))return n;if(e.isIdentifier(n)){var t=xv(n);if(sE(t)){var r=t.valueDeclaration;if(e.isVariableDeclaration(r)&&!r.type&&r.initializer&&e.isAccessExpression(r.initializer))return r.initializer;if(e.isBindingElement(r)&&!r.initializer){var i=r.parent.parent;if(e.isVariableDeclaration(i)&&!i.type&&i.initializer&&(e.isIdentifier(i.initializer)||e.isAccessExpression(i.initializer)))return r}}}}function Nv(n){var t;return 204===n.kind?n.name.escapedText:205===n.kind&&e.isStringOrNumericLiteralLike(n.argumentExpression)?e.escapeLeadingUnderscores(n.argumentExpression.text):201===n.kind&&(t=Xo(n))?e.escapeLeadingUnderscores(t):void 0}function kv(n,t){while(e.isAccessExpression(n))if(n=n.expression,Av(n,t))return!0;return!1}function Rv(n,t){while(e.isOptionalChain(n))if(n=n.expression,Av(n,t))return!0;return!1}function Ov(n,t){if(n&&1048576&n.flags){var r=yu(n,t);if(r&&2&e.getCheckFlags(r))return void 0===r.isDiscriminantProperty&&(r.isDiscriminantProperty=192===(192&r.checkFlags)&&!iA(Ws(r),331612160)),!!r.isDiscriminantProperty}return!1}function wv(e,n){for(var t,r=0,i=e;r<i.length;r++){var a=i[r];if(Ov(n,a.escapedName)){if(t){t.push(a);continue}t=[a]}}return t}function Pv(n,t){for(var r=new e.Map,i=0,a=function(e){if(61603840&e.flags){var n=Bo(e,t);if(n){if(!Uy(n))return{value:void 0};var a=!1;vb(n,function(n){var t=nf(Ym(n)),i=r.get(t);i?i!==Pe&&(r.set(t,Pe),a=!0):r.set(t,e)}),a||i++}}},o=0,s=n;o<s.length;o++){var c=s[o],l=a(c);if("object"===typeof l)return l.value}return i>=10&&2*i>=n.length?r:void 0}function Mv(n){var t=n.types;if(!(t.length<10||65536&e.getObjectFlags(n))){if(void 0===n.keyPropertyName){var r=e.forEach(t,function(n){return 59506688&n.flags?e.forEach(Hl(n),function(e){return Fy(Ws(e))?e.escapedName:void 0}):void 0}),i=r&&Pv(t,r);n.keyPropertyName=i?r:"",n.constituentMap=i}return n.keyPropertyName.length?n.keyPropertyName:void 0}}function Fv(e,n){var t,r=null===(t=e.constituentMap)||void 0===t?void 0:t.get(nf(Ym(n)));return r!==Pe?r:void 0}function Gv(e,n){var t=Mv(e),r=t&&Bo(n,t);return r&&Fv(e,r)}function Bv(n,t){var r=Mv(n),i=r&&e.find(t.properties,function(e){return e.symbol&&291===e.kind&&e.symbol.escapedName===r&&fT(e.initializer)}),a=i&&QA(i.initializer);return a&&Fv(n,a)}function Uv(e,n){return Av(e,n)||kv(e,n)}function Vv(e,n){if(e.arguments)for(var t=0,r=e.arguments;t<r.length;t++){var i=r[t];if(Uv(n,i))return!0}return!(204!==e.expression.kind||!Uv(n,e.expression.expression))}function jv(e){return(!e.id||e.id<0)&&(e.id=f,f++),e.id}function Kv(e,n){if(!(1048576&e.flags))return lg(e,n);for(var t=0,r=e.types;t<r.length;t++){var i=r[t];if(lg(i,n))return!0}return!1}function Hv(e,n){if(e!==n){if(131072&n.flags)return n;var t=Sb(e,function(e){return Kv(n,e)});if(512&n.flags&&Qm(n)&&(t=Db(t,Xm)),lg(n,t))return t}return e}function Wv(e){var n=Ul(e);return!!(n.callSignatures.length||n.constructSignatures.length||n.members.get("bind")&&cg(e,Cn))}function zv(n,t){void 0===t&&(t=!1);var r=n.flags;if(4&r)return z?16317953:16776705;if(128&r){var i=""===n.value;return z?i?12123649:7929345:i?12582401:16776705}if(40&r)return z?16317698:16776450;if(256&r){var a=0===n.value;return z?a?12123394:7929090:a?12582146:16776450}if(64&r)return z?16317188:16775940;if(2048&r){a=eh(n);return z?a?12122884:7928580:a?12581636:16775940}return 16&r?z?16316168:16774920:528&r?z?n===We||n===ze?12121864:7927560:n===We||n===ze?12580616:16774920:524288&r&&!t?16&e.getObjectFlags(n)&&Gg(n)?z?16318463:16777215:Wv(n)?z?7880640:16728e3:z?7888800:16736160:49152&r?9830144:65536&r?9363232:12288&r?z?7925520:16772880:67108864&r?z?7888800:16736160:131072&r?0:465829888&r?am(n)?z?7929345:16776705:zv(ru(n)||Pe,t):1048576&r?e.reduceLeft(n.types,function(e,n){return e|zv(n,t)},0):2097152&r?(t||(t=iA(n,131068)),e.reduceLeft(n.types,function(e,n){return e&zv(n,t)},16777215)):16777215}function Jv(e,n){return Sb(e,function(e){return 0!==(zv(e)&n)})}function qv(e,n){return n?ff([Wo(e),QA(n)]):e}function Xv(e,n){var t,r=Ff(n);if(!Fc(r))return Re;var i=Kc(r);return Bo(e,i)||Qv(null===(t=Fu(e,i))||void 0===t?void 0:t.type)||Re}function Yv(e,n){return Eb(e,Oy)&&Py(e,n)||Qv(fN(65,e,Me,void 0))||Re}function Qv(e){return e&&M.noUncheckedIndexedAccess?ff([e,Me]):e}function Zv(e){return Mp(fN(65,e,Me,void 0)||Re)}function $v(e){var n=202===e.parent.kind&&eb(e.parent)||291===e.parent.kind&&eb(e.parent.parent);return n?qv(ab(e),e.right):QA(e.right)}function eb(e){return 219===e.parent.kind&&e.parent.left===e||242===e.parent.kind&&e.parent.initializer===e}function nb(e,n){return Yv(ab(e),e.elements.indexOf(n))}function tb(e){return Zv(ab(e.parent))}function rb(e){return Xv(ab(e.parent),e.name)}function ib(e){return qv(rb(e),e.objectAssignmentInitializer)}function ab(e){var n=e.parent;switch(n.kind){case 241:return je;case 242:return pN(n)||Re;case 219:return $v(n);case 213:return Me;case 202:return nb(n,e);case 223:return tb(n);case 291:return rb(n);case 292:return ib(n)}return Re}function ob(e){var n=e.parent,t=lb(n.parent),r=199===n.kind?Xv(t,e.propertyName||e.name):e.dotDotDotToken?Zv(t):Yv(t,n.elements.indexOf(e));return qv(r,e.initializer)}function sb(e){var n=Wr(e);return n.resolvedType||QA(e)}function cb(e){return e.initializer?sb(e.initializer):241===e.parent.parent.kind?je:242===e.parent.parent.kind&&pN(e.parent.parent)||Re}function lb(e){return 252===e.kind?cb(e):ob(e)}function ub(e){return 252===e.kind&&e.initializer&&es(e.initializer)||201!==e.kind&&219===e.parent.kind&&es(e.parent.right)}function db(e){switch(e.kind){case 210:return db(e.expression);case 219:switch(e.operatorToken.kind){case 63:case 75:case 76:case 77:return db(e.left);case 27:return db(e.right)}}return e}function pb(e){var n=e.parent;return 210===n.kind||219===n.kind&&63===n.operatorToken.kind&&n.left===e||219===n.kind&&27===n.operatorToken.kind&&n.right===e?pb(n):e}function fb(e){return 287===e.kind?Ym(QA(e.expression)):Ze}function mb(e){var n=Wr(e);if(!n.switchTypes){n.switchTypes=[];for(var t=0,r=e.caseBlock.clauses;t<r.length;t++){var i=r[t];n.switchTypes.push(fb(i))}}return n.switchTypes}function _b(n,t){for(var r=[],i=0,a=n.caseBlock.clauses;i<a.length;i++){var o=a[i];if(287===o.kind){if(e.isStringLiteralLike(o.expression)){r.push(o.expression.text);continue}return e.emptyArray}t&&r.push(void 0)}return r}function gb(n,t){return 1048576&n.flags?!e.forEach(n.types,function(n){return!e.contains(t,n)}):e.contains(t,n)}function yb(e,n){return e===n||1048576&n.flags&&hb(e,n)}function hb(e,n){if(1048576&e.flags){for(var t=0,r=e.types;t<r.length;t++){var i=r[t];if(!tf(n.types,i))return!1}return!0}return!!(1024&e.flags&&Ec(e)===n)||tf(n.types,e)}function vb(n,t){return 1048576&n.flags?e.forEach(n.types,t):t(n)}function bb(n,t){return 1048576&n.flags?e.some(n.types,t):t(n)}function Eb(n,t){return 1048576&n.flags?e.every(n.types,t):t(n)}function Tb(n,t){return 3145728&n.flags?e.every(n.types,t):t(n)}function Sb(n,t){if(1048576&n.flags){var r=n.types,i=e.filter(r,t);if(i===r)return n;var a=n.origin,o=void 0;if(a&&1048576&a.flags){var s=a.types,c=e.filter(s,function(e){return!!(1048576&e.flags)||t(e)});if(s.length-c.length===r.length-i.length){if(1===c.length)return c[0];o=pf(1048576,c)}}return gf(i,n.objectFlags,void 0,void 0,o)}return 131072&n.flags||t(n)?n:Ze}function xb(e,n){return Sb(e,function(e){return e!==n})}function Lb(e){return 1048576&e.flags?e.types.length:1}function Db(e,n,t){if(131072&e.flags)return e;if(!(1048576&e.flags))return n(e);for(var r,i=e.origin,a=i&&1048576&i.flags?i.types:e.types,o=!1,s=0,c=a;s<c.length;s++){var l=c[s],u=1048576&l.flags?Db(l,n,t):n(l);o||(o=l!==u),u&&(r?r.push(u):r=[u])}return o?r&&ff(r,t?0:1):e}function Ab(n,t,r,i){return 1048576&n.flags&&r?ff(e.map(n.types,t),1,r,i):Db(n,t)}function Cb(e){return 3145728&e.flags?e.types.length:1}function Ib(e,n){return Sb(e,function(e){return 0!==(e.flags&n)})}function Nb(e,n){return yb(je,e)&&iA(n,128)||yb(Ke,e)&&iA(n,256)||yb(He,e)&&iA(n,2048)?Db(e,function(e){return 4&e.flags?Ib(n,132):8&e.flags?Ib(n,264):64&e.flags?Ib(n,2112):e}):e}function kb(e){return 0===e.flags}function Rb(e){return 0===e.flags?e.type:e}function Ob(e,n){return n?{flags:0,type:131072&e.flags?$e:e}:e}function wb(e){var n=ja(256);return n.elementType=e,n}function Pb(e){return Le[e.id]||(Le[e.id]=wb(e))}function Mb(e,n){var t=Eh(Vy($A(n)));return yb(t,e.elementType)?e:Pb(ff([e.elementType,t]))}function Fb(e){return 131072&e.flags?Bn:Mp(1048576&e.flags?ff(e.types,2):e)}function Gb(e){return e.finalArrayType||(e.finalArrayType=Fb(e.elementType))}function Bb(n){return 256&e.getObjectFlags(n)?Gb(n):n}function Ub(n){return 256&e.getObjectFlags(n)?n.elementType:Ze}function Vb(n){for(var t=!1,r=0,i=n;r<i.length;r++){var a=i[r];if(!(131072&a.flags)){if(!(256&e.getObjectFlags(a)))return!1;t=!0}}return t}function jb(n){var t=pb(n),r=t.parent,i=e.isPropertyAccessExpression(r)&&("length"===r.name.escapedText||206===r.parent.kind&&e.isIdentifier(r.name)&&e.isPushOrUnshiftIdentifier(r.name)),a=205===r.kind&&r.expression===t&&219===r.parent.kind&&63===r.parent.operatorToken.kind&&r.parent.left===r&&!e.isAssignmentTarget(r.parent)&&aA(QA(r.argumentExpression),296);return i||a}function Kb(n){return(252===n.kind||162===n.kind||165===n.kind||164===n.kind)&&!!e.getEffectiveTypeAnnotationNode(n)}function Hb(n,t){if(8752&n.flags)return Ws(n);if(7&n.flags){if(262144&e.getCheckFlags(n)){var r=n.syntheticOrigin;if(r&&Hb(r))return Ws(n)}var i=n.valueDeclaration;if(i){if(Kb(i))return Ws(n);if(e.isVariableDeclaration(i)&&242===i.parent.parent.kind){var a=i.parent.parent,o=Wb(a.expression,void 0);if(o){var s=a.awaitModifier?15:13;return fN(s,o,Me,void 0)}}t&&e.addRelatedInfo(t,e.createDiagnosticForNode(i,e.Diagnostics._0_needs_an_explicit_type_annotation,mo(n)))}}}function Wb(n,t){if(!(16777216&n.flags))switch(n.kind){case 79:var r=Ma(xv(n));return Hb(2097152&r.flags?Ji(r):r,t);case 108:return NE(n);case 106:return wE(n);case 204:var i=Wb(n.expression,t);if(i){var a=n.name,o=void 0;if(e.isPrivateIdentifier(a)){if(!i.symbol)return;o=Lu(i,e.getSymbolNameForPrivateIdentifier(i.symbol,a.escapedText))}else o=Lu(i,a.escapedText);return o&&Hb(o,t)}return;case 210:return Wb(n.expression,t)}}function zb(n){var t=Wr(n),r=t.effectsSignature;if(void 0===r){var i=void 0;236===n.parent.kind?i=Wb(n.expression,void 0):106!==n.expression.kind&&(i=e.isOptionalChain(n)?jS(fh(eC(n.expression),n.expression),n.expression):MS(n.expression));var a=Au(i&&mu(i)||Pe,0),o=1!==a.length||a[0].typeParameters?e.some(a,Jb)?ML(n):void 0:a[0];r=t.effectsSignature=o&&Jb(o)?o:Et}return r===Et?void 0:r}function Jb(e){return!!(td(e)||e.declaration&&131072&(od(e.declaration)||Pe).flags)}function qb(n,t){if(1===n.kind||3===n.kind)return t.arguments[n.parameterIndex];var r=e.skipParentheses(t.expression);return e.isAccessExpression(r)?e.skipParentheses(r.expression):void 0}function Xb(n){var t=e.findAncestor(n,e.isFunctionOrModuleBlock),r=e.getSourceFileOfNode(n),i=e.getSpanOfTokenAtPosition(r,t.statements.pos);ur.add(e.createFileDiagnostic(r,i.start,i.length,e.Diagnostics.The_containing_function_or_module_body_is_too_large_for_control_flow_analysis))}function Yb(e){var n=Zb(e,!1);return pt=e,ft=n,n}function Qb(n){var t=e.skipParentheses(n);return 95===t.kind||219===t.kind&&(55===t.operatorToken.kind&&(Qb(t.left)||Qb(t.right))||56===t.operatorToken.kind&&Qb(t.left)&&Qb(t.right))}function Zb(n,t){while(1){if(n===pt)return ft;var r=n.flags;if(4096&r){if(!t){var i=jv(n),a=rr[i];return void 0!==a?a:rr[i]=Zb(n,!0)}t=!1}if(368&r)n=n.antecedent;else if(512&r){var o=zb(n.node);if(o){var s=td(o);if(s&&3===s.kind&&!s.type){var c=n.node.arguments[s.parameterIndex];if(c&&Qb(c))return!1}if(131072&ad(o).flags)return!1}n=n.antecedent}else{if(4&r)return e.some(n.antecedents,function(e){return Zb(e,!1)});if(8&r){var l=n.antecedents;if(void 0===l||0===l.length)return!1;n=l[0]}else{if(!(128&r)){if(1024&r){pt=void 0;var u=n.target,d=u.antecedents;u.antecedents=n.antecedents;var p=Zb(n.antecedent,!1);return u.antecedents=d,p}return!(1&r)}if(n.clauseStart===n.clauseEnd&&MD(n.switchStatement))return!1;n=n.antecedent}}}}function $b(n,t){while(1){var r=n.flags;if(4096&r){if(!t){var i=jv(n),a=ir[i];return void 0!==a?a:ir[i]=$b(n,!0)}t=!1}if(496&r)n=n.antecedent;else if(512&r){if(106===n.node.expression.kind)return!0;n=n.antecedent}else{if(4&r)return e.every(n.antecedents,function(e){return $b(e,!1)});if(!(8&r)){if(1024&r){var o=n.target,s=o.antecedents;o.antecedents=n.antecedents;var c=$b(n.antecedent,!1);return o.antecedents=s,c}return!!(1&r)}n=n.antecedents[0]}}}function eE(n){switch(n.kind){case 79:var t=xv(n);return sE(t)||!!t.valueDeclaration&&162===e.getRootDeclaration(t.valueDeclaration).kind&&!iE(t);case 204:case 205:return eE(n.expression)&&JD(Wr(n).resolvedSymbol||Ae)}return!1}function nE(n,t,r,i){var a;void 0===r&&(r=t);var o=!1,s=0,c=0;if(Gt)return Re;if(!n.flowNode)return t;Bt++;var l=Ft,u=Rb(f(n.flowNode));Ft=l;var d=256&e.getObjectFlags(u)&&jb(n)?Bn:Bb(u);return d===tn||n.parent&&228===n.parent.kind&&!(131072&d.flags)&&131072&Jv(d,2097152).flags?t:d;function p(){return o?a:(o=!0,a=Dv(n,t,r,i))}function f(a){if(2e3===s)return null===e.tracing||void 0===e.tracing||e.tracing.instant("checkTypes","getTypeAtFlowNode_DepthLimit",{flowId:a.id}),Gt=!0,Xb(n),Re;var o;s++;while(1){var c=a.flags;if(4096&c){for(var u=l;u<Ft;u++)if(nr[u]===a)return s--,tr[u];o=a}var d=void 0;if(16&c){if(d=_(a),!d){a=a.antecedent;continue}}else if(512&c){if(d=y(a),!d){a=a.antecedent;continue}}else if(96&c)d=v(a);else if(128&c)d=b(a);else if(12&c){if(1===a.antecedents.length){a=a.antecedents[0];continue}d=4&c?E(a):x(a)}else if(256&c){if(d=h(a),!d){a=a.antecedent;continue}}else if(1024&c){var p=a.target,m=p.antecedents;p.antecedents=a.antecedents,d=f(a.antecedent),p.antecedents=m}else if(2&c){var g=a.node;if(g&&g!==i&&204!==n.kind&&205!==n.kind&&108!==n.kind){a=g.flowNode;continue}d=r}else d=zI(t);return o&&(nr[Ft]=o,tr[Ft]=d,Ft++),s--,d}}function m(e){var t=e.node;return fE(252===t.kind||201===t.kind?lb(t):ab(t),n)}function _(r){var i=r.node;if(Av(n,i)){if(!Yb(r))return tn;if(2===e.getAssignmentTargetKind(i)){var a=f(r.antecedent);return Ob(Vy(Rb(a)),kb(a))}if(t===Ne||t===Bn){if(ub(i))return Pb(Ze);var o=jy(m(r));return lg(o,t)?o:Gn}return 1048576&t.flags?Hv(t,m(r)):t}if(kv(n,i)){if(!Yb(r))return tn;if(e.isVariableDeclaration(i)&&(e.isInJSFile(i)||e.isVarConst(i))){var s=e.getDeclaredExpandoInitializer(i);if(s&&(211===s.kind||212===s.kind))return f(r.antecedent)}return t}if(e.isVariableDeclaration(i)&&241===i.parent.parent.kind&&Av(n,i.parent.parent.expression))return GS(Rb(f(r.antecedent)))}function g(n,t){var r=e.skipParentheses(t);if(95===r.kind)return tn;if(219===r.kind){if(55===r.operatorToken.kind)return g(g(n,r.left),r.right);if(56===r.operatorToken.kind)return ff([g(n,r.left),g(n,r.right)])}return X(n,r,!0)}function y(e){var n=zb(e.node);if(n){var t=td(n);if(t&&(2===t.kind||3===t.kind)){var r=f(e.antecedent),i=Bb(Rb(r)),a=t.type?q(i,t,e.node,!0):3===t.kind&&t.parameterIndex>=0&&t.parameterIndex<e.node.arguments.length?g(i,e.node.arguments[t.parameterIndex]):i;return a===i?r:Ob(a,kb(r))}if(131072&ad(n).flags)return tn}}function h(r){if(t===Ne||t===Bn){var i=r.node,a=206===i.kind?i.expression.expression:i.left.expression;if(Av(n,db(a))){var o=f(r.antecedent),s=Rb(o);if(256&e.getObjectFlags(s)){var c=s;if(206===i.kind)for(var l=0,u=i.arguments;l<u.length;l++){var d=u[l];c=Mb(c,d)}else{var p=$A(i.left.argumentExpression);aA(p,296)&&(c=Mb(c,i.right))}return c===s?o:Ob(c,kb(o))}return o}}}function v(e){var n=f(e.antecedent),t=Rb(n);if(131072&t.flags)return n;var r=0!==(32&e.flags),i=Bb(t),a=X(i,e.node,r);return a===i?n:Ob(a,kb(n))}function b(e){var t=e.switchStatement.expression,r=f(e.antecedent),i=Rb(r);if(Av(n,t))i=G(i,e.switchStatement,e.clauseStart,e.clauseEnd);else if(214===t.kind&&Av(n,t.expression))i=V(i,e.switchStatement,e.clauseStart,e.clauseEnd);else{z&&(Rv(t,n)?i=F(i,e.switchStatement,e.clauseStart,e.clauseEnd,function(e){return!(163840&e.flags)}):214===t.kind&&Rv(t.expression,n)&&(i=F(i,e.switchStatement,e.clauseStart,e.clauseEnd,function(e){return!(131072&e.flags||128&e.flags&&"undefined"===e.value)})));var a=D(t,i);a&&(i=I(i,a,e.switchStatement,e.clauseStart,e.clauseEnd))}return Ob(i,kb(r))}function E(n){for(var i,a=[],o=!1,s=!1,c=0,l=n.antecedents;c<l.length;c++){var u=l[c];if(!i&&128&u.flags&&u.clauseStart===u.clauseEnd)i=u;else{var d=f(u),p=Rb(d);if(p===t&&t===r)return p;e.pushIfUnique(a,p),yb(p,t)||(o=!0),kb(d)&&(s=!0)}}if(i){d=f(i),p=Rb(d);if(!e.contains(a,p)&&!MD(i.switchStatement)){if(p===t&&t===r)return p;a.push(p),yb(p,t)||(o=!0),kb(d)&&(s=!0)}}return Ob(L(a,o?2:1),s)}function x(n){var r=jv(n),i=Qt[r]||(Qt[r]=new e.Map),a=p();if(!a)return t;var o=i.get(a);if(o)return o;for(var s=Pt;s<Mt;s++)if(Zt[s]===n&&$t[s]===a&&er[s].length)return Ob(L(er[s],1),!0);for(var c,l=[],u=!1,d=0,m=n.antecedents;d<m.length;d++){var _=m[d],g=void 0;if(c){Zt[Mt]=n,$t[Mt]=a,er[Mt]=l,Mt++;var y=mt;mt=void 0,g=f(_),mt=y,Mt--;var h=i.get(a);if(h)return h}else g=c=f(_);var v=Rb(g);if(e.pushIfUnique(l,v),yb(v,t)||(u=!0),v===t)break}var b=L(l,u?2:1);return kb(c)?Ob(b,!0):(i.set(a,b),b)}function L(n,r){if(Vb(n))return Pb(ff(e.map(n,Ub)));var i=ff(e.sameMap(n,Bb),r);return i!==t&&i.flags&t.flags&1048576&&e.arraysEqual(i.types,t.types)?t:i}function D(r,i){var a,o,s=1048576&t.flags?t:i;return 1048576&s.flags&&(a=Iv(r))&&(o=Nv(a))&&Av(n,e.isAccessExpression(a)?a.expression:a.parent.parent.initializer)&&Ov(s,o)?a:void 0}function A(n,t,r){var i=Nv(t);if(void 0===i)return n;var a=z&&e.isOptionalChain(t)&&iA(n,98304),o=Bo(a?Jv(n,2097152):n,i);if(!o)return n;o=a?sh(o):o;var s=r(o);return Sb(n,function(e){var n=Uo(e,i);return!(131072&n.flags)&&dg(n,s)})}function C(e,n,t,r,i){if((36===t||37===t)&&1048576&e.flags){var a=Mv(e);if(a&&a===Nv(n)){var o=Fv(e,QA(r));if(o)return t===(i?36:37)?o:Fy(Bo(o,a)||Pe)?xb(e,o):e}}return A(e,n,function(e){return P(e,t,r,i)})}function I(n,t,r,i,a){if(i<a&&1048576&n.flags&&Mv(n)===Nv(t)){var o=mb(r).slice(i,a),s=ff(e.map(o,function(e){return Fv(n,e)||Pe}));if(s!==Pe)return s}return A(n,t,function(e){return G(e,r,i,a)})}function N(e,t,r){if(Av(n,t))return Jv(e,r?4194304:8388608);z&&r&&Rv(t,n)&&(e=Jv(e,2097152));var i=D(t,e);return i?A(e,i,function(e){return Jv(e,r?4194304:8388608)}):e}function k(e,n,t){var r=Lu(e,n);return r?!!(16777216&r.flags)||t:!!Fu(e,n)||!t}function R(n,r,i){return 1048576&n.flags||524288&n.flags&&t!==n||um(n)||2097152&n.flags&&e.every(n.types,function(e){return e.symbol!==se})?Sb(n,function(e){return k(e,r,i)}):n}function O(t,r,i){switch(r.operatorToken.kind){case 63:case 75:case 76:case 77:return N(X(t,r.right,i),r.left,i);case 34:case 35:case 36:case 37:var a=r.operatorToken.kind,o=db(r.left),s=db(r.right);if(214===o.kind&&e.isStringLiteralLike(s))return M(t,o,a,s,i);if(214===s.kind&&e.isStringLiteralLike(o))return M(t,s,a,o,i);if(Av(n,o))return P(t,a,s,i);if(Av(n,s))return P(t,a,o,i);z&&(Rv(o,n)?t=w(t,a,s,i):Rv(s,n)&&(t=w(t,a,o,i)));var c=D(o,t);if(c)return C(t,c,a,s,i);var l=D(s,t);if(l)return C(t,l,a,o,i);if(j(o))return K(t,a,s,i);if(j(s))return K(t,a,o,i);break;case 102:return H(t,r,i);case 101:var u=db(r.right),d=RR(r.left);if(128&d.flags){var p=e.escapeLeadingUnderscores(d.value);if(_h(t)&&e.isAccessExpression(n)&&Av(n.expression,u)&&Nv(n)===p)return Jv(t,i?524288:65536);if(Av(n,u))return R(t,p,i)}break;case 27:return X(t,r.right,i);case 55:return i?X(X(t,r.left,!0),r.right,!0):ff([X(t,r.left,!1),X(t,r.right,!1)]);case 56:return i?ff([X(t,r.left,!0),X(t,r.right,!0)]):X(X(t,r.left,!1),r.right,!1)}return t}function w(e,n,t,r){var i=34===n||36===n,a=34===n||35===n?98304:32768,o=QA(t),s=i!==r&&Eb(o,function(e){return!!(e.flags&a)})||i===r&&Eb(o,function(e){return!(e.flags&(3|a))});return s?Jv(e,2097152):e}function P(e,n,t,r){if(1&e.flags)return e;35!==n&&37!==n||(r=!r);var i=QA(t);if(2&e.flags&&r&&(36===n||37===n))return 67239932&i.flags?i:524288&i.flags?rn:e;if(98304&i.flags){if(!z)return e;var a=34===n||35===n,o=a?r?262144:2097152:65536&i.flags?r?131072:1048576:r?65536:524288;return Jv(e,o)}if(r){var s=34===n?function(e){return pg(e,i)||yh(e,i)}:function(e){return pg(e,i)};return Nb(Sb(e,s),i)}return Fy(i)?Sb(e,function(e){return!(Gy(e)&&pg(e,i))}):e}function M(e,t,r,i,a){35!==r&&37!==r||(a=!a);var o=db(t.expression);if(!Av(n,o))return z&&Rv(o,n)&&a===("undefined"!==i.text)?Jv(e,2097152):e;if(1&e.flags&&"function"===i.text)return e;if(a&&2&e.flags&&"object"===i.text){if(219===t.parent.parent.kind){var s=t.parent.parent;if(55===s.operatorToken.kind&&s.right===t.parent&&Cv(n,s.left))return rn}return ff([rn,Ue])}var c=a?T.get(i.text)||128:S.get(i.text)||32768,l=B(e,i.text);return Jv(a&&l?Db(e,U(l)):e,c)}function F(n,t,r,i,a){var o=r!==i&&e.every(mb(t).slice(r,i),a);return o?Jv(n,2097152):n}function G(n,t,r,i){var a=mb(t);if(!a.length)return n;var o=a.slice(r,i),s=r===i||e.contains(o,Ze);if(2&n.flags&&!s){for(var c=void 0,l=0;l<o.length;l+=1){var u=o[l];if(67239932&u.flags)void 0!==c&&c.push(u);else{if(!(524288&u.flags))return n;void 0===c&&(c=o.slice(0,l)),c.push(rn)}}return ff(void 0===c?o:c)}var d=ff(o),p=131072&d.flags?Ze:Nb(Sb(n,function(e){return pg(d,e)}),d);if(!s)return p;var f=Sb(n,function(n){return!(Gy(n)&&e.contains(a,Ym(By(n))))});return 131072&p.flags?f:ff([p,f])}function B(e,n){switch(n){case"function":return 1&e.flags?e:Cn;case"object":return 2&e.flags?ff([rn,Ue]):e;default:return pr.get(n)}}function U(e){return function(n){if(cg(n,e))return n;if(cg(e,n))return e;if(465829888&n.flags){var t=ru(n)||Ie;if(cg(e,t))return Df([n,e])}return n}}function V(n,t,r,i){var a=_b(t,!0);if(!a.length)return n;var o,s,c=e.findIndex(a,function(e){return void 0===e}),l=r===i||c>=r&&c<i;if(c>-1){var u=a.filter(function(e){return void 0!==e}),d=c<r?r-1:r,p=c<i?i-1:i;o=u.slice(d,p),s=PD(d,p,u,l)}else o=a.slice(r,i),s=PD(r,i,a,l);if(l)return Sb(n,function(e){return(zv(e)&s)===s});var f=Jv(ff(o.map(function(e){return B(n,e)||n})),s);return Jv(Db(n,U(f)),s)}function j(t){return(e.isPropertyAccessExpression(t)&&"constructor"===e.idText(t.name)||e.isElementAccessExpression(t)&&e.isStringLiteralLike(t.argumentExpression)&&"constructor"===t.argumentExpression.text)&&Av(n,t.expression)}function K(n,t,r,i){if(i?34!==t&&36!==t:35!==t&&37!==t)return n;var a=QA(r);if(!fO(a)&&!tc(a))return n;var o=Lu(a,"prototype");if(!o)return n;var s=Ws(o),c=Vo(s)?void 0:s;return c&&c!==An&&c!==Cn?Vo(n)?c:Sb(n,function(e){return l(e,c)}):n;function l(n,t){return 524288&n.flags&&1&e.getObjectFlags(n)||524288&t.flags&&1&e.getObjectFlags(t)?n.symbol===t.symbol:cg(n,t)}}function H(t,r,i){var a=db(r.left);if(!Av(n,a))return i&&z&&Rv(a,n)?Jv(t,2097152):t;var o,s=QA(r.right);if(!ug(s,Cn))return t;var c=Lu(s,"prototype");if(c){var l=Ws(c);Vo(l)||(o=l)}if(Vo(t)&&(o===An||o===Cn))return t;if(!o){var u=Au(s,1);o=u.length?ff(e.map(u,function(e){return ad(md(e))})):pn}if(!i&&1048576&s.flags){var d=e.find(s.types,function(e){return!tc(e)});if(!d)return t}return W(t,o,i,ug)}function W(e,n,t,r){if(!t)return Sb(e,function(e){if(!r(e,n))return!0;var t=ru(e);return!(!t||t===e)&&!r(t,n)});if(1048576&e.flags){var i=Sb(e,function(e){return r(e,n)});if(!(131072&i.flags))return i}return cg(n,e)?n:lg(e,n)?e:lg(n,e)?n:Df([e,n])}function J(t,r,i){if(Vv(r,n)){var a=i||!e.isCallChain(r)?zb(r):void 0,o=a&&td(a);if(o&&(0===o.kind||1===o.kind))return q(t,o,r,i)}if(_h(t)&&e.isAccessExpression(n)&&e.isPropertyAccessExpression(r.expression)){var s=r.expression;if(Av(n.expression,db(s.expression))&&e.isIdentifier(s.name)&&"hasOwnProperty"===s.name.escapedText&&1===r.arguments.length){var c=r.arguments[0];if(e.isStringLiteralLike(c)&&Nv(n)===e.escapeLeadingUnderscores(c.text))return Jv(t,i?524288:65536)}}return t}function q(e,t,r,i){if(t.type&&(!Vo(e)||t.type!==An&&t.type!==Cn)){var a=qb(t,r);if(a){if(Av(n,a))return W(e,t.type,i,cg);z&&i&&Rv(a,n)&&!(65536&zv(t.type))&&(e=Jv(e,2097152));var o=D(a,e);if(o)return A(e,o,function(e){return W(e,t.type,i,cg)})}}return e}function X(t,r,i){if(e.isExpressionOfOptionalChainRoot(r)||e.isBinaryExpression(r.parent)&&60===r.parent.operatorToken.kind&&r.parent.left===r)return Y(t,r,i);switch(r.kind){case 79:if(!Av(n,r)&&c<5){var a=xv(r);if(sE(a)){var o=a.valueDeclaration;if(o&&e.isVariableDeclaration(o)&&!o.type&&o.initializer&&eE(n)){c++;var s=X(t,o.initializer,i);return c--,s}}}case 108:case 106:case 204:case 205:return N(t,r,i);case 206:return J(t,r,i);case 210:case 228:return X(t,r.expression,i);case 219:return O(t,r,i);case 217:if(53===r.operator)return X(t,r.operand,!i);break}return t}function Y(e,t,r){if(Av(n,t))return Jv(e,r?2097152:262144);var i=D(t,e);return i?A(e,i,function(e){return Jv(e,r?2097152:262144)}):e}}function tE(n,t){if(n=n.exportSymbol||n,(79===t.kind||80===t.kind)&&(e.isRightSideOfQualifiedNameOrPropertyAccess(t)&&(t=t.parent),e.isExpressionNode(t)&&(!e.isAssignmentTarget(t)||e.isWriteAccess(t)))){var r=QA(t);if(Ma(Wr(t).resolvedSymbol)===n)return r}return e.isDeclarationName(t)&&e.isSetAccessor(t.parent)&&Cs(t.parent)?Ps(t.parent.symbol,!0):zs(n)}function rE(n){return e.findAncestor(n.parent,function(n){return e.isFunctionLike(n)&&!e.getImmediatelyInvokedFunctionExpression(n)||260===n.kind||300===n.kind||165===n.kind})}function iE(n){if(!n.valueDeclaration)return!1;var t=e.getRootDeclaration(n.valueDeclaration).parent,r=Wr(t);return 8388608&r.flags||(r.flags|=8388608,aE(t)||oE(t)),n.isAssigned||!1}function aE(n){return!!e.findAncestor(n.parent,function(n){return e.isFunctionLike(n)&&!!(8388608&Wr(n).flags)})}function oE(n){if(79===n.kind){if(e.isAssignmentTarget(n)){var t=xv(n);t.valueDeclaration&&162===e.getRootDeclaration(t.valueDeclaration).kind&&(t.isAssigned=!0)}}else e.forEachChild(n,oE)}function sE(e){return 3&e.flags&&0!==(2&kS(e))}function cE(e,n){if(Oo(n.symbol,2)){var t=z&&162===n.kind&&n.initializer&&32768&th(e)&&!(32768&th(eC(n.initializer)));return Mo(),t?Jv(e,524288):e}return js(n.symbol),e}function lE(e,n){var t=n.parent;return 204===t.kind||206===t.kind&&t.expression===n||205===t.kind&&t.expression===n&&!(dE(e)&&cm(QA(t.argumentExpression)))}function uE(e){return!!(465829888&e.flags&&1146880&iu(e).flags)}function dE(e){return!(!(465829888&e.flags)||iA(iu(e),98304))}function pE(n){var t=(e.isIdentifier(n)||e.isPropertyAccessExpression(n)||e.isElementAccessExpression(n))&&!((e.isJsxOpeningElement(n.parent)||e.isJsxSelfClosingElement(n.parent))&&n.parent.tagName===n)&&vT(n,8);return t&&!om(t)}function fE(e,n,t){var r=!(t&&2&t)&&bb(e,uE)&&(lE(e,n)||pE(n));return r?Db(e,function(e){return 465829888&e.flags?iu(e):e}):e}function mE(n){return!!e.findAncestor(n,function(n){return n.parent&&e.isExportAssignment(n.parent)&&n.parent.expression===n&&e.isEntityNameExpression(n)})}function _E(n,t){if(Wi(n,111551)&&!Lv(t)&&!Qi(n)){var r=Ji(n);111551&r.flags&&(M.isolatedModules||e.shouldPreserveConstEnums(M)&&mE(t)||!nO(r)?$i(n):ea(n))}}function gE(n,t){var r=xv(n);if(r===Ae)return Re;if(r===le){if(tx(n))return Dr(n,e.Diagnostics.arguments_cannot_be_referenced_in_property_initializers),Re;var i=e.getContainingFunction(n);return F<2&&(212===i.kind?Dr(n,e.Diagnostics.The_arguments_object_cannot_be_referenced_in_an_arrow_function_in_ES3_and_ES5_Consider_using_a_standard_function_expression):e.hasSyntacticModifier(i,256)&&Dr(n,e.Diagnostics.The_arguments_object_cannot_be_referenced_in_an_async_function_or_method_in_ES3_and_ES5_Consider_using_a_standard_function_or_method)),Wr(i).flags|=8192,Ws(r)}n.parent&&e.isPropertyAccessExpression(n.parent)&&n.parent.expression===n||_E(r,n);var a=Ma(r),o=2097152&a.flags?Ji(a):a;o.declarations&&134217728&kS(o)&&nm(n,o)&&kr(n,o.declarations,n.escapedText);var s=a.valueDeclaration;if(s&&32&a.flags)if(255===s.kind&&e.nodeIsDecorated(s)){i=e.getContainingClass(n);while(void 0!==i){if(i===s&&i.name!==n){Wr(s).flags|=16777216,Wr(n).flags|=33554432;break}i=e.getContainingClass(i)}}else if(224===s.kind){i=e.getThisContainer(n,!1);while(300!==i.kind){if(i.parent===s){(e.isPropertyDeclaration(i)&&e.isStatic(i)||e.isClassStaticBlockDeclaration(i))&&(Wr(s).flags|=16777216,Wr(n).flags|=33554432);break}i=e.getThisContainer(i,!1)}}bE(n,r);var c=Ws(a),l=e.getAssignmentTargetKind(n);if(l){if(!(3&a.flags)&&!(e.isInJSFile(n)&&512&a.flags)){var u=384&a.flags?e.Diagnostics.Cannot_assign_to_0_because_it_is_an_enum:32&a.flags?e.Diagnostics.Cannot_assign_to_0_because_it_is_a_class:1536&a.flags?e.Diagnostics.Cannot_assign_to_0_because_it_is_a_namespace:16&a.flags?e.Diagnostics.Cannot_assign_to_0_because_it_is_a_function:2097152&a.flags?e.Diagnostics.Cannot_assign_to_0_because_it_is_an_import:e.Diagnostics.Cannot_assign_to_0_because_it_is_not_a_variable;return Dr(n,u,mo(r)),Re}if(JD(a))return 3&a.flags?Dr(n,e.Diagnostics.Cannot_assign_to_0_because_it_is_a_constant,mo(r)):Dr(n,e.Diagnostics.Cannot_assign_to_0_because_it_is_a_read_only_property,mo(r)),Re}var d=2097152&a.flags;if(3&a.flags){if(1===l)return c}else{if(!d)return c;s=hi(r)}if(!s)return c;c=fE(c,n,t);var p=162===e.getRootDeclaration(s).kind,f=rE(s),m=rE(n),_=m!==f,g=n.parent&&n.parent.parent&&e.isSpreadAssignment(n.parent)&&eb(n.parent.parent),y=134217728&r.flags;while(m!==f&&(211===m.kind||212===m.kind||e.isObjectLiteralOrClassExpressionMethod(m))&&(sE(a)&&c!==Bn||p&&!iE(a)))m=rE(m);var h=p||d||_||g||y||e.isBindingElement(s)||c!==Ne&&c!==Bn&&(!z||0!==(16387&c.flags)||Lv(n)||273===n.parent.kind)||228===n.parent.kind||252===s.kind&&s.exclamationToken||8388608&s.flags,v=h?p?cE(c,s):c:c===Ne||c===Bn?Me:sh(c),b=nE(n,c,v,m);if(jb(n)||c!==Ne&&c!==Bn){if(!h&&!(32768&th(c))&&32768&th(b))return Dr(n,e.Diagnostics.Variable_0_is_used_before_being_assigned,mo(r)),c}else if(b===Ne||b===Bn)return Y&&(Dr(e.getNameOfDeclaration(s),e.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined,mo(r),go(b)),Dr(n,e.Diagnostics.Variable_0_implicitly_has_an_1_type,mo(r),go(b))),zI(b);return l?Vy(b):b}function yE(n,t){return!!e.findAncestor(n,function(n){return n===t?"quit":e.isFunctionLike(n)||n.parent&&e.isPropertyDeclaration(n.parent)&&!e.hasStaticModifier(n.parent)&&n.parent.initializer===n})}function hE(n,t){return e.findAncestor(n,function(e){return e===t?"quit":e===t.initializer||e===t.condition||e===t.incrementor||e===t.statement})}function vE(n){return e.findAncestor(n,function(n){return!n||e.nodeStartsNewLexicalEnvironment(n)?"quit":e.isIterationStatement(n,!1)})}function bE(n,t){if(!(F>=2||0===(34&t.flags)||!t.valueDeclaration||e.isSourceFile(t.valueDeclaration)||290===t.valueDeclaration.parent.kind)){var r=e.getEnclosingBlockScopeContainer(t.valueDeclaration),i=yE(n,r),a=vE(r);if(a){if(i){var o=!0;if(e.isForStatement(r)){var s=e.getAncestor(t.valueDeclaration,253);if(s&&s.parent===r){var c=hE(n.parent,r);if(c){var l=Wr(c);l.flags|=131072;var u=l.capturedBlockScopeBindings||(l.capturedBlockScopeBindings=[]);e.pushIfUnique(u,t),c===r.initializer&&(o=!1)}}}o&&(Wr(a).flags|=65536)}if(e.isForStatement(r)){s=e.getAncestor(t.valueDeclaration,253);s&&s.parent===r&&TE(n,r)&&(Wr(t.valueDeclaration).flags|=4194304)}Wr(t.valueDeclaration).flags|=524288}i&&(Wr(t.valueDeclaration).flags|=262144)}}function EE(n,t){var r=Wr(n);return!!r&&e.contains(r.capturedBlockScopeBindings,Ca(t))}function TE(n,t){var r=n;while(210===r.parent.kind)r=r.parent;var i=!1;if(e.isAssignmentTarget(r))i=!0;else if(217===r.parent.kind||218===r.parent.kind){var a=r.parent;i=45===a.operator||46===a.operator}return!!i&&!!e.findAncestor(r,function(e){return e===t?"quit":e===t.statement})}function SE(e,n){if(Wr(e).flags|=2,165===n.kind||169===n.kind){var t=n.parent;Wr(t).flags|=4}else Wr(n).flags|=4}function xE(n){return e.isSuperCall(n)?n:e.isFunctionLike(n)?void 0:e.forEachChild(n,xE)}function LE(e){var n=Ca(e),t=Dc(n),r=oc(t);return r===Ve}function DE(n,t,r){var i=t.parent,a=e.getClassExtendsHeritageElement(i);a&&!LE(i)&&n.flowNode&&!$b(n.flowNode,!1)&&Dr(n,r)}function AE(n,t){e.isPropertyDeclaration(t)&&e.hasStaticModifier(t)&&t.initializer&&e.textRangeContainsPositionInclusive(t.initializer,n.pos)&&e.length(t.parent.decorators)&&Dr(n,e.Diagnostics.Cannot_use_this_in_a_static_property_initializer_of_a_decorated_class)}function CE(n){var t=Lv(n),r=e.getThisContainer(n,!0),i=!1;switch(169===r.kind&&DE(n,r,e.Diagnostics.super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class),212===r.kind&&(r=e.getThisContainer(r,!1),i=!0),AE(n,r),r.kind){case 259:Dr(n,e.Diagnostics.this_cannot_be_referenced_in_a_module_or_namespace_body);break;case 258:Dr(n,e.Diagnostics.this_cannot_be_referenced_in_current_location);break;case 169:OE(n,r)&&Dr(n,e.Diagnostics.this_cannot_be_referenced_in_constructor_arguments);break;case 160:Dr(n,e.Diagnostics.this_cannot_be_referenced_in_a_computed_property_name);break}!t&&i&&F<2&&SE(n,r);var a=IE(n,!0,r);if(Q){var o=Ws(se);if(a===o&&i)Dr(n,e.Diagnostics.The_containing_arrow_function_captures_the_global_value_of_this);else if(!a){var s=Dr(n,e.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation);if(!e.isSourceFile(r)){var c=IE(r);c&&c!==o&&e.addRelatedInfo(s,e.createDiagnosticForNode(r,e.Diagnostics.An_outer_value_of_this_is_shadowed_by_this_container))}}}return a||Ie}function IE(n,t,r){void 0===t&&(t=!0),void 0===r&&(r=e.getThisContainer(n,!1));var i=e.isInJSFile(n);if(e.isFunctionLike(r)&&(!JE(n)||e.getThisParameter(r))){var a=ks(r)||i&&RE(r);if(!a){var o=kE(r);if(i&&o){var s=eC(o).symbol;s&&s.members&&16&s.flags&&(a=Dc(s).thisType)}else FL(r)&&(a=Dc(Aa(r.symbol)).thisType);a||(a=GE(r))}if(a)return nE(n,a)}if(e.isClassLike(r.parent)){var c=Ca(r.parent),l=e.isStatic(r)?Ws(c):Dc(c).thisType;return nE(n,l)}if(e.isSourceFile(r)){if(r.commonJsModuleIndicator){var u=Ca(r);return u&&Ws(u)}if(r.externalModuleIndicator)return Me;if(t)return Ws(se)}}function NE(n){var t=e.getThisContainer(n,!1);if(e.isFunctionLike(t)){var r=qu(t);if(r.thisParameter)return Hb(r.thisParameter)}if(e.isClassLike(t.parent)){var i=Ca(t.parent);return e.isStatic(t)?Ws(i):Dc(i).thisType}}function kE(n){return 211===n.kind&&e.isBinaryExpression(n.parent)&&3===e.getAssignmentDeclarationKind(n.parent)?n.parent.left.expression.expression:167===n.kind&&203===n.parent.kind&&e.isBinaryExpression(n.parent.parent)&&6===e.getAssignmentDeclarationKind(n.parent.parent)?n.parent.parent.left.expression:211===n.kind&&291===n.parent.kind&&203===n.parent.parent.kind&&e.isBinaryExpression(n.parent.parent.parent)&&6===e.getAssignmentDeclarationKind(n.parent.parent.parent)?n.parent.parent.parent.left.expression:211===n.kind&&e.isPropertyAssignment(n.parent)&&e.isIdentifier(n.parent.name)&&("value"===n.parent.name.escapedText||"get"===n.parent.name.escapedText||"set"===n.parent.name.escapedText)&&e.isObjectLiteralExpression(n.parent.parent)&&e.isCallExpression(n.parent.parent.parent)&&n.parent.parent.parent.arguments[2]===n.parent.parent&&9===e.getAssignmentDeclarationKind(n.parent.parent.parent)?n.parent.parent.parent.arguments[0].expression:e.isMethodDeclaration(n)&&e.isIdentifier(n.name)&&("value"===n.name.escapedText||"get"===n.name.escapedText||"set"===n.name.escapedText)&&e.isObjectLiteralExpression(n.parent)&&e.isCallExpression(n.parent.parent)&&n.parent.parent.arguments[2]===n.parent&&9===e.getAssignmentDeclarationKind(n.parent.parent)?n.parent.parent.arguments[0].expression:void 0}function RE(n){var t=e.getJSDocType(n);if(t&&312===t.kind){var r=t;if(r.parameters.length>0&&r.parameters[0].name&&"this"===r.parameters[0].name.escapedText)return u_(r.parameters[0].type)}var i=e.getJSDocThisTag(n);if(i&&i.typeExpression)return u_(i.typeExpression)}function OE(n,t){return!!e.findAncestor(n,function(n){return e.isFunctionLikeDeclaration(n)?"quit":162===n.kind&&n.parent===t})}function wE(n){var t=206===n.parent.kind&&n.parent.expression===n,r=e.getSuperContainer(n,!0),i=r,a=!1;if(!t)while(i&&212===i.kind)i=e.getSuperContainer(i,!0),a=F<2;var o=p(i),s=0;if(!o){var c=e.findAncestor(n,function(e){return e===i?"quit":160===e.kind});return c&&160===c.kind?Dr(n,e.Diagnostics.super_cannot_be_referenced_in_a_computed_property_name):t?Dr(n,e.Diagnostics.Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors):i&&i.parent&&(e.isClassLike(i.parent)||203===i.parent.kind)?Dr(n,e.Diagnostics.super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_derived_class):Dr(n,e.Diagnostics.super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions),Re}if(t||169!==r.kind||DE(n,i,e.Diagnostics.super_must_be_called_before_accessing_a_property_of_super_in_the_constructor_of_a_derived_class),e.isStatic(i)||t?(s=512,!t&&F>=2&&F<=8&&(e.isPropertyDeclaration(i)||e.isClassStaticBlockDeclaration(i))&&e.forEachEnclosingBlockScopeContainer(n.parent,function(n){e.isSourceFile(n)&&!e.isExternalOrCommonJsModule(n)||(Wr(n).flags|=134217728)})):s=256,Wr(n).flags|=s,167===i.kind&&e.hasSyntacticModifier(i,256)&&(e.isSuperProperty(n.parent)&&e.isAssignmentTarget(n.parent)?Wr(i).flags|=4096:Wr(i).flags|=2048),a&&SE(n.parent,i),203===i.parent.kind)return F<2?(Dr(n,e.Diagnostics.super_is_only_allowed_in_members_of_object_literal_expressions_when_option_target_is_ES2015_or_higher),Re):Ie;var l=i.parent;if(!e.getClassExtendsHeritageElement(l))return Dr(n,e.Diagnostics.super_can_only_be_referenced_in_a_derived_class),Re;var u=Dc(Ca(l)),d=u&&lc(u)[0];return d?169===i.kind&&OE(n,i)?(Dr(n,e.Diagnostics.super_cannot_be_referenced_in_constructor_arguments),Re):512===s?oc(u):Xc(d,u.thisType):Re;function p(n){return!!n&&(t?169===n.kind:!(!e.isClassLike(n.parent)&&203!==n.parent.kind)&&(e.isStatic(n)?167===n.kind||166===n.kind||170===n.kind||171===n.kind||165===n.kind||168===n.kind:167===n.kind||166===n.kind||170===n.kind||171===n.kind||165===n.kind||164===n.kind||169===n.kind))}}function PE(e){return 167!==e.kind&&170!==e.kind&&171!==e.kind||203!==e.parent.kind?211===e.kind&&291===e.parent.kind?e.parent.parent:void 0:e.parent}function ME(n){return 4&e.getObjectFlags(n)&&n.target===Fn?Md(n)[0]:void 0}function FE(n){return Db(n,function(n){return 2097152&n.flags?e.forEach(n.types,ME):ME(n)})}function GE(n){if(212!==n.kind){if(tg(n)){var t=MT(n);if(t){var r=t.thisParameter;if(r)return Ws(r)}}var i=e.isInJSFile(n);if(Q||i){var a=PE(n);if(a){var o=gT(a),s=a,c=o;while(c){var l=FE(c);if(l)return z_(l,Kh(bT(a)));if(291!==s.parent.kind)break;s=s.parent.parent,c=gT(s)}return Ch(o?lh(o):kA(a))}var u=e.walkUpParenthesizedExpressions(n.parent);if(219===u.kind&&63===u.operatorToken.kind){var d=u.left;if(e.isAccessExpression(d)){var p=d.expression;if(i&&e.isIdentifier(p)){var f=e.getSourceFileOfNode(u);if(f.commonJsModuleIndicator&&xv(p)===f.symbol)return}return Ch(kA(p))}}}}}function BE(n){var t=n.parent;if(tg(t)){var r=e.getImmediatelyInvokedFunctionExpression(t);if(r&&r.arguments){var i=tL(r),a=t.parameters.indexOf(n);if(n.dotDotDotToken)return Xx(i,a,i.length,Ie,void 0,0);var o=Wr(r),s=o.resolvedSignature;o.resolvedSignature=bt;var c=a<i.length?jy(eC(i[a])):n.initializer?void 0:Fe;return o.resolvedSignature=s,c}var l=MT(t);if(l){var u=t.parameters.indexOf(n)-(e.getThisParameter(t)?1:0);return n.dotDotDotToken&&e.lastOrUndefined(t.parameters)===n?mD(l,u):fD(l,u)}}}function UE(n){var t=e.getEffectiveTypeAnnotationNode(n);if(t)return u_(t);switch(n.kind){case 162:return BE(n);case 201:return VE(n);case 165:if(e.isStatic(n))return jE(n)}}function VE(n){var t=n.parent.parent,r=n.propertyName||n.name,i=UE(t)||201!==t.kind&&t.initializer&&OA(t);if(i&&!e.isBindingPattern(r)&&!e.isComputedNonLiteralName(r)){if(200===t.name.kind){var a=e.indexOfNode(n.parent.elements,n);if(a<0)return;return cT(i,a)}var o=Ff(r);if(Fc(o)){var s=Kc(o);return Bo(i,s)}}}function jE(n){var t=e.isExpression(n.parent)&&vT(n.parent);if(t)return aT(t,Ca(n).escapedName)}function KE(n,t){var r=n.parent;if(e.hasInitializer(r)&&n===r.initializer){var i=UE(r);if(i)return i;if(!(8&t)&&e.isBindingPattern(r.name))return bs(r.name,!0,!1)}}function HE(n){var t=e.getContainingFunction(n);if(t){var r=XE(t);if(r){var i=e.getFunctionFlags(t);if(1&i){var a=2&i?2:1,o=EN(r,a,void 0);if(!o)return;r=o.returnType}if(2&i){var s=Db(r,YC);return s&&ff([s,CD(s)])}return r}}}function WE(e,n){var t=vT(e,n);if(t){var r=YC(t);return r&&ff([r,CD(r)])}}function zE(n){var t=e.getContainingFunction(n);if(t){var r=e.getFunctionFlags(t),i=XE(t);if(i)return n.asteriskToken?i:BN(0,i,0!==(2&r))}}function JE(n){var t=!1;while(n.parent&&!e.isFunctionLike(n.parent)){if(e.isParameter(n.parent)&&(t||n.parent.initializer===n))return!0;e.isBindingElement(n.parent)&&n.parent.initializer===n&&(t=!0),n=n.parent}return!1}function qE(n,t){var r=!!(2&e.getFunctionFlags(t)),i=XE(t);if(i)return BN(n,i,r)||void 0}function XE(e){var n=od(e);if(n)return n;var t=PT(e);return t&&!sd(t)?ad(t):void 0}function YE(e,n){var t=tL(e),r=t.indexOf(n);return-1===r?void 0:QE(e,r)}function QE(n,t){var r=Wr(n).resolvedSignature===Tt?Tt:ML(n);if(e.isJsxOpeningLikeElement(n)&&0===t)return TT(r,n);var i=r.parameters.length-1;return V(r)&&t>=i?vm(Ws(r.parameters[i]),$m(t-i),256):pD(r,t)}function ZE(e,n){if(208===e.parent.kind)return YE(e.parent,n)}function $E(n,t){var r=n.parent,i=r.left,a=r.operatorToken,o=r.right;switch(a.kind){case 63:case 76:case 75:case 77:return n===o?nT(r):void 0;case 56:case 60:var s=vT(r,t);return n===o&&(s&&s.pattern||!s&&!e.isDefaultedExpandoInitializer(r))?QA(i):s;case 55:case 27:return n===o?vT(r,t):void 0;default:return}}function eT(n){if(n.symbol)return n.symbol;if(e.isIdentifier(n))return xv(n);if(e.isPropertyAccessExpression(n)){var t=QA(n.expression);return e.isPrivateIdentifier(n.name)?r(t,n.name):Lu(t,n.name.escapedText)}return;function r(e,n){var t=qS(n.escapedText,n);return t&&XS(e,t)}}function nT(n){var t,r,i=e.getAssignmentDeclarationKind(n);switch(i){case 0:case 4:var a=eT(n.left),o=a&&a.valueDeclaration;if(o&&(e.isPropertyDeclaration(o)||e.isPropertySignature(o))){var s=e.getEffectiveTypeAnnotationNode(o);return s&&z_(u_(s),Hr(a).mapper)||o.initializer&&QA(n.left)}return 0===i?QA(n.left):rT(n);case 5:if(tT(n,i))return rT(n);if(n.left.symbol){var c=n.left.symbol.valueDeclaration;if(!c)return;var l=e.cast(n.left,e.isAccessExpression);s=e.getEffectiveTypeAnnotationNode(c);if(s)return u_(s);if(e.isIdentifier(l.expression)){var u=l.expression,d=Qr(u,u.escapedText,111551,void 0,u.escapedText,!0);if(d){var p=d.valueDeclaration&&e.getEffectiveTypeAnnotationNode(d.valueDeclaration);if(p){var f=e.getElementOrPropertyAccessName(l);if(void 0!==f)return aT(u_(p),f)}return}}return e.isInJSFile(c)?void 0:QA(n.left)}return QA(n.left);case 1:case 6:case 3:var m=null===(t=n.left.symbol)||void 0===t?void 0:t.valueDeclaration;case 2:m||(m=null===(r=n.symbol)||void 0===r?void 0:r.valueDeclaration);var _=m&&e.getEffectiveTypeAnnotationNode(m);return _?u_(_):void 0;case 7:case 8:case 9:return e.Debug.fail("Does not apply");default:return e.Debug.assertNever(i)}}function tT(n,t){if(void 0===t&&(t=e.getAssignmentDeclarationKind(n)),4===t)return!0;if(!e.isInJSFile(n)||5!==t||!e.isIdentifier(n.left.expression))return!1;var r=n.left.expression.escapedText,i=Qr(n.left,r,111551,void 0,void 0,!0,!0);return e.isThisInitializedDeclaration(null===i||void 0===i?void 0:i.valueDeclaration)}function rT(n){if(!n.symbol)return QA(n.left);if(n.symbol.valueDeclaration){var t=e.getEffectiveTypeAnnotationNode(n.symbol.valueDeclaration);if(t){var r=u_(t);if(r)return r}}var i=e.cast(n.left,e.isAccessExpression);if(e.isObjectLiteralMethod(e.getThisContainer(i.expression,!1))){var a=CE(i.expression),o=e.getElementOrPropertyAccessName(i);return void 0!==o&&aT(a,o)||void 0}}function iT(n){return!!(262144&e.getCheckFlags(n)&&!n.type&&wo(n,0)>=0)}function aT(n,t){return Db(n,function(n){var r;if(Bl(n)){var i=Il(n),a=ru(i)||i,o=Zm(e.unescapeLeadingUnderscores(t));if(lg(o,a))return hm(n,o)}else if(3670016&n.flags){var s=Lu(n,t);if(s)return iT(s)?void 0:Ws(s);if(Jy(n)){var c=Yy(n);if(c&&HT(t)&&+t>=0)return c}return null===(r=Iu(ku(n),Zm(e.unescapeLeadingUnderscores(t))))||void 0===r?void 0:r.type}},!0)}function oT(n,t){if(e.Debug.assert(e.isObjectLiteralMethod(n)),!(16777216&n.flags))return sT(n,t)}function sT(n,t){var r=n.parent,i=e.isPropertyAssignment(n)&&UE(n);if(i)return i;var a=gT(r,t);if(a){if(Vc(n))return aT(a,Ca(n).escapedName);if(n.name){var o=Ff(n.name);return Db(a,function(e){var n;return null===(n=Iu(ku(e),o))||void 0===n?void 0:n.type},!0)}}}function cT(e,n){return e&&(aT(e,""+n)||Db(e,function(e){return mN(1,e,Me,void 0,!1)},!0))}function lT(e,n){var t=e.parent;return e===t.whenTrue||e===t.whenFalse?vT(t,n):void 0}function uT(n,t){var r=gT(n.openingElement.tagName),i=yS(fS(n));if(r&&!Vo(r)&&i&&""!==i){var a=e.getSemanticJsxChildren(n.children),o=a.indexOf(t),s=aT(r,i);return s&&(1===a.length?s:Db(s,function(e){return Iy(e)?vm(e,$m(o)):e},!0))}}function dT(n){var t=n.parent;return e.isJsxAttributeLike(t)?vT(n):e.isJsxElement(t)?uT(t,n):void 0}function pT(n){if(e.isJsxAttribute(n)){var t=gT(n.parent);if(!t||Vo(t))return;return aT(t,n.name.escapedText)}return vT(n.parent)}function fT(e){switch(e.kind){case 10:case 8:case 9:case 14:case 110:case 95:case 104:case 79:case 151:return!0;case 204:case 210:return fT(e.expression);case 286:return!e.expression||fT(e.expression)}return!1}function mT(n,t){return Bv(t,n)||Xg(t,e.concatenate(e.map(e.filter(n.properties,function(e){return!!e.symbol&&291===e.kind&&fT(e.initializer)&&Ov(t,e.symbol.escapedName)}),function(e){return[function(){return $A(e.initializer)},e.symbol.escapedName]}),e.map(e.filter(Hl(t),function(e){var r;return!!(16777216&e.flags)&&!!(null===(r=null===n||void 0===n?void 0:n.symbol)||void 0===r?void 0:r.members)&&!n.symbol.members.has(e.escapedName)&&Ov(t,e.escapedName)}),function(e){return[function(){return Me},e.escapedName]})),lg,t)}function _T(n,t){return Xg(t,e.concatenate(e.map(e.filter(n.properties,function(e){return!!e.symbol&&283===e.kind&&Ov(t,e.symbol.escapedName)&&(!e.initializer||fT(e.initializer))}),function(e){return[e.initializer?function(){return eC(e.initializer)}:function(){return Je},e.symbol.escapedName]}),e.map(e.filter(Hl(t),function(e){var r;return!!(16777216&e.flags)&&!!(null===(r=null===n||void 0===n?void 0:n.symbol)||void 0===r?void 0:r.members)&&!n.symbol.members.has(e.escapedName)&&Ov(t,e.escapedName)}),function(e){return[function(){return Me},e.escapedName]})),lg,t)}function gT(n,t){var r=e.isObjectLiteralMethod(n)?oT(n,t):vT(n,t),i=yT(r,n,t);if(i&&!(t&&2&t&&8650752&i.flags)){var a=Db(i,mu,!0);return 1048576&a.flags&&e.isObjectLiteralExpression(n)?mT(n,a):1048576&a.flags&&e.isJsxAttributes(n)?_T(n,a):a}}function yT(n,t,r){if(n&&iA(n,465829888)){var i=bT(t);if(i&&e.some(i.inferences,KA)){if(r&&1&r)return hT(n,i.nonFixingMapper);if(i.returnMapper)return hT(n,i.returnMapper)}}return n}function hT(n,t){return 465829888&n.flags?z_(n,t):1048576&n.flags?ff(e.map(n.types,function(e){return hT(e,t)}),0):2097152&n.flags?Df(e.map(n.types,function(e){return hT(e,t)})):n}function vT(n,t){if(16777216&n.flags);else{if(n.contextualType)return n.contextualType;var r=n.parent;switch(r.kind){case 252:case 162:case 165:case 164:case 201:return KE(n,t);case 212:case 245:return HE(n);case 222:return zE(r);case 216:return WE(r,t);case 206:if(100===r.expression.kind)return je;case 207:return YE(r,n);case 209:case 227:return e.isConstTypeReference(r.type)?s(r):u_(r.type);case 219:return $E(n,t);case 291:case 292:return sT(r,t);case 293:return vT(r.parent,t);case 202:var i=r,a=gT(i,t);return cT(a,e.indexOfNode(i.elements,n));case 220:return lT(n,t);case 231:return e.Debug.assert(221===r.parent.kind),ZE(r.parent,n);case 210:var o=e.isInJSFile(r)?e.getJSDocTypeTag(r):void 0;return o?u_(o.typeExpression.type):vT(r,t);case 228:return vT(r,t);case 286:return dT(r);case 283:case 285:return pT(r);case 278:case 277:return ET(r,t)}}function s(e){return vT(e)}}function bT(n){var t=e.findAncestor(n,function(e){return!!e.inferenceContext});return t&&t.inferenceContext}function ET(n,t){return e.isJsxOpeningElement(n)&&n.parent.contextualType&&4!==t?n.parent.contextualType:QE(n,0)}function TT(e,n){return 0!==Qx(n)?ST(e,n):AT(e,n)}function ST(e,n){var t=ED(e,Pe);t=DT(n,fS(n),t);var r=uS(I.IntrinsicAttributes,n);return r!==Re&&(t=_l(r,t)),t}function xT(e,n){if(e.compositeSignatures){for(var t=[],r=0,i=e.compositeSignatures;r<i.length;r++){var a=i[r],o=ad(a);if(Vo(o))return o;var s=Bo(o,n);if(!s)return;t.push(s)}return Df(t)}var c=ad(e);return Vo(c)?c:Bo(c,n)}function LT(e){if(iS(e.tagName)){var n=ES(e),t=RL(e,n);return vd(t)}var r=kA(e.tagName);if(128&r.flags){n=vS(r,e);if(!n)return Re;t=RL(e,n);return vd(t)}return r}function DT(n,t,r){var i=_S(t);if(i){var a=Dc(i),o=LT(n);if(524288&i.flags){var s=Hr(i).typeParameters;if(e.length(s)>=2){var c=Ju([o,r],s,2,e.isInJSFile(n));return Bd(i,c)}}if(e.length(a.typeParameters)>=2){c=Ju([o,r],a.typeParameters,2,e.isInJSFile(n));return Od(a,c)}}return r}function AT(n,t){var r=fS(t),i=gS(r),a=void 0===i?ED(n,Pe):""===i?ad(n):xT(n,i);if(!a)return i&&e.length(t.attributes.properties)&&Dr(t,e.Diagnostics.JSX_element_class_does_not_support_attributes_because_it_does_not_have_a_0_property,e.unescapeLeadingUnderscores(i)),Pe;if(a=DT(t,r,a),Vo(a))return a;var o=a,s=uS(I.IntrinsicClassAttributes,t);if(s!==Re){var c=$s(s.symbol),l=ad(n);o=_l(c?Od(s,Ju([l],c,zu(c),e.isInJSFile(t))):s,o)}var u=uS(I.IntrinsicAttributes,t);return u!==Re&&(o=_l(u,o)),o}function CT(n){return e.getStrictOptionValue(M,"noImplicitAny")?e.reduceLeft(n,function(e,n){return e!==n&&e?ll(e.typeParameters,n.typeParameters)?kT(e,n):void 0:e}):void 0}function IT(e,n,t){if(!e||!n)return e||n;var r=ff([Ws(e),z_(Ws(n),t)]);return vh(e,r)}function NT(e,n,t){for(var r=_D(e),i=_D(n),a=r>=i?e:n,o=a===e?n:e,s=a===e?r:i,c=yD(e)||yD(n),l=c&&!yD(a),u=new Array(s+(l?1:0)),d=0;d<s;d++){var p=fD(a,d);a===n&&(p=z_(p,t));var f=fD(o,d)||Pe;o===n&&(f=z_(f,t));var m=ff([p,f]),_=c&&!l&&d===s-1,g=d>=gD(a)&&d>=gD(o),y=d>=r?void 0:sD(e,d),h=d>=i?void 0:sD(n,d),v=y===h?y:y?h?void 0:y:h,b=Or(1|(g&&!_?16777216:0),v||"arg"+d);b.type=_?Mp(m):m,u[d]=b}if(l){var E=Or(1,"args");E.type=Mp(pD(o,s)),o===n&&(E.type=z_(E.type,t)),u[s]=E}return u}function kT(n,t){var r,i=n.typeParameters||t.typeParameters;n.typeParameters&&t.typeParameters&&(r=g_(t.typeParameters,n.typeParameters));var a=n.declaration,o=NT(n,t,r),s=IT(n.thisParameter,t.thisParameter,r),c=Math.max(n.minArgumentCount,t.minArgumentCount),l=$c(a,i,s,o,void 0,void 0,c,39&(n.flags|t.flags));return l.compositeKind=2097152,l.compositeSignatures=e.concatenate(2097152===n.compositeKind&&n.compositeSignatures||[n],[t]),r&&(l.mapper=2097152===n.compositeKind&&n.mapper&&n.compositeSignatures?x_(n.mapper,r):r),l}function RT(n,t){var r=Au(n,0),i=e.filter(r,function(e){return!OT(e,t)});return 1===i.length?i[0]:CT(i)}function OT(n,t){for(var r=0;r<t.parameters.length;r++){var i=t.parameters[r];if(i.initializer||i.questionToken||i.dotDotDotToken||Uu(i))break}return t.parameters.length&&e.parameterIsThisKeyword(t.parameters[0])&&r--,!yD(n)&&_D(n)<r}function wT(e){return 211===e.kind||212===e.kind}function PT(n){return wT(n)||e.isObjectLiteralMethod(n)?MT(n):void 0}function MT(n){e.Debug.assert(167!==n.kind||e.isObjectLiteralMethod(n));var t=Yu(n);if(t)return t;var r=gT(n,1);if(r){if(!(1048576&r.flags))return RT(r,n);for(var i,a=r.types,o=0,s=a;o<s.length;o++){var c=s[o],l=RT(c,n);if(l)if(i){if(!vy(i[0],l,!1,!0,!0,ag))return;i.push(l)}else i=[l]}return i?1===i.length?i[0]:nl(i[0],i):void 0}}function FT(e,n){F<2&&IO(e,M.downlevelIteration?1536:1024);var t=eC(e.expression,n);return fN(33,t,Me,e.expression)}function GT(e){return e.isSpread?vm(e.type,Ke):e.type}function BT(e){return 201===e.kind&&!!e.initializer||219===e.kind&&63===e.operatorToken.kind}function UT(n,t,r){for(var i=n.elements,a=i.length,o=[],s=[],c=gT(n),l=e.isAssignmentTarget(n),u=FA(n),d=!1,p=0;p<a;p++){var f=i[p];if(223===f.kind){F<2&&IO(f,M.downlevelIteration?1536:1024);var m=eC(f.expression,t,r);if(Iy(m))o.push(m),s.push(8);else if(l){var _=wu(m,Ke)||mN(65,m,Me,void 0,!1)||Pe;o.push(_),s.push(4)}else o.push(fN(33,m,Me,f.expression)),s.push(4)}else if(ne&&225===f.kind)d=!0,o.push(Be),s.push(2);else{var g=cT(c,o.length),y=GA(f,t,g,r);o.push(ns(y,!0,d)),s.push(d?2:1)}}return l?Wp(o,s):r||u||c&&bb(c,Oy)?VT(Wp(o,s,u)):VT(Mp(o.length?ff(e.sameMap(o,function(e,n){return 8&s[n]?Em(e,Ke)||Ie:e}),2):z?nn:Fe,u))}function VT(n){if(!(4&e.getObjectFlags(n)))return n;var t=n.literalType;return t||(t=n.literalType=wd(n),t.objectFlags|=294912),t}function jT(e){switch(e.kind){case 160:return KT(e);case 79:return HT(e.escapedText);case 8:case 10:return HT(e.text);default:return!1}}function KT(e){return aA(WT(e),296)}function HT(e){return(+e).toString()===e}function WT(n){var t=Wr(n.expression);if(!t.resolvedType){if(t.resolvedType=eC(n.expression),e.isPropertyDeclaration(n.parent)&&!e.hasStaticModifier(n.parent)&&e.isClassExpression(n.parent.parent)){var r=e.getEnclosingBlockScopeContainer(n.parent.parent),i=vE(r);i&&(Wr(i).flags|=65536,Wr(n).flags|=524288,Wr(n.parent.parent).flags|=524288)}(98304&t.resolvedType.flags||!aA(t.resolvedType,402665900)&&!lg(t.resolvedType,on))&&Dr(n,e.Diagnostics.A_computed_property_name_must_be_of_type_string_number_symbol_or_any)}return t.resolvedType}function zT(n){var t,r=null===(t=n.declarations)||void 0===t?void 0:t[0];return HT(n.escapedName)||r&&e.isNamedDeclaration(r)&&jT(r.name)}function JT(n){var t,r=null===(t=n.declarations)||void 0===t?void 0:t[0];return e.isKnownSymbol(n)||r&&e.isNamedDeclaration(r)&&e.isComputedPropertyName(r.name)&&aA(WT(r.name),4096)}function qT(e,n,t,r){for(var i=[],a=n;a<t.length;a++){var o=t[a];(r===je&&!JT(o)||r===Ke&&zT(o)||r===Ye&&JT(o))&&i.push(Ws(t[a]))}var s=i.length?ff(i,2):Me;return Td(r,s,FA(e))}function XT(n){e.Debug.assert(0!==(2097152&n.flags),"Should only get Alias here.");var t=Hr(n);if(!t.immediateTarget){var r=hi(n);if(!r)return e.Debug.fail();t.immediateTarget=Hi(r,!0)}return t.immediateTarget}function YT(n,t){var r=e.isAssignmentTarget(n);cw(n,r);for(var i=z?e.createSymbolTable():void 0,a=e.createSymbolTable(),o=[],s=pn,c=gT(n),l=c&&c.pattern&&(199===c.pattern.kind||203===c.pattern.kind),u=FA(n),d=u?8:0,p=e.isInJSFile(n)&&!e.isInJsonFile(n),f=e.getJSDocEnumTag(n),m=!c&&p&&!f,_=ee,g=!1,y=!1,h=!1,v=!1,b=0,E=n.properties;b<E.length;b++){var T=E[b];T.name&&e.isComputedPropertyName(T.name)&&WT(T.name)}for(var S=0,x=0,L=n.properties;x<L.length;x++){var D=L[x],A=Ca(D),C=D.name&&160===D.name.kind?WT(D.name):void 0;if(291===D.kind||292===D.kind||e.isObjectLiteralMethod(D)){var I=291===D.kind?BA(D,t):292===D.kind?GA(!r&&D.objectAssignmentInitializer?D.objectAssignmentInitializer:D.name,t):UA(D,t);if(p){var N=Zo(D);N?(fg(I,N,D),I=N):f&&f.typeExpression&&fg(I,u_(f.typeExpression),D)}_|=917504&e.getObjectFlags(I);var k=C&&Fc(C)?C:void 0,R=k?Or(4|A.flags,Kc(k),4096|d):Or(4|A.flags,A.escapedName,d);if(k&&(R.nameType=k),r){var O=291===D.kind&&BT(D.initializer)||292===D.kind&&D.objectAssignmentInitializer;O&&(R.flags|=16777216)}else if(l&&!(512&e.getObjectFlags(c))){var w=Lu(c,A.escapedName);w?R.flags|=16777216&w.flags:M.suppressExcessPropertyErrors||Ou(c,je)||Dr(D.name,e.Diagnostics.Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1,mo(A),go(c))}R.declarations=A.declarations,R.parent=A.parent,A.valueDeclaration&&(R.valueDeclaration=A.valueDeclaration),R.type=I,R.target=A,A=R,null===i||void 0===i||i.set(R.escapedName,R)}else{if(293===D.kind){F<2&&IO(D,2),o.length>0&&(s=Hm(s,U(),n.symbol,_,u),o=[],a=e.createSymbolTable(),y=!1,h=!1,v=!1);I=vu(eC(D.expression));if(QT(I)){var P=Km(I,u);if(i&&cS(P,i,D),S=o.length,s===Re)continue;s=Hm(s,P,n.symbol,_,u)}else Dr(D,e.Diagnostics.Spread_types_may_only_be_created_from_object_types),s=Re;continue}e.Debug.assert(170===D.kind||171===D.kind),rR(D)}!C||8576&C.flags?a.set(A.escapedName,A):lg(C,on)&&(lg(C,Ke)?h=!0:lg(C,Ye)?v=!0:y=!0,r&&(g=!0)),o.push(A)}if(l&&293!==n.parent.kind)for(var G=0,B=Hl(c);G<B.length;G++){R=B[G];a.get(R.escapedName)||Lu(s,R.escapedName)||(16777216&R.flags||Dr(R.valueDeclaration||R.bindingElement,e.Diagnostics.Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value),a.set(R.escapedName,R),o.push(R))}return s===Re?Re:s!==pn?(o.length>0&&(s=Hm(s,U(),n.symbol,_,u),o=[],a=e.createSymbolTable(),y=!1,h=!1),Db(s,function(e){return e===pn?U():e})):U();function U(){var t=[];y&&t.push(qT(n,S,o,je)),h&&t.push(qT(n,S,o,Ke)),v&&t.push(qT(n,S,o,Ye));var i=Xa(n.symbol,a,e.emptyArray,e.emptyArray,t);return i.objectFlags|=262272|_,m&&(i.objectFlags|=8192),g&&(i.objectFlags|=512),r&&(i.pattern=n),i}}function QT(n){if(465829888&n.flags){var t=ru(n);if(void 0!==t)return QT(t)}return!!(126615553&n.flags||117632&th(n)&&QT(rh(n))||3145728&n.flags&&e.every(n.types,QT))}function ZT(e){AS(e)}function $T(e,n){return rR(e),SS(e)||Ie}function eS(e){AS(e.openingElement),iS(e.closingElement.tagName)?dS(e.closingElement):eC(e.closingElement.tagName),sS(e)}function nS(e,n){return rR(e),SS(e)||Ie}function tS(n){AS(n.openingFragment);var t=e.getSourceFileOfNode(n);return!e.getJSXTransformEnabled(M)||!M.jsxFactory&&!t.pragmas.has("jsx")||M.jsxFragmentFactory||t.pragmas.has("jsxfrag")||Dr(n,M.jsxFactory?e.Diagnostics.The_jsxFragmentFactory_compiler_option_must_be_provided_to_use_JSX_fragments_with_the_jsxFactory_compiler_option:e.Diagnostics.An_jsxFrag_pragma_is_required_when_using_an_jsx_pragma_with_JSX_fragments),sS(n),SS(n)||Ie}function rS(n){return e.stringContains(n,"-")}function iS(n){return 79===n.kind&&e.isIntrinsicJsxName(n.escapedText)}function aS(e,n){return e.initializer?GA(e.initializer,n):Je}function oS(n,t){for(var r,i=n.attributes,a=z?e.createSymbolTable():void 0,o=e.createSymbolTable(),s=fn,c=!1,l=!1,u=2048,d=yS(fS(n)),p=0,f=i.properties;p<f.length;p++){var m=f[p],_=m.symbol;if(e.isJsxAttribute(m)){var g=aS(m,t);u|=917504&e.getObjectFlags(g);var y=Or(4|_.flags,_.escapedName);y.declarations=_.declarations,y.parent=_.parent,_.valueDeclaration&&(y.valueDeclaration=_.valueDeclaration),y.type=g,y.target=_,o.set(y.escapedName,y),null===a||void 0===a||a.set(y.escapedName,y),m.name.escapedText===d&&(l=!0)}else{e.Debug.assert(285===m.kind),o.size>0&&(s=Hm(s,x(),i.symbol,u,!1),o=e.createSymbolTable());g=vu(kA(m.expression,t));Vo(g)&&(c=!0),QT(g)?(s=Hm(s,g,i.symbol,u,!1),a&&cS(g,a,m)):r=r?Df([r,g]):g}}c||o.size>0&&(s=Hm(s,x(),i.symbol,u,!1));var h=276===n.parent.kind?n.parent:void 0;if(h&&h.openingElement===n&&h.children.length>0){var v=sS(h,t);if(!c&&d&&""!==d){l&&Dr(i,e.Diagnostics._0_are_specified_twice_The_attribute_named_0_will_be_overwritten,e.unescapeLeadingUnderscores(d));var b=gT(n.attributes),E=b&&aT(b,d),T=Or(4,d);T.type=1===v.length?v[0]:E&&bb(E,Oy)?Wp(v):Mp(ff(v)),T.valueDeclaration=e.factory.createPropertySignature(void 0,e.unescapeLeadingUnderscores(d),void 0,void 0),e.setParent(T.valueDeclaration,i),T.valueDeclaration.symbol=T;var S=e.createSymbolTable();S.set(d,T),s=Hm(s,Xa(i.symbol,S,e.emptyArray,e.emptyArray,e.emptyArray),i.symbol,u,!1)}}return c?Ie:r&&s!==fn?Df([r,s]):r||(s===fn?x():s);function x(){u|=ee;var n=Xa(i.symbol,o,e.emptyArray,e.emptyArray,e.emptyArray);return n.objectFlags|=262272|u,n}}function sS(e,n){for(var t=[],r=0,i=e.children;r<i.length;r++){var a=i[r];if(11===a.kind)a.containsOnlyTriviaWhiteSpaces||t.push(je);else{if(286===a.kind&&!a.expression)continue;t.push(GA(a,n))}}return t}function cS(n,t,r){for(var i=0,a=Hl(n);i<a.length;i++){var o=a[i];if(!(16777216&o.flags)){var s=t.get(o.escapedName);if(s){var c=Dr(s.valueDeclaration,e.Diagnostics._0_is_specified_more_than_once_so_this_usage_will_be_overwritten,e.unescapeLeadingUnderscores(s.escapedName));e.addRelatedInfo(c,e.createDiagnosticForNode(r,e.Diagnostics.This_spread_always_overwrites_this_property))}}}}function lS(e,n){return oS(e.parent,n)}function uS(e,n){var t=fS(n),r=t&&Sa(t),i=r&&Jr(r,e,788968);return i?Dc(i):Re}function dS(n){var t=Wr(n);if(!t.resolvedSymbol){var r=uS(I.IntrinsicElements,n);if(r!==Re){if(!e.isIdentifier(n.tagName))return e.Debug.fail();var i=Lu(r,n.tagName.escapedText);if(i)return t.jsxFlags|=1,t.resolvedSymbol=i;var a=wu(r,je);return a?(t.jsxFlags|=2,t.resolvedSymbol=r.symbol):(Dr(n,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.idText(n.tagName),"JSX."+I.IntrinsicElements),t.resolvedSymbol=Ae)}return Y&&Dr(n,e.Diagnostics.JSX_element_implicitly_has_type_any_because_no_interface_JSX_0_exists,e.unescapeLeadingUnderscores(I.IntrinsicElements)),t.resolvedSymbol=Ae}return t.resolvedSymbol}function pS(n){var t=n&&e.getSourceFileOfNode(n),r=t&&Wr(t);if(!r||!1!==r.jsxImplicitImportContainer){if(r&&r.jsxImplicitImportContainer)return r.jsxImplicitImportContainer;var i=e.getJSXRuntimeImport(e.getJSXImplicitImportBase(M,t),M);if(i){var a=e.getEmitModuleResolutionKind(M)===e.ModuleResolutionKind.Classic,o=a?e.Diagnostics.Cannot_find_module_0_Did_you_mean_to_set_the_moduleResolution_option_to_node_or_to_add_aliases_to_the_paths_option:e.Diagnostics.Cannot_find_module_0_or_its_corresponding_type_declarations,s=ua(n,i,o,n),c=s&&s!==Ae?Aa(zi(s)):void 0;return r&&(r.jsxImplicitImportContainer=c||!1),c}}}function fS(e){var n=e&&Wr(e);if(n&&n.jsxNamespace)return n.jsxNamespace;if(!n||!1!==n.jsxNamespace){var t=pS(e);if(!t||t===Ae){var r=Er(e);t=Qr(e,r,1920,void 0,r,!1)}if(t){var i=zi(Jr(Sa(zi(t)),I.JSX,1920));if(i&&i!==Ae)return n&&(n.jsxNamespace=i),i}n&&(n.jsxNamespace=!1)}var a=zi(op(I.JSX,1920,void 0));return a!==Ae?a:void 0}function mS(n,t){var r=t&&Jr(t.exports,n,788968),i=r&&Dc(r),a=i&&Hl(i);if(a){if(0===a.length)return"";if(1===a.length)return a[0].escapedName;a.length>1&&r.declarations&&Dr(r.declarations[0],e.Diagnostics.The_global_type_JSX_0_may_not_have_more_than_one_property,e.unescapeLeadingUnderscores(n))}}function _S(e){return e&&Jr(e.exports,I.LibraryManagedAttributes,788968)}function gS(e){return mS(I.ElementAttributesPropertyNameContainer,e)}function yS(e){return mS(I.ElementChildrenAttributeNameContainer,e)}function hS(n,t){if(4&n.flags)return[bt];if(128&n.flags){var r=vS(n,t);if(r){var i=RL(t,r);return[i]}return Dr(t,e.Diagnostics.Property_0_does_not_exist_on_type_1,n.value,"JSX."+I.IntrinsicElements),e.emptyArray}var a=mu(n),o=Au(a,1);return 0===o.length&&(o=Au(a,0)),0===o.length&&1048576&a.flags&&(o=cl(e.map(a.types,function(e){return hS(e,t)}))),o}function vS(n,t){var r=uS(I.IntrinsicElements,t);if(r!==Re){var i=n.value,a=Lu(r,e.escapeLeadingUnderscores(i));if(a)return Ws(a);var o=wu(r,je);return o||void 0}return Ie}function bS(n,t,r){if(1===n){var i=xS(r);i&&zg(t,i,gr,r.tagName,e.Diagnostics.Its_return_type_0_is_not_a_valid_JSX_element,s)}else if(0===n){var a=TS(r);a&&zg(t,a,gr,r.tagName,e.Diagnostics.Its_instance_type_0_is_not_a_valid_JSX_element,s)}else{i=xS(r),a=TS(r);if(!i||!a)return;var o=ff([i,a]);zg(t,o,gr,r.tagName,e.Diagnostics.Its_element_type_0_is_not_a_valid_JSX_element,s)}function s(){var n=e.getTextOfNode(r.tagName);return e.chainDiagnosticMessages(void 0,e.Diagnostics._0_cannot_be_used_as_a_JSX_component,n)}}function ES(n){e.Debug.assert(iS(n.tagName));var t=Wr(n);if(!t.resolvedJsxElementAttributesType){var r=dS(n);return 1&t.jsxFlags?t.resolvedJsxElementAttributesType=Ws(r)||Re:2&t.jsxFlags?t.resolvedJsxElementAttributesType=wu(uS(I.IntrinsicElements,n),je)||Re:t.resolvedJsxElementAttributesType=Re}return t.resolvedJsxElementAttributesType}function TS(e){var n=uS(I.ElementClass,e);if(n!==Re)return n}function SS(e){return uS(I.Element,e)}function xS(e){var n=SS(e);if(n)return ff([n,Ue])}function LS(n){var t=uS(I.IntrinsicElements,n);return t?Hl(t):e.emptyArray}function DS(n){0===(M.jsx||0)&&Dr(n,e.Diagnostics.Cannot_use_JSX_unless_the_jsx_flag_is_provided),void 0===SS(n)&&Y&&Dr(n,e.Diagnostics.JSX_element_implicitly_has_type_any_because_the_global_type_JSX_Element_does_not_exist)}function AS(n){var t=e.isJsxOpeningLikeElement(n);if(t&&lw(n),DS(n),!pS(n)){var r=ur&&2===M.jsx?e.Diagnostics.Cannot_find_name_0:void 0,i=Er(n),a=t?n.tagName:n,o=void 0;e.isJsxOpeningFragment(n)&&"null"===i||(o=Qr(a,i,111551,r,i,!0)),o&&(o.isReferenced=67108863,2097152&o.flags&&!Qi(o)&&$i(o))}if(t){var s=n,c=ML(s);KL(c,n),bS(Qx(s),ad(c),s)}}function CS(e,n,t){if(524288&e.flags){if(jl(e,n)||Fu(e,n)||Bc(n)&&Ou(e,je)||t&&rS(n))return!0}else if(3145728&e.flags&&IS(e))for(var r=0,i=e.types;r<i.length;r++){var a=i[r];if(CS(a,n,t))return!0}return!1}function IS(n){return!!(524288&n.flags&&!(512&e.getObjectFlags(n))||67108864&n.flags||1048576&n.flags&&e.some(n.types,IS)||2097152&n.flags&&e.every(n.types,IS))}function NS(n,t){if(dw(n),n.expression){var r=eC(n.expression,t);return n.dotDotDotToken&&r!==Ie&&!Ly(r)&&Dr(n,e.Diagnostics.JSX_spread_child_must_be_an_array_type),r}return Re}function kS(n){return n.valueDeclaration?e.getCombinedNodeFlags(n.valueDeclaration):0}function RS(n){if(8192&n.flags||4&e.getCheckFlags(n))return!0;if(e.isInJSFile(n.valueDeclaration)){var t=n.valueDeclaration.parent;return t&&e.isBinaryExpression(t)&&3===e.getAssignmentDeclarationKind(t)}}function OS(n,t,r,i,a,o){void 0===o&&(o=!0);var s=e.getDeclarationModifierFlagsFromSymbol(a,r),c=159===n.kind?n.right:198===n.kind?n:201===n.kind&&n.propertyName?n.propertyName:n.name;if(t){if(F<2&&PS(a))return o&&Dr(c,e.Diagnostics.Only_public_and_protected_methods_of_the_base_class_are_accessible_via_the_super_keyword),!1;if(128&s)return o&&Dr(c,e.Diagnostics.Abstract_method_0_in_class_1_cannot_be_accessed_via_super_expression,mo(a),go(ly(a))),!1}if(128&s&&PS(a)&&(e.isThisProperty(n)||e.isThisInitializedObjectBindingExpression(n)||e.isObjectBindingPattern(n.parent)&&e.isThisInitializedDeclaration(n.parent.parent))){var l=e.getClassLikeDeclarationOfSymbol(Ia(a));if(l&&bR(n))return o&&Dr(c,e.Diagnostics.Abstract_property_0_in_class_1_cannot_be_accessed_in_the_constructor,mo(a),e.getTextOfIdentifierOrLiteral(l.name)),!1}if(!(24&s))return!0;if(8&s){l=e.getClassLikeDeclarationOfSymbol(Ia(a));return!!ER(n,l)||(o&&Dr(c,e.Diagnostics.Property_0_is_private_and_only_accessible_within_class_1,mo(a),go(ly(a))),!1)}if(t)return!0;var u=vR(n,function(e){var n=Dc(Ca(e));return fy(n,a,r)?n:void 0});if(!u){var d=void 0;if(32&s||!(d=wS(n))||!d.type)return o&&Dr(c,e.Diagnostics.Property_0_is_protected_and_only_accessible_within_class_1_and_its_subclasses,mo(a),go(ly(a)||i)),!1;var p=u_(d.type);u=(262144&p.flags?ql(p):p).target}return!!(32&s)||(262144&i.flags&&(i=i.isThisType?ql(i):ru(i)),!(!i||!Xs(i,u))||(o&&Dr(c,e.Diagnostics.Property_0_is_protected_and_only_accessible_through_an_instance_of_class_1_This_is_an_instance_of_class_2,mo(a),go(u),go(i)),!1))}function wS(n){var t=e.getThisContainer(n,!1);return t&&e.isFunctionLike(t)?e.getThisParameter(t):void 0}function PS(e){return!!cy(e,function(e){return!(8192&e.flags)})}function MS(e){return jS(eC(e),e)}function FS(e){return!!(98304&(z?th(e):e.flags))}function GS(e){return FS(e)?lh(e):e}function BS(n,t){Dr(n,32768&t?65536&t?e.Diagnostics.Object_is_possibly_null_or_undefined:e.Diagnostics.Object_is_possibly_undefined:e.Diagnostics.Object_is_possibly_null)}function US(n,t){Dr(n,32768&t?65536&t?e.Diagnostics.Cannot_invoke_an_object_which_is_possibly_null_or_undefined:e.Diagnostics.Cannot_invoke_an_object_which_is_possibly_undefined:e.Diagnostics.Cannot_invoke_an_object_which_is_possibly_null)}function VS(n,t,r){if(z&&2&n.flags)return Dr(t,e.Diagnostics.Object_is_of_type_unknown),Re;var i=98304&(z?th(n):n.flags);if(i){r(t,i);var a=lh(n);return 229376&a.flags?Re:a}return n}function jS(e,n){return VS(e,n,BS)}function KS(n,t){var r=jS(n,t);return r!==Re&&16384&r.flags&&Dr(t,e.Diagnostics.Object_is_possibly_undefined),r}function HS(e,n){return 32&e.flags?WS(e,n):ZS(e,e.expression,MS(e.expression),e.name,n)}function WS(e,n){var t=eC(e.expression),r=fh(t,e.expression);return ph(ZS(e,e.expression,jS(r,e.expression),e.name,n),e,r!==t)}function zS(n,t){var r=e.isPartOfTypeQuery(n)&&e.isThisIdentifier(n.left)?jS(CE(n.left),n.left):MS(n.left);return ZS(n,n.left,r,n.right,t)}function JS(n){while(210===n.parent.kind)n=n.parent;return e.isCallOrNewExpression(n.parent)&&n.parent.expression===n}function qS(n,t){for(var r=e.getContainingClass(t);r;r=e.getContainingClass(r)){var i=r.symbol,a=e.getSymbolNameForPrivateIdentifier(i,n),o=i.members&&i.members.get(a)||i.exports&&i.exports.get(a);if(o)return o}}function XS(e,n){return Lu(e,n.escapedName)}function YS(n,t,r){var i,a=Hl(n);a&&e.forEach(a,function(n){var r=n.valueDeclaration;if(r&&e.isNamedDeclaration(r)&&e.isPrivateIdentifier(r.name)&&r.name.escapedText===t.escapedText)return i=n,!0});var o=ti(t);if(i){var s=e.Debug.checkDefined(i.valueDeclaration),c=e.Debug.checkDefined(e.getContainingClass(s));if(null===r||void 0===r?void 0:r.valueDeclaration){var u=r.valueDeclaration,d=e.getContainingClass(u);if(e.Debug.assert(!!d),e.findAncestor(d,function(e){return c===e})){var p=Dr(t,e.Diagnostics.The_property_0_cannot_be_accessed_on_type_1_within_this_class_because_it_is_shadowed_by_another_private_identifier_with_the_same_spelling,o,go(n));return e.addRelatedInfo(p,e.createDiagnosticForNode(u,e.Diagnostics.The_shadowing_declaration_of_0_is_defined_here,o),e.createDiagnosticForNode(s,e.Diagnostics.The_declaration_of_0_that_you_probably_intended_to_use_is_defined_here,o)),!0}}return Dr(t,e.Diagnostics.Property_0_is_not_accessible_outside_class_1_because_it_has_a_private_identifier,o,ti(c.name||l)),!0}return!1}function QS(n,t){return(rs(t)||e.isThisProperty(n)&&is(t))&&e.getThisContainer(n,!0)===as(t)}function ZS(n,t,r,i,a){var o,s,c=Wr(t).resolvedSymbol,l=e.getAssignmentTargetKind(n),u=mu(0!==l||JS(n)?Ch(r):r),d=Vo(u)||u===$e;if(e.isPrivateIdentifier(i)){F<99&&(0!==l&&IO(n,1048576),1!==l&&IO(n,524288));var p=qS(i.escapedText,i);if(l&&p&&p.valueDeclaration&&e.isMethodDeclaration(p.valueDeclaration)&&Mw(i,e.Diagnostics.Cannot_assign_to_private_method_0_Private_methods_are_not_writable,e.idText(i)),(null===p||void 0===p?void 0:p.valueDeclaration)&&99===M.target&&!H){var f=e.getContainingClass(p.valueDeclaration),m=e.findAncestor(n,function(n){return n===f?"quit":!(!e.isPropertyDeclaration(n.parent)||!e.hasStaticModifier(n.parent)||n.parent.initializer!==n||n.parent.parent!==f)});if(m){var _=Ca(m.parent);e.Debug.assert(_,"Initializer without declaration symbol");var g=Dr(n,e.Diagnostics.Property_0_may_not_be_used_in_a_static_property_s_initializer_in_the_same_class_when_target_is_esnext_and_useDefineForClassFields_is_false,e.symbolName(p));e.addRelatedInfo(g,e.createDiagnosticForNode(m.parent,e.Diagnostics.Initializer_for_property_0,e.symbolName(_)))}}if(d){if(p)return u;if(!e.getContainingClass(i))return Mw(i,e.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies),Ie}if(o=p?XS(r,p):void 0,!o&&YS(r,i,p))return Re;var y=o&&65536&o.flags&&!(32768&o.flags);y&&1!==l&&Dr(n,e.Diagnostics.Private_accessor_was_defined_without_a_getter)}else{if(d)return e.isIdentifier(t)&&c&&_E(c,n),u;o=Lu(u,i.escapedText)}if(e.isIdentifier(t)&&c&&(M.isolatedModules||!o||!nO(o)||e.shouldPreserveConstEnums(M)&&mE(n))&&_E(c,n),o){o.declarations&&134217728&kS(o)&&nm(n,o)&&kr(i,o.declarations,i.escapedText),nx(o,n,i),bx(o,n,Ex(t,c)),Wr(n).resolvedSymbol=o;var h=e.isWriteAccess(n);if(OS(n,106===t.kind,h,u,o),qD(n,o,l))return Dr(i,e.Diagnostics.Cannot_assign_to_0_because_it_is_a_read_only_property,e.idText(i)),Re;s=QS(n,o)?Ne:h?Hs(o):Ws(o)}else{var v=e.isPrivateIdentifier(i)||0!==l&&sm(r)&&!um(r)?void 0:Fu(u,i.escapedText);if(!v||!v.type){var b=$S(n,r.symbol,!0);return!b&&$f(r)?Ie:r.symbol===se?(se.exports.has(i.escapedText)&&418&se.exports.get(i.escapedText).flags?Dr(i,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.unescapeLeadingUnderscores(i.escapedText),go(r)):Y&&Dr(i,e.Diagnostics.Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature,go(r)),Ie):(i.escapedText&&!ai(n)&&ax(i,um(r)?u:r,b),Re)}v.isReadonly&&(e.isAssignmentTarget(n)||e.isDeleteTarget(n))&&Dr(n,e.Diagnostics.Index_signature_in_type_0_only_permits_reading,go(u)),s=M.noUncheckedIndexedAccess&&!e.isAssignmentTarget(n)?ff([v.type,Me]):v.type,M.noPropertyAccessFromIndexSignature&&e.isPropertyAccessExpression(n)&&Dr(i,e.Diagnostics.Property_0_comes_from_an_index_signature_so_it_must_be_accessed_with_0,e.unescapeLeadingUnderscores(i.escapedText))}return ex(n,o,s,i,a)}function $S(n,t,r){var i=e.getSourceFileOfNode(n);if(i&&void 0===M.checkJs&&void 0===i.checkJsDirective&&(1===i.scriptKind||2===i.scriptKind)){var a=e.forEach(null===t||void 0===t?void 0:t.declarations,e.getSourceFileOfNode);return!(i!==a&&a&&zr(a))&&!(r&&t&&32&t.flags)&&!(n&&r&&e.isPropertyAccessExpression(n)&&108===n.expression.kind)}return!1}function ex(n,t,r,i,a){var o=e.getAssignmentTargetKind(n);if(1===o)return mh(r,!!(t&&16777216&t.flags));if(t&&!(98311&t.flags)&&!(8192&t.flags&&1048576&r.flags)&&!Qk(t.declarations))return r;if(r===Ne)return ls(n,t);r=fE(r,n,a);var s=!1;if(z&&X&&e.isAccessExpression(n)&&108===n.expression.kind){var c=t&&t.valueDeclaration;if(c&&hk(c)&&!e.isStatic(c)){var l=rE(n);169!==l.kind||l.parent!==c.parent||8388608&c.flags||(s=!0)}}else z&&t&&t.valueDeclaration&&e.isPropertyAccessExpression(t.valueDeclaration)&&e.getAssignmentDeclarationPropertyAccessKind(t.valueDeclaration)&&rE(n)===rE(t.valueDeclaration)&&(s=!0);var u=nE(n,r,s?sh(r):r);return s&&!(32768&th(r))&&32768&th(u)?(Dr(i,e.Diagnostics.Property_0_is_used_before_being_assigned,mo(t)),r):o?Vy(u):u}function nx(n,t,r){var i=n.valueDeclaration;if(i&&!e.getSourceFileOfNode(t).isDeclarationFile){var a,o=e.idText(r);!tx(t)||Ku(i)||e.isAccessExpression(t)&&e.isAccessExpression(t.expression)||Xr(i,r)||!M.useDefineForClassFields&&rx(n)?255!==i.kind||176===t.parent.kind||8388608&i.flags||Xr(i,r)||(a=Dr(r,e.Diagnostics.Class_0_used_before_its_declaration,o)):a=Dr(r,e.Diagnostics.Property_0_is_used_before_its_initialization,o),a&&e.addRelatedInfo(a,e.createDiagnosticForNode(i,e.Diagnostics._0_is_declared_here,o))}}function tx(n){return!!e.findAncestor(n,function(n){switch(n.kind){case 165:return!0;case 291:case 167:case 170:case 171:case 293:case 160:case 231:case 286:case 283:case 284:case 285:case 278:case 226:case 289:return!1;case 212:case 236:return!(!e.isBlock(n.parent)||!e.isClassStaticBlockDeclaration(n.parent.parent))||"quit";default:return!e.isExpressionNode(n)&&"quit"}})}function rx(e){if(!(32&e.parent.flags))return!1;var n=Ws(e.parent);while(1){if(n=n.symbol&&ix(n),!n)return!1;var t=Lu(n,e.escapedName);if(t&&t.valueDeclaration)return!0}}function ix(e){var n=lc(e);if(0!==n.length)return Df(n)}function ax(n,t,r){var i,a;if(!e.isPrivateIdentifier(n)&&1048576&t.flags&&!(131068&t.flags))for(var o=0,s=t.types;o<s.length;o++){var c=s[o];if(!Lu(c,n.escapedText)&&!Fu(c,n.escapedText)){i=e.chainDiagnosticMessages(i,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.declarationNameToString(n),go(c));break}}if(sx(n.escapedText,t)){var l=e.declarationNameToString(n),u=go(t);i=e.chainDiagnosticMessages(i,e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_to_access_the_static_member_2_instead,l,u,u+"."+l)}else{var d=JC(t);if(d&&Lu(d,n.escapedText))i=e.chainDiagnosticMessages(i,e.Diagnostics.Property_0_does_not_exist_on_type_1,e.declarationNameToString(n),go(t)),a=e.createDiagnosticForNode(n,e.Diagnostics.Did_you_forget_to_use_await);else{var p=e.declarationNameToString(n),f=go(t),m=lx(p,t);if(void 0!==m)i=e.chainDiagnosticMessages(i,e.Diagnostics.Property_0_does_not_exist_on_type_1_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_2_or_later,p,f,m);else{var _=dx(n,t);if(void 0!==_){var g=e.symbolName(_),y=r?e.Diagnostics.Property_0_may_not_exist_on_type_1_Did_you_mean_2:e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2;i=e.chainDiagnosticMessages(i,y,p,f,g),a=_.valueDeclaration&&e.createDiagnosticForNode(_.valueDeclaration,e.Diagnostics._0_is_declared_here,g)}else{var h=ox(t)?e.Diagnostics.Property_0_does_not_exist_on_type_1_Try_changing_the_lib_compiler_option_to_include_dom:e.Diagnostics.Property_0_does_not_exist_on_type_1;i=e.chainDiagnosticMessages(xu(i,t),h,p,f)}}}}var v=e.createDiagnosticForNodeFromMessageChain(n,i);a&&e.addRelatedInfo(v,a),Ar(!r,v)}function ox(n){return M.lib&&!M.lib.includes("dom")&&Tb(n,function(n){return n.symbol&&/^(EventTarget|Node|((HTML[a-zA-Z]*)?Element))$/.test(e.unescapeLeadingUnderscores(n.symbol.escapedName))})&&Gg(n)}function sx(n,t){var r=t.symbol&&Lu(Ws(t.symbol),n);return void 0!==r&&!!r.valueDeclaration&&e.isStatic(r.valueDeclaration)}function cx(n){for(var t=ti(n),r=e.getScriptTargetFeatures(),i=e.getOwnKeys(r),a=0,o=i;a<o.length;a++){var s=o[a],c=e.getOwnKeys(r[s]);if(void 0!==c&&e.contains(c,t))return s}}function lx(n,t){var r=mu(t).symbol;if(r)for(var i=e.getScriptTargetFeatures(),a=e.getOwnKeys(i),o=0,s=a;o<s.length;o++){var c=s[o],l=i[c],u=l[e.symbolName(r)];if(void 0!==u&&e.contains(u,n))return c}}function ux(e,n){return vx(e,Hl(n),106500)}function dx(n,t){var r=Hl(t);if("string"!==typeof n){var i=n.parent;e.isPropertyAccessExpression(i)&&(r=e.filter(r,function(e){return Sx(i,t,e)})),n=e.idText(n)}return vx(n,r,111551)}function px(n,t){var r=e.isString(n)?n:e.idText(n),i=Hl(t),a="for"===r?e.find(i,function(n){return"htmlFor"===e.symbolName(n)}):"class"===r?e.find(i,function(n){return"className"===e.symbolName(n)}):void 0;return null!==a&&void 0!==a?a:vx(r,i,111551)}function fx(n,t){var r=dx(n,t);return r&&e.symbolName(r)}function mx(n,t,r){e.Debug.assert(void 0!==t,"outername should always be defined");var i=Zr(n,t,r,void 0,t,!1,!1,function(n,r,i){e.Debug.assertEqual(t,r,"name should equal outerName");var a=Jr(n,r,i);return a||vx(e.unescapeLeadingUnderscores(r),e.arrayFrom(n.values()),i)});return i}function _x(n,t,r){var i=mx(n,t,r);return i&&e.symbolName(i)}function gx(n,t){return t.exports&&vx(e.idText(n),ya(t),2623475)}function yx(n,t){var r=gx(n,t);return r&&e.symbolName(r)}function hx(n,t,r){function i(e){var t=jl(n,e);if(t){var i=Vx(Ws(t));return!!i&&gD(i)>=1&&lg(r,pD(i,0))}return!1}var a=e.isAssignmentTarget(t)?"set":"get";if(i(a)){var o=e.tryGetPropertyAccessOrIdentifierToString(t.expression);return void 0===o?o=a:o+="."+a,o}}function vx(n,t,r){return e.getSpellingSuggestion(n,t,i);function i(n){var t=e.symbolName(n);if(!e.startsWith(t,'"')){if(n.flags&r)return t;if(2097152&n.flags){var i=qi(n);if(i&&i.flags&r)return t}}}}function bx(n,t,r){var i=n&&106500&n.flags&&n.valueDeclaration;if(i){var a=e.hasEffectiveModifier(i,8),o=n.valueDeclaration&&e.isNamedDeclaration(n.valueDeclaration)&&e.isPrivateIdentifier(n.valueDeclaration.name);if((a||o)&&(!t||!e.isWriteOnlyAccess(t)||65536&n.flags)){if(r){var s=e.findAncestor(t,e.isFunctionLikeDeclaration);if(s&&s.symbol===n)return}(1&e.getCheckFlags(n)?Hr(n).target:n).isReferenced=67108863}}}function Ex(n,t){return 108===n.kind||!!t&&e.isEntityNameExpression(n)&&t===xv(e.getFirstIdentifier(n))}function Tx(e,n){switch(e.kind){case 204:return xx(e,106===e.expression.kind,n,Ch(eC(e.expression)));case 159:return xx(e,!1,n,Ch(eC(e.left)));case 198:return xx(e,!1,n,u_(e))}}function Sx(e,n,t){return xx(e,204===e.kind&&106===e.expression.kind,t.escapedName,n)}function xx(n,t,r,i){if(i===Re||Vo(i))return!0;var a=Lu(i,r);if(a){if(a.valueDeclaration&&e.isPrivateIdentifierClassElementDeclaration(a.valueDeclaration)){var o=e.getContainingClass(a.valueDeclaration);return!e.isOptionalChain(n)&&!!e.findAncestor(n,function(e){return e===o})}return OS(n,t,!1,i,a,!1)}return e.isInJSFile(n)&&0!==(1048576&i.flags)&&i.types.some(function(e){return xx(n,t,r,e)})}function Lx(n){var t=n.initializer;if(253===t.kind){var r=t.declarations[0];if(r&&!e.isBindingPattern(r.name))return Ca(r)}else if(79===t.kind)return xv(t)}function Dx(e){return 1===Ru(e).length&&!!Ou(e,Ke)}function Ax(n){var t=e.skipParentheses(n);if(79===t.kind){var r=xv(t);if(3&r.flags){var i=n,a=n.parent;while(a){if(241===a.kind&&i===a.statement&&Lx(a)===r&&Dx(QA(a.expression)))return!0;i=a,a=a.parent}}}return!1}function Cx(e,n){return 32&e.flags?Ix(e,n):Nx(e,MS(e.expression),n)}function Ix(e,n){var t=eC(e.expression),r=fh(t,e.expression);return ph(Nx(e,jS(r,e.expression),n),e,r!==t)}function Nx(n,t,r){var i=0!==e.getAssignmentTargetKind(n)||JS(n)?Ch(t):t,a=n.argumentExpression,o=eC(a);if(i===Re||i===$e)return i;if(sA(i)&&!e.isStringLiteralLike(a))return Dr(a,e.Diagnostics.A_const_enum_member_can_only_be_accessed_using_a_string_literal),Re;var s=Ax(a)?Ke:o,c=e.isAssignmentTarget(n)?4|(sm(i)&&!um(i)?2:0):32,l=Em(i,s,c,n)||Re;return RC(ex(n,Wr(n).resolvedSymbol,l,a,r),n)}function kx(n){return e.isCallOrNewExpression(n)||e.isTaggedTemplateExpression(n)||e.isJsxOpeningLikeElement(n)}function Rx(n){return kx(n)&&e.forEach(n.typeArguments,Zk),208===n.kind?eC(n.template):e.isJsxOpeningLikeElement(n)?eC(n.attributes):163!==n.kind&&e.forEach(n.arguments,function(e){eC(e)}),bt}function Ox(e){return Rx(e),Et}function wx(n,t,r){var i,a,o,s,c=0,l=-1;e.Debug.assert(!t.length);for(var u=0,d=n;u<d.length;u++){var p=d[u],f=p.declaration&&Ca(p.declaration),m=p.declaration&&p.declaration.parent;a&&f!==a?(o=c=t.length,i=m):i&&m===i?o+=1:(i=m,o=c),a=f,j(p)?(l++,s=l,c++):s=o,t.splice(s,0,r?tl(p,r):p)}}function Px(e){return!!e&&(223===e.kind||230===e.kind&&e.isSpread)}function Mx(n){return e.findIndex(n,Px)}function Fx(e){return!!(16384&e.flags)}function Gx(e){return!!(49155&e.flags)}function Bx(n,t,r,i){var a;void 0===i&&(i=!1);var o=!1,s=_D(r),c=gD(r);if(208===n.kind)if(a=t.length,221===n.template.kind){var l=e.last(n.template.templateSpans);o=e.nodeIsMissing(l.literal)||!!l.literal.isUnterminated}else{var u=n.template;e.Debug.assert(14===u.kind),o=!!u.isUnterminated}else if(163===n.kind)a=iL(n,r);else if(e.isJsxOpeningLikeElement(n)){if(o=n.attributes.end===n.end,o)return!0;a=0===c?t.length:1,s=0===t.length?s:1,c=Math.min(c,1)}else{if(!n.arguments)return e.Debug.assert(207===n.kind),0===gD(r);a=i?t.length+1:t.length,o=n.arguments.end===n.end;var d=Mx(t);if(d>=0)return d>=gD(r)&&(yD(r)||d<_D(r))}if(!yD(r)&&a>s)return!1;if(o||a>=c)return!0;for(var p=a;p<c;p++){var f=pD(r,p);if(131072&Sb(f,e.isInJSFile(n)&&!z?Gx:Fx).flags)return!1}return!0}function Ux(n,t){var r=e.length(n.typeParameters),i=zu(n.typeParameters);return!e.some(t)||t.length>=i&&t.length<=r}function Vx(e){return Kx(e,0,!1)}function jx(e){return Kx(e,0,!1)||Kx(e,1,!1)}function Kx(e,n,t){if(524288&e.flags){var r=Ul(e);if(t||0===r.properties.length&&0===r.indexInfos.length){if(0===n&&1===r.callSignatures.length&&0===r.constructSignatures.length)return r.callSignatures[0];if(1===n&&1===r.constructSignatures.length&&0===r.callSignatures.length)return r.constructSignatures[0]}}}function Hx(n,t,r,i){var a=Ph(n.typeParameters,n,0,i),o=hD(t),s=r&&(o&&262144&o.flags?r.nonFixingMapper:r.mapper),c=s?k_(t,s):t;return Oh(c,n,function(e,n){dv(a.inferences,e,n)}),r||wh(t,n,function(e,n){dv(a.inferences,e,n,128)}),ud(n,Tv(a),e.isInJSFile(t.declaration))}function Wx(e,n,t,r){var i=TT(n,e),a=NA(e.attributes,i,r,t);return dv(r.inferences,a,i),Tv(r)}function zx(n){if(!n)return Qe;var t=eC(n);return e.isOptionalChainRoot(n.parent)?lh(t):e.isOptionalChain(n.parent)?dh(t):t}function Jx(n,t,r,i,a){if(e.isJsxOpeningLikeElement(n))return Wx(n,t,i,a);if(163!==n.kind){var o=vT(n,e.every(t.typeParameters,function(e){return!!lu(e)})?8:0);if(o){var s=bT(n),c=Kh(Mh(s,1)),l=z_(o,c),u=Vx(l),d=u&&u.typeParameters?vd(dd(u,u.typeParameters)):l,p=ad(t);dv(a.inferences,d,p,128);var f=Ph(t.typeParameters,t,a.flags),m=z_(o,s&&s.returnMapper);dv(f.inferences,m,p),a.returnMapper=e.some(f.inferences,KA)?Kh(jh(f)):void 0}}var _=vD(t),g=_?Math.min(_D(t)-1,r.length):r.length;if(_&&262144&_.flags){var y=e.find(a.inferences,function(e){return e.typeParameter===_});y&&(y.impliedArity=e.findIndex(r,Px,g)<0?r.length-g:void 0)}var h=nd(t);if(h){var v=eL(n);dv(a.inferences,zx(v),h)}for(var b=0;b<g;b++){var E=r[b];if(225!==E.kind){var T=pD(t,b),S=NA(E,T,a,i);dv(a.inferences,S,T)}}if(_){var x=Xx(r,g,r.length,_,a,i);dv(a.inferences,x,_)}return Tv(a)}function qx(e){return 1048576&e.flags?Db(e,qx):1&e.flags||Ay(ru(e)||e)?e:Jy(e)?Wp(Md(e),e.target.elementFlags,!1,e.target.labeledElementDeclarations):Wp([e],[8])}function Xx(n,t,r,i,a,o){if(t>=r-1){var s=n[r-1];if(Px(s))return qx(230===s.kind?s.type:NA(s.expression,i,a,o))}for(var c=[],l=[],u=[],d=t;d<r;d++){s=n[d];if(Px(s)){var p=230===s.kind?s.type:eC(s.expression);Iy(p)?(c.push(p),l.push(8)):(c.push(fN(33,p,Me,223===s.kind?s.expression:s)),l.push(4))}else{var f=vm(i,$m(d-t),256),m=NA(s,f,a,o),_=iA(f,406978556);c.push(_?Ym(m):jy(m)),l.push(1)}230===s.kind&&s.tupleNameSource&&u.push(s.tupleNameSource)}return Wp(c,l,!1,e.length(u)===e.length(c)?u:void 0)}function Yx(n,t,r,i){for(var a,o=e.isInJSFile(n.declaration),s=n.typeParameters,c=Ju(e.map(t,u_),s,zu(s),o),l=0;l<t.length;l++){e.Debug.assert(void 0!==s[l],"Should not call checkTypeArguments with too many type arguments");var u=ql(s[l]);if(u){var d=r&&i?function(){return e.chainDiagnosticMessages(void 0,e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1)}:void 0,p=i||e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1;a||(a=g_(s,c));var f=c[l];if(!fg(f,Xc(z_(u,a),f),r?t[l]:void 0,p,d))return}}return c}function Qx(n){if(iS(n.tagName))return 2;var t=mu(eC(n.tagName));return e.length(Au(t,1))?0:e.length(Au(t,0))?1:2}function Zx(n,t,r,i,a,o,s){var c=TT(t,n),l=NA(n.attributes,c,void 0,i);return u()&&_g(l,c,r,a?n.tagName:void 0,n.attributes,void 0,o,s);function u(){var t;if(pS(n))return!0;var r=e.isJsxOpeningElement(n)||e.isJsxSelfClosingElement(n)&&!iS(n.tagName)?eC(n.tagName):void 0;if(!r)return!0;var i=Au(r,0);if(!e.length(i))return!0;var o=xO(n);if(!o)return!0;var c=ra(o,111551,!0,!1,n);if(!c)return!0;var l=Ws(c),u=Au(l,0);if(!e.length(u))return!0;for(var d=!1,p=0,f=0,m=u;f<m.length;f++){var _=m[f],g=pD(_,0),y=Au(g,0);if(e.length(y))for(var h=0,v=y;h<v.length;h++){var b=v[h];if(d=!0,yD(b))return!0;var E=_D(b);E>p&&(p=E)}}if(!d)return!0;for(var T=1/0,S=0,x=i;S<x.length;S++){var L=x[S],D=gD(L);D<T&&(T=D)}if(T<=p)return!0;if(a){var A=e.createDiagnosticForNode(n.tagName,e.Diagnostics.Tag_0_expects_at_least_1_arguments_but_the_JSX_factory_2_provides_at_most_3,e.entityNameToString(n.tagName),T,e.entityNameToString(o),p),C=null===(t=CR(n.tagName))||void 0===t?void 0:t.valueDeclaration;C&&e.addRelatedInfo(A,e.createDiagnosticForNode(C,e.Diagnostics._0_is_declared_here,e.entityNameToString(n.tagName))),s&&s.skipLogging&&(s.errors||(s.errors=[])).push(A),s.skipLogging||ur.add(A)}return!1}}function $x(n,t,r,i,a,o,s){var c={errors:void 0,skipLogging:!0};if(e.isJsxOpeningLikeElement(n))return Zx(n,r,i,a,o,s,c)?void 0:(e.Debug.assert(!o||!!c.errors,"jsx should have errors when reporting errors"),c.errors||e.emptyArray);var l=nd(r);if(l&&l!==Qe&&207!==n.kind){var u=eL(n),d=zx(u),p=o?u||n:void 0,f=e.Diagnostics.The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1;if(!zg(d,l,i,p,f,s,c))return e.Debug.assert(!o||!!c.errors,"this parameter should have errors when reporting errors"),c.errors||e.emptyArray}for(var m=e.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1,_=vD(r),g=_?Math.min(_D(r)-1,t.length):t.length,y=0;y<g;y++){var h=t[y];if(225!==h.kind){var v=pD(r,y),b=NA(h,v,void 0,a),E=4&a?Eh(b):b;if(!_g(E,v,i,o?h:void 0,h,m,s,c))return e.Debug.assert(!o||!!c.errors,"parameter should have errors when reporting errors"),x(h,E,v),c.errors||e.emptyArray}}if(_){var T=Xx(t,g,t.length,_,void 0,a),S=t.length-g;p=o?0===S?n:1===S?t[g]:e.setTextRangePosEnd(nL(n,T),t[g].pos,t[t.length-1].end):void 0;if(!zg(T,_,i,p,m,void 0,c))return e.Debug.assert(!o||!!c.errors,"rest parameter should have errors when reporting errors"),x(p,T,_),c.errors||e.emptyArray}return;function x(n,t,r){if(n&&o&&c.errors&&c.errors.length){if(zC(r))return;var a=zC(t);a&&Kg(a,r,i)&&e.addRelatedInfo(c.errors[0],e.createDiagnosticForNode(n,e.Diagnostics.Did_you_forget_to_use_await))}}}function eL(n){var t=206===n.kind?n.expression:208===n.kind?n.tag:void 0;if(t){var r=e.skipOuterExpressions(t);if(e.isAccessExpression(r))return r.expression}}function nL(n,t,r,i){var a=e.parseNodeFactory.createSyntheticExpression(t,r,i);return e.setTextRange(a,n),e.setParent(a,n),a}function tL(n){if(208===n.kind){var t=n.template,r=[nL(t,lp())];return 221===t.kind&&e.forEach(t.templateSpans,function(e){r.push(e.expression)}),r}if(163===n.kind)return rL(n);if(e.isJsxOpeningLikeElement(n))return n.attributes.properties.length>0||e.isJsxOpeningElement(n)&&n.parent.children.length>0?[n.attributes]:e.emptyArray;var i=n.arguments||e.emptyArray,a=Mx(i);if(a>=0){for(var o=i.slice(0,a),s=function(n){var t=i[n],r=223===t.kind&&(Mt?eC(t.expression):kA(t.expression));r&&Jy(r)?e.forEach(Md(r),function(e,n){var i,a=r.target.elementFlags[n],s=nL(t,4&a?Mp(e):e,!!(12&a),null===(i=r.target.labeledElementDeclarations)||void 0===i?void 0:i[n]);o.push(s)}):o.push(t)},c=a;c<i.length;c++)s(c);return o}return i}function rL(n){var t=n.parent,r=n.expression;switch(t.kind){case 255:case 224:return[nL(r,Ws(Ca(t)))];case 162:var i=t.parent;return[nL(r,169===t.parent.kind?Ws(Ca(i)):Re),nL(r,Ie),nL(r,Ke)];case 165:case 167:case 170:case 171:var a=165!==t.kind&&0!==F;return[nL(r,MR(t)),nL(r,FR(t)),nL(r,a?wp(RR(t)):Ie)]}return e.Debug.fail()}function iL(n,t){switch(n.parent.kind){case 255:case 224:return 1;case 165:return 2;case 167:case 170:case 171:return 0===F||t.parameters.length<=2?2:3;case 162:return 3;default:return e.Debug.fail()}}function aL(n,t){var r,i,a=e.getSourceFileOfNode(n);if(e.isPropertyAccessExpression(n.expression)){var o=e.getErrorSpanForNode(a,n.expression.name);r=o.start,i=t?o.length:n.end-r}else{var s=e.getErrorSpanForNode(a,n.expression);r=s.start,i=t?s.length:n.end-r}return{start:r,length:i,sourceFile:a}}function oL(n,t,r,i,a,o){if(e.isCallExpression(n)){var s=aL(n),c=s.sourceFile,l=s.start,u=s.length;return e.createFileDiagnostic(c,l,u,t,r,i,a,o)}return e.createDiagnosticForNode(n,t,r,i,a,o)}function sL(n){if(!e.isCallExpression(n)||!e.isIdentifier(n.expression))return!1;var t=Qr(n.expression,n.expression.escapedText,111551,void 0,void 0,!1),r=null===t||void 0===t?void 0:t.valueDeclaration;if(!r||!e.isParameter(r)||!wT(r.parent)||!e.isNewExpression(r.parent.parent)||!e.isIdentifier(r.parent.parent.expression))return!1;var i=yp(!1);if(!i)return!1;var a=CR(r.parent.parent.expression,!0);return a===i}function cL(n,t,r){var i,a=Mx(r);if(a>-1)return e.createDiagnosticForNode(r[a],e.Diagnostics.A_spread_argument_must_either_have_a_tuple_type_or_be_passed_to_a_rest_parameter);for(var o,s=Number.POSITIVE_INFINITY,c=Number.NEGATIVE_INFINITY,l=Number.NEGATIVE_INFINITY,u=Number.POSITIVE_INFINITY,d=0,p=t;d<p.length;d++){var f=p[d],m=gD(f),_=_D(f);m<s&&(s=m,o=f),c=Math.max(c,_),m<r.length&&m>l&&(l=m),r.length<_&&_<u&&(u=_)}var g=e.some(t,yD),y=g?s:s<c?s+"-"+c:s,h=g?e.Diagnostics.Expected_at_least_0_arguments_but_got_1:1===y&&0===r.length&&sL(n)?e.Diagnostics.Expected_0_arguments_but_got_1_Did_you_forget_to_include_void_in_your_type_argument_to_Promise:e.Diagnostics.Expected_0_arguments_but_got_1;if(s<r.length&&r.length<c)return oL(n,e.Diagnostics.No_overload_expects_0_arguments_but_overloads_do_exist_that_expect_either_1_or_2_arguments,r.length,l,u);if(r.length<s){var v=oL(n,h,y,r.length),b=null===(i=null===o||void 0===o?void 0:o.declaration)||void 0===i?void 0:i.parameters[o.thisParameter?r.length+1:r.length];if(b){var E=e.createDiagnosticForNode(b,e.isBindingPattern(b.name)?e.Diagnostics.An_argument_matching_this_binding_pattern_was_not_provided:e.isRestParameter(b)?e.Diagnostics.Arguments_for_the_rest_parameter_0_were_not_provided:e.Diagnostics.An_argument_for_0_was_not_provided,b.name?e.isBindingPattern(b.name)?void 0:e.idText(e.getFirstIdentifier(b.name)):r.length);return e.addRelatedInfo(v,E)}return v}var T=e.factory.createNodeArray(r.slice(c)),S=e.first(T).pos,x=e.last(T).end;return x===S&&x++,e.setTextRangePosEnd(T,S,x),e.createDiagnosticForNodeArray(e.getSourceFileOfNode(n),T,h,y,r.length)}function lL(n,t,r){var i=r.length;if(1===t.length){var a=t[0],o=zu(a.typeParameters),s=e.length(a.typeParameters);return e.createDiagnosticForNodeArray(e.getSourceFileOfNode(n),r,e.Diagnostics.Expected_0_type_arguments_but_got_1,o<s?o+"-"+s:o,i)}for(var c=-1/0,l=1/0,u=0,d=t;u<d.length;u++){a=d[u];var p=zu(a.typeParameters);s=e.length(a.typeParameters);p>i?l=Math.min(l,p):s<i&&(c=Math.max(c,s))}return c!==-1/0&&l!==1/0?e.createDiagnosticForNodeArray(e.getSourceFileOfNode(n),r,e.Diagnostics.No_overload_expects_0_type_arguments_but_overloads_do_exist_that_expect_either_1_or_2_type_arguments,i,c,l):e.createDiagnosticForNodeArray(e.getSourceFileOfNode(n),r,e.Diagnostics.Expected_0_type_arguments_but_got_1,c===-1/0?l:c,i)}function uL(n,r,i,o,s,c){var l,u=208===n.kind,d=163===n.kind,p=e.isJsxOpeningLikeElement(n),f=!i&&t;d||(l=n.typeArguments,(u||p||106!==n.expression.kind)&&e.forEach(l,Zk));var m=i||[];if(wx(r,m,s),!m.length)return f&&ur.add(oL(n,e.Diagnostics.Call_target_does_not_contain_any_signatures)),Ox(n);var _,g,y,h,v=tL(n),b=1===m.length&&!m[0].typeParameters,E=d||b||!e.some(v,$_)?0:4,T=!!(16&o)&&206===n.kind&&n.arguments.hasTrailingComma;if(m.length>1&&(h=q(m,mr,b,T)),h||(h=q(m,gr,b,T)),h)return h;if(f)if(_)if(1===_.length||_.length>3){var S,x=_[_.length-1];_.length>3&&(S=e.chainDiagnosticMessages(S,e.Diagnostics.The_last_overload_gave_the_following_error),S=e.chainDiagnosticMessages(S,e.Diagnostics.No_overload_matches_this_call));var L=$x(n,v,x,gr,0,!0,function(){return S});if(L)for(var D=0,A=L;D<A.length;D++){var C=A[D];x.declaration&&_.length>3&&e.addRelatedInfo(C,e.createDiagnosticForNode(x.declaration,e.Diagnostics.The_last_overload_is_declared_here)),J(x,C),ur.add(C)}else e.Debug.fail("No error for last overload signature")}else{for(var I=[],N=0,k=Number.MAX_VALUE,R=0,O=0,w=function(t){var r=function(){return e.chainDiagnosticMessages(void 0,e.Diagnostics.Overload_0_of_1_2_gave_the_following_error,O+1,m.length,_o(t))},i=$x(n,v,t,gr,0,!0,r);i?(i.length<=k&&(k=i.length,R=O),N=Math.max(N,i.length),I.push(i)):e.Debug.fail("No error for 3 or fewer overload signatures"),O++},P=0,M=_;P<M.length;P++){var F=M[P];w(F)}var G=N>1?I[R]:e.flatten(I);e.Debug.assert(G.length>0,"No errors reported for 3 or fewer overload signatures");var B=e.chainDiagnosticMessages(e.map(G,function(e){return"string"===typeof e.messageText?e:e.messageText}),e.Diagnostics.No_overload_matches_this_call),U=a([],e.flatMap(G,function(e){return e.relatedInformation}),!0),V=void 0;if(e.every(G,function(e){return e.start===G[0].start&&e.length===G[0].length&&e.file===G[0].file})){var j=G[0],K=j.file,H=j.start,W=j.length;V={file:K,start:H,length:W,code:B.code,category:B.category,messageText:B,relatedInformation:U}}else V=e.createDiagnosticForNodeFromMessageChain(n,B,U);J(_[0],V),ur.add(V)}else if(g)ur.add(cL(n,[g],v));else if(y)Yx(y,n.typeArguments,!0,c);else{var z=e.filter(r,function(e){return Ux(e,l)});0===z.length?ur.add(lL(n,r,l)):d?c&&ur.add(oL(n,c)):ur.add(cL(n,z,v))}return dL(n,m,v,!!i);function J(n,t){var r,i,a=_,o=g,s=y,c=(null===(i=null===(r=n.declaration)||void 0===r?void 0:r.symbol)||void 0===i?void 0:i.declarations)||e.emptyArray,l=c.length>1,u=l?e.find(c,function(n){return e.isFunctionLikeDeclaration(n)&&e.nodeIsPresent(n.body)}):void 0;if(u){var d=qu(u),p=!d.typeParameters;q([d],gr,p)&&e.addRelatedInfo(t,e.createDiagnosticForNode(u,e.Diagnostics.The_call_would_have_succeeded_against_this_implementation_but_implementation_signatures_of_overloads_are_not_externally_visible))}_=a,g=o,y=s}function q(t,r,i,a){if(void 0===a&&(a=!1),_=void 0,g=void 0,y=void 0,i){var o=t[0];if(e.some(l)||!Bx(n,v,o,a))return;return $x(n,v,o,r,0,!1,void 0)?void(_=[o]):o}for(var s=0;s<t.length;s++){o=t[s];if(Ux(o,l)&&Bx(n,v,o,a)){var c=void 0,u=void 0;if(o.typeParameters){var d=void 0;if(e.some(l)){if(d=Yx(o,l,!1),!d){y=o;continue}}else u=Ph(o.typeParameters,o,e.isInJSFile(n)?2:0),d=Jx(n,o,v,8|E,u),E|=4&u.flags?8:0;if(c=ud(o,d,e.isInJSFile(o.declaration),u&&u.inferredTypeParameters),vD(o)&&!Bx(n,v,c,a)){g=c;continue}}else c=o;if(!$x(n,v,c,r,E,!1,void 0)){if(E){if(E=0,u){d=Jx(n,o,v,E,u);if(c=ud(o,d,e.isInJSFile(o.declaration),u&&u.inferredTypeParameters),vD(o)&&!Bx(n,v,c,a)){g=c;continue}}if($x(n,v,c,r,E,!1,void 0)){(_||(_=[])).push(c);continue}}return t[s]=c,c}(_||(_=[])).push(c)}}}}function dL(n,t,r,i){return e.Debug.assert(t.length>0),rR(n),i||1===t.length||t.some(function(e){return!!e.typeParameters})?gL(n,t,r):pL(t)}function pL(n){var t,r=e.mapDefined(n,function(e){return e.thisParameter});r.length&&(t=mL(r,r.map(aD)));for(var i=e.minAndMax(n,fL),a=i.min,o=i.max,s=[],c=function(t){var r=e.mapDefined(n,function(n){return V(n)?t<n.parameters.length-1?n.parameters[t]:e.last(n.parameters):t<n.parameters.length?n.parameters[t]:void 0});e.Debug.assert(0!==r.length),s.push(mL(r,e.mapDefined(n,function(e){return fD(e,t)})))},l=0;l<o;l++)c(l);var u=e.mapDefined(n,function(n){return V(n)?e.last(n.parameters):void 0}),d=0;if(0!==u.length){var p=Mp(ff(e.mapDefined(n,ld),2));s.push(_L(u,p)),d|=1}return n.some(j)&&(d|=2),$c(n[0].declaration,void 0,t,s,Df(n.map(ad)),void 0,a,d)}function fL(e){var n=e.parameters.length;return V(e)?n-1:n}function mL(e,n){return _L(e,ff(n,2))}function _L(n,t){return vh(e.first(n),t)}function gL(n,t,r){var i=vL(t,void 0===ce?r.length:ce),a=t[i],o=a.typeParameters;if(!o)return a;var s=kx(n)?n.typeArguments:void 0,c=s?pd(a,yL(s,o,e.isInJSFile(n))):hL(n,o,a,r);return t[i]=c,c}function yL(e,n,t){var r=e.map(RR);while(r.length>n.length)r.pop();while(r.length<n.length)r.push(ql(n[r.length])||Ev(t));return r}function hL(n,t,r,i){var a=Ph(t,r,e.isInJSFile(n)?2:0),o=Jx(n,r,i,12,a);return pd(r,o)}function vL(e,n){for(var t=-1,r=-1,i=0;i<e.length;i++){var a=e[i],o=_D(a);if(yD(a)||o>=n)return i;o>r&&(r=o,t=i)}return t}function bL(n,t,r){if(106===n.expression.kind){var i=wE(n.expression);if(Vo(i)){for(var a=0,o=n.arguments;a<o.length;a++){var s=o[a];eC(s)}return bt}if(i!==Re){var c=e.getEffectiveBaseTypeNode(e.getContainingClass(n));if(c){var l=ac(i,c.typeArguments,c);return uL(n,l,t,r,0)}}return Rx(n)}var u,d=eC(n.expression);if(e.isCallChain(n)){var p=fh(d,n.expression);u=p===d?0:e.isOutermostOptionalChain(n)?16:8,d=p}else u=0;if(d=VS(d,n.expression,US),d===$e)return St;var f=mu(d);if(f===Re)return Ox(n);var m=Au(f,0),_=Au(f,1).length;if(TL(d,f,m.length,_))return d!==Re&&n.typeArguments&&Dr(n,e.Diagnostics.Untyped_function_calls_may_not_accept_type_arguments),Rx(n);if(!m.length){if(_)Dr(n,e.Diagnostics.Value_of_type_0_is_not_callable_Did_you_mean_to_include_new,go(d));else{var g=void 0;if(1===n.arguments.length){var y=e.getSourceFileOfNode(n).text;e.isLineBreak(y.charCodeAt(e.skipTrivia(y,n.expression.end,!0)-1))&&(g=e.createDiagnosticForNode(n.expression,e.Diagnostics.Are_you_missing_a_semicolon))}AL(n.expression,f,0,g)}return Ox(n)}return 8&r&&!n.typeArguments&&m.some(EL)?(jA(n,r),Tt):m.some(function(n){return e.isInJSFile(n.declaration)&&!!e.getJSDocClassTag(n.declaration)})?(Dr(n,e.Diagnostics.Value_of_type_0_is_not_callable_Did_you_mean_to_include_new,go(d)),Ox(n)):uL(n,m,t,r,u)}function EL(e){return!(!e.typeParameters||!fO(ad(e)))}function TL(e,n,t,r){return Vo(e)||Vo(n)&&!!(262144&e.flags)||!t&&!r&&!(1048576&n.flags)&&!(131072&vu(n).flags)&&lg(e,Cn)}function SL(n,t,r){if(n.arguments&&F<1){var i=Mx(n.arguments);i>=0&&Dr(n.arguments[i],e.Diagnostics.Spread_operator_in_new_expressions_is_only_available_when_targeting_ECMAScript_5_and_higher)}var a=MS(n.expression);if(a===$e)return St;if(a=mu(a),a===Re)return Ox(n);if(Vo(a))return n.typeArguments&&Dr(n,e.Diagnostics.Untyped_function_calls_may_not_accept_type_arguments),Rx(n);var o=Au(a,1);if(o.length){if(!LL(n,o[0]))return Ox(n);if(o.some(function(e){return 4&e.flags}))return Dr(n,e.Diagnostics.Cannot_create_an_instance_of_an_abstract_class),Ox(n);var s=a.symbol&&e.getClassLikeDeclarationOfSymbol(a.symbol);return s&&e.hasSyntacticModifier(s,128)?(Dr(n,e.Diagnostics.Cannot_create_an_instance_of_an_abstract_class),Ox(n)):uL(n,o,t,r,0)}var c=Au(a,0);if(c.length){var l=uL(n,c,t,r,0);return Y||(l.declaration&&!FL(l.declaration)&&ad(l)!==Qe&&Dr(n,e.Diagnostics.Only_a_void_function_can_be_called_with_the_new_keyword),nd(l)===Qe&&Dr(n,e.Diagnostics.A_function_that_is_called_with_the_new_keyword_cannot_have_a_this_type_that_is_void)),l}return AL(n.expression,a,1),Ox(n)}function xL(n,t){var r=lc(t);if(!e.length(r))return!1;var i=r[0];if(2097152&i.flags){for(var a=i.types,o=gl(a),s=0,c=0,l=i.types;c<l.length;c++){var u=l[c];if(!o[s]&&3&e.getObjectFlags(u)){if(u.symbol===n)return!0;if(xL(n,u))return!0}s++}return!1}return i.symbol===n||xL(n,i)}function LL(n,t){if(!t||!t.declaration)return!0;var r=t.declaration,i=e.getSelectedEffectiveModifierFlags(r,24);if(!i||169!==r.kind)return!0;var a=e.getClassLikeDeclarationOfSymbol(r.parent.symbol),o=Dc(r.parent.symbol);if(!ER(n,a)){var s=e.getContainingClass(n);if(s&&16&i){var c=RR(s);if(xL(r.parent.symbol,c))return!0}return 8&i&&Dr(n,e.Diagnostics.Constructor_of_class_0_is_private_and_only_accessible_within_the_class_declaration,go(o)),16&i&&Dr(n,e.Diagnostics.Constructor_of_class_0_is_protected_and_only_accessible_within_the_class_declaration,go(o)),!1}return!0}function DL(n,t,r){var i,a=0===r,o=YC(t),s=o&&Au(o,r).length>0;if(1048576&t.flags){for(var c=t.types,l=!1,u=0,d=c;u<d.length;u++){var p=d[u],f=Au(p,r);if(0!==f.length){if(l=!0,i)break}else if(i||(i=e.chainDiagnosticMessages(i,a?e.Diagnostics.Type_0_has_no_call_signatures:e.Diagnostics.Type_0_has_no_construct_signatures,go(p)),i=e.chainDiagnosticMessages(i,a?e.Diagnostics.Not_all_constituents_of_type_0_are_callable:e.Diagnostics.Not_all_constituents_of_type_0_are_constructable,go(t))),l)break}l||(i=e.chainDiagnosticMessages(void 0,a?e.Diagnostics.No_constituent_of_type_0_is_callable:e.Diagnostics.No_constituent_of_type_0_is_constructable,go(t))),i||(i=e.chainDiagnosticMessages(i,a?e.Diagnostics.Each_member_of_the_union_type_0_has_signatures_but_none_of_those_signatures_are_compatible_with_each_other:e.Diagnostics.Each_member_of_the_union_type_0_has_construct_signatures_but_none_of_those_signatures_are_compatible_with_each_other,go(t)))}else i=e.chainDiagnosticMessages(i,a?e.Diagnostics.Type_0_has_no_call_signatures:e.Diagnostics.Type_0_has_no_construct_signatures,go(t));var m=a?e.Diagnostics.This_expression_is_not_callable:e.Diagnostics.This_expression_is_not_constructable;if(e.isCallExpression(n.parent)&&0===n.parent.arguments.length){var _=Wr(n).resolvedSymbol;_&&32768&_.flags&&(m=e.Diagnostics.This_expression_is_not_callable_because_it_is_a_get_accessor_Did_you_mean_to_use_it_without)}return{messageChain:e.chainDiagnosticMessages(i,m),relatedMessage:s?e.Diagnostics.Did_you_forget_to_use_await:void 0}}function AL(n,t,r,i){var a=DL(n,t,r),o=a.messageChain,s=a.relatedMessage,c=e.createDiagnosticForNodeFromMessageChain(n,o);if(s&&e.addRelatedInfo(c,e.createDiagnosticForNode(n,s)),e.isCallExpression(n.parent)){var l=aL(n.parent,!0),u=l.start,d=l.length;c.start=u,c.length=d}ur.add(c),CL(t,r,i?e.addRelatedInfo(c,i):c)}function CL(n,t,r){if(n.symbol){var i=Hr(n.symbol).originatingImport;if(i&&!e.isImportCall(i)){var a=Au(Ws(Hr(n.symbol).target),t);if(!a||!a.length)return;e.addRelatedInfo(r,e.createDiagnosticForNode(i,e.Diagnostics.Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cause_a_failure_at_runtime_Consider_using_a_default_import_or_import_require_here_instead))}}}function IL(n,t,r){var i=eC(n.tag),a=mu(i);if(a===Re)return Ox(n);var o=Au(a,0),s=Au(a,1).length;if(TL(i,a,o.length,s))return Rx(n);if(!o.length){if(e.isArrayLiteralExpression(n.parent)){var c=e.createDiagnosticForNode(n.tag,e.Diagnostics.It_is_likely_that_you_are_missing_a_comma_to_separate_these_two_template_expressions_They_form_a_tagged_template_expression_which_cannot_be_invoked);return ur.add(c),Ox(n)}return AL(n.tag,a,0),Ox(n)}return uL(n,o,t,r,0)}function NL(n){switch(n.parent.kind){case 255:case 224:return e.Diagnostics.Unable_to_resolve_signature_of_class_decorator_when_called_as_an_expression;case 162:return e.Diagnostics.Unable_to_resolve_signature_of_parameter_decorator_when_called_as_an_expression;case 165:return e.Diagnostics.Unable_to_resolve_signature_of_property_decorator_when_called_as_an_expression;case 167:case 170:case 171:return e.Diagnostics.Unable_to_resolve_signature_of_method_decorator_when_called_as_an_expression;default:return e.Debug.fail()}}function kL(n,t,r){var i=eC(n.expression),a=mu(i);if(a===Re)return Ox(n);var o=Au(a,0),s=Au(a,1).length;if(TL(i,a,o.length,s))return Rx(n);if(wL(n,o)){var c=e.getTextOfNode(n.expression,!1);return Dr(n,e.Diagnostics._0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write_0,c),Ox(n)}var l=NL(n);if(!o.length){var u=DL(n.expression,a,0),d=e.chainDiagnosticMessages(u.messageChain,l),p=e.createDiagnosticForNodeFromMessageChain(n.expression,d);return u.relatedMessage&&e.addRelatedInfo(p,e.createDiagnosticForNode(n.expression,u.relatedMessage)),ur.add(p),CL(a,0,p),Ox(n)}return uL(n,o,t,r,0,l)}function RL(n,t){var r=fS(n),i=r&&Sa(r),a=i&&Jr(i,I.Element,788968),o=a&&ie.symbolToEntityName(a,788968,n),s=e.factory.createFunctionTypeNode(void 0,[e.factory.createParameterDeclaration(void 0,void 0,void 0,"props",void 0,ie.typeToTypeNode(t,n))],o?e.factory.createTypeReferenceNode(o,void 0):e.factory.createKeywordTypeNode(129)),c=Or(1,"props");return c.type=t,$c(s,void 0,void 0,[c],a?Dc(a):Re,void 0,1,0)}function OL(n,t,r){if(iS(n.tagName)){var i=ES(n),a=RL(n,i);return mg(NA(n.attributes,TT(a,n),void 0,0),i,n.tagName,n.attributes),e.length(n.typeArguments)&&(e.forEach(n.typeArguments,Zk),ur.add(e.createDiagnosticForNodeArray(e.getSourceFileOfNode(n),n.typeArguments,e.Diagnostics.Expected_0_type_arguments_but_got_1,0,e.length(n.typeArguments)))),a}var o=eC(n.tagName),s=mu(o);if(s===Re)return Ox(n);var c=hS(o,n);return TL(o,s,c.length,0)?Rx(n):0===c.length?(Dr(n.tagName,e.Diagnostics.JSX_element_type_0_does_not_have_any_construct_or_call_signatures,e.getTextOfNode(n.tagName)),Ox(n)):uL(n,c,t,r,0)}function wL(n,t){return t.length&&e.every(t,function(e){return 0===e.minArgumentCount&&!V(e)&&e.parameters.length<iL(n,e)})}function PL(n,t,r){switch(n.kind){case 206:return bL(n,t,r);case 207:return SL(n,t,r);case 208:return IL(n,t,r);case 163:return kL(n,t,r);case 278:case 277:return OL(n,t,r)}throw e.Debug.assertNever(n,"Branch in 'resolveSignature' should be unreachable.")}function ML(e,n,t){var r=Wr(e),i=r.resolvedSignature;if(i&&i!==Tt&&!n)return i;r.resolvedSignature=Tt;var a=PL(e,n,t||0);return a!==Tt&&(r.resolvedSignature=Pt===Mt?a:i),a}function FL(n){var t;if(!n||!e.isInJSFile(n))return!1;var r=e.isFunctionDeclaration(n)||e.isFunctionExpression(n)?n:e.isVariableDeclaration(n)&&n.initializer&&e.isFunctionExpression(n.initializer)?n.initializer:void 0;if(r){if(e.getJSDocClassTag(n))return!0;var i=Ca(r);return!!(null===(t=null===i||void 0===i?void 0:i.members)||void 0===t?void 0:t.size)}return!1}function GL(n,t){var r,i;if(t){var a=Hr(t);if(!a.inferredClassSymbol||!a.inferredClassSymbol.has(w(n))){var o=e.isTransientSymbol(n)?n:Mr(n);return o.exports=o.exports||e.createSymbolTable(),o.members=o.members||e.createSymbolTable(),o.flags|=32&t.flags,(null===(r=t.exports)||void 0===r?void 0:r.size)&&Vr(o.exports,t.exports),(null===(i=t.members)||void 0===i?void 0:i.size)&&Vr(o.members,t.members),(a.inferredClassSymbol||(a.inferredClassSymbol=new e.Map)).set(w(o),o),o}return a.inferredClassSymbol.get(w(n))}}function BL(e){var n,t=e&&UL(e,!0),r=null===(n=null===t||void 0===t?void 0:t.exports)||void 0===n?void 0:n.get("prototype"),i=(null===r||void 0===r?void 0:r.valueDeclaration)&&VL(r.valueDeclaration);return i?Ca(i):void 0}function UL(n,t){if(n.parent){var r,i;if(e.isVariableDeclaration(n.parent)&&n.parent.initializer===n){if(!e.isInJSFile(n)&&(!e.isVarConst(n.parent)||!e.isFunctionLikeDeclaration(n)))return;r=n.parent.name,i=n.parent}else if(e.isBinaryExpression(n.parent)){var a=n.parent,o=n.parent.operatorToken.kind;if(63!==o||!t&&a.right!==n){if((56===o||60===o)&&(e.isVariableDeclaration(a.parent)&&a.parent.initializer===a?(r=a.parent.name,i=a.parent):e.isBinaryExpression(a.parent)&&63===a.parent.operatorToken.kind&&(t||a.parent.right===a)&&(r=a.parent.left,i=r),!r||!e.isBindableStaticNameExpression(r)||!e.isSameEntityName(r,a.left)))return}else r=a.left,i=r}else t&&e.isFunctionDeclaration(n)&&(r=n.name,i=n);if(i&&r&&(t||e.getExpandoInitializer(n,e.isPrototypeAccess(r))))return Ca(i)}}function VL(n){if(!n.parent)return!1;var t=n.parent;while(t&&204===t.kind)t=t.parent;if(t&&e.isBinaryExpression(t)&&e.isPrototypeAccess(t.left)&&63===t.operatorToken.kind){var r=e.getInitializerOfBinaryExpression(t);return e.isObjectLiteralExpression(r)&&r}}function jL(n,t){var r;YO(n,n.typeArguments)||$O(n.arguments);var i=ML(n,void 0,t);if(i===Tt)return en;if(KL(i,n),106===n.expression.kind)return Qe;if(207===n.kind){var a=i.declaration;if(a&&169!==a.kind&&173!==a.kind&&178!==a.kind&&!e.isJSDocConstructSignature(a)&&!FL(a))return Y&&Dr(n,e.Diagnostics.new_expression_whose_target_lacks_a_construct_signature_implicitly_has_an_any_type),Ie}if(e.isInJSFile(n)&&qL(n))return ed(n.arguments[0]);var o=ad(i);if(12288&o.flags&&WL(n))return i_(e.walkUpParenthesizedExpressions(n.parent));if(206===n.kind&&!n.questionDotToken&&236===n.parent.kind&&16384&o.flags&&td(i))if(e.isDottedName(n.expression)){if(!zb(n)){var s=Dr(n.expression,e.Diagnostics.Assertions_require_every_name_in_the_call_target_to_be_declared_with_an_explicit_type_annotation);Wb(n.expression,s)}}else Dr(n.expression,e.Diagnostics.Assertions_require_the_call_target_to_be_an_identifier_or_qualified_name);if(e.isInJSFile(n)){var c=UL(n,!1);if(null===(r=null===c||void 0===c?void 0:c.exports)||void 0===r?void 0:r.size){var l=Xa(c,c.exports,e.emptyArray,e.emptyArray,e.emptyArray);return l.objectFlags|=8192,Df([o,l])}}return o}function KL(n,t){if(n.declaration&&134217728&n.declaration.flags){var r=HL(t),i=e.tryGetPropertyAccessOrIdentifierToString(e.getInvokedExpression(t));Rr(r,n.declaration,i,_o(n))}}function HL(n){switch(n=e.skipParentheses(n),n.kind){case 206:case 163:case 207:return HL(n.expression);case 208:return HL(n.tag);case 278:case 277:return HL(n.tagName);case 205:return n.argumentExpression;case 204:return n.name;case 176:var t=n;return e.isQualifiedName(t.typeName)?t.typeName.right:t;default:return n}}function WL(n){if(!e.isCallExpression(n))return!1;var t=n.expression;if(e.isPropertyAccessExpression(t)&&"for"===t.name.escapedText&&(t=t.expression),!e.isIdentifier(t)||"Symbol"!==t.escapedText)return!1;var r=pp(!1);return!!r&&r===Qr(t,"Symbol",111551,void 0,void 0,!1)}function zL(n){if($O(n.arguments)||Yw(n),0===n.arguments.length)return ID(n,Ie);for(var t=n.arguments[0],r=kA(t),i=1;i<n.arguments.length;++i)kA(n.arguments[i]);(32768&r.flags||65536&r.flags||!lg(r,je))&&Dr(t,e.Diagnostics.Dynamic_import_s_specifier_must_be_of_type_string_but_here_has_type_0,go(r));var a=ca(n,t);if(a){var o=_a(a,t,!0,!1);if(o)return ID(n,JL(Ws(o),o,a))}return ID(n,Ie)}function JL(n,t,r){var i;if(W&&n&&n!==Re){var a=n;if(!a.syntheticType){var o=null===(i=r.declarations)||void 0===i?void 0:i.find(e.isSourceFile),s=Li(o,r,!1);if(s){var c=e.createSymbolTable(),l=Or(2097152,"default");l.parent=r,l.nameType=Zm("default"),l.target=zi(t),c.set("default",l);var u=Or(2048,"__type"),d=Xa(u,c,e.emptyArray,e.emptyArray,e.emptyArray);u.type=d,a.syntheticType=QT(n)?Hm(n,d,u,0,!1):d}else a.syntheticType=n}return a.syntheticType}return n}function qL(n){if(!e.isRequireCall(n,!0))return!1;if(!e.isIdentifier(n.expression))return e.Debug.fail();var t=Qr(n.expression,n.expression.escapedText,111551,void 0,void 0,!0);if(t===ue)return!0;if(2097152&t.flags)return!1;var r=16&t.flags?254:3&t.flags?252:0;if(0!==r){var i=e.getDeclarationOfKind(t,r);return!!i&&!!(8388608&i.flags)}return!1}function XL(e){QO(e)||YO(e,e.typeArguments),F<2&&IO(e,262144);var n=ML(e);return KL(n,e),ad(n)}function YL(e){return ZL(e,e.type,e.expression)}function QL(n){switch(n.kind){case 10:case 14:case 8:case 9:case 110:case 95:case 202:case 203:case 221:return!0;case 210:return QL(n.expression);case 217:var t=n.operator,r=n.operand;return 40===t&&(8===r.kind||9===r.kind)||39===t&&8===r.kind;case 204:case 205:var i=n.expression;if(e.isIdentifier(i)){var a=CR(i);return a&&2097152&a.flags&&(a=Ji(a)),!!(a&&384&a.flags&&1===bc(a))}}return!1}function ZL(n,r,i,a){var o=eC(i,a);if(e.isConstTypeReference(r))return QL(i)||Dr(i,e.Diagnostics.A_const_assertions_can_only_be_applied_to_references_to_enum_members_or_string_number_boolean_array_or_object_literals),Ym(o);Zk(r),o=Eh(Vy(o));var s=u_(r);if(t&&s!==Re){var c=Ch(o);dg(s,c)||kg(o,s,n,e.Diagnostics.Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the_other_If_this_was_intentional_convert_the_expression_to_unknown_first)}return s}function $L(e){var n=eC(e.expression),t=fh(n,e.expression);return ph(lh(t),e,t!==n)}function eD(e){return 32&e.flags?$L(e):lh(eC(e.expression))}function nD(n){return kw(n),103===n.keywordToken?rD(n):100===n.keywordToken?iD(n):e.Debug.assertNever(n.keywordToken)}function tD(n){switch(n.keywordToken){case 100:return dp();case 103:var t=rD(n);return t===Re?Re:ND(t);default:e.Debug.assertNever(n.keywordToken)}}function rD(n){var t=e.getNewTargetContainer(n);if(t){if(169===t.kind){var r=Ca(t.parent);return Ws(r)}r=Ca(t);return Ws(r)}return Dr(n,e.Diagnostics.Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constructor,"new.target"),Re}function iD(n){K!==e.ModuleKind.ES2020&&K!==e.ModuleKind.ESNext&&K!==e.ModuleKind.System&&Dr(n,e.Diagnostics.The_import_meta_meta_property_is_only_allowed_when_the_module_option_is_es2020_esnext_or_system);var t=e.getSourceFileOfNode(n);return e.Debug.assert(!!(2097152&t.flags),"Containing file is missing import meta node flag."),"meta"===n.name.escapedText?up():Re}function aD(n){var t=Ws(n);if(z){var r=n.valueDeclaration;if(r&&e.hasInitializer(r))return sh(t)}return t}function oD(n){return e.Debug.assert(e.isIdentifier(n.name)),n.name.escapedText}function sD(e,n,t){var r=e.parameters.length-(V(e)?1:0);if(n<r)return e.parameters[n].escapedName;var i=e.parameters[r]||Ae,a=t||Ws(i);if(Jy(a)){var o=a.target.labeledElementDeclarations,s=n-r;return o&&oD(o[s])||i.escapedName+"_"+s}return i.escapedName}function cD(e,n){var t=e.parameters.length-(V(e)?1:0);if(n<t){var r=e.parameters[n];return lD(r)?[r.escapedName,!1]:void 0}var i=e.parameters[t]||Ae;if(lD(i)){var a=Ws(i);if(Jy(a)){var o=a.target.labeledElementDeclarations,s=n-t,c=null===o||void 0===o?void 0:o[s],l=!!(null===c||void 0===c?void 0:c.dotDotDotToken);return c?[oD(c),l]:void 0}return n===t?[i.escapedName,!0]:void 0}}function lD(n){return n.valueDeclaration&&e.isParameter(n.valueDeclaration)&&e.isIdentifier(n.valueDeclaration.name)}function uD(n){return 195===n.kind||e.isParameter(n)&&n.name&&e.isIdentifier(n.name)}function dD(e,n){var t=e.parameters.length-(V(e)?1:0);if(n<t){var r=e.parameters[n].valueDeclaration;return r&&uD(r)?r:void 0}var i=e.parameters[t]||Ae,a=Ws(i);if(Jy(a)){var o=a.target.labeledElementDeclarations,s=n-t;return o&&o[s]}return i.valueDeclaration&&uD(i.valueDeclaration)?i.valueDeclaration:void 0}function pD(e,n){return fD(e,n)||Ie}function fD(e,n){var t=e.parameters.length-(V(e)?1:0);if(n<t)return aD(e.parameters[n]);if(V(e)){var r=Ws(e.parameters[t]),i=n-t;if(!Jy(r)||r.target.hasRestElement||i<r.target.fixedLength)return vm(r,$m(i))}}function mD(n,t){var r=_D(n),i=gD(n),a=hD(n);if(a&&t>=r-1)return t===r-1?a:Mp(vm(a,Ke));for(var o=[],s=[],c=[],l=t;l<r;l++){!a||l<r-1?(o.push(pD(n,l)),s.push(l<i?1:2)):(o.push(a),s.push(8));var u=dD(n,l);u&&c.push(u)}return Wp(o,s,!1,e.length(c)===e.length(o)?c:void 0)}function _D(e){var n=e.parameters.length;if(V(e)){var t=Ws(e.parameters[n-1]);if(Jy(t))return n+t.target.fixedLength-(t.target.hasRestElement?0:1)}return n}function gD(n,t){var r=1&t,i=2&t;if(i||void 0===n.resolvedMinArgumentCount){var a=void 0;if(V(n)){var o=Ws(n.parameters[n.parameters.length-1]);if(Jy(o)){var s=e.findIndex(o.target.elementFlags,function(e){return!(1&e)}),c=s<0?o.target.fixedLength:s;c>0&&(a=n.parameters.length-1+c)}}if(void 0===a){if(!r&&32&n.flags)return 0;a=n.minArgumentCount}if(i)return a;for(var l=a-1;l>=0;l--){var u=pD(n,l);if(131072&Sb(u,Fx).flags)break;a=l}n.resolvedMinArgumentCount=a}return n.resolvedMinArgumentCount}function yD(e){if(V(e)){var n=Ws(e.parameters[e.parameters.length-1]);return!Jy(n)||n.target.hasRestElement}return!1}function hD(e){if(V(e)){var n=Ws(e.parameters[e.parameters.length-1]);if(!Jy(n))return n;if(n.target.hasRestElement)return Yp(n,n.target.fixedLength)}}function vD(e){var n=hD(e);return!n||Ly(n)||Vo(n)||0!==(131072&vu(n).flags)?void 0:n}function bD(e){return ED(e,Ze)}function ED(e,n){return e.parameters.length>0?pD(e,0):n}function TD(n,t,r){for(var i=n.parameters.length-(V(n)?1:0),a=0;a<i;a++){var o=n.parameters[a].valueDeclaration;if(o.type){var s=e.getEffectiveTypeAnnotationNode(o);s&&dv(r.inferences,u_(s),pD(t,a))}}var c=hD(t);if(c&&262144&c.flags){var l=k_(t,r.nonFixingMapper);SD(n,l);var u=_D(t)-1;dv(r.inferences,mD(n,u),c)}}function SD(n,t){if(t.typeParameters){if(n.typeParameters)return;n.typeParameters=t.typeParameters}if(t.thisParameter){var r=n.thisParameter;(!r||r.valueDeclaration&&!r.valueDeclaration.type)&&(r||(n.thisParameter=vh(t.thisParameter,void 0)),LD(n.thisParameter,Ws(t.thisParameter)))}for(var i=n.parameters.length-(V(n)?1:0),a=0;a<i;a++){r=n.parameters[a];if(!e.getEffectiveTypeAnnotationNode(r.valueDeclaration)){var o=fD(t,a);LD(r,o)}}if(V(n)){r=e.last(n.parameters);if(e.isTransientSymbol(r)||!e.getEffectiveTypeAnnotationNode(r.valueDeclaration)){o=mD(t,i);LD(r,o)}}}function xD(e){e.thisParameter&&LD(e.thisParameter);for(var n=0,t=e.parameters;n<t.length;n++){var r=t[n];LD(r)}}function LD(e,n){var t=Hr(e);if(!t.type){var r=e.valueDeclaration;t.type=n||Es(r,!0),79!==r.name.kind&&(t.type===Pe&&(t.type=bs(r.name)),DD(r.name))}}function DD(n){for(var t=0,r=n.elements;t<r.length;t++){var i=r[t];e.isOmittedExpression(i)||(79===i.name.kind?Hr(Ca(i)).type=Qo(i):DD(i.name))}}function AD(e){var n=_p(!0);return n!==gn?(e=YC(e)||Pe,Od(n,[e])):Pe}function CD(e){var n=gp(!0);return n!==gn?(e=YC(e)||Pe,Od(n,[e])):Pe}function ID(n,t){var r=AD(t);return r===Pe?(Dr(n,e.isImportCall(n)?e.Diagnostics.A_dynamic_import_call_returns_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option:e.Diagnostics.An_async_function_or_method_must_return_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option),Re):(yp(!0)||Dr(n,e.isImportCall(n)?e.Diagnostics.A_dynamic_import_call_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option:e.Diagnostics.An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option),r)}function ND(n){var t=Or(0,"NewTargetExpression"),r=Or(4,"target",8);r.parent=t,r.type=n;var i=e.createSymbolTable([r]);return t.members=i,Xa(t,i,e.emptyArray,e.emptyArray,e.emptyArray)}function kD(n,t){if(!n.body)return Re;var r,i,a,o=e.getFunctionFlags(n),s=0!==(2&o),c=0!==(1&o),l=Qe;if(233!==n.body.kind)r=kA(n.body,t&&-9&t),s&&(r=qC(r,n,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member));else if(c){var u=BD(n,t);u?u.length>0&&(r=ff(u,2)):l=Ze;var d=OD(n,t),p=d.yieldTypes,f=d.nextTypes;i=e.some(p)?ff(p,2):void 0,a=e.some(f)?Df(f):void 0}else{var m=BD(n,t);if(!m)return 2&o?ID(n,Ze):Ze;if(0===m.length)return 2&o?ID(n,Qe):Qe;r=ff(m,2)}if(r||i||a){if(i&&Rh(n,i,3),r&&Rh(n,r,1),a&&Rh(n,a,2),r&&Fy(r)||i&&Fy(i)||a&&Fy(a)){var _=PT(n),g=_?_===qu(n)?c?void 0:r:yT(ad(_),n):void 0;c?(i=zy(i,g,0,s),r=zy(r,g,1,s),a=zy(a,g,2,s)):r=Wy(r,g,s)}i&&(i=Ch(i)),r&&(r=Ch(r)),a&&(a=Ch(a))}return c?RD(i||Ze,r||l,a||qE(2,n)||Pe,s):s?AD(r||l):r||l}function RD(e,n,t,r){var i=r?Nt:kt,a=i.getGlobalGeneratorType(!1);if(e=i.resolveIterationType(e,void 0)||Pe,n=i.resolveIterationType(n,void 0)||Pe,t=i.resolveIterationType(t,void 0)||Pe,a===gn){var o=i.getGlobalIterableIteratorType(!1),s=o!==gn?LN(o,i):void 0,c=s?s.returnType:Ie,l=s?s.nextType:Me;return lg(n,c)&&lg(l,t)?o!==gn?Op(o,[e]):(i.getGlobalIterableIteratorType(!0),pn):(i.getGlobalGeneratorType(!0),pn)}return Op(a,[e,n,t])}function OD(n,t){var r=[],i=[],a=0!==(2&e.getFunctionFlags(n));return e.forEachYieldExpression(n.body,function(n){var o,s=n.expression?eC(n.expression,t):Fe;if(e.pushIfUnique(r,wD(n,s,Ie,a)),n.asteriskToken){var c=EN(s,a?19:17,n.expression);o=c&&c.nextType}else o=vT(n);o&&e.pushIfUnique(i,o)}),{yieldTypes:r,nextTypes:i}}function wD(n,t,r,i){var a=n.expression||n,o=n.asteriskToken?fN(i?19:17,t,r,a):t;return i?YC(o,a,n.asteriskToken?e.Diagnostics.Type_of_iterated_elements_of_a_yield_Asterisk_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member:e.Diagnostics.Type_of_yield_operand_in_an_async_generator_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member):o}function PD(e,n,t,r){var i=0;if(r){for(var a=n;a<t.length;a++)i|=S.get(t[a])||32768;for(a=e;a<n;a++)i&=~(S.get(t[a])||0);for(a=0;a<e;a++)i|=S.get(t[a])||32768}else{for(a=e;a<n;a++)i|=T.get(t[a])||128;for(a=0;a<e;a++)i&=~(T.get(t[a])||0)}return i}function MD(e){var n=Wr(e);return void 0!==n.isExhaustive?n.isExhaustive:n.isExhaustive=FD(e)}function FD(n){if(214===n.expression.kind){var t=QA(n.expression.expression),r=_b(n,!1),i=PD(0,0,r,!0),a=ru(t)||t;return 3&a.flags?556800===(556800&i):!!(131072&Sb(a,function(e){return(zv(e)&i)===i}).flags)}var o=QA(n.expression);if(!Uy(o))return!1;var s=mb(n);return!(!s.length||e.some(s,My))&&gb(Db(o,Ym),s)}function GD(e){return e.endFlowNode&&Yb(e.endFlowNode)}function BD(n,t){var r=e.getFunctionFlags(n),i=[],a=GD(n),o=!1;if(e.forEachReturnStatement(n.body,function(s){var c=s.expression;if(c){var l=kA(c,t&&-9&t);2&r&&(l=qC(l,n,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member)),131072&l.flags&&(o=!0),e.pushIfUnique(i,l)}else a=!0}),0!==i.length||a||!o&&!UD(n))return!(z&&i.length&&a)||FL(n)&&i.some(function(e){return e.symbol===n.symbol})||e.pushIfUnique(i,Me),i}function UD(e){switch(e.kind){case 211:case 212:return!0;case 167:return 203===e.parent.kind;default:return!1}}function VD(n,r){if(t){var i=e.getFunctionFlags(n),a=r&&jN(r,i);if((!a||!iA(a,16385))&&166!==n.kind&&!e.nodeIsMissing(n.body)&&233===n.body.kind&&GD(n)){var o=512&n.flags,s=e.getEffectiveReturnTypeNode(n)||n;if(a&&131072&a.flags)Dr(s,e.Diagnostics.A_function_returning_never_cannot_have_a_reachable_end_point);else if(a&&!o)Dr(s,e.Diagnostics.A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value);else if(a&&z&&!lg(Me,a))Dr(s,e.Diagnostics.Function_lacks_ending_return_statement_and_return_type_does_not_include_undefined);else if(M.noImplicitReturns){if(!a){if(!o)return;var c=ad(qu(n));if(KN(n,c))return}Dr(s,e.Diagnostics.Not_all_code_paths_return_a_value)}}}}function jD(n,t){if(e.Debug.assert(167!==n.kind||e.isObjectLiteralMethod(n)),rR(n),e.isFunctionExpression(n)&&HI(n,n.name),t&&4&t&&$_(n)){if(!e.getEffectiveReturnTypeNode(n)&&!e.hasContextSensitiveParameters(n)){var r=MT(n);if(r&&Hh(ad(r))){var i=Wr(n);if(i.contextFreeType)return i.contextFreeType;var a=kD(n,t),o=$c(void 0,void 0,void 0,e.emptyArray,a,void 0,0,0),s=Xa(n.symbol,D,[o],e.emptyArray,e.emptyArray);return s.objectFlags|=524288,i.contextFreeType=s}}return yn}var c=HO(n);return c||211!==n.kind||aw(n),KD(n,t),Ws(Ca(n))}function KD(n,t){var r=Wr(n);if(!(1024&r.flags)){var i=MT(n);if(!(1024&r.flags)){r.flags|=1024;var a=e.firstOrUndefined(Au(Ws(Ca(n)),0));if(!a)return;if($_(n))if(i){var o=bT(n);t&&2&t&&TD(a,i,o);var s=o?k_(i,o.mapper):i;SD(a,s)}else xD(a);if(i&&!od(n)&&!a.resolvedReturnType){var c=kD(n,t);a.resolvedReturnType||(a.resolvedReturnType=c)}lC(n)}}}function HD(n){e.Debug.assert(167!==n.kind||e.isObjectLiteralMethod(n));var t=e.getFunctionFlags(n),r=od(n);if(VD(n,r),n.body)if(e.getEffectiveReturnTypeNode(n)||ad(qu(n)),233===n.body.kind)Zk(n.body);else{var i=eC(n.body),a=r&&jN(r,t);if(a)if(2===(3&t)){var o=qC(i,n.body,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member);mg(o,a,n.body,n.body)}else mg(i,a,n.body,n.body)}}function WD(e,n,t,r){if(void 0===r&&(r=!1),!lg(n,cn)){var i=r&&zC(n);return Ir(e,!!i&&lg(i,cn),t),!1}return!0}function zD(n){if(!e.isCallExpression(n))return!1;if(!e.isBindableObjectDefinePropertyCall(n))return!1;var t=kA(n.arguments[2]),r=Bo(t,"value");if(r){var i=Lu(t,"writable"),a=i&&Ws(i);if(!a||a===We||a===ze)return!0;if(i&&i.valueDeclaration&&e.isPropertyAssignment(i.valueDeclaration)){var o=i.valueDeclaration.initializer,s=eC(o);if(s===We||s===ze)return!0}return!1}var c=Lu(t,"set");return!c}function JD(n){return!!(8&e.getCheckFlags(n)||4&n.flags&&64&e.getDeclarationModifierFlagsFromSymbol(n)||3&n.flags&&2&kS(n)||98304&n.flags&&!(65536&n.flags)||8&n.flags||e.some(n.declarations,zD))}function qD(n,t,r){var i,a;if(0===r)return!1;if(JD(t)){if(4&t.flags&&e.isAccessExpression(n)&&108===n.expression.kind){var o=e.getContainingFunction(n);if(!o||169!==o.kind&&!FL(o))return!0;if(t.valueDeclaration){var s=e.isBinaryExpression(t.valueDeclaration),c=o.parent===t.valueDeclaration.parent,l=o===t.valueDeclaration.parent,u=s&&(null===(i=t.parent)||void 0===i?void 0:i.valueDeclaration)===o.parent,d=s&&(null===(a=t.parent)||void 0===a?void 0:a.valueDeclaration)===o,p=c||l||u||d;return!p}}return!0}if(e.isAccessExpression(n)){var f=e.skipParentheses(n.expression);if(79===f.kind){var m=Wr(f).resolvedSymbol;if(2097152&m.flags){var _=hi(m);return!!_&&266===_.kind}}}return!1}function XD(n,t,r){var i=e.skipOuterExpressions(n,7);return 79===i.kind||e.isAccessExpression(i)?!(32&i.flags)||(Dr(n,r),!1):(Dr(n,t),!1)}function YD(n){eC(n.expression);var t=e.skipParentheses(n.expression);if(!e.isAccessExpression(t))return Dr(t,e.Diagnostics.The_operand_of_a_delete_operator_must_be_a_property_reference),Xe;e.isPropertyAccessExpression(t)&&e.isPrivateIdentifier(t.name)&&Dr(t,e.Diagnostics.The_operand_of_a_delete_operator_cannot_be_a_private_identifier);var r=Wr(t),i=Ma(r.resolvedSymbol);return i&&(JD(i)&&Dr(t,e.Diagnostics.The_operand_of_a_delete_operator_cannot_be_a_read_only_property),QD(t,i)),Xe}function QD(n,t){var r=Ws(t);!z||131075&r.flags||(ne?16777216&t.flags:32768&th(r))||Dr(n,e.Diagnostics.The_operand_of_a_delete_operator_must_be_optional)}function ZD(e){return eC(e.expression),fr}function $D(e){return eC(e.expression),Fe}function eA(n){if(t){var r=e.getContainingFunctionOrClassStaticBlock(n);if(r&&e.isClassStaticBlockDeclaration(r))Dr(n,e.Diagnostics.Await_expression_cannot_be_used_inside_a_class_static_block);else if(!(32768&n.flags))if(e.isInTopLevelContext(n)){var i=e.getSourceFileOfNode(n);if(!Rw(i)){var a=void 0;if(!e.isEffectiveExternalModule(i,M)){a||(a=e.getSpanOfTokenAtPosition(i,n.pos));var o=e.createFileDiagnostic(i,a.start,a.length,e.Diagnostics.await_expressions_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_has_no_imports_or_exports_Consider_adding_an_empty_export_to_make_this_file_a_module);ur.add(o)}if(K!==e.ModuleKind.ESNext&&K!==e.ModuleKind.System||F<4){a=e.getSpanOfTokenAtPosition(i,n.pos);o=e.createFileDiagnostic(i,a.start,a.length,e.Diagnostics.Top_level_await_expressions_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_target_option_is_set_to_es2017_or_higher);ur.add(o)}}}else{i=e.getSourceFileOfNode(n);if(!Rw(i)){a=e.getSpanOfTokenAtPosition(i,n.pos),o=e.createFileDiagnostic(i,a.start,a.length,e.Diagnostics.await_expressions_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules);if(r&&169!==r.kind&&0===(2&e.getFunctionFlags(r))){var s=e.createDiagnosticForNode(r,e.Diagnostics.Did_you_mean_to_mark_this_function_as_async);e.addRelatedInfo(o,s)}ur.add(o)}}JE(n)&&Dr(n,e.Diagnostics.await_expressions_cannot_be_used_in_a_parameter_initializer)}var c=eC(n.expression),l=qC(c,n,e.Diagnostics.Type_of_await_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member);return l!==c||l===Re||3&c.flags||Ar(!1,e.createDiagnosticForNode(n,e.Diagnostics.await_has_no_effect_on_the_type_of_this_expression)),l}function nA(n){var t=eC(n.operand);if(t===$e)return $e;switch(n.operand.kind){case 8:switch(n.operator){case 40:return Xm($m(-n.operand.text));case 39:return Xm($m(+n.operand.text))}break;case 9:if(40===n.operator)return Xm(e_({negative:!0,base10Value:e.parsePseudoBigInt(n.operand.text)}))}switch(n.operator){case 39:case 40:case 54:return jS(t,n.operand),iA(t,12288)&&Dr(n.operand,e.Diagnostics.The_0_operator_cannot_be_applied_to_type_symbol,e.tokenToString(n.operator)),39===n.operator?(iA(t,2112)&&Dr(n.operand,e.Diagnostics.Operator_0_cannot_be_applied_to_type_1,e.tokenToString(n.operator),go(Vy(t))),Ke):rA(t);case 53:sN(n.operand);var r=12582912&zv(t);return 4194304===r?We:8388608===r?Je:Xe;case 45:case 46:var i=WD(n.operand,jS(t,n.operand),e.Diagnostics.An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type);return i&&XD(n.operand,e.Diagnostics.The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access,e.Diagnostics.The_operand_of_an_increment_or_decrement_operator_may_not_be_an_optional_property_access),rA(t)}return Re}function tA(n){var t=eC(n.operand);if(t===$e)return $e;var r=WD(n.operand,jS(t,n.operand),e.Diagnostics.An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type);return r&&XD(n.operand,e.Diagnostics.The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access,e.Diagnostics.The_operand_of_an_increment_or_decrement_operator_may_not_be_an_optional_property_access),rA(t)}function rA(e){return iA(e,2112)?aA(e,3)||iA(e,296)?cn:He:Ke}function iA(e,n){if(e.flags&n)return!0;if(3145728&e.flags)for(var t=e.types,r=0,i=t;r<i.length;r++){var a=i[r];if(iA(a,n))return!0}return!1}function aA(e,n,t){return!!(e.flags&n)||!(t&&114691&e.flags)&&(!!(296&n)&&lg(e,Ke)||!!(2112&n)&&lg(e,He)||!!(402653316&n)&&lg(e,je)||!!(528&n)&&lg(e,Xe)||!!(16384&n)&&lg(e,Qe)||!!(131072&n)&&lg(e,Ze)||!!(65536&n)&&lg(e,Ue)||!!(32768&n)&&lg(e,Me)||!!(4096&n)&&lg(e,Ye)||!!(67108864&n)&&lg(e,rn))}function oA(n,t,r){return 1048576&n.flags?e.every(n.types,function(e){return oA(e,t,r)}):aA(n,t,r)}function sA(n){return!!(16&e.getObjectFlags(n))&&!!n.symbol&&cA(n.symbol)}function cA(e){return 0!==(128&e.flags)}function lA(n,t,r,i){return r===$e||i===$e?$e:(!Vo(r)&&oA(r,131068)&&Dr(n,e.Diagnostics.The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_parameter),Vo(i)||BR(i)||cg(i,Cn)||Dr(t,e.Diagnostics.The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_Function_interface_type),Xe)}function uA(n,t,r,i){if(r===$e||i===$e)return $e;r=jS(r,n),i=jS(i,t),oA(r,402665900)||aA(r,407109632)||Dr(n,e.Diagnostics.The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol);var a=Jl(i);return(!oA(i,126091264)||a&&(aA(i,3145728)&&!oA(a,126091264)||!iA(a,126615552)))&&Dr(t,e.Diagnostics.The_right_hand_side_of_an_in_expression_must_not_be_a_primitive),Xe}function dA(e,n,t){var r=e.properties;if(z&&0===r.length)return jS(n,e);for(var i=0;i<r.length;i++)pA(e,n,i,r,t);return n}function pA(n,t,r,i,a){void 0===a&&(a=!1);var o=n.properties,s=o[r];if(291===s.kind||292===s.kind){var c=s.name,l=Ff(c);if(Fc(l)){var u=Kc(l),d=Lu(t,u);d&&(bx(d,s,a),OS(s,!1,!0,t,d))}var p=vm(t,l,32,c),f=zo(s,p);return _A(292===s.kind?s:s.initializer,f)}if(293===s.kind){if(!(r<o.length-1)){F<99&&IO(s,4);var m=[];if(i)for(var _=0,g=i;_<g.length;_++){var y=g[_];e.isSpreadAssignment(y)||m.push(y.name)}f=Ko(t,m,t.symbol);return BO(i,e.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma),_A(s.expression,f)}Dr(s,e.Diagnostics.A_rest_element_must_be_last_in_a_destructuring_pattern)}else Dr(s,e.Diagnostics.Property_assignment_expected)}function fA(e,n,t){var r=e.elements;F<2&&M.downlevelIteration&&IO(e,512);for(var i=fN(193,n,Me,e)||Re,a=M.noUncheckedIndexedAccess?void 0:i,o=0;o<r.length;o++){var s=i;223===e.elements[o].kind&&(s=a=null!==a&&void 0!==a?a:fN(65,n,Me,e)||Re),mA(e,n,o,s,t)}return n}function mA(n,t,r,i,a){var o=n.elements,s=o[r];if(225!==s.kind){if(223!==s.kind){var c=$m(r);if(Iy(t)){var l=32|(BT(s)?16:0),u=Em(t,c,l,nL(s,c))||Re,d=BT(s)?Jv(u,524288):u,p=zo(s,d);return _A(s,p,a)}return _A(s,i,a)}if(r<o.length-1)Dr(s,e.Diagnostics.A_rest_element_must_be_last_in_a_destructuring_pattern);else{var f=s.expression;if(219!==f.kind||63!==f.operatorToken.kind){BO(n.elements,e.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma);p=Eb(t,Jy)?Db(t,function(e){return Yp(e,r)}):Mp(i);return _A(f,p,a)}Dr(f.operatorToken,e.Diagnostics.A_rest_element_cannot_have_an_initializer)}}}function _A(e,n,t,r){var i;if(292===e.kind){var a=e;a.objectAssignmentInitializer&&(!z||32768&th(eC(a.objectAssignmentInitializer))||(n=Jv(n,524288)),EA(a.name,a.equalsToken,a.objectAssignmentInitializer,t)),i=e.name}else i=e;return 219===i.kind&&63===i.operatorToken.kind&&(te(i,t),i=i.left),203===i.kind?dA(i,n,r):202===i.kind?fA(i,n,t):gA(i,n,t)}function gA(n,t,r){var i=eC(n,r),a=293===n.parent.kind?e.Diagnostics.The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access:e.Diagnostics.The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access,o=293===n.parent.kind?e.Diagnostics.The_target_of_an_object_rest_assignment_may_not_be_an_optional_property_access:e.Diagnostics.The_left_hand_side_of_an_assignment_expression_may_not_be_an_optional_property_access;return XD(n,a,o)&&mg(t,i,n,n),e.isPrivateIdentifierPropertyAccessExpression(n)&&IO(n.parent,1048576),t}function yA(n){switch(n=e.skipParentheses(n),n.kind){case 79:case 10:case 13:case 208:case 221:case 14:case 8:case 9:case 110:case 95:case 104:case 151:case 211:case 224:case 212:case 202:case 203:case 214:case 228:case 277:case 276:return!0;case 220:return yA(n.whenTrue)&&yA(n.whenFalse);case 219:return!e.isAssignmentOperator(n.operatorToken.kind)&&(yA(n.left)&&yA(n.right));case 217:case 218:switch(n.operator){case 53:case 39:case 40:case 54:return!0}return!1;case 215:case 209:case 227:default:return!1}}function hA(e,n){return 0!==(98304&n.flags)||dg(e,n)}function vA(){var n=e.createBinaryExpressionTrampoline(t,r,i,a,o,s);return function(t,r){var i=n(t,r);return e.Debug.assertIsDefined(i),i};function t(n,t,r){if(t?(t.stackIndex++,t.skip=!1,u(t,void 0),p(t,void 0)):t={checkMode:r,skip:!1,stackIndex:0,typeStack:[void 0,void 0]},e.isInJSFile(n)&&e.getAssignedExpandoInitializer(n))return t.skip=!0,p(t,eC(n.right,r)),t;bA(n);var i=n.operatorToken.kind;return 63!==i||203!==n.left.kind&&202!==n.left.kind?t:(t.skip=!0,p(t,_A(n.left,eC(n.right,r),r,108===n.right.kind)),t)}function r(e,n,t){if(!n.skip)return c(n,e)}function i(n,t,r){if(!t.skip){var i=d(t);e.Debug.assertIsDefined(i),u(t,i),p(t,void 0);var a=n.kind;if(55===a||56===a||60===a){if(55===a){var o=e.walkUpParenthesizedExpressions(r.parent);nN(r.left,i,e.isIfStatement(o)?o.thenStatement:void 0)}oN(i,r.left)}}}function a(e,n,t){if(!n.skip)return c(n,e)}function o(n,t){var r;if(t.skip)r=d(t);else{var i=l(t);e.Debug.assertIsDefined(i);var a=d(t);e.Debug.assertIsDefined(a),r=TA(n.left,n.operatorToken,n.right,i,a,n)}return t.skip=!1,u(t,void 0),p(t,void 0),t.stackIndex--,r}function s(e,n,t){return p(e,n),e}function c(n,t){if(e.isBinaryExpression(t))return t;p(n,eC(t,n.checkMode))}function l(e){return e.typeStack[e.stackIndex]}function u(e,n){e.typeStack[e.stackIndex]=n}function d(e){return e.typeStack[e.stackIndex+1]}function p(e,n){e.typeStack[e.stackIndex+1]=n}}function bA(n){var t=n.left,r=n.operatorToken,i=n.right;60===r.kind&&(!e.isBinaryExpression(t)||56!==t.operatorToken.kind&&55!==t.operatorToken.kind||Mw(t,e.Diagnostics._0_and_1_operations_cannot_be_mixed_without_parentheses,e.tokenToString(t.operatorToken.kind),e.tokenToString(r.kind)),!e.isBinaryExpression(i)||56!==i.operatorToken.kind&&55!==i.operatorToken.kind||Mw(i,e.Diagnostics._0_and_1_operations_cannot_be_mixed_without_parentheses,e.tokenToString(i.operatorToken.kind),e.tokenToString(r.kind)))}function EA(e,n,t,r,i){var a,o=n.kind;if(63===o&&(203===e.kind||202===e.kind))return _A(e,eC(t,r),r,108===t.kind);a=55===o||56===o||60===o?sN(e,r):eC(e,r);var s=eC(t,r);return TA(e,n,t,a,s,i)}function TA(n,r,i,a,o,s){var c=r.kind;switch(c){case 41:case 42:case 66:case 67:case 43:case 68:case 44:case 69:case 40:case 65:case 47:case 70:case 48:case 71:case 49:case 72:case 51:case 74:case 52:case 78:case 50:case 73:if(a===$e||o===$e)return $e;a=jS(a,n),o=jS(o,i);var l=void 0;if(528&a.flags&&528&o.flags&&void 0!==(l=A(r.kind)))return Dr(s||r,e.Diagnostics.The_0_operator_is_not_allowed_for_boolean_types_Consider_using_1_instead,e.tokenToString(r.kind),e.tokenToString(l)),Ke;var u,d=WD(n,a,e.Diagnostics.The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type,!0),p=WD(i,o,e.Diagnostics.The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type,!0);if(aA(a,3)&&aA(o,3)||!iA(a,2112)&&!iA(o,2112))u=Ke;else if(S(a,o)){switch(c){case 49:case 72:k();break;case 42:case 67:F<3&&Dr(s,e.Diagnostics.Exponentiation_cannot_be_performed_on_bigint_values_unless_the_target_option_is_set_to_es2016_or_later)}u=He}else k(S),u=Re;return d&&p&&C(u),u;case 39:case 64:if(a===$e||o===$e)return $e;aA(a,402653316)||aA(o,402653316)||(a=jS(a,n),o=jS(o,i));var f=void 0;if(aA(a,296,!0)&&aA(o,296,!0)?f=Ke:aA(a,2112,!0)&&aA(o,2112,!0)?f=He:aA(a,402653316,!0)||aA(o,402653316,!0)?f=je:(Vo(a)||Vo(o))&&(f=a===Re||o===Re?Re:Ie),f&&!D(c))return f;if(!f){var m=402655727;return k(function(e,n){return aA(e,m)&&aA(n,m)}),Ie}return 64===c&&C(f),f;case 29:case 31:case 32:case 33:return D(c)&&(a=Vy(jS(a,n)),o=Vy(jS(o,i)),N(function(e,n){return dg(e,n)||dg(n,e)||lg(e,cn)&&lg(n,cn)})),Xe;case 34:case 35:case 36:case 37:return N(function(e,n){return hA(e,n)||hA(n,e)}),Xe;case 102:return lA(n,i,a,o);case 101:return uA(n,i,a,o);case 55:case 76:var _=4194304&zv(a)?ff([ih(z?a:Vy(o)),o]):a;return 76===c&&C(o),_;case 56:case 75:var g=8388608&zv(a)?ff([rh(a),o],2):a;return 75===c&&C(o),g;case 60:case 77:var y=262144&zv(a)?ff([lh(a),o],2):a;return 77===c&&C(o),y;case 63:var h=e.isBinaryExpression(n.parent)?e.getAssignmentDeclarationKind(n.parent):0;return x(h,o),I(h)?(524288&o.flags&&(2===h||6===h||Gg(o)||Wv(o)||1&e.getObjectFlags(o))||C(o),a):(C(o),Eh(o));case 27:if(!M.allowUnreachableCode&&yA(n)&&!L(i)){var v=e.getSourceFileOfNode(n),b=v.text,E=e.skipTrivia(b,n.pos),T=v.parseDiagnostics.some(function(n){return n.code===e.Diagnostics.JSX_expressions_must_have_one_parent_element.code&&e.textSpanContainsPosition(n,E)});T||Dr(n,e.Diagnostics.Left_side_of_comma_operator_is_unused_and_has_no_side_effects)}return o;default:return e.Debug.fail()}function S(e,n){return aA(e,2112)&&aA(n,2112)}function x(n,t){if(2===n)for(var r=0,i=Vl(t);r<i.length;r++){var a=i[r],o=Ws(a);if(o.symbol&&32&o.symbol.flags){var s=a.escapedName,c=Qr(a.valueDeclaration,s,788968,void 0,s,!1);(null===c||void 0===c?void 0:c.declarations)&&c.declarations.some(e.isJSDocTypedefTag)&&(Gr(c,e.Diagnostics.Duplicate_identifier_0,e.unescapeLeadingUnderscores(s),a),Gr(a,e.Diagnostics.Duplicate_identifier_0,e.unescapeLeadingUnderscores(s),c))}}}function L(e){return 79===e.kind&&"eval"===e.escapedText}function D(t){var r=iA(a,12288)?n:iA(o,12288)?i:void 0;return!r||(Dr(r,e.Diagnostics.The_0_operator_cannot_be_applied_to_type_symbol,e.tokenToString(t)),!1)}function A(e){switch(e){case 51:case 74:return 56;case 52:case 78:return 37;case 50:case 73:return 55;default:return}}function C(r){t&&e.isAssignmentOperator(c)&&(!XD(n,e.Diagnostics.The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access,e.Diagnostics.The_left_hand_side_of_an_assignment_expression_may_not_be_an_optional_property_access)||e.isIdentifier(n)&&"exports"===e.unescapeLeadingUnderscores(n.escapedText)||mg(r,a,n,i))}function I(t){var r;switch(t){case 2:return!0;case 1:case 5:case 6:case 3:case 4:var a=Ca(n),o=e.getAssignedExpandoInitializer(i);return!!o&&e.isObjectLiteralExpression(o)&&!!(null===(r=null===a||void 0===a?void 0:a.exports)||void 0===r?void 0:r.size);default:return!1}}function N(e){return!e(a,o)&&(k(e),!0)}function k(n){var t,i=!1,c=s||r;if(n){var l=YC(a),u=YC(o);i=!(l===a&&u===o)&&!(!l||!u)&&n(l,u)}var d=a,p=o;!i&&n&&(t=SA(a,o,n),d=t[0],p=t[1]);var f=yo(d,p),m=f[0],_=f[1];R(c,i,m,_)||Ir(c,i,e.Diagnostics.Operator_0_cannot_be_applied_to_types_1_and_2,e.tokenToString(r.kind),m,_)}function R(n,t,i,a){var o;switch(r.kind){case 36:case 34:o="false";break;case 37:case 35:o="true"}if(o)return Ir(n,t,e.Diagnostics.This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap,o,i,a)}}function SA(e,n,t){var r=e,i=n,a=Vy(e),o=Vy(n);return t(a,o)||(r=a,i=o),[r,i]}function xA(n){t&&(8192&n.flags||Ow(n,e.Diagnostics.A_yield_expression_is_only_allowed_in_a_generator_body),JE(n)&&Dr(n,e.Diagnostics.yield_expressions_cannot_be_used_in_a_parameter_initializer));var r=e.getContainingFunction(n);if(!r)return Ie;var i=e.getFunctionFlags(r);if(!(1&i))return Ie;var a=0!==(2&i);n.asteriskToken&&(a&&F<99&&IO(n,26624),!a&&F<2&&M.downlevelIteration&&IO(n,256));var o=od(r),s=o&&UN(o,a),c=s&&s.yieldType||Ie,l=s&&s.nextType||Ie,u=a?YC(l)||Ie:l,d=n.expression?eC(n.expression):Fe,p=wD(n,d,u,a);if(o&&p&&mg(p,c,n.expression||n,n.expression),n.asteriskToken){var f=a?19:17;return gN(f,1,d,n.expression)||Ie}if(o)return BN(2,o,a)||Ie;var m=qE(2,r);if(!m&&(m=Ie,t&&Y&&!e.expressionResultIsUnused(n))){var _=vT(n);_&&!Vo(_)||Dr(n,e.Diagnostics.yield_expression_implicitly_results_in_an_any_type_because_its_containing_generator_lacks_a_return_type_annotation)}return m}function LA(e,n){var t=sN(e.condition);nN(e.condition,t,e.whenTrue);var r=eC(e.whenTrue,n),i=eC(e.whenFalse,n);return ff([r,i],2)}function DA(n){var t=n.parent;return e.isParenthesizedExpression(t)&&DA(t)||e.isElementAccessExpression(t)&&t.argumentExpression===n}function AA(n){for(var t=[n.head.text],r=[],i=0,a=n.templateSpans;i<a.length;i++){var o=a[i],s=eC(o.expression);iA(s,12288)&&Dr(o.expression,e.Diagnostics.Implicit_conversion_of_a_symbol_to_a_string_will_fail_at_runtime_Consider_wrapping_this_expression_in_String),t.push(o.literal.text),r.push(lg(s,ln)?s:je)}return FA(n)||DA(n)||bb(vT(n)||Pe,CA)?Wf(t,r):je}function CA(e){return!!(134217856&e.flags||58982400&e.flags&&iA(ru(e)||Pe,402653316))}function IA(n){return 284!==n.kind||e.isJsxSelfClosingElement(n.parent)?n:n.parent.parent}function NA(e,n,t,r){var i=IA(e),a=i.contextualType,o=i.inferenceContext;try{i.contextualType=n,i.inferenceContext=t;var s=eC(e,1|r|(t?2:0)),c=iA(s,2944)&&MA(s,yT(n,e))?Ym(s):s;return c}finally{i.contextualType=a,i.inferenceContext=o}}function kA(e,n){var t=Wr(e);if(!t.resolvedType){if(n&&0!==n)return eC(e,n);var r=Pt,i=mt;Pt=Mt,mt=void 0,t.resolvedType=eC(e,n),mt=i,Pt=r}return t.resolvedType}function RA(n){return n=e.skipParentheses(n),209===n.kind||227===n.kind}function OA(n,t){var r=e.getEffectiveInitializer(n),i=ZA(r)||(t?NA(r,t,void 0,0):kA(r));return e.isParameter(n)&&200===n.name.kind&&Jy(i)&&!i.target.hasRestElement&&Fd(i)<n.name.elements.length?wA(i,n.name):i}function wA(n,t){for(var r=t.elements,i=Md(n).slice(),a=n.target.elementFlags.slice(),o=Fd(n);o<r.length;o++){var s=r[o];(o<r.length-1||201!==s.kind||!s.dotDotDotToken)&&(i.push(!e.isOmittedExpression(s)&&BT(s)?ys(s,!1,!1):Ie),a.push(2),e.isOmittedExpression(s)||BT(s)||kh(s,Ie))}return Wp(i,a,n.target.readonly)}function PA(n,t){var r=2&e.getCombinedNodeFlags(n)||e.isDeclarationReadonly(n)?t:jy(t);if(e.isInJSFile(n)){if(ky(r))return kh(n,Ie),Ie;if(Ry(r))return kh(n,Gn),Gn}return r}function MA(n,t){if(t){if(3145728&t.flags){var r=t.types;return e.some(r,function(e){return MA(n,e)})}if(58982400&t.flags){var i=ru(t)||Pe;return iA(i,4)&&iA(n,128)||iA(i,8)&&iA(n,256)||iA(i,64)&&iA(n,2048)||iA(i,4096)&&iA(n,8192)||MA(n,i)}return!!(406847616&t.flags&&iA(n,128)||256&t.flags&&iA(n,256)||2048&t.flags&&iA(n,2048)||512&t.flags&&iA(n,512)||8192&t.flags&&iA(n,8192))}return!1}function FA(n){var t=n.parent;return e.isAssertionExpression(t)&&e.isConstTypeReference(t.type)||(e.isParenthesizedExpression(t)||e.isArrayLiteralExpression(t)||e.isSpreadElement(t))&&FA(t)||(e.isPropertyAssignment(t)||e.isShorthandPropertyAssignment(t)||e.isTemplateSpan(t))&&FA(t.parent)}function GA(e,n,t,r){var i=eC(e,n,r);return FA(e)?Ym(i):RA(e)?i:Hy(i,yT(2===arguments.length?vT(e):t,e))}function BA(e,n){return 160===e.name.kind&&WT(e.name),GA(e.initializer,n)}function UA(e,n){hw(e),160===e.name.kind&&WT(e.name);var t=jD(e,n);return VA(e,t,n)}function VA(n,t,r){if(r&&10&r){var i=Kx(t,0,!0),a=Kx(t,1,!0),o=i||a;if(o&&o.typeParameters){var s=gT(n,2);if(s){var c=Kx(lh(s),i?0:1,!1);if(c&&!c.typeParameters){if(8&r)return jA(n,r),yn;var l=bT(n),u=l.signature&&ad(l.signature),d=u&&jx(u);if(d&&!d.typeParameters&&!e.every(l.inferences,KA)){var p=zA(l,o.typeParameters),f=dd(o,p),m=e.map(l.inferences,function(e){return Uh(e.typeParameter)});if(Oh(f,c,function(e,n){dv(m,e,n,0,!0)}),e.some(m,KA)&&(wh(f,c,function(e,n){dv(m,e,n)}),!HA(l.inferences,m)))return WA(l.inferences,m),l.inferredTypeParameters=e.concatenate(l.inferredTypeParameters,p),vd(f)}return vd(Hx(o,c,l))}}}}return t}function jA(e,n){if(2&n){var t=bT(e);t.flags|=4}}function KA(e){return!(!e.candidates&&!e.contraCandidates)}function HA(e,n){for(var t=0;t<e.length;t++)if(KA(e[t])&&KA(n[t]))return!0;return!1}function WA(e,n){for(var t=0;t<e.length;t++)!KA(e[t])&&KA(n[t])&&(e[t]=n[t])}function zA(n,t){for(var r,i,a=[],o=0,s=t;o<s.length;o++){var c=s[o],l=c.symbol.escapedName;if(JA(n.inferredTypeParameters,l)||JA(a,l)){var u=qA(e.concatenate(n.inferredTypeParameters,a),l),d=Or(262144,u),p=Ha(d);p.target=c,r=e.append(r,c),i=e.append(i,p),a.push(p)}else a.push(c)}if(i)for(var f=g_(r,i),m=0,_=i;m<_.length;m++){c=_[m];c.mapper=f}return a}function JA(n,t){return e.some(n,function(e){return e.symbol.escapedName===t})}function qA(e,n){var t=n.length;while(t>1&&n.charCodeAt(t-1)>=48&&n.charCodeAt(t-1)<=57)t--;for(var r=n.slice(0,t),i=1;1;i++){var a=r+i;if(!JA(e,a))return a}}function XA(e){var n=Vx(e);if(n&&!n.typeParameters)return ad(n)}function YA(e){var n=eC(e.expression),t=fh(n,e.expression),r=XA(n);return r&&ph(r,e,t!==n)}function QA(n){var t=ZA(n);if(t)return t;if(67108864&n.flags&&mt){var r=mt[O(n)];if(r)return r}var i=Bt,a=eC(n);if(Bt!==i){var o=mt||(mt=[]);o[O(n)]=a,e.setNodeFlags(n,67108864|n.flags)}return a}function ZA(n){var t=e.skipParentheses(n);if(!e.isCallExpression(t)||106===t.expression.kind||e.isRequireCall(t,!0)||WL(t)){if(e.isAssertionExpression(t)&&!e.isConstTypeReference(t.type))return u_(t.type);if(8===n.kind||10===n.kind||110===n.kind||95===n.kind)return eC(n)}else{var r=e.isCallChain(t)?YA(t):XA(MS(t.expression));if(r)return r}}function $A(e){var n=Wr(e);if(n.contextFreeType)return n.contextFreeType;var t=e.contextualType;e.contextualType=Ie;try{var r=n.contextFreeType=eC(e,4);return r}finally{e.contextualType=t}}function eC(n,t,r){null===e.tracing||void 0===e.tracing||e.tracing.push("check","checkExpression",{kind:n.kind,pos:n.pos,end:n.end});var i=d;d=n,x=0;var a=rC(n,t,r),o=VA(n,a,t);return sA(o)&&nC(n,o),d=i,null===e.tracing||void 0===e.tracing||e.tracing.pop(),o}function nC(n,t){var r=204===n.parent.kind&&n.parent.expression===n||205===n.parent.kind&&n.parent.expression===n||(79===n.kind||159===n.kind)&&SR(n)||179===n.parent.kind&&n.parent.exprName===n||273===n.parent.kind;if(r||Dr(n,e.Diagnostics.const_enums_can_only_be_used_in_property_or_index_access_expressions_or_the_right_hand_side_of_an_import_declaration_or_export_assignment_or_type_query),M.isolatedModules){e.Debug.assert(!!(128&t.symbol.flags));var i=t.symbol.valueDeclaration;8388608&i.flags&&Dr(n,e.Diagnostics.Cannot_access_ambient_const_enums_when_the_isolatedModules_flag_is_provided)}}function tC(n,t){var r=e.isInJSFile(n)?e.getJSDocTypeTag(n):void 0;return r?ZL(r.typeExpression.type,r.typeExpression.type,n.expression,t):eC(n.expression,t)}function rC(n,t,i){var a=n.kind;if(r)switch(a){case 224:case 211:case 212:r.throwIfCancellationRequested()}switch(a){case 79:return gE(n,t);case 108:return CE(n);case 106:return wE(n);case 104:return Ve;case 14:case 10:return Xm(Zm(n.text));case 8:return Hw(n),Xm($m(+n.text));case 9:return zw(n),Xm(e_({negative:!1,base10Value:e.parsePseudoBigInt(n.text)}));case 110:return Je;case 95:return We;case 221:return AA(n);case 13:return Mn;case 202:return UT(n,t,i);case 203:return YT(n,t);case 204:return HS(n,t);case 159:return zS(n,t);case 205:return Cx(n,t);case 206:if(100===n.expression.kind)return zL(n);case 207:return jL(n,t);case 208:return XL(n);case 210:return tC(n,t);case 224:return ak(n);case 211:case 212:return jD(n,t);case 214:return ZD(n);case 209:case 227:return YL(n);case 228:return eD(n);case 229:return nD(n);case 213:return YD(n);case 215:return $D(n);case 216:return eA(n);case 217:return nA(n);case 218:return tA(n);case 219:return te(n,t);case 220:return LA(n,t);case 223:return FT(n,t);case 225:return Fe;case 222:return xA(n);case 230:return GT(n);case 286:return NS(n,t);case 276:return nS(n,t);case 277:return $T(n,t);case 280:return tS(n);case 284:return lS(n,t);case 278:e.Debug.fail("Shouldn't ever directly check a JsxOpeningElement")}return Re}function iC(n){n.expression&&Ow(n.expression,e.Diagnostics.Type_expected),Zk(n.constraint),Zk(n.default);var r=xc(Ca(n));ru(r),uu(r)||Dr(n.default,e.Diagnostics.Type_parameter_0_has_a_circular_default,go(r));var i=ql(r),a=lu(r);i&&a&&fg(a,Xc(z_(i,h_(r,a)),a),n.default,e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1),t&&$N(n.name,e.Diagnostics.Type_parameter_name_cannot_be_0)}function aC(n){RO(n),JI(n);var t=e.getContainingFunction(n);e.hasSyntacticModifier(n,16476)&&(169===t.kind&&e.nodeIsPresent(t.body)||Dr(n,e.Diagnostics.A_parameter_property_is_only_allowed_in_a_constructor_implementation),169===t.kind&&e.isIdentifier(n.name)&&"constructor"===n.name.escapedText&&Dr(n.name,e.Diagnostics.constructor_cannot_be_used_as_a_parameter_property_name)),n.questionToken&&e.isBindingPattern(n.name)&&t.body&&Dr(n,e.Diagnostics.A_binding_pattern_parameter_cannot_be_optional_in_an_implementation_signature),n.name&&e.isIdentifier(n.name)&&("this"===n.name.escapedText||"new"===n.name.escapedText)&&(0!==t.parameters.indexOf(n)&&Dr(n,e.Diagnostics.A_0_parameter_must_be_the_first_parameter,n.name.escapedText),169!==t.kind&&173!==t.kind&&178!==t.kind||Dr(n,e.Diagnostics.A_constructor_cannot_have_a_this_parameter),212===t.kind&&Dr(n,e.Diagnostics.An_arrow_function_cannot_have_a_this_parameter),170!==t.kind&&171!==t.kind||Dr(n,e.Diagnostics.get_and_set_accessors_cannot_declare_this_parameters)),!n.dotDotDotToken||e.isBindingPattern(n.name)||lg(vu(Ws(n.symbol)),Un)||Dr(n,e.Diagnostics.A_rest_parameter_must_be_of_an_array_type)}function oC(n){var t=sC(n);if(t){var r=qu(t),i=td(r);if(i){Zk(n.type);var a=n.parameterName;if(0===i.kind||2===i.kind)o_(a);else if(i.parameterIndex>=0){if(V(r)&&i.parameterIndex===r.parameters.length-1)Dr(a,e.Diagnostics.A_type_predicate_cannot_reference_a_rest_parameter);else if(i.type){var o=function(){return e.chainDiagnosticMessages(void 0,e.Diagnostics.A_type_predicate_s_type_must_be_assignable_to_its_parameter_s_type)};fg(i.type,Ws(r.parameters[i.parameterIndex]),n.type,void 0,o)}}else if(a){for(var s=!1,c=0,l=t.parameters;c<l.length;c++){var u=l[c].name;if(e.isBindingPattern(u)&&cC(u,a,i.parameterName)){s=!0;break}}s||Dr(n.parameterName,e.Diagnostics.Cannot_find_parameter_0,i.parameterName)}}}else Dr(n,e.Diagnostics.A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods)}function sC(e){switch(e.parent.kind){case 212:case 172:case 254:case 211:case 177:case 167:case 166:var n=e.parent;if(e===n.type)return n}}function cC(n,t,r){for(var i=0,a=n.elements;i<a.length;i++){var o=a[i];if(!e.isOmittedExpression(o)){var s=o.name;if(79===s.kind&&s.escapedText===r)return Dr(t,e.Diagnostics.A_type_predicate_cannot_reference_element_0_in_a_binding_pattern,r),!0;if((200===s.kind||199===s.kind)&&cC(s,t,r))return!0}}}function lC(n){174===n.kind?qO(n):177!==n.kind&&254!==n.kind&&178!==n.kind&&172!==n.kind&&169!==n.kind&&173!==n.kind||HO(n);var r=e.getFunctionFlags(n);if(4&r||(3===(3&r)&&F<99&&IO(n,6144),2===(3&r)&&F<4&&IO(n,64),0!==(3&r)&&F<2&&IO(n,128)),nk(n.typeParameters),e.forEach(n.parameters,aC),n.type&&Zk(n.type),t){wI(n);var i=e.getEffectiveReturnTypeNode(n);if(Y&&!i)switch(n.kind){case 173:Dr(n,e.Diagnostics.Construct_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type);break;case 172:Dr(n,e.Diagnostics.Call_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type);break}if(i){var a=e.getFunctionFlags(n);if(1===(5&a)){var o=u_(i);if(o===Qe)Dr(i,e.Diagnostics.A_generator_cannot_have_a_void_type_annotation);else{var s=BN(0,o,0!==(2&a))||Ie,c=BN(1,o,0!==(2&a))||s,l=BN(2,o,0!==(2&a))||Pe,u=RD(s,c,l,!!(2&a));fg(u,o,i)}}else 2===(3&a)&&ZC(n,i)}174!==n.kind&&312!==n.kind&&hI(n)}}function uC(n){for(var t=new e.Map,r=new e.Map,i=new e.Map,a=0,o=n.members;a<o.length;a++){var s=o[a];if(169===s.kind)for(var c=0,l=s.parameters;c<l.length;c++){var u=l[c];e.isParameterPropertyDeclaration(u,s)&&!e.isBindingPattern(u.name)&&y(t,u.name,u.name.escapedText,3)}else{var d=e.isStatic(s),p=s.name;if(!p)continue;var f=e.isPrivateIdentifier(p),m=f&&d?16:0,_=f?i:d?r:t,g=p&&e.getPropertyNameForPropertyNameNode(p);if(g)switch(s.kind){case 170:y(_,p,g,1|m);break;case 171:y(_,p,g,2|m);break;case 165:y(_,p,g,3|m);break;case 167:y(_,p,g,8|m);break}}}function y(n,t,r,i){var a=n.get(r);if(a)if((16&a)!==(16&i))Dr(t,e.Diagnostics.Duplicate_identifier_0_Static_and_instance_elements_cannot_share_the_same_private_name,e.getTextOfNode(t));else{var o=!!(8&a),s=!!(8&i);o||s?o!==s&&Dr(t,e.Diagnostics.Duplicate_identifier_0,e.getTextOfNode(t)):a&i&-17?Dr(t,e.Diagnostics.Duplicate_identifier_0,e.getTextOfNode(t)):n.set(r,a|i)}else n.set(r,i)}}function dC(n){for(var t=0,r=n.members;t<r.length;t++){var i=r[t],a=i.name,o=e.isStatic(i);if(o&&a){var s=e.getPropertyNameForPropertyNameNode(a);switch(s){case"name":case"length":case"caller":case"arguments":case"prototype":var c=e.Diagnostics.Static_property_0_conflicts_with_built_in_property_Function_0_of_constructor_function_1,l=No(Ca(n));Dr(a,c,s,l);break}}}}function pC(n){for(var t=new e.Map,r=0,i=n.members;r<i.length;r++){var a=i[r];if(164===a.kind){var o=void 0,s=a.name;switch(s.kind){case 10:case 8:o=s.text;break;case 79:o=e.idText(s);break;default:continue}t.get(o)?(Dr(e.getNameOfDeclaration(a.symbol.valueDeclaration),e.Diagnostics.Duplicate_identifier_0,o),Dr(a.name,e.Diagnostics.Duplicate_identifier_0,o)):t.set(o,!0)}}}function fC(n){if(256===n.kind){var t=Ca(n);if(t.declarations&&t.declarations.length>0&&t.declarations[0]!==n)return}var r=bd(Ca(n));if(null===r||void 0===r?void 0:r.declarations){for(var i=new e.Map,a=function(e){1===e.parameters.length&&e.parameters[0].type&&vb(u_(e.parameters[0].type),function(n){var t=i.get(nf(n));t?t.declarations.push(e):i.set(nf(n),{type:n,declarations:[e]})})},o=0,s=r.declarations;o<s.length;o++){var c=s[o];a(c)}i.forEach(function(n){if(n.declarations.length>1)for(var t=0,r=n.declarations;t<r.length;t++){var i=r[t];Dr(i,e.Diagnostics.Duplicate_index_signature_for_type_0,go(n.type))}})}}function mC(n){RO(n)||Bw(n)||iw(n.name),JI(n),yC(n),e.isPrivateIdentifier(n.name)&&e.hasStaticModifier(n)&&n.initializer&&99===F&&!M.useDefineForClassFields&&Dr(n.initializer,e.Diagnostics.Static_fields_with_private_names_can_t_have_initializers_when_the_useDefineForClassFields_flag_is_not_specified_with_a_target_of_esnext_Consider_adding_the_useDefineForClassFields_flag),e.hasSyntacticModifier(n,128)&&165===n.kind&&n.initializer&&Dr(n,e.Diagnostics.Property_0_cannot_have_an_initializer_because_it_is_marked_abstract,e.declarationNameToString(n.name))}function _C(n){return e.isPrivateIdentifier(n.name)&&Dr(n,e.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies),mC(n)}function gC(n){hw(n)||iw(n.name),yI(n),e.hasSyntacticModifier(n,128)&&167===n.kind&&n.body&&Dr(n,e.Diagnostics.Method_0_cannot_have_an_implementation_because_it_is_marked_abstract,e.declarationNameToString(n.name)),e.isPrivateIdentifier(n.name)&&!e.getContainingClass(n)&&Dr(n,e.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies),yC(n)}function yC(n){if(e.isPrivateIdentifier(n.name)&&F<99){for(var t=e.getEnclosingBlockScopeContainer(n);t;t=e.getEnclosingBlockScopeContainer(t))Wr(t).flags|=67108864;if(e.isClassExpression(n.parent)){var r=vE(n.parent);r&&(Wr(n.name).flags|=524288,Wr(r).flags|=65536)}}}function hC(n){RO(n),e.forEachChild(n,Zk)}function vC(n){lC(n),Fw(n)||Gw(n),Zk(n.body);var r=Ca(n),i=e.getDeclarationOfKind(r,n.kind);if(n===i&&HC(r),!e.nodeIsMissing(n.body)&&t){var a=n.parent;if(e.getClassExtendsHeritageElement(a)){SE(n.parent,a);var o=LE(a),s=xE(n.body);if(s){o&&Dr(s,e.Diagnostics.A_constructor_cannot_contain_a_super_call_when_its_class_extends_null);var c=(99!==M.target||!H)&&(e.some(n.parent.members,m)||e.some(n.parameters,function(n){return e.hasSyntacticModifier(n,16476)}));if(c){for(var l=n.body.statements,u=void 0,d=0,p=l;d<p.length;d++){var f=p[d];if(236===f.kind&&e.isSuperCall(f.expression)){u=f;break}if(!e.isPrologueDirective(f))break}u||Dr(n,e.Diagnostics.A_super_call_must_be_the_first_statement_in_the_constructor_when_a_class_contains_initialized_properties_parameter_properties_or_private_identifiers)}}else o||Dr(n,e.Diagnostics.Constructors_for_derived_classes_must_contain_a_super_call)}}function m(n){return!!e.isPrivateIdentifierClassElementDeclaration(n)||165===n.kind&&!e.isStatic(n)&&!!n.initializer}}function bC(n){if(t){if(HO(n)||fw(n)||iw(n.name),oI(n),lC(n),170===n.kind&&!(8388608&n.flags)&&e.nodeIsPresent(n.body)&&256&n.flags&&(512&n.flags||Dr(n.name,e.Diagnostics.A_get_accessor_must_return_a_value)),160===n.name.kind&&WT(n.name),Vc(n)){var r=Ca(n),i=e.getDeclarationOfKind(r,170),a=e.getDeclarationOfKind(r,171);if(i&&a&&!(1&lO(i))){Wr(i).flags|=1;var o=e.getEffectiveModifierFlags(i),s=e.getEffectiveModifierFlags(a);(128&o)!==(128&s)&&(Dr(i.name,e.Diagnostics.Accessors_must_both_be_abstract_or_non_abstract),Dr(a.name,e.Diagnostics.Accessors_must_both_be_abstract_or_non_abstract)),(16&o&&!(24&s)||8&o&&!(8&s))&&(Dr(i.name,e.Diagnostics.A_get_accessor_must_be_at_least_as_accessible_as_the_setter),Dr(a.name,e.Diagnostics.A_get_accessor_must_be_at_least_as_accessible_as_the_setter));var c=Is(i),l=Is(a);c&&l&&fg(c,l,i,e.Diagnostics.The_return_type_of_a_get_accessor_must_be_assignable_to_its_set_accessor_type)}}var u=Rs(Ca(n));170===n.kind&&VD(n,u)}Zk(n.body),yC(n)}function EC(e){oI(e)}function TC(n,t){return Ju(e.map(n.typeArguments,u_),t,zu(t),e.isInJSFile(n))}function SC(n,t){for(var r,i,a=!0,o=0;o<t.length;o++){var s=ql(t[o]);s&&(r||(r=TC(n,t),i=g_(t,r)),a=a&&fg(r[o],z_(s,i),n.typeArguments[o],e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1))}return a}function xC(n){var t=ep(n);if(t!==Re){var r=Wr(n).resolvedSymbol;if(r)return 524288&r.flags&&Hr(r).typeParameters||(4&e.getObjectFlags(t)?t.target.localTypeParameters:void 0)}}function LC(n){YO(n,n.typeArguments),176!==n.kind||void 0===n.typeName.jsdocDotPos||e.isInJSFile(n)||e.isInJSDoc(n)||ww(n,n.typeName.jsdocDotPos,1,e.Diagnostics.JSDoc_types_can_only_be_used_inside_documentation_comments),e.forEach(n.typeArguments,Zk);var r=ep(n);if(r!==Re){if(n.typeArguments&&t){var i=xC(n);i&&SC(n,i)}var a=Wr(n).resolvedSymbol;a&&(e.some(a.declarations,function(e){return gR(e)&&!!(134217728&e.flags)})&&kr(HL(n),a.declarations,a.escapedName),32&r.flags&&8&a.flags&&Dr(n,e.Diagnostics.Enum_type_0_has_members_with_initializers_that_are_not_literals,go(r)))}}function DC(n){var t=e.tryCast(n.parent,e.isTypeReferenceType);if(t){var r=xC(t),i=ql(r[t.typeArguments.indexOf(n)]);return i&&z_(i,g_(r,TC(t,r)))}}function AC(e){tp(e)}function CC(n){if(e.forEach(n.members,Zk),t){var r=Gm(n);YN(r),fC(n),pC(n)}}function IC(e){Zk(e.elementType)}function NC(n){for(var t=n.elements,r=!1,i=!1,a=e.some(t,e.isNamedTupleMember),o=0,s=t;o<s.length;o++){var c=s[o];if(195!==c.kind&&a){Mw(c,e.Diagnostics.Tuple_members_must_all_have_names_or_all_not_have_names);break}var l=Fp(c);if(8&l){var u=u_(c.type);if(!Iy(u)){Dr(c,e.Diagnostics.A_rest_element_type_must_be_an_array_type);break}(Ly(u)||Jy(u)&&4&u.target.combinedFlags)&&(i=!0)}else if(4&l){if(i){Mw(c,e.Diagnostics.A_rest_element_cannot_follow_another_rest_element);break}i=!0}else if(2&l){if(i){Mw(c,e.Diagnostics.An_optional_element_cannot_follow_a_rest_element);break}r=!0}else if(r){Mw(c,e.Diagnostics.A_required_element_cannot_follow_an_optional_element);break}}e.forEach(n.elements,Zk),u_(n)}function kC(n){e.forEach(n.types,Zk),u_(n)}function RC(n,t){if(!(8388608&n.flags))return n;var r=n.objectType,i=n.indexType;if(lg(i,Uf(r,!1)))return 205===t.kind&&e.isAssignmentTarget(t)&&32&e.getObjectFlags(r)&&1&Pl(r)&&Dr(t,e.Diagnostics.Index_signature_in_type_0_only_permits_reading,go(r)),n;var a=mu(r);if(Ou(a,Ke)&&aA(i,296))return n;if(sm(r)){var o=em(i,t);if(o){var s=vb(a,function(e){return Lu(e,o)});if(s&&24&e.getDeclarationModifierFlagsFromSymbol(s))return Dr(t,e.Diagnostics.Private_or_protected_member_0_cannot_be_accessed_on_a_type_parameter,e.unescapeLeadingUnderscores(o)),Re}}return Dr(t,e.Diagnostics.Type_0_cannot_be_used_to_index_type_1,go(i),go(r)),Re}function OC(e){Zk(e.objectType),Zk(e.indexType),RC(Tm(e),e)}function wC(n){Zk(n.typeParameter),Zk(n.nameType),Zk(n.type),n.type||kh(n,Ie);var t=Sm(n),r=Nl(t);if(r)fg(r,sn,n.nameType);else{var i=Il(t);fg(i,sn,e.getEffectiveConstraintOfTypeParameter(n.typeParameter))}}function PC(e){o_(e)}function MC(e){gw(e),Zk(e.type)}function FC(n){e.forEachChild(n,Zk)}function GC(n){e.findAncestor(n,function(e){return e.parent&&187===e.parent.kind&&e.parent.extendsType===e})||Mw(n,e.Diagnostics.infer_declarations_are_only_permitted_in_the_extends_clause_of_a_conditional_type),Zk(n.typeParameter),hI(n)}function BC(e){for(var n=0,t=e.templateSpans;n<t.length;n++){var r=t[n];Zk(r.type);var i=u_(r.type);fg(i,ln,r.type)}u_(e)}function UC(e){Zk(e.argument),u_(e)}function VC(n){n.dotDotDotToken&&n.questionToken&&Mw(n,e.Diagnostics.A_tuple_member_cannot_be_both_optional_and_rest),183===n.type.kind&&Mw(n.type,e.Diagnostics.A_labeled_tuple_element_is_declared_as_optional_with_a_question_mark_after_the_name_and_before_the_colon_rather_than_after_the_type),184===n.type.kind&&Mw(n.type,e.Diagnostics.A_labeled_tuple_element_is_declared_as_rest_with_a_before_the_name_rather_than_before_the_type),Zk(n.type),u_(n)}function jC(n){return(e.hasEffectiveModifier(n,8)||e.isPrivateIdentifierClassElementDeclaration(n))&&!!(8388608&n.flags)}function KC(n,t){var r=e.getCombinedModifierFlags(n);return 256!==n.parent.kind&&255!==n.parent.kind&&224!==n.parent.kind&&8388608&n.flags&&(2&r||e.isModuleBlock(n.parent)&&e.isModuleDeclaration(n.parent.parent)&&e.isGlobalScopeAugmentation(n.parent.parent)||(r|=1),r|=2),r&t}function HC(n){if(t){var r,i,o,s=155,c=0,l=s,u=!1,d=!0,p=!1,f=n.declarations,m=0!==(16384&n.flags),_=!1,g=!1,y=!1,h=[];if(f)for(var v=0,b=f;v<b.length;v++){var E=b[v],T=E,S=8388608&T.flags,x=T.parent&&(256===T.parent.kind||180===T.parent.kind)||S;if(x&&(o=void 0),255!==T.kind&&224!==T.kind||S||(y=!0),254===T.kind||167===T.kind||166===T.kind||169===T.kind){h.push(T);var L=KC(T,s);c|=L,l&=L,u=u||e.hasQuestionToken(T),d=d&&e.hasQuestionToken(T);var D=e.nodeIsPresent(T.body);D&&r?m?g=!0:_=!0:(null===o||void 0===o?void 0:o.parent)===T.parent&&o.end!==T.pos&&M(o),D?r||(r=T):p=!0,o=T,x||(i=T)}}if(g&&e.forEach(h,function(n){Dr(n,e.Diagnostics.Multiple_constructor_implementations_are_not_allowed)}),_&&e.forEach(h,function(n){Dr(e.getNameOfDeclaration(n)||n,e.Diagnostics.Duplicate_function_implementation)}),y&&!m&&16&n.flags&&f){var A=e.filter(f,function(e){return 255===e.kind}).map(function(n){return e.createDiagnosticForNode(n,e.Diagnostics.Consider_adding_a_declare_modifier_to_this_class)});e.forEach(f,function(t){var r=255===t.kind?e.Diagnostics.Class_declaration_cannot_implement_overload_list_for_0:254===t.kind?e.Diagnostics.Function_with_bodies_can_only_merge_with_classes_that_are_ambient:void 0;r&&e.addRelatedInfo.apply(void 0,a([Dr(e.getNameOfDeclaration(t)||t,r,e.symbolName(n))],A,!1))})}if(!i||i.body||e.hasSyntacticModifier(i,128)||i.questionToken||M(i),p&&(f&&(w(f,r,s,c,l),P(f,r,u,d)),r))for(var C=$u(n),I=qu(r),N=0,k=C;N<k.length;N++){var R=k[N];if(!Mg(I,R)){e.addRelatedInfo(Dr(R.declaration,e.Diagnostics.This_overload_signature_is_not_compatible_with_its_implementation_signature),e.createDiagnosticForNode(r,e.Diagnostics.The_implementation_signature_is_declared_here));break}}}function O(e,n){var t=void 0!==n&&n.parent===e[0].parent;return t?n:e[0]}function w(n,t,r,i,a){var o=i^a;if(0!==o){var s=KC(O(n,t),r);e.forEach(n,function(n){var t=KC(n,r)^s;1&t?Dr(e.getNameOfDeclaration(n),e.Diagnostics.Overload_signatures_must_all_be_exported_or_non_exported):2&t?Dr(e.getNameOfDeclaration(n),e.Diagnostics.Overload_signatures_must_all_be_ambient_or_non_ambient):24&t?Dr(e.getNameOfDeclaration(n)||n,e.Diagnostics.Overload_signatures_must_all_be_public_private_or_protected):128&t&&Dr(e.getNameOfDeclaration(n),e.Diagnostics.Overload_signatures_must_all_be_abstract_or_non_abstract)})}}function P(n,t,r,i){if(r!==i){var a=e.hasQuestionToken(O(n,t));e.forEach(n,function(n){var t=e.hasQuestionToken(n)!==a;t&&Dr(e.getNameOfDeclaration(n),e.Diagnostics.Overload_signatures_must_all_be_optional_or_required)})}}function M(n){if(!n.name||!e.nodeIsMissing(n.name)){var t=!1,r=e.forEachChild(n.parent,function(e){if(t)return e;t=e===n});if(r&&r.pos===n.end&&r.kind===n.kind){var i=r.name||r,a=r.name;if(n.name&&a&&(e.isPrivateIdentifier(n.name)&&e.isPrivateIdentifier(a)&&n.name.escapedText===a.escapedText||e.isComputedPropertyName(n.name)&&e.isComputedPropertyName(a)||e.isPropertyNameLiteral(n.name)&&e.isPropertyNameLiteral(a)&&e.getEscapedTextOfIdentifierOrLiteral(n.name)===e.getEscapedTextOfIdentifierOrLiteral(a))){var o=(167===n.kind||166===n.kind)&&e.isStatic(n)!==e.isStatic(r);if(o){var s=e.isStatic(n)?e.Diagnostics.Function_overload_must_be_static:e.Diagnostics.Function_overload_must_not_be_static;Dr(i,s)}return}if(e.nodeIsPresent(r.body))return void Dr(i,e.Diagnostics.Function_implementation_name_must_be_0,e.declarationNameToString(n.name))}var c=n.name||n;m?Dr(c,e.Diagnostics.Constructor_implementation_is_missing):e.hasSyntacticModifier(n,128)?Dr(c,e.Diagnostics.All_declarations_of_an_abstract_method_must_be_consecutive):Dr(c,e.Diagnostics.Function_implementation_is_missing_or_not_immediately_following_the_declaration)}}}function WC(n){if(t){var r=n.localSymbol;if((r||(r=Ca(n),r.exportSymbol))&&e.getDeclarationOfKind(r,n.kind)===n){for(var i=0,a=0,o=0,s=0,c=r.declarations;s<c.length;s++){var l=c[s],u=h(l),d=KC(l,513);1&d?512&d?o|=u:i|=u:a|=u}var p=i|a,f=i&a,m=o&p;if(f||m)for(var _=0,g=r.declarations;_<g.length;_++){l=g[_],u=h(l);var y=e.getNameOfDeclaration(l);u&m?Dr(y,e.Diagnostics.Merged_declaration_0_cannot_include_a_default_export_declaration_Consider_adding_a_separate_export_default_0_declaration_instead,e.declarationNameToString(y)):u&f&&Dr(y,e.Diagnostics.Individual_declarations_in_merged_declaration_0_must_be_all_exported_or_all_local,e.declarationNameToString(y))}}}function h(n){var t=n;switch(t.kind){case 256:case 257:case 340:case 333:case 334:return 2;case 259:return e.isAmbientModule(t)||0!==e.getModuleInstanceState(t)?5:4;case 255:case 258:case 294:return 3;case 300:return 7;case 269:case 219:var r=t,i=e.isExportAssignment(r)?r.expression:r.right;if(!e.isEntityNameExpression(i))return 1;t=i;case 263:case 266:case 265:var a=0,o=Ji(Ca(t));return e.forEach(o.declarations,function(e){a|=h(e)}),a;case 252:case 201:case 254:case 268:case 79:return 1;default:return e.Debug.failBadSyntaxKind(t)}}}function zC(e,n,t,r){var i=JC(e,n);return i&&YC(i,n,t,r)}function JC(n,t){if(!Vo(n)){var r=n;if(r.promisedTypeOfPromise)return r.promisedTypeOfPromise;if(Js(n,_p(!1)))return r.promisedTypeOfPromise=Md(n)[0];var i=Bo(n,"then");if(!Vo(i)){var a=i?Au(i,0):e.emptyArray;if(0!==a.length){var o=Jv(ff(e.map(a,bD)),2097152);if(!Vo(o)){var s=Au(o,0);if(0!==s.length)return r.promisedTypeOfPromise=ff(e.map(s,bD),2);t&&Dr(t,e.Diagnostics.The_first_parameter_of_the_then_method_of_a_promise_must_be_a_callback)}}else t&&Dr(t,e.Diagnostics.A_promise_must_have_a_then_method)}}}function qC(e,n,t,r){var i=YC(e,n,t,r);return i||Re}function XC(e){var n=Bo(e,"then");return!!n&&Au(Jv(n,2097152),0).length>0}function YC(e,n,t,r){if(Vo(e))return e;var i=e;return i.awaitedTypeOfType?i.awaitedTypeOfType:i.awaitedTypeOfType=Db(e,n?function(e){return QC(e,n,t,r)}:QC)}function QC(n,t,r,i){var a=n;if(a.awaitedTypeOfType)return a.awaitedTypeOfType;var o=JC(n);if(o){if(n.id===o.id||lr.lastIndexOf(o.id)>=0)return void(t&&Dr(t,e.Diagnostics.Type_is_referenced_directly_or_indirectly_in_the_fulfillment_callback_of_its_own_then_method));lr.push(n.id);var s=YC(o,t,r,i);if(lr.pop(),!s)return;return a.awaitedTypeOfType=s}if(!XC(n))return a.awaitedTypeOfType=n;if(t){if(!r)return e.Debug.fail();Dr(t,r,i)}}function ZC(n,t){var r=u_(t);if(F>=2){if(r===Re)return;var i=_p(!0);if(i!==gn&&!Js(r,i))return void Dr(t,e.Diagnostics.The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type_Did_you_mean_to_write_Promise_0,go(YC(r)||Qe))}else{if(eI(t),r===Re)return;var a=e.getEntityNameFromTypeNode(t);if(void 0===a)return void Dr(t,e.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value,go(r));var o=ra(a,111551,!0),s=o?Ws(o):Re;if(s===Re)return void(79===a.kind&&"Promise"===a.escapedText&&qs(r)===_p(!1)?Dr(t,e.Diagnostics.An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option):Dr(t,e.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value,e.entityNameToString(a)));var c=hp(!0);if(c===pn)return void Dr(t,e.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value,e.entityNameToString(a));if(!fg(s,c,t,e.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value))return;var l=a&&e.getFirstIdentifier(a),u=Jr(n.locals,l.escapedText,111551);if(u)return void Dr(u.valueDeclaration,e.Diagnostics.Duplicate_identifier_0_Compiler_uses_declaration_1_to_support_async_functions,e.idText(l),e.entityNameToString(a))}qC(r,n,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member)}function $C(n){var t=ML(n);KL(t,n);var r=ad(t);if(!(1&r.flags)){var i,a,o=NL(n);switch(n.parent.kind){case 255:var s=Ca(n.parent),c=Ws(s);i=ff([c,Qe]);break;case 162:i=Qe,a=e.chainDiagnosticMessages(void 0,e.Diagnostics.The_return_type_of_a_parameter_decorator_function_must_be_either_void_or_any);break;case 165:i=Qe,a=e.chainDiagnosticMessages(void 0,e.Diagnostics.The_return_type_of_a_property_decorator_function_must_be_either_void_or_any);break;case 167:case 170:case 171:var l=RR(n.parent),u=wp(l);i=ff([u,Qe]);break;default:return e.Debug.fail()}fg(r,i,n,o,function(){return a})}}function eI(n){nI(n&&e.getEntityNameFromTypeNode(n))}function nI(n){if(n){var t=e.getFirstIdentifier(n),r=2097152|(79===n.kind?788968:1920),i=Qr(t,t.escapedText,r,void 0,void 0,!0);i&&2097152&i.flags&&Fa(i)&&!nO(Ji(i))&&!Qi(i)&&$i(i)}}function tI(n){var t=rI(n);t&&e.isEntityName(t)&&nI(t)}function rI(e){if(e)switch(e.kind){case 186:case 185:return iI(e.types);case 187:return iI([e.trueType,e.falseType]);case 189:case 195:return rI(e.type);case 176:return e.typeName}}function iI(n){for(var t,r=0,i=n;r<i.length;r++){var a=i[r];while(189===a.kind||195===a.kind)a=a.type;if(142!==a.kind&&(z||(194!==a.kind||104!==a.literal.kind)&&151!==a.kind)){var o=rI(a);if(!o)return;if(t){if(!e.isIdentifier(t)||!e.isIdentifier(o)||t.escapedText!==o.escapedText)return}else t=o}}return t}function aI(n){var t=e.getEffectiveTypeAnnotationNode(n);return e.isRestParameter(n)?e.getRestParameterElementType(t):t}function oI(n){if(n.decorators&&e.nodeCanBeDecorated(n,n.parent,n.parent.parent)){M.experimentalDecorators||Dr(n,e.Diagnostics.Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_in_your_tsconfig_or_jsconfig_to_remove_this_warning);var t=n.decorators[0];if(IO(t,8),162===n.kind&&IO(t,32),M.emitDecoratorMetadata)switch(IO(t,16),n.kind){case 255:var r=e.getFirstConstructorWithBody(n);if(r)for(var i=0,a=r.parameters;i<a.length;i++){var o=a[i];tI(aI(o))}break;case 170:case 171:var s=170===n.kind?171:170,c=e.getDeclarationOfKind(Ca(n),s);tI(Cs(n)||c&&Cs(c));break;case 167:for(var l=0,u=n.parameters;l<u.length;l++){o=u[l];tI(aI(o))}tI(e.getEffectiveReturnTypeNode(n));break;case 165:tI(e.getEffectiveTypeAnnotationNode(n));break;case 162:tI(aI(n));for(var d=n.parent,p=0,f=d.parameters;p<f.length;p++){o=f[p];tI(aI(o))}break}e.forEach(n.decorators,$C)}}function sI(e){t&&(yI(e),aw(e),HI(e,e.name))}function cI(n){n.typeExpression||Dr(n.name,e.Diagnostics.JSDoc_typedef_tag_should_either_have_a_type_annotation_or_be_followed_by_property_or_member_tags),n.name&&$N(n.name,e.Diagnostics.Type_alias_name_cannot_be_0),Zk(n.typeExpression)}function lI(e){Zk(e.constraint);for(var n=0,t=e.typeParameters;n<t.length;n++){var r=t[n];Zk(r)}}function uI(e){Zk(e.typeExpression)}function dI(n){if(Zk(n.typeExpression),!e.getParameterSymbolFromJSDoc(n)){var t=e.getHostSignatureFromJSDoc(n);if(t){var r=e.getJSDocTags(t).filter(e.isJSDocParameterTag).indexOf(n);if(r>-1&&r<t.parameters.length&&e.isBindingPattern(t.parameters[r].name))return;Zu(t)?e.findLast(e.getJSDocTags(t),e.isJSDocParameterTag)===n&&n.typeExpression&&n.typeExpression.type&&!Ly(u_(n.typeExpression.type))&&Dr(n.name,e.Diagnostics.JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_It_would_match_arguments_if_it_had_an_array_type,e.idText(159===n.name.kind?n.name.right:n.name)):e.isQualifiedName(n.name)?Dr(n.name,e.Diagnostics.Qualified_name_0_is_not_allowed_without_a_leading_param_object_1,e.entityNameToString(n.name),e.entityNameToString(n.name.left)):Dr(n.name,e.Diagnostics.JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name,e.idText(n.name))}}}function pI(e){Zk(e.typeExpression)}function fI(n){!t||n.type||e.isJSDocConstructSignature(n)||kh(n,Ie),lC(n)}function mI(n){var t=e.getEffectiveJSDocHost(n);t&&(e.isClassDeclaration(t)||e.isClassExpression(t))||Dr(t,e.Diagnostics.JSDoc_0_is_not_attached_to_a_class,e.idText(n.tagName))}function _I(n){var t=e.getEffectiveJSDocHost(n);if(t&&(e.isClassDeclaration(t)||e.isClassExpression(t))){var r=e.getJSDocTags(t).filter(e.isJSDocAugmentsTag);e.Debug.assert(r.length>0),r.length>1&&Dr(r[1],e.Diagnostics.Class_declarations_cannot_have_more_than_one_augments_or_extends_tag);var i=gI(n.class.expression),a=e.getClassExtendsHeritageElement(t);if(a){var o=gI(a.expression);o&&i.escapedText!==o.escapedText&&Dr(i,e.Diagnostics.JSDoc_0_1_does_not_match_the_extends_2_clause,e.idText(n.tagName),e.idText(i),e.idText(o))}}else Dr(t,e.Diagnostics.JSDoc_0_is_not_attached_to_a_class,e.idText(n.tagName))}function gI(e){switch(e.kind){case 79:return e;case 204:return e.name;default:return}}function yI(n){var r;oI(n),lC(n);var i=e.getFunctionFlags(n);if(n.name&&160===n.name.kind&&WT(n.name),Vc(n)){var a=Ca(n),o=n.localSymbol||a,s=null===(r=o.declarations)||void 0===r?void 0:r.find(function(e){return e.kind===n.kind&&!(131072&e.flags)});n===s&&HC(o),a.parent&&HC(a)}var c=166===n.kind?void 0:n.body;if(Zk(c),VD(n,od(n)),t&&!e.getEffectiveReturnTypeNode(n)&&(e.nodeIsMissing(c)&&!jC(n)&&kh(n,Ie),1&i&&e.nodeIsPresent(c)&&ad(qu(n))),e.isInJSFile(n)){var l=e.getJSDocTypeTag(n);l&&l.typeExpression&&!RT(u_(l.typeExpression),n)&&Dr(l.typeExpression.type,e.Diagnostics.The_type_of_a_function_declaration_must_match_the_function_s_signature)}}function hI(n){if(t){var r=e.getSourceFileOfNode(n),i=wt.get(r.path);i||(i=[],wt.set(r.path,i)),i.push(n)}}function vI(n,t){for(var r=0,i=n;r<i.length;r++){var a=i[r];switch(a.kind){case 255:case 224:TI(a,t),xI(a,t);break;case 300:case 259:case 233:case 261:case 240:case 241:case 242:II(a,t);break;case 169:case 211:case 254:case 212:case 167:case 170:case 171:a.body&&II(a,t),xI(a,t);break;case 166:case 172:case 173:case 177:case 178:case 257:case 256:xI(a,t);break;case 188:SI(a,t);break;default:e.Debug.assertNever(a,"Node should not have been registered for unused identifiers check")}}}function bI(n,t,r){var i=e.getNameOfDeclaration(n)||n,a=gR(n)?e.Diagnostics._0_is_declared_but_never_used:e.Diagnostics._0_is_declared_but_its_value_is_never_read;r(n,0,e.createDiagnosticForNode(i,a,t))}function EI(n){return e.isIdentifier(n)&&95===e.idText(n).charCodeAt(0)}function TI(n,t){for(var r=0,i=n.members;r<i.length;r++){var a=i[r];switch(a.kind){case 167:case 165:case 170:case 171:if(171===a.kind&&32768&a.symbol.flags)break;var o=Ca(a);o.isReferenced||!(e.hasEffectiveModifier(a,8)||e.isNamedDeclaration(a)&&e.isPrivateIdentifier(a.name))||8388608&a.flags||t(a,0,e.createDiagnosticForNode(a.name,e.Diagnostics._0_is_declared_but_its_value_is_never_read,mo(o)));break;case 169:for(var s=0,c=a.parameters;s<c.length;s++){var l=c[s];!l.symbol.isReferenced&&e.hasSyntacticModifier(l,8)&&t(l,0,e.createDiagnosticForNode(l.name,e.Diagnostics.Property_0_is_declared_but_its_value_is_never_read,e.symbolName(l.symbol)))}break;case 174:case 232:case 168:break;default:e.Debug.fail("Unexpected class member")}}}function SI(n,t){var r=n.typeParameter;LI(r)&&t(n,1,e.createDiagnosticForNode(n,e.Diagnostics._0_is_declared_but_its_value_is_never_read,e.idText(r.name)))}function xI(n,t){var r=Ca(n).declarations;if(r&&e.last(r)===n)for(var i=e.getEffectiveTypeParameterDeclarations(n),a=new e.Set,o=0,s=i;o<s.length;o++){var c=s[o];if(LI(c)){var l=e.idText(c.name),u=c.parent;if(188!==u.kind&&u.typeParameters.every(LI)){if(e.tryAddToSet(a,u)){var d=e.getSourceFileOfNode(u),p=e.isJSDocTemplateTag(u)?e.rangeOfNode(u):e.rangeOfTypeParameters(d,u.typeParameters),f=1===u.typeParameters.length,m=f?e.Diagnostics._0_is_declared_but_its_value_is_never_read:e.Diagnostics.All_type_parameters_are_unused,_=f?l:void 0;t(c,1,e.createFileDiagnostic(d,p.pos,p.end-p.pos,m,_))}}else t(c,1,e.createDiagnosticForNode(c,e.Diagnostics._0_is_declared_but_its_value_is_never_read,l))}}}function LI(e){return!(262144&Aa(e.symbol).isReferenced)&&!EI(e.name)}function DI(e,n,t,r){var i=String(r(n)),a=e.get(i);a?a[1].push(t):e.set(i,[n,[t]])}function AI(n){return e.tryCast(e.getRootDeclaration(n),e.isParameter)}function CI(n){return e.isBindingElement(n)?e.isObjectBindingPattern(n.parent)?!(!n.propertyName||!EI(n.name)):EI(n.name):e.isAmbientModule(n)||(e.isVariableDeclaration(n)&&e.isForInOrOfStatement(n.parent.parent)||kI(n))&&EI(n.name)}function II(n,t){var r=new e.Map,i=new e.Map,a=new e.Map;n.locals.forEach(function(n){if((262144&n.flags?3&n.flags&&!(3&n.isReferenced):!n.isReferenced&&!n.exportSymbol)&&n.declarations)for(var o=0,s=n.declarations;o<s.length;o++){var c=s[o];if(!CI(c))if(kI(c))DI(r,RI(c),c,O);else if(e.isBindingElement(c)&&e.isObjectBindingPattern(c.parent)){var l=e.last(c.parent.elements);c!==l&&e.last(c.parent.elements).dotDotDotToken||DI(i,c.parent,c,O)}else if(e.isVariableDeclaration(c))DI(a,c.parent,c,O);else{var u=n.valueDeclaration&&AI(n.valueDeclaration),d=n.valueDeclaration&&e.getNameOfDeclaration(n.valueDeclaration);u&&d?e.isParameterPropertyDeclaration(u,u.parent)||e.parameterIsThisKeyword(u)||EI(d)||(e.isBindingElement(c)&&e.isArrayBindingPattern(c.parent)?DI(i,c.parent,c,O):t(u,1,e.createDiagnosticForNode(d,e.Diagnostics._0_is_declared_but_its_value_is_never_read,e.symbolName(n)))):bI(c,e.symbolName(n),t)}}}),r.forEach(function(n){var r=n[0],i=n[1],a=r.parent,o=(r.name?1:0)+(r.namedBindings?266===r.namedBindings.kind?1:r.namedBindings.elements.length:0);if(o===i.length)t(a,0,1===i.length?e.createDiagnosticForNode(a,e.Diagnostics._0_is_declared_but_its_value_is_never_read,e.idText(e.first(i).name)):e.createDiagnosticForNode(a,e.Diagnostics.All_imports_in_import_declaration_are_unused));else for(var s=0,c=i;s<c.length;s++){var l=c[s];bI(l,e.idText(l.name),t)}}),i.forEach(function(n){var r=n[0],i=n[1],o=AI(r.parent)?1:0;if(r.elements.length===i.length)1===i.length&&252===r.parent.kind&&253===r.parent.parent.kind?DI(a,r.parent.parent,r.parent,O):t(r,o,1===i.length?e.createDiagnosticForNode(r,e.Diagnostics._0_is_declared_but_its_value_is_never_read,NI(e.first(i).name)):e.createDiagnosticForNode(r,e.Diagnostics.All_destructured_elements_are_unused));else for(var s=0,c=i;s<c.length;s++){var l=c[s];t(l,o,e.createDiagnosticForNode(l,e.Diagnostics._0_is_declared_but_its_value_is_never_read,NI(l.name)))}}),a.forEach(function(n){var r=n[0],i=n[1];if(r.declarations.length===i.length)t(r,0,1===i.length?e.createDiagnosticForNode(e.first(i).name,e.Diagnostics._0_is_declared_but_its_value_is_never_read,NI(e.first(i).name)):e.createDiagnosticForNode(235===r.parent.kind?r.parent:r,e.Diagnostics.All_variables_are_unused));else for(var a=0,o=i;a<o.length;a++){var s=o[a];t(s,0,e.createDiagnosticForNode(s,e.Diagnostics._0_is_declared_but_its_value_is_never_read,NI(s.name)))}})}function NI(n){switch(n.kind){case 79:return e.idText(n);case 200:case 199:return NI(e.cast(e.first(n.elements),e.isBindingElement).name);default:return e.Debug.assertNever(n)}}function kI(e){return 265===e.kind||268===e.kind||266===e.kind}function RI(e){return 265===e.kind?e:266===e.kind?e.parent:e.parent.parent}function OI(n){if(233===n.kind&&Kw(n),e.isFunctionOrModuleBlock(n)){var t=Gt;e.forEach(n.statements,Zk),Gt=t}else e.forEach(n.statements,Zk);n.locals&&hI(n)}function wI(n){F>=2||!e.hasRestParameter(n)||8388608&n.flags||e.nodeIsMissing(n.body)||e.forEach(n.parameters,function(n){n.name&&!e.isBindingPattern(n.name)&&n.name.escapedText===le.escapedName&&xr("noEmit",n,e.Diagnostics.Duplicate_identifier_arguments_Compiler_uses_arguments_to_initialize_rest_parameters)})}function PI(n,t,r){if((null===t||void 0===t?void 0:t.escapedText)!==r)return!1;if(165===n.kind||164===n.kind||167===n.kind||166===n.kind||170===n.kind||171===n.kind||291===n.kind)return!1;if(8388608&n.flags)return!1;if((e.isImportClause(n)||e.isImportEqualsDeclaration(n)||e.isImportSpecifier(n))&&e.isTypeOnlyImportOrExportDeclaration(n))return!1;var i=e.getRootDeclaration(n);return!e.isParameter(i)||!e.nodeIsMissing(i.parent.body)}function MI(n){e.findAncestor(n,function(t){if(4&lO(t)){var r=79!==n.kind;return r?Dr(e.getNameOfDeclaration(n),e.Diagnostics.Duplicate_identifier_this_Compiler_uses_variable_declaration_this_to_capture_this_reference):Dr(n,e.Diagnostics.Expression_resolves_to_variable_declaration_this_that_compiler_uses_to_capture_this_reference),!0}return!1})}function FI(n){e.findAncestor(n,function(t){if(8&lO(t)){var r=79!==n.kind;return r?Dr(e.getNameOfDeclaration(n),e.Diagnostics.Duplicate_identifier_newTarget_Compiler_uses_variable_declaration_newTarget_to_capture_new_target_meta_property_reference):Dr(n,e.Diagnostics.Expression_resolves_to_variable_declaration_newTarget_that_compiler_uses_to_capture_new_target_meta_property_reference),!0}return!1})}function GI(n,t){if(!(K>=e.ModuleKind.ES2015)&&t&&(PI(n,t,"require")||PI(n,t,"exports"))&&(!e.isModuleDeclaration(n)||1===e.getModuleInstanceState(n))){var r=Fo(n);300===r.kind&&e.isExternalOrCommonJsModule(r)&&xr("noEmit",t,e.Diagnostics.Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module,e.declarationNameToString(t),e.declarationNameToString(t))}}function BI(n,t){if(t&&!(F>=4)&&PI(n,t,"Promise")&&(!e.isModuleDeclaration(n)||1===e.getModuleInstanceState(n))){var r=Fo(n);300===r.kind&&e.isExternalOrCommonJsModule(r)&&2048&r.flags&&xr("noEmit",t,e.Diagnostics.Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_containing_async_functions,e.declarationNameToString(t),e.declarationNameToString(t))}}function UI(e,n){F<=8&&(PI(e,n,"WeakMap")||PI(e,n,"WeakSet"))&&sr.push(e)}function VI(n){var t=e.getEnclosingBlockScopeContainer(n);67108864&lO(t)&&(e.Debug.assert(e.isNamedDeclaration(n)&&e.isIdentifier(n.name)&&"string"===typeof n.name.escapedText,"The target of a WeakMap/WeakSet collision check should be an identifier"),xr("noEmit",n,e.Diagnostics.Compiler_reserves_name_0_when_emitting_private_identifier_downlevel,n.name.escapedText))}function jI(e,n){n&&F>=2&&F<=8&&PI(e,n,"Reflect")&&cr.push(e)}function KI(n){var t=!1;if(e.isClassExpression(n))for(var r=0,i=n.members;r<i.length;r++){var a=i[r];if(134217728&lO(a)){t=!0;break}}else if(e.isFunctionExpression(n))134217728&lO(n)&&(t=!0);else{var o=e.getEnclosingBlockScopeContainer(n);o&&134217728&lO(o)&&(t=!0)}t&&(e.Debug.assert(e.isNamedDeclaration(n)&&e.isIdentifier(n.name),"The target of a Reflect collision check should be an identifier"),xr("noEmit",n,e.Diagnostics.Duplicate_identifier_0_Compiler_reserves_name_1_when_emitting_super_references_in_static_initializers,e.declarationNameToString(n.name),"Reflect"))}function HI(n,t){t&&(GI(n,t),BI(n,t),UI(n,t),jI(n,t),e.isClassLike(n)?($N(t,e.Diagnostics.Class_name_cannot_be_0),8388608&n.flags||ek(t)):e.isEnumDeclaration(n)&&$N(t,e.Diagnostics.Enum_name_cannot_be_0))}function WI(n){if(0===(3&e.getCombinedNodeFlags(n))&&!e.isParameterDeclaration(n)&&(252!==n.kind||n.initializer)){var t=Ca(n);if(1&t.flags){if(!e.isIdentifier(n.name))return e.Debug.fail();var r=Qr(n,n.name.escapedText,3,void 0,void 0,!1);if(r&&r!==t&&2&r.flags&&3&kS(r)){var i=e.getAncestor(r.valueDeclaration,253),a=235===i.parent.kind&&i.parent.parent?i.parent.parent:void 0,o=a&&(233===a.kind&&e.isFunctionLike(a.parent)||260===a.kind||259===a.kind||300===a.kind);if(!o){var s=mo(r);Dr(n,e.Diagnostics.Cannot_initialize_outer_scoped_variable_0_in_the_same_scope_as_block_scoped_declaration_1,s,s)}}}}}function zI(e){return e===Ne?Ie:e===Bn?Gn:e}function JI(n){var t;if(oI(n),e.isBindingElement(n)||Zk(n.type),n.name){if(160===n.name.kind&&(WT(n.name),n.initializer&&kA(n.initializer)),e.isBindingElement(n)){e.isObjectBindingPattern(n.parent)&&n.dotDotDotToken&&F<5&&IO(n,4),n.propertyName&&160===n.propertyName.kind&&WT(n.propertyName);var r=n.parent.parent,i=jo(r),a=n.propertyName||n.name;if(i&&!e.isBindingPattern(a)){var o=Ff(a);if(Fc(o)){var s=Kc(o),c=Lu(i,s);c&&(bx(c,void 0,!1),OS(n,!!r.initializer&&106===r.initializer.kind,!1,i,c))}}}if(e.isBindingPattern(n.name)&&(200===n.name.kind&&F<2&&M.downlevelIteration&&IO(n,512),e.forEach(n.name.elements,Zk)),n.initializer&&e.isParameterDeclaration(n)&&e.nodeIsMissing(e.getContainingFunction(n).body))Dr(n,e.Diagnostics.A_parameter_initializer_is_only_allowed_in_a_function_or_constructor_implementation);else if(e.isBindingPattern(n.name)){var l=n.initializer&&241!==n.parent.parent.kind,u=0===n.name.elements.length;if(l||u){var d=Es(n);if(l){var p=kA(n.initializer);z&&u?KS(p,n):mg(p,Es(n),n,n.initializer)}u&&(e.isArrayBindingPattern(n.name)?fN(65,d,Me,n):z&&KS(d,n))}}else{var f=Ca(n);if(2097152&f.flags&&e.isRequireVariableDeclaration(n))Pk(n);else{var m=zI(Ws(f));if(n===f.valueDeclaration){var _=e.getEffectiveInitializer(n);if(_){var g=e.isInJSFile(n)&&e.isObjectLiteralExpression(_)&&(0===_.properties.length||e.isPrototypeAccess(n.name))&&!!(null===(t=f.exports)||void 0===t?void 0:t.size);g||241===n.parent.parent.kind||mg(kA(_),m,n,_,void 0)}f.declarations&&f.declarations.length>1&&e.some(f.declarations,function(t){return t!==n&&e.isVariableLike(t)&&!XI(t,n)})&&Dr(n.name,e.Diagnostics.All_declarations_of_0_must_have_identical_modifiers,e.declarationNameToString(n.name))}else{var y=zI(Es(n));m===Re||y===Re||ig(m,y)||67108864&f.flags||qI(f.valueDeclaration,m,n,y),n.initializer&&mg(kA(n.initializer),y,n,n.initializer,void 0),f.valueDeclaration&&!XI(n,f.valueDeclaration)&&Dr(n.name,e.Diagnostics.All_declarations_of_0_must_have_identical_modifiers,e.declarationNameToString(n.name))}165!==n.kind&&164!==n.kind&&(WC(n),252!==n.kind&&201!==n.kind||WI(n),HI(n,n.name))}}}}function qI(n,t,r,i){var a=e.getNameOfDeclaration(r),o=165===r.kind||164===r.kind?e.Diagnostics.Subsequent_property_declarations_must_have_the_same_type_Property_0_must_be_of_type_1_but_here_has_type_2:e.Diagnostics.Subsequent_variable_declarations_must_have_the_same_type_Variable_0_must_be_of_type_1_but_here_has_type_2,s=e.declarationNameToString(a),c=Dr(a,o,s,go(t),go(i));n&&e.addRelatedInfo(c,e.createDiagnosticForNode(n,e.Diagnostics._0_was_also_declared_here,s))}function XI(n,t){if(162===n.kind&&252===t.kind||252===n.kind&&162===t.kind)return!0;if(e.hasQuestionToken(n)!==e.hasQuestionToken(t))return!1;var r=504;return e.getSelectedEffectiveModifierFlags(n,r)===e.getSelectedEffectiveModifierFlags(t,r)}function YI(n){null===e.tracing||void 0===e.tracing||e.tracing.push("check","checkVariableDeclaration",{kind:n.kind,pos:n.pos,end:n.end}),Lw(n),JI(n),null===e.tracing||void 0===e.tracing||e.tracing.pop()}function QI(e){return bw(e),JI(e)}function ZI(n){RO(n)||Cw(n.declarationList)||Nw(n),e.forEach(n.declarationList.declarations,Zk)}function $I(e){Kw(e),eC(e.expression)}function eN(n){Kw(n);var t=sN(n.expression);nN(n.expression,t,n.thenStatement),Zk(n.thenStatement),234===n.thenStatement.kind&&Dr(n.thenStatement,e.Diagnostics.The_body_of_an_if_statement_cannot_be_the_empty_statement),Zk(n.elseStatement)}function nN(n,t,r){if(z&&!th(t)){var i=e.isBinaryExpression(n)?n.right:n;if(!e.isPropertyAccessExpression(i)||!e.isAssertionExpression(e.skipParentheses(i.expression))){var a=e.isIdentifier(i)?i:e.isPropertyAccessExpression(i)?i.name:e.isBinaryExpression(i)&&e.isIdentifier(i.right)?i.right:void 0,o=Au(t,0),s=!!zC(t);if(0!==o.length||s){var c=a&&CR(a);if(c||s){var l=c&&e.isBinaryExpression(n.parent)&&rN(n.parent,c)||c&&r&&tN(n,r,a,c);l||(s?Ir(i,!0,e.Diagnostics.This_condition_will_always_return_true_since_this_0_is_always_defined,ho(t)):Dr(i,e.Diagnostics.This_condition_will_always_return_true_since_this_function_is_always_defined_Did_you_mean_to_call_it_instead))}}}}}function tN(n,t,r,i){return!!e.forEachChild(t,function t(a){if(e.isIdentifier(a)){var o=CR(a);if(o&&o===i){if(e.isIdentifier(n))return!0;var s=r.parent,c=a.parent;while(s&&c){if(e.isIdentifier(s)&&e.isIdentifier(c)||108===s.kind&&108===c.kind)return CR(s)===CR(c);if(e.isPropertyAccessExpression(s)&&e.isPropertyAccessExpression(c)){if(CR(s.name)!==CR(c.name))return!1;c=c.expression,s=s.expression}else{if(!e.isCallExpression(s)||!e.isCallExpression(c))return!1;c=c.expression,s=s.expression}}}}return e.forEachChild(a,t)})}function rN(n,t){while(e.isBinaryExpression(n)&&55===n.operatorToken.kind){var r=e.forEachChild(n.right,function n(r){if(e.isIdentifier(r)){var i=CR(r);if(i&&i===t)return!0}return e.forEachChild(r,n)});if(r)return!0;n=n.parent}return!1}function iN(e){Kw(e),Zk(e.statement),sN(e.expression)}function aN(e){Kw(e),sN(e.expression),Zk(e.statement)}function oN(n,t){return 16384&n.flags&&Dr(t,e.Diagnostics.An_expression_of_type_void_cannot_be_tested_for_truthiness),n}function sN(e,n){return oN(eC(e,n),e)}function cN(n){Kw(n)||n.initializer&&253===n.initializer.kind&&Cw(n.initializer),n.initializer&&(253===n.initializer.kind?e.forEach(n.initializer.declarations,YI):eC(n.initializer)),n.condition&&sN(n.condition),n.incrementor&&eC(n.incrementor),Zk(n.statement),n.locals&&hI(n)}function lN(n){pw(n);var t=e.getContainingFunctionOrClassStaticBlock(n);if(n.awaitModifier)if(t&&e.isClassStaticBlockDeclaration(t))Mw(n.awaitModifier,e.Diagnostics.For_await_loops_cannot_be_used_inside_a_class_static_block);else{var r=e.getFunctionFlags(t);2===(6&r)&&F<99&&IO(n,16384)}else M.downlevelIteration&&F<2&&IO(n,256);if(253===n.initializer.kind)dN(n);else{var i=n.initializer,a=pN(n);if(202===i.kind||203===i.kind)_A(i,a||Re);else{var o=eC(i);XD(i,e.Diagnostics.The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access,e.Diagnostics.The_left_hand_side_of_a_for_of_statement_may_not_be_an_optional_property_access),a&&mg(a,o,i,n.expression)}}Zk(n.statement),n.locals&&hI(n)}function uN(n){pw(n);var t=GS(eC(n.expression));if(253===n.initializer.kind){var r=n.initializer.declarations[0];r&&e.isBindingPattern(r.name)&&Dr(r.name,e.Diagnostics.The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern),dN(n)}else{var i=n.initializer,a=eC(i);202===i.kind||203===i.kind?Dr(i,e.Diagnostics.The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern):lg(jf(t),a)?XD(i,e.Diagnostics.The_left_hand_side_of_a_for_in_statement_must_be_a_variable_or_a_property_access,e.Diagnostics.The_left_hand_side_of_a_for_in_statement_may_not_be_an_optional_property_access):Dr(i,e.Diagnostics.The_left_hand_side_of_a_for_in_statement_must_be_of_type_string_or_any)}t!==Ze&&aA(t,126091264)||Dr(n.expression,e.Diagnostics.The_right_hand_side_of_a_for_in_statement_must_be_of_type_any_an_object_type_or_a_type_parameter_but_here_has_type_0,go(t)),Zk(n.statement),n.locals&&hI(n)}function dN(e){var n=e.initializer;if(n.declarations.length>=1){var t=n.declarations[0];YI(t)}}function pN(e){var n=e.awaitModifier?15:13;return fN(n,MS(e.expression),Me,e.expression)}function fN(e,n,t,r){return Vo(n)?n:mN(e,n,t,r,!0)||Ie}function mN(n,t,r,i,a){var o=0!==(2&n);if(t!==Ze){var s=F>=2,c=!s&&M.downlevelIteration,l=M.noUncheckedIndexedAccess&&!!(128&n);if(s||c||o){var u=EN(t,n,s?i:void 0);if(a&&u){var d=8&n?e.Diagnostics.Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_for_of_will_always_send_0:32&n?e.Diagnostics.Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_spread_will_always_send_0:64&n?e.Diagnostics.Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_destructuring_will_always_send_0:16&n?e.Diagnostics.Cannot_delegate_iteration_to_value_because_the_next_method_of_its_iterator_expects_type_1_but_the_containing_generator_will_always_send_0:void 0;d&&fg(r,u.nextType,i,d)}if(u||s)return l?Qv(u&&u.yieldType):u&&u.yieldType}var p=t,f=!1,m=!1;if(4&n){if(1048576&p.flags){var _=t.types,g=e.filter(_,function(e){return!(402653316&e.flags)});g!==_&&(p=ff(g,2))}else 402653316&p.flags&&(p=Ze);if(m=p!==t,m&&(F<1&&i&&(Dr(i,e.Diagnostics.Using_a_string_in_a_for_of_statement_is_only_supported_in_ECMAScript_5_and_higher),f=!0),131072&p.flags))return l?Qv(je):je}if(!Iy(p)){if(i&&!f){var y=!!(4&n)&&!m,h=T(y,c),v=h[0],b=h[1];Ir(i,b&&!!zC(p),v,go(p))}return m?l?Qv(je):je:void 0}var E=wu(p,Ke);return m&&E?402653316&E.flags&&!M.noUncheckedIndexedAccess?je:ff(l?[E,je,Me]:[E,je],2):128&n?Qv(E):E}function T(r,i){var a;if(i)return r?[e.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator,!0]:[e.Diagnostics.Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator,!0];var o=gN(n,0,t,void 0);return o?[e.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators,!1]:_N(null===(a=t.symbol)||void 0===a?void 0:a.escapedName)?[e.Diagnostics.Type_0_can_only_be_iterated_through_when_using_the_downlevelIteration_flag_or_with_a_target_of_es2015_or_higher,!0]:r?[e.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type,!0]:[e.Diagnostics.Type_0_is_not_an_array_type,!0]}IN(i,t,o)}function _N(e){switch(e){case"Float32Array":case"Float64Array":case"Int16Array":case"Int32Array":case"Int8Array":case"NodeList":case"Uint16Array":case"Uint32Array":case"Uint8Array":case"Uint8ClampedArray":return!0}return!1}function gN(e,n,t,r){if(!Vo(t)){var i=EN(t,e,r);return i&&i[U(n)]}}function yN(e,n,t){if(void 0===e&&(e=Ze),void 0===n&&(n=Ze),void 0===t&&(t=Pe),67359327&e.flags&&180227&n.flags&&180227&t.flags){var r=Nd([e,n,t]),i=Lt.get(r);return i||(i={yieldType:e,returnType:n,nextType:t},Lt.set(r,i)),i}return{yieldType:e,returnType:n,nextType:t}}function hN(n){for(var t,r,i,a=0,o=n;a<o.length;a++){var s=o[a];if(void 0!==s&&s!==Dt){if(s===At)return At;t=e.append(t,s.yieldType),r=e.append(r,s.returnType),i=e.append(i,s.nextType)}}return t||r||i?yN(t&&ff(t),r&&ff(r),i&&Df(i)):Dt}function vN(e,n){return e[n]}function bN(e,n,t){return e[n]=t}function EN(n,t,r){if(Vo(n))return At;if(!(1048576&n.flags)){var i=SN(n,t,r);return i===Dt?void(r&&IN(r,n,!!(2&t))):i}var a,o=2&t?"iterationTypesOfAsyncIterable":"iterationTypesOfIterable",s=vN(n,o);if(s)return s===Dt?void 0:s;for(var c=0,l=n.types;c<l.length;c++){var u=l[c],d=SN(u,t,r);if(d===Dt)return r&&IN(r,n,!!(2&t)),void bN(n,o,Dt);a=e.append(a,d)}var p=a?hN(a):Dt;return bN(n,o,p),p===Dt?void 0:p}function TN(e,n){if(e===Dt)return Dt;if(e===At)return At;var t=e.yieldType,r=e.returnType,i=e.nextType;return yN(YC(t,n)||Ie,YC(r,n)||Ie,i)}function SN(e,n,t){if(Vo(e))return At;if(2&n){var r=xN(e,Nt)||DN(e,Nt);if(r)return r}if(1&n){r=xN(e,kt)||DN(e,kt);if(r){if(!(2&n))return r;if(r!==Dt)return bN(e,"iterationTypesOfAsyncIterable",TN(r,t))}}if(2&n){r=CN(e,Nt,t);if(r!==Dt)return r}if(1&n){r=CN(e,kt,t);if(r!==Dt)return 2&n?bN(e,"iterationTypesOfAsyncIterable",r?TN(r,t):Dt):r}return Dt}function xN(e,n){return vN(e,n.iterableCacheKey)}function LN(e,n){var t=xN(e,n)||CN(e,n,void 0);return t===Dt?It:t}function DN(e,n){var t;if(Js(e,t=n.getGlobalIterableType(!1))||Js(e,t=n.getGlobalIterableIteratorType(!1))){var r=Md(e)[0],i=LN(t,n),a=i.returnType,o=i.nextType;return bN(e,n.iterableCacheKey,yN(r,a,o))}if(Js(e,n.getGlobalGeneratorType(!1))){var s=Md(e);r=s[0],a=s[1],o=s[2];return bN(e,n.iterableCacheKey,yN(r,a,o))}}function AN(n){var t=pp(!1),r=t&&Bo(Ws(t),e.escapeLeadingUnderscores(n));return r&&Fc(r)?Kc(r):"__@"+n}function CN(n,t,r){var i,a=Lu(n,AN(t.iteratorSymbolName)),o=!a||16777216&a.flags?void 0:Ws(a);if(Vo(o))return bN(n,t.iterableCacheKey,At);var s=o?Au(o,0):void 0;if(!e.some(s))return bN(n,t.iterableCacheKey,Dt);var c=Df(e.map(s,ad)),l=null!==(i=NN(c,t,r))&&void 0!==i?i:Dt;return bN(n,t.iterableCacheKey,l)}function IN(n,t,r){var i=r?e.Diagnostics.Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator:e.Diagnostics.Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator;Ir(n,!!zC(t),i,go(t))}function NN(e,n,t){if(Vo(e))return At;var r=kN(e,n)||RN(e,n)||GN(e,n,t);return r===Dt?void 0:r}function kN(e,n){return vN(e,n.iteratorCacheKey)}function RN(e,n){var t=n.getGlobalIterableIteratorType(!1);if(Js(e,t)){var r=Md(e)[0],i=kN(t,n)||GN(t,n,void 0),a=i===Dt?It:i,o=a.returnType,s=a.nextType;return bN(e,n.iteratorCacheKey,yN(r,o,s))}if(Js(e,n.getGlobalIteratorType(!1))||Js(e,n.getGlobalGeneratorType(!1))){var c=Md(e);r=c[0],o=c[1],s=c[2];return bN(e,n.iteratorCacheKey,yN(r,o,s))}}function ON(e,n){var t=Bo(e,"done")||We;return lg(0===n?We:Je,t)}function wN(e){return ON(e,0)}function PN(e){return ON(e,1)}function MN(e){if(Vo(e))return At;var n=vN(e,"iterationTypesOfIteratorResult");if(n)return n;if(Js(e,Ap(!1))){var t=Md(e)[0];return bN(e,"iterationTypesOfIteratorResult",yN(t,void 0,void 0))}if(Js(e,Cp(!1))){var r=Md(e)[0];return bN(e,"iterationTypesOfIteratorResult",yN(void 0,r,void 0))}var i=Sb(e,wN),a=i!==Ze?Bo(i,"value"):void 0,o=Sb(e,PN),s=o!==Ze?Bo(o,"value"):void 0;return bN(e,"iterationTypesOfIteratorResult",a||s?yN(a,s||Qe,void 0):Dt)}function FN(n,t,r,i){var a,o,s,c,l=Lu(n,r);if(l||"next"===r){var u=!l||"next"===r&&16777216&l.flags?void 0:"next"===r?Ws(l):Jv(Ws(l),2097152);if(Vo(u))return"next"===r?At:Ct;var d,p,f,m,_,g=u?Au(u,0):e.emptyArray;if(0===g.length){if(i){var y="next"===r?t.mustHaveANextMethodDiagnostic:t.mustBeAMethodDiagnostic;Dr(i,y,r)}return"next"===r?At:void 0}if((null===u||void 0===u?void 0:u.symbol)&&1===g.length){var h=t.getGlobalGeneratorType(!1),v=t.getGlobalIteratorType(!1),b=(null===(o=null===(a=h.symbol)||void 0===a?void 0:a.members)||void 0===o?void 0:o.get(r))===u.symbol,E=!b&&(null===(c=null===(s=v.symbol)||void 0===s?void 0:s.members)||void 0===c?void 0:c.get(r))===u.symbol;if(b||E){var T=b?h:v,S=u.mapper;return yN(y_(T.typeParameters[0],S),y_(T.typeParameters[1],S),"next"===r?y_(T.typeParameters[2],S):void 0)}}for(var x=0,L=g;x<L.length;x++){var D=L[x];"throw"!==r&&e.some(D.parameters)&&(d=e.append(d,pD(D,0))),p=e.append(p,ad(D))}if("throw"!==r){var A=d?ff(d):Pe;if("next"===r)m=A;else if("return"===r){var C=t.resolveIterationType(A,i)||Ie;f=e.append(f,C)}}var I=p?Df(p):Ze,N=t.resolveIterationType(I,i)||Ie,k=MN(N);return k===Dt?(i&&Dr(i,t.mustHaveAValueDiagnostic,r),_=Ie,f=e.append(f,Ie)):(_=k.yieldType,f=e.append(f,k.returnType)),yN(_,ff(f),m)}}function GN(e,n,t){var r=hN([FN(e,n,"next",t),FN(e,n,"return",t),FN(e,n,"throw",t)]);return bN(e,n.iteratorCacheKey,r)}function BN(e,n,t){if(!Vo(n)){var r=UN(n,t);return r&&r[U(e)]}}function UN(e,n){if(Vo(e))return At;var t=n?2:1,r=n?Nt:kt;return EN(e,t,void 0)||NN(e,r,void 0)}function VN(e){Kw(e)||vw(e)}function jN(e,n){var t,r,i=!!(1&n),a=!!(2&n);return i?null!==(t=BN(1,e,a))&&void 0!==t?t:Re:a?null!==(r=YC(e))&&void 0!==r?r:Re:e}function KN(n,t){var r=jN(t,e.getFunctionFlags(n));return!!r&&iA(r,16387)}function HN(n){var t;if(!Kw(n)){var r=e.getContainingFunctionOrClassStaticBlock(n);if(r&&e.isClassStaticBlockDeclaration(r))Ow(n,e.Diagnostics.A_return_statement_cannot_be_used_inside_a_class_static_block);else if(r){var i=qu(r),a=ad(i),o=e.getFunctionFlags(r);if(z||n.expression||131072&a.flags){var s=n.expression?kA(n.expression):Me;if(171===r.kind)n.expression&&Dr(n,e.Diagnostics.Setters_cannot_return_a_value);else if(169===r.kind)n.expression&&!mg(s,a,n,n.expression)&&Dr(n,e.Diagnostics.Return_type_of_constructor_signature_must_be_assignable_to_the_instance_type_of_the_class);else if(od(r)){var c=null!==(t=jN(a,o))&&void 0!==t?t:a,l=2&o?qC(s,n,e.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member):s;c&&mg(l,c,n,n.expression)}}else 169!==r.kind&&M.noImplicitReturns&&!KN(r,a)&&Dr(n,e.Diagnostics.Not_all_code_paths_return_a_value)}else Ow(n,e.Diagnostics.A_return_statement_can_only_be_used_within_a_function_body)}}function WN(n){Kw(n)||32768&n.flags&&Ow(n,e.Diagnostics.with_statements_are_not_allowed_in_an_async_function_block),eC(n.expression);var t=e.getSourceFileOfNode(n);if(!Rw(t)){var r=e.getSpanOfTokenAtPosition(t,n.pos).start,i=n.statement.pos;ww(t,r,i-r,e.Diagnostics.The_with_statement_is_not_supported_All_symbols_in_a_with_block_will_have_type_any)}}function zN(n){var r;Kw(n);var i=!1,a=eC(n.expression),o=Uy(a);e.forEach(n.caseBlock.clauses,function(n){if(288!==n.kind||i||(void 0===r?r=n:(Mw(n,e.Diagnostics.A_default_clause_cannot_appear_more_than_once_in_a_switch_statement),i=!0)),t&&287===n.kind){var s=eC(n.expression),c=Uy(s),l=a;c&&o||(s=c?Vy(s):s,l=Vy(a)),hA(l,s)||kg(s,l,n.expression,void 0)}e.forEach(n.statements,Zk),M.noFallthroughCasesInSwitch&&n.fallthroughFlowNode&&Yb(n.fallthroughFlowNode)&&Dr(n,e.Diagnostics.Fallthrough_case_in_switch)}),n.caseBlock.locals&&hI(n.caseBlock)}function JN(n){Kw(n)||e.findAncestor(n.parent,function(t){return e.isFunctionLike(t)?"quit":248===t.kind&&t.label.escapedText===n.label.escapedText&&(Mw(n.label,e.Diagnostics.Duplicate_label_0,e.getTextOfNode(n.label)),!0)}),Zk(n.statement)}function qN(n){Kw(n)||e.isIdentifier(n.expression)&&!n.expression.escapedText&&Jw(n,e.Diagnostics.Line_break_not_permitted_here),n.expression&&eC(n.expression)}function XN(n){Kw(n),OI(n.tryBlock);var t=n.catchClause;if(t){if(t.variableDeclaration){var r=t.variableDeclaration,i=e.getEffectiveTypeAnnotationNode(e.getRootDeclaration(r));if(i){var a=ts(r,!1);!a||3&a.flags||Ow(i,e.Diagnostics.Catch_clause_variable_type_annotation_must_be_any_or_unknown_if_specified)}else if(r.initializer)Ow(r.initializer,e.Diagnostics.Catch_clause_variable_cannot_have_an_initializer);else{var o=t.block.locals;o&&e.forEachKey(t.locals,function(n){var t=o.get(n);(null===t||void 0===t?void 0:t.valueDeclaration)&&0!==(2&t.flags)&&Mw(t.valueDeclaration,e.Diagnostics.Cannot_redeclare_identifier_0_in_catch_clause,n)})}}OI(t.block)}n.finallyBlock&&OI(n.finallyBlock)}function YN(n,t){var r=Ru(n);if(0!==r.length){for(var i=0,a=Vl(n);i<a.length;i++){var o=a[i];t&&4194304&o.flags||QN(n,o,Gf(o,8576,!0),zs(o))}var s=n.symbol.valueDeclaration;if(s&&e.isClassLike(s))for(var c=0,l=s.members;c<l.length;c++){var u=l[c];if(!e.isStatic(u)&&!Vc(u)){var d=Ca(u);QN(n,d,QA(u.name.expression),zs(d))}}if(r.length>1)for(var p=0,f=r;p<f.length;p++){var m=f[p];ZN(n,m)}}}function QN(n,t,r,i){var a=t.valueDeclaration,o=e.getNameOfDeclaration(a);if(!o||!e.isPrivateIdentifier(o))for(var s=Pu(n,r),c=2&e.getObjectFlags(n)?e.getDeclarationOfKind(n.symbol,256):void 0,l=a&&219===a.kind||o&&160===o.kind||Ia(t)===n.symbol?a:void 0,u=function(r){var a=r.declaration&&Ia(Ca(r.declaration))===n.symbol?r.declaration:void 0,o=l||a||(c&&!e.some(lc(n),function(e){return!!jl(e,t.escapedName)&&!!wu(e,r.keyType)})?c:void 0);o&&!lg(i,r.type)&&Dr(o,e.Diagnostics.Property_0_of_type_1_is_not_assignable_to_2_index_type_3,mo(t),go(i),go(r.keyType),go(r.type))},d=0,p=s;d<p.length;d++){var f=p[d];u(f)}}function ZN(n,t){for(var r=t.declaration,i=Pu(n,t.keyType),a=2&e.getObjectFlags(n)?e.getDeclarationOfKind(n.symbol,256):void 0,o=r&&Ia(Ca(r))===n.symbol?r:void 0,s=function(r){if(r===t)return"continue";var i=r.declaration&&Ia(Ca(r.declaration))===n.symbol?r.declaration:void 0,s=o||i||(a&&!e.some(lc(n),function(e){return!!Ou(e,t.keyType)&&!!wu(e,r.keyType)})?a:void 0);s&&!lg(t.type,r.type)&&Dr(s,e.Diagnostics._0_index_type_1_is_not_assignable_to_2_index_type_3,go(t.keyType),go(t.type),go(r.keyType),go(r.type))},c=0,l=i;c<l.length;c++){var u=l[c];s(u)}}function $N(e,n){switch(e.escapedText){case"any":case"unknown":case"never":case"number":case"bigint":case"boolean":case"string":case"symbol":case"void":case"object":Dr(e,n,e.escapedText)}}function ek(n){1===F&&"Object"===n.escapedText&&K<e.ModuleKind.ES2015&&Dr(n,e.Diagnostics.Class_name_cannot_be_Object_when_targeting_ES5_with_module_0,e.ModuleKind[K])}function nk(n){if(n)for(var r=!1,i=0;i<n.length;i++){var a=n[i];if(iC(a),t){a.default?(r=!0,tk(a.default,n,i)):r&&Dr(a,e.Diagnostics.Required_type_parameters_may_not_follow_optional_type_parameters);for(var o=0;o<i;o++)n[o].symbol===a.symbol&&Dr(a.name,e.Diagnostics.Duplicate_identifier_0,e.declarationNameToString(a.name))}}}function tk(n,t,r){function i(n){if(176===n.kind){var a=ep(n);if(262144&a.flags)for(var o=r;o<t.length;o++)a.symbol===Ca(t[o])&&Dr(n,e.Diagnostics.Type_parameter_defaults_can_only_reference_previously_declared_type_parameters)}e.forEachChild(n,i)}i(n)}function rk(n){if(!n.declarations||1!==n.declarations.length){var t=Hr(n);if(!t.typeParametersChecked){t.typeParametersChecked=!0;var r=fk(n);if(!r||r.length<=1)return;var i=Dc(n);if(!ik(r,i.localTypeParameters))for(var a=mo(n),o=0,s=r;o<s.length;o++){var c=s[o];Dr(c.name,e.Diagnostics.All_declarations_of_0_must_have_identical_type_parameters,a)}}}}function ik(n,t){for(var r=e.length(t),i=zu(t),a=0,o=n;a<o.length;a++){var s=o[a],c=e.getEffectiveTypeParameterDeclarations(s),l=c.length;if(l<i||l>r)return!1;for(var u=0;u<l;u++){var d=c[u],p=t[u];if(d.name.escapedText!==p.symbol.escapedName)return!1;var f=e.getEffectiveConstraintOfTypeParameter(d),m=f&&u_(f),_=ql(p);if(m&&_&&!ig(m,_))return!1;var g=d.default&&u_(d.default),y=lu(p);if(g&&y&&!ig(g,y))return!1}}return!0}function ak(e){return ck(e),rR(e),Ws(Ca(e))}function ok(n){e.forEach(n.members,Zk),hI(n)}function sk(n){e.some(n.decorators)&&e.some(n.members,function(n){return e.hasStaticModifier(n)&&e.isPrivateIdentifierClassElementDeclaration(n)})&&Mw(n.decorators[0],e.Diagnostics.Class_decorators_can_t_be_used_with_static_private_identifier_Consider_removing_the_experimental_decorator),n.name||e.hasSyntacticModifier(n,512)||Ow(n,e.Diagnostics.A_class_declaration_without_the_default_modifier_must_have_a_name),ck(n),e.forEach(n.members,Zk),hI(n)}function ck(n){WO(n),oI(n),HI(n,n.name),nk(e.getEffectiveTypeParameterDeclarations(n)),WC(n);var r=Ca(n),i=Dc(r),a=Xc(i),o=Ws(r);rk(r),HC(r),uC(n);var s=!!(8388608&n.flags);s||dC(n);var c=e.getEffectiveBaseTypeNode(n);if(c){e.forEach(c.typeArguments,Zk),F<2&&IO(c.parent,1);var l=e.getClassExtendsHeritageElement(n);l&&l!==c&&eC(l.expression);var u=lc(i);if(u.length&&t){var d=u[0],p=oc(i),f=mu(p);if(dk(f,c),Zk(c.expression),e.some(c.typeArguments)){e.forEach(c.typeArguments,Zk);for(var m=0,_=ic(f,c.typeArguments,c);m<_.length;m++){var g=_[m];if(!SC(c,g.typeParameters))break}}var y=Xc(d,i.thisType);if(fg(a,y,void 0)?fg(o,rg(f),n.name||n,e.Diagnostics.Class_static_side_0_incorrectly_extends_base_class_static_side_1):uk(n,a,y,e.Diagnostics.Class_0_incorrectly_extends_base_class_1),8650752&p.flags)if(nc(o)){var h=Au(p,1);h.some(function(e){return 4&e.flags})&&!e.hasSyntacticModifier(n,128)&&Dr(n.name||n,e.Diagnostics.A_mixin_class_that_extends_from_a_type_variable_containing_an_abstract_construct_signature_must_also_be_declared_abstract)}else Dr(n.name||n,e.Diagnostics.A_mixin_class_must_have_a_constructor_with_a_single_rest_parameter_of_type_any);if(!(f.symbol&&32&f.symbol.flags)&&!(8650752&p.flags)){var v=ac(f,c.typeArguments,c);e.forEach(v,function(e){return!FL(e.declaration)&&!ig(ad(e),d)})&&Dr(c.expression,e.Diagnostics.Base_constructors_must_all_have_the_same_return_type)}mk(i,d)}}lk(n,i,a,o);var b=e.getEffectiveImplementsTypeNodes(n);if(b)for(var E=0,T=b;E<T.length;E++){var S=T[E];if(e.isEntityNameExpression(S.expression)&&!e.isOptionalChain(S.expression)||Dr(S.expression,e.Diagnostics.A_class_can_only_implement_an_identifier_Slashqualified_name_with_optional_type_arguments),LC(S),t){var x=vu(u_(S));if(x!==Re)if(fc(x)){var L=x.symbol&&32&x.symbol.flags?e.Diagnostics.Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass:e.Diagnostics.Class_0_incorrectly_implements_interface_1;y=Xc(x,i.thisType);fg(a,y,void 0)||uk(n,a,y,L)}else Dr(S,e.Diagnostics.A_class_can_only_implement_an_object_type_or_intersection_of_object_types_with_statically_known_members)}}t&&(YN(i),YN(o,!0),fC(n),yk(n))}function lk(n,t,r,i){for(var a=!!(8388608&n.flags),o=e.getEffectiveBaseTypeNode(n),s=o&&lc(t),c=(null===s||void 0===s?void 0:s.length)?Xc(e.first(s),t.thisType):void 0,l=oc(t),u=function(n){if(e.hasAmbientModifier(n))return"continue";e.isConstructorDeclaration(n)&&e.forEach(n.parameters,function(t){e.isParameterPropertyDeclaration(t,n)&&m(t,!0)}),m(n)},d=0,p=n.members;d<p.length;d++){var f=p[d];u(f)}function m(n,o){var s=e.hasOverrideModifier(n),u=e.isStatic(n);if(c&&(s||M.noImplicitOverride)){var d=n.name&&CR(n.name)||CR(n);if(!d)return;var p=u?i:r,f=u?l:c,m=Lu(p,d.escapedName),_=Lu(f,d.escapedName),g=go(c);if(m&&!_&&s){var y=ux(e.symbolName(d),f);y?Dr(n,e.Diagnostics.This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0_Did_you_mean_1,g,mo(y)):Dr(n,e.Diagnostics.This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0,g)}else if(m&&(null===_||void 0===_?void 0:_.declarations)&&M.noImplicitOverride&&!a){var h=e.some(_.declarations,function(n){return e.hasAbstractModifier(n)});if(s)return;if(h)e.hasAbstractModifier(n)&&h&&Dr(n,e.Diagnostics.This_member_must_have_an_override_modifier_because_it_overrides_an_abstract_method_that_is_declared_in_the_base_class_0,g);else{var v=o?e.Diagnostics.This_parameter_property_must_have_an_override_modifier_because_it_overrides_a_member_in_base_class_0:e.Diagnostics.This_member_must_have_an_override_modifier_because_it_overrides_a_member_in_the_base_class_0;Dr(n,v,g)}}}else if(s){var b=go(t);Dr(n,e.Diagnostics.This_member_cannot_have_an_override_modifier_because_its_containing_class_0_does_not_extend_another_class,b)}}}function uk(n,t,r,i){for(var a=!1,o=function(n){if(e.isStatic(n))return"continue";var i=n.name&&CR(n.name)||CR(n);if(i){var o=Lu(t,i.escapedName),s=Lu(r,i.escapedName);if(o&&s){var c=function(){return e.chainDiagnosticMessages(void 0,e.Diagnostics.Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2,mo(i),go(t),go(r))};fg(Ws(o),Ws(s),n.name||n,void 0,c)||(a=!0)}}},s=0,c=n.members;s<c.length;s++){var l=c[s];o(l)}a||fg(t,r,n.name||n,i)}function dk(n,t){var r=Au(n,1);if(r.length){var i=r[0].declaration;if(i&&e.hasEffectiveModifier(i,8)){var a=e.getClassLikeDeclarationOfSymbol(n.symbol);ER(t,a)||Dr(t,e.Diagnostics.Cannot_extend_a_class_0_Class_constructor_is_marked_as_private,ta(n.symbol))}}}function pk(n){return 1&e.getCheckFlags(n)?n.target:n}function fk(n){return e.filter(n.declarations,function(e){return 255===e.kind||256===e.kind})}function mk(n,t){var r,i,a=Hl(t);e:for(var o=0,s=a;o<s.length;o++){var c=s[o],l=pk(c);if(!(4194304&l.flags)){var u=jl(n,l.escapedName);if(u){var d=pk(u),p=e.getDeclarationModifierFlagsFromSymbol(l);if(e.Debug.assert(!!d,"derived should point to something, even if it is the base class' declaration."),d===l){var f=e.getClassLikeDeclarationOfSymbol(n.symbol);if(128&p&&(!f||!e.hasSyntacticModifier(f,128))){for(var m=0,_=lc(n);m<_.length;m++){var g=_[m];if(g!==t){var y=jl(g,l.escapedName),h=y&&pk(y);if(h&&h!==l)continue e}}224===f.kind?Dr(f,e.Diagnostics.Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1,mo(c),go(t)):Dr(f,e.Diagnostics.Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2,go(n),mo(c),go(t))}}else{var v=e.getDeclarationModifierFlagsFromSymbol(d);if(8&p||8&v)continue;var b=void 0,E=98308&l.flags,T=98308&d.flags;if(E&&T){if(128&p&&!(l.valueDeclaration&&e.isPropertyDeclaration(l.valueDeclaration)&&l.valueDeclaration.initializer)||l.valueDeclaration&&256===l.valueDeclaration.parent.kind||d.valueDeclaration&&e.isBinaryExpression(d.valueDeclaration))continue;var S=4!==E&&4===T,x=4===E&&4!==T;if(S||x){var L=S?e.Diagnostics._0_is_defined_as_an_accessor_in_class_1_but_is_overridden_here_in_2_as_an_instance_property:e.Diagnostics._0_is_defined_as_a_property_in_class_1_but_is_overridden_here_in_2_as_an_accessor;Dr(e.getNameOfDeclaration(d.valueDeclaration)||d.valueDeclaration,L,mo(l),go(t),go(n))}else if(H){var D=null===(r=d.declarations)||void 0===r?void 0:r.find(function(e){return 165===e.kind&&!e.initializer});if(D&&!(33554432&d.flags)&&!(128&p)&&!(128&v)&&!(null===(i=d.declarations)||void 0===i?void 0:i.some(function(e){return!!(8388608&e.flags)}))){var A=Ga(e.getClassLikeDeclarationOfSymbol(n.symbol)),C=D.name;if(D.exclamationToken||!A||!e.isIdentifier(C)||!z||!bk(C,n,A)){var I=e.Diagnostics.Property_0_will_overwrite_the_base_property_in_1_If_this_is_intentional_add_an_initializer_Otherwise_add_a_declare_modifier_or_remove_the_redundant_declaration;Dr(e.getNameOfDeclaration(d.valueDeclaration)||d.valueDeclaration,I,mo(l),go(t))}}}continue}if(RS(l)){if(RS(d)||4&d.flags)continue;e.Debug.assert(!!(98304&d.flags)),b=e.Diagnostics.Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_accessor}else b=98304&l.flags?e.Diagnostics.Class_0_defines_instance_member_accessor_1_but_extended_class_2_defines_it_as_instance_member_function:e.Diagnostics.Class_0_defines_instance_member_property_1_but_extended_class_2_defines_it_as_instance_member_function;Dr(e.getNameOfDeclaration(d.valueDeclaration)||d.valueDeclaration,b,go(t),mo(l),go(n))}}}}}function _k(n,t,r){if(!e.length(t))return r;var i=new e.Map;e.forEach(r,function(e){i.set(e.escapedName,e)});for(var a=0,o=t;a<o.length;a++)for(var s=o[a],c=Hl(Xc(s,n.thisType)),l=0,u=c;l<u.length;l++){var d=u[l],p=i.get(d.escapedName);p&&!gy(p,d)&&i.delete(d.escapedName)}return e.arrayFrom(i.values())}function gk(n,t){var r=lc(n);if(r.length<2)return!0;var i=new e.Map;e.forEach(Mc(n).declaredProperties,function(e){i.set(e.escapedName,{prop:e,containingType:n})});for(var a=!0,o=0,s=r;o<s.length;o++)for(var c=s[o],l=Hl(Xc(c,n.thisType)),u=0,d=l;u<d.length;u++){var p=d[u],f=i.get(p.escapedName);if(f){var m=f.containingType!==n;if(m&&!gy(f.prop,p)){a=!1;var _=go(f.containingType),g=go(c),y=e.chainDiagnosticMessages(void 0,e.Diagnostics.Named_property_0_of_types_1_and_2_are_not_identical,mo(p),_,g);y=e.chainDiagnosticMessages(y,e.Diagnostics.Interface_0_cannot_simultaneously_extend_types_1_and_2,go(n),_,g),ur.add(e.createDiagnosticForNodeFromMessageChain(t,y))}}else i.set(p.escapedName,{prop:p,containingType:c})}return a}function yk(n){if(z&&X&&!(8388608&n.flags))for(var t=Ga(n),r=0,i=n.members;r<i.length;r++){var a=i[r];if(!(2&e.getEffectiveModifierFlags(a))&&(!e.isStatic(a)&&hk(a))){var o=a.name;if(e.isIdentifier(o)||e.isPrivateIdentifier(o)){var s=Ws(Ca(a));3&s.flags||32768&th(s)||t&&bk(o,s,t)||Dr(a.name,e.Diagnostics.Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor,e.declarationNameToString(o))}}}}function hk(n){return 165===n.kind&&!e.hasAbstractModifier(n)&&!n.exclamationToken&&!n.initializer}function vk(n,t,r,i,a){for(var o=0,s=r;o<s.length;o++){var c=s[o];if(c.pos>=i&&c.pos<=a){var l=e.factory.createPropertyAccessExpression(e.factory.createThis(),n);e.setParent(l.expression,l),e.setParent(l,c),l.flowNode=c.returnFlowNode;var u=nE(l,t,sh(t));if(!(32768&th(u)))return!0}}return!1}function bk(n,t,r){var i=e.factory.createPropertyAccessExpression(e.factory.createThis(),n);e.setParent(i.expression,i),e.setParent(i,r),i.flowNode=r.returnFlowNode;var a=nE(i,t,sh(t));return!(32768&th(a))}function Ek(n){if(RO(n)||rw(n),nk(n.typeParameters),t){$N(n.name,e.Diagnostics.Interface_name_cannot_be_0),WC(n);var r=Ca(n);rk(r);var i=e.getDeclarationOfKind(r,256);if(n===i){var a=Dc(r),o=Xc(a);if(gk(a,n.name)){for(var s=0,c=lc(a);s<c.length;s++){var l=c[s];fg(o,Xc(l,a.thisType),n.name,e.Diagnostics.Interface_0_incorrectly_extends_interface_1)}YN(a)}}pC(n)}e.forEach(e.getInterfaceBaseTypeNodes(n),function(n){e.isEntityNameExpression(n.expression)&&!e.isOptionalChain(n.expression)||Dr(n.expression,e.Diagnostics.An_interface_can_only_extend_an_identifier_Slashqualified_name_with_optional_type_arguments),LC(n)}),e.forEach(n.members,Zk),t&&(fC(n),hI(n))}function Tk(n){RO(n),$N(n.name,e.Diagnostics.Type_alias_name_cannot_be_0),WC(n),nk(n.typeParameters),137===n.type.kind?N.has(n.name.escapedText)&&1===e.length(n.typeParameters)||Dr(n.type,e.Diagnostics.The_intrinsic_keyword_can_only_be_used_to_declare_compiler_provided_intrinsic_types):(Zk(n.type),hI(n))}function Sk(e){var n=Wr(e);if(!(16384&n.flags)){n.flags|=16384;for(var t=0,r=0,i=e.members;r<i.length;r++){var a=i[r],o=xk(a,t);Wr(a).enumMemberValue=o,t="number"===typeof o?o+1:void 0}}}function xk(n,t){if(e.isComputedNonLiteralName(n.name))Dr(n.name,e.Diagnostics.Computed_property_names_are_not_allowed_in_enums);else{var r=e.getTextOfPropertyName(n.name);HT(r)&&!e.isInfinityOrNaNString(r)&&Dr(n.name,e.Diagnostics.An_enum_member_cannot_have_a_numeric_name)}return n.initializer?Lk(n):8388608&n.parent.flags&&!e.isEnumConst(n.parent)&&0===bc(Ca(n.parent))?void 0:void 0!==t?t:void Dr(n.name,e.Diagnostics.Enum_member_must_have_initializer)}function Lk(n){var t=bc(Ca(n.parent)),r=e.isEnumConst(n.parent),i=n.initializer,a=1!==t||vc(n)?s(i):void 0;if(void 0!==a)r&&"number"===typeof a&&!isFinite(a)&&Dr(i,isNaN(a)?e.Diagnostics.const_enum_member_initializer_was_evaluated_to_disallowed_value_NaN:e.Diagnostics.const_enum_member_initializer_was_evaluated_to_a_non_finite_value);else{if(1===t)return Dr(i,e.Diagnostics.Computed_values_are_not_permitted_in_an_enum_with_string_valued_members),0;if(r)Dr(i,e.Diagnostics.const_enum_member_initializers_can_only_contain_literal_values_and_other_computed_enum_values);else if(8388608&n.parent.flags)Dr(i,e.Diagnostics.In_ambient_enum_declarations_member_initializer_must_be_constant_expression);else{var o=eC(i);aA(o,296)?fg(o,Dc(Ca(n.parent)),i,void 0):Dr(i,e.Diagnostics.Only_numeric_enums_can_have_computed_members_but_this_expression_has_type_0_If_you_do_not_need_exhaustiveness_checks_consider_using_an_object_literal_instead,go(o))}}return a;function s(t){switch(t.kind){case 217:var r=s(t.operand);if("number"===typeof r)switch(t.operator){case 39:return r;case 40:return-r;case 54:return~r}break;case 219:var i=s(t.left),a=s(t.right);if("number"===typeof i&&"number"===typeof a)switch(t.operatorToken.kind){case 51:return i|a;case 50:return i&a;case 48:return i>>a;case 49:return i>>>a;case 47:return i<<a;case 52:return i^a;case 41:return i*a;case 43:return i/a;case 39:return i+a;case 40:return i-a;case 44:return i%a;case 42:return Math.pow(i,a)}else if("string"===typeof i&&"string"===typeof a&&39===t.operatorToken.kind)return i+a;break;case 10:case 14:return t.text;case 8:return Hw(t),+t.text;case 210:return s(t.expression);case 79:var o=t;return e.isInfinityOrNaNString(o.escapedText)?+o.escapedText:e.nodeIsMissing(t)?0:c(t,Ca(n.parent),o.escapedText);case 205:case 204:var l=t;if(Dk(l)){var u=QA(l.expression);if(u.symbol&&384&u.symbol.flags){var d=void 0;return d=204===l.kind?l.name.escapedText:e.escapeLeadingUnderscores(e.cast(l.argumentExpression,e.isLiteralExpression).text),c(t,u.symbol,d)}}break}}function c(t,r,i){var a=r.exports.get(i);if(a){var o=a.valueDeclaration;if(o!==n)return o&&Xr(o,n)?uO(o):(Dr(t,e.Diagnostics.A_member_initializer_in_a_enum_declaration_cannot_reference_members_declared_after_it_including_members_defined_in_other_enums),0);Dr(t,e.Diagnostics.Property_0_is_used_before_being_assigned,mo(a))}}}function Dk(n){return 79===n.kind||204===n.kind&&Dk(n.expression)||205===n.kind&&Dk(n.expression)&&e.isStringLiteralLike(n.argumentExpression)}function Ak(n){if(t){RO(n),HI(n,n.name),WC(n),n.members.forEach(Ck),Sk(n);var r=Ca(n),i=e.getDeclarationOfKind(r,n.kind);if(n===i){if(r.declarations&&r.declarations.length>1){var a=e.isEnumConst(n);e.forEach(r.declarations,function(n){e.isEnumDeclaration(n)&&e.isEnumConst(n)!==a&&Dr(e.getNameOfDeclaration(n),e.Diagnostics.Enum_declarations_must_all_be_const_or_non_const)})}var o=!1;e.forEach(r.declarations,function(n){if(258!==n.kind)return!1;var t=n;if(!t.members.length)return!1;var r=t.members[0];r.initializer||(o?Dr(r.name,e.Diagnostics.In_an_enum_with_multiple_declarations_only_one_declaration_can_omit_an_initializer_for_its_first_enum_element):o=!0)})}}}function Ck(n){e.isPrivateIdentifier(n.name)&&Dr(n,e.Diagnostics.An_enum_member_cannot_be_named_with_a_private_identifier)}function Ik(n){var t=n.declarations;if(t)for(var r=0,i=t;r<i.length;r++){var a=i[r];if((255===a.kind||254===a.kind&&e.nodeIsPresent(a.body))&&!(8388608&a.flags))return a}}function Nk(n,t){var r=e.getEnclosingBlockScopeContainer(n),i=e.getEnclosingBlockScopeContainer(t);return zr(r)?zr(i):!zr(i)&&r===i}function kk(n){if(t){var r=e.isGlobalScopeAugmentation(n),i=8388608&n.flags;r&&!i&&Dr(n.name,e.Diagnostics.Augmentations_for_the_global_scope_should_have_declare_modifier_unless_they_appear_in_already_ambient_context);var a=e.isAmbientModule(n),o=a?e.Diagnostics.An_ambient_module_declaration_is_only_allowed_at_the_top_level_in_a_file:e.Diagnostics.A_namespace_declaration_is_only_allowed_in_a_namespace_or_module;if(Vk(n,o))return;RO(n)||i||10!==n.name.kind||Mw(n.name,e.Diagnostics.Only_ambient_modules_can_use_quoted_names),e.isIdentifier(n.name)&&HI(n,n.name),WC(n);var s=Ca(n);if(512&s.flags&&!i&&s.declarations&&s.declarations.length>1&&P(n,e.shouldPreserveConstEnums(M))){var c=Ik(s);c&&(e.getSourceFileOfNode(n)!==e.getSourceFileOfNode(c)?Dr(n.name,e.Diagnostics.A_namespace_declaration_cannot_be_in_a_different_file_from_a_class_or_function_with_which_it_is_merged):n.pos<c.pos&&Dr(n.name,e.Diagnostics.A_namespace_declaration_cannot_be_located_prior_to_a_class_or_function_with_which_it_is_merged));var l=e.getDeclarationOfKind(s,255);l&&Nk(n,l)&&(Wr(n).flags|=32768)}if(a)if(e.isExternalModuleAugmentation(n)){var u=r||33554432&Ca(n).flags;if(u&&n.body)for(var d=0,p=n.body.statements;d<p.length;d++){var f=p[d];Rk(f,r)}}else zr(n.parent)?r?Dr(n.name,e.Diagnostics.Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_declarations):e.isExternalModuleNameRelative(e.getTextOfIdentifierOrLiteral(n.name))&&Dr(n.name,e.Diagnostics.Ambient_module_declaration_cannot_specify_relative_module_name):Dr(n.name,r?e.Diagnostics.Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_declarations:e.Diagnostics.Ambient_modules_cannot_be_nested_in_other_modules_or_namespaces)}n.body&&(Zk(n.body),e.isGlobalScopeAugmentation(n)||hI(n))}function Rk(n,t){var r;switch(n.kind){case 235:for(var i=0,a=n.declarationList.declarations;i<a.length;i++){var o=a[i];Rk(o,t)}break;case 269:case 270:Ow(n,e.Diagnostics.Exports_and_export_assignments_are_not_permitted_in_module_augmentations);break;case 263:case 264:Ow(n,e.Diagnostics.Imports_are_not_permitted_in_module_augmentations_Consider_moving_them_to_the_enclosing_external_module);break;case 201:case 252:var s=n.name;if(e.isBindingPattern(s)){for(var c=0,l=s.elements;c<l.length;c++){var u=l[c];Rk(u,t)}break}case 255:case 258:case 254:case 256:case 259:case 257:if(t)return;var d=Ca(n);if(d){var p=!(33554432&d.flags);p||(p=!!(null===(r=d.parent)||void 0===r?void 0:r.declarations)&&e.isExternalModuleAugmentation(d.parent.declarations[0]))}break}}function Ok(n){switch(n.kind){case 79:return n;case 159:do{n=n.left}while(79!==n.kind);return n;case 204:do{if(e.isModuleExportsAccessExpression(n.expression)&&!e.isPrivateIdentifier(n.name))return n.name;n=n.expression}while(79!==n.kind);return n}}function wk(n){var t=e.getExternalModuleName(n);if(!t||e.nodeIsMissing(t))return!1;if(!e.isStringLiteral(t))return Dr(t,e.Diagnostics.String_literal_expected),!1;var r=260===n.parent.kind&&e.isAmbientModule(n.parent.parent);return 300===n.parent.kind||r?!(r&&e.isExternalModuleNameRelative(t.text)&&!Ao(n))||(Dr(n,e.Diagnostics.Import_or_export_declaration_in_an_ambient_module_declaration_cannot_reference_module_through_relative_module_name),!1):(Dr(t,270===n.kind?e.Diagnostics.Export_declarations_are_not_permitted_in_a_namespace:e.Diagnostics.Import_declarations_in_a_namespace_cannot_reference_a_module),!1)}function Pk(n){var t,r=Ca(n),i=Ji(r);if(i!==Ae){r=Aa(r.exportSymbol||r);var a=(1160127&r.flags?111551:0)|(788968&r.flags?788968:0)|(1920&r.flags?1920:0);if(i.flags&a){var o=273===n.kind?e.Diagnostics.Export_declaration_conflicts_with_exported_declaration_of_0:e.Diagnostics.Import_declaration_conflicts_with_local_declaration_of_0;Dr(n,o,mo(r))}!M.isolatedModules||273!==n.kind||n.parent.parent.isTypeOnly||111551&i.flags||8388608&n.flags||Dr(n,e.Diagnostics.Re_exporting_a_type_when_the_isolatedModules_flag_is_provided_requires_using_export_type),e.isImportSpecifier(n)&&(null===(t=i.declarations)||void 0===t?void 0:t.every(function(n){return!!(134217728&e.getCombinedNodeFlags(n))}))&&kr(n.name,i.declarations,r.escapedName)}}function Mk(n){HI(n,n.name),Pk(n),268===n.kind&&"default"===e.idText(n.propertyName||n.name)&&M.esModuleInterop&&K!==e.ModuleKind.System&&K<e.ModuleKind.ES2015&&IO(n,131072)}function Fk(n){if(!Vk(n,e.Diagnostics.An_import_declaration_can_only_be_used_in_a_namespace_or_module)&&(!RO(n)&&e.hasEffectiveModifiers(n)&&Ow(n,e.Diagnostics.An_import_declaration_cannot_have_modifiers),wk(n))){var t=n.importClause;if(t&&!Xw(t)&&(t.name&&Mk(t),t.namedBindings))if(266===t.namedBindings.kind)Mk(t.namedBindings),K!==e.ModuleKind.System&&K<e.ModuleKind.ES2015&&M.esModuleInterop&&IO(n,65536);else{var r=ca(n,n.moduleSpecifier);r&&e.forEach(t.namedBindings.elements,Mk)}}}function Gk(n){if(!Vk(n,e.Diagnostics.An_import_declaration_can_only_be_used_in_a_namespace_or_module)&&(RO(n),e.isInternalModuleImportEqualsDeclaration(n)||wk(n)))if(Mk(n),e.hasSyntacticModifier(n,1)&&Zi(n),275!==n.moduleReference.kind){var t=Ji(Ca(n));if(t!==Ae){if(111551&t.flags){var r=e.getFirstIdentifier(n.moduleReference);1920&ra(r,112575).flags||Dr(r,e.Diagnostics.Module_0_is_hidden_by_a_local_declaration_with_the_same_name,e.declarationNameToString(r))}788968&t.flags&&$N(n.name,e.Diagnostics.Import_name_cannot_be_0)}n.isTypeOnly&&Mw(n,e.Diagnostics.An_import_alias_cannot_use_import_type)}else!(K>=e.ModuleKind.ES2015)||n.isTypeOnly||8388608&n.flags||Mw(n,e.Diagnostics.Import_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_import_Asterisk_as_ns_from_mod_import_a_from_mod_import_d_from_mod_or_another_module_format_instead)}function Bk(n){if(!Vk(n,e.Diagnostics.An_export_declaration_can_only_be_used_in_a_module)&&(!RO(n)&&e.hasEffectiveModifiers(n)&&Ow(n,e.Diagnostics.An_export_declaration_cannot_have_modifiers),n.moduleSpecifier&&n.exportClause&&e.isNamedExports(n.exportClause)&&e.length(n.exportClause.elements)&&0===F&&IO(n,2097152),Uk(n),!n.moduleSpecifier||wk(n)))if(n.exportClause&&!e.isNamespaceExport(n.exportClause)){e.forEach(n.exportClause.elements,Jk);var t=260===n.parent.kind&&e.isAmbientModule(n.parent.parent),r=!t&&260===n.parent.kind&&!n.moduleSpecifier&&8388608&n.flags;300===n.parent.kind||t||r||Dr(n,e.Diagnostics.Export_declarations_are_not_permitted_in_a_namespace)}else{var i=ca(n,n.moduleSpecifier);i&&ga(i)?Dr(n.moduleSpecifier,e.Diagnostics.Module_0_uses_export_and_cannot_be_used_with_export_Asterisk,mo(i)):n.exportClause&&Pk(n.exportClause),K!==e.ModuleKind.System&&K<e.ModuleKind.ES2015&&(n.exportClause?M.esModuleInterop&&IO(n,65536):IO(n,32768))}}function Uk(n){var t,r=n.isTypeOnly&&271!==(null===(t=n.exportClause)||void 0===t?void 0:t.kind);return r&&Mw(n,e.Diagnostics.Only_named_exports_may_use_export_type),!r}function Vk(e,n){var t=300===e.parent.kind||260===e.parent.kind||259===e.parent.kind;return t||Ow(e,n),!t}function jk(n){return e.forEachImportClauseDeclaration(n,function(e){return!!Ca(e).isReferenced})}function Kk(n){return e.forEachImportClauseDeclaration(n,function(e){return!!Hr(Ca(e)).constEnumReferenced})}function Hk(n){return e.isImportDeclaration(n)&&n.importClause&&!n.importClause.isTypeOnly&&jk(n.importClause)&&!tO(n.importClause,!0)&&!Kk(n.importClause)}function Wk(n){return e.isImportEqualsDeclaration(n)&&e.isExternalModuleReference(n.moduleReference)&&!n.isTypeOnly&&Ca(n).isReferenced&&!tO(n,!1)&&!Hr(Ca(n)).constEnumReferenced}function zk(n){for(var t=0,r=n.statements;t<r.length;t++){var i=r[t];(Hk(i)||Wk(i))&&Dr(i,e.Diagnostics.This_import_is_never_used_as_a_value_and_must_use_import_type_because_importsNotUsedAsValues_is_set_to_error)}}function Jk(n){if(Pk(n),e.getEmitDeclarations(M)&&Ro(n.propertyName||n.name,!0),n.parent.parent.moduleSpecifier)M.esModuleInterop&&K!==e.ModuleKind.System&&K<e.ModuleKind.ES2015&&"default"===e.idText(n.propertyName||n.name)&&IO(n,131072);else{var t=n.propertyName||n.name,r=Qr(t,t.escapedText,2998271,void 0,void 0,!0);if(r&&(r===oe||r===se||r.declarations&&zr(Fo(r.declarations[0]))))Dr(t,e.Diagnostics.Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module,e.idText(t));else{Zi(n);var i=r&&(2097152&r.flags?Ji(r):r);(!i||i===Ae||111551&i.flags)&&kA(n.propertyName||n.name)}}}function qk(n){var t=n.isExportEquals?e.Diagnostics.An_export_assignment_must_be_at_the_top_level_of_a_file_or_module_declaration:e.Diagnostics.A_default_export_must_be_at_the_top_level_of_a_file_or_module_declaration;if(!Vk(n,t)){var r=300===n.parent.kind?n.parent:n.parent.parent;if(259!==r.kind||e.isAmbientModule(r)){if(!RO(n)&&e.hasEffectiveModifiers(n)&&Ow(n,e.Diagnostics.An_export_assignment_cannot_have_modifiers),79===n.expression.kind){var i=n.expression,a=ra(i,67108863,!0,!0,n);if(a){_E(a,i);var o=2097152&a.flags?Ji(a):a;(o===Ae||111551&o.flags)&&kA(n.expression)}else kA(n.expression);e.getEmitDeclarations(M)&&Ro(n.expression,!0)}else kA(n.expression);Yk(r),8388608&n.flags&&!e.isEntityNameExpression(n.expression)&&Mw(n.expression,e.Diagnostics.The_expression_of_an_export_assignment_must_be_an_identifier_or_qualified_name_in_an_ambient_context),!n.isExportEquals||8388608&n.flags||(K>=e.ModuleKind.ES2015?Mw(n,e.Diagnostics.Export_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_export_default_or_another_module_format_instead):K===e.ModuleKind.System&&Mw(n,e.Diagnostics.Export_assignment_is_not_supported_when_module_flag_is_system))}else n.isExportEquals?Dr(n,e.Diagnostics.An_export_assignment_cannot_be_used_in_a_namespace):Dr(n,e.Diagnostics.A_default_export_can_only_be_used_in_an_ECMAScript_style_module)}}function Xk(n){return e.forEachEntry(n.exports,function(e,n){return"export="!==n})}function Yk(n){var t=Ca(n),r=Hr(t);if(!r.exportsChecked){var i=t.exports.get("export=");if(i&&Xk(t)){var a=hi(i)||i.valueDeclaration;!a||Ao(a)||e.isInJSFile(a)||Dr(a,e.Diagnostics.An_export_assignment_cannot_be_used_in_a_module_with_other_exported_elements)}var o=xa(t);o&&o.forEach(function(n,t){var r=n.declarations,i=n.flags;if("__export"!==t&&!(1984&i)){var a=e.countWhere(r,C);if(!(524288&i&&a<=2)&&a>1&&!Qk(r))for(var o=0,s=r;o<s.length;o++){var c=s[o];G(c)&&ur.add(e.createDiagnosticForNode(c,e.Diagnostics.Cannot_redeclare_exported_variable_0,e.unescapeLeadingUnderscores(t)))}}}),r.exportsChecked=!0}}function Qk(n){return n&&n.length>1&&n.every(function(n){return e.isInJSFile(n)&&e.isAccessExpression(n)&&(e.isExportsIdentifier(n.expression)||e.isModuleExportsAccessExpression(n.expression))})}function Zk(e){if(e){var n=d;d=e,x=0,$k(e),d=n}}function $k(n){e.isInJSFile(n)&&e.forEach(n.jsDoc,function(n){var t=n.tags;return e.forEach(t,Zk)});var t=n.kind;if(r)switch(t){case 259:case 255:case 256:case 254:r.throwIfCancellationRequested()}switch(t>=235&&t<=251&&n.flowNode&&!Yb(n.flowNode)&&Cr(!1===M.allowUnreachableCode,n,e.Diagnostics.Unreachable_code_detected),t){case 161:return iC(n);case 162:return aC(n);case 165:return mC(n);case 164:return _C(n);case 178:case 177:case 172:case 173:case 174:return lC(n);case 167:case 166:return gC(n);case 168:return hC(n);case 169:return vC(n);case 170:case 171:return bC(n);case 176:return LC(n);case 175:return oC(n);case 179:return AC(n);case 180:return CC(n);case 181:return IC(n);case 182:return NC(n);case 185:case 186:return kC(n);case 189:case 183:case 184:return Zk(n.type);case 190:return PC(n);case 191:return MC(n);case 187:return FC(n);case 188:return GC(n);case 196:return BC(n);case 198:return UC(n);case 195:return VC(n);case 323:return _I(n);case 324:return mI(n);case 340:case 333:case 334:return cI(n);case 339:return lI(n);case 338:return uI(n);case 335:return dI(n);case 342:return pI(n);case 312:fI(n);case 310:case 309:case 307:case 308:case 317:return eR(n),void e.forEachChild(n,Zk);case 313:return void nR(n);case 304:return Zk(n.type);case 192:return OC(n);case 193:return wC(n);case 254:return sI(n);case 233:case 260:return OI(n);case 235:return ZI(n);case 236:return $I(n);case 237:return eN(n);case 238:return iN(n);case 239:return aN(n);case 240:return cN(n);case 241:return uN(n);case 242:return lN(n);case 243:case 244:return VN(n);case 245:return HN(n);case 246:return WN(n);case 247:return zN(n);case 248:return JN(n);case 249:return qN(n);case 250:return XN(n);case 252:return YI(n);case 201:return QI(n);case 255:return sk(n);case 256:return Ek(n);case 257:return Tk(n);case 258:return Ak(n);case 259:return kk(n);case 264:return Fk(n);case 263:return Gk(n);case 270:return Bk(n);case 269:return qk(n);case 234:case 251:return void Kw(n);case 274:return EC(n)}}function eR(n){e.isInJSFile(n)||Mw(n,e.Diagnostics.JSDoc_types_can_only_be_used_inside_documentation_comments)}function nR(n){eR(n),Zk(n.type);var t=n.parent;if(e.isParameter(t)&&e.isJSDocFunctionType(t.parent))e.last(t.parent.parameters)!==t&&Dr(n,e.Diagnostics.A_rest_parameter_must_be_last_in_a_parameter_list);else{e.isJSDocTypeExpression(t)||Dr(n,e.Diagnostics.JSDoc_may_only_appear_in_the_last_parameter_of_a_signature);var r=n.parent.parent;if(e.isJSDocParameterTag(r)){var i=e.getParameterSymbolFromJSDoc(r);if(i){var a=e.getHostSignatureFromJSDoc(r);a&&e.last(a.parameters).symbol===i||Dr(n,e.Diagnostics.A_rest_parameter_must_be_last_in_a_parameter_list)}}else Dr(n,e.Diagnostics.JSDoc_may_only_appear_in_the_last_parameter_of_a_signature)}}function tR(n){var t=u_(n.type),r=n.parent,i=n.parent.parent;if(e.isJSDocTypeExpression(n.parent)&&e.isJSDocParameterTag(i)){var a=e.getHostSignatureFromJSDoc(i),o=e.isJSDocCallbackTag(i.parent.parent);if(a||o){var s=o?e.lastOrUndefined(i.parent.parent.typeExpression.parameters):e.lastOrUndefined(a.parameters),c=e.getParameterSymbolFromJSDoc(i);if(!s||c&&s.symbol===c&&e.isRestParameter(s))return Mp(t)}}return e.isParameter(r)&&e.isJSDocFunctionType(r.parent)?Mp(t):ns(t)}function rR(n){var t=e.getSourceFileOfNode(n),r=Wr(t);if(!(1&r.flags)){r.deferredNodes=r.deferredNodes||new e.Map;var i=O(n);r.deferredNodes.set(i,n)}}function iR(e){var n=Wr(e);n.deferredNodes&&n.deferredNodes.forEach(aR)}function aR(n){null===e.tracing||void 0===e.tracing||e.tracing.push("check","checkDeferredNode",{kind:n.kind,pos:n.pos,end:n.end});var t=d;switch(d=n,x=0,n.kind){case 206:case 207:case 208:case 163:case 278:Rx(n);break;case 211:case 212:case 167:case 166:HD(n);break;case 170:case 171:bC(n);break;case 224:ok(n);break;case 277:ZT(n);break;case 276:eS(n);break}d=t,null===e.tracing||void 0===e.tracing||e.tracing.pop()}function oR(n){null===e.tracing||void 0===e.tracing||e.tracing.push("check","checkSourceFile",{path:n.path},!0),e.performance.mark("beforeCheck"),lR(n),e.performance.mark("afterCheck"),e.performance.measure("Check","beforeCheck","afterCheck"),null===e.tracing||void 0===e.tracing||e.tracing.pop()}function sR(n,t){if(t)return!1;switch(n){case 0:return!!M.noUnusedLocals;case 1:return!!M.noUnusedParameters;default:return e.Debug.assertNever(n)}}function cR(n){return wt.get(n.path)||e.emptyArray}function lR(t){var r=Wr(t);if(!(1&r.flags)){if(e.skipTypeChecking(t,M,n))return;jw(t),e.clear(ar),e.clear(or),e.clear(sr),e.clear(cr),e.forEach(t.statements,Zk),Zk(t.endOfFileToken),iR(t),e.isExternalOrCommonJsModule(t)&&hI(t),t.isDeclarationFile||!M.noUnusedLocals&&!M.noUnusedParameters||vI(cR(t),function(n,t,r){!e.containsParseError(n)&&sR(t,!!(8388608&n.flags))&&ur.add(r)}),2===M.importsNotUsedAsValues&&!t.isDeclarationFile&&e.isExternalModule(t)&&zk(t),e.isExternalOrCommonJsModule(t)&&Yk(t),ar.length&&(e.forEach(ar,MI),e.clear(ar)),or.length&&(e.forEach(or,FI),e.clear(or)),sr.length&&(e.forEach(sr,VI),e.clear(sr)),cr.length&&(e.forEach(cr,KI),e.clear(cr)),r.flags|=1}}function uR(e,n){try{return r=n,dR(e)}finally{r=void 0}}function dR(t){if(fR(),t){var r=ur.getGlobalDiagnostics(),i=r.length;oR(t);var a=ur.getDiagnostics(t.fileName),o=ur.getGlobalDiagnostics();if(o!==r){var s=e.relativeComplement(r,o,e.compareDiagnostics);return e.concatenate(s,a)}return 0===i&&o.length>0?e.concatenate(o,a):a}return e.forEach(n.getSourceFiles(),oR),ur.getDiagnostics()}function pR(){return fR(),ur.getGlobalDiagnostics()}function fR(){if(!t)throw new Error("Trying to get diagnostics from a type checker that does not produce them.")}function mR(n,t){if(16777216&n.flags)return[];var r=e.createSymbolTable(),i=!1;return a(),r.delete("this"),Bu(r);function a(){while(n){switch(n.locals&&!zr(n)&&s(n.locals,t),n.kind){case 300:if(!e.isExternalModule(n))break;case 259:c(Ca(n).exports,2623475&t);break;case 258:s(Ca(n).exports,8&t);break;case 224:var r=n.name;r&&o(n.symbol,t);case 255:case 256:i||s(Jc(Ca(n)),788968&t);break;case 211:var a=n.name;a&&o(n.symbol,t);break}e.introducesArgumentsExoticObject(n)&&o(le,t),i=e.isStatic(n),n=n.parent}s(ae,t)}function o(n,t){if(e.getCombinedLocalAndExportSymbolFlags(n)&t){var i=n.escapedName;r.has(i)||r.set(i,n)}}function s(e,n){n&&e.forEach(function(e){o(e,n)})}function c(n,t){t&&n.forEach(function(n){e.getDeclarationOfKind(n,273)||e.getDeclarationOfKind(n,272)||o(n,t)})}}function _R(n){return 79===n.kind&&gR(n.parent)&&e.getNameOfDeclaration(n.parent)===n}function gR(e){switch(e.kind){case 161:case 255:case 256:case 257:case 258:case 340:case 333:case 334:return!0;case 265:return e.isTypeOnly;case 268:case 273:return e.parent.parent.isTypeOnly;default:return!1}}function yR(e){while(159===e.parent.kind)e=e.parent;return 176===e.parent.kind}function hR(e){while(204===e.parent.kind)e=e.parent;return 226===e.parent.kind}function vR(n,t){var r;while(1){if(n=e.getContainingClass(n),!n)break;if(r=t(n))break}return r}function bR(n){return!!e.findAncestor(n,function(n){return!!(e.isConstructorDeclaration(n)&&e.nodeIsPresent(n.body)||e.isPropertyDeclaration(n))||!(!e.isClassLike(n)&&!e.isFunctionLikeDeclaration(n))&&"quit"})}function ER(e,n){return!!vR(e,function(e){return e===n})}function TR(e){while(159===e.parent.kind)e=e.parent;return 263===e.parent.kind?e.parent.moduleReference===e?e.parent:void 0:269===e.parent.kind&&e.parent.expression===e?e.parent:void 0}function SR(e){return void 0!==TR(e)}function xR(n){var t=e.getAssignmentDeclarationKind(n.parent.parent);switch(t){case 1:case 3:return Ca(n.parent);case 4:case 2:case 5:return Ca(n.parent.parent)}}function LR(n){var t=n.parent;while(e.isQualifiedName(t))n=t,t=t.parent;if(t&&198===t.kind&&t.qualifier===n)return t}function DR(n){if(e.isDeclarationName(n))return Ca(n.parent);if(e.isInJSFile(n)&&204===n.parent.kind&&n.parent===n.parent.parent.left&&!e.isPrivateIdentifier(n)&&!e.isJSDocMemberName(n)){var t=xR(n);if(t)return t}if(269===n.parent.kind&&e.isEntityNameExpression(n)){var r=ra(n,2998271,!0);if(r&&r!==Ae)return r}else if(e.isEntityName(n)&&SR(n)){var i=e.getAncestor(n,263);return e.Debug.assert(void 0!==i),na(n,!0)}if(e.isEntityName(n)){var a=LR(n);if(a){u_(a);var o=Wr(n).resolvedSymbol;return o===Ae?void 0:o}}while(e.isRightSideOfQualifiedNameOrPropertyAccessOrJSDocMemberName(n))n=n.parent;if(hR(n)){var s=0;226===n.parent.kind?(s=788968,e.isExpressionWithTypeArgumentsInClassExtendsClause(n.parent)&&(s|=111551)):s=1920,s|=2097152;var c=e.isEntityNameExpression(n)?ra(n,s):void 0;if(c)return c}if(335===n.parent.kind)return e.getParameterSymbolFromJSDoc(n.parent);if(161===n.parent.kind&&339===n.parent.parent.kind){e.Debug.assert(!e.isInJSFile(n));var l=e.getTypeParameterFromJsDoc(n.parent);return l&&l.symbol}if(e.isExpressionNode(n)){if(e.nodeIsMissing(n))return;var u=e.findAncestor(n,e.or(e.isJSDocLinkLike,e.isJSDocNameReference,e.isJSDocMemberName));s=u?901119:111551;if(79===n.kind){if(e.isJSXTagName(n)&&iS(n)){var d=dS(n.parent);return d===Ae?void 0:d}var p=ra(n,s,!1,!u,e.getHostSignatureFromJSDoc(n));if(!p&&u){var f=e.findAncestor(n,e.or(e.isClassLike,e.isInterfaceDeclaration));if(f)return AR(n,Ca(f))}return p}if(204===n.kind||159===n.kind){var m=Wr(n);return m.resolvedSymbol?m.resolvedSymbol:(204===n.kind?HS(n,0):zS(n,0),!m.resolvedSymbol&&u&&e.isQualifiedName(n)?AR(n):m.resolvedSymbol)}if(e.isJSDocMemberName(n))return AR(n)}else if(yR(n)){s=176===n.parent.kind?788968:1920;return ra(n,s,!1,!0)}return 175===n.parent.kind?ra(n,1):void 0}function AR(n,t){if(e.isEntityName(n)){var r=901119,i=ra(n,r,!1,!0,e.getHostSignatureFromJSDoc(n));if(!i&&e.isIdentifier(n)&&t&&(i=Aa(Jr(Sa(t),n.escapedText,r))),i)return i}var a=e.isIdentifier(n)?t:AR(n.left),o=e.isIdentifier(n)?n.escapedText:n.right.escapedText;if(a){var s=111551&a.flags&&Lu(Ws(a),"prototype"),c=s?Ws(s):Dc(a);return Lu(c,o)}}function CR(n,t){if(300===n.kind)return e.isExternalModule(n)?Aa(n.symbol):void 0;var r=n.parent,i=r.parent;if(!(16777216&n.flags)){if(B(n)){var a=Ca(r);return e.isImportOrExportSpecifier(n.parent)&&n.parent.propertyName===n?XT(a):a}if(e.isLiteralComputedPropertyDeclarationName(n))return Ca(r.parent);if(79===n.kind){if(SR(n))return DR(n);if(201===r.kind&&199===i.kind&&n===r.propertyName){var o=RR(i),s=Lu(o,n.escapedText);if(s)return s}else if(e.isMetaProperty(r)){var c=RR(r);s=Lu(c,n.escapedText);if(s)return s;if(103===r.keywordToken)return rD(r).symbol}}switch(n.kind){case 79:case 80:case 204:case 159:return DR(n);case 108:var l=e.getThisContainer(n,!1);if(e.isFunctionLike(l)){var u=qu(l);if(u.thisParameter)return u.thisParameter}if(e.isInExpressionContext(n))return eC(n).symbol;case 190:return o_(n).symbol;case 106:return eC(n).symbol;case 133:var d=n.parent;return d&&169===d.kind?d.parent.symbol:void 0;case 10:case 14:if(e.isExternalModuleImportEqualsDeclaration(n.parent.parent)&&e.getExternalModuleImportEqualsDeclarationExpression(n.parent.parent)===n||(264===n.parent.kind||270===n.parent.kind)&&n.parent.moduleSpecifier===n||e.isInJSFile(n)&&e.isRequireCall(n.parent,!1)||e.isImportCall(n.parent)||e.isLiteralTypeNode(n.parent)&&e.isLiteralImportTypeNode(n.parent.parent)&&n.parent.parent.argument===n.parent)return ca(n,n,t);if(e.isCallExpression(r)&&e.isBindableObjectDefinePropertyCall(r)&&r.arguments[1]===n)return Ca(r);case 8:var p=e.isElementAccessExpression(r)?r.argumentExpression===n?QA(r.expression):void 0:e.isLiteralTypeNode(r)&&e.isIndexedAccessTypeNode(i)?u_(i.objectType):void 0;return p&&Lu(p,e.escapeLeadingUnderscores(n.text));case 88:case 98:case 38:case 84:return Ca(n.parent);case 198:return e.isLiteralImportTypeNode(n)?CR(n.argument.literal,t):void 0;case 93:return e.isExportAssignment(n.parent)?e.Debug.checkDefined(n.parent.symbol):void 0;case 100:case 103:return e.isMetaProperty(n.parent)?tD(n.parent).symbol:void 0;case 229:return eC(n).symbol;default:return}}}function IR(n){if(e.isIdentifier(n)&&e.isPropertyAccessExpression(n.parent)&&n.parent.name===n){var t=Ff(n),r=QA(n.parent.expression),i=1048576&r.flags?r.types:[r];return e.flatMap(i,function(n){return e.filter(Ru(n),function(e){return Nu(t,e.keyType)})})}}function NR(e){if(e&&292===e.kind)return ra(e.name,2208703)}function kR(n){return e.isExportSpecifier(n)?n.parent.parent.moduleSpecifier?Oi(n.parent.parent,n):ra(n.propertyName||n.name,2998271):ra(n,2998271)}function RR(n){if(e.isSourceFile(n)&&!e.isExternalModule(n))return Re;if(16777216&n.flags)return Re;var t=e.tryGetClassImplementingOrExtendingExpressionWithTypeArguments(n),r=t&&gc(Ca(t.class));if(e.isPartOfTypeNode(n)){var i=u_(n);return r?Xc(i,r.thisType):i}if(e.isExpressionNode(n))return PR(n);if(r&&!t.isImplements){var a=e.firstOrUndefined(lc(r));return a?Xc(a,r.thisType):Re}if(gR(n)){var o=Ca(n);return Dc(o)}if(_R(n)){o=CR(n);return o?Dc(o):Re}if(e.isDeclaration(n)){o=Ca(n);return Ws(o)}if(B(n)){o=CR(n);return o?Ws(o):Re}if(e.isBindingPattern(n))return ts(n.parent,!0)||Re;if(SR(n)){o=CR(n);if(o){var s=Dc(o);return s!==Re?s:Ws(o)}}return e.isMetaProperty(n.parent)&&n.parent.keywordToken===n.kind?tD(n.parent):Re}function OR(n){if(e.Debug.assert(203===n.kind||202===n.kind),242===n.parent.kind){var t=pN(n.parent);return _A(n,t||Re)}if(219===n.parent.kind){t=QA(n.parent.right);return _A(n,t||Re)}if(291===n.parent.kind){var r=e.cast(n.parent.parent,e.isObjectLiteralExpression),i=OR(r)||Re,a=e.indexOfNode(r.properties,n.parent);return pA(r,i,a)}var o=e.cast(n.parent,e.isArrayLiteralExpression),s=OR(o)||Re,c=fN(65,s,Me,n.parent)||Re;return mA(o,s,o.elements.indexOf(n),c)}function wR(n){var t=OR(e.cast(n.parent.parent,e.isAssignmentPattern));return t&&Lu(t,n.escapedText)}function PR(n){return e.isRightSideOfQualifiedNameOrPropertyAccess(n)&&(n=n.parent),Ym(QA(n))}function MR(n){var t=Ca(n.parent);return e.isStatic(n)?Ws(t):Dc(t)}function FR(n){var t=n.name;switch(t.kind){case 79:return Zm(e.idText(t));case 8:case 10:return Zm(t.text);case 160:var r=WT(t);return aA(r,12288)?r:je;default:return e.Debug.fail("Unsupported property name.")}}function GR(n){n=mu(n);var t=e.createSymbolTable(Hl(n)),r=Au(n,0).length?In:Au(n,1).length?Nn:void 0;return r&&e.forEach(Hl(r),function(e){t.has(e.escapedName)||t.set(e.escapedName,e)}),za(t)}function BR(n){return e.typeHasCallOrConstructSignatures(n,de)}function UR(n){var t=VR(n);return t?e.flatMap(t,UR):[n]}function VR(n){if(6&e.getCheckFlags(n))return e.mapDefined(Hr(n).containingType.types,function(e){return Lu(e,n.escapedName)});if(33554432&n.flags){var t=n,r=t.leftSpread,i=t.rightSpread,a=t.syntheticOrigin;return r?[r,i]:a?[a]:e.singleElementArray(jR(n))}}function jR(e){var n,t=e;while(t=Hr(t).target)n=t;return n}function KR(n){if(e.isGeneratedIdentifier(n))return!1;var t=e.getParseTreeNode(n,e.isIdentifier);if(!t)return!1;var r=t.parent;if(!r)return!1;var i=(e.isPropertyAccessExpression(r)||e.isPropertyAssignment(r))&&r.name===t;return!i&&vO(t)===le}function HR(n){var t=ca(n.parent,n);if(!t||e.isShorthandAmbientModuleSymbol(t))return!0;var r=ga(t);t=fa(t);var i=Hr(t);return void 0===i.exportsSomeValue&&(i.exportsSomeValue=r?!!(111551&t.flags):e.forEachEntry(xa(t),a)),i.exportsSomeValue;function a(e){return e=zi(e),e&&!!(111551&e.flags)}}function WR(n){return e.isModuleOrEnumDeclaration(n.parent)&&n===n.parent.name}function zR(n,t){var r,i=e.getParseTreeNode(n,e.isIdentifier);if(i){var a=vO(i,WR(i));if(a){if(1048576&a.flags){var o=Aa(a.exportSymbol);if(!t&&944&o.flags&&!(3&o.flags))return;a=o}var s=Ia(a);if(s){if(512&s.flags&&300===(null===(r=s.valueDeclaration)||void 0===r?void 0:r.kind)){var c=s.valueDeclaration,l=e.getSourceFileOfNode(i),u=c!==l;return u?void 0:c}return e.findAncestor(i.parent,function(n){return e.isModuleOrEnumDeclaration(n)&&Ca(n)===s})}}}}function JR(n){if(n.generatedImportReference)return n.generatedImportReference;var t=e.getParseTreeNode(n,e.isIdentifier);if(t){var r=vO(t);if(Wi(r,111551)&&!Qi(r))return hi(r)}}function qR(n){return n.valueDeclaration&&e.isBindingElement(n.valueDeclaration)&&290===e.walkUpBindingElementsAndPatterns(n.valueDeclaration).parent.kind}function XR(n){if(418&n.flags&&n.valueDeclaration&&!e.isSourceFile(n.valueDeclaration)){var t=Hr(n);if(void 0===t.isDeclarationWithCollidingName){var r=e.getEnclosingBlockScopeContainer(n.valueDeclaration);if(e.isStatementWithLocals(r)||qR(n)){var i=Wr(n.valueDeclaration);if(Qr(r.parent,n.escapedName,111551,void 0,void 0,!1))t.isDeclarationWithCollidingName=!0;else if(262144&i.flags){var a=524288&i.flags,o=e.isIterationStatement(r,!1),s=233===r.kind&&e.isIterationStatement(r.parent,!1);t.isDeclarationWithCollidingName=!e.isBlockScopedContainerTopLevel(r)&&(!a||!o&&!s)}else t.isDeclarationWithCollidingName=!1}}return t.isDeclarationWithCollidingName}return!1}function YR(n){if(!e.isGeneratedIdentifier(n)){var t=e.getParseTreeNode(n,e.isIdentifier);if(t){var r=vO(t);if(r&&XR(r))return r.valueDeclaration}}}function QR(n){var t=e.getParseTreeNode(n,e.isDeclaration);if(t){var r=Ca(t);if(r)return XR(r)}return!1}function ZR(n){switch(n.kind){case 263:return eO(Ca(n)||Ae);case 265:case 266:case 268:case 273:var t=Ca(n)||Ae;return eO(t)&&!Qi(t);case 270:var r=n.exportClause;return!!r&&(e.isNamespaceExport(r)||e.some(r.elements,ZR));case 269:return!n.expression||79!==n.expression.kind||eO(Ca(n)||Ae)}return!1}function $R(n){var t=e.getParseTreeNode(n,e.isImportEqualsDeclaration);if(void 0===t||300!==t.parent.kind||!e.isInternalModuleImportEqualsDeclaration(t))return!1;var r=eO(Ca(t));return r&&t.moduleReference&&!e.nodeIsMissing(t.moduleReference)}function eO(n){var t=Ji(n);return t===Ae||!!(111551&t.flags)&&(e.shouldPreserveConstEnums(M)||!nO(t))}function nO(e){return cA(e)||!!e.constEnumOnlyModule}function tO(n,t){if(vi(n)){var r=Ca(n),i=r&&Hr(r);if(null===i||void 0===i?void 0:i.referenced)return!0;var a=Hr(r).target;if(a&&1&e.getEffectiveModifierFlags(n)&&111551&a.flags&&(e.shouldPreserveConstEnums(M)||!nO(a)))return!0}return!!t&&!!e.forEachChild(n,function(e){return tO(e,t)})}function rO(n){if(e.nodeIsPresent(n.body)){if(e.isGetAccessor(n)||e.isSetAccessor(n))return!1;var t=Ca(n),r=$u(t);return r.length>1||1===r.length&&r[0].declaration!==n}return!1}function iO(n){return!!z&&!ju(n)&&!e.isJSDocParameterTag(n)&&!!n.initializer&&!e.hasSyntacticModifier(n,16476)}function aO(n){return z&&ju(n)&&!n.initializer&&e.hasSyntacticModifier(n,16476)}function oO(e){return!!z&&ju(e)&&!e.initializer}function sO(n){var t=e.getParseTreeNode(n,e.isFunctionDeclaration);if(!t)return!1;var r=Ca(t);return!!(r&&16&r.flags)&&!!e.forEachEntry(Sa(r),function(n){return 111551&n.flags&&n.valueDeclaration&&e.isPropertyAccessExpression(n.valueDeclaration)})}function cO(n){var t=e.getParseTreeNode(n,e.isFunctionDeclaration);if(!t)return e.emptyArray;var r=Ca(t);return r&&Hl(Ws(r))||e.emptyArray}function lO(e){var n,t=e.id||0;return t<0||t>=Yt.length?0:(null===(n=Yt[t])||void 0===n?void 0:n.flags)||0}function uO(e){return Sk(e.parent),Wr(e).enumMemberValue}function dO(e){switch(e.kind){case 294:case 204:case 205:return!0}return!1}function pO(n){if(294===n.kind)return uO(n);var t=Wr(n).resolvedSymbol;if(t&&8&t.flags){var r=t.valueDeclaration;if(e.isEnumConst(r.parent))return uO(r)}}function fO(e){return!!(524288&e.flags)&&Au(e,0).length>0}function mO(n,t){var r,i,a=e.getParseTreeNode(n,e.isEntityName);if(!a)return e.TypeReferenceSerializationKind.Unknown;if(t&&(t=e.getParseTreeNode(t),!t))return e.TypeReferenceSerializationKind.Unknown;var o=!1;if(e.isQualifiedName(a)){var s=ra(e.getFirstIdentifier(a),111551,!0,!0,t);o=!!(null===(r=null===s||void 0===s?void 0:s.declarations)||void 0===r?void 0:r.every(e.isTypeOnlyImportOrExportDeclaration))}var c=ra(a,111551,!0,!0,t),l=c&&2097152&c.flags?Ji(c):c;o||(o=!!(null===(i=null===c||void 0===c?void 0:c.declarations)||void 0===i?void 0:i.every(e.isTypeOnlyImportOrExportDeclaration)));var u=ra(a,788968,!0,!1,t);if(l&&l===u){var d=yp(!1);if(d&&l===d)return e.TypeReferenceSerializationKind.Promise;var p=Ws(l);if(p&&tc(p))return o?e.TypeReferenceSerializationKind.TypeWithCallSignature:e.TypeReferenceSerializationKind.TypeWithConstructSignatureAndValue}if(!u)return o?e.TypeReferenceSerializationKind.ObjectType:e.TypeReferenceSerializationKind.Unknown;var f=Dc(u);return f===Re?o?e.TypeReferenceSerializationKind.ObjectType:e.TypeReferenceSerializationKind.Unknown:3&f.flags?e.TypeReferenceSerializationKind.ObjectType:aA(f,245760)?e.TypeReferenceSerializationKind.VoidNullableOrNeverType:aA(f,528)?e.TypeReferenceSerializationKind.BooleanType:aA(f,296)?e.TypeReferenceSerializationKind.NumberLikeType:aA(f,2112)?e.TypeReferenceSerializationKind.BigIntLikeType:aA(f,402653316)?e.TypeReferenceSerializationKind.StringLikeType:Jy(f)?e.TypeReferenceSerializationKind.ArrayLikeType:aA(f,12288)?e.TypeReferenceSerializationKind.ESSymbolType:fO(f)?e.TypeReferenceSerializationKind.TypeWithCallSignature:Ly(f)?e.TypeReferenceSerializationKind.ArrayLikeType:e.TypeReferenceSerializationKind.ObjectType}function _O(n,t,r,i,a){var o=e.getParseTreeNode(n,e.isVariableLikeOrAccessor);if(!o)return e.factory.createToken(129);var s=Ca(o),c=!s||133120&s.flags?Re:jy(Ws(s));return 8192&c.flags&&c.symbol===s&&(r|=1048576),a&&(c=sh(c)),ie.typeToTypeNode(c,t,1024|r,i)}function gO(n,t,r,i){var a=e.getParseTreeNode(n,e.isFunctionLike);if(!a)return e.factory.createToken(129);var o=qu(a);return ie.typeToTypeNode(ad(o),t,1024|r,i)}function yO(n,t,r,i){var a=e.getParseTreeNode(n,e.isExpression);if(!a)return e.factory.createToken(129);var o=Ch(PR(a));return ie.typeToTypeNode(o,t,1024|r,i)}function hO(n){return ae.has(e.escapeLeadingUnderscores(n))}function vO(n,t){var r=Wr(n).resolvedSymbol;if(r)return r;var i=n;if(t){var a=n.parent;e.isDeclaration(a)&&n===a.name&&(i=Fo(a))}return Qr(i,n.escapedText,3257279,void 0,void 0,!0)}function bO(n){if(!e.isGeneratedIdentifier(n)){var t=e.getParseTreeNode(n,e.isIdentifier);if(t){var r=vO(t);if(r)return Ma(r).valueDeclaration}}}function EO(n){return!!(e.isDeclarationReadonly(n)||e.isVariableDeclaration(n)&&e.isVarConst(n))&&Qm(Ws(Ca(n)))}function TO(n,t,r){var i=1024&n.flags?ie.symbolToExpression(n.symbol,111551,t,void 0,r):n===Je?e.factory.createTrue():n===We&&e.factory.createFalse();if(i)return i;var a=n.value;return"object"===typeof a?e.factory.createBigIntLiteral(a):"number"===typeof a?e.factory.createNumericLiteral(a):e.factory.createStringLiteral(a)}function SO(e,n){var t=Ws(Ca(e));return TO(t,e,n)}function xO(n){return n?(Er(n),e.getSourceFileOfNode(n).localJsxFactory||gt):gt}function LO(n){if(n){var t=e.getSourceFileOfNode(n);if(t){if(t.localJsxFragmentFactory)return t.localJsxFragmentFactory;var r=t.pragmas.get("jsxfrag"),i=e.isArray(r)?r[0]:r;if(i)return t.localJsxFragmentFactory=e.parseIsolatedEntityName(i.arguments.factory,F),t.localJsxFragmentFactory}}if(M.jsxFragmentFactory)return e.parseIsolatedEntityName(M.jsxFragmentFactory,F)}function DO(){var t,r=n.getResolvedTypeReferenceDirectives();return r&&(t=new e.Map,r.forEach(function(e,t){if(e&&e.resolvedFileName){var r=n.getSourceFile(e.resolvedFileName);r&&l(r,t)}})),{getReferencedExportContainer:zR,getReferencedImportDeclaration:JR,getReferencedDeclarationWithCollidingName:YR,isDeclarationWithCollidingName:QR,isValueAliasDeclaration:function(n){var t=e.getParseTreeNode(n);return!t||ZR(t)},hasGlobalName:hO,isReferencedAliasDeclaration:function(n,t){var r=e.getParseTreeNode(n);return!r||tO(r,t)},getNodeCheckFlags:function(n){var t=e.getParseTreeNode(n);return t?lO(t):0},isTopLevelValueImportEqualsWithEntityName:$R,isDeclarationVisible:ko,isImplementationOfOverload:rO,isRequiredInitializedParameter:iO,isOptionalUninitializedParameterProperty:aO,isExpandoFunctionDeclaration:sO,getPropertiesOfContainerFunction:cO,createTypeOfDeclaration:_O,createReturnTypeOfSignatureDeclaration:gO,createTypeOfExpression:yO,createLiteralConstValue:SO,isSymbolAccessible:oo,isEntityNameVisible:fo,getConstantValue:function(n){var t=e.getParseTreeNode(n,dO);return t?pO(t):void 0},collectLinkedAliases:Ro,getReferencedValueDeclaration:bO,getTypeReferenceSerializationKind:mO,isOptionalParameter:ju,moduleExportsSomeValue:HR,isArgumentsLocalBinding:KR,getExternalModuleFileFromDeclaration:function(n){var t=e.getParseTreeNode(n,e.hasPossibleExternalModuleReference);return t&&AO(t)},getTypeReferenceDirectivesForEntityName:o,getTypeReferenceDirectivesForSymbol:s,isLiteralConstDeclaration:EO,isLateBound:function(n){var t=e.getParseTreeNode(n,e.isDeclaration),r=t&&Ca(t);return!!(r&&4096&e.getCheckFlags(r))},getJsxFactoryEntity:xO,getJsxFragmentFactoryEntity:LO,getAllAccessorDeclarations:function(n){n=e.getParseTreeNode(n,e.isGetOrSetAccessorDeclaration);var t=171===n.kind?170:171,r=e.getDeclarationOfKind(Ca(n),t),i=r&&r.pos<n.pos?r:n,a=r&&r.pos<n.pos?n:r,o=171===n.kind?n:r,s=170===n.kind?n:r;return{firstAccessor:i,secondAccessor:a,setAccessor:o,getAccessor:s}},getSymbolOfExternalModuleSpecifier:function(e){return la(e,e,void 0)},isBindingCapturedByNode:function(n,t){var r=e.getParseTreeNode(n),i=e.getParseTreeNode(t);return!!r&&!!i&&(e.isVariableDeclaration(i)||e.isBindingElement(i))&&EE(r,i)},getDeclarationStatementsForSourceFile:function(n,t,r,i){var a=e.getParseTreeNode(n);e.Debug.assert(a&&300===a.kind,"Non-sourcefile node passed into getDeclarationsForSourceFile");var o=Ca(n);return o?o.exports?ie.symbolTableToDeclarationStatements(o.exports,n,t,r,i):[]:n.locals?ie.symbolTableToDeclarationStatements(n.locals,n,t,r,i):[]},isImportRequiredByAugmentation:i};function i(n){var t=e.getSourceFileOfNode(n);if(!t.symbol)return!1;var r=AO(n);if(!r)return!1;if(r===t)return!1;for(var i=xa(t.symbol),a=0,o=e.arrayFrom(i.values());a<o.length;a++){var s=o[a];if(s.mergeId){var c=Aa(s);if(c.declarations)for(var l=0,u=c.declarations;l<u.length;l++){var d=u[l],p=e.getSourceFileOfNode(d);if(p===r)return!0}}}return!1}function a(e){return e.parent&&226===e.parent.kind&&e.parent.parent&&289===e.parent.parent.kind}function o(e){if(t){var n=790504;(79===e.kind&&Lv(e)||204===e.kind&&!a(e))&&(n=1160127);var r=ra(e,n,!0);return r&&r!==Ae?s(r,n):void 0}}function s(n,r){if(t&&c(n)){for(var i,a=0,o=n.declarations;a<o.length;a++){var s=o[a];if(s.symbol&&s.symbol.flags&r){var l=e.getSourceFileOfNode(s),u=t.get(l.path);if(!u)return;(i||(i=[])).push(u)}}return i}}function c(n){if(!n.declarations)return!1;var r=n;while(1){var i=Ia(r);if(!i)break;r=i}if(r.valueDeclaration&&300===r.valueDeclaration.kind&&512&r.flags)return!1;for(var a=0,o=n.declarations;a<o.length;a++){var s=o[a],c=e.getSourceFileOfNode(s);if(t.has(c.path))return!0}return!1}function l(r,i){if(!t.has(r.path)){t.set(r.path,i);for(var a=0,o=r.referencedFiles;a<o.length;a++){var s=o[a].fileName,c=e.resolveTripleslashReference(s,r.fileName),u=n.getSourceFile(c);u&&l(u,i)}}}}function AO(n){var t=259===n.kind?e.tryCast(n.name,e.isStringLiteral):e.getExternalModuleName(n),r=la(t,t,void 0);if(r)return e.getDeclarationOfKind(r,300)}function CO(){for(var t=0,r=n.getSourceFiles();t<r.length;t++){var i=r[t];e.bindSourceFile(i,M)}var a;Sn=new e.Map;for(var o=0,s=n.getSourceFiles();o<s.length;o++){i=s[o];if(!i.redirectInfo){if(!e.isExternalOrCommonJsModule(i)){var c=i.locals.get("globalThis");if(null===c||void 0===c?void 0:c.declarations)for(var l=0,u=c.declarations;l<u.length;l++){var d=u[l];ur.add(e.createDiagnosticForNode(d,e.Diagnostics.Declaration_name_conflicts_with_built_in_global_identifier_0,"globalThis"))}Vr(ae,i.locals)}if(i.jsGlobalAugmentations&&Vr(ae,i.jsGlobalAugmentations),i.patternAmbientModules&&i.patternAmbientModules.length&&(Ln=e.concatenate(Ln,i.patternAmbientModules)),i.moduleAugmentations.length&&(a||(a=[])).push(i.moduleAugmentations),i.symbol&&i.symbol.globalExports){var p=i.symbol.globalExports;p.forEach(function(e,n){ae.has(n)||ae.set(n,e)})}}}if(a)for(var f=0,m=a;f<m.length;f++)for(var _=m[f],g=0,y=_;g<y.length;g++){var h=y[g];e.isGlobalScopeAugmentation(h.parent)&&jr(h)}if(Kr(ae,br,e.Diagnostics.Declaration_name_conflicts_with_built_in_global_identifier_0),Hr(oe).type=Fe,Hr(le).type=sp("IArguments",0,!0),Hr(Ae).type=Re,Hr(se).type=ja(16,se),kn=sp("Array",1,!0),An=sp("Object",0,!0),Cn=sp("Function",0,!0),In=q&&sp("CallableFunction",0,!0)||Cn,Nn=q&&sp("NewableFunction",0,!0)||Cn,On=sp("String",0,!0),wn=sp("Number",0,!0),Pn=sp("Boolean",0,!0),Mn=sp("RegExp",0,!0),Gn=Mp(Ie),Bn=Mp(Ne),Bn===pn&&(Bn=Xa(void 0,D,e.emptyArray,e.emptyArray,e.emptyArray)),Rn=Ip("ReadonlyArray",1)||kn,Un=Rn?Op(Rn,[Ie]):Gn,Fn=Ip("ThisType",1),a)for(var v=0,b=a;v<b.length;v++){_=b[v];for(var E=0,T=_;E<T.length;E++){h=T[E];e.isGlobalScopeAugmentation(h.parent)||jr(h)}}Sn.forEach(function(n){var t=n.firstFile,r=n.secondFile,i=n.conflictingSymbols;if(i.size<8)i.forEach(function(n,t){for(var r=n.isBlockScoped,i=n.firstFileLocations,a=n.secondFileLocations,o=r?e.Diagnostics.Cannot_redeclare_block_scoped_variable_0:e.Diagnostics.Duplicate_identifier_0,s=0,c=i;s<c.length;s++){var l=c[s];Br(l,o,t,a)}for(var u=0,d=a;u<d.length;u++){l=d[u];Br(l,o,t,i)}});else{var a=e.arrayFrom(i.keys()).join(", ");ur.add(e.addRelatedInfo(e.createDiagnosticForNode(t,e.Diagnostics.Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0,a),e.createDiagnosticForNode(r,e.Diagnostics.Conflicts_are_in_this_file))),ur.add(e.addRelatedInfo(e.createDiagnosticForNode(r,e.Diagnostics.Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0,a),e.createDiagnosticForNode(t,e.Diagnostics.Conflicts_are_in_this_file)))}}),Sn=void 0}function IO(n,t){if((i&t)!==t&&M.importHelpers){var r=e.getSourceFileOfNode(n);if(e.isEffectiveExternalModule(r,M)&&!(8388608&n.flags)){var a=kO(r,n);if(a!==Ae)for(var o=t&~i,s=1;s<=2097152;s<<=1)if(o&s){var c=NO(s),l=Jr(a.exports,e.escapeLeadingUnderscores(c),111551);l?524288&s?e.some($u(l),function(e){return _D(e)>3})||Dr(n,e.Diagnostics.This_syntax_requires_an_imported_helper_named_1_with_2_parameters_which_is_not_compatible_with_the_one_in_0_Consider_upgrading_your_version_of_0,e.externalHelpersModuleNameText,c,4):1048576&s?e.some($u(l),function(e){return _D(e)>4})||Dr(n,e.Diagnostics.This_syntax_requires_an_imported_helper_named_1_with_2_parameters_which_is_not_compatible_with_the_one_in_0_Consider_upgrading_your_version_of_0,e.externalHelpersModuleNameText,c,5):1024&s&&(e.some($u(l),function(e){return _D(e)>2})||Dr(n,e.Diagnostics.This_syntax_requires_an_imported_helper_named_1_with_2_parameters_which_is_not_compatible_with_the_one_in_0_Consider_upgrading_your_version_of_0,e.externalHelpersModuleNameText,c,3)):Dr(n,e.Diagnostics.This_syntax_requires_an_imported_helper_named_1_which_does_not_exist_in_0_Consider_upgrading_your_version_of_0,e.externalHelpersModuleNameText,c)}i|=t}}}function NO(n){switch(n){case 1:return"__extends";case 2:return"__assign";case 4:return"__rest";case 8:return"__decorate";case 16:return"__metadata";case 32:return"__param";case 64:return"__awaiter";case 128:return"__generator";case 256:return"__values";case 512:return"__read";case 1024:return"__spreadArray";case 2048:return"__await";case 4096:return"__asyncGenerator";case 8192:return"__asyncDelegator";case 16384:return"__asyncValues";case 32768:return"__exportStar";case 65536:return"__importStar";case 131072:return"__importDefault";case 262144:return"__makeTemplateObject";case 524288:return"__classPrivateFieldGet";case 1048576:return"__classPrivateFieldSet";case 2097152:return"__createBinding";default:return e.Debug.fail("Unrecognized helper")}}function kO(n,t){return u||(u=ua(n,e.externalHelpersModuleNameText,e.Diagnostics.This_syntax_requires_an_imported_helper_but_module_0_cannot_be_found,t)||Ae),u}function RO(e){return OO(e)||wO(e)}function OO(n){if(!n.decorators)return!1;if(!e.nodeCanBeDecorated(n,n.parent,n.parent.parent))return 167!==n.kind||e.nodeIsPresent(n.body)?Ow(n,e.Diagnostics.Decorators_are_not_valid_here):Ow(n,e.Diagnostics.A_decorator_can_only_decorate_a_method_implementation_not_an_overload);if(170===n.kind||171===n.kind){var t=e.getAllAccessorDeclarations(n.parent.members,n);if(t.firstAccessor.decorators&&n===t.secondAccessor)return Ow(n,e.Diagnostics.Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name)}return!1}function wO(n){var t,r,i,a,o,s=PO(n);if(void 0!==s)return s;for(var c=0,l=0,u=n.modifiers;l<u.length;l++){var d=u[l];if(143!==d.kind){if(164===n.kind||166===n.kind)return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_a_type_member,e.tokenToString(d.kind));if(174===n.kind&&(124!==d.kind||!e.isClassLike(n.parent)))return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_an_index_signature,e.tokenToString(d.kind))}switch(d.kind){case 85:if(258!==n.kind)return Mw(n,e.Diagnostics.A_class_member_cannot_have_the_0_keyword,e.tokenToString(85));break;case 157:if(16384&c)return Mw(d,e.Diagnostics._0_modifier_already_seen,"override");if(2&c)return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,"override","declare");if(64&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"override","readonly");if(256&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"override","async");c|=16384,o=d;break;case 123:case 122:case 121:var p=Lo(e.modifierToFlag(d.kind));if(28&c)return Mw(d,e.Diagnostics.Accessibility_modifier_already_seen);if(16384&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,p,"override");if(32&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,p,"static");if(64&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,p,"readonly");if(256&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,p,"async");if(260===n.parent.kind||300===n.parent.kind)return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_a_module_or_namespace_element,p);if(128&c)return 121===d.kind?Mw(d,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,p,"abstract"):Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,p,"abstract");if(e.isPrivateIdentifierClassElementDeclaration(n))return Mw(d,e.Diagnostics.An_accessibility_modifier_cannot_be_used_with_a_private_identifier);c|=e.modifierToFlag(d.kind);break;case 124:if(32&c)return Mw(d,e.Diagnostics._0_modifier_already_seen,"static");if(64&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"static","readonly");if(256&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"static","async");if(260===n.parent.kind||300===n.parent.kind)return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_a_module_or_namespace_element,"static");if(162===n.kind)return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_a_parameter,"static");if(128&c)return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,"static","abstract");if(16384&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"static","override");c|=32,t=d;break;case 143:if(64&c)return Mw(d,e.Diagnostics._0_modifier_already_seen,"readonly");if(165!==n.kind&&164!==n.kind&&174!==n.kind&&162!==n.kind)return Mw(d,e.Diagnostics.readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature);c|=64,a=d;break;case 93:if(1&c)return Mw(d,e.Diagnostics._0_modifier_already_seen,"export");if(2&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"export","declare");if(128&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"export","abstract");if(256&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"export","async");if(e.isClassLike(n.parent))return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_class_elements_of_this_kind,"export");if(162===n.kind)return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_a_parameter,"export");c|=1;break;case 88:var f=300===n.parent.kind?n.parent:n.parent.parent;if(259===f.kind&&!e.isAmbientModule(f))return Mw(d,e.Diagnostics.A_default_export_can_only_be_used_in_an_ECMAScript_style_module);if(!(1&c))return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"export","default");c|=512;break;case 134:if(2&c)return Mw(d,e.Diagnostics._0_modifier_already_seen,"declare");if(256&c)return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_in_an_ambient_context,"async");if(16384&c)return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_in_an_ambient_context,"override");if(e.isClassLike(n.parent)&&!e.isPropertyDeclaration(n))return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_class_elements_of_this_kind,"declare");if(162===n.kind)return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_a_parameter,"declare");if(8388608&n.parent.flags&&260===n.parent.kind)return Mw(d,e.Diagnostics.A_declare_modifier_cannot_be_used_in_an_already_ambient_context);if(e.isPrivateIdentifierClassElementDeclaration(n))return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_with_a_private_identifier,"declare");c|=2,r=d;break;case 126:if(128&c)return Mw(d,e.Diagnostics._0_modifier_already_seen,"abstract");if(255!==n.kind&&178!==n.kind){if(167!==n.kind&&165!==n.kind&&170!==n.kind&&171!==n.kind)return Mw(d,e.Diagnostics.abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration);if(255!==n.parent.kind||!e.hasSyntacticModifier(n.parent,128))return Mw(d,e.Diagnostics.Abstract_methods_can_only_appear_within_an_abstract_class);if(32&c)return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,"static","abstract");if(8&c)return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,"private","abstract");if(256&c&&i)return Mw(i,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,"async","abstract");if(16384&c)return Mw(d,e.Diagnostics._0_modifier_must_precede_1_modifier,"abstract","override")}if(e.isNamedDeclaration(n)&&80===n.name.kind)return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_with_a_private_identifier,"abstract");c|=128;break;case 130:if(256&c)return Mw(d,e.Diagnostics._0_modifier_already_seen,"async");if(2&c||8388608&n.parent.flags)return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_in_an_ambient_context,"async");if(162===n.kind)return Mw(d,e.Diagnostics._0_modifier_cannot_appear_on_a_parameter,"async");if(128&c)return Mw(d,e.Diagnostics._0_modifier_cannot_be_used_with_1_modifier,"async","abstract");c|=256,i=d;break}}return 169===n.kind?32&c?Mw(t,e.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration,"static"):128&c?Mw(t,e.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration,"abstract"):16384&c?Mw(o,e.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration,"override"):256&c?Mw(i,e.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration,"async"):!!(64&c)&&Mw(a,e.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration,"readonly"):(264===n.kind||263===n.kind)&&2&c?Mw(r,e.Diagnostics.A_0_modifier_cannot_be_used_with_an_import_declaration,"declare"):162===n.kind&&16476&c&&e.isBindingPattern(n.name)?Mw(n,e.Diagnostics.A_parameter_property_may_not_be_declared_using_a_binding_pattern):162===n.kind&&16476&c&&n.dotDotDotToken?Mw(n,e.Diagnostics.A_parameter_property_cannot_be_declared_using_a_rest_parameter):!!(256&c)&&GO(n,i)}function PO(n){return!!n.modifiers&&(MO(n)?Ow(n,e.Diagnostics.Modifiers_cannot_appear_here):void 0)}function MO(n){switch(n.kind){case 170:case 171:case 169:case 165:case 164:case 167:case 166:case 174:case 259:case 264:case 263:case 270:case 269:case 211:case 212:case 162:return!1;default:if(260===n.parent.kind||300===n.parent.kind)return!1;switch(n.kind){case 254:return FO(n,130);case 255:case 178:return FO(n,126);case 256:case 235:case 257:case 168:return!0;case 258:return FO(n,85);default:e.Debug.fail()}}}function FO(e,n){return e.modifiers.length>1||e.modifiers[0].kind!==n}function GO(n,t){switch(n.kind){case 167:case 254:case 211:case 212:return!1}return Mw(t,e.Diagnostics._0_modifier_cannot_be_used_here,"async")}function BO(n,t){return void 0===t&&(t=e.Diagnostics.Trailing_comma_not_allowed),!(!n||!n.hasTrailingComma)&&ww(n[0],n.end-",".length,",".length,t)}function UO(n,t){if(n&&0===n.length){var r=n.pos-"<".length,i=e.skipTrivia(t.text,n.end)+">".length;return ww(t,r,i-r,e.Diagnostics.Type_parameter_list_cannot_be_empty)}return!1}function VO(n){for(var t=!1,r=n.length,i=0;i<r;i++){var a=n[i];if(a.dotDotDotToken){if(i!==r-1)return Mw(a.dotDotDotToken,e.Diagnostics.A_rest_parameter_must_be_last_in_a_parameter_list);if(8388608&a.flags||BO(n,e.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma),a.questionToken)return Mw(a.questionToken,e.Diagnostics.A_rest_parameter_cannot_be_optional);if(a.initializer)return Mw(a.name,e.Diagnostics.A_rest_parameter_cannot_have_an_initializer)}else if(ju(a)){if(t=!0,a.questionToken&&a.initializer)return Mw(a.name,e.Diagnostics.Parameter_cannot_have_question_mark_and_initializer)}else if(t&&!a.initializer)return Mw(a.name,e.Diagnostics.A_required_parameter_cannot_follow_an_optional_parameter)}}function jO(n){return e.filter(n,function(n){return!!n.initializer||e.isBindingPattern(n.name)||e.isRestParameter(n)})}function KO(n){if(F>=3){var t=n.body&&e.isBlock(n.body)&&e.findUseStrictPrologue(n.body.statements);if(t){var r=jO(n.parameters);if(e.length(r)){e.forEach(r,function(n){e.addRelatedInfo(Dr(n,e.Diagnostics.This_parameter_is_not_allowed_with_use_strict_directive),e.createDiagnosticForNode(t,e.Diagnostics.use_strict_directive_used_here))});var i=r.map(function(n,t){return 0===t?e.createDiagnosticForNode(n,e.Diagnostics.Non_simple_parameter_declared_here):e.createDiagnosticForNode(n,e.Diagnostics.and_here)});return e.addRelatedInfo.apply(void 0,a([Dr(t,e.Diagnostics.use_strict_directive_cannot_be_used_with_non_simple_parameter_list)],i,!1)),!0}}}return!1}function HO(n){var t=e.getSourceFileOfNode(n);return RO(n)||UO(n.typeParameters,t)||VO(n.parameters)||zO(n,t)||e.isFunctionLikeDeclaration(n)&&KO(n)}function WO(n){var t=e.getSourceFileOfNode(n);return tw(n)||UO(n.typeParameters,t)}function zO(n,t){if(!e.isArrowFunction(n))return!1;var r=n.equalsGreaterThanToken,i=e.getLineAndCharacterOfPosition(t,r.pos).line,a=e.getLineAndCharacterOfPosition(t,r.end).line;return i!==a&&Mw(r,e.Diagnostics.Line_terminator_not_permitted_before_arrow)}function JO(n){var t=n.parameters[0];if(1!==n.parameters.length)return Mw(t?t.name:n,e.Diagnostics.An_index_signature_must_have_exactly_one_parameter);if(BO(n.parameters,e.Diagnostics.An_index_signature_cannot_have_a_trailing_comma),t.dotDotDotToken)return Mw(t.dotDotDotToken,e.Diagnostics.An_index_signature_cannot_have_a_rest_parameter);if(e.hasEffectiveModifiers(t))return Mw(t.name,e.Diagnostics.An_index_signature_parameter_cannot_have_an_accessibility_modifier);if(t.questionToken)return Mw(t.questionToken,e.Diagnostics.An_index_signature_parameter_cannot_have_a_question_mark);if(t.initializer)return Mw(t.name,e.Diagnostics.An_index_signature_parameter_cannot_have_an_initializer);if(!t.type)return Mw(t.name,e.Diagnostics.An_index_signature_parameter_must_have_a_type_annotation);var r=u_(t.type);return bb(r,function(e){return!!(8576&e.flags)})||om(r)?Mw(t.name,e.Diagnostics.An_index_signature_parameter_type_cannot_be_a_literal_type_or_generic_type_Consider_using_a_mapped_object_type_instead):Eb(r,Ld)?!n.type&&Mw(n,e.Diagnostics.An_index_signature_must_have_a_type_annotation):Mw(t.name,e.Diagnostics.An_index_signature_parameter_type_must_be_string_number_symbol_or_a_template_literal_type)}function qO(e){return RO(e)||JO(e)}function XO(n,t){if(t&&0===t.length){var r=e.getSourceFileOfNode(n),i=t.pos-"<".length,a=e.skipTrivia(r.text,t.end)+">".length;return ww(r,i,a-i,e.Diagnostics.Type_argument_list_cannot_be_empty)}return!1}function YO(e,n){return BO(n)||XO(e,n)}function QO(n){return!!(n.questionDotToken||32&n.flags)&&Mw(n.template,e.Diagnostics.Tagged_template_expressions_are_not_permitted_in_an_optional_chain)}function ZO(n){if(n)for(var t=0,r=n;t<r.length;t++){var i=r[t];if(225===i.kind)return ww(i,i.pos,0,e.Diagnostics.Argument_expression_expected)}return!1}function $O(e){return ZO(e)}function ew(n){var t=n.types;if(BO(t))return!0;if(t&&0===t.length){var r=e.tokenToString(n.token);return ww(n,t.pos,0,e.Diagnostics._0_list_cannot_be_empty,r)}return e.some(t,nw)}function nw(e){return YO(e,e.typeArguments)}function tw(n){var t=!1,r=!1;if(!RO(n)&&n.heritageClauses)for(var i=0,a=n.heritageClauses;i<a.length;i++){var o=a[i];if(94===o.token){if(t)return Ow(o,e.Diagnostics.extends_clause_already_seen);if(r)return Ow(o,e.Diagnostics.extends_clause_must_precede_implements_clause);if(o.types.length>1)return Ow(o.types[1],e.Diagnostics.Classes_can_only_extend_a_single_class);t=!0}else{if(e.Debug.assert(117===o.token),r)return Ow(o,e.Diagnostics.implements_clause_already_seen);r=!0}ew(o)}}function rw(n){var t=!1;if(n.heritageClauses)for(var r=0,i=n.heritageClauses;r<i.length;r++){var a=i[r];if(94!==a.token)return e.Debug.assert(117===a.token),Ow(a,e.Diagnostics.Interface_declaration_cannot_have_implements_clause);if(t)return Ow(a,e.Diagnostics.extends_clause_already_seen);t=!0,ew(a)}return!1}function iw(n){if(160!==n.kind)return!1;var t=n;return 219===t.expression.kind&&27===t.expression.operatorToken.kind&&Mw(t.expression,e.Diagnostics.A_comma_expression_is_not_allowed_in_a_computed_property_name)}function aw(n){if(n.asteriskToken){if(e.Debug.assert(254===n.kind||211===n.kind||167===n.kind),8388608&n.flags)return Mw(n.asteriskToken,e.Diagnostics.Generators_are_not_allowed_in_an_ambient_context);if(!n.body)return Mw(n.asteriskToken,e.Diagnostics.An_overload_signature_cannot_be_declared_as_a_generator)}}function ow(e,n){return!!e&&Mw(e,n)}function sw(e,n){return!!e&&Mw(e,n)}function cw(n,t){for(var r=new e.Map,i=0,a=n.properties;i<a.length;i++){var o=a[i];if(293!==o.kind){var s=o.name;if(160===s.kind&&iw(s),292===o.kind&&!t&&o.objectAssignmentInitializer)return Mw(o.equalsToken,e.Diagnostics.Did_you_mean_to_use_a_Colon_An_can_only_follow_a_property_name_when_the_containing_object_literal_is_part_of_a_destructuring_pattern);if(80===s.kind&&Mw(s,e.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies),o.modifiers)for(var c=0,l=o.modifiers;c<l.length;c++){var u=l[c];130===u.kind&&167===o.kind||Mw(u,e.Diagnostics._0_modifier_cannot_be_used_here,e.getTextOfNode(u))}var d=void 0;switch(o.kind){case 292:sw(o.exclamationToken,e.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context);case 291:ow(o.questionToken,e.Diagnostics.An_object_member_cannot_be_declared_optional),8===s.kind&&Hw(s),d=4;break;case 167:d=8;break;case 170:d=1;break;case 171:d=2;break;default:throw e.Debug.assertNever(o,"Unexpected syntax kind:"+o.kind)}if(!t){var p=e.getPropertyNameForPropertyNameNode(s);if(void 0===p)continue;var f=r.get(p);if(f)if(12&d&&12&f)Mw(s,e.Diagnostics.Duplicate_identifier_0,e.getTextOfNode(s));else{if(!(3&d&&3&f))return Mw(s,e.Diagnostics.An_object_literal_cannot_have_property_and_accessor_with_the_same_name);if(3===f||d===f)return Mw(s,e.Diagnostics.An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name);r.set(p,d|f)}else r.set(p,d)}}else if(t){var m=e.skipParentheses(o.expression);if(e.isArrayLiteralExpression(m)||e.isObjectLiteralExpression(m))return Mw(o.expression,e.Diagnostics.A_rest_element_cannot_contain_a_binding_pattern)}}}function lw(n){uw(n.tagName),YO(n,n.typeArguments);for(var t=new e.Map,r=0,i=n.attributes.properties;r<i.length;r++){var a=i[r];if(285!==a.kind){var o=a.name,s=a.initializer;if(t.get(o.escapedText))return Mw(o,e.Diagnostics.JSX_elements_cannot_have_multiple_attributes_with_the_same_name);if(t.set(o.escapedText,!0),s&&286===s.kind&&!s.expression)return Mw(s,e.Diagnostics.JSX_attributes_must_only_be_assigned_a_non_empty_expression)}}}function uw(n){if(e.isPropertyAccessExpression(n)){var t=n;do{var r=a(t.name);if(r)return r;t=t.expression}while(e.isPropertyAccessExpression(t));var i=a(t);if(i)return i}function a(n){if(e.isIdentifier(n)&&-1!==e.idText(n).indexOf(":"))return Mw(n,e.Diagnostics.JSX_property_access_expressions_cannot_include_JSX_namespace_names)}}function dw(n){if(n.expression&&e.isCommaSequence(n.expression))return Mw(n.expression,e.Diagnostics.JSX_expressions_may_not_use_the_comma_operator_Did_you_mean_to_write_an_array)}function pw(n){if(Kw(n))return!0;if(242===n.kind&&n.awaitModifier&&!(32768&n.flags)){var t=e.getSourceFileOfNode(n);if(e.isInTopLevelContext(n))Rw(t)||(e.isEffectiveExternalModule(t,M)||ur.add(e.createDiagnosticForNode(n.awaitModifier,e.Diagnostics.for_await_loops_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_has_no_imports_or_exports_Consider_adding_an_empty_export_to_make_this_file_a_module)),(K!==e.ModuleKind.ESNext&&K!==e.ModuleKind.System||F<4)&&ur.add(e.createDiagnosticForNode(n.awaitModifier,e.Diagnostics.Top_level_for_await_loops_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_target_option_is_set_to_es2017_or_higher)));else if(!Rw(t)){var r=e.createDiagnosticForNode(n.awaitModifier,e.Diagnostics.for_await_loops_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules),i=e.getContainingFunction(n);if(i&&169!==i.kind){e.Debug.assert(0===(2&e.getFunctionFlags(i)),"Enclosing function should never be an async function.");var a=e.createDiagnosticForNode(i,e.Diagnostics.Did_you_mean_to_mark_this_function_as_async);e.addRelatedInfo(r,a)}return ur.add(r),!0}return!1}if(e.isForOfStatement(n)&&!(32768&n.flags)&&e.isIdentifier(n.initializer)&&"async"===n.initializer.escapedText)return Mw(n.initializer,e.Diagnostics.The_left_hand_side_of_a_for_of_statement_may_not_be_async),!1;if(253===n.initializer.kind){var o=n.initializer;if(!Cw(o)){var s=o.declarations;if(!s.length)return!1;if(s.length>1){r=241===n.kind?e.Diagnostics.Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement:e.Diagnostics.Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement;return Ow(o.declarations[1],r)}var c=s[0];if(c.initializer){r=241===n.kind?e.Diagnostics.The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer:e.Diagnostics.The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer;return Mw(c.name,r)}if(c.type){r=241===n.kind?e.Diagnostics.The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation:e.Diagnostics.The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation;return Mw(c,r)}}}return!1}function fw(n){if(!(8388608&n.flags)&&180!==n.parent.kind&&256!==n.parent.kind){if(F<1)return Mw(n.name,e.Diagnostics.Accessors_are_only_available_when_targeting_ECMAScript_5_and_higher);if(F<2&&e.isPrivateIdentifier(n.name))return Mw(n.name,e.Diagnostics.Private_identifiers_are_only_available_when_targeting_ECMAScript_2015_and_higher);if(void 0===n.body&&!e.hasSyntacticModifier(n,128))return ww(n,n.end-1,";".length,e.Diagnostics._0_expected,"{")}if(n.body){if(e.hasSyntacticModifier(n,128))return Mw(n,e.Diagnostics.An_abstract_accessor_cannot_have_an_implementation);if(180===n.parent.kind||256===n.parent.kind)return Mw(n.body,e.Diagnostics.An_implementation_cannot_be_declared_in_ambient_contexts)}if(n.typeParameters)return Mw(n.name,e.Diagnostics.An_accessor_cannot_have_type_parameters);if(!mw(n))return Mw(n.name,170===n.kind?e.Diagnostics.A_get_accessor_cannot_have_parameters:e.Diagnostics.A_set_accessor_must_have_exactly_one_parameter);if(171===n.kind){if(n.type)return Mw(n.name,e.Diagnostics.A_set_accessor_cannot_have_a_return_type_annotation);var t=e.Debug.checkDefined(e.getSetAccessorValueParameter(n),"Return value does not match parameter count assertion.");if(t.dotDotDotToken)return Mw(t.dotDotDotToken,e.Diagnostics.A_set_accessor_cannot_have_rest_parameter);if(t.questionToken)return Mw(t.questionToken,e.Diagnostics.A_set_accessor_cannot_have_an_optional_parameter);if(t.initializer)return Mw(n.name,e.Diagnostics.A_set_accessor_parameter_cannot_have_an_initializer)}return!1}function mw(e){return _w(e)||e.parameters.length===(170===e.kind?0:1)}function _w(n){if(n.parameters.length===(170===n.kind?1:2))return e.getThisParameter(n)}function gw(n){if(152===n.operator){if(149!==n.type.kind)return Mw(n.type,e.Diagnostics._0_expected,e.tokenToString(149));var t=e.walkUpParenthesizedTypes(n.parent);switch(e.isInJSFile(t)&&e.isJSDocTypeExpression(t)&&(t=t.parent,e.isJSDocTypeTag(t)&&(t=t.parent.parent)),t.kind){case 252:var r=t;if(79!==r.name.kind)return Mw(n,e.Diagnostics.unique_symbol_types_may_not_be_used_on_a_variable_declaration_with_a_binding_name);if(!e.isVariableDeclarationInVariableStatement(r))return Mw(n,e.Diagnostics.unique_symbol_types_are_only_allowed_on_variables_in_a_variable_statement);if(!(2&r.parent.flags))return Mw(t.name,e.Diagnostics.A_variable_whose_type_is_a_unique_symbol_type_must_be_const);break;case 165:if(!e.isStatic(t)||!e.hasEffectiveReadonlyModifier(t))return Mw(t.name,e.Diagnostics.A_property_of_a_class_whose_type_is_a_unique_symbol_type_must_be_both_static_and_readonly);break;case 164:if(!e.hasSyntacticModifier(t,64))return Mw(t.name,e.Diagnostics.A_property_of_an_interface_or_type_literal_whose_type_is_a_unique_symbol_type_must_be_readonly);break;default:return Mw(n,e.Diagnostics.unique_symbol_types_are_not_allowed_here)}}else if(143===n.operator&&181!==n.type.kind&&182!==n.type.kind)return Ow(n,e.Diagnostics.readonly_type_modifier_is_only_permitted_on_array_and_tuple_literal_types,e.tokenToString(149))}function yw(e,n){if(jc(e))return Mw(e,n)}function hw(n){if(HO(n))return!0;if(167===n.kind){if(203===n.parent.kind){if(n.modifiers&&(1!==n.modifiers.length||130!==e.first(n.modifiers).kind))return Ow(n,e.Diagnostics.Modifiers_cannot_appear_here);if(ow(n.questionToken,e.Diagnostics.An_object_member_cannot_be_declared_optional))return!0;if(sw(n.exclamationToken,e.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context))return!0;if(void 0===n.body)return ww(n,n.end-1,";".length,e.Diagnostics._0_expected,"{")}if(aw(n))return!0}if(e.isClassLike(n.parent)){if(F<2&&e.isPrivateIdentifier(n.name))return Mw(n.name,e.Diagnostics.Private_identifiers_are_only_available_when_targeting_ECMAScript_2015_and_higher);if(8388608&n.flags)return yw(n.name,e.Diagnostics.A_computed_property_name_in_an_ambient_context_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type);if(167===n.kind&&!n.body)return yw(n.name,e.Diagnostics.A_computed_property_name_in_a_method_overload_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type)}else{if(256===n.parent.kind)return yw(n.name,e.Diagnostics.A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type);if(180===n.parent.kind)return yw(n.name,e.Diagnostics.A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type)}}function vw(n){var t=n;while(t){if(e.isFunctionLikeOrClassStaticBlockDeclaration(t))return Mw(n,e.Diagnostics.Jump_target_cannot_cross_function_boundary);switch(t.kind){case 248:if(n.label&&t.label.escapedText===n.label.escapedText){var r=243===n.kind&&!e.isIterationStatement(t.statement,!0);return!!r&&Mw(n,e.Diagnostics.A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement)}break;case 247:if(244===n.kind&&!n.label)return!1;break;default:if(e.isIterationStatement(t,!1)&&!n.label)return!1;break}t=t.parent}if(n.label){var i=244===n.kind?e.Diagnostics.A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement:e.Diagnostics.A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement;return Mw(n,i)}i=244===n.kind?e.Diagnostics.A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement:e.Diagnostics.A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement;return Mw(n,i)}function bw(n){if(n.dotDotDotToken){var t=n.parent.elements;if(n!==e.last(t))return Mw(n,e.Diagnostics.A_rest_element_must_be_last_in_a_destructuring_pattern);if(BO(t,e.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma),n.propertyName)return Mw(n.name,e.Diagnostics.A_rest_element_cannot_have_a_property_name)}if(n.dotDotDotToken&&n.initializer)return ww(n,n.initializer.pos-1,1,e.Diagnostics.A_rest_element_cannot_have_an_initializer)}function Ew(n){return e.isStringOrNumericLiteralLike(n)||217===n.kind&&40===n.operator&&8===n.operand.kind}function Tw(e){return 9===e.kind||217===e.kind&&40===e.operator&&9===e.operand.kind}function Sw(n){if((e.isPropertyAccessExpression(n)||e.isElementAccessExpression(n)&&Ew(n.argumentExpression))&&e.isEntityNameExpression(n.expression))return!!(1024&kA(n).flags)}function xw(n){var t=n.initializer;if(t){var r=!(Ew(t)||Sw(t)||110===t.kind||95===t.kind||Tw(t)),i=e.isDeclarationReadonly(n)||e.isVariableDeclaration(n)&&e.isVarConst(n);if(!i||n.type)return Mw(t,e.Diagnostics.Initializers_are_not_allowed_in_ambient_contexts);if(r)return Mw(t,e.Diagnostics.A_const_initializer_in_an_ambient_context_must_be_a_string_or_numeric_literal_or_literal_enum_reference);if(!i||r)return Mw(t,e.Diagnostics.Initializers_are_not_allowed_in_ambient_contexts)}}function Lw(n){if(241!==n.parent.parent.kind&&242!==n.parent.parent.kind)if(8388608&n.flags)xw(n);else if(!n.initializer){if(e.isBindingPattern(n.name)&&!e.isBindingPattern(n.parent))return Mw(n,e.Diagnostics.A_destructuring_declaration_must_have_an_initializer);if(e.isVarConst(n))return Mw(n,e.Diagnostics.const_declarations_must_be_initialized)}if(n.exclamationToken&&(235!==n.parent.parent.kind||!n.type||n.initializer||8388608&n.flags)){var t=n.initializer?e.Diagnostics.Declarations_with_initializers_cannot_also_have_definite_assignment_assertions:n.type?e.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context:e.Diagnostics.Declarations_with_definite_assignment_assertions_must_also_have_type_annotations;return Mw(n.exclamationToken,t)}var r=e.getEmitModuleKind(M);r<e.ModuleKind.ES2015&&r!==e.ModuleKind.System&&!(8388608&n.parent.parent.flags)&&e.hasSyntacticModifier(n.parent.parent,1)&&Dw(n.name);var i=e.isLet(n)||e.isVarConst(n);return i&&Aw(n.name)}function Dw(n){if(79===n.kind){if("__esModule"===e.idText(n))return Pw("noEmit",n,e.Diagnostics.Identifier_expected_esModule_is_reserved_as_an_exported_marker_when_transforming_ECMAScript_modules)}else for(var t=n.elements,r=0,i=t;r<i.length;r++){var a=i[r];if(!e.isOmittedExpression(a))return Dw(a.name)}return!1}function Aw(n){if(79===n.kind){if(119===n.originalKeywordKind)return Mw(n,e.Diagnostics.let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations)}else for(var t=n.elements,r=0,i=t;r<i.length;r++){var a=i[r];e.isOmittedExpression(a)||Aw(a.name)}return!1}function Cw(n){var t=n.declarations;return!!BO(n.declarations)||!n.declarations.length&&ww(n,t.pos,t.end-t.pos,e.Diagnostics.Variable_declaration_list_cannot_be_empty)}function Iw(e){switch(e.kind){case 237:case 238:case 239:case 246:case 240:case 241:case 242:return!1;case 248:return Iw(e.parent)}return!0}function Nw(n){if(!Iw(n.parent)){if(e.isLet(n.declarationList))return Mw(n,e.Diagnostics.let_declarations_can_only_be_declared_inside_a_block);if(e.isVarConst(n.declarationList))return Mw(n,e.Diagnostics.const_declarations_can_only_be_declared_inside_a_block)}}function kw(n){var t=n.name.escapedText;switch(n.keywordToken){case 103:if("target"!==t)return Mw(n.name,e.Diagnostics._0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2,n.name.escapedText,e.tokenToString(n.keywordToken),"target");break;case 100:if("meta"!==t)return Mw(n.name,e.Diagnostics._0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2,n.name.escapedText,e.tokenToString(n.keywordToken),"meta");break}}function Rw(e){return e.parseDiagnostics.length>0}function Ow(n,t,r,i,a){var o=e.getSourceFileOfNode(n);if(!Rw(o)){var s=e.getSpanOfTokenAtPosition(o,n.pos);return ur.add(e.createFileDiagnostic(o,s.start,s.length,t,r,i,a)),!0}return!1}function ww(n,t,r,i,a,o,s){var c=e.getSourceFileOfNode(n);return!Rw(c)&&(ur.add(e.createFileDiagnostic(c,t,r,i,a,o,s)),!0)}function Pw(n,t,r,i,a,o){var s=e.getSourceFileOfNode(t);return!Rw(s)&&(xr(n,t,r,i,a,o),!0)}function Mw(n,t,r,i,a){var o=e.getSourceFileOfNode(n);return!Rw(o)&&(ur.add(e.createDiagnosticForNode(n,t,r,i,a)),!0)}function Fw(n){var t=e.isInJSFile(n)?e.getJSDocTypeParameterDeclarations(n):void 0,r=n.typeParameters||t&&e.firstOrUndefined(t);if(r){var i=r.pos===r.end?r.pos:e.skipTrivia(e.getSourceFileOfNode(n).text,r.pos);return ww(n,i,r.end-i,e.Diagnostics.Type_parameters_cannot_appear_on_a_constructor_declaration)}}function Gw(n){var t=e.getEffectiveReturnTypeNode(n);if(t)return Mw(t,e.Diagnostics.Type_annotation_cannot_appear_on_a_constructor_declaration)}function Bw(n){if(e.isClassLike(n.parent)){if(e.isStringLiteral(n.name)&&"constructor"===n.name.text)return Mw(n.name,e.Diagnostics.Classes_may_not_have_a_field_named_constructor);if(yw(n.name,e.Diagnostics.A_computed_property_name_in_a_class_property_declaration_must_have_a_simple_literal_type_or_a_unique_symbol_type))return!0;if(F<2&&e.isPrivateIdentifier(n.name))return Mw(n.name,e.Diagnostics.Private_identifiers_are_only_available_when_targeting_ECMAScript_2015_and_higher)}else if(256===n.parent.kind){if(yw(n.name,e.Diagnostics.A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type))return!0;if(n.initializer)return Mw(n.initializer,e.Diagnostics.An_interface_property_cannot_have_an_initializer)}else if(180===n.parent.kind){if(yw(n.name,e.Diagnostics.A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type))return!0;if(n.initializer)return Mw(n.initializer,e.Diagnostics.A_type_literal_property_cannot_have_an_initializer)}if(8388608&n.flags&&xw(n),e.isPropertyDeclaration(n)&&n.exclamationToken&&(!e.isClassLike(n.parent)||!n.type||n.initializer||8388608&n.flags||e.isStatic(n)||e.hasAbstractModifier(n))){var t=n.initializer?e.Diagnostics.Declarations_with_initializers_cannot_also_have_definite_assignment_assertions:n.type?e.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context:e.Diagnostics.Declarations_with_definite_assignment_assertions_must_also_have_type_annotations;return Mw(n.exclamationToken,t)}}function Uw(n){return 256!==n.kind&&257!==n.kind&&264!==n.kind&&263!==n.kind&&270!==n.kind&&269!==n.kind&&262!==n.kind&&!e.hasSyntacticModifier(n,515)&&Ow(n,e.Diagnostics.Top_level_declarations_in_d_ts_files_must_start_with_either_a_declare_or_export_modifier)}function Vw(n){for(var t=0,r=n.statements;t<r.length;t++){var i=r[t];if((e.isDeclaration(i)||235===i.kind)&&Uw(i))return!0}return!1}function jw(e){return!!(8388608&e.flags)&&Vw(e)}function Kw(n){if(8388608&n.flags){var t=Wr(n);if(!t.hasReportedStatementInAmbientContext&&(e.isFunctionLike(n.parent)||e.isAccessor(n.parent)))return Wr(n).hasReportedStatementInAmbientContext=Ow(n,e.Diagnostics.An_implementation_cannot_be_declared_in_ambient_contexts);if(233===n.parent.kind||260===n.parent.kind||300===n.parent.kind){var r=Wr(n.parent);if(!r.hasReportedStatementInAmbientContext)return r.hasReportedStatementInAmbientContext=Ow(n,e.Diagnostics.Statements_are_not_allowed_in_ambient_contexts)}}return!1}function Hw(n){if(32&n.numericLiteralFlags){var t=void 0;if(F>=1?t=e.Diagnostics.Octal_literals_are_not_available_when_targeting_ECMAScript_5_and_higher_Use_the_syntax_0:e.isChildOfNodeWithKind(n,194)?t=e.Diagnostics.Octal_literal_types_must_use_ES2015_syntax_Use_the_syntax_0:e.isChildOfNodeWithKind(n,294)&&(t=e.Diagnostics.Octal_literals_are_not_allowed_in_enums_members_initializer_Use_the_syntax_0),t){var r=e.isPrefixUnaryExpression(n.parent)&&40===n.parent.operator,i=(r?"-":"")+"0o"+n.text;return Mw(r?n.parent:n,t,i)}}return Ww(n),!1}function Ww(n){if(!(16&n.numericLiteralFlags||n.text.length<=15||-1!==n.text.indexOf("."))){var t=+e.getTextOfNode(n);t<=Math.pow(2,53)-1&&t+1>t||Ar(!1,e.createDiagnosticForNode(n,e.Diagnostics.Numeric_literals_with_absolute_values_equal_to_2_53_or_greater_are_too_large_to_be_represented_accurately_as_integers))}}function zw(n){var t=e.isLiteralTypeNode(n.parent)||e.isPrefixUnaryExpression(n.parent)&&e.isLiteralTypeNode(n.parent.parent);return!!(!t&&F<7&&Mw(n,e.Diagnostics.BigInt_literals_are_not_available_when_targeting_lower_than_ES2020))}function Jw(n,t,r,i,a){var o=e.getSourceFileOfNode(n);if(!Rw(o)){var s=e.getSpanOfTokenAtPosition(o,n.pos);return ur.add(e.createFileDiagnostic(o,e.textSpanEnd(s),0,t,r,i,a)),!0}return!1}function qw(){return xn||(xn=[],ae.forEach(function(e,n){s.test(n)&&xn.push(e)})),xn}function Xw(n){return!!(n.isTypeOnly&&n.name&&n.namedBindings)&&Mw(n,e.Diagnostics.A_type_only_import_can_specify_a_default_import_or_named_bindings_but_not_both)}function Yw(n){if(K===e.ModuleKind.ES2015)return Mw(n,e.Diagnostics.Dynamic_imports_are_only_supported_when_the_module_flag_is_set_to_es2020_esnext_commonjs_amd_system_or_umd);if(n.typeArguments)return Mw(n,e.Diagnostics.Dynamic_import_cannot_have_type_arguments);var t=n.arguments;return 1!==t.length?Mw(n,e.Diagnostics.Dynamic_import_must_have_one_specifier_as_an_argument):(BO(t),!!e.isSpreadElement(t[0])&&Mw(t[0],e.Diagnostics.Specifier_of_dynamic_import_cannot_be_spread_element))}function Qw(n,t){var r=e.getObjectFlags(n);if(20&r&&1048576&t.flags)return e.find(t.types,function(t){if(524288&t.flags){var i=r&e.getObjectFlags(t);if(4&i)return n.target===t.target;if(16&i)return!!n.aliasSymbol&&n.aliasSymbol===t.aliasSymbol}return!1})}function Zw(n,t){if(128&e.getObjectFlags(n)&&bb(t,Iy))return e.find(t.types,function(e){return!Iy(e)})}function $w(n,t){var r=0,i=Au(n,r).length>0||(r=1,Au(n,r).length>0);if(i)return e.find(t.types,function(e){return Au(e,r).length>0})}function eP(n,t){for(var r,i=0,a=0,o=t.types;a<o.length;a++){var s=o[a],c=Df([Uf(n),Uf(s)]);if(4194304&c.flags)r=s,i=1/0;else if(1048576&c.flags){var l=e.length(e.filter(c.types,Fy));l>=i&&(r=s,i=l)}else Fy(c)&&1>=i&&(r=s,i=1)}return r}function nP(e){if(iA(e,67108864)){var n=Sb(e,function(e){return!(131068&e.flags)});if(!(131072&n.flags))return n}return e}function tP(n,t,r,i){if(1048576&t.flags&&2621440&n.flags){var a=Gv(t,n);if(a)return a;var o=Hl(n);if(o){var s=wv(o,t);if(s)return Xg(t,e.map(s,function(e){return[function(){return Ws(e)},e.escapedName]}),r,void 0,i)}}}}function F(n){return!e.isAccessor(n)}function G(e){return 254!==e.kind&&167!==e.kind||!!e.body}function B(n){switch(n.parent.kind){case 268:case 273:return e.isIdentifier(n);default:return e.isDeclarationName(n)}}function U(e){switch(e){case 0:return"yieldType";case 1:return"returnType";case 2:return"nextType"}}function V(e){return!!(1&e.flags)}function j(e){return!!(2&e.flags)}e.getNodeId=O,e.getSymbolId=w,e.isInstantiatedModule=P,e.createTypeChecker=M,function(e){e.JSX="JSX",e.IntrinsicElements="IntrinsicElements",e.ElementClass="ElementClass",e.ElementAttributesPropertyNameContainer="ElementAttributesProperty",e.ElementChildrenAttributeNameContainer="ElementChildrenAttribute",e.Element="Element",e.IntrinsicAttributes="IntrinsicAttributes",e.IntrinsicClassAttributes="IntrinsicClassAttributes",e.LibraryManagedAttributes="LibraryManagedAttributes"}(I||(I={})),e.signatureHasRestParameter=V,e.signatureHasLiteralTypes=j}(d||(d={})),function(e){function n(n,t,r,i){if(void 0===n||void 0===t)return n;var a,o=t(n);return o===n?n:void 0!==o?(a=e.isArray(o)?(i||p)(o):o,e.Debug.assertNode(a,r),a):void 0}function t(n,t,r,i,a){if(void 0===n||void 0===t)return n;var o,s,c=n.length;(void 0===i||i<0)&&(i=0),(void 0===a||a>c-i)&&(a=c-i);var l=-1,u=-1;(i>0||a<c)&&(o=[],s=n.hasTrailingComma&&i+a===c);for(var d=0;d<a;d++){var p=n[d+i],f=void 0!==p?t(p):void 0;if((void 0!==o||void 0===f||f!==p)&&(void 0===o&&(o=n.slice(0,d),s=n.hasTrailingComma,l=n.pos,u=n.end),f))if(e.isArray(f))for(var m=0,_=f;m<_.length;m++){var g=_[m];e.Debug.assertNode(g,r),o.push(g)}else e.Debug.assertNode(f,r),o.push(f)}if(o){var y=e.factory.createNodeArray(o,s);return e.setTextRangePosEnd(y,l,u),y}return n}function r(n,r,i,a,o,s){return void 0===s&&(s=t),i.startLexicalEnvironment(),n=s(n,r,e.isStatement,a),o&&(n=i.factory.ensureUseStrict(n)),e.factory.mergeLexicalEnvironment(n,i.endLexicalEnvironment())}function i(n,r,i,o){var s;return void 0===o&&(o=t),i.startLexicalEnvironment(),n&&(i.setLexicalEnvironmentFlags(1,!0),s=o(n,r,e.isParameterDeclaration),2&i.getLexicalEnvironmentFlags()&&e.getEmitScriptTarget(i.getCompilerOptions())>=2&&(s=a(s,i)),i.setLexicalEnvironmentFlags(1,!1)),i.suspendLexicalEnvironment(),s}function a(n,t){for(var r,i=0;i<n.length;i++){var a=n[i],s=o(a,t);(r||s!==a)&&(r||(r=n.slice(0,i)),r[i]=s)}return r?e.setTextRange(t.factory.createNodeArray(r,n.hasTrailingComma),n):n}function o(n,t){return n.dotDotDotToken?n:e.isBindingPattern(n.name)?s(n,t):n.initializer?c(n,n.name,n.initializer,t):n}function s(e,n){var t=n.factory;return n.addInitializationStatement(t.createVariableStatement(void 0,t.createVariableDeclarationList([t.createVariableDeclaration(e.name,void 0,e.type,e.initializer?t.createConditionalExpression(t.createStrictEquality(t.getGeneratedNameForNode(e),t.createVoidZero()),void 0,e.initializer,void 0,t.getGeneratedNameForNode(e)):t.getGeneratedNameForNode(e))]))),t.updateParameterDeclaration(e,e.decorators,e.modifiers,e.dotDotDotToken,t.getGeneratedNameForNode(e),e.questionToken,e.type,void 0)}function c(n,t,r,i){var a=i.factory;return i.addInitializationStatement(a.createIfStatement(a.createTypeCheck(a.cloneNode(t),"undefined"),e.setEmitFlags(e.setTextRange(a.createBlock([a.createExpressionStatement(e.setEmitFlags(e.setTextRange(a.createAssignment(e.setEmitFlags(a.cloneNode(t),48),e.setEmitFlags(r,1584|e.getEmitFlags(r))),n),1536))]),n),1953))),a.updateParameterDeclaration(n,n.decorators,n.modifiers,n.dotDotDotToken,n.name,n.questionToken,n.type,void 0)}function l(t,r,i,a){void 0===a&&(a=n),i.resumeLexicalEnvironment();var o=a(t,r,e.isConciseBody),s=i.endLexicalEnvironment();if(e.some(s)){if(!o)return i.factory.createBlock(s);var c=i.factory.converters.convertToFunctionBlock(o),l=e.factory.mergeLexicalEnvironment(c.statements,s);return i.factory.updateBlock(c,l)}return o}function u(t,r,i){i.startBlockScope();var a=n(t,r,e.isStatement,i.factory.liftToBlock),o=i.endBlockScope();return e.some(o)?e.isBlock(a)?(o.push.apply(o,a.statements),i.factory.updateBlock(a,o)):(o.push(a),i.factory.createBlock(o)):a}function d(a,o,s,c,d,p){if(void 0===c&&(c=t),void 0===p&&(p=n),void 0!==a){var f=a.kind;if(f>0&&f<=158||190===f)return a;var m=s.factory;switch(f){case 79:return e.Debug.type(a),m.updateIdentifier(a,c(a.typeArguments,o,e.isTypeNodeOrTypeParameterDeclaration));case 159:return e.Debug.type(a),m.updateQualifiedName(a,p(a.left,o,e.isEntityName),p(a.right,o,e.isIdentifier));case 160:return e.Debug.type(a),m.updateComputedPropertyName(a,p(a.expression,o,e.isExpression));case 161:return e.Debug.type(a),m.updateTypeParameterDeclaration(a,p(a.name,o,e.isIdentifier),p(a.constraint,o,e.isTypeNode),p(a.default,o,e.isTypeNode));case 162:return e.Debug.type(a),m.updateParameterDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.dotDotDotToken,d,e.isDotDotDotToken),p(a.name,o,e.isBindingName),p(a.questionToken,d,e.isQuestionToken),p(a.type,o,e.isTypeNode),p(a.initializer,o,e.isExpression));case 163:return e.Debug.type(a),m.updateDecorator(a,p(a.expression,o,e.isExpression));case 164:return e.Debug.type(a),m.updatePropertySignature(a,c(a.modifiers,o,e.isModifier),p(a.name,o,e.isPropertyName),p(a.questionToken,d,e.isToken),p(a.type,o,e.isTypeNode));case 165:return e.Debug.type(a),m.updatePropertyDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.name,o,e.isPropertyName),p(a.questionToken||a.exclamationToken,d,e.isQuestionOrExclamationToken),p(a.type,o,e.isTypeNode),p(a.initializer,o,e.isExpression));case 166:return e.Debug.type(a),m.updateMethodSignature(a,c(a.modifiers,o,e.isModifier),p(a.name,o,e.isPropertyName),p(a.questionToken,d,e.isQuestionToken),c(a.typeParameters,o,e.isTypeParameterDeclaration),c(a.parameters,o,e.isParameterDeclaration),p(a.type,o,e.isTypeNode));case 167:return e.Debug.type(a),m.updateMethodDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.asteriskToken,d,e.isAsteriskToken),p(a.name,o,e.isPropertyName),p(a.questionToken,d,e.isQuestionToken),c(a.typeParameters,o,e.isTypeParameterDeclaration),i(a.parameters,o,s,c),p(a.type,o,e.isTypeNode),l(a.body,o,s,p));case 169:return e.Debug.type(a),m.updateConstructorDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),i(a.parameters,o,s,c),l(a.body,o,s,p));case 170:return e.Debug.type(a),m.updateGetAccessorDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.name,o,e.isPropertyName),i(a.parameters,o,s,c),p(a.type,o,e.isTypeNode),l(a.body,o,s,p));case 171:return e.Debug.type(a),m.updateSetAccessorDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.name,o,e.isPropertyName),i(a.parameters,o,s,c),l(a.body,o,s,p));case 168:return e.Debug.type(a),s.startLexicalEnvironment(),s.suspendLexicalEnvironment(),m.updateClassStaticBlockDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),l(a.body,o,s,p));case 172:return e.Debug.type(a),m.updateCallSignature(a,c(a.typeParameters,o,e.isTypeParameterDeclaration),c(a.parameters,o,e.isParameterDeclaration),p(a.type,o,e.isTypeNode));case 173:return e.Debug.type(a),m.updateConstructSignature(a,c(a.typeParameters,o,e.isTypeParameterDeclaration),c(a.parameters,o,e.isParameterDeclaration),p(a.type,o,e.isTypeNode));case 174:return e.Debug.type(a),m.updateIndexSignature(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),c(a.parameters,o,e.isParameterDeclaration),p(a.type,o,e.isTypeNode));case 175:return e.Debug.type(a),m.updateTypePredicateNode(a,p(a.assertsModifier,o,e.isAssertsKeyword),p(a.parameterName,o,e.isIdentifierOrThisTypeNode),p(a.type,o,e.isTypeNode));case 176:return e.Debug.type(a),m.updateTypeReferenceNode(a,p(a.typeName,o,e.isEntityName),c(a.typeArguments,o,e.isTypeNode));case 177:return e.Debug.type(a),m.updateFunctionTypeNode(a,c(a.typeParameters,o,e.isTypeParameterDeclaration),c(a.parameters,o,e.isParameterDeclaration),p(a.type,o,e.isTypeNode));case 178:return e.Debug.type(a),m.updateConstructorTypeNode(a,c(a.modifiers,o,e.isModifier),c(a.typeParameters,o,e.isTypeParameterDeclaration),c(a.parameters,o,e.isParameterDeclaration),p(a.type,o,e.isTypeNode));case 179:return e.Debug.type(a),m.updateTypeQueryNode(a,p(a.exprName,o,e.isEntityName));case 180:return e.Debug.type(a),m.updateTypeLiteralNode(a,c(a.members,o,e.isTypeElement));case 181:return e.Debug.type(a),m.updateArrayTypeNode(a,p(a.elementType,o,e.isTypeNode));case 182:return e.Debug.type(a),m.updateTupleTypeNode(a,c(a.elements,o,e.isTypeNode));case 183:return e.Debug.type(a),m.updateOptionalTypeNode(a,p(a.type,o,e.isTypeNode));case 184:return e.Debug.type(a),m.updateRestTypeNode(a,p(a.type,o,e.isTypeNode));case 185:return e.Debug.type(a),m.updateUnionTypeNode(a,c(a.types,o,e.isTypeNode));case 186:return e.Debug.type(a),m.updateIntersectionTypeNode(a,c(a.types,o,e.isTypeNode));case 187:return e.Debug.type(a),m.updateConditionalTypeNode(a,p(a.checkType,o,e.isTypeNode),p(a.extendsType,o,e.isTypeNode),p(a.trueType,o,e.isTypeNode),p(a.falseType,o,e.isTypeNode));case 188:return e.Debug.type(a),m.updateInferTypeNode(a,p(a.typeParameter,o,e.isTypeParameterDeclaration));case 198:return e.Debug.type(a),m.updateImportTypeNode(a,p(a.argument,o,e.isTypeNode),p(a.qualifier,o,e.isEntityName),t(a.typeArguments,o,e.isTypeNode),a.isTypeOf);case 195:return e.Debug.type(a),m.updateNamedTupleMember(a,n(a.dotDotDotToken,o,e.isDotDotDotToken),n(a.name,o,e.isIdentifier),n(a.questionToken,o,e.isQuestionToken),n(a.type,o,e.isTypeNode));case 189:return e.Debug.type(a),m.updateParenthesizedType(a,p(a.type,o,e.isTypeNode));case 191:return e.Debug.type(a),m.updateTypeOperatorNode(a,p(a.type,o,e.isTypeNode));case 192:return e.Debug.type(a),m.updateIndexedAccessTypeNode(a,p(a.objectType,o,e.isTypeNode),p(a.indexType,o,e.isTypeNode));case 193:return e.Debug.type(a),m.updateMappedTypeNode(a,p(a.readonlyToken,d,e.isReadonlyKeywordOrPlusOrMinusToken),p(a.typeParameter,o,e.isTypeParameterDeclaration),p(a.nameType,o,e.isTypeNode),p(a.questionToken,d,e.isQuestionOrPlusOrMinusToken),p(a.type,o,e.isTypeNode));case 194:return e.Debug.type(a),m.updateLiteralTypeNode(a,p(a.literal,o,e.isExpression));case 196:return e.Debug.type(a),m.updateTemplateLiteralType(a,p(a.head,o,e.isTemplateHead),c(a.templateSpans,o,e.isTemplateLiteralTypeSpan));case 197:return e.Debug.type(a),m.updateTemplateLiteralTypeSpan(a,p(a.type,o,e.isTypeNode),p(a.literal,o,e.isTemplateMiddleOrTemplateTail));case 199:return e.Debug.type(a),m.updateObjectBindingPattern(a,c(a.elements,o,e.isBindingElement));case 200:return e.Debug.type(a),m.updateArrayBindingPattern(a,c(a.elements,o,e.isArrayBindingElement));case 201:return e.Debug.type(a),m.updateBindingElement(a,p(a.dotDotDotToken,d,e.isDotDotDotToken),p(a.propertyName,o,e.isPropertyName),p(a.name,o,e.isBindingName),p(a.initializer,o,e.isExpression));case 202:return e.Debug.type(a),m.updateArrayLiteralExpression(a,c(a.elements,o,e.isExpression));case 203:return e.Debug.type(a),m.updateObjectLiteralExpression(a,c(a.properties,o,e.isObjectLiteralElementLike));case 204:return 32&a.flags?(e.Debug.type(a),m.updatePropertyAccessChain(a,p(a.expression,o,e.isExpression),p(a.questionDotToken,d,e.isQuestionDotToken),p(a.name,o,e.isMemberName))):(e.Debug.type(a),m.updatePropertyAccessExpression(a,p(a.expression,o,e.isExpression),p(a.name,o,e.isMemberName)));case 205:return 32&a.flags?(e.Debug.type(a),m.updateElementAccessChain(a,p(a.expression,o,e.isExpression),p(a.questionDotToken,d,e.isQuestionDotToken),p(a.argumentExpression,o,e.isExpression))):(e.Debug.type(a),m.updateElementAccessExpression(a,p(a.expression,o,e.isExpression),p(a.argumentExpression,o,e.isExpression)));case 206:return 32&a.flags?(e.Debug.type(a),m.updateCallChain(a,p(a.expression,o,e.isExpression),p(a.questionDotToken,d,e.isQuestionDotToken),c(a.typeArguments,o,e.isTypeNode),c(a.arguments,o,e.isExpression))):(e.Debug.type(a),m.updateCallExpression(a,p(a.expression,o,e.isExpression),c(a.typeArguments,o,e.isTypeNode),c(a.arguments,o,e.isExpression)));case 207:return e.Debug.type(a),m.updateNewExpression(a,p(a.expression,o,e.isExpression),c(a.typeArguments,o,e.isTypeNode),c(a.arguments,o,e.isExpression));case 208:return e.Debug.type(a),m.updateTaggedTemplateExpression(a,p(a.tag,o,e.isExpression),t(a.typeArguments,o,e.isTypeNode),p(a.template,o,e.isTemplateLiteral));case 209:return e.Debug.type(a),m.updateTypeAssertion(a,p(a.type,o,e.isTypeNode),p(a.expression,o,e.isExpression));case 210:return e.Debug.type(a),m.updateParenthesizedExpression(a,p(a.expression,o,e.isExpression));case 211:return e.Debug.type(a),m.updateFunctionExpression(a,c(a.modifiers,o,e.isModifier),p(a.asteriskToken,d,e.isAsteriskToken),p(a.name,o,e.isIdentifier),c(a.typeParameters,o,e.isTypeParameterDeclaration),i(a.parameters,o,s,c),p(a.type,o,e.isTypeNode),l(a.body,o,s,p));case 212:return e.Debug.type(a),m.updateArrowFunction(a,c(a.modifiers,o,e.isModifier),c(a.typeParameters,o,e.isTypeParameterDeclaration),i(a.parameters,o,s,c),p(a.type,o,e.isTypeNode),p(a.equalsGreaterThanToken,d,e.isEqualsGreaterThanToken),l(a.body,o,s,p));case 213:return e.Debug.type(a),m.updateDeleteExpression(a,p(a.expression,o,e.isExpression));case 214:return e.Debug.type(a),m.updateTypeOfExpression(a,p(a.expression,o,e.isExpression));case 215:return e.Debug.type(a),m.updateVoidExpression(a,p(a.expression,o,e.isExpression));case 216:return e.Debug.type(a),m.updateAwaitExpression(a,p(a.expression,o,e.isExpression));case 217:return e.Debug.type(a),m.updatePrefixUnaryExpression(a,p(a.operand,o,e.isExpression));case 218:return e.Debug.type(a),m.updatePostfixUnaryExpression(a,p(a.operand,o,e.isExpression));case 219:return e.Debug.type(a),m.updateBinaryExpression(a,p(a.left,o,e.isExpression),p(a.operatorToken,d,e.isBinaryOperatorToken),p(a.right,o,e.isExpression));case 220:return e.Debug.type(a),m.updateConditionalExpression(a,p(a.condition,o,e.isExpression),p(a.questionToken,d,e.isQuestionToken),p(a.whenTrue,o,e.isExpression),p(a.colonToken,d,e.isColonToken),p(a.whenFalse,o,e.isExpression));case 221:return e.Debug.type(a),m.updateTemplateExpression(a,p(a.head,o,e.isTemplateHead),c(a.templateSpans,o,e.isTemplateSpan));case 222:return e.Debug.type(a),m.updateYieldExpression(a,p(a.asteriskToken,d,e.isAsteriskToken),p(a.expression,o,e.isExpression));case 223:return e.Debug.type(a),m.updateSpreadElement(a,p(a.expression,o,e.isExpression));case 224:return e.Debug.type(a),m.updateClassExpression(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.name,o,e.isIdentifier),c(a.typeParameters,o,e.isTypeParameterDeclaration),c(a.heritageClauses,o,e.isHeritageClause),c(a.members,o,e.isClassElement));case 226:return e.Debug.type(a),m.updateExpressionWithTypeArguments(a,p(a.expression,o,e.isExpression),c(a.typeArguments,o,e.isTypeNode));case 227:return e.Debug.type(a),m.updateAsExpression(a,p(a.expression,o,e.isExpression),p(a.type,o,e.isTypeNode));case 228:return 32&a.flags?(e.Debug.type(a),m.updateNonNullChain(a,p(a.expression,o,e.isExpression))):(e.Debug.type(a),m.updateNonNullExpression(a,p(a.expression,o,e.isExpression)));case 229:return e.Debug.type(a),m.updateMetaProperty(a,p(a.name,o,e.isIdentifier));case 231:return e.Debug.type(a),m.updateTemplateSpan(a,p(a.expression,o,e.isExpression),p(a.literal,o,e.isTemplateMiddleOrTemplateTail));case 233:return e.Debug.type(a),m.updateBlock(a,c(a.statements,o,e.isStatement));case 235:return e.Debug.type(a),m.updateVariableStatement(a,c(a.modifiers,o,e.isModifier),p(a.declarationList,o,e.isVariableDeclarationList));case 236:return e.Debug.type(a),m.updateExpressionStatement(a,p(a.expression,o,e.isExpression));case 237:return e.Debug.type(a),m.updateIfStatement(a,p(a.expression,o,e.isExpression),p(a.thenStatement,o,e.isStatement,m.liftToBlock),p(a.elseStatement,o,e.isStatement,m.liftToBlock));case 238:return e.Debug.type(a),m.updateDoStatement(a,u(a.statement,o,s),p(a.expression,o,e.isExpression));case 239:return e.Debug.type(a),m.updateWhileStatement(a,p(a.expression,o,e.isExpression),u(a.statement,o,s));case 240:return e.Debug.type(a),m.updateForStatement(a,p(a.initializer,o,e.isForInitializer),p(a.condition,o,e.isExpression),p(a.incrementor,o,e.isExpression),u(a.statement,o,s));case 241:return e.Debug.type(a),m.updateForInStatement(a,p(a.initializer,o,e.isForInitializer),p(a.expression,o,e.isExpression),u(a.statement,o,s));case 242:return e.Debug.type(a),m.updateForOfStatement(a,p(a.awaitModifier,d,e.isAwaitKeyword),p(a.initializer,o,e.isForInitializer),p(a.expression,o,e.isExpression),u(a.statement,o,s));case 243:return e.Debug.type(a),m.updateContinueStatement(a,p(a.label,o,e.isIdentifier));case 244:return e.Debug.type(a),m.updateBreakStatement(a,p(a.label,o,e.isIdentifier));case 245:return e.Debug.type(a),m.updateReturnStatement(a,p(a.expression,o,e.isExpression));case 246:return e.Debug.type(a),m.updateWithStatement(a,p(a.expression,o,e.isExpression),p(a.statement,o,e.isStatement,m.liftToBlock));case 247:return e.Debug.type(a),m.updateSwitchStatement(a,p(a.expression,o,e.isExpression),p(a.caseBlock,o,e.isCaseBlock));case 248:return e.Debug.type(a),m.updateLabeledStatement(a,p(a.label,o,e.isIdentifier),p(a.statement,o,e.isStatement,m.liftToBlock));case 249:return e.Debug.type(a),m.updateThrowStatement(a,p(a.expression,o,e.isExpression));case 250:return e.Debug.type(a),m.updateTryStatement(a,p(a.tryBlock,o,e.isBlock),p(a.catchClause,o,e.isCatchClause),p(a.finallyBlock,o,e.isBlock));case 252:return e.Debug.type(a),m.updateVariableDeclaration(a,p(a.name,o,e.isBindingName),p(a.exclamationToken,d,e.isExclamationToken),p(a.type,o,e.isTypeNode),p(a.initializer,o,e.isExpression));case 253:return e.Debug.type(a),m.updateVariableDeclarationList(a,c(a.declarations,o,e.isVariableDeclaration));case 254:return e.Debug.type(a),m.updateFunctionDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.asteriskToken,d,e.isAsteriskToken),p(a.name,o,e.isIdentifier),c(a.typeParameters,o,e.isTypeParameterDeclaration),i(a.parameters,o,s,c),p(a.type,o,e.isTypeNode),l(a.body,o,s,p));case 255:return e.Debug.type(a),m.updateClassDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.name,o,e.isIdentifier),c(a.typeParameters,o,e.isTypeParameterDeclaration),c(a.heritageClauses,o,e.isHeritageClause),c(a.members,o,e.isClassElement));case 256:return e.Debug.type(a),m.updateInterfaceDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.name,o,e.isIdentifier),c(a.typeParameters,o,e.isTypeParameterDeclaration),c(a.heritageClauses,o,e.isHeritageClause),c(a.members,o,e.isTypeElement));case 257:return e.Debug.type(a),m.updateTypeAliasDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.name,o,e.isIdentifier),c(a.typeParameters,o,e.isTypeParameterDeclaration),p(a.type,o,e.isTypeNode));case 258:return e.Debug.type(a),m.updateEnumDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.name,o,e.isIdentifier),c(a.members,o,e.isEnumMember));case 259:return e.Debug.type(a),m.updateModuleDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.name,o,e.isModuleName),p(a.body,o,e.isModuleBody));case 260:return e.Debug.type(a),m.updateModuleBlock(a,c(a.statements,o,e.isStatement));case 261:return e.Debug.type(a),m.updateCaseBlock(a,c(a.clauses,o,e.isCaseOrDefaultClause));case 262:return e.Debug.type(a),m.updateNamespaceExportDeclaration(a,p(a.name,o,e.isIdentifier));case 263:return e.Debug.type(a),m.updateImportEqualsDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),a.isTypeOnly,p(a.name,o,e.isIdentifier),p(a.moduleReference,o,e.isModuleReference));case 264:return e.Debug.type(a),m.updateImportDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.importClause,o,e.isImportClause),p(a.moduleSpecifier,o,e.isExpression));case 265:return e.Debug.type(a),m.updateImportClause(a,a.isTypeOnly,p(a.name,o,e.isIdentifier),p(a.namedBindings,o,e.isNamedImportBindings));case 266:return e.Debug.type(a),m.updateNamespaceImport(a,p(a.name,o,e.isIdentifier));case 272:return e.Debug.type(a),m.updateNamespaceExport(a,p(a.name,o,e.isIdentifier));case 267:return e.Debug.type(a),m.updateNamedImports(a,c(a.elements,o,e.isImportSpecifier));case 268:return e.Debug.type(a),m.updateImportSpecifier(a,p(a.propertyName,o,e.isIdentifier),p(a.name,o,e.isIdentifier));case 269:return e.Debug.type(a),m.updateExportAssignment(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),p(a.expression,o,e.isExpression));case 270:return e.Debug.type(a),m.updateExportDeclaration(a,c(a.decorators,o,e.isDecorator),c(a.modifiers,o,e.isModifier),a.isTypeOnly,p(a.exportClause,o,e.isNamedExportBindings),p(a.moduleSpecifier,o,e.isExpression));case 271:return e.Debug.type(a),m.updateNamedExports(a,c(a.elements,o,e.isExportSpecifier));case 273:return e.Debug.type(a),m.updateExportSpecifier(a,p(a.propertyName,o,e.isIdentifier),p(a.name,o,e.isIdentifier));case 275:return e.Debug.type(a),m.updateExternalModuleReference(a,p(a.expression,o,e.isExpression));case 276:return e.Debug.type(a),m.updateJsxElement(a,p(a.openingElement,o,e.isJsxOpeningElement),c(a.children,o,e.isJsxChild),p(a.closingElement,o,e.isJsxClosingElement));case 277:return e.Debug.type(a),m.updateJsxSelfClosingElement(a,p(a.tagName,o,e.isJsxTagNameExpression),c(a.typeArguments,o,e.isTypeNode),p(a.attributes,o,e.isJsxAttributes));case 278:return e.Debug.type(a),m.updateJsxOpeningElement(a,p(a.tagName,o,e.isJsxTagNameExpression),c(a.typeArguments,o,e.isTypeNode),p(a.attributes,o,e.isJsxAttributes));case 279:return e.Debug.type(a),m.updateJsxClosingElement(a,p(a.tagName,o,e.isJsxTagNameExpression));case 280:return e.Debug.type(a),m.updateJsxFragment(a,p(a.openingFragment,o,e.isJsxOpeningFragment),c(a.children,o,e.isJsxChild),p(a.closingFragment,o,e.isJsxClosingFragment));case 283:return e.Debug.type(a),m.updateJsxAttribute(a,p(a.name,o,e.isIdentifier),p(a.initializer,o,e.isStringLiteralOrJsxExpression));case 284:return e.Debug.type(a),m.updateJsxAttributes(a,c(a.properties,o,e.isJsxAttributeLike));case 285:return e.Debug.type(a),m.updateJsxSpreadAttribute(a,p(a.expression,o,e.isExpression));case 286:return e.Debug.type(a),m.updateJsxExpression(a,p(a.expression,o,e.isExpression));case 287:return e.Debug.type(a),m.updateCaseClause(a,p(a.expression,o,e.isExpression),c(a.statements,o,e.isStatement));case 288:return e.Debug.type(a),m.updateDefaultClause(a,c(a.statements,o,e.isStatement));case 289:return e.Debug.type(a),m.updateHeritageClause(a,c(a.types,o,e.isExpressionWithTypeArguments));case 290:return e.Debug.type(a),m.updateCatchClause(a,p(a.variableDeclaration,o,e.isVariableDeclaration),p(a.block,o,e.isBlock));case 291:return e.Debug.type(a),m.updatePropertyAssignment(a,p(a.name,o,e.isPropertyName),p(a.initializer,o,e.isExpression));case 292:return e.Debug.type(a),m.updateShorthandPropertyAssignment(a,p(a.name,o,e.isIdentifier),p(a.objectAssignmentInitializer,o,e.isExpression));case 293:return e.Debug.type(a),m.updateSpreadAssignment(a,p(a.expression,o,e.isExpression));case 294:return e.Debug.type(a),m.updateEnumMember(a,p(a.name,o,e.isPropertyName),p(a.initializer,o,e.isExpression));case 300:return e.Debug.type(a),m.updateSourceFile(a,r(a.statements,o,s));case 345:return e.Debug.type(a),m.updatePartiallyEmittedExpression(a,p(a.expression,o,e.isExpression));case 346:return e.Debug.type(a),m.updateCommaListExpression(a,c(a.elements,o,e.isExpression));default:return a}}}function p(n){return e.Debug.assert(n.length<=1,"Too many nodes written to output."),e.singleOrUndefined(n)}e.visitNode=n,e.visitNodes=t,e.visitLexicalEnvironment=r,e.visitParameterList=i,e.visitFunctionBody=l,e.visitIterationBody=u,e.visitEachChild=d}(d||(d={})),function(e){function n(n,t,r,i,a){var o,s,c=a.extendedDiagnostics?e.performance.createTimer("Source Map","beforeSourcemap","afterSourcemap"):e.performance.nullTimer,u=c.enter,d=c.exit,f=[],m=[],_=new e.Map,g=[],y=[],h="",v=0,b=0,E=0,T=0,S=0,x=0,L=!1,D=0,A=0,C=0,I=0,N=0,k=0,R=!1,O=!1,w=!1;return{getSources:function(){return f},addSource:P,setSourceContent:M,addName:F,addMapping:U,appendSourceMap:V,toJSON:z,toString:function(){return JSON.stringify(z())}};function P(t){u();var r=e.getRelativePathToDirectoryOrUrl(i,t,n.getCurrentDirectory(),n.getCanonicalFileName,!0),a=_.get(r);return void 0===a&&(a=m.length,m.push(r),f.push(t),_.set(r,a)),d(),a}function M(e,n){if(u(),null!==n){o||(o=[]);while(o.length<e)o.push(null);o[e]=n}d()}function F(n){u(),s||(s=new e.Map);var t=s.get(n);return void 0===t&&(t=g.length,g.push(n),s.set(n,t)),d(),t}function G(e,n){return!R||D!==e||A!==n}function B(e,n,t){return void 0!==e&&void 0!==n&&void 0!==t&&C===e&&(I>n||I===n&&N>t)}function U(n,t,r,i,a,o){e.Debug.assert(n>=D,"generatedLine cannot backtrack"),e.Debug.assert(t>=0,"generatedCharacter cannot be negative"),e.Debug.assert(void 0===r||r>=0,"sourceIndex cannot be negative"),e.Debug.assert(void 0===i||i>=0,"sourceLine cannot be negative"),e.Debug.assert(void 0===a||a>=0,"sourceCharacter cannot be negative"),u(),(G(n,t)||B(r,i,a))&&(H(),D=n,A=t,O=!1,w=!1,R=!0),void 0!==r&&void 0!==i&&void 0!==a&&(C=r,I=i,N=a,O=!0,void 0!==o&&(k=o,w=!0)),d()}function V(n,t,r,i,a,o){e.Debug.assert(n>=D,"generatedLine cannot backtrack"),e.Debug.assert(t>=0,"generatedCharacter cannot be negative"),u();for(var s,c=[],p=l(r.mappings),f=p.next();!f.done;f=p.next()){var m=f.value;if(o&&(m.generatedLine>o.line||m.generatedLine===o.line&&m.generatedCharacter>o.character))break;if(!a||!(m.generatedLine<a.line||a.line===m.generatedLine&&m.generatedCharacter<a.character)){var _=void 0,g=void 0,y=void 0,h=void 0;if(void 0!==m.sourceIndex){if(_=c[m.sourceIndex],void 0===_){var v=r.sources[m.sourceIndex],b=r.sourceRoot?e.combinePaths(r.sourceRoot,v):v,E=e.combinePaths(e.getDirectoryPath(i),b);c[m.sourceIndex]=_=P(E),r.sourcesContent&&"string"===typeof r.sourcesContent[m.sourceIndex]&&M(_,r.sourcesContent[m.sourceIndex])}g=m.sourceLine,y=m.sourceCharacter,r.names&&void 0!==m.nameIndex&&(s||(s=[]),h=s[m.nameIndex],void 0===h&&(s[m.nameIndex]=h=F(r.names[m.nameIndex])))}var T=m.generatedLine-(a?a.line:0),S=T+n,x=a&&a.line===m.generatedLine?m.generatedCharacter-a.character:m.generatedCharacter,L=0===T?x+t:x;U(S,L,_,g,y,h)}}d()}function j(){return!L||v!==D||b!==A||E!==C||T!==I||S!==N||x!==k}function K(e){y.push(e),y.length>=1024&&W()}function H(){if(R&&j()){if(u(),v<D){do{K(59),v++}while(v<D);b=0}else e.Debug.assertEqual(v,D,"generatedLine cannot backtrack"),L&&K(44);J(A-b),b=A,O&&(J(C-E),E=C,J(I-T),T=I,J(N-S),S=N,w&&(J(k-x),x=k)),L=!0,d()}}function W(){y.length>0&&(h+=String.fromCharCode.apply(void 0,y),y.length=0)}function z(){return H(),W(),{version:3,file:t,sourceRoot:r,sources:m,names:g,mappings:h,sourcesContent:o}}function J(e){e<0?e=1+(-e<<1):e<<=1;do{var n=31&e;e>>=5,e>0&&(n|=32),K(p(n))}while(e>0)}}e.createSourceMapGenerator=n;var t=/^\/\/[@#] source[M]appingURL=(.+)$/,r=/^\s*(\/\/[@#] .*)?$/;function i(e,n){return{getLineCount:function(){return n.length},getLineText:function(t){return e.substring(n[t],n[t+1])}}}function a(n){for(var i=n.getLineCount()-1;i>=0;i--){var a=n.getLineText(i),o=t.exec(a);if(o)return e.trimStringEnd(o[1]);if(!a.match(r))break}}function o(e){return"string"===typeof e||null===e}function s(n){return null!==n&&"object"===typeof n&&3===n.version&&"string"===typeof n.file&&"string"===typeof n.mappings&&e.isArray(n.sources)&&e.every(n.sources,e.isString)&&(void 0===n.sourceRoot||null===n.sourceRoot||"string"===typeof n.sourceRoot)&&(void 0===n.sourcesContent||null===n.sourcesContent||e.isArray(n.sourcesContent)&&e.every(n.sourcesContent,o))&&(void 0===n.names||null===n.names||e.isArray(n.names)&&e.every(n.names,e.isString))}function c(e){try{var n=JSON.parse(e);if(s(n))return n}catch(t){}}function l(e){var n,t=!1,r=0,i=0,a=0,o=0,s=0,c=0,l=0;return{get pos(){return r},get error(){return n},get state(){return u(!0,!0)},next:function(){while(!t&&r<e.length){var n=e.charCodeAt(r);if(59!==n){if(44!==n){var p=!1,f=!1;if(a+=y(),_())return d();if(a<0)return m("Invalid generatedCharacter found");if(!g()){if(p=!0,o+=y(),_())return d();if(o<0)return m("Invalid sourceIndex found");if(g())return m("Unsupported Format: No entries after sourceIndex");if(s+=y(),_())return d();if(s<0)return m("Invalid sourceLine found");if(g())return m("Unsupported Format: No entries after sourceLine");if(c+=y(),_())return d();if(c<0)return m("Invalid sourceCharacter found");if(!g()){if(f=!0,l+=y(),_())return d();if(l<0)return m("Invalid nameIndex found");if(!g())return m("Unsupported Error Format: Entries after nameIndex")}}return{value:u(p,f),done:t}}r++}else i++,a=0,r++}return d()}};function u(e,n){return{generatedLine:i,generatedCharacter:a,sourceIndex:e?o:void 0,sourceLine:e?s:void 0,sourceCharacter:e?c:void 0,nameIndex:n?l:void 0}}function d(){return t=!0,{value:void 0,done:!0}}function p(e){void 0===n&&(n=e)}function m(e){return p(e),d()}function _(){return void 0!==n}function g(){return r===e.length||44===e.charCodeAt(r)||59===e.charCodeAt(r)}function y(){for(var n=!0,t=0,i=0;n;r++){if(r>=e.length)return p("Error in decoding base64VLQFormatDecode, past the mapping string"),-1;var a=f(e.charCodeAt(r));if(-1===a)return p("Invalid character in VLQ"),-1;n=0!==(32&a),i|=(31&a)<<t,t+=5}return 0===(1&i)?i>>=1:(i>>=1,i=-i),i}}function u(e,n){return e===n||e.generatedLine===n.generatedLine&&e.generatedCharacter===n.generatedCharacter&&e.sourceIndex===n.sourceIndex&&e.sourceLine===n.sourceLine&&e.sourceCharacter===n.sourceCharacter&&e.nameIndex===n.nameIndex}function d(e){return void 0!==e.sourceIndex&&void 0!==e.sourceLine&&void 0!==e.sourceCharacter}function p(n){return n>=0&&n<26?65+n:n>=26&&n<52?97+n-26:n>=52&&n<62?48+n-52:62===n?43:63===n?47:e.Debug.fail(n+": not a base64 value")}function f(e){return e>=65&&e<=90?e-65:e>=97&&e<=122?e-97+26:e>=48&&e<=57?e-48+52:43===e?62:47===e?63:-1}function m(e){return void 0!==e.sourceIndex&&void 0!==e.sourcePosition}function _(e,n){return e.generatedPosition===n.generatedPosition&&e.sourceIndex===n.sourceIndex&&e.sourcePosition===n.sourcePosition}function g(n,t){return e.Debug.assert(n.sourceIndex===t.sourceIndex),e.compareValues(n.sourcePosition,t.sourcePosition)}function y(n,t){return e.compareValues(n.generatedPosition,t.generatedPosition)}function h(e){return e.sourcePosition}function v(e){return e.generatedPosition}function b(n,t,r){var i,a,o,s=e.getDirectoryPath(r),c=t.sourceRoot?e.getNormalizedAbsolutePath(t.sourceRoot,s):s,u=e.getNormalizedAbsolutePath(t.file,s),p=n.getSourceFileLike(u),f=t.sources.map(function(n){return e.getNormalizedAbsolutePath(n,c)}),b=new e.Map(f.map(function(e,t){return[n.getCanonicalFileName(e),t]}));return{getSourcePosition:D,getGeneratedPosition:L};function E(r){var i,a,o=void 0!==p?e.getPositionOfLineAndCharacter(p,r.generatedLine,r.generatedCharacter,!0):-1;if(d(r)){var s=n.getSourceFileLike(f[r.sourceIndex]);i=t.sources[r.sourceIndex],a=void 0!==s?e.getPositionOfLineAndCharacter(s,r.sourceLine,r.sourceCharacter,!0):-1}return{generatedPosition:o,source:i,sourceIndex:r.sourceIndex,sourcePosition:a,nameIndex:r.nameIndex}}function T(){if(void 0===i){var r=l(t.mappings),a=e.arrayFrom(r,E);void 0!==r.error?(n.log&&n.log("Encountered error while decoding sourcemap: "+r.error),i=e.emptyArray):i=a}return i}function S(n){if(void 0===o){for(var t=[],r=0,i=T();r<i.length;r++){var a=i[r];if(m(a)){var s=t[a.sourceIndex];s||(t[a.sourceIndex]=s=[]),s.push(a)}}o=t.map(function(n){return e.sortAndDeduplicate(n,g,_)})}return o[n]}function x(){if(void 0===a){for(var n=[],t=0,r=T();t<r.length;t++){var i=r[t];n.push(i)}a=e.sortAndDeduplicate(n,y,_)}return a}function L(t){var r=b.get(n.getCanonicalFileName(t.fileName));if(void 0===r)return t;var i=S(r);if(!e.some(i))return t;var a=e.binarySearchKey(i,t.pos,h,e.compareValues);a<0&&(a=~a);var o=i[a];return void 0===o||o.sourceIndex!==r?t:{fileName:u,pos:o.generatedPosition}}function D(n){var t=x();if(!e.some(t))return n;var r=e.binarySearchKey(t,n.pos,v,e.compareValues);r<0&&(r=~r);var i=t[r];return void 0!==i&&m(i)?{fileName:f[i.sourceIndex],pos:i.sourcePosition}:n}}e.getLineInfo=i,e.tryGetSourceMappingURL=a,e.isRawSourceMap=s,e.tryParseRawSourceMap=c,e.decodeMappings=l,e.sameMapping=u,e.isSourceMapping=d,e.createDocumentPositionMapper=b,e.identitySourceMapConsumer={getSourcePosition:e.identity,getGeneratedPosition:e.identity}}(d||(d={})),function(e){function n(n){return n=e.getOriginalNode(n),n?e.getNodeId(n):0}function t(n){return!!n&&(!!e.isNamedImports(n)&&e.some(n.elements,r))}function r(e){return void 0!==e.propertyName&&"default"===e.propertyName.escapedText}function i(n,t){return r;function r(e){return 300===e.kind?t(e):i(e)}function i(r){return n.factory.createBundle(e.map(r.sourceFiles,t),r.prepends)}}function a(n){return!!e.getNamespaceDeclarationNode(n)}function o(n){if(e.getNamespaceDeclarationNode(n))return!0;var t=n.importClause&&n.importClause.namedBindings;if(!t)return!1;if(!e.isNamedImports(t))return!1;for(var i=0,a=0,o=t.elements;a<o.length;a++){var s=o[a];r(s)&&i++}return i>0&&i!==t.elements.length||!!(t.elements.length-i)&&e.isDefaultImport(n)}function s(n){return!o(n)&&(e.isDefaultImport(n)||!!n.importClause&&e.isNamedImports(n.importClause.namedBindings)&&t(n.importClause.namedBindings))}function c(t,r,i,a){for(var c,d,p=[],f=e.createMultiMap(),m=[],_=new e.Map,g=!1,y=!1,h=!1,v=!1,b=0,E=r.statements;b<E.length;b++){var T=E[b];switch(T.kind){case 264:p.push(T),!h&&o(T)&&(h=!0),!v&&s(T)&&(v=!0);break;case 263:275===T.moduleReference.kind&&p.push(T);break;case 270:if(T.moduleSpecifier)if(T.exportClause)if(p.push(T),e.isNamedExports(T.exportClause))C(T);else{var S=T.exportClause.name;_.get(e.idText(S))||(u(m,n(T),S),_.set(e.idText(S),!0),c=e.append(c,S)),h=!0}else p.push(T),y=!0;else C(T);break;case 269:T.isExportEquals&&!d&&(d=T);break;case 235:if(e.hasSyntacticModifier(T,1))for(var x=0,L=T.declarationList.declarations;x<L.length;x++){var D=L[x];c=l(D,_,c)}break;case 254:if(e.hasSyntacticModifier(T,1))if(e.hasSyntacticModifier(T,512))g||(u(m,n(T),t.factory.getDeclarationName(T)),g=!0);else{S=T.name;_.get(e.idText(S))||(u(m,n(T),S),_.set(e.idText(S),!0),c=e.append(c,S))}break;case 255:if(e.hasSyntacticModifier(T,1))if(e.hasSyntacticModifier(T,512))g||(u(m,n(T),t.factory.getDeclarationName(T)),g=!0);else{S=T.name;S&&!_.get(e.idText(S))&&(u(m,n(T),S),_.set(e.idText(S),!0),c=e.append(c,S))}break}}var A=e.createExternalHelpersImportDeclarationIfNeeded(t.factory,t.getEmitHelperFactory(),r,a,y,h,v);return A&&p.unshift(A),{externalImports:p,exportSpecifiers:f,exportEquals:d,hasExportStarsToExportValues:y,exportedBindings:m,exportedNames:c,externalHelpersImportDeclaration:A};function C(t){for(var r=0,a=e.cast(t.exportClause,e.isNamedExports).elements;r<a.length;r++){var o=a[r];if(!_.get(e.idText(o.name))){var s=o.propertyName||o.name;t.moduleSpecifier||f.add(e.idText(s),o);var l=i.getReferencedImportDeclaration(s)||i.getReferencedValueDeclaration(s);l&&u(m,n(l),o.name),_.set(e.idText(o.name),!0),c=e.append(c,o.name)}}}}function l(n,t,r){if(e.isBindingPattern(n.name))for(var i=0,a=n.name.elements;i<a.length;i++){var o=a[i];e.isOmittedExpression(o)||(r=l(o,t,r))}else if(!e.isGeneratedIdentifier(n.name)){var s=e.idText(n.name);t.get(s)||(t.set(s,!0),r=e.append(r,n.name))}return r}function u(e,n,t){var r=e[n];return r?r.push(t):e[n]=r=[t],r}function d(n){return e.isStringLiteralLike(n)||8===n.kind||e.isKeyword(n.kind)||e.isIdentifier(n)}function p(n){return!e.isIdentifier(n)&&d(n)}function f(e){return e>=64&&e<=78}function m(e){switch(e){case 64:return 39;case 65:return 40;case 66:return 41;case 67:return 42;case 68:return 43;case 69:return 44;case 70:return 47;case 71:return 48;case 72:return 49;case 73:return 50;case 74:return 51;case 78:return 52;case 75:return 56;case 76:return 55;case 77:return 60}}function _(n,t,r,i){if(t.body){var a=t.body.statements,o=n.copyPrologue(a,r,!1,i);if(o===a.length)return o;var s=e.findIndex(a,function(n){return e.isExpressionStatement(n)&&e.isSuperCall(n.expression)},o);if(s>-1){for(var c=o;c<=s;c++)r.push(e.visitNode(a[c],i,e.isStatement));return s+1}return o}return 0}function g(n,t,r){return e.filter(n.members,function(e){return v(e,t,r)})}function y(n){return b(n)||e.isClassStaticBlockDeclaration(n)}function h(n){return e.filter(n.members,y)}function v(n,t,r){return e.isPropertyDeclaration(n)&&(!!n.initializer||!t)&&e.hasStaticModifier(n)===r}function b(n){return e.isPropertyDeclaration(n)&&e.hasStaticModifier(n)}function E(e){return 165===e.kind&&void 0!==e.initializer}function T(n){return!e.isStatic(n)&&e.isMethodOrAccessor(n)&&e.isPrivateIdentifier(n.name)}e.getOriginalNodeId=n,e.chainBundle=i,e.getExportNeedsImportStarHelper=a,e.getImportNeedsImportStarHelper=o,e.getImportNeedsImportDefaultHelper=s,e.collectExternalModuleInfo=c,e.isSimpleCopiableExpression=d,e.isSimpleInlineableExpression=p,e.isCompoundAssignment=f,e.getNonAssignmentOperatorForCompoundAssignment=m,e.addPrologueDirectivesAndInitialSuperCall=_,e.getProperties=g,e.getStaticPropertiesAndClassStaticBlock=h,e.isInitializedProperty=E,e.isNonStaticMethodOrAccessorWithPrivateName=T}(d||(d={})),function(e){function n(n,r,a,o,c,l){var u,d,p=n;if(e.isDestructuringAssignment(n)){u=n.right;while(e.isEmptyArrayLiteral(n.left)||e.isEmptyObjectLiteral(n.left)){if(!e.isDestructuringAssignment(u))return e.visitNode(u,r,e.isExpression);p=n=u,u=n.right}}var m={context:a,level:o,downlevelIteration:!!a.getCompilerOptions().downlevelIteration,hoistTempVariables:!0,emitExpression:g,emitBindingOrAssignment:h,createArrayBindingOrAssignmentPattern:function(e){return _(a.factory,e)},createObjectBindingOrAssignmentPattern:function(e){return y(a.factory,e)},createArrayBindingOrAssignmentElement:v,visitor:r};if(u&&(u=e.visitNode(u,r,e.isExpression),e.isIdentifier(u)&&t(n,u.escapedText)||i(n)?u=f(m,u,!1,p):c?u=f(m,u,!0,p):e.nodeIsSynthesized(n)&&(p=u)),s(m,n,u,p,e.isDestructuringAssignment(n)),u&&c){if(!e.some(d))return u;d.push(u)}return a.factory.inlineExpressions(d)||a.factory.createOmittedExpression();function g(n){d=e.append(d,n)}function h(n,t,i,o){e.Debug.assertNode(n,l?e.isIdentifier:e.isExpression);var s=l?l(n,t,i):e.setTextRange(a.factory.createAssignment(e.visitNode(n,r,e.isExpression),t),i);s.original=o,g(s)}}function t(n,t){var i=e.getTargetOfBindingOrAssignmentElement(n);return e.isBindingOrAssignmentPattern(i)?r(i,t):!!e.isIdentifier(i)&&i.escapedText===t}function r(n,r){for(var i=e.getElementsOfBindingOrAssignmentPattern(n),a=0,o=i;a<o.length;a++){var s=o[a];if(t(s,r))return!0}return!1}function i(n){var t=e.tryGetPropertyNameOfBindingOrAssignmentElement(n);if(t&&e.isComputedPropertyName(t)&&!e.isLiteralExpression(t.expression))return!0;var r=e.getTargetOfBindingOrAssignmentElement(n);return!!r&&e.isBindingOrAssignmentPattern(r)&&a(r)}function a(n){return!!e.forEach(e.getElementsOfBindingOrAssignmentPattern(n),i)}function o(n,r,a,o,c,l,u){var d;void 0===l&&(l=!1);var p=[],_=[],y={context:a,level:o,downlevelIteration:!!a.getCompilerOptions().downlevelIteration,hoistTempVariables:l,emitExpression:k,emitBindingOrAssignment:R,createArrayBindingOrAssignmentPattern:function(e){return m(a.factory,e)},createObjectBindingOrAssignmentPattern:function(e){return g(a.factory,e)},createArrayBindingOrAssignmentElement:function(e){return h(a.factory,e)},visitor:r};if(e.isVariableDeclaration(n)){var v=e.getInitializerOfBindingOrAssignmentElement(n);v&&(e.isIdentifier(v)&&t(n,v.escapedText)||i(n))&&(v=f(y,e.visitNode(v,y.visitor),!1,v),n=a.factory.updateVariableDeclaration(n,n.name,void 0,void 0,v))}if(s(y,n,c,n,u),d){var b=a.factory.createTempVariable(void 0);if(l){var E=a.factory.inlineExpressions(d);d=void 0,R(b,E,void 0,void 0)}else{a.hoistVariableDeclaration(b);var T=e.last(p);T.pendingExpressions=e.append(T.pendingExpressions,a.factory.createAssignment(b,T.value)),e.addRange(T.pendingExpressions,d),T.value=b}}for(var S=0,x=p;S<x.length;S++){var L=x[S],D=L.pendingExpressions,A=L.name,C=(E=L.value,L.location),I=L.original,N=a.factory.createVariableDeclaration(A,void 0,void 0,D?a.factory.inlineExpressions(e.append(D,E)):E);N.original=I,e.setTextRange(N,C),_.push(N)}return _;function k(n){d=e.append(d,n)}function R(n,t,r,i){e.Debug.assertNode(n,e.isBindingName),d&&(t=a.factory.inlineExpressions(e.append(d,t)),d=void 0),p.push({pendingExpressions:d,name:n,value:t,location:r,original:i})}}function s(n,t,r,i,a){var o=e.getTargetOfBindingOrAssignmentElement(t);if(!a){var s=e.visitNode(e.getInitializerOfBindingOrAssignmentElement(t),n.visitor,e.isExpression);s?r?(r=d(n,r,s,i),!e.isSimpleInlineableExpression(s)&&e.isBindingOrAssignmentPattern(o)&&(r=f(n,r,!0,i))):r=s:r||(r=n.context.factory.createVoidZero())}e.isObjectBindingOrAssignmentPattern(o)?c(n,t,o,r,i):e.isArrayBindingOrAssignmentPattern(o)?l(n,t,o,r,i):n.emitBindingOrAssignment(o,r,i,t)}function c(n,t,r,i,a){var o,c,l=e.getElementsOfBindingOrAssignmentPattern(r),u=l.length;if(1!==u){var d=!e.isDeclarationBindingElement(t)||0!==u;i=f(n,i,d,a)}for(var m=0;m<u;m++){var _=l[m];if(e.getRestIndicatorOfBindingOrAssignmentElement(_)){if(m===u-1){o&&(n.emitBindingOrAssignment(n.createObjectBindingOrAssignmentPattern(o),i,a,r),o=void 0);y=n.context.getEmitHelperFactory().createRestHelper(i,l,c,r);s(n,_,y,_)}}else{var g=e.getPropertyNameOfBindingOrAssignmentElement(_);if(!(n.level>=1)||49152&_.transformFlags||49152&e.getTargetOfBindingOrAssignmentElement(_).transformFlags||e.isComputedPropertyName(g)){o&&(n.emitBindingOrAssignment(n.createObjectBindingOrAssignmentPattern(o),i,a,r),o=void 0);var y=p(n,i,g);e.isComputedPropertyName(g)&&(c=e.append(c,y.argumentExpression)),s(n,_,y,_)}else o=e.append(o,e.visitNode(_,n.visitor))}}o&&n.emitBindingOrAssignment(n.createObjectBindingOrAssignmentPattern(o),i,a,r)}function l(n,t,r,i,a){var o,c,l=e.getElementsOfBindingOrAssignmentPattern(r),d=l.length;if(n.level<1&&n.downlevelIteration)i=f(n,e.setTextRange(n.context.getEmitHelperFactory().createReadHelper(i,d>0&&e.getRestIndicatorOfBindingOrAssignmentElement(l[d-1])?void 0:d),a),!1,a);else if(1!==d&&(n.level<1||0===d)||e.every(l,e.isOmittedExpression)){var p=!e.isDeclarationBindingElement(t)||0!==d;i=f(n,i,p,a)}for(var m=0;m<d;m++){var _=l[m];if(n.level>=1)if(32768&_.transformFlags||n.hasTransformedPriorElement&&!u(_)){n.hasTransformedPriorElement=!0;var g=n.context.factory.createTempVariable(void 0);n.hoistTempVariables&&n.context.hoistVariableDeclaration(g),c=e.append(c,[g,_]),o=e.append(o,n.createArrayBindingOrAssignmentElement(g))}else o=e.append(o,_);else{if(e.isOmittedExpression(_))continue;if(e.getRestIndicatorOfBindingOrAssignmentElement(_)){if(m===d-1){y=n.context.factory.createArraySliceCall(i,m);s(n,_,y,_)}}else{var y=n.context.factory.createElementAccessExpression(i,m);s(n,_,y,_)}}}if(o&&n.emitBindingOrAssignment(n.createArrayBindingOrAssignmentPattern(o),i,a,r),c)for(var h=0,v=c;h<v.length;h++){var b=v[h],E=b[0];_=b[1];s(n,_,E,_)}}function u(n){var t=e.getTargetOfBindingOrAssignmentElement(n);if(!t||e.isOmittedExpression(t))return!0;var r=e.tryGetPropertyNameOfBindingOrAssignmentElement(n);if(r&&!e.isPropertyNameLiteral(r))return!1;var i=e.getInitializerOfBindingOrAssignmentElement(n);return!(i&&!e.isSimpleInlineableExpression(i))&&(e.isBindingOrAssignmentPattern(t)?e.every(e.getElementsOfBindingOrAssignmentPattern(t),u):e.isIdentifier(t))}function d(e,n,t,r){return n=f(e,n,!0,r),e.context.factory.createConditionalExpression(e.context.factory.createTypeCheck(n,"undefined"),void 0,t,void 0,n)}function p(n,t,r){if(e.isComputedPropertyName(r)){var i=f(n,e.visitNode(r.expression,n.visitor),!1,r);return n.context.factory.createElementAccessExpression(t,i)}if(e.isStringOrNumericLiteralLike(r)){i=e.factory.cloneNode(r);return n.context.factory.createElementAccessExpression(t,i)}var a=n.context.factory.createIdentifier(e.idText(r));return n.context.factory.createPropertyAccessExpression(t,a)}function f(n,t,r,i){if(e.isIdentifier(t)&&r)return t;var a=n.context.factory.createTempVariable(void 0);return n.hoistTempVariables?(n.context.hoistVariableDeclaration(a),n.emitExpression(e.setTextRange(n.context.factory.createAssignment(a,t),i))):n.emitBindingOrAssignment(a,t,i,void 0),a}function m(n,t){return e.Debug.assertEachNode(t,e.isArrayBindingElement),n.createArrayBindingPattern(t)}function _(n,t){return n.createArrayLiteralExpression(e.map(t,n.converters.convertToArrayAssignmentElement))}function g(n,t){return e.Debug.assertEachNode(t,e.isBindingElement),n.createObjectBindingPattern(t)}function y(n,t){return n.createObjectLiteralExpression(e.map(t,n.converters.convertToObjectAssignmentElement))}function h(e,n){return e.createBindingElement(void 0,void 0,n)}function v(e){return e}(function(e){e[e["All"]=0]="All",e[e["ObjectRest"]=1]="ObjectRest"})(e.FlattenLevel||(e.FlattenLevel={})),e.flattenDestructuringAssignment=n,e.flattenDestructuringBinding=o}(d||(d={})),function(e){var n;function t(t,a,o,s,c,l){var u=e.visitNode(a.tag,o,e.isExpression),d=[void 0],p=[],f=[],m=a.template;if(l===n.LiftRestriction&&!e.hasInvalidEscape(m))return e.visitEachChild(a,o,t);if(e.isNoSubstitutionTemplateLiteral(m))p.push(r(m)),f.push(i(m,s));else{p.push(r(m.head)),f.push(i(m.head,s));for(var _=0,g=m.templateSpans;_<g.length;_++){var y=g[_];p.push(r(y.literal)),f.push(i(y.literal,s)),d.push(e.visitNode(y.expression,o,e.isExpression))}}var h=t.getEmitHelperFactory().createTemplateObjectHelper(e.factory.createArrayLiteralExpression(p),e.factory.createArrayLiteralExpression(f));if(e.isExternalModule(s)){var v=e.factory.createUniqueName("templateObject");c(v),d[0]=e.factory.createLogicalOr(v,e.factory.createAssignment(v,h))}else d[0]=h;return e.factory.createCallExpression(u,void 0,d)}function r(n){return n.templateFlags?e.factory.createVoidZero():e.factory.createStringLiteral(n.text)}function i(n,t){var r=n.rawText;if(void 0===r){e.Debug.assertIsDefined(t,"Template literal node is missing 'rawText' and does not have a source file. Possibly bad transform."),r=e.getSourceTextOfNodeFromSourceFile(t,n);var i=14===n.kind||17===n.kind;r=r.substring(1,r.length-(i?1:2))}return r=r.replace(/\r\n?/g,"\n"),e.setTextRange(e.factory.createStringLiteral(r),n)}(function(e){e[e["LiftRestriction"]=0]="LiftRestriction",e[e["All"]=1]="All"})(n=e.ProcessLevel||(e.ProcessLevel={})),e.processTaggedTemplateExpression=t}(d||(d={})),function(e){var n,t,r=!1;function i(n){var t,i,a,o,s,c,l,u,d,p,f=n.factory,m=n.getEmitHelperFactory,_=n.startLexicalEnvironment,g=n.resumeLexicalEnvironment,y=n.endLexicalEnvironment,h=n.hoistVariableDeclaration,v=n.getEmitResolver(),b=n.getCompilerOptions(),E=e.getStrictOptionValue(b,"strictNullChecks"),T=e.getEmitScriptTarget(b),S=e.getEmitModuleKind(b),x=n.onEmitNode,L=n.onSubstituteNode;return n.onEmitNode=nt,n.onSubstituteNode=tt,n.enableSubstitution(204),n.enableSubstitution(205),D;function D(e){return 301===e.kind?A(e):C(e)}function A(n){return f.createBundle(n.sourceFiles.map(C),e.mapDefined(n.prepends,function(n){return 303===n.kind?e.createUnparsedSourceFile(n,"js"):n}))}function C(r){if(r.isDeclarationFile)return r;t=r;var i=I(r,j);return e.addEmitHelpers(i,n.readEmitHelpers()),t=void 0,i}function I(e,n){var t=o,r=s,i=c,a=l;N(e);var u=n(e);return o!==t&&(c=i),o=t,s=r,l=a,u}function N(n){switch(n.kind){case 300:case 261:case 260:case 233:o=n,s=void 0,c=void 0;break;case 255:case 254:if(e.hasSyntacticModifier(n,2))break;n.name?yn(n):e.Debug.assert(255===n.kind||e.hasSyntacticModifier(n,512)),e.isClassDeclaration(n)&&(s=n);break}}function k(e){return I(e,R)}function R(e){return 1&e.transformFlags?V(e):e}function O(e){return I(e,w)}function w(e){switch(e.kind){case 264:case 263:case 269:case 270:return P(e);default:return R(e)}}function P(t){var r=e.getParseTreeNode(t);if(r!==t)return 1&t.transformFlags?e.visitEachChild(t,k,n):t;switch(t.kind){case 264:return xn(t);case 263:return Pn(t);case 269:return Cn(t);case 270:return In(t);default:e.Debug.fail("Unhandled ellided statement")}}function M(e){return I(e,F)}function F(n){if(270!==n.kind&&264!==n.kind&&265!==n.kind&&(263!==n.kind||275!==n.moduleReference.kind))return 1&n.transformFlags||e.hasSyntacticModifier(n,1)?V(n):n}function G(e){return I(e,B)}function B(n){switch(n.kind){case 169:return Ve(n);case 165:return Ue(n);case 174:case 170:case 171:case 167:case 168:return R(n);case 232:return n;default:return e.Debug.failBadSyntaxKind(n)}}function U(n){if(!(18654&e.modifierToFlag(n.kind))&&(!i||93!==n.kind))return n}function V(t){if(e.isStatement(t)&&e.hasSyntacticModifier(t,2))return f.createNotEmittedStatement(t);switch(t.kind){case 93:case 88:return i?void 0:t;case 123:case 121:case 122:case 126:case 157:case 85:case 134:case 143:case 181:case 182:case 183:case 184:case 180:case 175:case 161:case 129:case 153:case 132:case 148:case 145:case 142:case 114:case 149:case 178:case 177:case 179:case 176:case 185:case 186:case 187:case 189:case 190:case 191:case 192:case 193:case 194:case 174:case 163:return;case 257:return f.createNotEmittedStatement(t);case 165:return Ue(t);case 262:return;case 169:return Ve(t);case 256:return f.createNotEmittedStatement(t);case 255:return z(t);case 224:return X(t);case 289:return Fe(t);case 226:return Ge(t);case 167:return He(t);case 170:return ze(t);case 171:return Je(t);case 254:return qe(t);case 211:return Xe(t);case 212:return Ye(t);case 162:return Qe(t);case 210:return nn(t);case 209:case 227:return tn(t);case 206:return an(t);case 207:return on(t);case 208:return sn(t);case 228:return rn(t);case 258:return dn(t);case 235:return Ze(t);case 252:return en(t);case 259:return En(t);case 263:return Pn(t);case 277:return cn(t);case 278:return ln(t);default:return e.visitEachChild(t,k,n)}}function j(t){var r=e.getStrictOptionValue(b,"alwaysStrict")&&!(e.isExternalModule(t)&&S>=e.ModuleKind.ES2015)&&!e.isJsonSourceFile(t);return f.updateSourceFile(t,e.visitLexicalEnvironment(t.statements,O,n,0,r))}function K(n,t){var r=0;e.some(t)&&(r|=1);var i=e.getEffectiveBaseTypeNode(n);return i&&104!==e.skipOuterExpressions(i.expression).kind&&(r|=64),e.classOrConstructorParameterIsDecorated(n)&&(r|=2),e.childIsDecorated(n)&&(r|=4),Mn(n)?r|=8:Bn(n)?r|=32:Gn(n)&&(r|=16),T<=1&&7&r&&(r|=128),r}function H(e){return!!(4096&e.transformFlags)}function W(n){return e.some(n.decorators)||e.some(n.typeParameters)||e.some(n.heritageClauses,H)||e.some(n.members,H)}function z(r){if(!W(r)&&(!i||!e.hasSyntacticModifier(r,1)))return e.visitEachChild(r,k,n);var a=e.getProperties(r,!0,!0),o=K(r,a);128&o&&n.startLexicalEnvironment();var s=r.name||(5&o?f.getGeneratedNameForNode(r):void 0),c=2&o?q(r,s):J(r,s,o),l=[c];if(ce(l,r,!1),ce(l,r,!0),de(l,r),128&o){var u=e.createTokenRange(e.skipTrivia(t.text,r.members.end),19),d=f.getInternalName(r),p=f.createPartiallyEmittedExpression(d);e.setTextRangeEnd(p,u.end),e.setEmitFlags(p,1536);var m=f.createReturnStatement(p);e.setTextRangePos(m,u.pos),e.setEmitFlags(m,1920),l.push(m),e.insertStatementsAfterStandardPrologue(l,n.endLexicalEnvironment());var _=f.createImmediatelyInvokedArrowFunction(l);e.setEmitFlags(_,33554432);var g=f.createVariableStatement(void 0,f.createVariableDeclarationList([f.createVariableDeclaration(f.getLocalName(r,!1,!1),void 0,void 0,_)]));e.setOriginalNode(g,r),e.setCommentRange(g,r),e.setSourceMapRange(g,e.moveRangePastDecorators(r)),e.startOnNewLine(g),l=[g]}return 8&o?Vn(l,r):(128&o||2&o)&&(32&o?l.push(f.createExportDefault(f.getLocalName(r,!1,!0))):16&o&&l.push(f.createExternalModuleExport(f.getLocalName(r,!1,!0)))),l.length>1&&(l.push(f.createEndOfDeclarationMarker(r)),e.setEmitFlags(c,4194304|e.getEmitFlags(c))),e.singleOrMany(l)}function J(n,t,r){var i=128&r?void 0:e.visitNodes(n.modifiers,U,e.isModifier),a=f.createClassDeclaration(void 0,i,t,void 0,e.visitNodes(n.heritageClauses,k,e.isHeritageClause),Y(n)),o=e.getEmitFlags(n);return 1&r&&(o|=32),e.setTextRange(a,n),e.setOriginalNode(a,n),e.setEmitFlags(a,o),a}function q(n,t){var r=e.moveRangePastDecorators(n),i=Jn(n),a=T<=2?f.getInternalName(n,!1,!0):f.getLocalName(n,!1,!0),o=e.visitNodes(n.heritageClauses,k,e.isHeritageClause),s=Y(n),c=f.createClassExpression(void 0,void 0,t,void 0,o,s);e.setOriginalNode(c,n),e.setTextRange(c,r);var l=f.createVariableStatement(void 0,f.createVariableDeclarationList([f.createVariableDeclaration(a,void 0,void 0,i?f.createAssignment(i,c):c)],1));return e.setOriginalNode(l,n),e.setTextRange(l,r),e.setCommentRange(l,n),l}function X(t){if(!W(t))return e.visitEachChild(t,k,n);var r=f.createClassExpression(void 0,void 0,t.name,void 0,e.visitNodes(t.heritageClauses,k,e.isHeritageClause),Y(t));return e.setOriginalNode(r,t),e.setTextRange(r,t),r}function Y(n){var t=[],r=e.getFirstConstructorWithBody(n),i=r&&e.filter(r.parameters,function(n){return e.isParameterPropertyDeclaration(n,r)});if(i)for(var a=0,o=i;a<o.length;a++){var s=o[a];e.isIdentifier(s.name)&&t.push(e.setOriginalNode(f.createPropertyDeclaration(void 0,void 0,s.name,void 0,void 0,void 0),s))}return e.addRange(t,e.visitNodes(n.members,G,e.isClassElement)),e.setTextRange(f.createNodeArray(t),n.members)}function Q(n,t){return e.filter(n.members,t?function(e){return Z(e,n)}:function(e){return $(e,n)})}function Z(e,n){return ee(e,!0,n)}function $(e,n){return ee(e,!1,n)}function ee(n,t,r){return e.nodeOrChildIsDecorated(n,r)&&t===e.isStatic(n)}function ne(n){var t;if(n)for(var r=n.parameters,i=r.length>0&&e.parameterIsThisKeyword(r[0]),a=i?1:0,o=i?r.length-1:r.length,s=0;s<o;s++){var c=r[s+a];(t||c.decorators)&&(t||(t=new Array(o)),t[s]=c.decorators)}return t}function te(n){var t=n.decorators,r=ne(e.getFirstConstructorWithBody(n));if(t||r)return{decorators:t,parameters:r}}function re(e,n){switch(n.kind){case 170:case 171:return ie(e,n);case 167:return ae(n);case 165:return oe(n);default:return}}function ie(n,t){if(t.body){var r=e.getAllAccessorDeclarations(n.members,t),i=r.firstAccessor,a=r.secondAccessor,o=r.setAccessor,s=i.decorators?i:a&&a.decorators?a:void 0;if(s&&t===s){var c=s.decorators,l=ne(o);if(c||l)return{decorators:c,parameters:l}}}}function ae(e){if(e.body){var n=e.decorators,t=ne(e);if(n||t)return{decorators:n,parameters:t}}}function oe(e){var n=e.decorators;if(n)return{decorators:n}}function se(n,t,r){if(r){var i=[];return e.addRange(i,e.map(r.decorators,fe)),e.addRange(i,e.flatMap(r.parameters,me)),_e(n,t,i),i}}function ce(n,t,r){e.addRange(n,e.map(le(t,r),Un))}function le(e,n){for(var t,r=Q(e,n),i=0,a=r;i<a.length;i++){var o=a[i],s=ue(e,o);s&&(t?t.push(s):t=[s])}return t}function ue(n,t){var r=re(n,t),i=se(t,n,r);if(i){var a=Xn(n,t),o=Pe(t,!0),s=T>0?165===t.kind?f.createVoidZero():f.createNull():void 0,c=m().createDecorateHelper(i,a,o,s);return e.setTextRange(c,e.moveRangePastDecorators(t)),e.setEmitFlags(c,1536),c}}function de(n,t){var r=pe(t);r&&n.push(e.setOriginalNode(f.createExpressionStatement(r),t))}function pe(n){var t=te(n),r=se(n,n,t);if(r){var i=d&&d[e.getOriginalNodeId(n)],a=T<=2?f.getInternalName(n,!1,!0):f.getLocalName(n,!1,!0),o=m().createDecorateHelper(r,a),s=f.createAssignment(a,i?f.createAssignment(i,o):o);return e.setEmitFlags(s,1536),e.setSourceMapRange(s,e.moveRangePastDecorators(n)),s}}function fe(n){return e.visitNode(n.expression,k,e.isExpression)}function me(n,t){var r;if(n){r=[];for(var i=0,a=n;i<a.length;i++){var o=a[i],s=m().createParamHelper(fe(o),t);e.setTextRange(s,o.expression),e.setEmitFlags(s,1536),r.push(s)}}return r}function _e(e,n,t){r?ye(e,n,t):ge(e,n,t)}function ge(e,n,t){b.emitDecoratorMetadata&&(he(e)&&t.push(m().createMetadataHelper("design:type",Te(e))),be(e)&&t.push(m().createMetadataHelper("design:paramtypes",Se(e,n))),ve(e)&&t.push(m().createMetadataHelper("design:returntype",Le(e))))}function ye(e,n,t){if(b.emitDecoratorMetadata){var r=void 0;he(e)&&(r||(r=[])).push(f.createPropertyAssignment("type",f.createArrowFunction(void 0,void 0,[],void 0,f.createToken(38),Te(e)))),be(e)&&(r||(r=[])).push(f.createPropertyAssignment("paramTypes",f.createArrowFunction(void 0,void 0,[],void 0,f.createToken(38),Se(e,n)))),ve(e)&&(r||(r=[])).push(f.createPropertyAssignment("returnType",f.createArrowFunction(void 0,void 0,[],void 0,f.createToken(38),Le(e)))),r&&t.push(m().createMetadataHelper("design:typeinfo",f.createObjectLiteralExpression(r,!0)))}}function he(e){var n=e.kind;return 167===n||170===n||171===n||165===n}function ve(e){return 167===e.kind}function be(n){switch(n.kind){case 255:case 224:return void 0!==e.getFirstConstructorWithBody(n);case 167:case 170:case 171:return!0}return!1}function Ee(n){var t=v.getAllAccessorDeclarations(n);return t.setAccessor&&e.getSetAccessorTypeAnnotationNode(t.setAccessor)||t.getAccessor&&e.getEffectiveReturnTypeNode(t.getAccessor)}function Te(e){switch(e.kind){case 165:case 162:return De(e.type);case 171:case 170:return De(Ee(e));case 255:case 224:case 167:return f.createIdentifier("Function");default:return f.createVoidZero()}}function Se(n,t){var r=e.isClassLike(n)?e.getFirstConstructorWithBody(n):e.isFunctionLike(n)&&e.nodeIsPresent(n.body)?n:void 0,i=[];if(r)for(var a=xe(r,t),o=a.length,s=0;s<o;s++){var c=a[s];0===s&&e.isIdentifier(c.name)&&"this"===c.name.escapedText||(c.dotDotDotToken?i.push(De(e.getRestParameterElementType(c.type))):i.push(Te(c)))}return f.createArrayLiteralExpression(i)}function xe(n,t){if(t&&170===n.kind){var r=e.getAllAccessorDeclarations(t.members,n).setAccessor;if(r)return r.parameters}return n.parameters}function Le(n){return e.isFunctionLike(n)&&n.type?De(n.type):e.isAsyncFunction(n)?f.createIdentifier("Promise"):f.createVoidZero()}function De(n){if(void 0===n)return f.createIdentifier("Object");switch(n.kind){case 114:case 151:case 142:return f.createVoidZero();case 189:return De(n.type);case 177:case 178:return f.createIdentifier("Function");case 181:case 182:return f.createIdentifier("Array");case 175:case 132:return f.createIdentifier("Boolean");case 148:return f.createIdentifier("String");case 146:return f.createIdentifier("Object");case 194:switch(n.literal.kind){case 10:case 14:return f.createIdentifier("String");case 217:case 8:return f.createIdentifier("Number");case 9:return we();case 110:case 95:return f.createIdentifier("Boolean");case 104:return f.createVoidZero();default:return e.Debug.failBadSyntaxKind(n.literal)}case 145:return f.createIdentifier("Number");case 156:return we();case 149:return T<2?Oe():f.createIdentifier("Symbol");case 176:return Ce(n);case 186:case 185:return Ae(n.types);case 187:return Ae([n.trueType,n.falseType]);case 191:if(143===n.operator)return De(n.type);break;case 179:case 192:case 193:case 180:case 129:case 153:case 190:case 198:break;case 307:case 308:case 312:case 313:case 314:break;case 309:case 310:case 311:return De(n.type);default:return e.Debug.failBadSyntaxKind(n)}return f.createIdentifier("Object")}function Ae(n){for(var t,r=0,i=n;r<i.length;r++){var a=i[r];while(189===a.kind)a=a.type;if(142!==a.kind&&(E||(194!==a.kind||104!==a.literal.kind)&&151!==a.kind)){var o=De(a);if(e.isIdentifier(o)&&"Object"===o.escapedText)return o;if(t){if(!e.isIdentifier(t)||!e.isIdentifier(o)||t.escapedText!==o.escapedText)return f.createIdentifier("Object")}else t=o}}return t||f.createVoidZero()}function Ce(n){var t=v.getTypeReferenceSerializationKind(n.typeName,s||o);switch(t){case e.TypeReferenceSerializationKind.Unknown:if(e.findAncestor(n,function(n){return n.parent&&e.isConditionalTypeNode(n.parent)&&(n.parent.trueType===n||n.parent.falseType===n)}))return f.createIdentifier("Object");var r=Ne(n.typeName),i=f.createTempVariable(h);return f.createConditionalExpression(f.createTypeCheck(f.createAssignment(i,r),"function"),void 0,i,void 0,f.createIdentifier("Object"));case e.TypeReferenceSerializationKind.TypeWithConstructSignatureAndValue:return ke(n.typeName);case e.TypeReferenceSerializationKind.VoidNullableOrNeverType:return f.createVoidZero();case e.TypeReferenceSerializationKind.BigIntLikeType:return we();case e.TypeReferenceSerializationKind.BooleanType:return f.createIdentifier("Boolean");case e.TypeReferenceSerializationKind.NumberLikeType:return f.createIdentifier("Number");case e.TypeReferenceSerializationKind.StringLikeType:return f.createIdentifier("String");case e.TypeReferenceSerializationKind.ArrayLikeType:return f.createIdentifier("Array");case e.TypeReferenceSerializationKind.ESSymbolType:return T<2?Oe():f.createIdentifier("Symbol");case e.TypeReferenceSerializationKind.TypeWithCallSignature:return f.createIdentifier("Function");case e.TypeReferenceSerializationKind.Promise:return f.createIdentifier("Promise");case e.TypeReferenceSerializationKind.ObjectType:return f.createIdentifier("Object");default:return e.Debug.assertNever(t)}}function Ie(e,n){return f.createLogicalAnd(f.createStrictInequality(f.createTypeOfExpression(e),f.createStringLiteral("undefined")),n)}function Ne(e){if(79===e.kind){var n=ke(e);return Ie(n,n)}if(79===e.left.kind)return Ie(ke(e.left),ke(e));var t=Ne(e.left),r=f.createTempVariable(h);return f.createLogicalAnd(f.createLogicalAnd(t.left,f.createStrictInequality(f.createAssignment(r,t.right),f.createVoidZero())),f.createPropertyAccessExpression(r,e.right))}function ke(n){switch(n.kind){case 79:var t=e.setParent(e.setTextRange(e.parseNodeFactory.cloneNode(n),n),n.parent);return t.original=void 0,e.setParent(t,e.getParseTreeNode(o)),t;case 159:return Re(n)}}function Re(e){return f.createPropertyAccessExpression(ke(e.left),e.right)}function Oe(){return f.createConditionalExpression(f.createTypeCheck(f.createIdentifier("Symbol"),"function"),void 0,f.createIdentifier("Symbol"),void 0,f.createIdentifier("Object"))}function we(){return T<99?f.createConditionalExpression(f.createTypeCheck(f.createIdentifier("BigInt"),"function"),void 0,f.createIdentifier("BigInt"),void 0,f.createIdentifier("Object")):f.createIdentifier("BigInt")}function Pe(n,t){var r=n.name;return e.isPrivateIdentifier(r)?f.createIdentifier(""):e.isComputedPropertyName(r)?t&&!e.isSimpleInlineableExpression(r.expression)?f.getGeneratedNameForNode(r):r.expression:e.isIdentifier(r)?f.createStringLiteral(e.idText(r)):f.cloneNode(r)}function Me(n){var t=n.name;if(e.isComputedPropertyName(t)&&(!e.hasStaticModifier(n)&&l||e.some(n.decorators))){var r=e.visitNode(t.expression,k,e.isExpression),i=e.skipPartiallyEmittedExpressions(r);if(!e.isSimpleInlineableExpression(i)){var a=f.getGeneratedNameForNode(t);return h(a),f.updateComputedPropertyName(t,f.createAssignment(a,r))}}return e.visitNode(t,k,e.isPropertyName)}function Fe(t){if(117!==t.token)return e.visitEachChild(t,k,n)}function Ge(n){return f.updateExpressionWithTypeArguments(n,e.visitNode(n.expression,k,e.isLeftHandSideExpression),void 0)}function Be(n){return!e.nodeIsMissing(n.body)}function Ue(n){if(!(8388608&n.flags||e.hasSyntacticModifier(n,128))){var t=f.updatePropertyDeclaration(n,void 0,e.visitNodes(n.modifiers,k,e.isModifier),Me(n),void 0,void 0,e.visitNode(n.initializer,k));return t!==n&&(e.setCommentRange(t,n),e.setSourceMapRange(t,e.moveRangePastDecorators(n))),t}}function Ve(t){if(Be(t))return f.updateConstructorDeclaration(t,void 0,void 0,e.visitParameterList(t.parameters,k,n),je(t.body,t))}function je(t,r){var i=r&&e.filter(r.parameters,function(n){return e.isParameterPropertyDeclaration(n,r)});if(!e.some(i))return e.visitFunctionBody(t,k,n);var a=[],o=0;g(),o=e.addPrologueDirectivesAndInitialSuperCall(f,r,a,k),e.addRange(a,e.map(i,Ke)),e.addRange(a,e.visitNodes(t.statements,k,e.isStatement,o)),a=f.mergeLexicalEnvironment(a,y());var s=f.createBlock(e.setTextRange(f.createNodeArray(a),t.statements),!0);return e.setTextRange(s,t),e.setOriginalNode(s,t),s}function Ke(n){var t=n.name;if(e.isIdentifier(t)){var r=e.setParent(e.setTextRange(f.cloneNode(t),t),t.parent);e.setEmitFlags(r,1584);var i=e.setParent(e.setTextRange(f.cloneNode(t),t),t.parent);return e.setEmitFlags(i,1536),e.startOnNewLine(e.removeAllComments(e.setTextRange(e.setOriginalNode(f.createExpressionStatement(f.createAssignment(e.setTextRange(f.createPropertyAccessExpression(f.createThis(),r),n.name),i)),n),e.moveRangePos(n,-1))))}}function He(t){if(Be(t)){var r=f.updateMethodDeclaration(t,void 0,e.visitNodes(t.modifiers,U,e.isModifier),t.asteriskToken,Me(t),void 0,void 0,e.visitParameterList(t.parameters,k,n),void 0,e.visitFunctionBody(t.body,k,n));return r!==t&&(e.setCommentRange(r,t),e.setSourceMapRange(r,e.moveRangePastDecorators(t))),r}}function We(n){return!(e.nodeIsMissing(n.body)&&e.hasSyntacticModifier(n,128))}function ze(t){if(We(t)){var r=f.updateGetAccessorDeclaration(t,void 0,e.visitNodes(t.modifiers,U,e.isModifier),Me(t),e.visitParameterList(t.parameters,k,n),void 0,e.visitFunctionBody(t.body,k,n)||f.createBlock([]));return r!==t&&(e.setCommentRange(r,t),e.setSourceMapRange(r,e.moveRangePastDecorators(t))),r}}function Je(t){if(We(t)){var r=f.updateSetAccessorDeclaration(t,void 0,e.visitNodes(t.modifiers,U,e.isModifier),Me(t),e.visitParameterList(t.parameters,k,n),e.visitFunctionBody(t.body,k,n)||f.createBlock([]));return r!==t&&(e.setCommentRange(r,t),e.setSourceMapRange(r,e.moveRangePastDecorators(t))),r}}function qe(t){if(!Be(t))return f.createNotEmittedStatement(t);var r=f.updateFunctionDeclaration(t,void 0,e.visitNodes(t.modifiers,U,e.isModifier),t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,k,n),void 0,e.visitFunctionBody(t.body,k,n)||f.createBlock([]));if(Mn(t)){var i=[r];return Vn(i,t),i}return r}function Xe(t){if(!Be(t))return f.createOmittedExpression();var r=f.updateFunctionExpression(t,e.visitNodes(t.modifiers,U,e.isModifier),t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,k,n),void 0,e.visitFunctionBody(t.body,k,n)||f.createBlock([]));return r}function Ye(t){var r=f.updateArrowFunction(t,e.visitNodes(t.modifiers,U,e.isModifier),void 0,e.visitParameterList(t.parameters,k,n),void 0,t.equalsGreaterThanToken,e.visitFunctionBody(t.body,k,n));return r}function Qe(n){if(!e.parameterIsThisKeyword(n)){var t=f.updateParameterDeclaration(n,void 0,void 0,n.dotDotDotToken,e.visitNode(n.name,k,e.isBindingName),void 0,void 0,e.visitNode(n.initializer,k,e.isExpression));return t!==n&&(e.setCommentRange(t,n),e.setTextRange(t,e.moveRangePastModifiers(n)),e.setSourceMapRange(t,e.moveRangePastModifiers(n)),e.setEmitFlags(t.name,32)),t}}function Ze(t){if(Mn(t)){var r=e.getInitializedVariables(t.declarationList);if(0===r.length)return;return e.setTextRange(f.createExpressionStatement(f.inlineExpressions(e.map(r,$e))),t)}return e.visitEachChild(t,k,n)}function $e(t){var r=t.name;return e.isBindingPattern(r)?e.flattenDestructuringAssignment(t,k,n,0,!1,Kn):e.setTextRange(f.createAssignment(Hn(r),e.visitNode(t.initializer,k,e.isExpression)),t)}function en(n){return f.updateVariableDeclaration(n,e.visitNode(n.name,k,e.isBindingName),void 0,void 0,e.visitNode(n.initializer,k,e.isExpression))}function nn(r){var i=e.skipOuterExpressions(r.expression,-7);if(e.isAssertionExpression(i)){var a=e.visitNode(r.expression,k,e.isExpression);return e.length(e.getLeadingCommentRangesOfNode(a,t))?f.updateParenthesizedExpression(r,a):f.createPartiallyEmittedExpression(a,r)}return e.visitEachChild(r,k,n)}function tn(n){var t=e.visitNode(n.expression,k,e.isExpression);return f.createPartiallyEmittedExpression(t,n)}function rn(n){var t=e.visitNode(n.expression,k,e.isLeftHandSideExpression);return f.createPartiallyEmittedExpression(t,n)}function an(n){return f.updateCallExpression(n,e.visitNode(n.expression,k,e.isExpression),void 0,e.visitNodes(n.arguments,k,e.isExpression))}function on(n){return f.updateNewExpression(n,e.visitNode(n.expression,k,e.isExpression),void 0,e.visitNodes(n.arguments,k,e.isExpression))}function sn(n){return f.updateTaggedTemplateExpression(n,e.visitNode(n.tag,k,e.isExpression),void 0,e.visitNode(n.template,k,e.isExpression))}function cn(n){return f.updateJsxSelfClosingElement(n,e.visitNode(n.tagName,k,e.isJsxTagNameExpression),void 0,e.visitNode(n.attributes,k,e.isJsxAttributes))}function ln(n){return f.updateJsxOpeningElement(n,e.visitNode(n.tagName,k,e.isJsxTagNameExpression),void 0,e.visitNode(n.attributes,k,e.isJsxAttributes))}function un(n){return!e.isEnumConst(n)||e.shouldPreserveConstEnums(b)}function dn(n){if(!un(n))return f.createNotEmittedStatement(n);var r=[],i=2,s=bn(r,n);s&&(S===e.ModuleKind.System&&o===t||(i|=512));var c=Wn(n),l=zn(n),u=e.hasSyntacticModifier(n,1)?f.getExternalModuleOrNamespaceExportName(a,n,!1,!0):f.getLocalName(n,!1,!0),d=f.createLogicalOr(u,f.createAssignment(u,f.createObjectLiteralExpression()));if(gn(n)){var p=f.getLocalName(n,!1,!0);d=f.createAssignment(p,d)}var m=f.createExpressionStatement(f.createCallExpression(f.createFunctionExpression(void 0,void 0,void 0,void 0,[f.createParameterDeclaration(void 0,void 0,void 0,c)],void 0,pn(n,l)),void 0,[d]));return e.setOriginalNode(m,n),s&&(e.setSyntheticLeadingComments(m,void 0),e.setSyntheticTrailingComments(m,void 0)),e.setTextRange(m,n),e.addEmitFlags(m,i),r.push(m),r.push(f.createEndOfDeclarationMarker(n)),r}function pn(n,t){var r=a;a=t;var i=[];_();var o=e.map(n.members,fn);return e.insertStatementsAfterStandardPrologue(i,y()),e.addRange(i,o),a=r,f.createBlock(e.setTextRange(f.createNodeArray(i),n.members),!0)}function fn(n){var t=Pe(n,!1),r=mn(n),i=f.createAssignment(f.createElementAccessExpression(a,t),r),o=10===r.kind?i:f.createAssignment(f.createElementAccessExpression(a,i),t);return e.setTextRange(f.createExpressionStatement(e.setTextRange(o,n)),n)}function mn(n){var t=v.getConstantValue(n);return void 0!==t?"string"===typeof t?f.createStringLiteral(t):f.createNumericLiteral(t):(Yn(),n.initializer?e.visitNode(n.initializer,k,e.isExpression):f.createVoidZero())}function _n(n){var t=e.getParseTreeNode(n,e.isModuleDeclaration);return!t||e.isInstantiatedModule(t,e.shouldPreserveConstEnums(b))}function gn(n){return Mn(n)||Fn(n)&&S!==e.ModuleKind.ES2015&&S!==e.ModuleKind.ES2020&&S!==e.ModuleKind.ESNext&&S!==e.ModuleKind.System}function yn(n){c||(c=new e.Map);var t=vn(n);c.has(t)||c.set(t,n)}function hn(e){if(c){var n=vn(e);return c.get(n)===e}return!0}function vn(n){return e.Debug.assertNode(n.name,e.isIdentifier),n.name.escapedText}function bn(n,t){var r=f.createVariableStatement(e.visitNodes(t.modifiers,U,e.isModifier),f.createVariableDeclarationList([f.createVariableDeclaration(f.getLocalName(t,!1,!0))],300===o.kind?0:1));if(e.setOriginalNode(r,t),yn(t),hn(t))return 258===t.kind?e.setSourceMapRange(r.declarationList,t):e.setSourceMapRange(r,t),e.setCommentRange(r,t),e.addEmitFlags(r,4195328),n.push(r),!0;var i=f.createMergeDeclarationMarker(r);return e.setEmitFlags(i,4195840),n.push(i),!1}function En(n){if(!_n(n))return f.createNotEmittedStatement(n);e.Debug.assertNode(n.name,e.isIdentifier,"A TypeScript namespace should have an Identifier name."),Zn();var r=[],i=2,s=bn(r,n);s&&(S===e.ModuleKind.System&&o===t||(i|=512));var c=Wn(n),l=zn(n),u=e.hasSyntacticModifier(n,1)?f.getExternalModuleOrNamespaceExportName(a,n,!1,!0):f.getLocalName(n,!1,!0),d=f.createLogicalOr(u,f.createAssignment(u,f.createObjectLiteralExpression()));if(gn(n)){var p=f.getLocalName(n,!1,!0);d=f.createAssignment(p,d)}var m=f.createExpressionStatement(f.createCallExpression(f.createFunctionExpression(void 0,void 0,void 0,void 0,[f.createParameterDeclaration(void 0,void 0,void 0,c)],void 0,Tn(n,l)),void 0,[d]));return e.setOriginalNode(m,n),s&&(e.setSyntheticLeadingComments(m,void 0),e.setSyntheticTrailingComments(m,void 0)),e.setTextRange(m,n),e.addEmitFlags(m,i),r.push(m),r.push(f.createEndOfDeclarationMarker(n)),r}function Tn(n,t){var r=a,o=i,s=c;a=t,i=n,c=void 0;var l,u,d=[];if(_(),n.body)if(260===n.body.kind)I(n.body,function(n){return e.addRange(d,e.visitNodes(n.statements,M,e.isStatement))}),l=n.body.statements,u=n.body;else{var p=En(n.body);p&&(e.isArray(p)?e.addRange(d,p):d.push(p));var m=Sn(n).body;l=e.moveRangePos(m.statements,-1)}e.insertStatementsAfterStandardPrologue(d,y()),a=r,i=o,c=s;var g=f.createBlock(e.setTextRange(f.createNodeArray(d),l),!0);return e.setTextRange(g,u),n.body&&260===n.body.kind||e.setEmitFlags(g,1536|e.getEmitFlags(g)),g}function Sn(e){if(259===e.body.kind){var n=Sn(e.body);return n||e.body}}function xn(n){if(!n.importClause)return n;if(!n.importClause.isTypeOnly){var t=e.visitNode(n.importClause,Ln,e.isImportClause);return t||1===b.importsNotUsedAsValues||2===b.importsNotUsedAsValues?f.updateImportDeclaration(n,void 0,void 0,t,n.moduleSpecifier):void 0}}function Ln(n){if(!n.isTypeOnly){var t=v.isReferencedAliasDeclaration(n)?n.name:void 0,r=e.visitNode(n.namedBindings,Dn,e.isNamedImportBindings);return t||r?f.updateImportClause(n,!1,t,r):void 0}}function Dn(n){if(266===n.kind)return v.isReferencedAliasDeclaration(n)?n:void 0;var t=e.visitNodes(n.elements,An,e.isImportSpecifier);return e.some(t)?f.updateNamedImports(n,t):void 0}function An(e){return v.isReferencedAliasDeclaration(e)?e:void 0}function Cn(t){return v.isValueAliasDeclaration(t)?e.visitEachChild(t,k,n):void 0}function In(n){if(!n.isTypeOnly){if(!n.exportClause||e.isNamespaceExport(n.exportClause))return n;if(v.isValueAliasDeclaration(n)){var t=e.visitNode(n.exportClause,Rn,e.isNamedExportBindings);return t?f.updateExportDeclaration(n,void 0,void 0,n.isTypeOnly,t,n.moduleSpecifier):void 0}}}function Nn(n){var t=e.visitNodes(n.elements,On,e.isExportSpecifier);return e.some(t)?f.updateNamedExports(n,t):void 0}function kn(n){return f.updateNamespaceExport(n,e.visitNode(n.name,k,e.isIdentifier))}function Rn(n){return e.isNamespaceExport(n)?kn(n):Nn(n)}function On(e){return v.isValueAliasDeclaration(e)?e:void 0}function wn(n){return v.isReferencedAliasDeclaration(n)||!e.isExternalModule(t)&&v.isTopLevelValueImportEqualsWithEntityName(n)}function Pn(t){if(!t.isTypeOnly){if(e.isExternalModuleImportEqualsDeclaration(t)){var r=v.isReferencedAliasDeclaration(t);return r||1!==b.importsNotUsedAsValues?r?e.visitEachChild(t,k,n):void 0:e.setOriginalNode(e.setTextRange(f.createImportDeclaration(void 0,void 0,void 0,t.moduleReference.expression),t),t)}if(wn(t)){var i=e.createExpressionFromEntityName(f,t.moduleReference);return e.setEmitFlags(i,3584),Gn(t)||!Mn(t)?e.setOriginalNode(e.setTextRange(f.createVariableStatement(e.visitNodes(t.modifiers,U,e.isModifier),f.createVariableDeclarationList([e.setOriginalNode(f.createVariableDeclaration(t.name,void 0,void 0,i),t)])),t),t):e.setOriginalNode(jn(t.name,i,t),t)}}}function Mn(n){return void 0!==i&&e.hasSyntacticModifier(n,1)}function Fn(n){return void 0===i&&e.hasSyntacticModifier(n,1)}function Gn(n){return Fn(n)&&!e.hasSyntacticModifier(n,512)}function Bn(n){return Fn(n)&&e.hasSyntacticModifier(n,512)}function Un(e){return f.createExpressionStatement(e)}function Vn(n,t){var r=f.createAssignment(f.getExternalModuleOrNamespaceExportName(a,t,!1,!0),f.getLocalName(t));e.setSourceMapRange(r,e.createRange(t.name?t.name.pos:t.pos,t.end));var i=f.createExpressionStatement(r);e.setSourceMapRange(i,e.createRange(-1,t.end)),n.push(i)}function jn(n,t,r){return e.setTextRange(f.createExpressionStatement(f.createAssignment(f.getNamespaceMemberName(a,n,!1,!0),t)),r)}function Kn(n,t,r){return e.setTextRange(f.createAssignment(Hn(n),t),r)}function Hn(e){return f.getNamespaceMemberName(a,e,!1,!0)}function Wn(n){var t=f.getGeneratedNameForNode(n);return e.setSourceMapRange(t,n.name),t}function zn(e){return f.getGeneratedNameForNode(e)}function Jn(n){if(16777216&v.getNodeCheckFlags(n)){Qn();var t=f.createUniqueName(n.name&&!e.isGeneratedIdentifier(n.name)?e.idText(n.name):"default");return d[e.getOriginalNodeId(n)]=t,h(t),t}}function qn(e){return f.createPropertyAccessExpression(f.getDeclarationName(e),"prototype")}function Xn(n,t){return e.isStatic(t)?f.getDeclarationName(n):qn(n)}function Yn(){0===(8&u)&&(u|=8,n.enableSubstitution(79))}function Qn(){0===(1&u)&&(u|=1,n.enableSubstitution(79),d=[])}function Zn(){0===(2&u)&&(u|=2,n.enableSubstitution(79),n.enableSubstitution(292),n.enableEmitNotification(259))}function $n(n){return 259===e.getOriginalNode(n).kind}function et(n){return 258===e.getOriginalNode(n).kind}function nt(n,r,i){var a=p,o=t;e.isSourceFile(r)&&(t=r),2&u&&$n(r)&&(p|=2),8&u&&et(r)&&(p|=8),x(n,r,i),p=a,t=o}function tt(n,t){return t=L(n,t),1===n?it(t):e.isShorthandPropertyAssignment(t)?rt(t):t}function rt(n){if(2&u){var t=n.name,r=st(t);if(r){if(n.objectAssignmentInitializer){var i=f.createAssignment(r,n.objectAssignmentInitializer);return e.setTextRange(f.createPropertyAssignment(t,i),n)}return e.setTextRange(f.createPropertyAssignment(t,r),n)}}return n}function it(e){switch(e.kind){case 79:return at(e);case 204:return ct(e);case 205:return lt(e)}return e}function at(e){return ot(e)||st(e)||e}function ot(n){if(1&u&&33554432&v.getNodeCheckFlags(n)){var t=v.getReferencedValueDeclaration(n);if(t){var r=d[t.id];if(r){var i=f.cloneNode(r);return e.setSourceMapRange(i,n),e.setCommentRange(i,n),i}}}}function st(n){if(u&p&&!e.isGeneratedIdentifier(n)&&!e.isLocalName(n)){var t=v.getReferencedExportContainer(n,!1);if(t&&300!==t.kind){var r=2&p&&259===t.kind||8&p&&258===t.kind;if(r)return e.setTextRange(f.createPropertyAccessExpression(f.getGeneratedNameForNode(t),n),n)}}}function ct(e){return ut(e)}function lt(e){return ut(e)}function ut(n){var t=dt(n);if(void 0!==t){e.setConstantValue(n,t);var r="string"===typeof t?f.createStringLiteral(t):f.createNumericLiteral(t);if(!b.removeComments){var i=e.getOriginalNode(n,e.isAccessExpression),a=e.isPropertyAccessExpression(i)?e.declarationNameToString(i.name):e.getTextOfNode(i.argumentExpression);e.addSyntheticTrailingComment(r,3," "+a+" ")}return r}return n}function dt(n){if(!b.isolatedModules)return e.isPropertyAccessExpression(n)||e.isElementAccessExpression(n)?v.getConstantValue(n):void 0}}(function(e){e[e["ClassAliases"]=1]="ClassAliases",e[e["NamespaceExports"]=2]="NamespaceExports",e[e["NonQualifiedEnumMembers"]=8]="NonQualifiedEnumMembers"})(n||(n={})),function(e){e[e["None"]=0]="None",e[e["HasStaticInitializedProperties"]=1]="HasStaticInitializedProperties",e[e["HasConstructorDecorators"]=2]="HasConstructorDecorators",e[e["HasMemberDecorators"]=4]="HasMemberDecorators",e[e["IsExportOfNamespace"]=8]="IsExportOfNamespace",e[e["IsNamedExternalExport"]=16]="IsNamedExternalExport",e[e["IsDefaultExternalExport"]=32]="IsDefaultExternalExport",e[e["IsDerivedClass"]=64]="IsDerivedClass",e[e["UseImmediatelyInvokedFunctionExpression"]=128]="UseImmediatelyInvokedFunctionExpression",e[e["HasAnyDecorators"]=6]="HasAnyDecorators",e[e["NeedsName"]=5]="NeedsName",e[e["MayNeedImmediatelyInvokedFunctionExpression"]=7]="MayNeedImmediatelyInvokedFunctionExpression",e[e["IsExported"]=56]="IsExported"}(t||(t={})),e.transformTypeScript=i}(d||(d={})),function(e){var n,t;function r(n){var t=n.factory,r=n.hoistVariableDeclaration,l=n.endLexicalEnvironment,u=n.startLexicalEnvironment,d=n.resumeLexicalEnvironment,p=n.addBlockScopedVariable,f=n.getEmitResolver(),m=n.getCompilerOptions(),_=e.getEmitScriptTarget(m),g=e.getUseDefineForClassFields(m),y=_<99,h=(_<=8||!g)&&_>=2,v=_<=8||!g,b=n.onSubstituteNode;n.onSubstituteNode=Le;var E,T,S,x,L=n.onEmitNode;n.onEmitNode=xe;var D,A,C,I=[],N=new e.Map;return e.chainBundle(n,k);function k(t){var r=n.getCompilerOptions();if(t.isDeclarationFile||g&&99===r.target)return t;var i=e.visitEachChild(t,w,n);return e.addEmitHelpers(i,n.readEmitHelpers()),i}function R(t,r){if(8388608&t.transformFlags)switch(t.kind){case 224:case 255:return re(t);case 165:return K(t);case 235:return B(t);case 80:return F(t);case 168:return ue(t)}if(8388608&t.transformFlags||33554432&t.transformFlags&&h&&C&&D)switch(t.kind){case 217:case 218:return q(t,r);case 219:return ne(t,r);case 206:return Z(t);case 208:return $(t);case 204:return z(t);case 205:return J(t);case 236:return Y(t);case 240:return X(t);case 254:case 211:case 169:case 167:case 170:case 171:var i=C;C=void 0;var a=e.visitEachChild(t,w,n);return C=i,a}return e.visitEachChild(t,w,n)}function O(e){return R(e,!0)}function w(e){return R(e,!1)}function P(t){switch(t.kind){case 289:return e.visitEachChild(t,P,n);case 226:return se(t)}return w(t)}function M(e){switch(e.kind){case 203:case 202:return Ke(e);default:return w(e)}}function F(n){return y?e.setOriginalNode(t.createIdentifier(""),n):n}function G(e){switch(e.kind){case 169:return;case 170:case 171:case 167:return V(e);case 165:return K(e);case 160:return U(e);case 232:return e;default:return w(e)}}function B(t){var r=x;x=[];var i=e.visitEachChild(t,w,n),o=e.some(x)?a([i],x,!0):i;return x=r,o}function U(r){var i=e.visitEachChild(r,w,n);if(e.some(S)){var a=S;a.push(i.expression),S=[],i=t.updateComputedPropertyName(i,t.inlineExpressions(a))}return i}function V(r){if(e.Debug.assert(!e.some(r.decorators)),!y||!e.isPrivateIdentifier(r.name))return e.visitEachChild(r,G,n);var i=Be(r.name);if(e.Debug.assert(i,"Undeclared private name for property declaration."),!i.isValid)return r;var a=j(r);a&&Pe().push(t.createAssignment(a,t.createFunctionExpression(e.filter(r.modifiers,function(n){return!e.isStaticModifier(n)}),r.asteriskToken,a,void 0,e.visitParameterList(r.parameters,G,n),void 0,e.visitFunctionBody(r.body,G,n))))}function j(n){e.Debug.assert(e.isPrivateIdentifier(n.name));var t=Be(n.name);if(e.Debug.assert(t,"Undeclared private name for property declaration."),"m"===t.kind)return t.methodName;if("a"===t.kind){if(e.isGetAccessor(n))return t.getterName;if(e.isSetAccessor(n))return t.setterName}}function K(n){if(e.Debug.assert(!e.some(n.decorators)),e.isPrivateIdentifier(n.name)){if(!y)return t.updatePropertyDeclaration(n,void 0,e.visitNodes(n.modifiers,w,e.isModifier),n.name,void 0,void 0,void 0);var r=Be(n.name);if(e.Debug.assert(r,"Undeclared private name for property declaration."),!r.isValid)return n}var i=Ne(n.name,!!n.initializer||g);i&&!e.isSimpleInlineableExpression(i)&&Pe().push(i)}function H(n,t){return W(n,e.visitNode(t,w,e.isExpression))}function W(t,r){switch(e.setCommentRange(r,e.moveRangePos(r,-1)),t.kind){case"a":return n.getEmitHelperFactory().createClassPrivateFieldGetHelper(r,t.brandCheckIdentifier,t.kind,t.getterName);case"m":return n.getEmitHelperFactory().createClassPrivateFieldGetHelper(r,t.brandCheckIdentifier,t.kind,t.methodName);case"f":return n.getEmitHelperFactory().createClassPrivateFieldGetHelper(r,t.brandCheckIdentifier,t.kind,t.variableName);default:e.Debug.assertNever(t,"Unknown private element type")}}function z(r){if(y&&e.isPrivateIdentifier(r.name)){var i=Be(r.name);if(i)return e.setTextRange(e.setOriginalNode(H(i,r.expression),r),r)}if(h&&e.isSuperProperty(r)&&e.isIdentifier(r.name)&&C&&D){var a=D.classConstructor,o=D.superClassReference,s=D.facts;if(1&s)return Se(r);if(a&&o){var c=t.createReflectGetCall(o,t.createStringLiteralFromNode(r.name),a);return e.setOriginalNode(c,r.expression),e.setTextRange(c,r.expression),c}}return e.visitEachChild(r,w,n)}function J(r){if(h&&e.isSuperProperty(r)&&C&&D){var i=D.classConstructor,a=D.superClassReference,o=D.facts;if(1&o)return Se(r);if(i&&a){var s=t.createReflectGetCall(a,e.visitNode(r.argumentExpression,w,e.isExpression),i);return e.setOriginalNode(s,r.expression),e.setTextRange(s,r.expression),s}}return e.visitEachChild(r,w,n)}function q(i,a){if(45===i.operator||46===i.operator)if(y&&e.isPrivateIdentifierPropertyAccessExpression(i.operand)){var o=void 0;if(o=Be(i.operand.name)){var s=e.visitNode(i.operand.expression,w,e.isExpression),c=Q(s),l=c.readExpression,u=c.initializeExpression,d=H(o,l),p=e.isPrefixUnaryExpression(i)||a?void 0:t.createTempVariable(r);return d=e.expandPreOrPostfixIncrementOrDecrementExpression(t,i,d,r,p),d=te(o,u||l,d,63),e.setOriginalNode(d,i),e.setTextRange(d,i),p&&(d=t.createComma(d,p),e.setTextRange(d,i)),d}}else if(h&&e.isSuperProperty(i.operand)&&C&&D){var f=D.classConstructor,m=D.superClassReference,_=D.facts;if(1&_){var g=Se(i.operand);return e.isPrefixUnaryExpression(i)?t.updatePrefixUnaryExpression(i,g):t.updatePostfixUnaryExpression(i,g)}if(f&&m){var v=void 0,b=void 0;if(e.isPropertyAccessExpression(i.operand)?e.isIdentifier(i.operand.name)&&(b=v=t.createStringLiteralFromNode(i.operand.name)):e.isSimpleInlineableExpression(i.operand.argumentExpression)?b=v=i.operand.argumentExpression:(b=t.createTempVariable(r),v=t.createAssignment(b,e.visitNode(i.operand.argumentExpression,w,e.isExpression))),v&&b){d=t.createReflectGetCall(m,b,f);e.setTextRange(d,i.operand);p=a?void 0:t.createTempVariable(r);return d=e.expandPreOrPostfixIncrementOrDecrementExpression(t,i,d,r,p),d=t.createReflectSetCall(m,v,d,f),e.setOriginalNode(d,i),e.setTextRange(d,i),p&&(d=t.createComma(d,p),e.setTextRange(d,i)),d}}}return e.visitEachChild(i,w,n)}function X(r){return t.updateForStatement(r,e.visitNode(r.initializer,O,e.isForInitializer),e.visitNode(r.condition,w,e.isExpression),e.visitNode(r.incrementor,O,e.isExpression),e.visitIterationBody(r.statement,w,n))}function Y(n){return t.updateExpressionStatement(n,e.visitNode(n.expression,O,e.isExpression))}function Q(n){var i=e.nodeIsSynthesized(n)?n:t.cloneNode(n);if(e.isSimpleInlineableExpression(n))return{readExpression:i,initializeExpression:void 0};var a=t.createTempVariable(r),o=t.createAssignment(a,i);return{readExpression:a,initializeExpression:o}}function Z(i){if(y&&e.isPrivateIdentifierPropertyAccessExpression(i.expression)){var o=t.createCallBinding(i.expression,r,_),s=o.thisArg,c=o.target;return e.isCallChain(i)?t.updateCallChain(i,t.createPropertyAccessChain(e.visitNode(c,w),i.questionDotToken,"call"),void 0,void 0,a([e.visitNode(s,w,e.isExpression)],e.visitNodes(i.arguments,w,e.isExpression),!0)):t.updateCallExpression(i,t.createPropertyAccessExpression(e.visitNode(c,w),"call"),void 0,a([e.visitNode(s,w,e.isExpression)],e.visitNodes(i.arguments,w,e.isExpression),!0))}if(h&&e.isSuperProperty(i.expression)&&C&&(null===D||void 0===D?void 0:D.classConstructor)){var l=t.createFunctionCallCall(e.visitNode(i.expression,w,e.isExpression),D.classConstructor,e.visitNodes(i.arguments,w,e.isExpression));return e.setOriginalNode(l,i),e.setTextRange(l,i),l}return e.visitEachChild(i,w,n)}function $(i){if(y&&e.isPrivateIdentifierPropertyAccessExpression(i.tag)){var a=t.createCallBinding(i.tag,r,_),o=a.thisArg,s=a.target;return t.updateTaggedTemplateExpression(i,t.createCallExpression(t.createPropertyAccessExpression(e.visitNode(s,w),"bind"),void 0,[e.visitNode(o,w,e.isExpression)]),void 0,e.visitNode(i.template,w,e.isTemplateLiteral))}if(h&&e.isSuperProperty(i.tag)&&C&&(null===D||void 0===D?void 0:D.classConstructor)){var c=t.createFunctionBindCall(e.visitNode(i.tag,w,e.isExpression),D.classConstructor,[]);return e.setOriginalNode(c,i),e.setTextRange(c,i),t.updateTaggedTemplateExpression(i,c,void 0,e.visitNode(i.template,w,e.isTemplateLiteral))}return e.visitEachChild(i,w,n)}function ee(n){if(y){D&&N.set(e.getOriginalNodeId(n),D),u();var r=C;C=n;var i=e.visitNodes(n.body.statements,w,e.isStatement);i=t.mergeLexicalEnvironment(i,l()),C=r;var a=t.createImmediatelyInvokedArrowFunction(i);return e.setOriginalNode(a,n),e.setTextRange(a,n),e.addEmitFlags(a,2),a}}function ne(i,o){if(e.isDestructuringAssignment(i)){var s=S;S=void 0,i=t.updateBinaryExpression(i,e.visitNode(i.left,M),i.operatorToken,e.visitNode(i.right,w));var c=e.some(S)?t.inlineExpressions(e.compact(a(a([],S,!0),[i],!1))):i;return S=s,c}if(e.isAssignmentExpression(i))if(y&&e.isPrivateIdentifierPropertyAccessExpression(i.left)){var l=Be(i.left.name);if(l)return e.setTextRange(e.setOriginalNode(te(l,i.left.expression,i.right,i.operatorToken.kind),i),i)}else if(h&&e.isSuperProperty(i.left)&&C&&D){var u=D.classConstructor,d=D.superClassReference,p=D.facts;if(1&p)return t.updateBinaryExpression(i,Se(i.left),i.operatorToken,e.visitNode(i.right,w,e.isExpression));if(u&&d){var f=e.isElementAccessExpression(i.left)?e.visitNode(i.left.argumentExpression,w,e.isExpression):e.isIdentifier(i.left.name)?t.createStringLiteralFromNode(i.left.name):void 0;if(f){var m=e.visitNode(i.right,w,e.isExpression);if(e.isCompoundAssignment(i.operatorToken.kind)){var _=f;e.isSimpleInlineableExpression(f)||(_=t.createTempVariable(r),f=t.createAssignment(_,f));var g=t.createReflectGetCall(d,_,u);e.setOriginalNode(g,i.left),e.setTextRange(g,i.left),m=t.createBinaryExpression(g,e.getNonAssignmentOperatorForCompoundAssignment(i.operatorToken.kind),m),e.setTextRange(m,i)}var v=o?void 0:t.createTempVariable(r);return v&&(m=t.createAssignment(v,m),e.setTextRange(v,i)),m=t.createReflectSetCall(d,f,m,u),e.setOriginalNode(m,i),e.setTextRange(m,i),v&&(m=t.createComma(m,v),e.setTextRange(m,i)),m}}}return e.visitEachChild(i,w,n)}function te(r,i,a,o){if(i=e.visitNode(i,w,e.isExpression),a=e.visitNode(a,w,e.isExpression),e.isCompoundAssignment(o)){var s=Q(i),c=s.readExpression,l=s.initializeExpression;i=l||c,a=t.createBinaryExpression(W(r,c),e.getNonAssignmentOperatorForCompoundAssignment(o),a)}switch(e.setCommentRange(i,e.moveRangePos(i,-1)),r.kind){case"a":return n.getEmitHelperFactory().createClassPrivateFieldSetHelper(i,r.brandCheckIdentifier,a,r.kind,r.setterName);case"m":return n.getEmitHelperFactory().createClassPrivateFieldSetHelper(i,r.brandCheckIdentifier,a,r.kind,void 0);case"f":return n.getEmitHelperFactory().createClassPrivateFieldSetHelper(i,r.brandCheckIdentifier,a,r.kind,r.variableName);default:e.Debug.assertNever(r,"Unknown private element type")}}function re(t){if(!e.forEach(t.members,ie))return e.visitEachChild(t,w,n);var r=S;if(S=void 0,ke(),y){var i=e.getNameOfDeclaration(t);i&&e.isIdentifier(i)&&(we().className=e.idText(i));var a=ae(t);e.some(a)&&(we().weakSetName=Fe("instances",a[0].name))}var o=e.isClassDeclaration(t)?ce(t):le(t);return Re(),S=r,o}function ie(n){return e.isPropertyDeclaration(n)||e.isClassStaticBlockDeclaration(n)||y&&n.name&&e.isPrivateIdentifier(n.name)}function ae(n){return e.filter(n.members,e.isNonStaticMethodOrAccessorWithPrivateName)}function oe(n){var t=0,r=e.getOriginalNode(n);e.isClassDeclaration(r)&&e.classOrConstructorParameterIsDecorated(r)&&(t|=1);for(var i=0,a=n.members;i<a.length;i++){var o=a[i];e.isStatic(o)&&(o.name&&e.isPrivateIdentifier(o.name)&&y&&(t|=2),(e.isPropertyDeclaration(o)||e.isClassStaticBlockDeclaration(o))&&(v&&8192&o.transformFlags&&(t|=8,1&t||(t|=2)),h&&33554432&o.transformFlags&&(1&t||(t|=6))))}return t}function se(i){var a=(null===D||void 0===D?void 0:D.facts)||0;if(4&a){var o=t.createTempVariable(r,!0);return Oe().superClassReference=o,t.updateExpressionWithTypeArguments(i,t.createAssignment(o,e.visitNode(i.expression,w,e.isExpression)),void 0)}return e.visitEachChild(i,w,n)}function ce(n){var i=oe(n);i&&(Oe().facts=i),8&i&&Ee();var a,o=e.getStaticPropertiesAndClassStaticBlock(n);if(2&i){var s=t.createTempVariable(r,!0);Oe().classConstructor=t.cloneNode(s),a=t.createAssignment(s,t.getInternalName(n))}var c=e.getEffectiveBaseTypeNode(n),l=!(!c||104===e.skipOuterExpressions(c.expression).kind),u=[t.updateClassDeclaration(n,void 0,n.modifiers,n.name,void 0,e.visitNodes(n.heritageClauses,P,e.isHeritageClause),de(n,l))];return a&&Pe().unshift(a),e.some(S)&&u.push(t.createExpressionStatement(t.inlineExpressions(S))),e.some(o)&&ge(u,o,t.getInternalName(n)),u}function le(n){var i=oe(n);i&&(Oe().facts=i),8&i&&Ee();var a,o=!!(1&i),s=e.getStaticPropertiesAndClassStaticBlock(n),c=e.getEffectiveBaseTypeNode(n),l=!(!c||104===e.skipOuterExpressions(c.expression).kind),u=16777216&f.getNodeCheckFlags(n);function d(){var e=f.getNodeCheckFlags(n),i=16777216&e,a=524288&e;return t.createTempVariable(a?p:r,!!i)}2&i&&(a=d(),Oe().classConstructor=t.cloneNode(a));var m=t.updateClassExpression(n,e.visitNodes(n.decorators,w,e.isDecorator),n.modifiers,n.name,void 0,e.visitNodes(n.heritageClauses,P,e.isHeritageClause),de(n,l)),_=e.some(s,function(n){return e.isClassStaticBlockDeclaration(n)||!!n.initializer||y&&e.isPrivateIdentifier(n.name)});if(_||e.some(S)){if(o)return e.Debug.assertIsDefined(x,"Decorated classes transformed by TypeScript are expected to be within a variable declaration."),x&&S&&e.some(S)&&x.push(t.createExpressionStatement(t.inlineExpressions(S))),x&&e.some(s)&&ge(x,s,t.getInternalName(n)),a?t.inlineExpressions([t.createAssignment(a,m),a]):m;var g=[];if(a||(a=d()),u){be();var h=t.cloneNode(a);h.autoGenerateFlags&=-9,T[e.getOriginalNodeId(n)]=h}return e.setEmitFlags(m,65536|e.getEmitFlags(m)),g.push(e.startOnNewLine(t.createAssignment(a,m))),e.addRange(g,e.map(S,e.startOnNewLine)),e.addRange(g,ye(s,a)),g.push(e.startOnNewLine(a)),t.inlineExpressions(g)}return m}function ue(t){if(!y)return e.visitEachChild(t,G,n)}function de(n,r){if(y){for(var i=0,a=n.members;i<a.length;i++){var o=a[i];e.isPrivateIdentifierClassElementDeclaration(o)&&Me(o)}e.some(ae(n))&&pe()}var s=[],c=me(n,r);return c&&s.push(c),e.addRange(s,e.visitNodes(n.members,G,e.isClassElement)),e.setTextRange(t.createNodeArray(s),n.members)}function pe(){var n=we().weakSetName;e.Debug.assert(n,"weakSetName should be set in private identifier environment"),Pe().push(t.createAssignment(n,t.createNewExpression(t.createIdentifier("WeakSet"),void 0,[])))}function fe(n){return!e.isStatic(n)&&!e.hasSyntacticModifier(e.getOriginalNode(n),128)&&(g?_<99:e.isInitializedProperty(n)||y&&e.isPrivateIdentifierClassElementDeclaration(n))}function me(r,i){var a=e.visitNode(e.getFirstConstructorWithBody(r),w,e.isConstructorDeclaration),o=r.members.filter(fe);if(!e.some(o))return a;var s=e.visitParameterList(a?a.parameters:void 0,w,n),c=_e(r,a,i);return c?e.startOnNewLine(e.setOriginalNode(e.setTextRange(t.createConstructorDeclaration(void 0,void 0,null!==s&&void 0!==s?s:[],c),a||r),a)):void 0}function _e(r,i,a){var o=e.getProperties(r,!1,!1);g||(o=e.filter(o,function(n){return!!n.initializer||e.isPrivateIdentifier(n.name)}));var s=ae(r),c=e.some(o)||e.some(s);if(!i&&!c)return e.visitFunctionBody(void 0,w,n);d();var u=0,p=[];if(!i&&a&&p.push(t.createExpressionStatement(t.createCallExpression(t.createSuper(),void 0,[t.createSpreadElement(t.createIdentifier("arguments"))]))),i&&(u=e.addPrologueDirectivesAndInitialSuperCall(t,i,p,w)),null===i||void 0===i?void 0:i.body){var f=e.findIndex(i.body.statements,function(n){return!e.isParameterPropertyDeclaration(e.getOriginalNode(n),i)},u);-1===f&&(f=i.body.statements.length),f>u&&(g||e.addRange(p,e.visitNodes(i.body.statements,w,e.isStatement,u,f-u)),u=f)}var m=t.createThis();return Te(p,s,m),ge(p,o,m),i&&e.addRange(p,e.visitNodes(i.body.statements,w,e.isStatement,u)),p=t.mergeLexicalEnvironment(p,l()),e.setTextRange(t.createBlock(e.setTextRange(t.createNodeArray(p),i?i.body.statements:r.members),!0),i?i.body:void 0)}function ge(n,r,i){for(var a=0,o=r;a<o.length;a++){var s=o[a],c=e.isClassStaticBlockDeclaration(s)?ee(s):he(s,i);if(c){var l=t.createExpressionStatement(c);e.setSourceMapRange(l,e.moveRangePastModifiers(s)),e.setCommentRange(l,s),e.setOriginalNode(l,s),n.push(l)}}}function ye(n,t){for(var r=[],i=0,a=n;i<a.length;i++){var o=a[i],s=e.isClassStaticBlockDeclaration(o)?ee(o):he(o,t);s&&(e.startOnNewLine(s),e.setSourceMapRange(s,e.moveRangePastModifiers(o)),e.setCommentRange(s,o),e.setOriginalNode(s,o),r.push(s))}return r}function he(n,t){var r=C,i=ve(n,t);return i&&e.hasStaticModifier(n)&&(null===D||void 0===D?void 0:D.facts)&&(e.setOriginalNode(i,n),e.addEmitFlags(i,2),N.set(e.getOriginalNodeId(i),D)),C=r,i}function ve(n,r){var a,s=!g,c=e.isComputedPropertyName(n.name)&&!e.isSimpleInlineableExpression(n.name.expression)?t.updateComputedPropertyName(n.name,t.getGeneratedNameForNode(n.name)):n.name;if(e.hasStaticModifier(n)&&(C=n),y&&e.isPrivateIdentifier(c)){var l=Be(c);if(l)return"f"===l.kind?l.isStatic?i(l.variableName,e.visitNode(n.initializer,w,e.isExpression)):o(r,e.visitNode(n.initializer,w,e.isExpression),l.brandCheckIdentifier):void 0;e.Debug.fail("Undeclared private name for property declaration.")}if(!e.isPrivateIdentifier(c)&&!e.hasStaticModifier(n)||n.initializer){var u=e.getOriginalNode(n);if(!e.hasSyntacticModifier(u,128)){var d=n.initializer||s?null!==(a=e.visitNode(n.initializer,w,e.isExpression))&&void 0!==a?a:t.createVoidZero():e.isParameterPropertyDeclaration(u,u.parent)&&e.isIdentifier(c)?c:t.createVoidZero();if(s||e.isPrivateIdentifier(c)){var p=e.createMemberAccessForPropertyName(t,r,c,c);return t.createAssignment(p,d)}var f=e.isComputedPropertyName(c)?c.expression:e.isIdentifier(c)?t.createStringLiteral(e.unescapeLeadingUnderscores(c.escapedText)):c,m=t.createPropertyDescriptor({value:d,configurable:!0,writable:!0,enumerable:!0});return t.createObjectDefinePropertyCall(r,f,m)}}}function be(){0===(1&E)&&(E|=1,n.enableSubstitution(79),T=[])}function Ee(){0===(2&E)&&(E|=2,n.enableSubstitution(108),n.enableEmitNotification(254),n.enableEmitNotification(211),n.enableEmitNotification(169),n.enableEmitNotification(170),n.enableEmitNotification(171),n.enableEmitNotification(167),n.enableEmitNotification(165),n.enableEmitNotification(160))}function Te(n,r,i){if(y&&e.some(r)){var a=we().weakSetName;e.Debug.assert(a,"weakSetName should be set in private identifier environment"),n.push(t.createExpressionStatement(s(i,a)))}}function Se(n){return e.isPropertyAccessExpression(n)?t.updatePropertyAccessExpression(n,t.createVoidZero(),n.name):t.updateElementAccessExpression(n,t.createVoidZero(),e.visitNode(n.argumentExpression,w,e.isExpression))}function xe(n,t,r){var i=e.getOriginalNode(t);if(i.id){var a=N.get(i.id);if(a){var o=D,s=A;return D=a,A=a,L(n,t,r),D=o,void(A=s)}}switch(t.kind){case 211:if(e.isArrowFunction(i)||262144&e.getEmitFlags(t))break;case 254:case 169:o=D,s=A;return D=void 0,A=void 0,L(n,t,r),D=o,void(A=s);case 170:case 171:case 167:case 165:o=D,s=A;return A=D,D=void 0,L(n,t,r),D=o,void(A=s);case 160:o=D,s=A;return D=A,A=void 0,L(n,t,r),D=o,void(A=s)}L(n,t,r)}function Le(e,n){return n=b(e,n),1===e?De(n):n}function De(e){switch(e.kind){case 79:return Ce(e);case 108:return Ae(e)}return e}function Ae(n){if(2&E&&D){var r=D.facts,i=D.classConstructor;if(1&r)return t.createParenthesizedExpression(t.createVoidZero());if(i)return e.setTextRange(e.setOriginalNode(t.cloneNode(i),n),n)}return n}function Ce(e){return Ie(e)||e}function Ie(n){if(1&E&&33554432&f.getNodeCheckFlags(n)){var r=f.getReferencedValueDeclaration(n);if(r){var i=T[r.id];if(i){var a=t.cloneNode(i);return e.setSourceMapRange(a,n),e.setCommentRange(a,n),a}}}}function Ne(n,i){if(e.isComputedPropertyName(n)){var a=e.visitNode(n.expression,w,e.isExpression),o=e.skipPartiallyEmittedExpressions(a),s=e.isSimpleInlineableExpression(o),c=e.isAssignmentExpression(o)&&e.isGeneratedIdentifier(o.left);if(!c&&!s&&i){var l=t.getGeneratedNameForNode(n);return 524288&f.getNodeCheckFlags(n)?p(l):r(l),t.createAssignment(l,a)}return s||e.isIdentifier(o)?void 0:a}}function ke(){I.push(D),D=void 0}function Re(){D=I.pop()}function Oe(){return D||(D={facts:0,classConstructor:void 0,superClassReference:void 0,privateIdentifierEnvironment:void 0})}function we(){var n=Oe();return n.privateIdentifierEnvironment||(n.privateIdentifierEnvironment={className:"",identifiers:new e.Map}),n.privateIdentifierEnvironment}function Pe(){return S||(S=[])}function Me(n){var r,i=e.getTextOfPropertyName(n.name),a=Oe(),o=a.classConstructor,s=we(),l=s.weakSetName,u=[],d=n.name.escapedText,p=s.identifiers.get(d),f=!c(n.name)&&void 0===p;if(e.hasStaticModifier(n))if(e.Debug.assert(o,"weakSetName should be set in private identifier environment"),e.isPropertyDeclaration(n)){var m=Ge(i,n);s.identifiers.set(d,{kind:"f",variableName:m,brandCheckIdentifier:o,isStatic:!0,isValid:f})}else if(e.isMethodDeclaration(n)){var _=Ge(i,n);s.identifiers.set(d,{kind:"m",methodName:_,brandCheckIdentifier:o,isStatic:!0,isValid:f})}else if(e.isGetAccessorDeclaration(n)){var g=Ge(i+"_get",n);"a"===(null===p||void 0===p?void 0:p.kind)&&p.isStatic&&!p.getterName?p.getterName=g:s.identifiers.set(d,{kind:"a",getterName:g,setterName:void 0,brandCheckIdentifier:o,isStatic:!0,isValid:f})}else if(e.isSetAccessorDeclaration(n)){var y=Ge(i+"_set",n);"a"===(null===p||void 0===p?void 0:p.kind)&&p.isStatic&&!p.setterName?p.setterName=y:s.identifiers.set(d,{kind:"a",getterName:void 0,setterName:y,brandCheckIdentifier:o,isStatic:!0,isValid:f})}else e.Debug.assertNever(n,"Unknown class element type.");else if(e.isPropertyDeclaration(n)){var h=Ge(i,n);s.identifiers.set(d,{kind:"f",brandCheckIdentifier:h,isStatic:!1,variableName:void 0,isValid:f}),u.push(t.createAssignment(h,t.createNewExpression(t.createIdentifier("WeakMap"),void 0,[])))}else if(e.isMethodDeclaration(n))e.Debug.assert(l,"weakSetName should be set in private identifier environment"),s.identifiers.set(d,{kind:"m",methodName:Ge(i,n),brandCheckIdentifier:l,isStatic:!1,isValid:f});else if(e.isAccessor(n))if(e.Debug.assert(l,"weakSetName should be set in private identifier environment"),e.isGetAccessor(n)){g=Ge(i+"_get",n);"a"!==(null===p||void 0===p?void 0:p.kind)||p.isStatic||p.getterName?s.identifiers.set(d,{kind:"a",getterName:g,setterName:void 0,brandCheckIdentifier:l,isStatic:!1,isValid:f}):p.getterName=g}else{y=Ge(i+"_set",n);"a"!==(null===p||void 0===p?void 0:p.kind)||p.isStatic||p.setterName?s.identifiers.set(d,{kind:"a",getterName:void 0,setterName:y,brandCheckIdentifier:l,isStatic:!1,isValid:f}):p.setterName=y}else e.Debug.assertNever(n,"Unknown class element type.");(r=Pe()).push.apply(r,u)}function Fe(e,n){var i=we().className,a=i?"_"+i:"",o=t.createUniqueName(a+"_"+e,16);return 524288&f.getNodeCheckFlags(n)?p(o):r(o),o}function Ge(e,n){return Fe(e.substring(1),n.name)}function Be(e){var n;if(null===D||void 0===D?void 0:D.privateIdentifierEnvironment){var t=D.privateIdentifierEnvironment.identifiers.get(e.escapedText);if(t)return t}for(var r=I.length-1;r>=0;--r){var i=I[r];if(i){t=null===(n=i.privateIdentifierEnvironment)||void 0===n?void 0:n.identifiers.get(e.escapedText);if(t)return t}}}function Ue(i){var a=t.getGeneratedNameForNode(i),o=Be(i.name);if(!o)return e.visitEachChild(i,w,n);var s=i.expression;return(e.isThisProperty(i)||e.isSuperProperty(i)||!e.isSimpleCopiableExpression(i.expression))&&(s=t.createTempVariable(r,!0),Pe().push(t.createBinaryExpression(s,63,e.visitNode(i.expression,w,e.isExpression)))),t.createAssignmentTargetWrapper(a,te(o,s,a,63))}function Ve(n){var r=e.getTargetOfBindingOrAssignmentElement(n);if(r){var i=void 0;if(e.isPrivateIdentifierPropertyAccessExpression(r))i=Ue(r);else if(h&&e.isSuperProperty(r)&&C&&D){var a=D.classConstructor,o=D.superClassReference,s=D.facts;if(1&s)i=Se(r);else if(a&&o){var c=e.isElementAccessExpression(r)?e.visitNode(r.argumentExpression,w,e.isExpression):e.isIdentifier(r.name)?t.createStringLiteralFromNode(r.name):void 0;if(c){var l=t.createTempVariable(void 0);i=t.createAssignmentTargetWrapper(l,t.createReflectSetCall(o,c,l,a))}}}if(i)return e.isAssignmentExpression(n)?t.updateBinaryExpression(n,i,n.operatorToken,e.visitNode(n.right,w,e.isExpression)):e.isSpreadElement(n)?t.updateSpreadElement(n,i):i}return e.visitNode(n,M)}function je(n){if(e.isObjectBindingOrAssignmentElement(n)&&!e.isShorthandPropertyAssignment(n)){var r=e.getTargetOfBindingOrAssignmentElement(n),i=void 0;if(r)if(e.isPrivateIdentifierPropertyAccessExpression(r))i=Ue(r);else if(h&&e.isSuperProperty(r)&&C&&D){var a=D.classConstructor,o=D.superClassReference,s=D.facts;if(1&s)i=Se(r);else if(a&&o){var c=e.isElementAccessExpression(r)?e.visitNode(r.argumentExpression,w,e.isExpression):e.isIdentifier(r.name)?t.createStringLiteralFromNode(r.name):void 0;if(c){var l=t.createTempVariable(void 0);i=t.createAssignmentTargetWrapper(l,t.createReflectSetCall(o,c,l,a))}}}if(e.isPropertyAssignment(n)){var u=e.getInitializerOfBindingOrAssignmentElement(n);return t.updatePropertyAssignment(n,e.visitNode(n.name,w,e.isPropertyName),i?u?t.createAssignment(i,e.visitNode(u,w)):i:e.visitNode(n.initializer,M,e.isExpression))}if(e.isSpreadAssignment(n))return t.updateSpreadAssignment(n,i||e.visitNode(n.expression,M,e.isExpression));e.Debug.assert(void 0===i,"Should not have generated a wrapped target")}return e.visitNode(n,w)}function Ke(n){return e.isArrayLiteralExpression(n)?t.updateArrayLiteralExpression(n,e.visitNodes(n.elements,Ve,e.isExpression)):t.updateObjectLiteralExpression(n,e.visitNodes(n.properties,je,e.isObjectLiteralElementLike))}}function i(n,t){return e.factory.createAssignment(n,e.factory.createObjectLiteralExpression([e.factory.createPropertyAssignment("value",t||e.factory.createVoidZero())]))}function o(n,t,r){return e.factory.createCallExpression(e.factory.createPropertyAccessExpression(r,"set"),void 0,[n,t||e.factory.createVoidZero()])}function s(n,t){return e.factory.createCallExpression(e.factory.createPropertyAccessExpression(t,"add"),void 0,[n])}function c(e){return"#constructor"===e.escapedText}(function(e){e[e["ClassAliases"]=1]="ClassAliases",e[e["ClassStaticThisOrSuperReference"]=2]="ClassStaticThisOrSuperReference"})(n||(n={})),function(e){e["Field"]="f",e["Method"]="m",e["Accessor"]="a"}(e.PrivateIdentifierKind||(e.PrivateIdentifierKind={})),function(e){e[e["None"]=0]="None",e[e["ClassWasDecorated"]=1]="ClassWasDecorated",e[e["NeedsClassConstructorReference"]=2]="NeedsClassConstructorReference",e[e["NeedsClassSuperReference"]=4]="NeedsClassSuperReference",e[e["NeedsSubstitutionForThisInClassStaticField"]=8]="NeedsSubstitutionForThisInClassStaticField"}(t||(t={})),e.transformClassFields=r}(d||(d={})),function(e){var n,t;function r(n){var t,r,o,s,c=n.factory,l=n.getEmitHelperFactory,u=n.resumeLexicalEnvironment,d=n.endLexicalEnvironment,p=n.hoistVariableDeclaration,f=n.getEmitResolver(),m=n.getCompilerOptions(),_=e.getEmitScriptTarget(m),g=0,y=[],h=0,v=n.onEmitNode,b=n.onSubstituteNode;return n.onEmitNode=Q,n.onSubstituteNode=Z,e.chainBundle(n,E);function E(t){if(t.isDeclarationFile)return t;T(1,!1),T(2,!e.isEffectiveStrictModeSourceFile(t,m));var r=e.visitEachChild(t,C,n);return e.addEmitHelpers(r,n.readEmitHelpers()),r}function T(e,n){h=n?h|e:h&~e}function S(e){return 0!==(h&e)}function x(){return!S(1)}function L(){return S(2)}function D(e,n,t){var r=e&~h;if(r){T(r,!0);var i=n(t);return T(r,!1),i}return n(t)}function A(t){return e.visitEachChild(t,C,n)}function C(t){if(0===(128&t.transformFlags))return t;switch(t.kind){case 130:return;case 216:return P(t);case 167:return D(3,M,t);case 254:return D(3,F,t);case 211:return D(3,G,t);case 212:return D(1,B,t);case 204:return o&&e.isPropertyAccessExpression(t)&&106===t.expression.kind&&o.add(t.name.escapedText),e.visitEachChild(t,C,n);case 205:return o&&106===t.expression.kind&&(s=!0),e.visitEachChild(t,C,n);case 170:case 171:case 169:case 255:case 224:return D(3,A,t);default:return e.visitEachChild(t,C,n)}}function I(t){if(e.isNodeWithPossibleHoistedDeclaration(t))switch(t.kind){case 235:return k(t);case 240:return w(t);case 241:return R(t);case 242:return O(t);case 290:return N(t);case 233:case 247:case 261:case 287:case 288:case 250:case 238:case 239:case 237:case 246:case 248:return e.visitEachChild(t,I,n);default:return e.Debug.assertNever(t,"Unhandled node.")}return C(t)}function N(t){var i,a=new e.Set;if(U(t.variableDeclaration,a),a.forEach(function(n,t){r.has(t)&&(i||(i=new e.Set(r)),i.delete(t))}),i){var o=r;r=i;var s=e.visitEachChild(t,I,n);return r=o,s}return e.visitEachChild(t,I,n)}function k(t){if(V(t.declarationList)){var r=j(t.declarationList,!1);return r?c.createExpressionStatement(r):void 0}return e.visitEachChild(t,C,n)}function R(t){return c.updateForInStatement(t,V(t.initializer)?j(t.initializer,!0):e.visitNode(t.initializer,C,e.isForInitializer),e.visitNode(t.expression,C,e.isExpression),e.visitIterationBody(t.statement,I,n))}function O(t){return c.updateForOfStatement(t,e.visitNode(t.awaitModifier,C,e.isToken),V(t.initializer)?j(t.initializer,!0):e.visitNode(t.initializer,C,e.isForInitializer),e.visitNode(t.expression,C,e.isExpression),e.visitIterationBody(t.statement,I,n))}function w(t){var r=t.initializer;return c.updateForStatement(t,V(r)?j(r,!1):e.visitNode(t.initializer,C,e.isForInitializer),e.visitNode(t.condition,C,e.isExpression),e.visitNode(t.incrementor,C,e.isExpression),e.visitIterationBody(t.statement,I,n))}function P(t){return x()?e.visitEachChild(t,C,n):e.setOriginalNode(e.setTextRange(c.createYieldExpression(void 0,e.visitNode(t.expression,C,e.isExpression)),t),t)}function M(t){return c.updateMethodDeclaration(t,void 0,e.visitNodes(t.modifiers,C,e.isModifier),t.asteriskToken,t.name,void 0,void 0,e.visitParameterList(t.parameters,C,n),void 0,2&e.getFunctionFlags(t)?J(t):e.visitFunctionBody(t.body,C,n))}function F(t){return c.updateFunctionDeclaration(t,void 0,e.visitNodes(t.modifiers,C,e.isModifier),t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,C,n),void 0,2&e.getFunctionFlags(t)?J(t):e.visitFunctionBody(t.body,C,n))}function G(t){return c.updateFunctionExpression(t,e.visitNodes(t.modifiers,C,e.isModifier),t.asteriskToken,t.name,void 0,e.visitParameterList(t.parameters,C,n),void 0,2&e.getFunctionFlags(t)?J(t):e.visitFunctionBody(t.body,C,n))}function B(t){return c.updateArrowFunction(t,e.visitNodes(t.modifiers,C,e.isModifier),void 0,e.visitParameterList(t.parameters,C,n),void 0,t.equalsGreaterThanToken,2&e.getFunctionFlags(t)?J(t):e.visitFunctionBody(t.body,C,n))}function U(n,t){var r=n.name;if(e.isIdentifier(r))t.add(r.escapedText);else for(var i=0,a=r.elements;i<a.length;i++){var o=a[i];e.isOmittedExpression(o)||U(o,t)}}function V(n){return!!n&&e.isVariableDeclarationList(n)&&!(3&n.flags)&&n.declarations.some(z)}function j(n,t){K(n);var r=e.getInitializedVariables(n);return 0===r.length?t?e.visitNode(c.converters.convertToAssignmentElementTarget(n.declarations[0].name),C,e.isExpression):void 0:c.inlineExpressions(e.map(r,W))}function K(n){e.forEach(n.declarations,H)}function H(n){var t=n.name;if(e.isIdentifier(t))p(t);else for(var r=0,i=t.elements;r<i.length;r++){var a=i[r];e.isOmittedExpression(a)||H(a)}}function W(n){var t=e.setSourceMapRange(c.createAssignment(c.converters.convertToAssignmentElementTarget(n.name),n.initializer),n);return e.visitNode(t,C,e.isExpression)}function z(n){var t=n.name;if(e.isIdentifier(t))return r.has(t.escapedText);for(var i=0,a=t.elements;i<a.length;i++){var o=a[i];if(!e.isOmittedExpression(o)&&z(o))return!0}return!1}function J(n){u();var t=e.getOriginalNode(n,e.isFunctionLike),a=t.type,p=_<2?X(a):void 0,m=212===n.kind,g=0!==(8192&f.getNodeCheckFlags(n)),h=r;r=new e.Set;for(var v=0,b=n.parameters;v<b.length;v++){var E=b[v];U(E,r)}var T,S=o,x=s;if(m||(o=new e.Set,s=!1),m){var D=l().createAwaiterHelper(L(),g,p,q(n.body)),A=d();if(e.some(A)){O=c.converters.convertToFunctionBlock(D);T=c.updateBlock(O,e.setTextRange(c.createNodeArray(e.concatenate(A,O.statements)),O.statements))}else T=D}else{var I=[],N=c.copyPrologue(n.body.statements,I,!1,C);I.push(c.createReturnStatement(l().createAwaiterHelper(L(),g,p,q(n.body,N)))),e.insertStatementsAfterStandardPrologue(I,d());var k=_>=2&&6144&f.getNodeCheckFlags(n);if(k&&(Y(),o.size)){var R=i(c,f,n,o);y[e.getNodeId(R)]=!0,e.insertStatementsAfterStandardPrologue(I,[R])}var O=c.createBlock(I,!0);e.setTextRange(O,n.body),k&&s&&(4096&f.getNodeCheckFlags(n)?e.addEmitHelper(O,e.advancedAsyncSuperHelper):2048&f.getNodeCheckFlags(n)&&e.addEmitHelper(O,e.asyncSuperHelper)),T=O}return r=h,m||(o=S,s=x),T}function q(n,t){return e.isBlock(n)?c.updateBlock(n,e.visitNodes(n.statements,I,e.isStatement,t)):c.converters.convertToFunctionBlock(e.visitNode(n,I,e.isConciseBody))}function X(n){var t=n&&e.getEntityNameFromTypeNode(n);if(t&&e.isEntityName(t)){var r=f.getTypeReferenceSerializationKind(t);if(r===e.TypeReferenceSerializationKind.TypeWithConstructSignatureAndValue||r===e.TypeReferenceSerializationKind.Unknown)return t}}function Y(){0===(1&t)&&(t|=1,n.enableSubstitution(206),n.enableSubstitution(204),n.enableSubstitution(205),n.enableEmitNotification(255),n.enableEmitNotification(167),n.enableEmitNotification(170),n.enableEmitNotification(171),n.enableEmitNotification(169),n.enableEmitNotification(235))}function Q(n,r,i){if(1&t&&re(r)){var a=6144&f.getNodeCheckFlags(r);if(a!==g){var o=g;return g=a,v(n,r,i),void(g=o)}}else if(t&&y[e.getNodeId(r)]){o=g;return g=0,v(n,r,i),void(g=o)}v(n,r,i)}function Z(e,n){return n=b(e,n),1===e&&g?$(n):n}function $(e){switch(e.kind){case 204:return ee(e);case 205:return ne(e);case 206:return te(e)}return e}function ee(n){return 106===n.expression.kind?e.setTextRange(c.createPropertyAccessExpression(c.createUniqueName("_super",48),n.name),n):n}function ne(e){return 106===e.expression.kind?ie(e.argumentExpression,e):e}function te(n){var t=n.expression;if(e.isSuperProperty(t)){var r=e.isPropertyAccessExpression(t)?ee(t):ne(t);return c.createCallExpression(c.createPropertyAccessExpression(r,"call"),void 0,a([c.createThis()],n.arguments,!0))}return n}function re(e){var n=e.kind;return 255===n||169===n||167===n||170===n||171===n}function ie(n,t){return 4096&g?e.setTextRange(c.createPropertyAccessExpression(c.createCallExpression(c.createUniqueName("_superIndex",48),void 0,[n]),"value"),t):e.setTextRange(c.createCallExpression(c.createUniqueName("_superIndex",48),void 0,[n]),t)}}function i(n,t,r,i){var a=0!==(4096&t.getNodeCheckFlags(r)),o=[];return i.forEach(function(t,r){var i=e.unescapeLeadingUnderscores(r),s=[];s.push(n.createPropertyAssignment("get",n.createArrowFunction(void 0,void 0,[],void 0,void 0,e.setEmitFlags(n.createPropertyAccessExpression(e.setEmitFlags(n.createSuper(),4),i),4)))),a&&s.push(n.createPropertyAssignment("set",n.createArrowFunction(void 0,void 0,[n.createParameterDeclaration(void 0,void 0,void 0,"v",void 0,void 0,void 0)],void 0,void 0,n.createAssignment(e.setEmitFlags(n.createPropertyAccessExpression(e.setEmitFlags(n.createSuper(),4),i),4),n.createIdentifier("v"))))),o.push(n.createPropertyAssignment(i,n.createObjectLiteralExpression(s)))}),n.createVariableStatement(void 0,n.createVariableDeclarationList([n.createVariableDeclaration(n.createUniqueName("_super",48),void 0,void 0,n.createCallExpression(n.createPropertyAccessExpression(n.createIdentifier("Object"),"create"),void 0,[n.createNull(),n.createObjectLiteralExpression(o,!0)]))],2))}(function(e){e[e["AsyncMethodsWithSuper"]=1]="AsyncMethodsWithSuper"})(n||(n={})),function(e){e[e["NonTopLevel"]=1]="NonTopLevel",e[e["HasLexicalThis"]=2]="HasLexicalThis"}(t||(t={})),e.transformES2017=r,e.createSuperAccessVariableStatement=i}(d||(d={})),function(e){var n,t;function r(n){var t=n.factory,r=n.getEmitHelperFactory,i=n.resumeLexicalEnvironment,o=n.endLexicalEnvironment,s=n.hoistVariableDeclaration,c=n.getEmitResolver(),l=n.getCompilerOptions(),u=e.getEmitScriptTarget(l),d=n.onEmitNode;n.onEmitNode=_e;var p=n.onSubstituteNode;n.onSubstituteNode=ge;var f,m,_,g,y,h,v=!1,b=0,E=0,T=[];return e.chainBundle(n,A);function S(e,n){return E!==(E&~e|n)}function x(e,n){var t=E;return E=3&(E&~e|n),t}function L(e){E=e}function D(n){g=e.append(g,t.createVariableDeclaration(n))}function A(t){if(t.isDeclarationFile)return t;_=t;var r=j(t);return e.addEmitHelpers(r,n.readEmitHelpers()),_=void 0,g=void 0,r}function C(e){return O(e,!1)}function I(e){return O(e,!0)}function N(e){if(130!==e.kind)return e}function k(e,n,t,r){if(S(t,r)){var i=x(t,r),a=e(n);return L(i),a}return e(n)}function R(t){return e.visitEachChild(t,C,n)}function O(t,r){if(0===(64&t.transformFlags))return t;switch(t.kind){case 216:return w(t);case 222:return P(t);case 245:return M(t);case 248:return F(t);case 203:return B(t);case 219:return H(t,r);case 346:return W(t,r);case 290:return z(t);case 235:return J(t);case 252:return q(t);case 238:case 239:case 241:return k(R,t,0,2);case 242:return Z(t,void 0);case 240:return k(Y,t,0,2);case 215:return Q(t);case 169:return k(ie,t,2,1);case 167:return k(se,t,2,1);case 170:return k(ae,t,2,1);case 171:return k(oe,t,2,1);case 254:return k(ce,t,2,1);case 211:return k(ue,t,2,1);case 212:return k(le,t,2,0);case 162:return re(t);case 236:return U(t);case 210:return V(t,r);case 208:return K(t);case 204:return y&&e.isPropertyAccessExpression(t)&&106===t.expression.kind&&y.add(t.name.escapedText),e.visitEachChild(t,C,n);case 205:return y&&106===t.expression.kind&&(h=!0),e.visitEachChild(t,C,n);case 255:case 224:return k(R,t,2,1);default:return e.visitEachChild(t,C,n)}}function w(i){return 2&m&&1&m?e.setOriginalNode(e.setTextRange(t.createYieldExpression(void 0,r().createAwaitHelper(e.visitNode(i.expression,C,e.isExpression))),i),i):e.visitEachChild(i,C,n)}function P(i){if(2&m&&1&m){if(i.asteriskToken){var a=e.visitNode(e.Debug.assertDefined(i.expression),C,e.isExpression);return e.setOriginalNode(e.setTextRange(t.createYieldExpression(void 0,r().createAwaitHelper(t.updateYieldExpression(i,i.asteriskToken,e.setTextRange(r().createAsyncDelegatorHelper(e.setTextRange(r().createAsyncValuesHelper(a),a)),a)))),i),i)}return e.setOriginalNode(e.setTextRange(t.createYieldExpression(void 0,ne(i.expression?e.visitNode(i.expression,C,e.isExpression):t.createVoidZero())),i),i)}return e.visitEachChild(i,C,n)}function M(r){return 2&m&&1&m?t.updateReturnStatement(r,ne(r.expression?e.visitNode(r.expression,C,e.isExpression):t.createVoidZero())):e.visitEachChild(r,C,n)}function F(r){if(2&m){var i=e.unwrapInnermostStatementOfLabel(r);return 242===i.kind&&i.awaitModifier?Z(i,r):t.restoreEnclosingLabel(e.visitNode(i,C,e.isStatement,t.liftToBlock),r)}return e.visitEachChild(r,C,n)}function G(n){for(var r,i=[],a=0,o=n;a<o.length;a++){var s=o[a];if(293===s.kind){r&&(i.push(t.createObjectLiteralExpression(r)),r=void 0);var c=s.expression;i.push(e.visitNode(c,C,e.isExpression))}else r=e.append(r,291===s.kind?t.createPropertyAssignment(s.name,e.visitNode(s.initializer,C,e.isExpression)):e.visitNode(s,C,e.isObjectLiteralElementLike))}return r&&i.push(t.createObjectLiteralExpression(r)),i}function B(i){if(32768&i.transformFlags){var a=G(i.properties);a.length&&203!==a[0].kind&&a.unshift(t.createObjectLiteralExpression());var o=a[0];if(a.length>1){for(var s=1;s<a.length;s++)o=r().createAssignHelper([o,a[s]]);return o}return r().createAssignHelper(a)}return e.visitEachChild(i,C,n)}function U(t){return e.visitEachChild(t,I,n)}function V(t,r){return e.visitEachChild(t,r?I:C,n)}function j(r){var i=x(2,e.isEffectiveStrictModeSourceFile(r,l)?0:1);v=!1;var a=e.visitEachChild(r,C,n),o=e.concatenate(a.statements,g&&[t.createVariableStatement(void 0,t.createVariableDeclarationList(g))]),s=t.updateSourceFile(a,e.setTextRange(t.createNodeArray(o),r.statements));return L(i),s}function K(t){return e.processTaggedTemplateExpression(n,t,C,_,D,e.ProcessLevel.LiftRestriction)}function H(r,i){return e.isDestructuringAssignment(r)&&32768&r.left.transformFlags?e.flattenDestructuringAssignment(r,C,n,1,!i):27===r.operatorToken.kind?t.updateBinaryExpression(r,e.visitNode(r.left,I,e.isExpression),r.operatorToken,e.visitNode(r.right,i?I:C,e.isExpression)):e.visitEachChild(r,C,n)}function W(r,i){if(i)return e.visitEachChild(r,I,n);for(var a,o=0;o<r.elements.length;o++){var s=r.elements[o],c=e.visitNode(s,o<r.elements.length-1?I:C,e.isExpression);(a||c!==s)&&(a||(a=r.elements.slice(0,o)),a.push(c))}var l=a?e.setTextRange(t.createNodeArray(a),r.elements):r.elements;return t.updateCommaListExpression(r,l)}function z(r){if(r.variableDeclaration&&e.isBindingPattern(r.variableDeclaration.name)&&32768&r.variableDeclaration.name.transformFlags){var i=t.getGeneratedNameForNode(r.variableDeclaration.name),o=t.updateVariableDeclaration(r.variableDeclaration,r.variableDeclaration.name,void 0,void 0,i),s=e.flattenDestructuringBinding(o,C,n,1),c=e.visitNode(r.block,C,e.isBlock);return e.some(s)&&(c=t.updateBlock(c,a([t.createVariableStatement(void 0,s)],c.statements,!0))),t.updateCatchClause(r,t.updateVariableDeclaration(r.variableDeclaration,i,void 0,void 0,void 0),c)}return e.visitEachChild(r,C,n)}function J(t){if(e.hasSyntacticModifier(t,1)){var r=v;v=!0;var i=e.visitEachChild(t,C,n);return v=r,i}return e.visitEachChild(t,C,n)}function q(e){if(v){var n=v;v=!1;var t=X(e,!0);return v=n,t}return X(e,!1)}function X(t,r){return e.isBindingPattern(t.name)&&32768&t.name.transformFlags?e.flattenDestructuringBinding(t,C,n,1,void 0,r):e.visitEachChild(t,C,n)}function Y(r){return t.updateForStatement(r,e.visitNode(r.initializer,I,e.isForInitializer),e.visitNode(r.condition,C,e.isExpression),e.visitNode(r.incrementor,I,e.isExpression),e.visitIterationBody(r.statement,C,n))}function Q(t){return e.visitEachChild(t,I,n)}function Z(r,i){var a=x(0,2);32768&r.initializer.transformFlags&&(r=$(r));var o=r.awaitModifier?te(r,i,a):t.restoreEnclosingLabel(e.visitEachChild(r,C,n),i);return L(a),o}function $(n){var r=e.skipParentheses(n.initializer);if(e.isVariableDeclarationList(r)||e.isAssignmentPattern(r)){var i=void 0,a=void 0,o=t.createTempVariable(void 0),s=[e.createForOfBindingStatement(t,r,o)];return e.isBlock(n.statement)?(e.addRange(s,n.statement.statements),i=n.statement,a=n.statement.statements):n.statement&&(e.append(s,n.statement),i=n.statement,a=n.statement),t.updateForOfStatement(n,n.awaitModifier,e.setTextRange(t.createVariableDeclarationList([e.setTextRange(t.createVariableDeclaration(o),n.initializer)],1),n.initializer),n.expression,e.setTextRange(t.createBlock(e.setTextRange(t.createNodeArray(s),a),!0),i))}return n}function ee(r,i){var a,o,s=e.createForOfBindingStatement(t,r.initializer,i),c=[e.visitNode(s,C,e.isStatement)],l=e.visitIterationBody(r.statement,C,n);return e.isBlock(l)?(e.addRange(c,l.statements),a=l,o=l.statements):c.push(l),e.setEmitFlags(e.setTextRange(t.createBlock(e.setTextRange(t.createNodeArray(c),o),!0),a),432)}function ne(e){return 1&m?t.createYieldExpression(void 0,r().createAwaitHelper(e)):t.createAwaitExpression(e)}function te(n,i,a){var o=e.visitNode(n.expression,C,e.isExpression),c=e.isIdentifier(o)?t.getGeneratedNameForNode(o):t.createTempVariable(void 0),l=e.isIdentifier(o)?t.getGeneratedNameForNode(c):t.createTempVariable(void 0),u=t.createUniqueName("e"),d=t.getGeneratedNameForNode(u),p=t.createTempVariable(void 0),f=e.setTextRange(r().createAsyncValuesHelper(o),n.expression),m=t.createCallExpression(t.createPropertyAccessExpression(c,"next"),void 0,[]),_=t.createPropertyAccessExpression(l,"done"),g=t.createPropertyAccessExpression(l,"value"),y=t.createFunctionCallCall(p,c,[]);s(u),s(p);var h=2&a?t.inlineExpressions([t.createAssignment(u,t.createVoidZero()),f]):f,v=e.setEmitFlags(e.setTextRange(t.createForStatement(e.setEmitFlags(e.setTextRange(t.createVariableDeclarationList([e.setTextRange(t.createVariableDeclaration(c,void 0,void 0,h),n.expression),t.createVariableDeclaration(l)]),n.expression),2097152),t.createComma(t.createAssignment(l,ne(m)),t.createLogicalNot(_)),void 0,ee(n,g)),n),256);return t.createTryStatement(t.createBlock([t.restoreEnclosingLabel(v,i)]),t.createCatchClause(t.createVariableDeclaration(d),e.setEmitFlags(t.createBlock([t.createExpressionStatement(t.createAssignment(u,t.createObjectLiteralExpression([t.createPropertyAssignment("error",d)])))]),1)),t.createBlock([t.createTryStatement(t.createBlock([e.setEmitFlags(t.createIfStatement(t.createLogicalAnd(t.createLogicalAnd(l,t.createLogicalNot(_)),t.createAssignment(p,t.createPropertyAccessExpression(c,"return"))),t.createExpressionStatement(ne(y))),1)]),void 0,e.setEmitFlags(t.createBlock([e.setEmitFlags(t.createIfStatement(u,t.createThrowStatement(t.createPropertyAccessExpression(u,"error"))),1)]),1))]))}function re(r){return 32768&r.transformFlags?t.updateParameterDeclaration(r,void 0,void 0,r.dotDotDotToken,t.getGeneratedNameForNode(r),void 0,void 0,e.visitNode(r.initializer,C,e.isExpression)):e.visitEachChild(r,C,n)}function ie(r){var i=m;m=0;var a=t.updateConstructorDeclaration(r,void 0,r.modifiers,e.visitParameterList(r.parameters,C,n),pe(r));return m=i,a}function ae(r){var i=m;m=0;var a=t.updateGetAccessorDeclaration(r,void 0,r.modifiers,e.visitNode(r.name,C,e.isPropertyName),e.visitParameterList(r.parameters,C,n),void 0,pe(r));return m=i,a}function oe(r){var i=m;m=0;var a=t.updateSetAccessorDeclaration(r,void 0,r.modifiers,e.visitNode(r.name,C,e.isPropertyName),e.visitParameterList(r.parameters,C,n),pe(r));return m=i,a}function se(r){var i=m;m=e.getFunctionFlags(r);var a=t.updateMethodDeclaration(r,void 0,1&m?e.visitNodes(r.modifiers,N,e.isModifier):r.modifiers,2&m?void 0:r.asteriskToken,e.visitNode(r.name,C,e.isPropertyName),e.visitNode(void 0,C,e.isToken),void 0,e.visitParameterList(r.parameters,C,n),void 0,2&m&&1&m?de(r):pe(r));return m=i,a}function ce(r){var i=m;m=e.getFunctionFlags(r);var a=t.updateFunctionDeclaration(r,void 0,1&m?e.visitNodes(r.modifiers,N,e.isModifier):r.modifiers,2&m?void 0:r.asteriskToken,r.name,void 0,e.visitParameterList(r.parameters,C,n),void 0,2&m&&1&m?de(r):pe(r));return m=i,a}function le(r){var i=m;m=e.getFunctionFlags(r);var a=t.updateArrowFunction(r,r.modifiers,void 0,e.visitParameterList(r.parameters,C,n),void 0,r.equalsGreaterThanToken,pe(r));return m=i,a}function ue(r){var i=m;m=e.getFunctionFlags(r);var a=t.updateFunctionExpression(r,1&m?e.visitNodes(r.modifiers,N,e.isModifier):r.modifiers,2&m?void 0:r.asteriskToken,r.name,void 0,e.visitParameterList(r.parameters,C,n),void 0,2&m&&1&m?de(r):pe(r));return m=i,a}function de(a){i();var s=[],l=t.copyPrologue(a.body.statements,s,!1,C);fe(s,a);var d=y,p=h;y=new e.Set,h=!1;var f=t.createReturnStatement(r().createAsyncGeneratorHelper(t.createFunctionExpression(void 0,t.createToken(41),a.name&&t.getGeneratedNameForNode(a.name),void 0,[],void 0,t.updateBlock(a.body,e.visitLexicalEnvironment(a.body.statements,C,n,l))),!!(1&E))),m=u>=2&&6144&c.getNodeCheckFlags(a);if(m){me();var _=e.createSuperAccessVariableStatement(t,c,a,y);T[e.getNodeId(_)]=!0,e.insertStatementsAfterStandardPrologue(s,[_])}s.push(f),e.insertStatementsAfterStandardPrologue(s,o());var g=t.updateBlock(a.body,s);return m&&h&&(4096&c.getNodeCheckFlags(a)?e.addEmitHelper(g,e.advancedAsyncSuperHelper):2048&c.getNodeCheckFlags(a)&&e.addEmitHelper(g,e.asyncSuperHelper)),y=d,h=p,g}function pe(n){var r;i();var a=0,s=[],c=null!==(r=e.visitNode(n.body,C,e.isConciseBody))&&void 0!==r?r:t.createBlock([]);e.isBlock(c)&&(a=t.copyPrologue(c.statements,s,!1,C)),e.addRange(s,fe(void 0,n));var l=o();if(a>0||e.some(s)||e.some(l)){var u=t.converters.convertToFunctionBlock(c,!0);return e.insertStatementsAfterStandardPrologue(s,l),e.addRange(s,u.statements.slice(a)),t.updateBlock(u,e.setTextRange(t.createNodeArray(s),u.statements))}return c}function fe(r,i){for(var a=0,o=i.parameters;a<o.length;a++){var s=o[a];if(32768&s.transformFlags){var c=t.getGeneratedNameForNode(s),l=e.flattenDestructuringBinding(s,C,n,1,c,!1,!0);if(e.some(l)){var u=t.createVariableStatement(void 0,t.createVariableDeclarationList(l));e.setEmitFlags(u,1048576),r=e.append(r,u)}}}return r}function me(){0===(1&f)&&(f|=1,n.enableSubstitution(206),n.enableSubstitution(204),n.enableSubstitution(205),n.enableEmitNotification(255),n.enableEmitNotification(167),n.enableEmitNotification(170),n.enableEmitNotification(171),n.enableEmitNotification(169),n.enableEmitNotification(235))}function _e(n,t,r){if(1&f&&Ee(t)){var i=6144&c.getNodeCheckFlags(t);if(i!==b){var a=b;return b=i,d(n,t,r),void(b=a)}}else if(f&&T[e.getNodeId(t)]){a=b;return b=0,d(n,t,r),void(b=a)}d(n,t,r)}function ge(e,n){return n=p(e,n),1===e&&b?ye(n):n}function ye(e){switch(e.kind){case 204:return he(e);case 205:return ve(e);case 206:return be(e)}return e}function he(n){return 106===n.expression.kind?e.setTextRange(t.createPropertyAccessExpression(t.createUniqueName("_super",48),n.name),n):n}function ve(e){return 106===e.expression.kind?Te(e.argumentExpression,e):e}function be(n){var r=n.expression;if(e.isSuperProperty(r)){var i=e.isPropertyAccessExpression(r)?he(r):ve(r);return t.createCallExpression(t.createPropertyAccessExpression(i,"call"),void 0,a([t.createThis()],n.arguments,!0))}return n}function Ee(e){var n=e.kind;return 255===n||169===n||167===n||170===n||171===n}function Te(n,r){return 4096&b?e.setTextRange(t.createPropertyAccessExpression(t.createCallExpression(t.createIdentifier("_superIndex"),void 0,[n]),"value"),r):e.setTextRange(t.createCallExpression(t.createIdentifier("_superIndex"),void 0,[n]),r)}}(function(e){e[e["AsyncMethodsWithSuper"]=1]="AsyncMethodsWithSuper"})(n||(n={})),function(e){e[e["None"]=0]="None",e[e["HasLexicalThis"]=1]="HasLexicalThis",e[e["IterationContainer"]=2]="IterationContainer",e[e["AncestorFactsMask"]=3]="AncestorFactsMask",e[e["SourceFileIncludes"]=1]="SourceFileIncludes",e[e["SourceFileExcludes"]=2]="SourceFileExcludes",e[e["StrictModeSourceFileIncludes"]=0]="StrictModeSourceFileIncludes",e[e["ClassOrFunctionIncludes"]=1]="ClassOrFunctionIncludes",e[e["ClassOrFunctionExcludes"]=2]="ClassOrFunctionExcludes",e[e["ArrowFunctionIncludes"]=0]="ArrowFunctionIncludes",e[e["ArrowFunctionExcludes"]=2]="ArrowFunctionExcludes",e[e["IterationStatementIncludes"]=2]="IterationStatementIncludes",e[e["IterationStatementExcludes"]=0]="IterationStatementExcludes"}(t||(t={})),e.transformES2018=r}(d||(d={})),function(e){function n(n){var t=n.factory;return e.chainBundle(n,r);function r(t){return t.isDeclarationFile?t:e.visitEachChild(t,i,n)}function i(t){if(0===(32&t.transformFlags))return t;switch(t.kind){case 290:return a(t);default:return e.visitEachChild(t,i,n)}}function a(r){return r.variableDeclaration?e.visitEachChild(r,i,n):t.updateCatchClause(r,t.createVariableDeclaration(t.createTempVariable(void 0)),e.visitNode(r.block,i,e.isBlock))}}e.transformES2019=n}(d||(d={})),function(e){function n(n){var t=n.factory,r=n.hoistVariableDeclaration;return e.chainBundle(n,i);function i(t){return t.isDeclarationFile?t:e.visitEachChild(t,a,n)}function a(t){if(0===(16&t.transformFlags))return t;switch(t.kind){case 206:var r=l(t,!1);return e.Debug.assertNotNode(r,e.isSyntheticReference),r;case 204:case 205:if(e.isOptionalChain(t)){r=d(t,!1,!1);return e.Debug.assertNotNode(r,e.isSyntheticReference),r}return e.visitEachChild(t,a,n);case 219:return 60===t.operatorToken.kind?f(t):e.visitEachChild(t,a,n);case 213:return m(t);default:return e.visitEachChild(t,a,n)}}function o(n){e.Debug.assertNotNode(n,e.isNonNullChain);var t=[n];while(!n.questionDotToken&&!e.isTaggedTemplateExpression(n))n=e.cast(e.skipPartiallyEmittedExpressions(n.expression),e.isOptionalChain),e.Debug.assertNotNode(n,e.isNonNullChain),t.unshift(n);return{expression:n.expression,chain:t}}function s(n,r,i){var a=u(n.expression,r,i);return e.isSyntheticReference(a)?t.createSyntheticReferenceExpression(t.updateParenthesizedExpression(n,a.expression),a.thisArg):t.updateParenthesizedExpression(n,a)}function c(n,i,o){if(e.isOptionalChain(n))return d(n,i,o);var s,c=e.visitNode(n.expression,a,e.isExpression);return e.Debug.assertNotNode(c,e.isSyntheticReference),i&&(e.isSimpleCopiableExpression(c)?s=c:(s=t.createTempVariable(r),c=t.createAssignment(s,c))),c=204===n.kind?t.updatePropertyAccessExpression(n,c,e.visitNode(n.name,a,e.isIdentifier)):t.updateElementAccessExpression(n,c,e.visitNode(n.argumentExpression,a,e.isExpression)),s?t.createSyntheticReferenceExpression(c,s):c}function l(r,i){if(e.isOptionalChain(r))return d(r,i,!1);if(e.isParenthesizedExpression(r.expression)&&e.isOptionalChain(e.skipParentheses(r.expression))){var o=s(r.expression,!0,!1),c=e.visitNodes(r.arguments,a,e.isExpression);return e.isSyntheticReference(o)?e.setTextRange(t.createFunctionCallCall(o.expression,o.thisArg,c),r):t.updateCallExpression(r,o,void 0,c)}return e.visitEachChild(r,a,n)}function u(n,t,r){switch(n.kind){case 210:return s(n,t,r);case 204:case 205:return c(n,t,r);case 206:return l(n,t);default:return e.visitNode(n,a,e.isExpression)}}function d(n,i,s){var c=o(n),l=c.expression,d=c.chain,f=u(l,e.isCallChain(d[0]),!1),m=e.isSyntheticReference(f)?f.thisArg:void 0,_=e.isSyntheticReference(f)?f.expression:f,g=_;e.isSimpleCopiableExpression(_)||(g=t.createTempVariable(r),_=t.createAssignment(g,_));for(var y,h=g,v=0;v<d.length;v++){var b=d[v];switch(b.kind){case 204:case 205:v===d.length-1&&i&&(e.isSimpleCopiableExpression(h)?y=h:(y=t.createTempVariable(r),h=t.createAssignment(y,h))),h=204===b.kind?t.createPropertyAccessExpression(h,e.visitNode(b.name,a,e.isIdentifier)):t.createElementAccessExpression(h,e.visitNode(b.argumentExpression,a,e.isExpression));break;case 206:h=0===v&&m?t.createFunctionCallCall(h,106===m.kind?t.createThis():m,e.visitNodes(b.arguments,a,e.isExpression)):t.createCallExpression(h,void 0,e.visitNodes(b.arguments,a,e.isExpression));break}e.setOriginalNode(h,b)}var E=s?t.createConditionalExpression(p(_,g,!0),void 0,t.createTrue(),void 0,t.createDeleteExpression(h)):t.createConditionalExpression(p(_,g,!0),void 0,t.createVoidZero(),void 0,h);return e.setTextRange(E,n),y?t.createSyntheticReferenceExpression(E,y):E}function p(e,n,r){return t.createBinaryExpression(t.createBinaryExpression(e,t.createToken(r?36:37),t.createNull()),t.createToken(r?56:55),t.createBinaryExpression(n,t.createToken(r?36:37),t.createVoidZero()))}function f(n){var i=e.visitNode(n.left,a,e.isExpression),o=i;return e.isSimpleCopiableExpression(i)||(o=t.createTempVariable(r),i=t.createAssignment(o,i)),e.setTextRange(t.createConditionalExpression(p(i,o),void 0,o,void 0,e.visitNode(n.right,a,e.isExpression)),n)}function m(n){return e.isOptionalChain(e.skipParentheses(n.expression))?e.setOriginalNode(u(n.expression,!1,!0),n):t.updateDeleteExpression(n,e.visitNode(n.expression,a,e.isExpression))}}e.transformES2020=n}(d||(d={})),function(e){function n(n){var t=n.hoistVariableDeclaration,r=n.factory;return e.chainBundle(n,i);function i(t){return t.isDeclarationFile?t:e.visitEachChild(t,a,n)}function a(t){if(0===(8&t.transformFlags))return t;switch(t.kind){case 219:var r=t;if(e.isLogicalOrCoalescingAssignmentExpression(r))return o(r);default:return e.visitEachChild(t,a,n)}}function o(n){var i=n.operatorToken,o=e.getNonAssignmentOperatorForCompoundAssignment(i.kind),s=e.skipParentheses(e.visitNode(n.left,a,e.isLeftHandSideExpression)),c=s,l=e.skipParentheses(e.visitNode(n.right,a,e.isExpression));if(e.isAccessExpression(s)){var u=e.isSimpleCopiableExpression(s.expression),d=u?s.expression:r.createTempVariable(t),p=u?s.expression:r.createAssignment(d,s.expression);if(e.isPropertyAccessExpression(s))c=r.createPropertyAccessExpression(d,s.name),s=r.createPropertyAccessExpression(p,s.name);else{var f=e.isSimpleCopiableExpression(s.argumentExpression),m=f?s.argumentExpression:r.createTempVariable(t);c=r.createElementAccessExpression(d,m),s=r.createElementAccessExpression(p,f?s.argumentExpression:r.createAssignment(m,s.argumentExpression))}}return r.createBinaryExpression(s,o,r.createParenthesizedExpression(r.createAssignment(c,l)))}}e.transformES2021=n}(d||(d={})),function(e){function n(n){return e.chainBundle(n,t);function t(t){return t.isDeclarationFile?t:e.visitEachChild(t,r,n)}function r(t){if(0===(4&t.transformFlags))return t;switch(t.kind){default:return e.visitEachChild(t,r,n)}}}e.transformESNext=n}(d||(d={})),function(e){function n(n){var r,i,a=n.factory,o=n.getEmitHelperFactory,s=n.getCompilerOptions();return e.chainBundle(n,f);function c(){if(i.filenameDeclaration)return i.filenameDeclaration.name;var e=a.createVariableDeclaration(a.createUniqueName("_jsxFileName",48),void 0,void 0,a.createStringLiteral(r.fileName));return i.filenameDeclaration=e,i.filenameDeclaration.name}function l(e){return 5===s.jsx?"jsxDEV":e>1?"jsxs":"jsx"}function u(e){var n=l(e);return p(n)}function d(){return p("Fragment")}function p(n){var t,r,o="createElement"===n?i.importSpecifier:e.getJSXRuntimeImport(i.importSpecifier,s),c=null===(r=null===(t=i.utilizedImplicitRuntimeImports)||void 0===t?void 0:t.get(o))||void 0===r?void 0:r.get(n);if(c)return c.name;i.utilizedImplicitRuntimeImports||(i.utilizedImplicitRuntimeImports=e.createMap());var l=i.utilizedImplicitRuntimeImports.get(o);l||(l=e.createMap(),i.utilizedImplicitRuntimeImports.set(o,l));var u=a.createUniqueName("_"+n,112),d=a.createImportSpecifier(a.createIdentifier(n),u);return u.generatedImportReference=d,l.set(n,d),u}function f(t){if(t.isDeclarationFile)return t;r=t,i={},i.importSpecifier=e.getJSXImplicitImportBase(s,t);var o=e.visitEachChild(t,m,n);e.addEmitHelpers(o,n.readEmitHelpers());var c=o.statements;if(i.filenameDeclaration&&(c=e.insertStatementAfterCustomPrologue(c.slice(),a.createVariableStatement(void 0,a.createVariableDeclarationList([i.filenameDeclaration],2)))),i.utilizedImplicitRuntimeImports)for(var l=0,u=e.arrayFrom(i.utilizedImplicitRuntimeImports.entries());l<u.length;l++){var d=u[l],p=d[0],f=d[1];if(e.isExternalModule(t)){var _=a.createImportDeclaration(void 0,void 0,a.createImportClause(!1,void 0,a.createNamedImports(e.arrayFrom(f.values()))),a.createStringLiteral(p));e.setParentRecursive(_,!1),c=e.insertStatementAfterCustomPrologue(c.slice(),_)}else if(e.isExternalOrCommonJsModule(t)){var g=a.createVariableStatement(void 0,a.createVariableDeclarationList([a.createVariableDeclaration(a.createObjectBindingPattern(e.map(e.arrayFrom(f.values()),function(e){return a.createBindingElement(void 0,e.propertyName,e.name)})),void 0,void 0,a.createCallExpression(a.createIdentifier("require"),void 0,[a.createStringLiteral(p)]))],2));e.setParentRecursive(g,!1),c=e.insertStatementAfterCustomPrologue(c.slice(),g)}}return c!==o.statements&&(o=a.updateSourceFile(o,c)),i=void 0,o}function m(e){return 2&e.transformFlags?_(e):e}function _(t){switch(t.kind){case 276:return v(t,!1);case 277:return b(t,!1);case 280:return E(t,!1);case 286:return B(t);default:return e.visitEachChild(t,m,n)}}function g(n){switch(n.kind){case 11:return R(n);case 286:return B(n);case 276:return v(n,!0);case 277:return b(n,!0);case 280:return E(n,!0);default:return e.Debug.failBadSyntaxKind(n)}}function y(n){for(var t=!1,r=0,i=n.attributes.properties;r<i.length;r++){var a=i[r];if(e.isJsxSpreadAttribute(a))t=!0;else if(t&&e.isJsxAttribute(a)&&"key"===a.name.escapedText)return!0}return!1}function h(e){return void 0===i.importSpecifier||y(e)}function v(e,n){var t=h(e.openingElement)?L:S;return t(e.openingElement,e.children,n,e)}function b(e,n){var t=h(e)?L:S;return t(e,void 0,n,e)}function E(e,n){var t=void 0===i.importSpecifier?A:D;return t(e.openingFragment,e.children,n,e)}function T(n){var t=e.getSemanticJsxChildren(n);if(1===e.length(t)){var r=g(t[0]);return r&&a.createObjectLiteralExpression([a.createPropertyAssignment("children",r)])}var i=e.mapDefined(n,g);return i.length?a.createObjectLiteralExpression([a.createPropertyAssignment("children",a.createArrayLiteralExpression(i))]):void 0}function S(n,t,r,i){var s,c=F(n),l=e.find(n.attributes.properties,function(n){return!!n.name&&e.isIdentifier(n.name)&&"key"===n.name.escapedText}),u=l?e.filter(n.attributes.properties,function(e){return e!==l}):n.attributes.properties,d=[];if(u.length&&(d=e.flatten(e.spanMap(u,e.isJsxSpreadAttribute,function(n,t){return t?e.map(n,I):a.createObjectLiteralExpression(e.map(n,N))})),e.isJsxSpreadAttribute(u[0])&&d.unshift(a.createObjectLiteralExpression())),t&&t.length){var p=T(t);p&&d.push(p)}return s=0===d.length?a.createObjectLiteralExpression([]):e.singleOrUndefined(d)||o().createAssignHelper(d),x(c,s,l,e.length(e.getSemanticJsxChildren(t||e.emptyArray)),r,i)}function x(n,t,i,o,l,d){var p=[n,t,i?k(i.initializer):a.createVoidZero()];if(5===s.jsx){var f=e.getOriginalNode(r);if(f&&e.isSourceFile(f)){p.push(o>1?a.createTrue():a.createFalse());var m=e.getLineAndCharacterOfPosition(f,d.pos);p.push(a.createObjectLiteralExpression([a.createPropertyAssignment("fileName",c()),a.createPropertyAssignment("lineNumber",a.createNumericLiteral(m.line+1)),a.createPropertyAssignment("columnNumber",a.createNumericLiteral(m.character+1))])),p.push(a.createThis())}}var _=e.setTextRange(a.createCallExpression(u(o),void 0,p),d);return l&&e.startOnNewLine(_),_}function L(t,c,l,u){var d,f=F(t),m=t.attributes.properties;if(0===m.length)d=a.createNull();else{var _=s.target;if(_&&_>=5)d=a.createObjectLiteralExpression(e.flatten(e.spanMap(m,e.isJsxSpreadAttribute,function(n,t){return t?e.map(n,C):e.map(n,N)})));else{var y=e.flatten(e.spanMap(m,e.isJsxSpreadAttribute,function(n,t){return t?e.map(n,I):a.createObjectLiteralExpression(e.map(n,N))}));e.isJsxSpreadAttribute(m[0])&&y.unshift(a.createObjectLiteralExpression()),d=e.singleOrUndefined(y),d||(d=o().createAssignHelper(y))}}var h=void 0===i.importSpecifier?e.createJsxFactoryExpression(a,n.getEmitResolver().getJsxFactoryEntity(r),s.reactNamespace,t):p("createElement"),v=e.createExpressionForJsxElement(a,h,f,d,e.mapDefined(c,g),u);return l&&e.startOnNewLine(v),v}function D(n,t,r,i){var o;if(t&&t.length){var s=T(t);s&&(o=s)}return x(d(),o||a.createObjectLiteralExpression([]),void 0,e.length(e.getSemanticJsxChildren(t)),r,i)}function A(t,i,o,c){var l=e.createExpressionForJsxFragment(a,n.getEmitResolver().getJsxFactoryEntity(r),n.getEmitResolver().getJsxFragmentFactoryEntity(r),s.reactNamespace,e.mapDefined(i,g),t,c);return o&&e.startOnNewLine(l),l}function C(n){return a.createSpreadAssignment(e.visitNode(n.expression,m,e.isExpression))}function I(n){return e.visitNode(n.expression,m,e.isExpression)}function N(e){var n=G(e),t=k(e.initializer);return a.createPropertyAssignment(n,t)}function k(n){if(void 0===n)return a.createTrue();if(10===n.kind){var t=void 0!==n.singleQuote?n.singleQuote:!e.isStringDoubleQuoted(n,r),i=a.createStringLiteral(M(n.text)||n.text,t);return e.setTextRange(i,n)}return 286===n.kind?void 0===n.expression?a.createTrue():e.visitNode(n.expression,m,e.isExpression):e.Debug.failBadSyntaxKind(n)}function R(e){var n=O(e.text);return void 0===n?void 0:a.createStringLiteral(n)}function O(n){for(var t,r=0,i=-1,a=0;a<n.length;a++){var o=n.charCodeAt(a);e.isLineBreak(o)?(-1!==r&&-1!==i&&(t=w(t,n.substr(r,i-r+1))),r=-1):e.isWhiteSpaceSingleLine(o)||(i=a,-1===r&&(r=a))}return-1!==r?w(t,n.substr(r)):t}function w(e,n){var t=P(n);return void 0===e?t:e+" "+t}function P(n){return n.replace(/&((#((\d+)|x([\da-fA-F]+)))|(\w+));/g,function(n,r,i,a,o,s,c){if(o)return e.utf16EncodeAsString(parseInt(o,10));if(s)return e.utf16EncodeAsString(parseInt(s,16));var l=t.get(c);return l?e.utf16EncodeAsString(l):n})}function M(e){var n=P(e);return n===e?void 0:n}function F(n){if(276===n.kind)return F(n.openingElement);var t=n.tagName;return e.isIdentifier(t)&&e.isIntrinsicJsxName(t.escapedText)?a.createStringLiteral(e.idText(t)):e.createExpressionFromEntityName(a,t)}function G(n){var t=n.name,r=e.idText(t);return/^[A-Za-z_]\w*$/.test(r)?t:a.createStringLiteral(r)}function B(n){return e.visitNode(n.expression,m,e.isExpression)}}e.transformJsx=n;var t=new e.Map(e.getEntries({quot:34,amp:38,apos:39,lt:60,gt:62,nbsp:160,iexcl:161,cent:162,pound:163,curren:164,yen:165,brvbar:166,sect:167,uml:168,copy:169,ordf:170,laquo:171,not:172,shy:173,reg:174,macr:175,deg:176,plusmn:177,sup2:178,sup3:179,acute:180,micro:181,para:182,middot:183,cedil:184,sup1:185,ordm:186,raquo:187,frac14:188,frac12:189,frac34:190,iquest:191,Agrave:192,Aacute:193,Acirc:194,Atilde:195,Auml:196,Aring:197,AElig:198,Ccedil:199,Egrave:200,Eacute:201,Ecirc:202,Euml:203,Igrave:204,Iacute:205,Icirc:206,Iuml:207,ETH:208,Ntilde:209,Ograve:210,Oacute:211,Ocirc:212,Otilde:213,Ouml:214,times:215,Oslash:216,Ugrave:217,Uacute:218,Ucirc:219,Uuml:220,Yacute:221,THORN:222,szlig:223,agrave:224,aacute:225,acirc:226,atilde:227,auml:228,aring:229,aelig:230,ccedil:231,egrave:232,eacute:233,ecirc:234,euml:235,igrave:236,iacute:237,icirc:238,iuml:239,eth:240,ntilde:241,ograve:242,oacute:243,ocirc:244,otilde:245,ouml:246,divide:247,oslash:248,ugrave:249,uacute:250,ucirc:251,uuml:252,yacute:253,thorn:254,yuml:255,OElig:338,oelig:339,Scaron:352,scaron:353,Yuml:376,fnof:402,circ:710,tilde:732,Alpha:913,Beta:914,Gamma:915,Delta:916,Epsilon:917,Zeta:918,Eta:919,Theta:920,Iota:921,Kappa:922,Lambda:923,Mu:924,Nu:925,Xi:926,Omicron:927,Pi:928,Rho:929,Sigma:931,Tau:932,Upsilon:933,Phi:934,Chi:935,Psi:936,Omega:937,alpha:945,beta:946,gamma:947,delta:948,epsilon:949,zeta:950,eta:951,theta:952,iota:953,kappa:954,lambda:955,mu:956,nu:957,xi:958,omicron:959,pi:960,rho:961,sigmaf:962,sigma:963,tau:964,upsilon:965,phi:966,chi:967,psi:968,omega:969,thetasym:977,upsih:978,piv:982,ensp:8194,emsp:8195,thinsp:8201,zwnj:8204,zwj:8205,lrm:8206,rlm:8207,ndash:8211,mdash:8212,lsquo:8216,rsquo:8217,sbquo:8218,ldquo:8220,rdquo:8221,bdquo:8222,dagger:8224,Dagger:8225,bull:8226,hellip:8230,permil:8240,prime:8242,Prime:8243,lsaquo:8249,rsaquo:8250,oline:8254,frasl:8260,euro:8364,image:8465,weierp:8472,real:8476,trade:8482,alefsym:8501,larr:8592,uarr:8593,rarr:8594,darr:8595,harr:8596,crarr:8629,lArr:8656,uArr:8657,rArr:8658,dArr:8659,hArr:8660,forall:8704,part:8706,exist:8707,empty:8709,nabla:8711,isin:8712,notin:8713,ni:8715,prod:8719,sum:8721,minus:8722,lowast:8727,radic:8730,prop:8733,infin:8734,ang:8736,and:8743,or:8744,cap:8745,cup:8746,int:8747,there4:8756,sim:8764,cong:8773,asymp:8776,ne:8800,equiv:8801,le:8804,ge:8805,sub:8834,sup:8835,nsub:8836,sube:8838,supe:8839,oplus:8853,otimes:8855,perp:8869,sdot:8901,lceil:8968,rceil:8969,lfloor:8970,rfloor:8971,lang:9001,rang:9002,loz:9674,spades:9824,clubs:9827,hearts:9829,diams:9830}))}(d||(d={})),function(e){function n(n){var t=n.factory,r=n.hoistVariableDeclaration;return e.chainBundle(n,i);function i(t){return t.isDeclarationFile?t:e.visitEachChild(t,a,n)}function a(t){if(0===(256&t.transformFlags))return t;switch(t.kind){case 219:return o(t);default:return e.visitEachChild(t,a,n)}}function o(t){switch(t.operatorToken.kind){case 67:return s(t);case 42:return c(t);default:return e.visitEachChild(t,a,n)}}function s(n){var i,o,s=e.visitNode(n.left,a,e.isExpression),c=e.visitNode(n.right,a,e.isExpression);if(e.isElementAccessExpression(s)){var l=t.createTempVariable(r),u=t.createTempVariable(r);i=e.setTextRange(t.createElementAccessExpression(e.setTextRange(t.createAssignment(l,s.expression),s.expression),e.setTextRange(t.createAssignment(u,s.argumentExpression),s.argumentExpression)),s),o=e.setTextRange(t.createElementAccessExpression(l,u),s)}else if(e.isPropertyAccessExpression(s)){l=t.createTempVariable(r);i=e.setTextRange(t.createPropertyAccessExpression(e.setTextRange(t.createAssignment(l,s.expression),s.expression),s.name),s),o=e.setTextRange(t.createPropertyAccessExpression(l,s.name),s)}else i=s,o=s;return e.setTextRange(t.createAssignment(i,e.setTextRange(t.createGlobalMethodCall("Math","pow",[o,c]),n)),n)}function c(n){var r=e.visitNode(n.left,a,e.isExpression),i=e.visitNode(n.right,a,e.isExpression);return e.setTextRange(t.createGlobalMethodCall("Math","pow",[r,i]),n)}}e.transformES2016=n}(d||(d={})),function(e){var n,t,r,i,o,s;function c(e,n){return{kind:e,expression:n}}function l(n){var t,r,i,o,s,l,u=n.factory,d=n.getEmitHelperFactory,p=n.startLexicalEnvironment,f=n.resumeLexicalEnvironment,m=n.endLexicalEnvironment,_=n.hoistVariableDeclaration,g=n.getCompilerOptions(),y=n.getEmitResolver(),h=n.onSubstituteNode,v=n.onEmitNode;function b(n){o=e.append(o,u.createVariableDeclaration(n))}return n.onEmitNode=Zn,n.onSubstituteNode=nt,e.chainBundle(n,E);function E(a){if(a.isDeclarationFile)return a;t=a,r=a.text;var s=R(a);return e.addEmitHelpers(s,n.readEmitHelpers()),t=void 0,r=void 0,o=void 0,i=0,s}function T(e,n){var t=i;return i=32767&(i&~e|n),t}function S(e,n,t){i=-32768&(i&~n|t)|e}function x(e){return 0!==(8192&i)&&245===e.kind&&!e.expression}function L(n){return 2097152&n.transformFlags&&(e.isReturnStatement(n)||e.isIfStatement(n)||e.isWithStatement(n)||e.isSwitchStatement(n)||e.isCaseBlock(n)||e.isCaseClause(n)||e.isDefaultClause(n)||e.isTryStatement(n)||e.isCatchClause(n)||e.isLabeledStatement(n)||e.isIterationStatement(n,!1)||e.isBlock(n))}function D(n){return 0!==(512&n.transformFlags)||void 0!==s||8192&i&&L(n)||e.isIterationStatement(n,!1)&&Ye(n)||0!==(33554432&e.getEmitFlags(n))}function A(e){return D(e)?k(e,!1):e}function C(e){return D(e)?k(e,!0):e}function I(n){if(D(n)){var t=e.getOriginalNode(n);if(e.isPropertyDeclaration(t)&&e.hasStaticModifier(t)){var r=T(32670,16449),i=k(n,!1);return S(r,98304,0),i}return k(n,!1)}return n}function N(e){return 106===e.kind?Yn(!0):A(e)}function k(t,r){switch(t.kind){case 124:return;case 255:return V(t);case 224:return j(t);case 162:return $(t);case 254:return ge(t);case 212:return me(t);case 211:return _e(t);case 252:return Ne(t);case 79:return B(t);case 253:return De(t);case 247:return O(t);case 261:return w(t);case 233:return ve(t,!1);case 244:case 243:return U(t);case 248:return Oe(t);case 238:case 239:return Me(t,void 0);case 240:return Fe(t,void 0);case 241:return Be(t,void 0);case 242:return Ue(t,void 0);case 236:return be(t);case 203:return We(t);case 290:return Sn(t);case 292:return An(t);case 160:return Cn(t);case 202:return Nn(t);case 206:return kn(t);case 207:return Pn(t);case 210:return Ee(t,r);case 219:return Te(t,r);case 346:return Se(t,r);case 14:case 15:case 16:case 17:return jn(t);case 10:return Kn(t);case 8:return Hn(t);case 208:return Wn(t);case 221:return zn(t);case 222:return In(t);case 223:return Vn(t);case 106:return Yn(!1);case 108:return F(t);case 229:return Qn(t);case 167:return Ln(t);case 170:case 171:return Dn(t);case 235:return Le(t);case 245:return M(t);case 215:return G(t);default:return e.visitEachChild(t,A,n)}}function R(n){var t=T(8064,64),r=[],i=[];p();var a=u.copyPrologue(n.statements,r,!1,A);return e.addRange(i,e.visitNodes(n.statements,A,e.isStatement,a)),o&&i.push(u.createVariableStatement(void 0,u.createVariableDeclarationList(o))),u.mergeLexicalEnvironment(r,m()),oe(r,n),S(t,0,0),u.updateSourceFile(n,e.setTextRange(u.createNodeArray(e.concatenate(r,i)),n.statements))}function O(t){if(void 0!==s){var r=s.allowedNonLabeledJumps;s.allowedNonLabeledJumps|=2;var i=e.visitEachChild(t,A,n);return s.allowedNonLabeledJumps=r,i}return e.visitEachChild(t,A,n)}function w(t){var r=T(7104,0),i=e.visitEachChild(t,A,n);return S(r,0,0),i}function P(n){return e.setOriginalNode(u.createReturnStatement(u.createUniqueName("_this",48)),n)}function M(t){return s?(s.nonLocalJumps|=8,x(t)&&(t=P(t)),u.createReturnStatement(u.createObjectLiteralExpression([u.createPropertyAssignment(u.createIdentifier("value"),t.expression?e.visitNode(t.expression,A,e.isExpression):u.createVoidZero())]))):x(t)?P(t):e.visitEachChild(t,A,n)}function F(e){return 2&i&&!(16384&i)&&(i|=65536),s?2&i?(s.containsLexicalThis=!0,e):s.thisName||(s.thisName=u.createUniqueName("this")):e}function G(t){return e.visitEachChild(t,C,n)}function B(e){return s&&y.isArgumentsLocalBinding(e)?s.argumentsName||(s.argumentsName=u.createUniqueName("arguments")):e}function U(t){if(s){var r=244===t.kind?2:4,i=t.label&&s.labels&&s.labels.get(e.idText(t.label))||!t.label&&s.allowedNonLabeledJumps&r;if(!i){var a=void 0,o=t.label;o?244===t.kind?(a="break-"+o.escapedText,gn(s,!0,e.idText(o),a)):(a="continue-"+o.escapedText,gn(s,!1,e.idText(o),a)):244===t.kind?(s.nonLocalJumps|=2,a="break"):(s.nonLocalJumps|=4,a="continue");var c=u.createStringLiteral(a);if(s.loopOutParameters.length){for(var l=s.loopOutParameters,d=void 0,p=0;p<l.length;p++){var f=pn(l[p],1);d=0===p?f:u.createBinaryExpression(d,27,f)}c=u.createBinaryExpression(d,27,c)}return u.createReturnStatement(c)}}return e.visitEachChild(t,A,n)}function V(n){var t=u.createVariableDeclaration(u.getLocalName(n,!0),void 0,void 0,K(n));e.setOriginalNode(t,n);var r=[],i=u.createVariableStatement(void 0,u.createVariableDeclarationList([t]));if(e.setOriginalNode(i,n),e.setTextRange(i,n),e.startOnNewLine(i),r.push(i),e.hasSyntacticModifier(n,1)){var a=e.hasSyntacticModifier(n,512)?u.createExportDefault(u.getLocalName(n)):u.createExternalModuleExport(u.getLocalName(n));e.setOriginalNode(a,i),r.push(a)}var o=e.getEmitFlags(n);return 0===(4194304&o)&&(r.push(u.createEndOfDeclarationMarker(n)),e.setEmitFlags(i,4194304|o)),e.singleOrMany(r)}function j(e){return K(e)}function K(n){n.name&&$n();var t=e.getClassExtendsHeritageElement(n),i=u.createFunctionExpression(void 0,void 0,void 0,void 0,t?[u.createParameterDeclaration(void 0,void 0,void 0,u.createUniqueName("_super",48))]:[],void 0,H(n,t));e.setEmitFlags(i,65536&e.getEmitFlags(n)|524288);var a=u.createPartiallyEmittedExpression(i);e.setTextRangeEnd(a,n.end),e.setEmitFlags(a,1536);var o=u.createPartiallyEmittedExpression(a);e.setTextRangeEnd(o,e.skipTrivia(r,n.pos)),e.setEmitFlags(o,1536);var s=u.createParenthesizedExpression(u.createCallExpression(o,void 0,t?[e.visitNode(t.expression,A,e.isExpression)]:[]));return e.addSyntheticLeadingComment(s,3,"* @class "),s}function H(n,t){var i=[],a=u.getInternalName(n),o=e.isIdentifierANonContextualKeyword(a)?u.getGeneratedNameForNode(a):a;p(),W(i,n,t),z(i,n,o,t),le(i,n);var s=e.createTokenRange(e.skipTrivia(r,n.members.end),19),c=u.createPartiallyEmittedExpression(o);e.setTextRangeEnd(c,s.end),e.setEmitFlags(c,1536);var l=u.createReturnStatement(c);e.setTextRangePos(l,s.pos),e.setEmitFlags(l,1920),i.push(l),e.insertStatementsAfterStandardPrologue(i,m());var d=u.createBlock(e.setTextRange(u.createNodeArray(i),n.members),!0);return e.setEmitFlags(d,1536),d}function W(n,t,r){r&&n.push(e.setTextRange(u.createExpressionStatement(d().createExtendsHelper(u.getInternalName(t))),r))}function z(n,t,r,i){var a=s;s=void 0;var o=T(32662,73),c=e.getFirstConstructorWithBody(t),l=lt(c,void 0!==i),d=u.createFunctionDeclaration(void 0,void 0,void 0,r,void 0,J(c,l),void 0,X(c,t,i,l));e.setTextRange(d,c||t),i&&e.setEmitFlags(d,8),n.push(d),S(o,98304,0),s=a}function J(t,r){return e.visitParameterList(t&&!r?t.parameters:void 0,A,n)||[]}function q(n,t){var r=[];f(),u.mergeLexicalEnvironment(r,m()),t&&r.push(u.createReturnStatement(Z()));var i=u.createNodeArray(r);e.setTextRange(i,n.members);var a=u.createBlock(i,!0);return e.setTextRange(a,n),e.setEmitFlags(a,1536),a}function X(n,t,r,a){var o=!!r&&104!==e.skipOuterExpressions(r.expression).kind;if(!n)return q(t,o);var s=[],c=[];f();var l,d=0;if(a||(d=u.copyStandardPrologue(n.body.statements,s,!1)),ne(c,n),ae(c,n,a),a||(d=u.copyCustomPrologue(n.body.statements,c,d,A)),a)l=Z();else if(o&&d<n.body.statements.length){var p=n.body.statements[d];e.isExpressionStatement(p)&&e.isSuperCall(p.expression)&&(l=On(p.expression))}if(l&&(i|=8192,d++),e.addRange(c,e.visitNodes(n.body.statements,A,e.isStatement,d)),u.mergeLexicalEnvironment(s,m()),ce(s,n,!1),o)if(!l||d!==n.body.statements.length||8192&n.body.transformFlags)se(c,n,l||Q()),Y(n.body)||c.push(u.createReturnStatement(u.createUniqueName("_this",48)));else{var _=e.cast(e.cast(l,e.isBinaryExpression).left,e.isCallExpression),g=u.createReturnStatement(l);e.setCommentRange(g,e.getCommentRange(_)),e.setEmitFlags(_,1536),c.push(g)}else oe(s,n);var y=u.createBlock(e.setTextRange(u.createNodeArray(e.concatenate(s,c)),n.body.statements),!0);return e.setTextRange(y,n.body),y}function Y(n){if(245===n.kind)return!0;if(237===n.kind){var t=n;if(t.elseStatement)return Y(t.thenStatement)&&Y(t.elseStatement)}else if(233===n.kind){var r=e.lastOrUndefined(n.statements);if(r&&Y(r))return!0}return!1}function Q(){return e.setEmitFlags(u.createThis(),4)}function Z(){return u.createLogicalOr(u.createLogicalAnd(u.createStrictInequality(u.createUniqueName("_super",48),u.createNull()),u.createFunctionApplyCall(u.createUniqueName("_super",48),Q(),u.createIdentifier("arguments"))),Q())}function $(n){return n.dotDotDotToken?void 0:e.isBindingPattern(n.name)?e.setOriginalNode(e.setTextRange(u.createParameterDeclaration(void 0,void 0,void 0,u.getGeneratedNameForNode(n),void 0,void 0,void 0),n),n):n.initializer?e.setOriginalNode(e.setTextRange(u.createParameterDeclaration(void 0,void 0,void 0,n.name,void 0,void 0,void 0),n),n):n}function ee(n){return void 0!==n.initializer||e.isBindingPattern(n.name)}function ne(n,t){if(!e.some(t.parameters,ee))return!1;for(var r=!1,i=0,a=t.parameters;i<a.length;i++){var o=a[i],s=o.name,c=o.initializer,l=o.dotDotDotToken;l||(e.isBindingPattern(s)?r=te(n,o,s,c)||r:c&&(re(n,o,s,c),r=!0))}return r}function te(t,r,i,a){return i.elements.length>0?(e.insertStatementAfterCustomPrologue(t,e.setEmitFlags(u.createVariableStatement(void 0,u.createVariableDeclarationList(e.flattenDestructuringBinding(r,A,n,0,u.getGeneratedNameForNode(r)))),1048576)),!0):!!a&&(e.insertStatementAfterCustomPrologue(t,e.setEmitFlags(u.createExpressionStatement(u.createAssignment(u.getGeneratedNameForNode(r),e.visitNode(a,A,e.isExpression))),1048576)),!0)}function re(n,t,r,i){i=e.visitNode(i,A,e.isExpression);var a=u.createIfStatement(u.createTypeCheck(u.cloneNode(r),"undefined"),e.setEmitFlags(e.setTextRange(u.createBlock([u.createExpressionStatement(e.setEmitFlags(e.setTextRange(u.createAssignment(e.setEmitFlags(e.setParent(e.setTextRange(u.cloneNode(r),r),r.parent),48),e.setEmitFlags(i,1584|e.getEmitFlags(i))),t),1536))]),t),1953));e.startOnNewLine(a),e.setTextRange(a,t),e.setEmitFlags(a,1050528),e.insertStatementAfterCustomPrologue(n,a)}function ie(e,n){return!(!e||!e.dotDotDotToken||n)}function ae(t,r,i){var a=[],o=e.lastOrUndefined(r.parameters);if(!ie(o,i))return!1;var s=79===o.name.kind?e.setParent(e.setTextRange(u.cloneNode(o.name),o.name),o.name.parent):u.createTempVariable(void 0);e.setEmitFlags(s,48);var c=79===o.name.kind?u.cloneNode(o.name):s,l=r.parameters.length-1,d=u.createLoopVariable();a.push(e.setEmitFlags(e.setTextRange(u.createVariableStatement(void 0,u.createVariableDeclarationList([u.createVariableDeclaration(s,void 0,void 0,u.createArrayLiteralExpression([]))])),o),1048576));var p=u.createForStatement(e.setTextRange(u.createVariableDeclarationList([u.createVariableDeclaration(d,void 0,void 0,u.createNumericLiteral(l))]),o),e.setTextRange(u.createLessThan(d,u.createPropertyAccessExpression(u.createIdentifier("arguments"),"length")),o),e.setTextRange(u.createPostfixIncrement(d),o),u.createBlock([e.startOnNewLine(e.setTextRange(u.createExpressionStatement(u.createAssignment(u.createElementAccessExpression(c,0===l?d:u.createSubtract(d,u.createNumericLiteral(l))),u.createElementAccessExpression(u.createIdentifier("arguments"),d))),o))]));return e.setEmitFlags(p,1048576),e.startOnNewLine(p),a.push(p),79!==o.name.kind&&a.push(e.setEmitFlags(e.setTextRange(u.createVariableStatement(void 0,u.createVariableDeclarationList(e.flattenDestructuringBinding(o,A,n,0,c))),o),1048576)),e.insertStatementsAfterCustomPrologue(t,a),!0}function oe(e,n){return!!(65536&i&&212!==n.kind)&&(se(e,n,u.createThis()),!0)}function se(n,t,r){et();var i=u.createVariableStatement(void 0,u.createVariableDeclarationList([u.createVariableDeclaration(u.createUniqueName("_this",48),void 0,void 0,r)]));e.setEmitFlags(i,1050112),e.setSourceMapRange(i,t),e.insertStatementAfterCustomPrologue(n,i)}function ce(n,t,r){if(32768&i){var a=void 0;switch(t.kind){case 212:return n;case 167:case 170:case 171:a=u.createVoidZero();break;case 169:a=u.createPropertyAccessExpression(e.setEmitFlags(u.createThis(),4),"constructor");break;case 254:case 211:a=u.createConditionalExpression(u.createLogicalAnd(e.setEmitFlags(u.createThis(),4),u.createBinaryExpression(e.setEmitFlags(u.createThis(),4),102,u.getLocalName(t))),void 0,u.createPropertyAccessExpression(e.setEmitFlags(u.createThis(),4),"constructor"),void 0,u.createVoidZero());break;default:return e.Debug.failBadSyntaxKind(t)}var o=u.createVariableStatement(void 0,u.createVariableDeclarationList([u.createVariableDeclaration(u.createUniqueName("_newTarget",48),void 0,void 0,a)]));e.setEmitFlags(o,1050112),r&&(n=n.slice()),e.insertStatementAfterCustomPrologue(n,o)}return n}function le(n,r){for(var i=0,a=r.members;i<a.length;i++){var o=a[i];switch(o.kind){case 232:n.push(ue(o));break;case 167:n.push(de(ct(r,o),o,r));break;case 170:case 171:var s=e.getAllAccessorDeclarations(r.members,o);o===s.firstAccessor&&n.push(pe(ct(r,o),s,r));break;case 169:case 168:break;default:e.Debug.failBadSyntaxKind(o,t&&t.fileName);break}}}function ue(n){return e.setTextRange(u.createEmptyStatement(),n)}function de(t,r,i){var a,o=e.getCommentRange(r),s=e.getSourceMapRange(r),c=ye(r,r,void 0,i),l=e.visitNode(r.name,A,e.isPropertyName);if(!e.isPrivateIdentifier(l)&&e.getUseDefineForClassFields(n.getCompilerOptions())){var d=e.isComputedPropertyName(l)?l.expression:e.isIdentifier(l)?u.createStringLiteral(e.unescapeLeadingUnderscores(l.escapedText)):l;a=u.createObjectDefinePropertyCall(t,d,u.createPropertyDescriptor({value:c,enumerable:!1,writable:!0,configurable:!0}))}else{var p=e.createMemberAccessForPropertyName(u,t,l,r.name);a=u.createAssignment(p,c)}e.setEmitFlags(c,1536),e.setSourceMapRange(c,s);var f=e.setTextRange(u.createExpressionStatement(a),r);return e.setOriginalNode(f,r),e.setCommentRange(f,o),e.setEmitFlags(f,48),f}function pe(n,t,r){var i=u.createExpressionStatement(fe(n,t,r,!1));return e.setEmitFlags(i,1536),e.setSourceMapRange(i,e.getSourceMapRange(t.firstAccessor)),i}function fe(n,t,r,i){var a=t.firstAccessor,o=t.getAccessor,s=t.setAccessor,c=e.setParent(e.setTextRange(u.cloneNode(n),n),n.parent);e.setEmitFlags(c,1568),e.setSourceMapRange(c,a.name);var l=e.visitNode(a.name,A,e.isPropertyName);if(e.isPrivateIdentifier(l))return e.Debug.failBadSyntaxKind(l,"Encountered unhandled private identifier while transforming ES2015.");var d=e.createExpressionForPropertyName(u,l);e.setEmitFlags(d,1552),e.setSourceMapRange(d,a.name);var p=[];if(o){var f=ye(o,void 0,void 0,r);e.setSourceMapRange(f,e.getSourceMapRange(o)),e.setEmitFlags(f,512);var m=u.createPropertyAssignment("get",f);e.setCommentRange(m,e.getCommentRange(o)),p.push(m)}if(s){var _=ye(s,void 0,void 0,r);e.setSourceMapRange(_,e.getSourceMapRange(s)),e.setEmitFlags(_,512);var g=u.createPropertyAssignment("set",_);e.setCommentRange(g,e.getCommentRange(s)),p.push(g)}p.push(u.createPropertyAssignment("enumerable",o||s?u.createFalse():u.createTrue()),u.createPropertyAssignment("configurable",u.createTrue()));var y=u.createCallExpression(u.createPropertyAccessExpression(u.createIdentifier("Object"),"defineProperty"),void 0,[c,d,u.createObjectLiteralExpression(p,!0)]);return i&&e.startOnNewLine(y),y}function me(t){8192&t.transformFlags&&!(16384&i)&&(i|=65536);var r=s;s=void 0;var a=T(15232,66),o=u.createFunctionExpression(void 0,void 0,void 0,void 0,e.visitParameterList(t.parameters,A,n),void 0,he(t));return e.setTextRange(o,t),e.setOriginalNode(o,t),e.setEmitFlags(o,8),S(a,0,0),s=r,o}function _e(t){var r=262144&e.getEmitFlags(t)?T(32662,69):T(32670,65),a=s;s=void 0;var o=e.visitParameterList(t.parameters,A,n),c=he(t),l=32768&i?u.getLocalName(t):t.name;return S(r,98304,0),s=a,u.updateFunctionExpression(t,void 0,t.asteriskToken,l,void 0,o,void 0,c)}function ge(t){var r=s;s=void 0;var a=T(32670,65),o=e.visitParameterList(t.parameters,A,n),c=he(t),l=32768&i?u.getLocalName(t):t.name;return S(a,98304,0),s=r,u.updateFunctionDeclaration(t,void 0,e.visitNodes(t.modifiers,A,e.isModifier),t.asteriskToken,l,void 0,o,void 0,c)}function ye(t,r,a,o){var c=s;s=void 0;var l=o&&e.isClassLike(o)&&!e.isStatic(t)?T(32670,73):T(32670,65),d=e.visitParameterList(t.parameters,A,n),p=he(t);return 32768&i&&!a&&(254===t.kind||211===t.kind)&&(a=u.getGeneratedNameForNode(t)),S(l,98304,0),s=c,e.setOriginalNode(e.setTextRange(u.createFunctionExpression(void 0,t.asteriskToken,a,void 0,d,void 0,p),r),t)}function he(n){var r,i,a,o=!1,s=!1,c=[],l=[],d=n.body;if(f(),e.isBlock(d)&&(a=u.copyStandardPrologue(d.statements,c,!1),a=u.copyCustomPrologue(d.statements,l,a,A,e.isHoistedFunction),a=u.copyCustomPrologue(d.statements,l,a,A,e.isHoistedVariableStatement)),o=ne(l,n)||o,o=ae(l,n,!1)||o,e.isBlock(d))a=u.copyCustomPrologue(d.statements,l,a,A),r=d.statements,e.addRange(l,e.visitNodes(d.statements,A,e.isStatement,a)),!o&&d.multiLine&&(o=!0);else{e.Debug.assert(212===n.kind),r=e.moveRangeEnd(d,-1);var p=n.equalsGreaterThanToken;e.nodeIsSynthesized(p)||e.nodeIsSynthesized(d)||(e.rangeEndIsOnSameLineAsRangeStart(p,d,t)?s=!0:o=!0);var _=e.visitNode(d,A,e.isExpression),g=u.createReturnStatement(_);e.setTextRange(g,d),e.moveSyntheticComments(g,d),e.setEmitFlags(g,1440),l.push(g),i=d}if(u.mergeLexicalEnvironment(c,m()),ce(c,n,!1),oe(c,n),e.some(c)&&(o=!0),l.unshift.apply(l,c),e.isBlock(d)&&e.arrayIsEqualTo(l,d.statements))return d;var y=u.createBlock(e.setTextRange(u.createNodeArray(l),r),o);return e.setTextRange(y,n.body),!o&&s&&e.setEmitFlags(y,1),i&&e.setTokenSourceMapRange(y,19,i),e.setOriginalNode(y,n.body),y}function ve(t,r){if(r)return e.visitEachChild(t,A,n);var a=256&i?T(7104,512):T(6976,128),o=e.visitEachChild(t,A,n);return S(a,0,0),o}function be(t){return e.visitEachChild(t,C,n)}function Ee(t,r){return e.visitEachChild(t,r?C:A,n)}function Te(t,r){return e.isDestructuringAssignment(t)?e.flattenDestructuringAssignment(t,A,n,0,!r):27===t.operatorToken.kind?u.updateBinaryExpression(t,e.visitNode(t.left,C,e.isExpression),t.operatorToken,e.visitNode(t.right,r?C:A,e.isExpression)):e.visitEachChild(t,A,n)}function Se(t,r){if(r)return e.visitEachChild(t,C,n);for(var i,a=0;a<t.elements.length;a++){var o=t.elements[a],s=e.visitNode(o,a<t.elements.length-1?C:A,e.isExpression);(i||s!==o)&&(i||(i=t.elements.slice(0,a)),i.push(s))}var c=i?e.setTextRange(u.createNodeArray(i),t.elements):t.elements;return u.updateCommaListExpression(t,c)}function xe(n){return 1===n.declarationList.declarations.length&&!!n.declarationList.declarations[0].initializer&&!!(33554432&e.getEmitFlags(n.declarationList.declarations[0].initializer))}function Le(t){var r,i=T(0,e.hasSyntacticModifier(t,1)?32:0);if(s&&0===(3&t.declarationList.flags)&&!xe(t)){for(var a=void 0,o=0,c=t.declarationList.declarations;o<c.length;o++){var l=c[o];if(Ze(s,l),l.initializer){var d=void 0;e.isBindingPattern(l.name)?d=e.flattenDestructuringAssignment(l,A,n,0):(d=u.createBinaryExpression(l.name,63,e.visitNode(l.initializer,A,e.isExpression)),e.setTextRange(d,l)),a=e.append(a,d)}}r=a?e.setTextRange(u.createExpressionStatement(u.inlineExpressions(a)),t):void 0}else r=e.visitEachChild(t,A,n);return S(i,0,0),r}function De(t){if(3&t.flags||262144&t.transformFlags){3&t.flags&&$n();var r=e.flatMap(t.declarations,1&t.flags?Ie:Ne),i=u.createVariableDeclarationList(r);return e.setOriginalNode(i,t),e.setTextRange(i,t),e.setCommentRange(i,t),262144&t.transformFlags&&(e.isBindingPattern(t.declarations[0].name)||e.isBindingPattern(e.last(t.declarations).name))&&e.setSourceMapRange(i,Ae(r)),i}return e.visitEachChild(t,A,n)}function Ae(n){for(var t=-1,r=-1,i=0,a=n;i<a.length;i++){var o=a[i];t=-1===t?o.pos:-1===o.pos?t:Math.min(t,o.pos),r=Math.max(r,o.end)}return e.createRange(t,r)}function Ce(e){var n=y.getNodeCheckFlags(e),t=262144&n,r=524288&n,a=0!==(64&i)||t&&r&&0!==(512&i),o=!a&&0===(4096&i)&&(!y.isDeclarationWithCollidingName(e)||r&&!t&&0===(6144&i));return o}function Ie(t){var r=t.name;return e.isBindingPattern(r)?Ne(t):!t.initializer&&Ce(t)?u.updateVariableDeclaration(t,t.name,void 0,void 0,u.createVoidZero()):e.visitEachChild(t,A,n)}function Ne(t){var r,i=T(32,0);return r=e.isBindingPattern(t.name)?e.flattenDestructuringBinding(t,A,n,0,void 0,0!==(32&i)):e.visitEachChild(t,A,n),S(i,0,0),r}function ke(n){s.labels.set(e.idText(n.label),!0)}function Re(n){s.labels.set(e.idText(n.label),!1)}function Oe(n){s&&!s.labels&&(s.labels=new e.Map);var t=e.unwrapInnermostStatementOfLabel(n,s&&ke);return e.isIterationStatement(t,!1)?we(t,n):u.restoreEnclosingLabel(e.visitNode(t,A,e.isStatement,u.liftToBlock),n,s&&Re)}function we(e,n){switch(e.kind){case 238:case 239:return Me(e,n);case 240:return Fe(e,n);case 241:return Be(e,n);case 242:return Ue(e,n)}}function Pe(e,n,t,r,i){var a=T(e,n),o=$e(t,r,a,i);return S(a,0,0),o}function Me(e,n){return Pe(0,1280,e,n)}function Fe(e,n){return Pe(5056,3328,e,n)}function Ge(n){return u.updateForStatement(n,e.visitNode(n.initializer,C,e.isForInitializer),e.visitNode(n.condition,A,e.isExpression),e.visitNode(n.incrementor,C,e.isExpression),e.visitNode(n.statement,A,e.isStatement,u.liftToBlock))}function Be(e,n){return Pe(3008,5376,e,n)}function Ue(e,n){return Pe(3008,5376,e,n,g.downlevelIteration?He:Ke)}function Ve(t,r,i){var a=[],o=t.initializer;if(e.isVariableDeclarationList(o)){3&t.initializer.flags&&$n();var s=e.firstOrUndefined(o.declarations);if(s&&e.isBindingPattern(s.name)){var c=e.flattenDestructuringBinding(s,A,n,0,r),l=e.setTextRange(u.createVariableDeclarationList(c),t.initializer);e.setOriginalNode(l,t.initializer),e.setSourceMapRange(l,e.createRange(c[0].pos,e.last(c).end)),a.push(u.createVariableStatement(void 0,l))}else a.push(e.setTextRange(u.createVariableStatement(void 0,e.setOriginalNode(e.setTextRange(u.createVariableDeclarationList([u.createVariableDeclaration(s?s.name:u.createTempVariable(void 0),void 0,void 0,r)]),e.moveRangePos(o,-1)),o)),e.moveRangeEnd(o,-1)))}else{var d=u.createAssignment(o,r);e.isDestructuringAssignment(d)?a.push(u.createExpressionStatement(Te(d,!0))):(e.setTextRangeEnd(d,o.end),a.push(e.setTextRange(u.createExpressionStatement(e.visitNode(d,A,e.isExpression)),e.moveRangeEnd(o,-1))))}if(i)return je(e.addRange(a,i));var p=e.visitNode(t.statement,A,e.isStatement,u.liftToBlock);return e.isBlock(p)?u.updateBlock(p,e.setTextRange(u.createNodeArray(e.concatenate(a,p.statements)),p.statements)):(a.push(p),je(a))}function je(n){return e.setEmitFlags(u.createBlock(u.createNodeArray(n),!0),432)}function Ke(n,t,r){var i=e.visitNode(n.expression,A,e.isExpression),a=u.createLoopVariable(),o=e.isIdentifier(i)?u.getGeneratedNameForNode(i):u.createTempVariable(void 0);e.setEmitFlags(i,48|e.getEmitFlags(i));var c=e.setTextRange(u.createForStatement(e.setEmitFlags(e.setTextRange(u.createVariableDeclarationList([e.setTextRange(u.createVariableDeclaration(a,void 0,void 0,u.createNumericLiteral(0)),e.moveRangePos(n.expression,-1)),e.setTextRange(u.createVariableDeclaration(o,void 0,void 0,i),n.expression)]),n.expression),2097152),e.setTextRange(u.createLessThan(a,u.createPropertyAccessExpression(o,"length")),n.expression),e.setTextRange(u.createPostfixIncrement(a),n.expression),Ve(n,u.createElementAccessExpression(o,a),r)),n);return e.setEmitFlags(c,256),e.setTextRange(c,n),u.restoreEnclosingLabel(c,t,s&&Re)}function He(n,t,r,i){var a=e.visitNode(n.expression,A,e.isExpression),o=e.isIdentifier(a)?u.getGeneratedNameForNode(a):u.createTempVariable(void 0),c=e.isIdentifier(a)?u.getGeneratedNameForNode(o):u.createTempVariable(void 0),l=u.createUniqueName("e"),p=u.getGeneratedNameForNode(l),f=u.createTempVariable(void 0),m=e.setTextRange(d().createValuesHelper(a),n.expression),g=u.createCallExpression(u.createPropertyAccessExpression(o,"next"),void 0,[]);_(l),_(f);var y=1024&i?u.inlineExpressions([u.createAssignment(l,u.createVoidZero()),m]):m,h=e.setEmitFlags(e.setTextRange(u.createForStatement(e.setEmitFlags(e.setTextRange(u.createVariableDeclarationList([e.setTextRange(u.createVariableDeclaration(o,void 0,void 0,y),n.expression),u.createVariableDeclaration(c,void 0,void 0,g)]),n.expression),2097152),u.createLogicalNot(u.createPropertyAccessExpression(c,"done")),u.createAssignment(c,g),Ve(n,u.createPropertyAccessExpression(c,"value"),r)),n),256);return u.createTryStatement(u.createBlock([u.restoreEnclosingLabel(h,t,s&&Re)]),u.createCatchClause(u.createVariableDeclaration(p),e.setEmitFlags(u.createBlock([u.createExpressionStatement(u.createAssignment(l,u.createObjectLiteralExpression([u.createPropertyAssignment("error",p)])))]),1)),u.createBlock([u.createTryStatement(u.createBlock([e.setEmitFlags(u.createIfStatement(u.createLogicalAnd(u.createLogicalAnd(c,u.createLogicalNot(u.createPropertyAccessExpression(c,"done"))),u.createAssignment(f,u.createPropertyAccessExpression(o,"return"))),u.createExpressionStatement(u.createFunctionCallCall(f,o,[]))),1)]),void 0,e.setEmitFlags(u.createBlock([e.setEmitFlags(u.createIfStatement(l,u.createThrowStatement(u.createPropertyAccessExpression(l,"error"))),1)]),1))]))}function We(t){for(var r=t.properties,a=-1,o=!1,s=0;s<r.length;s++){var c=r[s];if(524288&c.transformFlags&&4&i||(o=160===e.Debug.checkDefined(c.name).kind)){a=s;break}}if(a<0)return e.visitEachChild(t,A,n);var l=u.createTempVariable(_),d=[],p=u.createAssignment(l,e.setEmitFlags(u.createObjectLiteralExpression(e.visitNodes(r,A,e.isObjectLiteralElementLike,0,a),t.multiLine),o?65536:0));return t.multiLine&&e.startOnNewLine(p),d.push(p),vn(d,t,l,a),d.push(t.multiLine?e.startOnNewLine(e.setParent(e.setTextRange(u.cloneNode(l),l),l.parent)):l),u.inlineExpressions(d)}function ze(e){return 0!==(131072&y.getNodeCheckFlags(e))}function Je(n){return e.isForStatement(n)&&!!n.initializer&&ze(n.initializer)}function qe(n){return e.isForStatement(n)&&!!n.condition&&ze(n.condition)}function Xe(n){return e.isForStatement(n)&&!!n.incrementor&&ze(n.incrementor)}function Ye(e){return Qe(e)||Je(e)}function Qe(e){return 0!==(65536&y.getNodeCheckFlags(e))}function Ze(n,t){function r(t){if(79===t.kind)n.hoistedLocalVariables.push(t);else for(var i=0,a=t.elements;i<a.length;i++){var o=a[i];e.isOmittedExpression(o)||r(o.name)}}n.hoistedLocalVariables||(n.hoistedLocalVariables=[]),r(t.name)}function $e(t,r,i,a){if(!Ye(t)){var o=void 0;s&&(o=s.allowedNonLabeledJumps,s.allowedNonLabeledJumps=6);var c=a?a(t,r,void 0,i):u.restoreEnclosingLabel(e.isForStatement(t)?Ge(t):e.visitEachChild(t,A,n),r,s&&Re);return s&&(s.allowedNonLabeledJumps=o),c}var l=sn(t),d=[],p=s;s=l;var f,m=Je(t)?un(t,l):void 0,_=Qe(t)?dn(t,l,p):void 0;if(s=p,m&&d.push(m.functionDeclaration),_&&d.push(_.functionDeclaration),cn(d,l,p),m&&d.push(mn(m.functionName,m.containsYield)),_)if(a)f=a(t,r,_.part,i);else{var g=en(t,m,u.createBlock(_.part,!0));f=u.restoreEnclosingLabel(g,r,s&&Re)}else{var y=en(t,m,e.visitNode(t.statement,A,e.isStatement,u.liftToBlock));f=u.restoreEnclosingLabel(y,r,s&&Re)}return d.push(f),d}function en(n,t,r){switch(n.kind){case 240:return nn(n,t,r);case 241:return rn(n,r);case 242:return tn(n,r);case 238:return an(n,r);case 239:return on(n,r);default:return e.Debug.failBadSyntaxKind(n,"IterationStatement expected")}}function nn(n,t,r){var i=n.condition&&ze(n.condition),a=i||n.incrementor&&ze(n.incrementor);return u.updateForStatement(n,e.visitNode(t?t.part:n.initializer,C,e.isForInitializer),e.visitNode(i?void 0:n.condition,A,e.isExpression),e.visitNode(a?void 0:n.incrementor,C,e.isExpression),r)}function tn(n,t){return u.updateForOfStatement(n,void 0,e.visitNode(n.initializer,A,e.isForInitializer),e.visitNode(n.expression,A,e.isExpression),t)}function rn(n,t){return u.updateForInStatement(n,e.visitNode(n.initializer,A,e.isForInitializer),e.visitNode(n.expression,A,e.isExpression),t)}function an(n,t){return u.updateDoStatement(n,t,e.visitNode(n.expression,A,e.isExpression))}function on(n,t){return u.updateWhileStatement(n,e.visitNode(n.expression,A,e.isExpression),t)}function sn(n){var t;switch(n.kind){case 240:case 241:case 242:var r=n.initializer;r&&253===r.kind&&(t=r);break}var i=[],a=[];if(t&&3&e.getCombinedNodeFlags(t))for(var o=Je(n),c=0,l=t.declarations;c<l.length;c++){var u=l[c];hn(n,u,i,a,o)}var d={loopParameters:i,loopOutParameters:a};return s&&(s.argumentsName&&(d.argumentsName=s.argumentsName),s.thisName&&(d.thisName=s.thisName),s.hoistedLocalVariables&&(d.hoistedLocalVariables=s.hoistedLocalVariables)),d}function cn(e,n,t){var r;if(n.argumentsName&&(t?t.argumentsName=n.argumentsName:(r||(r=[])).push(u.createVariableDeclaration(n.argumentsName,void 0,void 0,u.createIdentifier("arguments")))),n.thisName&&(t?t.thisName=n.thisName:(r||(r=[])).push(u.createVariableDeclaration(n.thisName,void 0,void 0,u.createIdentifier("this")))),n.hoistedLocalVariables)if(t)t.hoistedLocalVariables=n.hoistedLocalVariables;else{r||(r=[]);for(var i=0,a=n.hoistedLocalVariables;i<a.length;i++){var o=a[i];r.push(u.createVariableDeclaration(o))}}if(n.loopOutParameters.length){r||(r=[]);for(var s=0,c=n.loopOutParameters;s<c.length;s++){var l=c[s];r.push(u.createVariableDeclaration(l.outParamName))}}n.conditionVariable&&(r||(r=[]),r.push(u.createVariableDeclaration(n.conditionVariable,void 0,void 0,u.createFalse()))),r&&e.push(u.createVariableStatement(void 0,u.createVariableDeclarationList(r)))}function ln(e){return u.createVariableDeclaration(e.originalName,void 0,void 0,e.outParamName)}function un(n,t){var r=u.createUniqueName("_loop_init"),a=0!==(524288&n.initializer.transformFlags),o=0;t.containsLexicalThis&&(o|=8),a&&4&i&&(o|=262144);var s=[];s.push(u.createVariableStatement(void 0,n.initializer)),fn(t.loopOutParameters,2,1,s);var c=u.createVariableStatement(void 0,e.setEmitFlags(u.createVariableDeclarationList([u.createVariableDeclaration(r,void 0,void 0,e.setEmitFlags(u.createFunctionExpression(void 0,a?u.createToken(41):void 0,void 0,void 0,void 0,void 0,e.visitNode(u.createBlock(s,!0),A,e.isBlock)),o))]),2097152)),l=u.createVariableDeclarationList(e.map(t.loopOutParameters,ln));return{functionName:r,containsYield:a,functionDeclaration:c,part:l}}function dn(n,t,r){var a=u.createUniqueName("_loop");p();var o=e.visitNode(n.statement,A,e.isStatement,u.liftToBlock),s=m(),c=[];(qe(n)||Xe(n))&&(t.conditionVariable=u.createUniqueName("inc"),n.incrementor?c.push(u.createIfStatement(t.conditionVariable,u.createExpressionStatement(e.visitNode(n.incrementor,A,e.isExpression)),u.createExpressionStatement(u.createAssignment(t.conditionVariable,u.createTrue())))):c.push(u.createIfStatement(u.createLogicalNot(t.conditionVariable),u.createExpressionStatement(u.createAssignment(t.conditionVariable,u.createTrue())))),qe(n)&&c.push(u.createIfStatement(u.createPrefixUnaryExpression(53,e.visitNode(n.condition,A,e.isExpression)),e.visitNode(u.createBreakStatement(),A,e.isStatement)))),e.isBlock(o)?e.addRange(c,o.statements):c.push(o),fn(t.loopOutParameters,1,1,c),e.insertStatementsAfterStandardPrologue(c,s);var l=u.createBlock(c,!0);e.isBlock(o)&&e.setOriginalNode(l,o);var d=0!==(524288&n.statement.transformFlags),f=524288;t.containsLexicalThis&&(f|=8),d&&0!==(4&i)&&(f|=262144);var _=u.createVariableStatement(void 0,e.setEmitFlags(u.createVariableDeclarationList([u.createVariableDeclaration(a,void 0,void 0,e.setEmitFlags(u.createFunctionExpression(void 0,d?u.createToken(41):void 0,void 0,void 0,t.loopParameters,void 0,l),f))]),2097152)),g=_n(a,t,r,d);return{functionName:a,containsYield:d,functionDeclaration:_,part:g}}function pn(e,n){var t=0===n?e.outParamName:e.originalName,r=0===n?e.originalName:e.outParamName;return u.createBinaryExpression(r,63,t)}function fn(e,n,t,r){for(var i=0,a=e;i<a.length;i++){var o=a[i];o.flags&n&&r.push(u.createExpressionStatement(pn(o,t)))}}function mn(n,t){var r=u.createCallExpression(n,void 0,[]),i=t?u.createYieldExpression(u.createToken(41),e.setEmitFlags(r,8388608)):r;return u.createExpressionStatement(i)}function _n(n,t,r,i){var a=[],o=!(-5&t.nonLocalJumps)&&!t.labeledNonLocalBreaks&&!t.labeledNonLocalContinues,s=u.createCallExpression(n,void 0,e.map(t.loopParameters,function(e){return e.name})),c=i?u.createYieldExpression(u.createToken(41),e.setEmitFlags(s,8388608)):s;if(o)a.push(u.createExpressionStatement(c)),fn(t.loopOutParameters,1,0,a);else{var l=u.createUniqueName("state"),d=u.createVariableStatement(void 0,u.createVariableDeclarationList([u.createVariableDeclaration(l,void 0,void 0,c)]));if(a.push(d),fn(t.loopOutParameters,1,0,a),8&t.nonLocalJumps){var p=void 0;r?(r.nonLocalJumps|=8,p=u.createReturnStatement(l)):p=u.createReturnStatement(u.createPropertyAccessExpression(l,"value")),a.push(u.createIfStatement(u.createTypeCheck(l,"object"),p))}if(2&t.nonLocalJumps&&a.push(u.createIfStatement(u.createStrictEquality(l,u.createStringLiteral("break")),u.createBreakStatement())),t.labeledNonLocalBreaks||t.labeledNonLocalContinues){var f=[];yn(t.labeledNonLocalBreaks,!0,l,r,f),yn(t.labeledNonLocalContinues,!1,l,r,f),a.push(u.createSwitchStatement(l,u.createCaseBlock(f)))}}return a}function gn(n,t,r,i){t?(n.labeledNonLocalBreaks||(n.labeledNonLocalBreaks=new e.Map),n.labeledNonLocalBreaks.set(r,i)):(n.labeledNonLocalContinues||(n.labeledNonLocalContinues=new e.Map),n.labeledNonLocalContinues.set(r,i))}function yn(e,n,t,r,i){e&&e.forEach(function(e,a){var o=[];if(!r||r.labels&&r.labels.get(a)){var s=u.createIdentifier(a);o.push(n?u.createBreakStatement(s):u.createContinueStatement(s))}else gn(r,n,a,e),o.push(u.createReturnStatement(t));i.push(u.createCaseClause(u.createStringLiteral(e),o))})}function hn(n,t,r,i,a){var o=t.name;if(e.isBindingPattern(o))for(var s=0,c=o.elements;s<c.length;s++){var l=c[s];e.isOmittedExpression(l)||hn(n,l,r,i,a)}else{r.push(u.createParameterDeclaration(void 0,void 0,void 0,o));var d=y.getNodeCheckFlags(t);if(4194304&d||a){var p=u.createUniqueName("out_"+e.idText(o)),f=0;4194304&d&&(f|=1),e.isForStatement(n)&&n.initializer&&y.isBindingCapturedByNode(n.initializer,t)&&(f|=2),i.push({flags:f,originalName:o,outParamName:p})}}}function vn(n,t,r,i){for(var a=t.properties,o=a.length,s=i;s<o;s++){var c=a[s];switch(c.kind){case 170:case 171:var l=e.getAllAccessorDeclarations(t.properties,c);c===l.firstAccessor&&n.push(fe(r,l,t,!!t.multiLine));break;case 167:n.push(Tn(c,r,t,t.multiLine));break;case 291:n.push(bn(c,r,t.multiLine));break;case 292:n.push(En(c,r,t.multiLine));break;default:e.Debug.failBadSyntaxKind(t);break}}}function bn(n,t,r){var i=u.createAssignment(e.createMemberAccessForPropertyName(u,t,e.visitNode(n.name,A,e.isPropertyName)),e.visitNode(n.initializer,A,e.isExpression));return e.setTextRange(i,n),r&&e.startOnNewLine(i),i}function En(n,t,r){var i=u.createAssignment(e.createMemberAccessForPropertyName(u,t,e.visitNode(n.name,A,e.isPropertyName)),u.cloneNode(n.name));return e.setTextRange(i,n),r&&e.startOnNewLine(i),i}function Tn(n,t,r,i){var a=u.createAssignment(e.createMemberAccessForPropertyName(u,t,e.visitNode(n.name,A,e.isPropertyName)),ye(n,n,void 0,r));return e.setTextRange(a,n),i&&e.startOnNewLine(a),a}function Sn(t){var r,i=T(7104,0);if(e.Debug.assert(!!t.variableDeclaration,"Catch clause variable should always be present when downleveling ES2015."),e.isBindingPattern(t.variableDeclaration.name)){var a=u.createTempVariable(void 0),o=u.createVariableDeclaration(a);e.setTextRange(o,t.variableDeclaration);var s=e.flattenDestructuringBinding(t.variableDeclaration,A,n,0,a),c=u.createVariableDeclarationList(s);e.setTextRange(c,t.variableDeclaration);var l=u.createVariableStatement(void 0,c);r=u.updateCatchClause(t,o,xn(t.block,l))}else r=e.visitEachChild(t,A,n);return S(i,0,0),r}function xn(n,t){var r=e.visitNodes(n.statements,A,e.isStatement);return u.updateBlock(n,a([t],r,!0))}function Ln(n){e.Debug.assert(!e.isComputedPropertyName(n.name));var t=ye(n,e.moveRangePos(n,-1),void 0,void 0);return e.setEmitFlags(t,512|e.getEmitFlags(t)),e.setTextRange(u.createPropertyAssignment(n.name,t),n)}function Dn(t){e.Debug.assert(!e.isComputedPropertyName(t.name));var r=s;s=void 0;var i,a=T(32670,65),o=e.visitParameterList(t.parameters,A,n),c=he(t);return i=170===t.kind?u.updateGetAccessorDeclaration(t,t.decorators,t.modifiers,t.name,o,t.type,c):u.updateSetAccessorDeclaration(t,t.decorators,t.modifiers,t.name,o,c),S(a,98304,0),s=r,i}function An(n){return e.setTextRange(u.createPropertyAssignment(n.name,B(u.cloneNode(n.name))),n)}function Cn(t){return e.visitEachChild(t,A,n)}function In(t){return e.visitEachChild(t,A,n)}function Nn(t){return e.some(t.elements,e.isSpreadElement)?Mn(t.elements,!1,!!t.multiLine,!!t.elements.hasTrailingComma):e.visitEachChild(t,A,n)}function kn(n){if(33554432&e.getEmitFlags(n))return Rn(n);var t=e.skipOuterExpressions(n.expression);return 106===t.kind||e.isSuperProperty(t)||e.some(n.arguments,e.isSpreadElement)?wn(n,!0):u.updateCallExpression(n,e.visitNode(n.expression,N,e.isExpression),void 0,e.visitNodes(n.arguments,A,e.isExpression))}function Rn(n){var t=e.cast(e.cast(e.skipOuterExpressions(n.expression),e.isArrowFunction).body,e.isBlock),r=function(n){return e.isVariableStatement(n)&&!!e.first(n.declarationList.declarations).initializer},i=s;s=void 0;var a=e.visitNodes(t.statements,I,e.isStatement);s=i;var o=e.filter(a,r),c=e.filter(a,function(e){return!r(e)}),l=e.cast(e.first(o),e.isVariableStatement),d=l.declarationList.declarations[0],p=e.skipOuterExpressions(d.initializer),f=e.tryCast(p,e.isAssignmentExpression);!f&&e.isBinaryExpression(p)&&27===p.operatorToken.kind&&(f=e.tryCast(p.left,e.isAssignmentExpression));var m=e.cast(f?e.skipOuterExpressions(f.right):p,e.isCallExpression),_=e.cast(e.skipOuterExpressions(m.expression),e.isFunctionExpression),g=_.body.statements,y=0,h=-1,v=[];if(f){var b=e.tryCast(g[y],e.isExpressionStatement);b&&(v.push(b),y++),v.push(g[y]),y++,v.push(u.createExpressionStatement(u.createAssignment(f.left,e.cast(d.name,e.isIdentifier))))}while(!e.isReturnStatement(e.elementAt(g,h)))h--;return e.addRange(v,g,y,h),h<-1&&e.addRange(v,g,h+1),e.addRange(v,c),e.addRange(v,o,1),u.restoreOuterExpressions(n.expression,u.restoreOuterExpressions(d.initializer,u.restoreOuterExpressions(f&&f.right,u.updateCallExpression(m,u.restoreOuterExpressions(m.expression,u.updateFunctionExpression(_,void 0,void 0,void 0,void 0,_.parameters,void 0,u.updateBlock(_.body,v))),void 0,m.arguments))))}function On(e){return wn(e,!1)}function wn(t,r){if(16384&t.transformFlags||106===t.expression.kind||e.isSuperProperty(e.skipOuterExpressions(t.expression))){var i=u.createCallBinding(t.expression,_),a=i.target,o=i.thisArg;106===t.expression.kind&&e.setEmitFlags(o,4);var s=void 0;if(s=16384&t.transformFlags?u.createFunctionApplyCall(e.visitNode(a,N,e.isExpression),106===t.expression.kind?o:e.visitNode(o,A,e.isExpression),Mn(t.arguments,!0,!1,!1)):e.setTextRange(u.createFunctionCallCall(e.visitNode(a,N,e.isExpression),106===t.expression.kind?o:e.visitNode(o,A,e.isExpression),e.visitNodes(t.arguments,A,e.isExpression)),t),106===t.expression.kind){var c=u.createLogicalOr(s,Q());s=r?u.createAssignment(u.createUniqueName("_this",48),c):c}return e.setOriginalNode(s,t)}return e.visitEachChild(t,A,n)}function Pn(t){if(e.some(t.arguments,e.isSpreadElement)){var r=u.createCallBinding(u.createPropertyAccessExpression(t.expression,"bind"),_),i=r.target,o=r.thisArg;return u.createNewExpression(u.createFunctionApplyCall(e.visitNode(i,A,e.isExpression),o,Mn(u.createNodeArray(a([u.createVoidZero()],t.arguments,!0)),!0,!1,!1)),void 0,[])}return e.visitEachChild(t,A,n)}function Mn(n,t,r,i){var a=n.length,o=e.flatten(e.spanMap(n,Fn,function(e,n,t,o){return n(e,r,i&&o===a)}));if(1===o.length){var s=o[0];if(t&&!g.downlevelIteration||e.isPackedArrayLiteral(s.expression)||e.isCallToHelper(s.expression,"___spreadArray"))return s.expression}for(var c=d(),l=0!==o[0].kind,p=l?u.createArrayLiteralExpression():o[0].expression,f=l?0:1;f<o.length;f++){var m=o[f];p=c.createSpreadArrayHelper(p,m.expression,1===m.kind&&!t)}return p}function Fn(n){return e.isSpreadElement(n)?Gn:Un}function Gn(n){return e.map(n,Bn)}function Bn(n){var t=e.visitNode(n.expression,A,e.isExpression),r=e.isCallToHelper(t,"___read"),i=r||e.isPackedArrayLiteral(t)?2:1;return!g.downlevelIteration||1!==i||e.isArrayLiteralExpression(t)||r||(t=d().createReadHelper(t,void 0),i=2),c(i,t)}function Un(n,t,r){var i=u.createArrayLiteralExpression(e.visitNodes(u.createNodeArray(n,r),A,e.isExpression),t);return c(0,i)}function Vn(n){return e.visitNode(n.expression,A,e.isExpression)}function jn(n){return e.setTextRange(u.createStringLiteral(n.text),n)}function Kn(n){return n.hasExtendedUnicodeEscape?e.setTextRange(u.createStringLiteral(n.text),n):n}function Hn(n){return 384&n.numericLiteralFlags?e.setTextRange(u.createNumericLiteral(n.text),n):n}function Wn(r){return e.processTaggedTemplateExpression(n,r,A,t,b,e.ProcessLevel.All)}function zn(n){var t=[];qn(t,n),Xn(t,n);var r=e.reduceLeft(t,u.createAdd);return e.nodeIsSynthesized(r)&&e.setTextRange(r,n),r}function Jn(n){return e.Debug.assert(0!==n.templateSpans.length),0!==n.head.text.length||0===n.templateSpans[0].literal.text.length}function qn(e,n){Jn(n)&&e.push(u.createStringLiteral(n.head.text))}function Xn(n,t){for(var r=0,i=t.templateSpans;r<i.length;r++){var a=i[r];n.push(e.visitNode(a.expression,A,e.isExpression)),0!==a.literal.text.length&&n.push(u.createStringLiteral(a.literal.text))}}function Yn(e){return 8&i&&!e?u.createPropertyAccessExpression(u.createUniqueName("_super",48),"prototype"):u.createUniqueName("_super",48)}function Qn(e){return 103===e.keywordToken&&"target"===e.name.escapedText?(i|=32768,u.createUniqueName("_newTarget",48)):e}function Zn(n,t,r){if(1&l&&e.isFunctionLike(t)){var i=T(32670,8&e.getEmitFlags(t)?81:65);return v(n,t,r),void S(i,0,0)}v(n,t,r)}function $n(){0===(2&l)&&(l|=2,n.enableSubstitution(79))}function et(){0===(1&l)&&(l|=1,n.enableSubstitution(108),n.enableEmitNotification(169),n.enableEmitNotification(167),n.enableEmitNotification(170),n.enableEmitNotification(171),n.enableEmitNotification(212),n.enableEmitNotification(211),n.enableEmitNotification(254))}function nt(n,t){return t=h(n,t),1===n?it(t):e.isIdentifier(t)?tt(t):t}function tt(n){if(2&l&&!e.isInternalName(n)){var t=e.getParseTreeNode(n,e.isIdentifier);if(t&&rt(t))return e.setTextRange(u.getGeneratedNameForNode(t),n)}return n}function rt(e){switch(e.parent.kind){case 201:case 255:case 258:case 252:return e.parent.name===e&&y.isDeclarationWithCollidingName(e.parent)}return!1}function it(e){switch(e.kind){case 79:return at(e);case 108:return st(e)}return e}function at(n){if(2&l&&!e.isInternalName(n)){var t=y.getReferencedDeclarationWithCollidingName(n);if(t&&(!e.isClassLike(t)||!ot(t,n)))return e.setTextRange(u.getGeneratedNameForNode(e.getNameOfDeclaration(t)),n)}return n}function ot(n,t){var r=e.getParseTreeNode(t);if(!r||r===n||r.end<=n.pos||r.pos>=n.end)return!1;var i=e.getEnclosingBlockScopeContainer(n);while(r){if(r===i||r===n)return!1;if(e.isClassElement(r)&&r.parent===n)return!0;r=r.parent}return!1}function st(n){return 1&l&&16&i?e.setTextRange(u.createUniqueName("_this",48),n):n}function ct(n,t){return e.isStatic(t)?u.getInternalName(n):u.createPropertyAccessExpression(u.getInternalName(n),"prototype")}function lt(n,t){if(!n||!t)return!1;if(e.some(n.parameters))return!1;var r=e.firstOrUndefined(n.body.statements);if(!r||!e.nodeIsSynthesized(r)||236!==r.kind)return!1;var i=r.expression;if(!e.nodeIsSynthesized(i)||206!==i.kind)return!1;var a=i.expression;if(!e.nodeIsSynthesized(a)||106!==a.kind)return!1;var o=e.singleOrUndefined(i.arguments);if(!o||!e.nodeIsSynthesized(o)||223!==o.kind)return!1;var s=o.expression;return e.isIdentifier(s)&&"arguments"===s.escapedText}}(function(e){e[e["CapturedThis"]=1]="CapturedThis",e[e["BlockScopedBindings"]=2]="BlockScopedBindings"})(n||(n={})),function(e){e[e["Body"]=1]="Body",e[e["Initializer"]=2]="Initializer"}(t||(t={})),function(e){e[e["ToOriginal"]=0]="ToOriginal",e[e["ToOutParameter"]=1]="ToOutParameter"}(r||(r={})),function(e){e[e["Break"]=2]="Break",e[e["Continue"]=4]="Continue",e[e["Return"]=8]="Return"}(i||(i={})),function(e){e[e["None"]=0]="None",e[e["Function"]=1]="Function",e[e["ArrowFunction"]=2]="ArrowFunction",e[e["AsyncFunctionBody"]=4]="AsyncFunctionBody",e[e["NonStaticClassElement"]=8]="NonStaticClassElement",e[e["CapturesThis"]=16]="CapturesThis",e[e["ExportedVariableStatement"]=32]="ExportedVariableStatement",e[e["TopLevel"]=64]="TopLevel",e[e["Block"]=128]="Block",e[e["IterationStatement"]=256]="IterationStatement",e[e["IterationStatementBlock"]=512]="IterationStatementBlock",e[e["IterationContainer"]=1024]="IterationContainer",e[e["ForStatement"]=2048]="ForStatement",e[e["ForInOrForOfStatement"]=4096]="ForInOrForOfStatement",e[e["ConstructorWithCapturedSuper"]=8192]="ConstructorWithCapturedSuper",e[e["StaticInitializer"]=16384]="StaticInitializer",e[e["AncestorFactsMask"]=32767]="AncestorFactsMask",e[e["BlockScopeIncludes"]=0]="BlockScopeIncludes",e[e["BlockScopeExcludes"]=7104]="BlockScopeExcludes",e[e["SourceFileIncludes"]=64]="SourceFileIncludes",e[e["SourceFileExcludes"]=8064]="SourceFileExcludes",e[e["FunctionIncludes"]=65]="FunctionIncludes",e[e["FunctionExcludes"]=32670]="FunctionExcludes",e[e["AsyncFunctionBodyIncludes"]=69]="AsyncFunctionBodyIncludes",e[e["AsyncFunctionBodyExcludes"]=32662]="AsyncFunctionBodyExcludes",e[e["ArrowFunctionIncludes"]=66]="ArrowFunctionIncludes",e[e["ArrowFunctionExcludes"]=15232]="ArrowFunctionExcludes",e[e["ConstructorIncludes"]=73]="ConstructorIncludes",e[e["ConstructorExcludes"]=32662]="ConstructorExcludes",e[e["DoOrWhileStatementIncludes"]=1280]="DoOrWhileStatementIncludes",e[e["DoOrWhileStatementExcludes"]=0]="DoOrWhileStatementExcludes",e[e["ForStatementIncludes"]=3328]="ForStatementIncludes",e[e["ForStatementExcludes"]=5056]="ForStatementExcludes",e[e["ForInOrForOfStatementIncludes"]=5376]="ForInOrForOfStatementIncludes",e[e["ForInOrForOfStatementExcludes"]=3008]="ForInOrForOfStatementExcludes",e[e["BlockIncludes"]=128]="BlockIncludes",e[e["BlockExcludes"]=6976]="BlockExcludes",e[e["IterationStatementBlockIncludes"]=512]="IterationStatementBlockIncludes",e[e["IterationStatementBlockExcludes"]=7104]="IterationStatementBlockExcludes",e[e["StaticInitializerIncludes"]=16449]="StaticInitializerIncludes",e[e["StaticInitializerExcludes"]=32670]="StaticInitializerExcludes",e[e["NewTarget"]=32768]="NewTarget",e[e["CapturedLexicalThis"]=65536]="CapturedLexicalThis",e[e["SubtreeFactsMask"]=-32768]="SubtreeFactsMask",e[e["ArrowFunctionSubtreeExcludes"]=0]="ArrowFunctionSubtreeExcludes",e[e["FunctionSubtreeExcludes"]=98304]="FunctionSubtreeExcludes"}(o||(o={})),function(e){e[e["None"]=0]="None",e[e["UnpackedSpread"]=1]="UnpackedSpread",e[e["PackedSpread"]=2]="PackedSpread"}(s||(s={})),e.transformES2015=l}(d||(d={})),function(e){function n(n){var t,r,i=n.factory,a=n.getCompilerOptions();1!==a.jsx&&3!==a.jsx||(t=n.onEmitNode,n.onEmitNode=c,n.enableEmitNotification(278),n.enableEmitNotification(279),n.enableEmitNotification(277),r=[]);var o=n.onSubstituteNode;return n.onSubstituteNode=l,n.enableSubstitution(204),n.enableSubstitution(291),e.chainBundle(n,s);function s(e){return e}function c(n,i,a){switch(i.kind){case 278:case 279:case 277:var o=i.tagName;r[e.getOriginalNodeId(o)]=!0;break}t(n,i,a)}function l(n,t){return t.id&&r&&r[t.id]?o(n,t):(t=o(n,t),e.isPropertyAccessExpression(t)?u(t):e.isPropertyAssignment(t)?d(t):t)}function u(n){if(e.isPrivateIdentifier(n.name))return n;var t=p(n.name);return t?e.setTextRange(i.createElementAccessExpression(n.expression,t),n):n}function d(n){var t=e.isIdentifier(n.name)&&p(n.name);return t?i.updatePropertyAssignment(n,t,n.initializer):n}function p(n){var t=n.originalKeywordKind||(e.nodeIsSynthesized(n)?e.stringToToken(e.idText(n)):void 0);if(void 0!==t&&t>=81&&t<=116)return e.setTextRange(i.createStringLiteralFromNode(n),n)}}e.transformES5=n}(d||(d={})),function(e){var n,t,r,i,o;function s(e){switch(e){case 2:return"return";case 3:return"break";case 4:return"yield";case 5:return"yield*";case 7:return"endfinally";default:return}}function c(n){var t,r,i,o,c,l,u,d,p,f,m=n.factory,_=n.getEmitHelperFactory,g=n.resumeLexicalEnvironment,y=n.endLexicalEnvironment,h=n.hoistFunctionDeclaration,v=n.hoistVariableDeclaration,b=n.getCompilerOptions(),E=e.getEmitScriptTarget(b),T=n.getEmitResolver(),S=n.onSubstituteNode;n.onSubstituteNode=je;var x,L,D,A,C,I,N,k,R,O,w,P,M=1,F=0,G=0;return e.chainBundle(n,B);function B(t){if(t.isDeclarationFile||0===(1024&t.transformFlags))return t;var r=e.visitEachChild(t,U,n);return e.addEmitHelpers(r,n.readEmitHelpers()),r}function U(t){var r=t.transformFlags;return o?V(t):i?j(t):e.isFunctionLikeDeclaration(t)&&t.asteriskToken?H(t):1024&r?e.visitEachChild(t,U,n):t}function V(e){switch(e.kind){case 238:return be(e);case 239:return Te(e);case 247:return Pe(e);case 248:return Fe(e);default:return j(e)}}function j(t){switch(t.kind){case 254:return W(t);case 211:return z(t);case 170:case 171:return J(t);case 235:return X(t);case 240:return xe(t);case 241:return De(t);case 244:return Ne(t);case 243:return Ce(t);case 245:return Re(t);default:return 524288&t.transformFlags?K(t):2098176&t.transformFlags?e.visitEachChild(t,U,n):t}}function K(t){switch(t.kind){case 219:return Y(t);case 346:return ee(t);case 220:return te(t);case 222:return re(t);case 202:return ie(t);case 203:return oe(t);case 205:return se(t);case 206:return ce(t);case 207:return le(t);default:return e.visitEachChild(t,U,n)}}function H(n){switch(n.kind){case 254:return W(n);case 211:return z(n);default:return e.Debug.failBadSyntaxKind(n)}}function W(t){if(t.asteriskToken)t=e.setOriginalNode(e.setTextRange(m.createFunctionDeclaration(void 0,t.modifiers,void 0,t.name,void 0,e.visitParameterList(t.parameters,U,n),void 0,q(t.body)),t),t);else{var r=i,a=o;i=!1,o=!1,t=e.visitEachChild(t,U,n),i=r,o=a}return i?void h(t):t}function z(t){if(t.asteriskToken)t=e.setOriginalNode(e.setTextRange(m.createFunctionExpression(void 0,void 0,t.name,void 0,e.visitParameterList(t.parameters,U,n),void 0,q(t.body)),t),t);else{var r=i,a=o;i=!1,o=!1,t=e.visitEachChild(t,U,n),i=r,o=a}return t}function J(t){var r=i,a=o;return i=!1,o=!1,t=e.visitEachChild(t,U,n),i=r,o=a,t}function q(n){var t=[],r=i,a=o,s=c,_=l,h=u,v=d,b=p,E=f,T=M,S=x,C=L,I=D,N=A;i=!0,o=!1,c=void 0,l=void 0,u=void 0,d=void 0,p=void 0,f=void 0,M=1,x=void 0,L=void 0,D=void 0,A=m.createTempVariable(void 0),g();var k=m.copyPrologue(n.statements,t,!1,U);ue(n.statements,k);var R=Gn();return e.insertStatementsAfterStandardPrologue(t,y()),t.push(m.createReturnStatement(R)),i=r,o=a,c=s,l=_,u=h,d=v,p=b,f=E,M=T,x=S,L=C,D=I,A=N,e.setTextRange(m.createBlock(t,n.multiLine),n)}function X(n){if(524288&n.transformFlags)ge(n.declarationList);else{if(1048576&e.getEmitFlags(n))return n;for(var t=0,r=n.declarationList.declarations;t<r.length;t++){var i=r[t];v(i.name)}var a=e.getInitializedVariables(n.declarationList);if(0!==a.length)return e.setSourceMapRange(m.createExpressionStatement(m.inlineExpressions(e.map(a,ye))),n)}}function Y(n){var t=e.getExpressionAssociativity(n);switch(t){case 0:return Z(n);case 1:return Q(n);default:return e.Debug.assertNever(t)}}function Q(t){var r=t.left,i=t.right;if(Ue(i)){var a=void 0;switch(r.kind){case 204:a=m.updatePropertyAccessExpression(r,We(e.visitNode(r.expression,U,e.isLeftHandSideExpression)),r.name);break;case 205:a=m.updateElementAccessExpression(r,We(e.visitNode(r.expression,U,e.isLeftHandSideExpression)),We(e.visitNode(r.argumentExpression,U,e.isExpression)));break;default:a=e.visitNode(r,U,e.isExpression);break}var o=t.operatorToken.kind;return e.isCompoundAssignment(o)?e.setTextRange(m.createAssignment(a,e.setTextRange(m.createBinaryExpression(We(a),e.getNonAssignmentOperatorForCompoundAssignment(o),e.visitNode(i,U,e.isExpression)),t)),t):m.updateBinaryExpression(t,a,t.operatorToken,e.visitNode(i,U,e.isExpression))}return e.visitEachChild(t,U,n)}function Z(t){return Ue(t.right)?e.isLogicalOperator(t.operatorToken.kind)?ne(t):27===t.operatorToken.kind?$(t):m.updateBinaryExpression(t,We(e.visitNode(t.left,U,e.isExpression)),t.operatorToken,e.visitNode(t.right,U,e.isExpression)):e.visitEachChild(t,U,n)}function $(n){var t=[];return r(n.left),r(n.right),m.inlineExpressions(t);function r(n){e.isBinaryExpression(n)&&27===n.operatorToken.kind?(r(n.left),r(n.right)):(Ue(n)&&t.length>0&&(Fn(1,[m.createExpressionStatement(m.inlineExpressions(t))]),t=[]),t.push(e.visitNode(n,U,e.isExpression)))}}function ee(n){for(var t=[],r=0,i=n.elements;r<i.length;r++){var a=i[r];e.isBinaryExpression(a)&&27===a.operatorToken.kind?t.push($(a)):(Ue(a)&&t.length>0&&(Fn(1,[m.createExpressionStatement(m.inlineExpressions(t))]),t=[]),t.push(e.visitNode(a,U,e.isExpression)))}return m.inlineExpressions(t)}function ne(n){var t=Je(),r=ze();return Cn(r,e.visitNode(n.left,U,e.isExpression),n.left),55===n.operatorToken.kind?kn(t,r,n.left):Nn(t,r,n.left),Cn(r,e.visitNode(n.right,U,e.isExpression),n.right),qe(t),r}function te(t){if(Ue(t.whenTrue)||Ue(t.whenFalse)){var r=Je(),i=Je(),a=ze();return kn(r,e.visitNode(t.condition,U,e.isExpression),t.condition),Cn(a,e.visitNode(t.whenTrue,U,e.isExpression),t.whenTrue),In(i),qe(r),Cn(a,e.visitNode(t.whenFalse,U,e.isExpression),t.whenFalse),qe(i),a}return e.visitEachChild(t,U,n)}function re(n){var t=Je(),r=e.visitNode(n.expression,U,e.isExpression);if(n.asteriskToken){var i=0===(8388608&e.getEmitFlags(n.expression))?e.setTextRange(_().createValuesHelper(r),n):r;Rn(i,n)}else On(r,n);return qe(t),Ln(n)}function ie(e){return ae(e.elements,void 0,void 0,e.multiLine)}function ae(n,t,r,i){var o,s=Ve(n);if(s>0){o=ze();var c=e.visitNodes(n,U,e.isExpression,0,s);Cn(o,m.createArrayLiteralExpression(t?a([t],c,!0):c)),t=void 0}var l=e.reduceLeft(n,u,[],s);return o?m.createArrayConcatCall(o,[m.createArrayLiteralExpression(l,i)]):e.setTextRange(m.createArrayLiteralExpression(t?a([t],l,!0):l,i),r);function u(n,r){if(Ue(r)&&n.length>0){var s=void 0!==o;o||(o=ze()),Cn(o,s?m.createArrayConcatCall(o,[m.createArrayLiteralExpression(n,i)]):m.createArrayLiteralExpression(t?a([t],n,!0):n,i)),t=void 0,n=[]}return n.push(e.visitNode(r,U,e.isExpression)),n}}function oe(n){var t=n.properties,r=n.multiLine,i=Ve(t),a=ze();Cn(a,m.createObjectLiteralExpression(e.visitNodes(t,U,e.isObjectLiteralElementLike,0,i),r));var o=e.reduceLeft(t,s,[],i);return o.push(r?e.startOnNewLine(e.setParent(e.setTextRange(m.cloneNode(a),a),a.parent)):a),m.inlineExpressions(o);function s(t,i){Ue(i)&&t.length>0&&(An(m.createExpressionStatement(m.inlineExpressions(t))),t=[]);var o=e.createExpressionForObjectLiteralElementLike(m,n,i,a),s=e.visitNode(o,U,e.isExpression);return s&&(r&&e.startOnNewLine(s),t.push(s)),t}}function se(t){return Ue(t.argumentExpression)?m.updateElementAccessExpression(t,We(e.visitNode(t.expression,U,e.isLeftHandSideExpression)),e.visitNode(t.argumentExpression,U,e.isExpression)):e.visitEachChild(t,U,n)}function ce(t){if(!e.isImportCall(t)&&e.forEach(t.arguments,Ue)){var r=m.createCallBinding(t.expression,v,E,!0),i=r.target,a=r.thisArg;return e.setOriginalNode(e.setTextRange(m.createFunctionApplyCall(We(e.visitNode(i,U,e.isLeftHandSideExpression)),a,ae(t.arguments)),t),t)}return e.visitEachChild(t,U,n)}function le(t){if(e.forEach(t.arguments,Ue)){var r=m.createCallBinding(m.createPropertyAccessExpression(t.expression,"bind"),v),i=r.target,a=r.thisArg;return e.setOriginalNode(e.setTextRange(m.createNewExpression(m.createFunctionApplyCall(We(e.visitNode(i,U,e.isExpression)),a,ae(t.arguments,m.createVoidZero())),void 0,[]),t),t)}return e.visitEachChild(t,U,n)}function ue(e,n){void 0===n&&(n=0);for(var t=e.length,r=n;r<t;r++)pe(e[r])}function de(n){e.isBlock(n)?ue(n.statements):pe(n)}function pe(e){var n=o;o||(o=Ue(e)),fe(e),o=n}function fe(n){switch(n.kind){case 233:return me(n);case 236:return _e(n);case 237:return he(n);case 238:return ve(n);case 239:return Ee(n);case 240:return Se(n);case 241:return Le(n);case 243:return Ae(n);case 244:return Ie(n);case 245:return ke(n);case 246:return Oe(n);case 247:return we(n);case 248:return Me(n);case 249:return Ge(n);case 250:return Be(n);default:return An(e.visitNode(n,U,e.isStatement))}}function me(n){Ue(n)?ue(n.statements):An(e.visitNode(n,U,e.isStatement))}function _e(n){An(e.visitNode(n,U,e.isStatement))}function ge(n){for(var t=0,r=n.declarations;t<r.length;t++){var i=r[t],a=m.cloneNode(i.name);e.setCommentRange(a,i.name),v(a)}var o=e.getInitializedVariables(n),s=o.length,c=0,l=[];while(c<s){for(var u=c;u<s;u++){i=o[u];if(Ue(i.initializer)&&l.length>0)break;l.push(ye(i))}l.length&&(An(m.createExpressionStatement(m.inlineExpressions(l))),c+=l.length,l=[])}}function ye(n){return e.setSourceMapRange(m.createAssignment(e.setSourceMapRange(m.cloneNode(n.name),n.name),e.visitNode(n.initializer,U,e.isExpression)),n)}function he(n){if(Ue(n))if(Ue(n.thenStatement)||Ue(n.elseStatement)){var t=Je(),r=n.elseStatement?Je():void 0;kn(n.elseStatement?r:t,e.visitNode(n.expression,U,e.isExpression),n.expression),de(n.thenStatement),n.elseStatement&&(In(t),qe(r),de(n.elseStatement)),qe(t)}else An(e.visitNode(n,U,e.isStatement));else An(e.visitNode(n,U,e.isStatement))}function ve(n){if(Ue(n)){var t=Je(),r=Je();sn(t),qe(r),de(n.statement),qe(t),Nn(r,e.visitNode(n.expression,U,e.isExpression)),cn()}else An(e.visitNode(n,U,e.isStatement))}function be(t){return o?(on(),t=e.visitEachChild(t,U,n),cn(),t):e.visitEachChild(t,U,n)}function Ee(n){if(Ue(n)){var t=Je(),r=sn(t);qe(t),kn(r,e.visitNode(n.expression,U,e.isExpression)),de(n.statement),In(t),cn()}else An(e.visitNode(n,U,e.isStatement))}function Te(t){return o?(on(),t=e.visitEachChild(t,U,n),cn(),t):e.visitEachChild(t,U,n)}function Se(n){if(Ue(n)){var t=Je(),r=Je(),i=sn(r);if(n.initializer){var a=n.initializer;e.isVariableDeclarationList(a)?ge(a):An(e.setTextRange(m.createExpressionStatement(e.visitNode(a,U,e.isExpression)),a))}qe(t),n.condition&&kn(i,e.visitNode(n.condition,U,e.isExpression)),de(n.statement),qe(r),n.incrementor&&An(e.setTextRange(m.createExpressionStatement(e.visitNode(n.incrementor,U,e.isExpression)),n.incrementor)),In(t),cn()}else An(e.visitNode(n,U,e.isStatement))}function xe(t){o&&on();var r=t.initializer;if(r&&e.isVariableDeclarationList(r)){for(var i=0,a=r.declarations;i<a.length;i++){var s=a[i];v(s.name)}var c=e.getInitializedVariables(r);t=m.updateForStatement(t,c.length>0?m.inlineExpressions(e.map(c,ye)):void 0,e.visitNode(t.condition,U,e.isExpression),e.visitNode(t.incrementor,U,e.isExpression),e.visitIterationBody(t.statement,U,n))}else t=e.visitEachChild(t,U,n);return o&&cn(),t}function Le(n){if(Ue(n)){var t=ze(),r=ze(),i=m.createLoopVariable(),a=n.initializer;v(i),Cn(t,m.createArrayLiteralExpression()),An(m.createForInStatement(r,e.visitNode(n.expression,U,e.isExpression),m.createExpressionStatement(m.createCallExpression(m.createPropertyAccessExpression(t,"push"),void 0,[r])))),Cn(i,m.createNumericLiteral(0));var o=Je(),s=Je(),c=sn(s);qe(o),kn(c,m.createLessThan(i,m.createPropertyAccessExpression(t,"length")));var l=void 0;if(e.isVariableDeclarationList(a)){for(var u=0,d=a.declarations;u<d.length;u++){var p=d[u];v(p.name)}l=m.cloneNode(a.declarations[0].name)}else l=e.visitNode(a,U,e.isExpression),e.Debug.assert(e.isLeftHandSideExpression(l));Cn(l,m.createElementAccessExpression(t,i)),de(n.statement),qe(s),An(m.createExpressionStatement(m.createPostfixIncrement(i))),In(o),cn()}else An(e.visitNode(n,U,e.isStatement))}function De(t){o&&on();var r=t.initializer;if(e.isVariableDeclarationList(r)){for(var i=0,a=r.declarations;i<a.length;i++){var s=a[i];v(s.name)}t=m.updateForInStatement(t,r.declarations[0].name,e.visitNode(t.expression,U,e.isExpression),e.visitNode(t.statement,U,e.isStatement,m.liftToBlock))}else t=e.visitEachChild(t,U,n);return o&&cn(),t}function Ae(n){var t=bn(n.label?e.idText(n.label):void 0);t>0?In(t,n):An(n)}function Ce(t){if(o){var r=bn(t.label&&e.idText(t.label));if(r>0)return Sn(r,t)}return e.visitEachChild(t,U,n)}function Ie(n){var t=vn(n.label?e.idText(n.label):void 0);t>0?In(t,n):An(n)}function Ne(t){if(o){var r=vn(t.label&&e.idText(t.label));if(r>0)return Sn(r,t)}return e.visitEachChild(t,U,n)}function ke(n){wn(e.visitNode(n.expression,U,e.isExpression),n)}function Re(n){return xn(e.visitNode(n.expression,U,e.isExpression),n)}function Oe(n){Ue(n)?($e(We(e.visitNode(n.expression,U,e.isExpression))),de(n.statement),en()):An(e.visitNode(n,U,e.isStatement))}function we(n){if(Ue(n.caseBlock)){for(var t=n.caseBlock,r=t.clauses.length,i=un(),a=We(e.visitNode(n.expression,U,e.isExpression)),o=[],s=-1,c=0;c<r;c++){var l=t.clauses[c];o.push(Je()),288===l.kind&&-1===s&&(s=c)}var u=0,d=[];while(u<r){var p=0;for(c=u;c<r;c++){l=t.clauses[c];if(287===l.kind){if(Ue(l.expression)&&d.length>0)break;d.push(m.createCaseClause(e.visitNode(l.expression,U,e.isExpression),[Sn(o[c],l.expression)]))}else p++}d.length&&(An(m.createSwitchStatement(a,m.createCaseBlock(d))),u+=d.length,d=[]),p>0&&(u+=p,p=0)}In(s>=0?o[s]:i);for(c=0;c<r;c++)qe(o[c]),ue(t.clauses[c].statements);dn()}else An(e.visitNode(n,U,e.isStatement))}function Pe(t){return o&&ln(),t=e.visitEachChild(t,U,n),o&&dn(),t}function Me(n){Ue(n)?(fn(e.idText(n.label)),de(n.statement),mn()):An(e.visitNode(n,U,e.isStatement))}function Fe(t){return o&&pn(e.idText(t.label)),t=e.visitEachChild(t,U,n),o&&mn(),t}function Ge(n){var t;Pn(e.visitNode(null!==(t=n.expression)&&void 0!==t?t:m.createVoidZero(),U,e.isExpression),n)}function Be(t){Ue(t)?(nn(),de(t.tryBlock),t.catchClause&&(tn(t.catchClause.variableDeclaration),de(t.catchClause.block)),t.finallyBlock&&(rn(),de(t.finallyBlock)),an()):An(e.visitEachChild(t,U,n))}function Ue(e){return!!e&&0!==(524288&e.transformFlags)}function Ve(e){for(var n=e.length,t=0;t<n;t++)if(Ue(e[t]))return t;return-1}function je(e,n){return n=S(e,n),1===e?Ke(n):n}function Ke(n){return e.isIdentifier(n)?He(n):n}function He(n){if(!e.isGeneratedIdentifier(n)&&t&&t.has(e.idText(n))){var i=e.getOriginalNode(n);if(e.isIdentifier(i)&&i.parent){var a=T.getReferencedValueDeclaration(i);if(a){var o=r[e.getOriginalNodeId(a)];if(o){var s=e.setParent(e.setTextRange(m.cloneNode(o),o),o.parent);return e.setSourceMapRange(s,n),e.setCommentRange(s,n),s}}}}return n}function We(n){if(e.isGeneratedIdentifier(n)||4096&e.getEmitFlags(n))return n;var t=m.createTempVariable(v);return Cn(t,n,n),t}function ze(e){var n=e?m.createUniqueName(e):m.createTempVariable(void 0);return v(n),n}function Je(){p||(p=[]);var e=M;return M++,p[e]=-1,e}function qe(n){e.Debug.assert(void 0!==p,"No labels were defined."),p[n]=x?x.length:0}function Xe(e){c||(c=[],u=[],l=[],d=[]);var n=u.length;return u[n]=0,l[n]=x?x.length:0,c[n]=e,d.push(e),n}function Ye(){var n=Qe();if(void 0===n)return e.Debug.fail("beginBlock was never called.");var t=u.length;return u[t]=1,l[t]=x?x.length:0,c[t]=n,d.pop(),n}function Qe(){return e.lastOrUndefined(d)}function Ze(){var e=Qe();return e&&e.kind}function $e(e){var n=Je(),t=Je();qe(n),Xe({kind:1,expression:e,startLabel:n,endLabel:t})}function en(){e.Debug.assert(1===Ze());var n=Ye();qe(n.endLabel)}function nn(){var e=Je(),n=Je();return qe(e),Xe({kind:0,state:0,startLabel:e,endLabel:n}),Dn(),n}function tn(i){var a;if(e.Debug.assert(0===Ze()),e.isGeneratedIdentifier(i.name))a=i.name,v(i.name);else{var o=e.idText(i.name);a=ze(o),t||(t=new e.Map,r=[],n.enableSubstitution(79)),t.set(o,!0),r[e.getOriginalNodeId(i)]=a}var s=Qe();e.Debug.assert(s.state<1);var c=s.endLabel;In(c);var l=Je();qe(l),s.state=1,s.catchVariable=a,s.catchLabel=l,Cn(a,m.createCallExpression(m.createPropertyAccessExpression(A,"sent"),void 0,[])),Dn()}function rn(){e.Debug.assert(0===Ze());var n=Qe();e.Debug.assert(n.state<2);var t=n.endLabel;In(t);var r=Je();qe(r),n.state=2,n.finallyLabel=r}function an(){e.Debug.assert(0===Ze());var n=Ye(),t=n.state;t<2?In(n.endLabel):Mn(),qe(n.endLabel),Dn(),n.state=3}function on(){Xe({kind:3,isScript:!0,breakLabel:-1,continueLabel:-1})}function sn(e){var n=Je();return Xe({kind:3,isScript:!1,breakLabel:n,continueLabel:e}),n}function cn(){e.Debug.assert(3===Ze());var n=Ye(),t=n.breakLabel;n.isScript||qe(t)}function ln(){Xe({kind:2,isScript:!0,breakLabel:-1})}function un(){var e=Je();return Xe({kind:2,isScript:!1,breakLabel:e}),e}function dn(){e.Debug.assert(2===Ze());var n=Ye(),t=n.breakLabel;n.isScript||qe(t)}function pn(e){Xe({kind:4,isScript:!0,labelText:e,breakLabel:-1})}function fn(e){var n=Je();Xe({kind:4,isScript:!1,labelText:e,breakLabel:n})}function mn(){e.Debug.assert(4===Ze());var n=Ye();n.isScript||qe(n.breakLabel)}function _n(e){return 2===e.kind||3===e.kind}function gn(e){return 4===e.kind}function yn(e){return 3===e.kind}function hn(e,n){for(var t=n;t>=0;t--){var r=d[t];if(!gn(r))break;if(r.labelText===e)return!0}return!1}function vn(e){if(d)if(e)for(var n=d.length-1;n>=0;n--){var t=d[n];if(gn(t)&&t.labelText===e)return t.breakLabel;if(_n(t)&&hn(e,n-1))return t.breakLabel}else for(n=d.length-1;n>=0;n--){t=d[n];if(_n(t))return t.breakLabel}return 0}function bn(e){if(d)if(e)for(var n=d.length-1;n>=0;n--){var t=d[n];if(yn(t)&&hn(e,n-1))return t.continueLabel}else for(n=d.length-1;n>=0;n--){t=d[n];if(yn(t))return t.continueLabel}return 0}function En(e){if(void 0!==e&&e>0){void 0===f&&(f=[]);var n=m.createNumericLiteral(-1);return void 0===f[e]?f[e]=[n]:f[e].push(n),n}return m.createOmittedExpression()}function Tn(n){var t=m.createNumericLiteral(n);return e.addSyntheticTrailingComment(t,3,s(n)),t}function Sn(n,t){return e.Debug.assertLessThan(0,n,"Invalid label"),e.setTextRange(m.createReturnStatement(m.createArrayLiteralExpression([Tn(3),En(n)])),t)}function xn(n,t){return e.setTextRange(m.createReturnStatement(m.createArrayLiteralExpression(n?[Tn(2),n]:[Tn(2)])),t)}function Ln(n){return e.setTextRange(m.createCallExpression(m.createPropertyAccessExpression(A,"sent"),void 0,[]),n)}function Dn(){Fn(0)}function An(e){e?Fn(1,[e]):Dn()}function Cn(e,n,t){Fn(2,[e,n],t)}function In(e,n){Fn(3,[e],n)}function Nn(e,n,t){Fn(4,[e,n],t)}function kn(e,n,t){Fn(5,[e,n],t)}function Rn(e,n){Fn(7,[e],n)}function On(e,n){Fn(6,[e],n)}function wn(e,n){Fn(8,[e],n)}function Pn(e,n){Fn(9,[e],n)}function Mn(){Fn(10)}function Fn(e,n,t){void 0===x&&(x=[],L=[],D=[]),void 0===p&&qe(Je());var r=x.length;x[r]=e,L[r]=n,D[r]=t}function Gn(){F=0,G=0,C=void 0,I=!1,N=!1,k=void 0,R=void 0,O=void 0,w=void 0,P=void 0;var n=Bn();return _().createGeneratorHelper(e.setEmitFlags(m.createFunctionExpression(void 0,void 0,void 0,void 0,[m.createParameterDeclaration(void 0,void 0,void 0,A)],void 0,m.createBlock(n,n.length>0)),524288))}function Bn(){if(x){for(var n=0;n<x.length;n++)Jn(n);Vn(x.length)}else Vn(0);if(k){var t=m.createPropertyAccessExpression(A,"label"),r=m.createSwitchStatement(t,m.createCaseBlock(k));return[e.startOnNewLine(r)]}return R||[]}function Un(){R&&(Kn(!I),I=!1,N=!1,G++)}function Vn(e){jn(e)&&(Hn(e),P=void 0,Qn(void 0,void 0)),R&&k&&Kn(!1),Wn()}function jn(e){if(!N)return!0;if(!p||!f)return!1;for(var n=0;n<p.length;n++)if(p[n]===e&&f[n])return!0;return!1}function Kn(e){if(k||(k=[]),R){if(P)for(var n=P.length-1;n>=0;n--){var t=P[n];R=[m.createWithStatement(t.expression,m.createBlock(R))]}if(w){var r=w.startLabel,i=w.catchLabel,a=w.finallyLabel,o=w.endLabel;R.unshift(m.createExpressionStatement(m.createCallExpression(m.createPropertyAccessExpression(m.createPropertyAccessExpression(A,"trys"),"push"),void 0,[m.createArrayLiteralExpression([En(r),En(i),En(a),En(o)])]))),w=void 0}e&&R.push(m.createExpressionStatement(m.createAssignment(m.createPropertyAccessExpression(A,"label"),m.createNumericLiteral(G+1))))}k.push(m.createCaseClause(m.createNumericLiteral(G),R||[])),R=void 0}function Hn(e){if(p)for(var n=0;n<p.length;n++)p[n]===e&&(Un(),void 0===C&&(C=[]),void 0===C[G]?C[G]=[n]:C[G].push(n))}function Wn(){if(void 0!==f&&void 0!==C)for(var e=0;e<C.length;e++){var n=C[e];if(void 0!==n)for(var t=0,r=n;t<r.length;t++){var i=r[t],a=f[i];if(void 0!==a)for(var o=0,s=a;o<s.length;o++){var c=s[o];c.text=String(e)}}}}function zn(e){if(c)for(;F<u.length&&l[F]<=e;F++){var n=c[F],t=u[F];switch(n.kind){case 0:0===t?(O||(O=[]),R||(R=[]),O.push(w),w=n):1===t&&(w=O.pop());break;case 1:0===t?(P||(P=[]),P.push(n)):1===t&&P.pop();break}}}function Jn(e){if(Hn(e),zn(e),!I){I=!1,N=!1;var n=x[e];if(0!==n){if(10===n)return rt();var t=L[e];if(1===n)return qn(t[0]);var r=D[e];switch(n){case 2:return Xn(t[0],t[1],r);case 3:return Zn(t[0],r);case 4:return $n(t[0],t[1],r);case 5:return et(t[0],t[1],r);case 6:return nt(t[0],r);case 7:return tt(t[0],r);case 8:return Qn(t[0],r);case 9:return Yn(t[0],r)}}}}function qn(e){e&&(R?R.push(e):R=[e])}function Xn(n,t,r){qn(e.setTextRange(m.createExpressionStatement(m.createAssignment(n,t)),r))}function Yn(n,t){I=!0,N=!0,qn(e.setTextRange(m.createThrowStatement(n),t))}function Qn(n,t){I=!0,N=!0,qn(e.setEmitFlags(e.setTextRange(m.createReturnStatement(m.createArrayLiteralExpression(n?[Tn(2),n]:[Tn(2)])),t),384))}function Zn(n,t){I=!0,qn(e.setEmitFlags(e.setTextRange(m.createReturnStatement(m.createArrayLiteralExpression([Tn(3),En(n)])),t),384))}function $n(n,t,r){qn(e.setEmitFlags(m.createIfStatement(t,e.setEmitFlags(e.setTextRange(m.createReturnStatement(m.createArrayLiteralExpression([Tn(3),En(n)])),r),384)),1))}function et(n,t,r){qn(e.setEmitFlags(m.createIfStatement(m.createLogicalNot(t),e.setEmitFlags(e.setTextRange(m.createReturnStatement(m.createArrayLiteralExpression([Tn(3),En(n)])),r),384)),1))}function nt(n,t){I=!0,qn(e.setEmitFlags(e.setTextRange(m.createReturnStatement(m.createArrayLiteralExpression(n?[Tn(4),n]:[Tn(4)])),t),384))}function tt(n,t){I=!0,qn(e.setEmitFlags(e.setTextRange(m.createReturnStatement(m.createArrayLiteralExpression([Tn(5),n])),t),384))}function rt(){I=!0,qn(m.createReturnStatement(m.createArrayLiteralExpression([Tn(7)])))}}(function(e){e[e["Nop"]=0]="Nop",e[e["Statement"]=1]="Statement",e[e["Assign"]=2]="Assign",e[e["Break"]=3]="Break",e[e["BreakWhenTrue"]=4]="BreakWhenTrue",e[e["BreakWhenFalse"]=5]="BreakWhenFalse",e[e["Yield"]=6]="Yield",e[e["YieldStar"]=7]="YieldStar",e[e["Return"]=8]="Return",e[e["Throw"]=9]="Throw",e[e["Endfinally"]=10]="Endfinally"})(n||(n={})),function(e){e[e["Open"]=0]="Open",e[e["Close"]=1]="Close"}(t||(t={})),function(e){e[e["Exception"]=0]="Exception",e[e["With"]=1]="With",e[e["Switch"]=2]="Switch",e[e["Loop"]=3]="Loop",e[e["Labeled"]=4]="Labeled"}(r||(r={})),function(e){e[e["Try"]=0]="Try",e[e["Catch"]=1]="Catch",e[e["Finally"]=2]="Finally",e[e["Done"]=3]="Done"}(i||(i={})),function(e){e[e["Next"]=0]="Next",e[e["Throw"]=1]="Throw",e[e["Return"]=2]="Return",e[e["Break"]=3]="Break",e[e["Yield"]=4]="Yield",e[e["YieldStar"]=5]="YieldStar",e[e["Catch"]=6]="Catch",e[e["Endfinally"]=7]="Endfinally"}(o||(o={})),e.transformGenerators=c}(d||(d={})),function(e){function n(n){function r(n){switch(n){case e.ModuleKind.AMD:return D;case e.ModuleKind.UMD:return A;default:return L}}var i=n.factory,o=n.getEmitHelperFactory,s=n.startLexicalEnvironment,c=n.endLexicalEnvironment,l=n.hoistVariableDeclaration,u=n.getCompilerOptions(),d=n.getEmitResolver(),p=n.getEmitHost(),f=e.getEmitScriptTarget(u),m=e.getEmitModuleKind(u),_=n.onSubstituteNode,g=n.onEmitNode;n.onSubstituteNode=be,n.onEmitNode=ve,n.enableSubstitution(206),n.enableSubstitution(208),n.enableSubstitution(79),n.enableSubstitution(219),n.enableSubstitution(292),n.enableEmitNotification(300);var y,h,v,b=[],E=[],T=[];return e.chainBundle(n,S);function S(t){if(t.isDeclarationFile||!(e.isEffectiveExternalModule(t,u)||4194304&t.transformFlags||e.isJsonSourceFile(t)&&e.hasJsonModuleEmitEnabled(u)&&e.outFile(u)))return t;y=t,h=e.collectExternalModuleInfo(n,t,d,u),b[e.getOriginalNodeId(t)]=h;var i=r(m),a=i(t);return y=void 0,h=void 0,v=!1,a}function x(){return!(h.exportEquals||!e.isExternalModule(y))}function L(t){s();var r=[],a=e.getStrictOptionValue(u,"alwaysStrict")||!u.noImplicitUseStrict&&e.isExternalModule(y),o=i.copyPrologue(t.statements,r,a&&!e.isJsonSourceFile(t),R);if(x()&&e.append(r,_e()),e.length(h.exportedNames))for(var l=50,d=0;d<h.exportedNames.length;d+=l)e.append(r,i.createExpressionStatement(e.reduceLeft(h.exportedNames.slice(d,d+l),function(n,t){return i.createAssignment(i.createPropertyAccessExpression(i.createIdentifier("exports"),i.createIdentifier(e.idText(t))),n)},i.createVoidZero())));e.append(r,e.visitNode(h.externalHelpersImportDeclaration,R,e.isStatement)),e.addRange(r,e.visitNodes(t.statements,R,e.isStatement,o)),k(r,!1),e.insertStatementsAfterStandardPrologue(r,c());var p=i.updateSourceFile(t,e.setTextRange(i.createNodeArray(r),t.statements));return e.addEmitHelpers(p,n.readEmitHelpers()),p}function D(t){var r=i.createIdentifier("define"),o=e.tryGetModuleNameFromFile(i,t,p,u),s=e.isJsonSourceFile(t)&&t,c=C(t,!0),l=c.aliasedModuleNames,d=c.unaliasedModuleNames,f=c.importAliasNames,m=i.updateSourceFile(t,e.setTextRange(i.createNodeArray([i.createExpressionStatement(i.createCallExpression(r,void 0,a(a([],o?[o]:[],!0),[i.createArrayLiteralExpression(s?e.emptyArray:a(a([i.createStringLiteral("require"),i.createStringLiteral("exports")],l,!0),d,!0)),s?s.statements.length?s.statements[0].expression:i.createObjectLiteralExpression():i.createFunctionExpression(void 0,void 0,void 0,void 0,a([i.createParameterDeclaration(void 0,void 0,void 0,"require"),i.createParameterDeclaration(void 0,void 0,void 0,"exports")],f,!0),void 0,N(t))],!1)))]),t.statements));return e.addEmitHelpers(m,n.readEmitHelpers()),m}function A(t){var r=C(t,!1),o=r.aliasedModuleNames,s=r.unaliasedModuleNames,c=r.importAliasNames,l=e.tryGetModuleNameFromFile(i,t,p,u),d=i.createFunctionExpression(void 0,void 0,void 0,void 0,[i.createParameterDeclaration(void 0,void 0,void 0,"factory")],void 0,e.setTextRange(i.createBlock([i.createIfStatement(i.createLogicalAnd(i.createTypeCheck(i.createIdentifier("module"),"object"),i.createTypeCheck(i.createPropertyAccessExpression(i.createIdentifier("module"),"exports"),"object")),i.createBlock([i.createVariableStatement(void 0,[i.createVariableDeclaration("v",void 0,void 0,i.createCallExpression(i.createIdentifier("factory"),void 0,[i.createIdentifier("require"),i.createIdentifier("exports")]))]),e.setEmitFlags(i.createIfStatement(i.createStrictInequality(i.createIdentifier("v"),i.createIdentifier("undefined")),i.createExpressionStatement(i.createAssignment(i.createPropertyAccessExpression(i.createIdentifier("module"),"exports"),i.createIdentifier("v")))),1)]),i.createIfStatement(i.createLogicalAnd(i.createTypeCheck(i.createIdentifier("define"),"function"),i.createPropertyAccessExpression(i.createIdentifier("define"),"amd")),i.createBlock([i.createExpressionStatement(i.createCallExpression(i.createIdentifier("define"),void 0,a(a([],l?[l]:[],!0),[i.createArrayLiteralExpression(a(a([i.createStringLiteral("require"),i.createStringLiteral("exports")],o,!0),s,!0)),i.createIdentifier("factory")],!1)))])))],!0),void 0)),f=i.updateSourceFile(t,e.setTextRange(i.createNodeArray([i.createExpressionStatement(i.createCallExpression(d,void 0,[i.createFunctionExpression(void 0,void 0,void 0,void 0,a([i.createParameterDeclaration(void 0,void 0,void 0,"require"),i.createParameterDeclaration(void 0,void 0,void 0,"exports")],c,!0),void 0,N(t))]))]),t.statements));return e.addEmitHelpers(f,n.readEmitHelpers()),f}function C(n,t){for(var r=[],a=[],o=[],s=0,c=n.amdDependencies;s<c.length;s++){var l=c[s];l.name?(r.push(i.createStringLiteral(l.path)),o.push(i.createParameterDeclaration(void 0,void 0,void 0,l.name))):a.push(i.createStringLiteral(l.path))}for(var f=0,m=h.externalImports;f<m.length;f++){var _=m[f],g=e.getExternalModuleNameLiteral(i,_,y,p,d,u),v=e.getLocalNameForExternalImport(i,_,y);g&&(t&&v?(e.setEmitFlags(v,4),r.push(g),o.push(i.createParameterDeclaration(void 0,void 0,void 0,v))):a.push(g))}return{aliasedModuleNames:r,unaliasedModuleNames:a,importAliasNames:o}}function I(n){if(!e.isImportEqualsDeclaration(n)&&!e.isExportDeclaration(n)&&e.getExternalModuleNameLiteral(i,n,y,p,d,u)){var t=e.getLocalNameForExternalImport(i,n,y),r=q(n,t);if(r!==t)return i.createExpressionStatement(i.createAssignment(t,r))}}function N(n){s();var r=[],a=i.copyPrologue(n.statements,r,!u.noImplicitUseStrict,R);x()&&e.append(r,_e()),e.length(h.exportedNames)&&e.append(r,i.createExpressionStatement(e.reduceLeft(h.exportedNames,function(n,t){return i.createAssignment(i.createPropertyAccessExpression(i.createIdentifier("exports"),i.createIdentifier(e.idText(t))),n)},i.createVoidZero()))),e.append(r,e.visitNode(h.externalHelpersImportDeclaration,R,e.isStatement)),m===e.ModuleKind.AMD&&e.addRange(r,e.mapDefined(h.externalImports,I)),e.addRange(r,e.visitNodes(n.statements,R,e.isStatement,a)),k(r,!0),e.insertStatementsAfterStandardPrologue(r,c());var o=i.createBlock(r,!0);return v&&e.addEmitHelper(o,t),o}function k(n,t){if(h.exportEquals){var r=e.visitNode(h.exportEquals.expression,w);if(r)if(t){var a=i.createReturnStatement(r);e.setTextRange(a,h.exportEquals),e.setEmitFlags(a,1920),n.push(a)}else{a=i.createExpressionStatement(i.createAssignment(i.createPropertyAccessExpression(i.createIdentifier("module"),"exports"),r));e.setTextRange(a,h.exportEquals),e.setEmitFlags(a,1536),n.push(a)}}}function R(e){switch(e.kind){case 264:return X(e);case 263:return Q(e);case 270:return Z(e);case 269:return $(e);case 235:return te(e);case 254:return ee(e);case 255:return ne(e);case 347:return ae(e);case 348:return se(e);default:return w(e)}}function O(t,r){if(!(71305216&t.transformFlags))return t;switch(t.kind){case 240:return G(t);case 236:return B(t);case 210:return U(t,r);case 345:return V(t,r);case 206:if(e.isImportCall(t))return K(t);break;case 219:if(e.isDestructuringAssignment(t))return F(t,r);break;case 217:case 218:return j(t,r)}return e.visitEachChild(t,w,n)}function w(e){return O(e,!1)}function P(e){return O(e,!0)}function M(n){if(e.isObjectLiteralExpression(n))for(var t=0,r=n.properties;t<r.length;t++){var i=r[t];switch(i.kind){case 291:if(M(i.initializer))return!0;break;case 292:if(M(i.name))return!0;break;case 293:if(M(i.expression))return!0;break;case 167:case 170:case 171:return!1;default:e.Debug.assertNever(i,"Unhandled object member kind")}}else if(e.isArrayLiteralExpression(n))for(var a=0,o=n.elements;a<o.length;a++){i=o[a];if(e.isSpreadElement(i)){if(M(i.expression))return!0}else if(M(i))return!0}else if(e.isIdentifier(n))return e.length(Ae(n))>(e.isExportName(n)?1:0);return!1}function F(t,r){return M(t.left)?e.flattenDestructuringAssignment(t,w,n,0,!r,re):e.visitEachChild(t,w,n)}function G(t){return i.updateForStatement(t,e.visitNode(t.initializer,P,e.isForInitializer),e.visitNode(t.condition,w,e.isExpression),e.visitNode(t.incrementor,P,e.isExpression),e.visitIterationBody(t.statement,w,n))}function B(n){return i.updateExpressionStatement(n,e.visitNode(n.expression,P,e.isExpression))}function U(n,t){return i.updateParenthesizedExpression(n,e.visitNode(n.expression,t?P:w,e.isExpression))}function V(n,t){return i.updatePartiallyEmittedExpression(n,e.visitNode(n.expression,t?P:w,e.isExpression))}function j(t,r){if((45===t.operator||46===t.operator)&&e.isIdentifier(t.operand)&&!e.isGeneratedIdentifier(t.operand)&&!e.isLocalName(t.operand)&&!e.isDeclarationNameOfEnumOrNamespace(t.operand)){var a=Ae(t.operand);if(a){var o=void 0,s=e.visitNode(t.operand,w,e.isExpression);e.isPrefixUnaryExpression(t)?s=i.updatePrefixUnaryExpression(t,s):(s=i.updatePostfixUnaryExpression(t,s),r||(o=i.createTempVariable(l),s=i.createAssignment(o,s),e.setTextRange(s,t)),s=i.createComma(s,i.cloneNode(t.operand)),e.setTextRange(s,t));for(var c=0,u=a;c<u.length;c++){var d=u[c];T[e.getNodeId(s)]=!0,s=ye(d,s),e.setTextRange(s,t)}return o&&(T[e.getNodeId(s)]=!0,s=i.createComma(s,o),e.setTextRange(s,t)),s}}return e.visitEachChild(t,w,n)}function K(n){var t=e.getExternalModuleNameLiteral(i,n,y,p,d,u),r=e.visitNode(e.firstOrUndefined(n.arguments),w),a=!t||r&&e.isStringLiteral(r)&&r.text===t.text?r:t,o=!!(8192&n.transformFlags);switch(u.module){case e.ModuleKind.AMD:return W(a,o);case e.ModuleKind.UMD:return H(null!==a&&void 0!==a?a:i.createVoidZero(),o);case e.ModuleKind.CommonJS:default:return z(a,o)}}function H(n,t){if(v=!0,e.isSimpleCopiableExpression(n)){var r=e.isGeneratedIdentifier(n)?n:e.isStringLiteral(n)?i.createStringLiteralFromNode(n):e.setEmitFlags(e.setTextRange(i.cloneNode(n),n),1536);return i.createConditionalExpression(i.createIdentifier("__syncRequire"),void 0,z(n,t),void 0,W(r,t))}var a=i.createTempVariable(l);return i.createComma(i.createAssignment(a,n),i.createConditionalExpression(i.createIdentifier("__syncRequire"),void 0,z(a,t),void 0,W(a,t)))}function W(n,t){var r,a=i.createUniqueName("resolve"),s=i.createUniqueName("reject"),c=[i.createParameterDeclaration(void 0,void 0,void 0,a),i.createParameterDeclaration(void 0,void 0,void 0,s)],l=i.createBlock([i.createExpressionStatement(i.createCallExpression(i.createIdentifier("require"),void 0,[i.createArrayLiteralExpression([n||i.createOmittedExpression()]),a,s]))]);f>=2?r=i.createArrowFunction(void 0,void 0,c,void 0,void 0,l):(r=i.createFunctionExpression(void 0,void 0,void 0,void 0,c,void 0,l),t&&e.setEmitFlags(r,8));var d=i.createNewExpression(i.createIdentifier("Promise"),void 0,[r]);return u.esModuleInterop?i.createCallExpression(i.createPropertyAccessExpression(d,i.createIdentifier("then")),void 0,[o().createImportStarCallbackHelper()]):d}function z(n,t){var r,a=i.createCallExpression(i.createPropertyAccessExpression(i.createIdentifier("Promise"),"resolve"),void 0,[]),s=i.createCallExpression(i.createIdentifier("require"),void 0,n?[n]:[]);return u.esModuleInterop&&(s=o().createImportStarHelper(s)),f>=2?r=i.createArrowFunction(void 0,void 0,[],void 0,void 0,s):(r=i.createFunctionExpression(void 0,void 0,void 0,void 0,[],void 0,i.createBlock([i.createReturnStatement(s)])),t&&e.setEmitFlags(r,8)),i.createCallExpression(i.createPropertyAccessExpression(a,"then"),void 0,[r])}function J(n,t){return!u.esModuleInterop||67108864&e.getEmitFlags(n)?t:e.getExportNeedsImportStarHelper(n)?o().createImportStarHelper(t):t}function q(n,t){return!u.esModuleInterop||67108864&e.getEmitFlags(n)?t:e.getImportNeedsImportStarHelper(n)?o().createImportStarHelper(t):e.getImportNeedsImportDefaultHelper(n)?o().createImportDefaultHelper(t):t}function X(n){var t,r=e.getNamespaceDeclarationNode(n);if(m!==e.ModuleKind.AMD){if(!n.importClause)return e.setOriginalNode(e.setTextRange(i.createExpressionStatement(Y(n)),n),n);var a=[];r&&!e.isDefaultImport(n)?a.push(i.createVariableDeclaration(i.cloneNode(r.name),void 0,void 0,q(n,Y(n)))):(a.push(i.createVariableDeclaration(i.getGeneratedNameForNode(n),void 0,void 0,q(n,Y(n)))),r&&e.isDefaultImport(n)&&a.push(i.createVariableDeclaration(i.cloneNode(r.name),void 0,void 0,i.getGeneratedNameForNode(n)))),t=e.append(t,e.setOriginalNode(e.setTextRange(i.createVariableStatement(void 0,i.createVariableDeclarationList(a,f>=2?2:0)),n),n))}else r&&e.isDefaultImport(n)&&(t=e.append(t,i.createVariableStatement(void 0,i.createVariableDeclarationList([e.setOriginalNode(e.setTextRange(i.createVariableDeclaration(i.cloneNode(r.name),void 0,void 0,i.getGeneratedNameForNode(n)),n),n)],f>=2?2:0))));if(oe(n)){var o=e.getOriginalNodeId(n);E[o]=ce(E[o],n)}else t=ce(t,n);return e.singleOrMany(t)}function Y(n){var t=e.getExternalModuleNameLiteral(i,n,y,p,d,u),r=[];return t&&r.push(t),i.createCallExpression(i.createIdentifier("require"),void 0,r)}function Q(n){var t;if(e.Debug.assert(e.isExternalModuleImportEqualsDeclaration(n),"import= for internal module references should be handled in an earlier transformer."),m!==e.ModuleKind.AMD?t=e.hasSyntacticModifier(n,1)?e.append(t,e.setOriginalNode(e.setTextRange(i.createExpressionStatement(ye(n.name,Y(n))),n),n)):e.append(t,e.setOriginalNode(e.setTextRange(i.createVariableStatement(void 0,i.createVariableDeclarationList([i.createVariableDeclaration(i.cloneNode(n.name),void 0,void 0,Y(n))],f>=2?2:0)),n),n)):e.hasSyntacticModifier(n,1)&&(t=e.append(t,e.setOriginalNode(e.setTextRange(i.createExpressionStatement(ye(i.getExportName(n),i.getLocalName(n))),n),n))),oe(n)){var r=e.getOriginalNodeId(n);E[r]=le(E[r],n)}else t=le(t,n);return e.singleOrMany(t)}function Z(n){if(n.moduleSpecifier){var t=i.getGeneratedNameForNode(n);if(n.exportClause&&e.isNamedExports(n.exportClause)){var r=[];m!==e.ModuleKind.AMD&&r.push(e.setOriginalNode(e.setTextRange(i.createVariableStatement(void 0,i.createVariableDeclarationList([i.createVariableDeclaration(t,void 0,void 0,Y(n))])),n),n));for(var a=0,s=n.exportClause.elements;a<s.length;a++){var c=s[a];if(0===f)r.push(e.setOriginalNode(e.setTextRange(i.createExpressionStatement(o().createCreateBindingHelper(t,i.createStringLiteralFromNode(c.propertyName||c.name),c.propertyName?i.createStringLiteralFromNode(c.name):void 0)),c),c));else{var l=!!u.esModuleInterop&&!(67108864&e.getEmitFlags(n))&&"default"===e.idText(c.propertyName||c.name),d=i.createPropertyAccessExpression(l?o().createImportDefaultHelper(t):t,c.propertyName||c.name);r.push(e.setOriginalNode(e.setTextRange(i.createExpressionStatement(ye(i.getExportName(c),d,void 0,!0)),c),c))}}return e.singleOrMany(r)}if(n.exportClause){r=[];return r.push(e.setOriginalNode(e.setTextRange(i.createExpressionStatement(ye(i.cloneNode(n.exportClause.name),J(n,m!==e.ModuleKind.AMD?Y(n):e.isExportNamespaceAsDefaultDeclaration(n)?t:i.createIdentifier(e.idText(n.exportClause.name))))),n),n)),e.singleOrMany(r)}return e.setOriginalNode(e.setTextRange(i.createExpressionStatement(o().createExportStarHelper(m!==e.ModuleKind.AMD?Y(n):t)),n),n)}}function $(n){if(!n.isExportEquals){var t,r=n.original;if(r&&oe(r)){var a=e.getOriginalNodeId(n);E[a]=me(E[a],i.createIdentifier("default"),e.visitNode(n.expression,w),n,!0)}else t=me(t,i.createIdentifier("default"),e.visitNode(n.expression,w),n,!0);return e.singleOrMany(t)}}function ee(t){var r;if(r=e.hasSyntacticModifier(t,1)?e.append(r,e.setOriginalNode(e.setTextRange(i.createFunctionDeclaration(void 0,e.visitNodes(t.modifiers,he,e.isModifier),t.asteriskToken,i.getDeclarationName(t,!0,!0),void 0,e.visitNodes(t.parameters,w),void 0,e.visitEachChild(t.body,w,n)),t),t)):e.append(r,e.visitEachChild(t,w,n)),oe(t)){var a=e.getOriginalNodeId(t);E[a]=pe(E[a],t)}else r=pe(r,t);return e.singleOrMany(r)}function ne(t){var r;if(r=e.hasSyntacticModifier(t,1)?e.append(r,e.setOriginalNode(e.setTextRange(i.createClassDeclaration(void 0,e.visitNodes(t.modifiers,he,e.isModifier),i.getDeclarationName(t,!0,!0),void 0,e.visitNodes(t.heritageClauses,w),e.visitNodes(t.members,w)),t),t)):e.append(r,e.visitEachChild(t,w,n)),oe(t)){var a=e.getOriginalNodeId(t);E[a]=pe(E[a],t)}else r=pe(r,t);return e.singleOrMany(r)}function te(t){var r,a,o;if(e.hasSyntacticModifier(t,1)){for(var s=void 0,c=!1,l=0,u=t.declarationList.declarations;l<u.length;l++){var d=u[l];if(e.isIdentifier(d.name)&&e.isLocalName(d.name))s||(s=e.visitNodes(t.modifiers,he,e.isModifier)),a=e.append(a,d);else if(d.initializer)if(!e.isBindingPattern(d.name)&&(e.isArrowFunction(d.initializer)||e.isFunctionExpression(d.initializer)||e.isClassExpression(d.initializer))){var p=i.createAssignment(e.setTextRange(i.createPropertyAccessExpression(i.createIdentifier("exports"),d.name),d.name),i.createIdentifier(e.getTextOfIdentifierOrLiteral(d.name))),f=i.createVariableDeclaration(d.name,d.exclamationToken,d.type,e.visitNode(d.initializer,w));a=e.append(a,f),o=e.append(o,p),c=!0}else o=e.append(o,ie(d))}if(a&&(r=e.append(r,i.updateVariableStatement(t,s,i.updateVariableDeclarationList(t.declarationList,a)))),o){var m=e.setOriginalNode(e.setTextRange(i.createExpressionStatement(i.inlineExpressions(o)),t),t);c&&e.removeAllComments(m),r=e.append(r,m)}}else r=e.append(r,e.visitEachChild(t,w,n));if(oe(t)){var _=e.getOriginalNodeId(t);E[_]=ue(E[_],t)}else r=ue(r,t);return e.singleOrMany(r)}function re(n,t,r){var a=Ae(n);if(a){for(var o=e.isExportName(n)?t:i.createAssignment(n,t),s=0,c=a;s<c.length;s++){var l=c[s];e.setEmitFlags(o,4),o=ye(l,o,r)}return o}return i.createAssignment(n,t)}function ie(t){return e.isBindingPattern(t.name)?e.flattenDestructuringAssignment(e.visitNode(t,w),void 0,n,0,!1,re):i.createAssignment(e.setTextRange(i.createPropertyAccessExpression(i.createIdentifier("exports"),t.name),t.name),t.initializer?e.visitNode(t.initializer,w):i.createVoidZero())}function ae(n){if(oe(n)&&235===n.original.kind){var t=e.getOriginalNodeId(n);E[t]=ue(E[t],n.original)}return n}function oe(n){return 0!==(4194304&e.getEmitFlags(n))}function se(n){var t=e.getOriginalNodeId(n),r=E[t];return r?(delete E[t],e.append(r,n)):n}function ce(e,n){if(h.exportEquals)return e;var t=n.importClause;if(!t)return e;t.name&&(e=fe(e,t));var r=t.namedBindings;if(r)switch(r.kind){case 266:e=fe(e,r);break;case 267:for(var i=0,a=r.elements;i<a.length;i++){var o=a[i];e=fe(e,o,!0)}break}return e}function le(e,n){return h.exportEquals?e:fe(e,n)}function ue(e,n){if(h.exportEquals)return e;for(var t=0,r=n.declarationList.declarations;t<r.length;t++){var i=r[t];e=de(e,i)}return e}function de(n,t){if(h.exportEquals)return n;if(e.isBindingPattern(t.name))for(var r=0,i=t.name.elements;r<i.length;r++){var a=i[r];e.isOmittedExpression(a)||(n=de(n,a))}else e.isGeneratedIdentifier(t.name)||(n=fe(n,t));return n}function pe(n,t){if(h.exportEquals)return n;if(e.hasSyntacticModifier(t,1)){var r=e.hasSyntacticModifier(t,512)?i.createIdentifier("default"):i.getDeclarationName(t);n=me(n,r,i.getLocalName(t),t)}return t.name&&(n=fe(n,t)),n}function fe(n,t,r){var a=i.getDeclarationName(t),o=h.exportSpecifiers.get(e.idText(a));if(o)for(var s=0,c=o;s<c.length;s++){var l=c[s];n=me(n,l.name,a,l.name,void 0,r)}return n}function me(n,t,r,i,a,o){return n=e.append(n,ge(t,r,i,a,o)),n}function _e(){var n;return n=0===f?i.createExpressionStatement(ye(i.createIdentifier("__esModule"),i.createTrue())):i.createExpressionStatement(i.createCallExpression(i.createPropertyAccessExpression(i.createIdentifier("Object"),"defineProperty"),void 0,[i.createIdentifier("exports"),i.createStringLiteral("__esModule"),i.createObjectLiteralExpression([i.createPropertyAssignment("value",i.createTrue())])])),e.setEmitFlags(n,1048576),n}function ge(n,t,r,a,o){var s=e.setTextRange(i.createExpressionStatement(ye(n,t,void 0,o)),r);return e.startOnNewLine(s),a||e.setEmitFlags(s,1536),s}function ye(n,t,r,a){return e.setTextRange(a&&0!==f?i.createCallExpression(i.createPropertyAccessExpression(i.createIdentifier("Object"),"defineProperty"),void 0,[i.createIdentifier("exports"),i.createStringLiteralFromNode(n),i.createObjectLiteralExpression([i.createPropertyAssignment("enumerable",i.createTrue()),i.createPropertyAssignment("get",i.createFunctionExpression(void 0,void 0,void 0,void 0,[],void 0,i.createBlock([i.createReturnStatement(t)])))])]):i.createAssignment(i.createPropertyAccessExpression(i.createIdentifier("exports"),i.cloneNode(n)),t),r)}function he(e){switch(e.kind){case 93:case 88:return}return e}function ve(n,t,r){300===t.kind?(y=t,h=b[e.getOriginalNodeId(y)],g(n,t,r),y=void 0,h=void 0):g(n,t,r)}function be(n,t){return t=_(n,t),t.id&&T[t.id]?t:1===n?Te(t):e.isShorthandPropertyAssignment(t)?Ee(t):t}function Ee(n){var t=n.name,r=Le(t);if(r!==t){if(n.objectAssignmentInitializer){var a=i.createAssignment(r,n.objectAssignmentInitializer);return e.setTextRange(i.createPropertyAssignment(t,a),n)}return e.setTextRange(i.createPropertyAssignment(t,r),n)}return n}function Te(e){switch(e.kind){case 79:return Le(e);case 206:return Se(e);case 208:return xe(e);case 219:return De(e)}return e}function Se(n){if(e.isIdentifier(n.expression)){var t=Le(n.expression);if(T[e.getNodeId(t)]=!0,!e.isIdentifier(t))return e.addEmitFlags(i.updateCallExpression(n,t,void 0,n.arguments),536870912)}return n}function xe(n){if(e.isIdentifier(n.tag)){var t=Le(n.tag);if(T[e.getNodeId(t)]=!0,!e.isIdentifier(t))return e.addEmitFlags(i.updateTaggedTemplateExpression(n,t,void 0,n.template),536870912)}return n}function Le(n){var t,r;if(4096&e.getEmitFlags(n)){var a=e.getExternalHelpersModuleName(y);return a?i.createPropertyAccessExpression(a,n):n}if((!e.isGeneratedIdentifier(n)||64&n.autoGenerateFlags)&&!e.isLocalName(n)){var o=d.getReferencedExportContainer(n,e.isExportName(n));if(o&&300===o.kind)return e.setTextRange(i.createPropertyAccessExpression(i.createIdentifier("exports"),i.cloneNode(n)),n);var s=d.getReferencedImportDeclaration(n);if(s){if(e.isImportClause(s))return e.setTextRange(i.createPropertyAccessExpression(i.getGeneratedNameForNode(s.parent),i.createIdentifier("default")),n);if(e.isImportSpecifier(s)){var c=s.propertyName||s.name;return e.setTextRange(i.createPropertyAccessExpression(i.getGeneratedNameForNode((null===(r=null===(t=s.parent)||void 0===t?void 0:t.parent)||void 0===r?void 0:r.parent)||s),i.cloneNode(c)),n)}}}return n}function De(n){if(e.isAssignmentOperator(n.operatorToken.kind)&&e.isIdentifier(n.left)&&!e.isGeneratedIdentifier(n.left)&&!e.isLocalName(n.left)&&!e.isDeclarationNameOfEnumOrNamespace(n.left)){var t=Ae(n.left);if(t){for(var r=n,i=0,a=t;i<a.length;i++){var o=a[i];T[e.getNodeId(r)]=!0,r=ye(o,r,n)}return r}}return n}function Ae(n){if(!e.isGeneratedIdentifier(n)){var t=d.getReferencedImportDeclaration(n)||d.getReferencedValueDeclaration(n);if(t)return h&&h.exportedBindings[e.getOriginalNodeId(t)]}}}e.transformModule=n;var t={name:"typescript:dynamicimport-sync-require",scoped:!0,text:'\n            var __syncRequire = typeof module === "object" && typeof module.exports === "object";'}}(d||(d={})),function(e){function n(n){var t=n.factory,r=n.startLexicalEnvironment,i=n.endLexicalEnvironment,a=n.hoistVariableDeclaration,o=n.getCompilerOptions(),s=n.getEmitResolver(),c=n.getEmitHost(),l=n.onSubstituteNode,u=n.onEmitNode;n.onSubstituteNode=Ne,n.onEmitNode=Ie,n.enableSubstitution(79),n.enableSubstitution(292),n.enableSubstitution(219),n.enableSubstitution(229),n.enableEmitNotification(300);var d,p,f,m,_,g,y,h=[],v=[],b=[],E=[],T=[];return e.chainBundle(n,S);function S(r){if(r.isDeclarationFile||!(e.isEffectiveExternalModule(r,o)||4194304&r.transformFlags))return r;var i=e.getOriginalNodeId(r);d=r,g=r,p=h[i]=e.collectExternalModuleInfo(n,r,s,o),f=t.createUniqueName("exports"),b[i]=f,m=T[i]=t.createUniqueName("context");var a=x(p.externalImports),l=L(r,a),u=t.createFunctionExpression(void 0,void 0,void 0,void 0,[t.createParameterDeclaration(void 0,void 0,void 0,f),t.createParameterDeclaration(void 0,void 0,void 0,m)],void 0,l),v=e.tryGetModuleNameFromFile(t,r,c,o),S=t.createArrayLiteralExpression(e.map(a,function(e){return e.name})),D=e.setEmitFlags(t.updateSourceFile(r,e.setTextRange(t.createNodeArray([t.createExpressionStatement(t.createCallExpression(t.createPropertyAccessExpression(t.createIdentifier("System"),"register"),void 0,v?[v,S,u]:[S,u]))]),r.statements)),1024);return e.outFile(o)||e.moveEmitHelpers(D,l,function(e){return!e.scoped}),y&&(E[i]=y,y=void 0),d=void 0,p=void 0,f=void 0,m=void 0,_=void 0,g=void 0,D}function x(n){for(var r=new e.Map,i=[],a=0,l=n;a<l.length;a++){var u=l[a],p=e.getExternalModuleNameLiteral(t,u,d,c,s,o);if(p){var f=p.text,m=r.get(f);void 0!==m?i[m].externalImports.push(u):(r.set(f,i.length),i.push({name:p,externalImports:[u]}))}}return i}function L(n,a){var s=[];r();var c=e.getStrictOptionValue(o,"alwaysStrict")||!o.noImplicitUseStrict&&e.isExternalModule(d),l=t.copyPrologue(n.statements,s,c,I);s.push(t.createVariableStatement(void 0,t.createVariableDeclarationList([t.createVariableDeclaration("__moduleName",void 0,void 0,t.createLogicalAnd(m,t.createPropertyAccessExpression(m,"id")))]))),e.visitNode(p.externalHelpersImportDeclaration,I,e.isStatement);var u=e.visitNodes(n.statements,I,e.isStatement,l);e.addRange(s,_),e.insertStatementsAfterStandardPrologue(s,i());var f=D(s),g=1048576&n.transformFlags?t.createModifiersFromModifierFlags(256):void 0,y=t.createObjectLiteralExpression([t.createPropertyAssignment("setters",C(f,a)),t.createPropertyAssignment("execute",t.createFunctionExpression(g,void 0,void 0,void 0,[],void 0,t.createBlock(u,!0)))],!0);return s.push(t.createReturnStatement(y)),t.createBlock(s,!0)}function D(e){if(p.hasExportStarsToExportValues){if(!p.exportedNames&&0===p.exportSpecifiers.size){for(var n=!1,r=0,i=p.externalImports;r<i.length;r++){var a=i[r];if(270===a.kind&&a.exportClause){n=!0;break}}if(!n){var o=A(void 0);return e.push(o),o.name}}var s=[];if(p.exportedNames)for(var c=0,l=p.exportedNames;c<l.length;c++){var u=l[c];"default"!==u.escapedText&&s.push(t.createPropertyAssignment(t.createStringLiteralFromNode(u),t.createTrue()))}var d=t.createUniqueName("exportedNames");e.push(t.createVariableStatement(void 0,t.createVariableDeclarationList([t.createVariableDeclaration(d,void 0,void 0,t.createObjectLiteralExpression(s,!0))])));var f=A(d);return e.push(f),f.name}}function A(n){var r=t.createUniqueName("exportStar"),i=t.createIdentifier("m"),a=t.createIdentifier("n"),o=t.createIdentifier("exports"),s=t.createStrictInequality(a,t.createStringLiteral("default"));return n&&(s=t.createLogicalAnd(s,t.createLogicalNot(t.createCallExpression(t.createPropertyAccessExpression(n,"hasOwnProperty"),void 0,[a])))),t.createFunctionDeclaration(void 0,void 0,void 0,r,void 0,[t.createParameterDeclaration(void 0,void 0,void 0,i)],void 0,t.createBlock([t.createVariableStatement(void 0,t.createVariableDeclarationList([t.createVariableDeclaration(o,void 0,void 0,t.createObjectLiteralExpression([]))])),t.createForInStatement(t.createVariableDeclarationList([t.createVariableDeclaration(a)]),i,t.createBlock([e.setEmitFlags(t.createIfStatement(s,t.createExpressionStatement(t.createAssignment(t.createElementAccessExpression(o,a),t.createElementAccessExpression(i,a)))),1)])),t.createExpressionStatement(t.createCallExpression(f,void 0,[o]))],!0))}function C(n,r){for(var i=[],a=0,o=r;a<o.length;a++){for(var s=o[a],c=e.forEach(s.externalImports,function(n){return e.getLocalNameForExternalImport(t,n,d)}),l=c?t.getGeneratedNameForNode(c):t.createUniqueName(""),u=[],p=0,m=s.externalImports;p<m.length;p++){var _=m[p],g=e.getLocalNameForExternalImport(t,_,d);switch(_.kind){case 264:if(!_.importClause)break;case 263:e.Debug.assert(void 0!==g),u.push(t.createExpressionStatement(t.createAssignment(g,l)));break;case 270:if(e.Debug.assert(void 0!==g),_.exportClause)if(e.isNamedExports(_.exportClause)){for(var y=[],h=0,v=_.exportClause.elements;h<v.length;h++){var b=v[h];y.push(t.createPropertyAssignment(t.createStringLiteral(e.idText(b.name)),t.createElementAccessExpression(l,t.createStringLiteral(e.idText(b.propertyName||b.name)))))}u.push(t.createExpressionStatement(t.createCallExpression(f,void 0,[t.createObjectLiteralExpression(y,!0)])))}else u.push(t.createExpressionStatement(t.createCallExpression(f,void 0,[t.createStringLiteral(e.idText(_.exportClause.name)),l])));else u.push(t.createExpressionStatement(t.createCallExpression(n,void 0,[l])));break}}i.push(t.createFunctionExpression(void 0,void 0,void 0,void 0,[t.createParameterDeclaration(void 0,void 0,void 0,l)],void 0,t.createBlock(u,!0)))}return t.createArrayLiteralExpression(i,!0)}function I(e){switch(e.kind){case 264:return N(e);case 263:return R(e);case 270:return k(e);case 269:return O(e);default:return ne(e)}}function N(n){var r;if(n.importClause&&a(e.getLocalNameForExternalImport(t,n,d)),H(n)){var i=e.getOriginalNodeId(n);v[i]=z(v[i],n)}else r=z(r,n);return e.singleOrMany(r)}function k(n){e.Debug.assertIsDefined(n)}function R(n){var r;if(e.Debug.assert(e.isExternalModuleImportEqualsDeclaration(n),"import= for internal module references should be handled in an earlier transformer."),a(e.getLocalNameForExternalImport(t,n,d)),H(n)){var i=e.getOriginalNodeId(n);v[i]=J(v[i],n)}else r=J(r,n);return e.singleOrMany(r)}function O(n){if(!n.isExportEquals){var r=e.visitNode(n.expression,ve,e.isExpression),i=n.original;if(!i||!H(i))return $(t.createIdentifier("default"),r,!0);var a=e.getOriginalNodeId(n);v[a]=Z(v[a],t.createIdentifier("default"),r,!0)}}function w(r){if(_=e.hasSyntacticModifier(r,1)?e.append(_,t.updateFunctionDeclaration(r,r.decorators,e.visitNodes(r.modifiers,Ce,e.isModifier),r.asteriskToken,t.getDeclarationName(r,!0,!0),void 0,e.visitNodes(r.parameters,ve,e.isParameterDeclaration),void 0,e.visitNode(r.body,ve,e.isBlock))):e.append(_,e.visitEachChild(r,ve,n)),H(r)){var i=e.getOriginalNodeId(r);v[i]=Y(v[i],r)}else _=Y(_,r)}function P(n){var r,i=t.getLocalName(n);if(a(i),r=e.append(r,e.setTextRange(t.createExpressionStatement(t.createAssignment(i,e.setTextRange(t.createClassExpression(e.visitNodes(n.decorators,ve,e.isDecorator),void 0,n.name,void 0,e.visitNodes(n.heritageClauses,ve,e.isHeritageClause),e.visitNodes(n.members,ve,e.isClassElement)),n))),n)),H(n)){var o=e.getOriginalNodeId(n);v[o]=Y(v[o],n)}else r=Y(r,n);return e.singleOrMany(r)}function M(n){if(!G(n.declarationList))return e.visitNode(n,ve,e.isStatement);for(var r,i,a=e.hasSyntacticModifier(n,1),o=H(n),s=0,c=n.declarationList.declarations;s<c.length;s++){var l=c[s];l.initializer?r=e.append(r,B(l,a&&!o)):F(l)}if(r&&(i=e.append(i,e.setTextRange(t.createExpressionStatement(t.inlineExpressions(r)),n))),o){var u=e.getOriginalNodeId(n);v[u]=q(v[u],n,a)}else i=q(i,n,!1);return e.singleOrMany(i)}function F(n){if(e.isBindingPattern(n.name))for(var r=0,i=n.name.elements;r<i.length;r++){var o=i[r];e.isOmittedExpression(o)||F(o)}else a(t.cloneNode(n.name))}function G(n){return 0===(2097152&e.getEmitFlags(n))&&(300===g.kind||0===(3&e.getOriginalNode(n).flags))}function B(t,r){var i=r?U:V;return e.isBindingPattern(t.name)?e.flattenDestructuringAssignment(t,ve,n,0,!1,i):t.initializer?i(t.name,e.visitNode(t.initializer,ve,e.isExpression)):t.name}function U(e,n,t){return j(e,n,t,!0)}function V(e,n,t){return j(e,n,t,!1)}function j(n,r,i,o){return a(t.cloneNode(n)),o?ee(n,Ge(e.setTextRange(t.createAssignment(n,r),i))):Ge(e.setTextRange(t.createAssignment(n,r),i))}function K(n){if(H(n)&&235===n.original.kind){var t=e.getOriginalNodeId(n),r=e.hasSyntacticModifier(n.original,1);v[t]=q(v[t],n.original,r)}return n}function H(n){return 0!==(4194304&e.getEmitFlags(n))}function W(n){var t=e.getOriginalNodeId(n),r=v[t];if(r)return delete v[t],e.append(r,n);var i=e.getOriginalNode(n);return e.isModuleOrEnumDeclaration(i)?e.append(Q(r,i),n):n}function z(e,n){if(p.exportEquals)return e;var t=n.importClause;if(!t)return e;t.name&&(e=Q(e,t));var r=t.namedBindings;if(r)switch(r.kind){case 266:e=Q(e,r);break;case 267:for(var i=0,a=r.elements;i<a.length;i++){var o=a[i];e=Q(e,o)}break}return e}function J(e,n){return p.exportEquals?e:Q(e,n)}function q(e,n,t){if(p.exportEquals)return e;for(var r=0,i=n.declarationList.declarations;r<i.length;r++){var a=i[r];(a.initializer||t)&&(e=X(e,a,t))}return e}function X(n,r,i){if(p.exportEquals)return n;if(e.isBindingPattern(r.name))for(var a=0,o=r.name.elements;a<o.length;a++){var s=o[a];e.isOmittedExpression(s)||(n=X(n,s,i))}else if(!e.isGeneratedIdentifier(r.name)){var c=void 0;i&&(n=Z(n,r.name,t.getLocalName(r)),c=e.idText(r.name)),n=Q(n,r,c)}return n}function Y(n,r){if(p.exportEquals)return n;var i;if(e.hasSyntacticModifier(r,1)){var a=e.hasSyntacticModifier(r,512)?t.createStringLiteral("default"):r.name;n=Z(n,a,t.getLocalName(r)),i=e.getTextOfIdentifierOrLiteral(a)}return r.name&&(n=Q(n,r,i)),n}function Q(n,r,i){if(p.exportEquals)return n;var a=t.getDeclarationName(r),o=p.exportSpecifiers.get(e.idText(a));if(o)for(var s=0,c=o;s<c.length;s++){var l=c[s];l.name.escapedText!==i&&(n=Z(n,l.name,a))}return n}function Z(n,t,r,i){return n=e.append(n,$(t,r,i)),n}function $(n,r,i){var a=t.createExpressionStatement(ee(n,r));return e.startOnNewLine(a),i||e.setEmitFlags(a,1536),a}function ee(n,r){var i=e.isIdentifier(n)?t.createStringLiteralFromNode(n):n;return e.setEmitFlags(r,1536|e.getEmitFlags(r)),e.setCommentRange(t.createCallExpression(f,void 0,[i,r]),r)}function ne(e){switch(e.kind){case 235:return M(e);case 254:return w(e);case 255:return P(e);case 240:return te(e,!0);case 241:return re(e);case 242:return ie(e);case 238:return se(e);case 239:return ce(e);case 248:return le(e);case 246:return ue(e);case 247:return de(e);case 261:return pe(e);case 287:return fe(e);case 288:return me(e);case 250:return _e(e);case 290:return ge(e);case 233:return ye(e);case 347:return K(e);case 348:return W(e);default:return ve(e)}}function te(r,i){var a=g;return g=r,r=t.updateForStatement(r,e.visitNode(r.initializer,i?oe:be,e.isForInitializer),e.visitNode(r.condition,ve,e.isExpression),e.visitNode(r.incrementor,be,e.isExpression),e.visitIterationBody(r.statement,i?ne:ve,n)),g=a,r}function re(r){var i=g;return g=r,r=t.updateForInStatement(r,oe(r.initializer),e.visitNode(r.expression,ve,e.isExpression),e.visitIterationBody(r.statement,ne,n)),g=i,r}function ie(r){var i=g;return g=r,r=t.updateForOfStatement(r,r.awaitModifier,oe(r.initializer),e.visitNode(r.expression,ve,e.isExpression),e.visitIterationBody(r.statement,ne,n)),g=i,r}function ae(n){return e.isVariableDeclarationList(n)&&G(n)}function oe(n){if(ae(n)){for(var r=void 0,i=0,a=n.declarations;i<a.length;i++){var o=a[i];r=e.append(r,B(o,!1)),o.initializer||F(o)}return r?t.inlineExpressions(r):t.createOmittedExpression()}return e.visitNode(n,be,e.isExpression)}function se(r){return t.updateDoStatement(r,e.visitIterationBody(r.statement,ne,n),e.visitNode(r.expression,ve,e.isExpression))}function ce(r){return t.updateWhileStatement(r,e.visitNode(r.expression,ve,e.isExpression),e.visitIterationBody(r.statement,ne,n))}function le(n){return t.updateLabeledStatement(n,n.label,e.visitNode(n.statement,ne,e.isStatement,t.liftToBlock))}function ue(n){return t.updateWithStatement(n,e.visitNode(n.expression,ve,e.isExpression),e.visitNode(n.statement,ne,e.isStatement,t.liftToBlock))}function de(n){return t.updateSwitchStatement(n,e.visitNode(n.expression,ve,e.isExpression),e.visitNode(n.caseBlock,ne,e.isCaseBlock))}function pe(n){var r=g;return g=n,n=t.updateCaseBlock(n,e.visitNodes(n.clauses,ne,e.isCaseOrDefaultClause)),g=r,n}function fe(n){return t.updateCaseClause(n,e.visitNode(n.expression,ve,e.isExpression),e.visitNodes(n.statements,ne,e.isStatement))}function me(t){return e.visitEachChild(t,ne,n)}function _e(t){return e.visitEachChild(t,ne,n)}function ge(n){var r=g;return g=n,n=t.updateCatchClause(n,n.variableDeclaration,e.visitNode(n.block,ne,e.isBlock)),g=r,n}function ye(t){var r=g;return g=t,t=e.visitEachChild(t,ne,n),g=r,t}function he(t,r){if(!(71305216&t.transformFlags))return t;switch(t.kind){case 240:return te(t,!1);case 236:return Ee(t);case 210:return Te(t,r);case 345:return Se(t,r);case 219:if(e.isDestructuringAssignment(t))return Le(t,r);break;case 206:if(e.isImportCall(t))return xe(t);break;case 217:case 218:return Ae(t,r)}return e.visitEachChild(t,ve,n)}function ve(e){return he(e,!1)}function be(e){return he(e,!0)}function Ee(n){return t.updateExpressionStatement(n,e.visitNode(n.expression,be,e.isExpression))}function Te(n,r){return t.updateParenthesizedExpression(n,e.visitNode(n.expression,r?be:ve,e.isExpression))}function Se(n,r){return t.updatePartiallyEmittedExpression(n,e.visitNode(n.expression,r?be:ve,e.isExpression))}function xe(n){var r=e.getExternalModuleNameLiteral(t,n,d,c,s,o),i=e.visitNode(e.firstOrUndefined(n.arguments),ve),a=!r||i&&e.isStringLiteral(i)&&i.text===r.text?i:r;return t.createCallExpression(t.createPropertyAccessExpression(m,t.createIdentifier("import")),void 0,a?[a]:[])}function Le(t,r){return De(t.left)?e.flattenDestructuringAssignment(t,ve,n,0,!r):e.visitEachChild(t,ve,n)}function De(n){if(e.isAssignmentExpression(n,!0))return De(n.left);if(e.isSpreadElement(n))return De(n.expression);if(e.isObjectLiteralExpression(n))return e.some(n.properties,De);if(e.isArrayLiteralExpression(n))return e.some(n.elements,De);if(e.isShorthandPropertyAssignment(n))return De(n.name);if(e.isPropertyAssignment(n))return De(n.initializer);if(e.isIdentifier(n)){var t=s.getReferencedExportContainer(n);return void 0!==t&&300===t.kind}return!1}function Ae(r,i){if((45===r.operator||46===r.operator)&&e.isIdentifier(r.operand)&&!e.isGeneratedIdentifier(r.operand)&&!e.isLocalName(r.operand)&&!e.isDeclarationNameOfEnumOrNamespace(r.operand)){var o=Fe(r.operand);if(o){var s=void 0,c=e.visitNode(r.operand,ve,e.isExpression);e.isPrefixUnaryExpression(r)?c=t.updatePrefixUnaryExpression(r,c):(c=t.updatePostfixUnaryExpression(r,c),i||(s=t.createTempVariable(a),c=t.createAssignment(s,c),e.setTextRange(c,r)),c=t.createComma(c,t.cloneNode(r.operand)),e.setTextRange(c,r));for(var l=0,u=o;l<u.length;l++){var d=u[l];c=ee(d,Ge(c))}return s&&(c=t.createComma(c,s),e.setTextRange(c,r)),c}}return e.visitEachChild(r,ve,n)}function Ce(e){switch(e.kind){case 93:case 88:return}return e}function Ie(n,t,r){if(300===t.kind){var i=e.getOriginalNodeId(t);d=t,p=h[i],f=b[i],y=E[i],m=T[i],y&&delete E[i],u(n,t,r),d=void 0,p=void 0,f=void 0,m=void 0,y=void 0}else u(n,t,r)}function Ne(e,n){return n=l(e,n),Be(n)?n:1===e?Oe(n):4===e?ke(n):n}function ke(e){switch(e.kind){case 292:return Re(e)}return e}function Re(n){var r,i,a=n.name;if(!e.isGeneratedIdentifier(a)&&!e.isLocalName(a)){var o=s.getReferencedImportDeclaration(a);if(o){if(e.isImportClause(o))return e.setTextRange(t.createPropertyAssignment(t.cloneNode(a),t.createPropertyAccessExpression(t.getGeneratedNameForNode(o.parent),t.createIdentifier("default"))),n);if(e.isImportSpecifier(o))return e.setTextRange(t.createPropertyAssignment(t.cloneNode(a),t.createPropertyAccessExpression(t.getGeneratedNameForNode((null===(i=null===(r=o.parent)||void 0===r?void 0:r.parent)||void 0===i?void 0:i.parent)||o),t.cloneNode(o.propertyName||o.name))),n)}}return n}function Oe(e){switch(e.kind){case 79:return we(e);case 219:return Pe(e);case 229:return Me(e)}return e}function we(n){var r,i;if(4096&e.getEmitFlags(n)){var a=e.getExternalHelpersModuleName(d);return a?t.createPropertyAccessExpression(a,n):n}if(!e.isGeneratedIdentifier(n)&&!e.isLocalName(n)){var o=s.getReferencedImportDeclaration(n);if(o){if(e.isImportClause(o))return e.setTextRange(t.createPropertyAccessExpression(t.getGeneratedNameForNode(o.parent),t.createIdentifier("default")),n);if(e.isImportSpecifier(o))return e.setTextRange(t.createPropertyAccessExpression(t.getGeneratedNameForNode((null===(i=null===(r=o.parent)||void 0===r?void 0:r.parent)||void 0===i?void 0:i.parent)||o),t.cloneNode(o.propertyName||o.name)),n)}}return n}function Pe(n){if(e.isAssignmentOperator(n.operatorToken.kind)&&e.isIdentifier(n.left)&&!e.isGeneratedIdentifier(n.left)&&!e.isLocalName(n.left)&&!e.isDeclarationNameOfEnumOrNamespace(n.left)){var t=Fe(n.left);if(t){for(var r=n,i=0,a=t;i<a.length;i++){var o=a[i];r=ee(o,Ge(r))}return r}}return n}function Me(n){return e.isImportMeta(n)?t.createPropertyAccessExpression(m,t.createIdentifier("meta")):n}function Fe(n){var r;if(!e.isGeneratedIdentifier(n)){var i=s.getReferencedImportDeclaration(n)||s.getReferencedValueDeclaration(n);if(i){var a=s.getReferencedExportContainer(n,!1);a&&300===a.kind&&(r=e.append(r,t.getDeclarationName(i))),r=e.addRange(r,p&&p.exportedBindings[e.getOriginalNodeId(i)])}}return r}function Ge(n){return void 0===y&&(y=[]),y[e.getNodeId(n)]=!0,n}function Be(e){return y&&e.id&&y[e.id]}}e.transformSystemModule=n}(d||(d={})),function(e){function n(n){var t,r=n.factory,i=n.getEmitHelperFactory,o=n.getCompilerOptions(),s=n.onEmitNode,c=n.onSubstituteNode;return n.onEmitNode=m,n.onSubstituteNode=_,n.enableEmitNotification(300),n.enableSubstitution(79),e.chainBundle(n,l);function l(n){if(n.isDeclarationFile)return n;if(e.isExternalModule(n)||o.isolatedModules){var t=u(n);return!e.isExternalModule(n)||e.some(t.statements,e.isExternalModuleIndicator)?t:r.updateSourceFile(t,e.setTextRange(r.createNodeArray(a(a([],t.statements,!0),[e.createEmptyExports(r)],!1)),t.statements))}return n}function u(t){var a=e.createExternalHelpersImportDeclarationIfNeeded(r,i(),t,o);if(a){var s=[],c=r.copyPrologue(t.statements,s);return e.append(s,a),e.addRange(s,e.visitNodes(t.statements,d,e.isStatement,c)),r.updateSourceFile(t,e.setTextRange(r.createNodeArray(s),t.statements))}return e.visitEachChild(t,d,n)}function d(e){switch(e.kind){case 263:return;case 269:return p(e);case 270:var n=e;return f(n)}return e}function p(e){return e.isExportEquals?void 0:e}function f(n){if(void 0!==o.module&&o.module>e.ModuleKind.ES2015)return n;if(!n.exportClause||!e.isNamespaceExport(n.exportClause)||!n.moduleSpecifier)return n;var t=n.exportClause.name,i=r.getGeneratedNameForNode(t),a=r.createImportDeclaration(void 0,void 0,r.createImportClause(!1,void 0,r.createNamespaceImport(i)),n.moduleSpecifier);e.setOriginalNode(a,n.exportClause);var s=e.isExportNamespaceAsDefaultDeclaration(n)?r.createExportDefault(i):r.createExportDeclaration(void 0,void 0,!1,r.createNamedExports([r.createExportSpecifier(i,t)]));return e.setOriginalNode(s,n),[a,s]}function m(n,r,i){e.isSourceFile(r)?((e.isExternalModule(r)||o.isolatedModules)&&o.importHelpers&&(t=new e.Map),s(n,r,i),t=void 0):s(n,r,i)}function _(n,r){return r=c(n,r),t&&e.isIdentifier(r)&&4096&e.getEmitFlags(r)?g(r):r}function g(n){var i=e.idText(n),a=t.get(i);return a||t.set(i,a=r.createUniqueName(i,48)),a}}e.transformECMAScriptModule=n}(d||(d={})),function(e){function n(n){return e.isVariableDeclaration(n)||e.isPropertyDeclaration(n)||e.isPropertySignature(n)||e.isBindingElement(n)||e.isSetAccessor(n)||e.isGetAccessor(n)||e.isConstructSignatureDeclaration(n)||e.isCallSignatureDeclaration(n)||e.isMethodDeclaration(n)||e.isMethodSignature(n)||e.isFunctionDeclaration(n)||e.isParameter(n)||e.isTypeParameterDeclaration(n)||e.isExpressionWithTypeArguments(n)||e.isImportEqualsDeclaration(n)||e.isTypeAliasDeclaration(n)||e.isConstructorDeclaration(n)||e.isIndexSignatureDeclaration(n)||e.isPropertyAccessExpression(n)||e.isJSDocTypeAlias(n)}function t(n){return e.isSetAccessor(n)||e.isGetAccessor(n)?t:e.isMethodSignature(n)||e.isMethodDeclaration(n)?a:r(n);function t(e){var t=i(e);return void 0!==t?{diagnosticMessage:t,errorNode:n,typeName:n.name}:void 0}function i(t){return e.isStatic(n)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_private_name_1:255===n.parent.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_private_name_1:t.errorModuleName?e.Diagnostics.Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Property_0_of_exported_interface_has_or_is_using_private_name_1}function a(e){var t=o(e);return void 0!==t?{diagnosticMessage:t,errorNode:n,typeName:n.name}:void 0}function o(t){return e.isStatic(n)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_static_method_0_of_exported_class_has_or_is_using_private_name_1:255===n.parent.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Public_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_method_0_of_exported_class_has_or_is_using_private_name_1:t.errorModuleName?e.Diagnostics.Method_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Method_0_of_exported_interface_has_or_is_using_private_name_1}}function r(n){return e.isVariableDeclaration(n)||e.isPropertyDeclaration(n)||e.isPropertySignature(n)||e.isPropertyAccessExpression(n)||e.isBindingElement(n)||e.isConstructorDeclaration(n)?r:e.isSetAccessor(n)||e.isGetAccessor(n)?i:e.isConstructSignatureDeclaration(n)||e.isCallSignatureDeclaration(n)||e.isMethodDeclaration(n)||e.isMethodSignature(n)||e.isFunctionDeclaration(n)||e.isIndexSignatureDeclaration(n)?a:e.isParameter(n)?e.isParameterPropertyDeclaration(n,n.parent)&&e.hasSyntacticModifier(n.parent,8)?r:o:e.isTypeParameterDeclaration(n)?c:e.isExpressionWithTypeArguments(n)?l:e.isImportEqualsDeclaration(n)?u:e.isTypeAliasDeclaration(n)||e.isJSDocTypeAlias(n)?d:e.Debug.assertNever(n,"Attempted to set a declaration diagnostic context for unhandled node kind: "+e.SyntaxKind[n.kind]);function t(t){return 252===n.kind||201===n.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Exported_variable_0_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Exported_variable_0_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Exported_variable_0_has_or_is_using_private_name_1:165===n.kind||204===n.kind||164===n.kind||162===n.kind&&e.hasSyntacticModifier(n.parent,8)?e.isStatic(n)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_private_name_1:255===n.parent.kind||162===n.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_private_name_1:t.errorModuleName?e.Diagnostics.Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Property_0_of_exported_interface_has_or_is_using_private_name_1:void 0}function r(e){var r=t(e);return void 0!==r?{diagnosticMessage:r,errorNode:n,typeName:n.name}:void 0}function i(t){var r;return r=171===n.kind?e.isStatic(n)?t.errorModuleName?e.Diagnostics.Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_private_name_1:t.errorModuleName?e.Diagnostics.Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_private_name_1:e.isStatic(n)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_private_name_1:t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Return_type_of_public_getter_0_from_exported_class_has_or_is_using_private_name_1,{diagnosticMessage:r,errorNode:n.name,typeName:n.name}}function a(t){var r;switch(n.kind){case 173:r=t.errorModuleName?e.Diagnostics.Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_0;break;case 172:r=t.errorModuleName?e.Diagnostics.Return_type_of_call_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_call_signature_from_exported_interface_has_or_is_using_private_name_0;break;case 174:r=t.errorModuleName?e.Diagnostics.Return_type_of_index_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_index_signature_from_exported_interface_has_or_is_using_private_name_0;break;case 167:case 166:r=e.isStatic(n)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:e.Diagnostics.Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_public_static_method_from_exported_class_has_or_is_using_private_name_0:255===n.parent.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:e.Diagnostics.Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_public_method_from_exported_class_has_or_is_using_private_name_0:t.errorModuleName?e.Diagnostics.Return_type_of_method_from_exported_interface_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_method_from_exported_interface_has_or_is_using_private_name_0;break;case 254:r=t.errorModuleName?2===t.accessibility?e.Diagnostics.Return_type_of_exported_function_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named:e.Diagnostics.Return_type_of_exported_function_has_or_is_using_name_0_from_private_module_1:e.Diagnostics.Return_type_of_exported_function_has_or_is_using_private_name_0;break;default:return e.Debug.fail("This is unknown kind for signature: "+n.kind)}return{diagnosticMessage:r,errorNode:n.name||n}}function o(e){var t=s(e);return void 0!==t?{diagnosticMessage:t,errorNode:n,typeName:n.name}:void 0}function s(t){switch(n.parent.kind){case 169:return t.errorModuleName?2===t.accessibility?e.Diagnostics.Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_constructor_from_exported_class_has_or_is_using_private_name_1;case 173:case 178:return t.errorModuleName?e.Diagnostics.Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1;case 172:return t.errorModuleName?e.Diagnostics.Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1;case 174:return t.errorModuleName?e.Diagnostics.Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_private_name_1;case 167:case 166:return e.isStatic(n.parent)?t.errorModuleName?2===t.accessibility?e.Diagnostics.Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1:255===n.parent.parent.kind?t.errorModuleName?2===t.accessibility?e.Diagnostics.Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1:t.errorModuleName?e.Diagnostics.Parameter_0_of_method_from_exported_interface_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1;case 254:case 177:return t.errorModuleName?2===t.accessibility?e.Diagnostics.Parameter_0_of_exported_function_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Parameter_0_of_exported_function_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_exported_function_has_or_is_using_private_name_1;case 171:case 170:return t.errorModuleName?2===t.accessibility?e.Diagnostics.Parameter_0_of_accessor_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named:e.Diagnostics.Parameter_0_of_accessor_has_or_is_using_name_1_from_private_module_2:e.Diagnostics.Parameter_0_of_accessor_has_or_is_using_private_name_1;default:return e.Debug.fail("Unknown parent for parameter: "+e.SyntaxKind[n.parent.kind])}}function c(){var t;switch(n.parent.kind){case 255:t=e.Diagnostics.Type_parameter_0_of_exported_class_has_or_is_using_private_name_1;break;case 256:t=e.Diagnostics.Type_parameter_0_of_exported_interface_has_or_is_using_private_name_1;break;case 193:t=e.Diagnostics.Type_parameter_0_of_exported_mapped_object_type_is_using_private_name_1;break;case 178:case 173:t=e.Diagnostics.Type_parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1;break;case 172:t=e.Diagnostics.Type_parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1;break;case 167:case 166:t=e.isStatic(n.parent)?e.Diagnostics.Type_parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1:255===n.parent.parent.kind?e.Diagnostics.Type_parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1:e.Diagnostics.Type_parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1;break;case 177:case 254:t=e.Diagnostics.Type_parameter_0_of_exported_function_has_or_is_using_private_name_1;break;case 257:t=e.Diagnostics.Type_parameter_0_of_exported_type_alias_has_or_is_using_private_name_1;break;default:return e.Debug.fail("This is unknown parent for type parameter: "+n.parent.kind)}return{diagnosticMessage:t,errorNode:n,typeName:n.name}}function l(){var t;return t=e.isClassDeclaration(n.parent.parent)?e.isHeritageClause(n.parent)&&117===n.parent.token?e.Diagnostics.Implements_clause_of_exported_class_0_has_or_is_using_private_name_1:n.parent.parent.name?e.Diagnostics.extends_clause_of_exported_class_0_has_or_is_using_private_name_1:e.Diagnostics.extends_clause_of_exported_class_has_or_is_using_private_name_0:e.Diagnostics.extends_clause_of_exported_interface_0_has_or_is_using_private_name_1,{diagnosticMessage:t,errorNode:n,typeName:e.getNameOfDeclaration(n.parent.parent)}}function u(){return{diagnosticMessage:e.Diagnostics.Import_declaration_0_is_using_private_name_1,errorNode:n,typeName:n.name}}function d(t){return{diagnosticMessage:t.errorModuleName?e.Diagnostics.Exported_type_alias_0_has_or_is_using_private_name_1_from_module_2:e.Diagnostics.Exported_type_alias_0_has_or_is_using_private_name_1,errorNode:e.isJSDocTypeAlias(n)?e.Debug.checkDefined(n.typeExpression):n.type,typeName:e.isJSDocTypeAlias(n)?e.getNameOfDeclaration(n):n.name}}}e.canProduceDiagnostics=n,e.createGetSymbolAccessibilityDiagnosticForNodeName=t,e.createGetSymbolAccessibilityDiagnosticForNode=r}(d||(d={})),function(e){function n(n,t,r){var i=n.getCompilerOptions(),a=e.transformNodes(t,n,e.factory,i,r?[r]:e.filter(n.getSourceFiles(),e.isSourceFileNotJson),[o],!1);return a.diagnostics}function t(n,t){var r=t.text.substring(n.pos,n.end);return e.stringContains(r,"@internal")}function r(n,r){var i=e.getParseTreeNode(n);if(i&&162===i.kind){var a=i.parent.parameters.indexOf(i),o=a>0?i.parent.parameters[a-1]:void 0,s=r.text,c=o?e.concatenate(e.getTrailingCommentRanges(s,e.skipTrivia(s,o.end+1,!1,!0)),e.getLeadingCommentRanges(s,n.pos)):e.getTrailingCommentRanges(s,e.skipTrivia(s,n.pos,!1,!0));return c&&c.length&&t(e.last(c),r)}var l=i&&e.getLeadingCommentRangesOfNode(i,r);return!!e.forEach(l,function(e){return t(e,r)})}e.getDeclarationDiagnostics=n,e.isInternalDeclaration=r;var i=531469;function o(n){var t,o,m,_,g,y,h,v,b,E,T,S,x=function(){return e.Debug.fail("Diagnostic emitted without context")},L=x,D=!0,A=!1,C=!1,I=!1,N=!1,k=n.factory,R=n.getEmitHost(),O={trackSymbol:j,reportInaccessibleThisError:J,reportInaccessibleUniqueSymbolError:W,reportCyclicStructureError:z,reportPrivateInBaseOfClassExpression:K,reportLikelyUnsafeImportRequiredError:q,reportTruncationError:X,moduleResolverHost:R,trackReferencedAmbientModule:B,trackExternalModuleSymbolOfImportTypeNode:V,reportNonlocalAugmentation:Y,reportNonSerializableProperty:Q},w=n.getEmitResolver(),P=n.getCompilerOptions(),M=P.noResolve,F=P.stripInternal;return $;function G(n){if(n){o=o||new e.Set;for(var t=0,r=n;t<r.length;t++){var i=r[t];o.add(i)}}}function B(n,t){var r=w.getTypeReferenceDirectivesForSymbol(t,67108863);if(e.length(r))return G(r);var i=e.getSourceFileOfNode(n);E.set(e.getOriginalNodeId(i),i)}function U(t){if(0===t.accessibility){if(t&&t.aliasesToMakeVisible)if(m)for(var r=0,i=t.aliasesToMakeVisible;r<i.length;r++){var a=i[r];e.pushIfUnique(m,a)}else m=t.aliasesToMakeVisible}else{var o=L(t);if(o)return o.typeName?n.addDiagnostic(e.createDiagnosticForNode(t.errorNode||o.errorNode,o.diagnosticMessage,e.getTextOfNode(o.typeName),t.errorSymbolName,t.errorModuleName)):n.addDiagnostic(e.createDiagnosticForNode(t.errorNode||o.errorNode,o.diagnosticMessage,t.errorSymbolName,t.errorModuleName)),!0}return!1}function V(e){A||(y||(y=[])).push(e)}function j(e,n,t){if(262144&e.flags)return!1;var r=U(w.isSymbolAccessible(e,n,t,!0));return G(w.getTypeReferenceDirectivesForSymbol(e,t)),r}function K(t){(h||v)&&n.addDiagnostic(e.createDiagnosticForNode(h||v,e.Diagnostics.Property_0_of_exported_class_expression_may_not_be_private_or_protected,t))}function H(){return h?e.declarationNameToString(h):v&&e.getNameOfDeclaration(v)?e.declarationNameToString(e.getNameOfDeclaration(v)):v&&e.isExportAssignment(v)?v.isExportEquals?"export=":"default":"(Missing)"}function W(){(h||v)&&n.addDiagnostic(e.createDiagnosticForNode(h||v,e.Diagnostics.The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary,H(),"unique symbol"))}function z(){(h||v)&&n.addDiagnostic(e.createDiagnosticForNode(h||v,e.Diagnostics.The_inferred_type_of_0_references_a_type_with_a_cyclic_structure_which_cannot_be_trivially_serialized_A_type_annotation_is_necessary,H()))}function J(){(h||v)&&n.addDiagnostic(e.createDiagnosticForNode(h||v,e.Diagnostics.The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary,H(),"this"))}function q(t){(h||v)&&n.addDiagnostic(e.createDiagnosticForNode(h||v,e.Diagnostics.The_inferred_type_of_0_cannot_be_named_without_a_reference_to_1_This_is_likely_not_portable_A_type_annotation_is_necessary,H(),t))}function X(){(h||v)&&n.addDiagnostic(e.createDiagnosticForNode(h||v,e.Diagnostics.The_inferred_type_of_this_node_exceeds_the_maximum_length_the_compiler_will_serialize_An_explicit_type_annotation_is_needed))}function Y(t,r,i){var a,o=null===(a=r.declarations)||void 0===a?void 0:a.find(function(n){return e.getSourceFileOfNode(n)===t}),s=e.filter(i.declarations,function(n){return e.getSourceFileOfNode(n)!==t});if(s)for(var c=0,l=s;c<l.length;c++){var u=l[c];n.addDiagnostic(e.addRelatedInfo(e.createDiagnosticForNode(u,e.Diagnostics.Declaration_augments_declaration_in_another_file_This_cannot_be_serialized),e.createDiagnosticForNode(o,e.Diagnostics.This_is_the_declaration_being_augmented_Consider_moving_the_augmenting_declaration_into_the_same_file)))}}function Q(t){(h||v)&&n.addDiagnostic(e.createDiagnosticForNode(h||v,e.Diagnostics.The_type_of_this_node_cannot_be_serialized_because_its_property_0_cannot_be_serialized,t))}function Z(n,t){var r=L;L=function(t){return t.errorNode&&e.canProduceDiagnostics(t.errorNode)?e.createGetSymbolAccessibilityDiagnosticForNode(t.errorNode)(t):{diagnosticMessage:t.errorModuleName?e.Diagnostics.Declaration_emit_for_this_file_requires_using_private_name_0_from_module_1_An_explicit_type_annotation_may_unblock_declaration_emit:e.Diagnostics.Declaration_emit_for_this_file_requires_using_private_name_0_An_explicit_type_annotation_may_unblock_declaration_emit,errorNode:t.errorNode||n}};var a=w.getDeclarationStatementsForSourceFile(n,i,O,t);return L=r,a}function $(r){if(300===r.kind&&r.isDeclarationFile)return r;if(301===r.kind){A=!0,E=new e.Map,T=new e.Map;var i=!1,s=k.createBundle(e.map(r.sourceFiles,function(r){if(!r.isDeclarationFile){if(i=i||r.hasNoDefaultLib,b=r,t=r,m=void 0,g=!1,_=new e.Map,L=x,I=!1,N=!1,ee(r,E),ne(r,T),e.isExternalOrCommonJsModule(r)||e.isJsonSourceFile(r)){C=!1,D=!1;var a=e.isSourceFileJS(r)?k.createNodeArray(Z(r,!0)):e.visitNodes(r.statements,Te),o=k.updateSourceFile(r,[k.createModuleDeclaration([],[k.createModifier(134)],k.createStringLiteral(e.getResolvedExternalModuleName(n.getEmitHost(),r)),k.createModuleBlock(e.setTextRange(k.createNodeArray(ve(a)),r.statements)))],!0,[],[],!1,[]);return o}D=!0;var s=e.isSourceFileJS(r)?k.createNodeArray(Z(r)):e.visitNodes(r.statements,Te);return k.updateSourceFile(r,ve(s),!0,[],[],!1,[])}}),e.mapDefined(r.prepends,function(n){if(303===n.kind){var t=e.createUnparsedSourceFile(n,"dts",F);return i=i||!!t.hasNoDefaultLib,ee(t,E),G(t.typeReferenceDirectives),ne(t,T),t}return n}));s.syntheticFileReferences=[],s.syntheticTypeReferences=w(),s.syntheticLibReferences=O(),s.hasNoDefaultLib=i;var c=e.getDirectoryPath(e.normalizeSlashes(e.getOutputPathsFor(r,R,!0).declarationFilePath)),l=B(s.syntheticFileReferences,c);return E.forEach(l),s}D=!0,I=!1,N=!1,t=r,b=r,L=x,A=!1,C=!1,g=!1,m=void 0,_=new e.Map,o=void 0,E=ee(b,new e.Map),T=ne(b,new e.Map);var u,d=[],p=e.getDirectoryPath(e.normalizeSlashes(e.getOutputPathsFor(r,R,!0).declarationFilePath)),f=B(d,p);if(e.isSourceFileJS(b))u=k.createNodeArray(Z(r)),E.forEach(f),S=e.filter(u,e.isAnyImportSyntax);else{var h=e.visitNodes(r.statements,Te);u=e.setTextRange(k.createNodeArray(ve(h)),r.statements),E.forEach(f),S=e.filter(u,e.isAnyImportSyntax),e.isExternalModule(r)&&(!C||I&&!N)&&(u=e.setTextRange(k.createNodeArray(a(a([],u,!0),[e.createEmptyExports(k)],!1)),u))}var v=k.updateSourceFile(r,u,!0,d,w(),r.hasNoDefaultLib,O());return v.exportedModulesFromDeclarationEmit=y,v;function O(){return e.map(e.arrayFrom(T.keys()),function(e){return{fileName:e,pos:-1,end:-1}})}function w(){return o?e.mapDefined(e.arrayFrom(o.keys()),M):[]}function M(n){if(S)for(var t=0,r=S;t<r.length;t++){var i=r[t];if(e.isImportEqualsDeclaration(i)&&e.isExternalModuleReference(i.moduleReference)){var a=i.moduleReference.expression;if(e.isStringLiteralLike(a)&&a.text===n)return}else if(e.isImportDeclaration(i)&&e.isStringLiteral(i.moduleSpecifier)&&i.moduleSpecifier.text===n)return}return{fileName:n,pos:-1,end:-1}}function B(n,t){return function(i){var a;if(i.isDeclarationFile)a=i.fileName;else{if(A&&e.contains(r.sourceFiles,i))return;var o=e.getOutputPathsFor(i,R,!0);a=o.declarationFilePath||o.jsFilePath||i.fileName}if(a){var s=e.moduleSpecifiers.getModuleSpecifier(P,b,e.toPath(t,R.getCurrentDirectory(),R.getCanonicalFileName),e.toPath(a,R.getCurrentDirectory(),R.getCanonicalFileName),R);if(!e.pathIsRelative(s))return void G([s]);var c=e.getRelativePathToDirectoryOrUrl(t,a,R.getCurrentDirectory(),R.getCanonicalFileName,!1);if(e.startsWith(c,"./")&&e.hasExtension(c)&&(c=c.substring(2)),e.startsWith(c,"node_modules/")||e.pathContainsNodeModules(c))return;n.push({pos:-1,end:-1,fileName:c})}}}}function ee(n,t){return M||!e.isUnparsedSource(n)&&e.isSourceFileJS(n)?t:(e.forEach(n.referencedFiles,function(r){var i=R.getSourceFileFromReference(n,r);i&&t.set(e.getOriginalNodeId(i),i)}),t)}function ne(n,t){return e.forEach(n.libReferenceDirectives,function(n){var r=R.getLibFileFromReference(n);r&&t.set(e.toFileNameLowerCase(n.fileName),!0)}),t}function te(n){return 79===n.kind?n:200===n.kind?k.updateArrayBindingPattern(n,e.visitNodes(n.elements,t)):k.updateObjectBindingPattern(n,e.visitNodes(n.elements,t));function t(e){return 225===e.kind?e:k.updateBindingElement(e,e.dotDotDotToken,e.propertyName,te(e.name),ie(e)?e.initializer:void 0)}}function re(n,t,r){var i;g||(i=L,L=e.createGetSymbolAccessibilityDiagnosticForNode(n));var a=k.updateParameterDeclaration(n,void 0,c(n,t),n.dotDotDotToken,te(n.name),w.isOptionalParameter(n)?n.questionToken||k.createToken(57):void 0,oe(n,r||n.type,!0),ae(n));return g||(L=i),a}function ie(n){return d(n)&&w.isLiteralConstDeclaration(e.getParseTreeNode(n))}function ae(n){if(ie(n))return w.createLiteralConstValue(e.getParseTreeNode(n),O)}function oe(n,r,a){if((a||!e.hasEffectiveModifier(n,8))&&!ie(n)){var o,s=162===n.kind&&(w.isRequiredInitializedParameter(n)||w.isOptionalUninitializedParameterProperty(n));return r&&!s?e.visitNode(r,be):e.getParseTreeNode(n)?171===n.kind?k.createKeywordTypeNode(129):(h=n.name,g||(o=L,L=e.createGetSymbolAccessibilityDiagnosticForNode(n)),252===n.kind||201===n.kind?c(w.createTypeOfDeclaration(n,t,i,O)):162===n.kind||165===n.kind||164===n.kind?n.initializer?c(w.createTypeOfDeclaration(n,t,i,O,s)||w.createTypeOfExpression(n.initializer,t,i,O)):c(w.createTypeOfDeclaration(n,t,i,O,s)):c(w.createReturnTypeOfSignatureDeclaration(n,t,i,O))):r?e.visitNode(r,be):k.createKeywordTypeNode(129)}function c(e){return h=void 0,g||(L=o),e||k.createKeywordTypeNode(129)}}function se(n){switch(n=e.getParseTreeNode(n),n.kind){case 254:case 259:case 256:case 255:case 257:case 258:return!w.isDeclarationVisible(n);case 252:return!le(n);case 263:case 264:case 270:case 269:return!1;case 168:return!0}return!1}function ce(n){var t;if(n.body)return!0;var r=null===(t=n.symbol.declarations)||void 0===t?void 0:t.filter(function(n){return e.isFunctionDeclaration(n)&&!n.body});return!r||r.indexOf(n)===r.length-1}function le(n){return!e.isOmittedExpression(n)&&(e.isBindingPattern(n.name)?e.some(n.name.elements,le):w.isDeclarationVisible(n))}function ue(n,t,r){if(!e.hasEffectiveModifier(n,8)){var i=e.map(t,function(e){return re(e,r)});if(i)return k.createNodeArray(i,t.hasTrailingComma)}}function de(n,t){var r;if(!t){var i=e.getThisParameter(n);i&&(r=[re(i)])}if(e.isSetAccessorDeclaration(n)){var a=void 0;if(!t){var o=e.getSetAccessorValueParameter(n);if(o){var s=we(n,w.getAllAccessorDeclarations(n));a=re(o,void 0,s)}}a||(a=k.createParameterDeclaration(void 0,void 0,void 0,"value")),r=e.append(r,a)}return k.createNodeArray(r||e.emptyArray)}function pe(n,t){return e.hasEffectiveModifier(n,8)?void 0:e.visitNodes(t,be)}function fe(n){return e.isSourceFile(n)||e.isTypeAliasDeclaration(n)||e.isModuleDeclaration(n)||e.isClassDeclaration(n)||e.isInterfaceDeclaration(n)||e.isFunctionLike(n)||e.isIndexSignatureDeclaration(n)||e.isMappedTypeNode(n)}function me(e,n){var t=w.isEntityNameVisible(e,n);U(t),G(w.getTypeReferenceDirectivesForEntityName(e))}function _e(n,t){return e.hasJSDocNodes(n)&&e.hasJSDocNodes(t)&&(n.jsDoc=t.jsDoc),e.setCommentRange(n,e.getCommentRange(t))}function ge(t,r){if(r){if(C=C||259!==t.kind&&198!==t.kind,e.isStringLiteralLike(r))if(A){var i=e.getExternalModuleNameFromDeclaration(n.getEmitHost(),w,t);if(i)return k.createStringLiteral(i)}else{var a=w.getSymbolOfExternalModuleSpecifier(r);a&&(y||(y=[])).push(a)}return r}}function ye(n){if(w.isDeclarationVisible(n)){if(275===n.moduleReference.kind){var r=e.getExternalModuleImportEqualsDeclarationExpression(n);return k.updateImportEqualsDeclaration(n,void 0,n.modifiers,n.isTypeOnly,n.name,k.updateExternalModuleReference(n.moduleReference,ge(n,r)))}var i=L;return L=e.createGetSymbolAccessibilityDiagnosticForNode(n),me(n.moduleReference,t),L=i,n}}function he(n){if(!n.importClause)return k.updateImportDeclaration(n,void 0,n.modifiers,n.importClause,ge(n,n.moduleSpecifier));var t=n.importClause&&n.importClause.name&&w.isDeclarationVisible(n.importClause)?n.importClause.name:void 0;if(!n.importClause.namedBindings)return t&&k.updateImportDeclaration(n,void 0,n.modifiers,k.updateImportClause(n.importClause,n.importClause.isTypeOnly,t,void 0),ge(n,n.moduleSpecifier));if(266===n.importClause.namedBindings.kind){var r=w.isDeclarationVisible(n.importClause.namedBindings)?n.importClause.namedBindings:void 0;return t||r?k.updateImportDeclaration(n,void 0,n.modifiers,k.updateImportClause(n.importClause,n.importClause.isTypeOnly,t,r),ge(n,n.moduleSpecifier)):void 0}var i=e.mapDefined(n.importClause.namedBindings.elements,function(e){return w.isDeclarationVisible(e)?e:void 0});return i&&i.length||t?k.updateImportDeclaration(n,void 0,n.modifiers,k.updateImportClause(n.importClause,n.importClause.isTypeOnly,t,i&&i.length?k.updateNamedImports(n.importClause.namedBindings,i):void 0),ge(n,n.moduleSpecifier)):w.isImportRequiredByAugmentation(n)?k.updateImportDeclaration(n,void 0,n.modifiers,void 0,ge(n,n.moduleSpecifier)):void 0}function ve(n){while(e.length(m)){var t=m.shift();if(!e.isLateVisibilityPaintedStatement(t))return e.Debug.fail("Late replaced statement was found which is not handled by the declaration transformer!: "+(e.SyntaxKind?e.SyntaxKind[t.kind]:t.kind));var r=D;D=t.parent&&e.isSourceFile(t.parent)&&!(e.isExternalModule(t.parent)&&A);var i=xe(t);D=r,_.set(e.getOriginalNodeId(t),i)}return e.visitNodes(n,a);function a(n){if(e.isLateVisibilityPaintedStatement(n)){var t=e.getOriginalNodeId(n);if(_.has(t)){var r=_.get(t);return _.delete(t),r&&((e.isArray(r)?e.some(r,e.needsScopeMarker):e.needsScopeMarker(r))&&(I=!0),e.isSourceFile(n.parent)&&(e.isArray(r)?e.some(r,e.isExternalModuleIndicator):e.isExternalModuleIndicator(r))&&(C=!0)),r}}return n}}function be(r){if(!Ie(r)){if(e.isDeclaration(r)){if(se(r))return;if(e.hasDynamicName(r)&&!w.isLateBound(e.getParseTreeNode(r)))return}if((!e.isFunctionLike(r)||!w.isImplementationOfOverload(r))&&!e.isSemicolonClassElement(r)){var i;fe(r)&&(i=t,t=r);var a=L,o=e.canProduceDiagnostics(r),s=g,c=(180===r.kind||193===r.kind)&&257!==r.parent.kind;if((e.isMethodDeclaration(r)||e.isMethodSignature(r))&&e.hasEffectiveModifier(r,8)){if(r.symbol&&r.symbol.declarations&&r.symbol.declarations[0]!==r)return;return E(k.createPropertyDeclaration(void 0,Re(r),r.name,void 0,void 0,void 0))}if(o&&!g&&(L=e.createGetSymbolAccessibilityDiagnosticForNode(r)),e.isTypeQueryNode(r)&&me(r.exprName,t),c&&(g=!0),f(r))switch(r.kind){case 226:(e.isEntityName(r.expression)||e.isEntityNameExpression(r.expression))&&me(r.expression,t);var l=e.visitEachChild(r,be,n);return E(k.updateExpressionWithTypeArguments(l,l.expression,l.typeArguments));case 176:me(r.typeName,t);l=e.visitEachChild(r,be,n);return E(k.updateTypeReferenceNode(l,l.typeName,l.typeArguments));case 173:return E(k.updateConstructSignature(r,pe(r,r.typeParameters),ue(r,r.parameters),oe(r,r.type)));case 169:var u=k.createConstructorDeclaration(void 0,Re(r),ue(r,r.parameters,0),void 0);return E(u);case 167:if(e.isPrivateIdentifier(r.name))return E(void 0);var d=k.createMethodDeclaration(void 0,Re(r),void 0,r.name,r.questionToken,pe(r,r.typeParameters),ue(r,r.parameters),oe(r,r.type),void 0);return E(d);case 170:if(e.isPrivateIdentifier(r.name))return E(void 0);var p=we(r,w.getAllAccessorDeclarations(r));return E(k.updateGetAccessorDeclaration(r,void 0,Re(r),r.name,de(r,e.hasEffectiveModifier(r,8)),oe(r,p),void 0));case 171:return e.isPrivateIdentifier(r.name)?E(void 0):E(k.updateSetAccessorDeclaration(r,void 0,Re(r),r.name,de(r,e.hasEffectiveModifier(r,8)),void 0));case 165:return e.isPrivateIdentifier(r.name)?E(void 0):E(k.updatePropertyDeclaration(r,void 0,Re(r),r.name,r.questionToken,oe(r,r.type),ae(r)));case 164:return e.isPrivateIdentifier(r.name)?E(void 0):E(k.updatePropertySignature(r,Re(r),r.name,r.questionToken,oe(r,r.type)));case 166:return e.isPrivateIdentifier(r.name)?E(void 0):E(k.updateMethodSignature(r,Re(r),r.name,r.questionToken,pe(r,r.typeParameters),ue(r,r.parameters),oe(r,r.type)));case 172:return E(k.updateCallSignature(r,pe(r,r.typeParameters),ue(r,r.parameters),oe(r,r.type)));case 174:return E(k.updateIndexSignature(r,void 0,Re(r),ue(r,r.parameters),e.visitNode(r.type,be)||k.createKeywordTypeNode(129)));case 252:return e.isBindingPattern(r.name)?De(r.name):(c=!0,g=!0,E(k.updateVariableDeclaration(r,r.name,void 0,oe(r,r.type),ae(r))));case 161:return Ee(r)&&(r.default||r.constraint)?E(k.updateTypeParameterDeclaration(r,r.name,void 0,void 0)):E(e.visitEachChild(r,be,n));case 187:var m=e.visitNode(r.checkType,be),_=e.visitNode(r.extendsType,be),y=t;t=r.trueType;var h=e.visitNode(r.trueType,be);t=y;var v=e.visitNode(r.falseType,be);return E(k.updateConditionalTypeNode(r,m,_,h,v));case 177:return E(k.updateFunctionTypeNode(r,e.visitNodes(r.typeParameters,be),ue(r,r.parameters),e.visitNode(r.type,be)));case 178:return E(k.updateConstructorTypeNode(r,Re(r),e.visitNodes(r.typeParameters,be),ue(r,r.parameters),e.visitNode(r.type,be)));case 198:return e.isLiteralImportTypeNode(r)?E(k.updateImportTypeNode(r,k.updateLiteralTypeNode(r.argument,ge(r,r.argument.literal)),r.qualifier,e.visitNodes(r.typeArguments,be,e.isTypeNode),r.isTypeOf)):E(r);default:e.Debug.assertNever(r,"Attempted to process unhandled node kind: "+e.SyntaxKind[r.kind])}return e.isTupleTypeNode(r)&&e.getLineAndCharacterOfPosition(b,r.pos).line===e.getLineAndCharacterOfPosition(b,r.end).line&&e.setEmitFlags(r,1),E(e.visitEachChild(r,be,n))}}function E(n){return n&&o&&e.hasDynamicName(r)&&Ce(r),fe(r)&&(t=i),o&&!g&&(L=a),c&&(g=s),n===r?n:n&&e.setOriginalNode(_e(n,r),r)}}function Ee(n){return 167===n.parent.kind&&e.hasEffectiveModifier(n.parent,8)}function Te(n){if(p(n)&&!Ie(n)){switch(n.kind){case 270:return e.isSourceFile(n.parent)&&(C=!0),N=!0,k.updateExportDeclaration(n,void 0,n.modifiers,n.isTypeOnly,n.exportClause,ge(n,n.moduleSpecifier));case 269:if(e.isSourceFile(n.parent)&&(C=!0),N=!0,79===n.expression.kind)return n;var t=k.createUniqueName("_default",16);L=function(){return{diagnosticMessage:e.Diagnostics.Default_export_of_the_module_has_or_is_using_private_name_0,errorNode:n}},v=n;var r=k.createVariableDeclaration(t,void 0,w.createTypeOfExpression(n.expression,n,i,O),void 0);v=void 0;var a=k.createVariableStatement(D?[k.createModifier(134)]:[],k.createVariableDeclarationList([r],2));return[a,k.updateExportAssignment(n,n.decorators,n.modifiers,t)]}var o=xe(n);return _.set(e.getOriginalNodeId(n),o),n}}function Se(n){if(e.isImportEqualsDeclaration(n)||e.hasEffectiveModifier(n,512)||!e.canHaveModifiers(n))return n;var t=k.createModifiersFromModifierFlags(27646&e.getEffectiveModifierFlags(n));return k.updateModifiers(n,t)}function xe(n){if(!Ie(n)){switch(n.kind){case 263:return ye(n);case 264:return he(n)}if((!e.isDeclaration(n)||!se(n))&&(!e.isFunctionLike(n)||!w.isImplementationOfOverload(n))){var r;fe(n)&&(r=t,t=n);var o=e.canProduceDiagnostics(n),s=L;o&&(L=e.createGetSymbolAccessibilityDiagnosticForNode(n));var c=D;switch(n.kind){case 257:return Z(k.updateTypeAliasDeclaration(n,void 0,Re(n),n.name,e.visitNodes(n.typeParameters,be,e.isTypeParameterDeclaration),e.visitNode(n.type,be,e.isTypeNode)));case 256:return Z(k.updateInterfaceDeclaration(n,void 0,Re(n),n.name,pe(n,n.typeParameters),Pe(n.heritageClauses),e.visitNodes(n.members,be)));case 254:var l=Z(k.updateFunctionDeclaration(n,void 0,Re(n),void 0,n.name,pe(n,n.typeParameters),ue(n,n.parameters),oe(n,n.type),void 0));if(l&&w.isExpandoFunctionDeclaration(n)&&ce(n)){var u=w.getPropertiesOfContainerFunction(n),d=e.parseNodeFactory.createModuleDeclaration(void 0,void 0,l.name||k.createIdentifier("_default"),k.createModuleBlock([]),16);e.setParent(d,t),d.locals=e.createSymbolTable(u),d.symbol=u[0].parent;var p=[],f=e.mapDefined(u,function(n){if(n.valueDeclaration&&e.isPropertyAccessExpression(n.valueDeclaration)){L=e.createGetSymbolAccessibilityDiagnosticForNode(n.valueDeclaration);var t=w.createTypeOfDeclaration(n.valueDeclaration,d,i,O);L=s;var r=e.unescapeLeadingUnderscores(n.escapedName),a=e.isStringANonContextualKeyword(r),o=a?k.getGeneratedNameForNode(n.valueDeclaration):k.createIdentifier(r);a&&p.push([o,r]);var c=k.createVariableDeclaration(o,void 0,t,void 0);return k.createVariableStatement(a?void 0:[k.createToken(93)],k.createVariableDeclarationList([c]))}});p.length?f.push(k.createExportDeclaration(void 0,void 0,!1,k.createNamedExports(e.map(p,function(e){var n=e[0],t=e[1];return k.createExportSpecifier(n,t)})))):f=e.mapDefined(f,function(e){return k.updateModifiers(e,0)});var m=k.createModuleDeclaration(void 0,Re(n),n.name,k.createModuleBlock(f),16);if(!e.hasEffectiveModifier(l,512))return[l,m];var g=k.createModifiersFromModifierFlags(-514&e.getEffectiveModifierFlags(l)|2),y=k.updateFunctionDeclaration(l,void 0,g,void 0,l.name,l.typeParameters,l.parameters,l.type,void 0),b=k.updateModuleDeclaration(m,void 0,g,m.name,m.body),E=k.createExportAssignment(void 0,void 0,!1,m.name);return e.isSourceFile(n.parent)&&(C=!0),N=!0,[y,b,E]}return l;case 259:D=!1;var T=n.body;if(T&&260===T.kind){var S=I,x=N;N=!1,I=!1;var A=e.visitNodes(T.statements,Te),R=ve(A);8388608&n.flags&&(I=!1),e.isGlobalScopeAugmentation(n)||ke(R)||N||(R=I?k.createNodeArray(a(a([],R,!0),[e.createEmptyExports(k)],!1)):e.visitNodes(R,Se));var P=k.updateModuleBlock(T,R);D=c,I=S,N=x;var M=Re(n);return Z(k.updateModuleDeclaration(n,void 0,M,e.isExternalModuleAugmentation(n)?ge(n,n.name):n.name,P))}D=c;M=Re(n);D=!1,e.visitNode(T,Te);var F=e.getOriginalNodeId(T);P=_.get(F);return _.delete(F),Z(k.updateModuleDeclaration(n,void 0,M,n.name,P));case 255:h=n.name,v=n;g=k.createNodeArray(Re(n));var G=pe(n,n.typeParameters),B=e.getFirstConstructorWithBody(n),U=void 0;if(B){var V=L;U=e.compact(e.flatMap(B.parameters,function(n){if(e.hasSyntacticModifier(n,16476)&&!Ie(n))return L=e.createGetSymbolAccessibilityDiagnosticForNode(n),79===n.name.kind?_e(k.createPropertyDeclaration(void 0,Re(n),n.name,n.questionToken,oe(n,n.type),ae(n)),n):t(n.name);function t(r){for(var i,a=0,o=r.elements;a<o.length;a++){var s=o[a];e.isOmittedExpression(s)||(e.isBindingPattern(s.name)&&(i=e.concatenate(i,t(s.name))),i=i||[],i.push(k.createPropertyDeclaration(void 0,Re(n),s.name,void 0,oe(s,void 0),void 0)))}return i}})),L=V}var j=e.some(n.members,function(n){return!!n.name&&e.isPrivateIdentifier(n.name)}),K=j?[k.createPropertyDeclaration(void 0,void 0,k.createPrivateIdentifier("#private"),void 0,void 0,void 0)]:void 0,H=e.concatenate(e.concatenate(K,U),e.visitNodes(n.members,be)),W=k.createNodeArray(H),z=e.getEffectiveBaseTypeNode(n);if(z&&!e.isEntityNameExpression(z.expression)&&104!==z.expression.kind){var J=n.name?e.unescapeLeadingUnderscores(n.name.escapedText):"default",q=k.createUniqueName(J+"_base",16);L=function(){return{diagnosticMessage:e.Diagnostics.extends_clause_of_exported_class_0_has_or_is_using_private_name_1,errorNode:z,typeName:n.name}};var X=k.createVariableDeclaration(q,void 0,w.createTypeOfExpression(z.expression,n,i,O),void 0),Y=k.createVariableStatement(D?[k.createModifier(134)]:[],k.createVariableDeclarationList([X],2)),Q=k.createNodeArray(e.map(n.heritageClauses,function(n){if(94===n.token){var t=L;L=e.createGetSymbolAccessibilityDiagnosticForNode(n.types[0]);var r=k.updateHeritageClause(n,e.map(n.types,function(n){return k.updateExpressionWithTypeArguments(n,q,e.visitNodes(n.typeArguments,be))}));return L=t,r}return k.updateHeritageClause(n,e.visitNodes(k.createNodeArray(e.filter(n.types,function(n){return e.isEntityNameExpression(n.expression)||104===n.expression.kind})),be))}));return[Y,Z(k.updateClassDeclaration(n,void 0,g,n.name,G,Q,W))]}Q=Pe(n.heritageClauses);return Z(k.updateClassDeclaration(n,void 0,g,n.name,G,Q,W));case 235:return Z(Le(n));case 258:return Z(k.updateEnumDeclaration(n,void 0,k.createNodeArray(Re(n)),n.name,k.createNodeArray(e.mapDefined(n.members,function(e){if(!Ie(e)){var n=w.getConstantValue(e);return _e(k.updateEnumMember(e,e.name,void 0!==n?"string"===typeof n?k.createStringLiteral(n):k.createNumericLiteral(n):void 0),e)}}))))}return e.Debug.assertNever(n,"Unhandled top-level node in declaration emit: "+e.SyntaxKind[n.kind])}}function Z(i){return fe(n)&&(t=r),o&&(L=s),259===n.kind&&(D=c),i===n?i:(v=void 0,h=void 0,i&&e.setOriginalNode(_e(i,n),n))}}function Le(n){if(e.forEach(n.declarationList.declarations,le)){var t=e.visitNodes(n.declarationList.declarations,be);if(e.length(t))return k.updateVariableStatement(n,k.createNodeArray(Re(n)),k.updateVariableDeclarationList(n.declarationList,t))}}function De(n){return e.flatten(e.mapDefined(n.elements,function(e){return Ae(e)}))}function Ae(n){if(225!==n.kind&&n.name){if(!le(n))return;return e.isBindingPattern(n.name)?De(n.name):k.createVariableDeclaration(n.name,void 0,oe(n,void 0),void 0)}}function Ce(n){var r;g||(r=L,L=e.createGetSymbolAccessibilityDiagnosticForNodeName(n)),h=n.name,e.Debug.assert(w.isLateBound(e.getParseTreeNode(n)));var i=n,a=i.name.expression;me(a,t),g||(L=r),h=void 0}function Ie(e){return!!F&&!!e&&r(e,b)}function Ne(n){return e.isExportAssignment(n)||e.isExportDeclaration(n)}function ke(n){return e.some(n,Ne)}function Re(n){var t=e.getEffectiveModifierFlags(n),r=Oe(n);return t===r?n.modifiers:k.createModifiersFromModifierFlags(r)}function Oe(n){var t=11003,r=D&&!s(n)?2:0,i=300===n.parent.kind;return(!i||A&&i&&e.isExternalModule(n.parent))&&(t^=2,r=0),l(n,t,r)}function we(n,t){var r=u(n);return r||n===t.firstAccessor||(r=u(t.firstAccessor),L=e.createGetSymbolAccessibilityDiagnosticForNode(t.firstAccessor)),!r&&t.secondAccessor&&n!==t.secondAccessor&&(r=u(t.secondAccessor),L=e.createGetSymbolAccessibilityDiagnosticForNode(t.secondAccessor)),r}function Pe(n){return k.createNodeArray(e.filter(e.map(n,function(n){return k.updateHeritageClause(n,e.visitNodes(k.createNodeArray(e.filter(n.types,function(t){return e.isEntityNameExpression(t.expression)||94===n.token&&104===t.expression.kind})),be))}),function(e){return e.types&&!!e.types.length}))}}function s(e){return 256===e.kind}function c(n,t,r){return e.factory.createModifiersFromModifierFlags(l(n,t,r))}function l(n,t,r){void 0===t&&(t=27643),void 0===r&&(r=0);var i=e.getEffectiveModifierFlags(n)&t|r;return 512&i&&!(1&i)&&(i^=1),512&i&&2&i&&(i^=2),i}function u(e){if(e)return 170===e.kind?e.type:e.parameters.length>0?e.parameters[0].type:void 0}function d(n){switch(n.kind){case 165:case 164:return!e.hasEffectiveModifier(n,8);case 162:case 252:return!0}return!1}function p(e){switch(e.kind){case 254:case 259:case 263:case 256:case 255:case 257:case 258:case 235:case 264:case 270:case 269:return!0}return!1}function f(e){switch(e.kind){case 173:case 169:case 167:case 170:case 171:case 165:case 164:case 166:case 172:case 174:case 252:case 161:case 226:case 176:case 187:case 177:case 178:case 198:return!0}return!1}e.transformDeclarations=o}(d||(d={})),function(e){function n(n){switch(n){case e.ModuleKind.ESNext:case e.ModuleKind.ES2020:case e.ModuleKind.ES2015:return e.transformECMAScriptModule;case e.ModuleKind.System:return e.transformSystemModule;default:return e.transformModule}}var t,r;function i(e,n,t){return{scriptTransformers:o(e,n,t),declarationTransformers:s(n)}}function o(t,r,i){if(i)return e.emptyArray;var a=e.getEmitScriptTarget(t),o=e.getEmitModuleKind(t),s=[];return e.addRange(s,r&&e.map(r.before,u)),s.push(e.transformTypeScript),s.push(e.transformClassFields),e.getJSXTransformEnabled(t)&&s.push(e.transformJsx),a<99&&s.push(e.transformESNext),a<8&&s.push(e.transformES2021),a<7&&s.push(e.transformES2020),a<6&&s.push(e.transformES2019),a<5&&s.push(e.transformES2018),a<4&&s.push(e.transformES2017),a<3&&s.push(e.transformES2016),a<2&&(s.push(e.transformES2015),s.push(e.transformGenerators)),s.push(n(o)),a<1&&s.push(e.transformES5),e.addRange(s,r&&e.map(r.after,u)),s}function s(n){var t=[];return t.push(e.transformDeclarations),e.addRange(t,n&&e.map(n.afterDeclarations,d)),t}function c(n){return function(t){return e.isBundle(t)?n.transformBundle(t):n.transformSourceFile(t)}}function l(e,n){return function(t){var r=e(t);return"function"===typeof r?n(t,r):c(r)}}function u(n){return l(n,e.chainBundle)}function d(e){return l(e,function(e,n){return n})}function p(e,n){return n}function f(e,n,t){t(e,n)}function m(n,t,r,i,o,s,c){for(var l,u,d,m,_,g=new Array(350),y=0,h=[],v=[],b=[],E=[],T=0,S=!1,x=[],L=0,D=p,A=f,C=0,I=[],N={factory:r,getCompilerOptions:function(){return i},getEmitResolver:function(){return n},getEmitHost:function(){return t},getEmitHelperFactory:e.memoize(function(){return e.createEmitHelperFactory(N)}),startLexicalEnvironment:X,suspendLexicalEnvironment:Y,resumeLexicalEnvironment:Q,endLexicalEnvironment:Z,setLexicalEnvironmentFlags:$,getLexicalEnvironmentFlags:ee,hoistVariableDeclaration:z,hoistFunctionDeclaration:J,addInitializationStatement:q,startBlockScope:ne,endBlockScope:te,addBlockScopedVariable:re,requestEmitHelper:ie,readEmitHelpers:ae,enableSubstitution:U,enableEmitNotification:K,isSubstitutionEnabled:V,isEmitNotificationEnabled:H,get onSubstituteNode(){return D},set onSubstituteNode(n){e.Debug.assert(C<1,"Cannot modify transformation hooks after initialization has completed."),e.Debug.assert(void 0!==n,"Value must not be 'undefined'"),D=n},get onEmitNode(){return A},set onEmitNode(n){e.Debug.assert(C<1,"Cannot modify transformation hooks after initialization has completed."),e.Debug.assert(void 0!==n,"Value must not be 'undefined'"),A=n},addDiagnostic:function(e){I.push(e)}},k=0,R=o;k<R.length;k++){var O=R[k];e.disposeEmitNodes(e.getSourceFileOfNode(e.getParseTreeNode(O)))}e.performance.mark("beforeTransform");var w=s.map(function(e){return e(N)}),P=function(e){for(var n=0,t=w;n<t.length;n++){var r=t[n];e=r(e)}return e};C=1;for(var M=[],F=0,G=o;F<G.length;F++){O=G[F];null===e.tracing||void 0===e.tracing||e.tracing.push("emit","transformNodes",300===O.kind?{path:O.path}:{kind:O.kind,pos:O.pos,end:O.end}),M.push((c?P:B)(O)),null===e.tracing||void 0===e.tracing||e.tracing.pop()}return C=2,e.performance.mark("afterTransform"),e.performance.measure("transformTime","beforeTransform","afterTransform"),{transformed:M,substituteNode:j,emitNodeWithNotification:W,isEmitNotificationEnabled:H,dispose:oe,diagnostics:I};function B(n){return!n||e.isSourceFile(n)&&n.isDeclarationFile?n:P(n)}function U(n){e.Debug.assert(C<2,"Cannot modify the transformation context after transformation has completed."),g[n]|=1}function V(n){return 0!==(1&g[n.kind])&&0===(4&e.getEmitFlags(n))}function j(n,t){return e.Debug.assert(C<3,"Cannot substitute a node after the result is disposed."),t&&V(t)&&D(n,t)||t}function K(n){e.Debug.assert(C<2,"Cannot modify the transformation context after transformation has completed."),g[n]|=2}function H(n){return 0!==(2&g[n.kind])||0!==(2&e.getEmitFlags(n))}function W(n,t,r){e.Debug.assert(C<3,"Cannot invoke TransformationResult callbacks after the result is disposed."),t&&(H(t)?A(n,t,r):r(n,t))}function z(n){e.Debug.assert(C>0,"Cannot modify the lexical environment during initialization."),e.Debug.assert(C<2,"Cannot modify the lexical environment after transformation has completed.");var t=e.setEmitFlags(r.createVariableDeclaration(n),64);l?l.push(t):l=[t],1&y&&(y|=2)}function J(n){e.Debug.assert(C>0,"Cannot modify the lexical environment during initialization."),e.Debug.assert(C<2,"Cannot modify the lexical environment after transformation has completed."),e.setEmitFlags(n,1048576),u?u.push(n):u=[n]}function q(n){e.Debug.assert(C>0,"Cannot modify the lexical environment during initialization."),e.Debug.assert(C<2,"Cannot modify the lexical environment after transformation has completed."),e.setEmitFlags(n,1048576),d?d.push(n):d=[n]}function X(){e.Debug.assert(C>0,"Cannot modify the lexical environment during initialization."),e.Debug.assert(C<2,"Cannot modify the lexical environment after transformation has completed."),e.Debug.assert(!S,"Lexical environment is suspended."),h[T]=l,v[T]=u,b[T]=d,E[T]=y,T++,l=void 0,u=void 0,d=void 0,y=0}function Y(){e.Debug.assert(C>0,"Cannot modify the lexical environment during initialization."),e.Debug.assert(C<2,"Cannot modify the lexical environment after transformation has completed."),e.Debug.assert(!S,"Lexical environment is already suspended."),S=!0}function Q(){e.Debug.assert(C>0,"Cannot modify the lexical environment during initialization."),e.Debug.assert(C<2,"Cannot modify the lexical environment after transformation has completed."),e.Debug.assert(S,"Lexical environment is not suspended."),S=!1}function Z(){var n;if(e.Debug.assert(C>0,"Cannot modify the lexical environment during initialization."),e.Debug.assert(C<2,"Cannot modify the lexical environment after transformation has completed."),e.Debug.assert(!S,"Lexical environment is suspended."),l||u||d){if(u&&(n=a([],u,!0)),l){var t=r.createVariableStatement(void 0,r.createVariableDeclarationList(l));e.setEmitFlags(t,1048576),n?n.push(t):n=[t]}d&&(n=a(n?a([],n,!0):[],d,!0))}return T--,l=h[T],u=v[T],d=b[T],y=E[T],0===T&&(h=[],v=[],b=[],E=[]),n}function $(e,n){y=n?y|e:y&~e}function ee(){return y}function ne(){e.Debug.assert(C>0,"Cannot start a block scope during initialization."),e.Debug.assert(C<2,"Cannot start a block scope after transformation has completed."),x[L]=m,L++,m=void 0}function te(){e.Debug.assert(C>0,"Cannot end a block scope during initialization."),e.Debug.assert(C<2,"Cannot end a block scope after transformation has completed.");var n=e.some(m)?[r.createVariableStatement(void 0,r.createVariableDeclarationList(m.map(function(e){return r.createVariableDeclaration(e)}),1))]:void 0;return L--,m=x[L],0===L&&(x=[]),n}function re(n){e.Debug.assert(L>0,"Cannot add a block scoped variable outside of an iteration body."),(m||(m=[])).push(n)}function ie(n){if(e.Debug.assert(C>0,"Cannot modify the transformation context during initialization."),e.Debug.assert(C<2,"Cannot modify the transformation context after transformation has completed."),e.Debug.assert(!n.scoped,"Cannot request a scoped emit helper."),n.dependencies)for(var t=0,r=n.dependencies;t<r.length;t++){var i=r[t];ie(i)}_=e.append(_,n)}function ae(){e.Debug.assert(C>0,"Cannot modify the transformation context during initialization."),e.Debug.assert(C<2,"Cannot modify the transformation context after transformation has completed.");var n=_;return _=void 0,n}function oe(){if(C<3){for(var n=0,t=o;n<t.length;n++){var r=t[n];e.disposeEmitNodes(e.getSourceFileOfNode(e.getParseTreeNode(r)))}l=void 0,h=void 0,u=void 0,v=void 0,D=void 0,A=void 0,_=void 0,C=3}}}(function(e){e[e["Uninitialized"]=0]="Uninitialized",e[e["Initialized"]=1]="Initialized",e[e["Completed"]=2]="Completed",e[e["Disposed"]=3]="Disposed"})(t||(t={})),function(e){e[e["Substitution"]=1]="Substitution",e[e["EmitNotifications"]=2]="EmitNotifications"}(r||(r={})),e.noTransformers={scriptTransformers:e.emptyArray,declarationTransformers:e.emptyArray},e.getTransformers=i,e.noEmitSubstitution=p,e.noEmitNotification=f,e.transformNodes=m,e.nullTransformationContext={factory:e.factory,getCompilerOptions:function(){return{}},getEmitResolver:e.notImplemented,getEmitHost:e.notImplemented,getEmitHelperFactory:e.notImplemented,startLexicalEnvironment:e.noop,resumeLexicalEnvironment:e.noop,suspendLexicalEnvironment:e.noop,endLexicalEnvironment:e.returnUndefined,setLexicalEnvironmentFlags:e.noop,getLexicalEnvironmentFlags:function(){return 0},hoistVariableDeclaration:e.noop,hoistFunctionDeclaration:e.noop,addInitializationStatement:e.noop,startBlockScope:e.noop,endBlockScope:e.returnUndefined,addBlockScopedVariable:e.noop,requestEmitHelper:e.noop,readEmitHelpers:e.notImplemented,enableSubstitution:e.noop,enableEmitNotification:e.noop,isSubstitutionEnabled:e.notImplemented,isEmitNotificationEnabled:e.notImplemented,onSubstituteNode:p,onEmitNode:f,addDiagnostic:e.noop}}(d||(d={})),function(e){var n,t,r=I();function i(n){return e.fileExtensionIs(n,".tsbuildinfo")}function o(n,t,r,i,a,o){void 0===i&&(i=!1);var c=e.isArray(r)?r:e.getSourceFilesToEmit(n,r,i),u=n.getCompilerOptions();if(e.outFile(u)){var d=n.getPrependNodes();if(c.length||d.length){var p=e.factory.createBundle(c,d),f=t(l(p,n,i),p);if(f)return f}}else{if(!a)for(var m=0,_=c;m<_.length;m++){var g=_[m];f=t(l(g,n,i),g);if(f)return f}if(o){var y=s(u);if(y)return t({buildInfoPath:y},void 0)}}}function s(n){var t=n.configFilePath;if(e.isIncrementalCompilation(n)){if(n.tsBuildInfoFile)return n.tsBuildInfoFile;var r,i=e.outFile(n);if(i)r=e.removeFileExtension(i);else{if(!t)return;var a=e.removeFileExtension(t);r=n.outDir?n.rootDir?e.resolvePath(n.outDir,e.getRelativePathFromDirectory(n.rootDir,a,!0)):e.combinePaths(n.outDir,e.getBaseFileName(a)):a}return r+".tsbuildinfo"}}function c(n,t){var r=e.outFile(n),i=n.emitDeclarationOnly?void 0:r,a=i&&u(i,n),o=t||e.getEmitDeclarations(n)?e.removeFileExtension(r)+".d.ts":void 0,c=o&&e.getAreDeclarationMapsEnabled(n)?o+".map":void 0,l=s(n);return{jsFilePath:i,sourceMapFilePath:a,declarationFilePath:o,declarationMapPath:c,buildInfoPath:l}}function l(n,t,r){var i=t.getCompilerOptions();if(301===n.kind)return c(i,r);var a=e.getOwnEmitOutputFilePath(n.fileName,t,d(n,i)),o=e.isJsonSourceFile(n),s=o&&0===e.comparePaths(n.fileName,a,t.getCurrentDirectory(),!t.useCaseSensitiveFileNames()),l=i.emitDeclarationOnly||s?void 0:a,p=!l||e.isJsonSourceFile(n)?void 0:u(l,i),f=r||e.getEmitDeclarations(i)&&!o?e.getDeclarationEmitOutputFilePath(n.fileName,t):void 0,m=f&&e.getAreDeclarationMapsEnabled(i)?f+".map":void 0;return{jsFilePath:l,sourceMapFilePath:p,declarationFilePath:f,declarationMapPath:m,buildInfoPath:void 0}}function u(e,n){return n.sourceMap&&!n.inlineSourceMap?e+".map":void 0}function d(n,t){if(e.isJsonSourceFile(n))return".json";if(1===t.jsx)if(e.isSourceFileJS(n)){if(e.fileExtensionIs(n.fileName,".jsx"))return".jsx"}else if(1===n.languageVariant)return".jsx";return".js"}function p(n,t,r,i,a){return i?e.resolvePath(i,e.getRelativePathFromDirectory(a?a():v(t,r),n,r)):n}function f(n,t,r,i){return e.Debug.assert(!e.fileExtensionIs(n,".d.ts")&&!e.fileExtensionIs(n,".json")),e.changeExtension(p(n,t,r,t.options.declarationDir||t.options.outDir,i),".d.ts")}function m(n,t,r,i){if(!t.options.emitDeclarationOnly){var a=e.fileExtensionIs(n,".json"),o=e.changeExtension(p(n,t,r,t.options.outDir,i),a?".json":1===t.options.jsx&&(e.fileExtensionIs(n,".tsx")||e.fileExtensionIs(n,".jsx"))?".jsx":".js");return a&&0===e.comparePaths(n,o,e.Debug.checkDefined(t.options.configFilePath),r)?void 0:o}}function _(){var n;return{addOutput:t,getOutputs:r};function t(e){e&&(n||(n=[])).push(e)}function r(){return n||e.emptyArray}}function g(e,n){var t=c(e.options,!1),r=t.jsFilePath,i=t.sourceMapFilePath,a=t.declarationFilePath,o=t.declarationMapPath,s=t.buildInfoPath;n(r),n(i),n(a),n(o),n(s)}function y(n,t,r,i,a){if(!e.fileExtensionIs(t,".d.ts")){var o=m(t,n,r,a);if(i(o),!e.fileExtensionIs(t,".json")&&(o&&n.options.sourceMap&&i(o+".map"),e.getEmitDeclarations(n.options))){var s=f(t,n,r,a);i(s),n.options.declarationMap&&i(s+".map")}}}function h(n,t,r,i,a){var o;return n.rootDir?(o=e.getNormalizedAbsolutePath(n.rootDir,r),null===a||void 0===a||a(n.rootDir)):n.composite&&n.configFilePath?(o=e.getDirectoryPath(e.normalizeSlashes(n.configFilePath)),null===a||void 0===a||a(o)):o=e.computeCommonSourceDirectoryOfFilenames(t(),r,i),o&&o[o.length-1]!==e.directorySeparator&&(o+=e.directorySeparator),o}function v(n,t){var r=n.options,i=n.fileNames;return h(r,function(){return e.filter(i,function(n){return!(r.noEmitForJsFiles&&e.fileExtensionIsOneOf(n,e.supportedJSExtensions))&&!e.fileExtensionIs(n,".d.ts")})},e.getDirectoryPath(e.normalizeSlashes(e.Debug.checkDefined(r.configFilePath))),e.createGetCanonicalFileName(!t))}function b(n,t){var r=_(),i=r.addOutput,a=r.getOutputs;if(e.outFile(n.options))g(n,i);else{for(var o=e.memoize(function(){return v(n,t)}),c=0,l=n.fileNames;c<l.length;c++){var u=l[c];y(n,u,t,i,o)}i(s(n.options))}return a()}function E(n,t,r){t=e.normalizePath(t),e.Debug.assert(e.contains(n.fileNames,t),"Expected fileName to be present in command line");var i=_(),a=i.addOutput,o=i.getOutputs;return e.outFile(n.options)?g(n,a):y(n,t,r,a),o()}function T(n,t){if(e.outFile(n.options)){var r=c(n.options,!1).jsFilePath;return e.Debug.checkDefined(r,"project "+n.options.configFilePath+" expected to have at least one output")}for(var i=e.memoize(function(){return v(n,t)}),a=0,o=n.fileNames;a<o.length;a++){var l=o[a];if(!e.fileExtensionIs(l,".d.ts")){r=m(l,n,t,i);if(r)return r;if(!e.fileExtensionIs(l,".json")&&e.getEmitDeclarations(n.options))return f(l,n,t,i)}}var u=s(n.options);return u||e.Debug.fail("project "+n.options.configFilePath+" expected to have at least one output")}function S(n,t,r,i,a,s,c){var l,u,d=i.scriptTransformers,p=i.declarationTransformers,f=t.getCompilerOptions(),m=f.sourceMap||f.inlineSourceMap||e.getAreDeclarationMapsEnabled(f)?[]:void 0,_=f.listEmittedFiles?[]:void 0,g=e.createDiagnosticCollection(),y=e.getNewLineCharacter(f,function(){return t.getNewLine()}),h=e.createTextWriter(y),v=e.performance.createTimer("printTime","beforePrint","afterPrint"),b=v.enter,E=v.exit,T=!1;return b(),o(t,S,e.getSourceFilesToEmit(t,r,c),c,s,!r),E(),{emitSkipped:T,diagnostics:g.getDiagnostics(),emittedFiles:_,sourceMaps:m,exportedModulesFromDeclarationEmit:u};function S(n,r){var i,o=n.jsFilePath,s=n.sourceMapFilePath,c=n.declarationFilePath,u=n.declarationMapPath,d=n.buildInfoPath;function p(n){return e.ensurePathIsNonModuleName(e.getRelativePathFromDirectory(i,n,t.getCanonicalFileName))}d&&r&&e.isBundle(r)&&(i=e.getDirectoryPath(e.getNormalizedAbsolutePath(d,t.getCurrentDirectory())),l={commonSourceDirectory:p(t.getCommonSourceDirectory()),sourceFiles:r.sourceFiles.map(function(n){return p(e.getNormalizedAbsolutePath(n.fileName,t.getCurrentDirectory()))})}),null===e.tracing||void 0===e.tracing||e.tracing.push("emit","emitJsFileOrBundle",{jsFilePath:o}),D(r,o,s,p),null===e.tracing||void 0===e.tracing||e.tracing.pop(),null===e.tracing||void 0===e.tracing||e.tracing.push("emit","emitDeclarationFileOrBundle",{declarationFilePath:c}),A(r,c,u,p),null===e.tracing||void 0===e.tracing||e.tracing.pop(),null===e.tracing||void 0===e.tracing||e.tracing.push("emit","emitBuildInfo",{buildInfoPath:d}),L(l,d),null===e.tracing||void 0===e.tracing||e.tracing.pop(),!T&&_&&(a||(o&&_.push(o),s&&_.push(s),d&&_.push(d)),c&&_.push(c),u&&_.push(u))}function L(n,i){if(i&&!r&&!T){var a=t.getProgramBuildInfo();if(t.isEmitBlocked(i))T=!0;else{var o=e.version;e.writeFile(t,g,i,x({bundle:n,program:a,version:o}),!1)}}}function D(r,i,o,s){if(r&&!a&&i)if(i&&t.isEmitBlocked(i)||f.noEmit)T=!0;else{var c=e.transformNodes(n,t,e.factory,f,[r],d,!1),u={removeComments:f.removeComments,newLine:f.newLine,noEmitHelpers:f.noEmitHelpers,module:f.module,target:f.target,sourceMap:f.sourceMap,inlineSourceMap:f.inlineSourceMap,inlineSources:f.inlineSources,extendedDiagnostics:f.extendedDiagnostics,writeBundleFileInfo:!!l,relativeToBuildInfo:s},p=C(u,{hasGlobalName:n.hasGlobalName,onEmitNode:c.emitNodeWithNotification,isEmitNotificationEnabled:c.isEmitNotificationEnabled,substituteNode:c.substituteNode});e.Debug.assert(1===c.transformed.length,"Should only see one output from the transform"),N(i,o,c.transformed[0],p,f),c.dispose(),l&&(l.js=p.bundleFileInfo)}}function A(r,i,o,s){if(r)if(i){var d=e.isSourceFile(r)?[r]:r.sourceFiles,m=c?d:e.filter(d,e.isSourceFileNotJson),_=e.outFile(f)?[e.factory.createBundle(m,e.isSourceFile(r)?void 0:r.prepends)]:m;a&&!e.getEmitDeclarations(f)&&m.forEach(I);var y=e.transformNodes(n,t,e.factory,f,_,p,!1);if(e.length(y.diagnostics))for(var h=0,v=y.diagnostics;h<v.length;h++){var b=v[h];g.add(b)}var E={removeComments:f.removeComments,newLine:f.newLine,noEmitHelpers:!0,module:f.module,target:f.target,sourceMap:f.sourceMap,inlineSourceMap:f.inlineSourceMap,extendedDiagnostics:f.extendedDiagnostics,onlyPrintJsDocStyle:!0,writeBundleFileInfo:!!l,recordInternalSection:!!l,relativeToBuildInfo:s},S=C(E,{hasGlobalName:n.hasGlobalName,onEmitNode:y.emitNodeWithNotification,isEmitNotificationEnabled:y.isEmitNotificationEnabled,substituteNode:y.substituteNode}),x=!!y.diagnostics&&!!y.diagnostics.length||!!t.isEmitBlocked(i)||!!f.noEmit;if(T=T||x,(!x||c)&&(e.Debug.assert(1===y.transformed.length,"Should only see one output from the decl transform"),N(i,o,y.transformed[0],S,{sourceMap:!c&&f.declarationMap,sourceRoot:f.sourceRoot,mapRoot:f.mapRoot,extendedDiagnostics:f.extendedDiagnostics}),c&&300===y.transformed[0].kind)){var L=y.transformed[0];u=L.exportedModulesFromDeclarationEmit}y.dispose(),l&&(l.dts=S.bundleFileInfo)}else(a||f.emitDeclarationOnly)&&(T=!0)}function I(t){e.isExportAssignment(t)?79===t.expression.kind&&n.collectLinkedAliases(t.expression,!0):e.isExportSpecifier(t)?n.collectLinkedAliases(t.propertyName||t.name,!0):e.forEachChild(t,I)}function N(n,r,i,a,o){var s,c=301===i.kind?i:void 0,l=300===i.kind?i:void 0,u=c?c.sourceFiles:[l];if(k(o,i)&&(s=e.createSourceMapGenerator(t,e.getBaseFileName(e.normalizeSlashes(n)),R(o),O(o,n,l),o)),c?a.writeBundle(c,h,s):a.writeFile(l,h,s),s){m&&m.push({inputSourceFileNames:s.getSources(),sourceMap:s.toJSON()});var d=w(o,s,n,r,l);if(d&&(h.isAtStartOfLine()||h.rawWrite(y),h.writeComment("//# sourceMappingURL="+d)),r){var p=s.toString();e.writeFile(t,g,r,p,!1,u)}}else h.writeLine();e.writeFile(t,g,n,h.getText(),!!f.emitBOM,u),h.clear()}function k(n,t){return(n.sourceMap||n.inlineSourceMap)&&(300!==t.kind||!e.fileExtensionIs(t.fileName,".json"))}function R(n){var t=e.normalizeSlashes(n.sourceRoot||"");return t?e.ensureTrailingDirectorySeparator(t):t}function O(n,r,i){if(n.sourceRoot)return t.getCommonSourceDirectory();if(n.mapRoot){var a=e.normalizeSlashes(n.mapRoot);return i&&(a=e.getDirectoryPath(e.getSourceFilePathInNewDir(i.fileName,t,a))),0===e.getRootLength(a)&&(a=e.combinePaths(t.getCommonSourceDirectory(),a)),a}return e.getDirectoryPath(e.normalizePath(r))}function w(n,r,i,a,o){if(n.inlineSourceMap){var s=r.toString(),c=e.base64encode(e.sys,s);return"data:application/json;base64,"+c}var l=e.getBaseFileName(e.normalizeSlashes(e.Debug.checkDefined(a)));if(n.mapRoot){var u=e.normalizeSlashes(n.mapRoot);return o&&(u=e.getDirectoryPath(e.getSourceFilePathInNewDir(o.fileName,t,u))),0===e.getRootLength(u)?(u=e.combinePaths(t.getCommonSourceDirectory(),u),encodeURI(e.getRelativePathToDirectoryOrUrl(e.getDirectoryPath(e.normalizePath(i)),e.combinePaths(u,l),t.getCurrentDirectory(),t.getCanonicalFileName,!0))):encodeURI(e.combinePaths(u,l))}return encodeURI(l)}}function x(e){return JSON.stringify(e)}function L(e){return JSON.parse(e)}function D(n,t,r){var i,a=e.Debug.checkDefined(n.js),o=(null===(i=a.sources)||void 0===i?void 0:i.prologues)&&e.arrayToMap(a.sources.prologues,function(e){return e.file});return n.sourceFiles.map(function(n,i){var a,s,c=null===o||void 0===o?void 0:o.get(i),l=null===c||void 0===c?void 0:c.directives.map(function(n){var t=e.setTextRange(e.factory.createStringLiteral(n.expression.text),n.expression),r=e.setTextRange(e.factory.createExpressionStatement(t),n);return e.setParent(t,r),r}),u=e.factory.createToken(1),d=e.factory.createSourceFile(null!==l&&void 0!==l?l:[],u,0);return d.fileName=e.getRelativePathFromDirectory(r.getCurrentDirectory(),e.getNormalizedAbsolutePath(n,t),!r.useCaseSensitiveFileNames()),d.text=null!==(a=null===c||void 0===c?void 0:c.text)&&void 0!==a?a:"",e.setTextRangePosWidth(d,0,null!==(s=null===c||void 0===c?void 0:c.text.length)&&void 0!==s?s:0),e.setEachParent(d.statements,d),e.setTextRangePosWidth(u,d.end,0),e.setParent(u,d),d})}function A(n,t,r,i){var o=c(n.options,!1),s=o.buildInfoPath,l=o.jsFilePath,u=o.sourceMapFilePath,d=o.declarationFilePath,p=o.declarationMapPath,f=t.readFile(e.Debug.checkDefined(s));if(!f)return s;var m=t.readFile(e.Debug.checkDefined(l));if(!m)return l;var _=u&&t.readFile(u);if(u&&!_||n.options.inlineSourceMap)return u||"inline sourcemap decoding";var g=d&&t.readFile(d);if(d&&!g)return d;var y=p&&t.readFile(p);if(p&&!y||n.options.inlineSourceMap)return p||"inline sourcemap decoding";var h=L(f);if(!h.bundle||!h.bundle.js||g&&!h.bundle.dts)return s;var v=e.getDirectoryPath(e.getNormalizedAbsolutePath(s,t.getCurrentDirectory())),b=e.createInputFiles(m,g,u,_,p,y,l,d,s,h,!0),E=[],T=e.createPrependNodes(n.projectReferences,r,function(e){return t.readFile(e)}),A=D(h.bundle,v,t),C={getPrependNodes:e.memoize(function(){return a(a([],T,!0),[b],!1)}),getCanonicalFileName:t.getCanonicalFileName,getCommonSourceDirectory:function(){return e.getNormalizedAbsolutePath(h.bundle.commonSourceDirectory,v)},getCompilerOptions:function(){return n.options},getCurrentDirectory:function(){return t.getCurrentDirectory()},getNewLine:function(){return t.getNewLine()},getSourceFile:e.returnUndefined,getSourceFileByPath:e.returnUndefined,getSourceFiles:function(){return A},getLibFileFromReference:e.notImplemented,isSourceFileFromExternalLibrary:e.returnFalse,getResolvedProjectReferenceToRedirect:e.returnUndefined,getProjectReferenceRedirect:e.returnUndefined,isSourceOfProjectReferenceRedirect:e.returnFalse,writeFile:function(n,t,r){switch(n){case l:if(m===t)return;break;case u:if(_===t)return;break;case s:var i=L(t);i.program=h.program;var a=h.bundle,o=a.js,c=a.dts,f=a.sourceFiles;return i.bundle.js.sources=o.sources,c&&(i.bundle.dts.sources=c.sources),i.bundle.sourceFiles=f,void E.push({name:n,text:x(i),writeByteOrderMark:r});case d:if(g===t)return;break;case p:if(y===t)return;break;default:e.Debug.fail("Unexpected path: "+n)}E.push({name:n,text:t,writeByteOrderMark:r})},isEmitBlocked:e.returnFalse,readFile:function(e){return t.readFile(e)},fileExists:function(e){return t.fileExists(e)},useCaseSensitiveFileNames:function(){return t.useCaseSensitiveFileNames()},getProgramBuildInfo:e.returnUndefined,getSourceFileFromReference:e.returnUndefined,redirectTargetsMap:e.createMultiMap(),getFileIncludeReasons:e.notImplemented};return S(e.notImplementedResolver,C,void 0,e.getTransformers(n.options,i)),E}function C(n,t){void 0===n&&(n={}),void 0===t&&(t={});var r,i,a,o,s,c,l,u,d,p,f,m,_,g,y,h,v,b,E,T=t.hasGlobalName,S=t.onEmitNode,x=void 0===S?e.noEmitNotification:S,L=t.isEmitNotificationEnabled,D=t.substituteNode,A=void 0===D?e.noEmitSubstitution:D,C=t.onBeforeEmitNode,I=t.onAfterEmitNode,R=t.onBeforeEmitNodeArray,O=t.onAfterEmitNodeArray,w=t.onBeforeEmitToken,P=t.onAfterEmitToken,M=!!n.extendedDiagnostics,F=e.getNewLineCharacter(n),G=e.getEmitModuleKind(n),B=new e.Map,U=n.preserveSourceNewlines,V=qi,j=n.writeBundleFileInfo?{sections:[]}:void 0,K=j?e.Debug.checkDefined(n.relativeToBuildInfo):void 0,H=n.recordInternalSection,W=0,z="text",J=!0,q=-1,X=-1,Y=-1,Q=-1,Z=-1,$=!1,ee=!!n.removeComments,ne=e.performance.createTimerIf(M,"commentTime","beforeComment","afterComment"),te=ne.enter,re=ne.exit,ie=e.factory.parenthesizer,ae=gt();return De(),{printNode:oe,printList:se,printFile:le,printBundle:ce,writeNode:de,writeList:pe,writeFile:be,writeBundle:he,bundleFileInfo:j};function oe(n,t,r){switch(n){case 0:e.Debug.assert(e.isSourceFile(t),"Expected a SourceFile node.");break;case 2:e.Debug.assert(e.isIdentifier(t),"Expected an Identifier node.");break;case 1:e.Debug.assert(e.isExpression(t),"Expected an Expression node.");break}switch(t.kind){case 300:return le(t);case 301:return ce(t);case 302:return ue(t)}return de(n,t,r,Ee()),Te()}function se(e,n,t){return pe(e,n,t,Ee()),Te()}function ce(e){return he(e,Ee(),void 0),Te()}function le(e){return be(e,Ee(),void 0),Te()}function ue(e){return ve(e,Ee()),Te()}function de(e,n,t,r){var i=p;Le(r,void 0),Se(e,n,t),De(),p=i}function pe(e,n,t,r){var i=p;Le(r,void 0),t&&xe(t),Ki(void 0,n,e),De(),p=i}function fe(){return p.getTextPosWithWriteLine?p.getTextPosWithWriteLine():p.getTextPos()}function me(n,t,r){var i=e.lastOrUndefined(j.sections);i&&i.kind===r?i.end=t:j.sections.push({pos:n,end:t,kind:r})}function _e(n){if(H&&j&&r&&(e.isDeclaration(n)||e.isVariableStatement(n))&&e.isInternalDeclaration(n,r)&&"internal"!==z){var t=z;return ye(p.getTextPos()),W=fe(),z="internal",t}}function ge(e){e&&(ye(p.getTextPos()),W=fe(),z=e)}function ye(e){return W<e&&(me(W,e,z),!0)}function he(n,t,r){var i;m=!1;var a=p;Le(t,r),Li(n),Si(n),We(n),_i(n);for(var o=0,s=n.prepends;o<s.length;o++){var c=s[o];ia();var l=p.getTextPos(),u=j&&j.sections;if(u&&(j.sections=[]),Se(4,c,void 0),j){var d=j.sections;j.sections=u,c.oldFileOfCurrentEmit?(i=j.sections).push.apply(i,d):(d.forEach(function(n){return e.Debug.assert(e.isBundleFileTextLike(n))}),j.sections.push({pos:l,end:p.getTextPos(),kind:"prepend",data:K(c.fileName),texts:d}))}}W=fe();for(var f=0,_=n.sourceFiles;f<_.length;f++){var g=_[f];Se(0,g,g)}if(j&&n.sourceFiles.length){var y=p.getTextPos();if(ye(y)){var h=xi(n);h&&(j.sources||(j.sources={}),j.sources.prologues=h);var v=He(n);v&&(j.sources||(j.sources={}),j.sources.helpers=v)}}De(),p=a}function ve(e,n){var t=p;Le(n,void 0),Se(4,e,void 0),De(),p=t}function be(e,n,t){m=!0;var r=p;Le(n,t),Li(e),Si(e),Se(0,e,e),De(),p=r}function Ee(){return f||(f=e.createTextWriter(F))}function Te(){var e=f.getText();return f.clear(),e}function Se(e,n,t){t&&xe(t),we(e,n,void 0)}function xe(e){r=e,h=void 0,v=void 0,e&&Oo(e)}function Le(t,r){t&&n.omitTrailingSemicolon&&(t=e.getTrailingSemicolonDeferringWriter(t)),p=t,_=r,J=!p||!_}function De(){i=[],a=[],o=new e.Set,s=[],c=0,l=[],r=void 0,h=void 0,v=void 0,Le(void 0,void 0)}function Ae(){return h||(h=e.getLineStarts(r))}function Ce(e,n){if(void 0!==e){var t=_e(e);we(4,e,n),ge(t)}}function Ie(e){void 0!==e&&we(2,e,void 0)}function Ne(e,n){void 0!==e&&we(1,e,n)}function ke(n){we(e.isStringLiteral(n)?6:4,n)}function Re(n){U&&134217728&e.getEmitFlags(n)&&(U=!1)}function Oe(e){U=e}function we(e,n,t){E=t;var r=Fe(0,e,n);r(e,n),E=void 0}function Pe(n){return!ee&&!e.isSourceFile(n)}function Me(n){return!J&&!e.isSourceFile(n)&&!e.isInJsonFile(n)&&!e.isUnparsedSource(n)&&!e.isUnparsedPrepend(n)}function Fe(n,t,r){switch(n){case 0:if(x!==e.noEmitNotification&&(!L||L(r)))return Be;case 1:if(A!==e.noEmitSubstitution&&(b=A(t,r)||r)!==r)return E&&(b=E(b)),Ke;case 2:if(Pe(r))return qa;case 3:if(Me(r))return Do;case 4:return Ue;default:return e.Debug.assertNever(n)}}function Ge(e,n,t){return Fe(e+1,n,t)}function Be(e,n){var t=Ge(0,e,n);x(e,n,t)}function Ue(e,n){if(null===C||void 0===C||C(n),U){var t=U;Re(n),Ve(e,n),Oe(t)}else Ve(e,n);null===I||void 0===I||I(n),E=void 0}function Ve(n,t){if(0===n)return mi(e.cast(t,e.isSourceFile));if(2===n)return $e(e.cast(t,e.isIdentifier));if(6===n)return qe(e.cast(t,e.isStringLiteral),!0);if(3===n)return je(e.cast(t,e.isTypeParameterDeclaration));if(5===n)return e.Debug.assertNode(t,e.isEmptyStatement),Nt(!0);if(4===n){switch(t.kind){case 15:case 16:case 17:return qe(t,!1);case 79:return $e(t);case 80:return en(t);case 159:return nn(t);case 160:return rn(t);case 161:return an(t);case 162:return on(t);case 163:return sn(t);case 164:return cn(t);case 165:return ln(t);case 166:return un(t);case 167:return dn(t);case 168:return pn(t);case 169:return fn(t);case 170:case 171:return mn(t);case 172:return _n(t);case 173:return gn(t);case 174:return yn(t);case 175:return bn(t);case 176:return En(t);case 177:return Tn(t);case 178:return An(t);case 179:return Cn(t);case 180:return In(t);case 181:return Nn(t);case 182:return Rn(t);case 183:return wn(t);case 185:return Pn(t);case 186:return Mn(t);case 187:return Fn(t);case 188:return Gn(t);case 189:return Bn(t);case 226:return Tt(t);case 190:return Un();case 191:return Vn(t);case 192:return jn(t);case 193:return Kn(t);case 194:return Hn(t);case 195:return On(t);case 196:return Wn(t);case 197:return hn(t);case 198:return zn(t);case 199:return Jn(t);case 200:return qn(t);case 201:return Xn(t);case 231:return Dt(t);case 232:return vn();case 233:return At(t);case 235:return It(t);case 234:return Nt(!1);case 236:return kt(t);case 237:return Rt(t);case 238:return wt(t);case 239:return Pt(t);case 240:return Mt(t);case 241:return Ft(t);case 242:return Gt(t);case 243:return Ut(t);case 244:return Vt(t);case 245:return Kt(t);case 246:return Ht(t);case 247:return Wt(t);case 248:return zt(t);case 249:return Jt(t);case 250:return qt(t);case 251:return Xt(t);case 252:return Yt(t);case 253:return Qt(t);case 254:return Zt(t);case 255:return or(t);case 256:return cr(t);case 257:return lr(t);case 258:return ur(t);case 259:return dr(t);case 260:return pr(t);case 261:return fr(t);case 262:return Sr(t);case 263:return mr(t);case 264:return gr(t);case 265:return yr(t);case 266:return hr(t);case 272:return xr(t);case 267:return vr(t);case 268:return br(t);case 269:return Er(t);case 270:return Tr(t);case 271:return Lr(t);case 273:return Dr(t);case 274:return;case 275:return Ir(t);case 11:return wr(t);case 278:case 281:return Or(t);case 279:case 282:return Pr(t);case 283:return Fr(t);case 284:return Mr(t);case 285:return Gr(t);case 286:return jr(t);case 287:return Hr(t);case 288:return Wr(t);case 289:return Jr(t);case 290:return qr(t);case 291:return Xr(t);case 292:return Yr(t);case 293:return Qr(t);case 294:return Zr(t);case 295:return Ye(t);case 302:case 296:return Xe(t);case 297:case 298:return Qe(t);case 299:return Ze(t);case 300:return mi(t);case 301:return e.Debug.fail("Bundles should be printed using printBundle");case 303:return e.Debug.fail("InputFiles should not be printed");case 304:return fi(t);case 305:return ti(t);case 307:return Yi("*");case 308:return Yi("?");case 309:return xn(t);case 310:return Ln(t);case 311:return Dn(t);case 312:return Sn(t);case 184:case 313:return kn(t);case 314:return;case 315:return $r(t);case 317:return ci(t);case 318:return li(t);case 322:case 327:return si(t);case 323:case 324:return ri(t);case 325:case 326:return;case 328:case 329:case 330:case 331:case 332:return;case 333:return oi(t);case 335:case 342:return ui(t);case 334:case 336:case 337:case 338:return ei(t);case 339:return ii(t);case 340:return ai(t);case 341:return ni(t);case 344:case 348:case 347:return}if(e.isExpression(t)&&(n=1,A!==e.noEmitSubstitution)){var r=A(n,t)||t;r!==t&&(t=r,E&&(t=E(t)))}}if(1===n)switch(t.kind){case 8:case 9:return Je(t);case 10:case 13:case 14:return qe(t,!1);case 79:return $e(t);case 202:return Yn(t);case 203:return Qn(t);case 204:return Zn(t);case 205:return et(t);case 206:return nt(t);case 207:return tt(t);case 208:return rt(t);case 209:return it(t);case 210:return at(t);case 211:return ot(t);case 212:return st(t);case 213:return lt(t);case 214:return ut(t);case 215:return dt(t);case 216:return pt(t);case 217:return ft(t);case 218:return _t(t);case 219:return ae(t);case 220:return yt(t);case 221:return ht(t);case 222:return vt(t);case 223:return bt(t);case 224:return Et(t);case 225:return;case 227:return St(t);case 228:return xt(t);case 229:return Lt(t);case 230:return e.Debug.fail("SyntheticExpression should never be printed.");case 276:return Nr(t);case 277:return kr(t);case 280:return Rr(t);case 343:return e.Debug.fail("SyntaxList should not be printed");case 344:return;case 345:return vi(t);case 346:return bi(t);case 347:case 348:return;case 349:return e.Debug.fail("SyntheticReferenceExpression should not be printed")}return e.isKeyword(t.kind)?ca(t,Zi):e.isTokenKind(t.kind)?ca(t,Yi):void e.Debug.fail("Unhandled SyntaxKind: "+e.Debug.formatSyntaxKind(t.kind)+".")}function je(e){Ce(e.name),ta(),Zi("in"),ta(),Ce(e.constraint)}function Ke(n,t){var r=Ge(1,n,t);e.Debug.assertIsDefined(b),t=b,b=void 0,r(n,t)}function He(t){var r;if(G!==e.ModuleKind.None&&!n.noEmitHelpers){for(var i=new e.Map,a=0,o=t.sourceFiles;a<o.length;a++){var s=o[a],c=void 0!==e.getExternalHelpersModuleName(s),l=ze(s);if(l)for(var u=0,d=l;u<d.length;u++){var p=d[u];p.scoped||c||i.get(p.name)||(i.set(p.name,!0),(r||(r=[])).push(p.name))}}return r}}function We(t){var i=!1,a=301===t.kind?t:void 0;if(!a||G!==e.ModuleKind.None){for(var o=a?a.prepends.length:0,s=a?a.sourceFiles.length+o:1,c=0;c<s;c++){var l=a?c<o?a.prepends[c]:a.sourceFiles[c-o]:t,u=e.isSourceFile(l)?l:e.isUnparsedSource(l)?void 0:r,d=n.noEmitHelpers||!!u&&e.hasRecordedExternalHelpers(u),f=(e.isSourceFile(l)||e.isUnparsedSource(l))&&!m,_=e.isUnparsedSource(l)?l.helpers:ze(l);if(_)for(var g=0,y=_;g<y.length;g++){var h=y[g];if(h.scoped){if(a)continue}else{if(d)continue;if(f){if(B.get(h.name))continue;B.set(h.name,!0)}}var v=fe();"string"===typeof h.text?da(h.text):da(h.text(Ba)),j&&j.sections.push({pos:v,end:p.getTextPos(),kind:"emitHelpers",data:h.name}),i=!0}}return i}}function ze(n){var t=e.getEmitHelpers(n);return t&&e.stableSort(t,e.compareEmitHelpers)}function Je(e){qe(e,!1)}function qe(t,r){var i=La(t,n.neverAsciiEscape,r);!n.sourceMap&&!n.inlineSourceMap||10!==t.kind&&!e.isTemplateLiteralKind(t.kind)?Ji(i):zi(i)}function Xe(e){for(var n=0,t=e.texts;n<t.length;n++){var r=t[n];ia(),Ce(r)}}function Ye(e){p.rawWrite(e.parent.text.substring(e.pos,e.end))}function Qe(e){var n=fe();Ye(e),j&&me(n,p.getTextPos(),297===e.kind?"text":"internal")}function Ze(n){var t=fe();if(Ye(n),j){var r=e.clone(n.section);r.pos=t,r.end=p.getTextPos(),j.sections.push(r)}}function $e(e){var n=e.symbol?Xi:V;n(xa(e,!1),e.symbol),Ki(e,e.typeArguments,53776)}function en(e){var n=e.symbol?Xi:V;n(xa(e,!1),e.symbol)}function nn(e){tn(e.left),Yi("."),Ce(e.right)}function tn(e){79===e.kind?Ne(e):Ce(e)}function rn(e){Yi("["),Ne(e.expression,ie.parenthesizeExpressionOfComputedPropertyName),Yi("]")}function an(e){Ce(e.name),e.constraint&&(ta(),Zi("extends"),ta(),Ce(e.constraint)),e.default&&(ta(),$i("="),ta(),Ce(e.default))}function on(e){Pi(e,e.decorators),Ai(e,e.modifiers),Ce(e.dotDotDotToken),Di(e.name,ea),Ce(e.questionToken),e.parent&&312===e.parent.kind&&!e.name?Ce(e.type):Ci(e.type),Ii(e.initializer,e.type?e.type.end:e.questionToken?e.questionToken.end:e.name?e.name.end:e.modifiers?e.modifiers.end:e.decorators?e.decorators.end:e.pos,e,ie.parenthesizeExpressionForDisallowedComma)}function sn(e){Yi("@"),Ne(e.expression,ie.parenthesizeLeftSideOfAccess)}function cn(e){Pi(e,e.decorators),Ai(e,e.modifiers),Di(e.name,ra),Ce(e.questionToken),Ci(e.type),Qi()}function ln(e){Pi(e,e.decorators),Ai(e,e.modifiers),Ce(e.name),Ce(e.questionToken),Ce(e.exclamationToken),Ci(e.type),Ii(e.initializer,e.type?e.type.end:e.questionToken?e.questionToken.end:e.name.end,e),Qi()}function un(e){Da(e),Pi(e,e.decorators),Ai(e,e.modifiers),Ce(e.name),Ce(e.questionToken),Fi(e,e.typeParameters),Gi(e,e.parameters),Ci(e.type),Qi(),Aa(e)}function dn(e){Pi(e,e.decorators),Ai(e,e.modifiers),Ce(e.asteriskToken),Ce(e.name),Ce(e.questionToken),er(e,nr)}function pn(e){Pi(e,e.decorators),Ai(e,e.modifiers),Zi("static"),rr(e.body)}function fn(e){Ai(e,e.modifiers),Zi("constructor"),er(e,nr)}function mn(e){Pi(e,e.decorators),Ai(e,e.modifiers),Zi(170===e.kind?"get":"set"),ta(),Ce(e.name),er(e,nr)}function _n(e){Da(e),Pi(e,e.decorators),Ai(e,e.modifiers),Fi(e,e.typeParameters),Gi(e,e.parameters),Ci(e.type),Qi(),Aa(e)}function gn(e){Da(e),Pi(e,e.decorators),Ai(e,e.modifiers),Zi("new"),ta(),Fi(e,e.typeParameters),Gi(e,e.parameters),Ci(e.type),Qi(),Aa(e)}function yn(e){Pi(e,e.decorators),Ai(e,e.modifiers),Vi(e,e.parameters),Ci(e.type),Qi()}function hn(e){Ce(e.type),Ce(e.literal)}function vn(){Qi()}function bn(e){e.assertsModifier&&(Ce(e.assertsModifier),ta()),Ce(e.parameterName),e.type&&(ta(),Zi("is"),ta(),Ce(e.type))}function En(e){Ce(e.typeName),Mi(e,e.typeArguments)}function Tn(e){Da(e),Fi(e,e.typeParameters),Ui(e,e.parameters),ta(),Yi("=>"),ta(),Ce(e.type),Aa(e)}function Sn(e){Zi("function"),Gi(e,e.parameters),Yi(":"),Ce(e.type)}function xn(e){Yi("?"),Ce(e.type)}function Ln(e){Yi("!"),Ce(e.type)}function Dn(e){Ce(e.type),Yi("=")}function An(e){Da(e),Ai(e,e.modifiers),Zi("new"),ta(),Fi(e,e.typeParameters),Gi(e,e.parameters),ta(),Yi("=>"),ta(),Ce(e.type),Aa(e)}function Cn(e){Zi("typeof"),ta(),Ce(e.exprName)}function In(n){Yi("{");var t=1&e.getEmitFlags(n)?768:32897;Ki(n,n.members,524288|t),Yi("}")}function Nn(e){Ce(e.elementType,ie.parenthesizeElementTypeOfArrayType),Yi("["),Yi("]")}function kn(e){Yi("..."),Ce(e.type)}function Rn(n){jt(22,n.pos,Yi,n);var t=1&e.getEmitFlags(n)?528:657;Ki(n,n.elements,524288|t),jt(23,n.elements.end,Yi,n)}function On(e){Ce(e.dotDotDotToken),Ce(e.name),Ce(e.questionToken),jt(58,e.name.end,Yi,e),ta(),Ce(e.type)}function wn(e){Ce(e.type,ie.parenthesizeElementTypeOfArrayType),Yi("?")}function Pn(e){Ki(e,e.types,516,ie.parenthesizeMemberOfElementType)}function Mn(e){Ki(e,e.types,520,ie.parenthesizeMemberOfElementType)}function Fn(e){Ce(e.checkType,ie.parenthesizeMemberOfConditionalType),ta(),Zi("extends"),ta(),Ce(e.extendsType,ie.parenthesizeMemberOfConditionalType),ta(),Yi("?"),ta(),Ce(e.trueType),ta(),Yi(":"),ta(),Ce(e.falseType)}function Gn(e){Zi("infer"),ta(),Ce(e.typeParameter)}function Bn(e){Yi("("),Ce(e.type),Yi(")")}function Un(){Zi("this")}function Vn(e){la(e.operator,Zi),ta(),Ce(e.type,ie.parenthesizeMemberOfElementType)}function jn(e){Ce(e.objectType,ie.parenthesizeMemberOfElementType),Yi("["),Ce(e.indexType),Yi("]")}function Kn(n){var t=e.getEmitFlags(n);Yi("{"),1&t?ta():(ia(),aa()),n.readonlyToken&&(Ce(n.readonlyToken),143!==n.readonlyToken.kind&&Zi("readonly"),ta()),Yi("["),we(3,n.typeParameter),n.nameType&&(ta(),Zi("as"),ta(),Ce(n.nameType)),Yi("]"),n.questionToken&&(Ce(n.questionToken),57!==n.questionToken.kind&&Yi("?")),Yi(":"),ta(),Ce(n.type),Qi(),1&t?ta():(ia(),oa()),Yi("}")}function Hn(e){Ne(e.literal)}function Wn(e){Ce(e.head),Ki(e,e.templateSpans,262144)}function zn(e){e.isTypeOf&&(Zi("typeof"),ta()),Zi("import"),Yi("("),Ce(e.argument),Yi(")"),e.qualifier&&(Yi("."),Ce(e.qualifier)),Mi(e,e.typeArguments)}function Jn(e){Yi("{"),Ki(e,e.elements,525136),Yi("}")}function qn(e){Yi("["),Ki(e,e.elements,524880),Yi("]")}function Xn(e){Ce(e.dotDotDotToken),e.propertyName&&(Ce(e.propertyName),Yi(":"),ta()),Ce(e.name),Ii(e.initializer,e.name.end,e,ie.parenthesizeExpressionForDisallowedComma)}function Yn(e){var n=e.elements,t=e.multiLine?65536:0;Hi(e,n,8914|t,ie.parenthesizeExpressionForDisallowedComma)}function Qn(n){e.forEach(n.properties,Na);var t=65536&e.getEmitFlags(n);t&&aa();var i=n.multiLine?65536:0,a=r.languageVersion>=1&&!e.isJsonSourceFile(r)?64:0;Ki(n,n.properties,526226|a|i),t&&oa()}function Zn(n){Ne(n.expression,ie.parenthesizeLeftSideOfAccess);var t=n.questionDotToken||e.setTextRangePosEnd(e.factory.createToken(24),n.expression.end,n.name.pos),r=Ea(n,n.expression,t),i=Ea(n,t,n.name);pa(r,!1);var a=28!==t.kind&&$n(n.expression)&&!p.hasTrailingComment()&&!p.hasTrailingWhitespace();a&&Yi("."),n.questionDotToken?Ce(t):jt(t.kind,n.expression.end,Yi,n),pa(i,!1),Ce(n.name),fa(r,i)}function $n(n){if(n=e.skipPartiallyEmittedExpressions(n),e.isNumericLiteral(n)){var t=La(n,!0,!1);return!n.numericLiteralFlags&&!e.stringContains(t,e.tokenToString(24))}if(e.isAccessExpression(n)){var r=e.getConstantValue(n);return"number"===typeof r&&isFinite(r)&&Math.floor(r)===r}}function et(e){Ne(e.expression,ie.parenthesizeLeftSideOfAccess),Ce(e.questionDotToken),jt(22,e.expression.end,Yi,e),Ne(e.argumentExpression),jt(23,e.argumentExpression.end,Yi,e)}function nt(n){var t=536870912&e.getEmitFlags(n);t&&(Yi("("),zi("0"),Yi(","),ta()),Ne(n.expression,ie.parenthesizeLeftSideOfAccess),t&&Yi(")"),Ce(n.questionDotToken),Mi(n,n.typeArguments),Hi(n,n.arguments,2576,ie.parenthesizeExpressionForDisallowedComma)}function tt(e){jt(103,e.pos,Zi,e),ta(),Ne(e.expression,ie.parenthesizeExpressionOfNew),Mi(e,e.typeArguments),Hi(e,e.arguments,18960,ie.parenthesizeExpressionForDisallowedComma)}function rt(n){var t=536870912&e.getEmitFlags(n);t&&(Yi("("),zi("0"),Yi(","),ta()),Ne(n.tag,ie.parenthesizeLeftSideOfAccess),t&&Yi(")"),Mi(n,n.typeArguments),ta(),Ne(n.template)}function it(e){Yi("<"),Ce(e.type),Yi(">"),Ne(e.expression,ie.parenthesizeOperandOfPrefixUnary)}function at(e){var n=jt(20,e.pos,Yi,e),t=ha(e.expression,e);Ne(e.expression,void 0),va(e.expression,e),fa(t),jt(21,e.expression?e.expression.end:n,Yi,e)}function ot(e){ka(e.name),$t(e)}function st(e){Pi(e,e.decorators),Ai(e,e.modifiers),er(e,ct)}function ct(e){Fi(e,e.typeParameters),Ui(e,e.parameters),Ci(e.type),ta(),Ce(e.equalsGreaterThanToken)}function lt(e){jt(89,e.pos,Zi,e),ta(),Ne(e.expression,ie.parenthesizeOperandOfPrefixUnary)}function ut(e){jt(112,e.pos,Zi,e),ta(),Ne(e.expression,ie.parenthesizeOperandOfPrefixUnary)}function dt(e){jt(114,e.pos,Zi,e),ta(),Ne(e.expression,ie.parenthesizeOperandOfPrefixUnary)}function pt(e){jt(131,e.pos,Zi,e),ta(),Ne(e.expression,ie.parenthesizeOperandOfPrefixUnary)}function ft(e){la(e.operator,$i),mt(e)&&ta(),Ne(e.operand,ie.parenthesizeOperandOfPrefixUnary)}function mt(e){var n=e.operand;return 217===n.kind&&(39===e.operator&&(39===n.operator||45===n.operator)||40===e.operator&&(40===n.operator||46===n.operator))}function _t(e){Ne(e.operand,ie.parenthesizeOperandOfPostfixUnary),la(e.operator,$i)}function gt(){return e.createBinaryExpressionTrampoline(n,t,r,i,a,void 0);function n(e,n){if(n){n.stackIndex++,n.preserveSourceNewlinesStack[n.stackIndex]=U,n.containerPosStack[n.stackIndex]=Y,n.containerEndStack[n.stackIndex]=Q,n.declarationListContainerEndStack[n.stackIndex]=Z;var t=n.shouldEmitCommentsStack[n.stackIndex]=Pe(e),r=n.shouldEmitSourceMapsStack[n.stackIndex]=Me(e);null===C||void 0===C||C(e),t&&Xa(e),r&&Ao(e),Re(e)}else n={stackIndex:0,preserveSourceNewlinesStack:[void 0],containerPosStack:[-1],containerEndStack:[-1],declarationListContainerEndStack:[-1],shouldEmitCommentsStack:[!1],shouldEmitSourceMapsStack:[!1]};return n}function t(e,n,t){return o(e,t,"left")}function r(e,n,t){var r=27!==e.kind,i=Ea(t,t.left,e),a=Ea(t,e,t.right);pa(i,r),po(e.pos),ca(e,101===e.kind?Zi:$i),_o(e.end,!0),pa(a,!0)}function i(e,n,t){return o(e,t,"right")}function a(e,n){var t=Ea(e,e.left,e.operatorToken),r=Ea(e,e.operatorToken,e.right);if(fa(t,r),n.stackIndex>0){var i=n.preserveSourceNewlinesStack[n.stackIndex],a=n.containerPosStack[n.stackIndex],o=n.containerEndStack[n.stackIndex],s=n.declarationListContainerEndStack[n.stackIndex],c=n.shouldEmitCommentsStack[n.stackIndex],l=n.shouldEmitSourceMapsStack[n.stackIndex];Oe(i),l&&Co(e),c&&Ya(e,a,o,s),null===I||void 0===I||I(e),n.stackIndex--}}function o(n,t,r){var i="left"===r?ie.getParenthesizeLeftSideOfBinaryForOperator(t.operatorToken.kind):ie.getParenthesizeRightSideOfBinaryForOperator(t.operatorToken.kind),a=Fe(0,1,n);if(a===Ke&&(e.Debug.assertIsDefined(b),n=i(e.cast(b,e.isExpression)),a=Ge(1,1,n),b=void 0),(a===qa||a===Do||a===Ue)&&e.isBinaryExpression(n))return n;E=i,a(1,n)}}function yt(e){var n=Ea(e,e.condition,e.questionToken),t=Ea(e,e.questionToken,e.whenTrue),r=Ea(e,e.whenTrue,e.colonToken),i=Ea(e,e.colonToken,e.whenFalse);Ne(e.condition,ie.parenthesizeConditionOfConditionalExpression),pa(n,!0),Ce(e.questionToken),pa(t,!0),Ne(e.whenTrue,ie.parenthesizeBranchOfConditionalExpression),fa(n,t),pa(r,!0),Ce(e.colonToken),pa(i,!0),Ne(e.whenFalse,ie.parenthesizeBranchOfConditionalExpression),fa(r,i)}function ht(e){Ce(e.head),Ki(e,e.templateSpans,262144)}function vt(e){jt(125,e.pos,Zi,e),Ce(e.asteriskToken),Ri(e.expression,ie.parenthesizeExpressionForDisallowedComma)}function bt(e){jt(25,e.pos,Yi,e),Ne(e.expression,ie.parenthesizeExpressionForDisallowedComma)}function Et(e){ka(e.name),sr(e)}function Tt(e){Ne(e.expression,ie.parenthesizeLeftSideOfAccess),Mi(e,e.typeArguments)}function St(e){Ne(e.expression,void 0),e.type&&(ta(),Zi("as"),ta(),Ce(e.type))}function xt(e){Ne(e.expression,ie.parenthesizeLeftSideOfAccess),$i("!")}function Lt(e){sa(e.keywordToken,e.pos,Yi),Yi("."),Ce(e.name)}function Dt(e){Ne(e.expression),Ce(e.literal)}function At(e){Ct(e,!e.multiLine&&Ta(e))}function Ct(n,t){jt(18,n.pos,Yi,n);var r=t||1&e.getEmitFlags(n)?768:129;Ki(n,n.statements,r),jt(19,n.statements.end,Yi,n,!!(1&r))}function It(e){Ai(e,e.modifiers),Ce(e.declarationList),Qi()}function Nt(e){e?Yi(";"):Qi()}function kt(n){Ne(n.expression,ie.parenthesizeExpressionOfExpressionStatement),e.isJsonSourceFile(r)&&!e.nodeIsSynthesized(n.expression)||Qi()}function Rt(e){var n=jt(99,e.pos,Zi,e);ta(),jt(20,n,Yi,e),Ne(e.expression),jt(21,e.expression.end,Yi,e),wi(e,e.thenStatement),e.elseStatement&&(ua(e,e.thenStatement,e.elseStatement),jt(91,e.thenStatement.end,Zi,e),237===e.elseStatement.kind?(ta(),Ce(e.elseStatement)):wi(e,e.elseStatement))}function Ot(e,n){var t=jt(115,n,Zi,e);ta(),jt(20,t,Yi,e),Ne(e.expression),jt(21,e.expression.end,Yi,e)}function wt(n){jt(90,n.pos,Zi,n),wi(n,n.statement),e.isBlock(n.statement)&&!U?ta():ua(n,n.statement,n.expression),Ot(n,n.statement.end),Qi()}function Pt(e){Ot(e,e.pos),wi(e,e.statement)}function Mt(e){var n=jt(97,e.pos,Zi,e);ta();var t=jt(20,n,Yi,e);Bt(e.initializer),t=jt(26,e.initializer?e.initializer.end:t,Yi,e),Ri(e.condition),t=jt(26,e.condition?e.condition.end:t,Yi,e),Ri(e.incrementor),jt(21,e.incrementor?e.incrementor.end:t,Yi,e),wi(e,e.statement)}function Ft(e){var n=jt(97,e.pos,Zi,e);ta(),jt(20,n,Yi,e),Bt(e.initializer),ta(),jt(101,e.initializer.end,Zi,e),ta(),Ne(e.expression),jt(21,e.expression.end,Yi,e),wi(e,e.statement)}function Gt(e){var n=jt(97,e.pos,Zi,e);ta(),Oi(e.awaitModifier),jt(20,n,Yi,e),Bt(e.initializer),ta(),jt(158,e.initializer.end,Zi,e),ta(),Ne(e.expression),jt(21,e.expression.end,Yi,e),wi(e,e.statement)}function Bt(e){void 0!==e&&(253===e.kind?Ce(e):Ne(e))}function Ut(e){jt(86,e.pos,Zi,e),ki(e.label),Qi()}function Vt(e){jt(81,e.pos,Zi,e),ki(e.label),Qi()}function jt(n,t,i,a,o){var s=e.getParseTreeNode(a),c=s&&s.kind===a.kind,l=t;if(c&&r&&(t=e.skipTrivia(r.text,t)),c&&a.pos!==l){var u=o&&r&&!e.positionsAreOnSameLine(l,t,r);u&&aa(),po(l),u&&oa()}if(t=la(n,i,t),c&&a.end!==t){var d=286===a.kind;_o(t,!d,d)}return t}function Kt(e){jt(105,e.pos,Zi,e),Ri(e.expression),Qi()}function Ht(e){var n=jt(116,e.pos,Zi,e);ta(),jt(20,n,Yi,e),Ne(e.expression),jt(21,e.expression.end,Yi,e),wi(e,e.statement)}function Wt(e){var n=jt(107,e.pos,Zi,e);ta(),jt(20,n,Yi,e),Ne(e.expression),jt(21,e.expression.end,Yi,e),ta(),Ce(e.caseBlock)}function zt(e){Ce(e.label),jt(58,e.label.end,Yi,e),ta(),Ce(e.statement)}function Jt(e){jt(109,e.pos,Zi,e),Ri(e.expression),Qi()}function qt(e){jt(111,e.pos,Zi,e),ta(),Ce(e.tryBlock),e.catchClause&&(ua(e,e.tryBlock,e.catchClause),Ce(e.catchClause)),e.finallyBlock&&(ua(e,e.catchClause||e.tryBlock,e.finallyBlock),jt(96,(e.catchClause||e.tryBlock).end,Zi,e),ta(),Ce(e.finallyBlock))}function Xt(e){sa(87,e.pos,Zi),Qi()}function Yt(e){Ce(e.name),Ce(e.exclamationToken),Ci(e.type),Ii(e.initializer,e.type?e.type.end:e.name.end,e,ie.parenthesizeExpressionForDisallowedComma)}function Qt(n){Zi(e.isLet(n)?"let":e.isVarConst(n)?"const":"var"),ta(),Ki(n,n.declarations,528)}function Zt(e){$t(e)}function $t(e){Pi(e,e.decorators),Ai(e,e.modifiers),Zi("function"),Ce(e.asteriskToken),ta(),Ie(e.name),er(e,nr)}function er(n,t){var r=n.body;if(r)if(e.isBlock(r)){var i=65536&e.getEmitFlags(n);i&&aa(),Da(n),e.forEach(n.parameters,Ia),Ia(n.body),t(n),rr(r),Aa(n),i&&oa()}else t(n),ta(),Ne(r,ie.parenthesizeConciseBodyOfArrowFunction);else t(n),Qi()}function nr(e){Fi(e,e.typeParameters),Gi(e,e.parameters),Ci(e.type)}function tr(n){if(1&e.getEmitFlags(n))return!0;if(n.multiLine)return!1;if(!e.nodeIsSynthesized(n)&&!e.rangeIsOnSingleLine(n,r))return!1;if(ma(n,n.statements,2)||ga(n,n.statements,2))return!1;for(var t,i=0,a=n.statements;i<a.length;i++){var o=a[i];if(_a(t,o,2)>0)return!1;t=o}return!0}function rr(e){null===C||void 0===C||C(e),ta(),Yi("{"),aa();var n=tr(e)?ir:ar;ro?ro(e,e.statements,n):n(e),oa(),sa(19,e.statements.end,Yi,e),null===I||void 0===I||I(e)}function ir(e){ar(e,!0)}function ar(e,n){var t=Ei(e.statements),r=p.getTextPos();We(e),0===t&&r===p.getTextPos()&&n?(oa(),Ki(e,e.statements,768),aa()):Ki(e,e.statements,1,void 0,t)}function or(e){sr(e)}function sr(n){e.forEach(n.members,Na),Pi(n,n.decorators),Ai(n,n.modifiers),Zi("class"),n.name&&(ta(),Ie(n.name));var t=65536&e.getEmitFlags(n);t&&aa(),Fi(n,n.typeParameters),Ki(n,n.heritageClauses,0),ta(),Yi("{"),Ki(n,n.members,129),Yi("}"),t&&oa()}function cr(e){Pi(e,e.decorators),Ai(e,e.modifiers),Zi("interface"),ta(),Ce(e.name),Fi(e,e.typeParameters),Ki(e,e.heritageClauses,512),ta(),Yi("{"),Ki(e,e.members,129),Yi("}")}function lr(e){Pi(e,e.decorators),Ai(e,e.modifiers),Zi("type"),ta(),Ce(e.name),Fi(e,e.typeParameters),ta(),Yi("="),ta(),Ce(e.type),Qi()}function ur(e){Ai(e,e.modifiers),Zi("enum"),ta(),Ce(e.name),ta(),Yi("{"),Ki(e,e.members,145),Yi("}")}function dr(n){Ai(n,n.modifiers),1024&~n.flags&&(Zi(16&n.flags?"namespace":"module"),ta()),Ce(n.name);var t=n.body;if(!t)return Qi();while(t&&e.isModuleDeclaration(t))Yi("."),Ce(t.name),t=t.body;ta(),Ce(t)}function pr(n){Da(n),e.forEach(n.statements,Ia),Ct(n,Ta(n)),Aa(n)}function fr(e){jt(18,e.pos,Yi,e),Ki(e,e.clauses,129),jt(19,e.clauses.end,Yi,e,!0)}function mr(e){Ai(e,e.modifiers),jt(100,e.modifiers?e.modifiers.end:e.pos,Zi,e),ta(),e.isTypeOnly&&(jt(150,e.pos,Zi,e),ta()),Ce(e.name),ta(),jt(63,e.name.end,Yi,e),ta(),_r(e.moduleReference),Qi()}function _r(e){79===e.kind?Ne(e):Ce(e)}function gr(e){Ai(e,e.modifiers),jt(100,e.modifiers?e.modifiers.end:e.pos,Zi,e),ta(),e.importClause&&(Ce(e.importClause),ta(),jt(154,e.importClause.end,Zi,e),ta()),Ne(e.moduleSpecifier),Qi()}function yr(e){e.isTypeOnly&&(jt(150,e.pos,Zi,e),ta()),Ce(e.name),e.name&&e.namedBindings&&(jt(27,e.name.end,Yi,e),ta()),Ce(e.namedBindings)}function hr(e){var n=jt(41,e.pos,Yi,e);ta(),jt(127,n,Zi,e),ta(),Ce(e.name)}function vr(e){Ar(e)}function br(e){Cr(e)}function Er(e){var n=jt(93,e.pos,Zi,e);ta(),e.isExportEquals?jt(63,n,$i,e):jt(88,n,Zi,e),ta(),Ne(e.expression,e.isExportEquals?ie.getParenthesizeRightSideOfBinaryForOperator(63):ie.parenthesizeExpressionOfExportDefault),Qi()}function Tr(e){var n=jt(93,e.pos,Zi,e);if(ta(),e.isTypeOnly&&(n=jt(150,n,Zi,e),ta()),e.exportClause?Ce(e.exportClause):n=jt(41,n,Yi,e),e.moduleSpecifier){ta();var t=e.exportClause?e.exportClause.end:n;jt(154,t,Zi,e),ta(),Ne(e.moduleSpecifier)}Qi()}function Sr(e){var n=jt(93,e.pos,Zi,e);ta(),n=jt(127,n,Zi,e),ta(),n=jt(141,n,Zi,e),ta(),Ce(e.name),Qi()}function xr(e){var n=jt(41,e.pos,Yi,e);ta(),jt(127,n,Zi,e),ta(),Ce(e.name)}function Lr(e){Ar(e)}function Dr(e){Cr(e)}function Ar(e){Yi("{"),Ki(e,e.elements,525136),Yi("}")}function Cr(e){e.propertyName&&(Ce(e.propertyName),ta(),jt(127,e.propertyName.end,Zi,e),ta()),Ce(e.name)}function Ir(e){Zi("require"),Yi("("),Ne(e.expression),Yi(")")}function Nr(e){Ce(e.openingElement),Ki(e,e.children,262144),Ce(e.closingElement)}function kr(e){Yi("<"),Kr(e.tagName),Mi(e,e.typeArguments),ta(),Ce(e.attributes),Yi("/>")}function Rr(e){Ce(e.openingFragment),Ki(e,e.children,262144),Ce(e.closingFragment)}function Or(n){if(Yi("<"),e.isJsxOpeningElement(n)){var t=ha(n.tagName,n);Kr(n.tagName),Mi(n,n.typeArguments),n.attributes.properties&&n.attributes.properties.length>0&&ta(),Ce(n.attributes),va(n.attributes,n),fa(t)}Yi(">")}function wr(e){p.writeLiteral(e.text)}function Pr(n){Yi("</"),e.isJsxClosingElement(n)&&Kr(n.tagName),Yi(">")}function Mr(e){Ki(e,e.properties,262656)}function Fr(e){Ce(e.name),Ni("=",Yi,e.initializer,ke)}function Gr(e){Yi("{..."),Ne(e.expression),Yi("}")}function Br(n){var t=!1;return e.forEachTrailingCommentRange((null===r||void 0===r?void 0:r.text)||"",n+1,function(){return t=!0}),t}function Ur(n){var t=!1;return e.forEachLeadingCommentRange((null===r||void 0===r?void 0:r.text)||"",n+1,function(){return t=!0}),t}function Vr(e){return Br(e)||Ur(e)}function jr(n){var t;if(n.expression||!ee&&!e.nodeIsSynthesized(n)&&Vr(n.pos)){var i=r&&!e.nodeIsSynthesized(n)&&e.getLineAndCharacterOfPosition(r,n.pos).line!==e.getLineAndCharacterOfPosition(r,n.end).line;i&&p.increaseIndent();var a=jt(18,n.pos,Yi,n);Ce(n.dotDotDotToken),Ne(n.expression),jt(19,(null===(t=n.expression)||void 0===t?void 0:t.end)||a,Yi,n),i&&p.decreaseIndent()}}function Kr(e){79===e.kind?Ne(e):Ce(e)}function Hr(e){jt(82,e.pos,Zi,e),ta(),Ne(e.expression,ie.parenthesizeExpressionForDisallowedComma),zr(e,e.statements,e.expression.end)}function Wr(e){var n=jt(88,e.pos,Zi,e);zr(e,e.statements,n)}function zr(n,t,i){var a=1===t.length&&(e.nodeIsSynthesized(n)||e.nodeIsSynthesized(t[0])||e.rangeStartPositionsAreOnSameLine(n,t[0],r)),o=163969;a?(sa(58,i,Yi,n),ta(),o&=-130):jt(58,i,Yi,n),Ki(n,t,o)}function Jr(e){ta(),la(e.token,Zi),ta(),Ki(e,e.types,528)}function qr(e){var n=jt(83,e.pos,Zi,e);ta(),e.variableDeclaration&&(jt(20,n,Yi,e),Ce(e.variableDeclaration),jt(21,e.variableDeclaration.end,Yi,e),ta()),Ce(e.block)}function Xr(n){Ce(n.name),Yi(":"),ta();var t=n.initializer;if(0===(512&e.getEmitFlags(t))){var r=e.getCommentRange(t);_o(r.pos)}Ne(t,ie.parenthesizeExpressionForDisallowedComma)}function Yr(e){Ce(e.name),e.objectAssignmentInitializer&&(ta(),Yi("="),ta(),Ne(e.objectAssignmentInitializer,ie.parenthesizeExpressionForDisallowedComma))}function Qr(e){e.expression&&(jt(25,e.pos,Yi,e),Ne(e.expression,ie.parenthesizeExpressionForDisallowedComma))}function Zr(e){Ce(e.name),Ii(e.initializer,e.name.end,e,ie.parenthesizeExpressionForDisallowedComma)}function $r(n){if(V("/**"),n.comment){var t=e.getTextOfJSDocComment(n.comment);if(t)for(var r=t.split(/\r\n?|\n/g),i=0,a=r;i<a.length;i++){var o=a[i];ia(),ta(),Yi("*"),ta(),V(o)}}n.tags&&(1!==n.tags.length||338!==n.tags[0].kind||n.comment?Ki(n,n.tags,33):(ta(),Ce(n.tags[0]))),ta(),V("*/")}function ei(e){di(e.tagName),fi(e.typeExpression),pi(e.comment)}function ni(e){di(e.tagName),Ce(e.name),pi(e.comment)}function ti(e){ta(),Yi("{"),Ce(e.name),Yi("}")}function ri(e){di(e.tagName),ta(),Yi("{"),Ce(e.class),Yi("}"),pi(e.comment)}function ii(e){di(e.tagName),fi(e.constraint),ta(),Ki(e,e.typeParameters,528),pi(e.comment)}function ai(e){di(e.tagName),e.typeExpression&&(304===e.typeExpression.kind?fi(e.typeExpression):(ta(),Yi("{"),V("Object"),e.typeExpression.isArrayType&&(Yi("["),Yi("]")),Yi("}"))),e.fullName&&(ta(),Ce(e.fullName)),pi(e.comment),e.typeExpression&&317===e.typeExpression.kind&&ci(e.typeExpression)}function oi(e){di(e.tagName),e.name&&(ta(),Ce(e.name)),pi(e.comment),li(e.typeExpression)}function si(e){di(e.tagName),pi(e.comment)}function ci(n){Ki(n,e.factory.createNodeArray(n.jsDocPropertyTags),33)}function li(n){n.typeParameters&&Ki(n,e.factory.createNodeArray(n.typeParameters),33),n.parameters&&Ki(n,e.factory.createNodeArray(n.parameters),33),n.type&&(ia(),ta(),Yi("*"),ta(),Ce(n.type))}function ui(e){di(e.tagName),fi(e.typeExpression),ta(),e.isBracketed&&Yi("["),Ce(e.name),e.isBracketed&&Yi("]"),pi(e.comment)}function di(e){Yi("@"),Ce(e)}function pi(n){var t=e.getTextOfJSDocComment(n);t&&(ta(),V(t))}function fi(e){e&&(ta(),Yi("{"),Ce(e.type),Yi("}"))}function mi(n){ia();var t=n.statements;if(ro){var r=0===t.length||!e.isPrologueDirective(t[0])||e.nodeIsSynthesized(t[0]);if(r)return void ro(n,t,hi)}hi(n)}function _i(n){yi(!!n.hasNoDefaultLib,n.syntheticFileReferences||[],n.syntheticTypeReferences||[],n.syntheticLibReferences||[]);for(var t=0,r=n.prepends;t<r.length;t++){var i=r[t];if(e.isUnparsedSource(i)&&i.syntheticReferences)for(var a=0,o=i.syntheticReferences;a<o.length;a++){var s=o[a];Ce(s),ia()}}}function gi(e){e.isDeclarationFile&&yi(e.hasNoDefaultLib,e.referencedFiles,e.typeReferenceDirectives,e.libReferenceDirectives)}function yi(e,n,t,i){if(e){var a=p.getTextPos();na('/// <reference no-default-lib="true"/>'),j&&j.sections.push({pos:a,end:p.getTextPos(),kind:"no-default-lib"}),ia()}if(r&&r.moduleName&&(na('/// <amd-module name="'+r.moduleName+'" />'),ia()),r&&r.amdDependencies)for(var o=0,s=r.amdDependencies;o<s.length;o++){var c=s[o];c.name?na('/// <amd-dependency name="'+c.name+'" path="'+c.path+'" />'):na('/// <amd-dependency path="'+c.path+'" />'),ia()}for(var l=0,u=n;l<u.length;l++){var d=u[l];a=p.getTextPos();na('/// <reference path="'+d.fileName+'" />'),j&&j.sections.push({pos:a,end:p.getTextPos(),kind:"reference",data:d.fileName}),ia()}for(var f=0,m=t;f<m.length;f++){d=m[f],a=p.getTextPos();na('/// <reference types="'+d.fileName+'" />'),j&&j.sections.push({pos:a,end:p.getTextPos(),kind:"type",data:d.fileName}),ia()}for(var _=0,g=i;_<g.length;_++){d=g[_],a=p.getTextPos();na('/// <reference lib="'+d.fileName+'" />'),j&&j.sections.push({pos:a,end:p.getTextPos(),kind:"lib",data:d.fileName}),ia()}}function hi(n){var t=n.statements;Da(n),e.forEach(n.statements,Ia),We(n);var r=e.findIndex(t,function(n){return!e.isPrologueDirective(n)});gi(n),Ki(n,t,1,void 0,-1===r?t.length:r),Aa(n)}function vi(e){Ne(e.expression)}function bi(e){Hi(e,e.elements,528,void 0)}function Ei(n,t,r,i){for(var a=!!t,o=0;o<n.length;o++){var s=n[o];if(!e.isPrologueDirective(s))return o;var c=!r||!r.has(s.expression.text);if(c){a&&(a=!1,xe(t)),ia();var l=p.getTextPos();Ce(s),i&&j&&j.sections.push({pos:l,end:p.getTextPos(),kind:"prologue",data:s.expression.text}),r&&r.add(s.expression.text)}}return n.length}function Ti(e,n){for(var t=0,r=e;t<r.length;t++){var i=r[t];if(!n.has(i.data)){ia();var a=p.getTextPos();Ce(i),j&&j.sections.push({pos:a,end:p.getTextPos(),kind:"prologue",data:i.data}),n&&n.add(i.data)}}}function Si(n){if(e.isSourceFile(n))Ei(n.statements,n);else{for(var t=new e.Set,r=0,i=n.prepends;r<i.length;r++){var a=i[r];Ti(a.prologues,t)}for(var o=0,s=n.sourceFiles;o<s.length;o++){var c=s[o];Ei(c.statements,c,t,!0)}xe(void 0)}}function xi(n){for(var t,r=new e.Set,i=0;i<n.sourceFiles.length;i++){for(var a=n.sourceFiles[i],o=void 0,s=0,c=0,l=a.statements;c<l.length;c++){var u=l[c];if(!e.isPrologueDirective(u))break;r.has(u.expression.text)||(r.add(u.expression.text),(o||(o=[])).push({pos:u.pos,end:u.end,expression:{pos:u.expression.pos,end:u.expression.end,text:u.expression.text}}),s=s<u.end?u.end:s)}o&&(t||(t=[])).push({file:i,text:a.text.substring(0,s),directives:o})}return t}function Li(n){if(e.isSourceFile(n)||e.isUnparsedSource(n)){var t=e.getShebang(n.text);if(t)return na(t),ia(),!0}else{for(var r=0,i=n.prepends;r<i.length;r++){var a=i[r];if(e.Debug.assertNode(a,e.isUnparsedSource),Li(a))return!0}for(var o=0,s=n.sourceFiles;o<s.length;o++){var c=s[o];if(Li(c))return!0}}}function Di(e,n){if(e){var t=V;V=n,Ce(e),V=t}}function Ai(e,n){n&&n.length&&(Ki(e,n,262656),ta())}function Ci(e){e&&(Yi(":"),ta(),Ce(e))}function Ii(e,n,t,r){e&&(ta(),jt(63,n,$i,t),ta(),Ne(e,r))}function Ni(e,n,t,r){t&&(n(e),r(t))}function ki(e){e&&(ta(),Ce(e))}function Ri(e,n){e&&(ta(),Ne(e,n))}function Oi(e){e&&(Ce(e),ta())}function wi(n,t){e.isBlock(t)||1&e.getEmitFlags(n)?(ta(),Ce(t)):(ia(),aa(),e.isEmptyStatement(t)?we(5,t):Ce(t),oa())}function Pi(e,n){Ki(e,n,2146305)}function Mi(e,n){Ki(e,n,53776,ie.parenthesizeMemberOfElementType)}function Fi(n,t){if(e.isFunctionLike(n)&&n.typeArguments)return Mi(n,n.typeArguments);Ki(n,t,53776)}function Gi(e,n){Ki(e,n,2576)}function Bi(n,t){var r=e.singleOrUndefined(t);return r&&r.pos===n.pos&&e.isArrowFunction(n)&&!n.type&&!e.some(n.decorators)&&!e.some(n.modifiers)&&!e.some(n.typeParameters)&&!e.some(r.decorators)&&!e.some(r.modifiers)&&!r.dotDotDotToken&&!r.questionToken&&!r.type&&!r.initializer&&e.isIdentifier(r.name)}function Ui(e,n){Bi(e,n)?Ki(e,n,528):Gi(e,n)}function Vi(e,n){Ki(e,n,8848)}function ji(e){switch(60&e){case 0:break;case 16:Yi(",");break;case 4:ta(),Yi("|");break;case 32:ta(),Yi("*"),ta();break;case 8:ta(),Yi("&");break}}function Ki(e,n,t,r,i,a){Wi(Ce,e,n,t,r,i,a)}function Hi(e,n,t,r,i,a){Wi(Ne,e,n,t,r,i,a)}function Wi(n,t,i,a,o,s,c){void 0===s&&(s=0),void 0===c&&(c=i?i.length-s:0);var l=void 0===i;if(!(l&&16384&a)){var u=void 0===i||s>=i.length||0===c;if(u&&32768&a)return R&&R(i),void(O&&O(i));if(15360&a&&(Yi(N(a)),u&&i&&_o(i.pos,!0)),R&&R(i),u)1&a&&(!U||t&&!e.rangeIsOnSingleLine(t,r))?ia():256&a&&!(524288&a)&&ta();else{e.Debug.type(i);var p=0===(262144&a),f=p,m=ma(t,i,a);m?(ia(m),f=!1):256&a&&ta(),128&a&&aa();for(var _=void 0,g=void 0,y=!1,h=0;h<c;h++){var v=i[s+h];if(32&a)ia(),ji(a);else if(_){60&a&&_.end!==(t?t.end:-1)&&po(_.end),ji(a),ge(g);var b=_a(_,v,a);b>0?(0===(131&a)&&(aa(),y=!0),ia(b),f=!1):_&&512&a&&ta()}if(g=_e(v),f){if(_o){var E=e.getCommentRange(v);_o(E.pos)}}else f=p;d=v.pos,1===n.length?n(v):n(v,o),y&&(oa(),y=!1),_=v}var T=_?e.getEmitFlags(_):0,S=ee||!!(1024&T),x=(null===i||void 0===i?void 0:i.hasTrailingComma)&&64&a&&16&a;x&&(_&&!S?jt(27,_.end,Yi,_):Yi(",")),_&&(t?t.end:-1)!==_.end&&60&a&&!S&&po(x&&(null===i||void 0===i?void 0:i.end)?i.end:_.end),128&a&&oa(),ge(g);var L=ga(t,i,a);L?ia(L):2097408&a&&ta()}O&&O(i),15360&a&&(u&&i&&po(i.end),Yi(k(a)))}}function zi(e){p.writeLiteral(e)}function Ji(e){p.writeStringLiteral(e)}function qi(e){p.write(e)}function Xi(e,n){p.writeSymbol(e,n)}function Yi(e){p.writePunctuation(e)}function Qi(){p.writeTrailingSemicolon(";")}function Zi(e){p.writeKeyword(e)}function $i(e){p.writeOperator(e)}function ea(e){p.writeParameter(e)}function na(e){p.writeComment(e)}function ta(){p.writeSpace(" ")}function ra(e){p.writeProperty(e)}function ia(e){void 0===e&&(e=1);for(var n=0;n<e;n++)p.writeLine(n>0)}function aa(){p.increaseIndent()}function oa(){p.decreaseIndent()}function sa(e,n,t,r){return J?la(e,t,n):Ro(r,e,t,n,la)}function ca(n,t){w&&w(n),t(e.tokenToString(n.kind)),P&&P(n)}function la(n,t,r){var i=e.tokenToString(n);return t(i),r<0?r:r+i.length}function ua(n,t,r){if(1&e.getEmitFlags(n))ta();else if(U){var i=Ea(n,t,r);i?ia(i):ta()}else ia()}function da(n){for(var t=n.split(/\r\n?|\n/g),r=e.guessIndentation(t),i=0,a=t;i<a.length;i++){var o=a[i],s=r?o.slice(r):o;s.length&&(ia(),V(s))}}function pa(e,n){e?(aa(),ia(e)):n&&ta()}function fa(e,n){e&&oa(),n&&oa()}function ma(n,t,i){if(2&i||U){if(65536&i)return 1;var a=t[0];if(void 0===a)return!n||e.rangeIsOnSingleLine(n,r)?0:1;if(a.pos===d)return 0;if(11===a.kind)return 0;if(n&&!e.positionIsSynthesized(n.pos)&&!e.nodeIsSynthesized(a)&&(!a.parent||e.getOriginalNode(a.parent)===e.getOriginalNode(n)))return U?ya(function(t){return e.getLinesBetweenPositionAndPrecedingNonWhitespaceCharacter(a.pos,n.pos,r,t)}):e.rangeStartPositionsAreOnSameLine(n,a,r)?0:1;if(ba(a,i))return 1}return 1&i?1:0}function _a(n,t,i){if(2&i||U){if(void 0===n||void 0===t)return 0;if(11===t.kind)return 0;if(!e.nodeIsSynthesized(n)&&!e.nodeIsSynthesized(t))return U&&ao(n,t)?ya(function(i){return e.getLinesBetweenRangeEndAndRangeStart(n,t,r,i)}):!U&&io(n,t)?e.rangeEndIsOnSameLineAsRangeStart(n,t,r)?0:1:65536&i?1:0;if(ba(n,i)||ba(t,i))return 1}else if(e.getStartsOnNewLine(t))return 1;return 1&i?1:0}function ga(n,t,i){if(2&i||U){if(65536&i)return 1;var a=e.lastOrUndefined(t);if(void 0===a)return!n||e.rangeIsOnSingleLine(n,r)?0:1;if(n&&!e.positionIsSynthesized(n.pos)&&!e.nodeIsSynthesized(a)&&(!a.parent||a.parent===n)){if(U){var o=e.isNodeArray(t)&&!e.positionIsSynthesized(t.end)?t.end:a.end;return ya(function(t){return e.getLinesBetweenPositionAndNextNonWhitespaceCharacter(o,n.end,r,t)})}return e.rangeEndPositionsAreOnSameLine(n,a,r)?0:1}if(ba(a,i))return 1}return 1&i&&!(131072&i)?1:0}function ya(n){e.Debug.assert(!!U);var t=n(!0);return 0===t?n(!1):t}function ha(e,n){var t=U&&ma(n,[e],0);return t&&pa(t,!1),!!t}function va(e,n){var t=U&&ga(n,[e],0);t&&ia(t)}function ba(n,t){if(e.nodeIsSynthesized(n)){var r=e.getStartsOnNewLine(n);return void 0===r?0!==(65536&t):r}return 0!==(65536&t)}function Ea(n,t,i){return 131072&e.getEmitFlags(n)?0:(n=Sa(n),t=Sa(t),i=Sa(i),e.getStartsOnNewLine(i)?1:e.nodeIsSynthesized(n)||e.nodeIsSynthesized(t)||e.nodeIsSynthesized(i)?0:U?ya(function(n){return e.getLinesBetweenRangeEndAndRangeStart(t,i,r,n)}):e.rangeEndIsOnSameLineAsRangeStart(t,i,r)?0:1)}function Ta(n){return 0===n.statements.length&&e.rangeEndIsOnSameLineAsRangeStart(n,n,r)}function Sa(n){while(210===n.kind&&e.nodeIsSynthesized(n))n=n.expression;return n}function xa(n,t){return e.isGeneratedIdentifier(n)?Ra(n):(e.isIdentifier(n)||e.isPrivateIdentifier(n))&&(e.nodeIsSynthesized(n)||!n.parent||!r||n.parent&&r&&e.getSourceFileOfNode(n)!==e.getOriginalNode(r))?e.idText(n):10===n.kind&&n.textSourceNode?xa(n.textSourceNode,t):!e.isLiteralExpression(n)||!e.nodeIsSynthesized(n)&&n.parent?e.getSourceTextOfNodeFromSourceFile(r,n,t):n.text}function La(t,i,a){if(10===t.kind&&t.textSourceNode){var o=t.textSourceNode;if(e.isIdentifier(o)||e.isNumericLiteral(o)){var s=e.isNumericLiteral(o)?o.text:xa(o);return a?'"'+e.escapeJsxAttributeString(s)+'"':i||16777216&e.getEmitFlags(t)?'"'+e.escapeString(s)+'"':'"'+e.escapeNonAsciiString(s)+'"'}return La(o,i,a)}var c=(i?1:0)|(a?2:0)|(n.terminateUnterminatedLiterals?4:0)|(n.target&&99===n.target?8:0);return e.getLiteralText(t,r,c)}function Da(n){n&&524288&e.getEmitFlags(n)||(s.push(c),c=0,l.push(u))}function Aa(n){n&&524288&e.getEmitFlags(n)||(c=s.pop(),u=l.pop())}function Ca(n){u&&u!==e.lastOrUndefined(l)||(u=new e.Set),u.add(n)}function Ia(n){if(n)switch(n.kind){case 233:e.forEach(n.statements,Ia);break;case 248:case 246:case 238:case 239:Ia(n.statement);break;case 237:Ia(n.thenStatement),Ia(n.elseStatement);break;case 240:case 242:case 241:Ia(n.initializer),Ia(n.statement);break;case 247:Ia(n.caseBlock);break;case 261:e.forEach(n.clauses,Ia);break;case 287:case 288:e.forEach(n.statements,Ia);break;case 250:Ia(n.tryBlock),Ia(n.catchClause),Ia(n.finallyBlock);break;case 290:Ia(n.variableDeclaration),Ia(n.block);break;case 235:Ia(n.declarationList);break;case 253:e.forEach(n.declarations,Ia);break;case 252:case 162:case 201:case 255:ka(n.name);break;case 254:ka(n.name),524288&e.getEmitFlags(n)&&(e.forEach(n.parameters,Ia),Ia(n.body));break;case 199:case 200:e.forEach(n.elements,Ia);break;case 264:Ia(n.importClause);break;case 265:ka(n.name),Ia(n.namedBindings);break;case 266:ka(n.name);break;case 272:ka(n.name);break;case 267:e.forEach(n.elements,Ia);break;case 268:ka(n.propertyName||n.name);break}}function Na(e){if(e)switch(e.kind){case 291:case 292:case 165:case 167:case 170:case 171:ka(e.name);break}}function ka(n){n&&(e.isGeneratedIdentifier(n)?Ra(n):e.isBindingPattern(n)&&Ia(n))}function Ra(e){if(4===(7&e.autoGenerateFlags))return Oa(Ja(e),e.autoGenerateFlags);var n=e.autoGenerateId;return a[n]||(a[n]=za(e))}function Oa(n,t){var r=e.getNodeId(n);return i[r]||(i[r]=Wa(n,t))}function wa(e){return Pa(e)&&!o.has(e)&&!(u&&u.has(e))}function Pa(n){return!r||e.isFileLevelUniqueName(r,n,T)}function Ma(n,t){for(var r=t;e.isNodeDescendantOf(r,t);r=r.nextContainer)if(r.locals){var i=r.locals.get(e.escapeLeadingUnderscores(n));if(i&&3257279&i.flags)return!1}return!0}function Fa(e,n){if(e&&!(c&e)){var t=268435456===e?"_i":"_n";if(wa(t))return c|=e,n&&Ca(t),t}while(1){var r=268435455&c;if(c++,8!==r&&13!==r){t=r<26?"_"+String.fromCharCode(97+r):"_"+(r-26);if(wa(t))return n&&Ca(t),t}}}function Ga(e,n,t,r){if(void 0===n&&(n=wa),t&&n(e))return r?Ca(e):o.add(e),e;95!==e.charCodeAt(e.length-1)&&(e+="_");var i=1;while(1){var a=e+i;if(n(a))return r?Ca(a):o.add(a),a;i++}}function Ba(e){return Ga(e,Pa,!0)}function Ua(e){var n=xa(e.name);return Ma(n,e)?n:Ga(n)}function Va(n){var t=e.getExternalModuleName(n),r=e.isStringLiteral(t)?e.makeIdentifierFromModuleName(t.text):"module";return Ga(r)}function ja(){return Ga("default")}function Ka(){return Ga("class")}function Ha(n){return e.isIdentifier(n.name)?Oa(n.name):Fa(0)}function Wa(e,n){switch(e.kind){case 79:return Ga(xa(e),wa,!!(16&n),!!(8&n));case 259:case 258:return Ua(e);case 264:case 270:return Va(e);case 254:case 255:case 269:return ja();case 224:return Ka();case 167:case 170:case 171:return Ha(e);case 160:return Fa(0,!0);default:return Fa(0)}}function za(n){switch(7&n.autoGenerateFlags){case 1:return Fa(0,!!(8&n.autoGenerateFlags));case 2:return Fa(268435456,!!(8&n.autoGenerateFlags));case 3:return Ga(e.idText(n),32&n.autoGenerateFlags?Pa:wa,!!(16&n.autoGenerateFlags),!!(8&n.autoGenerateFlags))}return e.Debug.fail("Unsupported GeneratedIdentifierKind.")}function Ja(n){var t=n.autoGenerateId,r=n,i=r.original;while(i){if(r=i,e.isIdentifier(r)&&4&r.autoGenerateFlags&&r.autoGenerateId!==t)break;i=r.original}return r}function qa(e,n){var t=Ge(2,e,n),r=Y,i=Q,a=Z;Xa(n),t(e,n),Ya(n,r,i,a)}function Xa(n){var t=e.getEmitFlags(n),r=e.getCommentRange(n);Qa(n,t,r.pos,r.end),2048&t&&(ee=!0)}function Ya(n,t,r,i){var a=e.getEmitFlags(n),o=e.getCommentRange(n);2048&a&&(ee=!1),Za(n,a,o.pos,o.end,t,r,i)}function Qa(n,t,r,i){te(),$=!1;var a=r<0||0!==(512&t)||11===n.kind,o=i<0||0!==(1024&t)||11===n.kind;(r>0||i>0)&&r!==i&&(a||oo(r,344!==n.kind),(!a||r>=0&&0!==(512&t))&&(Y=r),(!o||i>=0&&0!==(1024&t))&&(Q=i,253===n.kind&&(Z=i))),e.forEach(e.getSyntheticLeadingComments(n),$a),re()}function Za(n,t,r,i,a,o,s){te();var c=i<0||0!==(1024&t)||11===n.kind;e.forEach(e.getSyntheticTrailingComments(n),eo),(r>0||i>0)&&r!==i&&(Y=a,Q=o,Z=s,c||344===n.kind||fo(i)),re()}function $a(e){(e.hasLeadingNewline||2===e.kind)&&p.writeLine(),no(e),e.hasTrailingNewLine||2===e.kind?p.writeLine():p.writeSpace(" ")}function eo(e){p.isAtStartOfLine()||p.writeSpace(" "),no(e),e.hasTrailingNewLine&&p.writeLine()}function no(n){var t=to(n),r=3===n.kind?e.computeLineStarts(t):void 0;e.writeCommentRange(t,r,p,0,t.length,F)}function to(e){return 3===e.kind?"/*"+e.text+"*/":"//"+e.text}function ro(n,t,r){te();var i=t.pos,a=t.end,o=e.getEmitFlags(n),s=i<0||0!==(512&o),c=ee||a<0||0!==(1024&o);s||To(t),re(),2048&o&&!ee?(ee=!0,r(n),ee=!1):r(n),te(),c||(oo(t.end,!0),$&&!p.isAtStartOfLine()&&p.writeLine()),re()}function io(n,t){return n=e.getOriginalNode(n),n.parent&&n.parent===e.getOriginalNode(t).parent}function ao(n,t){if(t.pos<n.end)return!1;n=e.getOriginalNode(n),t=e.getOriginalNode(t);var r=n.parent;if(!r||r!==t.parent)return!1;var i=e.getContainingNodeArray(n),a=null===i||void 0===i?void 0:i.indexOf(n);return void 0!==a&&a>-1&&i.indexOf(t)===a+1}function oo(e,n){$=!1,n?0===e&&(null===r||void 0===r?void 0:r.isDeclarationFile)?ho(e,co):ho(e,uo):0===e&&ho(e,so)}function so(e,n,t,r,i){xo(e,n)&&uo(e,n,t,r,i)}function co(e,n,t,r,i){xo(e,n)||uo(e,n,t,r,i)}function lo(t,r){return!n.onlyPrintJsDocStyle||(e.isJSDocLikeText(t,r)||e.isPinnedComment(t,r))}function uo(n,t,i,a,o){lo(r.text,n)&&($||(e.emitNewLineBeforeLeadingCommentOfPosition(Ae(),p,o,n),$=!0),No(n),e.writeCommentRange(r.text,Ae(),p,n,t,F),No(t),a?p.writeLine():3===i&&p.writeSpace(" "))}function po(e){ee||-1===e||oo(e,!0)}function fo(e){vo(e,mo)}function mo(n,t,i,a){lo(r.text,n)&&(p.isAtStartOfLine()||p.writeSpace(" "),No(n),e.writeCommentRange(r.text,Ae(),p,n,t,F),No(t),a&&p.writeLine())}function _o(e,n,t){ee||(te(),vo(e,n?mo:t?go:yo),re())}function go(n,t,i){No(n),e.writeCommentRange(r.text,Ae(),p,n,t,F),No(t),2===i&&p.writeLine()}function yo(n,t,i,a){No(n),e.writeCommentRange(r.text,Ae(),p,n,t,F),No(t),a?p.writeLine():p.writeSpace(" ")}function ho(n,t){!r||-1!==Y&&n===Y||(bo(n)?Eo(t):e.forEachLeadingCommentRange(r.text,n,t,n))}function vo(n,t){r&&(-1===Q||n!==Q&&n!==Z)&&e.forEachTrailingCommentRange(r.text,n,t)}function bo(n){return void 0!==v&&e.last(v).nodePos===n}function Eo(n){var t=e.last(v).detachedCommentEndPos;v.length-1?v.pop():v=void 0,e.forEachLeadingCommentRange(r.text,t,n,t)}function To(n){var t=e.emitDetachedComments(r.text,Ae(),p,So,n,F,ee);t&&(v?v.push(t):v=[t])}function So(n,t,i,a,o,s){lo(r.text,a)&&(No(a),e.writeCommentRange(n,t,i,a,o,s),No(o))}function xo(n,t){return e.isRecognizedTripleSlashComment(r.text,n,t)}function Lo(n){return void 0===n.parsedSourceMap&&void 0!==n.sourceMapText&&(n.parsedSourceMap=e.tryParseRawSourceMap(n.sourceMapText)||!1),n.parsedSourceMap||void 0}function Do(e,n){var t=Ge(3,e,n);Ao(n),t(e,n),Co(n)}function Ao(n){var t=e.getEmitFlags(n),r=e.getSourceMapRange(n);if(e.isUnparsedNode(n)){e.Debug.assertIsDefined(n.parent,"UnparsedNodes must have parent pointers");var i=Lo(n.parent);i&&_&&_.appendSourceMap(p.getLine(),p.getColumn(),i,n.parent.sourceMapPath,n.parent.getLineAndCharacterOfPosition(n.pos),n.parent.getLineAndCharacterOfPosition(n.end))}else{var a=r.source||g;344!==n.kind&&0===(16&t)&&r.pos>=0&&ko(r.source||g,Io(a,r.pos)),64&t&&(J=!0)}}function Co(n){var t=e.getEmitFlags(n),r=e.getSourceMapRange(n);e.isUnparsedNode(n)||(64&t&&(J=!1),344!==n.kind&&0===(32&t)&&r.end>=0&&ko(r.source||g,r.end))}function Io(n,t){return n.skipTrivia?n.skipTrivia(t):e.skipTrivia(n.text,t)}function No(n){if(!(J||e.positionIsSynthesized(n)||Po(g))){var t=e.getLineAndCharacterOfPosition(g,n),r=t.line,i=t.character;_.addMapping(p.getLine(),p.getColumn(),q,r,i,void 0)}}function ko(e,n){if(e!==g){var t=g,r=q;Oo(e),No(n),wo(t,r)}else No(n)}function Ro(n,t,r,i,a){if(J||n&&e.isInJsonFile(n))return a(t,r,i);var o=n&&n.emitNode,s=o&&o.flags||0,c=o&&o.tokenSourceMapRanges&&o.tokenSourceMapRanges[t],l=c&&c.source||g;return i=Io(l,c?c.pos:i),0===(128&s)&&i>=0&&ko(l,i),i=a(t,r,i),c&&(i=c.end),0===(256&s)&&i>=0&&ko(l,i),i}function Oo(e){J||(g=e,e!==y?Po(e)||(q=_.addSource(e.fileName),n.inlineSources&&_.setSourceContent(q,e.text),y=e,X=q):q=X)}function wo(e,n){g=e,q=n}function Po(n){return e.fileExtensionIs(n.fileName,".json")}}function I(){var e=[];return e[1024]=["{","}"],e[2048]=["(",")"],e[4096]=["<",">"],e[8192]=["[","]"],e}function N(e){return r[15360&e][0]}function k(e){return r[15360&e][1]}e.isBuildInfoFile=i,e.forEachEmittedFile=o,e.getTsBuildInfoEmitOutputFilePath=s,e.getOutputPathsForBundle=c,e.getOutputPathsFor=l,e.getOutputExtension=d,e.getOutputDeclarationFileName=f,e.getCommonSourceDirectory=h,e.getCommonSourceDirectoryOfConfig=v,e.getAllProjectOutputs=b,e.getOutputFileNames=E,e.getFirstProjectOutput=T,e.emitFiles=S,e.getBuildInfoText=x,e.getBuildInfo=L,e.notImplementedResolver={hasGlobalName:e.notImplemented,getReferencedExportContainer:e.notImplemented,getReferencedImportDeclaration:e.notImplemented,getReferencedDeclarationWithCollidingName:e.notImplemented,isDeclarationWithCollidingName:e.notImplemented,isValueAliasDeclaration:e.notImplemented,isReferencedAliasDeclaration:e.notImplemented,isTopLevelValueImportEqualsWithEntityName:e.notImplemented,getNodeCheckFlags:e.notImplemented,isDeclarationVisible:e.notImplemented,isLateBound:function(e){return!1},collectLinkedAliases:e.notImplemented,isImplementationOfOverload:e.notImplemented,isRequiredInitializedParameter:e.notImplemented,isOptionalUninitializedParameterProperty:e.notImplemented,isExpandoFunctionDeclaration:e.notImplemented,getPropertiesOfContainerFunction:e.notImplemented,createTypeOfDeclaration:e.notImplemented,createReturnTypeOfSignatureDeclaration:e.notImplemented,createTypeOfExpression:e.notImplemented,createLiteralConstValue:e.notImplemented,isSymbolAccessible:e.notImplemented,isEntityNameVisible:e.notImplemented,getConstantValue:e.notImplemented,getReferencedValueDeclaration:e.notImplemented,getTypeReferenceSerializationKind:e.notImplemented,isOptionalParameter:e.notImplemented,moduleExportsSomeValue:e.notImplemented,isArgumentsLocalBinding:e.notImplemented,getExternalModuleFileFromDeclaration:e.notImplemented,getTypeReferenceDirectivesForEntityName:e.notImplemented,getTypeReferenceDirectivesForSymbol:e.notImplemented,isLiteralConstDeclaration:e.notImplemented,getJsxFactoryEntity:e.notImplemented,getJsxFragmentFactoryEntity:e.notImplemented,getAllAccessorDeclarations:e.notImplemented,getSymbolOfExternalModuleSpecifier:e.notImplemented,isBindingCapturedByNode:e.notImplemented,getDeclarationStatementsForSourceFile:e.notImplemented,isImportRequiredByAugmentation:e.notImplemented},e.emitUsingBuildInfo=A,function(e){e[e["Notification"]=0]="Notification",e[e["Substitution"]=1]="Substitution",e[e["Comments"]=2]="Comments",e[e["SourceMaps"]=3]="SourceMaps",e[e["Emit"]=4]="Emit"}(n||(n={})),e.createPrinter=C,function(e){e[e["Auto"]=0]="Auto",e[e["CountMask"]=268435455]="CountMask",e[e["_i"]=268435456]="_i"}(t||(t={}))}(d||(d={})),function(e){function n(n,t,r){if(n.getDirectories&&n.readDirectory){var i=new e.Map,a=e.createGetCanonicalFileName(r);return{useCaseSensitiveFileNames:r,fileExists:g,readFile:function(e,t){return n.readFile(e,t)},directoryExists:n.directoryExists&&y,getDirectories:v,readDirectory:b,createDirectory:n.createDirectory&&h,writeFile:n.writeFile&&_,addOrDeleteFileOrDirectory:T,addOrDeleteFile:S,clearCache:L,realpath:n.realpath&&E}}function o(n){return e.toPath(n,t,a)}function s(n){return i.get(e.ensureTrailingDirectorySeparator(n))}function c(n){return s(e.getDirectoryPath(n))}function l(n){return e.getBaseFileName(e.normalizePath(n))}function u(t,r){var a;if(!n.realpath||e.ensureTrailingDirectorySeparator(o(n.realpath(t)))===r){var s={files:e.map(n.readDirectory(t,void 0,void 0,["*.*"]),l)||[],directories:n.getDirectories(t)||[]};return i.set(e.ensureTrailingDirectorySeparator(r),s),s}if(null===(a=n.directoryExists)||void 0===a?void 0:a.call(n,t))return i.set(r,!1),!1}function d(n,t){t=e.ensureTrailingDirectorySeparator(t);var r=s(t);if(r)return r;try{return u(n,t)}catch(a){return void e.Debug.assert(!i.has(e.ensureTrailingDirectorySeparator(t)))}}function p(e,n){return a(e)===a(n)}function f(n,t){return e.some(n,function(e){return p(e,t)})}function m(n,t,r){if(f(n,t)){if(!r)return e.filterMutate(n,function(e){return!p(e,t)})}else if(r)return n.push(t)}function _(e,t,r){var i=o(e),a=c(i);return a&&x(a,l(e),!0),n.writeFile(e,t,r)}function g(e){var t=o(e),r=c(t);return r&&f(r.files,l(e))||n.fileExists(e)}function y(t){var r=o(t);return i.has(e.ensureTrailingDirectorySeparator(r))||n.directoryExists(t)}function h(e){var t=o(e),r=c(t),i=l(e);r&&m(r.directories,i,!0),n.createDirectory(e)}function v(e){var t=o(e),r=d(e,t);return r?r.directories.slice():n.getDirectories(e)}function b(i,a,s,c,u){var p,f=o(i),m=d(i,f);return void 0!==m?e.matchFiles(i,a,s,c,r,t,u,_,E,y):n.readDirectory(i,a,s,c,u);function _(n){var t=o(n);if(t===f)return m||g(n,t);var r=d(n,t);return void 0!==r?r||g(n,t):e.emptyFileSystemEntries}function g(t,r){if(p&&r===f)return p;var i={files:e.map(n.readDirectory(t,void 0,void 0,["*.*"]),l)||e.emptyArray,directories:n.getDirectories(t)||e.emptyArray};return r===f&&(p=i),i}}function E(e){return n.realpath?n.realpath(e):e}function T(e,t){var r=s(t);if(void 0===r){var i=c(t);if(i){if(n.directoryExists){var a=l(e),o={fileExists:n.fileExists(t),directoryExists:n.directoryExists(t)};return o.directoryExists||f(i.directories,a)?L():x(i,a,o.fileExists),o}L()}}else L()}function S(n,t,r){if(r!==e.FileWatcherEventKind.Changed){var i=c(t);i&&x(i,l(n),r===e.FileWatcherEventKind.Created)}}function x(e,n,t){m(e.files,n,t)}function L(){i.clear()}}var t;function r(n,t,r,i,a){var o,s=e.arrayToMap((null===(o=null===t||void 0===t?void 0:t.configFile)||void 0===o?void 0:o.extendedSourceFiles)||e.emptyArray,a);r.forEach(function(e,t){s.has(t)||(e.projects.delete(n),e.close())}),s.forEach(function(t,a){var o=r.get(a);o?o.projects.add(n):r.set(a,{projects:new e.Set([n]),watcher:i(t,a),close:function(){var e=r.get(a);e&&0===e.projects.size&&(e.watcher.close(),r.delete(a))}})})}function i(e,n){n.forEach(function(n){n.projects.delete(e)&&n.close()})}function o(e,n,t){e.delete(n)&&e.forEach(function(r,i){var a,s=r.extendedResult;(null===(a=s.extendedSourceFiles)||void 0===a?void 0:a.some(function(e){return t(e)===n}))&&o(e,i,t)})}function s(n,t,r){var i=new e.Map(n);e.mutateMap(t,i,{createNewValue:r,onDeleteValue:e.closeFileWatcher})}function c(n,t,r){var i=n.getMissingFilePaths(),a=e.arrayToMap(i,e.identity,e.returnTrue);e.mutateMap(t,a,{createNewValue:r,onDeleteValue:e.closeFileWatcher})}function l(n,t,r){function i(e,n){return{watcher:r(e,n),flags:n}}function a(e,t,r){e.flags!==t&&(e.watcher.close(),n.set(r,i(r,t)))}e.mutateMap(n,t,{createNewValue:i,onDeleteValue:_,onExistingValue:a})}function u(n){var t=n.watchedDirPath,r=n.fileOrDirectory,i=n.fileOrDirectoryPath,a=n.configFileName,o=n.options,s=n.program,c=n.extraFileExtensions,l=n.currentDirectory,u=n.useCaseSensitiveFileNames,p=n.writeLog,f=n.toPath,m=e.removeIgnoredPath(i);if(!m)return p("Project: "+a+" Detected ignored path: "+r),!0;if(i=m,i===t)return!1;if(e.hasExtension(i)&&!e.isSupportedSourceFileName(r,o,c))return p("Project: "+a+" Detected file add/remove of non supported extension: "+r),!0;if(e.isExcludedFile(r,o.configFile.configFileSpecs,e.getNormalizedAbsolutePath(e.getDirectoryPath(a),l),u,l))return p("Project: "+a+" Detected excluded file: "+r),!0;if(!s)return!1;if(e.outFile(o)||o.outDir)return!1;if(e.fileExtensionIs(i,".d.ts")){if(o.declarationDir)return!1}else if(!e.fileExtensionIsOneOf(i,e.supportedJSExtensions))return!1;var _=e.removeFileExtension(i),g=e.isArray(s)?void 0:d(s)?s.getProgramOrUndefined():s,y=g||e.isArray(s)?void 0:s;return!(!h(_+".ts")&&!h(_+".tsx"))&&(p("Project: "+a+" Detected output file: "+r),!0);function h(n){return g?!!g.getSourceFileByPath(n):y?y.getState().fileInfos.has(n):!!e.find(s,function(e){return f(e)===n})}}function d(e){return!!e.getState}function p(e,n){return!!e&&e.isEmittedFile(n)}function f(n,r,i,o){e.setSysLog(r===t.Verbose?i:e.noop);var s={watchFile:function(e,t,r,i){return n.watchFile(e,t,r,i)},watchDirectory:function(e,t,r,i){return n.watchDirectory(e,t,0!==(1&r),i)}},c=r!==t.None?{watchFile:g("watchFile"),watchDirectory:g("watchDirectory")}:void 0,l=r===t.Verbose?{watchFile:m,watchDirectory:_}:c||s,u=r===t.Verbose?f:e.returnNoopFileWatcher;return{watchFile:d("watchFile"),watchDirectory:d("watchDirectory")};function d(t){return function(r,i,a,o,s,c){var d;return e.matchesExclude(r,"watchFile"===t?null===o||void 0===o?void 0:o.excludeFiles:null===o||void 0===o?void 0:o.excludeDirectories,p(),(null===(d=n.getCurrentDirectory)||void 0===d?void 0:d.call(n))||"")?u(r,a,o,s,c):l[t].call(void 0,r,i,a,o,s,c)}}function p(){return"boolean"===typeof n.useCaseSensitiveFileNames?n.useCaseSensitiveFileNames:n.useCaseSensitiveFileNames()}function f(e,n,t,r,a){return i("ExcludeWatcher:: Added:: "+y(e,n,t,r,a,o)),{close:function(){return i("ExcludeWatcher:: Close:: "+y(e,n,t,r,a,o))}}}function m(e,n,t,r,a,s){i("FileWatcher:: Added:: "+y(e,t,r,a,s,o));var l=c.watchFile(e,n,t,r,a,s);return{close:function(){i("FileWatcher:: Close:: "+y(e,t,r,a,s,o)),l.close()}}}function _(n,t,r,a,s,l){var u="DirectoryWatcher:: Added:: "+y(n,r,a,s,l,o);i(u);var d=e.timestamp(),p=c.watchDirectory(n,t,r,a,s,l),f=e.timestamp()-d;return i("Elapsed:: "+f+"ms "+u),{close:function(){var t="DirectoryWatcher:: Close:: "+y(n,r,a,s,l,o);i(t);var c=e.timestamp();p.close();var u=e.timestamp()-c;i("Elapsed:: "+u+"ms "+t)}}}function g(n){return function(t,r,c,l,u,d){return s[n].call(void 0,t,function(){for(var s=[],p=0;p<arguments.length;p++)s[p]=arguments[p];var f=("watchFile"===n?"FileWatcher":"DirectoryWatcher")+":: Triggered with "+s[0]+" "+(void 0!==s[1]?s[1]:"")+":: "+y(t,c,l,u,d,o);i(f);var m=e.timestamp();r.call.apply(r,a([void 0],s,!1));var _=e.timestamp()-m;i("Elapsed:: "+_+"ms "+f)},c,l,u,d)}}function y(e,n,t,r,i,a){return"WatchInfo: "+e+" "+n+" "+JSON.stringify(t)+" "+(a?a(r,i):void 0===i?r:r+" "+i)}}function m(n){var t=null===n||void 0===n?void 0:n.fallbackPolling;return{watchFile:void 0!==t?t:e.WatchFileKind.PriorityPollingInterval}}function _(e){e.watcher.close()}e.createCachedDirectoryStructureHost=n,function(e){e[e["None"]=0]="None",e[e["Partial"]=1]="Partial",e[e["Full"]=2]="Full"}(e.ConfigFileProgramReloadLevel||(e.ConfigFileProgramReloadLevel={})),e.updateSharedExtendedConfigFileWatcher=r,e.clearSharedExtendedConfigFileWatcher=i,e.cleanExtendedConfigCache=o,e.updatePackageJsonWatch=s,e.updateMissingFilePathsWatch=c,e.updateWatchingWildcardDirectories=l,e.isIgnoredFileFromWildCardWatching=u,e.isEmittedFileOfProgram=p,function(e){e[e["None"]=0]="None",e[e["TriggerOnly"]=1]="TriggerOnly",e[e["Verbose"]=2]="Verbose"}(t=e.WatchLogLevel||(e.WatchLogLevel={})),e.getWatchFactory=f,e.getFallbackOptions=m,e.closeFileWatcherOf=_}(d||(d={})),function(e){function n(n,t,r){return void 0===r&&(r="tsconfig.json"),e.forEachAncestorDirectory(n,function(n){var i=e.combinePaths(n,r);return t(i)?i:void 0})}function t(n,t){var r=e.getDirectoryPath(t),i=e.isRootedDiskPath(n)?n:e.combinePaths(r,n);return e.normalizePath(i)}function r(n,t,r){var i,a=e.forEach(n,function(n){var a=e.getNormalizedPathComponents(n,t);if(a.pop(),i){for(var o=Math.min(i.length,a.length),s=0;s<o;s++)if(r(i[s])!==r(a[s])){if(0===s)return!0;i.length=s;break}a.length<i.length&&(i.length=a.length)}else i=a});return a?"":i?e.getPathFromPathComponents(i):t}function i(e,n){return o(e,n)}function o(n,t,r){void 0===r&&(r=e.sys);var i,a=new e.Map,o=e.createGetCanonicalFileName(r.useCaseSensitiveFileNames),s=e.maybeBind(r,r.createHash)||e.generateDjb2Hash;function c(n,r,i){var a;try{e.performance.mark("beforeIORead"),a=_.readFile(n),e.performance.mark("afterIORead"),e.performance.measure("I/O Read","beforeIORead","afterIORead")}catch(o){i&&i(o.message),a=""}return void 0!==a?e.createSourceFile(n,a,r,t):void 0}function l(e){return!!a.has(e)||!!(_.directoryExists||r.directoryExists)(e)&&(a.set(e,!0),!0)}function u(n,t,i,a){try{e.performance.mark("beforeIOWrite"),e.writeFileEnsuringDirectories(n,t,i,function(e,n,t){return d(e,n,t)},function(e){return(_.createDirectory||r.createDirectory)(e)},function(e){return l(e)}),e.performance.mark("afterIOWrite"),e.performance.measure("I/O Write","beforeIOWrite","afterIOWrite")}catch(o){a&&a(o.message)}}function d(t,a,o){if(e.isWatchSet(n)&&r.getModifiedTime){i||(i=new e.Map);var c=s(a),l=r.getModifiedTime(t);if(l){var u=i.get(t);if(u&&u.byteOrderMark===o&&u.hash===c&&u.mtime.getTime()===l.getTime())return}r.writeFile(t,a,o);var d=r.getModifiedTime(t)||e.missingFileModifiedTime;i.set(t,{hash:c,byteOrderMark:o,mtime:d})}else r.writeFile(t,a,o)}function p(){return e.getDirectoryPath(e.normalizePath(r.getExecutingFilePath()))}var f=e.getNewLineCharacter(n,function(){return r.newLine}),m=r.realpath&&function(e){return r.realpath(e)},_={getSourceFile:c,getDefaultLibLocation:p,getDefaultLibFileName:function(n){return e.combinePaths(p(),e.getDefaultLibFileName(n))},writeFile:u,getCurrentDirectory:e.memoize(function(){return r.getCurrentDirectory()}),useCaseSensitiveFileNames:function(){return r.useCaseSensitiveFileNames},getCanonicalFileName:o,getNewLine:function(){return f},fileExists:function(e){return r.fileExists(e)},readFile:function(e){return r.readFile(e)},trace:function(e){return r.write(e+f)},directoryExists:function(e){return r.directoryExists(e)},getEnvironmentVariable:function(e){return r.getEnvironmentVariable?r.getEnvironmentVariable(e):""},getDirectories:function(e){return r.getDirectories(e)},realpath:m,readDirectory:function(e,n,t,i,a){return r.readDirectory(e,n,t,i,a)},createDirectory:function(e){return r.createDirectory(e)},createHash:e.maybeBind(r,r.createHash)};return _}function s(n,t,r){var i=n.readFile,a=n.fileExists,o=n.directoryExists,s=n.createDirectory,c=n.writeFile,l=new e.Map,u=new e.Map,d=new e.Map,p=new e.Map,f=function(e){var n=t(e),r=l.get(n);return void 0!==r?!1!==r?r:void 0:m(n,e)},m=function(e,t){var r=i.call(n,t);return l.set(e,void 0!==r&&r),r};n.readFile=function(r){var a=t(r),o=l.get(a);return void 0!==o?!1!==o?o:void 0:e.fileExtensionIs(r,".json")||e.isBuildInfoFile(r)?m(a,r):i.call(n,r)};var _=r?function(n,i,a,o){var s=t(n),c=p.get(s);if(c)return c;var l=r(n,i,a,o);return l&&(e.isDeclarationFileName(n)||e.fileExtensionIs(n,".json"))&&p.set(s,l),l}:void 0;return n.fileExists=function(e){var r=t(e),i=u.get(r);if(void 0!==i)return i;var o=a.call(n,e);return u.set(r,!!o),o},c&&(n.writeFile=function(e,r,i,a,o){var s=t(e);u.delete(s);var d=l.get(s);if(void 0!==d&&d!==r)l.delete(s),p.delete(s);else if(_){var f=p.get(s);f&&f.text!==r&&p.delete(s)}c.call(n,e,r,i,a,o)}),o&&s&&(n.directoryExists=function(e){var r=t(e),i=d.get(r);if(void 0!==i)return i;var a=o.call(n,e);return d.set(r,!!a),a},n.createDirectory=function(e){var r=t(e);d.delete(r),s.call(n,e)}),{originalReadFile:i,originalFileExists:a,originalDirectoryExists:o,originalCreateDirectory:s,originalWriteFile:c,getSourceFileWithCache:_,readFileWithCache:f}}function c(n,t,r){var i;return i=e.addRange(i,n.getConfigFileParsingDiagnostics()),i=e.addRange(i,n.getOptionsDiagnostics(r)),i=e.addRange(i,n.getSyntacticDiagnostics(t,r)),i=e.addRange(i,n.getGlobalDiagnostics(r)),i=e.addRange(i,n.getSemanticDiagnostics(t,r)),e.getEmitDeclarations(n.getCompilerOptions())&&(i=e.addRange(i,n.getDeclarationDiagnostics(t,r))),e.sortAndDeduplicateDiagnostics(i||e.emptyArray)}function l(e,n){for(var t="",r=0,i=e;r<i.length;r++){var a=i[r];t+=u(a,n)}return t}function u(n,t){var r=e.diagnosticCategoryName(n)+" TS"+n.code+": "+S(n.messageText,t.getNewLine())+t.getNewLine();if(n.file){var i=e.getLineAndCharacterOfPosition(n.file,n.start),a=i.line,o=i.character,s=n.file.fileName,c=e.convertToRelativePath(s,t.getCurrentDirectory(),function(e){return t.getCanonicalFileName(e)});return c+"("+(a+1)+","+(o+1)+"): "+r}return r}var d;e.findConfigFile=n,e.resolveTripleslashReference=t,e.computeCommonSourceDirectoryOfFilenames=r,e.createCompilerHost=i,e.createCompilerHostWorker=o,e.changeCompilerHostLikeToUseCache=s,e.getPreEmitDiagnostics=c,e.formatDiagnostics=l,e.formatDiagnostic=u,function(e){e["Grey"]="[90m",e["Red"]="[91m",e["Yellow"]="[93m",e["Blue"]="[94m",e["Cyan"]="[96m"}(d=e.ForegroundColorEscapeSequences||(e.ForegroundColorEscapeSequences={}));var p="[7m",f=" ",m="[0m",_="...",g="  ",y="    ";function h(n){switch(n){case e.DiagnosticCategory.Error:return d.Red;case e.DiagnosticCategory.Warning:return d.Yellow;case e.DiagnosticCategory.Suggestion:return e.Debug.fail("Should never get an Info diagnostic on the command line.");case e.DiagnosticCategory.Message:return d.Blue}}function v(e,n){return n+e+m}function b(n,t,r,i,a,o){var s=e.getLineAndCharacterOfPosition(n,t),c=s.line,l=s.character,u=e.getLineAndCharacterOfPosition(n,t+r),d=u.line,g=u.character,y=e.getLineAndCharacterOfPosition(n,n.text.length).line,h=d-c>=4,b=(d+1+"").length;h&&(b=Math.max(_.length,b));for(var E="",T=c;T<=d;T++){E+=o.getNewLine(),h&&c+1<T&&T<d-1&&(E+=i+v(e.padLeft(_,b),p)+f+o.getNewLine(),T=d-1);var S=e.getPositionOfLineAndCharacter(n,T,0),x=T<y?e.getPositionOfLineAndCharacter(n,T+1,0):n.text.length,L=n.text.slice(S,x);if(L=e.trimStringEnd(L),L=L.replace(/\t/g," "),E+=i+v(e.padLeft(T+1+"",b),p)+f,E+=L+o.getNewLine(),E+=i+v(e.padLeft("",b),p)+f,E+=a,T===c){var D=T===d?g:void 0;E+=L.slice(0,l).replace(/\S/g," "),E+=L.slice(l,D).replace(/./g,"~")}else E+=T===d?L.slice(0,g).replace(/./g,"~"):L.replace(/./g,"~");E+=m}return E}function E(n,t,r,i){void 0===i&&(i=v);var a=e.getLineAndCharacterOfPosition(n,t),o=a.line,s=a.character,c=r?e.convertToRelativePath(n.fileName,r.getCurrentDirectory(),function(e){return r.getCanonicalFileName(e)}):n.fileName,l="";return l+=i(c,d.Cyan),l+=":",l+=i(""+(o+1),d.Yellow),l+=":",l+=i(""+(s+1),d.Yellow),l}function T(n,t){for(var r="",i=0,a=n;i<a.length;i++){var o=a[i];if(o.file){var s=o.file,c=o.start;r+=E(s,c,t),r+=" - "}if(r+=v(e.diagnosticCategoryName(o),h(o.category)),r+=v(" TS"+o.code+": ",d.Grey),r+=S(o.messageText,t.getNewLine()),o.file&&(r+=t.getNewLine(),r+=b(o.file,o.start,o.length,"",h(o.category),t)),o.relatedInformation){r+=t.getNewLine();for(var l=0,u=o.relatedInformation;l<u.length;l++){var p=u[l],f=(s=p.file,c=p.start,p.length),m=p.messageText;s&&(r+=t.getNewLine(),r+=g+E(s,c,t),r+=b(s,c,f,y,d.Cyan,t)),r+=t.getNewLine(),r+=y+S(m,t.getNewLine())}}r+=t.getNewLine()}return r}function S(n,t,r){if(void 0===r&&(r=0),e.isString(n))return n;if(void 0===n)return"";var i="";if(r){i+=t;for(var a=0;a<r;a++)i+="  "}if(i+=n.messageText,r++,n.next)for(var o=0,s=n.next;o<s.length;o++){var c=s[o];i+=S(c,t,r)}return i}function x(n,t,r,i){if(0===n.length)return[];for(var a=[],o=new e.Map,s=0,c=n;s<c.length;s++){var l=c[s],u=void 0;o.has(l)?u=o.get(l):o.set(l,u=i(l,t,r)),a.push(u)}return a}function L(e,n){return D(void 0,e,function(e,t){return e&&n(e,t)})}function D(n,t,r,i){var a;return o(n,t,void 0);function o(n,t,s){if(i){var c=i(n,s);if(c)return c}return e.forEach(t,function(n,t){if(!(n&&(null===a||void 0===a?void 0:a.has(n.sourceFile.path)))){var i=r(n,s,t);return i||!n?i:((a||(a=new e.Set)).add(n.sourceFile.path),o(n.commandLine.projectReferences,n.references,n))}})}}function A(n){switch(null===n||void 0===n?void 0:n.kind){case e.FileIncludeKind.Import:case e.FileIncludeKind.ReferenceFile:case e.FileIncludeKind.TypeReferenceDirective:case e.FileIncludeKind.LibReferenceDirective:return!0;default:return!1}}function C(e){return void 0!==e.pos}function I(n,t){var r,i,a,o,s,c,l,u,d,p,f=e.Debug.checkDefined(n(t.file)),m=t.kind,_=t.index;switch(m){case e.FileIncludeKind.Import:var g=K(f,_);if(p=null===(s=null===(o=f.resolvedModules)||void 0===o?void 0:o.get(g.text))||void 0===s?void 0:s.packageId,-1===g.pos)return{file:f,packageId:p,text:g.text};u=e.skipTrivia(f.text,g.pos),d=g.end;break;case e.FileIncludeKind.ReferenceFile:r=f.referencedFiles[_],u=r.pos,d=r.end;break;case e.FileIncludeKind.TypeReferenceDirective:i=f.typeReferenceDirectives[_],u=i.pos,d=i.end,p=null===(l=null===(c=f.resolvedTypeReferenceDirectiveNames)||void 0===c?void 0:c.get(e.toFileNameLowerCase(f.typeReferenceDirectives[_].fileName)))||void 0===l?void 0:l.packageId;break;case e.FileIncludeKind.LibReferenceDirective:a=f.libReferenceDirectives[_],u=a.pos,d=a.end;break;default:return e.Debug.assertNever(m)}return{file:f,pos:u,end:d,packageId:p}}function N(n,t,r,i,a,o,s,c,l){if(!n||(null===s||void 0===s?void 0:s()))return!1;if(!e.arrayIsEqualTo(n.getRootFileNames(),t))return!1;var u;if(!e.arrayIsEqualTo(n.getProjectReferences(),l,m))return!1;if(n.getSourceFiles().some(p))return!1;if(n.getMissingFilePaths().some(a))return!1;var d=n.getCompilerOptions();return!!e.compareDataObjects(d,r)&&(!d.configFile||!r.configFile||d.configFile.text===r.configFile.text);function p(e){return!f(e)||o(e.path)}function f(e){return e.version===i(e.resolvedPath,e.fileName)}function m(t,r,i){return e.projectReferenceIsEqualTo(t,r)&&_(n.getResolvedProjectReferences()[i],t)}function _(n,t){if(n){if(e.contains(u,n))return!0;var r=U(t),i=c(r);return!!i&&(n.commandLine.options.configFile===i.options.configFile&&(!!e.arrayIsEqualTo(n.commandLine.fileNames,i.fileNames)&&((u||(u=[])).push(n),!e.forEach(n.references,function(e,t){return!_(e,n.commandLine.projectReferences[t])}))))}var a=U(t);return!c(a)}}function k(e){return e.options.configFile?a(a([],e.options.configFile.parseDiagnostics,!0),e.errors,!0):e.errors}function R(n,t){return!!n&&e.optionsHaveChanges(n.getCompilerOptions(),t,e.sourceFileAffectingCompilerOptions)}function O(e,n,t,r,i){return{rootNames:e,options:n,host:t,oldProgram:r,configFileParsingDiagnostics:i}}function w(n,r,o,s,c){var l,u,d,p,f,m,_,g,y,h,v,b,E=e.isArray(n)?O(n,r,o,s,c):n,T=E.rootNames,S=E.options,L=E.configFileParsingDiagnostics,N=E.projectReferences,k=E.oldProgram,w=new e.Map,K=e.createMultiMap(),H={},W={},z=new e.Map,J="number"===typeof S.maxNodeModuleJsDepth?S.maxNodeModuleJsDepth:0,q=0,X=new e.Map,Y=new e.Map;null===e.tracing||void 0===e.tracing||e.tracing.push("program","createProgram",{configFilePath:S.configFilePath,rootDir:S.rootDir},!0),e.performance.mark("beforeProgram");var Q,Z,$,ee,ne,te=E.host||i(S),re=G(te),ie=S.noLib,ae=e.memoize(function(){return te.getDefaultLibFileName(S)}),oe=te.getDefaultLibLocation?te.getDefaultLibLocation():e.getDirectoryPath(ae()),se=e.createDiagnosticCollection(),ce=te.getCurrentDirectory(),le=e.getSupportedExtensions(S),ue=e.getSuppoertedExtensionsWithJsonIfResolveJsonModule(S,le),de=new e.Map,pe=te.hasInvalidatedResolution||e.returnFalse;if(te.resolveModuleNames)ee=function(n,t,r,i){return te.resolveModuleNames(e.Debug.checkEachDefined(n),t,r,i,S).map(function(n){if(!n||void 0!==n.extension)return n;var t=e.clone(n);return t.extension=e.extensionFromPath(n.resolvedFileName),t})};else{Z=e.createModuleResolutionCache(ce,bt,S);var fe=function(n,t,r){return e.resolveModuleName(n,t,S,te,Z,r).resolvedModule};ee=function(n,t,r,i){return x(e.Debug.checkEachDefined(n),t,i,fe)}}if(te.resolveTypeReferenceDirectives)ne=function(n,t,r){return te.resolveTypeReferenceDirectives(e.Debug.checkEachDefined(n),t,r,S)};else{$=e.createTypeReferenceDirectiveResolutionCache(ce,bt,void 0,null===Z||void 0===Z?void 0:Z.getPackageJsonInfoCache());var me=function(n,t,r){return e.resolveTypeReferenceDirective(n,t,S,te,r,$).resolvedTypeReferenceDirective};ne=function(n,t,r){return x(e.Debug.checkEachDefined(n),t,r,me)}}var _e,ge,ye,he,ve,be=new e.Map,Ee=new e.Map,Te=e.createMultiMap(),Se=!1,xe=new e.Map,Le=te.useCaseSensitiveFileNames()?new e.Map:void 0,De=!!(null===(l=te.useSourceOfProjectReferenceRedirect)||void 0===l?void 0:l.call(te))&&!S.disableSourceOfProjectReferenceRedirect,Ae=P({compilerHost:te,getSymlinkCache:Wt,useSourceOfProjectReferenceRedirect:De,toPath:Ze,getResolvedProjectReferences:sn,getSourceOfProjectReferenceRedirect:pt,forEachResolvedProjectReference:dt}),Ce=Ae.onProgramCreateComplete,Ie=Ae.fileExists,Ne=Ae.directoryExists;null===e.tracing||void 0===e.tracing||e.tracing.push("program","shouldProgramCreateNewSourceFiles",{hasOldProgram:!!k});var ke,Re=R(k,S);if(null===e.tracing||void 0===e.tracing||e.tracing.pop(),null===e.tracing||void 0===e.tracing||e.tracing.push("program","tryReuseStructureFromOldProgram",{}),ke=rn(),null===e.tracing||void 0===e.tracing||e.tracing.pop(),2!==ke){p=[],f=[],N&&(ge||(ge=N.map(St)),T.length&&(null===ge||void 0===ge||ge.forEach(function(n,t){if(n){var r=e.outFile(n.commandLine.options);if(De){if(r||e.getEmitModuleKind(n.commandLine.options)===e.ModuleKind.None)for(var i=0,a=n.commandLine.fileNames;i<a.length;i++){var o=a[i];et(o,{kind:e.FileIncludeKind.SourceFromProjectReference,index:t})}}else if(r)et(e.changeExtension(r,".d.ts"),{kind:e.FileIncludeKind.OutputFromProjectReference,index:t});else if(e.getEmitModuleKind(n.commandLine.options)===e.ModuleKind.None)for(var s=e.memoize(function(){return e.getCommonSourceDirectoryOfConfig(n.commandLine,!te.useCaseSensitiveFileNames())}),c=0,l=n.commandLine.fileNames;c<l.length;c++){o=l[c];e.fileExtensionIs(o,".d.ts")||e.fileExtensionIs(o,".json")||et(e.getOutputDeclarationFileName(o,n.commandLine,!te.useCaseSensitiveFileNames(),s),{kind:e.FileIncludeKind.OutputFromProjectReference,index:t})}}}))),null===e.tracing||void 0===e.tracing||e.tracing.push("program","processRootFiles",{count:T.length}),e.forEach(T,function(n,t){return Wn(n,!1,!1,{kind:e.FileIncludeKind.RootFile,index:t})}),null===e.tracing||void 0===e.tracing||e.tracing.pop();var Oe=T.length?e.getAutomaticTypeDirectiveNames(S,te):e.emptyArray;if(Oe.length){null===e.tracing||void 0===e.tracing||e.tracing.push("program","processTypeReferences",{count:Oe.length});for(var we=S.configFilePath?e.getDirectoryPath(S.configFilePath):te.getCurrentDirectory(),Pe=e.combinePaths(we,e.inferredTypesContainingFile),Me=ze(Oe,Pe),Fe=0;Fe<Oe.length;Fe++)yt(Oe[Fe],Me[Fe],{kind:e.FileIncludeKind.AutomaticTypeDirectiveFile,typeReference:Oe[Fe],packageId:null===(u=Me[Fe])||void 0===u?void 0:u.packageId});null===e.tracing||void 0===e.tracing||e.tracing.pop()}if(T.length&&!ie){var Ge=ae();!S.lib&&Ge?Wn(Ge,!0,!1,{kind:e.FileIncludeKind.LibFile}):e.forEach(S.lib,function(n,t){Wn(e.combinePaths(oe,n),!0,!1,{kind:e.FileIncludeKind.LibFile,index:t})})}_e=e.arrayFrom(e.mapDefinedIterator(xe.entries(),function(e){var n=e[0],t=e[1];return void 0===t?n:void 0})),m=e.stableSort(p,Xe).concat(f),p=void 0,f=void 0}if(e.Debug.assert(!!_e),k&&te.onReleaseOldSourceFile){for(var Be=k.getSourceFiles(),Ue=0,Ve=Be;Ue<Ve.length;Ue++){var je=Ve[Ue],Ke=vn(je.resolvedPath);(Re||!Ke||je.resolvedPath===je.path&&Ke.resolvedPath!==je.path)&&te.onReleaseOldSourceFile(je,k.getCompilerOptions(),!!vn(je.path))}te.getParsedCommandLine||k.forEachResolvedProjectReference(function(e){mt(e.sourceFile.path)||te.onReleaseOldSourceFile(e.sourceFile,k.getCompilerOptions(),!1)})}k&&te.onReleaseParsedCommandLine&&D(k.getProjectReferences(),k.getResolvedProjectReferences(),function(e,n,t){var r=(null===n||void 0===n?void 0:n.commandLine.projectReferences[t])||k.getProjectReferences()[t],i=U(r);(null===ye||void 0===ye?void 0:ye.has(Ze(i)))||te.onReleaseParsedCommandLine(i,e,k.getCompilerOptions())}),$=void 0,k=void 0;var He={getRootFileNames:function(){return T},getSourceFile:hn,getSourceFileByPath:vn,getSourceFiles:function(){return m},getMissingFilePaths:function(){return _e},getModuleResolutionCache:function(){return Z},getFilesByNameMap:function(){return xe},getCompilerOptions:function(){return S},getSyntacticDiagnostics:En,getOptionsDiagnostics:Vn,getGlobalDiagnostics:Kn,getSemanticDiagnostics:Tn,getCachedSemanticDiagnostics:Sn,getSuggestionDiagnostics:wn,getDeclarationDiagnostics:Dn,getBindAndCheckDiagnostics:xn,getProgramDiagnostics:Ln,getTypeChecker:mn,getClassifiableNames:en,getDiagnosticsProducingTypeChecker:pn,getCommonSourceDirectory:$e,emit:_n,getCurrentDirectory:function(){return ce},getNodeCount:function(){return pn().getNodeCount()},getIdentifierCount:function(){return pn().getIdentifierCount()},getSymbolCount:function(){return pn().getSymbolCount()},getTypeCount:function(){return pn().getTypeCount()},getInstantiationCount:function(){return pn().getInstantiationCount()},getRelationCacheSizes:function(){return pn().getRelationCacheSizes()},getFileProcessingDiagnostics:function(){return b},getResolvedTypeReferenceDirectives:function(){return z},isSourceFileFromExternalLibrary:un,isSourceFileDefaultLibrary:dn,dropDiagnosticsProducingTypeChecker:fn,getSourceFileFromReference:Qn,getLibFileFromReference:Yn,sourceFileToPackageName:Ee,redirectTargetsMap:Te,usesUriStyleNodeCoreModules:Se,isEmittedFile:Kt,getConfigFileParsingDiagnostics:Hn,getResolvedModuleWithFailedLookupLocationsFromCache:Qe,getProjectReferences:cn,getResolvedProjectReferences:sn,getProjectReferenceRedirect:st,getResolvedProjectReferenceToRedirect:ut,getResolvedProjectReferenceByPath:mt,forEachResolvedProjectReference:dt,isSourceOfProjectReferenceRedirect:ft,emitBuildInfo:on,fileExists:Ie,directoryExists:Ne,getSymlinkCache:Wt,realpath:null===(d=te.realpath)||void 0===d?void 0:d.bind(te),useCaseSensitiveFileNames:function(){return te.useCaseSensitiveFileNames()},getFileIncludeReasons:function(){return K},structureIsReused:ke};return Ce(),null===b||void 0===b||b.forEach(function(n){switch(n.kind){case 1:return se.add(Lt(n.file&&vn(n.file),n.fileProcessingReason,n.diagnostic,n.args||e.emptyArray));case 0:var t=I(vn,n.reason),r=t.file,i=t.pos,o=t.end;return se.add(e.createFileDiagnostic.apply(void 0,a([r,e.Debug.checkDefined(i),e.Debug.checkDefined(o)-i,n.diagnostic],n.args||e.emptyArray,!1)));default:e.Debug.assertNever(n)}}),xt(),e.performance.mark("afterProgram"),e.performance.measure("Program","beforeProgram","afterProgram"),null===e.tracing||void 0===e.tracing||e.tracing.pop(),He;function We(n,t,r){if(!n.length)return e.emptyArray;var i=e.getNormalizedAbsolutePath(t.originalFileName,ce),a=Je(t);null===e.tracing||void 0===e.tracing||e.tracing.push("program","resolveModuleNamesWorker",{containingFileName:i}),e.performance.mark("beforeResolveModule");var o=ee(n,i,r,a);return e.performance.mark("afterResolveModule"),e.performance.measure("ResolveModule","beforeResolveModule","afterResolveModule"),null===e.tracing||void 0===e.tracing||e.tracing.pop(),o}function ze(n,t){if(!n.length)return[];var r=e.isString(t)?t:e.getNormalizedAbsolutePath(t.originalFileName,ce),i=e.isString(t)?void 0:Je(t);null===e.tracing||void 0===e.tracing||e.tracing.push("program","resolveTypeReferenceDirectiveNamesWorker",{containingFileName:r}),e.performance.mark("beforeResolveTypeReference");var a=ne(n,r,i);return e.performance.mark("afterResolveTypeReference"),e.performance.measure("ResolveTypeReference","beforeResolveTypeReference","afterResolveTypeReference"),null===e.tracing||void 0===e.tracing||e.tracing.pop(),a}function Je(n){var t=ut(n.originalFileName);if(t||!e.fileExtensionIs(n.originalFileName,".d.ts"))return t;var r=qe(n.originalFileName,n.path);if(r)return r;if(te.realpath&&S.preserveSymlinks&&e.stringContains(n.originalFileName,e.nodeModulesPathPart)){var i=te.realpath(n.originalFileName),a=Ze(i);return a===n.path?void 0:qe(i,a)}}function qe(n,t){var r=pt(n);return e.isString(r)?ut(r):r?dt(function(n){var r=e.outFile(n.commandLine.options);if(r)return Ze(r)===t?n:void 0}):void 0}function Xe(n,t){return e.compareValues(Ye(n),Ye(t))}function Ye(n){if(e.containsPath(oe,n.fileName,!1)){var t=e.getBaseFileName(n.fileName);if("lib.d.ts"===t||"lib.es6.d.ts"===t)return 0;var r=e.removeSuffix(e.removePrefix(t,"lib."),".d.ts"),i=e.libs.indexOf(r);if(-1!==i)return i+1}return e.libs.length+2}function Qe(n,t){return Z&&e.resolveModuleNameFromCache(n,t,Z)}function Ze(n){return e.toPath(n,ce,bt)}function $e(){if(void 0===g){var n=e.filter(m,function(n){return e.sourceFileMayBeEmitted(n,He)});g=e.getCommonSourceDirectory(S,function(){return e.mapDefined(n,function(e){return e.isDeclarationFile?void 0:e.fileName})},ce,bt,function(e){return Tt(n,e)})}return g}function en(){var n;if(!v){mn(),v=new e.Set;for(var t=0,r=m;t<r.length;t++){var i=r[t];null===(n=i.classifiableNames)||void 0===n||n.forEach(function(e){return v.add(e)})}}return v}function nn(n,t){if(0===ke&&!t.ambientModuleNames.length)return We(n,t,void 0);var r,i,a,o=k&&k.getSourceFile(t.fileName);if(o!==t&&t.resolvedModules){for(var s=[],c=0,l=n;c<l.length;c++){var u=l[c],d=t.resolvedModules.get(u);s.push(d)}return s}for(var p={},f=0;f<n.length;f++){u=n[f];if(t===o&&!pe(o.path)){var m=e.getResolvedModule(o,u);if(m){e.isTraceEnabled(S,te)&&e.trace(te,m.packageId?e.Diagnostics.Reusing_resolution_of_module_0_from_1_of_old_program_it_was_successfully_resolved_to_2_with_Package_ID_3:e.Diagnostics.Reusing_resolution_of_module_0_from_1_of_old_program_it_was_successfully_resolved_to_2,u,e.getNormalizedAbsolutePath(t.originalFileName,ce),m.resolvedFileName,m.packageId&&e.packageIdToString(m.packageId)),(i||(i=new Array(n.length)))[f]=m,(a||(a=[])).push(u);continue}}var _=!1;e.contains(t.ambientModuleNames,u)?(_=!0,e.isTraceEnabled(S,te)&&e.trace(te,e.Diagnostics.Module_0_was_resolved_as_locally_declared_ambient_module_in_file_1,u,e.getNormalizedAbsolutePath(t.originalFileName,ce))):_=h(u),_?(i||(i=new Array(n.length)))[f]=p:(r||(r=[])).push(u)}var g=r&&r.length?We(r,t,a):e.emptyArray;if(!i)return e.Debug.assert(g.length===n.length),g;var y=0;for(f=0;f<i.length;f++)i[f]?i[f]===p&&(i[f]=void 0):(i[f]=g[y],y++);return e.Debug.assert(y===g.length),i;function h(n){var t=e.getResolvedModule(o,n),r=t&&k.getSourceFile(t.resolvedFileName);if(t&&r)return!1;var i=w.get(n);return!!i&&(e.isTraceEnabled(S,te)&&e.trace(te,e.Diagnostics.Module_0_was_resolved_as_ambient_module_declared_in_1_since_this_file_was_not_modified,n,i),!0)}}function tn(){return!D(k.getProjectReferences(),k.getResolvedProjectReferences(),function(n,t,r){var i=(t?t.commandLine.projectReferences:N)[r],a=St(i);return n?!a||a.sourceFile!==n.sourceFile||!e.arrayIsEqualTo(n.commandLine.fileNames,a.commandLine.fileNames):void 0!==a},function(n,t){var r=t?mt(t.sourceFile.path).commandLine.projectReferences:N;return!e.arrayIsEqualTo(n,r,e.projectReferenceIsEqualTo)})}function rn(){var n;if(!k)return 0;var t=k.getCompilerOptions();if(e.changesAffectModuleResolution(t,S))return 0;var r=k.getRootFileNames();if(!e.arrayIsEqualTo(r,T))return 0;if(!tn())return 0;N&&(ge=N.map(St));var i=[],a=[];if(ke=2,k.getMissingFilePaths().some(function(e){return te.fileExists(e)}))return 0;var o,s=k.getSourceFiles();(function(e){e[e["Exists"]=0]="Exists",e[e["Modified"]=1]="Modified"})(o||(o={}));for(var c=new e.Map,l=0,u=s;l<u.length;l++){var d=u[l],p=te.getSourceFileByPath?te.getSourceFileByPath(d.fileName,d.resolvedPath,S.target,void 0,Re):te.getSourceFile(d.fileName,S.target,void 0,Re);if(!p)return 0;e.Debug.assert(!p.redirectInfo,"Host should not return a redirect source file from `getSourceFile`");var f=void 0;if(d.redirectInfo){if(p!==d.redirectInfo.unredirected)return 0;f=!1,p=d}else if(k.redirectTargetsMap.has(d.path)){if(p!==d)return 0;f=!1}else f=p!==d;p.path=d.path,p.originalFileName=d.originalFileName,p.resolvedPath=d.resolvedPath,p.fileName=d.fileName;var _=k.sourceFileToPackageName.get(d.path);if(void 0!==_){var g=c.get(_),y=f?1:0;if(void 0!==g&&1===y||1===g)return 0;c.set(_,y)}f?(e.arrayIsEqualTo(d.libReferenceDirectives,p.libReferenceDirectives,zn)||(ke=1),d.hasNoDefaultLib!==p.hasNoDefaultLib&&(ke=1),e.arrayIsEqualTo(d.referencedFiles,p.referencedFiles,zn)||(ke=1),Xn(p),e.arrayIsEqualTo(d.imports,p.imports,Jn)||(ke=1),e.arrayIsEqualTo(d.moduleAugmentations,p.moduleAugmentations,Jn)||(ke=1),(3145728&d.flags)!==(3145728&p.flags)&&(ke=1),e.arrayIsEqualTo(d.typeReferenceDirectives,p.typeReferenceDirectives,zn)||(ke=1),a.push({oldFile:d,newFile:p})):pe(d.path)&&(ke=1,a.push({oldFile:d,newFile:p})),i.push(p)}if(2!==ke)return ke;for(var h=a.map(function(e){return e.oldFile}),v=0,E=s;v<E.length;v++){var x=E[v];if(!e.contains(h,x))for(var L=0,D=x.ambientModuleNames;L<D.length;L++){var A=D[L];w.set(A,x.fileName)}}for(var C=0,I=a;C<I.length;C++){var R=I[C],O=(d=R.oldFile,p=R.newFile,j(p)),P=nn(O,p),M=e.hasChangesInResolutions(O,P,d.resolvedModules,e.moduleResolutionIsEqualTo);M?(ke=1,p.resolvedModules=e.zipToMap(O,P)):p.resolvedModules=d.resolvedModules;var F=e.map(p.typeReferenceDirectives,function(n){return e.toFileNameLowerCase(n.fileName)}),G=ze(F,p),B=e.hasChangesInResolutions(F,G,d.resolvedTypeReferenceDirectiveNames,e.typeDirectiveIsEqualTo);B?(ke=1,p.resolvedTypeReferenceDirectiveNames=e.zipToMap(F,G)):p.resolvedTypeReferenceDirectiveNames=d.resolvedTypeReferenceDirectiveNames}if(2!==ke)return ke;if(e.changesAffectingProgramStructure(t,S)||(null===(n=te.hasChangedAutomaticTypeDirectiveNames)||void 0===n?void 0:n.call(te)))return 1;_e=k.getMissingFilePaths(),e.Debug.assert(i.length===k.getSourceFiles().length);for(var U=0,V=i;U<V.length;U++){p=V[U];xe.set(p.path,p)}var H=k.getFilesByNameMap();return H.forEach(function(e,n){e?e.path!==n?xe.set(n,xe.get(e.path)):k.isSourceFileFromExternalLibrary(e)&&Y.set(e.path,!0):xe.set(n,e)}),m=i,K=k.getFileIncludeReasons(),b=k.getFileProcessingDiagnostics(),z=k.getResolvedTypeReferenceDirectives(),Ee=k.sourceFileToPackageName,Te=k.redirectTargetsMap,Se=k.usesUriStyleNodeCoreModules,2}function an(n){return{getPrependNodes:ln,getCanonicalFileName:bt,getCommonSourceDirectory:He.getCommonSourceDirectory,getCompilerOptions:He.getCompilerOptions,getCurrentDirectory:function(){return ce},getNewLine:function(){return te.getNewLine()},getSourceFile:He.getSourceFile,getSourceFileByPath:He.getSourceFileByPath,getSourceFiles:He.getSourceFiles,getLibFileFromReference:He.getLibFileFromReference,isSourceFileFromExternalLibrary:un,getResolvedProjectReferenceToRedirect:ut,getProjectReferenceRedirect:st,isSourceOfProjectReferenceRedirect:ft,getSymlinkCache:Wt,writeFile:n||function(e,n,t,r,i){return te.writeFile(e,n,t,r,i)},isEmitBlocked:gn,readFile:function(e){return te.readFile(e)},fileExists:function(n){var t=Ze(n);return!!vn(t)||!e.contains(_e,t)&&te.fileExists(n)},useCaseSensitiveFileNames:function(){return te.useCaseSensitiveFileNames()},getProgramBuildInfo:function(){return He.getProgramBuildInfo&&He.getProgramBuildInfo()},getSourceFileFromReference:function(e,n){return He.getSourceFileFromReference(e,n)},redirectTargetsMap:Te,getFileIncludeReasons:He.getFileIncludeReasons}}function on(n){e.Debug.assert(!e.outFile(S)),null===e.tracing||void 0===e.tracing||e.tracing.push("emit","emitBuildInfo",{},!0),e.performance.mark("beforeEmit");var t=e.emitFiles(e.notImplementedResolver,an(n),void 0,e.noTransformers,!1,!0);return e.performance.mark("afterEmit"),e.performance.measure("Emit","beforeEmit","afterEmit"),null===e.tracing||void 0===e.tracing||e.tracing.pop(),t}function sn(){return ge}function cn(){return N}function ln(){return B(N,function(e,n){var t;return null===(t=ge[n])||void 0===t?void 0:t.commandLine},function(e){var n=Ze(e),t=vn(n);return t?t.text:xe.has(n)?void 0:te.readFile(n)})}function un(e){return!!Y.get(e.path)}function dn(n){if(n.hasNoDefaultLib)return!0;if(!S.noLib)return!1;var t=te.useCaseSensitiveFileNames()?e.equateStringsCaseSensitive:e.equateStringsCaseInsensitive;return S.lib?e.some(S.lib,function(r){return t(n.fileName,e.combinePaths(oe,r))}):t(n.fileName,ae())}function pn(){return y||(y=e.createTypeChecker(He,!0))}function fn(){y=void 0}function mn(){return h||(h=e.createTypeChecker(He,!1))}function _n(n,t,r,i,a,o){null===e.tracing||void 0===e.tracing||e.tracing.push("emit","emit",{path:null===n||void 0===n?void 0:n.path},!0);var s=Cn(function(){return yn(He,n,t,r,i,a,o)});return null===e.tracing||void 0===e.tracing||e.tracing.pop(),s}function gn(e){return de.has(Ze(e))}function yn(n,t,r,i,a,o,s){if(!s){var c=M(n,t,r,i);if(c)return c}var l=pn().getEmitResolver(e.outFile(S)?void 0:t,i);e.performance.mark("beforeEmit");var u=e.emitFiles(l,an(r),t,e.getTransformers(S,o,a),a,!1,s);return e.performance.mark("afterEmit"),e.performance.measure("Emit","beforeEmit","afterEmit"),u}function hn(e){return vn(Ze(e))}function vn(e){return xe.get(e)||void 0}function bn(n,t,r){return n?t(n,r):e.sortAndDeduplicateDiagnostics(e.flatMap(He.getSourceFiles(),function(e){return r&&r.throwIfCancellationRequested(),t(e,r)}))}function En(e,n){return bn(e,An,n)}function Tn(e,n){return bn(e,In,n)}function Sn(e){var n;return e?null===(n=H.perFile)||void 0===n?void 0:n.get(e.path):H.allDiagnostics}function xn(e,n){return Nn(e,n)}function Ln(n){var t;if(e.skipTypeChecking(n,S,He))return e.emptyArray;var r=se.getDiagnostics(n.fileName);return(null===(t=n.commentDirectives)||void 0===t?void 0:t.length)?On(n,n.commentDirectives,r).diagnostics:r}function Dn(n,t){var r=He.getCompilerOptions();return!n||e.outFile(r)?Fn(n,t):bn(n,Un,t)}function An(n){return e.isSourceFileJS(n)?(n.additionalSyntacticDiagnostics||(n.additionalSyntacticDiagnostics=Mn(n)),e.concatenate(n.additionalSyntacticDiagnostics,n.parseDiagnostics)):n.parseDiagnostics}function Cn(n){try{return n()}catch(t){throw t instanceof e.OperationCanceledException&&(h=void 0,y=void 0),t}}function In(n,t){return e.concatenate(F(Nn(n,t),S),Ln(n))}function Nn(e,n){return Bn(e,n,H,kn)}function kn(n,t){return Cn(function(){if(e.skipTypeChecking(n,S,He))return e.emptyArray;var r=pn();e.Debug.assert(!!n.bindDiagnostics);var i=e.isCheckJsEnabledForFile(n,S),a=!!n.checkJsDirective&&!1===n.checkJsDirective.enabled,o=!a&&(3===n.scriptKind||4===n.scriptKind||5===n.scriptKind||i||7===n.scriptKind),s=o?n.bindDiagnostics:e.emptyArray,c=o?r.getDiagnostics(n,t):e.emptyArray;return Rn(n,o,s,c,i?n.jsDocDiagnostics:void 0)})}function Rn(n,t){for(var r,i=[],a=2;a<arguments.length;a++)i[a-2]=arguments[a];var o=e.flatten(i);if(!t||!(null===(r=n.commentDirectives)||void 0===r?void 0:r.length))return o;for(var s=On(n,n.commentDirectives,o),c=s.diagnostics,l=s.directives,u=0,d=l.getUnusedExpectations();u<d.length;u++){var p=d[u];c.push(e.createDiagnosticForRange(n,p.range,e.Diagnostics.Unused_ts_expect_error_directive))}return c}function On(n,t,r){var i=e.createCommentDirectivesMap(n,t),a=r.filter(function(e){return-1===Pn(e,i)});return{diagnostics:a,directives:i}}function wn(e,n){return Cn(function(){return pn().getSuggestionDiagnostics(e,n)})}function Pn(n,t){var r=n.file,i=n.start;if(!r)return-1;var a=e.getLineStarts(r),o=e.computeLineAndCharacterOfPosition(a,i).line-1;while(o>=0){if(t.markUsed(o))return o;var s=r.text.slice(a[o],a[o+1]).trim();if(""!==s&&!/^(\s*)\/\/(.*)$/.test(s))return-1;o--}return-1}function Mn(n){return Cn(function(){var t=[];return r(n,n),e.forEachChildRecursively(n,r,i),t;function r(n,r){switch(r.kind){case 162:case 165:case 167:if(r.questionToken===n)return t.push(s(n,e.Diagnostics.The_0_modifier_can_only_be_used_in_TypeScript_files,"?")),"skip";case 166:case 169:case 170:case 171:case 211:case 254:case 212:case 252:if(r.type===n)return t.push(s(n,e.Diagnostics.Type_annotations_can_only_be_used_in_TypeScript_files)),"skip"}switch(n.kind){case 265:if(n.isTypeOnly)return t.push(s(r,e.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files,"import type")),"skip";break;case 270:if(n.isTypeOnly)return t.push(s(n,e.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files,"export type")),"skip";break;case 263:return t.push(s(n,e.Diagnostics.import_can_only_be_used_in_TypeScript_files)),"skip";case 269:if(n.isExportEquals)return t.push(s(n,e.Diagnostics.export_can_only_be_used_in_TypeScript_files)),"skip";break;case 289:var i=n;if(117===i.token)return t.push(s(n,e.Diagnostics.implements_clauses_can_only_be_used_in_TypeScript_files)),"skip";break;case 256:var a=e.tokenToString(118);return e.Debug.assertIsDefined(a),t.push(s(n,e.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files,a)),"skip";case 259:var o=16&n.flags?e.tokenToString(141):e.tokenToString(140);return e.Debug.assertIsDefined(o),t.push(s(n,e.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files,o)),"skip";case 257:return t.push(s(n,e.Diagnostics.Type_aliases_can_only_be_used_in_TypeScript_files)),"skip";case 258:var c=e.Debug.checkDefined(e.tokenToString(92));return t.push(s(n,e.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files,c)),"skip";case 228:return t.push(s(n,e.Diagnostics.Non_null_assertions_can_only_be_used_in_TypeScript_files)),"skip";case 227:return t.push(s(n.type,e.Diagnostics.Type_assertion_expressions_can_only_be_used_in_TypeScript_files)),"skip";case 209:e.Debug.fail()}}function i(n,r){switch(r.decorators!==n||S.experimentalDecorators||t.push(s(r,e.Diagnostics.Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_in_your_tsconfig_or_jsconfig_to_remove_this_warning)),r.kind){case 255:case 224:case 167:case 169:case 170:case 171:case 211:case 254:case 212:if(n===r.typeParameters)return t.push(o(n,e.Diagnostics.Type_parameter_declarations_can_only_be_used_in_TypeScript_files)),"skip";case 235:if(n===r.modifiers)return a(r.modifiers,235===r.kind),"skip";break;case 165:if(n===r.modifiers){for(var i=0,c=n;i<c.length;i++){var l=c[i];124!==l.kind&&t.push(s(l,e.Diagnostics.The_0_modifier_can_only_be_used_in_TypeScript_files,e.tokenToString(l.kind)))}return"skip"}break;case 162:if(n===r.modifiers)return t.push(o(n,e.Diagnostics.Parameter_modifiers_can_only_be_used_in_TypeScript_files)),"skip";break;case 206:case 207:case 226:case 277:case 278:case 208:if(n===r.typeArguments)return t.push(o(n,e.Diagnostics.Type_arguments_can_only_be_used_in_TypeScript_files)),"skip";break}}function a(n,r){for(var i=0,a=n;i<a.length;i++){var o=a[i];switch(o.kind){case 85:if(r)continue;case 123:case 121:case 122:case 143:case 134:case 126:case 157:t.push(s(o,e.Diagnostics.The_0_modifier_can_only_be_used_in_TypeScript_files,e.tokenToString(o.kind)));break;case 124:case 93:case 88:}}}function o(t,r,i,a,o){var s=t.pos;return e.createFileDiagnostic(n,s,t.end-s,r,i,a,o)}function s(t,r,i,a,o){return e.createDiagnosticForNodeInSourceFile(n,t,r,i,a,o)}})}function Fn(e,n){return Bn(e,n,W,Gn)}function Gn(n,t){return Cn(function(){var r=pn().getEmitResolver(n,t);return e.getDeclarationDiagnostics(an(e.noop),r,n)||e.emptyArray})}function Bn(n,t,r,i){var a,o=n?null===(a=r.perFile)||void 0===a?void 0:a.get(n.path):r.allDiagnostics;if(o)return o;var s=i(n,t);return n?(r.perFile||(r.perFile=new e.Map)).set(n.path,s):r.allDiagnostics=s,s}function Un(e,n){return e.isDeclarationFile?[]:Fn(e,n)}function Vn(){return e.sortAndDeduplicateDiagnostics(e.concatenate(se.getGlobalDiagnostics(),jn()))}function jn(){if(!S.configFile)return e.emptyArray;var n=se.getDiagnostics(S.configFile.fileName);return dt(function(t){n=e.concatenate(n,se.getDiagnostics(t.sourceFile.fileName))}),n}function Kn(){return T.length?e.sortAndDeduplicateDiagnostics(pn().getGlobalDiagnostics().slice()):e.emptyArray}function Hn(){return L||e.emptyArray}function Wn(n,t,r,i){$n(e.normalizePath(n),t,r,void 0,i)}function zn(e,n){return e.fileName===n.fileName}function Jn(e,n){return 79===e.kind?79===n.kind&&e.escapedText===n.escapedText:10===n.kind&&e.text===n.text}function qn(n,t){var r=e.factory.createStringLiteral(n),i=e.factory.createImportDeclaration(void 0,void 0,void 0,r);return e.addEmitFlags(i,67108864),e.setParent(r,i),e.setParent(i,t),r.flags&=-9,i.flags&=-9,r}function Xn(n){if(!n.imports){var t,r,i,a=e.isSourceFileJS(n),o=e.isExternalModule(n);if((S.isolatedModules||o)&&!n.isDeclarationFile){S.importHelpers&&(t=[qn(e.externalHelpersModuleNameText,n)]);var s=e.getJSXRuntimeImport(e.getJSXImplicitImportBase(S,n),S);s&&(t||(t=[])).push(qn(s,n))}for(var c=0,l=n.statements;c<l.length;c++){var u=l[c];d(u,!1)}return(1048576&n.flags||a)&&p(n),n.imports=t||e.emptyArray,n.moduleAugmentations=r||e.emptyArray,void(n.ambientModuleNames=i||e.emptyArray)}function d(a,s){if(e.isAnyImportOrReExport(a)){var c=e.getExternalModuleName(a);!(c&&e.isStringLiteral(c)&&c.text)||s&&e.isExternalModuleNameRelative(c.text)||(t=e.append(t,c),Se||0!==q||n.isDeclarationFile||(Se=e.startsWith(c.text,"node:")))}else if(e.isModuleDeclaration(a)&&e.isAmbientModule(a)&&(s||e.hasSyntacticModifier(a,2)||n.isDeclarationFile)){var l=e.getTextOfIdentifierOrLiteral(a.name);if(o||s&&!e.isExternalModuleNameRelative(l))(r||(r=[])).push(a.name);else if(!s){n.isDeclarationFile&&(i||(i=[])).push(l);var u=a.body;if(u)for(var p=0,f=u.statements;p<f.length;p++){var m=f[p];d(m,!0)}}}}function p(n){var r=/import|require/g;while(null!==r.exec(n.text)){var i=f(n,r.lastIndex);a&&e.isRequireCall(i,!0)?t=e.append(t,i.arguments[0]):e.isImportCall(i)&&1===i.arguments.length&&e.isStringLiteralLike(i.arguments[0])?t=e.append(t,i.arguments[0]):e.isLiteralImportTypeNode(i)&&(t=e.append(t,i.argument.literal))}}function f(n,t){var r=n,i=function(e){if(e.pos<=t&&(t<e.end||t===e.end&&1===e.kind))return e};while(1){var o=a&&e.hasJSDocNodes(r)&&e.forEach(r.jsDoc,i)||e.forEachChild(r,i);if(!o)return r;r=o}}}function Yn(n){var t=e.toFileNameLowerCase(n.fileName),r=e.libMap.get(t);if(r)return hn(e.combinePaths(oe,r))}function Qn(e,n){return Zn(t(n.fileName,e.fileName),hn)}function Zn(n,t,r,i){if(e.hasExtension(n)){var a=te.getCanonicalFileName(n);if(!S.allowNonTsExtensions&&!e.forEach(ue,function(n){return e.fileExtensionIs(a,n)}))return void(r&&(e.hasJSFileExtension(a)?r(e.Diagnostics.File_0_is_a_JavaScript_file_Did_you_mean_to_enable_the_allowJs_option,n):r(e.Diagnostics.File_0_has_an_unsupported_extension_The_only_supported_extensions_are_1,n,"'"+le.join("', '")+"'")));var o=t(n);if(r)if(o)A(i)&&a===te.getCanonicalFileName(vn(i.file).fileName)&&r(e.Diagnostics.A_file_cannot_have_a_reference_to_itself);else{var s=st(n);s?r(e.Diagnostics.Output_file_0_has_not_been_built_from_source_file_1,s,n):r(e.Diagnostics.File_0_not_found,n)}return o}var c=S.allowNonTsExtensions&&t(n);if(c)return c;if(!r||!S.allowNonTsExtensions){var l=e.forEach(le,function(e){return t(n+e)});return r&&!l&&r(e.Diagnostics.Could_not_resolve_the_path_0_with_the_extensions_Colon_1,n,"'"+le.join("', '")+"'"),l}r(e.Diagnostics.File_0_not_found,n)}function $n(e,n,t,r,i){Zn(e,function(e){return rt(e,Ze(e),n,t,i,r)},function(e){for(var n=[],t=1;t<arguments.length;t++)n[t-1]=arguments[t];return Dt(void 0,i,e,n)},i)}function et(e,n){return $n(e,!1,!1,void 0,n)}function nt(n,t,r){var i=!A(r)&&e.some(K.get(t.path),A);i?Dt(t,r,e.Diagnostics.Already_included_file_name_0_differs_from_file_name_1_only_in_casing,[t.fileName,n]):Dt(t,r,e.Diagnostics.File_name_0_differs_from_already_included_file_name_1_only_in_casing,[n,t.fileName])}function tt(e,n,t,r,i,a){var o=Object.create(e);return o.fileName=t,o.path=r,o.resolvedPath=i,o.originalFileName=a,o.redirectInfo={redirectTarget:e,unredirected:n},Y.set(r,q>0),Object.defineProperties(o,{id:{get:function(){return this.redirectInfo.redirectTarget.id},set:function(e){this.redirectInfo.redirectTarget.id=e}},symbol:{get:function(){return this.redirectInfo.redirectTarget.symbol},set:function(e){this.redirectInfo.redirectTarget.symbol=e}}}),o}function rt(n,t,r,i,a,o){null===e.tracing||void 0===e.tracing||e.tracing.push("program","findSourceFile",{fileName:n,isDefaultLib:r||void 0,fileIncludeKind:e.FileIncludeKind[a.kind]});var s=it(n,t,r,i,a,o);return null===e.tracing||void 0===e.tracing||e.tracing.pop(),s}function it(n,t,r,i,a,o){if(De){var s=pt(n);if(!s&&te.realpath&&S.preserveSymlinks&&e.isDeclarationFileName(n)&&e.stringContains(n,e.nodeModulesPathPart)){var c=te.realpath(n);c!==n&&(s=pt(c))}if(s){var l=e.isString(s)?rt(s,Ze(s),r,i,a,o):void 0;return l&&ot(l,t,void 0),l}}var u,d=n;if(xe.has(t)){var m=xe.get(t);if(at(m||void 0,a),m&&S.forceConsistentCasingInFileNames){var _=m.fileName,g=Ze(_)!==Ze(n);g&&(n=st(n)||n);var y=e.getNormalizedAbsolutePathWithoutRoot(_,ce),h=e.getNormalizedAbsolutePathWithoutRoot(n,ce);y!==h&&nt(n,m,a)}return m&&Y.get(m.path)&&0===q?(Y.set(m.path,!1),S.noResolve||(_t(m,r),gt(m)),S.noLib||vt(m),X.set(m.path,!1),Et(m)):m&&X.get(m.path)&&q<J&&(X.set(m.path,!1),Et(m)),m||void 0}if(A(a)&&!De){var v=ct(n);if(v){if(e.outFile(v.commandLine.options))return;var b=lt(v,n);n=b,u=Ze(b)}}var E=te.getSourceFile(n,S.target,function(t){return Dt(void 0,a,e.Diagnostics.Cannot_read_file_0_Colon_1,[n,t])},Re);if(o){var T=e.packageIdToString(o),x=be.get(T);if(x){var L=tt(x,E,n,t,Ze(n),d);return Te.add(x.path,n),ot(L,t,u),at(L,a),Ee.set(t,o.name),f.push(L),L}E&&(be.set(T,E),Ee.set(t,o.name))}if(ot(E,t,u),E){if(Y.set(t,q>0),E.fileName=n,E.path=t,E.resolvedPath=Ze(n),E.originalFileName=d,at(E,a),te.useCaseSensitiveFileNames()){var D=e.toFileNameLowerCase(t),C=Le.get(D);C?nt(n,C,a):Le.set(D,E)}ie=ie||E.hasNoDefaultLib&&!i,S.noResolve||(_t(E,r),gt(E)),S.noLib||vt(E),Et(E),r?p.push(E):f.push(E)}return E}function at(e,n){e&&K.add(e.path,n)}function ot(e,n,t){t?(xe.set(t,e),xe.set(n,e||!1)):xe.set(n,e)}function st(e){var n=ct(e);return n&&lt(n,e)}function ct(n){if(ge&&ge.length&&!e.fileExtensionIs(n,".d.ts")&&!e.fileExtensionIs(n,".json"))return ut(n)}function lt(n,t){var r=e.outFile(n.commandLine.options);return r?e.changeExtension(r,".d.ts"):e.getOutputDeclarationFileName(t,n.commandLine,!te.useCaseSensitiveFileNames())}function ut(n){void 0===he&&(he=new e.Map,dt(function(e){Ze(S.configFilePath)!==e.sourceFile.path&&e.commandLine.fileNames.forEach(function(n){return he.set(Ze(n),e.sourceFile.path)})}));var t=he.get(Ze(n));return t&&mt(t)}function dt(n){return e.forEachResolvedProjectReference(ge,n)}function pt(n){if(e.isDeclarationFileName(n))return void 0===ve&&(ve=new e.Map,dt(function(n){var t=e.outFile(n.commandLine.options);if(t){var r=e.changeExtension(t,".d.ts");ve.set(Ze(r),!0)}else{var i=e.memoize(function(){return e.getCommonSourceDirectoryOfConfig(n.commandLine,!te.useCaseSensitiveFileNames())});e.forEach(n.commandLine.fileNames,function(t){if(!e.fileExtensionIs(t,".d.ts")&&!e.fileExtensionIs(t,".json")){var r=e.getOutputDeclarationFileName(t,n.commandLine,!te.useCaseSensitiveFileNames(),i);ve.set(Ze(r),t)}})}})),ve.get(Ze(n))}function ft(e){return De&&!!ut(e)}function mt(e){if(ye)return ye.get(e)||void 0}function _t(n,r){e.forEach(n.referencedFiles,function(i,a){$n(t(i.fileName,n.fileName),r,!1,void 0,{kind:e.FileIncludeKind.ReferenceFile,file:n.path,index:a})})}function gt(n){var t=e.map(n.typeReferenceDirectives,function(n){return e.toFileNameLowerCase(n.fileName)});if(t)for(var r=ze(t,n),i=0;i<t.length;i++){var a=n.typeReferenceDirectives[i],o=r[i],s=e.toFileNameLowerCase(a.fileName);e.setResolvedTypeReferenceDirective(n,s,o),yt(s,o,{kind:e.FileIncludeKind.TypeReferenceDirective,file:n.path,index:i})}}function yt(n,t,r){null===e.tracing||void 0===e.tracing||e.tracing.push("program","processTypeReferenceDirective",{directive:n,hasResolved:!!nn,refKind:r.kind,refPath:A(r)?r.file:void 0}),ht(n,t,r),null===e.tracing||void 0===e.tracing||e.tracing.pop()}function ht(n,t,r){var i=z.get(n);if(!i||!i.primary){var a=!0;if(t){if(t.isExternalLibraryImport&&q++,t.primary)$n(t.resolvedFileName,!1,!1,t.packageId,r);else if(i){if(t.resolvedFileName!==i.resolvedFileName){var o=te.readFile(t.resolvedFileName),s=hn(i.resolvedFileName);o!==s.text&&Dt(s,r,e.Diagnostics.Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_library_to_resolve_the_conflict,[n,t.resolvedFileName,i.resolvedFileName])}a=!1}else $n(t.resolvedFileName,!1,!1,t.packageId,r);t.isExternalLibraryImport&&q--}else Dt(void 0,r,e.Diagnostics.Cannot_find_type_definition_file_for_0,[n]);a&&z.set(n,t)}}function vt(n){e.forEach(n.libReferenceDirectives,function(t,r){var i=e.toFileNameLowerCase(t.fileName),a=e.libMap.get(i);if(a)Wn(e.combinePaths(oe,a),!0,!0,{kind:e.FileIncludeKind.LibReferenceDirective,file:n.path,index:r});else{var o=e.removeSuffix(e.removePrefix(i,"lib."),".d.ts"),s=e.getSpellingSuggestion(o,e.libs,e.identity),c=s?e.Diagnostics.Cannot_find_lib_definition_for_0_Did_you_mean_1:e.Diagnostics.Cannot_find_lib_definition_for_0;(b||(b=[])).push({kind:0,reason:{kind:e.FileIncludeKind.LibReferenceDirective,file:n.path,index:r},diagnostic:c,args:[i,s]})}})}function bt(e){return te.getCanonicalFileName(e)}function Et(n){var t;if(Xn(n),n.imports.length||n.moduleAugmentations.length){var r=j(n),i=nn(r,n);e.Debug.assert(i.length===r.length);for(var a=(De?null===(t=Je(n))||void 0===t?void 0:t.commandLine.options:void 0)||S,o=0;o<r.length;o++){var s=i[o];if(e.setResolvedModule(n,r[o],s),s){var c=s.isExternalLibraryImport,l=!e.resolutionExtensionIsTSOrJson(s.extension),u=c&&l,d=s.resolvedFileName;c&&q++;var p=u&&q>J,f=d&&!V(a,s)&&!a.noResolve&&o<n.imports.length&&!p&&!(l&&!e.getAllowJSCompilerOption(a))&&(e.isInJSFile(n.imports[o])||!(4194304&n.imports[o].flags));if(p)X.set(n.path,!0);else if(f){var m=Ze(d);rt(d,m,!1,!1,{kind:e.FileIncludeKind.Import,file:n.path,index:o},s.packageId)}c&&q--}}}else n.resolvedModules=void 0}function Tt(n,t){for(var r=!0,i=te.getCanonicalFileName(e.getNormalizedAbsolutePath(t,ce)),a=0,o=n;a<o.length;a++){var s=o[a];if(!s.isDeclarationFile){var c=te.getCanonicalFileName(e.getNormalizedAbsolutePath(s.fileName,ce));0!==c.indexOf(i)&&(At(s,e.Diagnostics.File_0_is_not_under_rootDir_1_rootDir_is_expected_to_contain_all_source_files,[s.fileName,t]),r=!1)}}return r}function St(n){ye||(ye=new e.Map);var t,r,i=U(n),a=Ze(i),o=ye.get(a);if(void 0!==o)return o||void 0;if(te.getParsedCommandLine){if(t=te.getParsedCommandLine(i),!t)return ot(void 0,a,void 0),void ye.set(a,!1);r=e.Debug.checkDefined(t.options.configFile),e.Debug.assert(!r.path||r.path===a),ot(r,a,void 0)}else{var s=e.getNormalizedAbsolutePath(e.getDirectoryPath(i),te.getCurrentDirectory());if(r=te.getSourceFile(i,100),ot(r,a,void 0),void 0===r)return void ye.set(a,!1);t=e.parseJsonSourceFileConfigFileContent(r,re,s,void 0,i)}r.fileName=i,r.path=a,r.resolvedPath=a,r.originalFileName=i;var c={commandLine:t,sourceFile:r};return ye.set(a,c),t.projectReferences&&(c.references=t.projectReferences.map(St)),c}function xt(){S.strictPropertyInitialization&&!e.getStrictOptionValue(S,"strictNullChecks")&&Mt(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1,"strictPropertyInitialization","strictNullChecks"),S.exactOptionalPropertyTypes&&!e.getStrictOptionValue(S,"strictNullChecks")&&Mt(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1,"exactOptionalPropertyTypes","strictNullChecks"),S.isolatedModules&&(S.out&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"out","isolatedModules"),S.outFile&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"outFile","isolatedModules")),S.inlineSourceMap&&(S.sourceMap&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"sourceMap","inlineSourceMap"),S.mapRoot&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"mapRoot","inlineSourceMap")),S.composite&&(!1===S.declaration&&Mt(e.Diagnostics.Composite_projects_may_not_disable_declaration_emit,"declaration"),!1===S.incremental&&Mt(e.Diagnostics.Composite_projects_may_not_disable_incremental_compilation,"declaration"));var n=e.outFile(S);if(S.tsBuildInfoFile?e.isIncrementalCompilation(S)||Mt(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2,"tsBuildInfoFile","incremental","composite"):!S.incremental||n||S.configFilePath||se.add(e.createCompilerDiagnostic(e.Diagnostics.Option_incremental_can_only_be_specified_using_tsconfig_emitting_to_single_file_or_when_option_tsBuildInfoFile_is_specified)),It(),S.composite)for(var t=new e.Set(T.map(Ze)),r=0,i=m;r<i.length;r++){var a=i[r];e.sourceFileMayBeEmitted(a,He)&&!t.has(a.path)&&At(a,e.Diagnostics.File_0_is_not_listed_within_the_file_list_of_project_1_Projects_must_list_all_files_or_use_an_include_pattern,[a.fileName,S.configFilePath||""])}if(S.paths)for(var o in S.paths)if(e.hasProperty(S.paths,o))if(e.hasZeroOrOneAsteriskCharacter(o)||kt(!0,o,e.Diagnostics.Pattern_0_can_have_at_most_one_Asterisk_character,o),e.isArray(S.paths[o])){var s=S.paths[o].length;0===s&&kt(!1,o,e.Diagnostics.Substitutions_for_pattern_0_shouldn_t_be_an_empty_array,o);for(var c=0;c<s;c++){var l=S.paths[o][c],u=typeof l;"string"===u?(e.hasZeroOrOneAsteriskCharacter(l)||Nt(o,c,e.Diagnostics.Substitution_0_in_pattern_1_can_have_at_most_one_Asterisk_character,l,o),S.baseUrl||e.pathIsRelative(l)||e.pathIsAbsolute(l)||Nt(o,c,e.Diagnostics.Non_relative_paths_are_not_allowed_when_baseUrl_is_not_set_Did_you_forget_a_leading_Slash)):Nt(o,c,e.Diagnostics.Substitution_0_for_pattern_1_has_incorrect_type_expected_string_got_2,l,o,u)}}else kt(!1,o,e.Diagnostics.Substitutions_for_pattern_0_should_be_an_array,o);S.sourceMap||S.inlineSourceMap||(S.inlineSources&&Mt(e.Diagnostics.Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided,"inlineSources"),S.sourceRoot&&Mt(e.Diagnostics.Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided,"sourceRoot")),S.out&&S.outFile&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"out","outFile"),!S.mapRoot||S.sourceMap||S.declarationMap||Mt(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2,"mapRoot","sourceMap","declarationMap"),S.declarationDir&&(e.getEmitDeclarations(S)||Mt(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2,"declarationDir","declaration","composite"),n&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"declarationDir",S.out?"out":"outFile")),S.declarationMap&&!e.getEmitDeclarations(S)&&Mt(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2,"declarationMap","declaration","composite"),S.lib&&S.noLib&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"lib","noLib"),S.noImplicitUseStrict&&e.getStrictOptionValue(S,"alwaysStrict")&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"noImplicitUseStrict","alwaysStrict");var d=S.target||0,p=e.find(m,function(n){return e.isExternalModule(n)&&!n.isDeclarationFile});if(S.isolatedModules){S.module===e.ModuleKind.None&&d<2&&Mt(e.Diagnostics.Option_isolatedModules_can_only_be_used_when_either_option_module_is_provided_or_option_target_is_ES2015_or_higher,"isolatedModules","target"),!1===S.preserveConstEnums&&Mt(e.Diagnostics.Option_preserveConstEnums_cannot_be_disabled_when_isolatedModules_is_enabled,"preserveConstEnums","isolatedModules");var f=e.find(m,function(n){return!e.isExternalModule(n)&&!e.isSourceFileJS(n)&&!n.isDeclarationFile&&6!==n.scriptKind});if(f){var _=e.getErrorSpanForNode(f,f);se.add(e.createFileDiagnostic(f,_.start,_.length,e.Diagnostics._0_cannot_be_compiled_under_isolatedModules_because_it_is_considered_a_global_script_file_Add_an_import_export_or_an_empty_export_statement_to_make_it_a_module,e.getBaseFileName(f.fileName)))}}else if(p&&d<2&&S.module===e.ModuleKind.None){_=e.getErrorSpanForNode(p,p.externalModuleIndicator);se.add(e.createFileDiagnostic(p,_.start,_.length,e.Diagnostics.Cannot_use_imports_exports_or_module_augmentations_when_module_is_none))}if(n&&!S.emitDeclarationOnly)if(S.module&&S.module!==e.ModuleKind.AMD&&S.module!==e.ModuleKind.System)Mt(e.Diagnostics.Only_amd_and_system_modules_are_supported_alongside_0,S.out?"out":"outFile","module");else if(void 0===S.module&&p){_=e.getErrorSpanForNode(p,p.externalModuleIndicator);se.add(e.createFileDiagnostic(p,_.start,_.length,e.Diagnostics.Cannot_compile_modules_using_option_0_unless_the_module_flag_is_amd_or_system,S.out?"out":"outFile"))}if(S.resolveJsonModule&&(e.getEmitModuleResolutionKind(S)!==e.ModuleResolutionKind.NodeJs?Mt(e.Diagnostics.Option_resolveJsonModule_cannot_be_specified_without_node_module_resolution_strategy,"resolveJsonModule"):e.hasJsonModuleEmitEnabled(S)||Mt(e.Diagnostics.Option_resolveJsonModule_can_only_be_specified_when_module_code_generation_is_commonjs_amd_es2015_or_esNext,"resolveJsonModule","module")),S.outDir||S.rootDir||S.sourceRoot||S.mapRoot){var g=$e();S.outDir&&""===g&&m.some(function(n){return e.getRootLength(n.fileName)>1})&&Mt(e.Diagnostics.Cannot_find_the_common_subdirectory_path_for_the_input_files,"outDir")}if(S.useDefineForClassFields&&0===d&&Mt(e.Diagnostics.Option_0_cannot_be_specified_when_option_target_is_ES3,"useDefineForClassFields"),S.checkJs&&!e.getAllowJSCompilerOption(S)&&se.add(e.createCompilerDiagnostic(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1,"checkJs","allowJs")),S.emitDeclarationOnly&&(e.getEmitDeclarations(S)||Mt(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2,"emitDeclarationOnly","declaration","composite"),S.noEmit&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"emitDeclarationOnly","noEmit")),S.emitDecoratorMetadata&&!S.experimentalDecorators&&Mt(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1,"emitDecoratorMetadata","experimentalDecorators"),S.jsxFactory?(S.reactNamespace&&Mt(e.Diagnostics.Option_0_cannot_be_specified_with_option_1,"reactNamespace","jsxFactory"),4!==S.jsx&&5!==S.jsx||Mt(e.Diagnostics.Option_0_cannot_be_specified_when_option_jsx_is_1,"jsxFactory",e.inverseJsxOptionMap.get(""+S.jsx)),e.parseIsolatedEntityName(S.jsxFactory,d)||Ft("jsxFactory",e.Diagnostics.Invalid_value_for_jsxFactory_0_is_not_a_valid_identifier_or_qualified_name,S.jsxFactory)):S.reactNamespace&&!e.isIdentifierText(S.reactNamespace,d)&&Ft("reactNamespace",e.Diagnostics.Invalid_value_for_reactNamespace_0_is_not_a_valid_identifier,S.reactNamespace),S.jsxFragmentFactory&&(S.jsxFactory||Mt(e.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1,"jsxFragmentFactory","jsxFactory"),4!==S.jsx&&5!==S.jsx||Mt(e.Diagnostics.Option_0_cannot_be_specified_when_option_jsx_is_1,"jsxFragmentFactory",e.inverseJsxOptionMap.get(""+S.jsx)),e.parseIsolatedEntityName(S.jsxFragmentFactory,d)||Ft("jsxFragmentFactory",e.Diagnostics.Invalid_value_for_jsxFragmentFactory_0_is_not_a_valid_identifier_or_qualified_name,S.jsxFragmentFactory)),S.reactNamespace&&(4!==S.jsx&&5!==S.jsx||Mt(e.Diagnostics.Option_0_cannot_be_specified_when_option_jsx_is_1,"reactNamespace",e.inverseJsxOptionMap.get(""+S.jsx))),S.jsxImportSource&&2===S.jsx&&Mt(e.Diagnostics.Option_0_cannot_be_specified_when_option_jsx_is_1,"jsxImportSource",e.inverseJsxOptionMap.get(""+S.jsx)),!S.noEmit&&!S.suppressOutputPathCheck){var y=an(),h=new e.Set;e.forEachEmittedFile(y,function(e){S.emitDeclarationOnly||v(e.jsFilePath,h),v(e.declarationFilePath,h)})}function v(n,t){if(n){var r=Ze(n);if(xe.has(r)){var i=void 0;S.configFilePath||(i=e.chainDiagnosticMessages(void 0,e.Diagnostics.Adding_a_tsconfig_json_file_will_help_organize_projects_that_contain_both_TypeScript_and_JavaScript_files_Learn_more_at_https_Colon_Slash_Slashaka_ms_Slashtsconfig)),i=e.chainDiagnosticMessages(i,e.Diagnostics.Cannot_write_file_0_because_it_would_overwrite_input_file,n),jt(n,e.createCompilerDiagnosticFromMessageChain(i))}var a=te.useCaseSensitiveFileNames()?r:e.toFileNameLowerCase(r);t.has(a)?jt(n,e.createCompilerDiagnostic(e.Diagnostics.Cannot_write_file_0_because_it_would_be_overwritten_by_multiple_input_files,n)):t.add(a)}}}function Lt(n,t,r,i){var o,s,c,l=A(t)?t:void 0;n&&(null===(o=K.get(n.path))||void 0===o||o.forEach(m)),t&&m(t),l&&1===(null===s||void 0===s?void 0:s.length)&&(s=void 0);var u=l&&I(vn,l),d=s&&e.chainDiagnosticMessages(s,e.Diagnostics.The_file_is_in_the_program_because_Colon),p=n&&e.explainIfFileIsRedirect(n),f=e.chainDiagnosticMessages.apply(void 0,a([p?d?a([d],p,!0):p:d,r],i||e.emptyArray,!1));return u&&C(u)?e.createFileDiagnosticFromMessageChain(u.file,u.pos,u.end-u.pos,f,c):e.createCompilerDiagnosticFromMessageChain(f,c);function m(n){(s||(s=[])).push(e.fileIncludeReasonToDiagnostics(He,n)),!l&&A(n)?l=n:l!==n&&(c=e.append(c,Ct(n))),n===t&&(t=void 0)}}function Dt(e,n,t,r){(b||(b=[])).push({kind:1,file:e&&e.path,fileProcessingReason:n,diagnostic:t,args:r})}function At(e,n,t){se.add(Lt(e,void 0,n,t))}function Ct(n){if(A(n)){var t,r=I(vn,n);switch(n.kind){case e.FileIncludeKind.Import:t=e.Diagnostics.File_is_included_via_import_here;break;case e.FileIncludeKind.ReferenceFile:t=e.Diagnostics.File_is_included_via_reference_here;break;case e.FileIncludeKind.TypeReferenceDirective:t=e.Diagnostics.File_is_included_via_type_library_reference_here;break;case e.FileIncludeKind.LibReferenceDirective:t=e.Diagnostics.File_is_included_via_library_reference_here;break;default:e.Debug.assertNever(n)}return C(r)?e.createFileDiagnostic(r.file,r.pos,r.end-r.pos,t):void 0}if(S.configFile){var i,a;switch(n.kind){case e.FileIncludeKind.RootFile:if(!S.configFile.configFileSpecs)return;var o=e.getNormalizedAbsolutePath(T[n.index],ce),s=e.getMatchedFileSpec(He,o);if(s){i=e.getTsConfigPropArrayElementValue(S.configFile,"files",s),a=e.Diagnostics.File_is_matched_by_files_list_specified_here;break}var c=e.getMatchedIncludeSpec(He,o);if(!c)return;i=e.getTsConfigPropArrayElementValue(S.configFile,"include",c),a=e.Diagnostics.File_is_matched_by_include_pattern_specified_here;break;case e.FileIncludeKind.SourceFromProjectReference:case e.FileIncludeKind.OutputFromProjectReference:var l=e.Debug.checkDefined(null===ge||void 0===ge?void 0:ge[n.index]),u=D(N,ge,function(e,n,t){return e===l?{sourceFile:(null===n||void 0===n?void 0:n.sourceFile)||S.configFile,index:t}:void 0});if(!u)return;var d=u.sourceFile,p=u.index,f=e.firstDefined(e.getTsConfigPropArray(d,"references"),function(n){return e.isArrayLiteralExpression(n.initializer)?n.initializer:void 0});return f&&f.elements.length>p?e.createDiagnosticForNodeInSourceFile(d,f.elements[p],n.kind===e.FileIncludeKind.OutputFromProjectReference?e.Diagnostics.File_is_output_from_referenced_project_specified_here:e.Diagnostics.File_is_source_from_referenced_project_specified_here):void 0;case e.FileIncludeKind.AutomaticTypeDirectiveFile:if(!S.types)return;i=Pt("types",n.typeReference),a=e.Diagnostics.File_is_entry_point_of_type_library_specified_here;break;case e.FileIncludeKind.LibFile:if(void 0!==n.index){i=Pt("lib",S.lib[n.index]),a=e.Diagnostics.File_is_library_specified_here;break}var m=e.forEachEntry(e.targetOptionDeclaration.type,function(e,n){return e===S.target?n:void 0});i=m?wt("target",m):void 0,a=e.Diagnostics.File_is_default_library_for_target_specified_here;break;default:e.Debug.assertNever(n)}return i&&e.createDiagnosticForNodeInSourceFile(S.configFile,i,a)}}function It(){var n=S.suppressOutputPathCheck?void 0:e.getTsBuildInfoEmitOutputFilePath(S);D(N,ge,function(t,r,i){var a=(r?r.commandLine.projectReferences:N)[i],o=r&&r.sourceFile;if(t){var s=t.commandLine.options;if(!s.composite||s.noEmit){var c=r?r.commandLine.fileNames:T;c.length&&(s.composite||Gt(o,i,e.Diagnostics.Referenced_project_0_must_have_setting_composite_Colon_true,a.path),s.noEmit&&Gt(o,i,e.Diagnostics.Referenced_project_0_may_not_disable_emit,a.path))}if(a.prepend){var l=e.outFile(s);l?te.fileExists(l)||Gt(o,i,e.Diagnostics.Output_file_0_from_project_1_does_not_exist,l,a.path):Gt(o,i,e.Diagnostics.Cannot_prepend_project_0_because_it_does_not_have_outFile_set,a.path)}!r&&n&&n===e.getTsBuildInfoEmitOutputFilePath(s)&&(Gt(o,i,e.Diagnostics.Cannot_write_file_0_because_it_will_overwrite_tsbuildinfo_file_generated_by_referenced_project_1,n,a.path),de.set(Ze(n),!0))}else Gt(o,i,e.Diagnostics.File_0_not_found,a.path)})}function Nt(n,t,r,i,a,o){for(var s=!0,c=Ot(),l=0,u=c;l<u.length;l++){var d=u[l];if(e.isObjectLiteralExpression(d.initializer))for(var p=0,f=e.getPropertyAssignment(d.initializer,n);p<f.length;p++){var m=f[p],_=m.initializer;e.isArrayLiteralExpression(_)&&_.elements.length>t&&(se.add(e.createDiagnosticForNodeInSourceFile(S.configFile,_.elements[t],r,i,a,o)),s=!1)}}s&&se.add(e.createCompilerDiagnostic(r,i,a,o))}function kt(n,t,r,i){for(var a=!0,o=Ot(),s=0,c=o;s<c.length;s++){var l=c[s];e.isObjectLiteralExpression(l.initializer)&&Vt(l.initializer,n,t,void 0,r,i)&&(a=!1)}a&&se.add(e.createCompilerDiagnostic(r,i))}function Rt(n){var t=Ut();return t&&e.getPropertyAssignment(t,n)}function Ot(){return Rt("paths")||e.emptyArray}function wt(n,t){var r=Rt(n);return r&&e.firstDefined(r,function(n){return e.isStringLiteral(n.initializer)&&n.initializer.text===t?n.initializer:void 0})}function Pt(n,t){var r=Ut();return r&&e.getPropertyArrayElementValue(r,n,t)}function Mt(e,n,t,r){Bt(!0,n,t,e,n,t,r)}function Ft(e,n,t){Bt(!1,e,void 0,n,t)}function Gt(n,t,r,i,a){var o=e.firstDefined(e.getTsConfigPropArray(n||S.configFile,"references"),function(n){return e.isArrayLiteralExpression(n.initializer)?n.initializer:void 0});o&&o.elements.length>t?se.add(e.createDiagnosticForNodeInSourceFile(n||S.configFile,o.elements[t],r,i,a)):se.add(e.createCompilerDiagnostic(r,i,a))}function Bt(n,t,r,i,a,o,s){var c=Ut(),l=!c||!Vt(c,n,t,r,i,a,o,s);l&&se.add(e.createCompilerDiagnostic(i,a,o,s))}function Ut(){if(void 0===Q){Q=!1;var n=e.getTsConfigObjectLiteralExpression(S.configFile);if(n)for(var t=0,r=e.getPropertyAssignment(n,"compilerOptions");t<r.length;t++){var i=r[t];if(e.isObjectLiteralExpression(i.initializer)){Q=i.initializer;break}}}return Q||void 0}function Vt(n,t,r,i,a,o,s,c){for(var l=e.getPropertyAssignment(n,r,i),u=0,d=l;u<d.length;u++){var p=d[u];se.add(e.createDiagnosticForNodeInSourceFile(S.configFile,t?p.name:p.initializer,a,o,s,c))}return!!l.length}function jt(e,n){de.set(Ze(e),!0),se.add(n)}function Kt(n){if(S.noEmit)return!1;var t=Ze(n);if(vn(t))return!1;var r=e.outFile(S);if(r)return Ht(t,r)||Ht(t,e.removeFileExtension(r)+".d.ts");if(S.declarationDir&&e.containsPath(S.declarationDir,t,ce,!te.useCaseSensitiveFileNames()))return!0;if(S.outDir)return e.containsPath(S.outDir,t,ce,!te.useCaseSensitiveFileNames());if(e.fileExtensionIsOneOf(t,e.supportedJSExtensions)||e.fileExtensionIs(t,".d.ts")){var i=e.removeFileExtension(t);return!!vn(i+".ts")||!!vn(i+".tsx")}return!1}function Ht(n,t){return 0===e.comparePaths(n,t,ce,!te.useCaseSensitiveFileNames())}function Wt(){return te.getSymlinkCache?te.getSymlinkCache():(_||(_=e.createSymlinkCache(ce,bt)),m&&z&&!_.hasProcessedResolutions()&&_.setSymlinksFromResolutions(m,z),_)}}function P(n){var t,r,i=n.compilerHost.fileExists,a=n.compilerHost.directoryExists,o=n.compilerHost.getDirectories,s=n.compilerHost.realpath;return n.useSourceOfProjectReferenceRedirect?(n.compilerHost.fileExists=l,a&&(r=n.compilerHost.directoryExists=function(r){return a.call(n.compilerHost,r)?(p(r),!0):!!n.getResolvedProjectReferences()&&(t||(t=new e.Set,n.forEachResolvedProjectReference(function(r){var i=e.outFile(r.commandLine.options);if(i)t.add(e.getDirectoryPath(n.toPath(i)));else{var a=r.commandLine.options.declarationDir||r.commandLine.options.outDir;a&&t.add(n.toPath(a))}})),f(r,!1))}),o&&(n.compilerHost.getDirectories=function(e){return!n.getResolvedProjectReferences()||a&&a.call(n.compilerHost,e)?o.call(n.compilerHost,e):[]}),s&&(n.compilerHost.realpath=function(e){var t;return(null===(t=n.getSymlinkCache().getSymlinkedFiles())||void 0===t?void 0:t.get(n.toPath(e)))||s.call(n.compilerHost,e)}),{onProgramCreateComplete:c,fileExists:l,directoryExists:r}):{onProgramCreateComplete:e.noop,fileExists:l};function c(){n.compilerHost.fileExists=i,n.compilerHost.directoryExists=a,n.compilerHost.getDirectories=o}function l(t){return!!i.call(n.compilerHost,t)||!!n.getResolvedProjectReferences()&&(!!e.isDeclarationFileName(t)&&f(t,!0))}function u(t){var r=n.getSourceOfProjectReferenceRedirect(t);return void 0!==r?!e.isString(r)||i.call(n.compilerHost,r):void 0}function d(r){var i=n.toPath(r),a=""+i+e.directorySeparator;return e.forEachKey(t,function(n){return i===n||e.startsWith(n,a)||e.startsWith(i,n+"/")})}function p(t){var r;if(n.getResolvedProjectReferences()&&!e.containsIgnoredPath(t)&&s&&e.stringContains(t,e.nodeModulesPathPart)){var i=n.getSymlinkCache(),a=e.ensureTrailingDirectorySeparator(n.toPath(t));if(null===(r=i.getSymlinkedDirectories())||void 0===r||!r.has(a)){var o,c=e.normalizePath(s.call(n.compilerHost,t));c!==t&&(o=e.ensureTrailingDirectorySeparator(n.toPath(c)))!==a?i.setSymlinkedDirectory(t,{real:e.ensureTrailingDirectorySeparator(c),realPath:o}):i.setSymlinkedDirectory(a,!1)}}}function f(t,r){var i,a=r?function(e){return u(e)}:function(e){return d(e)},o=a(t);if(void 0!==o)return o;var s=n.getSymlinkCache(),c=s.getSymlinkedDirectories();if(!c)return!1;var l=n.toPath(t);return!!e.stringContains(l,e.nodeModulesPathPart)&&(!!(r&&(null===(i=s.getSymlinkedFiles())||void 0===i?void 0:i.has(l)))||(e.firstDefinedIterator(c.entries(),function(i){var o=i[0],c=i[1];if(c&&e.startsWith(l,o)){var u=a(l.replace(o,c.realPath));if(r&&u){var d=e.getNormalizedAbsolutePath(t,n.compilerHost.getCurrentDirectory());s.setSymlinkedFile(l,""+c.real+d.replace(new RegExp(o,"i"),""))}return u}})||!1))}}function M(n,t,r,i){var o=n.getCompilerOptions();if(o.noEmit)return n.getSemanticDiagnostics(t,i),t||e.outFile(o)?e.emitSkippedWithNoDiagnostics:n.emitBuildInfo(r,i);if(o.noEmitOnError){var s=a(a(a(a([],n.getOptionsDiagnostics(i),!0),n.getSyntacticDiagnostics(t,i),!0),n.getGlobalDiagnostics(i),!0),n.getSemanticDiagnostics(t,i),!0);if(0===s.length&&e.getEmitDeclarations(n.getCompilerOptions())&&(s=n.getDeclarationDiagnostics(void 0,i)),s.length){var c;if(!t&&!e.outFile(o)){var l=n.emitBuildInfo(r,i);l.diagnostics&&(s=a(a([],s,!0),l.diagnostics,!0)),c=l.emittedFiles}return{diagnostics:s,sourceMaps:void 0,emittedFiles:c,emitSkipped:!0}}}}function F(n,t){return e.filter(n,function(e){return!e.skippedOn||!t[e.skippedOn]})}function G(n,t){return void 0===t&&(t=n),{fileExists:function(e){return t.fileExists(e)},readDirectory:function(n,r,i,a,o){return e.Debug.assertIsDefined(t.readDirectory,"'CompilerHost.readDirectory' must be implemented to correctly process 'projectReferences'"),t.readDirectory(n,r,i,a,o)},readFile:function(e){return t.readFile(e)},useCaseSensitiveFileNames:n.useCaseSensitiveFileNames(),getCurrentDirectory:function(){return n.getCurrentDirectory()},onUnRecoverableConfigFileDiagnostic:n.onUnRecoverableConfigFileDiagnostic||e.returnUndefined,trace:n.trace?function(e){return n.trace(e)}:void 0}}function B(n,t,r){if(!n)return e.emptyArray;for(var i,a=0;a<n.length;a++){var o=n[a],s=t(o,a);if(o.prepend&&s&&s.options){var c=e.outFile(s.options);if(!c)continue;var l=e.getOutputPathsForBundle(s.options,!0),u=l.jsFilePath,d=l.sourceMapFilePath,p=l.declarationFilePath,f=l.declarationMapPath,m=l.buildInfoPath,_=e.createInputFiles(r,u,d,p,f,m);(i||(i=[])).push(_)}}return i||e.emptyArray}function U(n,t){var r=t||n;return e.resolveConfigFileProjectName(r.path)}function V(n,t){var r=t.extension;switch(r){case".ts":case".d.ts":return;case".tsx":return i();case".jsx":return i()||a();case".js":return a();case".json":return o()}function i(){return n.jsx?void 0:e.Diagnostics.Module_0_was_resolved_to_1_but_jsx_is_not_set}function a(){return e.getAllowJSCompilerOption(n)||!e.getStrictOptionValue(n,"noImplicitAny")?void 0:e.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type}function o(){return n.resolveJsonModule?void 0:e.Diagnostics.Module_0_was_resolved_to_1_but_resolveJsonModule_is_not_used}}function j(e){for(var n=e.imports,t=e.moduleAugmentations,r=n.map(function(e){return e.text}),i=0,a=t;i<a.length;i++){var o=a[i];10===o.kind&&r.push(o.text)}return r}function K(n,t){var r=n.imports,i=n.moduleAugmentations;if(t<r.length)return r[t];for(var a=r.length,o=0,s=i;o<s.length;o++){var c=s[o];if(10===c.kind){if(t===a)return c;a++}}e.Debug.fail("should never ask for module name at index higher than possible module name")}e.formatColorAndReset=v,e.formatLocation=E,e.formatDiagnosticsWithColorAndContext=T,e.flattenDiagnosticMessageText=S,e.loadWithLocalCache=x,e.forEachResolvedProjectReference=L,e.inferredTypesContainingFile="__inferred type names__.ts",e.isReferencedFile=A,e.isReferenceFileLocation=C,e.getReferencedFileLocation=I,e.isProgramUptoDate=N,e.getConfigFileParsingDiagnostics=k,e.createProgram=w,e.emitSkippedWithNoDiagnostics={diagnostics:e.emptyArray,sourceMaps:void 0,emittedFiles:void 0,emitSkipped:!0},e.handleNoEmitOptions=M,e.filterSemanticDiagnostics=F,e.parseConfigHostFromCompilerHostLike=G,e.createPrependNodes=B,e.resolveProjectReferencePath=U,e.getResolutionDiagnostic=V,e.getModuleNameStringLiteralAt=K}(d||(d={})),function(e){function n(e,n,t,r,i,a){var o=[],s=e.emit(n,d,r,t,i,a),c=s.emitSkipped,l=s.diagnostics,u=s.exportedModulesFromDeclarationEmit;return{outputFiles:o,emitSkipped:c,diagnostics:l,exportedModulesFromDeclarationEmit:u};function d(e,n,t){o.push({name:e,writeByteOrderMark:t,text:n})}}e.getFileEmitOutput=n,function(t){var r=0;function i(){function n(t,i,s){var c=0,l={id:r++,version:function(){return c},clone:function(){return n(new e.Map(t),new e.Map(i),s&&new e.Set(s))},forEach:function(e){return t.forEach(e)},getKeys:function(e){return i.get(e)},getValues:function(e){return t.get(e)},hasKey:function(e){return t.has(e)},keys:function(){return t.keys()},deletedKeys:function(){return s},deleteKey:function(n){(s||(s=new e.Set)).add(n);var r=t.get(n);return!!r&&(r.forEach(function(e){return o(i,e,n)}),t.delete(n),c++,!0)},set:function(e,n){var r=!!(null===s||void 0===s?void 0:s.delete(e)),u=t.get(e);return t.set(e,n),null===u||void 0===u||u.forEach(function(t){n.has(t)||(r=!0,o(i,t,e))}),n.forEach(function(n){(null===u||void 0===u?void 0:u.has(n))||(r=!0,a(i,n,e))}),r&&c++,l}};return l}return n(new e.Map,new e.Map,void 0)}function a(n,t,r){var i=n.get(t);i||(i=new e.Set,n.set(t,i)),i.add(r)}function o(e,n,t,r){void 0===r&&(r=!0);var i=e.get(n);return!(null===i||void 0===i||!i.delete(t))&&(r&&!i.size&&e.delete(n),!0)}function s(n){return e.mapDefined(n.declarations,function(n){var t;return null===(t=e.getSourceFileOfNode(n))||void 0===t?void 0:t.resolvedPath})}function c(e,n){var t=e.getSymbolAtLocation(n);return t&&s(t)}function l(n,t,r,i){return e.toPath(n.getProjectReferenceRedirect(t)||t,r,i)}function u(n,t,r){var i;if(t.imports&&t.imports.length>0)for(var a=n.getTypeChecker(),o=0,s=t.imports;o<s.length;o++){var u=s[o],d=c(a,u);null===d||void 0===d||d.forEach(L)}var p=e.getDirectoryPath(t.resolvedPath);if(t.referencedFiles&&t.referencedFiles.length>0)for(var f=0,m=t.referencedFiles;f<m.length;f++){var _=m[f],g=l(n,_.fileName,p,r);L(g)}if(t.resolvedTypeReferenceDirectiveNames&&t.resolvedTypeReferenceDirectiveNames.forEach(function(e){if(e){var t=e.resolvedFileName,i=l(n,t,p,r);L(i)}}),t.moduleAugmentations.length){a=n.getTypeChecker();for(var y=0,h=t.moduleAugmentations;y<h.length;y++){var v=h[y];if(e.isStringLiteral(v)){var b=a.getSymbolAtLocation(v);b&&x(b)}}}for(var E=0,T=n.getTypeChecker().getAmbientModules();E<T.length;E++){var S=T[E];S.declarations&&S.declarations.length>1&&x(S)}return i;function x(n){if(n.declarations)for(var r=0,i=n.declarations;r<i.length;r++){var a=i[r],o=e.getSourceFileOfNode(a);o&&o!==t&&L(o.resolvedPath)}}function L(n){(i||(i=new e.Set)).add(n)}}function d(e,n){return n&&!n.referencedMap===!e}function p(n,t,r,a){var o=new e.Map,s=n.getCompilerOptions().module!==e.ModuleKind.None?i():void 0,c=s?i():void 0,l=new e.Set,p=d(s,r);n.getTypeChecker();for(var f=0,m=n.getSourceFiles();f<m.length;f++){var _=m[f],g=e.Debug.checkDefined(_.version,"Program intended to be used with Builder should have source files with versions set"),y=p?r.fileInfos.get(_.resolvedPath):void 0;if(s){var h=u(n,_,t);if(h&&s.set(_.resolvedPath,h),p){var v=r.exportedModulesMap.getValues(_.resolvedPath);v&&c.set(_.resolvedPath,v)}}o.set(_.resolvedPath,{version:g,signature:y&&y.signature,affectsGlobalScope:D(_)||void 0})}return{fileInfos:o,referencedMap:s,exportedModulesMap:c,hasCalledUpdateShapeSignature:l,useFileVersionAsSignature:!a&&!p}}function f(e){e.allFilesExcludingDefaultLibraryFile=void 0,e.allFileNames=void 0}function m(n){var t,r;return{fileInfos:new e.Map(n.fileInfos),referencedMap:null===(t=n.referencedMap)||void 0===t?void 0:t.clone(),exportedModulesMap:null===(r=n.exportedModulesMap)||void 0===r?void 0:r.clone(),hasCalledUpdateShapeSignature:new e.Set(n.hasCalledUpdateShapeSignature),useFileVersionAsSignature:n.useFileVersionAsSignature}}function _(n,t,r,i,a,o,s){var c=o||new e.Map,l=t.getSourceFileByPath(r);if(!l)return e.emptyArray;if(!h(n,t,l,c,i,a,s))return[l];var u=(n.referencedMap?I:C)(n,t,l,c,i,a,s);return o||g(n,c),u}function g(e,n){n.forEach(function(n,t){return y(e,n,t)})}function y(e,n,t){e.fileInfos.get(t).signature=n,e.hasCalledUpdateShapeSignature.add(t)}function h(t,r,i,a,o,s,c,l){if(void 0===l&&(l=t.useFileVersionAsSignature),e.Debug.assert(!!i),e.Debug.assert(!c||!!t.exportedModulesMap,"Compute visible to outside map only if visibleToOutsideReferencedMap present in the state"),t.hasCalledUpdateShapeSignature.has(i.resolvedPath)||a.has(i.resolvedPath))return!1;var u=t.fileInfos.get(i.resolvedPath);if(!u)return e.Debug.fail();var d,p=u.signature;if(!i.isDeclarationFile&&!l){var f=n(r,i,!0,o,void 0,!0),m=e.firstOrUndefined(f.outputFiles);m&&(e.Debug.assert(e.fileExtensionIs(m.name,".d.ts"),"File extension for signature expected to be dts",function(){return"Found: "+e.getAnyExtensionFromPath(m.name)+" for "+m.name+":: All output files: "+JSON.stringify(f.outputFiles.map(function(e){return e.name}))}),d=(s||e.generateDjb2Hash)(m.text),c&&d!==p&&v(i,f.exportedModulesFromDeclarationEmit,c))}if(void 0===d&&(d=i.version,c&&d!==p)){var _=t.referencedMap?t.referencedMap.getValues(i.resolvedPath):void 0;_?c.set(i.resolvedPath,_):c.deleteKey(i.resolvedPath)}return a.set(i.resolvedPath,d),d!==p}function v(n,t,r){var i;t?(t.forEach(function(e){return a(s(e))}),i?r.set(n.resolvedPath,i):r.deleteKey(n.resolvedPath)):r.deleteKey(n.resolvedPath);function a(n){(null===n||void 0===n?void 0:n.length)&&(i||(i=new e.Set),n.forEach(function(e){return i.add(e)}))}}function b(n,t){var r;if(t){e.Debug.assert(!!n.exportedModulesMap);var i=t.id,a=t.version();if(n.previousCache){if(n.previousCache.id===i&&n.previousCache.version===a)return;n.previousCache.id=i,n.previousCache.version=a}else n.previousCache={id:i,version:a};null===(r=t.deletedKeys())||void 0===r||r.forEach(function(e){return n.exportedModulesMap.deleteKey(e)}),t.forEach(function(e,t){return n.exportedModulesMap.set(t,e)})}}function E(n,t,r){var i=t.getCompilerOptions();if(e.outFile(i))return T(n,t);if(!n.referencedMap||D(r))return T(n,t);var a=new e.Set,o=[r.resolvedPath];while(o.length){var s=o.pop();if(!a.has(s)){a.add(s);var c=n.referencedMap.getValues(s);if(c)for(var l=c.keys(),u=l.next();!u.done;u=l.next())o.push(u.value)}}return e.arrayFrom(e.mapDefinedIterator(a.keys(),function(e){var n,r;return null!==(r=null===(n=t.getSourceFileByPath(e))||void 0===n?void 0:n.fileName)&&void 0!==r?r:e}))}function T(n,t){if(!n.allFileNames){var r=t.getSourceFiles();n.allFileNames=r===e.emptyArray?e.emptyArray:r.map(function(e){return e.fileName})}return n.allFileNames}function S(n,t){var r=n.referencedMap.getKeys(t);return r?e.arrayFrom(r.keys()):[]}function x(n){for(var t=0,r=n.statements;t<r.length;t++){var i=r[t];if(!e.isModuleWithStringLiteralName(i))return!1}return!0}function L(n){return e.some(n.moduleAugmentations,function(n){return e.isGlobalScopeAugmentation(n.parent)})}function D(n){return L(n)||!e.isExternalOrCommonJsModule(n)&&!e.isJsonSourceFile(n)&&!x(n)}function A(n,t,r){if(n.allFilesExcludingDefaultLibraryFile)return n.allFilesExcludingDefaultLibraryFile;var i;r&&c(r);for(var a=0,o=t.getSourceFiles();a<o.length;a++){var s=o[a];s!==r&&c(s)}return n.allFilesExcludingDefaultLibraryFile=i||e.emptyArray,n.allFilesExcludingDefaultLibraryFile;function c(e){t.isSourceFileDefaultLibrary(e)||(i||(i=[])).push(e)}}function C(n,t,r){var i=t.getCompilerOptions();return i&&e.outFile(i)?[r]:A(n,t,r)}function I(n,t,r,i,a,o,s){if(D(r))return A(n,t,r);var c=t.getCompilerOptions();if(c&&(c.isolatedModules||e.outFile(c)))return[r];var l=new e.Map;l.set(r.resolvedPath,r);var u=S(n,r.resolvedPath);while(u.length>0){var d=u.pop();if(!l.has(d)){var p=t.getSourceFileByPath(d);l.set(d,p),p&&h(n,t,p,i,a,o,s)&&u.push.apply(u,S(n,p.resolvedPath))}}return e.arrayFrom(e.mapDefinedIterator(l.values(),function(e){return e}))}t.createManyToManyPathMap=i,t.canReuseOldState=d,t.create=p,t.releaseCache=f,t.clone=m,t.getFilesAffectedBy=_,t.updateSignaturesFromCache=g,t.updateSignatureOfFile=y,t.updateShapeSignature=h,t.updateExportedFilesMapFromCache=b,t.getAllDependencies=E,t.getReferencedByPaths=S,t.getAllFilesExcludingDefaultLibraryFile=A}(e.BuilderState||(e.BuilderState={}))}(d||(d={})),function(e){var n;function t(n,t){return n===t||void 0!==n&&void 0!==t&&n.size===t.size&&!e.forEachKey(n,function(e){return!t.has(e)})}function r(n,r,a,o){var s=e.BuilderState.create(n,r,a,o);s.program=n;var c=n.getCompilerOptions();s.compilerOptions=c,e.outFile(c)||(s.semanticDiagnosticsPerFile=new e.Map),s.changedFilesSet=new e.Set;var l=e.BuilderState.canReuseOldState(s.referencedMap,a),u=l?a.compilerOptions:void 0,d=l&&a.semanticDiagnosticsPerFile&&!!s.semanticDiagnosticsPerFile&&!e.compilerOptionsAffectSemanticDiagnostics(c,u);if(l){if(!a.currentChangedFilePath){var p=a.currentAffectedFilesSignatures;e.Debug.assert(!a.affectedFiles&&(!p||!p.size),"Cannot reuse if only few affected files of currentChangedFile were iterated")}var f=a.changedFilesSet;d&&e.Debug.assert(!f||!e.forEachKey(f,function(e){return a.semanticDiagnosticsPerFile.has(e)}),"Semantic diagnostics shouldnt be available for changed files"),null===f||void 0===f||f.forEach(function(e){return s.changedFilesSet.add(e)}),!e.outFile(c)&&a.affectedFilesPendingEmit&&(s.affectedFilesPendingEmit=a.affectedFilesPendingEmit.slice(),s.affectedFilesPendingEmitKind=a.affectedFilesPendingEmitKind&&new e.Map(a.affectedFilesPendingEmitKind),s.affectedFilesPendingEmitIndex=a.affectedFilesPendingEmitIndex,s.seenAffectedFiles=new e.Set)}var m=s.referencedMap,_=l?a.referencedMap:void 0,g=d&&!c.skipLibCheck===!u.skipLibCheck,y=g&&!c.skipDefaultLibCheck===!u.skipDefaultLibCheck;return s.fileInfos.forEach(function(o,c){var u,p;if(!l||!(u=a.fileInfos.get(c))||u.version!==o.version||!t(p=m&&m.getValues(c),_&&_.getValues(c))||p&&e.forEachKey(p,function(e){return!s.fileInfos.has(e)&&a.fileInfos.has(e)}))s.changedFilesSet.add(c);else if(d){var f=n.getSourceFileByPath(c);if(f.isDeclarationFile&&!g)return;if(f.hasNoDefaultLib&&!y)return;var h=a.semanticDiagnosticsPerFile.get(c);h&&(s.semanticDiagnosticsPerFile.set(c,a.hasReusableDiagnostic?i(h,n,r):h),s.semanticDiagnosticsFromOldState||(s.semanticDiagnosticsFromOldState=new e.Set),s.semanticDiagnosticsFromOldState.add(c))}}),l&&e.forEachEntry(a.fileInfos,function(e,n){return e.affectsGlobalScope&&!s.fileInfos.has(n)})?e.BuilderState.getAllFilesExcludingDefaultLibraryFile(s,n,void 0).forEach(function(e){return s.changedFilesSet.add(e.resolvedPath)}):u&&!e.outFile(c)&&e.compilerOptionsAffectEmit(c,u)&&(n.getSourceFiles().forEach(function(e){return k(s,e.resolvedPath,1)}),e.Debug.assert(!s.seenAffectedFiles||!s.seenAffectedFiles.size),s.seenAffectedFiles=s.seenAffectedFiles||new e.Set),s.buildInfoEmitPending=!!s.changedFilesSet.size,s}function i(n,t,r){if(!n.length)return e.emptyArray;var i=e.getDirectoryPath(e.getNormalizedAbsolutePath(e.getTsBuildInfoEmitOutputFilePath(t.getCompilerOptions()),t.getCurrentDirectory()));return n.map(function(e){var n=a(e,t,o);n.reportsUnnecessary=e.reportsUnnecessary,n.reportsDeprecated=e.reportDeprecated,n.source=e.source,n.skippedOn=e.skippedOn;var r=e.relatedInformation;return n.relatedInformation=r?r.length?r.map(function(e){return a(e,t,o)}):[]:void 0,n});function o(n){return e.toPath(n,i,r)}}function a(e,n,t){var r=e.file;return o(o({},e),{file:r?n.getSourceFileByPath(t(r)):void 0})}function s(n){e.BuilderState.releaseCache(n),n.program=void 0}function c(n){var t,r=e.BuilderState.clone(n);return r.semanticDiagnosticsPerFile=n.semanticDiagnosticsPerFile&&new e.Map(n.semanticDiagnosticsPerFile),r.changedFilesSet=new e.Set(n.changedFilesSet),r.affectedFiles=n.affectedFiles,r.affectedFilesIndex=n.affectedFilesIndex,r.currentChangedFilePath=n.currentChangedFilePath,r.currentAffectedFilesSignatures=n.currentAffectedFilesSignatures&&new e.Map(n.currentAffectedFilesSignatures),r.currentAffectedFilesExportedModulesMap=null===(t=n.currentAffectedFilesExportedModulesMap)||void 0===t?void 0:t.clone(),r.seenAffectedFiles=n.seenAffectedFiles&&new e.Set(n.seenAffectedFiles),r.cleanedDiagnosticsOfLibFiles=n.cleanedDiagnosticsOfLibFiles,r.semanticDiagnosticsFromOldState=n.semanticDiagnosticsFromOldState&&new e.Set(n.semanticDiagnosticsFromOldState),r.program=n.program,r.compilerOptions=n.compilerOptions,r.affectedFilesPendingEmit=n.affectedFilesPendingEmit&&n.affectedFilesPendingEmit.slice(),r.affectedFilesPendingEmitKind=n.affectedFilesPendingEmitKind&&new e.Map(n.affectedFilesPendingEmitKind),r.affectedFilesPendingEmitIndex=n.affectedFilesPendingEmitIndex,r.seenEmittedFiles=n.seenEmittedFiles&&new e.Map(n.seenEmittedFiles),r.programEmitComplete=n.programEmitComplete,r}function l(n,t){e.Debug.assert(!t||!n.affectedFiles||n.affectedFiles[n.affectedFilesIndex-1]!==t||!n.semanticDiagnosticsPerFile.has(t.resolvedPath))}function u(n,t,r){while(1){var i=n.affectedFiles;if(i){var a=n.seenAffectedFiles,o=n.affectedFilesIndex;while(o<i.length){var s=i[o];if(!a.has(s.resolvedPath))return n.affectedFilesIndex=o,p(n,s,t,r),s;o++}n.changedFilesSet.delete(n.currentChangedFilePath),n.currentChangedFilePath=void 0,e.BuilderState.updateSignaturesFromCache(n,n.currentAffectedFilesSignatures),n.currentAffectedFilesSignatures.clear(),e.BuilderState.updateExportedFilesMapFromCache(n,n.currentAffectedFilesExportedModulesMap),n.affectedFiles=void 0}var c=n.changedFilesSet.keys().next();if(c.done)return;var l=e.Debug.checkDefined(n.program),u=l.getCompilerOptions();if(e.outFile(u))return e.Debug.assert(!n.semanticDiagnosticsPerFile),l;n.currentAffectedFilesSignatures||(n.currentAffectedFilesSignatures=new e.Map),n.exportedModulesMap&&(n.currentAffectedFilesExportedModulesMap||(n.currentAffectedFilesExportedModulesMap=e.BuilderState.createManyToManyPathMap())),n.affectedFiles=e.BuilderState.getFilesAffectedBy(n,l,c.value,t,r,n.currentAffectedFilesSignatures,n.currentAffectedFilesExportedModulesMap),n.currentChangedFilePath=c.value,n.affectedFilesIndex=0,n.seenAffectedFiles||(n.seenAffectedFiles=new e.Set)}}function d(n){var t=n.affectedFilesPendingEmit;if(t){for(var r=n.seenEmittedFiles||(n.seenEmittedFiles=new e.Map),i=n.affectedFilesPendingEmitIndex;i<t.length;i++){var a=e.Debug.checkDefined(n.program).getSourceFileByPath(t[i]);if(a){var o=r.get(a.resolvedPath),s=e.Debug.checkDefined(e.Debug.checkDefined(n.affectedFilesPendingEmitKind).get(a.resolvedPath));if(void 0===o||o<s)return n.affectedFilesPendingEmitIndex=i,{affectedFile:a,emitKind:s}}}n.affectedFilesPendingEmit=void 0,n.affectedFilesPendingEmitKind=void 0,n.affectedFilesPendingEmitIndex=void 0}}function p(n,t,r,i){var a;if(m(n,t.resolvedPath),n.allFilesExcludingDefaultLibraryFile!==n.affectedFiles)e.Debug.assert(n.hasCalledUpdateShapeSignature.has(t.resolvedPath)||(null===(a=n.currentAffectedFilesSignatures)||void 0===a?void 0:a.has(t.resolvedPath)),"Signature not updated for affected file: "+t.fileName),n.compilerOptions.assumeChangesOnlyAffectDirectDependencies||g(n,t,function(e,n){return f(e,n,r,i)});else{if(!n.cleanedDiagnosticsOfLibFiles){n.cleanedDiagnosticsOfLibFiles=!0;var o=e.Debug.checkDefined(n.program),s=o.getCompilerOptions();e.forEach(o.getSourceFiles(),function(t){return o.isSourceFileDefaultLibrary(t)&&!e.skipTypeChecking(t,s,o)&&m(n,t.resolvedPath)})}e.BuilderState.updateShapeSignature(n,e.Debug.checkDefined(n.program),t,e.Debug.checkDefined(n.currentAffectedFilesSignatures),r,i,n.currentAffectedFilesExportedModulesMap)}}function f(n,t,r,i){if(m(n,t),!n.changedFilesSet.has(t)){var a=e.Debug.checkDefined(n.program),o=a.getSourceFileByPath(t);o&&(e.BuilderState.updateShapeSignature(n,a,o,e.Debug.checkDefined(n.currentAffectedFilesSignatures),r,i,n.currentAffectedFilesExportedModulesMap,!0),e.getEmitDeclarations(n.compilerOptions)&&k(n,t,0))}}function m(e,n){return!e.semanticDiagnosticsFromOldState||(e.semanticDiagnosticsFromOldState.delete(n),e.semanticDiagnosticsPerFile.delete(n),!e.semanticDiagnosticsFromOldState.size)}function _(n,t){var r=e.Debug.checkDefined(n.currentAffectedFilesSignatures).get(t),i=e.Debug.checkDefined(n.fileInfos.get(t)).signature;return r!==i}function g(n,t,r){var i,a;if(n.exportedModulesMap&&n.changedFilesSet.has(t.resolvedPath)&&_(n,t.resolvedPath)){if(n.compilerOptions.isolatedModules){var o=new e.Map;o.set(t.resolvedPath,!0);var s=e.BuilderState.getReferencedByPaths(n,t.resolvedPath);while(s.length>0){var c=s.pop();if(!o.has(c)&&(o.set(c,!0),r(n,c),_(n,c))){var l=e.Debug.checkDefined(n.program).getSourceFileByPath(c);s.push.apply(s,e.BuilderState.getReferencedByPaths(n,l.resolvedPath))}}}e.Debug.assert(!!n.currentAffectedFilesExportedModulesMap);var u=new e.Set;null===(i=n.currentAffectedFilesExportedModulesMap.getKeys(t.resolvedPath))||void 0===i||i.forEach(function(e){return y(n,e,u,r)}),null===(a=n.exportedModulesMap.getKeys(t.resolvedPath))||void 0===a||a.forEach(function(e){var t;return!n.currentAffectedFilesExportedModulesMap.hasKey(e)&&!(null===(t=n.currentAffectedFilesExportedModulesMap.deletedKeys())||void 0===t?void 0:t.has(e))&&y(n,e,u,r)})}}function y(e,n,t,r){var i;null===(i=e.referencedMap.getKeys(n))||void 0===i||i.forEach(function(n){return h(e,n,t,r)})}function h(n,t,r,i){var a,o,s;e.tryAddToSet(r,t)&&(i(n,t),e.Debug.assert(!!n.currentAffectedFilesExportedModulesMap),null===(a=n.currentAffectedFilesExportedModulesMap.getKeys(t))||void 0===a||a.forEach(function(e){return h(n,e,r,i)}),null===(o=n.exportedModulesMap.getKeys(t))||void 0===o||o.forEach(function(e){var t;return!n.currentAffectedFilesExportedModulesMap.hasKey(e)&&!(null===(t=n.currentAffectedFilesExportedModulesMap.deletedKeys())||void 0===t?void 0:t.has(e))&&h(n,e,r,i)}),null===(s=n.referencedMap.getKeys(t))||void 0===s||s.forEach(function(e){return!r.has(e)&&i(n,e)}))}function v(n,t,r,i,a){a?n.buildInfoEmitPending=!1:t===n.program?(n.changedFilesSet.clear(),n.programEmitComplete=!0):(n.seenAffectedFiles.add(t.resolvedPath),void 0!==r&&(n.seenEmittedFiles||(n.seenEmittedFiles=new e.Map)).set(t.resolvedPath,r),i?(n.affectedFilesPendingEmitIndex++,n.buildInfoEmitPending=!0):n.affectedFilesIndex++)}function b(e,n,t){return v(e,t),{result:n,affected:t}}function E(e,n,t,r,i,a){return v(e,t,r,i,a),{result:n,affected:t}}function T(n,t,r){return e.concatenate(S(n,t,r),e.Debug.checkDefined(n.program).getProgramDiagnostics(t))}function S(n,t,r){var i=t.resolvedPath;if(n.semanticDiagnosticsPerFile){var a=n.semanticDiagnosticsPerFile.get(i);if(a)return e.filterSemanticDiagnostics(a,n.compilerOptions)}var o=e.Debug.checkDefined(n.program).getBindAndCheckDiagnostics(t,r);return n.semanticDiagnosticsPerFile&&n.semanticDiagnosticsPerFile.set(i,o),e.filterSemanticDiagnostics(o,n.compilerOptions)}function x(n,t){if(!e.outFile(n.compilerOptions)){var r,i,a,o,s,c,l=e.Debug.checkDefined(n.program).getCurrentDirectory(),u=e.getDirectoryPath(e.getNormalizedAbsolutePath(e.getTsBuildInfoEmitOutputFilePath(n.compilerOptions),l)),d=[],p=new e.Map,f=e.arrayFrom(n.fileInfos.entries(),function(t){var r=t[0],i=t[1],a=x(r);e.Debug.assert(d[a-1]===S(r));var o=n.currentAffectedFilesSignatures&&n.currentAffectedFilesSignatures.get(r),s=null!==o&&void 0!==o?o:i.signature;return i.version===s?i.affectsGlobalScope?{version:i.version,signature:void 0,affectsGlobalScope:!0}:i.version:void 0!==s?void 0===o?i:{version:i.version,signature:o,affectsGlobalScope:i.affectsGlobalScope}:{version:i.version,signature:!1,affectsGlobalScope:i.affectsGlobalScope}});if(n.referencedMap&&(a=e.arrayFrom(n.referencedMap.keys()).sort(e.compareStringsCaseSensitive).map(function(e){return[x(e),D(n.referencedMap.getValues(e))]})),n.exportedModulesMap&&(o=e.mapDefined(e.arrayFrom(n.exportedModulesMap.keys()).sort(e.compareStringsCaseSensitive),function(e){var t;if(n.currentAffectedFilesExportedModulesMap){if(null===(t=n.currentAffectedFilesExportedModulesMap.deletedKeys())||void 0===t?void 0:t.has(e))return;var r=n.currentAffectedFilesExportedModulesMap.getValues(e);if(r)return[x(e),D(r)]}return[x(e),D(n.exportedModulesMap.getValues(e))]})),n.semanticDiagnosticsPerFile)for(var m=0,_=e.arrayFrom(n.semanticDiagnosticsPerFile.keys()).sort(e.compareStringsCaseSensitive);m<_.length;m++){var g=_[m],y=n.semanticDiagnosticsPerFile.get(g);(s||(s=[])).push(y.length?[x(g),n.hasReusableDiagnostic?y:A(y,S)]:x(g))}if(n.affectedFilesPendingEmit)for(var h=new e.Set,v=0,b=n.affectedFilesPendingEmit.slice(n.affectedFilesPendingEmitIndex).sort(e.compareStringsCaseSensitive);v<b.length;v++){var E=b[v];e.tryAddToSet(h,E)&&(c||(c=[])).push([x(E),n.affectedFilesPendingEmitKind.get(E)])}return{fileNames:d,fileInfos:f,options:L(n.compilerOptions,T),fileIdsList:r,referencedMap:a,exportedModulesMap:o,semanticDiagnosticsPerFile:s,affectedFilesPendingEmit:c}}function T(n){return S(e.getNormalizedAbsolutePath(n,l))}function S(n){return e.ensurePathIsNonModuleName(e.getRelativePathFromDirectory(u,n,t))}function x(e){var n=p.get(e);return void 0===n&&(d.push(S(e)),p.set(e,n=d.length)),n}function D(n){var t=e.arrayFrom(n.keys(),x).sort(e.compareValues),a=t.join(),o=null===i||void 0===i?void 0:i.get(a);return void 0===o&&((r||(r=[])).push(t),(i||(i=new e.Map)).set(a,o=r.length)),o}}function L(n,t){for(var r,i=e.getOptionsNameMap().optionsNameMap,a=0,o=e.getOwnKeys(n).sort(e.compareStringsCaseSensitive);a<o.length;a++){var s=o[a],c=s.toLowerCase(),l=i.get(c);((null===l||void 0===l?void 0:l.affectsEmit)||(null===l||void 0===l?void 0:l.affectsSemanticDiagnostics)||"strict"===c||"skiplibcheck"===c||"skipdefaultlibcheck"===c)&&((r||(r={}))[s]=D(l,n[s],t))}return r}function D(e,n,t){if(e)if("list"===e.type){var r=n;if(e.element.isFilePath&&r.length)return r.map(t)}else if(e.isFilePath)return t(n);return n}function A(n,t){return e.Debug.assert(!!n.length),n.map(function(e){var n=C(e,t);n.reportsUnnecessary=e.reportsUnnecessary,n.reportDeprecated=e.reportsDeprecated,n.source=e.source,n.skippedOn=e.skippedOn;var r=e.relatedInformation;return n.relatedInformation=r?r.length?r.map(function(e){return C(e,t)}):[]:void 0,n})}function C(e,n){var t=e.file;return o(o({},e),{file:t?n(t.resolvedPath):void 0})}function I(n,t,r,i,a,o){var s,c,l;return void 0===n?(e.Debug.assert(void 0===t),s=r,l=i,e.Debug.assert(!!l),c=l.getProgram()):e.isArray(n)?(l=i,c=e.createProgram({rootNames:n,options:t,host:r,oldProgram:l&&l.getProgramOrUndefined(),configFileParsingDiagnostics:a,projectReferences:o}),s=r):(c=n,s=t,l=r,a=i),{host:s,newProgram:c,oldProgram:l,configFileParsingDiagnostics:a||e.emptyArray}}function N(t,i){var a=i.newProgram,o=i.host,p=i.oldProgram,f=i.configFileParsingDiagnostics,m=p&&p.getState();if(m&&a===m.program&&f===a.getConfigFileParsingDiagnostics())return a=void 0,m=void 0,p;var _,g=e.createGetCanonicalFileName(o.useCaseSensitiveFileNames()),y=e.maybeBind(o,o.createHash),h=r(a,g,m,o.disableUseFileVersionAsSignature);a.getProgramBuildInfo=function(){return x(h,g)},a=void 0,p=void 0,m=void 0;var S=function(){return h},L=w(S,f);return L.getState=S,L.backupState=function(){e.Debug.assert(void 0===_),_=c(h)},L.restoreState=function(){h=e.Debug.checkDefined(_),_=void 0},L.getAllDependencies=function(n){return e.BuilderState.getAllDependencies(h,e.Debug.checkDefined(h.program),n)},L.getSemanticDiagnostics=N,L.emit=C,L.releaseProgram=function(){s(h),_=void 0},t===n.SemanticDiagnosticsBuilderProgram?L.getSemanticDiagnosticsOfNextAffectedFile=I:t===n.EmitAndSemanticDiagnosticsBuilderProgram?(L.getSemanticDiagnosticsOfNextAffectedFile=I,L.emitNextAffectedFile=A,L.emitBuildInfo=D):e.notImplemented(),L;function D(n,t){if(h.buildInfoEmitPending){var r=e.Debug.checkDefined(h.program).emitBuildInfo(n||e.maybeBind(o,o.writeFile),t);return h.buildInfoEmitPending=!1,r}return e.emitSkippedWithNoDiagnostics}function A(n,t,r,i){var a=u(h,t,y),s=1,c=!1;if(!a)if(e.outFile(h.compilerOptions)){var l=e.Debug.checkDefined(h.program);if(h.programEmitComplete)return;a=l}else{var p=d(h);if(!p){if(!h.buildInfoEmitPending)return;var f=e.Debug.checkDefined(h.program);return E(h,f.emitBuildInfo(n||e.maybeBind(o,o.writeFile),t),f,1,!1,!0)}a=p.affectedFile,s=p.emitKind,c=!0}return E(h,e.Debug.checkDefined(h.program).emit(a===h.program?void 0:a,n||e.maybeBind(o,o.writeFile),t,r||0===s,i),a,s,c)}function C(r,i,a,s,c){var u,d,p,f=!1;t===n.EmitAndSemanticDiagnosticsBuilderProgram||r||e.outFile(h.compilerOptions)||h.compilerOptions.noEmit||!h.compilerOptions.noEmitOnError||(f=!0,u=h.affectedFilesPendingEmit&&h.affectedFilesPendingEmit.slice(),d=h.affectedFilesPendingEmitKind&&new e.Map(h.affectedFilesPendingEmitKind),p=h.affectedFilesPendingEmitIndex),t===n.EmitAndSemanticDiagnosticsBuilderProgram&&l(h,r);var m=e.handleNoEmitOptions(L,r,i,a);if(m)return m;if(f&&(h.affectedFilesPendingEmit=u,h.affectedFilesPendingEmitKind=d,h.affectedFilesPendingEmitIndex=p),!r&&t===n.EmitAndSemanticDiagnosticsBuilderProgram){var _=[],g=!1,y=void 0,v=[],b=void 0;while(b=A(i,a,s,c))g=g||b.result.emitSkipped,y=e.addRange(y,b.result.diagnostics),v=e.addRange(v,b.result.emittedFiles),_=e.addRange(_,b.result.sourceMaps);return{emitSkipped:g,diagnostics:y||e.emptyArray,emittedFiles:v,sourceMaps:_}}return e.Debug.checkDefined(h.program).emit(r,i||e.maybeBind(o,o.writeFile),a,s,c)}function I(e,r){while(1){var i=u(h,e,y);if(!i)return;if(i===h.program)return b(h,h.program.getSemanticDiagnostics(void 0,e),i);if((t===n.EmitAndSemanticDiagnosticsBuilderProgram||h.compilerOptions.noEmit||h.compilerOptions.noEmitOnError)&&k(h,i.resolvedPath,1),!r||!r(i))return b(h,T(h,i,e),i);v(h,i)}}function N(n,t){l(h,n);var r,i=e.Debug.checkDefined(h.program).getCompilerOptions();if(e.outFile(i))return e.Debug.assert(!h.semanticDiagnosticsPerFile),e.Debug.checkDefined(h.program).getSemanticDiagnostics(n,t);if(n)return T(h,n,t);while(I(t));for(var a=0,o=e.Debug.checkDefined(h.program).getSourceFiles();a<o.length;a++){var s=o[a];r=e.addRange(r,T(h,s,t))}return r||e.emptyArray}}function k(n,t,r){n.affectedFilesPendingEmit||(n.affectedFilesPendingEmit=[]),n.affectedFilesPendingEmitKind||(n.affectedFilesPendingEmitKind=new e.Map);var i=n.affectedFilesPendingEmitKind.get(t);n.affectedFilesPendingEmit.push(t),n.affectedFilesPendingEmitKind.set(t,i||r),void 0===n.affectedFilesPendingEmitIndex&&(n.affectedFilesPendingEmitIndex=0)}function R(n){return e.isString(n)?{version:n,signature:n,affectsGlobalScope:void 0}:e.isString(n.signature)?n:{version:n.version,signature:!1===n.signature?void 0:n.version,affectsGlobalScope:n.affectsGlobalScope}}function O(n,t,r){var i,a=e.getDirectoryPath(e.getNormalizedAbsolutePath(t,r.getCurrentDirectory())),o=e.createGetCanonicalFileName(r.useCaseSensitiveFileNames()),s=n.fileNames.map(d),c=null===(i=n.fileIdsList)||void 0===i?void 0:i.map(function(n){return new e.Set(n.map(f))}),l=new e.Map;n.fileInfos.forEach(function(e,n){return l.set(f(n+1),R(e))});var u={fileInfos:l,compilerOptions:n.options?e.convertToOptionsWithAbsolutePaths(n.options,p):{},referencedMap:_(n.referencedMap),exportedModulesMap:_(n.exportedModulesMap),semanticDiagnosticsPerFile:n.semanticDiagnosticsPerFile&&e.arrayToMap(n.semanticDiagnosticsPerFile,function(n){return f(e.isNumber(n)?n:n[0])},function(n){return e.isNumber(n)?e.emptyArray:n[1]}),hasReusableDiagnostic:!0,affectedFilesPendingEmit:e.map(n.affectedFilesPendingEmit,function(e){return f(e[0])}),affectedFilesPendingEmitKind:n.affectedFilesPendingEmit&&e.arrayToMap(n.affectedFilesPendingEmit,function(e){return f(e[0])},function(e){return e[1]}),affectedFilesPendingEmitIndex:n.affectedFilesPendingEmit&&0};return{getState:function(){return u},backupState:e.noop,restoreState:e.noop,getProgram:e.notImplemented,getProgramOrUndefined:e.returnUndefined,releaseProgram:e.noop,getCompilerOptions:function(){return u.compilerOptions},getSourceFile:e.notImplemented,getSourceFiles:e.notImplemented,getOptionsDiagnostics:e.notImplemented,getGlobalDiagnostics:e.notImplemented,getConfigFileParsingDiagnostics:e.notImplemented,getSyntacticDiagnostics:e.notImplemented,getDeclarationDiagnostics:e.notImplemented,getSemanticDiagnostics:e.notImplemented,emit:e.notImplemented,getAllDependencies:e.notImplemented,getCurrentDirectory:e.notImplemented,emitNextAffectedFile:e.notImplemented,getSemanticDiagnosticsOfNextAffectedFile:e.notImplemented,emitBuildInfo:e.notImplemented,close:e.noop};function d(n){return e.toPath(n,a,o)}function p(n){return e.getNormalizedAbsolutePath(n,a)}function f(e){return s[e-1]}function m(e){return c[e-1]}function _(n){if(n){var t=e.BuilderState.createManyToManyPathMap();return n.forEach(function(e){var n=e[0],r=e[1];return t.set(f(n),m(r))}),t}}}function w(n,t){return{getState:e.notImplemented,backupState:e.noop,restoreState:e.noop,getProgram:r,getProgramOrUndefined:function(){return n().program},releaseProgram:function(){return n().program=void 0},getCompilerOptions:function(){return n().compilerOptions},getSourceFile:function(e){return r().getSourceFile(e)},getSourceFiles:function(){return r().getSourceFiles()},getOptionsDiagnostics:function(e){return r().getOptionsDiagnostics(e)},getGlobalDiagnostics:function(e){return r().getGlobalDiagnostics(e)},getConfigFileParsingDiagnostics:function(){return t},getSyntacticDiagnostics:function(e,n){return r().getSyntacticDiagnostics(e,n)},getDeclarationDiagnostics:function(e,n){return r().getDeclarationDiagnostics(e,n)},getSemanticDiagnostics:function(e,n){return r().getSemanticDiagnostics(e,n)},emit:function(e,n,t,i,a){return r().emit(e,n,t,i,a)},emitBuildInfo:function(e,n){return r().emitBuildInfo(e,n)},getAllDependencies:e.notImplemented,getCurrentDirectory:function(){return r().getCurrentDirectory()},close:e.noop};function r(){return e.Debug.checkDefined(n().program)}}(function(e){e[e["DtsOnly"]=0]="DtsOnly",e[e["Full"]=1]="Full"})(e.BuilderFileEmit||(e.BuilderFileEmit={})),function(e){e[e["SemanticDiagnosticsBuilderProgram"]=0]="SemanticDiagnosticsBuilderProgram",e[e["EmitAndSemanticDiagnosticsBuilderProgram"]=1]="EmitAndSemanticDiagnosticsBuilderProgram"}(n=e.BuilderProgramKind||(e.BuilderProgramKind={})),e.getBuilderCreationParameters=I,e.createBuilderProgram=N,e.toBuilderStateFileInfo=R,e.createBuildProgramUsingProgramBuildInfo=O,e.createRedirectedBuilderProgram=w}(d||(d={})),function(e){function n(n,t,r,i,a,o){return e.createBuilderProgram(e.BuilderProgramKind.SemanticDiagnosticsBuilderProgram,e.getBuilderCreationParameters(n,t,r,i,a,o))}function t(n,t,r,i,a,o){return e.createBuilderProgram(e.BuilderProgramKind.EmitAndSemanticDiagnosticsBuilderProgram,e.getBuilderCreationParameters(n,t,r,i,a,o))}function r(n,t,r,i,a,o){var s=e.getBuilderCreationParameters(n,t,r,i,a,o),c=s.newProgram,l=s.configFileParsingDiagnostics;return e.createRedirectedBuilderProgram(function(){return{program:c,compilerOptions:c.getCompilerOptions()}},l)}e.createSemanticDiagnosticsBuilderProgram=n,e.createEmitAndSemanticDiagnosticsBuilderProgram=t,e.createAbstractBuilder=r}(d||(d={})),function(e){function n(n){return e.endsWith(n,"/node_modules/.staging")?e.removeSuffix(n,"/.staging"):e.some(e.ignoredPaths,function(t){return e.stringContains(n,t)})?void 0:n}function t(n){var t=e.getRootLength(n);if(n.length===t)return!1;var r=n.indexOf(e.directorySeparator,t);if(-1===r)return!1;var i=n.substring(t,r+1),a=t>1||47!==n.charCodeAt(0);if(a&&0!==n.search(/[a-zA-Z]:/)&&0===i.search(/[a-zA-z]\$\//)){if(r=n.indexOf(e.directorySeparator,r+1),-1===r)return!1;i=n.substring(t+i.length,r+1)}if(a&&0!==i.search(/users\//i))return!0;for(var o=r+1,s=2;s>0;s--)if(o=n.indexOf(e.directorySeparator,o)+1,0===o)return!1;return!0}function r(r,i,a){var o,s,c,l,u,d,p=e.createMultiMap(),f=[],m=e.createMultiMap(),_=!1,g=e.memoize(function(){return r.getCurrentDirectory()}),y=r.getCachedDirectoryStructureHost(),h=new e.Map,v=e.createCacheWithRedirects(),b=e.createCacheWithRedirects(),E=e.createModuleResolutionCache(g(),r.getCanonicalFileName,void 0,v,b),T=new e.Map,S=e.createCacheWithRedirects(),x=e.createTypeReferenceDirectiveResolutionCache(g(),r.getCanonicalFileName,void 0,E.getPackageJsonInfoCache(),S),L=[".ts",".tsx",".js",".jsx",".json"],D=new e.Map,A=new e.Map,C=i&&e.removeTrailingDirectorySeparator(e.getNormalizedAbsolutePath(i,g())),I=C&&r.toPath(C),N=void 0!==I?I.split(e.directorySeparator).length:0,k=new e.Map;return{getModuleResolutionCache:function(){return E},startRecordingFilesWithChangedResolutions:M,finishRecordingFilesWithChangedResolutions:F,startCachingPerDirectoryResolution:U,finishCachingPerDirectoryResolution:V,resolveModuleNames:z,getResolvedModuleWithFailedLookupLocationsFromCache:J,resolveTypeReferenceDirectives:W,removeResolutionsFromProjectReferenceRedirects:oe,removeResolutionsOfFile:se,hasChangedAutomaticTypeDirectiveNames:function(){return _},invalidateResolutionOfFile:le,invalidateResolutionsOfFailedLookupLocations:pe,setFilesWithInvalidatedNonRelativeUnresolvedImports:ue,createHasInvalidatedResolution:B,isFileWithInvalidatedNonRelativeUnresolvedImports:G,updateTypeRootsWatch:ye,closeTypeRootsWatch:me,clear:P};function R(e){return e.resolvedModule}function O(e){return e.resolvedTypeReferenceDirective}function w(n,t){return!(void 0===n||t.length<=n.length)&&(e.startsWith(t,n)&&t[n.length]===e.directorySeparator)}function P(){e.clearMap(A,e.closeFileWatcherOf),D.clear(),p.clear(),me(),h.clear(),T.clear(),m.clear(),f.length=0,l=void 0,u=void 0,d=void 0,U(),_=!1}function M(){o=[]}function F(){var e=o;return o=void 0,e}function G(e){if(!c)return!1;var n=c.get(e);return!!n&&!!n.length}function B(n){if(pe(),n)return s=void 0,e.returnTrue;var t=s;return s=void 0,function(e){return!!t&&t.has(e)||G(e)}}function U(){E.clear(),x.clear(),p.forEach(ee),p.clear()}function V(){c=void 0,U(),A.forEach(function(e,n){0===e.refCount&&(A.delete(n),e.watcher.close())}),_=!1}function j(n,t,i,a,o){var s,c=e.resolveModuleName(n,t,i,a,E,o);if(!r.getGlobalCache)return c;var l=r.getGlobalCache();if(void 0!==l&&!e.isExternalModuleNameRelative(n)&&(!c.resolvedModule||!e.extensionIsTS(c.resolvedModule.extension))){var u=e.loadModuleFromGlobalCache(e.Debug.checkDefined(r.globalCacheResolutionModuleName)(n),r.projectName,i,a,l,E),d=u.resolvedModule,p=u.failedLookupLocations;if(d)return c.resolvedModule=d,(s=c.failedLookupLocations).push.apply(s,p),c}return c}function K(n,t,r,i,a){return e.resolveTypeReferenceDirective(n,t,r,i,a,x)}function H(n){var t,i,a,s=n.names,c=n.containingFile,l=n.redirectedReference,u=n.cache,d=n.perDirectoryCacheWithRedirects,p=n.loader,f=n.getResolutionWithResolvedFileName,m=n.shouldRetryResolution,_=n.reusedNames,g=n.logChanges,y=r.toPath(c),h=u.get(y)||u.set(y,new e.Map).get(y),v=e.getDirectoryPath(y),b=d.getOrCreateMapOfCacheRedirects(l),E=b.get(v);E||(E=new e.Map,b.set(v,E));for(var T=[],S=r.getCompilationSettings(),x=g&&G(y),L=r.getCurrentProgram(),D=L&&L.getResolvedProjectReferenceToRedirect(c),A=D?!l||l.sourceFile.path!==D.sourceFile.path:!!l,C=new e.Map,I=0,N=s;I<N.length;I++){var k=N[I],R=h.get(k);if(!C.has(k)&&A||!R||R.isInvalidated||x&&!e.isExternalModuleNameRelative(k)&&m(R)){var O=R,w=E.get(k);if(w){R=w;var P=(null===(t=r.getCompilerHost)||void 0===t?void 0:t.call(r))||r;if(e.isTraceEnabled(S,P)){var M=f(R);e.trace(P,p===j?(null===M||void 0===M?void 0:M.resolvedFileName)?M.packagetId?e.Diagnostics.Reusing_resolution_of_module_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_3_with_Package_ID_4:e.Diagnostics.Reusing_resolution_of_module_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_3:e.Diagnostics.Reusing_resolution_of_module_0_from_1_found_in_cache_from_location_2_it_was_not_resolved:(null===M||void 0===M?void 0:M.resolvedFileName)?M.packagetId?e.Diagnostics.Reusing_resolution_of_type_reference_directive_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_3_with_Package_ID_4:e.Diagnostics.Reusing_resolution_of_type_reference_directive_0_from_1_found_in_cache_from_location_2_it_was_successfully_resolved_to_3:e.Diagnostics.Reusing_resolution_of_type_reference_directive_0_from_1_found_in_cache_from_location_2_it_was_not_resolved,k,c,e.getDirectoryPath(c),null===M||void 0===M?void 0:M.resolvedFileName,(null===M||void 0===M?void 0:M.packagetId)&&e.packageIdToString(M.packagetId))}}else R=p(k,c,S,(null===(i=r.getCompilerHost)||void 0===i?void 0:i.call(r))||r,l),E.set(k,R);h.set(k,R),Z(k,R,y,f),O&&te(O,y,f),g&&o&&!F(O,R)&&(o.push(y),g=!1)}else{P=(null===(a=r.getCompilerHost)||void 0===a?void 0:a.call(r))||r;if(e.isTraceEnabled(S,P)&&!C.has(k)){M=f(R);e.trace(P,p===j?(null===M||void 0===M?void 0:M.resolvedFileName)?M.packagetId?e.Diagnostics.Reusing_resolution_of_module_0_from_1_of_old_program_it_was_successfully_resolved_to_2_with_Package_ID_3:e.Diagnostics.Reusing_resolution_of_module_0_from_1_of_old_program_it_was_successfully_resolved_to_2:e.Diagnostics.Reusing_resolution_of_module_0_from_1_of_old_program_it_was_not_resolved:(null===M||void 0===M?void 0:M.resolvedFileName)?M.packagetId?e.Diagnostics.Reusing_resolution_of_type_reference_directive_0_from_1_of_old_program_it_was_successfully_resolved_to_2_with_Package_ID_3:e.Diagnostics.Reusing_resolution_of_type_reference_directive_0_from_1_of_old_program_it_was_successfully_resolved_to_2:e.Diagnostics.Reusing_resolution_of_type_reference_directive_0_from_1_of_old_program_it_was_not_resolved,k,c,null===M||void 0===M?void 0:M.resolvedFileName,(null===M||void 0===M?void 0:M.packagetId)&&e.packageIdToString(M.packagetId))}}e.Debug.assert(void 0!==R&&!R.isInvalidated),C.set(k,!0),T.push(f(R))}return h.forEach(function(n,t){C.has(t)||e.contains(_,t)||(te(n,y,f),h.delete(t))}),T;function F(e,n){if(e===n)return!0;if(!e||!n)return!1;var t=f(e),r=f(n);return t===r||!(!t||!r)&&t.resolvedFileName===r.resolvedFileName}}function W(e,n,t){return H({names:e,containingFile:n,redirectedReference:t,cache:T,perDirectoryCacheWithRedirects:S,loader:K,getResolutionWithResolvedFileName:O,shouldRetryResolution:function(e){return void 0===e.resolvedTypeReferenceDirective}})}function z(n,t,r,i){return H({names:n,containingFile:t,redirectedReference:i,cache:h,perDirectoryCacheWithRedirects:v,loader:j,getResolutionWithResolvedFileName:R,shouldRetryResolution:function(n){return!n.resolvedModule||!e.resolutionExtensionIsTSOrJson(n.resolvedModule.extension)},reusedNames:r,logChanges:a})}function J(e,n){var t=h.get(r.toPath(n));return t&&t.get(e)}function q(n){return e.endsWith(n,"/node_modules/@types")}function X(n,t){if(w(I,t)){n=e.isRootedDiskPath(n)?e.normalizePath(n):e.getNormalizedAbsolutePath(n,g());var r=t.split(e.directorySeparator),i=n.split(e.directorySeparator);return e.Debug.assert(i.length===r.length,"FailedLookup: "+n+" failedLookupLocationPath: "+t),r.length>N+1?{dir:i.slice(0,N+1).join(e.directorySeparator),dirPath:r.slice(0,N+1).join(e.directorySeparator)}:{dir:C,dirPath:I,nonRecursive:!1}}return Y(e.getDirectoryPath(e.getNormalizedAbsolutePath(n,g())),e.getDirectoryPath(t))}function Y(n,r){while(e.pathContainsNodeModules(r))n=e.getDirectoryPath(n),r=e.getDirectoryPath(r);if(e.isNodeModulesDirectory(r))return t(e.getDirectoryPath(r))?{dir:n,dirPath:r}:void 0;var i,a,o=!0;if(void 0!==I)while(!w(r,I)){var s=e.getDirectoryPath(r);if(s===r)break;o=!1,i=r,a=n,r=s,n=e.getDirectoryPath(n)}return t(r)?{dir:a||n,dirPath:i||r,nonRecursive:o}:void 0}function Q(n){return e.fileExtensionIsOneOf(n,L)}function Z(n,t,i,a){if(t.refCount)t.refCount++,e.Debug.assertDefined(t.files);else{t.refCount=1,e.Debug.assert(0===e.length(t.files)),e.isExternalModuleNameRelative(n)?$(t):p.add(n,t);var o=a(t);o&&o.resolvedFileName&&m.add(r.toPath(o.resolvedFileName),t)}(t.files||(t.files=[])).push(i)}function $(n){e.Debug.assert(!!n.refCount);var t=n.failedLookupLocations;if(t.length){f.push(n);for(var i=!1,a=0,o=t;a<o.length;a++){var s=o[a],c=r.toPath(s),l=X(s,c);if(l){var u=l.dir,d=l.dirPath,p=l.nonRecursive;if(!Q(c)){var m=D.get(c)||0;D.set(c,m+1)}d===I?(e.Debug.assert(!p),i=!0):ne(u,d,p)}}i&&ne(C,I,!0)}}function ee(e,n){var t=r.getCurrentProgram();t&&t.getTypeChecker().tryFindAmbientModuleWithoutAugmentations(n)||e.forEach($)}function ne(n,t,r){var i=A.get(t);i?(e.Debug.assert(!!r===!!i.nonRecursive),i.refCount++):A.set(t,{watcher:ie(n,t,r),refCount:1,nonRecursive:r})}function te(n,t,i){if(e.unorderedRemoveItem(e.Debug.assertDefined(n.files),t),n.refCount--,!n.refCount){var a=i(n);if(a&&a.resolvedFileName&&m.remove(r.toPath(a.resolvedFileName),n),e.unorderedRemoveItem(f,n)){for(var o=n.failedLookupLocations,s=!1,c=0,l=o;c<l.length;c++){var u=l[c],d=r.toPath(u),p=X(u,d);if(p){var _=p.dirPath,g=D.get(d);g&&(1===g?D.delete(d):(e.Debug.assert(g>1),D.set(d,g-1))),_===I?s=!0:re(_)}}s&&re(I)}}}function re(e){var n=A.get(e);n.refCount--}function ie(e,n,t){return r.watchDirectoryOfFailedLookupLocation(e,function(e){var t=r.toPath(e);y&&y.addOrDeleteFileOrDirectory(e,t),de(t,n===t)},t?0:1)}function ae(e,n,t){var r=e.get(n);r&&(r.forEach(function(e){return te(e,n,t)}),e.delete(n))}function oe(n){if(e.fileExtensionIs(n,".json")){var t=r.getCurrentProgram();if(t){var i=t.getResolvedProjectReferenceByPath(n);i&&i.commandLine.fileNames.forEach(function(e){return se(r.toPath(e))})}}}function se(e){ae(h,e,R),ae(T,e,O)}function ce(n,t){if(!n)return!1;for(var r=!1,i=0,a=n;i<a.length;i++){var o=a[i];if(!o.isInvalidated&&t(o)){o.isInvalidated=r=!0;for(var c=0,l=e.Debug.assertDefined(o.files);c<l.length;c++){var u=l[c];(s||(s=new e.Set)).add(u),_=_||e.endsWith(u,e.inferredTypesContainingFile)}}}return r}function le(n){se(n);var t=_;ce(m.get(n),e.returnTrue)&&_&&!t&&r.onChangedAutomaticTypeDirectiveNames()}function ue(n){e.Debug.assert(c===n||void 0===c),c=n}function de(t,i){if(i)(d||(d=[])).push(t);else{var a=n(t);if(!a)return!1;if(t=a,r.fileIsOpen(t))return!1;var o=e.getDirectoryPath(t);if(q(t)||e.isNodeModulesDirectory(t)||q(o)||e.isNodeModulesDirectory(o))(l||(l=[])).push(t),(u||(u=new e.Set)).add(t);else{if(!Q(t)&&!D.has(t))return!1;if(e.isEmittedFileOfProgram(r.getCurrentProgram(),t))return!1;(l||(l=[])).push(t);var s=e.parseNodeModuleFromPath(t);s&&(u||(u=new e.Set)).add(s)}}r.scheduleInvalidateResolutionsOfFailedLookupLocations()}function pe(){if(!l&&!u&&!d)return!1;var e=ce(f,fe);return l=void 0,u=void 0,d=void 0,e}function fe(n){return n.failedLookupLocations.some(function(n){var t=r.toPath(n);return e.contains(l,t)||e.firstDefinedIterator((null===u||void 0===u?void 0:u.keys())||e.emptyIterator,function(n){return!!e.startsWith(t,n)||void 0})||(null===d||void 0===d?void 0:d.some(function(e){return w(e,t)}))})}function me(){e.clearMap(k,e.closeFileWatcher)}function _e(e,n){if(w(I,n))return I;var t=Y(e,n);return t&&A.has(t.dirPath)?t.dirPath:void 0}function ge(e,n){return r.watchTypeRootsDirectory(n,function(t){var i=r.toPath(t);y&&y.addOrDeleteFileOrDirectory(t,i),_=!0,r.onChangedAutomaticTypeDirectiveNames();var a=_e(n,e);a&&de(i,a===i)},1)}function ye(){var n=r.getCompilationSettings();if(n.types)me();else{var t=e.getEffectiveTypeRoots(n,{directoryExists:he,getCurrentDirectory:g});t?e.mutateMap(k,e.arrayToMap(t,function(e){return r.toPath(e)}),{createNewValue:ge,onDeleteValue:e.closeFileWatcher}):me()}}function he(n){var i=e.getDirectoryPath(e.getDirectoryPath(n)),a=r.toPath(i);return a===I||t(a)}}e.removeIgnoredPath=n,e.canWatchDirectory=t,e.createResolutionCache=r}(d||(d={})),function(e){(function(n){var t,r;function i(n,t,r){var i=n.importModuleSpecifierPreference,a=n.importModuleSpecifierEnding;return{relativePreference:"relative"===i?0:"non-relative"===i?1:"project-relative"===i?3:2,ending:o()};function o(){switch(a){case"minimal":return 0;case"index":return 1;case"js":return 2;default:return v(r)?2:e.getEmitModuleResolutionKind(t)!==e.ModuleResolutionKind.NodeJs?1:0}}}function o(n,t){return{relativePreference:e.isExternalModuleNameRelative(t)?0:1,ending:e.hasJSFileExtension(t)?2:e.getEmitModuleResolutionKind(n)!==e.ModuleResolutionKind.NodeJs||e.endsWith(t,"index")?1:0}}function s(e,n,t,r,i){var a=u(e,n,t,r,o(e,i),{});if(a!==i)return a}function c(e,n,t,r,a){return u(e,t,r,a,i({},e,n),{})}function l(n,t,r,i,a){var o=g(t,i),s=S(t,r,i,a);return e.firstDefined(s,function(e){return C(e,o,i,n,!0)})}function u(n,t,r,i,a,o){var s=g(t,i),c=S(t,r,i,o);return e.firstDefined(c,function(e){return C(e,s,i,n)})||y(r,s,n,i,a)}function d(e,n,t,r){return p(e,n,t,r)[0]}function p(n,t,r,i){var a,o=e.getSourceFileOfModule(n);if(!o)return e.emptyArray;var s=null===(a=r.getModuleSpecifierCache)||void 0===a?void 0:a.call(r),c=null===s||void 0===s?void 0:s.get(t.path,o.path,i);return[null===c||void 0===c?void 0:c.moduleSpecifiers,o,null===c||void 0===c?void 0:c.modulePaths,s]}function f(e,n,t,r,i,a){return m(e,n,t,r,i,a).moduleSpecifiers}function m(n,t,r,i,a,o){var s=!1,c=L(n,t);if(c)return{moduleSpecifiers:[c],computedWithoutCache:s};var l=p(n,i,a,o),u=l[0],d=l[1],f=l[2],m=l[3];if(u)return{moduleSpecifiers:u,computedWithoutCache:s};if(!d)return{moduleSpecifiers:e.emptyArray,computedWithoutCache:s};s=!0,f||(f=x(i.path,d.originalFileName,a));var g=_(f,r,i,a,o);return null===m||void 0===m||m.set(i.path,d.path,o,f,g),{moduleSpecifiers:g,computedWithoutCache:s}}function _(n,t,r,a,o){var s=g(r.path,a),c=i(o,t,r),l=e.forEach(n,function(n){return e.forEach(a.getFileIncludeReasons().get(e.toPath(n.path,a.getCurrentDirectory(),s.getCanonicalFileName)),function(n){if(n.kind===e.FileIncludeKind.Import&&n.file===r.path){var t=e.getModuleNameStringLiteralAt(r,n.index).text;return 1===c.relativePreference&&e.pathIsRelative(t)?void 0:t}})});if(l){var u=[l];return u}for(var d,p,f,m=e.some(n,function(e){return e.isInNodeModules}),_=0,h=n;_<h.length;_++){var v=h[_],b=C(v,s,a,t);if(d=e.append(d,b),b&&v.isRedirect)return d;if(!b&&!v.isRedirect){var E=y(v.path,s,t,a,c);e.pathIsBareSpecifier(E)?p=e.append(p,E):m&&!v.isInNodeModules||(f=e.append(f,E))}}return(null===p||void 0===p?void 0:p.length)?p:(null===d||void 0===d?void 0:d.length)?d:e.Debug.checkDefined(f)}function g(n,t){var r=e.createGetCanonicalFileName(!t.useCaseSensitiveFileNames||t.useCaseSensitiveFileNames()),i=e.getDirectoryPath(n);return{getCanonicalFileName:r,importingSourceFileName:n,sourceDirectory:i}}function y(n,t,r,i,a){var o=a.ending,s=a.relativePreference,c=r.baseUrl,l=r.paths,u=r.rootDirs,d=t.sourceDirectory,p=t.getCanonicalFileName,f=u&&A(u,n,d,p,o,r)||R(e.ensurePathIsNonModuleName(e.getRelativePathFromDirectory(d,n,p)),o,r);if(!c&&!l||0===s)return f;var m=e.getNormalizedAbsolutePath(e.getPathsBasePath(r,i)||c,i.getCurrentDirectory()),_=P(n,m,p);if(!_)return f;var g=R(_,o,r),y=l&&D(e.removeFileExtension(_),g,l),v=void 0===y&&void 0!==c?g:y;if(!v)return f;if(1===s)return v;if(3===s){var b=r.configFilePath?e.toPath(e.getDirectoryPath(r.configFilePath),i.getCurrentDirectory(),t.getCanonicalFileName):t.getCanonicalFileName(i.getCurrentDirectory()),T=e.toPath(n,b,p),S=e.startsWith(d,b),x=e.startsWith(T,b);if(S&&!x||!S&&x)return v;var L=E(i,e.getDirectoryPath(T)),C=E(i,d);return C!==L?v:f}return 2!==s&&e.Debug.assertNever(s),M(v)||h(f)<h(v)?f:v}function h(n){for(var t=0,r=e.startsWith(n,"./")?2:0;r<n.length;r++)47===n.charCodeAt(r)&&t++;return t}function v(n){var t=n.imports;return e.firstDefined(t,function(n){var t=n.text;return e.pathIsRelative(t)?e.hasJSFileExtension(t):void 0})||!1}function b(n,t){return e.compareBooleans(t.isRedirect,n.isRedirect)||e.compareNumberOfDirectorySeparators(n.path,t.path)}function E(n,t){return n.getNearestAncestorDirectoryWithPackageJson?n.getNearestAncestorDirectoryWithPackageJson(t):!!e.forEachAncestorDirectory(t,function(t){return!!n.fileExists(e.combinePaths(t,"package.json"))||void 0})}function T(n,t,r,i,o){var s,c=e.hostGetCanonicalFileName(r),l=r.getCurrentDirectory(),u=r.isSourceOfProjectReferenceRedirect(t)?r.getProjectReferenceRedirect(t):void 0,d=e.toPath(t,l,c),p=r.redirectTargetsMap.get(d)||e.emptyArray,f=a(a(a([],u?[u]:e.emptyArray,!0),[t],!1),p,!0),m=f.map(function(n){return e.getNormalizedAbsolutePath(n,l)}),_=!e.every(m,e.containsIgnoredPath);if(!i){var g=e.forEach(m,function(n){return!(_&&e.containsIgnoredPath(n))&&o(n,u===n)});if(g)return g}var y=null===(s=r.getSymlinkCache)||void 0===s?void 0:s.call(r).getSymlinkedDirectoriesByRealpath(),h=e.getNormalizedAbsolutePath(t,l),v=y&&e.forEachAncestorDirectory(e.getDirectoryPath(h),function(t){var r=y.get(e.ensureTrailingDirectorySeparator(e.toPath(t,l,c)));if(r)return!e.startsWithDirectory(n,t,c)&&e.forEach(m,function(n){if(e.startsWithDirectory(n,t,c))for(var i=e.getRelativePathFromDirectory(t,n,c),a=0,s=r;a<s.length;a++){var l=s[a],d=e.resolvePath(l,i),p=o(d,n===u);if(_=!0,p)return p}})});return v||(i?e.forEach(m,function(n){return _&&e.containsIgnoredPath(n)?void 0:o(n,n===u)}):void 0)}function S(n,t,r,i,a){var o;void 0===a&&(a=e.toPath(t,r.getCurrentDirectory(),e.hostGetCanonicalFileName(r)));var s=null===(o=r.getModuleSpecifierCache)||void 0===o?void 0:o.call(r);if(s){var c=s.get(n,a,i);if(null===c||void 0===c?void 0:c.modulePaths)return c.modulePaths}var l=x(n,t,r);return s&&s.setModulePaths(n,a,i,l),l}function x(n,t,r){var i=e.hostGetCanonicalFileName(r),a=new e.Map,o=!1;T(n,t,r,!0,function(n,t){var r=e.pathContainsNodeModules(n);a.set(n,{path:i(n),isRedirect:t,isInNodeModules:r}),o=o||r});for(var s,c=[],l=function(n){var t,r=e.ensureTrailingDirectorySeparator(n);a.forEach(function(n,i){var o=n.path,s=n.isRedirect,c=n.isInNodeModules;e.startsWith(o,r)&&((t||(t=[])).push({path:i,isRedirect:s,isInNodeModules:c}),a.delete(i))}),t&&(t.length>1&&t.sort(b),c.push.apply(c,t));var i=e.getDirectoryPath(n);if(i===n)return s=n,"break";n=i,s=n},u=e.getDirectoryPath(n);0!==a.size;){var d=l(u);if(u=s,"break"===d)break}if(a.size){var p=e.arrayFrom(a.values());p.length>1&&p.sort(b),c.push.apply(c,p)}return c}function L(n,t){var r,i=null===(r=n.declarations)||void 0===r?void 0:r.find(function(n){return e.isNonGlobalAmbientModule(n)&&(!e.isExternalModuleAugmentation(n)||!e.isExternalModuleNameRelative(e.getTextOfIdentifierOrLiteral(n.name)))});if(i)return i.name.text;var a=e.mapDefined(n.declarations,function(n){var r,i,a,o;if(e.isModuleDeclaration(n)){var s=d(n);if((null===(r=null===s||void 0===s?void 0:s.parent)||void 0===r?void 0:r.parent)&&e.isModuleBlock(s.parent)&&e.isAmbientModule(s.parent.parent)&&e.isSourceFile(s.parent.parent.parent)){var c=null===(o=null===(a=null===(i=s.parent.parent.symbol.exports)||void 0===i?void 0:i.get("export="))||void 0===a?void 0:a.valueDeclaration)||void 0===o?void 0:o.expression;if(c){var l=t.getSymbolAtLocation(c);if(l){var u=2097152&(null===l||void 0===l?void 0:l.flags)?t.getAliasedSymbol(l):l;if(u===n.symbol)return s.parent.parent}}}}function d(e){while(4&e.flags)e=e.parent;return e}}),o=a[0];return o?o.name.text:void 0}function D(n,t,r){for(var i in r)for(var a=0,o=r[i];a<o.length;a++){var s=o[a],c=e.removeFileExtension(e.normalizePath(s)),l=c.indexOf("*");if(-1!==l){var u=c.substr(0,l),d=c.substr(l+1);if(t.length>=u.length+d.length&&e.startsWith(t,u)&&e.endsWith(t,d)||!d&&t===e.removeTrailingDirectorySeparator(u)){var p=t.substr(u.length,t.length-d.length-u.length);return i.replace("*",p)}}else if(c===t||c===n)return i}}function A(n,t,r,i,a,o){var s=k(t,n,i);if(void 0!==s){var c=k(r,n,i),l=void 0!==c?e.ensurePathIsNonModuleName(e.getRelativePathFromDirectory(c,s,i)):s;return e.getEmitModuleResolutionKind(o)===e.ModuleResolutionKind.NodeJs?R(l,a,o):e.removeFileExtension(l)}}function C(n,t,r,i,a){var o=n.path,s=n.isRedirect,c=t.getCanonicalFileName,l=t.sourceDirectory;if(r.fileExists&&r.readFile){var u=N(o);if(u){var d=o,p=!1;if(!a){var f=u.packageRootIndex,m=void 0;while(1){var _=T(f),g=_.moduleFileToTry,y=_.packageRootPath;if(y){d=y,p=!0;break}if(m||(m=g),f=o.indexOf(e.directorySeparator,f+1),-1===f){d=S(m);break}}}if(!s||p){var h=r.getGlobalTypingsCacheLocation&&r.getGlobalTypingsCacheLocation(),v=c(d.substring(0,u.topLevelNodeModulesIndex));if(e.startsWith(l,v)||h&&e.startsWith(c(h),v)){var b=d.substring(u.topLevelPackageNameIndex+1),E=e.getPackageNameFromTypesPackageName(b);return e.getEmitModuleResolutionKind(i)!==e.ModuleResolutionKind.NodeJs&&E===b?void 0:E}}}}function T(n){var t=o.substring(0,n),a=e.combinePaths(t,"package.json"),s=o;if(r.fileExists(a)){var l=JSON.parse(r.readFile(a)),u=l.typesVersions?e.getPackageJsonTypesVersionsPaths(l.typesVersions):void 0;if(u){var d=o.slice(t.length+1),p=D(e.removeFileExtension(d),R(d,0,i),u.paths);void 0!==p&&(s=e.combinePaths(t,p))}var f=l.typings||l.types||l.main;if(e.isString(f)){var m=e.toPath(f,t,c);if(e.removeFileExtension(m)===e.removeFileExtension(c(s)))return{packageRootPath:t,moduleFileToTry:s}}}return{moduleFileToTry:s}}function S(n){var t=e.removeFileExtension(n);return"/index"!==c(t.substring(u.fileNameIndex))||I(r,t.substring(0,u.fileNameIndex))?t:t.substring(0,u.fileNameIndex)}}function I(n,t){if(n.fileExists)for(var r=e.getSupportedExtensions({allowJs:!0},[{extension:"node",isMixedContent:!1},{extension:"json",isMixedContent:!1,scriptKind:6}]),i=0,a=r;i<a.length;i++){var o=a[i],s=t+o;if(n.fileExists(s))return s}}function N(n){var t,r=0,i=0,a=0,o=0;(function(e){e[e["BeforeNodeModules"]=0]="BeforeNodeModules",e[e["NodeModules"]=1]="NodeModules",e[e["Scope"]=2]="Scope",e[e["PackageContent"]=3]="PackageContent"})(t||(t={}));var s=0,c=0,l=0;while(c>=0)switch(s=c,c=n.indexOf("/",s+1),l){case 0:n.indexOf(e.nodeModulesPathPart,s)===s&&(r=s,i=c,l=1);break;case 1:case 2:1===l&&"@"===n.charAt(s+1)?l=2:(a=c,l=3);break;case 3:l=n.indexOf(e.nodeModulesPathPart,s)===s?1:3;break}return o=s,l>1?{topLevelNodeModulesIndex:r,topLevelPackageNameIndex:i,packageRootIndex:a,fileNameIndex:o}:void 0}function k(n,t,r){return e.firstDefined(t,function(e){var t=P(n,e,r);return M(t)?void 0:t})}function R(n,t,r){if(e.fileExtensionIs(n,".json"))return n;var i=e.removeFileExtension(n);switch(t){case 0:return e.removeSuffix(i,"/index");case 1:return i;case 2:return i+O(n,r);default:return e.Debug.assertNever(t)}}function O(n,t){var r;return null!==(r=w(n,t))&&void 0!==r?r:e.Debug.fail("Extension "+e.extensionFromPath(n)+" is unsupported:: FileName:: "+n)}function w(n,t){var r=e.tryGetExtensionFromPath(n);switch(r){case".ts":case".d.ts":return".js";case".tsx":return 1===t.jsx?".jsx":".js";case".js":case".jsx":case".json":return r;default:return}}function P(n,t,r){var i=e.getRelativePathToDirectoryOrUrl(t,n,t,r,!1);return e.isRootedDiskPath(i)?void 0:i}function M(n){return e.startsWith(n,"..")}(function(e){e[e["Relative"]=0]="Relative",e[e["NonRelative"]=1]="NonRelative",e[e["Shortest"]=2]="Shortest",e[e["ExternalNonRelative"]=3]="ExternalNonRelative"})(t||(t={})),function(e){e[e["Minimal"]=0]="Minimal",e[e["Index"]=1]="Index",e[e["JsExtension"]=2]="JsExtension"}(r||(r={})),n.updateModuleSpecifier=s,n.getModuleSpecifier=c,n.getNodeModulesPackageName=l,n.tryGetModuleSpecifiersFromCache=d,n.getModuleSpecifiers=f,n.getModuleSpecifiersWithCacheInfo=m,n.countPathComponents=h,n.forEachFileNameOfModule=T,n.tryGetJSExtensionForFile=w})(e.moduleSpecifiers||(e.moduleSpecifiers={}))}(d||(d={})),function(e){var n=e.sys?{getCurrentDirectory:function(){return e.sys.getCurrentDirectory()},getNewLine:function(){return e.sys.newLine},getCanonicalFileName:e.createGetCanonicalFileName(e.sys.useCaseSensitiveFileNames)}:void 0;function t(t,r){var i=t===e.sys&&n?n:{getCurrentDirectory:function(){return t.getCurrentDirectory()},getNewLine:function(){return t.newLine},getCanonicalFileName:e.createGetCanonicalFileName(t.useCaseSensitiveFileNames)};if(!r)return function(n){return t.write(e.formatDiagnostic(n,i))};var a=new Array(1);return function(n){a[0]=n,t.write(e.formatDiagnosticsWithColorAndContext(a,i)+i.getNewLine()),a[0]=void 0}}function r(n,t,r){return!(!n.clearScreen||r.preserveWatchOutput||r.extendedDiagnostics||r.diagnostics||!e.contains(e.screenStartingMessageCodes,t.code))&&(n.clearScreen(),!0)}function i(n,t){return e.contains(e.screenStartingMessageCodes,n.code)?t+t:t}function o(e){return e.now?e.now().toLocaleTimeString("en-US",{timeZone:"UTC"}):(new Date).toLocaleTimeString()}function s(n,t){return t?function(t,i,a){r(n,t,a);var s="["+e.formatColorAndReset(o(n),e.ForegroundColorEscapeSequences.Grey)+"] ";s+=""+e.flattenDiagnosticMessageText(t.messageText,n.newLine)+(i+i),n.write(s)}:function(t,a,s){var c="";r(n,t,s)||(c+=a),c+=o(n)+" - ",c+=""+e.flattenDiagnosticMessageText(t.messageText,n.newLine)+i(t,a),n.write(c)}}function c(n,t,r,i,a,o){var s=a;s.onUnRecoverableConfigFileDiagnostic=function(e){return C(a,o,e)};var c=e.getParsedCommandLineOfConfigFile(n,t,s,r,i);return s.onUnRecoverableConfigFileDiagnostic=void 0,c}function l(n){return e.countWhere(n,function(n){return n.category===e.DiagnosticCategory.Error})}function u(n){return 1===n?e.Diagnostics.Found_1_error_Watching_for_file_changes:e.Diagnostics.Found_0_errors_Watching_for_file_changes}function d(n,t){if(0===n)return"";var r=e.createCompilerDiagnostic(1===n?e.Diagnostics.Found_1_error:e.Diagnostics.Found_0_errors,n);return""+t+e.flattenDiagnosticMessageText(r.messageText,t)+t+t}function p(e){return!!e.getState}function f(n,t){var r=n.getCompilerOptions();r.explainFiles?m(p(n)?n.getProgram():n,t):(r.listFiles||r.listFilesOnly)&&e.forEach(n.getSourceFiles(),function(e){t(e.fileName)})}function m(n,t){for(var r,i,a=n.getFileIncludeReasons(),o=e.createGetCanonicalFileName(n.useCaseSensitiveFileNames()),s=function(t){return e.convertToRelativePath(t,n.getCurrentDirectory(),o)},c=0,l=n.getSourceFiles();c<l.length;c++){var u=l[c];t(""+v(u,s)),null===(r=a.get(u.path))||void 0===r||r.forEach(function(e){return t("  "+h(n,e,s).messageText)}),null===(i=_(u,s))||void 0===i||i.forEach(function(e){return t("  "+e.messageText)})}}function _(n,t){var r;return n.path!==n.resolvedPath&&(r||(r=[])).push(e.chainDiagnosticMessages(void 0,e.Diagnostics.File_is_output_of_project_reference_source_0,v(n.originalFileName,t))),n.redirectInfo&&(r||(r=[])).push(e.chainDiagnosticMessages(void 0,e.Diagnostics.File_redirects_to_file_0,v(n.redirectInfo.redirectTarget,t))),r}function g(n,t){var r,i=n.getCompilerOptions().configFile;if(null===(r=null===i||void 0===i?void 0:i.configFileSpecs)||void 0===r?void 0:r.validatedFilesSpec){var a=e.createGetCanonicalFileName(n.useCaseSensitiveFileNames()),o=a(t),s=e.getDirectoryPath(e.getNormalizedAbsolutePath(i.fileName,n.getCurrentDirectory()));return e.find(i.configFileSpecs.validatedFilesSpec,function(n){return a(e.getNormalizedAbsolutePath(n,s))===o})}}function y(n,t){var r,i,a=n.getCompilerOptions().configFile;if(null===(r=null===a||void 0===a?void 0:a.configFileSpecs)||void 0===r?void 0:r.validatedIncludeSpecs){var o=e.fileExtensionIs(t,".json"),s=e.getDirectoryPath(e.getNormalizedAbsolutePath(a.fileName,n.getCurrentDirectory())),c=n.useCaseSensitiveFileNames();return e.find(null===(i=null===a||void 0===a?void 0:a.configFileSpecs)||void 0===i?void 0:i.validatedIncludeSpecs,function(n){if(o&&!e.endsWith(n,".json"))return!1;var r=e.getPatternFromSpec(n,s,"files");return!!r&&e.getRegexFromPattern("("+r+")$",c).test(t)})}}function h(n,t,r){var i,a,o=n.getCompilerOptions();if(e.isReferencedFile(t)){var s=e.getReferencedFileLocation(function(e){return n.getSourceFileByPath(e)},t),c=e.isReferenceFileLocation(s)?s.file.text.substring(s.pos,s.end):'"'+s.text+'"',l=void 0;switch(e.Debug.assert(e.isReferenceFileLocation(s)||t.kind===e.FileIncludeKind.Import,"Only synthetic references are imports"),t.kind){case e.FileIncludeKind.Import:l=e.isReferenceFileLocation(s)?s.packageId?e.Diagnostics.Imported_via_0_from_file_1_with_packageId_2:e.Diagnostics.Imported_via_0_from_file_1:s.text===e.externalHelpersModuleNameText?s.packageId?e.Diagnostics.Imported_via_0_from_file_1_with_packageId_2_to_import_importHelpers_as_specified_in_compilerOptions:e.Diagnostics.Imported_via_0_from_file_1_to_import_importHelpers_as_specified_in_compilerOptions:s.packageId?e.Diagnostics.Imported_via_0_from_file_1_with_packageId_2_to_import_jsx_and_jsxs_factory_functions:e.Diagnostics.Imported_via_0_from_file_1_to_import_jsx_and_jsxs_factory_functions;break;case e.FileIncludeKind.ReferenceFile:e.Debug.assert(!s.packageId),l=e.Diagnostics.Referenced_via_0_from_file_1;break;case e.FileIncludeKind.TypeReferenceDirective:l=s.packageId?e.Diagnostics.Type_library_referenced_via_0_from_file_1_with_packageId_2:e.Diagnostics.Type_library_referenced_via_0_from_file_1;break;case e.FileIncludeKind.LibReferenceDirective:e.Debug.assert(!s.packageId),l=e.Diagnostics.Library_referenced_via_0_from_file_1;break;default:e.Debug.assertNever(t)}return e.chainDiagnosticMessages(void 0,l,c,v(s.file,r),s.packageId&&e.packageIdToString(s.packageId))}switch(t.kind){case e.FileIncludeKind.RootFile:if(!(null===(i=o.configFile)||void 0===i?void 0:i.configFileSpecs))return e.chainDiagnosticMessages(void 0,e.Diagnostics.Root_file_specified_for_compilation);var u=e.getNormalizedAbsolutePath(n.getRootFileNames()[t.index],n.getCurrentDirectory()),d=g(n,u);if(d)return e.chainDiagnosticMessages(void 0,e.Diagnostics.Part_of_files_list_in_tsconfig_json);var p=y(n,u);return p?e.chainDiagnosticMessages(void 0,e.Diagnostics.Matched_by_include_pattern_0_in_1,p,v(o.configFile,r)):e.chainDiagnosticMessages(void 0,e.Diagnostics.Root_file_specified_for_compilation);case e.FileIncludeKind.SourceFromProjectReference:case e.FileIncludeKind.OutputFromProjectReference:var f=t.kind===e.FileIncludeKind.OutputFromProjectReference,m=e.Debug.checkDefined(null===(a=n.getResolvedProjectReferences())||void 0===a?void 0:a[t.index]);return e.chainDiagnosticMessages(void 0,e.outFile(o)?f?e.Diagnostics.Output_from_referenced_project_0_included_because_1_specified:e.Diagnostics.Source_from_referenced_project_0_included_because_1_specified:f?e.Diagnostics.Output_from_referenced_project_0_included_because_module_is_specified_as_none:e.Diagnostics.Source_from_referenced_project_0_included_because_module_is_specified_as_none,v(m.sourceFile.fileName,r),o.outFile?"--outFile":"--out");case e.FileIncludeKind.AutomaticTypeDirectiveFile:return e.chainDiagnosticMessages(void 0,o.types?t.packageId?e.Diagnostics.Entry_point_of_type_library_0_specified_in_compilerOptions_with_packageId_1:e.Diagnostics.Entry_point_of_type_library_0_specified_in_compilerOptions:t.packageId?e.Diagnostics.Entry_point_for_implicit_type_library_0_with_packageId_1:e.Diagnostics.Entry_point_for_implicit_type_library_0,t.typeReference,t.packageId&&e.packageIdToString(t.packageId));case e.FileIncludeKind.LibFile:if(void 0!==t.index)return e.chainDiagnosticMessages(void 0,e.Diagnostics.Library_0_specified_in_compilerOptions,o.lib[t.index]);var _=e.forEachEntry(e.targetOptionDeclaration.type,function(e,n){return e===o.target?n:void 0});return e.chainDiagnosticMessages(void 0,_?e.Diagnostics.Default_library_for_target_0:e.Diagnostics.Default_library,_);default:e.Debug.assertNever(t)}}function v(n,t){var r=e.isString(n)?n:n.fileName;return t?t(r):r}function b(n,t,r,i,a,o,s,c){var u=!!n.getCompilerOptions().listFilesOnly,d=n.getConfigFileParsingDiagnostics().slice(),p=d.length;e.addRange(d,n.getSyntacticDiagnostics(void 0,o)),d.length===p&&(e.addRange(d,n.getOptionsDiagnostics(o)),u||(e.addRange(d,n.getGlobalDiagnostics(o)),d.length===p&&e.addRange(d,n.getSemanticDiagnostics(void 0,o))));var m=u?{emitSkipped:!0,diagnostics:e.emptyArray}:n.emit(void 0,a,o,s,c),_=m.emittedFiles,g=m.diagnostics;e.addRange(d,g);var y=e.sortAndDeduplicateDiagnostics(d);if(y.forEach(t),r){var h=n.getCurrentDirectory();e.forEach(_,function(n){var t=e.getNormalizedAbsolutePath(n,h);r("TSFILE: "+t)}),f(n,r)}return i&&i(l(y)),{emitResult:m,diagnostics:y}}function E(n,t,r,i,a,o,s,c){var l=b(n,t,r,i,a,o,s,c),u=l.emitResult,d=l.diagnostics;return u.emitSkipped&&d.length>0?e.ExitStatus.DiagnosticsPresent_OutputsSkipped:d.length>0?e.ExitStatus.DiagnosticsPresent_OutputsGenerated:e.ExitStatus.Success}function T(n,t){void 0===n&&(n=e.sys);var r=t||s(n);return{onWatchStatusChange:r,watchFile:e.maybeBind(n,n.watchFile)||e.returnNoopFileWatcher,watchDirectory:e.maybeBind(n,n.watchDirectory)||e.returnNoopFileWatcher,setTimeout:e.maybeBind(n,n.setTimeout)||e.noop,clearTimeout:e.maybeBind(n,n.clearTimeout)||e.noop}}function S(n,t){var r=n.trace?t.extendedDiagnostics?e.WatchLogLevel.Verbose:t.diagnostics?e.WatchLogLevel.TriggerOnly:e.WatchLogLevel.None:e.WatchLogLevel.None,i=r!==e.WatchLogLevel.None?function(e){return n.trace(e)}:e.noop,a=e.getWatchFactory(n,r,i);return a.writeLog=i,a}function x(n,t,r){void 0===r&&(r=n);var i=n.useCaseSensitiveFileNames(),a=e.memoize(function(){return n.getNewLine()});return{getSourceFile:function(r,i,a){var o;try{e.performance.mark("beforeIORead"),o=n.readFile(r,t().charset),e.performance.mark("afterIORead"),e.performance.measure("I/O Read","beforeIORead","afterIORead")}catch(s){a&&a(s.message),o=""}return void 0!==o?e.createSourceFile(r,o,i):void 0},getDefaultLibLocation:e.maybeBind(n,n.getDefaultLibLocation),getDefaultLibFileName:function(e){return n.getDefaultLibFileName(e)},writeFile:o,getCurrentDirectory:e.memoize(function(){return n.getCurrentDirectory()}),useCaseSensitiveFileNames:function(){return i},getCanonicalFileName:e.createGetCanonicalFileName(i),getNewLine:function(){return e.getNewLineCharacter(t(),a)},fileExists:function(e){return n.fileExists(e)},readFile:function(e){return n.readFile(e)},trace:e.maybeBind(n,n.trace),directoryExists:e.maybeBind(r,r.directoryExists),getDirectories:e.maybeBind(r,r.getDirectories),realpath:e.maybeBind(n,n.realpath),getEnvironmentVariable:e.maybeBind(n,n.getEnvironmentVariable)||function(){return""},createHash:e.maybeBind(n,n.createHash),readDirectory:e.maybeBind(n,n.readDirectory),disableUseFileVersionAsSignature:n.disableUseFileVersionAsSignature};function o(t,r,i,a){try{e.performance.mark("beforeIOWrite"),e.writeFileEnsuringDirectories(t,r,i,function(e,t,r){return n.writeFile(e,t,r)},function(e){return n.createDirectory(e)},function(e){return n.directoryExists(e)}),e.performance.mark("afterIOWrite"),e.performance.measure("I/O Write","beforeIOWrite","afterIOWrite")}catch(o){a&&a(o.message)}}}function L(n,t){var r=n.getSourceFile,i=e.maybeBind(t,t.createHash)||e.generateDjb2Hash;n.getSourceFile=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];var o=r.call.apply(r,a([n],e,!1));return o&&(o.version=i(o.text)),o}}function D(n,t){var r=e.memoize(function(){return e.getDirectoryPath(e.normalizePath(n.getExecutingFilePath()))});return{useCaseSensitiveFileNames:function(){return n.useCaseSensitiveFileNames},getNewLine:function(){return n.newLine},getCurrentDirectory:e.memoize(function(){return n.getCurrentDirectory()}),getDefaultLibLocation:r,getDefaultLibFileName:function(n){return e.combinePaths(r(),e.getDefaultLibFileName(n))},fileExists:function(e){return n.fileExists(e)},readFile:function(e,t){return n.readFile(e,t)},directoryExists:function(e){return n.directoryExists(e)},getDirectories:function(e){return n.getDirectories(e)},readDirectory:function(e,t,r,i,a){return n.readDirectory(e,t,r,i,a)},realpath:e.maybeBind(n,n.realpath),getEnvironmentVariable:e.maybeBind(n,n.getEnvironmentVariable),trace:function(e){return n.write(e+n.newLine)},createDirectory:function(e){return n.createDirectory(e)},writeFile:function(e,t,r){return n.writeFile(e,t,r)},createHash:e.maybeBind(n,n.createHash),createProgram:t||e.createEmitAndSemanticDiagnosticsBuilderProgram,disableUseFileVersionAsSignature:n.disableUseFileVersionAsSignature}}function A(n,t,r,i){void 0===n&&(n=e.sys);var a=function(e){return n.write(e+n.newLine)},o=D(n,t);return e.copyProperties(o,T(n,i)),o.afterProgramCreate=function(t){var i=t.getCompilerOptions(),s=e.getNewLineCharacter(i,function(){return n.newLine});b(t,r,a,function(n){return o.onWatchStatusChange(e.createCompilerDiagnostic(u(n),n),s,i,n)})},o}function C(n,t,r){t(r),n.exit(e.ExitStatus.DiagnosticsPresent_OutputsSkipped)}function I(e){var n=e.configFileName,r=e.optionsToExtend,i=e.watchOptionsToExtend,a=e.extraFileExtensions,o=e.system,s=e.createProgram,c=e.reportDiagnostic,l=e.reportWatchStatus,u=c||t(o),d=A(o,s,u,l);return d.onUnRecoverableConfigFileDiagnostic=function(e){return C(o,u,e)},d.configFileName=n,d.optionsToExtend=r,d.watchOptionsToExtend=i,d.extraFileExtensions=a,d}function N(e){var n=e.rootFiles,r=e.options,i=e.watchOptions,a=e.projectReferences,o=e.system,s=e.createProgram,c=e.reportDiagnostic,l=e.reportWatchStatus,u=A(o,s,c||t(o),l);return u.rootFiles=n,u.options=r,u.watchOptions=i,u.projectReferences=a,u}function k(n){var r=n.system||e.sys,i=n.host||(n.host=e.createIncrementalCompilerHost(n.options,r)),a=e.createIncrementalProgram(n),o=E(a,n.reportDiagnostic||t(r),function(e){return i.trace&&i.trace(e)},n.reportErrorSummary||n.options.pretty?function(e){return r.write(d(e,r.newLine))}:void 0);return n.afterProgramEmitAndDiagnostics&&n.afterProgramEmitAndDiagnostics(a),o}e.createDiagnosticReporter=t,e.screenStartingMessageCodes=[e.Diagnostics.Starting_compilation_in_watch_mode.code,e.Diagnostics.File_change_detected_Starting_incremental_compilation.code],e.getLocaleTimeString=o,e.createWatchStatusReporter=s,e.parseConfigFileWithSystem=c,e.getErrorCountForSummary=l,e.getWatchErrorSummaryDiagnosticMessage=u,e.getErrorSummaryText=d,e.isBuilderProgram=p,e.listFiles=f,e.explainFiles=m,e.explainIfFileIsRedirect=_,e.getMatchedFileSpec=g,e.getMatchedIncludeSpec=y,e.fileIncludeReasonToDiagnostics=h,e.emitFilesAndReportErrors=b,e.emitFilesAndReportErrorsAndGetExitStatus=E,e.noopFileWatcher={close:e.noop},e.returnNoopFileWatcher=function(){return e.noopFileWatcher},e.createWatchHost=T,e.WatchType={ConfigFile:"Config file",ExtendedConfigFile:"Extended config file",SourceFile:"Source file",MissingFile:"Missing file",WildcardDirectory:"Wild card directory",FailedLookupLocations:"Failed Lookup Locations",TypeRoots:"Type roots",ConfigFileOfReferencedProject:"Config file of referened project",ExtendedConfigOfReferencedProject:"Extended config file of referenced project",WildcardDirectoryOfReferencedProject:"Wild card directory of referenced project",PackageJson:"package.json file"},e.createWatchFactory=S,e.createCompilerHostFromProgramHost=x,e.setGetSourceFileAsHashVersioned=L,e.createProgramHost=D,e.createWatchCompilerHostOfConfigFile=I,e.createWatchCompilerHostOfFilesAndCompilerOptions=N,e.performIncrementalCompilation=k}(d||(d={})),function(e){function n(n,t){if(!e.outFile(n)){var r=e.getTsBuildInfoEmitOutputFilePath(n);if(r){var i=t.readFile(r);if(i){var a=e.getBuildInfo(i);if(a.version===e.version&&a.program)return e.createBuildProgramUsingProgramBuildInfo(a.program,r,t)}}}}function t(n,t){void 0===t&&(t=e.sys);var r=e.createCompilerHostWorker(n,void 0,t);return r.createHash=e.maybeBind(t,t.createHash),r.disableUseFileVersionAsSignature=t.disableUseFileVersionAsSignature,e.setGetSourceFileAsHashVersioned(r,t),e.changeCompilerHostLikeToUseCache(r,function(n){return e.toPath(n,r.getCurrentDirectory(),r.getCanonicalFileName)}),r}function r(r){var i=r.rootNames,a=r.options,o=r.configFileParsingDiagnostics,s=r.projectReferences,c=r.host,l=r.createProgram;c=c||t(a),l=l||e.createEmitAndSemanticDiagnosticsBuilderProgram;var u=n(a,c);return l(i,a,c,u,o,s)}function i(n,t,r,i,a,o,s,c){return e.isArray(n)?e.createWatchCompilerHostOfFilesAndCompilerOptions({rootFiles:n,options:t,watchOptions:c,projectReferences:s,system:r,createProgram:i,reportDiagnostic:a,reportWatchStatus:o}):e.createWatchCompilerHostOfConfigFile({configFileName:n,optionsToExtend:t,watchOptionsToExtend:s,extraFileExtensions:c,system:r,createProgram:i,reportDiagnostic:a,reportWatchStatus:o})}function s(t){var r,i,s,c,l,u,d,p,f,m,_,g,y=t.extendedConfigCache,h=!1,v=new e.Map,b=!1,E=t.useCaseSensitiveFileNames(),T=t.getCurrentDirectory(),S=t.configFileName,x=t.optionsToExtend,L=void 0===x?{}:x,D=t.watchOptionsToExtend,A=t.extraFileExtensions,C=t.createProgram,I=t.rootFiles,N=t.options,k=t.watchOptions,R=t.projectReferences,O=!1,w=!1,P=void 0===S?void 0:e.createCachedDirectoryStructureHost(t,T,E),M=P||t,F=e.parseConfigHostFromCompilerHostLike(t,M),G=ne();S&&t.configFileParsingResult&&(Te(t.configFileParsingResult),G=ne()),ue(e.Diagnostics.Starting_compilation_in_watch_mode),S&&!t.configFileParsingResult&&(G=e.getNewLineCharacter(L,function(){return t.getNewLine()}),e.Debug.assert(!I),Ee(),G=ne());var B,U=e.createWatchFactory(t,N),V=U.watchFile,j=U.watchDirectory,K=U.writeLog,H=e.createGetCanonicalFileName(E);K("Current directory: "+T+" CaseSensitiveFileNames: "+E),S&&(B=V(S,ge,e.PollingInterval.High,k,e.WatchType.ConfigFile));var W=e.createCompilerHostFromProgramHost(t,function(){return N},M);e.setGetSourceFileAsHashVersioned(W,t);var z=W.getSourceFile;W.getSourceFile=function(e){for(var n=[],t=1;t<arguments.length;t++)n[t-1]=arguments[t];return oe.apply(void 0,a([e,te(e)],n,!1))},W.getSourceFileByPath=oe,W.getNewLine=function(){return G},W.fileExists=ae,W.onReleaseOldSourceFile=le,W.onReleaseParsedCommandLine=Le,W.toPath=te,W.getCompilationSettings=function(){return N},W.useSourceOfProjectReferenceRedirect=e.maybeBind(t,t.useSourceOfProjectReferenceRedirect),W.watchDirectoryOfFailedLookupLocation=function(n,t,r){return j(n,t,r,k,e.WatchType.FailedLookupLocations)},W.watchTypeRootsDirectory=function(n,t,r){return j(n,t,r,k,e.WatchType.TypeRoots)},W.getCachedDirectoryStructureHost=function(){return P},W.scheduleInvalidateResolutionsOfFailedLookupLocations=fe,W.onInvalidatedResolution=_e,W.onChangedAutomaticTypeDirectiveNames=_e,W.fileIsOpen=e.returnFalse,W.getCurrentProgram=Q,W.writeLog=K,W.getParsedCommandLine=Se;var J=e.createResolutionCache(W,S?e.getDirectoryPath(e.getNormalizedAbsolutePath(S,T)):T,!1);W.resolveModuleNames=t.resolveModuleNames?function(){for(var e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];return t.resolveModuleNames.apply(t,e)}:function(e,n,t,r){return J.resolveModuleNames(e,n,t,r)},W.resolveTypeReferenceDirectives=t.resolveTypeReferenceDirectives?function(){for(var e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];return t.resolveTypeReferenceDirectives.apply(t,e)}:function(e,n,t){return J.resolveTypeReferenceDirectives(e,n,t)};var q=!!t.resolveModuleNames||!!t.resolveTypeReferenceDirectives;return r=n(N,W),Z(),Oe(),S&&Pe(te(S),N,k,e.WatchType.ExtendedConfigFile),S?{getCurrentProgram:Y,getProgram:he,close:X}:{getCurrentProgram:Y,getProgram:he,updateRootFileNames:ee,close:X};function X(){pe(),J.clear(),e.clearMap(v,function(e){e&&e.fileWatcher&&(e.fileWatcher.close(),e.fileWatcher=void 0)}),B&&(B.close(),B=void 0),null===y||void 0===y||y.clear(),y=void 0,f&&(e.clearMap(f,e.closeFileWatcherOf),f=void 0),l&&(e.clearMap(l,e.closeFileWatcherOf),l=void 0),s&&(e.clearMap(s,e.closeFileWatcher),s=void 0),p&&(e.clearMap(p,function(n){var t;null===(t=n.watcher)||void 0===t||t.close(),n.watcher=void 0,n.watchedDirectories&&e.clearMap(n.watchedDirectories,e.closeFileWatcherOf),n.watchedDirectories=void 0}),p=void 0),c&&(e.clearMap(c,e.closeFileWatcher),c=void 0)}function Y(){return r}function Q(){return r&&r.getProgramOrUndefined()}function Z(){K("Synchronizing program"),pe();var n=Y();b&&(G=ne(),n&&(h||e.changesAffectModuleResolution(n.getCompilerOptions(),N))&&J.clear());var i=J.createHasInvalidatedResolution(q||h);return e.isProgramUptoDate(Q(),I,N,ce,ae,i,de,Se,R)?w&&(r=C(void 0,void 0,W,r,g,R),w=!1):$(i),h=!1,t.afterProgramCreate&&n!==r&&t.afterProgramCreate(r),r}function $(n){K("CreatingProgramWith::"),K("  roots: "+JSON.stringify(I)),K("  options: "+JSON.stringify(N)),R&&K("  projectReferences: "+JSON.stringify(R));var t=b||!Q();b=!1,w=!1,J.startCachingPerDirectoryResolution(),W.hasInvalidatedResolution=n,W.hasChangedAutomaticTypeDirectiveNames=de,r=C(I,N,W,r,g,R);var i=e.map(J.getModuleResolutionCache().getPackageJsonInfoCache().entries(),function(e){var n=e[0],t=e[1];return[W.realpath?te(W.realpath(n)):n,t]});if(J.finishCachingPerDirectoryResolution(),e.updateMissingFilePathsWatch(r.getProgram(),s||(s=new e.Map),Ie),e.updatePackageJsonWatch(i,c||(c=new e.Map),Ne),t&&J.updateTypeRootsWatch(),m){for(var a=0,o=m;a<o.length;a++){var l=o[a];s.has(l)||v.delete(l)}m=void 0}}function ee(n){e.Debug.assert(!S,"Cannot update root file names with config file watch mode"),I=n,_e()}function ne(){return e.getNewLineCharacter(N||L,function(){return t.getNewLine()})}function te(n){return e.toPath(n,T,H)}function re(e){return"boolean"===typeof e}function ie(e){return"boolean"===typeof e.version}function ae(e){var n=te(e);return!re(v.get(n))&&M.fileExists(e)}function oe(n,t,r,i,a){var o=v.get(t);if(!re(o)){if(void 0===o||a||ie(o)){var s=z(n,r,i);if(o)s?(o.sourceFile=s,o.version=s.version,o.fileWatcher||(o.fileWatcher=De(t,n,Ae,e.PollingInterval.Low,k,e.WatchType.SourceFile))):(o.fileWatcher&&o.fileWatcher.close(),v.set(t,!1));else if(s){var c=De(t,n,Ae,e.PollingInterval.Low,k,e.WatchType.SourceFile);v.set(t,{sourceFile:s,version:s.version,fileWatcher:c})}else v.set(t,!1);return s}return o.sourceFile}}function se(e){var n=v.get(e);void 0!==n&&(re(n)?v.set(e,{version:!1}):n.version=!1)}function ce(e){var n=v.get(e);return n&&n.version?n.version:void 0}function le(e,n,t){var r=v.get(e.resolvedPath);void 0!==r&&(re(r)?(m||(m=[])).push(e.path):r.sourceFile===e&&(r.fileWatcher&&r.fileWatcher.close(),v.delete(e.resolvedPath),t||J.removeResolutionsOfFile(e.path)))}function ue(n){t.onWatchStatusChange&&t.onWatchStatusChange(e.createCompilerDiagnostic(n),G,N||L)}function de(){return J.hasChangedAutomaticTypeDirectiveNames()}function pe(){return!!d&&(t.clearTimeout(d),d=void 0,!0)}function fe(){if(!t.setTimeout||!t.clearTimeout)return J.invalidateResolutionsOfFailedLookupLocations();var e=pe();K("Scheduling invalidateFailedLookup"+(e?", Cancelled earlier one":"")),d=t.setTimeout(me,250)}function me(){d=void 0,J.invalidateResolutionsOfFailedLookupLocations()&&_e()}function _e(){t.setTimeout&&t.clearTimeout&&(u&&t.clearTimeout(u),K("Scheduling update"),u=t.setTimeout(ye,250))}function ge(){e.Debug.assert(!!S),i=e.ConfigFileProgramReloadLevel.Full,_e()}function ye(){u=void 0,ue(e.Diagnostics.File_change_detected_Starting_incremental_compilation),he()}function he(){switch(i){case e.ConfigFileProgramReloadLevel.Partial:e.perfLogger.logStartUpdateProgram("PartialConfigReload"),ve();break;case e.ConfigFileProgramReloadLevel.Full:e.perfLogger.logStartUpdateProgram("FullConfigReload"),be();break;default:e.perfLogger.logStartUpdateProgram("SynchronizeProgram"),Z();break}return e.perfLogger.logStopUpdateProgram("Done"),Y()}function ve(){K("Reloading new file names and options"),I=e.getFileNamesFromConfigSpecs(N.configFile.configFileSpecs,e.getNormalizedAbsolutePath(e.getDirectoryPath(S),T),N,F,A),e.updateErrorForNoInputFiles(I,e.getNormalizedAbsolutePath(S,T),N.configFile.configFileSpecs,g,O)&&(w=!0),Z()}function be(){K("Reloading config file: "+S),i=e.ConfigFileProgramReloadLevel.None,P&&P.clearCache(),Ee(),b=!0,Z(),Oe(),Pe(te(S),N,k,e.WatchType.ExtendedConfigFile)}function Ee(){Te(e.getParsedCommandLineOfConfigFile(S,L,F,y||(y=new e.Map),D,A))}function Te(n){I=n.fileNames,N=n.options,k=n.watchOptions,R=n.projectReferences,_=n.wildcardDirectories,g=e.getConfigFileParsingDiagnostics(n).slice(),O=e.canJsonReportNoInputFiles(n.raw),w=!0}function Se(n){var r=te(n),i=null===p||void 0===p?void 0:p.get(r);if(i){if(!i.reloadLevel)return i.parsedCommandLine;if(i.parsedCommandLine&&i.reloadLevel===e.ConfigFileProgramReloadLevel.Partial&&!t.getParsedCommandLine){K("Reloading new file names and options");var a=e.getFileNamesFromConfigSpecs(i.parsedCommandLine.options.configFile.configFileSpecs,e.getNormalizedAbsolutePath(e.getDirectoryPath(n),T),N,F);return i.parsedCommandLine=o(o({},i.parsedCommandLine),{fileNames:a}),i.reloadLevel=void 0,i.parsedCommandLine}}K("Loading config file: "+n);var s=t.getParsedCommandLine?t.getParsedCommandLine(n):xe(n);return i?(i.parsedCommandLine=s,i.reloadLevel=void 0):(p||(p=new e.Map)).set(r,i={parsedCommandLine:s}),Me(n,r,i),s}function xe(n){var t=F.onUnRecoverableConfigFileDiagnostic;F.onUnRecoverableConfigFileDiagnostic=e.noop;var r=e.getParsedCommandLineOfConfigFile(n,void 0,F,y||(y=new e.Map),D);return F.onUnRecoverableConfigFileDiagnostic=t,r}function Le(n){var t,r=te(n),i=null===p||void 0===p?void 0:p.get(r);i&&(p.delete(r),i.watchedDirectories&&e.clearMap(i.watchedDirectories,e.closeFileWatcherOf),null===(t=i.watcher)||void 0===t||t.close(),e.clearSharedExtendedConfigFileWatcher(r,f))}function De(e,n,t,r,i,a){return V(n,function(n,r){return t(n,r,e)},r,i,a)}function Ae(n,t,r){Ce(n,r,t),t===e.FileWatcherEventKind.Deleted&&v.has(r)&&J.invalidateResolutionOfFile(r),se(r),_e()}function Ce(e,n,t){P&&P.addOrDeleteFile(e,n,t)}function Ie(n){return(null===p||void 0===p?void 0:p.has(n))?e.noopFileWatcher:De(n,n,Re,e.PollingInterval.Medium,k,e.WatchType.MissingFile)}function Ne(n){return v.has(n)?e.noopFileWatcher:De(n,n,ke,e.PollingInterval.High,k,e.WatchType.PackageJson)}function ke(n,t,r){Ce(n,r,t),i=e.ConfigFileProgramReloadLevel.Full,h=!0,_e()}function Re(n,t,r){Ce(n,r,t),t===e.FileWatcherEventKind.Created&&s.has(r)&&(s.get(r).close(),s.delete(r),se(r),_e())}function Oe(){_?e.updateWatchingWildcardDirectories(l||(l=new e.Map),new e.Map(e.getEntries(_)),we):l&&e.clearMap(l,e.closeFileWatcherOf)}function we(n,t){return j(n,function(t){e.Debug.assert(!!S);var r=te(t);P&&P.addOrDeleteFileOrDirectory(t,r),se(r),e.isIgnoredFileFromWildCardWatching({watchedDirPath:te(n),fileOrDirectory:t,fileOrDirectoryPath:r,configFileName:S,extraFileExtensions:A,options:N,program:Y()||I,currentDirectory:T,useCaseSensitiveFileNames:E,writeLog:K,toPath:te})||i!==e.ConfigFileProgramReloadLevel.Full&&(i=e.ConfigFileProgramReloadLevel.Partial,_e())},t,k,e.WatchType.WildcardDirectory)}function Pe(n,t,r,a){e.updateSharedExtendedConfigFileWatcher(n,t,f||(f=new e.Map),function(n,t){return V(n,function(r,a){var o;Ce(n,t,a),y&&e.cleanExtendedConfigCache(y,t,te);var s=null===(o=f.get(t))||void 0===o?void 0:o.projects;(null===s||void 0===s?void 0:s.size)&&s.forEach(function(n){if(te(S)===n)i=e.ConfigFileProgramReloadLevel.Full;else{var t=null===p||void 0===p?void 0:p.get(n);t&&(t.reloadLevel=e.ConfigFileProgramReloadLevel.Full),J.removeResolutionsFromProjectReferenceRedirects(n)}_e()})},e.PollingInterval.High,r,a)},te)}function Me(n,t,r){var i,a,o,s,c;r.watcher||(r.watcher=V(n,function(r,i){Ce(n,t,i);var a=null===p||void 0===p?void 0:p.get(t);a&&(a.reloadLevel=e.ConfigFileProgramReloadLevel.Full),J.removeResolutionsFromProjectReferenceRedirects(t),_e()},e.PollingInterval.High,(null===(i=r.parsedCommandLine)||void 0===i?void 0:i.watchOptions)||k,e.WatchType.ConfigFileOfReferencedProject)),(null===(a=r.parsedCommandLine)||void 0===a?void 0:a.wildcardDirectories)?e.updateWatchingWildcardDirectories(r.watchedDirectories||(r.watchedDirectories=new e.Map),new e.Map(e.getEntries(null===(o=r.parsedCommandLine)||void 0===o?void 0:o.wildcardDirectories)),function(i,a){var o;return j(i,function(r){var a=te(r);P&&P.addOrDeleteFileOrDirectory(r,a),se(a);var o=null===p||void 0===p?void 0:p.get(t);(null===o||void 0===o?void 0:o.parsedCommandLine)&&(e.isIgnoredFileFromWildCardWatching({watchedDirPath:te(i),fileOrDirectory:r,fileOrDirectoryPath:a,configFileName:n,options:o.parsedCommandLine.options,program:o.parsedCommandLine.fileNames,currentDirectory:T,useCaseSensitiveFileNames:E,writeLog:K,toPath:te})||o.reloadLevel!==e.ConfigFileProgramReloadLevel.Full&&(o.reloadLevel=e.ConfigFileProgramReloadLevel.Partial,_e()))},a,(null===(o=r.parsedCommandLine)||void 0===o?void 0:o.watchOptions)||k,e.WatchType.WildcardDirectoryOfReferencedProject)}):r.watchedDirectories&&(e.clearMap(r.watchedDirectories,e.closeFileWatcherOf),r.watchedDirectories=void 0),Pe(t,null===(s=r.parsedCommandLine)||void 0===s?void 0:s.options,(null===(c=r.parsedCommandLine)||void 0===c?void 0:c.watchOptions)||k,e.WatchType.ExtendedConfigOfReferencedProject)}}e.readBuilderProgram=n,e.createIncrementalCompilerHost=t,e.createIncrementalProgram=r,e.createWatchCompilerHost=i,e.createWatchProgram=s}(d||(d={})),function(e){function n(n){return e.fileExtensionIs(n,".json")?n:e.combinePaths(n,"tsconfig.json")}(function(e){e[e["Unbuildable"]=0]="Unbuildable",e[e["UpToDate"]=1]="UpToDate",e[e["UpToDateWithUpstreamTypes"]=2]="UpToDateWithUpstreamTypes",e[e["OutOfDateWithPrepend"]=3]="OutOfDateWithPrepend",e[e["OutputMissing"]=4]="OutputMissing",e[e["OutOfDateWithSelf"]=5]="OutOfDateWithSelf",e[e["OutOfDateWithUpstream"]=6]="OutOfDateWithUpstream",e[e["UpstreamOutOfDate"]=7]="UpstreamOutOfDate",e[e["UpstreamBlocked"]=8]="UpstreamBlocked",e[e["ComputingUpstream"]=9]="ComputingUpstream",e[e["TsVersionOutputOfDate"]=10]="TsVersionOutputOfDate",e[e["ContainerOnly"]=11]="ContainerOnly"})(e.UpToDateStatusType||(e.UpToDateStatusType={})),e.resolveConfigFileProjectName=n}(d||(d={})),function(e){var n,t,r,i=new Date(-864e13),o=new Date(864e13);function s(e,n,t){var r,i=e.get(n);return i||(r=t(),e.set(n,r)),i||r}function c(n,t){return s(n,t,function(){return new e.Map})}function l(e,n){return n>e?n:e}function u(n){return e.fileExtensionIs(n,".d.ts")}function d(e){return!!e&&!!e.buildOrder}function p(e){return d(e)?e.buildOrder:e}function f(n,t){return function(r){var i=t?"["+e.formatColorAndReset(e.getLocaleTimeString(n),e.ForegroundColorEscapeSequences.Grey)+"] ":e.getLocaleTimeString(n)+" - ";i+=""+e.flattenDiagnosticMessageText(r.messageText,n.newLine)+(n.newLine+n.newLine),n.write(i)}}function m(n,t,r,i){var a=e.createProgramHost(n,t);return a.getModifiedTime=n.getModifiedTime?function(e){return n.getModifiedTime(e)}:e.returnUndefined,a.setModifiedTime=n.setModifiedTime?function(e,t){return n.setModifiedTime(e,t)}:e.noop,a.deleteFile=n.deleteFile?function(e){return n.deleteFile(e)}:e.noop,a.reportDiagnostic=r||e.createDiagnosticReporter(n),a.reportSolutionBuilderStatus=i||f(n),a.now=e.maybeBind(n,n.now),a}function _(n,t,r,i,a){void 0===n&&(n=e.sys);var o=m(n,t,r,i);return o.reportErrorSummary=a,o}function g(n,t,r,i,a){void 0===n&&(n=e.sys);var o=m(n,t,r,i),s=e.createWatchHost(n,a);return e.copyProperties(o,s),o}function y(n){var t={};return e.commonOptionsWithBuild.forEach(function(r){e.hasProperty(n,r.name)&&(t[r.name]=n[r.name])}),t}function h(e,n,t){return ue(!1,e,n,t)}function v(e,n,t,r){return ue(!0,e,n,t,r)}function b(n,t,r,i,a){var o=t,s=t,c=o.getCurrentDirectory(),l=e.createGetCanonicalFileName(o.useCaseSensitiveFileNames()),u=y(i),d=e.createCompilerHostFromProgramHost(o,function(){return E.projectCompilerOptions});e.setGetSourceFileAsHashVersioned(d,o),d.getParsedCommandLine=function(e){return L(E,e,T(E,e))},d.resolveModuleNames=e.maybeBind(o,o.resolveModuleNames),d.resolveTypeReferenceDirectives=e.maybeBind(o,o.resolveTypeReferenceDirectives);var p=d.resolveModuleNames?void 0:e.createModuleResolutionCache(c,l),f=d.resolveTypeReferenceDirectives?void 0:e.createTypeReferenceDirectiveResolutionCache(c,l,void 0,null===p||void 0===p?void 0:p.getPackageJsonInfoCache());if(!d.resolveModuleNames){var m=function(n,t,r){return e.resolveModuleName(n,t,E.projectCompilerOptions,d,p,r).resolvedModule};d.resolveModuleNames=function(n,t,r,i){return e.loadWithLocalCache(e.Debug.checkEachDefined(n),t,i,m)}}if(!d.resolveTypeReferenceDirectives){var _=function(n,t,r){return e.resolveTypeReferenceDirective(n,t,E.projectCompilerOptions,d,r,E.typeReferenceDirectiveResolutionCache).resolvedTypeReferenceDirective};d.resolveTypeReferenceDirectives=function(n,t,r){return e.loadWithLocalCache(e.Debug.checkEachDefined(n),t,r,_)}}var g=e.createWatchFactory(s,i),h=g.watchFile,v=g.watchDirectory,b=g.writeLog,E={host:o,hostWithWatch:s,currentDirectory:c,getCanonicalFileName:l,parseConfigFileHost:e.parseConfigHostFromCompilerHostLike(o),write:e.maybeBind(o,o.trace),options:i,baseCompilerOptions:u,rootNames:r,baseWatchOptions:a,resolvedConfigFilePaths:new e.Map,configFileCache:new e.Map,projectStatus:new e.Map,buildInfoChecked:new e.Map,extendedConfigCache:new e.Map,builderPrograms:new e.Map,diagnostics:new e.Map,projectPendingBuild:new e.Map,projectErrorsReported:new e.Map,compilerHost:d,moduleResolutionCache:p,typeReferenceDirectiveResolutionCache:f,buildOrder:void 0,readFileWithCache:function(e){return o.readFile(e)},projectCompilerOptions:u,cache:void 0,allProjectBuildPending:!0,needsSummary:!0,watchAllProjectsPending:n,currentInvalidatedProject:void 0,watch:n,allWatchedWildcardDirectories:new e.Map,allWatchedInputFiles:new e.Map,allWatchedConfigFiles:new e.Map,allWatchedExtendedConfigFiles:new e.Map,allWatchedPackageJsonFiles:new e.Map,lastCachedPackageJsonLookups:new e.Map,timerToBuildInvalidatedProject:void 0,reportFileChangeDetected:!1,watchFile:h,watchDirectory:v,writeLog:b};return E}function E(n,t){return e.toPath(t,n.currentDirectory,n.getCanonicalFileName)}function T(e,n){var t=e.resolvedConfigFilePaths,r=t.get(n);if(void 0!==r)return r;var i=E(e,n);return t.set(n,i),i}function S(e){return!!e.options}function x(e,n){var t=e.configFileCache.get(n);return t&&S(t)?t:void 0}function L(n,t,r){var i,a=n.configFileCache,o=a.get(r);if(o)return S(o)?o:void 0;var s,c=n.parseConfigFileHost,l=n.baseCompilerOptions,u=n.baseWatchOptions,d=n.extendedConfigCache,p=n.host;return p.getParsedCommandLine?(s=p.getParsedCommandLine(t),s||(i=e.createCompilerDiagnostic(e.Diagnostics.File_0_not_found,t))):(c.onUnRecoverableConfigFileDiagnostic=function(e){return i=e},s=e.getParsedCommandLineOfConfigFile(t,l,c,d,u),c.onUnRecoverableConfigFileDiagnostic=e.noop),a.set(r,s||i),s}function D(n,t){return e.resolveConfigFileProjectName(e.resolvePath(n.currentDirectory,t))}function A(n,t){for(var r,i,a=new e.Map,o=new e.Map,s=[],c=0,l=t;c<l.length;c++){var u=l[c];d(u)}return i?{buildOrder:r||e.emptyArray,circularDiagnostics:i}:r||e.emptyArray;function d(t,c){var l=T(n,t);if(!o.has(l))if(a.has(l))c||(i||(i=[])).push(e.createCompilerDiagnostic(e.Diagnostics.Project_references_may_not_form_a_circular_graph_Cycle_detected_Colon_0,s.join("\r\n")));else{a.set(l,!0),s.push(t);var u=L(n,t,l);if(u&&u.projectReferences)for(var p=0,f=u.projectReferences;p<f.length;p++){var m=f[p],_=D(n,m.path);d(_,c||m.circular)}s.pop(),o.set(l,!0),(r||(r=[])).push(t)}}}function C(e){return e.buildOrder||I(e)}function I(n){var t=A(n,n.rootNames.map(function(e){return D(n,e)}));n.resolvedConfigFilePaths.clear();var r=new e.Map(p(t).map(function(e){return[T(n,e),!0]})),i={onDeleteValue:e.noop};return e.mutateMapSkippingNewValues(n.configFileCache,r,i),e.mutateMapSkippingNewValues(n.projectStatus,r,i),e.mutateMapSkippingNewValues(n.buildInfoChecked,r,i),e.mutateMapSkippingNewValues(n.builderPrograms,r,i),e.mutateMapSkippingNewValues(n.diagnostics,r,i),e.mutateMapSkippingNewValues(n.projectPendingBuild,r,i),e.mutateMapSkippingNewValues(n.projectErrorsReported,r,i),n.watch&&(e.mutateMapSkippingNewValues(n.allWatchedConfigFiles,r,{onDeleteValue:e.closeFileWatcher}),n.allWatchedExtendedConfigFiles.forEach(function(e){e.projects.forEach(function(n){r.has(n)||e.projects.delete(n)}),e.close()}),e.mutateMapSkippingNewValues(n.allWatchedWildcardDirectories,r,{onDeleteValue:function(n){return n.forEach(e.closeFileWatcherOf)}}),e.mutateMapSkippingNewValues(n.allWatchedInputFiles,r,{onDeleteValue:function(n){return n.forEach(e.closeFileWatcher)}}),e.mutateMapSkippingNewValues(n.allWatchedPackageJsonFiles,r,{onDeleteValue:function(n){return n.forEach(e.closeFileWatcher)}})),n.buildOrder=t}function N(n,t,r){var i=t&&D(n,t),a=C(n);if(d(a))return a;if(i){var o=T(n,i),s=e.findIndex(a,function(e){return T(n,e)===o});if(-1===s)return}var c=i?A(n,[i]):a;return e.Debug.assert(!d(c)),e.Debug.assert(!r||void 0!==i),e.Debug.assert(!r||c[c.length-1]===i),r?c.slice(0,c.length-1):c}function k(n){n.cache&&R(n);var t=n.compilerHost,r=n.host,i=n.readFileWithCache,o=t.getSourceFile,s=e.changeCompilerHostLikeToUseCache(r,function(e){return E(n,e)},function(){for(var e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];return o.call.apply(o,a([t],e,!1))}),c=s.originalReadFile,l=s.originalFileExists,u=s.originalDirectoryExists,d=s.originalCreateDirectory,p=s.originalWriteFile,f=s.getSourceFileWithCache,m=s.readFileWithCache;n.readFileWithCache=m,t.getSourceFile=f,n.cache={originalReadFile:c,originalFileExists:l,originalDirectoryExists:u,originalCreateDirectory:d,originalWriteFile:p,originalReadFileWithCache:i,originalGetSourceFile:o}}function R(e){if(e.cache){var n=e.cache,t=e.host,r=e.compilerHost,i=e.extendedConfigCache,a=e.moduleResolutionCache,o=e.typeReferenceDirectiveResolutionCache;t.readFile=n.originalReadFile,t.fileExists=n.originalFileExists,t.directoryExists=n.originalDirectoryExists,t.createDirectory=n.originalCreateDirectory,t.writeFile=n.originalWriteFile,r.getSourceFile=n.originalGetSourceFile,e.readFileWithCache=n.originalReadFileWithCache,i.clear(),null===a||void 0===a||a.clear(),null===o||void 0===o||o.clear(),e.cache=void 0}}function O(e,n){e.projectStatus.delete(n),e.diagnostics.delete(n)}function w(e,n,t){var r=e.projectPendingBuild,i=r.get(n);void 0===i?r.set(n,t):i<t&&r.set(n,t)}function P(n,t){if(n.allProjectBuildPending){n.allProjectBuildPending=!1,n.options.watch&&fe(n,e.Diagnostics.Starting_compilation_in_watch_mode),k(n);var r=p(C(n));r.forEach(function(t){return n.projectPendingBuild.set(T(n,t),e.ConfigFileProgramReloadLevel.None)}),t&&t.throwIfCancellationRequested()}}function M(n,t){return n.projectPendingBuild.delete(t),n.currentInvalidatedProject=void 0,n.diagnostics.has(t)?e.ExitStatus.DiagnosticsPresent_OutputsSkipped:e.ExitStatus.Success}function F(e,n,r,i,a){var o=!0;return{kind:t.UpdateOutputFileStamps,project:n,projectPath:r,buildOrder:a,getCompilerOptions:function(){return i.options},getCurrentDirectory:function(){return e.currentDirectory},updateOutputFileStatmps:function(){X(e,i,r),o=!1},done:function(){return o&&X(e,i,r),M(e,r)}}}function G(s,c,d,p,f,m,_){var g,y,h,v=s===t.Build?r.CreateProgram:r.EmitBundle;return s===t.Build?{kind:s,project:d,projectPath:p,buildOrder:_,getCompilerOptions:function(){return m.options},getCurrentDirectory:function(){return c.currentDirectory},getBuilderProgram:function(){return S(e.identity)},getProgram:function(){return S(function(e){return e.getProgramOrUndefined()})},getSourceFile:function(e){return S(function(n){return n.getSourceFile(e)})},getSourceFiles:function(){return x(function(e){return e.getSourceFiles()})},getOptionsDiagnostics:function(e){return x(function(n){return n.getOptionsDiagnostics(e)})},getGlobalDiagnostics:function(e){return x(function(n){return n.getGlobalDiagnostics(e)})},getConfigFileParsingDiagnostics:function(){return x(function(e){return e.getConfigFileParsingDiagnostics()})},getSyntacticDiagnostics:function(e,n){return x(function(t){return t.getSyntacticDiagnostics(e,n)})},getAllDependencies:function(e){return x(function(n){return n.getAllDependencies(e)})},getSemanticDiagnostics:function(e,n){return x(function(t){return t.getSemanticDiagnostics(e,n)})},getSemanticDiagnosticsOfNextAffectedFile:function(e,n){return S(function(t){return t.getSemanticDiagnosticsOfNextAffectedFile&&t.getSemanticDiagnosticsOfNextAffectedFile(e,n)})},emit:function(e,n,t,i,a){return e||i?S(function(r){var o,s;return r.emit(e,n,t,i,a||(null===(s=(o=c.host).getCustomTransformers)||void 0===s?void 0:s.call(o,d)))}):(P(r.SemanticDiagnostics,t),v===r.EmitBuildInfo?R(n,t):v===r.Emit?k(n,t,a):void 0)},done:b}:{kind:s,project:d,projectPath:p,buildOrder:_,getCompilerOptions:function(){return m.options},getCurrentDirectory:function(){return c.currentDirectory},emit:function(e,n){return v!==r.EmitBundle?h:w(e,n)},done:b};function b(e,n,t){return P(r.Done,e,n,t),M(c,p)}function S(e){return P(r.CreateProgram),g&&e(g)}function x(n){return S(n)||e.emptyArray}function A(){var t,i;if(e.Debug.assert(void 0===g),c.options.dry)return pe(c,e.Diagnostics.A_non_dry_build_would_build_project_0,d),y=n.Success,void(v=r.QueueReferencingProjects);if(c.options.verbose&&pe(c,e.Diagnostics.Building_project_0,d),0===m.fileNames.length)return _e(c,p,e.getConfigFileParsingDiagnostics(m)),y=n.None,void(v=r.QueueReferencingProjects);var a=c.host,o=c.compilerHost;c.projectCompilerOptions=m.options,null===(t=c.moduleResolutionCache)||void 0===t||t.update(m.options),null===(i=c.typeReferenceDirectiveResolutionCache)||void 0===i||i.update(m.options),g=a.createProgram(m.fileNames,m.options,o,j(c,p,m),e.getConfigFileParsingDiagnostics(m),m.projectReferences),c.watch&&(c.lastCachedPackageJsonLookups.set(p,c.moduleResolutionCache&&e.map(c.moduleResolutionCache.getPackageJsonInfoCache().entries(),function(e){var n=e[0],t=e[1];return[c.host.realpath&&t?E(c,c.host.realpath(n)):n,t]})),c.builderPrograms.set(p,g)),v++}function C(e,n,t){var r;e.length?(r=H(c,p,g,m,e,n,t),y=r.buildResult,v=r.step):v++}function I(t){e.Debug.assertIsDefined(g),C(a(a(a(a([],g.getConfigFileParsingDiagnostics(),!0),g.getOptionsDiagnostics(t),!0),g.getGlobalDiagnostics(t),!0),g.getSyntacticDiagnostics(void 0,t),!0),n.SyntaxErrors,"Syntactic")}function N(t){C(e.Debug.checkDefined(g).getSemanticDiagnostics(void 0,t),n.TypeErrors,"Semantic")}function k(t,a,o){var s,f,_,h;e.Debug.assertIsDefined(g),e.Debug.assert(v===r.Emit),g.backupState();var b=function(e){return(h||(h=[])).push(e)},T=[],S=e.emitFilesAndReportErrors(g,b,void 0,void 0,function(e,n,t){return T.push({name:e,text:n,writeByteOrderMark:t})},a,!1,o||(null===(_=(f=c.host).getCustomTransformers)||void 0===_?void 0:_.call(f,d))).emitResult;if(h)return g.restoreState(),s=H(c,p,g,m,h,n.DeclarationEmitErrors,"Declaration file"),y=s.buildResult,v=s.step,{emitSkipped:!0,diagnostics:S.diagnostics};var x=c.host,L=c.compilerHost,D=n.DeclarationOutputUnchanged,A=i,C=!1,I=e.createDiagnosticCollection(),N=new e.Map;return T.forEach(function(r){var i,a=r.name,o=r.text,s=r.writeByteOrderMark;!C&&u(a)&&(x.fileExists(a)&&c.readFileWithCache(a)===o?i=x.getModifiedTime(a):(D&=~n.DeclarationOutputUnchanged,C=!0)),N.set(E(c,a),a),e.writeFile(t?{writeFile:t}:L,I,a,o,s),void 0!==i&&(A=l(i,A))}),O(I,N,A,C,T.length?T[0].name:e.getFirstProjectOutput(m,!x.useCaseSensitiveFileNames()),D),S}function R(t,i){e.Debug.assertIsDefined(g),e.Debug.assert(v===r.EmitBuildInfo);var o=g.emitBuildInfo(t,i);return o.diagnostics.length&&(me(c,o.diagnostics),c.diagnostics.set(p,a(a([],c.diagnostics.get(p),!0),o.diagnostics,!0)),y=n.EmitErrors&y),o.emittedFiles&&c.write&&o.emittedFiles.forEach(function(e){return V(c,m,e)}),K(c,g,m),v=r.QueueReferencingProjects,o}function O(t,i,a,s,l,u){var d,f=t.getDiagnostics();if(f.length)return d=H(c,p,g,m,f,n.EmitErrors,"Emit"),y=d.buildResult,v=d.step,f;c.write&&i.forEach(function(e){return V(c,m,e)});var _=q(c,m,a,e.Diagnostics.Updating_unchanged_output_timestamps_of_project_0,i);return c.diagnostics.delete(p),c.projectStatus.set(p,{type:e.UpToDateStatusType.UpToDate,newestDeclarationFileContentChangedTime:s?o:_,oldestOutputFileName:l}),K(c,g,m),v=r.QueueReferencingProjects,y=u,f}function w(a,o){var l,u;if(e.Debug.assert(s===t.UpdateBundle),c.options.dry)return pe(c,e.Diagnostics.A_non_dry_build_would_update_output_of_project_0,d),y=n.Success,void(v=r.QueueReferencingProjects);c.options.verbose&&pe(c,e.Diagnostics.Updating_output_of_project_0,d);var g=c.compilerHost;c.projectCompilerOptions=m.options;var b=e.emitUsingBuildInfo(m,g,function(e){var n=D(c,e.path);return L(c,n,T(c,n))},o||(null===(u=(l=c.host).getCustomTransformers)||void 0===u?void 0:u.call(l,d)));if(e.isString(b))return pe(c,e.Diagnostics.Cannot_update_output_of_project_0_because_there_was_error_reading_file_1,d,de(c,b)),v=r.BuildInvalidatedProjectOfBundle,h=G(t.Build,c,d,p,f,m,_);e.Debug.assert(!!b.length);var S=e.createDiagnosticCollection(),x=new e.Map;b.forEach(function(n){var t=n.name,r=n.text,i=n.writeByteOrderMark;x.set(E(c,t),t),e.writeFile(a?{writeFile:a}:g,S,t,r,i)});var A=O(S,x,i,!1,b[0].name,n.DeclarationOutputUnchanged);return{emitSkipped:!1,diagnostics:A}}function P(n,t,i,a){while(v<=n&&v<r.Done){var o=v;switch(v){case r.CreateProgram:A();break;case r.SyntaxDiagnostics:I(t);break;case r.SemanticDiagnostics:N(t);break;case r.Emit:k(i,t,a);break;case r.EmitBuildInfo:R(i,t);break;case r.EmitBundle:w(i,a);break;case r.BuildInvalidatedProjectOfBundle:e.Debug.checkDefined(h).done(t,i,a),v=r.Done;break;case r.QueueReferencingProjects:Y(c,d,p,f,m,_,e.Debug.checkDefined(y)),v++;break;case r.Done:default:e.assertType(v)}e.Debug.assert(v>o)}}}function B(n,t,r){var i=n.options;return!(t.type===e.UpToDateStatusType.OutOfDateWithPrepend&&!i.force)||(0===r.fileNames.length||!!e.getConfigFileParsingDiagnostics(r).length||!e.isIncrementalCompilation(r.options))}function U(n,r,i){if(n.projectPendingBuild.size&&!d(r)){if(n.currentInvalidatedProject)return e.arrayIsEqualTo(n.currentInvalidatedProject.buildOrder,r)?n.currentInvalidatedProject:void 0;for(var a=n.options,o=n.projectPendingBuild,s=0;s<r.length;s++){var c=r[s],l=T(n,c),u=n.projectPendingBuild.get(l);if(void 0!==u){i&&(i=!1,he(n,r));var p=L(n,c,l);if(p){u===e.ConfigFileProgramReloadLevel.Full?(re(n,c,l,p),ie(n,l,p),ae(n,c,l,p),oe(n,c,l,p),se(n,c,l,p)):u===e.ConfigFileProgramReloadLevel.Partial&&(p.fileNames=e.getFileNamesFromConfigSpecs(p.options.configFile.configFileSpecs,e.getDirectoryPath(c),p.options,n.parseConfigFileHost),e.updateErrorForNoInputFiles(p.fileNames,c,p.options.configFile.configFileSpecs,p.errors,e.canJsonReportNoInputFiles(p.raw)),oe(n,c,l,p),se(n,c,l,p));var f=J(n,p,l);if(be(n,c,f),!a.force){if(f.type===e.UpToDateStatusType.UpToDate){_e(n,l,e.getConfigFileParsingDiagnostics(p)),o.delete(l),a.dry&&pe(n,e.Diagnostics.Project_0_is_up_to_date,c);continue}if(f.type===e.UpToDateStatusType.UpToDateWithUpstreamTypes)return _e(n,l,e.getConfigFileParsingDiagnostics(p)),F(n,c,l,p,r)}if(f.type!==e.UpToDateStatusType.UpstreamBlocked){if(f.type!==e.UpToDateStatusType.ContainerOnly)return G(B(n,f,p)?t.Build:t.UpdateBundle,n,c,l,s,p,r);_e(n,l,e.getConfigFileParsingDiagnostics(p)),o.delete(l)}else _e(n,l,e.getConfigFileParsingDiagnostics(p)),o.delete(l),a.verbose&&pe(n,f.upstreamProjectBlocked?e.Diagnostics.Skipping_build_of_project_0_because_its_dependency_1_was_not_built:e.Diagnostics.Skipping_build_of_project_0_because_its_dependency_1_has_errors,c,f.upstreamProjectName)}else ge(n,l),o.delete(l)}}}}function V(e,n,t){var r=e.write;r&&n.options.listEmittedFiles&&r("TSFILE: "+t)}function j(n,t,r){var i=n.options,a=n.builderPrograms,o=n.compilerHost;if(!i.force){var s=a.get(t);return s||e.readBuilderProgram(r.options,o)}}function K(n,t,r){t?(t&&n.write&&e.listFiles(t,n.write),n.host.afterProgramEmitAndDiagnostics&&n.host.afterProgramEmitAndDiagnostics(t),t.releaseProgram()):n.host.afterEmitBundle&&n.host.afterEmitBundle(r),n.projectCompilerOptions=n.baseCompilerOptions}function H(t,i,a,o,s,c,l){var u=!(c&n.SyntaxErrors)&&a&&!e.outFile(a.getCompilerOptions());return _e(t,i,s),t.projectStatus.set(i,{type:e.UpToDateStatusType.Unbuildable,reason:l+" errors"}),u?{buildResult:c,step:r.EmitBuildInfo}:(K(t,a,o),{buildResult:c,step:r.QueueReferencingProjects})}function W(n,t,r,i){var a=e.getModifiedTime(n.host,t);if(r<a)return{type:e.UpToDateStatusType.OutOfDateWithSelf,outOfDateOutputFileName:i,newerInputFileName:t}}function z(n,t,r){for(var a=!!n.options.force,s=void 0,c=i,d=n.host,p=0,f=t.fileNames;p<f.length;p++){var m=f[p];if(!d.fileExists(m))return{type:e.UpToDateStatusType.Unbuildable,reason:m+" does not exist"};if(!a){var _=e.getModifiedTime(d,m);d.getModifiedTime(m),_>c&&(s=m,c=_)}}if(!t.fileNames.length&&!e.canJsonReportNoInputFiles(t.raw))return{type:e.UpToDateStatusType.ContainerOnly};var g,y=e.getAllProjectOutputs(t,!d.useCaseSensitiveFileNames()),h="(none)",v=o,b="(none)",E=i,S=i,x=!1;if(!a)for(var D=0,A=y;D<A.length;D++){var C=A[D];if(!d.fileExists(C)){g=C;break}var I=e.getModifiedTime(d,C);if(I<v&&(v=I,h=C),I<c){x=!0;break}if(I>E&&(E=I,b=C),u(C)){var N=e.getModifiedTime(d,C);S=l(S,N)}}var k,R=!1,O=!1;if(t.projectReferences){n.projectStatus.set(r,{type:e.UpToDateStatusType.ComputingUpstream});for(var w=0,P=t.projectReferences;w<P.length;w++){var M=P[w];O=O||!!M.prepend;var F=e.resolveProjectReferencePath(M),G=T(n,F),B=J(n,L(n,F,G),G);if(B.type!==e.UpToDateStatusType.ComputingUpstream&&B.type!==e.UpToDateStatusType.ContainerOnly){if(B.type===e.UpToDateStatusType.Unbuildable||B.type===e.UpToDateStatusType.UpstreamBlocked)return{type:e.UpToDateStatusType.UpstreamBlocked,upstreamProjectName:M.path,upstreamProjectBlocked:B.type===e.UpToDateStatusType.UpstreamBlocked};if(B.type!==e.UpToDateStatusType.UpToDate)return{type:e.UpToDateStatusType.UpstreamOutOfDate,upstreamProjectName:M.path};if(!a&&!g){if(B.newestInputFileTime&&B.newestInputFileTime<=v)continue;if(B.newestDeclarationFileContentChangedTime&&B.newestDeclarationFileContentChangedTime<=v){R=!0,k=M.path;continue}return e.Debug.assert(void 0!==h,"Should have an oldest output filename here"),{type:e.UpToDateStatusType.OutOfDateWithUpstream,outOfDateOutputFileName:h,newerProjectName:M.path}}}}}if(void 0!==g)return{type:e.UpToDateStatusType.OutputMissing,missingOutputFileName:g};if(x)return{type:e.UpToDateStatusType.OutOfDateWithSelf,outOfDateOutputFileName:h,newerInputFileName:s};var U=W(n,t.options.configFilePath,v,h);if(U)return U;var V=e.forEach(t.options.configFile.extendedSourceFiles||e.emptyArray,function(e){return W(n,e,v,h)});if(V)return V;var j=e.forEach(n.lastCachedPackageJsonLookups.get(r)||e.emptyArray,function(e){var t=e[0];return W(n,t,v,h)});if(j)return j;if(!a&&!n.buildInfoChecked.has(r)){n.buildInfoChecked.set(r,!0);var K=e.getTsBuildInfoEmitOutputFilePath(t.options);if(K){var H=n.readFileWithCache(K),z=H&&e.getBuildInfo(H);if(z&&(z.bundle||z.program)&&z.version!==e.version)return{type:e.UpToDateStatusType.TsVersionOutputOfDate,version:z.version}}}return O&&R?{type:e.UpToDateStatusType.OutOfDateWithPrepend,outOfDateOutputFileName:h,newerProjectName:k}:{type:R?e.UpToDateStatusType.UpToDateWithUpstreamTypes:e.UpToDateStatusType.UpToDate,newestDeclarationFileContentChangedTime:S,newestInputFileTime:c,newestOutputFileTime:E,newestInputFileName:s,newestOutputFileName:b,oldestOutputFileName:h}}function J(n,t,r){if(void 0===t)return{type:e.UpToDateStatusType.Unbuildable,reason:"File deleted mid-build"};var i=n.projectStatus.get(r);if(void 0!==i)return i;var a=z(n,t,r);return n.projectStatus.set(r,a),a}function q(n,t,r,i,a){if(t.options.noEmit)return r;var o=n.host,s=e.getAllProjectOutputs(t,!o.useCaseSensitiveFileNames());if(!a||s.length!==a.size)for(var c=!!n.options.verbose,d=o.now?o.now():new Date,p=0,f=s;p<f.length;p++){var m=f[p];a&&a.has(E(n,m))||(c&&(c=!1,pe(n,i,t.options.configFilePath)),u(m)&&(r=l(r,e.getModifiedTime(o,m))),o.setModifiedTime(m,d))}return r}function X(n,t,r){if(n.options.dry)return pe(n,e.Diagnostics.A_non_dry_build_would_update_timestamps_for_output_of_project_0,t.options.configFilePath);var a=q(n,t,i,e.Diagnostics.Updating_output_timestamps_of_project_0);n.projectStatus.set(r,{type:e.UpToDateStatusType.UpToDate,newestDeclarationFileContentChangedTime:a,oldestOutputFileName:e.getFirstProjectOutput(t,!n.host.useCaseSensitiveFileNames())})}function Y(t,r,i,a,o,s,c){if(!(c&n.AnyErrors)&&o.options.composite)for(var l=a+1;l<s.length;l++){var u=s[l],d=T(t,u);if(!t.projectPendingBuild.has(d)){var p=L(t,u,d);if(p&&p.projectReferences)for(var f=0,m=p.projectReferences;f<m.length;f++){var _=m[f],g=D(t,_.path);if(T(t,g)===i){var y=t.projectStatus.get(d);if(y)switch(y.type){case e.UpToDateStatusType.UpToDate:if(c&n.DeclarationOutputUnchanged){_.prepend?t.projectStatus.set(d,{type:e.UpToDateStatusType.OutOfDateWithPrepend,outOfDateOutputFileName:y.oldestOutputFileName,newerProjectName:r}):y.type=e.UpToDateStatusType.UpToDateWithUpstreamTypes;break}case e.UpToDateStatusType.UpToDateWithUpstreamTypes:case e.UpToDateStatusType.OutOfDateWithPrepend:c&n.DeclarationOutputUnchanged||t.projectStatus.set(d,{type:e.UpToDateStatusType.OutOfDateWithUpstream,outOfDateOutputFileName:y.type===e.UpToDateStatusType.OutOfDateWithPrepend?y.outOfDateOutputFileName:y.oldestOutputFileName,newerProjectName:r});break;case e.UpToDateStatusType.UpstreamBlocked:T(t,D(t,y.upstreamProjectName))===i&&O(t,d);break}w(t,d,e.ConfigFileProgramReloadLevel.None);break}}}}}function Q(n,t,r,i,a,o){var s=N(n,t,o);if(!s)return e.ExitStatus.InvalidProject_OutputsSkipped;P(n,r);var c=!0,l=0;while(1){var u=U(n,s,c);if(!u)break;c=!1,u.done(r,i,null===a||void 0===a?void 0:a(u.project)),n.diagnostics.has(u.projectPath)||l++}return R(n),ye(n,s),ce(n,s),d(s)?e.ExitStatus.ProjectReferenceCycle_OutputsSkipped:s.some(function(e){return n.diagnostics.has(T(n,e))})?l?e.ExitStatus.DiagnosticsPresent_OutputsGenerated:e.ExitStatus.DiagnosticsPresent_OutputsSkipped:e.ExitStatus.Success}function Z(n,t,r){var i=N(n,t,r);if(!i)return e.ExitStatus.InvalidProject_OutputsSkipped;if(d(i))return me(n,i.circularDiagnostics),e.ExitStatus.ProjectReferenceCycle_OutputsSkipped;for(var a=n.options,o=n.host,s=a.dry?[]:void 0,c=0,l=i;c<l.length;c++){var u=l[c],p=T(n,u),f=L(n,u,p);if(void 0!==f){var m=e.getAllProjectOutputs(f,!o.useCaseSensitiveFileNames());if(m.length)for(var _=new e.Set(f.fileNames.map(function(e){return E(n,e)})),g=0,y=m;g<y.length;g++){var h=y[g];_.has(E(n,h))||o.fileExists(h)&&(s?s.push(h):(o.deleteFile(h),$(n,p,e.ConfigFileProgramReloadLevel.None)))}}else ge(n,p)}return s&&pe(n,e.Diagnostics.A_non_dry_build_would_delete_the_following_files_Colon_0,s.map(function(e){return"\r\n * "+e}).join("")),e.ExitStatus.Success}function $(n,t,r){n.host.getParsedCommandLine&&r===e.ConfigFileProgramReloadLevel.Partial&&(r=e.ConfigFileProgramReloadLevel.Full),r===e.ConfigFileProgramReloadLevel.Full&&(n.configFileCache.delete(t),n.buildOrder=void 0),n.needsSummary=!0,O(n,t),w(n,t,r),k(n)}function ee(e,n,t){e.reportFileChangeDetected=!0,$(e,n,t),ne(e)}function ne(e){var n=e.hostWithWatch;n.setTimeout&&n.clearTimeout&&(e.timerToBuildInvalidatedProject&&n.clearTimeout(e.timerToBuildInvalidatedProject),e.timerToBuildInvalidatedProject=n.setTimeout(te,250,e))}function te(n){n.timerToBuildInvalidatedProject=void 0,n.reportFileChangeDetected&&(n.reportFileChangeDetected=!1,n.projectErrorsReported.clear(),fe(n,e.Diagnostics.File_change_detected_Starting_incremental_compilation));var t=C(n),r=U(n,t,!1);r&&(r.done(),n.projectPendingBuild.size)?n.watch&&!n.timerToBuildInvalidatedProject&&ne(n):(R(n),ye(n,t))}function re(n,t,r,i){n.watch&&!n.allWatchedConfigFiles.has(r)&&n.allWatchedConfigFiles.set(r,n.watchFile(t,function(){ee(n,r,e.ConfigFileProgramReloadLevel.Full)},e.PollingInterval.High,null===i||void 0===i?void 0:i.watchOptions,e.WatchType.ConfigFile,t))}function ie(n,t,r){e.updateSharedExtendedConfigFileWatcher(t,null===r||void 0===r?void 0:r.options,n.allWatchedExtendedConfigFiles,function(t,i){return n.watchFile(t,function(){var t;return null===(t=n.allWatchedExtendedConfigFiles.get(i))||void 0===t?void 0:t.projects.forEach(function(t){return ee(n,t,e.ConfigFileProgramReloadLevel.Full)})},e.PollingInterval.High,null===r||void 0===r?void 0:r.watchOptions,e.WatchType.ExtendedConfigFile)},function(e){return E(n,e)})}function ae(n,t,r,i){n.watch&&e.updateWatchingWildcardDirectories(c(n.allWatchedWildcardDirectories,r),new e.Map(e.getEntries(i.wildcardDirectories)),function(a,o){return n.watchDirectory(a,function(o){var s;e.isIgnoredFileFromWildCardWatching({watchedDirPath:E(n,a),fileOrDirectory:o,fileOrDirectoryPath:E(n,o),configFileName:t,currentDirectory:n.currentDirectory,options:i.options,program:n.builderPrograms.get(r)||(null===(s=x(n,r))||void 0===s?void 0:s.fileNames),useCaseSensitiveFileNames:n.parseConfigFileHost.useCaseSensitiveFileNames,writeLog:function(e){return n.writeLog(e)},toPath:function(e){return E(n,e)}})||ee(n,r,e.ConfigFileProgramReloadLevel.Partial)},o,null===i||void 0===i?void 0:i.watchOptions,e.WatchType.WildcardDirectory,t)})}function oe(n,t,r,i){n.watch&&e.mutateMap(c(n.allWatchedInputFiles,r),e.arrayToMap(i.fileNames,function(e){return E(n,e)}),{createNewValue:function(a,o){return n.watchFile(o,function(){return ee(n,r,e.ConfigFileProgramReloadLevel.None)},e.PollingInterval.Low,null===i||void 0===i?void 0:i.watchOptions,e.WatchType.SourceFile,t)},onDeleteValue:e.closeFileWatcher})}function se(n,t,r,i){n.watch&&n.lastCachedPackageJsonLookups&&e.mutateMap(c(n.allWatchedPackageJsonFiles,r),new e.Map(n.lastCachedPackageJsonLookups.get(r)),{createNewValue:function(a,o){return n.watchFile(a,function(){return ee(n,r,e.ConfigFileProgramReloadLevel.Full)},e.PollingInterval.High,null===i||void 0===i?void 0:i.watchOptions,e.WatchType.PackageJson,t)},onDeleteValue:e.closeFileWatcher})}function ce(e,n){if(e.watchAllProjectsPending){e.watchAllProjectsPending=!1;for(var t=0,r=p(n);t<r.length;t++){var i=r[t],a=T(e,i),o=L(e,i,a);re(e,i,a,o),ie(e,a,o),o&&(ae(e,i,a,o),oe(e,i,a,o),se(e,i,a,o))}}}function le(n){e.clearMap(n.allWatchedConfigFiles,e.closeFileWatcher),e.clearMap(n.allWatchedExtendedConfigFiles,e.closeFileWatcherOf),e.clearMap(n.allWatchedWildcardDirectories,function(n){return e.clearMap(n,e.closeFileWatcherOf)}),e.clearMap(n.allWatchedInputFiles,function(n){return e.clearMap(n,e.closeFileWatcher)}),e.clearMap(n.allWatchedPackageJsonFiles,function(n){return e.clearMap(n,e.closeFileWatcher)})}function ue(n,t,r,i,a){var o=b(n,t,r,i,a);return{build:function(e,n,t,r){return Q(o,e,n,t,r)},clean:function(e){return Z(o,e)},buildReferences:function(e,n,t,r){return Q(o,e,n,t,r,!0)},cleanReferences:function(e){return Z(o,e,!0)},getNextInvalidatedProject:function(e){return P(o,e),U(o,C(o),!1)},getBuildOrder:function(){return C(o)},getUpToDateStatusOfProject:function(e){var n=D(o,e),t=T(o,n);return J(o,L(o,n,t),t)},invalidateProject:function(n,t){return $(o,n,t||e.ConfigFileProgramReloadLevel.None)},buildNextInvalidatedProject:function(){return te(o)},getAllParsedConfigs:function(){return e.arrayFrom(e.mapDefinedIterator(o.configFileCache.values(),function(e){return S(e)?e:void 0}))},close:function(){return le(o)}}}function de(n,t){return e.convertToRelativePath(t,n.currentDirectory,function(e){return n.getCanonicalFileName(e)})}function pe(n,t){for(var r=[],i=2;i<arguments.length;i++)r[i-2]=arguments[i];n.host.reportSolutionBuilderStatus(e.createCompilerDiagnostic.apply(void 0,a([t],r,!1)))}function fe(n,t){for(var r,i,o=[],s=2;s<arguments.length;s++)o[s-2]=arguments[s];null===(i=(r=n.hostWithWatch).onWatchStatusChange)||void 0===i||i.call(r,e.createCompilerDiagnostic.apply(void 0,a([t],o,!1)),n.host.getNewLine(),n.baseCompilerOptions)}function me(e,n){var t=e.host;n.forEach(function(e){return t.reportDiagnostic(e)})}function _e(e,n,t){me(e,t),e.projectErrorsReported.set(n,!0),t.length&&e.diagnostics.set(n,t)}function ge(e,n){_e(e,n,[e.configFileCache.get(n)])}function ye(n,t){if(n.needsSummary){n.needsSummary=!1;var r=n.watch||!!n.host.reportErrorSummary,i=n.diagnostics,a=0;d(t)?(he(n,t.buildOrder),me(n,t.circularDiagnostics),r&&(a+=e.getErrorCountForSummary(t.circularDiagnostics))):(t.forEach(function(t){var r=T(n,t);n.projectErrorsReported.has(r)||me(n,i.get(r)||e.emptyArray)}),r&&i.forEach(function(n){return a+=e.getErrorCountForSummary(n)})),n.watch?fe(n,e.getWatchErrorSummaryDiagnosticMessage(a),a):n.host.reportErrorSummary&&n.host.reportErrorSummary(a)}}function he(n,t){n.options.verbose&&pe(n,e.Diagnostics.Projects_in_this_build_Colon_0,t.map(function(e){return"\r\n    * "+de(n,e)}).join(""))}function ve(n,t,r){if(n.options.force&&(r.type===e.UpToDateStatusType.UpToDate||r.type===e.UpToDateStatusType.UpToDateWithUpstreamTypes))return pe(n,e.Diagnostics.Project_0_is_being_forcibly_rebuilt,de(n,t));switch(r.type){case e.UpToDateStatusType.OutOfDateWithSelf:return pe(n,e.Diagnostics.Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2,de(n,t),de(n,r.outOfDateOutputFileName),de(n,r.newerInputFileName));case e.UpToDateStatusType.OutOfDateWithUpstream:return pe(n,e.Diagnostics.Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2,de(n,t),de(n,r.outOfDateOutputFileName),de(n,r.newerProjectName));case e.UpToDateStatusType.OutputMissing:return pe(n,e.Diagnostics.Project_0_is_out_of_date_because_output_file_1_does_not_exist,de(n,t),de(n,r.missingOutputFileName));case e.UpToDateStatusType.UpToDate:if(void 0!==r.newestInputFileTime)return pe(n,e.Diagnostics.Project_0_is_up_to_date_because_newest_input_1_is_older_than_oldest_output_2,de(n,t),de(n,r.newestInputFileName||""),de(n,r.oldestOutputFileName||""));break;case e.UpToDateStatusType.OutOfDateWithPrepend:return pe(n,e.Diagnostics.Project_0_is_out_of_date_because_output_of_its_dependency_1_has_changed,de(n,t),de(n,r.newerProjectName));case e.UpToDateStatusType.UpToDateWithUpstreamTypes:return pe(n,e.Diagnostics.Project_0_is_up_to_date_with_d_ts_files_from_its_dependencies,de(n,t));case e.UpToDateStatusType.UpstreamOutOfDate:return pe(n,e.Diagnostics.Project_0_is_out_of_date_because_its_dependency_1_is_out_of_date,de(n,t),de(n,r.upstreamProjectName));case e.UpToDateStatusType.UpstreamBlocked:return pe(n,r.upstreamProjectBlocked?e.Diagnostics.Project_0_can_t_be_built_because_its_dependency_1_was_not_built:e.Diagnostics.Project_0_can_t_be_built_because_its_dependency_1_has_errors,de(n,t),de(n,r.upstreamProjectName));case e.UpToDateStatusType.Unbuildable:return pe(n,e.Diagnostics.Failed_to_parse_file_0_Colon_1,de(n,t),r.reason);case e.UpToDateStatusType.TsVersionOutputOfDate:return pe(n,e.Diagnostics.Project_0_is_out_of_date_because_output_for_it_was_generated_with_version_1_that_differs_with_current_version_2,de(n,t),r.version,e.version);case e.UpToDateStatusType.ContainerOnly:case e.UpToDateStatusType.ComputingUpstream:break;default:e.assertType(r)}}function be(e,n,t){e.options.verbose&&ve(e,n,t)}(function(e){e[e["None"]=0]="None",e[e["Success"]=1]="Success",e[e["DeclarationOutputUnchanged"]=2]="DeclarationOutputUnchanged",e[e["ConfigFileErrors"]=4]="ConfigFileErrors",e[e["SyntaxErrors"]=8]="SyntaxErrors",e[e["TypeErrors"]=16]="TypeErrors",e[e["DeclarationEmitErrors"]=32]="DeclarationEmitErrors",e[e["EmitErrors"]=64]="EmitErrors",e[e["AnyErrors"]=124]="AnyErrors"})(n||(n={})),e.isCircularBuildOrder=d,e.getBuildOrderFromAnyBuildOrder=p,e.createBuilderStatusReporter=f,e.createSolutionBuilderHost=_,e.createSolutionBuilderWithWatchHost=g,e.createSolutionBuilder=h,e.createSolutionBuilderWithWatch=v,function(e){e[e["Build"]=0]="Build",e[e["UpdateBundle"]=1]="UpdateBundle",e[e["UpdateOutputFileStamps"]=2]="UpdateOutputFileStamps"}(t=e.InvalidatedProjectKind||(e.InvalidatedProjectKind={})),function(e){e[e["CreateProgram"]=0]="CreateProgram",e[e["SyntaxDiagnostics"]=1]="SyntaxDiagnostics",e[e["SemanticDiagnostics"]=2]="SemanticDiagnostics",e[e["Emit"]=3]="Emit",e[e["EmitBundle"]=4]="EmitBundle",e[e["EmitBuildInfo"]=5]="EmitBuildInfo",e[e["BuildInvalidatedProjectOfBundle"]=6]="BuildInvalidatedProjectOfBundle",e[e["QueueReferencingProjects"]=7]="QueueReferencingProjects",e[e["Done"]=8]="Done"}(r||(r={}))}(d||(d={})),function(e){(function(n){function t(n){return e.sys.args.indexOf(n)>=0}function r(n){var t=e.sys.args.indexOf(n);return t>=0&&t<e.sys.args.length-1?e.sys.args[t+1]:void 0}function i(){var n=new Date;return e.padLeft(n.getHours().toString(),2,"0")+":"+e.padLeft(n.getMinutes().toString(),2,"0")+":"+e.padLeft(n.getSeconds().toString(),2,"0")+"."+e.padLeft(n.getMilliseconds().toString(),3,"0")}n.ActionSet="action::set",n.ActionInvalidate="action::invalidate",n.ActionPackageInstalled="action::packageInstalled",n.EventTypesRegistry="event::typesRegistry",n.EventBeginInstallTypes="event::beginInstallTypes",n.EventEndInstallTypes="event::endInstallTypes",n.EventInitializationFailed="event::initializationFailed",function(e){e.GlobalCacheLocation="--globalTypingsCacheLocation",e.LogFile="--logFile",e.EnableTelemetry="--enableTelemetry",e.TypingSafeListLocation="--typingSafeListLocation",e.TypesMapLocation="--typesMapLocation",e.NpmLocation="--npmLocation",e.ValidateDefaultNpmLocation="--validateDefaultNpmLocation"}(n.Arguments||(n.Arguments={})),n.hasArgument=t,n.findArgument=r,n.nowString=i})(e.server||(e.server={}))}(d||(d={})),function(e){(function(n){function t(n,t){var r=new e.Version(e.getProperty(t,"ts"+e.versionMajorMinor)||e.getProperty(t,"latest"));return r.compareTo(n.version)<=0}n.isTypingUpToDate=t;var r=["assert","assert/strict","async_hooks","buffer","child_process","cluster","console","constants","crypto","dgram","diagnostics_channel","dns","dns/promises","domain","events","fs","fs/promises","http","https","http2","inspector","module","net","os","path","perf_hooks","process","punycode","querystring","readline","repl","stream","stream/promises","string_decoder","timers","timers/promises","tls","trace_events","tty","url","util","util/types","v8","vm","wasi","worker_threads","zlib"];function i(e){return n.nodeCoreModules.has(e)?"node":e}function o(n,t){var r=e.readConfigFile(t,function(e){return n.readFile(e)});return new e.Map(e.getEntries(r.config))}function s(n,t){var r=e.readConfigFile(t,function(e){return n.readFile(e)});if(r.config)return new e.Map(e.getEntries(r.config.simpleMap))}function c(n,r,a,o,s,c,l,u,d){if(!l||!l.enable)return{cachedTypingPaths:[],newTypingNames:[],filesToWatch:[]};var p=new e.Map;a=e.mapDefined(a,function(n){var t=e.normalizePath(n);if(e.hasJSFileExtension(t))return t});var f=[];l.include&&L(l.include,"Explicitly included types");var m=l.exclude||[],_=new e.Set(a.map(e.getDirectoryPath));if(_.add(o),_.forEach(function(n){var t=e.combinePaths(n,"package.json");D(t,f);var r=e.combinePaths(n,"bower.json");D(r,f);var i=e.combinePaths(n,"bower_components");C(i,f);var a=e.combinePaths(n,"node_modules");C(a,f)}),l.disableFilenameBasedTypeAcquisition||A(a),u){var g=e.deduplicate(u.map(i),e.equateStringsCaseSensitive,e.compareStringsCaseSensitive);L(g,"Inferred typings from unresolved imports")}c.forEach(function(e,n){var r=d.get(n);p.has(n)&&void 0===p.get(n)&&void 0!==r&&t(e,r)&&p.set(n,e.typingLocation)});for(var y=0,h=m;y<h.length;y++){var v=h[y],b=p.delete(v);b&&r&&r("Typing for "+v+" is in exclude list, will be ignored.")}var E=[],T=[];p.forEach(function(e,n){void 0!==e?T.push(e):E.push(n)});var S={cachedTypingPaths:T,newTypingNames:E,filesToWatch:f};return r&&r("Result: "+JSON.stringify(S)),S;function x(e){p.has(e)||p.set(e,void 0)}function L(n,t){r&&r(t+": "+JSON.stringify(n)),e.forEach(n,x)}function D(t,r){if(n.fileExists(t)){r.push(t);var i=e.readConfigFile(t,function(e){return n.readFile(e)}).config,a=e.flatMap([i.dependencies,i.devDependencies,i.optionalDependencies,i.peerDependencies],e.getOwnKeys);L(a,"Typing names in '"+t+"' dependencies")}}function A(n){var t=e.mapDefined(n,function(n){if(e.hasJSFileExtension(n)){var t=e.removeFileExtension(e.getBaseFileName(n.toLowerCase())),r=e.removeMinAndVersionNumbers(t);return s.get(r)}});t.length&&L(t,"Inferred typings from file names");var i=e.some(n,function(n){return e.fileExtensionIs(n,".jsx")});i&&(r&&r("Inferred 'react' typings due to presence of '.jsx' extension"),x("react"))}function C(t,i){if(i.push(t),n.directoryExists(t)){var a=n.readDirectory(t,[".json"],void 0,void 0,2);r&&r("Searching for typing names in "+t+"; all files: "+JSON.stringify(a));for(var o=[],s=0,c=a;s<c.length;s++){var l=c[s],u=e.normalizePath(l),d=e.getBaseFileName(u);if("package.json"===d||"bower.json"===d){var f=e.readConfigFile(u,function(e){return n.readFile(e)}),m=f.config;if(("package.json"!==d||!m._requiredBy||0!==e.filter(m._requiredBy,function(e){return"#"===e[0]||"/"===e}).length)&&m.name){var _=m.types||m.typings;if(_){var g=e.getNormalizedAbsolutePath(_,e.getDirectoryPath(u));r&&r("    Package '"+m.name+"' provides its own types."),p.set(m.name,g)}else o.push(m.name)}}}L(o,"    Found package names")}}}n.prefixedNodeCoreModuleList=r.map(function(e){return"node:"+e}),n.nodeCoreModuleList=a(a([],r,!0),n.prefixedNodeCoreModuleList,!0),n.nodeCoreModules=new e.Set(n.nodeCoreModuleList),n.nonRelativeModuleNameForTypingCache=i,n.loadSafeList=o,n.loadTypesMap=s,n.discoverTypings=c,function(e){e[e["Ok"]=0]="Ok",e[e["EmptyName"]=1]="EmptyName",e[e["NameTooLong"]=2]="NameTooLong",e[e["NameStartsWithDot"]=3]="NameStartsWithDot",e[e["NameStartsWithUnderscore"]=4]="NameStartsWithUnderscore",e[e["NameContainsNonURISafeCharacters"]=5]="NameContainsNonURISafeCharacters"}(n.NameValidationResult||(n.NameValidationResult={}));var l=214;function u(e){return d(e,!0)}function d(e,n){if(!e)return 1;if(e.length>l)return 2;if(46===e.charCodeAt(0))return 3;if(95===e.charCodeAt(0))return 4;if(n){var t=/^@([^/]+)\/([^/]+)$/.exec(e);if(t){var r=d(t[1],!1);if(0!==r)return{name:t[1],isScopeName:!0,result:r};var i=d(t[2],!1);return 0!==i?{name:t[2],isScopeName:!1,result:i}:0}}return encodeURIComponent(e)!==e?5:0}function p(e,n){return"object"===typeof e?f(n,e.result,e.name,e.isScopeName):f(n,e,n,!1)}function f(n,t,r,i){var a=i?"Scope":"Package";switch(t){case 1:return"'"+n+"':: "+a+" name '"+r+"' cannot be empty";case 2:return"'"+n+"':: "+a+" name '"+r+"' should be less than "+l+" characters";case 3:return"'"+n+"':: "+a+" name '"+r+"' cannot start with '.'";case 4:return"'"+n+"':: "+a+" name '"+r+"' cannot start with '_'";case 5:return"'"+n+"':: "+a+" name '"+r+"' contains non URI safe characters";case 0:return e.Debug.fail();default:throw e.Debug.assertNever(t)}}n.validatePackageName=u,n.renderPackageNameValidationFailure=p})(e.JsTyping||(e.JsTyping={}))}(d||(d={})),function(e){var n,t;function r(e){return{indentSize:4,tabSize:4,newLineCharacter:e||"\n",convertTabsToSpaces:!0,indentStyle:n.Smart,insertSpaceAfterConstructor:!1,insertSpaceAfterCommaDelimiter:!0,insertSpaceAfterSemicolonInForStatements:!0,insertSpaceBeforeAndAfterBinaryOperators:!0,insertSpaceAfterKeywordsInControlFlowStatements:!0,insertSpaceAfterFunctionKeywordForAnonymousFunctions:!1,insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis:!1,insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets:!1,insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces:!0,insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces:!1,insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces:!1,insertSpaceBeforeFunctionParenthesis:!1,placeOpenBraceOnNewLineForFunctions:!1,placeOpenBraceOnNewLineForControlBlocks:!1,semicolons:t.Ignore,trimTrailingWhitespace:!0}}(function(e){var n=function(){function e(e){this.text=e}return e.prototype.getText=function(e,n){return 0===e&&n===this.text.length?this.text:this.text.substring(e,n)},e.prototype.getLength=function(){return this.text.length},e.prototype.getChangeRange=function(){},e}();function t(e){return new n(e)}e.fromString=t})(e.ScriptSnapshot||(e.ScriptSnapshot={})),function(e){e[e["Dependencies"]=1]="Dependencies",e[e["DevDependencies"]=2]="DevDependencies",e[e["PeerDependencies"]=4]="PeerDependencies",e[e["OptionalDependencies"]=8]="OptionalDependencies",e[e["All"]=15]="All"}(e.PackageJsonDependencyGroup||(e.PackageJsonDependencyGroup={})),function(e){e[e["Off"]=0]="Off",e[e["On"]=1]="On",e[e["Auto"]=2]="Auto"}(e.PackageJsonAutoImportPreference||(e.PackageJsonAutoImportPreference={})),function(e){e[e["Semantic"]=0]="Semantic",e[e["PartialSemantic"]=1]="PartialSemantic",e[e["Syntactic"]=2]="Syntactic"}(e.LanguageServiceMode||(e.LanguageServiceMode={})),e.emptyOptions={},function(e){e["Original"]="original",e["TwentyTwenty"]="2020"}(e.SemanticClassificationFormat||(e.SemanticClassificationFormat={})),function(e){e[e["Invoked"]=1]="Invoked",e[e["TriggerCharacter"]=2]="TriggerCharacter",e[e["TriggerForIncompleteCompletions"]=3]="TriggerForIncompleteCompletions"}(e.CompletionTriggerKind||(e.CompletionTriggerKind={})),function(e){e["Type"]="Type",e["Parameter"]="Parameter",e["Enum"]="Enum"}(e.InlayHintKind||(e.InlayHintKind={})),function(e){e["none"]="none",e["definition"]="definition",e["reference"]="reference",e["writtenReference"]="writtenReference"}(e.HighlightSpanKind||(e.HighlightSpanKind={})),function(e){e[e["None"]=0]="None",e[e["Block"]=1]="Block",e[e["Smart"]=2]="Smart"}(n=e.IndentStyle||(e.IndentStyle={})),function(e){e["Ignore"]="ignore",e["Insert"]="insert",e["Remove"]="remove"}(t=e.SemicolonPreference||(e.SemicolonPreference={})),e.getDefaultFormatCodeSettings=r,e.testFormatSettings=r("\n"),function(e){e[e["aliasName"]=0]="aliasName",e[e["className"]=1]="className",e[e["enumName"]=2]="enumName",e[e["fieldName"]=3]="fieldName",e[e["interfaceName"]=4]="interfaceName",e[e["keyword"]=5]="keyword",e[e["lineBreak"]=6]="lineBreak",e[e["numericLiteral"]=7]="numericLiteral",e[e["stringLiteral"]=8]="stringLiteral",e[e["localName"]=9]="localName",e[e["methodName"]=10]="methodName",e[e["moduleName"]=11]="moduleName",e[e["operator"]=12]="operator",e[e["parameterName"]=13]="parameterName",e[e["propertyName"]=14]="propertyName",e[e["punctuation"]=15]="punctuation",e[e["space"]=16]="space",e[e["text"]=17]="text",e[e["typeParameterName"]=18]="typeParameterName",e[e["enumMemberName"]=19]="enumMemberName",e[e["functionName"]=20]="functionName",e[e["regularExpressionLiteral"]=21]="regularExpressionLiteral",e[e["link"]=22]="link",e[e["linkName"]=23]="linkName",e[e["linkText"]=24]="linkText"}(e.SymbolDisplayPartKind||(e.SymbolDisplayPartKind={})),function(e){e["Comment"]="comment",e["Region"]="region",e["Code"]="code",e["Imports"]="imports"}(e.OutliningSpanKind||(e.OutliningSpanKind={})),function(e){e[e["JavaScript"]=0]="JavaScript",e[e["SourceMap"]=1]="SourceMap",e[e["Declaration"]=2]="Declaration"}(e.OutputFileType||(e.OutputFileType={})),function(e){e[e["None"]=0]="None",e[e["InMultiLineCommentTrivia"]=1]="InMultiLineCommentTrivia",e[e["InSingleQuoteStringLiteral"]=2]="InSingleQuoteStringLiteral",e[e["InDoubleQuoteStringLiteral"]=3]="InDoubleQuoteStringLiteral",e[e["InTemplateHeadOrNoSubstitutionTemplate"]=4]="InTemplateHeadOrNoSubstitutionTemplate",e[e["InTemplateMiddleOrTail"]=5]="InTemplateMiddleOrTail",e[e["InTemplateSubstitutionPosition"]=6]="InTemplateSubstitutionPosition"}(e.EndOfLineState||(e.EndOfLineState={})),function(e){e[e["Punctuation"]=0]="Punctuation",e[e["Keyword"]=1]="Keyword",e[e["Operator"]=2]="Operator",e[e["Comment"]=3]="Comment",e[e["Whitespace"]=4]="Whitespace",e[e["Identifier"]=5]="Identifier",e[e["NumberLiteral"]=6]="NumberLiteral",e[e["BigIntLiteral"]=7]="BigIntLiteral",e[e["StringLiteral"]=8]="StringLiteral",e[e["RegExpLiteral"]=9]="RegExpLiteral"}(e.TokenClass||(e.TokenClass={})),function(e){e["unknown"]="",e["warning"]="warning",e["keyword"]="keyword",e["scriptElement"]="script",e["moduleElement"]="module",e["classElement"]="class",e["localClassElement"]="local class",e["interfaceElement"]="interface",e["typeElement"]="type",e["enumElement"]="enum",e["enumMemberElement"]="enum member",e["variableElement"]="var",e["localVariableElement"]="local var",e["functionElement"]="function",e["localFunctionElement"]="local function",e["memberFunctionElement"]="method",e["memberGetAccessorElement"]="getter",e["memberSetAccessorElement"]="setter",e["memberVariableElement"]="property",e["constructorImplementationElement"]="constructor",e["callSignatureElement"]="call",e["indexSignatureElement"]="index",e["constructSignatureElement"]="construct",e["parameterElement"]="parameter",e["typeParameterElement"]="type parameter",e["primitiveType"]="primitive type",e["label"]="label",e["alias"]="alias",e["constElement"]="const",e["letElement"]="let",e["directory"]="directory",e["externalModuleName"]="external module name",e["jsxAttribute"]="JSX attribute",e["string"]="string",e["link"]="link",e["linkName"]="link name",e["linkText"]="link text"}(e.ScriptElementKind||(e.ScriptElementKind={})),function(e){e["none"]="",e["publicMemberModifier"]="public",e["privateMemberModifier"]="private",e["protectedMemberModifier"]="protected",e["exportedModifier"]="export",e["ambientModifier"]="declare",e["staticModifier"]="static",e["abstractModifier"]="abstract",e["optionalModifier"]="optional",e["deprecatedModifier"]="deprecated",e["dtsModifier"]=".d.ts",e["tsModifier"]=".ts",e["tsxModifier"]=".tsx",e["jsModifier"]=".js",e["jsxModifier"]=".jsx",e["jsonModifier"]=".json"}(e.ScriptElementKindModifier||(e.ScriptElementKindModifier={})),function(e){e["comment"]="comment",e["identifier"]="identifier",e["keyword"]="keyword",e["numericLiteral"]="number",e["bigintLiteral"]="bigint",e["operator"]="operator",e["stringLiteral"]="string",e["whiteSpace"]="whitespace",e["text"]="text",e["punctuation"]="punctuation",e["className"]="class name",e["enumName"]="enum name",e["interfaceName"]="interface name",e["moduleName"]="module name",e["typeParameterName"]="type parameter name",e["typeAliasName"]="type alias name",e["parameterName"]="parameter name",e["docCommentTagName"]="doc comment tag name",e["jsxOpenTagName"]="jsx open tag name",e["jsxCloseTagName"]="jsx close tag name",e["jsxSelfClosingTagName"]="jsx self closing tag name",e["jsxAttribute"]="jsx attribute",e["jsxText"]="jsx text",e["jsxAttributeStringLiteralValue"]="jsx attribute string literal value"}(e.ClassificationTypeNames||(e.ClassificationTypeNames={})),function(e){e[e["comment"]=1]="comment",e[e["identifier"]=2]="identifier",e[e["keyword"]=3]="keyword",e[e["numericLiteral"]=4]="numericLiteral",e[e["operator"]=5]="operator",e[e["stringLiteral"]=6]="stringLiteral",e[e["regularExpressionLiteral"]=7]="regularExpressionLiteral",e[e["whiteSpace"]=8]="whiteSpace",e[e["text"]=9]="text",e[e["punctuation"]=10]="punctuation",e[e["className"]=11]="className",e[e["enumName"]=12]="enumName",e[e["interfaceName"]=13]="interfaceName",e[e["moduleName"]=14]="moduleName",e[e["typeParameterName"]=15]="typeParameterName",e[e["typeAliasName"]=16]="typeAliasName",e[e["parameterName"]=17]="parameterName",e[e["docCommentTagName"]=18]="docCommentTagName",e[e["jsxOpenTagName"]=19]="jsxOpenTagName",e[e["jsxCloseTagName"]=20]="jsxCloseTagName",e[e["jsxSelfClosingTagName"]=21]="jsxSelfClosingTagName",e[e["jsxAttribute"]=22]="jsxAttribute",e[e["jsxText"]=23]="jsxText",e[e["jsxAttributeStringLiteralValue"]=24]="jsxAttributeStringLiteralValue",e[e["bigintLiteral"]=25]="bigintLiteral"}(e.ClassificationType||(e.ClassificationType={}))}(d||(d={})),function(e){function n(n){switch(n.kind){case 252:return e.isInJSFile(n)&&e.getJSDocEnumTag(n)?7:1;case 162:case 201:case 165:case 164:case 291:case 292:case 167:case 166:case 169:case 170:case 171:case 254:case 211:case 212:case 290:case 283:return 1;case 161:case 256:case 257:case 180:return 2;case 340:return void 0===n.name?3:2;case 294:case 255:return 3;case 259:return e.isAmbientModule(n)?5:1===e.getModuleInstanceState(n)?5:4;case 258:case 267:case 268:case 263:case 264:case 269:case 270:return 7;case 300:return 5}return 7}function t(t){t=me(t);var o=t.parent;if(300===t.kind)return 1;if(e.isExportAssignment(o)||e.isExportSpecifier(o)||e.isExternalModuleReference(o)||e.isImportSpecifier(o)||e.isImportClause(o)||e.isImportEqualsDeclaration(o)&&t===o.name){var s=o;while(s){if(e.isImportEqualsDeclaration(s)||e.isImportClause(s)||e.isExportDeclaration(s))return s.isTypeOnly?2:7;s=s.parent}return 7}return i(t)?r(t):e.isDeclarationName(t)?n(o):e.isEntityName(t)&&e.findAncestor(t,e.or(e.isJSDocNameReference,e.isJSDocLinkLike,e.isJSDocMemberName))?7:l(t)?2:a(t)?4:e.isTypeParameterDeclaration(o)?(e.Debug.assert(e.isJSDocTemplateTag(o.parent)),2):e.isLiteralTypeNode(o)?3:1}function r(n){var t=159===n.kind?n:e.isQualifiedName(n.parent)&&n.parent.right===n?n.parent:void 0;return t&&263===t.parent.kind?7:4}function i(n){while(159===n.parent.kind)n=n.parent;return e.isInternalModuleImportEqualsDeclaration(n.parent)&&n.parent.moduleReference===n}function a(e){return s(e)||c(e)}function s(e){var n=e,t=!0;if(159===n.parent.kind){while(n.parent&&159===n.parent.kind)n=n.parent;t=n.right===e}return 176===n.parent.kind&&!t}function c(e){var n=e,t=!0;if(204===n.parent.kind){while(n.parent&&204===n.parent.kind)n=n.parent;t=n.name===e}if(!t&&226===n.parent.kind&&289===n.parent.parent.kind){var r=n.parent.parent.parent;return 255===r.kind&&117===n.parent.parent.token||256===r.kind&&94===n.parent.parent.token}return!1}function l(n){switch(e.isRightSideOfQualifiedNameOrPropertyAccess(n)&&(n=n.parent),n.kind){case 108:return!e.isExpressionNode(n);case 190:return!0}switch(n.parent.kind){case 176:return!0;case 198:return!n.parent.isTypeOf;case 226:return!e.isExpressionWithTypeArgumentsInClassExtendsClause(n.parent)}return!1}function u(n,t,r){return void 0===t&&(t=!1),void 0===r&&(r=!1),v(n,e.isCallExpression,g,t,r)}function d(n,t,r){return void 0===t&&(t=!1),void 0===r&&(r=!1),v(n,e.isNewExpression,g,t,r)}function p(n,t,r){return void 0===t&&(t=!1),void 0===r&&(r=!1),v(n,e.isCallOrNewExpression,g,t,r)}function f(n,t,r){return void 0===t&&(t=!1),void 0===r&&(r=!1),v(n,e.isTaggedTemplateExpression,y,t,r)}function m(n,t,r){return void 0===t&&(t=!1),void 0===r&&(r=!1),v(n,e.isDecorator,g,t,r)}function _(n,t,r){return void 0===t&&(t=!1),void 0===r&&(r=!1),v(n,e.isJsxOpeningLikeElement,h,t,r)}function g(e){return e.expression}function y(e){return e.tag}function h(e){return e.tagName}function v(n,t,r,i,a){var o=i?E(n):b(n);return a&&(o=e.skipOuterExpressions(o)),!!o&&!!o.parent&&t(o.parent)&&r(o.parent)===o}function b(e){return I(e)?e.parent:e}function E(e){return I(e)||N(e)?e.parent:e}function T(e,n){while(e){if(248===e.kind&&e.label.escapedText===n)return e.label;e=e.parent}}function S(n,t){return!!e.isPropertyAccessExpression(n.expression)&&n.expression.name.text===t}function x(n){var t;return e.isIdentifier(n)&&(null===(t=e.tryCast(n.parent,e.isBreakOrContinueStatement))||void 0===t?void 0:t.label)===n}function L(n){var t;return e.isIdentifier(n)&&(null===(t=e.tryCast(n.parent,e.isLabeledStatement))||void 0===t?void 0:t.label)===n}function D(e){return L(e)||x(e)}function A(n){var t;return(null===(t=e.tryCast(n.parent,e.isJSDocTag))||void 0===t?void 0:t.tagName)===n}function C(n){var t;return(null===(t=e.tryCast(n.parent,e.isQualifiedName))||void 0===t?void 0:t.right)===n}function I(n){var t;return(null===(t=e.tryCast(n.parent,e.isPropertyAccessExpression))||void 0===t?void 0:t.name)===n}function N(n){var t;return(null===(t=e.tryCast(n.parent,e.isElementAccessExpression))||void 0===t?void 0:t.argumentExpression)===n}function k(n){var t;return(null===(t=e.tryCast(n.parent,e.isModuleDeclaration))||void 0===t?void 0:t.name)===n}function R(n){var t;return e.isIdentifier(n)&&(null===(t=e.tryCast(n.parent,e.isFunctionLike))||void 0===t?void 0:t.name)===n}function O(n){switch(n.parent.kind){case 165:case 164:case 291:case 294:case 167:case 166:case 170:case 171:case 259:return e.getNameOfDeclaration(n.parent)===n;case 205:return n.parent.argumentExpression===n;case 160:return!0;case 194:return 192===n.parent.parent.kind;default:return!1}}function w(n){return e.isExternalModuleImportEqualsDeclaration(n.parent.parent)&&e.getExternalModuleImportEqualsDeclarationExpression(n.parent.parent)===n}function P(n){e.isJSDocTypeAlias(n)&&(n=n.parent.parent);while(1){if(n=n.parent,!n)return;switch(n.kind){case 300:case 167:case 166:case 254:case 211:case 170:case 171:case 255:case 256:case 258:case 259:return n}}}function M(n){switch(n.kind){case 300:return e.isExternalModule(n)?"module":"script";case 259:return"module";case 255:case 224:return"class";case 256:return"interface";case 257:case 333:case 340:return"type";case 258:return"enum";case 252:return s(n);case 201:return s(e.getRootDeclaration(n));case 212:case 254:case 211:return"function";case 170:return"getter";case 171:return"setter";case 167:case 166:return"method";case 291:var t=n.initializer;return e.isFunctionLike(t)?"method":"property";case 165:case 164:case 292:case 293:return"property";case 174:return"index";case 173:return"construct";case 172:return"call";case 169:case 168:return"constructor";case 161:return"type parameter";case 294:return"enum member";case 162:return e.hasSyntacticModifier(n,16476)?"property":"parameter";case 263:case 268:case 273:case 266:case 272:return"alias";case 219:var r=e.getAssignmentDeclarationKind(n),i=n.right;switch(r){case 7:case 8:case 9:case 0:return"";case 1:case 2:var a=M(i);return""===a?"const":a;case 3:return e.isFunctionExpression(i)?"method":"property";case 4:return"property";case 5:return e.isFunctionExpression(i)?"method":"property";case 6:return"local class";default:return e.assertType(r),""}case 79:return e.isImportClause(n.parent)?"alias":"";case 269:var o=M(n.expression);return""===o?"const":o;default:return""}function s(n){return e.isVarConst(n)?"const":e.isLet(n)?"let":"var"}}function F(n){switch(n.kind){case 108:return!0;case 79:return e.identifierIsThisKeyword(n)&&162===n.parent.kind;default:return!1}}e.scanner=e.createScanner(99,!0),function(e){e[e["None"]=0]="None",e[e["Value"]=1]="Value",e[e["Type"]=2]="Type",e[e["Namespace"]=4]="Namespace",e[e["All"]=7]="All"}(e.SemanticMeaning||(e.SemanticMeaning={})),e.getMeaningFromDeclaration=n,e.getMeaningFromLocation=t,e.isInRightSideOfInternalImportEqualsDeclaration=i,e.isCallExpressionTarget=u,e.isNewExpressionTarget=d,e.isCallOrNewExpressionTarget=p,e.isTaggedTemplateTag=f,e.isDecoratorTarget=m,e.isJsxOpeningLikeElementTagName=_,e.climbPastPropertyAccess=b,e.climbPastPropertyOrElementAccess=E,e.getTargetLabel=T,e.hasPropertyAccessExpressionWithName=S,e.isJumpStatementTarget=x,e.isLabelOfLabeledStatement=L,e.isLabelName=D,e.isTagName=A,e.isRightSideOfQualifiedName=C,e.isRightSideOfPropertyAccess=I,e.isArgumentExpressionOfElementAccess=N,e.isNameOfModuleDeclaration=k,e.isNameOfFunctionDeclaration=R,e.isLiteralNameOfPropertyDeclarationOrIndexAccess=O,e.isExpressionOfExternalModuleImportEqualsDeclaration=w,e.getContainerNode=P,e.getNodeKind=M,e.isThis=F;var G=/^\/\/\/\s*</;function B(n,t){var r=e.getLineStarts(t),i=t.getLineAndCharacterOfPosition(n).line;return r[i]}function U(e,n){return H(e.pos,e.end,n)}function V(e,n){return K(e,n.pos)&&K(e,n.end)}function j(e,n){return e.pos<=n&&n<=e.end}function K(e,n){return e.pos<n&&n<e.end}function H(e,n,t){return e<=t.pos&&n>=t.end}function W(e,n,t){return e.pos<=n&&e.end>=t}function z(e,n,t){return q(e.pos,e.end,n,t)}function J(e,n,t,r){return q(e.getStart(n),e.end,t,r)}function q(e,n,t,r){var i=Math.max(e,t),a=Math.min(n,r);return i<a}function X(n,t,r){return e.Debug.assert(n.pos<=t),t<n.end||!Y(n,r)}function Y(n,t){if(void 0===n||e.nodeIsMissing(n))return!1;switch(n.kind){case 255:case 256:case 258:case 203:case 199:case 180:case 233:case 260:case 261:case 267:case 271:return Q(n,19,t);case 290:return Y(n.block,t);case 207:if(!n.arguments)return!0;case 206:case 210:case 189:return Q(n,21,t);case 177:case 178:return Y(n.type,t);case 169:case 170:case 171:case 254:case 211:case 167:case 166:case 173:case 172:case 212:return n.body?Y(n.body,t):n.type?Y(n.type,t):$(n,21,t);case 259:return!!n.body&&Y(n.body,t);case 237:return n.elseStatement?Y(n.elseStatement,t):Y(n.thenStatement,t);case 236:return Y(n.expression,t)||$(n,26,t);case 202:case 200:case 205:case 160:case 182:return Q(n,23,t);case 174:return n.type?Y(n.type,t):$(n,23,t);case 287:case 288:return!1;case 240:case 241:case 242:case 239:return Y(n.statement,t);case 238:return $(n,115,t)?Q(n,21,t):Y(n.statement,t);case 179:return Y(n.exprName,t);case 214:case 213:case 215:case 222:case 223:var r=n;return Y(r.expression,t);case 208:return Y(n.template,t);case 221:var i=e.lastOrUndefined(n.templateSpans);return Y(i,t);case 231:return e.nodeIsPresent(n.literal);case 270:case 264:return e.nodeIsPresent(n.moduleSpecifier);case 217:return Y(n.operand,t);case 219:return Y(n.right,t);case 220:return Y(n.whenFalse,t);default:return!0}}function Q(n,t,r){var i=n.getChildren(r);if(i.length){var a=e.last(i);if(a.kind===t)return!0;if(26===a.kind&&1!==i.length)return i[i.length-2].kind===t}return!1}function Z(n){var t=ne(n);if(t){var r=t.getChildren(),i=e.indexOfNode(r,n);return{listItemIndex:i,list:t}}}function $(e,n,t){return!!ee(e,n,t)}function ee(n,t,r){return e.find(n.getChildren(r),function(e){return e.kind===t})}function ne(n){var t=e.find(n.parent.getChildren(),function(t){return e.isSyntaxList(t)&&U(t,n)});return e.Debug.assert(!t||e.contains(t.getChildren(),n)),t}function te(e){return 88===e.kind}function re(e){return 84===e.kind}function ie(e){return 98===e.kind}function ae(n){if(e.isNamedDeclaration(n))return n.name;if(e.isClassDeclaration(n)){var t=n.modifiers&&e.find(n.modifiers,te);if(t)return t}if(e.isClassExpression(n)){var r=e.find(n.getChildren(),re);if(r)return r}}function oe(n){if(e.isNamedDeclaration(n))return n.name;if(e.isFunctionDeclaration(n)){var t=e.find(n.modifiers,te);if(t)return t}if(e.isFunctionExpression(n)){var r=e.find(n.getChildren(),ie);if(r)return r}}function se(n){var t;return e.findAncestor(n,function(n){return e.isTypeNode(n)&&(t=n),!e.isQualifiedName(n.parent)&&!e.isTypeNode(n.parent)&&!e.isTypeElement(n.parent)}),t}function ce(n,t){var r=t.getContextualType(n);if(r)return r;var i=n.parent;if(i&&e.isBinaryExpression(i)&&Kt(i.operatorToken.kind))return t.getTypeAtLocation(n===i.left?i.right:i.left);var a=se(n);return a&&t.getTypeAtLocation(a)}function le(n,t){if(!t)switch(n.kind){case 255:case 224:return ae(n);case 254:case 211:return oe(n)}if(e.isNamedDeclaration(n))return n.name}function ue(n,t){if(n.importClause){if(n.importClause.name&&n.importClause.namedBindings)return;if(n.importClause.name)return n.importClause.name;if(n.importClause.namedBindings){if(e.isNamedImports(n.importClause.namedBindings)){var r=e.singleOrUndefined(n.importClause.namedBindings.elements);if(!r)return;return r.name}if(e.isNamespaceImport(n.importClause.namedBindings))return n.importClause.namedBindings.name}}if(!t)return n.moduleSpecifier}function de(n,t){if(n.exportClause){if(e.isNamedExports(n.exportClause)){var r=e.singleOrUndefined(n.exportClause.elements);if(!r)return;return n.exportClause.elements[0].name}if(e.isNamespaceExport(n.exportClause))return n.exportClause.name}if(!t)return n.moduleSpecifier}function pe(e){if(1===e.types.length)return e.types[0].expression}function fe(n,t){var r=n.parent;if(e.isModifier(n)&&(t||88!==n.kind)?e.contains(r.modifiers,n):84===n.kind?e.isClassDeclaration(r)||e.isClassExpression(n):98===n.kind?e.isFunctionDeclaration(r)||e.isFunctionExpression(n):118===n.kind?e.isInterfaceDeclaration(r):92===n.kind?e.isEnumDeclaration(r):150===n.kind?e.isTypeAliasDeclaration(r):141===n.kind||140===n.kind?e.isModuleDeclaration(r):100===n.kind?e.isImportEqualsDeclaration(r):135===n.kind?e.isGetAccessorDeclaration(r):147===n.kind&&e.isSetAccessorDeclaration(r)){var i=le(r,t);if(i)return i}if((113===n.kind||85===n.kind||119===n.kind)&&e.isVariableDeclarationList(r)&&1===r.declarations.length){var a=r.declarations[0];if(e.isIdentifier(a.name))return a.name}if(150===n.kind){if(e.isImportClause(r)&&r.isTypeOnly){i=ue(r.parent,t);if(i)return i}if(e.isExportDeclaration(r)&&r.isTypeOnly){i=de(r,t);if(i)return i}}if(127===n.kind){if(e.isImportSpecifier(r)&&r.propertyName||e.isExportSpecifier(r)&&r.propertyName||e.isNamespaceImport(r)||e.isNamespaceExport(r))return r.name;if(e.isExportDeclaration(r)&&r.exportClause&&e.isNamespaceExport(r.exportClause))return r.exportClause.name}if(100===n.kind&&e.isImportDeclaration(r)){i=ue(r,t);if(i)return i}if(93===n.kind){if(e.isExportDeclaration(r)){i=de(r,t);if(i)return i}if(e.isExportAssignment(r))return e.skipOuterExpressions(r.expression)}if(144===n.kind&&e.isExternalModuleReference(r))return r.expression;if(154===n.kind&&(e.isImportDeclaration(r)||e.isExportDeclaration(r))&&r.moduleSpecifier)return r.moduleSpecifier;if((94===n.kind||117===n.kind)&&e.isHeritageClause(r)&&r.token===n.kind){i=pe(r);if(i)return i}if(94===n.kind){if(e.isTypeParameterDeclaration(r)&&r.constraint&&e.isTypeReferenceNode(r.constraint))return r.constraint.typeName;if(e.isConditionalTypeNode(r)&&e.isTypeReferenceNode(r.extendsType))return r.extendsType.typeName}if(136===n.kind&&e.isInferTypeNode(r))return r.typeParameter.name;if(101===n.kind&&e.isTypeParameterDeclaration(r)&&e.isMappedTypeNode(r.parent))return r.name;if(139===n.kind&&e.isTypeOperatorNode(r)&&139===r.operator&&e.isTypeReferenceNode(r.type))return r.type.typeName;if(143===n.kind&&e.isTypeOperatorNode(r)&&143===r.operator&&e.isArrayTypeNode(r.type)&&e.isTypeReferenceNode(r.type.elementType))return r.type.elementType.typeName;if(!t){if((103===n.kind&&e.isNewExpression(r)||114===n.kind&&e.isVoidExpression(r)||112===n.kind&&e.isTypeOfExpression(r)||131===n.kind&&e.isAwaitExpression(r)||125===n.kind&&e.isYieldExpression(r)||89===n.kind&&e.isDeleteExpression(r))&&r.expression)return e.skipOuterExpressions(r.expression);if((101===n.kind||102===n.kind)&&e.isBinaryExpression(r)&&r.operatorToken===n)return e.skipOuterExpressions(r.right);if(127===n.kind&&e.isAsExpression(r)&&e.isTypeReferenceNode(r.type))return r.type.typeName;if(101===n.kind&&e.isForInStatement(r)||158===n.kind&&e.isForOfStatement(r))return e.skipOuterExpressions(r.expression)}return n}function me(e){return fe(e,!1)}function _e(e){return fe(e,!0)}function ge(n,t){return ye(n,t,function(n){return e.isPropertyNameLiteral(n)||e.isKeyword(n.kind)||e.isPrivateIdentifier(n)})}function ye(e,n,t){return ve(e,n,!1,t,!1)}function he(e,n){return ve(e,n,!0,void 0,!1)}function ve(e,n,t,r,i){var a=e;e:while(1){for(var o=0,s=a.getChildren(e);o<s.length;o++){var c=s[o],l=t?c.getFullStart():c.getStart(e,!0);if(l>n)break;var u=c.getEnd();if(n<u||n===u&&(1===c.kind||i)){a=c;continue e}if(r&&u===n){var d=Se(n,e,c);if(d&&r(d))return d}}return a}}function be(e,n){var t=he(e,n);while(Ie(t)){var r=Te(t,t.parent,e);if(!r)return;t=r}return t}function Ee(n,t){var r=he(n,t);return e.isToken(r)&&t>r.getStart(n)&&t<r.getEnd()?r:Se(t,n)}function Te(n,t,r){return i(t);function i(t){return e.isToken(t)&&t.pos===n.end?t:e.firstDefined(t.getChildren(r),function(e){var t=e.pos<=n.pos&&e.end>n.end||e.pos===n.end;return t&&Ue(e,r)?i(e):void 0})}}function Se(n,t,r,i){var a=o(r||t);return e.Debug.assert(!(a&&Ie(a))),a;function o(a){if(xe(a)&&1!==a.kind)return a;var s=a.getChildren(t),c=e.binarySearchKey(s,n,function(e,n){return n},function(e,t){return n<s[e].end?!s[e-1]||n>=s[e-1].end?0:1:-1});if(c>=0&&s[c]){var l=s[c];if(n<l.end){var u=l.getStart(t,!i),d=u>=n||!Ue(l,t)||Ie(l);if(d){var p=De(s,c,t);return p&&Le(p,t)}return o(l)}}e.Debug.assert(void 0!==r||300===a.kind||1===a.kind||e.isJSDocCommentContainingNode(a));var f=De(s,s.length,t);return f&&Le(f,t)}}function xe(n){return e.isToken(n)&&!Ie(n)}function Le(e,n){if(xe(e))return e;var t=e.getChildren(n);if(0===t.length)return e;var r=De(t,t.length,n);return r&&Le(r,n)}function De(n,t,r){for(var i=t-1;i>=0;i--){var a=n[i];if(Ie(a))e.Debug.assert(i>0,"`JsxText` tokens should not be the first child of `JsxElement | JsxSelfClosingElement`");else if(Ue(n[i],r))return n[i]}}function Ae(n,t,r){if(void 0===r&&(r=Se(t,n)),r&&e.isStringTextContainingNode(r)){var i=r.getStart(n),a=r.getEnd();if(i<t&&t<a)return!0;if(t===a)return!!r.isUnterminated}return!1}function Ce(e,n){var t=he(e,n);return!!t&&(11===t.kind||(29===t.kind&&11===t.parent.kind||(29===t.kind&&286===t.parent.kind||(!(!t||19!==t.kind||286!==t.parent.kind)||29===t.kind&&279===t.parent.kind))))}function Ie(n){return e.isJsxText(n)&&n.containsOnlyTriviaWhiteSpaces}function Ne(n,t){var r=he(n,t);return e.isTemplateLiteralKind(r.kind)&&t>r.getStart(n)}function ke(n,t){var r=he(n,t);return!!e.isJsxText(r)||(!(18!==r.kind||!e.isJsxExpression(r.parent)||!e.isJsxElement(r.parent.parent))||!(29!==r.kind||!e.isJsxOpeningLikeElement(r.parent)||!e.isJsxElement(r.parent.parent)))}function Re(e,n){function t(t){while(t)if(t.kind>=277&&t.kind<=286||11===t.kind||29===t.kind||31===t.kind||79===t.kind||19===t.kind||18===t.kind||43===t.kind)t=t.parent;else{if(276!==t.kind)return!1;if(n>t.getStart(e))return!0;t=t.parent}return!1}return t(he(e,n))}function Oe(n,t,r){var i=e.tokenToString(n.kind),a=e.tokenToString(t),o=n.getFullStart(),s=r.text.lastIndexOf(a,o);if(-1!==s){if(r.text.lastIndexOf(i,o-1)<s){var c=Se(s+1,r);if(c&&c.kind===t)return c}var l=n.kind,u=0;while(1){var d=Se(n.getFullStart(),r);if(!d)return;if(n=d,n.kind===t){if(0===u)return n;u--}else n.kind===l&&u++}}}function we(e,n,t){return n?e.getNonNullableType():t?e.getNonOptionalType():e}function Pe(n,t,r){var i=Fe(n,t);return void 0!==i&&(e.isPartOfTypeNode(i.called)||0!==Me(i.called,i.nTypeArguments,r).length||Pe(i.called,t,r))}function Me(n,t,r){var i=r.getTypeAtLocation(n);e.isOptionalChain(n.parent)&&(i=we(i,e.isOptionalChainRoot(n.parent),!0));var a=e.isNewExpression(n.parent)?i.getConstructSignatures():i.getCallSignatures();return a.filter(function(e){return!!e.typeParameters&&e.typeParameters.length>=t})}function Fe(n,t){if(-1!==t.text.lastIndexOf("<",n?n.pos:t.text.length)){var r=n,i=0,a=0;while(r){switch(r.kind){case 29:if(r=Se(r.getFullStart(),t),r&&28===r.kind&&(r=Se(r.getFullStart(),t)),!r||!e.isIdentifier(r))return;if(!i)return e.isDeclarationName(r)?void 0:{called:r,nTypeArguments:a};i--;break;case 49:i=3;break;case 48:i=2;break;case 31:i++;break;case 19:if(r=Oe(r,18,t),!r)return;break;case 21:if(r=Oe(r,20,t),!r)return;break;case 23:if(r=Oe(r,22,t),!r)return;break;case 27:a++;break;case 38:case 79:case 10:case 8:case 9:case 110:case 95:case 112:case 94:case 139:case 24:case 51:case 57:case 58:break;default:if(e.isTypeNode(r))break;return}r=Se(r.getFullStart(),t)}}}function Ge(n,t,r){return e.formatting.getRangeOfEnclosingComment(n,t,void 0,r)}function Be(n,t){var r=he(n,t);return!!e.findAncestor(r,e.isJSDoc)}function Ue(e,n){return 1===e.kind?!!e.jsDoc:0!==e.getWidth(n)}function Ve(n,t){void 0===t&&(t=0);var r=[],i=e.isDeclaration(n)?e.getCombinedNodeFlagsAlwaysIncludeJSDoc(n)&~t:0;return 8&i&&r.push("private"),16&i&&r.push("protected"),4&i&&r.push("public"),(32&i||e.isClassStaticBlockDeclaration(n))&&r.push("static"),128&i&&r.push("abstract"),1&i&&r.push("export"),8192&i&&r.push("deprecated"),8388608&n.flags&&r.push("declare"),269===n.kind&&r.push("export"),r.length>0?r.join(","):""}function je(n){return 176===n.kind||206===n.kind?n.typeArguments:e.isFunctionLike(n)||255===n.kind||256===n.kind?n.typeParameters:void 0}function Ke(e){return 2===e||3===e}function He(n){return!(10!==n&&13!==n&&!e.isTemplateLiteralKind(n))}function We(e){return 18<=e&&e<=78}function ze(n,t,r){return e.isTemplateLiteralKind(n.kind)&&n.getStart(r)<t&&t<n.end||!!n.isUnterminated&&t===n.end}function Je(e){switch(e){case 123:case 121:case 122:return!0}return!1}function qe(n){var t=e.clone(n);return e.setConfigFileInOptions(t,n&&n.configFile),t}function Xe(e){if(202===e.kind||203===e.kind){if(219===e.parent.kind&&e.parent.left===e&&63===e.parent.operatorToken.kind)return!0;if(242===e.parent.kind&&e.parent.initializer===e)return!0;if(Xe(291===e.parent.kind?e.parent.parent:e.parent))return!0}return!1}function Ye(e,n){return Ze(e,n,!0)}function Qe(e,n){return Ze(e,n,!1)}function Ze(e,n,t){var r=Ge(e,n,void 0);return!!r&&t===G.test(e.text.substring(r.pos,r.end))}function $e(e){if(e)switch(e.kind){case 10:case 14:return nn(e);default:return en(e)}}function en(n,t,r){return e.createTextSpanFromBounds(n.getStart(t),(r||n).getEnd())}function nn(n){if(!n.isUnterminated)return e.createTextSpanFromBounds(n.getStart()+1,n.getEnd()-1)}function tn(n,t){return e.createRange(n.getStart(t),n.end)}function rn(n){return e.createTextSpanFromBounds(n.pos,n.end)}function an(n){return e.createRange(n.start,n.start+n.length)}function on(n,t,r){return sn(e.createTextSpan(n,t),r)}function sn(e,n){return{span:e,newText:n}}function cn(n){return e.contains(e.typeKeywords,n)}function ln(e){return 150===e.kind}function un(e){return!!(1536&e.flags)&&34===e.name.charCodeAt(0)}function dn(){var n=[];return function(t){var r=e.getNodeId(t);return!n[r]&&(n[r]=!0)}}function pn(e){return e.getText(0,e.getLength())}function fn(e,n){for(var t="",r=0;r<n;r++)t+=e;return t}function mn(e){return e.isTypeParameter()&&e.getConstraint()||e}function _n(n){return 160===n.kind?e.isStringOrNumericLiteralLike(n.expression)?n.expression.text:void 0:e.isPrivateIdentifier(n)?e.idText(n):e.getTextOfIdentifierOrLiteral(n)}function gn(e){return e.getSourceFiles().some(function(n){return!n.isDeclarationFile&&!e.isSourceFileFromExternalLibrary(n)&&!(!n.externalModuleIndicator&&!n.commonJsModuleIndicator)})}function yn(e){return e.getSourceFiles().some(function(n){return!n.isDeclarationFile&&!e.isSourceFileFromExternalLibrary(n)&&!!n.externalModuleIndicator})}function hn(e){return!!e.module||e.target>=2||!!e.noEmit}function vn(n,t){return{fileExists:function(e){return n.fileExists(e)},getCurrentDirectory:function(){return t.getCurrentDirectory()},readFile:e.maybeBind(t,t.readFile),useCaseSensitiveFileNames:e.maybeBind(t,t.useCaseSensitiveFileNames),getSymlinkCache:e.maybeBind(t,t.getSymlinkCache)||n.getSymlinkCache,getModuleSpecifierCache:e.maybeBind(t,t.getModuleSpecifierCache),getGlobalTypingsCacheLocation:e.maybeBind(t,t.getGlobalTypingsCacheLocation),redirectTargetsMap:n.redirectTargetsMap,getProjectReferenceRedirect:function(e){return n.getProjectReferenceRedirect(e)},isSourceOfProjectReferenceRedirect:function(e){return n.isSourceOfProjectReferenceRedirect(e)},getNearestAncestorDirectoryWithPackageJson:e.maybeBind(t,t.getNearestAncestorDirectoryWithPackageJson),getFileIncludeReasons:function(){return n.getFileIncludeReasons()}}}function bn(e,n){return o(o({},vn(e,n)),{getCommonSourceDirectory:function(){return e.getCommonSourceDirectory()}})}function En(e,n,t,r){return e||n&&n.length?Tn(e,n,t,r):void 0}function Tn(n,t,r,i,a){return e.factory.createImportDeclaration(void 0,void 0,n||t?e.factory.createImportClause(!!a,n,t&&t.length?e.factory.createNamedImports(t):void 0):void 0,"string"===typeof r?Sn(r,i):r)}function Sn(n,t){return e.factory.createStringLiteral(n,0===t)}function xn(n,t){return e.isStringDoubleQuoted(n,t)?1:0}function Ln(n,t){if(t.quotePreference&&"auto"!==t.quotePreference)return"single"===t.quotePreference?0:1;var r=n.imports&&e.find(n.imports,function(n){return e.isStringLiteral(n)&&!e.nodeIsSynthesized(n.parent)});return r?xn(r,n):1}function Dn(n){switch(n){case 0:return"'";case 1:return'"';default:return e.Debug.assertNever(n)}}function An(n){var t=Cn(n);return void 0===t?void 0:e.unescapeLeadingUnderscores(t)}function Cn(n){return"default"!==n.escapedName?n.escapedName:e.firstDefined(n.declarations,function(n){var t=e.getNameOfDeclaration(n);return t&&79===t.kind?t.escapedText:void 0})}function In(n){return e.isStringLiteralLike(n)&&(e.isExternalModuleReference(n.parent)||e.isImportDeclaration(n.parent)||e.isRequireCall(n.parent,!1)&&n.parent.arguments[0]===n||e.isImportCall(n.parent)&&n.parent.arguments[0]===n)}function Nn(n){return e.isBindingElement(n)&&e.isObjectBindingPattern(n.parent)&&e.isIdentifier(n.name)&&!n.propertyName}function kn(e,n){var t=e.getTypeAtLocation(n.parent);return t&&e.getPropertyOfType(t,n.name.text)}function Rn(n,t,r){if(n)while(n.parent){if(e.isSourceFile(n.parent)||!On(r,n.parent,t))return n;n=n.parent}}function On(n,t,r){return e.textSpanContainsPosition(n,t.getStart(r))&&t.getEnd()<=e.textSpanEnd(n)}function wn(n,t){return n.modifiers&&e.find(n.modifiers,function(e){return e.kind===t})}function Pn(n,t,r,i){var a=e.isArray(r)?r[0]:r,o=235===a.kind?e.isRequireVariableStatement:e.isAnyImportSyntax,s=e.filter(t.statements,o),c=e.isArray(r)?e.stableSort(r,e.OrganizeImports.compareImportsOrRequireStatements):[r];if(s.length)if(s&&e.OrganizeImports.importsAreSorted(s))for(var l=0,u=c;l<u.length;l++){var d=u[l],p=e.OrganizeImports.getImportDeclarationInsertionIndex(s,d);if(0===p){var f=s[0]===t.statements[0]?{leadingTriviaOption:e.textChanges.LeadingTriviaOption.Exclude}:{};n.insertNodeBefore(t,s[0],d,!1,f)}else{var m=s[p-1];n.insertNodeAfter(t,m,d)}}else{var _=e.lastOrUndefined(s);_?n.insertNodesAfter(t,_,c):n.insertNodesAtTopOfFile(t,c,i)}else n.insertNodesAtTopOfFile(t,c,i)}function Mn(n,t){return e.Debug.assert(n.isTypeOnly),e.cast(n.getChildAt(0,t),ln)}function Fn(e,n){return!!e&&!!n&&e.start===n.start&&e.length===n.length}function Gn(e,n){return e.fileName===n.fileName&&Fn(e.textSpan,n.textSpan)}function Bn(e,n){if(e)for(var t=0;t<e.length;t++)if(e.indexOf(e[t])===t){var r=n(e[t],t);if(r)return r}}function Un(n,t,r){for(var i=t;i<r;i++)if(!e.isWhiteSpaceLike(n.charCodeAt(i)))return!1;return!0}function Vn(e){return e.declarations&&e.declarations.length>0&&162===e.declarations[0].kind}e.getLineStartPositionForPosition=B,e.rangeContainsRange=U,e.rangeContainsRangeExclusive=V,e.rangeContainsPosition=j,e.rangeContainsPositionExclusive=K,e.startEndContainsRange=H,e.rangeContainsStartEnd=W,e.rangeOverlapsWithStartEnd=z,e.nodeOverlapsWithStartEnd=J,e.startEndOverlapsWithStartEnd=q,e.positionBelongsToNode=X,e.findListItemInfo=Z,e.hasChildOfKind=$,e.findChildOfKind=ee,e.findContainingList=ne,e.getContextualTypeOrAncestorTypeNodeType=ce,e.getAdjustedReferenceLocation=me,e.getAdjustedRenameLocation=_e,e.getTouchingPropertyName=ge,e.getTouchingToken=ye,e.getTokenAtPosition=he,e.findFirstNonJsxWhitespaceToken=be,e.findTokenOnLeftOfPosition=Ee,e.findNextToken=Te,e.findPrecedingToken=Se,e.isInString=Ae,e.isInsideJsxElementOrAttribute=Ce,e.isInTemplateString=Ne,e.isInJSXText=ke,e.isInsideJsxElement=Re,e.findPrecedingMatchingToken=Oe,e.removeOptionality=we,e.isPossiblyTypeArgumentPosition=Pe,e.getPossibleGenericSignatures=Me,e.getPossibleTypeArgumentsInfo=Fe,e.isInComment=Ge,e.hasDocComment=Be,e.getNodeModifiers=Ve,e.getTypeArgumentOrTypeParameterList=je,e.isComment=Ke,e.isStringOrRegularExpressionOrTemplateLiteral=He,e.isPunctuation=We,e.isInsideTemplateLiteral=ze,e.isAccessibilityModifier=Je,e.cloneCompilerOptions=qe,e.isArrayLiteralOrObjectLiteralDestructuringPattern=Xe,e.isInReferenceComment=Ye,e.isInNonReferenceComment=Qe,e.getReplacementSpanForContextToken=$e,e.createTextSpanFromNode=en,e.createTextSpanFromStringLiteralLikeContent=nn,e.createTextRangeFromNode=tn,e.createTextSpanFromRange=rn,e.createTextRangeFromSpan=an,e.createTextChangeFromStartLength=on,e.createTextChange=sn,e.typeKeywords=[129,128,156,132,95,136,139,142,104,145,146,143,148,149,110,114,151,152,153],e.isTypeKeyword=cn,e.isTypeKeywordToken=ln,e.isExternalModuleSymbol=un,e.nodeSeenTracker=dn,e.getSnapshotText=pn,e.repeatString=fn,e.skipConstraint=mn,e.getNameFromPropertyName=_n,e.programContainsModules=gn,e.programContainsEs6Modules=yn,e.compilerOptionsIndicateEs6Modules=hn,e.createModuleSpecifierResolutionHost=vn,e.getModuleSpecifierResolverHost=bn,e.makeImportIfNecessary=En,e.makeImport=Tn,e.makeStringLiteral=Sn,function(e){e[e["Single"]=0]="Single",e[e["Double"]=1]="Double"}(e.QuotePreference||(e.QuotePreference={})),e.quotePreferenceFromString=xn,e.getQuotePreference=Ln,e.getQuoteFromPreference=Dn,e.symbolNameNoDefault=An,e.symbolEscapedNameNoDefault=Cn,e.isModuleSpecifierLike=In,e.isObjectBindingElementWithoutPropertyName=Nn,e.getPropertySymbolFromBindingElement=kn,e.getParentNodeInSpan=Rn,e.findModifier=wn,e.insertImports=Pn,e.getTypeKeywordOfTypeOnlyImport=Mn,e.textSpansEqual=Fn,e.documentSpansEqual=Gn,e.forEachUnique=Bn,e.isTextWhiteSpaceLike=Un,e.isFirstDeclarationOfSymbolParameter=Vn;var jn=Kn();function Kn(){var n,t,r,i,a=10*e.defaultMaximumTruncationLength;d();var o=function(n){return c(n,e.SymbolDisplayPartKind.text)};return{displayParts:function(){var t=n.length&&n[n.length-1].text;return i>a&&t&&"..."!==t&&(e.isWhiteSpaceLike(t.charCodeAt(t.length-1))||n.push(Wn(" ",e.SymbolDisplayPartKind.space)),n.push(Wn("...",e.SymbolDisplayPartKind.punctuation))),n},writeKeyword:function(n){return c(n,e.SymbolDisplayPartKind.keyword)},writeOperator:function(n){return c(n,e.SymbolDisplayPartKind.operator)},writePunctuation:function(n){return c(n,e.SymbolDisplayPartKind.punctuation)},writeTrailingSemicolon:function(n){return c(n,e.SymbolDisplayPartKind.punctuation)},writeSpace:function(n){return c(n,e.SymbolDisplayPartKind.space)},writeStringLiteral:function(n){return c(n,e.SymbolDisplayPartKind.stringLiteral)},writeParameter:function(n){return c(n,e.SymbolDisplayPartKind.parameterName)},writeProperty:function(n){return c(n,e.SymbolDisplayPartKind.propertyName)},writeLiteral:function(n){return c(n,e.SymbolDisplayPartKind.stringLiteral)},writeSymbol:l,writeLine:u,write:o,writeComment:o,getText:function(){return""},getTextPos:function(){return 0},getColumn:function(){return 0},getLine:function(){return 0},isAtStartOfLine:function(){return!1},hasTrailingWhitespace:function(){return!1},hasTrailingComment:function(){return!1},rawWrite:e.notImplemented,getIndent:function(){return r},increaseIndent:function(){r++},decreaseIndent:function(){r--},clear:d,trackSymbol:function(){return!1},reportInaccessibleThisError:e.noop,reportInaccessibleUniqueSymbolError:e.noop,reportPrivateInBaseOfClassExpression:e.noop};function s(){if(!(i>a)&&t){var o=e.getIndentString(r);o&&(i+=o.length,n.push(Wn(o,e.SymbolDisplayPartKind.space))),t=!1}}function c(e,t){i>a||(s(),i+=e.length,n.push(Wn(e,t)))}function l(e,t){i>a||(s(),i+=e.length,n.push(Hn(e,t)))}function u(){i>a||(i+=1,n.push(ct()),t=!0)}function d(){n=[],t=!0,r=0,i=0}}function Hn(n,t){return Wn(n,r(t));function r(n){var t=n.flags;return 3&t?Vn(n)?e.SymbolDisplayPartKind.parameterName:e.SymbolDisplayPartKind.localName:4&t?e.SymbolDisplayPartKind.propertyName:32768&t?e.SymbolDisplayPartKind.propertyName:65536&t?e.SymbolDisplayPartKind.propertyName:8&t?e.SymbolDisplayPartKind.enumMemberName:16&t?e.SymbolDisplayPartKind.functionName:32&t?e.SymbolDisplayPartKind.className:64&t?e.SymbolDisplayPartKind.interfaceName:384&t?e.SymbolDisplayPartKind.enumName:1536&t?e.SymbolDisplayPartKind.moduleName:8192&t?e.SymbolDisplayPartKind.methodName:262144&t?e.SymbolDisplayPartKind.typeParameterName:524288&t?e.SymbolDisplayPartKind.aliasName:2097152&t?e.SymbolDisplayPartKind.aliasName:e.SymbolDisplayPartKind.text}}function Wn(n,t){return{text:n,kind:e.SymbolDisplayPartKind[t]}}function zn(){return Wn(" ",e.SymbolDisplayPartKind.space)}function Jn(n){return Wn(e.tokenToString(n),e.SymbolDisplayPartKind.keyword)}function qn(n){return Wn(e.tokenToString(n),e.SymbolDisplayPartKind.punctuation)}function Xn(n){return Wn(e.tokenToString(n),e.SymbolDisplayPartKind.operator)}function Yn(n){return Wn(n,e.SymbolDisplayPartKind.parameterName)}function Qn(n){return Wn(n,e.SymbolDisplayPartKind.propertyName)}function Zn(n){var t=e.stringToToken(n);return void 0===t?$n(n):Jn(t)}function $n(n){return Wn(n,e.SymbolDisplayPartKind.text)}function et(n){return Wn(n,e.SymbolDisplayPartKind.aliasName)}function nt(n){return Wn(n,e.SymbolDisplayPartKind.typeParameterName)}function tt(n){return Wn(n,e.SymbolDisplayPartKind.linkText)}function rt(n,t){return{text:e.getTextOfNode(n),kind:e.SymbolDisplayPartKind[e.SymbolDisplayPartKind.linkName],target:{fileName:e.getSourceFileOfNode(t).fileName,textSpan:en(t)}}}function it(n){return Wn(n,e.SymbolDisplayPartKind.link)}function at(n,t){var r,i=e.isJSDocLink(n)?"link":e.isJSDocLinkCode(n)?"linkcode":"linkplain",a=[it("{@"+i+" ")];if(n.name){var o=null===t||void 0===t?void 0:t.getSymbolAtLocation(n.name),s=(null===o||void 0===o?void 0:o.valueDeclaration)||(null===(r=null===o||void 0===o?void 0:o.declarations)||void 0===r?void 0:r[0]);s?(a.push(rt(n.name,s)),n.text&&a.push(tt(n.text))):a.push(tt(e.getTextOfNode(n.name)+" "+n.text))}else n.text&&a.push(tt(n.text));return a.push(it("}")),a}e.symbolPart=Hn,e.displayPart=Wn,e.spacePart=zn,e.keywordPart=Jn,e.punctuationPart=qn,e.operatorPart=Xn,e.parameterNamePart=Yn,e.propertyNamePart=Qn,e.textOrKeywordPart=Zn,e.textPart=$n,e.typeAliasNamePart=et,e.typeParameterNamePart=nt,e.linkTextPart=tt,e.linkNamePart=rt,e.linkPart=it,e.buildLinkParts=at;var ot="\r\n";function st(e,n){var t;return(null===n||void 0===n?void 0:n.newLineCharacter)||(null===(t=e.getNewLine)||void 0===t?void 0:t.call(e))||ot}function ct(){return Wn("\n",e.SymbolDisplayPartKind.lineBreak)}function lt(e){try{return e(jn),jn.displayParts()}finally{jn.clear()}}function ut(e,n,t,r){return void 0===r&&(r=0),lt(function(i){e.writeType(n,t,17408|r,i)})}function dt(e,n,t,r,i){return void 0===i&&(i=0),lt(function(a){e.writeSymbol(n,t,r,8|i,a)})}function pt(e,n,t,r){return void 0===r&&(r=0),r|=25632,lt(function(i){e.writeSignature(n,t,r,void 0,i)})}function ft(n){return!!n.parent&&e.isImportOrExportSpecifier(n.parent)&&n.parent.propertyName===n}function mt(n,t){return e.ensureScriptKind(n,t.getScriptKind&&t.getScriptKind(n))}function _t(n,t){var r=n;while(yt(r)||gt(r)&&r.target)r=gt(r)&&r.target?r.target:e.skipAlias(r,t);return r}function gt(e){return 0!==(33554432&e.flags)}function yt(e){return 0!==(2097152&e.flags)}function ht(n,t){return e.getSymbolId(e.skipAlias(n,t))}function vt(n,t){while(e.isWhiteSpaceLike(n.charCodeAt(t)))t+=1;return t}function bt(n,t){while(t>-1&&e.isWhiteSpaceSingleLine(n.charCodeAt(t)))t-=1;return t+1}function Et(e,n){void 0===n&&(n=!0);var t=e&&St(e);return t&&!n&&Dt(t),t}function Tt(n,t,r){var i=r(n);return i?e.setOriginalNode(i,n):i=St(n,r),i&&!t&&Dt(i),i}function St(n,t){var r=t?function(e){return Tt(e,!0,t)}:Et,i=t?function(e){return e&&Lt(e,!0,t)}:function(e){return e&&xt(e)},a=e.visitEachChild(n,r,e.nullTransformationContext,i,r);if(a===n){var o=e.isStringLiteral(n)?e.setOriginalNode(e.factory.createStringLiteralFromNode(n),n):e.isNumericLiteral(n)?e.setOriginalNode(e.factory.createNumericLiteral(n.text,n.numericLiteralFlags),n):e.factory.cloneNode(n);return e.setTextRange(o,n)}return a.parent=void 0,a}function xt(n,t){return void 0===t&&(t=!0),n&&e.factory.createNodeArray(n.map(function(e){return Et(e,t)}),n.hasTrailingComma)}function Lt(n,t,r){return e.factory.createNodeArray(n.map(function(e){return Tt(e,t,r)}),n.hasTrailingComma)}function Dt(e){At(e),Ct(e)}function At(e){kt(e,512,Rt)}function Ct(n){kt(n,1024,e.getLastChild)}function It(e,n){var t=e.getSourceFile(),r=t.text;Nt(e,r)?Pt(e,n,t):Ft(e,n,t),Mt(e,n,t)}function Nt(e,n){for(var t=e.getFullStart(),r=e.getStart(),i=t;i<r;i++)if(10===n.charCodeAt(i))return!0;return!1}function kt(n,t,r){e.addEmitFlags(n,t);var i=r(n);i&&kt(i,t,r)}function Rt(e){return e.forEachChild(function(e){return e})}function Ot(n,t){for(var r=n,i=1;!e.isFileLevelUniqueName(t,r);i++)r=n+"_"+i;return r}function wt(n,t,r,i){for(var a=0,o=-1,s=0,c=n;s<c.length;s++){var l=c[s],u=l.fileName,d=l.textChanges;e.Debug.assert(u===t);for(var p=0,f=d;p<f.length;p++){var m=f[p],_=m.span,g=m.newText,y=Bt(g,r);if(-1!==y&&(o=_.start+a+y,!i))return o;a+=g.length-_.length}}return e.Debug.assert(i),e.Debug.assert(o>=0),o}function Pt(n,t,r,i,a){e.forEachLeadingCommentRange(r.text,n.pos,Gt(t,r,i,a,e.addSyntheticLeadingComment))}function Mt(n,t,r,i,a){e.forEachTrailingCommentRange(r.text,n.end,Gt(t,r,i,a,e.addSyntheticTrailingComment))}function Ft(n,t,r,i,a){e.forEachTrailingCommentRange(r.text,n.pos,Gt(t,r,i,a,e.addSyntheticLeadingComment))}function Gt(e,n,t,r,i){return function(a,o,s,c){3===s?(a+=2,o-=2):a+=2,i(e,t||s,n.text.slice(a,o),void 0!==r?r:c)}}function Bt(n,t){if(e.startsWith(n,t))return 0;var r=n.indexOf(" "+t);return-1===r&&(r=n.indexOf("."+t)),-1===r&&(r=n.indexOf('"'+t)),-1===r?-1:r+1}function Ut(n){return e.isBinaryExpression(n)&&27===n.operatorToken.kind||e.isObjectLiteralExpression(n)||e.isAsExpression(n)&&e.isObjectLiteralExpression(n.expression)}function Vt(e,n){var t=e.parent;switch(t.kind){case 207:return n.getContextualType(t);case 219:var r=t,i=r.left,a=r.operatorToken,o=r.right;return Kt(a.kind)?n.getTypeAtLocation(e===o?i:o):n.getContextualType(e);case 287:return t.expression===e?zt(t,n):void 0;default:return n.getContextualType(e)}}function jt(n,t,r){var i=Ln(n,t),a=JSON.stringify(r);return 0===i?"'"+e.stripQuotes(a).replace(/'/g,"\\'").replace(/\\"/g,'"')+"'":a}function Kt(e){switch(e){case 36:case 34:case 37:case 35:return!0;default:return!1}}function Ht(e){switch(e.kind){case 10:case 14:case 221:case 208:return!0;default:return!1}}function Wt(e){return!!e.getStringIndexType()||!!e.getNumberIndexType()}function zt(e,n){return n.getTypeAtLocation(e.parent.parent.expression)}function Jt(e,n,t,r){var i=t.getTypeChecker(),a=!0,o=function(){a=!1},s=i.typeToTypeNode(e,n,1,{trackSymbol:function(e,n,t){return a=a&&0===i.isSymbolAccessible(e,n,t,!1).accessibility,!a},reportInaccessibleThisError:o,reportPrivateInBaseOfClassExpression:o,reportInaccessibleUniqueSymbolError:o,moduleResolverHost:bn(t,r)});return a?s:void 0}function qt(e){return 172===e||173===e||174===e||164===e||166===e}function Xt(e){return 254===e||169===e||167===e||170===e||171===e}function Yt(e){return 259===e}function Qt(e){return 235===e||236===e||238===e||243===e||244===e||245===e||249===e||251===e||165===e||257===e||264===e||263===e||270===e||262===e||269===e}function Zt(n,t){var r=n.getLastToken(t);if(r&&26===r.kind)return!1;if(qt(n.kind)){if(r&&27===r.kind)return!1}else if(Yt(n.kind)){var i=e.last(n.getChildren(t));if(i&&e.isModuleBlock(i))return!1}else if(Xt(n.kind)){i=e.last(n.getChildren(t));if(i&&e.isFunctionBlock(i))return!1}else if(!Qt(n.kind))return!1;if(238===n.kind)return!0;var a=e.findAncestor(n,function(e){return!e.parent}),o=Te(n,a,t);if(!o||19===o.kind)return!0;var s=t.getLineAndCharacterOfPosition(n.getEnd()).line,c=t.getLineAndCharacterOfPosition(o.getStart(t)).line;return s!==c}function $t(n,t,r){var i=e.findAncestor(t,function(t){return t.end!==n?"quit":e.syntaxMayBeASICandidate(t.kind)});return!!i&&Zt(i,r)}function er(n){var t=0,r=0,i=5;return e.forEachChild(n,function a(o){if(Qt(o.kind)){var s=o.getLastToken(n);s&&26===s.kind?t++:r++}return t+r>=i||e.forEachChild(o,a)}),0===t&&r<=1||t/r>1/i}function nr(e,n){return or(e,e.getDirectories,n)||[]}function tr(n,t,r,i,a){return or(n,n.readDirectory,t,r,i,a)||e.emptyArray}function rr(e,n){return or(e,e.fileExists,n)}function ir(n,t){return ar(function(){return e.directoryProbablyExists(t,n)})||!1}function ar(e){try{return e()}catch(n){return}}function or(e,n){for(var t=[],r=2;r<arguments.length;r++)t[r-2]=arguments[r];return ar(function(){return n&&n.apply(e,t)})}function sr(n,t,r){var i=[];return e.forEachAncestorDirectory(n,function(n){if(n===r)return!0;var a=e.combinePaths(n,"package.json");rr(t,a)&&i.push(a)}),i}function cr(n,t){var r;return e.forEachAncestorDirectory(n,function(n){return"node_modules"===n||(r=e.findConfigFile(n,function(e){return rr(t,e)},"package.json"),!!r||void 0)}),r}function lr(n,t){if(!t.fileExists)return[];var r=[];return e.forEachAncestorDirectory(e.getDirectoryPath(n),function(n){var i=e.combinePaths(n,"package.json");if(t.fileExists(i)){var a=ur(i,t);a&&r.push(a)}}),r}function ur(n,t){if(t.readFile){var r=["dependencies","devDependencies","optionalDependencies","peerDependencies"],i=t.readFile(n)||"",a=pr(i),s={};if(a)for(var c=0,l=r;c<l.length;c++){var u=l[c],d=a[u];if(d){var p=new e.Map;for(var f in d)p.set(f,d[f]);s[u]=p}}var m=[[1,s.dependencies],[2,s.devDependencies],[8,s.optionalDependencies],[4,s.peerDependencies]];return o(o({},s),{parseable:!!a,fileName:n,get:_,has:function(e,n){return!!_(e,n)}})}function _(e,n){void 0===n&&(n=15);for(var t=0,r=m;t<r.length;t++){var i=r[t],a=i[0],o=i[1];if(o&&n&a){var s=o.get(e);if(void 0!==s)return s}}}}function dr(n,t,r){var i,a=(r.getPackageJsonsVisibleToFile&&r.getPackageJsonsVisibleToFile(n.fileName)||lr(n.fileName,r)).filter(function(e){return e.parseable});return{allowsImportingAmbientModule:s,allowsImportingSourceFile:c,allowsImportingSpecifier:l};function o(n){for(var t=p(n),r=0,i=a;r<i.length;r++){var o=i[r];if(o.has(t)||o.has(e.getTypesPackageName(t)))return!0}return!1}function s(n,t){if(!a.length||!n.valueDeclaration)return!0;var r=n.valueDeclaration.getSourceFile(),i=d(r.fileName,t);if("undefined"===typeof i)return!0;var s=e.stripQuotes(n.getName());return!!u(s)||(o(i)||o(s))}function c(e,n){if(!a.length)return!0;var t=d(e.fileName,n);return!t||o(t)}function l(n){return!(a.length&&!u(n))||(!(!e.pathIsRelative(n)&&!e.isRootedDiskPath(n))||o(n))}function u(t){return!!(e.isSourceFileJS(n)&&e.JsTyping.nodeCoreModules.has(t)&&(void 0===i&&(i=fr(n)),i))}function d(i,a){if(e.stringContains(i,"node_modules")){var o=e.moduleSpecifiers.getNodeModulesPackageName(r.getCompilationSettings(),n.path,i,a,t);if(o)return e.pathIsRelative(o)||e.isRootedDiskPath(o)?void 0:p(o)}}function p(n){var t=e.getPathComponents(e.getPackageNameFromTypesPackageName(n)).slice(1);return e.startsWith(t[0],"@")?t[0]+"/"+t[1]:t[0]}}function pr(e){try{return JSON.parse(e)}catch(n){return}}function fr(n){return e.some(n.imports,function(n){var t=n.text;return e.JsTyping.nodeCoreModules.has(t)})}function mr(n){return e.contains(e.getPathComponents(n),"node_modules")}function _r(e){return void 0!==e.file&&void 0!==e.start&&void 0!==e.length}function gr(n,t){var r=en(n),i=e.binarySearchKey(t,r,e.identity,e.compareTextSpans);if(i>=0){var a=t[i];return e.Debug.assertEqual(a.file,n.getSourceFile(),"Diagnostics proided to 'findDiagnosticForNode' must be from a single SourceFile"),e.cast(a,_r)}}function yr(n,t){var r,i=e.binarySearchKey(t,n.start,function(e){return e.start},e.compareValues);i<0&&(i=~i);while((null===(r=t[i-1])||void 0===r?void 0:r.start)===n.start)i--;var a=[],o=e.textSpanEnd(n);while(1){var s=e.tryCast(t[i],_r);if(!s||s.start>o)break;e.textSpanContainsTextSpan(n,s)&&a.push(s),i++}return a}function hr(n){var t=n.startPosition,r=n.endPosition;return e.createTextSpanFromBounds(t,void 0===r?t:r)}function vr(n,t,r){return void 0===r&&(r=e.identity),n?e.isArray(n)?r(e.map(n,t)):t(n,0):void 0}function br(n){return e.isArray(n)?e.first(n):n}function Er(n,t){return 33554432&n.flags||"export="!==n.escapedName&&"default"!==n.escapedName?n.name:e.firstDefined(n.declarations,function(n){var t;return e.isExportAssignment(n)?null===(t=e.tryCast(e.skipOuterExpressions(n.expression),e.isIdentifier))||void 0===t?void 0:t.text:void 0})||e.codefix.moduleSymbolToValidIdentifier(Tr(n),t)}function Tr(n){var t;return e.Debug.checkDefined(n.parent,"Symbol parent was undefined. Flags: "+e.Debug.formatSymbolFlags(n.flags)+". Declarations: "+(null===(t=n.declarations)||void 0===t?void 0:t.map(function(n){var t=e.Debug.formatSyntaxKind(n.kind),r=e.isInJSFile(n),i=n.expression;return(r?"[JS]":"")+t+(i?" (expression: "+e.Debug.formatSyntaxKind(i.kind)+")":"")}).join(", "))+".")}function Sr(e,n,t){var r=n.length;if(r+t>e.length)return!1;for(var i=0;i<r;i++)if(n.charCodeAt(i)!==e.charCodeAt(i+t))return!1;return!0}function xr(e){return 95===e.charCodeAt(0)}function Lr(e){return!Dr(e)}function Dr(n){var t=n.getSourceFile();return!(!t.externalModuleIndicator&&!t.commonJsModuleIndicator)&&(e.isInJSFile(n)||!e.findAncestor(n,e.isGlobalScopeAugmentation))}function Ar(n){return!!(8192&e.getCombinedNodeFlagsAlwaysIncludeJSDoc(n))}function Cr(n,t){var r=e.firstDefined(n.imports,function(n){if(e.JsTyping.nodeCoreModules.has(n.text))return e.startsWith(n.text,"node:")});return null!==r&&void 0!==r?r:t.usesUriStyleNodeCoreModules}e.getNewLineOrDefaultFromHost=st,e.lineBreakPart=ct,e.mapToDisplayParts=lt,e.typeToDisplayParts=ut,e.symbolToDisplayParts=dt,e.signatureToDisplayParts=pt,e.isImportOrExportSpecifierName=ft,e.getScriptKind=mt,e.getSymbolTarget=_t,e.getUniqueSymbolId=ht,e.getFirstNonSpaceCharacterPosition=vt,e.getPrecedingNonSpaceCharacterPosition=bt,e.getSynthesizedDeepClone=Et,e.getSynthesizedDeepCloneWithReplacements=Tt,e.getSynthesizedDeepClones=xt,e.getSynthesizedDeepClonesWithReplacements=Lt,e.suppressLeadingAndTrailingTrivia=Dt,e.suppressLeadingTrivia=At,e.suppressTrailingTrivia=Ct,e.copyComments=It,e.getUniqueName=Ot,e.getRenameLocation=wt,e.copyLeadingComments=Pt,e.copyTrailingComments=Mt,e.copyTrailingAsLeadingComments=Ft,e.needsParentheses=Ut,e.getContextualTypeFromParent=Vt,e.quote=jt,e.isEqualityOperatorKind=Kt,e.isStringLiteralOrTemplate=Ht,e.hasIndexSignature=Wt,e.getSwitchedType=zt,e.ANONYMOUS="anonymous function",e.getTypeNodeIfAccessible=Jt,e.syntaxRequiresTrailingCommaOrSemicolonOrASI=qt,e.syntaxRequiresTrailingFunctionBlockOrSemicolonOrASI=Xt,e.syntaxRequiresTrailingModuleBlockOrSemicolonOrASI=Yt,e.syntaxRequiresTrailingSemicolonOrASI=Qt,e.syntaxMayBeASICandidate=e.or(qt,Xt,Yt,Qt),e.positionIsASICandidate=$t,e.probablyUsesSemicolons=er,e.tryGetDirectories=nr,e.tryReadDirectory=tr,e.tryFileExists=rr,e.tryDirectoryExists=ir,e.tryAndIgnoreErrors=ar,e.tryIOAndConsumeErrors=or,e.findPackageJsons=sr,e.findPackageJson=cr,e.getPackageJsonsVisibleToFile=lr,e.createPackageJsonInfo=ur,e.createPackageJsonImportFilter=dr,e.consumesNodeCoreModules=fr,e.isInsideNodeModules=mr,e.isDiagnosticWithLocation=_r,e.findDiagnosticForNode=gr,e.getDiagnosticsWithinSpan=yr,e.getRefactorContextSpan=hr,e.mapOneOrMany=vr,e.firstOrOnly=br,e.getNameForExportedSymbol=Er,e.stringContainsAt=Sr,e.startsWithUnderscore=xr,e.isGlobalDeclaration=Lr,e.isNonGlobalDeclaration=Dr,e.isDeprecatedDeclaration=Ar,e.shouldUseUriStyleNodeCoreModules=Cr}(d||(d={})),function(e){function n(n){var t,r=1,i=e.createMultiMap(),a=new e.Map,o={isUsableByFile:function(e){return e===t},isEmpty:function(){return!i.size},clear:function(){i.clear(),a.clear(),t=void 0},add:function(n,s,l,u,d,p,f,m,_){n!==t&&(o.clear(),t=n);var g=1===p,y=g&&e.getLocalSymbolForExportDefault(s)||s,h=e.isExternalModuleSymbol(y)?e.unescapeLeadingUnderscores(l):e.getNameForExportedSymbol(y,m),v=e.stripQuotes(u.name),b=r++,E=33554432&s.flags?void 0:s,T=33554432&u.flags?void 0:u;E&&T||a.set(b,[s,u]),i.add(c(h,s,v,_),{id:b,symbolTableKey:l,symbolName:h,moduleName:v,moduleFile:d,moduleFileName:null===d||void 0===d?void 0:d.fileName,exportKind:p,targetFlags:e.skipAlias(s,_).flags,isFromPackageJson:f,symbol:E,moduleSymbol:T})},get:function(e,n,r,a,o){if(e===t){var l=i.get(c(n,r,a,o));return null===l||void 0===l?void 0:l.map(s)}},forEach:function(e,n){e===t&&i.forEach(function(e,t){var r=l(t),i=r.symbolName,a=r.ambientModuleName;n(e.map(s),i,!!a)})},releaseSymbols:function(){a.clear()},onFileChanged:function(n,r,i){return(!d(n)||!d(r))&&(t&&t!==r.path||i&&e.consumesNodeCoreModules(n)!==e.consumesNodeCoreModules(r)||!e.arrayIsEqualTo(n.moduleAugmentations,r.moduleAugmentations)||!p(n,r)?(o.clear(),!0):(t=r.path,!1))}};return e.Debug.isDebugging&&Object.defineProperty(o,"__cache",{get:function(){return i}}),o;function s(t){if(t.symbol&&t.moduleSymbol)return t;var r=t.id,i=t.exportKind,o=t.targetFlags,s=t.isFromPackageJson,c=t.moduleFileName,l=a.get(r)||e.emptyArray,u=l[0],d=l[1];if(u&&d)return{symbol:u,moduleSymbol:d,moduleFileName:c,exportKind:i,targetFlags:o,isFromPackageJson:s};var p=(s?n.getPackageJsonAutoImportProvider():n.getCurrentProgram()).getTypeChecker(),f=t.moduleSymbol||d||e.Debug.checkDefined(t.moduleFile?p.getMergedSymbol(t.moduleFile.symbol):p.tryFindAmbientModule(t.moduleName)),m=t.symbol||u||e.Debug.checkDefined(2===i?p.resolveExternalModuleSymbol(f):p.tryGetMemberInModuleExportsAndProperties(e.unescapeLeadingUnderscores(t.symbolTableKey),f),"Could not find symbol '"+t.symbolName+"' by key '"+t.symbolTableKey+"' in module "+f.name);return a.set(r,[m,f]),{symbol:m,moduleSymbol:f,moduleFileName:c,exportKind:i,targetFlags:o,isFromPackageJson:s}}function c(n,t,r,i){var a=e.stripQuotes(r),o=e.isExternalModuleNameRelative(a)?"/":a,s=e.skipAlias(t,i);return n+"|"+u(s)+"|"+o}function l(e){var n=e.substring(0,e.indexOf("|")),t=e.substring(e.lastIndexOf("|")+1),r="/"===t?void 0:t;return{symbolName:n,ambientModuleName:r}}function u(e){var n=e.name;while(e.parent)n+=","+e.parent.name,e=e.parent;return n}function d(e){return!e.commonJsModuleIndicator&&!e.externalModuleIndicator&&!e.moduleAugmentations&&!e.ambientModuleNames}function p(n,t){if(!e.arrayIsEqualTo(n.ambientModuleNames,t.ambientModuleNames))return!1;for(var r=-1,i=-1,a=function(a){var o=function(n){return e.isNonGlobalAmbientModule(n)&&n.name.text===a};if(r=e.findIndex(n.statements,o,r+1),i=e.findIndex(t.statements,o,i+1),n.statements[r]!==t.statements[i])return{value:!1}},o=0,s=t.ambientModuleNames;o<s.length;o++){var c=s[o],l=a(c);if("object"===typeof l)return l.value}return!0}}function t(n,t,i,a,o,s,c){var l;if(t===i)return!1;var u=null===c||void 0===c?void 0:c.get(t.path,i.path,a);if(void 0!==(null===u||void 0===u?void 0:u.isAutoImportable))return u.isAutoImportable;var d=e.hostGetCanonicalFileName(s),p=null===(l=s.getGlobalTypingsCacheLocation)||void 0===l?void 0:l.call(s),f=!!e.moduleSpecifiers.forEachFileNameOfModule(t.fileName,i.fileName,s,!1,function(e){var a=n.getSourceFile(e);return(a===i||!a)&&r(t.fileName,e,d,p)});if(o){var m=f&&o.allowsImportingSourceFile(i,s);return null===c||void 0===c||c.setIsAutoImportable(t.path,i.path,a,m),m}return f}function r(n,t,r,i){var a=e.forEachAncestorDirectory(t,function(n){return"node_modules"===e.getBaseFileName(n)?n:void 0}),o=a&&e.getDirectoryPath(r(a));return void 0===o||e.startsWith(r(n),o)||!!i&&e.startsWith(r(i),o)}function i(n,t,r,i){var o,s;a(n.getTypeChecker(),n.getSourceFiles(),function(e,t){return i(e,t,n,!1)});var c=r&&(null===(o=t.getPackageJsonAutoImportProvider)||void 0===o?void 0:o.call(t));if(c){var l=e.timestamp();a(c.getTypeChecker(),c.getSourceFiles(),function(e,n){return i(e,n,c,!0)}),null===(s=t.log)||void 0===s||s.call(t,"forEachExternalModuleToImportFrom autoImportProvider: "+(e.timestamp()-l))}}function a(n,t,r){for(var i=0,a=n.getAmbientModules();i<a.length;i++){var o=a[i];e.stringContains(o.name,"*")||r(o,void 0)}for(var s=0,c=t;s<c.length;s++){var l=c[s];e.isExternalOrCommonJsModule(l)&&r(n.getMergedSymbol(l.symbol),l)}}function s(t,r,a,o){var s,u,d,p,f,m=e.timestamp();null===(s=r.getPackageJsonAutoImportProvider)||void 0===s||s.call(r);var _=(null===(u=r.getCachedExportInfoMap)||void 0===u?void 0:u.call(r))||n({getCurrentProgram:function(){return a},getPackageJsonAutoImportProvider:function(){var e;return null===(e=r.getPackageJsonAutoImportProvider)||void 0===e?void 0:e.call(r)}});if(_.isUsableByFile(t.path))return null===(d=r.log)||void 0===d||d.call(r,"getExportInfoMap: cache hit"),_;null===(p=r.log)||void 0===p||p.call(r,"getExportInfoMap: cache miss or empty; calculating new results");var g=a.getCompilerOptions(),y=e.getEmitScriptTarget(g),h=0;return i(a,r,!0,function(n,r,i,a){++h%100===0&&(null===o||void 0===o||o.throwIfCancellationRequested());var s=new e.Map,u=i.getTypeChecker(),d=c(n,u,g);d&&l(d.symbol,u)&&_.add(t.path,d.symbol,1===d.exportKind?"default":"export=",n,r,d.exportKind,a,y,u),u.forEachExportAndPropertyOfModule(n,function(i,o){i!==(null===d||void 0===d?void 0:d.symbol)&&l(i,u)&&e.addToSeen(s,i)&&_.add(t.path,i,o,n,r,0,a,y,u)})}),null===(f=r.log)||void 0===f||f.call(r,"getExportInfoMap: done in "+(e.timestamp()-m)+" ms"),_}function c(e,n,t){var r=u(e,n);if(r){var i=r.symbol,a=r.exportKind,s=d(i,n,t);return s&&o({symbol:i,exportKind:a},s)}}function l(n,t){return!t.isUndefinedSymbol(n)&&!t.isUnknownSymbol(n)&&!e.isKnownSymbol(n)&&!e.isPrivateIdentifierSymbol(n)}function u(e,n){var t=n.resolveExternalModuleSymbol(e);if(t!==e)return{symbol:t,exportKind:2};var r=n.tryGetMemberInModuleExports("default",e);return r?{symbol:r,exportKind:1}:void 0}function d(n,t,r){var i=e.getLocalSymbolForExportDefault(n);if(i)return{symbolForMeaning:i,name:i.name};var a=p(n);if(void 0!==a)return{symbolForMeaning:n,name:a};if(2097152&n.flags){var o=t.getImmediateAliasedSymbol(n);if(o&&o.parent)return d(o,t,r)}return"default"!==n.escapedName&&"export="!==n.escapedName?{symbolForMeaning:n,name:n.getName()}:{symbolForMeaning:n,name:e.getNameForExportedSymbol(n,r.target)}}function p(n){return n.declarations&&e.firstDefined(n.declarations,function(n){var t;return e.isExportAssignment(n)?null===(t=e.tryCast(e.skipOuterExpressions(n.expression),e.isIdentifier))||void 0===t?void 0:t.text:e.isExportSpecifier(n)?(e.Debug.assert("default"===n.name.text,"Expected the specifier to be a default export"),n.propertyName&&n.propertyName.text):void 0})}(function(e){e[e["Named"]=0]="Named",e[e["Default"]=1]="Default",e[e["Namespace"]=2]="Namespace",e[e["CommonJS"]=3]="CommonJS"})(e.ImportKind||(e.ImportKind={})),function(e){e[e["Named"]=0]="Named",e[e["Default"]=1]="Default",e[e["ExportEquals"]=2]="ExportEquals",e[e["UMD"]=3]="UMD"}(e.ExportKind||(e.ExportKind={})),e.createCacheableExportInfoMap=n,e.isImportableFile=t,e.forEachExternalModuleToImportFrom=i,e.getExportInfoMap=s,e.getDefaultLikeExportInfo=c}(d||(d={})),function(e){function n(){var n=e.createScanner(99,!1);function o(e,n,t){return a(l(e,n,t),e)}function l(a,o,l){var u=0,p=0,f=[],m=c(o),_=m.prefix,g=m.pushTemplate;a=_+a;var y=_.length;g&&f.push(15),n.setText(a);var h=0,v=[],b=0;do{u=n.scan(),e.isTrivia(u)||(S(),p=u);var E=n.getTextPos();if(i(n.getTokenPos(),E,y,d(u),v),E>=a.length){var T=r(n,u,e.lastOrUndefined(f));void 0!==T&&(h=T)}}while(1!==u);function S(){switch(u){case 43:case 68:t[p]||13!==n.reScanSlashToken()||(u=13);break;case 29:79===p&&b++;break;case 31:b>0&&b--;break;case 129:case 148:case 145:case 132:case 149:b>0&&!l&&(u=79);break;case 15:f.push(u);break;case 18:f.length>0&&f.push(u);break;case 19:if(f.length>0){var r=e.lastOrUndefined(f);15===r?(u=n.reScanTemplateToken(!1),17===u?f.pop():e.Debug.assertEqual(u,16,"Should have been a template middle.")):(e.Debug.assertEqual(r,18,"Should have been an open brace"),f.pop())}break;default:if(!e.isKeyword(u))break;24===p?u=79:e.isKeyword(p)&&e.isKeyword(u)&&!s(p,u)&&(u=79)}}return{endOfLineState:h,spans:v}}return{getClassificationsForLine:o,getEncodedLexicalClassifications:l}}e.createClassifier=n;var t=e.arrayToNumericMap([79,10,8,9,13,108,45,46,21,23,19,110,95],function(e){return e},function(){return!0});function r(n,t,r){switch(t){case 10:if(!n.isUnterminated())return;var i=n.getTokenText(),a=i.length-1,o=0;while(92===i.charCodeAt(a-o))o++;if(0===(1&o))return;return 34===i.charCodeAt(0)?3:2;case 3:return n.isUnterminated()?1:void 0;default:if(e.isTemplateLiteralKind(t)){if(!n.isUnterminated())return;switch(t){case 17:return 5;case 14:return 4;default:return e.Debug.fail("Only 'NoSubstitutionTemplateLiteral's and 'TemplateTail's can be unterminated; got SyntaxKind #"+t)}}return 15===r?6:void 0}}function i(e,n,t,r,i){if(8!==r){0===e&&t>0&&(e+=t);var a=n-e;a>0&&i.push(e-t,a,r)}}function a(n,t){for(var r=[],i=n.spans,a=0,s=0;s<i.length;s+=3){var c=i[s],l=i[s+1],u=i[s+2];if(a>=0){var d=c-a;d>0&&r.push({length:d,classification:e.TokenClass.Whitespace})}r.push({length:l,classification:o(u)}),a=c+l}var p=t.length-a;return p>0&&r.push({length:p,classification:e.TokenClass.Whitespace}),{entries:r,finalLexState:n.endOfLineState}}function o(n){switch(n){case 1:return e.TokenClass.Comment;case 3:return e.TokenClass.Keyword;case 4:return e.TokenClass.NumberLiteral;case 25:return e.TokenClass.BigIntLiteral;case 5:return e.TokenClass.Operator;case 6:return e.TokenClass.StringLiteral;case 8:return e.TokenClass.Whitespace;case 10:return e.TokenClass.Punctuation;case 2:case 11:case 12:case 13:case 14:case 15:case 16:case 9:case 17:return e.TokenClass.Identifier;default:return}}function s(n,t){if(!e.isAccessibilityModifier(n))return!0;switch(t){case 135:case 147:case 133:case 124:return!0;default:return!1}}function c(n){switch(n){case 3:return{prefix:'"\\\n'};case 2:return{prefix:"'\\\n"};case 1:return{prefix:"/*\n"};case 4:return{prefix:"`\n"};case 5:return{prefix:"}\n",pushTemplate:!0};case 6:return{prefix:"",pushTemplate:!0};case 0:return{prefix:""};default:return e.Debug.assertNever(n)}}function l(e){switch(e){case 41:case 43:case 44:case 39:case 40:case 47:case 48:case 49:case 29:case 31:case 32:case 33:case 102:case 101:case 127:case 34:case 35:case 36:case 37:case 50:case 52:case 51:case 55:case 56:case 74:case 73:case 78:case 70:case 71:case 72:case 64:case 65:case 66:case 68:case 69:case 63:case 27:case 60:case 75:case 76:case 77:return!0;default:return!1}}function u(e){switch(e){case 39:case 40:case 54:case 53:case 45:case 46:return!0;default:return!1}}function d(n){if(e.isKeyword(n))return 3;if(l(n)||u(n))return 5;if(n>=18&&n<=78)return 10;switch(n){case 8:return 4;case 9:return 25;case 10:return 6;case 13:return 7;case 7:case 3:case 2:return 1;case 5:case 4:return 8;case 79:default:return e.isTemplateLiteralKind(n)?6:2}}function p(e,n,t,r,i){return h(m(e,n,t,r,i))}function f(e,n){switch(n){case 259:case 255:case 256:case 254:case 224:case 211:case 212:e.throwIfCancellationRequested()}}function m(n,t,r,i,a){var o=[];return r.forEachChild(function o(c){if(c&&e.textSpanIntersectsWith(a,c.pos,c.getFullWidth())){if(f(t,c.kind),e.isIdentifier(c)&&!e.nodeIsMissing(c)&&i.has(c.escapedText)){var l=n.getSymbolAtLocation(c),u=l&&_(l,e.getMeaningFromLocation(c),n);u&&s(c.getStart(r),c.getEnd(),u)}c.forEachChild(o)}}),{spans:o,endOfLineState:0};function s(n,t,r){var i=t-n;e.Debug.assert(i>0,"Classification had non-positive length of "+i),o.push(n),o.push(i),o.push(r)}}function _(e,n,t){var r=e.getFlags();return 0===(2885600&r)?void 0:32&r?11:384&r?12:524288&r?16:1536&r?4&n||1&n&&g(e)?14:void 0:2097152&r?_(t.getAliasedSymbol(e),n,t):2&n?64&r?13:262144&r?15:void 0:void 0}function g(n){return e.some(n.declarations,function(n){return e.isModuleDeclaration(n)&&1===e.getModuleInstanceState(n)})}function y(e){switch(e){case 1:return"comment";case 2:return"identifier";case 3:return"keyword";case 4:return"number";case 25:return"bigint";case 5:return"operator";case 6:return"string";case 8:return"whitespace";case 9:return"text";case 10:return"punctuation";case 11:return"class name";case 12:return"enum name";case 13:return"interface name";case 14:return"module name";case 15:return"type parameter name";case 16:return"type alias name";case 17:return"parameter name";case 18:return"doc comment tag name";case 19:return"jsx open tag name";case 20:return"jsx close tag name";case 21:return"jsx self closing tag name";case 22:return"jsx attribute";case 23:return"jsx text";case 24:return"jsx attribute string literal value";default:return}}function h(n){e.Debug.assert(n.spans.length%3===0);for(var t=n.spans,r=[],i=0;i<t.length;i+=3)r.push({textSpan:e.createTextSpan(t[i],t[i+1]),classificationType:y(t[i+2])});return r}function v(e,n,t){return h(b(e,n,t))}function b(n,t,r){var i=r.start,a=r.length,o=e.createScanner(99,!1,t.languageVariant,t.text),s=e.createScanner(99,!1,t.languageVariant,t.text),c=[];return T(t),{spans:c,endOfLineState:0};function l(e,n,t){c.push(e),c.push(n),c.push(t)}function u(n){o.setTextPos(n.pos);while(1){var r=o.getTextPos();if(!e.couldStartTrivia(t.text,r))return r;var i=o.scan(),a=o.getTextPos(),s=a-r;if(!e.isTrivia(i))return r;switch(i){case 4:case 5:continue;case 2:case 3:d(n,i,r,s),o.setTextPos(a);continue;case 7:var c=t.text,u=c.charCodeAt(r);if(60===u||62===u){l(r,s,1);continue}e.Debug.assert(124===u||61===u),y(c,r,a);break;case 6:break;default:e.Debug.assertNever(i)}}}function d(n,r,i,a){if(3===r){var o=e.parseIsolatedJSDocComment(t.text,i,a);if(o&&o.jsDoc)return e.setParent(o.jsDoc,n),void m(o.jsDoc)}else if(2===r&&_(i,a))return;p(i,a)}function p(e,n){l(e,n,1)}function m(e){var n,t,r,i,a,o,s,c=e.pos;if(e.tags)for(var u=0,d=e.tags;u<d.length;u++){var f=d[u];f.pos!==c&&p(c,f.pos-c),l(f.pos,1,10),l(f.tagName.pos,f.tagName.end-f.tagName.pos,18),c=f.tagName.end;var m=f.tagName.end;switch(f.kind){case 335:var _=f;v(_),m=_.isNameFirst&&(null===(n=_.typeExpression)||void 0===n?void 0:n.end)||_.name.end;break;case 342:var y=f;m=y.isNameFirst&&(null===(t=y.typeExpression)||void 0===t?void 0:t.end)||y.name.end;break;case 339:g(f),c=f.end,m=f.typeParameters.end;break;case 340:var h=f;m=304===(null===(r=h.typeExpression)||void 0===r?void 0:r.kind)&&(null===(i=h.fullName)||void 0===i?void 0:i.end)||(null===(a=h.typeExpression)||void 0===a?void 0:a.end)||m;break;case 333:m=f.typeExpression.end;break;case 338:T(f.typeExpression),c=f.end,m=f.typeExpression.end;break;case 337:case 334:m=f.typeExpression.end;break;case 336:T(f.typeExpression),c=f.end,m=(null===(o=f.typeExpression)||void 0===o?void 0:o.end)||m;break;case 341:m=(null===(s=f.name)||void 0===s?void 0:s.end)||m;break;case 323:case 324:m=f.class.end;break}"object"===typeof f.comment?p(f.comment.pos,f.comment.end-f.comment.pos):"string"===typeof f.comment&&p(m,f.end-m)}return void(c!==e.end&&p(c,e.end-c));function v(e){e.isNameFirst&&(p(c,e.name.pos-c),l(e.name.pos,e.name.end-e.name.pos,17),c=e.name.end),e.typeExpression&&(p(c,e.typeExpression.pos-c),T(e.typeExpression),c=e.typeExpression.end),e.isNameFirst||(p(c,e.name.pos-c),l(e.name.pos,e.name.end-e.name.pos,17),c=e.name.end)}}function _(n,r){var i=/^(\/\/\/\s*)(<)(?:(\S+)((?:[^/]|\/[^>])*)(\/>)?)?/im,a=/(\s)(\S+)(\s*)(=)(\s*)('[^']+'|"[^"]+")/gim,o=t.text.substr(n,r),s=i.exec(o);if(!s)return!1;if(!s[3]||!(s[3]in e.commentPragmas))return!1;var c=n;p(c,s[1].length),c+=s[1].length,l(c,s[2].length,10),c+=s[2].length,l(c,s[3].length,21),c+=s[3].length;var u=s[4],d=c;while(1){var f=a.exec(u);if(!f)break;var m=c+f.index+f[1].length;m>d&&(p(d,m-d),d=m),l(d,f[2].length,22),d+=f[2].length,f[3].length&&(p(d,f[3].length),d+=f[3].length),l(d,f[4].length,5),d+=f[4].length,f[5].length&&(p(d,f[5].length),d+=f[5].length),l(d,f[6].length,24),d+=f[6].length}c+=s[4].length,c>d&&p(d,c-d),s[5]&&(l(c,s[5].length,10),c+=s[5].length);var _=n+r;return c<_&&p(c,_-c),!0}function g(e){for(var n=0,t=e.getChildren();n<t.length;n++){var r=t[n];T(r)}}function y(n,t,r){var i;for(i=t;i<r;i++)if(e.isLineBreak(n.charCodeAt(i)))break;l(t,i-t,1),s.setTextPos(i);while(s.getTextPos()<r)h()}function h(){var e=s.getTextPos(),n=s.scan(),t=s.getTextPos(),r=E(n);r&&l(e,t-e,r)}function v(n){if(e.isJSDoc(n))return!0;if(e.nodeIsMissing(n))return!0;var t=b(n);if(!e.isToken(n)&&11!==n.kind&&void 0===t)return!1;var r=11===n.kind?n.pos:u(n),i=n.end-r;if(e.Debug.assert(i>=0),i>0){var a=t||E(n.kind,n);a&&l(r,i,a)}return!0}function b(e){switch(e.parent&&e.parent.kind){case 278:if(e.parent.tagName===e)return 19;break;case 279:if(e.parent.tagName===e)return 20;break;case 277:if(e.parent.tagName===e)return 21;break;case 283:if(e.parent.name===e)return 22;break}}function E(n,t){if(e.isKeyword(n))return 3;if((29===n||31===n)&&t&&e.getTypeArgumentOrTypeParameterList(t.parent))return 10;if(e.isPunctuation(n)){if(t){var r=t.parent;if(63===n&&(252===r.kind||165===r.kind||162===r.kind||283===r.kind))return 5;if(219===r.kind||217===r.kind||218===r.kind||220===r.kind)return 5}return 10}if(8===n)return 4;if(9===n)return 25;if(10===n)return t&&283===t.parent.kind?24:6;if(13===n)return 6;if(e.isTemplateLiteralKind(n))return 6;if(11===n)return 23;if(79===n){if(t)switch(t.parent.kind){case 255:return t.parent.name===t?11:void 0;case 161:return t.parent.name===t?15:void 0;case 256:return t.parent.name===t?13:void 0;case 258:return t.parent.name===t?12:void 0;case 259:return t.parent.name===t?14:void 0;case 162:return t.parent.name===t?e.isThisIdentifier(t)?3:17:void 0}return 2}}function T(r){if(r&&e.decodedTextSpanIntersectsWith(i,a,r.pos,r.getFullWidth())){f(n,r.kind);for(var o=0,s=r.getChildren(t);o<s.length;o++){var c=s[o];v(c)||T(c)}}}}e.getSemanticClassifications=p,e.getEncodedSemanticClassifications=m,e.getSyntacticClassifications=v,e.getEncodedSyntacticClassifications=b}(d||(d={})),function(e){(function(n){(function(n){function t(n,t,i,a){var o=r(n,t,i,a);e.Debug.assert(o.spans.length%3===0);for(var s=o.spans,c=[],l=0;l<s.length;l+=3)c.push({textSpan:e.createTextSpan(s[l],s[l+1]),classificationType:s[l+2]});return c}function r(e,n,t,r){return{spans:i(e,t,r,n),endOfLineState:0}}function i(e,n,t,r){var i=[],o=function(e,t,r){i.push(e.getStart(n),e.getWidth(n),(t+1<<8)+r)};return e&&n&&a(e,n,t,o,r),i}function a(n,t,r,i,a){var l=n.getTypeChecker(),d=!1;function m(_){switch(_.kind){case 259:case 255:case 256:case 254:case 224:case 211:case 212:a.throwIfCancellationRequested()}if(_&&e.textSpanIntersectsWith(r,_.pos,_.getFullWidth())&&0!==_.getFullWidth()){var g=d;if((e.isJsxElement(_)||e.isJsxSelfClosingElement(_))&&(d=!0),e.isJsxExpression(_)&&(d=!1),e.isIdentifier(_)&&!d&&!u(_)&&!e.isInfinityOrNaNString(_.escapedText)){var y=l.getSymbolAtLocation(_);if(y){2097152&y.flags&&(y=l.getAliasedSymbol(y));var h=o(y,e.getMeaningFromLocation(_));if(void 0!==h){var v=0;if(_.parent){var b=e.isBindingElement(_.parent)||f.get(_.parent.kind)===h;b&&_.parent.name===_&&(v=1)}6===h&&p(_)&&(h=9),h=s(l,_,h);var E=y.valueDeclaration;if(E){var T=e.getCombinedModifierFlags(E),S=e.getCombinedNodeFlags(E);32&T&&(v|=2),256&T&&(v|=4),0!==h&&2!==h&&(64&T||2&S||8&y.getFlags())&&(v|=8),7!==h&&10!==h||!c(E,t)||(v|=32),n.isSourceFileDefaultLibrary(E.getSourceFile())&&(v|=16)}else y.declarations&&y.declarations.some(function(e){return n.isSourceFileDefaultLibrary(e.getSourceFile())})&&(v|=16);i(_,h,v)}}}e.forEachChild(_,m),d=g}}m(t)}function o(n,t){var r=n.getFlags();if(32&r)return 0;if(384&r)return 1;if(524288&r)return 5;if(64&r){if(2&t)return 2}else if(262144&r)return 4;var i=n.valueDeclaration||n.declarations&&n.declarations[0];return i&&e.isBindingElement(i)&&(i=l(i)),i&&f.get(i.kind)}function s(e,n,t){if(7===t||9===t||6===t){var r=e.getTypeAtLocation(n);if(r){var i=function(e){return e(r)||r.isUnion()&&r.types.some(e)};if(6!==t&&i(function(e){return e.getConstructSignatures().length>0}))return 0;if(i(function(e){return e.getCallSignatures().length>0})&&!i(function(e){return e.getProperties().length>0})||d(n))return 9===t?11:10}}return t}function c(n,t){return e.isBindingElement(n)&&(n=l(n)),e.isVariableDeclaration(n)?(!e.isSourceFile(n.parent.parent.parent)||e.isCatchClause(n.parent))&&n.getSourceFile()===t:!!e.isFunctionDeclaration(n)&&(!e.isSourceFile(n.parent)&&n.getSourceFile()===t)}function l(n){while(1){if(!e.isBindingElement(n.parent.parent))return n.parent.parent;n=n.parent.parent}}function u(n){var t=n.parent;return t&&(e.isImportClause(t)||e.isImportSpecifier(t)||e.isNamespaceImport(t))}function d(n){while(p(n))n=n.parent;return e.isCallExpression(n.parent)&&n.parent.expression===n}function p(n){return e.isQualifiedName(n.parent)&&n.parent.right===n||e.isPropertyAccessExpression(n.parent)&&n.parent.name===n}(function(e){e[e["typeOffset"]=8]="typeOffset",e[e["modifierMask"]=255]="modifierMask"})(n.TokenEncodingConsts||(n.TokenEncodingConsts={})),function(e){e[e["class"]=0]="class",e[e["enum"]=1]="enum",e[e["interface"]=2]="interface",e[e["namespace"]=3]="namespace",e[e["typeParameter"]=4]="typeParameter",e[e["type"]=5]="type",e[e["parameter"]=6]="parameter",e[e["variable"]=7]="variable",e[e["enumMember"]=8]="enumMember",e[e["property"]=9]="property",e[e["function"]=10]="function",e[e["member"]=11]="member"}(n.TokenType||(n.TokenType={})),function(e){e[e["declaration"]=0]="declaration",e[e["static"]=1]="static",e[e["async"]=2]="async",e[e["readonly"]=3]="readonly",e[e["defaultLibrary"]=4]="defaultLibrary",e[e["local"]=5]="local"}(n.TokenModifier||(n.TokenModifier={})),n.getSemanticClassifications=t,n.getEncodedSemanticClassifications=r;var f=new e.Map([[252,7],[162,6],[165,9],[259,3],[258,1],[294,8],[255,0],[167,11],[254,10],[211,10],[166,11],[170,9],[171,9],[164,9],[256,2],[257,5],[161,4],[291,9],[292,9]])})(n.v2020||(n.v2020={}))})(e.classifier||(e.classifier={}))}(d||(d={})),function(e){(function(n){(function(t){function r(n,t,r,a,o,s,l,u){if(e.isInReferenceComment(n,t)){var d=G(n,t,o,s);return d&&c(d)}if(e.isInString(n,t,r)){if(!r||!e.isStringLiteralLike(r))return;d=p(n,r,t,a,o,s,u);return i(d,r,n,a,l,o,u)}}function i(t,r,i,a,o,s,l){if(void 0!==t){var u=e.createTextSpanFromStringLiteralLikeContent(r);switch(t.kind){case 0:return c(t.paths);case 1:var d=[];return n.getCompletionEntriesFromSymbols(t.symbols,d,r,i,i,a,99,o,4,l,s),{isGlobalCompletion:!1,isMemberCompletion:!0,isNewIdentifierLocation:t.hasIndexSignature,optionalReplacementSpan:u,entries:d};case 2:d=t.types.map(function(t){return{name:t.value,kindModifiers:"",kind:"string",sortText:n.SortText.LocationPriority,replacementSpan:e.getReplacementSpanForContextToken(r)}});return{isGlobalCompletion:!1,isMemberCompletion:!1,isNewIdentifierLocation:t.isNewIdentifier,optionalReplacementSpan:u,entries:d};default:return e.Debug.assertNever(t)}}}function o(n,t,r,i,a,o,c,l,u){if(i&&e.isStringLiteralLike(i)){var d=p(t,i,r,a,o,c,u);return d&&s(n,i,d,t,a,l)}}function s(t,r,i,a,o,s){switch(i.kind){case 0:var c=e.find(i.paths,function(e){return e.name===t});return c&&n.createCompletionDetails(t,l(c.extension),c.kind,[e.textPart(t)]);case 1:c=e.find(i.symbols,function(e){return e.name===t});return c&&n.createCompletionDetailsForSymbol(c,o,a,r,s);case 2:return e.find(i.types,function(e){return e.value===t})?n.createCompletionDetails(t,"","type",[e.textPart(t)]):void 0;default:return e.Debug.assertNever(i)}}function c(e){var t=!1,r=!0,i=e.map(function(e){var t=e.name,r=e.kind,i=e.span,a=e.extension;return{name:t,kind:r,kindModifiers:l(a),sortText:n.SortText.LocationPriority,replacementSpan:i}});return{isGlobalCompletion:t,isMemberCompletion:!1,isNewIdentifierLocation:r,entries:i}}function l(n){switch(n){case".d.ts":return".d.ts";case".js":return".js";case".json":return".json";case".jsx":return".jsx";case".ts":return".ts";case".tsx":return".tsx";case".tsbuildinfo":return e.Debug.fail("Extension .tsbuildinfo is unsupported.");case void 0:return"";default:return e.Debug.assertNever(n)}}var u,d;function p(n,t,r,i,a,o,s){var c=f(t.parent);switch(c.kind){case 194:var l=f(c.parent);switch(l.kind){case 176:var u=l,d=e.findAncestor(c,function(e){return e.parent===u});return d?{kind:2,types:h(i.getTypeArgumentConstraint(d)),isNewIdentifier:!1}:void 0;case 192:var p=l,v=p.indexType,b=p.objectType;if(!e.rangeContainsPosition(v,r))return;return g(i.getTypeFromTypeNode(b));case 198:return{kind:0,paths:T(n,t,a,o,i,s)};case 185:if(!e.isTypeReferenceNode(l.parent))return;var E=m(l,c),S=h(i.getTypeArgumentConstraint(l)).filter(function(n){return!e.contains(E,n.value)});return{kind:2,types:S,isNewIdentifier:!1};default:return}case 291:return e.isObjectLiteralExpression(c.parent)&&c.name===t?y(i,c.parent):C();case 205:var x=c,L=x.expression,D=x.argumentExpression;return t===e.skipParentheses(D)?g(i.getTypeAtLocation(L)):void 0;case 206:case 207:if(!z(t)&&!e.isImportCall(c)){var A=e.SignatureHelp.getArgumentInfoForCompletions(t,r,n);return A?_(A,i):C()}case 264:case 270:case 275:return{kind:0,paths:T(n,t,a,o,i,s)};default:return C()}function C(){return{kind:2,types:h(e.getContextualTypeFromParent(t,i)),isNewIdentifier:!1}}}function f(n){switch(n.kind){case 189:return e.walkUpParenthesizedTypes(n);case 210:return e.walkUpParenthesizedExpressions(n);default:return n}}function m(n,t){return e.mapDefined(n.types,function(n){return n!==t&&e.isLiteralTypeNode(n)&&e.isStringLiteral(n.literal)?n.literal.text:void 0})}function _(n,t){var r=!1,i=new e.Map,a=[];t.getResolvedSignature(n.invocation,a,n.argumentCount);var o=e.flatMap(a,function(a){if(e.signatureHasRestParameter(a)||!(n.argumentCount>a.parameters.length)){var o=t.getParameterType(a,n.argumentIndex);return r=r||!!(4&o.flags),h(o,i)}});return{kind:2,types:o,isNewIdentifier:r}}function g(n){return n&&{kind:1,symbols:e.filter(n.getApparentProperties(),function(n){return!(n.valueDeclaration&&e.isPrivateIdentifierClassElementDeclaration(n.valueDeclaration))}),hasIndexSignature:e.hasIndexSignature(n)}}function y(t,r){var i=t.getContextualType(r);if(i){var a=t.getContextualType(r,4),o=n.getPropertiesForObjectExpression(i,a,r,t);return{kind:1,symbols:o,hasIndexSignature:e.hasIndexSignature(i)}}}function h(n,t){return void 0===t&&(t=new e.Map),n?(n=e.skipConstraint(n),n.isUnion()?e.flatMap(n.types,function(e){return h(e,t)}):!n.isStringLiteral()||1024&n.flags||!e.addToSeen(t,n.value)?e.emptyArray:[n]):e.emptyArray}function v(e,n,t){return{name:e,kind:n,extension:t}}function b(e){return v(e,"directory",void 0)}function E(n,t,r){var i=V(n,t),a=0===n.length?void 0:e.createTextSpan(t,n.length);return r.map(function(n){var t=n.name,r=n.kind,o=n.extension;return-1!==Math.max(t.indexOf(e.directorySeparator),t.indexOf(e.altDirectorySeparator))?{name:t,kind:r,extension:o,span:a}:{name:t,kind:r,extension:o,span:i}})}function T(e,n,t,r,i,a){return E(n.text,n.getStart(e)+1,S(e,n,t,r,i,a))}function S(n,t,r,i,a,o){var s=e.normalizeSlashes(t.text),c=n.path,l=e.getDirectoryPath(c);return j(s)||!r.baseUrl&&(e.isRootedDiskPath(s)||e.isUrl(s))?L(s,l,r,i,c,o):k(s,l,r,i,a)}function x(e,n){return void 0===n&&(n=0),{extensions:D(e),includeExtensionsOption:n}}function L(e,n,t,r,i,a){var o="js"===a.importModuleSpecifierEnding?2:0,s=x(t,o);return t.rootDirs?C(t.rootDirs,e,n,s,t,r,i):I(e,n,s,r,i)}function D(n){var t=e.getSupportedExtensions(n);return n.resolveJsonModule&&e.getEmitModuleResolutionKind(n)===e.ModuleResolutionKind.NodeJs?t.concat(".json"):t}function A(n,t,r,i){n=n.map(function(n){return e.normalizePath(e.isRootedDiskPath(n)?n:e.combinePaths(t,n))});var o=e.firstDefined(n,function(n){return e.containsPath(n,r,t,i)?r.substr(n.length):void 0});return e.deduplicate(a(a([],n.map(function(n){return e.combinePaths(n,o)}),!0),[r],!1),e.equateStringsCaseSensitive,e.compareStringsCaseSensitive)}function C(n,t,r,i,a,o,s){var c=a.project||o.getCurrentDirectory(),l=!(o.useCaseSensitiveFileNames&&o.useCaseSensitiveFileNames()),u=A(n,c,r,l);return e.flatMap(u,function(e){return I(t,e,i,o,s)})}function I(n,t,r,i,a,o){var s=r.extensions,c=r.includeExtensionsOption;void 0===o&&(o=[]),void 0===n&&(n=""),n=e.normalizeSlashes(n),e.hasTrailingDirectorySeparator(n)||(n=e.getDirectoryPath(n)),""===n&&(n="."+e.directorySeparator),n=e.ensureTrailingDirectorySeparator(n);var l=e.resolvePath(t,n),u=e.hasTrailingDirectorySeparator(l)?l:e.getDirectoryPath(l),d=!(i.useCaseSensitiveFileNames&&i.useCaseSensitiveFileNames());if(!e.tryDirectoryExists(i,u))return o;var p=e.tryReadDirectory(i,u,s,void 0,["./*"]);if(p){for(var f=new e.Map,m=0,_=p;m<_.length;m++){var g=_[m];if(g=e.normalizePath(g),!a||0!==e.comparePaths(g,a,t,d)){var y=void 0,h=e.moduleSpecifiers.tryGetJSExtensionForFile(g,i.getCompilationSettings());0!==c||e.fileExtensionIs(g,".json")?2===c&&h?(y=e.changeExtension(e.getBaseFileName(g),h),f.set(y,h)):(y=e.getBaseFileName(g),f.set(y,e.tryGetExtensionFromPath(g))):(y=e.removeFileExtension(e.getBaseFileName(g)),f.set(y,e.tryGetExtensionFromPath(g)))}}f.forEach(function(e,n){o.push(v(n,"script",e))})}var E=e.tryGetDirectories(i,u);if(E)for(var T=0,S=E;T<S.length;T++){var x=S[T],L=e.getBaseFileName(e.normalizePath(x));"@types"!==L&&o.push(b(L))}var D=e.findPackageJson(u,i);if(D){var A=e.readJson(D,i),C=A.typesVersions;if("object"===typeof C){var I=e.getPackageJsonTypesVersionsPaths(C),k=I&&I.paths,R=l.slice(e.ensureTrailingDirectorySeparator(u).length);k&&N(o,R,u,s,k,i)}}return o}function N(n,t,r,i,a,o){for(var s in a)if(e.hasProperty(a,s)){var c=a[s];if(c)for(var l=function(e,t,r){n.some(function(n){return n.name===e})||n.push(v(e,t,r))},u=0,d=O(s,c,t,r,i,o);u<d.length;u++){var p=d[u],f=p.name,m=p.kind,_=p.extension;l(f,m,_)}}}function k(n,t,r,i,a){var o=r.baseUrl,s=r.paths,c=[],l=x(r);if(o){var u=r.project||i.getCurrentDirectory(),d=e.normalizePath(e.combinePaths(u,o));I(n,d,l,i,void 0,c),s&&N(c,n,d,l.extensions,s,i)}for(var p=R(n),f=0,m=F(n,p,a);f<m.length;f++){var _=m[f];c.push(v(_,"external module name",void 0))}if(B(i,r,t,p,l,c),e.getEmitModuleResolutionKind(r)===e.ModuleResolutionKind.NodeJs){var g=!1;if(void 0===p)for(var y=function(e){c.some(function(n){return n.name===e})||(g=!0,c.push(v(e,"external module name",void 0)))},h=0,b=U(i,t);h<b.length;h++){var E=b[h];y(E)}g||e.forEachAncestorDirectory(t,function(t){var r=e.combinePaths(t,"node_modules");e.tryDirectoryExists(i,r)&&I(n,r,l,i,void 0,c)})}return c}function R(n){return W(n)?e.hasTrailingDirectorySeparator(n)?n:e.getDirectoryPath(n):void 0}function O(n,t,r,i,a,o){if(!e.endsWith(n,"*"))return e.stringContains(n,"*")?e.emptyArray:l(n);var s=n.slice(0,n.length-1),c=e.tryRemovePrefix(r,s);return void 0===c?l(s):e.flatMap(t,function(e){return w(c,i,e,a,o)});function l(n){return e.startsWith(n,r)?[b(n)]:e.emptyArray}}function w(n,t,r,i,o){if(o.readDirectory){var s=e.tryParsePattern(r);if(void 0!==s&&!e.isString(s)){var c=e.resolvePath(s.prefix),l=e.hasTrailingDirectorySeparator(s.prefix)?c:e.getDirectoryPath(c),u=e.hasTrailingDirectorySeparator(s.prefix)?"":e.getBaseFileName(c),d=W(n),p=d?e.hasTrailingDirectorySeparator(n)?n:e.getDirectoryPath(n):void 0,f=d?e.combinePaths(l,u+p):l,m=e.normalizePath(s.suffix),_=e.normalizePath(e.combinePaths(t,f)),g=d?_:e.ensureTrailingDirectorySeparator(_)+u,y=m?"**/*":"./*",h=e.mapDefined(e.tryReadDirectory(o,_,i,void 0,[y]),function(n){var t=e.tryGetExtensionFromPath(n),r=T(n);return void 0===r?void 0:v(e.removeFileExtension(r),"script",t)}),E=e.mapDefined(e.tryGetDirectories(o,_).map(function(n){return e.combinePaths(_,n)}),function(e){var n=T(e);return void 0===n?void 0:b(n)});return a(a([],h,!0),E,!0)}}function T(n){var t=P(e.normalizePath(n),g,m);return void 0===t?void 0:M(t)}}function P(n,t,r){return e.startsWith(n,t)&&e.endsWith(n,r)?n.slice(t.length,n.length-r.length):void 0}function M(n){return n[0]===e.directorySeparator?n.slice(1):n}function F(n,t,r){var i=r.getAmbientModules().map(function(n){return e.stripQuotes(n.name)}),a=i.filter(function(t){return e.startsWith(t,n)});if(void 0!==t){var o=e.ensureTrailingDirectorySeparator(t);return a.map(function(n){return e.removePrefix(n,o)})}return a}function G(n,t,r,i){var a=e.getTokenAtPosition(n,t),o=e.getLeadingCommentRanges(n.text,a.pos),s=o&&e.find(o,function(e){return t>=e.pos&&t<=e.end});if(s){var c=n.text.slice(s.pos,t),l=K.exec(c);if(l){var u=l[1],d=l[2],p=l[3],f=e.getDirectoryPath(n.path),m="path"===d?I(p,f,x(r,1),i,n.path):"types"===d?B(i,r,f,R(p),x(r)):e.Debug.fail();return E(p,s.pos+u.length,m)}}}function B(n,t,r,i,a,o){void 0===o&&(o=[]);for(var s=new e.Map,c=e.tryAndIgnoreErrors(function(){return e.getEffectiveTypeRoots(t,n)})||e.emptyArray,l=0,u=c;l<u.length;l++){var d=u[l];g(d)}for(var p=0,f=e.findPackageJsons(r,n);p<f.length;p++){var m=f[p],_=e.combinePaths(e.getDirectoryPath(m),"node_modules/@types");g(_)}return o;function g(r){if(e.tryDirectoryExists(n,r))for(var c=0,l=e.tryGetDirectories(n,r);c<l.length;c++){var u=l[c],d=e.unmangleScopedPackageName(u);if(!t.types||e.contains(t.types,d))if(void 0===i)s.has(d)||(o.push(v(d,"external module name",void 0)),s.set(d,!0));else{var p=e.combinePaths(r,u),f=e.tryRemoveDirectoryPrefix(i,d,e.hostGetCanonicalFileName(n));void 0!==f&&I(f,p,a,n,void 0,o)}}}}function U(n,t){if(!n.readFile||!n.fileExists)return e.emptyArray;for(var r=[],i=0,a=e.findPackageJsons(t,n);i<a.length;i++)for(var o=a[i],s=e.readJson(o,n),c=0,l=H;c<l.length;c++){var u=l[c],d=s[u];if(d)for(var p in d)d.hasOwnProperty(p)&&!e.startsWith(p,"@types/")&&r.push(p)}return r}function V(n,t){var r=Math.max(n.lastIndexOf(e.directorySeparator),n.lastIndexOf(e.altDirectorySeparator)),i=-1!==r?r+1:0,a=n.length-i;return 0===a||e.isIdentifierText(n.substr(i,a),99)?void 0:e.createTextSpan(t+i,a)}function j(e){if(e&&e.length>=2&&46===e.charCodeAt(0)){var n=e.length>=3&&46===e.charCodeAt(1)?2:1,t=e.charCodeAt(n);return 47===t||92===t}return!1}t.getStringLiteralCompletions=r,t.getStringLiteralCompletionDetails=o,function(e){e[e["Paths"]=0]="Paths",e[e["Properties"]=1]="Properties",e[e["Types"]=2]="Types"}(u||(u={})),function(e){e[e["Exclude"]=0]="Exclude",e[e["Include"]=1]="Include",e[e["ModuleSpecifierCompletion"]=2]="ModuleSpecifierCompletion"}(d||(d={}));var K=/^(\/\/\/\s*<reference\s+(path|types)\s*=\s*(?:'|"))([^\3"]*)$/,H=["dependencies","devDependencies","peerDependencies","optionalDependencies"];function W(n){return e.stringContains(n,e.directorySeparator)}function z(n){return e.isCallExpression(n.parent)&&e.firstOrUndefined(n.parent.arguments)===n&&e.isIdentifier(n.parent.expression)&&"require"===n.parent.expression.escapedText}})(n.StringCompletions||(n.StringCompletions={}))})(e.Completions||(e.Completions={}))}(d||(d={})),function(e){(function(n){var t,r,i,a,s,c,l;function u(e){return!!(1&e.kind)}function d(e){return!!(2&e.kind)}function p(e){return!!(e&&4&e.kind)}function f(e){return!(!e||32!==e.kind)}function m(e){return p(e)||f(e)}function _(e){return(p(e)||f(e))&&!!e.isFromPackageJson}function g(e){return!!(8&e.kind)}function y(e){return!!(16&e.kind)}function h(t,r,i,a,o,s,c){var l,u,d,p=e.timestamp(),f=!1,m=0,_=0,g=0,y=0,h=c({tryResolve:b,resolutionLimitExceeded:function(){return f}}),v=y?" ("+(g/y*100).toFixed(1)+"% hit rate)":"";return null===(l=r.log)||void 0===l||l.call(r,t+": resolved "+_+" module specifiers, plus "+m+" ambient and "+g+" from cache"+v),null===(u=r.log)||void 0===u||u.call(r,t+": response is "+(f?"incomplete":"complete")),null===(d=r.log)||void 0===d||d.call(r,t+": "+(e.timestamp()-p)),h;function b(t,c){if(c){var l=e.codefix.getModuleSpecifierForBestExportInfo(t,a,i,r,o);return l&&m++,l}var u=s||o.allowIncompleteCompletions&&_<n.moduleSpecifierResolutionLimit,d=!u&&o.allowIncompleteCompletions&&y<n.moduleSpecifierResolutionCacheAttemptLimit,p=u||d?e.codefix.getModuleSpecifierForBestExportInfo(t,a,i,r,o,d):void 0;return(!u&&!d||d&&!p)&&(f=!0),_+=(null===p||void 0===p?void 0:p.computedWithoutCacheCount)||0,g+=t.length-_,d&&y++,p}}function v(t,r,i,a,o,s,c,l,u){var d,p=Z(o,a).previousToken;if(!c||e.isInString(a,o,p)||ye(a,c,p,o)){if(" "===c)return s.includeCompletionsForImportStatements&&s.includeCompletionsWithInsertText?{isGlobalCompletion:!0,isMemberCompletion:!1,isNewIdentifierLocation:!0,isIncomplete:!0,entries:[]}:void 0;var f=r.getTypeChecker(),m=r.getCompilerOptions(),_=s.allowIncompleteCompletions?null===(d=t.getIncompleteCompletionsCache)||void 0===d?void 0:d.call(t):void 0;if(_&&3===l&&p&&e.isIdentifier(p)){var g=b(_,a,p,r,t,s,u);if(g)return g}else null===_||void 0===_||_.clear();var y=n.StringCompletions.getStringLiteralCompletions(a,o,p,f,m,t,i,s);if(y)return y;if(p&&e.isBreakOrContinueStatement(p.parent)&&(81===p.kind||86===p.kind||79===p.kind))return G(p.parent);var h=Q(r,i,a,L(a,m),o,s,void 0,t,u);if(h)switch(h.kind){case 0:var v=x(a,f,m,i,h,s);return(null===v||void 0===v?void 0:v.isIncomplete)&&(null===_||void 0===_||_.set(v)),v;case 1:return E(e.JsDoc.getJSDocTagNameCompletions());case 2:return E(e.JsDoc.getJSDocTagCompletions());case 3:return E(e.JsDoc.getJSDocParameterNameCompletions(h.tag));case 4:return T(h.keywords);default:return e.Debug.assertNever(h)}}}function b(n,t,r,i,a,s,c){var l,u=n.get();if(u){var d=r.text.toLowerCase(),p=e.getExportInfoMap(t,a,i,c),f=i.getTypeChecker(),m=null===(l=a.getPackageJsonAutoImportProvider)||void 0===l?void 0:l.call(a),_=null===m||void 0===m?void 0:m.getTypeChecker(),g=h("continuePreviousIncompleteResponse",a,i,t,s,!1,function(n){var r=e.mapDefined(u.entries,function(r){if(!r.hasAction||!r.source||!r.data||r.data.moduleSpecifier)return r;if(Ce(r.name,d)){var s=e.Debug.checkDefined($(r.name,r.data,i,a)),c=s.symbol,l=s.origin,u=p.get(t.path,r.name,c,l.moduleSymbol.name,l.isFromPackageJson?_:f),m=u&&n.tryResolve(u,!e.isExternalModuleNameRelative(e.stripQuotes(l.moduleSymbol.name)));if(!m)return r;var g=o(o({},l),{kind:32,moduleSpecifier:m.moduleSpecifier});return r.data=R(g),r.source=M(g),r.sourceDisplay=[e.textPart(g.moduleSpecifier)],r}});return n.resolutionLimitExceeded()||(u.isIncomplete=void 0),r});return u.entries=g,u}}function E(e){return{isGlobalCompletion:!1,isMemberCompletion:!1,isNewIdentifierLocation:!1,entries:e}}function T(n){return{isGlobalCompletion:!1,isMemberCompletion:!1,isNewIdentifierLocation:!1,entries:n.map(function(n){return{name:e.tokenToString(n),kind:"keyword",kindModifiers:"",sortText:t.GlobalsOrKeywords}})}}function S(n){return 79===(null===n||void 0===n?void 0:n.kind)?e.createTextSpanFromNode(n):void 0}function x(n,t,r,i,a,o){var s=a.symbols,c=a.completionKind,l=a.isInSnippetScope,u=a.isNewIdentifierLocation,d=a.location,p=a.propertyAccessToConvert,f=a.keywordFilters,m=a.literals,_=a.symbolToOriginInfoMap,g=a.recommendedCompletion,y=a.isJsxInitializer,h=a.isTypeOnlyLocation,v=a.isJsxIdentifierExpected,b=a.importCompletionNode,E=a.insideJsDocTagTypeExpression,T=a.symbolToSortTextIdMap,x=a.hasUnresolvedAutoImports;if(1===e.getLanguageVariant(n.scriptKind)){var I=A(d,n);if(I)return I}var k=[];if(L(n,r)){var R=F(s,k,void 0,d,n,t,r.target,i,c,o,r,h,p,v,y,b,g,_,T);C(n,d.pos,R,r.target,k)}else{if(!u&&(!s||0===s.length)&&0===f)return;F(s,k,void 0,d,n,t,r.target,i,c,o,r,h,p,v,y,b,g,_,T)}if(0!==f)for(var O=new e.Set(k.map(function(e){return e.name})),w=0,P=re(f,!E&&e.isSourceFileJS(n));w<P.length;w++){var M=P[w];O.has(M.name)||k.push(M)}for(var G=0,B=m;G<B.length;G++){var U=B[G];k.push(N(n,o,U))}return{isGlobalCompletion:l,isIncomplete:!(!o.allowIncompleteCompletions||!x)||void 0,isMemberCompletion:D(c),isNewIdentifierLocation:u,optionalReplacementSpan:S(d),entries:k}}function L(n,t){return e.isSourceFileJS(n)&&!e.isCheckJsEnabledForFile(n,t)}function D(e){switch(e){case 0:case 3:case 2:return!0;default:return!1}}function A(n,r){var i=e.findAncestor(n,function(e){switch(e.kind){case 279:return!0;case 43:case 31:case 79:case 204:return!1;default:return"quit"}});if(i){var a=!!e.findChildOfKind(i,31,r),o=i.parent.openingElement.tagName,s=o.getText(r),c=s+(a?"":">"),l=e.createTextSpanFromNode(i.tagName),u={name:c,kind:"class",kindModifiers:void 0,sortText:t.LocationPriority};return{isGlobalCompletion:!1,isMemberCompletion:!0,isNewIdentifierLocation:!1,optionalReplacementSpan:l,entries:[u]}}}function C(n,r,i,a,o){e.getNameTable(n).forEach(function(n,s){if(n!==r){var c=e.unescapeLeadingUnderscores(s);!i.has(c)&&e.isIdentifierText(c,a)&&(i.add(c),o.push({name:c,kind:"warning",kindModifiers:"",sortText:t.JavascriptIdentifiers,isFromUncheckedFile:!0}))}})}function I(n,t,r){return"object"===typeof r?e.pseudoBigIntToString(r)+"n":e.isString(r)?e.quote(n,t,r):JSON.stringify(r)}function N(e,n,r){return{name:I(e,n,r),kind:"string",kindModifiers:"",sortText:t.LocationPriority}}function k(n,t,r,i,a,o,s,c,l,m,h,v,b,E,T,S){var x,L,D,A,C,I,N=e.getReplacementSpanForContextToken(r),k=l&&y(l),F=l&&d(l)||c;if(l&&u(l))L=c?"this"+(k?"?.":"")+"["+w(a,S,s)+"]":"this"+(k?"?.":".")+s;else if((F||k)&&h){L=F?c?"["+w(a,S,s)+"]":"["+s+"]":s,(k||h.questionDotToken)&&(L="?."+L);var G=e.findChildOfKind(h,24,a)||e.findChildOfKind(h,28,a);if(!G)return;var B=e.startsWith(s,h.name.text)?h.name.end:G.end;N=e.createTextSpanFromBounds(G.getStart(a),B)}if(v&&(void 0===L&&(L=s),L="{"+L+"}","boolean"!==typeof v&&(N=e.createTextSpanFromNode(v,a))),l&&g(l)&&h){void 0===L&&(L=s);var U=e.findPrecedingToken(h.pos,a),V="";U&&e.positionIsASICandidate(U.end,U.parent,a)&&(V=";"),V+="(await "+h.expression.getText()+")",L=c?""+V+L:V+(k?"?.":".")+L,N=e.createTextSpanFromBounds(h.getStart(a),h.end)}if(f(l)&&(C=[e.textPart(l.moduleSpecifier)],b&&(x=O(s,b,l,E,T,S),L=x.insertText,N=x.replacementSpan,A=!!S.includeCompletionsWithSnippetText||void 0)),void 0===L||S.includeCompletionsWithInsertText)return(p(l)||f(l))&&(D=R(l),I=!b),{name:s,kind:e.SymbolDisplay.getSymbolKind(o,n,i),kindModifiers:e.SymbolDisplay.getSymbolModifiers(o,n),sortText:t,source:M(l),hasAction:!!I||void 0,isRecommended:P(n,m,o)||void 0,insertText:L,replacementSpan:N,sourceDisplay:C,isSnippet:A,isPackageJsonImport:_(l)||void 0,isImportStatementCompletion:!!b||void 0,data:D}}function R(n){return{exportName:n.exportName,fileName:n.fileName,ambientModuleName:n.fileName?void 0:e.stripQuotes(n.moduleSymbol.name),isPackageJsonImport:!!n.isFromPackageJson||void 0,moduleSpecifier:f(n)?n.moduleSpecifier:void 0}}function O(n,t,r,i,a,o){var s=t.getSourceFile(),c=e.createTextSpanFromNode(t,s),l=e.quote(s,o,r.moduleSpecifier),u=r.isDefaultExport?1:"export="===r.exportName?2:0,d=o.includeCompletionsWithSnippetText?"$1":"",p=e.codefix.getImportKind(s,u,a,!0),f=i?";":"";switch(p){case 3:return{replacementSpan:c,insertText:"import "+n+d+" = require("+l+")"+f};case 1:return{replacementSpan:c,insertText:"import "+n+d+" from "+l+f};case 2:return{replacementSpan:c,insertText:"import * as "+n+" from "+l+f};case 0:return{replacementSpan:c,insertText:"import { "+n+d+" } from "+l+f}}}function w(n,t,r){return/^\d+$/.test(r)?r:e.quote(n,t,r)}function P(e,n,t){return e===n||!!(1048576&e.flags)&&t.getExportSymbolOfSymbol(e)===n}function M(n){return p(n)?e.stripQuotes(n.moduleSymbol.name):f(n)?n.moduleSpecifier:1===(null===n||void 0===n?void 0:n.kind)?i.ThisProperty:void 0}function F(n,t,r,i,a,o,s,c,l,u,d,p,f,m,_,g,y,h,v){for(var b,E=e.timestamp(),T=xe(i),S=e.probablyUsesSemicolons(a),x=new e.Map,L=0;L<n.length;L++){var D=n[L],A=null===h||void 0===h?void 0:h[L],C=ee(D,s,A,l,!!m);if(C&&!x.get(C.name)&&(1!==l||!v||M(D,v))){var I=C.name,N=C.needsConvertPropertyAccess,R=null!==(b=null===v||void 0===v?void 0:v[e.getSymbolId(D)])&&void 0!==b?b:11,O=(Ae(D,o)?8+R:R).toString(),w=k(D,O,r,i,a,o,I,N,A,y,f,_,g,S,d,u);if(w){var P=!A&&!(void 0===D.parent&&!e.some(D.declarations,function(e){return e.getSourceFile()===i.getSourceFile()}));x.set(I,P),t.push(w)}}}return c("getCompletionsAtPosition: getCompletionEntriesFromSymbols: "+(e.timestamp()-E)),{has:function(e){return x.has(e)},add:function(e){return x.set(e,!0)}};function M(n,t){if(!e.isSourceFile(i)){if(e.isExportAssignment(i.parent))return!0;if(T&&n.valueDeclaration===T)return!1;var r=e.skipAlias(n,o);if(a.externalModuleIndicator&&!d.allowUmdGlobalAccess&&15===t[e.getSymbolId(n)]&&(16===t[e.getSymbolId(r)]||11===t[e.getSymbolId(r)]))return!1;if(n=r,e.isInRightSideOfInternalImportEqualsDeclaration(i))return!!(1920&n.flags);if(p)return De(n,o)}return!!(111551&e.getCombinedLocalAndExportSymbolFlags(n))}}function G(e){var n=B(e);if(n.length)return{isGlobalCompletion:!1,isMemberCompletion:!1,isNewIdentifierLocation:!1,entries:n}}function B(n){var r=[],i=new e.Map,a=n;while(a){if(e.isFunctionLike(a))break;if(e.isLabeledStatement(a)){var o=a.label.text;i.has(o)||(i.set(o,!0),r.push({name:o,kindModifiers:"",kind:"label",sortText:t.LocationPriority}))}a=a.parent}return r}function U(n,t,r,i,a,o,s){if(a.data){var c=$(a.name,a.data,n,o);if(c)return{type:"symbol",symbol:c.symbol,location:e.getTouchingPropertyName(r,i),previousToken:e.findPrecedingToken(i,r,void 0),isJsxInitializer:!1,isTypeOnlyLocation:!1,origin:c.origin}}var l=n.getCompilerOptions(),u=Q(n,t,r,L(r,l),i,{includeCompletionsForModuleExports:!0,includeCompletionsWithInsertText:!0},a,o);if(!u)return{type:"none"};if(0!==u.kind)return{type:"request",request:u};var d=u.symbols,p=u.literals,f=u.location,m=u.completionKind,_=u.symbolToOriginInfoMap,g=u.previousToken,y=u.isJsxInitializer,h=u.isTypeOnlyLocation,v=e.find(p,function(e){return I(r,s,e)===a.name});return void 0!==v?{type:"literal",literal:v}:e.firstDefined(d,function(e,n){var t=_[n],r=ee(e,l.target,t,m,u.isJsxIdentifierExpected);return r&&r.name===a.name&&M(t)===a.source?{type:"symbol",symbol:e,location:f,origin:t,previousToken:g,isJsxInitializer:y,isTypeOnlyLocation:h}:void 0})||{type:"none"}}function V(t,r,i,a,o,s,c,l,u){var d=t.getTypeChecker(),p=t.getCompilerOptions(),f=o.name,m=e.findPrecedingToken(a,i);if(e.isInString(i,a,m))return n.StringCompletions.getStringLiteralCompletionDetails(f,i,a,m,d,p,s,u,l);var _=U(t,r,i,a,o,s,l);switch(_.type){case"request":var g=_.request;switch(g.kind){case 1:return e.JsDoc.getJSDocTagNameCompletionDetails(f);case 2:return e.JsDoc.getJSDocTagCompletionDetails(f);case 3:return e.JsDoc.getJSDocParameterNameCompletionDetails(f);case 4:return g.keywords.indexOf(e.stringToToken(f))>-1?j(f,"keyword",e.SymbolDisplayPartKind.keyword):void 0;default:return e.Debug.assertNever(g)}case"symbol":var y=_.symbol,h=_.location,v=_.origin,b=_.previousToken,E=W(v,y,t,s,p,i,a,b,c,l,o.data),T=E.codeActions,S=E.sourceDisplay;return K(y,d,i,h,u,T,S);case"literal":var x=_.literal;return j(I(i,l,x),"string","string"===typeof x?e.SymbolDisplayPartKind.stringLiteral:e.SymbolDisplayPartKind.numericLiteral);case"none":return te().some(function(e){return e.name===f})?j(f,"keyword",e.SymbolDisplayPartKind.keyword):void 0;default:e.Debug.assertNever(_)}}function j(n,t,r){return H(n,"",t,[e.displayPart(n,r)])}function K(n,t,r,i,a,o,s){var c=t.runWithCancellationToken(a,function(t){return e.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(t,n,r,i,i,7)}),l=c.displayParts,u=c.documentation,d=c.symbolKind,p=c.tags;return H(n.name,e.SymbolDisplay.getSymbolModifiers(t,n),d,l,u,p,o,s)}function H(e,n,t,r,i,a,o,s){return{name:e,kindModifiers:n,kind:t,displayParts:r,documentation:i,tags:a,codeActions:o,source:s,sourceDisplay:s}}function W(n,t,r,i,a,o,s,c,l,u,d){if(null===d||void 0===d?void 0:d.moduleSpecifier){var m=Z(s,o),_=m.contextToken,g=m.previousToken;if(g&&Te(_||g))return{codeActions:void 0,sourceDisplay:[e.textPart(d.moduleSpecifier)]}}if(!n||!p(n)&&!f(n))return{codeActions:void 0,sourceDisplay:void 0};var y=n.isFromPackageJson?i.getPackageJsonAutoImportProvider().getTypeChecker():r.getTypeChecker(),h=n.moduleSymbol,v=y.getMergedSymbol(e.skipAlias(t.exportSymbol||t,y)),b=e.codefix.getImportCompletionAction(v,h,o,e.getNameForExportedSymbol(t,a.target),i,r,l,c&&e.isIdentifier(c)?c.getStart(o):s,u),E=b.moduleSpecifier,T=b.codeAction;return e.Debug.assert(!(null===d||void 0===d?void 0:d.moduleSpecifier)||E===d.moduleSpecifier),{sourceDisplay:[e.textPart(E)],codeActions:[T]}}function z(e,n,t,r,i,a,o){var s=U(e,n,t,r,i,a,o);return"symbol"===s.type?s.symbol:void 0}function J(n,t,r){return e.firstDefined(t&&(t.isUnion()?t.types:[t]),function(t){var i=t&&t.symbol;return i&&424&i.flags&&!e.isAbstractConstructorSymbol(i)?X(i,n,r):void 0})}function q(n,t,r,i){var a=n.parent;switch(n.kind){case 79:return e.getContextualTypeFromParent(n,i);case 63:switch(a.kind){case 252:return i.getContextualType(a.initializer);case 219:return i.getTypeAtLocation(a.left);case 283:return i.getContextualTypeForJsxAttribute(a);default:return}case 103:return i.getContextualType(a);case 82:return e.getSwitchedType(e.cast(a,e.isCaseClause),i);case 18:return e.isJsxExpression(a)&&276!==a.parent.kind?i.getContextualTypeForJsxAttribute(a.parent):void 0;default:var o=e.SignatureHelp.getArgumentInfoForCompletions(n,t,r);return o?i.getContextualTypeForArgumentAtIndex(o.invocation,o.argumentIndex+(27===n.kind?1:0)):e.isEqualityOperatorKind(n.kind)&&e.isBinaryExpression(a)&&e.isEqualityOperatorKind(a.operatorToken.kind)?i.getTypeAtLocation(a.left):i.getContextualType(n)}}function X(n,t,r){var i=r.getAccessibleSymbolChain(n,t,67108863,!1);return i?e.first(i):n.parent&&(Y(n.parent)?n:X(n.parent,t,r))}function Y(e){var n;return!!(null===(n=e.declarations)||void 0===n?void 0:n.some(function(e){return 300===e.kind}))}function Q(n,t,r,i,a,o,s,c,l){var u=n.getTypeChecker(),d=e.timestamp(),p=e.getTokenAtPosition(r,a);t("getCompletionData: Get current token: "+(e.timestamp()-d)),d=e.timestamp();var f=e.isInComment(r,a,p);t("getCompletionData: Is inside comment: "+(e.timestamp()-d));var m=!1,_=!1;if(f){if(e.hasDocComment(r,a)){if(64===r.text.charCodeAt(a-1))return{kind:1};var g=e.getLineStartPositionForPosition(a,r);if(!/[^\*|\s(/)]/.test(r.text.substring(g,a)))return{kind:2}}var y=ue(p,a);if(y){if(y.tagName.pos<=a&&a<=y.tagName.end)return{kind:1};if(te(y)&&y.typeExpression&&304===y.typeExpression.kind&&(p=e.getTokenAtPosition(r,a),p&&(e.isDeclarationName(p)||342===p.parent.kind&&p.parent.name===p)||(m=cn(y.typeExpression))),!m&&e.isJSDocParameterTag(y)&&(e.nodeIsMissing(y.name)||y.name.pos<=a&&a<=y.name.end))return{kind:3,tag:y}}if(!m)return void t("Returning an empty list because completion was inside a regular comment or plain text part of a JsDoc comment.")}d=e.timestamp();var v=Z(a,r),b=v.previousToken,E=v.contextToken;t("getCompletionData: Get previous token: "+(e.timestamp()-d));var T,S,x=p,L=!1,D=!1,A=!1,C=!1,I=!1,N=!1,k=e.getTouchingPropertyName(r,a);if(E){var R=Te(E);if(154===R)return{kind:4,keywords:[154]};if(R&&o.includeCompletionsForImportStatements&&o.includeCompletionsWithInsertText&&(S=R),!S&&Me(E))return void t("Returning an empty list because completion was requested in an invalid position.");var O=E.parent;if(24===E.kind||28===E.kind)switch(L=24===E.kind,D=28===E.kind,O.kind){case 204:T=O,x=T.expression;var w=e.getLeftmostAccessExpression(T);if(e.nodeIsMissing(w)||(e.isCallExpression(x)||e.isFunctionLike(x))&&x.end===E.pos&&x.getChildCount(r)&&21!==e.last(x.getChildren(r)).kind)return;break;case 159:x=O.left;break;case 259:x=O.name;break;case 198:x=O;break;case 229:x=O.getFirstToken(r),e.Debug.assert(100===x.kind||103===x.kind);break;default:return}else if(!S&&1===r.languageVariant){if(O&&204===O.kind&&(E=O,O=O.parent),p.parent===k)switch(p.kind){case 31:276!==p.parent.kind&&278!==p.parent.kind||(k=p);break;case 43:277===p.parent.kind&&(k=p);break}switch(O.kind){case 279:43===E.kind&&(C=!0,k=E);break;case 219:if(!he(O))break;case 277:case 276:case 278:N=!0,29===E.kind&&(A=!0,k=E);break;case 286:case 285:19===b.kind&&31===p.kind&&(N=!0);break;case 283:if(O.initializer===b&&b.end<a){N=!0;break}switch(b.kind){case 63:I=!0;break;case 79:N=!0,O!==b.parent&&!O.initializer&&e.findChildOfKind(O,63,r)&&(I=b)}break}}}var P=e.timestamp(),M=5,F=!1,G=!1,B=!1,U=0,V=[],j=[],K=[],H=new e.Map,W=Ne(),z=e.memoizeOne(function(t){return e.createModuleSpecifierResolutionHost(t?c.getPackageJsonAutoImportProvider():n,c)});if(L||D)re();else if(A)V=u.getJsxIntrinsicTagNamesAt(k),e.Debug.assertEachIsDefined(V,"getJsxIntrinsicTagNames() should all be defined"),ce(),M=1,U=0;else if(C){var Y=E.parent.parent.openingElement.tagName,Q=u.getSymbolAtLocation(Y);Q&&(V=[Q]),M=1,U=0}else if(!ce())return;t("getCompletionData: Semantic work: "+(e.timestamp()-P));var $=b&&q(b,a,r,u),ee=e.mapDefined($&&($.isUnion()?$.types:[$]),function(e){return e.isLiteral()?e.value:void 0}),ne=b&&$&&J(b,$,u);return{kind:0,symbols:V,completionKind:M,isInSnippetScope:_,propertyAccessToConvert:T,isNewIdentifierLocation:F,location:k,keywordFilters:U,literals:ee,symbolToOriginInfoMap:j,recommendedCompletion:ne,previousToken:b,isJsxInitializer:I,insideJsDocTagTypeExpression:m,symbolToSortTextIdMap:K,isTypeOnlyLocation:W,isJsxIdentifierExpected:N,importCompletionNode:S,hasUnresolvedAutoImports:B};function te(e){switch(e.kind){case 335:case 342:case 336:case 338:case 340:return!0;default:return!1}}function re(){M=2;var n=e.isLiteralImportTypeNode(x),t=m||n&&!x.isTypeOf||e.isPartOfTypeNode(x.parent)||e.isPossiblyTypeArgumentPosition(E,r,u),i=e.isInRightSideOfInternalImportEqualsDeclaration(x);if(e.isEntityName(x)||n||e.isPropertyAccessExpression(x)){var a=e.isModuleDeclaration(x.parent);a&&(F=!0);var s=u.getSymbolAtLocation(x);if(s&&(s=e.skipAlias(s,u),1920&s.flags)){var c=u.getExportsOfModule(s);e.Debug.assertEachIsDefined(c,"getExportsOfModule() should all be defined");for(var l=function(e){return u.isValidPropertyAccess(n?x:x.parent,e.name)},d=function(e){return De(e,u)},p=a?function(e){var n;return!!(1920&e.flags)&&!(null===(n=e.declarations)||void 0===n?void 0:n.every(function(e){return e.parent===x.parent}))}:i?function(e){return d(e)||l(e)}:t?d:l,f=0,_=c;f<_.length;f++){var g=_[f];p(g)&&V.push(g)}if(!t&&s.declarations&&s.declarations.some(function(e){return 300!==e.kind&&259!==e.kind&&258!==e.kind})){var y=u.getTypeOfSymbolAtLocation(s,x).getNonOptionalType(),h=!1;if(y.isNullableType()){var v=L&&!D&&!1!==o.includeAutomaticOptionalChainCompletions;(v||D)&&(y=y.getNonNullableType(),v&&(h=!0))}ie(y,!!(32768&x.flags),h)}return}}if(!t){var b=e.getThisContainer(x,!1);!e.isSourceFile(b)&&b.parent&&u.getTypeAtLocation(b);y=u.getTypeAtLocation(x).getNonOptionalType(),h=!1;if(y.isNullableType()){v=L&&!D&&!1!==o.includeAutomaticOptionalChainCompletions;(v||D)&&(y=y.getNonNullableType(),v&&(h=!0))}ie(y,!!(32768&x.flags),h)}}function ie(n,t,r){F=!!n.getStringIndexType(),D&&e.some(n.getCallSignatures())&&(F=!0);var a=198===x.kind?x:x.parent;if(i)V.push.apply(V,e.filter(pe(n,u),function(e){return u.isValidPropertyAccessForCompletions(a,n,e)}));else for(var s=0,c=n.getApparentProperties();s<c.length;s++){var l=c[s];u.isValidPropertyAccessForCompletions(a,n,l)&&ae(l,!1,r)}if(t&&o.includeCompletionsWithInsertText){var d=u.getPromisedTypeOfPromise(n);if(d)for(var p=0,f=d.getApparentProperties();p<f.length;p++){l=f[p];u.isValidPropertyAccessForCompletions(a,d,l)&&ae(l,!0,r)}}}function ae(n,t,r){var i=e.firstDefined(n.declarations,function(n){return e.tryCast(e.getNameOfDeclaration(n),e.isComputedPropertyName)});if(i){var a=oe(i.expression),s=a&&u.getSymbolAtLocation(a),c=s&&X(s,E,u);if(c&&e.addToSeen(H,e.getSymbolId(c))){var l=V.length;V.push(c);var d=c.parent;if(d&&e.isExternalModuleSymbol(d)){var p={kind:_(6),moduleSymbol:d,isDefaultExport:!1,symbolName:c.name,exportName:c.name,fileName:e.isExternalModuleNameRelative(e.stripQuotes(d.name))?e.cast(d.valueDeclaration,e.isSourceFile).fileName:void 0};j[l]=p}else j[l]={kind:_(2)}}else o.includeCompletionsWithInsertText&&(m(n),f(n),V.push(n))}else m(n),f(n),V.push(n);function f(n){be(n)&&(K[e.getSymbolId(n)]=10)}function m(n){o.includeCompletionsWithInsertText&&(t&&e.addToSeen(H,e.getSymbolId(n))?j[V.length]={kind:_(8)}:r&&(j[V.length]={kind:16}))}function _(e){return r?16|e:e}}function oe(n){return e.isIdentifier(n)?n:e.isPropertyAccessExpression(n)?oe(n.expression):void 0}function ce(){var e=Ue()||Ve()||xe()||je()||Ke()||ye()||He()||Se()||(Le(),1);return 1===e}function ye(){return Je(E)?(M=5,F=!0,U=4,1):0}function Se(){var n=Xe(E),t=n&&u.getContextualType(n.attributes);if(!t)return 0;var r=n&&u.getContextualType(n.attributes,4);return V=e.concatenate(V,sn(de(t,r,n.attributes,u),n.attributes.properties)),rn(),M=3,F=!1,1}function xe(){return S?(F=!0,Oe(),1):0}function Le(){U=qe(E)?5:1,M=1,F=Ge(),b!==E&&e.Debug.assert(!!b,"Expected 'contextToken' to be defined when different from 'previousToken'.");var n=b!==E?b.getStart():a,t=Pe(E,n,r)||r;_=Ie(t);var i=2887656|(W?0:111551);V=e.concatenate(V,u.getSymbolsInScope(t,i)),e.Debug.assertEachIsDefined(V,"getSymbolsInScope() should all be defined");for(var s=0,c=V;s<c.length;s++){var l=c[s];u.isArgumentsSymbol(l)||e.some(l.declarations,function(e){return e.getSourceFile()===r})||(K[e.getSymbolId(l)]=15)}if(o.includeCompletionsWithInsertText&&300!==t.kind){var d=u.tryGetThisTypeAt(t,!1);if(d&&!ve(d,r,u))for(var p=0,f=pe(d,u);p<f.length;p++){l=f[p];j[V.length]={kind:1},V.push(l),K[e.getSymbolId(l)]=14}}Oe(),W&&(U=E&&e.isAssertionExpression(E.parent)?6:7)}function Ae(){return!!S||!G&&(!!o.includeCompletionsForModuleExports&&(!(!r.externalModuleIndicator&&!r.commonJsModuleIndicator)||(!!e.compilerOptionsIndicateEs6Modules(n.getCompilerOptions())||e.programContainsModules(n))))}function Ie(n){switch(n.kind){case 300:case 221:case 286:case 233:return!0;default:return e.isStatement(n)}}function Ne(){return m||!ke(E)&&(e.isPossiblyTypeArgumentPosition(E,r,u)||e.isPartOfTypeNode(k)||Re(E))}function ke(n){return n&&(112===n.kind&&(179===n.parent.kind||e.isTypeOfExpression(n.parent))||128===n.kind&&175===n.parent.kind)}function Re(n){if(n){var t=n.parent.kind;switch(n.kind){case 58:return 165===t||164===t||162===t||252===t||e.isFunctionLikeKind(t);case 63:return 257===t;case 127:return 227===t;case 29:return 176===t||209===t;case 94:return 161===t}}return!1}function Oe(){var t,i;if(Ae()&&(e.Debug.assert(!(null===s||void 0===s?void 0:s.data),"Should not run 'collectAutoImports' when faster path is available via `data`"),!s||s.source)){var a=null===(t=c.getModuleSpecifierCache)||void 0===t?void 0:t.call(c),u=b&&e.isIdentifier(b)?b.text.toLowerCase():"",d=e.getExportInfoMap(r,c,n,l),p=null===(i=c.getPackageJsonAutoImportProvider)||void 0===i?void 0:i.call(c),f=s?void 0:e.createPackageJsonImportFilter(r,o,c);h("collectAutoImports",c,n,r,o,!!S,function(n){d.forEach(r.path,function(t,r,i){if(e.isIdentifierText(r,e.getEmitScriptTarget(c.getCompilationSettings()))&&(s||!e.isStringANonContextualKeyword(r))&&(W||S||111551&t[0].targetFlags)&&(!W||790504&t[0].targetFlags)){var a=s&&e.some(t,function(n){return s.source===e.stripQuotes(n.moduleSymbol.name)});if(a||!s&&Ce(r,u)){var o=e.find(t,m);if(!o)return;var l=n.tryResolve(t,i)||{},d=l.exportInfo,p=void 0===d?o:d,f=l.moduleSpecifier,_=1===p.exportKind,g=_&&e.getLocalSymbolForExportDefault(p.symbol)||p.symbol;we(g,{kind:f?32:4,moduleSpecifier:f,symbolName:r,exportName:2===p.exportKind?"export=":p.symbol.name,fileName:p.moduleFileName,isDefaultExport:_,moduleSymbol:p.moduleSymbol,isFromPackageJson:p.isFromPackageJson})}}}),B=n.resolutionLimitExceeded()})}function m(t){var i=e.tryCast(t.moduleSymbol.valueDeclaration,e.isSourceFile);if(!i){var s=e.stripQuotes(t.moduleSymbol.name);return(!e.JsTyping.nodeCoreModules.has(s)||e.startsWith(s,"node:")===e.shouldUseUriStyleNodeCoreModules(r,n))&&(!f||f.allowsImportingAmbientModule(t.moduleSymbol,z(t.isFromPackageJson)))}return e.isImportableFile(t.isFromPackageJson?p:n,r,i,o,f,z(t.isFromPackageJson),a)}}function we(n,t){var r=e.getSymbolId(n);15!==K[r]&&(j[V.length]=t,K[r]=S?11:16,V.push(n))}function Pe(n,t,r){var i=n;while(i&&!e.positionBelongsToNode(i,t,r))i=i.parent;return i}function Me(n){var r=e.timestamp(),i=Be(n)||Ye(n)||$e(n)||Fe(n)||e.isBigIntLiteral(n);return t("getCompletionsAtPosition: isCompletionListBlocker: "+(e.timestamp()-r)),i}function Fe(e){if(11===e.kind)return!0;if(31===e.kind&&e.parent){if(278===e.parent.kind)return 278!==k.parent.kind;if(279===e.parent.kind||277===e.parent.kind)return!!e.parent.parent&&276===e.parent.parent.kind}return!1}function Ge(){if(E){var e=E.parent.kind;switch(le(E)){case 27:return 206===e||169===e||207===e||202===e||219===e||177===e||203===e;case 20:return 206===e||169===e||207===e||210===e||189===e;case 22:return 202===e||174===e||160===e;case 140:case 141:return!0;case 24:return 259===e;case 18:return 255===e||203===e;case 63:return 252===e||219===e;case 15:return 221===e;case 16:return 231===e;case 123:case 121:case 122:return 165===e}}return!1}function Be(n){return(e.isRegularExpressionLiteral(n)||e.isStringTextContainingNode(n))&&(e.rangeContainsPositionExclusive(e.createTextRangeFromSpan(e.createTextSpanFromNode(n)),a)||a===n.end&&(!!n.isUnterminated||e.isRegularExpressionLiteral(n)))}function Ue(){var n=me(E);if(!n)return 0;var t=e.isIntersectionTypeNode(n.parent)?n.parent:void 0,r=t||n,i=_e(r,u);if(!i)return 0;var a=u.getTypeFromTypeNode(r),o=pe(i,u),s=pe(a,u),c=new e.Set;return s.forEach(function(e){return c.add(e.escapedName)}),V=e.concatenate(V,e.filter(o,function(e){return!c.has(e.escapedName)})),M=0,F=!0,1}function Ve(){var n,t,r=We(E);if(!r)return 0;if(M=0,203===r.kind){var i=Ee(r,u);if(void 0===i)return 16777216&r.flags?2:(G=!0,0);var a=u.getContextualType(r,4),o=(a||i).getStringIndexType(),s=(a||i).getNumberIndexType();if(F=!!o||!!s,n=de(i,a,r,u),t=r.properties,0===n.length&&!s)return G=!0,0}else{e.Debug.assert(199===r.kind),F=!1;var c=e.getRootDeclaration(r.parent);if(!e.isVariableLike(c))return e.Debug.fail("Root declaration is not variable-like.");var l=e.hasInitializer(c)||e.hasType(c)||242===c.parent.parent.kind;if(l||162!==c.kind||(e.isExpression(c.parent)?l=!!u.getContextualType(c.parent):167!==c.parent.kind&&171!==c.parent.kind||(l=e.isExpression(c.parent.parent)&&!!u.getContextualType(c.parent.parent))),l){var d=u.getTypeAtLocation(r);if(!d)return 2;var p=e.getContainingClass(r);n=u.getPropertiesOfType(d).filter(function(n){return!(24&e.getDeclarationModifierFlagsFromSymbol(n))||p&&e.contains(d.symbol.declarations,p)}),t=r.elements}}return n&&n.length>0&&(V=e.concatenate(V,nn(n,e.Debug.checkDefined(t)))),rn(),1}function je(){var n=!E||18!==E.kind&&27!==E.kind?void 0:e.tryCast(E.parent,e.isNamedImportsOrExports);if(!n)return 0;var t=(267===n.kind?n.parent.parent:n.parent).moduleSpecifier;if(!t)return 267===n.kind?2:0;var r=u.getSymbolAtLocation(t);if(!r)return 2;M=3,F=!1;var i=u.getExportsAndPropertiesOfModule(r),a=new e.Set(n.elements.filter(function(e){return!cn(e)}).map(function(e){return(e.propertyName||e.name).escapedText}));return V=e.concatenate(V,i.filter(function(e){return"default"!==e.escapedName&&!a.has(e.escapedName)})),1}function Ke(){var n,t=!E||18!==E.kind&&27!==E.kind?void 0:e.tryCast(E.parent,e.isNamedExports);if(!t)return 0;var r=e.findAncestor(t,e.or(e.isSourceFile,e.isModuleDeclaration));return M=5,F=!1,null===(n=r.locals)||void 0===n||n.forEach(function(n,t){var i,a;V.push(n),(null===(a=null===(i=r.symbol)||void 0===i?void 0:i.exports)||void 0===a?void 0:a.has(t))&&(K[e.getSymbolId(n)]=12)}),1}function He(){var n=fe(r,E,k,a);if(!n)return 0;if(M=3,F=!0,U=41===E.kind?0:e.isClassLike(n)?2:3,!e.isClassLike(n))return 1;var t=26===E.kind?E.parent.parent:E.parent,i=e.isClassElement(t)?e.getEffectiveModifierFlags(t):0;if(79===E.kind&&!cn(E))switch(E.getText()){case"private":i|=8;break;case"static":i|=32;break;case"override":i|=16384;break}if(e.isClassStaticBlockDeclaration(t)&&(i|=32),!(8&i)){var o=e.isClassLike(n)&&16384&i?e.singleElementArray(e.getEffectiveBaseTypeNode(n)):e.getAllSuperTypeNodes(n),s=e.flatMap(o,function(e){var t=u.getTypeAtLocation(e);return 32&i?(null===t||void 0===t?void 0:t.symbol)&&u.getPropertiesOfType(u.getTypeOfSymbolAtLocation(t.symbol,n)):t&&u.getPropertiesOfType(t)});V=e.concatenate(V,on(s,n.members,i))}return 1}function We(n){if(n){var t=n.parent;switch(n.kind){case 18:case 27:if(e.isObjectLiteralExpression(t)||e.isObjectBindingPattern(t))return t;break;case 41:return e.isMethodDeclaration(t)?e.tryCast(t.parent,e.isObjectLiteralExpression):void 0;case 79:return"async"===n.text&&e.isShorthandPropertyAssignment(n.parent)?n.parent.parent:void 0}}}function ze(n){return!!n.parent&&e.isParameter(n.parent)&&e.isConstructorDeclaration(n.parent.parent)&&(e.isParameterPropertyModifier(n.kind)||e.isDeclarationName(n))}function Je(n){if(n){var t=n.parent;switch(n.kind){case 20:case 27:return e.isConstructorDeclaration(n.parent)?n.parent:void 0;default:if(ze(n))return t.parent}}}function qe(n){if(n){var t,r=e.findAncestor(n.parent,function(n){return e.isClassLike(n)?"quit":!(!e.isFunctionLikeDeclaration(n)||t!==n.body)||(t=n,!1)});return r&&r}}function Xe(n){if(n){var t=n.parent;switch(n.kind){case 31:case 30:case 43:case 79:case 204:case 284:case 283:case 285:if(t&&(277===t.kind||278===t.kind)){if(31===n.kind){var i=e.findPrecedingToken(n.pos,r,void 0);if(!t.typeArguments||i&&43===i.kind)break}return t}if(283===t.kind)return t.parent.parent;break;case 10:if(t&&(283===t.kind||285===t.kind))return t.parent.parent;break;case 19:if(t&&286===t.kind&&t.parent&&283===t.parent.kind)return t.parent.parent.parent;if(t&&285===t.kind)return t.parent.parent;break}}}function Ye(n){var t=n.parent,r=t.kind;switch(n.kind){case 27:return 252===r||en(n)||235===r||258===r||Ze(r)||256===r||200===r||257===r||e.isClassLike(t)&&!!t.typeParameters&&t.typeParameters.end>=n.pos;case 24:return 200===r;case 58:return 201===r;case 22:return 200===r;case 20:return 290===r||Ze(r);case 18:return 258===r;case 29:return 255===r||224===r||256===r||257===r||e.isFunctionLikeKind(r);case 124:return 165===r&&!e.isClassLike(t.parent);case 25:return 162===r||!!t.parent&&200===t.parent.kind;case 123:case 121:case 122:return 162===r&&!e.isConstructorDeclaration(t.parent);case 127:return 268===r||273===r||266===r;case 135:case 147:return!ge(n);case 84:case 92:case 118:case 98:case 113:case 100:case 119:case 85:case 136:case 150:return!0;case 41:return e.isFunctionLike(n.parent)&&!e.isMethodDeclaration(n.parent)}if(se(le(n))&&ge(n))return!1;if(ze(n)&&(!e.isIdentifier(n)||e.isParameterPropertyModifier(le(n))||cn(n)))return!1;switch(le(n)){case 126:case 84:case 85:case 134:case 92:case 98:case 118:case 119:case 121:case 122:case 123:case 124:case 113:return!0;case 130:return e.isPropertyDeclaration(n.parent)}var i=e.findAncestor(n.parent,e.isClassLike);if(i&&n===b&&Qe(n,a))return!1;var o=e.getAncestor(n.parent,165);if(o&&n!==b&&e.isClassLike(b.parent.parent)&&a<=b.end){if(Qe(n,b.end))return!1;if(63!==n.kind&&(e.isInitializedProperty(o)||e.hasType(o)))return!0}return e.isDeclarationName(n)&&!e.isShorthandPropertyAssignment(n.parent)&&!e.isJsxAttribute(n.parent)&&!(e.isClassLike(n.parent)&&(n!==b||a>b.end))}function Qe(n,t){return 63!==n.kind&&(26===n.kind||!e.positionsAreOnSameLine(n.end,t,r))}function Ze(n){return e.isFunctionLikeKind(n)&&169!==n}function $e(e){if(8===e.kind){var n=e.getFullText();return"."===n.charAt(n.length-1)}return!1}function en(n){return 253===n.parent.kind&&!e.isPossiblyTypeArgumentPosition(n,r,u)}function nn(n,t){if(0===t.length)return n;for(var r=new e.Set,i=new e.Set,a=0,o=t;a<o.length;a++){var s=o[a];if((291===s.kind||292===s.kind||201===s.kind||167===s.kind||170===s.kind||171===s.kind||293===s.kind)&&!cn(s)){var c=void 0;if(e.isSpreadAssignment(s))tn(s,r);else if(e.isBindingElement(s)&&s.propertyName)79===s.propertyName.kind&&(c=s.propertyName.escapedText);else{var l=e.getNameOfDeclaration(s);c=l&&e.isPropertyNameLiteral(l)?e.getEscapedTextOfIdentifierOrLiteral(l):void 0}void 0!==c&&i.add(c)}}var u=n.filter(function(e){return!i.has(e.escapedName)});return an(r,u),u}function tn(e,n){var t=e.expression,r=u.getSymbolAtLocation(t),i=r&&u.getTypeOfSymbolAtLocation(r,t),a=i&&i.properties;a&&a.forEach(function(e){n.add(e.name)})}function rn(){V.forEach(function(n){var t;if(16777216&n.flags){var r=e.getSymbolId(n);K[r]=null!==(t=K[r])&&void 0!==t?t:12}})}function an(n,t){if(0!==n.size)for(var r=0,i=t;r<i.length;r++){var a=i[r];n.has(a.name)&&(K[e.getSymbolId(a)]=13)}}function on(n,t,r){for(var i=new e.Set,a=0,o=t;a<o.length;a++){var s=o[a];if((165===s.kind||167===s.kind||170===s.kind||171===s.kind)&&(!cn(s)&&!e.hasEffectiveModifier(s,8)&&e.isStatic(s)===!!(32&r))){var c=e.getPropertyNameForPropertyNameNode(s.name);c&&i.add(c)}}return n.filter(function(n){return!i.has(n.escapedName)&&!!n.declarations&&!(8&e.getDeclarationModifierFlagsFromSymbol(n))&&!(n.valueDeclaration&&e.isPrivateIdentifierClassElementDeclaration(n.valueDeclaration))})}function sn(n,t){for(var r=new e.Set,i=new e.Set,a=0,o=t;a<o.length;a++){var s=o[a];cn(s)||(283===s.kind?r.add(s.name.escapedText):e.isJsxSpreadAttribute(s)&&tn(s,i))}var c=n.filter(function(e){return!r.has(e.escapedName)});return an(i,c),c}function cn(e){return e.getStart(r)<=a&&a<=e.getEnd()}}function Z(n,t){var r=e.findPrecedingToken(n,t);if(r&&n<=r.end&&(e.isMemberName(r)||e.isKeyword(r.kind))){var i=e.findPrecedingToken(r.getFullStart(),t,void 0);return{contextToken:i,previousToken:r}}return{contextToken:r,previousToken:r}}function $(n,t,r,i){var a=t.isPackageJsonImport?i.getPackageJsonAutoImportProvider():r,o=a.getTypeChecker(),s=t.ambientModuleName?o.tryFindAmbientModule(t.ambientModuleName):t.fileName?o.getMergedSymbol(e.Debug.checkDefined(a.getSourceFile(t.fileName)).symbol):void 0;if(s){var c="export="===t.exportName?o.resolveExternalModuleSymbol(s):o.tryGetMemberInModuleExportsAndProperties(t.exportName,s);if(c){var l="default"===t.exportName;return c=l&&e.getLocalSymbolForExportDefault(c)||c,{symbol:c,origin:{kind:t.moduleSpecifier?32:4,moduleSymbol:s,symbolName:n,isDefaultExport:l,exportName:t.exportName,fileName:t.fileName,isFromPackageJson:!!t.isPackageJsonImport}}}}}function ee(n,t,r,i,a){var o=m(r)?r.symbolName:n.name;if(!(void 0===o||1536&n.flags&&e.isSingleOrDoubleQuote(o.charCodeAt(0))||e.isKnownSymbol(n))){var s={name:o,needsConvertPropertyAccess:!1};if(e.isIdentifierText(o,t,a?1:0)||n.valueDeclaration&&e.isPrivateIdentifierClassElementDeclaration(n.valueDeclaration))return s;switch(i){case 3:return;case 0:return{name:JSON.stringify(o),needsConvertPropertyAccess:!1};case 2:case 1:return 32===o.charCodeAt(0)?void 0:{name:o,needsConvertPropertyAccess:!0};case 5:case 4:return s;default:e.Debug.assertNever(i)}}}n.moduleSpecifierResolutionLimit=100,n.moduleSpecifierResolutionCacheAttemptLimit=1e3,function(e){e["LocalDeclarationPriority"]="10",e["LocationPriority"]="11",e["OptionalMember"]="12",e["MemberDeclaredBySpreadAssignment"]="13",e["SuggestedClassMembers"]="14",e["GlobalsOrKeywords"]="15",e["AutoImportSuggestions"]="16",e["JavascriptIdentifiers"]="17",e["DeprecatedLocalDeclarationPriority"]="18",e["DeprecatedLocationPriority"]="19",e["DeprecatedOptionalMember"]="20",e["DeprecatedMemberDeclaredBySpreadAssignment"]="21",e["DeprecatedSuggestedClassMembers"]="22",e["DeprecatedGlobalsOrKeywords"]="23",e["DeprecatedAutoImportSuggestions"]="24"}(t=n.SortText||(n.SortText={})),function(e){e[e["LocalDeclarationPriority"]=10]="LocalDeclarationPriority",e[e["LocationPriority"]=11]="LocationPriority",e[e["OptionalMember"]=12]="OptionalMember",e[e["MemberDeclaredBySpreadAssignment"]=13]="MemberDeclaredBySpreadAssignment",e[e["SuggestedClassMembers"]=14]="SuggestedClassMembers",e[e["GlobalsOrKeywords"]=15]="GlobalsOrKeywords",e[e["AutoImportSuggestions"]=16]="AutoImportSuggestions",e[e["_JavaScriptIdentifiers"]=17]="_JavaScriptIdentifiers",e[e["_DeprecatedStart"]=18]="_DeprecatedStart",e[e["_First"]=10]="_First",e[e["DeprecatedOffset"]=8]="DeprecatedOffset"}(r||(r={})),function(e){e["ThisProperty"]="ThisProperty/"}(i=n.CompletionSource||(n.CompletionSource={})),function(e){e[e["ThisType"]=1]="ThisType",e[e["SymbolMember"]=2]="SymbolMember",e[e["Export"]=4]="Export",e[e["Promise"]=8]="Promise",e[e["Nullable"]=16]="Nullable",e[e["ResolvedExport"]=32]="ResolvedExport",e[e["SymbolMemberNoExport"]=2]="SymbolMemberNoExport",e[e["SymbolMemberExport"]=6]="SymbolMemberExport"}(a||(a={})),function(e){e[e["None"]=0]="None",e[e["All"]=1]="All",e[e["ClassElementKeywords"]=2]="ClassElementKeywords",e[e["InterfaceElementKeywords"]=3]="InterfaceElementKeywords",e[e["ConstructorParameterKeywords"]=4]="ConstructorParameterKeywords",e[e["FunctionLikeBodyKeywords"]=5]="FunctionLikeBodyKeywords",e[e["TypeAssertionKeywords"]=6]="TypeAssertionKeywords",e[e["TypeKeywords"]=7]="TypeKeywords",e[e["Last"]=7]="Last"}(s||(s={})),function(e){e[e["Continue"]=0]="Continue",e[e["Success"]=1]="Success",e[e["Fail"]=2]="Fail"}(c||(c={})),n.getCompletionsAtPosition=v,n.getCompletionEntriesFromSymbols=F,n.getCompletionEntryDetails=V,n.createCompletionDetailsForSymbol=K,n.createCompletionDetails=H,n.getCompletionEntrySymbol=z,function(e){e[e["Data"]=0]="Data",e[e["JsDocTagName"]=1]="JsDocTagName",e[e["JsDocTag"]=2]="JsDocTag",e[e["JsDocParameterName"]=3]="JsDocParameterName",e[e["Keywords"]=4]="Keywords"}(l||(l={})),function(e){e[e["ObjectPropertyDeclaration"]=0]="ObjectPropertyDeclaration",e[e["Global"]=1]="Global",e[e["PropertyAccess"]=2]="PropertyAccess",e[e["MemberLike"]=3]="MemberLike",e[e["String"]=4]="String",e[e["None"]=5]="None"}(n.CompletionKind||(n.CompletionKind={}));var ne=[],te=e.memoize(function(){for(var n=[],r=81;r<=158;r++)n.push({name:e.tokenToString(r),kind:"keyword",kindModifiers:"",sortText:t.GlobalsOrKeywords});return n});function re(n,t){if(!t)return ie(n);var r=n+7+1;return ne[r]||(ne[r]=ie(n).filter(function(n){return!ae(e.stringToToken(n.name))}))}function ie(n){return ne[n]||(ne[n]=te().filter(function(t){var r=e.stringToToken(t.name);switch(n){case 0:return!1;case 1:return ce(r)||134===r||140===r||150===r||141===r||e.isTypeKeyword(r)&&151!==r;case 5:return ce(r);case 2:return se(r);case 3:return oe(r);case 4:return e.isParameterPropertyModifier(r);case 6:return e.isTypeKeyword(r)||85===r;case 7:return e.isTypeKeyword(r);default:return e.Debug.assertNever(n)}}))}function ae(e){switch(e){case 126:case 129:case 156:case 132:case 134:case 92:case 155:case 117:case 136:case 118:case 138:case 139:case 140:case 141:case 142:case 145:case 146:case 157:case 121:case 122:case 123:case 143:case 148:case 149:case 150:case 152:case 153:return!0;default:return!1}}function oe(e){return 143===e}function se(n){switch(n){case 126:case 133:case 135:case 147:case 130:case 134:case 157:return!0;default:return e.isClassMemberModifier(n)}}function ce(n){return 130===n||131===n||127===n||!e.isContextualKeyword(n)&&!se(n)}function le(n){return e.isIdentifier(n)?n.originalKeywordKind||0:n.kind}function ue(n,t){var r=e.findAncestor(n,e.isJSDoc);return r&&r.tags&&(e.rangeContainsPosition(r,t)?e.findLast(r.tags,function(e){return e.pos<t}):void 0)}function de(n,t,r,i){var a=t&&t!==n,o=!a||3&t.flags?n:i.getUnionType([n,t]),s=o.isUnion()?i.getAllPossiblePropertiesOfTypes(o.types.filter(function(n){return!(131068&n.flags||i.isArrayLikeType(n)||e.typeHasCallOrConstructSignatures(n,i)||i.isTypeInvalidDueToUnionDiscriminant(n,r))})):o.getApparentProperties();return a?s.filter(c):s;function c(n){return e.some(n.declarations,function(e){return e.parent!==r})}}function pe(n,t){return n.isUnion()?e.Debug.checkEachDefined(t.getAllPossiblePropertiesOfTypes(n.types),"getAllPossiblePropertiesOfTypes() should all be defined"):e.Debug.checkEachDefined(n.getApparentProperties(),"getApparentProperties() should all be defined")}function fe(n,t,r,i){switch(r.kind){case 343:return e.tryCast(r.parent,e.isObjectTypeDeclaration);case 1:var a=e.tryCast(e.lastOrUndefined(e.cast(r.parent,e.isSourceFile).statements),e.isObjectTypeDeclaration);if(a&&!e.findChildOfKind(a,19,n))return a;break;case 79:if(e.isPropertyDeclaration(r.parent)&&r.parent.initializer===r)return;if(ge(r))return e.findAncestor(r,e.isObjectTypeDeclaration)}if(t){if(133===r.kind||e.isIdentifier(t)&&e.isPropertyDeclaration(t.parent)&&e.isClassLike(r))return e.findAncestor(t,e.isClassLike);switch(t.kind){case 63:return;case 26:case 19:return ge(r)&&r.parent.name===r?r.parent.parent:e.tryCast(r,e.isObjectTypeDeclaration);case 18:case 27:return e.tryCast(t.parent,e.isObjectTypeDeclaration);default:if(!ge(t))return e.getLineAndCharacterOfPosition(n,t.getEnd()).line!==e.getLineAndCharacterOfPosition(n,i).line&&e.isObjectTypeDeclaration(r)?r:void 0;var o=e.isClassLike(t.parent.parent)?se:oe;return o(t.kind)||41===t.kind||e.isIdentifier(t)&&o(e.stringToToken(t.text))?t.parent.parent:void 0}}}function me(n){if(n){var t=n.parent;switch(n.kind){case 18:if(e.isTypeLiteralNode(t))return t;break;case 26:case 27:case 79:if(164===t.kind&&e.isTypeLiteralNode(t.parent))return t.parent;break}}}function _e(n,t){if(n){if(e.isTypeNode(n)&&e.isTypeReferenceType(n.parent))return t.getTypeArgumentConstraint(n);var r=_e(n.parent,t);if(r)switch(n.kind){case 164:return t.getTypeOfPropertyOfContextualType(r,n.symbol.escapedName);case 186:case 180:case 185:return r}}}function ge(n){return n.parent&&e.isClassOrTypeElement(n.parent)&&e.isObjectTypeDeclaration(n.parent.parent)}function ye(n,t,r,i){switch(t){case".":case"@":return!0;case'"':case"'":case"`":return!!r&&e.isStringLiteralOrTemplate(r)&&i===r.getStart(n)+1;case"#":return!!r&&e.isPrivateIdentifier(r)&&!!e.getContainingClass(r);case"<":return!!r&&29===r.kind&&(!e.isBinaryExpression(r.parent)||he(r.parent));case"/":return!!r&&(e.isStringLiteralLike(r)?!!e.tryGetImportFromModuleSpecifier(r):43===r.kind&&e.isJsxClosingElement(r.parent));case" ":return!!r&&e.isImportKeyword(r)&&300===r.parent.kind;default:return e.Debug.assertNever(t)}}function he(n){var t=n.left;return e.nodeIsMissing(t)}function ve(e,n,t){var r=t.resolveName("self",void 0,111551,!1);if(r&&t.getTypeOfSymbolAtLocation(r,n)===e)return!0;var i=t.resolveName("global",void 0,111551,!1);if(i&&t.getTypeOfSymbolAtLocation(i,n)===e)return!0;var a=t.resolveName("globalThis",void 0,111551,!1);return!(!a||t.getTypeOfSymbolAtLocation(a,n)!==e)}function be(n){return!!(n.valueDeclaration&&32&e.getEffectiveModifierFlags(n.valueDeclaration)&&e.isClassLike(n.valueDeclaration.parent))}function Ee(n,t){var r=t.getContextualType(n);return r||(e.isBinaryExpression(n.parent)&&63===n.parent.operatorToken.kind&&n===n.parent.left?t.getTypeAtLocation(n.parent):void 0)}function Te(n){var t=r();return 154===t||t&&e.rangeIsOnSingleLine(t,t.getSourceFile())?t:void 0;function r(){var t=n.parent;return e.isImportEqualsDeclaration(t)?Se(t.moduleReference)?t:void 0:e.isNamedImports(t)||e.isNamespaceImport(t)?Se(t.parent.parent.moduleSpecifier)&&(e.isNamespaceImport(t)||t.elements.length<2)&&!t.parent.name?19===n.kind||79===n.kind?154:t.parent.parent:void 0:e.isImportKeyword(n)&&e.isSourceFile(t)?n:e.isImportKeyword(n)&&e.isImportDeclaration(t)&&Se(t.moduleSpecifier)?t:void 0}}function Se(n){var t;return!!e.nodeIsMissing(n)||!(null===(t=e.tryCast(e.isExternalModuleReference(n)?n.expression:n,e.isStringLiteralLike))||void 0===t?void 0:t.text)}function xe(n){var t=e.findAncestor(n,function(n){return e.isFunctionBlock(n)||Le(n)||e.isBindingPattern(n)?"quit":e.isVariableDeclaration(n)});return t}function Le(n){return n.parent&&e.isArrowFunction(n.parent)&&n.parent.body===n}function De(n,t,r){void 0===r&&(r=new e.Map);var i=e.skipAlias(n.exportSymbol||n,t);return!!(788968&i.flags)||t.isUnknownSymbol(i)||!!(1536&i.flags)&&e.addToSeen(r,e.getSymbolId(i))&&t.getExportsOfModule(i).some(function(e){return De(e,t,r)})}function Ae(n,t){var r=e.skipAlias(n,t).declarations;return!!e.length(r)&&e.every(r,e.isDeprecatedDeclaration)}function Ce(e,n){if(0===n.length)return!0;for(var t,r=!1,i=0,a=e.length,o=0;o<a;o++){var s=e.charCodeAt(o),c=n.charCodeAt(i);if((s===c||s===Ie(c))&&(r||(r=void 0===t||97<=t&&t<=122&&65<=s&&s<=90||95===t&&95!==s),r&&i++,i===n.length))return!0;t=s}return!1}function Ie(e){return 97<=e&&e<=122?e-32:e}n.getPropertiesForObjectExpression=de})(e.Completions||(e.Completions={}))}(d||(d={})),function(e){(function(n){function t(n,t,a,s,c){var l=e.getTouchingPropertyName(a,s);if(l.parent&&(e.isJsxOpeningElement(l.parent)&&l.parent.tagName===l||e.isJsxClosingElement(l.parent))){var u=l.parent.parent,d=u.openingElement,p=u.closingElement,f=[d,p].map(function(e){var n=e.tagName;return r(n,a)});return[{fileName:a.fileName,highlightSpans:f}]}return i(s,l,n,t,c)||o(l,a)}function r(n,t){return{fileName:t.fileName,textSpan:e.createTextSpanFromNode(n,t),kind:"none"}}function i(n,t,r,i,a){var o=new e.Set(a.map(function(e){return e.fileName})),s=e.FindAllReferences.getReferenceEntriesForNode(n,t,r,a,i,void 0,o);if(s){var c=e.arrayToMultiMap(s.map(e.FindAllReferences.toHighlightSpan),function(e){return e.fileName},function(e){return e.span}),l=e.createGetCanonicalFileName(r.useCaseSensitiveFileNames());return e.mapDefined(e.arrayFrom(c.entries()),function(n){var t=n[0],i=n[1];if(!o.has(t)){if(!r.redirectTargetsMap.has(e.toPath(t,r.getCurrentDirectory(),l)))return;var s=r.getSourceFile(t),c=e.find(a,function(e){return!!e.redirectInfo&&e.redirectInfo.redirectTarget===s});t=c.fileName,e.Debug.assert(o.has(t))}return{fileName:t,highlightSpans:i}})}}function o(e,n){var t=s(e,n);return t&&[{fileName:n.fileName,highlightSpans:t}]}function s(n,t){switch(n.kind){case 99:case 91:return e.isIfStatement(n.parent)?D(n.parent,t):void 0;case 105:return o(n.parent,e.isReturnStatement,T);case 109:return o(n.parent,e.isThrowStatement,E);case 111:case 83:case 96:var i=83===n.kind?n.parent.parent:n.parent;return o(i,e.isTryStatement,b);case 107:return o(n.parent,e.isSwitchStatement,v);case 82:case 88:return e.isDefaultClause(n.parent)||e.isCaseClause(n.parent)?o(n.parent.parent.parent,e.isSwitchStatement,v):void 0;case 81:case 86:return o(n.parent,e.isBreakOrContinueStatement,h);case 97:case 115:case 90:return o(n.parent,function(n){return e.isIterationStatement(n,!0)},y);case 133:return a(e.isConstructorDeclaration,[133]);case 135:case 147:return a(e.isAccessor,[135,147]);case 131:return o(n.parent,e.isAwaitExpression,S);case 130:return s(S(n));case 125:return s(x(n));default:return e.isModifierKind(n.kind)&&(e.isDeclaration(n.parent)||e.isVariableStatement(n.parent))?s(m(n.kind,n.parent)):void 0}function a(r,i){return o(n.parent,r,function(n){return e.mapDefined(n.symbol.declarations,function(n){return r(n)?e.find(n.getChildren(t),function(n){return e.contains(i,n.kind)}):void 0})})}function o(e,n,r){return n(e)?s(r(e,t)):void 0}function s(e){return e&&e.map(function(e){return r(e,t)})}}function c(n){return e.isThrowStatement(n)?[n]:e.isTryStatement(n)?e.concatenate(n.catchClause?c(n.catchClause):n.tryBlock&&c(n.tryBlock),n.finallyBlock&&c(n.finallyBlock)):e.isFunctionLike(n)?void 0:d(n,c)}function l(n){var t=n;while(t.parent){var r=t.parent;if(e.isFunctionBlock(r)||300===r.kind)return r;if(e.isTryStatement(r)&&r.tryBlock===t&&r.catchClause)return t;t=r}}function u(n){return e.isBreakOrContinueStatement(n)?[n]:e.isFunctionLike(n)?void 0:d(n,u)}function d(n,t){var r=[];return n.forEachChild(function(n){var i=t(n);void 0!==i&&r.push.apply(r,e.toArray(i))}),r}function p(e,n){var t=f(n);return!!t&&t===e}function f(n){return e.findAncestor(n,function(t){switch(t.kind){case 247:if(243===n.kind)return!1;case 240:case 241:case 242:case 239:case 238:return!n.label||C(t,n.label.escapedText);default:return e.isFunctionLike(t)&&"quit"}})}function m(n,t){return e.mapDefined(_(t,e.modifierToFlag(n)),function(t){return e.findModifier(t,n)})}function _(n,t){var r=n.parent;switch(r.kind){case 260:case 300:case 233:case 287:case 288:return 128&t&&e.isClassDeclaration(n)?a(a([],n.members,!0),[n],!1):r.statements;case 169:case 167:case 254:return a(a([],r.parameters,!0),e.isClassLike(r.parent)?r.parent.members:[],!0);case 255:case 224:case 256:case 180:var i=r.members;if(92&t){var o=e.find(r.members,e.isConstructorDeclaration);if(o)return a(a([],i,!0),o.parameters,!0)}else if(128&t)return a(a([],i,!0),[r],!1);return i;case 203:return;default:e.Debug.assertNever(r,"Invalid container kind.")}}function g(n,t){for(var r=[],i=2;i<arguments.length;i++)r[i-2]=arguments[i];return!(!t||!e.contains(r,t.kind))&&(n.push(t),!0)}function y(n){var t=[];if(g(t,n.getFirstToken(),97,115,90)&&238===n.kind)for(var r=n.getChildren(),i=r.length-1;i>=0;i--)if(g(t,r[i],115))break;return e.forEach(u(n.statement),function(e){p(n,e)&&g(t,e.getFirstToken(),81,86)}),t}function h(e){var n=f(e);if(n)switch(n.kind){case 240:case 241:case 242:case 238:case 239:return y(n);case 247:return v(n)}}function v(n){var t=[];return g(t,n.getFirstToken(),107),e.forEach(n.caseBlock.clauses,function(r){g(t,r.getFirstToken(),82,88),e.forEach(u(r),function(e){p(n,e)&&g(t,e.getFirstToken(),81)})}),t}function b(n,t){var r=[];if(g(r,n.getFirstToken(),111),n.catchClause&&g(r,n.catchClause.getFirstToken(),83),n.finallyBlock){var i=e.findChildOfKind(n,96,t);g(r,i,96)}return r}function E(n,t){var r=l(n);if(r){var i=[];return e.forEach(c(r),function(n){i.push(e.findChildOfKind(n,109,t))}),e.isFunctionBlock(r)&&e.forEachReturnStatement(r,function(n){i.push(e.findChildOfKind(n,105,t))}),i}}function T(n,t){var r=e.getContainingFunction(n);if(r){var i=[];return e.forEachReturnStatement(e.cast(r.body,e.isBlock),function(n){i.push(e.findChildOfKind(n,105,t))}),e.forEach(c(r.body),function(n){i.push(e.findChildOfKind(n,109,t))}),i}}function S(n){var t=e.getContainingFunction(n);if(t){var r=[];return t.modifiers&&t.modifiers.forEach(function(e){g(r,e,130)}),e.forEachChild(t,function(n){L(n,function(n){e.isAwaitExpression(n)&&g(r,n.getFirstToken(),131)})}),r}}function x(n){var t=e.getContainingFunction(n);if(t){var r=[];return e.forEachChild(t,function(n){L(n,function(n){e.isYieldExpression(n)&&g(r,n.getFirstToken(),125)})}),r}}function L(n,t){t(n),e.isFunctionLike(n)||e.isClassLike(n)||e.isInterfaceDeclaration(n)||e.isModuleDeclaration(n)||e.isTypeAliasDeclaration(n)||e.isTypeNode(n)||e.forEachChild(n,function(e){return L(e,t)})}function D(n,t){for(var i=A(n,t),a=[],o=0;o<i.length;o++){if(91===i[o].kind&&o<i.length-1){for(var s=i[o],c=i[o+1],l=!0,u=c.getStart(t)-1;u>=s.end;u--)if(!e.isWhiteSpaceSingleLine(t.text.charCodeAt(u))){l=!1;break}if(l){a.push({fileName:t.fileName,textSpan:e.createTextSpanFromBounds(s.getStart(),c.end),kind:"reference"}),o++;continue}}a.push(r(i[o],t))}return a}function A(n,t){var r=[];while(e.isIfStatement(n.parent)&&n.parent.elseStatement===n)n=n.parent;while(1){var i=n.getChildren(t);g(r,i[0],99);for(var a=i.length-1;a>=0;a--)if(g(r,i[a],91))break;if(!n.elseStatement||!e.isIfStatement(n.elseStatement))break;n=n.elseStatement}return r}function C(n,t){return!!e.findAncestor(n.parent,function(n){return e.isLabeledStatement(n)?n.label.escapedText===t:"quit"})}n.getDocumentHighlights=t})(e.DocumentHighlights||(e.DocumentHighlights={}))}(d||(d={})),function(e){function n(e){return!!e.sourceFile}function t(e,n){return r(e,n)}function r(t,r,a){void 0===r&&(r="");var o=new e.Map,s=e.createGetCanonicalFileName(!!t);function c(){var t=e.arrayFrom(o.keys()).filter(function(e){return e&&"_"===e.charAt(0)}).map(function(e){var t=o.get(e),r=[];return t.forEach(function(e,t){n(e)?r.push({name:t,scriptKind:e.sourceFile.scriptKind,refCount:e.languageServiceRefCount}):e.forEach(function(e,n){return r.push({name:t,scriptKind:n,refCount:e.languageServiceRefCount})})}),r.sort(function(e,n){return n.refCount-e.refCount}),{bucket:e,sourceFiles:r}});return JSON.stringify(t,void 0,2)}function l(n,t,a,o,c){var l=e.toPath(n,r,s),d=i(t);return u(n,l,t,d,a,o,c)}function u(e,n,t,r,i,a,o){return m(e,n,t,r,i,a,!0,o)}function d(n,t,a,o,c){var l=e.toPath(n,r,s),u=i(t);return p(n,l,t,u,a,o,c)}function p(e,n,t,r,i,a,o){return m(e,n,t,r,i,a,!1,o)}function f(t,r){var i=n(t)?t:t.get(e.Debug.checkDefined(r,"If there are more than one scriptKind's for same document the scriptKind should be provided"));return e.Debug.assert(void 0===r||!i||i.sourceFile.scriptKind===r,"Script kind should match provided ScriptKind:"+r+" and sourceFile.scriptKind: "+(null===i||void 0===i?void 0:i.sourceFile.scriptKind)+", !entry: "+!i),i}function m(t,r,i,s,c,l,u,d){d=e.ensureScriptKind(t,d);var p=6===d?100:i.target||1,m=e.getOrUpdate(o,s,function(){return new e.Map}),_=m.get(r),g=_&&f(_,d);if(!g&&a){var y=a.getDocument(s,r);y&&(e.Debug.assert(u),g={sourceFile:y,languageServiceRefCount:0},h())}if(g)g.sourceFile.version!==l&&(g.sourceFile=e.updateLanguageServiceSourceFile(g.sourceFile,c,l,c.getChangeRange(g.sourceFile.scriptSnapshot)),a&&a.setDocument(s,r,g.sourceFile)),u&&g.languageServiceRefCount++;else{y=e.createLanguageServiceSourceFile(t,c,p,l,!1,d);a&&a.setDocument(s,r,y),g={sourceFile:y,languageServiceRefCount:1},h()}return e.Debug.assert(0!==g.languageServiceRefCount),g.sourceFile;function h(){if(_)if(n(_)){var t=new e.Map;t.set(_.sourceFile.scriptKind,_),t.set(d,g),m.set(r,t)}else _.set(d,g);else m.set(r,g)}}function _(n,t,a){var o=e.toPath(n,r,s),c=i(t);return g(o,c,a)}function g(t,r,i){var a=e.Debug.checkDefined(o.get(r)),s=a.get(t),c=f(s,i);c.languageServiceRefCount--,e.Debug.assert(c.languageServiceRefCount>=0),0===c.languageServiceRefCount&&(n(s)?a.delete(t):(s.delete(i),1===s.size&&a.set(t,e.firstDefinedIterator(s.values(),e.identity))))}function y(n,t){return e.arrayFrom(o.entries(),function(e){var r=e[0],i=e[1],a=i.get(n),o=a&&f(a,t);return[r,o&&o.languageServiceRefCount]})}return{acquireDocument:l,acquireDocumentWithKey:u,updateDocument:d,updateDocumentWithKey:p,releaseDocument:_,releaseDocumentWithKey:g,getLanguageServiceRefCounts:y,reportStats:c,getKeyForCompilationSettings:i}}function i(n){return e.sourceFileAffectingCompilerOptions.map(function(t){return e.getCompilerOptionValue(n,t)}).join("|")}e.createDocumentRegistry=t,e.createDocumentRegistryInternal=r}(d||(d={})),function(e){(function(n){function t(e,n,t,a){var s=c(e,t,a);return function(c,l,u){var d=r(e,n,s,l,t,a),p=d.directImports,f=d.indirectUsers;return o({indirectUsers:f},i(p,c,l.exportKind,t,u))}}function r(n,t,r,i,o,s){var c=i.exportingModuleSymbol,l=i.exportKind,u=e.nodeSeenTracker(),d=e.nodeSeenTracker(),p=[],f=!!c.globalExports,m=f?void 0:[];return g(c),{directImports:p,indirectUsers:_()};function _(){if(f)return n;if(c.declarations)for(var r=0,i=c.declarations;r<i.length;r++){var a=i[r];e.isExternalModuleAugmentation(a)&&t.has(a.getSourceFile().fileName)&&S(a)}return m.map(e.getSourceFileOfNode)}function g(n){var t=x(n);if(t)for(var r=0,i=t;r<i.length;r++){var a=i[r];if(u(a))switch(s&&s.throwIfCancellationRequested(),a.kind){case 206:if(e.isImportCall(a)){b(a);break}if(!f){var c=a.parent;if(2===l&&252===c.kind){var d=c.name;if(79===d.kind){p.push(d);break}}}break;case 79:break;case 263:T(a,a.name,e.hasSyntacticModifier(a,1),!1);break;case 264:p.push(a);var m=a.importClause&&a.importClause.namedBindings;m&&266===m.kind?T(a,m.name,!1,!0):!f&&e.isDefaultImport(a)&&S(h(a));break;case 270:a.exportClause?272===a.exportClause.kind?S(h(a),!0):p.push(a):g(y(a,o));break;case 198:a.isTypeOf&&!a.qualifier&&E(a)&&S(a.getSourceFile(),!0),p.push(a);break;default:e.Debug.failBadSyntaxKind(a,"Unexpected import kind.")}}}function b(n){var t=e.findAncestor(n,v)||n.getSourceFile();S(t,!!E(n,!0))}function E(n,t){return void 0===t&&(t=!1),e.findAncestor(n,function(n){return t&&v(n)?"quit":e.some(n.modifiers,function(e){return 93===e.kind})})}function T(n,t,r,i){if(2===l)i||p.push(n);else if(!f){var s=h(n);e.Debug.assert(300===s.kind||259===s.kind),r||a(s,t,o)?S(s,!0):S(s)}}function S(n,t){void 0===t&&(t=!1),e.Debug.assert(!f);var r=d(n);if(r&&(m.push(n),t)){var i=o.getMergedSymbol(n.symbol);if(i){e.Debug.assert(!!(1536&i.flags));var a=x(i);if(a)for(var s=0,c=a;s<c.length;s++){var l=c[s];e.isImportTypeNode(l)||S(h(l),!0)}}}}function x(n){return r.get(e.getSymbolId(n).toString())}}function i(n,t,r,i,a){var o=[],s=[];function c(e,n){o.push([e,n])}if(n)for(var l=0,u=n;l<u.length;l++){var d=u[l];p(d)}return{importSearches:o,singleReferences:s};function p(n){if(263!==n.kind)if(79!==n.kind)if(198!==n.kind){if(10===n.moduleSpecifier.kind)if(270!==n.kind){var o=n.importClause||{name:void 0,namedBindings:void 0},l=o.name,u=o.namedBindings;if(u)switch(u.kind){case 266:f(u.name);break;case 267:0!==r&&1!==r||m(u);break;default:e.Debug.assertNever(u)}if(l&&(1===r||2===r)&&(!a||l.escapedText===e.symbolEscapedNameNoDefault(t))){var d=i.getSymbolAtLocation(l);c(l,d)}}else n.exportClause&&e.isNamedExports(n.exportClause)&&m(n.exportClause)}else if(n.qualifier){var p=e.getFirstIdentifier(n.qualifier);p.escapedText===e.symbolName(t)&&s.push(p)}else 2===r&&s.push(n.argument.literal);else f(n);else b(n)&&f(n.name)}function f(e){2!==r||a&&!_(e.escapedText)||c(e,i.getSymbolAtLocation(e))}function m(e){if(e)for(var n=0,r=e.elements;n<r.length;n++){var o=r[n],l=o.name,u=o.propertyName;if(_((u||l).escapedText))if(u)s.push(u),a&&l.escapedText!==t.escapedName||c(l,i.getSymbolAtLocation(l));else{var d=273===o.kind&&o.propertyName?i.getExportSpecifierLocalTargetSymbol(o):i.getSymbolAtLocation(l);c(l,d)}}}function _(e){return e===t.escapedName||0!==r&&"default"===e}}function a(n,t,r){var i=r.getSymbolAtLocation(t);return!!l(n,function(n){if(e.isExportDeclaration(n)){var t=n.exportClause,a=n.moduleSpecifier;return!a&&t&&e.isNamedExports(t)&&t.elements.some(function(e){return r.getExportSpecifierLocalTargetSymbol(e)===i})}})}function s(e,n,t){for(var r=[],i=e.getTypeChecker(),a=0,o=n;a<o.length;a++){var s=o[a],c=t.valueDeclaration;if(300===(null===c||void 0===c?void 0:c.kind)){for(var l=0,d=s.referencedFiles;l<d.length;l++){var p=d[l];e.getSourceFileFromReference(s,p)===c&&r.push({kind:"reference",referencingFile:s,ref:p})}for(var f=0,m=s.typeReferenceDirectives;f<m.length;f++){p=m[f];var _=e.getResolvedTypeReferenceDirectives().get(p.fileName);void 0!==_&&_.resolvedFileName===c.fileName&&r.push({kind:"reference",referencingFile:s,ref:p})}}u(s,function(e,n){var a=i.getSymbolAtLocation(n);a===t&&r.push({kind:"import",literal:n})})}return r}function c(n,t,r){for(var i=new e.Map,a=0,o=n;a<o.length;a++){var s=o[a];r&&r.throwIfCancellationRequested(),u(s,function(n,r){var a=t.getSymbolAtLocation(r);if(a){var o=e.getSymbolId(a).toString(),s=i.get(o);s||i.set(o,s=[]),s.push(n)}})}return i}function l(n,t){return e.forEach(300===n.kind?n.statements:n.body.statements,function(n){return t(n)||v(n)&&e.forEach(n.body&&n.body.statements,t)})}function u(n,t){if(n.externalModuleIndicator||void 0!==n.imports)for(var r=0,i=n.imports;r<i.length;r++){var a=i[r];t(e.importFromModuleSpecifier(a),a)}else l(n,function(n){switch(n.kind){case 270:case 264:var r=n;r.moduleSpecifier&&e.isStringLiteral(r.moduleSpecifier)&&t(r,r.moduleSpecifier);break;case 263:r=n;b(r)&&t(r,r.moduleReference.expression);break}})}function d(n,t,r,i){return i?a():a()||o();function a(){var a,o=n.parent,l=o.parent;if(t.exportSymbol)return 204===o.kind?(null===(a=t.declarations)||void 0===a?void 0:a.some(function(e){return e===o}))&&e.isBinaryExpression(l)?m(l,!1):void 0:s(t.exportSymbol,c(o));var u=f(o,n);if(u&&e.hasSyntacticModifier(u,1)){if(e.isImportEqualsDeclaration(u)&&u.moduleReference===n){if(i)return;var d=r.getSymbolAtLocation(u.name);return{kind:0,symbol:d}}return s(t,c(u))}if(e.isNamespaceExport(o))return s(t,0);if(e.isExportAssignment(o))return p(o);if(e.isExportAssignment(l))return p(l);if(e.isBinaryExpression(o))return m(o,!0);if(e.isBinaryExpression(l))return m(l,!0);if(e.isJSDocTypedefTag(o))return s(t,0);function p(e){if(e.symbol.parent){var n=e.isExportEquals?2:1;return{kind:1,symbol:t,exportInfo:{exportingModuleSymbol:e.symbol.parent,exportKind:n}}}}function m(n,i){var a;switch(e.getAssignmentDeclarationKind(n)){case 1:a=0;break;case 2:a=2;break;default:return}var o=i?r.getSymbolAtLocation(e.getNameOfAccessExpression(e.cast(n.left,e.isAccessExpression))):t;return o&&s(o,a)}}function o(){var i=m(n);if(i){var a=r.getImmediateAliasedSymbol(t);if(a){a=g(a,r),"export="===a.escapedName&&(a=p(a,r));var o=e.symbolEscapedNameNoDefault(a);return void 0===o||"default"===o||o===t.escapedName?{kind:0,symbol:a}:void 0}}}function s(e,n){var t=_(e,n,r);return t&&{kind:1,symbol:e,exportInfo:t}}function c(n){return e.hasSyntacticModifier(n,512)?1:0}}function p(n,t){if(2097152&n.flags)return e.Debug.checkDefined(t.getImmediateAliasedSymbol(n));var r=e.Debug.checkDefined(n.valueDeclaration);return e.isExportAssignment(r)?e.Debug.checkDefined(r.expression.symbol):e.isBinaryExpression(r)?e.Debug.checkDefined(r.right.symbol):e.isSourceFile(r)?e.Debug.checkDefined(r.symbol):e.Debug.fail()}function f(n,t){var r=e.isVariableDeclaration(n)?n:e.isBindingElement(n)?e.walkUpBindingElementsAndPatterns(n):void 0;return r?n.name!==t?void 0:e.isCatchClause(r.parent)?void 0:e.isVariableStatement(r.parent.parent)?r.parent.parent:void 0:n}function m(n){var t=n.parent;switch(t.kind){case 263:return t.name===n&&b(t);case 268:return!t.propertyName;case 265:case 266:return e.Debug.assert(t.name===n),!0;case 201:return e.isInJSFile(n)&&e.isRequireVariableDeclaration(t);default:return!1}}function _(n,t,r){var i=n.parent;if(i){var a=r.getMergedSymbol(i);return e.isExternalModuleSymbol(a)?{exportingModuleSymbol:a,exportKind:t}:void 0}}function g(n,t){if(n.declarations)for(var r=0,i=n.declarations;r<i.length;r++){var a=i[r];if(e.isExportSpecifier(a)&&!a.propertyName&&!a.parent.parent.moduleSpecifier)return t.getExportSpecifierLocalTargetSymbol(a);if(e.isPropertyAccessExpression(a)&&e.isModuleExportsAccessExpression(a.expression)&&!e.isPrivateIdentifier(a.name))return t.getSymbolAtLocation(a);if(e.isShorthandPropertyAssignment(a)&&e.isBinaryExpression(a.parent.parent)&&2===e.getAssignmentDeclarationKind(a.parent.parent))return t.getExportSpecifierLocalTargetSymbol(a.name)}return n}function y(e,n){return n.getMergedSymbol(h(e).symbol)}function h(n){if(206===n.kind)return n.getSourceFile();var t=n.parent;return 300===t.kind?t:(e.Debug.assert(260===t.kind),e.cast(t.parent,v))}function v(e){return 259===e.kind&&10===e.name.kind}function b(e){return 275===e.moduleReference.kind&&10===e.moduleReference.expression.kind}n.createImportTracker=t,function(e){e[e["Named"]=0]="Named",e[e["Default"]=1]="Default",e[e["ExportEquals"]=2]="ExportEquals"}(n.ExportKind||(n.ExportKind={})),function(e){e[e["Import"]=0]="Import",e[e["Export"]=1]="Export"}(n.ImportExport||(n.ImportExport={})),n.findModuleReferences=s,n.getImportOrExportSymbol=d,n.getExportInfo=_})(e.FindAllReferences||(e.FindAllReferences={}))}(d||(d={})),function(e){(function(n){var t;function r(e,n){return void 0===n&&(n=1),{kind:n,node:e.name||e,context:s(e)}}function i(e){return e&&void 0===e.kind}function s(n){if(e.isDeclaration(n))return c(n);if(n.parent){if(!e.isDeclaration(n.parent)&&!e.isExportAssignment(n.parent)){if(e.isInJSFile(n)){var t=e.isBinaryExpression(n.parent)?n.parent:e.isAccessExpression(n.parent)&&e.isBinaryExpression(n.parent.parent)&&n.parent.parent.left===n.parent?n.parent.parent:void 0;if(t&&0!==e.getAssignmentDeclarationKind(t))return c(t)}if(e.isJsxOpeningElement(n.parent)||e.isJsxClosingElement(n.parent))return n.parent.parent;if(e.isJsxSelfClosingElement(n.parent)||e.isLabeledStatement(n.parent)||e.isBreakOrContinueStatement(n.parent))return n.parent;if(e.isStringLiteralLike(n)){var r=e.tryGetImportFromModuleSpecifier(n);if(r){var i=e.findAncestor(r,function(n){return e.isDeclaration(n)||e.isStatement(n)||e.isJSDocTag(n)});return e.isDeclaration(i)?c(i):i}}var a=e.findAncestor(n,e.isComputedPropertyName);return a?c(a.parent):void 0}return n.parent.name===n||e.isConstructorDeclaration(n.parent)||e.isExportAssignment(n.parent)||(e.isImportOrExportSpecifier(n.parent)||e.isBindingElement(n.parent))&&n.parent.propertyName===n||88===n.kind&&e.hasSyntacticModifier(n.parent,513)?c(n.parent):void 0}}function c(n){if(n)switch(n.kind){case 252:return e.isVariableDeclarationList(n.parent)&&1===n.parent.declarations.length?e.isVariableStatement(n.parent.parent)?n.parent.parent:e.isForInOrOfStatement(n.parent.parent)?c(n.parent.parent):n.parent:n;case 201:return c(n.parent.parent);case 268:return n.parent.parent.parent;case 273:case 266:return n.parent.parent;case 265:case 272:return n.parent;case 219:return e.isExpressionStatement(n.parent)?n.parent:n;case 242:case 241:return{start:n.initializer,end:n.expression};case 291:case 292:return e.isArrayLiteralOrObjectLiteralDestructuringPattern(n.parent)?c(e.findAncestor(n.parent,function(n){return e.isBinaryExpression(n)||e.isForInOrOfStatement(n)})):n;default:return n}}function l(e,n,t){if(t){var r=i(t)?D(t.start,n,t.end):D(t,n);return r.start!==e.start||r.length!==e.length?{contextSpan:r}:void 0}}function u(n,r,i,a,o){var s=e.getTouchingPropertyName(a,o),c=t.getReferencedSymbolsForNode(o,s,n,i,r,{use:1}),l=n.getTypeChecker();return c&&c.length?e.mapDefined(c,function(e){var n=e.definition,t=e.references;return n&&{definition:l.runWithCancellationToken(r,function(e){return g(n,e,s)}),references:t.map(b)}}):void 0}function d(n,t,r,i,o){var s,c=e.getTouchingPropertyName(i,o),l=p(n,t,r,c,o);if(204===c.parent.kind||201===c.parent.kind||205===c.parent.kind||106===c.kind)s=l&&a([],l,!0);else{var u=l&&a([],l,!0),d=new e.Map;while(u&&u.length){var f=u.shift();if(e.addToSeen(d,e.getNodeId(f.node))){s=e.append(s,f);var m=p(n,t,r,f.node,f.node.pos);m&&u.push.apply(u,m)}}}var _=n.getTypeChecker();return e.map(s,function(e){return S(e,_)})}function p(n,i,a,o,s){if(300!==o.kind){var c=n.getTypeChecker();if(292===o.parent.kind){var l=[];return t.getReferenceEntriesForShorthandPropertyAssignment(o,c,function(e){return l.push(r(e))}),l}if(106===o.kind||e.isSuperProperty(o.parent)){var u=c.getSymbolAtLocation(o);return u.valueDeclaration&&[r(u.valueDeclaration)]}return m(s,o,n,a,i,{implementations:!0,use:1})}}function f(n,r,i,a,o,s,c){return e.map(_(t.getReferencedSymbolsForNode(o,a,n,i,r,s)),function(e){return c(e,a,n.getTypeChecker())})}function m(n,r,i,a,o,s,c){return void 0===s&&(s={}),void 0===c&&(c=new e.Set(a.map(function(e){return e.fileName}))),_(t.getReferencedSymbolsForNode(n,r,i,a,o,s,c))}function _(n){return n&&e.flatMap(n,function(e){return e.references})}function g(n,t,r){var i=function(){switch(n.type){case 0:var i=n.symbol,a=h(i,t,r),s=a.displayParts,l=a.kind,u=s.map(function(e){return e.text}).join(""),d=i.declarations&&e.firstOrUndefined(i.declarations),p=d?e.getNameOfDeclaration(d)||d:r;return o(o({},y(p)),{name:u,kind:l,displayParts:s,context:c(d)});case 1:p=n.node;return o(o({},y(p)),{name:p.text,kind:"label",displayParts:[e.displayPart(p.text,e.SymbolDisplayPartKind.text)]});case 2:p=n.node;var f=e.tokenToString(p.kind);return o(o({},y(p)),{name:f,kind:"keyword",displayParts:[{text:f,kind:"keyword"}]});case 3:p=n.node,i=t.getSymbolAtLocation(p);var m=i&&e.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(t,i,p.getSourceFile(),e.getContainerNode(p),p).displayParts||[e.textPart("this")];return o(o({},y(p)),{name:"this",kind:"var",displayParts:m});case 4:p=n.node;return o(o({},y(p)),{name:p.text,kind:"var",displayParts:[e.displayPart(e.getTextOfNode(p),e.SymbolDisplayPartKind.stringLiteral)]});case 5:return{textSpan:e.createTextSpanFromRange(n.reference),sourceFile:n.file,name:n.reference.fileName,kind:"string",displayParts:[e.displayPart('"'+n.reference.fileName+'"',e.SymbolDisplayPartKind.stringLiteral)]};default:return e.Debug.assertNever(n)}}(),a=i.sourceFile,s=i.textSpan,u=i.name,d=i.kind,p=i.displayParts,f=i.context;return o({containerKind:"",containerName:"",fileName:a.fileName,kind:d,name:u,textSpan:s,displayParts:p},l(s,a,f))}function y(n){var t=n.getSourceFile();return{sourceFile:t,textSpan:D(e.isComputedPropertyName(n)?n.expression:n,t)}}function h(n,r,i){var a=t.getIntersectingMeaningFromDeclarations(i,n),o=n.declarations&&e.firstOrUndefined(n.declarations)||i,s=e.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(r,n,o.getSourceFile(),o,o,a),c=s.displayParts,l=s.symbolKind;return{displayParts:c,kind:l}}function v(e,n,t,r){return o(o({},E(e)),r&&T(e,n,t))}function b(e){var n=E(e);if(0===e.kind)return o(o({},n),{isWriteAccess:!1,isDefinition:!1});var t=e.kind,r=e.node;return o(o({},n),{isWriteAccess:C(r),isDefinition:I(r),isInString:2===t||void 0})}function E(e){if(0===e.kind)return{textSpan:e.textSpan,fileName:e.fileName};var n=e.node.getSourceFile(),t=D(e.node,n);return o({textSpan:t,fileName:n.fileName},l(t,n,e.context))}function T(n,t,r){if(0!==n.kind&&e.isIdentifier(t)){var i=n.node,a=n.kind,o=i.parent,s=t.text,c=e.isShorthandPropertyAssignment(o);if(c||e.isObjectBindingElementWithoutPropertyName(o)&&o.name===i&&void 0===o.dotDotDotToken){var l={prefixText:s+": "},u={suffixText:": "+s};if(3===a)return l;if(4===a)return u;if(c){var d=o.parent;return e.isObjectLiteralExpression(d)&&e.isBinaryExpression(d.parent)&&e.isModuleExportsAccessExpression(d.parent.left)?l:u}return l}if(e.isImportSpecifier(o)&&!o.propertyName){var p=e.isExportSpecifier(t.parent)?r.getExportSpecifierLocalTargetSymbol(t.parent):r.getSymbolAtLocation(t);return e.contains(p.declarations,o)?{prefixText:s+" as "}:e.emptyOptions}if(e.isExportSpecifier(o)&&!o.propertyName)return t===n.node||r.getSymbolAtLocation(t)===r.getSymbolAtLocation(n.node)?{prefixText:s+" as "}:{suffixText:" as "+s}}return e.emptyOptions}function S(e,n){var t=E(e);if(0!==e.kind){var r=e.node;return o(o({},t),x(r,n))}return o(o({},t),{kind:"",displayParts:[]})}function x(n,t){var r=t.getSymbolAtLocation(e.isDeclaration(n)&&n.name?n.name:n);return r?h(r,t,n):203===n.kind?{kind:"interface",displayParts:[e.punctuationPart(20),e.textPart("object literal"),e.punctuationPart(21)]}:224===n.kind?{kind:"local class",displayParts:[e.punctuationPart(20),e.textPart("anonymous local class"),e.punctuationPart(21)]}:{kind:e.getNodeKind(n),displayParts:[]}}function L(e){var n=E(e);if(0===e.kind)return{fileName:n.fileName,span:{textSpan:n.textSpan,kind:"reference"}};var t=C(e.node),r=o({textSpan:n.textSpan,kind:t?"writtenReference":"reference",isInString:2===e.kind||void 0},n.contextSpan&&{contextSpan:n.contextSpan});return{fileName:n.fileName,span:r}}function D(n,t,r){var i=n.getStart(t),a=(r||n).getEnd();return e.isStringLiteralLike(n)&&(e.Debug.assert(void 0===r),i+=1,a-=1),e.createTextSpanFromBounds(i,a)}function A(e){return 0===e.kind?e.textSpan:D(e.node,e.node.getSourceFile())}function C(n){var t=e.getDeclarationFromName(n);return!!t&&N(t)||88===n.kind||e.isWriteAccess(n)}function I(n){return 88===n.kind||!!e.getDeclarationFromName(n)||e.isLiteralComputedPropertyDeclarationName(n)||133===n.kind&&e.isConstructorDeclaration(n.parent)}function N(n){if(8388608&n.flags)return!0;switch(n.kind){case 219:case 201:case 255:case 224:case 88:case 258:case 294:case 273:case 265:case 263:case 268:case 256:case 333:case 340:case 283:case 259:case 262:case 266:case 272:case 162:case 292:case 257:case 161:return!0;case 291:return!e.isArrayLiteralOrObjectLiteralDestructuringPattern(n.parent);case 254:case 211:case 169:case 167:case 170:case 171:return!!n.body;case 252:case 165:return!!n.initializer||e.isCatchClause(n.parent);case 166:case 164:case 342:case 335:return!1;default:return e.Debug.failBadSyntaxKind(n)}}(function(e){e[e["Symbol"]=0]="Symbol",e[e["Label"]=1]="Label",e[e["Keyword"]=2]="Keyword",e[e["This"]=3]="This",e[e["String"]=4]="String",e[e["TripleSlashReference"]=5]="TripleSlashReference"})(n.DefinitionKind||(n.DefinitionKind={})),function(e){e[e["Span"]=0]="Span",e[e["Node"]=1]="Node",e[e["StringLiteral"]=2]="StringLiteral",e[e["SearchedLocalFoundProperty"]=3]="SearchedLocalFoundProperty",e[e["SearchedPropertyFoundLocal"]=4]="SearchedPropertyFoundLocal"}(n.EntryKind||(n.EntryKind={})),n.nodeEntry=r,n.isContextWithStartAndEndNode=i,n.getContextNode=c,n.toContextSpan=l,function(e){e[e["Other"]=0]="Other",e[e["References"]=1]="References",e[e["Rename"]=2]="Rename"}(n.FindReferencesUse||(n.FindReferencesUse={})),n.findReferencedSymbols=u,n.getImplementationsAtPosition=d,n.findReferenceOrRenameEntries=f,n.getReferenceEntriesForNode=m,n.toRenameLocation=v,n.toReferenceEntry=b,n.toHighlightSpan=L,n.getTextSpanOfEntry=A,function(t){function i(n,t,r,i,a,u,p){var _,g;if(void 0===u&&(u={}),void 0===p&&(p=new e.Set(i.map(function(e){return e.fileName}))),1===u.use?t=e.getAdjustedReferenceLocation(t):2===u.use&&(t=e.getAdjustedRenameLocation(t)),e.isSourceFile(t)){var y=e.GoToDefinition.getReferenceAtPosition(t,n,r);if(!(null===y||void 0===y?void 0:y.file))return;var h=r.getTypeChecker().getMergedSymbol(y.file.symbol);if(h)return d(r,h,!1,i,p);var v=r.getFileIncludeReasons();if(!v)return;return[{definition:{type:5,reference:y.reference,file:t},references:o(y.file,v,r)||e.emptyArray}]}if(!u.implementations){var b=f(t,i,a);if(b)return b}var E=r.getTypeChecker(),T=E.getSymbolAtLocation(e.isConstructorDeclaration(t)&&t.parent.name||t);if(T){if("export="===T.escapedName)return d(r,T.parent,!1,i,p);var S=c(T,r,i,a,u,p);if(S&&!(33554432&T.flags))return S;var x=s(t,T,E),L=x&&c(x,r,i,a,u,p),D=m(T,t,i,p,E,a,u);return l(r,S,D,L)}if(!u.implementations&&e.isStringLiteralLike(t)){if(e.isModuleSpecifierLike(t)){v=r.getFileIncludeReasons();var A=null===(g=null===(_=t.getSourceFile().resolvedModules)||void 0===_?void 0:_.get(t.text))||void 0===g?void 0:g.resolvedFileName,C=A?r.getSourceFile(A):void 0;if(C)return[{definition:{type:4,node:t},references:o(C,v,r)||e.emptyArray}]}return se(t,i,E,a)}}function a(n,t,r,i){var a,s;void 0===i&&(i=new e.Set(r.map(function(e){return e.fileName})));var c=null===(a=t.getSourceFile(n))||void 0===a?void 0:a.symbol;if(c)return(null===(s=d(t,c,!1,r,i)[0])||void 0===s?void 0:s.references)||e.emptyArray;var l=t.getFileIncludeReasons(),u=t.getSourceFile(n);return u&&l&&o(u,l,t)||e.emptyArray}function o(n,t,r){for(var i,a=t.get(n.path)||e.emptyArray,o=0,s=a;o<s.length;o++){var c=s[o];if(e.isReferencedFile(c)){var l=r.getSourceFileByPath(c.file),u=e.getReferencedFileLocation(r.getSourceFileByPath,c);e.isReferenceFileLocation(u)&&(i=e.append(i,{kind:0,fileName:l.fileName,textSpan:e.createTextSpanFromRange(u)}))}}return i}function s(n,t,r){if(n.parent&&e.isNamespaceExportDeclaration(n.parent)){var i=r.getAliasedSymbol(t),a=r.getMergedSymbol(i);if(i!==a)return a}}function c(n,t,r,i,a,o){var s=1536&n.flags&&n.declarations&&e.find(n.declarations,e.isSourceFile);if(s){var c=n.exports.get("export="),u=d(t,n,!!c,r,o);if(!c||!o.has(s.fileName))return u;var p=t.getTypeChecker();return n=e.skipAlias(c,p),l(t,u,m(n,void 0,r,o,p,i,a))}}function l(n){for(var t,r=[],i=1;i<arguments.length;i++)r[i-1]=arguments[i];for(var a=0,o=r;a<o.length;a++){var s=o[a];if(s&&s.length)if(t)for(var c=function(r){if(!r.definition||0!==r.definition.type)return t.push(r),"continue";var i=r.definition.symbol,a=e.findIndex(t,function(e){return!!e.definition&&0===e.definition.type&&e.definition.symbol===i});if(-1===a)return t.push(r),"continue";var o=t[a];t[a]={definition:o.definition,references:o.references.concat(r.references).sort(function(t,r){var i=u(n,t),a=u(n,r);if(i!==a)return e.compareValues(i,a);var o=A(t),s=A(r);return o.start!==s.start?e.compareValues(o.start,s.start):e.compareValues(o.length,s.length)})}},l=0,d=s;l<d.length;l++){var p=d[l];c(p)}else t=s}return t}function u(e,n){var t=0===n.kind?e.getSourceFile(n.fileName):n.node.getSourceFile();return e.getSourceFiles().indexOf(t)}function d(t,i,a,o,s){e.Debug.assert(!!i.valueDeclaration);var c=e.mapDefined(n.findModuleReferences(t,o,i),function(n){if("import"===n.kind){var t=n.literal.parent;if(e.isLiteralTypeNode(t)){var i=e.cast(t.parent,e.isImportTypeNode);if(a&&!i.qualifier)return}return r(n.literal)}return{kind:0,fileName:n.referencingFile.fileName,textSpan:e.createTextSpanFromRange(n.ref)}});if(i.declarations)for(var l=0,u=i.declarations;l<u.length;l++){var d=u[l];switch(d.kind){case 300:break;case 259:s.has(d.getSourceFile().fileName)&&c.push(r(d.name));break;default:e.Debug.assert(!!(33554432&i.flags),"Expected a module symbol to be declared by a SourceFile or ModuleDeclaration.")}}var p=i.exports.get("export=");if(null===p||void 0===p?void 0:p.declarations)for(var f=0,m=p.declarations;f<m.length;f++){d=m[f];var _=d.getSourceFile();if(s.has(_.fileName)){var g=e.isBinaryExpression(d)&&e.isPropertyAccessExpression(d.left)?d.left.expression:e.isExportAssignment(d)?e.Debug.checkDefined(e.findChildOfKind(d,93,_)):e.getNameOfDeclaration(d)||d;c.push(r(g))}}return c.length?[{definition:{type:0,symbol:i},references:c}]:e.emptyArray}function p(n){return 143===n.kind&&e.isTypeOperatorNode(n.parent)&&143===n.parent.operator}function f(n,t,i){if(e.isTypeKeyword(n.kind)){if(114===n.kind&&e.isVoidExpression(n.parent))return;if(143===n.kind&&!p(n))return;return M(t,n.kind,i,143===n.kind?p:void 0)}if(e.isStaticModifier(n)&&e.isClassStaticBlockDeclaration(n.parent))return[{definition:{type:2,node:n},references:[r(n)]}];if(e.isJumpStatementTarget(n)){var a=e.getTargetLabel(n.parent,n.text);return a&&w(a.parent,a)}return e.isLabelOfLabeledStatement(n)?w(n.parent,n):e.isThis(n)?oe(n,t,i):106===n.kind?ie(n):void 0}function m(n,t,r,i,a,o,s){var c=t&&y(n,t,a,!ve(s))||n,l=t?fe(t,c):7,u=[],d=new b(r,i,t?g(t):0,a,o,l,s,u),p=ve(s)&&c.declarations?e.find(c.declarations,e.isExportSpecifier):void 0;if(p)V(p.name,c,p,d.createSearch(t,n,void 0),d,!0,!0);else if(t&&88===t.kind&&"default"===c.escapedName&&c.parent)z(t,c,d),E(t,c,{exportingModuleSymbol:c.parent,exportKind:1},d);else{var f=d.createSearch(t,c,void 0,{allSearchSymbols:t?ce(c,t,a,2===s.use,!!s.providePrefixAndSuffixTextForRename,!!s.implementations):[c]});_(c,d,f)}return u}function _(n,t,r){var i=C(n);if(i)G(i,i.getSourceFile(),r,t,!(e.isSourceFile(i)&&!e.contains(t.sourceFiles,i)));else for(var a=0,o=t.sourceFiles;a<o.length;a++){var s=o[a];t.cancellationToken.throwIfCancellationRequested(),L(s,r,t)}}function g(n){switch(n.kind){case 169:case 133:return 1;case 79:if(e.isClassLike(n.parent))return e.Debug.assert(n.parent.name===n),2;default:return 0}}function y(n,t,r,i){var a=t.parent;return e.isExportSpecifier(a)&&i?j(t,n,a,r):e.firstDefined(n.declarations,function(i){if(!i.parent){if(33554432&n.flags)return;e.Debug.fail("Unexpected symbol at "+e.Debug.formatSyntaxKind(t.kind)+": "+e.Debug.formatSymbol(n))}return e.isTypeLiteralNode(i.parent)&&e.isUnionTypeNode(i.parent.parent)?r.getPropertyOfType(r.getTypeFromTypeNode(i.parent.parent),n.name):void 0})}var h;function v(n){if(33555968&n.flags){var t=n.declarations&&e.find(n.declarations,function(n){return!e.isSourceFile(n)&&!e.isModuleDeclaration(n)});return t&&t.symbol}}t.getReferencedSymbolsForNode=i,t.getReferencesForFileName=a,function(e){e[e["None"]=0]="None",e[e["Constructor"]=1]="Constructor",e[e["Class"]=2]="Class"}(h||(h={}));var b=function(){function t(n,t,r,i,a,o,s,c){this.sourceFiles=n,this.sourceFilesSet=t,this.specialSearchKind=r,this.checker=i,this.cancellationToken=a,this.searchMeaning=o,this.options=s,this.result=c,this.inheritsFromCache=new e.Map,this.markSeenContainingTypeReference=e.nodeSeenTracker(),this.markSeenReExportRHS=e.nodeSeenTracker(),this.symbolIdToReferences=[],this.sourceFileToSeenSymbols=[]}return t.prototype.includesSourceFile=function(e){return this.sourceFilesSet.has(e.fileName)},t.prototype.getImportSearches=function(e,t){return this.importTracker||(this.importTracker=n.createImportTracker(this.sourceFiles,this.sourceFilesSet,this.checker,this.cancellationToken)),this.importTracker(e,t,2===this.options.use)},t.prototype.createSearch=function(n,t,r,i){void 0===i&&(i={});var a=i.text,o=void 0===a?e.stripQuotes(e.symbolName(e.getLocalSymbolForExportDefault(t)||v(t)||t)):a,s=i.allSearchSymbols,c=void 0===s?[t]:s,l=e.escapeLeadingUnderscores(o),u=this.options.implementations&&n?he(n,t,this.checker):void 0;return{symbol:t,comingFrom:r,text:o,escapedText:l,parents:u,allSearchSymbols:c,includes:function(n){return e.contains(c,n)}}},t.prototype.referenceAdder=function(n){var t=e.getSymbolId(n),i=this.symbolIdToReferences[t];return i||(i=this.symbolIdToReferences[t]=[],this.result.push({definition:{type:0,symbol:n},references:i})),function(e,n){return i.push(r(e,n))}},t.prototype.addStringOrCommentReference=function(e,n){this.result.push({definition:void 0,references:[{kind:0,fileName:e,textSpan:n}]})},t.prototype.markSearchedSymbols=function(n,t){for(var r=e.getNodeId(n),i=this.sourceFileToSeenSymbols[r]||(this.sourceFileToSeenSymbols[r]=new e.Set),a=!1,o=0,s=t;o<s.length;o++){var c=s[o];a=e.tryAddToSet(i,e.getSymbolId(c))||a}return a},t}();function E(e,n,t,r){var i=r.getImportSearches(n,t),a=i.importSearches,o=i.singleReferences,s=i.indirectUsers;if(o.length)for(var c=r.referenceAdder(n),l=0,u=o;l<u.length;l++){var d=u[l];S(d,r)&&c(d)}for(var p=0,f=a;p<f.length;p++){var m=f[p],_=m[0],g=m[1];F(_.getSourceFile(),r.createSearch(_,g,1),r)}if(s.length){var y=void 0;switch(t.exportKind){case 0:y=r.createSearch(e,n,1);break;case 1:y=2===r.options.use?void 0:r.createSearch(e,n,1,{text:"default"});break;case 2:break}if(y)for(var h=0,v=s;h<v.length;h++){var b=v[h];L(b,y,r)}}}function T(t,r,i,a,o,s,c,l){for(var u=n.createImportTracker(t,new e.Set(t.map(function(e){return e.fileName})),r,i),d=u(a,{exportKind:c?1:0,exportingModuleSymbol:o},!1),p=d.importSearches,f=d.indirectUsers,m=0,_=p;m<_.length;m++){var g=_[m][0];l(g)}for(var y=0,h=f;y<h.length;y++)for(var v=h[y],b=0,E=R(v,c?"default":s);b<E.length;b++){var T=E[b],S=r.getSymbolAtLocation(T),x=e.some(null===S||void 0===S?void 0:S.declarations,function(n){return!!e.tryCast(n,e.isExportAssignment)});!e.isIdentifier(T)||e.isImportOrExportSpecifier(T.parent)||S!==a&&!x||l(T)}}function S(n,t){return!!B(n,t)&&(2!==t.options.use||!!e.isIdentifier(n)&&!(e.isImportOrExportSpecifier(n.parent)&&"default"===n.escapedText))}function x(e,n){if(e.declarations)for(var t=0,r=e.declarations;t<r.length;t++){var i=r[t],a=i.getSourceFile();F(a,n.createSearch(i,e,0),n,n.includesSourceFile(a))}}function L(n,t,r){void 0!==e.getNameTable(n).get(t.escapedText)&&F(n,t,r)}function D(n,t){return e.isArrayLiteralOrObjectLiteralDestructuringPattern(n.parent.parent)?t.getPropertySymbolOfDestructuringAssignment(n):void 0}function C(n){var t=n.declarations,r=n.flags,i=n.parent,a=n.valueDeclaration;if(a&&(211===a.kind||224===a.kind))return a;if(t){if(8196&r){var o=e.find(t,function(n){return e.hasEffectiveModifier(n,8)||e.isPrivateIdentifierClassElementDeclaration(n)});return o?e.getAncestor(o,255):void 0}if(!t.some(e.isObjectBindingElementWithoutPropertyName)){var s=i&&!(262144&n.flags);if(!s||e.isExternalModuleSymbol(i)&&!i.globalExports){for(var c,l=0,u=t;l<u.length;l++){var d=u[l],p=e.getContainerNode(d);if(c&&c!==p)return;if(!p||300===p.kind&&!e.isExternalOrCommonJsModule(p))return;if(c=p,e.isFunctionExpression(c)){var f=void 0;while(f=e.getNextJSDocCommentLocation(c))c=f}}return s?c.getSourceFile():c}}}}function I(e,n,t,r){return void 0===r&&(r=t),N(e,n,t,function(){return!0},r)||!1}function N(n,t,r,i,a){void 0===a&&(a=r);var o=e.isParameterPropertyDeclaration(n.parent,n.parent.parent)?e.first(t.getSymbolsOfParameterPropertyDeclaration(n.parent,n.text)):t.getSymbolAtLocation(n);if(o)for(var s=0,c=R(r,o.name,a);s<c.length;s++){var l=c[s];if(e.isIdentifier(l)&&l!==n&&l.escapedText===n.escapedText){var u=t.getSymbolAtLocation(l);if(u===o||t.getShorthandAssignmentValueSymbol(l.parent)===o||e.isExportSpecifier(l.parent)&&j(l,u,l.parent,t)===o){var d=i(l);if(d)return d}}}}function k(n,t,r,i){if(!n.name||!e.isIdentifier(n.name))return!1;for(var a=e.Debug.checkDefined(r.getSymbolAtLocation(n.name)),o=0,s=t;o<s.length;o++)for(var c=s[o],l=0,u=R(c,a.name);l<u.length;l++){var d=u[l];if(e.isIdentifier(d)&&d!==n.name&&d.escapedText===n.name.escapedText){var p=e.climbPastPropertyAccess(d),f=e.isCallExpression(p.parent)&&p.parent.expression===p?p.parent:void 0,m=r.getSymbolAtLocation(d);if(m&&r.getRootSymbols(m).some(function(e){return e===a})&&i(d,f))return!0}}return!1}function R(n,t,r){return void 0===r&&(r=n),O(n,t,r).map(function(t){return e.getTouchingPropertyName(n,t)})}function O(n,t,r){void 0===r&&(r=n);var i=[];if(!t||!t.length)return i;var a=n.text,o=a.length,s=t.length,c=a.indexOf(t,r.pos);while(c>=0){if(c>r.end)break;var l=c+s;0!==c&&e.isIdentifierPart(a.charCodeAt(c-1),99)||l!==o&&e.isIdentifierPart(a.charCodeAt(l),99)||i.push(c),c=a.indexOf(t,c+s+1)}return i}function w(n,t){var i=n.getSourceFile(),a=t.text,o=e.mapDefined(R(i,a,n),function(n){return n===t||e.isJumpStatementTarget(n)&&e.getTargetLabel(n,a)===t?r(n):void 0});return[{definition:{type:1,node:t},references:o}]}function P(n,t){switch(n.kind){case 80:if(e.isJSDocMemberName(n.parent))return!0;case 79:return n.text.length===t.length;case 14:case 10:var r=n;return(e.isLiteralNameOfPropertyDeclarationOrIndexAccess(r)||e.isNameOfModuleDeclaration(n)||e.isExpressionOfExternalModuleImportEqualsDeclaration(n)||e.isCallExpression(n.parent)&&e.isBindableObjectDefinePropertyCall(n.parent)&&n.parent.arguments[1]===n)&&r.text.length===t.length;case 8:return e.isLiteralNameOfPropertyDeclarationOrIndexAccess(n)&&n.text.length===t.length;case 88:return"default".length===t.length;default:return!1}}function M(n,t,i,a){var o=e.flatMap(n,function(n){return i.throwIfCancellationRequested(),e.mapDefined(R(n,e.tokenToString(t),n),function(e){if(e.kind===t&&(!a||a(e)))return r(e)})});return o.length?[{definition:{type:2,node:o[0].node},references:o}]:void 0}function F(e,n,t,r){return void 0===r&&(r=!0),t.cancellationToken.throwIfCancellationRequested(),G(e,e,n,t,r)}function G(e,n,t,r,i){if(r.markSearchedSymbols(n,t.allSearchSymbols))for(var a=0,o=O(n,t.text,e);a<o.length;a++){var s=o[a];U(n,s,t,r,i)}}function B(n,t){return!!(e.getMeaningFromLocation(n)&t.searchMeaning)}function U(n,t,r,i,a){var o=e.getTouchingPropertyName(n,t);if(P(o,r.text)){if(B(o,i)){var s=i.checker.getSymbolAtLocation(o);if(s){var c=o.parent;if(!e.isImportSpecifier(c)||c.propertyName!==o){if(e.isExportSpecifier(c))return e.Debug.assert(79===o.kind),void V(o,s,c,r,i,a);var l=pe(r,s,o,i);if(l){switch(i.specialSearchKind){case 0:a&&z(o,l,i);break;case 1:J(o,n,r,i);break;case 2:q(o,r,i);break;default:e.Debug.assertNever(i.specialSearchKind)}s=e.isInJSFile(o)&&201===o.parent.kind&&e.isRequireVariableDeclaration(o.parent)?o.parent.symbol:s,H(o,s,r,i)}else W(s,r,i)}}}}else!i.options.implementations&&(i.options.findInStrings&&e.isInString(n,t)||i.options.findInComments&&e.isInNonReferenceComment(n,t))&&i.addStringOrCommentReference(n.fileName,e.createTextSpan(t,r.text.length))}function V(t,r,i,a,o,s,c){e.Debug.assert(!c||!!o.options.providePrefixAndSuffixTextForRename,"If alwaysGetReferences is true, then prefix/suffix text must be enabled");var l=i.parent,u=i.propertyName,d=i.name,p=l.parent,f=j(t,r,i,o.checker);if(c||a.includes(f)){if(u?t===u?(p.moduleSpecifier||v(),s&&2!==o.options.use&&o.markSeenReExportRHS(d)&&z(d,e.Debug.checkDefined(i.symbol),o)):o.markSeenReExportRHS(t)&&v():2===o.options.use&&"default"===d.escapedText||v(),!ve(o.options)||c){var m=88===t.originalKeywordKind||88===i.name.originalKeywordKind,_=m?1:0,g=e.Debug.checkDefined(i.symbol),y=n.getExportInfo(g,_,o.checker);y&&E(t,g,y,o)}if(1!==a.comingFrom&&p.moduleSpecifier&&!u&&!ve(o.options)){var h=o.checker.getExportSpecifierLocalTargetSymbol(i);h&&x(h,o)}}function v(){s&&z(t,f,o)}}function j(e,n,t,r){return K(e,t)&&r.getExportSpecifierLocalTargetSymbol(t)||n}function K(n,t){var r=t.parent,i=t.propertyName,a=t.name;return e.Debug.assert(i===n||a===n),i?i===n:!r.parent.moduleSpecifier}function H(e,t,r,i){var a=n.getImportOrExportSymbol(e,t,i.checker,1===r.comingFrom);if(a){var o=a.symbol;0===a.kind?ve(i.options)||x(o,i):E(e,o,a.exportInfo,i)}}function W(n,t,r){var i=n.flags,a=n.valueDeclaration,o=r.checker.getShorthandAssignmentValueSymbol(a),s=a&&e.getNameOfDeclaration(a);33554432&i||!s||!t.includes(o)||z(s,o,r)}function z(e,n,t){var r="kind"in n?n:{kind:void 0,symbol:n},i=r.kind,a=r.symbol,o=t.referenceAdder(a);t.options.implementations?ee(e,o,t):o(e,i)}function J(n,t,r,i){e.isNewExpressionTarget(n)&&z(n,r.symbol,i);var a=function(){return i.referenceAdder(r.symbol)};if(e.isClassLike(n.parent))e.Debug.assert(88===n.kind||n.parent.name===n),X(r.symbol,t,a());else{var o=ye(n);o&&(Q(o,a()),$(o,i))}}function q(n,t,r){z(n,t.symbol,r);var i=n.parent;if(2!==r.options.use&&e.isClassLike(i)){e.Debug.assert(i.name===n);for(var a=r.referenceAdder(t.symbol),o=0,s=i.members;o<s.length;o++){var c=s[o];e.isMethodOrAccessor(c)&&e.isStatic(c)&&(c.body&&c.body.forEachChild(function n(t){108===t.kind?a(t):e.isFunctionLike(t)||e.isClassLike(t)||t.forEachChild(n)}))}}}function X(n,t,r){var i=Y(n);if(i&&i.declarations)for(var a=0,o=i.declarations;a<o.length;a++){var s=o[a],c=e.findChildOfKind(s,133,t);e.Debug.assert(169===s.kind&&!!c),r(c)}n.exports&&n.exports.forEach(function(n){var t=n.valueDeclaration;if(t&&167===t.kind){var i=t.body;i&&ge(i,108,function(n){e.isNewExpressionTarget(n)&&r(n)})}})}function Y(e){return e.members&&e.members.get("__constructor")}function Q(n,t){var r=Y(n.symbol);if(r&&r.declarations)for(var i=0,a=r.declarations;i<a.length;i++){var o=a[i];e.Debug.assert(169===o.kind);var s=o.body;s&&ge(s,106,function(n){e.isCallExpressionTarget(n)&&t(n)})}}function Z(e){return!!Y(e.symbol)}function $(e,n){if(!Z(e)){var t=e.symbol,r=n.createSearch(void 0,t,void 0);_(t,n,r)}}function ee(n,t,r){if(e.isDeclarationName(n)&&me(n.parent))t(n);else if(79===n.kind){292===n.parent.kind&&_e(n,r.checker,t);var i=ne(n);if(i)t(i);else{var a=e.findAncestor(n,function(n){return!e.isQualifiedName(n.parent)&&!e.isTypeNode(n.parent)&&!e.isTypeElement(n.parent)}),o=a.parent;if(e.hasType(o)&&o.type===a&&r.markSeenContainingTypeReference(o))if(e.hasInitializer(o))c(o.initializer);else if(e.isFunctionLike(o)&&o.body){var s=o.body;233===s.kind?e.forEachReturnStatement(s,function(e){e.expression&&c(e.expression)}):c(s)}else e.isAssertionExpression(o)&&c(o.expression)}}function c(e){te(e)&&t(e)}}function ne(n){return e.isIdentifier(n)||e.isPropertyAccessExpression(n)?ne(n.parent):e.isExpressionWithTypeArguments(n)?e.tryCast(n.parent.parent,e.isClassLike):void 0}function te(e){switch(e.kind){case 210:return te(e.expression);case 212:case 211:case 203:case 224:case 202:return!0;default:return!1}}function re(n,t,r,i){if(n===t)return!0;var a=e.getSymbolId(n)+","+e.getSymbolId(t),o=r.get(a);if(void 0!==o)return o;r.set(a,!1);var s=!!n.declarations&&n.declarations.some(function(n){return e.getAllSuperTypeNodes(n).some(function(e){var n=i.getTypeAtLocation(e);return!!n&&!!n.symbol&&re(n.symbol,t,r,i)})});return r.set(a,s),s}function ie(n){var t=e.getSuperContainer(n,!1);if(t){var i=32;switch(t.kind){case 165:case 164:case 167:case 166:case 169:case 170:case 171:i&=e.getSyntacticModifierFlags(t),t=t.parent;break;default:return}var a=t.getSourceFile(),o=e.mapDefined(R(a,"super",t),function(n){if(106===n.kind){var a=e.getSuperContainer(n,!1);return a&&e.isStatic(a)===!!i&&a.parent.symbol===t.symbol?r(n):void 0}});return[{definition:{type:0,symbol:t.symbol},references:o}]}}function ae(e){return 79===e.kind&&162===e.parent.kind&&e.parent.name===e}function oe(n,t,i){var a=e.getThisContainer(n,!1),o=32;switch(a.kind){case 167:case 166:if(e.isObjectLiteralMethod(a)){o&=e.getSyntacticModifierFlags(a),a=a.parent;break}case 165:case 164:case 169:case 170:case 171:o&=e.getSyntacticModifierFlags(a),a=a.parent;break;case 300:if(e.isExternalModule(a)||ae(n))return;case 254:case 211:break;default:return}var s=e.flatMap(300===a.kind?t:[a.getSourceFile()],function(n){return i.throwIfCancellationRequested(),R(n,"this",e.isSourceFile(a)?n:a).filter(function(n){if(!e.isThis(n))return!1;var t=e.getThisContainer(n,!1);switch(a.kind){case 211:case 254:return a.symbol===t.symbol;case 167:case 166:return e.isObjectLiteralMethod(a)&&a.symbol===t.symbol;case 224:case 255:case 203:return t.parent&&a.symbol===t.parent.symbol&&e.isStatic(t)===!!o;case 300:return 300===t.kind&&!e.isExternalModule(t)&&!ae(n)}})}).map(function(e){return r(e)}),c=e.firstDefined(s,function(n){return e.isParameter(n.node.parent)?n.node:void 0});return[{definition:{type:3,node:c||n},references:s}]}function se(n,t,i,a){var o=e.getContextualTypeOrAncestorTypeNodeType(n,i),s=e.flatMap(t,function(t){return a.throwIfCancellationRequested(),e.mapDefined(R(t,n.text),function(t){if(e.isStringLiteralLike(t)&&t.text===n.text){if(!o)return r(t,2);var a=e.getContextualTypeOrAncestorTypeNodeType(t,i);if(o!==i.getStringType()&&o===a)return r(t,2)}})});return[{definition:{type:4,node:n},references:s}]}function ce(e,n,t,r,i,a){var o=[];return le(e,n,t,r,!(r&&i),function(n,t,r){r&&de(e)!==de(r)&&(r=void 0),o.push(r||t||n)},function(){return!a}),o}function le(n,t,r,i,a,o,c){var l=e.getContainingObjectLiteralElement(t);if(l){var u=r.getShorthandAssignmentValueSymbol(t.parent);if(u&&i)return o(u,void 0,void 0,3);var d=r.getContextualType(l.parent),p=d&&e.firstDefined(e.getPropertySymbolsFromContextualType(l,r,d,!0),function(e){return L(e,4)});if(p)return p;var f=D(t,r),m=f&&o(f,void 0,void 0,4);if(m)return m;var _=u&&o(u,void 0,void 0,3);if(_)return _}var g=s(t,n,r);if(g){var y=o(g,void 0,void 0,1);if(y)return y}var h=L(n);if(h)return h;if(n.valueDeclaration&&e.isParameterPropertyDeclaration(n.valueDeclaration,n.valueDeclaration.parent)){var v=r.getSymbolsOfParameterPropertyDeclaration(e.cast(n.valueDeclaration,e.isParameter),n.name);return e.Debug.assert(2===v.length&&!!(1&v[0].flags)&&!!(4&v[1].flags)),L(1&n.flags?v[1]:v[0])}var b=e.getDeclarationOfKind(n,273);if(!i||b&&!b.propertyName){var E=b&&r.getExportSpecifierLocalTargetSymbol(b);if(E){var T=o(E,void 0,void 0,1);if(T)return T}}if(!i){var S=void 0;return S=a?e.isObjectBindingElementWithoutPropertyName(t.parent)?e.getPropertySymbolFromBindingElement(r,t.parent):void 0:A(n,r),S&&L(S,4)}e.Debug.assert(i);var x=a;if(x){S=A(n,r);return S&&L(S,4)}function L(n,t){return e.firstDefined(r.getRootSymbols(n),function(e){return o(n,e,void 0,t)||(e.parent&&96&e.parent.flags&&c(e)?ue(e.parent,e.name,r,function(r){return o(n,e,r,t)}):void 0)})}function A(n,t){var r=e.getDeclarationOfKind(n,201);if(r&&e.isObjectBindingElementWithoutPropertyName(r))return e.getPropertySymbolFromBindingElement(t,r)}}function ue(n,t,r,i){var a=new e.Map;return o(n);function o(n){if(96&n.flags&&e.addToSeen(a,e.getSymbolId(n)))return e.firstDefined(n.declarations,function(n){return e.firstDefined(e.getAllSuperTypeNodes(n),function(n){var a=r.getTypeAtLocation(n),s=a&&a.symbol&&r.getPropertyOfType(a,t);return a&&s&&(e.firstDefined(r.getRootSymbols(s),i)||o(a.symbol))})})}}function de(n){if(!n.valueDeclaration)return!1;var t=e.getEffectiveModifierFlags(n.valueDeclaration);return!!(32&t)}function pe(n,t,r,i){var a=i.checker;return le(t,r,a,!1,2!==i.options.use||!!i.options.providePrefixAndSuffixTextForRename,function(r,i,a,o){return a&&de(t)!==de(a)&&(a=void 0),n.includes(a||i||r)?{symbol:!i||6&e.getCheckFlags(r)?r:i,kind:o}:void 0},function(e){return!(n.parents&&!n.parents.some(function(n){return re(e.parent,n,i.inheritsFromCache,a)}))})}function fe(n,t){var r=e.getMeaningFromLocation(n),i=t.declarations;if(i){var a=void 0;do{a=r;for(var o=0,s=i;o<s.length;o++){var c=s[o],l=e.getMeaningFromDeclaration(c);l&r&&(r|=l)}}while(r!==a)}return r}function me(n){return 8388608&n.flags?!(e.isInterfaceDeclaration(n)||e.isTypeAliasDeclaration(n)):e.isVariableLike(n)?e.hasInitializer(n):e.isFunctionLikeDeclaration(n)?!!n.body:e.isClassLike(n)||e.isModuleOrEnumDeclaration(n)}function _e(n,t,r){var i=t.getSymbolAtLocation(n),a=t.getShorthandAssignmentValueSymbol(i.valueDeclaration);if(a)for(var o=0,s=a.getDeclarations();o<s.length;o++){var c=s[o];1&e.getMeaningFromDeclaration(c)&&r(c)}}function ge(n,t,r){e.forEachChild(n,function(e){e.kind===t&&r(e),ge(e,t,r)})}function ye(n){return e.tryGetClassExtendingExpressionWithTypeArguments(e.climbPastPropertyAccess(n).parent)}function he(n,t,r){var i=e.isRightSideOfPropertyAccess(n)?n.parent:void 0,a=i&&r.getTypeAtLocation(i.expression),o=e.mapDefined(a&&(a.isUnionOrIntersection()?a.types:a.symbol===t.parent?void 0:[a]),function(e){return e.symbol&&96&e.symbol.flags?e.symbol:void 0});return 0===o.length?void 0:o}function ve(e){return 2===e.use&&e.providePrefixAndSuffixTextForRename}t.eachExportReference=T,t.isSymbolReferencedInFile=I,t.eachSymbolReferenceInFile=N,t.someSignatureUsage=k,t.getIntersectingMeaningFromDeclarations=fe,t.getReferenceEntriesForShorthandPropertyAssignment=_e}(t=n.Core||(n.Core={}))})(e.FindAllReferences||(e.FindAllReferences={}))}(d||(d={})),function(e){(function(n){function t(n){return(e.isFunctionExpression(n)||e.isClassExpression(n))&&e.isNamedDeclaration(n)}function r(n){return(e.isFunctionExpression(n)||e.isArrowFunction(n)||e.isClassExpression(n))&&e.isVariableDeclaration(n.parent)&&n===n.parent.initializer&&e.isIdentifier(n.parent.name)&&!!(2&e.getCombinedNodeFlags(n.parent))}function i(n){return e.isSourceFile(n)||e.isModuleDeclaration(n)||e.isFunctionDeclaration(n)||e.isFunctionExpression(n)||e.isClassDeclaration(n)||e.isClassExpression(n)||e.isClassStaticBlockDeclaration(n)||e.isMethodDeclaration(n)||e.isMethodSignature(n)||e.isGetAccessorDeclaration(n)||e.isSetAccessorDeclaration(n)}function a(n){return e.isSourceFile(n)||e.isModuleDeclaration(n)&&e.isIdentifier(n.name)||e.isFunctionDeclaration(n)||e.isClassDeclaration(n)||e.isClassStaticBlockDeclaration(n)||e.isMethodDeclaration(n)||e.isMethodSignature(n)||e.isGetAccessorDeclaration(n)||e.isSetAccessorDeclaration(n)||t(n)||r(n)}function o(n){return e.isSourceFile(n)?n:e.isNamedDeclaration(n)?n.name:r(n)?n.parent.name:e.Debug.checkDefined(n.modifiers&&e.find(n.modifiers,s))}function s(e){return 88===e.kind}function c(e,n){var t=o(n);return t&&e.getSymbolAtLocation(t)}function l(n,t){if(e.isSourceFile(t))return{text:t.fileName,pos:0,end:0};if((e.isFunctionDeclaration(t)||e.isClassDeclaration(t))&&!e.isNamedDeclaration(t)){var i=t.modifiers&&e.find(t.modifiers,s);if(i)return{text:"default",pos:i.getStart(),end:i.getEnd()}}if(e.isClassStaticBlockDeclaration(t)){var a=t.getSourceFile(),o=e.skipTrivia(a.text,e.moveRangePastModifiers(t).pos),c=o+6,l=n.getTypeChecker(),u=l.getSymbolAtLocation(t.parent),d=u?l.symbolToString(u,t.parent)+" ":"";return{text:d+"static {}",pos:o,end:c}}var p=r(t)?t.parent.name:e.Debug.checkDefined(e.getNameOfDeclaration(t),"Expected call hierarchy item to have a name"),f=e.isIdentifier(p)?e.idText(p):e.isStringOrNumericLiteralLike(p)?p.text:e.isComputedPropertyName(p)&&e.isStringOrNumericLiteralLike(p.expression)?p.expression.text:void 0;if(void 0===f){l=n.getTypeChecker(),u=l.getSymbolAtLocation(p);u&&(f=l.symbolToString(u,t))}if(void 0===f){var m=e.createPrinter({removeComments:!0,omitTrailingSemicolon:!0});f=e.usingSingleLineStringWriter(function(e){return m.writeNode(4,t,t.getSourceFile(),e)})}return{text:f,pos:p.getStart(),end:p.getEnd()}}function u(n){var t,i;if(r(n))return e.isModuleBlock(n.parent.parent.parent.parent)&&e.isIdentifier(n.parent.parent.parent.parent.parent.name)?n.parent.parent.parent.parent.parent.name.getText():void 0;switch(n.kind){case 170:case 171:case 167:return 203===n.parent.kind?null===(t=e.getAssignedName(n.parent))||void 0===t?void 0:t.getText():null===(i=e.getNameOfDeclaration(n.parent))||void 0===i?void 0:i.getText();case 254:case 255:case 259:if(e.isModuleBlock(n.parent)&&e.isIdentifier(n.parent.parent.name))return n.parent.parent.name.getText()}}function d(n,t){if(t.body)return t;if(e.isConstructorDeclaration(t))return e.getFirstConstructorWithBody(t.parent);if(e.isFunctionDeclaration(t)||e.isMethodDeclaration(t)){var r=c(n,t);return r&&r.valueDeclaration&&e.isFunctionLikeDeclaration(r.valueDeclaration)&&r.valueDeclaration.body?r.valueDeclaration:void 0}return t}function p(n,t){var r,i=c(n,t);if(i&&i.declarations){var o=e.indicesOf(i.declarations),s=e.map(i.declarations,function(e){return{file:e.getSourceFile().fileName,pos:e.pos}});o.sort(function(n,t){return e.compareStringsCaseSensitive(s[n].file,s[t].file)||s[n].pos-s[t].pos});for(var l=e.map(o,function(e){return i.declarations[e]}),u=void 0,d=0,p=l;d<p.length;d++){var f=p[d];a(f)&&(u&&u.parent===f.parent&&u.end===f.pos||(r=e.append(r,f)),u=f)}}return r}function f(n,t){var r,i,a;return e.isClassStaticBlockDeclaration(t)?t:e.isFunctionLikeDeclaration(t)?null!==(i=null!==(r=d(n,t))&&void 0!==r?r:p(n,t))&&void 0!==i?i:t:null!==(a=p(n,t))&&void 0!==a?a:t}function m(n,t){var o=n.getTypeChecker(),s=!1;while(1){if(a(t))return f(o,t);if(i(t)){var c=e.findAncestor(t,a);return c&&f(o,c)}if(e.isDeclarationName(t)){if(a(t.parent))return f(o,t.parent);if(i(t.parent)){c=e.findAncestor(t.parent,a);return c&&f(o,c)}return e.isVariableDeclaration(t.parent)&&t.parent.initializer&&r(t.parent.initializer)?t.parent.initializer:void 0}if(e.isConstructorDeclaration(t))return a(t.parent)?t.parent:void 0;if(124!==t.kind||!e.isClassStaticBlockDeclaration(t.parent)){if(e.isVariableDeclaration(t)&&t.initializer&&r(t.initializer))return t.initializer;if(!s){var l=o.getSymbolAtLocation(t);if(l&&(2097152&l.flags&&(l=o.getAliasedSymbol(l)),l.valueDeclaration)){s=!0,t=l.valueDeclaration;continue}}return}t=t.parent}}function _(n,t){var r=t.getSourceFile(),i=l(n,t),a=u(t),o=e.getNodeKind(t),s=e.getNodeModifiers(t),c=e.createTextSpanFromBounds(e.skipTrivia(r.text,t.getFullStart(),!1,!0),t.getEnd()),d=e.createTextSpanFromBounds(i.pos,i.end);return{file:r.fileName,kind:o,kindModifiers:s,name:i.text,containerName:a,span:c,selectionSpan:d}}function g(e){return void 0!==e}function y(n){if(1===n.kind){var t=n.node;if(e.isCallOrNewExpressionTarget(t,!0,!0)||e.isTaggedTemplateTag(t,!0,!0)||e.isDecoratorTarget(t,!0,!0)||e.isJsxOpeningLikeElementTagName(t,!0,!0)||e.isRightSideOfPropertyAccess(t)||e.isArgumentExpressionOfElementAccess(t)){var r=t.getSourceFile(),i=e.findAncestor(t,a)||r;return{declaration:i,range:e.createTextRangeFromNode(t,r)}}}}function h(n){return e.getNodeId(n.declaration)}function v(e,n){return{from:e,fromSpans:n}}function b(n,t){return v(_(n,t[0].declaration),e.map(t,function(n){return e.createTextSpanFromRange(n.range)}))}function E(n,t,r){if(e.isSourceFile(t)||e.isModuleDeclaration(t)||e.isClassStaticBlockDeclaration(t))return[];var i=o(t),a=e.filter(e.FindAllReferences.findReferenceOrRenameEntries(n,r,n.getSourceFiles(),i,0,{use:1},y),g);return a?e.group(a,h,function(e){return b(n,e)}):[]}function T(n,t){function r(r){var i=e.isTaggedTemplateExpression(r)?r.tag:e.isJsxOpeningLikeElement(r)?r.tagName:e.isAccessExpression(r)?r:e.isClassStaticBlockDeclaration(r)?r:r.expression,a=m(n,i);if(a){var o=e.createTextRangeFromNode(i,r.getSourceFile());if(e.isArray(a))for(var s=0,c=a;s<c.length;s++){var l=c[s];t.push({declaration:l,range:o})}else t.push({declaration:a,range:o})}}function i(n){if(n&&!(8388608&n.flags))if(a(n)){if(e.isClassLike(n))for(var t=0,o=n.members;t<o.length;t++){var s=o[t];s.name&&e.isComputedPropertyName(s.name)&&i(s.name.expression)}}else{switch(n.kind){case 79:case 263:case 264:case 270:case 256:case 257:return;case 168:return void r(n);case 209:case 227:return void i(n.expression);case 252:case 162:return i(n.name),void i(n.initializer);case 206:return r(n),i(n.expression),void e.forEach(n.arguments,i);case 207:return r(n),i(n.expression),void e.forEach(n.arguments,i);case 208:return r(n),i(n.tag),void i(n.template);case 278:case 277:return r(n),i(n.tagName),void i(n.attributes);case 163:return r(n),void i(n.expression);case 204:case 205:r(n),e.forEachChild(n,i);break}e.isPartOfTypeNode(n)||e.forEachChild(n,i)}}return i}function S(n,t){e.forEach(n.statements,t)}function x(n,t){!e.hasSyntacticModifier(n,2)&&n.body&&e.isModuleBlock(n.body)&&e.forEach(n.body.statements,t)}function L(n,t,r){var i=d(n,t);i&&(e.forEach(i.parameters,r),r(i.body))}function D(e,n){n(e.body)}function A(n,t){e.forEach(n.decorators,t);var r=e.getClassExtendsHeritageElement(n);r&&t(r.expression);for(var i=0,a=n.members;i<a.length;i++){var o=a[i];e.forEach(o.decorators,t),e.isPropertyDeclaration(o)?t(o.initializer):e.isConstructorDeclaration(o)&&o.body?(e.forEach(o.parameters,t),t(o.body)):e.isClassStaticBlockDeclaration(o)&&t(o)}}function C(n,t){var r=[],i=T(n,r);switch(t.kind){case 300:S(t,i);break;case 259:x(t,i);break;case 254:case 211:case 212:case 167:case 170:case 171:L(n.getTypeChecker(),t,i);break;case 255:case 224:A(t,i);break;case 168:D(t,i);break;default:e.Debug.assertNever(t)}return r}function I(e,n){return{to:e,fromSpans:n}}function N(n,t){return I(_(n,t[0].declaration),e.map(t,function(n){return e.createTextSpanFromRange(n.range)}))}function k(n,t){return 8388608&t.flags||e.isMethodSignature(t)?[]:e.group(C(n,t),h,function(e){return N(n,e)})}n.resolveCallHierarchyDeclaration=m,n.createCallHierarchyItem=_,n.getIncomingCalls=E,n.getOutgoingCalls=k})(e.CallHierarchy||(e.CallHierarchy={}))}(d||(d={})),function(e){function n(n,r,o,s,c,l,u){var d=e.hostUsesCaseSensitiveFileNames(s),p=e.createGetCanonicalFileName(d),f=t(r,o,p,u),m=t(o,r,p,u);return e.textChanges.ChangeTracker.with({host:s,formatContext:c,preferences:l},function(e){i(n,e,f,r,o,s.getCurrentDirectory(),d),a(n,e,f,m,s,p)})}function t(n,t,i,a){var o=i(n);return function(e){var n=a&&a.tryGetSourcePosition({fileName:e,pos:0}),t=s(n?n.fileName:e);return n?void 0===t?void 0:r(n.fileName,t,e,i):t};function s(n){if(i(n)===o)return t;var r=e.tryRemoveDirectoryPrefix(n,o,i);return void 0===r?void 0:t+"/"+r}}function r(n,t,r,i){var a=e.getRelativePathFromFile(n,t,i);return s(e.getDirectoryPath(r),a)}function i(n,t,r,i,a,o,c){var l=n.getCompilerOptions().configFile;if(l){var u=e.getDirectoryPath(l.fileName),f=e.getTsConfigObjectLiteralExpression(l);f&&p(f,function(n,r){switch(r){case"files":case"include":case"exclude":var s=m(n);if(s||"include"!==r||!e.isArrayLiteralExpression(n.initializer))return;var d=e.mapDefined(n.initializer.elements,function(n){return e.isStringLiteral(n)?n.text:void 0});if(0===d.length)return;var f=e.getFileMatcherPatterns(u,[],d,c,o);return void(e.getRegexFromPattern(e.Debug.checkDefined(f.includeFilePattern),c).test(i)&&!e.getRegexFromPattern(e.Debug.checkDefined(f.includeFilePattern),c).test(a)&&t.insertNodeAfter(l,e.last(n.initializer.elements),e.factory.createStringLiteral(g(a))));case"compilerOptions":return void p(n.initializer,function(n,t){var r=e.getOptionFromName(t);r&&(r.isFilePath||"list"===r.type&&r.element.isFilePath)?m(n):"paths"===t&&p(n.initializer,function(n){if(e.isArrayLiteralExpression(n.initializer))for(var t=0,r=n.initializer.elements;t<r.length;t++){var i=r[t];_(i)}})})}})}function m(n){for(var t=e.isArrayLiteralExpression(n.initializer)?n.initializer.elements:[n.initializer],r=!1,i=0,a=t;i<a.length;i++){var o=a[i];r=_(o)||r}return r}function _(n){if(!e.isStringLiteral(n))return!1;var i=s(u,n.text),a=r(i);return void 0!==a&&(t.replaceRangeWithText(l,d(n,l),g(a)),!0)}function g(n){return e.getRelativePathFromDirectory(u,n,!c)}}function a(n,t,r,i,a,o){for(var d=n.getSourceFiles(),p=function(p){var f=r(p.fileName),m=null!==f&&void 0!==f?f:p.fileName,_=e.getDirectoryPath(m),g=i(p.fileName),y=g||p.fileName,h=e.getDirectoryPath(y),v=void 0!==f||void 0!==g;u(p,t,function(n){if(e.pathIsRelative(n)){var t=s(h,n),i=r(t);return void 0===i?void 0:e.ensurePathIsNonModuleName(e.getRelativePathFromDirectory(_,i,o))}},function(t){var i=n.getTypeChecker().getSymbolAtLocation(t);if(!((null===i||void 0===i?void 0:i.declarations)&&i.declarations.some(function(n){return e.isAmbientModule(n)}))){var s=void 0!==g?l(t,e.resolveModuleName(t.text,y,n.getCompilerOptions(),a),r,d):c(i,t,p,n,a,r);return void 0!==s&&(s.updated||v&&e.pathIsRelative(t.text))?e.moduleSpecifiers.updateModuleSpecifier(n.getCompilerOptions(),o(m),s.newFileName,e.createModuleSpecifierResolutionHost(n,a),t.text):void 0}})},f=0,m=d;f<m.length;f++){var _=m[f];p(_)}}function o(n,t){return e.normalizePath(e.combinePaths(n,t))}function s(n,t){return e.ensurePathIsNonModuleName(o(n,t))}function c(n,t,r,i,a,o){if(n){var s=e.find(n.declarations,e.isSourceFile).fileName,c=o(s);return void 0===c?{newFileName:s,updated:!1}:{newFileName:c,updated:!0}}var u=a.resolveModuleNames?a.getResolvedModuleWithFailedLookupLocationsFromCache&&a.getResolvedModuleWithFailedLookupLocationsFromCache(t.text,r.fileName):i.getResolvedModuleWithFailedLookupLocationsFromCache(t.text,r.fileName);return l(t,u,o,i.getSourceFiles())}function l(n,t,r,i){if(t){if(t.resolvedModule){var a=l(t.resolvedModule.resolvedFileName);if(a)return a}var o=e.forEach(t.failedLookupLocations,s)||e.pathIsRelative(n.text)&&e.forEach(t.failedLookupLocations,c);return o||t.resolvedModule&&{newFileName:t.resolvedModule.resolvedFileName,updated:!1}}function s(n){var t=r(n);return t&&e.find(i,function(e){return e.fileName===t})?c(n):void 0}function c(n){return e.endsWith(n,"/package.json")?void 0:l(n)}function l(e){var n=r(e);return n&&{newFileName:n,updated:!0}}}function u(n,t,r,i){for(var a=0,o=n.referencedFiles||e.emptyArray;a<o.length;a++){var s=o[a],c=r(s.fileName);void 0!==c&&c!==n.text.slice(s.pos,s.end)&&t.replaceRangeWithText(n,s,c)}for(var l=0,u=n.imports;l<u.length;l++){var p=u[l];c=i(p);void 0!==c&&c!==p.text&&t.replaceRangeWithText(n,d(p,n),c)}}function d(n,t){return e.createRange(n.getStart(t)+1,n.end-1)}function p(n,t){if(e.isObjectLiteralExpression(n))for(var r=0,i=n.properties;r<i.length;r++){var a=i[r];e.isPropertyAssignment(a)&&e.isStringLiteral(a.name)&&t(a,a.name.text)}}e.getEditsForFileRename=n,e.getPathUpdater=t}(d||(d={})),function(e){(function(n){function t(n,t,o){var l=c(t,o,n),u=l&&[T(l.reference.fileName,l.fileName,l.unverified)]||e.emptyArray;if(null===l||void 0===l?void 0:l.file)return u;var d=e.getTouchingPropertyName(t,o);if(d!==t){var p=d.parent,_=n.getTypeChecker();if(157===d.kind||e.isJSDocOverrideTag(d)&&e.rangeContainsPosition(d.tagName,o))return s(_,d)||e.emptyArray;if(e.isJumpStatementTarget(d)){var v=e.getTargetLabel(d.parent,d.text);return v?[h(_,v,"label",d.text,void 0)]:void 0}if(e.isStaticModifier(d)&&e.isClassStaticBlockDeclaration(d.parent)){var E=d.parent.parent,S=m(E,_),D=e.filter(E.members,e.isClassStaticBlockDeclaration),A=S?_.symbolToString(S,E):"",C=d.getSourceFile();return e.map(D,function(n){var t=e.moveRangePastModifiers(n).pos;return t=e.skipTrivia(C.text,t),h(_,n,"constructor","static {}",A,{start:t,length:"static".length})})}var I=m(d,_);if(!I)return e.concatenate(u,f(d,_));var N=x(_,d);if(N&&(!e.isJsxOpeningLikeElement(d.parent)||!L(N))){var k=b(_,N);if(_.getRootSymbols(I).some(function(e){return r(e,N)}))return[k];var R=g(_,I,d,N)||e.emptyArray;return 106===d.kind?a([k],R,!0):a(a([],R,!0),[k],!1)}if(292===d.parent.kind){var O=_.getShorthandAssignmentValueSymbol(I.valueDeclaration),w=(null===O||void 0===O?void 0:O.declarations)?O.declarations.map(function(e){return y(e,_,O,d)}):e.emptyArray;return e.concatenate(w,i(_,d)||e.emptyArray)}if(e.isPropertyName(d)&&e.isBindingElement(p)&&e.isObjectBindingPattern(p.parent)&&d===(p.propertyName||p.name)){var P=e.getNameFromPropertyName(d),M=_.getTypeAtLocation(p.parent);return void 0===P?e.emptyArray:e.flatMap(M.isUnion()?M.types:[M],function(e){var n=e.getProperty(P);return n&&g(_,n,d)})}return e.concatenate(u,i(_,d)||g(_,I,d))}}function r(n,t){return n===t.symbol||n===t.symbol.parent||e.isAssignmentExpression(t.parent)||!e.isCallLikeExpression(t.parent)&&n===t.parent.symbol}function i(n,t){var r=e.getContainingObjectLiteralElement(t);if(r){var i=r&&n.getContextualType(r.parent);if(i)return e.flatMap(e.getPropertySymbolsFromContextualType(r,n,i,!1),function(e){return g(n,e,t)})}}function s(n,t){var r=e.findAncestor(t,e.isClassElement);if(r&&r.name){var i=e.findAncestor(r,e.isClassLike);if(i){var a=e.getEffectiveBaseTypeNode(i),o=a?n.getTypeAtLocation(a):void 0;if(o){var s=e.unescapeLeadingUnderscores(e.getTextOfPropertyName(r.name)),c=e.hasStaticModifier(r)?n.getPropertyOfType(n.getTypeOfSymbolAtLocation(o.symbol,i),s):n.getPropertyOfType(o,s);if(c)return g(n,c,t)}}}}function c(n,t,r){var i,a,o=E(n.referencedFiles,t);if(o){var s=r.getSourceFileFromReference(n,o);return s&&{reference:o,fileName:s.fileName,file:s,unverified:!1}}var c=E(n.typeReferenceDirectives,t);if(c){var l=r.getResolvedTypeReferenceDirectives().get(c.fileName);s=l&&r.getSourceFile(l.resolvedFileName);return s&&{reference:c,fileName:s.fileName,file:s,unverified:!1}}var u=E(n.libReferenceDirectives,t);if(u){s=r.getLibFileFromReference(u);return s&&{reference:u,fileName:s.fileName,file:s,unverified:!1}}if(null===(i=n.resolvedModules)||void 0===i?void 0:i.size){var d=e.getTokenAtPosition(n,t);if(e.isModuleSpecifierLike(d)&&e.isExternalModuleNameRelative(d.text)&&n.resolvedModules.has(d.text)){var p=null===(a=n.resolvedModules.get(d.text))||void 0===a?void 0:a.resolvedFileName,f=p||e.resolvePath(e.getDirectoryPath(n.fileName),d.text);return{file:r.getSourceFile(f),fileName:f,reference:{pos:d.getStart(),end:d.getEnd(),fileName:d.text},unverified:!p}}}}function l(n,t,r){var i=e.getTouchingPropertyName(t,r);if(i!==t){var a=n.getSymbolAtLocation(i);if(a){var o=n.getTypeOfSymbolAtLocation(a,i),s=d(a,o,n),c=s&&u(s,n,i);return c&&0!==c.length?c:u(o,n,i)}}}function u(n,t,r){return e.flatMap(!n.isUnion()||32&n.flags?[n]:n.types,function(e){return e.symbol&&g(t,e.symbol,r)})}function d(n,t,r){if(t.symbol===n||n.valueDeclaration&&t.symbol&&e.isVariableDeclaration(n.valueDeclaration)&&n.valueDeclaration.initializer===t.symbol.valueDeclaration){var i=t.getCallSignatures();if(1===i.length)return r.getReturnTypeOfSignature(e.first(i))}}function p(n,r,i){var a=t(n,r,i);if(a&&0!==a.length){var o=E(r.referencedFiles,i)||E(r.typeReferenceDirectives,i)||E(r.libReferenceDirectives,i);if(o)return{definitions:a,textSpan:e.createTextSpanFromRange(o)};var s=e.getTouchingPropertyName(r,i),c=e.createTextSpan(s.getStart(),s.getWidth());return{definitions:a,textSpan:c}}}function f(n,t){return e.mapDefined(t.getIndexInfosAtLocation(n),function(e){return e.declaration&&b(t,e.declaration)})}function m(e,n){var t=n.getSymbolAtLocation(e);if((null===t||void 0===t?void 0:t.declarations)&&2097152&t.flags&&_(e,t.declarations[0])){var r=n.getAliasedSymbol(t);if(r.declarations)return r}return t}function _(n,t){if(79!==n.kind)return!1;if(n.parent===t)return!0;switch(t.kind){case 265:case 263:return!0;case 268:return 267===t.parent.kind;case 201:case 252:return e.isInJSFile(t)&&e.isRequireVariableDeclaration(t);default:return!1}}function g(n,t,r,i){var a=e.filter(t.declarations,function(n){return n!==i&&(!e.isAssignmentDeclaration(n)||n===t.valueDeclaration)})||void 0;return o()||s()||e.map(a,function(e){return y(e,n,t,r)});function o(){if(32&t.flags&&!(19&t.flags)&&(e.isNewExpressionTarget(r)||133===r.kind)){var n=e.find(a,e.isClassLike)||e.Debug.fail("Expected declaration to have at least one class-like declaration");return c(n.members,!0)}}function s(){return e.isCallOrNewExpressionTarget(r)||e.isNameOfFunctionDeclaration(r)?c(a,!1):void 0}function c(i,a){if(i){var o=i.filter(a?e.isConstructorDeclaration:e.isFunctionLike),s=o.filter(function(e){return!!e.body});return o.length?0!==s.length?s.map(function(e){return y(e,n,t,r)}):[y(e.last(o),n,t,r)]:void 0}}}function y(n,t,r,i){var a=t.symbolToString(r),o=e.SymbolDisplay.getSymbolKind(t,r,i),s=r.parent?t.symbolToString(r.parent,i):"";return h(t,n,o,a,s)}function h(n,t,r,i,a,s){var c=t.getSourceFile();if(!s){var l=e.getNameOfDeclaration(t)||t;s=e.createTextSpanFromNode(l,c)}return o(o({fileName:c.fileName,textSpan:s,kind:r,name:i,containerKind:void 0,containerName:a},e.FindAllReferences.toContextSpan(s,c,e.FindAllReferences.getContextNode(t))),{isLocal:!v(n,t)})}function v(n,t){if(n.isDeclarationVisible(t))return!0;if(!t.parent)return!1;if(e.hasInitializer(t.parent)&&t.parent.initializer===t)return v(n,t.parent);switch(t.kind){case 165:case 170:case 171:case 167:if(e.hasEffectiveModifier(t,8))return!1;case 169:case 291:case 292:case 203:case 224:case 212:case 211:return v(n,t.parent);default:return!1}}function b(e,n){return y(n,e,n.symbol,n)}function E(n,t){return e.find(n,function(n){return e.textRangeContainsPositionInclusive(n,t)})}function T(n,t,r){return{fileName:t,textSpan:e.createTextSpanFromBounds(0,0),kind:"script",name:n,containerName:void 0,containerKind:void 0,unverified:r}}function S(n){var t=e.findAncestor(n,function(n){return!e.isRightSideOfPropertyAccess(n)}),r=null===t||void 0===t?void 0:t.parent;return r&&e.isCallLikeExpression(r)&&e.getInvokedExpression(r)===t?r:void 0}function x(n,t){var r=S(t),i=r&&n.getResolvedSignature(r);return e.tryCast(i&&i.declaration,function(n){return e.isFunctionLike(n)&&!e.isFunctionTypeNode(n)})}function L(e){switch(e.kind){case 169:case 178:case 173:return!0;default:return!1}}n.getDefinitionAtPosition=t,n.getReferenceAtPosition=c,n.getTypeDefinitionAtPosition=l,n.getDefinitionAndBoundSpan=p,n.findReferenceInPosition=E})(e.GoToDefinition||(e.GoToDefinition={}))}(d||(d={})),function(e){(function(n){var t,r,i=["abstract","access","alias","argument","async","augments","author","borrows","callback","class","classdesc","constant","constructor","constructs","copyright","default","deprecated","description","emits","enum","event","example","exports","extends","external","field","file","fileoverview","fires","function","generator","global","hideconstructor","host","ignore","implements","inheritdoc","inner","instance","interface","kind","lends","license","link","listens","member","memberof","method","mixes","module","name","namespace","override","package","param","private","property","protected","public","readonly","requires","returns","see","since","static","summary","template","this","throws","todo","tutorial","type","typedef","var","variation","version","virtual","yields"];function o(n,t){var r=[];return e.forEachUnique(n,function(n){for(var i=0,a=c(n);i<a.length;i++){var o=a[i];if(!(void 0===o.comment||e.isJSDoc(o)&&340!==n.kind&&333!==n.kind&&o.tags&&o.tags.some(function(e){return 340===e.kind||333===e.kind})&&!o.tags.some(function(e){return 335===e.kind||336===e.kind}))){var l=u(o.comment,t);e.contains(r,l,s)||r.push(l)}}}),e.flatten(e.intersperse(r,[e.lineBreakPart()]))}function s(n,t){return e.arraysEqual(n,t,function(e,n){return e.kind===n.kind&&e.text===n.text})}function c(n){switch(n.kind){case 335:case 342:return[n];case 333:case 340:return[n,n.parent];default:return e.getJSDocCommentsAndTags(n)}}function l(n,t){var r=[];return e.forEachUnique(n,function(n){var i=e.getJSDocTags(n);if(!i.some(function(e){return 340===e.kind||333===e.kind})||i.some(function(e){return 335===e.kind||336===e.kind}))for(var a=0,o=i;a<o.length;a++){var s=o[a];r.push({name:s.tagName.text,text:d(s,t)})}}),r}function u(n,t){return"string"===typeof n?[e.textPart(n)]:e.flatMap(n,function(n){return 316===n.kind?[e.textPart(n.text)]:e.buildLinkParts(n,t)})}function d(n,t){var r=n.comment,i=n.kind,o=p(i);switch(i){case 324:return c(n.class);case 323:return c(n.class);case 339:return l(n.typeParameters.map(function(e){return e.getText()}).join(", "));case 338:return c(n.typeExpression);case 340:case 333:case 342:case 335:case 341:var s=n.name;return s?c(s):void 0===r?void 0:u(r,t);default:return void 0===r?void 0:u(r,t)}function c(e){return l(e.getText())}function l(n){return r?n.match(/^https?$/)?a([e.textPart(n)],u(r,t),!0):a([o(n),e.spacePart()],u(r,t),!0):[e.textPart(n)]}}function p(n){switch(n){case 335:return e.parameterNamePart;case 342:return e.propertyNamePart;case 339:return e.typeParameterNamePart;case 340:case 333:return e.typeAliasNamePart;default:return e.textPart}}function f(){return t||(t=e.map(i,function(n){return{name:n,kind:"keyword",kindModifiers:"",sortText:e.Completions.SortText.LocationPriority}}))}function m(){return r||(r=e.map(i,function(n){return{name:"@"+n,kind:"keyword",kindModifiers:"",sortText:e.Completions.SortText.LocationPriority}}))}function _(n){return{name:n,kind:"",kindModifiers:"",displayParts:[e.textPart(n)],documentation:e.emptyArray,tags:void 0,codeActions:void 0}}function g(n){if(!e.isIdentifier(n.name))return e.emptyArray;var t=n.name.text,r=n.parent,i=r.parent;return e.isFunctionLike(i)?e.mapDefined(i.parameters,function(i){if(e.isIdentifier(i.name)){var a=i.name.text;if(!r.tags.some(function(t){return t!==n&&e.isJSDocParameterTag(t)&&e.isIdentifier(t.name)&&t.name.escapedText===a})&&(void 0===t||e.startsWith(a,t)))return{name:a,kind:"parameter",kindModifiers:"",sortText:e.Completions.SortText.LocationPriority}}}):[]}function y(n){return{name:n,kind:"parameter",kindModifiers:"",displayParts:[e.textPart(n)],documentation:e.emptyArray,tags:void 0,codeActions:void 0}}function h(n,t,r,i){var a=e.getTokenAtPosition(t,r),o=e.findAncestor(a,e.isJSDoc);if(!o||void 0===o.comment&&!e.length(o.tags)){var s=a.getStart(t);if(o||!(s<r)){var c=T(a,i);if(c){var l=c.commentOwner,u=c.parameters,d=c.hasReturn;if(!(l.getStart(t)<r)){var p=v(t,r),f=e.hasJSFileExtension(t.fileName),m=(u?b(u||[],f,p,n):"")+(d?E(p,n):""),_="/**",g=" */";if(m){var y=_+n+p+" * ",h=s===r?n+p:"",S=y+n+m+p+g+h;return{newText:S,caretOffset:y.length}}return{newText:_+g,caretOffset:3}}}}}}function v(n,t){for(var r=n.text,i=e.getLineStartPositionForPosition(t,n),a=i;a<=t&&e.isWhiteSpaceSingleLine(r.charCodeAt(a));a++);return r.slice(i,a)}function b(e,n,t,r){return e.map(function(e,i){var a=e.name,o=e.dotDotDotToken,s=79===a.kind?a.text:"param"+i,c=n?o?"{...any} ":"{any} ":"";return t+" * @param "+c+s+r}).join("")}function E(e,n){return e+" * @returns"+n}function T(n,t){return e.forEachAncestor(n,function(e){return S(e,t)})}function S(n,t){switch(n.kind){case 254:case 211:case 167:case 169:case 166:case 212:var r=n;return{commentOwner:n,parameters:r.parameters,hasReturn:x(r,t)};case 291:return S(n.initializer,t);case 255:case 256:case 164:case 258:case 294:case 257:return{commentOwner:n};case 235:var i=n,a=i.declarationList.declarations,o=1===a.length&&a[0].initializer?L(a[0].initializer):void 0;return o?{commentOwner:n,parameters:o.parameters,hasReturn:x(o,t)}:{commentOwner:n};case 300:return"quit";case 259:return 259===n.parent.kind?void 0:{commentOwner:n};case 236:return S(n.expression,t);case 219:var s=n;return 0===e.getAssignmentDeclarationKind(s)?"quit":e.isFunctionLike(s.right)?{commentOwner:n,parameters:s.right.parameters,hasReturn:x(s.right,t)}:{commentOwner:n};case 165:var c=n.initializer;if(c&&(e.isFunctionExpression(c)||e.isArrowFunction(c)))return{commentOwner:n,parameters:c.parameters,hasReturn:x(c,t)}}}function x(n,t){return!!(null===t||void 0===t?void 0:t.generateReturnInDocTemplate)&&(e.isArrowFunction(n)&&e.isExpression(n.body)||e.isFunctionLikeDeclaration(n)&&n.body&&e.isBlock(n.body)&&!!e.forEachReturnStatement(n.body,function(e){return e}))}function L(n){while(210===n.kind)n=n.expression;switch(n.kind){case 211:case 212:return n;case 224:return e.find(n.members,e.isConstructorDeclaration)}}n.getJsDocCommentsFromDeclarations=o,n.getJsDocTagsFromDeclarations=l,n.getJSDocTagNameCompletions=f,n.getJSDocTagNameCompletionDetails=_,n.getJSDocTagCompletions=m,n.getJSDocTagCompletionDetails=_,n.getJSDocParameterNameCompletions=g,n.getJSDocParameterNameCompletionDetails=y,n.getDocCommentTemplateAtPosition=h})(e.JsDoc||(e.JsDoc={}))}(d||(d={})),function(e){(function(n){function t(n,t,i,a,o,s){var c=e.createPatternMatcher(a);if(!c)return e.emptyArray;for(var d=[],p=function(e){if(i.throwIfCancellationRequested(),s&&e.isDeclarationFile)return"continue";e.getNamedDeclarations().forEach(function(n,i){r(c,i,n,t,e.fileName,d)})},f=0,m=n;f<m.length;f++){var _=m[f];p(_)}return d.sort(l),(void 0===o?d:d.slice(0,o)).map(u)}function r(e,n,t,r,a,o){var s=e.getMatchForLastSegmentOfPattern(n);if(s)for(var l=0,u=t;l<u.length;l++){var d=u[l];if(i(d,r))if(e.patternContainsDots){var p=e.getFullMatch(c(d),n);p&&o.push({name:n,fileName:a,matchKind:p.kind,isCaseSensitive:p.isCaseSensitive,declaration:d})}else o.push({name:n,fileName:a,matchKind:s.kind,isCaseSensitive:s.isCaseSensitive,declaration:d})}}function i(e,n){switch(e.kind){case 265:case 268:case 263:var t=n.getSymbolAtLocation(e.name),r=n.getAliasedSymbol(t);return t.escapedName!==r.escapedName;default:return!0}}function a(n,t){var r=e.getNameOfDeclaration(n);return!!r&&(s(r,t)||160===r.kind&&o(r.expression,t))}function o(n,t){return s(n,t)||e.isPropertyAccessExpression(n)&&(t.push(n.name.text),!0)&&o(n.expression,t)}function s(n,t){return e.isPropertyNameLiteral(n)&&(t.push(e.getTextOfIdentifierOrLiteral(n)),!0)}function c(n){var t=[],r=e.getNameOfDeclaration(n);if(r&&160===r.kind&&!o(r.expression,t))return e.emptyArray;t.shift();var i=e.getContainerNode(n);while(i){if(!a(i,t))return e.emptyArray;i=e.getContainerNode(i)}return t.reverse()}function l(n,t){return e.compareValues(n.matchKind,t.matchKind)||e.compareStringsCaseSensitiveUI(n.name,t.name)}function u(n){var t=n.declaration,r=e.getContainerNode(t),i=r&&e.getNameOfDeclaration(r);return{name:n.name,kind:e.getNodeKind(t),kindModifiers:e.getNodeModifiers(t),matchKind:e.PatternMatchKind[n.matchKind],isCaseSensitive:n.isCaseSensitive,fileName:n.fileName,textSpan:e.createTextSpanFromNode(t),containerName:i?i.text:"",containerKind:i?e.getNodeKind(r):""}}n.getNavigateToItems=t})(e.NavigateTo||(e.NavigateTo={}))}(d||(d={})),function(e){(function(n){var t,r,i,a,s,c=/\s+/g,l=150,u=[],d=[],p=[];function f(n,t){r=t,i=n;try{return e.map(H(v(n)),z)}finally{_()}}function m(e,n){r=n,i=e;try{return W(v(e))}finally{_()}}function _(){i=void 0,r=void 0,u=[],a=void 0,p=[]}function g(e){return re(e.getText(i))}function y(e){return e.node.kind}function h(e,n){e.children?e.children.push(n):e.children=[n]}function v(n){e.Debug.assert(!u.length);var t={node:n,name:void 0,additionalNodes:void 0,parent:void 0,children:void 0,indent:0};a=t;for(var r=0,i=n.statements;r<i.length;r++){var o=i[r];N(o)}return D(),e.Debug.assert(!a&&!u.length),t}function b(e,n){h(a,E(e,n))}function E(n,t){return{node:n,name:t||(e.isDeclaration(n)||e.isExpression(n)?e.getNameOfDeclaration(n):void 0),additionalNodes:void 0,parent:a,children:void 0,indent:a.indent+1}}function T(n){s||(s=new e.Map),s.set(n,!0)}function S(e){for(var n=0;n<e;n++)D()}function x(n,t){var r=[];while(!e.isPropertyNameLiteral(t)){var i=e.getNameOrArgument(t),a=e.getElementOrPropertyAccessName(t);t=t.expression,"prototype"===a||e.isPrivateIdentifier(i)||r.push(i)}r.push(t);for(var o=r.length-1;o>0;o--){i=r[o];L(n,i)}return[r.length-1,r[0]]}function L(e,n){var t=E(e,n);h(a,t),u.push(a),d.push(s),s=void 0,a=t}function D(){a.children&&(k(a.children,a),U(a.children)),a=u.pop(),s=d.pop()}function A(e,n,t){L(e,t),N(n),D()}function C(n){n.initializer&&te(n.initializer)?(L(n),e.forEachChild(n.initializer,N),D()):A(n,n.initializer)}function I(n){return!e.hasDynamicName(n)||219!==n.kind&&e.isPropertyAccessExpression(n.name.expression)&&e.isIdentifier(n.name.expression.expression)&&"Symbol"===e.idText(n.name.expression.expression)}function N(n){var t;if(r.throwIfCancellationRequested(),n&&!e.isToken(n))switch(n.kind){case 169:var i=n;A(i,i.body);for(var a=0,o=i.parameters;a<o.length;a++){var c=o[a];e.isParameterPropertyDeclaration(c,i)&&b(c)}break;case 167:case 170:case 171:case 166:I(n)&&A(n,n.body);break;case 165:I(n)&&C(n);break;case 164:I(n)&&b(n);break;case 265:var l=n;l.name&&b(l.name);var u=l.namedBindings;if(u)if(266===u.kind)b(u);else for(var d=0,p=u.elements;d<p.length;d++){var f=p[d];b(f)}break;case 292:A(n,n.name);break;case 293:var m=n.expression;e.isIdentifier(m)?b(n,m):b(n);break;case 201:case 291:case 252:var _=n;e.isBindingPattern(_.name)?N(_.name):C(_);break;case 254:var g=n.name;g&&e.isIdentifier(g)&&T(g.text),A(n,n.body);break;case 212:case 211:A(n,n.body);break;case 258:L(n);for(var y=0,h=n.members;y<h.length;y++){var v=h[y];Q(v)||b(v)}D();break;case 255:case 224:case 256:L(n);for(var E=0,k=n.members;E<k.length;E++){v=k[E];N(v)}D();break;case 259:A(n,Y(n).body);break;case 269:var R=n.expression;_=e.isObjectLiteralExpression(R)||e.isCallExpression(R)?R:e.isArrowFunction(R)||e.isFunctionExpression(R)?R.body:void 0;_?(L(n),N(_),D()):b(n);break;case 273:case 263:case 174:case 172:case 173:case 257:b(n);break;case 206:case 219:var O=e.getAssignmentDeclarationKind(n);switch(O){case 1:case 2:return void A(n,n.right);case 6:case 3:var w=n,P=w.left,M=3===O?P.expression:P,F=0,G=void 0;return e.isIdentifier(M.expression)?(T(M.expression.text),G=M.expression):(t=x(w,M.expression),F=t[0],G=t[1]),6===O?e.isObjectLiteralExpression(w.right)&&w.right.properties.length>0&&(L(w,G),e.forEachChild(w.right,N),D()):e.isFunctionExpression(w.right)||e.isArrowFunction(w.right)?A(n,w.right,G):(L(w,G),A(n,w.right,P.name),D()),void S(F);case 7:case 9:var B=n,U=(G=7===O?B.arguments[0]:B.arguments[0].expression,B.arguments[1]),V=x(n,G),j=(F=V[0],V[1]);return L(n,j),L(n,e.setTextRange(e.factory.createIdentifier(U.text),U)),N(n.arguments[2]),D(),D(),void S(F);case 5:w=n,P=w.left;var K=P.expression;if(e.isIdentifier(K)&&"prototype"!==e.getElementOrPropertyAccessName(P)&&s&&s.has(K.text))return void(e.isFunctionExpression(w.right)||e.isArrowFunction(w.right)?A(n,w.right,K):e.isBindableStaticAccessExpression(P)&&(L(w,K),A(w.left,w.right,e.getNameOrArgument(P)),D()));break;case 4:case 0:case 8:break;default:e.Debug.assertNever(O)}default:e.hasJSDocNodes(n)&&e.forEach(n.jsDoc,function(n){e.forEach(n.tags,function(n){e.isJSDocTypeAlias(n)&&b(n)})}),e.forEachChild(n,N)}}function k(n,t){var r=new e.Map;e.filterMutate(n,function(n,i){var a=n.name||e.getNameOfDeclaration(n.node),o=a&&g(a);if(!o)return!0;var s=r.get(o);if(!s)return r.set(o,n),!0;if(s instanceof Array){for(var c=0,l=s;c<l.length;c++){var u=l[c];if(w(u,n,i,t))return!1}return s.push(n),!0}u=s;return!w(u,n,i,t)&&(r.set(o,[u,n]),!0)})}n.getNavigationBarItems=f,n.getNavigationTree=m;var R=(t={},t[5]=!0,t[3]=!0,t[7]=!0,t[9]=!0,t[0]=!1,t[1]=!1,t[2]=!1,t[8]=!1,t[6]=!0,t[4]=!1,t);function O(n,t,r,i){function a(n){return e.isFunctionExpression(n)||e.isFunctionDeclaration(n)||e.isVariableDeclaration(n)}var s=e.isBinaryExpression(t.node)||e.isCallExpression(t.node)?e.getAssignmentDeclarationKind(t.node):0,c=e.isBinaryExpression(n.node)||e.isCallExpression(n.node)?e.getAssignmentDeclarationKind(n.node):0;if(R[s]&&R[c]||a(n.node)&&R[s]||a(t.node)&&R[c]||e.isClassDeclaration(n.node)&&M(n.node)&&R[s]||e.isClassDeclaration(t.node)&&R[c]||e.isClassDeclaration(n.node)&&M(n.node)&&a(t.node)||e.isClassDeclaration(t.node)&&a(n.node)&&M(n.node)){var l=n.additionalNodes&&e.lastOrUndefined(n.additionalNodes)||n.node;if(!e.isClassDeclaration(n.node)&&!e.isClassDeclaration(t.node)||a(n.node)||a(t.node)){var u=a(n.node)?n.node:a(t.node)?t.node:void 0;if(void 0!==u){var d=e.setTextRange(e.factory.createConstructorDeclaration(void 0,void 0,[],void 0),u),p=E(d);p.indent=n.indent+1,p.children=n.node===u?n.children:t.children,n.children=n.node===u?e.concatenate([p],t.children||[t]):e.concatenate(n.children||[o({},n)],[p])}else(n.children||t.children)&&(n.children=e.concatenate(n.children||[o({},n)],t.children||[t]),n.children&&(k(n.children,n),U(n.children)));l=n.node=e.setTextRange(e.factory.createClassDeclaration(void 0,void 0,n.name||e.factory.createIdentifier("__class__"),void 0,void 0,[]),n.node)}else n.children=e.concatenate(n.children,t.children),n.children&&k(n.children,n);var f=t.node;return i.children[r-1].node.end===l.end?e.setTextRange(l,{pos:l.pos,end:f.end}):(n.additionalNodes||(n.additionalNodes=[]),n.additionalNodes.push(e.setTextRange(e.factory.createClassDeclaration(void 0,void 0,n.name||e.factory.createIdentifier("__class__"),void 0,void 0,[]),t.node))),!0}return 0!==s}function w(e,n,t,r){return!!O(e,n,t,r)||!!P(e.node,n.node,r)&&(B(e,n),!0)}function P(n,t,r){if(n.kind!==t.kind||n.parent!==t.parent&&(!F(n,r)||!F(t,r)))return!1;switch(n.kind){case 165:case 167:case 170:case 171:return e.isStatic(n)===e.isStatic(t);case 259:return G(n,t)&&X(n)===X(t);default:return!0}}function M(e){return!!(8&e.flags)}function F(n,t){var r=e.isModuleBlock(n.parent)?n.parent.parent:n.parent;return r===t.node||e.contains(t.additionalNodes,r)}function G(e,n){return e.body.kind===n.body.kind&&(259!==e.body.kind||G(e.body,n.body))}function B(n,t){var r;n.additionalNodes=n.additionalNodes||[],n.additionalNodes.push(t.node),t.additionalNodes&&(r=n.additionalNodes).push.apply(r,t.additionalNodes),n.children=e.concatenate(n.children,t.children),n.children&&(k(n.children,n),U(n.children))}function U(e){e.sort(V)}function V(n,t){return e.compareStringsCaseSensitiveUI(j(n.node),j(t.node))||e.compareValues(y(n),y(t))}function j(n){if(259===n.kind)return q(n);var t=e.getNameOfDeclaration(n);if(t&&e.isPropertyName(t)){var r=e.getPropertyNameForPropertyNameNode(t);return r&&e.unescapeLeadingUnderscores(r)}switch(n.kind){case 211:case 212:case 224:return ee(n);default:return}}function K(n,t){if(259===n.kind)return re(q(n));if(t){var r=e.isIdentifier(t)?t.text:e.isElementAccessExpression(t)?"["+g(t.argumentExpression)+"]":g(t);if(r.length>0)return re(r)}switch(n.kind){case 300:var i=n;return e.isExternalModule(i)?'"'+e.escapeString(e.getBaseFileName(e.removeFileExtension(e.normalizePath(i.fileName))))+'"':"<global>";case 269:return e.isExportAssignment(n)&&n.isExportEquals?"export=":"default";case 212:case 254:case 211:case 255:case 224:return 512&e.getSyntacticModifierFlags(n)?"default":ee(n);case 169:return"constructor";case 173:return"new()";case 172:return"()";case 174:return"[]";default:return"<unknown>"}}function H(e){var n=[];function t(e){if(r(e)&&(n.push(e),e.children))for(var i=0,a=e.children;i<a.length;i++){var o=a[i];t(o)}}return t(e),n;function r(e){if(e.children)return!0;switch(y(e)){case 255:case 224:case 258:case 256:case 259:case 300:case 257:case 340:case 333:return!0;case 212:case 254:case 211:return n(e);default:return!1}function n(e){if(!e.node.body)return!1;switch(y(e.parent)){case 260:case 300:case 167:case 169:return!0;default:return!1}}}}function W(n){return{text:K(n.node,n.name),kind:e.getNodeKind(n.node),kindModifiers:$(n.node),spans:J(n),nameSpan:n.name&&Z(n.name),childItems:e.map(n.children,W)}}function z(n){return{text:K(n.node,n.name),kind:e.getNodeKind(n.node),kindModifiers:$(n.node),spans:J(n),childItems:e.map(n.children,t)||p,indent:n.indent,bolded:!1,grayed:!1};function t(n){return{text:K(n.node,n.name),kind:e.getNodeKind(n.node),kindModifiers:e.getNodeModifiers(n.node),spans:J(n),childItems:p,indent:0,bolded:!1,grayed:!1}}}function J(e){var n=[Z(e.node)];if(e.additionalNodes)for(var t=0,r=e.additionalNodes;t<r.length;t++){var i=r[t];n.push(Z(i))}return n}function q(n){return e.isAmbientModule(n)?e.getTextOfNode(n.name):X(n)}function X(n){var t=[e.getTextOfIdentifierOrLiteral(n.name)];while(n.body&&259===n.body.kind)n=n.body,t.push(e.getTextOfIdentifierOrLiteral(n.name));return t.join(".")}function Y(n){return n.body&&e.isModuleDeclaration(n.body)?Y(n.body):n}function Q(e){return!e.name||160===e.name.kind}function Z(n){return 300===n.kind?e.createTextSpanFromRange(n):e.createTextSpanFromNode(n,i)}function $(n){return n.parent&&252===n.parent.kind&&(n=n.parent),e.getNodeModifiers(n)}function ee(n){var t=n.parent;if(n.name&&e.getFullWidth(n.name)>0)return re(e.declarationNameToString(n.name));if(e.isVariableDeclaration(t))return re(e.declarationNameToString(t.name));if(e.isBinaryExpression(t)&&63===t.operatorToken.kind)return g(t.left).replace(c,"");if(e.isPropertyAssignment(t))return g(t.name);if(512&e.getSyntacticModifierFlags(n))return"default";if(e.isClassLike(n))return"<class>";if(e.isCallExpression(t)){var r=ne(t.expression);if(void 0!==r){if(r=re(r),r.length>l)return r+" callback";var a=re(e.mapDefined(t.arguments,function(n){return e.isStringLiteralLike(n)?n.getText(i):void 0}).join(", "));return r+"("+a+") callback"}}return"<function>"}function ne(n){if(e.isIdentifier(n))return n.text;if(e.isPropertyAccessExpression(n)){var t=ne(n.expression),r=n.name.text;return void 0===t?r:t+"."+r}}function te(e){switch(e.kind){case 212:case 211:case 224:return!0;default:return!1}}function re(e){return e=e.length>l?e.substring(0,l)+"...":e,e.replace(/\\?(\r?\n|\r|\u2028|\u2029)/g,"")}})(e.NavigationBar||(e.NavigationBar={}))}(d||(d={})),function(e){(function(n){function t(n,t,i,s,l,u){var d=e.textChanges.ChangeTracker.fromContext({host:i,formatContext:t,preferences:l}),f=function(t){return e.stableSort(o(r(t,n,s,u)),function(e,n){return v(e,n)})},m=n.statements.filter(e.isImportDeclaration);T(m,f);var _=n.statements.filter(e.isExportDeclaration);T(_,c);for(var g=0,y=n.statements.filter(e.isAmbientModule);g<y.length;g++){var h=y[g];if(h.body){var b=h.body.statements.filter(e.isImportDeclaration);T(b,f);var E=h.body.statements.filter(e.isExportDeclaration);T(E,c)}}return d.getChanges();function T(r,o){if(0!==e.length(r)){e.suppressLeadingTrivia(r[0]);var s=e.group(r,function(e){return a(e.moduleSpecifier)}),c=e.stableSort(s,function(e,n){return p(e[0].moduleSpecifier,n[0].moduleSpecifier)}),l=e.flatMap(c,function(e){return a(e[0].moduleSpecifier)?o(e):e});if(0===l.length)d.deleteNodes(n,r,{trailingTriviaOption:e.textChanges.TrailingTriviaOption.Include},!0);else{var u={leadingTriviaOption:e.textChanges.LeadingTriviaOption.Exclude,trailingTriviaOption:e.textChanges.TrailingTriviaOption.Include,suffix:e.getNewLineOrDefaultFromHost(i,t.options)};d.replaceNodeWithNodes(n,r[0],l,u);var f=d.nodeHasTrailingComment(n,r[0],u);d.deleteNodes(n,r.slice(1),{trailingTriviaOption:e.textChanges.TrailingTriviaOption.Include},f)}}}}function r(n,t,r,a){if(a)return n;for(var o=r.getTypeChecker(),s=o.getJsxNamespace(t),c=o.getJsxFragmentFactory(t),u=!!(2&t.transformFlags),d=[],p=0,f=n;p<f.length;p++){var m=f[p],_=m.importClause,g=m.moduleSpecifier;if(_){var y=_.name,h=_.namedBindings;if(y&&!b(y)&&(y=void 0),h)if(e.isNamespaceImport(h))b(h.name)||(h=void 0);else{var v=h.elements.filter(function(e){return b(e.name)});v.length<h.elements.length&&(h=v.length?e.factory.updateNamedImports(h,v):void 0)}y||h?d.push(l(m,y,h)):i(t,g)&&(t.isDeclarationFile?d.push(e.factory.createImportDeclaration(m.decorators,m.modifiers,void 0,g)):d.push(m))}else d.push(m)}return d;function b(n){return u&&(n.text===s||c&&n.text===c)||e.FindAllReferences.Core.isSymbolReferencedInFile(n,o,t)}}function i(n,t){var r=e.isStringLiteral(t)&&t.text;return e.isString(r)&&e.some(n.moduleAugmentations,function(n){return e.isStringLiteral(n)&&n.text===r})}function a(n){return void 0!==n&&e.isStringLiteralLike(n)?n.text:void 0}function o(n){var t;if(0===n.length)return n;var r=s(n),i=r.importWithoutClause,a=r.typeOnlyImports,o=r.regularImports,c=[];i&&c.push(i);for(var d=0,p=[o,a];d<p.length;d++){var m=p[d],_=m===a,g=m.defaultImports,y=m.namespaceImports,h=m.namedImports;if(_||1!==g.length||1!==y.length||0!==h.length){for(var v=e.stableSort(y,function(e,n){return f(e.importClause.namedBindings.name,n.importClause.namedBindings.name)}),b=0,E=v;b<E.length;b++){var T=E[b];c.push(l(T,void 0,T.importClause.namedBindings))}if(0!==g.length||0!==h.length){var S=void 0,x=[];if(1===g.length)S=g[0].importClause.name;else for(var L=0,D=g;L<D.length;L++){N=D[L];x.push(e.factory.createImportSpecifier(e.factory.createIdentifier("default"),N.importClause.name))}x.push.apply(x,e.flatMap(h,function(e){return e.importClause.namedBindings.elements}));var A=u(x),C=g.length>0?g[0]:h[0],I=0===A.length?S?void 0:e.factory.createNamedImports(e.emptyArray):0===h.length?e.factory.createNamedImports(A):e.factory.updateNamedImports(h[0].importClause.namedBindings,A);_&&S&&I?(c.push(l(C,S,void 0)),c.push(l(null!==(t=h[0])&&void 0!==t?t:C,void 0,I))):c.push(l(C,S,I))}}else{var N=g[0];c.push(l(N,N.importClause.name,y[0].importClause.namedBindings))}}return c}function s(n){for(var t,r={defaultImports:[],namespaceImports:[],namedImports:[]},i={defaultImports:[],namespaceImports:[],namedImports:[]},a=0,o=n;a<o.length;a++){var s=o[a];if(void 0!==s.importClause){var c=s.importClause.isTypeOnly?r:i,l=s.importClause,u=l.name,d=l.namedBindings;u&&c.defaultImports.push(s),d&&(e.isNamespaceImport(d)?c.namespaceImports.push(s):c.namedImports.push(s))}else t=t||s}return{importWithoutClause:t,typeOnlyImports:r,regularImports:i}}function c(n){if(0===n.length)return n;var t=m(n),r=t.exportWithoutClause,i=t.namedExports,a=t.typeOnlyExports,o=[];r&&o.push(r);for(var s=0,c=[i,a];s<c.length;s++){var l=c[s];if(0!==l.length){var d=[];d.push.apply(d,e.flatMap(l,function(n){return n.exportClause&&e.isNamedExports(n.exportClause)?n.exportClause.elements:e.emptyArray}));var p=u(d),f=l[0];o.push(e.factory.updateExportDeclaration(f,f.decorators,f.modifiers,f.isTypeOnly,f.exportClause&&(e.isNamedExports(f.exportClause)?e.factory.updateNamedExports(f.exportClause,p):e.factory.updateNamespaceExport(f.exportClause,f.exportClause.name)),f.moduleSpecifier))}}return o;function m(e){for(var n,t=[],r=[],i=0,a=e;i<a.length;i++){var o=a[i];void 0===o.exportClause?n=n||o:o.isTypeOnly?r.push(o):t.push(o)}return{exportWithoutClause:n,namedExports:t,typeOnlyExports:r}}}function l(n,t,r){return e.factory.updateImportDeclaration(n,n.decorators,n.modifiers,e.factory.updateImportClause(n.importClause,n.importClause.isTypeOnly,t,r),n.moduleSpecifier)}function u(n){return e.stableSort(n,d)}function d(e,n){return f(e.propertyName||e.name,n.propertyName||n.name)||f(e.name,n.name)}function p(n,t){var r=void 0===n?void 0:a(n),i=void 0===t?void 0:a(t);return e.compareBooleans(void 0===r,void 0===i)||e.compareBooleans(e.isExternalModuleNameRelative(r),e.isExternalModuleNameRelative(i))||e.compareStringsCaseInsensitive(r,i)}function f(n,t){return e.compareStringsCaseInsensitive(n.text,t.text)}function m(n){var t;switch(n.kind){case 263:return null===(t=e.tryCast(n.moduleReference,e.isExternalModuleReference))||void 0===t?void 0:t.expression;case 264:return n.moduleSpecifier;case 235:return n.declarationList.declarations[0].initializer.arguments[0]}}function _(n){return e.arrayIsSorted(n,v)}function g(n){return e.arrayIsSorted(n,d)}function y(n,t){var r=e.binarySearch(n,t,e.identity,v);return r<0?~r:r}function h(n,t){var r=e.binarySearch(n,t,e.identity,d);return r<0?~r:r}function v(e,n){return p(m(e),m(n))||b(e,n)}function b(n,t){return e.compareValues(E(n),E(t))}function E(e){var n;switch(e.kind){case 264:return e.importClause?e.importClause.isTypeOnly?1:266===(null===(n=e.importClause.namedBindings)||void 0===n?void 0:n.kind)?2:e.importClause.name?3:4:0;case 263:return 5;case 235:return 6}}n.organizeImports=t,n.coalesceImports=o,n.coalesceExports=c,n.compareImportOrExportSpecifiers=d,n.compareModuleSpecifiers=p,n.importsAreSorted=_,n.importSpecifiersAreSorted=g,n.getImportDeclarationInsertionIndex=y,n.getImportSpecifierInsertionIndex=h,n.compareImportsOrRequireStatements=v})(e.OrganizeImports||(e.OrganizeImports={}))}(d||(d={})),function(e){(function(n){function t(e,n){var t=[];return r(e,n,t),i(e,t),t.sort(function(e,n){return e.textSpan.start-n.textSpan.start})}function r(n,t,r){var i=40,o=0,s=a(a([],n.statements,!0),[n.endOfFileToken],!1),p=s.length;while(o<p){while(o<p&&!e.isAnyImportSyntax(s[o]))_(s[o]),o++;if(o===p)break;var f=o;while(o<p&&e.isAnyImportSyntax(s[o]))l(s[o],n,t,r),o++;var m=o-1;m!==f&&r.push(u(e.findChildOfKind(s[f],100,n).getStart(n),s[m].getEnd(),"imports"))}function _(a){var o;if(0!==i){t.throwIfCancellationRequested(),(e.isDeclaration(a)||e.isVariableStatement(a)||e.isReturnStatement(a)||1===a.kind)&&l(a,n,t,r),e.isFunctionLike(a)&&e.isBinaryExpression(a.parent)&&e.isPropertyAccessExpression(a.parent.left)&&l(a.parent.left,n,t,r),(e.isBlock(a)||e.isModuleBlock(a))&&c(a.statements.end,n,t,r),(e.isClassLike(a)||e.isInterfaceDeclaration(a))&&c(a.members.end,n,t,r);var s=d(a,n);s&&r.push(s),i--,e.isCallExpression(a)?(i++,_(a.expression),i--,a.arguments.forEach(_),null===(o=a.typeArguments)||void 0===o||o.forEach(_)):e.isIfStatement(a)&&a.elseStatement&&e.isIfStatement(a.elseStatement)?(_(a.expression),_(a.thenStatement),i++,_(a.elseStatement),i--):a.forEachChild(_),i++}}}function i(n,t){for(var r=[],i=n.getLineStarts(),a=0,o=i;a<o.length;a++){var c=o[a],l=n.getLineEndOfPosition(c),u=n.text.substring(c,l),d=s(u);if(d&&!e.isInComment(n,c))if(d[1]){var p=r.pop();p&&(p.textSpan.length=l-p.textSpan.start,p.hintSpan.length=l-p.textSpan.start,t.push(p))}else{var f=e.createTextSpanFromBounds(n.text.indexOf("//",c),l);r.push(m(f,"region",f,!1,d[2]||"#region"))}}}n.collectElements=t;var o=/^#(end)?region(?:\s+(.*))?(?:\r)?$/;function s(n){return n=e.trimStringStart(n),e.startsWith(n,"//")?(n=e.trimString(n.slice(2)),o.exec(n)):null}function c(n,t,r,i){var a=e.getLeadingCommentRanges(t.text,n);if(a){for(var o=-1,c=-1,l=0,d=t.getFullText(),p=0,f=a;p<f.length;p++){var m=f[p],_=m.kind,g=m.pos,y=m.end;switch(r.throwIfCancellationRequested(),_){case 2:var h=d.slice(g,y);if(s(h)){v(),l=0;break}0===l&&(o=g),c=y,l++;break;case 3:v(),i.push(u(g,y,"comment")),l=0;break;default:e.Debug.assertNever(_)}}v()}function v(){l>1&&i.push(u(o,c,"comment"))}}function l(n,t,r,i){e.isJsxText(n)||c(n.pos,t,r,i)}function u(n,t,r){return m(e.createTextSpanFromBounds(n,t),r)}function d(n,t){switch(n.kind){case 233:if(e.isFunctionLike(n.parent))return p(n.parent,n,t);switch(n.parent.kind){case 238:case 241:case 242:case 240:case 237:case 239:case 246:case 290:return g(n.parent);case 250:var r=n.parent;if(r.tryBlock===n)return g(n.parent);if(r.finallyBlock===n){var i=e.findChildOfKind(r,96,t);if(i)return g(i)}default:return m(e.createTextSpanFromNode(n,t),"code")}case 260:return g(n.parent);case 255:case 224:case 256:case 258:case 261:case 180:case 199:return g(n);case 182:return g(n,!1,!e.isTupleTypeNode(n.parent),22);case 287:case 288:return y(n.statements);case 203:return _(n);case 202:return _(n,22);case 276:return s(n);case 280:return c(n);case 277:case 278:return l(n.attributes);case 221:case 14:return d(n);case 200:return g(n,!1,!e.isBindingElement(n.parent),22);case 212:return o(n);case 206:return a(n)}function a(n){if(n.arguments.length){var r=e.findChildOfKind(n,20,t),i=e.findChildOfKind(n,21,t);if(r&&i&&!e.positionsAreOnSameLine(r.pos,i.pos,t))return f(r,i,n,t,!1,!0)}}function o(n){if(!e.isBlock(n.body)&&!e.positionsAreOnSameLine(n.body.getFullStart(),n.body.getEnd(),t)){var r=e.createTextSpanFromBounds(n.body.getFullStart(),n.body.getEnd());return m(r,"code",e.createTextSpanFromNode(n))}}function s(n){var r=e.createTextSpanFromBounds(n.openingElement.getStart(t),n.closingElement.getEnd()),i=n.openingElement.tagName.getText(t),a="<"+i+">...</"+i+">";return m(r,"code",r,!1,a)}function c(n){var r=e.createTextSpanFromBounds(n.openingFragment.getStart(t),n.closingFragment.getEnd()),i="<>...</>";return m(r,"code",r,!1,i)}function l(e){if(0!==e.properties.length)return u(e.getStart(t),e.getEnd(),"code")}function d(e){if(14!==e.kind||0!==e.text.length)return u(e.getStart(t),e.getEnd(),"code")}function _(n,t){return void 0===t&&(t=18),g(n,!1,!e.isArrayLiteralExpression(n.parent)&&!e.isCallExpression(n.parent),t)}function g(r,i,a,o,s){void 0===i&&(i=!1),void 0===a&&(a=!0),void 0===o&&(o=18),void 0===s&&(s=18===o?19:23);var c=e.findChildOfKind(n,o,t),l=e.findChildOfKind(n,s,t);return c&&l&&f(c,l,r,t,i,a)}function y(n){return n.length?m(e.createTextSpanFromRange(n),"code"):void 0}}function p(n,t,r){var i=_(n,t,r),a=e.findChildOfKind(t,19,r);return i&&a&&f(i,a,n,r,212!==n.kind)}function f(n,t,r,i,a,o){void 0===a&&(a=!1),void 0===o&&(o=!0);var s=e.createTextSpanFromBounds(o?n.getFullStart():n.getStart(i),t.getEnd());return m(s,"code",e.createTextSpanFromNode(r,i),a)}function m(e,n,t,r,i){return void 0===t&&(t=e),void 0===r&&(r=!1),void 0===i&&(i="..."),{textSpan:e,kind:n,hintSpan:t,bannerText:i,autoCollapse:r}}function _(n,t,r){if(e.isNodeArrayMultiLine(n.parameters,r)){var i=e.findChildOfKind(n,20,r);if(i)return i}return e.findChildOfKind(t,18,r)}})(e.OutliningElementsCollector||(e.OutliningElementsCollector={}))}(d||(d={})),function(e){var n;function t(e,n){return{kind:e,isCaseSensitive:n}}function r(n){var t=new e.Map,r=n.trim().split(".").map(function(e){return f(e.trim())});if(!r.some(function(e){return!e.subWordTextChunks.length}))return{getFullMatch:function(e,n){return i(e,n,r,t)},getMatchForLastSegmentOfPattern:function(n){return s(n,e.last(r),t)},patternContainsDots:r.length>1}}function i(n,t,r,i){var a=s(t,e.last(r),i);if(a&&!(r.length-1>n.length)){for(var o,l=r.length-2,u=n.length-1;l>=0;l-=1,u-=1)o=c(o,s(n[u],r[l],i));return o}}function a(e,n){var t=n.get(e);return t||n.set(e,t=S(e)),t}function o(r,i,o){var s=g(r,i.textLowerCase);if(0===s)return t(i.text.length===r.length?n.exact:n.prefix,e.startsWith(r,i.text));if(i.isLowerCase){if(-1===s)return;for(var c=a(r,o),l=0,d=c;l<d.length;l++){var f=d[l];if(u(r,f,i.text,!0))return t(n.substring,u(r,f,i.text,!1))}if(i.text.length<r.length&&m(r.charCodeAt(s)))return t(n.substring,!1)}else{if(r.indexOf(i.text)>0)return t(n.substring,!0);if(i.characterSpans.length>0){var _=a(r,o),y=!!p(r,_,i,!1)||!p(r,_,i,!0)&&void 0;if(void 0!==y)return t(n.camelCase,y)}}}function s(e,n,t){if(N(n.totalTextChunk.text,function(e){return 32!==e&&42!==e})){var r=o(e,n.totalTextChunk,t);if(r)return r}for(var i,a=n.subWordTextChunks,s=0,l=a;s<l.length;s++){var u=l[s];i=c(i,o(e,u,t))}return i}function c(n,t){return e.min(n,t,l)}function l(n,t){return void 0===n?1:void 0===t?-1:e.compareValues(n.kind,t.kind)||e.compareBooleans(!n.isCaseSensitive,!t.isCaseSensitive)}function u(e,n,t,r,i){return void 0===i&&(i={start:0,length:t.length}),i.length<=n.length&&I(0,i.length,function(a){return d(t.charCodeAt(i.start+a),e.charCodeAt(n.start+a),r)})}function d(e,n,t){return t?y(e)===y(n):e===n}function p(n,t,r,i){var a,o,s=r.characterSpans,c=0,l=0;while(1){if(l===s.length)return!0;if(c===t.length)return!1;for(var d=t[c],p=!1;l<s.length;l++){var f=s[l];if(p&&(!m(r.text.charCodeAt(s[l-1].start))||!m(r.text.charCodeAt(s[l].start))))break;if(!u(n,d,r.text,i,f))break;p=!0,a=void 0===a?c:a,o=void 0===o||o,d=e.createTextSpan(d.start+f.length,d.length-f.length)}p||void 0===o||(o=!1),c++}}function f(e){return{totalTextChunk:E(e),subWordTextChunks:b(e)}}function m(n){if(n>=65&&n<=90)return!0;if(n<127||!e.isUnicodeIdentifierStart(n,99))return!1;var t=String.fromCharCode(n);return t===t.toUpperCase()}function _(n){if(n>=97&&n<=122)return!0;if(n<127||!e.isUnicodeIdentifierStart(n,99))return!1;var t=String.fromCharCode(n);return t===t.toLowerCase()}function g(e,n){for(var t=e.length-n.length,r=function(t){if(N(n,function(n,r){return y(e.charCodeAt(r+t))===n}))return{value:t}},i=0;i<=t;i++){var a=r(i);if("object"===typeof a)return a.value}return-1}function y(e){return e>=65&&e<=90?e-65+97:e<127?e:String.fromCharCode(e).toLowerCase().charCodeAt(0)}function h(e){return e>=48&&e<=57}function v(e){return m(e)||_(e)||h(e)||95===e||36===e}function b(e){for(var n=[],t=0,r=0,i=0;i<e.length;i++){var a=e.charCodeAt(i);v(a)?(0===r&&(t=i),r++):r>0&&(n.push(E(e.substr(t,r))),r=0)}return r>0&&n.push(E(e.substr(t,r))),n}function E(e){var n=e.toLowerCase();return{text:e,textLowerCase:n,isLowerCase:e===n,characterSpans:T(e)}}function T(e){return x(e,!1)}function S(e){return x(e,!0)}function x(n,t){for(var r=[],i=0,a=1;a<n.length;a++){var o=h(n.charCodeAt(a-1)),s=h(n.charCodeAt(a)),c=C(n,t,a),l=t&&A(n,a,i);(L(n.charCodeAt(a-1))||L(n.charCodeAt(a))||o!==s||c||l)&&(D(n,i,a)||r.push(e.createTextSpan(i,a-i)),i=a)}return D(n,i,n.length)||r.push(e.createTextSpan(i,n.length-i)),r}function L(e){switch(e){case 33:case 34:case 35:case 37:case 38:case 39:case 40:case 41:case 42:case 44:case 45:case 46:case 47:case 58:case 59:case 63:case 64:case 91:case 92:case 93:case 95:case 123:case 125:return!0}return!1}function D(e,n,t){return N(e,function(e){return L(e)&&95!==e},n,t)}function A(e,n,t){return n!==t&&n+1<e.length&&m(e.charCodeAt(n))&&_(e.charCodeAt(n+1))&&N(e,m,t,n)}function C(e,n,t){var r=m(e.charCodeAt(t-1)),i=m(e.charCodeAt(t));return i&&(!n||!r)}function I(e,n,t){for(var r=e;r<n;r++)if(!t(r))return!1;return!0}function N(e,n,t,r){return void 0===t&&(t=0),void 0===r&&(r=e.length),I(t,r,function(t){return n(e.charCodeAt(t),t)})}(function(e){e[e["exact"]=0]="exact",e[e["prefix"]=1]="prefix",e[e["substring"]=2]="substring",e[e["camelCase"]=3]="camelCase"})(n=e.PatternMatchKind||(e.PatternMatchKind={})),e.createPatternMatcher=r,e.breakIntoCharacterSpans=T,e.breakIntoWordSpans=S}(d||(d={})),function(e){function n(n,t,r){void 0===t&&(t=!0),void 0===r&&(r=!1);var i,a,o,s={languageVersion:1,pragmas:void 0,checkJsDirective:void 0,referencedFiles:[],typeReferenceDirectives:[],libReferenceDirectives:[],amdDependencies:[],hasNoDefaultLib:void 0,moduleName:void 0},c=[],l=0,u=!1;function d(){return a=o,o=e.scanner.scan(),18===o?l++:19===o&&l--,o}function p(){var n=e.scanner.getTokenValue(),t=e.scanner.getTokenPos();return{fileName:n,pos:t,end:t+n.length}}function f(){i||(i=[]),i.push({ref:p(),depth:l})}function m(){c.push(p()),_()}function _(){0===l&&(u=!0)}function g(){var n=e.scanner.getToken();return 134===n&&(n=d(),140===n&&(n=d(),10===n&&f()),!0)}function y(){if(24===a)return!1;var n=e.scanner.getToken();if(100===n){if(n=d(),20===n){if(n=d(),10===n||14===n)return m(),!0}else{if(10===n)return m(),!0;if(150===n){var t=e.scanner.lookAhead(function(){var n=e.scanner.scan();return 154!==n&&(41===n||18===n||79===n||e.isKeyword(n))});t&&(n=d())}if(79===n||e.isKeyword(n))if(n=d(),154===n){if(n=d(),10===n)return m(),!0}else if(63===n){if(v(!0))return!0}else{if(27!==n)return!0;n=d()}if(18===n){n=d();while(19!==n&&1!==n)n=d();19===n&&(n=d(),154===n&&(n=d(),10===n&&m()))}else 41===n&&(n=d(),127===n&&(n=d(),(79===n||e.isKeyword(n))&&(n=d(),154===n&&(n=d(),10===n&&m()))))}return!0}return!1}function h(){var n=e.scanner.getToken();if(93===n){if(_(),n=d(),150===n){var t=e.scanner.lookAhead(function(){var n=e.scanner.scan();return 41===n||18===n});t&&(n=d())}if(18===n){n=d();while(19!==n&&1!==n)n=d();19===n&&(n=d(),154===n&&(n=d(),10===n&&m()))}else if(41===n)n=d(),154===n&&(n=d(),10===n&&m());else if(100===n){if(n=d(),150===n){t=e.scanner.lookAhead(function(){var n=e.scanner.scan();return 79===n||e.isKeyword(n)});t&&(n=d())}if((79===n||e.isKeyword(n))&&(n=d(),63===n&&v(!0)))return!0}return!0}return!1}function v(n,t){void 0===t&&(t=!1);var r=n?d():e.scanner.getToken();return 144===r&&(r=d(),20===r&&(r=d(),(10===r||t&&14===r)&&m()),!0)}function b(){var n=e.scanner.getToken();if(79===n&&"define"===e.scanner.getTokenValue()){if(n=d(),20!==n)return!0;if(n=d(),10===n||14===n){if(n=d(),27!==n)return!0;n=d()}if(22!==n)return!0;n=d();while(23!==n&&1!==n)10!==n&&14!==n||m(),n=d();return!0}return!1}function E(){e.scanner.setText(n),d();while(1){if(1===e.scanner.getToken())break;g()||y()||h()||r&&(v(!1,!0)||b())||d()}e.scanner.setText(void 0)}if(t&&E(),e.processCommentPragmas(s,n),e.processPragmasIntoFields(s,e.noop),u){if(i)for(var T=0,S=i;T<S.length;T++){var x=S[T];c.push(x.ref)}return{referencedFiles:s.referencedFiles,typeReferenceDirectives:s.typeReferenceDirectives,libReferenceDirectives:s.libReferenceDirectives,importedFiles:c,isLibFile:!!s.hasNoDefaultLib,ambientExternalModules:void 0}}var L=void 0;if(i)for(var D=0,A=i;D<A.length;D++){x=A[D];0===x.depth?(L||(L=[]),L.push(x.ref.fileName)):c.push(x.ref)}return{referencedFiles:s.referencedFiles,typeReferenceDirectives:s.typeReferenceDirectives,libReferenceDirectives:s.libReferenceDirectives,importedFiles:c,isLibFile:!!s.hasNoDefaultLib,ambientExternalModules:L}}e.preProcessFile=n}(d||(d={})),function(e){(function(n){function t(n,t,i,a){var o=e.getAdjustedRenameLocation(e.getTouchingPropertyName(t,i));if(l(o)){var c=r(o,n.getTypeChecker(),t,n,a);if(c)return c}return s(e.Diagnostics.You_cannot_rename_this_element)}function r(n,t,r,c,l){var u=t.getSymbolAtLocation(n);if(u){var d=u.declarations;if(d&&0!==d.length){if(d.some(function(e){return i(c,e)}))return s(e.Diagnostics.You_cannot_rename_elements_that_are_defined_in_the_standard_TypeScript_library);if(!(e.isIdentifier(n)&&88===n.originalKeywordKind&&u.parent&&1536&u.parent.flags)){if(e.isStringLiteralLike(n)&&e.tryGetImportFromModuleSpecifier(n))return l&&l.allowRenameOfImportPath?a(n,r,u):void 0;var p=e.SymbolDisplay.getSymbolKind(t,u,n),f=e.isImportOrExportSpecifierName(n)||e.isStringOrNumericLiteralLike(n)&&160===n.parent.kind?e.stripQuotes(e.getTextOfIdentifierOrLiteral(n)):void 0,m=f||t.symbolToString(u),_=f||t.getFullyQualifiedName(u);return o(m,_,p,e.SymbolDisplay.getSymbolModifiers(t,u),n,r)}}}else if(e.isStringLiteralLike(n)){var g=e.getContextualTypeOrAncestorTypeNodeType(n,t);if(g&&(128&g.flags||1048576&g.flags&&e.every(g.types,function(e){return!!(128&e.flags)})))return o(n.text,n.text,"string","",n,r)}else if(e.isLabelName(n)){var y=e.getTextOfNode(n);return o(y,y,"label","",n,r)}}function i(n,t){var r=t.getSourceFile();return n.isSourceFileDefaultLibrary(r)&&e.fileExtensionIs(r.fileName,".d.ts")}function a(n,t,r){if(!e.isExternalModuleNameRelative(n.text))return s(e.Diagnostics.You_cannot_rename_a_module_via_a_global_import);var i=r.declarations&&e.find(r.declarations,e.isSourceFile);if(i){var a=e.endsWith(n.text,"/index")||e.endsWith(n.text,"/index.js")?void 0:e.tryRemoveSuffix(e.removeFileExtension(i.fileName),"/index"),o=void 0===a?i.fileName:a,c=void 0===a?"module":"directory",l=n.text.lastIndexOf("/")+1,u=e.createTextSpan(n.getStart(t)+1+l,n.text.length-l);return{canRename:!0,fileToRename:o,kind:c,displayName:o,fullDisplayName:o,kindModifiers:"",triggerSpan:u}}}function o(e,n,t,r,i,a){return{canRename:!0,fileToRename:void 0,kind:t,displayName:e,fullDisplayName:n,kindModifiers:r,triggerSpan:c(i,a)}}function s(n){return{canRename:!1,localizedErrorMessage:e.getLocaleSpecificMessage(n)}}function c(n,t){var r=n.getStart(t),i=n.getWidth(t);return e.isStringLiteralLike(n)&&(r+=1,i-=2),e.createTextSpan(r,i)}function l(n){switch(n.kind){case 79:case 80:case 10:case 14:case 108:return!0;case 8:return e.isLiteralNameOfPropertyDeclarationOrIndexAccess(n);default:return!1}}n.getRenameInfo=t})(e.Rename||(e.Rename={}))}(d||(d={})),function(e){(function(n){function t(n,t){var i,s={textSpan:e.createTextSpanFromBounds(t.getFullStart(),t.getEnd())},c=t;e:while(1){var l=a(c);if(!l.length)break;for(var f=0;f<l.length;f++){var m=l[f-1],_=l[f],g=l[f+1];if(e.getTokenPosOfNode(_,t,!0)>n)break e;var y=e.singleOrUndefined(e.getTrailingCommentRanges(t.text,_.end));if(y&&2===y.kind&&x(y.pos,y.end),r(t,n,_)){if(e.isBlock(_)||e.isTemplateSpan(_)||e.isTemplateHead(_)||e.isTemplateTail(_)||m&&e.isTemplateHead(m)||e.isVariableDeclarationList(_)&&e.isVariableStatement(c)||e.isSyntaxList(_)&&e.isVariableDeclarationList(c)||e.isVariableDeclaration(_)&&e.isSyntaxList(c)&&1===l.length||e.isJSDocTypeExpression(_)||e.isJSDocSignature(_)||e.isJSDocTypeLiteral(_)){c=_;break}if(e.isTemplateSpan(c)&&g&&e.isTemplateMiddleOrTemplateTail(g)){var h=_.getFullStart()-"${".length,v=g.getStart()+"}".length;S(h,v)}var b=e.isSyntaxList(_)&&u(m)&&d(g)&&!e.positionsAreOnSameLine(m.getStart(),g.getStart(),t),E=b?m.getEnd():_.getStart(),T=b?g.getStart():p(t,_);e.hasJSDocNodes(_)&&(null===(i=_.jsDoc)||void 0===i?void 0:i.length)&&S(e.first(_.jsDoc).getStart(),T),S(E,T),(e.isStringLiteral(_)||e.isTemplateLiteral(_))&&S(E+1,T-1),c=_;break}if(f===l.length-1)break e}}return s;function S(t,r){if(t!==r){var i=e.createTextSpanFromBounds(t,r);(!s||!e.textSpansEqual(i,s.textSpan)&&e.textSpanIntersectsWithPosition(i,n))&&(s=o({textSpan:i},s&&{parent:s}))}}function x(e,n){S(e,n);var r=e;while(47===t.text.charCodeAt(r))r++;S(r,n)}}function r(n,t,r){if(e.Debug.assert(r.pos<=t),t<r.end)return!0;var i=r.getEnd();return i===t&&e.getTouchingPropertyName(n,t).pos<r.end}n.getSmartSelectionRange=t;var i=e.or(e.isImportDeclaration,e.isImportEqualsDeclaration);function a(n){if(e.isSourceFile(n))return s(n.getChildAt(0).getChildren(),i);if(e.isMappedTypeNode(n)){var t=n.getChildren(),r=t[0],a=t.slice(1),o=e.Debug.checkDefined(a.pop());e.Debug.assertEqual(r.kind,18),e.Debug.assertEqual(o.kind,19);var u=s(a,function(e){return e===n.readonlyToken||143===e.kind||e===n.questionToken||57===e.kind}),d=s(u,function(e){var n=e.kind;return 22===n||161===n||23===n});return[r,l(c(d,function(e){var n=e.kind;return 58===n})),o]}if(e.isPropertySignature(n)){a=s(n.getChildren(),function(t){return t===n.name||e.contains(n.modifiers,t)});return c(a,function(e){var n=e.kind;return 58===n})}if(e.isParameter(n)){var p=s(n.getChildren(),function(e){return e===n.dotDotDotToken||e===n.name}),f=s(p,function(e){return e===p[0]||e===n.questionToken});return c(f,function(e){var n=e.kind;return 63===n})}return e.isBindingElement(n)?c(n.getChildren(),function(e){var n=e.kind;return 63===n}):n.getChildren()}function s(e,n){for(var t,r=[],i=0,a=e;i<a.length;i++){var o=a[i];n(o)?(t=t||[],t.push(o)):(t&&(r.push(l(t)),t=void 0),r.push(o))}return t&&r.push(l(t)),r}function c(n,t,r){if(void 0===r&&(r=!0),n.length<2)return n;var i=e.findIndex(n,t);if(-1===i)return n;var a=n.slice(0,i),o=n[i],s=e.last(n),c=r&&26===s.kind,u=n.slice(i+1,c?n.length-1:void 0),d=e.compact([a.length?l(a):void 0,o,u.length?l(u):void 0]);return c?d.concat(s):d}function l(n){return e.Debug.assertGreaterThanOrEqual(n.length,1),e.setTextRangePosEnd(e.parseNodeFactory.createSyntaxList(n),n[0].pos,e.last(n).end)}function u(e){var n=e&&e.kind;return 18===n||22===n||20===n||278===n}function d(e){var n=e&&e.kind;return 19===n||23===n||21===n||279===n}function p(e,n){switch(n.kind){case 335:case 333:case 342:case 340:case 337:return e.getLineEndOfPosition(n.getStart());default:return n.getEnd()}}})(e.SmartSelectionRange||(e.SmartSelectionRange={}))}(d||(d={})),function(e){(function(n){var t,r;function i(n,t,r,i,a){var s=n.getTypeChecker(),l=e.findTokenOnLeftOfPosition(t,r);if(l){var u=!!i&&"characterTyped"===i.kind;if(!u||!e.isInString(t,r,l)&&!e.isInComment(t,r)){var d=!!i&&"invoked"===i.kind,p=D(l,r,t,s,d);if(p){a.throwIfCancellationRequested();var f=o(p,s,t,l,u);return a.throwIfCancellationRequested(),f?s.runWithCancellationToken(a,function(e){return 0===f.kind?k(f.candidates,f.resolvedSignature,p,t,e):R(f.symbol,p,t,e)}):e.isSourceFileJS(t)?c(p,n,a):void 0}}}}function o(n,t,r,i,a){var o=n.invocation,c=n.argumentCount;switch(o.kind){case 0:if(a&&!s(i,o.node,r))return;var u=[],d=t.getResolvedSignatureForSignatureHelp(o.node,u,c);return 0===u.length?void 0:{kind:0,candidates:u,resolvedSignature:d};case 1:var p=o.called;if(a&&!l(i,r,e.isIdentifier(p)?p.parent:p))return;u=e.getPossibleGenericSignatures(p,c,t);if(0!==u.length)return{kind:0,candidates:u,resolvedSignature:e.first(u)};var f=t.getSymbolAtLocation(p);return f&&{kind:1,symbol:f};case 2:return{kind:0,candidates:[o.signature],resolvedSignature:o.signature};default:return e.Debug.assertNever(o)}}function s(n,t,r){if(!e.isCallOrNewExpression(t))return!1;var i=t.getChildren(r);switch(n.kind){case 20:return e.contains(i,n);case 27:var a=e.findContainingList(n);return!!a&&e.contains(i,a);case 29:return l(n,r,t.expression);default:return!1}}function c(n,t,r){if(2!==n.invocation.kind){var i=C(n.invocation),a=e.isPropertyAccessExpression(i)?i.name.text:void 0,o=t.getTypeChecker();return void 0===a?void 0:e.firstDefined(t.getSourceFiles(),function(t){return e.firstDefined(t.getNamedDeclarations().get(a),function(e){var i=e.symbol&&o.getTypeOfSymbolAtLocation(e.symbol,e),a=i&&i.getCallSignatures();if(a&&a.length)return o.runWithCancellationToken(r,function(e){return k(a,a[0],n,t,e,!0)})})})}}function l(n,t,r){var i=n.getFullStart(),a=n.parent;while(a){var o=e.findPrecedingToken(i,t,a,!0);if(o)return e.rangeContainsRange(r,o);a=a.parent}return e.Debug.fail("Could not find preceding token")}function u(e,n,t){var r=f(e,n,t);return!r||r.isTypeParameterList||0!==r.invocation.kind?void 0:{invocation:r.invocation.node,argumentCount:r.argumentCount,argumentIndex:r.argumentIndex}}function d(n,t){var r=p(n,t);if(r){var i=r.list,a=r.argumentIndex,o=E(i);0!==a&&e.Debug.assertLessThan(a,o);var s=x(i,t);return{list:i,argumentIndex:a,argumentCount:o,argumentsSpan:s}}}function p(n,t){if(29===n.kind||20===n.kind)return{list:A(n.parent,n,t),argumentIndex:0};var r=e.findContainingList(n);return r&&{list:r,argumentIndex:b(r,n)}}function f(n,t,r){var i=n.parent;if(e.isCallOrNewExpression(i)){var a=i,o=d(n,r);if(!o)return;var s=o.list,c=o.argumentIndex,l=o.argumentCount,u=o.argumentsSpan,p=!!i.typeArguments&&i.typeArguments.pos===s.pos;return{isTypeParameterList:p,invocation:{kind:0,node:a},argumentsSpan:u,argumentIndex:c,argumentCount:l}}if(e.isNoSubstitutionTemplateLiteral(n)&&e.isTaggedTemplateExpression(i))return e.isInsideTemplateLiteral(n,t,r)?S(i,0,r):void 0;if(e.isTemplateHead(n)&&208===i.parent.kind){var f=i,m=f.parent;e.Debug.assert(221===f.kind);c=e.isInsideTemplateLiteral(n,t,r)?0:1;return S(m,c,r)}if(e.isTemplateSpan(i)&&e.isTaggedTemplateExpression(i.parent.parent)){var _=i;m=i.parent.parent;if(e.isTemplateTail(n)&&!e.isInsideTemplateLiteral(n,t,r))return;var g=_.parent.templateSpans.indexOf(_);c=T(g,n,t,r);return S(m,c,r)}if(e.isJsxOpeningLikeElement(i)){var y=i.attributes.pos,h=e.skipTrivia(r.text,i.attributes.end,!1);return{isTypeParameterList:!1,invocation:{kind:0,node:i},argumentsSpan:e.createTextSpan(y,h-y),argumentIndex:0,argumentCount:1}}var v=e.getPossibleTypeArgumentsInfo(n,r);if(v){var b=v.called,E=v.nTypeArguments;a={kind:1,called:b},u=e.createTextSpanFromBounds(b.getStart(r),n.end);return{isTypeParameterList:!0,invocation:a,argumentsSpan:u,argumentIndex:E,argumentCount:E+1}}}function m(e,n,t,r){return y(e,n,t,r)||f(e,n,t)}function _(n){return e.isBinaryExpression(n.parent)?_(n.parent):n}function g(n){return e.isBinaryExpression(n.left)?g(n.left)+1:2}function y(n,t,r,i){var a=h(n,r,i);if(a){var o=a.contextualType,s=a.argumentIndex,c=a.argumentCount,l=a.argumentsSpan,u=o.getNonNullableType(),d=u.getCallSignatures();if(1===d.length){var p={kind:2,signature:e.first(d),node:n,symbol:v(u.symbol)};return{isTypeParameterList:!1,invocation:p,argumentsSpan:l,argumentIndex:s,argumentCount:c}}}}function h(n,t,r){if(20===n.kind||27===n.kind){var i=n.parent;switch(i.kind){case 210:case 167:case 211:case 212:var a=d(n,t);if(!a)return;var o=a.argumentIndex,s=a.argumentCount,c=a.argumentsSpan,l=e.isMethodDeclaration(i)?r.getContextualTypeForObjectLiteralElement(i):r.getContextualType(i);return l&&{contextualType:l,argumentIndex:o,argumentCount:s,argumentsSpan:c};case 219:var u=_(i),p=r.getContextualType(u),f=20===n.kind?0:g(i)-1,m=g(u);return p&&{contextualType:p,argumentIndex:f,argumentCount:m,argumentsSpan:e.createTextSpanFromNode(i)};default:return}}}function v(n){return"__type"===n.name&&e.firstDefined(n.declarations,function(n){return e.isFunctionTypeNode(n)?n.parent.symbol:void 0})||n}function b(e,n){for(var t=0,r=0,i=e.getChildren();r<i.length;r++){var a=i[r];if(a===n)break;27!==a.kind&&t++}return t}function E(n){var t=n.getChildren(),r=e.countWhere(t,function(e){return 27!==e.kind});return t.length>0&&27===e.last(t).kind&&r++,r}function T(n,t,r,i){return e.Debug.assert(r>=t.getStart(),"Assumed 'position' could not occur before node."),e.isTemplateLiteralToken(t)?e.isInsideTemplateLiteral(t,r,i)?0:n+2:n+1}function S(n,t,r){var i=e.isNoSubstitutionTemplateLiteral(n.template)?1:n.template.templateSpans.length+1;return 0!==t&&e.Debug.assertLessThan(t,i),{isTypeParameterList:!1,invocation:{kind:0,node:n},argumentsSpan:L(n,r),argumentIndex:t,argumentCount:i}}function x(n,t){var r=n.getFullStart(),i=e.skipTrivia(t.text,n.getEnd(),!1);return e.createTextSpan(r,i-r)}function L(n,t){var r=n.template,i=r.getStart(),a=r.getEnd();if(221===r.kind){var o=e.last(r.templateSpans);0===o.literal.getFullWidth()&&(a=e.skipTrivia(t.text,a,!1))}return e.createTextSpan(i,a-i)}function D(n,t,r,i,a){for(var o=function(n){e.Debug.assert(e.rangeContainsRange(n.parent,n),"Not a subspan",function(){return"Child: "+e.Debug.formatSyntaxKind(n.kind)+", parent: "+e.Debug.formatSyntaxKind(n.parent.kind)});var a=m(n,t,r,i);if(a)return{value:a}},s=n;!e.isSourceFile(s)&&(a||!e.isBlock(s));s=s.parent){var c=o(s);if("object"===typeof c)return c.value}}function A(n,t,r){var i=n.getChildren(r),a=i.indexOf(t);return e.Debug.assert(a>=0&&i.length>a+1),i[a+1]}function C(n){return 0===n.kind?e.getInvokedExpression(n.node):n.called}function I(e){return 0===e.kind?e.node:1===e.kind?e.called:e.node}(function(e){e[e["Call"]=0]="Call",e[e["TypeArgs"]=1]="TypeArgs",e[e["Contextual"]=2]="Contextual"})(t||(t={})),n.getSignatureHelpItems=i,function(e){e[e["Candidate"]=0]="Candidate",e[e["Type"]=1]="Type"}(r||(r={})),n.getArgumentInfoForCompletions=u;var N=70246400;function k(n,t,r,i,a,o){var s,c=r.isTypeParameterList,l=r.argumentCount,u=r.argumentsSpan,d=r.invocation,p=r.argumentIndex,f=I(d),m=2===d.kind?d.symbol:a.getSymbolAtLocation(C(d))||o&&(null===(s=t.declaration)||void 0===s?void 0:s.symbol),_=m?e.symbolToDisplayParts(a,m,o?i:void 0,void 0):e.emptyArray,g=e.map(n,function(e){return P(e,_,c,a,f,i)});0!==p&&e.Debug.assertLessThan(p,l);for(var y=0,h=0,v=0;v<g.length;v++){var b=g[v];if(n[v]===t&&(y=h,b.length>1))for(var E=0,T=0,S=b;T<S.length;T++){var x=S[T];if(x.isVariadic||x.parameters.length>=l){y=h+E;break}E++}h+=b.length}e.Debug.assert(-1!==y);var L={items:e.flatMapToMutable(g,e.identity),applicableSpan:u,selectedItemIndex:y,argumentIndex:p,argumentCount:l},D=L.items[y];if(D.isVariadic){var A=e.findIndex(D.parameters,function(e){return!!e.isRest});-1<A&&A<D.parameters.length-1?L.argumentIndex=D.parameters.length:L.argumentIndex=Math.min(L.argumentIndex,D.parameters.length-1)}return L}function R(e,n,t,r){var i=n.argumentCount,a=n.argumentsSpan,o=n.invocation,s=n.argumentIndex,c=r.getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(e);if(c){var l=[O(e,c,r,I(o),t)];return{items:l,applicableSpan:a,selectedItemIndex:0,argumentIndex:s,argumentCount:i}}}function O(n,t,r,i,o){var s=e.symbolToDisplayParts(r,n),c=e.createPrinter({removeComments:!0}),l=t.map(function(e){return U(e,r,i,o,c)}),u=n.getDocumentationComment(r),d=n.getJsDocTags(r),p=a(a([],s,!0),[e.punctuationPart(29)],!1);return{isVariadic:!1,prefixDisplayParts:p,suffixDisplayParts:[e.punctuationPart(31)],separatorDisplayParts:w,parameters:l,documentation:u,tags:d}}var w=[e.punctuationPart(27),e.spacePart()];function P(n,t,r,i,o,s){var c=(r?F:G)(n,i,o,s);return e.map(c,function(e){var r=e.isVariadic,s=e.parameters,c=e.prefix,l=e.suffix,u=a(a([],t,!0),c,!0),d=a(a([],l,!0),M(n,o,i),!0),p=n.getDocumentationComment(i),f=n.getJsDocTags();return{isVariadic:r,prefixDisplayParts:u,suffixDisplayParts:d,separatorDisplayParts:w,parameters:s,documentation:p,tags:f}})}function M(n,t,r){return e.mapToDisplayParts(function(e){e.writePunctuation(":"),e.writeSpace(" ");var i=r.getTypePredicateOfSignature(n);i?r.writeTypePredicate(i,t,void 0,e):r.writeType(r.getReturnTypeOfSignature(n),t,void 0,e)})}function F(n,t,r,i){var o=(n.target||n).typeParameters,s=e.createPrinter({removeComments:!0}),c=(o||e.emptyArray).map(function(e){return U(e,t,r,i,s)}),l=n.thisParameter?[t.symbolToParameterDeclaration(n.thisParameter,r,N)]:[];return t.getExpandedParameters(n).map(function(n){var o=e.factory.createNodeArray(a(a([],l,!0),e.map(n,function(e){return t.symbolToParameterDeclaration(e,r,N)}),!0)),u=e.mapToDisplayParts(function(e){s.writeList(2576,o,i,e)});return{isVariadic:!1,parameters:c,prefix:[e.punctuationPart(29)],suffix:a([e.punctuationPart(31)],u,!0)}})}function G(n,t,r,i){var o=e.createPrinter({removeComments:!0}),s=e.mapToDisplayParts(function(a){if(n.typeParameters&&n.typeParameters.length){var s=e.factory.createNodeArray(n.typeParameters.map(function(e){return t.typeParameterToDeclaration(e,r,N)}));o.writeList(53776,s,i,a)}}),c=t.getExpandedParameters(n),l=t.hasEffectiveRestParameter(n)?1===c.length?function(e){return!0}:function(e){return!!(e.length&&32768&e[e.length-1].checkFlags)}:function(e){return!1};return c.map(function(n){return{isVariadic:l(n),parameters:n.map(function(e){return B(e,t,r,i,o)}),prefix:a(a([],s,!0),[e.punctuationPart(20)],!1),suffix:[e.punctuationPart(21)]}})}function B(n,t,r,i,a){var o=e.mapToDisplayParts(function(e){var o=t.symbolToParameterDeclaration(n,r,N);a.writeNode(4,o,i,e)}),s=t.isOptionalParameter(n.valueDeclaration),c=!!(32768&n.checkFlags);return{name:n.name,documentation:n.getDocumentationComment(t),displayParts:o,isOptional:s,isRest:c}}function U(n,t,r,i,a){var o=e.mapToDisplayParts(function(e){var o=t.typeParameterToDeclaration(n,r,N);a.writeNode(4,o,i,e)});return{name:n.symbol.name,documentation:n.symbol.getDocumentationComment(t),displayParts:o,isOptional:!1,isRest:!1}}})(e.SignatureHelp||(e.SignatureHelp={}))}(d||(d={})),function(e){(function(n){var t=30,r=function(e){return new RegExp("^\\s?/\\*\\*?\\s?"+e+"\\s?\\*\\/\\s?$")};function i(e){return"literals"===e.includeInlayParameterNameHints||"all"===e.includeInlayParameterNameHints}function a(e){return"literals"===e.includeInlayParameterNameHints}function o(n){var o=n.file,s=n.program,c=n.span,l=n.cancellationToken,u=n.preferences,d=o.text,p=s.getCompilerOptions(),f=s.getTypeChecker(),m=[];return _(o),m;function _(n){if(n&&0!==n.getFullWidth()){switch(n.kind){case 259:case 255:case 256:case 254:case 224:case 211:case 167:case 212:l.throwIfCancellationRequested()}if(e.textSpanIntersectsWith(c,n.pos,n.getFullWidth())&&!e.isTypeNode(n))return u.includeInlayVariableTypeHints&&e.isVariableDeclaration(n)?T(n):u.includeInlayPropertyDeclarationTypeHints&&e.isPropertyDeclaration(n)?T(n):u.includeInlayEnumMemberValueHints&&e.isEnumMember(n)?b(n):i(u)&&(e.isCallExpression(n)||e.isNewExpression(n))?S(n):(u.includeInlayFunctionParameterTypeHints&&e.isFunctionLikeDeclaration(n)&&e.hasContextSensitiveParameters(n)&&I(n),u.includeInlayFunctionLikeReturnTypeHints&&g(n)&&A(n)),e.forEachChild(n,_)}}function g(n){return e.isArrowFunction(n)||e.isFunctionExpression(n)||e.isFunctionDeclaration(n)||e.isMethodDeclaration(n)||e.isGetAccessorDeclaration(n)}function y(e,n,r){m.push({text:(r?"...":"")+k(e,t)+":",position:n,kind:"Parameter",whitespaceAfter:!0})}function h(e,n){m.push({text:": "+k(e,t),position:n,kind:"Type",whitespaceBefore:!0})}function v(e,n){m.push({text:"= "+k(e,t),position:n,kind:"Enum",whitespaceBefore:!0})}function b(e){if(!e.initializer){var n=f.getConstantValue(e);void 0!==n&&v(n.toString(),e.end)}}function E(e){return e.symbol&&1536&e.symbol.flags}function T(n){if(n.initializer&&!e.isBindingPattern(n.name)){var t=e.getEffectiveTypeAnnotationNode(n);if(!t){var r=f.getTypeAtLocation(n);if(!E(r)){var i=R(r);i&&h(i,n.name.end)}}}}function S(n){var t=n.arguments;if(t&&t.length){var r=[],i=f.getResolvedSignatureForSignatureHelp(n,r);if(i&&r.length)for(var o=0;o<t.length;++o){var s=t[o],c=e.skipParentheses(s);if(!a(u)||D(c)){var l=f.getParameterIdentifierNameAtPosition(i,o);if(l){var d=l[0],p=l[1],m=u.includeInlayParameterNameHintsWhenArgumentMatchesName||!x(c,d);if(!m&&!p)continue;var _=e.unescapeLeadingUnderscores(d);if(L(c,_))continue;y(_,s.getStart(),p)}}}}}function x(n,t){return e.isIdentifier(n)?n.text===t:!!e.isPropertyAccessExpression(n)&&n.name.text===t}function L(n,t){if(!e.isIdentifierText(t,p.target,e.getLanguageVariant(o.scriptKind)))return!1;var i=e.getLeadingCommentRanges(d,n.pos);if(!(null===i||void 0===i?void 0:i.length))return!1;var a=r(t);return e.some(i,function(e){return a.test(d.substring(e.pos,e.end))})}function D(n){switch(n.kind){case 217:var t=n.operand;return e.isLiteralExpression(t)||e.isIdentifier(t)&&e.isInfinityOrNaNString(t.escapedText);case 110:case 95:case 104:return!0;case 79:var r=n.escapedText;return O(r)||e.isInfinityOrNaNString(r)}return e.isLiteralExpression(n)}function A(n){if(!e.isArrowFunction(n)||e.findChildOfKind(n,20,o)){var t=e.getEffectiveReturnTypeNode(n);if(!t&&n.body){var r=f.getSignatureFromDeclaration(n);if(r){var i=f.getReturnTypeOfSignature(r);if(!E(i)){var a=R(i);a&&h(a,C(n))}}}}}function C(n){var t=e.findChildOfKind(n,21,o);return t?t.end:n.parameters.end}function I(n){var t=f.getSignatureFromDeclaration(n);if(t)for(var r=0;r<n.parameters.length&&r<t.parameters.length;++r){var i=n.parameters[r],a=e.getEffectiveTypeAnnotationNode(i);if(!a){var o=N(t.parameters[r]);o&&h(o,i.end)}}}function N(n){var t=n.valueDeclaration;if(t&&e.isParameter(t)){var r=f.getTypeOfSymbolAtLocation(n,t);if(!E(r))return R(r)}}function k(e,n){return e.length>n?e.substr(0,n-"...".length)+"...":e}function R(n){var t=71286784,r={removeComments:!0},i=e.createPrinter(r);return e.usingSingleLineStringWriter(function(r){var a=f.typeToTypeNode(n,void 0,t,r);e.Debug.assertIsDefined(a,"should always get typenode"),i.writeNode(4,a,o,r)})}function O(e){return"undefined"===e}}n.provideInlayHints=o})(e.InlayHints||(e.InlayHints={}))}(d||(d={})),function(e){var n=/^data:(?:application\/json(?:;charset=[uU][tT][fF]-8);base64,([A-Za-z0-9+\/=]+)$)?/;function t(n){var t=e.createGetCanonicalFileName(n.useCaseSensitiveFileNames()),r=n.getCurrentDirectory(),i=new e.Map,o=new e.Map;return{tryGetSourcePosition:l,tryGetGeneratedPosition:u,toLineColumnOffset:m,clearCache:_};function s(n){return e.toPath(n,r,t)}function c(r,i){var a,c=s(r),l=o.get(c);if(l)return l;if(n.getDocumentPositionMapper)a=n.getDocumentPositionMapper(r,i);else if(n.readFile){var u=f(r);a=u&&e.getDocumentPositionMapper({getSourceFileLike:f,getCanonicalFileName:t,log:function(e){return n.log(e)}},r,e.getLineInfo(u.text,e.getLineStarts(u)),function(e){return!n.fileExists||n.fileExists(e)?n.readFile(e):void 0})}return o.set(c,a||e.identitySourceMapConsumer),a||e.identitySourceMapConsumer}function l(n){if(e.isDeclarationFileName(n.fileName)){var t=d(n.fileName);if(t){var r=c(n.fileName).getSourcePosition(n);return r&&r!==n?l(r)||r:void 0}}}function u(i){if(!e.isDeclarationFileName(i.fileName)){var a=d(i.fileName);if(a){var o=n.getProgram();if(!o.isSourceOfProjectReferenceRedirect(a.fileName)){var s=o.getCompilerOptions(),l=e.outFile(s),u=l?e.removeFileExtension(l)+".d.ts":e.getDeclarationEmitOutputFilePathWorker(i.fileName,o.getCompilerOptions(),r,o.getCommonSourceDirectory(),t);if(void 0!==u){var p=c(u,i.fileName).getGeneratedPosition(i);return p===i?void 0:p}}}}}function d(e){var t=n.getProgram();if(t){var r=s(e),i=t.getSourceFileByPath(r);return i&&i.resolvedPath===r?i:void 0}}function p(e){var t=s(e),r=i.get(t);if(void 0!==r)return r||void 0;if(n.readFile&&(!n.fileExists||n.fileExists(t))){var o=n.readFile(t),c=!!o&&a(o);return i.set(t,c),c||void 0}i.set(t,!1)}function f(e){return n.getSourceFileLike?n.getSourceFileLike(e):d(e)||p(e)}function m(e,n){var t=f(e);return t.getLineAndCharacterOfPosition(n)}function _(){i.clear(),o.clear()}}function r(t,r,a,o){var s=e.tryGetSourceMappingURL(a);if(s){var c=n.exec(s);if(c){if(c[1]){var l=c[1];return i(t,e.base64decode(e.sys,l),r)}s=void 0}}var u=[];s&&u.push(s),u.push(r+".map");for(var d=s&&e.getNormalizedAbsolutePath(s,e.getDirectoryPath(r)),p=0,f=u;p<f.length;p++){var m=f[p],_=e.getNormalizedAbsolutePath(m,e.getDirectoryPath(r)),g=o(_,d);if(e.isString(g))return i(t,g,_);if(void 0!==g)return g||void 0}}function i(n,t,r){var i=e.tryParseRawSourceMap(t);if(i&&i.sources&&i.file&&i.mappings&&(!i.sourcesContent||!i.sourcesContent.some(e.isString)))return e.createDocumentPositionMapper(n,i,r)}function a(n,t){return{text:n,lineMap:t,getLineAndCharacterOfPosition:function(n){return e.computeLineAndCharacterOfPosition(e.getLineStarts(this),n)}}}e.getSourceMapper=t,e.getDocumentPositionMapper=r}(d||(d={})),function(e){var n=new e.Map;function t(t,i,s){i.getSemanticDiagnostics(t,s);var c=[],u=i.getTypeChecker();t.commonJsModuleIndicator&&(e.programContainsEs6Modules(i)||e.compilerOptionsIndicateEs6Modules(i.getCompilerOptions()))&&r(t)&&c.push(e.createDiagnosticForNode(l(t.commonJsModuleIndicator),e.Diagnostics.File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module));var d=e.isSourceFileJS(t);if(n.clear(),E(t),e.getAllowSyntheticDefaultImports(i.getCompilerOptions()))for(var p=0,f=t.imports;p<f.length;p++){var m=f[p],_=e.importFromModuleSpecifier(m),g=a(_);if(g){var v=e.getResolvedModule(t,m.text),b=v&&i.getSourceFile(v.resolvedFileName);b&&b.externalModuleIndicator&&e.isExportAssignment(b.externalModuleIndicator)&&b.externalModuleIndicator.isExportEquals&&c.push(e.createDiagnosticForNode(g,e.Diagnostics.Import_may_be_converted_to_a_default_import))}}return e.addRange(c,t.bindSuggestionDiagnostics),e.addRange(c,i.getSuggestionDiagnostics(t,s)),c.sort(function(e,n){return e.start-n.start});function E(n){if(d)y(n,u)&&c.push(e.createDiagnosticForNode(e.isVariableDeclaration(n.parent)?n.parent.name:n,e.Diagnostics.This_constructor_function_may_be_converted_to_a_class_declaration));else{if(e.isVariableStatement(n)&&n.parent===t&&2&n.declarationList.flags&&1===n.declarationList.declarations.length){var r=n.declarationList.declarations[0].initializer;r&&e.isRequireCall(r,!0)&&c.push(e.createDiagnosticForNode(r,e.Diagnostics.require_call_may_be_converted_to_an_import))}e.codefix.parameterShouldGetTypeFromJSDoc(n)&&c.push(e.createDiagnosticForNode(n.name||n,e.Diagnostics.JSDoc_types_may_be_moved_to_TypeScript_types))}h(n)&&o(n,u,c),n.forEachChild(E)}}function r(n){return n.statements.some(function(n){switch(n.kind){case 235:return n.declarationList.declarations.some(function(n){return!!n.initializer&&e.isRequireCall(i(n.initializer),!0)});case 236:var t=n.expression;if(!e.isBinaryExpression(t))return e.isRequireCall(t,!0);var r=e.getAssignmentDeclarationKind(t);return 1===r||2===r;default:return!1}})}function i(n){return e.isPropertyAccessExpression(n)?i(n.expression):n}function a(n){switch(n.kind){case 264:var t=n.importClause,r=n.moduleSpecifier;return t&&!t.name&&t.namedBindings&&266===t.namedBindings.kind&&e.isStringLiteral(r)?t.namedBindings.name:void 0;case 263:return n.name;default:return}}function o(t,r,i){s(t,r)&&!n.has(g(t))&&i.push(e.createDiagnosticForNode(!t.name&&e.isVariableDeclaration(t.parent)&&e.isIdentifier(t.parent.name)?t.parent.name:t,e.Diagnostics.This_may_be_converted_to_an_async_function))}function s(n,t){return!e.isAsyncFunction(n)&&n.body&&e.isBlock(n.body)&&u(n.body,t)&&c(n,t)}function c(e,n){var t=n.getSignatureFromDeclaration(e),r=t?n.getReturnTypeOfSignature(t):void 0;return!!r&&!!n.getPromisedTypeOfPromise(r)}function l(n){return e.isBinaryExpression(n)?n.left:n}function u(n,t){return!!e.forEachReturnStatement(n,function(e){return d(e,t)})}function d(n,t){return e.isReturnStatement(n)&&!!n.expression&&p(n.expression,t)}function p(n,t){if(!f(n)||!n.arguments.every(function(e){return _(e,t)}))return!1;var r=n.expression;while(f(r)||e.isPropertyAccessExpression(r)){if(e.isCallExpression(r)&&!r.arguments.every(function(e){return _(e,t)}))return!1;r=r.expression}return!0}function f(n){return e.isCallExpression(n)&&(e.hasPropertyAccessExpressionWithName(n,"then")&&m(n)||e.hasPropertyAccessExpressionWithName(n,"catch"))}function m(n){return!(n.arguments.length>2)&&(n.arguments.length<2||e.some(n.arguments,function(n){return 104===n.kind||e.isIdentifier(n)&&"undefined"===n.text}))}function _(t,r){switch(t.kind){case 254:case 211:var i=e.getFunctionFlags(t);if(1&i)return!1;case 212:n.set(g(t),!0);case 104:return!0;case 79:case 204:var a=r.getSymbolAtLocation(t);return!!a&&(r.isUndefinedSymbol(a)||e.some(e.skipAlias(a,r).declarations,function(n){return e.isFunctionLike(n)||e.hasInitializer(n)&&!!n.initializer&&e.isFunctionLike(n.initializer)}));default:return!1}}function g(e){return e.pos.toString()+":"+e.end.toString()}function y(n,t){var r,i,a,o;if(211===n.kind){if(e.isVariableDeclaration(n.parent)&&(null===(r=n.symbol.members)||void 0===r?void 0:r.size))return!0;var s=t.getSymbolOfExpando(n,!1);return!(!s||!((null===(i=s.exports)||void 0===i?void 0:i.size)||(null===(a=s.members)||void 0===a?void 0:a.size)))}return 254===n.kind&&!!(null===(o=n.symbol.members)||void 0===o?void 0:o.size)}function h(e){switch(e.kind){case 254:case 167:case 211:case 212:return!0;default:return!1}}e.computeSuggestionDiagnostics=t,e.returnsPromise=c,e.isReturnStatementWithFixablePromiseHandler=d,e.isFixablePromiseHandler=p,e.canBeConvertedToAsync=h}(d||(d={})),function(e){(function(n){var t=70246400;function r(n,t,r){var a=i(n,t,r);if(""!==a)return a;var o=e.getCombinedLocalAndExportSymbolFlags(t);return 32&o?e.getDeclarationOfKind(t,224)?"local class":"class":384&o?"enum":524288&o?"type":64&o?"interface":262144&o?"type parameter":8&o?"enum member":2097152&o?"alias":1536&o?"module":a}function i(n,t,r){var i=n.getRootSymbols(t);if(1===i.length&&8192&e.first(i).flags&&0!==n.getTypeOfSymbolAtLocation(t,r).getNonNullableType().getCallSignatures().length)return"method";if(n.isUndefinedSymbol(t))return"var";if(n.isArgumentsSymbol(t))return"local var";if(108===r.kind&&e.isExpression(r))return"parameter";var a=e.getCombinedLocalAndExportSymbolFlags(t);if(3&a)return e.isFirstDeclarationOfSymbolParameter(t)?"parameter":t.valueDeclaration&&e.isVarConst(t.valueDeclaration)?"const":e.forEach(t.declarations,e.isLet)?"let":c(t)?"local var":"var";if(16&a)return c(t)?"local function":"function";if(32768&a)return"getter";if(65536&a)return"setter";if(8192&a)return"method";if(16384&a)return"constructor";if(4&a){if(33554432&a&&6&t.checkFlags){var o=e.forEach(n.getRootSymbols(t),function(e){var n=e.getFlags();if(98311&n)return"property"});if(!o){var s=n.getTypeOfSymbolAtLocation(t,r);return s.getCallSignatures().length?"method":"property"}return o}switch(r.parent&&r.parent.kind){case 278:case 276:case 277:return 79===r.kind?"property":"JSX attribute";case 283:return"JSX attribute";default:return"property"}}return""}function a(n){if(n.declarations&&n.declarations.length){var t=n.declarations,r=t[0],i=t.slice(1),a=e.length(i)&&e.isDeprecatedDeclaration(r)&&e.some(i,function(n){return!e.isDeprecatedDeclaration(n)})?8192:0,o=e.getNodeModifiers(r,a);if(o)return o.split(",")}return[]}function o(n,t){if(!t)return"";var r=new e.Set(a(t));if(2097152&t.flags){var i=n.getAliasedSymbol(t);i!==t&&e.forEach(a(i),function(e){r.add(e)})}return 16777216&t.flags&&r.add("optional"),r.size>0?e.arrayFrom(r.values()).join(","):""}function s(n,a,o,c,l,u,d){var p;void 0===u&&(u=e.getMeaningFromLocation(l));var f,m,_,g,y=[],h=[],v=[],b=e.getCombinedLocalAndExportSymbolFlags(a),E=1&u?i(n,a,l):"",T=!1,S=108===l.kind&&e.isInExpressionContext(l),x=!1;if(108===l.kind&&!S)return{displayParts:[e.keywordPart(108)],documentation:[],symbolKind:"primitive type",tags:void 0};if(""!==E||32&b||2097152&b){"getter"!==E&&"setter"!==E||(E="property");var L=void 0;if(f=S?n.getTypeAtLocation(l):n.getTypeOfSymbolAtLocation(a,l),l.parent&&204===l.parent.kind){var D=l.parent.name;(D===l||D&&0===D.getFullWidth())&&(l=l.parent)}var A=void 0;if(e.isCallOrNewExpression(l)?A=l:e.isCallExpressionTarget(l)||e.isNewExpressionTarget(l)?A=l.parent:l.parent&&(e.isJsxOpeningLikeElement(l.parent)||e.isTaggedTemplateExpression(l.parent))&&e.isFunctionLike(a.valueDeclaration)&&(A=l.parent),A){L=n.getResolvedSignature(A);var C=207===A.kind||e.isCallExpression(A)&&106===A.expression.kind,I=C?f.getConstructSignatures():f.getCallSignatures();if(!L||e.contains(I,L.target)||e.contains(I,L)||(L=I.length?I[0]:void 0),L){switch(C&&32&b?(E="constructor",Z(f.symbol,E)):2097152&b?(E="alias",$(E),y.push(e.spacePart()),C&&(4&L.flags&&(y.push(e.keywordPart(126)),y.push(e.spacePart())),y.push(e.keywordPart(103)),y.push(e.spacePart())),Q(a)):Z(a,E),E){case"JSX attribute":case"property":case"var":case"const":case"let":case"parameter":case"local var":y.push(e.punctuationPart(58)),y.push(e.spacePart()),16&e.getObjectFlags(f)||!f.symbol||(e.addRange(y,e.symbolToDisplayParts(n,f.symbol,c,void 0,5)),y.push(e.lineBreakPart())),C&&(4&L.flags&&(y.push(e.keywordPart(126)),y.push(e.spacePart())),y.push(e.keywordPart(103)),y.push(e.spacePart())),ee(L,I,262144);break;default:ee(L,I)}T=!0,x=I.length>1}}else if(e.isNameOfFunctionDeclaration(l)&&!(98304&b)||133===l.kind&&169===l.parent.kind){var N=l.parent,k=a.declarations&&e.find(a.declarations,function(e){return e===(133===l.kind?N.parent:N)});if(k){I=169===N.kind?f.getNonNullableType().getConstructSignatures():f.getNonNullableType().getCallSignatures();L=n.isImplementationOfOverload(N)?I[0]:n.getSignatureFromDeclaration(N),169===N.kind?(E="constructor",Z(f.symbol,E)):Z(172!==N.kind||2048&f.symbol.flags||4096&f.symbol.flags?a:f.symbol,E),L&&ee(L,I),T=!0,x=I.length>1}}}if(32&b&&!T&&!S&&(X(),e.getDeclarationOfKind(a,224)?$("local class"):y.push(e.keywordPart(84)),y.push(e.spacePart()),Q(a),ne(a,o)),64&b&&2&u&&(q(),y.push(e.keywordPart(118)),y.push(e.spacePart()),Q(a),ne(a,o)),524288&b&&2&u&&(q(),y.push(e.keywordPart(150)),y.push(e.spacePart()),Q(a),ne(a,o),y.push(e.spacePart()),y.push(e.operatorPart(63)),y.push(e.spacePart()),e.addRange(y,e.typeToDisplayParts(n,n.getDeclaredTypeOfSymbol(a),c,8388608))),384&b&&(q(),e.some(a.declarations,function(n){return e.isEnumDeclaration(n)&&e.isEnumConst(n)})&&(y.push(e.keywordPart(85)),y.push(e.spacePart())),y.push(e.keywordPart(92)),y.push(e.spacePart()),Q(a)),1536&b&&!S){q();var R=e.getDeclarationOfKind(a,259),O=R&&R.name&&79===R.name.kind;y.push(e.keywordPart(O?141:140)),y.push(e.spacePart()),Q(a)}if(262144&b&&2&u)if(q(),y.push(e.punctuationPart(20)),y.push(e.textPart("type parameter")),y.push(e.punctuationPart(21)),y.push(e.spacePart()),Q(a),a.parent)Y(),Q(a.parent,c),ne(a.parent,c);else{var w=e.getDeclarationOfKind(a,161);if(void 0===w)return e.Debug.fail();R=w.parent;if(R)if(e.isFunctionLikeKind(R.kind)){Y();L=n.getSignatureFromDeclaration(R);173===R.kind?(y.push(e.keywordPart(103)),y.push(e.spacePart())):172!==R.kind&&R.name&&Q(R.symbol),e.addRange(y,e.signatureToDisplayParts(n,L,o,32))}else 257===R.kind&&(Y(),y.push(e.keywordPart(150)),y.push(e.spacePart()),Q(R.symbol),ne(R.symbol,o))}if(8&b){E="enum member",Z(a,"enum member");R=null===(p=a.declarations)||void 0===p?void 0:p[0];if(294===(null===R||void 0===R?void 0:R.kind)){var P=n.getConstantValue(R);void 0!==P&&(y.push(e.spacePart()),y.push(e.operatorPart(63)),y.push(e.spacePart()),y.push(e.displayPart(e.getTextOfConstantValue(P),"number"===typeof P?e.SymbolDisplayPartKind.numericLiteral:e.SymbolDisplayPartKind.stringLiteral)))}}if(2097152&a.flags){if(q(),!T){var M=n.getAliasedSymbol(a);if(M!==a&&M.declarations&&M.declarations.length>0){var F=M.declarations[0],G=e.getNameOfDeclaration(F);if(G){var B=e.isModuleWithStringLiteralName(F)&&e.hasSyntacticModifier(F,2),U="default"!==a.name&&!B,V=s(n,M,e.getSourceFileOfNode(F),F,G,u,U?a:M);y.push.apply(y,V.displayParts),y.push(e.lineBreakPart()),_=V.documentation,g=V.tags}else _=M.getContextualDocumentationComment(F,n),g=M.getJsDocTags(n)}}if(a.declarations)switch(a.declarations[0].kind){case 262:y.push(e.keywordPart(93)),y.push(e.spacePart()),y.push(e.keywordPart(141));break;case 269:y.push(e.keywordPart(93)),y.push(e.spacePart()),y.push(e.keywordPart(a.declarations[0].isExportEquals?63:88));break;case 273:y.push(e.keywordPart(93));break;default:y.push(e.keywordPart(100))}y.push(e.spacePart()),Q(a),e.forEach(a.declarations,function(t){if(263===t.kind){var r=t;if(e.isExternalModuleImportEqualsDeclaration(r))y.push(e.spacePart()),y.push(e.operatorPart(63)),y.push(e.spacePart()),y.push(e.keywordPart(144)),y.push(e.punctuationPart(20)),y.push(e.displayPart(e.getTextOfNode(e.getExternalModuleImportEqualsDeclarationExpression(r)),e.SymbolDisplayPartKind.stringLiteral)),y.push(e.punctuationPart(21));else{var i=n.getSymbolAtLocation(r.moduleReference);i&&(y.push(e.spacePart()),y.push(e.operatorPart(63)),y.push(e.spacePart()),Q(i,c))}return!0}})}if(!T)if(""!==E){if(f)if(S?(q(),y.push(e.keywordPart(108))):Z(a,E),"property"===E||"JSX attribute"===E||3&b||"local var"===E||S){if(y.push(e.punctuationPart(58)),y.push(e.spacePart()),f.symbol&&262144&f.symbol.flags){var j=e.mapToDisplayParts(function(r){var i=n.typeParameterToDeclaration(f,c,t);J().writeNode(4,i,e.getSourceFileOfNode(e.getParseTreeNode(c)),r)});e.addRange(y,j)}else e.addRange(y,e.typeToDisplayParts(n,f,c));if(a.target&&a.target.tupleLabelDeclaration){var K=a.target.tupleLabelDeclaration;e.Debug.assertNode(K.name,e.isIdentifier),y.push(e.spacePart()),y.push(e.punctuationPart(20)),y.push(e.textPart(e.idText(K.name))),y.push(e.punctuationPart(21))}}else if(16&b||8192&b||16384&b||131072&b||98304&b||"method"===E){I=f.getNonNullableType().getCallSignatures();I.length&&(ee(I[0],I),x=I.length>1)}}else E=r(n,a,l);if(0!==h.length||x||(h=a.getContextualDocumentationComment(c,n)),0===h.length&&4&b&&a.parent&&a.declarations&&e.forEach(a.parent.declarations,function(e){return 300===e.kind}))for(var H=0,W=a.declarations;H<W.length;H++){R=W[H];if(R.parent&&219===R.parent.kind){var z=n.getSymbolAtLocation(R.parent.right);if(z&&(h=z.getDocumentationComment(n),v=z.getJsDocTags(n),h.length>0))break}}return 0!==v.length||x||(v=a.getJsDocTags(n)),0===h.length&&_&&(h=_),0===v.length&&g&&(v=g),{displayParts:y,documentation:h,symbolKind:E,tags:0===v.length?void 0:v};function J(){return m||(m=e.createPrinter({removeComments:!0})),m}function q(){y.length&&y.push(e.lineBreakPart()),X()}function X(){d&&($("alias"),y.push(e.spacePart()))}function Y(){y.push(e.spacePart()),y.push(e.keywordPart(101)),y.push(e.spacePart())}function Q(t,r){d&&t===a&&(t=d);var i=e.symbolToDisplayParts(n,t,r||o,void 0,7);e.addRange(y,i),16777216&a.flags&&y.push(e.punctuationPart(57))}function Z(n,t){q(),t&&($(t),n&&!e.some(n.declarations,function(n){return e.isArrowFunction(n)||(e.isFunctionExpression(n)||e.isClassExpression(n))&&!n.name})&&(y.push(e.spacePart()),Q(n)))}function $(n){switch(n){case"var":case"function":case"let":case"const":case"constructor":return void y.push(e.textOrKeywordPart(n));default:return y.push(e.punctuationPart(20)),y.push(e.textOrKeywordPart(n)),void y.push(e.punctuationPart(21))}}function ee(t,r,i){void 0===i&&(i=0),e.addRange(y,e.signatureToDisplayParts(n,t,c,32|i)),r.length>1&&(y.push(e.spacePart()),y.push(e.punctuationPart(20)),y.push(e.operatorPart(39)),y.push(e.displayPart((r.length-1).toString(),e.SymbolDisplayPartKind.numericLiteral)),y.push(e.spacePart()),y.push(e.textPart(2===r.length?"overload":"overloads")),y.push(e.punctuationPart(21))),h=t.getDocumentationComment(n),v=t.getJsDocTags(),r.length>1&&0===h.length&&0===v.length&&(h=r[0].getDocumentationComment(n),v=r[0].getJsDocTags())}function ne(r,i){var a=e.mapToDisplayParts(function(a){var o=n.symbolToTypeParameterDeclarations(r,i,t);J().writeList(53776,o,e.getSourceFileOfNode(e.getParseTreeNode(i)),a)});e.addRange(y,a)}}function c(n){return!n.parent&&e.forEach(n.declarations,function(n){if(211===n.kind)return!0;if(252!==n.kind&&254!==n.kind)return!1;for(var t=n.parent;!e.isFunctionBlock(t);t=t.parent)if(300===t.kind||260===t.kind)return!1;return!0})}n.getSymbolKind=r,n.getSymbolModifiers=o,n.getSymbolDisplayPartsDocumentationAndSymbolKind=s})(e.SymbolDisplay||(e.SymbolDisplay={}))}(d||(d={})),function(e){function n(n,t){var r=[],a=t.compilerOptions?i(t.compilerOptions,r):{},o=e.getDefaultCompilerOptions();for(var s in o)e.hasProperty(o,s)&&void 0===a[s]&&(a[s]=o[s]);for(var c=0,l=e.transpileOptionValueCompilerOptions;c<l.length;c++){var u=l[c];a[u.name]=u.transpileOptionValue}a.suppressOutputPathCheck=!0,a.allowNonTsExtensions=!0;var d=t.fileName||(t.compilerOptions&&t.compilerOptions.jsx?"module.tsx":"module.ts"),p=e.createSourceFile(d,n,a.target);t.moduleName&&(p.moduleName=t.moduleName),t.renamedDependencies&&(p.renamedDependencies=new e.Map(e.getEntries(t.renamedDependencies)));var f,m,_=e.getNewLineCharacter(a),g={getSourceFile:function(n){return n===e.normalizePath(d)?p:void 0},writeFile:function(n,t){e.fileExtensionIs(n,".map")?(e.Debug.assertEqual(m,void 0,"Unexpected multiple source map outputs, file:",n),m=t):(e.Debug.assertEqual(f,void 0,"Unexpected multiple outputs, file:",n),f=t)},getDefaultLibFileName:function(){return"lib.d.ts"},useCaseSensitiveFileNames:function(){return!1},getCanonicalFileName:function(e){return e},getCurrentDirectory:function(){return""},getNewLine:function(){return _},fileExists:function(e){return e===d},readFile:function(){return""},directoryExists:function(){return!0},getDirectories:function(){return[]}},y=e.createProgram([d],a,g);return t.reportDiagnostics&&(e.addRange(r,y.getSyntacticDiagnostics(p)),e.addRange(r,y.getOptionsDiagnostics())),y.emit(void 0,void 0,void 0,void 0,t.transformers),void 0===f?e.Debug.fail("Output generation failed"):{outputText:f,diagnostics:r,sourceMapText:m}}function t(t,r,i,a,o){var s=n(t,{compilerOptions:r,fileName:i,reportDiagnostics:!!a,moduleName:o});return e.addRange(a,s.diagnostics),s.outputText}var r;function i(n,t){r=r||e.filter(e.optionDeclarations,function(n){return"object"===typeof n.type&&!e.forEachEntry(n.type,function(e){return"number"!==typeof e})}),n=e.cloneCompilerOptions(n);for(var i=function(r){if(!e.hasProperty(n,r.name))return"continue";var i=n[r.name];e.isString(i)?n[r.name]=e.parseCustomTypeOption(r,i,t):e.forEachEntry(r.type,function(e){return e===i})||t.push(e.createCompilerDiagnosticForInvalidCustomType(r))},a=0,o=r;a<o.length;a++){var s=o[a];i(s)}return n}e.transpileModule=n,e.transpile=t,e.fixupCompilerOptions=i}(d||(d={})),function(e){(function(n){(function(e){e[e["FormatDocument"]=0]="FormatDocument",e[e["FormatSelection"]=1]="FormatSelection",e[e["FormatOnEnter"]=2]="FormatOnEnter",e[e["FormatOnSemicolon"]=3]="FormatOnSemicolon",e[e["FormatOnOpeningCurlyBrace"]=4]="FormatOnOpeningCurlyBrace",e[e["FormatOnClosingCurlyBrace"]=5]="FormatOnClosingCurlyBrace"})(n.FormattingRequestKind||(n.FormattingRequestKind={}));var t=function(){function n(e,n,t){this.sourceFile=e,this.formattingRequestKind=n,this.options=t}return n.prototype.updateContext=function(n,t,r,i,a){this.currentTokenSpan=e.Debug.checkDefined(n),this.currentTokenParent=e.Debug.checkDefined(t),this.nextTokenSpan=e.Debug.checkDefined(r),this.nextTokenParent=e.Debug.checkDefined(i),this.contextNode=e.Debug.checkDefined(a),this.contextNodeAllOnSameLine=void 0,this.nextNodeAllOnSameLine=void 0,this.tokensAreOnSameLine=void 0,this.contextNodeBlockIsOnOneLine=void 0,this.nextNodeBlockIsOnOneLine=void 0},n.prototype.ContextNodeAllOnSameLine=function(){return void 0===this.contextNodeAllOnSameLine&&(this.contextNodeAllOnSameLine=this.NodeIsOnOneLine(this.contextNode)),this.contextNodeAllOnSameLine},n.prototype.NextNodeAllOnSameLine=function(){return void 0===this.nextNodeAllOnSameLine&&(this.nextNodeAllOnSameLine=this.NodeIsOnOneLine(this.nextTokenParent)),this.nextNodeAllOnSameLine},n.prototype.TokensAreOnSameLine=function(){if(void 0===this.tokensAreOnSameLine){var e=this.sourceFile.getLineAndCharacterOfPosition(this.currentTokenSpan.pos).line,n=this.sourceFile.getLineAndCharacterOfPosition(this.nextTokenSpan.pos).line;this.tokensAreOnSameLine=e===n}return this.tokensAreOnSameLine},n.prototype.ContextNodeBlockIsOnOneLine=function(){return void 0===this.contextNodeBlockIsOnOneLine&&(this.contextNodeBlockIsOnOneLine=this.BlockIsOnOneLine(this.contextNode)),this.contextNodeBlockIsOnOneLine},n.prototype.NextNodeBlockIsOnOneLine=function(){return void 0===this.nextNodeBlockIsOnOneLine&&(this.nextNodeBlockIsOnOneLine=this.BlockIsOnOneLine(this.nextTokenParent)),this.nextNodeBlockIsOnOneLine},n.prototype.NodeIsOnOneLine=function(e){var n=this.sourceFile.getLineAndCharacterOfPosition(e.getStart(this.sourceFile)).line,t=this.sourceFile.getLineAndCharacterOfPosition(e.getEnd()).line;return n===t},n.prototype.BlockIsOnOneLine=function(n){var t=e.findChildOfKind(n,18,this.sourceFile),r=e.findChildOfKind(n,19,this.sourceFile);if(t&&r){var i=this.sourceFile.getLineAndCharacterOfPosition(t.getEnd()).line,a=this.sourceFile.getLineAndCharacterOfPosition(r.getStart(this.sourceFile)).line;return i===a}return!1},n}();n.FormattingContext=t})(e.formatting||(e.formatting={}))}(d||(d={})),function(e){(function(n){var t,r=e.createScanner(99,!1,0),i=e.createScanner(99,!1,1);function a(t,a,o,s,c){var l=1===a?i:r;l.setText(t),l.setTextPos(o);var u,d,p,f,m,_=!0,g=c({advance:y,readTokenInfo:L,readEOFTokenRange:A,isOnToken:C,isOnEOF:I,getCurrentLeadingTrivia:function(){return u},lastTrailingTriviaWasNewLine:function(){return _},skipToEndOf:k,skipToStartOf:R});return m=void 0,l.setText(void 0),g;function y(){m=void 0;var n=l.getStartPos()!==o;n?_=!!d&&4===e.last(d).kind:l.scan(),u=void 0,d=void 0;var t=l.getStartPos();while(t<s){var r=l.getToken();if(!e.isTrivia(r))break;l.scan();var i={pos:t,end:l.getStartPos(),kind:r};t=l.getStartPos(),u=e.append(u,i)}p=l.getStartPos()}function h(e){switch(e.kind){case 33:case 71:case 72:case 49:case 48:return!0}return!1}function v(n){if(n.parent)switch(n.parent.kind){case 283:case 278:case 279:case 277:return e.isKeyword(n.kind)||79===n.kind}return!1}function b(n){return e.isJsxText(n)}function E(e){return 13===e.kind}function T(e){return 16===e.kind||17===e.kind}function S(n){return n.parent&&e.isJsxAttribute(n.parent)&&n.parent.initializer===n}function x(e){return 43===e||68===e}function L(t){e.Debug.assert(C());var r=h(t)?1:E(t)?2:T(t)?3:v(t)?4:b(t)?5:S(t)?6:0;if(m&&r===f)return N(m,t);l.getStartPos()!==p&&(e.Debug.assert(void 0!==m),l.setTextPos(p),l.scan());var i=D(t,r),a=n.createTextRangeWithKind(l.getStartPos(),l.getTextPos(),i);d&&(d=void 0);while(l.getStartPos()<s){if(i=l.scan(),!e.isTrivia(i))break;var o=n.createTextRangeWithKind(l.getStartPos(),l.getTextPos(),i);if(d||(d=[]),d.push(o),4===i){l.scan();break}}return m={leadingTrivia:u,trailingTrivia:d,token:a},N(m,t)}function D(n,t){var r=l.getToken();switch(f=0,t){case 1:if(31===r){f=1;var i=l.reScanGreaterToken();return e.Debug.assert(n.kind===i),i}break;case 2:if(x(r)){f=2;i=l.reScanSlashToken();return e.Debug.assert(n.kind===i),i}break;case 3:if(19===r)return f=3,l.reScanTemplateToken(!1);break;case 4:return f=4,l.scanJsxIdentifier();case 5:return f=5,l.reScanJsxToken(!1);case 6:return f=6,l.reScanJsxAttributeValue();case 0:break;default:e.Debug.assertNever(t)}return r}function A(){return e.Debug.assert(I()),n.createTextRangeWithKind(l.getStartPos(),l.getTextPos(),1)}function C(){var n=m?m.token.kind:l.getToken(),t=m?m.token.pos:l.getStartPos();return t<s&&1!==n&&!e.isTrivia(n)}function I(){var e=m?m.token.kind:l.getToken();return 1===e}function N(n,t){return e.isToken(t)&&n.token.kind!==t.kind&&(n.token.kind=t.kind),n}function k(e){l.setTextPos(e.end),p=l.getStartPos(),f=void 0,m=void 0,_=!1,u=void 0,d=void 0}function R(e){l.setTextPos(e.pos),p=l.getStartPos(),f=void 0,m=void 0,_=!1,u=void 0,d=void 0}}(function(e){e[e["Scan"]=0]="Scan",e[e["RescanGreaterThanToken"]=1]="RescanGreaterThanToken",e[e["RescanSlashToken"]=2]="RescanSlashToken",e[e["RescanTemplateToken"]=3]="RescanTemplateToken",e[e["RescanJsxIdentifier"]=4]="RescanJsxIdentifier",e[e["RescanJsxText"]=5]="RescanJsxText",e[e["RescanJsxAttributeValue"]=6]="RescanJsxAttributeValue"})(t||(t={})),n.getFormattingScanner=a})(e.formatting||(e.formatting={}))}(d||(d={})),function(e){(function(n){n.anyContext=e.emptyArray,function(e){e[e["StopProcessingSpaceActions"]=1]="StopProcessingSpaceActions",e[e["StopProcessingTokenActions"]=2]="StopProcessingTokenActions",e[e["InsertSpace"]=4]="InsertSpace",e[e["InsertNewLine"]=8]="InsertNewLine",e[e["DeleteSpace"]=16]="DeleteSpace",e[e["DeleteToken"]=32]="DeleteToken",e[e["InsertTrailingSemicolon"]=64]="InsertTrailingSemicolon",e[e["StopAction"]=3]="StopAction",e[e["ModifySpaceAction"]=28]="ModifySpaceAction",e[e["ModifyTokenAction"]=96]="ModifyTokenAction"}(n.RuleAction||(n.RuleAction={})),function(e){e[e["None"]=0]="None",e[e["CanDeleteNewLines"]=1]="CanDeleteNewLines"}(n.RuleFlags||(n.RuleFlags={}))})(e.formatting||(e.formatting={}))}(d||(d={})),function(e){(function(n){function t(){for(var t=[],o=0;o<=158;o++)1!==o&&t.push(o);function L(){for(var e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];return{tokens:t.filter(function(n){return!e.some(function(e){return e===n})}),isSpecific:!1}}var D={tokens:t,isSpecific:!1},C=i(a(a([],t,!0),[3],!1)),O=i(a(a([],t,!0),[1],!1)),F=s(81,158),G=s(29,78),ee=[101,102,158,127,138],oe=[45,46,54,53],fe=[8,9,79,20,22,18,108,103],ge=[79,20,108,103],ye=[79,21,23,103],he=[79,20,108,103],ve=[79,21,23,103],be=[2,3],Ee=a([79],e.typeKeywords,!0),Te=C,Se=i([79,3,84,93,100]),xe=i([21,3,90,111,96,91]),Le=[r("IgnoreBeforeComment",D,be,n.anyContext,1),r("IgnoreAfterLineComment",2,D,n.anyContext,1),r("NotSpaceBeforeColon",D,58,[W,y,h],16),r("SpaceAfterColon",58,D,[W,y],4),r("NoSpaceBeforeQuestionMark",D,57,[W,y,h],16),r("SpaceAfterQuestionMarkInConditionalOperator",57,D,[W,b],4),r("NoSpaceAfterQuestionMark",57,D,[W],16),r("NoSpaceBeforeDot",D,[24,28],[W],16),r("NoSpaceAfterDot",[24,28],D,[W],16),r("NoSpaceBetweenImportParenInImportType",100,20,[W,H],16),r("NoSpaceAfterUnaryPrefixOperator",oe,fe,[W,y],16),r("NoSpaceAfterUnaryPreincrementOperator",45,ge,[W],16),r("NoSpaceAfterUnaryPredecrementOperator",46,he,[W],16),r("NoSpaceBeforeUnaryPostincrementOperator",ye,45,[W,pe],16),r("NoSpaceBeforeUnaryPostdecrementOperator",ve,46,[W,pe],16),r("SpaceAfterPostincrementWhenFollowedByAdd",45,39,[W,g],4),r("SpaceAfterAddWhenFollowedByUnaryPlus",39,39,[W,g],4),r("SpaceAfterAddWhenFollowedByPreincrement",39,45,[W,g],4),r("SpaceAfterPostdecrementWhenFollowedBySubtract",46,40,[W,g],4),r("SpaceAfterSubtractWhenFollowedByUnaryMinus",40,40,[W,g],4),r("SpaceAfterSubtractWhenFollowedByPredecrement",40,46,[W,g],4),r("NoSpaceAfterCloseBrace",19,[27,26],[W],16),r("NewLineBeforeCloseBraceInBlockContext",C,19,[x],8),r("SpaceAfterCloseBrace",19,L(21),[W,w],4),r("SpaceBetweenCloseBraceAndElse",19,91,[W],4),r("SpaceBetweenCloseBraceAndWhile",19,115,[W],4),r("NoSpaceBetweenEmptyBraceBrackets",18,19,[W,M],16),r("SpaceAfterConditionalClosingParen",21,22,[P],4),r("NoSpaceBetweenFunctionKeywordAndStar",98,41,[k],16),r("SpaceAfterStarInGeneratorDeclaration",41,79,[k],4),r("SpaceAfterFunctionInFuncDecl",98,D,[I],4),r("NewLineAfterOpenBraceInBlockContext",18,D,[x],8),r("SpaceAfterGetSetInMember",[135,147],79,[I],4),r("NoSpaceBetweenYieldKeywordAndStar",125,41,[W,ue],16),r("SpaceBetweenYieldOrYieldStarAndOperand",[125,41],D,[W,ue],4),r("NoSpaceBetweenReturnAndSemicolon",105,26,[W],16),r("SpaceAfterCertainKeywords",[113,109,103,89,105,112,131],D,[W],4),r("SpaceAfterLetConstInVariableDeclaration",[119,85],D,[W,ne],4),r("NoSpaceBeforeOpenParenInFuncCall",D,20,[W,B,U],16),r("SpaceBeforeBinaryKeywordOperator",D,ee,[W,g],4),r("SpaceAfterBinaryKeywordOperator",ee,D,[W,g],4),r("SpaceAfterVoidOperator",114,D,[W,le],4),r("SpaceBetweenAsyncAndOpenParen",130,20,[K,W],4),r("SpaceBetweenAsyncAndFunctionKeyword",130,[98,79],[W],4),r("NoSpaceBetweenTagAndTemplateString",[79,21],[14,15],[W],16),r("SpaceBeforeJsxAttribute",D,79,[X,W],4),r("SpaceBeforeSlashInJsxOpeningElement",D,43,[Q,W],4),r("NoSpaceBeforeGreaterThanTokenInJsxOpeningElement",43,31,[Q,W],16),r("NoSpaceBeforeEqualInJsxAttribute",D,63,[Y,W],16),r("NoSpaceAfterEqualInJsxAttribute",63,D,[Y,W],16),r("NoSpaceAfterModuleImport",[140,144],20,[W],16),r("SpaceAfterCertainTypeScriptKeywords",[126,84,134,88,92,93,94,135,117,100,118,140,141,121,123,122,143,147,124,150,154,139,136],D,[W],4),r("SpaceBeforeCertainTypeScriptKeywords",D,[94,117,154],[W],4),r("SpaceAfterModuleName",10,18,[re],4),r("SpaceBeforeArrow",D,38,[W],4),r("SpaceAfterArrow",38,D,[W],4),r("NoSpaceAfterEllipsis",25,79,[W],16),r("NoSpaceAfterOptionalParameters",57,[21,27],[W,y],16),r("NoSpaceBetweenEmptyInterfaceBraceBrackets",18,19,[W,ie],16),r("NoSpaceBeforeOpenAngularBracket",Ee,29,[W,se],16),r("NoSpaceBetweenCloseParenAndAngularBracket",21,29,[W,se],16),r("NoSpaceAfterOpenAngularBracket",29,D,[W,se],16),r("NoSpaceBeforeCloseAngularBracket",D,31,[W,se],16),r("NoSpaceAfterCloseAngularBracket",31,[20,22,31,27],[W,se,N],16),r("SpaceBeforeAt",[21,79],59,[W],4),r("NoSpaceAfterAt",59,D,[W],16),r("SpaceAfterDecorator",D,[126,79,93,88,84,124,123,121,122,135,147,22,41],[$],4),r("NoSpaceBeforeNonNullAssertionOperator",D,53,[W,de],16),r("NoSpaceAfterNewKeywordOnConstructorSignature",103,20,[W,ae],16),r("SpaceLessThanAndNonJSXTypeAnnotation",29,29,[W],4)],De=[r("SpaceAfterConstructor",133,20,[l("insertSpaceAfterConstructor"),W],4),r("NoSpaceAfterConstructor",133,20,[d("insertSpaceAfterConstructor"),W],16),r("SpaceAfterComma",27,D,[l("insertSpaceAfterCommaDelimiter"),W,J,V,j],4),r("NoSpaceAfterComma",27,D,[d("insertSpaceAfterCommaDelimiter"),W,J],16),r("SpaceAfterAnonymousFunctionKeyword",[98,41],20,[l("insertSpaceAfterFunctionKeywordForAnonymousFunctions"),I],4),r("NoSpaceAfterAnonymousFunctionKeyword",[98,41],20,[d("insertSpaceAfterFunctionKeywordForAnonymousFunctions"),I],16),r("SpaceAfterKeywordInControl",F,20,[l("insertSpaceAfterKeywordsInControlFlowStatements"),P],4),r("NoSpaceAfterKeywordInControl",F,20,[d("insertSpaceAfterKeywordsInControlFlowStatements"),P],16),r("SpaceAfterOpenParen",20,D,[l("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"),W],4),r("SpaceBeforeCloseParen",D,21,[l("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"),W],4),r("SpaceBetweenOpenParens",20,20,[l("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"),W],4),r("NoSpaceBetweenParens",20,21,[W],16),r("NoSpaceAfterOpenParen",20,D,[d("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"),W],16),r("NoSpaceBeforeCloseParen",D,21,[d("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"),W],16),r("SpaceAfterOpenBracket",22,D,[l("insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets"),W],4),r("SpaceBeforeCloseBracket",D,23,[l("insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets"),W],4),r("NoSpaceBetweenBrackets",22,23,[W],16),r("NoSpaceAfterOpenBracket",22,D,[d("insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets"),W],16),r("NoSpaceBeforeCloseBracket",D,23,[d("insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets"),W],16),r("SpaceAfterOpenBrace",18,D,[f("insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces"),T],4),r("SpaceBeforeCloseBrace",D,19,[f("insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces"),T],4),r("NoSpaceBetweenEmptyBraceBrackets",18,19,[W,M],16),r("NoSpaceAfterOpenBrace",18,D,[u("insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces"),W],16),r("NoSpaceBeforeCloseBrace",D,19,[u("insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces"),W],16),r("SpaceBetweenEmptyBraceBrackets",18,19,[l("insertSpaceAfterOpeningAndBeforeClosingEmptyBraces")],4),r("NoSpaceBetweenEmptyBraceBrackets",18,19,[u("insertSpaceAfterOpeningAndBeforeClosingEmptyBraces"),W],16),r("SpaceAfterTemplateHeadAndMiddle",[15,16],D,[l("insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces"),z],4,1),r("SpaceBeforeTemplateMiddleAndTail",D,[16,17],[l("insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces"),W],4),r("NoSpaceAfterTemplateHeadAndMiddle",[15,16],D,[d("insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces"),z],16,1),r("NoSpaceBeforeTemplateMiddleAndTail",D,[16,17],[d("insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces"),W],16),r("SpaceAfterOpenBraceInJsxExpression",18,D,[l("insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces"),W,q],4),r("SpaceBeforeCloseBraceInJsxExpression",D,19,[l("insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces"),W,q],4),r("NoSpaceAfterOpenBraceInJsxExpression",18,D,[d("insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces"),W,q],16),r("NoSpaceBeforeCloseBraceInJsxExpression",D,19,[d("insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces"),W,q],16),r("SpaceAfterSemicolonInFor",26,D,[l("insertSpaceAfterSemicolonInForStatements"),W,m],4),r("NoSpaceAfterSemicolonInFor",26,D,[d("insertSpaceAfterSemicolonInForStatements"),W,m],16),r("SpaceBeforeBinaryOperator",D,G,[l("insertSpaceBeforeAndAfterBinaryOperators"),W,g],4),r("SpaceAfterBinaryOperator",G,D,[l("insertSpaceBeforeAndAfterBinaryOperators"),W,g],4),r("NoSpaceBeforeBinaryOperator",D,G,[d("insertSpaceBeforeAndAfterBinaryOperators"),W,g],16),r("NoSpaceAfterBinaryOperator",G,D,[d("insertSpaceBeforeAndAfterBinaryOperators"),W,g],16),r("SpaceBeforeOpenParenInFuncDecl",D,20,[l("insertSpaceBeforeFunctionParenthesis"),W,I],4),r("NoSpaceBeforeOpenParenInFuncDecl",D,20,[d("insertSpaceBeforeFunctionParenthesis"),W,I],16),r("NewLineBeforeOpenBraceInControl",xe,18,[l("placeOpenBraceOnNewLineForControlBlocks"),P,S],8,1),r("NewLineBeforeOpenBraceInFunction",Te,18,[l("placeOpenBraceOnNewLineForFunctions"),I,S],8,1),r("NewLineBeforeOpenBraceInTypeScriptDeclWithBlock",Se,18,[l("placeOpenBraceOnNewLineForFunctions"),R,S],8,1),r("SpaceAfterTypeAssertion",31,D,[l("insertSpaceAfterTypeAssertion"),W,ce],4),r("NoSpaceAfterTypeAssertion",31,D,[d("insertSpaceAfterTypeAssertion"),W,ce],16),r("SpaceBeforeTypeAnnotation",D,[57,58],[l("insertSpaceBeforeTypeAnnotation"),W,v],4),r("NoSpaceBeforeTypeAnnotation",D,[57,58],[d("insertSpaceBeforeTypeAnnotation"),W,v],16),r("NoOptionalSemicolon",26,O,[c("semicolons",e.SemicolonPreference.Remove),me],32),r("OptionalSemicolon",D,O,[c("semicolons",e.SemicolonPreference.Insert),_e],64)],Ae=[r("NoSpaceBeforeSemicolon",D,26,[W],16),r("SpaceBeforeOpenBraceInControl",xe,18,[p("placeOpenBraceOnNewLineForControlBlocks"),P,te,E],4,1),r("SpaceBeforeOpenBraceInFunction",Te,18,[p("placeOpenBraceOnNewLineForFunctions"),I,A,te,E],4,1),r("SpaceBeforeOpenBraceInTypeScriptDeclWithBlock",Se,18,[p("placeOpenBraceOnNewLineForFunctions"),R,te,E],4,1),r("NoSpaceBeforeComma",D,27,[W],16),r("NoSpaceBeforeOpenBracket",L(130,82),22,[W],16),r("NoSpaceAfterCloseBracket",23,D,[W,Z],16),r("SpaceAfterSemicolon",26,D,[W],4),r("SpaceBetweenForAndAwaitKeyword",97,131,[W],4),r("SpaceBetweenStatements",[21,90,91,82],D,[W,J,_],4),r("SpaceAfterTryCatchFinally",[111,83,96],18,[W],4)];return a(a(a([],Le,!0),De,!0),Ae,!0)}function r(e,n,t,r,i,a){return void 0===a&&(a=0),{leftTokenRange:o(n),rightTokenRange:o(t),rule:{debugName:e,context:r,action:i,flags:a}}}function i(e){return{tokens:e,isSpecific:!0}}function o(n){return"number"===typeof n?i([n]):e.isArray(n)?i(n):n}function s(n,t,r){void 0===r&&(r=[]);for(var a=[],o=n;o<=t;o++)e.contains(r,o)||a.push(o);return i(a)}function c(e,n){return function(t){return t.options&&t.options[e]===n}}function l(e){return function(n){return n.options&&n.options.hasOwnProperty(e)&&!!n.options[e]}}function u(e){return function(n){return n.options&&n.options.hasOwnProperty(e)&&!n.options[e]}}function d(e){return function(n){return!n.options||!n.options.hasOwnProperty(e)||!n.options[e]}}function p(e){return function(n){return!n.options||!n.options.hasOwnProperty(e)||!n.options[e]||n.TokensAreOnSameLine()}}function f(e){return function(n){return!n.options||!n.options.hasOwnProperty(e)||!!n.options[e]}}function m(e){return 240===e.contextNode.kind}function _(e){return!m(e)}function g(e){switch(e.contextNode.kind){case 219:return 27!==e.contextNode.operatorToken.kind;case 220:case 187:case 227:case 273:case 268:case 175:case 185:case 186:return!0;case 201:case 257:case 263:case 269:case 252:case 162:case 294:case 165:case 164:return 63===e.currentTokenSpan.kind||63===e.nextTokenSpan.kind;case 241:case 161:return 101===e.currentTokenSpan.kind||101===e.nextTokenSpan.kind||63===e.currentTokenSpan.kind||63===e.nextTokenSpan.kind;case 242:return 158===e.currentTokenSpan.kind||158===e.nextTokenSpan.kind}return!1}function y(e){return!g(e)}function h(e){return!v(e)}function v(n){var t=n.contextNode.kind;return 165===t||164===t||162===t||252===t||e.isFunctionLikeKind(t)}function b(e){return 220===e.contextNode.kind||187===e.contextNode.kind}function E(e){return e.TokensAreOnSameLine()||A(e)}function T(e){return 199===e.contextNode.kind||193===e.contextNode.kind||L(e)}function S(e){return A(e)&&!(e.NextNodeAllOnSameLine()||e.NextNodeBlockIsOnOneLine())}function x(e){return D(e)&&!(e.ContextNodeAllOnSameLine()||e.ContextNodeBlockIsOnOneLine())}function L(e){return D(e)&&(e.ContextNodeAllOnSameLine()||e.ContextNodeBlockIsOnOneLine())}function D(e){return C(e.contextNode)}function A(e){return C(e.nextTokenParent)}function C(e){if(O(e))return!0;switch(e.kind){case 233:case 261:case 203:case 260:return!0}return!1}function I(e){switch(e.contextNode.kind){case 254:case 167:case 166:case 170:case 171:case 172:case 211:case 169:case 212:case 256:return!0}return!1}function N(e){return!I(e)}function k(e){return 254===e.contextNode.kind||211===e.contextNode.kind}function R(e){return O(e.contextNode)}function O(e){switch(e.kind){case 255:case 224:case 256:case 258:case 180:case 259:case 270:case 271:case 264:case 267:return!0}return!1}function w(e){switch(e.currentTokenParent.kind){case 255:case 259:case 258:case 290:case 260:case 247:return!0;case 233:var n=e.currentTokenParent.parent;if(!n||212!==n.kind&&211!==n.kind)return!0}return!1}function P(e){switch(e.contextNode.kind){case 237:case 247:case 240:case 241:case 242:case 239:case 250:case 238:case 246:case 290:return!0;default:return!1}}function M(e){return 203===e.contextNode.kind}function F(e){return 206===e.contextNode.kind}function G(e){return 207===e.contextNode.kind}function B(e){return F(e)||G(e)}function U(e){return 27!==e.currentTokenSpan.kind}function V(e){return 23!==e.nextTokenSpan.kind}function j(e){return 21!==e.nextTokenSpan.kind}function K(e){return 212===e.contextNode.kind}function H(e){return 198===e.contextNode.kind}function W(e){return e.TokensAreOnSameLine()&&11!==e.contextNode.kind}function z(e){return 11!==e.contextNode.kind}function J(e){return 276!==e.contextNode.kind&&280!==e.contextNode.kind}function q(e){return 286===e.contextNode.kind||285===e.contextNode.kind}function X(e){return 283===e.nextTokenParent.kind}function Y(e){return 283===e.contextNode.kind}function Q(e){return 277===e.contextNode.kind}function Z(e){return!I(e)&&!A(e)}function $(e){return e.TokensAreOnSameLine()&&!!e.contextNode.decorators&&ee(e.currentTokenParent)&&!ee(e.nextTokenParent)}function ee(n){while(e.isExpressionNode(n))n=n.parent;return 163===n.kind}function ne(e){return 253===e.currentTokenParent.kind&&e.currentTokenParent.getStart(e.sourceFile)===e.currentTokenSpan.pos}function te(e){return 2!==e.formattingRequestKind}function re(e){return 259===e.contextNode.kind}function ie(e){return 180===e.contextNode.kind}function ae(e){return 173===e.contextNode.kind}function oe(e,n){if(29!==e.kind&&31!==e.kind)return!1;switch(n.kind){case 176:case 209:case 257:case 255:case 224:case 256:case 254:case 211:case 212:case 167:case 166:case 172:case 173:case 206:case 207:case 226:return!0;default:return!1}}function se(e){return oe(e.currentTokenSpan,e.currentTokenParent)||oe(e.nextTokenSpan,e.nextTokenParent)}function ce(e){return 209===e.contextNode.kind}function le(e){return 114===e.currentTokenSpan.kind&&215===e.currentTokenParent.kind}function ue(e){return 222===e.contextNode.kind&&void 0!==e.contextNode.expression}function de(e){return 228===e.contextNode.kind}function pe(e){return!fe(e)}function fe(e){switch(e.contextNode.kind){case 237:case 240:case 241:case 242:case 238:case 239:return!0;default:return!1}}function me(n){var t=n.nextTokenSpan.kind,r=n.nextTokenSpan.pos;if(e.isTrivia(t)){var i=n.nextTokenParent===n.currentTokenParent?e.findNextToken(n.currentTokenParent,e.findAncestor(n.currentTokenParent,function(e){return!e.parent}),n.sourceFile):n.nextTokenParent.getFirstToken(n.sourceFile);if(!i)return!0;t=i.kind,r=i.getStart(n.sourceFile)}var a=n.sourceFile.getLineAndCharacterOfPosition(n.currentTokenSpan.pos).line,o=n.sourceFile.getLineAndCharacterOfPosition(r).line;return a===o?19===t||1===t:232!==t&&26!==t&&(256===n.contextNode.kind||257===n.contextNode.kind?!e.isPropertySignature(n.currentTokenParent)||!!n.currentTokenParent.type||20!==t:e.isPropertyDeclaration(n.currentTokenParent)?!n.currentTokenParent.initializer:240!==n.currentTokenParent.kind&&234!==n.currentTokenParent.kind&&232!==n.currentTokenParent.kind&&22!==t&&20!==t&&39!==t&&40!==t&&43!==t&&13!==t&&27!==t&&221!==t&&15!==t&&14!==t&&24!==t)}function _e(n){return e.positionIsASICandidate(n.currentTokenSpan.end,n.currentTokenParent,n.sourceFile)}n.getAllRules=t})(e.formatting||(e.formatting={}))}(d||(d={})),function(e){(function(n){function t(e,n){return{options:e,getRules:i(),host:n}}var r;function i(){return void 0===r&&(r=o(n.getAllRules())),r}function a(e){var n=0;return 1&e&&(n|=28),2&e&&(n|=96),28&e&&(n|=28),96&e&&(n|=96),n}function o(n){var t=s(n);return function(n){var r=t[c(n.currentTokenSpan.kind,n.nextTokenSpan.kind)];if(r){for(var i=[],o=0,s=0,l=r;s<l.length;s++){var u=l[s],d=~a(o);u.action&d&&e.every(u.context,function(e){return e(n)})&&(i.push(u),o|=u.action)}if(i.length)return i}}}function s(e){for(var n=new Array(p*p),t=new Array(n.length),r=0,i=e;r<i.length;r++)for(var a=i[r],o=a.leftTokenRange.isSpecific&&a.rightTokenRange.isSpecific,s=0,l=a.leftTokenRange.tokens;s<l.length;s++)for(var u=l[s],d=0,m=a.rightTokenRange.tokens;d<m.length;d++){var _=m[d],g=c(u,_),y=n[g];void 0===y&&(y=n[g]=[]),f(y,a.rule,o,t,g)}return n}function c(n,t){return e.Debug.assert(n<=158&&t<=158,"Must compute formatting context from tokens"),n*p+t}n.getFormatContext=t;var l,u=5,d=31,p=159;function f(e,t,r,i,a){var o=3&t.action?r?l.StopRulesSpecific:l.StopRulesAny:t.context!==n.anyContext?r?l.ContextRulesSpecific:l.ContextRulesAny:r?l.NoContextRulesSpecific:l.NoContextRulesAny,s=i[a]||0;e.splice(m(s,o),0,t),i[a]=_(s,o)}function m(e,n){for(var t=0,r=0;r<=n;r+=u)t+=e&d,e>>=u;return t}function _(n,t){var r=1+(n>>t&d);return e.Debug.assert((r&d)===r,"Adding more rules into the sub-bucket than allowed. Maximum allowed is 32 rules."),n&~(d<<t)|r<<t}(function(e){e[e["StopRulesSpecific"]=0]="StopRulesSpecific",e[e["StopRulesAny"]=1*u]="StopRulesAny",e[e["ContextRulesSpecific"]=2*u]="ContextRulesSpecific",e[e["ContextRulesAny"]=3*u]="ContextRulesAny",e[e["NoContextRulesSpecific"]=4*u]="NoContextRulesSpecific",e[e["NoContextRulesAny"]=5*u]="NoContextRulesAny"})(l||(l={}))})(e.formatting||(e.formatting={}))}(d||(d={})),function(e){(function(n){function t(n,t,r){var i={pos:n,end:t,kind:r};return e.Debug.isDebugging&&Object.defineProperty(i,"__debugKind",{get:function(){return e.Debug.formatSyntaxKind(r)}}),i}var r,i,a,o,s;function c(n,t,r){var i=t.getLineAndCharacterOfPosition(n).line;if(0===i)return[];var a=e.getEndLinePosition(i,t);while(e.isWhiteSpaceSingleLine(t.text.charCodeAt(a)))a--;e.isLineBreak(t.text.charCodeAt(a))&&a--;var o={pos:e.getStartPositionOfLine(i-1,t),end:a+1};return S(o,t,r,2)}function l(e,n,t){var r=m(e,26,n);return T(_(r),n,t,3)}function u(n,t,r){var i=m(n,18,t);if(!i)return[];var a=i.parent,o=_(a),s={pos:e.getLineStartPositionForPosition(o.getStart(t),t),end:n};return S(s,t,r,4)}function d(e,n,t){var r=m(e,19,n);return T(_(r),n,t,5)}function p(e,n){var t={pos:0,end:e.text.length};return S(t,e,n,0)}function f(n,t,r,i){var a={pos:e.getLineStartPositionForPosition(n,r),end:t};return S(a,r,i,1)}function m(n,t,r){var i=e.findPrecedingToken(n,r);return i&&i.kind===t&&n===i.getEnd()?i:void 0}function _(e){var n=e;while(n&&n.parent&&n.parent.end===e.end&&!g(n.parent,n))n=n.parent;return n}function g(n,t){switch(n.kind){case 255:case 256:return e.rangeContainsRange(n.members,t);case 259:var r=n.body;return!!r&&260===r.kind&&e.rangeContainsRange(r.statements,t);case 300:case 233:case 260:return e.rangeContainsRange(n.statements,t);case 290:return e.rangeContainsRange(n.block.statements,t)}return!1}function y(n,t){return r(t);function r(i){var a=e.forEachChild(i,function(r){return e.startEndContainsRange(r.getStart(t),r.end,n)&&r});if(a){var o=r(a);if(o)return o}return i}}function h(n,t){if(!n.length)return a;var r=n.filter(function(n){return e.rangeOverlapsWithStartEnd(t,n.start,n.start+n.length)}).sort(function(e,n){return e.start-n.start});if(!r.length)return a;var i=0;return function(n){while(1){if(i>=r.length)return!1;var t=r[i];if(n.end<=t.start)return!1;if(e.startEndOverlapsWithStartEnd(n.pos,n.end,t.start,t.start+t.length))return!0;i++}};function a(){return!1}}function v(n,t,r){var i=n.getStart(r);if(i===t.pos&&n.end===t.end)return i;var a=e.findPrecedingToken(t.pos,r);return a?a.end>=t.pos?n.pos:a.end:n.pos}function b(e,t,r){var i,a=-1;while(e){var o=r.getLineAndCharacterOfPosition(e.getStart(r)).line;if(-1!==a&&o!==a)break;if(n.SmartIndenter.shouldIndentChildNode(t,e,i,r))return t.indentSize;a=o,i=e,e=e.parent}return 0}function E(e,t,r,i,a,o){var s={pos:0,end:t.text.length};return n.getFormattingScanner(t.text,r,s.pos,s.end,function(n){return x(s,e,i,a,n,o,1,function(e){return!1},t)})}function T(n,t,r,i){if(!n)return[];var a={pos:e.getLineStartPositionForPosition(n.getStart(t),t),end:n.end};return S(a,t,r,i)}function S(e,t,r,i){var a=y(e,t);return n.getFormattingScanner(t.text,t.languageVariant,v(a,e,t),e.end,function(o){return x(e,a,n.SmartIndenter.getIndentationForNode(a,e,t,r.options),b(a,r.options,t),o,r,i,h(t.parseDiagnostics,e),t)})}function x(t,r,i,a,o,s,c,l,u){var d,p,f,m,_=s.options,g=s.getRules,y=s.host,h=new n.FormattingContext(u,c,_),v=-1,b=[];if(o.advance(),o.isOnToken()){var E=u.getLineAndCharacterOfPosition(r.getStart(u)).line,T=E;r.decorators&&(T=u.getLineAndCharacterOfPosition(e.getNonDecoratorTokenPosOfNode(r,u)).line),R(r,r,E,T,i,a)}if(!o.isOnToken()){var S=n.SmartIndenter.nodeWillIndentChild(_,r,void 0,u,!1)?i+_.indentSize:i,x=o.getCurrentLeadingTrivia();x&&O(x,S,!1,function(e){return P(e,u.getLineAndCharacterOfPosition(e.pos),r,r,void 0)})}return!1!==_.trimTrailingWhitespace&&K(),b;function L(t,r,i,a,o){if(e.rangeOverlapsWithStartEnd(a,t,r)||e.rangeContainsStartEnd(a,t,r)){if(-1!==o)return o}else{var s=u.getLineAndCharacterOfPosition(t).line,c=e.getLineStartPositionForPosition(t,u),l=n.SmartIndenter.findFirstNonWhitespaceColumn(c,t,u,_);if(s!==i||t===l){var d=n.SmartIndenter.getBaseIndentation(_);return d>l?d:l}}return-1}function I(e,t,r,i,a,o){var s=n.SmartIndenter.shouldIndentChildNode(_,e)?_.indentSize:0;return o===t?{indentation:t===m?v:a.getIndentation(),delta:Math.min(_.indentSize,a.getDelta(e)+s)}:-1===r?20===e.kind&&t===m?{indentation:v,delta:a.getDelta(e)}:n.SmartIndenter.childStartsOnTheSameLineWithElseInIfStatement(i,e,t,u)||n.SmartIndenter.childIsUnindentedBranchOfConditionalExpression(i,e,t,u)||n.SmartIndenter.argumentStartsOnSameLineAsPreviousArgument(i,e,t,u)?{indentation:a.getIndentation(),delta:s}:{indentation:a.getIndentation()+a.getDelta(e),delta:s}:{indentation:r,delta:s}}function N(n){if(n.modifiers&&n.modifiers.length)return n.modifiers[0].kind;switch(n.kind){case 255:return 84;case 256:return 118;case 254:return 98;case 258:return 258;case 170:return 135;case 171:return 147;case 167:if(n.asteriskToken)return 41;case 165:case 162:var t=e.getNameOfDeclaration(n);if(t)return t.kind}}function k(e,t,r,i){return{getIndentationForComment:function(e,n,t){switch(e){case 19:case 23:case 21:return r+o(t)}return-1!==n?n:r},getIndentationForToken:function(e,n,t,i){return!i&&a(e,n,t)?r+o(t):r},getIndentation:function(){return r},getDelta:o,recomputeIndentation:function(t,a){n.SmartIndenter.shouldIndentChildNode(_,a,e,u)&&(r+=t?_.indentSize:-_.indentSize,i=n.SmartIndenter.shouldIndentChildNode(_,e)?_.indentSize:0)}};function a(n,r,i){switch(r){case 18:case 19:case 21:case 91:case 115:case 59:return!1;case 43:case 31:switch(i.kind){case 278:case 279:case 277:case 226:return!1}break;case 22:case 23:if(193!==i.kind)return!1;break}return t!==n&&!(e.decorators&&r===N(e))}function o(t){return n.SmartIndenter.nodeWillIndentChild(_,e,t,u,!0)?i:0}}function R(r,i,a,s,c,g){if(e.rangeOverlapsWithStartEnd(t,r.getStart(u),r.getEnd())){var y=k(r,a,c,g),h=i;e.forEachChild(r,function(e){T(e,-1,r,y,a,s,!1)},function(e){S(e,r,a,y)});while(o.isOnToken()){var b=o.readTokenInfo(r);if(b.token.end>r.end)break;x(b,r,y,r)}if(!r.parent&&o.isOnEOF()){var E=o.readEOFTokenRange();E.end<=r.end&&d&&M(E,u.getLineAndCharacterOfPosition(E.pos).line,r,d,f,p,i,y)}}function T(n,i,a,s,c,l,d,p){var f=n.getStart(u),m=u.getLineAndCharacterOfPosition(f).line,_=m;n.decorators&&(_=u.getLineAndCharacterOfPosition(e.getNonDecoratorTokenPosOfNode(n,u)).line);var g=-1;if(d&&e.rangeContainsRange(t,a)&&(g=L(f,n.end,c,t,i),-1!==g&&(i=g)),!e.rangeOverlapsWithStartEnd(t,n.pos,n.end))return n.end<t.pos&&o.skipToEndOf(n),i;if(0===n.getFullWidth())return i;while(o.isOnToken()){var y=o.readTokenInfo(r);if(y.token.end>f){y.token.pos>f&&o.skipToStartOf(n);break}x(y,r,s,r)}if(!o.isOnToken())return i;if(e.isToken(n)){y=o.readTokenInfo(n);if(11!==n.kind)return e.Debug.assert(y.token.end===n.end,"Token end is child end"),x(y,r,s,n),i}var v=163===n.kind?m:l,b=I(n,m,g,r,s,v);return R(n,h,m,_,b.indentation,b.delta),h=r,p&&202===a.kind&&-1===i&&(i=b.indentation),i}function S(t,i,a,s){e.Debug.assert(e.isNodeArray(t));var c=D(i,t),l=s,d=a;if(0!==c)while(o.isOnToken()){var p=o.readTokenInfo(i);if(p.token.end>t.pos)break;if(p.token.kind===c){d=u.getLineAndCharacterOfPosition(p.token.pos).line,x(p,i,s,i);var f=void 0;if(-1!==v)f=v;else{var m=e.getLineStartPositionForPosition(p.token.pos,u);f=n.SmartIndenter.findFirstNonWhitespaceColumn(m,p.token.pos,u,_)}l=k(i,a,f,_.indentSize)}else x(p,i,s,i)}for(var g=-1,y=0;y<t.length;y++){var h=t[y];g=T(h,g,r,l,d,d,!0,0===y)}var b=A(c);if(0!==b&&o.isOnToken()){p=o.readTokenInfo(i);if(27===p.token.kind&&e.isCallLikeExpression(i)){var E=u.getLineAndCharacterOfPosition(p.token.pos).line;d!==E&&(o.advance(),p=o.isOnToken()?o.readTokenInfo(i):void 0)}p&&p.token.kind===b&&e.rangeContainsRange(i,p.token)&&x(p,i,l,i,!0)}}function x(n,r,i,a,s){e.Debug.assert(e.rangeContainsRange(r,n.token));var c=o.lastTrailingTriviaWasNewLine(),p=!1;n.leadingTrivia&&w(n.leadingTrivia,r,h,i);var f=0,_=e.rangeContainsRange(t,n.token),g=u.getLineAndCharacterOfPosition(n.token.pos);if(_){var y=l(n.token),b=d;if(f=P(n.token,g,r,h,i),!y)if(0===f){var E=b&&u.getLineAndCharacterOfPosition(b.end).line;p=c&&g.line!==E}else p=1===f}if(n.trailingTrivia&&w(n.trailingTrivia,r,h,i),p){var T=_&&!l(n.token)?i.getIndentationForToken(g.line,n.token.kind,a,!!s):-1,S=!0;if(n.leadingTrivia){var x=i.getIndentationForComment(n.token.kind,T,a);S=O(n.leadingTrivia,x,S,function(e){return F(e.pos,x,!1)})}-1!==T&&S&&(F(n.token.pos,T,1===f),m=g.line,v=T)}o.advance(),h=r}}function O(n,r,i,a){for(var o=0,s=n;o<s.length;o++){var c=s[o],l=e.rangeContainsRange(t,c);switch(c.kind){case 3:l&&U(c,r,!i),i=!1;break;case 2:i&&l&&a(c),i=!1;break;case 4:i=!0;break}}return i}function w(n,r,i,a){for(var o=0,s=n;o<s.length;o++){var c=s[o];if(e.isComment(c.kind)&&e.rangeContainsRange(t,c)){var l=u.getLineAndCharacterOfPosition(c.pos);P(c,l,r,i,a)}}}function P(e,n,r,i,a){var o=l(e),s=0;if(!o)if(d)s=M(e,n.line,r,d,f,p,i,a);else{var c=u.getLineAndCharacterOfPosition(t.pos);V(c.line,n.line)}return d=e,p=r,f=n.line,s}function M(n,t,r,i,a,o,s,c){h.updateContext(i,o,n,r,s);var l=g(h),d=!1!==h.options.trimTrailingWhitespace,p=0;return l?e.forEachRight(l,function(o){switch(p=J(o,i,a,n,t),p){case 2:r.getStart(u)===n.pos&&c.recomputeIndentation(!1,s);break;case 1:r.getStart(u)===n.pos&&c.recomputeIndentation(!0,s);break;default:e.Debug.assert(0===p)}d=d&&!(16&o.action)&&1!==o.flags}):d=d&&1!==n.kind,t!==a&&d&&V(a,t,i),p}function F(n,t,r){var i=C(t,_);if(r)W(n,0,i);else{var a=u.getLineAndCharacterOfPosition(n),o=e.getStartPositionOfLine(a.line,u);(t!==G(o,a.character)||B(i,o))&&W(o,a.character,i)}}function G(e,n){for(var t=0,r=0;r<n;r++)9===u.text.charCodeAt(e+r)?t+=_.tabSize-t%_.tabSize:t++;return t}function B(e,n){return e!==u.text.substr(n,e.length)}function U(t,r,i,a){void 0===a&&(a=!0);var o=u.getLineAndCharacterOfPosition(t.pos).line,s=u.getLineAndCharacterOfPosition(t.end).line;if(o!==s){for(var c=[],l=t.pos,d=o;d<s;d++){var p=e.getEndLinePosition(d,u);c.push({pos:l,end:p}),l=e.getStartPositionOfLine(d+1,u)}if(a&&c.push({pos:l,end:t.end}),0!==c.length){var f=e.getStartPositionOfLine(o,u),m=n.SmartIndenter.findFirstNonWhitespaceCharacterAndColumn(f,c[0].pos,u,_),g=0;i&&(g=1,o++);for(var y=r-m.column,h=g;h<c.length;h++,o++){var v=e.getStartPositionOfLine(o,u),b=0===h?m:n.SmartIndenter.findFirstNonWhitespaceCharacterAndColumn(c[h].pos,c[h].end,u,_),E=b.column+y;if(E>0){var T=C(E,_);W(v,b.character,T)}else H(v,b.character)}}}else i||F(t.pos,r,!1)}function V(n,t,r){for(var i=n;i<t;i++){var a=e.getStartPositionOfLine(i,u),o=e.getEndLinePosition(i,u);if(!(r&&(e.isComment(r.kind)||e.isStringOrRegularExpressionOrTemplateLiteral(r.kind))&&r.pos<=o&&r.end>o)){var s=j(a,o);-1!==s&&(e.Debug.assert(s===a||!e.isWhiteSpaceSingleLine(u.text.charCodeAt(s-1))),H(s,o+1-s))}}}function j(n,t){var r=t;while(r>=n&&e.isWhiteSpaceSingleLine(u.text.charCodeAt(r)))r--;return r!==t?r+1:-1}function K(){var e=d?d.end:t.pos,n=u.getLineAndCharacterOfPosition(e).line,r=u.getLineAndCharacterOfPosition(t.end).line;V(n,r+1,d)}function H(n,t){t&&b.push(e.createTextChangeFromStartLength(n,t,""))}function W(n,t,r){(t||r)&&b.push(e.createTextChangeFromStartLength(n,t,r))}function z(n,t){t&&b.push(e.createTextChangeFromStartLength(n,0,t))}function J(n,t,r,i,a){var o=a!==r;switch(n.action){case 1:return 0;case 16:if(t.end!==i.pos)return H(t.end,i.pos-t.end),o?2:0;break;case 32:H(t.pos,t.end-t.pos);break;case 8:if(1!==n.flags&&r!==a)return 0;var s=a-r;if(1!==s)return W(t.end,i.pos-t.end,e.getNewLineOrDefaultFromHost(y,_)),o?0:1;break;case 4:if(1!==n.flags&&r!==a)return 0;var c=i.pos-t.end;if(1!==c||32!==u.text.charCodeAt(t.end))return W(t.end,i.pos-t.end," "),o?2:0;break;case 64:z(t.end,";")}return 0}}function L(n,t,r,i){void 0===i&&(i=e.getTokenAtPosition(n,t));var a=e.findAncestor(i,e.isJSDoc);a&&(i=a.parent);var o=i.getStart(n);if(!(o<=t&&t<i.getEnd())){r=null===r?void 0:void 0===r?e.findPrecedingToken(t,n):r;var s=r&&e.getTrailingCommentRanges(n.text,r.end),c=e.getLeadingCommentRangesOfNode(i,n),l=e.concatenate(s,c);return l&&e.find(l,function(r){return e.rangeContainsPositionExclusive(r,t)||t===r.end&&(2===r.kind||t===n.getFullWidth())})}}function D(e,n){switch(e.kind){case 169:case 254:case 211:case 167:case 166:case 212:if(e.typeParameters===n)return 29;if(e.parameters===n)return 20;break;case 206:case 207:if(e.typeArguments===n)return 29;if(e.arguments===n)return 20;break;case 176:if(e.typeArguments===n)return 29;break;case 180:return 18}return 0}function A(e){switch(e){case 20:return 21;case 29:return 31;case 18:return 19}return 0}function C(n,t){var r=!a||a.tabSize!==t.tabSize||a.indentSize!==t.indentSize;if(r&&(a={tabSize:t.tabSize,indentSize:t.indentSize},o=s=void 0),t.convertTabsToSpaces){var i=void 0,c=Math.floor(n/t.indentSize),l=n%t.indentSize;return s||(s=[]),void 0===s[c]?(i=e.repeatString(" ",t.indentSize*c),s[c]=i):i=s[c],l?i+e.repeatString(" ",l):i}var u=Math.floor(n/t.tabSize),d=n-u*t.tabSize,p=void 0;return o||(o=[]),void 0===o[u]?o[u]=p=e.repeatString("\t",u):p=o[u],d?p+e.repeatString(" ",d):p}n.createTextRangeWithKind=t,function(e){e[e["Unknown"]=-1]="Unknown"}(r||(r={})),n.formatOnEnter=c,n.formatOnSemicolon=l,n.formatOnOpeningCurly=u,n.formatOnClosingCurly=d,n.formatDocument=p,n.formatSelection=f,n.formatNodeGivenIndentation=E,function(e){e[e["None"]=0]="None",e[e["LineAdded"]=1]="LineAdded",e[e["LineRemoved"]=2]="LineRemoved"}(i||(i={})),n.getRangeOfEnclosingComment=L,n.getIndentationString=C})(e.formatting||(e.formatting={}))}(d||(d={})),function(e){(function(n){(function(t){var r,i;function a(t,r,i,a){if(void 0===a&&(a=!1),t>r.text.length)return u(i);if(i.indentStyle===e.IndentStyle.None)return 0;var l=e.findPrecedingToken(t,r,void 0,!0),d=n.getRangeOfEnclosingComment(r,t,l||null);if(d&&3===d.kind)return o(r,t,i,d);if(!l)return u(i);var p=e.isStringOrRegularExpressionOrTemplateLiteral(l.kind);if(p&&l.getStart(r)<=t&&t<l.end)return 0;var m=r.getLineAndCharacterOfPosition(t).line;if(i.indentStyle===e.IndentStyle.Block)return s(r,t,i);if(27===l.kind&&219!==l.parent.kind){var _=f(l,r,i);if(-1!==_)return _}var g=T(t,l.parent,r);return g&&!e.rangeContainsRange(g,l)?L(g,r,i)+i.indentSize:c(r,t,l,m,a,i)}function o(n,t,r,i){var a=e.getLineAndCharacterOfPosition(n,t).line-1,o=e.getLineAndCharacterOfPosition(n,i.pos).line;if(e.Debug.assert(o>=0),a<=o)return N(e.getStartPositionOfLine(o,n),t,n,r);var s=e.getStartPositionOfLine(a,n),c=I(s,t,n,r),l=c.column,u=c.character;if(0===l)return l;var d=n.text.charCodeAt(s+u);return 42===d?l-1:l}function s(n,t,r){var i=t;while(i>0){var a=n.text.charCodeAt(i);if(!e.isWhiteSpaceLike(a))break;i--}var o=e.getLineStartPositionForPosition(i,n);return N(o,i,n,r)}function c(n,t,r,i,a,o){var s,c=r;while(c){if(e.positionBelongsToNode(c,t,n)&&O(o,c,s,n,!0)){var l=g(c,n),p=_(r,c,i,n),f=0!==p?a&&2===p?o.indentSize:0:i!==l.line?o.indentSize:0;return d(c,l,void 0,f,n,!0,o)}var m=D(c,n,o,!0);if(-1!==m)return m;s=c,c=c.parent}return u(o)}function l(e,n,t,r){var i=t.getLineAndCharacterOfPosition(e.getStart(t));return d(e,i,n,0,t,!1,r)}function u(e){return e.baseIndentSize||0}function d(e,n,t,r,i,a,o){var s,c=e.parent;while(c){var l=!0;if(t){var d=e.getStart(i);l=d<t.pos||d>t.end}var f=p(c,e,i),_=f.line===n.line||h(c,e,n.line,i);if(l){var v=null===(s=E(e,i))||void 0===s?void 0:s[0],b=!!v&&g(v,i).line>f.line,T=D(e,i,o,b);if(-1!==T)return T+r;if(T=m(e,c,n,_,i,o),-1!==T)return T+r}O(o,c,e,i,a)&&!_&&(r+=o.indentSize);var S=y(c,e,n.line,i);e=c,c=e.parent,n=S?i.getLineAndCharacterOfPosition(e.getStart(i)):f}return r+u(o)}function p(e,n,t){var r=E(n,t),i=r?r.pos:e.getStart(t);return t.getLineAndCharacterOfPosition(i)}function f(n,t,r){var i=e.findListItemInfo(n);return i&&i.listItemIndex>0?A(i.list.getChildren(),i.listItemIndex-1,t,r):-1}function m(n,t,r,i,a,o){var s=(e.isDeclaration(n)||e.isStatementButNotDeclaration(n))&&(300===t.kind||!i);return s?C(r,a,o):-1}function _(n,t,r,i){var a=e.findNextToken(n,t,i);if(!a)return 0;if(18===a.kind)return 1;if(19===a.kind){var o=g(a,i).line;return r===o?2:0}return 0}function g(e,n){return n.getLineAndCharacterOfPosition(e.getStart(n))}function y(n,t,r,i){if(!e.isCallExpression(n)||!e.contains(n.arguments,t))return!1;var a=n.expression.getEnd(),o=e.getLineAndCharacterOfPosition(i,a).line;return o===r}function h(n,t,r,i){if(237===n.kind&&n.elseStatement===t){var a=e.findChildOfKind(n,91,i);e.Debug.assert(void 0!==a);var o=g(a,i).line;return o===r}return!1}function v(n,t,r,i){if(e.isConditionalExpression(n)&&(t===n.whenTrue||t===n.whenFalse)){var a=e.getLineAndCharacterOfPosition(i,n.condition.end).line;if(t===n.whenTrue)return r===a;var o=g(n.whenTrue,i).line,s=e.getLineAndCharacterOfPosition(i,n.whenTrue.end).line;return a===o&&s===r}return!1}function b(n,t,r,i){if(e.isCallOrNewExpression(n)){if(!n.arguments)return!1;var a=e.find(n.arguments,function(e){return e.pos===t.pos});if(!a)return!1;var o=n.arguments.indexOf(a);if(0===o)return!1;var s=n.arguments[o-1],c=e.getLineAndCharacterOfPosition(i,s.getEnd()).line;if(r===c)return!0}return!1}function E(e,n){return e.parent&&S(e.getStart(n),e.getEnd(),e.parent,n)}function T(e,n,t){return n&&S(e,e,n,t)}function S(n,t,r,i){switch(r.kind){case 176:return a(r.typeArguments);case 203:return a(r.properties);case 202:return a(r.elements);case 180:return a(r.members);case 254:case 211:case 212:case 167:case 166:case 172:case 169:case 178:case 173:return a(r.typeParameters)||a(r.parameters);case 255:case 224:case 256:case 257:case 339:return a(r.typeParameters);case 207:case 206:return a(r.typeArguments)||a(r.arguments);case 253:return a(r.declarations);case 267:case 271:return a(r.elements);case 199:case 200:return a(r.elements)}function a(a){return a&&e.rangeContainsStartEnd(x(r,a,i),n,t)?a:void 0}}function x(e,n,t){for(var r=e.getChildren(t),i=1;i<r.length-1;i++)if(r[i].pos===n.pos&&r[i].end===n.end)return{pos:r[i-1].end,end:r[i+1].getStart(t)};return n}function L(e,n,t){return e?C(n.getLineAndCharacterOfPosition(e.pos),n,t):-1}function D(e,n,t,r){if(e.parent&&253===e.parent.kind)return-1;var i=E(e,n);if(i){var a=i.indexOf(e);if(-1!==a){var o=A(i,a,n,t);if(-1!==o)return o}return L(i,n,t)+(r?t.indentSize:0)}return-1}function A(n,t,r,i){e.Debug.assert(t>=0&&t<n.length);for(var a=n[t],o=g(a,r),s=t-1;s>=0;s--)if(27!==n[s].kind){var c=r.getLineAndCharacterOfPosition(n[s].end).line;if(c!==o.line)return C(o,r,i);o=g(n[s],r)}return-1}function C(e,n,t){var r=n.getPositionOfLineAndCharacter(e.line,0);return N(r,r+e.character,n,t)}function I(n,t,r,i){for(var a=0,o=0,s=n;s<t;s++){var c=r.text.charCodeAt(s);if(!e.isWhiteSpaceSingleLine(c))break;9===c?o+=i.tabSize+o%i.tabSize:o++,a++}return{column:o,character:a}}function N(e,n,t,r){return I(e,n,t,r).column}function k(e,n,t,r,i){var a=t?t.kind:0;switch(n.kind){case 236:case 255:case 224:case 256:case 258:case 257:case 202:case 233:case 260:case 203:case 180:case 193:case 182:case 261:case 288:case 287:case 210:case 204:case 206:case 207:case 235:case 269:case 245:case 220:case 200:case 199:case 278:case 281:case 277:case 286:case 166:case 172:case 173:case 162:case 177:case 178:case 189:case 208:case 216:case 271:case 267:case 273:case 268:case 165:return!0;case 252:case 291:case 219:if(!e.indentMultiLineObjectLiteralBeginningOnBlankLine&&r&&203===a)return w(r,t);if(219!==n.kind)return!0;break;case 238:case 239:case 241:case 242:case 240:case 237:case 254:case 211:case 167:case 169:case 170:case 171:return 233!==a;case 212:return r&&210===a?w(r,t):233!==a;case 270:return 271!==a;case 264:return 265!==a||!!t.namedBindings&&267!==t.namedBindings.kind;case 276:return 279!==a;case 280:return 282!==a;case 186:case 185:if(180===a||182===a)return!1;break}return i}function R(e,n){switch(e){case 245:case 249:case 243:case 244:return 233!==n.kind;default:return!1}}function O(e,n,t,r,i){return void 0===i&&(i=!1),k(e,n,t,r,!1)&&!(i&&t&&R(t.kind,n))}function w(n,t){var r=e.skipTrivia(n.text,t.pos),i=n.getLineAndCharacterOfPosition(r).line,a=n.getLineAndCharacterOfPosition(t.end).line;return i===a}(function(e){e[e["Unknown"]=-1]="Unknown"})(r||(r={})),t.getIndentation=a,t.getIndentationForNode=l,t.getBaseIndentation=u,function(e){e[e["Unknown"]=0]="Unknown",e[e["OpenBrace"]=1]="OpenBrace",e[e["CloseBrace"]=2]="CloseBrace"}(i||(i={})),t.isArgumentAndStartLineOverlapsExpressionBeingCalled=y,t.childStartsOnTheSameLineWithElseInIfStatement=h,t.childIsUnindentedBranchOfConditionalExpression=v,t.argumentStartsOnSameLineAsPreviousArgument=b,t.getContainingList=E,t.findFirstNonWhitespaceCharacterAndColumn=I,t.findFirstNonWhitespaceColumn=N,t.nodeWillIndentChild=k,t.shouldIndentChildNode=O})(n.SmartIndenter||(n.SmartIndenter={}))})(e.formatting||(e.formatting={}))}(d||(d={})),function(e){(function(n){function t(n){var t=n.__pos;return e.Debug.assert("number"===typeof t),t}function r(n,t){e.Debug.assert("number"===typeof t),n.__pos=t}function i(n){var t=n.__end;return e.Debug.assert("number"===typeof t),t}function s(n,t){e.Debug.assert("number"===typeof t),n.__end=t}var c,l;function u(n,t){return e.skipTrivia(n,t,!1,!0)}function d(n,t){var r=t;while(r<n.length){var i=n.charCodeAt(r);if(!e.isWhiteSpaceSingleLine(i))return 47===i;r++}return!1}(function(e){e[e["Exclude"]=0]="Exclude",e[e["IncludeAll"]=1]="IncludeAll",e[e["JSDoc"]=2]="JSDoc",e[e["StartLine"]=3]="StartLine"})(c=n.LeadingTriviaOption||(n.LeadingTriviaOption={})),function(e){e[e["Exclude"]=0]="Exclude",e[e["ExcludeWhitespace"]=1]="ExcludeWhitespace",e[e["Include"]=2]="Include"}(l=n.TrailingTriviaOption||(n.TrailingTriviaOption={}));var p,f={leadingTriviaOption:c.Exclude,trailingTriviaOption:l.Exclude};function m(e,n,t,r){return{pos:_(e,n,r),end:y(e,t,r)}}function _(n,t,r,i){var a,o;void 0===i&&(i=!1);var s=r.leadingTriviaOption;if(s===c.Exclude)return t.getStart(n);if(s===c.StartLine){var l=t.getStart(n),d=e.getLineStartPositionForPosition(l,n);return e.rangeContainsPosition(t,d)?d:l}if(s===c.JSDoc){var p=e.getJSDocCommentRanges(t,n.text);if(null===p||void 0===p?void 0:p.length)return e.getLineStartPositionForPosition(p[0].pos,n)}var f=t.getFullStart(),m=t.getStart(n);if(f===m)return m;var _=e.getLineStartPositionForPosition(f,n),g=e.getLineStartPositionForPosition(m,n);if(g===_)return s===c.IncludeAll?f:m;if(i){var y=(null===(a=e.getLeadingCommentRanges(n.text,f))||void 0===a?void 0:a[0])||(null===(o=e.getTrailingCommentRanges(n.text,f))||void 0===o?void 0:o[0]);if(y)return e.skipTrivia(n.text,y.end,!0,!0)}var h=f>0?1:0,v=e.getStartPositionOfLine(e.getLineOfLocalPosition(n,_)+h,n);return v=u(n.text,v),e.getStartPositionOfLine(e.getLineOfLocalPosition(n,v),n)}function g(n,t,r){var i=t.end,a=r.trailingTriviaOption;if(a===l.Include){var o=e.getTrailingCommentRanges(n.text,i);if(o)for(var s=e.getLineOfLocalPosition(n,t.end),c=0,u=o;c<u.length;c++){var d=u[c];if(2===d.kind||e.getLineOfLocalPosition(n,d.pos)>s)break;var p=e.getLineOfLocalPosition(n,d.end);if(p>s)return e.skipTrivia(n.text,d.end,!0,!0)}}}function y(n,t,r){var i,a=t.end,o=r.trailingTriviaOption;if(o===l.Exclude)return a;if(o===l.ExcludeWhitespace){var s=e.concatenate(e.getTrailingCommentRanges(n.text,a),e.getLeadingCommentRanges(n.text,a)),c=null===(i=null===s||void 0===s?void 0:s[s.length-1])||void 0===i?void 0:i.end;return c||a}var u=g(n,t,r);if(u)return u;var d=e.skipTrivia(n.text,a,!0);return d===a||o!==l.Include&&!e.isLineBreak(n.text.charCodeAt(d-1))?a:d}function h(e,n){return!!n&&!!e.parent&&(27===n.kind||26===n.kind&&203===e.parent.kind)}function v(n){return e.isFunctionExpression(n)||e.isFunctionDeclaration(n)}(function(e){e[e["Remove"]=0]="Remove",e[e["ReplaceWithSingleNode"]=1]="ReplaceWithSingleNode",e[e["ReplaceWithMultipleNodes"]=2]="ReplaceWithMultipleNodes",e[e["Text"]=3]="Text"})(p||(p={})),n.isThisTypeAnnotatable=v;var b,E,T=function(){function n(n,t){this.newLineCharacter=n,this.formatContext=t,this.changes=[],this.newFiles=[],this.classesWithNodesInsertedAtStart=new e.Map,this.deletedNodes=[]}return n.fromContext=function(t){return new n(e.getNewLineOrDefaultFromHost(t.host,t.formatContext.options),t.formatContext)},n.with=function(e,t){var r=n.fromContext(e);return t(r),r.getChanges()},n.prototype.pushRaw=function(n,t){e.Debug.assertEqual(n.fileName,t.fileName);for(var r=0,i=t.textChanges;r<i.length;r++){var a=i[r];this.changes.push({kind:p.Text,sourceFile:n,text:a.newText,range:e.createTextRangeFromSpan(a.span)})}},n.prototype.deleteRange=function(e,n){this.changes.push({kind:p.Remove,sourceFile:e,range:n})},n.prototype.delete=function(e,n){this.deletedNodes.push({sourceFile:e,node:n})},n.prototype.deleteNode=function(e,n,t){void 0===t&&(t={leadingTriviaOption:c.IncludeAll}),this.deleteRange(e,m(e,n,n,t))},n.prototype.deleteNodes=function(e,n,t,r){void 0===t&&(t={leadingTriviaOption:c.IncludeAll});for(var i=0,a=n;i<a.length;i++){var o=a[i],s=_(e,o,t,r),l=y(e,o,t);this.deleteRange(e,{pos:s,end:l}),r=!!g(e,o,t)}},n.prototype.deleteModifier=function(n,t){this.deleteRange(n,{pos:t.getStart(n),end:e.skipTrivia(n.text,t.end,!0)})},n.prototype.deleteNodeRange=function(e,n,t,r){void 0===r&&(r={leadingTriviaOption:c.IncludeAll});var i=_(e,n,r),a=y(e,t,r);this.deleteRange(e,{pos:i,end:a})},n.prototype.deleteNodeRangeExcludingEnd=function(e,n,t,r){void 0===r&&(r={leadingTriviaOption:c.IncludeAll});var i=_(e,n,r),a=void 0===t?e.text.length:_(e,t,r);this.deleteRange(e,{pos:i,end:a})},n.prototype.replaceRange=function(e,n,t,r){void 0===r&&(r={}),this.changes.push({kind:p.ReplaceWithSingleNode,sourceFile:e,range:n,options:r,node:t})},n.prototype.replaceNode=function(e,n,t,r){void 0===r&&(r=f),this.replaceRange(e,m(e,n,n,r),t,r)},n.prototype.replaceNodeRange=function(e,n,t,r,i){void 0===i&&(i=f),this.replaceRange(e,m(e,n,t,i),r,i)},n.prototype.replaceRangeWithNodes=function(e,n,t,r){void 0===r&&(r={}),this.changes.push({kind:p.ReplaceWithMultipleNodes,sourceFile:e,range:n,options:r,nodes:t})},n.prototype.replaceNodeWithNodes=function(e,n,t,r){void 0===r&&(r=f),this.replaceRangeWithNodes(e,m(e,n,n,r),t,r)},n.prototype.replaceNodeWithText=function(e,n,t){this.replaceRangeWithText(e,m(e,n,n,f),t)},n.prototype.replaceNodeRangeWithNodes=function(e,n,t,r,i){void 0===i&&(i=f),this.replaceRangeWithNodes(e,m(e,n,t,i),r,i)},n.prototype.nodeHasTrailingComment=function(e,n,t){return void 0===t&&(t=f),!!g(e,n,t)},n.prototype.nextCommaToken=function(n,t){var r=e.findNextToken(t,t.parent,n);return r&&27===r.kind?r:void 0},n.prototype.replacePropertyAssignment=function(e,n,t){var r=this.nextCommaToken(e,n)?"":","+this.newLineCharacter;this.replaceNode(e,n,t,{suffix:r})},n.prototype.insertNodeAt=function(n,t,r,i){void 0===i&&(i={}),this.replaceRange(n,e.createRange(t),r,i)},n.prototype.insertNodesAt=function(n,t,r,i){void 0===i&&(i={}),this.replaceRangeWithNodes(n,e.createRange(t),r,i)},n.prototype.insertNodeAtTopOfFile=function(e,n,t){this.insertAtTopOfFile(e,n,t)},n.prototype.insertNodesAtTopOfFile=function(e,n,t){this.insertAtTopOfFile(e,n,t)},n.prototype.insertAtTopOfFile=function(n,t,r){var i=R(n),a={prefix:0===i?void 0:this.newLineCharacter,suffix:(e.isLineBreak(n.text.charCodeAt(i))?"":this.newLineCharacter)+(r?this.newLineCharacter:"")};e.isArray(t)?this.insertNodesAt(n,i,t,a):this.insertNodeAt(n,i,t,a)},n.prototype.insertFirstParameter=function(n,t,r){var i=e.firstOrUndefined(t);i?this.insertNodeBefore(n,i,r):this.insertNodeAt(n,t.pos,r)},n.prototype.insertNodeBefore=function(e,n,t,r,i){void 0===r&&(r=!1),void 0===i&&(i={}),this.insertNodeAt(e,_(e,n,i),t,this.getOptionsForInsertNodeBefore(n,t,r))},n.prototype.insertModifierAt=function(n,t,r,i){void 0===i&&(i={}),this.insertNodeAt(n,t,e.factory.createToken(r),i)},n.prototype.insertModifierBefore=function(e,n,t){return this.insertModifierAt(e,t.getStart(e),n,{suffix:" "})},n.prototype.insertCommentBeforeLine=function(n,t,r,i){var a=e.getStartPositionOfLine(t,n),o=e.getFirstNonSpaceCharacterPosition(n.text,a),s=O(n,o),c=e.getTouchingToken(n,s?o:r),l=n.text.slice(a,o),u=(s?"":this.newLineCharacter)+"//"+i+this.newLineCharacter+l;this.insertText(n,c.getStart(n),u)},n.prototype.insertJsdocCommentBefore=function(n,t,r){var i=t.getStart(n);if(t.jsDoc)for(var a=0,o=t.jsDoc;a<o.length;a++){var s=o[a];this.deleteRange(n,{pos:e.getLineStartPositionForPosition(s.getStart(n),n),end:y(n,s,{})})}var c=e.getPrecedingNonSpaceCharacterPosition(n.text,i-1),l=n.text.slice(c,i);this.insertNodeAt(n,i,r,{preserveLeadingWhitespace:!1,suffix:this.newLineCharacter+l})},n.prototype.replaceRangeWithText=function(e,n,t){this.changes.push({kind:p.Text,sourceFile:e,range:n,text:t})},n.prototype.insertText=function(n,t,r){this.replaceRangeWithText(n,e.createRange(t),r)},n.prototype.tryInsertTypeAnnotation=function(n,t,r){var i,a;if(e.isFunctionLike(t)){if(a=e.findChildOfKind(t,21,n),!a){if(!e.isArrowFunction(t))return!1;a=e.first(t.parameters)}}else a=null!==(i=252===t.kind?t.exclamationToken:t.questionToken)&&void 0!==i?i:t.name;return this.insertNodeAt(n,a.end,r,{prefix:": "}),!0},n.prototype.tryInsertThisTypeAnnotation=function(n,t,r){var i=e.findChildOfKind(t,20,n).getStart(n)+1,a=t.parameters.length?", ":"";this.insertNodeAt(n,i,r,{prefix:"this: ",suffix:a})},n.prototype.insertTypeParameters=function(n,t,r){var i=(e.findChildOfKind(t,20,n)||e.first(t.parameters)).getStart(n);this.insertNodesAt(n,i,r,{prefix:"<",suffix:">",joiner:", "})},n.prototype.getOptionsForInsertNodeBefore=function(n,t,r){return e.isStatement(n)||e.isClassElement(n)?{suffix:r?this.newLineCharacter+this.newLineCharacter:this.newLineCharacter}:e.isVariableDeclaration(n)?{suffix:", "}:e.isParameter(n)?e.isParameter(t)?{suffix:", "}:{}:e.isStringLiteral(n)&&e.isImportDeclaration(n.parent)||e.isNamedImports(n)?{suffix:", "}:e.isImportSpecifier(n)?{suffix:","+(r?this.newLineCharacter:" ")}:e.Debug.failBadSyntaxKind(n)},n.prototype.insertNodeAtConstructorStart=function(n,t,r){var i=e.firstOrUndefined(t.body.statements);i&&t.body.multiLine?this.insertNodeBefore(n,i,r):this.replaceConstructorBody(n,t,a([r],t.body.statements,!0))},n.prototype.insertNodeAtConstructorStartAfterSuperCall=function(n,t,r){var i=e.find(t.body.statements,function(n){return e.isExpressionStatement(n)&&e.isSuperCall(n.expression)});i&&t.body.multiLine?this.insertNodeAfter(n,i,r):this.replaceConstructorBody(n,t,a(a([],t.body.statements,!0),[r],!1))},n.prototype.insertNodeAtConstructorEnd=function(n,t,r){var i=e.lastOrUndefined(t.body.statements);i&&t.body.multiLine?this.insertNodeAfter(n,i,r):this.replaceConstructorBody(n,t,a(a([],t.body.statements,!0),[r],!1))},n.prototype.replaceConstructorBody=function(n,t,r){this.replaceNode(n,t.body,e.factory.createBlock(r,!0))},n.prototype.insertNodeAtEndOfScope=function(n,t,r){var i=_(n,t.getLastToken(),{});this.insertNodeAt(n,i,r,{prefix:e.isLineBreak(n.text.charCodeAt(t.getLastToken().pos))?this.newLineCharacter:this.newLineCharacter+this.newLineCharacter,suffix:this.newLineCharacter})},n.prototype.insertNodeAtClassStart=function(e,n,t){this.insertNodeAtStartWorker(e,n,t)},n.prototype.insertNodeAtObjectStart=function(e,n,t){this.insertNodeAtStartWorker(e,n,t)},n.prototype.insertNodeAtStartWorker=function(e,n,t){var r,i=null!==(r=this.guessIndentationFromExistingMembers(e,n))&&void 0!==r?r:this.computeIndentationForNewMember(e,n);this.insertNodeAt(e,L(n).pos,t,this.getInsertNodeAtStartInsertOptions(e,n,i))},n.prototype.guessIndentationFromExistingMembers=function(n,t){for(var r,i=t,a=0,o=L(t);a<o.length;a++){var s=o[a];if(e.rangeStartPositionsAreOnSameLine(i,s,n))return;var c=s.getStart(n),l=e.formatting.SmartIndenter.findFirstNonWhitespaceColumn(e.getLineStartPositionForPosition(c,n),c,n,this.formatContext.options);if(void 0===r)r=l;else if(l!==r)return;i=s}return r},n.prototype.computeIndentationForNewMember=function(n,t){var r,i=t.getStart(n);return e.formatting.SmartIndenter.findFirstNonWhitespaceColumn(e.getLineStartPositionForPosition(i,n),i,n,this.formatContext.options)+(null!==(r=this.formatContext.options.indentSize)&&void 0!==r?r:4)},n.prototype.getInsertNodeAtStartInsertOptions=function(n,t,r){var i=L(t),a=0===i.length,o=e.addToSeen(this.classesWithNodesInsertedAtStart,e.getNodeId(t),{node:t,sourceFile:n}),s=e.isObjectLiteralExpression(t)&&(!e.isJsonSourceFile(n)||!a),c=e.isObjectLiteralExpression(t)&&e.isJsonSourceFile(n)&&a&&!o;return{indentation:r,prefix:(c?",":"")+this.newLineCharacter,suffix:s?",":""}},n.prototype.insertNodeAfterComma=function(e,n,t){var r=this.insertNodeAfterWorker(e,this.nextCommaToken(e,n)||n,t);this.insertNodeAt(e,r,t,this.getInsertNodeAfterOptions(e,n))},n.prototype.insertNodeAfter=function(e,n,t){var r=this.insertNodeAfterWorker(e,n,t);this.insertNodeAt(e,r,t,this.getInsertNodeAfterOptions(e,n))},n.prototype.insertNodeAtEndOfList=function(e,n,t){this.insertNodeAt(e,n.end,t,{prefix:", "})},n.prototype.insertNodesAfter=function(n,t,r){var i=this.insertNodeAfterWorker(n,t,e.first(r));this.insertNodesAt(n,i,r,this.getInsertNodeAfterOptions(n,t))},n.prototype.insertNodeAfterWorker=function(n,t,r){w(t,r)&&59!==n.text.charCodeAt(t.end-1)&&this.replaceRange(n,e.createRange(t.end),e.factory.createToken(26));var i=y(n,t,{});return i},n.prototype.getInsertNodeAfterOptions=function(n,t){var r=this.getInsertNodeAfterOptionsWorker(t);return o(o({},r),{prefix:t.end===n.end&&e.isStatement(t)?r.prefix?"\n"+r.prefix:"\n":r.prefix})},n.prototype.getInsertNodeAfterOptionsWorker=function(n){switch(n.kind){case 255:case 259:return{prefix:this.newLineCharacter,suffix:this.newLineCharacter};case 252:case 10:case 79:return{prefix:", "};case 291:return{suffix:","+this.newLineCharacter};case 93:return{prefix:" "};case 162:return{};default:return e.Debug.assert(e.isStatement(n)||e.isClassOrTypeElement(n)),{suffix:this.newLineCharacter}}},n.prototype.insertName=function(n,t,r){if(e.Debug.assert(!t.name),212===t.kind){var i=e.findChildOfKind(t,38,n),a=e.findChildOfKind(t,20,n);a?(this.insertNodesAt(n,a.getStart(n),[e.factory.createToken(98),e.factory.createIdentifier(r)],{joiner:" "}),P(this,n,i)):(this.insertText(n,e.first(t.parameters).getStart(n),"function "+r+"("),this.replaceRange(n,i,e.factory.createToken(21))),233!==t.body.kind&&(this.insertNodesAt(n,t.body.getStart(n),[e.factory.createToken(18),e.factory.createToken(105)],{joiner:" ",suffix:" "}),this.insertNodesAt(n,t.body.end,[e.factory.createToken(26),e.factory.createToken(19)],{joiner:" "}))}else{var o=e.findChildOfKind(t,211===t.kind?98:84,n).end;this.insertNodeAt(n,o,e.factory.createIdentifier(r),{prefix:" "})}},n.prototype.insertExportModifier=function(e,n){this.insertText(e,n.getStart(e),"export ")},n.prototype.insertNodeInListAfter=function(n,t,r,i){if(void 0===i&&(i=e.formatting.SmartIndenter.getContainingList(t,n)),i){var a=e.indexOfNode(i,t);if(!(a<0)){var o=t.getEnd();if(a!==i.length-1){var s=e.getTokenAtPosition(n,t.end);if(s&&h(t,s)){var c=i[a+1],l=u(n.text,c.getFullStart()),p=""+e.tokenToString(s.kind)+n.text.substring(s.end,l);this.insertNodesAt(n,l,[r],{suffix:p})}}else{var f=t.getStart(n),m=e.getLineStartPositionForPosition(f,n),_=void 0,g=!1;if(1===i.length)_=27;else{var y=e.findPrecedingToken(t.pos,n);_=h(t,y)?y.kind:27;var v=e.getLineStartPositionForPosition(i[a-1].getStart(n),n);g=v!==m}if(d(n.text,t.end)&&(g=!0),g){this.replaceRange(n,e.createRange(o),e.factory.createToken(_));var b=e.formatting.SmartIndenter.findFirstNonWhitespaceColumn(m,f,n,this.formatContext.options),E=e.skipTrivia(n.text,o,!0,!1);while(E!==o&&e.isLineBreak(n.text.charCodeAt(E-1)))E--;this.replaceRange(n,e.createRange(E),r,{indentation:b,prefix:this.newLineCharacter})}else this.replaceRange(n,e.createRange(o),r,{prefix:e.tokenToString(_)+" "})}}}else e.Debug.fail("node is not a list element")},n.prototype.parenthesizeExpression=function(n,t){this.replaceRange(n,e.rangeOfNode(t),e.factory.createParenthesizedExpression(t))},n.prototype.finishClassesWithNodesInsertedAtStart=function(){var n=this;this.classesWithNodesInsertedAtStart.forEach(function(t){var r=t.node,i=t.sourceFile,a=x(r,i),o=a[0],s=a[1];if(void 0!==o&&void 0!==s){var c=0===L(r).length,l=e.positionsAreOnSameLine(o,s,i);c&&l&&o!==s-1&&n.deleteRange(i,e.createRange(o,s-1)),l&&n.insertText(i,s-1,n.newLineCharacter)}})},n.prototype.finishDeleteDeclarations=function(){for(var n=this,t=new e.Set,r=function(n,r){i.deletedNodes.some(function(t){return t.sourceFile===n&&e.rangeContainsRangeExclusive(t.node,r)})||(e.isArray(r)?i.deleteRange(n,e.rangeOfTypeParameters(n,r)):E.deleteDeclaration(i,t,n,r))},i=this,a=0,o=this.deletedNodes;a<o.length;a++){var s=o[a],c=s.sourceFile,l=s.node;r(c,l)}t.forEach(function(r){var i=r.getSourceFile(),a=e.formatting.SmartIndenter.getContainingList(r,i);if(r===e.last(a)){var o=e.findLastIndex(a,function(e){return!t.has(e)},a.length-2);-1!==o&&n.deleteRange(i,{pos:a[o].end,end:S(i,a[o+1])})}})},n.prototype.getChanges=function(e){this.finishDeleteDeclarations(),this.finishClassesWithNodesInsertedAtStart();for(var n=b.getTextChangesFromChanges(this.changes,this.newLineCharacter,this.formatContext,e),t=0,r=this.newFiles;t<r.length;t++){var i=r[t],a=i.oldFile,o=i.fileName,s=i.statements;n.push(b.newFileChanges(a,o,s,this.newLineCharacter,this.formatContext))}return n},n.prototype.createNewFile=function(e,n,t){this.newFiles.push({oldFile:e,fileName:n,statements:t})},n}();function S(n,t){return e.skipTrivia(n.text,_(n,t,{leadingTriviaOption:c.IncludeAll}),!1,!0)}function x(n,t){var r=e.findChildOfKind(n,18,t),i=e.findChildOfKind(n,19,t);return[null===r||void 0===r?void 0:r.end,null===i||void 0===i?void 0:i.end]}function L(n){return e.isObjectLiteralExpression(n)?n.properties:n.members}function D(e,n,t,r){return b.newFileChangesWorker(void 0,n,e,t,r)}function A(n,t){for(var r=t.length-1;r>=0;r--){var i=t[r],a=i.span,o=i.newText;n=""+n.substring(0,a.start)+o+n.substring(e.textSpanEnd(a))}return n}function C(n){return e.skipTrivia(n,0)===n.length}function I(n){var r=e.visitEachChild(n,I,e.nullTransformationContext,N,I),a=e.nodeIsSynthesized(r)?r:Object.create(r);return e.setTextRangePosEnd(a,t(n),i(n)),a}function N(n,r,a,o,s){var c=e.visitNodes(n,r,a,o,s);if(!c)return c;var l=c===n?e.factory.createNodeArray(c.slice(0)):c;return e.setTextRangePosEnd(l,t(n),i(n)),l}function k(n){var t=0,i=e.createTextWriter(n),a=function(e){e&&r(e,t)},o=function(e){e&&s(e,t)},c=function(e){e&&r(e,t)},l=function(e){e&&s(e,t)},u=function(e){e&&r(e,t)},d=function(e){e&&s(e,t)};function p(n,r){if(r||!C(n)){t=i.getTextPos();var a=0;while(e.isWhiteSpaceLike(n.charCodeAt(n.length-a-1)))a++;t-=a}}function f(e){i.write(e),p(e,!1)}function m(e){i.writeComment(e)}function _(e){i.writeKeyword(e),p(e,!1)}function g(e){i.writeOperator(e),p(e,!1)}function y(e){i.writePunctuation(e),p(e,!1)}function h(e){i.writeTrailingSemicolon(e),p(e,!1)}function v(e){i.writeParameter(e),p(e,!1)}function b(e){i.writeProperty(e),p(e,!1)}function E(e){i.writeSpace(e),p(e,!1)}function T(e){i.writeStringLiteral(e),p(e,!1)}function S(e,n){i.writeSymbol(e,n),p(e,!1)}function x(e){i.writeLine(e)}function L(){i.increaseIndent()}function D(){i.decreaseIndent()}function A(){return i.getText()}function I(e){i.rawWrite(e),p(e,!1)}function N(e){i.writeLiteral(e),p(e,!0)}function k(){return i.getTextPos()}function R(){return i.getLine()}function O(){return i.getColumn()}function w(){return i.getIndent()}function P(){return i.isAtStartOfLine()}function M(){i.clear(),t=0}return{onBeforeEmitNode:a,onAfterEmitNode:o,onBeforeEmitNodeArray:c,onAfterEmitNodeArray:l,onBeforeEmitToken:u,onAfterEmitToken:d,write:f,writeComment:m,writeKeyword:_,writeOperator:g,writePunctuation:y,writeTrailingSemicolon:h,writeParameter:v,writeProperty:b,writeSpace:E,writeStringLiteral:T,writeSymbol:S,writeLine:x,increaseIndent:L,decreaseIndent:D,getText:A,rawWrite:I,writeLiteral:N,getTextPos:k,getLine:R,getColumn:O,getIndent:w,isAtStartOfLine:P,hasTrailingComment:function(){return i.hasTrailingComment()},hasTrailingWhitespace:function(){return i.hasTrailingWhitespace()},clear:M}}function R(n){for(var t,r=0,i=n.statements;r<i.length;r++){var a=i[r];if(!e.isPrologueDirective(a))break;t=a}var o=0,s=n.text;if(t)return o=t.end,h(),o;var c=e.getShebang(s);void 0!==c&&(o=c.length,h());var l,u,d=e.getLeadingCommentRanges(s,o);if(!d)return o;for(var p=0,f=d;p<f.length;p++){var m=f[p];if(3===m.kind){if(e.isPinnedComment(s,m.pos)){l={range:m,pinnedOrTripleSlash:!0};continue}}else if(e.isRecognizedTripleSlashComment(s,m.pos,m.end)){l={range:m,pinnedOrTripleSlash:!0};continue}if(l){if(l.pinnedOrTripleSlash)break;var _=n.getLineAndCharacterOfPosition(m.pos).line,g=n.getLineAndCharacterOfPosition(l.range.end).line;if(_>=g+2)break}if(n.statements.length){void 0===u&&(u=n.getLineAndCharacterOfPosition(n.statements[0].getStart()).line);var y=n.getLineAndCharacterOfPosition(m.end).line;if(u<y+2)break}l={range:m,pinnedOrTripleSlash:!1}}return l&&(o=l.range.end,h()),o;function h(){if(o<s.length){var n=s.charCodeAt(o);e.isLineBreak(n)&&(o++,o<s.length&&13===n&&10===s.charCodeAt(o)&&o++)}}}function O(n,t){return!e.isInComment(n,t)&&!e.isInString(n,t)&&!e.isInTemplateString(n,t)&&!e.isInJSXText(n,t)}function w(n,t){return(e.isPropertySignature(n)||e.isPropertyDeclaration(n))&&e.isClassOrTypeElement(t)&&160===t.name.kind||e.isStatementButNotDeclaration(n)&&e.isStatementButNotDeclaration(t)}function P(e,n,t,r){void 0===r&&(r={leadingTriviaOption:c.IncludeAll});var i=_(n,t,r),a=y(n,t,r);e.deleteRange(n,{pos:i,end:a})}function M(n,t,r,i){var a=e.Debug.checkDefined(e.formatting.SmartIndenter.getContainingList(i,r)),o=e.indexOfNode(a,i);e.Debug.assert(-1!==o),1!==a.length?(e.Debug.assert(!t.has(i),"Deleting a node twice"),t.add(i),n.deleteRange(r,{pos:S(r,i),end:o===a.length-1?y(r,i,{}):S(r,a[o+1])})):P(n,r,i)}n.ChangeTracker=T,n.getNewFileText=D,function(n){function t(n,t,r,i){return e.mapDefined(e.group(n,function(e){return e.sourceFile.path}),function(n){for(var o=n[0].sourceFile,s=e.stableSort(n,function(e,n){return e.range.pos-n.range.pos||e.range.end-n.range.end}),c=function(n){e.Debug.assert(s[n].range.end<=s[n+1].range.pos,"Changes overlap",function(){return JSON.stringify(s[n].range)+" and "+JSON.stringify(s[n+1].range)})},l=0;l<s.length-1;l++)c(l);var u=e.mapDefined(s,function(n){var s=e.createTextSpanFromRange(n.range),c=a(n,o,t,r,i);if(s.length!==c.length||!e.stringContainsAt(o.text,c,s.start))return e.createTextChange(s,c)});return u.length>0?{fileName:o.fileName,textChanges:u}:void 0})}function r(n,t,r,a,o){var s=i(n,e.getScriptKindFromFileName(t),r,a,o);return{fileName:t,textChanges:[e.createTextChange(e.createTextSpan(0,0),s)],isNewFile:!0}}function i(n,t,r,i,a){var o=r.map(function(e){return 4===e?"":l(e,n,i).text}).join(i),s=e.createSourceFile("any file name",o,99,!0,t),c=e.formatting.formatDocument(s,a);return A(o,c)+i}function a(n,t,r,i,a){var o;if(n.kind===p.Remove)return"";if(n.kind===p.Text)return n.text;var s=n.options,l=void 0===s?{}:s,u=n.range.pos,d=function(e){return c(e,t,u,l,r,i,a)},f=n.kind===p.ReplaceWithMultipleNodes?n.nodes.map(function(n){return e.removeSuffix(d(n),r)}).join((null===(o=n.options)||void 0===o?void 0:o.joiner)||r):d(n.node),m=l.preserveLeadingWhitespace||void 0!==l.indentation||e.getLineStartPositionForPosition(u,t)===u?f:f.replace(/^\s+/,"");return(l.prefix||"")+m+(!l.suffix||e.endsWith(m,l.suffix)?"":l.suffix)}function s(n,t){var r=n.options,i=!r.semicolons||r.semicolons===e.SemicolonPreference.Ignore,a=r.semicolons===e.SemicolonPreference.Remove||i&&!e.probablyUsesSemicolons(t);return o(o({},r),{semicolons:a?e.SemicolonPreference.Remove:e.SemicolonPreference.Ignore})}function c(n,t,r,i,a,c,u){var d=i.indentation,p=i.prefix,f=i.delta,m=l(n,t,a),_=m.node,g=m.text;u&&u(_,g);var y=s(c,t),h=void 0!==d?d:e.formatting.SmartIndenter.getIndentation(r,t,y,p===a||e.getLineStartPositionForPosition(r,t)===r);void 0===f&&(f=e.formatting.SmartIndenter.shouldIndentChildNode(y,n)&&y.indentSize||0);var v={text:g,getLineAndCharacterOfPosition:function(n){return e.getLineAndCharacterOfPosition(this,n)}},b=e.formatting.formatNodeGivenIndentation(_,v,t.languageVariant,h,f,o(o({},c),{options:y}));return A(g,b)}function l(n,t,r){var i=k(r),a="\n"===r?1:0;return e.createPrinter({newLine:a,neverAsciiEscape:!0,preserveSourceNewlines:!0,terminateUnterminatedLiterals:!0},i).writeNode(4,n,t,i),{text:i.getText(),node:I(n)}}n.getTextChangesFromChanges=t,n.newFileChanges=r,n.newFileChangesWorker=i,n.getNonformattedText=l}(b||(b={})),n.applyChanges=A,n.isValidLocationToAddComment=O,function(n){function t(n,t,o,s){switch(s.kind){case 162:var u=s.parent;e.isArrowFunction(u)&&1===u.parameters.length&&!e.findChildOfKind(u,20,o)?n.replaceNodeWithText(o,s,"()"):M(n,t,o,s);break;case 264:case 263:var d=o.imports.length&&s===e.first(o.imports).parent||s===e.find(o.statements,e.isAnyImportSyntax);P(n,o,s,{leadingTriviaOption:d?c.Exclude:e.hasJSDocNodes(s)?c.JSDoc:c.StartLine});break;case 201:var p=s.parent,f=200===p.kind&&s!==e.last(p.elements);f?P(n,o,s):M(n,t,o,s);break;case 252:a(n,t,o,s);break;case 161:M(n,t,o,s);break;case 268:var m=s.parent;1===m.elements.length?i(n,o,m):M(n,t,o,s);break;case 266:i(n,o,s);break;case 26:P(n,o,s,{trailingTriviaOption:l.Exclude});break;case 98:P(n,o,s,{leadingTriviaOption:c.Exclude});break;case 255:case 254:P(n,o,s,{leadingTriviaOption:e.hasJSDocNodes(s)?c.JSDoc:c.StartLine});break;default:s.parent?e.isImportClause(s.parent)&&s.parent.name===s?r(n,o,s.parent):e.isCallExpression(s.parent)&&e.contains(s.parent.arguments,s)?M(n,t,o,s):P(n,o,s):P(n,o,s)}}function r(n,t,r){if(r.namedBindings){var i=r.name.getStart(t),a=e.getTokenAtPosition(t,r.name.end);if(a&&27===a.kind){var o=e.skipTrivia(t.text,a.end,!1,!0);n.deleteRange(t,{pos:i,end:o})}else P(n,t,r.name)}else P(n,t,r.parent)}function i(n,t,r){if(r.parent.name){var i=e.Debug.checkDefined(e.getTokenAtPosition(t,r.pos-1));n.deleteRange(t,{pos:i.getStart(t),end:r.end})}else{var a=e.getAncestor(r,264);P(n,t,a)}}function a(n,t,r,i){var a=i.parent;if(290!==a.kind)if(1===a.declarations.length){var o=a.parent;switch(o.kind){case 242:case 241:n.replaceNode(r,i,e.factory.createObjectLiteralExpression());break;case 240:P(n,r,a);break;case 235:P(n,r,o,{leadingTriviaOption:e.hasJSDocNodes(o)?c.JSDoc:c.StartLine});break;default:e.Debug.assertNever(o)}}else M(n,t,r,i);else n.deleteNodeRange(r,e.findChildOfKind(a,20,r),e.findChildOfKind(a,21,r))}n.deleteDeclaration=t}(E||(E={})),n.deleteNode=P})(e.textChanges||(e.textChanges={}))}(d||(d={})),function(e){(function(n){var t=e.createMultiMap(),r=new e.Map;function i(n){return e.isArray(n)?e.formatStringFromArgs(e.getLocaleSpecificMessage(n[0]),n.slice(1)):e.getLocaleSpecificMessage(n)}function s(e,n,t){return d(e,i(t),n,void 0,void 0)}function c(e,n,t,r,a,o){return d(e,i(t),n,r,i(a),o)}function u(e,n,t,r,a,o){return d(e,i(t),n,r,a&&i(a),o)}function d(e,n,t,r,i,a){return{fixName:e,description:n,changes:t,fixId:r,fixAllDescription:i,commands:a?[a]:void 0}}function p(n){for(var i=0,a=n.errorCodes;i<a.length;i++){var o=a[i];t.add(String(o),n)}if(n.fixIds)for(var s=0,c=n.fixIds;s<c.length;s++){var l=c[s];e.Debug.assert(!r.has(l)),r.set(l,n)}}function f(){return e.arrayFrom(t.keys())}function m(n,t){for(var r=n.errorCodes,i=0,a=0,s=t;a<s.length;a++){var c=s[a];if(e.contains(r,c.code)&&i++,i>1)break}var u=i<2;return function(e){var n=e.fixId,t=e.fixAllDescription,r=l(e,["fixId","fixAllDescription"]);return u?r:o(o({},r),{fixId:n,fixAllDescription:t})}}function _(n){var r=E(n),i=t.get(String(n.errorCode));return e.flatMap(i,function(t){return e.map(t.getCodeActions(n),m(t,r))})}function g(n){return r.get(e.cast(n.fixId,e.isString)).getAllCodeActions(n)}function y(e,n){return{changes:e,commands:n}}function h(e,n){return{fileName:e,textChanges:n}}function v(n,t,r){var i=[],a=e.textChanges.ChangeTracker.with(n,function(e){return b(n,t,function(n){return r(e,n,i)})});return y(a,0===i.length?void 0:i)}function b(n,t,r){for(var i=0,a=E(n);i<a.length;i++){var o=a[i];e.contains(t,o.code)&&r(o)}}function E(n){var t=n.program,r=n.sourceFile,i=n.cancellationToken;return a(a(a([],t.getSemanticDiagnostics(r,i),!0),t.getSyntacticDiagnostics(r,i),!0),e.computeSuggestionDiagnostics(r,t,i),!0)}n.createCodeFixActionWithoutFixAll=s,n.createCodeFixAction=c,n.createCodeFixActionMaybeFixAll=u,n.registerCodeFix=p,n.getSupportedErrorCodes=f,n.getFixes=_,n.getAllFixes=g,n.createCombinedCodeActions=y,n.createFileTextChanges=h,n.codeFixAll=v,n.eachDiagnostic=b})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t=new e.Map;function r(e,n){t.set(e,n)}function i(r){return e.arrayFrom(e.flatMapIterator(t.values(),function(e){var t;return r.cancellationToken&&r.cancellationToken.isCancellationRequested()||!(null===(t=e.kinds)||void 0===t?void 0:t.some(function(e){return n.refactorKindBeginsWith(e,r.kind)}))?void 0:e.getAvailableActions(r)}))}function a(e,n,r){var i=t.get(n);return i&&i.getEditsForAction(e,r)}n.registerRefactor=r,n.getApplicableRefactors=i,n.getEditsForRefactor=a})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){var t="addConvertToUnknownForNonOverlappingTypes",r=[e.Diagnostics.Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the_other_If_this_was_intentional_convert_the_expression_to_unknown_first.code];function i(n,t,r){var i=e.getTokenAtPosition(t,r),a=e.Debug.checkDefined(e.findAncestor(i,function(n){return e.isAsExpression(n)||e.isTypeAssertionExpression(n)}),"Expected to find an assertion expression"),o=e.isAsExpression(a)?e.factory.createAsExpression(a.expression,e.factory.createKeywordTypeNode(153)):e.factory.createTypeAssertion(e.factory.createKeywordTypeNode(153),a.expression);n.replaceNode(t,a.expression,o)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span.start)});return[n.createCodeFixAction(t,a,e.Diagnostics.Add_unknown_conversion_for_non_overlapping_types,t,e.Diagnostics.Add_unknown_to_all_conversions_of_non_overlapping_types)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return i(e,n.file,n.start)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){n.registerCodeFix({errorCodes:[e.Diagnostics.await_expressions_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_has_no_imports_or_exports_Consider_adding_an_empty_export_to_make_this_file_a_module.code,e.Diagnostics.for_await_loops_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_has_no_imports_or_exports_Consider_adding_an_empty_export_to_make_this_file_a_module.code],getCodeActions:function(t){var r=t.sourceFile,i=e.textChanges.ChangeTracker.with(t,function(n){var t=e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports([]),void 0);n.insertNodeAtEndOfScope(r,r,t)});return[n.createCodeFixActionWithoutFixAll("addEmptyExportDeclaration",i,e.Diagnostics.Add_export_to_make_this_file_into_a_module)]}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="addMissingAsync",r=[e.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1.code,e.Diagnostics.Type_0_is_not_assignable_to_type_1.code,e.Diagnostics.Type_0_is_not_comparable_to_type_1.code];function i(r,i,o,s){var c=o(function(e){return a(e,r.sourceFile,i,s)});return n.createCodeFixAction(t,c,e.Diagnostics.Add_async_modifier_to_containing_function,t,e.Diagnostics.Add_all_missing_async_modifiers)}function a(n,t,r,i){if(!i||!i.has(e.getNodeId(r))){null===i||void 0===i||i.add(e.getNodeId(r));var a=e.factory.updateModifiers(e.getSynthesizedDeepClone(r,!0),e.factory.createNodeArray(e.factory.createModifiersFromModifierFlags(256|e.getSyntacticModifierFlags(r))));n.replaceNode(t,r,a)}}function o(n,t){if(t){var r=e.getTokenAtPosition(n,t.start),i=e.findAncestor(r,function(r){return r.getStart(n)<t.start||r.getEnd()>e.textSpanEnd(t)?"quit":(e.isArrowFunction(r)||e.isMethodDeclaration(r)||e.isFunctionExpression(r)||e.isFunctionDeclaration(r))&&e.textSpansEqual(t,e.createTextSpanFromNode(r,n))});return i}}function s(n,t){return function(r){var i=r.start,a=r.length,o=r.relatedInformation,s=r.code;return e.isNumber(i)&&e.isNumber(a)&&e.textSpansEqual({start:i,length:a},n)&&s===t&&!!o&&e.some(o,function(n){return n.code===e.Diagnostics.Did_you_mean_to_mark_this_function_as_async.code})}}n.registerCodeFix({fixIds:[t],errorCodes:r,getCodeActions:function(n){var t=n.sourceFile,r=n.errorCode,a=n.cancellationToken,c=n.program,l=n.span,u=e.find(c.getDiagnosticsProducingTypeChecker().getDiagnostics(t,a),s(l,r)),d=u&&u.relatedInformation&&e.find(u.relatedInformation,function(n){return n.code===e.Diagnostics.Did_you_mean_to_mark_this_function_as_async.code}),p=o(t,d);if(p){var f=function(t){return e.textChanges.ChangeTracker.with(n,t)};return[i(n,p,f)]}},getAllCodeActions:function(t){var a=t.sourceFile,s=new e.Set;return n.codeFixAll(t,r,function(n,r){var c=r.relatedInformation&&e.find(r.relatedInformation,function(n){return n.code===e.Diagnostics.Did_you_mean_to_mark_this_function_as_async.code}),l=o(a,c);if(l){var u=function(e){return e(n),[]};return i(t,l,u,s)}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="addMissingAwait",r=e.Diagnostics.Property_0_does_not_exist_on_type_1.code,i=[e.Diagnostics.This_expression_is_not_callable.code,e.Diagnostics.This_expression_is_not_constructable.code],o=a([e.Diagnostics.An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type.code,e.Diagnostics.The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type.code,e.Diagnostics.The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type.code,e.Diagnostics.Operator_0_cannot_be_applied_to_type_1.code,e.Diagnostics.Operator_0_cannot_be_applied_to_types_1_and_2.code,e.Diagnostics.This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap.code,e.Diagnostics.This_condition_will_always_return_true_since_this_0_is_always_defined.code,e.Diagnostics.Type_0_is_not_an_array_type.code,e.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type.code,e.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators.code,e.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator.code,e.Diagnostics.Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator.code,e.Diagnostics.Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator.code,e.Diagnostics.Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator.code,e.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1.code,r],i,!0);function s(t,r,i,a,o,s){var c=t.sourceFile,l=t.program,u=t.cancellationToken,p=d(r,c,u,l,a);if(p){var f=o(function(n){e.forEach(p.initializers,function(e){var t=e.expression;return _(n,i,c,a,t,s)}),s&&p.needsSecondPassForFixAll&&_(n,i,c,a,r,s)});return n.createCodeFixActionWithoutFixAll("addMissingAwaitToInitializer",f,1===p.initializers.length?[e.Diagnostics.Add_await_to_initializer_for_0,p.initializers[0].declarationSymbol.name]:e.Diagnostics.Add_await_to_initializers)}}function c(r,i,a,o,s,c){var l=s(function(e){return _(e,a,r.sourceFile,o,i,c)});return n.createCodeFixAction(t,l,e.Diagnostics.Add_await,t,e.Diagnostics.Fix_all_expressions_possibly_missing_await)}function l(n,t,r,i,a){var o=a.getDiagnosticsProducingTypeChecker(),s=o.getDiagnostics(n,i);return e.some(s,function(n){var i=n.start,a=n.length,o=n.relatedInformation,s=n.code;return e.isNumber(i)&&e.isNumber(a)&&e.textSpansEqual({start:i,length:a},r)&&s===t&&!!o&&e.some(o,function(n){return n.code===e.Diagnostics.Did_you_forget_to_use_await.code})})}function u(n,t,r,i,a){var o=e.getTokenAtPosition(n,r.start),s=e.findAncestor(o,function(t){return t.getStart(n)<r.start||t.getEnd()>e.textSpanEnd(r)?"quit":e.isExpression(t)&&e.textSpansEqual(r,e.createTextSpanFromNode(t,n))});return s&&l(n,t,r,i,a)&&m(s)?s:void 0}function d(n,t,r,i,a){var o=p(n,a);if(o){for(var s,c=o.isCompleteFix,l=function(n){var o=a.getSymbolAtLocation(n);if(!o)return"continue";var l=e.tryCast(o.valueDeclaration,e.isVariableDeclaration),u=l&&e.tryCast(l.name,e.isIdentifier),d=e.getAncestor(l,235);if(!l||!d||l.type||!l.initializer||d.getSourceFile()!==t||e.hasSyntacticModifier(d,1)||!u||!m(l.initializer))return c=!1,"continue";var p=i.getSemanticDiagnostics(t,r),_=e.FindAllReferences.Core.eachSymbolReferenceInFile(u,a,t,function(e){return n!==e&&!f(e,p,t,a)});if(_)return c=!1,"continue";(s||(s=[])).push({expression:l.initializer,declarationSymbol:o})},u=0,d=o.identifiers;u<d.length;u++){var _=d[u];l(_)}return s&&{initializers:s,needsSecondPassForFixAll:!c}}}function p(n,t){if(e.isPropertyAccessExpression(n.parent)&&e.isIdentifier(n.parent.expression))return{identifiers:[n.parent.expression],isCompleteFix:!0};if(e.isIdentifier(n))return{identifiers:[n],isCompleteFix:!0};if(e.isBinaryExpression(n)){for(var r=void 0,i=!0,a=0,o=[n.left,n.right];a<o.length;a++){var s=o[a],c=t.getTypeAtLocation(s);if(t.getPromisedTypeOfPromise(c)){if(!e.isIdentifier(s)){i=!1;continue}(r||(r=[])).push(s)}}return r&&{identifiers:r,isCompleteFix:i}}}function f(n,t,r,i){var a=e.isPropertyAccessExpression(n.parent)?n.parent.name:e.isBinaryExpression(n.parent)?n.parent:n,s=e.find(t,function(e){return e.start===a.getStart(r)&&e.start+e.length===a.getEnd()});return s&&e.contains(o,s.code)||1&i.getTypeAtLocation(a).flags}function m(n){return 32768&n.kind||!!e.findAncestor(n,function(n){return n.parent&&e.isArrowFunction(n.parent)&&n.parent.body===n||e.isBlock(n)&&(254===n.parent.kind||211===n.parent.kind||212===n.parent.kind||167===n.parent.kind)})}function _(n,t,a,o,s,c){if(e.isBinaryExpression(s))for(var l=0,u=[s.left,s.right];l<u.length;l++){var d=u[l];if(c&&e.isIdentifier(d)){var p=o.getSymbolAtLocation(d);if(p&&c.has(e.getSymbolId(p)))continue}var f=o.getTypeAtLocation(d),m=o.getPromisedTypeOfPromise(f)?e.factory.createAwaitExpression(d):d;n.replaceNode(a,d,m)}else if(t===r&&e.isPropertyAccessExpression(s.parent)){if(c&&e.isIdentifier(s.parent.expression)){p=o.getSymbolAtLocation(s.parent.expression);if(p&&c.has(e.getSymbolId(p)))return}n.replaceNode(a,s.parent.expression,e.factory.createParenthesizedExpression(e.factory.createAwaitExpression(s.parent.expression))),g(n,s.parent.expression,a)}else if(e.contains(i,t)&&e.isCallOrNewExpression(s.parent)){if(c&&e.isIdentifier(s)){p=o.getSymbolAtLocation(s);if(p&&c.has(e.getSymbolId(p)))return}n.replaceNode(a,s,e.factory.createParenthesizedExpression(e.factory.createAwaitExpression(s))),g(n,s,a)}else{if(c&&e.isVariableDeclaration(s.parent)&&e.isIdentifier(s.parent.name)){p=o.getSymbolAtLocation(s.parent.name);if(p&&!e.tryAddToSet(c,e.getSymbolId(p)))return}n.replaceNode(a,s,e.factory.createAwaitExpression(s))}}function g(n,t,r){var i=e.findPrecedingToken(t.pos,r);i&&e.positionIsASICandidate(i.end,i.parent,r)&&n.insertText(r,t.getStart(r),";")}n.registerCodeFix({fixIds:[t],errorCodes:o,getCodeActions:function(n){var t=n.sourceFile,r=n.errorCode,i=n.span,a=n.cancellationToken,o=n.program,l=u(t,r,i,a,o);if(l){var d=n.program.getTypeChecker(),p=function(t){return e.textChanges.ChangeTracker.with(n,t)};return e.compact([s(n,l,r,d,p),c(n,l,r,d,p)])}},getAllCodeActions:function(t){var r=t.sourceFile,i=t.program,a=t.cancellationToken,l=t.program.getTypeChecker(),d=new e.Set;return n.codeFixAll(t,o,function(e,n){var o=u(r,n.code,n,a,i);if(o){var p=function(n){return n(e),[]};return s(t,o,n.code,l,p,d)||c(t,o,n.code,l,p,d)}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="addMissingConst",r=[e.Diagnostics.Cannot_find_name_0.code,e.Diagnostics.No_value_exists_in_scope_for_the_shorthand_property_0_Either_declare_one_or_provide_an_initializer.code];function i(n,t,r,i,u){var d=e.getTokenAtPosition(t,r),p=e.findAncestor(d,function(n){return e.isForInOrOfStatement(n.parent)?n.parent.initializer===n:!o(n)&&"quit"});if(p)return a(n,p,t,u);var f=d.parent;if(e.isBinaryExpression(f)&&63===f.operatorToken.kind&&e.isExpressionStatement(f.parent))return a(n,d,t,u);if(e.isArrayLiteralExpression(f)){var m=i.getTypeChecker();if(!e.every(f.elements,function(e){return s(e,m)}))return;return a(n,f,t,u)}var _=e.findAncestor(d,function(n){return!!e.isExpressionStatement(n.parent)||!c(n)&&"quit"});if(_){var g=i.getTypeChecker();if(!l(_,g))return;return a(n,_,t,u)}}function a(n,t,r,i){i&&!e.tryAddToSet(i,t)||n.insertModifierBefore(r,85,t)}function o(e){switch(e.kind){case 79:case 202:case 203:case 291:case 292:return!0;default:return!1}}function s(n,t){var r=e.isIdentifier(n)?n:e.isAssignmentExpression(n,!0)&&e.isIdentifier(n.left)?n.left:void 0;return!!r&&!t.getSymbolAtLocation(r)}function c(e){switch(e.kind){case 79:case 219:case 27:return!0;default:return!1}}function l(n,t){return!!e.isBinaryExpression(n)&&(27===n.operatorToken.kind?e.every([n.left,n.right],function(e){return l(e,t)}):63===n.operatorToken.kind&&e.isIdentifier(n.left)&&!t.getSymbolAtLocation(n.left))}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span.start,r.program)});if(a.length>0)return[n.createCodeFixAction(t,a,e.Diagnostics.Add_const_to_unresolved_variable,t,e.Diagnostics.Add_const_to_all_unresolved_variables)]},fixIds:[t],getAllCodeActions:function(t){var a=new e.Set;return n.codeFixAll(t,r,function(e,n){return i(e,n.file,n.start,t.program,a)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="addMissingDeclareProperty",r=[e.Diagnostics.Property_0_will_overwrite_the_base_property_in_1_If_this_is_intentional_add_an_initializer_Otherwise_add_a_declare_modifier_or_remove_the_redundant_declaration.code];function i(n,t,r,i){var a=e.getTokenAtPosition(t,r);if(e.isIdentifier(a)){var o=a.parent;165!==o.kind||i&&!e.tryAddToSet(i,o)||n.insertModifierBefore(t,134,o)}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span.start)});if(a.length>0)return[n.createCodeFixAction(t,a,e.Diagnostics.Prefix_with_declare,t,e.Diagnostics.Prefix_all_incorrect_property_declarations_with_declare)]},fixIds:[t],getAllCodeActions:function(t){var a=new e.Set;return n.codeFixAll(t,r,function(e,n){return i(e,n.file,n.start,a)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="addMissingInvocationForDecorator",r=[e.Diagnostics._0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write_0.code];function i(n,t,r){var i=e.getTokenAtPosition(t,r),a=e.findAncestor(i,e.isDecorator);e.Debug.assert(!!a,"Expected position to be owned by a decorator.");var o=e.factory.createCallExpression(a.expression,void 0,void 0);n.replaceNode(t,a.expression,o)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span.start)});return[n.createCodeFixAction(t,a,e.Diagnostics.Call_decorator_expression,t,e.Diagnostics.Add_to_all_uncalled_decorators)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return i(e,n.file,n.start)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="addNameToNamelessParameter",r=[e.Diagnostics.Parameter_has_a_name_but_no_type_Did_you_mean_0_Colon_1.code];function i(n,t,r){var i=e.getTokenAtPosition(t,r);if(!e.isIdentifier(i))return e.Debug.fail("add-name-to-nameless-parameter operates on identifiers, but got a "+e.Debug.formatSyntaxKind(i.kind));var a=i.parent;if(!e.isParameter(a))return e.Debug.fail("Tried to add a parameter name to a non-parameter: "+e.Debug.formatSyntaxKind(i.kind));var o=a.parent.parameters.indexOf(a);e.Debug.assert(!a.type,"Tried to add a parameter name to a parameter that already had one."),e.Debug.assert(o>-1,"Parameter not found in parent parameter list.");var s=e.factory.createParameterDeclaration(void 0,a.modifiers,a.dotDotDotToken,"arg"+o,a.questionToken,e.factory.createTypeReferenceNode(i,void 0),a.initializer);n.replaceNode(t,i,s)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span.start)});return[n.createCodeFixAction(t,a,e.Diagnostics.Add_parameter_name,t,e.Diagnostics.Add_names_to_all_parameters_without_names)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return i(e,n.file,n.start)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="annotateWithTypeFromJSDoc",r=[e.Diagnostics.JSDoc_types_may_be_moved_to_TypeScript_types.code];function i(n,t){var r=e.getTokenAtPosition(n,t);return e.tryCast(e.isParameter(r.parent)?r.parent.parent:r.parent,a)}function a(e){return c(e)&&o(e)}function o(n){return e.isFunctionLikeDeclaration(n)?n.parameters.some(o)||!n.type&&!!e.getJSDocReturnType(n):!n.type&&!!e.getJSDocType(n)}function s(n,t,r){if(e.isFunctionLikeDeclaration(r)&&(e.getJSDocReturnType(r)||r.parameters.some(function(n){return!!e.getJSDocType(n)}))){if(!r.typeParameters){var i=e.getJSDocTypeParameterDeclarations(r);i.length&&n.insertTypeParameters(t,r,i)}var a=e.isArrowFunction(r)&&!e.findChildOfKind(r,20,t);a&&n.insertNodeBefore(t,e.first(r.parameters),e.factory.createToken(20));for(var o=0,s=r.parameters;o<s.length;o++){var c=s[o];if(!c.type){var u=e.getJSDocType(c);u&&n.tryInsertTypeAnnotation(t,c,l(u))}}if(a&&n.insertNodeAfter(t,e.last(r.parameters),e.factory.createToken(21)),!r.type){var d=e.getJSDocReturnType(r);d&&n.tryInsertTypeAnnotation(t,r,l(d))}}else{var p=e.Debug.checkDefined(e.getJSDocType(r),"A JSDocType for this declaration should exist");e.Debug.assert(!r.type,"The JSDocType decl should have a type"),n.tryInsertTypeAnnotation(t,r,l(p))}}function c(n){return e.isFunctionLikeDeclaration(n)||252===n.kind||164===n.kind||165===n.kind}function l(n){switch(n.kind){case 307:case 308:return e.factory.createTypeReferenceNode("any",e.emptyArray);case 311:return u(n);case 310:return l(n.type);case 309:return d(n);case 313:return p(n);case 312:return f(n);case 176:return _(n);default:var t=e.visitEachChild(n,l,e.nullTransformationContext);return e.setEmitFlags(t,1),t}}function u(n){return e.factory.createUnionTypeNode([e.visitNode(n.type,l),e.factory.createTypeReferenceNode("undefined",e.emptyArray)])}function d(n){return e.factory.createUnionTypeNode([e.visitNode(n.type,l),e.factory.createTypeReferenceNode("null",e.emptyArray)])}function p(n){return e.factory.createArrayTypeNode(e.visitNode(n.type,l))}function f(n){var t;return e.factory.createFunctionTypeNode(e.emptyArray,n.parameters.map(m),null!==(t=n.type)&&void 0!==t?t:e.factory.createKeywordTypeNode(129))}function m(n){var t=n.parent.parameters.indexOf(n),r=313===n.type.kind&&t===n.parent.parameters.length-1,i=n.name||(r?"rest":"arg"+t),a=r?e.factory.createToken(25):n.dotDotDotToken;return e.factory.createParameterDeclaration(n.decorators,n.modifiers,a,i,n.questionToken,e.visitNode(n.type,l),n.initializer)}function _(n){var t=n.typeName,r=n.typeArguments;if(e.isIdentifier(n.typeName)){if(e.isJSDocIndexSignature(n))return g(n);var i=n.typeName.text;switch(n.typeName.text){case"String":case"Boolean":case"Object":case"Number":i=i.toLowerCase();break;case"array":case"date":case"promise":i=i[0].toUpperCase()+i.slice(1);break}t=e.factory.createIdentifier(i),r="Array"!==i&&"Promise"!==i||n.typeArguments?e.visitNodes(n.typeArguments,l):e.factory.createNodeArray([e.factory.createTypeReferenceNode("any",e.emptyArray)])}return e.factory.createTypeReferenceNode(t,r)}function g(n){var t=e.factory.createParameterDeclaration(void 0,void 0,void 0,145===n.typeArguments[0].kind?"n":"s",void 0,e.factory.createTypeReferenceNode(145===n.typeArguments[0].kind?"number":"string",[]),void 0),r=e.factory.createTypeLiteralNode([e.factory.createIndexSignature(void 0,void 0,[t],n.typeArguments[1])]);return e.setEmitFlags(r,1),r}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=i(r.sourceFile,r.span.start);if(a){var o=e.textChanges.ChangeTracker.with(r,function(e){return s(e,r.sourceFile,a)});return[n.createCodeFixAction(t,o,e.Diagnostics.Annotate_with_type_from_JSDoc,t,e.Diagnostics.Annotate_everything_with_types_from_JSDoc)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){var t=i(n.file,n.start);t&&s(e,n.file,t)})}}),n.parameterShouldGetTypeFromJSDoc=a})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="convertFunctionToEs6Class",r=[e.Diagnostics.This_constructor_function_may_be_converted_to_a_class_declaration.code];function i(n,t,r,i,c,l){var u=i.getSymbolAtLocation(e.getTokenAtPosition(t,r));if(u&&u.valueDeclaration&&19&u.flags){var d=u.valueDeclaration;if(e.isFunctionDeclaration(d))n.replaceNode(t,d,g(d));else if(e.isVariableDeclaration(d)){var p=_(d);if(!p)return;var f=d.parent.parent;e.isVariableDeclarationList(d.parent)&&d.parent.declarations.length>1?(n.delete(t,d),n.insertNodeAfter(t,f,p)):n.replaceNode(t,f,p)}}function m(r){var i=[];return r.members&&r.members.forEach(function(e,r){if("constructor"===r&&e.valueDeclaration)n.delete(t,e.valueDeclaration.parent);else{var a=d(e,void 0);a&&i.push.apply(i,a)}}),r.exports&&r.exports.forEach(function(n){if("prototype"===n.name&&n.declarations){var t=n.declarations[0];if(1===n.declarations.length&&e.isPropertyAccessExpression(t)&&e.isBinaryExpression(t.parent)&&63===t.parent.operatorToken.kind&&e.isObjectLiteralExpression(t.parent.right)){var r=t.parent.right,a=d(r.symbol,void 0);a&&i.push.apply(i,a)}}else{a=d(n,[e.factory.createToken(124)]);a&&i.push.apply(i,a)}}),i;function u(n,t){return e.isAccessExpression(n)?!(!e.isPropertyAccessExpression(n)||!o(n))||e.isFunctionLike(t):e.every(n.properties,function(n){return!(!e.isMethodDeclaration(n)&&!e.isGetOrSetAccessorDeclaration(n))||(!!(e.isPropertyAssignment(n)&&e.isFunctionExpression(n.initializer)&&n.name)||!!o(n))})}function d(r,i){var d=[];if(!(8192&r.flags)&&!(4096&r.flags))return d;var p=r.valueDeclaration,f=p.parent,m=f.right;if(!u(p,m))return d;var _=f.parent&&236===f.parent.kind?f.parent:f;if(n.delete(t,_),!m)return d.push(e.factory.createPropertyDeclaration([],i,r.name,void 0,void 0,void 0)),d;if(e.isAccessExpression(p)&&(e.isFunctionExpression(m)||e.isArrowFunction(m))){var g=e.getQuotePreference(t,c),y=s(p,l,g);return y?v(d,m,y):d}if(e.isObjectLiteralExpression(m))return e.flatMap(m.properties,function(n){return e.isMethodDeclaration(n)||e.isGetOrSetAccessorDeclaration(n)?d.concat(n):e.isPropertyAssignment(n)&&e.isFunctionExpression(n.initializer)?v(d,n.initializer,n.name):o(n)?d:[]});if(e.isSourceFileJS(t))return d;if(!e.isPropertyAccessExpression(p))return d;var h=e.factory.createPropertyDeclaration(void 0,i,p.name,void 0,void 0,m);return e.copyLeadingComments(f.parent,h,t),d.push(h),d;function v(n,t,r){return e.isFunctionExpression(t)?b(n,t,r):E(n,t,r)}function b(n,r,o){var s=e.concatenate(i,a(r,130)),c=e.factory.createMethodDeclaration(void 0,s,void 0,o,void 0,void 0,r.parameters,void 0,r.body);return e.copyLeadingComments(f,c,t),n.concat(c)}function E(n,r,o){var s,c=r.body;s=233===c.kind?c:e.factory.createBlock([e.factory.createReturnStatement(c)]);var l=e.concatenate(i,a(r,130)),u=e.factory.createMethodDeclaration(void 0,l,void 0,o,void 0,void 0,r.parameters,void 0,s);return e.copyLeadingComments(f,u,t),n.concat(u)}}}function _(n){var t=n.initializer;if(t&&e.isFunctionExpression(t)&&e.isIdentifier(n.name)){var r=m(n.symbol);t.body&&r.unshift(e.factory.createConstructorDeclaration(void 0,void 0,t.parameters,t.body));var i=a(n.parent.parent,93),o=e.factory.createClassDeclaration(void 0,i,n.name,void 0,void 0,r);return o}}function g(n){var t=m(u);n.body&&t.unshift(e.factory.createConstructorDeclaration(void 0,void 0,n.parameters,n.body));var r=a(n,93),i=e.factory.createClassDeclaration(void 0,r,n.name,void 0,void 0,t);return i}}function a(n,t){return e.filter(n.modifiers,function(e){return e.kind===t})}function o(n){return!!n.name&&!(!e.isIdentifier(n.name)||"constructor"!==n.name.text)}function s(n,t,r){if(e.isPropertyAccessExpression(n))return n.name;var i=n.argumentExpression;return e.isNumericLiteral(i)?i:e.isStringLiteralLike(i)?e.isIdentifierText(i.text,t.target)?e.factory.createIdentifier(i.text):e.isNoSubstitutionTemplateLiteral(i)?e.factory.createStringLiteral(i.text,0===r):i:void 0}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span.start,r.program.getTypeChecker(),r.preferences,r.program.getCompilerOptions())});return[n.createCodeFixAction(t,a,e.Diagnostics.Convert_function_to_an_ES2015_class,t,e.Diagnostics.Convert_all_constructor_functions_to_classes)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){return i(n,t.file,t.start,e.program.getTypeChecker(),e.preferences,e.program.getCompilerOptions())})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t,r="convertToAsyncFunction",i=[e.Diagnostics.This_may_be_converted_to_an_async_function.code],o=!0;function s(n,t,r,i){var a,o=e.getTokenAtPosition(t,r);if(a=e.isIdentifier(o)&&e.isVariableDeclaration(o.parent)&&o.parent.initializer&&e.isFunctionLikeDeclaration(o.parent.initializer)?o.parent.initializer:e.tryCast(e.getContainingFunction(e.getTokenAtPosition(t,r)),e.canBeConvertedToAsync),a){var s=new e.Map,u=e.isInJSFile(a),d=l(a,i),f=p(a,i,s);if(e.returnsPromise(f,i)){var m=f.body&&e.isBlock(f.body)?c(f.body,i):e.emptyArray,g={checker:i,synthNamesMap:s,setOfExpressionsToReturn:d,isInJSFile:u};if(m.length){var y=a.modifiers?a.modifiers.end:a.decorators?e.skipTrivia(t.text,a.decorators.end):a.getStart(t),h=a.modifiers?{prefix:" "}:{suffix:" "};n.insertModifierAt(t,y,130,h);for(var v=function(r){e.forEachChild(r,function i(a){if(e.isCallExpression(a)){var o=_(a,g);n.replaceNodeWithNodes(t,r,o)}else e.isFunctionLike(a)||e.forEachChild(a,i)})},b=0,E=m;b<E.length;b++){var T=E[b];v(T)}}}}}function c(n,t){var r=[];return e.forEachReturnStatement(n,function(n){e.isReturnStatementWithFixablePromiseHandler(n,t)&&r.push(n)}),r}function l(n,t){if(!n.body)return new e.Set;var r=new e.Set;return e.forEachChild(n.body,function n(i){u(i,t,"then")?(r.add(e.getNodeId(i)),e.forEach(i.arguments,n)):u(i,t,"catch")?(r.add(e.getNodeId(i)),e.forEachChild(i,n)):d(i,t)?r.add(e.getNodeId(i)):e.forEachChild(i,n)}),r}function u(n,t,r){if(!e.isCallExpression(n))return!1;var i=e.hasPropertyAccessExpressionWithName(n,r),a=i&&t.getTypeAtLocation(n);return!(!a||!t.getPromisedTypeOfPromise(a))}function d(n,t){return!!e.isExpression(n)&&!!t.getPromisedTypeOfPromise(t.getTypeAtLocation(n))}function p(n,t,r){var i=new e.Map,a=e.createMultiMap();return e.forEachChild(n,function n(o){if(e.isIdentifier(o)){var s=t.getSymbolAtLocation(o);if(s){var c=t.getTypeAtLocation(o),l=x(c,t),u=e.getSymbolId(s).toString();if(!l||e.isParameter(o.parent)||e.isFunctionLikeDeclaration(o.parent)||r.has(u)){if(o.parent&&(e.isParameter(o.parent)||e.isVariableDeclaration(o.parent)||e.isBindingElement(o.parent))){var d=o.text,p=a.get(d);if(p&&p.some(function(e){return e!==s})){var m=f(o,a);i.set(u,m.identifier),r.set(u,m),a.add(d,s)}else{var _=e.getSynthesizedDeepClone(o);r.set(u,N(_)),a.add(d,s)}}}else{var g=e.firstOrUndefined(l.parameters),y=(null===g||void 0===g?void 0:g.valueDeclaration)&&e.isParameter(g.valueDeclaration)&&e.tryCast(g.valueDeclaration.name,e.isIdentifier)||e.factory.createUniqueName("result",16),h=f(y,a);r.set(u,h),a.add(y.text,s)}}}else e.forEachChild(o,n)}),e.getSynthesizedDeepCloneWithReplacements(n,!0,function(n){if(e.isBindingElement(n)&&e.isIdentifier(n.name)&&e.isObjectBindingPattern(n.parent)){var r=t.getSymbolAtLocation(n.name),a=r&&i.get(String(e.getSymbolId(r)));if(a&&a.text!==(n.name||n.propertyName).getText())return e.factory.createBindingElement(n.dotDotDotToken,n.propertyName||n.name,a,n.initializer)}else if(e.isIdentifier(n)){r=t.getSymbolAtLocation(n),a=r&&i.get(String(e.getSymbolId(r)));if(a)return e.factory.createIdentifier(a.text)}})}function f(n,t){var r=(t.get(n.text)||e.emptyArray).length,i=0===r?n:e.factory.createIdentifier(n.text+"_"+r);return N(i)}function m(){return o=!1,e.emptyArray}function _(n,t,r){if(u(n,t.checker,"then"))return 0===n.arguments.length?m():h(n,t,r);if(u(n,t.checker,"catch"))return g(n,t,r);if(e.isPropertyAccessExpression(n))return _(n.expression,t,r);var i=t.checker.getTypeAtLocation(n);return i&&t.checker.getPromisedTypeOfPromise(i)?(e.Debug.assertNode(n.original.parent,e.isPropertyAccessExpression),v(n,t,r)):m()}function g(n,t,r){var i,a=e.singleOrUndefined(n.arguments),o=a?A(a,t):void 0;r&&!w(n,t)&&(R(r)?(i=r,t.synthNamesMap.forEach(function(e,n){if(e.identifier.text===r.identifier.text){var i=y(r);t.synthNamesMap.set(n,i)}})):i=N(e.factory.createUniqueName("result",16),r.types),i.hasBeenDeclared=!0);var s,c,l=e.factory.createBlock(_(n.expression,t,i)),u=a?T(a,i,o,n,t):e.emptyArray,d=o?R(o)?o.identifier.text:o.bindingPattern:"e",p=e.factory.createVariableDeclaration(d),f=e.factory.createCatchClause(p,e.factory.createBlock(u));if(i&&!w(n,t)){c=e.getSynthesizedDeepClone(i.identifier);var m=i.types,g=t.checker.getUnionType(m,2),h=t.isInJSFile?void 0:t.checker.typeToTypeNode(g,void 0,void 0),v=[e.factory.createVariableDeclaration(c,void 0,h)];s=e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList(v,1))}var b=e.factory.createTryStatement(l,f,void 0),E=r&&c&&O(r)&&e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(e.getSynthesizedDeepClone(r.bindingPattern),void 0,void 0,c)],2));return e.compact([s,b,E])}function y(n){var t=e.factory.createUniqueName(n.identifier.text,16);return N(t)}function h(n,t,r){var i=n.arguments,a=i[0],o=i[1],s=A(a,t),c=T(a,r,s,n,t);if(o){var l=A(o,t),u=e.factory.createBlock(_(n.expression,t,s).concat(c)),d=T(o,r,l,n,t),p=l?R(l)?l.identifier.text:l.bindingPattern:"e",f=e.factory.createVariableDeclaration(p),m=e.factory.createCatchClause(f,e.factory.createBlock(d));return[e.factory.createTryStatement(u,m,void 0)]}return _(n.expression,t,s).concat(c)}function v(n,t,r){return w(n,t)?[e.factory.createReturnStatement(e.getSynthesizedDeepClone(n))]:b(r,e.factory.createAwaitExpression(n),void 0)}function b(n,t,r){return!n||C(n)?[e.factory.createExpressionStatement(t)]:R(n)&&n.hasBeenDeclared?[e.factory.createExpressionStatement(e.factory.createAssignment(e.getSynthesizedDeepClone(n.identifier),t))]:[e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(e.getSynthesizedDeepClone(I(n)),void 0,r,t)],2))]}function E(n,t){if(t&&n){var r=e.factory.createUniqueName("result",16);return a(a([],b(N(r),n,t),!0),[e.factory.createReturnStatement(r)],!1)}return[e.factory.createReturnStatement(n)]}function T(n,t,r,i,a){var o,s,c,l,u;switch(n.kind){case 104:break;case 204:case 79:if(!r)break;var d=e.factory.createCallExpression(e.getSynthesizedDeepClone(n),void 0,R(r)?[r.identifier]:[]);if(w(i,a))return E(d,null===(o=i.typeArguments)||void 0===o?void 0:o[0]);var p=a.checker.getTypeAtLocation(n),f=a.checker.getSignaturesOfType(p,0);if(!f.length)return m();var _=f[0].getReturnType(),g=b(t,e.factory.createAwaitExpression(d),null===(s=i.typeArguments)||void 0===s?void 0:s[0]);return t&&t.types.push(_),g;case 211:case 212:var y=n.body,h=null===(c=x(a.checker.getTypeAtLocation(n),a.checker))||void 0===c?void 0:c.getReturnType();if(e.isBlock(y)){for(var v=[],T=!1,A=0,C=y.statements;A<C.length;A++){var I=C[A];if(e.isReturnStatement(I))if(T=!0,e.isReturnStatementWithFixablePromiseHandler(I,a.checker))v=v.concat(D(a,[I],t));else{var N=h&&I.expression?S(a.checker,h,I.expression):I.expression;v.push.apply(v,E(N,null===(l=i.typeArguments)||void 0===l?void 0:l[0]))}else v.push(I)}return w(i,a)?v.map(function(n){return e.getSynthesizedDeepClone(n)}):L(v,t,a,T)}var k=e.isFixablePromiseHandler(y,a.checker)?[e.factory.createReturnStatement(y)]:e.emptyArray,O=D(a,k,t);if(O.length>0)return O;if(h){N=S(a.checker,h,y);if(w(i,a))return E(N,null===(u=i.typeArguments)||void 0===u?void 0:u[0]);var P=b(t,N,void 0);return t&&t.types.push(h),P}return m();default:return m()}return e.emptyArray}function S(n,t,r){var i=e.getSynthesizedDeepClone(r);return n.getPromisedTypeOfPromise(t)?e.factory.createAwaitExpression(i):i}function x(n,t){var r=t.getSignaturesOfType(n,0);return e.lastOrUndefined(r)}function L(n,t,r,i){for(var a=[],o=0,s=n;o<s.length;o++){var c=s[o];if(e.isReturnStatement(c)){if(c.expression){var l=d(c.expression,r.checker)?e.factory.createAwaitExpression(c.expression):c.expression;void 0===t?a.push(e.factory.createExpressionStatement(l)):a.push(e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(I(t),void 0,void 0,l)],2)))}}else a.push(e.getSynthesizedDeepClone(c))}return i||void 0===t||a.push(e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(I(t),void 0,void 0,e.factory.createIdentifier("undefined"))],2))),a}function D(n,t,r){for(var i=[],a=0,o=t;a<o.length;a++){var s=o[a];e.forEachChild(s,function t(a){if(e.isCallExpression(a)){var o=_(a,n,r);if(i=i.concat(o),i.length>0)return}else e.isFunctionLike(a)||e.forEachChild(a,t)})}return i}function A(n,t){var r,i=[];if(e.isFunctionLikeDeclaration(n)){if(n.parameters.length>0){var a=n.parameters[0].name;r=o(a)}}else e.isIdentifier(n)?r=s(n):e.isPropertyAccessExpression(n)&&e.isIdentifier(n.name)&&(r=s(n.name));if(r&&!("identifier"in r&&"undefined"===r.identifier.text))return r;function o(n){if(e.isIdentifier(n))return s(n);var t=e.flatMap(n.elements,function(n){return e.isOmittedExpression(n)?[]:[o(n.name)]});return k(n,t)}function s(n){var r=l(n),a=c(r);if(!a)return N(n,i);var o=t.synthNamesMap.get(e.getSymbolId(a).toString());return o||N(n,i)}function c(e){return e.symbol?e.symbol:t.checker.getSymbolAtLocation(e)}function l(e){return e.original?e.original:e}}function C(n){return!n||(R(n)?!n.identifier.text:e.every(n.elements,C))}function I(e){return R(e)?e.identifier:e.bindingPattern}function N(e,n){return void 0===n&&(n=[]),{kind:0,identifier:e,types:n,hasBeenDeclared:!1}}function k(n,t,r){return void 0===t&&(t=e.emptyArray),void 0===r&&(r=[]),{kind:1,bindingPattern:n,elements:t,types:r}}function R(e){return 0===e.kind}function O(e){return 1===e.kind}function w(n,t){return!!n.original&&t.setOfExpressionsToReturn.has(e.getNodeId(n.original))}n.registerCodeFix({errorCodes:i,getCodeActions:function(t){o=!0;var i=e.textChanges.ChangeTracker.with(t,function(e){return s(e,t.sourceFile,t.span.start,t.program.getTypeChecker())});return o?[n.createCodeFixAction(r,i,e.Diagnostics.Convert_to_async_function,r,e.Diagnostics.Convert_all_to_async_functions)]:[]},fixIds:[r],getAllCodeActions:function(e){return n.codeFixAll(e,i,function(n,t){return s(n,t.file,t.start,e.program.getTypeChecker())})}}),function(e){e[e["Identifier"]=0]="Identifier",e[e["BindingPattern"]=1]="BindingPattern"}(t||(t={}))})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){function t(n,t,r,i){for(var a=0,o=n.imports;a<o.length;a++){var s=o[a],c=e.getResolvedModule(n,s.text);if(c&&c.resolvedFileName===t.fileName){var l=e.importFromModuleSpecifier(s);switch(l.kind){case 263:r.replaceNode(n,l,e.makeImport(l.name,void 0,s,i));break;case 206:e.isRequireCall(l,!1)&&r.replaceNode(n,l,e.factory.createPropertyAccessExpression(e.getSynthesizedDeepClone(l),"default"));break}}}}function r(n,t,r,o,l){var u={original:T(n),additional:new e.Set},d=i(n,t,u);a(n,d,r);for(var p,f=!1,m=0,_=e.filter(n.statements,e.isVariableStatement);m<_.length;m++){var g=_[m],y=c(n,g,r,t,u,o,l);y&&e.copyEntries(y,null!==p&&void 0!==p?p:p=new e.Map)}for(var h=0,v=e.filter(n.statements,function(n){return!e.isVariableStatement(n)});h<v.length;h++){g=v[h];var b=s(n,g,t,r,u,o,d,p,l);f=f||b}return null===p||void 0===p||p.forEach(function(e,t){r.replaceNode(n,t,e)}),f}function i(n,t,r){var i=new e.Map;return o(n,function(n){var a=n.name,o=a.text,s=a.originalKeywordKind;!i.has(o)&&(void 0!==s&&e.isNonContextualKeyword(s)||t.resolveName(o,n,111551,!0))&&i.set(o,E("_"+o,r))}),i}function a(n,t,r){o(n,function(i,a){if(!a){var o=i.name.text;r.replaceNode(n,i,e.factory.createIdentifier(t.get(o)||o))}})}function o(n,t){n.forEachChild(function r(i){if(e.isPropertyAccessExpression(i)&&e.isExportsOrModuleExportsOrAlias(n,i.expression)&&e.isIdentifier(i.name)){var a=i.parent;t(i,e.isBinaryExpression(a)&&a.left===i&&63===a.operatorToken.kind)}i.forEachChild(r)})}function s(n,t,r,i,a,o,s,l,d){switch(t.kind){case 235:return c(n,t,i,r,a,o,d),!1;case 236:var p=t.expression;switch(p.kind){case 206:return e.isRequireCall(p,!0)&&i.replaceNode(n,t,e.makeImport(void 0,void 0,p.arguments[0],d)),!1;case 219:var f=p.operatorToken;return 63===f.kind&&u(n,r,p,i,s,l)}default:return!1}}function c(n,t,r,i,a,o,s){var c,u=t.declarationList,d=!1,p=e.map(u.declarations,function(t){var r=t.name,c=t.initializer;if(c){if(e.isExportsOrModuleExportsOrAlias(n,c))return d=!0,k([]);if(e.isRequireCall(c,!0))return d=!0,v(r,c.arguments[0],i,a,o,s);if(e.isPropertyAccessExpression(c)&&e.isRequireCall(c.expression,!0))return d=!0,l(r,c.name.text,c.expression.arguments[0],a,s)}return k([e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([t],u.flags))])});if(d)return r.replaceNodeWithNodes(n,t,e.flatMap(p,function(e){return e.newImports})),e.forEach(p,function(n){n.useSitesToUnqualify&&e.copyEntries(n.useSitesToUnqualify,null!==c&&void 0!==c?c:c=new e.Map)}),c}function l(n,t,r,i,a){switch(n.kind){case 199:case 200:var o=E(t,i);return k([A(o,t,r,a),I(void 0,n,e.factory.createIdentifier(o))]);case 79:return k([A(n.text,t,r,a)]);default:return e.Debug.assertNever(n,"Convert to ES6 module got invalid syntax form "+n.kind)}}function u(n,t,r,i,a,o){var s=r.left,c=r.right;if(!e.isPropertyAccessExpression(s))return!1;if(e.isExportsOrModuleExportsOrAlias(n,s)){if(!e.isExportsOrModuleExportsOrAlias(n,c)){var l=e.isObjectLiteralExpression(c)?d(c,o):e.isRequireCall(c,!0)?f(c.arguments[0],t):void 0;return l?(i.replaceNodeWithNodes(n,r.parent,l[0]),l[1]):(i.replaceRangeWithText(n,e.createRange(s.getStart(n),c.pos),"export default"),!0)}i.delete(n,r.parent)}else e.isExportsOrModuleExportsOrAlias(n,s.expression)&&p(n,r,i,a);return!1}function d(n,t){var r=e.mapAllOrFail(n.properties,function(n){switch(n.kind){case 170:case 171:case 292:case 293:return;case 291:return e.isIdentifier(n.name)?y(n.name.text,n.initializer,t):void 0;case 167:return e.isIdentifier(n.name)?L(n.name.text,[e.factory.createToken(93)],n,t):void 0;default:e.Debug.assertNever(n,"Convert to ES6 got invalid prop kind "+n.kind)}});return r&&[r,!1]}function p(n,t,r,i){var a=t.left.name.text,o=i.get(a);if(void 0!==o){var s=[I(void 0,o,t.right),N([e.factory.createExportSpecifier(o,a)])];r.replaceNodeWithNodes(n,t.parent,s)}else g(t,n,r)}function f(n,t){var r=n.text,i=t.getSymbolAtLocation(n),a=i?i.exports:e.emptyMap;return a.has("export=")?[[_(r)],!0]:a.has("default")?a.size>1?[[m(r),_(r)],!0]:[[_(r)],!0]:[[m(r)],!1]}function m(e){return N(void 0,e)}function _(n){return N([e.factory.createExportSpecifier(void 0,"default")],n)}function g(n,t,r){var i=n.left,a=n.right,o=n.parent,s=i.name.text;if(!(e.isFunctionExpression(a)||e.isArrowFunction(a)||e.isClassExpression(a))||a.name&&a.name.text!==s)r.replaceNodeRangeWithNodes(t,i.expression,e.findChildOfKind(i,24,t),[e.factory.createToken(93),e.factory.createToken(85)],{joiner:" ",suffix:" "});else{r.replaceRange(t,{pos:i.getStart(t),end:a.getStart(t)},e.factory.createToken(93),{suffix:" "}),a.name||r.insertName(t,a,s);var c=e.findChildOfKind(o,26,t);c&&r.delete(t,c)}}function y(n,t,r){var i=[e.factory.createToken(93)];switch(t.kind){case 211:var a=t.name;if(a&&a.text!==n)return o();case 212:return L(n,i,t,r);case 224:return D(n,i,t,r);default:return o()}function o(){return I(i,e.factory.createIdentifier(n),h(t,r))}}function h(n,t){return t&&e.some(e.arrayFrom(t.keys()),function(t){return e.rangeContainsRange(n,t)})?e.isArray(n)?e.getSynthesizedDeepClonesWithReplacements(n,!0,r):e.getSynthesizedDeepCloneWithReplacements(n,!0,r):n;function r(e){if(204===e.kind){var n=t.get(e);return t.delete(e),n}}}function v(t,r,i,a,o,s){switch(t.kind){case 199:var c=e.mapAllOrFail(t.elements,function(n){return n.dotDotDotToken||n.initializer||n.propertyName&&!e.isIdentifier(n.propertyName)||!e.isIdentifier(n.name)?void 0:C(n.propertyName&&n.propertyName.text,n.name.text)});if(c)return k([e.makeImport(void 0,c,r,s)]);case 200:var l=E(n.moduleSpecifierToValidIdentifier(r.text,o),a);return k([e.makeImport(e.factory.createIdentifier(l),void 0,r,s),I(void 0,e.getSynthesizedDeepClone(t),e.factory.createIdentifier(l))]);case 79:return b(t,r,i,a,s);default:return e.Debug.assertNever(t,"Convert to ES6 module got invalid name kind "+t.kind)}}function b(n,t,r,i,a){for(var o,s=r.getSymbolAtLocation(n),c=new e.Map,l=!1,u=0,d=i.original.get(n.text);u<d.length;u++){var p=d[u];if(r.getSymbolAtLocation(p)===s&&p!==n){var f=p.parent;if(e.isPropertyAccessExpression(f)){var m=f.name.text;if("default"===m){l=!0;var _=p.getText();(null!==o&&void 0!==o?o:o=new e.Map).set(f,e.factory.createIdentifier(_))}else{e.Debug.assert(f.expression===p,"Didn't expect expression === use");var g=c.get(m);void 0===g&&(g=E(m,i),c.set(m,g)),(null!==o&&void 0!==o?o:o=new e.Map).set(f,e.factory.createIdentifier(g))}}else l=!0}}var y=0===c.size?void 0:e.arrayFrom(e.mapIterator(c.entries(),function(n){var t=n[0],r=n[1];return e.factory.createImportSpecifier(t===r?void 0:e.factory.createIdentifier(t),e.factory.createIdentifier(r))}));return y||(l=!0),k([e.makeImport(l?e.getSynthesizedDeepClone(n):void 0,y,t,a)],o)}function E(e,n){while(n.original.has(e)||n.additional.has(e))e="_"+e;return n.additional.add(e),e}function T(n){var t=e.createMultiMap();return S(n,function(e){return t.add(e.text,e)}),t}function S(n,t){e.isIdentifier(n)&&x(n)&&t(n),n.forEachChild(function(e){return S(e,t)})}function x(e){var n=e.parent;switch(n.kind){case 204:return n.name!==e;case 201:return n.propertyName!==e;case 268:return n.propertyName!==e;default:return!0}}function L(n,t,r,i){return e.factory.createFunctionDeclaration(e.getSynthesizedDeepClones(r.decorators),e.concatenate(t,e.getSynthesizedDeepClones(r.modifiers)),e.getSynthesizedDeepClone(r.asteriskToken),n,e.getSynthesizedDeepClones(r.typeParameters),e.getSynthesizedDeepClones(r.parameters),e.getSynthesizedDeepClone(r.type),e.factory.converters.convertToFunctionBlock(h(r.body,i)))}function D(n,t,r,i){return e.factory.createClassDeclaration(e.getSynthesizedDeepClones(r.decorators),e.concatenate(t,e.getSynthesizedDeepClones(r.modifiers)),n,e.getSynthesizedDeepClones(r.typeParameters),e.getSynthesizedDeepClones(r.heritageClauses),h(r.members,i))}function A(n,t,r,i){return"default"===t?e.makeImport(e.factory.createIdentifier(n),void 0,r,i):e.makeImport(void 0,[C(t,n)],r,i)}function C(n,t){return e.factory.createImportSpecifier(void 0!==n&&n!==t?e.factory.createIdentifier(n):void 0,e.factory.createIdentifier(t))}function I(n,t,r){return e.factory.createVariableStatement(n,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(t,void 0,void 0,r)],2))}function N(n,t){return e.factory.createExportDeclaration(void 0,void 0,!1,n&&e.factory.createNamedExports(n),void 0===t?void 0:e.factory.createStringLiteral(t))}function k(e,n){return{newImports:e,useSitesToUnqualify:n}}n.registerCodeFix({errorCodes:[e.Diagnostics.File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module.code],getCodeActions:function(i){var a=i.sourceFile,o=i.program,s=i.preferences,c=e.textChanges.ChangeTracker.with(i,function(n){var i=r(a,o.getTypeChecker(),n,o.getCompilerOptions().target,e.getQuotePreference(a,s));if(i)for(var c=0,l=o.getSourceFiles();c<l.length;c++){var u=l[c];t(u,a,n,e.getQuotePreference(u,s))}});return[n.createCodeFixActionWithoutFixAll("convertToEs6Module",c,e.Diagnostics.Convert_to_ES6_module)]}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="correctQualifiedNameToIndexedAccessType",r=[e.Diagnostics.Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_property_1_in_0_with_0_1.code];function i(n,t){var r=e.findAncestor(e.getTokenAtPosition(n,t),e.isQualifiedName);return e.Debug.assert(!!r,"Expected position to be owned by a qualified name."),e.isIdentifier(r.left)?r:void 0}function a(n,t,r){var i=r.right.text,a=e.factory.createIndexedAccessTypeNode(e.factory.createTypeReferenceNode(r.left,void 0),e.factory.createLiteralTypeNode(e.factory.createStringLiteral(i)));n.replaceNode(t,r,a)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=i(r.sourceFile,r.span.start);if(o){var s=e.textChanges.ChangeTracker.with(r,function(e){return a(e,r.sourceFile,o)}),c=o.left.text+'["'+o.right.text+'"]';return[n.createCodeFixAction(t,s,[e.Diagnostics.Rewrite_as_the_indexed_access_type_0,c],t,e.Diagnostics.Rewrite_all_as_indexed_access_types)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){var t=i(n.file,n.start);t&&a(e,n.file,t)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t=[e.Diagnostics.Re_exporting_a_type_when_the_isolatedModules_flag_is_provided_requires_using_export_type.code],r="convertToTypeOnlyExport";function i(n,t){return e.tryCast(e.getTokenAtPosition(t,n.start).parent,e.isExportSpecifier)}function a(n,t,r){if(t){var i=t.parent,a=i.parent,s=o(t,r);if(s.length===i.elements.length)n.insertModifierBefore(r.sourceFile,150,i);else{var c=e.factory.updateExportDeclaration(a,a.decorators,a.modifiers,!1,e.factory.updateNamedExports(i,e.filter(i.elements,function(n){return!e.contains(s,n)})),a.moduleSpecifier),l=e.factory.createExportDeclaration(void 0,void 0,!0,e.factory.createNamedExports(s),a.moduleSpecifier);n.replaceNode(r.sourceFile,a,c,{leadingTriviaOption:e.textChanges.LeadingTriviaOption.IncludeAll,trailingTriviaOption:e.textChanges.TrailingTriviaOption.Exclude}),n.insertNodeAfter(r.sourceFile,a,l)}}}function o(n,r){var i=n.parent;if(1===i.elements.length)return i.elements;var a=e.getDiagnosticsWithinSpan(e.createTextSpanFromNode(i),r.program.getSemanticDiagnostics(r.sourceFile,r.cancellationToken));return e.filter(i.elements,function(r){var i;return r===n||(null===(i=e.findDiagnosticForNode(r,a))||void 0===i?void 0:i.code)===t[0]})}n.registerCodeFix({errorCodes:t,getCodeActions:function(t){var o=e.textChanges.ChangeTracker.with(t,function(e){return a(e,i(t.span,t.sourceFile),t)});if(o.length)return[n.createCodeFixAction(r,o,e.Diagnostics.Convert_to_type_only_export,r,e.Diagnostics.Convert_all_re_exported_types_to_type_only_exports)]},fixIds:[r],getAllCodeActions:function(r){var o=new e.Map;return n.codeFixAll(r,t,function(n,t){var s=i(t,r.sourceFile);s&&e.addToSeen(o,e.getNodeId(s.parent.parent))&&a(n,s,r)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t=[e.Diagnostics.This_import_is_never_used_as_a_value_and_must_use_import_type_because_importsNotUsedAsValues_is_set_to_error.code],r="convertToTypeOnlyImport";function i(n,t){return e.tryCast(e.getTokenAtPosition(t,n.start).parent,e.isImportDeclaration)}function a(n,t,r){if(null===t||void 0===t?void 0:t.importClause){var i=t.importClause;n.insertText(r.sourceFile,t.getStart()+"import".length," type"),i.name&&i.namedBindings&&(n.deleteNodeRangeExcludingEnd(r.sourceFile,i.name,t.importClause.namedBindings),n.insertNodeBefore(r.sourceFile,t,e.factory.updateImportDeclaration(t,void 0,void 0,e.factory.createImportClause(!0,i.name,void 0),t.moduleSpecifier)))}}n.registerCodeFix({errorCodes:t,getCodeActions:function(t){var o=e.textChanges.ChangeTracker.with(t,function(e){var n=i(t.span,t.sourceFile);a(e,n,t)});if(o.length)return[n.createCodeFixAction(r,o,e.Diagnostics.Convert_to_type_only_import,r,e.Diagnostics.Convert_all_imports_not_used_as_a_value_to_type_only_imports)]},fixIds:[r],getAllCodeActions:function(e){return n.codeFixAll(e,t,function(n,t){var r=i(t,e.sourceFile);a(n,r,e)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="convertLiteralTypeToMappedType",r=[e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Did_you_mean_to_use_1_in_0.code];function i(n,t){var r=e.getTokenAtPosition(n,t);if(e.isIdentifier(r)){var i=e.cast(r.parent.parent,e.isPropertySignature),a=r.getText(n);return{container:e.cast(i.parent,e.isTypeLiteralNode),typeNode:i.type,constraint:a,name:"K"===a?"P":"K"}}}function a(n,t,r){var i=r.container,a=r.typeNode,o=r.constraint,s=r.name;n.replaceNode(t,i,e.factory.createMappedTypeNode(void 0,e.factory.createTypeParameterDeclaration(s,e.factory.createTypeReferenceNode(o)),void 0,void 0,a))}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,c=i(o,s.start);if(c){var l=c.name,u=c.constraint,d=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,c)});return[n.createCodeFixAction(t,d,[e.Diagnostics.Convert_0_to_1_in_0,u,l],t,e.Diagnostics.Convert_all_type_literals_to_mapped_type)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){var t=i(n.file,n.start);t&&a(e,n.file,t)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t=[e.Diagnostics.Class_0_incorrectly_implements_interface_1.code,e.Diagnostics.Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass.code],r="fixClassIncorrectlyImplementsInterface";function i(n,t){return e.Debug.checkDefined(e.getContainingClass(e.getTokenAtPosition(n,t)),"There should be a containing class")}function a(n){return!n.valueDeclaration||!(8&e.getEffectiveModifierFlags(n.valueDeclaration))}function o(t,r,i,o,c,l){var u=t.program.getTypeChecker(),d=s(o,u),p=u.getTypeAtLocation(r),f=u.getPropertiesOfType(p),m=f.filter(e.and(a,function(e){return!d.has(e.escapedName)})),_=u.getTypeAtLocation(o),g=e.find(o.members,function(n){return e.isConstructorDeclaration(n)});_.getNumberIndexType()||h(p,1),_.getStringIndexType()||h(p,0);var y=n.createImportAdder(i,t.program,l,t.host);function h(e,r){var a=u.getIndexInfoOfType(e,r);a&&v(i,o,u.indexInfoToIndexSignatureDeclaration(a,o,void 0,n.getNoopSymbolTrackerWithResolver(t)))}function v(e,n,t){g?c.insertNodeAfter(e,g,t):c.insertNodeAtClassStart(e,n,t)}n.createMissingMemberNodes(o,m,i,t,l,y,function(e){return v(i,o,e)}),y.writeFixes(c)}function s(n,t){var r=e.getEffectiveBaseTypeNode(n);if(!r)return e.createSymbolTable();var i=t.getTypeAtLocation(r),o=t.getPropertiesOfType(i);return e.createSymbolTable(o.filter(a))}n.registerCodeFix({errorCodes:t,getCodeActions:function(t){var a=t.sourceFile,s=t.span,c=i(a,s.start);return e.mapDefined(e.getEffectiveImplementsTypeNodes(c),function(i){var s=e.textChanges.ChangeTracker.with(t,function(e){return o(t,i,a,c,e,t.preferences)});return 0===s.length?void 0:n.createCodeFixAction(r,s,[e.Diagnostics.Implement_interface_0,i.getText(a)],r,e.Diagnostics.Implement_all_unimplemented_interfaces)})},fixIds:[r],getAllCodeActions:function(r){var a=new e.Map;return n.codeFixAll(r,t,function(n,t){var s=i(t.file,t.start);if(e.addToSeen(a,e.getNodeId(s)))for(var c=0,l=e.getEffectiveImplementsTypeNodes(s);c<l.length;c++){var u=l[c];o(r,u,t.file,s,n,r.preferences)}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){n.importFixName="import";var t,r="fixMissingImport",i=[e.Diagnostics.Cannot_find_name_0.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_1.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0.code,e.Diagnostics.Cannot_find_namespace_0.code,e.Diagnostics._0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead.code,e.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here.code,e.Diagnostics.No_value_exists_in_scope_for_the_shorthand_property_0_Either_declare_one_or_provide_an_initializer.code];function s(e,n,t,r){return c(e,n,!1,t,r)}function c(n,t,r,i,a){var o=t.getCompilerOptions(),s=[],c=[],u=new e.Map,p=new e.Map;return{addImportFromDiagnostic:f,addImportFromExportedSymbol:_,writeFixes:y};function f(e,n){var t=A(n,e.code,e.start,r);t&&t.fixes.length&&g(t)}function _(s,c){var l=e.Debug.checkDefined(s.parent),u=e.getNameForExportedSymbol(s,e.getEmitScriptTarget(o)),p=t.getTypeChecker(),f=p.getMergedSymbol(e.skipAlias(s,p)),_=m(n,f,l,u,a,t,i,r),y=!!c&&2===o.importsNotUsedAsValues,h=S(n,t),v=d(n,_,l,u,t,void 0,y,h,a,i);v&&g({fixes:[v],symbolName:u})}function g(n){var t=n.fixes,r=n.symbolName,i=e.first(t);switch(i.kind){case 0:s.push(i);break;case 1:c.push(i);break;case 2:var a=i.importClauseOrBindingPattern,o=i.importKind,l=i.canUseTypeOnlyImport,d=String(e.getNodeId(a)),f=u.get(d);f||u.set(d,f={importClauseOrBindingPattern:a,defaultImport:void 0,namedImports:[],canUseTypeOnlyImport:l}),0===o?e.pushIfUnique(f.namedImports,r):(e.Debug.assert(void 0===f.defaultImport||f.defaultImport===r,"(Add to Existing) Default import should be missing or match symbolName"),f.defaultImport=r);break;case 3:var m=i.moduleSpecifier,_=(o=i.importKind,i.useRequire),g=i.typeOnly;f=p.get(m);switch(f?f.typeOnly=f.typeOnly&&g:p.set(m,f={namedImports:[],namespaceLikeImport:void 0,typeOnly:g,useRequire:_}),o){case 1:e.Debug.assert(void 0===f.defaultImport||f.defaultImport===r,"(Add new) Default import should be missing or match symbolName"),f.defaultImport=r;break;case 0:e.pushIfUnique(f.namedImports||(f.namedImports=[]),r);break;case 3:case 2:e.Debug.assert(void 0===f.namespaceLikeImport||f.namespaceLikeImport.name===r,"Namespacelike import shoudl be missing or match symbolName"),f.namespaceLikeImport={importKind:o,name:r};break}break;default:e.Debug.assertNever(i,"fix wasn't never - got kind "+i.kind)}}function y(t){for(var r,a=e.getQuotePreference(n,i),o=0,d=s;o<d.length;o++){var f=d[o];K(t,n,f)}for(var m=0,_=c;m<_.length;m++){f=_[m];H(t,n,f,a)}u.forEach(function(e){var r=e.importClauseOrBindingPattern,i=e.defaultImport,a=e.namedImports,o=e.canUseTypeOnlyImport;j(t,n,r,i,a,o)}),p.forEach(function(n,t){var i=n.useRequire,o=l(n,["useRequire"]),s=i?J:z;r=e.combine(r,s(t,a,o))}),r&&e.insertImports(t,n,r,!0)}}function u(n,t,r,i,a,o,s,c,l){var u=o.getCompilerOptions(),_=e.pathIsBareSpecifier(e.stripQuotes(t.name))?[f(n,t,o,a)]:m(r,n,t,i,a,o,l,!0),g=S(r,o),y=2===u.importsNotUsedAsValues&&!e.isSourceFileJS(r)&&e.isValidTypeOnlyAliasUseSite(e.getTokenAtPosition(r,c)),h=e.Debug.checkDefined(d(r,_,t,i,o,c,y,g,a,l));return{moduleSpecifier:h.moduleSpecifier,codeAction:p(U({host:a,formatContext:s,preferences:l},r,i,h,e.getQuotePreference(r,l)))}}function d(n,t,r,i,a,o,s,c,l,u){return e.Debug.assert(t.some(function(e){return e.moduleSymbol===r||e.symbol.parent===r}),"Some exportInfo should match the specified moduleSymbol"),I(y(t,i,o,s,c,a,n,l,u),n,a,l,u)}function p(e){var n=e.description,t=e.changes,r=e.commands;return{description:n,changes:t,commands:r}}function f(n,t,r,i){var a,o,s=r.getCompilerOptions(),c=u(r.getTypeChecker(),!1);if(c)return c;var l=null===(o=null===(a=i.getPackageJsonAutoImportProvider)||void 0===a?void 0:a.call(i))||void 0===o?void 0:o.getTypeChecker();return e.Debug.checkDefined(l&&u(l,!0),"Could not find symbol in specified module for code actions");function u(r,i){var a=e.getDefaultLikeExportInfo(t,r,s);if(a&&e.skipAlias(a.symbol,r)===n)return{symbol:a.symbol,moduleSymbol:t,moduleFileName:void 0,exportKind:a.exportKind,targetFlags:e.skipAlias(n,r).flags,isFromPackageJson:i};var o=r.tryGetMemberInModuleExportsAndProperties(n.name,t);return o&&e.skipAlias(o,r)===n?{symbol:o,moduleSymbol:t,moduleFileName:void 0,exportKind:0,targetFlags:e.skipAlias(n,r).flags,isFromPackageJson:i}:void 0}}function m(n,t,r,i,a,o,s,c){var l=[],u=o.getCompilerOptions(),d=e.memoizeOne(function(n){return e.createModuleSpecifierResolutionHost(n?a.getPackageJsonAutoImportProvider():o,a)});return e.forEachExternalModuleToImportFrom(o,a,c,function(a,o,s,c){var d=s.getTypeChecker();if(!o||a===r||!e.startsWith(n.fileName,e.getDirectoryPath(o.fileName))){var f=e.getDefaultLikeExportInfo(a,d,u);f&&(f.name===i||Y(a,u.target)===i)&&e.skipAlias(f.symbol,d)===t&&p(s,o,c)&&l.push({symbol:f.symbol,moduleSymbol:a,moduleFileName:null===o||void 0===o?void 0:o.fileName,exportKind:f.exportKind,targetFlags:e.skipAlias(f.symbol,d).flags,isFromPackageJson:c});for(var m=0,_=d.getExportsAndPropertiesOfModule(a);m<_.length;m++){var g=_[m];g.name===i&&e.skipAlias(g,d)===t&&p(s,o,c)&&l.push({symbol:g,moduleSymbol:a,moduleFileName:null===o||void 0===o?void 0:o.fileName,exportKind:0,targetFlags:e.skipAlias(g,d).flags,isFromPackageJson:c})}}}),l;function p(t,r,i){var o;return!r||e.isImportableFile(t,n,r,s,void 0,d(i),null===(o=a.getModuleSpecifierCache)||void 0===o?void 0:o.call(a))}}function _(e,n,t,r,i,a){var s=x(t,n,void 0,!1,!1,e,r,i,a),c=s.fixes,l=s.computedWithoutCacheCount,u=I(c,n,t,r,i);return u&&o(o({},u),{computedWithoutCacheCount:l})}function g(n,t){return e.isValidTypeOnlyAliasUseSite(e.getTokenAtPosition(n,t))}function y(n,t,r,i,o,s,c,l,u){var d=s.getTypeChecker(),p=e.flatMap(n,function(e){return T(e,d,c,s.getCompilerOptions())}),f=void 0===r?void 0:h(p,t,r,d),m=E(p,void 0!==r&&g(c,r)),_=m?[m]:L(n,p,s,c,r,i,o,l,u);return a(a([],f?[f]:e.emptyArray,!0),_,!0)}function h(n,t,r,i){return e.firstDefined(n,function(n){var a=n.declaration,o=b(a),s=e.tryGetModuleSpecifierFromDeclaration(a);if(o&&s){var c=v(a,i);if(c&&c.exports.has(e.escapeLeadingUnderscores(t)))return{kind:0,namespacePrefix:o,position:r,moduleSpecifier:s}}})}function v(n,t){var r;switch(n.kind){case 252:return t.resolveExternalModuleName(n.initializer.arguments[0]);case 263:return t.getAliasedSymbol(n.symbol);case 264:var i=e.tryCast(null===(r=n.importClause)||void 0===r?void 0:r.namedBindings,e.isNamespaceImport);return i&&t.getAliasedSymbol(i.symbol);default:return e.Debug.assertNever(n)}}function b(n){var t,r,i;switch(n.kind){case 252:return null===(t=e.tryCast(n.name,e.isIdentifier))||void 0===t?void 0:t.text;case 263:return n.name.text;case 264:return null===(i=e.tryCast(null===(r=n.importClause)||void 0===r?void 0:r.namedBindings,e.isNamespaceImport))||void 0===i?void 0:i.name.text;default:return e.Debug.assertNever(n)}}function E(n,t){return e.firstDefined(n,function(n){var r=n.declaration,i=n.importKind;if(263!==r.kind){if(252===r.kind)return 0!==i&&1!==i||199!==r.name.kind?void 0:{kind:2,importClauseOrBindingPattern:r.name,importKind:i,moduleSpecifier:r.initializer.arguments[0].text,canUseTypeOnlyImport:!1};var a=r.importClause;if(a&&e.isStringLiteralLike(r.moduleSpecifier)){var o=a.name,s=a.namedBindings;if(!a.isTypeOnly||0===i&&s)return 1===i&&!o||0===i&&(!s||267===s.kind)?{kind:2,importClauseOrBindingPattern:a,importKind:i,moduleSpecifier:r.moduleSpecifier.text,canUseTypeOnlyImport:t}:void 0}}})}function T(n,t,r,i){var a=n.moduleSymbol,o=n.exportKind,s=n.targetFlags;if(!(111551&s)&&e.isSourceFileJS(r))return e.emptyArray;var c=w(r,o,i);return e.mapDefined(r.imports,function(n){var r=e.importFromModuleSpecifier(n);return e.isRequireVariableDeclaration(r.parent)?t.resolveExternalModuleName(n)===a?{declaration:r.parent,importKind:c}:void 0:(264===r.kind||263===r.kind)&&t.getSymbolAtLocation(n)===a?{declaration:r,importKind:c}:void 0})}function S(n,t){if(!e.isSourceFileJS(n))return!1;if(n.commonJsModuleIndicator&&!n.externalModuleIndicator)return!0;if(n.externalModuleIndicator&&!n.commonJsModuleIndicator)return!1;var r=t.getCompilerOptions();if(r.configFile)return e.getEmitModuleKind(r)<e.ModuleKind.ES2015;for(var i=0,a=t.getSourceFiles();i<a.length;i++){var o=a[i];if(o!==n&&e.isSourceFileJS(o)&&!t.isSourceFileFromExternalLibrary(o)){if(o.commonJsModuleIndicator&&!o.externalModuleIndicator)return!0;if(o.externalModuleIndicator&&!o.commonJsModuleIndicator)return!1}}return!0}function x(n,t,r,i,a,o,s,c,l){var u=e.isSourceFileJS(t),d=n.getCompilerOptions(),p=e.createModuleSpecifierResolutionHost(n,s),f=n.getTypeChecker(),m=l?function(n){return{moduleSpecifiers:e.moduleSpecifiers.tryGetModuleSpecifiersFromCache(n,t,p,c),computedWithoutCache:!1}}:function(n){return e.moduleSpecifiers.getModuleSpecifiersWithCacheInfo(n,f,d,t,p,c)},_=0,g=e.flatMap(o,function(e){var n=m(e.moduleSymbol),o=n.computedWithoutCache,s=n.moduleSpecifiers;return _+=o?1:0,null===s||void 0===s?void 0:s.map(function(n){return 111551&e.targetFlags||!u||void 0===r?{kind:3,moduleSpecifier:n,importKind:w(t,e.exportKind,d),useRequire:a,typeOnly:i,exportInfo:e}:{kind:1,moduleSpecifier:n,position:r,exportInfo:e}})});return{computedWithoutCacheCount:_,fixes:g}}function L(n,t,r,i,a,o,s,c,l){var u=e.firstDefined(t,function(e){return D(e,o,s)});return u?[u]:x(r,i,a,o,s,n,c,l).fixes}function D(n,t,r){var i=n.declaration,a=n.importKind,o=e.tryGetModuleSpecifierFromDeclaration(i);return o?{kind:3,moduleSpecifier:o,importKind:a,typeOnly:t,useRequire:r}:void 0}function A(n,t,r,i){var a=e.getTokenAtPosition(n.sourceFile,r),s=t===e.Diagnostics._0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead.code?R(n,a):e.isIdentifier(a)?M(n,a,i):void 0;return s&&o(o({},s),{fixes:C(s.fixes,n.sourceFile,n.program,n.host,n.preferences)})}function C(n,t,r,i,a){var o=e.createPackageJsonImportFilter(t,a,i).allowsImportingSpecifier;return e.sort(n,function(n,i){return e.compareValues(n.kind,i.kind)||N(n,i,t,r,o)})}function I(n,t,r,i,a){if(e.some(n)){if(0===n[0].kind||2===n[0].kind)return n[0];var o=e.createPackageJsonImportFilter(t,a,i).allowsImportingSpecifier;return n.reduce(function(e,n){return-1===N(n,e,t,r,o)?n:e})}}function N(n,t,r,i,a){return 0!==n.kind&&0!==t.kind?e.compareBooleans(a(n.moduleSpecifier),a(t.moduleSpecifier))||k(n.moduleSpecifier,t.moduleSpecifier,r,i)||e.compareNumberOfDirectorySeparators(n.moduleSpecifier,t.moduleSpecifier):0}function k(n,t,r,i){return e.startsWith(n,"node:")&&!e.startsWith(t,"node:")?e.shouldUseUriStyleNodeCoreModules(r,i)?-1:1:e.startsWith(t,"node:")&&!e.startsWith(n,"node:")?e.shouldUseUriStyleNodeCoreModules(r,i)?1:-1:0}function R(n,t){var r=n.sourceFile,i=n.program,a=n.host,o=n.preferences,s=i.getTypeChecker(),c=O(t,s);if(c){var l=s.getAliasedSymbol(c),u=c.name,d=[{symbol:c,moduleSymbol:l,moduleFileName:void 0,exportKind:3,targetFlags:l.flags,isFromPackageJson:!1}],p=S(r,i),f=y(d,u,e.isIdentifier(t)?t.getStart(r):void 0,!1,p,i,r,a,o);return{fixes:f,symbolName:u}}}function O(n,t){var r=e.isIdentifier(n)?t.getSymbolAtLocation(n):void 0;if(e.isUMDExportSymbol(r))return r;var i=n.parent;return e.isJsxOpeningLikeElement(i)&&i.tagName===n||e.isJsxOpeningFragment(i)?e.tryCast(t.resolveName(t.getJsxNamespace(i),e.isJsxOpeningLikeElement(i)?n:i,111551,!1),e.isUMDExportSymbol):void 0}function w(n,t,r,i){switch(t){case 0:return 0;case 1:return 1;case 2:return B(n,r,!!i);case 3:return P(n,r,!!i);default:return e.Debug.assertNever(t)}}function P(n,t,r){if(e.getAllowSyntheticDefaultImports(t))return 1;var i=e.getEmitModuleKind(t);switch(i){case e.ModuleKind.AMD:case e.ModuleKind.CommonJS:case e.ModuleKind.UMD:return e.isInJSFile(n)&&(e.isExternalModule(n)||r)?2:3;case e.ModuleKind.System:case e.ModuleKind.ES2015:case e.ModuleKind.ES2020:case e.ModuleKind.ESNext:case e.ModuleKind.None:return 2;default:return e.Debug.assertNever(i,"Unexpected moduleKind "+i)}}function M(n,t,r){var i=n.sourceFile,a=n.program,o=n.cancellationToken,s=n.host,c=n.preferences,l=a.getTypeChecker(),u=a.getCompilerOptions(),d=F(i,l,t,u);e.Debug.assert("default"!==d,"'default' isn't a legal identifier and couldn't occur here");var p=2===u.importsNotUsedAsValues&&e.isValidTypeOnlyAliasUseSite(t),f=S(i,a),m=G(d,e.getMeaningFromLocation(t),o,i,a,r,s,c),_=e.arrayFrom(e.flatMapIterator(m.entries(),function(e){e[0];var n=e[1];return y(n,d,t.getStart(i),p,f,a,i,s,c)}));return{fixes:_,symbolName:d}}function F(n,t,r,i){var a=r.parent;if((e.isJsxOpeningLikeElement(a)||e.isJsxClosingElement(a))&&a.tagName===r&&4!==i.jsx&&5!==i.jsx){var o=t.getJsxNamespace(n);if(e.isIntrinsicJsxName(r.text)||!t.resolveName(o,a,111551,!0))return o}return r.text}function G(n,t,r,i,a,o,s,c){var l,u=e.createMultiMap(),d=e.createPackageJsonImportFilter(i,c,s),p=null===(l=s.getModuleSpecifierCache)||void 0===l?void 0:l.call(s),f=e.memoizeOne(function(n){return e.createModuleSpecifierResolutionHost(n?s.getPackageJsonAutoImportProvider():a,s)});function m(n,t,r,a,o,s){var l=f(s);if(t&&e.isImportableFile(o,i,t,c,d,l,p)||!t&&d.allowsImportingAmbientModule(n,l)){var m=o.getTypeChecker();u.add(e.getUniqueSymbolId(r,m).toString(),{symbol:r,moduleSymbol:n,moduleFileName:null===t||void 0===t?void 0:t.fileName,exportKind:a,targetFlags:e.skipAlias(r,m).flags,isFromPackageJson:s})}}return e.forEachExternalModuleToImportFrom(a,s,o,function(i,a,o,s){var c=o.getTypeChecker();r.throwIfCancellationRequested();var l=o.getCompilerOptions(),u=e.getDefaultLikeExportInfo(i,c,l);u&&(u.name===n||Y(i,l.target)===n)&&X(u.symbolForMeaning,t)&&m(i,a,u.symbol,u.exportKind,o,s);var d=c.tryGetMemberInModuleExportsAndProperties(n,i);d&&X(d,t)&&m(i,a,d,0,o,s)}),u}function B(n,t,r){var i=e.getAllowSyntheticDefaultImports(t),a=e.isInJSFile(n);if(!a&&e.getEmitModuleKind(t)>=e.ModuleKind.ES2015)return i?1:2;if(a)return e.isExternalModule(n)||r?i?1:2:3;for(var o=0,s=n.statements;o<s.length;o++){var c=s[o];if(e.isImportEqualsDeclaration(c)&&!e.nodeIsMissing(c.moduleReference))return 3}return i?1:3}function U(t,i,a,o,s){var c,l=e.textChanges.ChangeTracker.with(t,function(e){c=V(e,i,a,o,s)});return n.createCodeFixAction(n.importFixName,l,c,r,e.Diagnostics.Add_all_missing_imports)}function V(n,t,r,i,a){switch(i.kind){case 0:return K(n,t,i),[e.Diagnostics.Change_0_to_1,r,i.namespacePrefix+"."+r];case 1:return H(n,t,i,a),[e.Diagnostics.Change_0_to_1,r,W(i.moduleSpecifier,a)+r];case 2:var o=i.importClauseOrBindingPattern,s=i.importKind,c=i.canUseTypeOnlyImport,l=i.moduleSpecifier;j(n,t,o,1===s?r:void 0,0===s?[r]:e.emptyArray,c);var u=e.stripQuotes(l);return[1===s?e.Diagnostics.Add_default_import_0_to_existing_import_declaration_from_1:e.Diagnostics.Add_0_to_existing_import_declaration_from_1,r,u];case 3:s=i.importKind,l=i.moduleSpecifier;var d=i.typeOnly,p=i.useRequire,f=p?J:z,m=1===s?{defaultImport:r,typeOnly:d}:0===s?{namedImports:[r],typeOnly:d}:{namespaceLikeImport:{importKind:s,name:r},typeOnly:d};return e.insertImports(n,t,f(l,a,m),!0),[1===s?e.Diagnostics.Import_default_0_from_module_1:e.Diagnostics.Import_0_from_module_1,r,l];default:return e.Debug.assertNever(i,"Unexpected fix kind "+i.kind)}}function j(n,t,r,i,a,o){if(199!==r.kind){var s=!o&&r.isTypeOnly;if(i&&(e.Debug.assert(!r.name,"Cannot add a default import to an import clause that already has one"),n.insertNodeAt(t,r.getStart(t),e.factory.createIdentifier(i),{suffix:", "})),a.length){var c=r.namedBindings&&e.cast(r.namedBindings,e.isNamedImports).elements,l=e.stableSort(a.map(function(n){return e.factory.createImportSpecifier(void 0,e.factory.createIdentifier(n))}),e.OrganizeImports.compareImportOrExportSpecifiers);if((null===c||void 0===c?void 0:c.length)&&e.OrganizeImports.importSpecifiersAreSorted(c))for(var u=0,d=l;u<d.length;u++){var p=d[u],f=e.OrganizeImports.getImportSpecifierInsertionIndex(c,p),m=r.namedBindings.elements[f-1];m?n.insertNodeInListAfter(t,m,p):n.insertNodeBefore(t,c[0],p,!e.positionsAreOnSameLine(c[0].getStart(),r.parent.getStart(),t))}else if(null===c||void 0===c?void 0:c.length)for(var _=0,g=l;_<g.length;_++){p=g[_];n.insertNodeInListAfter(t,e.last(c),p,c)}else if(l.length){var y=e.factory.createNamedImports(l);r.namedBindings?n.replaceNode(t,r.namedBindings,y):n.insertNodeAfter(t,e.Debug.checkDefined(r.name,"Import clause must have either named imports or a default import"),y)}}s&&n.delete(t,e.getTypeKeywordOfTypeOnlyImport(r,t))}else{i&&E(r,i,"default");for(var h=0,v=a;h<v.length;h++){var b=v[h];E(r,b,void 0)}}function E(r,i,a){var o=e.factory.createBindingElement(void 0,a,i);r.elements.length?n.insertNodeInListAfter(t,e.last(r.elements),o):n.replaceNode(t,r,e.factory.createObjectBindingPattern([o]))}}function K(e,n,t){var r=t.namespacePrefix,i=t.position;e.insertText(n,i,r+".")}function H(e,n,t,r){var i=t.moduleSpecifier,a=t.position;e.insertText(n,a,W(i,r))}function W(n,t){var r=e.getQuoteFromPreference(t);return"import("+r+n+r+")."}function z(n,t,r){var i,a,o,s=e.makeStringLiteral(n,t);(void 0!==r.defaultImport||(null===(i=r.namedImports)||void 0===i?void 0:i.length))&&(o=e.combine(o,e.makeImport(void 0===r.defaultImport?void 0:e.factory.createIdentifier(r.defaultImport),null===(a=r.namedImports)||void 0===a?void 0:a.map(function(n){return e.factory.createImportSpecifier(void 0,e.factory.createIdentifier(n))}),n,t,r.typeOnly)));var c=r.namespaceLikeImport,l=r.typeOnly;if(c){var u=3===c.importKind?e.factory.createImportEqualsDeclaration(void 0,void 0,l,e.factory.createIdentifier(c.name),e.factory.createExternalModuleReference(s)):e.factory.createImportDeclaration(void 0,void 0,e.factory.createImportClause(l,void 0,e.factory.createNamespaceImport(e.factory.createIdentifier(c.name))),s);o=e.combine(o,u)}return e.Debug.checkDefined(o)}function J(n,t,r){var i,a,o,s=e.makeStringLiteral(n,t);if(r.defaultImport||(null===(i=r.namedImports)||void 0===i?void 0:i.length)){var c=(null===(a=r.namedImports)||void 0===a?void 0:a.map(function(n){return e.factory.createBindingElement(void 0,void 0,n)}))||[];r.defaultImport&&c.unshift(e.factory.createBindingElement(void 0,"default",r.defaultImport));var l=q(e.factory.createObjectBindingPattern(c),s);o=e.combine(o,l)}if(r.namespaceLikeImport){l=q(r.namespaceLikeImport.name,s);o=e.combine(o,l)}return e.Debug.checkDefined(o)}function q(n,t){return e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration("string"===typeof n?e.factory.createIdentifier(n):n,void 0,void 0,e.factory.createCallExpression(e.factory.createIdentifier("require"),void 0,[t]))],2))}function X(n,t){var r=n.declarations;return e.some(r,function(n){return!!(e.getMeaningFromDeclaration(n)&t)})}function Y(n,t){return Q(e.removeFileExtension(e.stripQuotes(n.name)),t)}function Q(n,t){var r=e.getBaseFileName(e.removeSuffix(n,"/index")),i="",a=!0,o=r.charCodeAt(0);e.isIdentifierStart(o,t)?i+=String.fromCharCode(o):a=!1;for(var s=1;s<r.length;s++){var c=r.charCodeAt(s),l=e.isIdentifierPart(c,t);if(l){var u=String.fromCharCode(c);a||(u=u.toUpperCase()),i+=u}a=l}return e.isStringANonContextualKeyword(i)?"_"+i:i||"_"}n.registerCodeFix({errorCodes:i,getCodeActions:function(n){var t=n.errorCode,r=n.preferences,i=n.sourceFile,a=n.span,o=A(n,t,a.start,!0);if(o){var s=o.fixes,c=o.symbolName,l=e.getQuotePreference(i,r);return s.map(function(e){return U(n,i,c,e,l)})}},fixIds:[r],getAllCodeActions:function(t){var r=t.sourceFile,a=t.program,o=t.preferences,s=t.host,l=c(r,a,!0,o,s);return n.eachDiagnostic(t,i,function(e){return l.addImportFromDiagnostic(e,t)}),n.createCombinedCodeActions(e.textChanges.ChangeTracker.with(t,l.writeFixes))}}),n.createImportAdder=s,function(e){e[e["UseNamespace"]=0]="UseNamespace",e[e["ImportType"]=1]="ImportType",e[e["AddToExisting"]=2]="AddToExisting",e[e["AddNew"]=3]="AddNew"}(t||(t={})),n.getImportCompletionAction=u,n.getModuleSpecifierForBestExportInfo=_,n.getImportKind=w,n.moduleSymbolToValidIdentifier=Y,n.moduleSpecifierToValidIdentifier=Q})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t,r="fixOverrideModifier",i="fixAddOverrideModifier",a="fixRemoveOverrideModifier",o=[e.Diagnostics.This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0.code,e.Diagnostics.This_member_cannot_have_an_override_modifier_because_its_containing_class_0_does_not_extend_another_class.code,e.Diagnostics.This_member_must_have_an_override_modifier_because_it_overrides_an_abstract_method_that_is_declared_in_the_base_class_0.code,e.Diagnostics.This_member_must_have_an_override_modifier_because_it_overrides_a_member_in_the_base_class_0.code,e.Diagnostics.This_parameter_property_must_have_an_override_modifier_because_it_overrides_a_member_in_base_class_0.code],s=(t={},t[e.Diagnostics.This_member_must_have_an_override_modifier_because_it_overrides_a_member_in_the_base_class_0.code]=[e.Diagnostics.Add_override_modifier,i,e.Diagnostics.Add_all_missing_override_modifiers],t[e.Diagnostics.This_member_cannot_have_an_override_modifier_because_its_containing_class_0_does_not_extend_another_class.code]=[e.Diagnostics.Remove_override_modifier,a,e.Diagnostics.Remove_all_unnecessary_override_modifiers],t[e.Diagnostics.This_parameter_property_must_have_an_override_modifier_because_it_overrides_a_member_in_base_class_0.code]=[e.Diagnostics.Add_override_modifier,i,e.Diagnostics.Add_all_missing_override_modifiers],t[e.Diagnostics.This_member_must_have_an_override_modifier_because_it_overrides_an_abstract_method_that_is_declared_in_the_base_class_0.code]=[e.Diagnostics.Add_override_modifier,i,e.Diagnostics.Remove_all_unnecessary_override_modifiers],t[e.Diagnostics.This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0.code]=[e.Diagnostics.Remove_override_modifier,a,e.Diagnostics.Remove_all_unnecessary_override_modifiers],t);function c(n,t,r,i){switch(r){case e.Diagnostics.This_member_must_have_an_override_modifier_because_it_overrides_a_member_in_the_base_class_0.code:case e.Diagnostics.This_member_must_have_an_override_modifier_because_it_overrides_an_abstract_method_that_is_declared_in_the_base_class_0.code:case e.Diagnostics.This_parameter_property_must_have_an_override_modifier_because_it_overrides_a_member_in_base_class_0.code:return l(n,t.sourceFile,i);case e.Diagnostics.This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0.code:case e.Diagnostics.This_member_cannot_have_an_override_modifier_because_its_containing_class_0_does_not_extend_another_class.code:return u(n,t.sourceFile,i);default:e.Debug.fail("Unexpected error code: "+r)}}function l(n,t,r){var i=p(t,r),a=i.modifiers||e.emptyArray,o=e.find(a,e.isStaticModifier),s=e.find(a,e.isAbstractModifier),c=e.find(a,function(n){return e.isAccessibilityModifier(n.kind)}),l=s?s.end:o?o.end:c?c.end:i.decorators?e.skipTrivia(t.text,i.decorators.end):i.getStart(t),u=c||o||s?{prefix:" "}:{suffix:" "};n.insertModifierAt(t,l,157,u)}function u(n,t,r){var i=p(t,r),a=i.modifiers&&e.find(i.modifiers,function(e){return 157===e.kind});e.Debug.assertIsDefined(a),n.deleteModifier(t,a)}function d(n){switch(n.kind){case 169:case 165:case 167:case 170:case 171:return!0;case 162:return e.isParameterPropertyDeclaration(n,n.parent);default:return!1}}function p(n,t){var r=e.getTokenAtPosition(n,t),i=e.findAncestor(r,function(n){return e.isClassLike(n)?"quit":d(n)});return e.Debug.assert(i&&d(i)),i}n.registerCodeFix({errorCodes:o,getCodeActions:function(t){var i=t.errorCode,a=t.span,o=t.sourceFile,l=s[i];if(!l)return e.emptyArray;var u=l[0],d=l[1],p=l[2];if(e.isSourceFileJS(o))return e.emptyArray;var f=e.textChanges.ChangeTracker.with(t,function(e){return c(e,t,i,a.start)});return[n.createCodeFixActionMaybeFixAll(r,f,u,d,p)]},fixIds:[r,i,a],getAllCodeActions:function(t){return n.codeFixAll(t,o,function(n,r){var i=r.code,a=r.start,o=r.file,l=s[i];l&&l[1]===t.fixId&&!e.isSourceFileJS(o)&&c(n,t,i,a)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixNoPropertyAccessFromIndexSignature",r=[e.Diagnostics.Property_0_comes_from_an_index_signature_so_it_must_be_accessed_with_0.code];function i(n,t,r,i){var a=e.getQuotePreference(t,i),o=e.factory.createStringLiteral(r.name.text,0===a);n.replaceNode(t,r,e.isPropertyAccessChain(r)?e.factory.createElementAccessChain(r.expression,r.questionDotToken,o):e.factory.createElementAccessExpression(r.expression,o))}function a(n,t){return e.cast(e.getTokenAtPosition(n,t).parent,e.isPropertyAccessExpression)}n.registerCodeFix({errorCodes:r,fixIds:[t],getCodeActions:function(r){var o=r.sourceFile,s=r.span,c=r.preferences,l=a(o,s.start),u=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,l,c)});return[n.createCodeFixAction(t,u,[e.Diagnostics.Use_element_access_for_0,l.name.text],t,e.Diagnostics.Use_element_access_for_all_undeclared_properties)]},getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){return i(n,t.file,a(t.file,t.start),e.preferences)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixImplicitThis",r=[e.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation.code];function i(n,t,r,i){var a=e.getTokenAtPosition(t,r);e.Debug.assert(108===a.kind);var o=e.getThisContainer(a,!1);if((e.isFunctionDeclaration(o)||e.isFunctionExpression(o))&&!e.isSourceFile(e.getThisContainer(o,!1))){var s=e.Debug.assertDefined(e.findChildOfKind(o,98,t)),c=o.name,l=e.Debug.assertDefined(o.body);if(e.isFunctionExpression(o)){if(c&&e.FindAllReferences.Core.isSymbolReferencedInFile(c,i,t,l))return;return n.delete(t,s),c&&n.delete(t,c),n.insertText(t,l.pos," =>"),[e.Diagnostics.Convert_function_expression_0_to_arrow_function,c?c.text:e.ANONYMOUS]}return n.replaceNode(t,s,e.factory.createToken(85)),n.insertText(t,c.end," = "),n.insertText(t,l.pos," =>"),[e.Diagnostics.Convert_function_declaration_0_to_arrow_function,c.text]}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a,o=r.sourceFile,s=r.program,c=r.span,l=e.textChanges.ChangeTracker.with(r,function(e){a=i(e,o,c.start,s.getTypeChecker())});return a?[n.createCodeFixAction(t,l,a,t,e.Diagnostics.Fix_all_implicit_this_errors)]:e.emptyArray},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){i(n,t.file,t.start,e.program.getTypeChecker())})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixIncorrectNamedTupleSyntax",r=[e.Diagnostics.A_labeled_tuple_element_is_declared_as_optional_with_a_question_mark_after_the_name_and_before_the_colon_rather_than_after_the_type.code,e.Diagnostics.A_labeled_tuple_element_is_declared_as_rest_with_a_before_the_name_rather_than_before_the_type.code];function i(n,t){var r=e.getTokenAtPosition(n,t);return e.findAncestor(r,function(e){return 195===e.kind})}function a(n,t,r){if(r){var i=r.type,a=!1,o=!1;while(183===i.kind||184===i.kind||189===i.kind)183===i.kind?a=!0:184===i.kind&&(o=!0),i=i.type;var s=e.factory.updateNamedTupleMember(r,r.dotDotDotToken||(o?e.factory.createToken(25):void 0),r.name,r.questionToken||(a?e.factory.createToken(57):void 0),i);s!==r&&n.replaceNode(t,r,s)}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,c=i(o,s.start),l=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,c)});return[n.createCodeFixAction(t,l,e.Diagnostics.Move_labeled_tuple_element_modifiers_to_labels,t,e.Diagnostics.Move_labeled_tuple_element_modifiers_to_labels)]},fixIds:[t]})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixSpelling",r=[e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2.code,e.Diagnostics.Property_0_may_not_exist_on_type_1_Did_you_mean_2.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_1.code,e.Diagnostics.Could_not_find_name_0_Did_you_mean_1.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0.code,e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0.code,e.Diagnostics._0_has_no_exported_member_named_1_Did_you_mean_2.code,e.Diagnostics.This_member_cannot_have_an_override_modifier_because_it_is_not_declared_in_the_base_class_0_Did_you_mean_1.code,e.Diagnostics.No_overload_matches_this_call.code,e.Diagnostics.Type_0_is_not_assignable_to_type_1.code];function i(n,t,r,i){var a=e.getTokenAtPosition(n,t),c=a.parent;if(i!==e.Diagnostics.No_overload_matches_this_call.code&&i!==e.Diagnostics.Type_0_is_not_assignable_to_type_1.code||e.isJsxAttribute(c)){var l,u=r.program.getTypeChecker();if(e.isPropertyAccessExpression(c)&&c.name===a){e.Debug.assert(e.isMemberName(a),"Expected an identifier for spelling (property access)");var d=u.getTypeAtLocation(c.expression);32&c.flags&&(d=u.getNonNullableType(d)),l=u.getSuggestedSymbolForNonexistentProperty(a,d)}else if(e.isQualifiedName(c)&&c.right===a){var p=u.getSymbolAtLocation(c.left);p&&1536&p.flags&&(l=u.getSuggestedSymbolForNonexistentModule(c.right,p))}else if(e.isImportSpecifier(c)&&c.name===a){e.Debug.assertNode(a,e.isIdentifier,"Expected an identifier for spelling (import)");var f=e.findAncestor(a,e.isImportDeclaration),m=s(n,r,f);m&&m.symbol&&(l=u.getSuggestedSymbolForNonexistentModule(a,m.symbol))}else if(e.isJsxAttribute(c)&&c.name===a){e.Debug.assertNode(a,e.isIdentifier,"Expected an identifier for JSX attribute");var _=e.findAncestor(a,e.isJsxOpeningLikeElement),g=u.getContextualTypeForArgumentAtIndex(_,0);l=u.getSuggestedSymbolForNonexistentJSXAttribute(a,g)}else if(e.hasSyntacticModifier(c,16384)&&e.isClassElement(c)&&c.name===a){var y=e.findAncestor(a,e.isClassLike),h=y?e.getEffectiveBaseTypeNode(y):void 0,v=h?u.getTypeAtLocation(h):void 0;v&&(l=u.getSuggestedSymbolForNonexistentClassMember(e.getTextOfNode(a),v))}else{var b=e.getMeaningFromLocation(a),E=e.getTextOfNode(a);e.Debug.assert(void 0!==E,"name should be defined"),l=u.getSuggestedSymbolForNonexistentSymbol(a,E,o(b))}return void 0===l?void 0:{node:a,suggestedSymbol:l}}}function a(n,t,r,i,a){var o=e.symbolName(i);if(!e.isIdentifierText(o,a)&&e.isPropertyAccessExpression(r.parent)){var s=i.valueDeclaration;s&&e.isNamedDeclaration(s)&&e.isPrivateIdentifier(s.name)?n.replaceNode(t,r,e.factory.createIdentifier(o)):n.replaceNode(t,r.parent,e.factory.createElementAccessExpression(r.parent.expression,e.factory.createStringLiteral(o)))}else n.replaceNode(t,r,e.factory.createIdentifier(o))}function o(e){var n=0;return 4&e&&(n|=1920),2&e&&(n|=788968),1&e&&(n|=111551),n}function s(n,t,r){if(r&&e.isStringLiteralLike(r.moduleSpecifier)){var i=e.getResolvedModule(n,r.moduleSpecifier.text);if(i)return t.program.getSourceFile(i.resolvedFileName)}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.errorCode,c=i(o,r.span.start,r,s);if(c){var l=c.node,u=c.suggestedSymbol,d=r.host.getCompilationSettings().target,p=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,l,u,d)});return[n.createCodeFixAction("spelling",p,[e.Diagnostics.Change_spelling_to_0,e.symbolName(u)],t,e.Diagnostics.Fix_all_detected_spelling_errors)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){var r=i(t.file,t.start,e,t.code),o=e.host.getCompilationSettings().target;r&&a(n,e.sourceFile,r.node,r.suggestedSymbol,o)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t,r="returnValueCorrect",i="fixAddReturnStatement",a="fixRemoveBracesFromArrowFunctionBody",o="fixWrapTheBlockWithParen",s=[e.Diagnostics.A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value.code,e.Diagnostics.Type_0_is_not_assignable_to_type_1.code,e.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1.code];function c(n,t,r){var i=n.createSymbol(4,t.escapedText);i.type=n.getTypeAtLocation(r);var a=e.createSymbolTable([i]);return n.createAnonymousType(void 0,a,[],[],[])}function l(n,r,i,a){if(r.body&&e.isBlock(r.body)&&1===e.length(r.body.statements)){var o=e.first(r.body.statements);if(e.isExpressionStatement(o)&&u(n,r,n.getTypeAtLocation(o.expression),i,a))return{declaration:r,kind:t.MissingReturnStatement,expression:o.expression,statement:o,commentSource:o.expression};if(e.isLabeledStatement(o)&&e.isExpressionStatement(o.statement)){var s=e.factory.createObjectLiteralExpression([e.factory.createPropertyAssignment(o.label,o.statement.expression)]),l=c(n,o.label,o.statement.expression);if(u(n,r,l,i,a))return e.isArrowFunction(r)?{declaration:r,kind:t.MissingParentheses,expression:s,statement:o,commentSource:o.statement.expression}:{declaration:r,kind:t.MissingReturnStatement,expression:s,statement:o,commentSource:o.statement.expression}}else if(e.isBlock(o)&&1===e.length(o.statements)){var d=e.first(o.statements);if(e.isLabeledStatement(d)&&e.isExpressionStatement(d.statement)){s=e.factory.createObjectLiteralExpression([e.factory.createPropertyAssignment(d.label,d.statement.expression)]),l=c(n,d.label,d.statement.expression);if(u(n,r,l,i,a))return{declaration:r,kind:t.MissingReturnStatement,expression:s,statement:o,commentSource:d}}}}}function u(n,t,r,i,a){if(a){var o=n.getSignatureFromDeclaration(t);if(o){e.hasSyntacticModifier(t,256)&&(r=n.createPromiseType(r));var s=n.createSignature(t,o.typeParameters,o.thisParameter,o.parameters,r,void 0,o.minArgumentCount,o.flags);r=n.createAnonymousType(void 0,e.createSymbolTable(),[s],[],[])}else r=n.getAnyType()}return n.isTypeAssignableTo(r,i)}function d(n,t,r,i){var a=e.getTokenAtPosition(t,r);if(a.parent){var o=e.findAncestor(a.parent,e.isFunctionLikeDeclaration);switch(i){case e.Diagnostics.A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value.code:if(!o||!o.body||!o.type||!e.rangeContainsRange(o.type,a))return;return l(n,o,n.getTypeFromTypeNode(o.type),!1);case e.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1.code:if(!o||!e.isCallExpression(o.parent)||!o.body)return;var s=o.parent.arguments.indexOf(o),c=n.getContextualTypeForArgumentAtIndex(o.parent,s);if(!c)return;return l(n,o,c,!0);case e.Diagnostics.Type_0_is_not_assignable_to_type_1.code:if(!e.isDeclarationName(a)||!e.isVariableLike(a.parent)&&!e.isJsxAttribute(a.parent))return;var u=p(a.parent);if(!u||!e.isFunctionLikeDeclaration(u)||!u.body)return;return l(n,u,n.getTypeAtLocation(a.parent),!0)}}}function p(n){switch(n.kind){case 252:case 162:case 201:case 165:case 291:return n.initializer;case 283:return n.initializer&&(e.isJsxExpression(n.initializer)?n.initializer.expression:void 0);case 292:case 164:case 294:case 342:case 335:return}}function f(n,t,r,i){e.suppressLeadingAndTrailingTrivia(r);var a=e.probablyUsesSemicolons(t);n.replaceNode(t,i,e.factory.createReturnStatement(r),{leadingTriviaOption:e.textChanges.LeadingTriviaOption.Exclude,trailingTriviaOption:e.textChanges.TrailingTriviaOption.Exclude,suffix:a?";":void 0})}function m(n,t,r,i,a,o){var s=o||e.needsParentheses(i)?e.factory.createParenthesizedExpression(i):i;e.suppressLeadingAndTrailingTrivia(a),e.copyComments(a,s),n.replaceNode(t,r.body,s)}function _(n,t,r,i){n.replaceNode(t,r.body,e.factory.createParenthesizedExpression(i))}function g(t,a,o){var s=e.textChanges.ChangeTracker.with(t,function(e){return f(e,t.sourceFile,a,o)});return n.createCodeFixAction(r,s,e.Diagnostics.Add_a_return_statement,i,e.Diagnostics.Add_all_missing_return_statement)}function y(t,i,o,s){var c=e.textChanges.ChangeTracker.with(t,function(e){return m(e,t.sourceFile,i,o,s,!1)});return n.createCodeFixAction(r,c,e.Diagnostics.Remove_braces_from_arrow_function_body,a,e.Diagnostics.Remove_braces_from_all_arrow_function_bodies_with_relevant_issues)}function h(t,i,a){var s=e.textChanges.ChangeTracker.with(t,function(e){return _(e,t.sourceFile,i,a)});return n.createCodeFixAction(r,s,e.Diagnostics.Wrap_the_following_body_with_parentheses_which_should_be_an_object_literal,o,e.Diagnostics.Wrap_all_object_literal_with_parentheses)}(function(e){e[e["MissingReturnStatement"]=0]="MissingReturnStatement",e[e["MissingParentheses"]=1]="MissingParentheses"})(t||(t={})),n.registerCodeFix({errorCodes:s,fixIds:[i,a,o],getCodeActions:function(n){var r=n.program,i=n.sourceFile,a=n.span.start,o=n.errorCode,s=d(r.getTypeChecker(),i,a,o);if(s)return s.kind===t.MissingReturnStatement?e.append([g(n,s.expression,s.statement)],e.isArrowFunction(s.declaration)?y(n,s.declaration,s.expression,s.commentSource):void 0):[h(n,s.declaration,s.expression)]},getAllCodeActions:function(t){return n.codeFixAll(t,s,function(n,r){var s=d(t.program.getTypeChecker(),r.file,r.start,r.code);if(s)switch(t.fixId){case i:f(n,r.file,s.expression,s.statement);break;case a:if(!e.isArrowFunction(s.declaration))return;m(n,r.file,s.declaration,s.expression,s.commentSource,!1);break;case o:if(!e.isArrowFunction(s.declaration))return;_(n,r.file,s.declaration,s.expression);break;default:e.Debug.fail(JSON.stringify(t.fixId))}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t,r="fixMissingMember",i="fixMissingProperties",o="fixMissingAttributes",s="fixMissingFunctionDeclaration",c=[e.Diagnostics.Property_0_does_not_exist_on_type_1.code,e.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2.code,e.Diagnostics.Property_0_is_missing_in_type_1_but_required_in_type_2.code,e.Diagnostics.Type_0_is_missing_the_following_properties_from_type_1_Colon_2.code,e.Diagnostics.Type_0_is_missing_the_following_properties_from_type_1_Colon_2_and_3_more.code,e.Diagnostics.Cannot_find_name_0.code];function l(n,t,r,i){var a=e.getTokenAtPosition(n,t);if(e.isIdentifier(a)||e.isPrivateIdentifier(a)){var o=a.parent;if(e.isIdentifier(a)&&e.hasInitializer(o)&&o.initializer&&e.isObjectLiteralExpression(o.initializer)){var s=e.arrayFrom(r.getUnmatchedProperties(r.getTypeAtLocation(o.initializer),r.getTypeAtLocation(a),!1,!1));if(e.length(s))return{kind:3,token:a,properties:s,parentDeclaration:o.initializer}}if(e.isIdentifier(a)&&e.isJsxOpeningLikeElement(a.parent)){var c=I(r,a.parent);if(e.length(c))return{kind:4,token:a,attributes:c,parentDeclaration:a.parent}}if(e.isIdentifier(a)&&e.isCallExpression(o))return{kind:2,token:a,call:o,sourceFile:n,modifierFlags:0,parentDeclaration:n};if(e.isPropertyAccessExpression(o)){var l=e.skipConstraint(r.getTypeAtLocation(o.expression)),d=l.symbol;if(d&&d.declarations){if(e.isIdentifier(a)&&e.isCallExpression(o.parent)){var p=e.find(d.declarations,e.isModuleDeclaration),f=null===p||void 0===p?void 0:p.getSourceFile();if(p&&f&&!u(i,f))return{kind:2,token:a,call:o.parent,sourceFile:n,modifierFlags:1,parentDeclaration:p};var m=e.find(d.declarations,e.isSourceFile);if(n.commonJsModuleIndicator)return;if(m&&!u(i,m))return{kind:2,token:a,call:o.parent,sourceFile:m,modifierFlags:1,parentDeclaration:m}}var _=e.find(d.declarations,e.isClassLike);if(_||!e.isPrivateIdentifier(a)){var g=_||e.find(d.declarations,e.isInterfaceDeclaration);if(g&&!u(i,g.getSourceFile())){var y=(l.target||l)!==r.getDeclaredTypeOfSymbol(d);if(y&&(e.isPrivateIdentifier(a)||e.isInterfaceDeclaration(g)))return;var h=g.getSourceFile(),v=(y?32:0)|(e.startsWithUnderscore(a.text)?8:0),b=e.isSourceFileJS(h),E=e.tryCast(o.parent,e.isCallExpression);return{kind:1,token:a,call:E,modifierFlags:v,parentDeclaration:g,declSourceFile:h,isJSFile:b}}var T=e.find(d.declarations,e.isEnumDeclaration);return!T||e.isPrivateIdentifier(a)||u(i,T.getSourceFile())?void 0:{kind:0,token:a,parentDeclaration:T}}}}}}function u(e,n){return e.isSourceFileFromExternalLibrary(n)||e.isSourceFileDefaultLibrary(n)}function d(n,t){return t.isJSFile?e.singleElementArray(p(n,t)):_(n,t)}function p(t,i){var a=i.parentDeclaration,o=i.declSourceFile,s=i.modifierFlags,c=i.token;if(!e.isInterfaceDeclaration(a)){var l=e.textChanges.ChangeTracker.with(t,function(e){return f(e,o,a,c,!!(32&s))});if(0!==l.length){var u=32&s?e.Diagnostics.Initialize_static_property_0:e.isPrivateIdentifier(c)?e.Diagnostics.Declare_a_private_field_named_0:e.Diagnostics.Initialize_property_0_in_the_constructor;return n.createCodeFixAction(r,l,[u,c.text],r,e.Diagnostics.Add_all_missing_members)}}}function f(n,t,r,i,a){var o=i.text;if(a){if(224===r.kind)return;var s=r.name.getText(),c=m(e.factory.createIdentifier(s),o);n.insertNodeAfter(t,r,c)}else if(e.isPrivateIdentifier(i)){var l=e.factory.createPropertyDeclaration(void 0,void 0,o,void 0,void 0,void 0),u=h(r);u?n.insertNodeAfter(t,u,l):n.insertNodeAtClassStart(t,r,l)}else{var d=e.getFirstConstructorWithBody(r);if(!d)return;var p=m(e.factory.createThis(),o);n.insertNodeAtConstructorEnd(t,d,p)}}function m(n,t){return e.factory.createExpressionStatement(e.factory.createAssignment(e.factory.createPropertyAccessExpression(n,t),A()))}function _(t,i){var a=i.parentDeclaration,o=i.declSourceFile,s=i.modifierFlags,c=i.token,l=c.text,u=32&s,d=g(t.program.getTypeChecker(),a,c),p=function(n){return e.textChanges.ChangeTracker.with(t,function(e){return y(e,o,a,l,d,n)})},f=[n.createCodeFixAction(r,p(32&s),[u?e.Diagnostics.Declare_static_property_0:e.Diagnostics.Declare_property_0,l],r,e.Diagnostics.Add_all_missing_members)];return u||e.isPrivateIdentifier(c)?f:(8&s&&f.unshift(n.createCodeFixActionWithoutFixAll(r,p(8),[e.Diagnostics.Declare_private_property_0,l])),f.push(v(t,o,a,c.text,d)),f)}function g(n,t,r){var i;if(219===r.parent.parent.kind){var a=r.parent.parent,o=r.parent===a.left?a.right:a.left,s=n.getWidenedType(n.getBaseTypeOfLiteralType(n.getTypeAtLocation(o)));i=n.typeToTypeNode(s,t,1)}else{var c=n.getContextualType(r.parent);i=c?n.typeToTypeNode(c,void 0,1):void 0}return i||e.factory.createKeywordTypeNode(129)}function y(n,t,r,i,a,o){var s=e.factory.createPropertyDeclaration(void 0,o?e.factory.createNodeArray(e.factory.createModifiersFromModifierFlags(o)):void 0,i,void 0,a,void 0),c=h(r);c?n.insertNodeAfter(t,c,s):n.insertNodeAtClassStart(t,r,s)}function h(n){for(var t,r=0,i=n.members;r<i.length;r++){var a=i[r];if(!e.isPropertyDeclaration(a))break;t=a}return t}function v(t,i,a,o,s){var c=e.factory.createKeywordTypeNode(148),l=e.factory.createParameterDeclaration(void 0,void 0,void 0,"x",void 0,c,void 0),u=e.factory.createIndexSignature(void 0,void 0,[l],s),d=e.textChanges.ChangeTracker.with(t,function(e){return e.insertNodeAtClassStart(i,a,u)});return n.createCodeFixActionWithoutFixAll(r,d,[e.Diagnostics.Add_index_signature_for_property_0,o])}function b(t,i){var a=i.parentDeclaration,o=i.declSourceFile,s=i.modifierFlags,c=i.token,l=i.call;if(void 0!==l&&!e.isPrivateIdentifier(c)){var u=c.text,d=function(n){return e.textChanges.ChangeTracker.with(t,function(e){return E(t,e,l,c,n,a,o)})},p=[n.createCodeFixAction(r,d(32&s),[32&s?e.Diagnostics.Declare_static_method_0:e.Diagnostics.Declare_method_0,u],r,e.Diagnostics.Add_all_missing_members)];return 8&s&&p.unshift(n.createCodeFixActionWithoutFixAll(r,d(8),[e.Diagnostics.Declare_private_method_0,u])),p}}function E(t,r,i,a,o,s,c){var l=n.createImportAdder(c,t.program,t.preferences,t.host),u=n.createSignatureDeclarationFromCallExpression(167,t,l,i,a,o,s),d=e.findAncestor(i,function(n){return e.isMethodDeclaration(n)||e.isConstructorDeclaration(n)});d&&d.parent===s?r.insertNodeAfter(c,d,u):r.insertNodeAtClassStart(c,s,u),l.writeFixes(r)}function T(n,t,r){var i=r.token,a=r.parentDeclaration,o=e.some(a.members,function(e){var n=t.getTypeAtLocation(e);return!!(n&&402653316&n.flags)}),s=e.factory.createEnumMember(i,o?e.factory.createStringLiteral(i.text):void 0);n.replaceNode(a.getSourceFile(),a,e.factory.updateEnumDeclaration(a,a.decorators,a.modifiers,a.name,e.concatenate(a.members,e.singleElementArray(s))),{leadingTriviaOption:e.textChanges.LeadingTriviaOption.IncludeAll,trailingTriviaOption:e.textChanges.TrailingTriviaOption.Exclude})}function S(t,r,i){var a=n.createImportAdder(r.sourceFile,r.program,r.preferences,r.host),o=n.createSignatureDeclarationFromCallExpression(254,r,a,i.call,e.idText(i.token),i.modifierFlags,i.parentDeclaration);t.insertNodeAtEndOfScope(i.sourceFile,i.parentDeclaration,o)}function x(t,r,i){var o=n.createImportAdder(r.sourceFile,r.program,r.preferences,r.host),s=e.getQuotePreference(r.sourceFile,r.preferences),c=r.program.getTypeChecker(),l=i.parentDeclaration.attributes,u=e.some(l.properties,e.isJsxSpreadAttribute),d=e.map(i.attributes,function(n){var t=n.valueDeclaration?D(r,c,o,s,c.getTypeAtLocation(n.valueDeclaration)):A();return e.factory.createJsxAttribute(e.factory.createIdentifier(n.name),e.factory.createJsxExpression(void 0,t))}),p=e.factory.createJsxAttributes(u?a(a([],d,!0),l.properties,!0):a(a([],l.properties,!0),d,!0)),f={prefix:l.pos===l.end?" ":void 0};t.replaceNode(r.sourceFile,l,p,f)}function L(t,r,i){var o=n.createImportAdder(r.sourceFile,r.program,r.preferences,r.host),s=e.getQuotePreference(r.sourceFile,r.preferences),c=r.program.getTypeChecker(),l=e.map(i.properties,function(n){var t=n.valueDeclaration?D(r,c,o,s,c.getTypeAtLocation(n.valueDeclaration)):A();return e.factory.createPropertyAssignment(n.name,t)});t.replaceNode(r.sourceFile,i.parentDeclaration,e.factory.createObjectLiteralExpression(a(a([],i.parentDeclaration.properties,!0),l,!0),!0))}function D(t,r,i,a,o){if(3&o.flags)return A();if(134217732&o.flags)return e.factory.createStringLiteral("",0===a);if(8&o.flags)return e.factory.createNumericLiteral(0);if(64&o.flags)return e.factory.createBigIntLiteral("0n");if(16&o.flags)return e.factory.createFalse();if(1056&o.flags){var s=o.symbol.exports?e.firstOrUndefined(e.arrayFrom(o.symbol.exports.values())):o.symbol,c=r.symbolToExpression(o.symbol.parent?o.symbol.parent:o.symbol,111551,void 0,void 0);return void 0===s||void 0===c?e.factory.createNumericLiteral(0):e.factory.createPropertyAccessExpression(c,r.symbolToString(s))}if(256&o.flags)return e.factory.createNumericLiteral(o.value);if(2048&o.flags)return e.factory.createBigIntLiteral(o.value);if(128&o.flags)return e.factory.createStringLiteral(o.value,0===a);if(512&o.flags)return o===r.getFalseType()||o===r.getFalseType(!0)?e.factory.createFalse():e.factory.createTrue();if(65536&o.flags)return e.factory.createNull();if(1048576&o.flags){var l=e.firstDefined(o.types,function(e){return D(t,r,i,a,e)});return null!==l&&void 0!==l?l:A()}if(r.isArrayLikeType(o))return e.factory.createArrayLiteralExpression();if(C(o)){var u=e.map(r.getPropertiesOfType(o),function(n){var o=n.valueDeclaration?D(t,r,i,a,r.getTypeAtLocation(n.valueDeclaration)):A();return e.factory.createPropertyAssignment(n.name,o)});return e.factory.createObjectLiteralExpression(u,!0)}if(16&e.getObjectFlags(o)){var d=e.find(o.symbol.declarations||e.emptyArray,e.or(e.isFunctionTypeNode,e.isMethodSignature,e.isMethodDeclaration));if(void 0===d)return A();var p=r.getSignaturesOfType(o,0);if(void 0===p)return A();var f=n.createSignatureDeclarationFromSignature(211,t,a,p[0],n.createStubbedBody(e.Diagnostics.Function_not_implemented.message,a),void 0,void 0,void 0,void 0,i);return null!==f&&void 0!==f?f:A()}if(1&e.getObjectFlags(o)){var m=e.getClassLikeDeclarationOfSymbol(o.symbol);if(void 0===m||e.hasAbstractModifier(m))return A();var _=e.getFirstConstructorWithBody(m);return _&&e.length(_.parameters)?A():e.factory.createNewExpression(e.factory.createIdentifier(o.symbol.name),void 0,void 0)}return A()}function A(){return e.factory.createIdentifier("undefined")}function C(n){return 524288&n.flags&&(128&e.getObjectFlags(n)||n.symbol&&e.tryCast(e.singleOrUndefined(n.symbol.declarations),e.isTypeLiteralNode))}function I(n,t){var r=n.getContextualType(t.attributes);if(void 0===r)return e.emptyArray;var i=r.getProperties();if(!e.length(i))return e.emptyArray;for(var a=new e.Set,o=0,s=t.attributes.properties;o<s.length;o++){var c=s[o];if(e.isJsxAttribute(c)&&a.add(c.name.escapedText),e.isJsxSpreadAttribute(c))for(var l=n.getTypeAtLocation(c.expression),u=0,d=l.getProperties();u<d.length;u++){var p=d[u];a.add(p.escapedName)}}return e.filter(i,function(n){return!(16777216&n.flags||48&e.getCheckFlags(n)||a.has(n.escapedName))})}n.registerCodeFix({errorCodes:c,getCodeActions:function(t){var a=t.program.getTypeChecker(),c=l(t.sourceFile,t.span.start,a,t.program);if(c){if(3===c.kind){var u=e.textChanges.ChangeTracker.with(t,function(e){return L(e,t,c)});return[n.createCodeFixAction(i,u,e.Diagnostics.Add_missing_properties,i,e.Diagnostics.Add_all_missing_properties)]}if(4===c.kind){u=e.textChanges.ChangeTracker.with(t,function(e){return x(e,t,c)});return[n.createCodeFixAction(o,u,e.Diagnostics.Add_missing_attributes,o,e.Diagnostics.Add_all_missing_attributes)]}if(2===c.kind){u=e.textChanges.ChangeTracker.with(t,function(e){return S(e,t,c)});return[n.createCodeFixAction(s,u,[e.Diagnostics.Add_missing_function_declaration_0,c.token.text],s,e.Diagnostics.Add_all_missing_function_declarations)]}if(0===c.kind){u=e.textChanges.ChangeTracker.with(t,function(e){return T(e,t.program.getTypeChecker(),c)});return[n.createCodeFixAction(r,u,[e.Diagnostics.Add_missing_enum_member_0,c.token.text],r,e.Diagnostics.Add_all_missing_members)]}return e.concatenate(b(t,c),d(t,c))}},fixIds:[r,s,i,o],getAllCodeActions:function(t){var r=t.program,a=t.fixId,u=r.getTypeChecker(),d=new e.Map,p=new e.Map;return n.createCombinedCodeActions(e.textChanges.ChangeTracker.with(t,function(m){n.eachDiagnostic(t,c,function(n){var r=l(n.file,n.start,u,t.program);if(r&&e.addToSeen(d,e.getNodeId(r.parentDeclaration)+"#"+r.token.text))if(a===s&&2===r.kind)S(m,t,r);else if(a===i&&3===r.kind)L(m,t,r);else if(a===o&&4===r.kind)x(m,t,r);else if(0===r.kind&&T(m,u,r),1===r.kind){var c=r.parentDeclaration,f=r.token,_=e.getOrUpdate(p,c,function(){return[]});_.some(function(e){return e.token.text===f.text})||_.push(r)}}),p.forEach(function(i,a){for(var o=n.getAllSupers(a,u),s=function(n){if(o.some(function(e){var t=p.get(e);return!!t&&t.some(function(e){var t=e.token;return t.text===n.token.text})}))return"continue";var i=n.parentDeclaration,a=n.declSourceFile,s=n.modifierFlags,c=n.token,l=n.call,u=n.isJSFile;if(l&&!e.isPrivateIdentifier(c))E(t,m,l,c,32&s,i,a);else if(u&&!e.isInterfaceDeclaration(i))f(m,a,i,c,!!(32&s));else{var d=g(r.getTypeChecker(),i,c);y(m,a,i,c.text,d,32&s)}},c=0,l=i;c<l.length;c++){var d=l[c];s(d)}})}))}}),function(e){e[e["Enum"]=0]="Enum",e[e["ClassOrInterface"]=1]="ClassOrInterface",e[e["Function"]=2]="Function",e[e["ObjectLiteral"]=3]="ObjectLiteral",e[e["JsxAttributes"]=4]="JsxAttributes"}(t||(t={}))})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="addMissingNewOperator",r=[e.Diagnostics.Value_of_type_0_is_not_callable_Did_you_mean_to_include_new.code];function i(n,t,r){var i=e.cast(a(t,r),e.isCallExpression),o=e.factory.createNewExpression(i.expression,i.typeArguments,i.arguments);n.replaceNode(t,i,o)}function a(n,t){var r=e.getTokenAtPosition(n,t.start),i=e.textSpanEnd(t);while(r.end<i)r=r.parent;return r}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=r.sourceFile,o=r.span,s=e.textChanges.ChangeTracker.with(r,function(e){return i(e,a,o)});return[n.createCodeFixAction(t,s,e.Diagnostics.Add_missing_new_operator_to_call,t,e.Diagnostics.Add_missing_new_operator_to_all_calls)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){return i(n,e.sourceFile,t)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixCannotFindModule",r="installTypesPackage",i=e.Diagnostics.Cannot_find_module_0_or_its_corresponding_type_declarations.code,a=[i,e.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type.code];function o(e,n){return{type:"install package",file:e,packageName:n}}function s(n,t){var r=e.tryCast(e.getTokenAtPosition(n,t),e.isStringLiteral);if(r){var i=r.text,a=e.parsePackageName(i).packageName;return e.isExternalModuleNameRelative(a)?void 0:a}}function c(n,t,r){var a;return r===i?e.JsTyping.nodeCoreModules.has(n)?"@types/node":void 0:(null===(a=t.isKnownTypesPackageName)||void 0===a?void 0:a.call(t,n))?e.getTypesPackageName(n):void 0}n.registerCodeFix({errorCodes:a,getCodeActions:function(i){var a=i.host,l=i.sourceFile,u=i.span.start,d=s(l,u);if(void 0!==d){var p=c(d,a,i.errorCode);return void 0===p?[]:[n.createCodeFixAction(t,[],[e.Diagnostics.Install_0,p],r,e.Diagnostics.Install_all_missing_types_packages,o(l.fileName,p))]}},fixIds:[r],getAllCodeActions:function(t){return n.codeFixAll(t,a,function(n,i,a){var l=s(i.file,i.start);if(void 0!==l)switch(t.fixId){case r:var u=c(l,t.host,i.code);u&&a.push(o(i.file.fileName,u));break;default:e.Debug.fail("Bad fixId: "+t.fixId)}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t=[e.Diagnostics.Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2.code,e.Diagnostics.Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1.code],r="fixClassDoesntImplementInheritedAbstractMember";function i(n,t){var r=e.getTokenAtPosition(n,t);return e.cast(r.parent,e.isClassLike)}function a(t,r,i,a,s){var c=e.getEffectiveBaseTypeNode(t),l=i.program.getTypeChecker(),u=l.getTypeAtLocation(c),d=l.getPropertiesOfType(u).filter(o),p=n.createImportAdder(r,i.program,s,i.host);n.createMissingMemberNodes(t,d,r,i,s,p,function(e){return a.insertNodeAtClassStart(r,t,e)}),p.writeFixes(a)}function o(n){var t=e.getSyntacticModifierFlags(e.first(n.getDeclarations()));return!(8&t)&&!!(128&t)}n.registerCodeFix({errorCodes:t,getCodeActions:function(t){var o=t.sourceFile,s=t.span,c=e.textChanges.ChangeTracker.with(t,function(e){return a(i(o,s.start),o,t,e,t.preferences)});return 0===c.length?void 0:[n.createCodeFixAction(r,c,e.Diagnostics.Implement_inherited_abstract_class,r,e.Diagnostics.Implement_all_inherited_abstract_classes)]},fixIds:[r],getAllCodeActions:function(r){var o=new e.Map;return n.codeFixAll(r,t,function(n,t){var s=i(t.file,t.start);e.addToSeen(o,e.getNodeId(s))&&a(s,r.sourceFile,r,n,r.preferences)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="classSuperMustPrecedeThisAccess",r=[e.Diagnostics.super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class.code];function i(e,n,t,r){e.insertNodeAtConstructorStart(n,t,r),e.delete(n,r)}function a(n,t){var r=e.getTokenAtPosition(n,t);if(108===r.kind){var i=e.getContainingFunction(r),a=o(i.body);return a&&!a.expression.arguments.some(function(n){return e.isPropertyAccessExpression(n)&&n.expression===r})?{constructor:i,superCall:a}:void 0}}function o(n){return e.isExpressionStatement(n)&&e.isSuperCall(n.expression)?n:e.isFunctionLike(n)?void 0:e.forEachChild(n,o)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,c=a(o,s.start);if(c){var l=c.constructor,u=c.superCall,d=e.textChanges.ChangeTracker.with(r,function(e){return i(e,o,l,u)});return[n.createCodeFixAction(t,d,e.Diagnostics.Make_super_call_the_first_statement_in_the_constructor,t,e.Diagnostics.Make_all_super_calls_the_first_statement_in_their_constructor)]}},fixIds:[t],getAllCodeActions:function(t){var o=t.sourceFile,s=new e.Map;return n.codeFixAll(t,r,function(n,t){var r=a(t.file,t.start);if(r){var c=r.constructor,l=r.superCall;e.addToSeen(s,e.getNodeId(c.parent))&&i(n,o,c,l)}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="constructorForDerivedNeedSuperCall",r=[e.Diagnostics.Constructors_for_derived_classes_must_contain_a_super_call.code];function i(n,t){var r=e.getTokenAtPosition(n,t);return e.Debug.assert(e.isConstructorDeclaration(r.parent),"token should be at the constructor declaration"),r.parent}function a(n,t,r){var i=e.factory.createExpressionStatement(e.factory.createCallExpression(e.factory.createSuper(),void 0,e.emptyArray));n.insertNodeAtConstructorStart(t,r,i)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,c=i(o,s.start),l=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,c)});return[n.createCodeFixAction(t,l,e.Diagnostics.Add_missing_super_call,t,e.Diagnostics.Add_all_missing_super_calls)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){return a(n,e.sourceFile,i(t.file,t.start))})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="enableExperimentalDecorators",r=[e.Diagnostics.Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_in_your_tsconfig_or_jsconfig_to_remove_this_warning.code];function i(t,r){n.setJsonCompilerOptionValue(t,r,"experimentalDecorators",e.factory.createTrue())}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=r.program.getCompilerOptions().configFile;if(void 0!==a){var o=e.textChanges.ChangeTracker.with(r,function(e){return i(e,a)});return[n.createCodeFixActionWithoutFixAll(t,o,e.Diagnostics.Enable_the_experimentalDecorators_option_in_your_configuration_file)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n){var t=e.program.getCompilerOptions().configFile;void 0!==t&&i(n,t)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixEnableJsxFlag",r=[e.Diagnostics.Cannot_use_JSX_unless_the_jsx_flag_is_provided.code];function i(t,r){n.setJsonCompilerOptionValue(t,r,"jsx",e.factory.createStringLiteral("react"))}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=r.program.getCompilerOptions().configFile;if(void 0!==a){var o=e.textChanges.ChangeTracker.with(r,function(e){return i(e,a)});return[n.createCodeFixActionWithoutFixAll(t,o,e.Diagnostics.Enable_the_jsx_flag_in_your_configuration_file)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n){var t=e.program.getCompilerOptions().configFile;void 0!==t&&i(n,t)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){n.registerCodeFix({errorCodes:[e.Diagnostics.Top_level_await_expressions_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_target_option_is_set_to_es2017_or_higher.code,e.Diagnostics.Top_level_for_await_loops_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_target_option_is_set_to_es2017_or_higher.code],getCodeActions:function(t){var r=t.program.getCompilerOptions(),i=r.configFile;if(void 0!==i){var a=[],o=e.getEmitModuleKind(r),s=o>=e.ModuleKind.ES2015&&o<e.ModuleKind.ESNext;if(s){var c=e.textChanges.ChangeTracker.with(t,function(t){n.setJsonCompilerOptionValue(t,i,"module",e.factory.createStringLiteral("esnext"))});a.push(n.createCodeFixActionWithoutFixAll("fixModuleOption",c,[e.Diagnostics.Set_the_module_option_in_your_configuration_file_to_0,"esnext"]))}var l=e.getEmitScriptTarget(r),u=l<4||l>99;if(u){c=e.textChanges.ChangeTracker.with(t,function(t){var r=e.getTsConfigObjectLiteralExpression(i);if(r){var a=[["target",e.factory.createStringLiteral("es2017")]];o===e.ModuleKind.CommonJS&&a.push(["module",e.factory.createStringLiteral("commonjs")]),n.setJsonCompilerOptionValues(t,i,a)}});a.push(n.createCodeFixActionWithoutFixAll("fixTargetOption",c,[e.Diagnostics.Set_the_target_option_in_your_configuration_file_to_0,"es2017"]))}return a.length?a:void 0}}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixPropertyAssignment",r=[e.Diagnostics.Did_you_mean_to_use_a_Colon_An_can_only_follow_a_property_name_when_the_containing_object_literal_is_part_of_a_destructuring_pattern.code];function i(n,t,r){n.replaceNode(t,r,e.factory.createPropertyAssignment(r.name,r.objectAssignmentInitializer))}function a(n,t){return e.cast(e.getTokenAtPosition(n,t).parent,e.isShorthandPropertyAssignment)}n.registerCodeFix({errorCodes:r,fixIds:[t],getCodeActions:function(r){var o=r.sourceFile,s=r.span,c=a(o,s.start),l=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,c)});return[n.createCodeFixAction(t,l,[e.Diagnostics.Change_0_to_1,"=",":"],t,[e.Diagnostics.Switch_each_misused_0_to_1,"=",":"])]},getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return i(e,n.file,a(n.file,n.start))})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="extendsInterfaceBecomesImplements",r=[e.Diagnostics.Cannot_extend_an_interface_0_Did_you_mean_implements.code];function i(n,t){var r=e.getTokenAtPosition(n,t),i=e.getContainingClass(r).heritageClauses,a=i[0].getFirstToken();return 94===a.kind?{extendsToken:a,heritageClauses:i}:void 0}function a(n,t,r,i){if(n.replaceNode(t,r,e.factory.createToken(117)),2===i.length&&94===i[0].token&&117===i[1].token){var a=i[1].getFirstToken(),o=a.getFullStart();n.replaceRange(t,{pos:o,end:o},e.factory.createToken(27));var s=t.text,c=a.end;while(c<s.length&&e.isWhiteSpaceSingleLine(s.charCodeAt(c)))c++;n.deleteRange(t,{pos:a.getStart(),end:c})}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=i(o,r.span.start);if(s){var c=s.extendsToken,l=s.heritageClauses,u=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,c,l)});return[n.createCodeFixAction(t,u,e.Diagnostics.Change_extends_to_implements,t,e.Diagnostics.Change_all_extended_interfaces_to_implements)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){var t=i(n.file,n.start);t&&a(e,n.file,t.extendsToken,t.heritageClauses)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="forgottenThisPropertyAccess",r=e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0.code,i=[e.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0.code,e.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies.code,r];function a(n,t,i){var a=e.getTokenAtPosition(n,t);if(e.isIdentifier(a))return{node:a,className:i===r?e.getContainingClass(a).name.text:void 0}}function o(n,t,r){var i=r.node,a=r.className;e.suppressLeadingAndTrailingTrivia(i),n.replaceNode(t,i,e.factory.createPropertyAccessExpression(a?e.factory.createIdentifier(a):e.factory.createThis(),i))}n.registerCodeFix({errorCodes:i,getCodeActions:function(r){var i=r.sourceFile,s=a(i,r.span.start,r.errorCode);if(s){var c=e.textChanges.ChangeTracker.with(r,function(e){return o(e,i,s)});return[n.createCodeFixAction(t,c,[e.Diagnostics.Add_0_to_unresolved_variable,s.className||"this"],t,e.Diagnostics.Add_qualifier_to_all_unresolved_variables_matching_a_member_name)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,i,function(n,t){var r=a(t.file,t.start,t.code);r&&o(n,e.sourceFile,r)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixInvalidJsxCharacters_expression",r="fixInvalidJsxCharacters_htmlEntity",i=[e.Diagnostics.Unexpected_token_Did_you_mean_or_gt.code,e.Diagnostics.Unexpected_token_Did_you_mean_or_rbrace.code];n.registerCodeFix({errorCodes:i,fixIds:[t,r],getCodeActions:function(i){var a=i.sourceFile,o=i.preferences,c=i.span,l=e.textChanges.ChangeTracker.with(i,function(e){return s(e,o,a,c.start,!1)}),u=e.textChanges.ChangeTracker.with(i,function(e){return s(e,o,a,c.start,!0)});return[n.createCodeFixAction(t,l,e.Diagnostics.Wrap_invalid_character_in_an_expression_container,t,e.Diagnostics.Wrap_all_invalid_characters_in_an_expression_container),n.createCodeFixAction(r,u,e.Diagnostics.Convert_invalid_character_to_its_html_entity_code,r,e.Diagnostics.Convert_all_invalid_characters_to_HTML_entity_code)]},getAllCodeActions:function(e){return n.codeFixAll(e,i,function(n,t){return s(n,e.preferences,t.file,t.start,e.fixId===r)})}});var a={">":"&gt;","}":"&rbrace;"};function o(n){return e.hasProperty(a,n)}function s(n,t,r,i,s){var c=r.getText()[i];if(o(c)){var l=s?a[c]:"{"+e.quote(r,t,c)+"}";n.replaceRangeWithText(r,{pos:i,end:i+1},l)}}})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="unusedIdentifier",r="unusedIdentifier_prefix",i="unusedIdentifier_delete",a="unusedIdentifier_deleteImports",o="unusedIdentifier_infer",s=[e.Diagnostics._0_is_declared_but_its_value_is_never_read.code,e.Diagnostics._0_is_declared_but_never_used.code,e.Diagnostics.Property_0_is_declared_but_its_value_is_never_read.code,e.Diagnostics.All_imports_in_import_declaration_are_unused.code,e.Diagnostics.All_destructured_elements_are_unused.code,e.Diagnostics.All_variables_are_unused.code,e.Diagnostics.All_type_parameters_are_unused.code];function c(n,t,r){n.replaceNode(t,r.parent,e.factory.createKeywordTypeNode(153))}function l(r,a){return n.createCodeFixAction(t,r,a,i,e.Diagnostics.Delete_all_unused_declarations)}function u(n,t,r){n.delete(t,e.Debug.checkDefined(e.cast(r.parent,e.isDeclarationWithTypeParameterChildren).typeParameters,"The type parameter to delete should exist"))}function d(e){return 100===e.kind||79===e.kind&&(268===e.parent.kind||265===e.parent.kind)}function p(n){return 100===n.kind?e.tryCast(n.parent,e.isImportDeclaration):void 0}function f(n,t){return e.isVariableDeclarationList(t.parent)&&e.first(t.parent.getChildren(n))===t}function m(e,n,t){e.delete(n,235===t.parent.kind?t.parent:t)}function _(n,t,r){e.forEach(r.elements,function(e){return n.delete(t,e)})}function g(n,t,r,i){t!==e.Diagnostics.Property_0_is_declared_but_its_value_is_never_read.code&&(136===i.kind&&(i=e.cast(i.parent,e.isInferTypeNode).typeParameter.name),e.isIdentifier(i)&&y(i)&&(n.replaceNode(r,i,e.factory.createIdentifier("_"+i.text)),e.isParameter(i.parent)&&e.getJSDocParameterTags(i.parent).forEach(function(t){e.isIdentifier(t.name)&&n.replaceNode(r,t.name,e.factory.createIdentifier("_"+t.name.text))})))}function y(e){switch(e.parent.kind){case 162:case 161:return!0;case 252:var n=e.parent;switch(n.parent.parent.kind){case 242:case 241:return!0}}return!1}function h(n,t,r,i,a,o,s,c){v(t,r,n,i,a,o,s,c),e.isIdentifier(t)&&e.FindAllReferences.Core.eachSymbolReferenceInFile(t,i,n,function(t){e.isPropertyAccessExpression(t.parent)&&t.parent.name===t&&(t=t.parent),!c&&L(t)&&r.delete(n,t.parent.parent)})}function v(n,t,r,i,a,o,s,c){var l=n.parent;if(e.isParameter(l))b(t,r,l,i,a,o,s,c);else if(!(c&&e.isIdentifier(n)&&e.FindAllReferences.Core.isSymbolReferencedInFile(n,i,r))){var u=e.isImportClause(l)?n:e.isComputedPropertyName(l)?l.parent:l;e.Debug.assert(u!==r,"should not delete whole source file"),t.delete(r,u)}}function b(n,t,r,i,a,o,s,c){void 0===c&&(c=!1),T(i,t,r,a,o,s,c)&&(r.modifiers&&r.modifiers.length>0&&(!e.isIdentifier(r.name)||e.FindAllReferences.Core.isSymbolReferencedInFile(r.name,i,t))?r.modifiers.forEach(function(e){return n.deleteModifier(t,e)}):!r.initializer&&E(r,i,a)&&n.delete(t,r))}function E(n,t,r){var i=n.parent.parameters.indexOf(n);return!e.FindAllReferences.Core.someSignatureUsage(n.parent,r,t,function(e,n){return!n||n.arguments.length>i})}function T(n,t,r,i,a,o,s){var c=r.parent;switch(c.kind){case 167:case 169:var l=c.parameters.indexOf(r),u=e.isMethodDeclaration(c)?c.name:c,d=e.FindAllReferences.Core.getReferencedSymbolsForNode(c.pos,u,a,i,o);if(d)for(var p=0,f=d;p<f.length;p++)for(var m=f[p],_=0,g=m.references;_<g.length;_++){var y=g[_];if(1===y.kind){var h=e.isSuperKeyword(y.node)&&e.isCallExpression(y.node.parent)&&y.node.parent.arguments.length>l,v=e.isPropertyAccessExpression(y.node.parent)&&e.isSuperKeyword(y.node.parent.expression)&&e.isCallExpression(y.node.parent.parent)&&y.node.parent.parent.arguments.length>l,b=(e.isMethodDeclaration(y.node.parent)||e.isMethodSignature(y.node.parent))&&y.node.parent!==r.parent&&y.node.parent.parameters.length>l;if(h||v||b)return!1}}return!0;case 254:return!c.name||!S(n,t,c.name)||x(c,r,s);case 211:case 212:return x(c,r,s);case 171:return!1;default:return e.Debug.failBadSyntaxKind(c)}}function S(n,t,r){return!!e.FindAllReferences.Core.eachSymbolReferenceInFile(r,n,t,function(n){return e.isIdentifier(n)&&e.isCallExpression(n.parent)&&n.parent.arguments.indexOf(n)>=0})}function x(n,t,r){var i=n.parameters,a=i.indexOf(t);return e.Debug.assert(-1!==a,"The parameter should already be in the list"),r?i.slice(a+1).every(function(n){return e.isIdentifier(n.name)&&!n.symbol.isReferenced}):a===i.length-1}function L(n){return(e.isBinaryExpression(n.parent)&&n.parent.left===n||(e.isPostfixUnaryExpression(n.parent)||e.isPrefixUnaryExpression(n.parent))&&n.parent.operand===n)&&e.isExpressionStatement(n.parent.parent)}n.registerCodeFix({errorCodes:s,getCodeActions:function(i){var s=i.errorCode,y=i.sourceFile,v=i.program,b=i.cancellationToken,E=v.getTypeChecker(),T=v.getSourceFiles(),S=e.getTokenAtPosition(y,i.span.start);if(e.isJSDocTemplateTag(S))return[l(e.textChanges.ChangeTracker.with(i,function(e){return e.delete(y,S)}),e.Diagnostics.Remove_template_tag)];if(29===S.kind){var x=e.textChanges.ChangeTracker.with(i,function(e){return u(e,y,S)});return[l(x,e.Diagnostics.Remove_type_parameters)]}var L=p(S);if(L){x=e.textChanges.ChangeTracker.with(i,function(e){return e.delete(y,L)});return[n.createCodeFixAction(t,x,[e.Diagnostics.Remove_import_from_0,e.showModuleSpecifier(L)],a,e.Diagnostics.Delete_all_unused_imports)]}if(d(S)){var D=e.textChanges.ChangeTracker.with(i,function(e){return h(y,S,e,E,T,v,b,!1)});if(D.length)return[n.createCodeFixAction(t,D,[e.Diagnostics.Remove_unused_declaration_for_Colon_0,S.getText(y)],a,e.Diagnostics.Delete_all_unused_imports)]}if(e.isObjectBindingPattern(S.parent)||e.isArrayBindingPattern(S.parent)){if(e.isParameter(S.parent.parent)){var A=S.parent.elements,C=[A.length>1?e.Diagnostics.Remove_unused_declarations_for_Colon_0:e.Diagnostics.Remove_unused_declaration_for_Colon_0,e.map(A,function(e){return e.getText(y)}).join(", ")];return[l(e.textChanges.ChangeTracker.with(i,function(e){return _(e,y,S.parent)}),C)]}return[l(e.textChanges.ChangeTracker.with(i,function(e){return e.delete(y,S.parent.parent)}),e.Diagnostics.Remove_unused_destructuring_declaration)]}if(f(y,S))return[l(e.textChanges.ChangeTracker.with(i,function(e){return m(e,y,S.parent)}),e.Diagnostics.Remove_variable_statement)];var I=[];if(136===S.kind){x=e.textChanges.ChangeTracker.with(i,function(e){return c(e,y,S)});var N=e.cast(S.parent,e.isInferTypeNode).typeParameter.name.text;I.push(n.createCodeFixAction(t,x,[e.Diagnostics.Replace_infer_0_with_unknown,N],o,e.Diagnostics.Replace_all_unused_infer_with_unknown))}else{D=e.textChanges.ChangeTracker.with(i,function(e){return h(y,S,e,E,T,v,b,!1)});if(D.length){N=e.isComputedPropertyName(S.parent)?S.parent:S;I.push(l(D,[e.Diagnostics.Remove_unused_declaration_for_Colon_0,N.getText(y)]))}}var k=e.textChanges.ChangeTracker.with(i,function(e){return g(e,s,y,S)});return k.length&&I.push(n.createCodeFixAction(t,k,[e.Diagnostics.Prefix_0_with_an_underscore,S.getText(y)],r,e.Diagnostics.Prefix_all_unused_declarations_with_where_possible)),I},fixIds:[r,i,a,o],getAllCodeActions:function(t){var l=t.sourceFile,_=t.program,y=t.cancellationToken,v=_.getTypeChecker(),b=_.getSourceFiles();return n.codeFixAll(t,s,function(n,s){var T=e.getTokenAtPosition(l,s.start);switch(t.fixId){case r:g(n,s.code,l,T);break;case a:var S=p(T);S?n.delete(l,S):d(T)&&h(l,T,n,v,b,_,y,!0);break;case i:if(136===T.kind||d(T))break;if(e.isJSDocTemplateTag(T))n.delete(l,T);else if(29===T.kind)u(n,l,T);else if(e.isObjectBindingPattern(T.parent)){if(T.parent.parent.initializer)break;e.isParameter(T.parent.parent)&&!E(T.parent.parent,v,b)||n.delete(l,T.parent.parent)}else{if(e.isArrayBindingPattern(T.parent.parent)&&T.parent.parent.parent.initializer)break;f(l,T)?m(n,l,T.parent):h(l,T,n,v,b,_,y,!0)}break;case o:136===T.kind&&c(n,l,T);break;default:e.Debug.fail(JSON.stringify(t.fixId))}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixUnreachableCode",r=[e.Diagnostics.Unreachable_code_detected.code];function i(n,t,r,i,o){var s=e.getTokenAtPosition(t,r),c=e.findAncestor(s,e.isStatement);if(c.getStart(t)!==s.getStart(t)){var l=JSON.stringify({statementKind:e.Debug.formatSyntaxKind(c.kind),tokenKind:e.Debug.formatSyntaxKind(s.kind),errorCode:o,start:r,length:i});e.Debug.fail("Token and statement should start at the same point. "+l)}var u=(e.isBlock(c.parent)?c.parent:c).parent;if(!e.isBlock(c.parent)||c===e.first(c.parent.statements))switch(u.kind){case 237:if(u.elseStatement){if(e.isBlock(c.parent))break;return void n.replaceNode(t,c,e.factory.createBlock(e.emptyArray))}case 239:case 240:return void n.delete(t,u)}if(e.isBlock(c.parent)){var d=r+i,p=e.Debug.checkDefined(a(e.sliceAfter(c.parent.statements,c),function(e){return e.pos<d}),"Some statement should be last");n.deleteNodeRange(t,c,p)}else n.delete(t,c)}function a(e,n){for(var t,r=0,i=e;r<i.length;r++){var a=i[r];if(!n(a))break;t=a}return t}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span.start,r.span.length,r.errorCode)});return[n.createCodeFixAction(t,a,e.Diagnostics.Remove_unreachable_code,t,e.Diagnostics.Remove_all_unreachable_code)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return i(e,n.file,n.start,n.length,n.code)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixUnusedLabel",r=[e.Diagnostics.Unused_label.code];function i(n,t,r){var i=e.getTokenAtPosition(t,r),a=e.cast(i.parent,e.isLabeledStatement),o=i.getStart(t),s=a.statement.getStart(t),c=e.positionsAreOnSameLine(o,s,t)?s:e.skipTrivia(t.text,e.findChildOfKind(a,58,t).end,!0);n.deleteRange(t,{pos:o,end:c})}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span.start)});return[n.createCodeFixAction(t,a,e.Diagnostics.Remove_unused_label,t,e.Diagnostics.Remove_all_unused_labels)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return i(e,n.file,n.start)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixJSDocTypes_plain",r="fixJSDocTypes_nullable",i=[e.Diagnostics.JSDoc_types_can_only_be_used_inside_documentation_comments.code];function a(e,n,t,r,i){e.replaceNode(n,t,i.typeToTypeNode(r,t,void 0))}function o(n,t,r){var i=e.findAncestor(e.getTokenAtPosition(n,t),s),a=i&&i.type;return a&&{typeNode:a,type:r.getTypeFromTypeNode(a)}}function s(e){switch(e.kind){case 227:case 172:case 173:case 254:case 170:case 174:case 193:case 167:case 166:case 162:case 165:case 164:case 171:case 257:case 209:case 252:return!0;default:return!1}}n.registerCodeFix({errorCodes:i,getCodeActions:function(i){var s=i.sourceFile,c=i.program.getTypeChecker(),l=o(s,i.span.start,c);if(l){var u=l.typeNode,d=l.type,p=u.getText(s),f=[m(d,t,e.Diagnostics.Change_all_jsdoc_style_types_to_TypeScript)];return 309===u.kind&&f.push(m(c.getNullableType(d,32768),r,e.Diagnostics.Change_all_jsdoc_style_types_to_TypeScript_and_add_undefined_to_nullable_types)),f}function m(t,r,o){var l=e.textChanges.ChangeTracker.with(i,function(e){return a(e,s,u,t,c)});return n.createCodeFixAction("jdocTypes",l,[e.Diagnostics.Change_0_to_1,p,c.typeToString(t)],r,o)}},fixIds:[t,r],getAllCodeActions:function(e){var t=e.fixId,s=e.program,c=e.sourceFile,l=s.getTypeChecker();return n.codeFixAll(e,i,function(e,n){var i=o(n.file,n.start,l);if(i){var s=i.typeNode,u=i.type,d=309===s.kind&&t===r?l.getNullableType(u,32768):u;a(e,c,s,d,l)}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixMissingCallParentheses",r=[e.Diagnostics.This_condition_will_always_return_true_since_this_function_is_always_defined_Did_you_mean_to_call_it_instead.code];function i(e,n,t){e.replaceNodeWithText(n,t,t.text+"()")}function a(n,t){var r=e.getTokenAtPosition(n,t);if(e.isPropertyAccessExpression(r.parent)){var i=r.parent;while(e.isPropertyAccessExpression(i.parent))i=i.parent;return i.name}if(e.isIdentifier(r))return r}n.registerCodeFix({errorCodes:r,fixIds:[t],getCodeActions:function(r){var o=r.sourceFile,s=r.span,c=a(o,s.start);if(c){var l=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,c)});return[n.createCodeFixAction(t,l,e.Diagnostics.Add_missing_call_parentheses,t,e.Diagnostics.Add_all_missing_call_parentheses)]}},getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){var t=a(n.file,n.start);t&&i(e,n.file,t)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixAwaitInSyncFunction",r=[e.Diagnostics.await_expressions_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules.code,e.Diagnostics.for_await_loops_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules.code];function i(n){return n.type?n.type:e.isVariableDeclaration(n.parent)&&n.parent.type&&e.isFunctionTypeNode(n.parent.type)?n.parent.type.type:void 0}function a(n,t){var r=e.getTokenAtPosition(n,t),a=e.getContainingFunction(r);if(a){var o;switch(a.kind){case 167:o=a.name;break;case 254:case 211:o=e.findChildOfKind(a,98,n);break;case 212:o=e.findChildOfKind(a,20,n)||e.first(a.parameters);break;default:return}return o&&{insertBefore:o,returnType:i(a)}}}function o(n,t,r){var i=r.insertBefore,a=r.returnType;if(a){var o=e.getEntityNameFromTypeNode(a);o&&79===o.kind&&"Promise"===o.text||n.replaceNode(t,a,e.factory.createTypeReferenceNode("Promise",e.factory.createNodeArray([a])))}n.insertModifierBefore(t,130,i)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var i=r.sourceFile,s=r.span,c=a(i,s.start);if(c){var l=e.textChanges.ChangeTracker.with(r,function(e){return o(e,i,c)});return[n.createCodeFixAction(t,l,e.Diagnostics.Add_async_modifier_to_containing_function,t,e.Diagnostics.Add_all_missing_async_modifiers)]}},fixIds:[t],getAllCodeActions:function(t){var i=new e.Map;return n.codeFixAll(t,r,function(n,r){var s=a(r.file,r.start);s&&e.addToSeen(i,e.getNodeId(s.insertBefore))&&o(n,t.sourceFile,s)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t=[e.Diagnostics._0_is_defined_as_an_accessor_in_class_1_but_is_overridden_here_in_2_as_an_instance_property.code,e.Diagnostics._0_is_defined_as_a_property_in_class_1_but_is_overridden_here_in_2_as_an_accessor.code],r="fixPropertyOverrideAccessor";function i(t,r,i,a,o){var s,c;if(a===e.Diagnostics._0_is_defined_as_an_accessor_in_class_1_but_is_overridden_here_in_2_as_an_instance_property.code)s=r,c=r+i;else if(a===e.Diagnostics._0_is_defined_as_a_property_in_class_1_but_is_overridden_here_in_2_as_an_accessor.code){var l=o.program.getTypeChecker(),u=e.getTokenAtPosition(t,r).parent;e.Debug.assert(e.isAccessor(u),"error span of fixPropertyOverrideAccessor should only be on an accessor");var d=u.parent;e.Debug.assert(e.isClassLike(d),"erroneous accessors should only be inside classes");var p=e.singleOrUndefined(n.getAllSupers(d,l));if(!p)return[];var f=e.unescapeLeadingUnderscores(e.getTextOfPropertyName(u.name)),m=l.getPropertyOfType(l.getTypeAtLocation(p),f);if(!m||!m.valueDeclaration)return[];s=m.valueDeclaration.pos,c=m.valueDeclaration.end,t=e.getSourceFileOfNode(m.valueDeclaration)}else e.Debug.fail("fixPropertyOverrideAccessor codefix got unexpected error code "+a);return n.generateAccessorFromProperty(t,o.program,s,c,o,e.Diagnostics.Generate_get_and_set_accessors.message)}n.registerCodeFix({errorCodes:t,getCodeActions:function(t){var a=i(t.sourceFile,t.span.start,t.span.length,t.errorCode,t);if(a)return[n.createCodeFixAction(r,a,e.Diagnostics.Generate_get_and_set_accessors,r,e.Diagnostics.Generate_get_and_set_accessors_for_all_overriding_properties)]},fixIds:[r],getAllCodeActions:function(e){return n.codeFixAll(e,t,function(n,t){var r=i(t.file,t.start,t.length,t.code,e);if(r)for(var a=0,o=r;a<o.length;a++){var s=o[a];n.pushRaw(e.sourceFile,s)}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="inferFromUsage",r=[e.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined.code,e.Diagnostics.Variable_0_implicitly_has_an_1_type.code,e.Diagnostics.Parameter_0_implicitly_has_an_1_type.code,e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code,e.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation.code,e.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type.code,e.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation.code,e.Diagnostics.Member_0_implicitly_has_an_1_type.code,e.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_but_a_better_type_may_be_inferred_from_usage.code,e.Diagnostics.Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code,e.Diagnostics.Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code,e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage.code,e.Diagnostics.Property_0_implicitly_has_type_any_but_a_better_type_for_its_get_accessor_may_be_inferred_from_usage.code,e.Diagnostics._0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage.code,e.Diagnostics.Property_0_implicitly_has_type_any_but_a_better_type_for_its_set_accessor_may_be_inferred_from_usage.code,e.Diagnostics.Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code,e.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation.code];function i(n,t){switch(n){case e.Diagnostics.Parameter_0_implicitly_has_an_1_type.code:case e.Diagnostics.Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code:return e.isSetAccessorDeclaration(e.getContainingFunction(t))?e.Diagnostics.Infer_type_of_0_from_usage:e.Diagnostics.Infer_parameter_types_from_usage;case e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code:case e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage.code:return e.Diagnostics.Infer_parameter_types_from_usage;case e.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation.code:return e.Diagnostics.Infer_this_type_of_0_from_usage;default:return e.Diagnostics.Infer_type_of_0_from_usage}}function o(n){switch(n){case e.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_but_a_better_type_may_be_inferred_from_usage.code:return e.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined.code;case e.Diagnostics.Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code:return e.Diagnostics.Variable_0_implicitly_has_an_1_type.code;case e.Diagnostics.Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code:return e.Diagnostics.Parameter_0_implicitly_has_an_1_type.code;case e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage.code:return e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code;case e.Diagnostics.Property_0_implicitly_has_type_any_but_a_better_type_for_its_get_accessor_may_be_inferred_from_usage.code:return e.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation.code;case e.Diagnostics._0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage.code:return e.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type.code;case e.Diagnostics.Property_0_implicitly_has_type_any_but_a_better_type_for_its_set_accessor_may_be_inferred_from_usage.code:return e.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation.code;case e.Diagnostics.Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code:return e.Diagnostics.Member_0_implicitly_has_an_1_type.code}return n}function s(t,r,i,a,s,d,m,_,y){if(e.isParameterPropertyModifier(i.kind)||79===i.kind||25===i.kind||108===i.kind){var h=i.parent,v=n.createImportAdder(r,s,y,_);switch(a=o(a),a){case e.Diagnostics.Member_0_implicitly_has_an_1_type.code:case e.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined.code:if(e.isVariableDeclaration(h)&&m(h)||e.isPropertyDeclaration(h)||e.isPropertySignature(h))return c(t,v,r,h,s,_,d),v.writeFixes(t),h;if(e.isPropertyAccessExpression(h)){var E=b(h.name,s,d),T=e.getTypeNodeIfAccessible(E,h,s,_);if(T){var S=e.factory.createJSDocTypeTag(void 0,e.factory.createJSDocTypeExpression(T),void 0);g(t,r,e.cast(h.parent.parent,e.isExpressionStatement),[S])}return v.writeFixes(t),h}return;case e.Diagnostics.Variable_0_implicitly_has_an_1_type.code:var x=s.getTypeChecker().getSymbolAtLocation(i);return x&&x.valueDeclaration&&e.isVariableDeclaration(x.valueDeclaration)&&m(x.valueDeclaration)?(c(t,v,r,x.valueDeclaration,s,_,d),v.writeFixes(t),x.valueDeclaration):void 0}var L=e.getContainingFunction(i);if(void 0!==L){var D;switch(a){case e.Diagnostics.Parameter_0_implicitly_has_an_1_type.code:if(e.isSetAccessorDeclaration(L)){p(t,v,r,L,s,_,d),D=L;break}case e.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code:if(m(L)){var A=e.cast(h,e.isParameter);l(t,v,r,A,L,s,_,d),D=A}break;case e.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation.code:case e.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type.code:e.isGetAccessorDeclaration(L)&&e.isIdentifier(L.name)&&(f(t,v,r,L,b(L.name,s,d),s,_),D=L);break;case e.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation.code:e.isSetAccessorDeclaration(L)&&(p(t,v,r,L,s,_,d),D=L);break;case e.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation.code:e.textChanges.isThisTypeAnnotatable(L)&&m(L)&&(u(t,r,L,s,_,d),D=L);break;default:return e.Debug.fail(String(a))}return v.writeFixes(t),D}}}function c(n,t,r,i,a,o,s){e.isIdentifier(i.name)&&f(n,t,r,i,b(i.name,a,s),a,o)}function l(n,t,r,i,a,o,s,c){if(e.isIdentifier(i.name)){var l=E(a,r,o,c);if(e.Debug.assert(a.parameters.length===l.length,"Parameter count and inference count should match"),e.isInJSFile(a))_(n,r,l,o,s);else{var u=e.isArrowFunction(a)&&!e.findChildOfKind(a,20,r);u&&n.insertNodeBefore(r,e.first(a.parameters),e.factory.createToken(20));for(var d=0,p=l;d<p.length;d++){var m=p[d],g=m.declaration,y=m.type;!g||g.type||g.initializer||f(n,t,r,g,y,o,s)}u&&n.insertNodeAfter(r,e.last(a.parameters),e.factory.createToken(21))}}}function u(n,t,r,i,a,o){var s=T(r,t,i,o);if(s&&s.length){var c=S(i,s,o).thisParameter(),l=e.getTypeNodeIfAccessible(c,r,i,a);l&&(e.isInJSFile(r)?d(n,t,r,l):n.tryInsertThisTypeAnnotation(t,r,l))}}function d(n,t,r,i){g(n,t,r,[e.factory.createJSDocThisTag(void 0,e.factory.createJSDocTypeExpression(i))])}function p(n,t,r,i,a,o,s){var c=e.firstOrUndefined(i.parameters);if(c&&e.isIdentifier(i.name)&&e.isIdentifier(c.name)){var l=b(i.name,a,s);l===a.getTypeChecker().getAnyType()&&(l=b(c.name,a,s)),e.isInJSFile(i)?_(n,r,[{declaration:c,type:l}],a,o):f(n,t,r,c,l,a,o)}}function f(n,t,r,i,a,o,s){var c=e.getTypeNodeIfAccessible(a,i,o,s);if(c)if(e.isInJSFile(r)&&164!==i.kind){var l=e.isVariableDeclaration(i)?e.tryCast(i.parent.parent,e.isVariableStatement):i;if(!l)return;var u=e.factory.createJSDocTypeExpression(c),d=e.isGetAccessorDeclaration(i)?e.factory.createJSDocReturnTag(void 0,u,void 0):e.factory.createJSDocTypeTag(void 0,u,void 0);g(n,r,l,[d])}else m(c,i,r,n,t,e.getEmitScriptTarget(o.getCompilerOptions()))||n.tryInsertTypeAnnotation(r,i,c)}function m(t,r,i,a,o,s){var c=n.tryGetAutoImportableReferenceFromTypeNode(t,s);return!(!c||!a.tryInsertTypeAnnotation(i,r,c.typeNode))&&(e.forEach(c.symbols,function(e){return o.addImportFromExportedSymbol(e,!0)}),!0)}function _(n,t,r,i,a){var o=r.length&&r[0].declaration.parent;if(o){var s=e.mapDefined(r,function(n){var t=n.declaration;if(!t.initializer&&!e.getJSDocType(t)&&e.isIdentifier(t.name)){var r=n.type&&e.getTypeNodeIfAccessible(n.type,t,i,a);if(r){var o=e.factory.cloneNode(t.name);return e.setEmitFlags(o,3584),{name:e.factory.cloneNode(t.name),param:t,isOptional:!!n.isOptional,typeNode:r}}}});if(s.length)if(e.isArrowFunction(o)||e.isFunctionExpression(o)){var c=e.isArrowFunction(o)&&!e.findChildOfKind(o,20,t);c&&n.insertNodeBefore(t,e.first(o.parameters),e.factory.createToken(20)),e.forEach(s,function(r){var i=r.typeNode,a=r.param,o=e.factory.createJSDocTypeTag(void 0,e.factory.createJSDocTypeExpression(i)),s=e.factory.createJSDocComment(void 0,[o]);n.insertNodeAt(t,a.getStart(t),s,{suffix:" "})}),c&&n.insertNodeAfter(t,e.last(o.parameters),e.factory.createToken(21))}else{var l=e.map(s,function(n){var t=n.name,r=n.typeNode,i=n.isOptional;return e.factory.createJSDocParameterTag(void 0,t,!!i,e.factory.createJSDocTypeExpression(r),!1,void 0)});g(n,t,o,l)}}}function g(n,t,r,i){var o=e.flatMap(r.jsDoc,function(n){return"string"===typeof n.comment?e.factory.createJSDocText(n.comment):n.comment}),s=e.flatMapToMutable(r.jsDoc,function(e){return e.tags}),c=i.filter(function(e){return!s||!s.some(function(n,t){var r=h(n,e);return r&&(s[t]=r),!!r})}),l=e.factory.createJSDocComment(e.factory.createNodeArray(e.intersperse(o,e.factory.createJSDocText("\n"))),e.factory.createNodeArray(a(a([],s||e.emptyArray,!0),c,!0))),u=212===r.kind?y(r):r;u.jsDoc=r.jsDoc,u.jsDocCache=r.jsDocCache,n.insertJsdocCommentBefore(t,u,l)}function y(e){return 165===e.parent.kind?e.parent:e.parent.parent}function h(n,t){if(n.kind===t.kind)switch(n.kind){case 335:var r=n,i=t;return e.isIdentifier(r.name)&&e.isIdentifier(i.name)&&r.name.escapedText===i.name.escapedText?e.factory.createJSDocParameterTag(void 0,i.name,!1,i.typeExpression,i.isNameFirst,r.comment):void 0;case 336:return e.factory.createJSDocReturnTag(void 0,t.typeExpression,n.comment)}}function v(n,t,r){return e.mapDefined(e.FindAllReferences.getReferenceEntriesForNode(-1,n,t,t.getSourceFiles(),r),function(n){return 0!==n.kind?e.tryCast(n.node,e.isIdentifier):void 0})}function b(e,n,t){var r=v(e,n,t);return S(n,r,t).single()}function E(n,t,r,i){var a=T(n,t,r,i);return a&&S(r,a,i).parameters(n)||n.parameters.map(function(n){return{declaration:n,type:e.isIdentifier(n.name)?b(n.name,r,i):r.getTypeChecker().getAnyType()}})}function T(n,t,r,i){var a;switch(n.kind){case 169:a=e.findChildOfKind(n,133,t);break;case 212:case 211:var o=n.parent;a=(e.isVariableDeclaration(o)||e.isPropertyDeclaration(o))&&e.isIdentifier(o.name)?o.name:n.name;break;case 254:case 167:case 166:a=n.name;break}if(a)return v(a,r,i)}function S(n,t,r){var i=n.getTypeChecker(),o={string:function(){return i.getStringType()},number:function(){return i.getNumberType()},Array:function(e){return i.createArrayType(e)},Promise:function(e){return i.createPromiseType(e)}},s=[i.getStringType(),i.getNumberType(),i.createArrayType(i.getAnyType()),i.createPromiseType(i.getAnyType())];return{single:u,parameters:d,thisParameter:p};function c(){return{isNumber:void 0,isString:void 0,isNumberOrString:void 0,candidateTypes:void 0,properties:void 0,calls:void 0,constructs:void 0,numberIndex:void 0,stringIndex:void 0,candidateThisTypes:void 0,inferredTypes:void 0}}function l(n){for(var t=new e.Map,r=0,i=n;r<i.length;r++){var a=i[r];a.properties&&a.properties.forEach(function(e,n){t.has(n)||t.set(n,[]),t.get(n).push(e)})}var o=new e.Map;return t.forEach(function(e,n){o.set(n,l(e))}),{isNumber:n.some(function(e){return e.isNumber}),isString:n.some(function(e){return e.isString}),isNumberOrString:n.some(function(e){return e.isNumberOrString}),candidateTypes:e.flatMap(n,function(e){return e.candidateTypes}),properties:o,calls:e.flatMap(n,function(e){return e.calls}),constructs:e.flatMap(n,function(e){return e.constructs}),numberIndex:e.forEach(n,function(e){return e.numberIndex}),stringIndex:e.forEach(n,function(e){return e.stringIndex}),candidateThisTypes:e.flatMap(n,function(e){return e.candidateThisTypes}),inferredTypes:void 0}}function u(){return C(f(t))}function d(o){if(0!==t.length&&o.parameters){for(var s=c(),l=0,u=t;l<u.length;l++){var d=u[l];r.throwIfCancellationRequested(),m(d,s)}var p=a(a([],s.constructs||[],!0),s.calls||[],!0);return o.parameters.map(function(t,a){for(var s=[],c=e.isRestParameter(t),l=!1,u=0,d=p;u<d.length;u++){var m=d[u];if(m.argumentTypes.length<=a)l=e.isInJSFile(o),s.push(i.getUndefinedType());else if(c)for(var _=a;_<m.argumentTypes.length;_++)s.push(i.getBaseTypeOfLiteralType(m.argumentTypes[_]));else s.push(i.getBaseTypeOfLiteralType(m.argumentTypes[a]))}if(e.isIdentifier(t.name)){var g=f(v(t.name,n,r));s.push.apply(s,c?e.mapDefined(g,i.getElementTypeOfArrayType):g)}var y=C(s);return{type:c?i.createArrayType(y):y,isOptional:l&&!c,declaration:t}})}}function p(){for(var n=c(),i=0,a=t;i<a.length;i++){var o=a[i];r.throwIfCancellationRequested(),m(o,n)}return C(n.candidateThisTypes||e.emptyArray)}function f(e){for(var n=c(),t=0,i=e;t<i.length;t++){var a=i[t];r.throwIfCancellationRequested(),m(a,n)}return N(n)}function m(n,t){while(e.isRightSideOfQualifiedNameOrPropertyAccess(n))n=n.parent;switch(n.parent.kind){case 236:g(n,t);break;case 218:t.isNumber=!0;break;case 217:y(n.parent,t);break;case 219:h(n,n.parent,t);break;case 287:case 288:b(n.parent,t);break;case 206:case 207:n.parent.expression===n?E(n.parent,t):_(n,t);break;case 204:T(n.parent,t);break;case 205:S(n.parent,n,t);break;case 291:case 292:x(n.parent,t);break;case 165:L(n.parent,t);break;case 252:var r=n.parent,a=r.name,o=r.initializer;if(n===a){o&&B(t,i.getTypeAtLocation(o));break}default:return _(n,t)}}function _(n,t){e.isExpressionNode(n)&&B(t,i.getContextualType(n))}function g(n,t){B(t,e.isCallExpression(n)?i.getVoidType():i.getAnyType())}function y(e,n){switch(e.operator){case 45:case 46:case 40:case 54:n.isNumber=!0;break;case 39:n.isNumberOrString=!0;break}}function h(n,t,r){switch(t.operatorToken.kind){case 42:case 41:case 43:case 44:case 47:case 48:case 49:case 50:case 51:case 52:case 65:case 67:case 66:case 68:case 69:case 73:case 74:case 78:case 70:case 72:case 71:case 40:case 29:case 32:case 31:case 33:var a=i.getTypeAtLocation(t.left===n?t.right:t.left);1056&a.flags?B(r,a):r.isNumber=!0;break;case 64:case 39:var o=i.getTypeAtLocation(t.left===n?t.right:t.left);1056&o.flags?B(r,o):296&o.flags?r.isNumber=!0:402653316&o.flags?r.isString=!0:1&o.flags||(r.isNumberOrString=!0);break;case 63:case 34:case 36:case 37:case 35:B(r,i.getTypeAtLocation(t.left===n?t.right:t.left));break;case 101:n===t.left&&(r.isString=!0);break;case 56:case 60:n!==t.left||252!==n.parent.parent.kind&&!e.isAssignmentExpression(n.parent.parent,!0)||B(r,i.getTypeAtLocation(t.right));break;case 55:case 27:case 102:break}}function b(e,n){B(n,i.getTypeAtLocation(e.parent.parent.expression))}function E(e,n){var t={argumentTypes:[],return_:c()};if(e.arguments)for(var r=0,a=e.arguments;r<a.length;r++){var o=a[r];t.argumentTypes.push(i.getTypeAtLocation(o))}m(e,t.return_),206===e.kind?(n.calls||(n.calls=[])).push(t):(n.constructs||(n.constructs=[])).push(t)}function T(n,t){var r=e.escapeLeadingUnderscores(n.name.text);t.properties||(t.properties=new e.Map);var i=t.properties.get(r)||c();m(n,i),t.properties.set(r,i)}function S(e,n,t){if(n!==e.argumentExpression){var r=i.getTypeAtLocation(e.argumentExpression),a=c();m(e,a),296&r.flags?t.numberIndex=a:t.stringIndex=a}else t.isNumberOrString=!0}function x(n,t){var r=e.isVariableDeclaration(n.parent.parent)?n.parent.parent:n.parent;U(t,i.getTypeAtLocation(r))}function L(e,n){U(n,i.getTypeAtLocation(e.parent))}function D(n,t){for(var r=[],i=0,a=n;i<a.length;i++)for(var o=a[i],s=0,c=t;s<c.length;s++){var l=c[s],u=l.high,d=l.low;u(o)&&(e.Debug.assert(!d(o),"Priority can't have both low and high"),r.push(d))}return n.filter(function(e){return r.every(function(n){return!n(e)})})}function A(e){return C(N(e))}function C(n){if(!n.length)return i.getAnyType();var t=i.getUnionType([i.getStringType(),i.getNumberType()]),r=[{high:function(e){return e===i.getStringType()||e===i.getNumberType()},low:function(e){return e===t}},{high:function(e){return!(16385&e.flags)},low:function(e){return!!(16385&e.flags)}},{high:function(n){return!(114689&n.flags)&&!(16&e.getObjectFlags(n))},low:function(n){return!!(16&e.getObjectFlags(n))}}],a=D(n,r),o=a.filter(function(n){return 16&e.getObjectFlags(n)});return o.length&&(a=a.filter(function(n){return!(16&e.getObjectFlags(n))}),a.push(I(o))),i.getWidenedType(i.getUnionType(a.map(i.getBaseTypeOfLiteralType),2))}function I(n){if(1===n.length)return n[0];for(var t=[],r=[],a=[],o=[],s=!1,c=!1,l=e.createMultiMap(),u=0,d=n;u<d.length;u++){for(var p=d[u],f=0,m=i.getPropertiesOfType(p);f<m.length;f++){var _=m[f];l.add(_.name,_.valueDeclaration?i.getTypeOfSymbolAtLocation(_,_.valueDeclaration):i.getAnyType())}t.push.apply(t,i.getSignaturesOfType(p,0)),r.push.apply(r,i.getSignaturesOfType(p,1));var g=i.getIndexInfoOfType(p,0);g&&(a.push(g.type),s=s||g.isReadonly);var y=i.getIndexInfoOfType(p,1);y&&(o.push(y.type),c=c||y.isReadonly)}var h=e.mapEntries(l,function(e,t){var r=t.length<n.length?16777216:0,a=i.createSymbol(4|r,e);return a.type=i.getUnionType(t),[e,a]}),v=[];return a.length&&v.push(i.createIndexInfo(i.getStringType(),i.getUnionType(a),s)),o.length&&v.push(i.createIndexInfo(i.getNumberType(),i.getUnionType(o),c)),i.createAnonymousType(n[0].symbol,h,t,r,v)}function N(e){var n,t,r,a=[];return e.isNumber&&a.push(i.getNumberType()),e.isString&&a.push(i.getStringType()),e.isNumberOrString&&a.push(i.getUnionType([i.getStringType(),i.getNumberType()])),e.numberIndex&&a.push(i.createArrayType(A(e.numberIndex))),((null===(n=e.properties)||void 0===n?void 0:n.size)||(null===(t=e.calls)||void 0===t?void 0:t.length)||(null===(r=e.constructs)||void 0===r?void 0:r.length)||e.stringIndex)&&a.push(k(e)),a.push.apply(a,(e.candidateTypes||[]).map(function(e){return i.getBaseTypeOfLiteralType(e)})),a.push.apply(a,R(e)),a}function k(n){var t=new e.Map;n.properties&&n.properties.forEach(function(e,n){var r=i.createSymbol(4,n);r.type=A(e),t.set(n,r)});var r=n.calls?[G(n.calls)]:[],a=n.constructs?[G(n.constructs)]:[],o=n.stringIndex?[i.createIndexInfo(i.getStringType(),A(n.stringIndex),!1)]:[];return i.createAnonymousType(void 0,t,r,a,o)}function R(e){if(!e.properties||!e.properties.size)return[];var n=s.filter(function(n){return O(n,e)});return 0<n.length&&n.length<3?n.map(function(n){return w(n,e)}):[]}function O(n,t){return!!t.properties&&!e.forEachEntry(t.properties,function(e,t){var r=i.getTypeOfPropertyOfType(n,t);if(!r)return!0;if(e.calls){var a=i.getSignaturesOfType(r,0);return!a.length||!i.isTypeAssignableTo(r,F(e.calls))}return!i.isTypeAssignableTo(r,A(e))})}function w(n,t){if(!(4&e.getObjectFlags(n))||!t.properties)return n;var r=n.target,a=e.singleOrUndefined(r.typeParameters);if(!a)return n;var s=[];return t.properties.forEach(function(n,t){var o=i.getTypeOfPropertyOfType(r,t);e.Debug.assert(!!o,"generic should have all the properties of its reference."),s.push.apply(s,P(o,A(n),a))}),o[n.symbol.escapedName](C(s))}function P(n,t,r){if(n===r)return[t];if(3145728&n.flags)return e.flatMap(n.types,function(e){return P(e,t,r)});if(4&e.getObjectFlags(n)&&4&e.getObjectFlags(t)){var a=i.getTypeArguments(n),o=i.getTypeArguments(t),s=[];if(a&&o)for(var c=0;c<a.length;c++)o[c]&&s.push.apply(s,P(a[c],o[c],r));return s}var l=i.getSignaturesOfType(n,0),u=i.getSignaturesOfType(t,0);return 1===l.length&&1===u.length?M(l[0],u[0],r):[]}function M(n,t,r){for(var a=[],o=0;o<n.parameters.length;o++){var s=n.parameters[o],c=t.parameters[o],l=n.declaration&&e.isRestParameter(n.declaration.parameters[o]);if(!c)break;var u=s.valueDeclaration?i.getTypeOfSymbolAtLocation(s,s.valueDeclaration):i.getAnyType(),d=l&&i.getElementTypeOfArrayType(u);d&&(u=d);var p=c.type||(c.valueDeclaration?i.getTypeOfSymbolAtLocation(c,c.valueDeclaration):i.getAnyType());a.push.apply(a,P(u,p,r))}var f=i.getReturnTypeOfSignature(n),m=i.getReturnTypeOfSignature(t);return a.push.apply(a,P(f,m,r)),a}function F(n){return i.createAnonymousType(void 0,e.createSymbolTable(),[G(n)],e.emptyArray,e.emptyArray)}function G(n){for(var t=[],r=Math.max.apply(Math,n.map(function(e){return e.argumentTypes.length})),a=function(r){var a=i.createSymbol(1,e.escapeLeadingUnderscores("arg"+r));a.type=C(n.map(function(e){return e.argumentTypes[r]||i.getUndefinedType()})),n.some(function(e){return void 0===e.argumentTypes[r]})&&(a.flags|=16777216),t.push(a)},o=0;o<r;o++)a(o);var s=A(l(n.map(function(e){return e.return_})));return i.createSignature(void 0,void 0,void 0,t,s,void 0,r,0)}function B(e,n){!n||1&n.flags||131072&n.flags||(e.candidateTypes||(e.candidateTypes=[])).push(n)}function U(e,n){!n||1&n.flags||131072&n.flags||(e.candidateThisTypes||(e.candidateThisTypes=[])).push(n)}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a,o=r.sourceFile,c=r.program,l=r.span.start,u=r.errorCode,d=r.cancellationToken,p=r.host,f=r.preferences,m=e.getTokenAtPosition(o,l),_=e.textChanges.ChangeTracker.with(r,function(n){a=s(n,o,m,u,c,d,e.returnTrue,p,f)}),g=a&&e.getNameOfDeclaration(a);return g&&0!==_.length?[n.createCodeFixAction(t,_,[i(u,m),g.getText(o)],t,e.Diagnostics.Infer_all_types_from_usage)]:void 0},fixIds:[t],getAllCodeActions:function(t){var i=t.sourceFile,a=t.program,o=t.cancellationToken,c=t.host,l=t.preferences,u=e.nodeSeenTracker();return n.codeFixAll(t,r,function(n,t){s(n,i,e.getTokenAtPosition(t.file,t.start),t.code,a,o,u,c,l)})}}),n.addJSDocTags=g})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixReturnTypeInAsyncFunction",r=[e.Diagnostics.The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type_Did_you_mean_to_write_Promise_0.code];function i(n,t,r){if(!e.isInJSFile(n)){var i=e.getTokenAtPosition(n,r),a=e.findAncestor(i,e.isFunctionLikeDeclaration),o=null===a||void 0===a?void 0:a.type;if(o){var s=t.getTypeFromTypeNode(o),c=t.getAwaitedType(s)||t.getVoidType(),l=t.typeToTypeNode(c,o,void 0);return l?{returnTypeNode:o,returnType:s,promisedTypeNode:l,promisedType:c}:void 0}}}function a(n,t,r,i){n.replaceNode(t,r,e.factory.createTypeReferenceNode("Promise",[i]))}n.registerCodeFix({errorCodes:r,fixIds:[t],getCodeActions:function(r){var o=r.sourceFile,s=r.program,c=r.span,l=s.getTypeChecker(),u=i(o,s.getTypeChecker(),c.start);if(u){var d=u.returnTypeNode,p=u.returnType,f=u.promisedTypeNode,m=u.promisedType,_=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,d,f)});return[n.createCodeFixAction(t,_,[e.Diagnostics.Replace_0_with_Promise_1,l.typeToString(p),l.typeToString(m)],t,e.Diagnostics.Fix_all_incorrect_return_type_of_an_async_functions)]}},getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){var r=i(t.file,e.program.getTypeChecker(),t.start);r&&a(n,t.file,r.returnTypeNode,r.promisedTypeNode)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="disableJsDiagnostics",r="disableJsDiagnostics",i=e.mapDefined(Object.keys(e.Diagnostics),function(n){var t=e.Diagnostics[n];return t.category===e.DiagnosticCategory.Error?t.code:void 0});function a(n,t,r,i){var a=e.getLineAndCharacterOfPosition(t,r).line;i&&!e.tryAddToSet(i,a)||n.insertCommentBeforeLine(t,a,r," @ts-ignore")}n.registerCodeFix({errorCodes:i,getCodeActions:function(i){var o=i.sourceFile,s=i.program,c=i.span,l=i.host,u=i.formatContext;if(e.isInJSFile(o)&&e.isCheckJsEnabledForFile(o,s.getCompilerOptions())){var d=o.checkJsDirective?"":e.getNewLineOrDefaultFromHost(l,u.options),p=[n.createCodeFixActionWithoutFixAll(t,[n.createFileTextChanges(o.fileName,[e.createTextChange(o.checkJsDirective?e.createTextSpanFromBounds(o.checkJsDirective.pos,o.checkJsDirective.end):e.createTextSpan(0,0),"// @ts-nocheck"+d)])],e.Diagnostics.Disable_checking_for_this_file)];return e.textChanges.isValidLocationToAddComment(o,c.start)&&p.unshift(n.createCodeFixAction(t,e.textChanges.ChangeTracker.with(i,function(e){return a(e,o,c.start)}),e.Diagnostics.Ignore_this_error_message,r,e.Diagnostics.Add_ts_ignore_to_all_error_messages)),p}},fixIds:[r],getAllCodeActions:function(t){var r=new e.Set;return n.codeFixAll(t,i,function(n,t){e.textChanges.isValidLocationToAddComment(t.file,t.start)&&a(n,t.file,t.start,r)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){function t(e,n,t,r,a,o,s){for(var c=e.symbol.members,l=0,u=n;l<u.length;l++){var d=u[l];c.has(d.escapedName)||i(d,e,t,r,a,o,s)}}function r(n){return{trackSymbol:function(){return!1},moduleResolverHost:e.getModuleSpecifierResolverHost(n.program,n.host)}}function i(n,t,i,o,s,u,d){var f=n.getDeclarations();if(f&&f.length){var _=o.program.getTypeChecker(),g=e.getEmitScriptTarget(o.program.getCompilerOptions()),y=f[0],h=e.getSynthesizedDeepClone(e.getNameOfDeclaration(y),!1),b=m(e.getEffectiveModifierFlags(y)),T=b?e.factory.createNodeArray([b]):void 0,S=_.getWidenedType(_.getTypeOfSymbolAtLocation(n,t)),x=!!(16777216&n.flags),L=!!(8388608&t.flags),D=e.getQuotePreference(i,s);switch(y.kind){case 164:case 165:var A=0===D?268435456:void 0,C=_.typeToTypeNode(S,t,A,r(o));if(u){var I=v(C,g);I&&(C=I.typeNode,E(u,I.symbols))}d(e.factory.createPropertyDeclaration(void 0,T,h,x?e.factory.createToken(57):void 0,C,void 0));break;case 170:case 171:var N=_.typeToTypeNode(S,t,void 0,r(o)),k=e.getAllAccessorDeclarations(f,y),R=k.secondAccessor?[k.firstAccessor,k.secondAccessor]:[k.firstAccessor];if(u){I=v(N,g);I&&(N=I.typeNode,E(u,I.symbols))}for(var O=0,w=R;O<w.length;O++){var P=w[O];if(e.isGetAccessorDeclaration(P))d(e.factory.createGetAccessorDeclaration(void 0,T,h,e.emptyArray,N,L?void 0:p(D)));else{e.Debug.assertNode(P,e.isSetAccessorDeclaration,"The counterpart to a getter should be a setter");var M=e.getSetAccessorValueParameter(P),F=M&&e.isIdentifier(M.name)?e.idText(M.name):void 0;d(e.factory.createSetAccessorDeclaration(void 0,T,h,c(1,[F],[N],1,!1),L?void 0:p(D)))}}break;case 166:case 167:var G=_.getSignaturesOfType(S,0);if(!e.some(G))break;if(1===f.length){e.Debug.assert(1===G.length,"One declaration implies one signature");var B=G[0];j(D,B,T,h,L?void 0:p(D));break}for(var U=0,V=G;U<V.length;U++){B=V[U];j(D,B,e.getSynthesizedDeepClones(T,!1),e.getSynthesizedDeepClone(h,!1))}if(!L)if(f.length>G.length){B=_.getSignatureFromDeclaration(f[f.length-1]);j(D,B,T,h,p(D))}else e.Debug.assert(f.length===G.length,"Declarations and signatures should match count"),d(l(_,o,t,G,h,x,T,D));break}}function j(e,n,r,i,s){var c=a(167,o,e,n,s,i,r,x,t,u);c&&d(c)}}function a(n,t,i,a,o,s,c,l,u,d){var p=t.program,f=p.getTypeChecker(),m=e.getEmitScriptTarget(p.getCompilerOptions()),_=1073742081|(0===i?268435456:0),g=f.signatureToSignatureDeclaration(a,n,u,_,r(t));if(g){var y=g.typeParameters,h=g.parameters,b=g.type;if(d){if(y){var T=e.sameMap(y,function(n){var t=n.constraint,r=n.default;if(t){var i=v(t,m);i&&(t=i.typeNode,E(d,i.symbols))}if(r){i=v(r,m);i&&(r=i.typeNode,E(d,i.symbols))}return e.factory.updateTypeParameterDeclaration(n,n.name,t,r)});y!==T&&(y=e.setTextRange(e.factory.createNodeArray(T,y.hasTrailingComma),y))}var S=e.sameMap(h,function(n){var t=v(n.type,m),r=n.type;return t&&(r=t.typeNode,E(d,t.symbols)),e.factory.updateParameterDeclaration(n,n.decorators,n.modifiers,n.dotDotDotToken,n.name,n.questionToken,r,n.initializer)});if(h!==S&&(h=e.setTextRange(e.factory.createNodeArray(S,h.hasTrailingComma),h)),b){var x=v(b,m);x&&(b=x.typeNode,E(d,x.symbols))}}var L=l?e.factory.createToken(57):void 0,D=g.asteriskToken;return e.isFunctionExpression(g)?e.factory.updateFunctionExpression(g,c,g.asteriskToken,e.tryCast(s,e.isIdentifier),y,h,b,null!==o&&void 0!==o?o:g.body):e.isArrowFunction(g)?e.factory.updateArrowFunction(g,c,y,h,b,g.equalsGreaterThanToken,null!==o&&void 0!==o?o:g.body):e.isMethodDeclaration(g)?e.factory.updateMethodDeclaration(g,void 0,c,D,null!==s&&void 0!==s?s:e.factory.createIdentifier(""),L,y,h,b,o):void 0}}function o(n,t,i,a,o,l,u){var d=e.getQuotePreference(t.sourceFile,t.preferences),m=e.getEmitScriptTarget(t.program.getCompilerOptions()),_=r(t),g=t.program.getTypeChecker(),y=e.isInJSFile(u),h=a.typeArguments,v=a.arguments,b=a.parent,E=y?void 0:g.getContextualType(a),T=e.map(v,function(n){return e.isIdentifier(n)?n.text:e.isPropertyAccessExpression(n)&&e.isIdentifier(n.name)?n.name.text:void 0}),S=y?[]:e.map(v,function(e){return s(g,i,g.getBaseTypeOfLiteralType(g.getTypeAtLocation(e)),u,m,void 0,_)}),x=l?e.factory.createNodeArray(e.factory.createModifiersFromModifierFlags(l)):void 0,L=e.isYieldExpression(b)?e.factory.createToken(41):void 0,D=y||void 0===h?void 0:e.map(h,function(n,t){return e.factory.createTypeParameterDeclaration(84+h.length-1<=90?String.fromCharCode(84+t):"T"+t)}),A=c(v.length,T,S,void 0,y),C=y||void 0===E?void 0:g.typeToTypeNode(E,u,void 0,_);return 167===n?e.factory.createMethodDeclaration(void 0,x,L,o,void 0,D,A,C,e.isInterfaceDeclaration(u)?void 0:p(d)):e.factory.createFunctionDeclaration(void 0,x,L,o,D,A,C,f(e.Diagnostics.Function_not_implemented.message,d))}function s(n,t,r,i,a,o,s){var c=n.typeToTypeNode(r,i,o,s);if(c&&e.isImportTypeNode(c)){var l=v(c,a);l&&(E(t,l.symbols),c=l.typeNode)}return e.getSynthesizedDeepClone(c)}function c(n,t,r,i,a){for(var o=[],s=0;s<n;s++){var c=e.factory.createParameterDeclaration(void 0,void 0,void 0,t&&t[s]||"arg"+s,void 0!==i&&s>=i?e.factory.createToken(57):void 0,a?void 0:r&&r[s]||e.factory.createKeywordTypeNode(129),void 0);o.push(c)}return o}function l(n,t,r,i,a,o,s,l){for(var p=i[0],f=i[0].minArgumentCount,m=!1,_=0,g=i;_<g.length;_++){var y=g[_];f=Math.min(y.minArgumentCount,f),e.signatureHasRestParameter(y)&&(m=!0),y.parameters.length>=p.parameters.length&&(!e.signatureHasRestParameter(y)||e.signatureHasRestParameter(p))&&(p=y)}var h=p.parameters.length-(e.signatureHasRestParameter(p)?1:0),v=p.parameters.map(function(e){return e.name}),b=c(h,v,void 0,f,!1);if(m){var E=e.factory.createArrayTypeNode(e.factory.createKeywordTypeNode(129)),T=e.factory.createParameterDeclaration(void 0,void 0,e.factory.createToken(25),v[h]||"rest",h>=f?e.factory.createToken(57):void 0,E,void 0);b.push(T)}return d(s,a,o,void 0,b,u(i,n,t,r),l)}function u(n,t,i,a){if(e.length(n)){var o=t.getUnionType(e.map(n,t.getReturnTypeOfSignature));return t.typeToTypeNode(o,a,void 0,r(i))}}function d(n,t,r,i,a,o,s){return e.factory.createMethodDeclaration(void 0,n,void 0,t,r?e.factory.createToken(57):void 0,i,a,o,p(s))}function p(n){return f(e.Diagnostics.Method_not_implemented.message,n)}function f(n,t){return e.factory.createBlock([e.factory.createThrowStatement(e.factory.createNewExpression(e.factory.createIdentifier("Error"),void 0,[e.factory.createStringLiteral(n,0===t)]))],!0)}function m(n){return 4&n?e.factory.createToken(123):16&n?e.factory.createToken(122):void 0}function _(n,t,r){var i=e.getTsConfigObjectLiteralExpression(t);if(i){var a=h(i,"compilerOptions");if(void 0!==a){var o=a.initializer;if(e.isObjectLiteralExpression(o))for(var s=0,c=r;s<c.length;s++){var l=c[s],u=l[0],d=l[1],p=h(o,u);void 0===p?n.insertNodeAtObjectStart(t,o,y(u,d)):n.replaceNode(t,p.initializer,d)}}else n.insertNodeAtObjectStart(t,i,y("compilerOptions",e.factory.createObjectLiteralExpression(r.map(function(e){var n=e[0],t=e[1];return y(n,t)}),!0)))}}function g(e,n,t,r){_(e,n,[[t,r]])}function y(n,t){return e.factory.createPropertyAssignment(e.factory.createStringLiteral(n),t)}function h(n,t){return e.find(n.properties,function(n){return e.isPropertyAssignment(n)&&!!n.name&&e.isStringLiteral(n.name)&&n.name.text===t})}function v(n,t){var r,i=e.visitNode(n,a);if(r&&i)return{typeNode:i,symbols:r};function a(n){var i;if(e.isLiteralImportTypeNode(n)&&n.qualifier){var o=e.getFirstIdentifier(n.qualifier),s=e.getNameForExportedSymbol(o.symbol,t),c=s!==o.text?b(n.qualifier,e.factory.createIdentifier(s)):n.qualifier;r=e.append(r,o.symbol);var l=null===(i=n.typeArguments)||void 0===i?void 0:i.map(a);return e.factory.createTypeReferenceNode(c,l)}return e.visitEachChild(n,a,e.nullTransformationContext)}}function b(n,t){return 79===n.kind?t:e.factory.createQualifiedName(b(n.left,t),n.right)}function E(e,n){n.forEach(function(n){return e.addImportFromExportedSymbol(n,!0)})}n.createMissingMemberNodes=t,n.getNoopSymbolTrackerWithResolver=r,n.createSignatureDeclarationFromSignature=a,n.createSignatureDeclarationFromCallExpression=o,n.typeToAutoImportableTypeNode=s,n.createStubbedBody=f,n.setJsonCompilerOptionValues=_,n.setJsonCompilerOptionValue=g,n.createJsonPropertyAssignment=y,n.findJsonProperty=h,n.tryGetAutoImportableReferenceFromTypeNode=v,n.importSymbols=E})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){function t(n,t,r,i,a,o){var s=u(n,t,r,i);if(s&&!e.refactor.isRefactorErrorInfo(s)){var f,m,h=e.textChanges.ChangeTracker.fromContext(a),v=s.isStatic,b=s.isReadonly,E=s.fieldName,T=s.accessorName,S=s.originalName,x=s.type,L=s.container,D=s.declaration;if(e.suppressLeadingAndTrailingTrivia(E),e.suppressLeadingAndTrailingTrivia(T),e.suppressLeadingAndTrailingTrivia(D),e.suppressLeadingAndTrailingTrivia(L),e.isClassLike(L)){var A=e.getEffectiveModifierFlags(D);if(e.isSourceFileJS(n)){var C=e.createModifiers(A);f=C,m=C}else f=e.createModifiers(c(A)),m=e.createModifiers(l(A))}_(h,n,D,x,E,m);var I=d(E,T,x,f,v,L);if(e.suppressLeadingAndTrailingTrivia(I),g(h,n,I,D,L),b){var N=e.getFirstConstructorWithBody(L);N&&y(h,n,N,E.text,S)}else{var k=p(E,T,x,f,v,L);e.suppressLeadingAndTrailingTrivia(k),g(h,n,k,D,L)}return h.getChanges()}}function r(n){return e.isIdentifier(n)||e.isStringLiteral(n)}function i(n){return e.isParameterPropertyDeclaration(n,n.parent)||e.isPropertyDeclaration(n)||e.isPropertyAssignment(n)}function o(n,t){return e.isIdentifier(t)?e.factory.createIdentifier(n):e.factory.createStringLiteral(n)}function s(n,t,r){var i=t?r.name:e.factory.createThis();return e.isIdentifier(n)?e.factory.createPropertyAccessExpression(i,n):e.factory.createElementAccessExpression(i,e.factory.createStringLiteralFromNode(n))}function c(e){return e&=-65,e&=-9,16&e||(e|=4),e}function l(e){return e&=-5,e&=-17,e|=8,e}function u(n,t,a,s,c){void 0===c&&(c=!0);var l=e.getTokenAtPosition(n,a),u=a===s&&c,d=e.findAncestor(l.parent,i),p=124;if(!d||!e.nodeOverlapsWithStartEnd(d.name,n,a,s)&&!u)return{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_property_for_which_to_generate_accessor)};if(!r(d.name))return{error:e.getLocaleSpecificMessage(e.Diagnostics.Name_is_not_valid)};if((e.getEffectiveModifierFlags(d)|p)!==p)return{error:e.getLocaleSpecificMessage(e.Diagnostics.Can_only_convert_property_with_modifier)};var f=d.name.text,m=e.startsWithUnderscore(f),_=o(m?f:e.getUniqueName("_"+f,n),d.name),g=o(m?e.getUniqueName(f.substring(1),n):f,d.name);return{isStatic:e.hasStaticModifier(d),isReadonly:e.hasEffectiveReadonlyModifier(d),type:h(d,t),container:162===d.kind?d.parent.parent:d.parent,originalName:d.name.text,declaration:d,fieldName:_,accessorName:g,renameAccessor:m}}function d(n,t,r,i,a,o){return e.factory.createGetAccessorDeclaration(void 0,i,t,void 0,r,e.factory.createBlock([e.factory.createReturnStatement(s(n,a,o))],!0))}function p(n,t,r,i,a,o){return e.factory.createSetAccessorDeclaration(void 0,i,t,[e.factory.createParameterDeclaration(void 0,void 0,void 0,e.factory.createIdentifier("value"),void 0,r)],e.factory.createBlock([e.factory.createExpressionStatement(e.factory.createAssignment(s(n,a,o),e.factory.createIdentifier("value")))],!0))}function f(n,t,r,i,a,o){var s=e.factory.updatePropertyDeclaration(r,r.decorators,o,a,r.questionToken||r.exclamationToken,i,r.initializer);n.replaceNode(t,r,s)}function m(n,t,r,i){var a=e.factory.updatePropertyAssignment(r,i,r.initializer);n.replacePropertyAssignment(t,r,a)}function _(n,t,r,i,a,o){e.isPropertyDeclaration(r)?f(n,t,r,i,a,o):e.isPropertyAssignment(r)?m(n,t,r,a):n.replaceNode(t,r,e.factory.updateParameterDeclaration(r,r.decorators,o,r.dotDotDotToken,e.cast(a,e.isIdentifier),r.questionToken,r.type,r.initializer))}function g(n,t,r,i,a){e.isParameterPropertyDeclaration(i,i.parent)?n.insertNodeAtClassStart(t,a,r):e.isPropertyAssignment(i)?n.insertNodeAfterComma(t,i,r):n.insertNodeAfter(t,i,r)}function y(n,t,r,i,a){r.body&&r.body.forEachChild(function r(o){e.isElementAccessExpression(o)&&108===o.expression.kind&&e.isStringLiteral(o.argumentExpression)&&o.argumentExpression.text===a&&e.isWriteAccess(o)&&n.replaceNode(t,o.argumentExpression,e.factory.createStringLiteral(i)),e.isPropertyAccessExpression(o)&&108===o.expression.kind&&o.name.text===a&&e.isWriteAccess(o)&&n.replaceNode(t,o.name,e.factory.createIdentifier(i)),e.isFunctionLike(o)||e.isClassLike(o)||o.forEachChild(r)})}function h(n,t){var r=e.getTypeAnnotationNode(n);if(e.isPropertyDeclaration(n)&&r&&n.questionToken){var i=t.getTypeChecker(),o=i.getTypeFromTypeNode(r);if(!i.isTypeAssignableTo(i.getUndefinedType(),o)){var s=e.isUnionTypeNode(r)?r.types:[r];return e.factory.createUnionTypeNode(a(a([],s,!0),[e.factory.createKeywordTypeNode(151)],!1))}}return r}function v(n,t){var r=[];while(n){var i=e.getClassExtendsHeritageElement(n),a=i&&t.getSymbolAtLocation(i.expression);if(!a)break;var o=2097152&a.flags?t.getAliasedSymbol(a):a,s=o.declarations&&e.find(o.declarations,e.isClassLike);if(!s)break;r.push(s),n=s}return r}n.generateAccessorFromProperty=t,n.getAccessorConvertiblePropertyAtPosition=u,n.getAllSupers=v})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="invalidImportSyntax";function r(n,t){var r=e.getSourceFileOfNode(t),a=e.getNamespaceDeclarationNode(t),o=n.program.getCompilerOptions(),s=[];return s.push(i(n,r,t,e.makeImport(a.name,void 0,t.moduleSpecifier,e.getQuotePreference(r,n.preferences)))),e.getEmitModuleKind(o)===e.ModuleKind.CommonJS&&s.push(i(n,r,t,e.factory.createImportEqualsDeclaration(void 0,void 0,!1,a.name,e.factory.createExternalModuleReference(t.moduleSpecifier)))),s}function i(r,i,a,o){var s=e.textChanges.ChangeTracker.with(r,function(e){return e.replaceNode(i,a,o)});return n.createCodeFixActionWithoutFixAll(t,s,[e.Diagnostics.Replace_import_with_0,s[0].textChanges[0].newText])}function a(n){var t=n.sourceFile,r=e.Diagnostics.This_expression_is_not_callable.code===n.errorCode?206:207,i=e.findAncestor(e.getTokenAtPosition(t,n.span.start),function(e){return e.kind===r});if(!i)return[];var a=i.expression;return s(n,a)}function o(n){var t=n.sourceFile,r=e.findAncestor(e.getTokenAtPosition(t,n.span.start),function(e){return e.getStart()===n.span.start&&e.getEnd()===n.span.start+n.span.length});return r?s(n,r):[]}function s(i,a){var o=i.program.getTypeChecker().getTypeAtLocation(a);if(!o.symbol||!o.symbol.originatingImport)return[];var s=[],c=o.symbol.originatingImport;if(e.isImportCall(c)||e.addRange(s,r(i,c)),e.isExpression(a)&&(!e.isNamedDeclaration(a.parent)||a.parent.name!==a)){var l=i.sourceFile,u=e.textChanges.ChangeTracker.with(i,function(n){return n.replaceNode(l,a,e.factory.createPropertyAccessExpression(a,"default"),{})});s.push(n.createCodeFixActionWithoutFixAll(t,u,e.Diagnostics.Use_synthetic_default_member))}return s}n.registerCodeFix({errorCodes:[e.Diagnostics.This_expression_is_not_callable.code,e.Diagnostics.This_expression_is_not_constructable.code],getCodeActions:a}),n.registerCodeFix({errorCodes:[e.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1.code,e.Diagnostics.Type_0_does_not_satisfy_the_constraint_1.code,e.Diagnostics.Type_0_is_not_assignable_to_type_1.code,e.Diagnostics.Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated.code,e.Diagnostics.Type_predicate_0_is_not_assignable_to_1.code,e.Diagnostics.Property_0_of_type_1_is_not_assignable_to_2_index_type_3.code,e.Diagnostics._0_index_type_1_is_not_assignable_to_2_index_type_3.code,e.Diagnostics.Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2.code,e.Diagnostics.Property_0_in_type_1_is_not_assignable_to_type_2.code,e.Diagnostics.Property_0_of_JSX_spread_attribute_is_not_assignable_to_target_property.code,e.Diagnostics.The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1.code],getCodeActions:o})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="strictClassInitialization",r="addMissingPropertyDefiniteAssignmentAssertions",i="addMissingPropertyUndefinedType",a="addMissingPropertyInitializer",o=[e.Diagnostics.Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor.code];function s(n,t){var r=e.getTokenAtPosition(n,t);return e.isIdentifier(r)?e.cast(r.parent,e.isPropertyDeclaration):void 0}function c(i,a){var o=e.textChanges.ChangeTracker.with(i,function(e){return l(e,i.sourceFile,a)});return n.createCodeFixAction(t,o,[e.Diagnostics.Add_definite_assignment_assertion_to_property_0,a.getText()],r,e.Diagnostics.Add_definite_assignment_assertions_to_all_uninitialized_properties)}function l(n,t,r){var i=e.factory.updatePropertyDeclaration(r,r.decorators,r.modifiers,r.name,e.factory.createToken(53),r.type,r.initializer);n.replaceNode(t,r,i)}function u(r,a){var o=e.textChanges.ChangeTracker.with(r,function(e){return d(e,r.sourceFile,a)});return n.createCodeFixAction(t,o,[e.Diagnostics.Add_undefined_type_to_property_0,a.name.getText()],i,e.Diagnostics.Add_undefined_type_to_all_uninitialized_properties)}function d(n,t,r){var i=e.factory.createKeywordTypeNode(151),a=r.type,o=e.isUnionTypeNode(a)?a.types.concat(i):[a,i];n.replaceNode(t,a,e.factory.createUnionTypeNode(o))}function p(r,i){var o=r.program.getTypeChecker(),s=m(o,i);if(s){var c=e.textChanges.ChangeTracker.with(r,function(e){return f(e,r.sourceFile,i,s)});return n.createCodeFixAction(t,c,[e.Diagnostics.Add_initializer_to_property_0,i.name.getText()],a,e.Diagnostics.Add_initializers_to_all_uninitialized_properties)}}function f(n,t,r,i){var a=e.factory.updatePropertyDeclaration(r,r.decorators,r.modifiers,r.name,r.questionToken,r.type,i);n.replaceNode(t,r,a)}function m(e,n){return _(e,e.getTypeFromTypeNode(n.type))}function _(n,t){if(512&t.flags)return t===n.getFalseType()||t===n.getFalseType(!0)?e.factory.createFalse():e.factory.createTrue();if(t.isStringLiteral())return e.factory.createStringLiteral(t.value);if(t.isNumberLiteral())return e.factory.createNumericLiteral(t.value);if(2048&t.flags)return e.factory.createBigIntLiteral(t.value);if(t.isUnion())return e.firstDefined(t.types,function(e){return _(n,e)});if(t.isClass()){var r=e.getClassLikeDeclarationOfSymbol(t.symbol);if(!r||e.hasSyntacticModifier(r,128))return;var i=e.getFirstConstructorWithBody(r);if(i&&i.parameters.length)return;return e.factory.createNewExpression(e.factory.createIdentifier(t.symbol.name),void 0,void 0)}return n.isArrayLikeType(t)?e.factory.createArrayLiteralExpression():void 0}n.registerCodeFix({errorCodes:o,getCodeActions:function(n){var t=s(n.sourceFile,n.span.start);if(t){var r=[u(n,t),c(n,t)];return e.append(r,p(n,t)),r}},fixIds:[r,i,a],getAllCodeActions:function(t){return n.codeFixAll(t,o,function(n,o){var c=s(o.file,o.start);if(c)switch(t.fixId){case r:l(n,o.file,c);break;case i:d(n,o.file,c);break;case a:var u=t.program.getTypeChecker(),p=m(u,c);if(!p)return;f(n,o.file,c,p);break;default:e.Debug.fail(JSON.stringify(t.fixId))}})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="requireInTs",r=[e.Diagnostics.require_call_may_be_converted_to_an_import.code];function i(n,t,r){var i=r.allowSyntheticDefaults,a=r.defaultImportName,o=r.namedImports,s=r.statement,c=r.required;n.replaceNode(t,s,a&&!i?e.factory.createImportEqualsDeclaration(void 0,void 0,!1,a,e.factory.createExternalModuleReference(c)):e.factory.createImportDeclaration(void 0,void 0,e.factory.createImportClause(!1,a,o),c))}function a(n,t,r){var i=e.getTokenAtPosition(n,r).parent;if(!e.isRequireCall(i,!0))throw e.Debug.failBadSyntaxKind(i);var a=e.cast(i.parent,e.isVariableDeclaration),s=e.tryCast(a.name,e.isIdentifier),c=e.isObjectBindingPattern(a.name)?o(a.name):void 0;if(s||c)return{allowSyntheticDefaults:e.getAllowSyntheticDefaultImports(t.getCompilerOptions()),defaultImportName:s,namedImports:c,statement:e.cast(a.parent.parent,e.isVariableStatement),required:e.first(i.arguments)}}function o(n){for(var t=[],r=0,i=n.elements;r<i.length;r++){var a=i[r];if(!e.isIdentifier(a.name)||a.initializer)return;t.push(e.factory.createImportSpecifier(e.tryCast(a.propertyName,e.isIdentifier),a.name))}if(t.length)return e.factory.createNamedImports(t)}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=a(r.sourceFile,r.program,r.span.start);if(o){var s=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,o)});return[n.createCodeFixAction(t,s,e.Diagnostics.Convert_require_to_import,t,e.Diagnostics.Convert_all_require_to_import)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){var r=a(t.file,e.program,t.start);r&&i(n,e.sourceFile,r)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="useDefaultImport",r=[e.Diagnostics.Import_may_be_converted_to_a_default_import.code];function i(n,t){var r=e.getTokenAtPosition(n,t);if(e.isIdentifier(r)){var i=r.parent;if(e.isImportEqualsDeclaration(i)&&e.isExternalModuleReference(i.moduleReference))return{importNode:i,name:r,moduleSpecifier:i.moduleReference.expression};if(e.isNamespaceImport(i)){var a=i.parent.parent;return{importNode:a,name:r,moduleSpecifier:a.moduleSpecifier}}}}function a(n,t,r,i){n.replaceNode(t,r.importNode,e.makeImport(r.name,void 0,r.moduleSpecifier,e.getQuotePreference(t,i)))}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span.start,c=i(o,s);if(c){var l=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,c,r.preferences)});return[n.createCodeFixAction(t,l,e.Diagnostics.Convert_to_default_import,t,e.Diagnostics.Convert_all_to_default_imports)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){var r=i(t.file,t.start);r&&a(n,t.file,r,e.preferences)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="useBigintLiteral",r=[e.Diagnostics.Numeric_literals_with_absolute_values_equal_to_2_53_or_greater_are_too_large_to_be_represented_accurately_as_integers.code];function i(n,t,r){var i=e.tryCast(e.getTokenAtPosition(t,r.start),e.isNumericLiteral);if(i){var a=i.getText(t)+"n";n.replaceNode(t,i,e.factory.createBigIntLiteral(a))}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span)});if(a.length>0)return[n.createCodeFixAction(t,a,e.Diagnostics.Convert_to_a_bigint_numeric_literal,t,e.Diagnostics.Convert_all_to_bigint_numeric_literals)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return i(e,n.file,n)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixAddModuleReferTypeMissingTypeof",r=t,i=[e.Diagnostics.Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0.code];function a(n,t){var r=e.getTokenAtPosition(n,t);return e.Debug.assert(100===r.kind,"This token should be an ImportKeyword"),e.Debug.assert(198===r.parent.kind,"Token parent should be an ImportType"),r.parent}function o(n,t,r){var i=e.factory.updateImportTypeNode(r,r.argument,r.qualifier,r.typeArguments,!0);n.replaceNode(t,r,i)}n.registerCodeFix({errorCodes:i,getCodeActions:function(t){var i=t.sourceFile,s=t.span,c=a(i,s.start),l=e.textChanges.ChangeTracker.with(t,function(e){return o(e,i,c)});return[n.createCodeFixAction(r,l,e.Diagnostics.Add_missing_typeof,r,e.Diagnostics.Add_missing_typeof)]},fixIds:[r],getAllCodeActions:function(e){return n.codeFixAll(e,i,function(n,t){return o(n,e.sourceFile,a(t.file,t.start))})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="wrapJsxInFragment",r=[e.Diagnostics.JSX_expressions_must_have_one_parent_element.code];function i(n,t){var r=e.getTokenAtPosition(n,t),i=r.parent,a=i.parent;if((e.isBinaryExpression(a)||(a=a.parent,e.isBinaryExpression(a)))&&e.nodeIsMissing(a.operatorToken))return a}function a(n,t,r){var i=o(r);i&&n.replaceNode(t,r,e.factory.createJsxFragment(e.factory.createJsxOpeningFragment(),i,e.factory.createJsxJsxClosingFragment()))}function o(n){var t=[],r=n;while(1){if(e.isBinaryExpression(r)&&e.nodeIsMissing(r.operatorToken)&&27===r.operatorToken.kind){if(t.push(r.left),e.isJsxChild(r.right))return t.push(r.right),t;if(e.isBinaryExpression(r.right)){r=r.right;continue}return}return}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,c=i(o,s.start);if(c){var l=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,c)});return[n.createCodeFixAction(t,l,e.Diagnostics.Wrap_in_JSX_fragment,t,e.Diagnostics.Wrap_all_unparented_JSX_in_JSX_fragment)]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(n,t){var r=i(e.sourceFile,t.start);r&&a(n,e.sourceFile,r)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixConvertToMappedObjectType",r=t,i=[e.Diagnostics.An_index_signature_parameter_type_cannot_be_a_literal_type_or_generic_type_Consider_using_a_mapped_object_type_instead.code];function o(n,t){var r=e.getTokenAtPosition(n,t),i=e.cast(r.parent.parent,e.isIndexSignatureDeclaration);if(!e.isClassDeclaration(i.parent)){var a=e.isInterfaceDeclaration(i.parent)?i.parent:e.cast(i.parent.parent,e.isTypeAliasDeclaration);return{indexSignature:i,container:a}}}function s(n,t){return e.factory.createTypeAliasDeclaration(n.decorators,n.modifiers,n.name,n.typeParameters,t)}function c(n,t,r){var i=r.indexSignature,o=r.container,c=e.isInterfaceDeclaration(o)?o.members:o.type.members,l=c.filter(function(n){return!e.isIndexSignatureDeclaration(n)}),u=e.first(i.parameters),d=e.factory.createTypeParameterDeclaration(e.cast(u.name,e.isIdentifier),u.type),p=e.factory.createMappedTypeNode(e.hasEffectiveReadonlyModifier(i)?e.factory.createModifier(143):void 0,d,void 0,i.questionToken,i.type),f=e.factory.createIntersectionTypeNode(a(a(a([],e.getAllSuperTypeNodes(o),!0),[p],!1),l.length?[e.factory.createTypeLiteralNode(l)]:e.emptyArray,!0));n.replaceNode(t,o,s(o,f))}n.registerCodeFix({errorCodes:i,getCodeActions:function(t){var i=t.sourceFile,a=t.span,s=o(i,a.start);if(s){var l=e.textChanges.ChangeTracker.with(t,function(e){return c(e,i,s)}),u=e.idText(s.container.name);return[n.createCodeFixAction(r,l,[e.Diagnostics.Convert_0_to_mapped_object_type,u],r,[e.Diagnostics.Convert_0_to_mapped_object_type,u])]}},fixIds:[r],getAllCodeActions:function(e){return n.codeFixAll(e,i,function(e,n){var t=o(n.file,n.start);t&&c(e,n.file,t)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="removeAccidentalCallParentheses",r=[e.Diagnostics.This_expression_is_not_callable_because_it_is_a_get_accessor_Did_you_mean_to_use_it_without.code];n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var i=e.findAncestor(e.getTokenAtPosition(r.sourceFile,r.span.start),e.isCallExpression);if(i){var a=e.textChanges.ChangeTracker.with(r,function(e){e.deleteRange(r.sourceFile,{pos:i.expression.end,end:i.end})});return[n.createCodeFixActionWithoutFixAll(t,a,e.Diagnostics.Remove_parentheses)]}},fixIds:[t]})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="removeUnnecessaryAwait",r=[e.Diagnostics.await_has_no_effect_on_the_type_of_this_expression.code];function i(n,t,r){var i=e.tryCast(e.getTokenAtPosition(t,r.start),function(e){return 131===e.kind}),a=i&&e.tryCast(i.parent,e.isAwaitExpression);if(a){var o=a,s=e.isParenthesizedExpression(a.parent);if(s){var c=e.getLeftmostExpression(a.expression,!1);if(e.isIdentifier(c)){var l=e.findPrecedingToken(a.parent.pos,t);l&&103!==l.kind&&(o=a.parent)}}n.replaceNode(t,o,a.expression)}}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var a=e.textChanges.ChangeTracker.with(r,function(e){return i(e,r.sourceFile,r.span)});if(a.length>0)return[n.createCodeFixAction(t,a,e.Diagnostics.Remove_unnecessary_await,t,e.Diagnostics.Remove_all_unnecessary_uses_of_await)]},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,r,function(e,n){return i(e,n.file,n)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t=[e.Diagnostics.A_type_only_import_can_specify_a_default_import_or_named_bindings_but_not_both.code],r="splitTypeOnlyImport";function i(n,t){return e.findAncestor(e.getTokenAtPosition(n,t.start),e.isImportDeclaration)}function a(n,t,r){if(t){var i=e.Debug.checkDefined(t.importClause);n.replaceNode(r.sourceFile,t,e.factory.updateImportDeclaration(t,t.decorators,t.modifiers,e.factory.updateImportClause(i,i.isTypeOnly,i.name,void 0),t.moduleSpecifier)),n.insertNodeAfter(r.sourceFile,t,e.factory.createImportDeclaration(void 0,void 0,e.factory.updateImportClause(i,i.isTypeOnly,void 0,i.namedBindings),t.moduleSpecifier))}}n.registerCodeFix({errorCodes:t,fixIds:[r],getCodeActions:function(t){var o=e.textChanges.ChangeTracker.with(t,function(e){return a(e,i(t.sourceFile,t.span),t)});if(o.length)return[n.createCodeFixAction(r,o,e.Diagnostics.Split_into_two_separate_import_declarations,r,e.Diagnostics.Split_all_invalid_type_only_imports)]},getAllCodeActions:function(e){return n.codeFixAll(e,t,function(n,t){a(n,i(e.sourceFile,t),e)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixConvertConstToLet",r=[e.Diagnostics.Cannot_assign_to_0_because_it_is_a_constant.code];function i(n,t,r){var i,a=r.getTypeChecker(),o=a.getSymbolAtLocation(e.getTokenAtPosition(n,t)),s=e.tryCast(null===(i=null===o||void 0===o?void 0:o.valueDeclaration)||void 0===i?void 0:i.parent,e.isVariableDeclarationList);if(void 0!==s){var c=e.findChildOfKind(s,85,n);if(void 0!==c)return e.createRange(c.pos,c.end)}}function a(e,n,t){e.replaceRangeWithText(n,t,"let")}n.registerCodeFix({errorCodes:r,getCodeActions:function(r){var o=r.sourceFile,s=r.span,c=r.program,l=i(o,s.start,c);if(void 0!==l){var u=e.textChanges.ChangeTracker.with(r,function(e){return a(e,o,l)});return[n.createCodeFixAction(t,u,e.Diagnostics.Convert_const_to_let,t,e.Diagnostics.Convert_const_to_let)]}},fixIds:[t]})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="fixExpectedComma",r=e.Diagnostics._0_expected.code,i=[r];function a(n,t,r){var i=e.getTokenAtPosition(n,t);return 26===i.kind&&i.parent&&(e.isObjectLiteralExpression(i.parent)||e.isArrayLiteralExpression(i.parent))?{node:i}:void 0}function o(n,t,r){var i=r.node,a=e.factory.createToken(27);n.replaceNode(t,i,a)}n.registerCodeFix({errorCodes:i,getCodeActions:function(r){var i=r.sourceFile,s=a(i,r.span.start,r.errorCode);if(s){var c=e.textChanges.ChangeTracker.with(r,function(e){return o(e,i,s)});return[n.createCodeFixAction(t,c,[e.Diagnostics.Change_0_to_1,";",","],t,[e.Diagnostics.Change_0_to_1,";",","])]}},fixIds:[t],getAllCodeActions:function(e){return n.codeFixAll(e,i,function(n,t){var r=a(t.file,t.start,t.code);r&&o(n,e.sourceFile,r)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="addVoidToPromise",r="addVoidToPromise",i=[e.Diagnostics.Expected_0_arguments_but_got_1_Did_you_forget_to_include_void_in_your_type_argument_to_Promise.code];function a(n,t,r,i,a){var s=e.getTokenAtPosition(t,r.start);if(e.isIdentifier(s)&&e.isCallExpression(s.parent)&&s.parent.expression===s&&0===s.parent.arguments.length){var c=i.getTypeChecker(),l=c.getSymbolAtLocation(s),u=null===l||void 0===l?void 0:l.valueDeclaration;if(u&&e.isParameter(u)&&e.isNewExpression(u.parent.parent)&&(null===a||void 0===a||!a.has(u))){null===a||void 0===a||a.add(u);var d=o(u.parent.parent);if(e.some(d)){var p=d[0],f=!e.isUnionTypeNode(p)&&!e.isParenthesizedTypeNode(p)&&e.isParenthesizedTypeNode(e.factory.createUnionTypeNode([p,e.factory.createKeywordTypeNode(114)]).types[0]);f&&n.insertText(t,p.pos,"("),n.insertText(t,p.end,f?") | void":" | void")}else{var m=c.getResolvedSignature(s.parent),_=null===m||void 0===m?void 0:m.parameters[0],g=_&&c.getTypeOfSymbolAtLocation(_,u.parent.parent);e.isInJSFile(u)?(!g||3&g.flags)&&(n.insertText(t,u.parent.parent.end,")"),n.insertText(t,e.skipTrivia(t.text,u.parent.parent.pos),"/** @type {Promise<void>} */(")):(!g||2&g.flags)&&n.insertText(t,u.parent.parent.expression.end,"<void>")}}}}function o(n){var t;if(!e.isInJSFile(n))return n.typeArguments;if(e.isParenthesizedExpression(n.parent)){var r=null===(t=e.getJSDocTypeTag(n.parent))||void 0===t?void 0:t.typeExpression.type;if(r&&e.isTypeReferenceNode(r)&&e.isIdentifier(r.typeName)&&"Promise"===e.idText(r.typeName))return r.typeArguments}}n.registerCodeFix({errorCodes:i,fixIds:[r],getCodeActions:function(i){var o=e.textChanges.ChangeTracker.with(i,function(e){return a(e,i.sourceFile,i.span,i.program)});if(o.length>0)return[n.createCodeFixAction(t,o,e.Diagnostics.Add_void_to_Promise_resolved_without_a_value,r,e.Diagnostics.Add_void_to_all_Promises_resolved_without_a_value)]},getAllCodeActions:function(t){return n.codeFixAll(t,i,function(n,r){return a(n,r.file,r,t.program,new e.Set)})}})})(e.codefix||(e.codefix={}))}(d||(d={})),function(e){(function(n){var t="Convert export",r={name:"Convert default export to named export",description:e.Diagnostics.Convert_default_export_to_named_export.message,kind:"refactor.rewrite.export.named"},i={name:"Convert named export to default export",description:e.Diagnostics.Convert_named_export_to_default_export.message,kind:"refactor.rewrite.export.default"};function a(n,t){void 0===t&&(t=!0);var r=n.file,i=n.program,a=e.getRefactorContextSpan(n),o=e.getTokenAtPosition(r,a.start),s=o.parent&&1&e.getSyntacticModifierFlags(o.parent)&&t?o.parent:e.getParentNodeInSpan(o,r,a);if(!s||!e.isSourceFile(s.parent)&&(!e.isModuleBlock(s.parent)||!e.isAmbientModule(s.parent.parent)))return{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_export_statement)};var c=e.isSourceFile(s.parent)?s.parent.symbol:s.parent.parent.symbol,l=e.getSyntacticModifierFlags(s)||(e.isExportAssignment(s)&&!s.isExportEquals?513:0),u=!!(512&l);if(!(1&l)||!u&&c.exports.has("default"))return{error:e.getLocaleSpecificMessage(e.Diagnostics.This_file_already_has_a_default_export)};var d=i.getTypeChecker(),p=function(n){return e.isIdentifier(n)&&d.getSymbolAtLocation(n)?void 0:{error:e.getLocaleSpecificMessage(e.Diagnostics.Can_only_convert_named_export)}};switch(s.kind){case 254:case 255:case 256:case 258:case 257:case 259:var f=s;if(!f.name)return;return p(f.name)||{exportNode:f,exportName:f.name,wasDefault:u,exportingModuleSymbol:c};case 235:var m=s;if(!(2&m.declarationList.flags)||1!==m.declarationList.declarations.length)return;var _=e.first(m.declarationList.declarations);if(!_.initializer)return;return e.Debug.assert(!u,"Can't have a default flag here"),p(_.name)||{exportNode:m,exportName:_.name,wasDefault:u,exportingModuleSymbol:c};case 269:f=s;if(f.isExportEquals)return;return p(f.expression)||{exportNode:f,exportName:f.expression,wasDefault:u,exportingModuleSymbol:c};default:return}}function s(e,n,t,r,i){c(e,t,r,n.getTypeChecker()),l(n,t,r,i)}function c(n,t,r,i){var a=t.wasDefault,o=t.exportNode,s=t.exportName;if(a)if(e.isExportAssignment(o)&&!o.isExportEquals){var c=o.expression,l=f(c.text,c.text);r.replaceNode(n,o,e.factory.createExportDeclaration(void 0,void 0,!1,e.factory.createNamedExports([l])))}else r.delete(n,e.Debug.checkDefined(e.findModifier(o,88),"Should find a default keyword in modifier list"));else{var u=e.Debug.checkDefined(e.findModifier(o,93),"Should find an export keyword in modifier list");switch(o.kind){case 254:case 255:case 256:r.insertNodeAfter(n,u,e.factory.createToken(88));break;case 235:var d=e.first(o.declarationList.declarations);if(!e.FindAllReferences.Core.isSymbolReferencedInFile(s,i,n)&&!d.type){r.replaceNode(n,o,e.factory.createExportDefault(e.Debug.checkDefined(d.initializer,"Initializer was previously known to be present")));break}case 258:case 257:case 259:r.deleteModifier(n,u),r.insertNodeAfter(n,o,e.factory.createExportDefault(e.factory.createIdentifier(s.text)));break;default:e.Debug.fail("Unexpected exportNode kind "+o.kind)}}}function l(n,t,r,i){var a=t.wasDefault,o=t.exportName,s=t.exportingModuleSymbol,c=n.getTypeChecker(),l=e.Debug.checkDefined(c.getSymbolAtLocation(o),"Export name should resolve to a symbol");e.FindAllReferences.Core.eachExportReference(n.getSourceFiles(),c,i,l,s,o.text,a,function(e){var n=e.getSourceFile();a?u(n,e,r,o.text):d(n,e,r)})}function u(n,t,r,i){var a=t.parent;switch(a.kind){case 204:r.replaceNode(n,t,e.factory.createIdentifier(i));break;case 268:case 273:var o=a;r.replaceNode(n,o,p(i,o.name.text));break;case 265:var s=a;e.Debug.assert(s.name===t,"Import clause name should match provided ref");o=p(i,t.text);var c=s.namedBindings;if(c)if(266===c.kind){r.deleteRange(n,{pos:t.getStart(n),end:c.getStart(n)});var l=e.isStringLiteral(s.parent.moduleSpecifier)?e.quotePreferenceFromString(s.parent.moduleSpecifier,n):1,u=e.makeImport(void 0,[p(i,t.text)],s.parent.moduleSpecifier,l);r.insertNodeAfter(n,s.parent,u)}else r.delete(n,t),r.insertNodeAtEndOfList(n,c.elements,o);else r.replaceNode(n,t,e.factory.createNamedImports([o]));break;default:e.Debug.failBadSyntaxKind(a)}}function d(n,t,r){var i=t.parent;switch(i.kind){case 204:r.replaceNode(n,t,e.factory.createIdentifier("default"));break;case 268:var a=e.factory.createIdentifier(i.name.text);1===i.parent.elements.length?r.replaceNode(n,i.parent,a):(r.delete(n,i),r.insertNodeBefore(n,i.parent,a));break;case 273:r.replaceNode(n,i,f("default",i.name.text));break;default:e.Debug.assertNever(i,"Unexpected parent kind "+i.kind)}}function p(n,t){return e.factory.createImportSpecifier(n===t?void 0:e.factory.createIdentifier(n),e.factory.createIdentifier(t))}function f(n,t){return e.factory.createExportSpecifier(n===t?void 0:e.factory.createIdentifier(n),e.factory.createIdentifier(t))}n.registerRefactor(t,{kinds:[r.kind,i.kind],getAvailableActions:function(s){var c=a(s,"invoked"===s.triggerReason);if(!c)return e.emptyArray;if(!n.isRefactorErrorInfo(c)){var l=c.wasDefault?r:i;return[{name:t,description:l.description,actions:[l]}]}return s.preferences.provideRefactorNotApplicableReason?[{name:t,description:e.Diagnostics.Convert_default_export_to_named_export.message,actions:[o(o({},r),{notApplicableReason:c.error}),o(o({},i),{notApplicableReason:c.error})]}]:e.emptyArray},getEditsForAction:function(t,o){e.Debug.assert(o===r.name||o===i.name,"Unexpected action name");var c=a(t);e.Debug.assert(c&&!n.isRefactorErrorInfo(c),"Expected applicable refactor info");var l=e.textChanges.ChangeTracker.with(t,function(e){return s(t.file,t.program,c,e,t.cancellationToken)});return{edits:l,renameFilename:void 0,renameLocation:void 0}}})})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){var t="Convert import",r={name:"Convert namespace import to named imports",description:e.Diagnostics.Convert_namespace_import_to_named_imports.message,kind:"refactor.rewrite.import.named"},i={name:"Convert named imports to namespace import",description:e.Diagnostics.Convert_named_imports_to_namespace_import.message,kind:"refactor.rewrite.import.namespace"};function a(n,t){void 0===t&&(t=!0);var r=n.file,i=e.getRefactorContextSpan(n),a=e.getTokenAtPosition(r,i.start),o=t?e.findAncestor(a,e.isImportDeclaration):e.getParentNodeInSpan(a,r,i);if(!o||!e.isImportDeclaration(o))return{error:"Selection is not an import declaration."};if(!(o.getEnd()<i.start+i.length)){var s=o.importClause;return s?s.namedBindings?s.namedBindings:{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_namespace_import_or_named_imports)}:{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_import_clause)}}}function s(n,t,r,i){var a=t.getTypeChecker();266===i.kind?c(n,a,r,i,e.getAllowSyntheticDefaultImports(t.getCompilerOptions())):d(n,a,r,i)}function c(n,t,r,i,a){var o=!1,s=[],c=new e.Map;e.FindAllReferences.Core.eachSymbolReferenceInFile(i.name,t,n,function(n){if(e.isPropertyAccessOrQualifiedName(n.parent)){var r=l(n.parent).text;t.resolveName(r,n,67108863,!0)&&c.set(r,!0),e.Debug.assert(u(n.parent)===n,"Parent expression should match id"),s.push(n.parent)}else o=!0});for(var d=new e.Map,f=0,m=s;f<m.length;f++){var _=m[f],g=l(_).text,y=d.get(g);void 0===y&&d.set(g,y=c.has(g)?e.getUniqueName(g,n):g),r.replaceNode(n,_,e.factory.createIdentifier(y))}var h=[];d.forEach(function(n,t){h.push(e.factory.createImportSpecifier(n===t?void 0:e.factory.createIdentifier(t),e.factory.createIdentifier(n)))});var v=i.parent.parent;o&&!a?r.insertNodeAfter(n,v,p(v,void 0,h)):r.replaceNode(n,v,p(v,o?e.factory.createIdentifier(i.name.text):void 0,h))}function l(n){return e.isPropertyAccessExpression(n)?n.name:n.right}function u(n){return e.isPropertyAccessExpression(n)?n.expression:n.left}function d(n,t,r,i){var a=i.parent.parent,o=a.moduleSpecifier,s=new e.Set;i.elements.forEach(function(e){var n=t.getSymbolAtLocation(e.name);n&&s.add(n)});var c=o&&e.isStringLiteral(o)?e.codefix.moduleSpecifierToValidIdentifier(o.text,99):"module";function l(r){return!!e.FindAllReferences.Core.eachSymbolReferenceInFile(r.name,t,n,function(n){var r=t.resolveName(c,n,67108863,!0);return!!r&&(!s.has(r)||e.isExportSpecifier(n.parent))})}for(var u=i.elements.some(l),d=u?e.getUniqueName(c,n):c,f=new e.Set,m=function(i){var a=(i.propertyName||i.name).text;e.FindAllReferences.Core.eachSymbolReferenceInFile(i.name,t,n,function(t){var o=e.factory.createPropertyAccessExpression(e.factory.createIdentifier(d),a);e.isShorthandPropertyAssignment(t.parent)?r.replaceNode(n,t.parent,e.factory.createPropertyAssignment(t.text,o)):e.isExportSpecifier(t.parent)?f.add(i):r.replaceNode(n,t,o)})},_=0,g=i.elements;_<g.length;_++){var y=g[_];m(y)}if(r.replaceNode(n,i,e.factory.createNamespaceImport(e.factory.createIdentifier(d))),f.size){var h=e.arrayFrom(f.values()).map(function(n){return e.factory.createImportSpecifier(n.propertyName&&e.factory.createIdentifier(n.propertyName.text),e.factory.createIdentifier(n.name.text))});r.insertNodeAfter(n,i.parent.parent,p(a,void 0,h))}}function p(n,t,r){return e.factory.createImportDeclaration(void 0,void 0,e.factory.createImportClause(!1,t,r&&r.length?e.factory.createNamedImports(r):void 0),n.moduleSpecifier)}n.registerRefactor(t,{kinds:[r.kind,i.kind],getAvailableActions:function(s){var c=a(s,"invoked"===s.triggerReason);if(!c)return e.emptyArray;if(!n.isRefactorErrorInfo(c)){var l=266===c.kind,u=l?r:i;return[{name:t,description:u.description,actions:[u]}]}return s.preferences.provideRefactorNotApplicableReason?[{name:t,description:r.description,actions:[o(o({},r),{notApplicableReason:c.error})]},{name:t,description:i.description,actions:[o(o({},i),{notApplicableReason:c.error})]}]:e.emptyArray},getEditsForAction:function(t,o){e.Debug.assert(o===r.name||o===i.name,"Unexpected action name");var c=a(t);e.Debug.assert(c&&!n.isRefactorErrorInfo(c),"Expected applicable refactor info");var l=e.textChanges.ChangeTracker.with(t,function(e){return s(t.file,t.program,e,c)});return{edits:l,renameFilename:void 0,renameLocation:void 0}}})})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){(function(t){var r="Convert to optional chain expression",i=e.getLocaleSpecificMessage(e.Diagnostics.Convert_to_optional_chain_expression),a={name:r,description:i,kind:"refactor.rewrite.expression.optionalChain"};function s(t){var s=p(t,"invoked"===t.triggerReason);return s?n.isRefactorErrorInfo(s)?t.preferences.provideRefactorNotApplicableReason?[{name:r,description:i,actions:[o(o({},a),{notApplicableReason:s.error})]}]:e.emptyArray:[{name:r,description:i,actions:[a]}]:e.emptyArray}function c(t,r){var i=p(t);e.Debug.assert(i&&!n.isRefactorErrorInfo(i),"Expected applicable refactor info");var a=e.textChanges.ChangeTracker.with(t,function(e){return x(t.file,t.program.getTypeChecker(),e,i,r)});return{edits:a,renameFilename:void 0,renameLocation:void 0}}function l(n){return e.isBinaryExpression(n)||e.isConditionalExpression(n)}function u(n){return e.isExpressionStatement(n)||e.isReturnStatement(n)||e.isVariableStatement(n)}function d(e){return l(e)||u(e)}function p(n,t){void 0===t&&(t=!0);var r=n.file,i=n.program,a=e.getRefactorContextSpan(n),o=0===a.length;if(!o||t){var s=e.getTokenAtPosition(r,a.start),c=e.findTokenOnLeftOfPosition(r,a.start+a.length),l=e.createTextSpanFromBounds(s.pos,c&&c.end>=s.pos?c.getEnd():s.getEnd()),u=o?b(s):v(s,l),p=u&&d(u)?E(u):void 0;if(!p)return{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_convertible_access_expression)};var _=i.getTypeChecker();return e.isConditionalExpression(p)?f(p,_):m(p)}}function f(n,t){var r=n.condition,i=T(n.whenTrue);if(!i||t.isNullableType(t.getTypeAtLocation(i)))return{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_convertible_access_expression)};if((e.isPropertyAccessExpression(r)||e.isIdentifier(r))&&g(r,i.expression))return{finalExpression:i,occurrences:[r],expression:n};if(e.isBinaryExpression(r)){var a=_(i.expression,r);return a?{finalExpression:i,occurrences:a,expression:n}:{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_matching_access_expressions)}}}function m(n){if(55!==n.operatorToken.kind)return{error:e.getLocaleSpecificMessage(e.Diagnostics.Can_only_convert_logical_AND_access_chains)};var t=T(n.right);if(!t)return{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_convertible_access_expression)};var r=_(t.expression,n.left);return r?{finalExpression:t,occurrences:r,expression:n}:{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_matching_access_expressions)}}function _(n,t){var r=[];while(e.isBinaryExpression(t)&&55===t.operatorToken.kind){var i=g(e.skipParentheses(n),e.skipParentheses(t.right));if(!i)break;r.push(i),n=i,t=t.left}var a=g(n,t);return a&&r.push(a),r.length>0?r:void 0}function g(n,t){if(e.isIdentifier(t)||e.isPropertyAccessExpression(t)||e.isElementAccessExpression(t))return y(n,t)?t:void 0}function y(n,t){while(e.isCallExpression(n)||e.isPropertyAccessExpression(n)||e.isElementAccessExpression(n)){if(h(n)===h(t))break;n=n.expression}while(e.isPropertyAccessExpression(n)&&e.isPropertyAccessExpression(t)||e.isElementAccessExpression(n)&&e.isElementAccessExpression(t)){if(h(n)!==h(t))return!1;n=n.expression,t=t.expression}return e.isIdentifier(n)&&e.isIdentifier(t)&&n.getText()===t.getText()}function h(n){return e.isIdentifier(n)||e.isStringOrNumericLiteralLike(n)?n.getText():e.isPropertyAccessExpression(n)?h(n.name):e.isElementAccessExpression(n)?h(n.argumentExpression):void 0}function v(e,n){while(e.parent){if(d(e)&&0!==n.length&&e.end>=n.start+n.length)return e;e=e.parent}}function b(e){while(e.parent){if(d(e)&&!d(e.parent))return e;e=e.parent}}function E(n){if(l(n))return n;if(e.isVariableStatement(n)){var t=e.getSingleVariableOfVariableStatement(n),r=null===t||void 0===t?void 0:t.initializer;return r&&l(r)?r:void 0}return n.expression&&l(n.expression)?n.expression:void 0}function T(n){return n=e.skipParentheses(n),e.isBinaryExpression(n)?T(n.left):(e.isPropertyAccessExpression(n)||e.isElementAccessExpression(n)||e.isCallExpression(n))&&!e.isOptionalChain(n)?n:void 0}function S(n,t,r){if(e.isPropertyAccessExpression(t)||e.isElementAccessExpression(t)||e.isCallExpression(t)){var i=S(n,t.expression,r),a=r.length>0?r[r.length-1]:void 0,o=(null===a||void 0===a?void 0:a.getText())===t.expression.getText();if(o&&r.pop(),e.isCallExpression(t))return o?e.factory.createCallChain(i,e.factory.createToken(28),t.typeArguments,t.arguments):e.factory.createCallChain(i,t.questionDotToken,t.typeArguments,t.arguments);if(e.isPropertyAccessExpression(t))return o?e.factory.createPropertyAccessChain(i,e.factory.createToken(28),t.name):e.factory.createPropertyAccessChain(i,t.questionDotToken,t.name);if(e.isElementAccessExpression(t))return o?e.factory.createElementAccessChain(i,e.factory.createToken(28),t.argumentExpression):e.factory.createElementAccessChain(i,t.questionDotToken,t.argumentExpression)}return t}function x(n,t,r,i,a){var o=i.finalExpression,s=i.occurrences,c=i.expression,l=s[s.length-1],u=S(t,o,s);u&&(e.isPropertyAccessExpression(u)||e.isElementAccessExpression(u)||e.isCallExpression(u))&&(e.isBinaryExpression(c)?r.replaceNodeRange(n,l,o,u):e.isConditionalExpression(c)&&r.replaceNode(n,c,e.factory.createBinaryExpression(u,e.factory.createToken(60),c.whenFalse)))}n.registerRefactor(r,{kinds:[a.kind],getAvailableActions:s,getEditsForAction:c})})(n.convertToOptionalChainExpression||(n.convertToOptionalChainExpression={}))})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){(function(t){var r="Convert overload list to single signature",i=e.Diagnostics.Convert_overload_list_to_single_signature.message,a={name:r,description:i,kind:"refactor.rewrite.function.overloadList"};function o(n){var t=n.file,o=n.startPosition,s=n.program,c=l(t,o,s);return c?[{name:r,description:i,actions:[a]}]:e.emptyArray}function s(n){var t=n.file,r=n.startPosition,i=n.program,a=l(t,r,i);if(a){var o=i.getTypeChecker(),s=a[a.length-1],c=s;switch(s.kind){case 166:c=e.factory.updateMethodSignature(s,s.modifiers,s.name,s.questionToken,s.typeParameters,d(a),s.type);break;case 167:c=e.factory.updateMethodDeclaration(s,s.decorators,s.modifiers,s.asteriskToken,s.name,s.questionToken,s.typeParameters,d(a),s.type,s.body);break;case 172:c=e.factory.updateCallSignature(s,s.typeParameters,d(a),s.type);break;case 169:c=e.factory.updateConstructorDeclaration(s,s.decorators,s.modifiers,d(a),s.body);break;case 173:c=e.factory.updateConstructSignature(s,s.typeParameters,d(a),s.type);break;case 254:c=e.factory.updateFunctionDeclaration(s,s.decorators,s.modifiers,s.asteriskToken,s.name,s.typeParameters,d(a),s.type,s.body);break;default:return e.Debug.failBadSyntaxKind(s,"Unhandled signature kind in overload list conversion refactoring")}if(c!==s){var u=e.textChanges.ChangeTracker.with(n,function(e){e.replaceNodeRange(t,a[0],a[a.length-1],c)});return{renameFilename:void 0,renameLocation:void 0,edits:u}}}function d(n){var t=n[n.length-1];return e.isFunctionLikeDeclaration(t)&&t.body&&(n=n.slice(0,n.length-1)),e.factory.createNodeArray([e.factory.createParameterDeclaration(void 0,void 0,e.factory.createToken(25),"args",void 0,e.factory.createUnionTypeNode(e.map(n,p)))])}function p(n){var t=e.map(n.parameters,f);return e.setEmitFlags(e.factory.createTupleTypeNode(t),e.some(t,function(n){return!!e.length(e.getSyntheticLeadingComments(n))})?0:1)}function f(n){e.Debug.assert(e.isIdentifier(n.name));var t=e.setTextRange(e.factory.createNamedTupleMember(n.dotDotDotToken,n.name,n.questionToken,n.type||e.factory.createKeywordTypeNode(129)),n),r=n.symbol&&n.symbol.getDocumentationComment(o);if(r){var i=e.displayPartsToString(r);i.length&&e.setSyntheticLeadingComments(t,[{text:"*\n"+i.split("\n").map(function(e){return" * "+e}).join("\n")+"\n ",kind:3,pos:-1,end:-1,hasTrailingNewLine:!0,hasLeadingNewline:!0}])}return t}}function c(e){switch(e.kind){case 166:case 167:case 172:case 169:case 173:case 254:return!0}return!1}function l(n,t,r){var i=e.getTokenAtPosition(n,t),a=e.findAncestor(i,c);if(a){var o=r.getTypeChecker(),s=a.symbol;if(s){var l=s.declarations;if(!(e.length(l)<=1)&&e.every(l,function(t){return e.getSourceFileOfNode(t)===n})&&c(l[0])){var u=l[0].kind;if(e.every(l,function(e){return e.kind===u})){var d=l;if(!e.some(d,function(n){return!!n.typeParameters||e.some(n.parameters,function(n){return!!n.decorators||!!n.modifiers||!e.isIdentifier(n.name)})})){var p=e.mapDefined(d,function(e){return o.getSignatureFromDeclaration(e)});if(e.length(p)===e.length(l)){var f=o.getReturnTypeOfSignature(p[0]);if(e.every(p,function(e){return o.getReturnTypeOfSignature(e)===f}))return d}}}}}}}n.registerRefactor(r,{kinds:[a.kind],getEditsForAction:s,getAvailableActions:o})})(n.addOrRemoveBracesToArrowFunction||(n.addOrRemoveBracesToArrowFunction={}))})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){(function(t){var r,i,a,s,c="Extract Symbol",l={name:"Extract Constant",description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_constant),kind:"refactor.extract.constant"},u={name:"Extract Function",description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_function),kind:"refactor.extract.function"};function d(t){var r=t.kind,i=f(t.file,e.getRefactorContextSpan(t),"invoked"===t.triggerReason),a=i.targetRange;if(void 0===a){if(!i.errors||0===i.errors.length||!t.preferences.provideRefactorNotApplicableReason)return e.emptyArray;var s=[];return n.refactorKindBeginsWith(u.kind,r)&&s.push({name:c,description:u.description,actions:[o(o({},u),{notApplicableReason:I(i.errors)})]}),n.refactorKindBeginsWith(l.kind,r)&&s.push({name:c,description:l.description,actions:[o(o({},l),{notApplicableReason:I(i.errors)})]}),s}var d=b(a,t);if(void 0===d)return e.emptyArray;for(var p,m,_=[],g=new e.Map,y=[],h=new e.Map,v=0,E=0,T=d;E<T.length;E++){var S=T[E],x=S.functionExtraction,L=S.constantExtraction,D=x.description;if(n.refactorKindBeginsWith(u.kind,r)&&(0===x.errors.length?g.has(D)||(g.set(D,!0),_.push({description:D,name:"function_scope_"+v,kind:u.kind})):p||(p={description:D,name:"function_scope_"+v,notApplicableReason:I(x.errors),kind:u.kind})),n.refactorKindBeginsWith(l.kind,r))if(0===L.errors.length){var A=L.description;h.has(A)||(h.set(A,!0),y.push({description:A,name:"constant_scope_"+v,kind:l.kind}))}else m||(m={description:D,name:"constant_scope_"+v,notApplicableReason:I(L.errors),kind:l.kind});v++}var C=[];return _.length?C.push({name:c,description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_function),actions:_}):t.preferences.provideRefactorNotApplicableReason&&p&&C.push({name:c,description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_function),actions:[p]}),y.length?C.push({name:c,description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_constant),actions:y}):t.preferences.provideRefactorNotApplicableReason&&m&&C.push({name:c,description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_constant),actions:[m]}),C.length?C:e.emptyArray;function I(e){var n=e[0].messageText;return"string"!==typeof n&&(n=n.messageText),n}}function p(n,t){var r=f(n.file,e.getRefactorContextSpan(n)),i=r.targetRange,a=/^function_scope_(\d+)$/.exec(t);if(a){var o=+a[1];return e.Debug.assert(isFinite(o),"Expected to parse a finite number from the function scope index"),h(i,n,o)}var s=/^constant_scope_(\d+)$/.exec(t);if(s){o=+s[1];return e.Debug.assert(isFinite(o),"Expected to parse a finite number from the constant scope index"),v(i,n,o)}e.Debug.fail("Unrecognized action name")}function f(n,t,a){void 0===a&&(a=!0);var o=t.length;if(0===o&&!a)return{errors:[e.createFileDiagnostic(n,t.start,o,r.cannotExtractEmpty)]};var s=0===o&&a,c=e.findFirstNonJsxWhitespaceToken(n,t.start),l=e.findTokenOnLeftOfPosition(n,e.textSpanEnd(t)),u=c&&l&&a?m(c,l,n):t,d=s?K(c):e.getParentNodeInSpan(c,n,u),p=s?d:e.getParentNodeInSpan(l,n,u),f=[],g=i.None;if(!d||!p)return{errors:[e.createFileDiagnostic(n,t.start,o,r.cannotExtractRange)]};if(e.isJSDoc(d))return{errors:[e.createFileDiagnostic(n,t.start,o,r.cannotExtractJSDoc)]};if(d.parent!==p.parent)return{errors:[e.createFileDiagnostic(n,t.start,o,r.cannotExtractRange)]};if(d!==p){if(!W(d.parent))return{errors:[e.createFileDiagnostic(n,t.start,o,r.cannotExtractRange)]};for(var y=[],h=d,v=0,b=h.parent.statements;v<b.length;v++){var E=b[v];if(E===d||y.length){var T=C(E);if(T)return{errors:T};y.push(E)}if(E===p)break}return y.length?{targetRange:{range:y,facts:g,declarations:f}}:{errors:[e.createFileDiagnostic(n,t.start,o,r.cannotExtractRange)]}}if(e.isReturnStatement(d)&&!d.expression)return{errors:[e.createFileDiagnostic(n,t.start,o,r.cannotExtractRange)]};var S=L(d),x=D(S)||C(S);return x?{errors:x}:{targetRange:{range:_(S),facts:g,declarations:f}};function L(n){if(e.isReturnStatement(n)){if(n.expression)return n.expression}else if(e.isVariableStatement(n)){for(var t=0,r=void 0,i=0,a=n.declarationList.declarations;i<a.length;i++){var o=a[i];o.initializer&&(t++,r=o.initializer)}if(1===t)return r}else if(e.isVariableDeclaration(n)&&n.initializer)return n.initializer;return n}function D(n){if(e.isIdentifier(e.isExpressionStatement(n)?n.expression:n))return[e.createDiagnosticForNode(n,r.cannotExtractIdentifier)]}function A(n,t){var r=n;while(r!==t){if(165===r.kind){e.isStatic(r)&&(g|=i.InStaticRegion);break}if(162===r.kind){var a=e.getContainingFunction(r);169===a.kind&&(g|=i.InStaticRegion);break}167===r.kind&&e.isStatic(r)&&(g|=i.InStaticRegion),r=r.parent}}function C(n){var a;if(function(e){e[e["None"]=0]="None",e[e["Break"]=1]="Break",e[e["Continue"]=2]="Continue",e[e["Return"]=4]="Return"}(a||(a={})),e.Debug.assert(n.pos<=n.end,"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809 (1)"),e.Debug.assert(!e.positionIsSynthesized(n.pos),"This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809 (2)"),!e.isStatement(n)&&(!e.isExpressionNode(n)||!H(n)))return[e.createDiagnosticForNode(n,r.statementOrExpressionExpected)];if(8388608&n.flags)return[e.createDiagnosticForNode(n,r.cannotExtractAmbientBlock)];var o,s=e.getContainingClass(n);s&&A(n,s);var c,l=4;return u(n),o;function u(n){if(o)return!0;if(e.isDeclaration(n)){var a=252===n.kind?n.parent.parent:n;if(e.hasSyntacticModifier(a,1))return(o||(o=[])).push(e.createDiagnosticForNode(n,r.cannotExtractExportedEntity)),!0;f.push(n.symbol)}switch(n.kind){case 264:return(o||(o=[])).push(e.createDiagnosticForNode(n,r.cannotExtractImport)),!0;case 269:return(o||(o=[])).push(e.createDiagnosticForNode(n,r.cannotExtractExportedEntity)),!0;case 106:if(206===n.parent.kind){var s=e.getContainingClass(n);if(s.pos<t.start||s.end>=t.start+t.length)return(o||(o=[])).push(e.createDiagnosticForNode(n,r.cannotExtractSuper)),!0}else g|=i.UsesThis;break;case 212:e.forEachChild(n,function n(t){if(e.isThis(t))g|=i.UsesThis;else{if(e.isClassLike(t)||e.isFunctionLike(t)&&!e.isArrowFunction(t))return!1;e.forEachChild(t,n)}});case 255:case 254:e.isSourceFile(n.parent)&&void 0===n.parent.externalModuleIndicator&&(o||(o=[])).push(e.createDiagnosticForNode(n,r.functionWillNotBeVisibleInTheNewScope));case 224:case 211:case 167:case 169:case 170:case 171:return!1}var d=l;switch(n.kind){case 237:l=0;break;case 250:l=0;break;case 233:n.parent&&250===n.parent.kind&&n.parent.finallyBlock===n&&(l=4);break;case 288:case 287:l|=1;break;default:e.isIterationStatement(n,!1)&&(l|=3);break}switch(n.kind){case 190:case 108:g|=i.UsesThis;break;case 248:var p=n.label;(c||(c=[])).push(p.escapedText),e.forEachChild(n,u),c.pop();break;case 244:case 243:p=n.label;p?e.contains(c,p.escapedText)||(o||(o=[])).push(e.createDiagnosticForNode(n,r.cannotExtractRangeContainingLabeledBreakOrContinueStatementWithTargetOutsideOfTheRange)):l&(244===n.kind?1:2)||(o||(o=[])).push(e.createDiagnosticForNode(n,r.cannotExtractRangeContainingConditionalBreakOrContinueStatements));break;case 216:g|=i.IsAsyncFunction;break;case 222:g|=i.IsGenerator;break;case 245:4&l?g|=i.HasReturn:(o||(o=[])).push(e.createDiagnosticForNode(n,r.cannotExtractRangeContainingConditionalReturnStatement));break;default:e.forEachChild(n,u);break}l=d}}}function m(e,n,t){var r=e.getStart(t),i=n.getEnd();return 59===t.text.charCodeAt(i)&&i++,{start:r,length:i-r}}function _(n){return e.isStatement(n)?[n]:e.isExpressionNode(n)?e.isExpressionStatement(n.parent)?[n.parent]:n:void 0}function g(n){return e.isFunctionLikeDeclaration(n)||e.isSourceFile(n)||e.isModuleBlock(n)||e.isClassLike(n)}function y(n){var t=U(n.range)?e.first(n.range):n.range;if(n.facts&i.UsesThis){var r=e.getContainingClass(t);if(r){var a=e.findAncestor(t,e.isFunctionLikeDeclaration);return a?[a,r]:[r]}}var o=[];while(1)if(t=t.parent,162===t.kind&&(t=e.findAncestor(t,function(n){return e.isFunctionLikeDeclaration(n)}).parent),g(t)&&(o.push(t),300===t.kind))return o}function h(n,t,r){var i=E(n,t),a=i.scopes,o=i.readsAndWrites,s=o.target,c=o.usagesPerScope,l=o.functionErrorsPerScope,u=o.exposedVariableDeclarations;return e.Debug.assert(!l[r].length,"The extraction went missing? How?"),t.cancellationToken.throwIfCancellationRequested(),A(s,a[r],c[r],u,n,t)}function v(n,t,r){var i=E(n,t),a=i.scopes,o=i.readsAndWrites,s=o.target,c=o.usagesPerScope,l=o.constantErrorsPerScope,u=o.exposedVariableDeclarations;e.Debug.assert(!l[r].length,"The extraction went missing? How?"),e.Debug.assert(0===u.length,"Extract constant accepted a range containing a variable declaration?"),t.cancellationToken.throwIfCancellationRequested();var d=e.isExpression(s)?s:s.statements[0].expression;return C(d,a[r],c[r],n.facts,t)}function b(n,t){var r=E(n,t),i=r.scopes,a=r.readsAndWrites,o=a.functionErrorsPerScope,s=a.constantErrorsPerScope,c=i.map(function(n,t){var r,i,a=T(n),c=S(n),l=e.isFunctionLikeDeclaration(n)?x(n):e.isClassLike(n)?L(n):D(n);return 1===l?(r=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1_scope),[a,"global"]),i=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1_scope),[c,"global"])):0===l?(r=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1_scope),[a,"module"]),i=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1_scope),[c,"module"])):(r=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1),[a,l]),i=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_1),[c,l])),0!==t||e.isClassLike(n)||(i=e.formatStringFromArgs(e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_0_in_enclosing_scope),[c])),{functionExtraction:{description:r,errors:o[t]},constantExtraction:{description:i,errors:s[t]}}});return c}function E(e,n){var t=n.file,r=y(e),i=V(e,t),a=j(e,r,i,t,n.program.getTypeChecker(),n.cancellationToken);return{scopes:r,readsAndWrites:a}}function T(n){return e.isFunctionLikeDeclaration(n)?"inner function":e.isClassLike(n)?"method":"function"}function S(n){return e.isClassLike(n)?"readonly field":"constant"}function x(n){switch(n.kind){case 169:return"constructor";case 211:case 254:return n.name?"function '"+n.name.text+"'":e.ANONYMOUS;case 212:return"arrow function";case 167:return"method '"+n.name.getText()+"'";case 170:return"'get "+n.name.getText()+"'";case 171:return"'set "+n.name.getText()+"'";default:throw e.Debug.assertNever(n,"Unexpected scope kind "+n.kind)}}function L(e){return 255===e.kind?e.name?"class '"+e.name.text+"'":"anonymous class declaration":e.name?"class expression '"+e.name.text+"'":"anonymous class expression"}function D(e){return 260===e.kind?"namespace '"+e.parent.name.getText()+"'":e.externalModuleIndicator?0:1}function A(n,t,r,a,o,s){var c,l,u=r.usages,d=r.typeParameterUsages,p=r.substitutions,f=s.program.getTypeChecker(),m=e.getEmitScriptTarget(s.program.getCompilerOptions()),_=e.codefix.createImportAdder(s.file,s.program,s.preferences,s.host),g=t.getSourceFile(),y=e.getUniqueName(e.isClassLike(t)?"newMethod":"newFunction",g),h=e.isInJSFile(t),v=e.factory.createIdentifier(y),b=[],E=[];u.forEach(function(n,r){var i;if(!h){var a=f.getTypeOfSymbolAtLocation(n.symbol,n.node);a=f.getBaseTypeOfLiteralType(a),i=e.codefix.typeToAutoImportableTypeNode(f,_,a,t,m,1)}var o=e.factory.createParameterDeclaration(void 0,void 0,void 0,r,void 0,i);b.push(o),2===n.usage&&(l||(l=[])).push(n),E.push(e.factory.createIdentifier(r))});var T=e.arrayFrom(d.values()).map(function(e){return{type:e,declaration:N(e)}}),S=T.sort(k),x=0===S.length?void 0:S.map(function(e){return e.declaration}),L=void 0!==x?x.map(function(n){return e.factory.createTypeReferenceNode(n.name,void 0)}):void 0;if(e.isExpression(n)&&!h){var D=f.getContextualType(n);c=f.typeToTypeNode(D,t,1)}var A,C=O(n,a,l,p,!!(o.facts&i.HasReturn)),I=C.body,w=C.returnValueProperty;if(e.suppressLeadingAndTrailingTrivia(I),e.isClassLike(t)){var P=h?[]:[e.factory.createModifier(121)];o.facts&i.InStaticRegion&&P.push(e.factory.createModifier(124)),o.facts&i.IsAsyncFunction&&P.push(e.factory.createModifier(130)),A=e.factory.createMethodDeclaration(void 0,P.length?P:void 0,o.facts&i.IsGenerator?e.factory.createToken(41):void 0,v,void 0,x,b,c,I)}else A=e.factory.createFunctionDeclaration(void 0,o.facts&i.IsAsyncFunction?[e.factory.createToken(130)]:void 0,o.facts&i.IsGenerator?e.factory.createToken(41):void 0,v,x,b,c,I);var F=e.textChanges.ChangeTracker.fromContext(s),G=(U(o.range)?e.last(o.range):o.range).end,V=M(G,t);V?F.insertNodeBefore(s.file,V,A,!0):F.insertNodeAtEndOfScope(s.file,t,A),_.writeFixes(F);var j=[],K=R(t,o,y),H=e.factory.createCallExpression(K,L,E);if(o.facts&i.IsGenerator&&(H=e.factory.createYieldExpression(e.factory.createToken(41),H)),o.facts&i.IsAsyncFunction&&(H=e.factory.createAwaitExpression(H)),z(n)&&(H=e.factory.createJsxExpression(void 0,H)),a.length&&!l)if(e.Debug.assert(!w,"Expected no returnValueProperty"),e.Debug.assert(!(o.facts&i.HasReturn),"Expected RangeFacts.HasReturn flag to be unset"),1===a.length){var W=a[0];j.push(e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(e.getSynthesizedDeepClone(W.name),void 0,e.getSynthesizedDeepClone(W.type),H)],W.parent.flags)))}else{for(var J=[],q=[],X=a[0].parent.flags,Y=!1,Q=0,Z=a;Q<Z.length;Q++){W=Z[Q];J.push(e.factory.createBindingElement(void 0,void 0,e.getSynthesizedDeepClone(W.name)));var $=f.typeToTypeNode(f.getBaseTypeOfLiteralType(f.getTypeAtLocation(W)),t,1);q.push(e.factory.createPropertySignature(void 0,W.symbol.name,void 0,$)),Y=Y||void 0!==W.type,X&=W.parent.flags}var ee=Y?e.factory.createTypeLiteralNode(q):void 0;ee&&e.setEmitFlags(ee,1),j.push(e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(e.factory.createObjectBindingPattern(J),void 0,ee,H)],X)))}else if(a.length||l){if(a.length)for(var ne=0,te=a;ne<te.length;ne++){W=te[ne];var re=W.parent.flags;2&re&&(re=-3&re|1),j.push(e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(W.symbol.name,void 0,le(W.type))],re)))}w&&j.push(e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(w,void 0,le(c))],1)));var ie=B(a,l);w&&ie.unshift(e.factory.createShorthandPropertyAssignment(w)),1===ie.length?(e.Debug.assert(!w,"Shouldn't have returnValueProperty here"),j.push(e.factory.createExpressionStatement(e.factory.createAssignment(ie[0].name,H))),o.facts&i.HasReturn&&j.push(e.factory.createReturnStatement())):(j.push(e.factory.createExpressionStatement(e.factory.createAssignment(e.factory.createObjectLiteralExpression(ie),H))),w&&j.push(e.factory.createReturnStatement(e.factory.createIdentifier(w))))}else o.facts&i.HasReturn?j.push(e.factory.createReturnStatement(H)):U(o.range)?j.push(e.factory.createExpressionStatement(H)):j.push(H);U(o.range)?F.replaceNodeRangeWithNodes(s.file,e.first(o.range),e.last(o.range),j):F.replaceNodeWithNodes(s.file,o.range,j);var ae=F.getChanges(),oe=U(o.range)?e.first(o.range):o.range,se=oe.getSourceFile().fileName,ce=e.getRenameLocation(ae,se,y,!1);return{renameFilename:se,renameLocation:ce,edits:ae};function le(n){if(void 0!==n){var t=e.getSynthesizedDeepClone(n),r=t;while(e.isParenthesizedTypeNode(r))r=r.type;return e.isUnionTypeNode(r)&&e.find(r.types,function(e){return 151===e.kind})?t:e.factory.createUnionTypeNode([t,e.factory.createKeywordTypeNode(151)])}}}function C(n,t,r,a,o){var s,c=r.substitutions,l=o.program.getTypeChecker(),u=t.getSourceFile(),d=e.getUniqueName(e.isClassLike(t)?"newProperty":"newLocal",u),p=e.isInJSFile(t),f=p||!l.isContextSensitive(n)?void 0:l.typeToTypeNode(l.getContextualType(n),t,1),m=w(n,c);s=C(f,m),f=s.variableType,m=s.initializer,e.suppressLeadingAndTrailingTrivia(m);var _=e.textChanges.ChangeTracker.fromContext(o);if(e.isClassLike(t)){e.Debug.assert(!p,"Cannot extract to a JS class");var y=[];y.push(e.factory.createModifier(121)),a&i.InStaticRegion&&y.push(e.factory.createModifier(124)),y.push(e.factory.createModifier(143));var h=e.factory.createPropertyDeclaration(void 0,y,d,void 0,f,m),v=e.factory.createPropertyAccessExpression(a&i.InStaticRegion?e.factory.createIdentifier(t.name.getText()):e.factory.createThis(),e.factory.createIdentifier(d));z(n)&&(v=e.factory.createJsxExpression(void 0,v));var b=n.pos,E=F(b,t);_.insertNodeBefore(o.file,E,h,!0),_.replaceNode(o.file,n,v)}else{var T=e.factory.createVariableDeclaration(d,void 0,f,m),S=I(n,t);if(S){_.insertNodeBefore(o.file,S,T);v=e.factory.createIdentifier(d);_.replaceNode(o.file,n,v)}else if(236===n.parent.kind&&t===e.findAncestor(n,g)){var x=e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([T],2));_.replaceNode(o.file,n.parent,x)}else{x=e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([T],2)),E=G(n,t);if(0===E.pos?_.insertNodeAtTopOfFile(o.file,x,!1):_.insertNodeBefore(o.file,E,x,!1),236===n.parent.kind)_.delete(o.file,n.parent);else{v=e.factory.createIdentifier(d);z(n)&&(v=e.factory.createJsxExpression(void 0,v)),_.replaceNode(o.file,n,v)}}}var L=_.getChanges(),D=n.getSourceFile().fileName,A=e.getRenameLocation(L,D,d,!0);return{renameFilename:D,renameLocation:A,edits:L};function C(r,i){if(void 0===r)return{variableType:r,initializer:i};if(!e.isFunctionExpression(i)&&!e.isArrowFunction(i)||i.typeParameters)return{variableType:r,initializer:i};var a=l.getTypeAtLocation(n),o=e.singleOrUndefined(l.getSignaturesOfType(a,0));if(!o)return{variableType:r,initializer:i};if(o.getTypeParameters())return{variableType:r,initializer:i};for(var s=[],c=!1,u=0,d=i.parameters;u<d.length;u++){var p=d[u];if(p.type)s.push(p);else{var f=l.getTypeAtLocation(p);f===l.getAnyType()&&(c=!0),s.push(e.factory.updateParameterDeclaration(p,p.decorators,p.modifiers,p.dotDotDotToken,p.name,p.questionToken,p.type||l.typeToTypeNode(f,t,1),p.initializer))}}if(c)return{variableType:r,initializer:i};if(r=void 0,e.isArrowFunction(i))i=e.factory.updateArrowFunction(i,n.modifiers,i.typeParameters,s,i.type||l.typeToTypeNode(o.getReturnType(),t,1),i.equalsGreaterThanToken,i.body);else{if(o&&o.thisParameter){var m=e.firstOrUndefined(s);if(!m||e.isIdentifier(m.name)&&"this"!==m.name.escapedText){var _=l.getTypeOfSymbolAtLocation(o.thisParameter,n);s.splice(0,0,e.factory.createParameterDeclaration(void 0,void 0,void 0,"this",void 0,l.typeToTypeNode(_,t,1)))}}i=e.factory.updateFunctionExpression(i,n.modifiers,i.asteriskToken,i.name,i.typeParameters,s,i.type||l.typeToTypeNode(o.getReturnType(),t,1),i.body)}return{variableType:r,initializer:i}}}function I(n,t){var r;while(void 0!==n&&n!==t){if(e.isVariableDeclaration(n)&&n.initializer===r&&e.isVariableDeclarationList(n.parent)&&n.parent.declarations.length>1)return n;r=n,n=n.parent}}function N(e){var n,t=e.symbol;if(t&&t.declarations)for(var r=0,i=t.declarations;r<i.length;r++){var a=i[r];(void 0===n||a.pos<n.pos)&&(n=a)}return n}function k(n,t){var r=n.type,i=n.declaration,a=t.type,o=t.declaration;return e.compareProperties(i,o,"pos",e.compareValues)||e.compareStringsCaseSensitive(r.symbol?r.symbol.getName():"",a.symbol?a.symbol.getName():"")||e.compareValues(r.id,a.id)}function R(n,t,r){var a=e.factory.createIdentifier(r);if(e.isClassLike(n)){var o=t.facts&i.InStaticRegion?e.factory.createIdentifier(n.name.text):e.factory.createThis();return e.factory.createPropertyAccessExpression(o,a)}return a}function O(n,t,r,i,a){var o,s=void 0!==r||t.length>0;if(e.isBlock(n)&&!s&&0===i.size)return{body:e.factory.createBlock(n.statements,!0),returnValueProperty:void 0};var c=!1,l=e.factory.createNodeArray(e.isBlock(n)?n.statements.slice(0):[e.isStatement(n)?n:e.factory.createReturnStatement(n)]);if(s||i.size){var u=e.visitNodes(l,p).slice();if(s&&!a&&e.isStatement(n)){var d=B(t,r);1===d.length?u.push(e.factory.createReturnStatement(d[0].name)):u.push(e.factory.createReturnStatement(e.factory.createObjectLiteralExpression(d)))}return{body:e.factory.createBlock(u,!0),returnValueProperty:o}}return{body:e.factory.createBlock(l,!0),returnValueProperty:void 0};function p(n){if(!c&&e.isReturnStatement(n)&&s){var a=B(t,r);return n.expression&&(o||(o="__return"),a.unshift(e.factory.createPropertyAssignment(o,e.visitNode(n.expression,p)))),1===a.length?e.factory.createReturnStatement(a[0].name):e.factory.createReturnStatement(e.factory.createObjectLiteralExpression(a))}var l=c;c=c||e.isFunctionLikeDeclaration(n)||e.isClassLike(n);var u=i.get(e.getNodeId(n).toString()),d=u?e.getSynthesizedDeepClone(u):e.visitEachChild(n,p,e.nullTransformationContext);return c=l,d}}function w(n,t){return t.size?r(n):n;function r(n){var i=t.get(e.getNodeId(n).toString());return i?e.getSynthesizedDeepClone(i):e.visitEachChild(n,r,e.nullTransformationContext)}}function P(n){if(e.isFunctionLikeDeclaration(n)){var t=n.body;if(e.isBlock(t))return t.statements}else{if(e.isModuleBlock(n)||e.isSourceFile(n))return n.statements;if(e.isClassLike(n))return n.members;e.assertType(n)}return e.emptyArray}function M(n,t){return e.find(P(t),function(t){return t.pos>=n&&e.isFunctionLikeDeclaration(t)&&!e.isConstructorDeclaration(t)})}function F(n,t){var r,i=t.members;e.Debug.assert(i.length>0,"Found no members");for(var a=!0,o=0,s=i;o<s.length;o++){var c=s[o];if(c.pos>n)return r||i[0];if(a&&!e.isPropertyDeclaration(c)){if(void 0!==r)return c;a=!1}r=c}return void 0===r?e.Debug.fail():r}function G(n,t){var r;e.Debug.assert(!e.isClassLike(t));for(var i=n;i!==t;i=i.parent)g(i)&&(r=i);for(i=(r||n).parent;;i=i.parent){if(W(i)){for(var a=void 0,o=0,s=i.statements;o<s.length;o++){var c=s[o];if(c.pos>n.pos)break;a=c}return!a&&e.isCaseClause(i)?(e.Debug.assert(e.isSwitchStatement(i.parent.parent),"Grandparent isn't a switch statement"),i.parent.parent):e.Debug.checkDefined(a,"prevStatement failed to get set")}e.Debug.assert(i!==t,"Didn't encounter a block-like before encountering scope")}}function B(n,t){var r=e.map(n,function(n){return e.factory.createShorthandPropertyAssignment(n.symbol.name)}),i=e.map(t,function(n){return e.factory.createShorthandPropertyAssignment(n.symbol.name)});return void 0===r?i:void 0===i?r:r.concat(i)}function U(n){return e.isArray(n)}function V(n,t){return U(n.range)?{pos:e.first(n.range).getStart(t),end:e.last(n.range).getEnd()}:n.range}function j(n,t,a,o,s,c){var l,u,d=new e.Map,p=[],f=[],m=[],_=[],g=[],y=new e.Map,h=[],v=U(n.range)?1===n.range.length&&e.isExpressionStatement(n.range[0])?n.range[0].expression:void 0:n.range;if(void 0===v){var b=n.range,E=e.first(b).getStart(),T=e.last(b).end;u=e.createFileDiagnostic(o,E,T-E,r.expressionExpected)}else 147456&s.getTypeAtLocation(v).flags&&(u=e.createDiagnosticForNode(v,r.uselessConstantType));for(var S=0,x=t;S<x.length;S++){var L=x[S];p.push({usages:new e.Map,typeParameterUsages:new e.Map,substitutions:new e.Map}),f.push(new e.Map),m.push([]);var D=[];u&&D.push(u),e.isClassLike(L)&&e.isInJSFile(L)&&D.push(e.createDiagnosticForNode(L,r.cannotExtractToJSClass)),e.isArrowFunction(L)&&!e.isBlock(L.body)&&D.push(e.createDiagnosticForNode(L,r.cannotExtractToExpressionArrowFunction)),_.push(D)}var A=new e.Map,C=U(n.range)?e.factory.createBlock(n.range):n.range,I=U(n.range)?e.first(n.range):n.range,N=K(I);if(W(C),N&&!U(n.range)){var k=s.getContextualType(n.range);H(k)}if(d.size>0){for(var R=new e.Map,O=0,w=I;void 0!==w&&O<t.length;w=w.parent)if(w===t[O]&&(R.forEach(function(e,n){p[O].typeParameterUsages.set(n,e)}),O++),e.isDeclarationWithTypeParameters(w))for(var P=0,M=e.getEffectiveTypeParameterDeclarations(w);P<M.length;P++){var F=M[P],G=s.getTypeAtLocation(F);d.has(G.id.toString())&&R.set(G.id.toString(),G)}e.Debug.assert(O===t.length,"Should have iterated all scopes")}if(g.length){var B=e.isBlockScope(t[0],t[0].parent)?t[0]:e.getEnclosingBlockScopeContainer(t[0]);e.forEachChild(B,q)}for(var V=function(t){var i=p[t];if(t>0&&(i.usages.size>0||i.typeParameterUsages.size>0)){var a=U(n.range)?n.range[0]:n.range;_[t].push(e.createDiagnosticForNode(a,r.cannotAccessVariablesFromNestedScopes))}var o,s=!1;if(p[t].usages.forEach(function(n){2===n.usage&&(s=!0,106500&n.symbol.flags&&n.symbol.valueDeclaration&&e.hasEffectiveModifier(n.symbol.valueDeclaration,64)&&(o=n.symbol.valueDeclaration))}),e.Debug.assert(U(n.range)||0===h.length,"No variable declarations expected if something was extracted"),s&&!U(n.range)){var c=e.createDiagnosticForNode(n.range,r.cannotWriteInExpression);m[t].push(c),_[t].push(c)}else if(o&&t>0){c=e.createDiagnosticForNode(o,r.cannotExtractReadonlyPropertyInitializerOutsideConstructor);m[t].push(c),_[t].push(c)}else if(l){c=e.createDiagnosticForNode(l,r.cannotExtractExportedEntity);m[t].push(c),_[t].push(c)}},j=0;j<t.length;j++)V(j);return{target:C,usagesPerScope:p,functionErrorsPerScope:m,constantErrorsPerScope:_,exposedVariableDeclarations:h};function K(n){return!!e.findAncestor(n,function(n){return e.isDeclarationWithTypeParameters(n)&&0!==e.getEffectiveTypeParameterDeclarations(n).length})}function H(e){for(var n=s.getSymbolWalker(function(){return c.throwIfCancellationRequested(),!0}),t=n.walkType(e).visitedTypes,r=0,i=t;r<i.length;r++){var a=i[r];a.isTypeParameter()&&d.set(a.id.toString(),a)}}function W(n,t){if(void 0===t&&(t=1),N){var r=s.getTypeAtLocation(n);H(r)}if(e.isDeclaration(n)&&n.symbol&&g.push(n),e.isAssignmentExpression(n))W(n.left,2),W(n.right);else if(e.isUnaryExpressionWithWrite(n))W(n.operand,2);else if(e.isPropertyAccessExpression(n)||e.isElementAccessExpression(n))e.forEachChild(n,W);else if(e.isIdentifier(n)){if(!n.parent)return;if(e.isQualifiedName(n.parent)&&n!==n.parent.left)return;if(e.isPropertyAccessExpression(n.parent)&&n!==n.parent.expression)return;z(n,t,e.isPartOfTypeNode(n))}else e.forEachChild(n,W)}function z(n,r,i){var a=J(n,r,i);if(a)for(var o=0;o<t.length;o++){var s=f[o].get(a);s&&p[o].substitutions.set(e.getNodeId(n).toString(),s)}}function J(c,l,u){var d=X(c);if(d){var g=e.getSymbolId(d).toString(),y=A.get(g);if(y&&y>=l)return g;if(A.set(g,l),y){for(var h=0,v=p;h<v.length;h++){var b=v[h],E=b.usages.get(c.text);E&&b.usages.set(c.text,{usage:l,symbol:d,node:c})}return g}var T=d.getDeclarations(),S=T&&e.find(T,function(e){return e.getSourceFile()===o});if(S&&!e.rangeContainsStartEnd(a,S.getStart(),S.end)){if(n.facts&i.IsGenerator&&2===l){for(var x=e.createDiagnosticForNode(c,r.cannotExtractRangeThatContainsWritesToReferencesLocatedOutsideOfTheTargetRangeInGenerators),L=0,D=m;L<D.length;L++){var C=D[L];C.push(x)}for(var I=0,N=_;I<N.length;I++){C=N[I];C.push(x)}}for(var k=0;k<t.length;k++){var R=t[k],O=s.resolveName(d.name,R,d.flags,!1);if(O!==d&&!f[k].has(g)){var w=Y(d.exportSymbol||d,R,u);if(w)f[k].set(g,w);else if(u){if(!(262144&d.flags)){x=e.createDiagnosticForNode(c,r.typeWillNotBeVisibleInTheNewScope);m[k].push(x),_[k].push(x)}}else p[k].usages.set(c.text,{usage:l,symbol:d,node:c})}}return g}}}function q(t){if(!(t===n.range||U(n.range)&&n.range.indexOf(t)>=0)){var r=e.isIdentifier(t)?X(t):s.getSymbolAtLocation(t);if(r){var i=e.find(g,function(e){return e.symbol===r});if(i)if(e.isVariableDeclaration(i)){var a=i.symbol.id.toString();y.has(a)||(h.push(i),y.set(a,!0))}else l=l||i}e.forEachChild(t,q)}}function X(n){return n.parent&&e.isShorthandPropertyAssignment(n.parent)&&n.parent.name===n?s.getShorthandAssignmentValueSymbol(n.parent):s.getSymbolAtLocation(n)}function Y(n,t,r){if(n){var i=n.getDeclarations();if(i&&i.some(function(e){return e.parent===t}))return e.factory.createIdentifier(n.name);var a=Y(n.parent,t,r);if(void 0!==a)return r?e.factory.createQualifiedName(a,e.factory.createIdentifier(n.name)):e.factory.createPropertyAccessExpression(a,n.name)}}}function K(n){return e.findAncestor(n,function(n){return n.parent&&H(n)&&!e.isBinaryExpression(n.parent)})}function H(e){var n=e.parent;switch(n.kind){case 294:return!1}switch(e.kind){case 10:return 264!==n.kind&&268!==n.kind;case 223:case 199:case 201:return!1;case 79:return 201!==n.kind&&268!==n.kind&&273!==n.kind}return!0}function W(e){switch(e.kind){case 233:case 300:case 260:case 287:return!0;default:return!1}}function z(n){return(e.isJsxElement(n)||e.isJsxSelfClosingElement(n)||e.isJsxFragment(n))&&(e.isJsxElement(n.parent)||e.isJsxFragment(n.parent))}n.registerRefactor(c,{kinds:[l.kind,u.kind],getAvailableActions:d,getEditsForAction:p}),t.getAvailableActions=d,t.getEditsForAction=p,function(n){function t(n){return{message:n,code:0,category:e.DiagnosticCategory.Message,key:n}}n.cannotExtractRange=t("Cannot extract range."),n.cannotExtractImport=t("Cannot extract import statement."),n.cannotExtractSuper=t("Cannot extract super call."),n.cannotExtractJSDoc=t("Cannot extract JSDoc."),n.cannotExtractEmpty=t("Cannot extract empty range."),n.expressionExpected=t("expression expected."),n.uselessConstantType=t("No reason to extract constant of type."),n.statementOrExpressionExpected=t("Statement or expression expected."),n.cannotExtractRangeContainingConditionalBreakOrContinueStatements=t("Cannot extract range containing conditional break or continue statements."),n.cannotExtractRangeContainingConditionalReturnStatement=t("Cannot extract range containing conditional return statement."),n.cannotExtractRangeContainingLabeledBreakOrContinueStatementWithTargetOutsideOfTheRange=t("Cannot extract range containing labeled break or continue with target outside of the range."),n.cannotExtractRangeThatContainsWritesToReferencesLocatedOutsideOfTheTargetRangeInGenerators=t("Cannot extract range containing writes to references located outside of the target range in generators."),n.typeWillNotBeVisibleInTheNewScope=t("Type will not visible in the new scope."),n.functionWillNotBeVisibleInTheNewScope=t("Function will not visible in the new scope."),n.cannotExtractIdentifier=t("Select more than a single identifier."),n.cannotExtractExportedEntity=t("Cannot extract exported declaration"),n.cannotWriteInExpression=t("Cannot write back side-effects when extracting an expression"),n.cannotExtractReadonlyPropertyInitializerOutsideConstructor=t("Cannot move initialization of read-only class property outside of the constructor"),n.cannotExtractAmbientBlock=t("Cannot extract code from ambient contexts"),n.cannotAccessVariablesFromNestedScopes=t("Cannot access variables from nested scopes"),n.cannotExtractToJSClass=t("Cannot extract constant to a class scope in JS"),n.cannotExtractToExpressionArrowFunction=t("Cannot extract constant to an arrow function without a block")}(r=t.Messages||(t.Messages={})),function(e){e[e["None"]=0]="None",e[e["HasReturn"]=1]="HasReturn",e[e["IsGenerator"]=2]="IsGenerator",e[e["IsAsyncFunction"]=4]="IsAsyncFunction",e[e["UsesThis"]=8]="UsesThis",e[e["InStaticRegion"]=16]="InStaticRegion"}(i||(i={})),t.getRangeToExtract=f,function(e){e[e["Module"]=0]="Module",e[e["Global"]=1]="Global"}(a||(a={})),function(e){e[e["Read"]=1]="Read",e[e["Write"]=2]="Write"}(s||(s={}))})(n.extractSymbol||(n.extractSymbol={}))})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){var t="Extract type",r={name:"Extract to type alias",description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_type_alias),kind:"refactor.extract.type"},i={name:"Extract to interface",description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_interface),kind:"refactor.extract.interface"},a={name:"Extract to typedef",description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_to_typedef),kind:"refactor.extract.typedef"};function s(n,t){void 0===t&&(t=!0);var r=n.file,i=n.startPosition,a=e.isSourceFileJS(r),o=e.getTokenAtPosition(r,i),s=e.createTextRangeFromSpan(e.getRefactorContextSpan(n)),d=s.pos===s.end&&t,p=e.findAncestor(o,function(n){return n.parent&&e.isTypeNode(n)&&!l(s,n.parent,r)&&(d||e.nodeOverlapsWithStartEnd(o,r,s.pos,s.end))});if(!p||!e.isTypeNode(p))return{error:e.getLocaleSpecificMessage(e.Diagnostics.Selection_is_not_a_valid_type_node)};var f=n.program.getTypeChecker(),m=e.Debug.checkDefined(e.findAncestor(p,e.isStatement),"Should find a statement"),_=u(f,p,m,r);if(!_)return{error:e.getLocaleSpecificMessage(e.Diagnostics.No_type_could_be_extracted_from_this_type_node)};var g=c(f,p);return{isJS:a,selection:p,firstStatement:m,typeParameters:_,typeElements:g}}function c(n,t){if(t){if(e.isIntersectionTypeNode(t)){for(var r=[],i=new e.Map,a=0,o=t.types;a<o.length;a++){var s=o[a],l=c(n,s);if(!l||!l.every(function(n){return n.name&&e.addToSeen(i,e.getNameFromPropertyName(n.name))}))return;e.addRange(r,l)}return r}return e.isParenthesizedTypeNode(t)?c(n,t.type):e.isTypeLiteralNode(t)?t.members:void 0}}function l(n,t,r){return e.rangeContainsStartEnd(n,e.skipTrivia(r.text,t.pos),t.end)}function u(n,t,r,i){var a=[];return o(t)?void 0:a;function o(s){if(e.isTypeReferenceNode(s)){if(e.isIdentifier(s.typeName)){var c=n.resolveName(s.typeName.text,s.typeName,262144,!0);if(null===c||void 0===c?void 0:c.declarations){var u=e.cast(e.first(c.declarations),e.isTypeParameterDeclaration);l(r,u,i)&&!l(t,u,i)&&e.pushIfUnique(a,u)}}}else if(e.isInferTypeNode(s)){var d=e.findAncestor(s,function(n){return e.isConditionalTypeNode(n)&&l(n.extendsType,s,i)});if(!d||!l(t,d,i))return!0}else if(e.isTypePredicateNode(s)||e.isThisTypeNode(s)){var p=e.findAncestor(s.parent,e.isFunctionLike);if(p&&p.type&&l(p.type,s,i)&&!l(t,p,i))return!0}else if(e.isTypeQueryNode(s))if(e.isIdentifier(s.exprName)){c=n.resolveName(s.exprName.text,s.exprName,111551,!1);if((null===c||void 0===c?void 0:c.valueDeclaration)&&l(r,c.valueDeclaration,i)&&!l(t,c.valueDeclaration,i))return!0}else if(e.isThisIdentifier(s.exprName.left)&&!l(t,s.parent,i))return!0;return i&&e.isTupleTypeNode(s)&&e.getLineAndCharacterOfPosition(i,s.pos).line===e.getLineAndCharacterOfPosition(i,s.end).line&&e.setEmitFlags(s,1),e.forEachChild(s,o)}}function d(n,t,r,i){var a=i.firstStatement,o=i.selection,s=i.typeParameters,c=e.factory.createTypeAliasDeclaration(void 0,void 0,r,s.map(function(n){return e.factory.updateTypeParameterDeclaration(n,n.name,n.constraint,void 0)}),o);n.insertNodeBefore(t,a,e.ignoreSourceNewlines(c),!0),n.replaceNode(t,o,e.factory.createTypeReferenceNode(r,s.map(function(n){return e.factory.createTypeReferenceNode(n.name,void 0)})),{leadingTriviaOption:e.textChanges.LeadingTriviaOption.Exclude,trailingTriviaOption:e.textChanges.TrailingTriviaOption.ExcludeWhitespace})}function p(n,t,r,i){var a,o=i.firstStatement,s=i.selection,c=i.typeParameters,l=i.typeElements,u=e.factory.createInterfaceDeclaration(void 0,void 0,r,c,void 0,l);e.setTextRange(u,null===(a=l[0])||void 0===a?void 0:a.parent),n.insertNodeBefore(t,o,e.ignoreSourceNewlines(u),!0),n.replaceNode(t,s,e.factory.createTypeReferenceNode(r,c.map(function(n){return e.factory.createTypeReferenceNode(n.name,void 0)})),{leadingTriviaOption:e.textChanges.LeadingTriviaOption.Exclude,trailingTriviaOption:e.textChanges.TrailingTriviaOption.ExcludeWhitespace})}function f(n,t,r,i){var a=i.firstStatement,o=i.selection,s=i.typeParameters,c=e.factory.createJSDocTypedefTag(e.factory.createIdentifier("typedef"),e.factory.createJSDocTypeExpression(o),e.factory.createIdentifier(r)),l=[];e.forEach(s,function(n){var t=e.getEffectiveConstraintOfTypeParameter(n),r=e.factory.createTypeParameterDeclaration(n.name),i=e.factory.createJSDocTemplateTag(e.factory.createIdentifier("template"),t&&e.cast(t,e.isJSDocTypeExpression),[r]);l.push(i)}),n.insertNodeBefore(t,a,e.factory.createJSDocComment(void 0,e.factory.createNodeArray(e.concatenate(l,[c]))),!0),n.replaceNode(t,o,e.factory.createTypeReferenceNode(r,s.map(function(n){return e.factory.createTypeReferenceNode(n.name,void 0)})))}n.registerRefactor(t,{kinds:[r.kind,i.kind,a.kind],getAvailableActions:function(c){var l=s(c,"invoked"===c.triggerReason);return l?n.isRefactorErrorInfo(l)?c.preferences.provideRefactorNotApplicableReason?[{name:t,description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_type),actions:[o(o({},a),{notApplicableReason:l.error}),o(o({},r),{notApplicableReason:l.error}),o(o({},i),{notApplicableReason:l.error})]}]:e.emptyArray:[{name:t,description:e.getLocaleSpecificMessage(e.Diagnostics.Extract_type),actions:l.isJS?[a]:e.append([r],l.typeElements&&i)}]:e.emptyArray},getEditsForAction:function(t,o){var c=t.file,l=s(t);e.Debug.assert(l&&!n.isRefactorErrorInfo(l),"Expected to find a range to extract");var u=e.getUniqueName("NewType",c),m=e.textChanges.ChangeTracker.with(t,function(n){switch(o){case r.name:return e.Debug.assert(!l.isJS,"Invalid actionName/JS combo"),d(n,c,u,l);case a.name:return e.Debug.assert(l.isJS,"Invalid actionName/JS combo"),f(n,c,u,l);case i.name:return e.Debug.assert(!l.isJS&&!!l.typeElements,"Invalid actionName/JS combo"),p(n,c,u,l);default:e.Debug.fail("Unexpected action name")}}),_=c.fileName,g=e.getRenameLocation(m,_,u,!1);return{edits:m,renameFilename:_,renameLocation:g}}})})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){(function(t){var r="Generate 'get' and 'set' accessors",i=e.Diagnostics.Generate_get_and_set_accessors.message,a={name:r,description:i,kind:"refactor.rewrite.property.generateAccessors"};n.registerRefactor(r,{kinds:[a.kind],getEditsForAction:function(t,r){if(t.endPosition){var i=e.codefix.getAccessorConvertiblePropertyAtPosition(t.file,t.program,t.startPosition,t.endPosition);e.Debug.assert(i&&!n.isRefactorErrorInfo(i),"Expected applicable refactor info");var a=e.codefix.generateAccessorFromProperty(t.file,t.program,t.startPosition,t.endPosition,t,r);if(a){var o=t.file.fileName,s=i.renameAccessor?i.accessorName:i.fieldName,c=e.isIdentifier(s)?0:-1,l=c+e.getRenameLocation(a,o,s.text,e.isParameter(i.declaration));return{renameFilename:o,renameLocation:l,edits:a}}}},getAvailableActions:function(t){if(!t.endPosition)return e.emptyArray;var s=e.codefix.getAccessorConvertiblePropertyAtPosition(t.file,t.program,t.startPosition,t.endPosition,"invoked"===t.triggerReason);return s?n.isRefactorErrorInfo(s)?t.preferences.provideRefactorNotApplicableReason?[{name:r,description:i,actions:[o(o({},a),{notApplicableReason:s.error})]}]:e.emptyArray:[{name:r,description:i,actions:[a]}]:e.emptyArray}})})(n.generateGetAccessorAndSetAccessor||(n.generateGetAccessorAndSetAccessor={}))})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(e){function n(e){return void 0!==e.error}function t(e,n){return!n||e.substr(0,n.length)===n}e.isRefactorErrorInfo=n,e.refactorKindBeginsWith=t})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){var t="Move to a new file",r=e.getLocaleSpecificMessage(e.Diagnostics.Move_to_a_new_file),i={name:t,description:r,kind:"refactor.move.newFile"};function s(n){var t=n.file,r=e.createTextRangeFromSpan(e.getRefactorContextSpan(n)),i=t.statements,a=e.findIndex(i,function(e){return e.end>r.pos});if(-1!==a){var o=i[a];if(e.isNamedDeclaration(o)&&o.name&&e.rangeContainsRange(o.name,r))return{toMove:[i[a]],afterLast:i[a+1]};if(!(r.pos>o.getStart(t))){var s=e.findIndex(i,function(e){return e.end>r.end},a);if(-1===s||!(0===s||i[s].getStart(t)<r.end))return{toMove:i.slice(a,-1===s?i.length:s),afterLast:-1===s?void 0:i[s]}}}}function c(n,t,r,i,a,o){var s=t.getTypeChecker(),c=O(n,r.all,s),l=e.getDirectoryPath(n.fileName),u=e.extensionFromPath(n.fileName),d=k(R(c.movedSymbols),u,l,a),m=d+u;i.createNewFile(n,e.combinePaths(l,m),f(n,c,i,r,t,d,o)),p(t,i,n.fileName,m,e.hostGetCanonicalFileName(a))}function l(n){var t=s(n);if(void 0!==t){var r=[],i=[],a=t.toMove,o=t.afterLast;return e.getRangesWhere(a,u,function(e,n){for(var t=e;t<n;t++)r.push(a[t]);i.push({first:a[e],afterLast:o})}),0===r.length?void 0:{all:r,ranges:i}}}function u(n){return!d(n)&&!e.isPrologueDirective(n)}function d(n){switch(n.kind){case 264:return!0;case 263:return!e.hasSyntacticModifier(n,1);case 235:return n.declarationList.declarations.every(function(n){return!!n.initializer&&e.isRequireCall(n.initializer,!0)});default:return!1}}function p(n,t,r,i,a){var o=n.getCompilerOptions().configFile;if(o){var s=e.normalizePath(e.combinePaths(r,"..",i)),c=e.getRelativePathFromFile(o.fileName,s,a),l=o.statements[0]&&e.tryCast(o.statements[0].expression,e.isObjectLiteralExpression),u=l&&e.find(l.properties,function(n){return e.isPropertyAssignment(n)&&e.isStringLiteral(n.name)&&"files"===n.name.text});u&&e.isArrayLiteralExpression(u.initializer)&&t.insertNodeInListAfter(o,e.last(u.initializer.elements),e.factory.createStringLiteral(c),u.initializer.elements)}}function f(n,t,r,i,o,s,c){var l=o.getTypeChecker(),u=e.takeWhile(n.statements,e.isPrologueDirective);if(!n.externalModuleIndicator&&!n.commonJsModuleIndicator)return m(n,i.ranges,r),a(a([],u,!0),i.all,!0);var d=!!n.externalModuleIndicator,p=e.getQuotePreference(n,c),f=T(t.oldFileImportsFromNewFile,s,d,p);f&&e.insertImports(r,n,f,!0),_(n,i.all,r,t.unusedImportsFromOldFile,l),m(n,i.ranges,r),g(r,o,n,t.movedSymbols,s);var y=N(n,t.oldImportsNeededByNewFile,t.newFileImportsFromOldFile,r,l,d,p),h=D(n,i.all,t.oldFileImportsFromNewFile,d);return y.length&&h.length?a(a(a(a([],u,!0),y,!0),[4],!1),h,!0):a(a(a([],u,!0),y,!0),h,!0)}function m(e,n,t){for(var r=0,i=n;r<i.length;r++){var a=i[r],o=a.first,s=a.afterLast;t.deleteNodeRangeExcludingEnd(e,o,s)}}function _(n,t,r,i,a){for(var o=0,s=n.statements;o<s.length;o++){var c=s[o];e.contains(t,c)||E(c,function(e){return A(n,e,r,function(e){return i.has(a.getSymbolAtLocation(e))})})}}function g(n,t,r,i,a){for(var o=t.getTypeChecker(),s=function(t){if(t===r)return"continue";for(var s=function(s){E(s,function(c){if(o.getSymbolAtLocation(b(c))===r.symbol){var l=function(n){var t=e.isBindingElement(n.parent)?e.getPropertySymbolFromBindingElement(o,n.parent):e.skipAlias(o.getSymbolAtLocation(n),o);return!!t&&i.has(t)};A(t,c,n,l);var u=e.combinePaths(e.getDirectoryPath(b(c).text),a),d=M(c,e.factory.createStringLiteral(u),l);d&&n.insertNodeAfter(t,s,d);var p=y(c);p&&h(n,t,o,i,a,u,p,c)}})},c=0,l=t.statements;c<l.length;c++){var u=l[c];s(u)}},c=0,l=t.getSourceFiles();c<l.length;c++){var u=l[c];s(u)}}function y(n){switch(n.kind){case 264:return n.importClause&&n.importClause.namedBindings&&266===n.importClause.namedBindings.kind?n.importClause.namedBindings.name:void 0;case 263:return n.name;case 252:return e.tryCast(n.name,e.isIdentifier);default:return e.Debug.assertNever(n,"Unexpected node kind "+n.kind)}}function h(n,t,r,i,a,o,s,c){var l=e.codefix.moduleSpecifierToValidIdentifier(a,99),u=!1,d=[];if(e.FindAllReferences.Core.eachSymbolReferenceInFile(s,r,t,function(n){e.isPropertyAccessExpression(n.parent)&&(u=u||!!r.resolveName(l,n,67108863,!0),i.has(r.getSymbolAtLocation(n.parent.name))&&d.push(n))}),d.length){for(var p=u?e.getUniqueName(l,t):l,f=0,m=d;f<m.length;f++){var _=m[f];n.replaceNode(t,_,e.factory.createIdentifier(p))}n.insertNodeAfter(t,c,v(c,a,o))}}function v(n,t,r){var i=e.factory.createIdentifier(t),a=e.factory.createStringLiteral(r);switch(n.kind){case 264:return e.factory.createImportDeclaration(void 0,void 0,e.factory.createImportClause(!1,void 0,e.factory.createNamespaceImport(i)),a);case 263:return e.factory.createImportEqualsDeclaration(void 0,void 0,!1,i,e.factory.createExternalModuleReference(a));case 252:return e.factory.createVariableDeclaration(i,void 0,void 0,L(a));default:return e.Debug.assertNever(n,"Unexpected node kind "+n.kind)}}function b(e){return 264===e.kind?e.moduleSpecifier:263===e.kind?e.moduleReference.expression:e.initializer.arguments[0]}function E(n,t){if(e.isImportDeclaration(n))e.isStringLiteral(n.moduleSpecifier)&&t(n);else if(e.isImportEqualsDeclaration(n))e.isExternalModuleReference(n.moduleReference)&&e.isStringLiteralLike(n.moduleReference.expression)&&t(n);else if(e.isVariableStatement(n))for(var r=0,i=n.declarationList.declarations;r<i.length;r++){var a=i[r];a.initializer&&e.isRequireCall(a.initializer,!0)&&t(a)}}function T(n,t,r,i){var a,o=[];return n.forEach(function(n){"default"===n.escapedName?a=e.factory.createIdentifier(e.symbolNameNoDefault(n)):o.push(n.name)}),S(a,o,t,r,i)}function S(n,t,r,i,a){if(r=e.ensurePathIsNonModuleName(r),i){var o=t.map(function(n){return e.factory.createImportSpecifier(void 0,e.factory.createIdentifier(n))});return e.makeImportIfNecessary(n,o,r,a)}e.Debug.assert(!n,"No default import should exist");var s=t.map(function(n){return e.factory.createBindingElement(void 0,void 0,n)});return s.length?x(e.factory.createObjectBindingPattern(s),void 0,L(e.factory.createStringLiteral(r))):void 0}function x(n,t,r,i){return void 0===i&&(i=2),e.factory.createVariableStatement(void 0,e.factory.createVariableDeclarationList([e.factory.createVariableDeclaration(n,void 0,t,r)],i))}function L(n){return e.factory.createCallExpression(e.factory.createIdentifier("require"),void 0,[n])}function D(n,t,r,i){return e.flatMap(t,function(t){if(K(t)&&!Y(n,t,i)&&W(t,function(n){return r.has(e.Debug.checkDefined(n.symbol))})){var a=Q(t,i);if(a)return a}return t})}function A(n,t,r,i){switch(t.kind){case 264:C(n,t,r,i);break;case 263:i(t.name)&&r.delete(n,t);break;case 252:I(n,t,r,i);break;default:e.Debug.assertNever(t,"Unexpected import decl kind "+t.kind)}}function C(n,t,r,i){if(t.importClause){var a=t.importClause,o=a.name,s=a.namedBindings,c=!o||i(o),l=!s||(266===s.kind?i(s.name):0!==s.elements.length&&s.elements.every(function(e){return i(e.name)}));if(c&&l)r.delete(n,t);else if(o&&c&&r.delete(n,o),s)if(l)r.replaceNode(n,t.importClause,e.factory.updateImportClause(t.importClause,t.importClause.isTypeOnly,o,void 0));else if(267===s.kind)for(var u=0,d=s.elements;u<d.length;u++){var p=d[u];i(p.name)&&r.delete(n,p)}}}function I(n,t,r,i){var a=t.name;switch(a.kind){case 79:i(a)&&r.delete(n,a);break;case 200:break;case 199:if(a.elements.every(function(n){return e.isIdentifier(n.name)&&i(n.name)}))r.delete(n,e.isVariableDeclarationList(t.parent)&&1===t.parent.declarations.length?t.parent.parent:t);else for(var o=0,s=a.elements;o<s.length;o++){var c=s[o];e.isIdentifier(c.name)&&i(c.name)&&r.delete(n,c.name)}break}}function N(n,t,r,i,a,o,s){for(var c,l=[],u=0,d=n.statements;u<d.length;u++){var p=d[u];E(p,function(n){e.append(l,M(n,b(n),function(e){return t.has(a.getSymbolAtLocation(e))}))})}var f=[],m=e.nodeSeenTracker();return r.forEach(function(t){if(t.declarations)for(var r=0,a=t.declarations;r<a.length;r++){var s=a[r];if(V(s)){var l=J(s);if(l){var u=q(s);m(u)&&X(n,u,l,i,o),e.hasSyntacticModifier(s,512)?c=l:f.push(l.text)}}}}),e.append(l,S(c,f,e.removeFileExtension(e.getBaseFileName(n.fileName)),o,s)),l}function k(n,t,r,i){for(var a=n,o=1;;o++){var s=e.combinePaths(r,a+t);if(!i.fileExists(s))return a;a=n+"."+o}}function R(n){return n.forEachEntry(e.symbolNameNoDefault)||"newFile"}function O(n,t,r){var i=new U,a=new U,o=new U,s=e.find(t,function(e){return!!(2&e.transformFlags)}),c=h(s);c&&a.add(c);for(var l=0,u=t;l<u.length;l++){var d=u[l];W(d,function(n){i.add(e.Debug.checkDefined(e.isExpressionStatement(n)?r.getSymbolAtLocation(n.expression.left):n.symbol,"Need a symbol here"))})}for(var p=0,f=t;p<f.length;p++){d=f[p];B(d,r,function(e){if(e.declarations)for(var t=0,r=e.declarations;t<r.length;t++){var s=r[t];w(s)?a.add(e):V(s)&&j(s)===n&&!i.has(e)&&o.add(e)}})}for(var m=a.clone(),_=new U,g=0,y=n.statements;g<y.length;g++){d=y[g];e.contains(t,d)||(c&&2&d.transformFlags&&m.delete(c),B(d,r,function(e){i.has(e)&&_.add(e),m.delete(e)}))}return{movedSymbols:i,newFileImportsFromOldFile:o,oldFileImportsFromNewFile:_,oldImportsNeededByNewFile:a,unusedImportsFromOldFile:m};function h(n){if(void 0!==n){var t=r.getJsxNamespace(n),i=r.resolveName(t,n,1920,!0);return i&&e.some(i.declarations,w)?i:void 0}}}function w(n){switch(n.kind){case 263:case 268:case 265:case 266:return!0;case 252:return P(n);case 201:return e.isVariableDeclaration(n.parent.parent)&&P(n.parent.parent);default:return!1}}function P(n){return e.isSourceFile(n.parent.parent.parent)&&!!n.initializer&&e.isRequireCall(n.initializer,!0)}function M(n,t,r){switch(n.kind){case 264:var i=n.importClause;if(!i)return;var a=i.name&&r(i.name)?i.name:void 0,o=i.namedBindings&&F(i.namedBindings,r);return a||o?e.factory.createImportDeclaration(void 0,void 0,e.factory.createImportClause(!1,a,o),t):void 0;case 263:return r(n.name)?n:void 0;case 252:var s=G(n.name,r);return s?x(s,n.type,L(t),n.parent.flags):void 0;default:return e.Debug.assertNever(n,"Unexpected import kind "+n.kind)}}function F(n,t){if(266===n.kind)return t(n.name)?n:void 0;var r=n.elements.filter(function(e){return t(e.name)});return r.length?e.factory.createNamedImports(r):void 0}function G(n,t){switch(n.kind){case 79:return t(n)?n:void 0;case 200:return n;case 199:var r=n.elements.filter(function(n){return n.propertyName||!e.isIdentifier(n.name)||t(n.name)});return r.length?e.factory.createObjectBindingPattern(r):void 0}}function B(n,t,r){n.forEachChild(function n(i){if(e.isIdentifier(i)&&!e.isDeclarationName(i)){var a=t.getSymbolAtLocation(i);a&&r(a)}else i.forEachChild(n)})}n.registerRefactor(t,{kinds:[i.kind],getAvailableActions:function(n){var a=l(n);return n.preferences.allowTextChangesInNewFiles&&a?[{name:t,description:r,actions:[i]}]:n.preferences.provideRefactorNotApplicableReason?[{name:t,description:r,actions:[o(o({},i),{notApplicableReason:e.getLocaleSpecificMessage(e.Diagnostics.Selection_is_not_a_valid_statement_or_statements)})]}]:e.emptyArray},getEditsForAction:function(n,r){e.Debug.assert(r===t,"Wrong refactor invoked");var i=e.Debug.checkDefined(l(n)),a=e.textChanges.ChangeTracker.with(n,function(e){return c(n.file,n.program,i,e,n.host,n.preferences)});return{edits:a,renameFilename:void 0,renameLocation:void 0}}});var U=function(){function n(){this.map=new e.Map}return n.prototype.add=function(n){this.map.set(String(e.getSymbolId(n)),n)},n.prototype.has=function(n){return this.map.has(String(e.getSymbolId(n)))},n.prototype.delete=function(n){this.map.delete(String(e.getSymbolId(n)))},n.prototype.forEach=function(e){this.map.forEach(e)},n.prototype.forEachEntry=function(n){return e.forEachEntry(this.map,n)},n.prototype.clone=function(){var t=new n;return e.copyEntries(this.map,t.map),t},n}();function V(n){return H(n)&&e.isSourceFile(n.parent)||e.isVariableDeclaration(n)&&e.isSourceFile(n.parent.parent.parent)}function j(n){return e.isVariableDeclaration(n)?n.parent.parent.parent:n.parent}function K(n){return e.Debug.assert(e.isSourceFile(n.parent),"Node parent should be a SourceFile"),H(n)||e.isVariableStatement(n)}function H(e){switch(e.kind){case 254:case 255:case 259:case 258:case 257:case 256:case 263:return!0;default:return!1}}function W(n,t){switch(n.kind){case 254:case 255:case 259:case 258:case 257:case 256:case 263:return t(n);case 235:return e.firstDefined(n.declarationList.declarations,function(e){return z(e.name,t)});case 236:var r=n.expression;return e.isBinaryExpression(r)&&1===e.getAssignmentDeclarationKind(r)?t(n):void 0}}function z(n,t){switch(n.kind){case 79:return t(e.cast(n.parent,function(n){return e.isVariableDeclaration(n)||e.isBindingElement(n)}));case 200:case 199:return e.firstDefined(n.elements,function(n){return e.isOmittedExpression(n)?void 0:z(n.name,t)});default:return e.Debug.assertNever(n,"Unexpected name kind "+n.kind)}}function J(n){return e.isExpressionStatement(n)?e.tryCast(n.expression.left.name,e.isIdentifier):e.tryCast(n.name,e.isIdentifier)}function q(n){switch(n.kind){case 252:return n.parent.parent;case 201:return q(e.cast(n.parent.parent,function(n){return e.isVariableDeclaration(n)||e.isBindingElement(n)}));default:return n}}function X(n,t,r,i,a){if(!Y(n,t,a,r))if(a)e.isExpressionStatement(t)||i.insertExportModifier(n,t);else{var o=ee(t);0!==o.length&&i.insertNodesAfter(n,t,o.map(ne))}}function Y(n,t,r,i){var a;return r?!e.isExpressionStatement(t)&&e.hasSyntacticModifier(t,1)||!!(i&&(null===(a=n.symbol.exports)||void 0===a?void 0:a.has(i.escapedText))):ee(t).some(function(t){return n.symbol.exports.has(e.escapeLeadingUnderscores(t))})}function Q(e,n){return n?[Z(e)]:$(e)}function Z(n){var t=e.concatenate([e.factory.createModifier(93)],n.modifiers);switch(n.kind){case 254:return e.factory.updateFunctionDeclaration(n,n.decorators,t,n.asteriskToken,n.name,n.typeParameters,n.parameters,n.type,n.body);case 255:return e.factory.updateClassDeclaration(n,n.decorators,t,n.name,n.typeParameters,n.heritageClauses,n.members);case 235:return e.factory.updateVariableStatement(n,t,n.declarationList);case 259:return e.factory.updateModuleDeclaration(n,n.decorators,t,n.name,n.body);case 258:return e.factory.updateEnumDeclaration(n,n.decorators,t,n.name,n.members);case 257:return e.factory.updateTypeAliasDeclaration(n,n.decorators,t,n.name,n.typeParameters,n.type);case 256:return e.factory.updateInterfaceDeclaration(n,n.decorators,t,n.name,n.typeParameters,n.heritageClauses,n.members);case 263:return e.factory.updateImportEqualsDeclaration(n,n.decorators,t,n.isTypeOnly,n.name,n.moduleReference);case 236:return e.Debug.fail();default:return e.Debug.assertNever(n,"Unexpected declaration kind "+n.kind)}}function $(e){return a([e],ee(e).map(ne),!0)}function ee(n){switch(n.kind){case 254:case 255:return[n.name.text];case 235:return e.mapDefined(n.declarationList.declarations,function(n){return e.isIdentifier(n.name)?n.name.text:void 0});case 259:case 258:case 257:case 256:case 263:return e.emptyArray;case 236:return e.Debug.fail("Can't export an ExpressionStatement");default:return e.Debug.assertNever(n,"Unexpected decl kind "+n.kind)}}function ne(n){return e.factory.createExpressionStatement(e.factory.createBinaryExpression(e.factory.createPropertyAccessExpression(e.factory.createIdentifier("exports"),e.factory.createIdentifier(n)),63,e.factory.createIdentifier(n)))}})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){(function(t){var r="Add or remove braces in an arrow function",i=e.Diagnostics.Add_or_remove_braces_in_an_arrow_function.message,a={name:"Add braces to arrow function",description:e.Diagnostics.Add_braces_to_arrow_function.message,kind:"refactor.rewrite.arrow.braces.add"},s={name:"Remove braces from arrow function",description:e.Diagnostics.Remove_braces_from_arrow_function.message,kind:"refactor.rewrite.arrow.braces.remove"};function c(t){var c=t.file,l=t.startPosition,d=t.triggerReason,p=u(c,l,"invoked"===d);return p?n.isRefactorErrorInfo(p)?t.preferences.provideRefactorNotApplicableReason?[{name:r,description:i,actions:[o(o({},a),{notApplicableReason:p.error}),o(o({},s),{notApplicableReason:p.error})]}]:e.emptyArray:[{name:r,description:i,actions:[p.addBraces?a:s]}]:e.emptyArray}function l(t,r){var i=t.file,o=t.startPosition,c=u(i,o);e.Debug.assert(c&&!n.isRefactorErrorInfo(c),"Expected applicable refactor info");var l,d=c.expression,p=c.returnStatement,f=c.func;if(r===a.name){var m=e.factory.createReturnStatement(d);l=e.factory.createBlock([m],!0),e.suppressLeadingAndTrailingTrivia(l),e.copyLeadingComments(d,m,i,3,!0)}else if(r===s.name&&p){var _=d||e.factory.createVoidZero();l=e.needsParentheses(_)?e.factory.createParenthesizedExpression(_):_,e.suppressLeadingAndTrailingTrivia(l),e.copyTrailingAsLeadingComments(p,l,i,3,!1),e.copyLeadingComments(p,l,i,3,!1),e.copyTrailingComments(p,l,i,3,!1)}else e.Debug.fail("invalid action");var g=e.textChanges.ChangeTracker.with(t,function(e){e.replaceNode(i,f.body,l)});return{renameFilename:void 0,renameLocation:void 0,edits:g}}function u(t,r,i,o){void 0===i&&(i=!0);var c=e.getTokenAtPosition(t,r),l=e.getContainingFunction(c);if(!l)return{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_find_a_containing_arrow_function)};if(!e.isArrowFunction(l))return{error:e.getLocaleSpecificMessage(e.Diagnostics.Containing_function_is_not_an_arrow_function)};if(e.rangeContainsRange(l,c)&&(!e.rangeContainsRange(l.body,c)||i)){if(n.refactorKindBeginsWith(a.kind,o)&&e.isExpression(l.body))return{func:l,addBraces:!0,expression:l.body};if(n.refactorKindBeginsWith(s.kind,o)&&e.isBlock(l.body)&&1===l.body.statements.length){var u=e.first(l.body.statements);if(e.isReturnStatement(u))return{func:l,addBraces:!1,expression:u.expression,returnStatement:u}}}}n.registerRefactor(r,{kinds:[s.kind],getEditsForAction:l,getAvailableActions:c})})(n.addOrRemoveBracesToArrowFunction||(n.addOrRemoveBracesToArrowFunction={}))})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){(function(t){var r="Convert parameters to destructured object",i=2,o=e.getLocaleSpecificMessage(e.Diagnostics.Convert_parameters_to_destructured_object),s={name:r,description:o,kind:"refactor.rewrite.parameters.toDestructured"};function c(n){var t=n.file,i=n.startPosition,a=e.isSourceFileJS(t);if(a)return e.emptyArray;var c=h(t,i,n.program.getTypeChecker());return c?[{name:r,description:o,actions:[s]}]:e.emptyArray}function l(n,t){e.Debug.assert(t===r,"Unexpected action name");var i=n.file,a=n.startPosition,o=n.program,s=n.cancellationToken,c=n.host,l=h(i,a,o.getTypeChecker());if(l&&s){var p=d(l,o,s);if(p.valid){var f=e.textChanges.ChangeTracker.with(n,function(e){return u(i,o,c,e,l,p)});return{renameFilename:void 0,renameLocation:void 0,edits:f}}return{edits:[]}}}function u(n,t,r,i,a,o){var s=o.signature,c=e.map(R(a,t,r),function(n){return e.getSynthesizedDeepClone(n)});if(s){var l=e.map(R(s,t,r),function(n){return e.getSynthesizedDeepClone(n)});_(s,l)}_(a,c);for(var u=e.sortAndDeduplicate(o.functionCalls,function(n,t){return e.compareValues(n.pos,t.pos)}),d=0,p=u;d<p.length;d++){var f=p[d];if(f.arguments&&f.arguments.length){var m=e.getSynthesizedDeepClone(k(a,f.arguments),!0);i.replaceNodeRange(e.getSourceFileOfNode(f),e.first(f.arguments),e.last(f.arguments),m,{leadingTriviaOption:e.textChanges.LeadingTriviaOption.IncludeAll,trailingTriviaOption:e.textChanges.TrailingTriviaOption.Include})}}function _(t,r){i.replaceNodeRangeWithNodes(n,e.first(t.parameters),e.last(t.parameters),r,{joiner:", ",indentation:0,leadingTriviaOption:e.textChanges.LeadingTriviaOption.IncludeAll,trailingTriviaOption:e.textChanges.TrailingTriviaOption.Include})}}function d(n,t,r){var i=P(n),o=e.isConstructorDeclaration(n)?w(n):[],s=e.deduplicate(a(a([],i,!0),o,!0),e.equateValues),c=t.getTypeChecker(),l=e.flatMap(s,function(n){return e.FindAllReferences.getReferenceEntriesForNode(-1,n,t,t.getSourceFiles(),r)}),u=d(l);return e.every(u.declarations,function(n){return e.contains(s,n)})||(u.valid=!1),u;function d(t){for(var r={accessExpressions:[],typeUsages:[]},a={functionCalls:[],declarations:[],classReferences:r,valid:!0},s=e.map(i,h),l=e.map(o,h),u=e.isConstructorDeclaration(n),d=e.map(i,function(e){return p(e,c)}),v=0,E=t;v<E.length;v++){var T=E[v];if(0!==T.kind){if(e.contains(d,h(T.node))){if(b(T.node.parent)){a.signature=T.node.parent;continue}var S=_(T);if(S){a.functionCalls.push(S);continue}}var x=p(T.node,c);if(x&&e.contains(d,x)){var L=m(T);if(L){a.declarations.push(L);continue}}if(e.contains(s,h(T.node))||e.isNewExpressionTarget(T.node)){var D=f(T);if(D)continue;L=m(T);if(L){a.declarations.push(L);continue}S=_(T);if(S){a.functionCalls.push(S);continue}}if(u&&e.contains(l,h(T.node))){D=f(T);if(D)continue;L=m(T);if(L){a.declarations.push(L);continue}var A=g(T);if(A){r.accessExpressions.push(A);continue}if(e.isClassDeclaration(n.parent)){var C=y(T);if(C){r.typeUsages.push(C);continue}}}a.valid=!1}else a.valid=!1}return a}function h(n){var t=c.getSymbolAtLocation(n);return t&&e.getSymbolTarget(t,c)}}function p(n,t){var r=e.getContainingObjectLiteralElement(n);if(r){var i=t.getContextualTypeForObjectLiteralElement(r),a=null===i||void 0===i?void 0:i.getSymbol();if(a&&!(6&e.getCheckFlags(a)))return a}}function f(n){var t=n.node;return e.isImportSpecifier(t.parent)||e.isImportClause(t.parent)||e.isImportEqualsDeclaration(t.parent)||e.isNamespaceImport(t.parent)?t:e.isExportSpecifier(t.parent)||e.isExportAssignment(t.parent)?t:void 0}function m(n){if(e.isDeclaration(n.node.parent))return n.node}function _(n){if(n.node.parent){var t=n.node,r=t.parent;switch(r.kind){case 206:case 207:var i=e.tryCast(r,e.isCallOrNewExpression);if(i&&i.expression===t)return i;break;case 204:var a=e.tryCast(r,e.isPropertyAccessExpression);if(a&&a.parent&&a.name===t){var o=e.tryCast(a.parent,e.isCallOrNewExpression);if(o&&o.expression===a)return o}break;case 205:var s=e.tryCast(r,e.isElementAccessExpression);if(s&&s.parent&&s.argumentExpression===t){var c=e.tryCast(s.parent,e.isCallOrNewExpression);if(c&&c.expression===s)return c}break}}}function g(n){if(n.node.parent){var t=n.node,r=t.parent;switch(r.kind){case 204:var i=e.tryCast(r,e.isPropertyAccessExpression);if(i&&i.expression===t)return i;break;case 205:var a=e.tryCast(r,e.isElementAccessExpression);if(a&&a.expression===t)return a;break}}}function y(n){var t=n.node;if(2===e.getMeaningFromLocation(t)||e.isExpressionWithTypeArgumentsInClassExtendsClause(t.parent))return t}function h(n,t,r){var i=e.getTouchingToken(n,t),a=e.getContainingFunctionDeclaration(i);if(!v(i))return!(a&&E(a,r)&&e.rangeContainsRange(a,i))||a.body&&e.rangeContainsRange(a.body,i)?void 0:a}function v(n){var t=e.findAncestor(n,e.isJSDocNode);if(t){var r=e.findAncestor(t,function(n){return!e.isJSDocNode(n)});return!!r&&e.isFunctionLikeDeclaration(r)}return!1}function b(n){return e.isMethodSignature(n)&&(e.isInterfaceDeclaration(n.parent)||e.isTypeLiteralNode(n.parent))}function E(n,t){var r;if(!x(n.parameters,t))return!1;switch(n.kind){case 254:return S(n)&&T(n,t);case 167:if(e.isObjectLiteralExpression(n.parent)){var i=p(n.name,t);return 1===(null===(r=null===i||void 0===i?void 0:i.declarations)||void 0===r?void 0:r.length)&&T(n,t)}return T(n,t);case 169:return e.isClassDeclaration(n.parent)?S(n.parent)&&T(n,t):D(n.parent.parent)&&T(n,t);case 211:case 212:return D(n.parent)}return!1}function T(e,n){return!!e.body&&!n.isImplementationOfOverload(e)}function S(n){if(!n.name){var t=e.findModifier(n,88);return!!t}return!0}function x(n,t){return C(n)>=i&&e.every(n,function(e){return L(e,t)})}function L(n,t){if(e.isRestParameter(n)){var r=t.getTypeAtLocation(n);if(!t.isArrayType(r)&&!t.isTupleType(r))return!1}return!n.modifiers&&!n.decorators&&e.isIdentifier(n.name)}function D(n){return e.isVariableDeclaration(n)&&e.isVarConst(n)&&e.isIdentifier(n.name)&&!n.type}function A(n){return n.length>0&&e.isThis(n[0].name)}function C(e){return A(e)?e.length-1:e.length}function I(n){return A(n)&&(n=e.factory.createNodeArray(n.slice(1),n.hasTrailingComma)),n}function N(n,t){return e.isIdentifier(t)&&e.getTextOfIdentifierOrLiteral(t)===n?e.factory.createShorthandPropertyAssignment(n):e.factory.createPropertyAssignment(n,t)}function k(n,t){var r=I(n.parameters),i=e.isRestParameter(e.last(r)),a=i?t.slice(0,r.length-1):t,o=e.map(a,function(n,t){var i=O(r[t]),a=N(i,n);return e.suppressLeadingAndTrailingTrivia(a.name),e.isPropertyAssignment(a)&&e.suppressLeadingAndTrailingTrivia(a.initializer),e.copyComments(n,a),a});if(i&&t.length>=r.length){var s=t.slice(r.length-1),c=e.factory.createPropertyAssignment(O(e.last(r)),e.factory.createArrayLiteralExpression(s));o.push(c)}var l=e.factory.createObjectLiteralExpression(o,!1);return l}function R(n,t,r){var i,a=t.getTypeChecker(),o=I(n.parameters),s=e.map(o,f),c=e.factory.createObjectBindingPattern(s),l=m(o);e.every(o,y)&&(i=e.factory.createObjectLiteralExpression());var u=e.factory.createParameterDeclaration(void 0,void 0,void 0,c,void 0,l,i);if(A(n.parameters)){var d=n.parameters[0],p=e.factory.createParameterDeclaration(void 0,void 0,void 0,d.name,void 0,d.type);return e.suppressLeadingAndTrailingTrivia(p.name),e.copyComments(d.name,p.name),d.type&&(e.suppressLeadingAndTrailingTrivia(p.type),e.copyComments(d.type,p.type)),e.factory.createNodeArray([p,u])}return e.factory.createNodeArray([u]);function f(n){var t=e.factory.createBindingElement(void 0,void 0,O(n),e.isRestParameter(n)&&y(n)?e.factory.createArrayLiteralExpression():n.initializer);return e.suppressLeadingAndTrailingTrivia(t),n.initializer&&t.initializer&&e.copyComments(n.initializer,t.initializer),t}function m(n){var t=e.map(n,_),r=e.addEmitFlags(e.factory.createTypeLiteralNode(t),1);return r}function _(n){var t=n.type;t||!n.initializer&&!e.isRestParameter(n)||(t=g(n));var r=e.factory.createPropertySignature(void 0,O(n),y(n)?e.factory.createToken(57):n.questionToken,t);return e.suppressLeadingAndTrailingTrivia(r),e.copyComments(n.name,r.name),n.type&&r.type&&e.copyComments(n.type,r.type),r}function g(n){var i=a.getTypeAtLocation(n);return e.getTypeNodeIfAccessible(i,n,t,r)}function y(n){if(e.isRestParameter(n)){var t=a.getTypeAtLocation(n);return!a.isTupleType(t)}return a.isOptionalParameter(n)}}function O(n){return e.getTextOfIdentifierOrLiteral(n.name)}function w(n){switch(n.parent.kind){case 255:var t=n.parent;if(t.name)return[t.name];var r=e.Debug.checkDefined(e.findModifier(t,88),"Nameless class declaration should be a default export");return[r];case 224:var i=n.parent,a=n.parent.parent,o=i.name;return o?[o,a.name]:[a.name]}}function P(n){switch(n.kind){case 254:if(n.name)return[n.name];var t=e.Debug.checkDefined(e.findModifier(n,88),"Nameless function declaration should be a default export");return[t];case 167:return[n.name];case 169:var r=e.Debug.checkDefined(e.findChildOfKind(n,133,n.getSourceFile()),"Constructor declaration should have constructor keyword");if(224===n.parent.kind){var i=n.parent.parent;return[i.name,r]}return[r];case 212:return[n.parent.name];case 211:return n.name?[n.name,n.parent.name]:[n.parent.name];default:return e.Debug.assertNever(n,"Unexpected function declaration kind "+n.kind)}}n.registerRefactor(r,{kinds:[s.kind],getEditsForAction:l,getAvailableActions:c})})(n.convertParamsToDestructuredObject||(n.convertParamsToDestructuredObject={}))})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){(function(t){var r="Convert to template string",i=e.getLocaleSpecificMessage(e.Diagnostics.Convert_to_template_string),a={name:r,description:i,kind:"refactor.rewrite.string"};function s(n){var t=n.file,s=n.startPosition,l=c(t,s),u=p(l),d={name:r,description:i,actions:[]};return e.isBinaryExpression(u)&&f(u).isValidConcatenation?(d.actions.push(a),[d]):n.preferences.provideRefactorNotApplicableReason?(d.actions.push(o(o({},a),{notApplicableReason:e.getLocaleSpecificMessage(e.Diagnostics.Can_only_convert_string_concatenation)})),[d]):e.emptyArray}function c(n,t){var r=e.getTokenAtPosition(n,t),i=p(r),a=!f(i).isValidConcatenation;return a&&e.isParenthesizedExpression(i.parent)&&e.isBinaryExpression(i.parent.parent)?i.parent.parent:r}function l(n,t){var r=n.file,a=n.startPosition,o=c(r,a);switch(t){case i:return{edits:u(n,o)};default:return e.Debug.fail("invalid action")}}function u(n,t){var r=p(t),i=n.file,a=v(f(r),i),o=e.getTrailingCommentRanges(i.text,r.end);if(o){var s=o[o.length-1],c={pos:o[0].pos,end:s.end};return e.textChanges.ChangeTracker.with(n,function(e){e.deleteRange(i,c),e.replaceNode(i,r,a)})}return e.textChanges.ChangeTracker.with(n,function(e){return e.replaceNode(i,r,a)})}function d(e){return 63!==e.operatorToken.kind}function p(n){var t=e.findAncestor(n.parent,function(n){switch(n.kind){case 204:case 205:return!1;case 221:case 219:return!(e.isBinaryExpression(n.parent)&&d(n.parent));default:return"quit"}});return t||n}function f(n){var t=function(n){if(!e.isBinaryExpression(n))return{nodes:[n],operators:[],validOperators:!0,hasString:e.isStringLiteral(n)||e.isNoSubstitutionTemplateLiteral(n)};var r=t(n.left),i=r.nodes,a=r.operators,o=r.hasString,s=r.validOperators;if(!(o||e.isStringLiteral(n.right)||e.isTemplateExpression(n.right)))return{nodes:[n],operators:[],hasString:!1,validOperators:!0};var c=39===n.operatorToken.kind,l=s&&c;return i.push(n.right),a.push(n.operatorToken),{nodes:i,operators:a,hasString:!0,validOperators:l}},r=t(n),i=r.nodes,a=r.operators,o=r.validOperators,s=r.hasString;return{nodes:i,operators:a,isValidConcatenation:o&&s}}n.registerRefactor(r,{kinds:[a.kind],getEditsForAction:l,getAvailableActions:s});var m=function(n,t){return function(r,i){r<n.length&&e.copyTrailingComments(n[r],i,t,3,!1)}},_=function(n,t,r){return function(i,a){while(i.length>0){var o=i.shift();e.copyTrailingComments(n[o],a,t,3,!1),r(o,a)}}};function g(e){return e.replace(/\\.|[$`]/g,function(e){return"\\"===e[0]?e:"\\"+e})}function y(n){var t=e.isTemplateHead(n)||e.isTemplateMiddle(n)?-2:-1;return e.getTextOfNode(n).slice(1,t)}function h(n,t){var r=[],i="",a="";while(n<t.length){var o=t[n];if(!e.isStringLiteralLike(o)){if(e.isTemplateExpression(o)){i+=o.head.text,a+=y(o.head);break}break}i+=o.text,a+=g(e.getTextOfNode(o).slice(1,-1)),r.push(n),n++}return[n,i,a,r]}function v(n,t){var r=n.nodes,i=n.operators,a=m(i,t),o=_(r,t,a),s=h(0,r),c=s[0],l=s[1],u=s[2],d=s[3];if(c===r.length){var p=e.factory.createNoSubstitutionTemplateLiteral(l,u);return o(d,p),p}var f=[],g=e.factory.createTemplateHead(l,u);o(d,g);for(var v,T=function(n){var t=E(r[n]);a(n,t);var i=h(n+1,r),s=i[0],c=i[1],l=i[2],u=i[3];n=s-1;var d=n===r.length-1;if(e.isTemplateExpression(t)){var p=e.map(t.templateSpans,function(n,r){b(n);var i=r===t.templateSpans.length-1,a=n.literal.text+(i?c:""),o=y(n.literal)+(i?l:"");return e.factory.createTemplateSpan(n.expression,d?e.factory.createTemplateTail(a,o):e.factory.createTemplateMiddle(a,o))});f.push.apply(f,p)}else{var m=d?e.factory.createTemplateTail(c,l):e.factory.createTemplateMiddle(c,l);o(u,m),f.push(e.factory.createTemplateSpan(t,m))}v=n},S=c;S<r.length;S++)T(S),S=v;return e.factory.createTemplateExpression(g,f)}function b(n){var t=n.getSourceFile();e.copyTrailingComments(n,n.expression,t,3,!1),e.copyTrailingAsLeadingComments(n.expression,n.expression,t,3,!1)}function E(n){return e.isParenthesizedExpression(n)&&(b(n),n=n.expression),n}})(n.convertStringOrTemplateLiteral||(n.convertStringOrTemplateLiteral={}))})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){(function(t){var r="Convert arrow function or function expression",i=e.getLocaleSpecificMessage(e.Diagnostics.Convert_arrow_function_or_function_expression),a={name:"Convert to anonymous function",description:e.getLocaleSpecificMessage(e.Diagnostics.Convert_to_anonymous_function),kind:"refactor.rewrite.function.anonymous"},s={name:"Convert to named function",description:e.getLocaleSpecificMessage(e.Diagnostics.Convert_to_named_function),kind:"refactor.rewrite.function.named"},c={name:"Convert to arrow function",description:e.getLocaleSpecificMessage(e.Diagnostics.Convert_to_arrow_function),kind:"refactor.rewrite.function.arrow"};function l(t){var l=t.file,u=t.startPosition,d=t.program,f=t.kind,m=p(l,u,d);if(!m)return e.emptyArray;var _=m.selectedVariableDeclaration,g=m.func,y=[],h=[];if(n.refactorKindBeginsWith(s.kind,f)){var v=_||e.isArrowFunction(g)&&e.isVariableDeclaration(g.parent)?void 0:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_convert_to_named_function);v?h.push(o(o({},s),{notApplicableReason:v})):y.push(s)}if(n.refactorKindBeginsWith(a.kind,f)){v=!_&&e.isArrowFunction(g)?void 0:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_convert_to_anonymous_function);v?h.push(o(o({},a),{notApplicableReason:v})):y.push(a)}if(n.refactorKindBeginsWith(c.kind,f)){v=e.isFunctionExpression(g)?void 0:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_convert_to_arrow_function);v?h.push(o(o({},c),{notApplicableReason:v})):y.push(c)}return[{name:r,description:i,actions:0===y.length&&t.preferences.provideRefactorNotApplicableReason?h:y}]}function u(n,t){var r=n.file,i=n.startPosition,o=n.program,l=p(r,i,o);if(l){var u=l.func,d=[];switch(t){case a.name:d.push.apply(d,y(n,u));break;case s.name:var f=g(u);if(!f)return;d.push.apply(d,h(n,u,f));break;case c.name:if(!e.isFunctionExpression(u))return;d.push.apply(d,v(n,u));break;default:return e.Debug.fail("invalid action")}return{renameFilename:void 0,renameLocation:void 0,edits:d}}}function d(n){var t=!1;return n.forEachChild(function n(r){e.isThis(r)?t=!0:e.isClassLike(r)||e.isFunctionDeclaration(r)||e.isFunctionExpression(r)||e.forEachChild(r,n)}),t}function p(n,t,r){var i=e.getTokenAtPosition(n,t),a=r.getTypeChecker(),o=m(n,a,i.parent);if(o&&!d(o.body)&&!a.containsArgumentsReference(o))return{selectedVariableDeclaration:!0,func:o};var s=e.getContainingFunction(i);if(s&&(e.isFunctionExpression(s)||e.isArrowFunction(s))&&!e.rangeContainsRange(s.body,i)&&!d(s.body)&&!a.containsArgumentsReference(s)){if(e.isFunctionExpression(s)&&E(n,a,s))return;return{selectedVariableDeclaration:!1,func:s}}}function f(n){return e.isVariableDeclaration(n)||e.isVariableDeclarationList(n)&&1===n.declarations.length}function m(n,t,r){if(f(r)){var i=e.isVariableDeclaration(r)?r:e.first(r.declarations),a=i.initializer;return a&&(e.isArrowFunction(a)||e.isFunctionExpression(a)&&!E(n,t,a))?a:void 0}}function _(n){if(e.isExpression(n)){var t=e.factory.createReturnStatement(n),r=n.getSourceFile();return e.suppressLeadingAndTrailingTrivia(t),e.copyTrailingAsLeadingComments(n,t,r,void 0,!0),e.factory.createBlock([t],!0)}return n}function g(n){var t=n.parent;if(e.isVariableDeclaration(t)&&e.isVariableDeclarationInVariableStatement(t)){var r=t.parent,i=r.parent;if(e.isVariableDeclarationList(r)&&e.isVariableStatement(i)&&e.isIdentifier(t.name))return{variableDeclaration:t,variableDeclarationList:r,statement:i,name:t.name}}}function y(n,t){var r=n.file,i=_(t.body),a=e.factory.createFunctionExpression(t.modifiers,t.asteriskToken,void 0,t.typeParameters,t.parameters,t.type,i);return e.textChanges.ChangeTracker.with(n,function(e){return e.replaceNode(r,t,a)})}function h(n,t,r){var i=n.file,a=_(t.body),o=r.variableDeclaration,s=r.variableDeclarationList,c=r.statement,l=r.name;e.suppressLeadingTrivia(c);var u=1&e.getCombinedModifierFlags(o)|e.getEffectiveModifierFlags(t),d=e.factory.createModifiersFromModifierFlags(u),p=e.factory.createFunctionDeclaration(t.decorators,e.length(d)?d:void 0,t.asteriskToken,l,t.typeParameters,t.parameters,t.type,a);return 1===s.declarations.length?e.textChanges.ChangeTracker.with(n,function(e){return e.replaceNode(i,c,p)}):e.textChanges.ChangeTracker.with(n,function(e){e.delete(i,o),e.insertNodeAfter(i,c,p)})}function v(n,t){var r,i=n.file,a=t.body.statements,o=a[0];b(t.body,o)?(r=o.expression,e.suppressLeadingAndTrailingTrivia(r),e.copyComments(o,r)):r=t.body;var s=e.factory.createArrowFunction(t.modifiers,t.typeParameters,t.parameters,t.type,e.factory.createToken(38),r);return e.textChanges.ChangeTracker.with(n,function(e){return e.replaceNode(i,t,s)})}function b(n,t){return 1===n.statements.length&&e.isReturnStatement(t)&&!!t.expression}function E(n,t,r){return!!r.name&&e.FindAllReferences.Core.isSymbolReferencedInFile(r.name,t,n)}n.registerRefactor(r,{kinds:[a.kind,s.kind,c.kind],getEditsForAction:u,getAvailableActions:l})})(n.convertArrowFunctionOrFunctionExpression||(n.convertArrowFunctionOrFunctionExpression={}))})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){(function(n){(function(t){var r="Infer function return type",i=e.Diagnostics.Infer_function_return_type.message,a={name:r,description:i,kind:"refactor.rewrite.function.returnType"};function s(t){var r=u(t);if(r&&!n.isRefactorErrorInfo(r)){var i=e.textChanges.ChangeTracker.with(t,function(e){return l(t.file,e,r.declaration,r.returnTypeNode)});return{renameFilename:void 0,renameLocation:void 0,edits:i}}}function c(t){var s=u(t);return s?n.isRefactorErrorInfo(s)?t.preferences.provideRefactorNotApplicableReason?[{name:r,description:i,actions:[o(o({},a),{notApplicableReason:s.error})]}]:e.emptyArray:[{name:r,description:i,actions:[a]}]:e.emptyArray}function l(n,t,r,i){var a=e.findChildOfKind(r,21,n),o=e.isArrowFunction(r)&&void 0===a,s=o?e.first(r.parameters):a;s&&(o&&(t.insertNodeBefore(n,s,e.factory.createToken(20)),t.insertNodeAfter(n,s,e.factory.createToken(21))),t.insertNodeAt(n,s.end,i,{prefix:": "}))}function u(t){if(!e.isInJSFile(t.file)&&n.refactorKindBeginsWith(a.kind,t.kind)){var r=e.getTokenAtPosition(t.file,t.startPosition),i=e.findAncestor(r,function(n){return e.isBlock(n)||n.parent&&e.isArrowFunction(n.parent)&&(38===n.kind||n.parent.body===n)?"quit":d(n)});if(!i||!i.body||i.type)return{error:e.getLocaleSpecificMessage(e.Diagnostics.Return_type_must_be_inferred_from_a_function)};var o=t.program.getTypeChecker(),s=p(o,i);if(!s)return{error:e.getLocaleSpecificMessage(e.Diagnostics.Could_not_determine_function_return_type)};var c=o.typeToTypeNode(s,i,1);return c?{declaration:i,returnTypeNode:c}:void 0}}function d(e){switch(e.kind){case 254:case 211:case 212:case 167:return!0;default:return!1}}function p(n,t){if(n.isImplementationOfOverload(t)){var r=n.getTypeAtLocation(t).getCallSignatures();if(r.length>1)return n.getUnionType(e.mapDefined(r,function(e){return e.getReturnType()}))}var i=n.getSignatureFromDeclaration(t);if(i)return n.getReturnTypeOfSignature(i)}n.registerRefactor(r,{kinds:[a.kind],getEditsForAction:s,getAvailableActions:c})})(n.inferFunctionReturnType||(n.inferFunctionReturnType={}))})(e.refactor||(e.refactor={}))}(d||(d={})),function(e){function n(n,r,i,a){var o=e.isNodeKind(n)?new t(n,r,i):79===n?new f(79,r,i):80===n?new m(80,r,i):new p(n,r,i);return o.parent=a,o.flags=25358336&a.flags,o}e.servicesVersion="0.8";var t=function(){function n(e,n,t){this.pos=n,this.end=t,this.flags=0,this.modifierFlagsCache=0,this.transformFlags=0,this.parent=void 0,this.kind=e}return n.prototype.assertHasRealPosition=function(n){e.Debug.assert(!e.positionIsSynthesized(this.pos)&&!e.positionIsSynthesized(this.end),n||"Node must have a real position for this operation")},n.prototype.getSourceFile=function(){return e.getSourceFileOfNode(this)},n.prototype.getStart=function(n,t){return this.assertHasRealPosition(),e.getTokenPosOfNode(this,n,t)},n.prototype.getFullStart=function(){return this.assertHasRealPosition(),this.pos},n.prototype.getEnd=function(){return this.assertHasRealPosition(),this.end},n.prototype.getWidth=function(e){return this.assertHasRealPosition(),this.getEnd()-this.getStart(e)},n.prototype.getFullWidth=function(){return this.assertHasRealPosition(),this.end-this.pos},n.prototype.getLeadingTriviaWidth=function(e){return this.assertHasRealPosition(),this.getStart(e)-this.pos},n.prototype.getFullText=function(e){return this.assertHasRealPosition(),(e||this.getSourceFile()).text.substring(this.pos,this.end)},n.prototype.getText=function(e){return this.assertHasRealPosition(),e||(e=this.getSourceFile()),e.text.substring(this.getStart(e),this.getEnd())},n.prototype.getChildCount=function(e){return this.getChildren(e).length},n.prototype.getChildAt=function(e,n){return this.getChildren(n)[e]},n.prototype.getChildren=function(e){return this.assertHasRealPosition("Node without a real position cannot be scanned and thus has no token nodes - use forEachChild and collect the result if that's fine"),this._children||(this._children=i(this,e))},n.prototype.getFirstToken=function(n){this.assertHasRealPosition();var t=this.getChildren(n);if(t.length){var r=e.find(t,function(e){return e.kind<304||e.kind>342});return r.kind<159?r:r.getFirstToken(n)}},n.prototype.getLastToken=function(n){this.assertHasRealPosition();var t=this.getChildren(n),r=e.lastOrUndefined(t);if(r)return r.kind<159?r:r.getLastToken(n)},n.prototype.forEachChild=function(n,t){return e.forEachChild(this,n,t)},n}();function i(n,t){if(!e.isNodeKind(n.kind))return e.emptyArray;var r=[];if(e.isJSDocCommentContainingNode(n))return n.forEachChild(function(e){r.push(e)}),r;e.scanner.setText((t||n.getSourceFile()).text);var i=n.pos,a=function(e){s(r,i,e.pos,n),r.push(e),i=e.end},o=function(e){s(r,i,e.pos,n),r.push(c(e,n)),i=e.end};return e.forEach(n.jsDoc,a),i=n.pos,n.forEachChild(a,o),s(r,i,n.end,n),e.scanner.setText(void 0),r}function s(t,r,i,a){e.scanner.setTextPos(r);while(r<i){var o=e.scanner.scan(),s=e.scanner.getTextPos();if(s<=i&&(79===o&&e.Debug.fail("Did not expect "+e.Debug.formatSyntaxKind(a.kind)+" to have an Identifier in its trivia"),t.push(n(o,r,s,a))),r=s,1===o)break}}function c(e,t){var r=n(343,e.pos,e.end,t);r._children=[];for(var i=e.pos,a=0,o=e;a<o.length;a++){var c=o[a];s(r._children,i,c.pos,t),r._children.push(c),i=c.end}return s(r._children,i,e.end,t),r}var l=function(){function n(e,n){this.pos=e,this.end=n,this.flags=0,this.modifierFlagsCache=0,this.transformFlags=0,this.parent=void 0}return n.prototype.getSourceFile=function(){return e.getSourceFileOfNode(this)},n.prototype.getStart=function(n,t){return e.getTokenPosOfNode(this,n,t)},n.prototype.getFullStart=function(){return this.pos},n.prototype.getEnd=function(){return this.end},n.prototype.getWidth=function(e){return this.getEnd()-this.getStart(e)},n.prototype.getFullWidth=function(){return this.end-this.pos},n.prototype.getLeadingTriviaWidth=function(e){return this.getStart(e)-this.pos},n.prototype.getFullText=function(e){return(e||this.getSourceFile()).text.substring(this.pos,this.end)},n.prototype.getText=function(e){return e||(e=this.getSourceFile()),e.text.substring(this.getStart(e),this.getEnd())},n.prototype.getChildCount=function(){return this.getChildren().length},n.prototype.getChildAt=function(e){return this.getChildren()[e]},n.prototype.getChildren=function(){return 1===this.kind&&this.jsDoc||e.emptyArray},n.prototype.getFirstToken=function(){},n.prototype.getLastToken=function(){},n.prototype.forEachChild=function(){},n}(),d=function(){function n(e,n){this.flags=e,this.escapedName=n}return n.prototype.getFlags=function(){return this.flags},Object.defineProperty(n.prototype,"name",{get:function(){return e.symbolName(this)},enumerable:!1,configurable:!0}),n.prototype.getEscapedName=function(){return this.escapedName},n.prototype.getName=function(){return this.name},n.prototype.getDeclarations=function(){return this.declarations},n.prototype.getDocumentationComment=function(n){if(!this.documentationComment)if(this.documentationComment=e.emptyArray,!this.declarations&&this.target&&this.target.tupleLabelDeclaration){var t=this.target.tupleLabelDeclaration;this.documentationComment=v([t],n)}else this.documentationComment=v(this.declarations,n);return this.documentationComment},n.prototype.getContextualDocumentationComment=function(n,t){switch(null===n||void 0===n?void 0:n.kind){case 170:return this.contextualGetAccessorDocumentationComment||(this.contextualGetAccessorDocumentationComment=e.emptyArray,this.contextualGetAccessorDocumentationComment=v(e.filter(this.declarations,e.isGetAccessor),t)),this.contextualGetAccessorDocumentationComment;case 171:return this.contextualSetAccessorDocumentationComment||(this.contextualSetAccessorDocumentationComment=e.emptyArray,this.contextualSetAccessorDocumentationComment=v(e.filter(this.declarations,e.isSetAccessor),t)),this.contextualSetAccessorDocumentationComment;default:return this.getDocumentationComment(t)}},n.prototype.getJsDocTags=function(n){return void 0===this.tags&&(this.tags=e.JsDoc.getJsDocTagsFromDeclarations(this.declarations,n)),this.tags},n}(),p=function(e){function n(n,t,r){var i=e.call(this,t,r)||this;return i.kind=n,i}return u(n,e),n}(l),f=function(n){function t(e,t,r){var i=n.call(this,t,r)||this;return i.kind=79,i}return u(t,n),Object.defineProperty(t.prototype,"text",{get:function(){return e.idText(this)},enumerable:!1,configurable:!0}),t}(l);f.prototype.kind=79;var m=function(n){function t(e,t,r){return n.call(this,t,r)||this}return u(t,n),Object.defineProperty(t.prototype,"text",{get:function(){return e.idText(this)},enumerable:!1,configurable:!0}),t}(l);m.prototype.kind=80;var _=function(){function n(e,n){this.checker=e,this.flags=n}return n.prototype.getFlags=function(){return this.flags},n.prototype.getSymbol=function(){return this.symbol},n.prototype.getProperties=function(){return this.checker.getPropertiesOfType(this)},n.prototype.getProperty=function(e){return this.checker.getPropertyOfType(this,e)},n.prototype.getApparentProperties=function(){return this.checker.getAugmentedPropertiesOfType(this)},n.prototype.getCallSignatures=function(){return this.checker.getSignaturesOfType(this,0)},n.prototype.getConstructSignatures=function(){return this.checker.getSignaturesOfType(this,1)},n.prototype.getStringIndexType=function(){return this.checker.getIndexTypeOfType(this,0)},n.prototype.getNumberIndexType=function(){return this.checker.getIndexTypeOfType(this,1)},n.prototype.getBaseTypes=function(){return this.isClassOrInterface()?this.checker.getBaseTypes(this):void 0},n.prototype.isNullableType=function(){return this.checker.isNullableType(this)},n.prototype.getNonNullableType=function(){return this.checker.getNonNullableType(this)},n.prototype.getNonOptionalType=function(){return this.checker.getNonOptionalType(this)},n.prototype.getConstraint=function(){return this.checker.getBaseConstraintOfType(this)},n.prototype.getDefault=function(){return this.checker.getDefaultFromTypeParameter(this)},n.prototype.isUnion=function(){return!!(1048576&this.flags)},n.prototype.isIntersection=function(){return!!(2097152&this.flags)},n.prototype.isUnionOrIntersection=function(){return!!(3145728&this.flags)},n.prototype.isLiteral=function(){return!!(384&this.flags)},n.prototype.isStringLiteral=function(){return!!(128&this.flags)},n.prototype.isNumberLiteral=function(){return!!(256&this.flags)},n.prototype.isTypeParameter=function(){return!!(262144&this.flags)},n.prototype.isClassOrInterface=function(){return!!(3&e.getObjectFlags(this))},n.prototype.isClass=function(){return!!(1&e.getObjectFlags(this))},Object.defineProperty(n.prototype,"typeArguments",{get:function(){if(4&e.getObjectFlags(this))return this.checker.getTypeArguments(this)},enumerable:!1,configurable:!0}),n}(),g=function(){function n(e,n){this.checker=e,this.flags=n}return n.prototype.getDeclaration=function(){return this.declaration},n.prototype.getTypeParameters=function(){return this.typeParameters},n.prototype.getParameters=function(){return this.parameters},n.prototype.getReturnType=function(){return this.checker.getReturnTypeOfSignature(this)},n.prototype.getDocumentationComment=function(){return this.documentationComment||(this.documentationComment=v(e.singleElementArray(this.declaration),this.checker))},n.prototype.getJsDocTags=function(){return void 0===this.jsDocTags&&(this.jsDocTags=this.declaration?h(this.declaration,this.checker):[]),this.jsDocTags},n}();function y(n){return e.getJSDocTags(n).some(function(e){return"inheritDoc"===e.tagName.text})}function h(n,t){var r=e.JsDoc.getJsDocTagsFromDeclarations([n],t);if(0===r.length||y(n)){var i=b(t,n,function(e){var n;return 1===(null===(n=e.declarations)||void 0===n?void 0:n.length)?e.getJsDocTags():void 0});i&&(r=a(a([],i,!0),r,!0))}return r}function v(n,t){if(!n)return e.emptyArray;var r=e.JsDoc.getJsDocCommentsFromDeclarations(n,t);if(t&&(0===r.length||n.some(y)))for(var i=new e.Set,a=0,o=n;a<o.length;a++){var s=o[a],c=b(t,s,function(e){if(!i.has(e))return i.add(e),e.getDocumentationComment(t)});c&&(r=0===r.length?c.slice():c.concat(e.lineBreakPart(),r))}return r}function b(n,t,r){var i,a=169===(null===(i=t.parent)||void 0===i?void 0:i.kind)?t.parent.parent:t.parent;if(a)return e.firstDefined(e.getAllSuperTypeNodes(a),function(e){var i=n.getPropertyOfType(n.getTypeAtLocation(e),t.symbol.name);return i?r(i):void 0})}var E=function(n){function t(e,t,r){var i=n.call(this,e,t,r)||this;return i.kind=300,i}return u(t,n),t.prototype.update=function(n,t){return e.updateSourceFile(this,n,t)},t.prototype.getLineAndCharacterOfPosition=function(n){return e.getLineAndCharacterOfPosition(this,n)},t.prototype.getLineStarts=function(){return e.getLineStarts(this)},t.prototype.getPositionOfLineAndCharacter=function(n,t,r){return e.computePositionOfLineAndCharacter(e.getLineStarts(this),n,t,this.text,r)},t.prototype.getLineEndOfPosition=function(e){var n,t=this.getLineAndCharacterOfPosition(e).line,r=this.getLineStarts();t+1>=r.length&&(n=this.getEnd()),n||(n=r[t+1]-1);var i=this.getFullText();return"\n"===i[n]&&"\r"===i[n-1]?n-1:n},t.prototype.getNamedDeclarations=function(){return this.namedDeclarations||(this.namedDeclarations=this.computeNamedDeclarations()),this.namedDeclarations},t.prototype.computeNamedDeclarations=function(){var n=e.createMultiMap();return this.forEachChild(a),n;function t(e){var t=i(e);t&&n.add(t,e)}function r(e){var t=n.get(e);return t||n.set(e,t=[]),t}function i(n){var t=e.getNonAssignedNameOfDeclaration(n);return t&&(e.isComputedPropertyName(t)&&e.isPropertyAccessExpression(t.expression)?t.expression.name.text:e.isPropertyName(t)?e.getNameFromPropertyName(t):void 0)}function a(n){switch(n.kind){case 254:case 211:case 167:case 166:var o=n,s=i(o);if(s){var c=r(s),l=e.lastOrUndefined(c);l&&o.parent===l.parent&&o.symbol===l.symbol?o.body&&!l.body&&(c[c.length-1]=o):c.push(o)}e.forEachChild(n,a);break;case 255:case 224:case 256:case 257:case 258:case 259:case 263:case 273:case 268:case 265:case 266:case 170:case 171:case 180:t(n),e.forEachChild(n,a);break;case 162:if(!e.hasSyntacticModifier(n,16476))break;case 252:case 201:var u=n;if(e.isBindingPattern(u.name)){e.forEachChild(u.name,a);break}u.initializer&&a(u.initializer);case 294:case 165:case 164:t(n);break;case 270:var d=n;d.exportClause&&(e.isNamedExports(d.exportClause)?e.forEach(d.exportClause.elements,a):a(d.exportClause.name));break;case 264:var p=n.importClause;p&&(p.name&&t(p.name),p.namedBindings&&(266===p.namedBindings.kind?t(p.namedBindings):e.forEach(p.namedBindings.elements,a)));break;case 219:0!==e.getAssignmentDeclarationKind(n)&&t(n);default:e.forEachChild(n,a)}}},t}(t),T=function(){function n(e,n,t){this.fileName=e,this.text=n,this.skipTrivia=t}return n.prototype.getLineAndCharacterOfPosition=function(n){return e.getLineAndCharacterOfPosition(this,n)},n}();function S(){return{getNodeConstructor:function(){return t},getTokenConstructor:function(){return p},getIdentifierConstructor:function(){return f},getPrivateIdentifierConstructor:function(){return m},getSourceFileConstructor:function(){return E},getSymbolConstructor:function(){return d},getTypeConstructor:function(){return _},getSignatureConstructor:function(){return g},getSourceMapSourceConstructor:function(){return T}}}function x(n){var t=!0;for(var r in n)if(e.hasProperty(n,r)&&!L(r)){t=!1;break}if(t)return n;var i={};for(var r in n)if(e.hasProperty(n,r)){var a=L(r)?r:r.charAt(0).toLowerCase()+r.substr(1);i[a]=n[r]}return i}function L(e){return!e.length||e.charAt(0)===e.charAt(0).toLowerCase()}function D(n){return n?e.map(n,function(e){return e.text}).join(""):""}function A(){return{target:1,jsx:1}}function C(){return e.codefix.getSupportedErrorCodes()}e.toEditorSettings=x,e.displayPartsToString=D,e.getDefaultCompilerOptions=A,e.getSupportedCodeFixes=C;var I=function(){function n(n,t){this.host=n,this.currentDirectory=n.getCurrentDirectory(),this.fileNameToEntry=new e.Map;for(var r=n.getScriptFileNames(),i=0,a=r;i<a.length;i++){var o=a[i];this.createEntry(o,e.toPath(o,this.currentDirectory,t))}}return n.prototype.createEntry=function(n,t){var r,i=this.host.getScriptSnapshot(n);return r=i?{hostFileName:n,version:this.host.getScriptVersion(n),scriptSnapshot:i,scriptKind:e.getScriptKind(n,this.host)}:n,this.fileNameToEntry.set(t,r),r},n.prototype.getEntryByPath=function(e){return this.fileNameToEntry.get(e)},n.prototype.getHostFileInformation=function(n){var t=this.fileNameToEntry.get(n);return e.isString(t)?void 0:t},n.prototype.getOrCreateEntryByPath=function(n,t){var r=this.getEntryByPath(t)||this.createEntry(n,t);return e.isString(r)?void 0:r},n.prototype.getRootFileNames=function(){var n=[];return this.fileNameToEntry.forEach(function(t){e.isString(t)?n.push(t):n.push(t.hostFileName)}),n},n.prototype.getScriptSnapshot=function(e){var n=this.getHostFileInformation(e);return n&&n.scriptSnapshot},n}(),N=function(){function n(e){this.host=e}return n.prototype.getCurrentSourceFile=function(n){var t=this.host.getScriptSnapshot(n);if(!t)throw new Error("Could not find file: '"+n+"'.");var r,i=e.getScriptKind(n,this.host),a=this.host.getScriptVersion(n);if(this.currentFileName!==n)r=R(n,t,99,a,!0,i);else if(this.currentFileVersion!==a){var o=t.getChangeRange(this.currentFileScriptSnapshot);r=O(this.currentSourceFile,t,a,o)}return r&&(this.currentFileVersion=a,this.currentFileName=n,this.currentFileScriptSnapshot=t,this.currentSourceFile=r),this.currentSourceFile},n}();function k(e,n,t){e.version=t,e.scriptSnapshot=n}function R(n,t,r,i,a,o){var s=e.createSourceFile(n,e.getSnapshotText(t),r,a,o);return k(s,t,i),s}function O(n,t,r,i,a){if(i&&r!==n.version){var o=void 0,s=0!==i.span.start?n.text.substr(0,i.span.start):"",c=e.textSpanEnd(i.span)!==n.text.length?n.text.substr(e.textSpanEnd(i.span)):"";if(0===i.newLength)o=s&&c?s+c:s||c;else{var l=t.getText(i.span.start,i.span.start+i.newLength);o=s&&c?s+l+c:s?s+l:l+c}var u=e.updateSourceFile(n,o,i,a);return k(u,t,r),u.nameTable=void 0,n!==u&&n.scriptSnapshot&&(n.scriptSnapshot.dispose&&n.scriptSnapshot.dispose(),n.scriptSnapshot=void 0),u}return R(n.fileName,t,n.languageVersion,r,!0,n.scriptKind)}e.createLanguageServiceSourceFile=R,e.updateLanguageServiceSourceFile=O;var w={isCancellationRequested:e.returnFalse,throwIfCancellationRequested:e.noop},P=function(){function n(e){this.cancellationToken=e}return n.prototype.isCancellationRequested=function(){return this.cancellationToken.isCancellationRequested()},n.prototype.throwIfCancellationRequested=function(){if(this.isCancellationRequested())throw null===e.tracing||void 0===e.tracing||e.tracing.instant("session","cancellationThrown",{kind:"CancellationTokenObject"}),new e.OperationCanceledException},n}(),M=function(){function n(e,n){void 0===n&&(n=20),this.hostCancellationToken=e,this.throttleWaitMilliseconds=n,this.lastCancellationCheckTime=0}return n.prototype.isCancellationRequested=function(){var n=e.timestamp(),t=Math.abs(n-this.lastCancellationCheckTime);return t>=this.throttleWaitMilliseconds&&(this.lastCancellationCheckTime=n,this.hostCancellationToken.isCancellationRequested())},n.prototype.throwIfCancellationRequested=function(){if(this.isCancellationRequested())throw null===e.tracing||void 0===e.tracing||e.tracing.instant("session","cancellationThrown",{kind:"ThrottledCancellationToken"}),new e.OperationCanceledException},n}();e.ThrottledCancellationToken=M;var F=["getSemanticDiagnostics","getSuggestionDiagnostics","getCompilerOptionsDiagnostics","getSemanticClassifications","getEncodedSemanticClassifications","getCodeFixesAtPosition","getCombinedCodeFix","applyCodeActionCommand","organizeImports","getEditsForFileRename","getEmitOutput","getApplicableRefactors","getEditsForRefactor","prepareCallHierarchy","provideCallHierarchyIncomingCalls","provideCallHierarchyOutgoingCalls","provideInlayHints"],G=a(a([],F,!0),["getCompletionsAtPosition","getCompletionEntryDetails","getCompletionEntrySymbol","getSignatureHelpItems","getQuickInfoAtPosition","getDefinitionAtPosition","getDefinitionAndBoundSpan","getImplementationAtPosition","getTypeDefinitionAtPosition","getReferencesAtPosition","findReferences","getOccurrencesAtPosition","getDocumentHighlights","getNavigateToItems","getRenameInfo","findRenameLocations","getApplicableRefactors"],!1);function B(n,t,r){var i,s;void 0===t&&(t=e.createDocumentRegistry(n.useCaseSensitiveFileNames&&n.useCaseSensitiveFileNames(),n.getCurrentDirectory())),s=void 0===r?e.LanguageServiceMode.Semantic:"boolean"===typeof r?r?e.LanguageServiceMode.Syntactic:e.LanguageServiceMode.Semantic:r;var c,l,u=new N(n),d=0,p=n.getCancellationToken?new P(n.getCancellationToken()):w,f=n.getCurrentDirectory();function m(e){n.log&&n.log(e)}!e.localizedDiagnosticMessages&&n.getLocalizedDiagnosticMessages&&e.setLocalizedDiagnosticMessages(n.getLocalizedDiagnosticMessages());var _=e.hostUsesCaseSensitiveFileNames(n),g=e.createGetCanonicalFileName(_),y=e.getSourceMapper({useCaseSensitiveFileNames:function(){return _},getCurrentDirectory:function(){return f},getProgram:b,fileExists:e.maybeBind(n,n.fileExists),readFile:e.maybeBind(n,n.readFile),getDocumentPositionMapper:e.maybeBind(n,n.getDocumentPositionMapper),getSourceFileLike:e.maybeBind(n,n.getSourceFileLike),log:m});function h(e){var n=c.getSourceFile(e);if(!n){var t=new Error("Could not find source file: '"+e+"'.");throw t.ProgramFiles=c.getSourceFiles().map(function(e){return e.fileName}),t}return n}function v(){var r,i,a;if(e.Debug.assert(s!==e.LanguageServiceMode.Syntactic),n.getProjectVersion){var o=n.getProjectVersion();if(o){if(l===o&&!(null===(r=n.hasChangedAutomaticTypeDirectiveNames)||void 0===r?void 0:r.call(n)))return;l=o}}var u=n.getTypeRootsVersion?n.getTypeRootsVersion():0;d!==u&&(m("TypeRoots version has changed; provide new program"),c=void 0,d=u);var h,v=new I(n,g),b=v.getRootFileNames(),E=n.getCompilationSettings()||A(),T=n.hasInvalidatedResolution||e.returnFalse,S=e.maybeBind(n,n.hasChangedAutomaticTypeDirectiveNames),x=null===(i=n.getProjectReferences)||void 0===i?void 0:i.call(n),L={useCaseSensitiveFileNames:_,fileExists:w,readFile:P,readDirectory:M,trace:e.maybeBind(n,n.trace),getCurrentDirectory:function(){return f},onUnRecoverableConfigFileDiagnostic:e.noop};if(!e.isProgramUptoDate(c,b,E,function(e,t){return n.getScriptVersion(t)},w,T,S,k,x)){var D={getSourceFile:G,getSourceFileByPath:B,getCancellationToken:function(){return p},getCanonicalFileName:g,useCaseSensitiveFileNames:function(){return _},getNewLine:function(){return e.getNewLineCharacter(E,function(){return e.getNewLineOrDefaultFromHost(n)})},getDefaultLibFileName:function(e){return n.getDefaultLibFileName(e)},writeFile:e.noop,getCurrentDirectory:function(){return f},fileExists:w,readFile:P,getSymlinkCache:e.maybeBind(n,n.getSymlinkCache),realpath:e.maybeBind(n,n.realpath),directoryExists:function(t){return e.directoryProbablyExists(t,n)},getDirectories:function(e){return n.getDirectories?n.getDirectories(e):[]},readDirectory:M,onReleaseOldSourceFile:F,onReleaseParsedCommandLine:O,hasInvalidatedResolution:T,hasChangedAutomaticTypeDirectiveNames:S,trace:L.trace,resolveModuleNames:e.maybeBind(n,n.resolveModuleNames),resolveTypeReferenceDirectives:e.maybeBind(n,n.resolveTypeReferenceDirectives),useSourceOfProjectReferenceRedirect:e.maybeBind(n,n.useSourceOfProjectReferenceRedirect),getParsedCommandLine:k};null===(a=n.setCompilerHost)||void 0===a||a.call(n,D);var C=t.getKeyForCompilationSettings(E),N={rootNames:b,options:E,host:D,oldProgram:c,projectReferences:x};return c=e.createProgram(N),v=void 0,h=void 0,y.clearCache(),void c.getTypeChecker()}function k(t){var r=e.toPath(t,f,g),i=null===h||void 0===h?void 0:h.get(r);if(void 0!==i)return i||void 0;var a=n.getParsedCommandLine?n.getParsedCommandLine(t):R(t);return(h||(h=new e.Map)).set(r,a||!1),a}function R(n){var t=G(n,100);if(t)return t.path=e.toPath(n,f,g),t.resolvedPath=t.path,t.originalFileName=t.fileName,e.parseJsonSourceFileConfigFileContent(t,L,e.getNormalizedAbsolutePath(e.getDirectoryPath(n),f),void 0,e.getNormalizedAbsolutePath(n,f))}function O(e,t,r){var i;n.getParsedCommandLine?null===(i=n.onReleaseParsedCommandLine)||void 0===i||i.call(n,e,t,r):t&&F(t.sourceFile,r)}function w(t){var r=e.toPath(t,f,g),i=v&&v.getEntryByPath(r);return i?!e.isString(i):!!n.fileExists&&n.fileExists(t)}function P(t){var r=e.toPath(t,f,g),i=v&&v.getEntryByPath(r);return i?e.isString(i)?void 0:e.getSnapshotText(i.scriptSnapshot):n.readFile&&n.readFile(t)}function M(t,r,i,a,o){return e.Debug.checkDefined(n.readDirectory,"'LanguageServiceHost.readDirectory' must be implemented to correctly process 'projectReferences'"),n.readDirectory(t,r,i,a,o)}function F(e,n){var r=t.getKeyForCompilationSettings(n);t.releaseDocumentWithKey(e.resolvedPath,r,e.scriptKind)}function G(n,t,r,i){return B(n,e.toPath(n,f,g),t,r,i)}function B(n,r,i,a,o){e.Debug.assert(void 0!==v,"getOrCreateSourceFileByPath called after typical CompilerHost lifetime, check the callstack something with a reference to an old host.");var s=v&&v.getOrCreateEntryByPath(n,r);if(s){if(!o){var l=c&&c.getSourceFileByPath(r);if(l){if(s.scriptKind===l.scriptKind)return t.updateDocumentWithKey(n,r,E,C,s.scriptSnapshot,s.version,s.scriptKind);t.releaseDocumentWithKey(l.resolvedPath,t.getKeyForCompilationSettings(c.getCompilerOptions()),l.scriptKind)}}return t.acquireDocumentWithKey(n,r,E,C,s.scriptSnapshot,s.version,s.scriptKind)}}}function b(){if(s!==e.LanguageServiceMode.Syntactic)return v(),c;e.Debug.assert(void 0===c)}function E(){var e;return null===(e=n.getPackageJsonAutoImportProvider)||void 0===e?void 0:e.call(n)}function T(){c=void 0}function S(){if(c){var r=t.getKeyForCompilationSettings(c.getCompilerOptions());e.forEach(c.getSourceFiles(),function(e){return t.releaseDocumentWithKey(e.resolvedPath,r,e.scriptKind)}),c=void 0}n=void 0}function L(e){return v(),c.getSyntacticDiagnostics(h(e),p).slice()}function D(n){v();var t=h(n),r=c.getSemanticDiagnostics(t,p);if(!e.getEmitDeclarations(c.getCompilerOptions()))return r.slice();var i=c.getDeclarationDiagnostics(t,p);return a(a([],r,!0),i,!0)}function C(n){return v(),e.computeSuggestionDiagnostics(h(n),c,p)}function k(){return v(),a(a([],c.getOptionsDiagnostics(p),!0),c.getGlobalDiagnostics(p),!0)}function R(t,r,i){void 0===i&&(i=e.emptyOptions);var a=o(o({},e.identity(i)),{includeCompletionsForModuleExports:i.includeCompletionsForModuleExports||i.includeExternalModuleExports,includeCompletionsWithInsertText:i.includeCompletionsWithInsertText||i.includeInsertTextCompletions});return v(),e.Completions.getCompletionsAtPosition(n,c,m,h(t),r,a,i.triggerCharacter,i.triggerKind,p)}function O(t,r,i,a,o,s,l){return void 0===s&&(s=e.emptyOptions),v(),e.Completions.getCompletionEntryDetails(c,m,h(t),r,{name:i,source:o,data:l},n,a&&e.formatting.getFormatContext(a,n),s,p)}function M(t,r,i,a,o){return void 0===o&&(o=e.emptyOptions),v(),e.Completions.getCompletionEntrySymbol(c,m,h(t),r,{name:i,source:a},n,o)}function B(n,t){v();var r=h(n),i=e.getTouchingPropertyName(r,t);if(i!==r){var a=c.getTypeChecker(),o=U(i),s=W(o,a);if(!s||a.isUnknownSymbol(s)){var l=V(r,o,t)?a.getTypeAtLocation(o):void 0;return l&&{kind:"",kindModifiers:"",textSpan:e.createTextSpanFromNode(o,r),displayParts:a.runWithCancellationToken(p,function(n){return e.typeToDisplayParts(n,l,e.getContainerNode(o))}),documentation:l.symbol?l.symbol.getDocumentationComment(a):void 0,tags:l.symbol?l.symbol.getJsDocTags(a):void 0}}var u=a.runWithCancellationToken(p,function(n){return e.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(n,s,r,e.getContainerNode(o),o)}),d=u.symbolKind,f=u.displayParts,m=u.documentation,_=u.tags;return{kind:d,kindModifiers:e.SymbolDisplay.getSymbolModifiers(a,s),textSpan:e.createTextSpanFromNode(o,r),displayParts:f,documentation:m,tags:_}}}function U(n){return e.isNewExpression(n.parent)&&n.pos===n.parent.pos?n.parent.expression:e.isNamedTupleMember(n.parent)&&n.pos===n.parent.pos?n.parent:n}function V(n,t,r){switch(t.kind){case 79:return!e.isLabelName(t)&&!e.isTagName(t)&&!e.isConstTypeReference(t.parent);case 204:case 159:return!e.isInComment(n,r);case 108:case 190:case 106:case 195:return!0;default:return!1}}function j(n,t){return v(),e.GoToDefinition.getDefinitionAtPosition(c,h(n),t)}function K(n,t){return v(),e.GoToDefinition.getDefinitionAndBoundSpan(c,h(n),t)}function H(n,t){return v(),e.GoToDefinition.getTypeDefinitionAtPosition(c.getTypeChecker(),h(n),t)}function z(n,t){return v(),e.FindAllReferences.getImplementationsAtPosition(c,p,c.getSourceFiles(),h(n),t)}function J(n,t){return e.flatMap(q(n,t,[n]),function(e){return e.highlightSpans.map(function(n){return o(o({fileName:e.fileName,textSpan:n.textSpan,isWriteAccess:"writtenReference"===n.kind,isDefinition:!1},n.isInString&&{isInString:!0}),n.contextSpan&&{contextSpan:n.contextSpan})})})}function q(n,t,r){var i=e.normalizePath(n);e.Debug.assert(r.some(function(n){return e.normalizePath(n)===i})),v();var a=e.mapDefined(r,function(e){return c.getSourceFile(e)}),o=h(n);return e.DocumentHighlights.getDocumentHighlights(c,p,o,t,a)}function X(n,t,r,i,a){v();var s=h(n),c=e.getAdjustedRenameLocation(e.getTouchingPropertyName(s,t));if(e.isIdentifier(c)&&(e.isJsxOpeningElement(c.parent)||e.isJsxClosingElement(c.parent))&&e.isIntrinsicJsxName(c.escapedText)){var l=c.parent.parent,u=l.openingElement,d=l.closingElement;return[u,d].map(function(n){var t=e.createTextSpanFromNode(n.tagName,s);return o({fileName:s.fileName,textSpan:t},e.FindAllReferences.toContextSpan(t,s,n.parent))})}return Q(c,t,{findInStrings:r,findInComments:i,providePrefixAndSuffixTextForRename:a,use:2},function(n,t,r){return e.FindAllReferences.toRenameLocation(n,t,r,a||!1)})}function Y(n,t){return v(),Q(e.getTouchingPropertyName(h(n),t),t,{use:1},e.FindAllReferences.toReferenceEntry)}function Q(n,t,r,i){v();var a=r&&2===r.use?c.getSourceFiles().filter(function(e){return!c.isSourceFileDefaultLibrary(e)}):c.getSourceFiles();return e.FindAllReferences.findReferenceOrRenameEntries(c,p,a,n,t,r,i)}function Z(n,t){return v(),e.FindAllReferences.findReferencedSymbols(c,p,c.getSourceFiles(),h(n),t)}function $(n){return v(),e.FindAllReferences.Core.getReferencesForFileName(n,c,c.getSourceFiles()).map(e.FindAllReferences.toReferenceEntry)}function ee(n,t,r,i){void 0===i&&(i=!1),v();var a=r?[h(r)]:c.getSourceFiles();return e.NavigateTo.getNavigateToItems(a,c.getTypeChecker(),p,n,t,i)}function ne(t,r,i){v();var a=h(t),o=n.getCustomTransformers&&n.getCustomTransformers();return e.getFileEmitOutput(c,a,!!r,p,o,i)}function te(n,t,r){var i=void 0===r?e.emptyOptions:r,a=i.triggerReason;v();var o=h(n);return e.SignatureHelp.getSignatureHelpItems(c,o,t,a,p)}function re(e){return u.getCurrentSourceFile(e)}function ie(n,t,r){var i=u.getCurrentSourceFile(n),a=e.getTouchingPropertyName(i,t);if(a!==i){switch(a.kind){case 204:case 159:case 10:case 95:case 110:case 104:case 106:case 108:case 190:case 79:break;default:return}var o=a;while(1)if(e.isRightSideOfPropertyAccess(o)||e.isRightSideOfQualifiedName(o))o=o.parent;else{if(!e.isNameOfModuleDeclaration(o))break;if(259!==o.parent.parent.kind||o.parent.parent.body!==o.parent)break;o=o.parent.parent.name}return e.createTextSpanFromBounds(o.getStart(),a.getEnd())}}function ae(n,t){var r=u.getCurrentSourceFile(n);return e.BreakpointResolver.spanInSourceFileAtLocation(r,t)}function oe(n){return e.NavigationBar.getNavigationBarItems(u.getCurrentSourceFile(n),p)}function se(n){return e.NavigationBar.getNavigationTree(u.getCurrentSourceFile(n),p)}function ce(n,t,r){v();var i=r||"original";return"2020"===i?e.classifier.v2020.getSemanticClassifications(c,p,h(n),t):e.getSemanticClassifications(c.getTypeChecker(),p,h(n),c.getClassifiableNames(),t)}function le(n,t,r){v();var i=r||"original";return"original"===i?e.getEncodedSemanticClassifications(c.getTypeChecker(),p,h(n),c.getClassifiableNames(),t):e.classifier.v2020.getEncodedSemanticClassifications(c,p,h(n),t)}function ue(n,t){return e.getSyntacticClassifications(p,u.getCurrentSourceFile(n),t)}function de(n,t){return e.getEncodedSyntacticClassifications(p,u.getCurrentSourceFile(n),t)}function pe(n){var t=u.getCurrentSourceFile(n);return e.OutliningElementsCollector.collectElements(t,p)}var fe=new e.Map(e.getEntries((i={},i[18]=19,i[20]=21,i[22]=23,i[31]=29,i)));function me(n,t){var r=u.getCurrentSourceFile(n),i=e.getTouchingToken(r,t),a=i.getStart(r)===t?fe.get(i.kind.toString()):void 0,o=a&&e.findChildOfKind(i.parent,a,r);return o?[e.createTextSpanFromNode(i,r),e.createTextSpanFromNode(o,r)].sort(function(e,n){return e.start-n.start}):e.emptyArray}function _e(n,t,r){var i=e.timestamp(),a=x(r),o=u.getCurrentSourceFile(n);m("getIndentationAtPosition: getCurrentSourceFile: "+(e.timestamp()-i)),i=e.timestamp();var s=e.formatting.SmartIndenter.getIndentation(t,o,a);return m("getIndentationAtPosition: computeIndentation  : "+(e.timestamp()-i)),s}function ge(t,r,i,a){var o=u.getCurrentSourceFile(t);return e.formatting.formatSelection(r,i,o,e.formatting.getFormatContext(x(a),n))}function ye(t,r){return e.formatting.formatDocument(u.getCurrentSourceFile(t),e.formatting.getFormatContext(x(r),n))}function he(t,r,i,a){var o=u.getCurrentSourceFile(t),s=e.formatting.getFormatContext(x(a),n);if(!e.isInComment(o,r))switch(i){case"{":return e.formatting.formatOnOpeningCurly(r,o,s);case"}":return e.formatting.formatOnClosingCurly(r,o,s);case";":return e.formatting.formatOnSemicolon(r,o,s);case"\n":return e.formatting.formatOnEnter(r,o,s)}return[]}function ve(t,r,i,a,o,s){void 0===s&&(s=e.emptyOptions),v();var l=h(t),u=e.createTextSpanFromBounds(r,i),d=e.formatting.getFormatContext(o,n);return e.flatMap(e.deduplicate(a,e.equateValues,e.compareValues),function(t){return p.throwIfCancellationRequested(),e.codefix.getFixes({errorCode:t,sourceFile:l,span:u,program:c,host:n,cancellationToken:p,formatContext:d,preferences:s})})}function be(t,r,i,a){void 0===a&&(a=e.emptyOptions),v(),e.Debug.assert("file"===t.type);var o=h(t.fileName),s=e.formatting.getFormatContext(i,n);return e.codefix.getAllFixes({fixId:r,sourceFile:o,program:c,host:n,cancellationToken:p,formatContext:s,preferences:a})}function Ee(t,r,i){void 0===i&&(i=e.emptyOptions),v(),e.Debug.assert("file"===t.type);var a=h(t.fileName),o=e.formatting.getFormatContext(r,n);return e.OrganizeImports.organizeImports(a,o,n,c,i,t.skipDestructiveCodeActions)}function Te(t,r,i,a){return void 0===a&&(a=e.emptyOptions),e.getEditsForFileRename(b(),t,r,n,e.formatting.getFormatContext(i,n),a,y)}function Se(n,t){var r="string"===typeof n?t:n;return e.isArray(r)?Promise.all(r.map(function(e){return xe(e)})):xe(r)}function xe(t){var r=function(n){return e.toPath(n,f,g)};return e.Debug.assertEqual(t.type,"install package"),n.installPackage?n.installPackage({fileName:r(t.file),packageName:t.packageName}):Promise.reject("Host does not implement `installPackage`")}function Le(t,r,i){return e.JsDoc.getDocCommentTemplateAtPosition(e.getNewLineOrDefaultFromHost(n),u.getCurrentSourceFile(t),r,i)}function De(n,t,r){if(60===r)return!1;var i=u.getCurrentSourceFile(n);if(e.isInString(i,t))return!1;if(e.isInsideJsxElementOrAttribute(i,t))return 123===r;if(e.isInTemplateString(i,t))return!1;switch(r){case 39:case 34:case 96:return!e.isInComment(i,t)}return!0}function Ae(n,t){var r=u.getCurrentSourceFile(n),i=e.findPrecedingToken(t,r);if(i){var a=31===i.kind&&e.isJsxOpeningElement(i.parent)?i.parent.parent:e.isJsxText(i)?i.parent:void 0;return a&&Oe(a)?{newText:"</"+a.openingElement.tagName.getText(r)+">"}:void 0}}function Ce(e,n){return{lineStarts:e.getLineStarts(),firstLine:e.getLineAndCharacterOfPosition(n.pos).line,lastLine:e.getLineAndCharacterOfPosition(n.end).line}}function Ie(n,t,r){for(var i=u.getCurrentSourceFile(n),a=[],o=Ce(i,t),s=o.lineStarts,c=o.firstLine,l=o.lastLine,d=r||!1,p=Number.MAX_VALUE,f=new e.Map,m=new RegExp(/\S/),_=e.isInsideJsxElement(i,s[c]),g=_?"{/*":"//",y=c;y<=l;y++){var h=i.text.substring(s[y],i.getLineEndOfPosition(s[y])),v=m.exec(h);v&&(p=Math.min(p,v.index),f.set(y.toString(),v.index),h.substr(v.index,g.length)!==g&&(d=void 0===r||r))}for(y=c;y<=l;y++)if(c===l||s[y]!==t.end){var b=f.get(y.toString());void 0!==b&&(_?a.push.apply(a,Ne(n,{pos:s[y]+p,end:i.getLineEndOfPosition(s[y])},d,_)):d?a.push({newText:g,span:{length:0,start:s[y]+p}}):i.text.substr(s[y]+b,g.length)===g&&a.push({newText:"",span:{length:g.length,start:s[y]+b}}))}return a}function Ne(n,t,r,i){var a,o=u.getCurrentSourceFile(n),s=[],c=o.text,l=!1,d=r||!1,p=[],f=t.pos,m=void 0!==i?i:e.isInsideJsxElement(o,f),_=m?"{/*":"/*",g=m?"*/}":"*/",y=m?"\\{\\/\\*":"\\/\\*",h=m?"\\*\\/\\}":"\\*\\/";while(f<=t.end){var v=c.substr(f,_.length)===_?_.length:0,b=e.isInComment(o,f+v);if(b)m&&(b.pos--,b.end++),p.push(b.pos),3===b.kind&&p.push(b.end),l=!0,f=b.end+1;else{var E=c.substring(f,t.end).search("("+y+")|("+h+")");d=void 0!==r?r:d||!e.isTextWhiteSpaceLike(c,f,-1===E?t.end:f+E),f=-1===E?t.end+1:f+E+g.length}}if(d||!l){2!==(null===(a=e.isInComment(o,t.pos))||void 0===a?void 0:a.kind)&&e.insertSorted(p,t.pos,e.compareValues),e.insertSorted(p,t.end,e.compareValues);var T=p[0];c.substr(T,_.length)!==_&&s.push({newText:_,span:{length:0,start:T}});for(var S=1;S<p.length-1;S++)c.substr(p[S]-g.length,g.length)!==g&&s.push({newText:g,span:{length:0,start:p[S]}}),c.substr(p[S],_.length)!==_&&s.push({newText:_,span:{length:0,start:p[S]}});s.length%2!==0&&s.push({newText:g,span:{length:0,start:p[p.length-1]}})}else for(var x=0,L=p;x<L.length;x++){var D=L[x],A=D-g.length>0?D-g.length:0;v=c.substr(A,g.length)===g?g.length:0;s.push({newText:"",span:{length:_.length,start:D-v}})}return s}function ke(e,n){var t=u.getCurrentSourceFile(e),r=Ce(t,n),i=r.firstLine,a=r.lastLine;return i===a&&n.pos!==n.end?Ne(e,n,!0):Ie(e,n,!0)}function Re(n,t){var r=u.getCurrentSourceFile(n),i=[],a=t.pos,o=t.end;a===o&&(o+=e.isInsideJsxElement(r,a)?2:1);for(var s=a;s<=o;s++){var c=e.isInComment(r,s);if(c){switch(c.kind){case 2:i.push.apply(i,Ie(n,{end:c.end,pos:c.pos+1},!1));break;case 3:i.push.apply(i,Ne(n,{end:c.end,pos:c.pos+1},!1))}s=c.end+1}}return i}function Oe(n){var t=n.openingElement,r=n.closingElement,i=n.parent;return!e.tagNamesAreEquivalent(t.tagName,r.tagName)||e.isJsxElement(i)&&e.tagNamesAreEquivalent(t.tagName,i.openingElement.tagName)&&Oe(i)}function we(n,t,r){var i=u.getCurrentSourceFile(n),a=e.formatting.getRangeOfEnclosingComment(i,t);return!a||r&&3!==a.kind?void 0:e.createTextSpanFromRange(a)}function Pe(n,t){v();var r=h(n);p.throwIfCancellationRequested();var i=r.text,a=[];if(t.length>0&&!b(r.fileName)){var o=g(),s=void 0;while(s=o.exec(i)){p.throwIfCancellationRequested();var c=3;e.Debug.assert(s.length===t.length+c);var l=s[1],u=s.index+l.length;if(e.isInComment(r,u)){for(var d=void 0,f=0;f<t.length;f++)s[f+c]&&(d=t[f]);if(void 0===d)return e.Debug.fail();if(!y(i.charCodeAt(u+d.text.length))){var m=s[2];a.push({descriptor:d,message:m,position:u})}}}}return a;function _(e){return e.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g,"\\$&")}function g(){var n=/(?:\/\/+\s*)/.source,r=/(?:\/\*+\s*)/.source,i=/(?:^(?:\s|\*)*)/.source,a="("+i+"|"+n+"|"+r+")",o="(?:"+e.map(t,function(e){return"("+_(e.text)+")"}).join("|")+")",s=/(?:$|\*\/)/.source,c=/(?:.*?)/.source,l="("+o+c+")",u=a+l+s;return new RegExp(u,"gim")}function y(e){return e>=97&&e<=122||e>=65&&e<=90||e>=48&&e<=57}function b(n){return e.stringContains(n,"/node_modules/")}}function Me(n,t,r){return v(),e.Rename.getRenameInfo(c,h(n),t,r)}function Fe(t,r,i,a,o,s){var c="number"===typeof r?[r,void 0]:[r.pos,r.end],l=c[0],u=c[1];return{file:t,startPosition:l,endPosition:u,program:b(),host:n,formatContext:e.formatting.getFormatContext(a,n),cancellationToken:p,preferences:i,triggerReason:o,kind:s}}function Ge(e,t,r){return{file:e,program:b(),host:n,span:t,preferences:r,cancellationToken:p}}function Be(n,t){return e.SmartSelectionRange.getSmartSelectionRange(t,u.getCurrentSourceFile(n))}function Ue(n,t,r,i,a){void 0===r&&(r=e.emptyOptions),v();var o=h(n);return e.refactor.getApplicableRefactors(Fe(o,t,r,e.emptyOptions,i,a))}function Ve(n,t,r,i,a,o){void 0===o&&(o=e.emptyOptions),v();var s=h(n);return e.refactor.getEditsForRefactor(Fe(s,r,o,t),i,a)}function je(e,n){return 0===n?{line:0,character:0}:y.toLineColumnOffset(e,n)}function Ke(n,t){v();var r=e.CallHierarchy.resolveCallHierarchyDeclaration(c,e.getTouchingPropertyName(h(n),t));return r&&e.mapOneOrMany(r,function(n){return e.CallHierarchy.createCallHierarchyItem(c,n)})}function He(n,t){v();var r=h(n),i=e.firstOrOnly(e.CallHierarchy.resolveCallHierarchyDeclaration(c,0===t?r:e.getTouchingPropertyName(r,t)));return i?e.CallHierarchy.getIncomingCalls(c,i,p):[]}function We(n,t){v();var r=h(n),i=e.firstOrOnly(e.CallHierarchy.resolveCallHierarchyDeclaration(c,0===t?r:e.getTouchingPropertyName(r,t)));return i?e.CallHierarchy.getOutgoingCalls(c,i):[]}function ze(n,t,r){void 0===r&&(r=e.emptyOptions),v();var i=h(n);return e.InlayHints.provideInlayHints(Ge(i,t,r))}fe.forEach(function(e,n){return fe.set(e.toString(),Number(n))});var Je={dispose:S,cleanupSemanticCache:T,getSyntacticDiagnostics:L,getSemanticDiagnostics:D,getSuggestionDiagnostics:C,getCompilerOptionsDiagnostics:k,getSyntacticClassifications:ue,getSemanticClassifications:ce,getEncodedSyntacticClassifications:de,getEncodedSemanticClassifications:le,getCompletionsAtPosition:R,getCompletionEntryDetails:O,getCompletionEntrySymbol:M,getSignatureHelpItems:te,getQuickInfoAtPosition:B,getDefinitionAtPosition:j,getDefinitionAndBoundSpan:K,getImplementationAtPosition:z,getTypeDefinitionAtPosition:H,getReferencesAtPosition:Y,findReferences:Z,getFileReferences:$,getOccurrencesAtPosition:J,getDocumentHighlights:q,getNameOrDottedNameSpan:ie,getBreakpointStatementAtPosition:ae,getNavigateToItems:ee,getRenameInfo:Me,getSmartSelectionRange:Be,findRenameLocations:X,getNavigationBarItems:oe,getNavigationTree:se,getOutliningSpans:pe,getTodoComments:Pe,getBraceMatchingAtPosition:me,getIndentationAtPosition:_e,getFormattingEditsForRange:ge,getFormattingEditsForDocument:ye,getFormattingEditsAfterKeystroke:he,getDocCommentTemplateAtPosition:Le,isValidBraceCompletionAtPosition:De,getJsxClosingTagAtPosition:Ae,getSpanOfEnclosingComment:we,getCodeFixesAtPosition:ve,getCombinedCodeFix:be,applyCodeActionCommand:Se,organizeImports:Ee,getEditsForFileRename:Te,getEmitOutput:ne,getNonBoundSourceFile:re,getProgram:b,getAutoImportProvider:E,getApplicableRefactors:Ue,getEditsForRefactor:Ve,toLineColumnOffset:je,getSourceMapper:function(){return y},clearSourceMapperCache:function(){return y.clearCache()},prepareCallHierarchy:Ke,provideCallHierarchyIncomingCalls:He,provideCallHierarchyOutgoingCalls:We,toggleLineComment:Ie,toggleMultilineComment:Ne,commentSelection:ke,uncommentSelection:Re,provideInlayHints:ze};switch(s){case e.LanguageServiceMode.Semantic:break;case e.LanguageServiceMode.PartialSemantic:F.forEach(function(e){return Je[e]=function(){throw new Error("LanguageService Operation: "+e+" not allowed in LanguageServiceMode.PartialSemantic")}});break;case e.LanguageServiceMode.Syntactic:G.forEach(function(e){return Je[e]=function(){throw new Error("LanguageService Operation: "+e+" not allowed in LanguageServiceMode.Syntactic")}});break;default:e.Debug.assertNever(s)}return Je}function U(e){return e.nameTable||V(e),e.nameTable}function V(n){var t=n.nameTable=new e.Map;n.forEachChild(function n(r){if(e.isIdentifier(r)&&!e.isTagName(r)&&r.escapedText||e.isStringOrNumericLiteralLike(r)&&j(r)){var i=e.getEscapedTextOfIdentifierOrLiteral(r);t.set(i,void 0===t.get(i)?r.pos:-1)}else if(e.isPrivateIdentifier(r)){i=r.escapedText;t.set(i,void 0===t.get(i)?r.pos:-1)}if(e.forEachChild(r,n),e.hasJSDocNodes(r))for(var a=0,o=r.jsDoc;a<o.length;a++){var s=o[a];e.forEachChild(s,n)}})}function j(n){return e.isDeclarationName(n)||275===n.parent.kind||J(n)||e.isLiteralComputedPropertyDeclarationName(n)}function K(n){var t=H(n);return t&&(e.isObjectLiteralExpression(t.parent)||e.isJsxAttributes(t.parent))?t:void 0}function H(n){switch(n.kind){case 10:case 14:case 8:if(160===n.parent.kind)return e.isObjectLiteralElement(n.parent.parent)?n.parent.parent:void 0;case 79:return!e.isObjectLiteralElement(n.parent)||203!==n.parent.parent.kind&&284!==n.parent.parent.kind||n.parent.name!==n?void 0:n.parent}}function W(n,t){var r=K(n);if(r){var i=t.getContextualType(r.parent),a=i&&z(r,t,i,!1);if(a&&1===a.length)return e.first(a)}return t.getSymbolAtLocation(n)}function z(n,t,r,i){var a=e.getNameFromPropertyName(n.name);if(!a)return e.emptyArray;if(!r.isUnion()){var o=r.getProperty(a);return o?[o]:e.emptyArray}var s=e.mapDefined(r.types,function(r){return(e.isObjectLiteralExpression(n.parent)||e.isJsxAttributes(n.parent))&&t.isTypeInvalidDueToUnionDiscriminant(r,n.parent)?void 0:r.getProperty(a)});if(i&&(0===s.length||s.length===r.types.length)){o=r.getProperty(a);if(o)return[o]}return 0===s.length?e.mapDefined(r.types,function(e){return e.getProperty(a)}):s}function J(e){return e&&e.parent&&205===e.parent.kind&&e.parent.argumentExpression===e}function q(n){return r+e.directorySeparator+e.getDefaultLibFileName(n)}e.createLanguageService=B,e.getNameTable=U,e.getContainingObjectLiteralElement=K,e.getPropertySymbolsFromContextualType=z,e.getDefaultLibFilePath=q,e.setObjectAllocator(S())}(d||(d={})),function(e){(function(n){function t(n,t){if(!n.isDeclarationFile){var r=e.getTokenAtPosition(n,t),i=n.getLineAndCharacterOfPosition(t).line;if(n.getLineAndCharacterOfPosition(r.getStart(n)).line>i){var a=e.findPrecedingToken(r.pos,n);if(!a||n.getLineAndCharacterOfPosition(a.getEnd()).line!==i)return;r=a}if(!(8388608&r.flags))return p(r)}function o(t,r){var i=t.decorators?e.skipTrivia(n.text,t.decorators.end):t.getStart(n);return e.createTextSpanFromBounds(i,(r||t).getEnd())}function s(t,r){return o(t,e.findNextToken(r,r.parent,n))}function c(e,t){return e&&i===n.getLineAndCharacterOfPosition(e.getStart(n)).line?p(e):p(t)}function l(t){return e.createTextSpanFromBounds(e.skipTrivia(n.text,t.pos),t.end)}function u(t){return p(e.findPrecedingToken(t.pos,n))}function d(t){return p(e.findNextToken(t,t.parent,n))}function p(t){if(t){var r=t.parent;switch(t.kind){case 235:return h(t.declarationList.declarations[0]);case 252:case 165:case 164:return h(t);case 162:return b(t);case 254:case 167:case 166:case 170:case 171:case 169:case 211:case 212:return T(t);case 233:if(e.isFunctionBlock(t))return S(t);case 260:return x(t);case 290:return x(t.block);case 236:return o(t.expression);case 245:return o(t.getChildAt(0),t.expression);case 239:return s(t,t.expression);case 238:return p(t.statement);case 251:return o(t.getChildAt(0));case 237:return s(t,t.expression);case 248:return p(t.statement);case 244:case 243:return o(t.getChildAt(0),t.label);case 240:return D(t);case 241:return s(t,t.expression);case 242:return L(t);case 247:return s(t,t.expression);case 287:case 288:return p(t.statements[0]);case 250:return x(t.tryBlock);case 249:return o(t,t.expression);case 269:return o(t,t.expression);case 263:return o(t,t.moduleReference);case 264:return o(t,t.moduleSpecifier);case 270:return o(t,t.moduleSpecifier);case 259:if(1!==e.getModuleInstanceState(t))return;case 255:case 258:case 294:case 201:return o(t);case 246:return p(t.statement);case 163:return l(r.decorators);case 199:case 200:return A(t);case 256:case 257:return;case 26:case 1:return c(e.findPrecedingToken(t.pos,n));case 27:return u(t);case 18:return I(t);case 19:return N(t);case 23:return k(t);case 20:return R(t);case 21:return O(t);case 58:return w(t);case 31:case 29:return P(t);case 115:return M(t);case 91:case 83:case 96:return d(t);case 158:return F(t);default:if(e.isArrayLiteralOrObjectLiteralDestructuringPattern(t))return C(t);if((79===t.kind||223===t.kind||291===t.kind||292===t.kind)&&e.isArrayLiteralOrObjectLiteralDestructuringPattern(r))return o(t);if(219===t.kind){var i=t,a=i.left,f=i.operatorToken;if(e.isArrayLiteralOrObjectLiteralDestructuringPattern(a))return C(a);if(63===f.kind&&e.isArrayLiteralOrObjectLiteralDestructuringPattern(t.parent))return o(t);if(27===f.kind)return p(a)}if(e.isExpressionNode(t))switch(r.kind){case 238:return u(t);case 163:return p(t.parent);case 240:case 242:return o(t);case 219:if(27===t.parent.operatorToken.kind)return o(t);break;case 212:if(t.parent.body===t)return o(t);break}switch(t.parent.kind){case 291:if(t.parent.name===t&&!e.isArrayLiteralOrObjectLiteralDestructuringPattern(t.parent.parent))return p(t.parent.initializer);break;case 209:if(t.parent.type===t)return d(t.parent.type);break;case 252:case 162:var m=t.parent,_=m.initializer,g=m.type;if(_===t||g===t||e.isAssignmentOperator(t.kind))return u(t);break;case 219:a=t.parent.left;if(e.isArrayLiteralOrObjectLiteralDestructuringPattern(a)&&t!==a)return u(t);break;default:if(e.isFunctionLike(t.parent)&&t.parent.type===t)return u(t)}return p(t.parent)}}function y(t){return e.isVariableDeclarationList(t.parent)&&t.parent.declarations[0]===t?o(e.findPrecedingToken(t.pos,n,t.parent),t):o(t)}function h(t){if(241===t.parent.parent.kind)return p(t.parent.parent);var r=t.parent;return e.isBindingPattern(t.name)?A(t.name):t.initializer||e.hasSyntacticModifier(t,1)||242===r.parent.kind?y(t):e.isVariableDeclarationList(t.parent)&&t.parent.declarations[0]!==t?p(e.findPrecedingToken(t.pos,n,t.parent)):void 0}function v(n){return!!n.initializer||void 0!==n.dotDotDotToken||e.hasSyntacticModifier(n,12)}function b(n){if(e.isBindingPattern(n.name))return A(n.name);if(v(n))return o(n);var t=n.parent,r=t.parameters.indexOf(n);return e.Debug.assert(-1!==r),0!==r?b(t.parameters[r-1]):p(t.body)}function E(n){return e.hasSyntacticModifier(n,1)||255===n.parent.kind&&169!==n.kind}function T(e){if(e.body)return E(e)?o(e):p(e.body)}function S(e){var n=e.statements.length?e.statements[0]:e.getLastToken();return E(e.parent)?c(e.parent,n):p(n)}function x(t){switch(t.parent.kind){case 259:if(1!==e.getModuleInstanceState(t.parent))return;case 239:case 237:case 241:return c(t.parent,t.statements[0]);case 240:case 242:return c(e.findPrecedingToken(t.pos,n,t.parent),t.statements[0])}return p(t.statements[0])}function L(e){if(253!==e.initializer.kind)return p(e.initializer);var n=e.initializer;return n.declarations.length>0?p(n.declarations[0]):void 0}function D(e){return e.initializer?L(e):e.condition?o(e.condition):e.incrementor?o(e.incrementor):void 0}function A(n){var t=e.forEach(n.elements,function(e){return 225!==e.kind?e:void 0});return t?p(t):201===n.parent.kind?o(n.parent):y(n.parent)}function C(n){e.Debug.assert(200!==n.kind&&199!==n.kind);var t=202===n.kind?n.elements:n.properties,r=e.forEach(t,function(e){return 225!==e.kind?e:void 0});return r?p(r):o(219===n.parent.kind?n.parent:n)}function I(t){switch(t.parent.kind){case 258:var r=t.parent;return c(e.findPrecedingToken(t.pos,n,t.parent),r.members.length?r.members[0]:r.getLastToken(n));case 255:var i=t.parent;return c(e.findPrecedingToken(t.pos,n,t.parent),i.members.length?i.members[0]:i.getLastToken(n));case 261:return c(t.parent.parent,t.parent.clauses[0])}return p(t.parent)}function N(n){switch(n.parent.kind){case 260:if(1!==e.getModuleInstanceState(n.parent.parent))return;case 258:case 255:return o(n);case 233:if(e.isFunctionBlock(n.parent))return o(n);case 290:return p(e.lastOrUndefined(n.parent.statements));case 261:var t=n.parent,r=e.lastOrUndefined(t.clauses);return r?p(e.lastOrUndefined(r.statements)):void 0;case 199:var i=n.parent;return p(e.lastOrUndefined(i.elements)||i);default:if(e.isArrayLiteralOrObjectLiteralDestructuringPattern(n.parent)){var a=n.parent;return o(e.lastOrUndefined(a.properties)||a)}return p(n.parent)}}function k(n){switch(n.parent.kind){case 200:var t=n.parent;return o(e.lastOrUndefined(t.elements)||t);default:if(e.isArrayLiteralOrObjectLiteralDestructuringPattern(n.parent)){var r=n.parent;return o(e.lastOrUndefined(r.elements)||r)}return p(n.parent)}}function R(e){return 238===e.parent.kind||206===e.parent.kind||207===e.parent.kind?u(e):210===e.parent.kind?d(e):p(e.parent)}function O(e){switch(e.parent.kind){case 211:case 254:case 212:case 167:case 166:case 170:case 171:case 169:case 239:case 238:case 240:case 242:case 206:case 207:case 210:return u(e);default:return p(e.parent)}}function w(n){return e.isFunctionLike(n.parent)||291===n.parent.kind||162===n.parent.kind?u(n):p(n.parent)}function P(e){return 209===e.parent.kind?d(e):p(e.parent)}function M(e){return 238===e.parent.kind?s(e,e.parent.expression):p(e.parent)}function F(e){return 242===e.parent.kind?d(e):p(e.parent)}}}n.spanInSourceFileAtLocation=t})(e.BreakpointResolver||(e.BreakpointResolver={}))}(d||(d={})),function(e){function n(n,t,r){var i=[];r=e.fixupCompilerOptions(r,i);var a=e.isArray(n)?n:[n],o=e.transformNodes(void 0,void 0,e.factory,r,a,t,!0);return o.diagnostics=e.concatenate(o.diagnostics,i),o}e.transform=n}(d||(d={}));var d,p=function(){return this}();(function(e){function n(e,n){e&&e.log("*INTERNAL ERROR* - Exception in typescript services: "+n.message)}var t=function(){function n(e){this.scriptSnapshotShim=e}return n.prototype.getText=function(e,n){return this.scriptSnapshotShim.getText(e,n)},n.prototype.getLength=function(){return this.scriptSnapshotShim.getLength()},n.prototype.getChangeRange=function(n){var t=n,r=this.scriptSnapshotShim.getChangeRange(t.scriptSnapshotShim);if(null===r)return null;var i=JSON.parse(r);return e.createTextChangeRange(e.createTextSpan(i.span.start,i.span.length),i.newLength)},n.prototype.dispose=function(){"dispose"in this.scriptSnapshotShim&&this.scriptSnapshotShim.dispose()},n}(),r=function(){function n(n){var t=this;this.shimHost=n,this.loggingEnabled=!1,this.tracingEnabled=!1,"getModuleResolutionsForFile"in this.shimHost&&(this.resolveModuleNames=function(n,r){var i=JSON.parse(t.shimHost.getModuleResolutionsForFile(r));return e.map(n,function(n){var t=e.getProperty(i,n);return t?{resolvedFileName:t,extension:e.extensionFromPath(t),isExternalLibraryImport:!1}:void 0})}),"directoryExists"in this.shimHost&&(this.directoryExists=function(e){return t.shimHost.directoryExists(e)}),"getTypeReferenceDirectiveResolutionsForFile"in this.shimHost&&(this.resolveTypeReferenceDirectives=function(n,r){var i=JSON.parse(t.shimHost.getTypeReferenceDirectiveResolutionsForFile(r));return e.map(n,function(n){return e.getProperty(i,n)})})}return n.prototype.log=function(e){this.loggingEnabled&&this.shimHost.log(e)},n.prototype.trace=function(e){this.tracingEnabled&&this.shimHost.trace(e)},n.prototype.error=function(e){this.shimHost.error(e)},n.prototype.getProjectVersion=function(){if(this.shimHost.getProjectVersion)return this.shimHost.getProjectVersion()},n.prototype.getTypeRootsVersion=function(){return this.shimHost.getTypeRootsVersion?this.shimHost.getTypeRootsVersion():0},n.prototype.useCaseSensitiveFileNames=function(){return!!this.shimHost.useCaseSensitiveFileNames&&this.shimHost.useCaseSensitiveFileNames()},n.prototype.getCompilationSettings=function(){var e=this.shimHost.getCompilationSettings();if(null===e||""===e)throw Error("LanguageServiceShimHostAdapter.getCompilationSettings: empty compilationSettings");var n=JSON.parse(e);return n.allowNonTsExtensions=!0,n},n.prototype.getScriptFileNames=function(){var e=this.shimHost.getScriptFileNames();return JSON.parse(e)},n.prototype.getScriptSnapshot=function(e){var n=this.shimHost.getScriptSnapshot(e);return n&&new t(n)},n.prototype.getScriptKind=function(e){return"getScriptKind"in this.shimHost?this.shimHost.getScriptKind(e):0},n.prototype.getScriptVersion=function(e){return this.shimHost.getScriptVersion(e)},n.prototype.getLocalizedDiagnosticMessages=function(){var e=this.shimHost.getLocalizedDiagnosticMessages();if(null===e||""===e)return null;try{return JSON.parse(e)}catch(n){return this.log(n.description||"diagnosticMessages.generated.json has invalid JSON format"),null}},n.prototype.getCancellationToken=function(){var n=this.shimHost.getCancellationToken();return new e.ThrottledCancellationToken(n)},n.prototype.getCurrentDirectory=function(){return this.shimHost.getCurrentDirectory()},n.prototype.getDirectories=function(e){return JSON.parse(this.shimHost.getDirectories(e))},n.prototype.getDefaultLibFileName=function(e){return this.shimHost.getDefaultLibFileName(JSON.stringify(e))},n.prototype.readDirectory=function(n,t,r,i,a){var o=e.getFileMatcherPatterns(n,r,i,this.shimHost.useCaseSensitiveFileNames(),this.shimHost.getCurrentDirectory());return JSON.parse(this.shimHost.readDirectory(n,JSON.stringify(t),JSON.stringify(o.basePaths),o.excludePattern,o.includeFilePattern,o.includeDirectoryPattern,a))},n.prototype.readFile=function(e,n){return this.shimHost.readFile(e,n)},n.prototype.fileExists=function(e){return this.shimHost.fileExists(e)},n}();e.LanguageServiceShimHostAdapter=r;var i=function(){function n(e){var n=this;this.shimHost=e,this.useCaseSensitiveFileNames=!!this.shimHost.useCaseSensitiveFileNames&&this.shimHost.useCaseSensitiveFileNames(),"directoryExists"in this.shimHost?this.directoryExists=function(e){return n.shimHost.directoryExists(e)}:this.directoryExists=void 0,"realpath"in this.shimHost?this.realpath=function(e){return n.shimHost.realpath(e)}:this.realpath=void 0}return n.prototype.readDirectory=function(n,t,r,i,a){var o=e.getFileMatcherPatterns(n,r,i,this.shimHost.useCaseSensitiveFileNames(),this.shimHost.getCurrentDirectory());return JSON.parse(this.shimHost.readDirectory(n,JSON.stringify(t),JSON.stringify(o.basePaths),o.excludePattern,o.includeFilePattern,o.includeDirectoryPattern,a))},n.prototype.fileExists=function(e){return this.shimHost.fileExists(e)},n.prototype.readFile=function(e){return this.shimHost.readFile(e)},n.prototype.getDirectories=function(e){return JSON.parse(this.shimHost.getDirectories(e))},n}();function s(n,t,r,i){var a;i&&(n.log(t),a=e.timestamp());var o=r();if(i){var s=e.timestamp();if(n.log(t+" completed in "+(s-a)+" msec"),e.isString(o)){var c=o;c.length>128&&(c=c.substring(0,128)+"..."),n.log("  result.length="+c.length+", result='"+JSON.stringify(c)+"'")}}return o}function c(e,n,t,r){return d(e,n,!0,t,r)}function d(t,r,i,a,o){try{var c=s(t,r,a,o);return i?JSON.stringify({result:c}):c}catch(l){return l instanceof e.OperationCanceledException?JSON.stringify({canceled:!0}):(n(t,l),l.description=r,JSON.stringify({error:l}))}}e.CoreServicesShimHostAdapter=i;var f=function(){function e(e){this.factory=e,e.registerShim(this)}return e.prototype.dispose=function(e){this.factory.unregisterShim(this)},e}();function m(e,n){return e.map(function(e){return _(e,n)})}function _(n,t){return{message:e.flattenDiagnosticMessageText(n.messageText,t),start:n.start,length:n.length,category:e.diagnosticCategoryName(n),code:n.code,reportsUnnecessary:n.reportsUnnecessary,reportsDeprecated:n.reportsDeprecated}}e.realizeDiagnostics=m;var g=function(n){function t(e,t,r){var i=n.call(this,e)||this;return i.host=t,i.languageService=r,i.logPerformance=!1,i.logger=i.host,i}return u(t,n),t.prototype.forwardJSONCall=function(e,n){return c(this.logger,e,n,this.logPerformance)},t.prototype.dispose=function(e){this.logger.log("dispose()"),this.languageService.dispose(),this.languageService=null,p&&p.CollectGarbage&&(p.CollectGarbage(),this.logger.log("CollectGarbage()")),this.logger=null,n.prototype.dispose.call(this,e)},t.prototype.refresh=function(e){this.forwardJSONCall("refresh("+e+")",function(){return null})},t.prototype.cleanupSemanticCache=function(){var e=this;this.forwardJSONCall("cleanupSemanticCache()",function(){return e.languageService.cleanupSemanticCache(),null})},t.prototype.realizeDiagnostics=function(n){var t=e.getNewLineOrDefaultFromHost(this.host);return m(n,t)},t.prototype.getSyntacticClassifications=function(n,t,r){var i=this;return this.forwardJSONCall("getSyntacticClassifications('"+n+"', "+t+", "+r+")",function(){return i.languageService.getSyntacticClassifications(n,e.createTextSpan(t,r))})},t.prototype.getSemanticClassifications=function(n,t,r){var i=this;return this.forwardJSONCall("getSemanticClassifications('"+n+"', "+t+", "+r+")",function(){return i.languageService.getSemanticClassifications(n,e.createTextSpan(t,r))})},t.prototype.getEncodedSyntacticClassifications=function(n,t,r){var i=this;return this.forwardJSONCall("getEncodedSyntacticClassifications('"+n+"', "+t+", "+r+")",function(){return y(i.languageService.getEncodedSyntacticClassifications(n,e.createTextSpan(t,r)))})},t.prototype.getEncodedSemanticClassifications=function(n,t,r){var i=this;return this.forwardJSONCall("getEncodedSemanticClassifications('"+n+"', "+t+", "+r+")",function(){return y(i.languageService.getEncodedSemanticClassifications(n,e.createTextSpan(t,r)))})},t.prototype.getSyntacticDiagnostics=function(e){var n=this;return this.forwardJSONCall("getSyntacticDiagnostics('"+e+"')",function(){var t=n.languageService.getSyntacticDiagnostics(e);return n.realizeDiagnostics(t)})},t.prototype.getSemanticDiagnostics=function(e){var n=this;return this.forwardJSONCall("getSemanticDiagnostics('"+e+"')",function(){var t=n.languageService.getSemanticDiagnostics(e);return n.realizeDiagnostics(t)})},t.prototype.getSuggestionDiagnostics=function(e){var n=this;return this.forwardJSONCall("getSuggestionDiagnostics('"+e+"')",function(){return n.realizeDiagnostics(n.languageService.getSuggestionDiagnostics(e))})},t.prototype.getCompilerOptionsDiagnostics=function(){var e=this;return this.forwardJSONCall("getCompilerOptionsDiagnostics()",function(){var n=e.languageService.getCompilerOptionsDiagnostics();return e.realizeDiagnostics(n)})},t.prototype.getQuickInfoAtPosition=function(e,n){var t=this;return this.forwardJSONCall("getQuickInfoAtPosition('"+e+"', "+n+")",function(){return t.languageService.getQuickInfoAtPosition(e,n)})},t.prototype.getNameOrDottedNameSpan=function(e,n,t){var r=this;return this.forwardJSONCall("getNameOrDottedNameSpan('"+e+"', "+n+", "+t+")",function(){return r.languageService.getNameOrDottedNameSpan(e,n,t)})},t.prototype.getBreakpointStatementAtPosition=function(e,n){var t=this;return this.forwardJSONCall("getBreakpointStatementAtPosition('"+e+"', "+n+")",function(){return t.languageService.getBreakpointStatementAtPosition(e,n)})},t.prototype.getSignatureHelpItems=function(e,n,t){var r=this;return this.forwardJSONCall("getSignatureHelpItems('"+e+"', "+n+")",function(){return r.languageService.getSignatureHelpItems(e,n,t)})},t.prototype.getDefinitionAtPosition=function(e,n){var t=this;return this.forwardJSONCall("getDefinitionAtPosition('"+e+"', "+n+")",function(){return t.languageService.getDefinitionAtPosition(e,n)})},t.prototype.getDefinitionAndBoundSpan=function(e,n){var t=this;return this.forwardJSONCall("getDefinitionAndBoundSpan('"+e+"', "+n+")",function(){return t.languageService.getDefinitionAndBoundSpan(e,n)})},t.prototype.getTypeDefinitionAtPosition=function(e,n){var t=this;return this.forwardJSONCall("getTypeDefinitionAtPosition('"+e+"', "+n+")",function(){return t.languageService.getTypeDefinitionAtPosition(e,n)})},t.prototype.getImplementationAtPosition=function(e,n){var t=this;return this.forwardJSONCall("getImplementationAtPosition('"+e+"', "+n+")",function(){return t.languageService.getImplementationAtPosition(e,n)})},t.prototype.getRenameInfo=function(e,n,t){var r=this;return this.forwardJSONCall("getRenameInfo('"+e+"', "+n+")",function(){return r.languageService.getRenameInfo(e,n,t)})},t.prototype.getSmartSelectionRange=function(e,n){var t=this;return this.forwardJSONCall("getSmartSelectionRange('"+e+"', "+n+")",function(){return t.languageService.getSmartSelectionRange(e,n)})},t.prototype.findRenameLocations=function(e,n,t,r,i){var a=this;return this.forwardJSONCall("findRenameLocations('"+e+"', "+n+", "+t+", "+r+", "+i+")",function(){return a.languageService.findRenameLocations(e,n,t,r,i)})},t.prototype.getBraceMatchingAtPosition=function(e,n){var t=this;return this.forwardJSONCall("getBraceMatchingAtPosition('"+e+"', "+n+")",function(){return t.languageService.getBraceMatchingAtPosition(e,n)})},t.prototype.isValidBraceCompletionAtPosition=function(e,n,t){var r=this;return this.forwardJSONCall("isValidBraceCompletionAtPosition('"+e+"', "+n+", "+t+")",function(){return r.languageService.isValidBraceCompletionAtPosition(e,n,t)})},t.prototype.getSpanOfEnclosingComment=function(e,n,t){var r=this;return this.forwardJSONCall("getSpanOfEnclosingComment('"+e+"', "+n+")",function(){return r.languageService.getSpanOfEnclosingComment(e,n,t)})},t.prototype.getIndentationAtPosition=function(e,n,t){var r=this;return this.forwardJSONCall("getIndentationAtPosition('"+e+"', "+n+")",function(){var i=JSON.parse(t);return r.languageService.getIndentationAtPosition(e,n,i)})},t.prototype.getReferencesAtPosition=function(e,n){var t=this;return this.forwardJSONCall("getReferencesAtPosition('"+e+"', "+n+")",function(){return t.languageService.getReferencesAtPosition(e,n)})},t.prototype.findReferences=function(e,n){var t=this;return this.forwardJSONCall("findReferences('"+e+"', "+n+")",function(){return t.languageService.findReferences(e,n)})},t.prototype.getFileReferences=function(e){var n=this;return this.forwardJSONCall("getFileReferences('"+e+")",function(){return n.languageService.getFileReferences(e)})},t.prototype.getOccurrencesAtPosition=function(e,n){var t=this;return this.forwardJSONCall("getOccurrencesAtPosition('"+e+"', "+n+")",function(){return t.languageService.getOccurrencesAtPosition(e,n)})},t.prototype.getDocumentHighlights=function(n,t,r){var i=this;return this.forwardJSONCall("getDocumentHighlights('"+n+"', "+t+")",function(){var a=i.languageService.getDocumentHighlights(n,t,JSON.parse(r)),o=e.toFileNameLowerCase(e.normalizeSlashes(n));return e.filter(a,function(n){return e.toFileNameLowerCase(e.normalizeSlashes(n.fileName))===o})})},t.prototype.getCompletionsAtPosition=function(e,n,t){var r=this;return this.forwardJSONCall("getCompletionsAtPosition('"+e+"', "+n+", "+t+")",function(){return r.languageService.getCompletionsAtPosition(e,n,t)})},t.prototype.getCompletionEntryDetails=function(e,n,t,r,i,a,o){var s=this;return this.forwardJSONCall("getCompletionEntryDetails('"+e+"', "+n+", '"+t+"')",function(){var c=void 0===r?void 0:JSON.parse(r);return s.languageService.getCompletionEntryDetails(e,n,t,c,i,a,o)})},t.prototype.getFormattingEditsForRange=function(e,n,t,r){var i=this;return this.forwardJSONCall("getFormattingEditsForRange('"+e+"', "+n+", "+t+")",function(){var a=JSON.parse(r);return i.languageService.getFormattingEditsForRange(e,n,t,a)})},t.prototype.getFormattingEditsForDocument=function(e,n){var t=this;return this.forwardJSONCall("getFormattingEditsForDocument('"+e+"')",function(){var r=JSON.parse(n);return t.languageService.getFormattingEditsForDocument(e,r)})},t.prototype.getFormattingEditsAfterKeystroke=function(e,n,t,r){var i=this;return this.forwardJSONCall("getFormattingEditsAfterKeystroke('"+e+"', "+n+", '"+t+"')",function(){var a=JSON.parse(r);return i.languageService.getFormattingEditsAfterKeystroke(e,n,t,a)})},t.prototype.getDocCommentTemplateAtPosition=function(e,n,t){var r=this;return this.forwardJSONCall("getDocCommentTemplateAtPosition('"+e+"', "+n+")",function(){return r.languageService.getDocCommentTemplateAtPosition(e,n,t)})},t.prototype.getNavigateToItems=function(e,n,t){var r=this;return this.forwardJSONCall("getNavigateToItems('"+e+"', "+n+", "+t+")",function(){return r.languageService.getNavigateToItems(e,n,t)})},t.prototype.getNavigationBarItems=function(e){var n=this;return this.forwardJSONCall("getNavigationBarItems('"+e+"')",function(){return n.languageService.getNavigationBarItems(e)})},t.prototype.getNavigationTree=function(e){var n=this;return this.forwardJSONCall("getNavigationTree('"+e+"')",function(){return n.languageService.getNavigationTree(e)})},t.prototype.getOutliningSpans=function(e){var n=this;return this.forwardJSONCall("getOutliningSpans('"+e+"')",function(){return n.languageService.getOutliningSpans(e)})},t.prototype.getTodoComments=function(e,n){var t=this;return this.forwardJSONCall("getTodoComments('"+e+"')",function(){return t.languageService.getTodoComments(e,JSON.parse(n))})},t.prototype.prepareCallHierarchy=function(e,n){var t=this;return this.forwardJSONCall("prepareCallHierarchy('"+e+"', "+n+")",function(){return t.languageService.prepareCallHierarchy(e,n)})},t.prototype.provideCallHierarchyIncomingCalls=function(e,n){var t=this;return this.forwardJSONCall("provideCallHierarchyIncomingCalls('"+e+"', "+n+")",function(){return t.languageService.provideCallHierarchyIncomingCalls(e,n)})},t.prototype.provideCallHierarchyOutgoingCalls=function(e,n){var t=this;return this.forwardJSONCall("provideCallHierarchyOutgoingCalls('"+e+"', "+n+")",function(){return t.languageService.provideCallHierarchyOutgoingCalls(e,n)})},t.prototype.provideInlayHints=function(e,n,t){var r=this;return this.forwardJSONCall("provideInlayHints('"+e+"', '"+JSON.stringify(n)+"', "+JSON.stringify(t)+")",function(){return r.languageService.provideInlayHints(e,n,t)})},t.prototype.getEmitOutput=function(e){var n=this;return this.forwardJSONCall("getEmitOutput('"+e+"')",function(){var t=n.languageService.getEmitOutput(e),r=t.diagnostics,i=l(t,["diagnostics"]);return o(o({},i),{diagnostics:n.realizeDiagnostics(r)})})},t.prototype.getEmitOutputObject=function(e){var n=this;return d(this.logger,"getEmitOutput('"+e+"')",!1,function(){return n.languageService.getEmitOutput(e)},this.logPerformance)},t.prototype.toggleLineComment=function(e,n){var t=this;return this.forwardJSONCall("toggleLineComment('"+e+"', '"+JSON.stringify(n)+"')",function(){return t.languageService.toggleLineComment(e,n)})},t.prototype.toggleMultilineComment=function(e,n){var t=this;return this.forwardJSONCall("toggleMultilineComment('"+e+"', '"+JSON.stringify(n)+"')",function(){return t.languageService.toggleMultilineComment(e,n)})},t.prototype.commentSelection=function(e,n){var t=this;return this.forwardJSONCall("commentSelection('"+e+"', '"+JSON.stringify(n)+"')",function(){return t.languageService.commentSelection(e,n)})},t.prototype.uncommentSelection=function(e,n){var t=this;return this.forwardJSONCall("uncommentSelection('"+e+"', '"+JSON.stringify(n)+"')",function(){return t.languageService.uncommentSelection(e,n)})},t}(f);function y(e){return{spans:e.spans.join(","),endOfLineState:e.endOfLineState}}var h=function(n){function t(t,r){var i=n.call(this,t)||this;return i.logger=r,i.logPerformance=!1,i.classifier=e.createClassifier(),i}return u(t,n),t.prototype.getEncodedLexicalClassifications=function(e,n,t){var r=this;return void 0===t&&(t=!1),c(this.logger,"getEncodedLexicalClassifications",function(){return y(r.classifier.getEncodedLexicalClassifications(e,n,t))},this.logPerformance)},t.prototype.getClassificationsForLine=function(e,n,t){void 0===t&&(t=!1);for(var r=this.classifier.getClassificationsForLine(e,n,t),i="",a=0,o=r.entries;a<o.length;a++){var s=o[a];i+=s.length+"\n",i+=s.classification+"\n"}return i+=r.finalLexState,i},t}(f),v=function(n){function t(e,t,r){var i=n.call(this,e)||this;return i.logger=t,i.host=r,i.logPerformance=!1,i}return u(t,n),t.prototype.forwardJSONCall=function(e,n){return c(this.logger,e,n,this.logPerformance)},t.prototype.resolveModuleName=function(n,t,r){var i=this;return this.forwardJSONCall("resolveModuleName('"+n+"')",function(){var a=JSON.parse(r),o=e.resolveModuleName(t,e.normalizeSlashes(n),a,i.host),s=o.resolvedModule?o.resolvedModule.resolvedFileName:void 0;return o.resolvedModule&&".ts"!==o.resolvedModule.extension&&".tsx"!==o.resolvedModule.extension&&".d.ts"!==o.resolvedModule.extension&&(s=void 0),{resolvedFileName:s,failedLookupLocations:o.failedLookupLocations}})},t.prototype.resolveTypeReferenceDirective=function(n,t,r){var i=this;return this.forwardJSONCall("resolveTypeReferenceDirective("+n+")",function(){var a=JSON.parse(r),o=e.resolveTypeReferenceDirective(t,e.normalizeSlashes(n),a,i.host);return{resolvedFileName:o.resolvedTypeReferenceDirective?o.resolvedTypeReferenceDirective.resolvedFileName:void 0,primary:!o.resolvedTypeReferenceDirective||o.resolvedTypeReferenceDirective.primary,failedLookupLocations:o.failedLookupLocations}})},t.prototype.getPreProcessedFileInfo=function(n,t){var r=this;return this.forwardJSONCall("getPreProcessedFileInfo('"+n+"')",function(){var n=e.preProcessFile(e.getSnapshotText(t),!0,!0);return{referencedFiles:r.convertFileReferences(n.referencedFiles),importedFiles:r.convertFileReferences(n.importedFiles),ambientExternalModules:n.ambientExternalModules,isLibFile:n.isLibFile,typeReferenceDirectives:r.convertFileReferences(n.typeReferenceDirectives),libReferenceDirectives:r.convertFileReferences(n.libReferenceDirectives)}})},t.prototype.getAutomaticTypeDirectiveNames=function(n){var t=this;return this.forwardJSONCall("getAutomaticTypeDirectiveNames('"+n+"')",function(){var r=JSON.parse(n);return e.getAutomaticTypeDirectiveNames(r,t.host)})},t.prototype.convertFileReferences=function(n){if(n){for(var t=[],r=0,i=n;r<i.length;r++){var a=i[r];t.push({path:e.normalizeSlashes(a.fileName),position:a.pos,length:a.end-a.pos})}return t}},t.prototype.getTSConfigFileInfo=function(n,t){var r=this;return this.forwardJSONCall("getTSConfigFileInfo('"+n+"')",function(){var i=e.parseJsonText(n,e.getSnapshotText(t)),o=e.normalizeSlashes(n),s=e.parseJsonSourceFileConfigFileContent(i,r.host,e.getDirectoryPath(o),{},o);return{options:s.options,typeAcquisition:s.typeAcquisition,files:s.fileNames,raw:s.raw,errors:m(a(a([],i.parseDiagnostics,!0),s.errors,!0),"\r\n")}})},t.prototype.getDefaultCompilationSettings=function(){return this.forwardJSONCall("getDefaultCompilationSettings()",function(){return e.getDefaultCompilerOptions()})},t.prototype.discoverTypings=function(n){var t=this,r=e.createGetCanonicalFileName(!1);return this.forwardJSONCall("discoverTypings()",function(){var i=JSON.parse(n);return void 0===t.safeList&&(t.safeList=e.JsTyping.loadSafeList(t.host,e.toPath(i.safeListPath,i.safeListPath,r))),e.JsTyping.discoverTypings(t.host,function(e){return t.logger.log(e)},i.fileNames,e.toPath(i.projectRootPath,i.projectRootPath,r),t.safeList,i.packageNameToTypingLocation,i.typeAcquisition,i.unresolvedImports,i.typesRegistry)})},t}(f),b=function(){function t(){this._shims=[]}return t.prototype.getServicesVersion=function(){return e.servicesVersion},t.prototype.createLanguageServiceShim=function(t){try{void 0===this.documentRegistry&&(this.documentRegistry=e.createDocumentRegistry(t.useCaseSensitiveFileNames&&t.useCaseSensitiveFileNames(),t.getCurrentDirectory()));var i=new r(t),a=e.createLanguageService(i,this.documentRegistry,!1);return new g(this,t,a)}catch(o){throw n(t,o),o}},t.prototype.createClassifierShim=function(e){try{return new h(this,e)}catch(t){throw n(e,t),t}},t.prototype.createCoreServicesShim=function(e){try{var t=new i(e);return new v(this,e,t)}catch(r){throw n(e,r),r}},t.prototype.close=function(){e.clear(this._shims),this.documentRegistry=void 0},t.prototype.registerShim=function(e){this._shims.push(e)},t.prototype.unregisterShim=function(e){for(var n=0;n<this._shims.length;n++)if(this._shims[n]===e)return void delete this._shims[n];throw new Error("Invalid operation")},t}();e.TypeScriptServicesFactory=b})(d||(d={})),function(){if("object"!==typeof globalThis)try{Object.defineProperty(Object.prototype,"__magic__",{get:function(){return this},configurable:!0}),__magic__.globalThis=__magic__,"undefined"===typeof globalThis&&(window.globalThis=window),delete Object.prototype.__magic__}catch(e){window.globalThis=window}}(),("undefined"===typeof e||e.browser)&&(globalThis.TypeScript=globalThis.TypeScript||{},globalThis.TypeScript.Services=globalThis.TypeScript.Services||{},globalThis.TypeScript.Services.TypeScriptServicesFactory=d.TypeScriptServicesFactory,globalThis.toolsVersion=d.versionMajorMinor),i.exports&&(i.exports=d),function(e){var n={since:"4.0",warnAfter:"4.1",message:"Use the appropriate method on 'ts.factory' or the 'factory' supplied by your transformation context instead."};e.createNodeArray=e.Debug.deprecate(e.factory.createNodeArray,n),e.createNumericLiteral=e.Debug.deprecate(e.factory.createNumericLiteral,n),e.createBigIntLiteral=e.Debug.deprecate(e.factory.createBigIntLiteral,n),e.createStringLiteral=e.Debug.deprecate(e.factory.createStringLiteral,n),e.createStringLiteralFromNode=e.Debug.deprecate(e.factory.createStringLiteralFromNode,n),e.createRegularExpressionLiteral=e.Debug.deprecate(e.factory.createRegularExpressionLiteral,n),e.createLoopVariable=e.Debug.deprecate(e.factory.createLoopVariable,n),e.createUniqueName=e.Debug.deprecate(e.factory.createUniqueName,n),e.createPrivateIdentifier=e.Debug.deprecate(e.factory.createPrivateIdentifier,n),e.createSuper=e.Debug.deprecate(e.factory.createSuper,n),e.createThis=e.Debug.deprecate(e.factory.createThis,n),e.createNull=e.Debug.deprecate(e.factory.createNull,n),e.createTrue=e.Debug.deprecate(e.factory.createTrue,n),e.createFalse=e.Debug.deprecate(e.factory.createFalse,n),e.createModifier=e.Debug.deprecate(e.factory.createModifier,n),e.createModifiersFromModifierFlags=e.Debug.deprecate(e.factory.createModifiersFromModifierFlags,n),e.createQualifiedName=e.Debug.deprecate(e.factory.createQualifiedName,n),e.updateQualifiedName=e.Debug.deprecate(e.factory.updateQualifiedName,n),e.createComputedPropertyName=e.Debug.deprecate(e.factory.createComputedPropertyName,n),e.updateComputedPropertyName=e.Debug.deprecate(e.factory.updateComputedPropertyName,n),e.createTypeParameterDeclaration=e.Debug.deprecate(e.factory.createTypeParameterDeclaration,n),e.updateTypeParameterDeclaration=e.Debug.deprecate(e.factory.updateTypeParameterDeclaration,n),e.createParameter=e.Debug.deprecate(e.factory.createParameterDeclaration,n),e.updateParameter=e.Debug.deprecate(e.factory.updateParameterDeclaration,n),e.createDecorator=e.Debug.deprecate(e.factory.createDecorator,n),e.updateDecorator=e.Debug.deprecate(e.factory.updateDecorator,n),e.createProperty=e.Debug.deprecate(e.factory.createPropertyDeclaration,n),e.updateProperty=e.Debug.deprecate(e.factory.updatePropertyDeclaration,n),e.createMethod=e.Debug.deprecate(e.factory.createMethodDeclaration,n),e.updateMethod=e.Debug.deprecate(e.factory.updateMethodDeclaration,n),e.createConstructor=e.Debug.deprecate(e.factory.createConstructorDeclaration,n),e.updateConstructor=e.Debug.deprecate(e.factory.updateConstructorDeclaration,n),e.createGetAccessor=e.Debug.deprecate(e.factory.createGetAccessorDeclaration,n),e.updateGetAccessor=e.Debug.deprecate(e.factory.updateGetAccessorDeclaration,n),e.createSetAccessor=e.Debug.deprecate(e.factory.createSetAccessorDeclaration,n),e.updateSetAccessor=e.Debug.deprecate(e.factory.updateSetAccessorDeclaration,n),e.createCallSignature=e.Debug.deprecate(e.factory.createCallSignature,n),e.updateCallSignature=e.Debug.deprecate(e.factory.updateCallSignature,n),e.createConstructSignature=e.Debug.deprecate(e.factory.createConstructSignature,n),e.updateConstructSignature=e.Debug.deprecate(e.factory.updateConstructSignature,n),e.updateIndexSignature=e.Debug.deprecate(e.factory.updateIndexSignature,n),e.createKeywordTypeNode=e.Debug.deprecate(e.factory.createKeywordTypeNode,n),e.createTypePredicateNodeWithModifier=e.Debug.deprecate(e.factory.createTypePredicateNode,n),e.updateTypePredicateNodeWithModifier=e.Debug.deprecate(e.factory.updateTypePredicateNode,n),e.createTypeReferenceNode=e.Debug.deprecate(e.factory.createTypeReferenceNode,n),e.updateTypeReferenceNode=e.Debug.deprecate(e.factory.updateTypeReferenceNode,n),e.createFunctionTypeNode=e.Debug.deprecate(e.factory.createFunctionTypeNode,n),e.updateFunctionTypeNode=e.Debug.deprecate(e.factory.updateFunctionTypeNode,n),e.createConstructorTypeNode=e.Debug.deprecate(function(n,t,r){return e.factory.createConstructorTypeNode(void 0,n,t,r)},n),e.updateConstructorTypeNode=e.Debug.deprecate(function(n,t,r,i){return e.factory.updateConstructorTypeNode(n,n.modifiers,t,r,i)},n),e.createTypeQueryNode=e.Debug.deprecate(e.factory.createTypeQueryNode,n),e.updateTypeQueryNode=e.Debug.deprecate(e.factory.updateTypeQueryNode,n),e.createTypeLiteralNode=e.Debug.deprecate(e.factory.createTypeLiteralNode,n),e.updateTypeLiteralNode=e.Debug.deprecate(e.factory.updateTypeLiteralNode,n),e.createArrayTypeNode=e.Debug.deprecate(e.factory.createArrayTypeNode,n),e.updateArrayTypeNode=e.Debug.deprecate(e.factory.updateArrayTypeNode,n),e.createTupleTypeNode=e.Debug.deprecate(e.factory.createTupleTypeNode,n),e.updateTupleTypeNode=e.Debug.deprecate(e.factory.updateTupleTypeNode,n),e.createOptionalTypeNode=e.Debug.deprecate(e.factory.createOptionalTypeNode,n),e.updateOptionalTypeNode=e.Debug.deprecate(e.factory.updateOptionalTypeNode,n),e.createRestTypeNode=e.Debug.deprecate(e.factory.createRestTypeNode,n),e.updateRestTypeNode=e.Debug.deprecate(e.factory.updateRestTypeNode,n),e.createUnionTypeNode=e.Debug.deprecate(e.factory.createUnionTypeNode,n),e.updateUnionTypeNode=e.Debug.deprecate(e.factory.updateUnionTypeNode,n),e.createIntersectionTypeNode=e.Debug.deprecate(e.factory.createIntersectionTypeNode,n),e.updateIntersectionTypeNode=e.Debug.deprecate(e.factory.updateIntersectionTypeNode,n),e.createConditionalTypeNode=e.Debug.deprecate(e.factory.createConditionalTypeNode,n),e.updateConditionalTypeNode=e.Debug.deprecate(e.factory.updateConditionalTypeNode,n),e.createInferTypeNode=e.Debug.deprecate(e.factory.createInferTypeNode,n),e.updateInferTypeNode=e.Debug.deprecate(e.factory.updateInferTypeNode,n),e.createImportTypeNode=e.Debug.deprecate(e.factory.createImportTypeNode,n),e.updateImportTypeNode=e.Debug.deprecate(e.factory.updateImportTypeNode,n),e.createParenthesizedType=e.Debug.deprecate(e.factory.createParenthesizedType,n),e.updateParenthesizedType=e.Debug.deprecate(e.factory.updateParenthesizedType,n),e.createThisTypeNode=e.Debug.deprecate(e.factory.createThisTypeNode,n),e.updateTypeOperatorNode=e.Debug.deprecate(e.factory.updateTypeOperatorNode,n),e.createIndexedAccessTypeNode=e.Debug.deprecate(e.factory.createIndexedAccessTypeNode,n),e.updateIndexedAccessTypeNode=e.Debug.deprecate(e.factory.updateIndexedAccessTypeNode,n),e.createMappedTypeNode=e.Debug.deprecate(e.factory.createMappedTypeNode,n),e.updateMappedTypeNode=e.Debug.deprecate(e.factory.updateMappedTypeNode,n),e.createLiteralTypeNode=e.Debug.deprecate(e.factory.createLiteralTypeNode,n),e.updateLiteralTypeNode=e.Debug.deprecate(e.factory.updateLiteralTypeNode,n),e.createObjectBindingPattern=e.Debug.deprecate(e.factory.createObjectBindingPattern,n),e.updateObjectBindingPattern=e.Debug.deprecate(e.factory.updateObjectBindingPattern,n),e.createArrayBindingPattern=e.Debug.deprecate(e.factory.createArrayBindingPattern,n),e.updateArrayBindingPattern=e.Debug.deprecate(e.factory.updateArrayBindingPattern,n),e.createBindingElement=e.Debug.deprecate(e.factory.createBindingElement,n),e.updateBindingElement=e.Debug.deprecate(e.factory.updateBindingElement,n),e.createArrayLiteral=e.Debug.deprecate(e.factory.createArrayLiteralExpression,n),e.updateArrayLiteral=e.Debug.deprecate(e.factory.updateArrayLiteralExpression,n),e.createObjectLiteral=e.Debug.deprecate(e.factory.createObjectLiteralExpression,n),e.updateObjectLiteral=e.Debug.deprecate(e.factory.updateObjectLiteralExpression,n),e.createPropertyAccess=e.Debug.deprecate(e.factory.createPropertyAccessExpression,n),e.updatePropertyAccess=e.Debug.deprecate(e.factory.updatePropertyAccessExpression,n),e.createPropertyAccessChain=e.Debug.deprecate(e.factory.createPropertyAccessChain,n),e.updatePropertyAccessChain=e.Debug.deprecate(e.factory.updatePropertyAccessChain,n),e.createElementAccess=e.Debug.deprecate(e.factory.createElementAccessExpression,n),e.updateElementAccess=e.Debug.deprecate(e.factory.updateElementAccessExpression,n),e.createElementAccessChain=e.Debug.deprecate(e.factory.createElementAccessChain,n),e.updateElementAccessChain=e.Debug.deprecate(e.factory.updateElementAccessChain,n),e.createCall=e.Debug.deprecate(e.factory.createCallExpression,n),e.updateCall=e.Debug.deprecate(e.factory.updateCallExpression,n),e.createCallChain=e.Debug.deprecate(e.factory.createCallChain,n),e.updateCallChain=e.Debug.deprecate(e.factory.updateCallChain,n),e.createNew=e.Debug.deprecate(e.factory.createNewExpression,n),e.updateNew=e.Debug.deprecate(e.factory.updateNewExpression,n),e.createTypeAssertion=e.Debug.deprecate(e.factory.createTypeAssertion,n),e.updateTypeAssertion=e.Debug.deprecate(e.factory.updateTypeAssertion,n),e.createParen=e.Debug.deprecate(e.factory.createParenthesizedExpression,n),e.updateParen=e.Debug.deprecate(e.factory.updateParenthesizedExpression,n),e.createFunctionExpression=e.Debug.deprecate(e.factory.createFunctionExpression,n),e.updateFunctionExpression=e.Debug.deprecate(e.factory.updateFunctionExpression,n),e.createDelete=e.Debug.deprecate(e.factory.createDeleteExpression,n),e.updateDelete=e.Debug.deprecate(e.factory.updateDeleteExpression,n),e.createTypeOf=e.Debug.deprecate(e.factory.createTypeOfExpression,n),e.updateTypeOf=e.Debug.deprecate(e.factory.updateTypeOfExpression,n),e.createVoid=e.Debug.deprecate(e.factory.createVoidExpression,n),e.updateVoid=e.Debug.deprecate(e.factory.updateVoidExpression,n),e.createAwait=e.Debug.deprecate(e.factory.createAwaitExpression,n),e.updateAwait=e.Debug.deprecate(e.factory.updateAwaitExpression,n),e.createPrefix=e.Debug.deprecate(e.factory.createPrefixUnaryExpression,n),e.updatePrefix=e.Debug.deprecate(e.factory.updatePrefixUnaryExpression,n),e.createPostfix=e.Debug.deprecate(e.factory.createPostfixUnaryExpression,n),e.updatePostfix=e.Debug.deprecate(e.factory.updatePostfixUnaryExpression,n),e.createBinary=e.Debug.deprecate(e.factory.createBinaryExpression,n),e.updateConditional=e.Debug.deprecate(e.factory.updateConditionalExpression,n),e.createTemplateExpression=e.Debug.deprecate(e.factory.createTemplateExpression,n),e.updateTemplateExpression=e.Debug.deprecate(e.factory.updateTemplateExpression,n),e.createTemplateHead=e.Debug.deprecate(e.factory.createTemplateHead,n),e.createTemplateMiddle=e.Debug.deprecate(e.factory.createTemplateMiddle,n),e.createTemplateTail=e.Debug.deprecate(e.factory.createTemplateTail,n),e.createNoSubstitutionTemplateLiteral=e.Debug.deprecate(e.factory.createNoSubstitutionTemplateLiteral,n),e.updateYield=e.Debug.deprecate(e.factory.updateYieldExpression,n),e.createSpread=e.Debug.deprecate(e.factory.createSpreadElement,n),e.updateSpread=e.Debug.deprecate(e.factory.updateSpreadElement,n),e.createOmittedExpression=e.Debug.deprecate(e.factory.createOmittedExpression,n),e.createAsExpression=e.Debug.deprecate(e.factory.createAsExpression,n),e.updateAsExpression=e.Debug.deprecate(e.factory.updateAsExpression,n),e.createNonNullExpression=e.Debug.deprecate(e.factory.createNonNullExpression,n),e.updateNonNullExpression=e.Debug.deprecate(e.factory.updateNonNullExpression,n),e.createNonNullChain=e.Debug.deprecate(e.factory.createNonNullChain,n),e.updateNonNullChain=e.Debug.deprecate(e.factory.updateNonNullChain,n),e.createMetaProperty=e.Debug.deprecate(e.factory.createMetaProperty,n),e.updateMetaProperty=e.Debug.deprecate(e.factory.updateMetaProperty,n),e.createTemplateSpan=e.Debug.deprecate(e.factory.createTemplateSpan,n),e.updateTemplateSpan=e.Debug.deprecate(e.factory.updateTemplateSpan,n),e.createSemicolonClassElement=e.Debug.deprecate(e.factory.createSemicolonClassElement,n),e.createBlock=e.Debug.deprecate(e.factory.createBlock,n),e.updateBlock=e.Debug.deprecate(e.factory.updateBlock,n),e.createVariableStatement=e.Debug.deprecate(e.factory.createVariableStatement,n),e.updateVariableStatement=e.Debug.deprecate(e.factory.updateVariableStatement,n),e.createEmptyStatement=e.Debug.deprecate(e.factory.createEmptyStatement,n),e.createExpressionStatement=e.Debug.deprecate(e.factory.createExpressionStatement,n),e.updateExpressionStatement=e.Debug.deprecate(e.factory.updateExpressionStatement,n),e.createStatement=e.Debug.deprecate(e.factory.createExpressionStatement,n),e.updateStatement=e.Debug.deprecate(e.factory.updateExpressionStatement,n),e.createIf=e.Debug.deprecate(e.factory.createIfStatement,n),e.updateIf=e.Debug.deprecate(e.factory.updateIfStatement,n),e.createDo=e.Debug.deprecate(e.factory.createDoStatement,n),e.updateDo=e.Debug.deprecate(e.factory.updateDoStatement,n),e.createWhile=e.Debug.deprecate(e.factory.createWhileStatement,n),e.updateWhile=e.Debug.deprecate(e.factory.updateWhileStatement,n),e.createFor=e.Debug.deprecate(e.factory.createForStatement,n),e.updateFor=e.Debug.deprecate(e.factory.updateForStatement,n),e.createForIn=e.Debug.deprecate(e.factory.createForInStatement,n),e.updateForIn=e.Debug.deprecate(e.factory.updateForInStatement,n),e.createForOf=e.Debug.deprecate(e.factory.createForOfStatement,n),e.updateForOf=e.Debug.deprecate(e.factory.updateForOfStatement,n),e.createContinue=e.Debug.deprecate(e.factory.createContinueStatement,n),e.updateContinue=e.Debug.deprecate(e.factory.updateContinueStatement,n),e.createBreak=e.Debug.deprecate(e.factory.createBreakStatement,n),e.updateBreak=e.Debug.deprecate(e.factory.updateBreakStatement,n),e.createReturn=e.Debug.deprecate(e.factory.createReturnStatement,n),e.updateReturn=e.Debug.deprecate(e.factory.updateReturnStatement,n),e.createWith=e.Debug.deprecate(e.factory.createWithStatement,n),e.updateWith=e.Debug.deprecate(e.factory.updateWithStatement,n),e.createSwitch=e.Debug.deprecate(e.factory.createSwitchStatement,n),e.updateSwitch=e.Debug.deprecate(e.factory.updateSwitchStatement,n),e.createLabel=e.Debug.deprecate(e.factory.createLabeledStatement,n),e.updateLabel=e.Debug.deprecate(e.factory.updateLabeledStatement,n),e.createThrow=e.Debug.deprecate(e.factory.createThrowStatement,n),e.updateThrow=e.Debug.deprecate(e.factory.updateThrowStatement,n),e.createTry=e.Debug.deprecate(e.factory.createTryStatement,n),e.updateTry=e.Debug.deprecate(e.factory.updateTryStatement,n),e.createDebuggerStatement=e.Debug.deprecate(e.factory.createDebuggerStatement,n),e.createVariableDeclarationList=e.Debug.deprecate(e.factory.createVariableDeclarationList,n),e.updateVariableDeclarationList=e.Debug.deprecate(e.factory.updateVariableDeclarationList,n),e.createFunctionDeclaration=e.Debug.deprecate(e.factory.createFunctionDeclaration,n),e.updateFunctionDeclaration=e.Debug.deprecate(e.factory.updateFunctionDeclaration,n),e.createClassDeclaration=e.Debug.deprecate(e.factory.createClassDeclaration,n),e.updateClassDeclaration=e.Debug.deprecate(e.factory.updateClassDeclaration,n),e.createInterfaceDeclaration=e.Debug.deprecate(e.factory.createInterfaceDeclaration,n),e.updateInterfaceDeclaration=e.Debug.deprecate(e.factory.updateInterfaceDeclaration,n),e.createTypeAliasDeclaration=e.Debug.deprecate(e.factory.createTypeAliasDeclaration,n),e.updateTypeAliasDeclaration=e.Debug.deprecate(e.factory.updateTypeAliasDeclaration,n),e.createEnumDeclaration=e.Debug.deprecate(e.factory.createEnumDeclaration,n),e.updateEnumDeclaration=e.Debug.deprecate(e.factory.updateEnumDeclaration,n),e.createModuleDeclaration=e.Debug.deprecate(e.factory.createModuleDeclaration,n),e.updateModuleDeclaration=e.Debug.deprecate(e.factory.updateModuleDeclaration,n),e.createModuleBlock=e.Debug.deprecate(e.factory.createModuleBlock,n),e.updateModuleBlock=e.Debug.deprecate(e.factory.updateModuleBlock,n),e.createCaseBlock=e.Debug.deprecate(e.factory.createCaseBlock,n),e.updateCaseBlock=e.Debug.deprecate(e.factory.updateCaseBlock,n),e.createNamespaceExportDeclaration=e.Debug.deprecate(e.factory.createNamespaceExportDeclaration,n),e.updateNamespaceExportDeclaration=e.Debug.deprecate(e.factory.updateNamespaceExportDeclaration,n),e.createImportEqualsDeclaration=e.Debug.deprecate(e.factory.createImportEqualsDeclaration,n),e.updateImportEqualsDeclaration=e.Debug.deprecate(e.factory.updateImportEqualsDeclaration,n),e.createImportDeclaration=e.Debug.deprecate(e.factory.createImportDeclaration,n),e.updateImportDeclaration=e.Debug.deprecate(e.factory.updateImportDeclaration,n),e.createNamespaceImport=e.Debug.deprecate(e.factory.createNamespaceImport,n),e.updateNamespaceImport=e.Debug.deprecate(e.factory.updateNamespaceImport,n),e.createNamedImports=e.Debug.deprecate(e.factory.createNamedImports,n),e.updateNamedImports=e.Debug.deprecate(e.factory.updateNamedImports,n),e.createImportSpecifier=e.Debug.deprecate(e.factory.createImportSpecifier,n),e.updateImportSpecifier=e.Debug.deprecate(e.factory.updateImportSpecifier,n),e.createExportAssignment=e.Debug.deprecate(e.factory.createExportAssignment,n),e.updateExportAssignment=e.Debug.deprecate(e.factory.updateExportAssignment,n),e.createNamedExports=e.Debug.deprecate(e.factory.createNamedExports,n),e.updateNamedExports=e.Debug.deprecate(e.factory.updateNamedExports,n),e.createExportSpecifier=e.Debug.deprecate(e.factory.createExportSpecifier,n),e.updateExportSpecifier=e.Debug.deprecate(e.factory.updateExportSpecifier,n),e.createExternalModuleReference=e.Debug.deprecate(e.factory.createExternalModuleReference,n),e.updateExternalModuleReference=e.Debug.deprecate(e.factory.updateExternalModuleReference,n),e.createJSDocTypeExpression=e.Debug.deprecate(e.factory.createJSDocTypeExpression,n),e.createJSDocTypeTag=e.Debug.deprecate(e.factory.createJSDocTypeTag,n),e.createJSDocReturnTag=e.Debug.deprecate(e.factory.createJSDocReturnTag,n),e.createJSDocThisTag=e.Debug.deprecate(e.factory.createJSDocThisTag,n),e.createJSDocComment=e.Debug.deprecate(e.factory.createJSDocComment,n),e.createJSDocParameterTag=e.Debug.deprecate(e.factory.createJSDocParameterTag,n),e.createJSDocClassTag=e.Debug.deprecate(e.factory.createJSDocClassTag,n),e.createJSDocAugmentsTag=e.Debug.deprecate(e.factory.createJSDocAugmentsTag,n),e.createJSDocEnumTag=e.Debug.deprecate(e.factory.createJSDocEnumTag,n),e.createJSDocTemplateTag=e.Debug.deprecate(e.factory.createJSDocTemplateTag,n),e.createJSDocTypedefTag=e.Debug.deprecate(e.factory.createJSDocTypedefTag,n),e.createJSDocCallbackTag=e.Debug.deprecate(e.factory.createJSDocCallbackTag,n),e.createJSDocSignature=e.Debug.deprecate(e.factory.createJSDocSignature,n),e.createJSDocPropertyTag=e.Debug.deprecate(e.factory.createJSDocPropertyTag,n),e.createJSDocTypeLiteral=e.Debug.deprecate(e.factory.createJSDocTypeLiteral,n),e.createJSDocImplementsTag=e.Debug.deprecate(e.factory.createJSDocImplementsTag,n),e.createJSDocAuthorTag=e.Debug.deprecate(e.factory.createJSDocAuthorTag,n),e.createJSDocPublicTag=e.Debug.deprecate(e.factory.createJSDocPublicTag,n),e.createJSDocPrivateTag=e.Debug.deprecate(e.factory.createJSDocPrivateTag,n),e.createJSDocProtectedTag=e.Debug.deprecate(e.factory.createJSDocProtectedTag,n),e.createJSDocReadonlyTag=e.Debug.deprecate(e.factory.createJSDocReadonlyTag,n),e.createJSDocTag=e.Debug.deprecate(e.factory.createJSDocUnknownTag,n),e.createJsxElement=e.Debug.deprecate(e.factory.createJsxElement,n),e.updateJsxElement=e.Debug.deprecate(e.factory.updateJsxElement,n),e.createJsxSelfClosingElement=e.Debug.deprecate(e.factory.createJsxSelfClosingElement,n),e.updateJsxSelfClosingElement=e.Debug.deprecate(e.factory.updateJsxSelfClosingElement,n),e.createJsxOpeningElement=e.Debug.deprecate(e.factory.createJsxOpeningElement,n),e.updateJsxOpeningElement=e.Debug.deprecate(e.factory.updateJsxOpeningElement,n),e.createJsxClosingElement=e.Debug.deprecate(e.factory.createJsxClosingElement,n),e.updateJsxClosingElement=e.Debug.deprecate(e.factory.updateJsxClosingElement,n),e.createJsxFragment=e.Debug.deprecate(e.factory.createJsxFragment,n),e.createJsxText=e.Debug.deprecate(e.factory.createJsxText,n),e.updateJsxText=e.Debug.deprecate(e.factory.updateJsxText,n),e.createJsxOpeningFragment=e.Debug.deprecate(e.factory.createJsxOpeningFragment,n),e.createJsxJsxClosingFragment=e.Debug.deprecate(e.factory.createJsxJsxClosingFragment,n),e.updateJsxFragment=e.Debug.deprecate(e.factory.updateJsxFragment,n),e.createJsxAttribute=e.Debug.deprecate(e.factory.createJsxAttribute,n),e.updateJsxAttribute=e.Debug.deprecate(e.factory.updateJsxAttribute,n),e.createJsxAttributes=e.Debug.deprecate(e.factory.createJsxAttributes,n),e.updateJsxAttributes=e.Debug.deprecate(e.factory.updateJsxAttributes,n),e.createJsxSpreadAttribute=e.Debug.deprecate(e.factory.createJsxSpreadAttribute,n),e.updateJsxSpreadAttribute=e.Debug.deprecate(e.factory.updateJsxSpreadAttribute,n),e.createJsxExpression=e.Debug.deprecate(e.factory.createJsxExpression,n),e.updateJsxExpression=e.Debug.deprecate(e.factory.updateJsxExpression,n),e.createCaseClause=e.Debug.deprecate(e.factory.createCaseClause,n),e.updateCaseClause=e.Debug.deprecate(e.factory.updateCaseClause,n),e.createDefaultClause=e.Debug.deprecate(e.factory.createDefaultClause,n),e.updateDefaultClause=e.Debug.deprecate(e.factory.updateDefaultClause,n),e.createHeritageClause=e.Debug.deprecate(e.factory.createHeritageClause,n),e.updateHeritageClause=e.Debug.deprecate(e.factory.updateHeritageClause,n),e.createCatchClause=e.Debug.deprecate(e.factory.createCatchClause,n),e.updateCatchClause=e.Debug.deprecate(e.factory.updateCatchClause,n),e.createPropertyAssignment=e.Debug.deprecate(e.factory.createPropertyAssignment,n),e.updatePropertyAssignment=e.Debug.deprecate(e.factory.updatePropertyAssignment,n),e.createShorthandPropertyAssignment=e.Debug.deprecate(e.factory.createShorthandPropertyAssignment,n),e.updateShorthandPropertyAssignment=e.Debug.deprecate(e.factory.updateShorthandPropertyAssignment,n),e.createSpreadAssignment=e.Debug.deprecate(e.factory.createSpreadAssignment,n),e.updateSpreadAssignment=e.Debug.deprecate(e.factory.updateSpreadAssignment,n),e.createEnumMember=e.Debug.deprecate(e.factory.createEnumMember,n),e.updateEnumMember=e.Debug.deprecate(e.factory.updateEnumMember,n),e.updateSourceFileNode=e.Debug.deprecate(e.factory.updateSourceFile,n),e.createNotEmittedStatement=e.Debug.deprecate(e.factory.createNotEmittedStatement,n),e.createPartiallyEmittedExpression=e.Debug.deprecate(e.factory.createPartiallyEmittedExpression,n),e.updatePartiallyEmittedExpression=e.Debug.deprecate(e.factory.updatePartiallyEmittedExpression,n),e.createCommaList=e.Debug.deprecate(e.factory.createCommaListExpression,n),e.updateCommaList=e.Debug.deprecate(e.factory.updateCommaListExpression,n),e.createBundle=e.Debug.deprecate(e.factory.createBundle,n),e.updateBundle=e.Debug.deprecate(e.factory.updateBundle,n),e.createImmediatelyInvokedFunctionExpression=e.Debug.deprecate(e.factory.createImmediatelyInvokedFunctionExpression,n),e.createImmediatelyInvokedArrowFunction=e.Debug.deprecate(e.factory.createImmediatelyInvokedArrowFunction,n),e.createVoidZero=e.Debug.deprecate(e.factory.createVoidZero,n),e.createExportDefault=e.Debug.deprecate(e.factory.createExportDefault,n),e.createExternalModuleExport=e.Debug.deprecate(e.factory.createExternalModuleExport,n),e.createNamespaceExport=e.Debug.deprecate(e.factory.createNamespaceExport,n),e.updateNamespaceExport=e.Debug.deprecate(e.factory.updateNamespaceExport,n),e.createToken=e.Debug.deprecate(function(n){return e.factory.createToken(n)},n),e.createIdentifier=e.Debug.deprecate(function(n){return e.factory.createIdentifier(n,void 0,void 0)},n),e.createTempVariable=e.Debug.deprecate(function(n){return e.factory.createTempVariable(n,void 0)},n),e.getGeneratedNameForNode=e.Debug.deprecate(function(n){return e.factory.getGeneratedNameForNode(n,void 0)},n),e.createOptimisticUniqueName=e.Debug.deprecate(function(n){return e.factory.createUniqueName(n,16)},n),e.createFileLevelUniqueName=e.Debug.deprecate(function(n){return e.factory.createUniqueName(n,48)},n),e.createIndexSignature=e.Debug.deprecate(function(n,t,r,i){return e.factory.createIndexSignature(n,t,r,i)},n),e.createTypePredicateNode=e.Debug.deprecate(function(n,t){return e.factory.createTypePredicateNode(void 0,n,t)},n),e.updateTypePredicateNode=e.Debug.deprecate(function(n,t,r){return e.factory.updateTypePredicateNode(n,void 0,t,r)},n),e.createLiteral=e.Debug.deprecate(function(n){return"number"===typeof n?e.factory.createNumericLiteral(n):"object"===typeof n&&"base10Value"in n?e.factory.createBigIntLiteral(n):"boolean"===typeof n?n?e.factory.createTrue():e.factory.createFalse():"string"===typeof n?e.factory.createStringLiteral(n,void 0):e.factory.createStringLiteralFromNode(n)},{since:"4.0",warnAfter:"4.1",message:"Use `factory.createStringLiteral`, `factory.createStringLiteralFromNode`, `factory.createNumericLiteral`, `factory.createBigIntLiteral`, `factory.createTrue`, `factory.createFalse`, or the factory supplied by your transformation context instead."}),e.createMethodSignature=e.Debug.deprecate(function(n,t,r,i,a){return e.factory.createMethodSignature(void 0,i,a,n,t,r)},n),e.updateMethodSignature=e.Debug.deprecate(function(n,t,r,i,a,o){return e.factory.updateMethodSignature(n,n.modifiers,a,o,t,r,i)},n),e.createTypeOperatorNode=e.Debug.deprecate(function(n,t){var r;return t?r=n:(t=n,r=139),e.factory.createTypeOperatorNode(r,t)},n),e.createTaggedTemplate=e.Debug.deprecate(function(n,t,r){var i;return r?i=t:r=t,e.factory.createTaggedTemplateExpression(n,i,r)},n),e.updateTaggedTemplate=e.Debug.deprecate(function(n,t,r,i){var a;return i?a=r:i=r,e.factory.updateTaggedTemplateExpression(n,t,a,i)},n),e.updateBinary=e.Debug.deprecate(function(n,t,r,i){return void 0===i&&(i=n.operatorToken),"number"===typeof i&&(i=i===n.operatorToken.kind?n.operatorToken:e.factory.createToken(i)),e.factory.updateBinaryExpression(n,t,i,r)},n),e.createConditional=e.Debug.deprecate(function(n,t,r,i,a){return 5===arguments.length?e.factory.createConditionalExpression(n,t,r,i,a):3===arguments.length?e.factory.createConditionalExpression(n,e.factory.createToken(57),t,e.factory.createToken(58),r):e.Debug.fail("Argument count mismatch")},n),e.createYield=e.Debug.deprecate(function(n,t){var r;return t?r=n:t=n,e.factory.createYieldExpression(r,t)},n),e.createClassExpression=e.Debug.deprecate(function(n,t,r,i,a){return e.factory.createClassExpression(void 0,n,t,r,i,a)},n),e.updateClassExpression=e.Debug.deprecate(function(n,t,r,i,a,o){return e.factory.updateClassExpression(n,void 0,t,r,i,a,o)},n),e.createPropertySignature=e.Debug.deprecate(function(n,t,r,i,a){var o=e.factory.createPropertySignature(n,t,r,i);return o.initializer=a,o},n),e.updatePropertySignature=e.Debug.deprecate(function(n,t,r,i,a,o){var s=e.factory.updatePropertySignature(n,t,r,i,a);return n.initializer!==o&&(s===n&&(s=e.factory.cloneNode(n)),s.initializer=o),s},n),e.createExpressionWithTypeArguments=e.Debug.deprecate(function(n,t){return e.factory.createExpressionWithTypeArguments(t,n)},n),e.updateExpressionWithTypeArguments=e.Debug.deprecate(function(n,t,r){return e.factory.updateExpressionWithTypeArguments(n,r,t)},n),e.createArrowFunction=e.Debug.deprecate(function(n,t,r,i,a,o){return 6===arguments.length?e.factory.createArrowFunction(n,t,r,i,a,o):5===arguments.length?e.factory.createArrowFunction(n,t,r,i,void 0,a):e.Debug.fail("Argument count mismatch")},n),e.updateArrowFunction=e.Debug.deprecate(function(n,t,r,i,a,o,s){return 7===arguments.length?e.factory.updateArrowFunction(n,t,r,i,a,o,s):6===arguments.length?e.factory.updateArrowFunction(n,t,r,i,a,n.equalsGreaterThanToken,o):e.Debug.fail("Argument count mismatch")},n),e.createVariableDeclaration=e.Debug.deprecate(function(n,t,r,i){return 4===arguments.length?e.factory.createVariableDeclaration(n,t,r,i):arguments.length>=1&&arguments.length<=3?e.factory.createVariableDeclaration(n,void 0,t,r):e.Debug.fail("Argument count mismatch")},n),e.updateVariableDeclaration=e.Debug.deprecate(function(n,t,r,i,a){return 5===arguments.length?e.factory.updateVariableDeclaration(n,t,r,i,a):4===arguments.length?e.factory.updateVariableDeclaration(n,t,n.exclamationToken,r,i):e.Debug.fail("Argument count mismatch")},n),e.createImportClause=e.Debug.deprecate(function(n,t,r){return void 0===r&&(r=!1),e.factory.createImportClause(r,n,t)},n),e.updateImportClause=e.Debug.deprecate(function(n,t,r,i){return e.factory.updateImportClause(n,i,t,r)},n),e.createExportDeclaration=e.Debug.deprecate(function(n,t,r,i,a){return void 0===a&&(a=!1),e.factory.createExportDeclaration(n,t,a,r,i)},n),e.updateExportDeclaration=e.Debug.deprecate(function(n,t,r,i,a,o){return e.factory.updateExportDeclaration(n,t,r,o,i,a)},n),e.createJSDocParamTag=e.Debug.deprecate(function(n,t,r,i){return e.factory.createJSDocParameterTag(void 0,n,t,r,!1,i?e.factory.createNodeArray([e.factory.createJSDocText(i)]):void 0)},n),e.createComma=e.Debug.deprecate(function(n,t){return e.factory.createComma(n,t)},n),e.createLessThan=e.Debug.deprecate(function(n,t){return e.factory.createLessThan(n,t)},n),e.createAssignment=e.Debug.deprecate(function(n,t){return e.factory.createAssignment(n,t)},n),e.createStrictEquality=e.Debug.deprecate(function(n,t){return e.factory.createStrictEquality(n,t)},n),e.createStrictInequality=e.Debug.deprecate(function(n,t){return e.factory.createStrictInequality(n,t)},n),e.createAdd=e.Debug.deprecate(function(n,t){return e.factory.createAdd(n,t)},n),e.createSubtract=e.Debug.deprecate(function(n,t){return e.factory.createSubtract(n,t)},n),e.createLogicalAnd=e.Debug.deprecate(function(n,t){return e.factory.createLogicalAnd(n,t)},n),e.createLogicalOr=e.Debug.deprecate(function(n,t){return e.factory.createLogicalOr(n,t)},n),e.createPostfixIncrement=e.Debug.deprecate(function(n){return e.factory.createPostfixIncrement(n)},n),e.createLogicalNot=e.Debug.deprecate(function(n){return e.factory.createLogicalNot(n)},n),e.createNode=e.Debug.deprecate(function(n,t,r){return void 0===t&&(t=0),void 0===r&&(r=0),e.setTextRangePosEnd(300===n?e.parseBaseNodeFactory.createBaseSourceFileNode(n):79===n?e.parseBaseNodeFactory.createBaseIdentifierNode(n):80===n?e.parseBaseNodeFactory.createBasePrivateIdentifierNode(n):e.isNodeKind(n)?e.parseBaseNodeFactory.createBaseNode(n):e.parseBaseNodeFactory.createBaseTokenNode(n),t,r)},{since:"4.0",warnAfter:"4.1",message:"Use an appropriate `factory` method instead."}),e.getMutableClone=e.Debug.deprecate(function(n){var t=e.factory.cloneNode(n);return e.setTextRange(t,n),e.setParent(t,n.parent),t},{since:"4.0",warnAfter:"4.1",message:"Use an appropriate `factory.update...` method instead, use `setCommentRange` or `setSourceMapRange`, and avoid setting `parent`."}),e.isTypeAssertion=e.Debug.deprecate(function(e){return 209===e.kind},{since:"4.0",warnAfter:"4.1",message:"Use `isTypeAssertionExpression` instead."}),e.isIdentifierOrPrivateIdentifier=e.Debug.deprecate(function(n){return e.isMemberName(n)},{since:"4.2",warnAfter:"4.3",message:"Use `isMemberName` instead."})}(d||(d={}));var f=d.createClassifier,m=d.createLanguageService,_=d.displayPartsToString,g=d.EndOfLineState,y=d.flattenDiagnosticMessageText,h=d.IndentStyle,v=d.ScriptKind,b=d.ScriptTarget,E=d.TokenClass}.call(this,t("f28c"),"/",t("dd40")(e))},c317:function(e,n,t){"use strict";(function(e){t.d(n,"a",function(){return a}),t.d(n,"b",function(){return o}),t.d(n,"c",function(){return s});var r=t("30db");let i;if("undefined"!==typeof r["a"].vscode&&"undefined"!==typeof r["a"].vscode.process){const e=r["a"].vscode.process;i={get platform(){return e.platform},get arch(){return e.arch},get env(){return e.env},cwd(){return e.cwd()},nextTick(e){return Object(r["e"])(e)}}}else i="undefined"!==typeof e?{get platform(){return e.platform},get arch(){return e.arch},get env(){return Object({NODE_ENV:"production",BASE_URL:"/ui/static/"})},cwd(){return Object({NODE_ENV:"production",BASE_URL:"/ui/static/"})["VSCODE_CWD"]||e.cwd()},nextTick(n){return e.nextTick(n)}}:{get platform(){return r["d"]?"win32":r["b"]?"darwin":"linux"},get arch(){},nextTick(e){return Object(r["e"])(e)},get env(){return{}},cwd(){return"/"}};const a=i.cwd,o=i.env,s=i.platform}).call(this,t("f28c"))},c8ba:function(e,n){var t;t=function(){return this}();try{t=t||new Function("return this")()}catch(r){"object"===typeof window&&(t=window)}e.exports=t},d0c7:function(e,n,t){"use strict";t.r(n);class r{constructor(){this.listeners=[],this.unexpectedErrorHandler=function(e){setTimeout(()=>{if(e.stack)throw new Error(e.message+"\n\n"+e.stack);throw e},0)}}emit(e){this.listeners.forEach(n=>{n(e)})}onUnexpectedError(e){this.unexpectedErrorHandler(e),this.emit(e)}onUnexpectedExternalError(e){this.unexpectedErrorHandler(e)}}const i=new r;function a(e){c(e)||i.onUnexpectedError(e)}function o(e){if(e instanceof Error){let{name:n,message:t}=e;const r=e.stacktrace||e.stack;return{$isError:!0,name:n,message:t,stack:r}}return e}const s="Canceled";function c(e){return e instanceof Error&&e.name===s&&e.message===s}function l(e){const n=this;let t,r=!1;return function(){return r?t:(r=!0,t=e.apply(n,arguments),t)}}var u;(function(e){function n(e){return e&&"object"===typeof e&&"function"===typeof e[Symbol.iterator]}e.is=n;const t=Object.freeze([]);function r(){return t}function*i(e){yield e}function a(e){return e||t}function o(e){return!e||!0===e[Symbol.iterator]().next().done}function s(e){return e[Symbol.iterator]().next().value}function c(e,n){for(const t of e)if(n(t))return!0;return!1}function l(e,n){for(const t of e)if(n(t))return t}function*u(e,n){for(const t of e)n(t)&&(yield t)}function*d(e,n){let t=0;for(const r of e)yield n(r,t++)}function*p(...e){for(const n of e)for(const e of n)yield e}function*f(e){for(const n of e)for(const e of n)yield e}function m(e,n,t){let r=t;for(const i of e)r=n(r,i);return r}function*_(e,n,t=e.length){for(n<0&&(n+=e.length),t<0?t+=e.length:t>e.length&&(t=e.length);n<t;n++)yield e[n]}function g(n,t=Number.POSITIVE_INFINITY){const r=[];if(0===t)return[r,n];const i=n[Symbol.iterator]();for(let a=0;a<t;a++){const n=i.next();if(n.done)return[r,e.empty()];r.push(n.value)}return[r,{[Symbol.iterator](){return i}}]}function y(e,n,t=((e,n)=>e===n)){const r=e[Symbol.iterator](),i=n[Symbol.iterator]();while(1){const e=r.next(),n=i.next();if(e.done!==n.done)return!1;if(e.done)return!0;if(!t(e.value,n.value))return!1}}e.empty=r,e.single=i,e.from=a,e.isEmpty=o,e.first=s,e.some=c,e.find=l,e.filter=u,e.map=d,e.concat=p,e.concatNested=f,e.reduce=m,e.slice=_,e.consume=g,e.equals=y})(u||(u={}));const d=!1;let p=null;function f(e){p=e}if(d){const e="__is_disposable_tracked__";f(new class{trackDisposable(n){const t=new Error("Potentially leaked disposable").stack;setTimeout(()=>{n[e]||console.log(t)},3e3)}setParent(n,t){if(n&&n!==S.None)try{n[e]=!0}catch(r){}}markAsDisposed(n){if(n&&n!==S.None)try{n[e]=!0}catch(t){}}markAsSingleton(e){}})}function m(e){return null===p||void 0===p||p.trackDisposable(e),e}function _(e){null===p||void 0===p||p.markAsDisposed(e)}function g(e,n){null===p||void 0===p||p.setParent(e,n)}function y(e,n){if(p)for(const t of e)p.setParent(t,n)}class h extends Error{constructor(e){super(`Encountered errors while disposing of store. Errors: [${e.join(", ")}]`),this.errors=e}}function v(e){if(u.is(e)){let t=[];for(const r of e)if(r)try{r.dispose()}catch(n){t.push(n)}if(1===t.length)throw t[0];if(t.length>1)throw new h(t);return Array.isArray(e)?[]:e}if(e)return e.dispose(),e}function b(...e){const n=E(()=>v(e));return y(e,n),n}function E(e){const n=m({dispose:l(()=>{_(n),e()})});return n}class T{constructor(){this._toDispose=new Set,this._isDisposed=!1,m(this)}dispose(){this._isDisposed||(_(this),this._isDisposed=!0,this.clear())}clear(){try{v(this._toDispose.values())}finally{this._toDispose.clear()}}add(e){if(!e)return e;if(e===this)throw new Error("Cannot register a disposable on itself!");return g(e,this),this._isDisposed?T.DISABLE_DISPOSED_WARNING||console.warn(new Error("Trying to add a disposable to a DisposableStore that has already been disposed of. The added object will be leaked!").stack):this._toDispose.add(e),e}}T.DISABLE_DISPOSED_WARNING=!1;class S{constructor(){this._store=new T,m(this),g(this._store,this)}dispose(){_(this),this._store.dispose()}_register(e){if(e===this)throw new Error("Cannot register a disposable on itself!");return this._store.add(e)}}S.None=Object.freeze({dispose(){}});class x{constructor(e){this.element=e,this.next=x.Undefined,this.prev=x.Undefined}}x.Undefined=new x(void 0);class L{constructor(){this._first=x.Undefined,this._last=x.Undefined,this._size=0}get size(){return this._size}isEmpty(){return this._first===x.Undefined}clear(){let e=this._first;while(e!==x.Undefined){const n=e.next;e.prev=x.Undefined,e.next=x.Undefined,e=n}this._first=x.Undefined,this._last=x.Undefined,this._size=0}unshift(e){return this._insert(e,!1)}push(e){return this._insert(e,!0)}_insert(e,n){const t=new x(e);if(this._first===x.Undefined)this._first=t,this._last=t;else if(n){const e=this._last;this._last=t,t.prev=e,e.next=t}else{const e=this._first;this._first=t,t.next=e,e.prev=t}this._size+=1;let r=!1;return()=>{r||(r=!0,this._remove(t))}}shift(){if(this._first!==x.Undefined){const e=this._first.element;return this._remove(this._first),e}}pop(){if(this._last!==x.Undefined){const e=this._last.element;return this._remove(this._last),e}}_remove(e){if(e.prev!==x.Undefined&&e.next!==x.Undefined){const n=e.prev;n.next=e.next,e.next.prev=n}else e.prev===x.Undefined&&e.next===x.Undefined?(this._first=x.Undefined,this._last=x.Undefined):e.next===x.Undefined?(this._last=this._last.prev,this._last.next=x.Undefined):e.prev===x.Undefined&&(this._first=this._first.next,this._first.prev=x.Undefined);this._size-=1}*[Symbol.iterator](){let e=this._first;while(e!==x.Undefined)yield e.element,e=e.next}}var D=t("30db");const A=D["a"].performance&&"function"===typeof D["a"].performance.now;class C{constructor(e){this._highResolution=A&&e,this._startTime=this._now(),this._stopTime=-1}static create(e=!0){return new C(e)}stop(){this._stopTime=this._now()}elapsed(){return-1!==this._stopTime?this._stopTime-this._startTime:this._now()-this._startTime}_now(){return this._highResolution?D["a"].performance.now():Date.now()}}var I;(function(e){function n(e){return(n,t=null,r)=>{let i,a=!1;return i=e(e=>{if(!a)return i?i.dispose():a=!0,n.call(t,e)},null,r),a&&i.dispose(),i}}function t(e,n){return c((t,r=null,i)=>e(e=>t.call(r,n(e)),null,i))}function r(e,n){return c((t,r=null,i)=>e(e=>{n(e),t.call(r,e)},null,i))}function i(e,n){return c((t,r=null,i)=>e(e=>n(e)&&t.call(r,e),null,i))}function a(e){return e}function o(...e){return(n,t=null,r)=>b(...e.map(e=>e(e=>n.call(t,e),null,r)))}function s(e,n,r){let i=r;return t(e,e=>{return i=n(i,e),i})}function c(e){let n;const t=new O({onFirstListenerAdd(){n=e(t.fire,t)},onLastListenerRemove(){n.dispose()}});return t.event}function l(e,n,t=100,r=!1,i){let a,o=void 0,s=void 0,c=0;const l=new O({leakWarningThreshold:i,onFirstListenerAdd(){a=e(e=>{c++,o=n(o,e),r&&!s&&(l.fire(o),o=void 0),clearTimeout(s),s=setTimeout(()=>{const e=o;o=void 0,s=void 0,(!r||c>1)&&l.fire(e),c=0},t)})},onLastListenerRemove(){a.dispose()}});return l.event}function u(e,n=((e,n)=>e===n)){let t,r=!0;return i(e,e=>{const i=r||!n(e,t);return r=!1,t=e,i})}function d(n,t){return[e.filter(n,t),e.filter(n,e=>!t(e))]}function p(e,n=!1,t=[]){let r=t.slice(),i=e(e=>{r?r.push(e):o.fire(e)});const a=()=>{r&&r.forEach(e=>o.fire(e)),r=null},o=new O({onFirstListenerAdd(){i||(i=e(e=>o.fire(e)))},onFirstListenerDidAdd(){r&&(n?setTimeout(a):a())},onLastListenerRemove(){i&&i.dispose(),i=null}});return o.event}e.None=()=>S.None,e.once=n,e.map=t,e.forEach=r,e.filter=i,e.signal=a,e.any=o,e.reduce=s,e.debounce=l,e.latch=u,e.split=d,e.buffer=p;class f{constructor(e){this.event=e}map(e){return new f(t(this.event,e))}forEach(e){return new f(r(this.event,e))}filter(e){return new f(i(this.event,e))}reduce(e,n){return new f(s(this.event,e,n))}latch(){return new f(u(this.event))}debounce(e,n=100,t=!1,r){return new f(l(this.event,e,n,t,r))}on(e,n,t){return this.event(e,n,t)}once(e,t,r){return n(this.event)(e,t,r)}}function m(e){return new f(e)}function _(e,n,t=(e=>e)){const r=(...e)=>o.fire(t(...e)),i=()=>e.on(n,r),a=()=>e.removeListener(n,r),o=new O({onFirstListenerAdd:i,onLastListenerRemove:a});return o.event}function g(e,n,t=(e=>e)){const r=(...e)=>o.fire(t(...e)),i=()=>e.addEventListener(n,r),a=()=>e.removeEventListener(n,r),o=new O({onFirstListenerAdd:i,onLastListenerRemove:a});return o.event}function y(e){return new Promise(t=>n(e)(t))}e.chain=m,e.fromNodeEventEmitter=_,e.fromDOMEventEmitter=g,e.toPromise=y})(I||(I={}));class N{constructor(e){this._listenerCount=0,this._invocationCount=0,this._elapsedOverall=0,this._name=`${e}_${N._idPool++}`}start(e){this._stopWatch=new C(!0),this._listenerCount=e}stop(){if(this._stopWatch){const e=this._stopWatch.elapsed();this._elapsedOverall+=e,this._invocationCount+=1,console.info(`did FIRE ${this._name}: elapsed_ms: ${e.toFixed(5)}, listener: ${this._listenerCount} (elapsed_overall: ${this._elapsedOverall.toFixed(2)}, invocations: ${this._invocationCount})`),this._stopWatch=void 0}}}N._idPool=0;let k=-1;class R{constructor(e,n=Math.random().toString(18).slice(2,5)){this.customThreshold=e,this.name=n,this._warnCountdown=0}dispose(){this._stacks&&this._stacks.clear()}check(e){let n=k;if("number"===typeof this.customThreshold&&(n=this.customThreshold),n<=0||e<n)return;this._stacks||(this._stacks=new Map);const t=(new Error).stack.split("\n").slice(3).join("\n"),r=this._stacks.get(t)||0;if(this._stacks.set(t,r+1),this._warnCountdown-=1,this._warnCountdown<=0){let t;this._warnCountdown=.5*n;let r=0;for(const[e,n]of this._stacks)(!t||r<n)&&(t=e,r=n);console.warn(`[${this.name}] potential listener LEAK detected, having ${e} listeners already. MOST frequent listener (${r}):`),console.warn(t)}return()=>{const e=this._stacks.get(t)||0;this._stacks.set(t,e-1)}}}class O{constructor(e){var n;this._disposed=!1,this._options=e,this._leakageMon=k>0?new R(this._options&&this._options.leakWarningThreshold):void 0,this._perfMon=(null===(n=this._options)||void 0===n?void 0:n._profName)?new N(this._options._profName):void 0}get event(){return this._event||(this._event=(e,n,t)=>{var r;this._listeners||(this._listeners=new L);const i=this._listeners.isEmpty();i&&this._options&&this._options.onFirstListenerAdd&&this._options.onFirstListenerAdd(this);const a=this._listeners.push(n?[e,n]:e);i&&this._options&&this._options.onFirstListenerDidAdd&&this._options.onFirstListenerDidAdd(this),this._options&&this._options.onListenerDidAdd&&this._options.onListenerDidAdd(this,e,n);const o=null===(r=this._leakageMon)||void 0===r?void 0:r.check(this._listeners.size),s=E(()=>{if(o&&o(),!this._disposed&&(a(),this._options&&this._options.onLastListenerRemove)){const e=this._listeners&&!this._listeners.isEmpty();e||this._options.onLastListenerRemove(this)}});return t instanceof T?t.add(s):Array.isArray(t)&&t.push(s),s}),this._event}fire(e){var n,t;if(this._listeners){this._deliveryQueue||(this._deliveryQueue=new L);for(let n of this._listeners)this._deliveryQueue.push([n,e]);null===(n=this._perfMon)||void 0===n||n.start(this._deliveryQueue.size);while(this._deliveryQueue.size>0){const[e,n]=this._deliveryQueue.shift();try{"function"===typeof e?e.call(void 0,n):e[0].call(e[1],n)}catch(r){a(r)}}null===(t=this._perfMon)||void 0===t||t.stop()}}dispose(){var e,n,t,r,i;this._disposed||(this._disposed=!0,null===(e=this._listeners)||void 0===e||e.clear(),null===(n=this._deliveryQueue)||void 0===n||n.clear(),null===(r=null===(t=this._options)||void 0===t?void 0:t.onLastListenerRemove)||void 0===r||r.call(t),null===(i=this._leakageMon)||void 0===i||i.dispose())}}function w(e){let n=[],t=Object.getPrototypeOf(e);while(Object.prototype!==t)n=n.concat(Object.getOwnPropertyNames(t)),t=Object.getPrototypeOf(t);return n}function P(e){const n=[];for(const t of w(e))"function"===typeof e[t]&&n.push(t);return n}function M(e,n){const t=e=>{return function(){const t=Array.prototype.slice.call(arguments,0);return n(e,t)}};let r={};for(const i of e)r[i]=t(i);return r}function F(e){return e.split(/\r\n|\r|\n/)}function G(e){for(let n=0,t=e.length;n<t;n++){const t=e.charCodeAt(n);if(32!==t&&9!==t)return n}return-1}function B(e,n=e.length-1){for(let t=n;t>=0;t--){const n=e.charCodeAt(t);if(32!==n&&9!==n)return t}return-1}function U(e){return e>=65&&e<=90}function V(e){return 55296<=e&&e<=56319}function j(e){return 56320<=e&&e<=57343}function K(e,n){return n-56320+(e-55296<<10)+65536}String.fromCharCode(65279);class H{constructor(){this._data=W()}static getInstance(){return H._INSTANCE||(H._INSTANCE=new H),H._INSTANCE}getGraphemeBreakType(e){if(e<32)return 10===e?3:13===e?2:4;if(e<127)return 0;const n=this._data,t=n.length/3;let r=1;while(r<=t)if(e<n[3*r])r*=2;else{if(!(e>n[3*r+1]))return n[3*r+2];r=2*r+1}return 0}}function W(){return JSON.parse("[0,0,0,51592,51592,11,44424,44424,11,72251,72254,5,7150,7150,7,48008,48008,11,55176,55176,11,128420,128420,14,3276,3277,5,9979,9980,14,46216,46216,11,49800,49800,11,53384,53384,11,70726,70726,5,122915,122916,5,129320,129327,14,2558,2558,5,5906,5908,5,9762,9763,14,43360,43388,8,45320,45320,11,47112,47112,11,48904,48904,11,50696,50696,11,52488,52488,11,54280,54280,11,70082,70083,1,71350,71350,7,73111,73111,5,127892,127893,14,128726,128727,14,129473,129474,14,2027,2035,5,2901,2902,5,3784,3789,5,6754,6754,5,8418,8420,5,9877,9877,14,11088,11088,14,44008,44008,5,44872,44872,11,45768,45768,11,46664,46664,11,47560,47560,11,48456,48456,11,49352,49352,11,50248,50248,11,51144,51144,11,52040,52040,11,52936,52936,11,53832,53832,11,54728,54728,11,69811,69814,5,70459,70460,5,71096,71099,7,71998,71998,5,72874,72880,5,119149,119149,7,127374,127374,14,128335,128335,14,128482,128482,14,128765,128767,14,129399,129400,14,129680,129685,14,1476,1477,5,2377,2380,7,2759,2760,5,3137,3140,7,3458,3459,7,4153,4154,5,6432,6434,5,6978,6978,5,7675,7679,5,9723,9726,14,9823,9823,14,9919,9923,14,10035,10036,14,42736,42737,5,43596,43596,5,44200,44200,11,44648,44648,11,45096,45096,11,45544,45544,11,45992,45992,11,46440,46440,11,46888,46888,11,47336,47336,11,47784,47784,11,48232,48232,11,48680,48680,11,49128,49128,11,49576,49576,11,50024,50024,11,50472,50472,11,50920,50920,11,51368,51368,11,51816,51816,11,52264,52264,11,52712,52712,11,53160,53160,11,53608,53608,11,54056,54056,11,54504,54504,11,54952,54952,11,68108,68111,5,69933,69940,5,70197,70197,7,70498,70499,7,70845,70845,5,71229,71229,5,71727,71735,5,72154,72155,5,72344,72345,5,73023,73029,5,94095,94098,5,121403,121452,5,126981,127182,14,127538,127546,14,127990,127990,14,128391,128391,14,128445,128449,14,128500,128505,14,128752,128752,14,129160,129167,14,129356,129356,14,129432,129442,14,129648,129651,14,129751,131069,14,173,173,4,1757,1757,1,2274,2274,1,2494,2494,5,2641,2641,5,2876,2876,5,3014,3016,7,3262,3262,7,3393,3396,5,3570,3571,7,3968,3972,5,4228,4228,7,6086,6086,5,6679,6680,5,6912,6915,5,7080,7081,5,7380,7392,5,8252,8252,14,9096,9096,14,9748,9749,14,9784,9786,14,9833,9850,14,9890,9894,14,9938,9938,14,9999,9999,14,10085,10087,14,12349,12349,14,43136,43137,7,43454,43456,7,43755,43755,7,44088,44088,11,44312,44312,11,44536,44536,11,44760,44760,11,44984,44984,11,45208,45208,11,45432,45432,11,45656,45656,11,45880,45880,11,46104,46104,11,46328,46328,11,46552,46552,11,46776,46776,11,47000,47000,11,47224,47224,11,47448,47448,11,47672,47672,11,47896,47896,11,48120,48120,11,48344,48344,11,48568,48568,11,48792,48792,11,49016,49016,11,49240,49240,11,49464,49464,11,49688,49688,11,49912,49912,11,50136,50136,11,50360,50360,11,50584,50584,11,50808,50808,11,51032,51032,11,51256,51256,11,51480,51480,11,51704,51704,11,51928,51928,11,52152,52152,11,52376,52376,11,52600,52600,11,52824,52824,11,53048,53048,11,53272,53272,11,53496,53496,11,53720,53720,11,53944,53944,11,54168,54168,11,54392,54392,11,54616,54616,11,54840,54840,11,55064,55064,11,65438,65439,5,69633,69633,5,69837,69837,1,70018,70018,7,70188,70190,7,70368,70370,7,70465,70468,7,70712,70719,5,70835,70840,5,70850,70851,5,71132,71133,5,71340,71340,7,71458,71461,5,71985,71989,7,72002,72002,7,72193,72202,5,72281,72283,5,72766,72766,7,72885,72886,5,73104,73105,5,92912,92916,5,113824,113827,4,119173,119179,5,121505,121519,5,125136,125142,5,127279,127279,14,127489,127490,14,127570,127743,14,127900,127901,14,128254,128254,14,128369,128370,14,128400,128400,14,128425,128432,14,128468,128475,14,128489,128494,14,128715,128720,14,128745,128745,14,128759,128760,14,129004,129023,14,129296,129304,14,129340,129342,14,129388,129392,14,129404,129407,14,129454,129455,14,129485,129487,14,129659,129663,14,129719,129727,14,917536,917631,5,13,13,2,1160,1161,5,1564,1564,4,1807,1807,1,2085,2087,5,2363,2363,7,2402,2403,5,2507,2508,7,2622,2624,7,2691,2691,7,2786,2787,5,2881,2884,5,3006,3006,5,3072,3072,5,3170,3171,5,3267,3268,7,3330,3331,7,3406,3406,1,3538,3540,5,3655,3662,5,3897,3897,5,4038,4038,5,4184,4185,5,4352,4447,8,6068,6069,5,6155,6157,5,6448,6449,7,6742,6742,5,6783,6783,5,6966,6970,5,7042,7042,7,7143,7143,7,7212,7219,5,7412,7412,5,8206,8207,4,8294,8303,4,8596,8601,14,9410,9410,14,9742,9742,14,9757,9757,14,9770,9770,14,9794,9794,14,9828,9828,14,9855,9855,14,9882,9882,14,9900,9903,14,9929,9933,14,9963,9967,14,9987,9988,14,10006,10006,14,10062,10062,14,10175,10175,14,11744,11775,5,42607,42607,5,43043,43044,7,43263,43263,5,43444,43445,7,43569,43570,5,43698,43700,5,43766,43766,5,44032,44032,11,44144,44144,11,44256,44256,11,44368,44368,11,44480,44480,11,44592,44592,11,44704,44704,11,44816,44816,11,44928,44928,11,45040,45040,11,45152,45152,11,45264,45264,11,45376,45376,11,45488,45488,11,45600,45600,11,45712,45712,11,45824,45824,11,45936,45936,11,46048,46048,11,46160,46160,11,46272,46272,11,46384,46384,11,46496,46496,11,46608,46608,11,46720,46720,11,46832,46832,11,46944,46944,11,47056,47056,11,47168,47168,11,47280,47280,11,47392,47392,11,47504,47504,11,47616,47616,11,47728,47728,11,47840,47840,11,47952,47952,11,48064,48064,11,48176,48176,11,48288,48288,11,48400,48400,11,48512,48512,11,48624,48624,11,48736,48736,11,48848,48848,11,48960,48960,11,49072,49072,11,49184,49184,11,49296,49296,11,49408,49408,11,49520,49520,11,49632,49632,11,49744,49744,11,49856,49856,11,49968,49968,11,50080,50080,11,50192,50192,11,50304,50304,11,50416,50416,11,50528,50528,11,50640,50640,11,50752,50752,11,50864,50864,11,50976,50976,11,51088,51088,11,51200,51200,11,51312,51312,11,51424,51424,11,51536,51536,11,51648,51648,11,51760,51760,11,51872,51872,11,51984,51984,11,52096,52096,11,52208,52208,11,52320,52320,11,52432,52432,11,52544,52544,11,52656,52656,11,52768,52768,11,52880,52880,11,52992,52992,11,53104,53104,11,53216,53216,11,53328,53328,11,53440,53440,11,53552,53552,11,53664,53664,11,53776,53776,11,53888,53888,11,54000,54000,11,54112,54112,11,54224,54224,11,54336,54336,11,54448,54448,11,54560,54560,11,54672,54672,11,54784,54784,11,54896,54896,11,55008,55008,11,55120,55120,11,64286,64286,5,66272,66272,5,68900,68903,5,69762,69762,7,69817,69818,5,69927,69931,5,70003,70003,5,70070,70078,5,70094,70094,7,70194,70195,7,70206,70206,5,70400,70401,5,70463,70463,7,70475,70477,7,70512,70516,5,70722,70724,5,70832,70832,5,70842,70842,5,70847,70848,5,71088,71089,7,71102,71102,7,71219,71226,5,71231,71232,5,71342,71343,7,71453,71455,5,71463,71467,5,71737,71738,5,71995,71996,5,72000,72000,7,72145,72147,7,72160,72160,5,72249,72249,7,72273,72278,5,72330,72342,5,72752,72758,5,72850,72871,5,72882,72883,5,73018,73018,5,73031,73031,5,73109,73109,5,73461,73462,7,94031,94031,5,94192,94193,7,119142,119142,7,119155,119162,4,119362,119364,5,121476,121476,5,122888,122904,5,123184,123190,5,126976,126979,14,127184,127231,14,127344,127345,14,127405,127461,14,127514,127514,14,127561,127567,14,127778,127779,14,127896,127896,14,127985,127986,14,127995,127999,5,128326,128328,14,128360,128366,14,128378,128378,14,128394,128397,14,128405,128406,14,128422,128423,14,128435,128443,14,128453,128464,14,128479,128480,14,128484,128487,14,128496,128498,14,128640,128709,14,128723,128724,14,128736,128741,14,128747,128748,14,128755,128755,14,128762,128762,14,128981,128991,14,129096,129103,14,129292,129292,14,129311,129311,14,129329,129330,14,129344,129349,14,129360,129374,14,129394,129394,14,129402,129402,14,129413,129425,14,129445,129450,14,129466,129471,14,129483,129483,14,129511,129535,14,129653,129655,14,129667,129670,14,129705,129711,14,129731,129743,14,917505,917505,4,917760,917999,5,10,10,3,127,159,4,768,879,5,1471,1471,5,1536,1541,1,1648,1648,5,1767,1768,5,1840,1866,5,2070,2073,5,2137,2139,5,2307,2307,7,2366,2368,7,2382,2383,7,2434,2435,7,2497,2500,5,2519,2519,5,2563,2563,7,2631,2632,5,2677,2677,5,2750,2752,7,2763,2764,7,2817,2817,5,2879,2879,5,2891,2892,7,2914,2915,5,3008,3008,5,3021,3021,5,3076,3076,5,3146,3149,5,3202,3203,7,3264,3265,7,3271,3272,7,3298,3299,5,3390,3390,5,3402,3404,7,3426,3427,5,3535,3535,5,3544,3550,7,3635,3635,7,3763,3763,7,3893,3893,5,3953,3966,5,3981,3991,5,4145,4145,7,4157,4158,5,4209,4212,5,4237,4237,5,4520,4607,10,5970,5971,5,6071,6077,5,6089,6099,5,6277,6278,5,6439,6440,5,6451,6456,7,6683,6683,5,6744,6750,5,6765,6770,7,6846,6846,5,6964,6964,5,6972,6972,5,7019,7027,5,7074,7077,5,7083,7085,5,7146,7148,7,7154,7155,7,7222,7223,5,7394,7400,5,7416,7417,5,8204,8204,5,8233,8233,4,8288,8292,4,8413,8416,5,8482,8482,14,8986,8987,14,9193,9203,14,9654,9654,14,9733,9733,14,9745,9745,14,9752,9752,14,9760,9760,14,9766,9766,14,9774,9775,14,9792,9792,14,9800,9811,14,9825,9826,14,9831,9831,14,9852,9853,14,9872,9873,14,9880,9880,14,9885,9887,14,9896,9897,14,9906,9916,14,9926,9927,14,9936,9936,14,9941,9960,14,9974,9974,14,9982,9985,14,9992,9997,14,10002,10002,14,10017,10017,14,10055,10055,14,10071,10071,14,10145,10145,14,11013,11015,14,11503,11505,5,12334,12335,5,12951,12951,14,42612,42621,5,43014,43014,5,43047,43047,7,43204,43205,5,43335,43345,5,43395,43395,7,43450,43451,7,43561,43566,5,43573,43574,5,43644,43644,5,43710,43711,5,43758,43759,7,44005,44005,5,44012,44012,7,44060,44060,11,44116,44116,11,44172,44172,11,44228,44228,11,44284,44284,11,44340,44340,11,44396,44396,11,44452,44452,11,44508,44508,11,44564,44564,11,44620,44620,11,44676,44676,11,44732,44732,11,44788,44788,11,44844,44844,11,44900,44900,11,44956,44956,11,45012,45012,11,45068,45068,11,45124,45124,11,45180,45180,11,45236,45236,11,45292,45292,11,45348,45348,11,45404,45404,11,45460,45460,11,45516,45516,11,45572,45572,11,45628,45628,11,45684,45684,11,45740,45740,11,45796,45796,11,45852,45852,11,45908,45908,11,45964,45964,11,46020,46020,11,46076,46076,11,46132,46132,11,46188,46188,11,46244,46244,11,46300,46300,11,46356,46356,11,46412,46412,11,46468,46468,11,46524,46524,11,46580,46580,11,46636,46636,11,46692,46692,11,46748,46748,11,46804,46804,11,46860,46860,11,46916,46916,11,46972,46972,11,47028,47028,11,47084,47084,11,47140,47140,11,47196,47196,11,47252,47252,11,47308,47308,11,47364,47364,11,47420,47420,11,47476,47476,11,47532,47532,11,47588,47588,11,47644,47644,11,47700,47700,11,47756,47756,11,47812,47812,11,47868,47868,11,47924,47924,11,47980,47980,11,48036,48036,11,48092,48092,11,48148,48148,11,48204,48204,11,48260,48260,11,48316,48316,11,48372,48372,11,48428,48428,11,48484,48484,11,48540,48540,11,48596,48596,11,48652,48652,11,48708,48708,11,48764,48764,11,48820,48820,11,48876,48876,11,48932,48932,11,48988,48988,11,49044,49044,11,49100,49100,11,49156,49156,11,49212,49212,11,49268,49268,11,49324,49324,11,49380,49380,11,49436,49436,11,49492,49492,11,49548,49548,11,49604,49604,11,49660,49660,11,49716,49716,11,49772,49772,11,49828,49828,11,49884,49884,11,49940,49940,11,49996,49996,11,50052,50052,11,50108,50108,11,50164,50164,11,50220,50220,11,50276,50276,11,50332,50332,11,50388,50388,11,50444,50444,11,50500,50500,11,50556,50556,11,50612,50612,11,50668,50668,11,50724,50724,11,50780,50780,11,50836,50836,11,50892,50892,11,50948,50948,11,51004,51004,11,51060,51060,11,51116,51116,11,51172,51172,11,51228,51228,11,51284,51284,11,51340,51340,11,51396,51396,11,51452,51452,11,51508,51508,11,51564,51564,11,51620,51620,11,51676,51676,11,51732,51732,11,51788,51788,11,51844,51844,11,51900,51900,11,51956,51956,11,52012,52012,11,52068,52068,11,52124,52124,11,52180,52180,11,52236,52236,11,52292,52292,11,52348,52348,11,52404,52404,11,52460,52460,11,52516,52516,11,52572,52572,11,52628,52628,11,52684,52684,11,52740,52740,11,52796,52796,11,52852,52852,11,52908,52908,11,52964,52964,11,53020,53020,11,53076,53076,11,53132,53132,11,53188,53188,11,53244,53244,11,53300,53300,11,53356,53356,11,53412,53412,11,53468,53468,11,53524,53524,11,53580,53580,11,53636,53636,11,53692,53692,11,53748,53748,11,53804,53804,11,53860,53860,11,53916,53916,11,53972,53972,11,54028,54028,11,54084,54084,11,54140,54140,11,54196,54196,11,54252,54252,11,54308,54308,11,54364,54364,11,54420,54420,11,54476,54476,11,54532,54532,11,54588,54588,11,54644,54644,11,54700,54700,11,54756,54756,11,54812,54812,11,54868,54868,11,54924,54924,11,54980,54980,11,55036,55036,11,55092,55092,11,55148,55148,11,55216,55238,9,65056,65071,5,65529,65531,4,68097,68099,5,68159,68159,5,69446,69456,5,69688,69702,5,69808,69810,7,69815,69816,7,69821,69821,1,69888,69890,5,69932,69932,7,69957,69958,7,70016,70017,5,70067,70069,7,70079,70080,7,70089,70092,5,70095,70095,5,70191,70193,5,70196,70196,5,70198,70199,5,70367,70367,5,70371,70378,5,70402,70403,7,70462,70462,5,70464,70464,5,70471,70472,7,70487,70487,5,70502,70508,5,70709,70711,7,70720,70721,7,70725,70725,7,70750,70750,5,70833,70834,7,70841,70841,7,70843,70844,7,70846,70846,7,70849,70849,7,71087,71087,5,71090,71093,5,71100,71101,5,71103,71104,5,71216,71218,7,71227,71228,7,71230,71230,7,71339,71339,5,71341,71341,5,71344,71349,5,71351,71351,5,71456,71457,7,71462,71462,7,71724,71726,7,71736,71736,7,71984,71984,5,71991,71992,7,71997,71997,7,71999,71999,1,72001,72001,1,72003,72003,5,72148,72151,5,72156,72159,7,72164,72164,7,72243,72248,5,72250,72250,1,72263,72263,5,72279,72280,7,72324,72329,1,72343,72343,7,72751,72751,7,72760,72765,5,72767,72767,5,72873,72873,7,72881,72881,7,72884,72884,7,73009,73014,5,73020,73021,5,73030,73030,1,73098,73102,7,73107,73108,7,73110,73110,7,73459,73460,5,78896,78904,4,92976,92982,5,94033,94087,7,94180,94180,5,113821,113822,5,119141,119141,5,119143,119145,5,119150,119154,5,119163,119170,5,119210,119213,5,121344,121398,5,121461,121461,5,121499,121503,5,122880,122886,5,122907,122913,5,122918,122922,5,123628,123631,5,125252,125258,5,126980,126980,14,127183,127183,14,127245,127247,14,127340,127343,14,127358,127359,14,127377,127386,14,127462,127487,6,127491,127503,14,127535,127535,14,127548,127551,14,127568,127569,14,127744,127777,14,127780,127891,14,127894,127895,14,127897,127899,14,127902,127984,14,127987,127989,14,127991,127994,14,128000,128253,14,128255,128317,14,128329,128334,14,128336,128359,14,128367,128368,14,128371,128377,14,128379,128390,14,128392,128393,14,128398,128399,14,128401,128404,14,128407,128419,14,128421,128421,14,128424,128424,14,128433,128434,14,128444,128444,14,128450,128452,14,128465,128467,14,128476,128478,14,128481,128481,14,128483,128483,14,128488,128488,14,128495,128495,14,128499,128499,14,128506,128591,14,128710,128714,14,128721,128722,14,128725,128725,14,128728,128735,14,128742,128744,14,128746,128746,14,128749,128751,14,128753,128754,14,128756,128758,14,128761,128761,14,128763,128764,14,128884,128895,14,128992,129003,14,129036,129039,14,129114,129119,14,129198,129279,14,129293,129295,14,129305,129310,14,129312,129319,14,129328,129328,14,129331,129338,14,129343,129343,14,129351,129355,14,129357,129359,14,129375,129387,14,129393,129393,14,129395,129398,14,129401,129401,14,129403,129403,14,129408,129412,14,129426,129431,14,129443,129444,14,129451,129453,14,129456,129465,14,129472,129472,14,129475,129482,14,129484,129484,14,129488,129510,14,129536,129647,14,129652,129652,14,129656,129658,14,129664,129666,14,129671,129679,14,129686,129704,14,129712,129718,14,129728,129730,14,129744,129750,14,917504,917504,4,917506,917535,4,917632,917759,4,918000,921599,4,0,9,4,11,12,4,14,31,4,169,169,14,174,174,14,1155,1159,5,1425,1469,5,1473,1474,5,1479,1479,5,1552,1562,5,1611,1631,5,1750,1756,5,1759,1764,5,1770,1773,5,1809,1809,5,1958,1968,5,2045,2045,5,2075,2083,5,2089,2093,5,2259,2273,5,2275,2306,5,2362,2362,5,2364,2364,5,2369,2376,5,2381,2381,5,2385,2391,5,2433,2433,5,2492,2492,5,2495,2496,7,2503,2504,7,2509,2509,5,2530,2531,5,2561,2562,5,2620,2620,5,2625,2626,5,2635,2637,5,2672,2673,5,2689,2690,5,2748,2748,5,2753,2757,5,2761,2761,7,2765,2765,5,2810,2815,5,2818,2819,7,2878,2878,5,2880,2880,7,2887,2888,7,2893,2893,5,2903,2903,5,2946,2946,5,3007,3007,7,3009,3010,7,3018,3020,7,3031,3031,5,3073,3075,7,3134,3136,5,3142,3144,5,3157,3158,5,3201,3201,5,3260,3260,5,3263,3263,5,3266,3266,5,3270,3270,5,3274,3275,7,3285,3286,5,3328,3329,5,3387,3388,5,3391,3392,7,3398,3400,7,3405,3405,5,3415,3415,5,3457,3457,5,3530,3530,5,3536,3537,7,3542,3542,5,3551,3551,5,3633,3633,5,3636,3642,5,3761,3761,5,3764,3772,5,3864,3865,5,3895,3895,5,3902,3903,7,3967,3967,7,3974,3975,5,3993,4028,5,4141,4144,5,4146,4151,5,4155,4156,7,4182,4183,7,4190,4192,5,4226,4226,5,4229,4230,5,4253,4253,5,4448,4519,9,4957,4959,5,5938,5940,5,6002,6003,5,6070,6070,7,6078,6085,7,6087,6088,7,6109,6109,5,6158,6158,4,6313,6313,5,6435,6438,7,6441,6443,7,6450,6450,5,6457,6459,5,6681,6682,7,6741,6741,7,6743,6743,7,6752,6752,5,6757,6764,5,6771,6780,5,6832,6845,5,6847,6848,5,6916,6916,7,6965,6965,5,6971,6971,7,6973,6977,7,6979,6980,7,7040,7041,5,7073,7073,7,7078,7079,7,7082,7082,7,7142,7142,5,7144,7145,5,7149,7149,5,7151,7153,5,7204,7211,7,7220,7221,7,7376,7378,5,7393,7393,7,7405,7405,5,7415,7415,7,7616,7673,5,8203,8203,4,8205,8205,13,8232,8232,4,8234,8238,4,8265,8265,14,8293,8293,4,8400,8412,5,8417,8417,5,8421,8432,5,8505,8505,14,8617,8618,14,9000,9000,14,9167,9167,14,9208,9210,14,9642,9643,14,9664,9664,14,9728,9732,14,9735,9741,14,9743,9744,14,9746,9746,14,9750,9751,14,9753,9756,14,9758,9759,14,9761,9761,14,9764,9765,14,9767,9769,14,9771,9773,14,9776,9783,14,9787,9791,14,9793,9793,14,9795,9799,14,9812,9822,14,9824,9824,14,9827,9827,14,9829,9830,14,9832,9832,14,9851,9851,14,9854,9854,14,9856,9861,14,9874,9876,14,9878,9879,14,9881,9881,14,9883,9884,14,9888,9889,14,9895,9895,14,9898,9899,14,9904,9905,14,9917,9918,14,9924,9925,14,9928,9928,14,9934,9935,14,9937,9937,14,9939,9940,14,9961,9962,14,9968,9973,14,9975,9978,14,9981,9981,14,9986,9986,14,9989,9989,14,9998,9998,14,10000,10001,14,10004,10004,14,10013,10013,14,10024,10024,14,10052,10052,14,10060,10060,14,10067,10069,14,10083,10084,14,10133,10135,14,10160,10160,14,10548,10549,14,11035,11036,14,11093,11093,14,11647,11647,5,12330,12333,5,12336,12336,14,12441,12442,5,12953,12953,14,42608,42610,5,42654,42655,5,43010,43010,5,43019,43019,5,43045,43046,5,43052,43052,5,43188,43203,7,43232,43249,5,43302,43309,5,43346,43347,7,43392,43394,5,43443,43443,5,43446,43449,5,43452,43453,5,43493,43493,5,43567,43568,7,43571,43572,7,43587,43587,5,43597,43597,7,43696,43696,5,43703,43704,5,43713,43713,5,43756,43757,5,43765,43765,7,44003,44004,7,44006,44007,7,44009,44010,7,44013,44013,5,44033,44059,12,44061,44087,12,44089,44115,12,44117,44143,12,44145,44171,12,44173,44199,12,44201,44227,12,44229,44255,12,44257,44283,12,44285,44311,12,44313,44339,12,44341,44367,12,44369,44395,12,44397,44423,12,44425,44451,12,44453,44479,12,44481,44507,12,44509,44535,12,44537,44563,12,44565,44591,12,44593,44619,12,44621,44647,12,44649,44675,12,44677,44703,12,44705,44731,12,44733,44759,12,44761,44787,12,44789,44815,12,44817,44843,12,44845,44871,12,44873,44899,12,44901,44927,12,44929,44955,12,44957,44983,12,44985,45011,12,45013,45039,12,45041,45067,12,45069,45095,12,45097,45123,12,45125,45151,12,45153,45179,12,45181,45207,12,45209,45235,12,45237,45263,12,45265,45291,12,45293,45319,12,45321,45347,12,45349,45375,12,45377,45403,12,45405,45431,12,45433,45459,12,45461,45487,12,45489,45515,12,45517,45543,12,45545,45571,12,45573,45599,12,45601,45627,12,45629,45655,12,45657,45683,12,45685,45711,12,45713,45739,12,45741,45767,12,45769,45795,12,45797,45823,12,45825,45851,12,45853,45879,12,45881,45907,12,45909,45935,12,45937,45963,12,45965,45991,12,45993,46019,12,46021,46047,12,46049,46075,12,46077,46103,12,46105,46131,12,46133,46159,12,46161,46187,12,46189,46215,12,46217,46243,12,46245,46271,12,46273,46299,12,46301,46327,12,46329,46355,12,46357,46383,12,46385,46411,12,46413,46439,12,46441,46467,12,46469,46495,12,46497,46523,12,46525,46551,12,46553,46579,12,46581,46607,12,46609,46635,12,46637,46663,12,46665,46691,12,46693,46719,12,46721,46747,12,46749,46775,12,46777,46803,12,46805,46831,12,46833,46859,12,46861,46887,12,46889,46915,12,46917,46943,12,46945,46971,12,46973,46999,12,47001,47027,12,47029,47055,12,47057,47083,12,47085,47111,12,47113,47139,12,47141,47167,12,47169,47195,12,47197,47223,12,47225,47251,12,47253,47279,12,47281,47307,12,47309,47335,12,47337,47363,12,47365,47391,12,47393,47419,12,47421,47447,12,47449,47475,12,47477,47503,12,47505,47531,12,47533,47559,12,47561,47587,12,47589,47615,12,47617,47643,12,47645,47671,12,47673,47699,12,47701,47727,12,47729,47755,12,47757,47783,12,47785,47811,12,47813,47839,12,47841,47867,12,47869,47895,12,47897,47923,12,47925,47951,12,47953,47979,12,47981,48007,12,48009,48035,12,48037,48063,12,48065,48091,12,48093,48119,12,48121,48147,12,48149,48175,12,48177,48203,12,48205,48231,12,48233,48259,12,48261,48287,12,48289,48315,12,48317,48343,12,48345,48371,12,48373,48399,12,48401,48427,12,48429,48455,12,48457,48483,12,48485,48511,12,48513,48539,12,48541,48567,12,48569,48595,12,48597,48623,12,48625,48651,12,48653,48679,12,48681,48707,12,48709,48735,12,48737,48763,12,48765,48791,12,48793,48819,12,48821,48847,12,48849,48875,12,48877,48903,12,48905,48931,12,48933,48959,12,48961,48987,12,48989,49015,12,49017,49043,12,49045,49071,12,49073,49099,12,49101,49127,12,49129,49155,12,49157,49183,12,49185,49211,12,49213,49239,12,49241,49267,12,49269,49295,12,49297,49323,12,49325,49351,12,49353,49379,12,49381,49407,12,49409,49435,12,49437,49463,12,49465,49491,12,49493,49519,12,49521,49547,12,49549,49575,12,49577,49603,12,49605,49631,12,49633,49659,12,49661,49687,12,49689,49715,12,49717,49743,12,49745,49771,12,49773,49799,12,49801,49827,12,49829,49855,12,49857,49883,12,49885,49911,12,49913,49939,12,49941,49967,12,49969,49995,12,49997,50023,12,50025,50051,12,50053,50079,12,50081,50107,12,50109,50135,12,50137,50163,12,50165,50191,12,50193,50219,12,50221,50247,12,50249,50275,12,50277,50303,12,50305,50331,12,50333,50359,12,50361,50387,12,50389,50415,12,50417,50443,12,50445,50471,12,50473,50499,12,50501,50527,12,50529,50555,12,50557,50583,12,50585,50611,12,50613,50639,12,50641,50667,12,50669,50695,12,50697,50723,12,50725,50751,12,50753,50779,12,50781,50807,12,50809,50835,12,50837,50863,12,50865,50891,12,50893,50919,12,50921,50947,12,50949,50975,12,50977,51003,12,51005,51031,12,51033,51059,12,51061,51087,12,51089,51115,12,51117,51143,12,51145,51171,12,51173,51199,12,51201,51227,12,51229,51255,12,51257,51283,12,51285,51311,12,51313,51339,12,51341,51367,12,51369,51395,12,51397,51423,12,51425,51451,12,51453,51479,12,51481,51507,12,51509,51535,12,51537,51563,12,51565,51591,12,51593,51619,12,51621,51647,12,51649,51675,12,51677,51703,12,51705,51731,12,51733,51759,12,51761,51787,12,51789,51815,12,51817,51843,12,51845,51871,12,51873,51899,12,51901,51927,12,51929,51955,12,51957,51983,12,51985,52011,12,52013,52039,12,52041,52067,12,52069,52095,12,52097,52123,12,52125,52151,12,52153,52179,12,52181,52207,12,52209,52235,12,52237,52263,12,52265,52291,12,52293,52319,12,52321,52347,12,52349,52375,12,52377,52403,12,52405,52431,12,52433,52459,12,52461,52487,12,52489,52515,12,52517,52543,12,52545,52571,12,52573,52599,12,52601,52627,12,52629,52655,12,52657,52683,12,52685,52711,12,52713,52739,12,52741,52767,12,52769,52795,12,52797,52823,12,52825,52851,12,52853,52879,12,52881,52907,12,52909,52935,12,52937,52963,12,52965,52991,12,52993,53019,12,53021,53047,12,53049,53075,12,53077,53103,12,53105,53131,12,53133,53159,12,53161,53187,12,53189,53215,12,53217,53243,12,53245,53271,12,53273,53299,12,53301,53327,12,53329,53355,12,53357,53383,12,53385,53411,12,53413,53439,12,53441,53467,12,53469,53495,12,53497,53523,12,53525,53551,12,53553,53579,12,53581,53607,12,53609,53635,12,53637,53663,12,53665,53691,12,53693,53719,12,53721,53747,12,53749,53775,12,53777,53803,12,53805,53831,12,53833,53859,12,53861,53887,12,53889,53915,12,53917,53943,12,53945,53971,12,53973,53999,12,54001,54027,12,54029,54055,12,54057,54083,12,54085,54111,12,54113,54139,12,54141,54167,12,54169,54195,12,54197,54223,12,54225,54251,12,54253,54279,12,54281,54307,12,54309,54335,12,54337,54363,12,54365,54391,12,54393,54419,12,54421,54447,12,54449,54475,12,54477,54503,12,54505,54531,12,54533,54559,12,54561,54587,12,54589,54615,12,54617,54643,12,54645,54671,12,54673,54699,12,54701,54727,12,54729,54755,12,54757,54783,12,54785,54811,12,54813,54839,12,54841,54867,12,54869,54895,12,54897,54923,12,54925,54951,12,54953,54979,12,54981,55007,12,55009,55035,12,55037,55063,12,55065,55091,12,55093,55119,12,55121,55147,12,55149,55175,12,55177,55203,12,55243,55291,10,65024,65039,5,65279,65279,4,65520,65528,4,66045,66045,5,66422,66426,5,68101,68102,5,68152,68154,5,68325,68326,5,69291,69292,5,69632,69632,7,69634,69634,7,69759,69761,5]")}H._INSTANCE=null;const z="$initialize";class J{constructor(e,n,t,r){this.vsWorker=e,this.req=n,this.method=t,this.args=r,this.type=0}}class q{constructor(e,n,t,r){this.vsWorker=e,this.seq=n,this.res=t,this.err=r,this.type=1}}class X{constructor(e,n,t,r){this.vsWorker=e,this.req=n,this.eventName=t,this.arg=r,this.type=2}}class Y{constructor(e,n,t){this.vsWorker=e,this.req=n,this.event=t,this.type=3}}class Q{constructor(e,n){this.vsWorker=e,this.req=n,this.type=4}}class Z{constructor(e){this._workerId=-1,this._handler=e,this._lastSentReq=0,this._pendingReplies=Object.create(null),this._pendingEmitters=new Map,this._pendingEvents=new Map}setWorkerId(e){this._workerId=e}sendMessage(e,n){const t=String(++this._lastSentReq);return new Promise((r,i)=>{this._pendingReplies[t]={resolve:r,reject:i},this._send(new J(this._workerId,t,e,n))})}listen(e,n){let t=null;const r=new O({onFirstListenerAdd:()=>{t=String(++this._lastSentReq),this._pendingEmitters.set(t,r),this._send(new X(this._workerId,t,e,n))},onLastListenerRemove:()=>{this._pendingEmitters.delete(t),this._send(new Q(this._workerId,t)),t=null}});return r.event}handleMessage(e){e&&e.vsWorker&&(-1!==this._workerId&&e.vsWorker!==this._workerId||this._handleMessage(e))}_handleMessage(e){switch(e.type){case 1:return this._handleReplyMessage(e);case 0:return this._handleRequestMessage(e);case 2:return this._handleSubscribeEventMessage(e);case 3:return this._handleEventMessage(e);case 4:return this._handleUnsubscribeEventMessage(e)}}_handleReplyMessage(e){if(!this._pendingReplies[e.seq])return void console.warn("Got reply to unknown seq");let n=this._pendingReplies[e.seq];if(delete this._pendingReplies[e.seq],e.err){let t=e.err;return e.err.$isError&&(t=new Error,t.name=e.err.name,t.message=e.err.message,t.stack=e.err.stack),void n.reject(t)}n.resolve(e.res)}_handleRequestMessage(e){let n=e.req,t=this._handler.handleMessage(e.method,e.args);t.then(e=>{this._send(new q(this._workerId,n,e,void 0))},e=>{e.detail instanceof Error&&(e.detail=o(e.detail)),this._send(new q(this._workerId,n,void 0,o(e)))})}_handleSubscribeEventMessage(e){const n=e.req,t=this._handler.handleEvent(e.eventName,e.arg)(e=>{this._send(new Y(this._workerId,n,e))});this._pendingEvents.set(n,t)}_handleEventMessage(e){this._pendingEmitters.has(e.req)?this._pendingEmitters.get(e.req).fire(e.event):console.warn("Got event for unknown req")}_handleUnsubscribeEventMessage(e){this._pendingEvents.has(e.req)?(this._pendingEvents.get(e.req).dispose(),this._pendingEvents.delete(e.req)):console.warn("Got unsubscribe for unknown req")}_send(e){let n=[];if(0===e.type)for(let t=0;t<e.args.length;t++)e.args[t]instanceof ArrayBuffer&&n.push(e.args[t]);else 1===e.type&&e.res instanceof ArrayBuffer&&n.push(e.res);this._handler.sendMessage(e,n)}}function $(e){return"o"===e[0]&&"n"===e[1]&&U(e.charCodeAt(2))}function ee(e){return/^onDynamic/.test(e)&&U(e.charCodeAt(9))}function ne(e,n,t){const r=e=>{return function(){const t=Array.prototype.slice.call(arguments,0);return n(e,t)}},i=e=>{return function(n){return t(e,n)}};let a={};for(const o of e)ee(o)?a[o]=i(o):$(o)?a[o]=t(o,void 0):a[o]=r(o);return a}class te{constructor(e,n){this._requestHandlerFactory=n,this._requestHandler=null,this._protocol=new Z({sendMessage:(n,t)=>{e(n,t)},handleMessage:(e,n)=>this._handleMessage(e,n),handleEvent:(e,n)=>this._handleEvent(e,n)})}onmessage(e){this._protocol.handleMessage(e)}_handleMessage(e,n){if(e===z)return this.initialize(n[0],n[1],n[2],n[3]);if(!this._requestHandler||"function"!==typeof this._requestHandler[e])return Promise.reject(new Error("Missing requestHandler or method: "+e));try{return Promise.resolve(this._requestHandler[e].apply(this._requestHandler,n))}catch(t){return Promise.reject(t)}}_handleEvent(e,n){if(!this._requestHandler)throw new Error("Missing requestHandler");if(ee(e)){const t=this._requestHandler[e].call(this._requestHandler,n);if("function"!==typeof t)throw new Error(`Missing dynamic event ${e} on request handler.`);return t}if($(e)){const n=this._requestHandler[e];if("function"!==typeof n)throw new Error(`Missing event ${e} on request handler.`);return n}throw new Error(`Malformed event name ${e}`)}initialize(e,n,t,r){this._protocol.setWorkerId(e);const i=(e,n)=>{return this._protocol.sendMessage(e,n)},a=(e,n)=>{return this._protocol.listen(e,n)},o=ne(r,i,a);return this._requestHandlerFactory?(this._requestHandler=this._requestHandlerFactory(o),Promise.resolve(P(this._requestHandler))):(n&&("undefined"!==typeof n.baseUrl&&delete n["baseUrl"],"undefined"!==typeof n.paths&&"undefined"!==typeof n.paths.vs&&delete n.paths["vs"],void 0!==typeof n.trustedTypesPolicy&&delete n["trustedTypesPolicy"],n.catchError=!0,D["a"].require.config(n)),new Promise((e,n)=>{const r=D["a"].require;r([t],t=>{this._requestHandler=t.create(o),this._requestHandler?e(P(this._requestHandler)):n(new Error("No RequestHandler!"))},n)}))}}class re{constructor(e,n,t,r){this.originalStart=e,this.originalLength=n,this.modifiedStart=t,this.modifiedLength=r}getOriginalEnd(){return this.originalStart+this.originalLength}getModifiedEnd(){return this.modifiedStart+this.modifiedLength}}function ie(e,n){return(n<<5)-n+e|0}function ae(e,n){n=ie(149417,n);for(let t=0,r=e.length;t<r;t++)n=ie(e.charCodeAt(t),n);return n}function oe(e,n,t=32){const r=t-n,i=~((1<<r)-1);return(e<<n|(i&e)>>>r)>>>0}function se(e,n=0,t=e.byteLength,r=0){for(let i=0;i<t;i++)e[n+i]=r}function ce(e,n,t="0"){while(e.length<n)e=t+e;return e}function le(e,n=32){return e instanceof ArrayBuffer?Array.from(new Uint8Array(e)).map(e=>e.toString(16).padStart(2,"0")).join(""):ce((e>>>0).toString(16),n/4)}class ue{constructor(){this._h0=1732584193,this._h1=4023233417,this._h2=2562383102,this._h3=271733878,this._h4=3285377520,this._buff=new Uint8Array(67),this._buffDV=new DataView(this._buff.buffer),this._buffLen=0,this._totalLen=0,this._leftoverHighSurrogate=0,this._finished=!1}update(e){const n=e.length;if(0===n)return;const t=this._buff;let r,i,a=this._buffLen,o=this._leftoverHighSurrogate;0!==o?(r=o,i=-1,o=0):(r=e.charCodeAt(0),i=0);while(1){let s=r;if(V(r)){if(!(i+1<n)){o=r;break}{const n=e.charCodeAt(i+1);j(n)?(i++,s=K(r,n)):s=65533}}else j(r)&&(s=65533);if(a=this._push(t,a,s),i++,!(i<n))break;r=e.charCodeAt(i)}this._buffLen=a,this._leftoverHighSurrogate=o}_push(e,n,t){return t<128?e[n++]=t:t<2048?(e[n++]=192|(1984&t)>>>6,e[n++]=128|(63&t)>>>0):t<65536?(e[n++]=224|(61440&t)>>>12,e[n++]=128|(4032&t)>>>6,e[n++]=128|(63&t)>>>0):(e[n++]=240|(1835008&t)>>>18,e[n++]=128|(258048&t)>>>12,e[n++]=128|(4032&t)>>>6,e[n++]=128|(63&t)>>>0),n>=64&&(this._step(),n-=64,this._totalLen+=64,e[0]=e[64],e[1]=e[65],e[2]=e[66]),n}digest(){return this._finished||(this._finished=!0,this._leftoverHighSurrogate&&(this._leftoverHighSurrogate=0,this._buffLen=this._push(this._buff,this._buffLen,65533)),this._totalLen+=this._buffLen,this._wrapUp()),le(this._h0)+le(this._h1)+le(this._h2)+le(this._h3)+le(this._h4)}_wrapUp(){this._buff[this._buffLen++]=128,se(this._buff,this._buffLen),this._buffLen>56&&(this._step(),se(this._buff));const e=8*this._totalLen;this._buffDV.setUint32(56,Math.floor(e/4294967296),!1),this._buffDV.setUint32(60,e%4294967296,!1),this._step()}_step(){const e=ue._bigBlock32,n=this._buffDV;for(let u=0;u<64;u+=4)e.setUint32(u,n.getUint32(u,!1),!1);for(let u=64;u<320;u+=4)e.setUint32(u,oe(e.getUint32(u-12,!1)^e.getUint32(u-32,!1)^e.getUint32(u-56,!1)^e.getUint32(u-64,!1),1),!1);let t,r,i,a=this._h0,o=this._h1,s=this._h2,c=this._h3,l=this._h4;for(let u=0;u<80;u++)u<20?(t=o&s|~o&c,r=1518500249):u<40?(t=o^s^c,r=1859775393):u<60?(t=o&s|o&c|s&c,r=2400959708):(t=o^s^c,r=3395469782),i=oe(a,5)+t+l+r+e.getUint32(4*u,!1)&4294967295,l=c,c=s,s=oe(o,30),o=a,a=i;this._h0=this._h0+a&4294967295,this._h1=this._h1+o&4294967295,this._h2=this._h2+s&4294967295,this._h3=this._h3+c&4294967295,this._h4=this._h4+l&4294967295}}ue._bigBlock32=new DataView(new ArrayBuffer(320));class de{constructor(e){this.source=e}getElements(){const e=this.source,n=new Int32Array(e.length);for(let t=0,r=e.length;t<r;t++)n[t]=e.charCodeAt(t);return n}}function pe(e,n,t){return new ge(new de(e),new de(n)).ComputeDiff(t).changes}class fe{static Assert(e,n){if(!e)throw new Error(n)}}class me{static Copy(e,n,t,r,i){for(let a=0;a<i;a++)t[r+a]=e[n+a]}static Copy2(e,n,t,r,i){for(let a=0;a<i;a++)t[r+a]=e[n+a]}}class _e{constructor(){this.m_changes=[],this.m_originalStart=1073741824,this.m_modifiedStart=1073741824,this.m_originalCount=0,this.m_modifiedCount=0}MarkNextChange(){(this.m_originalCount>0||this.m_modifiedCount>0)&&this.m_changes.push(new re(this.m_originalStart,this.m_originalCount,this.m_modifiedStart,this.m_modifiedCount)),this.m_originalCount=0,this.m_modifiedCount=0,this.m_originalStart=1073741824,this.m_modifiedStart=1073741824}AddOriginalElement(e,n){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,n),this.m_originalCount++}AddModifiedElement(e,n){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,n),this.m_modifiedCount++}getChanges(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes}getReverseChanges(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes.reverse(),this.m_changes}}class ge{constructor(e,n,t=null){this.ContinueProcessingPredicate=t,this._originalSequence=e,this._modifiedSequence=n;const[r,i,a]=ge._getElements(e),[o,s,c]=ge._getElements(n);this._hasStrings=a&&c,this._originalStringElements=r,this._originalElementsOrHash=i,this._modifiedStringElements=o,this._modifiedElementsOrHash=s,this.m_forwardHistory=[],this.m_reverseHistory=[]}static _isStringArray(e){return e.length>0&&"string"===typeof e[0]}static _getElements(e){const n=e.getElements();if(ge._isStringArray(n)){const e=new Int32Array(n.length);for(let t=0,r=n.length;t<r;t++)e[t]=ae(n[t],0);return[n,e,!0]}return n instanceof Int32Array?[[],n,!1]:[[],new Int32Array(n),!1]}ElementsAreEqual(e,n){return this._originalElementsOrHash[e]===this._modifiedElementsOrHash[n]&&(!this._hasStrings||this._originalStringElements[e]===this._modifiedStringElements[n])}ElementsAreStrictEqual(e,n){if(!this.ElementsAreEqual(e,n))return!1;const t=ge._getStrictElement(this._originalSequence,e),r=ge._getStrictElement(this._modifiedSequence,n);return t===r}static _getStrictElement(e,n){return"function"===typeof e.getStrictElement?e.getStrictElement(n):null}OriginalElementsAreEqual(e,n){return this._originalElementsOrHash[e]===this._originalElementsOrHash[n]&&(!this._hasStrings||this._originalStringElements[e]===this._originalStringElements[n])}ModifiedElementsAreEqual(e,n){return this._modifiedElementsOrHash[e]===this._modifiedElementsOrHash[n]&&(!this._hasStrings||this._modifiedStringElements[e]===this._modifiedStringElements[n])}ComputeDiff(e){return this._ComputeDiff(0,this._originalElementsOrHash.length-1,0,this._modifiedElementsOrHash.length-1,e)}_ComputeDiff(e,n,t,r,i){const a=[!1];let o=this.ComputeDiffRecursive(e,n,t,r,a);return i&&(o=this.PrettifyChanges(o)),{quitEarly:a[0],changes:o}}ComputeDiffRecursive(e,n,t,r,i){i[0]=!1;while(e<=n&&t<=r&&this.ElementsAreEqual(e,t))e++,t++;while(n>=e&&r>=t&&this.ElementsAreEqual(n,r))n--,r--;if(e>n||t>r){let i;return t<=r?(fe.Assert(e===n+1,"originalStart should only be one more than originalEnd"),i=[new re(e,0,t,r-t+1)]):e<=n?(fe.Assert(t===r+1,"modifiedStart should only be one more than modifiedEnd"),i=[new re(e,n-e+1,t,0)]):(fe.Assert(e===n+1,"originalStart should only be one more than originalEnd"),fe.Assert(t===r+1,"modifiedStart should only be one more than modifiedEnd"),i=[]),i}const a=[0],o=[0],s=this.ComputeRecursionPoint(e,n,t,r,a,o,i),c=a[0],l=o[0];if(null!==s)return s;if(!i[0]){const a=this.ComputeDiffRecursive(e,c,t,l,i);let o=[];return o=i[0]?[new re(c+1,n-(c+1)+1,l+1,r-(l+1)+1)]:this.ComputeDiffRecursive(c+1,n,l+1,r,i),this.ConcatenateChanges(a,o)}return[new re(e,n-e+1,t,r-t+1)]}WALKTRACE(e,n,t,r,i,a,o,s,c,l,u,d,p,f,m,_,g,y){let h=null,v=null,b=new _e,E=n,T=t,S=p[0]-_[0]-r,x=-1073741824,L=this.m_forwardHistory.length-1;do{const n=S+e;n===E||n<T&&c[n-1]<c[n+1]?(u=c[n+1],f=u-S-r,u<x&&b.MarkNextChange(),x=u,b.AddModifiedElement(u+1,f),S=n+1-e):(u=c[n-1]+1,f=u-S-r,u<x&&b.MarkNextChange(),x=u-1,b.AddOriginalElement(u,f+1),S=n-1-e),L>=0&&(c=this.m_forwardHistory[L],e=c[0],E=1,T=c.length-1)}while(--L>=-1);if(h=b.getReverseChanges(),y[0]){let e=p[0]+1,n=_[0]+1;if(null!==h&&h.length>0){const t=h[h.length-1];e=Math.max(e,t.getOriginalEnd()),n=Math.max(n,t.getModifiedEnd())}v=[new re(e,d-e+1,n,m-n+1)]}else{b=new _e,E=a,T=o,S=p[0]-_[0]-s,x=1073741824,L=g?this.m_reverseHistory.length-1:this.m_reverseHistory.length-2;do{const e=S+i;e===E||e<T&&l[e-1]>=l[e+1]?(u=l[e+1]-1,f=u-S-s,u>x&&b.MarkNextChange(),x=u+1,b.AddOriginalElement(u+1,f+1),S=e+1-i):(u=l[e-1],f=u-S-s,u>x&&b.MarkNextChange(),x=u,b.AddModifiedElement(u+1,f+1),S=e-1-i),L>=0&&(l=this.m_reverseHistory[L],i=l[0],E=1,T=l.length-1)}while(--L>=-1);v=b.getChanges()}return this.ConcatenateChanges(h,v)}ComputeRecursionPoint(e,n,t,r,i,a,o){let s=0,c=0,l=0,u=0,d=0,p=0;e--,t--,i[0]=0,a[0]=0,this.m_forwardHistory=[],this.m_reverseHistory=[];const f=n-e+(r-t),m=f+1,_=new Int32Array(m),g=new Int32Array(m),y=r-t,h=n-e,v=e-t,b=n-r,E=h-y,T=E%2===0;_[y]=e,g[h]=n,o[0]=!1;for(let S=1;S<=f/2+1;S++){let f=0,E=0;l=this.ClipDiagonalBound(y-S,S,y,m),u=this.ClipDiagonalBound(y+S,S,y,m);for(let e=l;e<=u;e+=2){s=e===l||e<u&&_[e-1]<_[e+1]?_[e+1]:_[e-1]+1,c=s-(e-y)-v;const t=s;while(s<n&&c<r&&this.ElementsAreEqual(s+1,c+1))s++,c++;if(_[e]=s,s+c>f+E&&(f=s,E=c),!T&&Math.abs(e-h)<=S-1&&s>=g[e])return i[0]=s,a[0]=c,t<=g[e]&&S<=1448?this.WALKTRACE(y,l,u,v,h,d,p,b,_,g,s,n,i,c,r,a,T,o):null}const x=(f-e+(E-t)-S)/2;if(null!==this.ContinueProcessingPredicate&&!this.ContinueProcessingPredicate(f,x))return o[0]=!0,i[0]=f,a[0]=E,x>0&&S<=1448?this.WALKTRACE(y,l,u,v,h,d,p,b,_,g,s,n,i,c,r,a,T,o):(e++,t++,[new re(e,n-e+1,t,r-t+1)]);d=this.ClipDiagonalBound(h-S,S,h,m),p=this.ClipDiagonalBound(h+S,S,h,m);for(let m=d;m<=p;m+=2){s=m===d||m<p&&g[m-1]>=g[m+1]?g[m+1]-1:g[m-1],c=s-(m-h)-b;const f=s;while(s>e&&c>t&&this.ElementsAreEqual(s,c))s--,c--;if(g[m]=s,T&&Math.abs(m-y)<=S&&s<=_[m])return i[0]=s,a[0]=c,f>=_[m]&&S<=1448?this.WALKTRACE(y,l,u,v,h,d,p,b,_,g,s,n,i,c,r,a,T,o):null}if(S<=1447){let e=new Int32Array(u-l+2);e[0]=y-l+1,me.Copy2(_,l,e,1,u-l+1),this.m_forwardHistory.push(e),e=new Int32Array(p-d+2),e[0]=h-d+1,me.Copy2(g,d,e,1,p-d+1),this.m_reverseHistory.push(e)}}return this.WALKTRACE(y,l,u,v,h,d,p,b,_,g,s,n,i,c,r,a,T,o)}PrettifyChanges(e){for(let n=0;n<e.length;n++){const t=e[n],r=n<e.length-1?e[n+1].originalStart:this._originalElementsOrHash.length,i=n<e.length-1?e[n+1].modifiedStart:this._modifiedElementsOrHash.length,a=t.originalLength>0,o=t.modifiedLength>0;while(t.originalStart+t.originalLength<r&&t.modifiedStart+t.modifiedLength<i&&(!a||this.OriginalElementsAreEqual(t.originalStart,t.originalStart+t.originalLength))&&(!o||this.ModifiedElementsAreEqual(t.modifiedStart,t.modifiedStart+t.modifiedLength))){const e=this.ElementsAreStrictEqual(t.originalStart,t.modifiedStart),n=this.ElementsAreStrictEqual(t.originalStart+t.originalLength,t.modifiedStart+t.modifiedLength);if(n&&!e)break;t.originalStart++,t.modifiedStart++}let s=[null];n<e.length-1&&this.ChangesOverlap(e[n],e[n+1],s)&&(e[n]=s[0],e.splice(n+1,1),n--)}for(let n=e.length-1;n>=0;n--){const t=e[n];let r=0,i=0;if(n>0){const t=e[n-1];r=t.originalStart+t.originalLength,i=t.modifiedStart+t.modifiedLength}const a=t.originalLength>0,o=t.modifiedLength>0;let s=0,c=this._boundaryScore(t.originalStart,t.originalLength,t.modifiedStart,t.modifiedLength);for(let e=1;;e++){const n=t.originalStart-e,l=t.modifiedStart-e;if(n<r||l<i)break;if(a&&!this.OriginalElementsAreEqual(n,n+t.originalLength))break;if(o&&!this.ModifiedElementsAreEqual(l,l+t.modifiedLength))break;const u=n===r&&l===i,d=(u?5:0)+this._boundaryScore(n,t.originalLength,l,t.modifiedLength);d>c&&(c=d,s=e)}t.originalStart-=s,t.modifiedStart-=s;const l=[null];n>0&&this.ChangesOverlap(e[n-1],e[n],l)&&(e[n-1]=l[0],e.splice(n,1),n++)}if(this._hasStrings)for(let n=1,t=e.length;n<t;n++){const t=e[n-1],r=e[n],i=r.originalStart-t.originalStart-t.originalLength,a=t.originalStart,o=r.originalStart+r.originalLength,s=o-a,c=t.modifiedStart,l=r.modifiedStart+r.modifiedLength,u=l-c;if(i<5&&s<20&&u<20){const e=this._findBetterContiguousSequence(a,s,c,u,i);if(e){const[n,a]=e;n===t.originalStart+t.originalLength&&a===t.modifiedStart+t.modifiedLength||(t.originalLength=n-t.originalStart,t.modifiedLength=a-t.modifiedStart,r.originalStart=n+i,r.modifiedStart=a+i,r.originalLength=o-r.originalStart,r.modifiedLength=l-r.modifiedStart)}}}return e}_findBetterContiguousSequence(e,n,t,r,i){if(n<i||r<i)return null;const a=e+n-i+1,o=t+r-i+1;let s=0,c=0,l=0;for(let u=e;u<a;u++)for(let e=t;e<o;e++){const n=this._contiguousSequenceScore(u,e,i);n>0&&n>s&&(s=n,c=u,l=e)}return s>0?[c,l]:null}_contiguousSequenceScore(e,n,t){let r=0;for(let i=0;i<t;i++){if(!this.ElementsAreEqual(e+i,n+i))return 0;r+=this._originalStringElements[e+i].length}return r}_OriginalIsBoundary(e){return e<=0||e>=this._originalElementsOrHash.length-1||this._hasStrings&&/^\s*$/.test(this._originalStringElements[e])}_OriginalRegionIsBoundary(e,n){if(this._OriginalIsBoundary(e)||this._OriginalIsBoundary(e-1))return!0;if(n>0){const t=e+n;if(this._OriginalIsBoundary(t-1)||this._OriginalIsBoundary(t))return!0}return!1}_ModifiedIsBoundary(e){return e<=0||e>=this._modifiedElementsOrHash.length-1||this._hasStrings&&/^\s*$/.test(this._modifiedStringElements[e])}_ModifiedRegionIsBoundary(e,n){if(this._ModifiedIsBoundary(e)||this._ModifiedIsBoundary(e-1))return!0;if(n>0){const t=e+n;if(this._ModifiedIsBoundary(t-1)||this._ModifiedIsBoundary(t))return!0}return!1}_boundaryScore(e,n,t,r){const i=this._OriginalRegionIsBoundary(e,n)?1:0,a=this._ModifiedRegionIsBoundary(t,r)?1:0;return i+a}ConcatenateChanges(e,n){let t=[];if(0===e.length||0===n.length)return n.length>0?n:e;if(this.ChangesOverlap(e[e.length-1],n[0],t)){const r=new Array(e.length+n.length-1);return me.Copy(e,0,r,0,e.length-1),r[e.length-1]=t[0],me.Copy(n,1,r,e.length,n.length-1),r}{const t=new Array(e.length+n.length);return me.Copy(e,0,t,0,e.length),me.Copy(n,0,t,e.length,n.length),t}}ChangesOverlap(e,n,t){if(fe.Assert(e.originalStart<=n.originalStart,"Left change is not less than or equal to right change"),fe.Assert(e.modifiedStart<=n.modifiedStart,"Left change is not less than or equal to right change"),e.originalStart+e.originalLength>=n.originalStart||e.modifiedStart+e.modifiedLength>=n.modifiedStart){const r=e.originalStart;let i=e.originalLength;const a=e.modifiedStart;let o=e.modifiedLength;return e.originalStart+e.originalLength>=n.originalStart&&(i=n.originalStart+n.originalLength-e.originalStart),e.modifiedStart+e.modifiedLength>=n.modifiedStart&&(o=n.modifiedStart+n.modifiedLength-e.modifiedStart),t[0]=new re(r,i,a,o),!0}return t[0]=null,!1}ClipDiagonalBound(e,n,t,r){if(e>=0&&e<r)return e;const i=t,a=r-t-1,o=n%2===0;if(e<0){const e=i%2===0;return o===e?0:1}{const e=a%2===0;return o===e?r-1:r-2}}}var ye=t("c317");const he=65,ve=97,be=90,Ee=122,Te=46,Se=47,xe=92,Le=58,De=63;class Ae extends Error{constructor(e,n,t){let r;"string"===typeof n&&0===n.indexOf("not ")?(r="must not be",n=n.replace(/^not /,"")):r="must be";const i=-1!==e.indexOf(".")?"property":"argument";let a=`The "${e}" ${i} ${r} of type ${n}`;a+=`. Received type ${typeof t}`,super(a),this.code="ERR_INVALID_ARG_TYPE"}}function Ce(e,n){if("string"!==typeof e)throw new Ae(n,"string",e)}function Ie(e){return e===Se||e===xe}function Ne(e){return e===Se}function ke(e){return e>=he&&e<=be||e>=ve&&e<=Ee}function Re(e,n,t,r){let i="",a=0,o=-1,s=0,c=0;for(let l=0;l<=e.length;++l){if(l<e.length)c=e.charCodeAt(l);else{if(r(c))break;c=Se}if(r(c)){if(o===l-1||1===s);else if(2===s){if(i.length<2||2!==a||i.charCodeAt(i.length-1)!==Te||i.charCodeAt(i.length-2)!==Te){if(i.length>2){const e=i.lastIndexOf(t);-1===e?(i="",a=0):(i=i.slice(0,e),a=i.length-1-i.lastIndexOf(t)),o=l,s=0;continue}if(0!==i.length){i="",a=0,o=l,s=0;continue}}n&&(i+=i.length>0?`${t}..`:"..",a=2)}else i.length>0?i+=`${t}${e.slice(o+1,l)}`:i=e.slice(o+1,l),a=l-o-1;o=l,s=0}else c===Te&&-1!==s?++s:s=-1}return i}function Oe(e,n){if(null===n||"object"!==typeof n)throw new Ae("pathObject","Object",n);const t=n.dir||n.root,r=n.base||`${n.name||""}${n.ext||""}`;return t?t===n.root?`${t}${r}`:`${t}${e}${r}`:r}const we={resolve(...e){let n="",t="",r=!1;for(let i=e.length-1;i>=-1;i--){let a;if(i>=0){if(a=e[i],Ce(a,"path"),0===a.length)continue}else 0===n.length?a=ye["a"]():(a=ye["b"][`=${n}`]||ye["a"](),(void 0===a||a.slice(0,2).toLowerCase()!==n.toLowerCase()&&a.charCodeAt(2)===xe)&&(a=`${n}\\`));const o=a.length;let s=0,c="",l=!1;const u=a.charCodeAt(0);if(1===o)Ie(u)&&(s=1,l=!0);else if(Ie(u))if(l=!0,Ie(a.charCodeAt(1))){let e=2,n=e;while(e<o&&!Ie(a.charCodeAt(e)))e++;if(e<o&&e!==n){const t=a.slice(n,e);n=e;while(e<o&&Ie(a.charCodeAt(e)))e++;if(e<o&&e!==n){n=e;while(e<o&&!Ie(a.charCodeAt(e)))e++;e!==o&&e===n||(c=`\\\\${t}\\${a.slice(n,e)}`,s=e)}}}else s=1;else ke(u)&&a.charCodeAt(1)===Le&&(c=a.slice(0,2),s=2,o>2&&Ie(a.charCodeAt(2))&&(l=!0,s=3));if(c.length>0)if(n.length>0){if(c.toLowerCase()!==n.toLowerCase())continue}else n=c;if(r){if(n.length>0)break}else if(t=`${a.slice(s)}\\${t}`,r=l,l&&n.length>0)break}return t=Re(t,!r,"\\",Ie),r?`${n}\\${t}`:`${n}${t}`||"."},normalize(e){Ce(e,"path");const n=e.length;if(0===n)return".";let t,r=0,i=!1;const a=e.charCodeAt(0);if(1===n)return Ne(a)?"\\":e;if(Ie(a))if(i=!0,Ie(e.charCodeAt(1))){let i=2,a=i;while(i<n&&!Ie(e.charCodeAt(i)))i++;if(i<n&&i!==a){const o=e.slice(a,i);a=i;while(i<n&&Ie(e.charCodeAt(i)))i++;if(i<n&&i!==a){a=i;while(i<n&&!Ie(e.charCodeAt(i)))i++;if(i===n)return`\\\\${o}\\${e.slice(a)}\\`;i!==a&&(t=`\\\\${o}\\${e.slice(a,i)}`,r=i)}}}else r=1;else ke(a)&&e.charCodeAt(1)===Le&&(t=e.slice(0,2),r=2,n>2&&Ie(e.charCodeAt(2))&&(i=!0,r=3));let o=r<n?Re(e.slice(r),!i,"\\",Ie):"";return 0!==o.length||i||(o="."),o.length>0&&Ie(e.charCodeAt(n-1))&&(o+="\\"),void 0===t?i?`\\${o}`:o:i?`${t}\\${o}`:`${t}${o}`},isAbsolute(e){Ce(e,"path");const n=e.length;if(0===n)return!1;const t=e.charCodeAt(0);return Ie(t)||n>2&&ke(t)&&e.charCodeAt(1)===Le&&Ie(e.charCodeAt(2))},join(...e){if(0===e.length)return".";let n,t;for(let a=0;a<e.length;++a){const r=e[a];Ce(r,"path"),r.length>0&&(void 0===n?n=t=r:n+=`\\${r}`)}if(void 0===n)return".";let r=!0,i=0;if("string"===typeof t&&Ie(t.charCodeAt(0))){++i;const e=t.length;e>1&&Ie(t.charCodeAt(1))&&(++i,e>2&&(Ie(t.charCodeAt(2))?++i:r=!1))}if(r){while(i<n.length&&Ie(n.charCodeAt(i)))i++;i>=2&&(n=`\\${n.slice(i)}`)}return we.normalize(n)},relative(e,n){if(Ce(e,"from"),Ce(n,"to"),e===n)return"";const t=we.resolve(e),r=we.resolve(n);if(t===r)return"";if(e=t.toLowerCase(),n=r.toLowerCase(),e===n)return"";let i=0;while(i<e.length&&e.charCodeAt(i)===xe)i++;let a=e.length;while(a-1>i&&e.charCodeAt(a-1)===xe)a--;const o=a-i;let s=0;while(s<n.length&&n.charCodeAt(s)===xe)s++;let c=n.length;while(c-1>s&&n.charCodeAt(c-1)===xe)c--;const l=c-s,u=o<l?o:l;let d=-1,p=0;for(;p<u;p++){const t=e.charCodeAt(i+p);if(t!==n.charCodeAt(s+p))break;t===xe&&(d=p)}if(p!==u){if(-1===d)return r}else{if(l>u){if(n.charCodeAt(s+p)===xe)return r.slice(s+p+1);if(2===p)return r.slice(s+p)}o>u&&(e.charCodeAt(i+p)===xe?d=p:2===p&&(d=3)),-1===d&&(d=0)}let f="";for(p=i+d+1;p<=a;++p)p!==a&&e.charCodeAt(p)!==xe||(f+=0===f.length?"..":"\\..");return s+=d,f.length>0?`${f}${r.slice(s,c)}`:(r.charCodeAt(s)===xe&&++s,r.slice(s,c))},toNamespacedPath(e){if("string"!==typeof e)return e;if(0===e.length)return"";const n=we.resolve(e);if(n.length<=2)return e;if(n.charCodeAt(0)===xe){if(n.charCodeAt(1)===xe){const e=n.charCodeAt(2);if(e!==De&&e!==Te)return`\\\\?\\UNC\\${n.slice(2)}`}}else if(ke(n.charCodeAt(0))&&n.charCodeAt(1)===Le&&n.charCodeAt(2)===xe)return`\\\\?\\${n}`;return e},dirname(e){Ce(e,"path");const n=e.length;if(0===n)return".";let t=-1,r=0;const i=e.charCodeAt(0);if(1===n)return Ie(i)?e:".";if(Ie(i)){if(t=r=1,Ie(e.charCodeAt(1))){let i=2,a=i;while(i<n&&!Ie(e.charCodeAt(i)))i++;if(i<n&&i!==a){a=i;while(i<n&&Ie(e.charCodeAt(i)))i++;if(i<n&&i!==a){a=i;while(i<n&&!Ie(e.charCodeAt(i)))i++;if(i===n)return e;i!==a&&(t=r=i+1)}}}}else ke(i)&&e.charCodeAt(1)===Le&&(t=n>2&&Ie(e.charCodeAt(2))?3:2,r=t);let a=-1,o=!0;for(let s=n-1;s>=r;--s)if(Ie(e.charCodeAt(s))){if(!o){a=s;break}}else o=!1;if(-1===a){if(-1===t)return".";a=t}return e.slice(0,a)},basename(e,n){void 0!==n&&Ce(n,"ext"),Ce(e,"path");let t,r=0,i=-1,a=!0;if(e.length>=2&&ke(e.charCodeAt(0))&&e.charCodeAt(1)===Le&&(r=2),void 0!==n&&n.length>0&&n.length<=e.length){if(n===e)return"";let o=n.length-1,s=-1;for(t=e.length-1;t>=r;--t){const c=e.charCodeAt(t);if(Ie(c)){if(!a){r=t+1;break}}else-1===s&&(a=!1,s=t+1),o>=0&&(c===n.charCodeAt(o)?-1===--o&&(i=t):(o=-1,i=s))}return r===i?i=s:-1===i&&(i=e.length),e.slice(r,i)}for(t=e.length-1;t>=r;--t)if(Ie(e.charCodeAt(t))){if(!a){r=t+1;break}}else-1===i&&(a=!1,i=t+1);return-1===i?"":e.slice(r,i)},extname(e){Ce(e,"path");let n=0,t=-1,r=0,i=-1,a=!0,o=0;e.length>=2&&e.charCodeAt(1)===Le&&ke(e.charCodeAt(0))&&(n=r=2);for(let s=e.length-1;s>=n;--s){const n=e.charCodeAt(s);if(Ie(n)){if(!a){r=s+1;break}}else-1===i&&(a=!1,i=s+1),n===Te?-1===t?t=s:1!==o&&(o=1):-1!==t&&(o=-1)}return-1===t||-1===i||0===o||1===o&&t===i-1&&t===r+1?"":e.slice(t,i)},format:Oe.bind(null,"\\"),parse(e){Ce(e,"path");const n={root:"",dir:"",base:"",ext:"",name:""};if(0===e.length)return n;const t=e.length;let r=0,i=e.charCodeAt(0);if(1===t)return Ie(i)?(n.root=n.dir=e,n):(n.base=n.name=e,n);if(Ie(i)){if(r=1,Ie(e.charCodeAt(1))){let n=2,i=n;while(n<t&&!Ie(e.charCodeAt(n)))n++;if(n<t&&n!==i){i=n;while(n<t&&Ie(e.charCodeAt(n)))n++;if(n<t&&n!==i){i=n;while(n<t&&!Ie(e.charCodeAt(n)))n++;n===t?r=n:n!==i&&(r=n+1)}}}}else if(ke(i)&&e.charCodeAt(1)===Le){if(t<=2)return n.root=n.dir=e,n;if(r=2,Ie(e.charCodeAt(2))){if(3===t)return n.root=n.dir=e,n;r=3}}r>0&&(n.root=e.slice(0,r));let a=-1,o=r,s=-1,c=!0,l=e.length-1,u=0;for(;l>=r;--l)if(i=e.charCodeAt(l),Ie(i)){if(!c){o=l+1;break}}else-1===s&&(c=!1,s=l+1),i===Te?-1===a?a=l:1!==u&&(u=1):-1!==a&&(u=-1);return-1!==s&&(-1===a||0===u||1===u&&a===s-1&&a===o+1?n.base=n.name=e.slice(o,s):(n.name=e.slice(o,a),n.base=e.slice(o,s),n.ext=e.slice(a,s))),n.dir=o>0&&o!==r?e.slice(0,o-1):n.root,n},sep:"\\",delimiter:";",win32:null,posix:null},Pe={resolve(...e){let n="",t=!1;for(let r=e.length-1;r>=-1&&!t;r--){const i=r>=0?e[r]:ye["a"]();Ce(i,"path"),0!==i.length&&(n=`${i}/${n}`,t=i.charCodeAt(0)===Se)}return n=Re(n,!t,"/",Ne),t?`/${n}`:n.length>0?n:"."},normalize(e){if(Ce(e,"path"),0===e.length)return".";const n=e.charCodeAt(0)===Se,t=e.charCodeAt(e.length-1)===Se;return e=Re(e,!n,"/",Ne),0===e.length?n?"/":t?"./":".":(t&&(e+="/"),n?`/${e}`:e)},isAbsolute(e){return Ce(e,"path"),e.length>0&&e.charCodeAt(0)===Se},join(...e){if(0===e.length)return".";let n;for(let t=0;t<e.length;++t){const r=e[t];Ce(r,"path"),r.length>0&&(void 0===n?n=r:n+=`/${r}`)}return void 0===n?".":Pe.normalize(n)},relative(e,n){if(Ce(e,"from"),Ce(n,"to"),e===n)return"";if(e=Pe.resolve(e),n=Pe.resolve(n),e===n)return"";const t=1,r=e.length,i=r-t,a=1,o=n.length-a,s=i<o?i:o;let c=-1,l=0;for(;l<s;l++){const r=e.charCodeAt(t+l);if(r!==n.charCodeAt(a+l))break;r===Se&&(c=l)}if(l===s)if(o>s){if(n.charCodeAt(a+l)===Se)return n.slice(a+l+1);if(0===l)return n.slice(a+l)}else i>s&&(e.charCodeAt(t+l)===Se?c=l:0===l&&(c=0));let u="";for(l=t+c+1;l<=r;++l)l!==r&&e.charCodeAt(l)!==Se||(u+=0===u.length?"..":"/..");return`${u}${n.slice(a+c)}`},toNamespacedPath(e){return e},dirname(e){if(Ce(e,"path"),0===e.length)return".";const n=e.charCodeAt(0)===Se;let t=-1,r=!0;for(let i=e.length-1;i>=1;--i)if(e.charCodeAt(i)===Se){if(!r){t=i;break}}else r=!1;return-1===t?n?"/":".":n&&1===t?"//":e.slice(0,t)},basename(e,n){void 0!==n&&Ce(n,"ext"),Ce(e,"path");let t,r=0,i=-1,a=!0;if(void 0!==n&&n.length>0&&n.length<=e.length){if(n===e)return"";let o=n.length-1,s=-1;for(t=e.length-1;t>=0;--t){const c=e.charCodeAt(t);if(c===Se){if(!a){r=t+1;break}}else-1===s&&(a=!1,s=t+1),o>=0&&(c===n.charCodeAt(o)?-1===--o&&(i=t):(o=-1,i=s))}return r===i?i=s:-1===i&&(i=e.length),e.slice(r,i)}for(t=e.length-1;t>=0;--t)if(e.charCodeAt(t)===Se){if(!a){r=t+1;break}}else-1===i&&(a=!1,i=t+1);return-1===i?"":e.slice(r,i)},extname(e){Ce(e,"path");let n=-1,t=0,r=-1,i=!0,a=0;for(let o=e.length-1;o>=0;--o){const s=e.charCodeAt(o);if(s!==Se)-1===r&&(i=!1,r=o+1),s===Te?-1===n?n=o:1!==a&&(a=1):-1!==n&&(a=-1);else if(!i){t=o+1;break}}return-1===n||-1===r||0===a||1===a&&n===r-1&&n===t+1?"":e.slice(n,r)},format:Oe.bind(null,"/"),parse(e){Ce(e,"path");const n={root:"",dir:"",base:"",ext:"",name:""};if(0===e.length)return n;const t=e.charCodeAt(0)===Se;let r;t?(n.root="/",r=1):r=0;let i=-1,a=0,o=-1,s=!0,c=e.length-1,l=0;for(;c>=r;--c){const n=e.charCodeAt(c);if(n!==Se)-1===o&&(s=!1,o=c+1),n===Te?-1===i?i=c:1!==l&&(l=1):-1!==i&&(l=-1);else if(!s){a=c+1;break}}if(-1!==o){const r=0===a&&t?1:a;-1===i||0===l||1===l&&i===o-1&&i===a+1?n.base=n.name=e.slice(r,o):(n.name=e.slice(r,i),n.base=e.slice(r,o),n.ext=e.slice(i,o))}return a>0?n.dir=e.slice(0,a-1):t&&(n.dir="/"),n},sep:"/",delimiter:":",win32:null,posix:null};Pe.win32=we.win32=we,Pe.posix=we.posix=Pe;"win32"===ye["c"]?we.normalize:Pe.normalize,"win32"===ye["c"]?we.resolve:Pe.resolve,"win32"===ye["c"]?we.relative:Pe.relative,"win32"===ye["c"]?we.dirname:Pe.dirname,"win32"===ye["c"]?we.basename:Pe.basename,"win32"===ye["c"]?we.extname:Pe.extname,"win32"===ye["c"]?we.sep:Pe.sep;const Me=/^\w[\w\d+.-]*$/,Fe=/^\//,Ge=/^\/\//;function Be(e,n){if(!e.scheme&&n)throw new Error(`[UriError]: Scheme is missing: {scheme: "", authority: "${e.authority}", path: "${e.path}", query: "${e.query}", fragment: "${e.fragment}"}`);if(e.scheme&&!Me.test(e.scheme))throw new Error("[UriError]: Scheme contains illegal characters.");if(e.path)if(e.authority){if(!Fe.test(e.path))throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash ("/") character')}else if(Ge.test(e.path))throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters ("//")')}function Ue(e,n){return e||n?e:"file"}function Ve(e,n){switch(e){case"https":case"http":case"file":n?n[0]!==Ke&&(n=Ke+n):n=Ke;break}return n}const je="",Ke="/",He=/^(([^:/?#]+?):)?(\/\/([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?/;class We{constructor(e,n,t,r,i,a=!1){"object"===typeof e?(this.scheme=e.scheme||je,this.authority=e.authority||je,this.path=e.path||je,this.query=e.query||je,this.fragment=e.fragment||je):(this.scheme=Ue(e,a),this.authority=n||je,this.path=Ve(this.scheme,t||je),this.query=r||je,this.fragment=i||je,Be(this,a))}static isUri(e){return e instanceof We||!!e&&("string"===typeof e.authority&&"string"===typeof e.fragment&&"string"===typeof e.path&&"string"===typeof e.query&&"string"===typeof e.scheme&&"string"===typeof e.fsPath&&"function"===typeof e.with&&"function"===typeof e.toString)}get fsPath(){return Qe(this,!1)}with(e){if(!e)return this;let{scheme:n,authority:t,path:r,query:i,fragment:a}=e;return void 0===n?n=this.scheme:null===n&&(n=je),void 0===t?t=this.authority:null===t&&(t=je),void 0===r?r=this.path:null===r&&(r=je),void 0===i?i=this.query:null===i&&(i=je),void 0===a?a=this.fragment:null===a&&(a=je),n===this.scheme&&t===this.authority&&r===this.path&&i===this.query&&a===this.fragment?this:new Je(n,t,r,i,a)}static parse(e,n=!1){const t=He.exec(e);return t?new Je(t[2]||je,nn(t[4]||je),nn(t[5]||je),nn(t[7]||je),nn(t[9]||je),n):new Je(je,je,je,je,je)}static file(e){let n=je;if(D["d"]&&(e=e.replace(/\\/g,Ke)),e[0]===Ke&&e[1]===Ke){const t=e.indexOf(Ke,2);-1===t?(n=e.substring(2),e=Ke):(n=e.substring(2,t),e=e.substring(t)||Ke)}return new Je("file",n,e,je,je)}static from(e){const n=new Je(e.scheme,e.authority,e.path,e.query,e.fragment);return Be(n,!0),n}static joinPath(e,...n){if(!e.path)throw new Error("[UriError]: cannot call joinPath on URI without path");let t;return t=D["d"]&&"file"===e.scheme?We.file(we.join(Qe(e,!0),...n)).path:Pe.join(e.path,...n),e.with({path:t})}toString(e=!1){return Ze(this,e)}toJSON(){return this}static revive(e){if(e){if(e instanceof We)return e;{const n=new Je(e);return n._formatted=e.external,n._fsPath=e._sep===ze?e.fsPath:null,n}}return e}}const ze=D["d"]?1:void 0;class Je extends We{constructor(){super(...arguments),this._formatted=null,this._fsPath=null}get fsPath(){return this._fsPath||(this._fsPath=Qe(this,!1)),this._fsPath}toString(e=!1){return e?Ze(this,!0):(this._formatted||(this._formatted=Ze(this,!1)),this._formatted)}toJSON(){const e={$mid:1};return this._fsPath&&(e.fsPath=this._fsPath,e._sep=ze),this._formatted&&(e.external=this._formatted),this.path&&(e.path=this.path),this.scheme&&(e.scheme=this.scheme),this.authority&&(e.authority=this.authority),this.query&&(e.query=this.query),this.fragment&&(e.fragment=this.fragment),e}}const qe={[58]:"%3A",[47]:"%2F",[63]:"%3F",[35]:"%23",[91]:"%5B",[93]:"%5D",[64]:"%40",[33]:"%21",[36]:"%24",[38]:"%26",[39]:"%27",[40]:"%28",[41]:"%29",[42]:"%2A",[43]:"%2B",[44]:"%2C",[59]:"%3B",[61]:"%3D",[32]:"%20"};function Xe(e,n){let t=void 0,r=-1;for(let i=0;i<e.length;i++){const a=e.charCodeAt(i);if(a>=97&&a<=122||a>=65&&a<=90||a>=48&&a<=57||45===a||46===a||95===a||126===a||n&&47===a)-1!==r&&(t+=encodeURIComponent(e.substring(r,i)),r=-1),void 0!==t&&(t+=e.charAt(i));else{void 0===t&&(t=e.substr(0,i));const n=qe[a];void 0!==n?(-1!==r&&(t+=encodeURIComponent(e.substring(r,i)),r=-1),t+=n):-1===r&&(r=i)}}return-1!==r&&(t+=encodeURIComponent(e.substring(r))),void 0!==t?t:e}function Ye(e){let n=void 0;for(let t=0;t<e.length;t++){const r=e.charCodeAt(t);35===r||63===r?(void 0===n&&(n=e.substr(0,t)),n+=qe[r]):void 0!==n&&(n+=e[t])}return void 0!==n?n:e}function Qe(e,n){let t;return t=e.authority&&e.path.length>1&&"file"===e.scheme?`//${e.authority}${e.path}`:47===e.path.charCodeAt(0)&&(e.path.charCodeAt(1)>=65&&e.path.charCodeAt(1)<=90||e.path.charCodeAt(1)>=97&&e.path.charCodeAt(1)<=122)&&58===e.path.charCodeAt(2)?n?e.path.substr(1):e.path[1].toLowerCase()+e.path.substr(2):e.path,D["d"]&&(t=t.replace(/\//g,"\\")),t}function Ze(e,n){const t=n?Ye:Xe;let r="",{scheme:i,authority:a,path:o,query:s,fragment:c}=e;if(i&&(r+=i,r+=":"),(a||"file"===i)&&(r+=Ke,r+=Ke),a){let e=a.indexOf("@");if(-1!==e){const n=a.substr(0,e);a=a.substr(e+1),e=n.indexOf(":"),-1===e?r+=t(n,!1):(r+=t(n.substr(0,e),!1),r+=":",r+=t(n.substr(e+1),!1)),r+="@"}a=a.toLowerCase(),e=a.indexOf(":"),-1===e?r+=t(a,!1):(r+=t(a.substr(0,e),!1),r+=a.substr(e))}if(o){if(o.length>=3&&47===o.charCodeAt(0)&&58===o.charCodeAt(2)){const e=o.charCodeAt(1);e>=65&&e<=90&&(o=`/${String.fromCharCode(e+32)}:${o.substr(3)}`)}else if(o.length>=2&&58===o.charCodeAt(1)){const e=o.charCodeAt(0);e>=65&&e<=90&&(o=`${String.fromCharCode(e+32)}:${o.substr(2)}`)}r+=t(o,!0)}return s&&(r+="?",r+=t(s,!1)),c&&(r+="#",r+=n?c:Xe(c,!1)),r}function $e(e){try{return decodeURIComponent(e)}catch(n){return e.length>3?e.substr(0,3)+$e(e.substr(3)):e}}const en=/(%[0-9A-Za-z][0-9A-Za-z])+/g;function nn(e){return e.match(en)?e.replace(en,e=>$e(e)):e}class tn{constructor(e,n){this.lineNumber=e,this.column=n}with(e=this.lineNumber,n=this.column){return e===this.lineNumber&&n===this.column?this:new tn(e,n)}delta(e=0,n=0){return this.with(this.lineNumber+e,this.column+n)}equals(e){return tn.equals(this,e)}static equals(e,n){return!e&&!n||!!e&&!!n&&e.lineNumber===n.lineNumber&&e.column===n.column}isBefore(e){return tn.isBefore(this,e)}static isBefore(e,n){return e.lineNumber<n.lineNumber||!(n.lineNumber<e.lineNumber)&&e.column<n.column}isBeforeOrEqual(e){return tn.isBeforeOrEqual(this,e)}static isBeforeOrEqual(e,n){return e.lineNumber<n.lineNumber||!(n.lineNumber<e.lineNumber)&&e.column<=n.column}static compare(e,n){let t=0|e.lineNumber,r=0|n.lineNumber;if(t===r){let t=0|e.column,r=0|n.column;return t-r}return t-r}clone(){return new tn(this.lineNumber,this.column)}toString(){return"("+this.lineNumber+","+this.column+")"}static lift(e){return new tn(e.lineNumber,e.column)}static isIPosition(e){return e&&"number"===typeof e.lineNumber&&"number"===typeof e.column}}class rn{constructor(e,n,t,r){e>t||e===t&&n>r?(this.startLineNumber=t,this.startColumn=r,this.endLineNumber=e,this.endColumn=n):(this.startLineNumber=e,this.startColumn=n,this.endLineNumber=t,this.endColumn=r)}isEmpty(){return rn.isEmpty(this)}static isEmpty(e){return e.startLineNumber===e.endLineNumber&&e.startColumn===e.endColumn}containsPosition(e){return rn.containsPosition(this,e)}static containsPosition(e,n){return!(n.lineNumber<e.startLineNumber||n.lineNumber>e.endLineNumber)&&(!(n.lineNumber===e.startLineNumber&&n.column<e.startColumn)&&!(n.lineNumber===e.endLineNumber&&n.column>e.endColumn))}containsRange(e){return rn.containsRange(this,e)}static containsRange(e,n){return!(n.startLineNumber<e.startLineNumber||n.endLineNumber<e.startLineNumber)&&(!(n.startLineNumber>e.endLineNumber||n.endLineNumber>e.endLineNumber)&&(!(n.startLineNumber===e.startLineNumber&&n.startColumn<e.startColumn)&&!(n.endLineNumber===e.endLineNumber&&n.endColumn>e.endColumn)))}strictContainsRange(e){return rn.strictContainsRange(this,e)}static strictContainsRange(e,n){return!(n.startLineNumber<e.startLineNumber||n.endLineNumber<e.startLineNumber)&&(!(n.startLineNumber>e.endLineNumber||n.endLineNumber>e.endLineNumber)&&(!(n.startLineNumber===e.startLineNumber&&n.startColumn<=e.startColumn)&&!(n.endLineNumber===e.endLineNumber&&n.endColumn>=e.endColumn)))}plusRange(e){return rn.plusRange(this,e)}static plusRange(e,n){let t,r,i,a;return n.startLineNumber<e.startLineNumber?(t=n.startLineNumber,r=n.startColumn):n.startLineNumber===e.startLineNumber?(t=n.startLineNumber,r=Math.min(n.startColumn,e.startColumn)):(t=e.startLineNumber,r=e.startColumn),n.endLineNumber>e.endLineNumber?(i=n.endLineNumber,a=n.endColumn):n.endLineNumber===e.endLineNumber?(i=n.endLineNumber,a=Math.max(n.endColumn,e.endColumn)):(i=e.endLineNumber,a=e.endColumn),new rn(t,r,i,a)}intersectRanges(e){return rn.intersectRanges(this,e)}static intersectRanges(e,n){let t=e.startLineNumber,r=e.startColumn,i=e.endLineNumber,a=e.endColumn,o=n.startLineNumber,s=n.startColumn,c=n.endLineNumber,l=n.endColumn;return t<o?(t=o,r=s):t===o&&(r=Math.max(r,s)),i>c?(i=c,a=l):i===c&&(a=Math.min(a,l)),t>i?null:t===i&&r>a?null:new rn(t,r,i,a)}equalsRange(e){return rn.equalsRange(this,e)}static equalsRange(e,n){return!!e&&!!n&&e.startLineNumber===n.startLineNumber&&e.startColumn===n.startColumn&&e.endLineNumber===n.endLineNumber&&e.endColumn===n.endColumn}getEndPosition(){return rn.getEndPosition(this)}static getEndPosition(e){return new tn(e.endLineNumber,e.endColumn)}getStartPosition(){return rn.getStartPosition(this)}static getStartPosition(e){return new tn(e.startLineNumber,e.startColumn)}toString(){return"["+this.startLineNumber+","+this.startColumn+" -> "+this.endLineNumber+","+this.endColumn+"]"}setEndPosition(e,n){return new rn(this.startLineNumber,this.startColumn,e,n)}setStartPosition(e,n){return new rn(e,n,this.endLineNumber,this.endColumn)}collapseToStart(){return rn.collapseToStart(this)}static collapseToStart(e){return new rn(e.startLineNumber,e.startColumn,e.startLineNumber,e.startColumn)}static fromPositions(e,n=e){return new rn(e.lineNumber,e.column,n.lineNumber,n.column)}static lift(e){return e?new rn(e.startLineNumber,e.startColumn,e.endLineNumber,e.endColumn):null}static isIRange(e){return e&&"number"===typeof e.startLineNumber&&"number"===typeof e.startColumn&&"number"===typeof e.endLineNumber&&"number"===typeof e.endColumn}static areIntersectingOrTouching(e,n){return!(e.endLineNumber<n.startLineNumber||e.endLineNumber===n.startLineNumber&&e.endColumn<n.startColumn)&&!(n.endLineNumber<e.startLineNumber||n.endLineNumber===e.startLineNumber&&n.endColumn<e.startColumn)}static areIntersecting(e,n){return!(e.endLineNumber<n.startLineNumber||e.endLineNumber===n.startLineNumber&&e.endColumn<=n.startColumn)&&!(n.endLineNumber<e.startLineNumber||n.endLineNumber===e.startLineNumber&&n.endColumn<=e.startColumn)}static compareRangesUsingStarts(e,n){if(e&&n){const t=0|e.startLineNumber,r=0|n.startLineNumber;if(t===r){const t=0|e.startColumn,r=0|n.startColumn;if(t===r){const t=0|e.endLineNumber,r=0|n.endLineNumber;if(t===r){const t=0|e.endColumn,r=0|n.endColumn;return t-r}return t-r}return t-r}return t-r}const t=e?1:0,r=n?1:0;return t-r}static compareRangesUsingEnds(e,n){return e.endLineNumber===n.endLineNumber?e.endColumn===n.endColumn?e.startLineNumber===n.startLineNumber?e.startColumn-n.startColumn:e.startLineNumber-n.startLineNumber:e.endColumn-n.endColumn:e.endLineNumber-n.endLineNumber}static spansMultipleLines(e){return e.endLineNumber>e.startLineNumber}}const an=3;function on(e,n,t,r){const i=new ge(e,n,t);return i.ComputeDiff(r)}class sn{constructor(e){const n=[],t=[];for(let r=0,i=e.length;r<i;r++)n[r]=fn(e[r],1),t[r]=mn(e[r],1);this.lines=e,this._startColumns=n,this._endColumns=t}getElements(){const e=[];for(let n=0,t=this.lines.length;n<t;n++)e[n]=this.lines[n].substring(this._startColumns[n]-1,this._endColumns[n]-1);return e}getStrictElement(e){return this.lines[e]}getStartLineNumber(e){return e+1}getEndLineNumber(e){return e+1}createCharSequence(e,n,t){const r=[],i=[],a=[];let o=0;for(let s=n;s<=t;s++){const n=this.lines[s],t=e?this._startColumns[s]:1,c=e?this._endColumns[s]:n.length+1;for(let e=t;e<c;e++)r[o]=n.charCodeAt(e-1),i[o]=s+1,a[o]=e,o++}return new cn(r,i,a)}}class cn{constructor(e,n,t){this._charCodes=e,this._lineNumbers=n,this._columns=t}getElements(){return this._charCodes}getStartLineNumber(e){return this._lineNumbers[e]}getStartColumn(e){return this._columns[e]}getEndLineNumber(e){return this._lineNumbers[e]}getEndColumn(e){return this._columns[e]+1}}class ln{constructor(e,n,t,r,i,a,o,s){this.originalStartLineNumber=e,this.originalStartColumn=n,this.originalEndLineNumber=t,this.originalEndColumn=r,this.modifiedStartLineNumber=i,this.modifiedStartColumn=a,this.modifiedEndLineNumber=o,this.modifiedEndColumn=s}static createFromDiffChange(e,n,t){let r,i,a,o,s,c,l,u;return 0===e.originalLength?(r=0,i=0,a=0,o=0):(r=n.getStartLineNumber(e.originalStart),i=n.getStartColumn(e.originalStart),a=n.getEndLineNumber(e.originalStart+e.originalLength-1),o=n.getEndColumn(e.originalStart+e.originalLength-1)),0===e.modifiedLength?(s=0,c=0,l=0,u=0):(s=t.getStartLineNumber(e.modifiedStart),c=t.getStartColumn(e.modifiedStart),l=t.getEndLineNumber(e.modifiedStart+e.modifiedLength-1),u=t.getEndColumn(e.modifiedStart+e.modifiedLength-1)),new ln(r,i,a,o,s,c,l,u)}}function un(e){if(e.length<=1)return e;const n=[e[0]];let t=n[0];for(let r=1,i=e.length;r<i;r++){const i=e[r],a=i.originalStart-(t.originalStart+t.originalLength),o=i.modifiedStart-(t.modifiedStart+t.modifiedLength),s=Math.min(a,o);s<an?(t.originalLength=i.originalStart+i.originalLength-t.originalStart,t.modifiedLength=i.modifiedStart+i.modifiedLength-t.modifiedStart):(n.push(i),t=i)}return n}class dn{constructor(e,n,t,r,i){this.originalStartLineNumber=e,this.originalEndLineNumber=n,this.modifiedStartLineNumber=t,this.modifiedEndLineNumber=r,this.charChanges=i}static createFromDiffResult(e,n,t,r,i,a,o){let s,c,l,u,d=void 0;if(0===n.originalLength?(s=t.getStartLineNumber(n.originalStart)-1,c=0):(s=t.getStartLineNumber(n.originalStart),c=t.getEndLineNumber(n.originalStart+n.originalLength-1)),0===n.modifiedLength?(l=r.getStartLineNumber(n.modifiedStart)-1,u=0):(l=r.getStartLineNumber(n.modifiedStart),u=r.getEndLineNumber(n.modifiedStart+n.modifiedLength-1)),a&&n.originalLength>0&&n.originalLength<20&&n.modifiedLength>0&&n.modifiedLength<20&&i()){const a=t.createCharSequence(e,n.originalStart,n.originalStart+n.originalLength-1),s=r.createCharSequence(e,n.modifiedStart,n.modifiedStart+n.modifiedLength-1);let c=on(a,s,i,!0).changes;o&&(c=un(c)),d=[];for(let e=0,n=c.length;e<n;e++)d.push(ln.createFromDiffChange(c[e],a,s))}return new dn(s,c,l,u,d)}}class pn{constructor(e,n,t){this.shouldComputeCharChanges=t.shouldComputeCharChanges,this.shouldPostProcessCharChanges=t.shouldPostProcessCharChanges,this.shouldIgnoreTrimWhitespace=t.shouldIgnoreTrimWhitespace,this.shouldMakePrettyDiff=t.shouldMakePrettyDiff,this.originalLines=e,this.modifiedLines=n,this.original=new sn(e),this.modified=new sn(n),this.continueLineDiff=_n(t.maxComputationTime),this.continueCharDiff=_n(0===t.maxComputationTime?0:Math.min(t.maxComputationTime,5e3))}computeDiff(){if(1===this.original.lines.length&&0===this.original.lines[0].length)return 1===this.modified.lines.length&&0===this.modified.lines[0].length?{quitEarly:!1,changes:[]}:{quitEarly:!1,changes:[{originalStartLineNumber:1,originalEndLineNumber:1,modifiedStartLineNumber:1,modifiedEndLineNumber:this.modified.lines.length,charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}]};if(1===this.modified.lines.length&&0===this.modified.lines[0].length)return{quitEarly:!1,changes:[{originalStartLineNumber:1,originalEndLineNumber:this.original.lines.length,modifiedStartLineNumber:1,modifiedEndLineNumber:1,charChanges:[{modifiedEndColumn:0,modifiedEndLineNumber:0,modifiedStartColumn:0,modifiedStartLineNumber:0,originalEndColumn:0,originalEndLineNumber:0,originalStartColumn:0,originalStartLineNumber:0}]}]};const e=on(this.original,this.modified,this.continueLineDiff,this.shouldMakePrettyDiff),n=e.changes,t=e.quitEarly;if(this.shouldIgnoreTrimWhitespace){const e=[];for(let t=0,r=n.length;t<r;t++)e.push(dn.createFromDiffResult(this.shouldIgnoreTrimWhitespace,n[t],this.original,this.modified,this.continueCharDiff,this.shouldComputeCharChanges,this.shouldPostProcessCharChanges));return{quitEarly:t,changes:e}}const r=[];let i=0,a=0;for(let o=-1,s=n.length;o<s;o++){const e=o+1<s?n[o+1]:null,t=e?e.originalStart:this.originalLines.length,c=e?e.modifiedStart:this.modifiedLines.length;while(i<t&&a<c){const e=this.originalLines[i],n=this.modifiedLines[a];if(e!==n){{let t=fn(e,1),o=fn(n,1);while(t>1&&o>1){const r=e.charCodeAt(t-2),i=n.charCodeAt(o-2);if(r!==i)break;t--,o--}(t>1||o>1)&&this._pushTrimWhitespaceCharChange(r,i+1,1,t,a+1,1,o)}{let t=mn(e,1),o=mn(n,1);const s=e.length+1,c=n.length+1;while(t<s&&o<c){const n=e.charCodeAt(t-1),r=e.charCodeAt(o-1);if(n!==r)break;t++,o++}(t<s||o<c)&&this._pushTrimWhitespaceCharChange(r,i+1,t,s,a+1,o,c)}}i++,a++}e&&(r.push(dn.createFromDiffResult(this.shouldIgnoreTrimWhitespace,e,this.original,this.modified,this.continueCharDiff,this.shouldComputeCharChanges,this.shouldPostProcessCharChanges)),i+=e.originalLength,a+=e.modifiedLength)}return{quitEarly:t,changes:r}}_pushTrimWhitespaceCharChange(e,n,t,r,i,a,o){if(this._mergeTrimWhitespaceCharChange(e,n,t,r,i,a,o))return;let s=void 0;this.shouldComputeCharChanges&&(s=[new ln(n,t,n,r,i,a,i,o)]),e.push(new dn(n,n,i,i,s))}_mergeTrimWhitespaceCharChange(e,n,t,r,i,a,o){const s=e.length;if(0===s)return!1;const c=e[s-1];return 0!==c.originalEndLineNumber&&0!==c.modifiedEndLineNumber&&(c.originalEndLineNumber+1===n&&c.modifiedEndLineNumber+1===i&&(c.originalEndLineNumber=n,c.modifiedEndLineNumber=i,this.shouldComputeCharChanges&&c.charChanges&&c.charChanges.push(new ln(n,t,n,r,i,a,i,o)),!0))}}function fn(e,n){const t=G(e);return-1===t?n:t+1}function mn(e,n){const t=B(e);return-1===t?n:t+2}function _n(e){if(0===e)return()=>!0;const n=Date.now();return()=>{return Date.now()-n<e}}function gn(e){return e<0?0:e>255?255:0|e}function yn(e){return e<0?0:e>4294967295?4294967295:0|e}class hn{constructor(e,n){this._prefixSumIndexOfResultBrand=void 0,this.index=e,this.remainder=n}}class vn{constructor(e){this.values=e,this.prefixSum=new Uint32Array(e.length),this.prefixSumValidIndex=new Int32Array(1),this.prefixSumValidIndex[0]=-1}insertValues(e,n){e=yn(e);const t=this.values,r=this.prefixSum,i=n.length;return 0!==i&&(this.values=new Uint32Array(t.length+i),this.values.set(t.subarray(0,e),0),this.values.set(t.subarray(e),e+i),this.values.set(n,e),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSum=new Uint32Array(this.values.length),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)}changeValue(e,n){return e=yn(e),n=yn(n),this.values[e]!==n&&(this.values[e]=n,e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),!0)}removeValues(e,n){e=yn(e),n=yn(n);const t=this.values,r=this.prefixSum;if(e>=t.length)return!1;let i=t.length-e;return n>=i&&(n=i),0!==n&&(this.values=new Uint32Array(t.length-n),this.values.set(t.subarray(0,e),0),this.values.set(t.subarray(e+n),e),this.prefixSum=new Uint32Array(this.values.length),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(r.subarray(0,this.prefixSumValidIndex[0]+1)),!0)}getTotalSum(){return 0===this.values.length?0:this._getPrefixSum(this.values.length-1)}getPrefixSum(e){return e<0?0:(e=yn(e),this._getPrefixSum(e))}_getPrefixSum(e){if(e<=this.prefixSumValidIndex[0])return this.prefixSum[e];let n=this.prefixSumValidIndex[0]+1;0===n&&(this.prefixSum[0]=this.values[0],n++),e>=this.values.length&&(e=this.values.length-1);for(let t=n;t<=e;t++)this.prefixSum[t]=this.prefixSum[t-1]+this.values[t];return this.prefixSumValidIndex[0]=Math.max(this.prefixSumValidIndex[0],e),this.prefixSum[e]}getIndexOf(e){e=Math.floor(e),this.getTotalSum();let n=0,t=this.values.length-1,r=0,i=0,a=0;while(n<=t)if(r=n+(t-n)/2|0,i=this.prefixSum[r],a=i-this.values[r],e<a)t=r-1;else{if(!(e>=i))break;n=r+1}return new hn(r,e-a)}}class bn{constructor(e,n,t,r){this._uri=e,this._lines=n,this._eol=t,this._versionId=r,this._lineStarts=null,this._cachedTextValue=null}dispose(){this._lines.length=0}get version(){return this._versionId}getText(){return null===this._cachedTextValue&&(this._cachedTextValue=this._lines.join(this._eol)),this._cachedTextValue}onEvents(e){e.eol&&e.eol!==this._eol&&(this._eol=e.eol,this._lineStarts=null);const n=e.changes;for(const t of n)this._acceptDeleteRange(t.range),this._acceptInsertText(new tn(t.range.startLineNumber,t.range.startColumn),t.text);this._versionId=e.versionId,this._cachedTextValue=null}_ensureLineStarts(){if(!this._lineStarts){const e=this._eol.length,n=this._lines.length,t=new Uint32Array(n);for(let r=0;r<n;r++)t[r]=this._lines[r].length+e;this._lineStarts=new vn(t)}}_setLineText(e,n){this._lines[e]=n,this._lineStarts&&this._lineStarts.changeValue(e,this._lines[e].length+this._eol.length)}_acceptDeleteRange(e){if(e.startLineNumber!==e.endLineNumber)this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.endLineNumber-1].substring(e.endColumn-1)),this._lines.splice(e.startLineNumber,e.endLineNumber-e.startLineNumber),this._lineStarts&&this._lineStarts.removeValues(e.startLineNumber,e.endLineNumber-e.startLineNumber);else{if(e.startColumn===e.endColumn)return;this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.startLineNumber-1].substring(e.endColumn-1))}}_acceptInsertText(e,n){if(0===n.length)return;let t=F(n);if(1===t.length)return void this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+t[0]+this._lines[e.lineNumber-1].substring(e.column-1));t[t.length-1]+=this._lines[e.lineNumber-1].substring(e.column-1),this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+t[0]);let r=new Uint32Array(t.length-1);for(let i=1;i<t.length;i++)this._lines.splice(e.lineNumber+i-1,0,t[i]),r[i-1]=t[i].length+this._eol.length;this._lineStarts&&this._lineStarts.insertValues(e.lineNumber,r)}}const En="`~!@#$%^&*()-=+[{]}\\|;:'\",.<>/?";function Tn(e=""){let n="(-?\\d*\\.\\d\\w*)|([^";for(const t of En)e.indexOf(t)>=0||(n+="\\"+t);return n+="\\s]+)",new RegExp(n,"g")}const Sn=Tn();function xn(e){let n=Sn;if(e&&e instanceof RegExp)if(e.global)n=e;else{let t="g";e.ignoreCase&&(t+="i"),e.multiline&&(t+="m"),e.unicode&&(t+="u"),n=new RegExp(e.source,t)}return n.lastIndex=0,n}const Ln={maxLen:1e3,windowSize:15,timeBudget:150};function Dn(e,n,t,r,i=Ln){if(t.length>i.maxLen){let a=e-i.maxLen/2;return a<0?a=0:r+=a,t=t.substring(a,e+i.maxLen/2),Dn(e,n,t,r,i)}const a=Date.now(),o=e-1-r;let s=-1,c=null;for(let l=1;;l++){if(Date.now()-a>=i.timeBudget)break;const e=o-i.windowSize*l;n.lastIndex=Math.max(0,e);const r=An(n,t,o,s);if(!r&&c)break;if(c=r,e<=0)break;s=e}if(c){let e={word:c[0],startColumn:r+1+c.index,endColumn:r+1+c.index+c[0].length};return n.lastIndex=0,e}return null}function An(e,n,t,r){let i;while(i=e.exec(n)){const n=i.index||0;if(n<=t&&e.lastIndex>=t)return i;if(r>0&&n>r)return null}return null}class Cn{constructor(e){let n=gn(e);this._defaultValue=n,this._asciiMap=Cn._createAsciiMap(n),this._map=new Map}static _createAsciiMap(e){let n=new Uint8Array(256);for(let t=0;t<256;t++)n[t]=e;return n}set(e,n){let t=gn(n);e>=0&&e<256?this._asciiMap[e]=t:this._map.set(e,t)}get(e){return e>=0&&e<256?this._asciiMap[e]:this._map.get(e)||this._defaultValue}}class In{constructor(e,n,t){const r=new Uint8Array(e*n);for(let i=0,a=e*n;i<a;i++)r[i]=t;this._data=r,this.rows=e,this.cols=n}get(e,n){return this._data[e*this.cols+n]}set(e,n,t){this._data[e*this.cols+n]=t}}class Nn{constructor(e){let n=0,t=0;for(let i=0,a=e.length;i<a;i++){let[r,a,o]=e[i];a>n&&(n=a),r>t&&(t=r),o>t&&(t=o)}n++,t++;let r=new In(t,n,0);for(let i=0,a=e.length;i<a;i++){let[n,t,a]=e[i];r.set(n,t,a)}this._states=r,this._maxCharCode=n}nextState(e,n){return n<0||n>=this._maxCharCode?0:this._states.get(e,n)}}let kn=null;function Rn(){return null===kn&&(kn=new Nn([[1,104,2],[1,72,2],[1,102,6],[1,70,6],[2,116,3],[2,84,3],[3,116,4],[3,84,4],[4,112,5],[4,80,5],[5,115,9],[5,83,9],[5,58,10],[6,105,7],[6,73,7],[7,108,8],[7,76,8],[8,101,9],[8,69,9],[9,58,10],[10,47,11],[11,47,12]])),kn}let On=null;function wn(){if(null===On){On=new Cn(0);const e=" \t<>'\"、。｡､，．：；‘〈「『〔（［｛｢｣｝］）〕』」〉’｀～…";for(let t=0;t<e.length;t++)On.set(e.charCodeAt(t),1);const n=".,;";for(let t=0;t<n.length;t++)On.set(n.charCodeAt(t),2)}return On}class Pn{static _createLink(e,n,t,r,i){let a=i-1;do{const t=n.charCodeAt(a),r=e.get(t);if(2!==r)break;a--}while(a>r);if(r>0){const e=n.charCodeAt(r-1),t=n.charCodeAt(a);(40===e&&41===t||91===e&&93===t||123===e&&125===t)&&a--}return{range:{startLineNumber:t,startColumn:r+1,endLineNumber:t,endColumn:a+2},url:n.substring(r,a+1)}}static computeLinks(e,n=Rn()){const t=wn();let r=[];for(let i=1,a=e.getLineCount();i<=a;i++){const a=e.getLineContent(i),o=a.length;let s=0,c=0,l=0,u=1,d=!1,p=!1,f=!1,m=!1;while(s<o){let e=!1;const o=a.charCodeAt(s);if(13===u){let n;switch(o){case 40:d=!0,n=0;break;case 41:n=d?0:1;break;case 91:f=!0,p=!0,n=0;break;case 93:f=!1,n=p?0:1;break;case 123:m=!0,n=0;break;case 125:n=m?0:1;break;case 39:n=34===l||96===l?0:1;break;case 34:n=39===l||96===l?0:1;break;case 96:n=39===l||34===l?0:1;break;case 42:n=42===l?1:0;break;case 124:n=124===l?1:0;break;case 32:n=f?0:1;break;default:n=t.get(o)}1===n&&(r.push(Pn._createLink(t,a,i,c,s)),e=!0)}else if(12===u){let n;91===o?(p=!0,n=0):n=t.get(o),1===n?e=!0:u=13}else u=n.nextState(u,o),0===u&&(e=!0);e&&(u=1,d=!1,p=!1,m=!1,c=s+1,l=o),s++}13===u&&r.push(Pn._createLink(t,a,i,c,o))}return r}}function Mn(e){return e&&"function"===typeof e.getLineCount&&"function"===typeof e.getLineContent?Pn.computeLinks(e):[]}class Fn{constructor(){this._defaultValueSet=[["true","false"],["True","False"],["Private","Public","Friend","ReadOnly","Partial","Protected","WriteOnly"],["public","protected","private"]]}navigateValueSet(e,n,t,r,i){if(e&&n){let t=this.doNavigateValueSet(n,i);if(t)return{range:e,value:t}}if(t&&r){let e=this.doNavigateValueSet(r,i);if(e)return{range:t,value:e}}return null}doNavigateValueSet(e,n){let t=this.numberReplace(e,n);return null!==t?t:this.textReplace(e,n)}numberReplace(e,n){let t=Math.pow(10,e.length-(e.lastIndexOf(".")+1)),r=Number(e),i=parseFloat(e);return isNaN(r)||isNaN(i)||r!==i?null:0!==r||n?(r=Math.floor(r*t),r+=n?t:-t,String(r/t)):null}textReplace(e,n){return this.valueSetsReplace(this._defaultValueSet,e,n)}valueSetsReplace(e,n,t){let r=null;for(let i=0,a=e.length;null===r&&i<a;i++)r=this.valueSetReplace(e[i],n,t);return r}valueSetReplace(e,n,t){let r=e.indexOf(n);return r>=0?(r+=t?1:-1,r<0?r=e.length-1:r%=e.length,e[r]):null}}Fn.INSTANCE=new Fn;const Gn=Object.freeze(function(e,n){const t=setTimeout(e.bind(n),0);return{dispose(){clearTimeout(t)}}});var Bn;(function(e){function n(n){return n===e.None||n===e.Cancelled||(n instanceof Un||!(!n||"object"!==typeof n)&&("boolean"===typeof n.isCancellationRequested&&"function"===typeof n.onCancellationRequested))}e.isCancellationToken=n,e.None=Object.freeze({isCancellationRequested:!1,onCancellationRequested:I.None}),e.Cancelled=Object.freeze({isCancellationRequested:!0,onCancellationRequested:Gn})})(Bn||(Bn={}));class Un{constructor(){this._isCancelled=!1,this._emitter=null}cancel(){this._isCancelled||(this._isCancelled=!0,this._emitter&&(this._emitter.fire(void 0),this.dispose()))}get isCancellationRequested(){return this._isCancelled}get onCancellationRequested(){return this._isCancelled?Gn:(this._emitter||(this._emitter=new O),this._emitter.event)}dispose(){this._emitter&&(this._emitter.dispose(),this._emitter=null)}}class Vn{constructor(e){this._token=void 0,this._parentListener=void 0,this._parentListener=e&&e.onCancellationRequested(this.cancel,this)}get token(){return this._token||(this._token=new Un),this._token}cancel(){this._token?this._token instanceof Un&&this._token.cancel():this._token=Bn.Cancelled}dispose(e=!1){e&&this.cancel(),this._parentListener&&this._parentListener.dispose(),this._token?this._token instanceof Un&&this._token.dispose():this._token=Bn.None}}class jn{constructor(){this._keyCodeToStr=[],this._strToKeyCode=Object.create(null)}define(e,n){this._keyCodeToStr[e]=n,this._strToKeyCode[n.toLowerCase()]=e}keyCodeToStr(e){return this._keyCodeToStr[e]}strToKeyCode(e){return this._strToKeyCode[e.toLowerCase()]||0}}const Kn=new jn,Hn=new jn,Wn=new jn,zn=new Array(230),Jn={},qn=[],Xn=Object.create(null),Yn=Object.create(null),Qn=[],Zn=[];for(let Zt=0;Zt<=193;Zt++)Qn[Zt]=-1;for(let Zt=0;Zt<=126;Zt++)Zn[Zt]=-1;var $n,et,nt,tt,rt,it,at,ot,st,ct,lt,ut,dt,pt,ft,mt,_t,gt,yt,ht,vt,bt,Et,Tt,St,xt,Lt,Dt,At,Ct,It,Nt,kt,Rt,Ot,wt;function Pt(e,n){const t=(65535&n)<<16>>>0;return(e|t)>>>0}(function(){const e="",n=[[0,1,0,"None",0,"unknown",0,"VK_UNKNOWN",e,e],[0,1,1,"Hyper",0,e,0,e,e,e],[0,1,2,"Super",0,e,0,e,e,e],[0,1,3,"Fn",0,e,0,e,e,e],[0,1,4,"FnLock",0,e,0,e,e,e],[0,1,5,"Suspend",0,e,0,e,e,e],[0,1,6,"Resume",0,e,0,e,e,e],[0,1,7,"Turbo",0,e,0,e,e,e],[0,1,8,"Sleep",0,e,0,"VK_SLEEP",e,e],[0,1,9,"WakeUp",0,e,0,e,e,e],[31,0,10,"KeyA",31,"A",65,"VK_A",e,e],[32,0,11,"KeyB",32,"B",66,"VK_B",e,e],[33,0,12,"KeyC",33,"C",67,"VK_C",e,e],[34,0,13,"KeyD",34,"D",68,"VK_D",e,e],[35,0,14,"KeyE",35,"E",69,"VK_E",e,e],[36,0,15,"KeyF",36,"F",70,"VK_F",e,e],[37,0,16,"KeyG",37,"G",71,"VK_G",e,e],[38,0,17,"KeyH",38,"H",72,"VK_H",e,e],[39,0,18,"KeyI",39,"I",73,"VK_I",e,e],[40,0,19,"KeyJ",40,"J",74,"VK_J",e,e],[41,0,20,"KeyK",41,"K",75,"VK_K",e,e],[42,0,21,"KeyL",42,"L",76,"VK_L",e,e],[43,0,22,"KeyM",43,"M",77,"VK_M",e,e],[44,0,23,"KeyN",44,"N",78,"VK_N",e,e],[45,0,24,"KeyO",45,"O",79,"VK_O",e,e],[46,0,25,"KeyP",46,"P",80,"VK_P",e,e],[47,0,26,"KeyQ",47,"Q",81,"VK_Q",e,e],[48,0,27,"KeyR",48,"R",82,"VK_R",e,e],[49,0,28,"KeyS",49,"S",83,"VK_S",e,e],[50,0,29,"KeyT",50,"T",84,"VK_T",e,e],[51,0,30,"KeyU",51,"U",85,"VK_U",e,e],[52,0,31,"KeyV",52,"V",86,"VK_V",e,e],[53,0,32,"KeyW",53,"W",87,"VK_W",e,e],[54,0,33,"KeyX",54,"X",88,"VK_X",e,e],[55,0,34,"KeyY",55,"Y",89,"VK_Y",e,e],[56,0,35,"KeyZ",56,"Z",90,"VK_Z",e,e],[22,0,36,"Digit1",22,"1",49,"VK_1",e,e],[23,0,37,"Digit2",23,"2",50,"VK_2",e,e],[24,0,38,"Digit3",24,"3",51,"VK_3",e,e],[25,0,39,"Digit4",25,"4",52,"VK_4",e,e],[26,0,40,"Digit5",26,"5",53,"VK_5",e,e],[27,0,41,"Digit6",27,"6",54,"VK_6",e,e],[28,0,42,"Digit7",28,"7",55,"VK_7",e,e],[29,0,43,"Digit8",29,"8",56,"VK_8",e,e],[30,0,44,"Digit9",30,"9",57,"VK_9",e,e],[21,0,45,"Digit0",21,"0",48,"VK_0",e,e],[3,1,46,"Enter",3,"Enter",13,"VK_RETURN",e,e],[9,1,47,"Escape",9,"Escape",27,"VK_ESCAPE",e,e],[1,1,48,"Backspace",1,"Backspace",8,"VK_BACK",e,e],[2,1,49,"Tab",2,"Tab",9,"VK_TAB",e,e],[10,1,50,"Space",10,"Space",32,"VK_SPACE",e,e],[83,0,51,"Minus",83,"-",189,"VK_OEM_MINUS","-","OEM_MINUS"],[81,0,52,"Equal",81,"=",187,"VK_OEM_PLUS","=","OEM_PLUS"],[87,0,53,"BracketLeft",87,"[",219,"VK_OEM_4","[","OEM_4"],[89,0,54,"BracketRight",89,"]",221,"VK_OEM_6","]","OEM_6"],[88,0,55,"Backslash",88,"\\",220,"VK_OEM_5","\\","OEM_5"],[0,0,56,"IntlHash",0,e,0,e,e,e],[80,0,57,"Semicolon",80,";",186,"VK_OEM_1",";","OEM_1"],[90,0,58,"Quote",90,"'",222,"VK_OEM_7","'","OEM_7"],[86,0,59,"Backquote",86,"`",192,"VK_OEM_3","`","OEM_3"],[82,0,60,"Comma",82,",",188,"VK_OEM_COMMA",",","OEM_COMMA"],[84,0,61,"Period",84,".",190,"VK_OEM_PERIOD",".","OEM_PERIOD"],[85,0,62,"Slash",85,"/",191,"VK_OEM_2","/","OEM_2"],[8,1,63,"CapsLock",8,"CapsLock",20,"VK_CAPITAL",e,e],[59,1,64,"F1",59,"F1",112,"VK_F1",e,e],[60,1,65,"F2",60,"F2",113,"VK_F2",e,e],[61,1,66,"F3",61,"F3",114,"VK_F3",e,e],[62,1,67,"F4",62,"F4",115,"VK_F4",e,e],[63,1,68,"F5",63,"F5",116,"VK_F5",e,e],[64,1,69,"F6",64,"F6",117,"VK_F6",e,e],[65,1,70,"F7",65,"F7",118,"VK_F7",e,e],[66,1,71,"F8",66,"F8",119,"VK_F8",e,e],[67,1,72,"F9",67,"F9",120,"VK_F9",e,e],[68,1,73,"F10",68,"F10",121,"VK_F10",e,e],[69,1,74,"F11",69,"F11",122,"VK_F11",e,e],[70,1,75,"F12",70,"F12",123,"VK_F12",e,e],[0,1,76,"PrintScreen",0,e,0,e,e,e],[79,1,77,"ScrollLock",79,"ScrollLock",145,"VK_SCROLL",e,e],[7,1,78,"Pause",7,"PauseBreak",19,"VK_PAUSE",e,e],[19,1,79,"Insert",19,"Insert",45,"VK_INSERT",e,e],[14,1,80,"Home",14,"Home",36,"VK_HOME",e,e],[11,1,81,"PageUp",11,"PageUp",33,"VK_PRIOR",e,e],[20,1,82,"Delete",20,"Delete",46,"VK_DELETE",e,e],[13,1,83,"End",13,"End",35,"VK_END",e,e],[12,1,84,"PageDown",12,"PageDown",34,"VK_NEXT",e,e],[17,1,85,"ArrowRight",17,"RightArrow",39,"VK_RIGHT","Right",e],[15,1,86,"ArrowLeft",15,"LeftArrow",37,"VK_LEFT","Left",e],[18,1,87,"ArrowDown",18,"DownArrow",40,"VK_DOWN","Down",e],[16,1,88,"ArrowUp",16,"UpArrow",38,"VK_UP","Up",e],[78,1,89,"NumLock",78,"NumLock",144,"VK_NUMLOCK",e,e],[108,1,90,"NumpadDivide",108,"NumPad_Divide",111,"VK_DIVIDE",e,e],[103,1,91,"NumpadMultiply",103,"NumPad_Multiply",106,"VK_MULTIPLY",e,e],[106,1,92,"NumpadSubtract",106,"NumPad_Subtract",109,"VK_SUBTRACT",e,e],[104,1,93,"NumpadAdd",104,"NumPad_Add",107,"VK_ADD",e,e],[3,1,94,"NumpadEnter",3,e,0,e,e,e],[94,1,95,"Numpad1",94,"NumPad1",97,"VK_NUMPAD1",e,e],[95,1,96,"Numpad2",95,"NumPad2",98,"VK_NUMPAD2",e,e],[96,1,97,"Numpad3",96,"NumPad3",99,"VK_NUMPAD3",e,e],[97,1,98,"Numpad4",97,"NumPad4",100,"VK_NUMPAD4",e,e],[98,1,99,"Numpad5",98,"NumPad5",101,"VK_NUMPAD5",e,e],[99,1,100,"Numpad6",99,"NumPad6",102,"VK_NUMPAD6",e,e],[100,1,101,"Numpad7",100,"NumPad7",103,"VK_NUMPAD7",e,e],[101,1,102,"Numpad8",101,"NumPad8",104,"VK_NUMPAD8",e,e],[102,1,103,"Numpad9",102,"NumPad9",105,"VK_NUMPAD9",e,e],[93,1,104,"Numpad0",93,"NumPad0",96,"VK_NUMPAD0",e,e],[107,1,105,"NumpadDecimal",107,"NumPad_Decimal",110,"VK_DECIMAL",e,e],[92,0,106,"IntlBackslash",92,"OEM_102",226,"VK_OEM_102",e,e],[58,1,107,"ContextMenu",58,"ContextMenu",93,e,e,e],[0,1,108,"Power",0,e,0,e,e,e],[0,1,109,"NumpadEqual",0,e,0,e,e,e],[71,1,110,"F13",71,"F13",124,"VK_F13",e,e],[72,1,111,"F14",72,"F14",125,"VK_F14",e,e],[73,1,112,"F15",73,"F15",126,"VK_F15",e,e],[74,1,113,"F16",74,"F16",127,"VK_F16",e,e],[75,1,114,"F17",75,"F17",128,"VK_F17",e,e],[76,1,115,"F18",76,"F18",129,"VK_F18",e,e],[77,1,116,"F19",77,"F19",130,"VK_F19",e,e],[0,1,117,"F20",0,e,0,"VK_F20",e,e],[0,1,118,"F21",0,e,0,"VK_F21",e,e],[0,1,119,"F22",0,e,0,"VK_F22",e,e],[0,1,120,"F23",0,e,0,"VK_F23",e,e],[0,1,121,"F24",0,e,0,"VK_F24",e,e],[0,1,122,"Open",0,e,0,e,e,e],[0,1,123,"Help",0,e,0,e,e,e],[0,1,124,"Select",0,e,0,e,e,e],[0,1,125,"Again",0,e,0,e,e,e],[0,1,126,"Undo",0,e,0,e,e,e],[0,1,127,"Cut",0,e,0,e,e,e],[0,1,128,"Copy",0,e,0,e,e,e],[0,1,129,"Paste",0,e,0,e,e,e],[0,1,130,"Find",0,e,0,e,e,e],[0,1,131,"AudioVolumeMute",112,"AudioVolumeMute",173,"VK_VOLUME_MUTE",e,e],[0,1,132,"AudioVolumeUp",113,"AudioVolumeUp",175,"VK_VOLUME_UP",e,e],[0,1,133,"AudioVolumeDown",114,"AudioVolumeDown",174,"VK_VOLUME_DOWN",e,e],[105,1,134,"NumpadComma",105,"NumPad_Separator",108,"VK_SEPARATOR",e,e],[110,0,135,"IntlRo",110,"ABNT_C1",193,"VK_ABNT_C1",e,e],[0,1,136,"KanaMode",0,e,0,e,e,e],[0,0,137,"IntlYen",0,e,0,e,e,e],[0,1,138,"Convert",0,e,0,e,e,e],[0,1,139,"NonConvert",0,e,0,e,e,e],[0,1,140,"Lang1",0,e,0,e,e,e],[0,1,141,"Lang2",0,e,0,e,e,e],[0,1,142,"Lang3",0,e,0,e,e,e],[0,1,143,"Lang4",0,e,0,e,e,e],[0,1,144,"Lang5",0,e,0,e,e,e],[0,1,145,"Abort",0,e,0,e,e,e],[0,1,146,"Props",0,e,0,e,e,e],[0,1,147,"NumpadParenLeft",0,e,0,e,e,e],[0,1,148,"NumpadParenRight",0,e,0,e,e,e],[0,1,149,"NumpadBackspace",0,e,0,e,e,e],[0,1,150,"NumpadMemoryStore",0,e,0,e,e,e],[0,1,151,"NumpadMemoryRecall",0,e,0,e,e,e],[0,1,152,"NumpadMemoryClear",0,e,0,e,e,e],[0,1,153,"NumpadMemoryAdd",0,e,0,e,e,e],[0,1,154,"NumpadMemorySubtract",0,e,0,e,e,e],[0,1,155,"NumpadClear",0,e,0,e,e,e],[0,1,156,"NumpadClearEntry",0,e,0,e,e,e],[5,1,0,e,5,"Ctrl",17,"VK_CONTROL",e,e],[4,1,0,e,4,"Shift",16,"VK_SHIFT",e,e],[6,1,0,e,6,"Alt",18,"VK_MENU",e,e],[57,1,0,e,57,"Meta",0,"VK_COMMAND",e,e],[5,1,157,"ControlLeft",5,e,0,"VK_LCONTROL",e,e],[4,1,158,"ShiftLeft",4,e,0,"VK_LSHIFT",e,e],[6,1,159,"AltLeft",6,e,0,"VK_LMENU",e,e],[57,1,160,"MetaLeft",57,e,0,"VK_LWIN",e,e],[5,1,161,"ControlRight",5,e,0,"VK_RCONTROL",e,e],[4,1,162,"ShiftRight",4,e,0,"VK_RSHIFT",e,e],[6,1,163,"AltRight",6,e,0,"VK_RMENU",e,e],[57,1,164,"MetaRight",57,e,0,"VK_RWIN",e,e],[0,1,165,"BrightnessUp",0,e,0,e,e,e],[0,1,166,"BrightnessDown",0,e,0,e,e,e],[0,1,167,"MediaPlay",0,e,0,e,e,e],[0,1,168,"MediaRecord",0,e,0,e,e,e],[0,1,169,"MediaFastForward",0,e,0,e,e,e],[0,1,170,"MediaRewind",0,e,0,e,e,e],[114,1,171,"MediaTrackNext",119,"MediaTrackNext",176,"VK_MEDIA_NEXT_TRACK",e,e],[115,1,172,"MediaTrackPrevious",120,"MediaTrackPrevious",177,"VK_MEDIA_PREV_TRACK",e,e],[116,1,173,"MediaStop",121,"MediaStop",178,"VK_MEDIA_STOP",e,e],[0,1,174,"Eject",0,e,0,e,e,e],[117,1,175,"MediaPlayPause",122,"MediaPlayPause",179,"VK_MEDIA_PLAY_PAUSE",e,e],[0,1,176,"MediaSelect",123,"LaunchMediaPlayer",181,"VK_MEDIA_LAUNCH_MEDIA_SELECT",e,e],[0,1,177,"LaunchMail",124,"LaunchMail",180,"VK_MEDIA_LAUNCH_MAIL",e,e],[0,1,178,"LaunchApp2",125,"LaunchApp2",183,"VK_MEDIA_LAUNCH_APP2",e,e],[0,1,179,"LaunchApp1",0,e,0,"VK_MEDIA_LAUNCH_APP1",e,e],[0,1,180,"SelectTask",0,e,0,e,e,e],[0,1,181,"LaunchScreenSaver",0,e,0,e,e,e],[0,1,182,"BrowserSearch",115,"BrowserSearch",170,"VK_BROWSER_SEARCH",e,e],[0,1,183,"BrowserHome",116,"BrowserHome",172,"VK_BROWSER_HOME",e,e],[112,1,184,"BrowserBack",117,"BrowserBack",166,"VK_BROWSER_BACK",e,e],[113,1,185,"BrowserForward",118,"BrowserForward",167,"VK_BROWSER_FORWARD",e,e],[0,1,186,"BrowserStop",0,e,0,"VK_BROWSER_STOP",e,e],[0,1,187,"BrowserRefresh",0,e,0,"VK_BROWSER_REFRESH",e,e],[0,1,188,"BrowserFavorites",0,e,0,"VK_BROWSER_FAVORITES",e,e],[0,1,189,"ZoomToggle",0,e,0,e,e,e],[0,1,190,"MailReply",0,e,0,e,e,e],[0,1,191,"MailForward",0,e,0,e,e,e],[0,1,192,"MailSend",0,e,0,e,e,e],[109,1,0,e,109,"KeyInComposition",229,e,e,e],[111,1,0,e,111,"ABNT_C2",194,"VK_ABNT_C2",e,e],[91,1,0,e,91,"OEM_8",223,"VK_OEM_8",e,e],[0,1,0,e,0,e,0,"VK_CLEAR",e,e],[0,1,0,e,0,e,0,"VK_KANA",e,e],[0,1,0,e,0,e,0,"VK_HANGUL",e,e],[0,1,0,e,0,e,0,"VK_JUNJA",e,e],[0,1,0,e,0,e,0,"VK_FINAL",e,e],[0,1,0,e,0,e,0,"VK_HANJA",e,e],[0,1,0,e,0,e,0,"VK_KANJI",e,e],[0,1,0,e,0,e,0,"VK_CONVERT",e,e],[0,1,0,e,0,e,0,"VK_NONCONVERT",e,e],[0,1,0,e,0,e,0,"VK_ACCEPT",e,e],[0,1,0,e,0,e,0,"VK_MODECHANGE",e,e],[0,1,0,e,0,e,0,"VK_SELECT",e,e],[0,1,0,e,0,e,0,"VK_PRINT",e,e],[0,1,0,e,0,e,0,"VK_EXECUTE",e,e],[0,1,0,e,0,e,0,"VK_SNAPSHOT",e,e],[0,1,0,e,0,e,0,"VK_HELP",e,e],[0,1,0,e,0,e,0,"VK_APPS",e,e],[0,1,0,e,0,e,0,"VK_PROCESSKEY",e,e],[0,1,0,e,0,e,0,"VK_PACKET",e,e],[0,1,0,e,0,e,0,"VK_DBE_SBCSCHAR",e,e],[0,1,0,e,0,e,0,"VK_DBE_DBCSCHAR",e,e],[0,1,0,e,0,e,0,"VK_ATTN",e,e],[0,1,0,e,0,e,0,"VK_CRSEL",e,e],[0,1,0,e,0,e,0,"VK_EXSEL",e,e],[0,1,0,e,0,e,0,"VK_EREOF",e,e],[0,1,0,e,0,e,0,"VK_PLAY",e,e],[0,1,0,e,0,e,0,"VK_ZOOM",e,e],[0,1,0,e,0,e,0,"VK_NONAME",e,e],[0,1,0,e,0,e,0,"VK_PA1",e,e],[0,1,0,e,0,e,0,"VK_OEM_CLEAR",e,e]];let t=[],r=[];for(const i of n){const[e,n,a,o,s,c,l,u,d,p]=i;if(r[a]||(r[a]=!0,qn[a]=o,Xn[o]=a,Yn[o.toLowerCase()]=a,n&&(Qn[a]=s,0!==s&&3!==s&&5!==s&&4!==s&&6!==s&&57!==s&&(Zn[s]=a))),!t[s]){if(t[s]=!0,!c)throw new Error(`String representation missing for key code ${s} around scan code ${o}`);Kn.define(s,c),Hn.define(s,d||c),Wn.define(s,p||d||c)}l&&(zn[l]=s),u&&(Jn[u]=s)}Zn[3]=46})(),function(e){function n(e){return Kn.keyCodeToStr(e)}function t(e){return Kn.strToKeyCode(e)}function r(e){return Hn.keyCodeToStr(e)}function i(e){return Wn.keyCodeToStr(e)}function a(e){return Hn.strToKeyCode(e)||Wn.strToKeyCode(e)}function o(e){if(e>=93&&e<=108)return null;switch(e){case 16:return"Up";case 18:return"Down";case 15:return"Left";case 17:return"Right"}return Kn.keyCodeToStr(e)}e.toString=n,e.fromString=t,e.toUserSettingsUS=r,e.toUserSettingsGeneral=i,e.fromUserSettings=a,e.toElectronAccelerator=o}($n||($n={}));class Mt extends rn{constructor(e,n,t,r){super(e,n,t,r),this.selectionStartLineNumber=e,this.selectionStartColumn=n,this.positionLineNumber=t,this.positionColumn=r}toString(){return"["+this.selectionStartLineNumber+","+this.selectionStartColumn+" -> "+this.positionLineNumber+","+this.positionColumn+"]"}equalsSelection(e){return Mt.selectionsEqual(this,e)}static selectionsEqual(e,n){return e.selectionStartLineNumber===n.selectionStartLineNumber&&e.selectionStartColumn===n.selectionStartColumn&&e.positionLineNumber===n.positionLineNumber&&e.positionColumn===n.positionColumn}getDirection(){return this.selectionStartLineNumber===this.startLineNumber&&this.selectionStartColumn===this.startColumn?0:1}setEndPosition(e,n){return 0===this.getDirection()?new Mt(this.startLineNumber,this.startColumn,e,n):new Mt(e,n,this.startLineNumber,this.startColumn)}getPosition(){return new tn(this.positionLineNumber,this.positionColumn)}setStartPosition(e,n){return 0===this.getDirection()?new Mt(e,n,this.endLineNumber,this.endColumn):new Mt(this.endLineNumber,this.endColumn,e,n)}static fromPositions(e,n=e){return new Mt(e.lineNumber,e.column,n.lineNumber,n.column)}static liftSelection(e){return new Mt(e.selectionStartLineNumber,e.selectionStartColumn,e.positionLineNumber,e.positionColumn)}static selectionsArrEqual(e,n){if(e&&!n||!e&&n)return!1;if(!e&&!n)return!0;if(e.length!==n.length)return!1;for(let t=0,r=e.length;t<r;t++)if(!this.selectionsEqual(e[t],n[t]))return!1;return!0}static isISelection(e){return e&&"number"===typeof e.selectionStartLineNumber&&"number"===typeof e.selectionStartColumn&&"number"===typeof e.positionLineNumber&&"number"===typeof e.positionColumn}static createWithDirection(e,n,t,r,i){return 0===i?new Mt(e,n,t,r):new Mt(t,r,e,n)}}class Ft{constructor(e,n,t){this._tokenBrand=void 0,this.offset=0|e,this.type=n,this.language=t}toString(){return"("+this.offset+", "+this.type+")"}}(function(e){e[e["Unknown"]=0]="Unknown",e[e["Disabled"]=1]="Disabled",e[e["Enabled"]=2]="Enabled"})(et||(et={})),function(e){e[e["KeepWhitespace"]=1]="KeepWhitespace",e[e["InsertAsSnippet"]=4]="InsertAsSnippet"}(nt||(nt={})),function(e){e[e["Method"]=0]="Method",e[e["Function"]=1]="Function",e[e["Constructor"]=2]="Constructor",e[e["Field"]=3]="Field",e[e["Variable"]=4]="Variable",e[e["Class"]=5]="Class",e[e["Struct"]=6]="Struct",e[e["Interface"]=7]="Interface",e[e["Module"]=8]="Module",e[e["Property"]=9]="Property",e[e["Event"]=10]="Event",e[e["Operator"]=11]="Operator",e[e["Unit"]=12]="Unit",e[e["Value"]=13]="Value",e[e["Constant"]=14]="Constant",e[e["Enum"]=15]="Enum",e[e["EnumMember"]=16]="EnumMember",e[e["Keyword"]=17]="Keyword",e[e["Text"]=18]="Text",e[e["Color"]=19]="Color",e[e["File"]=20]="File",e[e["Reference"]=21]="Reference",e[e["Customcolor"]=22]="Customcolor",e[e["Folder"]=23]="Folder",e[e["TypeParameter"]=24]="TypeParameter",e[e["User"]=25]="User",e[e["Issue"]=26]="Issue",e[e["Snippet"]=27]="Snippet"}(tt||(tt={})),function(e){e[e["Deprecated"]=1]="Deprecated"}(rt||(rt={})),function(e){e[e["Invoke"]=0]="Invoke",e[e["TriggerCharacter"]=1]="TriggerCharacter",e[e["TriggerForIncompleteCompletions"]=2]="TriggerForIncompleteCompletions"}(it||(it={})),function(e){e[e["EXACT"]=0]="EXACT",e[e["ABOVE"]=1]="ABOVE",e[e["BELOW"]=2]="BELOW"}(at||(at={})),function(e){e[e["NotSet"]=0]="NotSet",e[e["ContentFlush"]=1]="ContentFlush",e[e["RecoverFromMarkers"]=2]="RecoverFromMarkers",e[e["Explicit"]=3]="Explicit",e[e["Paste"]=4]="Paste",e[e["Undo"]=5]="Undo",e[e["Redo"]=6]="Redo"}(ot||(ot={})),function(e){e[e["LF"]=1]="LF",e[e["CRLF"]=2]="CRLF"}(st||(st={})),function(e){e[e["Text"]=0]="Text",e[e["Read"]=1]="Read",e[e["Write"]=2]="Write"}(ct||(ct={})),function(e){e[e["None"]=0]="None",e[e["Keep"]=1]="Keep",e[e["Brackets"]=2]="Brackets",e[e["Advanced"]=3]="Advanced",e[e["Full"]=4]="Full"}(lt||(lt={})),function(e){e[e["acceptSuggestionOnCommitCharacter"]=0]="acceptSuggestionOnCommitCharacter",e[e["acceptSuggestionOnEnter"]=1]="acceptSuggestionOnEnter",e[e["accessibilitySupport"]=2]="accessibilitySupport",e[e["accessibilityPageSize"]=3]="accessibilityPageSize",e[e["ariaLabel"]=4]="ariaLabel",e[e["autoClosingBrackets"]=5]="autoClosingBrackets",e[e["autoClosingDelete"]=6]="autoClosingDelete",e[e["autoClosingOvertype"]=7]="autoClosingOvertype",e[e["autoClosingQuotes"]=8]="autoClosingQuotes",e[e["autoIndent"]=9]="autoIndent",e[e["automaticLayout"]=10]="automaticLayout",e[e["autoSurround"]=11]="autoSurround",e[e["bracketPairColorization"]=12]="bracketPairColorization",e[e["guides"]=13]="guides",e[e["codeLens"]=14]="codeLens",e[e["codeLensFontFamily"]=15]="codeLensFontFamily",e[e["codeLensFontSize"]=16]="codeLensFontSize",e[e["colorDecorators"]=17]="colorDecorators",e[e["columnSelection"]=18]="columnSelection",e[e["comments"]=19]="comments",e[e["contextmenu"]=20]="contextmenu",e[e["copyWithSyntaxHighlighting"]=21]="copyWithSyntaxHighlighting",e[e["cursorBlinking"]=22]="cursorBlinking",e[e["cursorSmoothCaretAnimation"]=23]="cursorSmoothCaretAnimation",e[e["cursorStyle"]=24]="cursorStyle",e[e["cursorSurroundingLines"]=25]="cursorSurroundingLines",e[e["cursorSurroundingLinesStyle"]=26]="cursorSurroundingLinesStyle",e[e["cursorWidth"]=27]="cursorWidth",e[e["disableLayerHinting"]=28]="disableLayerHinting",e[e["disableMonospaceOptimizations"]=29]="disableMonospaceOptimizations",e[e["domReadOnly"]=30]="domReadOnly",e[e["dragAndDrop"]=31]="dragAndDrop",e[e["emptySelectionClipboard"]=32]="emptySelectionClipboard",e[e["extraEditorClassName"]=33]="extraEditorClassName",e[e["fastScrollSensitivity"]=34]="fastScrollSensitivity",e[e["find"]=35]="find",e[e["fixedOverflowWidgets"]=36]="fixedOverflowWidgets",e[e["folding"]=37]="folding",e[e["foldingStrategy"]=38]="foldingStrategy",e[e["foldingHighlight"]=39]="foldingHighlight",e[e["foldingImportsByDefault"]=40]="foldingImportsByDefault",e[e["unfoldOnClickAfterEndOfLine"]=41]="unfoldOnClickAfterEndOfLine",e[e["fontFamily"]=42]="fontFamily",e[e["fontInfo"]=43]="fontInfo",e[e["fontLigatures"]=44]="fontLigatures",e[e["fontSize"]=45]="fontSize",e[e["fontWeight"]=46]="fontWeight",e[e["formatOnPaste"]=47]="formatOnPaste",e[e["formatOnType"]=48]="formatOnType",e[e["glyphMargin"]=49]="glyphMargin",e[e["gotoLocation"]=50]="gotoLocation",e[e["hideCursorInOverviewRuler"]=51]="hideCursorInOverviewRuler",e[e["hover"]=52]="hover",e[e["inDiffEditor"]=53]="inDiffEditor",e[e["inlineSuggest"]=54]="inlineSuggest",e[e["letterSpacing"]=55]="letterSpacing",e[e["lightbulb"]=56]="lightbulb",e[e["lineDecorationsWidth"]=57]="lineDecorationsWidth",e[e["lineHeight"]=58]="lineHeight",e[e["lineNumbers"]=59]="lineNumbers",e[e["lineNumbersMinChars"]=60]="lineNumbersMinChars",e[e["linkedEditing"]=61]="linkedEditing",e[e["links"]=62]="links",e[e["matchBrackets"]=63]="matchBrackets",e[e["minimap"]=64]="minimap",e[e["mouseStyle"]=65]="mouseStyle",e[e["mouseWheelScrollSensitivity"]=66]="mouseWheelScrollSensitivity",e[e["mouseWheelZoom"]=67]="mouseWheelZoom",e[e["multiCursorMergeOverlapping"]=68]="multiCursorMergeOverlapping",e[e["multiCursorModifier"]=69]="multiCursorModifier",e[e["multiCursorPaste"]=70]="multiCursorPaste",e[e["occurrencesHighlight"]=71]="occurrencesHighlight",e[e["overviewRulerBorder"]=72]="overviewRulerBorder",e[e["overviewRulerLanes"]=73]="overviewRulerLanes",e[e["padding"]=74]="padding",e[e["parameterHints"]=75]="parameterHints",e[e["peekWidgetDefaultFocus"]=76]="peekWidgetDefaultFocus",e[e["definitionLinkOpensInPeek"]=77]="definitionLinkOpensInPeek",e[e["quickSuggestions"]=78]="quickSuggestions",e[e["quickSuggestionsDelay"]=79]="quickSuggestionsDelay",e[e["readOnly"]=80]="readOnly",e[e["renameOnType"]=81]="renameOnType",e[e["renderControlCharacters"]=82]="renderControlCharacters",e[e["renderFinalNewline"]=83]="renderFinalNewline",e[e["renderLineHighlight"]=84]="renderLineHighlight",e[e["renderLineHighlightOnlyWhenFocus"]=85]="renderLineHighlightOnlyWhenFocus",e[e["renderValidationDecorations"]=86]="renderValidationDecorations",e[e["renderWhitespace"]=87]="renderWhitespace",e[e["revealHorizontalRightPadding"]=88]="revealHorizontalRightPadding",e[e["roundedSelection"]=89]="roundedSelection",e[e["rulers"]=90]="rulers",e[e["scrollbar"]=91]="scrollbar",e[e["scrollBeyondLastColumn"]=92]="scrollBeyondLastColumn",e[e["scrollBeyondLastLine"]=93]="scrollBeyondLastLine",e[e["scrollPredominantAxis"]=94]="scrollPredominantAxis",e[e["selectionClipboard"]=95]="selectionClipboard",e[e["selectionHighlight"]=96]="selectionHighlight",e[e["selectOnLineNumbers"]=97]="selectOnLineNumbers",e[e["showFoldingControls"]=98]="showFoldingControls",e[e["showUnused"]=99]="showUnused",e[e["snippetSuggestions"]=100]="snippetSuggestions",e[e["smartSelect"]=101]="smartSelect",e[e["smoothScrolling"]=102]="smoothScrolling",e[e["stickyTabStops"]=103]="stickyTabStops",e[e["stopRenderingLineAfter"]=104]="stopRenderingLineAfter",e[e["suggest"]=105]="suggest",e[e["suggestFontSize"]=106]="suggestFontSize",e[e["suggestLineHeight"]=107]="suggestLineHeight",e[e["suggestOnTriggerCharacters"]=108]="suggestOnTriggerCharacters",e[e["suggestSelection"]=109]="suggestSelection",e[e["tabCompletion"]=110]="tabCompletion",e[e["tabIndex"]=111]="tabIndex",e[e["unusualLineTerminators"]=112]="unusualLineTerminators",e[e["useShadowDOM"]=113]="useShadowDOM",e[e["useTabStops"]=114]="useTabStops",e[e["wordSeparators"]=115]="wordSeparators",e[e["wordWrap"]=116]="wordWrap",e[e["wordWrapBreakAfterCharacters"]=117]="wordWrapBreakAfterCharacters",e[e["wordWrapBreakBeforeCharacters"]=118]="wordWrapBreakBeforeCharacters",e[e["wordWrapColumn"]=119]="wordWrapColumn",e[e["wordWrapOverride1"]=120]="wordWrapOverride1",e[e["wordWrapOverride2"]=121]="wordWrapOverride2",e[e["wrappingIndent"]=122]="wrappingIndent",e[e["wrappingStrategy"]=123]="wrappingStrategy",e[e["showDeprecated"]=124]="showDeprecated",e[e["inlayHints"]=125]="inlayHints",e[e["editorClassName"]=126]="editorClassName",e[e["pixelRatio"]=127]="pixelRatio",e[e["tabFocusMode"]=128]="tabFocusMode",e[e["layoutInfo"]=129]="layoutInfo",e[e["wrappingInfo"]=130]="wrappingInfo"}(ut||(ut={})),function(e){e[e["TextDefined"]=0]="TextDefined",e[e["LF"]=1]="LF",e[e["CRLF"]=2]="CRLF"}(dt||(dt={})),function(e){e[e["LF"]=0]="LF",e[e["CRLF"]=1]="CRLF"}(pt||(pt={})),function(e){e[e["None"]=0]="None",e[e["Indent"]=1]="Indent",e[e["IndentOutdent"]=2]="IndentOutdent",e[e["Outdent"]=3]="Outdent"}(ft||(ft={})),function(e){e[e["Other"]=0]="Other",e[e["Type"]=1]="Type",e[e["Parameter"]=2]="Parameter"}(mt||(mt={})),function(e){e[e["Automatic"]=0]="Automatic",e[e["Explicit"]=1]="Explicit"}(_t||(_t={})),function(e){e[e["DependsOnKbLayout"]=-1]="DependsOnKbLayout",e[e["Unknown"]=0]="Unknown",e[e["Backspace"]=1]="Backspace",e[e["Tab"]=2]="Tab",e[e["Enter"]=3]="Enter",e[e["Shift"]=4]="Shift",e[e["Ctrl"]=5]="Ctrl",e[e["Alt"]=6]="Alt",e[e["PauseBreak"]=7]="PauseBreak",e[e["CapsLock"]=8]="CapsLock",e[e["Escape"]=9]="Escape",e[e["Space"]=10]="Space",e[e["PageUp"]=11]="PageUp",e[e["PageDown"]=12]="PageDown",e[e["End"]=13]="End",e[e["Home"]=14]="Home",e[e["LeftArrow"]=15]="LeftArrow",e[e["UpArrow"]=16]="UpArrow",e[e["RightArrow"]=17]="RightArrow",e[e["DownArrow"]=18]="DownArrow",e[e["Insert"]=19]="Insert",e[e["Delete"]=20]="Delete",e[e["Digit0"]=21]="Digit0",e[e["Digit1"]=22]="Digit1",e[e["Digit2"]=23]="Digit2",e[e["Digit3"]=24]="Digit3",e[e["Digit4"]=25]="Digit4",e[e["Digit5"]=26]="Digit5",e[e["Digit6"]=27]="Digit6",e[e["Digit7"]=28]="Digit7",e[e["Digit8"]=29]="Digit8",e[e["Digit9"]=30]="Digit9",e[e["KeyA"]=31]="KeyA",e[e["KeyB"]=32]="KeyB",e[e["KeyC"]=33]="KeyC",e[e["KeyD"]=34]="KeyD",e[e["KeyE"]=35]="KeyE",e[e["KeyF"]=36]="KeyF",e[e["KeyG"]=37]="KeyG",e[e["KeyH"]=38]="KeyH",e[e["KeyI"]=39]="KeyI",e[e["KeyJ"]=40]="KeyJ",e[e["KeyK"]=41]="KeyK",e[e["KeyL"]=42]="KeyL",e[e["KeyM"]=43]="KeyM",e[e["KeyN"]=44]="KeyN",e[e["KeyO"]=45]="KeyO",e[e["KeyP"]=46]="KeyP",e[e["KeyQ"]=47]="KeyQ",e[e["KeyR"]=48]="KeyR",e[e["KeyS"]=49]="KeyS",e[e["KeyT"]=50]="KeyT",e[e["KeyU"]=51]="KeyU",e[e["KeyV"]=52]="KeyV",e[e["KeyW"]=53]="KeyW",e[e["KeyX"]=54]="KeyX",e[e["KeyY"]=55]="KeyY",e[e["KeyZ"]=56]="KeyZ",e[e["Meta"]=57]="Meta",e[e["ContextMenu"]=58]="ContextMenu",e[e["F1"]=59]="F1",e[e["F2"]=60]="F2",e[e["F3"]=61]="F3",e[e["F4"]=62]="F4",e[e["F5"]=63]="F5",e[e["F6"]=64]="F6",e[e["F7"]=65]="F7",e[e["F8"]=66]="F8",e[e["F9"]=67]="F9",e[e["F10"]=68]="F10",e[e["F11"]=69]="F11",e[e["F12"]=70]="F12",e[e["F13"]=71]="F13",e[e["F14"]=72]="F14",e[e["F15"]=73]="F15",e[e["F16"]=74]="F16",e[e["F17"]=75]="F17",e[e["F18"]=76]="F18",e[e["F19"]=77]="F19",e[e["NumLock"]=78]="NumLock",e[e["ScrollLock"]=79]="ScrollLock",e[e["Semicolon"]=80]="Semicolon",e[e["Equal"]=81]="Equal",e[e["Comma"]=82]="Comma",e[e["Minus"]=83]="Minus",e[e["Period"]=84]="Period",e[e["Slash"]=85]="Slash",e[e["Backquote"]=86]="Backquote",e[e["BracketLeft"]=87]="BracketLeft",e[e["Backslash"]=88]="Backslash",e[e["BracketRight"]=89]="BracketRight",e[e["Quote"]=90]="Quote",e[e["OEM_8"]=91]="OEM_8",e[e["IntlBackslash"]=92]="IntlBackslash",e[e["Numpad0"]=93]="Numpad0",e[e["Numpad1"]=94]="Numpad1",e[e["Numpad2"]=95]="Numpad2",e[e["Numpad3"]=96]="Numpad3",e[e["Numpad4"]=97]="Numpad4",e[e["Numpad5"]=98]="Numpad5",e[e["Numpad6"]=99]="Numpad6",e[e["Numpad7"]=100]="Numpad7",e[e["Numpad8"]=101]="Numpad8",e[e["Numpad9"]=102]="Numpad9",e[e["NumpadMultiply"]=103]="NumpadMultiply",e[e["NumpadAdd"]=104]="NumpadAdd",e[e["NUMPAD_SEPARATOR"]=105]="NUMPAD_SEPARATOR",e[e["NumpadSubtract"]=106]="NumpadSubtract",e[e["NumpadDecimal"]=107]="NumpadDecimal",e[e["NumpadDivide"]=108]="NumpadDivide",e[e["KEY_IN_COMPOSITION"]=109]="KEY_IN_COMPOSITION",e[e["ABNT_C1"]=110]="ABNT_C1",e[e["ABNT_C2"]=111]="ABNT_C2",e[e["AudioVolumeMute"]=112]="AudioVolumeMute",e[e["AudioVolumeUp"]=113]="AudioVolumeUp",e[e["AudioVolumeDown"]=114]="AudioVolumeDown",e[e["BrowserSearch"]=115]="BrowserSearch",e[e["BrowserHome"]=116]="BrowserHome",e[e["BrowserBack"]=117]="BrowserBack",e[e["BrowserForward"]=118]="BrowserForward",e[e["MediaTrackNext"]=119]="MediaTrackNext",e[e["MediaTrackPrevious"]=120]="MediaTrackPrevious",e[e["MediaStop"]=121]="MediaStop",e[e["MediaPlayPause"]=122]="MediaPlayPause",e[e["LaunchMediaPlayer"]=123]="LaunchMediaPlayer",e[e["LaunchMail"]=124]="LaunchMail",e[e["LaunchApp2"]=125]="LaunchApp2",e[e["MAX_VALUE"]=126]="MAX_VALUE"}(gt||(gt={})),function(e){e[e["Hint"]=1]="Hint",e[e["Info"]=2]="Info",e[e["Warning"]=4]="Warning",e[e["Error"]=8]="Error"}(yt||(yt={})),function(e){e[e["Unnecessary"]=1]="Unnecessary",e[e["Deprecated"]=2]="Deprecated"}(ht||(ht={})),function(e){e[e["Inline"]=1]="Inline",e[e["Gutter"]=2]="Gutter"}(vt||(vt={})),function(e){e[e["UNKNOWN"]=0]="UNKNOWN",e[e["TEXTAREA"]=1]="TEXTAREA",e[e["GUTTER_GLYPH_MARGIN"]=2]="GUTTER_GLYPH_MARGIN",e[e["GUTTER_LINE_NUMBERS"]=3]="GUTTER_LINE_NUMBERS",e[e["GUTTER_LINE_DECORATIONS"]=4]="GUTTER_LINE_DECORATIONS",e[e["GUTTER_VIEW_ZONE"]=5]="GUTTER_VIEW_ZONE",e[e["CONTENT_TEXT"]=6]="CONTENT_TEXT",e[e["CONTENT_EMPTY"]=7]="CONTENT_EMPTY",e[e["CONTENT_VIEW_ZONE"]=8]="CONTENT_VIEW_ZONE",e[e["CONTENT_WIDGET"]=9]="CONTENT_WIDGET",e[e["OVERVIEW_RULER"]=10]="OVERVIEW_RULER",e[e["SCROLLBAR"]=11]="SCROLLBAR",e[e["OVERLAY_WIDGET"]=12]="OVERLAY_WIDGET",e[e["OUTSIDE_EDITOR"]=13]="OUTSIDE_EDITOR"}(bt||(bt={})),function(e){e[e["TOP_RIGHT_CORNER"]=0]="TOP_RIGHT_CORNER",e[e["BOTTOM_RIGHT_CORNER"]=1]="BOTTOM_RIGHT_CORNER",e[e["TOP_CENTER"]=2]="TOP_CENTER"}(Et||(Et={})),function(e){e[e["Left"]=1]="Left",e[e["Center"]=2]="Center",e[e["Right"]=4]="Right",e[e["Full"]=7]="Full"}(Tt||(Tt={})),function(e){e[e["Off"]=0]="Off",e[e["On"]=1]="On",e[e["Relative"]=2]="Relative",e[e["Interval"]=3]="Interval",e[e["Custom"]=4]="Custom"}(St||(St={})),function(e){e[e["None"]=0]="None",e[e["Text"]=1]="Text",e[e["Blocks"]=2]="Blocks"}(xt||(xt={})),function(e){e[e["Smooth"]=0]="Smooth",e[e["Immediate"]=1]="Immediate"}(Lt||(Lt={})),function(e){e[e["Auto"]=1]="Auto",e[e["Hidden"]=2]="Hidden",e[e["Visible"]=3]="Visible"}(Dt||(Dt={})),function(e){e[e["LTR"]=0]="LTR",e[e["RTL"]=1]="RTL"}(At||(At={})),function(e){e[e["Invoke"]=1]="Invoke",e[e["TriggerCharacter"]=2]="TriggerCharacter",e[e["ContentChange"]=3]="ContentChange"}(Ct||(Ct={})),function(e){e[e["File"]=0]="File",e[e["Module"]=1]="Module",e[e["Namespace"]=2]="Namespace",e[e["Package"]=3]="Package",e[e["Class"]=4]="Class",e[e["Method"]=5]="Method",e[e["Property"]=6]="Property",e[e["Field"]=7]="Field",e[e["Constructor"]=8]="Constructor",e[e["Enum"]=9]="Enum",e[e["Interface"]=10]="Interface",e[e["Function"]=11]="Function",e[e["Variable"]=12]="Variable",e[e["Constant"]=13]="Constant",e[e["String"]=14]="String",e[e["Number"]=15]="Number",e[e["Boolean"]=16]="Boolean",e[e["Array"]=17]="Array",e[e["Object"]=18]="Object",e[e["Key"]=19]="Key",e[e["Null"]=20]="Null",e[e["EnumMember"]=21]="EnumMember",e[e["Struct"]=22]="Struct",e[e["Event"]=23]="Event",e[e["Operator"]=24]="Operator",e[e["TypeParameter"]=25]="TypeParameter"}(It||(It={})),function(e){e[e["Deprecated"]=1]="Deprecated"}(Nt||(Nt={})),function(e){e[e["Hidden"]=0]="Hidden",e[e["Blink"]=1]="Blink",e[e["Smooth"]=2]="Smooth",e[e["Phase"]=3]="Phase",e[e["Expand"]=4]="Expand",e[e["Solid"]=5]="Solid"}(kt||(kt={})),function(e){e[e["Line"]=1]="Line",e[e["Block"]=2]="Block",e[e["Underline"]=3]="Underline",e[e["LineThin"]=4]="LineThin",e[e["BlockOutline"]=5]="BlockOutline",e[e["UnderlineThin"]=6]="UnderlineThin"}(Rt||(Rt={})),function(e){e[e["AlwaysGrowsWhenTypingAtEdges"]=0]="AlwaysGrowsWhenTypingAtEdges",e[e["NeverGrowsWhenTypingAtEdges"]=1]="NeverGrowsWhenTypingAtEdges",e[e["GrowsOnlyWhenTypingBefore"]=2]="GrowsOnlyWhenTypingBefore",e[e["GrowsOnlyWhenTypingAfter"]=3]="GrowsOnlyWhenTypingAfter"}(Ot||(Ot={})),function(e){e[e["None"]=0]="None",e[e["Same"]=1]="Same",e[e["Indent"]=2]="Indent",e[e["DeepIndent"]=3]="DeepIndent"}(wt||(wt={}));class Gt{static chord(e,n){return Pt(e,n)}}function Bt(){return{editor:void 0,languages:void 0,CancellationTokenSource:Vn,Emitter:O,KeyCode:gt,KeyMod:Gt,Position:tn,Range:rn,Selection:Mt,SelectionDirection:At,MarkerSeverity:yt,MarkerTag:ht,Uri:We,Token:Ft}}Gt.CtrlCmd=2048,Gt.Shift=1024,Gt.Alt=512,Gt.WinCtrl=256;var Ut=function(e,n,t,r){function i(e){return e instanceof t?e:new t(function(n){n(e)})}return new(t||(t=Promise))(function(t,a){function o(e){try{c(r.next(e))}catch(n){a(n)}}function s(e){try{c(r["throw"](e))}catch(n){a(n)}}function c(e){e.done?t(e.value):i(e.value).then(o,s)}c((r=r.apply(e,n||[])).next())})};class Vt extends bn{get uri(){return this._uri}get eol(){return this._eol}getValue(){return this.getText()}getLinesContent(){return this._lines.slice(0)}getLineCount(){return this._lines.length}getLineContent(e){return this._lines[e-1]}getWordAtPosition(e,n){let t=Dn(e.column,xn(n),this._lines[e.lineNumber-1],0);return t?new rn(e.lineNumber,t.startColumn,e.lineNumber,t.endColumn):null}words(e){const n=this._lines,t=this._wordenize.bind(this);let r=0,i="",a=0,o=[];return{*[Symbol.iterator](){while(1)if(a<o.length){const e=i.substring(o[a].start,o[a].end);a+=1,yield e}else{if(!(r<n.length))break;i=n[r],o=t(i,e),a=0,r+=1}}}}getLineWords(e,n){let t=this._lines[e-1],r=this._wordenize(t,n),i=[];for(const a of r)i.push({word:t.substring(a.start,a.end),startColumn:a.start+1,endColumn:a.end+1});return i}_wordenize(e,n){const t=[];let r;n.lastIndex=0;while(r=n.exec(e)){if(0===r[0].length)break;t.push({start:r.index,end:r.index+r[0].length})}return t}getValueInRange(e){if(e=this._validateRange(e),e.startLineNumber===e.endLineNumber)return this._lines[e.startLineNumber-1].substring(e.startColumn-1,e.endColumn-1);let n=this._eol,t=e.startLineNumber-1,r=e.endLineNumber-1,i=[];i.push(this._lines[t].substring(e.startColumn-1));for(let a=t+1;a<r;a++)i.push(this._lines[a]);return i.push(this._lines[r].substring(0,e.endColumn-1)),i.join(n)}offsetAt(e){return e=this._validatePosition(e),this._ensureLineStarts(),this._lineStarts.getPrefixSum(e.lineNumber-2)+(e.column-1)}positionAt(e){e=Math.floor(e),e=Math.max(0,e),this._ensureLineStarts();let n=this._lineStarts.getIndexOf(e),t=this._lines[n.index].length;return{lineNumber:1+n.index,column:1+Math.min(n.remainder,t)}}_validateRange(e){const n=this._validatePosition({lineNumber:e.startLineNumber,column:e.startColumn}),t=this._validatePosition({lineNumber:e.endLineNumber,column:e.endColumn});return n.lineNumber!==e.startLineNumber||n.column!==e.startColumn||t.lineNumber!==e.endLineNumber||t.column!==e.endColumn?{startLineNumber:n.lineNumber,startColumn:n.column,endLineNumber:t.lineNumber,endColumn:t.column}:e}_validatePosition(e){if(!tn.isIPosition(e))throw new Error("bad position");let{lineNumber:n,column:t}=e,r=!1;if(n<1)n=1,t=1,r=!0;else if(n>this._lines.length)n=this._lines.length,t=this._lines[n-1].length+1,r=!0;else{let e=this._lines[n-1].length+1;t<1?(t=1,r=!0):t>e&&(t=e,r=!0)}return r?{lineNumber:n,column:t}:e}}class jt{constructor(e,n){this._host=e,this._models=Object.create(null),this._foreignModuleFactory=n,this._foreignModule=null}dispose(){this._models=Object.create(null)}_getModel(e){return this._models[e]}_getModels(){let e=[];return Object.keys(this._models).forEach(n=>e.push(this._models[n])),e}acceptNewModel(e){this._models[e.url]=new Vt(We.parse(e.url),e.lines,e.EOL,e.versionId)}acceptModelChanged(e,n){if(!this._models[e])return;let t=this._models[e];t.onEvents(n)}acceptRemovedModel(e){this._models[e]&&delete this._models[e]}computeDiff(e,n,t,r){return Ut(this,void 0,void 0,function*(){const i=this._getModel(e),a=this._getModel(n);if(!i||!a)return null;const o=i.getLinesContent(),s=a.getLinesContent(),c=new pn(o,s,{shouldComputeCharChanges:!0,shouldPostProcessCharChanges:!0,shouldIgnoreTrimWhitespace:t,shouldMakePrettyDiff:!0,maxComputationTime:r}),l=c.computeDiff(),u=!(l.changes.length>0)&&this._modelsAreIdentical(i,a);return{quitEarly:l.quitEarly,identical:u,changes:l.changes}})}_modelsAreIdentical(e,n){const t=e.getLineCount(),r=n.getLineCount();if(t!==r)return!1;for(let i=1;i<=t;i++){const t=e.getLineContent(i),r=n.getLineContent(i);if(t!==r)return!1}return!0}computeMoreMinimalEdits(e,n){return Ut(this,void 0,void 0,function*(){const t=this._getModel(e);if(!t)return n;const r=[];let i=void 0;n=n.slice(0).sort((e,n)=>{if(e.range&&n.range)return rn.compareRangesUsingStarts(e.range,n.range);let t=e.range?0:1,r=n.range?0:1;return t-r});for(let{range:e,text:a,eol:o}of n){if("number"===typeof o&&(i=o),rn.isEmpty(e)&&!a)continue;const n=t.getValueInRange(e);if(a=a.replace(/\r\n|\n|\r/g,t.eol),n===a)continue;if(Math.max(a.length,n.length)>jt._diffLimit){r.push({range:e,text:a});continue}const s=pe(n,a,!1),c=t.offsetAt(rn.lift(e).getStartPosition());for(const e of s){const n=t.positionAt(c+e.originalStart),i=t.positionAt(c+e.originalStart+e.originalLength),o={text:a.substr(e.modifiedStart,e.modifiedLength),range:{startLineNumber:n.lineNumber,startColumn:n.column,endLineNumber:i.lineNumber,endColumn:i.column}};t.getValueInRange(o.range)!==o.text&&r.push(o)}}return"number"===typeof i&&r.push({eol:i,text:"",range:{startLineNumber:0,startColumn:0,endLineNumber:0,endColumn:0}}),r})}computeLinks(e){return Ut(this,void 0,void 0,function*(){let n=this._getModel(e);return n?Mn(n):null})}textualSuggest(e,n,t,r){return Ut(this,void 0,void 0,function*(){const i=new C(!0),a=new RegExp(t,r),o=new Set;e:for(let t of e){const e=this._getModel(t);if(e)for(let t of e.words(a))if(t!==n&&isNaN(Number(t))&&(o.add(t),o.size>jt._suggestionsLimit))break e}return{words:Array.from(o),duration:i.elapsed()}})}computeWordRanges(e,n,t,r){return Ut(this,void 0,void 0,function*(){let i=this._getModel(e);if(!i)return Object.create(null);const a=new RegExp(t,r),o=Object.create(null);for(let e=n.startLineNumber;e<n.endLineNumber;e++){let n=i.getLineWords(e,a);for(const t of n){if(!isNaN(Number(t.word)))continue;let n=o[t.word];n||(n=[],o[t.word]=n),n.push({startLineNumber:e,startColumn:t.startColumn,endLineNumber:e,endColumn:t.endColumn})}}return o})}navigateValueSet(e,n,t,r,i){return Ut(this,void 0,void 0,function*(){let a=this._getModel(e);if(!a)return null;let o=new RegExp(r,i);n.startColumn===n.endColumn&&(n={startLineNumber:n.startLineNumber,startColumn:n.startColumn,endLineNumber:n.endLineNumber,endColumn:n.endColumn+1});let s=a.getValueInRange(n),c=a.getWordAtPosition({lineNumber:n.startLineNumber,column:n.startColumn},o);if(!c)return null;let l=a.getValueInRange(c),u=Fn.INSTANCE.navigateValueSet(n,s,c,l,t);return u})}loadForeignModule(e,n,t){const r=(e,n)=>{return this._host.fhr(e,n)},i=M(t,r);let a={host:i,getMirrorModels:()=>{return this._getModels()}};return this._foreignModuleFactory?(this._foreignModule=this._foreignModuleFactory(a,n),Promise.resolve(P(this._foreignModule))):Promise.reject(new Error("Unexpected usage"))}fmr(e,n){if(!this._foreignModule||"function"!==typeof this._foreignModule[e])return Promise.reject(new Error("Missing requestHandler or method: "+e));try{return Promise.resolve(this._foreignModule[e].apply(this._foreignModule,n))}catch(t){return Promise.reject(t)}}}jt._diffLimit=1e5,jt._suggestionsLimit=1e4,"function"===typeof importScripts&&(D["a"].monaco=Bt());let Kt=!1;function Ht(e){if(Kt)return;Kt=!0;const n=new te(e=>{self.postMessage(e)},n=>new jt(n,e));self.onmessage=e=>{n.onmessage(e.data)}}self.onmessage=e=>{Kt||Ht(null)};var Wt=t("973d"),zt={"lib.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es5" />\n/// <reference lib="dom" />\n/// <reference lib="webworker.importscripts" />\n/// <reference lib="scripthost" />\n',"lib.dom.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/////////////////////////////\n/// Window APIs\n/////////////////////////////\n\ninterface AddEventListenerOptions extends EventListenerOptions {\n    once?: boolean;\n    passive?: boolean;\n    signal?: AbortSignal;\n}\n\ninterface AesCbcParams extends Algorithm {\n    iv: BufferSource;\n}\n\ninterface AesCtrParams extends Algorithm {\n    counter: BufferSource;\n    length: number;\n}\n\ninterface AesDerivedKeyParams extends Algorithm {\n    length: number;\n}\n\ninterface AesGcmParams extends Algorithm {\n    additionalData?: BufferSource;\n    iv: BufferSource;\n    tagLength?: number;\n}\n\ninterface AesKeyAlgorithm extends KeyAlgorithm {\n    length: number;\n}\n\ninterface AesKeyGenParams extends Algorithm {\n    length: number;\n}\n\ninterface Algorithm {\n    name: string;\n}\n\ninterface AnalyserOptions extends AudioNodeOptions {\n    fftSize?: number;\n    maxDecibels?: number;\n    minDecibels?: number;\n    smoothingTimeConstant?: number;\n}\n\ninterface AnimationEventInit extends EventInit {\n    animationName?: string;\n    elapsedTime?: number;\n    pseudoElement?: string;\n}\n\ninterface AnimationPlaybackEventInit extends EventInit {\n    currentTime?: number | null;\n    timelineTime?: number | null;\n}\n\ninterface AssignedNodesOptions {\n    flatten?: boolean;\n}\n\ninterface AudioBufferOptions {\n    length: number;\n    numberOfChannels?: number;\n    sampleRate: number;\n}\n\ninterface AudioBufferSourceOptions {\n    buffer?: AudioBuffer | null;\n    detune?: number;\n    loop?: boolean;\n    loopEnd?: number;\n    loopStart?: number;\n    playbackRate?: number;\n}\n\ninterface AudioConfiguration {\n    bitrate?: number;\n    channels?: string;\n    contentType: string;\n    samplerate?: number;\n    spatialRendering?: boolean;\n}\n\ninterface AudioContextOptions {\n    latencyHint?: AudioContextLatencyCategory | number;\n    sampleRate?: number;\n}\n\ninterface AudioNodeOptions {\n    channelCount?: number;\n    channelCountMode?: ChannelCountMode;\n    channelInterpretation?: ChannelInterpretation;\n}\n\ninterface AudioProcessingEventInit extends EventInit {\n    inputBuffer: AudioBuffer;\n    outputBuffer: AudioBuffer;\n    playbackTime: number;\n}\n\ninterface AudioTimestamp {\n    contextTime?: number;\n    performanceTime?: DOMHighResTimeStamp;\n}\n\ninterface AudioWorkletNodeOptions extends AudioNodeOptions {\n    numberOfInputs?: number;\n    numberOfOutputs?: number;\n    outputChannelCount?: number[];\n    parameterData?: Record<string, number>;\n    processorOptions?: any;\n}\n\ninterface AuthenticationExtensionsClientInputs {\n    appid?: string;\n    appidExclude?: string;\n    credProps?: boolean;\n    uvm?: boolean;\n}\n\ninterface AuthenticationExtensionsClientOutputs {\n    appid?: boolean;\n    credProps?: CredentialPropertiesOutput;\n    uvm?: UvmEntries;\n}\n\ninterface AuthenticatorSelectionCriteria {\n    authenticatorAttachment?: AuthenticatorAttachment;\n    requireResidentKey?: boolean;\n    residentKey?: ResidentKeyRequirement;\n    userVerification?: UserVerificationRequirement;\n}\n\ninterface BiquadFilterOptions extends AudioNodeOptions {\n    Q?: number;\n    detune?: number;\n    frequency?: number;\n    gain?: number;\n    type?: BiquadFilterType;\n}\n\ninterface BlobEventInit {\n    data: Blob;\n    timecode?: DOMHighResTimeStamp;\n}\n\ninterface BlobPropertyBag {\n    endings?: EndingType;\n    type?: string;\n}\n\ninterface CSSStyleSheetInit {\n    baseURL?: string;\n    disabled?: boolean;\n    media?: MediaList | string;\n}\n\ninterface CacheQueryOptions {\n    ignoreMethod?: boolean;\n    ignoreSearch?: boolean;\n    ignoreVary?: boolean;\n}\n\ninterface CanvasRenderingContext2DSettings {\n    alpha?: boolean;\n    colorSpace?: PredefinedColorSpace;\n    desynchronized?: boolean;\n    willReadFrequently?: boolean;\n}\n\ninterface ChannelMergerOptions extends AudioNodeOptions {\n    numberOfInputs?: number;\n}\n\ninterface ChannelSplitterOptions extends AudioNodeOptions {\n    numberOfOutputs?: number;\n}\n\ninterface ClientQueryOptions {\n    includeUncontrolled?: boolean;\n    type?: ClientTypes;\n}\n\ninterface ClipboardEventInit extends EventInit {\n    clipboardData?: DataTransfer | null;\n}\n\ninterface ClipboardItemOptions {\n    presentationStyle?: PresentationStyle;\n}\n\ninterface CloseEventInit extends EventInit {\n    code?: number;\n    reason?: string;\n    wasClean?: boolean;\n}\n\ninterface CompositionEventInit extends UIEventInit {\n    data?: string;\n}\n\ninterface ComputedEffectTiming extends EffectTiming {\n    activeDuration?: CSSNumberish;\n    currentIteration?: number | null;\n    endTime?: CSSNumberish;\n    localTime?: CSSNumberish | null;\n    progress?: CSSNumberish | null;\n    startTime?: CSSNumberish;\n}\n\ninterface ComputedKeyframe {\n    composite: CompositeOperationOrAuto;\n    computedOffset: number;\n    easing: string;\n    offset: number | null;\n    [property: string]: string | number | null | undefined;\n}\n\ninterface ConstantSourceOptions {\n    offset?: number;\n}\n\ninterface ConstrainBooleanParameters {\n    exact?: boolean;\n    ideal?: boolean;\n}\n\ninterface ConstrainDOMStringParameters {\n    exact?: string | string[];\n    ideal?: string | string[];\n}\n\ninterface ConstrainDoubleRange extends DoubleRange {\n    exact?: number;\n    ideal?: number;\n}\n\ninterface ConstrainULongRange extends ULongRange {\n    exact?: number;\n    ideal?: number;\n}\n\ninterface ConvolverOptions extends AudioNodeOptions {\n    buffer?: AudioBuffer | null;\n    disableNormalization?: boolean;\n}\n\ninterface CredentialCreationOptions {\n    publicKey?: PublicKeyCredentialCreationOptions;\n    signal?: AbortSignal;\n}\n\ninterface CredentialPropertiesOutput {\n    rk?: boolean;\n}\n\ninterface CredentialRequestOptions {\n    mediation?: CredentialMediationRequirement;\n    publicKey?: PublicKeyCredentialRequestOptions;\n    signal?: AbortSignal;\n}\n\ninterface CryptoKeyPair {\n    privateKey?: CryptoKey;\n    publicKey?: CryptoKey;\n}\n\ninterface CustomEventInit<T = any> extends EventInit {\n    detail?: T;\n}\n\ninterface DOMMatrix2DInit {\n    a?: number;\n    b?: number;\n    c?: number;\n    d?: number;\n    e?: number;\n    f?: number;\n    m11?: number;\n    m12?: number;\n    m21?: number;\n    m22?: number;\n    m41?: number;\n    m42?: number;\n}\n\ninterface DOMMatrixInit extends DOMMatrix2DInit {\n    is2D?: boolean;\n    m13?: number;\n    m14?: number;\n    m23?: number;\n    m24?: number;\n    m31?: number;\n    m32?: number;\n    m33?: number;\n    m34?: number;\n    m43?: number;\n    m44?: number;\n}\n\ninterface DOMPointInit {\n    w?: number;\n    x?: number;\n    y?: number;\n    z?: number;\n}\n\ninterface DOMQuadInit {\n    p1?: DOMPointInit;\n    p2?: DOMPointInit;\n    p3?: DOMPointInit;\n    p4?: DOMPointInit;\n}\n\ninterface DOMRectInit {\n    height?: number;\n    width?: number;\n    x?: number;\n    y?: number;\n}\n\ninterface DelayOptions extends AudioNodeOptions {\n    delayTime?: number;\n    maxDelayTime?: number;\n}\n\ninterface DeviceMotionEventAccelerationInit {\n    x?: number | null;\n    y?: number | null;\n    z?: number | null;\n}\n\ninterface DeviceMotionEventInit extends EventInit {\n    acceleration?: DeviceMotionEventAccelerationInit;\n    accelerationIncludingGravity?: DeviceMotionEventAccelerationInit;\n    interval?: number;\n    rotationRate?: DeviceMotionEventRotationRateInit;\n}\n\ninterface DeviceMotionEventRotationRateInit {\n    alpha?: number | null;\n    beta?: number | null;\n    gamma?: number | null;\n}\n\ninterface DeviceOrientationEventInit extends EventInit {\n    absolute?: boolean;\n    alpha?: number | null;\n    beta?: number | null;\n    gamma?: number | null;\n}\n\ninterface DisplayMediaStreamConstraints {\n    audio?: boolean | MediaTrackConstraints;\n    video?: boolean | MediaTrackConstraints;\n}\n\ninterface DocumentTimelineOptions {\n    originTime?: DOMHighResTimeStamp;\n}\n\ninterface DoubleRange {\n    max?: number;\n    min?: number;\n}\n\ninterface DragEventInit extends MouseEventInit {\n    dataTransfer?: DataTransfer | null;\n}\n\ninterface DynamicsCompressorOptions extends AudioNodeOptions {\n    attack?: number;\n    knee?: number;\n    ratio?: number;\n    release?: number;\n    threshold?: number;\n}\n\ninterface EcKeyAlgorithm extends KeyAlgorithm {\n    namedCurve: NamedCurve;\n}\n\ninterface EcKeyGenParams extends Algorithm {\n    namedCurve: NamedCurve;\n}\n\ninterface EcKeyImportParams extends Algorithm {\n    namedCurve: NamedCurve;\n}\n\ninterface EcdhKeyDeriveParams extends Algorithm {\n    public: CryptoKey;\n}\n\ninterface EcdsaParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n}\n\ninterface EffectTiming {\n    delay?: number;\n    direction?: PlaybackDirection;\n    duration?: number | string;\n    easing?: string;\n    endDelay?: number;\n    fill?: FillMode;\n    iterationStart?: number;\n    iterations?: number;\n    playbackRate?: number;\n}\n\ninterface ElementCreationOptions {\n    is?: string;\n}\n\ninterface ElementDefinitionOptions {\n    extends?: string;\n}\n\ninterface ErrorEventInit extends EventInit {\n    colno?: number;\n    error?: any;\n    filename?: string;\n    lineno?: number;\n    message?: string;\n}\n\ninterface EventInit {\n    bubbles?: boolean;\n    cancelable?: boolean;\n    composed?: boolean;\n}\n\ninterface EventListenerOptions {\n    capture?: boolean;\n}\n\ninterface EventModifierInit extends UIEventInit {\n    altKey?: boolean;\n    ctrlKey?: boolean;\n    metaKey?: boolean;\n    modifierAltGraph?: boolean;\n    modifierCapsLock?: boolean;\n    modifierFn?: boolean;\n    modifierFnLock?: boolean;\n    modifierHyper?: boolean;\n    modifierNumLock?: boolean;\n    modifierScrollLock?: boolean;\n    modifierSuper?: boolean;\n    modifierSymbol?: boolean;\n    modifierSymbolLock?: boolean;\n    shiftKey?: boolean;\n}\n\ninterface EventSourceInit {\n    withCredentials?: boolean;\n}\n\ninterface FilePropertyBag extends BlobPropertyBag {\n    lastModified?: number;\n}\n\ninterface FileSystemFlags {\n    create?: boolean;\n    exclusive?: boolean;\n}\n\ninterface FocusEventInit extends UIEventInit {\n    relatedTarget?: EventTarget | null;\n}\n\ninterface FocusOptions {\n    preventScroll?: boolean;\n}\n\ninterface FontFaceDescriptors {\n    display?: string;\n    featureSettings?: string;\n    stretch?: string;\n    style?: string;\n    unicodeRange?: string;\n    variant?: string;\n    weight?: string;\n}\n\ninterface FontFaceSetLoadEventInit extends EventInit {\n    fontfaces?: FontFace[];\n}\n\ninterface FormDataEventInit extends EventInit {\n    formData: FormData;\n}\n\ninterface FullscreenOptions {\n    navigationUI?: FullscreenNavigationUI;\n}\n\ninterface GainOptions extends AudioNodeOptions {\n    gain?: number;\n}\n\ninterface GamepadEventInit extends EventInit {\n    gamepad: Gamepad;\n}\n\ninterface GetAnimationsOptions {\n    subtree?: boolean;\n}\n\ninterface GetNotificationOptions {\n    tag?: string;\n}\n\ninterface GetRootNodeOptions {\n    composed?: boolean;\n}\n\ninterface HashChangeEventInit extends EventInit {\n    newURL?: string;\n    oldURL?: string;\n}\n\ninterface HkdfParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n    info: BufferSource;\n    salt: BufferSource;\n}\n\ninterface HmacImportParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n    length?: number;\n}\n\ninterface HmacKeyAlgorithm extends KeyAlgorithm {\n    hash: KeyAlgorithm;\n    length: number;\n}\n\ninterface HmacKeyGenParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n    length?: number;\n}\n\ninterface IDBDatabaseInfo {\n    name?: string;\n    version?: number;\n}\n\ninterface IDBIndexParameters {\n    multiEntry?: boolean;\n    unique?: boolean;\n}\n\ninterface IDBObjectStoreParameters {\n    autoIncrement?: boolean;\n    keyPath?: string | string[] | null;\n}\n\ninterface IDBVersionChangeEventInit extends EventInit {\n    newVersion?: number | null;\n    oldVersion?: number;\n}\n\ninterface IIRFilterOptions extends AudioNodeOptions {\n    feedback: number[];\n    feedforward: number[];\n}\n\ninterface IdleRequestOptions {\n    timeout?: number;\n}\n\ninterface ImageBitmapOptions {\n    colorSpaceConversion?: ColorSpaceConversion;\n    imageOrientation?: ImageOrientation;\n    premultiplyAlpha?: PremultiplyAlpha;\n    resizeHeight?: number;\n    resizeQuality?: ResizeQuality;\n    resizeWidth?: number;\n}\n\ninterface ImageBitmapRenderingContextSettings {\n    alpha?: boolean;\n}\n\ninterface ImageDataSettings {\n    colorSpace?: PredefinedColorSpace;\n}\n\ninterface ImportMeta {\n    url: string;\n}\n\ninterface InputEventInit extends UIEventInit {\n    data?: string | null;\n    dataTransfer?: DataTransfer | null;\n    inputType?: string;\n    isComposing?: boolean;\n    targetRanges?: StaticRange[];\n}\n\ninterface IntersectionObserverEntryInit {\n    boundingClientRect: DOMRectInit;\n    intersectionRatio: number;\n    intersectionRect: DOMRectInit;\n    isIntersecting: boolean;\n    rootBounds: DOMRectInit | null;\n    target: Element;\n    time: DOMHighResTimeStamp;\n}\n\ninterface IntersectionObserverInit {\n    root?: Element | Document | null;\n    rootMargin?: string;\n    threshold?: number | number[];\n}\n\ninterface JsonWebKey {\n    alg?: string;\n    crv?: string;\n    d?: string;\n    dp?: string;\n    dq?: string;\n    e?: string;\n    ext?: boolean;\n    k?: string;\n    key_ops?: string[];\n    kty?: string;\n    n?: string;\n    oth?: RsaOtherPrimesInfo[];\n    p?: string;\n    q?: string;\n    qi?: string;\n    use?: string;\n    x?: string;\n    y?: string;\n}\n\ninterface KeyAlgorithm {\n    name: string;\n}\n\ninterface KeyboardEventInit extends EventModifierInit {\n    /** @deprecated */\n    charCode?: number;\n    code?: string;\n    isComposing?: boolean;\n    key?: string;\n    /** @deprecated */\n    keyCode?: number;\n    location?: number;\n    repeat?: boolean;\n}\n\ninterface Keyframe {\n    composite?: CompositeOperationOrAuto;\n    easing?: string;\n    offset?: number | null;\n    [property: string]: string | number | null | undefined;\n}\n\ninterface KeyframeAnimationOptions extends KeyframeEffectOptions {\n    id?: string;\n}\n\ninterface KeyframeEffectOptions extends EffectTiming {\n    composite?: CompositeOperation;\n    iterationComposite?: IterationCompositeOperation;\n    pseudoElement?: string | null;\n}\n\ninterface MediaCapabilitiesDecodingInfo extends MediaCapabilitiesInfo {\n    configuration?: MediaDecodingConfiguration;\n}\n\ninterface MediaCapabilitiesEncodingInfo extends MediaCapabilitiesInfo {\n    configuration?: MediaEncodingConfiguration;\n}\n\ninterface MediaCapabilitiesInfo {\n    powerEfficient: boolean;\n    smooth: boolean;\n    supported: boolean;\n}\n\ninterface MediaConfiguration {\n    audio?: AudioConfiguration;\n    video?: VideoConfiguration;\n}\n\ninterface MediaDecodingConfiguration extends MediaConfiguration {\n    type: MediaDecodingType;\n}\n\ninterface MediaElementAudioSourceOptions {\n    mediaElement: HTMLMediaElement;\n}\n\ninterface MediaEncodingConfiguration extends MediaConfiguration {\n    type: MediaEncodingType;\n}\n\ninterface MediaEncryptedEventInit extends EventInit {\n    initData?: ArrayBuffer | null;\n    initDataType?: string;\n}\n\ninterface MediaImage {\n    sizes?: string;\n    src: string;\n    type?: string;\n}\n\ninterface MediaKeyMessageEventInit extends EventInit {\n    message: ArrayBuffer;\n    messageType: MediaKeyMessageType;\n}\n\ninterface MediaKeySystemConfiguration {\n    audioCapabilities?: MediaKeySystemMediaCapability[];\n    distinctiveIdentifier?: MediaKeysRequirement;\n    initDataTypes?: string[];\n    label?: string;\n    persistentState?: MediaKeysRequirement;\n    sessionTypes?: string[];\n    videoCapabilities?: MediaKeySystemMediaCapability[];\n}\n\ninterface MediaKeySystemMediaCapability {\n    contentType?: string;\n    encryptionScheme?: string | null;\n    robustness?: string;\n}\n\ninterface MediaMetadataInit {\n    album?: string;\n    artist?: string;\n    artwork?: MediaImage[];\n    title?: string;\n}\n\ninterface MediaPositionState {\n    duration?: number;\n    playbackRate?: number;\n    position?: number;\n}\n\ninterface MediaQueryListEventInit extends EventInit {\n    matches?: boolean;\n    media?: string;\n}\n\ninterface MediaRecorderErrorEventInit extends EventInit {\n    error: DOMException;\n}\n\ninterface MediaRecorderOptions {\n    audioBitsPerSecond?: number;\n    bitsPerSecond?: number;\n    mimeType?: string;\n    videoBitsPerSecond?: number;\n}\n\ninterface MediaSessionActionDetails {\n    action: MediaSessionAction;\n    fastSeek?: boolean | null;\n    seekOffset?: number | null;\n    seekTime?: number | null;\n}\n\ninterface MediaStreamAudioSourceOptions {\n    mediaStream: MediaStream;\n}\n\ninterface MediaStreamConstraints {\n    audio?: boolean | MediaTrackConstraints;\n    peerIdentity?: string;\n    video?: boolean | MediaTrackConstraints;\n}\n\ninterface MediaStreamTrackEventInit extends EventInit {\n    track: MediaStreamTrack;\n}\n\ninterface MediaTrackCapabilities {\n    aspectRatio?: DoubleRange;\n    autoGainControl?: boolean[];\n    channelCount?: ULongRange;\n    cursor?: string[];\n    deviceId?: string;\n    displaySurface?: string;\n    echoCancellation?: boolean[];\n    facingMode?: string[];\n    frameRate?: DoubleRange;\n    groupId?: string;\n    height?: ULongRange;\n    latency?: DoubleRange;\n    logicalSurface?: boolean;\n    noiseSuppression?: boolean[];\n    resizeMode?: string[];\n    sampleRate?: ULongRange;\n    sampleSize?: ULongRange;\n    width?: ULongRange;\n}\n\ninterface MediaTrackConstraintSet {\n    aspectRatio?: ConstrainDouble;\n    channelCount?: ConstrainULong;\n    deviceId?: ConstrainDOMString;\n    echoCancellation?: ConstrainBoolean;\n    facingMode?: ConstrainDOMString;\n    frameRate?: ConstrainDouble;\n    groupId?: ConstrainDOMString;\n    height?: ConstrainULong;\n    latency?: ConstrainDouble;\n    sampleRate?: ConstrainULong;\n    sampleSize?: ConstrainULong;\n    suppressLocalAudioPlayback?: ConstrainBoolean;\n    width?: ConstrainULong;\n}\n\ninterface MediaTrackConstraints extends MediaTrackConstraintSet {\n    advanced?: MediaTrackConstraintSet[];\n}\n\ninterface MediaTrackSettings {\n    aspectRatio?: number;\n    deviceId?: string;\n    echoCancellation?: boolean;\n    facingMode?: string;\n    frameRate?: number;\n    groupId?: string;\n    height?: number;\n    restrictOwnAudio?: boolean;\n    sampleRate?: number;\n    sampleSize?: number;\n    width?: number;\n}\n\ninterface MediaTrackSupportedConstraints {\n    aspectRatio?: boolean;\n    deviceId?: boolean;\n    echoCancellation?: boolean;\n    facingMode?: boolean;\n    frameRate?: boolean;\n    groupId?: boolean;\n    height?: boolean;\n    sampleRate?: boolean;\n    sampleSize?: boolean;\n    suppressLocalAudioPlayback?: boolean;\n    width?: boolean;\n}\n\ninterface MessageEventInit<T = any> extends EventInit {\n    data?: T;\n    lastEventId?: string;\n    origin?: string;\n    ports?: MessagePort[];\n    source?: MessageEventSource | null;\n}\n\ninterface MouseEventInit extends EventModifierInit {\n    button?: number;\n    buttons?: number;\n    clientX?: number;\n    clientY?: number;\n    movementX?: number;\n    movementY?: number;\n    relatedTarget?: EventTarget | null;\n    screenX?: number;\n    screenY?: number;\n}\n\ninterface MultiCacheQueryOptions extends CacheQueryOptions {\n    cacheName?: string;\n}\n\ninterface MutationObserverInit {\n    /**\n     * Set to a list of attribute local names (without namespace) if not all attribute mutations need to be observed and attributes is true or omitted.\n     */\n    attributeFilter?: string[];\n    /**\n     * Set to true if attributes is true or omitted and target\'s attribute value before the mutation needs to be recorded.\n     */\n    attributeOldValue?: boolean;\n    /**\n     * Set to true if mutations to target\'s attributes are to be observed. Can be omitted if attributeOldValue or attributeFilter is specified.\n     */\n    attributes?: boolean;\n    /**\n     * Set to true if mutations to target\'s data are to be observed. Can be omitted if characterDataOldValue is specified.\n     */\n    characterData?: boolean;\n    /**\n     * Set to true if characterData is set to true or omitted and target\'s data before the mutation needs to be recorded.\n     */\n    characterDataOldValue?: boolean;\n    /**\n     * Set to true if mutations to target\'s children are to be observed.\n     */\n    childList?: boolean;\n    /**\n     * Set to true if mutations to not just target, but also target\'s descendants are to be observed.\n     */\n    subtree?: boolean;\n}\n\ninterface NotificationAction {\n    action: string;\n    icon?: string;\n    title: string;\n}\n\ninterface NotificationOptions {\n    actions?: NotificationAction[];\n    badge?: string;\n    body?: string;\n    data?: any;\n    dir?: NotificationDirection;\n    icon?: string;\n    image?: string;\n    lang?: string;\n    renotify?: boolean;\n    requireInteraction?: boolean;\n    silent?: boolean;\n    tag?: string;\n    timestamp?: DOMTimeStamp;\n    vibrate?: VibratePattern;\n}\n\ninterface OfflineAudioCompletionEventInit extends EventInit {\n    renderedBuffer: AudioBuffer;\n}\n\ninterface OfflineAudioContextOptions {\n    length: number;\n    numberOfChannels?: number;\n    sampleRate: number;\n}\n\ninterface OptionalEffectTiming {\n    delay?: number;\n    direction?: PlaybackDirection;\n    duration?: number | string;\n    easing?: string;\n    endDelay?: number;\n    fill?: FillMode;\n    iterationStart?: number;\n    iterations?: number;\n    playbackRate?: number;\n}\n\ninterface OscillatorOptions extends AudioNodeOptions {\n    detune?: number;\n    frequency?: number;\n    periodicWave?: PeriodicWave;\n    type?: OscillatorType;\n}\n\ninterface PageTransitionEventInit extends EventInit {\n    persisted?: boolean;\n}\n\ninterface PannerOptions extends AudioNodeOptions {\n    coneInnerAngle?: number;\n    coneOuterAngle?: number;\n    coneOuterGain?: number;\n    distanceModel?: DistanceModelType;\n    maxDistance?: number;\n    orientationX?: number;\n    orientationY?: number;\n    orientationZ?: number;\n    panningModel?: PanningModelType;\n    positionX?: number;\n    positionY?: number;\n    positionZ?: number;\n    refDistance?: number;\n    rolloffFactor?: number;\n}\n\ninterface PaymentCurrencyAmount {\n    currency: string;\n    value: string;\n}\n\ninterface PaymentDetailsBase {\n    displayItems?: PaymentItem[];\n    modifiers?: PaymentDetailsModifier[];\n}\n\ninterface PaymentDetailsInit extends PaymentDetailsBase {\n    id?: string;\n    total: PaymentItem;\n}\n\ninterface PaymentDetailsModifier {\n    additionalDisplayItems?: PaymentItem[];\n    data?: any;\n    supportedMethods: string;\n    total?: PaymentItem;\n}\n\ninterface PaymentDetailsUpdate extends PaymentDetailsBase {\n    paymentMethodErrors?: any;\n    total?: PaymentItem;\n}\n\ninterface PaymentItem {\n    amount: PaymentCurrencyAmount;\n    label: string;\n    pending?: boolean;\n}\n\ninterface PaymentMethodChangeEventInit extends PaymentRequestUpdateEventInit {\n    methodDetails?: any;\n    methodName?: string;\n}\n\ninterface PaymentMethodData {\n    data?: any;\n    supportedMethods: string;\n}\n\ninterface PaymentRequestUpdateEventInit extends EventInit {\n}\n\ninterface PaymentValidationErrors {\n    error?: string;\n    paymentMethod?: any;\n}\n\ninterface Pbkdf2Params extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n    iterations: number;\n    salt: BufferSource;\n}\n\ninterface PerformanceMarkOptions {\n    detail?: any;\n    startTime?: DOMHighResTimeStamp;\n}\n\ninterface PerformanceMeasureOptions {\n    detail?: any;\n    duration?: DOMHighResTimeStamp;\n    end?: string | DOMHighResTimeStamp;\n    start?: string | DOMHighResTimeStamp;\n}\n\ninterface PerformanceObserverInit {\n    buffered?: boolean;\n    entryTypes?: string[];\n    type?: string;\n}\n\ninterface PeriodicWaveConstraints {\n    disableNormalization?: boolean;\n}\n\ninterface PeriodicWaveOptions extends PeriodicWaveConstraints {\n    imag?: number[] | Float32Array;\n    real?: number[] | Float32Array;\n}\n\ninterface PermissionDescriptor {\n    name: PermissionName;\n}\n\ninterface PointerEventInit extends MouseEventInit {\n    coalescedEvents?: PointerEvent[];\n    height?: number;\n    isPrimary?: boolean;\n    pointerId?: number;\n    pointerType?: string;\n    predictedEvents?: PointerEvent[];\n    pressure?: number;\n    tangentialPressure?: number;\n    tiltX?: number;\n    tiltY?: number;\n    twist?: number;\n    width?: number;\n}\n\ninterface PopStateEventInit extends EventInit {\n    state?: any;\n}\n\ninterface PositionOptions {\n    enableHighAccuracy?: boolean;\n    maximumAge?: number;\n    timeout?: number;\n}\n\ninterface PostMessageOptions {\n    transfer?: any[];\n}\n\ninterface ProgressEventInit extends EventInit {\n    lengthComputable?: boolean;\n    loaded?: number;\n    total?: number;\n}\n\ninterface PromiseRejectionEventInit extends EventInit {\n    promise: Promise<any>;\n    reason?: any;\n}\n\ninterface PropertyIndexedKeyframes {\n    composite?: CompositeOperationOrAuto | CompositeOperationOrAuto[];\n    easing?: string | string[];\n    offset?: number | (number | null)[];\n    [property: string]: string | string[] | number | null | (number | null)[] | undefined;\n}\n\ninterface PublicKeyCredentialCreationOptions {\n    attestation?: AttestationConveyancePreference;\n    authenticatorSelection?: AuthenticatorSelectionCriteria;\n    challenge: BufferSource;\n    excludeCredentials?: PublicKeyCredentialDescriptor[];\n    extensions?: AuthenticationExtensionsClientInputs;\n    pubKeyCredParams: PublicKeyCredentialParameters[];\n    rp: PublicKeyCredentialRpEntity;\n    timeout?: number;\n    user: PublicKeyCredentialUserEntity;\n}\n\ninterface PublicKeyCredentialDescriptor {\n    id: BufferSource;\n    transports?: AuthenticatorTransport[];\n    type: PublicKeyCredentialType;\n}\n\ninterface PublicKeyCredentialEntity {\n    name: string;\n}\n\ninterface PublicKeyCredentialParameters {\n    alg: COSEAlgorithmIdentifier;\n    type: PublicKeyCredentialType;\n}\n\ninterface PublicKeyCredentialRequestOptions {\n    allowCredentials?: PublicKeyCredentialDescriptor[];\n    challenge: BufferSource;\n    extensions?: AuthenticationExtensionsClientInputs;\n    rpId?: string;\n    timeout?: number;\n    userVerification?: UserVerificationRequirement;\n}\n\ninterface PublicKeyCredentialRpEntity extends PublicKeyCredentialEntity {\n    id?: string;\n}\n\ninterface PublicKeyCredentialUserEntity extends PublicKeyCredentialEntity {\n    displayName: string;\n    id: BufferSource;\n}\n\ninterface PushSubscriptionJSON {\n    endpoint?: string;\n    expirationTime?: DOMTimeStamp | null;\n    keys?: Record<string, string>;\n}\n\ninterface PushSubscriptionOptionsInit {\n    applicationServerKey?: BufferSource | string | null;\n    userVisibleOnly?: boolean;\n}\n\ninterface QueuingStrategy<T = any> {\n    highWaterMark?: number;\n    size?: QueuingStrategySize<T>;\n}\n\ninterface QueuingStrategyInit {\n    /**\n     * Creates a new ByteLengthQueuingStrategy with the provided high water mark.\n     *\n     * Note that the provided high water mark will not be validated ahead of time. Instead, if it is negative, NaN, or not a number, the resulting ByteLengthQueuingStrategy will cause the corresponding stream constructor to throw.\n     */\n    highWaterMark: number;\n}\n\ninterface RTCAnswerOptions extends RTCOfferAnswerOptions {\n}\n\ninterface RTCCertificateExpiration {\n    expires?: DOMTimeStamp;\n}\n\ninterface RTCConfiguration {\n    bundlePolicy?: RTCBundlePolicy;\n    certificates?: RTCCertificate[];\n    iceCandidatePoolSize?: number;\n    iceServers?: RTCIceServer[];\n    iceTransportPolicy?: RTCIceTransportPolicy;\n    rtcpMuxPolicy?: RTCRtcpMuxPolicy;\n}\n\ninterface RTCDTMFToneChangeEventInit extends EventInit {\n    tone?: string;\n}\n\ninterface RTCDataChannelEventInit extends EventInit {\n    channel: RTCDataChannel;\n}\n\ninterface RTCDataChannelInit {\n    id?: number;\n    maxPacketLifeTime?: number;\n    maxRetransmits?: number;\n    negotiated?: boolean;\n    ordered?: boolean;\n    protocol?: string;\n}\n\ninterface RTCDtlsFingerprint {\n    algorithm?: string;\n    value?: string;\n}\n\ninterface RTCIceCandidateInit {\n    candidate?: string;\n    sdpMLineIndex?: number | null;\n    sdpMid?: string | null;\n    usernameFragment?: string | null;\n}\n\ninterface RTCIceCandidatePairStats extends RTCStats {\n    availableIncomingBitrate?: number;\n    availableOutgoingBitrate?: number;\n    bytesReceived?: number;\n    bytesSent?: number;\n    currentRoundTripTime?: number;\n    localCandidateId: string;\n    nominated?: boolean;\n    remoteCandidateId: string;\n    requestsReceived?: number;\n    requestsSent?: number;\n    responsesReceived?: number;\n    responsesSent?: number;\n    state: RTCStatsIceCandidatePairState;\n    totalRoundTripTime?: number;\n    transportId: string;\n}\n\ninterface RTCIceServer {\n    credential?: string;\n    credentialType?: RTCIceCredentialType;\n    urls: string | string[];\n    username?: string;\n}\n\ninterface RTCInboundRtpStreamStats extends RTCReceivedRtpStreamStats {\n    firCount?: number;\n    framesDecoded?: number;\n    nackCount?: number;\n    pliCount?: number;\n    qpSum?: number;\n    remoteId?: string;\n}\n\ninterface RTCLocalSessionDescriptionInit {\n    sdp?: string;\n    type?: RTCSdpType;\n}\n\ninterface RTCOfferAnswerOptions {\n}\n\ninterface RTCOfferOptions extends RTCOfferAnswerOptions {\n    iceRestart?: boolean;\n    offerToReceiveAudio?: boolean;\n    offerToReceiveVideo?: boolean;\n}\n\ninterface RTCOutboundRtpStreamStats extends RTCSentRtpStreamStats {\n    firCount?: number;\n    framesEncoded?: number;\n    nackCount?: number;\n    pliCount?: number;\n    qpSum?: number;\n    remoteId?: string;\n}\n\ninterface RTCPeerConnectionIceErrorEventInit extends EventInit {\n    address?: string | null;\n    errorCode: number;\n    errorText?: string;\n    port?: number | null;\n    url?: string;\n}\n\ninterface RTCPeerConnectionIceEventInit extends EventInit {\n    candidate?: RTCIceCandidate | null;\n    url?: string | null;\n}\n\ninterface RTCReceivedRtpStreamStats extends RTCRtpStreamStats {\n    jitter?: number;\n    packetsDiscarded?: number;\n    packetsLost?: number;\n    packetsReceived?: number;\n}\n\ninterface RTCRtcpParameters {\n    cname?: string;\n    reducedSize?: boolean;\n}\n\ninterface RTCRtpCapabilities {\n    codecs: RTCRtpCodecCapability[];\n    headerExtensions: RTCRtpHeaderExtensionCapability[];\n}\n\ninterface RTCRtpCodecCapability {\n    channels?: number;\n    clockRate: number;\n    mimeType: string;\n    sdpFmtpLine?: string;\n}\n\ninterface RTCRtpCodecParameters {\n    channels?: number;\n    clockRate: number;\n    mimeType: string;\n    payloadType: number;\n    sdpFmtpLine?: string;\n}\n\ninterface RTCRtpCodingParameters {\n    rid?: string;\n}\n\ninterface RTCRtpContributingSource {\n    audioLevel?: number;\n    rtpTimestamp: number;\n    source: number;\n    timestamp: DOMHighResTimeStamp;\n}\n\ninterface RTCRtpEncodingParameters extends RTCRtpCodingParameters {\n    active?: boolean;\n    maxBitrate?: number;\n    priority?: RTCPriorityType;\n    scaleResolutionDownBy?: number;\n}\n\ninterface RTCRtpHeaderExtensionCapability {\n    uri?: string;\n}\n\ninterface RTCRtpHeaderExtensionParameters {\n    encrypted?: boolean;\n    id: number;\n    uri: string;\n}\n\ninterface RTCRtpParameters {\n    codecs: RTCRtpCodecParameters[];\n    headerExtensions: RTCRtpHeaderExtensionParameters[];\n    rtcp: RTCRtcpParameters;\n}\n\ninterface RTCRtpReceiveParameters extends RTCRtpParameters {\n}\n\ninterface RTCRtpSendParameters extends RTCRtpParameters {\n    degradationPreference?: RTCDegradationPreference;\n    encodings: RTCRtpEncodingParameters[];\n    transactionId: string;\n}\n\ninterface RTCRtpStreamStats extends RTCStats {\n    codecId?: string;\n    kind: string;\n    ssrc: number;\n    transportId?: string;\n}\n\ninterface RTCRtpSynchronizationSource extends RTCRtpContributingSource {\n}\n\ninterface RTCRtpTransceiverInit {\n    direction?: RTCRtpTransceiverDirection;\n    sendEncodings?: RTCRtpEncodingParameters[];\n    streams?: MediaStream[];\n}\n\ninterface RTCSentRtpStreamStats extends RTCRtpStreamStats {\n    bytesSent?: number;\n    packetsSent?: number;\n}\n\ninterface RTCSessionDescriptionInit {\n    sdp?: string;\n    type: RTCSdpType;\n}\n\ninterface RTCStats {\n    id: string;\n    timestamp: DOMHighResTimeStamp;\n    type: RTCStatsType;\n}\n\ninterface RTCTrackEventInit extends EventInit {\n    receiver: RTCRtpReceiver;\n    streams?: MediaStream[];\n    track: MediaStreamTrack;\n    transceiver: RTCRtpTransceiver;\n}\n\ninterface RTCTransportStats extends RTCStats {\n    bytesReceived?: number;\n    bytesSent?: number;\n    dtlsCipher?: string;\n    dtlsState: RTCDtlsTransportState;\n    localCertificateId?: string;\n    remoteCertificateId?: string;\n    rtcpTransportStatsId?: string;\n    selectedCandidatePairId?: string;\n    srtpCipher?: string;\n    tlsVersion?: string;\n}\n\ninterface ReadableStreamDefaultReadDoneResult {\n    done: true;\n    value?: undefined;\n}\n\ninterface ReadableStreamDefaultReadValueResult<T> {\n    done: false;\n    value: T;\n}\n\ninterface ReadableWritablePair<R = any, W = any> {\n    readable: ReadableStream<R>;\n    /**\n     * Provides a convenient, chainable way of piping this readable stream through a transform stream (or any other { writable, readable } pair). It simply pipes the stream into the writable side of the supplied pair, and returns the readable side for further use.\n     *\n     * Piping a stream will lock it for the duration of the pipe, preventing any other consumer from acquiring a reader.\n     */\n    writable: WritableStream<W>;\n}\n\ninterface RegistrationOptions {\n    scope?: string;\n    type?: WorkerType;\n    updateViaCache?: ServiceWorkerUpdateViaCache;\n}\n\ninterface RequestInit {\n    /**\n     * A BodyInit object or null to set request\'s body.\n     */\n    body?: BodyInit | null;\n    /**\n     * A string indicating how the request will interact with the browser\'s cache to set request\'s cache.\n     */\n    cache?: RequestCache;\n    /**\n     * A string indicating whether credentials will be sent with the request always, never, or only when sent to a same-origin URL. Sets request\'s credentials.\n     */\n    credentials?: RequestCredentials;\n    /**\n     * A Headers object, an object literal, or an array of two-item arrays to set request\'s headers.\n     */\n    headers?: HeadersInit;\n    /**\n     * A cryptographic hash of the resource to be fetched by request. Sets request\'s integrity.\n     */\n    integrity?: string;\n    /**\n     * A boolean to set request\'s keepalive.\n     */\n    keepalive?: boolean;\n    /**\n     * A string to set request\'s method.\n     */\n    method?: string;\n    /**\n     * A string to indicate whether the request will use CORS, or will be restricted to same-origin URLs. Sets request\'s mode.\n     */\n    mode?: RequestMode;\n    /**\n     * A string indicating whether request follows redirects, results in an error upon encountering a redirect, or returns the redirect (in an opaque fashion). Sets request\'s redirect.\n     */\n    redirect?: RequestRedirect;\n    /**\n     * A string whose value is a same-origin URL, "about:client", or the empty string, to set request\'s referrer.\n     */\n    referrer?: string;\n    /**\n     * A referrer policy to set request\'s referrerPolicy.\n     */\n    referrerPolicy?: ReferrerPolicy;\n    /**\n     * An AbortSignal to set request\'s signal.\n     */\n    signal?: AbortSignal | null;\n    /**\n     * Can only be null. Used to disassociate request from any Window.\n     */\n    window?: any;\n}\n\ninterface ResizeObserverOptions {\n    box?: ResizeObserverBoxOptions;\n}\n\ninterface ResponseInit {\n    headers?: HeadersInit;\n    status?: number;\n    statusText?: string;\n}\n\ninterface RsaHashedImportParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n}\n\ninterface RsaHashedKeyAlgorithm extends RsaKeyAlgorithm {\n    hash: KeyAlgorithm;\n}\n\ninterface RsaHashedKeyGenParams extends RsaKeyGenParams {\n    hash: HashAlgorithmIdentifier;\n}\n\ninterface RsaKeyAlgorithm extends KeyAlgorithm {\n    modulusLength: number;\n    publicExponent: BigInteger;\n}\n\ninterface RsaKeyGenParams extends Algorithm {\n    modulusLength: number;\n    publicExponent: BigInteger;\n}\n\ninterface RsaOaepParams extends Algorithm {\n    label?: BufferSource;\n}\n\ninterface RsaOtherPrimesInfo {\n    d?: string;\n    r?: string;\n    t?: string;\n}\n\ninterface RsaPssParams extends Algorithm {\n    saltLength: number;\n}\n\ninterface SVGBoundingBoxOptions {\n    clipped?: boolean;\n    fill?: boolean;\n    markers?: boolean;\n    stroke?: boolean;\n}\n\ninterface ScrollIntoViewOptions extends ScrollOptions {\n    block?: ScrollLogicalPosition;\n    inline?: ScrollLogicalPosition;\n}\n\ninterface ScrollOptions {\n    behavior?: ScrollBehavior;\n}\n\ninterface ScrollToOptions extends ScrollOptions {\n    left?: number;\n    top?: number;\n}\n\ninterface SecurityPolicyViolationEventInit extends EventInit {\n    blockedURI?: string;\n    columnNumber?: number;\n    disposition: SecurityPolicyViolationEventDisposition;\n    documentURI: string;\n    effectiveDirective: string;\n    lineNumber?: number;\n    originalPolicy: string;\n    referrer?: string;\n    sample?: string;\n    sourceFile?: string;\n    statusCode: number;\n    violatedDirective: string;\n}\n\ninterface ShadowRootInit {\n    delegatesFocus?: boolean;\n    mode: ShadowRootMode;\n    slotAssignment?: SlotAssignmentMode;\n}\n\ninterface ShareData {\n    files?: File[];\n    text?: string;\n    title?: string;\n    url?: string;\n}\n\ninterface SpeechSynthesisErrorEventInit extends SpeechSynthesisEventInit {\n    error: SpeechSynthesisErrorCode;\n}\n\ninterface SpeechSynthesisEventInit extends EventInit {\n    charIndex?: number;\n    charLength?: number;\n    elapsedTime?: number;\n    name?: string;\n    utterance: SpeechSynthesisUtterance;\n}\n\ninterface StaticRangeInit {\n    endContainer: Node;\n    endOffset: number;\n    startContainer: Node;\n    startOffset: number;\n}\n\ninterface StereoPannerOptions extends AudioNodeOptions {\n    pan?: number;\n}\n\ninterface StorageEstimate {\n    quota?: number;\n    usage?: number;\n}\n\ninterface StorageEventInit extends EventInit {\n    key?: string | null;\n    newValue?: string | null;\n    oldValue?: string | null;\n    storageArea?: Storage | null;\n    url?: string;\n}\n\ninterface StreamPipeOptions {\n    preventAbort?: boolean;\n    preventCancel?: boolean;\n    /**\n     * Pipes this readable stream to a given writable stream destination. The way in which the piping process behaves under various error conditions can be customized with a number of passed options. It returns a promise that fulfills when the piping process completes successfully, or rejects if any errors were encountered.\n     *\n     * Piping a stream will lock it for the duration of the pipe, preventing any other consumer from acquiring a reader.\n     *\n     * Errors and closures of the source and destination streams propagate as follows:\n     *\n     * An error in this source readable stream will abort destination, unless preventAbort is truthy. The returned promise will be rejected with the source\'s error, or with any error that occurs during aborting the destination.\n     *\n     * An error in destination will cancel this source readable stream, unless preventCancel is truthy. The returned promise will be rejected with the destination\'s error, or with any error that occurs during canceling the source.\n     *\n     * When this source readable stream closes, destination will be closed, unless preventClose is truthy. The returned promise will be fulfilled once this process completes, unless an error is encountered while closing the destination, in which case it will be rejected with that error.\n     *\n     * If destination starts out closed or closing, this source readable stream will be canceled, unless preventCancel is true. The returned promise will be rejected with an error indicating piping to a closed stream failed, or with any error that occurs during canceling the source.\n     *\n     * The signal option can be set to an AbortSignal to allow aborting an ongoing pipe operation via the corresponding AbortController. In this case, this source readable stream will be canceled, and destination aborted, unless the respective options preventCancel or preventAbort are set.\n     */\n    preventClose?: boolean;\n    signal?: AbortSignal;\n}\n\ninterface SubmitEventInit extends EventInit {\n    submitter?: HTMLElement | null;\n}\n\ninterface TextDecodeOptions {\n    stream?: boolean;\n}\n\ninterface TextDecoderOptions {\n    fatal?: boolean;\n    ignoreBOM?: boolean;\n}\n\ninterface TextEncoderEncodeIntoResult {\n    read?: number;\n    written?: number;\n}\n\ninterface TouchEventInit extends EventModifierInit {\n    changedTouches?: Touch[];\n    targetTouches?: Touch[];\n    touches?: Touch[];\n}\n\ninterface TouchInit {\n    altitudeAngle?: number;\n    azimuthAngle?: number;\n    clientX?: number;\n    clientY?: number;\n    force?: number;\n    identifier: number;\n    pageX?: number;\n    pageY?: number;\n    radiusX?: number;\n    radiusY?: number;\n    rotationAngle?: number;\n    screenX?: number;\n    screenY?: number;\n    target: EventTarget;\n    touchType?: TouchType;\n}\n\ninterface TrackEventInit extends EventInit {\n    track?: TextTrack | null;\n}\n\ninterface Transformer<I = any, O = any> {\n    flush?: TransformerFlushCallback<O>;\n    readableType?: undefined;\n    start?: TransformerStartCallback<O>;\n    transform?: TransformerTransformCallback<I, O>;\n    writableType?: undefined;\n}\n\ninterface TransitionEventInit extends EventInit {\n    elapsedTime?: number;\n    propertyName?: string;\n    pseudoElement?: string;\n}\n\ninterface UIEventInit extends EventInit {\n    detail?: number;\n    view?: Window | null;\n    /** @deprecated */\n    which?: number;\n}\n\ninterface ULongRange {\n    max?: number;\n    min?: number;\n}\n\ninterface UnderlyingSink<W = any> {\n    abort?: UnderlyingSinkAbortCallback;\n    close?: UnderlyingSinkCloseCallback;\n    start?: UnderlyingSinkStartCallback;\n    type?: undefined;\n    write?: UnderlyingSinkWriteCallback<W>;\n}\n\ninterface UnderlyingSource<R = any> {\n    cancel?: UnderlyingSourceCancelCallback;\n    pull?: UnderlyingSourcePullCallback<R>;\n    start?: UnderlyingSourceStartCallback<R>;\n    type?: undefined;\n}\n\ninterface VideoConfiguration {\n    bitrate: number;\n    colorGamut?: ColorGamut;\n    contentType: string;\n    framerate: number;\n    hdrMetadataType?: HdrMetadataType;\n    height: number;\n    scalabilityMode?: string;\n    transferFunction?: TransferFunction;\n    width: number;\n}\n\ninterface WaveShaperOptions extends AudioNodeOptions {\n    curve?: number[] | Float32Array;\n    oversample?: OverSampleType;\n}\n\ninterface WebGLContextAttributes {\n    alpha?: boolean;\n    antialias?: boolean;\n    depth?: boolean;\n    desynchronized?: boolean;\n    failIfMajorPerformanceCaveat?: boolean;\n    powerPreference?: WebGLPowerPreference;\n    premultipliedAlpha?: boolean;\n    preserveDrawingBuffer?: boolean;\n    stencil?: boolean;\n}\n\ninterface WebGLContextEventInit extends EventInit {\n    statusMessage?: string;\n}\n\ninterface WheelEventInit extends MouseEventInit {\n    deltaMode?: number;\n    deltaX?: number;\n    deltaY?: number;\n    deltaZ?: number;\n}\n\ninterface WindowPostMessageOptions extends PostMessageOptions {\n    targetOrigin?: string;\n}\n\ninterface WorkerOptions {\n    credentials?: RequestCredentials;\n    name?: string;\n    type?: WorkerType;\n}\n\ninterface WorkletOptions {\n    credentials?: RequestCredentials;\n}\n\ntype NodeFilter = ((node: Node) => number) | { acceptNode(node: Node): number; };\n\ndeclare var NodeFilter: {\n    readonly FILTER_ACCEPT: number;\n    readonly FILTER_REJECT: number;\n    readonly FILTER_SKIP: number;\n    readonly SHOW_ALL: number;\n    readonly SHOW_ATTRIBUTE: number;\n    readonly SHOW_CDATA_SECTION: number;\n    readonly SHOW_COMMENT: number;\n    readonly SHOW_DOCUMENT: number;\n    readonly SHOW_DOCUMENT_FRAGMENT: number;\n    readonly SHOW_DOCUMENT_TYPE: number;\n    readonly SHOW_ELEMENT: number;\n    readonly SHOW_ENTITY: number;\n    readonly SHOW_ENTITY_REFERENCE: number;\n    readonly SHOW_NOTATION: number;\n    readonly SHOW_PROCESSING_INSTRUCTION: number;\n    readonly SHOW_TEXT: number;\n};\n\ntype XPathNSResolver = ((prefix: string | null) => string | null) | { lookupNamespaceURI(prefix: string | null): string | null; };\n\n/** The ANGLE_instanced_arrays extension is part of the WebGL API and allows to draw the same object, or groups of similar objects multiple times, if they share the same vertex data, primitive count and type. */\ninterface ANGLE_instanced_arrays {\n    drawArraysInstancedANGLE(mode: GLenum, first: GLint, count: GLsizei, primcount: GLsizei): void;\n    drawElementsInstancedANGLE(mode: GLenum, count: GLsizei, type: GLenum, offset: GLintptr, primcount: GLsizei): void;\n    vertexAttribDivisorANGLE(index: GLuint, divisor: GLuint): void;\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE: GLenum;\n}\n\ninterface ARIAMixin {\n    ariaAtomic: string;\n    ariaAutoComplete: string;\n    ariaBusy: string;\n    ariaChecked: string;\n    ariaColCount: string;\n    ariaColIndex: string;\n    ariaColSpan: string;\n    ariaCurrent: string;\n    ariaDisabled: string;\n    ariaExpanded: string;\n    ariaHasPopup: string;\n    ariaHidden: string;\n    ariaKeyShortcuts: string;\n    ariaLabel: string;\n    ariaLevel: string;\n    ariaLive: string;\n    ariaModal: string;\n    ariaMultiLine: string;\n    ariaMultiSelectable: string;\n    ariaOrientation: string;\n    ariaPlaceholder: string;\n    ariaPosInSet: string;\n    ariaPressed: string;\n    ariaReadOnly: string;\n    ariaRequired: string;\n    ariaRoleDescription: string;\n    ariaRowCount: string;\n    ariaRowIndex: string;\n    ariaRowSpan: string;\n    ariaSelected: string;\n    ariaSetSize: string;\n    ariaSort: string;\n    ariaValueMax: string;\n    ariaValueMin: string;\n    ariaValueNow: string;\n    ariaValueText: string;\n}\n\n/** A controller object that allows you to abort one or more DOM requests as and when desired. */\ninterface AbortController {\n    /**\n     * Returns the AbortSignal object associated with this object.\n     */\n    readonly signal: AbortSignal;\n    /**\n     * Invoking this method will set this object\'s AbortSignal\'s aborted flag and signal to any observers that the associated activity is to be aborted.\n     */\n    abort(): void;\n}\n\ndeclare var AbortController: {\n    prototype: AbortController;\n    new(): AbortController;\n};\n\ninterface AbortSignalEventMap {\n    "abort": Event;\n}\n\n/** A signal object that allows you to communicate with a DOM request (such as a Fetch) and abort it if required via an AbortController object. */\ninterface AbortSignal extends EventTarget {\n    /**\n     * Returns true if this AbortSignal\'s AbortController has signaled to abort, and false otherwise.\n     */\n    readonly aborted: boolean;\n    onabort: ((this: AbortSignal, ev: Event) => any) | null;\n    addEventListener<K extends keyof AbortSignalEventMap>(type: K, listener: (this: AbortSignal, ev: AbortSignalEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AbortSignalEventMap>(type: K, listener: (this: AbortSignal, ev: AbortSignalEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var AbortSignal: {\n    prototype: AbortSignal;\n    new(): AbortSignal;\n};\n\ninterface AbstractRange {\n    /**\n     * Returns true if range is collapsed, and false otherwise.\n     */\n    readonly collapsed: boolean;\n    /**\n     * Returns range\'s end node.\n     */\n    readonly endContainer: Node;\n    /**\n     * Returns range\'s end offset.\n     */\n    readonly endOffset: number;\n    /**\n     * Returns range\'s start node.\n     */\n    readonly startContainer: Node;\n    /**\n     * Returns range\'s start offset.\n     */\n    readonly startOffset: number;\n}\n\ndeclare var AbstractRange: {\n    prototype: AbstractRange;\n    new(): AbstractRange;\n};\n\ninterface AbstractWorkerEventMap {\n    "error": ErrorEvent;\n}\n\ninterface AbstractWorker {\n    onerror: ((this: AbstractWorker, ev: ErrorEvent) => any) | null;\n    addEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: AbstractWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: AbstractWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** A node able to provide real-time frequency and time-domain analysis information. It is an AudioNode that passes the audio stream unchanged from the input to the output, but allows you to take the generated data, process it, and create audio visualizations. */\ninterface AnalyserNode extends AudioNode {\n    fftSize: number;\n    readonly frequencyBinCount: number;\n    maxDecibels: number;\n    minDecibels: number;\n    smoothingTimeConstant: number;\n    getByteFrequencyData(array: Uint8Array): void;\n    getByteTimeDomainData(array: Uint8Array): void;\n    getFloatFrequencyData(array: Float32Array): void;\n    getFloatTimeDomainData(array: Float32Array): void;\n}\n\ndeclare var AnalyserNode: {\n    prototype: AnalyserNode;\n    new(context: BaseAudioContext, options?: AnalyserOptions): AnalyserNode;\n};\n\ninterface Animatable {\n    animate(keyframes: Keyframe[] | PropertyIndexedKeyframes | null, options?: number | KeyframeAnimationOptions): Animation;\n    getAnimations(options?: GetAnimationsOptions): Animation[];\n}\n\ninterface AnimationEventMap {\n    "cancel": AnimationPlaybackEvent;\n    "finish": AnimationPlaybackEvent;\n    "remove": Event;\n}\n\ninterface Animation extends EventTarget {\n    currentTime: number | null;\n    effect: AnimationEffect | null;\n    readonly finished: Promise<Animation>;\n    id: string;\n    oncancel: ((this: Animation, ev: AnimationPlaybackEvent) => any) | null;\n    onfinish: ((this: Animation, ev: AnimationPlaybackEvent) => any) | null;\n    onremove: ((this: Animation, ev: Event) => any) | null;\n    readonly pending: boolean;\n    readonly playState: AnimationPlayState;\n    playbackRate: number;\n    readonly ready: Promise<Animation>;\n    readonly replaceState: AnimationReplaceState;\n    startTime: number | null;\n    timeline: AnimationTimeline | null;\n    cancel(): void;\n    commitStyles(): void;\n    finish(): void;\n    pause(): void;\n    persist(): void;\n    play(): void;\n    reverse(): void;\n    updatePlaybackRate(playbackRate: number): void;\n    addEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: Animation, ev: AnimationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: Animation, ev: AnimationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var Animation: {\n    prototype: Animation;\n    new(effect?: AnimationEffect | null, timeline?: AnimationTimeline | null): Animation;\n};\n\ninterface AnimationEffect {\n    getComputedTiming(): ComputedEffectTiming;\n    getTiming(): EffectTiming;\n    updateTiming(timing?: OptionalEffectTiming): void;\n}\n\ndeclare var AnimationEffect: {\n    prototype: AnimationEffect;\n    new(): AnimationEffect;\n};\n\n/** Events providing information related to animations. */\ninterface AnimationEvent extends Event {\n    readonly animationName: string;\n    readonly elapsedTime: number;\n    readonly pseudoElement: string;\n}\n\ndeclare var AnimationEvent: {\n    prototype: AnimationEvent;\n    new(type: string, animationEventInitDict?: AnimationEventInit): AnimationEvent;\n};\n\ninterface AnimationFrameProvider {\n    cancelAnimationFrame(handle: number): void;\n    requestAnimationFrame(callback: FrameRequestCallback): number;\n}\n\ninterface AnimationPlaybackEvent extends Event {\n    readonly currentTime: number | null;\n    readonly timelineTime: number | null;\n}\n\ndeclare var AnimationPlaybackEvent: {\n    prototype: AnimationPlaybackEvent;\n    new(type: string, eventInitDict?: AnimationPlaybackEventInit): AnimationPlaybackEvent;\n};\n\ninterface AnimationTimeline {\n    readonly currentTime: number | null;\n}\n\ndeclare var AnimationTimeline: {\n    prototype: AnimationTimeline;\n    new(): AnimationTimeline;\n};\n\n/** A DOM element\'s attribute as an object. In most DOM methods, you will probably directly retrieve the attribute as a string (e.g., Element.getAttribute(), but certain functions (e.g., Element.getAttributeNode()) or means of iterating give Attr types. */\ninterface Attr extends Node {\n    readonly localName: string;\n    readonly name: string;\n    readonly namespaceURI: string | null;\n    readonly ownerDocument: Document;\n    readonly ownerElement: Element | null;\n    readonly prefix: string | null;\n    readonly specified: boolean;\n    value: string;\n}\n\ndeclare var Attr: {\n    prototype: Attr;\n    new(): Attr;\n};\n\n/** A short audio asset residing in memory, created from an audio file using the AudioContext.decodeAudioData() method, or from raw data using AudioContext.createBuffer(). Once put into an AudioBuffer, the audio can then be played by being passed into an AudioBufferSourceNode. */\ninterface AudioBuffer {\n    readonly duration: number;\n    readonly length: number;\n    readonly numberOfChannels: number;\n    readonly sampleRate: number;\n    copyFromChannel(destination: Float32Array, channelNumber: number, bufferOffset?: number): void;\n    copyToChannel(source: Float32Array, channelNumber: number, bufferOffset?: number): void;\n    getChannelData(channel: number): Float32Array;\n}\n\ndeclare var AudioBuffer: {\n    prototype: AudioBuffer;\n    new(options: AudioBufferOptions): AudioBuffer;\n};\n\n/** An AudioScheduledSourceNode which represents an audio source consisting of in-memory audio data, stored in an AudioBuffer. It\'s especially useful for playing back audio which has particularly stringent timing accuracy requirements, such as for sounds that must match a specific rhythm and can be kept in memory rather than being played from disk or the network. */\ninterface AudioBufferSourceNode extends AudioScheduledSourceNode {\n    buffer: AudioBuffer | null;\n    readonly detune: AudioParam;\n    loop: boolean;\n    loopEnd: number;\n    loopStart: number;\n    readonly playbackRate: AudioParam;\n    start(when?: number, offset?: number, duration?: number): void;\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioBufferSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioBufferSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var AudioBufferSourceNode: {\n    prototype: AudioBufferSourceNode;\n    new(context: BaseAudioContext, options?: AudioBufferSourceOptions): AudioBufferSourceNode;\n};\n\n/** An audio-processing graph built from audio modules linked together, each represented by an AudioNode. */\ninterface AudioContext extends BaseAudioContext {\n    readonly baseLatency: number;\n    close(): Promise<void>;\n    createMediaElementSource(mediaElement: HTMLMediaElement): MediaElementAudioSourceNode;\n    createMediaStreamDestination(): MediaStreamAudioDestinationNode;\n    createMediaStreamSource(mediaStream: MediaStream): MediaStreamAudioSourceNode;\n    getOutputTimestamp(): AudioTimestamp;\n    resume(): Promise<void>;\n    suspend(): Promise<void>;\n    addEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: AudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: AudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var AudioContext: {\n    prototype: AudioContext;\n    new(contextOptions?: AudioContextOptions): AudioContext;\n};\n\n/** AudioDestinationNode has no output (as it is the output, no more AudioNode can be linked after it in the audio graph) and one input. The number of channels in the input must be between 0 and the maxChannelCount value or an exception is raised. */\ninterface AudioDestinationNode extends AudioNode {\n    readonly maxChannelCount: number;\n}\n\ndeclare var AudioDestinationNode: {\n    prototype: AudioDestinationNode;\n    new(): AudioDestinationNode;\n};\n\n/** The position and orientation of the unique person listening to the audio scene, and is used in audio spatialization. All PannerNodes spatialize in relation to the AudioListener stored in the BaseAudioContext.listener attribute. */\ninterface AudioListener {\n    readonly forwardX: AudioParam;\n    readonly forwardY: AudioParam;\n    readonly forwardZ: AudioParam;\n    readonly positionX: AudioParam;\n    readonly positionY: AudioParam;\n    readonly positionZ: AudioParam;\n    readonly upX: AudioParam;\n    readonly upY: AudioParam;\n    readonly upZ: AudioParam;\n    /** @deprecated */\n    setOrientation(x: number, y: number, z: number, xUp: number, yUp: number, zUp: number): void;\n    /** @deprecated */\n    setPosition(x: number, y: number, z: number): void;\n}\n\ndeclare var AudioListener: {\n    prototype: AudioListener;\n    new(): AudioListener;\n};\n\n/** A generic interface for representing an audio processing module. Examples include: */\ninterface AudioNode extends EventTarget {\n    channelCount: number;\n    channelCountMode: ChannelCountMode;\n    channelInterpretation: ChannelInterpretation;\n    readonly context: BaseAudioContext;\n    readonly numberOfInputs: number;\n    readonly numberOfOutputs: number;\n    connect(destinationNode: AudioNode, output?: number, input?: number): AudioNode;\n    connect(destinationParam: AudioParam, output?: number): void;\n    disconnect(): void;\n    disconnect(output: number): void;\n    disconnect(destinationNode: AudioNode): void;\n    disconnect(destinationNode: AudioNode, output: number): void;\n    disconnect(destinationNode: AudioNode, output: number, input: number): void;\n    disconnect(destinationParam: AudioParam): void;\n    disconnect(destinationParam: AudioParam, output: number): void;\n}\n\ndeclare var AudioNode: {\n    prototype: AudioNode;\n    new(): AudioNode;\n};\n\n/** The Web Audio API\'s AudioParam interface represents an audio-related parameter, usually a parameter of an AudioNode (such as GainNode.gain). */\ninterface AudioParam {\n    automationRate: AutomationRate;\n    readonly defaultValue: number;\n    readonly maxValue: number;\n    readonly minValue: number;\n    value: number;\n    cancelAndHoldAtTime(cancelTime: number): AudioParam;\n    cancelScheduledValues(cancelTime: number): AudioParam;\n    exponentialRampToValueAtTime(value: number, endTime: number): AudioParam;\n    linearRampToValueAtTime(value: number, endTime: number): AudioParam;\n    setTargetAtTime(target: number, startTime: number, timeConstant: number): AudioParam;\n    setValueAtTime(value: number, startTime: number): AudioParam;\n    setValueCurveAtTime(values: number[] | Float32Array, startTime: number, duration: number): AudioParam;\n}\n\ndeclare var AudioParam: {\n    prototype: AudioParam;\n    new(): AudioParam;\n};\n\ninterface AudioParamMap {\n    forEach(callbackfn: (value: AudioParam, key: string, parent: AudioParamMap) => void, thisArg?: any): void;\n}\n\ndeclare var AudioParamMap: {\n    prototype: AudioParamMap;\n    new(): AudioParamMap;\n};\n\n/**\n * The Web Audio API events that occur when a ScriptProcessorNode input buffer is ready to be processed.\n * @deprecated As of the August 29 2014 Web Audio API spec publication, this feature has been marked as deprecated, and is soon to be replaced by AudioWorklet.\n */\ninterface AudioProcessingEvent extends Event {\n    /** @deprecated */\n    readonly inputBuffer: AudioBuffer;\n    /** @deprecated */\n    readonly outputBuffer: AudioBuffer;\n    /** @deprecated */\n    readonly playbackTime: number;\n}\n\n/** @deprecated */\ndeclare var AudioProcessingEvent: {\n    prototype: AudioProcessingEvent;\n    new(type: string, eventInitDict: AudioProcessingEventInit): AudioProcessingEvent;\n};\n\ninterface AudioScheduledSourceNodeEventMap {\n    "ended": Event;\n}\n\ninterface AudioScheduledSourceNode extends AudioNode {\n    onended: ((this: AudioScheduledSourceNode, ev: Event) => any) | null;\n    start(when?: number): void;\n    stop(when?: number): void;\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioScheduledSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: AudioScheduledSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var AudioScheduledSourceNode: {\n    prototype: AudioScheduledSourceNode;\n    new(): AudioScheduledSourceNode;\n};\n\ninterface AudioWorklet extends Worklet {\n}\n\ndeclare var AudioWorklet: {\n    prototype: AudioWorklet;\n    new(): AudioWorklet;\n};\n\ninterface AudioWorkletNodeEventMap {\n    "processorerror": Event;\n}\n\ninterface AudioWorkletNode extends AudioNode {\n    onprocessorerror: ((this: AudioWorkletNode, ev: Event) => any) | null;\n    readonly parameters: AudioParamMap;\n    readonly port: MessagePort;\n    addEventListener<K extends keyof AudioWorkletNodeEventMap>(type: K, listener: (this: AudioWorkletNode, ev: AudioWorkletNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AudioWorkletNodeEventMap>(type: K, listener: (this: AudioWorkletNode, ev: AudioWorkletNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var AudioWorkletNode: {\n    prototype: AudioWorkletNode;\n    new(context: BaseAudioContext, name: string, options?: AudioWorkletNodeOptions): AudioWorkletNode;\n};\n\ninterface AuthenticatorAssertionResponse extends AuthenticatorResponse {\n    readonly authenticatorData: ArrayBuffer;\n    readonly signature: ArrayBuffer;\n    readonly userHandle: ArrayBuffer | null;\n}\n\ndeclare var AuthenticatorAssertionResponse: {\n    prototype: AuthenticatorAssertionResponse;\n    new(): AuthenticatorAssertionResponse;\n};\n\ninterface AuthenticatorAttestationResponse extends AuthenticatorResponse {\n    readonly attestationObject: ArrayBuffer;\n}\n\ndeclare var AuthenticatorAttestationResponse: {\n    prototype: AuthenticatorAttestationResponse;\n    new(): AuthenticatorAttestationResponse;\n};\n\ninterface AuthenticatorResponse {\n    readonly clientDataJSON: ArrayBuffer;\n}\n\ndeclare var AuthenticatorResponse: {\n    prototype: AuthenticatorResponse;\n    new(): AuthenticatorResponse;\n};\n\ninterface BarProp {\n    readonly visible: boolean;\n}\n\ndeclare var BarProp: {\n    prototype: BarProp;\n    new(): BarProp;\n};\n\ninterface BaseAudioContextEventMap {\n    "statechange": Event;\n}\n\ninterface BaseAudioContext extends EventTarget {\n    readonly audioWorklet: AudioWorklet;\n    readonly currentTime: number;\n    readonly destination: AudioDestinationNode;\n    readonly listener: AudioListener;\n    onstatechange: ((this: BaseAudioContext, ev: Event) => any) | null;\n    readonly sampleRate: number;\n    readonly state: AudioContextState;\n    createAnalyser(): AnalyserNode;\n    createBiquadFilter(): BiquadFilterNode;\n    createBuffer(numberOfChannels: number, length: number, sampleRate: number): AudioBuffer;\n    createBufferSource(): AudioBufferSourceNode;\n    createChannelMerger(numberOfInputs?: number): ChannelMergerNode;\n    createChannelSplitter(numberOfOutputs?: number): ChannelSplitterNode;\n    createConstantSource(): ConstantSourceNode;\n    createConvolver(): ConvolverNode;\n    createDelay(maxDelayTime?: number): DelayNode;\n    createDynamicsCompressor(): DynamicsCompressorNode;\n    createGain(): GainNode;\n    createIIRFilter(feedforward: number[], feedback: number[]): IIRFilterNode;\n    createOscillator(): OscillatorNode;\n    createPanner(): PannerNode;\n    createPeriodicWave(real: number[] | Float32Array, imag: number[] | Float32Array, constraints?: PeriodicWaveConstraints): PeriodicWave;\n    /** @deprecated */\n    createScriptProcessor(bufferSize?: number, numberOfInputChannels?: number, numberOfOutputChannels?: number): ScriptProcessorNode;\n    createStereoPanner(): StereoPannerNode;\n    createWaveShaper(): WaveShaperNode;\n    decodeAudioData(audioData: ArrayBuffer, successCallback?: DecodeSuccessCallback | null, errorCallback?: DecodeErrorCallback | null): Promise<AudioBuffer>;\n    addEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: BaseAudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof BaseAudioContextEventMap>(type: K, listener: (this: BaseAudioContext, ev: BaseAudioContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var BaseAudioContext: {\n    prototype: BaseAudioContext;\n    new(): BaseAudioContext;\n};\n\n/** The beforeunload event is fired when the window, the document and its resources are about to be unloaded. */\ninterface BeforeUnloadEvent extends Event {\n    returnValue: any;\n}\n\ndeclare var BeforeUnloadEvent: {\n    prototype: BeforeUnloadEvent;\n    new(): BeforeUnloadEvent;\n};\n\n/** A simple low-order filter, and is created using the AudioContext.createBiquadFilter() method. It is an AudioNode that can represent different kinds of filters, tone control devices, and graphic equalizers. */\ninterface BiquadFilterNode extends AudioNode {\n    readonly Q: AudioParam;\n    readonly detune: AudioParam;\n    readonly frequency: AudioParam;\n    readonly gain: AudioParam;\n    type: BiquadFilterType;\n    getFrequencyResponse(frequencyHz: Float32Array, magResponse: Float32Array, phaseResponse: Float32Array): void;\n}\n\ndeclare var BiquadFilterNode: {\n    prototype: BiquadFilterNode;\n    new(context: BaseAudioContext, options?: BiquadFilterOptions): BiquadFilterNode;\n};\n\n/** A file-like object of immutable, raw data. Blobs represent data that isn\'t necessarily in a JavaScript-native format. The File interface is based on Blob, inheriting blob functionality and expanding it to support files on the user\'s system. */\ninterface Blob {\n    readonly size: number;\n    readonly type: string;\n    arrayBuffer(): Promise<ArrayBuffer>;\n    slice(start?: number, end?: number, contentType?: string): Blob;\n    stream(): ReadableStream;\n    text(): Promise<string>;\n}\n\ndeclare var Blob: {\n    prototype: Blob;\n    new(blobParts?: BlobPart[], options?: BlobPropertyBag): Blob;\n};\n\ninterface BlobEvent extends Event {\n    readonly data: Blob;\n    readonly timecode: DOMHighResTimeStamp;\n}\n\ndeclare var BlobEvent: {\n    prototype: BlobEvent;\n    new(type: string, eventInitDict: BlobEventInit): BlobEvent;\n};\n\ninterface Body {\n    readonly body: ReadableStream<Uint8Array> | null;\n    readonly bodyUsed: boolean;\n    arrayBuffer(): Promise<ArrayBuffer>;\n    blob(): Promise<Blob>;\n    formData(): Promise<FormData>;\n    json(): Promise<any>;\n    text(): Promise<string>;\n}\n\ninterface BroadcastChannelEventMap {\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n}\n\ninterface BroadcastChannel extends EventTarget {\n    /**\n     * Returns the channel name (as passed to the constructor).\n     */\n    readonly name: string;\n    onmessage: ((this: BroadcastChannel, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: BroadcastChannel, ev: MessageEvent) => any) | null;\n    /**\n     * Closes the BroadcastChannel object, opening it up to garbage collection.\n     */\n    close(): void;\n    /**\n     * Sends the given message to other BroadcastChannel objects set up for this channel. Messages can be structured objects, e.g. nested objects and arrays.\n     */\n    postMessage(message: any): void;\n    addEventListener<K extends keyof BroadcastChannelEventMap>(type: K, listener: (this: BroadcastChannel, ev: BroadcastChannelEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof BroadcastChannelEventMap>(type: K, listener: (this: BroadcastChannel, ev: BroadcastChannelEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var BroadcastChannel: {\n    prototype: BroadcastChannel;\n    new(name: string): BroadcastChannel;\n};\n\n/** This Streams API interface provides a built-in byte length queuing strategy that can be used when constructing streams. */\ninterface ByteLengthQueuingStrategy extends QueuingStrategy<ArrayBufferView> {\n    readonly highWaterMark: number;\n    readonly size: QueuingStrategySize<ArrayBufferView>;\n}\n\ndeclare var ByteLengthQueuingStrategy: {\n    prototype: ByteLengthQueuingStrategy;\n    new(init: QueuingStrategyInit): ByteLengthQueuingStrategy;\n};\n\n/** A CDATA section that can be used within XML to include extended portions of unescaped text. The symbols < and & don’t need escaping as they normally do when inside a CDATA section. */\ninterface CDATASection extends Text {\n}\n\ndeclare var CDATASection: {\n    prototype: CDATASection;\n    new(): CDATASection;\n};\n\ninterface CSSAnimation extends Animation {\n    readonly animationName: string;\n    addEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: CSSAnimation, ev: AnimationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: CSSAnimation, ev: AnimationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var CSSAnimation: {\n    prototype: CSSAnimation;\n    new(): CSSAnimation;\n};\n\n/** A single condition CSS at-rule, which consists of a condition and a statement block. It is a child of CSSGroupingRule. */\ninterface CSSConditionRule extends CSSGroupingRule {\n    conditionText: string;\n}\n\ndeclare var CSSConditionRule: {\n    prototype: CSSConditionRule;\n    new(): CSSConditionRule;\n};\n\ninterface CSSCounterStyleRule extends CSSRule {\n    additiveSymbols: string;\n    fallback: string;\n    name: string;\n    negative: string;\n    pad: string;\n    prefix: string;\n    range: string;\n    speakAs: string;\n    suffix: string;\n    symbols: string;\n    system: string;\n}\n\ndeclare var CSSCounterStyleRule: {\n    prototype: CSSCounterStyleRule;\n    new(): CSSCounterStyleRule;\n};\n\ninterface CSSFontFaceRule extends CSSRule {\n    readonly style: CSSStyleDeclaration;\n}\n\ndeclare var CSSFontFaceRule: {\n    prototype: CSSFontFaceRule;\n    new(): CSSFontFaceRule;\n};\n\n/** Any CSS at-rule that contains other rules nested within it. */\ninterface CSSGroupingRule extends CSSRule {\n    readonly cssRules: CSSRuleList;\n    deleteRule(index: number): void;\n    insertRule(rule: string, index?: number): number;\n}\n\ndeclare var CSSGroupingRule: {\n    prototype: CSSGroupingRule;\n    new(): CSSGroupingRule;\n};\n\ninterface CSSImportRule extends CSSRule {\n    readonly href: string;\n    readonly media: MediaList;\n    readonly styleSheet: CSSStyleSheet;\n}\n\ndeclare var CSSImportRule: {\n    prototype: CSSImportRule;\n    new(): CSSImportRule;\n};\n\n/** An object representing a set of style for a given keyframe. It corresponds to the contains of a single keyframe of a @keyframes at-rule. It implements the CSSRule interface with a type value of 8 (CSSRule.KEYFRAME_RULE). */\ninterface CSSKeyframeRule extends CSSRule {\n    keyText: string;\n    readonly style: CSSStyleDeclaration;\n}\n\ndeclare var CSSKeyframeRule: {\n    prototype: CSSKeyframeRule;\n    new(): CSSKeyframeRule;\n};\n\n/** An object representing a complete set of keyframes for a CSS animation. It corresponds to the contains of a whole @keyframes at-rule. It implements the CSSRule interface with a type value of 7 (CSSRule.KEYFRAMES_RULE). */\ninterface CSSKeyframesRule extends CSSRule {\n    readonly cssRules: CSSRuleList;\n    name: string;\n    appendRule(rule: string): void;\n    deleteRule(select: string): void;\n    findRule(select: string): CSSKeyframeRule | null;\n}\n\ndeclare var CSSKeyframesRule: {\n    prototype: CSSKeyframesRule;\n    new(): CSSKeyframesRule;\n};\n\n/** A single CSS @media rule. It implements the CSSConditionRule interface, and therefore the CSSGroupingRule and the CSSRule interface with a type value of 4 (CSSRule.MEDIA_RULE). */\ninterface CSSMediaRule extends CSSConditionRule {\n    readonly media: MediaList;\n}\n\ndeclare var CSSMediaRule: {\n    prototype: CSSMediaRule;\n    new(): CSSMediaRule;\n};\n\n/** An object representing a single CSS @namespace at-rule. It implements the CSSRule interface, with a type value of 10 (CSSRule.NAMESPACE_RULE). */\ninterface CSSNamespaceRule extends CSSRule {\n    readonly namespaceURI: string;\n    readonly prefix: string;\n}\n\ndeclare var CSSNamespaceRule: {\n    prototype: CSSNamespaceRule;\n    new(): CSSNamespaceRule;\n};\n\n/** CSSPageRule is an interface representing a single CSS @page rule. It implements the CSSRule interface with a type value of 6 (CSSRule.PAGE_RULE). */\ninterface CSSPageRule extends CSSGroupingRule {\n    selectorText: string;\n    readonly style: CSSStyleDeclaration;\n}\n\ndeclare var CSSPageRule: {\n    prototype: CSSPageRule;\n    new(): CSSPageRule;\n};\n\n/** A single CSS rule. There are several types of rules, listed in the Type constants section below. */\ninterface CSSRule {\n    cssText: string;\n    readonly parentRule: CSSRule | null;\n    readonly parentStyleSheet: CSSStyleSheet | null;\n    /** @deprecated */\n    readonly type: number;\n    readonly CHARSET_RULE: number;\n    readonly FONT_FACE_RULE: number;\n    readonly IMPORT_RULE: number;\n    readonly KEYFRAMES_RULE: number;\n    readonly KEYFRAME_RULE: number;\n    readonly MEDIA_RULE: number;\n    readonly NAMESPACE_RULE: number;\n    readonly PAGE_RULE: number;\n    readonly STYLE_RULE: number;\n    readonly SUPPORTS_RULE: number;\n}\n\ndeclare var CSSRule: {\n    prototype: CSSRule;\n    new(): CSSRule;\n    readonly CHARSET_RULE: number;\n    readonly FONT_FACE_RULE: number;\n    readonly IMPORT_RULE: number;\n    readonly KEYFRAMES_RULE: number;\n    readonly KEYFRAME_RULE: number;\n    readonly MEDIA_RULE: number;\n    readonly NAMESPACE_RULE: number;\n    readonly PAGE_RULE: number;\n    readonly STYLE_RULE: number;\n    readonly SUPPORTS_RULE: number;\n};\n\n/** A CSSRuleList is an (indirect-modify only) array-like object containing an ordered collection of CSSRule objects. */\ninterface CSSRuleList {\n    readonly length: number;\n    item(index: number): CSSRule | null;\n    [index: number]: CSSRule;\n}\n\ndeclare var CSSRuleList: {\n    prototype: CSSRuleList;\n    new(): CSSRuleList;\n};\n\n/** An object that is a CSS declaration block, and exposes style information and various style-related methods and properties. */\ninterface CSSStyleDeclaration {\n    alignContent: string;\n    alignItems: string;\n    alignSelf: string;\n    alignmentBaseline: string;\n    all: string;\n    animation: string;\n    animationDelay: string;\n    animationDirection: string;\n    animationDuration: string;\n    animationFillMode: string;\n    animationIterationCount: string;\n    animationName: string;\n    animationPlayState: string;\n    animationTimingFunction: string;\n    appearance: string;\n    aspectRatio: string;\n    backfaceVisibility: string;\n    background: string;\n    backgroundAttachment: string;\n    backgroundBlendMode: string;\n    backgroundClip: string;\n    backgroundColor: string;\n    backgroundImage: string;\n    backgroundOrigin: string;\n    backgroundPosition: string;\n    backgroundPositionX: string;\n    backgroundPositionY: string;\n    backgroundRepeat: string;\n    backgroundSize: string;\n    baselineShift: string;\n    blockSize: string;\n    border: string;\n    borderBlock: string;\n    borderBlockColor: string;\n    borderBlockEnd: string;\n    borderBlockEndColor: string;\n    borderBlockEndStyle: string;\n    borderBlockEndWidth: string;\n    borderBlockStart: string;\n    borderBlockStartColor: string;\n    borderBlockStartStyle: string;\n    borderBlockStartWidth: string;\n    borderBlockStyle: string;\n    borderBlockWidth: string;\n    borderBottom: string;\n    borderBottomColor: string;\n    borderBottomLeftRadius: string;\n    borderBottomRightRadius: string;\n    borderBottomStyle: string;\n    borderBottomWidth: string;\n    borderCollapse: string;\n    borderColor: string;\n    borderEndEndRadius: string;\n    borderEndStartRadius: string;\n    borderImage: string;\n    borderImageOutset: string;\n    borderImageRepeat: string;\n    borderImageSlice: string;\n    borderImageSource: string;\n    borderImageWidth: string;\n    borderInline: string;\n    borderInlineColor: string;\n    borderInlineEnd: string;\n    borderInlineEndColor: string;\n    borderInlineEndStyle: string;\n    borderInlineEndWidth: string;\n    borderInlineStart: string;\n    borderInlineStartColor: string;\n    borderInlineStartStyle: string;\n    borderInlineStartWidth: string;\n    borderInlineStyle: string;\n    borderInlineWidth: string;\n    borderLeft: string;\n    borderLeftColor: string;\n    borderLeftStyle: string;\n    borderLeftWidth: string;\n    borderRadius: string;\n    borderRight: string;\n    borderRightColor: string;\n    borderRightStyle: string;\n    borderRightWidth: string;\n    borderSpacing: string;\n    borderStartEndRadius: string;\n    borderStartStartRadius: string;\n    borderStyle: string;\n    borderTop: string;\n    borderTopColor: string;\n    borderTopLeftRadius: string;\n    borderTopRightRadius: string;\n    borderTopStyle: string;\n    borderTopWidth: string;\n    borderWidth: string;\n    bottom: string;\n    boxShadow: string;\n    boxSizing: string;\n    breakAfter: string;\n    breakBefore: string;\n    breakInside: string;\n    captionSide: string;\n    caretColor: string;\n    clear: string;\n    /** @deprecated */\n    clip: string;\n    clipPath: string;\n    clipRule: string;\n    color: string;\n    colorInterpolation: string;\n    colorInterpolationFilters: string;\n    colorScheme: string;\n    columnCount: string;\n    columnFill: string;\n    columnGap: string;\n    columnRule: string;\n    columnRuleColor: string;\n    columnRuleStyle: string;\n    columnRuleWidth: string;\n    columnSpan: string;\n    columnWidth: string;\n    columns: string;\n    contain: string;\n    content: string;\n    counterIncrement: string;\n    counterReset: string;\n    counterSet: string;\n    cssFloat: string;\n    cssText: string;\n    cursor: string;\n    direction: string;\n    display: string;\n    dominantBaseline: string;\n    emptyCells: string;\n    fill: string;\n    fillOpacity: string;\n    fillRule: string;\n    filter: string;\n    flex: string;\n    flexBasis: string;\n    flexDirection: string;\n    flexFlow: string;\n    flexGrow: string;\n    flexShrink: string;\n    flexWrap: string;\n    float: string;\n    floodColor: string;\n    floodOpacity: string;\n    font: string;\n    fontFamily: string;\n    fontFeatureSettings: string;\n    fontKerning: string;\n    fontOpticalSizing: string;\n    fontSize: string;\n    fontSizeAdjust: string;\n    fontStretch: string;\n    fontStyle: string;\n    fontSynthesis: string;\n    fontVariant: string;\n    /** @deprecated */\n    fontVariantAlternates: string;\n    fontVariantCaps: string;\n    fontVariantEastAsian: string;\n    fontVariantLigatures: string;\n    fontVariantNumeric: string;\n    fontVariantPosition: string;\n    fontVariationSettings: string;\n    fontWeight: string;\n    gap: string;\n    grid: string;\n    gridArea: string;\n    gridAutoColumns: string;\n    gridAutoFlow: string;\n    gridAutoRows: string;\n    gridColumn: string;\n    gridColumnEnd: string;\n    gridColumnGap: string;\n    gridColumnStart: string;\n    gridGap: string;\n    gridRow: string;\n    gridRowEnd: string;\n    gridRowGap: string;\n    gridRowStart: string;\n    gridTemplate: string;\n    gridTemplateAreas: string;\n    gridTemplateColumns: string;\n    gridTemplateRows: string;\n    height: string;\n    hyphens: string;\n    /** @deprecated */\n    imageOrientation: string;\n    imageRendering: string;\n    inlineSize: string;\n    inset: string;\n    insetBlock: string;\n    insetBlockEnd: string;\n    insetBlockStart: string;\n    insetInline: string;\n    insetInlineEnd: string;\n    insetInlineStart: string;\n    isolation: string;\n    justifyContent: string;\n    justifyItems: string;\n    justifySelf: string;\n    left: string;\n    readonly length: number;\n    letterSpacing: string;\n    lightingColor: string;\n    lineBreak: string;\n    lineHeight: string;\n    listStyle: string;\n    listStyleImage: string;\n    listStylePosition: string;\n    listStyleType: string;\n    margin: string;\n    marginBlock: string;\n    marginBlockEnd: string;\n    marginBlockStart: string;\n    marginBottom: string;\n    marginInline: string;\n    marginInlineEnd: string;\n    marginInlineStart: string;\n    marginLeft: string;\n    marginRight: string;\n    marginTop: string;\n    marker: string;\n    markerEnd: string;\n    markerMid: string;\n    markerStart: string;\n    mask: string;\n    maskType: string;\n    maxBlockSize: string;\n    maxHeight: string;\n    maxInlineSize: string;\n    maxWidth: string;\n    minBlockSize: string;\n    minHeight: string;\n    minInlineSize: string;\n    minWidth: string;\n    mixBlendMode: string;\n    objectFit: string;\n    objectPosition: string;\n    offset: string;\n    offsetAnchor: string;\n    offsetDistance: string;\n    offsetPath: string;\n    offsetRotate: string;\n    opacity: string;\n    order: string;\n    orphans: string;\n    outline: string;\n    outlineColor: string;\n    outlineOffset: string;\n    outlineStyle: string;\n    outlineWidth: string;\n    overflow: string;\n    overflowAnchor: string;\n    overflowWrap: string;\n    overflowX: string;\n    overflowY: string;\n    overscrollBehavior: string;\n    overscrollBehaviorBlock: string;\n    overscrollBehaviorInline: string;\n    overscrollBehaviorX: string;\n    overscrollBehaviorY: string;\n    padding: string;\n    paddingBlock: string;\n    paddingBlockEnd: string;\n    paddingBlockStart: string;\n    paddingBottom: string;\n    paddingInline: string;\n    paddingInlineEnd: string;\n    paddingInlineStart: string;\n    paddingLeft: string;\n    paddingRight: string;\n    paddingTop: string;\n    pageBreakAfter: string;\n    pageBreakBefore: string;\n    pageBreakInside: string;\n    paintOrder: string;\n    readonly parentRule: CSSRule | null;\n    perspective: string;\n    perspectiveOrigin: string;\n    placeContent: string;\n    placeItems: string;\n    placeSelf: string;\n    pointerEvents: string;\n    position: string;\n    quotes: string;\n    resize: string;\n    right: string;\n    rotate: string;\n    rowGap: string;\n    rubyPosition: string;\n    scale: string;\n    scrollBehavior: string;\n    scrollMargin: string;\n    scrollMarginBlock: string;\n    scrollMarginBlockEnd: string;\n    scrollMarginBlockStart: string;\n    scrollMarginBottom: string;\n    scrollMarginInline: string;\n    scrollMarginInlineEnd: string;\n    scrollMarginInlineStart: string;\n    scrollMarginLeft: string;\n    scrollMarginRight: string;\n    scrollMarginTop: string;\n    scrollPadding: string;\n    scrollPaddingBlock: string;\n    scrollPaddingBlockEnd: string;\n    scrollPaddingBlockStart: string;\n    scrollPaddingBottom: string;\n    scrollPaddingInline: string;\n    scrollPaddingInlineEnd: string;\n    scrollPaddingInlineStart: string;\n    scrollPaddingLeft: string;\n    scrollPaddingRight: string;\n    scrollPaddingTop: string;\n    scrollSnapAlign: string;\n    scrollSnapStop: string;\n    scrollSnapType: string;\n    shapeImageThreshold: string;\n    shapeMargin: string;\n    shapeOutside: string;\n    shapeRendering: string;\n    stopColor: string;\n    stopOpacity: string;\n    stroke: string;\n    strokeDasharray: string;\n    strokeDashoffset: string;\n    strokeLinecap: string;\n    strokeLinejoin: string;\n    strokeMiterlimit: string;\n    strokeOpacity: string;\n    strokeWidth: string;\n    tabSize: string;\n    tableLayout: string;\n    textAlign: string;\n    textAlignLast: string;\n    textAnchor: string;\n    textCombineUpright: string;\n    textDecoration: string;\n    textDecorationColor: string;\n    textDecorationLine: string;\n    textDecorationSkipInk: string;\n    textDecorationStyle: string;\n    textDecorationThickness: string;\n    textEmphasis: string;\n    textEmphasisColor: string;\n    textEmphasisPosition: string;\n    textEmphasisStyle: string;\n    textIndent: string;\n    textOrientation: string;\n    textOverflow: string;\n    textRendering: string;\n    textShadow: string;\n    textTransform: string;\n    textUnderlineOffset: string;\n    textUnderlinePosition: string;\n    top: string;\n    touchAction: string;\n    transform: string;\n    transformBox: string;\n    transformOrigin: string;\n    transformStyle: string;\n    transition: string;\n    transitionDelay: string;\n    transitionDuration: string;\n    transitionProperty: string;\n    transitionTimingFunction: string;\n    translate: string;\n    unicodeBidi: string;\n    userSelect: string;\n    verticalAlign: string;\n    visibility: string;\n    /** @deprecated This is a legacy alias of `alignContent`. */\n    webkitAlignContent: string;\n    /** @deprecated This is a legacy alias of `alignItems`. */\n    webkitAlignItems: string;\n    /** @deprecated This is a legacy alias of `alignSelf`. */\n    webkitAlignSelf: string;\n    /** @deprecated This is a legacy alias of `animation`. */\n    webkitAnimation: string;\n    /** @deprecated This is a legacy alias of `animationDelay`. */\n    webkitAnimationDelay: string;\n    /** @deprecated This is a legacy alias of `animationDirection`. */\n    webkitAnimationDirection: string;\n    /** @deprecated This is a legacy alias of `animationDuration`. */\n    webkitAnimationDuration: string;\n    /** @deprecated This is a legacy alias of `animationFillMode`. */\n    webkitAnimationFillMode: string;\n    /** @deprecated This is a legacy alias of `animationIterationCount`. */\n    webkitAnimationIterationCount: string;\n    /** @deprecated This is a legacy alias of `animationName`. */\n    webkitAnimationName: string;\n    /** @deprecated This is a legacy alias of `animationPlayState`. */\n    webkitAnimationPlayState: string;\n    /** @deprecated This is a legacy alias of `animationTimingFunction`. */\n    webkitAnimationTimingFunction: string;\n    /** @deprecated This is a legacy alias of `appearance`. */\n    webkitAppearance: string;\n    /** @deprecated This is a legacy alias of `backfaceVisibility`. */\n    webkitBackfaceVisibility: string;\n    /** @deprecated This is a legacy alias of `backgroundClip`. */\n    webkitBackgroundClip: string;\n    /** @deprecated This is a legacy alias of `backgroundOrigin`. */\n    webkitBackgroundOrigin: string;\n    /** @deprecated This is a legacy alias of `backgroundSize`. */\n    webkitBackgroundSize: string;\n    /** @deprecated This is a legacy alias of `borderBottomLeftRadius`. */\n    webkitBorderBottomLeftRadius: string;\n    /** @deprecated This is a legacy alias of `borderBottomRightRadius`. */\n    webkitBorderBottomRightRadius: string;\n    /** @deprecated This is a legacy alias of `borderRadius`. */\n    webkitBorderRadius: string;\n    /** @deprecated This is a legacy alias of `borderTopLeftRadius`. */\n    webkitBorderTopLeftRadius: string;\n    /** @deprecated This is a legacy alias of `borderTopRightRadius`. */\n    webkitBorderTopRightRadius: string;\n    /** @deprecated */\n    webkitBoxAlign: string;\n    /** @deprecated */\n    webkitBoxFlex: string;\n    /** @deprecated */\n    webkitBoxOrdinalGroup: string;\n    /** @deprecated */\n    webkitBoxOrient: string;\n    /** @deprecated */\n    webkitBoxPack: string;\n    /** @deprecated This is a legacy alias of `boxShadow`. */\n    webkitBoxShadow: string;\n    /** @deprecated This is a legacy alias of `boxSizing`. */\n    webkitBoxSizing: string;\n    /** @deprecated This is a legacy alias of `filter`. */\n    webkitFilter: string;\n    /** @deprecated This is a legacy alias of `flex`. */\n    webkitFlex: string;\n    /** @deprecated This is a legacy alias of `flexBasis`. */\n    webkitFlexBasis: string;\n    /** @deprecated This is a legacy alias of `flexDirection`. */\n    webkitFlexDirection: string;\n    /** @deprecated This is a legacy alias of `flexFlow`. */\n    webkitFlexFlow: string;\n    /** @deprecated This is a legacy alias of `flexGrow`. */\n    webkitFlexGrow: string;\n    /** @deprecated This is a legacy alias of `flexShrink`. */\n    webkitFlexShrink: string;\n    /** @deprecated This is a legacy alias of `flexWrap`. */\n    webkitFlexWrap: string;\n    /** @deprecated This is a legacy alias of `justifyContent`. */\n    webkitJustifyContent: string;\n    webkitLineClamp: string;\n    /** @deprecated This is a legacy alias of `mask`. */\n    webkitMask: string;\n    /** @deprecated This is a legacy alias of `maskBorder`. */\n    webkitMaskBoxImage: string;\n    /** @deprecated This is a legacy alias of `maskBorderOutset`. */\n    webkitMaskBoxImageOutset: string;\n    /** @deprecated This is a legacy alias of `maskBorderRepeat`. */\n    webkitMaskBoxImageRepeat: string;\n    /** @deprecated This is a legacy alias of `maskBorderSlice`. */\n    webkitMaskBoxImageSlice: string;\n    /** @deprecated This is a legacy alias of `maskBorderSource`. */\n    webkitMaskBoxImageSource: string;\n    /** @deprecated This is a legacy alias of `maskBorderWidth`. */\n    webkitMaskBoxImageWidth: string;\n    /** @deprecated This is a legacy alias of `maskClip`. */\n    webkitMaskClip: string;\n    webkitMaskComposite: string;\n    /** @deprecated This is a legacy alias of `maskImage`. */\n    webkitMaskImage: string;\n    /** @deprecated This is a legacy alias of `maskOrigin`. */\n    webkitMaskOrigin: string;\n    /** @deprecated This is a legacy alias of `maskPosition`. */\n    webkitMaskPosition: string;\n    /** @deprecated This is a legacy alias of `maskRepeat`. */\n    webkitMaskRepeat: string;\n    /** @deprecated This is a legacy alias of `maskSize`. */\n    webkitMaskSize: string;\n    /** @deprecated This is a legacy alias of `order`. */\n    webkitOrder: string;\n    /** @deprecated This is a legacy alias of `perspective`. */\n    webkitPerspective: string;\n    /** @deprecated This is a legacy alias of `perspectiveOrigin`. */\n    webkitPerspectiveOrigin: string;\n    webkitTextFillColor: string;\n    webkitTextStroke: string;\n    webkitTextStrokeColor: string;\n    webkitTextStrokeWidth: string;\n    /** @deprecated This is a legacy alias of `transform`. */\n    webkitTransform: string;\n    /** @deprecated This is a legacy alias of `transformOrigin`. */\n    webkitTransformOrigin: string;\n    /** @deprecated This is a legacy alias of `transformStyle`. */\n    webkitTransformStyle: string;\n    /** @deprecated This is a legacy alias of `transition`. */\n    webkitTransition: string;\n    /** @deprecated This is a legacy alias of `transitionDelay`. */\n    webkitTransitionDelay: string;\n    /** @deprecated This is a legacy alias of `transitionDuration`. */\n    webkitTransitionDuration: string;\n    /** @deprecated This is a legacy alias of `transitionProperty`. */\n    webkitTransitionProperty: string;\n    /** @deprecated This is a legacy alias of `transitionTimingFunction`. */\n    webkitTransitionTimingFunction: string;\n    /** @deprecated This is a legacy alias of `userSelect`. */\n    webkitUserSelect: string;\n    whiteSpace: string;\n    widows: string;\n    width: string;\n    willChange: string;\n    wordBreak: string;\n    wordSpacing: string;\n    /** @deprecated */\n    wordWrap: string;\n    writingMode: string;\n    zIndex: string;\n    getPropertyPriority(property: string): string;\n    getPropertyValue(property: string): string;\n    item(index: number): string;\n    removeProperty(property: string): string;\n    setProperty(property: string, value: string | null, priority?: string): void;\n    [index: number]: string;\n}\n\ndeclare var CSSStyleDeclaration: {\n    prototype: CSSStyleDeclaration;\n    new(): CSSStyleDeclaration;\n};\n\n/** CSSStyleRule represents a single CSS style rule. It implements the CSSRule interface with a type value of 1 (CSSRule.STYLE_RULE). */\ninterface CSSStyleRule extends CSSRule {\n    selectorText: string;\n    readonly style: CSSStyleDeclaration;\n}\n\ndeclare var CSSStyleRule: {\n    prototype: CSSStyleRule;\n    new(): CSSStyleRule;\n};\n\n/** A single CSS style sheet. It inherits properties and methods from its parent, StyleSheet. */\ninterface CSSStyleSheet extends StyleSheet {\n    readonly cssRules: CSSRuleList;\n    readonly ownerRule: CSSRule | null;\n    /** @deprecated */\n    readonly rules: CSSRuleList;\n    /** @deprecated */\n    addRule(selector?: string, style?: string, index?: number): number;\n    deleteRule(index: number): void;\n    insertRule(rule: string, index?: number): number;\n    /** @deprecated */\n    removeRule(index?: number): void;\n}\n\ndeclare var CSSStyleSheet: {\n    prototype: CSSStyleSheet;\n    new(options?: CSSStyleSheetInit): CSSStyleSheet;\n};\n\n/** An object representing a single CSS @supports at-rule. It implements the CSSConditionRule interface, and therefore the CSSRule and CSSGroupingRule interfaces with a type value of 12 (CSSRule.SUPPORTS_RULE). */\ninterface CSSSupportsRule extends CSSConditionRule {\n}\n\ndeclare var CSSSupportsRule: {\n    prototype: CSSSupportsRule;\n    new(): CSSSupportsRule;\n};\n\ninterface CSSTransition extends Animation {\n    readonly transitionProperty: string;\n    addEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: CSSTransition, ev: AnimationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AnimationEventMap>(type: K, listener: (this: CSSTransition, ev: AnimationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var CSSTransition: {\n    prototype: CSSTransition;\n    new(): CSSTransition;\n};\n\n/** Provides a storage mechanism for Request / Response object pairs that are cached, for example as part of the ServiceWorker life cycle. Note that the Cache interface is exposed to windowed scopes as well as workers. You don\'t have to use it in conjunction with service workers, even though it is defined in the service worker spec. */\ninterface Cache {\n    add(request: RequestInfo): Promise<void>;\n    addAll(requests: RequestInfo[]): Promise<void>;\n    delete(request: RequestInfo, options?: CacheQueryOptions): Promise<boolean>;\n    keys(request?: RequestInfo, options?: CacheQueryOptions): Promise<ReadonlyArray<Request>>;\n    match(request: RequestInfo, options?: CacheQueryOptions): Promise<Response | undefined>;\n    matchAll(request?: RequestInfo, options?: CacheQueryOptions): Promise<ReadonlyArray<Response>>;\n    put(request: RequestInfo, response: Response): Promise<void>;\n}\n\ndeclare var Cache: {\n    prototype: Cache;\n    new(): Cache;\n};\n\n/** The storage for Cache objects. */\ninterface CacheStorage {\n    delete(cacheName: string): Promise<boolean>;\n    has(cacheName: string): Promise<boolean>;\n    keys(): Promise<string[]>;\n    match(request: RequestInfo, options?: MultiCacheQueryOptions): Promise<Response | undefined>;\n    open(cacheName: string): Promise<Cache>;\n}\n\ndeclare var CacheStorage: {\n    prototype: CacheStorage;\n    new(): CacheStorage;\n};\n\ninterface CanvasCompositing {\n    globalAlpha: number;\n    globalCompositeOperation: string;\n}\n\ninterface CanvasDrawImage {\n    drawImage(image: CanvasImageSource, dx: number, dy: number): void;\n    drawImage(image: CanvasImageSource, dx: number, dy: number, dw: number, dh: number): void;\n    drawImage(image: CanvasImageSource, sx: number, sy: number, sw: number, sh: number, dx: number, dy: number, dw: number, dh: number): void;\n}\n\ninterface CanvasDrawPath {\n    beginPath(): void;\n    clip(fillRule?: CanvasFillRule): void;\n    clip(path: Path2D, fillRule?: CanvasFillRule): void;\n    fill(fillRule?: CanvasFillRule): void;\n    fill(path: Path2D, fillRule?: CanvasFillRule): void;\n    isPointInPath(x: number, y: number, fillRule?: CanvasFillRule): boolean;\n    isPointInPath(path: Path2D, x: number, y: number, fillRule?: CanvasFillRule): boolean;\n    isPointInStroke(x: number, y: number): boolean;\n    isPointInStroke(path: Path2D, x: number, y: number): boolean;\n    stroke(): void;\n    stroke(path: Path2D): void;\n}\n\ninterface CanvasFillStrokeStyles {\n    fillStyle: string | CanvasGradient | CanvasPattern;\n    strokeStyle: string | CanvasGradient | CanvasPattern;\n    createLinearGradient(x0: number, y0: number, x1: number, y1: number): CanvasGradient;\n    createPattern(image: CanvasImageSource, repetition: string | null): CanvasPattern | null;\n    createRadialGradient(x0: number, y0: number, r0: number, x1: number, y1: number, r1: number): CanvasGradient;\n}\n\ninterface CanvasFilters {\n    filter: string;\n}\n\n/** An opaque object describing a gradient. It is returned by the methods CanvasRenderingContext2D.createLinearGradient() or CanvasRenderingContext2D.createRadialGradient(). */\ninterface CanvasGradient {\n    /**\n     * Adds a color stop with the given color to the gradient at the given offset. 0.0 is the offset at one end of the gradient, 1.0 is the offset at the other end.\n     *\n     * Throws an "IndexSizeError" DOMException if the offset is out of range. Throws a "SyntaxError" DOMException if the color cannot be parsed.\n     */\n    addColorStop(offset: number, color: string): void;\n}\n\ndeclare var CanvasGradient: {\n    prototype: CanvasGradient;\n    new(): CanvasGradient;\n};\n\ninterface CanvasImageData {\n    createImageData(sw: number, sh: number, settings?: ImageDataSettings): ImageData;\n    createImageData(imagedata: ImageData): ImageData;\n    getImageData(sx: number, sy: number, sw: number, sh: number, settings?: ImageDataSettings): ImageData;\n    putImageData(imagedata: ImageData, dx: number, dy: number): void;\n    putImageData(imagedata: ImageData, dx: number, dy: number, dirtyX: number, dirtyY: number, dirtyWidth: number, dirtyHeight: number): void;\n}\n\ninterface CanvasImageSmoothing {\n    imageSmoothingEnabled: boolean;\n    imageSmoothingQuality: ImageSmoothingQuality;\n}\n\ninterface CanvasPath {\n    arc(x: number, y: number, radius: number, startAngle: number, endAngle: number, counterclockwise?: boolean): void;\n    arcTo(x1: number, y1: number, x2: number, y2: number, radius: number): void;\n    bezierCurveTo(cp1x: number, cp1y: number, cp2x: number, cp2y: number, x: number, y: number): void;\n    closePath(): void;\n    ellipse(x: number, y: number, radiusX: number, radiusY: number, rotation: number, startAngle: number, endAngle: number, counterclockwise?: boolean): void;\n    lineTo(x: number, y: number): void;\n    moveTo(x: number, y: number): void;\n    quadraticCurveTo(cpx: number, cpy: number, x: number, y: number): void;\n    rect(x: number, y: number, w: number, h: number): void;\n}\n\ninterface CanvasPathDrawingStyles {\n    lineCap: CanvasLineCap;\n    lineDashOffset: number;\n    lineJoin: CanvasLineJoin;\n    lineWidth: number;\n    miterLimit: number;\n    getLineDash(): number[];\n    setLineDash(segments: number[]): void;\n}\n\n/** An opaque object describing a pattern, based on an image, a canvas, or a video, created by the CanvasRenderingContext2D.createPattern() method. */\ninterface CanvasPattern {\n    /**\n     * Sets the transformation matrix that will be used when rendering the pattern during a fill or stroke painting operation.\n     */\n    setTransform(transform?: DOMMatrix2DInit): void;\n}\n\ndeclare var CanvasPattern: {\n    prototype: CanvasPattern;\n    new(): CanvasPattern;\n};\n\ninterface CanvasRect {\n    clearRect(x: number, y: number, w: number, h: number): void;\n    fillRect(x: number, y: number, w: number, h: number): void;\n    strokeRect(x: number, y: number, w: number, h: number): void;\n}\n\n/** The CanvasRenderingContext2D interface, part of the Canvas API, provides the 2D rendering context for the drawing surface of a <canvas> element. It is used for drawing shapes, text, images, and other objects. */\ninterface CanvasRenderingContext2D extends CanvasCompositing, CanvasDrawImage, CanvasDrawPath, CanvasFillStrokeStyles, CanvasFilters, CanvasImageData, CanvasImageSmoothing, CanvasPath, CanvasPathDrawingStyles, CanvasRect, CanvasShadowStyles, CanvasState, CanvasText, CanvasTextDrawingStyles, CanvasTransform, CanvasUserInterface {\n    readonly canvas: HTMLCanvasElement;\n    getContextAttributes(): CanvasRenderingContext2DSettings;\n}\n\ndeclare var CanvasRenderingContext2D: {\n    prototype: CanvasRenderingContext2D;\n    new(): CanvasRenderingContext2D;\n};\n\ninterface CanvasShadowStyles {\n    shadowBlur: number;\n    shadowColor: string;\n    shadowOffsetX: number;\n    shadowOffsetY: number;\n}\n\ninterface CanvasState {\n    restore(): void;\n    save(): void;\n}\n\ninterface CanvasText {\n    fillText(text: string, x: number, y: number, maxWidth?: number): void;\n    measureText(text: string): TextMetrics;\n    strokeText(text: string, x: number, y: number, maxWidth?: number): void;\n}\n\ninterface CanvasTextDrawingStyles {\n    direction: CanvasDirection;\n    font: string;\n    textAlign: CanvasTextAlign;\n    textBaseline: CanvasTextBaseline;\n}\n\ninterface CanvasTransform {\n    getTransform(): DOMMatrix;\n    resetTransform(): void;\n    rotate(angle: number): void;\n    scale(x: number, y: number): void;\n    setTransform(a: number, b: number, c: number, d: number, e: number, f: number): void;\n    setTransform(transform?: DOMMatrix2DInit): void;\n    transform(a: number, b: number, c: number, d: number, e: number, f: number): void;\n    translate(x: number, y: number): void;\n}\n\ninterface CanvasUserInterface {\n    drawFocusIfNeeded(element: Element): void;\n    drawFocusIfNeeded(path: Path2D, element: Element): void;\n}\n\n/** The ChannelMergerNode interface, often used in conjunction with its opposite, ChannelSplitterNode, reunites different mono inputs into a single output. Each input is used to fill a channel of the output. This is useful for accessing each channels separately, e.g. for performing channel mixing where gain must be separately controlled on each channel. */\ninterface ChannelMergerNode extends AudioNode {\n}\n\ndeclare var ChannelMergerNode: {\n    prototype: ChannelMergerNode;\n    new(context: BaseAudioContext, options?: ChannelMergerOptions): ChannelMergerNode;\n};\n\n/** The ChannelSplitterNode interface, often used in conjunction with its opposite, ChannelMergerNode, separates the different channels of an audio source into a set of mono outputs. This is useful for accessing each channel separately, e.g. for performing channel mixing where gain must be separately controlled on each channel. */\ninterface ChannelSplitterNode extends AudioNode {\n}\n\ndeclare var ChannelSplitterNode: {\n    prototype: ChannelSplitterNode;\n    new(context: BaseAudioContext, options?: ChannelSplitterOptions): ChannelSplitterNode;\n};\n\n/** The CharacterData abstract interface represents a Node object that contains characters. This is an abstract interface, meaning there aren\'t any object of type CharacterData: it is implemented by other interfaces, like Text, Comment, or ProcessingInstruction which aren\'t abstract. */\ninterface CharacterData extends Node, ChildNode, NonDocumentTypeChildNode {\n    data: string;\n    readonly length: number;\n    readonly ownerDocument: Document;\n    appendData(data: string): void;\n    deleteData(offset: number, count: number): void;\n    insertData(offset: number, data: string): void;\n    replaceData(offset: number, count: number, data: string): void;\n    substringData(offset: number, count: number): string;\n}\n\ndeclare var CharacterData: {\n    prototype: CharacterData;\n    new(): CharacterData;\n};\n\ninterface ChildNode extends Node {\n    /**\n     * Inserts nodes just after node, while replacing strings in nodes with equivalent Text nodes.\n     *\n     * Throws a "HierarchyRequestError" DOMException if the constraints of the node tree are violated.\n     */\n    after(...nodes: (Node | string)[]): void;\n    /**\n     * Inserts nodes just before node, while replacing strings in nodes with equivalent Text nodes.\n     *\n     * Throws a "HierarchyRequestError" DOMException if the constraints of the node tree are violated.\n     */\n    before(...nodes: (Node | string)[]): void;\n    /**\n     * Removes node.\n     */\n    remove(): void;\n    /**\n     * Replaces node with nodes, while replacing strings in nodes with equivalent Text nodes.\n     *\n     * Throws a "HierarchyRequestError" DOMException if the constraints of the node tree are violated.\n     */\n    replaceWith(...nodes: (Node | string)[]): void;\n}\n\n/** @deprecated */\ninterface ClientRect extends DOMRect {\n}\n\ninterface Clipboard extends EventTarget {\n    read(): Promise<ClipboardItems>;\n    readText(): Promise<string>;\n    write(data: ClipboardItems): Promise<void>;\n    writeText(data: string): Promise<void>;\n}\n\ndeclare var Clipboard: {\n    prototype: Clipboard;\n    new(): Clipboard;\n};\n\n/** Events providing information related to modification of the clipboard, that is cut, copy, and paste events. */\ninterface ClipboardEvent extends Event {\n    readonly clipboardData: DataTransfer | null;\n}\n\ndeclare var ClipboardEvent: {\n    prototype: ClipboardEvent;\n    new(type: string, eventInitDict?: ClipboardEventInit): ClipboardEvent;\n};\n\ninterface ClipboardItem {\n    readonly types: ReadonlyArray<string>;\n    getType(type: string): Promise<Blob>;\n}\n\ndeclare var ClipboardItem: {\n    prototype: ClipboardItem;\n    new(items: Record<string, ClipboardItemData>, options?: ClipboardItemOptions): ClipboardItem;\n};\n\n/** A CloseEvent is sent to clients using WebSockets when the connection is closed. This is delivered to the listener indicated by the WebSocket object\'s onclose attribute. */\ninterface CloseEvent extends Event {\n    /**\n     * Returns the WebSocket connection close code provided by the server.\n     */\n    readonly code: number;\n    /**\n     * Returns the WebSocket connection close reason provided by the server.\n     */\n    readonly reason: string;\n    /**\n     * Returns true if the connection closed cleanly; false otherwise.\n     */\n    readonly wasClean: boolean;\n}\n\ndeclare var CloseEvent: {\n    prototype: CloseEvent;\n    new(type: string, eventInitDict?: CloseEventInit): CloseEvent;\n};\n\n/** Textual notations within markup; although it is generally not visually shown, such comments are available to be read in the source view. */\ninterface Comment extends CharacterData {\n}\n\ndeclare var Comment: {\n    prototype: Comment;\n    new(data?: string): Comment;\n};\n\n/** The DOM CompositionEvent represents events that occur due to the user indirectly entering text. */\ninterface CompositionEvent extends UIEvent {\n    readonly data: string;\n    /** @deprecated */\n    initCompositionEvent(typeArg: string, bubblesArg?: boolean, cancelableArg?: boolean, viewArg?: WindowProxy | null, dataArg?: string): void;\n}\n\ndeclare var CompositionEvent: {\n    prototype: CompositionEvent;\n    new(type: string, eventInitDict?: CompositionEventInit): CompositionEvent;\n};\n\ninterface ConstantSourceNode extends AudioScheduledSourceNode {\n    readonly offset: AudioParam;\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: ConstantSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: ConstantSourceNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var ConstantSourceNode: {\n    prototype: ConstantSourceNode;\n    new(context: BaseAudioContext, options?: ConstantSourceOptions): ConstantSourceNode;\n};\n\n/** An AudioNode that performs a Linear Convolution on a given AudioBuffer, often used to achieve a reverb effect. A ConvolverNode always has exactly one input and one output. */\ninterface ConvolverNode extends AudioNode {\n    buffer: AudioBuffer | null;\n    normalize: boolean;\n}\n\ndeclare var ConvolverNode: {\n    prototype: ConvolverNode;\n    new(context: BaseAudioContext, options?: ConvolverOptions): ConvolverNode;\n};\n\n/** This Streams API interface provides a built-in byte length queuing strategy that can be used when constructing streams. */\ninterface CountQueuingStrategy extends QueuingStrategy {\n    readonly highWaterMark: number;\n    readonly size: QueuingStrategySize;\n}\n\ndeclare var CountQueuingStrategy: {\n    prototype: CountQueuingStrategy;\n    new(init: QueuingStrategyInit): CountQueuingStrategy;\n};\n\ninterface Credential {\n    readonly id: string;\n    readonly type: string;\n}\n\ndeclare var Credential: {\n    prototype: Credential;\n    new(): Credential;\n};\n\ninterface CredentialsContainer {\n    create(options?: CredentialCreationOptions): Promise<Credential | null>;\n    get(options?: CredentialRequestOptions): Promise<Credential | null>;\n    preventSilentAccess(): Promise<void>;\n    store(credential: Credential): Promise<Credential>;\n}\n\ndeclare var CredentialsContainer: {\n    prototype: CredentialsContainer;\n    new(): CredentialsContainer;\n};\n\n/** Basic cryptography features available in the current context. It allows access to a cryptographically strong random number generator and to cryptographic primitives. */\ninterface Crypto {\n    readonly subtle: SubtleCrypto;\n    getRandomValues<T extends ArrayBufferView | null>(array: T): T;\n}\n\ndeclare var Crypto: {\n    prototype: Crypto;\n    new(): Crypto;\n};\n\n/** The CryptoKey dictionary of the Web Crypto API represents a cryptographic key. */\ninterface CryptoKey {\n    readonly algorithm: KeyAlgorithm;\n    readonly extractable: boolean;\n    readonly type: KeyType;\n    readonly usages: KeyUsage[];\n}\n\ndeclare var CryptoKey: {\n    prototype: CryptoKey;\n    new(): CryptoKey;\n};\n\ninterface CustomElementRegistry {\n    define(name: string, constructor: CustomElementConstructor, options?: ElementDefinitionOptions): void;\n    get(name: string): CustomElementConstructor | undefined;\n    upgrade(root: Node): void;\n    whenDefined(name: string): Promise<CustomElementConstructor>;\n}\n\ndeclare var CustomElementRegistry: {\n    prototype: CustomElementRegistry;\n    new(): CustomElementRegistry;\n};\n\ninterface CustomEvent<T = any> extends Event {\n    /**\n     * Returns any custom data event was created with. Typically used for synthetic events.\n     */\n    readonly detail: T;\n    /** @deprecated */\n    initCustomEvent(type: string, bubbles?: boolean, cancelable?: boolean, detail?: T): void;\n}\n\ndeclare var CustomEvent: {\n    prototype: CustomEvent;\n    new<T>(type: string, eventInitDict?: CustomEventInit<T>): CustomEvent<T>;\n};\n\n/** An abnormal event (called an exception) which occurs as a result of calling a method or accessing a property of a web API. */\ninterface DOMException {\n    readonly code: number;\n    readonly message: string;\n    readonly name: string;\n    readonly ABORT_ERR: number;\n    readonly DATA_CLONE_ERR: number;\n    readonly DOMSTRING_SIZE_ERR: number;\n    readonly HIERARCHY_REQUEST_ERR: number;\n    readonly INDEX_SIZE_ERR: number;\n    readonly INUSE_ATTRIBUTE_ERR: number;\n    readonly INVALID_ACCESS_ERR: number;\n    readonly INVALID_CHARACTER_ERR: number;\n    readonly INVALID_MODIFICATION_ERR: number;\n    readonly INVALID_NODE_TYPE_ERR: number;\n    readonly INVALID_STATE_ERR: number;\n    readonly NAMESPACE_ERR: number;\n    readonly NETWORK_ERR: number;\n    readonly NOT_FOUND_ERR: number;\n    readonly NOT_SUPPORTED_ERR: number;\n    readonly NO_DATA_ALLOWED_ERR: number;\n    readonly NO_MODIFICATION_ALLOWED_ERR: number;\n    readonly QUOTA_EXCEEDED_ERR: number;\n    readonly SECURITY_ERR: number;\n    readonly SYNTAX_ERR: number;\n    readonly TIMEOUT_ERR: number;\n    readonly TYPE_MISMATCH_ERR: number;\n    readonly URL_MISMATCH_ERR: number;\n    readonly VALIDATION_ERR: number;\n    readonly WRONG_DOCUMENT_ERR: number;\n}\n\ndeclare var DOMException: {\n    prototype: DOMException;\n    new(message?: string, name?: string): DOMException;\n    readonly ABORT_ERR: number;\n    readonly DATA_CLONE_ERR: number;\n    readonly DOMSTRING_SIZE_ERR: number;\n    readonly HIERARCHY_REQUEST_ERR: number;\n    readonly INDEX_SIZE_ERR: number;\n    readonly INUSE_ATTRIBUTE_ERR: number;\n    readonly INVALID_ACCESS_ERR: number;\n    readonly INVALID_CHARACTER_ERR: number;\n    readonly INVALID_MODIFICATION_ERR: number;\n    readonly INVALID_NODE_TYPE_ERR: number;\n    readonly INVALID_STATE_ERR: number;\n    readonly NAMESPACE_ERR: number;\n    readonly NETWORK_ERR: number;\n    readonly NOT_FOUND_ERR: number;\n    readonly NOT_SUPPORTED_ERR: number;\n    readonly NO_DATA_ALLOWED_ERR: number;\n    readonly NO_MODIFICATION_ALLOWED_ERR: number;\n    readonly QUOTA_EXCEEDED_ERR: number;\n    readonly SECURITY_ERR: number;\n    readonly SYNTAX_ERR: number;\n    readonly TIMEOUT_ERR: number;\n    readonly TYPE_MISMATCH_ERR: number;\n    readonly URL_MISMATCH_ERR: number;\n    readonly VALIDATION_ERR: number;\n    readonly WRONG_DOCUMENT_ERR: number;\n};\n\n/** An object providing methods which are not dependent on any particular document. Such an object is returned by the Document.implementation property. */\ninterface DOMImplementation {\n    createDocument(namespace: string | null, qualifiedName: string | null, doctype?: DocumentType | null): XMLDocument;\n    createDocumentType(qualifiedName: string, publicId: string, systemId: string): DocumentType;\n    createHTMLDocument(title?: string): Document;\n    /** @deprecated */\n    hasFeature(...args: any[]): true;\n}\n\ndeclare var DOMImplementation: {\n    prototype: DOMImplementation;\n    new(): DOMImplementation;\n};\n\ninterface DOMMatrix extends DOMMatrixReadOnly {\n    a: number;\n    b: number;\n    c: number;\n    d: number;\n    e: number;\n    f: number;\n    m11: number;\n    m12: number;\n    m13: number;\n    m14: number;\n    m21: number;\n    m22: number;\n    m23: number;\n    m24: number;\n    m31: number;\n    m32: number;\n    m33: number;\n    m34: number;\n    m41: number;\n    m42: number;\n    m43: number;\n    m44: number;\n    invertSelf(): DOMMatrix;\n    multiplySelf(other?: DOMMatrixInit): DOMMatrix;\n    preMultiplySelf(other?: DOMMatrixInit): DOMMatrix;\n    rotateAxisAngleSelf(x?: number, y?: number, z?: number, angle?: number): DOMMatrix;\n    rotateFromVectorSelf(x?: number, y?: number): DOMMatrix;\n    rotateSelf(rotX?: number, rotY?: number, rotZ?: number): DOMMatrix;\n    scale3dSelf(scale?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\n    scaleSelf(scaleX?: number, scaleY?: number, scaleZ?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\n    setMatrixValue(transformList: string): DOMMatrix;\n    skewXSelf(sx?: number): DOMMatrix;\n    skewYSelf(sy?: number): DOMMatrix;\n    translateSelf(tx?: number, ty?: number, tz?: number): DOMMatrix;\n}\n\ndeclare var DOMMatrix: {\n    prototype: DOMMatrix;\n    new(init?: string | number[]): DOMMatrix;\n    fromFloat32Array(array32: Float32Array): DOMMatrix;\n    fromFloat64Array(array64: Float64Array): DOMMatrix;\n    fromMatrix(other?: DOMMatrixInit): DOMMatrix;\n};\n\ntype SVGMatrix = DOMMatrix;\ndeclare var SVGMatrix: typeof DOMMatrix;\n\ntype WebKitCSSMatrix = DOMMatrix;\ndeclare var WebKitCSSMatrix: typeof DOMMatrix;\n\ninterface DOMMatrixReadOnly {\n    readonly a: number;\n    readonly b: number;\n    readonly c: number;\n    readonly d: number;\n    readonly e: number;\n    readonly f: number;\n    readonly is2D: boolean;\n    readonly isIdentity: boolean;\n    readonly m11: number;\n    readonly m12: number;\n    readonly m13: number;\n    readonly m14: number;\n    readonly m21: number;\n    readonly m22: number;\n    readonly m23: number;\n    readonly m24: number;\n    readonly m31: number;\n    readonly m32: number;\n    readonly m33: number;\n    readonly m34: number;\n    readonly m41: number;\n    readonly m42: number;\n    readonly m43: number;\n    readonly m44: number;\n    flipX(): DOMMatrix;\n    flipY(): DOMMatrix;\n    inverse(): DOMMatrix;\n    multiply(other?: DOMMatrixInit): DOMMatrix;\n    rotate(rotX?: number, rotY?: number, rotZ?: number): DOMMatrix;\n    rotateAxisAngle(x?: number, y?: number, z?: number, angle?: number): DOMMatrix;\n    rotateFromVector(x?: number, y?: number): DOMMatrix;\n    scale(scaleX?: number, scaleY?: number, scaleZ?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\n    scale3d(scale?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\n    /** @deprecated */\n    scaleNonUniform(scaleX?: number, scaleY?: number): DOMMatrix;\n    skewX(sx?: number): DOMMatrix;\n    skewY(sy?: number): DOMMatrix;\n    toFloat32Array(): Float32Array;\n    toFloat64Array(): Float64Array;\n    toJSON(): any;\n    transformPoint(point?: DOMPointInit): DOMPoint;\n    translate(tx?: number, ty?: number, tz?: number): DOMMatrix;\n    toString(): string;\n}\n\ndeclare var DOMMatrixReadOnly: {\n    prototype: DOMMatrixReadOnly;\n    new(init?: string | number[]): DOMMatrixReadOnly;\n    fromFloat32Array(array32: Float32Array): DOMMatrixReadOnly;\n    fromFloat64Array(array64: Float64Array): DOMMatrixReadOnly;\n    fromMatrix(other?: DOMMatrixInit): DOMMatrixReadOnly;\n    toString(): string;\n};\n\n/** Provides the ability to parse XML or HTML source code from a string into a DOM Document. */\ninterface DOMParser {\n    /**\n     * Parses string using either the HTML or XML parser, according to type, and returns the resulting Document. type can be "text/html" (which will invoke the HTML parser), or any of "text/xml", "application/xml", "application/xhtml+xml", or "image/svg+xml" (which will invoke the XML parser).\n     *\n     * For the XML parser, if string cannot be parsed, then the returned Document will contain elements describing the resulting error.\n     *\n     * Note that script elements are not evaluated during parsing, and the resulting document\'s encoding will always be UTF-8.\n     *\n     * Values other than the above for type will cause a TypeError exception to be thrown.\n     */\n    parseFromString(string: string, type: DOMParserSupportedType): Document;\n}\n\ndeclare var DOMParser: {\n    prototype: DOMParser;\n    new(): DOMParser;\n};\n\ninterface DOMPoint extends DOMPointReadOnly {\n    w: number;\n    x: number;\n    y: number;\n    z: number;\n}\n\ndeclare var DOMPoint: {\n    prototype: DOMPoint;\n    new(x?: number, y?: number, z?: number, w?: number): DOMPoint;\n    fromPoint(other?: DOMPointInit): DOMPoint;\n};\n\ntype SVGPoint = DOMPoint;\ndeclare var SVGPoint: typeof DOMPoint;\n\ninterface DOMPointReadOnly {\n    readonly w: number;\n    readonly x: number;\n    readonly y: number;\n    readonly z: number;\n    matrixTransform(matrix?: DOMMatrixInit): DOMPoint;\n    toJSON(): any;\n}\n\ndeclare var DOMPointReadOnly: {\n    prototype: DOMPointReadOnly;\n    new(x?: number, y?: number, z?: number, w?: number): DOMPointReadOnly;\n    fromPoint(other?: DOMPointInit): DOMPointReadOnly;\n};\n\ninterface DOMQuad {\n    readonly p1: DOMPoint;\n    readonly p2: DOMPoint;\n    readonly p3: DOMPoint;\n    readonly p4: DOMPoint;\n    getBounds(): DOMRect;\n    toJSON(): any;\n}\n\ndeclare var DOMQuad: {\n    prototype: DOMQuad;\n    new(p1?: DOMPointInit, p2?: DOMPointInit, p3?: DOMPointInit, p4?: DOMPointInit): DOMQuad;\n    fromQuad(other?: DOMQuadInit): DOMQuad;\n    fromRect(other?: DOMRectInit): DOMQuad;\n};\n\ninterface DOMRect extends DOMRectReadOnly {\n    height: number;\n    width: number;\n    x: number;\n    y: number;\n}\n\ndeclare var DOMRect: {\n    prototype: DOMRect;\n    new(x?: number, y?: number, width?: number, height?: number): DOMRect;\n    fromRect(other?: DOMRectInit): DOMRect;\n};\n\ntype SVGRect = DOMRect;\ndeclare var SVGRect: typeof DOMRect;\n\ninterface DOMRectList {\n    readonly length: number;\n    item(index: number): DOMRect | null;\n    [index: number]: DOMRect;\n}\n\ndeclare var DOMRectList: {\n    prototype: DOMRectList;\n    new(): DOMRectList;\n};\n\ninterface DOMRectReadOnly {\n    readonly bottom: number;\n    readonly height: number;\n    readonly left: number;\n    readonly right: number;\n    readonly top: number;\n    readonly width: number;\n    readonly x: number;\n    readonly y: number;\n    toJSON(): any;\n}\n\ndeclare var DOMRectReadOnly: {\n    prototype: DOMRectReadOnly;\n    new(x?: number, y?: number, width?: number, height?: number): DOMRectReadOnly;\n    fromRect(other?: DOMRectInit): DOMRectReadOnly;\n};\n\n/** A type returned by some APIs which contains a list of DOMString (strings). */\ninterface DOMStringList {\n    /**\n     * Returns the number of strings in strings.\n     */\n    readonly length: number;\n    /**\n     * Returns true if strings contains string, and false otherwise.\n     */\n    contains(string: string): boolean;\n    /**\n     * Returns the string with index index from strings.\n     */\n    item(index: number): string | null;\n    [index: number]: string;\n}\n\ndeclare var DOMStringList: {\n    prototype: DOMStringList;\n    new(): DOMStringList;\n};\n\n/** Used by the dataset HTML attribute to represent data for custom attributes added to elements. */\ninterface DOMStringMap {\n    [name: string]: string | undefined;\n}\n\ndeclare var DOMStringMap: {\n    prototype: DOMStringMap;\n    new(): DOMStringMap;\n};\n\n/** A set of space-separated tokens. Such a set is returned by Element.classList, HTMLLinkElement.relList, HTMLAnchorElement.relList, HTMLAreaElement.relList, HTMLIframeElement.sandbox, or HTMLOutputElement.htmlFor. It is indexed beginning with 0 as with JavaScript Array objects. DOMTokenList is always case-sensitive. */\ninterface DOMTokenList {\n    /**\n     * Returns the number of tokens.\n     */\n    readonly length: number;\n    /**\n     * Returns the associated set as string.\n     *\n     * Can be set, to change the associated attribute.\n     */\n    value: string;\n    toString(): string;\n    /**\n     * Adds all arguments passed, except those already present.\n     *\n     * Throws a "SyntaxError" DOMException if one of the arguments is the empty string.\n     *\n     * Throws an "InvalidCharacterError" DOMException if one of the arguments contains any ASCII whitespace.\n     */\n    add(...tokens: string[]): void;\n    /**\n     * Returns true if token is present, and false otherwise.\n     */\n    contains(token: string): boolean;\n    /**\n     * Returns the token with index index.\n     */\n    item(index: number): string | null;\n    /**\n     * Removes arguments passed, if they are present.\n     *\n     * Throws a "SyntaxError" DOMException if one of the arguments is the empty string.\n     *\n     * Throws an "InvalidCharacterError" DOMException if one of the arguments contains any ASCII whitespace.\n     */\n    remove(...tokens: string[]): void;\n    /**\n     * Replaces token with newToken.\n     *\n     * Returns true if token was replaced with newToken, and false otherwise.\n     *\n     * Throws a "SyntaxError" DOMException if one of the arguments is the empty string.\n     *\n     * Throws an "InvalidCharacterError" DOMException if one of the arguments contains any ASCII whitespace.\n     */\n    replace(token: string, newToken: string): boolean;\n    /**\n     * Returns true if token is in the associated attribute\'s supported tokens. Returns false otherwise.\n     *\n     * Throws a TypeError if the associated attribute has no supported tokens defined.\n     */\n    supports(token: string): boolean;\n    /**\n     * If force is not given, "toggles" token, removing it if it\'s present and adding it if it\'s not present. If force is true, adds token (same as add()). If force is false, removes token (same as remove()).\n     *\n     * Returns true if token is now present, and false otherwise.\n     *\n     * Throws a "SyntaxError" DOMException if token is empty.\n     *\n     * Throws an "InvalidCharacterError" DOMException if token contains any spaces.\n     */\n    toggle(token: string, force?: boolean): boolean;\n    forEach(callbackfn: (value: string, key: number, parent: DOMTokenList) => void, thisArg?: any): void;\n    [index: number]: string;\n}\n\ndeclare var DOMTokenList: {\n    prototype: DOMTokenList;\n    new(): DOMTokenList;\n};\n\n/** Used to hold the data that is being dragged during a drag and drop operation. It may hold one or more data items, each of one or more data types. For more information about drag and drop, see HTML Drag and Drop API. */\ninterface DataTransfer {\n    /**\n     * Returns the kind of operation that is currently selected. If the kind of operation isn\'t one of those that is allowed by the effectAllowed attribute, then the operation will fail.\n     *\n     * Can be set, to change the selected operation.\n     *\n     * The possible values are "none", "copy", "link", and "move".\n     */\n    dropEffect: "none" | "copy" | "link" | "move";\n    /**\n     * Returns the kinds of operations that are to be allowed.\n     *\n     * Can be set (during the dragstart event), to change the allowed operations.\n     *\n     * The possible values are "none", "copy", "copyLink", "copyMove", "link", "linkMove", "move", "all", and "uninitialized",\n     */\n    effectAllowed: "none" | "copy" | "copyLink" | "copyMove" | "link" | "linkMove" | "move" | "all" | "uninitialized";\n    /**\n     * Returns a FileList of the files being dragged, if any.\n     */\n    readonly files: FileList;\n    /**\n     * Returns a DataTransferItemList object, with the drag data.\n     */\n    readonly items: DataTransferItemList;\n    /**\n     * Returns a frozen array listing the formats that were set in the dragstart event. In addition, if any files are being dragged, then one of the types will be the string "Files".\n     */\n    readonly types: ReadonlyArray<string>;\n    /**\n     * Removes the data of the specified formats. Removes all data if the argument is omitted.\n     */\n    clearData(format?: string): void;\n    /**\n     * Returns the specified data. If there is no such data, returns the empty string.\n     */\n    getData(format: string): string;\n    /**\n     * Adds the specified data.\n     */\n    setData(format: string, data: string): void;\n    /**\n     * Uses the given element to update the drag feedback, replacing any previously specified feedback.\n     */\n    setDragImage(image: Element, x: number, y: number): void;\n}\n\ndeclare var DataTransfer: {\n    prototype: DataTransfer;\n    new(): DataTransfer;\n};\n\n/** One drag data item. During a drag operation, each drag event has a dataTransfer property which contains a list of drag data items. Each item in the list is a DataTransferItem object. */\ninterface DataTransferItem {\n    /**\n     * Returns the drag data item kind, one of: "string", "file".\n     */\n    readonly kind: string;\n    /**\n     * Returns the drag data item type string.\n     */\n    readonly type: string;\n    /**\n     * Returns a File object, if the drag data item kind is File.\n     */\n    getAsFile(): File | null;\n    /**\n     * Invokes the callback with the string data as the argument, if the drag data item kind is text.\n     */\n    getAsString(callback: FunctionStringCallback | null): void;\n    webkitGetAsEntry(): FileSystemEntry | null;\n}\n\ndeclare var DataTransferItem: {\n    prototype: DataTransferItem;\n    new(): DataTransferItem;\n};\n\n/** A list of DataTransferItem objects representing items being dragged. During a drag operation, each DragEvent has a dataTransfer property and that property is a DataTransferItemList. */\ninterface DataTransferItemList {\n    /**\n     * Returns the number of items in the drag data store.\n     */\n    readonly length: number;\n    /**\n     * Adds a new entry for the given data to the drag data store. If the data is plain text then a type string has to be provided also.\n     */\n    add(data: string, type: string): DataTransferItem | null;\n    add(data: File): DataTransferItem | null;\n    /**\n     * Removes all the entries in the drag data store.\n     */\n    clear(): void;\n    /**\n     * Removes the indexth entry in the drag data store.\n     */\n    remove(index: number): void;\n    [index: number]: DataTransferItem;\n}\n\ndeclare var DataTransferItemList: {\n    prototype: DataTransferItemList;\n    new(): DataTransferItemList;\n};\n\n/** A delay-line; an AudioNode audio-processing module that causes a delay between the arrival of an input data and its propagation to the output. */\ninterface DelayNode extends AudioNode {\n    readonly delayTime: AudioParam;\n}\n\ndeclare var DelayNode: {\n    prototype: DelayNode;\n    new(context: BaseAudioContext, options?: DelayOptions): DelayNode;\n};\n\n/** The DeviceMotionEvent provides web developers with information about the speed of changes for the device\'s position and orientation. */\ninterface DeviceMotionEvent extends Event {\n    readonly acceleration: DeviceMotionEventAcceleration | null;\n    readonly accelerationIncludingGravity: DeviceMotionEventAcceleration | null;\n    readonly interval: number;\n    readonly rotationRate: DeviceMotionEventRotationRate | null;\n}\n\ndeclare var DeviceMotionEvent: {\n    prototype: DeviceMotionEvent;\n    new(type: string, eventInitDict?: DeviceMotionEventInit): DeviceMotionEvent;\n};\n\ninterface DeviceMotionEventAcceleration {\n    readonly x: number | null;\n    readonly y: number | null;\n    readonly z: number | null;\n}\n\ninterface DeviceMotionEventRotationRate {\n    readonly alpha: number | null;\n    readonly beta: number | null;\n    readonly gamma: number | null;\n}\n\n/** The DeviceOrientationEvent provides web developers with information from the physical orientation of the device running the web page. */\ninterface DeviceOrientationEvent extends Event {\n    readonly absolute: boolean;\n    readonly alpha: number | null;\n    readonly beta: number | null;\n    readonly gamma: number | null;\n}\n\ndeclare var DeviceOrientationEvent: {\n    prototype: DeviceOrientationEvent;\n    new(type: string, eventInitDict?: DeviceOrientationEventInit): DeviceOrientationEvent;\n};\n\ninterface DocumentEventMap extends DocumentAndElementEventHandlersEventMap, GlobalEventHandlersEventMap {\n    "fullscreenchange": Event;\n    "fullscreenerror": Event;\n    "pointerlockchange": Event;\n    "pointerlockerror": Event;\n    "readystatechange": Event;\n    "visibilitychange": Event;\n}\n\n/** Any web page loaded in the browser and serves as an entry point into the web page\'s content, which is the DOM tree. */\ninterface Document extends Node, DocumentAndElementEventHandlers, DocumentOrShadowRoot, FontFaceSource, GlobalEventHandlers, NonElementParentNode, ParentNode, XPathEvaluatorBase {\n    /**\n     * Sets or gets the URL for the current document.\n     */\n    readonly URL: string;\n    /**\n     * Sets or gets the color of all active links in the document.\n     * @deprecated\n     */\n    alinkColor: string;\n    /**\n     * Returns a reference to the collection of elements contained by the object.\n     * @deprecated\n     */\n    readonly all: HTMLAllCollection;\n    /**\n     * Retrieves a collection of all a objects that have a name and/or id property. Objects in this collection are in HTML source order.\n     * @deprecated\n     */\n    readonly anchors: HTMLCollectionOf<HTMLAnchorElement>;\n    /**\n     * Retrieves a collection of all applet objects in the document.\n     * @deprecated\n     */\n    readonly applets: HTMLCollection;\n    /**\n     * Deprecated. Sets or retrieves a value that indicates the background color behind the object.\n     * @deprecated\n     */\n    bgColor: string;\n    /**\n     * Specifies the beginning and end of the document body.\n     */\n    body: HTMLElement;\n    /**\n     * Returns document\'s encoding.\n     */\n    readonly characterSet: string;\n    /**\n     * Gets or sets the character set used to encode the object.\n     * @deprecated This is a legacy alias of `characterSet`.\n     */\n    readonly charset: string;\n    /**\n     * Gets a value that indicates whether standards-compliant mode is switched on for the object.\n     */\n    readonly compatMode: string;\n    /**\n     * Returns document\'s content type.\n     */\n    readonly contentType: string;\n    /**\n     * Returns the HTTP cookies that apply to the Document. If there are no cookies or cookies can\'t be applied to this resource, the empty string will be returned.\n     *\n     * Can be set, to add a new cookie to the element\'s set of HTTP cookies.\n     *\n     * If the contents are sandboxed into a unique origin (e.g. in an iframe with the sandbox attribute), a "SecurityError" DOMException will be thrown on getting and setting.\n     */\n    cookie: string;\n    /**\n     * Returns the script element, or the SVG script element, that is currently executing, as long as the element represents a classic script. In the case of reentrant script execution, returns the one that most recently started executing amongst those that have not yet finished executing.\n     *\n     * Returns null if the Document is not currently executing a script or SVG script element (e.g., because the running script is an event handler, or a timeout), or if the currently executing script or SVG script element represents a module script.\n     */\n    readonly currentScript: HTMLOrSVGScriptElement | null;\n    /**\n     * Returns the Window object of the active document.\n     */\n    readonly defaultView: (WindowProxy & typeof globalThis) | null;\n    /**\n     * Sets or gets a value that indicates whether the document can be edited.\n     */\n    designMode: string;\n    /**\n     * Sets or retrieves a value that indicates the reading order of the object.\n     */\n    dir: string;\n    /**\n     * Gets an object representing the document type declaration associated with the current document.\n     */\n    readonly doctype: DocumentType | null;\n    /**\n     * Gets a reference to the root node of the document.\n     */\n    readonly documentElement: HTMLElement;\n    /**\n     * Returns document\'s URL.\n     */\n    readonly documentURI: string;\n    /**\n     * Sets or gets the security domain of the document.\n     */\n    domain: string;\n    /**\n     * Retrieves a collection of all embed objects in the document.\n     */\n    readonly embeds: HTMLCollectionOf<HTMLEmbedElement>;\n    /**\n     * Sets or gets the foreground (text) color of the document.\n     * @deprecated\n     */\n    fgColor: string;\n    /**\n     * Retrieves a collection, in source order, of all form objects in the document.\n     */\n    readonly forms: HTMLCollectionOf<HTMLFormElement>;\n    /** @deprecated */\n    readonly fullscreen: boolean;\n    /**\n     * Returns true if document has the ability to display elements fullscreen and fullscreen is supported, or false otherwise.\n     */\n    readonly fullscreenEnabled: boolean;\n    /**\n     * Returns the head element.\n     */\n    readonly head: HTMLHeadElement;\n    readonly hidden: boolean;\n    /**\n     * Retrieves a collection, in source order, of img objects in the document.\n     */\n    readonly images: HTMLCollectionOf<HTMLImageElement>;\n    /**\n     * Gets the implementation object of the current document.\n     */\n    readonly implementation: DOMImplementation;\n    /**\n     * Returns the character encoding used to create the webpage that is loaded into the document object.\n     * @deprecated This is a legacy alias of `characterSet`.\n     */\n    readonly inputEncoding: string;\n    /**\n     * Gets the date that the page was last modified, if the page supplies one.\n     */\n    readonly lastModified: string;\n    /**\n     * Sets or gets the color of the document links.\n     * @deprecated\n     */\n    linkColor: string;\n    /**\n     * Retrieves a collection of all a objects that specify the href property and all area objects in the document.\n     */\n    readonly links: HTMLCollectionOf<HTMLAnchorElement | HTMLAreaElement>;\n    /**\n     * Contains information about the current URL.\n     */\n    get location(): Location;\n    set location(href: string | Location);\n    onfullscreenchange: ((this: Document, ev: Event) => any) | null;\n    onfullscreenerror: ((this: Document, ev: Event) => any) | null;\n    onpointerlockchange: ((this: Document, ev: Event) => any) | null;\n    onpointerlockerror: ((this: Document, ev: Event) => any) | null;\n    /**\n     * Fires when the state of the object has changed.\n     * @param ev The event\n     */\n    onreadystatechange: ((this: Document, ev: Event) => any) | null;\n    onvisibilitychange: ((this: Document, ev: Event) => any) | null;\n    readonly ownerDocument: null;\n    readonly pictureInPictureEnabled: boolean;\n    /**\n     * Return an HTMLCollection of the embed elements in the Document.\n     */\n    readonly plugins: HTMLCollectionOf<HTMLEmbedElement>;\n    /**\n     * Retrieves a value that indicates the current state of the object.\n     */\n    readonly readyState: DocumentReadyState;\n    /**\n     * Gets the URL of the location that referred the user to the current page.\n     */\n    readonly referrer: string;\n    /** @deprecated */\n    readonly rootElement: SVGSVGElement | null;\n    /**\n     * Retrieves a collection of all script objects in the document.\n     */\n    readonly scripts: HTMLCollectionOf<HTMLScriptElement>;\n    readonly scrollingElement: Element | null;\n    readonly timeline: DocumentTimeline;\n    /**\n     * Contains the title of the document.\n     */\n    title: string;\n    readonly visibilityState: VisibilityState;\n    /**\n     * Sets or gets the color of the links that the user has visited.\n     * @deprecated\n     */\n    vlinkColor: string;\n    /**\n     * Moves node from another document and returns it.\n     *\n     * If node is a document, throws a "NotSupportedError" DOMException or, if node is a shadow root, throws a "HierarchyRequestError" DOMException.\n     */\n    adoptNode<T extends Node>(node: T): T;\n    /** @deprecated */\n    captureEvents(): void;\n    /** @deprecated */\n    caretRangeFromPoint(x: number, y: number): Range | null;\n    /** @deprecated */\n    clear(): void;\n    /**\n     * Closes an output stream and forces the sent data to display.\n     */\n    close(): void;\n    /**\n     * Creates an attribute object with a specified name.\n     * @param name String that sets the attribute object\'s name.\n     */\n    createAttribute(localName: string): Attr;\n    createAttributeNS(namespace: string | null, qualifiedName: string): Attr;\n    /**\n     * Returns a CDATASection node whose data is data.\n     */\n    createCDATASection(data: string): CDATASection;\n    /**\n     * Creates a comment object with the specified data.\n     * @param data Sets the comment object\'s data.\n     */\n    createComment(data: string): Comment;\n    /**\n     * Creates a new document.\n     */\n    createDocumentFragment(): DocumentFragment;\n    /**\n     * Creates an instance of the element for the specified tag.\n     * @param tagName The name of an element.\n     */\n    createElement<K extends keyof HTMLElementTagNameMap>(tagName: K, options?: ElementCreationOptions): HTMLElementTagNameMap[K];\n    /** @deprecated */\n    createElement<K extends keyof HTMLElementDeprecatedTagNameMap>(tagName: K, options?: ElementCreationOptions): HTMLElementDeprecatedTagNameMap[K];\n    createElement(tagName: string, options?: ElementCreationOptions): HTMLElement;\n    /**\n     * Returns an element with namespace namespace. Its namespace prefix will be everything before ":" (U+003E) in qualifiedName or null. Its local name will be everything after ":" (U+003E) in qualifiedName or qualifiedName.\n     *\n     * If localName does not match the Name production an "InvalidCharacterError" DOMException will be thrown.\n     *\n     * If one of the following conditions is true a "NamespaceError" DOMException will be thrown:\n     *\n     * localName does not match the QName production.\n     * Namespace prefix is not null and namespace is the empty string.\n     * Namespace prefix is "xml" and namespace is not the XML namespace.\n     * qualifiedName or namespace prefix is "xmlns" and namespace is not the XMLNS namespace.\n     * namespace is the XMLNS namespace and neither qualifiedName nor namespace prefix is "xmlns".\n     *\n     * When supplied, options\'s is can be used to create a customized built-in element.\n     */\n    createElementNS(namespaceURI: "http://www.w3.org/1999/xhtml", qualifiedName: string): HTMLElement;\n    createElementNS<K extends keyof SVGElementTagNameMap>(namespaceURI: "http://www.w3.org/2000/svg", qualifiedName: K): SVGElementTagNameMap[K];\n    createElementNS(namespaceURI: "http://www.w3.org/2000/svg", qualifiedName: string): SVGElement;\n    createElementNS(namespaceURI: string | null, qualifiedName: string, options?: ElementCreationOptions): Element;\n    createElementNS(namespace: string | null, qualifiedName: string, options?: string | ElementCreationOptions): Element;\n    createEvent(eventInterface: "AnimationEvent"): AnimationEvent;\n    createEvent(eventInterface: "AnimationPlaybackEvent"): AnimationPlaybackEvent;\n    createEvent(eventInterface: "AudioProcessingEvent"): AudioProcessingEvent;\n    createEvent(eventInterface: "BeforeUnloadEvent"): BeforeUnloadEvent;\n    createEvent(eventInterface: "BlobEvent"): BlobEvent;\n    createEvent(eventInterface: "ClipboardEvent"): ClipboardEvent;\n    createEvent(eventInterface: "CloseEvent"): CloseEvent;\n    createEvent(eventInterface: "CompositionEvent"): CompositionEvent;\n    createEvent(eventInterface: "CustomEvent"): CustomEvent;\n    createEvent(eventInterface: "DeviceMotionEvent"): DeviceMotionEvent;\n    createEvent(eventInterface: "DeviceOrientationEvent"): DeviceOrientationEvent;\n    createEvent(eventInterface: "DragEvent"): DragEvent;\n    createEvent(eventInterface: "ErrorEvent"): ErrorEvent;\n    createEvent(eventInterface: "FocusEvent"): FocusEvent;\n    createEvent(eventInterface: "FontFaceSetLoadEvent"): FontFaceSetLoadEvent;\n    createEvent(eventInterface: "FormDataEvent"): FormDataEvent;\n    createEvent(eventInterface: "GamepadEvent"): GamepadEvent;\n    createEvent(eventInterface: "HashChangeEvent"): HashChangeEvent;\n    createEvent(eventInterface: "IDBVersionChangeEvent"): IDBVersionChangeEvent;\n    createEvent(eventInterface: "InputEvent"): InputEvent;\n    createEvent(eventInterface: "KeyboardEvent"): KeyboardEvent;\n    createEvent(eventInterface: "MediaEncryptedEvent"): MediaEncryptedEvent;\n    createEvent(eventInterface: "MediaKeyMessageEvent"): MediaKeyMessageEvent;\n    createEvent(eventInterface: "MediaQueryListEvent"): MediaQueryListEvent;\n    createEvent(eventInterface: "MediaRecorderErrorEvent"): MediaRecorderErrorEvent;\n    createEvent(eventInterface: "MediaStreamTrackEvent"): MediaStreamTrackEvent;\n    createEvent(eventInterface: "MessageEvent"): MessageEvent;\n    createEvent(eventInterface: "MouseEvent"): MouseEvent;\n    createEvent(eventInterface: "MouseEvents"): MouseEvent;\n    createEvent(eventInterface: "MutationEvent"): MutationEvent;\n    createEvent(eventInterface: "MutationEvents"): MutationEvent;\n    createEvent(eventInterface: "OfflineAudioCompletionEvent"): OfflineAudioCompletionEvent;\n    createEvent(eventInterface: "PageTransitionEvent"): PageTransitionEvent;\n    createEvent(eventInterface: "PaymentMethodChangeEvent"): PaymentMethodChangeEvent;\n    createEvent(eventInterface: "PaymentRequestUpdateEvent"): PaymentRequestUpdateEvent;\n    createEvent(eventInterface: "PointerEvent"): PointerEvent;\n    createEvent(eventInterface: "PopStateEvent"): PopStateEvent;\n    createEvent(eventInterface: "ProgressEvent"): ProgressEvent;\n    createEvent(eventInterface: "PromiseRejectionEvent"): PromiseRejectionEvent;\n    createEvent(eventInterface: "RTCDTMFToneChangeEvent"): RTCDTMFToneChangeEvent;\n    createEvent(eventInterface: "RTCDataChannelEvent"): RTCDataChannelEvent;\n    createEvent(eventInterface: "RTCPeerConnectionIceErrorEvent"): RTCPeerConnectionIceErrorEvent;\n    createEvent(eventInterface: "RTCPeerConnectionIceEvent"): RTCPeerConnectionIceEvent;\n    createEvent(eventInterface: "RTCTrackEvent"): RTCTrackEvent;\n    createEvent(eventInterface: "SecurityPolicyViolationEvent"): SecurityPolicyViolationEvent;\n    createEvent(eventInterface: "SpeechSynthesisErrorEvent"): SpeechSynthesisErrorEvent;\n    createEvent(eventInterface: "SpeechSynthesisEvent"): SpeechSynthesisEvent;\n    createEvent(eventInterface: "StorageEvent"): StorageEvent;\n    createEvent(eventInterface: "SubmitEvent"): SubmitEvent;\n    createEvent(eventInterface: "TouchEvent"): TouchEvent;\n    createEvent(eventInterface: "TrackEvent"): TrackEvent;\n    createEvent(eventInterface: "TransitionEvent"): TransitionEvent;\n    createEvent(eventInterface: "UIEvent"): UIEvent;\n    createEvent(eventInterface: "UIEvents"): UIEvent;\n    createEvent(eventInterface: "WebGLContextEvent"): WebGLContextEvent;\n    createEvent(eventInterface: "WheelEvent"): WheelEvent;\n    createEvent(eventInterface: string): Event;\n    /**\n     * Creates a NodeIterator object that you can use to traverse filtered lists of nodes or elements in a document.\n     * @param root The root element or node to start traversing on.\n     * @param whatToShow The type of nodes or elements to appear in the node list\n     * @param filter A custom NodeFilter function to use. For more information, see filter. Use null for no filter.\n     */\n    createNodeIterator(root: Node, whatToShow?: number, filter?: NodeFilter | null): NodeIterator;\n    /**\n     * Returns a ProcessingInstruction node whose target is target and data is data. If target does not match the Name production an "InvalidCharacterError" DOMException will be thrown. If data contains "?>" an "InvalidCharacterError" DOMException will be thrown.\n     */\n    createProcessingInstruction(target: string, data: string): ProcessingInstruction;\n    /**\n     *  Returns an empty range object that has both of its boundary points positioned at the beginning of the document.\n     */\n    createRange(): Range;\n    /**\n     * Creates a text string from the specified value.\n     * @param data String that specifies the nodeValue property of the text node.\n     */\n    createTextNode(data: string): Text;\n    /**\n     * Creates a TreeWalker object that you can use to traverse filtered lists of nodes or elements in a document.\n     * @param root The root element or node to start traversing on.\n     * @param whatToShow The type of nodes or elements to appear in the node list. For more information, see whatToShow.\n     * @param filter A custom NodeFilter function to use.\n     */\n    createTreeWalker(root: Node, whatToShow?: number, filter?: NodeFilter | null): TreeWalker;\n    /**\n     * Returns the element for the specified x coordinate and the specified y coordinate.\n     * @param x The x-offset\n     * @param y The y-offset\n     */\n    elementFromPoint(x: number, y: number): Element | null;\n    elementsFromPoint(x: number, y: number): Element[];\n    /**\n     * Executes a command on the current document, current selection, or the given range.\n     * @param commandId String that specifies the command to execute. This command can be any of the command identifiers that can be executed in script.\n     * @param showUI Display the user interface, defaults to false.\n     * @param value Value to assign.\n     * @deprecated\n     */\n    execCommand(commandId: string, showUI?: boolean, value?: string): boolean;\n    /**\n     * Stops document\'s fullscreen element from being displayed fullscreen and resolves promise when done.\n     */\n    exitFullscreen(): Promise<void>;\n    exitPictureInPicture(): Promise<void>;\n    exitPointerLock(): void;\n    /**\n     * Returns a reference to the first object with the specified value of the ID attribute.\n     * @param elementId String that specifies the ID value.\n     */\n    getElementById(elementId: string): HTMLElement | null;\n    /**\n     * Returns a HTMLCollection of the elements in the object on which the method was invoked (a document or an element) that have all the classes given by classNames. The classNames argument is interpreted as a space-separated list of classes.\n     */\n    getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;\n    /**\n     * Gets a collection of objects based on the value of the NAME or ID attribute.\n     * @param elementName Gets a collection of objects based on the value of the NAME or ID attribute.\n     */\n    getElementsByName(elementName: string): NodeListOf<HTMLElement>;\n    /**\n     * Retrieves a collection of objects based on the specified element name.\n     * @param name Specifies the name of an element.\n     */\n    getElementsByTagName<K extends keyof HTMLElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<HTMLElementTagNameMap[K]>;\n    getElementsByTagName<K extends keyof SVGElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<SVGElementTagNameMap[K]>;\n    getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;\n    /**\n     * If namespace and localName are "*" returns a HTMLCollection of all descendant elements.\n     *\n     * If only namespace is "*" returns a HTMLCollection of all descendant elements whose local name is localName.\n     *\n     * If only localName is "*" returns a HTMLCollection of all descendant elements whose namespace is namespace.\n     *\n     * Otherwise, returns a HTMLCollection of all descendant elements whose namespace is namespace and local name is localName.\n     */\n    getElementsByTagNameNS(namespaceURI: "http://www.w3.org/1999/xhtml", localName: string): HTMLCollectionOf<HTMLElement>;\n    getElementsByTagNameNS(namespaceURI: "http://www.w3.org/2000/svg", localName: string): HTMLCollectionOf<SVGElement>;\n    getElementsByTagNameNS(namespace: string | null, localName: string): HTMLCollectionOf<Element>;\n    /**\n     * Returns an object representing the current selection of the document that is loaded into the object displaying a webpage.\n     */\n    getSelection(): Selection | null;\n    /**\n     * Gets a value indicating whether the object currently has focus.\n     */\n    hasFocus(): boolean;\n    hasStorageAccess(): Promise<boolean>;\n    /**\n     * Returns a copy of node. If deep is true, the copy also includes the node\'s descendants.\n     *\n     * If node is a document or a shadow root, throws a "NotSupportedError" DOMException.\n     */\n    importNode<T extends Node>(node: T, deep?: boolean): T;\n    /**\n     * Opens a new window and loads a document specified by a given URL. Also, opens a new window that uses the url parameter and the name parameter to collect the output of the write method and the writeln method.\n     * @param url Specifies a MIME type for the document.\n     * @param name Specifies the name of the window. This name is used as the value for the TARGET attribute on a form or an anchor element.\n     * @param features Contains a list of items separated by commas. Each item consists of an option and a value, separated by an equals sign (for example, "fullscreen=yes, toolbar=yes"). The following values are supported.\n     * @param replace Specifies whether the existing entry for the document is replaced in the history list.\n     */\n    open(unused1?: string, unused2?: string): Document;\n    open(url: string | URL, name: string, features: string): WindowProxy | null;\n    /**\n     * Returns a Boolean value that indicates whether a specified command can be successfully executed using execCommand, given the current state of the document.\n     * @param commandId Specifies a command identifier.\n     * @deprecated\n     */\n    queryCommandEnabled(commandId: string): boolean;\n    /**\n     * Returns a Boolean value that indicates whether the specified command is in the indeterminate state.\n     * @param commandId String that specifies a command identifier.\n     */\n    queryCommandIndeterm(commandId: string): boolean;\n    /**\n     * Returns a Boolean value that indicates the current state of the command.\n     * @param commandId String that specifies a command identifier.\n     * @deprecated\n     */\n    queryCommandState(commandId: string): boolean;\n    /**\n     * Returns a Boolean value that indicates whether the current command is supported on the current range.\n     * @param commandId Specifies a command identifier.\n     * @deprecated\n     */\n    queryCommandSupported(commandId: string): boolean;\n    /**\n     * Returns the current value of the document, range, or current selection for the given command.\n     * @param commandId String that specifies a command identifier.\n     */\n    queryCommandValue(commandId: string): string;\n    /** @deprecated */\n    releaseEvents(): void;\n    requestStorageAccess(): Promise<void>;\n    /**\n     * Writes one or more HTML expressions to a document in the specified window.\n     * @param content Specifies the text and HTML tags to write.\n     */\n    write(...text: string[]): void;\n    /**\n     * Writes one or more HTML expressions, followed by a carriage return, to a document in the specified window.\n     * @param content The text and HTML tags to write.\n     */\n    writeln(...text: string[]): void;\n    addEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: Document, ev: DocumentEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: Document, ev: DocumentEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var Document: {\n    prototype: Document;\n    new(): Document;\n};\n\ninterface DocumentAndElementEventHandlersEventMap {\n    "copy": ClipboardEvent;\n    "cut": ClipboardEvent;\n    "paste": ClipboardEvent;\n}\n\ninterface DocumentAndElementEventHandlers {\n    oncopy: ((this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any) | null;\n    oncut: ((this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any) | null;\n    onpaste: ((this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any) | null;\n    addEventListener<K extends keyof DocumentAndElementEventHandlersEventMap>(type: K, listener: (this: DocumentAndElementEventHandlers, ev: DocumentAndElementEventHandlersEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof DocumentAndElementEventHandlersEventMap>(type: K, listener: (this: DocumentAndElementEventHandlers, ev: DocumentAndElementEventHandlersEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** A minimal document object that has no parent. It is used as a lightweight version of Document that stores a segment of a document structure comprised of nodes just like a standard document. The key difference is that because the document fragment isn\'t part of the active document tree structure, changes made to the fragment don\'t affect the document, cause reflow, or incur any performance impact that can occur when changes are made. */\ninterface DocumentFragment extends Node, NonElementParentNode, ParentNode {\n    readonly ownerDocument: Document;\n    getElementById(elementId: string): HTMLElement | null;\n}\n\ndeclare var DocumentFragment: {\n    prototype: DocumentFragment;\n    new(): DocumentFragment;\n};\n\ninterface DocumentOrShadowRoot {\n    /**\n     * Returns the deepest element in the document through which or to which key events are being routed. This is, roughly speaking, the focused element in the document.\n     *\n     * For the purposes of this API, when a child browsing context is focused, its container is focused in the parent browsing context. For example, if the user moves the focus to a text control in an iframe, the iframe is the element returned by the activeElement API in the iframe\'s node document.\n     *\n     * Similarly, when the focused element is in a different node tree than documentOrShadowRoot, the element returned will be the host that\'s located in the same node tree as documentOrShadowRoot if documentOrShadowRoot is a shadow-including inclusive ancestor of the focused element, and null if not.\n     */\n    readonly activeElement: Element | null;\n    /**\n     * Returns document\'s fullscreen element.\n     */\n    readonly fullscreenElement: Element | null;\n    readonly pictureInPictureElement: Element | null;\n    readonly pointerLockElement: Element | null;\n    /**\n     * Retrieves a collection of styleSheet objects representing the style sheets that correspond to each instance of a link or style object in the document.\n     */\n    readonly styleSheets: StyleSheetList;\n    getAnimations(): Animation[];\n}\n\ninterface DocumentTimeline extends AnimationTimeline {\n}\n\ndeclare var DocumentTimeline: {\n    prototype: DocumentTimeline;\n    new(options?: DocumentTimelineOptions): DocumentTimeline;\n};\n\n/** A Node containing a doctype. */\ninterface DocumentType extends Node, ChildNode {\n    readonly name: string;\n    readonly ownerDocument: Document;\n    readonly publicId: string;\n    readonly systemId: string;\n}\n\ndeclare var DocumentType: {\n    prototype: DocumentType;\n    new(): DocumentType;\n};\n\n/** A DOM event that represents a drag and drop interaction. The user initiates a drag by placing a pointer device (such as a mouse) on the touch surface and then dragging the pointer to a new location (such as another DOM element). Applications are free to interpret a drag and drop interaction in an application-specific way. */\ninterface DragEvent extends MouseEvent {\n    /**\n     * Returns the DataTransfer object for the event.\n     */\n    readonly dataTransfer: DataTransfer | null;\n}\n\ndeclare var DragEvent: {\n    prototype: DragEvent;\n    new(type: string, eventInitDict?: DragEventInit): DragEvent;\n};\n\n/** Inherits properties from its parent, AudioNode. */\ninterface DynamicsCompressorNode extends AudioNode {\n    readonly attack: AudioParam;\n    readonly knee: AudioParam;\n    readonly ratio: AudioParam;\n    readonly reduction: number;\n    readonly release: AudioParam;\n    readonly threshold: AudioParam;\n}\n\ndeclare var DynamicsCompressorNode: {\n    prototype: DynamicsCompressorNode;\n    new(context: BaseAudioContext, options?: DynamicsCompressorOptions): DynamicsCompressorNode;\n};\n\ninterface EXT_blend_minmax {\n    readonly MAX_EXT: GLenum;\n    readonly MIN_EXT: GLenum;\n}\n\ninterface EXT_color_buffer_float {\n}\n\ninterface EXT_color_buffer_half_float {\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT: GLenum;\n    readonly RGB16F_EXT: GLenum;\n    readonly RGBA16F_EXT: GLenum;\n    readonly UNSIGNED_NORMALIZED_EXT: GLenum;\n}\n\ninterface EXT_float_blend {\n}\n\n/** The EXT_frag_depth extension is part of the WebGL API and enables to set a depth value of a fragment from within the fragment shader. */\ninterface EXT_frag_depth {\n}\n\ninterface EXT_sRGB {\n    readonly FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT: GLenum;\n    readonly SRGB8_ALPHA8_EXT: GLenum;\n    readonly SRGB_ALPHA_EXT: GLenum;\n    readonly SRGB_EXT: GLenum;\n}\n\ninterface EXT_shader_texture_lod {\n}\n\ninterface EXT_texture_compression_rgtc {\n    readonly COMPRESSED_RED_GREEN_RGTC2_EXT: GLenum;\n    readonly COMPRESSED_RED_RGTC1_EXT: GLenum;\n    readonly COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT: GLenum;\n    readonly COMPRESSED_SIGNED_RED_RGTC1_EXT: GLenum;\n}\n\n/** The EXT_texture_filter_anisotropic extension is part of the WebGL API and exposes two constants for anisotropic filtering (AF). */\ninterface EXT_texture_filter_anisotropic {\n    readonly MAX_TEXTURE_MAX_ANISOTROPY_EXT: GLenum;\n    readonly TEXTURE_MAX_ANISOTROPY_EXT: GLenum;\n}\n\ninterface ElementEventMap {\n    "fullscreenchange": Event;\n    "fullscreenerror": Event;\n}\n\n/** Element is the most general base class from which all objects in a Document inherit. It only has methods and properties common to all kinds of elements. More specific classes inherit from Element. */\ninterface Element extends Node, ARIAMixin, Animatable, ChildNode, InnerHTML, NonDocumentTypeChildNode, ParentNode, Slottable {\n    readonly attributes: NamedNodeMap;\n    /**\n     * Allows for manipulation of element\'s class content attribute as a set of whitespace-separated tokens through a DOMTokenList object.\n     */\n    readonly classList: DOMTokenList;\n    /**\n     * Returns the value of element\'s class content attribute. Can be set to change it.\n     */\n    className: string;\n    readonly clientHeight: number;\n    readonly clientLeft: number;\n    readonly clientTop: number;\n    readonly clientWidth: number;\n    /**\n     * Returns the value of element\'s id content attribute. Can be set to change it.\n     */\n    id: string;\n    /**\n     * Returns the local name.\n     */\n    readonly localName: string;\n    /**\n     * Returns the namespace.\n     */\n    readonly namespaceURI: string | null;\n    onfullscreenchange: ((this: Element, ev: Event) => any) | null;\n    onfullscreenerror: ((this: Element, ev: Event) => any) | null;\n    outerHTML: string;\n    readonly ownerDocument: Document;\n    readonly part: DOMTokenList;\n    /**\n     * Returns the namespace prefix.\n     */\n    readonly prefix: string | null;\n    readonly scrollHeight: number;\n    scrollLeft: number;\n    scrollTop: number;\n    readonly scrollWidth: number;\n    /**\n     * Returns element\'s shadow root, if any, and if shadow root\'s mode is "open", and null otherwise.\n     */\n    readonly shadowRoot: ShadowRoot | null;\n    /**\n     * Returns the value of element\'s slot content attribute. Can be set to change it.\n     */\n    slot: string;\n    /**\n     * Returns the HTML-uppercased qualified name.\n     */\n    readonly tagName: string;\n    /**\n     * Creates a shadow root for element and returns it.\n     */\n    attachShadow(init: ShadowRootInit): ShadowRoot;\n    /**\n     * Returns the first (starting at element) inclusive ancestor that matches selectors, and null otherwise.\n     */\n    closest<K extends keyof HTMLElementTagNameMap>(selector: K): HTMLElementTagNameMap[K] | null;\n    closest<K extends keyof SVGElementTagNameMap>(selector: K): SVGElementTagNameMap[K] | null;\n    closest<E extends Element = Element>(selectors: string): E | null;\n    /**\n     * Returns element\'s first attribute whose qualified name is qualifiedName, and null if there is no such attribute otherwise.\n     */\n    getAttribute(qualifiedName: string): string | null;\n    /**\n     * Returns element\'s attribute whose namespace is namespace and local name is localName, and null if there is no such attribute otherwise.\n     */\n    getAttributeNS(namespace: string | null, localName: string): string | null;\n    /**\n     * Returns the qualified names of all element\'s attributes. Can contain duplicates.\n     */\n    getAttributeNames(): string[];\n    getAttributeNode(qualifiedName: string): Attr | null;\n    getAttributeNodeNS(namespace: string | null, localName: string): Attr | null;\n    getBoundingClientRect(): DOMRect;\n    getClientRects(): DOMRectList;\n    /**\n     * Returns a HTMLCollection of the elements in the object on which the method was invoked (a document or an element) that have all the classes given by classNames. The classNames argument is interpreted as a space-separated list of classes.\n     */\n    getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;\n    getElementsByTagName<K extends keyof HTMLElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<HTMLElementTagNameMap[K]>;\n    getElementsByTagName<K extends keyof SVGElementTagNameMap>(qualifiedName: K): HTMLCollectionOf<SVGElementTagNameMap[K]>;\n    getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;\n    getElementsByTagNameNS(namespaceURI: "http://www.w3.org/1999/xhtml", localName: string): HTMLCollectionOf<HTMLElement>;\n    getElementsByTagNameNS(namespaceURI: "http://www.w3.org/2000/svg", localName: string): HTMLCollectionOf<SVGElement>;\n    getElementsByTagNameNS(namespace: string | null, localName: string): HTMLCollectionOf<Element>;\n    /**\n     * Returns true if element has an attribute whose qualified name is qualifiedName, and false otherwise.\n     */\n    hasAttribute(qualifiedName: string): boolean;\n    /**\n     * Returns true if element has an attribute whose namespace is namespace and local name is localName.\n     */\n    hasAttributeNS(namespace: string | null, localName: string): boolean;\n    /**\n     * Returns true if element has attributes, and false otherwise.\n     */\n    hasAttributes(): boolean;\n    hasPointerCapture(pointerId: number): boolean;\n    insertAdjacentElement(where: InsertPosition, element: Element): Element | null;\n    insertAdjacentHTML(position: InsertPosition, text: string): void;\n    insertAdjacentText(where: InsertPosition, data: string): void;\n    /**\n     * Returns true if matching selectors against element\'s root yields element, and false otherwise.\n     */\n    matches(selectors: string): boolean;\n    releasePointerCapture(pointerId: number): void;\n    /**\n     * Removes element\'s first attribute whose qualified name is qualifiedName.\n     */\n    removeAttribute(qualifiedName: string): void;\n    /**\n     * Removes element\'s attribute whose namespace is namespace and local name is localName.\n     */\n    removeAttributeNS(namespace: string | null, localName: string): void;\n    removeAttributeNode(attr: Attr): Attr;\n    /**\n     * Displays element fullscreen and resolves promise when done.\n     *\n     * When supplied, options\'s navigationUI member indicates whether showing navigation UI while in fullscreen is preferred or not. If set to "show", navigation simplicity is preferred over screen space, and if set to "hide", more screen space is preferred. User agents are always free to honor user preference over the application\'s. The default value "auto" indicates no application preference.\n     */\n    requestFullscreen(options?: FullscreenOptions): Promise<void>;\n    requestPointerLock(): void;\n    scroll(options?: ScrollToOptions): void;\n    scroll(x: number, y: number): void;\n    scrollBy(options?: ScrollToOptions): void;\n    scrollBy(x: number, y: number): void;\n    scrollIntoView(arg?: boolean | ScrollIntoViewOptions): void;\n    scrollTo(options?: ScrollToOptions): void;\n    scrollTo(x: number, y: number): void;\n    /**\n     * Sets the value of element\'s first attribute whose qualified name is qualifiedName to value.\n     */\n    setAttribute(qualifiedName: string, value: string): void;\n    /**\n     * Sets the value of element\'s attribute whose namespace is namespace and local name is localName to value.\n     */\n    setAttributeNS(namespace: string | null, qualifiedName: string, value: string): void;\n    setAttributeNode(attr: Attr): Attr | null;\n    setAttributeNodeNS(attr: Attr): Attr | null;\n    setPointerCapture(pointerId: number): void;\n    /**\n     * If force is not given, "toggles" qualifiedName, removing it if it is present and adding it if it is not present. If force is true, adds qualifiedName. If force is false, removes qualifiedName.\n     *\n     * Returns true if qualifiedName is now present, and false otherwise.\n     */\n    toggleAttribute(qualifiedName: string, force?: boolean): boolean;\n    /** @deprecated This is a legacy alias of `matches`. */\n    webkitMatchesSelector(selectors: string): boolean;\n    addEventListener<K extends keyof ElementEventMap>(type: K, listener: (this: Element, ev: ElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ElementEventMap>(type: K, listener: (this: Element, ev: ElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var Element: {\n    prototype: Element;\n    new(): Element;\n};\n\ninterface ElementCSSInlineStyle {\n    readonly style: CSSStyleDeclaration;\n}\n\ninterface ElementContentEditable {\n    contentEditable: string;\n    enterKeyHint: string;\n    inputMode: string;\n    readonly isContentEditable: boolean;\n}\n\n/** Events providing information related to errors in scripts or in files. */\ninterface ErrorEvent extends Event {\n    readonly colno: number;\n    readonly error: any;\n    readonly filename: string;\n    readonly lineno: number;\n    readonly message: string;\n}\n\ndeclare var ErrorEvent: {\n    prototype: ErrorEvent;\n    new(type: string, eventInitDict?: ErrorEventInit): ErrorEvent;\n};\n\n/** An event which takes place in the DOM. */\ninterface Event {\n    /**\n     * Returns true or false depending on how event was initialized. True if event goes through its target\'s ancestors in reverse tree order, and false otherwise.\n     */\n    readonly bubbles: boolean;\n    cancelBubble: boolean;\n    /**\n     * Returns true or false depending on how event was initialized. Its return value does not always carry meaning, but true can indicate that part of the operation during which event was dispatched, can be canceled by invoking the preventDefault() method.\n     */\n    readonly cancelable: boolean;\n    /**\n     * Returns true or false depending on how event was initialized. True if event invokes listeners past a ShadowRoot node that is the root of its target, and false otherwise.\n     */\n    readonly composed: boolean;\n    /**\n     * Returns the object whose event listener\'s callback is currently being invoked.\n     */\n    readonly currentTarget: EventTarget | null;\n    /**\n     * Returns true if preventDefault() was invoked successfully to indicate cancelation, and false otherwise.\n     */\n    readonly defaultPrevented: boolean;\n    /**\n     * Returns the event\'s phase, which is one of NONE, CAPTURING_PHASE, AT_TARGET, and BUBBLING_PHASE.\n     */\n    readonly eventPhase: number;\n    /**\n     * Returns true if event was dispatched by the user agent, and false otherwise.\n     */\n    readonly isTrusted: boolean;\n    /** @deprecated */\n    returnValue: boolean;\n    /** @deprecated */\n    readonly srcElement: EventTarget | null;\n    /**\n     * Returns the object to which event is dispatched (its target).\n     */\n    readonly target: EventTarget | null;\n    /**\n     * Returns the event\'s timestamp as the number of milliseconds measured relative to the time origin.\n     */\n    readonly timeStamp: DOMHighResTimeStamp;\n    /**\n     * Returns the type of event, e.g. "click", "hashchange", or "submit".\n     */\n    readonly type: string;\n    /**\n     * Returns the invocation target objects of event\'s path (objects on which listeners will be invoked), except for any nodes in shadow trees of which the shadow root\'s mode is "closed" that are not reachable from event\'s currentTarget.\n     */\n    composedPath(): EventTarget[];\n    /** @deprecated */\n    initEvent(type: string, bubbles?: boolean, cancelable?: boolean): void;\n    /**\n     * If invoked when the cancelable attribute value is true, and while executing a listener for the event with passive set to false, signals to the operation that caused event to be dispatched that it needs to be canceled.\n     */\n    preventDefault(): void;\n    /**\n     * Invoking this method prevents event from reaching any registered event listeners after the current one finishes running and, when dispatched in a tree, also prevents event from reaching any other objects.\n     */\n    stopImmediatePropagation(): void;\n    /**\n     * When dispatched in a tree, invoking this method prevents event from reaching any objects other than the current object.\n     */\n    stopPropagation(): void;\n    readonly AT_TARGET: number;\n    readonly BUBBLING_PHASE: number;\n    readonly CAPTURING_PHASE: number;\n    readonly NONE: number;\n}\n\ndeclare var Event: {\n    prototype: Event;\n    new(type: string, eventInitDict?: EventInit): Event;\n    readonly AT_TARGET: number;\n    readonly BUBBLING_PHASE: number;\n    readonly CAPTURING_PHASE: number;\n    readonly NONE: number;\n};\n\ninterface EventListener {\n    (evt: Event): void;\n}\n\ninterface EventListenerObject {\n    handleEvent(object: Event): void;\n}\n\ninterface EventSourceEventMap {\n    "error": Event;\n    "message": MessageEvent;\n    "open": Event;\n}\n\ninterface EventSource extends EventTarget {\n    onerror: ((this: EventSource, ev: Event) => any) | null;\n    onmessage: ((this: EventSource, ev: MessageEvent) => any) | null;\n    onopen: ((this: EventSource, ev: Event) => any) | null;\n    /**\n     * Returns the state of this EventSource object\'s connection. It can have the values described below.\n     */\n    readonly readyState: number;\n    /**\n     * Returns the URL providing the event stream.\n     */\n    readonly url: string;\n    /**\n     * Returns true if the credentials mode for connection requests to the URL providing the event stream is set to "include", and false otherwise.\n     */\n    readonly withCredentials: boolean;\n    /**\n     * Aborts any instances of the fetch algorithm started for this EventSource object, and sets the readyState attribute to CLOSED.\n     */\n    close(): void;\n    readonly CLOSED: number;\n    readonly CONNECTING: number;\n    readonly OPEN: number;\n    addEventListener<K extends keyof EventSourceEventMap>(type: K, listener: (this: EventSource, ev: EventSourceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof EventSourceEventMap>(type: K, listener: (this: EventSource, ev: EventSourceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var EventSource: {\n    prototype: EventSource;\n    new(url: string | URL, eventSourceInitDict?: EventSourceInit): EventSource;\n    readonly CLOSED: number;\n    readonly CONNECTING: number;\n    readonly OPEN: number;\n};\n\n/** EventTarget is a DOM interface implemented by objects that can receive events and may have listeners for them. */\ninterface EventTarget {\n    /**\n     * Appends an event listener for events whose type attribute value is type. The callback argument sets the callback that will be invoked when the event is dispatched.\n     *\n     * The options argument sets listener-specific options. For compatibility this can be a boolean, in which case the method behaves exactly as if the value was specified as options\'s capture.\n     *\n     * When set to true, options\'s capture prevents callback from being invoked when the event\'s eventPhase attribute value is BUBBLING_PHASE. When false (or not present), callback will not be invoked when event\'s eventPhase attribute value is CAPTURING_PHASE. Either way, callback will be invoked if event\'s eventPhase attribute value is AT_TARGET.\n     *\n     * When set to true, options\'s passive indicates that the callback will not cancel the event by invoking preventDefault(). This is used to enable performance optimizations described in § 2.8 Observing event listeners.\n     *\n     * When set to true, options\'s once indicates that the callback will only be invoked once after which the event listener will be removed.\n     *\n     * If an AbortSignal is passed for options\'s signal, then the event listener will be removed when signal is aborted.\n     *\n     * The event listener is appended to target\'s event listener list and is not appended if it has the same type, callback, and capture.\n     */\n    addEventListener(type: string, callback: EventListenerOrEventListenerObject | null, options?: AddEventListenerOptions | boolean): void;\n    /**\n     * Dispatches a synthetic event event to target and returns true if either event\'s cancelable attribute value is false or its preventDefault() method was not invoked, and false otherwise.\n     */\n    dispatchEvent(event: Event): boolean;\n    /**\n     * Removes the event listener in target\'s event listener list with the same type, callback, and options.\n     */\n    removeEventListener(type: string, callback: EventListenerOrEventListenerObject | null, options?: EventListenerOptions | boolean): void;\n}\n\ndeclare var EventTarget: {\n    prototype: EventTarget;\n    new(): EventTarget;\n};\n\n/** @deprecated */\ninterface External {\n    /** @deprecated */\n    AddSearchProvider(): void;\n    /** @deprecated */\n    IsSearchProviderInstalled(): void;\n}\n\n/** @deprecated */\ndeclare var External: {\n    prototype: External;\n    new(): External;\n};\n\n/** Provides information about files and allows JavaScript in a web page to access their content. */\ninterface File extends Blob {\n    readonly lastModified: number;\n    readonly name: string;\n    readonly webkitRelativePath: string;\n}\n\ndeclare var File: {\n    prototype: File;\n    new(fileBits: BlobPart[], fileName: string, options?: FilePropertyBag): File;\n};\n\n/** An object of this type is returned by the files property of the HTML <input> element; this lets you access the list of files selected with the <input type="file"> element. It\'s also used for a list of files dropped into web content when using the drag and drop API; see the DataTransfer object for details on this usage. */\ninterface FileList {\n    readonly length: number;\n    item(index: number): File | null;\n    [index: number]: File;\n}\n\ndeclare var FileList: {\n    prototype: FileList;\n    new(): FileList;\n};\n\ninterface FileReaderEventMap {\n    "abort": ProgressEvent<FileReader>;\n    "error": ProgressEvent<FileReader>;\n    "load": ProgressEvent<FileReader>;\n    "loadend": ProgressEvent<FileReader>;\n    "loadstart": ProgressEvent<FileReader>;\n    "progress": ProgressEvent<FileReader>;\n}\n\n/** Lets web applications asynchronously read the contents of files (or raw data buffers) stored on the user\'s computer, using File or Blob objects to specify the file or data to read. */\ninterface FileReader extends EventTarget {\n    readonly error: DOMException | null;\n    onabort: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onerror: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onload: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onloadend: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onloadstart: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onprogress: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    readonly readyState: number;\n    readonly result: string | ArrayBuffer | null;\n    abort(): void;\n    readAsArrayBuffer(blob: Blob): void;\n    readAsBinaryString(blob: Blob): void;\n    readAsDataURL(blob: Blob): void;\n    readAsText(blob: Blob, encoding?: string): void;\n    readonly DONE: number;\n    readonly EMPTY: number;\n    readonly LOADING: number;\n    addEventListener<K extends keyof FileReaderEventMap>(type: K, listener: (this: FileReader, ev: FileReaderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof FileReaderEventMap>(type: K, listener: (this: FileReader, ev: FileReaderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var FileReader: {\n    prototype: FileReader;\n    new(): FileReader;\n    readonly DONE: number;\n    readonly EMPTY: number;\n    readonly LOADING: number;\n};\n\ninterface FileSystem {\n    readonly name: string;\n    readonly root: FileSystemDirectoryEntry;\n}\n\ndeclare var FileSystem: {\n    prototype: FileSystem;\n    new(): FileSystem;\n};\n\ninterface FileSystemDirectoryEntry extends FileSystemEntry {\n    createReader(): FileSystemDirectoryReader;\n    getDirectory(path?: string | null, options?: FileSystemFlags, successCallback?: FileSystemEntryCallback, errorCallback?: ErrorCallback): void;\n    getFile(path?: string | null, options?: FileSystemFlags, successCallback?: FileSystemEntryCallback, errorCallback?: ErrorCallback): void;\n}\n\ndeclare var FileSystemDirectoryEntry: {\n    prototype: FileSystemDirectoryEntry;\n    new(): FileSystemDirectoryEntry;\n};\n\n/** @deprecated */\ninterface FileSystemDirectoryReader {\n    /** @deprecated */\n    readEntries(successCallback: FileSystemEntriesCallback, errorCallback?: ErrorCallback): void;\n}\n\n/** @deprecated */\ndeclare var FileSystemDirectoryReader: {\n    prototype: FileSystemDirectoryReader;\n    new(): FileSystemDirectoryReader;\n};\n\ninterface FileSystemEntry {\n    readonly filesystem: FileSystem;\n    readonly fullPath: string;\n    readonly isDirectory: boolean;\n    readonly isFile: boolean;\n    readonly name: string;\n    getParent(successCallback?: FileSystemEntryCallback, errorCallback?: ErrorCallback): void;\n}\n\ndeclare var FileSystemEntry: {\n    prototype: FileSystemEntry;\n    new(): FileSystemEntry;\n};\n\ninterface FileSystemFileEntry extends FileSystemEntry {\n    file(successCallback: FileCallback, errorCallback?: ErrorCallback): void;\n}\n\ndeclare var FileSystemFileEntry: {\n    prototype: FileSystemFileEntry;\n    new(): FileSystemFileEntry;\n};\n\n/** Focus-related events like focus, blur, focusin, or focusout. */\ninterface FocusEvent extends UIEvent {\n    readonly relatedTarget: EventTarget | null;\n}\n\ndeclare var FocusEvent: {\n    prototype: FocusEvent;\n    new(type: string, eventInitDict?: FocusEventInit): FocusEvent;\n};\n\ninterface FontFace {\n    ascentOverride: string;\n    descentOverride: string;\n    display: string;\n    family: string;\n    featureSettings: string;\n    lineGapOverride: string;\n    readonly loaded: Promise<FontFace>;\n    readonly status: FontFaceLoadStatus;\n    stretch: string;\n    style: string;\n    unicodeRange: string;\n    variant: string;\n    variationSettings: string;\n    weight: string;\n    load(): Promise<FontFace>;\n}\n\ndeclare var FontFace: {\n    prototype: FontFace;\n    new(family: string, source: string | BinaryData, descriptors?: FontFaceDescriptors): FontFace;\n};\n\ninterface FontFaceSetEventMap {\n    "loading": Event;\n    "loadingdone": Event;\n    "loadingerror": Event;\n}\n\ninterface FontFaceSet extends EventTarget {\n    onloading: ((this: FontFaceSet, ev: Event) => any) | null;\n    onloadingdone: ((this: FontFaceSet, ev: Event) => any) | null;\n    onloadingerror: ((this: FontFaceSet, ev: Event) => any) | null;\n    readonly ready: Promise<FontFaceSet>;\n    readonly status: FontFaceSetLoadStatus;\n    check(font: string, text?: string): boolean;\n    load(font: string, text?: string): Promise<FontFace[]>;\n    forEach(callbackfn: (value: FontFace, key: FontFace, parent: FontFaceSet) => void, thisArg?: any): void;\n    addEventListener<K extends keyof FontFaceSetEventMap>(type: K, listener: (this: FontFaceSet, ev: FontFaceSetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof FontFaceSetEventMap>(type: K, listener: (this: FontFaceSet, ev: FontFaceSetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var FontFaceSet: {\n    prototype: FontFaceSet;\n    new(initialFaces: FontFace[]): FontFaceSet;\n};\n\ninterface FontFaceSetLoadEvent extends Event {\n    readonly fontfaces: ReadonlyArray<FontFace>;\n}\n\ndeclare var FontFaceSetLoadEvent: {\n    prototype: FontFaceSetLoadEvent;\n    new(type: string, eventInitDict?: FontFaceSetLoadEventInit): FontFaceSetLoadEvent;\n};\n\ninterface FontFaceSource {\n    readonly fonts: FontFaceSet;\n}\n\n/** Provides a way to easily construct a set of key/value pairs representing form fields and their values, which can then be easily sent using the XMLHttpRequest.send() method. It uses the same format a form would use if the encoding type were set to "multipart/form-data". */\ninterface FormData {\n    append(name: string, value: string | Blob, fileName?: string): void;\n    delete(name: string): void;\n    get(name: string): FormDataEntryValue | null;\n    getAll(name: string): FormDataEntryValue[];\n    has(name: string): boolean;\n    set(name: string, value: string | Blob, fileName?: string): void;\n    forEach(callbackfn: (value: FormDataEntryValue, key: string, parent: FormData) => void, thisArg?: any): void;\n}\n\ndeclare var FormData: {\n    prototype: FormData;\n    new(form?: HTMLFormElement): FormData;\n};\n\ninterface FormDataEvent extends Event {\n    /**\n     * Returns a FormData object representing names and values of elements associated to the target form. Operations on the FormData object will affect form data to be submitted.\n     */\n    readonly formData: FormData;\n}\n\ndeclare var FormDataEvent: {\n    prototype: FormDataEvent;\n    new(type: string, eventInitDict: FormDataEventInit): FormDataEvent;\n};\n\n/** A change in volume. It is an AudioNode audio-processing module that causes a given gain to be applied to the input data before its propagation to the output. A GainNode always has exactly one input and one output, both with the same number of channels. */\ninterface GainNode extends AudioNode {\n    readonly gain: AudioParam;\n}\n\ndeclare var GainNode: {\n    prototype: GainNode;\n    new(context: BaseAudioContext, options?: GainOptions): GainNode;\n};\n\n/** This Gamepad API interface defines an individual gamepad or other controller, allowing access to information such as button presses, axis positions, and id. */\ninterface Gamepad {\n    readonly axes: ReadonlyArray<number>;\n    readonly buttons: ReadonlyArray<GamepadButton>;\n    readonly connected: boolean;\n    readonly hapticActuators: ReadonlyArray<GamepadHapticActuator>;\n    readonly id: string;\n    readonly index: number;\n    readonly mapping: GamepadMappingType;\n    readonly timestamp: DOMHighResTimeStamp;\n}\n\ndeclare var Gamepad: {\n    prototype: Gamepad;\n    new(): Gamepad;\n};\n\n/** An individual button of a gamepad or other controller, allowing access to the current state of different types of buttons available on the control device. */\ninterface GamepadButton {\n    readonly pressed: boolean;\n    readonly touched: boolean;\n    readonly value: number;\n}\n\ndeclare var GamepadButton: {\n    prototype: GamepadButton;\n    new(): GamepadButton;\n};\n\n/** This Gamepad API interface contains references to gamepads connected to the system, which is what the gamepad events Window.gamepadconnected and Window.gamepaddisconnected are fired in response to. */\ninterface GamepadEvent extends Event {\n    readonly gamepad: Gamepad;\n}\n\ndeclare var GamepadEvent: {\n    prototype: GamepadEvent;\n    new(type: string, eventInitDict: GamepadEventInit): GamepadEvent;\n};\n\n/** This Gamepad API interface represents hardware in the controller designed to provide haptic feedback to the user (if available), most commonly vibration hardware. */\ninterface GamepadHapticActuator {\n    readonly type: GamepadHapticActuatorType;\n}\n\ndeclare var GamepadHapticActuator: {\n    prototype: GamepadHapticActuator;\n    new(): GamepadHapticActuator;\n};\n\ninterface GenericTransformStream {\n    readonly readable: ReadableStream;\n    readonly writable: WritableStream;\n}\n\n/** An object able to programmatically obtain the position of the device. It gives Web content access to the location of the device. This allows a Web site or app to offer customized results based on the user\'s location. */\ninterface Geolocation {\n    clearWatch(watchId: number): void;\n    getCurrentPosition(successCallback: PositionCallback, errorCallback?: PositionErrorCallback | null, options?: PositionOptions): void;\n    watchPosition(successCallback: PositionCallback, errorCallback?: PositionErrorCallback | null, options?: PositionOptions): number;\n}\n\ndeclare var Geolocation: {\n    prototype: Geolocation;\n    new(): Geolocation;\n};\n\ninterface GeolocationCoordinates {\n    readonly accuracy: number;\n    readonly altitude: number | null;\n    readonly altitudeAccuracy: number | null;\n    readonly heading: number | null;\n    readonly latitude: number;\n    readonly longitude: number;\n    readonly speed: number | null;\n}\n\ndeclare var GeolocationCoordinates: {\n    prototype: GeolocationCoordinates;\n    new(): GeolocationCoordinates;\n};\n\ninterface GeolocationPosition {\n    readonly coords: GeolocationCoordinates;\n    readonly timestamp: DOMTimeStamp;\n}\n\ndeclare var GeolocationPosition: {\n    prototype: GeolocationPosition;\n    new(): GeolocationPosition;\n};\n\ninterface GeolocationPositionError {\n    readonly code: number;\n    readonly message: string;\n    readonly PERMISSION_DENIED: number;\n    readonly POSITION_UNAVAILABLE: number;\n    readonly TIMEOUT: number;\n}\n\ndeclare var GeolocationPositionError: {\n    prototype: GeolocationPositionError;\n    new(): GeolocationPositionError;\n    readonly PERMISSION_DENIED: number;\n    readonly POSITION_UNAVAILABLE: number;\n    readonly TIMEOUT: number;\n};\n\ninterface GlobalEventHandlersEventMap {\n    "abort": UIEvent;\n    "animationcancel": AnimationEvent;\n    "animationend": AnimationEvent;\n    "animationiteration": AnimationEvent;\n    "animationstart": AnimationEvent;\n    "auxclick": MouseEvent;\n    "beforeinput": InputEvent;\n    "blur": FocusEvent;\n    "canplay": Event;\n    "canplaythrough": Event;\n    "change": Event;\n    "click": MouseEvent;\n    "close": Event;\n    "compositionend": CompositionEvent;\n    "compositionstart": CompositionEvent;\n    "compositionupdate": CompositionEvent;\n    "contextmenu": MouseEvent;\n    "cuechange": Event;\n    "dblclick": MouseEvent;\n    "drag": DragEvent;\n    "dragend": DragEvent;\n    "dragenter": DragEvent;\n    "dragleave": DragEvent;\n    "dragover": DragEvent;\n    "dragstart": DragEvent;\n    "drop": DragEvent;\n    "durationchange": Event;\n    "emptied": Event;\n    "ended": Event;\n    "error": ErrorEvent;\n    "focus": FocusEvent;\n    "focusin": FocusEvent;\n    "focusout": FocusEvent;\n    "formdata": FormDataEvent;\n    "gotpointercapture": PointerEvent;\n    "input": Event;\n    "invalid": Event;\n    "keydown": KeyboardEvent;\n    "keypress": KeyboardEvent;\n    "keyup": KeyboardEvent;\n    "load": Event;\n    "loadeddata": Event;\n    "loadedmetadata": Event;\n    "loadstart": Event;\n    "lostpointercapture": PointerEvent;\n    "mousedown": MouseEvent;\n    "mouseenter": MouseEvent;\n    "mouseleave": MouseEvent;\n    "mousemove": MouseEvent;\n    "mouseout": MouseEvent;\n    "mouseover": MouseEvent;\n    "mouseup": MouseEvent;\n    "pause": Event;\n    "play": Event;\n    "playing": Event;\n    "pointercancel": PointerEvent;\n    "pointerdown": PointerEvent;\n    "pointerenter": PointerEvent;\n    "pointerleave": PointerEvent;\n    "pointermove": PointerEvent;\n    "pointerout": PointerEvent;\n    "pointerover": PointerEvent;\n    "pointerup": PointerEvent;\n    "progress": ProgressEvent;\n    "ratechange": Event;\n    "reset": Event;\n    "resize": UIEvent;\n    "scroll": Event;\n    "securitypolicyviolation": SecurityPolicyViolationEvent;\n    "seeked": Event;\n    "seeking": Event;\n    "select": Event;\n    "selectionchange": Event;\n    "selectstart": Event;\n    "stalled": Event;\n    "submit": Event;\n    "suspend": Event;\n    "timeupdate": Event;\n    "toggle": Event;\n    "touchcancel": TouchEvent;\n    "touchend": TouchEvent;\n    "touchmove": TouchEvent;\n    "touchstart": TouchEvent;\n    "transitioncancel": TransitionEvent;\n    "transitionend": TransitionEvent;\n    "transitionrun": TransitionEvent;\n    "transitionstart": TransitionEvent;\n    "volumechange": Event;\n    "waiting": Event;\n    "webkitanimationend": Event;\n    "webkitanimationiteration": Event;\n    "webkitanimationstart": Event;\n    "webkittransitionend": Event;\n    "wheel": WheelEvent;\n}\n\ninterface GlobalEventHandlers {\n    /**\n     * Fires when the user aborts the download.\n     * @param ev The event.\n     */\n    onabort: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\n    onanimationcancel: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\n    onanimationend: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\n    onanimationiteration: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\n    onanimationstart: ((this: GlobalEventHandlers, ev: AnimationEvent) => any) | null;\n    onauxclick: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    /**\n     * Fires when the object loses the input focus.\n     * @param ev The focus event.\n     */\n    onblur: ((this: GlobalEventHandlers, ev: FocusEvent) => any) | null;\n    /**\n     * Occurs when playback is possible, but would require further buffering.\n     * @param ev The event.\n     */\n    oncanplay: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    oncanplaythrough: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Fires when the contents of the object or selection have changed.\n     * @param ev The event.\n     */\n    onchange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Fires when the user clicks the left mouse button on the object\n     * @param ev The mouse event.\n     */\n    onclick: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    onclose: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Fires when the user clicks the right mouse button in the client area, opening the context menu.\n     * @param ev The mouse event.\n     */\n    oncontextmenu: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    oncuechange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Fires when the user double-clicks the object.\n     * @param ev The mouse event.\n     */\n    ondblclick: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    /**\n     * Fires on the source object continuously during a drag operation.\n     * @param ev The event.\n     */\n    ondrag: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\n    /**\n     * Fires on the source object when the user releases the mouse at the close of a drag operation.\n     * @param ev The event.\n     */\n    ondragend: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\n    /**\n     * Fires on the target element when the user drags the object to a valid drop target.\n     * @param ev The drag event.\n     */\n    ondragenter: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\n    /**\n     * Fires on the target object when the user moves the mouse out of a valid drop target during a drag operation.\n     * @param ev The drag event.\n     */\n    ondragleave: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\n    /**\n     * Fires on the target element continuously while the user drags the object over a valid drop target.\n     * @param ev The event.\n     */\n    ondragover: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\n    /**\n     * Fires on the source object when the user starts to drag a text selection or selected object.\n     * @param ev The event.\n     */\n    ondragstart: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\n    ondrop: ((this: GlobalEventHandlers, ev: DragEvent) => any) | null;\n    /**\n     * Occurs when the duration attribute is updated.\n     * @param ev The event.\n     */\n    ondurationchange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when the media element is reset to its initial state.\n     * @param ev The event.\n     */\n    onemptied: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when the end of playback is reached.\n     * @param ev The event\n     */\n    onended: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Fires when an error occurs during object loading.\n     * @param ev The event.\n     */\n    onerror: OnErrorEventHandler;\n    /**\n     * Fires when the object receives focus.\n     * @param ev The event.\n     */\n    onfocus: ((this: GlobalEventHandlers, ev: FocusEvent) => any) | null;\n    onformdata: ((this: GlobalEventHandlers, ev: FormDataEvent) => any) | null;\n    ongotpointercapture: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    oninput: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    oninvalid: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Fires when the user presses a key.\n     * @param ev The keyboard event\n     */\n    onkeydown: ((this: GlobalEventHandlers, ev: KeyboardEvent) => any) | null;\n    /**\n     * Fires when the user presses an alphanumeric key.\n     * @param ev The event.\n     * @deprecated\n     */\n    onkeypress: ((this: GlobalEventHandlers, ev: KeyboardEvent) => any) | null;\n    /**\n     * Fires when the user releases a key.\n     * @param ev The keyboard event\n     */\n    onkeyup: ((this: GlobalEventHandlers, ev: KeyboardEvent) => any) | null;\n    /**\n     * Fires immediately after the browser loads the object.\n     * @param ev The event.\n     */\n    onload: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when media data is loaded at the current playback position.\n     * @param ev The event.\n     */\n    onloadeddata: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when the duration and dimensions of the media have been determined.\n     * @param ev The event.\n     */\n    onloadedmetadata: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when Internet Explorer begins looking for media data.\n     * @param ev The event.\n     */\n    onloadstart: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onlostpointercapture: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    /**\n     * Fires when the user clicks the object with either mouse button.\n     * @param ev The mouse event.\n     */\n    onmousedown: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    onmouseenter: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    onmouseleave: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    /**\n     * Fires when the user moves the mouse over the object.\n     * @param ev The mouse event.\n     */\n    onmousemove: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    /**\n     * Fires when the user moves the mouse pointer outside the boundaries of the object.\n     * @param ev The mouse event.\n     */\n    onmouseout: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    /**\n     * Fires when the user moves the mouse pointer into the object.\n     * @param ev The mouse event.\n     */\n    onmouseover: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    /**\n     * Fires when the user releases a mouse button while the mouse is over the object.\n     * @param ev The mouse event.\n     */\n    onmouseup: ((this: GlobalEventHandlers, ev: MouseEvent) => any) | null;\n    /**\n     * Occurs when playback is paused.\n     * @param ev The event.\n     */\n    onpause: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when the play method is requested.\n     * @param ev The event.\n     */\n    onplay: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when the audio or video has started playing.\n     * @param ev The event.\n     */\n    onplaying: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onpointercancel: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    onpointerdown: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    onpointerenter: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    onpointerleave: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    onpointermove: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    onpointerout: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    onpointerover: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    onpointerup: ((this: GlobalEventHandlers, ev: PointerEvent) => any) | null;\n    /**\n     * Occurs to indicate progress while downloading media data.\n     * @param ev The event.\n     */\n    onprogress: ((this: GlobalEventHandlers, ev: ProgressEvent) => any) | null;\n    /**\n     * Occurs when the playback rate is increased or decreased.\n     * @param ev The event.\n     */\n    onratechange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Fires when the user resets a form.\n     * @param ev The event.\n     */\n    onreset: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onresize: ((this: GlobalEventHandlers, ev: UIEvent) => any) | null;\n    /**\n     * Fires when the user repositions the scroll box in the scroll bar on the object.\n     * @param ev The event.\n     */\n    onscroll: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when the seek operation ends.\n     * @param ev The event.\n     */\n    onseeked: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when the current playback position is moved.\n     * @param ev The event.\n     */\n    onseeking: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Fires when the current selection changes.\n     * @param ev The event.\n     */\n    onselect: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onselectionchange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onselectstart: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when the download has stopped.\n     * @param ev The event.\n     */\n    onstalled: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onsubmit: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs if the load operation has been intentionally halted.\n     * @param ev The event.\n     */\n    onsuspend: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs to indicate the current playback position.\n     * @param ev The event.\n     */\n    ontimeupdate: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    ontoggle: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    ontouchcancel?: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null | undefined;\n    ontouchend?: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null | undefined;\n    ontouchmove?: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null | undefined;\n    ontouchstart?: ((this: GlobalEventHandlers, ev: TouchEvent) => any) | null | undefined;\n    ontransitioncancel: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\n    ontransitionend: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\n    ontransitionrun: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\n    ontransitionstart: ((this: GlobalEventHandlers, ev: TransitionEvent) => any) | null;\n    /**\n     * Occurs when the volume is changed, or playback is muted or unmuted.\n     * @param ev The event.\n     */\n    onvolumechange: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    /**\n     * Occurs when playback stops because the next frame of a video resource is not available.\n     * @param ev The event.\n     */\n    onwaiting: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onwebkitanimationend: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onwebkitanimationiteration: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onwebkitanimationstart: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onwebkittransitionend: ((this: GlobalEventHandlers, ev: Event) => any) | null;\n    onwheel: ((this: GlobalEventHandlers, ev: WheelEvent) => any) | null;\n    addEventListener<K extends keyof GlobalEventHandlersEventMap>(type: K, listener: (this: GlobalEventHandlers, ev: GlobalEventHandlersEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof GlobalEventHandlersEventMap>(type: K, listener: (this: GlobalEventHandlers, ev: GlobalEventHandlersEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ninterface HTMLAllCollection {\n    /**\n     * Returns the number of elements in the collection.\n     */\n    readonly length: number;\n    /**\n     * Returns the item with index index from the collection (determined by tree order).\n     */\n    item(nameOrIndex?: string): HTMLCollection | Element | null;\n    /**\n     * Returns the item with ID or name name from the collection.\n     *\n     * If there are multiple matching items, then an HTMLCollection object containing all those elements is returned.\n     *\n     * Only button, form, iframe, input, map, meta, object, select, and textarea elements can have a name for the purpose of this method; their name is given by the value of their name attribute.\n     */\n    namedItem(name: string): HTMLCollection | Element | null;\n    [index: number]: Element;\n}\n\ndeclare var HTMLAllCollection: {\n    prototype: HTMLAllCollection;\n    new(): HTMLAllCollection;\n};\n\n/** Hyperlink elements and provides special properties and methods (beyond those of the regular HTMLElement object interface that they inherit from) for manipulating the layout and presentation of such elements. */\ninterface HTMLAnchorElement extends HTMLElement, HTMLHyperlinkElementUtils {\n    /**\n     * Sets or retrieves the character set used to encode the object.\n     * @deprecated\n     */\n    charset: string;\n    /**\n     * Sets or retrieves the coordinates of the object.\n     * @deprecated\n     */\n    coords: string;\n    download: string;\n    /**\n     * Sets or retrieves the language code of the object.\n     */\n    hreflang: string;\n    /**\n     * Sets or retrieves the shape of the object.\n     * @deprecated\n     */\n    name: string;\n    ping: string;\n    referrerPolicy: string;\n    /**\n     * Sets or retrieves the relationship between the object and the destination of the link.\n     */\n    rel: string;\n    readonly relList: DOMTokenList;\n    /**\n     * Sets or retrieves the relationship between the object and the destination of the link.\n     * @deprecated\n     */\n    rev: string;\n    /**\n     * Sets or retrieves the shape of the object.\n     * @deprecated\n     */\n    shape: string;\n    /**\n     * Sets or retrieves the window or frame at which to target content.\n     */\n    target: string;\n    /**\n     * Retrieves or sets the text of the object as a string.\n     */\n    text: string;\n    type: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAnchorElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAnchorElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLAnchorElement: {\n    prototype: HTMLAnchorElement;\n    new(): HTMLAnchorElement;\n};\n\n/** Provides special properties and methods (beyond those of the regular object HTMLElement interface it also has available to it by inheritance) for manipulating the layout and presentation of <area> elements. */\ninterface HTMLAreaElement extends HTMLElement, HTMLHyperlinkElementUtils {\n    /**\n     * Sets or retrieves a text alternative to the graphic.\n     */\n    alt: string;\n    /**\n     * Sets or retrieves the coordinates of the object.\n     */\n    coords: string;\n    download: string;\n    /**\n     * Sets or gets whether clicks in this region cause action.\n     * @deprecated\n     */\n    noHref: boolean;\n    ping: string;\n    referrerPolicy: string;\n    rel: string;\n    readonly relList: DOMTokenList;\n    /**\n     * Sets or retrieves the shape of the object.\n     */\n    shape: string;\n    /**\n     * Sets or retrieves the window or frame at which to target content.\n     */\n    target: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLAreaElement: {\n    prototype: HTMLAreaElement;\n    new(): HTMLAreaElement;\n};\n\n/** Provides access to the properties of <audio> elements, as well as methods to manipulate them. It derives from the HTMLMediaElement interface. */\ninterface HTMLAudioElement extends HTMLMediaElement {\n    addEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLAudioElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLAudioElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLAudioElement: {\n    prototype: HTMLAudioElement;\n    new(): HTMLAudioElement;\n};\n\n/** A HTML line break element (<br>). It inherits from HTMLElement. */\ninterface HTMLBRElement extends HTMLElement {\n    /**\n     * Sets or retrieves the side on which floating objects are not to be positioned when any IHTMLBlockElement is inserted into the document.\n     * @deprecated\n     */\n    clear: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLBRElement: {\n    prototype: HTMLBRElement;\n    new(): HTMLBRElement;\n};\n\n/** Contains the base URI for a document. This object inherits all of the properties and methods as described in the HTMLElement interface. */\ninterface HTMLBaseElement extends HTMLElement {\n    /**\n     * Gets or sets the baseline URL on which relative links are based.\n     */\n    href: string;\n    /**\n     * Sets or retrieves the window or frame at which to target content.\n     */\n    target: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLBaseElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLBaseElement: {\n    prototype: HTMLBaseElement;\n    new(): HTMLBaseElement;\n};\n\ninterface HTMLBodyElementEventMap extends HTMLElementEventMap, WindowEventHandlersEventMap {\n    "orientationchange": Event;\n}\n\n/** Provides special properties (beyond those inherited from the regular HTMLElement interface) for manipulating <body> elements. */\ninterface HTMLBodyElement extends HTMLElement, WindowEventHandlers {\n    /** @deprecated */\n    aLink: string;\n    /** @deprecated */\n    background: string;\n    /** @deprecated */\n    bgColor: string;\n    /** @deprecated */\n    link: string;\n    /** @deprecated */\n    onorientationchange: ((this: HTMLBodyElement, ev: Event) => any) | null;\n    /** @deprecated */\n    text: string;\n    /** @deprecated */\n    vLink: string;\n    addEventListener<K extends keyof HTMLBodyElementEventMap>(type: K, listener: (this: HTMLBodyElement, ev: HTMLBodyElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLBodyElementEventMap>(type: K, listener: (this: HTMLBodyElement, ev: HTMLBodyElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLBodyElement: {\n    prototype: HTMLBodyElement;\n    new(): HTMLBodyElement;\n};\n\n/** Provides properties and methods (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating <button> elements. */\ninterface HTMLButtonElement extends HTMLElement {\n    disabled: boolean;\n    /**\n     * Retrieves a reference to the form that the object is embedded in.\n     */\n    readonly form: HTMLFormElement | null;\n    /**\n     * Overrides the action attribute (where the data on a form is sent) on the parent form element.\n     */\n    formAction: string;\n    /**\n     * Used to override the encoding (formEnctype attribute) specified on the form element.\n     */\n    formEnctype: string;\n    /**\n     * Overrides the submit method attribute previously specified on a form element.\n     */\n    formMethod: string;\n    /**\n     * Overrides any validation or required attributes on a form or form elements to allow it to be submitted without validation. This can be used to create a "save draft"-type submit option.\n     */\n    formNoValidate: boolean;\n    /**\n     * Overrides the target attribute on a form element.\n     */\n    formTarget: string;\n    readonly labels: NodeListOf<HTMLLabelElement>;\n    /**\n     * Sets or retrieves the name of the object.\n     */\n    name: string;\n    /**\n     * Gets the classification and default behavior of the button.\n     */\n    type: string;\n    /**\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as "this is a required field". The result is that the user sees validation messages without actually submitting.\n     */\n    readonly validationMessage: string;\n    /**\n     * Returns a  ValidityState object that represents the validity states of an element.\n     */\n    readonly validity: ValidityState;\n    /**\n     * Sets or retrieves the default or selected value of the control.\n     */\n    value: string;\n    /**\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\n     */\n    readonly willValidate: boolean;\n    /**\n     * Returns whether a form will validate when it is submitted, without having to submit it.\n     */\n    checkValidity(): boolean;\n    reportValidity(): boolean;\n    /**\n     * Sets a custom error message that is displayed when a form is submitted.\n     * @param error Sets a custom error message that is displayed when a form is submitted.\n     */\n    setCustomValidity(error: string): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLButtonElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLButtonElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLButtonElement: {\n    prototype: HTMLButtonElement;\n    new(): HTMLButtonElement;\n};\n\n/** Provides properties and methods for manipulating the layout and presentation of <canvas> elements. The HTMLCanvasElement interface also inherits the properties and methods of the HTMLElement interface. */\ninterface HTMLCanvasElement extends HTMLElement {\n    /**\n     * Gets or sets the height of a canvas element on a document.\n     */\n    height: number;\n    /**\n     * Gets or sets the width of a canvas element on a document.\n     */\n    width: number;\n    captureStream(frameRequestRate?: number): MediaStream;\n    /**\n     * Returns an object that provides methods and properties for drawing and manipulating images and graphics on a canvas element in a document. A context object includes information about colors, line widths, fonts, and other graphic parameters that can be drawn on a canvas.\n     * @param contextId The identifier (ID) of the type of canvas to create. Internet Explorer 9 and Internet Explorer 10 support only a 2-D context using canvas.getContext("2d"); IE11 Preview also supports 3-D or WebGL context using canvas.getContext("experimental-webgl");\n     */\n    getContext(contextId: "2d", options?: CanvasRenderingContext2DSettings): CanvasRenderingContext2D | null;\n    getContext(contextId: "bitmaprenderer", options?: ImageBitmapRenderingContextSettings): ImageBitmapRenderingContext | null;\n    getContext(contextId: "webgl", options?: WebGLContextAttributes): WebGLRenderingContext | null;\n    getContext(contextId: "webgl2", options?: WebGLContextAttributes): WebGL2RenderingContext | null;\n    getContext(contextId: string, options?: any): RenderingContext | null;\n    toBlob(callback: BlobCallback, type?: string, quality?: any): void;\n    /**\n     * Returns the content of the current canvas as an image that you can use as a source for another canvas or an HTML element.\n     * @param type The standard MIME type for the image format to return. If you do not specify this parameter, the default value is a PNG format image.\n     */\n    toDataURL(type?: string, quality?: any): string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLCanvasElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLCanvasElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLCanvasElement: {\n    prototype: HTMLCanvasElement;\n    new(): HTMLCanvasElement;\n};\n\n/** A generic collection (array-like object similar to arguments) of elements (in document order) and offers methods and properties for selecting from the list. */\ninterface HTMLCollectionBase {\n    /**\n     * Sets or retrieves the number of objects in a collection.\n     */\n    readonly length: number;\n    /**\n     * Retrieves an object from various collections.\n     */\n    item(index: number): Element | null;\n    [index: number]: Element;\n}\n\ninterface HTMLCollection extends HTMLCollectionBase {\n    /**\n     * Retrieves a select object or an object from an options collection.\n     */\n    namedItem(name: string): Element | null;\n}\n\ndeclare var HTMLCollection: {\n    prototype: HTMLCollection;\n    new(): HTMLCollection;\n};\n\ninterface HTMLCollectionOf<T extends Element> extends HTMLCollectionBase {\n    item(index: number): T | null;\n    namedItem(name: string): T | null;\n    [index: number]: T;\n}\n\n/** Provides special properties (beyond those of the regular HTMLElement interface it also has available to it by inheritance) for manipulating definition list (<dl>) elements. */\ninterface HTMLDListElement extends HTMLElement {\n    /** @deprecated */\n    compact: boolean;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLDListElement: {\n    prototype: HTMLDListElement;\n    new(): HTMLDListElement;\n};\n\n/** Provides special properties (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating <data> elements. */\ninterface HTMLDataElement extends HTMLElement {\n    value: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLDataElement: {\n    prototype: HTMLDataElement;\n    new(): HTMLDataElement;\n};\n\n/** Provides special properties (beyond the HTMLElement object interface it also has available to it by inheritance) to manipulate <datalist> elements and their content. */\ninterface HTMLDataListElement extends HTMLElement {\n    /**\n     * Returns an HTMLCollection of the option elements of the datalist element.\n     */\n    readonly options: HTMLCollectionOf<HTMLOptionElement>;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDataListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLDataListElement: {\n    prototype: HTMLDataListElement;\n    new(): HTMLDataListElement;\n};\n\ninterface HTMLDetailsElement extends HTMLElement {\n    open: boolean;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDetailsElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDetailsElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLDetailsElement: {\n    prototype: HTMLDetailsElement;\n    new(): HTMLDetailsElement;\n};\n\n/** @deprecated this is not available in most browsers */\ninterface HTMLDialogElement extends HTMLElement {\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDialogElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDialogElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** @deprecated */\ninterface HTMLDirectoryElement extends HTMLElement {\n    /** @deprecated */\n    compact: boolean;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDirectoryElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDirectoryElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** @deprecated */\ndeclare var HTMLDirectoryElement: {\n    prototype: HTMLDirectoryElement;\n    new(): HTMLDirectoryElement;\n};\n\n/** Provides special properties (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating <div> elements. */\ninterface HTMLDivElement extends HTMLElement {\n    /**\n     * Sets or retrieves how the object is aligned with adjacent text.\n     * @deprecated\n     */\n    align: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDivElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLDivElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLDivElement: {\n    prototype: HTMLDivElement;\n    new(): HTMLDivElement;\n};\n\n/** @deprecated use Document */\ninterface HTMLDocument extends Document {\n    addEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: HTMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: HTMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** @deprecated */\ndeclare var HTMLDocument: {\n    prototype: HTMLDocument;\n    new(): HTMLDocument;\n};\n\ninterface HTMLElementEventMap extends ElementEventMap, DocumentAndElementEventHandlersEventMap, GlobalEventHandlersEventMap {\n}\n\n/** Any HTML element. Some elements directly implement this interface, while others implement it via an interface that inherits it. */\ninterface HTMLElement extends Element, DocumentAndElementEventHandlers, ElementCSSInlineStyle, ElementCSSInlineStyle, ElementContentEditable, GlobalEventHandlers, HTMLOrSVGElement {\n    accessKey: string;\n    readonly accessKeyLabel: string;\n    autocapitalize: string;\n    dir: string;\n    draggable: boolean;\n    hidden: boolean;\n    innerText: string;\n    lang: string;\n    readonly offsetHeight: number;\n    readonly offsetLeft: number;\n    readonly offsetParent: Element | null;\n    readonly offsetTop: number;\n    readonly offsetWidth: number;\n    outerText: string;\n    spellcheck: boolean;\n    title: string;\n    translate: boolean;\n    click(): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLElement: {\n    prototype: HTMLElement;\n    new(): HTMLElement;\n};\n\n/** Provides special properties (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating <embed> elements. */\ninterface HTMLEmbedElement extends HTMLElement {\n    /** @deprecated */\n    align: string;\n    /**\n     * Sets or retrieves the height of the object.\n     */\n    height: string;\n    /**\n     * Sets or retrieves the name of the object.\n     * @deprecated\n     */\n    name: string;\n    /**\n     * Sets or retrieves a URL to be loaded by the object.\n     */\n    src: string;\n    type: string;\n    /**\n     * Sets or retrieves the width of the object.\n     */\n    width: string;\n    getSVGDocument(): Document | null;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLEmbedElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLEmbedElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLEmbedElement: {\n    prototype: HTMLEmbedElement;\n    new(): HTMLEmbedElement;\n};\n\n/** Provides special properties and methods (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating the layout and presentation of <fieldset> elements. */\ninterface HTMLFieldSetElement extends HTMLElement {\n    disabled: boolean;\n    /**\n     * Returns an HTMLCollection of the form controls in the element.\n     */\n    readonly elements: HTMLCollection;\n    /**\n     * Retrieves a reference to the form that the object is embedded in.\n     */\n    readonly form: HTMLFormElement | null;\n    name: string;\n    /**\n     * Returns the string "fieldset".\n     */\n    readonly type: string;\n    /**\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as "this is a required field". The result is that the user sees validation messages without actually submitting.\n     */\n    readonly validationMessage: string;\n    /**\n     * Returns a  ValidityState object that represents the validity states of an element.\n     */\n    readonly validity: ValidityState;\n    /**\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\n     */\n    readonly willValidate: boolean;\n    /**\n     * Returns whether a form will validate when it is submitted, without having to submit it.\n     */\n    checkValidity(): boolean;\n    reportValidity(): boolean;\n    /**\n     * Sets a custom error message that is displayed when a form is submitted.\n     * @param error Sets a custom error message that is displayed when a form is submitted.\n     */\n    setCustomValidity(error: string): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFieldSetElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFieldSetElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLFieldSetElement: {\n    prototype: HTMLFieldSetElement;\n    new(): HTMLFieldSetElement;\n};\n\n/**\n * Implements the document object model (DOM) representation of the font element. The HTML Font Element <font> defines the font size, font face and color of text.\n * @deprecated\n */\ninterface HTMLFontElement extends HTMLElement {\n    /** @deprecated */\n    color: string;\n    /**\n     * Sets or retrieves the current typeface family.\n     * @deprecated\n     */\n    face: string;\n    /** @deprecated */\n    size: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFontElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** @deprecated */\ndeclare var HTMLFontElement: {\n    prototype: HTMLFontElement;\n    new(): HTMLFontElement;\n};\n\n/** A collection of HTML form control elements.  */\ninterface HTMLFormControlsCollection extends HTMLCollectionBase {\n    /**\n     * Returns the item with ID or name name from the collection.\n     *\n     * If there are multiple matching items, then a RadioNodeList object containing all those elements is returned.\n     */\n    namedItem(name: string): RadioNodeList | Element | null;\n}\n\ndeclare var HTMLFormControlsCollection: {\n    prototype: HTMLFormControlsCollection;\n    new(): HTMLFormControlsCollection;\n};\n\n/** A <form> element in the DOM; it allows access to and in some cases modification of aspects of the form, as well as access to its component elements. */\ninterface HTMLFormElement extends HTMLElement {\n    /**\n     * Sets or retrieves a list of character encodings for input data that must be accepted by the server processing the form.\n     */\n    acceptCharset: string;\n    /**\n     * Sets or retrieves the URL to which the form content is sent for processing.\n     */\n    action: string;\n    /**\n     * Specifies whether autocomplete is applied to an editable text field.\n     */\n    autocomplete: string;\n    /**\n     * Retrieves a collection, in source order, of all controls in a given form.\n     */\n    readonly elements: HTMLFormControlsCollection;\n    /**\n     * Sets or retrieves the MIME encoding for the form.\n     */\n    encoding: string;\n    /**\n     * Sets or retrieves the encoding type for the form.\n     */\n    enctype: string;\n    /**\n     * Sets or retrieves the number of objects in a collection.\n     */\n    readonly length: number;\n    /**\n     * Sets or retrieves how to send the form data to the server.\n     */\n    method: string;\n    /**\n     * Sets or retrieves the name of the object.\n     */\n    name: string;\n    /**\n     * Designates a form that is not validated when submitted.\n     */\n    noValidate: boolean;\n    /**\n     * Sets or retrieves the window or frame at which to target content.\n     */\n    target: string;\n    /**\n     * Returns whether a form will validate when it is submitted, without having to submit it.\n     */\n    checkValidity(): boolean;\n    reportValidity(): boolean;\n    requestSubmit(submitter?: HTMLElement | null): void;\n    /**\n     * Fires when the user resets a form.\n     */\n    reset(): void;\n    /**\n     * Fires when a FORM is about to be submitted.\n     */\n    submit(): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFormElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFormElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n    [index: number]: Element;\n    [name: string]: any;\n}\n\ndeclare var HTMLFormElement: {\n    prototype: HTMLFormElement;\n    new(): HTMLFormElement;\n};\n\n/** @deprecated */\ninterface HTMLFrameElement extends HTMLElement {\n    /**\n     * Retrieves the document object of the page or frame.\n     * @deprecated\n     */\n    readonly contentDocument: Document | null;\n    /**\n     * Retrieves the object of the specified.\n     * @deprecated\n     */\n    readonly contentWindow: WindowProxy | null;\n    /**\n     * Sets or retrieves whether to display a border for the frame.\n     * @deprecated\n     */\n    frameBorder: string;\n    /**\n     * Sets or retrieves a URI to a long description of the object.\n     * @deprecated\n     */\n    longDesc: string;\n    /**\n     * Sets or retrieves the top and bottom margin heights before displaying the text in a frame.\n     * @deprecated\n     */\n    marginHeight: string;\n    /**\n     * Sets or retrieves the left and right margin widths before displaying the text in a frame.\n     * @deprecated\n     */\n    marginWidth: string;\n    /**\n     * Sets or retrieves the frame name.\n     * @deprecated\n     */\n    name: string;\n    /**\n     * Sets or retrieves whether the user can resize the frame.\n     * @deprecated\n     */\n    noResize: boolean;\n    /**\n     * Sets or retrieves whether the frame can be scrolled.\n     * @deprecated\n     */\n    scrolling: string;\n    /**\n     * Sets or retrieves a URL to be loaded by the object.\n     * @deprecated\n     */\n    src: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** @deprecated */\ndeclare var HTMLFrameElement: {\n    prototype: HTMLFrameElement;\n    new(): HTMLFrameElement;\n};\n\ninterface HTMLFrameSetElementEventMap extends HTMLElementEventMap, WindowEventHandlersEventMap {\n}\n\n/**\n * Provides special properties (beyond those of the regular HTMLElement interface they also inherit) for manipulating <frameset> elements.\n * @deprecated\n */\ninterface HTMLFrameSetElement extends HTMLElement, WindowEventHandlers {\n    /**\n     * Sets or retrieves the frame widths of the object.\n     * @deprecated\n     */\n    cols: string;\n    /**\n     * Sets or retrieves the frame heights of the object.\n     * @deprecated\n     */\n    rows: string;\n    addEventListener<K extends keyof HTMLFrameSetElementEventMap>(type: K, listener: (this: HTMLFrameSetElement, ev: HTMLFrameSetElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLFrameSetElementEventMap>(type: K, listener: (this: HTMLFrameSetElement, ev: HTMLFrameSetElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** @deprecated */\ndeclare var HTMLFrameSetElement: {\n    prototype: HTMLFrameSetElement;\n    new(): HTMLFrameSetElement;\n};\n\n/** Provides special properties (beyond those of the HTMLElement interface it also has available to it by inheritance) for manipulating <hr> elements. */\ninterface HTMLHRElement extends HTMLElement {\n    /**\n     * Sets or retrieves how the object is aligned with adjacent text.\n     * @deprecated\n     */\n    align: string;\n    /** @deprecated */\n    color: string;\n    /**\n     * Sets or retrieves whether the horizontal rule is drawn with 3-D shading.\n     * @deprecated\n     */\n    noShade: boolean;\n    /** @deprecated */\n    size: string;\n    /**\n     * Sets or retrieves the width of the object.\n     * @deprecated\n     */\n    width: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHRElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLHRElement: {\n    prototype: HTMLHRElement;\n    new(): HTMLHRElement;\n};\n\n/** Contains the descriptive information, or metadata, for a document. This object inherits all of the properties and methods described in the HTMLElement interface. */\ninterface HTMLHeadElement extends HTMLElement {\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLHeadElement: {\n    prototype: HTMLHeadElement;\n    new(): HTMLHeadElement;\n};\n\n/** The different heading elements. It inherits methods and properties from the HTMLElement interface. */\ninterface HTMLHeadingElement extends HTMLElement {\n    /**\n     * Sets or retrieves a value that indicates the table alignment.\n     * @deprecated\n     */\n    align: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadingElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHeadingElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLHeadingElement: {\n    prototype: HTMLHeadingElement;\n    new(): HTMLHeadingElement;\n};\n\n/** Serves as the root node for a given HTML document. This object inherits the properties and methods described in the HTMLElement interface. */\ninterface HTMLHtmlElement extends HTMLElement {\n    /**\n     * Sets or retrieves the DTD version that governs the current document.\n     * @deprecated\n     */\n    version: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHtmlElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLHtmlElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLHtmlElement: {\n    prototype: HTMLHtmlElement;\n    new(): HTMLHtmlElement;\n};\n\ninterface HTMLHyperlinkElementUtils {\n    /**\n     * Returns the hyperlink\'s URL\'s fragment (includes leading "#" if non-empty).\n     *\n     * Can be set, to change the URL\'s fragment (ignores leading "#").\n     */\n    hash: string;\n    /**\n     * Returns the hyperlink\'s URL\'s host and port (if different from the default port for the scheme).\n     *\n     * Can be set, to change the URL\'s host and port.\n     */\n    host: string;\n    /**\n     * Returns the hyperlink\'s URL\'s host.\n     *\n     * Can be set, to change the URL\'s host.\n     */\n    hostname: string;\n    /**\n     * Returns the hyperlink\'s URL.\n     *\n     * Can be set, to change the URL.\n     */\n    href: string;\n    toString(): string;\n    /**\n     * Returns the hyperlink\'s URL\'s origin.\n     */\n    readonly origin: string;\n    /**\n     * Returns the hyperlink\'s URL\'s password.\n     *\n     * Can be set, to change the URL\'s password.\n     */\n    password: string;\n    /**\n     * Returns the hyperlink\'s URL\'s path.\n     *\n     * Can be set, to change the URL\'s path.\n     */\n    pathname: string;\n    /**\n     * Returns the hyperlink\'s URL\'s port.\n     *\n     * Can be set, to change the URL\'s port.\n     */\n    port: string;\n    /**\n     * Returns the hyperlink\'s URL\'s scheme.\n     *\n     * Can be set, to change the URL\'s scheme.\n     */\n    protocol: string;\n    /**\n     * Returns the hyperlink\'s URL\'s query (includes leading "?" if non-empty).\n     *\n     * Can be set, to change the URL\'s query (ignores leading "?").\n     */\n    search: string;\n    /**\n     * Returns the hyperlink\'s URL\'s username.\n     *\n     * Can be set, to change the URL\'s username.\n     */\n    username: string;\n}\n\n/** Provides special properties and methods (beyond those of the HTMLElement interface it also has available to it by inheritance) for manipulating the layout and presentation of inline frame elements. */\ninterface HTMLIFrameElement extends HTMLElement {\n    /**\n     * Sets or retrieves how the object is aligned with adjacent text.\n     * @deprecated\n     */\n    align: string;\n    allow: string;\n    allowFullscreen: boolean;\n    /**\n     * Retrieves the document object of the page or frame.\n     */\n    readonly contentDocument: Document | null;\n    /**\n     * Retrieves the object of the specified.\n     */\n    readonly contentWindow: WindowProxy | null;\n    /**\n     * Sets or retrieves whether to display a border for the frame.\n     * @deprecated\n     */\n    frameBorder: string;\n    /**\n     * Sets or retrieves the height of the object.\n     */\n    height: string;\n    /**\n     * Sets or retrieves a URI to a long description of the object.\n     * @deprecated\n     */\n    longDesc: string;\n    /**\n     * Sets or retrieves the top and bottom margin heights before displaying the text in a frame.\n     * @deprecated\n     */\n    marginHeight: string;\n    /**\n     * Sets or retrieves the left and right margin widths before displaying the text in a frame.\n     * @deprecated\n     */\n    marginWidth: string;\n    /**\n     * Sets or retrieves the frame name.\n     */\n    name: string;\n    referrerPolicy: ReferrerPolicy;\n    readonly sandbox: DOMTokenList;\n    /**\n     * Sets or retrieves whether the frame can be scrolled.\n     * @deprecated\n     */\n    scrolling: string;\n    /**\n     * Sets or retrieves a URL to be loaded by the object.\n     */\n    src: string;\n    /**\n     * Sets or retrives the content of the page that is to contain.\n     */\n    srcdoc: string;\n    /**\n     * Sets or retrieves the width of the object.\n     */\n    width: string;\n    getSVGDocument(): Document | null;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLIFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLIFrameElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLIFrameElement: {\n    prototype: HTMLIFrameElement;\n    new(): HTMLIFrameElement;\n};\n\n/** Provides special properties and methods for manipulating <img> elements. */\ninterface HTMLImageElement extends HTMLElement {\n    /**\n     * Sets or retrieves how the object is aligned with adjacent text.\n     * @deprecated\n     */\n    align: string;\n    /**\n     * Sets or retrieves a text alternative to the graphic.\n     */\n    alt: string;\n    /**\n     * Specifies the properties of a border drawn around an object.\n     * @deprecated\n     */\n    border: string;\n    /**\n     * Retrieves whether the object is fully loaded.\n     */\n    readonly complete: boolean;\n    crossOrigin: string | null;\n    readonly currentSrc: string;\n    decoding: "async" | "sync" | "auto";\n    /**\n     * Sets or retrieves the height of the object.\n     */\n    height: number;\n    /**\n     * Sets or retrieves the width of the border to draw around the object.\n     * @deprecated\n     */\n    hspace: number;\n    /**\n     * Sets or retrieves whether the image is a server-side image map.\n     */\n    isMap: boolean;\n    loading: string;\n    /**\n     * Sets or retrieves a Uniform Resource Identifier (URI) to a long description of the object.\n     * @deprecated\n     */\n    longDesc: string;\n    /** @deprecated */\n    lowsrc: string;\n    /**\n     * Sets or retrieves the name of the object.\n     * @deprecated\n     */\n    name: string;\n    /**\n     * The original height of the image resource before sizing.\n     */\n    readonly naturalHeight: number;\n    /**\n     * The original width of the image resource before sizing.\n     */\n    readonly naturalWidth: number;\n    referrerPolicy: string;\n    sizes: string;\n    /**\n     * The address or URL of the a media resource that is to be considered.\n     */\n    src: string;\n    srcset: string;\n    /**\n     * Sets or retrieves the URL, often with a bookmark extension (#name), to use as a client-side image map.\n     */\n    useMap: string;\n    /**\n     * Sets or retrieves the vertical margin for the object.\n     * @deprecated\n     */\n    vspace: number;\n    /**\n     * Sets or retrieves the width of the object.\n     */\n    width: number;\n    readonly x: number;\n    readonly y: number;\n    decode(): Promise<void>;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLImageElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLImageElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLImageElement: {\n    prototype: HTMLImageElement;\n    new(): HTMLImageElement;\n};\n\n/** Provides special properties and methods for manipulating the options, layout, and presentation of <input> elements. */\ninterface HTMLInputElement extends HTMLElement {\n    /**\n     * Sets or retrieves a comma-separated list of content types.\n     */\n    accept: string;\n    /**\n     * Sets or retrieves how the object is aligned with adjacent text.\n     * @deprecated\n     */\n    align: string;\n    /**\n     * Sets or retrieves a text alternative to the graphic.\n     */\n    alt: string;\n    /**\n     * Specifies whether autocomplete is applied to an editable text field.\n     */\n    autocomplete: string;\n    capture: string;\n    /**\n     * Sets or retrieves the state of the check box or radio button.\n     */\n    checked: boolean;\n    /**\n     * Sets or retrieves the state of the check box or radio button.\n     */\n    defaultChecked: boolean;\n    /**\n     * Sets or retrieves the initial contents of the object.\n     */\n    defaultValue: string;\n    dirName: string;\n    disabled: boolean;\n    /**\n     * Returns a FileList object on a file type input object.\n     */\n    files: FileList | null;\n    /**\n     * Retrieves a reference to the form that the object is embedded in.\n     */\n    readonly form: HTMLFormElement | null;\n    /**\n     * Overrides the action attribute (where the data on a form is sent) on the parent form element.\n     */\n    formAction: string;\n    /**\n     * Used to override the encoding (formEnctype attribute) specified on the form element.\n     */\n    formEnctype: string;\n    /**\n     * Overrides the submit method attribute previously specified on a form element.\n     */\n    formMethod: string;\n    /**\n     * Overrides any validation or required attributes on a form or form elements to allow it to be submitted without validation. This can be used to create a "save draft"-type submit option.\n     */\n    formNoValidate: boolean;\n    /**\n     * Overrides the target attribute on a form element.\n     */\n    formTarget: string;\n    /**\n     * Sets or retrieves the height of the object.\n     */\n    height: number;\n    /**\n     * When set, overrides the rendering of checkbox controls so that the current value is not visible.\n     */\n    indeterminate: boolean;\n    readonly labels: NodeListOf<HTMLLabelElement> | null;\n    /**\n     * Specifies the ID of a pre-defined datalist of options for an input element.\n     */\n    readonly list: HTMLElement | null;\n    /**\n     * Defines the maximum acceptable value for an input element with type="number".When used with the min and step attributes, lets you control the range and increment (such as only even numbers) that the user can enter into an input field.\n     */\n    max: string;\n    /**\n     * Sets or retrieves the maximum number of characters that the user can enter in a text control.\n     */\n    maxLength: number;\n    /**\n     * Defines the minimum acceptable value for an input element with type="number". When used with the max and step attributes, lets you control the range and increment (such as even numbers only) that the user can enter into an input field.\n     */\n    min: string;\n    minLength: number;\n    /**\n     * Sets or retrieves the Boolean value indicating whether multiple items can be selected from a list.\n     */\n    multiple: boolean;\n    /**\n     * Sets or retrieves the name of the object.\n     */\n    name: string;\n    /**\n     * Gets or sets a string containing a regular expression that the user\'s input must match.\n     */\n    pattern: string;\n    /**\n     * Gets or sets a text string that is displayed in an input field as a hint or prompt to users as the format or type of information they need to enter.The text appears in an input field until the user puts focus on the field.\n     */\n    placeholder: string;\n    readOnly: boolean;\n    /**\n     * When present, marks an element that can\'t be submitted without a value.\n     */\n    required: boolean;\n    selectionDirection: "forward" | "backward" | "none" | null;\n    /**\n     * Gets or sets the end position or offset of a text selection.\n     */\n    selectionEnd: number | null;\n    /**\n     * Gets or sets the starting position or offset of a text selection.\n     */\n    selectionStart: number | null;\n    size: number;\n    /**\n     * The address or URL of the a media resource that is to be considered.\n     */\n    src: string;\n    /**\n     * Defines an increment or jump between values that you want to allow the user to enter. When used with the max and min attributes, lets you control the range and increment (for example, allow only even numbers) that the user can enter into an input field.\n     */\n    step: string;\n    /**\n     * Returns the content type of the object.\n     */\n    type: string;\n    /**\n     * Sets or retrieves the URL, often with a bookmark extension (#name), to use as a client-side image map.\n     * @deprecated\n     */\n    useMap: string;\n    /**\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as "this is a required field". The result is that the user sees validation messages without actually submitting.\n     */\n    readonly validationMessage: string;\n    /**\n     * Returns a  ValidityState object that represents the validity states of an element.\n     */\n    readonly validity: ValidityState;\n    /**\n     * Returns the value of the data at the cursor\'s current position.\n     */\n    value: string;\n    /**\n     * Returns a Date object representing the form control\'s value, if applicable; otherwise, returns null. Can be set, to change the value. Throws an "InvalidStateError" DOMException if the control isn\'t date- or time-based.\n     */\n    valueAsDate: Date | null;\n    /**\n     * Returns the input field value as a number.\n     */\n    valueAsNumber: number;\n    readonly webkitEntries: ReadonlyArray<FileSystemEntry>;\n    webkitdirectory: boolean;\n    /**\n     * Sets or retrieves the width of the object.\n     */\n    width: number;\n    /**\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\n     */\n    readonly willValidate: boolean;\n    /**\n     * Returns whether a form will validate when it is submitted, without having to submit it.\n     */\n    checkValidity(): boolean;\n    reportValidity(): boolean;\n    /**\n     * Makes the selection equal to the current object.\n     */\n    select(): void;\n    /**\n     * Sets a custom error message that is displayed when a form is submitted.\n     * @param error Sets a custom error message that is displayed when a form is submitted.\n     */\n    setCustomValidity(error: string): void;\n    setRangeText(replacement: string): void;\n    setRangeText(replacement: string, start: number, end: number, selectionMode?: SelectionMode): void;\n    /**\n     * Sets the start and end positions of a selection in a text field.\n     * @param start The offset into the text field for the start of the selection.\n     * @param end The offset into the text field for the end of the selection.\n     * @param direction The direction in which the selection is performed.\n     */\n    setSelectionRange(start: number | null, end: number | null, direction?: "forward" | "backward" | "none"): void;\n    /**\n     * Decrements a range input control\'s value by the value given by the Step attribute. If the optional parameter is used, it will decrement the input control\'s step value multiplied by the parameter\'s value.\n     * @param n Value to decrement the value by.\n     */\n    stepDown(n?: number): void;\n    /**\n     * Increments a range input control\'s value by the value given by the Step attribute. If the optional parameter is used, will increment the input control\'s value by that value.\n     * @param n Value to increment the value by.\n     */\n    stepUp(n?: number): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLInputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLInputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLInputElement: {\n    prototype: HTMLInputElement;\n    new(): HTMLInputElement;\n};\n\n/** Exposes specific properties and methods (beyond those defined by regular HTMLElement interface it also has available to it by inheritance) for manipulating list elements. */\ninterface HTMLLIElement extends HTMLElement {\n    /** @deprecated */\n    type: string;\n    /**\n     * Sets or retrieves the value of a list item.\n     */\n    value: number;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLIElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLIElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLLIElement: {\n    prototype: HTMLLIElement;\n    new(): HTMLLIElement;\n};\n\n/** Gives access to properties specific to <label> elements. It inherits methods and properties from the base HTMLElement interface. */\ninterface HTMLLabelElement extends HTMLElement {\n    /**\n     * Returns the form control that is associated with this element.\n     */\n    readonly control: HTMLElement | null;\n    /**\n     * Retrieves a reference to the form that the object is embedded in.\n     */\n    readonly form: HTMLFormElement | null;\n    /**\n     * Sets or retrieves the object to which the given label object is assigned.\n     */\n    htmlFor: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLabelElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLabelElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLLabelElement: {\n    prototype: HTMLLabelElement;\n    new(): HTMLLabelElement;\n};\n\n/** The HTMLLegendElement is an interface allowing to access properties of the <legend> elements. It inherits properties and methods from the HTMLElement interface. */\ninterface HTMLLegendElement extends HTMLElement {\n    /** @deprecated */\n    align: string;\n    /**\n     * Retrieves a reference to the form that the object is embedded in.\n     */\n    readonly form: HTMLFormElement | null;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLegendElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLegendElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLLegendElement: {\n    prototype: HTMLLegendElement;\n    new(): HTMLLegendElement;\n};\n\n/** Reference information for external resources and the relationship of those resources to a document and vice-versa. This object inherits all of the properties and methods of the HTMLElement interface. */\ninterface HTMLLinkElement extends HTMLElement, LinkStyle {\n    as: string;\n    /**\n     * Sets or retrieves the character set used to encode the object.\n     * @deprecated\n     */\n    charset: string;\n    crossOrigin: string | null;\n    disabled: boolean;\n    /**\n     * Sets or retrieves a destination URL or an anchor point.\n     */\n    href: string;\n    /**\n     * Sets or retrieves the language code of the object.\n     */\n    hreflang: string;\n    imageSizes: string;\n    imageSrcset: string;\n    integrity: string;\n    /**\n     * Sets or retrieves the media type.\n     */\n    media: string;\n    referrerPolicy: string;\n    /**\n     * Sets or retrieves the relationship between the object and the destination of the link.\n     */\n    rel: string;\n    readonly relList: DOMTokenList;\n    /**\n     * Sets or retrieves the relationship between the object and the destination of the link.\n     * @deprecated\n     */\n    rev: string;\n    readonly sizes: DOMTokenList;\n    /**\n     * Sets or retrieves the window or frame at which to target content.\n     * @deprecated\n     */\n    target: string;\n    /**\n     * Sets or retrieves the MIME type of the object.\n     */\n    type: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLinkElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLLinkElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLLinkElement: {\n    prototype: HTMLLinkElement;\n    new(): HTMLLinkElement;\n};\n\n/** Provides special properties and methods (beyond those of the regular object HTMLElement interface it also has available to it by inheritance) for manipulating the layout and presentation of map elements. */\ninterface HTMLMapElement extends HTMLElement {\n    /**\n     * Retrieves a collection of the area objects defined for the given map object.\n     */\n    readonly areas: HTMLCollection;\n    /**\n     * Sets or retrieves the name of the object.\n     */\n    name: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMapElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMapElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLMapElement: {\n    prototype: HTMLMapElement;\n    new(): HTMLMapElement;\n};\n\n/**\n * Provides methods to manipulate <marquee> elements.\n * @deprecated\n */\ninterface HTMLMarqueeElement extends HTMLElement {\n    /** @deprecated */\n    behavior: string;\n    /** @deprecated */\n    bgColor: string;\n    /** @deprecated */\n    direction: string;\n    /** @deprecated */\n    height: string;\n    /** @deprecated */\n    hspace: number;\n    /** @deprecated */\n    loop: number;\n    /** @deprecated */\n    scrollAmount: number;\n    /** @deprecated */\n    scrollDelay: number;\n    /** @deprecated */\n    trueSpeed: boolean;\n    /** @deprecated */\n    vspace: number;\n    /** @deprecated */\n    width: string;\n    /** @deprecated */\n    start(): void;\n    /** @deprecated */\n    stop(): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMarqueeElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMarqueeElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** @deprecated */\ndeclare var HTMLMarqueeElement: {\n    prototype: HTMLMarqueeElement;\n    new(): HTMLMarqueeElement;\n};\n\ninterface HTMLMediaElementEventMap extends HTMLElementEventMap {\n    "encrypted": MediaEncryptedEvent;\n    "waitingforkey": Event;\n}\n\n/** Adds to HTMLElement the properties and methods needed to support basic media-related capabilities that are common to audio and video. */\ninterface HTMLMediaElement extends HTMLElement {\n    /**\n     * Gets or sets a value that indicates whether to start playing the media automatically.\n     */\n    autoplay: boolean;\n    /**\n     * Gets a collection of buffered time ranges.\n     */\n    readonly buffered: TimeRanges;\n    /**\n     * Gets or sets a flag that indicates whether the client provides a set of controls for the media (in case the developer does not include controls for the player).\n     */\n    controls: boolean;\n    crossOrigin: string | null;\n    /**\n     * Gets the address or URL of the current media resource that is selected by IHTMLMediaElement.\n     */\n    readonly currentSrc: string;\n    /**\n     * Gets or sets the current playback position, in seconds.\n     */\n    currentTime: number;\n    defaultMuted: boolean;\n    /**\n     * Gets or sets the default playback rate when the user is not using fast forward or reverse for a video or audio resource.\n     */\n    defaultPlaybackRate: number;\n    disableRemotePlayback: boolean;\n    /**\n     * Returns the duration in seconds of the current media resource. A NaN value is returned if duration is not available, or Infinity if the media resource is streaming.\n     */\n    readonly duration: number;\n    /**\n     * Gets information about whether the playback has ended or not.\n     */\n    readonly ended: boolean;\n    /**\n     * Returns an object representing the current error state of the audio or video element.\n     */\n    readonly error: MediaError | null;\n    /**\n     * Gets or sets a flag to specify whether playback should restart after it completes.\n     */\n    loop: boolean;\n    readonly mediaKeys: MediaKeys | null;\n    /**\n     * Gets or sets a flag that indicates whether the audio (either audio or the audio track on video media) is muted.\n     */\n    muted: boolean;\n    /**\n     * Gets the current network activity for the element.\n     */\n    readonly networkState: number;\n    onencrypted: ((this: HTMLMediaElement, ev: MediaEncryptedEvent) => any) | null;\n    onwaitingforkey: ((this: HTMLMediaElement, ev: Event) => any) | null;\n    /**\n     * Gets a flag that specifies whether playback is paused.\n     */\n    readonly paused: boolean;\n    /**\n     * Gets or sets the current rate of speed for the media resource to play. This speed is expressed as a multiple of the normal speed of the media resource.\n     */\n    playbackRate: number;\n    /**\n     * Gets TimeRanges for the current media resource that has been played.\n     */\n    readonly played: TimeRanges;\n    /**\n     * Gets or sets the current playback position, in seconds.\n     */\n    preload: string;\n    readonly readyState: number;\n    readonly remote: RemotePlayback;\n    /**\n     * Returns a TimeRanges object that represents the ranges of the current media resource that can be seeked.\n     */\n    readonly seekable: TimeRanges;\n    /**\n     * Gets a flag that indicates whether the client is currently moving to a new playback position in the media resource.\n     */\n    readonly seeking: boolean;\n    /**\n     * The address or URL of the a media resource that is to be considered.\n     */\n    src: string;\n    srcObject: MediaProvider | null;\n    readonly textTracks: TextTrackList;\n    /**\n     * Gets or sets the volume level for audio portions of the media element.\n     */\n    volume: number;\n    addTextTrack(kind: TextTrackKind, label?: string, language?: string): TextTrack;\n    /**\n     * Returns a string that specifies whether the client can play a given media resource type.\n     */\n    canPlayType(type: string): CanPlayTypeResult;\n    fastSeek(time: number): void;\n    /**\n     * Resets the audio or video object and loads a new media resource.\n     */\n    load(): void;\n    /**\n     * Pauses the current playback and sets paused to TRUE. This can be used to test whether the media is playing or paused. You can also use the pause or play events to tell whether the media is playing or not.\n     */\n    pause(): void;\n    /**\n     * Loads and starts playback of a media resource.\n     */\n    play(): Promise<void>;\n    setMediaKeys(mediaKeys: MediaKeys | null): Promise<void>;\n    readonly HAVE_CURRENT_DATA: number;\n    readonly HAVE_ENOUGH_DATA: number;\n    readonly HAVE_FUTURE_DATA: number;\n    readonly HAVE_METADATA: number;\n    readonly HAVE_NOTHING: number;\n    readonly NETWORK_EMPTY: number;\n    readonly NETWORK_IDLE: number;\n    readonly NETWORK_LOADING: number;\n    readonly NETWORK_NO_SOURCE: number;\n    addEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLMediaElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLMediaElementEventMap>(type: K, listener: (this: HTMLMediaElement, ev: HTMLMediaElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLMediaElement: {\n    prototype: HTMLMediaElement;\n    new(): HTMLMediaElement;\n    readonly HAVE_CURRENT_DATA: number;\n    readonly HAVE_ENOUGH_DATA: number;\n    readonly HAVE_FUTURE_DATA: number;\n    readonly HAVE_METADATA: number;\n    readonly HAVE_NOTHING: number;\n    readonly NETWORK_EMPTY: number;\n    readonly NETWORK_IDLE: number;\n    readonly NETWORK_LOADING: number;\n    readonly NETWORK_NO_SOURCE: number;\n};\n\ninterface HTMLMenuElement extends HTMLElement {\n    /** @deprecated */\n    compact: boolean;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMenuElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMenuElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLMenuElement: {\n    prototype: HTMLMenuElement;\n    new(): HTMLMenuElement;\n};\n\n/** Contains descriptive metadata about a document. It inherits all of the properties and methods described in the HTMLElement interface. */\ninterface HTMLMetaElement extends HTMLElement {\n    /**\n     * Gets or sets meta-information to associate with httpEquiv or name.\n     */\n    content: string;\n    /**\n     * Gets or sets information used to bind the value of a content attribute of a meta element to an HTTP response header.\n     */\n    httpEquiv: string;\n    /**\n     * Sets or retrieves the value specified in the content attribute of the meta object.\n     */\n    name: string;\n    /**\n     * Sets or retrieves a scheme to be used in interpreting the value of a property specified for the object.\n     * @deprecated\n     */\n    scheme: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMetaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMetaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLMetaElement: {\n    prototype: HTMLMetaElement;\n    new(): HTMLMetaElement;\n};\n\n/** The HTML <meter> elements expose the HTMLMeterElement interface, which provides special properties and methods (beyond the HTMLElement object interface they also have available to them by inheritance) for manipulating the layout and presentation of <meter> elements. */\ninterface HTMLMeterElement extends HTMLElement {\n    high: number;\n    readonly labels: NodeListOf<HTMLLabelElement>;\n    low: number;\n    max: number;\n    min: number;\n    optimum: number;\n    value: number;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMeterElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLMeterElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLMeterElement: {\n    prototype: HTMLMeterElement;\n    new(): HTMLMeterElement;\n};\n\n/** Provides special properties (beyond the regular methods and properties available through the HTMLElement interface they also have available to them by inheritance) for manipulating modification elements, that is <del> and <ins>. */\ninterface HTMLModElement extends HTMLElement {\n    /**\n     * Sets or retrieves reference information about the object.\n     */\n    cite: string;\n    /**\n     * Sets or retrieves the date and time of a modification to the object.\n     */\n    dateTime: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLModElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLModElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLModElement: {\n    prototype: HTMLModElement;\n    new(): HTMLModElement;\n};\n\n/** Provides special properties (beyond those defined on the regular HTMLElement interface it also has available to it by inheritance) for manipulating ordered list elements. */\ninterface HTMLOListElement extends HTMLElement {\n    /** @deprecated */\n    compact: boolean;\n    reversed: boolean;\n    /**\n     * The starting number.\n     */\n    start: number;\n    type: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLOListElement: {\n    prototype: HTMLOListElement;\n    new(): HTMLOListElement;\n};\n\n/** Provides special properties and methods (beyond those on the HTMLElement interface it also has available to it by inheritance) for manipulating the layout and presentation of <object> element, representing external resources. */\ninterface HTMLObjectElement extends HTMLElement {\n    /** @deprecated */\n    align: string;\n    /**\n     * Sets or retrieves a character string that can be used to implement your own archive functionality for the object.\n     * @deprecated\n     */\n    archive: string;\n    /** @deprecated */\n    border: string;\n    /**\n     * Sets or retrieves the URL of the file containing the compiled Java class.\n     * @deprecated\n     */\n    code: string;\n    /**\n     * Sets or retrieves the URL of the component.\n     * @deprecated\n     */\n    codeBase: string;\n    /**\n     * Sets or retrieves the Internet media type for the code associated with the object.\n     * @deprecated\n     */\n    codeType: string;\n    /**\n     * Retrieves the document object of the page or frame.\n     */\n    readonly contentDocument: Document | null;\n    readonly contentWindow: WindowProxy | null;\n    /**\n     * Sets or retrieves the URL that references the data of the object.\n     */\n    data: string;\n    /** @deprecated */\n    declare: boolean;\n    /**\n     * Retrieves a reference to the form that the object is embedded in.\n     */\n    readonly form: HTMLFormElement | null;\n    /**\n     * Sets or retrieves the height of the object.\n     */\n    height: string;\n    /** @deprecated */\n    hspace: number;\n    /**\n     * Sets or retrieves the name of the object.\n     */\n    name: string;\n    /**\n     * Sets or retrieves a message to be displayed while an object is loading.\n     * @deprecated\n     */\n    standby: string;\n    /**\n     * Sets or retrieves the MIME type of the object.\n     */\n    type: string;\n    /**\n     * Sets or retrieves the URL, often with a bookmark extension (#name), to use as a client-side image map.\n     */\n    useMap: string;\n    /**\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as "this is a required field". The result is that the user sees validation messages without actually submitting.\n     */\n    readonly validationMessage: string;\n    /**\n     * Returns a  ValidityState object that represents the validity states of an element.\n     */\n    readonly validity: ValidityState;\n    /** @deprecated */\n    vspace: number;\n    /**\n     * Sets or retrieves the width of the object.\n     */\n    width: string;\n    /**\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\n     */\n    readonly willValidate: boolean;\n    /**\n     * Returns whether a form will validate when it is submitted, without having to submit it.\n     */\n    checkValidity(): boolean;\n    getSVGDocument(): Document | null;\n    reportValidity(): boolean;\n    /**\n     * Sets a custom error message that is displayed when a form is submitted.\n     * @param error Sets a custom error message that is displayed when a form is submitted.\n     */\n    setCustomValidity(error: string): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLObjectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLObjectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLObjectElement: {\n    prototype: HTMLObjectElement;\n    new(): HTMLObjectElement;\n};\n\n/** Provides special properties and methods (beyond the regular HTMLElement object interface they also have available to them by inheritance) for manipulating the layout and presentation of <optgroup> elements. */\ninterface HTMLOptGroupElement extends HTMLElement {\n    disabled: boolean;\n    /**\n     * Sets or retrieves a value that you can use to implement your own label functionality for the object.\n     */\n    label: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptGroupElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptGroupElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLOptGroupElement: {\n    prototype: HTMLOptGroupElement;\n    new(): HTMLOptGroupElement;\n};\n\n/** <option> elements and inherits all classes and methods of the HTMLElement interface. */\ninterface HTMLOptionElement extends HTMLElement {\n    /**\n     * Sets or retrieves the status of an option.\n     */\n    defaultSelected: boolean;\n    disabled: boolean;\n    /**\n     * Retrieves a reference to the form that the object is embedded in.\n     */\n    readonly form: HTMLFormElement | null;\n    /**\n     * Sets or retrieves the ordinal position of an option in a list box.\n     */\n    readonly index: number;\n    /**\n     * Sets or retrieves a value that you can use to implement your own label functionality for the object.\n     */\n    label: string;\n    /**\n     * Sets or retrieves whether the option in the list box is the default item.\n     */\n    selected: boolean;\n    /**\n     * Sets or retrieves the text string specified by the option tag.\n     */\n    text: string;\n    /**\n     * Sets or retrieves the value which is returned to the server when the form control is submitted.\n     */\n    value: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLOptionElement: {\n    prototype: HTMLOptionElement;\n    new(): HTMLOptionElement;\n};\n\n/** HTMLOptionsCollection is an interface representing a collection of HTML option elements (in document order) and offers methods and properties for traversing the list as well as optionally altering its items. This type is returned solely by the "options" property of select. */\ninterface HTMLOptionsCollection extends HTMLCollectionOf<HTMLOptionElement> {\n    /**\n     * Returns the number of elements in the collection.\n     *\n     * When set to a smaller number, truncates the number of option elements in the corresponding container.\n     *\n     * When set to a greater number, adds new blank option elements to that container.\n     */\n    length: number;\n    /**\n     * Returns the index of the first selected item, if any, or −1 if there is no selected item.\n     *\n     * Can be set, to change the selection.\n     */\n    selectedIndex: number;\n    /**\n     * Inserts element before the node given by before.\n     *\n     * The before argument can be a number, in which case element is inserted before the item with that number, or an element from the collection, in which case element is inserted before that element.\n     *\n     * If before is omitted, null, or a number out of range, then element will be added at the end of the list.\n     *\n     * This method will throw a "HierarchyRequestError" DOMException if element is an ancestor of the element into which it is to be inserted.\n     */\n    add(element: HTMLOptionElement | HTMLOptGroupElement, before?: HTMLElement | number | null): void;\n    /**\n     * Removes the item with index index from the collection.\n     */\n    remove(index: number): void;\n}\n\ndeclare var HTMLOptionsCollection: {\n    prototype: HTMLOptionsCollection;\n    new(): HTMLOptionsCollection;\n};\n\ninterface HTMLOrSVGElement {\n    readonly dataset: DOMStringMap;\n    nonce?: string;\n    tabIndex: number;\n    blur(): void;\n    focus(options?: FocusOptions): void;\n}\n\n/** Provides properties and methods (beyond those inherited from HTMLElement) for manipulating the layout and presentation of <output> elements. */\ninterface HTMLOutputElement extends HTMLElement {\n    defaultValue: string;\n    readonly form: HTMLFormElement | null;\n    readonly htmlFor: DOMTokenList;\n    readonly labels: NodeListOf<HTMLLabelElement>;\n    name: string;\n    /**\n     * Returns the string "output".\n     */\n    readonly type: string;\n    readonly validationMessage: string;\n    readonly validity: ValidityState;\n    /**\n     * Returns the element\'s current value.\n     *\n     * Can be set, to change the value.\n     */\n    value: string;\n    readonly willValidate: boolean;\n    checkValidity(): boolean;\n    reportValidity(): boolean;\n    setCustomValidity(error: string): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOutputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLOutputElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLOutputElement: {\n    prototype: HTMLOutputElement;\n    new(): HTMLOutputElement;\n};\n\n/** Provides special properties (beyond those of the regular HTMLElement object interface it inherits) for manipulating <p> elements. */\ninterface HTMLParagraphElement extends HTMLElement {\n    /**\n     * Sets or retrieves how the object is aligned with adjacent text.\n     * @deprecated\n     */\n    align: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParagraphElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParagraphElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLParagraphElement: {\n    prototype: HTMLParagraphElement;\n    new(): HTMLParagraphElement;\n};\n\n/** Provides special properties (beyond those of the regular HTMLElement object interface it inherits) for manipulating <param> elements, representing a pair of a key and a value that acts as a parameter for an <object> element. */\ninterface HTMLParamElement extends HTMLElement {\n    /**\n     * Sets or retrieves the name of an input parameter for an element.\n     */\n    name: string;\n    /**\n     * Sets or retrieves the content type of the resource designated by the value attribute.\n     * @deprecated\n     */\n    type: string;\n    /**\n     * Sets or retrieves the value of an input parameter for an element.\n     */\n    value: string;\n    /**\n     * Sets or retrieves the data type of the value attribute.\n     * @deprecated\n     */\n    valueType: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParamElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLParamElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLParamElement: {\n    prototype: HTMLParamElement;\n    new(): HTMLParamElement;\n};\n\n/** A <picture> HTML element. It doesn\'t implement specific properties or methods. */\ninterface HTMLPictureElement extends HTMLElement {\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPictureElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPictureElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLPictureElement: {\n    prototype: HTMLPictureElement;\n    new(): HTMLPictureElement;\n};\n\n/** Exposes specific properties and methods (beyond those of the HTMLElement interface it also has available to it by inheritance) for manipulating a block of preformatted text (<pre>). */\ninterface HTMLPreElement extends HTMLElement {\n    /**\n     * Sets or gets a value that you can use to implement your own width functionality for the object.\n     * @deprecated\n     */\n    width: number;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPreElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLPreElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLPreElement: {\n    prototype: HTMLPreElement;\n    new(): HTMLPreElement;\n};\n\n/** Provides special properties and methods (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating the layout and presentation of <progress> elements. */\ninterface HTMLProgressElement extends HTMLElement {\n    readonly labels: NodeListOf<HTMLLabelElement>;\n    /**\n     * Defines the maximum, or "done" value for a progress element.\n     */\n    max: number;\n    /**\n     * Returns the quotient of value/max when the value attribute is set (determinate progress bar), or -1 when the value attribute is missing (indeterminate progress bar).\n     */\n    readonly position: number;\n    /**\n     * Sets or gets the current value of a progress element. The value must be a non-negative number between 0 and the max value.\n     */\n    value: number;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLProgressElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLProgressElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLProgressElement: {\n    prototype: HTMLProgressElement;\n    new(): HTMLProgressElement;\n};\n\n/** Provides special properties and methods (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating quoting elements, like <blockquote> and <q>, but not the <cite> element. */\ninterface HTMLQuoteElement extends HTMLElement {\n    /**\n     * Sets or retrieves reference information about the object.\n     */\n    cite: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLQuoteElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLQuoteElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLQuoteElement: {\n    prototype: HTMLQuoteElement;\n    new(): HTMLQuoteElement;\n};\n\n/** HTML <script> elements expose the HTMLScriptElement interface, which provides special properties and methods for manipulating the behavior and execution of <script> elements (beyond the inherited HTMLElement interface). */\ninterface HTMLScriptElement extends HTMLElement {\n    async: boolean;\n    /**\n     * Sets or retrieves the character set used to encode the object.\n     * @deprecated\n     */\n    charset: string;\n    crossOrigin: string | null;\n    /**\n     * Sets or retrieves the status of the script.\n     */\n    defer: boolean;\n    /**\n     * Sets or retrieves the event for which the script is written.\n     * @deprecated\n     */\n    event: string;\n    /**\n     * Sets or retrieves the object that is bound to the event script.\n     * @deprecated\n     */\n    htmlFor: string;\n    integrity: string;\n    noModule: boolean;\n    referrerPolicy: string;\n    /**\n     * Retrieves the URL to an external file that contains the source code or data.\n     */\n    src: string;\n    /**\n     * Retrieves or sets the text of the object as a string.\n     */\n    text: string;\n    /**\n     * Sets or retrieves the MIME type for the associated scripting engine.\n     */\n    type: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLScriptElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLScriptElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLScriptElement: {\n    prototype: HTMLScriptElement;\n    new(): HTMLScriptElement;\n};\n\n/** A <select> HTML Element. These elements also share all of the properties and methods of other HTML elements via the HTMLElement interface. */\ninterface HTMLSelectElement extends HTMLElement {\n    autocomplete: string;\n    disabled: boolean;\n    /**\n     * Retrieves a reference to the form that the object is embedded in.\n     */\n    readonly form: HTMLFormElement | null;\n    readonly labels: NodeListOf<HTMLLabelElement>;\n    /**\n     * Sets or retrieves the number of objects in a collection.\n     */\n    length: number;\n    /**\n     * Sets or retrieves the Boolean value indicating whether multiple items can be selected from a list.\n     */\n    multiple: boolean;\n    /**\n     * Sets or retrieves the name of the object.\n     */\n    name: string;\n    /**\n     * Returns an HTMLOptionsCollection of the list of options.\n     */\n    readonly options: HTMLOptionsCollection;\n    /**\n     * When present, marks an element that can\'t be submitted without a value.\n     */\n    required: boolean;\n    /**\n     * Sets or retrieves the index of the selected option in a select object.\n     */\n    selectedIndex: number;\n    readonly selectedOptions: HTMLCollectionOf<HTMLOptionElement>;\n    /**\n     * Sets or retrieves the number of rows in the list box.\n     */\n    size: number;\n    /**\n     * Retrieves the type of select control based on the value of the MULTIPLE attribute.\n     */\n    readonly type: string;\n    /**\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as "this is a required field". The result is that the user sees validation messages without actually submitting.\n     */\n    readonly validationMessage: string;\n    /**\n     * Returns a  ValidityState object that represents the validity states of an element.\n     */\n    readonly validity: ValidityState;\n    /**\n     * Sets or retrieves the value which is returned to the server when the form control is submitted.\n     */\n    value: string;\n    /**\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\n     */\n    readonly willValidate: boolean;\n    /**\n     * Adds an element to the areas, controlRange, or options collection.\n     * @param element Variant of type Number that specifies the index position in the collection where the element is placed. If no value is given, the method places the element at the end of the collection.\n     * @param before Variant of type Object that specifies an element to insert before, or null to append the object to the collection.\n     */\n    add(element: HTMLOptionElement | HTMLOptGroupElement, before?: HTMLElement | number | null): void;\n    /**\n     * Returns whether a form will validate when it is submitted, without having to submit it.\n     */\n    checkValidity(): boolean;\n    /**\n     * Retrieves a select object or an object from an options collection.\n     * @param name Variant of type Number or String that specifies the object or collection to retrieve. If this parameter is an integer, it is the zero-based index of the object. If this parameter is a string, all objects with matching name or id properties are retrieved, and a collection is returned if more than one match is made.\n     * @param index Variant of type Number that specifies the zero-based index of the object to retrieve when a collection is returned.\n     */\n    item(index: number): HTMLOptionElement | null;\n    /**\n     * Retrieves a select object or an object from an options collection.\n     * @param namedItem A String that specifies the name or id property of the object to retrieve. A collection is returned if more than one match is made.\n     */\n    namedItem(name: string): HTMLOptionElement | null;\n    /**\n     * Removes an element from the collection.\n     * @param index Number that specifies the zero-based index of the element to remove from the collection.\n     */\n    remove(): void;\n    remove(index: number): void;\n    reportValidity(): boolean;\n    /**\n     * Sets a custom error message that is displayed when a form is submitted.\n     * @param error Sets a custom error message that is displayed when a form is submitted.\n     */\n    setCustomValidity(error: string): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSelectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSelectElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n    [name: number]: HTMLOptionElement | HTMLOptGroupElement;\n}\n\ndeclare var HTMLSelectElement: {\n    prototype: HTMLSelectElement;\n    new(): HTMLSelectElement;\n};\n\ninterface HTMLSlotElement extends HTMLElement {\n    name: string;\n    assignedElements(options?: AssignedNodesOptions): Element[];\n    assignedNodes(options?: AssignedNodesOptions): Node[];\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSlotElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSlotElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLSlotElement: {\n    prototype: HTMLSlotElement;\n    new(): HTMLSlotElement;\n};\n\n/** Provides special properties (beyond the regular HTMLElement object interface it also has available to it by inheritance) for manipulating <source> elements. */\ninterface HTMLSourceElement extends HTMLElement {\n    /**\n     * Gets or sets the intended media type of the media source.\n     */\n    media: string;\n    sizes: string;\n    /**\n     * The address or URL of the a media resource that is to be considered.\n     */\n    src: string;\n    srcset: string;\n    /**\n     * Gets or sets the MIME type of a media resource.\n     */\n    type: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSourceElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSourceElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLSourceElement: {\n    prototype: HTMLSourceElement;\n    new(): HTMLSourceElement;\n};\n\n/** A <span> element and derives from the HTMLElement interface, but without implementing any additional properties or methods. */\ninterface HTMLSpanElement extends HTMLElement {\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSpanElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLSpanElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLSpanElement: {\n    prototype: HTMLSpanElement;\n    new(): HTMLSpanElement;\n};\n\n/** A <style> element. It inherits properties and methods from its parent, HTMLElement, and from LinkStyle. */\ninterface HTMLStyleElement extends HTMLElement, LinkStyle {\n    /**\n     * Sets or retrieves the media type.\n     */\n    media: string;\n    /**\n     * Retrieves the CSS language in which the style sheet is written.\n     * @deprecated\n     */\n    type: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLStyleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLStyleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLStyleElement: {\n    prototype: HTMLStyleElement;\n    new(): HTMLStyleElement;\n};\n\n/** Special properties (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating table caption elements. */\ninterface HTMLTableCaptionElement extends HTMLElement {\n    /**\n     * Sets or retrieves the alignment of the caption or legend.\n     * @deprecated\n     */\n    align: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCaptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCaptionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTableCaptionElement: {\n    prototype: HTMLTableCaptionElement;\n    new(): HTMLTableCaptionElement;\n};\n\n/** Provides special properties and methods (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating the layout and presentation of table cells, either header or data cells, in an HTML document. */\ninterface HTMLTableCellElement extends HTMLElement {\n    /**\n     * Sets or retrieves abbreviated text for the object.\n     */\n    abbr: string;\n    /**\n     * Sets or retrieves how the object is aligned with adjacent text.\n     * @deprecated\n     */\n    align: string;\n    /**\n     * Sets or retrieves a comma-delimited list of conceptual categories associated with the object.\n     * @deprecated\n     */\n    axis: string;\n    /** @deprecated */\n    bgColor: string;\n    /**\n     * Retrieves the position of the object in the cells collection of a row.\n     */\n    readonly cellIndex: number;\n    /** @deprecated */\n    ch: string;\n    /** @deprecated */\n    chOff: string;\n    /**\n     * Sets or retrieves the number columns in the table that the object should span.\n     */\n    colSpan: number;\n    /**\n     * Sets or retrieves a list of header cells that provide information for the object.\n     */\n    headers: string;\n    /**\n     * Sets or retrieves the height of the object.\n     * @deprecated\n     */\n    height: string;\n    /**\n     * Sets or retrieves whether the browser automatically performs wordwrap.\n     * @deprecated\n     */\n    noWrap: boolean;\n    /**\n     * Sets or retrieves how many rows in a table the cell should span.\n     */\n    rowSpan: number;\n    /**\n     * Sets or retrieves the group of cells in a table to which the object\'s information applies.\n     */\n    scope: string;\n    /** @deprecated */\n    vAlign: string;\n    /**\n     * Sets or retrieves the width of the object.\n     * @deprecated\n     */\n    width: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTableCellElement: {\n    prototype: HTMLTableCellElement;\n    new(): HTMLTableCellElement;\n};\n\n/** Provides special properties (beyond the HTMLElement interface it also has available to it inheritance) for manipulating single or grouped table column elements. */\ninterface HTMLTableColElement extends HTMLElement {\n    /**\n     * Sets or retrieves the alignment of the object relative to the display or table.\n     * @deprecated\n     */\n    align: string;\n    /** @deprecated */\n    ch: string;\n    /** @deprecated */\n    chOff: string;\n    /**\n     * Sets or retrieves the number of columns in the group.\n     */\n    span: number;\n    /** @deprecated */\n    vAlign: string;\n    /**\n     * Sets or retrieves the width of the object.\n     * @deprecated\n     */\n    width: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableColElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableColElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTableColElement: {\n    prototype: HTMLTableColElement;\n    new(): HTMLTableColElement;\n};\n\n/** @deprecated prefer HTMLTableCellElement */\ninterface HTMLTableDataCellElement extends HTMLTableCellElement {\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableDataCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableDataCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** Provides special properties and methods (beyond the regular HTMLElement object interface it also has available to it by inheritance) for manipulating the layout and presentation of tables in an HTML document. */\ninterface HTMLTableElement extends HTMLElement {\n    /**\n     * Sets or retrieves a value that indicates the table alignment.\n     * @deprecated\n     */\n    align: string;\n    /** @deprecated */\n    bgColor: string;\n    /**\n     * Sets or retrieves the width of the border to draw around the object.\n     * @deprecated\n     */\n    border: string;\n    /**\n     * Retrieves the caption object of a table.\n     */\n    caption: HTMLTableCaptionElement | null;\n    /**\n     * Sets or retrieves the amount of space between the border of the cell and the content of the cell.\n     * @deprecated\n     */\n    cellPadding: string;\n    /**\n     * Sets or retrieves the amount of space between cells in a table.\n     * @deprecated\n     */\n    cellSpacing: string;\n    /**\n     * Sets or retrieves the way the border frame around the table is displayed.\n     * @deprecated\n     */\n    frame: string;\n    /**\n     * Sets or retrieves the number of horizontal rows contained in the object.\n     */\n    readonly rows: HTMLCollectionOf<HTMLTableRowElement>;\n    /**\n     * Sets or retrieves which dividing lines (inner borders) are displayed.\n     * @deprecated\n     */\n    rules: string;\n    /**\n     * Sets or retrieves a description and/or structure of the object.\n     * @deprecated\n     */\n    summary: string;\n    /**\n     * Retrieves a collection of all tBody objects in the table. Objects in this collection are in source order.\n     */\n    readonly tBodies: HTMLCollectionOf<HTMLTableSectionElement>;\n    /**\n     * Retrieves the tFoot object of the table.\n     */\n    tFoot: HTMLTableSectionElement | null;\n    /**\n     * Retrieves the tHead object of the table.\n     */\n    tHead: HTMLTableSectionElement | null;\n    /**\n     * Sets or retrieves the width of the object.\n     * @deprecated\n     */\n    width: string;\n    /**\n     * Creates an empty caption element in the table.\n     */\n    createCaption(): HTMLTableCaptionElement;\n    /**\n     * Creates an empty tBody element in the table.\n     */\n    createTBody(): HTMLTableSectionElement;\n    /**\n     * Creates an empty tFoot element in the table.\n     */\n    createTFoot(): HTMLTableSectionElement;\n    /**\n     * Returns the tHead element object if successful, or null otherwise.\n     */\n    createTHead(): HTMLTableSectionElement;\n    /**\n     * Deletes the caption element and its contents from the table.\n     */\n    deleteCaption(): void;\n    /**\n     * Removes the specified row (tr) from the element and from the rows collection.\n     * @param index Number that specifies the zero-based position in the rows collection of the row to remove.\n     */\n    deleteRow(index: number): void;\n    /**\n     * Deletes the tFoot element and its contents from the table.\n     */\n    deleteTFoot(): void;\n    /**\n     * Deletes the tHead element and its contents from the table.\n     */\n    deleteTHead(): void;\n    /**\n     * Creates a new row (tr) in the table, and adds the row to the rows collection.\n     * @param index Number that specifies where to insert the row in the rows collection. The default value is -1, which appends the new row to the end of the rows collection.\n     */\n    insertRow(index?: number): HTMLTableRowElement;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTableElement: {\n    prototype: HTMLTableElement;\n    new(): HTMLTableElement;\n};\n\n/** @deprecated prefer HTMLTableCellElement */\ninterface HTMLTableHeaderCellElement extends HTMLTableCellElement {\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableHeaderCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableHeaderCellElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** Provides special properties and methods (beyond the HTMLElement interface it also has available to it by inheritance) for manipulating the layout and presentation of rows in an HTML table. */\ninterface HTMLTableRowElement extends HTMLElement {\n    /**\n     * Sets or retrieves how the object is aligned with adjacent text.\n     * @deprecated\n     */\n    align: string;\n    /** @deprecated */\n    bgColor: string;\n    /**\n     * Retrieves a collection of all cells in the table row.\n     */\n    readonly cells: HTMLCollectionOf<HTMLTableCellElement>;\n    /** @deprecated */\n    ch: string;\n    /** @deprecated */\n    chOff: string;\n    /**\n     * Retrieves the position of the object in the rows collection for the table.\n     */\n    readonly rowIndex: number;\n    /**\n     * Retrieves the position of the object in the collection.\n     */\n    readonly sectionRowIndex: number;\n    /** @deprecated */\n    vAlign: string;\n    /**\n     * Removes the specified cell from the table row, as well as from the cells collection.\n     * @param index Number that specifies the zero-based position of the cell to remove from the table row. If no value is provided, the last cell in the cells collection is deleted.\n     */\n    deleteCell(index: number): void;\n    /**\n     * Creates a new cell in the table row, and adds the cell to the cells collection.\n     * @param index Number that specifies where to insert the cell in the tr. The default value is -1, which appends the new cell to the end of the cells collection.\n     */\n    insertCell(index?: number): HTMLTableCellElement;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableRowElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableRowElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTableRowElement: {\n    prototype: HTMLTableRowElement;\n    new(): HTMLTableRowElement;\n};\n\n/** Provides special properties and methods (beyond the HTMLElement interface it also has available to it by inheritance) for manipulating the layout and presentation of sections, that is headers, footers and bodies, in an HTML table. */\ninterface HTMLTableSectionElement extends HTMLElement {\n    /**\n     * Sets or retrieves a value that indicates the table alignment.\n     * @deprecated\n     */\n    align: string;\n    /** @deprecated */\n    ch: string;\n    /** @deprecated */\n    chOff: string;\n    /**\n     * Sets or retrieves the number of horizontal rows contained in the object.\n     */\n    readonly rows: HTMLCollectionOf<HTMLTableRowElement>;\n    /** @deprecated */\n    vAlign: string;\n    /**\n     * Removes the specified row (tr) from the element and from the rows collection.\n     * @param index Number that specifies the zero-based position in the rows collection of the row to remove.\n     */\n    deleteRow(index: number): void;\n    /**\n     * Creates a new row (tr) in the table, and adds the row to the rows collection.\n     * @param index Number that specifies where to insert the row in the rows collection. The default value is -1, which appends the new row to the end of the rows collection.\n     */\n    insertRow(index?: number): HTMLTableRowElement;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableSectionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTableSectionElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTableSectionElement: {\n    prototype: HTMLTableSectionElement;\n    new(): HTMLTableSectionElement;\n};\n\n/** Enables access to the contents of an HTML <template> element. */\ninterface HTMLTemplateElement extends HTMLElement {\n    /**\n     * Returns the template contents (a DocumentFragment).\n     */\n    readonly content: DocumentFragment;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTemplateElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTemplateElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTemplateElement: {\n    prototype: HTMLTemplateElement;\n    new(): HTMLTemplateElement;\n};\n\n/** Provides special properties and methods for manipulating the layout and presentation of <textarea> elements. */\ninterface HTMLTextAreaElement extends HTMLElement {\n    autocomplete: string;\n    /**\n     * Sets or retrieves the width of the object.\n     */\n    cols: number;\n    /**\n     * Sets or retrieves the initial contents of the object.\n     */\n    defaultValue: string;\n    dirName: string;\n    disabled: boolean;\n    /**\n     * Retrieves a reference to the form that the object is embedded in.\n     */\n    readonly form: HTMLFormElement | null;\n    readonly labels: NodeListOf<HTMLLabelElement>;\n    /**\n     * Sets or retrieves the maximum number of characters that the user can enter in a text control.\n     */\n    maxLength: number;\n    minLength: number;\n    /**\n     * Sets or retrieves the name of the object.\n     */\n    name: string;\n    /**\n     * Gets or sets a text string that is displayed in an input field as a hint or prompt to users as the format or type of information they need to enter.The text appears in an input field until the user puts focus on the field.\n     */\n    placeholder: string;\n    /**\n     * Sets or retrieves the value indicated whether the content of the object is read-only.\n     */\n    readOnly: boolean;\n    /**\n     * When present, marks an element that can\'t be submitted without a value.\n     */\n    required: boolean;\n    /**\n     * Sets or retrieves the number of horizontal rows contained in the object.\n     */\n    rows: number;\n    selectionDirection: "forward" | "backward" | "none";\n    /**\n     * Gets or sets the end position or offset of a text selection.\n     */\n    selectionEnd: number;\n    /**\n     * Gets or sets the starting position or offset of a text selection.\n     */\n    selectionStart: number;\n    readonly textLength: number;\n    /**\n     * Retrieves the type of control.\n     */\n    readonly type: string;\n    /**\n     * Returns the error message that would be displayed if the user submits the form, or an empty string if no error message. It also triggers the standard error message, such as "this is a required field". The result is that the user sees validation messages without actually submitting.\n     */\n    readonly validationMessage: string;\n    /**\n     * Returns a  ValidityState object that represents the validity states of an element.\n     */\n    readonly validity: ValidityState;\n    /**\n     * Retrieves or sets the text in the entry field of the textArea element.\n     */\n    value: string;\n    /**\n     * Returns whether an element will successfully validate based on forms validation rules and constraints.\n     */\n    readonly willValidate: boolean;\n    /**\n     * Sets or retrieves how to handle wordwrapping in the object.\n     */\n    wrap: string;\n    /**\n     * Returns whether a form will validate when it is submitted, without having to submit it.\n     */\n    checkValidity(): boolean;\n    reportValidity(): boolean;\n    /**\n     * Highlights the input area of a form element.\n     */\n    select(): void;\n    /**\n     * Sets a custom error message that is displayed when a form is submitted.\n     * @param error Sets a custom error message that is displayed when a form is submitted.\n     */\n    setCustomValidity(error: string): void;\n    setRangeText(replacement: string): void;\n    setRangeText(replacement: string, start: number, end: number, selectionMode?: SelectionMode): void;\n    /**\n     * Sets the start and end positions of a selection in a text field.\n     * @param start The offset into the text field for the start of the selection.\n     * @param end The offset into the text field for the end of the selection.\n     * @param direction The direction in which the selection is performed.\n     */\n    setSelectionRange(start: number | null, end: number | null, direction?: "forward" | "backward" | "none"): void;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTextAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTextAreaElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTextAreaElement: {\n    prototype: HTMLTextAreaElement;\n    new(): HTMLTextAreaElement;\n};\n\n/** Provides special properties (beyond the regular HTMLElement interface it also has available to it by inheritance) for manipulating <time> elements. */\ninterface HTMLTimeElement extends HTMLElement {\n    dateTime: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTimeElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTimeElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTimeElement: {\n    prototype: HTMLTimeElement;\n    new(): HTMLTimeElement;\n};\n\n/** Contains the title for a document. This element inherits all of the properties and methods of the HTMLElement interface. */\ninterface HTMLTitleElement extends HTMLElement {\n    /**\n     * Retrieves or sets the text of the object as a string.\n     */\n    text: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTitleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTitleElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTitleElement: {\n    prototype: HTMLTitleElement;\n    new(): HTMLTitleElement;\n};\n\n/** The HTMLTrackElement */\ninterface HTMLTrackElement extends HTMLElement {\n    default: boolean;\n    kind: string;\n    label: string;\n    readonly readyState: number;\n    src: string;\n    srclang: string;\n    /**\n     * Returns the TextTrack object corresponding to the text track of the track element.\n     */\n    readonly track: TextTrack;\n    readonly ERROR: number;\n    readonly LOADED: number;\n    readonly LOADING: number;\n    readonly NONE: number;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTrackElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLTrackElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLTrackElement: {\n    prototype: HTMLTrackElement;\n    new(): HTMLTrackElement;\n    readonly ERROR: number;\n    readonly LOADED: number;\n    readonly LOADING: number;\n    readonly NONE: number;\n};\n\n/** Provides special properties (beyond those defined on the regular HTMLElement interface it also has available to it by inheritance) for manipulating unordered list elements. */\ninterface HTMLUListElement extends HTMLElement {\n    /** @deprecated */\n    compact: boolean;\n    /** @deprecated */\n    type: string;\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUListElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLUListElement: {\n    prototype: HTMLUListElement;\n    new(): HTMLUListElement;\n};\n\n/** An invalid HTML element and derives from the HTMLElement interface, but without implementing any additional properties or methods. */\ninterface HTMLUnknownElement extends HTMLElement {\n    addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUnknownElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLUnknownElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLUnknownElement: {\n    prototype: HTMLUnknownElement;\n    new(): HTMLUnknownElement;\n};\n\ninterface HTMLVideoElementEventMap extends HTMLMediaElementEventMap {\n    "enterpictureinpicture": Event;\n    "leavepictureinpicture": Event;\n}\n\n/** Provides special properties and methods for manipulating video objects. It also inherits properties and methods of HTMLMediaElement and HTMLElement. */\ninterface HTMLVideoElement extends HTMLMediaElement {\n    disablePictureInPicture: boolean;\n    /**\n     * Gets or sets the height of the video element.\n     */\n    height: number;\n    onenterpictureinpicture: ((this: HTMLVideoElement, ev: Event) => any) | null;\n    onleavepictureinpicture: ((this: HTMLVideoElement, ev: Event) => any) | null;\n    /**\n     * Gets or sets the playsinline of the video element. for example, On iPhone, video elements will now be allowed to play inline, and will not automatically enter fullscreen mode when playback begins.\n     */\n    playsInline: boolean;\n    /**\n     * Gets or sets a URL of an image to display, for example, like a movie poster. This can be a still frame from the video, or another image if no video data is available.\n     */\n    poster: string;\n    /**\n     * Gets the intrinsic height of a video in CSS pixels, or zero if the dimensions are not known.\n     */\n    readonly videoHeight: number;\n    /**\n     * Gets the intrinsic width of a video in CSS pixels, or zero if the dimensions are not known.\n     */\n    readonly videoWidth: number;\n    /**\n     * Gets or sets the width of the video element.\n     */\n    width: number;\n    getVideoPlaybackQuality(): VideoPlaybackQuality;\n    requestPictureInPicture(): Promise<PictureInPictureWindow>;\n    addEventListener<K extends keyof HTMLVideoElementEventMap>(type: K, listener: (this: HTMLVideoElement, ev: HTMLVideoElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof HTMLVideoElementEventMap>(type: K, listener: (this: HTMLVideoElement, ev: HTMLVideoElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var HTMLVideoElement: {\n    prototype: HTMLVideoElement;\n    new(): HTMLVideoElement;\n};\n\n/** Events that fire when the fragment identifier of the URL has changed. */\ninterface HashChangeEvent extends Event {\n    /**\n     * Returns the URL of the session history entry that is now current.\n     */\n    readonly newURL: string;\n    /**\n     * Returns the URL of the session history entry that was previously current.\n     */\n    readonly oldURL: string;\n}\n\ndeclare var HashChangeEvent: {\n    prototype: HashChangeEvent;\n    new(type: string, eventInitDict?: HashChangeEventInit): HashChangeEvent;\n};\n\n/** This Fetch API interface allows you to perform various actions on HTTP request and response headers. These actions include retrieving, setting, adding to, and removing. A Headers object has an associated header list, which is initially empty and consists of zero or more name and value pairs.  You can add to this using methods like append() (see Examples.) In all methods of this interface, header names are matched by case-insensitive byte sequence. */\ninterface Headers {\n    append(name: string, value: string): void;\n    delete(name: string): void;\n    get(name: string): string | null;\n    has(name: string): boolean;\n    set(name: string, value: string): void;\n    forEach(callbackfn: (value: string, key: string, parent: Headers) => void, thisArg?: any): void;\n}\n\ndeclare var Headers: {\n    prototype: Headers;\n    new(init?: HeadersInit): Headers;\n};\n\n/** Allows manipulation of the browser session history, that is the pages visited in the tab or frame that the current page is loaded in. */\ninterface History {\n    readonly length: number;\n    scrollRestoration: ScrollRestoration;\n    readonly state: any;\n    back(): void;\n    forward(): void;\n    go(delta?: number): void;\n    pushState(data: any, unused: string, url?: string | URL | null): void;\n    replaceState(data: any, unused: string, url?: string | URL | null): void;\n}\n\ndeclare var History: {\n    prototype: History;\n    new(): History;\n};\n\n/** This IndexedDB API interface represents a cursor for traversing or iterating over multiple records in a database. */\ninterface IDBCursor {\n    /**\n     * Returns the direction ("next", "nextunique", "prev" or "prevunique") of the cursor.\n     */\n    readonly direction: IDBCursorDirection;\n    /**\n     * Returns the key of the cursor. Throws a "InvalidStateError" DOMException if the cursor is advancing or is finished.\n     */\n    readonly key: IDBValidKey;\n    /**\n     * Returns the effective key of the cursor. Throws a "InvalidStateError" DOMException if the cursor is advancing or is finished.\n     */\n    readonly primaryKey: IDBValidKey;\n    readonly request: IDBRequest;\n    /**\n     * Returns the IDBObjectStore or IDBIndex the cursor was opened from.\n     */\n    readonly source: IDBObjectStore | IDBIndex;\n    /**\n     * Advances the cursor through the next count records in range.\n     */\n    advance(count: number): void;\n    /**\n     * Advances the cursor to the next record in range.\n     */\n    continue(key?: IDBValidKey): void;\n    /**\n     * Advances the cursor to the next record in range matching or after key and primaryKey. Throws an "InvalidAccessError" DOMException if the source is not an index.\n     */\n    continuePrimaryKey(key: IDBValidKey, primaryKey: IDBValidKey): void;\n    /**\n     * Delete the record pointed at by the cursor with a new value.\n     *\n     * If successful, request\'s result will be undefined.\n     */\n    delete(): IDBRequest<undefined>;\n    /**\n     * Updated the record pointed at by the cursor with a new value.\n     *\n     * Throws a "DataError" DOMException if the effective object store uses in-line keys and the key would have changed.\n     *\n     * If successful, request\'s result will be the record\'s key.\n     */\n    update(value: any): IDBRequest<IDBValidKey>;\n}\n\ndeclare var IDBCursor: {\n    prototype: IDBCursor;\n    new(): IDBCursor;\n};\n\n/** This IndexedDB API interface represents a cursor for traversing or iterating over multiple records in a database. It is the same as the IDBCursor, except that it includes the value property. */\ninterface IDBCursorWithValue extends IDBCursor {\n    /**\n     * Returns the cursor\'s current value.\n     */\n    readonly value: any;\n}\n\ndeclare var IDBCursorWithValue: {\n    prototype: IDBCursorWithValue;\n    new(): IDBCursorWithValue;\n};\n\ninterface IDBDatabaseEventMap {\n    "abort": Event;\n    "close": Event;\n    "error": Event;\n    "versionchange": IDBVersionChangeEvent;\n}\n\n/** This IndexedDB API interface provides a connection to a database; you can use an IDBDatabase object to open a transaction on your database then create, manipulate, and delete objects (data) in that database. The interface provides the only way to get and manage versions of the database. */\ninterface IDBDatabase extends EventTarget {\n    /**\n     * Returns the name of the database.\n     */\n    readonly name: string;\n    /**\n     * Returns a list of the names of object stores in the database.\n     */\n    readonly objectStoreNames: DOMStringList;\n    onabort: ((this: IDBDatabase, ev: Event) => any) | null;\n    onclose: ((this: IDBDatabase, ev: Event) => any) | null;\n    onerror: ((this: IDBDatabase, ev: Event) => any) | null;\n    onversionchange: ((this: IDBDatabase, ev: IDBVersionChangeEvent) => any) | null;\n    /**\n     * Returns the version of the database.\n     */\n    readonly version: number;\n    /**\n     * Closes the connection once all running transactions have finished.\n     */\n    close(): void;\n    /**\n     * Creates a new object store with the given name and options and returns a new IDBObjectStore.\n     *\n     * Throws a "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    createObjectStore(name: string, options?: IDBObjectStoreParameters): IDBObjectStore;\n    /**\n     * Deletes the object store with the given name.\n     *\n     * Throws a "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    deleteObjectStore(name: string): void;\n    /**\n     * Returns a new transaction with the given mode ("readonly" or "readwrite") and scope which can be a single object store name or an array of names.\n     */\n    transaction(storeNames: string | string[], mode?: IDBTransactionMode): IDBTransaction;\n    addEventListener<K extends keyof IDBDatabaseEventMap>(type: K, listener: (this: IDBDatabase, ev: IDBDatabaseEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof IDBDatabaseEventMap>(type: K, listener: (this: IDBDatabase, ev: IDBDatabaseEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var IDBDatabase: {\n    prototype: IDBDatabase;\n    new(): IDBDatabase;\n};\n\n/** In the following code snippet, we make a request to open a database, and include handlers for the success and error cases. For a full working example, see our To-do Notifications app (view example live.) */\ninterface IDBFactory {\n    /**\n     * Compares two values as keys. Returns -1 if key1 precedes key2, 1 if key2 precedes key1, and 0 if the keys are equal.\n     *\n     * Throws a "DataError" DOMException if either input is not a valid key.\n     */\n    cmp(first: any, second: any): number;\n    databases(): Promise<IDBDatabaseInfo[]>;\n    /**\n     * Attempts to delete the named database. If the database already exists and there are open connections that don\'t close in response to a versionchange event, the request will be blocked until all they close. If the request is successful request\'s result will be null.\n     */\n    deleteDatabase(name: string): IDBOpenDBRequest;\n    /**\n     * Attempts to open a connection to the named database with the current version, or 1 if it does not already exist. If the request is successful request\'s result will be the connection.\n     */\n    open(name: string, version?: number): IDBOpenDBRequest;\n}\n\ndeclare var IDBFactory: {\n    prototype: IDBFactory;\n    new(): IDBFactory;\n};\n\n/** IDBIndex interface of the IndexedDB API provides asynchronous access to an index in a database. An index is a kind of object store for looking up records in another object store, called the referenced object store. You use this interface to retrieve data. */\ninterface IDBIndex {\n    readonly keyPath: string | string[];\n    readonly multiEntry: boolean;\n    /**\n     * Returns the name of the index.\n     */\n    name: string;\n    /**\n     * Returns the IDBObjectStore the index belongs to.\n     */\n    readonly objectStore: IDBObjectStore;\n    readonly unique: boolean;\n    /**\n     * Retrieves the number of records matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the count.\n     */\n    count(query?: IDBValidKey | IDBKeyRange): IDBRequest<number>;\n    /**\n     * Retrieves the value of the first record matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the value, or undefined if there was no matching record.\n     */\n    get(query: IDBValidKey | IDBKeyRange): IDBRequest<any>;\n    /**\n     * Retrieves the values of the records matching the given key or key range in query (up to count if given).\n     *\n     * If successful, request\'s result will be an Array of the values.\n     */\n    getAll(query?: IDBValidKey | IDBKeyRange | null, count?: number): IDBRequest<any[]>;\n    /**\n     * Retrieves the keys of records matching the given key or key range in query (up to count if given).\n     *\n     * If successful, request\'s result will be an Array of the keys.\n     */\n    getAllKeys(query?: IDBValidKey | IDBKeyRange | null, count?: number): IDBRequest<IDBValidKey[]>;\n    /**\n     * Retrieves the key of the first record matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the key, or undefined if there was no matching record.\n     */\n    getKey(query: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey | undefined>;\n    /**\n     * Opens a cursor over the records matching query, ordered by direction. If query is null, all records in index are matched.\n     *\n     * If successful, request\'s result will be an IDBCursorWithValue, or null if there were no matching records.\n     */\n    openCursor(query?: IDBValidKey | IDBKeyRange | null, direction?: IDBCursorDirection): IDBRequest<IDBCursorWithValue | null>;\n    /**\n     * Opens a cursor with key only flag set over the records matching query, ordered by direction. If query is null, all records in index are matched.\n     *\n     * If successful, request\'s result will be an IDBCursor, or null if there were no matching records.\n     */\n    openKeyCursor(query?: IDBValidKey | IDBKeyRange | null, direction?: IDBCursorDirection): IDBRequest<IDBCursor | null>;\n}\n\ndeclare var IDBIndex: {\n    prototype: IDBIndex;\n    new(): IDBIndex;\n};\n\n/** A key range can be a single value or a range with upper and lower bounds or endpoints. If the key range has both upper and lower bounds, then it is bounded; if it has no bounds, it is unbounded. A bounded key range can either be open (the endpoints are excluded) or closed (the endpoints are included). To retrieve all keys within a certain range, you can use the following code constructs: */\ninterface IDBKeyRange {\n    /**\n     * Returns lower bound, or undefined if none.\n     */\n    readonly lower: any;\n    /**\n     * Returns true if the lower open flag is set, and false otherwise.\n     */\n    readonly lowerOpen: boolean;\n    /**\n     * Returns upper bound, or undefined if none.\n     */\n    readonly upper: any;\n    /**\n     * Returns true if the upper open flag is set, and false otherwise.\n     */\n    readonly upperOpen: boolean;\n    /**\n     * Returns true if key is included in the range, and false otherwise.\n     */\n    includes(key: any): boolean;\n}\n\ndeclare var IDBKeyRange: {\n    prototype: IDBKeyRange;\n    new(): IDBKeyRange;\n    /**\n     * Returns a new IDBKeyRange spanning from lower to upper. If lowerOpen is true, lower is not included in the range. If upperOpen is true, upper is not included in the range.\n     */\n    bound(lower: any, upper: any, lowerOpen?: boolean, upperOpen?: boolean): IDBKeyRange;\n    /**\n     * Returns a new IDBKeyRange starting at key with no upper bound. If open is true, key is not included in the range.\n     */\n    lowerBound(lower: any, open?: boolean): IDBKeyRange;\n    /**\n     * Returns a new IDBKeyRange spanning only key.\n     */\n    only(value: any): IDBKeyRange;\n    /**\n     * Returns a new IDBKeyRange with no lower bound and ending at key. If open is true, key is not included in the range.\n     */\n    upperBound(upper: any, open?: boolean): IDBKeyRange;\n};\n\n/** This example shows a variety of different uses of object stores, from updating the data structure with IDBObjectStore.createIndex inside an onupgradeneeded function, to adding a new item to our object store with IDBObjectStore.add. For a full working example, see our To-do Notifications app (view example live.) */\ninterface IDBObjectStore {\n    /**\n     * Returns true if the store has a key generator, and false otherwise.\n     */\n    readonly autoIncrement: boolean;\n    /**\n     * Returns a list of the names of indexes in the store.\n     */\n    readonly indexNames: DOMStringList;\n    /**\n     * Returns the key path of the store, or null if none.\n     */\n    readonly keyPath: string | string[];\n    /**\n     * Returns the name of the store.\n     */\n    name: string;\n    /**\n     * Returns the associated transaction.\n     */\n    readonly transaction: IDBTransaction;\n    /**\n     * Adds or updates a record in store with the given value and key.\n     *\n     * If the store uses in-line keys and key is specified a "DataError" DOMException will be thrown.\n     *\n     * If put() is used, any existing record with the key will be replaced. If add() is used, and if a record with the key already exists the request will fail, with request\'s error set to a "ConstraintError" DOMException.\n     *\n     * If successful, request\'s result will be the record\'s key.\n     */\n    add(value: any, key?: IDBValidKey): IDBRequest<IDBValidKey>;\n    /**\n     * Deletes all records in store.\n     *\n     * If successful, request\'s result will be undefined.\n     */\n    clear(): IDBRequest<undefined>;\n    /**\n     * Retrieves the number of records matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the count.\n     */\n    count(query?: IDBValidKey | IDBKeyRange): IDBRequest<number>;\n    /**\n     * Creates a new index in store with the given name, keyPath and options and returns a new IDBIndex. If the keyPath and options define constraints that cannot be satisfied with the data already in store the upgrade transaction will abort with a "ConstraintError" DOMException.\n     *\n     * Throws an "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    createIndex(name: string, keyPath: string | string[], options?: IDBIndexParameters): IDBIndex;\n    /**\n     * Deletes records in store with the given key or in the given key range in query.\n     *\n     * If successful, request\'s result will be undefined.\n     */\n    delete(query: IDBValidKey | IDBKeyRange): IDBRequest<undefined>;\n    /**\n     * Deletes the index in store with the given name.\n     *\n     * Throws an "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    deleteIndex(name: string): void;\n    /**\n     * Retrieves the value of the first record matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the value, or undefined if there was no matching record.\n     */\n    get(query: IDBValidKey | IDBKeyRange): IDBRequest<any>;\n    /**\n     * Retrieves the values of the records matching the given key or key range in query (up to count if given).\n     *\n     * If successful, request\'s result will be an Array of the values.\n     */\n    getAll(query?: IDBValidKey | IDBKeyRange | null, count?: number): IDBRequest<any[]>;\n    /**\n     * Retrieves the keys of records matching the given key or key range in query (up to count if given).\n     *\n     * If successful, request\'s result will be an Array of the keys.\n     */\n    getAllKeys(query?: IDBValidKey | IDBKeyRange | null, count?: number): IDBRequest<IDBValidKey[]>;\n    /**\n     * Retrieves the key of the first record matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the key, or undefined if there was no matching record.\n     */\n    getKey(query: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey | undefined>;\n    index(name: string): IDBIndex;\n    /**\n     * Opens a cursor over the records matching query, ordered by direction. If query is null, all records in store are matched.\n     *\n     * If successful, request\'s result will be an IDBCursorWithValue pointing at the first matching record, or null if there were no matching records.\n     */\n    openCursor(query?: IDBValidKey | IDBKeyRange | null, direction?: IDBCursorDirection): IDBRequest<IDBCursorWithValue | null>;\n    /**\n     * Opens a cursor with key only flag set over the records matching query, ordered by direction. If query is null, all records in store are matched.\n     *\n     * If successful, request\'s result will be an IDBCursor pointing at the first matching record, or null if there were no matching records.\n     */\n    openKeyCursor(query?: IDBValidKey | IDBKeyRange | null, direction?: IDBCursorDirection): IDBRequest<IDBCursor | null>;\n    /**\n     * Adds or updates a record in store with the given value and key.\n     *\n     * If the store uses in-line keys and key is specified a "DataError" DOMException will be thrown.\n     *\n     * If put() is used, any existing record with the key will be replaced. If add() is used, and if a record with the key already exists the request will fail, with request\'s error set to a "ConstraintError" DOMException.\n     *\n     * If successful, request\'s result will be the record\'s key.\n     */\n    put(value: any, key?: IDBValidKey): IDBRequest<IDBValidKey>;\n}\n\ndeclare var IDBObjectStore: {\n    prototype: IDBObjectStore;\n    new(): IDBObjectStore;\n};\n\ninterface IDBOpenDBRequestEventMap extends IDBRequestEventMap {\n    "blocked": Event;\n    "upgradeneeded": IDBVersionChangeEvent;\n}\n\n/** Also inherits methods from its parents IDBRequest and EventTarget. */\ninterface IDBOpenDBRequest extends IDBRequest<IDBDatabase> {\n    onblocked: ((this: IDBOpenDBRequest, ev: Event) => any) | null;\n    onupgradeneeded: ((this: IDBOpenDBRequest, ev: IDBVersionChangeEvent) => any) | null;\n    addEventListener<K extends keyof IDBOpenDBRequestEventMap>(type: K, listener: (this: IDBOpenDBRequest, ev: IDBOpenDBRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof IDBOpenDBRequestEventMap>(type: K, listener: (this: IDBOpenDBRequest, ev: IDBOpenDBRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var IDBOpenDBRequest: {\n    prototype: IDBOpenDBRequest;\n    new(): IDBOpenDBRequest;\n};\n\ninterface IDBRequestEventMap {\n    "error": Event;\n    "success": Event;\n}\n\n/** The request object does not initially contain any information about the result of the operation, but once information becomes available, an event is fired on the request, and the information becomes available through the properties of the IDBRequest instance. */\ninterface IDBRequest<T = any> extends EventTarget {\n    /**\n     * When a request is completed, returns the error (a DOMException), or null if the request succeeded. Throws a "InvalidStateError" DOMException if the request is still pending.\n     */\n    readonly error: DOMException | null;\n    onerror: ((this: IDBRequest<T>, ev: Event) => any) | null;\n    onsuccess: ((this: IDBRequest<T>, ev: Event) => any) | null;\n    /**\n     * Returns "pending" until a request is complete, then returns "done".\n     */\n    readonly readyState: IDBRequestReadyState;\n    /**\n     * When a request is completed, returns the result, or undefined if the request failed. Throws a "InvalidStateError" DOMException if the request is still pending.\n     */\n    readonly result: T;\n    /**\n     * Returns the IDBObjectStore, IDBIndex, or IDBCursor the request was made against, or null if is was an open request.\n     */\n    readonly source: IDBObjectStore | IDBIndex | IDBCursor;\n    /**\n     * Returns the IDBTransaction the request was made within. If this as an open request, then it returns an upgrade transaction while it is running, or null otherwise.\n     */\n    readonly transaction: IDBTransaction | null;\n    addEventListener<K extends keyof IDBRequestEventMap>(type: K, listener: (this: IDBRequest<T>, ev: IDBRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof IDBRequestEventMap>(type: K, listener: (this: IDBRequest<T>, ev: IDBRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var IDBRequest: {\n    prototype: IDBRequest;\n    new(): IDBRequest;\n};\n\ninterface IDBTransactionEventMap {\n    "abort": Event;\n    "complete": Event;\n    "error": Event;\n}\n\ninterface IDBTransaction extends EventTarget {\n    /**\n     * Returns the transaction\'s connection.\n     */\n    readonly db: IDBDatabase;\n    /**\n     * If the transaction was aborted, returns the error (a DOMException) providing the reason.\n     */\n    readonly error: DOMException | null;\n    /**\n     * Returns the mode the transaction was created with ("readonly" or "readwrite"), or "versionchange" for an upgrade transaction.\n     */\n    readonly mode: IDBTransactionMode;\n    /**\n     * Returns a list of the names of object stores in the transaction\'s scope. For an upgrade transaction this is all object stores in the database.\n     */\n    readonly objectStoreNames: DOMStringList;\n    onabort: ((this: IDBTransaction, ev: Event) => any) | null;\n    oncomplete: ((this: IDBTransaction, ev: Event) => any) | null;\n    onerror: ((this: IDBTransaction, ev: Event) => any) | null;\n    /**\n     * Aborts the transaction. All pending requests will fail with a "AbortError" DOMException and all changes made to the database will be reverted.\n     */\n    abort(): void;\n    commit(): void;\n    /**\n     * Returns an IDBObjectStore in the transaction\'s scope.\n     */\n    objectStore(name: string): IDBObjectStore;\n    addEventListener<K extends keyof IDBTransactionEventMap>(type: K, listener: (this: IDBTransaction, ev: IDBTransactionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof IDBTransactionEventMap>(type: K, listener: (this: IDBTransaction, ev: IDBTransactionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var IDBTransaction: {\n    prototype: IDBTransaction;\n    new(): IDBTransaction;\n};\n\n/** This IndexedDB API interface indicates that the version of the database has changed, as the result of an IDBOpenDBRequest.onupgradeneeded event handler function. */\ninterface IDBVersionChangeEvent extends Event {\n    readonly newVersion: number | null;\n    readonly oldVersion: number;\n}\n\ndeclare var IDBVersionChangeEvent: {\n    prototype: IDBVersionChangeEvent;\n    new(type: string, eventInitDict?: IDBVersionChangeEventInit): IDBVersionChangeEvent;\n};\n\n/** The IIRFilterNode interface of the Web Audio API is a AudioNode processor which implements a general infinite impulse response (IIR)  filter; this type of filter can be used to implement tone control devices and graphic equalizers as well. It lets the parameters of the filter response be specified, so that it can be tuned as needed. */\ninterface IIRFilterNode extends AudioNode {\n    getFrequencyResponse(frequencyHz: Float32Array, magResponse: Float32Array, phaseResponse: Float32Array): void;\n}\n\ndeclare var IIRFilterNode: {\n    prototype: IIRFilterNode;\n    new(context: BaseAudioContext, options: IIRFilterOptions): IIRFilterNode;\n};\n\ninterface IdleDeadline {\n    readonly didTimeout: boolean;\n    timeRemaining(): DOMHighResTimeStamp;\n}\n\ndeclare var IdleDeadline: {\n    prototype: IdleDeadline;\n    new(): IdleDeadline;\n};\n\ninterface ImageBitmap {\n    /**\n     * Returns the intrinsic height of the image, in CSS pixels.\n     */\n    readonly height: number;\n    /**\n     * Returns the intrinsic width of the image, in CSS pixels.\n     */\n    readonly width: number;\n    /**\n     * Releases imageBitmap\'s underlying bitmap data.\n     */\n    close(): void;\n}\n\ndeclare var ImageBitmap: {\n    prototype: ImageBitmap;\n    new(): ImageBitmap;\n};\n\ninterface ImageBitmapRenderingContext {\n    /**\n     * Returns the canvas element that the context is bound to.\n     */\n    readonly canvas: HTMLCanvasElement;\n    /**\n     * Transfers the underlying bitmap data from imageBitmap to context, and the bitmap becomes the contents of the canvas element to which context is bound.\n     */\n    transferFromImageBitmap(bitmap: ImageBitmap | null): void;\n}\n\ndeclare var ImageBitmapRenderingContext: {\n    prototype: ImageBitmapRenderingContext;\n    new(): ImageBitmapRenderingContext;\n};\n\n/** The underlying pixel data of an area of a <canvas> element. It is created using the ImageData() constructor or creator methods on the CanvasRenderingContext2D object associated with a canvas: createImageData() and getImageData(). It can also be used to set a part of the canvas by using putImageData(). */\ninterface ImageData {\n    /**\n     * Returns the one-dimensional array containing the data in RGBA order, as integers in the range 0 to 255.\n     */\n    readonly data: Uint8ClampedArray;\n    /**\n     * Returns the actual dimensions of the data in the ImageData object, in pixels.\n     */\n    readonly height: number;\n    /**\n     * Returns the actual dimensions of the data in the ImageData object, in pixels.\n     */\n    readonly width: number;\n}\n\ndeclare var ImageData: {\n    prototype: ImageData;\n    new(sw: number, sh: number, settings?: ImageDataSettings): ImageData;\n    new(data: Uint8ClampedArray, sw: number, sh?: number, settings?: ImageDataSettings): ImageData;\n};\n\ninterface InnerHTML {\n    innerHTML: string;\n}\n\ninterface InputEvent extends UIEvent {\n    readonly data: string | null;\n    readonly dataTransfer: DataTransfer | null;\n    readonly inputType: string;\n    readonly isComposing: boolean;\n    getTargetRanges(): StaticRange[];\n}\n\ndeclare var InputEvent: {\n    prototype: InputEvent;\n    new(type: string, eventInitDict?: InputEventInit): InputEvent;\n};\n\n/** provides a way to asynchronously observe changes in the intersection of a target element with an ancestor element or with a top-level document\'s viewport. */\ninterface IntersectionObserver {\n    readonly root: Element | Document | null;\n    readonly rootMargin: string;\n    readonly thresholds: ReadonlyArray<number>;\n    disconnect(): void;\n    observe(target: Element): void;\n    takeRecords(): IntersectionObserverEntry[];\n    unobserve(target: Element): void;\n}\n\ndeclare var IntersectionObserver: {\n    prototype: IntersectionObserver;\n    new(callback: IntersectionObserverCallback, options?: IntersectionObserverInit): IntersectionObserver;\n};\n\n/** This Intersection Observer API interface describes the intersection between the target element and its root container at a specific moment of transition. */\ninterface IntersectionObserverEntry {\n    readonly boundingClientRect: DOMRectReadOnly;\n    readonly intersectionRatio: number;\n    readonly intersectionRect: DOMRectReadOnly;\n    readonly isIntersecting: boolean;\n    readonly rootBounds: DOMRectReadOnly | null;\n    readonly target: Element;\n    readonly time: DOMHighResTimeStamp;\n}\n\ndeclare var IntersectionObserverEntry: {\n    prototype: IntersectionObserverEntry;\n    new(intersectionObserverEntryInit: IntersectionObserverEntryInit): IntersectionObserverEntry;\n};\n\ninterface KHR_parallel_shader_compile {\n    readonly COMPLETION_STATUS_KHR: GLenum;\n}\n\n/** KeyboardEvent objects describe a user interaction with the keyboard; each event describes a single interaction between the user and a key (or combination of a key with modifier keys) on the keyboard. */\ninterface KeyboardEvent extends UIEvent {\n    readonly altKey: boolean;\n    /** @deprecated */\n    readonly charCode: number;\n    readonly code: string;\n    readonly ctrlKey: boolean;\n    readonly isComposing: boolean;\n    readonly key: string;\n    /** @deprecated */\n    readonly keyCode: number;\n    readonly location: number;\n    readonly metaKey: boolean;\n    readonly repeat: boolean;\n    readonly shiftKey: boolean;\n    getModifierState(keyArg: string): boolean;\n    /** @deprecated */\n    initKeyboardEvent(typeArg: string, bubblesArg?: boolean, cancelableArg?: boolean, viewArg?: Window | null, keyArg?: string, locationArg?: number, ctrlKey?: boolean, altKey?: boolean, shiftKey?: boolean, metaKey?: boolean): void;\n    readonly DOM_KEY_LOCATION_LEFT: number;\n    readonly DOM_KEY_LOCATION_NUMPAD: number;\n    readonly DOM_KEY_LOCATION_RIGHT: number;\n    readonly DOM_KEY_LOCATION_STANDARD: number;\n}\n\ndeclare var KeyboardEvent: {\n    prototype: KeyboardEvent;\n    new(type: string, eventInitDict?: KeyboardEventInit): KeyboardEvent;\n    readonly DOM_KEY_LOCATION_LEFT: number;\n    readonly DOM_KEY_LOCATION_NUMPAD: number;\n    readonly DOM_KEY_LOCATION_RIGHT: number;\n    readonly DOM_KEY_LOCATION_STANDARD: number;\n};\n\ninterface KeyframeEffect extends AnimationEffect {\n    composite: CompositeOperation;\n    iterationComposite: IterationCompositeOperation;\n    pseudoElement: string | null;\n    target: Element | null;\n    getKeyframes(): ComputedKeyframe[];\n    setKeyframes(keyframes: Keyframe[] | PropertyIndexedKeyframes | null): void;\n}\n\ndeclare var KeyframeEffect: {\n    prototype: KeyframeEffect;\n    new(target: Element | null, keyframes: Keyframe[] | PropertyIndexedKeyframes | null, options?: number | KeyframeEffectOptions): KeyframeEffect;\n    new(source: KeyframeEffect): KeyframeEffect;\n};\n\ninterface LinkStyle {\n    readonly sheet: CSSStyleSheet | null;\n}\n\n/** The location (URL) of the object it is linked to. Changes done on it are reflected on the object it relates to. Both the Document and Window interface have such a linked Location, accessible via Document.location and Window.location respectively. */\ninterface Location {\n    /**\n     * Returns a DOMStringList object listing the origins of the ancestor browsing contexts, from the parent browsing context to the top-level browsing context.\n     */\n    readonly ancestorOrigins: DOMStringList;\n    /**\n     * Returns the Location object\'s URL\'s fragment (includes leading "#" if non-empty).\n     *\n     * Can be set, to navigate to the same URL with a changed fragment (ignores leading "#").\n     */\n    hash: string;\n    /**\n     * Returns the Location object\'s URL\'s host and port (if different from the default port for the scheme).\n     *\n     * Can be set, to navigate to the same URL with a changed host and port.\n     */\n    host: string;\n    /**\n     * Returns the Location object\'s URL\'s host.\n     *\n     * Can be set, to navigate to the same URL with a changed host.\n     */\n    hostname: string;\n    /**\n     * Returns the Location object\'s URL.\n     *\n     * Can be set, to navigate to the given URL.\n     */\n    href: string;\n    toString(): string;\n    /**\n     * Returns the Location object\'s URL\'s origin.\n     */\n    readonly origin: string;\n    /**\n     * Returns the Location object\'s URL\'s path.\n     *\n     * Can be set, to navigate to the same URL with a changed path.\n     */\n    pathname: string;\n    /**\n     * Returns the Location object\'s URL\'s port.\n     *\n     * Can be set, to navigate to the same URL with a changed port.\n     */\n    port: string;\n    /**\n     * Returns the Location object\'s URL\'s scheme.\n     *\n     * Can be set, to navigate to the same URL with a changed scheme.\n     */\n    protocol: string;\n    /**\n     * Returns the Location object\'s URL\'s query (includes leading "?" if non-empty).\n     *\n     * Can be set, to navigate to the same URL with a changed query (ignores leading "?").\n     */\n    search: string;\n    /**\n     * Navigates to the given URL.\n     */\n    assign(url: string | URL): void;\n    /**\n     * Reloads the current page.\n     */\n    reload(): void;\n    /**\n     * Removes the current page from the session history and navigates to the given URL.\n     */\n    replace(url: string | URL): void;\n}\n\ndeclare var Location: {\n    prototype: Location;\n    new(): Location;\n};\n\ninterface MathMLElementEventMap extends ElementEventMap, DocumentAndElementEventHandlersEventMap, GlobalEventHandlersEventMap {\n}\n\ninterface MathMLElement extends Element, DocumentAndElementEventHandlers, ElementCSSInlineStyle, GlobalEventHandlers, HTMLOrSVGElement {\n    addEventListener<K extends keyof MathMLElementEventMap>(type: K, listener: (this: MathMLElement, ev: MathMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MathMLElementEventMap>(type: K, listener: (this: MathMLElement, ev: MathMLElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MathMLElement: {\n    prototype: MathMLElement;\n    new(): MathMLElement;\n};\n\ninterface MediaCapabilities {\n    decodingInfo(configuration: MediaDecodingConfiguration): Promise<MediaCapabilitiesDecodingInfo>;\n    encodingInfo(configuration: MediaEncodingConfiguration): Promise<MediaCapabilitiesEncodingInfo>;\n}\n\ndeclare var MediaCapabilities: {\n    prototype: MediaCapabilities;\n    new(): MediaCapabilities;\n};\n\n/** The MediaDevicesInfo interface contains information that describes a single media input or output device. */\ninterface MediaDeviceInfo {\n    readonly deviceId: string;\n    readonly groupId: string;\n    readonly kind: MediaDeviceKind;\n    readonly label: string;\n    toJSON(): any;\n}\n\ndeclare var MediaDeviceInfo: {\n    prototype: MediaDeviceInfo;\n    new(): MediaDeviceInfo;\n};\n\ninterface MediaDevicesEventMap {\n    "devicechange": Event;\n}\n\n/** Provides access to connected media input devices like cameras and microphones, as well as screen sharing. In essence, it lets you obtain access to any hardware source of media data. */\ninterface MediaDevices extends EventTarget {\n    ondevicechange: ((this: MediaDevices, ev: Event) => any) | null;\n    enumerateDevices(): Promise<MediaDeviceInfo[]>;\n    getDisplayMedia(constraints?: DisplayMediaStreamConstraints): Promise<MediaStream>;\n    getSupportedConstraints(): MediaTrackSupportedConstraints;\n    getUserMedia(constraints?: MediaStreamConstraints): Promise<MediaStream>;\n    addEventListener<K extends keyof MediaDevicesEventMap>(type: K, listener: (this: MediaDevices, ev: MediaDevicesEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MediaDevicesEventMap>(type: K, listener: (this: MediaDevices, ev: MediaDevicesEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MediaDevices: {\n    prototype: MediaDevices;\n    new(): MediaDevices;\n};\n\n/** A MediaElementSourceNode has no inputs and exactly one output, and is created using the AudioContext.createMediaElementSource method. The amount of channels in the output equals the number of channels of the audio referenced by the HTMLMediaElement used in the creation of the node, or is 1 if the HTMLMediaElement has no audio. */\ninterface MediaElementAudioSourceNode extends AudioNode {\n    readonly mediaElement: HTMLMediaElement;\n}\n\ndeclare var MediaElementAudioSourceNode: {\n    prototype: MediaElementAudioSourceNode;\n    new(context: AudioContext, options: MediaElementAudioSourceOptions): MediaElementAudioSourceNode;\n};\n\ninterface MediaEncryptedEvent extends Event {\n    readonly initData: ArrayBuffer | null;\n    readonly initDataType: string;\n}\n\ndeclare var MediaEncryptedEvent: {\n    prototype: MediaEncryptedEvent;\n    new(type: string, eventInitDict?: MediaEncryptedEventInit): MediaEncryptedEvent;\n};\n\n/** An error which occurred while handling media in an HTML media element based on HTMLMediaElement, such as <audio> or <video>. */\ninterface MediaError {\n    readonly code: number;\n    readonly message: string;\n    readonly MEDIA_ERR_ABORTED: number;\n    readonly MEDIA_ERR_DECODE: number;\n    readonly MEDIA_ERR_NETWORK: number;\n    readonly MEDIA_ERR_SRC_NOT_SUPPORTED: number;\n}\n\ndeclare var MediaError: {\n    prototype: MediaError;\n    new(): MediaError;\n    readonly MEDIA_ERR_ABORTED: number;\n    readonly MEDIA_ERR_DECODE: number;\n    readonly MEDIA_ERR_NETWORK: number;\n    readonly MEDIA_ERR_SRC_NOT_SUPPORTED: number;\n};\n\n/** This EncryptedMediaExtensions API interface contains the content and related data when the content decryption module generates a message for the session. */\ninterface MediaKeyMessageEvent extends Event {\n    readonly message: ArrayBuffer;\n    readonly messageType: MediaKeyMessageType;\n}\n\ndeclare var MediaKeyMessageEvent: {\n    prototype: MediaKeyMessageEvent;\n    new(type: string, eventInitDict: MediaKeyMessageEventInit): MediaKeyMessageEvent;\n};\n\ninterface MediaKeySessionEventMap {\n    "keystatuseschange": Event;\n    "message": MediaKeyMessageEvent;\n}\n\n/** This EncryptedMediaExtensions API interface represents a context for message exchange with a content decryption module (CDM). */\ninterface MediaKeySession extends EventTarget {\n    readonly closed: Promise<MediaKeySessionClosedReason>;\n    readonly expiration: number;\n    readonly keyStatuses: MediaKeyStatusMap;\n    onkeystatuseschange: ((this: MediaKeySession, ev: Event) => any) | null;\n    onmessage: ((this: MediaKeySession, ev: MediaKeyMessageEvent) => any) | null;\n    readonly sessionId: string;\n    close(): Promise<void>;\n    generateRequest(initDataType: string, initData: BufferSource): Promise<void>;\n    load(sessionId: string): Promise<boolean>;\n    remove(): Promise<void>;\n    update(response: BufferSource): Promise<void>;\n    addEventListener<K extends keyof MediaKeySessionEventMap>(type: K, listener: (this: MediaKeySession, ev: MediaKeySessionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MediaKeySessionEventMap>(type: K, listener: (this: MediaKeySession, ev: MediaKeySessionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MediaKeySession: {\n    prototype: MediaKeySession;\n    new(): MediaKeySession;\n};\n\n/** This EncryptedMediaExtensions API interface is a read-only map of media key statuses by key IDs. */\ninterface MediaKeyStatusMap {\n    readonly size: number;\n    get(keyId: BufferSource): MediaKeyStatus | undefined;\n    has(keyId: BufferSource): boolean;\n    forEach(callbackfn: (value: MediaKeyStatus, key: BufferSource, parent: MediaKeyStatusMap) => void, thisArg?: any): void;\n}\n\ndeclare var MediaKeyStatusMap: {\n    prototype: MediaKeyStatusMap;\n    new(): MediaKeyStatusMap;\n};\n\n/** This EncryptedMediaExtensions API interface provides access to a Key System for decryption and/or a content protection provider. You can request an instance of this object using the Navigator.requestMediaKeySystemAccess method. */\ninterface MediaKeySystemAccess {\n    readonly keySystem: string;\n    createMediaKeys(): Promise<MediaKeys>;\n    getConfiguration(): MediaKeySystemConfiguration;\n}\n\ndeclare var MediaKeySystemAccess: {\n    prototype: MediaKeySystemAccess;\n    new(): MediaKeySystemAccess;\n};\n\n/** This EncryptedMediaExtensions API interface the represents a set of keys that an associated HTMLMediaElement can use for decryption of media data during playback. */\ninterface MediaKeys {\n    createSession(sessionType?: MediaKeySessionType): MediaKeySession;\n    setServerCertificate(serverCertificate: BufferSource): Promise<boolean>;\n}\n\ndeclare var MediaKeys: {\n    prototype: MediaKeys;\n    new(): MediaKeys;\n};\n\ninterface MediaList {\n    readonly length: number;\n    mediaText: string;\n    toString(): string;\n    appendMedium(medium: string): void;\n    deleteMedium(medium: string): void;\n    item(index: number): string | null;\n    [index: number]: string;\n}\n\ndeclare var MediaList: {\n    prototype: MediaList;\n    new(): MediaList;\n};\n\ninterface MediaMetadata {\n    album: string;\n    artist: string;\n    artwork: ReadonlyArray<MediaImage>;\n    title: string;\n}\n\ndeclare var MediaMetadata: {\n    prototype: MediaMetadata;\n    new(init?: MediaMetadataInit): MediaMetadata;\n};\n\ninterface MediaQueryListEventMap {\n    "change": MediaQueryListEvent;\n}\n\n/** Stores information on a media query applied to a document, and handles sending notifications to listeners when the media query state change (i.e. when the media query test starts or stops evaluating to true). */\ninterface MediaQueryList extends EventTarget {\n    readonly matches: boolean;\n    readonly media: string;\n    onchange: ((this: MediaQueryList, ev: MediaQueryListEvent) => any) | null;\n    /** @deprecated */\n    addListener(callback: ((this: MediaQueryList, ev: MediaQueryListEvent) => any) | null): void;\n    /** @deprecated */\n    removeListener(callback: ((this: MediaQueryList, ev: MediaQueryListEvent) => any) | null): void;\n    addEventListener<K extends keyof MediaQueryListEventMap>(type: K, listener: (this: MediaQueryList, ev: MediaQueryListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MediaQueryListEventMap>(type: K, listener: (this: MediaQueryList, ev: MediaQueryListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MediaQueryList: {\n    prototype: MediaQueryList;\n    new(): MediaQueryList;\n};\n\ninterface MediaQueryListEvent extends Event {\n    readonly matches: boolean;\n    readonly media: string;\n}\n\ndeclare var MediaQueryListEvent: {\n    prototype: MediaQueryListEvent;\n    new(type: string, eventInitDict?: MediaQueryListEventInit): MediaQueryListEvent;\n};\n\ninterface MediaRecorderEventMap {\n    "dataavailable": BlobEvent;\n    "error": Event;\n    "pause": Event;\n    "resume": Event;\n    "start": Event;\n    "stop": Event;\n}\n\ninterface MediaRecorder extends EventTarget {\n    readonly audioBitsPerSecond: number;\n    readonly mimeType: string;\n    ondataavailable: ((this: MediaRecorder, ev: BlobEvent) => any) | null;\n    onerror: ((this: MediaRecorder, ev: Event) => any) | null;\n    onpause: ((this: MediaRecorder, ev: Event) => any) | null;\n    onresume: ((this: MediaRecorder, ev: Event) => any) | null;\n    onstart: ((this: MediaRecorder, ev: Event) => any) | null;\n    onstop: ((this: MediaRecorder, ev: Event) => any) | null;\n    readonly state: RecordingState;\n    readonly stream: MediaStream;\n    readonly videoBitsPerSecond: number;\n    pause(): void;\n    requestData(): void;\n    resume(): void;\n    start(timeslice?: number): void;\n    stop(): void;\n    addEventListener<K extends keyof MediaRecorderEventMap>(type: K, listener: (this: MediaRecorder, ev: MediaRecorderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MediaRecorderEventMap>(type: K, listener: (this: MediaRecorder, ev: MediaRecorderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MediaRecorder: {\n    prototype: MediaRecorder;\n    new(stream: MediaStream, options?: MediaRecorderOptions): MediaRecorder;\n    isTypeSupported(type: string): boolean;\n};\n\ninterface MediaRecorderErrorEvent extends Event {\n    readonly error: DOMException;\n}\n\ndeclare var MediaRecorderErrorEvent: {\n    prototype: MediaRecorderErrorEvent;\n    new(type: string, eventInitDict: MediaRecorderErrorEventInit): MediaRecorderErrorEvent;\n};\n\ninterface MediaSession {\n    metadata: MediaMetadata | null;\n    playbackState: MediaSessionPlaybackState;\n    setActionHandler(action: MediaSessionAction, handler: MediaSessionActionHandler | null): void;\n    setPositionState(state?: MediaPositionState): void;\n}\n\ndeclare var MediaSession: {\n    prototype: MediaSession;\n    new(): MediaSession;\n};\n\ninterface MediaSourceEventMap {\n    "sourceclose": Event;\n    "sourceended": Event;\n    "sourceopen": Event;\n}\n\n/** This Media Source Extensions API interface represents a source of media data for an HTMLMediaElement object. A MediaSource object can be attached to a HTMLMediaElement to be played in the user agent. */\ninterface MediaSource extends EventTarget {\n    readonly activeSourceBuffers: SourceBufferList;\n    duration: number;\n    onsourceclose: ((this: MediaSource, ev: Event) => any) | null;\n    onsourceended: ((this: MediaSource, ev: Event) => any) | null;\n    onsourceopen: ((this: MediaSource, ev: Event) => any) | null;\n    readonly readyState: ReadyState;\n    readonly sourceBuffers: SourceBufferList;\n    addSourceBuffer(type: string): SourceBuffer;\n    clearLiveSeekableRange(): void;\n    endOfStream(error?: EndOfStreamError): void;\n    removeSourceBuffer(sourceBuffer: SourceBuffer): void;\n    setLiveSeekableRange(start: number, end: number): void;\n    addEventListener<K extends keyof MediaSourceEventMap>(type: K, listener: (this: MediaSource, ev: MediaSourceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MediaSourceEventMap>(type: K, listener: (this: MediaSource, ev: MediaSourceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MediaSource: {\n    prototype: MediaSource;\n    new(): MediaSource;\n    isTypeSupported(type: string): boolean;\n};\n\ninterface MediaStreamEventMap {\n    "addtrack": MediaStreamTrackEvent;\n    "removetrack": MediaStreamTrackEvent;\n}\n\n/** A stream of media content. A stream consists of several tracks such as video or audio tracks. Each track is specified as an instance of MediaStreamTrack. */\ninterface MediaStream extends EventTarget {\n    readonly active: boolean;\n    readonly id: string;\n    onaddtrack: ((this: MediaStream, ev: MediaStreamTrackEvent) => any) | null;\n    onremovetrack: ((this: MediaStream, ev: MediaStreamTrackEvent) => any) | null;\n    addTrack(track: MediaStreamTrack): void;\n    clone(): MediaStream;\n    getAudioTracks(): MediaStreamTrack[];\n    getTrackById(trackId: string): MediaStreamTrack | null;\n    getTracks(): MediaStreamTrack[];\n    getVideoTracks(): MediaStreamTrack[];\n    removeTrack(track: MediaStreamTrack): void;\n    addEventListener<K extends keyof MediaStreamEventMap>(type: K, listener: (this: MediaStream, ev: MediaStreamEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MediaStreamEventMap>(type: K, listener: (this: MediaStream, ev: MediaStreamEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MediaStream: {\n    prototype: MediaStream;\n    new(): MediaStream;\n    new(stream: MediaStream): MediaStream;\n    new(tracks: MediaStreamTrack[]): MediaStream;\n};\n\ninterface MediaStreamAudioDestinationNode extends AudioNode {\n    readonly stream: MediaStream;\n}\n\ndeclare var MediaStreamAudioDestinationNode: {\n    prototype: MediaStreamAudioDestinationNode;\n    new(context: AudioContext, options?: AudioNodeOptions): MediaStreamAudioDestinationNode;\n};\n\n/** A type of AudioNode which operates as an audio source whose media is received from a MediaStream obtained using the WebRTC or Media Capture and Streams APIs. */\ninterface MediaStreamAudioSourceNode extends AudioNode {\n    readonly mediaStream: MediaStream;\n}\n\ndeclare var MediaStreamAudioSourceNode: {\n    prototype: MediaStreamAudioSourceNode;\n    new(context: AudioContext, options: MediaStreamAudioSourceOptions): MediaStreamAudioSourceNode;\n};\n\ninterface MediaStreamTrackEventMap {\n    "ended": Event;\n    "mute": Event;\n    "unmute": Event;\n}\n\n/** A single media track within a stream; typically, these are audio or video tracks, but other track types may exist as well. */\ninterface MediaStreamTrack extends EventTarget {\n    contentHint: string;\n    enabled: boolean;\n    readonly id: string;\n    readonly kind: string;\n    readonly label: string;\n    readonly muted: boolean;\n    onended: ((this: MediaStreamTrack, ev: Event) => any) | null;\n    onmute: ((this: MediaStreamTrack, ev: Event) => any) | null;\n    onunmute: ((this: MediaStreamTrack, ev: Event) => any) | null;\n    readonly readyState: MediaStreamTrackState;\n    applyConstraints(constraints?: MediaTrackConstraints): Promise<void>;\n    clone(): MediaStreamTrack;\n    getCapabilities(): MediaTrackCapabilities;\n    getConstraints(): MediaTrackConstraints;\n    getSettings(): MediaTrackSettings;\n    stop(): void;\n    addEventListener<K extends keyof MediaStreamTrackEventMap>(type: K, listener: (this: MediaStreamTrack, ev: MediaStreamTrackEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MediaStreamTrackEventMap>(type: K, listener: (this: MediaStreamTrack, ev: MediaStreamTrackEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MediaStreamTrack: {\n    prototype: MediaStreamTrack;\n    new(): MediaStreamTrack;\n};\n\n/** Events which indicate that a MediaStream has had tracks added to or removed from the stream through calls to Media Stream API methods. These events are sent to the stream when these changes occur. */\ninterface MediaStreamTrackEvent extends Event {\n    readonly track: MediaStreamTrack;\n}\n\ndeclare var MediaStreamTrackEvent: {\n    prototype: MediaStreamTrackEvent;\n    new(type: string, eventInitDict: MediaStreamTrackEventInit): MediaStreamTrackEvent;\n};\n\n/** This Channel Messaging API interface allows us to create a new message channel and send data through it via its two MessagePort properties. */\ninterface MessageChannel {\n    /**\n     * Returns the first MessagePort object.\n     */\n    readonly port1: MessagePort;\n    /**\n     * Returns the second MessagePort object.\n     */\n    readonly port2: MessagePort;\n}\n\ndeclare var MessageChannel: {\n    prototype: MessageChannel;\n    new(): MessageChannel;\n};\n\n/** A message received by a target object. */\ninterface MessageEvent<T = any> extends Event {\n    /**\n     * Returns the data of the message.\n     */\n    readonly data: T;\n    /**\n     * Returns the last event ID string, for server-sent events.\n     */\n    readonly lastEventId: string;\n    /**\n     * Returns the origin of the message, for server-sent events and cross-document messaging.\n     */\n    readonly origin: string;\n    /**\n     * Returns the MessagePort array sent with the message, for cross-document messaging and channel messaging.\n     */\n    readonly ports: ReadonlyArray<MessagePort>;\n    /**\n     * Returns the WindowProxy of the source window, for cross-document messaging, and the MessagePort being attached, in the connect event fired at SharedWorkerGlobalScope objects.\n     */\n    readonly source: MessageEventSource | null;\n    /** @deprecated */\n    initMessageEvent(type: string, bubbles?: boolean, cancelable?: boolean, data?: any, origin?: string, lastEventId?: string, source?: MessageEventSource | null, ports?: MessagePort[]): void;\n}\n\ndeclare var MessageEvent: {\n    prototype: MessageEvent;\n    new<T>(type: string, eventInitDict?: MessageEventInit<T>): MessageEvent<T>;\n};\n\ninterface MessagePortEventMap {\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n}\n\n/** This Channel Messaging API interface represents one of the two ports of a MessageChannel, allowing messages to be sent from one port and listening out for them arriving at the other. */\ninterface MessagePort extends EventTarget {\n    onmessage: ((this: MessagePort, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: MessagePort, ev: MessageEvent) => any) | null;\n    /**\n     * Disconnects the port, so that it is no longer active.\n     */\n    close(): void;\n    /**\n     * Posts a message through the channel. Objects listed in transfer are transferred, not just cloned, meaning that they are no longer usable on the sending side.\n     *\n     * Throws a "DataCloneError" DOMException if transfer contains duplicate objects or port, or if message could not be cloned.\n     */\n    postMessage(message: any, transfer: Transferable[]): void;\n    postMessage(message: any, options?: PostMessageOptions): void;\n    /**\n     * Begins dispatching messages received on the port.\n     */\n    start(): void;\n    addEventListener<K extends keyof MessagePortEventMap>(type: K, listener: (this: MessagePort, ev: MessagePortEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MessagePortEventMap>(type: K, listener: (this: MessagePort, ev: MessagePortEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MessagePort: {\n    prototype: MessagePort;\n    new(): MessagePort;\n};\n\n/**\n * Provides contains information about a MIME type associated with a particular plugin. NavigatorPlugins.mimeTypes returns an array of this object.\n * @deprecated\n */\ninterface MimeType {\n    /**\n     * Returns the MIME type\'s description.\n     * @deprecated\n     */\n    readonly description: string;\n    /**\n     * Returns the Plugin object that implements this MIME type.\n     * @deprecated\n     */\n    readonly enabledPlugin: Plugin;\n    /**\n     * Returns the MIME type\'s typical file extensions, in a comma-separated list.\n     * @deprecated\n     */\n    readonly suffixes: string;\n    /**\n     * Returns the MIME type.\n     * @deprecated\n     */\n    readonly type: string;\n}\n\n/** @deprecated */\ndeclare var MimeType: {\n    prototype: MimeType;\n    new(): MimeType;\n};\n\n/**\n * Returns an array of MimeType instances, each of which contains information about a supported browser plugins. This object is returned by NavigatorPlugins.mimeTypes.\n * @deprecated\n */\ninterface MimeTypeArray {\n    /** @deprecated */\n    readonly length: number;\n    /** @deprecated */\n    item(index: number): MimeType | null;\n    /** @deprecated */\n    namedItem(name: string): MimeType | null;\n    [index: number]: MimeType;\n}\n\n/** @deprecated */\ndeclare var MimeTypeArray: {\n    prototype: MimeTypeArray;\n    new(): MimeTypeArray;\n};\n\n/** Events that occur due to the user interacting with a pointing device (such as a mouse). Common events using this interface include click, dblclick, mouseup, mousedown. */\ninterface MouseEvent extends UIEvent {\n    readonly altKey: boolean;\n    readonly button: number;\n    readonly buttons: number;\n    readonly clientX: number;\n    readonly clientY: number;\n    readonly ctrlKey: boolean;\n    readonly metaKey: boolean;\n    readonly movementX: number;\n    readonly movementY: number;\n    readonly offsetX: number;\n    readonly offsetY: number;\n    readonly pageX: number;\n    readonly pageY: number;\n    readonly relatedTarget: EventTarget | null;\n    readonly screenX: number;\n    readonly screenY: number;\n    readonly shiftKey: boolean;\n    readonly x: number;\n    readonly y: number;\n    getModifierState(keyArg: string): boolean;\n    /** @deprecated */\n    initMouseEvent(typeArg: string, canBubbleArg: boolean, cancelableArg: boolean, viewArg: Window, detailArg: number, screenXArg: number, screenYArg: number, clientXArg: number, clientYArg: number, ctrlKeyArg: boolean, altKeyArg: boolean, shiftKeyArg: boolean, metaKeyArg: boolean, buttonArg: number, relatedTargetArg: EventTarget | null): void;\n}\n\ndeclare var MouseEvent: {\n    prototype: MouseEvent;\n    new(type: string, eventInitDict?: MouseEventInit): MouseEvent;\n};\n\n/**\n * Provides event properties that are specific to modifications to the Document Object Model (DOM) hierarchy and nodes.\n * @deprecated DOM4 [DOM] provides a new mechanism using a MutationObserver interface which addresses the use cases that mutation events solve, but in a more performant manner. Thus, this specification describes mutation events for reference and completeness of legacy behavior, but deprecates the use of the MutationEvent interface.\n */\ninterface MutationEvent extends Event {\n    /** @deprecated */\n    readonly attrChange: number;\n    /** @deprecated */\n    readonly attrName: string;\n    /** @deprecated */\n    readonly newValue: string;\n    /** @deprecated */\n    readonly prevValue: string;\n    /** @deprecated */\n    readonly relatedNode: Node | null;\n    /** @deprecated */\n    initMutationEvent(typeArg: string, bubblesArg?: boolean, cancelableArg?: boolean, relatedNodeArg?: Node | null, prevValueArg?: string, newValueArg?: string, attrNameArg?: string, attrChangeArg?: number): void;\n    readonly ADDITION: number;\n    readonly MODIFICATION: number;\n    readonly REMOVAL: number;\n}\n\n/** @deprecated */\ndeclare var MutationEvent: {\n    prototype: MutationEvent;\n    new(): MutationEvent;\n    readonly ADDITION: number;\n    readonly MODIFICATION: number;\n    readonly REMOVAL: number;\n};\n\n/** Provides the ability to watch for changes being made to the DOM tree. It is designed as a replacement for the older Mutation Events feature which was part of the DOM3 Events specification. */\ninterface MutationObserver {\n    /**\n     * Stops observer from observing any mutations. Until the observe() method is used again, observer\'s callback will not be invoked.\n     */\n    disconnect(): void;\n    /**\n     * Instructs the user agent to observe a given target (a node) and report any mutations based on the criteria given by options (an object).\n     *\n     * The options argument allows for setting mutation observation options via object members.\n     */\n    observe(target: Node, options?: MutationObserverInit): void;\n    /**\n     * Empties the record queue and returns what was in there.\n     */\n    takeRecords(): MutationRecord[];\n}\n\ndeclare var MutationObserver: {\n    prototype: MutationObserver;\n    new(callback: MutationCallback): MutationObserver;\n};\n\n/** A MutationRecord represents an individual DOM mutation. It is the object that is passed to MutationObserver\'s callback. */\ninterface MutationRecord {\n    /**\n     * Return the nodes added and removed respectively.\n     */\n    readonly addedNodes: NodeList;\n    /**\n     * Returns the local name of the changed attribute, and null otherwise.\n     */\n    readonly attributeName: string | null;\n    /**\n     * Returns the namespace of the changed attribute, and null otherwise.\n     */\n    readonly attributeNamespace: string | null;\n    /**\n     * Return the previous and next sibling respectively of the added or removed nodes, and null otherwise.\n     */\n    readonly nextSibling: Node | null;\n    /**\n     * The return value depends on type. For "attributes", it is the value of the changed attribute before the change. For "characterData", it is the data of the changed node before the change. For "childList", it is null.\n     */\n    readonly oldValue: string | null;\n    /**\n     * Return the previous and next sibling respectively of the added or removed nodes, and null otherwise.\n     */\n    readonly previousSibling: Node | null;\n    /**\n     * Return the nodes added and removed respectively.\n     */\n    readonly removedNodes: NodeList;\n    /**\n     * Returns the node the mutation affected, depending on the type. For "attributes", it is the element whose attribute changed. For "characterData", it is the CharacterData node. For "childList", it is the node whose children changed.\n     */\n    readonly target: Node;\n    /**\n     * Returns "attributes" if it was an attribute mutation. "characterData" if it was a mutation to a CharacterData node. And "childList" if it was a mutation to the tree of nodes.\n     */\n    readonly type: MutationRecordType;\n}\n\ndeclare var MutationRecord: {\n    prototype: MutationRecord;\n    new(): MutationRecord;\n};\n\n/** A collection of Attr objects. Objects inside a NamedNodeMap are not in any particular order, unlike NodeList, although they may be accessed by an index as in an array. */\ninterface NamedNodeMap {\n    readonly length: number;\n    getNamedItem(qualifiedName: string): Attr | null;\n    getNamedItemNS(namespace: string | null, localName: string): Attr | null;\n    item(index: number): Attr | null;\n    removeNamedItem(qualifiedName: string): Attr;\n    removeNamedItemNS(namespace: string | null, localName: string): Attr;\n    setNamedItem(attr: Attr): Attr | null;\n    setNamedItemNS(attr: Attr): Attr | null;\n    [index: number]: Attr;\n}\n\ndeclare var NamedNodeMap: {\n    prototype: NamedNodeMap;\n    new(): NamedNodeMap;\n};\n\n/** The state and the identity of the user agent. It allows scripts to query it and to register themselves to carry on some activities. */\ninterface Navigator extends NavigatorAutomationInformation, NavigatorConcurrentHardware, NavigatorContentUtils, NavigatorCookies, NavigatorID, NavigatorLanguage, NavigatorNetworkInformation, NavigatorOnLine, NavigatorPlugins, NavigatorStorage {\n    readonly clipboard: Clipboard;\n    readonly credentials: CredentialsContainer;\n    readonly doNotTrack: string | null;\n    readonly geolocation: Geolocation;\n    readonly maxTouchPoints: number;\n    readonly mediaCapabilities: MediaCapabilities;\n    readonly mediaDevices: MediaDevices;\n    readonly mediaSession: MediaSession;\n    readonly permissions: Permissions;\n    readonly pointerEnabled: boolean;\n    readonly serviceWorker: ServiceWorkerContainer;\n    getGamepads(): (Gamepad | null)[];\n    requestMediaKeySystemAccess(keySystem: string, supportedConfigurations: MediaKeySystemConfiguration[]): Promise<MediaKeySystemAccess>;\n    sendBeacon(url: string | URL, data?: BodyInit | null): boolean;\n    share(data?: ShareData): Promise<void>;\n    vibrate(pattern: VibratePattern): boolean;\n}\n\ndeclare var Navigator: {\n    prototype: Navigator;\n    new(): Navigator;\n};\n\ninterface NavigatorAutomationInformation {\n    readonly webdriver: boolean;\n}\n\ninterface NavigatorConcurrentHardware {\n    readonly hardwareConcurrency: number;\n}\n\ninterface NavigatorContentUtils {\n    registerProtocolHandler(scheme: string, url: string | URL): void;\n}\n\ninterface NavigatorCookies {\n    readonly cookieEnabled: boolean;\n}\n\ninterface NavigatorID {\n    /** @deprecated */\n    readonly appCodeName: string;\n    /** @deprecated */\n    readonly appName: string;\n    /** @deprecated */\n    readonly appVersion: string;\n    /** @deprecated */\n    readonly platform: string;\n    /** @deprecated */\n    readonly product: string;\n    /** @deprecated */\n    readonly productSub: string;\n    readonly userAgent: string;\n    readonly vendor: string;\n    /** @deprecated */\n    readonly vendorSub: string;\n}\n\ninterface NavigatorLanguage {\n    readonly language: string;\n    readonly languages: ReadonlyArray<string>;\n}\n\ninterface NavigatorNetworkInformation {\n    readonly connection: NetworkInformation;\n}\n\ninterface NavigatorOnLine {\n    readonly onLine: boolean;\n}\n\ninterface NavigatorPlugins {\n    /** @deprecated */\n    readonly mimeTypes: MimeTypeArray;\n    /** @deprecated */\n    readonly plugins: PluginArray;\n    /** @deprecated */\n    javaEnabled(): boolean;\n}\n\ninterface NavigatorStorage {\n    readonly storage: StorageManager;\n}\n\ninterface NetworkInformation extends EventTarget {\n    readonly type: ConnectionType;\n}\n\ndeclare var NetworkInformation: {\n    prototype: NetworkInformation;\n    new(): NetworkInformation;\n};\n\n/** Node is an interface from which a number of DOM API object types inherit. It allows those types to be treated similarly; for example, inheriting the same set of methods, or being tested in the same way. */\ninterface Node extends EventTarget {\n    /**\n     * Returns node\'s node document\'s document base URL.\n     */\n    readonly baseURI: string;\n    /**\n     * Returns the children.\n     */\n    readonly childNodes: NodeListOf<ChildNode>;\n    /**\n     * Returns the first child.\n     */\n    readonly firstChild: ChildNode | null;\n    /**\n     * Returns true if node is connected and false otherwise.\n     */\n    readonly isConnected: boolean;\n    /**\n     * Returns the last child.\n     */\n    readonly lastChild: ChildNode | null;\n    /**\n     * Returns the next sibling.\n     */\n    readonly nextSibling: ChildNode | null;\n    /**\n     * Returns a string appropriate for the type of node.\n     */\n    readonly nodeName: string;\n    /**\n     * Returns the type of node.\n     */\n    readonly nodeType: number;\n    nodeValue: string | null;\n    /**\n     * Returns the node document. Returns null for documents.\n     */\n    readonly ownerDocument: Document | null;\n    /**\n     * Returns the parent element.\n     */\n    readonly parentElement: HTMLElement | null;\n    /**\n     * Returns the parent.\n     */\n    readonly parentNode: ParentNode | null;\n    /**\n     * Returns the previous sibling.\n     */\n    readonly previousSibling: ChildNode | null;\n    textContent: string | null;\n    appendChild<T extends Node>(node: T): T;\n    /**\n     * Returns a copy of node. If deep is true, the copy also includes the node\'s descendants.\n     */\n    cloneNode(deep?: boolean): Node;\n    /**\n     * Returns a bitmask indicating the position of other relative to node.\n     */\n    compareDocumentPosition(other: Node): number;\n    /**\n     * Returns true if other is an inclusive descendant of node, and false otherwise.\n     */\n    contains(other: Node | null): boolean;\n    /**\n     * Returns node\'s root.\n     */\n    getRootNode(options?: GetRootNodeOptions): Node;\n    /**\n     * Returns whether node has children.\n     */\n    hasChildNodes(): boolean;\n    insertBefore<T extends Node>(node: T, child: Node | null): T;\n    isDefaultNamespace(namespace: string | null): boolean;\n    /**\n     * Returns whether node and otherNode have the same properties.\n     */\n    isEqualNode(otherNode: Node | null): boolean;\n    isSameNode(otherNode: Node | null): boolean;\n    lookupNamespaceURI(prefix: string | null): string | null;\n    lookupPrefix(namespace: string | null): string | null;\n    /**\n     * Removes empty exclusive Text nodes and concatenates the data of remaining contiguous exclusive Text nodes into the first of their nodes.\n     */\n    normalize(): void;\n    removeChild<T extends Node>(child: T): T;\n    replaceChild<T extends Node>(node: Node, child: T): T;\n    readonly ATTRIBUTE_NODE: number;\n    /**\n     * node is a CDATASection node.\n     */\n    readonly CDATA_SECTION_NODE: number;\n    /**\n     * node is a Comment node.\n     */\n    readonly COMMENT_NODE: number;\n    /**\n     * node is a DocumentFragment node.\n     */\n    readonly DOCUMENT_FRAGMENT_NODE: number;\n    /**\n     * node is a document.\n     */\n    readonly DOCUMENT_NODE: number;\n    /**\n     * Set when other is a descendant of node.\n     */\n    readonly DOCUMENT_POSITION_CONTAINED_BY: number;\n    /**\n     * Set when other is an ancestor of node.\n     */\n    readonly DOCUMENT_POSITION_CONTAINS: number;\n    /**\n     * Set when node and other are not in the same tree.\n     */\n    readonly DOCUMENT_POSITION_DISCONNECTED: number;\n    /**\n     * Set when other is following node.\n     */\n    readonly DOCUMENT_POSITION_FOLLOWING: number;\n    readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;\n    /**\n     * Set when other is preceding node.\n     */\n    readonly DOCUMENT_POSITION_PRECEDING: number;\n    /**\n     * node is a doctype.\n     */\n    readonly DOCUMENT_TYPE_NODE: number;\n    /**\n     * node is an element.\n     */\n    readonly ELEMENT_NODE: number;\n    readonly ENTITY_NODE: number;\n    readonly ENTITY_REFERENCE_NODE: number;\n    readonly NOTATION_NODE: number;\n    /**\n     * node is a ProcessingInstruction node.\n     */\n    readonly PROCESSING_INSTRUCTION_NODE: number;\n    /**\n     * node is a Text node.\n     */\n    readonly TEXT_NODE: number;\n}\n\ndeclare var Node: {\n    prototype: Node;\n    new(): Node;\n    readonly ATTRIBUTE_NODE: number;\n    /**\n     * node is a CDATASection node.\n     */\n    readonly CDATA_SECTION_NODE: number;\n    /**\n     * node is a Comment node.\n     */\n    readonly COMMENT_NODE: number;\n    /**\n     * node is a DocumentFragment node.\n     */\n    readonly DOCUMENT_FRAGMENT_NODE: number;\n    /**\n     * node is a document.\n     */\n    readonly DOCUMENT_NODE: number;\n    /**\n     * Set when other is a descendant of node.\n     */\n    readonly DOCUMENT_POSITION_CONTAINED_BY: number;\n    /**\n     * Set when other is an ancestor of node.\n     */\n    readonly DOCUMENT_POSITION_CONTAINS: number;\n    /**\n     * Set when node and other are not in the same tree.\n     */\n    readonly DOCUMENT_POSITION_DISCONNECTED: number;\n    /**\n     * Set when other is following node.\n     */\n    readonly DOCUMENT_POSITION_FOLLOWING: number;\n    readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;\n    /**\n     * Set when other is preceding node.\n     */\n    readonly DOCUMENT_POSITION_PRECEDING: number;\n    /**\n     * node is a doctype.\n     */\n    readonly DOCUMENT_TYPE_NODE: number;\n    /**\n     * node is an element.\n     */\n    readonly ELEMENT_NODE: number;\n    readonly ENTITY_NODE: number;\n    readonly ENTITY_REFERENCE_NODE: number;\n    readonly NOTATION_NODE: number;\n    /**\n     * node is a ProcessingInstruction node.\n     */\n    readonly PROCESSING_INSTRUCTION_NODE: number;\n    /**\n     * node is a Text node.\n     */\n    readonly TEXT_NODE: number;\n};\n\n/** An iterator over the members of a list of the nodes in a subtree of the DOM. The nodes will be returned in document order. */\ninterface NodeIterator {\n    readonly filter: NodeFilter | null;\n    readonly pointerBeforeReferenceNode: boolean;\n    readonly referenceNode: Node;\n    readonly root: Node;\n    readonly whatToShow: number;\n    /** @deprecated */\n    detach(): void;\n    nextNode(): Node | null;\n    previousNode(): Node | null;\n}\n\ndeclare var NodeIterator: {\n    prototype: NodeIterator;\n    new(): NodeIterator;\n};\n\n/** NodeList objects are collections of nodes, usually returned by properties such as Node.childNodes and methods such as document.querySelectorAll(). */\ninterface NodeList {\n    /**\n     * Returns the number of nodes in the collection.\n     */\n    readonly length: number;\n    /**\n     * Returns the node with index index from the collection. The nodes are sorted in tree order.\n     */\n    item(index: number): Node | null;\n    /**\n     * Performs the specified action for each node in an list.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the list.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: Node, key: number, parent: NodeList) => void, thisArg?: any): void;\n    [index: number]: Node;\n}\n\ndeclare var NodeList: {\n    prototype: NodeList;\n    new(): NodeList;\n};\n\ninterface NodeListOf<TNode extends Node> extends NodeList {\n    item(index: number): TNode;\n    /**\n     * Performs the specified action for each node in an list.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the list.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: TNode, key: number, parent: NodeListOf<TNode>) => void, thisArg?: any): void;\n    [index: number]: TNode;\n}\n\ninterface NonDocumentTypeChildNode {\n    /**\n     * Returns the first following sibling that is an element, and null otherwise.\n     */\n    readonly nextElementSibling: Element | null;\n    /**\n     * Returns the first preceding sibling that is an element, and null otherwise.\n     */\n    readonly previousElementSibling: Element | null;\n}\n\ninterface NonElementParentNode {\n    /**\n     * Returns the first element within node\'s descendants whose ID is elementId.\n     */\n    getElementById(elementId: string): Element | null;\n}\n\ninterface NotificationEventMap {\n    "click": Event;\n    "close": Event;\n    "error": Event;\n    "show": Event;\n}\n\n/** This Notifications API interface is used to configure and display desktop notifications to the user. */\ninterface Notification extends EventTarget {\n    readonly body: string;\n    readonly data: any;\n    readonly dir: NotificationDirection;\n    readonly icon: string;\n    readonly lang: string;\n    onclick: ((this: Notification, ev: Event) => any) | null;\n    onclose: ((this: Notification, ev: Event) => any) | null;\n    onerror: ((this: Notification, ev: Event) => any) | null;\n    onshow: ((this: Notification, ev: Event) => any) | null;\n    readonly tag: string;\n    readonly title: string;\n    close(): void;\n    addEventListener<K extends keyof NotificationEventMap>(type: K, listener: (this: Notification, ev: NotificationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof NotificationEventMap>(type: K, listener: (this: Notification, ev: NotificationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var Notification: {\n    prototype: Notification;\n    new(title: string, options?: NotificationOptions): Notification;\n    readonly permission: NotificationPermission;\n    requestPermission(deprecatedCallback?: NotificationPermissionCallback): Promise<NotificationPermission>;\n};\n\n/** The OES_element_index_uint extension is part of the WebGL API and adds support for gl.UNSIGNED_INT types to WebGLRenderingContext.drawElements(). */\ninterface OES_element_index_uint {\n}\n\ninterface OES_fbo_render_mipmap {\n}\n\n/** The OES_standard_derivatives extension is part of the WebGL API and adds the GLSL derivative functions dFdx, dFdy, and fwidth. */\ninterface OES_standard_derivatives {\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT_OES: GLenum;\n}\n\n/** The OES_texture_float extension is part of the WebGL API and exposes floating-point pixel types for textures. */\ninterface OES_texture_float {\n}\n\n/** The OES_texture_float_linear extension is part of the WebGL API and allows linear filtering with floating-point pixel types for textures. */\ninterface OES_texture_float_linear {\n}\n\n/** The OES_texture_half_float extension is part of the WebGL API and adds texture formats with 16- (aka half float) and 32-bit floating-point components. */\ninterface OES_texture_half_float {\n    readonly HALF_FLOAT_OES: GLenum;\n}\n\n/** The OES_texture_half_float_linear extension is part of the WebGL API and allows linear filtering with half floating-point pixel types for textures. */\ninterface OES_texture_half_float_linear {\n}\n\ninterface OES_vertex_array_object {\n    bindVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES | null): void;\n    createVertexArrayOES(): WebGLVertexArrayObjectOES | null;\n    deleteVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES | null): void;\n    isVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES | null): GLboolean;\n    readonly VERTEX_ARRAY_BINDING_OES: GLenum;\n}\n\ninterface OVR_multiview2 {\n    framebufferTextureMultiviewOVR(target: GLenum, attachment: GLenum, texture: WebGLTexture | null, level: GLint, baseViewIndex: GLint, numViews: GLsizei): void;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_BASE_VIEW_INDEX_OVR: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_NUM_VIEWS_OVR: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_VIEW_TARGETS_OVR: GLenum;\n    readonly MAX_VIEWS_OVR: GLenum;\n}\n\n/** The Web Audio API OfflineAudioCompletionEvent interface represents events that occur when the processing of an OfflineAudioContext is terminated. The complete event implements this interface. */\ninterface OfflineAudioCompletionEvent extends Event {\n    readonly renderedBuffer: AudioBuffer;\n}\n\ndeclare var OfflineAudioCompletionEvent: {\n    prototype: OfflineAudioCompletionEvent;\n    new(type: string, eventInitDict: OfflineAudioCompletionEventInit): OfflineAudioCompletionEvent;\n};\n\ninterface OfflineAudioContextEventMap extends BaseAudioContextEventMap {\n    "complete": OfflineAudioCompletionEvent;\n}\n\n/** An AudioContext interface representing an audio-processing graph built from linked together AudioNodes. In contrast with a standard AudioContext, an OfflineAudioContext doesn\'t render the audio to the device hardware; instead, it generates it, as fast as it can, and outputs the result to an AudioBuffer. */\ninterface OfflineAudioContext extends BaseAudioContext {\n    readonly length: number;\n    oncomplete: ((this: OfflineAudioContext, ev: OfflineAudioCompletionEvent) => any) | null;\n    resume(): Promise<void>;\n    startRendering(): Promise<AudioBuffer>;\n    suspend(suspendTime: number): Promise<void>;\n    addEventListener<K extends keyof OfflineAudioContextEventMap>(type: K, listener: (this: OfflineAudioContext, ev: OfflineAudioContextEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof OfflineAudioContextEventMap>(type: K, listener: (this: OfflineAudioContext, ev: OfflineAudioContextEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var OfflineAudioContext: {\n    prototype: OfflineAudioContext;\n    new(contextOptions: OfflineAudioContextOptions): OfflineAudioContext;\n    new(numberOfChannels: number, length: number, sampleRate: number): OfflineAudioContext;\n};\n\n/** The OscillatorNode interface represents a periodic waveform, such as a sine wave. It is an AudioScheduledSourceNode audio-processing module that causes a specified frequency of a given wave to be created—in effect, a constant tone. */\ninterface OscillatorNode extends AudioScheduledSourceNode {\n    readonly detune: AudioParam;\n    readonly frequency: AudioParam;\n    type: OscillatorType;\n    setPeriodicWave(periodicWave: PeriodicWave): void;\n    addEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: OscillatorNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AudioScheduledSourceNodeEventMap>(type: K, listener: (this: OscillatorNode, ev: AudioScheduledSourceNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var OscillatorNode: {\n    prototype: OscillatorNode;\n    new(context: BaseAudioContext, options?: OscillatorOptions): OscillatorNode;\n};\n\ninterface OverconstrainedError extends Error {\n    readonly constraint: string;\n}\n\ndeclare var OverconstrainedError: {\n    prototype: OverconstrainedError;\n    new(constraint: string, message?: string): OverconstrainedError;\n};\n\n/** The PageTransitionEvent is fired when a document is being loaded or unloaded. */\ninterface PageTransitionEvent extends Event {\n    /**\n     * For the pageshow event, returns false if the page is newly being loaded (and the load event will fire). Otherwise, returns true.\n     *\n     * For the pagehide event, returns false if the page is going away for the last time. Otherwise, returns true, meaning that (if nothing conspires to make the page unsalvageable) the page might be reused if the user navigates back to this page.\n     *\n     * Things that can cause the page to be unsalvageable include:\n     *\n     * The user agent decided to not keep the Document alive in a session history entry after unload\n     * Having iframes that are not salvageable\n     * Active WebSocket objects\n     * Aborting a Document\n     */\n    readonly persisted: boolean;\n}\n\ndeclare var PageTransitionEvent: {\n    prototype: PageTransitionEvent;\n    new(type: string, eventInitDict?: PageTransitionEventInit): PageTransitionEvent;\n};\n\n/** A PannerNode always has exactly one input and one output: the input can be mono or stereo but the output is always stereo (2 channels); you can\'t have panning effects without at least two audio channels! */\ninterface PannerNode extends AudioNode {\n    coneInnerAngle: number;\n    coneOuterAngle: number;\n    coneOuterGain: number;\n    distanceModel: DistanceModelType;\n    maxDistance: number;\n    readonly orientationX: AudioParam;\n    readonly orientationY: AudioParam;\n    readonly orientationZ: AudioParam;\n    panningModel: PanningModelType;\n    readonly positionX: AudioParam;\n    readonly positionY: AudioParam;\n    readonly positionZ: AudioParam;\n    refDistance: number;\n    rolloffFactor: number;\n    /** @deprecated */\n    setOrientation(x: number, y: number, z: number): void;\n    /** @deprecated */\n    setPosition(x: number, y: number, z: number): void;\n}\n\ndeclare var PannerNode: {\n    prototype: PannerNode;\n    new(context: BaseAudioContext, options?: PannerOptions): PannerNode;\n};\n\ninterface ParentNode extends Node {\n    readonly childElementCount: number;\n    /**\n     * Returns the child elements.\n     */\n    readonly children: HTMLCollection;\n    /**\n     * Returns the first child that is an element, and null otherwise.\n     */\n    readonly firstElementChild: Element | null;\n    /**\n     * Returns the last child that is an element, and null otherwise.\n     */\n    readonly lastElementChild: Element | null;\n    /**\n     * Inserts nodes after the last child of node, while replacing strings in nodes with equivalent Text nodes.\n     *\n     * Throws a "HierarchyRequestError" DOMException if the constraints of the node tree are violated.\n     */\n    append(...nodes: (Node | string)[]): void;\n    /**\n     * Inserts nodes before the first child of node, while replacing strings in nodes with equivalent Text nodes.\n     *\n     * Throws a "HierarchyRequestError" DOMException if the constraints of the node tree are violated.\n     */\n    prepend(...nodes: (Node | string)[]): void;\n    /**\n     * Returns the first element that is a descendant of node that matches selectors.\n     */\n    querySelector<K extends keyof HTMLElementTagNameMap>(selectors: K): HTMLElementTagNameMap[K] | null;\n    querySelector<K extends keyof SVGElementTagNameMap>(selectors: K): SVGElementTagNameMap[K] | null;\n    querySelector<E extends Element = Element>(selectors: string): E | null;\n    /**\n     * Returns all element descendants of node that match selectors.\n     */\n    querySelectorAll<K extends keyof HTMLElementTagNameMap>(selectors: K): NodeListOf<HTMLElementTagNameMap[K]>;\n    querySelectorAll<K extends keyof SVGElementTagNameMap>(selectors: K): NodeListOf<SVGElementTagNameMap[K]>;\n    querySelectorAll<E extends Element = Element>(selectors: string): NodeListOf<E>;\n    /**\n     * Replace all children of node with nodes, while replacing strings in nodes with equivalent Text nodes.\n     *\n     * Throws a "HierarchyRequestError" DOMException if the constraints of the node tree are violated.\n     */\n    replaceChildren(...nodes: (Node | string)[]): void;\n}\n\n/** This Canvas 2D API interface is used to declare a path that can then be used on a CanvasRenderingContext2D object. The path methods of the CanvasRenderingContext2D interface are also present on this interface, which gives you the convenience of being able to retain and replay your path whenever desired. */\ninterface Path2D extends CanvasPath {\n    /**\n     * Adds to the path the path given by the argument.\n     */\n    addPath(path: Path2D, transform?: DOMMatrix2DInit): void;\n}\n\ndeclare var Path2D: {\n    prototype: Path2D;\n    new(path?: Path2D | string): Path2D;\n};\n\n/**\n * This Payment Request API interface is used to store shipping or payment address information.\n * @deprecated\n */\ninterface PaymentAddress {\n    /** @deprecated */\n    readonly addressLine: ReadonlyArray<string>;\n    /** @deprecated */\n    readonly city: string;\n    /** @deprecated */\n    readonly country: string;\n    /** @deprecated */\n    readonly dependentLocality: string;\n    /** @deprecated */\n    readonly organization: string;\n    /** @deprecated */\n    readonly phone: string;\n    /** @deprecated */\n    readonly postalCode: string;\n    /** @deprecated */\n    readonly recipient: string;\n    /** @deprecated */\n    readonly region: string;\n    /** @deprecated */\n    readonly sortingCode: string;\n    /** @deprecated */\n    toJSON(): any;\n}\n\n/** @deprecated */\ndeclare var PaymentAddress: {\n    prototype: PaymentAddress;\n    new(): PaymentAddress;\n};\n\ninterface PaymentMethodChangeEvent extends PaymentRequestUpdateEvent {\n    readonly methodDetails: any;\n    readonly methodName: string;\n}\n\ndeclare var PaymentMethodChangeEvent: {\n    prototype: PaymentMethodChangeEvent;\n    new(type: string, eventInitDict?: PaymentMethodChangeEventInit): PaymentMethodChangeEvent;\n};\n\ninterface PaymentRequestEventMap {\n    "paymentmethodchange": Event;\n}\n\n/** This Payment Request API interface is the primary access point into the API, and lets web content and apps accept payments from the end user. */\ninterface PaymentRequest extends EventTarget {\n    readonly id: string;\n    onpaymentmethodchange: ((this: PaymentRequest, ev: Event) => any) | null;\n    abort(): Promise<void>;\n    canMakePayment(): Promise<boolean>;\n    show(detailsPromise?: PaymentDetailsUpdate | PromiseLike<PaymentDetailsUpdate>): Promise<PaymentResponse>;\n    addEventListener<K extends keyof PaymentRequestEventMap>(type: K, listener: (this: PaymentRequest, ev: PaymentRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof PaymentRequestEventMap>(type: K, listener: (this: PaymentRequest, ev: PaymentRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var PaymentRequest: {\n    prototype: PaymentRequest;\n    new(methodData: PaymentMethodData[], details: PaymentDetailsInit): PaymentRequest;\n};\n\n/** This Payment Request API interface enables a web page to update the details of a PaymentRequest in response to a user action. */\ninterface PaymentRequestUpdateEvent extends Event {\n    updateWith(detailsPromise: PaymentDetailsUpdate | PromiseLike<PaymentDetailsUpdate>): void;\n}\n\ndeclare var PaymentRequestUpdateEvent: {\n    prototype: PaymentRequestUpdateEvent;\n    new(type: string, eventInitDict?: PaymentRequestUpdateEventInit): PaymentRequestUpdateEvent;\n};\n\n/** This Payment Request API interface is returned after a user selects a payment method and approves a payment request. */\ninterface PaymentResponse extends EventTarget {\n    readonly details: any;\n    readonly methodName: string;\n    readonly requestId: string;\n    complete(result?: PaymentComplete): Promise<void>;\n    retry(errorFields?: PaymentValidationErrors): Promise<void>;\n    toJSON(): any;\n}\n\ndeclare var PaymentResponse: {\n    prototype: PaymentResponse;\n    new(): PaymentResponse;\n};\n\ninterface PerformanceEventMap {\n    "resourcetimingbufferfull": Event;\n}\n\n/** Provides access to performance-related information for the current page. It\'s part of the High Resolution Time API, but is enhanced by the Performance Timeline API, the Navigation Timing API, the User Timing API, and the Resource Timing API. */\ninterface Performance extends EventTarget {\n    /** @deprecated */\n    readonly navigation: PerformanceNavigation;\n    onresourcetimingbufferfull: ((this: Performance, ev: Event) => any) | null;\n    readonly timeOrigin: DOMHighResTimeStamp;\n    /** @deprecated */\n    readonly timing: PerformanceTiming;\n    clearMarks(markName?: string): void;\n    clearMeasures(measureName?: string): void;\n    clearResourceTimings(): void;\n    getEntries(): PerformanceEntryList;\n    getEntriesByName(name: string, type?: string): PerformanceEntryList;\n    getEntriesByType(type: string): PerformanceEntryList;\n    mark(markName: string, markOptions?: PerformanceMarkOptions): PerformanceMark;\n    measure(measureName: string, startOrMeasureOptions?: string | PerformanceMeasureOptions, endMark?: string): PerformanceMeasure;\n    now(): DOMHighResTimeStamp;\n    setResourceTimingBufferSize(maxSize: number): void;\n    toJSON(): any;\n    addEventListener<K extends keyof PerformanceEventMap>(type: K, listener: (this: Performance, ev: PerformanceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof PerformanceEventMap>(type: K, listener: (this: Performance, ev: PerformanceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var Performance: {\n    prototype: Performance;\n    new(): Performance;\n};\n\n/** Encapsulates a single performance metric that is part of the performance timeline. A performance entry can be directly created by making a performance mark or measure (for example by calling the mark() method) at an explicit point in an application. Performance entries are also created in indirect ways such as loading a resource (such as an image). */\ninterface PerformanceEntry {\n    readonly duration: DOMHighResTimeStamp;\n    readonly entryType: string;\n    readonly name: string;\n    readonly startTime: DOMHighResTimeStamp;\n    toJSON(): any;\n}\n\ndeclare var PerformanceEntry: {\n    prototype: PerformanceEntry;\n    new(): PerformanceEntry;\n};\n\ninterface PerformanceEventTiming extends PerformanceEntry {\n    readonly cancelable: boolean;\n    readonly processingEnd: DOMHighResTimeStamp;\n    readonly processingStart: DOMHighResTimeStamp;\n    readonly target: Node | null;\n}\n\ndeclare var PerformanceEventTiming: {\n    prototype: PerformanceEventTiming;\n    new(): PerformanceEventTiming;\n};\n\n/** PerformanceMark is an abstract interface for PerformanceEntry objects with an entryType of "mark". Entries of this type are created by calling performance.mark() to add a named DOMHighResTimeStamp (the mark) to the browser\'s performance timeline. */\ninterface PerformanceMark extends PerformanceEntry {\n    readonly detail: any;\n}\n\ndeclare var PerformanceMark: {\n    prototype: PerformanceMark;\n    new(markName: string, markOptions?: PerformanceMarkOptions): PerformanceMark;\n};\n\n/** PerformanceMeasure is an abstract interface for PerformanceEntry objects with an entryType of "measure". Entries of this type are created by calling performance.measure() to add a named DOMHighResTimeStamp (the measure) between two marks to the browser\'s performance timeline. */\ninterface PerformanceMeasure extends PerformanceEntry {\n    readonly detail: any;\n}\n\ndeclare var PerformanceMeasure: {\n    prototype: PerformanceMeasure;\n    new(): PerformanceMeasure;\n};\n\n/**\n * The legacy PerformanceNavigation interface represents information about how the navigation to the current document was done.\n * @deprecated This interface is deprecated in the Navigation Timing Level 2 specification. Please use the PerformanceNavigationTiming interface instead.\n */\ninterface PerformanceNavigation {\n    /** @deprecated */\n    readonly redirectCount: number;\n    /** @deprecated */\n    readonly type: number;\n    /** @deprecated */\n    toJSON(): any;\n    readonly TYPE_BACK_FORWARD: number;\n    readonly TYPE_NAVIGATE: number;\n    readonly TYPE_RELOAD: number;\n    readonly TYPE_RESERVED: number;\n}\n\n/** @deprecated */\ndeclare var PerformanceNavigation: {\n    prototype: PerformanceNavigation;\n    new(): PerformanceNavigation;\n    readonly TYPE_BACK_FORWARD: number;\n    readonly TYPE_NAVIGATE: number;\n    readonly TYPE_RELOAD: number;\n    readonly TYPE_RESERVED: number;\n};\n\n/** Provides methods and properties to store and retrieve metrics regarding the browser\'s document navigation events. For example, this interface can be used to determine how much time it takes to load or unload a document. */\ninterface PerformanceNavigationTiming extends PerformanceResourceTiming {\n    readonly domComplete: DOMHighResTimeStamp;\n    readonly domContentLoadedEventEnd: DOMHighResTimeStamp;\n    readonly domContentLoadedEventStart: DOMHighResTimeStamp;\n    readonly domInteractive: DOMHighResTimeStamp;\n    readonly loadEventEnd: DOMHighResTimeStamp;\n    readonly loadEventStart: DOMHighResTimeStamp;\n    readonly redirectCount: number;\n    readonly type: NavigationType;\n    readonly unloadEventEnd: DOMHighResTimeStamp;\n    readonly unloadEventStart: DOMHighResTimeStamp;\n    toJSON(): any;\n}\n\ndeclare var PerformanceNavigationTiming: {\n    prototype: PerformanceNavigationTiming;\n    new(): PerformanceNavigationTiming;\n};\n\ninterface PerformanceObserver {\n    disconnect(): void;\n    observe(options?: PerformanceObserverInit): void;\n    takeRecords(): PerformanceEntryList;\n}\n\ndeclare var PerformanceObserver: {\n    prototype: PerformanceObserver;\n    new(callback: PerformanceObserverCallback): PerformanceObserver;\n    readonly supportedEntryTypes: ReadonlyArray<string>;\n};\n\ninterface PerformanceObserverEntryList {\n    getEntries(): PerformanceEntryList;\n    getEntriesByName(name: string, type?: string): PerformanceEntryList;\n    getEntriesByType(type: string): PerformanceEntryList;\n}\n\ndeclare var PerformanceObserverEntryList: {\n    prototype: PerformanceObserverEntryList;\n    new(): PerformanceObserverEntryList;\n};\n\ninterface PerformancePaintTiming extends PerformanceEntry {\n}\n\ndeclare var PerformancePaintTiming: {\n    prototype: PerformancePaintTiming;\n    new(): PerformancePaintTiming;\n};\n\n/** Enables retrieval and analysis of detailed network timing data regarding the loading of an application\'s resources. An application can use the timing metrics to determine, for example, the length of time it takes to fetch a specific resource, such as an XMLHttpRequest, <SVG>, image, or script. */\ninterface PerformanceResourceTiming extends PerformanceEntry {\n    readonly connectEnd: DOMHighResTimeStamp;\n    readonly connectStart: DOMHighResTimeStamp;\n    readonly decodedBodySize: number;\n    readonly domainLookupEnd: DOMHighResTimeStamp;\n    readonly domainLookupStart: DOMHighResTimeStamp;\n    readonly encodedBodySize: number;\n    readonly fetchStart: DOMHighResTimeStamp;\n    readonly initiatorType: string;\n    readonly nextHopProtocol: string;\n    readonly redirectEnd: DOMHighResTimeStamp;\n    readonly redirectStart: DOMHighResTimeStamp;\n    readonly requestStart: DOMHighResTimeStamp;\n    readonly responseEnd: DOMHighResTimeStamp;\n    readonly responseStart: DOMHighResTimeStamp;\n    readonly secureConnectionStart: DOMHighResTimeStamp;\n    readonly serverTiming: ReadonlyArray<PerformanceServerTiming>;\n    readonly transferSize: number;\n    readonly workerStart: DOMHighResTimeStamp;\n    toJSON(): any;\n}\n\ndeclare var PerformanceResourceTiming: {\n    prototype: PerformanceResourceTiming;\n    new(): PerformanceResourceTiming;\n};\n\ninterface PerformanceServerTiming {\n    readonly description: string;\n    readonly duration: DOMHighResTimeStamp;\n    readonly name: string;\n    toJSON(): any;\n}\n\ndeclare var PerformanceServerTiming: {\n    prototype: PerformanceServerTiming;\n    new(): PerformanceServerTiming;\n};\n\n/**\n * A legacy interface kept for backwards compatibility and contains properties that offer performance timing information for various events which occur during the loading and use of the current page. You get a PerformanceTiming object describing your page using the window.performance.timing property.\n * @deprecated This interface is deprecated in the Navigation Timing Level 2 specification. Please use the PerformanceNavigationTiming interface instead.\n */\ninterface PerformanceTiming {\n    /** @deprecated */\n    readonly connectEnd: number;\n    /** @deprecated */\n    readonly connectStart: number;\n    /** @deprecated */\n    readonly domComplete: number;\n    /** @deprecated */\n    readonly domContentLoadedEventEnd: number;\n    /** @deprecated */\n    readonly domContentLoadedEventStart: number;\n    /** @deprecated */\n    readonly domInteractive: number;\n    /** @deprecated */\n    readonly domLoading: number;\n    /** @deprecated */\n    readonly domainLookupEnd: number;\n    /** @deprecated */\n    readonly domainLookupStart: number;\n    /** @deprecated */\n    readonly fetchStart: number;\n    /** @deprecated */\n    readonly loadEventEnd: number;\n    /** @deprecated */\n    readonly loadEventStart: number;\n    /** @deprecated */\n    readonly navigationStart: number;\n    /** @deprecated */\n    readonly redirectEnd: number;\n    /** @deprecated */\n    readonly redirectStart: number;\n    /** @deprecated */\n    readonly requestStart: number;\n    /** @deprecated */\n    readonly responseEnd: number;\n    /** @deprecated */\n    readonly responseStart: number;\n    /** @deprecated */\n    readonly secureConnectionStart: number;\n    /** @deprecated */\n    readonly unloadEventEnd: number;\n    /** @deprecated */\n    readonly unloadEventStart: number;\n    /** @deprecated */\n    toJSON(): any;\n}\n\n/** @deprecated */\ndeclare var PerformanceTiming: {\n    prototype: PerformanceTiming;\n    new(): PerformanceTiming;\n};\n\n/** PeriodicWave has no inputs or outputs; it is used to define custom oscillators when calling OscillatorNode.setPeriodicWave(). The PeriodicWave itself is created/returned by AudioContext.createPeriodicWave(). */\ninterface PeriodicWave {\n}\n\ndeclare var PeriodicWave: {\n    prototype: PeriodicWave;\n    new(context: BaseAudioContext, options?: PeriodicWaveOptions): PeriodicWave;\n};\n\ninterface PermissionStatusEventMap {\n    "change": Event;\n}\n\ninterface PermissionStatus extends EventTarget {\n    onchange: ((this: PermissionStatus, ev: Event) => any) | null;\n    readonly state: PermissionState;\n    addEventListener<K extends keyof PermissionStatusEventMap>(type: K, listener: (this: PermissionStatus, ev: PermissionStatusEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof PermissionStatusEventMap>(type: K, listener: (this: PermissionStatus, ev: PermissionStatusEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var PermissionStatus: {\n    prototype: PermissionStatus;\n    new(): PermissionStatus;\n};\n\ninterface Permissions {\n    query(permissionDesc: PermissionDescriptor): Promise<PermissionStatus>;\n}\n\ndeclare var Permissions: {\n    prototype: Permissions;\n    new(): Permissions;\n};\n\ninterface PictureInPictureWindowEventMap {\n    "resize": Event;\n}\n\ninterface PictureInPictureWindow extends EventTarget {\n    readonly height: number;\n    onresize: ((this: PictureInPictureWindow, ev: Event) => any) | null;\n    readonly width: number;\n    addEventListener<K extends keyof PictureInPictureWindowEventMap>(type: K, listener: (this: PictureInPictureWindow, ev: PictureInPictureWindowEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof PictureInPictureWindowEventMap>(type: K, listener: (this: PictureInPictureWindow, ev: PictureInPictureWindowEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var PictureInPictureWindow: {\n    prototype: PictureInPictureWindow;\n    new(): PictureInPictureWindow;\n};\n\n/**\n * Provides information about a browser plugin.\n * @deprecated\n */\ninterface Plugin {\n    /**\n     * Returns the plugin\'s description.\n     * @deprecated\n     */\n    readonly description: string;\n    /**\n     * Returns the plugin library\'s filename, if applicable on the current platform.\n     * @deprecated\n     */\n    readonly filename: string;\n    /**\n     * Returns the number of MIME types, represented by MimeType objects, supported by the plugin.\n     * @deprecated\n     */\n    readonly length: number;\n    /**\n     * Returns the plugin\'s name.\n     * @deprecated\n     */\n    readonly name: string;\n    /**\n     * Returns the specified MimeType object.\n     * @deprecated\n     */\n    item(index: number): MimeType | null;\n    /** @deprecated */\n    namedItem(name: string): MimeType | null;\n    [index: number]: MimeType;\n}\n\n/** @deprecated */\ndeclare var Plugin: {\n    prototype: Plugin;\n    new(): Plugin;\n};\n\n/**\n * Used to store a list of Plugin objects describing the available plugins; it\'s returned by the window.navigator.plugins property. The PluginArray is not a JavaScript array, but has the length property and supports accessing individual items using bracket notation (plugins[2]), as well as via item(index) and namedItem("name") methods.\n * @deprecated\n */\ninterface PluginArray {\n    /** @deprecated */\n    readonly length: number;\n    /** @deprecated */\n    item(index: number): Plugin | null;\n    /** @deprecated */\n    namedItem(name: string): Plugin | null;\n    /** @deprecated */\n    refresh(): void;\n    [index: number]: Plugin;\n}\n\n/** @deprecated */\ndeclare var PluginArray: {\n    prototype: PluginArray;\n    new(): PluginArray;\n};\n\n/** The state of a DOM event produced by a pointer such as the geometry of the contact point, the device type that generated the event, the amount of pressure that was applied on the contact surface, etc. */\ninterface PointerEvent extends MouseEvent {\n    readonly height: number;\n    readonly isPrimary: boolean;\n    readonly pointerId: number;\n    readonly pointerType: string;\n    readonly pressure: number;\n    readonly tangentialPressure: number;\n    readonly tiltX: number;\n    readonly tiltY: number;\n    readonly twist: number;\n    readonly width: number;\n    getCoalescedEvents(): PointerEvent[];\n    getPredictedEvents(): PointerEvent[];\n}\n\ndeclare var PointerEvent: {\n    prototype: PointerEvent;\n    new(type: string, eventInitDict?: PointerEventInit): PointerEvent;\n};\n\n/** PopStateEvent is an event handler for the popstate event on the window. */\ninterface PopStateEvent extends Event {\n    /**\n     * Returns a copy of the information that was provided to pushState() or replaceState().\n     */\n    readonly state: any;\n}\n\ndeclare var PopStateEvent: {\n    prototype: PopStateEvent;\n    new(type: string, eventInitDict?: PopStateEventInit): PopStateEvent;\n};\n\n/** A processing instruction embeds application-specific instructions in XML which can be ignored by other applications that don\'t recognize them. */\ninterface ProcessingInstruction extends CharacterData, LinkStyle {\n    readonly ownerDocument: Document;\n    readonly target: string;\n}\n\ndeclare var ProcessingInstruction: {\n    prototype: ProcessingInstruction;\n    new(): ProcessingInstruction;\n};\n\n/** Events measuring progress of an underlying process, like an HTTP request (for an XMLHttpRequest, or the loading of the underlying resource of an <img>, <audio>, <video>, <style> or <link>). */\ninterface ProgressEvent<T extends EventTarget = EventTarget> extends Event {\n    readonly lengthComputable: boolean;\n    readonly loaded: number;\n    readonly target: T | null;\n    readonly total: number;\n}\n\ndeclare var ProgressEvent: {\n    prototype: ProgressEvent;\n    new(type: string, eventInitDict?: ProgressEventInit): ProgressEvent;\n};\n\ninterface PromiseRejectionEvent extends Event {\n    readonly promise: Promise<any>;\n    readonly reason: any;\n}\n\ndeclare var PromiseRejectionEvent: {\n    prototype: PromiseRejectionEvent;\n    new(type: string, eventInitDict: PromiseRejectionEventInit): PromiseRejectionEvent;\n};\n\ninterface PublicKeyCredential extends Credential {\n    readonly rawId: ArrayBuffer;\n    readonly response: AuthenticatorResponse;\n    getClientExtensionResults(): AuthenticationExtensionsClientOutputs;\n}\n\ndeclare var PublicKeyCredential: {\n    prototype: PublicKeyCredential;\n    new(): PublicKeyCredential;\n    isUserVerifyingPlatformAuthenticatorAvailable(): Promise<boolean>;\n};\n\n/** This Push API interface provides a way to receive notifications from third-party servers as well as request URLs for push notifications. */\ninterface PushManager {\n    getSubscription(): Promise<PushSubscription | null>;\n    permissionState(options?: PushSubscriptionOptionsInit): Promise<PushPermissionState>;\n    subscribe(options?: PushSubscriptionOptionsInit): Promise<PushSubscription>;\n}\n\ndeclare var PushManager: {\n    prototype: PushManager;\n    new(): PushManager;\n    readonly supportedContentEncodings: ReadonlyArray<string>;\n};\n\n/** This Push API interface provides a subcription\'s URL endpoint and allows unsubscription from a push service. */\ninterface PushSubscription {\n    readonly endpoint: string;\n    readonly options: PushSubscriptionOptions;\n    getKey(name: PushEncryptionKeyName): ArrayBuffer | null;\n    toJSON(): PushSubscriptionJSON;\n    unsubscribe(): Promise<boolean>;\n}\n\ndeclare var PushSubscription: {\n    prototype: PushSubscription;\n    new(): PushSubscription;\n};\n\ninterface PushSubscriptionOptions {\n    readonly applicationServerKey: ArrayBuffer | null;\n}\n\ndeclare var PushSubscriptionOptions: {\n    prototype: PushSubscriptionOptions;\n    new(): PushSubscriptionOptions;\n};\n\ninterface RTCCertificate {\n    readonly expires: DOMTimeStamp;\n    getFingerprints(): RTCDtlsFingerprint[];\n}\n\ndeclare var RTCCertificate: {\n    prototype: RTCCertificate;\n    new(): RTCCertificate;\n};\n\ninterface RTCDTMFSenderEventMap {\n    "tonechange": RTCDTMFToneChangeEvent;\n}\n\ninterface RTCDTMFSender extends EventTarget {\n    readonly canInsertDTMF: boolean;\n    ontonechange: ((this: RTCDTMFSender, ev: RTCDTMFToneChangeEvent) => any) | null;\n    readonly toneBuffer: string;\n    insertDTMF(tones: string, duration?: number, interToneGap?: number): void;\n    addEventListener<K extends keyof RTCDTMFSenderEventMap>(type: K, listener: (this: RTCDTMFSender, ev: RTCDTMFSenderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof RTCDTMFSenderEventMap>(type: K, listener: (this: RTCDTMFSender, ev: RTCDTMFSenderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var RTCDTMFSender: {\n    prototype: RTCDTMFSender;\n    new(): RTCDTMFSender;\n};\n\n/** Events sent to indicate that DTMF tones have started or finished playing. This interface is used by the tonechange event. */\ninterface RTCDTMFToneChangeEvent extends Event {\n    readonly tone: string;\n}\n\ndeclare var RTCDTMFToneChangeEvent: {\n    prototype: RTCDTMFToneChangeEvent;\n    new(type: string, eventInitDict?: RTCDTMFToneChangeEventInit): RTCDTMFToneChangeEvent;\n};\n\ninterface RTCDataChannelEventMap {\n    "bufferedamountlow": Event;\n    "close": Event;\n    "error": Event;\n    "message": MessageEvent;\n    "open": Event;\n}\n\ninterface RTCDataChannel extends EventTarget {\n    binaryType: BinaryType;\n    readonly bufferedAmount: number;\n    bufferedAmountLowThreshold: number;\n    readonly id: number | null;\n    readonly label: string;\n    readonly maxPacketLifeTime: number | null;\n    readonly maxRetransmits: number | null;\n    readonly negotiated: boolean;\n    onbufferedamountlow: ((this: RTCDataChannel, ev: Event) => any) | null;\n    onclose: ((this: RTCDataChannel, ev: Event) => any) | null;\n    onerror: ((this: RTCDataChannel, ev: Event) => any) | null;\n    onmessage: ((this: RTCDataChannel, ev: MessageEvent) => any) | null;\n    onopen: ((this: RTCDataChannel, ev: Event) => any) | null;\n    readonly ordered: boolean;\n    readonly protocol: string;\n    readonly readyState: RTCDataChannelState;\n    close(): void;\n    send(data: string): void;\n    send(data: Blob): void;\n    send(data: ArrayBuffer): void;\n    send(data: ArrayBufferView): void;\n    addEventListener<K extends keyof RTCDataChannelEventMap>(type: K, listener: (this: RTCDataChannel, ev: RTCDataChannelEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof RTCDataChannelEventMap>(type: K, listener: (this: RTCDataChannel, ev: RTCDataChannelEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var RTCDataChannel: {\n    prototype: RTCDataChannel;\n    new(): RTCDataChannel;\n};\n\ninterface RTCDataChannelEvent extends Event {\n    readonly channel: RTCDataChannel;\n}\n\ndeclare var RTCDataChannelEvent: {\n    prototype: RTCDataChannelEvent;\n    new(type: string, eventInitDict: RTCDataChannelEventInit): RTCDataChannelEvent;\n};\n\ninterface RTCDtlsTransportEventMap {\n    "statechange": Event;\n}\n\ninterface RTCDtlsTransport extends EventTarget {\n    onstatechange: ((this: RTCDtlsTransport, ev: Event) => any) | null;\n    readonly state: RTCDtlsTransportState;\n    addEventListener<K extends keyof RTCDtlsTransportEventMap>(type: K, listener: (this: RTCDtlsTransport, ev: RTCDtlsTransportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof RTCDtlsTransportEventMap>(type: K, listener: (this: RTCDtlsTransport, ev: RTCDtlsTransportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var RTCDtlsTransport: {\n    prototype: RTCDtlsTransport;\n    new(): RTCDtlsTransport;\n};\n\n/** The RTCIceCandidate interface—part of the WebRTC API—represents a candidate Internet Connectivity Establishment (ICE) configuration which may be used to establish an RTCPeerConnection. */\ninterface RTCIceCandidate {\n    readonly address: string | null;\n    readonly candidate: string;\n    readonly component: RTCIceComponent | null;\n    readonly foundation: string | null;\n    readonly port: number | null;\n    readonly priority: number | null;\n    readonly protocol: RTCIceProtocol | null;\n    readonly relatedAddress: string | null;\n    readonly relatedPort: number | null;\n    readonly sdpMLineIndex: number | null;\n    readonly sdpMid: string | null;\n    readonly tcpType: RTCIceTcpCandidateType | null;\n    readonly type: RTCIceCandidateType | null;\n    readonly usernameFragment: string | null;\n    toJSON(): RTCIceCandidateInit;\n}\n\ndeclare var RTCIceCandidate: {\n    prototype: RTCIceCandidate;\n    new(candidateInitDict?: RTCIceCandidateInit): RTCIceCandidate;\n};\n\n/** Provides access to information about the ICE transport layer over which the data is being sent and received. */\ninterface RTCIceTransport extends EventTarget {\n    readonly gatheringState: RTCIceGathererState;\n    readonly state: RTCIceTransportState;\n}\n\ndeclare var RTCIceTransport: {\n    prototype: RTCIceTransport;\n    new(): RTCIceTransport;\n};\n\ninterface RTCPeerConnectionEventMap {\n    "connectionstatechange": Event;\n    "datachannel": RTCDataChannelEvent;\n    "icecandidate": RTCPeerConnectionIceEvent;\n    "icecandidateerror": Event;\n    "iceconnectionstatechange": Event;\n    "icegatheringstatechange": Event;\n    "negotiationneeded": Event;\n    "signalingstatechange": Event;\n    "track": RTCTrackEvent;\n}\n\n/** A WebRTC connection between the local computer and a remote peer. It provides methods to connect to a remote peer, maintain and monitor the connection, and close the connection once it\'s no longer needed. */\ninterface RTCPeerConnection extends EventTarget {\n    readonly canTrickleIceCandidates: boolean | null;\n    readonly connectionState: RTCPeerConnectionState;\n    readonly currentLocalDescription: RTCSessionDescription | null;\n    readonly currentRemoteDescription: RTCSessionDescription | null;\n    readonly iceConnectionState: RTCIceConnectionState;\n    readonly iceGatheringState: RTCIceGatheringState;\n    readonly localDescription: RTCSessionDescription | null;\n    onconnectionstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\n    ondatachannel: ((this: RTCPeerConnection, ev: RTCDataChannelEvent) => any) | null;\n    onicecandidate: ((this: RTCPeerConnection, ev: RTCPeerConnectionIceEvent) => any) | null;\n    onicecandidateerror: ((this: RTCPeerConnection, ev: Event) => any) | null;\n    oniceconnectionstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\n    onicegatheringstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\n    onnegotiationneeded: ((this: RTCPeerConnection, ev: Event) => any) | null;\n    onsignalingstatechange: ((this: RTCPeerConnection, ev: Event) => any) | null;\n    ontrack: ((this: RTCPeerConnection, ev: RTCTrackEvent) => any) | null;\n    readonly pendingLocalDescription: RTCSessionDescription | null;\n    readonly pendingRemoteDescription: RTCSessionDescription | null;\n    readonly remoteDescription: RTCSessionDescription | null;\n    readonly signalingState: RTCSignalingState;\n    addIceCandidate(candidate?: RTCIceCandidateInit): Promise<void>;\n    /** @deprecated */\n    addIceCandidate(candidate: RTCIceCandidateInit, successCallback: VoidFunction, failureCallback: RTCPeerConnectionErrorCallback): Promise<void>;\n    addTrack(track: MediaStreamTrack, ...streams: MediaStream[]): RTCRtpSender;\n    addTransceiver(trackOrKind: MediaStreamTrack | string, init?: RTCRtpTransceiverInit): RTCRtpTransceiver;\n    close(): void;\n    createAnswer(options?: RTCAnswerOptions): Promise<RTCSessionDescriptionInit>;\n    /** @deprecated */\n    createAnswer(successCallback: RTCSessionDescriptionCallback, failureCallback: RTCPeerConnectionErrorCallback): Promise<void>;\n    createDataChannel(label: string, dataChannelDict?: RTCDataChannelInit): RTCDataChannel;\n    createOffer(options?: RTCOfferOptions): Promise<RTCSessionDescriptionInit>;\n    /** @deprecated */\n    createOffer(successCallback: RTCSessionDescriptionCallback, failureCallback: RTCPeerConnectionErrorCallback, options?: RTCOfferOptions): Promise<void>;\n    getConfiguration(): RTCConfiguration;\n    getReceivers(): RTCRtpReceiver[];\n    getSenders(): RTCRtpSender[];\n    getStats(selector?: MediaStreamTrack | null): Promise<RTCStatsReport>;\n    getTransceivers(): RTCRtpTransceiver[];\n    removeTrack(sender: RTCRtpSender): void;\n    restartIce(): void;\n    setConfiguration(configuration?: RTCConfiguration): void;\n    setLocalDescription(description?: RTCLocalSessionDescriptionInit): Promise<void>;\n    /** @deprecated */\n    setLocalDescription(description: RTCLocalSessionDescriptionInit, successCallback: VoidFunction, failureCallback: RTCPeerConnectionErrorCallback): Promise<void>;\n    setRemoteDescription(description: RTCSessionDescriptionInit): Promise<void>;\n    /** @deprecated */\n    setRemoteDescription(description: RTCSessionDescriptionInit, successCallback: VoidFunction, failureCallback: RTCPeerConnectionErrorCallback): Promise<void>;\n    addEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: RTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof RTCPeerConnectionEventMap>(type: K, listener: (this: RTCPeerConnection, ev: RTCPeerConnectionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var RTCPeerConnection: {\n    prototype: RTCPeerConnection;\n    new(configuration?: RTCConfiguration): RTCPeerConnection;\n    generateCertificate(keygenAlgorithm: AlgorithmIdentifier): Promise<RTCCertificate>;\n};\n\ninterface RTCPeerConnectionIceErrorEvent extends Event {\n    readonly address: string | null;\n    readonly errorCode: number;\n    readonly errorText: string;\n    readonly port: number | null;\n    readonly url: string;\n}\n\ndeclare var RTCPeerConnectionIceErrorEvent: {\n    prototype: RTCPeerConnectionIceErrorEvent;\n    new(type: string, eventInitDict: RTCPeerConnectionIceErrorEventInit): RTCPeerConnectionIceErrorEvent;\n};\n\n/** Events that occurs in relation to ICE candidates with the target, usually an RTCPeerConnection. Only one event is of this type: icecandidate. */\ninterface RTCPeerConnectionIceEvent extends Event {\n    readonly candidate: RTCIceCandidate | null;\n}\n\ndeclare var RTCPeerConnectionIceEvent: {\n    prototype: RTCPeerConnectionIceEvent;\n    new(type: string, eventInitDict?: RTCPeerConnectionIceEventInit): RTCPeerConnectionIceEvent;\n};\n\n/** This WebRTC API interface manages the reception and decoding of data for a MediaStreamTrack on an RTCPeerConnection. */\ninterface RTCRtpReceiver {\n    readonly track: MediaStreamTrack;\n    readonly transport: RTCDtlsTransport | null;\n    getContributingSources(): RTCRtpContributingSource[];\n    getParameters(): RTCRtpReceiveParameters;\n    getStats(): Promise<RTCStatsReport>;\n    getSynchronizationSources(): RTCRtpSynchronizationSource[];\n}\n\ndeclare var RTCRtpReceiver: {\n    prototype: RTCRtpReceiver;\n    new(): RTCRtpReceiver;\n    getCapabilities(kind: string): RTCRtpCapabilities | null;\n};\n\n/** Provides the ability to control and obtain details about how a particular MediaStreamTrack is encoded and sent to a remote peer. */\ninterface RTCRtpSender {\n    readonly dtmf: RTCDTMFSender | null;\n    readonly track: MediaStreamTrack | null;\n    readonly transport: RTCDtlsTransport | null;\n    getParameters(): RTCRtpSendParameters;\n    getStats(): Promise<RTCStatsReport>;\n    replaceTrack(withTrack: MediaStreamTrack | null): Promise<void>;\n    setParameters(parameters: RTCRtpSendParameters): Promise<void>;\n    setStreams(...streams: MediaStream[]): void;\n}\n\ndeclare var RTCRtpSender: {\n    prototype: RTCRtpSender;\n    new(): RTCRtpSender;\n    getCapabilities(kind: string): RTCRtpCapabilities | null;\n};\n\ninterface RTCRtpTransceiver {\n    readonly currentDirection: RTCRtpTransceiverDirection | null;\n    direction: RTCRtpTransceiverDirection;\n    readonly mid: string | null;\n    readonly receiver: RTCRtpReceiver;\n    readonly sender: RTCRtpSender;\n    stop(): void;\n}\n\ndeclare var RTCRtpTransceiver: {\n    prototype: RTCRtpTransceiver;\n    new(): RTCRtpTransceiver;\n};\n\n/** One end of a connection—or potential connection—and how it\'s configured. Each RTCSessionDescription consists of a description type indicating which part of the offer/answer negotiation process it describes and of the SDP descriptor of the session. */\ninterface RTCSessionDescription {\n    readonly sdp: string;\n    readonly type: RTCSdpType;\n    toJSON(): any;\n}\n\ndeclare var RTCSessionDescription: {\n    prototype: RTCSessionDescription;\n    new(descriptionInitDict: RTCSessionDescriptionInit): RTCSessionDescription;\n};\n\ninterface RTCStatsReport {\n    forEach(callbackfn: (value: any, key: string, parent: RTCStatsReport) => void, thisArg?: any): void;\n}\n\ndeclare var RTCStatsReport: {\n    prototype: RTCStatsReport;\n    new(): RTCStatsReport;\n};\n\ninterface RTCTrackEvent extends Event {\n    readonly receiver: RTCRtpReceiver;\n    readonly streams: ReadonlyArray<MediaStream>;\n    readonly track: MediaStreamTrack;\n    readonly transceiver: RTCRtpTransceiver;\n}\n\ndeclare var RTCTrackEvent: {\n    prototype: RTCTrackEvent;\n    new(type: string, eventInitDict: RTCTrackEventInit): RTCTrackEvent;\n};\n\ninterface RadioNodeList extends NodeList {\n    value: string;\n}\n\ndeclare var RadioNodeList: {\n    prototype: RadioNodeList;\n    new(): RadioNodeList;\n};\n\n/** A fragment of a document that can contain nodes and parts of text nodes. */\ninterface Range extends AbstractRange {\n    /**\n     * Returns the node, furthest away from the document, that is an ancestor of both range\'s start node and end node.\n     */\n    readonly commonAncestorContainer: Node;\n    cloneContents(): DocumentFragment;\n    cloneRange(): Range;\n    collapse(toStart?: boolean): void;\n    compareBoundaryPoints(how: number, sourceRange: Range): number;\n    /**\n     * Returns −1 if the point is before the range, 0 if the point is in the range, and 1 if the point is after the range.\n     */\n    comparePoint(node: Node, offset: number): number;\n    createContextualFragment(fragment: string): DocumentFragment;\n    deleteContents(): void;\n    detach(): void;\n    extractContents(): DocumentFragment;\n    getBoundingClientRect(): DOMRect;\n    getClientRects(): DOMRectList;\n    insertNode(node: Node): void;\n    /**\n     * Returns whether range intersects node.\n     */\n    intersectsNode(node: Node): boolean;\n    isPointInRange(node: Node, offset: number): boolean;\n    selectNode(node: Node): void;\n    selectNodeContents(node: Node): void;\n    setEnd(node: Node, offset: number): void;\n    setEndAfter(node: Node): void;\n    setEndBefore(node: Node): void;\n    setStart(node: Node, offset: number): void;\n    setStartAfter(node: Node): void;\n    setStartBefore(node: Node): void;\n    surroundContents(newParent: Node): void;\n    toString(): string;\n    readonly END_TO_END: number;\n    readonly END_TO_START: number;\n    readonly START_TO_END: number;\n    readonly START_TO_START: number;\n}\n\ndeclare var Range: {\n    prototype: Range;\n    new(): Range;\n    readonly END_TO_END: number;\n    readonly END_TO_START: number;\n    readonly START_TO_END: number;\n    readonly START_TO_START: number;\n    toString(): string;\n};\n\n/** This Streams API interface represents a readable stream of byte data. The Fetch API offers a concrete instance of a ReadableStream through the body property of a Response object. */\ninterface ReadableStream<R = any> {\n    readonly locked: boolean;\n    cancel(reason?: any): Promise<void>;\n    getReader(): ReadableStreamDefaultReader<R>;\n    pipeThrough<T>(transform: ReadableWritablePair<T, R>, options?: StreamPipeOptions): ReadableStream<T>;\n    pipeTo(destination: WritableStream<R>, options?: StreamPipeOptions): Promise<void>;\n    tee(): [ReadableStream<R>, ReadableStream<R>];\n    forEach(callbackfn: (value: any, key: number, parent: ReadableStream<R>) => void, thisArg?: any): void;\n}\n\ndeclare var ReadableStream: {\n    prototype: ReadableStream;\n    new<R = any>(underlyingSource?: UnderlyingSource<R>, strategy?: QueuingStrategy<R>): ReadableStream<R>;\n};\n\ninterface ReadableStreamDefaultController<R = any> {\n    readonly desiredSize: number | null;\n    close(): void;\n    enqueue(chunk?: R): void;\n    error(e?: any): void;\n}\n\ndeclare var ReadableStreamDefaultController: {\n    prototype: ReadableStreamDefaultController;\n    new(): ReadableStreamDefaultController;\n};\n\ninterface ReadableStreamDefaultReader<R = any> extends ReadableStreamGenericReader {\n    read(): Promise<ReadableStreamDefaultReadResult<R>>;\n    releaseLock(): void;\n}\n\ndeclare var ReadableStreamDefaultReader: {\n    prototype: ReadableStreamDefaultReader;\n    new<R = any>(stream: ReadableStream<R>): ReadableStreamDefaultReader<R>;\n};\n\ninterface ReadableStreamGenericReader {\n    readonly closed: Promise<undefined>;\n    cancel(reason?: any): Promise<void>;\n}\n\ninterface RemotePlaybackEventMap {\n    "connect": Event;\n    "connecting": Event;\n    "disconnect": Event;\n}\n\ninterface RemotePlayback extends EventTarget {\n    onconnect: ((this: RemotePlayback, ev: Event) => any) | null;\n    onconnecting: ((this: RemotePlayback, ev: Event) => any) | null;\n    ondisconnect: ((this: RemotePlayback, ev: Event) => any) | null;\n    readonly state: RemotePlaybackState;\n    cancelWatchAvailability(id?: number): Promise<void>;\n    prompt(): Promise<void>;\n    watchAvailability(callback: RemotePlaybackAvailabilityCallback): Promise<number>;\n    addEventListener<K extends keyof RemotePlaybackEventMap>(type: K, listener: (this: RemotePlayback, ev: RemotePlaybackEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof RemotePlaybackEventMap>(type: K, listener: (this: RemotePlayback, ev: RemotePlaybackEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var RemotePlayback: {\n    prototype: RemotePlayback;\n    new(): RemotePlayback;\n};\n\n/** This Fetch API interface represents a resource request. */\ninterface Request extends Body {\n    /**\n     * Returns the cache mode associated with request, which is a string indicating how the request will interact with the browser\'s cache when fetching.\n     */\n    readonly cache: RequestCache;\n    /**\n     * Returns the credentials mode associated with request, which is a string indicating whether credentials will be sent with the request always, never, or only when sent to a same-origin URL.\n     */\n    readonly credentials: RequestCredentials;\n    /**\n     * Returns the kind of resource requested by request, e.g., "document" or "script".\n     */\n    readonly destination: RequestDestination;\n    /**\n     * Returns a Headers object consisting of the headers associated with request. Note that headers added in the network layer by the user agent will not be accounted for in this object, e.g., the "Host" header.\n     */\n    readonly headers: Headers;\n    /**\n     * Returns request\'s subresource integrity metadata, which is a cryptographic hash of the resource being fetched. Its value consists of multiple hashes separated by whitespace. [SRI]\n     */\n    readonly integrity: string;\n    /**\n     * Returns a boolean indicating whether or not request can outlive the global in which it was created.\n     */\n    readonly keepalive: boolean;\n    /**\n     * Returns request\'s HTTP method, which is "GET" by default.\n     */\n    readonly method: string;\n    /**\n     * Returns the mode associated with request, which is a string indicating whether the request will use CORS, or will be restricted to same-origin URLs.\n     */\n    readonly mode: RequestMode;\n    /**\n     * Returns the redirect mode associated with request, which is a string indicating how redirects for the request will be handled during fetching. A request will follow redirects by default.\n     */\n    readonly redirect: RequestRedirect;\n    /**\n     * Returns the referrer of request. Its value can be a same-origin URL if explicitly set in init, the empty string to indicate no referrer, and "about:client" when defaulting to the global\'s default. This is used during fetching to determine the value of the `Referer` header of the request being made.\n     */\n    readonly referrer: string;\n    /**\n     * Returns the referrer policy associated with request. This is used during fetching to compute the value of the request\'s referrer.\n     */\n    readonly referrerPolicy: ReferrerPolicy;\n    /**\n     * Returns the signal associated with request, which is an AbortSignal object indicating whether or not request has been aborted, and its abort event handler.\n     */\n    readonly signal: AbortSignal;\n    /**\n     * Returns the URL of request as a string.\n     */\n    readonly url: string;\n    clone(): Request;\n}\n\ndeclare var Request: {\n    prototype: Request;\n    new(input: RequestInfo, init?: RequestInit): Request;\n};\n\ninterface ResizeObserver {\n    disconnect(): void;\n    observe(target: Element, options?: ResizeObserverOptions): void;\n    unobserve(target: Element): void;\n}\n\ndeclare var ResizeObserver: {\n    prototype: ResizeObserver;\n    new(callback: ResizeObserverCallback): ResizeObserver;\n};\n\ninterface ResizeObserverEntry {\n    readonly borderBoxSize: ReadonlyArray<ResizeObserverSize>;\n    readonly contentBoxSize: ReadonlyArray<ResizeObserverSize>;\n    readonly contentRect: DOMRectReadOnly;\n    readonly target: Element;\n}\n\ndeclare var ResizeObserverEntry: {\n    prototype: ResizeObserverEntry;\n    new(): ResizeObserverEntry;\n};\n\ninterface ResizeObserverSize {\n    readonly blockSize: number;\n    readonly inlineSize: number;\n}\n\ndeclare var ResizeObserverSize: {\n    prototype: ResizeObserverSize;\n    new(): ResizeObserverSize;\n};\n\n/** This Fetch API interface represents the response to a request. */\ninterface Response extends Body {\n    readonly headers: Headers;\n    readonly ok: boolean;\n    readonly redirected: boolean;\n    readonly status: number;\n    readonly statusText: string;\n    readonly type: ResponseType;\n    readonly url: string;\n    clone(): Response;\n}\n\ndeclare var Response: {\n    prototype: Response;\n    new(body?: BodyInit | null, init?: ResponseInit): Response;\n    error(): Response;\n    redirect(url: string | URL, status?: number): Response;\n};\n\n/** Provides access to the properties of <a> element, as well as methods to manipulate them. */\ninterface SVGAElement extends SVGGraphicsElement, SVGURIReference {\n    rel: string;\n    readonly relList: DOMTokenList;\n    readonly target: SVGAnimatedString;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGAElement: {\n    prototype: SVGAElement;\n    new(): SVGAElement;\n};\n\n/** Used to represent a value that can be an <angle> or <number> value. An SVGAngle reflected through the animVal attribute is always read only. */\ninterface SVGAngle {\n    readonly unitType: number;\n    value: number;\n    valueAsString: string;\n    valueInSpecifiedUnits: number;\n    convertToSpecifiedUnits(unitType: number): void;\n    newValueSpecifiedUnits(unitType: number, valueInSpecifiedUnits: number): void;\n    readonly SVG_ANGLETYPE_DEG: number;\n    readonly SVG_ANGLETYPE_GRAD: number;\n    readonly SVG_ANGLETYPE_RAD: number;\n    readonly SVG_ANGLETYPE_UNKNOWN: number;\n    readonly SVG_ANGLETYPE_UNSPECIFIED: number;\n}\n\ndeclare var SVGAngle: {\n    prototype: SVGAngle;\n    new(): SVGAngle;\n    readonly SVG_ANGLETYPE_DEG: number;\n    readonly SVG_ANGLETYPE_GRAD: number;\n    readonly SVG_ANGLETYPE_RAD: number;\n    readonly SVG_ANGLETYPE_UNKNOWN: number;\n    readonly SVG_ANGLETYPE_UNSPECIFIED: number;\n};\n\ninterface SVGAnimateElement extends SVGAnimationElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAnimateElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAnimateElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGAnimateElement: {\n    prototype: SVGAnimateElement;\n    new(): SVGAnimateElement;\n};\n\ninterface SVGAnimateMotionElement extends SVGAnimationElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAnimateMotionElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAnimateMotionElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGAnimateMotionElement: {\n    prototype: SVGAnimateMotionElement;\n    new(): SVGAnimateMotionElement;\n};\n\ninterface SVGAnimateTransformElement extends SVGAnimationElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAnimateTransformElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAnimateTransformElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGAnimateTransformElement: {\n    prototype: SVGAnimateTransformElement;\n    new(): SVGAnimateTransformElement;\n};\n\n/** Used for attributes of basic type <angle> which can be animated. */\ninterface SVGAnimatedAngle {\n    readonly animVal: SVGAngle;\n    readonly baseVal: SVGAngle;\n}\n\ndeclare var SVGAnimatedAngle: {\n    prototype: SVGAnimatedAngle;\n    new(): SVGAnimatedAngle;\n};\n\n/** Used for attributes of type boolean which can be animated. */\ninterface SVGAnimatedBoolean {\n    readonly animVal: boolean;\n    baseVal: boolean;\n}\n\ndeclare var SVGAnimatedBoolean: {\n    prototype: SVGAnimatedBoolean;\n    new(): SVGAnimatedBoolean;\n};\n\n/** Used for attributes whose value must be a constant from a particular enumeration and which can be animated. */\ninterface SVGAnimatedEnumeration {\n    readonly animVal: number;\n    baseVal: number;\n}\n\ndeclare var SVGAnimatedEnumeration: {\n    prototype: SVGAnimatedEnumeration;\n    new(): SVGAnimatedEnumeration;\n};\n\n/** Used for attributes of basic type <integer> which can be animated. */\ninterface SVGAnimatedInteger {\n    readonly animVal: number;\n    baseVal: number;\n}\n\ndeclare var SVGAnimatedInteger: {\n    prototype: SVGAnimatedInteger;\n    new(): SVGAnimatedInteger;\n};\n\n/** Used for attributes of basic type <length> which can be animated. */\ninterface SVGAnimatedLength {\n    readonly animVal: SVGLength;\n    readonly baseVal: SVGLength;\n}\n\ndeclare var SVGAnimatedLength: {\n    prototype: SVGAnimatedLength;\n    new(): SVGAnimatedLength;\n};\n\n/** Used for attributes of type SVGLengthList which can be animated. */\ninterface SVGAnimatedLengthList {\n    readonly animVal: SVGLengthList;\n    readonly baseVal: SVGLengthList;\n}\n\ndeclare var SVGAnimatedLengthList: {\n    prototype: SVGAnimatedLengthList;\n    new(): SVGAnimatedLengthList;\n};\n\n/** Used for attributes of basic type <Number> which can be animated. */\ninterface SVGAnimatedNumber {\n    readonly animVal: number;\n    baseVal: number;\n}\n\ndeclare var SVGAnimatedNumber: {\n    prototype: SVGAnimatedNumber;\n    new(): SVGAnimatedNumber;\n};\n\n/** The SVGAnimatedNumber interface is used for attributes which take a list of numbers and which can be animated. */\ninterface SVGAnimatedNumberList {\n    readonly animVal: SVGNumberList;\n    readonly baseVal: SVGNumberList;\n}\n\ndeclare var SVGAnimatedNumberList: {\n    prototype: SVGAnimatedNumberList;\n    new(): SVGAnimatedNumberList;\n};\n\ninterface SVGAnimatedPoints {\n    readonly animatedPoints: SVGPointList;\n    readonly points: SVGPointList;\n}\n\n/** Used for attributes of type SVGPreserveAspectRatio which can be animated. */\ninterface SVGAnimatedPreserveAspectRatio {\n    readonly animVal: SVGPreserveAspectRatio;\n    readonly baseVal: SVGPreserveAspectRatio;\n}\n\ndeclare var SVGAnimatedPreserveAspectRatio: {\n    prototype: SVGAnimatedPreserveAspectRatio;\n    new(): SVGAnimatedPreserveAspectRatio;\n};\n\n/** Used for attributes of basic SVGRect which can be animated. */\ninterface SVGAnimatedRect {\n    readonly animVal: DOMRectReadOnly;\n    readonly baseVal: DOMRect;\n}\n\ndeclare var SVGAnimatedRect: {\n    prototype: SVGAnimatedRect;\n    new(): SVGAnimatedRect;\n};\n\n/** The SVGAnimatedString interface represents string attributes which can be animated from each SVG declaration. You need to create SVG attribute before doing anything else, everything should be declared inside this. */\ninterface SVGAnimatedString {\n    readonly animVal: string;\n    baseVal: string;\n}\n\ndeclare var SVGAnimatedString: {\n    prototype: SVGAnimatedString;\n    new(): SVGAnimatedString;\n};\n\n/** Used for attributes which take a list of numbers and which can be animated. */\ninterface SVGAnimatedTransformList {\n    readonly animVal: SVGTransformList;\n    readonly baseVal: SVGTransformList;\n}\n\ndeclare var SVGAnimatedTransformList: {\n    prototype: SVGAnimatedTransformList;\n    new(): SVGAnimatedTransformList;\n};\n\ninterface SVGAnimationElement extends SVGElement, SVGTests {\n    readonly targetElement: SVGElement | null;\n    beginElement(): void;\n    beginElementAt(offset: number): void;\n    endElement(): void;\n    endElementAt(offset: number): void;\n    getCurrentTime(): number;\n    getSimpleDuration(): number;\n    getStartTime(): number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAnimationElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGAnimationElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGAnimationElement: {\n    prototype: SVGAnimationElement;\n    new(): SVGAnimationElement;\n};\n\n/** An interface for the <circle> element. The circle element is defined by the cx and cy attributes that denote the coordinates of the centre of the circle. */\ninterface SVGCircleElement extends SVGGeometryElement {\n    readonly cx: SVGAnimatedLength;\n    readonly cy: SVGAnimatedLength;\n    readonly r: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGCircleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGCircleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGCircleElement: {\n    prototype: SVGCircleElement;\n    new(): SVGCircleElement;\n};\n\n/** Provides access to the properties of <clipPath> elements, as well as methods to manipulate them. */\ninterface SVGClipPathElement extends SVGElement {\n    readonly clipPathUnits: SVGAnimatedEnumeration;\n    readonly transform: SVGAnimatedTransformList;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGClipPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGClipPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGClipPathElement: {\n    prototype: SVGClipPathElement;\n    new(): SVGClipPathElement;\n};\n\n/** A base interface used by the component transfer function interfaces. */\ninterface SVGComponentTransferFunctionElement extends SVGElement {\n    readonly amplitude: SVGAnimatedNumber;\n    readonly exponent: SVGAnimatedNumber;\n    readonly intercept: SVGAnimatedNumber;\n    readonly offset: SVGAnimatedNumber;\n    readonly slope: SVGAnimatedNumber;\n    readonly tableValues: SVGAnimatedNumberList;\n    readonly type: SVGAnimatedEnumeration;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_DISCRETE: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_GAMMA: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_IDENTITY: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_LINEAR: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_TABLE: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGComponentTransferFunctionElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGComponentTransferFunctionElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGComponentTransferFunctionElement: {\n    prototype: SVGComponentTransferFunctionElement;\n    new(): SVGComponentTransferFunctionElement;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_DISCRETE: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_GAMMA: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_IDENTITY: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_LINEAR: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_TABLE: number;\n    readonly SVG_FECOMPONENTTRANSFER_TYPE_UNKNOWN: number;\n};\n\n/** Corresponds to the <defs> element. */\ninterface SVGDefsElement extends SVGGraphicsElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDefsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDefsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGDefsElement: {\n    prototype: SVGDefsElement;\n    new(): SVGDefsElement;\n};\n\n/** Corresponds to the <desc> element. */\ninterface SVGDescElement extends SVGElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDescElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGDescElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGDescElement: {\n    prototype: SVGDescElement;\n    new(): SVGDescElement;\n};\n\ninterface SVGElementEventMap extends ElementEventMap, DocumentAndElementEventHandlersEventMap, GlobalEventHandlersEventMap {\n}\n\n/** All of the SVG DOM interfaces that correspond directly to elements in the SVG language derive from the SVGElement interface. */\ninterface SVGElement extends Element, DocumentAndElementEventHandlers, DocumentAndElementEventHandlers, ElementCSSInlineStyle, GlobalEventHandlers, GlobalEventHandlers, HTMLOrSVGElement {\n    /** @deprecated */\n    readonly className: any;\n    readonly ownerSVGElement: SVGSVGElement | null;\n    readonly viewportElement: SVGElement | null;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGElement: {\n    prototype: SVGElement;\n    new(): SVGElement;\n};\n\n/** Provides access to the properties of <ellipse> elements. */\ninterface SVGEllipseElement extends SVGGeometryElement {\n    readonly cx: SVGAnimatedLength;\n    readonly cy: SVGAnimatedLength;\n    readonly rx: SVGAnimatedLength;\n    readonly ry: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGEllipseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGEllipseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGEllipseElement: {\n    prototype: SVGEllipseElement;\n    new(): SVGEllipseElement;\n};\n\n/** Corresponds to the <feBlend> element. */\ninterface SVGFEBlendElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly in1: SVGAnimatedString;\n    readonly in2: SVGAnimatedString;\n    readonly mode: SVGAnimatedEnumeration;\n    readonly SVG_FEBLEND_MODE_COLOR: number;\n    readonly SVG_FEBLEND_MODE_COLOR_BURN: number;\n    readonly SVG_FEBLEND_MODE_COLOR_DODGE: number;\n    readonly SVG_FEBLEND_MODE_DARKEN: number;\n    readonly SVG_FEBLEND_MODE_DIFFERENCE: number;\n    readonly SVG_FEBLEND_MODE_EXCLUSION: number;\n    readonly SVG_FEBLEND_MODE_HARD_LIGHT: number;\n    readonly SVG_FEBLEND_MODE_HUE: number;\n    readonly SVG_FEBLEND_MODE_LIGHTEN: number;\n    readonly SVG_FEBLEND_MODE_LUMINOSITY: number;\n    readonly SVG_FEBLEND_MODE_MULTIPLY: number;\n    readonly SVG_FEBLEND_MODE_NORMAL: number;\n    readonly SVG_FEBLEND_MODE_OVERLAY: number;\n    readonly SVG_FEBLEND_MODE_SATURATION: number;\n    readonly SVG_FEBLEND_MODE_SCREEN: number;\n    readonly SVG_FEBLEND_MODE_SOFT_LIGHT: number;\n    readonly SVG_FEBLEND_MODE_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEBlendElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEBlendElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEBlendElement: {\n    prototype: SVGFEBlendElement;\n    new(): SVGFEBlendElement;\n    readonly SVG_FEBLEND_MODE_COLOR: number;\n    readonly SVG_FEBLEND_MODE_COLOR_BURN: number;\n    readonly SVG_FEBLEND_MODE_COLOR_DODGE: number;\n    readonly SVG_FEBLEND_MODE_DARKEN: number;\n    readonly SVG_FEBLEND_MODE_DIFFERENCE: number;\n    readonly SVG_FEBLEND_MODE_EXCLUSION: number;\n    readonly SVG_FEBLEND_MODE_HARD_LIGHT: number;\n    readonly SVG_FEBLEND_MODE_HUE: number;\n    readonly SVG_FEBLEND_MODE_LIGHTEN: number;\n    readonly SVG_FEBLEND_MODE_LUMINOSITY: number;\n    readonly SVG_FEBLEND_MODE_MULTIPLY: number;\n    readonly SVG_FEBLEND_MODE_NORMAL: number;\n    readonly SVG_FEBLEND_MODE_OVERLAY: number;\n    readonly SVG_FEBLEND_MODE_SATURATION: number;\n    readonly SVG_FEBLEND_MODE_SCREEN: number;\n    readonly SVG_FEBLEND_MODE_SOFT_LIGHT: number;\n    readonly SVG_FEBLEND_MODE_UNKNOWN: number;\n};\n\n/** Corresponds to the <feColorMatrix> element. */\ninterface SVGFEColorMatrixElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly in1: SVGAnimatedString;\n    readonly type: SVGAnimatedEnumeration;\n    readonly values: SVGAnimatedNumberList;\n    readonly SVG_FECOLORMATRIX_TYPE_HUEROTATE: number;\n    readonly SVG_FECOLORMATRIX_TYPE_LUMINANCETOALPHA: number;\n    readonly SVG_FECOLORMATRIX_TYPE_MATRIX: number;\n    readonly SVG_FECOLORMATRIX_TYPE_SATURATE: number;\n    readonly SVG_FECOLORMATRIX_TYPE_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEColorMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEColorMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEColorMatrixElement: {\n    prototype: SVGFEColorMatrixElement;\n    new(): SVGFEColorMatrixElement;\n    readonly SVG_FECOLORMATRIX_TYPE_HUEROTATE: number;\n    readonly SVG_FECOLORMATRIX_TYPE_LUMINANCETOALPHA: number;\n    readonly SVG_FECOLORMATRIX_TYPE_MATRIX: number;\n    readonly SVG_FECOLORMATRIX_TYPE_SATURATE: number;\n    readonly SVG_FECOLORMATRIX_TYPE_UNKNOWN: number;\n};\n\n/** Corresponds to the <feComponentTransfer> element. */\ninterface SVGFEComponentTransferElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly in1: SVGAnimatedString;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEComponentTransferElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEComponentTransferElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEComponentTransferElement: {\n    prototype: SVGFEComponentTransferElement;\n    new(): SVGFEComponentTransferElement;\n};\n\n/** Corresponds to the <feComposite> element. */\ninterface SVGFECompositeElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly in1: SVGAnimatedString;\n    readonly in2: SVGAnimatedString;\n    readonly k1: SVGAnimatedNumber;\n    readonly k2: SVGAnimatedNumber;\n    readonly k3: SVGAnimatedNumber;\n    readonly k4: SVGAnimatedNumber;\n    readonly operator: SVGAnimatedEnumeration;\n    readonly SVG_FECOMPOSITE_OPERATOR_ARITHMETIC: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_ATOP: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_IN: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_OUT: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_OVER: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_UNKNOWN: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_XOR: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFECompositeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFECompositeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFECompositeElement: {\n    prototype: SVGFECompositeElement;\n    new(): SVGFECompositeElement;\n    readonly SVG_FECOMPOSITE_OPERATOR_ARITHMETIC: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_ATOP: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_IN: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_OUT: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_OVER: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_UNKNOWN: number;\n    readonly SVG_FECOMPOSITE_OPERATOR_XOR: number;\n};\n\n/** Corresponds to the <feConvolveMatrix> element. */\ninterface SVGFEConvolveMatrixElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly bias: SVGAnimatedNumber;\n    readonly divisor: SVGAnimatedNumber;\n    readonly edgeMode: SVGAnimatedEnumeration;\n    readonly in1: SVGAnimatedString;\n    readonly kernelMatrix: SVGAnimatedNumberList;\n    readonly kernelUnitLengthX: SVGAnimatedNumber;\n    readonly kernelUnitLengthY: SVGAnimatedNumber;\n    readonly orderX: SVGAnimatedInteger;\n    readonly orderY: SVGAnimatedInteger;\n    readonly preserveAlpha: SVGAnimatedBoolean;\n    readonly targetX: SVGAnimatedInteger;\n    readonly targetY: SVGAnimatedInteger;\n    readonly SVG_EDGEMODE_DUPLICATE: number;\n    readonly SVG_EDGEMODE_NONE: number;\n    readonly SVG_EDGEMODE_UNKNOWN: number;\n    readonly SVG_EDGEMODE_WRAP: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEConvolveMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEConvolveMatrixElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEConvolveMatrixElement: {\n    prototype: SVGFEConvolveMatrixElement;\n    new(): SVGFEConvolveMatrixElement;\n    readonly SVG_EDGEMODE_DUPLICATE: number;\n    readonly SVG_EDGEMODE_NONE: number;\n    readonly SVG_EDGEMODE_UNKNOWN: number;\n    readonly SVG_EDGEMODE_WRAP: number;\n};\n\n/** Corresponds to the <feDiffuseLighting> element. */\ninterface SVGFEDiffuseLightingElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly diffuseConstant: SVGAnimatedNumber;\n    readonly in1: SVGAnimatedString;\n    readonly kernelUnitLengthX: SVGAnimatedNumber;\n    readonly kernelUnitLengthY: SVGAnimatedNumber;\n    readonly surfaceScale: SVGAnimatedNumber;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDiffuseLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDiffuseLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEDiffuseLightingElement: {\n    prototype: SVGFEDiffuseLightingElement;\n    new(): SVGFEDiffuseLightingElement;\n};\n\n/** Corresponds to the <feDisplacementMap> element. */\ninterface SVGFEDisplacementMapElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly in1: SVGAnimatedString;\n    readonly in2: SVGAnimatedString;\n    readonly scale: SVGAnimatedNumber;\n    readonly xChannelSelector: SVGAnimatedEnumeration;\n    readonly yChannelSelector: SVGAnimatedEnumeration;\n    readonly SVG_CHANNEL_A: number;\n    readonly SVG_CHANNEL_B: number;\n    readonly SVG_CHANNEL_G: number;\n    readonly SVG_CHANNEL_R: number;\n    readonly SVG_CHANNEL_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDisplacementMapElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDisplacementMapElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEDisplacementMapElement: {\n    prototype: SVGFEDisplacementMapElement;\n    new(): SVGFEDisplacementMapElement;\n    readonly SVG_CHANNEL_A: number;\n    readonly SVG_CHANNEL_B: number;\n    readonly SVG_CHANNEL_G: number;\n    readonly SVG_CHANNEL_R: number;\n    readonly SVG_CHANNEL_UNKNOWN: number;\n};\n\n/** Corresponds to the <feDistantLight> element. */\ninterface SVGFEDistantLightElement extends SVGElement {\n    readonly azimuth: SVGAnimatedNumber;\n    readonly elevation: SVGAnimatedNumber;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDistantLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDistantLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEDistantLightElement: {\n    prototype: SVGFEDistantLightElement;\n    new(): SVGFEDistantLightElement;\n};\n\ninterface SVGFEDropShadowElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly dx: SVGAnimatedNumber;\n    readonly dy: SVGAnimatedNumber;\n    readonly in1: SVGAnimatedString;\n    readonly stdDeviationX: SVGAnimatedNumber;\n    readonly stdDeviationY: SVGAnimatedNumber;\n    setStdDeviation(stdDeviationX: number, stdDeviationY: number): void;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDropShadowElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEDropShadowElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEDropShadowElement: {\n    prototype: SVGFEDropShadowElement;\n    new(): SVGFEDropShadowElement;\n};\n\n/** Corresponds to the <feFlood> element. */\ninterface SVGFEFloodElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFloodElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFloodElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEFloodElement: {\n    prototype: SVGFEFloodElement;\n    new(): SVGFEFloodElement;\n};\n\n/** Corresponds to the <feFuncA> element. */\ninterface SVGFEFuncAElement extends SVGComponentTransferFunctionElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncAElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEFuncAElement: {\n    prototype: SVGFEFuncAElement;\n    new(): SVGFEFuncAElement;\n};\n\n/** Corresponds to the <feFuncB> element. */\ninterface SVGFEFuncBElement extends SVGComponentTransferFunctionElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncBElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncBElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEFuncBElement: {\n    prototype: SVGFEFuncBElement;\n    new(): SVGFEFuncBElement;\n};\n\n/** Corresponds to the <feFuncG> element. */\ninterface SVGFEFuncGElement extends SVGComponentTransferFunctionElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEFuncGElement: {\n    prototype: SVGFEFuncGElement;\n    new(): SVGFEFuncGElement;\n};\n\n/** Corresponds to the <feFuncR> element. */\ninterface SVGFEFuncRElement extends SVGComponentTransferFunctionElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncRElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEFuncRElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEFuncRElement: {\n    prototype: SVGFEFuncRElement;\n    new(): SVGFEFuncRElement;\n};\n\n/** Corresponds to the <feGaussianBlur> element. */\ninterface SVGFEGaussianBlurElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly in1: SVGAnimatedString;\n    readonly stdDeviationX: SVGAnimatedNumber;\n    readonly stdDeviationY: SVGAnimatedNumber;\n    setStdDeviation(stdDeviationX: number, stdDeviationY: number): void;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEGaussianBlurElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEGaussianBlurElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEGaussianBlurElement: {\n    prototype: SVGFEGaussianBlurElement;\n    new(): SVGFEGaussianBlurElement;\n};\n\n/** Corresponds to the <feImage> element. */\ninterface SVGFEImageElement extends SVGElement, SVGFilterPrimitiveStandardAttributes, SVGURIReference {\n    readonly preserveAspectRatio: SVGAnimatedPreserveAspectRatio;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEImageElement: {\n    prototype: SVGFEImageElement;\n    new(): SVGFEImageElement;\n};\n\n/** Corresponds to the <feMerge> element. */\ninterface SVGFEMergeElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEMergeElement: {\n    prototype: SVGFEMergeElement;\n    new(): SVGFEMergeElement;\n};\n\n/** Corresponds to the <feMergeNode> element. */\ninterface SVGFEMergeNodeElement extends SVGElement {\n    readonly in1: SVGAnimatedString;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeNodeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMergeNodeElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEMergeNodeElement: {\n    prototype: SVGFEMergeNodeElement;\n    new(): SVGFEMergeNodeElement;\n};\n\n/** Corresponds to the <feMorphology> element. */\ninterface SVGFEMorphologyElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly in1: SVGAnimatedString;\n    readonly operator: SVGAnimatedEnumeration;\n    readonly radiusX: SVGAnimatedNumber;\n    readonly radiusY: SVGAnimatedNumber;\n    readonly SVG_MORPHOLOGY_OPERATOR_DILATE: number;\n    readonly SVG_MORPHOLOGY_OPERATOR_ERODE: number;\n    readonly SVG_MORPHOLOGY_OPERATOR_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMorphologyElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEMorphologyElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEMorphologyElement: {\n    prototype: SVGFEMorphologyElement;\n    new(): SVGFEMorphologyElement;\n    readonly SVG_MORPHOLOGY_OPERATOR_DILATE: number;\n    readonly SVG_MORPHOLOGY_OPERATOR_ERODE: number;\n    readonly SVG_MORPHOLOGY_OPERATOR_UNKNOWN: number;\n};\n\n/** Corresponds to the <feOffset> element. */\ninterface SVGFEOffsetElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly dx: SVGAnimatedNumber;\n    readonly dy: SVGAnimatedNumber;\n    readonly in1: SVGAnimatedString;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEOffsetElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEOffsetElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEOffsetElement: {\n    prototype: SVGFEOffsetElement;\n    new(): SVGFEOffsetElement;\n};\n\n/** Corresponds to the <fePointLight> element. */\ninterface SVGFEPointLightElement extends SVGElement {\n    readonly x: SVGAnimatedNumber;\n    readonly y: SVGAnimatedNumber;\n    readonly z: SVGAnimatedNumber;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEPointLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFEPointLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFEPointLightElement: {\n    prototype: SVGFEPointLightElement;\n    new(): SVGFEPointLightElement;\n};\n\n/** Corresponds to the <feSpecularLighting> element. */\ninterface SVGFESpecularLightingElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly in1: SVGAnimatedString;\n    readonly kernelUnitLengthX: SVGAnimatedNumber;\n    readonly kernelUnitLengthY: SVGAnimatedNumber;\n    readonly specularConstant: SVGAnimatedNumber;\n    readonly specularExponent: SVGAnimatedNumber;\n    readonly surfaceScale: SVGAnimatedNumber;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpecularLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpecularLightingElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFESpecularLightingElement: {\n    prototype: SVGFESpecularLightingElement;\n    new(): SVGFESpecularLightingElement;\n};\n\n/** Corresponds to the <feSpotLight> element. */\ninterface SVGFESpotLightElement extends SVGElement {\n    readonly limitingConeAngle: SVGAnimatedNumber;\n    readonly pointsAtX: SVGAnimatedNumber;\n    readonly pointsAtY: SVGAnimatedNumber;\n    readonly pointsAtZ: SVGAnimatedNumber;\n    readonly specularExponent: SVGAnimatedNumber;\n    readonly x: SVGAnimatedNumber;\n    readonly y: SVGAnimatedNumber;\n    readonly z: SVGAnimatedNumber;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpotLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFESpotLightElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFESpotLightElement: {\n    prototype: SVGFESpotLightElement;\n    new(): SVGFESpotLightElement;\n};\n\n/** Corresponds to the <feTile> element. */\ninterface SVGFETileElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly in1: SVGAnimatedString;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETileElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETileElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFETileElement: {\n    prototype: SVGFETileElement;\n    new(): SVGFETileElement;\n};\n\n/** Corresponds to the <feTurbulence> element. */\ninterface SVGFETurbulenceElement extends SVGElement, SVGFilterPrimitiveStandardAttributes {\n    readonly baseFrequencyX: SVGAnimatedNumber;\n    readonly baseFrequencyY: SVGAnimatedNumber;\n    readonly numOctaves: SVGAnimatedInteger;\n    readonly seed: SVGAnimatedNumber;\n    readonly stitchTiles: SVGAnimatedEnumeration;\n    readonly type: SVGAnimatedEnumeration;\n    readonly SVG_STITCHTYPE_NOSTITCH: number;\n    readonly SVG_STITCHTYPE_STITCH: number;\n    readonly SVG_STITCHTYPE_UNKNOWN: number;\n    readonly SVG_TURBULENCE_TYPE_FRACTALNOISE: number;\n    readonly SVG_TURBULENCE_TYPE_TURBULENCE: number;\n    readonly SVG_TURBULENCE_TYPE_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETurbulenceElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFETurbulenceElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFETurbulenceElement: {\n    prototype: SVGFETurbulenceElement;\n    new(): SVGFETurbulenceElement;\n    readonly SVG_STITCHTYPE_NOSTITCH: number;\n    readonly SVG_STITCHTYPE_STITCH: number;\n    readonly SVG_STITCHTYPE_UNKNOWN: number;\n    readonly SVG_TURBULENCE_TYPE_FRACTALNOISE: number;\n    readonly SVG_TURBULENCE_TYPE_TURBULENCE: number;\n    readonly SVG_TURBULENCE_TYPE_UNKNOWN: number;\n};\n\n/** Provides access to the properties of <filter> elements, as well as methods to manipulate them. */\ninterface SVGFilterElement extends SVGElement, SVGURIReference {\n    readonly filterUnits: SVGAnimatedEnumeration;\n    readonly height: SVGAnimatedLength;\n    readonly primitiveUnits: SVGAnimatedEnumeration;\n    readonly width: SVGAnimatedLength;\n    readonly x: SVGAnimatedLength;\n    readonly y: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFilterElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGFilterElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGFilterElement: {\n    prototype: SVGFilterElement;\n    new(): SVGFilterElement;\n};\n\ninterface SVGFilterPrimitiveStandardAttributes {\n    readonly height: SVGAnimatedLength;\n    readonly result: SVGAnimatedString;\n    readonly width: SVGAnimatedLength;\n    readonly x: SVGAnimatedLength;\n    readonly y: SVGAnimatedLength;\n}\n\ninterface SVGFitToViewBox {\n    readonly preserveAspectRatio: SVGAnimatedPreserveAspectRatio;\n    readonly viewBox: SVGAnimatedRect;\n}\n\n/** Provides access to the properties of <foreignObject> elements, as well as methods to manipulate them. */\ninterface SVGForeignObjectElement extends SVGGraphicsElement {\n    readonly height: SVGAnimatedLength;\n    readonly width: SVGAnimatedLength;\n    readonly x: SVGAnimatedLength;\n    readonly y: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGForeignObjectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGForeignObjectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGForeignObjectElement: {\n    prototype: SVGForeignObjectElement;\n    new(): SVGForeignObjectElement;\n};\n\n/** Corresponds to the <g> element. */\ninterface SVGGElement extends SVGGraphicsElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGGElement: {\n    prototype: SVGGElement;\n    new(): SVGGElement;\n};\n\ninterface SVGGeometryElement extends SVGGraphicsElement {\n    readonly pathLength: SVGAnimatedNumber;\n    getPointAtLength(distance: number): DOMPoint;\n    getTotalLength(): number;\n    isPointInFill(point?: DOMPointInit): boolean;\n    isPointInStroke(point?: DOMPointInit): boolean;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGeometryElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGeometryElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGGeometryElement: {\n    prototype: SVGGeometryElement;\n    new(): SVGGeometryElement;\n};\n\n/** The SVGGradient interface is a base interface used by SVGLinearGradientElement and SVGRadialGradientElement. */\ninterface SVGGradientElement extends SVGElement, SVGURIReference {\n    readonly gradientTransform: SVGAnimatedTransformList;\n    readonly gradientUnits: SVGAnimatedEnumeration;\n    readonly spreadMethod: SVGAnimatedEnumeration;\n    readonly SVG_SPREADMETHOD_PAD: number;\n    readonly SVG_SPREADMETHOD_REFLECT: number;\n    readonly SVG_SPREADMETHOD_REPEAT: number;\n    readonly SVG_SPREADMETHOD_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGGradientElement: {\n    prototype: SVGGradientElement;\n    new(): SVGGradientElement;\n    readonly SVG_SPREADMETHOD_PAD: number;\n    readonly SVG_SPREADMETHOD_REFLECT: number;\n    readonly SVG_SPREADMETHOD_REPEAT: number;\n    readonly SVG_SPREADMETHOD_UNKNOWN: number;\n};\n\n/** SVG elements whose primary purpose is to directly render graphics into a group. */\ninterface SVGGraphicsElement extends SVGElement, SVGTests {\n    readonly transform: SVGAnimatedTransformList;\n    getBBox(options?: SVGBoundingBoxOptions): DOMRect;\n    getCTM(): DOMMatrix | null;\n    getScreenCTM(): DOMMatrix | null;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGraphicsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGGraphicsElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGGraphicsElement: {\n    prototype: SVGGraphicsElement;\n    new(): SVGGraphicsElement;\n};\n\n/** Corresponds to the <image> element. */\ninterface SVGImageElement extends SVGGraphicsElement, SVGURIReference {\n    readonly height: SVGAnimatedLength;\n    readonly preserveAspectRatio: SVGAnimatedPreserveAspectRatio;\n    readonly width: SVGAnimatedLength;\n    readonly x: SVGAnimatedLength;\n    readonly y: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGImageElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGImageElement: {\n    prototype: SVGImageElement;\n    new(): SVGImageElement;\n};\n\n/** Correspond to the <length> basic data type. */\ninterface SVGLength {\n    readonly unitType: number;\n    value: number;\n    valueAsString: string;\n    valueInSpecifiedUnits: number;\n    convertToSpecifiedUnits(unitType: number): void;\n    newValueSpecifiedUnits(unitType: number, valueInSpecifiedUnits: number): void;\n    readonly SVG_LENGTHTYPE_CM: number;\n    readonly SVG_LENGTHTYPE_EMS: number;\n    readonly SVG_LENGTHTYPE_EXS: number;\n    readonly SVG_LENGTHTYPE_IN: number;\n    readonly SVG_LENGTHTYPE_MM: number;\n    readonly SVG_LENGTHTYPE_NUMBER: number;\n    readonly SVG_LENGTHTYPE_PC: number;\n    readonly SVG_LENGTHTYPE_PERCENTAGE: number;\n    readonly SVG_LENGTHTYPE_PT: number;\n    readonly SVG_LENGTHTYPE_PX: number;\n    readonly SVG_LENGTHTYPE_UNKNOWN: number;\n}\n\ndeclare var SVGLength: {\n    prototype: SVGLength;\n    new(): SVGLength;\n    readonly SVG_LENGTHTYPE_CM: number;\n    readonly SVG_LENGTHTYPE_EMS: number;\n    readonly SVG_LENGTHTYPE_EXS: number;\n    readonly SVG_LENGTHTYPE_IN: number;\n    readonly SVG_LENGTHTYPE_MM: number;\n    readonly SVG_LENGTHTYPE_NUMBER: number;\n    readonly SVG_LENGTHTYPE_PC: number;\n    readonly SVG_LENGTHTYPE_PERCENTAGE: number;\n    readonly SVG_LENGTHTYPE_PT: number;\n    readonly SVG_LENGTHTYPE_PX: number;\n    readonly SVG_LENGTHTYPE_UNKNOWN: number;\n};\n\n/** The SVGLengthList defines a list of SVGLength objects. */\ninterface SVGLengthList {\n    readonly length: number;\n    readonly numberOfItems: number;\n    appendItem(newItem: SVGLength): SVGLength;\n    clear(): void;\n    getItem(index: number): SVGLength;\n    initialize(newItem: SVGLength): SVGLength;\n    insertItemBefore(newItem: SVGLength, index: number): SVGLength;\n    removeItem(index: number): SVGLength;\n    replaceItem(newItem: SVGLength, index: number): SVGLength;\n    [index: number]: SVGLength;\n}\n\ndeclare var SVGLengthList: {\n    prototype: SVGLengthList;\n    new(): SVGLengthList;\n};\n\n/** Provides access to the properties of <line> elements, as well as methods to manipulate them. */\ninterface SVGLineElement extends SVGGeometryElement {\n    readonly x1: SVGAnimatedLength;\n    readonly x2: SVGAnimatedLength;\n    readonly y1: SVGAnimatedLength;\n    readonly y2: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGLineElement: {\n    prototype: SVGLineElement;\n    new(): SVGLineElement;\n};\n\n/** Corresponds to the <linearGradient> element. */\ninterface SVGLinearGradientElement extends SVGGradientElement {\n    readonly x1: SVGAnimatedLength;\n    readonly x2: SVGAnimatedLength;\n    readonly y1: SVGAnimatedLength;\n    readonly y2: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLinearGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGLinearGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGLinearGradientElement: {\n    prototype: SVGLinearGradientElement;\n    new(): SVGLinearGradientElement;\n};\n\ninterface SVGMPathElement extends SVGElement, SVGURIReference {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGMPathElement: {\n    prototype: SVGMPathElement;\n    new(): SVGMPathElement;\n};\n\ninterface SVGMarkerElement extends SVGElement, SVGFitToViewBox {\n    readonly markerHeight: SVGAnimatedLength;\n    readonly markerUnits: SVGAnimatedEnumeration;\n    readonly markerWidth: SVGAnimatedLength;\n    readonly orientAngle: SVGAnimatedAngle;\n    readonly orientType: SVGAnimatedEnumeration;\n    readonly refX: SVGAnimatedLength;\n    readonly refY: SVGAnimatedLength;\n    setOrientToAngle(angle: SVGAngle): void;\n    setOrientToAuto(): void;\n    readonly SVG_MARKERUNITS_STROKEWIDTH: number;\n    readonly SVG_MARKERUNITS_UNKNOWN: number;\n    readonly SVG_MARKERUNITS_USERSPACEONUSE: number;\n    readonly SVG_MARKER_ORIENT_ANGLE: number;\n    readonly SVG_MARKER_ORIENT_AUTO: number;\n    readonly SVG_MARKER_ORIENT_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMarkerElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMarkerElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGMarkerElement: {\n    prototype: SVGMarkerElement;\n    new(): SVGMarkerElement;\n    readonly SVG_MARKERUNITS_STROKEWIDTH: number;\n    readonly SVG_MARKERUNITS_UNKNOWN: number;\n    readonly SVG_MARKERUNITS_USERSPACEONUSE: number;\n    readonly SVG_MARKER_ORIENT_ANGLE: number;\n    readonly SVG_MARKER_ORIENT_AUTO: number;\n    readonly SVG_MARKER_ORIENT_UNKNOWN: number;\n};\n\n/** Provides access to the properties of <mask> elements, as well as methods to manipulate them. */\ninterface SVGMaskElement extends SVGElement {\n    readonly height: SVGAnimatedLength;\n    readonly maskContentUnits: SVGAnimatedEnumeration;\n    readonly maskUnits: SVGAnimatedEnumeration;\n    readonly width: SVGAnimatedLength;\n    readonly x: SVGAnimatedLength;\n    readonly y: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMaskElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMaskElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGMaskElement: {\n    prototype: SVGMaskElement;\n    new(): SVGMaskElement;\n};\n\n/** Corresponds to the <metadata> element. */\ninterface SVGMetadataElement extends SVGElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMetadataElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGMetadataElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGMetadataElement: {\n    prototype: SVGMetadataElement;\n    new(): SVGMetadataElement;\n};\n\n/** Corresponds to the <number> basic data type. */\ninterface SVGNumber {\n    value: number;\n}\n\ndeclare var SVGNumber: {\n    prototype: SVGNumber;\n    new(): SVGNumber;\n};\n\n/** The SVGNumberList defines a list of SVGNumber objects. */\ninterface SVGNumberList {\n    readonly length: number;\n    readonly numberOfItems: number;\n    appendItem(newItem: SVGNumber): SVGNumber;\n    clear(): void;\n    getItem(index: number): SVGNumber;\n    initialize(newItem: SVGNumber): SVGNumber;\n    insertItemBefore(newItem: SVGNumber, index: number): SVGNumber;\n    removeItem(index: number): SVGNumber;\n    replaceItem(newItem: SVGNumber, index: number): SVGNumber;\n    [index: number]: SVGNumber;\n}\n\ndeclare var SVGNumberList: {\n    prototype: SVGNumberList;\n    new(): SVGNumberList;\n};\n\n/** Corresponds to the <path> element. */\ninterface SVGPathElement extends SVGGeometryElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGPathElement: {\n    prototype: SVGPathElement;\n    new(): SVGPathElement;\n};\n\n/** Corresponds to the <pattern> element. */\ninterface SVGPatternElement extends SVGElement, SVGFitToViewBox, SVGURIReference {\n    readonly height: SVGAnimatedLength;\n    readonly patternContentUnits: SVGAnimatedEnumeration;\n    readonly patternTransform: SVGAnimatedTransformList;\n    readonly patternUnits: SVGAnimatedEnumeration;\n    readonly width: SVGAnimatedLength;\n    readonly x: SVGAnimatedLength;\n    readonly y: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPatternElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPatternElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGPatternElement: {\n    prototype: SVGPatternElement;\n    new(): SVGPatternElement;\n};\n\ninterface SVGPointList {\n    readonly length: number;\n    readonly numberOfItems: number;\n    appendItem(newItem: DOMPoint): DOMPoint;\n    clear(): void;\n    getItem(index: number): DOMPoint;\n    initialize(newItem: DOMPoint): DOMPoint;\n    insertItemBefore(newItem: DOMPoint, index: number): DOMPoint;\n    removeItem(index: number): DOMPoint;\n    replaceItem(newItem: DOMPoint, index: number): DOMPoint;\n    [index: number]: DOMPoint;\n}\n\ndeclare var SVGPointList: {\n    prototype: SVGPointList;\n    new(): SVGPointList;\n};\n\n/** Provides access to the properties of <polygon> elements, as well as methods to manipulate them. */\ninterface SVGPolygonElement extends SVGGeometryElement, SVGAnimatedPoints {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolygonElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolygonElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGPolygonElement: {\n    prototype: SVGPolygonElement;\n    new(): SVGPolygonElement;\n};\n\n/** Provides access to the properties of <polyline> elements, as well as methods to manipulate them. */\ninterface SVGPolylineElement extends SVGGeometryElement, SVGAnimatedPoints {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolylineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGPolylineElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGPolylineElement: {\n    prototype: SVGPolylineElement;\n    new(): SVGPolylineElement;\n};\n\n/** Corresponds to the preserveAspectRatio attribute, which is available for some of SVG\'s elements. */\ninterface SVGPreserveAspectRatio {\n    align: number;\n    meetOrSlice: number;\n    readonly SVG_MEETORSLICE_MEET: number;\n    readonly SVG_MEETORSLICE_SLICE: number;\n    readonly SVG_MEETORSLICE_UNKNOWN: number;\n    readonly SVG_PRESERVEASPECTRATIO_NONE: number;\n    readonly SVG_PRESERVEASPECTRATIO_UNKNOWN: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMAX: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMID: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMIN: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMAX: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMID: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMIN: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMAX: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMID: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMIN: number;\n}\n\ndeclare var SVGPreserveAspectRatio: {\n    prototype: SVGPreserveAspectRatio;\n    new(): SVGPreserveAspectRatio;\n    readonly SVG_MEETORSLICE_MEET: number;\n    readonly SVG_MEETORSLICE_SLICE: number;\n    readonly SVG_MEETORSLICE_UNKNOWN: number;\n    readonly SVG_PRESERVEASPECTRATIO_NONE: number;\n    readonly SVG_PRESERVEASPECTRATIO_UNKNOWN: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMAX: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMID: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMAXYMIN: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMAX: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMID: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMIDYMIN: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMAX: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMID: number;\n    readonly SVG_PRESERVEASPECTRATIO_XMINYMIN: number;\n};\n\n/** Corresponds to the <RadialGradient> element. */\ninterface SVGRadialGradientElement extends SVGGradientElement {\n    readonly cx: SVGAnimatedLength;\n    readonly cy: SVGAnimatedLength;\n    readonly fr: SVGAnimatedLength;\n    readonly fx: SVGAnimatedLength;\n    readonly fy: SVGAnimatedLength;\n    readonly r: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRadialGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRadialGradientElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGRadialGradientElement: {\n    prototype: SVGRadialGradientElement;\n    new(): SVGRadialGradientElement;\n};\n\n/** Provides access to the properties of <rect> elements, as well as methods to manipulate them. */\ninterface SVGRectElement extends SVGGeometryElement {\n    readonly height: SVGAnimatedLength;\n    readonly rx: SVGAnimatedLength;\n    readonly ry: SVGAnimatedLength;\n    readonly width: SVGAnimatedLength;\n    readonly x: SVGAnimatedLength;\n    readonly y: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGRectElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGRectElement: {\n    prototype: SVGRectElement;\n    new(): SVGRectElement;\n};\n\ninterface SVGSVGElementEventMap extends SVGElementEventMap, WindowEventHandlersEventMap {\n}\n\n/** Provides access to the properties of <svg> elements, as well as methods to manipulate them. This interface contains also various miscellaneous commonly-used utility methods, such as matrix operations and the ability to control the time of redraw on visual rendering devices. */\ninterface SVGSVGElement extends SVGGraphicsElement, SVGFitToViewBox, WindowEventHandlers {\n    currentScale: number;\n    readonly currentTranslate: DOMPointReadOnly;\n    readonly height: SVGAnimatedLength;\n    readonly width: SVGAnimatedLength;\n    readonly x: SVGAnimatedLength;\n    readonly y: SVGAnimatedLength;\n    animationsPaused(): boolean;\n    checkEnclosure(element: SVGElement, rect: DOMRectReadOnly): boolean;\n    checkIntersection(element: SVGElement, rect: DOMRectReadOnly): boolean;\n    createSVGAngle(): SVGAngle;\n    createSVGLength(): SVGLength;\n    createSVGMatrix(): DOMMatrix;\n    createSVGNumber(): SVGNumber;\n    createSVGPoint(): DOMPoint;\n    createSVGRect(): DOMRect;\n    createSVGTransform(): SVGTransform;\n    createSVGTransformFromMatrix(matrix?: DOMMatrix2DInit): SVGTransform;\n    deselectAll(): void;\n    /** @deprecated */\n    forceRedraw(): void;\n    getCurrentTime(): number;\n    getElementById(elementId: string): Element;\n    getEnclosureList(rect: DOMRectReadOnly, referenceElement: SVGElement | null): NodeListOf<SVGCircleElement | SVGEllipseElement | SVGImageElement | SVGLineElement | SVGPathElement | SVGPolygonElement | SVGPolylineElement | SVGRectElement | SVGTextElement | SVGUseElement>;\n    getIntersectionList(rect: DOMRectReadOnly, referenceElement: SVGElement | null): NodeListOf<SVGCircleElement | SVGEllipseElement | SVGImageElement | SVGLineElement | SVGPathElement | SVGPolygonElement | SVGPolylineElement | SVGRectElement | SVGTextElement | SVGUseElement>;\n    pauseAnimations(): void;\n    setCurrentTime(seconds: number): void;\n    /** @deprecated */\n    suspendRedraw(maxWaitMilliseconds: number): number;\n    unpauseAnimations(): void;\n    /** @deprecated */\n    unsuspendRedraw(suspendHandleID: number): void;\n    /** @deprecated */\n    unsuspendRedrawAll(): void;\n    addEventListener<K extends keyof SVGSVGElementEventMap>(type: K, listener: (this: SVGSVGElement, ev: SVGSVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGSVGElementEventMap>(type: K, listener: (this: SVGSVGElement, ev: SVGSVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGSVGElement: {\n    prototype: SVGSVGElement;\n    new(): SVGSVGElement;\n};\n\n/** Corresponds to the SVG <script> element. */\ninterface SVGScriptElement extends SVGElement, SVGURIReference {\n    type: string;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGScriptElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGScriptElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGScriptElement: {\n    prototype: SVGScriptElement;\n    new(): SVGScriptElement;\n};\n\ninterface SVGSetElement extends SVGAnimationElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSetElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSetElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGSetElement: {\n    prototype: SVGSetElement;\n    new(): SVGSetElement;\n};\n\n/** Corresponds to the <stop> element. */\ninterface SVGStopElement extends SVGElement {\n    readonly offset: SVGAnimatedNumber;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStopElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStopElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGStopElement: {\n    prototype: SVGStopElement;\n    new(): SVGStopElement;\n};\n\n/** The SVGStringList defines a list of DOMString objects. */\ninterface SVGStringList {\n    readonly length: number;\n    readonly numberOfItems: number;\n    appendItem(newItem: string): string;\n    clear(): void;\n    getItem(index: number): string;\n    initialize(newItem: string): string;\n    insertItemBefore(newItem: string, index: number): string;\n    removeItem(index: number): string;\n    replaceItem(newItem: string, index: number): string;\n    [index: number]: string;\n}\n\ndeclare var SVGStringList: {\n    prototype: SVGStringList;\n    new(): SVGStringList;\n};\n\n/** Corresponds to the SVG <style> element. */\ninterface SVGStyleElement extends SVGElement, LinkStyle {\n    disabled: boolean;\n    media: string;\n    title: string;\n    type: string;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStyleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGStyleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGStyleElement: {\n    prototype: SVGStyleElement;\n    new(): SVGStyleElement;\n};\n\n/** Corresponds to the <switch> element. */\ninterface SVGSwitchElement extends SVGGraphicsElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSwitchElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSwitchElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGSwitchElement: {\n    prototype: SVGSwitchElement;\n    new(): SVGSwitchElement;\n};\n\n/** Corresponds to the <symbol> element. */\ninterface SVGSymbolElement extends SVGElement, SVGFitToViewBox {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSymbolElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGSymbolElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGSymbolElement: {\n    prototype: SVGSymbolElement;\n    new(): SVGSymbolElement;\n};\n\n/** A <tspan> element. */\ninterface SVGTSpanElement extends SVGTextPositioningElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTSpanElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTSpanElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGTSpanElement: {\n    prototype: SVGTSpanElement;\n    new(): SVGTSpanElement;\n};\n\ninterface SVGTests {\n    readonly requiredExtensions: SVGStringList;\n    readonly systemLanguage: SVGStringList;\n}\n\n/** Implemented by elements that support rendering child text content. It is inherited by various text-related interfaces, such as SVGTextElement, SVGTSpanElement, SVGTRefElement, SVGAltGlyphElement and SVGTextPathElement. */\ninterface SVGTextContentElement extends SVGGraphicsElement {\n    readonly lengthAdjust: SVGAnimatedEnumeration;\n    readonly textLength: SVGAnimatedLength;\n    getCharNumAtPosition(point?: DOMPointInit): number;\n    getComputedTextLength(): number;\n    getEndPositionOfChar(charnum: number): DOMPoint;\n    getExtentOfChar(charnum: number): DOMRect;\n    getNumberOfChars(): number;\n    getRotationOfChar(charnum: number): number;\n    getStartPositionOfChar(charnum: number): DOMPoint;\n    getSubStringLength(charnum: number, nchars: number): number;\n    /** @deprecated */\n    selectSubString(charnum: number, nchars: number): void;\n    readonly LENGTHADJUST_SPACING: number;\n    readonly LENGTHADJUST_SPACINGANDGLYPHS: number;\n    readonly LENGTHADJUST_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextContentElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextContentElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGTextContentElement: {\n    prototype: SVGTextContentElement;\n    new(): SVGTextContentElement;\n    readonly LENGTHADJUST_SPACING: number;\n    readonly LENGTHADJUST_SPACINGANDGLYPHS: number;\n    readonly LENGTHADJUST_UNKNOWN: number;\n};\n\n/** Corresponds to the <text> elements. */\ninterface SVGTextElement extends SVGTextPositioningElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGTextElement: {\n    prototype: SVGTextElement;\n    new(): SVGTextElement;\n};\n\n/** Corresponds to the <textPath> element. */\ninterface SVGTextPathElement extends SVGTextContentElement, SVGURIReference {\n    readonly method: SVGAnimatedEnumeration;\n    readonly spacing: SVGAnimatedEnumeration;\n    readonly startOffset: SVGAnimatedLength;\n    readonly TEXTPATH_METHODTYPE_ALIGN: number;\n    readonly TEXTPATH_METHODTYPE_STRETCH: number;\n    readonly TEXTPATH_METHODTYPE_UNKNOWN: number;\n    readonly TEXTPATH_SPACINGTYPE_AUTO: number;\n    readonly TEXTPATH_SPACINGTYPE_EXACT: number;\n    readonly TEXTPATH_SPACINGTYPE_UNKNOWN: number;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPathElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGTextPathElement: {\n    prototype: SVGTextPathElement;\n    new(): SVGTextPathElement;\n    readonly TEXTPATH_METHODTYPE_ALIGN: number;\n    readonly TEXTPATH_METHODTYPE_STRETCH: number;\n    readonly TEXTPATH_METHODTYPE_UNKNOWN: number;\n    readonly TEXTPATH_SPACINGTYPE_AUTO: number;\n    readonly TEXTPATH_SPACINGTYPE_EXACT: number;\n    readonly TEXTPATH_SPACINGTYPE_UNKNOWN: number;\n};\n\n/** Implemented by elements that support attributes that position individual text glyphs. It is inherited by SVGTextElement, SVGTSpanElement, SVGTRefElement and SVGAltGlyphElement. */\ninterface SVGTextPositioningElement extends SVGTextContentElement {\n    readonly dx: SVGAnimatedLengthList;\n    readonly dy: SVGAnimatedLengthList;\n    readonly rotate: SVGAnimatedNumberList;\n    readonly x: SVGAnimatedLengthList;\n    readonly y: SVGAnimatedLengthList;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPositioningElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTextPositioningElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGTextPositioningElement: {\n    prototype: SVGTextPositioningElement;\n    new(): SVGTextPositioningElement;\n};\n\n/** Corresponds to the <title> element. */\ninterface SVGTitleElement extends SVGElement {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTitleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGTitleElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGTitleElement: {\n    prototype: SVGTitleElement;\n    new(): SVGTitleElement;\n};\n\n/** SVGTransform is the interface for one of the component transformations within an SVGTransformList; thus, an SVGTransform object corresponds to a single component (e.g., scale(…) or matrix(…)) within a transform attribute. */\ninterface SVGTransform {\n    readonly angle: number;\n    readonly matrix: DOMMatrix;\n    readonly type: number;\n    setMatrix(matrix?: DOMMatrix2DInit): void;\n    setRotate(angle: number, cx: number, cy: number): void;\n    setScale(sx: number, sy: number): void;\n    setSkewX(angle: number): void;\n    setSkewY(angle: number): void;\n    setTranslate(tx: number, ty: number): void;\n    readonly SVG_TRANSFORM_MATRIX: number;\n    readonly SVG_TRANSFORM_ROTATE: number;\n    readonly SVG_TRANSFORM_SCALE: number;\n    readonly SVG_TRANSFORM_SKEWX: number;\n    readonly SVG_TRANSFORM_SKEWY: number;\n    readonly SVG_TRANSFORM_TRANSLATE: number;\n    readonly SVG_TRANSFORM_UNKNOWN: number;\n}\n\ndeclare var SVGTransform: {\n    prototype: SVGTransform;\n    new(): SVGTransform;\n    readonly SVG_TRANSFORM_MATRIX: number;\n    readonly SVG_TRANSFORM_ROTATE: number;\n    readonly SVG_TRANSFORM_SCALE: number;\n    readonly SVG_TRANSFORM_SKEWX: number;\n    readonly SVG_TRANSFORM_SKEWY: number;\n    readonly SVG_TRANSFORM_TRANSLATE: number;\n    readonly SVG_TRANSFORM_UNKNOWN: number;\n};\n\n/** The SVGTransformList defines a list of SVGTransform objects. */\ninterface SVGTransformList {\n    readonly length: number;\n    readonly numberOfItems: number;\n    appendItem(newItem: SVGTransform): SVGTransform;\n    clear(): void;\n    consolidate(): SVGTransform | null;\n    createSVGTransformFromMatrix(matrix?: DOMMatrix2DInit): SVGTransform;\n    getItem(index: number): SVGTransform;\n    initialize(newItem: SVGTransform): SVGTransform;\n    insertItemBefore(newItem: SVGTransform, index: number): SVGTransform;\n    removeItem(index: number): SVGTransform;\n    replaceItem(newItem: SVGTransform, index: number): SVGTransform;\n    [index: number]: SVGTransform;\n}\n\ndeclare var SVGTransformList: {\n    prototype: SVGTransformList;\n    new(): SVGTransformList;\n};\n\ninterface SVGURIReference {\n    readonly href: SVGAnimatedString;\n}\n\n/** A commonly used set of constants used for reflecting gradientUnits, patternContentUnits and other similar attributes. */\ninterface SVGUnitTypes {\n    readonly SVG_UNIT_TYPE_OBJECTBOUNDINGBOX: number;\n    readonly SVG_UNIT_TYPE_UNKNOWN: number;\n    readonly SVG_UNIT_TYPE_USERSPACEONUSE: number;\n}\n\ndeclare var SVGUnitTypes: {\n    prototype: SVGUnitTypes;\n    new(): SVGUnitTypes;\n    readonly SVG_UNIT_TYPE_OBJECTBOUNDINGBOX: number;\n    readonly SVG_UNIT_TYPE_UNKNOWN: number;\n    readonly SVG_UNIT_TYPE_USERSPACEONUSE: number;\n};\n\n/** Corresponds to the <use> element. */\ninterface SVGUseElement extends SVGGraphicsElement, SVGURIReference {\n    readonly height: SVGAnimatedLength;\n    readonly width: SVGAnimatedLength;\n    readonly x: SVGAnimatedLength;\n    readonly y: SVGAnimatedLength;\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGUseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGUseElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGUseElement: {\n    prototype: SVGUseElement;\n    new(): SVGUseElement;\n};\n\n/** Provides access to the properties of <view> elements, as well as methods to manipulate them. */\ninterface SVGViewElement extends SVGElement, SVGFitToViewBox {\n    addEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGViewElement, ev: SVGElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SVGElementEventMap>(type: K, listener: (this: SVGViewElement, ev: SVGElementEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SVGViewElement: {\n    prototype: SVGViewElement;\n    new(): SVGViewElement;\n};\n\n/** A screen, usually the one on which the current window is being rendered, and is obtained using window.screen. */\ninterface Screen {\n    readonly availHeight: number;\n    readonly availWidth: number;\n    readonly colorDepth: number;\n    readonly height: number;\n    readonly orientation: ScreenOrientation;\n    readonly pixelDepth: number;\n    readonly width: number;\n}\n\ndeclare var Screen: {\n    prototype: Screen;\n    new(): Screen;\n};\n\ninterface ScreenOrientationEventMap {\n    "change": Event;\n}\n\ninterface ScreenOrientation extends EventTarget {\n    readonly angle: number;\n    onchange: ((this: ScreenOrientation, ev: Event) => any) | null;\n    readonly type: OrientationType;\n    lock(orientation: OrientationLockType): Promise<void>;\n    unlock(): void;\n    addEventListener<K extends keyof ScreenOrientationEventMap>(type: K, listener: (this: ScreenOrientation, ev: ScreenOrientationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ScreenOrientationEventMap>(type: K, listener: (this: ScreenOrientation, ev: ScreenOrientationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var ScreenOrientation: {\n    prototype: ScreenOrientation;\n    new(): ScreenOrientation;\n};\n\ninterface ScriptProcessorNodeEventMap {\n    "audioprocess": AudioProcessingEvent;\n}\n\n/**\n * Allows the generation, processing, or analyzing of audio using JavaScript.\n * @deprecated As of the August 29 2014 Web Audio API spec publication, this feature has been marked as deprecated, and was replaced by AudioWorklet (see AudioWorkletNode).\n */\ninterface ScriptProcessorNode extends AudioNode {\n    /** @deprecated */\n    readonly bufferSize: number;\n    /** @deprecated */\n    onaudioprocess: ((this: ScriptProcessorNode, ev: AudioProcessingEvent) => any) | null;\n    addEventListener<K extends keyof ScriptProcessorNodeEventMap>(type: K, listener: (this: ScriptProcessorNode, ev: ScriptProcessorNodeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ScriptProcessorNodeEventMap>(type: K, listener: (this: ScriptProcessorNode, ev: ScriptProcessorNodeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\n/** @deprecated */\ndeclare var ScriptProcessorNode: {\n    prototype: ScriptProcessorNode;\n    new(): ScriptProcessorNode;\n};\n\n/** Inherits from Event, and represents the event object of an event sent on a document or worker when its content security policy is violated. */\ninterface SecurityPolicyViolationEvent extends Event {\n    readonly blockedURI: string;\n    readonly columnNumber: number;\n    readonly disposition: SecurityPolicyViolationEventDisposition;\n    readonly documentURI: string;\n    readonly effectiveDirective: string;\n    readonly lineNumber: number;\n    readonly originalPolicy: string;\n    readonly referrer: string;\n    readonly sample: string;\n    readonly sourceFile: string;\n    readonly statusCode: number;\n    readonly violatedDirective: string;\n}\n\ndeclare var SecurityPolicyViolationEvent: {\n    prototype: SecurityPolicyViolationEvent;\n    new(type: string, eventInitDict?: SecurityPolicyViolationEventInit): SecurityPolicyViolationEvent;\n};\n\n/** A Selection object represents the range of text selected by the user or the current position of the caret. To obtain a Selection object for examination or modification, call Window.getSelection(). */\ninterface Selection {\n    readonly anchorNode: Node | null;\n    readonly anchorOffset: number;\n    readonly focusNode: Node | null;\n    readonly focusOffset: number;\n    readonly isCollapsed: boolean;\n    readonly rangeCount: number;\n    readonly type: string;\n    addRange(range: Range): void;\n    collapse(node: Node | null, offset?: number): void;\n    collapseToEnd(): void;\n    collapseToStart(): void;\n    containsNode(node: Node, allowPartialContainment?: boolean): boolean;\n    deleteFromDocument(): void;\n    empty(): void;\n    extend(node: Node, offset?: number): void;\n    getRangeAt(index: number): Range;\n    removeAllRanges(): void;\n    removeRange(range: Range): void;\n    selectAllChildren(node: Node): void;\n    setBaseAndExtent(anchorNode: Node, anchorOffset: number, focusNode: Node, focusOffset: number): void;\n    setPosition(node: Node | null, offset?: number): void;\n    toString(): string;\n}\n\ndeclare var Selection: {\n    prototype: Selection;\n    new(): Selection;\n    toString(): string;\n};\n\ninterface ServiceWorkerEventMap extends AbstractWorkerEventMap {\n    "statechange": Event;\n}\n\n/** This ServiceWorker API interface provides a reference to a service worker. Multiple browsing contexts (e.g. pages, workers, etc.) can be associated with the same service worker, each through a unique ServiceWorker object. */\ninterface ServiceWorker extends EventTarget, AbstractWorker {\n    onstatechange: ((this: ServiceWorker, ev: Event) => any) | null;\n    readonly scriptURL: string;\n    readonly state: ServiceWorkerState;\n    postMessage(message: any, transfer: Transferable[]): void;\n    postMessage(message: any, options?: PostMessageOptions): void;\n    addEventListener<K extends keyof ServiceWorkerEventMap>(type: K, listener: (this: ServiceWorker, ev: ServiceWorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ServiceWorkerEventMap>(type: K, listener: (this: ServiceWorker, ev: ServiceWorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var ServiceWorker: {\n    prototype: ServiceWorker;\n    new(): ServiceWorker;\n};\n\ninterface ServiceWorkerContainerEventMap {\n    "controllerchange": Event;\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n}\n\n/** The ServiceWorkerContainer interface of the ServiceWorker API provides an object representing the service worker as an overall unit in the network ecosystem, including facilities to register, unregister and update service workers, and access the state of service workers and their registrations. */\ninterface ServiceWorkerContainer extends EventTarget {\n    readonly controller: ServiceWorker | null;\n    oncontrollerchange: ((this: ServiceWorkerContainer, ev: Event) => any) | null;\n    onmessage: ((this: ServiceWorkerContainer, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: ServiceWorkerContainer, ev: MessageEvent) => any) | null;\n    readonly ready: Promise<ServiceWorkerRegistration>;\n    getRegistration(clientURL?: string | URL): Promise<ServiceWorkerRegistration | undefined>;\n    getRegistrations(): Promise<ReadonlyArray<ServiceWorkerRegistration>>;\n    register(scriptURL: string | URL, options?: RegistrationOptions): Promise<ServiceWorkerRegistration>;\n    startMessages(): void;\n    addEventListener<K extends keyof ServiceWorkerContainerEventMap>(type: K, listener: (this: ServiceWorkerContainer, ev: ServiceWorkerContainerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ServiceWorkerContainerEventMap>(type: K, listener: (this: ServiceWorkerContainer, ev: ServiceWorkerContainerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var ServiceWorkerContainer: {\n    prototype: ServiceWorkerContainer;\n    new(): ServiceWorkerContainer;\n};\n\ninterface ServiceWorkerRegistrationEventMap {\n    "updatefound": Event;\n}\n\n/** This ServiceWorker API interface represents the service worker registration. You register a service worker to control one or more pages that share the same origin. */\ninterface ServiceWorkerRegistration extends EventTarget {\n    readonly active: ServiceWorker | null;\n    readonly installing: ServiceWorker | null;\n    onupdatefound: ((this: ServiceWorkerRegistration, ev: Event) => any) | null;\n    readonly pushManager: PushManager;\n    readonly scope: string;\n    readonly updateViaCache: ServiceWorkerUpdateViaCache;\n    readonly waiting: ServiceWorker | null;\n    getNotifications(filter?: GetNotificationOptions): Promise<Notification[]>;\n    showNotification(title: string, options?: NotificationOptions): Promise<void>;\n    unregister(): Promise<boolean>;\n    update(): Promise<void>;\n    addEventListener<K extends keyof ServiceWorkerRegistrationEventMap>(type: K, listener: (this: ServiceWorkerRegistration, ev: ServiceWorkerRegistrationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ServiceWorkerRegistrationEventMap>(type: K, listener: (this: ServiceWorkerRegistration, ev: ServiceWorkerRegistrationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var ServiceWorkerRegistration: {\n    prototype: ServiceWorkerRegistration;\n    new(): ServiceWorkerRegistration;\n};\n\ninterface ShadowRoot extends DocumentFragment, DocumentOrShadowRoot, InnerHTML {\n    readonly delegatesFocus: boolean;\n    readonly host: Element;\n    readonly mode: ShadowRootMode;\n    /**\n     * Throws a "NotSupportedError" DOMException if context object is a shadow root.\n     */\n}\n\ndeclare var ShadowRoot: {\n    prototype: ShadowRoot;\n    new(): ShadowRoot;\n};\n\ninterface SharedWorker extends EventTarget, AbstractWorker {\n    /**\n     * Returns sharedWorker\'s MessagePort object which can be used to communicate with the global environment.\n     */\n    readonly port: MessagePort;\n    addEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: SharedWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: SharedWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SharedWorker: {\n    prototype: SharedWorker;\n    new(scriptURL: string | URL, options?: string | WorkerOptions): SharedWorker;\n};\n\ninterface Slottable {\n    readonly assignedSlot: HTMLSlotElement | null;\n}\n\ninterface SourceBufferEventMap {\n    "abort": Event;\n    "error": Event;\n    "update": Event;\n    "updateend": Event;\n    "updatestart": Event;\n}\n\n/** A chunk of media to be passed into an HTMLMediaElement and played, via a MediaSource object. This can be made up of one or several media segments. */\ninterface SourceBuffer extends EventTarget {\n    appendWindowEnd: number;\n    appendWindowStart: number;\n    readonly buffered: TimeRanges;\n    mode: AppendMode;\n    onabort: ((this: SourceBuffer, ev: Event) => any) | null;\n    onerror: ((this: SourceBuffer, ev: Event) => any) | null;\n    onupdate: ((this: SourceBuffer, ev: Event) => any) | null;\n    onupdateend: ((this: SourceBuffer, ev: Event) => any) | null;\n    onupdatestart: ((this: SourceBuffer, ev: Event) => any) | null;\n    timestampOffset: number;\n    readonly updating: boolean;\n    abort(): void;\n    appendBuffer(data: BufferSource): void;\n    changeType(type: string): void;\n    remove(start: number, end: number): void;\n    addEventListener<K extends keyof SourceBufferEventMap>(type: K, listener: (this: SourceBuffer, ev: SourceBufferEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SourceBufferEventMap>(type: K, listener: (this: SourceBuffer, ev: SourceBufferEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SourceBuffer: {\n    prototype: SourceBuffer;\n    new(): SourceBuffer;\n};\n\ninterface SourceBufferListEventMap {\n    "addsourcebuffer": Event;\n    "removesourcebuffer": Event;\n}\n\n/** A simple container list for multiple SourceBuffer objects. */\ninterface SourceBufferList extends EventTarget {\n    readonly length: number;\n    onaddsourcebuffer: ((this: SourceBufferList, ev: Event) => any) | null;\n    onremovesourcebuffer: ((this: SourceBufferList, ev: Event) => any) | null;\n    addEventListener<K extends keyof SourceBufferListEventMap>(type: K, listener: (this: SourceBufferList, ev: SourceBufferListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SourceBufferListEventMap>(type: K, listener: (this: SourceBufferList, ev: SourceBufferListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n    [index: number]: SourceBuffer;\n}\n\ndeclare var SourceBufferList: {\n    prototype: SourceBufferList;\n    new(): SourceBufferList;\n};\n\ninterface SpeechRecognitionAlternative {\n    readonly confidence: number;\n    readonly transcript: string;\n}\n\ndeclare var SpeechRecognitionAlternative: {\n    prototype: SpeechRecognitionAlternative;\n    new(): SpeechRecognitionAlternative;\n};\n\ninterface SpeechRecognitionResult {\n    readonly isFinal: boolean;\n    readonly length: number;\n    item(index: number): SpeechRecognitionAlternative;\n    [index: number]: SpeechRecognitionAlternative;\n}\n\ndeclare var SpeechRecognitionResult: {\n    prototype: SpeechRecognitionResult;\n    new(): SpeechRecognitionResult;\n};\n\ninterface SpeechRecognitionResultList {\n    readonly length: number;\n    item(index: number): SpeechRecognitionResult;\n    [index: number]: SpeechRecognitionResult;\n}\n\ndeclare var SpeechRecognitionResultList: {\n    prototype: SpeechRecognitionResultList;\n    new(): SpeechRecognitionResultList;\n};\n\ninterface SpeechSynthesisEventMap {\n    "voiceschanged": Event;\n}\n\n/** This Web Speech API interface is the controller interface for the speech service; this can be used to retrieve information about the synthesis voices available on the device, start and pause speech, and other commands besides. */\ninterface SpeechSynthesis extends EventTarget {\n    onvoiceschanged: ((this: SpeechSynthesis, ev: Event) => any) | null;\n    readonly paused: boolean;\n    readonly pending: boolean;\n    readonly speaking: boolean;\n    cancel(): void;\n    getVoices(): SpeechSynthesisVoice[];\n    pause(): void;\n    resume(): void;\n    speak(utterance: SpeechSynthesisUtterance): void;\n    addEventListener<K extends keyof SpeechSynthesisEventMap>(type: K, listener: (this: SpeechSynthesis, ev: SpeechSynthesisEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SpeechSynthesisEventMap>(type: K, listener: (this: SpeechSynthesis, ev: SpeechSynthesisEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SpeechSynthesis: {\n    prototype: SpeechSynthesis;\n    new(): SpeechSynthesis;\n};\n\ninterface SpeechSynthesisErrorEvent extends SpeechSynthesisEvent {\n    readonly error: SpeechSynthesisErrorCode;\n}\n\ndeclare var SpeechSynthesisErrorEvent: {\n    prototype: SpeechSynthesisErrorEvent;\n    new(type: string, eventInitDict: SpeechSynthesisErrorEventInit): SpeechSynthesisErrorEvent;\n};\n\n/** This Web Speech API interface contains information about the current state of SpeechSynthesisUtterance objects that have been processed in the speech service. */\ninterface SpeechSynthesisEvent extends Event {\n    readonly charIndex: number;\n    readonly charLength: number;\n    readonly elapsedTime: number;\n    readonly name: string;\n    readonly utterance: SpeechSynthesisUtterance;\n}\n\ndeclare var SpeechSynthesisEvent: {\n    prototype: SpeechSynthesisEvent;\n    new(type: string, eventInitDict: SpeechSynthesisEventInit): SpeechSynthesisEvent;\n};\n\ninterface SpeechSynthesisUtteranceEventMap {\n    "boundary": SpeechSynthesisEvent;\n    "end": SpeechSynthesisEvent;\n    "error": SpeechSynthesisErrorEvent;\n    "mark": SpeechSynthesisEvent;\n    "pause": SpeechSynthesisEvent;\n    "resume": SpeechSynthesisEvent;\n    "start": SpeechSynthesisEvent;\n}\n\n/** This Web Speech API interface represents a speech request. It contains the content the speech service should read and information about how to read it (e.g. language, pitch and volume.) */\ninterface SpeechSynthesisUtterance extends EventTarget {\n    lang: string;\n    onboundary: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\n    onend: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\n    onerror: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisErrorEvent) => any) | null;\n    onmark: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\n    onpause: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\n    onresume: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\n    onstart: ((this: SpeechSynthesisUtterance, ev: SpeechSynthesisEvent) => any) | null;\n    pitch: number;\n    rate: number;\n    text: string;\n    voice: SpeechSynthesisVoice | null;\n    volume: number;\n    addEventListener<K extends keyof SpeechSynthesisUtteranceEventMap>(type: K, listener: (this: SpeechSynthesisUtterance, ev: SpeechSynthesisUtteranceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SpeechSynthesisUtteranceEventMap>(type: K, listener: (this: SpeechSynthesisUtterance, ev: SpeechSynthesisUtteranceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SpeechSynthesisUtterance: {\n    prototype: SpeechSynthesisUtterance;\n    new(text?: string): SpeechSynthesisUtterance;\n};\n\n/** This Web Speech API interface represents a voice that the system supports. Every SpeechSynthesisVoice has its own relative speech service including information about language, name and URI. */\ninterface SpeechSynthesisVoice {\n    readonly default: boolean;\n    readonly lang: string;\n    readonly localService: boolean;\n    readonly name: string;\n    readonly voiceURI: string;\n}\n\ndeclare var SpeechSynthesisVoice: {\n    prototype: SpeechSynthesisVoice;\n    new(): SpeechSynthesisVoice;\n};\n\ninterface StaticRange extends AbstractRange {\n}\n\ndeclare var StaticRange: {\n    prototype: StaticRange;\n    new(init: StaticRangeInit): StaticRange;\n};\n\n/** The pan property takes a unitless value between -1 (full left pan) and 1 (full right pan). This interface was introduced as a much simpler way to apply a simple panning effect than having to use a full PannerNode. */\ninterface StereoPannerNode extends AudioNode {\n    readonly pan: AudioParam;\n}\n\ndeclare var StereoPannerNode: {\n    prototype: StereoPannerNode;\n    new(context: BaseAudioContext, options?: StereoPannerOptions): StereoPannerNode;\n};\n\n/** This Web Storage API interface provides access to a particular domain\'s session or local storage. It allows, for example, the addition, modification, or deletion of stored data items. */\ninterface Storage {\n    /**\n     * Returns the number of key/value pairs.\n     */\n    readonly length: number;\n    /**\n     * Removes all key/value pairs, if there are any.\n     *\n     * Dispatches a storage event on Window objects holding an equivalent Storage object.\n     */\n    clear(): void;\n    /**\n     * Returns the current value associated with the given key, or null if the given key does not exist.\n     */\n    getItem(key: string): string | null;\n    /**\n     * Returns the name of the nth key, or null if n is greater than or equal to the number of key/value pairs.\n     */\n    key(index: number): string | null;\n    /**\n     * Removes the key/value pair with the given key, if a key/value pair with the given key exists.\n     *\n     * Dispatches a storage event on Window objects holding an equivalent Storage object.\n     */\n    removeItem(key: string): void;\n    /**\n     * Sets the value of the pair identified by key to value, creating a new key/value pair if none existed for key previously.\n     *\n     * Throws a "QuotaExceededError" DOMException exception if the new value couldn\'t be set. (Setting could fail if, e.g., the user has disabled storage for the site, or if the quota has been exceeded.)\n     *\n     * Dispatches a storage event on Window objects holding an equivalent Storage object.\n     */\n    setItem(key: string, value: string): void;\n    [name: string]: any;\n}\n\ndeclare var Storage: {\n    prototype: Storage;\n    new(): Storage;\n};\n\n/** A StorageEvent is sent to a window when a storage area it has access to is changed within the context of another document. */\ninterface StorageEvent extends Event {\n    /**\n     * Returns the key of the storage item being changed.\n     */\n    readonly key: string | null;\n    /**\n     * Returns the new value of the key of the storage item whose value is being changed.\n     */\n    readonly newValue: string | null;\n    /**\n     * Returns the old value of the key of the storage item whose value is being changed.\n     */\n    readonly oldValue: string | null;\n    /**\n     * Returns the Storage object that was affected.\n     */\n    readonly storageArea: Storage | null;\n    /**\n     * Returns the URL of the document whose storage item changed.\n     */\n    readonly url: string;\n    initStorageEvent(type: string, bubbles?: boolean, cancelable?: boolean, key?: string | null, oldValue?: string | null, newValue?: string | null, url?: string | URL, storageArea?: Storage | null): void;\n}\n\ndeclare var StorageEvent: {\n    prototype: StorageEvent;\n    new(type: string, eventInitDict?: StorageEventInit): StorageEvent;\n};\n\ninterface StorageManager {\n    estimate(): Promise<StorageEstimate>;\n    persist(): Promise<boolean>;\n    persisted(): Promise<boolean>;\n}\n\ndeclare var StorageManager: {\n    prototype: StorageManager;\n    new(): StorageManager;\n};\n\n/** @deprecated */\ninterface StyleMedia {\n    type: string;\n    matchMedium(mediaquery: string): boolean;\n}\n\n/** A single style sheet. CSS style sheets will further implement the more specialized CSSStyleSheet interface. */\ninterface StyleSheet {\n    disabled: boolean;\n    readonly href: string | null;\n    readonly media: MediaList;\n    readonly ownerNode: Element | ProcessingInstruction | null;\n    readonly parentStyleSheet: CSSStyleSheet | null;\n    readonly title: string | null;\n    readonly type: string;\n}\n\ndeclare var StyleSheet: {\n    prototype: StyleSheet;\n    new(): StyleSheet;\n};\n\n/** A list of StyleSheet. */\ninterface StyleSheetList {\n    readonly length: number;\n    item(index: number): CSSStyleSheet | null;\n    [index: number]: CSSStyleSheet;\n}\n\ndeclare var StyleSheetList: {\n    prototype: StyleSheetList;\n    new(): StyleSheetList;\n};\n\ninterface SubmitEvent extends Event {\n    /**\n     * Returns the element representing the submit button that triggered the form submission, or null if the submission was not triggered by a button.\n     */\n    readonly submitter: HTMLElement | null;\n}\n\ndeclare var SubmitEvent: {\n    prototype: SubmitEvent;\n    new(type: string, eventInitDict?: SubmitEventInit): SubmitEvent;\n};\n\n/** This Web Crypto API interface provides a number of low-level cryptographic functions. It is accessed via the Crypto.subtle properties available in a window context (via Window.crypto). */\ninterface SubtleCrypto {\n    decrypt(algorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams, key: CryptoKey, data: BufferSource): Promise<any>;\n    deriveBits(algorithm: AlgorithmIdentifier | EcdhKeyDeriveParams | HkdfParams | Pbkdf2Params, baseKey: CryptoKey, length: number): Promise<ArrayBuffer>;\n    deriveKey(algorithm: AlgorithmIdentifier | EcdhKeyDeriveParams | HkdfParams | Pbkdf2Params, baseKey: CryptoKey, derivedKeyType: AlgorithmIdentifier | AesDerivedKeyParams | HmacImportParams | HkdfParams | Pbkdf2Params, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    digest(algorithm: AlgorithmIdentifier, data: BufferSource): Promise<ArrayBuffer>;\n    encrypt(algorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams, key: CryptoKey, data: BufferSource): Promise<any>;\n    exportKey(format: "jwk", key: CryptoKey): Promise<JsonWebKey>;\n    exportKey(format: Exclude<KeyFormat, "jwk">, key: CryptoKey): Promise<ArrayBuffer>;\n    generateKey(algorithm: RsaHashedKeyGenParams | EcKeyGenParams, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKeyPair>;\n    generateKey(algorithm: AesKeyGenParams | HmacKeyGenParams | Pbkdf2Params, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    generateKey(algorithm: AlgorithmIdentifier, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKeyPair | CryptoKey>;\n    importKey(format: "jwk", keyData: JsonWebKey, algorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    importKey(format: Exclude<KeyFormat, "jwk">, keyData: BufferSource, algorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    sign(algorithm: AlgorithmIdentifier | RsaPssParams | EcdsaParams, key: CryptoKey, data: BufferSource): Promise<ArrayBuffer>;\n    unwrapKey(format: KeyFormat, wrappedKey: BufferSource, unwrappingKey: CryptoKey, unwrapAlgorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams, unwrappedKeyAlgorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    verify(algorithm: AlgorithmIdentifier | RsaPssParams | EcdsaParams, key: CryptoKey, signature: BufferSource, data: BufferSource): Promise<boolean>;\n    wrapKey(format: KeyFormat, key: CryptoKey, wrappingKey: CryptoKey, wrapAlgorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams): Promise<ArrayBuffer>;\n}\n\ndeclare var SubtleCrypto: {\n    prototype: SubtleCrypto;\n    new(): SubtleCrypto;\n};\n\n/** The textual content of Element or Attr. If an element has no markup within its content, it has a single child implementing Text that contains the element\'s text. However, if the element contains markup, it is parsed into information items and Text nodes that form its children. */\ninterface Text extends CharacterData, Slottable {\n    readonly assignedSlot: HTMLSlotElement | null;\n    /**\n     * Returns the combined data of all direct Text node siblings.\n     */\n    readonly wholeText: string;\n    /**\n     * Splits data at the given offset and returns the remainder as Text node.\n     */\n    splitText(offset: number): Text;\n}\n\ndeclare var Text: {\n    prototype: Text;\n    new(data?: string): Text;\n};\n\n/** A decoder for a specific method, that is a specific character encoding, like utf-8, iso-8859-2, koi8, cp1261, gbk, etc. A decoder takes a stream of bytes as input and emits a stream of code points. For a more scalable, non-native library, see StringView – a C-like representation of strings based on typed arrays. */\ninterface TextDecoder extends TextDecoderCommon {\n    /**\n     * Returns the result of running encoding\'s decoder. The method can be invoked zero or more times with options\'s stream set to true, and then once without options\'s stream (or set to false), to process a fragmented input. If the invocation without options\'s stream (or set to false) has no input, it\'s clearest to omit both arguments.\n     *\n     * ```\n     * var string = "", decoder = new TextDecoder(encoding), buffer;\n     * while(buffer = next_chunk()) {\n     *   string += decoder.decode(buffer, {stream:true});\n     * }\n     * string += decoder.decode(); // end-of-queue\n     * ```\n     *\n     * If the error mode is "fatal" and encoding\'s decoder returns error, throws a TypeError.\n     */\n    decode(input?: BufferSource, options?: TextDecodeOptions): string;\n}\n\ndeclare var TextDecoder: {\n    prototype: TextDecoder;\n    new(label?: string, options?: TextDecoderOptions): TextDecoder;\n};\n\ninterface TextDecoderCommon {\n    /**\n     * Returns encoding\'s name, lowercased.\n     */\n    readonly encoding: string;\n    /**\n     * Returns true if error mode is "fatal", otherwise false.\n     */\n    readonly fatal: boolean;\n    /**\n     * Returns the value of ignore BOM.\n     */\n    readonly ignoreBOM: boolean;\n}\n\ninterface TextDecoderStream extends GenericTransformStream, TextDecoderCommon {\n    readonly readable: ReadableStream<string>;\n    readonly writable: WritableStream<BufferSource>;\n}\n\ndeclare var TextDecoderStream: {\n    prototype: TextDecoderStream;\n    new(label?: string, options?: TextDecoderOptions): TextDecoderStream;\n};\n\n/** TextEncoder takes a stream of code points as input and emits a stream of bytes. For a more scalable, non-native library, see StringView – a C-like representation of strings based on typed arrays. */\ninterface TextEncoder extends TextEncoderCommon {\n    /**\n     * Returns the result of running UTF-8\'s encoder.\n     */\n    encode(input?: string): Uint8Array;\n    /**\n     * Runs the UTF-8 encoder on source, stores the result of that operation into destination, and returns the progress made as an object wherein read is the number of converted code units of source and written is the number of bytes modified in destination.\n     */\n    encodeInto(source: string, destination: Uint8Array): TextEncoderEncodeIntoResult;\n}\n\ndeclare var TextEncoder: {\n    prototype: TextEncoder;\n    new(): TextEncoder;\n};\n\ninterface TextEncoderCommon {\n    /**\n     * Returns "utf-8".\n     */\n    readonly encoding: string;\n}\n\ninterface TextEncoderStream extends GenericTransformStream, TextEncoderCommon {\n    readonly readable: ReadableStream<Uint8Array>;\n    readonly writable: WritableStream<string>;\n}\n\ndeclare var TextEncoderStream: {\n    prototype: TextEncoderStream;\n    new(): TextEncoderStream;\n};\n\n/** The dimensions of a piece of text in the canvas, as created by the CanvasRenderingContext2D.measureText() method. */\ninterface TextMetrics {\n    /**\n     * Returns the measurement described below.\n     */\n    readonly actualBoundingBoxAscent: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly actualBoundingBoxDescent: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly actualBoundingBoxLeft: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly actualBoundingBoxRight: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly fontBoundingBoxAscent: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly fontBoundingBoxDescent: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly width: number;\n}\n\ndeclare var TextMetrics: {\n    prototype: TextMetrics;\n    new(): TextMetrics;\n};\n\ninterface TextTrackEventMap {\n    "cuechange": Event;\n}\n\n/** This interface also inherits properties from EventTarget. */\ninterface TextTrack extends EventTarget {\n    /**\n     * Returns the text track cues from the text track list of cues that are currently active (i.e. that start before the current playback position and end after it), as a TextTrackCueList object.\n     */\n    readonly activeCues: TextTrackCueList | null;\n    /**\n     * Returns the text track list of cues, as a TextTrackCueList object.\n     */\n    readonly cues: TextTrackCueList | null;\n    /**\n     * Returns the ID of the given track.\n     *\n     * For in-band tracks, this is the ID that can be used with a fragment if the format supports media fragment syntax, and that can be used with the getTrackById() method.\n     *\n     * For TextTrack objects corresponding to track elements, this is the ID of the track element.\n     */\n    readonly id: string;\n    /**\n     * Returns the text track in-band metadata track dispatch type string.\n     */\n    readonly inBandMetadataTrackDispatchType: string;\n    /**\n     * Returns the text track kind string.\n     */\n    readonly kind: TextTrackKind;\n    /**\n     * Returns the text track label, if there is one, or the empty string otherwise (indicating that a custom label probably needs to be generated from the other attributes of the object if the object is exposed to the user).\n     */\n    readonly label: string;\n    /**\n     * Returns the text track language string.\n     */\n    readonly language: string;\n    /**\n     * Returns the text track mode, represented by a string from the following list:\n     *\n     * Can be set, to change the mode.\n     */\n    mode: TextTrackMode;\n    oncuechange: ((this: TextTrack, ev: Event) => any) | null;\n    /**\n     * Adds the given cue to textTrack\'s text track list of cues.\n     */\n    addCue(cue: TextTrackCue): void;\n    /**\n     * Removes the given cue from textTrack\'s text track list of cues.\n     */\n    removeCue(cue: TextTrackCue): void;\n    addEventListener<K extends keyof TextTrackEventMap>(type: K, listener: (this: TextTrack, ev: TextTrackEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof TextTrackEventMap>(type: K, listener: (this: TextTrack, ev: TextTrackEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var TextTrack: {\n    prototype: TextTrack;\n    new(): TextTrack;\n};\n\ninterface TextTrackCueEventMap {\n    "enter": Event;\n    "exit": Event;\n}\n\n/** TextTrackCues represent a string of text that will be displayed for some duration of time on a TextTrack. This includes the start and end times that the cue will be displayed. A TextTrackCue cannot be used directly, instead one of the derived types (e.g. VTTCue) must be used. */\ninterface TextTrackCue extends EventTarget {\n    /**\n     * Returns the text track cue end time, in seconds.\n     *\n     * Can be set.\n     */\n    endTime: number;\n    /**\n     * Returns the text track cue identifier.\n     *\n     * Can be set.\n     */\n    id: string;\n    onenter: ((this: TextTrackCue, ev: Event) => any) | null;\n    onexit: ((this: TextTrackCue, ev: Event) => any) | null;\n    /**\n     * Returns true if the text track cue pause-on-exit flag is set, false otherwise.\n     *\n     * Can be set.\n     */\n    pauseOnExit: boolean;\n    /**\n     * Returns the text track cue start time, in seconds.\n     *\n     * Can be set.\n     */\n    startTime: number;\n    /**\n     * Returns the TextTrack object to which this text track cue belongs, if any, or null otherwise.\n     */\n    readonly track: TextTrack | null;\n    addEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: TextTrackCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: TextTrackCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var TextTrackCue: {\n    prototype: TextTrackCue;\n    new(): TextTrackCue;\n};\n\ninterface TextTrackCueList {\n    /**\n     * Returns the number of cues in the list.\n     */\n    readonly length: number;\n    /**\n     * Returns the first text track cue (in text track cue order) with text track cue identifier id.\n     *\n     * Returns null if none of the cues have the given identifier or if the argument is the empty string.\n     */\n    getCueById(id: string): TextTrackCue | null;\n    [index: number]: TextTrackCue;\n}\n\ndeclare var TextTrackCueList: {\n    prototype: TextTrackCueList;\n    new(): TextTrackCueList;\n};\n\ninterface TextTrackListEventMap {\n    "addtrack": TrackEvent;\n    "change": Event;\n    "removetrack": TrackEvent;\n}\n\ninterface TextTrackList extends EventTarget {\n    readonly length: number;\n    onaddtrack: ((this: TextTrackList, ev: TrackEvent) => any) | null;\n    onchange: ((this: TextTrackList, ev: Event) => any) | null;\n    onremovetrack: ((this: TextTrackList, ev: TrackEvent) => any) | null;\n    getTrackById(id: string): TextTrack | null;\n    addEventListener<K extends keyof TextTrackListEventMap>(type: K, listener: (this: TextTrackList, ev: TextTrackListEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof TextTrackListEventMap>(type: K, listener: (this: TextTrackList, ev: TextTrackListEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n    [index: number]: TextTrack;\n}\n\ndeclare var TextTrackList: {\n    prototype: TextTrackList;\n    new(): TextTrackList;\n};\n\n/** Used to represent a set of time ranges, primarily for the purpose of tracking which portions of media have been buffered when loading it for use by the <audio> and <video> elements. */\ninterface TimeRanges {\n    /**\n     * Returns the number of ranges in the object.\n     */\n    readonly length: number;\n    /**\n     * Returns the time for the end of the range with the given index.\n     *\n     * Throws an "IndexSizeError" DOMException if the index is out of range.\n     */\n    end(index: number): number;\n    /**\n     * Returns the time for the start of the range with the given index.\n     *\n     * Throws an "IndexSizeError" DOMException if the index is out of range.\n     */\n    start(index: number): number;\n}\n\ndeclare var TimeRanges: {\n    prototype: TimeRanges;\n    new(): TimeRanges;\n};\n\n/** A single contact point on a touch-sensitive device. The contact point is commonly a finger or stylus and the device may be a touchscreen or trackpad. */\ninterface Touch {\n    readonly clientX: number;\n    readonly clientY: number;\n    readonly force: number;\n    readonly identifier: number;\n    readonly pageX: number;\n    readonly pageY: number;\n    readonly radiusX: number;\n    readonly radiusY: number;\n    readonly rotationAngle: number;\n    readonly screenX: number;\n    readonly screenY: number;\n    readonly target: EventTarget;\n}\n\ndeclare var Touch: {\n    prototype: Touch;\n    new(touchInitDict: TouchInit): Touch;\n};\n\n/** An event sent when the state of contacts with a touch-sensitive surface changes. This surface can be a touch screen or trackpad, for example. The event can describe one or more points of contact with the screen and includes support for detecting movement, addition and removal of contact points, and so forth. */\ninterface TouchEvent extends UIEvent {\n    readonly altKey: boolean;\n    readonly changedTouches: TouchList;\n    readonly ctrlKey: boolean;\n    readonly metaKey: boolean;\n    readonly shiftKey: boolean;\n    readonly targetTouches: TouchList;\n    readonly touches: TouchList;\n}\n\ndeclare var TouchEvent: {\n    prototype: TouchEvent;\n    new(type: string, eventInitDict?: TouchEventInit): TouchEvent;\n};\n\n/** A list of contact points on a touch surface. For example, if the user has three fingers on the touch surface (such as a screen or trackpad), the corresponding TouchList object would have one Touch object for each finger, for a total of three entries. */\ninterface TouchList {\n    readonly length: number;\n    item(index: number): Touch | null;\n    [index: number]: Touch;\n}\n\ndeclare var TouchList: {\n    prototype: TouchList;\n    new(): TouchList;\n};\n\n/** The TrackEvent interface, part of the HTML DOM specification, is used for events which represent changes to the set of available tracks on an HTML media element; these events are addtrack and removetrack. */\ninterface TrackEvent extends Event {\n    /**\n     * Returns the track object (TextTrack, AudioTrack, or VideoTrack) to which the event relates.\n     */\n    readonly track: TextTrack | null;\n}\n\ndeclare var TrackEvent: {\n    prototype: TrackEvent;\n    new(type: string, eventInitDict?: TrackEventInit): TrackEvent;\n};\n\ninterface TransformStream<I = any, O = any> {\n    readonly readable: ReadableStream<O>;\n    readonly writable: WritableStream<I>;\n}\n\ndeclare var TransformStream: {\n    prototype: TransformStream;\n    new<I = any, O = any>(transformer?: Transformer<I, O>, writableStrategy?: QueuingStrategy<I>, readableStrategy?: QueuingStrategy<O>): TransformStream<I, O>;\n};\n\ninterface TransformStreamDefaultController<O = any> {\n    readonly desiredSize: number | null;\n    enqueue(chunk?: O): void;\n    error(reason?: any): void;\n    terminate(): void;\n}\n\ndeclare var TransformStreamDefaultController: {\n    prototype: TransformStreamDefaultController;\n    new(): TransformStreamDefaultController;\n};\n\n/** Events providing information related to transitions. */\ninterface TransitionEvent extends Event {\n    readonly elapsedTime: number;\n    readonly propertyName: string;\n    readonly pseudoElement: string;\n}\n\ndeclare var TransitionEvent: {\n    prototype: TransitionEvent;\n    new(type: string, transitionEventInitDict?: TransitionEventInit): TransitionEvent;\n};\n\n/** The nodes of a document subtree and a position within them. */\ninterface TreeWalker {\n    currentNode: Node;\n    readonly filter: NodeFilter | null;\n    readonly root: Node;\n    readonly whatToShow: number;\n    firstChild(): Node | null;\n    lastChild(): Node | null;\n    nextNode(): Node | null;\n    nextSibling(): Node | null;\n    parentNode(): Node | null;\n    previousNode(): Node | null;\n    previousSibling(): Node | null;\n}\n\ndeclare var TreeWalker: {\n    prototype: TreeWalker;\n    new(): TreeWalker;\n};\n\n/** Simple user interface events. */\ninterface UIEvent extends Event {\n    readonly detail: number;\n    readonly view: Window | null;\n    /** @deprecated */\n    readonly which: number;\n    /** @deprecated */\n    initUIEvent(typeArg: string, bubblesArg?: boolean, cancelableArg?: boolean, viewArg?: Window | null, detailArg?: number): void;\n}\n\ndeclare var UIEvent: {\n    prototype: UIEvent;\n    new(type: string, eventInitDict?: UIEventInit): UIEvent;\n};\n\n/** The URL interface represents an object providing static methods used for creating object URLs. */\ninterface URL {\n    hash: string;\n    host: string;\n    hostname: string;\n    href: string;\n    toString(): string;\n    readonly origin: string;\n    password: string;\n    pathname: string;\n    port: string;\n    protocol: string;\n    search: string;\n    readonly searchParams: URLSearchParams;\n    username: string;\n    toJSON(): string;\n}\n\ndeclare var URL: {\n    prototype: URL;\n    new(url: string | URL, base?: string | URL): URL;\n    createObjectURL(object: any): string;\n    revokeObjectURL(url: string): void;\n};\n\ntype webkitURL = URL;\ndeclare var webkitURL: typeof URL;\n\ninterface URLSearchParams {\n    /**\n     * Appends a specified key/value pair as a new search parameter.\n     */\n    append(name: string, value: string): void;\n    /**\n     * Deletes the given search parameter, and its associated value, from the list of all search parameters.\n     */\n    delete(name: string): void;\n    /**\n     * Returns the first value associated to the given search parameter.\n     */\n    get(name: string): string | null;\n    /**\n     * Returns all the values association with a given search parameter.\n     */\n    getAll(name: string): string[];\n    /**\n     * Returns a Boolean indicating if such a search parameter exists.\n     */\n    has(name: string): boolean;\n    /**\n     * Sets the value associated to a given search parameter to the given value. If there were several values, delete the others.\n     */\n    set(name: string, value: string): void;\n    sort(): void;\n    /**\n     * Returns a string containing a query string suitable for use in a URL. Does not include the question mark.\n     */\n    toString(): string;\n    forEach(callbackfn: (value: string, key: string, parent: URLSearchParams) => void, thisArg?: any): void;\n}\n\ndeclare var URLSearchParams: {\n    prototype: URLSearchParams;\n    new(init?: string[][] | Record<string, string> | string | URLSearchParams): URLSearchParams;\n    toString(): string;\n};\n\ninterface VTTCue extends TextTrackCue {\n    align: AlignSetting;\n    line: LineAndPositionSetting;\n    lineAlign: LineAlignSetting;\n    position: LineAndPositionSetting;\n    positionAlign: PositionAlignSetting;\n    region: VTTRegion | null;\n    size: number;\n    snapToLines: boolean;\n    text: string;\n    vertical: DirectionSetting;\n    getCueAsHTML(): DocumentFragment;\n    addEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: VTTCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof TextTrackCueEventMap>(type: K, listener: (this: VTTCue, ev: TextTrackCueEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var VTTCue: {\n    prototype: VTTCue;\n    new(startTime: number, endTime: number, text: string): VTTCue;\n};\n\ninterface VTTRegion {\n    id: string;\n    lines: number;\n    regionAnchorX: number;\n    regionAnchorY: number;\n    scroll: ScrollSetting;\n    viewportAnchorX: number;\n    viewportAnchorY: number;\n    width: number;\n}\n\ndeclare var VTTRegion: {\n    prototype: VTTRegion;\n    new(): VTTRegion;\n};\n\n/** The validity states that an element can be in, with respect to constraint validation. Together, they help explain why an element\'s value fails to validate, if it\'s not valid. */\ninterface ValidityState {\n    readonly badInput: boolean;\n    readonly customError: boolean;\n    readonly patternMismatch: boolean;\n    readonly rangeOverflow: boolean;\n    readonly rangeUnderflow: boolean;\n    readonly stepMismatch: boolean;\n    readonly tooLong: boolean;\n    readonly tooShort: boolean;\n    readonly typeMismatch: boolean;\n    readonly valid: boolean;\n    readonly valueMissing: boolean;\n}\n\ndeclare var ValidityState: {\n    prototype: ValidityState;\n    new(): ValidityState;\n};\n\n/** Returned by the HTMLVideoElement.getVideoPlaybackQuality() method and contains metrics that can be used to determine the playback quality of a video. */\ninterface VideoPlaybackQuality {\n    /** @deprecated */\n    readonly corruptedVideoFrames: number;\n    readonly creationTime: DOMHighResTimeStamp;\n    readonly droppedVideoFrames: number;\n    readonly totalVideoFrames: number;\n}\n\ndeclare var VideoPlaybackQuality: {\n    prototype: VideoPlaybackQuality;\n    new(): VideoPlaybackQuality;\n};\n\ninterface VisualViewportEventMap {\n    "resize": Event;\n    "scroll": Event;\n}\n\ninterface VisualViewport extends EventTarget {\n    readonly height: number;\n    readonly offsetLeft: number;\n    readonly offsetTop: number;\n    onresize: ((this: VisualViewport, ev: Event) => any) | null;\n    onscroll: ((this: VisualViewport, ev: Event) => any) | null;\n    readonly pageLeft: number;\n    readonly pageTop: number;\n    readonly scale: number;\n    readonly width: number;\n    addEventListener<K extends keyof VisualViewportEventMap>(type: K, listener: (this: VisualViewport, ev: VisualViewportEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof VisualViewportEventMap>(type: K, listener: (this: VisualViewport, ev: VisualViewportEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var VisualViewport: {\n    prototype: VisualViewport;\n    new(): VisualViewport;\n};\n\ninterface WEBGL_color_buffer_float {\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT: GLenum;\n    readonly RGBA32F_EXT: GLenum;\n    readonly UNSIGNED_NORMALIZED_EXT: GLenum;\n}\n\ninterface WEBGL_compressed_texture_astc {\n    getSupportedProfiles(): string[];\n    readonly COMPRESSED_RGBA_ASTC_10x10_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_10x5_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_10x6_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_10x8_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_12x10_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_12x12_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_4x4_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_5x4_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_5x5_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_6x5_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_6x6_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_8x5_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_8x6_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_8x8_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x10_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x5_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x6_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x8_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_12x10_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_12x12_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_5x4_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_5x5_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_6x5_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_6x6_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x5_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x6_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x8_KHR: GLenum;\n}\n\ninterface WEBGL_compressed_texture_etc {\n    readonly COMPRESSED_R11_EAC: GLenum;\n    readonly COMPRESSED_RG11_EAC: GLenum;\n    readonly COMPRESSED_RGB8_ETC2: GLenum;\n    readonly COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2: GLenum;\n    readonly COMPRESSED_RGBA8_ETC2_EAC: GLenum;\n    readonly COMPRESSED_SIGNED_R11_EAC: GLenum;\n    readonly COMPRESSED_SIGNED_RG11_EAC: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ETC2_EAC: GLenum;\n    readonly COMPRESSED_SRGB8_ETC2: GLenum;\n    readonly COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2: GLenum;\n}\n\ninterface WEBGL_compressed_texture_etc1 {\n    readonly COMPRESSED_RGB_ETC1_WEBGL: GLenum;\n}\n\ninterface WEBGL_compressed_texture_pvrtc {\n    readonly COMPRESSED_RGBA_PVRTC_2BPPV1_IMG: GLenum;\n    readonly COMPRESSED_RGBA_PVRTC_4BPPV1_IMG: GLenum;\n    readonly COMPRESSED_RGB_PVRTC_2BPPV1_IMG: GLenum;\n    readonly COMPRESSED_RGB_PVRTC_4BPPV1_IMG: GLenum;\n}\n\n/** The WEBGL_compressed_texture_s3tc extension is part of the WebGL API and exposes four S3TC compressed texture formats. */\ninterface WEBGL_compressed_texture_s3tc {\n    readonly COMPRESSED_RGBA_S3TC_DXT1_EXT: GLenum;\n    readonly COMPRESSED_RGBA_S3TC_DXT3_EXT: GLenum;\n    readonly COMPRESSED_RGBA_S3TC_DXT5_EXT: GLenum;\n    readonly COMPRESSED_RGB_S3TC_DXT1_EXT: GLenum;\n}\n\ninterface WEBGL_compressed_texture_s3tc_srgb {\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT: GLenum;\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT: GLenum;\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT: GLenum;\n    readonly COMPRESSED_SRGB_S3TC_DXT1_EXT: GLenum;\n}\n\n/** The WEBGL_debug_renderer_info extension is part of the WebGL API and exposes two constants with information about the graphics driver for debugging purposes. */\ninterface WEBGL_debug_renderer_info {\n    readonly UNMASKED_RENDERER_WEBGL: GLenum;\n    readonly UNMASKED_VENDOR_WEBGL: GLenum;\n}\n\ninterface WEBGL_debug_shaders {\n    getTranslatedShaderSource(shader: WebGLShader): string;\n}\n\n/** The WEBGL_depth_texture extension is part of the WebGL API and defines 2D depth and depth-stencil textures. */\ninterface WEBGL_depth_texture {\n    readonly UNSIGNED_INT_24_8_WEBGL: GLenum;\n}\n\ninterface WEBGL_draw_buffers {\n    drawBuffersWEBGL(buffers: GLenum[]): void;\n    readonly COLOR_ATTACHMENT0_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT10_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT11_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT12_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT13_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT14_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT15_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT1_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT2_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT3_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT4_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT5_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT6_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT7_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT8_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT9_WEBGL: GLenum;\n    readonly DRAW_BUFFER0_WEBGL: GLenum;\n    readonly DRAW_BUFFER10_WEBGL: GLenum;\n    readonly DRAW_BUFFER11_WEBGL: GLenum;\n    readonly DRAW_BUFFER12_WEBGL: GLenum;\n    readonly DRAW_BUFFER13_WEBGL: GLenum;\n    readonly DRAW_BUFFER14_WEBGL: GLenum;\n    readonly DRAW_BUFFER15_WEBGL: GLenum;\n    readonly DRAW_BUFFER1_WEBGL: GLenum;\n    readonly DRAW_BUFFER2_WEBGL: GLenum;\n    readonly DRAW_BUFFER3_WEBGL: GLenum;\n    readonly DRAW_BUFFER4_WEBGL: GLenum;\n    readonly DRAW_BUFFER5_WEBGL: GLenum;\n    readonly DRAW_BUFFER6_WEBGL: GLenum;\n    readonly DRAW_BUFFER7_WEBGL: GLenum;\n    readonly DRAW_BUFFER8_WEBGL: GLenum;\n    readonly DRAW_BUFFER9_WEBGL: GLenum;\n    readonly MAX_COLOR_ATTACHMENTS_WEBGL: GLenum;\n    readonly MAX_DRAW_BUFFERS_WEBGL: GLenum;\n}\n\ninterface WEBGL_lose_context {\n    loseContext(): void;\n    restoreContext(): void;\n}\n\n/** A WaveShaperNode always has exactly one input and one output. */\ninterface WaveShaperNode extends AudioNode {\n    curve: Float32Array | null;\n    oversample: OverSampleType;\n}\n\ndeclare var WaveShaperNode: {\n    prototype: WaveShaperNode;\n    new(context: BaseAudioContext, options?: WaveShaperOptions): WaveShaperNode;\n};\n\ninterface WebGL2RenderingContext extends WebGL2RenderingContextBase, WebGL2RenderingContextOverloads, WebGLRenderingContextBase {\n}\n\ndeclare var WebGL2RenderingContext: {\n    prototype: WebGL2RenderingContext;\n    new(): WebGL2RenderingContext;\n    readonly ACTIVE_UNIFORM_BLOCKS: GLenum;\n    readonly ALREADY_SIGNALED: GLenum;\n    readonly ANY_SAMPLES_PASSED: GLenum;\n    readonly ANY_SAMPLES_PASSED_CONSERVATIVE: GLenum;\n    readonly COLOR: GLenum;\n    readonly COLOR_ATTACHMENT1: GLenum;\n    readonly COLOR_ATTACHMENT10: GLenum;\n    readonly COLOR_ATTACHMENT11: GLenum;\n    readonly COLOR_ATTACHMENT12: GLenum;\n    readonly COLOR_ATTACHMENT13: GLenum;\n    readonly COLOR_ATTACHMENT14: GLenum;\n    readonly COLOR_ATTACHMENT15: GLenum;\n    readonly COLOR_ATTACHMENT2: GLenum;\n    readonly COLOR_ATTACHMENT3: GLenum;\n    readonly COLOR_ATTACHMENT4: GLenum;\n    readonly COLOR_ATTACHMENT5: GLenum;\n    readonly COLOR_ATTACHMENT6: GLenum;\n    readonly COLOR_ATTACHMENT7: GLenum;\n    readonly COLOR_ATTACHMENT8: GLenum;\n    readonly COLOR_ATTACHMENT9: GLenum;\n    readonly COMPARE_REF_TO_TEXTURE: GLenum;\n    readonly CONDITION_SATISFIED: GLenum;\n    readonly COPY_READ_BUFFER: GLenum;\n    readonly COPY_READ_BUFFER_BINDING: GLenum;\n    readonly COPY_WRITE_BUFFER: GLenum;\n    readonly COPY_WRITE_BUFFER_BINDING: GLenum;\n    readonly CURRENT_QUERY: GLenum;\n    readonly DEPTH: GLenum;\n    readonly DEPTH24_STENCIL8: GLenum;\n    readonly DEPTH32F_STENCIL8: GLenum;\n    readonly DEPTH_COMPONENT24: GLenum;\n    readonly DEPTH_COMPONENT32F: GLenum;\n    readonly DRAW_BUFFER0: GLenum;\n    readonly DRAW_BUFFER1: GLenum;\n    readonly DRAW_BUFFER10: GLenum;\n    readonly DRAW_BUFFER11: GLenum;\n    readonly DRAW_BUFFER12: GLenum;\n    readonly DRAW_BUFFER13: GLenum;\n    readonly DRAW_BUFFER14: GLenum;\n    readonly DRAW_BUFFER15: GLenum;\n    readonly DRAW_BUFFER2: GLenum;\n    readonly DRAW_BUFFER3: GLenum;\n    readonly DRAW_BUFFER4: GLenum;\n    readonly DRAW_BUFFER5: GLenum;\n    readonly DRAW_BUFFER6: GLenum;\n    readonly DRAW_BUFFER7: GLenum;\n    readonly DRAW_BUFFER8: GLenum;\n    readonly DRAW_BUFFER9: GLenum;\n    readonly DRAW_FRAMEBUFFER: GLenum;\n    readonly DRAW_FRAMEBUFFER_BINDING: GLenum;\n    readonly DYNAMIC_COPY: GLenum;\n    readonly DYNAMIC_READ: GLenum;\n    readonly FLOAT_32_UNSIGNED_INT_24_8_REV: GLenum;\n    readonly FLOAT_MAT2x3: GLenum;\n    readonly FLOAT_MAT2x4: GLenum;\n    readonly FLOAT_MAT3x2: GLenum;\n    readonly FLOAT_MAT3x4: GLenum;\n    readonly FLOAT_MAT4x2: GLenum;\n    readonly FLOAT_MAT4x3: GLenum;\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_BLUE_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_GREEN_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_RED_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER: GLenum;\n    readonly FRAMEBUFFER_DEFAULT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MULTISAMPLE: GLenum;\n    readonly HALF_FLOAT: GLenum;\n    readonly INTERLEAVED_ATTRIBS: GLenum;\n    readonly INT_2_10_10_10_REV: GLenum;\n    readonly INT_SAMPLER_2D: GLenum;\n    readonly INT_SAMPLER_2D_ARRAY: GLenum;\n    readonly INT_SAMPLER_3D: GLenum;\n    readonly INT_SAMPLER_CUBE: GLenum;\n    readonly INVALID_INDEX: GLenum;\n    readonly MAX: GLenum;\n    readonly MAX_3D_TEXTURE_SIZE: GLenum;\n    readonly MAX_ARRAY_TEXTURE_LAYERS: GLenum;\n    readonly MAX_CLIENT_WAIT_TIMEOUT_WEBGL: GLenum;\n    readonly MAX_COLOR_ATTACHMENTS: GLenum;\n    readonly MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_COMBINED_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_DRAW_BUFFERS: GLenum;\n    readonly MAX_ELEMENTS_INDICES: GLenum;\n    readonly MAX_ELEMENTS_VERTICES: GLenum;\n    readonly MAX_ELEMENT_INDEX: GLenum;\n    readonly MAX_FRAGMENT_INPUT_COMPONENTS: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_PROGRAM_TEXEL_OFFSET: GLenum;\n    readonly MAX_SAMPLES: GLenum;\n    readonly MAX_SERVER_WAIT_TIMEOUT: GLenum;\n    readonly MAX_TEXTURE_LOD_BIAS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS: GLenum;\n    readonly MAX_UNIFORM_BLOCK_SIZE: GLenum;\n    readonly MAX_UNIFORM_BUFFER_BINDINGS: GLenum;\n    readonly MAX_VARYING_COMPONENTS: GLenum;\n    readonly MAX_VERTEX_OUTPUT_COMPONENTS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_COMPONENTS: GLenum;\n    readonly MIN: GLenum;\n    readonly MIN_PROGRAM_TEXEL_OFFSET: GLenum;\n    readonly OBJECT_TYPE: GLenum;\n    readonly PACK_ROW_LENGTH: GLenum;\n    readonly PACK_SKIP_PIXELS: GLenum;\n    readonly PACK_SKIP_ROWS: GLenum;\n    readonly PIXEL_PACK_BUFFER: GLenum;\n    readonly PIXEL_PACK_BUFFER_BINDING: GLenum;\n    readonly PIXEL_UNPACK_BUFFER: GLenum;\n    readonly PIXEL_UNPACK_BUFFER_BINDING: GLenum;\n    readonly QUERY_RESULT: GLenum;\n    readonly QUERY_RESULT_AVAILABLE: GLenum;\n    readonly R11F_G11F_B10F: GLenum;\n    readonly R16F: GLenum;\n    readonly R16I: GLenum;\n    readonly R16UI: GLenum;\n    readonly R32F: GLenum;\n    readonly R32I: GLenum;\n    readonly R32UI: GLenum;\n    readonly R8: GLenum;\n    readonly R8I: GLenum;\n    readonly R8UI: GLenum;\n    readonly R8_SNORM: GLenum;\n    readonly RASTERIZER_DISCARD: GLenum;\n    readonly READ_BUFFER: GLenum;\n    readonly READ_FRAMEBUFFER: GLenum;\n    readonly READ_FRAMEBUFFER_BINDING: GLenum;\n    readonly RED: GLenum;\n    readonly RED_INTEGER: GLenum;\n    readonly RENDERBUFFER_SAMPLES: GLenum;\n    readonly RG: GLenum;\n    readonly RG16F: GLenum;\n    readonly RG16I: GLenum;\n    readonly RG16UI: GLenum;\n    readonly RG32F: GLenum;\n    readonly RG32I: GLenum;\n    readonly RG32UI: GLenum;\n    readonly RG8: GLenum;\n    readonly RG8I: GLenum;\n    readonly RG8UI: GLenum;\n    readonly RG8_SNORM: GLenum;\n    readonly RGB10_A2: GLenum;\n    readonly RGB10_A2UI: GLenum;\n    readonly RGB16F: GLenum;\n    readonly RGB16I: GLenum;\n    readonly RGB16UI: GLenum;\n    readonly RGB32F: GLenum;\n    readonly RGB32I: GLenum;\n    readonly RGB32UI: GLenum;\n    readonly RGB8: GLenum;\n    readonly RGB8I: GLenum;\n    readonly RGB8UI: GLenum;\n    readonly RGB8_SNORM: GLenum;\n    readonly RGB9_E5: GLenum;\n    readonly RGBA16F: GLenum;\n    readonly RGBA16I: GLenum;\n    readonly RGBA16UI: GLenum;\n    readonly RGBA32F: GLenum;\n    readonly RGBA32I: GLenum;\n    readonly RGBA32UI: GLenum;\n    readonly RGBA8: GLenum;\n    readonly RGBA8I: GLenum;\n    readonly RGBA8UI: GLenum;\n    readonly RGBA8_SNORM: GLenum;\n    readonly RGBA_INTEGER: GLenum;\n    readonly RGB_INTEGER: GLenum;\n    readonly RG_INTEGER: GLenum;\n    readonly SAMPLER_2D_ARRAY: GLenum;\n    readonly SAMPLER_2D_ARRAY_SHADOW: GLenum;\n    readonly SAMPLER_2D_SHADOW: GLenum;\n    readonly SAMPLER_3D: GLenum;\n    readonly SAMPLER_BINDING: GLenum;\n    readonly SAMPLER_CUBE_SHADOW: GLenum;\n    readonly SEPARATE_ATTRIBS: GLenum;\n    readonly SIGNALED: GLenum;\n    readonly SIGNED_NORMALIZED: GLenum;\n    readonly SRGB: GLenum;\n    readonly SRGB8: GLenum;\n    readonly SRGB8_ALPHA8: GLenum;\n    readonly STATIC_COPY: GLenum;\n    readonly STATIC_READ: GLenum;\n    readonly STENCIL: GLenum;\n    readonly STREAM_COPY: GLenum;\n    readonly STREAM_READ: GLenum;\n    readonly SYNC_CONDITION: GLenum;\n    readonly SYNC_FENCE: GLenum;\n    readonly SYNC_FLAGS: GLenum;\n    readonly SYNC_FLUSH_COMMANDS_BIT: GLenum;\n    readonly SYNC_GPU_COMMANDS_COMPLETE: GLenum;\n    readonly SYNC_STATUS: GLenum;\n    readonly TEXTURE_2D_ARRAY: GLenum;\n    readonly TEXTURE_3D: GLenum;\n    readonly TEXTURE_BASE_LEVEL: GLenum;\n    readonly TEXTURE_BINDING_2D_ARRAY: GLenum;\n    readonly TEXTURE_BINDING_3D: GLenum;\n    readonly TEXTURE_COMPARE_FUNC: GLenum;\n    readonly TEXTURE_COMPARE_MODE: GLenum;\n    readonly TEXTURE_IMMUTABLE_FORMAT: GLenum;\n    readonly TEXTURE_IMMUTABLE_LEVELS: GLenum;\n    readonly TEXTURE_MAX_LEVEL: GLenum;\n    readonly TEXTURE_MAX_LOD: GLenum;\n    readonly TEXTURE_MIN_LOD: GLenum;\n    readonly TEXTURE_WRAP_R: GLenum;\n    readonly TIMEOUT_EXPIRED: GLenum;\n    readonly TIMEOUT_IGNORED: GLint64;\n    readonly TRANSFORM_FEEDBACK: GLenum;\n    readonly TRANSFORM_FEEDBACK_ACTIVE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BINDING: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_BINDING: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_MODE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_SIZE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_START: GLenum;\n    readonly TRANSFORM_FEEDBACK_PAUSED: GLenum;\n    readonly TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN: GLenum;\n    readonly TRANSFORM_FEEDBACK_VARYINGS: GLenum;\n    readonly UNIFORM_ARRAY_STRIDE: GLenum;\n    readonly UNIFORM_BLOCK_ACTIVE_UNIFORMS: GLenum;\n    readonly UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES: GLenum;\n    readonly UNIFORM_BLOCK_BINDING: GLenum;\n    readonly UNIFORM_BLOCK_DATA_SIZE: GLenum;\n    readonly UNIFORM_BLOCK_INDEX: GLenum;\n    readonly UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER: GLenum;\n    readonly UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER: GLenum;\n    readonly UNIFORM_BUFFER: GLenum;\n    readonly UNIFORM_BUFFER_BINDING: GLenum;\n    readonly UNIFORM_BUFFER_OFFSET_ALIGNMENT: GLenum;\n    readonly UNIFORM_BUFFER_SIZE: GLenum;\n    readonly UNIFORM_BUFFER_START: GLenum;\n    readonly UNIFORM_IS_ROW_MAJOR: GLenum;\n    readonly UNIFORM_MATRIX_STRIDE: GLenum;\n    readonly UNIFORM_OFFSET: GLenum;\n    readonly UNIFORM_SIZE: GLenum;\n    readonly UNIFORM_TYPE: GLenum;\n    readonly UNPACK_IMAGE_HEIGHT: GLenum;\n    readonly UNPACK_ROW_LENGTH: GLenum;\n    readonly UNPACK_SKIP_IMAGES: GLenum;\n    readonly UNPACK_SKIP_PIXELS: GLenum;\n    readonly UNPACK_SKIP_ROWS: GLenum;\n    readonly UNSIGNALED: GLenum;\n    readonly UNSIGNED_INT_10F_11F_11F_REV: GLenum;\n    readonly UNSIGNED_INT_24_8: GLenum;\n    readonly UNSIGNED_INT_2_10_10_10_REV: GLenum;\n    readonly UNSIGNED_INT_5_9_9_9_REV: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_2D: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_2D_ARRAY: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_3D: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_CUBE: GLenum;\n    readonly UNSIGNED_INT_VEC2: GLenum;\n    readonly UNSIGNED_INT_VEC3: GLenum;\n    readonly UNSIGNED_INT_VEC4: GLenum;\n    readonly UNSIGNED_NORMALIZED: GLenum;\n    readonly VERTEX_ARRAY_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_INTEGER: GLenum;\n    readonly WAIT_FAILED: GLenum;\n    readonly ACTIVE_ATTRIBUTES: GLenum;\n    readonly ACTIVE_TEXTURE: GLenum;\n    readonly ACTIVE_UNIFORMS: GLenum;\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\n    readonly ALPHA: GLenum;\n    readonly ALPHA_BITS: GLenum;\n    readonly ALWAYS: GLenum;\n    readonly ARRAY_BUFFER: GLenum;\n    readonly ARRAY_BUFFER_BINDING: GLenum;\n    readonly ATTACHED_SHADERS: GLenum;\n    readonly BACK: GLenum;\n    readonly BLEND: GLenum;\n    readonly BLEND_COLOR: GLenum;\n    readonly BLEND_DST_ALPHA: GLenum;\n    readonly BLEND_DST_RGB: GLenum;\n    readonly BLEND_EQUATION: GLenum;\n    readonly BLEND_EQUATION_ALPHA: GLenum;\n    readonly BLEND_EQUATION_RGB: GLenum;\n    readonly BLEND_SRC_ALPHA: GLenum;\n    readonly BLEND_SRC_RGB: GLenum;\n    readonly BLUE_BITS: GLenum;\n    readonly BOOL: GLenum;\n    readonly BOOL_VEC2: GLenum;\n    readonly BOOL_VEC3: GLenum;\n    readonly BOOL_VEC4: GLenum;\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\n    readonly BUFFER_SIZE: GLenum;\n    readonly BUFFER_USAGE: GLenum;\n    readonly BYTE: GLenum;\n    readonly CCW: GLenum;\n    readonly CLAMP_TO_EDGE: GLenum;\n    readonly COLOR_ATTACHMENT0: GLenum;\n    readonly COLOR_BUFFER_BIT: GLenum;\n    readonly COLOR_CLEAR_VALUE: GLenum;\n    readonly COLOR_WRITEMASK: GLenum;\n    readonly COMPILE_STATUS: GLenum;\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\n    readonly CONSTANT_ALPHA: GLenum;\n    readonly CONSTANT_COLOR: GLenum;\n    readonly CONTEXT_LOST_WEBGL: GLenum;\n    readonly CULL_FACE: GLenum;\n    readonly CULL_FACE_MODE: GLenum;\n    readonly CURRENT_PROGRAM: GLenum;\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\n    readonly CW: GLenum;\n    readonly DECR: GLenum;\n    readonly DECR_WRAP: GLenum;\n    readonly DELETE_STATUS: GLenum;\n    readonly DEPTH_ATTACHMENT: GLenum;\n    readonly DEPTH_BITS: GLenum;\n    readonly DEPTH_BUFFER_BIT: GLenum;\n    readonly DEPTH_CLEAR_VALUE: GLenum;\n    readonly DEPTH_COMPONENT: GLenum;\n    readonly DEPTH_COMPONENT16: GLenum;\n    readonly DEPTH_FUNC: GLenum;\n    readonly DEPTH_RANGE: GLenum;\n    readonly DEPTH_STENCIL: GLenum;\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\n    readonly DEPTH_TEST: GLenum;\n    readonly DEPTH_WRITEMASK: GLenum;\n    readonly DITHER: GLenum;\n    readonly DONT_CARE: GLenum;\n    readonly DST_ALPHA: GLenum;\n    readonly DST_COLOR: GLenum;\n    readonly DYNAMIC_DRAW: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\n    readonly EQUAL: GLenum;\n    readonly FASTEST: GLenum;\n    readonly FLOAT: GLenum;\n    readonly FLOAT_MAT2: GLenum;\n    readonly FLOAT_MAT3: GLenum;\n    readonly FLOAT_MAT4: GLenum;\n    readonly FLOAT_VEC2: GLenum;\n    readonly FLOAT_VEC3: GLenum;\n    readonly FLOAT_VEC4: GLenum;\n    readonly FRAGMENT_SHADER: GLenum;\n    readonly FRAMEBUFFER: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\n    readonly FRAMEBUFFER_BINDING: GLenum;\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\n    readonly FRONT: GLenum;\n    readonly FRONT_AND_BACK: GLenum;\n    readonly FRONT_FACE: GLenum;\n    readonly FUNC_ADD: GLenum;\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\n    readonly FUNC_SUBTRACT: GLenum;\n    readonly GENERATE_MIPMAP_HINT: GLenum;\n    readonly GEQUAL: GLenum;\n    readonly GREATER: GLenum;\n    readonly GREEN_BITS: GLenum;\n    readonly HIGH_FLOAT: GLenum;\n    readonly HIGH_INT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\n    readonly INCR: GLenum;\n    readonly INCR_WRAP: GLenum;\n    readonly INT: GLenum;\n    readonly INT_VEC2: GLenum;\n    readonly INT_VEC3: GLenum;\n    readonly INT_VEC4: GLenum;\n    readonly INVALID_ENUM: GLenum;\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\n    readonly INVALID_OPERATION: GLenum;\n    readonly INVALID_VALUE: GLenum;\n    readonly INVERT: GLenum;\n    readonly KEEP: GLenum;\n    readonly LEQUAL: GLenum;\n    readonly LESS: GLenum;\n    readonly LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\n    readonly LINES: GLenum;\n    readonly LINE_LOOP: GLenum;\n    readonly LINE_STRIP: GLenum;\n    readonly LINE_WIDTH: GLenum;\n    readonly LINK_STATUS: GLenum;\n    readonly LOW_FLOAT: GLenum;\n    readonly LOW_INT: GLenum;\n    readonly LUMINANCE: GLenum;\n    readonly LUMINANCE_ALPHA: GLenum;\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_TEXTURE_SIZE: GLenum;\n    readonly MAX_VARYING_VECTORS: GLenum;\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\n    readonly MAX_VIEWPORT_DIMS: GLenum;\n    readonly MEDIUM_FLOAT: GLenum;\n    readonly MEDIUM_INT: GLenum;\n    readonly MIRRORED_REPEAT: GLenum;\n    readonly NEAREST: GLenum;\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\n    readonly NEVER: GLenum;\n    readonly NICEST: GLenum;\n    readonly NONE: GLenum;\n    readonly NOTEQUAL: GLenum;\n    readonly NO_ERROR: GLenum;\n    readonly ONE: GLenum;\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\n    readonly ONE_MINUS_DST_COLOR: GLenum;\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\n    readonly OUT_OF_MEMORY: GLenum;\n    readonly PACK_ALIGNMENT: GLenum;\n    readonly POINTS: GLenum;\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\n    readonly POLYGON_OFFSET_FILL: GLenum;\n    readonly POLYGON_OFFSET_UNITS: GLenum;\n    readonly RED_BITS: GLenum;\n    readonly RENDERBUFFER: GLenum;\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\n    readonly RENDERBUFFER_BINDING: GLenum;\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\n    readonly RENDERBUFFER_HEIGHT: GLenum;\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\n    readonly RENDERBUFFER_WIDTH: GLenum;\n    readonly RENDERER: GLenum;\n    readonly REPEAT: GLenum;\n    readonly REPLACE: GLenum;\n    readonly RGB: GLenum;\n    readonly RGB565: GLenum;\n    readonly RGB5_A1: GLenum;\n    readonly RGBA: GLenum;\n    readonly RGBA4: GLenum;\n    readonly SAMPLER_2D: GLenum;\n    readonly SAMPLER_CUBE: GLenum;\n    readonly SAMPLES: GLenum;\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\n    readonly SAMPLE_BUFFERS: GLenum;\n    readonly SAMPLE_COVERAGE: GLenum;\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\n    readonly SCISSOR_BOX: GLenum;\n    readonly SCISSOR_TEST: GLenum;\n    readonly SHADER_TYPE: GLenum;\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\n    readonly SHORT: GLenum;\n    readonly SRC_ALPHA: GLenum;\n    readonly SRC_ALPHA_SATURATE: GLenum;\n    readonly SRC_COLOR: GLenum;\n    readonly STATIC_DRAW: GLenum;\n    readonly STENCIL_ATTACHMENT: GLenum;\n    readonly STENCIL_BACK_FAIL: GLenum;\n    readonly STENCIL_BACK_FUNC: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_BACK_REF: GLenum;\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\n    readonly STENCIL_BITS: GLenum;\n    readonly STENCIL_BUFFER_BIT: GLenum;\n    readonly STENCIL_CLEAR_VALUE: GLenum;\n    readonly STENCIL_FAIL: GLenum;\n    readonly STENCIL_FUNC: GLenum;\n    readonly STENCIL_INDEX8: GLenum;\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_REF: GLenum;\n    readonly STENCIL_TEST: GLenum;\n    readonly STENCIL_VALUE_MASK: GLenum;\n    readonly STENCIL_WRITEMASK: GLenum;\n    readonly STREAM_DRAW: GLenum;\n    readonly SUBPIXEL_BITS: GLenum;\n    readonly TEXTURE: GLenum;\n    readonly TEXTURE0: GLenum;\n    readonly TEXTURE1: GLenum;\n    readonly TEXTURE10: GLenum;\n    readonly TEXTURE11: GLenum;\n    readonly TEXTURE12: GLenum;\n    readonly TEXTURE13: GLenum;\n    readonly TEXTURE14: GLenum;\n    readonly TEXTURE15: GLenum;\n    readonly TEXTURE16: GLenum;\n    readonly TEXTURE17: GLenum;\n    readonly TEXTURE18: GLenum;\n    readonly TEXTURE19: GLenum;\n    readonly TEXTURE2: GLenum;\n    readonly TEXTURE20: GLenum;\n    readonly TEXTURE21: GLenum;\n    readonly TEXTURE22: GLenum;\n    readonly TEXTURE23: GLenum;\n    readonly TEXTURE24: GLenum;\n    readonly TEXTURE25: GLenum;\n    readonly TEXTURE26: GLenum;\n    readonly TEXTURE27: GLenum;\n    readonly TEXTURE28: GLenum;\n    readonly TEXTURE29: GLenum;\n    readonly TEXTURE3: GLenum;\n    readonly TEXTURE30: GLenum;\n    readonly TEXTURE31: GLenum;\n    readonly TEXTURE4: GLenum;\n    readonly TEXTURE5: GLenum;\n    readonly TEXTURE6: GLenum;\n    readonly TEXTURE7: GLenum;\n    readonly TEXTURE8: GLenum;\n    readonly TEXTURE9: GLenum;\n    readonly TEXTURE_2D: GLenum;\n    readonly TEXTURE_BINDING_2D: GLenum;\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\n    readonly TEXTURE_MAG_FILTER: GLenum;\n    readonly TEXTURE_MIN_FILTER: GLenum;\n    readonly TEXTURE_WRAP_S: GLenum;\n    readonly TEXTURE_WRAP_T: GLenum;\n    readonly TRIANGLES: GLenum;\n    readonly TRIANGLE_FAN: GLenum;\n    readonly TRIANGLE_STRIP: GLenum;\n    readonly UNPACK_ALIGNMENT: GLenum;\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\n    readonly UNSIGNED_BYTE: GLenum;\n    readonly UNSIGNED_INT: GLenum;\n    readonly UNSIGNED_SHORT: GLenum;\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\n    readonly VALIDATE_STATUS: GLenum;\n    readonly VENDOR: GLenum;\n    readonly VERSION: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\n    readonly VERTEX_SHADER: GLenum;\n    readonly VIEWPORT: GLenum;\n    readonly ZERO: GLenum;\n};\n\ninterface WebGL2RenderingContextBase {\n    beginQuery(target: GLenum, query: WebGLQuery): void;\n    beginTransformFeedback(primitiveMode: GLenum): void;\n    bindBufferBase(target: GLenum, index: GLuint, buffer: WebGLBuffer | null): void;\n    bindBufferRange(target: GLenum, index: GLuint, buffer: WebGLBuffer | null, offset: GLintptr, size: GLsizeiptr): void;\n    bindSampler(unit: GLuint, sampler: WebGLSampler | null): void;\n    bindTransformFeedback(target: GLenum, tf: WebGLTransformFeedback | null): void;\n    bindVertexArray(array: WebGLVertexArrayObject | null): void;\n    blitFramebuffer(srcX0: GLint, srcY0: GLint, srcX1: GLint, srcY1: GLint, dstX0: GLint, dstY0: GLint, dstX1: GLint, dstY1: GLint, mask: GLbitfield, filter: GLenum): void;\n    clearBufferfi(buffer: GLenum, drawbuffer: GLint, depth: GLfloat, stencil: GLint): void;\n    clearBufferfv(buffer: GLenum, drawbuffer: GLint, values: Float32List, srcOffset?: GLuint): void;\n    clearBufferiv(buffer: GLenum, drawbuffer: GLint, values: Int32List, srcOffset?: GLuint): void;\n    clearBufferuiv(buffer: GLenum, drawbuffer: GLint, values: Uint32List, srcOffset?: GLuint): void;\n    clientWaitSync(sync: WebGLSync, flags: GLbitfield, timeout: GLuint64): GLenum;\n    compressedTexImage3D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, imageSize: GLsizei, offset: GLintptr): void;\n    compressedTexImage3D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, srcData: ArrayBufferView, srcOffset?: GLuint, srcLengthOverride?: GLuint): void;\n    compressedTexSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, imageSize: GLsizei, offset: GLintptr): void;\n    compressedTexSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, srcData: ArrayBufferView, srcOffset?: GLuint, srcLengthOverride?: GLuint): void;\n    copyBufferSubData(readTarget: GLenum, writeTarget: GLenum, readOffset: GLintptr, writeOffset: GLintptr, size: GLsizeiptr): void;\n    copyTexSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    createQuery(): WebGLQuery | null;\n    createSampler(): WebGLSampler | null;\n    createTransformFeedback(): WebGLTransformFeedback | null;\n    createVertexArray(): WebGLVertexArrayObject | null;\n    deleteQuery(query: WebGLQuery | null): void;\n    deleteSampler(sampler: WebGLSampler | null): void;\n    deleteSync(sync: WebGLSync | null): void;\n    deleteTransformFeedback(tf: WebGLTransformFeedback | null): void;\n    deleteVertexArray(vertexArray: WebGLVertexArrayObject | null): void;\n    drawArraysInstanced(mode: GLenum, first: GLint, count: GLsizei, instanceCount: GLsizei): void;\n    drawBuffers(buffers: GLenum[]): void;\n    drawElementsInstanced(mode: GLenum, count: GLsizei, type: GLenum, offset: GLintptr, instanceCount: GLsizei): void;\n    drawRangeElements(mode: GLenum, start: GLuint, end: GLuint, count: GLsizei, type: GLenum, offset: GLintptr): void;\n    endQuery(target: GLenum): void;\n    endTransformFeedback(): void;\n    fenceSync(condition: GLenum, flags: GLbitfield): WebGLSync | null;\n    framebufferTextureLayer(target: GLenum, attachment: GLenum, texture: WebGLTexture | null, level: GLint, layer: GLint): void;\n    getActiveUniformBlockName(program: WebGLProgram, uniformBlockIndex: GLuint): string | null;\n    getActiveUniformBlockParameter(program: WebGLProgram, uniformBlockIndex: GLuint, pname: GLenum): any;\n    getActiveUniforms(program: WebGLProgram, uniformIndices: GLuint[], pname: GLenum): any;\n    getBufferSubData(target: GLenum, srcByteOffset: GLintptr, dstBuffer: ArrayBufferView, dstOffset?: GLuint, length?: GLuint): void;\n    getFragDataLocation(program: WebGLProgram, name: string): GLint;\n    getIndexedParameter(target: GLenum, index: GLuint): any;\n    getInternalformatParameter(target: GLenum, internalformat: GLenum, pname: GLenum): any;\n    getQuery(target: GLenum, pname: GLenum): WebGLQuery | null;\n    getQueryParameter(query: WebGLQuery, pname: GLenum): any;\n    getSamplerParameter(sampler: WebGLSampler, pname: GLenum): any;\n    getSyncParameter(sync: WebGLSync, pname: GLenum): any;\n    getTransformFeedbackVarying(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\n    getUniformBlockIndex(program: WebGLProgram, uniformBlockName: string): GLuint;\n    getUniformIndices(program: WebGLProgram, uniformNames: string[]): GLuint[] | null;\n    invalidateFramebuffer(target: GLenum, attachments: GLenum[]): void;\n    invalidateSubFramebuffer(target: GLenum, attachments: GLenum[], x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    isQuery(query: WebGLQuery | null): GLboolean;\n    isSampler(sampler: WebGLSampler | null): GLboolean;\n    isSync(sync: WebGLSync | null): GLboolean;\n    isTransformFeedback(tf: WebGLTransformFeedback | null): GLboolean;\n    isVertexArray(vertexArray: WebGLVertexArrayObject | null): GLboolean;\n    pauseTransformFeedback(): void;\n    readBuffer(src: GLenum): void;\n    renderbufferStorageMultisample(target: GLenum, samples: GLsizei, internalformat: GLenum, width: GLsizei, height: GLsizei): void;\n    resumeTransformFeedback(): void;\n    samplerParameterf(sampler: WebGLSampler, pname: GLenum, param: GLfloat): void;\n    samplerParameteri(sampler: WebGLSampler, pname: GLenum, param: GLint): void;\n    texImage3D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, format: GLenum, type: GLenum, pboOffset: GLintptr): void;\n    texImage3D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texImage3D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, format: GLenum, type: GLenum, srcData: ArrayBufferView | null): void;\n    texImage3D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, format: GLenum, type: GLenum, srcData: ArrayBufferView, srcOffset: GLuint): void;\n    texStorage2D(target: GLenum, levels: GLsizei, internalformat: GLenum, width: GLsizei, height: GLsizei): void;\n    texStorage3D(target: GLenum, levels: GLsizei, internalformat: GLenum, width: GLsizei, height: GLsizei, depth: GLsizei): void;\n    texSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, type: GLenum, pboOffset: GLintptr): void;\n    texSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, type: GLenum, srcData: ArrayBufferView | null, srcOffset?: GLuint): void;\n    transformFeedbackVaryings(program: WebGLProgram, varyings: string[], bufferMode: GLenum): void;\n    uniform1ui(location: WebGLUniformLocation | null, v0: GLuint): void;\n    uniform1uiv(location: WebGLUniformLocation | null, data: Uint32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2ui(location: WebGLUniformLocation | null, v0: GLuint, v1: GLuint): void;\n    uniform2uiv(location: WebGLUniformLocation | null, data: Uint32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3ui(location: WebGLUniformLocation | null, v0: GLuint, v1: GLuint, v2: GLuint): void;\n    uniform3uiv(location: WebGLUniformLocation | null, data: Uint32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4ui(location: WebGLUniformLocation | null, v0: GLuint, v1: GLuint, v2: GLuint, v3: GLuint): void;\n    uniform4uiv(location: WebGLUniformLocation | null, data: Uint32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformBlockBinding(program: WebGLProgram, uniformBlockIndex: GLuint, uniformBlockBinding: GLuint): void;\n    uniformMatrix2x3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2x4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3x2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3x4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4x2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4x3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    vertexAttribDivisor(index: GLuint, divisor: GLuint): void;\n    vertexAttribI4i(index: GLuint, x: GLint, y: GLint, z: GLint, w: GLint): void;\n    vertexAttribI4iv(index: GLuint, values: Int32List): void;\n    vertexAttribI4ui(index: GLuint, x: GLuint, y: GLuint, z: GLuint, w: GLuint): void;\n    vertexAttribI4uiv(index: GLuint, values: Uint32List): void;\n    vertexAttribIPointer(index: GLuint, size: GLint, type: GLenum, stride: GLsizei, offset: GLintptr): void;\n    waitSync(sync: WebGLSync, flags: GLbitfield, timeout: GLint64): void;\n    readonly ACTIVE_UNIFORM_BLOCKS: GLenum;\n    readonly ALREADY_SIGNALED: GLenum;\n    readonly ANY_SAMPLES_PASSED: GLenum;\n    readonly ANY_SAMPLES_PASSED_CONSERVATIVE: GLenum;\n    readonly COLOR: GLenum;\n    readonly COLOR_ATTACHMENT1: GLenum;\n    readonly COLOR_ATTACHMENT10: GLenum;\n    readonly COLOR_ATTACHMENT11: GLenum;\n    readonly COLOR_ATTACHMENT12: GLenum;\n    readonly COLOR_ATTACHMENT13: GLenum;\n    readonly COLOR_ATTACHMENT14: GLenum;\n    readonly COLOR_ATTACHMENT15: GLenum;\n    readonly COLOR_ATTACHMENT2: GLenum;\n    readonly COLOR_ATTACHMENT3: GLenum;\n    readonly COLOR_ATTACHMENT4: GLenum;\n    readonly COLOR_ATTACHMENT5: GLenum;\n    readonly COLOR_ATTACHMENT6: GLenum;\n    readonly COLOR_ATTACHMENT7: GLenum;\n    readonly COLOR_ATTACHMENT8: GLenum;\n    readonly COLOR_ATTACHMENT9: GLenum;\n    readonly COMPARE_REF_TO_TEXTURE: GLenum;\n    readonly CONDITION_SATISFIED: GLenum;\n    readonly COPY_READ_BUFFER: GLenum;\n    readonly COPY_READ_BUFFER_BINDING: GLenum;\n    readonly COPY_WRITE_BUFFER: GLenum;\n    readonly COPY_WRITE_BUFFER_BINDING: GLenum;\n    readonly CURRENT_QUERY: GLenum;\n    readonly DEPTH: GLenum;\n    readonly DEPTH24_STENCIL8: GLenum;\n    readonly DEPTH32F_STENCIL8: GLenum;\n    readonly DEPTH_COMPONENT24: GLenum;\n    readonly DEPTH_COMPONENT32F: GLenum;\n    readonly DRAW_BUFFER0: GLenum;\n    readonly DRAW_BUFFER1: GLenum;\n    readonly DRAW_BUFFER10: GLenum;\n    readonly DRAW_BUFFER11: GLenum;\n    readonly DRAW_BUFFER12: GLenum;\n    readonly DRAW_BUFFER13: GLenum;\n    readonly DRAW_BUFFER14: GLenum;\n    readonly DRAW_BUFFER15: GLenum;\n    readonly DRAW_BUFFER2: GLenum;\n    readonly DRAW_BUFFER3: GLenum;\n    readonly DRAW_BUFFER4: GLenum;\n    readonly DRAW_BUFFER5: GLenum;\n    readonly DRAW_BUFFER6: GLenum;\n    readonly DRAW_BUFFER7: GLenum;\n    readonly DRAW_BUFFER8: GLenum;\n    readonly DRAW_BUFFER9: GLenum;\n    readonly DRAW_FRAMEBUFFER: GLenum;\n    readonly DRAW_FRAMEBUFFER_BINDING: GLenum;\n    readonly DYNAMIC_COPY: GLenum;\n    readonly DYNAMIC_READ: GLenum;\n    readonly FLOAT_32_UNSIGNED_INT_24_8_REV: GLenum;\n    readonly FLOAT_MAT2x3: GLenum;\n    readonly FLOAT_MAT2x4: GLenum;\n    readonly FLOAT_MAT3x2: GLenum;\n    readonly FLOAT_MAT3x4: GLenum;\n    readonly FLOAT_MAT4x2: GLenum;\n    readonly FLOAT_MAT4x3: GLenum;\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_BLUE_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_GREEN_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_RED_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER: GLenum;\n    readonly FRAMEBUFFER_DEFAULT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MULTISAMPLE: GLenum;\n    readonly HALF_FLOAT: GLenum;\n    readonly INTERLEAVED_ATTRIBS: GLenum;\n    readonly INT_2_10_10_10_REV: GLenum;\n    readonly INT_SAMPLER_2D: GLenum;\n    readonly INT_SAMPLER_2D_ARRAY: GLenum;\n    readonly INT_SAMPLER_3D: GLenum;\n    readonly INT_SAMPLER_CUBE: GLenum;\n    readonly INVALID_INDEX: GLenum;\n    readonly MAX: GLenum;\n    readonly MAX_3D_TEXTURE_SIZE: GLenum;\n    readonly MAX_ARRAY_TEXTURE_LAYERS: GLenum;\n    readonly MAX_CLIENT_WAIT_TIMEOUT_WEBGL: GLenum;\n    readonly MAX_COLOR_ATTACHMENTS: GLenum;\n    readonly MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_COMBINED_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_DRAW_BUFFERS: GLenum;\n    readonly MAX_ELEMENTS_INDICES: GLenum;\n    readonly MAX_ELEMENTS_VERTICES: GLenum;\n    readonly MAX_ELEMENT_INDEX: GLenum;\n    readonly MAX_FRAGMENT_INPUT_COMPONENTS: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_PROGRAM_TEXEL_OFFSET: GLenum;\n    readonly MAX_SAMPLES: GLenum;\n    readonly MAX_SERVER_WAIT_TIMEOUT: GLenum;\n    readonly MAX_TEXTURE_LOD_BIAS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS: GLenum;\n    readonly MAX_UNIFORM_BLOCK_SIZE: GLenum;\n    readonly MAX_UNIFORM_BUFFER_BINDINGS: GLenum;\n    readonly MAX_VARYING_COMPONENTS: GLenum;\n    readonly MAX_VERTEX_OUTPUT_COMPONENTS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_COMPONENTS: GLenum;\n    readonly MIN: GLenum;\n    readonly MIN_PROGRAM_TEXEL_OFFSET: GLenum;\n    readonly OBJECT_TYPE: GLenum;\n    readonly PACK_ROW_LENGTH: GLenum;\n    readonly PACK_SKIP_PIXELS: GLenum;\n    readonly PACK_SKIP_ROWS: GLenum;\n    readonly PIXEL_PACK_BUFFER: GLenum;\n    readonly PIXEL_PACK_BUFFER_BINDING: GLenum;\n    readonly PIXEL_UNPACK_BUFFER: GLenum;\n    readonly PIXEL_UNPACK_BUFFER_BINDING: GLenum;\n    readonly QUERY_RESULT: GLenum;\n    readonly QUERY_RESULT_AVAILABLE: GLenum;\n    readonly R11F_G11F_B10F: GLenum;\n    readonly R16F: GLenum;\n    readonly R16I: GLenum;\n    readonly R16UI: GLenum;\n    readonly R32F: GLenum;\n    readonly R32I: GLenum;\n    readonly R32UI: GLenum;\n    readonly R8: GLenum;\n    readonly R8I: GLenum;\n    readonly R8UI: GLenum;\n    readonly R8_SNORM: GLenum;\n    readonly RASTERIZER_DISCARD: GLenum;\n    readonly READ_BUFFER: GLenum;\n    readonly READ_FRAMEBUFFER: GLenum;\n    readonly READ_FRAMEBUFFER_BINDING: GLenum;\n    readonly RED: GLenum;\n    readonly RED_INTEGER: GLenum;\n    readonly RENDERBUFFER_SAMPLES: GLenum;\n    readonly RG: GLenum;\n    readonly RG16F: GLenum;\n    readonly RG16I: GLenum;\n    readonly RG16UI: GLenum;\n    readonly RG32F: GLenum;\n    readonly RG32I: GLenum;\n    readonly RG32UI: GLenum;\n    readonly RG8: GLenum;\n    readonly RG8I: GLenum;\n    readonly RG8UI: GLenum;\n    readonly RG8_SNORM: GLenum;\n    readonly RGB10_A2: GLenum;\n    readonly RGB10_A2UI: GLenum;\n    readonly RGB16F: GLenum;\n    readonly RGB16I: GLenum;\n    readonly RGB16UI: GLenum;\n    readonly RGB32F: GLenum;\n    readonly RGB32I: GLenum;\n    readonly RGB32UI: GLenum;\n    readonly RGB8: GLenum;\n    readonly RGB8I: GLenum;\n    readonly RGB8UI: GLenum;\n    readonly RGB8_SNORM: GLenum;\n    readonly RGB9_E5: GLenum;\n    readonly RGBA16F: GLenum;\n    readonly RGBA16I: GLenum;\n    readonly RGBA16UI: GLenum;\n    readonly RGBA32F: GLenum;\n    readonly RGBA32I: GLenum;\n    readonly RGBA32UI: GLenum;\n    readonly RGBA8: GLenum;\n    readonly RGBA8I: GLenum;\n    readonly RGBA8UI: GLenum;\n    readonly RGBA8_SNORM: GLenum;\n    readonly RGBA_INTEGER: GLenum;\n    readonly RGB_INTEGER: GLenum;\n    readonly RG_INTEGER: GLenum;\n    readonly SAMPLER_2D_ARRAY: GLenum;\n    readonly SAMPLER_2D_ARRAY_SHADOW: GLenum;\n    readonly SAMPLER_2D_SHADOW: GLenum;\n    readonly SAMPLER_3D: GLenum;\n    readonly SAMPLER_BINDING: GLenum;\n    readonly SAMPLER_CUBE_SHADOW: GLenum;\n    readonly SEPARATE_ATTRIBS: GLenum;\n    readonly SIGNALED: GLenum;\n    readonly SIGNED_NORMALIZED: GLenum;\n    readonly SRGB: GLenum;\n    readonly SRGB8: GLenum;\n    readonly SRGB8_ALPHA8: GLenum;\n    readonly STATIC_COPY: GLenum;\n    readonly STATIC_READ: GLenum;\n    readonly STENCIL: GLenum;\n    readonly STREAM_COPY: GLenum;\n    readonly STREAM_READ: GLenum;\n    readonly SYNC_CONDITION: GLenum;\n    readonly SYNC_FENCE: GLenum;\n    readonly SYNC_FLAGS: GLenum;\n    readonly SYNC_FLUSH_COMMANDS_BIT: GLenum;\n    readonly SYNC_GPU_COMMANDS_COMPLETE: GLenum;\n    readonly SYNC_STATUS: GLenum;\n    readonly TEXTURE_2D_ARRAY: GLenum;\n    readonly TEXTURE_3D: GLenum;\n    readonly TEXTURE_BASE_LEVEL: GLenum;\n    readonly TEXTURE_BINDING_2D_ARRAY: GLenum;\n    readonly TEXTURE_BINDING_3D: GLenum;\n    readonly TEXTURE_COMPARE_FUNC: GLenum;\n    readonly TEXTURE_COMPARE_MODE: GLenum;\n    readonly TEXTURE_IMMUTABLE_FORMAT: GLenum;\n    readonly TEXTURE_IMMUTABLE_LEVELS: GLenum;\n    readonly TEXTURE_MAX_LEVEL: GLenum;\n    readonly TEXTURE_MAX_LOD: GLenum;\n    readonly TEXTURE_MIN_LOD: GLenum;\n    readonly TEXTURE_WRAP_R: GLenum;\n    readonly TIMEOUT_EXPIRED: GLenum;\n    readonly TIMEOUT_IGNORED: GLint64;\n    readonly TRANSFORM_FEEDBACK: GLenum;\n    readonly TRANSFORM_FEEDBACK_ACTIVE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BINDING: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_BINDING: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_MODE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_SIZE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_START: GLenum;\n    readonly TRANSFORM_FEEDBACK_PAUSED: GLenum;\n    readonly TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN: GLenum;\n    readonly TRANSFORM_FEEDBACK_VARYINGS: GLenum;\n    readonly UNIFORM_ARRAY_STRIDE: GLenum;\n    readonly UNIFORM_BLOCK_ACTIVE_UNIFORMS: GLenum;\n    readonly UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES: GLenum;\n    readonly UNIFORM_BLOCK_BINDING: GLenum;\n    readonly UNIFORM_BLOCK_DATA_SIZE: GLenum;\n    readonly UNIFORM_BLOCK_INDEX: GLenum;\n    readonly UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER: GLenum;\n    readonly UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER: GLenum;\n    readonly UNIFORM_BUFFER: GLenum;\n    readonly UNIFORM_BUFFER_BINDING: GLenum;\n    readonly UNIFORM_BUFFER_OFFSET_ALIGNMENT: GLenum;\n    readonly UNIFORM_BUFFER_SIZE: GLenum;\n    readonly UNIFORM_BUFFER_START: GLenum;\n    readonly UNIFORM_IS_ROW_MAJOR: GLenum;\n    readonly UNIFORM_MATRIX_STRIDE: GLenum;\n    readonly UNIFORM_OFFSET: GLenum;\n    readonly UNIFORM_SIZE: GLenum;\n    readonly UNIFORM_TYPE: GLenum;\n    readonly UNPACK_IMAGE_HEIGHT: GLenum;\n    readonly UNPACK_ROW_LENGTH: GLenum;\n    readonly UNPACK_SKIP_IMAGES: GLenum;\n    readonly UNPACK_SKIP_PIXELS: GLenum;\n    readonly UNPACK_SKIP_ROWS: GLenum;\n    readonly UNSIGNALED: GLenum;\n    readonly UNSIGNED_INT_10F_11F_11F_REV: GLenum;\n    readonly UNSIGNED_INT_24_8: GLenum;\n    readonly UNSIGNED_INT_2_10_10_10_REV: GLenum;\n    readonly UNSIGNED_INT_5_9_9_9_REV: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_2D: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_2D_ARRAY: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_3D: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_CUBE: GLenum;\n    readonly UNSIGNED_INT_VEC2: GLenum;\n    readonly UNSIGNED_INT_VEC3: GLenum;\n    readonly UNSIGNED_INT_VEC4: GLenum;\n    readonly UNSIGNED_NORMALIZED: GLenum;\n    readonly VERTEX_ARRAY_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_INTEGER: GLenum;\n    readonly WAIT_FAILED: GLenum;\n}\n\ninterface WebGL2RenderingContextOverloads {\n    bufferData(target: GLenum, size: GLsizeiptr, usage: GLenum): void;\n    bufferData(target: GLenum, srcData: BufferSource | null, usage: GLenum): void;\n    bufferData(target: GLenum, srcData: ArrayBufferView, usage: GLenum, srcOffset: GLuint, length?: GLuint): void;\n    bufferSubData(target: GLenum, dstByteOffset: GLintptr, srcData: BufferSource): void;\n    bufferSubData(target: GLenum, dstByteOffset: GLintptr, srcData: ArrayBufferView, srcOffset: GLuint, length?: GLuint): void;\n    compressedTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, border: GLint, imageSize: GLsizei, offset: GLintptr): void;\n    compressedTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, border: GLint, srcData: ArrayBufferView, srcOffset?: GLuint, srcLengthOverride?: GLuint): void;\n    compressedTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, imageSize: GLsizei, offset: GLintptr): void;\n    compressedTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, srcData: ArrayBufferView, srcOffset?: GLuint, srcLengthOverride?: GLuint): void;\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, dstData: ArrayBufferView | null): void;\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, offset: GLintptr): void;\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, dstData: ArrayBufferView, dstOffset: GLuint): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, pboOffset: GLintptr): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, srcData: ArrayBufferView, srcOffset: GLuint): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pboOffset: GLintptr): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, srcData: ArrayBufferView, srcOffset: GLuint): void;\n    uniform1fv(location: WebGLUniformLocation | null, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform1iv(location: WebGLUniformLocation | null, data: Int32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2fv(location: WebGLUniformLocation | null, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2iv(location: WebGLUniformLocation | null, data: Int32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3fv(location: WebGLUniformLocation | null, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3iv(location: WebGLUniformLocation | null, data: Int32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4fv(location: WebGLUniformLocation | null, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4iv(location: WebGLUniformLocation | null, data: Int32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n}\n\n/** Part of the WebGL API and represents the information returned by calling the WebGLRenderingContext.getActiveAttrib() and WebGLRenderingContext.getActiveUniform() methods. */\ninterface WebGLActiveInfo {\n    readonly name: string;\n    readonly size: GLint;\n    readonly type: GLenum;\n}\n\ndeclare var WebGLActiveInfo: {\n    prototype: WebGLActiveInfo;\n    new(): WebGLActiveInfo;\n};\n\n/** Part of the WebGL API and represents an opaque buffer object storing data such as vertices or colors. */\ninterface WebGLBuffer {\n}\n\ndeclare var WebGLBuffer: {\n    prototype: WebGLBuffer;\n    new(): WebGLBuffer;\n};\n\n/** The WebContextEvent interface is part of the WebGL API and is an interface for an event that is generated in response to a status change to the WebGL rendering context. */\ninterface WebGLContextEvent extends Event {\n    readonly statusMessage: string;\n}\n\ndeclare var WebGLContextEvent: {\n    prototype: WebGLContextEvent;\n    new(type: string, eventInit?: WebGLContextEventInit): WebGLContextEvent;\n};\n\n/** Part of the WebGL API and represents a collection of buffers that serve as a rendering destination. */\ninterface WebGLFramebuffer {\n}\n\ndeclare var WebGLFramebuffer: {\n    prototype: WebGLFramebuffer;\n    new(): WebGLFramebuffer;\n};\n\n/** The WebGLProgram is part of the WebGL API and is a combination of two compiled WebGLShaders consisting of a vertex shader and a fragment shader (both written in GLSL). */\ninterface WebGLProgram {\n}\n\ndeclare var WebGLProgram: {\n    prototype: WebGLProgram;\n    new(): WebGLProgram;\n};\n\ninterface WebGLQuery {\n}\n\ndeclare var WebGLQuery: {\n    prototype: WebGLQuery;\n    new(): WebGLQuery;\n};\n\n/** Part of the WebGL API and represents a buffer that can contain an image, or can be source or target of an rendering operation. */\ninterface WebGLRenderbuffer {\n}\n\ndeclare var WebGLRenderbuffer: {\n    prototype: WebGLRenderbuffer;\n    new(): WebGLRenderbuffer;\n};\n\n/** Provides an interface to the OpenGL ES 2.0 graphics rendering context for the drawing surface of an HTML <canvas> element. */\ninterface WebGLRenderingContext extends WebGLRenderingContextBase, WebGLRenderingContextOverloads {\n}\n\ndeclare var WebGLRenderingContext: {\n    prototype: WebGLRenderingContext;\n    new(): WebGLRenderingContext;\n    readonly ACTIVE_ATTRIBUTES: GLenum;\n    readonly ACTIVE_TEXTURE: GLenum;\n    readonly ACTIVE_UNIFORMS: GLenum;\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\n    readonly ALPHA: GLenum;\n    readonly ALPHA_BITS: GLenum;\n    readonly ALWAYS: GLenum;\n    readonly ARRAY_BUFFER: GLenum;\n    readonly ARRAY_BUFFER_BINDING: GLenum;\n    readonly ATTACHED_SHADERS: GLenum;\n    readonly BACK: GLenum;\n    readonly BLEND: GLenum;\n    readonly BLEND_COLOR: GLenum;\n    readonly BLEND_DST_ALPHA: GLenum;\n    readonly BLEND_DST_RGB: GLenum;\n    readonly BLEND_EQUATION: GLenum;\n    readonly BLEND_EQUATION_ALPHA: GLenum;\n    readonly BLEND_EQUATION_RGB: GLenum;\n    readonly BLEND_SRC_ALPHA: GLenum;\n    readonly BLEND_SRC_RGB: GLenum;\n    readonly BLUE_BITS: GLenum;\n    readonly BOOL: GLenum;\n    readonly BOOL_VEC2: GLenum;\n    readonly BOOL_VEC3: GLenum;\n    readonly BOOL_VEC4: GLenum;\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\n    readonly BUFFER_SIZE: GLenum;\n    readonly BUFFER_USAGE: GLenum;\n    readonly BYTE: GLenum;\n    readonly CCW: GLenum;\n    readonly CLAMP_TO_EDGE: GLenum;\n    readonly COLOR_ATTACHMENT0: GLenum;\n    readonly COLOR_BUFFER_BIT: GLenum;\n    readonly COLOR_CLEAR_VALUE: GLenum;\n    readonly COLOR_WRITEMASK: GLenum;\n    readonly COMPILE_STATUS: GLenum;\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\n    readonly CONSTANT_ALPHA: GLenum;\n    readonly CONSTANT_COLOR: GLenum;\n    readonly CONTEXT_LOST_WEBGL: GLenum;\n    readonly CULL_FACE: GLenum;\n    readonly CULL_FACE_MODE: GLenum;\n    readonly CURRENT_PROGRAM: GLenum;\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\n    readonly CW: GLenum;\n    readonly DECR: GLenum;\n    readonly DECR_WRAP: GLenum;\n    readonly DELETE_STATUS: GLenum;\n    readonly DEPTH_ATTACHMENT: GLenum;\n    readonly DEPTH_BITS: GLenum;\n    readonly DEPTH_BUFFER_BIT: GLenum;\n    readonly DEPTH_CLEAR_VALUE: GLenum;\n    readonly DEPTH_COMPONENT: GLenum;\n    readonly DEPTH_COMPONENT16: GLenum;\n    readonly DEPTH_FUNC: GLenum;\n    readonly DEPTH_RANGE: GLenum;\n    readonly DEPTH_STENCIL: GLenum;\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\n    readonly DEPTH_TEST: GLenum;\n    readonly DEPTH_WRITEMASK: GLenum;\n    readonly DITHER: GLenum;\n    readonly DONT_CARE: GLenum;\n    readonly DST_ALPHA: GLenum;\n    readonly DST_COLOR: GLenum;\n    readonly DYNAMIC_DRAW: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\n    readonly EQUAL: GLenum;\n    readonly FASTEST: GLenum;\n    readonly FLOAT: GLenum;\n    readonly FLOAT_MAT2: GLenum;\n    readonly FLOAT_MAT3: GLenum;\n    readonly FLOAT_MAT4: GLenum;\n    readonly FLOAT_VEC2: GLenum;\n    readonly FLOAT_VEC3: GLenum;\n    readonly FLOAT_VEC4: GLenum;\n    readonly FRAGMENT_SHADER: GLenum;\n    readonly FRAMEBUFFER: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\n    readonly FRAMEBUFFER_BINDING: GLenum;\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\n    readonly FRONT: GLenum;\n    readonly FRONT_AND_BACK: GLenum;\n    readonly FRONT_FACE: GLenum;\n    readonly FUNC_ADD: GLenum;\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\n    readonly FUNC_SUBTRACT: GLenum;\n    readonly GENERATE_MIPMAP_HINT: GLenum;\n    readonly GEQUAL: GLenum;\n    readonly GREATER: GLenum;\n    readonly GREEN_BITS: GLenum;\n    readonly HIGH_FLOAT: GLenum;\n    readonly HIGH_INT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\n    readonly INCR: GLenum;\n    readonly INCR_WRAP: GLenum;\n    readonly INT: GLenum;\n    readonly INT_VEC2: GLenum;\n    readonly INT_VEC3: GLenum;\n    readonly INT_VEC4: GLenum;\n    readonly INVALID_ENUM: GLenum;\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\n    readonly INVALID_OPERATION: GLenum;\n    readonly INVALID_VALUE: GLenum;\n    readonly INVERT: GLenum;\n    readonly KEEP: GLenum;\n    readonly LEQUAL: GLenum;\n    readonly LESS: GLenum;\n    readonly LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\n    readonly LINES: GLenum;\n    readonly LINE_LOOP: GLenum;\n    readonly LINE_STRIP: GLenum;\n    readonly LINE_WIDTH: GLenum;\n    readonly LINK_STATUS: GLenum;\n    readonly LOW_FLOAT: GLenum;\n    readonly LOW_INT: GLenum;\n    readonly LUMINANCE: GLenum;\n    readonly LUMINANCE_ALPHA: GLenum;\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_TEXTURE_SIZE: GLenum;\n    readonly MAX_VARYING_VECTORS: GLenum;\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\n    readonly MAX_VIEWPORT_DIMS: GLenum;\n    readonly MEDIUM_FLOAT: GLenum;\n    readonly MEDIUM_INT: GLenum;\n    readonly MIRRORED_REPEAT: GLenum;\n    readonly NEAREST: GLenum;\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\n    readonly NEVER: GLenum;\n    readonly NICEST: GLenum;\n    readonly NONE: GLenum;\n    readonly NOTEQUAL: GLenum;\n    readonly NO_ERROR: GLenum;\n    readonly ONE: GLenum;\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\n    readonly ONE_MINUS_DST_COLOR: GLenum;\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\n    readonly OUT_OF_MEMORY: GLenum;\n    readonly PACK_ALIGNMENT: GLenum;\n    readonly POINTS: GLenum;\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\n    readonly POLYGON_OFFSET_FILL: GLenum;\n    readonly POLYGON_OFFSET_UNITS: GLenum;\n    readonly RED_BITS: GLenum;\n    readonly RENDERBUFFER: GLenum;\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\n    readonly RENDERBUFFER_BINDING: GLenum;\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\n    readonly RENDERBUFFER_HEIGHT: GLenum;\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\n    readonly RENDERBUFFER_WIDTH: GLenum;\n    readonly RENDERER: GLenum;\n    readonly REPEAT: GLenum;\n    readonly REPLACE: GLenum;\n    readonly RGB: GLenum;\n    readonly RGB565: GLenum;\n    readonly RGB5_A1: GLenum;\n    readonly RGBA: GLenum;\n    readonly RGBA4: GLenum;\n    readonly SAMPLER_2D: GLenum;\n    readonly SAMPLER_CUBE: GLenum;\n    readonly SAMPLES: GLenum;\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\n    readonly SAMPLE_BUFFERS: GLenum;\n    readonly SAMPLE_COVERAGE: GLenum;\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\n    readonly SCISSOR_BOX: GLenum;\n    readonly SCISSOR_TEST: GLenum;\n    readonly SHADER_TYPE: GLenum;\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\n    readonly SHORT: GLenum;\n    readonly SRC_ALPHA: GLenum;\n    readonly SRC_ALPHA_SATURATE: GLenum;\n    readonly SRC_COLOR: GLenum;\n    readonly STATIC_DRAW: GLenum;\n    readonly STENCIL_ATTACHMENT: GLenum;\n    readonly STENCIL_BACK_FAIL: GLenum;\n    readonly STENCIL_BACK_FUNC: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_BACK_REF: GLenum;\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\n    readonly STENCIL_BITS: GLenum;\n    readonly STENCIL_BUFFER_BIT: GLenum;\n    readonly STENCIL_CLEAR_VALUE: GLenum;\n    readonly STENCIL_FAIL: GLenum;\n    readonly STENCIL_FUNC: GLenum;\n    readonly STENCIL_INDEX8: GLenum;\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_REF: GLenum;\n    readonly STENCIL_TEST: GLenum;\n    readonly STENCIL_VALUE_MASK: GLenum;\n    readonly STENCIL_WRITEMASK: GLenum;\n    readonly STREAM_DRAW: GLenum;\n    readonly SUBPIXEL_BITS: GLenum;\n    readonly TEXTURE: GLenum;\n    readonly TEXTURE0: GLenum;\n    readonly TEXTURE1: GLenum;\n    readonly TEXTURE10: GLenum;\n    readonly TEXTURE11: GLenum;\n    readonly TEXTURE12: GLenum;\n    readonly TEXTURE13: GLenum;\n    readonly TEXTURE14: GLenum;\n    readonly TEXTURE15: GLenum;\n    readonly TEXTURE16: GLenum;\n    readonly TEXTURE17: GLenum;\n    readonly TEXTURE18: GLenum;\n    readonly TEXTURE19: GLenum;\n    readonly TEXTURE2: GLenum;\n    readonly TEXTURE20: GLenum;\n    readonly TEXTURE21: GLenum;\n    readonly TEXTURE22: GLenum;\n    readonly TEXTURE23: GLenum;\n    readonly TEXTURE24: GLenum;\n    readonly TEXTURE25: GLenum;\n    readonly TEXTURE26: GLenum;\n    readonly TEXTURE27: GLenum;\n    readonly TEXTURE28: GLenum;\n    readonly TEXTURE29: GLenum;\n    readonly TEXTURE3: GLenum;\n    readonly TEXTURE30: GLenum;\n    readonly TEXTURE31: GLenum;\n    readonly TEXTURE4: GLenum;\n    readonly TEXTURE5: GLenum;\n    readonly TEXTURE6: GLenum;\n    readonly TEXTURE7: GLenum;\n    readonly TEXTURE8: GLenum;\n    readonly TEXTURE9: GLenum;\n    readonly TEXTURE_2D: GLenum;\n    readonly TEXTURE_BINDING_2D: GLenum;\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\n    readonly TEXTURE_MAG_FILTER: GLenum;\n    readonly TEXTURE_MIN_FILTER: GLenum;\n    readonly TEXTURE_WRAP_S: GLenum;\n    readonly TEXTURE_WRAP_T: GLenum;\n    readonly TRIANGLES: GLenum;\n    readonly TRIANGLE_FAN: GLenum;\n    readonly TRIANGLE_STRIP: GLenum;\n    readonly UNPACK_ALIGNMENT: GLenum;\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\n    readonly UNSIGNED_BYTE: GLenum;\n    readonly UNSIGNED_INT: GLenum;\n    readonly UNSIGNED_SHORT: GLenum;\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\n    readonly VALIDATE_STATUS: GLenum;\n    readonly VENDOR: GLenum;\n    readonly VERSION: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\n    readonly VERTEX_SHADER: GLenum;\n    readonly VIEWPORT: GLenum;\n    readonly ZERO: GLenum;\n};\n\ninterface WebGLRenderingContextBase {\n    readonly canvas: HTMLCanvasElement;\n    readonly drawingBufferHeight: GLsizei;\n    readonly drawingBufferWidth: GLsizei;\n    activeTexture(texture: GLenum): void;\n    attachShader(program: WebGLProgram, shader: WebGLShader): void;\n    bindAttribLocation(program: WebGLProgram, index: GLuint, name: string): void;\n    bindBuffer(target: GLenum, buffer: WebGLBuffer | null): void;\n    bindFramebuffer(target: GLenum, framebuffer: WebGLFramebuffer | null): void;\n    bindRenderbuffer(target: GLenum, renderbuffer: WebGLRenderbuffer | null): void;\n    bindTexture(target: GLenum, texture: WebGLTexture | null): void;\n    blendColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;\n    blendEquation(mode: GLenum): void;\n    blendEquationSeparate(modeRGB: GLenum, modeAlpha: GLenum): void;\n    blendFunc(sfactor: GLenum, dfactor: GLenum): void;\n    blendFuncSeparate(srcRGB: GLenum, dstRGB: GLenum, srcAlpha: GLenum, dstAlpha: GLenum): void;\n    checkFramebufferStatus(target: GLenum): GLenum;\n    clear(mask: GLbitfield): void;\n    clearColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;\n    clearDepth(depth: GLclampf): void;\n    clearStencil(s: GLint): void;\n    colorMask(red: GLboolean, green: GLboolean, blue: GLboolean, alpha: GLboolean): void;\n    compileShader(shader: WebGLShader): void;\n    copyTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, x: GLint, y: GLint, width: GLsizei, height: GLsizei, border: GLint): void;\n    copyTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    createBuffer(): WebGLBuffer | null;\n    createFramebuffer(): WebGLFramebuffer | null;\n    createProgram(): WebGLProgram | null;\n    createRenderbuffer(): WebGLRenderbuffer | null;\n    createShader(type: GLenum): WebGLShader | null;\n    createTexture(): WebGLTexture | null;\n    cullFace(mode: GLenum): void;\n    deleteBuffer(buffer: WebGLBuffer | null): void;\n    deleteFramebuffer(framebuffer: WebGLFramebuffer | null): void;\n    deleteProgram(program: WebGLProgram | null): void;\n    deleteRenderbuffer(renderbuffer: WebGLRenderbuffer | null): void;\n    deleteShader(shader: WebGLShader | null): void;\n    deleteTexture(texture: WebGLTexture | null): void;\n    depthFunc(func: GLenum): void;\n    depthMask(flag: GLboolean): void;\n    depthRange(zNear: GLclampf, zFar: GLclampf): void;\n    detachShader(program: WebGLProgram, shader: WebGLShader): void;\n    disable(cap: GLenum): void;\n    disableVertexAttribArray(index: GLuint): void;\n    drawArrays(mode: GLenum, first: GLint, count: GLsizei): void;\n    drawElements(mode: GLenum, count: GLsizei, type: GLenum, offset: GLintptr): void;\n    enable(cap: GLenum): void;\n    enableVertexAttribArray(index: GLuint): void;\n    finish(): void;\n    flush(): void;\n    framebufferRenderbuffer(target: GLenum, attachment: GLenum, renderbuffertarget: GLenum, renderbuffer: WebGLRenderbuffer | null): void;\n    framebufferTexture2D(target: GLenum, attachment: GLenum, textarget: GLenum, texture: WebGLTexture | null, level: GLint): void;\n    frontFace(mode: GLenum): void;\n    generateMipmap(target: GLenum): void;\n    getActiveAttrib(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\n    getActiveUniform(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\n    getAttachedShaders(program: WebGLProgram): WebGLShader[] | null;\n    getAttribLocation(program: WebGLProgram, name: string): GLint;\n    getBufferParameter(target: GLenum, pname: GLenum): any;\n    getContextAttributes(): WebGLContextAttributes | null;\n    getError(): GLenum;\n    getExtension(extensionName: "EXT_blend_minmax"): EXT_blend_minmax | null;\n    getExtension(extensionName: "EXT_color_buffer_float"): EXT_color_buffer_float | null;\n    getExtension(extensionName: "EXT_color_buffer_half_float"): EXT_color_buffer_half_float | null;\n    getExtension(extensionName: "EXT_float_blend"): EXT_float_blend | null;\n    getExtension(extensionName: "EXT_texture_filter_anisotropic"): EXT_texture_filter_anisotropic | null;\n    getExtension(extensionName: "EXT_frag_depth"): EXT_frag_depth | null;\n    getExtension(extensionName: "EXT_shader_texture_lod"): EXT_shader_texture_lod | null;\n    getExtension(extensionName: "EXT_sRGB"): EXT_sRGB | null;\n    getExtension(extensionName: "KHR_parallel_shader_compile"): KHR_parallel_shader_compile | null;\n    getExtension(extensionName: "OES_vertex_array_object"): OES_vertex_array_object | null;\n    getExtension(extensionName: "OVR_multiview2"): OVR_multiview2 | null;\n    getExtension(extensionName: "WEBGL_color_buffer_float"): WEBGL_color_buffer_float | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_astc"): WEBGL_compressed_texture_astc | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_etc"): WEBGL_compressed_texture_etc | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_etc1"): WEBGL_compressed_texture_etc1 | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_pvrtc"): WEBGL_compressed_texture_pvrtc | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_s3tc_srgb"): WEBGL_compressed_texture_s3tc_srgb | null;\n    getExtension(extensionName: "WEBGL_debug_shaders"): WEBGL_debug_shaders | null;\n    getExtension(extensionName: "WEBGL_draw_buffers"): WEBGL_draw_buffers | null;\n    getExtension(extensionName: "WEBGL_lose_context"): WEBGL_lose_context | null;\n    getExtension(extensionName: "WEBGL_depth_texture"): WEBGL_depth_texture | null;\n    getExtension(extensionName: "WEBGL_debug_renderer_info"): WEBGL_debug_renderer_info | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_s3tc"): WEBGL_compressed_texture_s3tc | null;\n    getExtension(extensionName: "OES_texture_half_float_linear"): OES_texture_half_float_linear | null;\n    getExtension(extensionName: "OES_texture_half_float"): OES_texture_half_float | null;\n    getExtension(extensionName: "OES_texture_float_linear"): OES_texture_float_linear | null;\n    getExtension(extensionName: "OES_texture_float"): OES_texture_float | null;\n    getExtension(extensionName: "OES_standard_derivatives"): OES_standard_derivatives | null;\n    getExtension(extensionName: "OES_element_index_uint"): OES_element_index_uint | null;\n    getExtension(extensionName: "ANGLE_instanced_arrays"): ANGLE_instanced_arrays | null;\n    getExtension(name: string): any;\n    getFramebufferAttachmentParameter(target: GLenum, attachment: GLenum, pname: GLenum): any;\n    getParameter(pname: GLenum): any;\n    getProgramInfoLog(program: WebGLProgram): string | null;\n    getProgramParameter(program: WebGLProgram, pname: GLenum): any;\n    getRenderbufferParameter(target: GLenum, pname: GLenum): any;\n    getShaderInfoLog(shader: WebGLShader): string | null;\n    getShaderParameter(shader: WebGLShader, pname: GLenum): any;\n    getShaderPrecisionFormat(shadertype: GLenum, precisiontype: GLenum): WebGLShaderPrecisionFormat | null;\n    getShaderSource(shader: WebGLShader): string | null;\n    getSupportedExtensions(): string[] | null;\n    getTexParameter(target: GLenum, pname: GLenum): any;\n    getUniform(program: WebGLProgram, location: WebGLUniformLocation): any;\n    getUniformLocation(program: WebGLProgram, name: string): WebGLUniformLocation | null;\n    getVertexAttrib(index: GLuint, pname: GLenum): any;\n    getVertexAttribOffset(index: GLuint, pname: GLenum): GLintptr;\n    hint(target: GLenum, mode: GLenum): void;\n    isBuffer(buffer: WebGLBuffer | null): GLboolean;\n    isContextLost(): boolean;\n    isEnabled(cap: GLenum): GLboolean;\n    isFramebuffer(framebuffer: WebGLFramebuffer | null): GLboolean;\n    isProgram(program: WebGLProgram | null): GLboolean;\n    isRenderbuffer(renderbuffer: WebGLRenderbuffer | null): GLboolean;\n    isShader(shader: WebGLShader | null): GLboolean;\n    isTexture(texture: WebGLTexture | null): GLboolean;\n    lineWidth(width: GLfloat): void;\n    linkProgram(program: WebGLProgram): void;\n    pixelStorei(pname: GLenum, param: GLint | GLboolean): void;\n    polygonOffset(factor: GLfloat, units: GLfloat): void;\n    renderbufferStorage(target: GLenum, internalformat: GLenum, width: GLsizei, height: GLsizei): void;\n    sampleCoverage(value: GLclampf, invert: GLboolean): void;\n    scissor(x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    shaderSource(shader: WebGLShader, source: string): void;\n    stencilFunc(func: GLenum, ref: GLint, mask: GLuint): void;\n    stencilFuncSeparate(face: GLenum, func: GLenum, ref: GLint, mask: GLuint): void;\n    stencilMask(mask: GLuint): void;\n    stencilMaskSeparate(face: GLenum, mask: GLuint): void;\n    stencilOp(fail: GLenum, zfail: GLenum, zpass: GLenum): void;\n    stencilOpSeparate(face: GLenum, fail: GLenum, zfail: GLenum, zpass: GLenum): void;\n    texParameterf(target: GLenum, pname: GLenum, param: GLfloat): void;\n    texParameteri(target: GLenum, pname: GLenum, param: GLint): void;\n    uniform1f(location: WebGLUniformLocation | null, x: GLfloat): void;\n    uniform1i(location: WebGLUniformLocation | null, x: GLint): void;\n    uniform2f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat): void;\n    uniform2i(location: WebGLUniformLocation | null, x: GLint, y: GLint): void;\n    uniform3f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat, z: GLfloat): void;\n    uniform3i(location: WebGLUniformLocation | null, x: GLint, y: GLint, z: GLint): void;\n    uniform4f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat, z: GLfloat, w: GLfloat): void;\n    uniform4i(location: WebGLUniformLocation | null, x: GLint, y: GLint, z: GLint, w: GLint): void;\n    useProgram(program: WebGLProgram | null): void;\n    validateProgram(program: WebGLProgram): void;\n    vertexAttrib1f(index: GLuint, x: GLfloat): void;\n    vertexAttrib1fv(index: GLuint, values: Float32List): void;\n    vertexAttrib2f(index: GLuint, x: GLfloat, y: GLfloat): void;\n    vertexAttrib2fv(index: GLuint, values: Float32List): void;\n    vertexAttrib3f(index: GLuint, x: GLfloat, y: GLfloat, z: GLfloat): void;\n    vertexAttrib3fv(index: GLuint, values: Float32List): void;\n    vertexAttrib4f(index: GLuint, x: GLfloat, y: GLfloat, z: GLfloat, w: GLfloat): void;\n    vertexAttrib4fv(index: GLuint, values: Float32List): void;\n    vertexAttribPointer(index: GLuint, size: GLint, type: GLenum, normalized: GLboolean, stride: GLsizei, offset: GLintptr): void;\n    viewport(x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    readonly ACTIVE_ATTRIBUTES: GLenum;\n    readonly ACTIVE_TEXTURE: GLenum;\n    readonly ACTIVE_UNIFORMS: GLenum;\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\n    readonly ALPHA: GLenum;\n    readonly ALPHA_BITS: GLenum;\n    readonly ALWAYS: GLenum;\n    readonly ARRAY_BUFFER: GLenum;\n    readonly ARRAY_BUFFER_BINDING: GLenum;\n    readonly ATTACHED_SHADERS: GLenum;\n    readonly BACK: GLenum;\n    readonly BLEND: GLenum;\n    readonly BLEND_COLOR: GLenum;\n    readonly BLEND_DST_ALPHA: GLenum;\n    readonly BLEND_DST_RGB: GLenum;\n    readonly BLEND_EQUATION: GLenum;\n    readonly BLEND_EQUATION_ALPHA: GLenum;\n    readonly BLEND_EQUATION_RGB: GLenum;\n    readonly BLEND_SRC_ALPHA: GLenum;\n    readonly BLEND_SRC_RGB: GLenum;\n    readonly BLUE_BITS: GLenum;\n    readonly BOOL: GLenum;\n    readonly BOOL_VEC2: GLenum;\n    readonly BOOL_VEC3: GLenum;\n    readonly BOOL_VEC4: GLenum;\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\n    readonly BUFFER_SIZE: GLenum;\n    readonly BUFFER_USAGE: GLenum;\n    readonly BYTE: GLenum;\n    readonly CCW: GLenum;\n    readonly CLAMP_TO_EDGE: GLenum;\n    readonly COLOR_ATTACHMENT0: GLenum;\n    readonly COLOR_BUFFER_BIT: GLenum;\n    readonly COLOR_CLEAR_VALUE: GLenum;\n    readonly COLOR_WRITEMASK: GLenum;\n    readonly COMPILE_STATUS: GLenum;\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\n    readonly CONSTANT_ALPHA: GLenum;\n    readonly CONSTANT_COLOR: GLenum;\n    readonly CONTEXT_LOST_WEBGL: GLenum;\n    readonly CULL_FACE: GLenum;\n    readonly CULL_FACE_MODE: GLenum;\n    readonly CURRENT_PROGRAM: GLenum;\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\n    readonly CW: GLenum;\n    readonly DECR: GLenum;\n    readonly DECR_WRAP: GLenum;\n    readonly DELETE_STATUS: GLenum;\n    readonly DEPTH_ATTACHMENT: GLenum;\n    readonly DEPTH_BITS: GLenum;\n    readonly DEPTH_BUFFER_BIT: GLenum;\n    readonly DEPTH_CLEAR_VALUE: GLenum;\n    readonly DEPTH_COMPONENT: GLenum;\n    readonly DEPTH_COMPONENT16: GLenum;\n    readonly DEPTH_FUNC: GLenum;\n    readonly DEPTH_RANGE: GLenum;\n    readonly DEPTH_STENCIL: GLenum;\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\n    readonly DEPTH_TEST: GLenum;\n    readonly DEPTH_WRITEMASK: GLenum;\n    readonly DITHER: GLenum;\n    readonly DONT_CARE: GLenum;\n    readonly DST_ALPHA: GLenum;\n    readonly DST_COLOR: GLenum;\n    readonly DYNAMIC_DRAW: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\n    readonly EQUAL: GLenum;\n    readonly FASTEST: GLenum;\n    readonly FLOAT: GLenum;\n    readonly FLOAT_MAT2: GLenum;\n    readonly FLOAT_MAT3: GLenum;\n    readonly FLOAT_MAT4: GLenum;\n    readonly FLOAT_VEC2: GLenum;\n    readonly FLOAT_VEC3: GLenum;\n    readonly FLOAT_VEC4: GLenum;\n    readonly FRAGMENT_SHADER: GLenum;\n    readonly FRAMEBUFFER: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\n    readonly FRAMEBUFFER_BINDING: GLenum;\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\n    readonly FRONT: GLenum;\n    readonly FRONT_AND_BACK: GLenum;\n    readonly FRONT_FACE: GLenum;\n    readonly FUNC_ADD: GLenum;\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\n    readonly FUNC_SUBTRACT: GLenum;\n    readonly GENERATE_MIPMAP_HINT: GLenum;\n    readonly GEQUAL: GLenum;\n    readonly GREATER: GLenum;\n    readonly GREEN_BITS: GLenum;\n    readonly HIGH_FLOAT: GLenum;\n    readonly HIGH_INT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\n    readonly INCR: GLenum;\n    readonly INCR_WRAP: GLenum;\n    readonly INT: GLenum;\n    readonly INT_VEC2: GLenum;\n    readonly INT_VEC3: GLenum;\n    readonly INT_VEC4: GLenum;\n    readonly INVALID_ENUM: GLenum;\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\n    readonly INVALID_OPERATION: GLenum;\n    readonly INVALID_VALUE: GLenum;\n    readonly INVERT: GLenum;\n    readonly KEEP: GLenum;\n    readonly LEQUAL: GLenum;\n    readonly LESS: GLenum;\n    readonly LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\n    readonly LINES: GLenum;\n    readonly LINE_LOOP: GLenum;\n    readonly LINE_STRIP: GLenum;\n    readonly LINE_WIDTH: GLenum;\n    readonly LINK_STATUS: GLenum;\n    readonly LOW_FLOAT: GLenum;\n    readonly LOW_INT: GLenum;\n    readonly LUMINANCE: GLenum;\n    readonly LUMINANCE_ALPHA: GLenum;\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_TEXTURE_SIZE: GLenum;\n    readonly MAX_VARYING_VECTORS: GLenum;\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\n    readonly MAX_VIEWPORT_DIMS: GLenum;\n    readonly MEDIUM_FLOAT: GLenum;\n    readonly MEDIUM_INT: GLenum;\n    readonly MIRRORED_REPEAT: GLenum;\n    readonly NEAREST: GLenum;\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\n    readonly NEVER: GLenum;\n    readonly NICEST: GLenum;\n    readonly NONE: GLenum;\n    readonly NOTEQUAL: GLenum;\n    readonly NO_ERROR: GLenum;\n    readonly ONE: GLenum;\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\n    readonly ONE_MINUS_DST_COLOR: GLenum;\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\n    readonly OUT_OF_MEMORY: GLenum;\n    readonly PACK_ALIGNMENT: GLenum;\n    readonly POINTS: GLenum;\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\n    readonly POLYGON_OFFSET_FILL: GLenum;\n    readonly POLYGON_OFFSET_UNITS: GLenum;\n    readonly RED_BITS: GLenum;\n    readonly RENDERBUFFER: GLenum;\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\n    readonly RENDERBUFFER_BINDING: GLenum;\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\n    readonly RENDERBUFFER_HEIGHT: GLenum;\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\n    readonly RENDERBUFFER_WIDTH: GLenum;\n    readonly RENDERER: GLenum;\n    readonly REPEAT: GLenum;\n    readonly REPLACE: GLenum;\n    readonly RGB: GLenum;\n    readonly RGB565: GLenum;\n    readonly RGB5_A1: GLenum;\n    readonly RGBA: GLenum;\n    readonly RGBA4: GLenum;\n    readonly SAMPLER_2D: GLenum;\n    readonly SAMPLER_CUBE: GLenum;\n    readonly SAMPLES: GLenum;\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\n    readonly SAMPLE_BUFFERS: GLenum;\n    readonly SAMPLE_COVERAGE: GLenum;\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\n    readonly SCISSOR_BOX: GLenum;\n    readonly SCISSOR_TEST: GLenum;\n    readonly SHADER_TYPE: GLenum;\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\n    readonly SHORT: GLenum;\n    readonly SRC_ALPHA: GLenum;\n    readonly SRC_ALPHA_SATURATE: GLenum;\n    readonly SRC_COLOR: GLenum;\n    readonly STATIC_DRAW: GLenum;\n    readonly STENCIL_ATTACHMENT: GLenum;\n    readonly STENCIL_BACK_FAIL: GLenum;\n    readonly STENCIL_BACK_FUNC: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_BACK_REF: GLenum;\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\n    readonly STENCIL_BITS: GLenum;\n    readonly STENCIL_BUFFER_BIT: GLenum;\n    readonly STENCIL_CLEAR_VALUE: GLenum;\n    readonly STENCIL_FAIL: GLenum;\n    readonly STENCIL_FUNC: GLenum;\n    readonly STENCIL_INDEX8: GLenum;\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_REF: GLenum;\n    readonly STENCIL_TEST: GLenum;\n    readonly STENCIL_VALUE_MASK: GLenum;\n    readonly STENCIL_WRITEMASK: GLenum;\n    readonly STREAM_DRAW: GLenum;\n    readonly SUBPIXEL_BITS: GLenum;\n    readonly TEXTURE: GLenum;\n    readonly TEXTURE0: GLenum;\n    readonly TEXTURE1: GLenum;\n    readonly TEXTURE10: GLenum;\n    readonly TEXTURE11: GLenum;\n    readonly TEXTURE12: GLenum;\n    readonly TEXTURE13: GLenum;\n    readonly TEXTURE14: GLenum;\n    readonly TEXTURE15: GLenum;\n    readonly TEXTURE16: GLenum;\n    readonly TEXTURE17: GLenum;\n    readonly TEXTURE18: GLenum;\n    readonly TEXTURE19: GLenum;\n    readonly TEXTURE2: GLenum;\n    readonly TEXTURE20: GLenum;\n    readonly TEXTURE21: GLenum;\n    readonly TEXTURE22: GLenum;\n    readonly TEXTURE23: GLenum;\n    readonly TEXTURE24: GLenum;\n    readonly TEXTURE25: GLenum;\n    readonly TEXTURE26: GLenum;\n    readonly TEXTURE27: GLenum;\n    readonly TEXTURE28: GLenum;\n    readonly TEXTURE29: GLenum;\n    readonly TEXTURE3: GLenum;\n    readonly TEXTURE30: GLenum;\n    readonly TEXTURE31: GLenum;\n    readonly TEXTURE4: GLenum;\n    readonly TEXTURE5: GLenum;\n    readonly TEXTURE6: GLenum;\n    readonly TEXTURE7: GLenum;\n    readonly TEXTURE8: GLenum;\n    readonly TEXTURE9: GLenum;\n    readonly TEXTURE_2D: GLenum;\n    readonly TEXTURE_BINDING_2D: GLenum;\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\n    readonly TEXTURE_MAG_FILTER: GLenum;\n    readonly TEXTURE_MIN_FILTER: GLenum;\n    readonly TEXTURE_WRAP_S: GLenum;\n    readonly TEXTURE_WRAP_T: GLenum;\n    readonly TRIANGLES: GLenum;\n    readonly TRIANGLE_FAN: GLenum;\n    readonly TRIANGLE_STRIP: GLenum;\n    readonly UNPACK_ALIGNMENT: GLenum;\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\n    readonly UNSIGNED_BYTE: GLenum;\n    readonly UNSIGNED_INT: GLenum;\n    readonly UNSIGNED_SHORT: GLenum;\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\n    readonly VALIDATE_STATUS: GLenum;\n    readonly VENDOR: GLenum;\n    readonly VERSION: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\n    readonly VERTEX_SHADER: GLenum;\n    readonly VIEWPORT: GLenum;\n    readonly ZERO: GLenum;\n}\n\ninterface WebGLRenderingContextOverloads {\n    bufferData(target: GLenum, size: GLsizeiptr, usage: GLenum): void;\n    bufferData(target: GLenum, data: BufferSource | null, usage: GLenum): void;\n    bufferSubData(target: GLenum, offset: GLintptr, data: BufferSource): void;\n    compressedTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, border: GLint, data: ArrayBufferView): void;\n    compressedTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, data: ArrayBufferView): void;\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    uniform1fv(location: WebGLUniformLocation | null, v: Float32List): void;\n    uniform1iv(location: WebGLUniformLocation | null, v: Int32List): void;\n    uniform2fv(location: WebGLUniformLocation | null, v: Float32List): void;\n    uniform2iv(location: WebGLUniformLocation | null, v: Int32List): void;\n    uniform3fv(location: WebGLUniformLocation | null, v: Float32List): void;\n    uniform3iv(location: WebGLUniformLocation | null, v: Int32List): void;\n    uniform4fv(location: WebGLUniformLocation | null, v: Float32List): void;\n    uniform4iv(location: WebGLUniformLocation | null, v: Int32List): void;\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\n}\n\ninterface WebGLSampler {\n}\n\ndeclare var WebGLSampler: {\n    prototype: WebGLSampler;\n    new(): WebGLSampler;\n};\n\n/** The WebGLShader is part of the WebGL API and can either be a vertex or a fragment shader. A WebGLProgram requires both types of shaders. */\ninterface WebGLShader {\n}\n\ndeclare var WebGLShader: {\n    prototype: WebGLShader;\n    new(): WebGLShader;\n};\n\n/** Part of the WebGL API and represents the information returned by calling the WebGLRenderingContext.getShaderPrecisionFormat() method. */\ninterface WebGLShaderPrecisionFormat {\n    readonly precision: GLint;\n    readonly rangeMax: GLint;\n    readonly rangeMin: GLint;\n}\n\ndeclare var WebGLShaderPrecisionFormat: {\n    prototype: WebGLShaderPrecisionFormat;\n    new(): WebGLShaderPrecisionFormat;\n};\n\ninterface WebGLSync {\n}\n\ndeclare var WebGLSync: {\n    prototype: WebGLSync;\n    new(): WebGLSync;\n};\n\n/** Part of the WebGL API and represents an opaque texture object providing storage and state for texturing operations. */\ninterface WebGLTexture {\n}\n\ndeclare var WebGLTexture: {\n    prototype: WebGLTexture;\n    new(): WebGLTexture;\n};\n\ninterface WebGLTransformFeedback {\n}\n\ndeclare var WebGLTransformFeedback: {\n    prototype: WebGLTransformFeedback;\n    new(): WebGLTransformFeedback;\n};\n\n/** Part of the WebGL API and represents the location of a uniform variable in a shader program. */\ninterface WebGLUniformLocation {\n}\n\ndeclare var WebGLUniformLocation: {\n    prototype: WebGLUniformLocation;\n    new(): WebGLUniformLocation;\n};\n\ninterface WebGLVertexArrayObject {\n}\n\ndeclare var WebGLVertexArrayObject: {\n    prototype: WebGLVertexArrayObject;\n    new(): WebGLVertexArrayObject;\n};\n\ninterface WebGLVertexArrayObjectOES {\n}\n\ninterface WebSocketEventMap {\n    "close": CloseEvent;\n    "error": Event;\n    "message": MessageEvent;\n    "open": Event;\n}\n\n/** Provides the API for creating and managing a WebSocket connection to a server, as well as for sending and receiving data on the connection. */\ninterface WebSocket extends EventTarget {\n    /**\n     * Returns a string that indicates how binary data from the WebSocket object is exposed to scripts:\n     *\n     * Can be set, to change how binary data is returned. The default is "blob".\n     */\n    binaryType: BinaryType;\n    /**\n     * Returns the number of bytes of application data (UTF-8 text and binary data) that have been queued using send() but not yet been transmitted to the network.\n     *\n     * If the WebSocket connection is closed, this attribute\'s value will only increase with each call to the send() method. (The number does not reset to zero once the connection closes.)\n     */\n    readonly bufferedAmount: number;\n    /**\n     * Returns the extensions selected by the server, if any.\n     */\n    readonly extensions: string;\n    onclose: ((this: WebSocket, ev: CloseEvent) => any) | null;\n    onerror: ((this: WebSocket, ev: Event) => any) | null;\n    onmessage: ((this: WebSocket, ev: MessageEvent) => any) | null;\n    onopen: ((this: WebSocket, ev: Event) => any) | null;\n    /**\n     * Returns the subprotocol selected by the server, if any. It can be used in conjunction with the array form of the constructor\'s second argument to perform subprotocol negotiation.\n     */\n    readonly protocol: string;\n    /**\n     * Returns the state of the WebSocket object\'s connection. It can have the values described below.\n     */\n    readonly readyState: number;\n    /**\n     * Returns the URL that was used to establish the WebSocket connection.\n     */\n    readonly url: string;\n    /**\n     * Closes the WebSocket connection, optionally using code as the the WebSocket connection close code and reason as the the WebSocket connection close reason.\n     */\n    close(code?: number, reason?: string): void;\n    /**\n     * Transmits data using the WebSocket connection. data can be a string, a Blob, an ArrayBuffer, or an ArrayBufferView.\n     */\n    send(data: string | ArrayBufferLike | Blob | ArrayBufferView): void;\n    readonly CLOSED: number;\n    readonly CLOSING: number;\n    readonly CONNECTING: number;\n    readonly OPEN: number;\n    addEventListener<K extends keyof WebSocketEventMap>(type: K, listener: (this: WebSocket, ev: WebSocketEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof WebSocketEventMap>(type: K, listener: (this: WebSocket, ev: WebSocketEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var WebSocket: {\n    prototype: WebSocket;\n    new(url: string | URL, protocols?: string | string[]): WebSocket;\n    readonly CLOSED: number;\n    readonly CLOSING: number;\n    readonly CONNECTING: number;\n    readonly OPEN: number;\n};\n\n/** Events that occur due to the user moving a mouse wheel or similar input device. */\ninterface WheelEvent extends MouseEvent {\n    readonly deltaMode: number;\n    readonly deltaX: number;\n    readonly deltaY: number;\n    readonly deltaZ: number;\n    readonly DOM_DELTA_LINE: number;\n    readonly DOM_DELTA_PAGE: number;\n    readonly DOM_DELTA_PIXEL: number;\n}\n\ndeclare var WheelEvent: {\n    prototype: WheelEvent;\n    new(type: string, eventInitDict?: WheelEventInit): WheelEvent;\n    readonly DOM_DELTA_LINE: number;\n    readonly DOM_DELTA_PAGE: number;\n    readonly DOM_DELTA_PIXEL: number;\n};\n\ninterface WindowEventMap extends GlobalEventHandlersEventMap, WindowEventHandlersEventMap {\n    "devicemotion": DeviceMotionEvent;\n    "deviceorientation": DeviceOrientationEvent;\n    "gamepadconnected": GamepadEvent;\n    "gamepaddisconnected": GamepadEvent;\n    "orientationchange": Event;\n}\n\n/** A window containing a DOM document; the document property points to the DOM document loaded in that window. */\ninterface Window extends EventTarget, AnimationFrameProvider, GlobalEventHandlers, WindowEventHandlers, WindowLocalStorage, WindowOrWorkerGlobalScope, WindowSessionStorage {\n    /** @deprecated This is a legacy alias of `navigator`. */\n    readonly clientInformation: Navigator;\n    /**\n     * Returns true if the window has been closed, false otherwise.\n     */\n    readonly closed: boolean;\n    /**\n     * Defines a new custom element, mapping the given name to the given constructor as an autonomous custom element.\n     */\n    readonly customElements: CustomElementRegistry;\n    readonly devicePixelRatio: number;\n    readonly document: Document;\n    /** @deprecated */\n    readonly event: Event | undefined;\n    /** @deprecated */\n    readonly external: External;\n    readonly frameElement: Element | null;\n    readonly frames: WindowProxy;\n    readonly history: History;\n    readonly innerHeight: number;\n    readonly innerWidth: number;\n    readonly length: number;\n    get location(): Location;\n    set location(href: string | Location);\n    /**\n     * Returns true if the location bar is visible; otherwise, returns false.\n     */\n    readonly locationbar: BarProp;\n    /**\n     * Returns true if the menu bar is visible; otherwise, returns false.\n     */\n    readonly menubar: BarProp;\n    name: string;\n    readonly navigator: Navigator;\n    ondevicemotion: ((this: Window, ev: DeviceMotionEvent) => any) | null;\n    ondeviceorientation: ((this: Window, ev: DeviceOrientationEvent) => any) | null;\n    /** @deprecated */\n    onorientationchange: ((this: Window, ev: Event) => any) | null;\n    opener: any;\n    /** @deprecated */\n    readonly orientation: number;\n    readonly outerHeight: number;\n    readonly outerWidth: number;\n    /** @deprecated This is a legacy alias of `scrollX`. */\n    readonly pageXOffset: number;\n    /** @deprecated This is a legacy alias of `scrollY`. */\n    readonly pageYOffset: number;\n    /** Refers to either the parent WindowProxy, or itself. It can rarely be null e.g. for contentWindow of an iframe that is already removed from the parent. */\n    readonly parent: WindowProxy;\n    /**\n     * Returns true if the personal bar is visible; otherwise, returns false.\n     */\n    readonly personalbar: BarProp;\n    readonly screen: Screen;\n    readonly screenLeft: number;\n    readonly screenTop: number;\n    readonly screenX: number;\n    readonly screenY: number;\n    readonly scrollX: number;\n    readonly scrollY: number;\n    /**\n     * Returns true if the scrollbars are visible; otherwise, returns false.\n     */\n    readonly scrollbars: BarProp;\n    readonly self: Window & typeof globalThis;\n    readonly speechSynthesis: SpeechSynthesis;\n    /** @deprecated */\n    status: string;\n    /**\n     * Returns true if the status bar is visible; otherwise, returns false.\n     */\n    readonly statusbar: BarProp;\n    /**\n     * Returns true if the toolbar is visible; otherwise, returns false.\n     */\n    readonly toolbar: BarProp;\n    readonly top: WindowProxy | null;\n    readonly visualViewport: VisualViewport;\n    readonly window: Window & typeof globalThis;\n    alert(message?: any): void;\n    blur(): void;\n    cancelIdleCallback(handle: number): void;\n    /** @deprecated */\n    captureEvents(): void;\n    /**\n     * Closes the window.\n     */\n    close(): void;\n    confirm(message?: string): boolean;\n    /**\n     * Moves the focus to the window\'s browsing context, if any.\n     */\n    focus(): void;\n    getComputedStyle(elt: Element, pseudoElt?: string | null): CSSStyleDeclaration;\n    getSelection(): Selection | null;\n    matchMedia(query: string): MediaQueryList;\n    moveBy(x: number, y: number): void;\n    moveTo(x: number, y: number): void;\n    open(url?: string | URL, target?: string, features?: string): WindowProxy | null;\n    /**\n     * Posts a message to the given window. Messages can be structured objects, e.g. nested objects and arrays, can contain JavaScript values (strings, numbers, Date objects, etc), and can contain certain data objects such as File Blob, FileList, and ArrayBuffer objects.\n     *\n     * Objects listed in the transfer member of options are transferred, not just cloned, meaning that they are no longer usable on the sending side.\n     *\n     * A target origin can be specified using the targetOrigin member of options. If not provided, it defaults to "/". This default restricts the message to same-origin targets only.\n     *\n     * If the origin of the target window doesn\'t match the given target origin, the message is discarded, to avoid information leakage. To send the message to the target regardless of origin, set the target origin to "*".\n     *\n     * Throws a "DataCloneError" DOMException if transfer array contains duplicate objects or if message could not be cloned.\n     */\n    postMessage(message: any, targetOrigin: string, transfer?: Transferable[]): void;\n    postMessage(message: any, options?: WindowPostMessageOptions): void;\n    print(): void;\n    prompt(message?: string, _default?: string): string | null;\n    /** @deprecated */\n    releaseEvents(): void;\n    requestIdleCallback(callback: IdleRequestCallback, options?: IdleRequestOptions): number;\n    resizeBy(x: number, y: number): void;\n    resizeTo(width: number, height: number): void;\n    scroll(options?: ScrollToOptions): void;\n    scroll(x: number, y: number): void;\n    scrollBy(options?: ScrollToOptions): void;\n    scrollBy(x: number, y: number): void;\n    scrollTo(options?: ScrollToOptions): void;\n    scrollTo(x: number, y: number): void;\n    /**\n     * Cancels the document load.\n     */\n    stop(): void;\n    addEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n    [index: number]: Window;\n}\n\ndeclare var Window: {\n    prototype: Window;\n    new(): Window;\n};\n\ninterface WindowEventHandlersEventMap {\n    "afterprint": Event;\n    "beforeprint": Event;\n    "beforeunload": BeforeUnloadEvent;\n    "gamepadconnected": Event;\n    "gamepaddisconnected": Event;\n    "hashchange": Event;\n    "languagechange": Event;\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n    "offline": Event;\n    "online": Event;\n    "pagehide": PageTransitionEvent;\n    "pageshow": PageTransitionEvent;\n    "popstate": PopStateEvent;\n    "rejectionhandled": PromiseRejectionEvent;\n    "storage": StorageEvent;\n    "unhandledrejection": PromiseRejectionEvent;\n    "unload": Event;\n}\n\ninterface WindowEventHandlers {\n    onafterprint: ((this: WindowEventHandlers, ev: Event) => any) | null;\n    onbeforeprint: ((this: WindowEventHandlers, ev: Event) => any) | null;\n    onbeforeunload: ((this: WindowEventHandlers, ev: BeforeUnloadEvent) => any) | null;\n    ongamepadconnected: ((this: WindowEventHandlers, ev: Event) => any) | null;\n    ongamepaddisconnected: ((this: WindowEventHandlers, ev: Event) => any) | null;\n    onhashchange: ((this: WindowEventHandlers, ev: Event) => any) | null;\n    onlanguagechange: ((this: WindowEventHandlers, ev: Event) => any) | null;\n    onmessage: ((this: WindowEventHandlers, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: WindowEventHandlers, ev: MessageEvent) => any) | null;\n    onoffline: ((this: WindowEventHandlers, ev: Event) => any) | null;\n    ononline: ((this: WindowEventHandlers, ev: Event) => any) | null;\n    onpagehide: ((this: WindowEventHandlers, ev: PageTransitionEvent) => any) | null;\n    onpageshow: ((this: WindowEventHandlers, ev: PageTransitionEvent) => any) | null;\n    onpopstate: ((this: WindowEventHandlers, ev: PopStateEvent) => any) | null;\n    onrejectionhandled: ((this: WindowEventHandlers, ev: PromiseRejectionEvent) => any) | null;\n    onstorage: ((this: WindowEventHandlers, ev: StorageEvent) => any) | null;\n    onunhandledrejection: ((this: WindowEventHandlers, ev: PromiseRejectionEvent) => any) | null;\n    onunload: ((this: WindowEventHandlers, ev: Event) => any) | null;\n    addEventListener<K extends keyof WindowEventHandlersEventMap>(type: K, listener: (this: WindowEventHandlers, ev: WindowEventHandlersEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof WindowEventHandlersEventMap>(type: K, listener: (this: WindowEventHandlers, ev: WindowEventHandlersEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ninterface WindowLocalStorage {\n    readonly localStorage: Storage;\n}\n\ninterface WindowOrWorkerGlobalScope {\n    readonly caches: CacheStorage;\n    readonly crossOriginIsolated: boolean;\n    readonly crypto: Crypto;\n    readonly indexedDB: IDBFactory;\n    readonly isSecureContext: boolean;\n    readonly origin: string;\n    readonly performance: Performance;\n    atob(data: string): string;\n    btoa(data: string): string;\n    clearInterval(handle?: number): void;\n    clearTimeout(handle?: number): void;\n    createImageBitmap(image: ImageBitmapSource, options?: ImageBitmapOptions): Promise<ImageBitmap>;\n    createImageBitmap(image: ImageBitmapSource, sx: number, sy: number, sw: number, sh: number, options?: ImageBitmapOptions): Promise<ImageBitmap>;\n    fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\n    queueMicrotask(callback: VoidFunction): void;\n    setInterval(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\n    setTimeout(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\n}\n\ninterface WindowSessionStorage {\n    readonly sessionStorage: Storage;\n}\n\ninterface WorkerEventMap extends AbstractWorkerEventMap {\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n}\n\n/** This Web Workers API interface represents a background task that can be easily created and can send messages back to its creator. Creating a worker is as simple as calling the Worker() constructor and specifying a script to be run in the worker thread. */\ninterface Worker extends EventTarget, AbstractWorker {\n    onmessage: ((this: Worker, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: Worker, ev: MessageEvent) => any) | null;\n    /**\n     * Clones message and transmits it to worker\'s global environment. transfer can be passed as a list of objects that are to be transferred rather than cloned.\n     */\n    postMessage(message: any, transfer: Transferable[]): void;\n    postMessage(message: any, options?: PostMessageOptions): void;\n    /**\n     * Aborts worker\'s associated global environment.\n     */\n    terminate(): void;\n    addEventListener<K extends keyof WorkerEventMap>(type: K, listener: (this: Worker, ev: WorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof WorkerEventMap>(type: K, listener: (this: Worker, ev: WorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var Worker: {\n    prototype: Worker;\n    new(scriptURL: string | URL, options?: WorkerOptions): Worker;\n};\n\ninterface Worklet {\n    /**\n     * Loads and executes the module script given by moduleURL into all of worklet\'s global scopes. It can also create additional global scopes as part of this process, depending on the worklet type. The returned promise will fulfill once the script has been successfully loaded and run in all global scopes.\n     *\n     * The credentials option can be set to a credentials mode to modify the script-fetching process. It defaults to "same-origin".\n     *\n     * Any failures in fetching the script or its dependencies will cause the returned promise to be rejected with an "AbortError" DOMException. Any errors in parsing the script or its dependencies will cause the returned promise to be rejected with the exception generated during parsing.\n     */\n    addModule(moduleURL: string | URL, options?: WorkletOptions): Promise<void>;\n}\n\ndeclare var Worklet: {\n    prototype: Worklet;\n    new(): Worklet;\n};\n\n/** This Streams API interface provides a standard abstraction for writing streaming data to a destination, known as a sink. This object comes with built-in backpressure and queuing. */\ninterface WritableStream<W = any> {\n    readonly locked: boolean;\n    abort(reason?: any): Promise<void>;\n    close(): Promise<void>;\n    getWriter(): WritableStreamDefaultWriter<W>;\n}\n\ndeclare var WritableStream: {\n    prototype: WritableStream;\n    new<W = any>(underlyingSink?: UnderlyingSink<W>, strategy?: QueuingStrategy<W>): WritableStream<W>;\n};\n\n/** This Streams API interface represents a controller allowing control of a WritableStream\'s state. When constructing a WritableStream, the underlying sink is given a corresponding WritableStreamDefaultController instance to manipulate. */\ninterface WritableStreamDefaultController {\n    error(e?: any): void;\n}\n\ndeclare var WritableStreamDefaultController: {\n    prototype: WritableStreamDefaultController;\n    new(): WritableStreamDefaultController;\n};\n\n/** This Streams API interface is the object returned by WritableStream.getWriter() and once created locks the < writer to the WritableStream ensuring that no other streams can write to the underlying sink. */\ninterface WritableStreamDefaultWriter<W = any> {\n    readonly closed: Promise<undefined>;\n    readonly desiredSize: number | null;\n    readonly ready: Promise<undefined>;\n    abort(reason?: any): Promise<void>;\n    close(): Promise<void>;\n    releaseLock(): void;\n    write(chunk?: W): Promise<void>;\n}\n\ndeclare var WritableStreamDefaultWriter: {\n    prototype: WritableStreamDefaultWriter;\n    new<W = any>(stream: WritableStream<W>): WritableStreamDefaultWriter<W>;\n};\n\n/** An XML document. It inherits from the generic Document and does not add any specific methods or properties to it: nevertheless, several algorithms behave differently with the two types of documents. */\ninterface XMLDocument extends Document {\n    addEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: XMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof DocumentEventMap>(type: K, listener: (this: XMLDocument, ev: DocumentEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var XMLDocument: {\n    prototype: XMLDocument;\n    new(): XMLDocument;\n};\n\ninterface XMLHttpRequestEventMap extends XMLHttpRequestEventTargetEventMap {\n    "readystatechange": Event;\n}\n\n/** Use XMLHttpRequest (XHR) objects to interact with servers. You can retrieve data from a URL without having to do a full page refresh. This enables a Web page to update just part of a page without disrupting what the user is doing. */\ninterface XMLHttpRequest extends XMLHttpRequestEventTarget {\n    onreadystatechange: ((this: XMLHttpRequest, ev: Event) => any) | null;\n    /**\n     * Returns client\'s state.\n     */\n    readonly readyState: number;\n    /**\n     * Returns the response body.\n     */\n    readonly response: any;\n    /**\n     * Returns response as text.\n     *\n     * Throws an "InvalidStateError" DOMException if responseType is not the empty string or "text".\n     */\n    readonly responseText: string;\n    /**\n     * Returns the response type.\n     *\n     * Can be set to change the response type. Values are: the empty string (default), "arraybuffer", "blob", "document", "json", and "text".\n     *\n     * When set: setting to "document" is ignored if current global object is not a Window object.\n     *\n     * When set: throws an "InvalidStateError" DOMException if state is loading or done.\n     *\n     * When set: throws an "InvalidAccessError" DOMException if the synchronous flag is set and current global object is a Window object.\n     */\n    responseType: XMLHttpRequestResponseType;\n    readonly responseURL: string;\n    /**\n     * Returns the response as document.\n     *\n     * Throws an "InvalidStateError" DOMException if responseType is not the empty string or "document".\n     */\n    readonly responseXML: Document | null;\n    readonly status: number;\n    readonly statusText: string;\n    /**\n     * Can be set to a time in milliseconds. When set to a non-zero value will cause fetching to terminate after the given time has passed. When the time has passed, the request has not yet completed, and this\'s synchronous flag is unset, a timeout event will then be dispatched, or a "TimeoutError" DOMException will be thrown otherwise (for the send() method).\n     *\n     * When set: throws an "InvalidAccessError" DOMException if the synchronous flag is set and current global object is a Window object.\n     */\n    timeout: number;\n    /**\n     * Returns the associated XMLHttpRequestUpload object. It can be used to gather transmission information when data is transferred to a server.\n     */\n    readonly upload: XMLHttpRequestUpload;\n    /**\n     * True when credentials are to be included in a cross-origin request. False when they are to be excluded in a cross-origin request and when cookies are to be ignored in its response. Initially false.\n     *\n     * When set: throws an "InvalidStateError" DOMException if state is not unsent or opened, or if the send() flag is set.\n     */\n    withCredentials: boolean;\n    /**\n     * Cancels any network activity.\n     */\n    abort(): void;\n    getAllResponseHeaders(): string;\n    getResponseHeader(name: string): string | null;\n    /**\n     * Sets the request method, request URL, and synchronous flag.\n     *\n     * Throws a "SyntaxError" DOMException if either method is not a valid method or url cannot be parsed.\n     *\n     * Throws a "SecurityError" DOMException if method is a case-insensitive match for `CONNECT`, `TRACE`, or `TRACK`.\n     *\n     * Throws an "InvalidAccessError" DOMException if async is false, current global object is a Window object, and the timeout attribute is not zero or the responseType attribute is not the empty string.\n     */\n    open(method: string, url: string | URL): void;\n    open(method: string, url: string | URL, async: boolean, username?: string | null, password?: string | null): void;\n    /**\n     * Acts as if the `Content-Type` header value for a response is mime. (It does not change the header.)\n     *\n     * Throws an "InvalidStateError" DOMException if state is loading or done.\n     */\n    overrideMimeType(mime: string): void;\n    /**\n     * Initiates the request. The body argument provides the request body, if any, and is ignored if the request method is GET or HEAD.\n     *\n     * Throws an "InvalidStateError" DOMException if either state is not opened or the send() flag is set.\n     */\n    send(body?: Document | XMLHttpRequestBodyInit | null): void;\n    /**\n     * Combines a header in author request headers.\n     *\n     * Throws an "InvalidStateError" DOMException if either state is not opened or the send() flag is set.\n     *\n     * Throws a "SyntaxError" DOMException if name is not a header name or if value is not a header value.\n     */\n    setRequestHeader(name: string, value: string): void;\n    readonly DONE: number;\n    readonly HEADERS_RECEIVED: number;\n    readonly LOADING: number;\n    readonly OPENED: number;\n    readonly UNSENT: number;\n    addEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var XMLHttpRequest: {\n    prototype: XMLHttpRequest;\n    new(): XMLHttpRequest;\n    readonly DONE: number;\n    readonly HEADERS_RECEIVED: number;\n    readonly LOADING: number;\n    readonly OPENED: number;\n    readonly UNSENT: number;\n};\n\ninterface XMLHttpRequestEventTargetEventMap {\n    "abort": ProgressEvent<XMLHttpRequestEventTarget>;\n    "error": ProgressEvent<XMLHttpRequestEventTarget>;\n    "load": ProgressEvent<XMLHttpRequestEventTarget>;\n    "loadend": ProgressEvent<XMLHttpRequestEventTarget>;\n    "loadstart": ProgressEvent<XMLHttpRequestEventTarget>;\n    "progress": ProgressEvent<XMLHttpRequestEventTarget>;\n    "timeout": ProgressEvent<XMLHttpRequestEventTarget>;\n}\n\ninterface XMLHttpRequestEventTarget extends EventTarget {\n    onabort: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onerror: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onload: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onloadend: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onloadstart: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onprogress: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    ontimeout: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    addEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestEventTarget, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestEventTarget, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var XMLHttpRequestEventTarget: {\n    prototype: XMLHttpRequestEventTarget;\n    new(): XMLHttpRequestEventTarget;\n};\n\ninterface XMLHttpRequestUpload extends XMLHttpRequestEventTarget {\n    addEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestUpload, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestUpload, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var XMLHttpRequestUpload: {\n    prototype: XMLHttpRequestUpload;\n    new(): XMLHttpRequestUpload;\n};\n\n/** Provides the serializeToString() method to construct an XML string representing a DOM tree. */\ninterface XMLSerializer {\n    serializeToString(root: Node): string;\n}\n\ndeclare var XMLSerializer: {\n    prototype: XMLSerializer;\n    new(): XMLSerializer;\n};\n\n/** The XPathEvaluator interface allows to compile and evaluate XPath expressions. */\ninterface XPathEvaluator extends XPathEvaluatorBase {\n}\n\ndeclare var XPathEvaluator: {\n    prototype: XPathEvaluator;\n    new(): XPathEvaluator;\n};\n\ninterface XPathEvaluatorBase {\n    createExpression(expression: string, resolver?: XPathNSResolver | null): XPathExpression;\n    createNSResolver(nodeResolver: Node): XPathNSResolver;\n    evaluate(expression: string, contextNode: Node, resolver?: XPathNSResolver | null, type?: number, result?: XPathResult | null): XPathResult;\n}\n\n/** This interface is a compiled XPath expression that can be evaluated on a document or specific node to return information its DOM tree. */\ninterface XPathExpression {\n    evaluate(contextNode: Node, type?: number, result?: XPathResult | null): XPathResult;\n}\n\ndeclare var XPathExpression: {\n    prototype: XPathExpression;\n    new(): XPathExpression;\n};\n\n/** The results generated by evaluating an XPath expression within the context of a given node. */\ninterface XPathResult {\n    readonly booleanValue: boolean;\n    readonly invalidIteratorState: boolean;\n    readonly numberValue: number;\n    readonly resultType: number;\n    readonly singleNodeValue: Node | null;\n    readonly snapshotLength: number;\n    readonly stringValue: string;\n    iterateNext(): Node | null;\n    snapshotItem(index: number): Node | null;\n    readonly ANY_TYPE: number;\n    readonly ANY_UNORDERED_NODE_TYPE: number;\n    readonly BOOLEAN_TYPE: number;\n    readonly FIRST_ORDERED_NODE_TYPE: number;\n    readonly NUMBER_TYPE: number;\n    readonly ORDERED_NODE_ITERATOR_TYPE: number;\n    readonly ORDERED_NODE_SNAPSHOT_TYPE: number;\n    readonly STRING_TYPE: number;\n    readonly UNORDERED_NODE_ITERATOR_TYPE: number;\n    readonly UNORDERED_NODE_SNAPSHOT_TYPE: number;\n}\n\ndeclare var XPathResult: {\n    prototype: XPathResult;\n    new(): XPathResult;\n    readonly ANY_TYPE: number;\n    readonly ANY_UNORDERED_NODE_TYPE: number;\n    readonly BOOLEAN_TYPE: number;\n    readonly FIRST_ORDERED_NODE_TYPE: number;\n    readonly NUMBER_TYPE: number;\n    readonly ORDERED_NODE_ITERATOR_TYPE: number;\n    readonly ORDERED_NODE_SNAPSHOT_TYPE: number;\n    readonly STRING_TYPE: number;\n    readonly UNORDERED_NODE_ITERATOR_TYPE: number;\n    readonly UNORDERED_NODE_SNAPSHOT_TYPE: number;\n};\n\n/** An XSLTProcessor applies an XSLT stylesheet transformation to an XML document to produce a new XML document as output. It has methods to load the XSLT stylesheet, to manipulate <xsl:param> parameter values, and to apply the transformation to documents. */\ninterface XSLTProcessor {\n    clearParameters(): void;\n    getParameter(namespaceURI: string | null, localName: string): any;\n    importStylesheet(style: Node): void;\n    removeParameter(namespaceURI: string | null, localName: string): void;\n    reset(): void;\n    setParameter(namespaceURI: string | null, localName: string, value: any): void;\n    transformToDocument(source: Node): Document;\n    transformToFragment(source: Node, output: Document): DocumentFragment;\n}\n\ndeclare var XSLTProcessor: {\n    prototype: XSLTProcessor;\n    new(): XSLTProcessor;\n};\n\ninterface Console {\n    assert(condition?: boolean, ...data: any[]): void;\n    clear(): void;\n    count(label?: string): void;\n    countReset(label?: string): void;\n    debug(...data: any[]): void;\n    dir(item?: any, options?: any): void;\n    dirxml(...data: any[]): void;\n    error(...data: any[]): void;\n    group(...data: any[]): void;\n    groupCollapsed(...data: any[]): void;\n    groupEnd(): void;\n    info(...data: any[]): void;\n    log(...data: any[]): void;\n    table(tabularData?: any, properties?: string[]): void;\n    time(label?: string): void;\n    timeEnd(label?: string): void;\n    timeLog(label?: string, ...data: any[]): void;\n    timeStamp(label?: string): void;\n    trace(...data: any[]): void;\n    warn(...data: any[]): void;\n}\n\ndeclare var console: Console;\n\n/** Holds useful CSS-related methods. No object with this interface are implemented: it contains only static methods and therefore is a utilitarian interface. */\ndeclare namespace CSS {\n    function escape(ident: string): string;\n    function supports(property: string, value: string): boolean;\n    function supports(conditionText: string): boolean;\n}\n\ndeclare namespace WebAssembly {\n    interface CompileError extends Error {\n    }\n\n    var CompileError: {\n        prototype: CompileError;\n        new(): CompileError;\n    };\n\n    interface Global {\n        value: any;\n        valueOf(): any;\n    }\n\n    var Global: {\n        prototype: Global;\n        new(descriptor: GlobalDescriptor, v?: any): Global;\n    };\n\n    interface Instance {\n        readonly exports: Exports;\n    }\n\n    var Instance: {\n        prototype: Instance;\n        new(module: Module, importObject?: Imports): Instance;\n    };\n\n    interface LinkError extends Error {\n    }\n\n    var LinkError: {\n        prototype: LinkError;\n        new(): LinkError;\n    };\n\n    interface Memory {\n        readonly buffer: ArrayBuffer;\n        grow(delta: number): number;\n    }\n\n    var Memory: {\n        prototype: Memory;\n        new(descriptor: MemoryDescriptor): Memory;\n    };\n\n    interface Module {\n    }\n\n    var Module: {\n        prototype: Module;\n        new(bytes: BufferSource): Module;\n        customSections(moduleObject: Module, sectionName: string): ArrayBuffer[];\n        exports(moduleObject: Module): ModuleExportDescriptor[];\n        imports(moduleObject: Module): ModuleImportDescriptor[];\n    };\n\n    interface RuntimeError extends Error {\n    }\n\n    var RuntimeError: {\n        prototype: RuntimeError;\n        new(): RuntimeError;\n    };\n\n    interface Table {\n        readonly length: number;\n        get(index: number): any;\n        grow(delta: number, value?: any): number;\n        set(index: number, value?: any): void;\n    }\n\n    var Table: {\n        prototype: Table;\n        new(descriptor: TableDescriptor, value?: any): Table;\n    };\n\n    interface GlobalDescriptor {\n        mutable?: boolean;\n        value: ValueType;\n    }\n\n    interface MemoryDescriptor {\n        initial: number;\n        maximum?: number;\n        shared?: boolean;\n    }\n\n    interface ModuleExportDescriptor {\n        kind: ImportExportKind;\n        name: string;\n    }\n\n    interface ModuleImportDescriptor {\n        kind: ImportExportKind;\n        module: string;\n        name: string;\n    }\n\n    interface TableDescriptor {\n        element: TableKind;\n        initial: number;\n        maximum?: number;\n    }\n\n    interface WebAssemblyInstantiatedSource {\n        instance: Instance;\n        module: Module;\n    }\n\n    type ImportExportKind = "function" | "global" | "memory" | "table";\n    type TableKind = "anyfunc" | "externref";\n    type ValueType = "anyfunc" | "externref" | "f32" | "f64" | "i32" | "i64";\n    type ExportValue = Function | Global | Memory | Table;\n    type Exports = Record<string, ExportValue>;\n    type ImportValue = ExportValue | number;\n    type Imports = Record<string, ModuleImports>;\n    type ModuleImports = Record<string, ImportValue>;\n    function compile(bytes: BufferSource): Promise<Module>;\n    function compileStreaming(source: Response | PromiseLike<Response>): Promise<Module>;\n    function instantiate(bytes: BufferSource, importObject?: Imports): Promise<WebAssemblyInstantiatedSource>;\n    function instantiate(moduleObject: Module, importObject?: Imports): Promise<Instance>;\n    function instantiateStreaming(source: Response | PromiseLike<Response>, importObject?: Imports): Promise<WebAssemblyInstantiatedSource>;\n    function validate(bytes: BufferSource): boolean;\n}\n\ninterface BlobCallback {\n    (blob: Blob | null): void;\n}\n\ninterface CustomElementConstructor {\n    new (...params: any[]): HTMLElement;\n}\n\ninterface DecodeErrorCallback {\n    (error: DOMException): void;\n}\n\ninterface DecodeSuccessCallback {\n    (decodedData: AudioBuffer): void;\n}\n\ninterface ErrorCallback {\n    (err: DOMException): void;\n}\n\ninterface FileCallback {\n    (file: File): void;\n}\n\ninterface FileSystemEntriesCallback {\n    (entries: FileSystemEntry[]): void;\n}\n\ninterface FileSystemEntryCallback {\n    (entry: FileSystemEntry): void;\n}\n\ninterface FrameRequestCallback {\n    (time: DOMHighResTimeStamp): void;\n}\n\ninterface FunctionStringCallback {\n    (data: string): void;\n}\n\ninterface IdleRequestCallback {\n    (deadline: IdleDeadline): void;\n}\n\ninterface IntersectionObserverCallback {\n    (entries: IntersectionObserverEntry[], observer: IntersectionObserver): void;\n}\n\ninterface MediaSessionActionHandler {\n    (details: MediaSessionActionDetails): void;\n}\n\ninterface MutationCallback {\n    (mutations: MutationRecord[], observer: MutationObserver): void;\n}\n\ninterface NotificationPermissionCallback {\n    (permission: NotificationPermission): void;\n}\n\ninterface OnBeforeUnloadEventHandlerNonNull {\n    (event: Event): string | null;\n}\n\ninterface OnErrorEventHandlerNonNull {\n    (event: Event | string, source?: string, lineno?: number, colno?: number, error?: Error): any;\n}\n\ninterface PerformanceObserverCallback {\n    (entries: PerformanceObserverEntryList, observer: PerformanceObserver): void;\n}\n\ninterface PositionCallback {\n    (position: GeolocationPosition): void;\n}\n\ninterface PositionErrorCallback {\n    (positionError: GeolocationPositionError): void;\n}\n\ninterface QueuingStrategySize<T = any> {\n    (chunk?: T): number;\n}\n\ninterface RTCPeerConnectionErrorCallback {\n    (error: DOMException): void;\n}\n\ninterface RTCSessionDescriptionCallback {\n    (description: RTCSessionDescriptionInit): void;\n}\n\ninterface RemotePlaybackAvailabilityCallback {\n    (available: boolean): void;\n}\n\ninterface ResizeObserverCallback {\n    (entries: ResizeObserverEntry[], observer: ResizeObserver): void;\n}\n\ninterface TransformerFlushCallback<O> {\n    (controller: TransformStreamDefaultController<O>): void | PromiseLike<void>;\n}\n\ninterface TransformerStartCallback<O> {\n    (controller: TransformStreamDefaultController<O>): any;\n}\n\ninterface TransformerTransformCallback<I, O> {\n    (chunk: I, controller: TransformStreamDefaultController<O>): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSinkAbortCallback {\n    (reason?: any): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSinkCloseCallback {\n    (): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSinkStartCallback {\n    (controller: WritableStreamDefaultController): any;\n}\n\ninterface UnderlyingSinkWriteCallback<W> {\n    (chunk: W, controller: WritableStreamDefaultController): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSourceCancelCallback {\n    (reason?: any): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSourcePullCallback<R> {\n    (controller: ReadableStreamController<R>): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSourceStartCallback<R> {\n    (controller: ReadableStreamController<R>): any;\n}\n\ninterface VoidFunction {\n    (): void;\n}\n\ninterface HTMLElementTagNameMap {\n    "a": HTMLAnchorElement;\n    "abbr": HTMLElement;\n    "address": HTMLElement;\n    "area": HTMLAreaElement;\n    "article": HTMLElement;\n    "aside": HTMLElement;\n    "audio": HTMLAudioElement;\n    "b": HTMLElement;\n    "base": HTMLBaseElement;\n    "bdi": HTMLElement;\n    "bdo": HTMLElement;\n    "blockquote": HTMLQuoteElement;\n    "body": HTMLBodyElement;\n    "br": HTMLBRElement;\n    "button": HTMLButtonElement;\n    "canvas": HTMLCanvasElement;\n    "caption": HTMLTableCaptionElement;\n    "cite": HTMLElement;\n    "code": HTMLElement;\n    "col": HTMLTableColElement;\n    "colgroup": HTMLTableColElement;\n    "data": HTMLDataElement;\n    "datalist": HTMLDataListElement;\n    "dd": HTMLElement;\n    "del": HTMLModElement;\n    "details": HTMLDetailsElement;\n    "dfn": HTMLElement;\n    "dialog": HTMLDialogElement;\n    "dir": HTMLDirectoryElement;\n    "div": HTMLDivElement;\n    "dl": HTMLDListElement;\n    "dt": HTMLElement;\n    "em": HTMLElement;\n    "embed": HTMLEmbedElement;\n    "fieldset": HTMLFieldSetElement;\n    "figcaption": HTMLElement;\n    "figure": HTMLElement;\n    "font": HTMLFontElement;\n    "footer": HTMLElement;\n    "form": HTMLFormElement;\n    "frame": HTMLFrameElement;\n    "frameset": HTMLFrameSetElement;\n    "h1": HTMLHeadingElement;\n    "h2": HTMLHeadingElement;\n    "h3": HTMLHeadingElement;\n    "h4": HTMLHeadingElement;\n    "h5": HTMLHeadingElement;\n    "h6": HTMLHeadingElement;\n    "head": HTMLHeadElement;\n    "header": HTMLElement;\n    "hgroup": HTMLElement;\n    "hr": HTMLHRElement;\n    "html": HTMLHtmlElement;\n    "i": HTMLElement;\n    "iframe": HTMLIFrameElement;\n    "img": HTMLImageElement;\n    "input": HTMLInputElement;\n    "ins": HTMLModElement;\n    "kbd": HTMLElement;\n    "label": HTMLLabelElement;\n    "legend": HTMLLegendElement;\n    "li": HTMLLIElement;\n    "link": HTMLLinkElement;\n    "main": HTMLElement;\n    "map": HTMLMapElement;\n    "mark": HTMLElement;\n    "marquee": HTMLMarqueeElement;\n    "menu": HTMLMenuElement;\n    "meta": HTMLMetaElement;\n    "meter": HTMLMeterElement;\n    "nav": HTMLElement;\n    "noscript": HTMLElement;\n    "object": HTMLObjectElement;\n    "ol": HTMLOListElement;\n    "optgroup": HTMLOptGroupElement;\n    "option": HTMLOptionElement;\n    "output": HTMLOutputElement;\n    "p": HTMLParagraphElement;\n    "param": HTMLParamElement;\n    "picture": HTMLPictureElement;\n    "pre": HTMLPreElement;\n    "progress": HTMLProgressElement;\n    "q": HTMLQuoteElement;\n    "rp": HTMLElement;\n    "rt": HTMLElement;\n    "ruby": HTMLElement;\n    "s": HTMLElement;\n    "samp": HTMLElement;\n    "script": HTMLScriptElement;\n    "section": HTMLElement;\n    "select": HTMLSelectElement;\n    "slot": HTMLSlotElement;\n    "small": HTMLElement;\n    "source": HTMLSourceElement;\n    "span": HTMLSpanElement;\n    "strong": HTMLElement;\n    "style": HTMLStyleElement;\n    "sub": HTMLElement;\n    "summary": HTMLElement;\n    "sup": HTMLElement;\n    "table": HTMLTableElement;\n    "tbody": HTMLTableSectionElement;\n    "td": HTMLTableCellElement;\n    "template": HTMLTemplateElement;\n    "textarea": HTMLTextAreaElement;\n    "tfoot": HTMLTableSectionElement;\n    "th": HTMLTableCellElement;\n    "thead": HTMLTableSectionElement;\n    "time": HTMLTimeElement;\n    "title": HTMLTitleElement;\n    "tr": HTMLTableRowElement;\n    "track": HTMLTrackElement;\n    "u": HTMLElement;\n    "ul": HTMLUListElement;\n    "var": HTMLElement;\n    "video": HTMLVideoElement;\n    "wbr": HTMLElement;\n}\n\ninterface HTMLElementDeprecatedTagNameMap {\n    "listing": HTMLPreElement;\n    "xmp": HTMLPreElement;\n}\n\ninterface SVGElementTagNameMap {\n    "a": SVGAElement;\n    "animate": SVGAnimateElement;\n    "animateMotion": SVGAnimateMotionElement;\n    "animateTransform": SVGAnimateTransformElement;\n    "circle": SVGCircleElement;\n    "clipPath": SVGClipPathElement;\n    "defs": SVGDefsElement;\n    "desc": SVGDescElement;\n    "ellipse": SVGEllipseElement;\n    "feBlend": SVGFEBlendElement;\n    "feColorMatrix": SVGFEColorMatrixElement;\n    "feComponentTransfer": SVGFEComponentTransferElement;\n    "feComposite": SVGFECompositeElement;\n    "feConvolveMatrix": SVGFEConvolveMatrixElement;\n    "feDiffuseLighting": SVGFEDiffuseLightingElement;\n    "feDisplacementMap": SVGFEDisplacementMapElement;\n    "feDistantLight": SVGFEDistantLightElement;\n    "feDropShadow": SVGFEDropShadowElement;\n    "feFlood": SVGFEFloodElement;\n    "feFuncA": SVGFEFuncAElement;\n    "feFuncB": SVGFEFuncBElement;\n    "feFuncG": SVGFEFuncGElement;\n    "feFuncR": SVGFEFuncRElement;\n    "feGaussianBlur": SVGFEGaussianBlurElement;\n    "feImage": SVGFEImageElement;\n    "feMerge": SVGFEMergeElement;\n    "feMergeNode": SVGFEMergeNodeElement;\n    "feMorphology": SVGFEMorphologyElement;\n    "feOffset": SVGFEOffsetElement;\n    "fePointLight": SVGFEPointLightElement;\n    "feSpecularLighting": SVGFESpecularLightingElement;\n    "feSpotLight": SVGFESpotLightElement;\n    "feTile": SVGFETileElement;\n    "feTurbulence": SVGFETurbulenceElement;\n    "filter": SVGFilterElement;\n    "foreignObject": SVGForeignObjectElement;\n    "g": SVGGElement;\n    "image": SVGImageElement;\n    "line": SVGLineElement;\n    "linearGradient": SVGLinearGradientElement;\n    "marker": SVGMarkerElement;\n    "mask": SVGMaskElement;\n    "metadata": SVGMetadataElement;\n    "mpath": SVGMPathElement;\n    "path": SVGPathElement;\n    "pattern": SVGPatternElement;\n    "polygon": SVGPolygonElement;\n    "polyline": SVGPolylineElement;\n    "radialGradient": SVGRadialGradientElement;\n    "rect": SVGRectElement;\n    "script": SVGScriptElement;\n    "set": SVGSetElement;\n    "stop": SVGStopElement;\n    "style": SVGStyleElement;\n    "svg": SVGSVGElement;\n    "switch": SVGSwitchElement;\n    "symbol": SVGSymbolElement;\n    "text": SVGTextElement;\n    "textPath": SVGTextPathElement;\n    "title": SVGTitleElement;\n    "tspan": SVGTSpanElement;\n    "use": SVGUseElement;\n    "view": SVGViewElement;\n}\n\n/** @deprecated Directly use HTMLElementTagNameMap or SVGElementTagNameMap as appropriate, instead. */\ntype ElementTagNameMap = HTMLElementTagNameMap & Pick<SVGElementTagNameMap, Exclude<keyof SVGElementTagNameMap, keyof HTMLElementTagNameMap>>;\n\ndeclare var Audio: {\n    new(src?: string): HTMLAudioElement;\n};\ndeclare var Image: {\n    new(width?: number, height?: number): HTMLImageElement;\n};\ndeclare var Option: {\n    new(text?: string, value?: string, defaultSelected?: boolean, selected?: boolean): HTMLOptionElement;\n};\n/** @deprecated This is a legacy alias of `navigator`. */\ndeclare var clientInformation: Navigator;\n/**\n * Returns true if the window has been closed, false otherwise.\n */\ndeclare var closed: boolean;\n/**\n * Defines a new custom element, mapping the given name to the given constructor as an autonomous custom element.\n */\ndeclare var customElements: CustomElementRegistry;\ndeclare var devicePixelRatio: number;\ndeclare var document: Document;\n/** @deprecated */\ndeclare var event: Event | undefined;\n/** @deprecated */\ndeclare var external: External;\ndeclare var frameElement: Element | null;\ndeclare var frames: WindowProxy;\ndeclare var history: History;\ndeclare var innerHeight: number;\ndeclare var innerWidth: number;\ndeclare var length: number;\ndeclare var location: Location;\n/**\n * Returns true if the location bar is visible; otherwise, returns false.\n */\ndeclare var locationbar: BarProp;\n/**\n * Returns true if the menu bar is visible; otherwise, returns false.\n */\ndeclare var menubar: BarProp;\n/** @deprecated */\ndeclare const name: void;\ndeclare var navigator: Navigator;\ndeclare var ondevicemotion: ((this: Window, ev: DeviceMotionEvent) => any) | null;\ndeclare var ondeviceorientation: ((this: Window, ev: DeviceOrientationEvent) => any) | null;\n/** @deprecated */\ndeclare var onorientationchange: ((this: Window, ev: Event) => any) | null;\ndeclare var opener: any;\n/** @deprecated */\ndeclare var orientation: number;\ndeclare var outerHeight: number;\ndeclare var outerWidth: number;\n/** @deprecated This is a legacy alias of `scrollX`. */\ndeclare var pageXOffset: number;\n/** @deprecated This is a legacy alias of `scrollY`. */\ndeclare var pageYOffset: number;\n/** Refers to either the parent WindowProxy, or itself. It can rarely be null e.g. for contentWindow of an iframe that is already removed from the parent. */\ndeclare var parent: WindowProxy;\n/**\n * Returns true if the personal bar is visible; otherwise, returns false.\n */\ndeclare var personalbar: BarProp;\ndeclare var screen: Screen;\ndeclare var screenLeft: number;\ndeclare var screenTop: number;\ndeclare var screenX: number;\ndeclare var screenY: number;\ndeclare var scrollX: number;\ndeclare var scrollY: number;\n/**\n * Returns true if the scrollbars are visible; otherwise, returns false.\n */\ndeclare var scrollbars: BarProp;\ndeclare var self: Window & typeof globalThis;\ndeclare var speechSynthesis: SpeechSynthesis;\n/** @deprecated */\ndeclare var status: string;\n/**\n * Returns true if the status bar is visible; otherwise, returns false.\n */\ndeclare var statusbar: BarProp;\n/**\n * Returns true if the toolbar is visible; otherwise, returns false.\n */\ndeclare var toolbar: BarProp;\ndeclare var top: WindowProxy | null;\ndeclare var visualViewport: VisualViewport;\ndeclare var window: Window & typeof globalThis;\ndeclare function alert(message?: any): void;\ndeclare function blur(): void;\ndeclare function cancelIdleCallback(handle: number): void;\n/** @deprecated */\ndeclare function captureEvents(): void;\n/**\n * Closes the window.\n */\ndeclare function close(): void;\ndeclare function confirm(message?: string): boolean;\n/**\n * Moves the focus to the window\'s browsing context, if any.\n */\ndeclare function focus(): void;\ndeclare function getComputedStyle(elt: Element, pseudoElt?: string | null): CSSStyleDeclaration;\ndeclare function getSelection(): Selection | null;\ndeclare function matchMedia(query: string): MediaQueryList;\ndeclare function moveBy(x: number, y: number): void;\ndeclare function moveTo(x: number, y: number): void;\ndeclare function open(url?: string | URL, target?: string, features?: string): WindowProxy | null;\n/**\n * Posts a message to the given window. Messages can be structured objects, e.g. nested objects and arrays, can contain JavaScript values (strings, numbers, Date objects, etc), and can contain certain data objects such as File Blob, FileList, and ArrayBuffer objects.\n *\n * Objects listed in the transfer member of options are transferred, not just cloned, meaning that they are no longer usable on the sending side.\n *\n * A target origin can be specified using the targetOrigin member of options. If not provided, it defaults to "/". This default restricts the message to same-origin targets only.\n *\n * If the origin of the target window doesn\'t match the given target origin, the message is discarded, to avoid information leakage. To send the message to the target regardless of origin, set the target origin to "*".\n *\n * Throws a "DataCloneError" DOMException if transfer array contains duplicate objects or if message could not be cloned.\n */\ndeclare function postMessage(message: any, targetOrigin: string, transfer?: Transferable[]): void;\ndeclare function postMessage(message: any, options?: WindowPostMessageOptions): void;\ndeclare function print(): void;\ndeclare function prompt(message?: string, _default?: string): string | null;\n/** @deprecated */\ndeclare function releaseEvents(): void;\ndeclare function requestIdleCallback(callback: IdleRequestCallback, options?: IdleRequestOptions): number;\ndeclare function resizeBy(x: number, y: number): void;\ndeclare function resizeTo(width: number, height: number): void;\ndeclare function scroll(options?: ScrollToOptions): void;\ndeclare function scroll(x: number, y: number): void;\ndeclare function scrollBy(options?: ScrollToOptions): void;\ndeclare function scrollBy(x: number, y: number): void;\ndeclare function scrollTo(options?: ScrollToOptions): void;\ndeclare function scrollTo(x: number, y: number): void;\n/**\n * Cancels the document load.\n */\ndeclare function stop(): void;\ndeclare function toString(): string;\n/**\n * Dispatches a synthetic event event to target and returns true if either event\'s cancelable attribute value is false or its preventDefault() method was not invoked, and false otherwise.\n */\ndeclare function dispatchEvent(event: Event): boolean;\ndeclare function cancelAnimationFrame(handle: number): void;\ndeclare function requestAnimationFrame(callback: FrameRequestCallback): number;\n/**\n * Fires when the user aborts the download.\n * @param ev The event.\n */\ndeclare var onabort: ((this: Window, ev: UIEvent) => any) | null;\ndeclare var onanimationcancel: ((this: Window, ev: AnimationEvent) => any) | null;\ndeclare var onanimationend: ((this: Window, ev: AnimationEvent) => any) | null;\ndeclare var onanimationiteration: ((this: Window, ev: AnimationEvent) => any) | null;\ndeclare var onanimationstart: ((this: Window, ev: AnimationEvent) => any) | null;\ndeclare var onauxclick: ((this: Window, ev: MouseEvent) => any) | null;\n/**\n * Fires when the object loses the input focus.\n * @param ev The focus event.\n */\ndeclare var onblur: ((this: Window, ev: FocusEvent) => any) | null;\n/**\n * Occurs when playback is possible, but would require further buffering.\n * @param ev The event.\n */\ndeclare var oncanplay: ((this: Window, ev: Event) => any) | null;\ndeclare var oncanplaythrough: ((this: Window, ev: Event) => any) | null;\n/**\n * Fires when the contents of the object or selection have changed.\n * @param ev The event.\n */\ndeclare var onchange: ((this: Window, ev: Event) => any) | null;\n/**\n * Fires when the user clicks the left mouse button on the object\n * @param ev The mouse event.\n */\ndeclare var onclick: ((this: Window, ev: MouseEvent) => any) | null;\ndeclare var onclose: ((this: Window, ev: Event) => any) | null;\n/**\n * Fires when the user clicks the right mouse button in the client area, opening the context menu.\n * @param ev The mouse event.\n */\ndeclare var oncontextmenu: ((this: Window, ev: MouseEvent) => any) | null;\ndeclare var oncuechange: ((this: Window, ev: Event) => any) | null;\n/**\n * Fires when the user double-clicks the object.\n * @param ev The mouse event.\n */\ndeclare var ondblclick: ((this: Window, ev: MouseEvent) => any) | null;\n/**\n * Fires on the source object continuously during a drag operation.\n * @param ev The event.\n */\ndeclare var ondrag: ((this: Window, ev: DragEvent) => any) | null;\n/**\n * Fires on the source object when the user releases the mouse at the close of a drag operation.\n * @param ev The event.\n */\ndeclare var ondragend: ((this: Window, ev: DragEvent) => any) | null;\n/**\n * Fires on the target element when the user drags the object to a valid drop target.\n * @param ev The drag event.\n */\ndeclare var ondragenter: ((this: Window, ev: DragEvent) => any) | null;\n/**\n * Fires on the target object when the user moves the mouse out of a valid drop target during a drag operation.\n * @param ev The drag event.\n */\ndeclare var ondragleave: ((this: Window, ev: DragEvent) => any) | null;\n/**\n * Fires on the target element continuously while the user drags the object over a valid drop target.\n * @param ev The event.\n */\ndeclare var ondragover: ((this: Window, ev: DragEvent) => any) | null;\n/**\n * Fires on the source object when the user starts to drag a text selection or selected object.\n * @param ev The event.\n */\ndeclare var ondragstart: ((this: Window, ev: DragEvent) => any) | null;\ndeclare var ondrop: ((this: Window, ev: DragEvent) => any) | null;\n/**\n * Occurs when the duration attribute is updated.\n * @param ev The event.\n */\ndeclare var ondurationchange: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when the media element is reset to its initial state.\n * @param ev The event.\n */\ndeclare var onemptied: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when the end of playback is reached.\n * @param ev The event\n */\ndeclare var onended: ((this: Window, ev: Event) => any) | null;\n/**\n * Fires when an error occurs during object loading.\n * @param ev The event.\n */\ndeclare var onerror: OnErrorEventHandler;\n/**\n * Fires when the object receives focus.\n * @param ev The event.\n */\ndeclare var onfocus: ((this: Window, ev: FocusEvent) => any) | null;\ndeclare var onformdata: ((this: Window, ev: FormDataEvent) => any) | null;\ndeclare var ongotpointercapture: ((this: Window, ev: PointerEvent) => any) | null;\ndeclare var oninput: ((this: Window, ev: Event) => any) | null;\ndeclare var oninvalid: ((this: Window, ev: Event) => any) | null;\n/**\n * Fires when the user presses a key.\n * @param ev The keyboard event\n */\ndeclare var onkeydown: ((this: Window, ev: KeyboardEvent) => any) | null;\n/**\n * Fires when the user presses an alphanumeric key.\n * @param ev The event.\n * @deprecated\n */\ndeclare var onkeypress: ((this: Window, ev: KeyboardEvent) => any) | null;\n/**\n * Fires when the user releases a key.\n * @param ev The keyboard event\n */\ndeclare var onkeyup: ((this: Window, ev: KeyboardEvent) => any) | null;\n/**\n * Fires immediately after the browser loads the object.\n * @param ev The event.\n */\ndeclare var onload: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when media data is loaded at the current playback position.\n * @param ev The event.\n */\ndeclare var onloadeddata: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when the duration and dimensions of the media have been determined.\n * @param ev The event.\n */\ndeclare var onloadedmetadata: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when Internet Explorer begins looking for media data.\n * @param ev The event.\n */\ndeclare var onloadstart: ((this: Window, ev: Event) => any) | null;\ndeclare var onlostpointercapture: ((this: Window, ev: PointerEvent) => any) | null;\n/**\n * Fires when the user clicks the object with either mouse button.\n * @param ev The mouse event.\n */\ndeclare var onmousedown: ((this: Window, ev: MouseEvent) => any) | null;\ndeclare var onmouseenter: ((this: Window, ev: MouseEvent) => any) | null;\ndeclare var onmouseleave: ((this: Window, ev: MouseEvent) => any) | null;\n/**\n * Fires when the user moves the mouse over the object.\n * @param ev The mouse event.\n */\ndeclare var onmousemove: ((this: Window, ev: MouseEvent) => any) | null;\n/**\n * Fires when the user moves the mouse pointer outside the boundaries of the object.\n * @param ev The mouse event.\n */\ndeclare var onmouseout: ((this: Window, ev: MouseEvent) => any) | null;\n/**\n * Fires when the user moves the mouse pointer into the object.\n * @param ev The mouse event.\n */\ndeclare var onmouseover: ((this: Window, ev: MouseEvent) => any) | null;\n/**\n * Fires when the user releases a mouse button while the mouse is over the object.\n * @param ev The mouse event.\n */\ndeclare var onmouseup: ((this: Window, ev: MouseEvent) => any) | null;\n/**\n * Occurs when playback is paused.\n * @param ev The event.\n */\ndeclare var onpause: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when the play method is requested.\n * @param ev The event.\n */\ndeclare var onplay: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when the audio or video has started playing.\n * @param ev The event.\n */\ndeclare var onplaying: ((this: Window, ev: Event) => any) | null;\ndeclare var onpointercancel: ((this: Window, ev: PointerEvent) => any) | null;\ndeclare var onpointerdown: ((this: Window, ev: PointerEvent) => any) | null;\ndeclare var onpointerenter: ((this: Window, ev: PointerEvent) => any) | null;\ndeclare var onpointerleave: ((this: Window, ev: PointerEvent) => any) | null;\ndeclare var onpointermove: ((this: Window, ev: PointerEvent) => any) | null;\ndeclare var onpointerout: ((this: Window, ev: PointerEvent) => any) | null;\ndeclare var onpointerover: ((this: Window, ev: PointerEvent) => any) | null;\ndeclare var onpointerup: ((this: Window, ev: PointerEvent) => any) | null;\n/**\n * Occurs to indicate progress while downloading media data.\n * @param ev The event.\n */\ndeclare var onprogress: ((this: Window, ev: ProgressEvent) => any) | null;\n/**\n * Occurs when the playback rate is increased or decreased.\n * @param ev The event.\n */\ndeclare var onratechange: ((this: Window, ev: Event) => any) | null;\n/**\n * Fires when the user resets a form.\n * @param ev The event.\n */\ndeclare var onreset: ((this: Window, ev: Event) => any) | null;\ndeclare var onresize: ((this: Window, ev: UIEvent) => any) | null;\n/**\n * Fires when the user repositions the scroll box in the scroll bar on the object.\n * @param ev The event.\n */\ndeclare var onscroll: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when the seek operation ends.\n * @param ev The event.\n */\ndeclare var onseeked: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when the current playback position is moved.\n * @param ev The event.\n */\ndeclare var onseeking: ((this: Window, ev: Event) => any) | null;\n/**\n * Fires when the current selection changes.\n * @param ev The event.\n */\ndeclare var onselect: ((this: Window, ev: Event) => any) | null;\ndeclare var onselectionchange: ((this: Window, ev: Event) => any) | null;\ndeclare var onselectstart: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when the download has stopped.\n * @param ev The event.\n */\ndeclare var onstalled: ((this: Window, ev: Event) => any) | null;\ndeclare var onsubmit: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs if the load operation has been intentionally halted.\n * @param ev The event.\n */\ndeclare var onsuspend: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs to indicate the current playback position.\n * @param ev The event.\n */\ndeclare var ontimeupdate: ((this: Window, ev: Event) => any) | null;\ndeclare var ontoggle: ((this: Window, ev: Event) => any) | null;\ndeclare var ontouchcancel: ((this: Window, ev: TouchEvent) => any) | null | undefined;\ndeclare var ontouchend: ((this: Window, ev: TouchEvent) => any) | null | undefined;\ndeclare var ontouchmove: ((this: Window, ev: TouchEvent) => any) | null | undefined;\ndeclare var ontouchstart: ((this: Window, ev: TouchEvent) => any) | null | undefined;\ndeclare var ontransitioncancel: ((this: Window, ev: TransitionEvent) => any) | null;\ndeclare var ontransitionend: ((this: Window, ev: TransitionEvent) => any) | null;\ndeclare var ontransitionrun: ((this: Window, ev: TransitionEvent) => any) | null;\ndeclare var ontransitionstart: ((this: Window, ev: TransitionEvent) => any) | null;\n/**\n * Occurs when the volume is changed, or playback is muted or unmuted.\n * @param ev The event.\n */\ndeclare var onvolumechange: ((this: Window, ev: Event) => any) | null;\n/**\n * Occurs when playback stops because the next frame of a video resource is not available.\n * @param ev The event.\n */\ndeclare var onwaiting: ((this: Window, ev: Event) => any) | null;\ndeclare var onwebkitanimationend: ((this: Window, ev: Event) => any) | null;\ndeclare var onwebkitanimationiteration: ((this: Window, ev: Event) => any) | null;\ndeclare var onwebkitanimationstart: ((this: Window, ev: Event) => any) | null;\ndeclare var onwebkittransitionend: ((this: Window, ev: Event) => any) | null;\ndeclare var onwheel: ((this: Window, ev: WheelEvent) => any) | null;\ndeclare var onafterprint: ((this: Window, ev: Event) => any) | null;\ndeclare var onbeforeprint: ((this: Window, ev: Event) => any) | null;\ndeclare var onbeforeunload: ((this: Window, ev: BeforeUnloadEvent) => any) | null;\ndeclare var ongamepadconnected: ((this: Window, ev: Event) => any) | null;\ndeclare var ongamepaddisconnected: ((this: Window, ev: Event) => any) | null;\ndeclare var onhashchange: ((this: Window, ev: Event) => any) | null;\ndeclare var onlanguagechange: ((this: Window, ev: Event) => any) | null;\ndeclare var onmessage: ((this: Window, ev: MessageEvent) => any) | null;\ndeclare var onmessageerror: ((this: Window, ev: MessageEvent) => any) | null;\ndeclare var onoffline: ((this: Window, ev: Event) => any) | null;\ndeclare var ononline: ((this: Window, ev: Event) => any) | null;\ndeclare var onpagehide: ((this: Window, ev: PageTransitionEvent) => any) | null;\ndeclare var onpageshow: ((this: Window, ev: PageTransitionEvent) => any) | null;\ndeclare var onpopstate: ((this: Window, ev: PopStateEvent) => any) | null;\ndeclare var onrejectionhandled: ((this: Window, ev: PromiseRejectionEvent) => any) | null;\ndeclare var onstorage: ((this: Window, ev: StorageEvent) => any) | null;\ndeclare var onunhandledrejection: ((this: Window, ev: PromiseRejectionEvent) => any) | null;\ndeclare var onunload: ((this: Window, ev: Event) => any) | null;\ndeclare var localStorage: Storage;\ndeclare var caches: CacheStorage;\ndeclare var crossOriginIsolated: boolean;\ndeclare var crypto: Crypto;\ndeclare var indexedDB: IDBFactory;\ndeclare var isSecureContext: boolean;\ndeclare var origin: string;\ndeclare var performance: Performance;\ndeclare function atob(data: string): string;\ndeclare function btoa(data: string): string;\ndeclare function clearInterval(handle?: number): void;\ndeclare function clearTimeout(handle?: number): void;\ndeclare function createImageBitmap(image: ImageBitmapSource, options?: ImageBitmapOptions): Promise<ImageBitmap>;\ndeclare function createImageBitmap(image: ImageBitmapSource, sx: number, sy: number, sw: number, sh: number, options?: ImageBitmapOptions): Promise<ImageBitmap>;\ndeclare function fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\ndeclare function queueMicrotask(callback: VoidFunction): void;\ndeclare function setInterval(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\ndeclare function setTimeout(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\ndeclare var sessionStorage: Storage;\ndeclare function addEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\ndeclare function addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\ndeclare function removeEventListener<K extends keyof WindowEventMap>(type: K, listener: (this: Window, ev: WindowEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\ndeclare function removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\ntype AlgorithmIdentifier = Algorithm | string;\ntype BigInteger = Uint8Array;\ntype BinaryData = ArrayBuffer | ArrayBufferView;\ntype BlobPart = BufferSource | Blob | string;\ntype BodyInit = ReadableStream | XMLHttpRequestBodyInit;\ntype BufferSource = ArrayBufferView | ArrayBuffer;\ntype COSEAlgorithmIdentifier = number;\ntype CSSNumberish = number;\ntype CanvasImageSource = HTMLOrSVGImageElement | HTMLVideoElement | HTMLCanvasElement | ImageBitmap;\ntype ClipboardItemData = Promise<ClipboardItemDataType>;\ntype ClipboardItemDataType = string | Blob;\ntype ClipboardItems = ClipboardItem[];\ntype ConstrainBoolean = boolean | ConstrainBooleanParameters;\ntype ConstrainDOMString = string | string[] | ConstrainDOMStringParameters;\ntype ConstrainDouble = number | ConstrainDoubleRange;\ntype ConstrainULong = number | ConstrainULongRange;\ntype DOMHighResTimeStamp = number;\ntype DOMTimeStamp = number;\ntype EventListenerOrEventListenerObject = EventListener | EventListenerObject;\ntype Float32List = Float32Array | GLfloat[];\ntype FormDataEntryValue = File | string;\ntype GLbitfield = number;\ntype GLboolean = boolean;\ntype GLclampf = number;\ntype GLenum = number;\ntype GLfloat = number;\ntype GLint = number;\ntype GLint64 = number;\ntype GLintptr = number;\ntype GLsizei = number;\ntype GLsizeiptr = number;\ntype GLuint = number;\ntype GLuint64 = number;\ntype HTMLOrSVGImageElement = HTMLImageElement | SVGImageElement;\ntype HTMLOrSVGScriptElement = HTMLScriptElement | SVGScriptElement;\ntype HashAlgorithmIdentifier = AlgorithmIdentifier;\ntype HeadersInit = string[][] | Record<string, string> | Headers;\ntype IDBValidKey = number | string | Date | BufferSource | IDBValidKey[];\ntype ImageBitmapSource = CanvasImageSource | Blob | ImageData;\ntype InsertPosition = "beforebegin" | "afterbegin" | "beforeend" | "afterend";\ntype Int32List = Int32Array | GLint[];\ntype LineAndPositionSetting = number | AutoKeyword;\ntype MediaProvider = MediaStream | MediaSource | Blob;\ntype MessageEventSource = WindowProxy | MessagePort | ServiceWorker;\ntype MutationRecordType = "attributes" | "characterData" | "childList";\ntype NamedCurve = string;\ntype OnBeforeUnloadEventHandler = OnBeforeUnloadEventHandlerNonNull | null;\ntype OnErrorEventHandler = OnErrorEventHandlerNonNull | null;\ntype PerformanceEntryList = PerformanceEntry[];\ntype ReadableStreamController<T> = ReadableStreamDefaultController<T>;\ntype ReadableStreamDefaultReadResult<T> = ReadableStreamDefaultReadValueResult<T> | ReadableStreamDefaultReadDoneResult;\ntype ReadableStreamReader<T> = ReadableStreamDefaultReader<T>;\ntype RenderingContext = CanvasRenderingContext2D | ImageBitmapRenderingContext | WebGLRenderingContext | WebGL2RenderingContext;\ntype RequestInfo = Request | string;\ntype TexImageSource = ImageBitmap | ImageData | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement;\ntype TimerHandler = string | Function;\ntype Transferable = ArrayBuffer | MessagePort | ImageBitmap;\ntype Uint32List = Uint32Array | GLuint[];\ntype UvmEntries = UvmEntry[];\ntype UvmEntry = number[];\ntype VibratePattern = number | number[];\ntype WindowProxy = Window;\ntype XMLHttpRequestBodyInit = Blob | BufferSource | FormData | URLSearchParams | string;\ntype AlignSetting = "center" | "end" | "left" | "right" | "start";\ntype AnimationPlayState = "finished" | "idle" | "paused" | "running";\ntype AnimationReplaceState = "active" | "persisted" | "removed";\ntype AppendMode = "segments" | "sequence";\ntype AttestationConveyancePreference = "direct" | "enterprise" | "indirect" | "none";\ntype AudioContextLatencyCategory = "balanced" | "interactive" | "playback";\ntype AudioContextState = "closed" | "running" | "suspended";\ntype AuthenticatorAttachment = "cross-platform" | "platform";\ntype AuthenticatorTransport = "ble" | "internal" | "nfc" | "usb";\ntype AutoKeyword = "auto";\ntype AutomationRate = "a-rate" | "k-rate";\ntype BinaryType = "arraybuffer" | "blob";\ntype BiquadFilterType = "allpass" | "bandpass" | "highpass" | "highshelf" | "lowpass" | "lowshelf" | "notch" | "peaking";\ntype CanPlayTypeResult = "" | "maybe" | "probably";\ntype CanvasDirection = "inherit" | "ltr" | "rtl";\ntype CanvasFillRule = "evenodd" | "nonzero";\ntype CanvasFontKerning = "auto" | "none" | "normal";\ntype CanvasFontStretch = "condensed" | "expanded" | "extra-condensed" | "extra-expanded" | "normal" | "semi-condensed" | "semi-expanded" | "ultra-condensed" | "ultra-expanded";\ntype CanvasFontVariantCaps = "all-petite-caps" | "all-small-caps" | "normal" | "petite-caps" | "small-caps" | "titling-caps" | "unicase";\ntype CanvasLineCap = "butt" | "round" | "square";\ntype CanvasLineJoin = "bevel" | "miter" | "round";\ntype CanvasTextAlign = "center" | "end" | "left" | "right" | "start";\ntype CanvasTextBaseline = "alphabetic" | "bottom" | "hanging" | "ideographic" | "middle" | "top";\ntype CanvasTextRendering = "auto" | "geometricPrecision" | "optimizeLegibility" | "optimizeSpeed";\ntype ChannelCountMode = "clamped-max" | "explicit" | "max";\ntype ChannelInterpretation = "discrete" | "speakers";\ntype ClientTypes = "all" | "sharedworker" | "window" | "worker";\ntype ColorGamut = "p3" | "rec2020" | "srgb";\ntype ColorSpaceConversion = "default" | "none";\ntype CompositeOperation = "accumulate" | "add" | "replace";\ntype CompositeOperationOrAuto = "accumulate" | "add" | "auto" | "replace";\ntype ConnectionType = "bluetooth" | "cellular" | "ethernet" | "mixed" | "none" | "other" | "unknown" | "wifi";\ntype CredentialMediationRequirement = "optional" | "required" | "silent";\ntype DOMParserSupportedType = "application/xhtml+xml" | "application/xml" | "image/svg+xml" | "text/html" | "text/xml";\ntype DirectionSetting = "" | "lr" | "rl";\ntype DisplayCaptureSurfaceType = "application" | "browser" | "monitor" | "window";\ntype DistanceModelType = "exponential" | "inverse" | "linear";\ntype DocumentReadyState = "complete" | "interactive" | "loading";\ntype EndOfStreamError = "decode" | "network";\ntype EndingType = "native" | "transparent";\ntype FillMode = "auto" | "backwards" | "both" | "forwards" | "none";\ntype FontFaceLoadStatus = "error" | "loaded" | "loading" | "unloaded";\ntype FontFaceSetLoadStatus = "loaded" | "loading";\ntype FullscreenNavigationUI = "auto" | "hide" | "show";\ntype GamepadHapticActuatorType = "vibration";\ntype GamepadMappingType = "" | "standard" | "xr-standard";\ntype HdrMetadataType = "smpteSt2086" | "smpteSt2094-10" | "smpteSt2094-40";\ntype IDBCursorDirection = "next" | "nextunique" | "prev" | "prevunique";\ntype IDBRequestReadyState = "done" | "pending";\ntype IDBTransactionMode = "readonly" | "readwrite" | "versionchange";\ntype ImageOrientation = "flipY" | "none";\ntype ImageSmoothingQuality = "high" | "low" | "medium";\ntype IterationCompositeOperation = "accumulate" | "replace";\ntype KeyFormat = "jwk" | "pkcs8" | "raw" | "spki";\ntype KeyType = "private" | "public" | "secret";\ntype KeyUsage = "decrypt" | "deriveBits" | "deriveKey" | "encrypt" | "sign" | "unwrapKey" | "verify" | "wrapKey";\ntype LineAlignSetting = "center" | "end" | "start";\ntype MediaDecodingType = "file" | "media-source" | "webrtc";\ntype MediaDeviceKind = "audioinput" | "audiooutput" | "videoinput";\ntype MediaEncodingType = "record" | "webrtc";\ntype MediaKeyMessageType = "individualization-request" | "license-release" | "license-renewal" | "license-request";\ntype MediaKeySessionClosedReason = "closed-by-application" | "hardware-context-reset" | "internal-error" | "release-acknowledged" | "resource-evicted";\ntype MediaKeySessionType = "persistent-license" | "temporary";\ntype MediaKeyStatus = "expired" | "internal-error" | "output-downscaled" | "output-restricted" | "released" | "status-pending" | "usable" | "usable-in-future";\ntype MediaKeysRequirement = "not-allowed" | "optional" | "required";\ntype MediaSessionAction = "hangup" | "nexttrack" | "pause" | "play" | "previoustrack" | "seekbackward" | "seekforward" | "seekto" | "skipad" | "stop" | "togglecamera" | "togglemicrophone";\ntype MediaSessionPlaybackState = "none" | "paused" | "playing";\ntype MediaStreamTrackState = "ended" | "live";\ntype NavigationType = "back_forward" | "navigate" | "prerender" | "reload";\ntype NotificationDirection = "auto" | "ltr" | "rtl";\ntype NotificationPermission = "default" | "denied" | "granted";\ntype OrientationLockType = "any" | "landscape" | "landscape-primary" | "landscape-secondary" | "natural" | "portrait" | "portrait-primary" | "portrait-secondary";\ntype OrientationType = "landscape-primary" | "landscape-secondary" | "portrait-primary" | "portrait-secondary";\ntype OscillatorType = "custom" | "sawtooth" | "sine" | "square" | "triangle";\ntype OverSampleType = "2x" | "4x" | "none";\ntype PanningModelType = "HRTF" | "equalpower";\ntype PaymentComplete = "fail" | "success" | "unknown";\ntype PermissionName = "gamepad" | "geolocation" | "notifications" | "persistent-storage" | "push" | "screen-wake-lock";\ntype PermissionState = "denied" | "granted" | "prompt";\ntype PlaybackDirection = "alternate" | "alternate-reverse" | "normal" | "reverse";\ntype PositionAlignSetting = "auto" | "center" | "line-left" | "line-right";\ntype PredefinedColorSpace = "display-p3" | "srgb";\ntype PremultiplyAlpha = "default" | "none" | "premultiply";\ntype PresentationStyle = "attachment" | "inline" | "unspecified";\ntype PublicKeyCredentialType = "public-key";\ntype PushEncryptionKeyName = "auth" | "p256dh";\ntype PushPermissionState = "denied" | "granted" | "prompt";\ntype RTCBundlePolicy = "balanced" | "max-bundle" | "max-compat";\ntype RTCDataChannelState = "closed" | "closing" | "connecting" | "open";\ntype RTCDegradationPreference = "balanced" | "maintain-framerate" | "maintain-resolution";\ntype RTCDtlsTransportState = "closed" | "connected" | "connecting" | "failed" | "new";\ntype RTCIceCandidateType = "host" | "prflx" | "relay" | "srflx";\ntype RTCIceComponent = "rtcp" | "rtp";\ntype RTCIceConnectionState = "checking" | "closed" | "completed" | "connected" | "disconnected" | "failed" | "new";\ntype RTCIceCredentialType = "password";\ntype RTCIceGathererState = "complete" | "gathering" | "new";\ntype RTCIceGatheringState = "complete" | "gathering" | "new";\ntype RTCIceProtocol = "tcp" | "udp";\ntype RTCIceTcpCandidateType = "active" | "passive" | "so";\ntype RTCIceTransportPolicy = "all" | "relay";\ntype RTCIceTransportState = "checking" | "closed" | "completed" | "connected" | "disconnected" | "failed" | "new";\ntype RTCPeerConnectionState = "closed" | "connected" | "connecting" | "disconnected" | "failed" | "new";\ntype RTCPriorityType = "high" | "low" | "medium" | "very-low";\ntype RTCRtcpMuxPolicy = "require";\ntype RTCRtpTransceiverDirection = "inactive" | "recvonly" | "sendonly" | "sendrecv" | "stopped";\ntype RTCSdpType = "answer" | "offer" | "pranswer" | "rollback";\ntype RTCSignalingState = "closed" | "have-local-offer" | "have-local-pranswer" | "have-remote-offer" | "have-remote-pranswer" | "stable";\ntype RTCStatsIceCandidatePairState = "failed" | "frozen" | "in-progress" | "inprogress" | "succeeded" | "waiting";\ntype RTCStatsType = "candidate-pair" | "certificate" | "codec" | "csrc" | "data-channel" | "inbound-rtp" | "local-candidate" | "media-source" | "outbound-rtp" | "peer-connection" | "remote-candidate" | "remote-inbound-rtp" | "remote-outbound-rtp" | "track" | "transport";\ntype ReadyState = "closed" | "ended" | "open";\ntype RecordingState = "inactive" | "paused" | "recording";\ntype ReferrerPolicy = "" | "no-referrer" | "no-referrer-when-downgrade" | "origin" | "origin-when-cross-origin" | "same-origin" | "strict-origin" | "strict-origin-when-cross-origin" | "unsafe-url";\ntype RemotePlaybackState = "connected" | "connecting" | "disconnected";\ntype RequestCache = "default" | "force-cache" | "no-cache" | "no-store" | "only-if-cached" | "reload";\ntype RequestCredentials = "include" | "omit" | "same-origin";\ntype RequestDestination = "" | "audio" | "audioworklet" | "document" | "embed" | "font" | "frame" | "iframe" | "image" | "manifest" | "object" | "paintworklet" | "report" | "script" | "sharedworker" | "style" | "track" | "video" | "worker" | "xslt";\ntype RequestMode = "cors" | "navigate" | "no-cors" | "same-origin";\ntype RequestRedirect = "error" | "follow" | "manual";\ntype ResidentKeyRequirement = "discouraged" | "preferred" | "required";\ntype ResizeObserverBoxOptions = "border-box" | "content-box" | "device-pixel-content-box";\ntype ResizeQuality = "high" | "low" | "medium" | "pixelated";\ntype ResponseType = "basic" | "cors" | "default" | "error" | "opaque" | "opaqueredirect";\ntype ScrollBehavior = "auto" | "smooth";\ntype ScrollLogicalPosition = "center" | "end" | "nearest" | "start";\ntype ScrollRestoration = "auto" | "manual";\ntype ScrollSetting = "" | "up";\ntype SecurityPolicyViolationEventDisposition = "enforce" | "report";\ntype SelectionMode = "end" | "preserve" | "select" | "start";\ntype ServiceWorkerState = "activated" | "activating" | "installed" | "installing" | "parsed" | "redundant";\ntype ServiceWorkerUpdateViaCache = "all" | "imports" | "none";\ntype ShadowRootMode = "closed" | "open";\ntype SlotAssignmentMode = "manual" | "named";\ntype SpeechSynthesisErrorCode = "audio-busy" | "audio-hardware" | "canceled" | "interrupted" | "invalid-argument" | "language-unavailable" | "network" | "not-allowed" | "synthesis-failed" | "synthesis-unavailable" | "text-too-long" | "voice-unavailable";\ntype TextTrackKind = "captions" | "chapters" | "descriptions" | "metadata" | "subtitles";\ntype TextTrackMode = "disabled" | "hidden" | "showing";\ntype TouchType = "direct" | "stylus";\ntype TransferFunction = "hlg" | "pq" | "srgb";\ntype UserVerificationRequirement = "discouraged" | "preferred" | "required";\ntype VideoFacingModeEnum = "environment" | "left" | "right" | "user";\ntype VisibilityState = "hidden" | "visible";\ntype WebGLPowerPreference = "default" | "high-performance" | "low-power";\ntype WorkerType = "classic" | "module";\ntype XMLHttpRequestResponseType = "" | "arraybuffer" | "blob" | "document" | "json" | "text";\n',"lib.dom.iterable.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/////////////////////////////\n/// Window Iterable APIs\n/////////////////////////////\n\ninterface AudioParam {\n    setValueCurveAtTime(values: Iterable<number>, startTime: number, duration: number): AudioParam;\n}\n\ninterface AudioParamMap extends ReadonlyMap<string, AudioParam> {\n}\n\ninterface BaseAudioContext {\n    createIIRFilter(feedforward: Iterable<number>, feedback: Iterable<number>): IIRFilterNode;\n    createPeriodicWave(real: Iterable<number>, imag: Iterable<number>, constraints?: PeriodicWaveConstraints): PeriodicWave;\n}\n\ninterface CSSRuleList {\n    [Symbol.iterator](): IterableIterator<CSSRule>;\n}\n\ninterface CSSStyleDeclaration {\n    [Symbol.iterator](): IterableIterator<string>;\n}\n\ninterface Cache {\n    addAll(requests: Iterable<RequestInfo>): Promise<void>;\n}\n\ninterface CanvasPathDrawingStyles {\n    setLineDash(segments: Iterable<number>): void;\n}\n\ninterface DOMRectList {\n    [Symbol.iterator](): IterableIterator<DOMRect>;\n}\n\ninterface DOMStringList {\n    [Symbol.iterator](): IterableIterator<string>;\n}\n\ninterface DOMTokenList {\n    [Symbol.iterator](): IterableIterator<string>;\n    entries(): IterableIterator<[number, string]>;\n    keys(): IterableIterator<number>;\n    values(): IterableIterator<string>;\n}\n\ninterface DataTransferItemList {\n    [Symbol.iterator](): IterableIterator<DataTransferItem>;\n}\n\ninterface FileList {\n    [Symbol.iterator](): IterableIterator<File>;\n}\n\ninterface FontFaceSet extends Set<FontFace> {\n}\n\ninterface FormData {\n    [Symbol.iterator](): IterableIterator<[string, FormDataEntryValue]>;\n    /**\n     * Returns an array of key, value pairs for every entry in the list.\n     */\n    entries(): IterableIterator<[string, FormDataEntryValue]>;\n    /**\n     * Returns a list of keys in the list.\n     */\n    keys(): IterableIterator<string>;\n    /**\n     * Returns a list of values in the list.\n     */\n    values(): IterableIterator<FormDataEntryValue>;\n}\n\ninterface HTMLAllCollection {\n    [Symbol.iterator](): IterableIterator<Element>;\n}\n\ninterface HTMLCollectionBase {\n    [Symbol.iterator](): IterableIterator<Element>;\n}\n\ninterface HTMLCollectionOf<T extends Element> {\n    [Symbol.iterator](): IterableIterator<T>;\n}\n\ninterface HTMLFormElement {\n    [Symbol.iterator](): IterableIterator<Element>;\n}\n\ninterface HTMLSelectElement {\n    [Symbol.iterator](): IterableIterator<HTMLOptionElement>;\n}\n\ninterface Headers {\n    [Symbol.iterator](): IterableIterator<[string, string]>;\n    /**\n     * Returns an iterator allowing to go through all key/value pairs contained in this object.\n     */\n    entries(): IterableIterator<[string, string]>;\n    /**\n     * Returns an iterator allowing to go through all keys of the key/value pairs contained in this object.\n     */\n    keys(): IterableIterator<string>;\n    /**\n     * Returns an iterator allowing to go through all values of the key/value pairs contained in this object.\n     */\n    values(): IterableIterator<string>;\n}\n\ninterface IDBDatabase {\n    /**\n     * Returns a new transaction with the given mode ("readonly" or "readwrite") and scope which can be a single object store name or an array of names.\n     */\n    transaction(storeNames: string | Iterable<string>, mode?: IDBTransactionMode): IDBTransaction;\n}\n\ninterface IDBObjectStore {\n    /**\n     * Creates a new index in store with the given name, keyPath and options and returns a new IDBIndex. If the keyPath and options define constraints that cannot be satisfied with the data already in store the upgrade transaction will abort with a "ConstraintError" DOMException.\n     *\n     * Throws an "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    createIndex(name: string, keyPath: string | Iterable<string>, options?: IDBIndexParameters): IDBIndex;\n}\n\ninterface MediaKeyStatusMap {\n    [Symbol.iterator](): IterableIterator<[BufferSource, MediaKeyStatus]>;\n    entries(): IterableIterator<[BufferSource, MediaKeyStatus]>;\n    keys(): IterableIterator<BufferSource>;\n    values(): IterableIterator<MediaKeyStatus>;\n}\n\ninterface MediaList {\n    [Symbol.iterator](): IterableIterator<string>;\n}\n\ninterface MessageEvent<T = any> {\n    /** @deprecated */\n    initMessageEvent(type: string, bubbles?: boolean, cancelable?: boolean, data?: any, origin?: string, lastEventId?: string, source?: MessageEventSource | null, ports?: Iterable<MessagePort>): void;\n}\n\ninterface MimeTypeArray {\n    [Symbol.iterator](): IterableIterator<MimeType>;\n}\n\ninterface NamedNodeMap {\n    [Symbol.iterator](): IterableIterator<Attr>;\n}\n\ninterface Navigator {\n    requestMediaKeySystemAccess(keySystem: string, supportedConfigurations: Iterable<MediaKeySystemConfiguration>): Promise<MediaKeySystemAccess>;\n    vibrate(pattern: Iterable<number>): boolean;\n}\n\ninterface NodeList {\n    [Symbol.iterator](): IterableIterator<Node>;\n    /**\n     * Returns an array of key, value pairs for every entry in the list.\n     */\n    entries(): IterableIterator<[number, Node]>;\n    /**\n     * Returns an list of keys in the list.\n     */\n    keys(): IterableIterator<number>;\n    /**\n     * Returns an list of values in the list.\n     */\n    values(): IterableIterator<Node>;\n}\n\ninterface NodeListOf<TNode extends Node> {\n    [Symbol.iterator](): IterableIterator<TNode>;\n    /**\n     * Returns an array of key, value pairs for every entry in the list.\n     */\n    entries(): IterableIterator<[number, TNode]>;\n    /**\n     * Returns an list of keys in the list.\n     */\n    keys(): IterableIterator<number>;\n    /**\n     * Returns an list of values in the list.\n     */\n    values(): IterableIterator<TNode>;\n}\n\ninterface Plugin {\n    [Symbol.iterator](): IterableIterator<MimeType>;\n}\n\ninterface PluginArray {\n    [Symbol.iterator](): IterableIterator<Plugin>;\n}\n\ninterface RTCStatsReport extends ReadonlyMap<string, any> {\n}\n\ninterface ReadableStream<R = any> {\n    [Symbol.iterator](): IterableIterator<any>;\n    entries(): IterableIterator<[number, any]>;\n    keys(): IterableIterator<number>;\n    values(): IterableIterator<any>;\n}\n\ninterface SVGLengthList {\n    [Symbol.iterator](): IterableIterator<SVGLength>;\n}\n\ninterface SVGNumberList {\n    [Symbol.iterator](): IterableIterator<SVGNumber>;\n}\n\ninterface SVGPointList {\n    [Symbol.iterator](): IterableIterator<DOMPoint>;\n}\n\ninterface SVGStringList {\n    [Symbol.iterator](): IterableIterator<string>;\n}\n\ninterface SVGTransformList {\n    [Symbol.iterator](): IterableIterator<SVGTransform>;\n}\n\ninterface SourceBufferList {\n    [Symbol.iterator](): IterableIterator<SourceBuffer>;\n}\n\ninterface SpeechRecognitionResult {\n    [Symbol.iterator](): IterableIterator<SpeechRecognitionAlternative>;\n}\n\ninterface SpeechRecognitionResultList {\n    [Symbol.iterator](): IterableIterator<SpeechRecognitionResult>;\n}\n\ninterface StyleSheetList {\n    [Symbol.iterator](): IterableIterator<CSSStyleSheet>;\n}\n\ninterface SubtleCrypto {\n    deriveKey(algorithm: AlgorithmIdentifier | EcdhKeyDeriveParams | HkdfParams | Pbkdf2Params, baseKey: CryptoKey, derivedKeyType: AlgorithmIdentifier | AesDerivedKeyParams | HmacImportParams | HkdfParams | Pbkdf2Params, extractable: boolean, keyUsages: Iterable<KeyUsage>): Promise<CryptoKey>;\n    generateKey(algorithm: RsaHashedKeyGenParams | EcKeyGenParams, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKeyPair>;\n    generateKey(algorithm: AesKeyGenParams | HmacKeyGenParams | Pbkdf2Params, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    generateKey(algorithm: AlgorithmIdentifier, extractable: boolean, keyUsages: Iterable<KeyUsage>): Promise<CryptoKeyPair | CryptoKey>;\n    importKey(format: "jwk", keyData: JsonWebKey, algorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    importKey(format: Exclude<KeyFormat, "jwk">, keyData: BufferSource, algorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: Iterable<KeyUsage>): Promise<CryptoKey>;\n    unwrapKey(format: KeyFormat, wrappedKey: BufferSource, unwrappingKey: CryptoKey, unwrapAlgorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams, unwrappedKeyAlgorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: Iterable<KeyUsage>): Promise<CryptoKey>;\n}\n\ninterface TextTrackCueList {\n    [Symbol.iterator](): IterableIterator<TextTrackCue>;\n}\n\ninterface TextTrackList {\n    [Symbol.iterator](): IterableIterator<TextTrack>;\n}\n\ninterface TouchList {\n    [Symbol.iterator](): IterableIterator<Touch>;\n}\n\ninterface URLSearchParams {\n    [Symbol.iterator](): IterableIterator<[string, string]>;\n    /**\n     * Returns an array of key, value pairs for every entry in the search params.\n     */\n    entries(): IterableIterator<[string, string]>;\n    /**\n     * Returns a list of keys in the search params.\n     */\n    keys(): IterableIterator<string>;\n    /**\n     * Returns a list of values in the search params.\n     */\n    values(): IterableIterator<string>;\n}\n\ninterface WEBGL_draw_buffers {\n    drawBuffersWEBGL(buffers: Iterable<GLenum>): void;\n}\n\ninterface WebGL2RenderingContextBase {\n    clearBufferfv(buffer: GLenum, drawbuffer: GLint, values: Iterable<GLfloat>, srcOffset?: GLuint): void;\n    clearBufferiv(buffer: GLenum, drawbuffer: GLint, values: Iterable<GLint>, srcOffset?: GLuint): void;\n    clearBufferuiv(buffer: GLenum, drawbuffer: GLint, values: Iterable<GLuint>, srcOffset?: GLuint): void;\n    drawBuffers(buffers: Iterable<GLenum>): void;\n    getActiveUniforms(program: WebGLProgram, uniformIndices: Iterable<GLuint>, pname: GLenum): any;\n    getUniformIndices(program: WebGLProgram, uniformNames: Iterable<string>): Iterable<GLuint> | null;\n    invalidateFramebuffer(target: GLenum, attachments: Iterable<GLenum>): void;\n    invalidateSubFramebuffer(target: GLenum, attachments: Iterable<GLenum>, x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    transformFeedbackVaryings(program: WebGLProgram, varyings: Iterable<string>, bufferMode: GLenum): void;\n    uniform1uiv(location: WebGLUniformLocation | null, data: Iterable<GLuint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2uiv(location: WebGLUniformLocation | null, data: Iterable<GLuint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3uiv(location: WebGLUniformLocation | null, data: Iterable<GLuint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4uiv(location: WebGLUniformLocation | null, data: Iterable<GLuint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2x3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2x4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3x2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3x4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4x2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4x3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    vertexAttribI4iv(index: GLuint, values: Iterable<GLint>): void;\n    vertexAttribI4uiv(index: GLuint, values: Iterable<GLuint>): void;\n}\n\ninterface WebGL2RenderingContextOverloads {\n    uniform1fv(location: WebGLUniformLocation | null, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform1iv(location: WebGLUniformLocation | null, data: Iterable<GLint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2fv(location: WebGLUniformLocation | null, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2iv(location: WebGLUniformLocation | null, data: Iterable<GLint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3fv(location: WebGLUniformLocation | null, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3iv(location: WebGLUniformLocation | null, data: Iterable<GLint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4fv(location: WebGLUniformLocation | null, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4iv(location: WebGLUniformLocation | null, data: Iterable<GLint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n}\n\ninterface WebGLRenderingContextBase {\n    vertexAttrib1fv(index: GLuint, values: Iterable<GLfloat>): void;\n    vertexAttrib2fv(index: GLuint, values: Iterable<GLfloat>): void;\n    vertexAttrib3fv(index: GLuint, values: Iterable<GLfloat>): void;\n    vertexAttrib4fv(index: GLuint, values: Iterable<GLfloat>): void;\n}\n\ninterface WebGLRenderingContextOverloads {\n    uniform1fv(location: WebGLUniformLocation | null, v: Iterable<GLfloat>): void;\n    uniform1iv(location: WebGLUniformLocation | null, v: Iterable<GLint>): void;\n    uniform2fv(location: WebGLUniformLocation | null, v: Iterable<GLfloat>): void;\n    uniform2iv(location: WebGLUniformLocation | null, v: Iterable<GLint>): void;\n    uniform3fv(location: WebGLUniformLocation | null, v: Iterable<GLfloat>): void;\n    uniform3iv(location: WebGLUniformLocation | null, v: Iterable<GLint>): void;\n    uniform4fv(location: WebGLUniformLocation | null, v: Iterable<GLfloat>): void;\n    uniform4iv(location: WebGLUniformLocation | null, v: Iterable<GLint>): void;\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Iterable<GLfloat>): void;\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Iterable<GLfloat>): void;\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Iterable<GLfloat>): void;\n}\n',"lib.es2015.collection.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface Map<K, V> {\n    clear(): void;\n    delete(key: K): boolean;\n    forEach(callbackfn: (value: V, key: K, map: Map<K, V>) => void, thisArg?: any): void;\n    get(key: K): V | undefined;\n    has(key: K): boolean;\n    set(key: K, value: V): this;\n    readonly size: number;\n}\n\ninterface MapConstructor {\n    new(): Map<any, any>;\n    new<K, V>(entries?: readonly (readonly [K, V])[] | null): Map<K, V>;\n    readonly prototype: Map<any, any>;\n}\ndeclare var Map: MapConstructor;\n\ninterface ReadonlyMap<K, V> {\n    forEach(callbackfn: (value: V, key: K, map: ReadonlyMap<K, V>) => void, thisArg?: any): void;\n    get(key: K): V | undefined;\n    has(key: K): boolean;\n    readonly size: number;\n}\n\ninterface WeakMap<K extends object, V> {\n    delete(key: K): boolean;\n    get(key: K): V | undefined;\n    has(key: K): boolean;\n    set(key: K, value: V): this;\n}\n\ninterface WeakMapConstructor {\n    new <K extends object = object, V = any>(entries?: readonly [K, V][] | null): WeakMap<K, V>;\n    readonly prototype: WeakMap<object, any>;\n}\ndeclare var WeakMap: WeakMapConstructor;\n\ninterface Set<T> {\n    add(value: T): this;\n    clear(): void;\n    delete(value: T): boolean;\n    forEach(callbackfn: (value: T, value2: T, set: Set<T>) => void, thisArg?: any): void;\n    has(value: T): boolean;\n    readonly size: number;\n}\n\ninterface SetConstructor {\n    new <T = any>(values?: readonly T[] | null): Set<T>;\n    readonly prototype: Set<any>;\n}\ndeclare var Set: SetConstructor;\n\ninterface ReadonlySet<T> {\n    forEach(callbackfn: (value: T, value2: T, set: ReadonlySet<T>) => void, thisArg?: any): void;\n    has(value: T): boolean;\n    readonly size: number;\n}\n\ninterface WeakSet<T extends object> {\n    add(value: T): this;\n    delete(value: T): boolean;\n    has(value: T): boolean;\n}\n\ninterface WeakSetConstructor {\n    new <T extends object = object>(values?: readonly T[] | null): WeakSet<T>;\n    readonly prototype: WeakSet<object>;\n}\ndeclare var WeakSet: WeakSetConstructor;\n',"lib.es2015.core.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface Array<T> {\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find<S extends T>(predicate: (this: void, value: T, index: number, obj: T[]) => value is S, thisArg?: any): S | undefined;\n    find(predicate: (value: T, index: number, obj: T[]) => unknown, thisArg?: any): T | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: T, index: number, obj: T[]) => unknown, thisArg?: any): number;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: T, start?: number, end?: number): this;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n}\n\ninterface ArrayConstructor {\n    /**\n     * Creates an array from an array-like object.\n     * @param arrayLike An array-like object to convert to an array.\n     */\n    from<T>(arrayLike: ArrayLike<T>): T[];\n\n    /**\n     * Creates an array from an iterable object.\n     * @param arrayLike An array-like object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of \'this\' used to invoke the mapfn.\n     */\n    from<T, U>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => U, thisArg?: any): U[];\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of<T>(...items: T[]): T[];\n}\n\ninterface DateConstructor {\n    new (value: number | string | Date): Date;\n}\n\ninterface Function {\n    /**\n     * Returns the name of the function. Function names are read-only and can not be changed.\n     */\n    readonly name: string;\n}\n\ninterface Math {\n    /**\n     * Returns the number of leading zero bits in the 32-bit binary representation of a number.\n     * @param x A numeric expression.\n     */\n    clz32(x: number): number;\n\n    /**\n     * Returns the result of 32-bit multiplication of two numbers.\n     * @param x First number\n     * @param y Second number\n     */\n    imul(x: number, y: number): number;\n\n    /**\n     * Returns the sign of the x, indicating whether x is positive, negative or zero.\n     * @param x The numeric expression to test\n     */\n    sign(x: number): number;\n\n    /**\n     * Returns the base 10 logarithm of a number.\n     * @param x A numeric expression.\n     */\n    log10(x: number): number;\n\n    /**\n     * Returns the base 2 logarithm of a number.\n     * @param x A numeric expression.\n     */\n    log2(x: number): number;\n\n    /**\n     * Returns the natural logarithm of 1 + x.\n     * @param x A numeric expression.\n     */\n    log1p(x: number): number;\n\n    /**\n     * Returns the result of (e^x - 1), which is an implementation-dependent approximation to\n     * subtracting 1 from the exponential function of x (e raised to the power of x, where e\n     * is the base of the natural logarithms).\n     * @param x A numeric expression.\n     */\n    expm1(x: number): number;\n\n    /**\n     * Returns the hyperbolic cosine of a number.\n     * @param x A numeric expression that contains an angle measured in radians.\n     */\n    cosh(x: number): number;\n\n    /**\n     * Returns the hyperbolic sine of a number.\n     * @param x A numeric expression that contains an angle measured in radians.\n     */\n    sinh(x: number): number;\n\n    /**\n     * Returns the hyperbolic tangent of a number.\n     * @param x A numeric expression that contains an angle measured in radians.\n     */\n    tanh(x: number): number;\n\n    /**\n     * Returns the inverse hyperbolic cosine of a number.\n     * @param x A numeric expression that contains an angle measured in radians.\n     */\n    acosh(x: number): number;\n\n    /**\n     * Returns the inverse hyperbolic sine of a number.\n     * @param x A numeric expression that contains an angle measured in radians.\n     */\n    asinh(x: number): number;\n\n    /**\n     * Returns the inverse hyperbolic tangent of a number.\n     * @param x A numeric expression that contains an angle measured in radians.\n     */\n    atanh(x: number): number;\n\n    /**\n     * Returns the square root of the sum of squares of its arguments.\n     * @param values Values to compute the square root for.\n     *     If no arguments are passed, the result is +0.\n     *     If there is only one argument, the result is the absolute value.\n     *     If any argument is +Infinity or -Infinity, the result is +Infinity.\n     *     If any argument is NaN, the result is NaN.\n     *     If all arguments are either +0 or −0, the result is +0.\n     */\n    hypot(...values: number[]): number;\n\n    /**\n     * Returns the integral part of the a numeric expression, x, removing any fractional digits.\n     * If x is already an integer, the result is x.\n     * @param x A numeric expression.\n     */\n    trunc(x: number): number;\n\n    /**\n     * Returns the nearest single precision float representation of a number.\n     * @param x A numeric expression.\n     */\n    fround(x: number): number;\n\n    /**\n     * Returns an implementation-dependent approximation to the cube root of number.\n     * @param x A numeric expression.\n     */\n    cbrt(x: number): number;\n}\n\ninterface NumberConstructor {\n    /**\n     * The value of Number.EPSILON is the difference between 1 and the smallest value greater than 1\n     * that is representable as a Number value, which is approximately:\n     * 2.2204460492503130808472633361816 x 10‍−‍16.\n     */\n    readonly EPSILON: number;\n\n    /**\n     * Returns true if passed value is finite.\n     * Unlike the global isFinite, Number.isFinite doesn\'t forcibly convert the parameter to a\n     * number. Only finite values of the type number, result in true.\n     * @param number A numeric value.\n     */\n    isFinite(number: unknown): boolean;\n\n    /**\n     * Returns true if the value passed is an integer, false otherwise.\n     * @param number A numeric value.\n     */\n    isInteger(number: unknown): boolean;\n\n    /**\n     * Returns a Boolean value that indicates whether a value is the reserved value NaN (not a\n     * number). Unlike the global isNaN(), Number.isNaN() doesn\'t forcefully convert the parameter\n     * to a number. Only values of the type number, that are also NaN, result in true.\n     * @param number A numeric value.\n     */\n    isNaN(number: unknown): boolean;\n\n    /**\n     * Returns true if the value passed is a safe integer.\n     * @param number A numeric value.\n     */\n    isSafeInteger(number: unknown): boolean;\n\n    /**\n     * The value of the largest integer n such that n and n + 1 are both exactly representable as\n     * a Number value.\n     * The value of Number.MAX_SAFE_INTEGER is 9007199254740991 2^53 − 1.\n     */\n    readonly MAX_SAFE_INTEGER: number;\n\n    /**\n     * The value of the smallest integer n such that n and n − 1 are both exactly representable as\n     * a Number value.\n     * The value of Number.MIN_SAFE_INTEGER is −9007199254740991 (−(2^53 − 1)).\n     */\n    readonly MIN_SAFE_INTEGER: number;\n\n    /**\n     * Converts a string to a floating-point number.\n     * @param string A string that contains a floating-point number.\n     */\n    parseFloat(string: string): number;\n\n    /**\n     * Converts A string to an integer.\n     * @param string A string to convert into a number.\n     * @param radix A value between 2 and 36 that specifies the base of the number in `string`.\n     * If this argument is not supplied, strings with a prefix of \'0x\' are considered hexadecimal.\n     * All other strings are considered decimal.\n     */\n    parseInt(string: string, radix?: number): number;\n}\n\ninterface ObjectConstructor {\n    /**\n     * Copy the values of all of the enumerable own properties from one or more source objects to a\n     * target object. Returns the target object.\n     * @param target The target object to copy to.\n     * @param source The source object from which to copy properties.\n     */\n    assign<T, U>(target: T, source: U): T & U;\n\n    /**\n     * Copy the values of all of the enumerable own properties from one or more source objects to a\n     * target object. Returns the target object.\n     * @param target The target object to copy to.\n     * @param source1 The first source object from which to copy properties.\n     * @param source2 The second source object from which to copy properties.\n     */\n    assign<T, U, V>(target: T, source1: U, source2: V): T & U & V;\n\n    /**\n     * Copy the values of all of the enumerable own properties from one or more source objects to a\n     * target object. Returns the target object.\n     * @param target The target object to copy to.\n     * @param source1 The first source object from which to copy properties.\n     * @param source2 The second source object from which to copy properties.\n     * @param source3 The third source object from which to copy properties.\n     */\n    assign<T, U, V, W>(target: T, source1: U, source2: V, source3: W): T & U & V & W;\n\n    /**\n     * Copy the values of all of the enumerable own properties from one or more source objects to a\n     * target object. Returns the target object.\n     * @param target The target object to copy to.\n     * @param sources One or more source objects from which to copy properties\n     */\n    assign(target: object, ...sources: any[]): any;\n\n    /**\n     * Returns an array of all symbol properties found directly on object o.\n     * @param o Object to retrieve the symbols from.\n     */\n    getOwnPropertySymbols(o: any): symbol[];\n\n    /**\n     * Returns the names of the enumerable string properties and methods of an object.\n     * @param o Object that contains the properties and methods. This can be an object that you created or an existing Document Object Model (DOM) object.\n     */\n    keys(o: {}): string[];\n\n    /**\n     * Returns true if the values are the same value, false otherwise.\n     * @param value1 The first value.\n     * @param value2 The second value.\n     */\n    is(value1: any, value2: any): boolean;\n\n    /**\n     * Sets the prototype of a specified object o to object proto or null. Returns the object o.\n     * @param o The object to change its prototype.\n     * @param proto The value of the new prototype or null.\n     */\n    setPrototypeOf(o: any, proto: object | null): any;\n}\n\ninterface ReadonlyArray<T> {\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find<S extends T>(predicate: (this: void, value: T, index: number, obj: readonly T[]) => value is S, thisArg?: any): S | undefined;\n    find(predicate: (value: T, index: number, obj: readonly T[]) => unknown, thisArg?: any): T | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: T, index: number, obj: readonly T[]) => unknown, thisArg?: any): number;\n}\n\ninterface RegExp {\n    /**\n     * Returns a string indicating the flags of the regular expression in question. This field is read-only.\n     * The characters in this string are sequenced and concatenated in the following order:\n     *\n     *    - "g" for global\n     *    - "i" for ignoreCase\n     *    - "m" for multiline\n     *    - "u" for unicode\n     *    - "y" for sticky\n     *\n     * If no flags are set, the value is the empty string.\n     */\n    readonly flags: string;\n\n    /**\n     * Returns a Boolean value indicating the state of the sticky flag (y) used with a regular\n     * expression. Default is false. Read-only.\n     */\n    readonly sticky: boolean;\n\n    /**\n     * Returns a Boolean value indicating the state of the Unicode flag (u) used with a regular\n     * expression. Default is false. Read-only.\n     */\n    readonly unicode: boolean;\n}\n\ninterface RegExpConstructor {\n    new (pattern: RegExp | string, flags?: string): RegExp;\n    (pattern: RegExp | string, flags?: string): RegExp;\n}\n\ninterface String {\n    /**\n     * Returns a nonnegative integer Number less than 1114112 (0x110000) that is the code point\n     * value of the UTF-16 encoded code point starting at the string element at position pos in\n     * the String resulting from converting this object to a String.\n     * If there is no element at that position, the result is undefined.\n     * If a valid UTF-16 surrogate pair does not begin at pos, the result is the code unit at pos.\n     */\n    codePointAt(pos: number): number | undefined;\n\n    /**\n     * Returns true if searchString appears as a substring of the result of converting this\n     * object to a String, at one or more positions that are\n     * greater than or equal to position; otherwise, returns false.\n     * @param searchString search string\n     * @param position If position is undefined, 0 is assumed, so as to search all of the String.\n     */\n    includes(searchString: string, position?: number): boolean;\n\n    /**\n     * Returns true if the sequence of elements of searchString converted to a String is the\n     * same as the corresponding elements of this object (converted to a String) starting at\n     * endPosition – length(this). Otherwise returns false.\n     */\n    endsWith(searchString: string, endPosition?: number): boolean;\n\n    /**\n     * Returns the String value result of normalizing the string into the normalization form\n     * named by form as specified in Unicode Standard Annex #15, Unicode Normalization Forms.\n     * @param form Applicable values: "NFC", "NFD", "NFKC", or "NFKD", If not specified default\n     * is "NFC"\n     */\n    normalize(form: "NFC" | "NFD" | "NFKC" | "NFKD"): string;\n\n    /**\n     * Returns the String value result of normalizing the string into the normalization form\n     * named by form as specified in Unicode Standard Annex #15, Unicode Normalization Forms.\n     * @param form Applicable values: "NFC", "NFD", "NFKC", or "NFKD", If not specified default\n     * is "NFC"\n     */\n    normalize(form?: string): string;\n\n    /**\n     * Returns a String value that is made from count copies appended together. If count is 0,\n     * the empty string is returned.\n     * @param count number of copies to append\n     */\n    repeat(count: number): string;\n\n    /**\n     * Returns true if the sequence of elements of searchString converted to a String is the\n     * same as the corresponding elements of this object (converted to a String) starting at\n     * position. Otherwise returns false.\n     */\n    startsWith(searchString: string, position?: number): boolean;\n\n    /**\n     * Returns an `<a>` HTML anchor element and sets the name attribute to the text value\n     * @param name\n     */\n    anchor(name: string): string;\n\n    /** Returns a `<big>` HTML element */\n    big(): string;\n\n    /** Returns a `<blink>` HTML element */\n    blink(): string;\n\n    /** Returns a `<b>` HTML element */\n    bold(): string;\n\n    /** Returns a `<tt>` HTML element */\n    fixed(): string;\n\n    /** Returns a `<font>` HTML element and sets the color attribute value */\n    fontcolor(color: string): string;\n\n    /** Returns a `<font>` HTML element and sets the size attribute value */\n    fontsize(size: number): string;\n\n    /** Returns a `<font>` HTML element and sets the size attribute value */\n    fontsize(size: string): string;\n\n    /** Returns an `<i>` HTML element */\n    italics(): string;\n\n    /** Returns an `<a>` HTML element and sets the href attribute value */\n    link(url: string): string;\n\n    /** Returns a `<small>` HTML element */\n    small(): string;\n\n    /** Returns a `<strike>` HTML element */\n    strike(): string;\n\n    /** Returns a `<sub>` HTML element */\n    sub(): string;\n\n    /** Returns a `<sup>` HTML element */\n    sup(): string;\n}\n\ninterface StringConstructor {\n    /**\n     * Return the String value whose elements are, in order, the elements in the List elements.\n     * If length is 0, the empty string is returned.\n     */\n    fromCodePoint(...codePoints: number[]): string;\n\n    /**\n     * String.raw is usually used as a tag function of a Tagged Template String. When called as\n     * such, the first argument will be a well formed template call site object and the rest\n     * parameter will contain the substitution values. It can also be called directly, for example,\n     * to interleave strings and values from your own tag function, and in this case the only thing\n     * it needs from the first argument is the raw property.\n     * @param template A well-formed template string call site representation.\n     * @param substitutions A set of substitution values.\n     */\n    raw(template: { raw: readonly string[] | ArrayLike<string>}, ...substitutions: any[]): string;\n}\n',"lib.es2015.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es5" />\n/// <reference lib="es2015.core" />\n/// <reference lib="es2015.collection" />\n/// <reference lib="es2015.iterable" />\n/// <reference lib="es2015.generator" />\n/// <reference lib="es2015.promise" />\n/// <reference lib="es2015.proxy" />\n/// <reference lib="es2015.reflect" />\n/// <reference lib="es2015.symbol" />\n/// <reference lib="es2015.symbol.wellknown" />\n',"lib.es2015.generator.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2015.iterable" />\n\ninterface Generator<T = unknown, TReturn = any, TNext = unknown> extends Iterator<T, TReturn, TNext> {\n    // NOTE: \'next\' is defined using a tuple to ensure we report the correct assignability errors in all places.\n    next(...args: [] | [TNext]): IteratorResult<T, TReturn>;\n    return(value: TReturn): IteratorResult<T, TReturn>;\n    throw(e: any): IteratorResult<T, TReturn>;\n    [Symbol.iterator](): Generator<T, TReturn, TNext>;\n}\n\ninterface GeneratorFunction {\n    /**\n     * Creates a new Generator object.\n     * @param args A list of arguments the function accepts.\n     */\n    new (...args: any[]): Generator;\n    /**\n     * Creates a new Generator object.\n     * @param args A list of arguments the function accepts.\n     */\n    (...args: any[]): Generator;\n    /**\n     * The length of the arguments.\n     */\n    readonly length: number;\n    /**\n     * Returns the name of the function.\n     */\n    readonly name: string;\n    /**\n     * A reference to the prototype.\n     */\n    readonly prototype: Generator;\n}\n\ninterface GeneratorFunctionConstructor {\n    /**\n     * Creates a new Generator function.\n     * @param args A list of arguments the function accepts.\n     */\n    new (...args: string[]): GeneratorFunction;\n    /**\n     * Creates a new Generator function.\n     * @param args A list of arguments the function accepts.\n     */\n    (...args: string[]): GeneratorFunction;\n    /**\n     * The length of the arguments.\n     */\n    readonly length: number;\n    /**\n     * Returns the name of the function.\n     */\n    readonly name: string;\n    /**\n     * A reference to the prototype.\n     */\n    readonly prototype: GeneratorFunction;\n}\n',"lib.es2015.iterable.d.ts":"/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib=\"true\"/>\n\n\n/// <reference lib=\"es2015.symbol\" />\n\ninterface SymbolConstructor {\n    /**\n     * A method that returns the default iterator for an object. Called by the semantics of the\n     * for-of statement.\n     */\n    readonly iterator: unique symbol;\n}\n\ninterface IteratorYieldResult<TYield> {\n    done?: false;\n    value: TYield;\n}\n\ninterface IteratorReturnResult<TReturn> {\n    done: true;\n    value: TReturn;\n}\n\ntype IteratorResult<T, TReturn = any> = IteratorYieldResult<T> | IteratorReturnResult<TReturn>;\n\ninterface Iterator<T, TReturn = any, TNext = undefined> {\n    // NOTE: 'next' is defined using a tuple to ensure we report the correct assignability errors in all places.\n    next(...args: [] | [TNext]): IteratorResult<T, TReturn>;\n    return?(value?: TReturn): IteratorResult<T, TReturn>;\n    throw?(e?: any): IteratorResult<T, TReturn>;\n}\n\ninterface Iterable<T> {\n    [Symbol.iterator](): Iterator<T>;\n}\n\ninterface IterableIterator<T> extends Iterator<T> {\n    [Symbol.iterator](): IterableIterator<T>;\n}\n\ninterface Array<T> {\n    /** Iterator */\n    [Symbol.iterator](): IterableIterator<T>;\n\n    /**\n     * Returns an iterable of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, T]>;\n\n    /**\n     * Returns an iterable of keys in the array\n     */\n    keys(): IterableIterator<number>;\n\n    /**\n     * Returns an iterable of values in the array\n     */\n    values(): IterableIterator<T>;\n}\n\ninterface ArrayConstructor {\n    /**\n     * Creates an array from an iterable object.\n     * @param iterable An iterable object to convert to an array.\n     */\n    from<T>(iterable: Iterable<T> | ArrayLike<T>): T[];\n\n    /**\n     * Creates an array from an iterable object.\n     * @param iterable An iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T, U>(iterable: Iterable<T> | ArrayLike<T>, mapfn: (v: T, k: number) => U, thisArg?: any): U[];\n}\n\ninterface ReadonlyArray<T> {\n    /** Iterator of values in the array. */\n    [Symbol.iterator](): IterableIterator<T>;\n\n    /**\n     * Returns an iterable of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, T]>;\n\n    /**\n     * Returns an iterable of keys in the array\n     */\n    keys(): IterableIterator<number>;\n\n    /**\n     * Returns an iterable of values in the array\n     */\n    values(): IterableIterator<T>;\n}\n\ninterface IArguments {\n    /** Iterator */\n    [Symbol.iterator](): IterableIterator<any>;\n}\n\ninterface Map<K, V> {\n    /** Returns an iterable of entries in the map. */\n    [Symbol.iterator](): IterableIterator<[K, V]>;\n\n    /**\n     * Returns an iterable of key, value pairs for every entry in the map.\n     */\n    entries(): IterableIterator<[K, V]>;\n\n    /**\n     * Returns an iterable of keys in the map\n     */\n    keys(): IterableIterator<K>;\n\n    /**\n     * Returns an iterable of values in the map\n     */\n    values(): IterableIterator<V>;\n}\n\ninterface ReadonlyMap<K, V> {\n    /** Returns an iterable of entries in the map. */\n    [Symbol.iterator](): IterableIterator<[K, V]>;\n\n    /**\n     * Returns an iterable of key, value pairs for every entry in the map.\n     */\n    entries(): IterableIterator<[K, V]>;\n\n    /**\n     * Returns an iterable of keys in the map\n     */\n    keys(): IterableIterator<K>;\n\n    /**\n     * Returns an iterable of values in the map\n     */\n    values(): IterableIterator<V>;\n}\n\ninterface MapConstructor {\n    new <K, V>(iterable: Iterable<readonly [K, V]>): Map<K, V>;\n}\n\ninterface WeakMap<K extends object, V> { }\n\ninterface WeakMapConstructor {\n    new <K extends object, V>(iterable: Iterable<readonly [K, V]>): WeakMap<K, V>;\n}\n\ninterface Set<T> {\n    /** Iterates over values in the set. */\n    [Symbol.iterator](): IterableIterator<T>;\n    /**\n     * Returns an iterable of [v,v] pairs for every value `v` in the set.\n     */\n    entries(): IterableIterator<[T, T]>;\n    /**\n     * Despite its name, returns an iterable of the values in the set.\n     */\n    keys(): IterableIterator<T>;\n\n    /**\n     * Returns an iterable of values in the set.\n     */\n    values(): IterableIterator<T>;\n}\n\ninterface ReadonlySet<T> {\n    /** Iterates over values in the set. */\n    [Symbol.iterator](): IterableIterator<T>;\n\n    /**\n     * Returns an iterable of [v,v] pairs for every value `v` in the set.\n     */\n    entries(): IterableIterator<[T, T]>;\n\n    /**\n     * Despite its name, returns an iterable of the values in the set.\n     */\n    keys(): IterableIterator<T>;\n\n    /**\n     * Returns an iterable of values in the set.\n     */\n    values(): IterableIterator<T>;\n}\n\ninterface SetConstructor {\n    new <T>(iterable?: Iterable<T> | null): Set<T>;\n}\n\ninterface WeakSet<T extends object> { }\n\ninterface WeakSetConstructor {\n    new <T extends object = object>(iterable: Iterable<T>): WeakSet<T>;\n}\n\ninterface Promise<T> { }\n\ninterface PromiseConstructor {\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An iterable of Promises.\n     * @returns A new Promise.\n     */\n    all<T>(values: Iterable<T | PromiseLike<T>>): Promise<T[]>;\n\n    /**\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\n     * or rejected.\n     * @param values An iterable of Promises.\n     * @returns A new Promise.\n     */\n    race<T>(values: Iterable<T>): Promise<T extends PromiseLike<infer U> ? U : T>;\n\n    /**\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\n     * or rejected.\n     * @param values An iterable of Promises.\n     * @returns A new Promise.\n     */\n    race<T>(values: Iterable<T | PromiseLike<T>>): Promise<T>;\n}\n\ninterface String {\n    /** Iterator */\n    [Symbol.iterator](): IterableIterator<string>;\n}\n\ninterface Int8Array {\n    [Symbol.iterator](): IterableIterator<number>;\n    /**\n     * Returns an array of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, number]>;\n    /**\n     * Returns an list of keys in the array\n     */\n    keys(): IterableIterator<number>;\n    /**\n     * Returns an list of values in the array\n     */\n    values(): IterableIterator<number>;\n}\n\ninterface Int8ArrayConstructor {\n    new (elements: Iterable<number>): Int8Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int8Array;\n}\n\ninterface Uint8Array {\n    [Symbol.iterator](): IterableIterator<number>;\n    /**\n     * Returns an array of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, number]>;\n    /**\n     * Returns an list of keys in the array\n     */\n    keys(): IterableIterator<number>;\n    /**\n     * Returns an list of values in the array\n     */\n    values(): IterableIterator<number>;\n}\n\ninterface Uint8ArrayConstructor {\n    new (elements: Iterable<number>): Uint8Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint8Array;\n}\n\ninterface Uint8ClampedArray {\n    [Symbol.iterator](): IterableIterator<number>;\n    /**\n     * Returns an array of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, number]>;\n\n    /**\n     * Returns an list of keys in the array\n     */\n    keys(): IterableIterator<number>;\n\n    /**\n     * Returns an list of values in the array\n     */\n    values(): IterableIterator<number>;\n}\n\ninterface Uint8ClampedArrayConstructor {\n    new (elements: Iterable<number>): Uint8ClampedArray;\n\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint8ClampedArray;\n}\n\ninterface Int16Array {\n    [Symbol.iterator](): IterableIterator<number>;\n    /**\n     * Returns an array of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, number]>;\n\n    /**\n     * Returns an list of keys in the array\n     */\n    keys(): IterableIterator<number>;\n\n    /**\n     * Returns an list of values in the array\n     */\n    values(): IterableIterator<number>;\n}\n\ninterface Int16ArrayConstructor {\n    new (elements: Iterable<number>): Int16Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int16Array;\n}\n\ninterface Uint16Array {\n    [Symbol.iterator](): IterableIterator<number>;\n    /**\n     * Returns an array of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, number]>;\n    /**\n     * Returns an list of keys in the array\n     */\n    keys(): IterableIterator<number>;\n    /**\n     * Returns an list of values in the array\n     */\n    values(): IterableIterator<number>;\n}\n\ninterface Uint16ArrayConstructor {\n    new (elements: Iterable<number>): Uint16Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint16Array;\n}\n\ninterface Int32Array {\n    [Symbol.iterator](): IterableIterator<number>;\n    /**\n     * Returns an array of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, number]>;\n    /**\n     * Returns an list of keys in the array\n     */\n    keys(): IterableIterator<number>;\n    /**\n     * Returns an list of values in the array\n     */\n    values(): IterableIterator<number>;\n}\n\ninterface Int32ArrayConstructor {\n    new (elements: Iterable<number>): Int32Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Int32Array;\n}\n\ninterface Uint32Array {\n    [Symbol.iterator](): IterableIterator<number>;\n    /**\n     * Returns an array of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, number]>;\n    /**\n     * Returns an list of keys in the array\n     */\n    keys(): IterableIterator<number>;\n    /**\n     * Returns an list of values in the array\n     */\n    values(): IterableIterator<number>;\n}\n\ninterface Uint32ArrayConstructor {\n    new (elements: Iterable<number>): Uint32Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Uint32Array;\n}\n\ninterface Float32Array {\n    [Symbol.iterator](): IterableIterator<number>;\n    /**\n     * Returns an array of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, number]>;\n    /**\n     * Returns an list of keys in the array\n     */\n    keys(): IterableIterator<number>;\n    /**\n     * Returns an list of values in the array\n     */\n    values(): IterableIterator<number>;\n}\n\ninterface Float32ArrayConstructor {\n    new (elements: Iterable<number>): Float32Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Float32Array;\n}\n\ninterface Float64Array {\n    [Symbol.iterator](): IterableIterator<number>;\n    /**\n     * Returns an array of key, value pairs for every entry in the array\n     */\n    entries(): IterableIterator<[number, number]>;\n    /**\n     * Returns an list of keys in the array\n     */\n    keys(): IterableIterator<number>;\n    /**\n     * Returns an list of values in the array\n     */\n    values(): IterableIterator<number>;\n}\n\ninterface Float64ArrayConstructor {\n    new (elements: Iterable<number>): Float64Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from(arrayLike: Iterable<number>, mapfn?: (v: number, k: number) => number, thisArg?: any): Float64Array;\n}\n","lib.es2015.promise.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface PromiseConstructor {\n    /**\n     * A reference to the prototype.\n     */\n    readonly prototype: Promise<any>;\n\n    /**\n     * Creates a new Promise.\n     * @param executor A callback used to initialize the promise. This callback is passed two arguments:\n     * a resolve callback used to resolve the promise with a value or the result of another promise,\n     * and a reject callback used to reject the promise with a provided reason or error.\n     */\n    new <T>(executor: (resolve: (value: T | PromiseLike<T>) => void, reject: (reason?: any) => void) => void): Promise<T>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10>(values: readonly [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>, T10 | PromiseLike<T10>]): Promise<[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10]>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T1, T2, T3, T4, T5, T6, T7, T8, T9>(values: readonly [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>]): Promise<[T1, T2, T3, T4, T5, T6, T7, T8, T9]>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T1, T2, T3, T4, T5, T6, T7, T8>(values: readonly [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>]): Promise<[T1, T2, T3, T4, T5, T6, T7, T8]>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T1, T2, T3, T4, T5, T6, T7>(values: readonly [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>]): Promise<[T1, T2, T3, T4, T5, T6, T7]>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T1, T2, T3, T4, T5, T6>(values: readonly [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>]): Promise<[T1, T2, T3, T4, T5, T6]>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T1, T2, T3, T4, T5>(values: readonly [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>]): Promise<[T1, T2, T3, T4, T5]>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T1, T2, T3, T4>(values: readonly [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>]): Promise<[T1, T2, T3, T4]>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T1, T2, T3>(values: readonly [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>]): Promise<[T1, T2, T3]>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T1, T2>(values: readonly [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>]): Promise<[T1, T2]>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all of the provided Promises\n     * resolve, or rejected when any Promise is rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    all<T>(values: readonly (T | PromiseLike<T>)[]): Promise<T[]>;\n\n    // see: lib.es2015.iterable.d.ts\n    // all<T>(values: Iterable<T | PromiseLike<T>>): Promise<T[]>;\n\n    /**\n     * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved\n     * or rejected.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    race<T>(values: readonly T[]): Promise<T extends PromiseLike<infer U> ? U : T>;\n\n    // see: lib.es2015.iterable.d.ts\n    // race<T>(values: Iterable<T>): Promise<T extends PromiseLike<infer U> ? U : T>;\n\n    /**\n     * Creates a new rejected promise for the provided reason.\n     * @param reason The reason the promise was rejected.\n     * @returns A new rejected Promise.\n     */\n    reject<T = never>(reason?: any): Promise<T>;\n\n    /**\n     * Creates a new resolved promise.\n     * @returns A resolved promise.\n     */\n    resolve(): Promise<void>;\n\n    /**\n     * Creates a new resolved promise for the provided value.\n     * @param value A promise.\n     * @returns A promise whose internal state matches the provided promise.\n     */\n    resolve<T>(value: T | PromiseLike<T>): Promise<T>;\n}\n\ndeclare var Promise: PromiseConstructor;\n',"lib.es2015.proxy.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface ProxyHandler<T extends object> {\n    apply?(target: T, thisArg: any, argArray: any[]): any;\n    construct?(target: T, argArray: any[], newTarget: Function): object;\n    defineProperty?(target: T, p: string | symbol, attributes: PropertyDescriptor): boolean;\n    deleteProperty?(target: T, p: string | symbol): boolean;\n    get?(target: T, p: string | symbol, receiver: any): any;\n    getOwnPropertyDescriptor?(target: T, p: string | symbol): PropertyDescriptor | undefined;\n    getPrototypeOf?(target: T): object | null;\n    has?(target: T, p: string | symbol): boolean;\n    isExtensible?(target: T): boolean;\n    ownKeys?(target: T): ArrayLike<string | symbol>;\n    preventExtensions?(target: T): boolean;\n    set?(target: T, p: string | symbol, value: any, receiver: any): boolean;\n    setPrototypeOf?(target: T, v: object | null): boolean;\n}\n\ninterface ProxyConstructor {\n    revocable<T extends object>(target: T, handler: ProxyHandler<T>): { proxy: T; revoke: () => void; };\n    new <T extends object>(target: T, handler: ProxyHandler<T>): T;\n}\ndeclare var Proxy: ProxyConstructor;\n',"lib.es2015.reflect.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ndeclare namespace Reflect {\n    /**\n     * Calls the function with the specified object as the this value\n     * and the elements of specified array as the arguments.\n     * @param target The function to call.\n     * @param thisArgument The object to be used as the this object.\n     * @param argumentsList An array of argument values to be passed to the function.\n     */\n    function apply(target: Function, thisArgument: any, argumentsList: ArrayLike<any>): any;\n\n    /**\n     * Constructs the target with the elements of specified array as the arguments\n     * and the specified constructor as the `new.target` value.\n     * @param target The constructor to invoke.\n     * @param argumentsList An array of argument values to be passed to the constructor.\n     * @param newTarget The constructor to be used as the `new.target` object.\n     */\n    function construct(target: Function, argumentsList: ArrayLike<any>, newTarget?: Function): any;\n\n    /**\n     * Adds a property to an object, or modifies attributes of an existing property.\n     * @param target Object on which to add or modify the property. This can be a native JavaScript object\n     *        (that is, a user-defined object or a built in object) or a DOM object.\n     * @param propertyKey The property name.\n     * @param attributes Descriptor for the property. It can be for a data property or an accessor property.\n     */\n    function defineProperty(target: object, propertyKey: PropertyKey, attributes: PropertyDescriptor): boolean;\n\n    /**\n     * Removes a property from an object, equivalent to `delete target[propertyKey]`,\n     * except it won\'t throw if `target[propertyKey]` is non-configurable.\n     * @param target Object from which to remove the own property.\n     * @param propertyKey The property name.\n     */\n    function deleteProperty(target: object, propertyKey: PropertyKey): boolean;\n\n    /**\n     * Gets the property of target, equivalent to `target[propertyKey]` when `receiver === target`.\n     * @param target Object that contains the property on itself or in its prototype chain.\n     * @param propertyKey The property name.\n     * @param receiver The reference to use as the `this` value in the getter function,\n     *        if `target[propertyKey]` is an accessor property.\n     */\n    function get(target: object, propertyKey: PropertyKey, receiver?: any): any;\n\n    /**\n     * Gets the own property descriptor of the specified object.\n     * An own property descriptor is one that is defined directly on the object and is not inherited from the object\'s prototype.\n     * @param target Object that contains the property.\n     * @param propertyKey The property name.\n     */\n    function getOwnPropertyDescriptor(target: object, propertyKey: PropertyKey): PropertyDescriptor | undefined;\n\n    /**\n     * Returns the prototype of an object.\n     * @param target The object that references the prototype.\n     */\n    function getPrototypeOf(target: object): object | null;\n\n    /**\n     * Equivalent to `propertyKey in target`.\n     * @param target Object that contains the property on itself or in its prototype chain.\n     * @param propertyKey Name of the property.\n     */\n    function has(target: object, propertyKey: PropertyKey): boolean;\n\n    /**\n     * Returns a value that indicates whether new properties can be added to an object.\n     * @param target Object to test.\n     */\n    function isExtensible(target: object): boolean;\n\n    /**\n     * Returns the string and symbol keys of the own properties of an object. The own properties of an object\n     * are those that are defined directly on that object, and are not inherited from the object\'s prototype.\n     * @param target Object that contains the own properties.\n     */\n    function ownKeys(target: object): (string | symbol)[];\n\n    /**\n     * Prevents the addition of new properties to an object.\n     * @param target Object to make non-extensible.\n     * @return Whether the object has been made non-extensible.\n     */\n    function preventExtensions(target: object): boolean;\n\n    /**\n     * Sets the property of target, equivalent to `target[propertyKey] = value` when `receiver === target`.\n     * @param target Object that contains the property on itself or in its prototype chain.\n     * @param propertyKey Name of the property.\n     * @param receiver The reference to use as the `this` value in the setter function,\n     *        if `target[propertyKey]` is an accessor property.\n     */\n    function set(target: object, propertyKey: PropertyKey, value: any, receiver?: any): boolean;\n\n    /**\n     * Sets the prototype of a specified object o to object proto or null.\n     * @param target The object to change its prototype.\n     * @param proto The value of the new prototype or null.\n     * @return Whether setting the prototype was successful.\n     */\n    function setPrototypeOf(target: object, proto: object | null): boolean;\n}\n',"lib.es2015.symbol.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface SymbolConstructor {\n    /**\n     * A reference to the prototype.\n     */\n    readonly prototype: Symbol;\n\n    /**\n     * Returns a new unique Symbol value.\n     * @param  description Description of the new Symbol object.\n     */\n    (description?: string | number): symbol;\n\n    /**\n     * Returns a Symbol object from the global symbol registry matching the given key if found.\n     * Otherwise, returns a new symbol with this key.\n     * @param key key to search for.\n     */\n    for(key: string): symbol;\n\n    /**\n     * Returns a key from the global symbol registry matching the given Symbol if found.\n     * Otherwise, returns a undefined.\n     * @param sym Symbol to find the key for.\n     */\n    keyFor(sym: symbol): string | undefined;\n}\n\ndeclare var Symbol: SymbolConstructor;',"lib.es2015.symbol.wellknown.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2015.symbol" />\n\ninterface SymbolConstructor {\n    /**\n     * A method that determines if a constructor object recognizes an object as one of the\n     * constructor’s instances. Called by the semantics of the instanceof operator.\n     */\n    readonly hasInstance: unique symbol;\n\n    /**\n     * A Boolean value that if true indicates that an object should flatten to its array elements\n     * by Array.prototype.concat.\n     */\n    readonly isConcatSpreadable: unique symbol;\n\n    /**\n     * A regular expression method that matches the regular expression against a string. Called\n     * by the String.prototype.match method.\n     */\n    readonly match: unique symbol;\n\n    /**\n     * A regular expression method that replaces matched substrings of a string. Called by the\n     * String.prototype.replace method.\n     */\n    readonly replace: unique symbol;\n\n    /**\n     * A regular expression method that returns the index within a string that matches the\n     * regular expression. Called by the String.prototype.search method.\n     */\n    readonly search: unique symbol;\n\n    /**\n     * A function valued property that is the constructor function that is used to create\n     * derived objects.\n     */\n    readonly species: unique symbol;\n\n    /**\n     * A regular expression method that splits a string at the indices that match the regular\n     * expression. Called by the String.prototype.split method.\n     */\n    readonly split: unique symbol;\n\n    /**\n     * A method that converts an object to a corresponding primitive value.\n     * Called by the ToPrimitive abstract operation.\n     */\n    readonly toPrimitive: unique symbol;\n\n    /**\n     * A String value that is used in the creation of the default string description of an object.\n     * Called by the built-in method Object.prototype.toString.\n     */\n    readonly toStringTag: unique symbol;\n\n    /**\n     * An Object whose own property names are property names that are excluded from the \'with\'\n     * environment bindings of the associated objects.\n     */\n    readonly unscopables: unique symbol;\n}\n\ninterface Symbol {\n    /**\n     * Converts a Symbol object to a symbol.\n     */\n    [Symbol.toPrimitive](hint: string): symbol;\n\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface Array<T> {\n    /**\n     * Returns an object whose properties have the value \'true\'\n     * when they will be absent when used in a \'with\' statement.\n     */\n    [Symbol.unscopables](): {\n        copyWithin: boolean;\n        entries: boolean;\n        fill: boolean;\n        find: boolean;\n        findIndex: boolean;\n        keys: boolean;\n        values: boolean;\n    };\n}\n\ninterface Date {\n    /**\n     * Converts a Date object to a string.\n     */\n    [Symbol.toPrimitive](hint: "default"): string;\n    /**\n     * Converts a Date object to a string.\n     */\n    [Symbol.toPrimitive](hint: "string"): string;\n    /**\n     * Converts a Date object to a number.\n     */\n    [Symbol.toPrimitive](hint: "number"): number;\n    /**\n     * Converts a Date object to a string or number.\n     *\n     * @param hint The strings "number", "string", or "default" to specify what primitive to return.\n     *\n     * @throws {TypeError} If \'hint\' was given something other than "number", "string", or "default".\n     * @returns A number if \'hint\' was "number", a string if \'hint\' was "string" or "default".\n     */\n    [Symbol.toPrimitive](hint: string): string | number;\n}\n\ninterface Map<K, V> {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface WeakMap<K extends object, V> {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface Set<T> {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface WeakSet<T extends object> {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface JSON {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface Function {\n    /**\n     * Determines whether the given value inherits from this function if this function was used\n     * as a constructor function.\n     *\n     * A constructor function can control which objects are recognized as its instances by\n     * \'instanceof\' by overriding this method.\n     */\n    [Symbol.hasInstance](value: any): boolean;\n}\n\ninterface GeneratorFunction {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface Math {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface Promise<T> {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface PromiseConstructor {\n    readonly [Symbol.species]: PromiseConstructor;\n}\n\ninterface RegExp {\n    /**\n     * Matches a string with this regular expression, and returns an array containing the results of\n     * that search.\n     * @param string A string to search within.\n     */\n    [Symbol.match](string: string): RegExpMatchArray | null;\n\n    /**\n     * Replaces text in a string, using this regular expression.\n     * @param string A String object or string literal whose contents matching against\n     *               this regular expression will be replaced\n     * @param replaceValue A String object or string literal containing the text to replace for every\n     *                     successful match of this regular expression.\n     */\n    [Symbol.replace](string: string, replaceValue: string): string;\n\n    /**\n     * Replaces text in a string, using this regular expression.\n     * @param string A String object or string literal whose contents matching against\n     *               this regular expression will be replaced\n     * @param replacer A function that returns the replacement text.\n     */\n    [Symbol.replace](string: string, replacer: (substring: string, ...args: any[]) => string): string;\n\n    /**\n     * Finds the position beginning first substring match in a regular expression search\n     * using this regular expression.\n     *\n     * @param string The string to search within.\n     */\n    [Symbol.search](string: string): number;\n\n    /**\n     * Returns an array of substrings that were delimited by strings in the original input that\n     * match against this regular expression.\n     *\n     * If the regular expression contains capturing parentheses, then each time this\n     * regular expression matches, the results (including any undefined results) of the\n     * capturing parentheses are spliced.\n     *\n     * @param string string value to split\n     * @param limit if not undefined, the output array is truncated so that it contains no more\n     * than \'limit\' elements.\n     */\n    [Symbol.split](string: string, limit?: number): string[];\n}\n\ninterface RegExpConstructor {\n    readonly [Symbol.species]: RegExpConstructor;\n}\n\ninterface String {\n    /**\n     * Matches a string or an object that supports being matched against, and returns an array\n     * containing the results of that search, or null if no matches are found.\n     * @param matcher An object that supports being matched against.\n     */\n    match(matcher: { [Symbol.match](string: string): RegExpMatchArray | null; }): RegExpMatchArray | null;\n\n    /**\n     * Replaces first match with string or all matches with RegExp.\n     * @param searchValue A string or RegExp search value.\n     * @param replaceValue A string containing the text to replace for match.\n     */\n    replace(searchValue: { [Symbol.replace](string: string, replaceValue: string): string; }, replaceValue: string): string;\n\n    /**\n     * Replaces text in a string, using an object that supports replacement within a string.\n     * @param searchValue A object can search for and replace matches within a string.\n     * @param replacer A function that returns the replacement text.\n     */\n    replace(searchValue: { [Symbol.replace](string: string, replacer: (substring: string, ...args: any[]) => string): string; }, replacer: (substring: string, ...args: any[]) => string): string;\n\n    /**\n     * Finds the first substring match in a regular expression search.\n     * @param searcher An object which supports searching within a string.\n     */\n    search(searcher: { [Symbol.search](string: string): number; }): number;\n\n    /**\n     * Split a string into substrings using the specified separator and return them as an array.\n     * @param splitter An object that can split a string.\n     * @param limit A value used to limit the number of elements returned in the array.\n     */\n    split(splitter: { [Symbol.split](string: string, limit?: number): string[]; }, limit?: number): string[];\n}\n\ninterface ArrayBuffer {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface DataView {\n    readonly [Symbol.toStringTag]: string;\n}\n\ninterface Int8Array {\n    readonly [Symbol.toStringTag]: "Int8Array";\n}\n\ninterface Uint8Array {\n    readonly [Symbol.toStringTag]: "Uint8Array";\n}\n\ninterface Uint8ClampedArray {\n    readonly [Symbol.toStringTag]: "Uint8ClampedArray";\n}\n\ninterface Int16Array {\n    readonly [Symbol.toStringTag]: "Int16Array";\n}\n\ninterface Uint16Array {\n    readonly [Symbol.toStringTag]: "Uint16Array";\n}\n\ninterface Int32Array {\n    readonly [Symbol.toStringTag]: "Int32Array";\n}\n\ninterface Uint32Array {\n    readonly [Symbol.toStringTag]: "Uint32Array";\n}\n\ninterface Float32Array {\n    readonly [Symbol.toStringTag]: "Float32Array";\n}\n\ninterface Float64Array {\n    readonly [Symbol.toStringTag]: "Float64Array";\n}\n\ninterface ArrayConstructor {\n    readonly [Symbol.species]: ArrayConstructor;\n}\ninterface MapConstructor {\n    readonly [Symbol.species]: MapConstructor;\n}\ninterface SetConstructor {\n    readonly [Symbol.species]: SetConstructor;\n}\ninterface ArrayBufferConstructor {\n    readonly [Symbol.species]: ArrayBufferConstructor;\n}\n',"lib.es2016.array.include.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface Array<T> {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: T, fromIndex?: number): boolean;\n}\n\ninterface ReadonlyArray<T> {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: T, fromIndex?: number): boolean;\n}\n\ninterface Int8Array {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: number, fromIndex?: number): boolean;\n}\n\ninterface Uint8Array {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: number, fromIndex?: number): boolean;\n}\n\ninterface Uint8ClampedArray {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: number, fromIndex?: number): boolean;\n}\n\ninterface Int16Array {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: number, fromIndex?: number): boolean;\n}\n\ninterface Uint16Array {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: number, fromIndex?: number): boolean;\n}\n\ninterface Int32Array {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: number, fromIndex?: number): boolean;\n}\n\ninterface Uint32Array {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: number, fromIndex?: number): boolean;\n}\n\ninterface Float32Array {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: number, fromIndex?: number): boolean;\n}\n\ninterface Float64Array {\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: number, fromIndex?: number): boolean;\n}',"lib.es2016.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2015" />\n/// <reference lib="es2016.array.include" />',"lib.es2016.full.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2016" />\n/// <reference lib="dom" />\n/// <reference lib="webworker.importscripts" />\n/// <reference lib="scripthost" />\n/// <reference lib="dom.iterable" />',"lib.es2017.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2016" />\n/// <reference lib="es2017.object" />\n/// <reference lib="es2017.sharedmemory" />\n/// <reference lib="es2017.string" />\n/// <reference lib="es2017.intl" />\n/// <reference lib="es2017.typedarrays" />\n',"lib.es2017.full.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2017" />\n/// <reference lib="dom" />\n/// <reference lib="webworker.importscripts" />\n/// <reference lib="scripthost" />\n/// <reference lib="dom.iterable" />',"lib.es2017.intl.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ndeclare namespace Intl {\n    type DateTimeFormatPartTypes = "day" | "dayPeriod" | "era" | "hour" | "literal" | "minute" | "month" | "second" | "timeZoneName" | "weekday" | "year";\n\n    interface DateTimeFormatPart {\n        type: DateTimeFormatPartTypes;\n        value: string;\n    }\n\n    interface DateTimeFormat {\n        formatToParts(date?: Date | number): DateTimeFormatPart[];\n    }\n}\n',"lib.es2017.object.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface ObjectConstructor {\n    /**\n     * Returns an array of values of the enumerable properties of an object\n     * @param o Object that contains the properties and methods. This can be an object that you created or an existing Document Object Model (DOM) object.\n     */\n    values<T>(o: { [s: string]: T } | ArrayLike<T>): T[];\n\n    /**\n     * Returns an array of values of the enumerable properties of an object\n     * @param o Object that contains the properties and methods. This can be an object that you created or an existing Document Object Model (DOM) object.\n     */\n    values(o: {}): any[];\n\n    /**\n     * Returns an array of key/values of the enumerable properties of an object\n     * @param o Object that contains the properties and methods. This can be an object that you created or an existing Document Object Model (DOM) object.\n     */\n    entries<T>(o: { [s: string]: T } | ArrayLike<T>): [string, T][];\n\n    /**\n     * Returns an array of key/values of the enumerable properties of an object\n     * @param o Object that contains the properties and methods. This can be an object that you created or an existing Document Object Model (DOM) object.\n     */\n    entries(o: {}): [string, any][];\n\n    /**\n     * Returns an object containing all own property descriptors of an object\n     * @param o Object that contains the properties and methods. This can be an object that you created or an existing Document Object Model (DOM) object.\n     */\n    getOwnPropertyDescriptors<T>(o: T): {[P in keyof T]: TypedPropertyDescriptor<T[P]>} & { [x: string]: PropertyDescriptor };\n}\n',"lib.es2017.sharedmemory.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2015.symbol" />\n/// <reference lib="es2015.symbol.wellknown" />\n\ninterface SharedArrayBuffer {\n    /**\n     * Read-only. The length of the ArrayBuffer (in bytes).\n     */\n    readonly byteLength: number;\n\n    /**\n     * Returns a section of an SharedArrayBuffer.\n     */\n    slice(begin: number, end?: number): SharedArrayBuffer;\n    readonly [Symbol.species]: SharedArrayBuffer;\n    readonly [Symbol.toStringTag]: "SharedArrayBuffer";\n}\n\ninterface SharedArrayBufferConstructor {\n    readonly prototype: SharedArrayBuffer;\n    new (byteLength: number): SharedArrayBuffer;\n}\ndeclare var SharedArrayBuffer: SharedArrayBufferConstructor;\n\ninterface ArrayBufferTypes {\n    SharedArrayBuffer: SharedArrayBuffer;\n}\n\ninterface Atomics {\n    /**\n     * Adds a value to the value at the given position in the array, returning the original value.\n     * Until this atomic operation completes, any other read or write operation against the array\n     * will block.\n     */\n    add(typedArray: Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array, index: number, value: number): number;\n\n    /**\n     * Stores the bitwise AND of a value with the value at the given position in the array,\n     * returning the original value. Until this atomic operation completes, any other read or\n     * write operation against the array will block.\n     */\n    and(typedArray: Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array, index: number, value: number): number;\n\n    /**\n     * Replaces the value at the given position in the array if the original value equals the given\n     * expected value, returning the original value. Until this atomic operation completes, any\n     * other read or write operation against the array will block.\n     */\n    compareExchange(typedArray: Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array, index: number, expectedValue: number, replacementValue: number): number;\n\n    /**\n     * Replaces the value at the given position in the array, returning the original value. Until\n     * this atomic operation completes, any other read or write operation against the array will\n     * block.\n     */\n    exchange(typedArray: Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array, index: number, value: number): number;\n\n    /**\n     * Returns a value indicating whether high-performance algorithms can use atomic operations\n     * (`true`) or must use locks (`false`) for the given number of bytes-per-element of a typed\n     * array.\n     */\n    isLockFree(size: number): boolean;\n\n    /**\n     * Returns the value at the given position in the array. Until this atomic operation completes,\n     * any other read or write operation against the array will block.\n     */\n    load(typedArray: Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array, index: number): number;\n\n    /**\n     * Stores the bitwise OR of a value with the value at the given position in the array,\n     * returning the original value. Until this atomic operation completes, any other read or write\n     * operation against the array will block.\n     */\n    or(typedArray: Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array, index: number, value: number): number;\n\n    /**\n     * Stores a value at the given position in the array, returning the new value. Until this\n     * atomic operation completes, any other read or write operation against the array will block.\n     */\n    store(typedArray: Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array, index: number, value: number): number;\n\n    /**\n     * Subtracts a value from the value at the given position in the array, returning the original\n     * value. Until this atomic operation completes, any other read or write operation against the\n     * array will block.\n     */\n    sub(typedArray: Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array, index: number, value: number): number;\n\n    /**\n     * If the value at the given position in the array is equal to the provided value, the current\n     * agent is put to sleep causing execution to suspend until the timeout expires (returning\n     * `"timed-out"`) or until the agent is awoken (returning `"ok"`); otherwise, returns\n     * `"not-equal"`.\n     */\n    wait(typedArray: Int32Array, index: number, value: number, timeout?: number): "ok" | "not-equal" | "timed-out";\n\n    /**\n     * Wakes up sleeping agents that are waiting on the given index of the array, returning the\n     * number of agents that were awoken.\n     * @param typedArray A shared Int32Array.\n     * @param index The position in the typedArray to wake up on.\n     * @param count The number of sleeping agents to notify. Defaults to +Infinity.\n     */\n    notify(typedArray: Int32Array, index: number, count?: number): number;\n\n    /**\n     * Stores the bitwise XOR of a value with the value at the given position in the array,\n     * returning the original value. Until this atomic operation completes, any other read or write\n     * operation against the array will block.\n     */\n    xor(typedArray: Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array, index: number, value: number): number;\n\n    readonly [Symbol.toStringTag]: "Atomics";\n}\n\ndeclare var Atomics: Atomics;\n',"lib.es2017.string.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface String {\n    /**\n     * Pads the current string with a given string (possibly repeated) so that the resulting string reaches a given length.\n     * The padding is applied from the start (left) of the current string.\n     *\n     * @param maxLength The length of the resulting string once the current string has been padded.\n     *        If this parameter is smaller than the current string\'s length, the current string will be returned as it is.\n     *\n     * @param fillString The string to pad the current string with.\n     *        If this string is too long, it will be truncated and the left-most part will be applied.\n     *        The default value for this parameter is " " (U+0020).\n     */\n    padStart(maxLength: number, fillString?: string): string;\n\n    /**\n     * Pads the current string with a given string (possibly repeated) so that the resulting string reaches a given length.\n     * The padding is applied from the end (right) of the current string.\n     *\n     * @param maxLength The length of the resulting string once the current string has been padded.\n     *        If this parameter is smaller than the current string\'s length, the current string will be returned as it is.\n     *\n     * @param fillString The string to pad the current string with.\n     *        If this string is too long, it will be truncated and the left-most part will be applied.\n     *        The default value for this parameter is " " (U+0020).\n     */\n    padEnd(maxLength: number, fillString?: string): string;\n}\n',"lib.es2017.typedarrays.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface Int8ArrayConstructor {\n    new (): Int8Array;\n}\n\ninterface Uint8ArrayConstructor {\n    new (): Uint8Array;\n}\n\ninterface Uint8ClampedArrayConstructor {\n    new (): Uint8ClampedArray;\n}\n\ninterface Int16ArrayConstructor {\n    new (): Int16Array;\n}\n\ninterface Uint16ArrayConstructor {\n    new (): Uint16Array;\n}\n\ninterface Int32ArrayConstructor {\n    new (): Int32Array;\n}\n\ninterface Uint32ArrayConstructor {\n    new (): Uint32Array;\n}\n\ninterface Float32ArrayConstructor {\n    new (): Float32Array;\n}\n\ninterface Float64ArrayConstructor {\n    new (): Float64Array;\n}\n',"lib.es2018.asyncgenerator.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2018.asynciterable" />\n\ninterface AsyncGenerator<T = unknown, TReturn = any, TNext = unknown> extends AsyncIterator<T, TReturn, TNext> {\n    // NOTE: \'next\' is defined using a tuple to ensure we report the correct assignability errors in all places.\n    next(...args: [] | [TNext]): Promise<IteratorResult<T, TReturn>>;\n    return(value: TReturn | PromiseLike<TReturn>): Promise<IteratorResult<T, TReturn>>;\n    throw(e: any): Promise<IteratorResult<T, TReturn>>;\n    [Symbol.asyncIterator](): AsyncGenerator<T, TReturn, TNext>;\n}\n\ninterface AsyncGeneratorFunction {\n    /**\n     * Creates a new AsyncGenerator object.\n     * @param args A list of arguments the function accepts.\n     */\n    new (...args: any[]): AsyncGenerator;\n    /**\n     * Creates a new AsyncGenerator object.\n     * @param args A list of arguments the function accepts.\n     */\n    (...args: any[]): AsyncGenerator;\n    /**\n     * The length of the arguments.\n     */\n    readonly length: number;\n    /**\n     * Returns the name of the function.\n     */\n    readonly name: string;\n    /**\n     * A reference to the prototype.\n     */\n    readonly prototype: AsyncGenerator;\n}\n\ninterface AsyncGeneratorFunctionConstructor {\n    /**\n     * Creates a new AsyncGenerator function.\n     * @param args A list of arguments the function accepts.\n     */\n    new (...args: string[]): AsyncGeneratorFunction;\n    /**\n     * Creates a new AsyncGenerator function.\n     * @param args A list of arguments the function accepts.\n     */\n    (...args: string[]): AsyncGeneratorFunction;\n    /**\n     * The length of the arguments.\n     */\n    readonly length: number;\n    /**\n     * Returns the name of the function.\n     */\n    readonly name: string;\n    /**\n     * A reference to the prototype.\n     */\n    readonly prototype: AsyncGeneratorFunction;\n}\n',"lib.es2018.asynciterable.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2015.symbol" />\n/// <reference lib="es2015.iterable" />\n\ninterface SymbolConstructor {\n    /**\n     * A method that returns the default async iterator for an object. Called by the semantics of\n     * the for-await-of statement.\n     */\n    readonly asyncIterator: unique symbol;\n}\n\ninterface AsyncIterator<T, TReturn = any, TNext = undefined> {\n    // NOTE: \'next\' is defined using a tuple to ensure we report the correct assignability errors in all places.\n    next(...args: [] | [TNext]): Promise<IteratorResult<T, TReturn>>;\n    return?(value?: TReturn | PromiseLike<TReturn>): Promise<IteratorResult<T, TReturn>>;\n    throw?(e?: any): Promise<IteratorResult<T, TReturn>>;\n}\n\ninterface AsyncIterable<T> {\n    [Symbol.asyncIterator](): AsyncIterator<T>;\n}\n\ninterface AsyncIterableIterator<T> extends AsyncIterator<T> {\n    [Symbol.asyncIterator](): AsyncIterableIterator<T>;\n}',"lib.es2018.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2017" />\n/// <reference lib="es2018.asynciterable" />\n/// <reference lib="es2018.asyncgenerator" />\n/// <reference lib="es2018.promise" />\n/// <reference lib="es2018.regexp" />\n/// <reference lib="es2018.intl" />\n',"lib.es2018.full.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2018" />\n/// <reference lib="dom" />\n/// <reference lib="webworker.importscripts" />\n/// <reference lib="scripthost" />\n/// <reference lib="dom.iterable" />',"lib.es2018.intl.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ndeclare namespace Intl {\n\n    // http://cldr.unicode.org/index/cldr-spec/plural-rules#TOC-Determining-Plural-Categories\n    type LDMLPluralRule = "zero" | "one" | "two" | "few" | "many" | "other";\n    type PluralRuleType = "cardinal" | "ordinal";\n\n    interface PluralRulesOptions {\n        localeMatcher?: "lookup" | "best fit";\n        type?: PluralRuleType;\n        minimumIntegerDigits?: number;\n        minimumFractionDigits?: number;\n        maximumFractionDigits?: number;\n        minimumSignificantDigits?: number;\n        maximumSignificantDigits?: number;\n    }\n\n    interface ResolvedPluralRulesOptions {\n        locale: string;\n        pluralCategories: LDMLPluralRule[];\n        type: PluralRuleType;\n        minimumIntegerDigits: number;\n        minimumFractionDigits: number;\n        maximumFractionDigits: number;\n        minimumSignificantDigits?: number;\n        maximumSignificantDigits?: number;\n    }\n\n    interface PluralRules {\n        resolvedOptions(): ResolvedPluralRulesOptions;\n        select(n: number): LDMLPluralRule;\n    }\n\n    const PluralRules: {\n        new (locales?: string | string[], options?: PluralRulesOptions): PluralRules;\n        (locales?: string | string[], options?: PluralRulesOptions): PluralRules;\n        supportedLocalesOf(\n            locales: string | string[],\n            options?: PluralRulesOptions,\n        ): string[];\n    };\n}\n',"lib.es2018.promise.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/**\n * Represents the completion of an asynchronous operation\n */\ninterface Promise<T> {\n    /**\n     * Attaches a callback that is invoked when the Promise is settled (fulfilled or rejected). The\n     * resolved value cannot be modified from the callback.\n     * @param onfinally The callback to execute when the Promise is settled (fulfilled or rejected).\n     * @returns A Promise for the completion of the callback.\n     */\n    finally(onfinally?: (() => void) | undefined | null): Promise<T>\n}\n',"lib.es2018.regexp.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface RegExpMatchArray {\n    groups?: {\n        [key: string]: string\n    }\n}\n\ninterface RegExpExecArray {\n    groups?: {\n        [key: string]: string\n    }\n}\n\ninterface RegExp {\n    /**\n     * Returns a Boolean value indicating the state of the dotAll flag (s) used with a regular expression.\n     * Default is false. Read-only.\n     */\n    readonly dotAll: boolean;\n}',"lib.es2019.array.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ntype FlatArray<Arr, Depth extends number> = {\n    "done": Arr,\n    "recur": Arr extends ReadonlyArray<infer InnerArr>\n        ? FlatArray<InnerArr, [-1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20][Depth]>\n        : Arr\n}[Depth extends -1 ? "done" : "recur"];\n\ninterface ReadonlyArray<T> {\n\n    /**\n     * Calls a defined callback function on each element of an array. Then, flattens the result into\n     * a new array.\n     * This is identical to a map followed by flat with depth 1.\n     *\n     * @param callback A function that accepts up to three arguments. The flatMap method calls the\n     * callback function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callback function. If\n     * thisArg is omitted, undefined is used as the this value.\n     */\n    flatMap<U, This = undefined> (\n        callback: (this: This, value: T, index: number, array: T[]) => U | ReadonlyArray<U>,\n        thisArg?: This\n    ): U[]\n\n\n    /**\n     * Returns a new array with all sub-array elements concatenated into it recursively up to the\n     * specified depth.\n     *\n     * @param depth The maximum recursion depth\n     */\n    flat<A, D extends number = 1>(\n        this: A,\n        depth?: D\n    ): FlatArray<A, D>[]\n  }\n\ninterface Array<T> {\n\n    /**\n     * Calls a defined callback function on each element of an array. Then, flattens the result into\n     * a new array.\n     * This is identical to a map followed by flat with depth 1.\n     *\n     * @param callback A function that accepts up to three arguments. The flatMap method calls the\n     * callback function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callback function. If\n     * thisArg is omitted, undefined is used as the this value.\n     */\n    flatMap<U, This = undefined> (\n        callback: (this: This, value: T, index: number, array: T[]) => U | ReadonlyArray<U>,\n        thisArg?: This\n    ): U[]\n\n    /**\n     * Returns a new array with all sub-array elements concatenated into it recursively up to the\n     * specified depth.\n     *\n     * @param depth The maximum recursion depth\n     */\n    flat<A, D extends number = 1>(\n        this: A,\n        depth?: D\n    ): FlatArray<A, D>[]\n}\n',"lib.es2019.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2018" />\n/// <reference lib="es2019.array" />\n/// <reference lib="es2019.object" />\n/// <reference lib="es2019.string" />\n/// <reference lib="es2019.symbol" />\n',"lib.es2019.full.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2019" />\n/// <reference lib="dom" />\n/// <reference lib="webworker.importscripts" />\n/// <reference lib="scripthost" />\n/// <reference lib="dom.iterable" />\n',"lib.es2019.object.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2015.iterable" />\n\ninterface ObjectConstructor {\n    /**\n     * Returns an object created by key-value entries for properties and methods\n     * @param entries An iterable object that contains key-value entries for properties and methods.\n     */\n    fromEntries<T = any>(entries: Iterable<readonly [PropertyKey, T]>): { [k: string]: T };\n\n    /**\n     * Returns an object created by key-value entries for properties and methods\n     * @param entries An iterable object that contains key-value entries for properties and methods.\n     */\n    fromEntries(entries: Iterable<readonly any[]>): any;\n}\n',"lib.es2019.string.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface String {\n    /** Removes the trailing white space and line terminator characters from a string. */\n    trimEnd(): string;\n\n    /** Removes the leading white space and line terminator characters from a string. */\n    trimStart(): string;\n\n    /** Removes the leading white space and line terminator characters from a string. */\n    trimLeft(): string;\n\n    /** Removes the trailing white space and line terminator characters from a string. */\n    trimRight(): string;\n}\n',"lib.es2019.symbol.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface Symbol {\n    /**\n     * Expose the [[Description]] internal slot of a symbol directly.\n     */\n    readonly description: string | undefined;\n}\n',"lib.es2020.bigint.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface BigIntToLocaleStringOptions {\n    /**\n     * The locale matching algorithm to use.The default is "best fit". For information about this option, see the {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl#Locale_negotiation Intl page}.\n     */\n    localeMatcher?: string;\n    /**\n     * The formatting style to use , the default is "decimal".\n     */\n    style?: string;\n\n    numberingSystem?: string;\n    /**\n     * The unit to use in unit formatting, Possible values are core unit identifiers, defined in UTS #35, Part 2, Section 6. A subset of units from the full list was selected for use in ECMAScript. Pairs of simple units can be concatenated with "-per-" to make a compound unit. There is no default value; if the style is "unit", the unit property must be provided.\n     */\n    unit?: string;\n\n    /**\n     * The unit formatting style to use in unit formatting, the defaults is "short".\n     */\n    unitDisplay?: string;\n\n    /**\n     * The currency to use in currency formatting. Possible values are the ISO 4217 currency codes, such as "USD" for the US dollar, "EUR" for the euro, or "CNY" for the Chinese RMB — see the Current currency & funds code list. There is no default value; if the style is "currency", the currency property must be provided. It is only used when [[Style]] has the value "currency".\n     */\n    currency?: string;\n\n    /**\n     * How to display the currency in currency formatting. It is only used when [[Style]] has the value "currency". The default is "symbol".\n     *\n     * "symbol" to use a localized currency symbol such as €,\n     *\n     * "code" to use the ISO currency code,\n     *\n     * "name" to use a localized currency name such as "dollar"\n     */\n    currencyDisplay?: string;\n\n    /**\n     * Whether to use grouping separators, such as thousands separators or thousand/lakh/crore separators. The default is true.\n     */\n    useGrouping?: boolean;\n\n    /**\n     * The minimum number of integer digits to use. Possible values are from 1 to 21; the default is 1.\n     */\n    minimumIntegerDigits?: 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 | 21;\n\n    /**\n     * The minimum number of fraction digits to use. Possible values are from 0 to 20; the default for plain number and percent formatting is 0; the default for currency formatting is the number of minor unit digits provided by the {@link http://www.currency-iso.org/en/home/tables/table-a1.html ISO 4217 currency codes list} (2 if the list doesn\'t provide that information).\n     */\n    minimumFractionDigits?: 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20;\n\n    /**\n     * The maximum number of fraction digits to use. Possible values are from 0 to 20; the default for plain number formatting is the larger of minimumFractionDigits and 3; the default for currency formatting is the larger of minimumFractionDigits and the number of minor unit digits provided by the {@link http://www.currency-iso.org/en/home/tables/table-a1.html ISO 4217 currency codes list} (2 if the list doesn\'t provide that information); the default for percent formatting is the larger of minimumFractionDigits and 0.\n     */\n    maximumFractionDigits?: 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20;\n\n    /**\n     * The minimum number of significant digits to use. Possible values are from 1 to 21; the default is 1.\n     */\n    minimumSignificantDigits?: 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 | 21;\n\n    /**\n     * The maximum number of significant digits to use. Possible values are from 1 to 21; the default is 21.\n     */\n    maximumSignificantDigits?: 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 | 21;\n\n    /**\n     * The formatting that should be displayed for the number, the defaults is "standard"\n     *\n     *     "standard" plain number formatting\n     *\n     *     "scientific" return the order-of-magnitude for formatted number.\n     *\n     *     "engineering" return the exponent of ten when divisible by three\n     *\n     *     "compact" string representing exponent, defaults is using the "short" form\n     */\n    notation?: string;\n\n    /**\n     * used only when notation is "compact"\n     */\n    compactDisplay?: string;\n}\n\ninterface BigInt {\n    /**\n     * Returns a string representation of an object.\n     * @param radix Specifies a radix for converting numeric values to strings.\n     */\n    toString(radix?: number): string;\n\n    /** Returns a string representation appropriate to the host environment\'s current locale. */\n    toLocaleString(locales?: string, options?: BigIntToLocaleStringOptions): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): bigint;\n\n    readonly [Symbol.toStringTag]: "BigInt";\n}\n\ninterface BigIntConstructor {\n    (value: bigint | boolean | number | string): bigint;\n    readonly prototype: BigInt;\n\n    /**\n     * Interprets the low bits of a BigInt as a 2\'s-complement signed integer.\n     * All higher bits are discarded.\n     * @param bits The number of low bits to use\n     * @param int The BigInt whose bits to extract\n     */\n    asIntN(bits: number, int: bigint): bigint;\n    /**\n     * Interprets the low bits of a BigInt as an unsigned integer.\n     * All higher bits are discarded.\n     * @param bits The number of low bits to use\n     * @param int The BigInt whose bits to extract\n     */\n    asUintN(bits: number, int: bigint): bigint;\n}\n\ndeclare var BigInt: BigIntConstructor;\n\n/**\n * A typed array of 64-bit signed integer values. The contents are initialized to 0. If the\n * requested number of bytes could not be allocated, an exception is raised.\n */\ninterface BigInt64Array {\n    /** The size in bytes of each element in the array. */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /** The ArrayBuffer instance referenced by the array. */\n    readonly buffer: ArrayBufferLike;\n\n    /** The length in bytes of the array. */\n    readonly byteLength: number;\n\n    /** The offset in bytes of the array. */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /** Yields index, value pairs for every entry in the array. */\n    entries(): IterableIterator<[number, bigint]>;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns false,\n     * or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: bigint, index: number, array: BigInt64Array) => boolean, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: bigint, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: bigint, index: number, array: BigInt64Array) => any, thisArg?: any): BigInt64Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: bigint, index: number, array: BigInt64Array) => boolean, thisArg?: any): bigint | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: bigint, index: number, array: BigInt64Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: bigint, index: number, array: BigInt64Array) => void, thisArg?: any): void;\n\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: bigint, fromIndex?: number): boolean;\n\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    indexOf(searchElement: bigint, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /** Yields each index in the array. */\n    keys(): IterableIterator<number>;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: bigint, fromIndex?: number): number;\n\n    /** The length of the array. */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: bigint, index: number, array: BigInt64Array) => bigint, thisArg?: any): BigInt64Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: bigint, currentValue: bigint, currentIndex: number, array: BigInt64Array) => bigint): bigint;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: bigint, currentIndex: number, array: BigInt64Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: bigint, currentValue: bigint, currentIndex: number, array: BigInt64Array) => bigint): bigint;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: bigint, currentIndex: number, array: BigInt64Array) => U, initialValue: U): U;\n\n    /** Reverses the elements in the array. */\n    reverse(): this;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<bigint>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array.\n     */\n    slice(start?: number, end?: number): BigInt64Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls the\n     * predicate function for each element in the array until the predicate returns true, or until\n     * the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: bigint, index: number, array: BigInt64Array) => boolean, thisArg?: any): boolean;\n\n    /**\n     * Sorts the array.\n     * @param compareFn The function used to determine the order of the elements. If omitted, the elements are sorted in ascending order.\n     */\n    sort(compareFn?: (a: bigint, b: bigint) => number | bigint): this;\n\n    /**\n     * Gets a new BigInt64Array view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): BigInt64Array;\n\n    /** Converts the array to a string by using the current locale. */\n    toLocaleString(): string;\n\n    /** Returns a string representation of the array. */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): BigInt64Array;\n\n    /** Yields each value in the array. */\n    values(): IterableIterator<bigint>;\n\n    [Symbol.iterator](): IterableIterator<bigint>;\n\n    readonly [Symbol.toStringTag]: "BigInt64Array";\n\n    [index: number]: bigint;\n}\n\ninterface BigInt64ArrayConstructor {\n    readonly prototype: BigInt64Array;\n    new(length?: number): BigInt64Array;\n    new(array: Iterable<bigint>): BigInt64Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): BigInt64Array;\n\n    /** The size in bytes of each element in the array. */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: bigint[]): BigInt64Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of \'this\' used to invoke the mapfn.\n     */\n    from(arrayLike: ArrayLike<bigint>): BigInt64Array;\n    from<U>(arrayLike: ArrayLike<U>, mapfn: (v: U, k: number) => bigint, thisArg?: any): BigInt64Array;\n}\n\ndeclare var BigInt64Array: BigInt64ArrayConstructor;\n\n/**\n * A typed array of 64-bit unsigned integer values. The contents are initialized to 0. If the\n * requested number of bytes could not be allocated, an exception is raised.\n */\ninterface BigUint64Array {\n    /** The size in bytes of each element in the array. */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /** The ArrayBuffer instance referenced by the array. */\n    readonly buffer: ArrayBufferLike;\n\n    /** The length in bytes of the array. */\n    readonly byteLength: number;\n\n    /** The offset in bytes of the array. */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /** Yields index, value pairs for every entry in the array. */\n    entries(): IterableIterator<[number, bigint]>;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns false,\n     * or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: bigint, index: number, array: BigUint64Array) => boolean, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: bigint, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: bigint, index: number, array: BigUint64Array) => any, thisArg?: any): BigUint64Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: bigint, index: number, array: BigUint64Array) => boolean, thisArg?: any): bigint | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: bigint, index: number, array: BigUint64Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: bigint, index: number, array: BigUint64Array) => void, thisArg?: any): void;\n\n    /**\n     * Determines whether an array includes a certain element, returning true or false as appropriate.\n     * @param searchElement The element to search for.\n     * @param fromIndex The position in this array at which to begin searching for searchElement.\n     */\n    includes(searchElement: bigint, fromIndex?: number): boolean;\n\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    indexOf(searchElement: bigint, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /** Yields each index in the array. */\n    keys(): IterableIterator<number>;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: bigint, fromIndex?: number): number;\n\n    /** The length of the array. */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: bigint, index: number, array: BigUint64Array) => bigint, thisArg?: any): BigUint64Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: bigint, currentValue: bigint, currentIndex: number, array: BigUint64Array) => bigint): bigint;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: bigint, currentIndex: number, array: BigUint64Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: bigint, currentValue: bigint, currentIndex: number, array: BigUint64Array) => bigint): bigint;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: bigint, currentIndex: number, array: BigUint64Array) => U, initialValue: U): U;\n\n    /** Reverses the elements in the array. */\n    reverse(): this;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<bigint>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array.\n     */\n    slice(start?: number, end?: number): BigUint64Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls the\n     * predicate function for each element in the array until the predicate returns true, or until\n     * the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: bigint, index: number, array: BigUint64Array) => boolean, thisArg?: any): boolean;\n\n    /**\n     * Sorts the array.\n     * @param compareFn The function used to determine the order of the elements. If omitted, the elements are sorted in ascending order.\n     */\n    sort(compareFn?: (a: bigint, b: bigint) => number | bigint): this;\n\n    /**\n     * Gets a new BigUint64Array view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): BigUint64Array;\n\n    /** Converts the array to a string by using the current locale. */\n    toLocaleString(): string;\n\n    /** Returns a string representation of the array. */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): BigUint64Array;\n\n    /** Yields each value in the array. */\n    values(): IterableIterator<bigint>;\n\n    [Symbol.iterator](): IterableIterator<bigint>;\n\n    readonly [Symbol.toStringTag]: "BigUint64Array";\n\n    [index: number]: bigint;\n}\n\ninterface BigUint64ArrayConstructor {\n    readonly prototype: BigUint64Array;\n    new(length?: number): BigUint64Array;\n    new(array: Iterable<bigint>): BigUint64Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): BigUint64Array;\n\n    /** The size in bytes of each element in the array. */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: bigint[]): BigUint64Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of \'this\' used to invoke the mapfn.\n     */\n    from(arrayLike: ArrayLike<bigint>): BigUint64Array;\n    from<U>(arrayLike: ArrayLike<U>, mapfn: (v: U, k: number) => bigint, thisArg?: any): BigUint64Array;\n}\n\ndeclare var BigUint64Array: BigUint64ArrayConstructor;\n\ninterface DataView {\n    /**\n     * Gets the BigInt64 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getBigInt64(byteOffset: number, littleEndian?: boolean): bigint;\n\n    /**\n     * Gets the BigUint64 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getBigUint64(byteOffset: number, littleEndian?: boolean): bigint;\n\n    /**\n     * Stores a BigInt64 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     * @param littleEndian If false or undefined, a big-endian value should be written,\n     * otherwise a little-endian value should be written.\n     */\n    setBigInt64(byteOffset: number, value: bigint, littleEndian?: boolean): void;\n\n    /**\n     * Stores a BigUint64 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     * @param littleEndian If false or undefined, a big-endian value should be written,\n     * otherwise a little-endian value should be written.\n     */\n    setBigUint64(byteOffset: number, value: bigint, littleEndian?: boolean): void;\n}\n\ndeclare namespace Intl{\n    interface NumberFormat {\n        format(value: number | bigint): string;\n        resolvedOptions(): ResolvedNumberFormatOptions;\n    }\n}\n',"lib.es2020.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2019" />\n/// <reference lib="es2020.bigint" />\n/// <reference lib="es2020.promise" />\n/// <reference lib="es2020.sharedmemory" />\n/// <reference lib="es2020.string" />\n/// <reference lib="es2020.symbol.wellknown" />\n/// <reference lib="es2020.intl" />\n',"lib.es2020.full.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2020" />\n/// <reference lib="dom" />\n/// <reference lib="webworker.importscripts" />\n/// <reference lib="scripthost" />\n/// <reference lib="dom.iterable" />\n',"lib.es2020.intl.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ndeclare namespace Intl {\n\n    /**\n     * [Unicode BCP 47 Locale Identifiers](https://unicode.org/reports/tr35/#Unicode_Language_and_Locale_Identifiers) definition.\n     *\n     * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl#locales_argument).\n     *\n     */\n    type UnicodeBCP47LocaleIdentifier = string;\n\n    /**\n     * Unit to use in the relative time internationalized message.\n     *\n     * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/format#Parameters).\n     *\n     * [Specification](https://tc39.es/ecma402/#sec-singularrelativetimeunit).\n     */\n    type RelativeTimeFormatUnit =\n        | "year" | "years"\n        | "quarter" | "quarters"\n        | "month" | "months"\n        | "week" | "weeks"\n        | "day" | "days"\n        | "hour" | "hours"\n        | "minute" | "minutes"\n        | "second" | "seconds"\n        ;\n\n    /**\n     * The locale matching algorithm to use.\n     *\n     * [MDN](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Intl#Locale_negotiation).\n     *\n     * [Specification](https://tc39.es/ecma402/#sec-InitializeRelativeTimeFormat).\n     */\n    type RelativeTimeFormatLocaleMatcher = "lookup" | "best fit";\n\n    /**\n     * The format of output message.\n     *\n     * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/RelativeTimeFormat#Parameters).\n     *\n     * [Specification](https://tc39.es/ecma402/#sec-InitializeRelativeTimeFormat).\n     */\n    type RelativeTimeFormatNumeric = "always" | "auto";\n\n    /**\n     * The length of the internationalized message.\n     *\n     * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/RelativeTimeFormat#Parameters).\n     *\n     * [Specification](https://tc39.es/ecma402/#sec-InitializeRelativeTimeFormat).\n     */\n    type RelativeTimeFormatStyle = "long" | "short" | "narrow";\n\n    /**\n     * An object with some or all of properties of `options` parameter\n     * of `Intl.RelativeTimeFormat` constructor.\n     *\n     * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/RelativeTimeFormat#Parameters).\n     *\n     * [Specification](https://tc39.es/ecma402/#sec-InitializeRelativeTimeFormat).\n     */\n    interface RelativeTimeFormatOptions {\n        localeMatcher?: RelativeTimeFormatLocaleMatcher;\n        numeric?: RelativeTimeFormatNumeric;\n        style?: RelativeTimeFormatStyle;\n    }\n\n    /**\n     * An object with properties reflecting the locale\n     * and formatting options computed during initialization\n     * of the `Intel.RelativeTimeFormat` object\n     *\n     * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/resolvedOptions#Description).\n     *\n     * [Specification](https://tc39.es/ecma402/#table-relativetimeformat-resolvedoptions-properties)\n     */\n    interface ResolvedRelativeTimeFormatOptions {\n        locale: UnicodeBCP47LocaleIdentifier;\n        style: RelativeTimeFormatStyle;\n        numeric: RelativeTimeFormatNumeric;\n        numberingSystem: string;\n    }\n\n    /**\n     * An object representing the relative time format in parts\n     * that can be used for custom locale-aware formatting.\n     *\n     * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/formatToParts#Using_formatToParts).\n     *\n     * [Specification](https://tc39.es/ecma402/#sec-FormatRelativeTimeToParts).\n     */\n    interface RelativeTimeFormatPart {\n        type: string;\n        value: string;\n        unit?: RelativeTimeFormatUnit;\n    }\n\n    interface RelativeTimeFormat {\n        /**\n         * Formats a value and a unit according to the locale\n         * and formatting options of the given\n         * [`Intl.RelativeTimeFormat`](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/RelativeTimeFormat)\n         * object.\n         *\n         * While this method automatically provides the correct plural forms,\n         * the grammatical form is otherwise as neutral as possible.\n         * It is the caller\'s responsibility to handle cut-off logic\n         * such as deciding between displaying "in 7 days" or "in 1 week".\n         * This API does not support relative dates involving compound units.\n         * e.g "in 5 days and 4 hours".\n         *\n         * @param value -  Numeric value to use in the internationalized relative time message\n         *\n         * @param unit - [Unit](https://tc39.es/ecma402/#sec-singularrelativetimeunit)\n         *  to use in the relative time internationalized message.\n         *  Possible values are: `"year"`, `"quarter"`, `"month"`, `"week"`,\n         *  `"day"`, `"hour"`, `"minute"`, `"second"`.\n         *  Plural forms are also permitted.\n         *\n         * @throws `RangeError` if `unit` was given something other than `unit` possible values\n         *\n         * @returns Internationalized relative time message as string\n         *\n         * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/format).\n         *\n         * [Specification](https://tc39.es/ecma402/#sec-Intl.RelativeTimeFormat.prototype.format).\n         */\n        format(\n            value: number,\n            unit: RelativeTimeFormatUnit,\n        ): string;\n\n        /**\n         *  A version of the format method which it returns an array of objects\n         *  which represent "parts" of the object,\n         *  separating the formatted number into its constituent parts\n         *  and separating it from other surrounding text.\n         *  These objects have two properties:\n         * `type` a NumberFormat formatToParts type, and `value`,\n         *  which is the String which is the component of the output.\n         *  If a "part" came from NumberFormat,\n         *  it will have a unit property which indicates the `unit` being formatted;\n         *  literals which are part of the larger frame will not have this property.\n         *\n         *  @param value - Numeric value to use in the internationalized relative time message\n         *\n         *  @param unit - [Unit](https://tc39.es/ecma402/#sec-singularrelativetimeunit)\n         *   to use in the relative time internationalized message.\n         *   Possible values are: `"year"`, `"quarter"`, `"month"`, `"week"`,\n         *   `"day"`, `"hour"`, `"minute"`, `"second"`.\n         *   Plural forms are also permitted.\n         *\n         *  @throws `RangeError` if `unit` was given something other than `unit` possible values\n         *\n         *  @returns Array of [FormatRelativeTimeToParts](https://tc39.es/ecma402/#sec-FormatRelativeTimeToParts)\n         *\n         *  [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/formatToParts).\n         *\n         *  [Specification](https://tc39.es/ecma402/#sec-Intl.RelativeTimeFormat.prototype.formatToParts).\n         */\n        formatToParts(\n            value: number,\n            unit: RelativeTimeFormatUnit,\n        ): RelativeTimeFormatPart[];\n\n        /**\n         * Provides access to the locale and options computed during initialization of this `Intl.RelativeTimeFormat` object.\n         *\n         * @returns A new object with properties reflecting the locale\n         *  and formatting options computed during initialization\n         *  of the `Intel.RelativeTimeFormat` object.\n         *\n         * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/resolvedOptions).\n         *\n         * [Specification](https://tc39.es/ecma402/#sec-intl.relativetimeformat.prototype.resolvedoptions)\n         */\n        resolvedOptions(): ResolvedRelativeTimeFormatOptions;\n    }\n\n    /**\n     * The [`Intl.RelativeTimeFormat`](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/RelativeTimeFormat)\n     * object is a constructor for objects that enable language-sensitive relative time formatting.\n     *\n     * Part of [Intl object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl)\n     * namespace and the [ECMAScript Internationalization API](https://www.ecma-international.org/publications/standards/Ecma-402.htm).\n     *\n     * [Compatibility](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat#Browser_compatibility).\n     *\n     * [Polyfills](https://github.com/tc39/proposal-intl-relative-time#polyfills).\n     */\n    const RelativeTimeFormat: {\n        /**\n         * Constructor creates [Intl.RelativeTimeFormat](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RelativeTimeFormat)\n         * objects\n         *\n         * @param locales - A string with a [BCP 47 language tag](http://tools.ietf.org/html/rfc5646), or an array of such strings.\n         *  For the general form and interpretation of the locales argument,\n         *  see the [`Intl` page](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl#Locale_identification_and_negotiation).\n         *\n         * @param options - An [object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/RelativeTimeFormat#Parameters)\n         *  with some or all of options of the formatting.\n         *  An object with some or all of the following properties:\n         *  - `localeMatcher` - The locale matching algorithm to use.\n         *    Possible values are `"lookup"` and `"best fit"`; the default is `"best fit"`.\n         *    For information about this option, see [Intl page](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Intl#Locale_negotiation).\n         *  - `numeric` - The format of output message.\n         *    Possible values are: `"always"` (default, e.g., `1 day ago`) or `"auto"` (e.g., `yesterday`).\n         *    The `"auto"` value allows to not always have to use numeric values in the output.\n         *  - `style` - The length of the internationalized message. Possible values are:\n         *    `"long"` (default, e.g., in 1 month),\n         *    `"short"` (e.g., in 1 mo.)\n         *    or `"narrow"` (e.g., in 1 mo.). The narrow style could be similar to the short style for some locales.\n         *\n         * @returns [Intl.RelativeTimeFormat](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RelativeTimeFormat) object.\n         *\n         * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/RelativeTimeFormat).\n         *\n         * [Specification](https://tc39.es/ecma402/#sec-intl-relativetimeformat-constructor).\n         */\n        new(\n            locales?: UnicodeBCP47LocaleIdentifier | UnicodeBCP47LocaleIdentifier[],\n            options?: RelativeTimeFormatOptions,\n        ): RelativeTimeFormat;\n\n        /**\n         * Returns an array containing those of the provided locales\n         * that are supported in date and time formatting\n         * without having to fall back to the runtime\'s default locale.\n         *\n         * @param locales - A string with a [BCP 47 language tag](http://tools.ietf.org/html/rfc5646), or an array of such strings.\n         *  For the general form and interpretation of the locales argument,\n         *  see the [`Intl` page](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl#Locale_identification_and_negotiation).\n         *\n         * @param options - An [object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/RelativeTimeFormat#Parameters)\n         *  with some or all of options of the formatting.\n         *  An object with some or all of the following properties:\n         *  - `localeMatcher` - The locale matching algorithm to use.\n         *    Possible values are `"lookup"` and `"best fit"`; the default is `"best fit"`.\n         *    For information about this option, see [Intl page](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Intl#Locale_negotiation).\n         *  - `numeric` - The format of output message.\n         *    Possible values are: `"always"` (default, e.g., `1 day ago`) or `"auto"` (e.g., `yesterday`).\n         *    The `"auto"` value allows to not always have to use numeric values in the output.\n         *  - `style` - The length of the internationalized message. Possible values are:\n         *    `"long"` (default, e.g., in 1 month),\n         *    `"short"` (e.g., in 1 mo.)\n         *    or `"narrow"` (e.g., in 1 mo.). The narrow style could be similar to the short style for some locales.\n         *\n         * @returns An array containing those of the provided locales\n         *  that are supported in date and time formatting\n         *  without having to fall back to the runtime\'s default locale.\n         *\n         * [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/RelativeTimeFormat/supportedLocalesOf).\n         *\n         * [Specification](https://tc39.es/ecma402/#sec-Intl.RelativeTimeFormat.supportedLocalesOf).\n         */\n        supportedLocalesOf(\n            locales?: UnicodeBCP47LocaleIdentifier | UnicodeBCP47LocaleIdentifier[],\n            options?: RelativeTimeFormatOptions,\n        ): UnicodeBCP47LocaleIdentifier[];\n    };\n\n    interface NumberFormatOptions {\n        compactDisplay?: string;\n        notation?: string;\n        signDisplay?: string;\n        unit?: string;\n        unitDisplay?: string;\n    }\n\n    interface ResolvedNumberFormatOptions {\n        compactDisplay?: string;\n        notation?: string;\n        signDisplay?: string;\n        unit?: string;\n        unitDisplay?: string;\n    }\n\n    interface DateTimeFormatOptions {\n        dateStyle?: "full" | "long" | "medium" | "short";\n        timeStyle?: "full" | "long" | "medium" | "short";\n        calendar?: string;\n        dayPeriod?: "narrow" | "short" | "long";\n        numberingSystem?: string;\n        hourCycle?: "h11" | "h12" | "h23" | "h24";\n        fractionalSecondDigits?: 0 | 1 | 2 | 3;\n    }\n}\n',"lib.es2020.promise.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface PromiseFulfilledResult<T> {\n    status: "fulfilled";\n    value: T;\n}\n\ninterface PromiseRejectedResult {\n    status: "rejected";\n    reason: any;\n}\n\ntype PromiseSettledResult<T> = PromiseFulfilledResult<T> | PromiseRejectedResult;\n\ninterface PromiseConstructor {\n    /**\n     * Creates a Promise that is resolved with an array of results when all\n     * of the provided Promises resolve or reject.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    allSettled<T extends readonly unknown[] | readonly [unknown]>(values: T):\n        Promise<{ -readonly [P in keyof T]: PromiseSettledResult<T[P] extends PromiseLike<infer U> ? U : T[P]> }>;\n\n    /**\n     * Creates a Promise that is resolved with an array of results when all\n     * of the provided Promises resolve or reject.\n     * @param values An array of Promises.\n     * @returns A new Promise.\n     */\n    allSettled<T>(values: Iterable<T>): Promise<PromiseSettledResult<T extends PromiseLike<infer U> ? U : T>[]>;\n}\n',"lib.es2020.sharedmemory.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface Atomics {\n    /**\n     * Adds a value to the value at the given position in the array, returning the original value.\n     * Until this atomic operation completes, any other read or write operation against the array\n     * will block.\n     */\n    add(typedArray: BigInt64Array | BigUint64Array, index: number, value: bigint): bigint;\n\n    /**\n     * Stores the bitwise AND of a value with the value at the given position in the array,\n     * returning the original value. Until this atomic operation completes, any other read or\n     * write operation against the array will block.\n     */\n    and(typedArray: BigInt64Array | BigUint64Array, index: number, value: bigint): bigint;\n\n    /**\n     * Replaces the value at the given position in the array if the original value equals the given\n     * expected value, returning the original value. Until this atomic operation completes, any\n     * other read or write operation against the array will block.\n     */\n    compareExchange(typedArray: BigInt64Array | BigUint64Array, index: number, expectedValue: bigint, replacementValue: bigint): bigint;\n\n    /**\n     * Replaces the value at the given position in the array, returning the original value. Until\n     * this atomic operation completes, any other read or write operation against the array will\n     * block.\n     */\n    exchange(typedArray: BigInt64Array | BigUint64Array, index: number, value: bigint): bigint;\n\n    /**\n     * Returns the value at the given position in the array. Until this atomic operation completes,\n     * any other read or write operation against the array will block.\n     */\n    load(typedArray: BigInt64Array | BigUint64Array, index: number): bigint;\n\n    /**\n     * Stores the bitwise OR of a value with the value at the given position in the array,\n     * returning the original value. Until this atomic operation completes, any other read or write\n     * operation against the array will block.\n     */\n    or(typedArray: BigInt64Array | BigUint64Array, index: number, value: bigint): bigint;\n\n    /**\n     * Stores a value at the given position in the array, returning the new value. Until this\n     * atomic operation completes, any other read or write operation against the array will block.\n     */\n    store(typedArray: BigInt64Array | BigUint64Array, index: number, value: bigint): bigint;\n\n    /**\n     * Subtracts a value from the value at the given position in the array, returning the original\n     * value. Until this atomic operation completes, any other read or write operation against the\n     * array will block.\n     */\n    sub(typedArray: BigInt64Array | BigUint64Array, index: number, value: bigint): bigint;\n\n    /**\n     * If the value at the given position in the array is equal to the provided value, the current\n     * agent is put to sleep causing execution to suspend until the timeout expires (returning\n     * `"timed-out"`) or until the agent is awoken (returning `"ok"`); otherwise, returns\n     * `"not-equal"`.\n     */\n    wait(typedArray: BigInt64Array, index: number, value: bigint, timeout?: number): "ok" | "not-equal" | "timed-out";\n\n    /**\n     * Wakes up sleeping agents that are waiting on the given index of the array, returning the\n     * number of agents that were awoken.\n     * @param typedArray A shared BigInt64Array.\n     * @param index The position in the typedArray to wake up on.\n     * @param count The number of sleeping agents to notify. Defaults to +Infinity.\n     */\n    notify(typedArray: BigInt64Array, index: number, count?: number): number;\n\n    /**\n     * Stores the bitwise XOR of a value with the value at the given position in the array,\n     * returning the original value. Until this atomic operation completes, any other read or write\n     * operation against the array will block.\n     */\n    xor(typedArray: BigInt64Array | BigUint64Array, index: number, value: bigint): bigint;\n}\n',"lib.es2020.string.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2015.iterable" />\n\ninterface String {\n    /**\n     * Matches a string with a regular expression, and returns an iterable of matches\n     * containing the results of that search.\n     * @param regexp A variable name or string literal containing the regular expression pattern and flags.\n     */\n    matchAll(regexp: RegExp): IterableIterator<RegExpMatchArray>;\n}\n',"lib.es2020.symbol.wellknown.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2015.iterable" />\n/// <reference lib="es2015.symbol" />\n\ninterface SymbolConstructor {\n    /**\n     * A regular expression method that matches the regular expression against a string. Called\n     * by the String.prototype.matchAll method.\n     */\n    readonly matchAll: unique symbol;\n}\n\ninterface RegExp {\n    /**\n     * Matches a string with this regular expression, and returns an iterable of matches\n     * containing the results of that search.\n     * @param string A string to search within.\n     */\n    [Symbol.matchAll](str: string): IterableIterator<RegExpMatchArray>;\n}\n',"lib.es2021.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2020" />\n/// <reference lib="es2021.promise" />\n/// <reference lib="es2021.string" />\n/// <reference lib="es2021.weakref" />\n',"lib.es2021.full.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2021" />\n/// <reference lib="dom" />\n/// <reference lib="webworker.importscripts" />\n/// <reference lib="scripthost" />\n/// <reference lib="dom.iterable" />\n',"lib.es2021.promise.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface AggregateError extends Error {\n    errors: any[]\n}\n\ninterface AggregateErrorConstructor {\n    new(errors: Iterable<any>, message?: string): AggregateError;\n    (errors: Iterable<any>, message?: string): AggregateError;\n    readonly prototype: AggregateError;\n}\n\ndeclare var AggregateError: AggregateErrorConstructor;\n\n/**\n * Represents the completion of an asynchronous operation\n */\ninterface PromiseConstructor {\n    /**\n     * The any function returns a promise that is fulfilled by the first given promise to be fulfilled, or rejected with an AggregateError containing an array of rejection reasons if all of the given promises are rejected. It resolves all elements of the passed iterable to promises as it runs this algorithm.\n     * @param values An array or iterable of Promises.\n     * @returns A new Promise.\n     */\n    any<T>(values: (T | PromiseLike<T>)[] | Iterable<T | PromiseLike<T>>): Promise<T>\n}\n',"lib.es2021.string.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface String {\n    /**\n     * Replace all instances of a substring in a string, using a regular expression or search string.\n     * @param searchValue A string to search for.\n     * @param replaceValue A string containing the text to replace for every successful match of searchValue in this string.\n     */\n    replaceAll(searchValue: string | RegExp, replaceValue: string): string;\n\n    /**\n     * Replace all instances of a substring in a string, using a regular expression or search string.\n     * @param searchValue A string to search for.\n     * @param replacer A function that returns the replacement text.\n     */\n    replaceAll(searchValue: string | RegExp, replacer: (substring: string, ...args: any[]) => string): string;\n}\n',"lib.es2021.weakref.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface WeakRef<T extends object> {\n    readonly [Symbol.toStringTag]: "WeakRef";\n\n    /**\n     * Returns the WeakRef instance\'s target object, or undefined if the target object has been\n     * reclaimed.\n     */\n    deref(): T | undefined;\n}\n\ninterface WeakRefConstructor {\n    readonly prototype: WeakRef<any>;\n\n    /**\n     * Creates a WeakRef instance for the given target object.\n     * @param target The target object for the WeakRef instance.\n     */\n    new<T extends object>(target: T): WeakRef<T>;\n}\n\ndeclare var WeakRef: WeakRefConstructor;\n\ninterface FinalizationRegistry<T> {\n    readonly [Symbol.toStringTag]: "FinalizationRegistry";\n\n    /**\n     * Registers an object with the registry.\n     * @param target The target object to register.\n     * @param heldValue The value to pass to the finalizer for this object. This cannot be the\n     * target object.\n     * @param unregisterToken The token to pass to the unregister method to unregister the target\n     * object. If provided (and not undefined), this must be an object. If not provided, the target\n     * cannot be unregistered.\n     */\n    register(target: object, heldValue: T, unregisterToken?: object): void;\n\n    /**\n     * Unregisters an object from the registry.\n     * @param unregisterToken The token that was used as the unregisterToken argument when calling\n     * register to register the target object.\n     */\n    unregister(unregisterToken: object): void;\n}\n\ninterface FinalizationRegistryConstructor {\n    readonly prototype: FinalizationRegistry<any>;\n\n    /**\n     * Creates a finalization registry with an associated cleanup callback\n     * @param cleanupCallback The callback to call after an object in the registry has been reclaimed.\n     */\n    new<T>(cleanupCallback: (heldValue: T) => void): FinalizationRegistry<T>;\n}\n\ndeclare var FinalizationRegistry: FinalizationRegistryConstructor;\n',"lib.es5.d.ts":"/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib=\"true\"/>\n\n\n/////////////////////////////\n/// ECMAScript APIs\n/////////////////////////////\n\ndeclare var NaN: number;\ndeclare var Infinity: number;\n\n/**\n * Evaluates JavaScript code and executes it.\n * @param x A String value that contains valid JavaScript code.\n */\ndeclare function eval(x: string): any;\n\n/**\n * Converts a string to an integer.\n * @param string A string to convert into a number.\n * @param radix A value between 2 and 36 that specifies the base of the number in `string`.\n * If this argument is not supplied, strings with a prefix of '0x' are considered hexadecimal.\n * All other strings are considered decimal.\n */\ndeclare function parseInt(string: string, radix?: number): number;\n\n/**\n * Converts a string to a floating-point number.\n * @param string A string that contains a floating-point number.\n */\ndeclare function parseFloat(string: string): number;\n\n/**\n * Returns a Boolean value that indicates whether a value is the reserved value NaN (not a number).\n * @param number A numeric value.\n */\ndeclare function isNaN(number: number): boolean;\n\n/**\n * Determines whether a supplied number is finite.\n * @param number Any numeric value.\n */\ndeclare function isFinite(number: number): boolean;\n\n/**\n * Gets the unencoded version of an encoded Uniform Resource Identifier (URI).\n * @param encodedURI A value representing an encoded URI.\n */\ndeclare function decodeURI(encodedURI: string): string;\n\n/**\n * Gets the unencoded version of an encoded component of a Uniform Resource Identifier (URI).\n * @param encodedURIComponent A value representing an encoded URI component.\n */\ndeclare function decodeURIComponent(encodedURIComponent: string): string;\n\n/**\n * Encodes a text string as a valid Uniform Resource Identifier (URI)\n * @param uri A value representing an encoded URI.\n */\ndeclare function encodeURI(uri: string): string;\n\n/**\n * Encodes a text string as a valid component of a Uniform Resource Identifier (URI).\n * @param uriComponent A value representing an encoded URI component.\n */\ndeclare function encodeURIComponent(uriComponent: string | number | boolean): string;\n\n/**\n * Computes a new string in which certain characters have been replaced by a hexadecimal escape sequence.\n * @param string A string value\n */\ndeclare function escape(string: string): string;\n\n/**\n * Computes a new string in which hexadecimal escape sequences are replaced with the character that it represents.\n * @param string A string value\n */\ndeclare function unescape(string: string): string;\n\ninterface Symbol {\n    /** Returns a string representation of an object. */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): symbol;\n}\n\ndeclare type PropertyKey = string | number | symbol;\n\ninterface PropertyDescriptor {\n    configurable?: boolean;\n    enumerable?: boolean;\n    value?: any;\n    writable?: boolean;\n    get?(): any;\n    set?(v: any): void;\n}\n\ninterface PropertyDescriptorMap {\n    [s: string]: PropertyDescriptor;\n}\n\ninterface Object {\n    /** The initial value of Object.prototype.constructor is the standard built-in Object constructor. */\n    constructor: Function;\n\n    /** Returns a string representation of an object. */\n    toString(): string;\n\n    /** Returns a date converted to a string using the current locale. */\n    toLocaleString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Object;\n\n    /**\n     * Determines whether an object has a property with the specified name.\n     * @param v A property name.\n     */\n    hasOwnProperty(v: PropertyKey): boolean;\n\n    /**\n     * Determines whether an object exists in another object's prototype chain.\n     * @param v Another object whose prototype chain is to be checked.\n     */\n    isPrototypeOf(v: Object): boolean;\n\n    /**\n     * Determines whether a specified property is enumerable.\n     * @param v A property name.\n     */\n    propertyIsEnumerable(v: PropertyKey): boolean;\n}\n\ninterface ObjectConstructor {\n    new(value?: any): Object;\n    (): any;\n    (value: any): any;\n\n    /** A reference to the prototype for a class of objects. */\n    readonly prototype: Object;\n\n    /**\n     * Returns the prototype of an object.\n     * @param o The object that references the prototype.\n     */\n    getPrototypeOf(o: any): any;\n\n    /**\n     * Gets the own property descriptor of the specified object.\n     * An own property descriptor is one that is defined directly on the object and is not inherited from the object's prototype.\n     * @param o Object that contains the property.\n     * @param p Name of the property.\n     */\n    getOwnPropertyDescriptor(o: any, p: PropertyKey): PropertyDescriptor | undefined;\n\n    /**\n     * Returns the names of the own properties of an object. The own properties of an object are those that are defined directly\n     * on that object, and are not inherited from the object's prototype. The properties of an object include both fields (objects) and functions.\n     * @param o Object that contains the own properties.\n     */\n    getOwnPropertyNames(o: any): string[];\n\n    /**\n     * Creates an object that has the specified prototype or that has null prototype.\n     * @param o Object to use as a prototype. May be null.\n     */\n    create(o: object | null): any;\n\n    /**\n     * Creates an object that has the specified prototype, and that optionally contains specified properties.\n     * @param o Object to use as a prototype. May be null\n     * @param properties JavaScript object that contains one or more property descriptors.\n     */\n    create(o: object | null, properties: PropertyDescriptorMap & ThisType<any>): any;\n\n    /**\n     * Adds a property to an object, or modifies attributes of an existing property.\n     * @param o Object on which to add or modify the property. This can be a native JavaScript object (that is, a user-defined object or a built in object) or a DOM object.\n     * @param p The property name.\n     * @param attributes Descriptor for the property. It can be for a data property or an accessor property.\n     */\n    defineProperty<T>(o: T, p: PropertyKey, attributes: PropertyDescriptor & ThisType<any>): T;\n\n    /**\n     * Adds one or more properties to an object, and/or modifies attributes of existing properties.\n     * @param o Object on which to add or modify the properties. This can be a native JavaScript object or a DOM object.\n     * @param properties JavaScript object that contains one or more descriptor objects. Each descriptor object describes a data property or an accessor property.\n     */\n    defineProperties<T>(o: T, properties: PropertyDescriptorMap & ThisType<any>): T;\n\n    /**\n     * Prevents the modification of attributes of existing properties, and prevents the addition of new properties.\n     * @param o Object on which to lock the attributes.\n     */\n    seal<T>(o: T): T;\n\n    /**\n     * Prevents the modification of existing property attributes and values, and prevents the addition of new properties.\n     * @param o Object on which to lock the attributes.\n     */\n    freeze<T>(a: T[]): readonly T[];\n\n    /**\n     * Prevents the modification of existing property attributes and values, and prevents the addition of new properties.\n     * @param o Object on which to lock the attributes.\n     */\n    freeze<T extends Function>(f: T): T;\n\n    /**\n     * Prevents the modification of existing property attributes and values, and prevents the addition of new properties.\n     * @param o Object on which to lock the attributes.\n     */\n    freeze<T>(o: T): Readonly<T>;\n\n    /**\n     * Prevents the addition of new properties to an object.\n     * @param o Object to make non-extensible.\n     */\n    preventExtensions<T>(o: T): T;\n\n    /**\n     * Returns true if existing property attributes cannot be modified in an object and new properties cannot be added to the object.\n     * @param o Object to test.\n     */\n    isSealed(o: any): boolean;\n\n    /**\n     * Returns true if existing property attributes and values cannot be modified in an object, and new properties cannot be added to the object.\n     * @param o Object to test.\n     */\n    isFrozen(o: any): boolean;\n\n    /**\n     * Returns a value that indicates whether new properties can be added to an object.\n     * @param o Object to test.\n     */\n    isExtensible(o: any): boolean;\n\n    /**\n     * Returns the names of the enumerable string properties and methods of an object.\n     * @param o Object that contains the properties and methods. This can be an object that you created or an existing Document Object Model (DOM) object.\n     */\n    keys(o: object): string[];\n}\n\n/**\n * Provides functionality common to all JavaScript objects.\n */\ndeclare var Object: ObjectConstructor;\n\n/**\n * Creates a new function.\n */\ninterface Function {\n    /**\n     * Calls the function, substituting the specified object for the this value of the function, and the specified array for the arguments of the function.\n     * @param thisArg The object to be used as the this object.\n     * @param argArray A set of arguments to be passed to the function.\n     */\n    apply(this: Function, thisArg: any, argArray?: any): any;\n\n    /**\n     * Calls a method of an object, substituting another object for the current object.\n     * @param thisArg The object to be used as the current object.\n     * @param argArray A list of arguments to be passed to the method.\n     */\n    call(this: Function, thisArg: any, ...argArray: any[]): any;\n\n    /**\n     * For a given function, creates a bound function that has the same body as the original function.\n     * The this object of the bound function is associated with the specified object, and has the specified initial parameters.\n     * @param thisArg An object to which the this keyword can refer inside the new function.\n     * @param argArray A list of arguments to be passed to the new function.\n     */\n    bind(this: Function, thisArg: any, ...argArray: any[]): any;\n\n    /** Returns a string representation of a function. */\n    toString(): string;\n\n    prototype: any;\n    readonly length: number;\n\n    // Non-standard extensions\n    arguments: any;\n    caller: Function;\n}\n\ninterface FunctionConstructor {\n    /**\n     * Creates a new function.\n     * @param args A list of arguments the function accepts.\n     */\n    new(...args: string[]): Function;\n    (...args: string[]): Function;\n    readonly prototype: Function;\n}\n\ndeclare var Function: FunctionConstructor;\n\n/**\n * Extracts the type of the 'this' parameter of a function type, or 'unknown' if the function type has no 'this' parameter.\n */\ntype ThisParameterType<T> = T extends (this: infer U, ...args: any[]) => any ? U : unknown;\n\n/**\n * Removes the 'this' parameter from a function type.\n */\ntype OmitThisParameter<T> = unknown extends ThisParameterType<T> ? T : T extends (...args: infer A) => infer R ? (...args: A) => R : T;\n\ninterface CallableFunction extends Function {\n    /**\n     * Calls the function with the specified object as the this value and the elements of specified array as the arguments.\n     * @param thisArg The object to be used as the this object.\n     * @param args An array of argument values to be passed to the function.\n     */\n    apply<T, R>(this: (this: T) => R, thisArg: T): R;\n    apply<T, A extends any[], R>(this: (this: T, ...args: A) => R, thisArg: T, args: A): R;\n\n    /**\n     * Calls the function with the specified object as the this value and the specified rest arguments as the arguments.\n     * @param thisArg The object to be used as the this object.\n     * @param args Argument values to be passed to the function.\n     */\n    call<T, A extends any[], R>(this: (this: T, ...args: A) => R, thisArg: T, ...args: A): R;\n\n    /**\n     * For a given function, creates a bound function that has the same body as the original function.\n     * The this object of the bound function is associated with the specified object, and has the specified initial parameters.\n     * @param thisArg The object to be used as the this object.\n     * @param args Arguments to bind to the parameters of the function.\n     */\n    bind<T>(this: T, thisArg: ThisParameterType<T>): OmitThisParameter<T>;\n    bind<T, A0, A extends any[], R>(this: (this: T, arg0: A0, ...args: A) => R, thisArg: T, arg0: A0): (...args: A) => R;\n    bind<T, A0, A1, A extends any[], R>(this: (this: T, arg0: A0, arg1: A1, ...args: A) => R, thisArg: T, arg0: A0, arg1: A1): (...args: A) => R;\n    bind<T, A0, A1, A2, A extends any[], R>(this: (this: T, arg0: A0, arg1: A1, arg2: A2, ...args: A) => R, thisArg: T, arg0: A0, arg1: A1, arg2: A2): (...args: A) => R;\n    bind<T, A0, A1, A2, A3, A extends any[], R>(this: (this: T, arg0: A0, arg1: A1, arg2: A2, arg3: A3, ...args: A) => R, thisArg: T, arg0: A0, arg1: A1, arg2: A2, arg3: A3): (...args: A) => R;\n    bind<T, AX, R>(this: (this: T, ...args: AX[]) => R, thisArg: T, ...args: AX[]): (...args: AX[]) => R;\n}\n\ninterface NewableFunction extends Function {\n    /**\n     * Calls the function with the specified object as the this value and the elements of specified array as the arguments.\n     * @param thisArg The object to be used as the this object.\n     * @param args An array of argument values to be passed to the function.\n     */\n    apply<T>(this: new () => T, thisArg: T): void;\n    apply<T, A extends any[]>(this: new (...args: A) => T, thisArg: T, args: A): void;\n\n    /**\n     * Calls the function with the specified object as the this value and the specified rest arguments as the arguments.\n     * @param thisArg The object to be used as the this object.\n     * @param args Argument values to be passed to the function.\n     */\n    call<T, A extends any[]>(this: new (...args: A) => T, thisArg: T, ...args: A): void;\n\n    /**\n     * For a given function, creates a bound function that has the same body as the original function.\n     * The this object of the bound function is associated with the specified object, and has the specified initial parameters.\n     * @param thisArg The object to be used as the this object.\n     * @param args Arguments to bind to the parameters of the function.\n     */\n    bind<T>(this: T, thisArg: any): T;\n    bind<A0, A extends any[], R>(this: new (arg0: A0, ...args: A) => R, thisArg: any, arg0: A0): new (...args: A) => R;\n    bind<A0, A1, A extends any[], R>(this: new (arg0: A0, arg1: A1, ...args: A) => R, thisArg: any, arg0: A0, arg1: A1): new (...args: A) => R;\n    bind<A0, A1, A2, A extends any[], R>(this: new (arg0: A0, arg1: A1, arg2: A2, ...args: A) => R, thisArg: any, arg0: A0, arg1: A1, arg2: A2): new (...args: A) => R;\n    bind<A0, A1, A2, A3, A extends any[], R>(this: new (arg0: A0, arg1: A1, arg2: A2, arg3: A3, ...args: A) => R, thisArg: any, arg0: A0, arg1: A1, arg2: A2, arg3: A3): new (...args: A) => R;\n    bind<AX, R>(this: new (...args: AX[]) => R, thisArg: any, ...args: AX[]): new (...args: AX[]) => R;\n}\n\ninterface IArguments {\n    [index: number]: any;\n    length: number;\n    callee: Function;\n}\n\ninterface String {\n    /** Returns a string representation of a string. */\n    toString(): string;\n\n    /**\n     * Returns the character at the specified index.\n     * @param pos The zero-based index of the desired character.\n     */\n    charAt(pos: number): string;\n\n    /**\n     * Returns the Unicode value of the character at the specified location.\n     * @param index The zero-based index of the desired character. If there is no character at the specified index, NaN is returned.\n     */\n    charCodeAt(index: number): number;\n\n    /**\n     * Returns a string that contains the concatenation of two or more strings.\n     * @param strings The strings to append to the end of the string.\n     */\n    concat(...strings: string[]): string;\n\n    /**\n     * Returns the position of the first occurrence of a substring.\n     * @param searchString The substring to search for in the string\n     * @param position The index at which to begin searching the String object. If omitted, search starts at the beginning of the string.\n     */\n    indexOf(searchString: string, position?: number): number;\n\n    /**\n     * Returns the last occurrence of a substring in the string.\n     * @param searchString The substring to search for.\n     * @param position The index at which to begin searching. If omitted, the search begins at the end of the string.\n     */\n    lastIndexOf(searchString: string, position?: number): number;\n\n    /**\n     * Determines whether two strings are equivalent in the current locale.\n     * @param that String to compare to target string\n     */\n    localeCompare(that: string): number;\n\n    /**\n     * Matches a string with a regular expression, and returns an array containing the results of that search.\n     * @param regexp A variable name or string literal containing the regular expression pattern and flags.\n     */\n    match(regexp: string | RegExp): RegExpMatchArray | null;\n\n    /**\n     * Replaces text in a string, using a regular expression or search string.\n     * @param searchValue A string to search for.\n     * @param replaceValue A string containing the text to replace for every successful match of searchValue in this string.\n     */\n    replace(searchValue: string | RegExp, replaceValue: string): string;\n\n    /**\n     * Replaces text in a string, using a regular expression or search string.\n     * @param searchValue A string to search for.\n     * @param replacer A function that returns the replacement text.\n     */\n    replace(searchValue: string | RegExp, replacer: (substring: string, ...args: any[]) => string): string;\n\n    /**\n     * Finds the first substring match in a regular expression search.\n     * @param regexp The regular expression pattern and applicable flags.\n     */\n    search(regexp: string | RegExp): number;\n\n    /**\n     * Returns a section of a string.\n     * @param start The index to the beginning of the specified portion of stringObj.\n     * @param end The index to the end of the specified portion of stringObj. The substring includes the characters up to, but not including, the character indicated by end.\n     * If this value is not specified, the substring continues to the end of stringObj.\n     */\n    slice(start?: number, end?: number): string;\n\n    /**\n     * Split a string into substrings using the specified separator and return them as an array.\n     * @param separator A string that identifies character or characters to use in separating the string. If omitted, a single-element array containing the entire string is returned.\n     * @param limit A value used to limit the number of elements returned in the array.\n     */\n    split(separator: string | RegExp, limit?: number): string[];\n\n    /**\n     * Returns the substring at the specified location within a String object.\n     * @param start The zero-based index number indicating the beginning of the substring.\n     * @param end Zero-based index number indicating the end of the substring. The substring includes the characters up to, but not including, the character indicated by end.\n     * If end is omitted, the characters from start through the end of the original string are returned.\n     */\n    substring(start: number, end?: number): string;\n\n    /** Converts all the alphabetic characters in a string to lowercase. */\n    toLowerCase(): string;\n\n    /** Converts all alphabetic characters to lowercase, taking into account the host environment's current locale. */\n    toLocaleLowerCase(locales?: string | string[]): string;\n\n    /** Converts all the alphabetic characters in a string to uppercase. */\n    toUpperCase(): string;\n\n    /** Returns a string where all alphabetic characters have been converted to uppercase, taking into account the host environment's current locale. */\n    toLocaleUpperCase(locales?: string | string[]): string;\n\n    /** Removes the leading and trailing white space and line terminator characters from a string. */\n    trim(): string;\n\n    /** Returns the length of a String object. */\n    readonly length: number;\n\n    // IE extensions\n    /**\n     * Gets a substring beginning at the specified location and having the specified length.\n     * @param from The starting position of the desired substring. The index of the first character in the string is zero.\n     * @param length The number of characters to include in the returned substring.\n     */\n    substr(from: number, length?: number): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): string;\n\n    readonly [index: number]: string;\n}\n\ninterface StringConstructor {\n    new(value?: any): String;\n    (value?: any): string;\n    readonly prototype: String;\n    fromCharCode(...codes: number[]): string;\n}\n\n/**\n * Allows manipulation and formatting of text strings and determination and location of substrings within strings.\n */\ndeclare var String: StringConstructor;\n\ninterface Boolean {\n    /** Returns the primitive value of the specified object. */\n    valueOf(): boolean;\n}\n\ninterface BooleanConstructor {\n    new(value?: any): Boolean;\n    <T>(value?: T): boolean;\n    readonly prototype: Boolean;\n}\n\ndeclare var Boolean: BooleanConstructor;\n\ninterface Number {\n    /**\n     * Returns a string representation of an object.\n     * @param radix Specifies a radix for converting numeric values to strings. This value is only used for numbers.\n     */\n    toString(radix?: number): string;\n\n    /**\n     * Returns a string representing a number in fixed-point notation.\n     * @param fractionDigits Number of digits after the decimal point. Must be in the range 0 - 20, inclusive.\n     */\n    toFixed(fractionDigits?: number): string;\n\n    /**\n     * Returns a string containing a number represented in exponential notation.\n     * @param fractionDigits Number of digits after the decimal point. Must be in the range 0 - 20, inclusive.\n     */\n    toExponential(fractionDigits?: number): string;\n\n    /**\n     * Returns a string containing a number represented either in exponential or fixed-point notation with a specified number of digits.\n     * @param precision Number of significant digits. Must be in the range 1 - 21, inclusive.\n     */\n    toPrecision(precision?: number): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): number;\n}\n\ninterface NumberConstructor {\n    new(value?: any): Number;\n    (value?: any): number;\n    readonly prototype: Number;\n\n    /** The largest number that can be represented in JavaScript. Equal to approximately 1.79E+308. */\n    readonly MAX_VALUE: number;\n\n    /** The closest number to zero that can be represented in JavaScript. Equal to approximately 5.00E-324. */\n    readonly MIN_VALUE: number;\n\n    /**\n     * A value that is not a number.\n     * In equality comparisons, NaN does not equal any value, including itself. To test whether a value is equivalent to NaN, use the isNaN function.\n     */\n    readonly NaN: number;\n\n    /**\n     * A value that is less than the largest negative number that can be represented in JavaScript.\n     * JavaScript displays NEGATIVE_INFINITY values as -infinity.\n     */\n    readonly NEGATIVE_INFINITY: number;\n\n    /**\n     * A value greater than the largest number that can be represented in JavaScript.\n     * JavaScript displays POSITIVE_INFINITY values as infinity.\n     */\n    readonly POSITIVE_INFINITY: number;\n}\n\n/** An object that represents a number of any kind. All JavaScript numbers are 64-bit floating-point numbers. */\ndeclare var Number: NumberConstructor;\n\ninterface TemplateStringsArray extends ReadonlyArray<string> {\n    readonly raw: readonly string[];\n}\n\n/**\n * The type of `import.meta`.\n *\n * If you need to declare that a given property exists on `import.meta`,\n * this type may be augmented via interface merging.\n */\ninterface ImportMeta {\n}\n\ninterface Math {\n    /** The mathematical constant e. This is Euler's number, the base of natural logarithms. */\n    readonly E: number;\n    /** The natural logarithm of 10. */\n    readonly LN10: number;\n    /** The natural logarithm of 2. */\n    readonly LN2: number;\n    /** The base-2 logarithm of e. */\n    readonly LOG2E: number;\n    /** The base-10 logarithm of e. */\n    readonly LOG10E: number;\n    /** Pi. This is the ratio of the circumference of a circle to its diameter. */\n    readonly PI: number;\n    /** The square root of 0.5, or, equivalently, one divided by the square root of 2. */\n    readonly SQRT1_2: number;\n    /** The square root of 2. */\n    readonly SQRT2: number;\n    /**\n     * Returns the absolute value of a number (the value without regard to whether it is positive or negative).\n     * For example, the absolute value of -5 is the same as the absolute value of 5.\n     * @param x A numeric expression for which the absolute value is needed.\n     */\n    abs(x: number): number;\n    /**\n     * Returns the arc cosine (or inverse cosine) of a number.\n     * @param x A numeric expression.\n     */\n    acos(x: number): number;\n    /**\n     * Returns the arcsine of a number.\n     * @param x A numeric expression.\n     */\n    asin(x: number): number;\n    /**\n     * Returns the arctangent of a number.\n     * @param x A numeric expression for which the arctangent is needed.\n     */\n    atan(x: number): number;\n    /**\n     * Returns the angle (in radians) from the X axis to a point.\n     * @param y A numeric expression representing the cartesian y-coordinate.\n     * @param x A numeric expression representing the cartesian x-coordinate.\n     */\n    atan2(y: number, x: number): number;\n    /**\n     * Returns the smallest integer greater than or equal to its numeric argument.\n     * @param x A numeric expression.\n     */\n    ceil(x: number): number;\n    /**\n     * Returns the cosine of a number.\n     * @param x A numeric expression that contains an angle measured in radians.\n     */\n    cos(x: number): number;\n    /**\n     * Returns e (the base of natural logarithms) raised to a power.\n     * @param x A numeric expression representing the power of e.\n     */\n    exp(x: number): number;\n    /**\n     * Returns the greatest integer less than or equal to its numeric argument.\n     * @param x A numeric expression.\n     */\n    floor(x: number): number;\n    /**\n     * Returns the natural logarithm (base e) of a number.\n     * @param x A numeric expression.\n     */\n    log(x: number): number;\n    /**\n     * Returns the larger of a set of supplied numeric expressions.\n     * @param values Numeric expressions to be evaluated.\n     */\n    max(...values: number[]): number;\n    /**\n     * Returns the smaller of a set of supplied numeric expressions.\n     * @param values Numeric expressions to be evaluated.\n     */\n    min(...values: number[]): number;\n    /**\n     * Returns the value of a base expression taken to a specified power.\n     * @param x The base value of the expression.\n     * @param y The exponent value of the expression.\n     */\n    pow(x: number, y: number): number;\n    /** Returns a pseudorandom number between 0 and 1. */\n    random(): number;\n    /**\n     * Returns a supplied numeric expression rounded to the nearest integer.\n     * @param x The value to be rounded to the nearest integer.\n     */\n    round(x: number): number;\n    /**\n     * Returns the sine of a number.\n     * @param x A numeric expression that contains an angle measured in radians.\n     */\n    sin(x: number): number;\n    /**\n     * Returns the square root of a number.\n     * @param x A numeric expression.\n     */\n    sqrt(x: number): number;\n    /**\n     * Returns the tangent of a number.\n     * @param x A numeric expression that contains an angle measured in radians.\n     */\n    tan(x: number): number;\n}\n/** An intrinsic object that provides basic mathematics functionality and constants. */\ndeclare var Math: Math;\n\n/** Enables basic storage and retrieval of dates and times. */\ninterface Date {\n    /** Returns a string representation of a date. The format of the string depends on the locale. */\n    toString(): string;\n    /** Returns a date as a string value. */\n    toDateString(): string;\n    /** Returns a time as a string value. */\n    toTimeString(): string;\n    /** Returns a value as a string value appropriate to the host environment's current locale. */\n    toLocaleString(): string;\n    /** Returns a date as a string value appropriate to the host environment's current locale. */\n    toLocaleDateString(): string;\n    /** Returns a time as a string value appropriate to the host environment's current locale. */\n    toLocaleTimeString(): string;\n    /** Returns the stored time value in milliseconds since midnight, January 1, 1970 UTC. */\n    valueOf(): number;\n    /** Gets the time value in milliseconds. */\n    getTime(): number;\n    /** Gets the year, using local time. */\n    getFullYear(): number;\n    /** Gets the year using Universal Coordinated Time (UTC). */\n    getUTCFullYear(): number;\n    /** Gets the month, using local time. */\n    getMonth(): number;\n    /** Gets the month of a Date object using Universal Coordinated Time (UTC). */\n    getUTCMonth(): number;\n    /** Gets the day-of-the-month, using local time. */\n    getDate(): number;\n    /** Gets the day-of-the-month, using Universal Coordinated Time (UTC). */\n    getUTCDate(): number;\n    /** Gets the day of the week, using local time. */\n    getDay(): number;\n    /** Gets the day of the week using Universal Coordinated Time (UTC). */\n    getUTCDay(): number;\n    /** Gets the hours in a date, using local time. */\n    getHours(): number;\n    /** Gets the hours value in a Date object using Universal Coordinated Time (UTC). */\n    getUTCHours(): number;\n    /** Gets the minutes of a Date object, using local time. */\n    getMinutes(): number;\n    /** Gets the minutes of a Date object using Universal Coordinated Time (UTC). */\n    getUTCMinutes(): number;\n    /** Gets the seconds of a Date object, using local time. */\n    getSeconds(): number;\n    /** Gets the seconds of a Date object using Universal Coordinated Time (UTC). */\n    getUTCSeconds(): number;\n    /** Gets the milliseconds of a Date, using local time. */\n    getMilliseconds(): number;\n    /** Gets the milliseconds of a Date object using Universal Coordinated Time (UTC). */\n    getUTCMilliseconds(): number;\n    /** Gets the difference in minutes between the time on the local computer and Universal Coordinated Time (UTC). */\n    getTimezoneOffset(): number;\n    /**\n     * Sets the date and time value in the Date object.\n     * @param time A numeric value representing the number of elapsed milliseconds since midnight, January 1, 1970 GMT.\n     */\n    setTime(time: number): number;\n    /**\n     * Sets the milliseconds value in the Date object using local time.\n     * @param ms A numeric value equal to the millisecond value.\n     */\n    setMilliseconds(ms: number): number;\n    /**\n     * Sets the milliseconds value in the Date object using Universal Coordinated Time (UTC).\n     * @param ms A numeric value equal to the millisecond value.\n     */\n    setUTCMilliseconds(ms: number): number;\n\n    /**\n     * Sets the seconds value in the Date object using local time.\n     * @param sec A numeric value equal to the seconds value.\n     * @param ms A numeric value equal to the milliseconds value.\n     */\n    setSeconds(sec: number, ms?: number): number;\n    /**\n     * Sets the seconds value in the Date object using Universal Coordinated Time (UTC).\n     * @param sec A numeric value equal to the seconds value.\n     * @param ms A numeric value equal to the milliseconds value.\n     */\n    setUTCSeconds(sec: number, ms?: number): number;\n    /**\n     * Sets the minutes value in the Date object using local time.\n     * @param min A numeric value equal to the minutes value.\n     * @param sec A numeric value equal to the seconds value.\n     * @param ms A numeric value equal to the milliseconds value.\n     */\n    setMinutes(min: number, sec?: number, ms?: number): number;\n    /**\n     * Sets the minutes value in the Date object using Universal Coordinated Time (UTC).\n     * @param min A numeric value equal to the minutes value.\n     * @param sec A numeric value equal to the seconds value.\n     * @param ms A numeric value equal to the milliseconds value.\n     */\n    setUTCMinutes(min: number, sec?: number, ms?: number): number;\n    /**\n     * Sets the hour value in the Date object using local time.\n     * @param hours A numeric value equal to the hours value.\n     * @param min A numeric value equal to the minutes value.\n     * @param sec A numeric value equal to the seconds value.\n     * @param ms A numeric value equal to the milliseconds value.\n     */\n    setHours(hours: number, min?: number, sec?: number, ms?: number): number;\n    /**\n     * Sets the hours value in the Date object using Universal Coordinated Time (UTC).\n     * @param hours A numeric value equal to the hours value.\n     * @param min A numeric value equal to the minutes value.\n     * @param sec A numeric value equal to the seconds value.\n     * @param ms A numeric value equal to the milliseconds value.\n     */\n    setUTCHours(hours: number, min?: number, sec?: number, ms?: number): number;\n    /**\n     * Sets the numeric day-of-the-month value of the Date object using local time.\n     * @param date A numeric value equal to the day of the month.\n     */\n    setDate(date: number): number;\n    /**\n     * Sets the numeric day of the month in the Date object using Universal Coordinated Time (UTC).\n     * @param date A numeric value equal to the day of the month.\n     */\n    setUTCDate(date: number): number;\n    /**\n     * Sets the month value in the Date object using local time.\n     * @param month A numeric value equal to the month. The value for January is 0, and other month values follow consecutively.\n     * @param date A numeric value representing the day of the month. If this value is not supplied, the value from a call to the getDate method is used.\n     */\n    setMonth(month: number, date?: number): number;\n    /**\n     * Sets the month value in the Date object using Universal Coordinated Time (UTC).\n     * @param month A numeric value equal to the month. The value for January is 0, and other month values follow consecutively.\n     * @param date A numeric value representing the day of the month. If it is not supplied, the value from a call to the getUTCDate method is used.\n     */\n    setUTCMonth(month: number, date?: number): number;\n    /**\n     * Sets the year of the Date object using local time.\n     * @param year A numeric value for the year.\n     * @param month A zero-based numeric value for the month (0 for January, 11 for December). Must be specified if numDate is specified.\n     * @param date A numeric value equal for the day of the month.\n     */\n    setFullYear(year: number, month?: number, date?: number): number;\n    /**\n     * Sets the year value in the Date object using Universal Coordinated Time (UTC).\n     * @param year A numeric value equal to the year.\n     * @param month A numeric value equal to the month. The value for January is 0, and other month values follow consecutively. Must be supplied if numDate is supplied.\n     * @param date A numeric value equal to the day of the month.\n     */\n    setUTCFullYear(year: number, month?: number, date?: number): number;\n    /** Returns a date converted to a string using Universal Coordinated Time (UTC). */\n    toUTCString(): string;\n    /** Returns a date as a string value in ISO format. */\n    toISOString(): string;\n    /** Used by the JSON.stringify method to enable the transformation of an object's data for JavaScript Object Notation (JSON) serialization. */\n    toJSON(key?: any): string;\n}\n\ninterface DateConstructor {\n    new(): Date;\n    new(value: number | string): Date;\n    new(year: number, month: number, date?: number, hours?: number, minutes?: number, seconds?: number, ms?: number): Date;\n    (): string;\n    readonly prototype: Date;\n    /**\n     * Parses a string containing a date, and returns the number of milliseconds between that date and midnight, January 1, 1970.\n     * @param s A date string\n     */\n    parse(s: string): number;\n    /**\n     * Returns the number of milliseconds between midnight, January 1, 1970 Universal Coordinated Time (UTC) (or GMT) and the specified date.\n     * @param year The full year designation is required for cross-century date accuracy. If year is between 0 and 99 is used, then year is assumed to be 1900 + year.\n     * @param month The month as a number between 0 and 11 (January to December).\n     * @param date The date as a number between 1 and 31.\n     * @param hours Must be supplied if minutes is supplied. A number from 0 to 23 (midnight to 11pm) that specifies the hour.\n     * @param minutes Must be supplied if seconds is supplied. A number from 0 to 59 that specifies the minutes.\n     * @param seconds Must be supplied if milliseconds is supplied. A number from 0 to 59 that specifies the seconds.\n     * @param ms A number from 0 to 999 that specifies the milliseconds.\n     */\n    UTC(year: number, month: number, date?: number, hours?: number, minutes?: number, seconds?: number, ms?: number): number;\n    now(): number;\n}\n\ndeclare var Date: DateConstructor;\n\ninterface RegExpMatchArray extends Array<string> {\n    index?: number;\n    input?: string;\n}\n\ninterface RegExpExecArray extends Array<string> {\n    index: number;\n    input: string;\n}\n\ninterface RegExp {\n    /**\n     * Executes a search on a string using a regular expression pattern, and returns an array containing the results of that search.\n     * @param string The String object or string literal on which to perform the search.\n     */\n    exec(string: string): RegExpExecArray | null;\n\n    /**\n     * Returns a Boolean value that indicates whether or not a pattern exists in a searched string.\n     * @param string String on which to perform the search.\n     */\n    test(string: string): boolean;\n\n    /** Returns a copy of the text of the regular expression pattern. Read-only. The regExp argument is a Regular expression object. It can be a variable name or a literal. */\n    readonly source: string;\n\n    /** Returns a Boolean value indicating the state of the global flag (g) used with a regular expression. Default is false. Read-only. */\n    readonly global: boolean;\n\n    /** Returns a Boolean value indicating the state of the ignoreCase flag (i) used with a regular expression. Default is false. Read-only. */\n    readonly ignoreCase: boolean;\n\n    /** Returns a Boolean value indicating the state of the multiline flag (m) used with a regular expression. Default is false. Read-only. */\n    readonly multiline: boolean;\n\n    lastIndex: number;\n\n    // Non-standard extensions\n    compile(): this;\n}\n\ninterface RegExpConstructor {\n    new(pattern: RegExp | string): RegExp;\n    new(pattern: string, flags?: string): RegExp;\n    (pattern: RegExp | string): RegExp;\n    (pattern: string, flags?: string): RegExp;\n    readonly prototype: RegExp;\n\n    // Non-standard extensions\n    $1: string;\n    $2: string;\n    $3: string;\n    $4: string;\n    $5: string;\n    $6: string;\n    $7: string;\n    $8: string;\n    $9: string;\n    lastMatch: string;\n}\n\ndeclare var RegExp: RegExpConstructor;\n\ninterface Error {\n    name: string;\n    message: string;\n    stack?: string;\n}\n\ninterface ErrorConstructor {\n    new(message?: string): Error;\n    (message?: string): Error;\n    readonly prototype: Error;\n}\n\ndeclare var Error: ErrorConstructor;\n\ninterface EvalError extends Error {\n}\n\ninterface EvalErrorConstructor extends ErrorConstructor {\n    new(message?: string): EvalError;\n    (message?: string): EvalError;\n    readonly prototype: EvalError;\n}\n\ndeclare var EvalError: EvalErrorConstructor;\n\ninterface RangeError extends Error {\n}\n\ninterface RangeErrorConstructor extends ErrorConstructor {\n    new(message?: string): RangeError;\n    (message?: string): RangeError;\n    readonly prototype: RangeError;\n}\n\ndeclare var RangeError: RangeErrorConstructor;\n\ninterface ReferenceError extends Error {\n}\n\ninterface ReferenceErrorConstructor extends ErrorConstructor {\n    new(message?: string): ReferenceError;\n    (message?: string): ReferenceError;\n    readonly prototype: ReferenceError;\n}\n\ndeclare var ReferenceError: ReferenceErrorConstructor;\n\ninterface SyntaxError extends Error {\n}\n\ninterface SyntaxErrorConstructor extends ErrorConstructor {\n    new(message?: string): SyntaxError;\n    (message?: string): SyntaxError;\n    readonly prototype: SyntaxError;\n}\n\ndeclare var SyntaxError: SyntaxErrorConstructor;\n\ninterface TypeError extends Error {\n}\n\ninterface TypeErrorConstructor extends ErrorConstructor {\n    new(message?: string): TypeError;\n    (message?: string): TypeError;\n    readonly prototype: TypeError;\n}\n\ndeclare var TypeError: TypeErrorConstructor;\n\ninterface URIError extends Error {\n}\n\ninterface URIErrorConstructor extends ErrorConstructor {\n    new(message?: string): URIError;\n    (message?: string): URIError;\n    readonly prototype: URIError;\n}\n\ndeclare var URIError: URIErrorConstructor;\n\ninterface JSON {\n    /**\n     * Converts a JavaScript Object Notation (JSON) string into an object.\n     * @param text A valid JSON string.\n     * @param reviver A function that transforms the results. This function is called for each member of the object.\n     * If a member contains nested objects, the nested objects are transformed before the parent object is.\n     */\n    parse(text: string, reviver?: (this: any, key: string, value: any) => any): any;\n    /**\n     * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.\n     * @param value A JavaScript value, usually an object or array, to be converted.\n     * @param replacer A function that transforms the results.\n     * @param space Adds indentation, white space, and line break characters to the return-value JSON text to make it easier to read.\n     */\n    stringify(value: any, replacer?: (this: any, key: string, value: any) => any, space?: string | number): string;\n    /**\n     * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.\n     * @param value A JavaScript value, usually an object or array, to be converted.\n     * @param replacer An array of strings and numbers that acts as an approved list for selecting the object properties that will be stringified.\n     * @param space Adds indentation, white space, and line break characters to the return-value JSON text to make it easier to read.\n     */\n    stringify(value: any, replacer?: (number | string)[] | null, space?: string | number): string;\n}\n\n/**\n * An intrinsic object that provides functions to convert JavaScript values to and from the JavaScript Object Notation (JSON) format.\n */\ndeclare var JSON: JSON;\n\n\n/////////////////////////////\n/// ECMAScript Array API (specially handled by compiler)\n/////////////////////////////\n\ninterface ReadonlyArray<T> {\n    /**\n     * Gets the length of the array. This is a number one higher than the highest element defined in an array.\n     */\n    readonly length: number;\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n    /**\n     * Returns a string representation of an array. The elements are converted to string using their toLocaleString methods.\n     */\n    toLocaleString(): string;\n    /**\n     * Combines two or more arrays.\n     * @param items Additional items to add to the end of array1.\n     */\n    concat(...items: ConcatArray<T>[]): T[];\n    /**\n     * Combines two or more arrays.\n     * @param items Additional items to add to the end of array1.\n     */\n    concat(...items: (T | ConcatArray<T>)[]): T[];\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): T[];\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at index 0.\n     */\n    indexOf(searchElement: T, fromIndex?: number): number;\n    /**\n     * Returns the index of the last occurrence of a specified value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at the last index in the array.\n     */\n    lastIndexOf(searchElement: T, fromIndex?: number): number;\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every<S extends T>(predicate: (value: T, index: number, array: readonly T[]) => value is S, thisArg?: any): this is readonly S[];\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: T, index: number, array: readonly T[]) => unknown, thisArg?: any): boolean;\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: T, index: number, array: readonly T[]) => unknown, thisArg?: any): boolean;\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: T, index: number, array: readonly T[]) => void, thisArg?: any): void;\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\n     */\n    map<U>(callbackfn: (value: T, index: number, array: readonly T[]) => U, thisArg?: any): U[];\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function. If thisArg is omitted, undefined is used as the this value.\n     */\n    filter<S extends T>(predicate: (value: T, index: number, array: readonly T[]) => value is S, thisArg?: any): S[];\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function. If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: T, index: number, array: readonly T[]) => unknown, thisArg?: any): T[];\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: readonly T[]) => T): T;\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: readonly T[]) => T, initialValue: T): T;\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: readonly T[]) => U, initialValue: U): U;\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: readonly T[]) => T): T;\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: readonly T[]) => T, initialValue: T): T;\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: readonly T[]) => U, initialValue: U): U;\n\n    readonly [n: number]: T;\n}\n\ninterface ConcatArray<T> {\n    readonly length: number;\n    readonly [n: number]: T;\n    join(separator?: string): string;\n    slice(start?: number, end?: number): T[];\n}\n\ninterface Array<T> {\n    /**\n     * Gets or sets the length of the array. This is a number one higher than the highest index in the array.\n     */\n    length: number;\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n    /**\n     * Returns a string representation of an array. The elements are converted to string using their toLocaleString methods.\n     */\n    toLocaleString(): string;\n    /**\n     * Removes the last element from an array and returns it.\n     * If the array is empty, undefined is returned and the array is not modified.\n     */\n    pop(): T | undefined;\n    /**\n     * Appends new elements to the end of an array, and returns the new length of the array.\n     * @param items New elements to add to the array.\n     */\n    push(...items: T[]): number;\n    /**\n     * Combines two or more arrays.\n     * This method returns a new array without modifying any existing arrays.\n     * @param items Additional arrays and/or items to add to the end of the array.\n     */\n    concat(...items: ConcatArray<T>[]): T[];\n    /**\n     * Combines two or more arrays.\n     * This method returns a new array without modifying any existing arrays.\n     * @param items Additional arrays and/or items to add to the end of the array.\n     */\n    concat(...items: (T | ConcatArray<T>)[]): T[];\n    /**\n     * Adds all the elements of an array into a string, separated by the specified separator string.\n     * @param separator A string used to separate one element of the array from the next in the resulting string. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n    /**\n     * Reverses the elements in an array in place.\n     * This method mutates the array and returns a reference to the same array.\n     */\n    reverse(): T[];\n    /**\n     * Removes the first element from an array and returns it.\n     * If the array is empty, undefined is returned and the array is not modified.\n     */\n    shift(): T | undefined;\n    /**\n     * Returns a copy of a section of an array.\n     * For both start and end, a negative index can be used to indicate an offset from the end of the array.\n     * For example, -2 refers to the second to last element of the array.\n     * @param start The beginning index of the specified portion of the array.\n     * If start is undefined, then the slice begins at index 0.\n     * @param end The end index of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     * If end is undefined, then the slice extends to the end of the array.\n     */\n    slice(start?: number, end?: number): T[];\n    /**\n     * Sorts an array in place.\n     * This method mutates the array and returns a reference to the same array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: T, b: T) => number): this;\n    /**\n     * Removes elements from an array and, if necessary, inserts new elements in their place, returning the deleted elements.\n     * @param start The zero-based location in the array from which to start removing elements.\n     * @param deleteCount The number of elements to remove.\n     * @returns An array containing the elements that were deleted.\n     */\n    splice(start: number, deleteCount?: number): T[];\n    /**\n     * Removes elements from an array and, if necessary, inserts new elements in their place, returning the deleted elements.\n     * @param start The zero-based location in the array from which to start removing elements.\n     * @param deleteCount The number of elements to remove.\n     * @param items Elements to insert into the array in place of the deleted elements.\n     * @returns An array containing the elements that were deleted.\n     */\n    splice(start: number, deleteCount: number, ...items: T[]): T[];\n    /**\n     * Inserts new elements at the start of an array, and returns the new length of the array.\n     * @param items Elements to insert at the start of the array.\n     */\n    unshift(...items: T[]): number;\n    /**\n     * Returns the index of the first occurrence of a value in an array, or -1 if it is not present.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the search starts at index 0.\n     */\n    indexOf(searchElement: T, fromIndex?: number): number;\n    /**\n     * Returns the index of the last occurrence of a specified value in an array, or -1 if it is not present.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin searching backward. If fromIndex is omitted, the search starts at the last index in the array.\n     */\n    lastIndexOf(searchElement: T, fromIndex?: number): number;\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every<S extends T>(predicate: (value: T, index: number, array: T[]) => value is S, thisArg?: any): this is S[];\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: T, index: number, array: T[]) => unknown, thisArg?: any): boolean;\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: T, index: number, array: T[]) => unknown, thisArg?: any): boolean;\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: T, index: number, array: T[]) => void, thisArg?: any): void;\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.\n     */\n    map<U>(callbackfn: (value: T, index: number, array: T[]) => U, thisArg?: any): U[];\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function. If thisArg is omitted, undefined is used as the this value.\n     */\n    filter<S extends T>(predicate: (value: T, index: number, array: T[]) => value is S, thisArg?: any): S[];\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function. If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: T, index: number, array: T[]) => unknown, thisArg?: any): T[];\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T): T;\n    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T, initialValue: T): T;\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: T[]) => U, initialValue: U): U;\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T): T;\n    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T, initialValue: T): T;\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order. The return value of the callback function is the accumulated result, and is provided as an argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start the accumulation. The first call to the callbackfn function provides this value as an argument instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, array: T[]) => U, initialValue: U): U;\n\n    [n: number]: T;\n}\n\ninterface ArrayConstructor {\n    new(arrayLength?: number): any[];\n    new <T>(arrayLength: number): T[];\n    new <T>(...items: T[]): T[];\n    (arrayLength?: number): any[];\n    <T>(arrayLength: number): T[];\n    <T>(...items: T[]): T[];\n    isArray(arg: any): arg is any[];\n    readonly prototype: any[];\n}\n\ndeclare var Array: ArrayConstructor;\n\ninterface TypedPropertyDescriptor<T> {\n    enumerable?: boolean;\n    configurable?: boolean;\n    writable?: boolean;\n    value?: T;\n    get?: () => T;\n    set?: (value: T) => void;\n}\n\ndeclare type ClassDecorator = <TFunction extends Function>(target: TFunction) => TFunction | void;\ndeclare type PropertyDecorator = (target: Object, propertyKey: string | symbol) => void;\ndeclare type MethodDecorator = <T>(target: Object, propertyKey: string | symbol, descriptor: TypedPropertyDescriptor<T>) => TypedPropertyDescriptor<T> | void;\ndeclare type ParameterDecorator = (target: Object, propertyKey: string | symbol, parameterIndex: number) => void;\n\ndeclare type PromiseConstructorLike = new <T>(executor: (resolve: (value: T | PromiseLike<T>) => void, reject: (reason?: any) => void) => void) => PromiseLike<T>;\n\ninterface PromiseLike<T> {\n    /**\n     * Attaches callbacks for the resolution and/or rejection of the Promise.\n     * @param onfulfilled The callback to execute when the Promise is resolved.\n     * @param onrejected The callback to execute when the Promise is rejected.\n     * @returns A Promise for the completion of which ever callback is executed.\n     */\n    then<TResult1 = T, TResult2 = never>(onfulfilled?: ((value: T) => TResult1 | PromiseLike<TResult1>) | undefined | null, onrejected?: ((reason: any) => TResult2 | PromiseLike<TResult2>) | undefined | null): PromiseLike<TResult1 | TResult2>;\n}\n\n/**\n * Represents the completion of an asynchronous operation\n */\ninterface Promise<T> {\n    /**\n     * Attaches callbacks for the resolution and/or rejection of the Promise.\n     * @param onfulfilled The callback to execute when the Promise is resolved.\n     * @param onrejected The callback to execute when the Promise is rejected.\n     * @returns A Promise for the completion of which ever callback is executed.\n     */\n    then<TResult1 = T, TResult2 = never>(onfulfilled?: ((value: T) => TResult1 | PromiseLike<TResult1>) | undefined | null, onrejected?: ((reason: any) => TResult2 | PromiseLike<TResult2>) | undefined | null): Promise<TResult1 | TResult2>;\n\n    /**\n     * Attaches a callback for only the rejection of the Promise.\n     * @param onrejected The callback to execute when the Promise is rejected.\n     * @returns A Promise for the completion of the callback.\n     */\n    catch<TResult = never>(onrejected?: ((reason: any) => TResult | PromiseLike<TResult>) | undefined | null): Promise<T | TResult>;\n}\n\ninterface ArrayLike<T> {\n    readonly length: number;\n    readonly [n: number]: T;\n}\n\n/**\n * Make all properties in T optional\n */\ntype Partial<T> = {\n    [P in keyof T]?: T[P];\n};\n\n/**\n * Make all properties in T required\n */\ntype Required<T> = {\n    [P in keyof T]-?: T[P];\n};\n\n/**\n * Make all properties in T readonly\n */\ntype Readonly<T> = {\n    readonly [P in keyof T]: T[P];\n};\n\n/**\n * From T, pick a set of properties whose keys are in the union K\n */\ntype Pick<T, K extends keyof T> = {\n    [P in K]: T[P];\n};\n\n/**\n * Construct a type with a set of properties K of type T\n */\ntype Record<K extends keyof any, T> = {\n    [P in K]: T;\n};\n\n/**\n * Exclude from T those types that are assignable to U\n */\ntype Exclude<T, U> = T extends U ? never : T;\n\n/**\n * Extract from T those types that are assignable to U\n */\ntype Extract<T, U> = T extends U ? T : never;\n\n/**\n * Construct a type with the properties of T except for those in type K.\n */\ntype Omit<T, K extends keyof any> = Pick<T, Exclude<keyof T, K>>;\n\n/**\n * Exclude null and undefined from T\n */\ntype NonNullable<T> = T extends null | undefined ? never : T;\n\n/**\n * Obtain the parameters of a function type in a tuple\n */\ntype Parameters<T extends (...args: any) => any> = T extends (...args: infer P) => any ? P : never;\n\n/**\n * Obtain the parameters of a constructor function type in a tuple\n */\ntype ConstructorParameters<T extends abstract new (...args: any) => any> = T extends abstract new (...args: infer P) => any ? P : never;\n\n/**\n * Obtain the return type of a function type\n */\ntype ReturnType<T extends (...args: any) => any> = T extends (...args: any) => infer R ? R : any;\n\n/**\n * Obtain the return type of a constructor function type\n */\ntype InstanceType<T extends abstract new (...args: any) => any> = T extends abstract new (...args: any) => infer R ? R : any;\n\n/**\n * Convert string literal type to uppercase\n */\ntype Uppercase<S extends string> = intrinsic;\n\n/**\n * Convert string literal type to lowercase\n */\ntype Lowercase<S extends string> = intrinsic;\n\n/**\n * Convert first character of string literal type to uppercase\n */\ntype Capitalize<S extends string> = intrinsic;\n\n/**\n * Convert first character of string literal type to lowercase\n */\ntype Uncapitalize<S extends string> = intrinsic;\n\n/**\n * Marker for contextual 'this' type\n */\ninterface ThisType<T> { }\n\n/**\n * Represents a raw buffer of binary data, which is used to store data for the\n * different typed arrays. ArrayBuffers cannot be read from or written to directly,\n * but can be passed to a typed array or DataView Object to interpret the raw\n * buffer as needed.\n */\ninterface ArrayBuffer {\n    /**\n     * Read-only. The length of the ArrayBuffer (in bytes).\n     */\n    readonly byteLength: number;\n\n    /**\n     * Returns a section of an ArrayBuffer.\n     */\n    slice(begin: number, end?: number): ArrayBuffer;\n}\n\n/**\n * Allowed ArrayBuffer types for the buffer of an ArrayBufferView and related Typed Arrays.\n */\ninterface ArrayBufferTypes {\n    ArrayBuffer: ArrayBuffer;\n}\ntype ArrayBufferLike = ArrayBufferTypes[keyof ArrayBufferTypes];\n\ninterface ArrayBufferConstructor {\n    readonly prototype: ArrayBuffer;\n    new(byteLength: number): ArrayBuffer;\n    isView(arg: any): arg is ArrayBufferView;\n}\ndeclare var ArrayBuffer: ArrayBufferConstructor;\n\ninterface ArrayBufferView {\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    byteOffset: number;\n}\n\ninterface DataView {\n    readonly buffer: ArrayBuffer;\n    readonly byteLength: number;\n    readonly byteOffset: number;\n    /**\n     * Gets the Float32 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getFloat32(byteOffset: number, littleEndian?: boolean): number;\n\n    /**\n     * Gets the Float64 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getFloat64(byteOffset: number, littleEndian?: boolean): number;\n\n    /**\n     * Gets the Int8 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getInt8(byteOffset: number): number;\n\n    /**\n     * Gets the Int16 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getInt16(byteOffset: number, littleEndian?: boolean): number;\n    /**\n     * Gets the Int32 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getInt32(byteOffset: number, littleEndian?: boolean): number;\n\n    /**\n     * Gets the Uint8 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getUint8(byteOffset: number): number;\n\n    /**\n     * Gets the Uint16 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getUint16(byteOffset: number, littleEndian?: boolean): number;\n\n    /**\n     * Gets the Uint32 value at the specified byte offset from the start of the view. There is\n     * no alignment constraint; multi-byte values may be fetched from any offset.\n     * @param byteOffset The place in the buffer at which the value should be retrieved.\n     */\n    getUint32(byteOffset: number, littleEndian?: boolean): number;\n\n    /**\n     * Stores an Float32 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     * @param littleEndian If false or undefined, a big-endian value should be written,\n     * otherwise a little-endian value should be written.\n     */\n    setFloat32(byteOffset: number, value: number, littleEndian?: boolean): void;\n\n    /**\n     * Stores an Float64 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     * @param littleEndian If false or undefined, a big-endian value should be written,\n     * otherwise a little-endian value should be written.\n     */\n    setFloat64(byteOffset: number, value: number, littleEndian?: boolean): void;\n\n    /**\n     * Stores an Int8 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     */\n    setInt8(byteOffset: number, value: number): void;\n\n    /**\n     * Stores an Int16 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     * @param littleEndian If false or undefined, a big-endian value should be written,\n     * otherwise a little-endian value should be written.\n     */\n    setInt16(byteOffset: number, value: number, littleEndian?: boolean): void;\n\n    /**\n     * Stores an Int32 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     * @param littleEndian If false or undefined, a big-endian value should be written,\n     * otherwise a little-endian value should be written.\n     */\n    setInt32(byteOffset: number, value: number, littleEndian?: boolean): void;\n\n    /**\n     * Stores an Uint8 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     */\n    setUint8(byteOffset: number, value: number): void;\n\n    /**\n     * Stores an Uint16 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     * @param littleEndian If false or undefined, a big-endian value should be written,\n     * otherwise a little-endian value should be written.\n     */\n    setUint16(byteOffset: number, value: number, littleEndian?: boolean): void;\n\n    /**\n     * Stores an Uint32 value at the specified byte offset from the start of the view.\n     * @param byteOffset The place in the buffer at which the value should be set.\n     * @param value The value to set.\n     * @param littleEndian If false or undefined, a big-endian value should be written,\n     * otherwise a little-endian value should be written.\n     */\n    setUint32(byteOffset: number, value: number, littleEndian?: boolean): void;\n}\n\ninterface DataViewConstructor {\n    readonly prototype: DataView;\n    new(buffer: ArrayBufferLike, byteOffset?: number, byteLength?: number): DataView;\n}\ndeclare var DataView: DataViewConstructor;\n\n/**\n * A typed array of 8-bit integer values. The contents are initialized to 0. If the requested\n * number of bytes could not be allocated an exception is raised.\n */\ninterface Int8Array {\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    readonly buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    readonly byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: number, index: number, array: Int8Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: number, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: number, index: number, array: Int8Array) => any, thisArg?: any): Int8Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: number, index: number, obj: Int8Array) => boolean, thisArg?: any): number | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: number, index: number, obj: Int8Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: number, index: number, array: Int8Array) => void, thisArg?: any): void;\n\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     *  search starts at index 0.\n     */\n    indexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * The length of the array.\n     */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: number, index: number, array: Int8Array) => number, thisArg?: any): Int8Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number): number;\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int8Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number): number;\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int8Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int8Array) => U, initialValue: U): U;\n\n    /**\n     * Reverses the elements in an Array.\n     */\n    reverse(): Int8Array;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<number>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): Int8Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: number, index: number, array: Int8Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Sorts an array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: number, b: number) => number): this;\n\n    /**\n     * Gets a new Int8Array view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): Int8Array;\n\n    /**\n     * Converts a number to a string by using the current locale.\n     */\n    toLocaleString(): string;\n\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Int8Array;\n\n    [index: number]: number;\n}\ninterface Int8ArrayConstructor {\n    readonly prototype: Int8Array;\n    new(length: number): Int8Array;\n    new(array: ArrayLike<number> | ArrayBufferLike): Int8Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): Int8Array;\n\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: number[]): Int8Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     */\n    from(arrayLike: ArrayLike<number>): Int8Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => number, thisArg?: any): Int8Array;\n\n\n}\ndeclare var Int8Array: Int8ArrayConstructor;\n\n/**\n * A typed array of 8-bit unsigned integer values. The contents are initialized to 0. If the\n * requested number of bytes could not be allocated an exception is raised.\n */\ninterface Uint8Array {\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    readonly buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    readonly byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: number, index: number, array: Uint8Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: number, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: number, index: number, array: Uint8Array) => any, thisArg?: any): Uint8Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: number, index: number, obj: Uint8Array) => boolean, thisArg?: any): number | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: number, index: number, obj: Uint8Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: number, index: number, array: Uint8Array) => void, thisArg?: any): void;\n\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     *  search starts at index 0.\n     */\n    indexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * The length of the array.\n     */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: number, index: number, array: Uint8Array) => number, thisArg?: any): Uint8Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number): number;\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number): number;\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8Array) => U, initialValue: U): U;\n\n    /**\n     * Reverses the elements in an Array.\n     */\n    reverse(): Uint8Array;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<number>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): Uint8Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: number, index: number, array: Uint8Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Sorts an array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: number, b: number) => number): this;\n\n    /**\n     * Gets a new Uint8Array view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): Uint8Array;\n\n    /**\n     * Converts a number to a string by using the current locale.\n     */\n    toLocaleString(): string;\n\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Uint8Array;\n\n    [index: number]: number;\n}\n\ninterface Uint8ArrayConstructor {\n    readonly prototype: Uint8Array;\n    new(length: number): Uint8Array;\n    new(array: ArrayLike<number> | ArrayBufferLike): Uint8Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): Uint8Array;\n\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: number[]): Uint8Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     */\n    from(arrayLike: ArrayLike<number>): Uint8Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => number, thisArg?: any): Uint8Array;\n\n}\ndeclare var Uint8Array: Uint8ArrayConstructor;\n\n/**\n * A typed array of 8-bit unsigned integer (clamped) values. The contents are initialized to 0.\n * If the requested number of bytes could not be allocated an exception is raised.\n */\ninterface Uint8ClampedArray {\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    readonly buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    readonly byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: number, index: number, array: Uint8ClampedArray) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: number, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: number, index: number, array: Uint8ClampedArray) => any, thisArg?: any): Uint8ClampedArray;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: number, index: number, obj: Uint8ClampedArray) => boolean, thisArg?: any): number | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: number, index: number, obj: Uint8ClampedArray) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => void, thisArg?: any): void;\n\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     *  search starts at index 0.\n     */\n    indexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * The length of the array.\n     */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: number, index: number, array: Uint8ClampedArray) => number, thisArg?: any): Uint8ClampedArray;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number): number;\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number): number;\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint8ClampedArray) => U, initialValue: U): U;\n\n    /**\n     * Reverses the elements in an Array.\n     */\n    reverse(): Uint8ClampedArray;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<number>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): Uint8ClampedArray;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: number, index: number, array: Uint8ClampedArray) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Sorts an array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: number, b: number) => number): this;\n\n    /**\n     * Gets a new Uint8ClampedArray view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): Uint8ClampedArray;\n\n    /**\n     * Converts a number to a string by using the current locale.\n     */\n    toLocaleString(): string;\n\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Uint8ClampedArray;\n\n    [index: number]: number;\n}\n\ninterface Uint8ClampedArrayConstructor {\n    readonly prototype: Uint8ClampedArray;\n    new(length: number): Uint8ClampedArray;\n    new(array: ArrayLike<number> | ArrayBufferLike): Uint8ClampedArray;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): Uint8ClampedArray;\n\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: number[]): Uint8ClampedArray;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     */\n    from(arrayLike: ArrayLike<number>): Uint8ClampedArray;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => number, thisArg?: any): Uint8ClampedArray;\n}\ndeclare var Uint8ClampedArray: Uint8ClampedArrayConstructor;\n\n/**\n * A typed array of 16-bit signed integer values. The contents are initialized to 0. If the\n * requested number of bytes could not be allocated an exception is raised.\n */\ninterface Int16Array {\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    readonly buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    readonly byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: number, index: number, array: Int16Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: number, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: number, index: number, array: Int16Array) => any, thisArg?: any): Int16Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: number, index: number, obj: Int16Array) => boolean, thisArg?: any): number | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: number, index: number, obj: Int16Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: number, index: number, array: Int16Array) => void, thisArg?: any): void;\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     *  search starts at index 0.\n     */\n    indexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * The length of the array.\n     */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: number, index: number, array: Int16Array) => number, thisArg?: any): Int16Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number): number;\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int16Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number): number;\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int16Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int16Array) => U, initialValue: U): U;\n\n    /**\n     * Reverses the elements in an Array.\n     */\n    reverse(): Int16Array;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<number>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): Int16Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: number, index: number, array: Int16Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Sorts an array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: number, b: number) => number): this;\n\n    /**\n     * Gets a new Int16Array view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): Int16Array;\n\n    /**\n     * Converts a number to a string by using the current locale.\n     */\n    toLocaleString(): string;\n\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Int16Array;\n\n    [index: number]: number;\n}\n\ninterface Int16ArrayConstructor {\n    readonly prototype: Int16Array;\n    new(length: number): Int16Array;\n    new(array: ArrayLike<number> | ArrayBufferLike): Int16Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): Int16Array;\n\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: number[]): Int16Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     */\n    from(arrayLike: ArrayLike<number>): Int16Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => number, thisArg?: any): Int16Array;\n\n\n}\ndeclare var Int16Array: Int16ArrayConstructor;\n\n/**\n * A typed array of 16-bit unsigned integer values. The contents are initialized to 0. If the\n * requested number of bytes could not be allocated an exception is raised.\n */\ninterface Uint16Array {\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    readonly buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    readonly byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: number, index: number, array: Uint16Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: number, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: number, index: number, array: Uint16Array) => any, thisArg?: any): Uint16Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: number, index: number, obj: Uint16Array) => boolean, thisArg?: any): number | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: number, index: number, obj: Uint16Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: number, index: number, array: Uint16Array) => void, thisArg?: any): void;\n\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     *  search starts at index 0.\n     */\n    indexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * The length of the array.\n     */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: number, index: number, array: Uint16Array) => number, thisArg?: any): Uint16Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number): number;\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint16Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number): number;\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint16Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint16Array) => U, initialValue: U): U;\n\n    /**\n     * Reverses the elements in an Array.\n     */\n    reverse(): Uint16Array;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<number>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): Uint16Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: number, index: number, array: Uint16Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Sorts an array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: number, b: number) => number): this;\n\n    /**\n     * Gets a new Uint16Array view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): Uint16Array;\n\n    /**\n     * Converts a number to a string by using the current locale.\n     */\n    toLocaleString(): string;\n\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Uint16Array;\n\n    [index: number]: number;\n}\n\ninterface Uint16ArrayConstructor {\n    readonly prototype: Uint16Array;\n    new(length: number): Uint16Array;\n    new(array: ArrayLike<number> | ArrayBufferLike): Uint16Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): Uint16Array;\n\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: number[]): Uint16Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     */\n    from(arrayLike: ArrayLike<number>): Uint16Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => number, thisArg?: any): Uint16Array;\n\n\n}\ndeclare var Uint16Array: Uint16ArrayConstructor;\n/**\n * A typed array of 32-bit signed integer values. The contents are initialized to 0. If the\n * requested number of bytes could not be allocated an exception is raised.\n */\ninterface Int32Array {\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    readonly buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    readonly byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: number, index: number, array: Int32Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: number, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: number, index: number, array: Int32Array) => any, thisArg?: any): Int32Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: number, index: number, obj: Int32Array) => boolean, thisArg?: any): number | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: number, index: number, obj: Int32Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: number, index: number, array: Int32Array) => void, thisArg?: any): void;\n\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     *  search starts at index 0.\n     */\n    indexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * The length of the array.\n     */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: number, index: number, array: Int32Array) => number, thisArg?: any): Int32Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number): number;\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int32Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number): number;\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Int32Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Int32Array) => U, initialValue: U): U;\n\n    /**\n     * Reverses the elements in an Array.\n     */\n    reverse(): Int32Array;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<number>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): Int32Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: number, index: number, array: Int32Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Sorts an array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: number, b: number) => number): this;\n\n    /**\n     * Gets a new Int32Array view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): Int32Array;\n\n    /**\n     * Converts a number to a string by using the current locale.\n     */\n    toLocaleString(): string;\n\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Int32Array;\n\n    [index: number]: number;\n}\n\ninterface Int32ArrayConstructor {\n    readonly prototype: Int32Array;\n    new(length: number): Int32Array;\n    new(array: ArrayLike<number> | ArrayBufferLike): Int32Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): Int32Array;\n\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: number[]): Int32Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     */\n    from(arrayLike: ArrayLike<number>): Int32Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => number, thisArg?: any): Int32Array;\n\n}\ndeclare var Int32Array: Int32ArrayConstructor;\n\n/**\n * A typed array of 32-bit unsigned integer values. The contents are initialized to 0. If the\n * requested number of bytes could not be allocated an exception is raised.\n */\ninterface Uint32Array {\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    readonly buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    readonly byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: number, index: number, array: Uint32Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: number, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: number, index: number, array: Uint32Array) => any, thisArg?: any): Uint32Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: number, index: number, obj: Uint32Array) => boolean, thisArg?: any): number | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: number, index: number, obj: Uint32Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: number, index: number, array: Uint32Array) => void, thisArg?: any): void;\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     *  search starts at index 0.\n     */\n    indexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * The length of the array.\n     */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: number, index: number, array: Uint32Array) => number, thisArg?: any): Uint32Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number): number;\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint32Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number): number;\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Uint32Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Uint32Array) => U, initialValue: U): U;\n\n    /**\n     * Reverses the elements in an Array.\n     */\n    reverse(): Uint32Array;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<number>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): Uint32Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: number, index: number, array: Uint32Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Sorts an array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: number, b: number) => number): this;\n\n    /**\n     * Gets a new Uint32Array view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): Uint32Array;\n\n    /**\n     * Converts a number to a string by using the current locale.\n     */\n    toLocaleString(): string;\n\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Uint32Array;\n\n    [index: number]: number;\n}\n\ninterface Uint32ArrayConstructor {\n    readonly prototype: Uint32Array;\n    new(length: number): Uint32Array;\n    new(array: ArrayLike<number> | ArrayBufferLike): Uint32Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): Uint32Array;\n\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: number[]): Uint32Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     */\n    from(arrayLike: ArrayLike<number>): Uint32Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => number, thisArg?: any): Uint32Array;\n\n}\ndeclare var Uint32Array: Uint32ArrayConstructor;\n\n/**\n * A typed array of 32-bit float values. The contents are initialized to 0. If the requested number\n * of bytes could not be allocated an exception is raised.\n */\ninterface Float32Array {\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    readonly buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    readonly byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: number, index: number, array: Float32Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: number, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: number, index: number, array: Float32Array) => any, thisArg?: any): Float32Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: number, index: number, obj: Float32Array) => boolean, thisArg?: any): number | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: number, index: number, obj: Float32Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: number, index: number, array: Float32Array) => void, thisArg?: any): void;\n\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     *  search starts at index 0.\n     */\n    indexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * The length of the array.\n     */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: number, index: number, array: Float32Array) => number, thisArg?: any): Float32Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number): number;\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float32Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number): number;\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float32Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float32Array) => U, initialValue: U): U;\n\n    /**\n     * Reverses the elements in an Array.\n     */\n    reverse(): Float32Array;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<number>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): Float32Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: number, index: number, array: Float32Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Sorts an array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: number, b: number) => number): this;\n\n    /**\n     * Gets a new Float32Array view of the ArrayBuffer store for this array, referencing the elements\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): Float32Array;\n\n    /**\n     * Converts a number to a string by using the current locale.\n     */\n    toLocaleString(): string;\n\n    /**\n     * Returns a string representation of an array.\n     */\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Float32Array;\n\n    [index: number]: number;\n}\n\ninterface Float32ArrayConstructor {\n    readonly prototype: Float32Array;\n    new(length: number): Float32Array;\n    new(array: ArrayLike<number> | ArrayBufferLike): Float32Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): Float32Array;\n\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: number[]): Float32Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     */\n    from(arrayLike: ArrayLike<number>): Float32Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => number, thisArg?: any): Float32Array;\n\n\n}\ndeclare var Float32Array: Float32ArrayConstructor;\n\n/**\n * A typed array of 64-bit float values. The contents are initialized to 0. If the requested\n * number of bytes could not be allocated an exception is raised.\n */\ninterface Float64Array {\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * The ArrayBuffer instance referenced by the array.\n     */\n    readonly buffer: ArrayBufferLike;\n\n    /**\n     * The length in bytes of the array.\n     */\n    readonly byteLength: number;\n\n    /**\n     * The offset in bytes of the array.\n     */\n    readonly byteOffset: number;\n\n    /**\n     * Returns the this object after copying a section of the array identified by start and end\n     * to the same array starting at position target\n     * @param target If target is negative, it is treated as length+target where length is the\n     * length of the array.\n     * @param start If start is negative, it is treated as length+start. If end is negative, it\n     * is treated as length+end.\n     * @param end If not specified, length of the this object is used as its default value.\n     */\n    copyWithin(target: number, start: number, end?: number): this;\n\n    /**\n     * Determines whether all the members of an array satisfy the specified test.\n     * @param predicate A function that accepts up to three arguments. The every method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value false, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    every(predicate: (value: number, index: number, array: Float64Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Changes all array elements from `start` to `end` index to a static `value` and returns the modified array\n     * @param value value to fill array section with\n     * @param start index to start filling the array at. If start is negative, it is treated as\n     * length+start where length is the length of the array.\n     * @param end index to stop filling the array at. If end is negative, it is treated as\n     * length+end.\n     */\n    fill(value: number, start?: number, end?: number): this;\n\n    /**\n     * Returns the elements of an array that meet the condition specified in a callback function.\n     * @param predicate A function that accepts up to three arguments. The filter method calls\n     * the predicate function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    filter(predicate: (value: number, index: number, array: Float64Array) => any, thisArg?: any): Float64Array;\n\n    /**\n     * Returns the value of the first element in the array where predicate is true, and undefined\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found, find\n     * immediately returns that element value. Otherwise, find returns undefined.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    find(predicate: (value: number, index: number, obj: Float64Array) => boolean, thisArg?: any): number | undefined;\n\n    /**\n     * Returns the index of the first element in the array where predicate is true, and -1\n     * otherwise.\n     * @param predicate find calls predicate once for each element of the array, in ascending\n     * order, until it finds one where predicate returns true. If such an element is found,\n     * findIndex immediately returns that element index. Otherwise, findIndex returns -1.\n     * @param thisArg If provided, it will be used as the this value for each invocation of\n     * predicate. If it is not provided, undefined is used instead.\n     */\n    findIndex(predicate: (value: number, index: number, obj: Float64Array) => boolean, thisArg?: any): number;\n\n    /**\n     * Performs the specified action for each element in an array.\n     * @param callbackfn  A function that accepts up to three arguments. forEach calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg  An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    forEach(callbackfn: (value: number, index: number, array: Float64Array) => void, thisArg?: any): void;\n\n    /**\n     * Returns the index of the first occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     *  search starts at index 0.\n     */\n    indexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * Adds all the elements of an array separated by the specified separator string.\n     * @param separator A string used to separate one element of an array from the next in the\n     * resulting String. If omitted, the array elements are separated with a comma.\n     */\n    join(separator?: string): string;\n\n    /**\n     * Returns the index of the last occurrence of a value in an array.\n     * @param searchElement The value to locate in the array.\n     * @param fromIndex The array index at which to begin the search. If fromIndex is omitted, the\n     * search starts at index 0.\n     */\n    lastIndexOf(searchElement: number, fromIndex?: number): number;\n\n    /**\n     * The length of the array.\n     */\n    readonly length: number;\n\n    /**\n     * Calls a defined callback function on each element of an array, and returns an array that\n     * contains the results.\n     * @param callbackfn A function that accepts up to three arguments. The map method calls the\n     * callbackfn function one time for each element in the array.\n     * @param thisArg An object to which the this keyword can refer in the callbackfn function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    map(callbackfn: (value: number, index: number, array: Float64Array) => number, thisArg?: any): Float64Array;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number): number;\n    reduce(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array. The return value of\n     * the callback function is the accumulated result, and is provided as an argument in the next\n     * call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduce method calls the\n     * callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduce<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float64Array) => U, initialValue: U): U;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an\n     * argument instead of an array value.\n     */\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number): number;\n    reduceRight(callbackfn: (previousValue: number, currentValue: number, currentIndex: number, array: Float64Array) => number, initialValue: number): number;\n\n    /**\n     * Calls the specified callback function for all the elements in an array, in descending order.\n     * The return value of the callback function is the accumulated result, and is provided as an\n     * argument in the next call to the callback function.\n     * @param callbackfn A function that accepts up to four arguments. The reduceRight method calls\n     * the callbackfn function one time for each element in the array.\n     * @param initialValue If initialValue is specified, it is used as the initial value to start\n     * the accumulation. The first call to the callbackfn function provides this value as an argument\n     * instead of an array value.\n     */\n    reduceRight<U>(callbackfn: (previousValue: U, currentValue: number, currentIndex: number, array: Float64Array) => U, initialValue: U): U;\n\n    /**\n     * Reverses the elements in an Array.\n     */\n    reverse(): Float64Array;\n\n    /**\n     * Sets a value or an array of values.\n     * @param array A typed or untyped array of values to set.\n     * @param offset The index in the current array at which the values are to be written.\n     */\n    set(array: ArrayLike<number>, offset?: number): void;\n\n    /**\n     * Returns a section of an array.\n     * @param start The beginning of the specified portion of the array.\n     * @param end The end of the specified portion of the array. This is exclusive of the element at the index 'end'.\n     */\n    slice(start?: number, end?: number): Float64Array;\n\n    /**\n     * Determines whether the specified callback function returns true for any element of an array.\n     * @param predicate A function that accepts up to three arguments. The some method calls\n     * the predicate function for each element in the array until the predicate returns a value\n     * which is coercible to the Boolean value true, or until the end of the array.\n     * @param thisArg An object to which the this keyword can refer in the predicate function.\n     * If thisArg is omitted, undefined is used as the this value.\n     */\n    some(predicate: (value: number, index: number, array: Float64Array) => unknown, thisArg?: any): boolean;\n\n    /**\n     * Sorts an array.\n     * @param compareFn Function used to determine the order of the elements. It is expected to return\n     * a negative value if first argument is less than second argument, zero if they're equal and a positive\n     * value otherwise. If omitted, the elements are sorted in ascending, ASCII character order.\n     * ```ts\n     * [11,2,22,1].sort((a, b) => a - b)\n     * ```\n     */\n    sort(compareFn?: (a: number, b: number) => number): this;\n\n    /**\n     * at begin, inclusive, up to end, exclusive.\n     * @param begin The index of the beginning of the array.\n     * @param end The index of the end of the array.\n     */\n    subarray(begin?: number, end?: number): Float64Array;\n\n    toString(): string;\n\n    /** Returns the primitive value of the specified object. */\n    valueOf(): Float64Array;\n\n    [index: number]: number;\n}\n\ninterface Float64ArrayConstructor {\n    readonly prototype: Float64Array;\n    new(length: number): Float64Array;\n    new(array: ArrayLike<number> | ArrayBufferLike): Float64Array;\n    new(buffer: ArrayBufferLike, byteOffset?: number, length?: number): Float64Array;\n\n    /**\n     * The size in bytes of each element in the array.\n     */\n    readonly BYTES_PER_ELEMENT: number;\n\n    /**\n     * Returns a new array from a set of elements.\n     * @param items A set of elements to include in the new array object.\n     */\n    of(...items: number[]): Float64Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     */\n    from(arrayLike: ArrayLike<number>): Float64Array;\n\n    /**\n     * Creates an array from an array-like or iterable object.\n     * @param arrayLike An array-like or iterable object to convert to an array.\n     * @param mapfn A mapping function to call on every element of the array.\n     * @param thisArg Value of 'this' used to invoke the mapfn.\n     */\n    from<T>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => number, thisArg?: any): Float64Array;\n\n}\ndeclare var Float64Array: Float64ArrayConstructor;\n\n/////////////////////////////\n/// ECMAScript Internationalization API\n/////////////////////////////\n\ndeclare namespace Intl {\n    interface CollatorOptions {\n        usage?: string;\n        localeMatcher?: string;\n        numeric?: boolean;\n        caseFirst?: string;\n        sensitivity?: string;\n        ignorePunctuation?: boolean;\n    }\n\n    interface ResolvedCollatorOptions {\n        locale: string;\n        usage: string;\n        sensitivity: string;\n        ignorePunctuation: boolean;\n        collation: string;\n        caseFirst: string;\n        numeric: boolean;\n    }\n\n    interface Collator {\n        compare(x: string, y: string): number;\n        resolvedOptions(): ResolvedCollatorOptions;\n    }\n    var Collator: {\n        new(locales?: string | string[], options?: CollatorOptions): Collator;\n        (locales?: string | string[], options?: CollatorOptions): Collator;\n        supportedLocalesOf(locales: string | string[], options?: CollatorOptions): string[];\n    };\n\n    interface NumberFormatOptions {\n        localeMatcher?: string;\n        style?: string;\n        currency?: string;\n        currencyDisplay?: string;\n        currencySign?: string;\n        useGrouping?: boolean;\n        minimumIntegerDigits?: number;\n        minimumFractionDigits?: number;\n        maximumFractionDigits?: number;\n        minimumSignificantDigits?: number;\n        maximumSignificantDigits?: number;\n    }\n\n    interface ResolvedNumberFormatOptions {\n        locale: string;\n        numberingSystem: string;\n        style: string;\n        currency?: string;\n        currencyDisplay?: string;\n        minimumIntegerDigits: number;\n        minimumFractionDigits: number;\n        maximumFractionDigits: number;\n        minimumSignificantDigits?: number;\n        maximumSignificantDigits?: number;\n        useGrouping: boolean;\n    }\n\n    interface NumberFormat {\n        format(value: number): string;\n        resolvedOptions(): ResolvedNumberFormatOptions;\n    }\n    var NumberFormat: {\n        new(locales?: string | string[], options?: NumberFormatOptions): NumberFormat;\n        (locales?: string | string[], options?: NumberFormatOptions): NumberFormat;\n        supportedLocalesOf(locales: string | string[], options?: NumberFormatOptions): string[];\n    };\n\n    interface DateTimeFormatOptions {\n        localeMatcher?: \"best fit\" | \"lookup\";\n        weekday?: \"long\" | \"short\" | \"narrow\";\n        era?: \"long\" | \"short\" | \"narrow\";\n        year?: \"numeric\" | \"2-digit\";\n        month?: \"numeric\" | \"2-digit\" | \"long\" | \"short\" | \"narrow\";\n        day?: \"numeric\" | \"2-digit\";\n        hour?: \"numeric\" | \"2-digit\";\n        minute?: \"numeric\" | \"2-digit\";\n        second?: \"numeric\" | \"2-digit\";\n        timeZoneName?: \"long\" | \"short\";\n        formatMatcher?: \"best fit\" | \"basic\";\n        hour12?: boolean;\n        timeZone?: string;\n    }\n\n    interface ResolvedDateTimeFormatOptions {\n        locale: string;\n        calendar: string;\n        numberingSystem: string;\n        timeZone: string;\n        hour12?: boolean;\n        weekday?: string;\n        era?: string;\n        year?: string;\n        month?: string;\n        day?: string;\n        hour?: string;\n        minute?: string;\n        second?: string;\n        timeZoneName?: string;\n    }\n\n    interface DateTimeFormat {\n        format(date?: Date | number): string;\n        resolvedOptions(): ResolvedDateTimeFormatOptions;\n    }\n    var DateTimeFormat: {\n        new(locales?: string | string[], options?: DateTimeFormatOptions): DateTimeFormat;\n        (locales?: string | string[], options?: DateTimeFormatOptions): DateTimeFormat;\n        supportedLocalesOf(locales: string | string[], options?: DateTimeFormatOptions): string[];\n    };\n}\n\ninterface String {\n    /**\n     * Determines whether two strings are equivalent in the current or specified locale.\n     * @param that String to compare to target string\n     * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used. This parameter must conform to BCP 47 standards; see the Intl.Collator object for details.\n     * @param options An object that contains one or more properties that specify comparison options. see the Intl.Collator object for details.\n     */\n    localeCompare(that: string, locales?: string | string[], options?: Intl.CollatorOptions): number;\n}\n\ninterface Number {\n    /**\n     * Converts a number to a string by using the current or specified locale.\n     * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\n     * @param options An object that contains one or more properties that specify comparison options.\n     */\n    toLocaleString(locales?: string | string[], options?: Intl.NumberFormatOptions): string;\n}\n\ninterface Date {\n    /**\n     * Converts a date and time to a string by using the current or specified locale.\n     * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\n     * @param options An object that contains one or more properties that specify comparison options.\n     */\n    toLocaleString(locales?: string | string[], options?: Intl.DateTimeFormatOptions): string;\n    /**\n     * Converts a date to a string by using the current or specified locale.\n     * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\n     * @param options An object that contains one or more properties that specify comparison options.\n     */\n    toLocaleDateString(locales?: string | string[], options?: Intl.DateTimeFormatOptions): string;\n\n    /**\n     * Converts a time to a string by using the current or specified locale.\n     * @param locales A locale string or array of locale strings that contain one or more language or locale tags. If you include more than one locale string, list them in descending order of priority so that the first entry is the preferred locale. If you omit this parameter, the default locale of the JavaScript runtime is used.\n     * @param options An object that contains one or more properties that specify comparison options.\n     */\n    toLocaleTimeString(locales?: string | string[], options?: Intl.DateTimeFormatOptions): string;\n}\n","lib.es6.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2015" />\n/// <reference lib="dom" />\n/// <reference lib="dom.iterable" />\n/// <reference lib="webworker.importscripts" />\n/// <reference lib="scripthost" />\n',"lib.esnext.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="es2021" />\n/// <reference lib="esnext.intl" />\n',"lib.esnext.full.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/// <reference lib="esnext" />\n/// <reference lib="dom" />\n/// <reference lib="webworker.importscripts" />\n/// <reference lib="scripthost" />\n/// <reference lib="dom.iterable" />',"lib.esnext.intl.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ndeclare namespace Intl {\n    type NumberFormatPartTypes = "compact" | "currency" | "decimal" | "exponentInteger" | "exponentMinusSign" | "exponentSeparator" | "fraction" | "group" | "infinity" | "integer" | "literal" | "minusSign" | "nan" | "plusSign" | "percentSign" | "unit" | "unknown";\n\n    interface NumberFormatPart {\n        type: NumberFormatPartTypes;\n        value: string;\n    }\n\n    interface NumberFormat {\n        formatToParts(number?: number | bigint): NumberFormatPart[];\n    }\n}\n',"lib.esnext.promise.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface AggregateError extends Error {\n    errors: any[]\n}\n\ninterface AggregateErrorConstructor {\n    new(errors: Iterable<any>, message?: string): AggregateError;\n    (errors: Iterable<any>, message?: string): AggregateError;\n    readonly prototype: AggregateError;\n}\n\ndeclare var AggregateError: AggregateErrorConstructor;\n\n/**\n * Represents the completion of an asynchronous operation\n */\ninterface PromiseConstructor {\n    /**\n     * The any function returns a promise that is fulfilled by the first given promise to be fulfilled, or rejected with an AggregateError containing an array of rejection reasons if all of the given promises are rejected. It resolves all elements of the passed iterable to promises as it runs this algorithm.\n     * @param values An array or iterable of Promises.\n     * @returns A new Promise.\n     */\n    any<T>(values: (T | PromiseLike<T>)[] | Iterable<T | PromiseLike<T>>): Promise<T>\n}\n',"lib.esnext.string.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface String {\n    /**\n     * Replace all instances of a substring in a string, using a regular expression or search string.\n     * @param searchValue A string to search for.\n     * @param replaceValue A string containing the text to replace for every successful match of searchValue in this string.\n     */\n    replaceAll(searchValue: string | RegExp, replaceValue: string): string;\n\n    /**\n     * Replace all instances of a substring in a string, using a regular expression or search string.\n     * @param searchValue A string to search for.\n     * @param replacer A function that returns the replacement text.\n     */\n    replaceAll(searchValue: string | RegExp, replacer: (substring: string, ...args: any[]) => string): string;\n}\n',"lib.esnext.weakref.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\ninterface WeakRef<T extends object> {\n    readonly [Symbol.toStringTag]: "WeakRef";\n\n    /**\n     * Returns the WeakRef instance\'s target object, or undefined if the target object has been\n     * reclaimed.\n     */\n    deref(): T | undefined;\n}\n\ninterface WeakRefConstructor {\n    readonly prototype: WeakRef<any>;\n\n    /**\n     * Creates a WeakRef instance for the given target object.\n     * @param target The target object for the WeakRef instance.\n     */\n    new<T extends object>(target?: T): WeakRef<T>;\n}\n\ndeclare var WeakRef: WeakRefConstructor;\n\ninterface FinalizationRegistry {\n    readonly [Symbol.toStringTag]: "FinalizationRegistry";\n\n    /**\n     * Registers an object with the registry.\n     * @param target The target object to register.\n     * @param heldValue The value to pass to the finalizer for this object. This cannot be the\n     * target object.\n     * @param unregisterToken The token to pass to the unregister method to unregister the target\n     * object. If provided (and not undefined), this must be an object. If not provided, the target\n     * cannot be unregistered.\n     */\n    register(target: object, heldValue: any, unregisterToken?: object): void;\n\n    /**\n     * Unregisters an object from the registry.\n     * @param unregisterToken The token that was used as the unregisterToken argument when calling\n     * register to register the target object.\n     */\n    unregister(unregisterToken: object): void;\n}\n\ninterface FinalizationRegistryConstructor {\n    readonly prototype: FinalizationRegistry;\n\n    /**\n     * Creates a finalization registry with an associated cleanup callback\n     * @param cleanupCallback The callback to call after an object in the registry has been reclaimed.\n     */\n    new(cleanupCallback: (heldValue: any) => void): FinalizationRegistry;\n}\n\ndeclare var FinalizationRegistry: FinalizationRegistryConstructor;\n',"lib.scripthost.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n\n\n/////////////////////////////\n/// Windows Script Host APIS\n/////////////////////////////\n\n\ninterface ActiveXObject {\n    new (s: string): any;\n}\ndeclare var ActiveXObject: ActiveXObject;\n\ninterface ITextWriter {\n    Write(s: string): void;\n    WriteLine(s: string): void;\n    Close(): void;\n}\n\ninterface TextStreamBase {\n    /**\n     * The column number of the current character position in an input stream.\n     */\n    Column: number;\n\n    /**\n     * The current line number in an input stream.\n     */\n    Line: number;\n\n    /**\n     * Closes a text stream.\n     * It is not necessary to close standard streams; they close automatically when the process ends. If\n     * you close a standard stream, be aware that any other pointers to that standard stream become invalid.\n     */\n    Close(): void;\n}\n\ninterface TextStreamWriter extends TextStreamBase {\n    /**\n     * Sends a string to an output stream.\n     */\n    Write(s: string): void;\n\n    /**\n     * Sends a specified number of blank lines (newline characters) to an output stream.\n     */\n    WriteBlankLines(intLines: number): void;\n\n    /**\n     * Sends a string followed by a newline character to an output stream.\n     */\n    WriteLine(s: string): void;\n}\n\ninterface TextStreamReader extends TextStreamBase {\n    /**\n     * Returns a specified number of characters from an input stream, starting at the current pointer position.\n     * Does not return until the ENTER key is pressed.\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\n     */\n    Read(characters: number): string;\n\n    /**\n     * Returns all characters from an input stream.\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\n     */\n    ReadAll(): string;\n\n    /**\n     * Returns an entire line from an input stream.\n     * Although this method extracts the newline character, it does not add it to the returned string.\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\n     */\n    ReadLine(): string;\n\n    /**\n     * Skips a specified number of characters when reading from an input text stream.\n     * Can only be used on a stream in reading mode; causes an error in writing or appending mode.\n     * @param characters Positive number of characters to skip forward. (Backward skipping is not supported.)\n     */\n    Skip(characters: number): void;\n\n    /**\n     * Skips the next line when reading from an input text stream.\n     * Can only be used on a stream in reading mode, not writing or appending mode.\n     */\n    SkipLine(): void;\n\n    /**\n     * Indicates whether the stream pointer position is at the end of a line.\n     */\n    AtEndOfLine: boolean;\n\n    /**\n     * Indicates whether the stream pointer position is at the end of a stream.\n     */\n    AtEndOfStream: boolean;\n}\n\ndeclare var WScript: {\n    /**\n     * Outputs text to either a message box (under WScript.exe) or the command console window followed by\n     * a newline (under CScript.exe).\n     */\n    Echo(s: any): void;\n\n    /**\n     * Exposes the write-only error output stream for the current script.\n     * Can be accessed only while using CScript.exe.\n     */\n    StdErr: TextStreamWriter;\n\n    /**\n     * Exposes the write-only output stream for the current script.\n     * Can be accessed only while using CScript.exe.\n     */\n    StdOut: TextStreamWriter;\n    Arguments: { length: number; Item(n: number): string; };\n\n    /**\n     *  The full path of the currently running script.\n     */\n    ScriptFullName: string;\n\n    /**\n     * Forces the script to stop immediately, with an optional exit code.\n     */\n    Quit(exitCode?: number): number;\n\n    /**\n     * The Windows Script Host build version number.\n     */\n    BuildVersion: number;\n\n    /**\n     * Fully qualified path of the host executable.\n     */\n    FullName: string;\n\n    /**\n     * Gets/sets the script mode - interactive(true) or batch(false).\n     */\n    Interactive: boolean;\n\n    /**\n     * The name of the host executable (WScript.exe or CScript.exe).\n     */\n    Name: string;\n\n    /**\n     * Path of the directory containing the host executable.\n     */\n    Path: string;\n\n    /**\n     * The filename of the currently running script.\n     */\n    ScriptName: string;\n\n    /**\n     * Exposes the read-only input stream for the current script.\n     * Can be accessed only while using CScript.exe.\n     */\n    StdIn: TextStreamReader;\n\n    /**\n     * Windows Script Host version\n     */\n    Version: string;\n\n    /**\n     * Connects a COM object\'s event sources to functions named with a given prefix, in the form prefix_event.\n     */\n    ConnectObject(objEventSource: any, strPrefix: string): void;\n\n    /**\n     * Creates a COM object.\n     * @param strProgiID\n     * @param strPrefix Function names in the form prefix_event will be bound to this object\'s COM events.\n     */\n    CreateObject(strProgID: string, strPrefix?: string): any;\n\n    /**\n     * Disconnects a COM object from its event sources.\n     */\n    DisconnectObject(obj: any): void;\n\n    /**\n     * Retrieves an existing object with the specified ProgID from memory, or creates a new one from a file.\n     * @param strPathname Fully qualified path to the file containing the object persisted to disk.\n     *                       For objects in memory, pass a zero-length string.\n     * @param strProgID\n     * @param strPrefix Function names in the form prefix_event will be bound to this object\'s COM events.\n     */\n    GetObject(strPathname: string, strProgID?: string, strPrefix?: string): any;\n\n    /**\n     * Suspends script execution for a specified length of time, then continues execution.\n     * @param intTime Interval (in milliseconds) to suspend script execution.\n     */\n    Sleep(intTime: number): void;\n};\n\n/**\n * WSH is an alias for WScript under Windows Script Host\n */\ndeclare var WSH: typeof WScript;\n\n/**\n * Represents an Automation SAFEARRAY\n */\ndeclare class SafeArray<T = any> {\n    private constructor();\n    private SafeArray_typekey: SafeArray<T>;\n}\n\n/**\n * Allows enumerating over a COM collection, which may not have indexed item access.\n */\ninterface Enumerator<T = any> {\n    /**\n     * Returns true if the current item is the last one in the collection, or the collection is empty,\n     * or the current item is undefined.\n     */\n    atEnd(): boolean;\n\n    /**\n     * Returns the current item in the collection\n     */\n    item(): T;\n\n    /**\n     * Resets the current item in the collection to the first item. If there are no items in the collection,\n     * the current item is set to undefined.\n     */\n    moveFirst(): void;\n\n    /**\n     * Moves the current item to the next item in the collection. If the enumerator is at the end of\n     * the collection or the collection is empty, the current item is set to undefined.\n     */\n    moveNext(): void;\n}\n\ninterface EnumeratorConstructor {\n    new <T = any>(safearray: SafeArray<T>): Enumerator<T>;\n    new <T = any>(collection: { Item(index: any): T }): Enumerator<T>;\n    new <T = any>(collection: any): Enumerator<T>;\n}\n\ndeclare var Enumerator: EnumeratorConstructor;\n\n/**\n * Enables reading from a COM safe array, which might have an alternate lower bound, or multiple dimensions.\n */\ninterface VBArray<T = any> {\n    /**\n     * Returns the number of dimensions (1-based).\n     */\n    dimensions(): number;\n\n    /**\n     * Takes an index for each dimension in the array, and returns the item at the corresponding location.\n     */\n    getItem(dimension1Index: number, ...dimensionNIndexes: number[]): T;\n\n    /**\n     * Returns the smallest available index for a given dimension.\n     * @param dimension 1-based dimension (defaults to 1)\n     */\n    lbound(dimension?: number): number;\n\n    /**\n     * Returns the largest available index for a given dimension.\n     * @param dimension 1-based dimension (defaults to 1)\n     */\n    ubound(dimension?: number): number;\n\n    /**\n     * Returns a Javascript array with all the elements in the VBArray. If there are multiple dimensions,\n     * each successive dimension is appended to the end of the array.\n     * Example: [[1,2,3],[4,5,6]] becomes [1,2,3,4,5,6]\n     */\n    toArray(): T[];\n}\n\ninterface VBArrayConstructor {\n    new <T = any>(safeArray: SafeArray<T>): VBArray<T>;\n}\n\ndeclare var VBArray: VBArrayConstructor;\n\n/**\n * Automation date (VT_DATE)\n */\ndeclare class VarDate {\n    private constructor();\n    private VarDate_typekey: VarDate;\n}\n\ninterface DateConstructor {\n    new (vd: VarDate): Date;\n}\n\ninterface Date {\n    getVarDate: () => VarDate;\n}\n',"lib.webworker.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/////////////////////////////\n/// Worker APIs\n/////////////////////////////\n\ninterface AddEventListenerOptions extends EventListenerOptions {\n    once?: boolean;\n    passive?: boolean;\n    signal?: AbortSignal;\n}\n\ninterface AesCbcParams extends Algorithm {\n    iv: BufferSource;\n}\n\ninterface AesCtrParams extends Algorithm {\n    counter: BufferSource;\n    length: number;\n}\n\ninterface AesDerivedKeyParams extends Algorithm {\n    length: number;\n}\n\ninterface AesGcmParams extends Algorithm {\n    additionalData?: BufferSource;\n    iv: BufferSource;\n    tagLength?: number;\n}\n\ninterface AesKeyAlgorithm extends KeyAlgorithm {\n    length: number;\n}\n\ninterface AesKeyGenParams extends Algorithm {\n    length: number;\n}\n\ninterface Algorithm {\n    name: string;\n}\n\ninterface AudioConfiguration {\n    bitrate?: number;\n    channels?: string;\n    contentType: string;\n    samplerate?: number;\n    spatialRendering?: boolean;\n}\n\ninterface BlobPropertyBag {\n    endings?: EndingType;\n    type?: string;\n}\n\ninterface CacheQueryOptions {\n    ignoreMethod?: boolean;\n    ignoreSearch?: boolean;\n    ignoreVary?: boolean;\n}\n\ninterface ClientQueryOptions {\n    includeUncontrolled?: boolean;\n    type?: ClientTypes;\n}\n\ninterface CloseEventInit extends EventInit {\n    code?: number;\n    reason?: string;\n    wasClean?: boolean;\n}\n\ninterface CryptoKeyPair {\n    privateKey?: CryptoKey;\n    publicKey?: CryptoKey;\n}\n\ninterface CustomEventInit<T = any> extends EventInit {\n    detail?: T;\n}\n\ninterface DOMMatrix2DInit {\n    a?: number;\n    b?: number;\n    c?: number;\n    d?: number;\n    e?: number;\n    f?: number;\n    m11?: number;\n    m12?: number;\n    m21?: number;\n    m22?: number;\n    m41?: number;\n    m42?: number;\n}\n\ninterface DOMMatrixInit extends DOMMatrix2DInit {\n    is2D?: boolean;\n    m13?: number;\n    m14?: number;\n    m23?: number;\n    m24?: number;\n    m31?: number;\n    m32?: number;\n    m33?: number;\n    m34?: number;\n    m43?: number;\n    m44?: number;\n}\n\ninterface DOMPointInit {\n    w?: number;\n    x?: number;\n    y?: number;\n    z?: number;\n}\n\ninterface DOMQuadInit {\n    p1?: DOMPointInit;\n    p2?: DOMPointInit;\n    p3?: DOMPointInit;\n    p4?: DOMPointInit;\n}\n\ninterface DOMRectInit {\n    height?: number;\n    width?: number;\n    x?: number;\n    y?: number;\n}\n\ninterface EcKeyGenParams extends Algorithm {\n    namedCurve: NamedCurve;\n}\n\ninterface EcKeyImportParams extends Algorithm {\n    namedCurve: NamedCurve;\n}\n\ninterface EcdhKeyDeriveParams extends Algorithm {\n    public: CryptoKey;\n}\n\ninterface EcdsaParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n}\n\ninterface ErrorEventInit extends EventInit {\n    colno?: number;\n    error?: any;\n    filename?: string;\n    lineno?: number;\n    message?: string;\n}\n\ninterface EventInit {\n    bubbles?: boolean;\n    cancelable?: boolean;\n    composed?: boolean;\n}\n\ninterface EventListenerOptions {\n    capture?: boolean;\n}\n\ninterface EventSourceInit {\n    withCredentials?: boolean;\n}\n\ninterface ExtendableEventInit extends EventInit {\n}\n\ninterface ExtendableMessageEventInit extends ExtendableEventInit {\n    data?: any;\n    lastEventId?: string;\n    origin?: string;\n    ports?: MessagePort[];\n    source?: Client | ServiceWorker | MessagePort | null;\n}\n\ninterface FetchEventInit extends ExtendableEventInit {\n    clientId?: string;\n    handled?: Promise<undefined>;\n    preloadResponse?: Promise<any>;\n    replacesClientId?: string;\n    request: Request;\n    resultingClientId?: string;\n}\n\ninterface FilePropertyBag extends BlobPropertyBag {\n    lastModified?: number;\n}\n\ninterface FontFaceDescriptors {\n    display?: string;\n    featureSettings?: string;\n    stretch?: string;\n    style?: string;\n    unicodeRange?: string;\n    variant?: string;\n    weight?: string;\n}\n\ninterface FontFaceSetLoadEventInit extends EventInit {\n    fontfaces?: FontFace[];\n}\n\ninterface GetNotificationOptions {\n    tag?: string;\n}\n\ninterface HkdfParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n    info: BufferSource;\n    salt: BufferSource;\n}\n\ninterface HmacImportParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n    length?: number;\n}\n\ninterface HmacKeyGenParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n    length?: number;\n}\n\ninterface IDBDatabaseInfo {\n    name?: string;\n    version?: number;\n}\n\ninterface IDBIndexParameters {\n    multiEntry?: boolean;\n    unique?: boolean;\n}\n\ninterface IDBObjectStoreParameters {\n    autoIncrement?: boolean;\n    keyPath?: string | string[] | null;\n}\n\ninterface IDBVersionChangeEventInit extends EventInit {\n    newVersion?: number | null;\n    oldVersion?: number;\n}\n\ninterface ImageBitmapOptions {\n    colorSpaceConversion?: ColorSpaceConversion;\n    imageOrientation?: ImageOrientation;\n    premultiplyAlpha?: PremultiplyAlpha;\n    resizeHeight?: number;\n    resizeQuality?: ResizeQuality;\n    resizeWidth?: number;\n}\n\ninterface ImageBitmapRenderingContextSettings {\n    alpha?: boolean;\n}\n\ninterface ImageDataSettings {\n    colorSpace?: PredefinedColorSpace;\n}\n\ninterface ImportMeta {\n    url: string;\n}\n\ninterface JsonWebKey {\n    alg?: string;\n    crv?: string;\n    d?: string;\n    dp?: string;\n    dq?: string;\n    e?: string;\n    ext?: boolean;\n    k?: string;\n    key_ops?: string[];\n    kty?: string;\n    n?: string;\n    oth?: RsaOtherPrimesInfo[];\n    p?: string;\n    q?: string;\n    qi?: string;\n    use?: string;\n    x?: string;\n    y?: string;\n}\n\ninterface KeyAlgorithm {\n    name: string;\n}\n\ninterface MediaCapabilitiesDecodingInfo extends MediaCapabilitiesInfo {\n    configuration?: MediaDecodingConfiguration;\n}\n\ninterface MediaCapabilitiesEncodingInfo extends MediaCapabilitiesInfo {\n    configuration?: MediaEncodingConfiguration;\n}\n\ninterface MediaCapabilitiesInfo {\n    powerEfficient: boolean;\n    smooth: boolean;\n    supported: boolean;\n}\n\ninterface MediaConfiguration {\n    audio?: AudioConfiguration;\n    video?: VideoConfiguration;\n}\n\ninterface MediaDecodingConfiguration extends MediaConfiguration {\n    type: MediaDecodingType;\n}\n\ninterface MediaEncodingConfiguration extends MediaConfiguration {\n    type: MediaEncodingType;\n}\n\ninterface MessageEventInit<T = any> extends EventInit {\n    data?: T;\n    lastEventId?: string;\n    origin?: string;\n    ports?: MessagePort[];\n    source?: MessageEventSource | null;\n}\n\ninterface MultiCacheQueryOptions extends CacheQueryOptions {\n    cacheName?: string;\n}\n\ninterface NotificationAction {\n    action: string;\n    icon?: string;\n    title: string;\n}\n\ninterface NotificationEventInit extends ExtendableEventInit {\n    action?: string;\n    notification: Notification;\n}\n\ninterface NotificationOptions {\n    actions?: NotificationAction[];\n    badge?: string;\n    body?: string;\n    data?: any;\n    dir?: NotificationDirection;\n    icon?: string;\n    image?: string;\n    lang?: string;\n    renotify?: boolean;\n    requireInteraction?: boolean;\n    silent?: boolean;\n    tag?: string;\n    timestamp?: DOMTimeStamp;\n    vibrate?: VibratePattern;\n}\n\ninterface Pbkdf2Params extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n    iterations: number;\n    salt: BufferSource;\n}\n\ninterface PerformanceMarkOptions {\n    detail?: any;\n    startTime?: DOMHighResTimeStamp;\n}\n\ninterface PerformanceMeasureOptions {\n    detail?: any;\n    duration?: DOMHighResTimeStamp;\n    end?: string | DOMHighResTimeStamp;\n    start?: string | DOMHighResTimeStamp;\n}\n\ninterface PerformanceObserverInit {\n    buffered?: boolean;\n    entryTypes?: string[];\n    type?: string;\n}\n\ninterface PermissionDescriptor {\n    name: PermissionName;\n}\n\ninterface PostMessageOptions {\n    transfer?: any[];\n}\n\ninterface ProgressEventInit extends EventInit {\n    lengthComputable?: boolean;\n    loaded?: number;\n    total?: number;\n}\n\ninterface PromiseRejectionEventInit extends EventInit {\n    promise: Promise<any>;\n    reason?: any;\n}\n\ninterface PushEventInit extends ExtendableEventInit {\n    data?: PushMessageDataInit;\n}\n\ninterface PushSubscriptionJSON {\n    endpoint?: string;\n    expirationTime?: DOMTimeStamp | null;\n    keys?: Record<string, string>;\n}\n\ninterface PushSubscriptionOptionsInit {\n    applicationServerKey?: BufferSource | string | null;\n    userVisibleOnly?: boolean;\n}\n\ninterface QueuingStrategy<T = any> {\n    highWaterMark?: number;\n    size?: QueuingStrategySize<T>;\n}\n\ninterface QueuingStrategyInit {\n    /**\n     * Creates a new ByteLengthQueuingStrategy with the provided high water mark.\n     *\n     * Note that the provided high water mark will not be validated ahead of time. Instead, if it is negative, NaN, or not a number, the resulting ByteLengthQueuingStrategy will cause the corresponding stream constructor to throw.\n     */\n    highWaterMark: number;\n}\n\ninterface ReadableStreamDefaultReadDoneResult {\n    done: true;\n    value?: undefined;\n}\n\ninterface ReadableStreamDefaultReadValueResult<T> {\n    done: false;\n    value: T;\n}\n\ninterface ReadableWritablePair<R = any, W = any> {\n    readable: ReadableStream<R>;\n    /**\n     * Provides a convenient, chainable way of piping this readable stream through a transform stream (or any other { writable, readable } pair). It simply pipes the stream into the writable side of the supplied pair, and returns the readable side for further use.\n     *\n     * Piping a stream will lock it for the duration of the pipe, preventing any other consumer from acquiring a reader.\n     */\n    writable: WritableStream<W>;\n}\n\ninterface RegistrationOptions {\n    scope?: string;\n    type?: WorkerType;\n    updateViaCache?: ServiceWorkerUpdateViaCache;\n}\n\ninterface RequestInit {\n    /**\n     * A BodyInit object or null to set request\'s body.\n     */\n    body?: BodyInit | null;\n    /**\n     * A string indicating how the request will interact with the browser\'s cache to set request\'s cache.\n     */\n    cache?: RequestCache;\n    /**\n     * A string indicating whether credentials will be sent with the request always, never, or only when sent to a same-origin URL. Sets request\'s credentials.\n     */\n    credentials?: RequestCredentials;\n    /**\n     * A Headers object, an object literal, or an array of two-item arrays to set request\'s headers.\n     */\n    headers?: HeadersInit;\n    /**\n     * A cryptographic hash of the resource to be fetched by request. Sets request\'s integrity.\n     */\n    integrity?: string;\n    /**\n     * A boolean to set request\'s keepalive.\n     */\n    keepalive?: boolean;\n    /**\n     * A string to set request\'s method.\n     */\n    method?: string;\n    /**\n     * A string to indicate whether the request will use CORS, or will be restricted to same-origin URLs. Sets request\'s mode.\n     */\n    mode?: RequestMode;\n    /**\n     * A string indicating whether request follows redirects, results in an error upon encountering a redirect, or returns the redirect (in an opaque fashion). Sets request\'s redirect.\n     */\n    redirect?: RequestRedirect;\n    /**\n     * A string whose value is a same-origin URL, "about:client", or the empty string, to set request\'s referrer.\n     */\n    referrer?: string;\n    /**\n     * A referrer policy to set request\'s referrerPolicy.\n     */\n    referrerPolicy?: ReferrerPolicy;\n    /**\n     * An AbortSignal to set request\'s signal.\n     */\n    signal?: AbortSignal | null;\n    /**\n     * Can only be null. Used to disassociate request from any Window.\n     */\n    window?: any;\n}\n\ninterface ResponseInit {\n    headers?: HeadersInit;\n    status?: number;\n    statusText?: string;\n}\n\ninterface RsaHashedImportParams extends Algorithm {\n    hash: HashAlgorithmIdentifier;\n}\n\ninterface RsaHashedKeyGenParams extends RsaKeyGenParams {\n    hash: HashAlgorithmIdentifier;\n}\n\ninterface RsaKeyGenParams extends Algorithm {\n    modulusLength: number;\n    publicExponent: BigInteger;\n}\n\ninterface RsaOaepParams extends Algorithm {\n    label?: BufferSource;\n}\n\ninterface RsaOtherPrimesInfo {\n    d?: string;\n    r?: string;\n    t?: string;\n}\n\ninterface RsaPssParams extends Algorithm {\n    saltLength: number;\n}\n\ninterface SecurityPolicyViolationEventInit extends EventInit {\n    blockedURI?: string;\n    columnNumber?: number;\n    disposition: SecurityPolicyViolationEventDisposition;\n    documentURI: string;\n    effectiveDirective: string;\n    lineNumber?: number;\n    originalPolicy: string;\n    referrer?: string;\n    sample?: string;\n    sourceFile?: string;\n    statusCode: number;\n    violatedDirective: string;\n}\n\ninterface StorageEstimate {\n    quota?: number;\n    usage?: number;\n}\n\ninterface StreamPipeOptions {\n    preventAbort?: boolean;\n    preventCancel?: boolean;\n    /**\n     * Pipes this readable stream to a given writable stream destination. The way in which the piping process behaves under various error conditions can be customized with a number of passed options. It returns a promise that fulfills when the piping process completes successfully, or rejects if any errors were encountered.\n     *\n     * Piping a stream will lock it for the duration of the pipe, preventing any other consumer from acquiring a reader.\n     *\n     * Errors and closures of the source and destination streams propagate as follows:\n     *\n     * An error in this source readable stream will abort destination, unless preventAbort is truthy. The returned promise will be rejected with the source\'s error, or with any error that occurs during aborting the destination.\n     *\n     * An error in destination will cancel this source readable stream, unless preventCancel is truthy. The returned promise will be rejected with the destination\'s error, or with any error that occurs during canceling the source.\n     *\n     * When this source readable stream closes, destination will be closed, unless preventClose is truthy. The returned promise will be fulfilled once this process completes, unless an error is encountered while closing the destination, in which case it will be rejected with that error.\n     *\n     * If destination starts out closed or closing, this source readable stream will be canceled, unless preventCancel is true. The returned promise will be rejected with an error indicating piping to a closed stream failed, or with any error that occurs during canceling the source.\n     *\n     * The signal option can be set to an AbortSignal to allow aborting an ongoing pipe operation via the corresponding AbortController. In this case, this source readable stream will be canceled, and destination aborted, unless the respective options preventCancel or preventAbort are set.\n     */\n    preventClose?: boolean;\n    signal?: AbortSignal;\n}\n\ninterface TextDecodeOptions {\n    stream?: boolean;\n}\n\ninterface TextDecoderOptions {\n    fatal?: boolean;\n    ignoreBOM?: boolean;\n}\n\ninterface TextEncoderEncodeIntoResult {\n    read?: number;\n    written?: number;\n}\n\ninterface Transformer<I = any, O = any> {\n    flush?: TransformerFlushCallback<O>;\n    readableType?: undefined;\n    start?: TransformerStartCallback<O>;\n    transform?: TransformerTransformCallback<I, O>;\n    writableType?: undefined;\n}\n\ninterface UnderlyingSink<W = any> {\n    abort?: UnderlyingSinkAbortCallback;\n    close?: UnderlyingSinkCloseCallback;\n    start?: UnderlyingSinkStartCallback;\n    type?: undefined;\n    write?: UnderlyingSinkWriteCallback<W>;\n}\n\ninterface UnderlyingSource<R = any> {\n    cancel?: UnderlyingSourceCancelCallback;\n    pull?: UnderlyingSourcePullCallback<R>;\n    start?: UnderlyingSourceStartCallback<R>;\n    type?: undefined;\n}\n\ninterface VideoConfiguration {\n    bitrate: number;\n    colorGamut?: ColorGamut;\n    contentType: string;\n    framerate: number;\n    hdrMetadataType?: HdrMetadataType;\n    height: number;\n    scalabilityMode?: string;\n    transferFunction?: TransferFunction;\n    width: number;\n}\n\ninterface WebGLContextAttributes {\n    alpha?: boolean;\n    antialias?: boolean;\n    depth?: boolean;\n    desynchronized?: boolean;\n    failIfMajorPerformanceCaveat?: boolean;\n    powerPreference?: WebGLPowerPreference;\n    premultipliedAlpha?: boolean;\n    preserveDrawingBuffer?: boolean;\n    stencil?: boolean;\n}\n\ninterface WebGLContextEventInit extends EventInit {\n    statusMessage?: string;\n}\n\ninterface WorkerOptions {\n    credentials?: RequestCredentials;\n    name?: string;\n    type?: WorkerType;\n}\n\n/** The ANGLE_instanced_arrays extension is part of the WebGL API and allows to draw the same object, or groups of similar objects multiple times, if they share the same vertex data, primitive count and type. */\ninterface ANGLE_instanced_arrays {\n    drawArraysInstancedANGLE(mode: GLenum, first: GLint, count: GLsizei, primcount: GLsizei): void;\n    drawElementsInstancedANGLE(mode: GLenum, count: GLsizei, type: GLenum, offset: GLintptr, primcount: GLsizei): void;\n    vertexAttribDivisorANGLE(index: GLuint, divisor: GLuint): void;\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE: GLenum;\n}\n\n/** A controller object that allows you to abort one or more DOM requests as and when desired. */\ninterface AbortController {\n    /**\n     * Returns the AbortSignal object associated with this object.\n     */\n    readonly signal: AbortSignal;\n    /**\n     * Invoking this method will set this object\'s AbortSignal\'s aborted flag and signal to any observers that the associated activity is to be aborted.\n     */\n    abort(): void;\n}\n\ndeclare var AbortController: {\n    prototype: AbortController;\n    new(): AbortController;\n};\n\ninterface AbortSignalEventMap {\n    "abort": Event;\n}\n\n/** A signal object that allows you to communicate with a DOM request (such as a Fetch) and abort it if required via an AbortController object. */\ninterface AbortSignal extends EventTarget {\n    /**\n     * Returns true if this AbortSignal\'s AbortController has signaled to abort, and false otherwise.\n     */\n    readonly aborted: boolean;\n    onabort: ((this: AbortSignal, ev: Event) => any) | null;\n    addEventListener<K extends keyof AbortSignalEventMap>(type: K, listener: (this: AbortSignal, ev: AbortSignalEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AbortSignalEventMap>(type: K, listener: (this: AbortSignal, ev: AbortSignalEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var AbortSignal: {\n    prototype: AbortSignal;\n    new(): AbortSignal;\n};\n\ninterface AbstractWorkerEventMap {\n    "error": ErrorEvent;\n}\n\ninterface AbstractWorker {\n    onerror: ((this: AbstractWorker, ev: ErrorEvent) => any) | null;\n    addEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: AbstractWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof AbstractWorkerEventMap>(type: K, listener: (this: AbstractWorker, ev: AbstractWorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ninterface AnimationFrameProvider {\n    cancelAnimationFrame(handle: number): void;\n    requestAnimationFrame(callback: FrameRequestCallback): number;\n}\n\n/** A file-like object of immutable, raw data. Blobs represent data that isn\'t necessarily in a JavaScript-native format. The File interface is based on Blob, inheriting blob functionality and expanding it to support files on the user\'s system. */\ninterface Blob {\n    readonly size: number;\n    readonly type: string;\n    arrayBuffer(): Promise<ArrayBuffer>;\n    slice(start?: number, end?: number, contentType?: string): Blob;\n    stream(): ReadableStream;\n    text(): Promise<string>;\n}\n\ndeclare var Blob: {\n    prototype: Blob;\n    new(blobParts?: BlobPart[], options?: BlobPropertyBag): Blob;\n};\n\ninterface Body {\n    readonly body: ReadableStream<Uint8Array> | null;\n    readonly bodyUsed: boolean;\n    arrayBuffer(): Promise<ArrayBuffer>;\n    blob(): Promise<Blob>;\n    formData(): Promise<FormData>;\n    json(): Promise<any>;\n    text(): Promise<string>;\n}\n\ninterface BroadcastChannelEventMap {\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n}\n\ninterface BroadcastChannel extends EventTarget {\n    /**\n     * Returns the channel name (as passed to the constructor).\n     */\n    readonly name: string;\n    onmessage: ((this: BroadcastChannel, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: BroadcastChannel, ev: MessageEvent) => any) | null;\n    /**\n     * Closes the BroadcastChannel object, opening it up to garbage collection.\n     */\n    close(): void;\n    /**\n     * Sends the given message to other BroadcastChannel objects set up for this channel. Messages can be structured objects, e.g. nested objects and arrays.\n     */\n    postMessage(message: any): void;\n    addEventListener<K extends keyof BroadcastChannelEventMap>(type: K, listener: (this: BroadcastChannel, ev: BroadcastChannelEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof BroadcastChannelEventMap>(type: K, listener: (this: BroadcastChannel, ev: BroadcastChannelEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var BroadcastChannel: {\n    prototype: BroadcastChannel;\n    new(name: string): BroadcastChannel;\n};\n\n/** This Streams API interface provides a built-in byte length queuing strategy that can be used when constructing streams. */\ninterface ByteLengthQueuingStrategy extends QueuingStrategy<ArrayBufferView> {\n    readonly highWaterMark: number;\n    readonly size: QueuingStrategySize<ArrayBufferView>;\n}\n\ndeclare var ByteLengthQueuingStrategy: {\n    prototype: ByteLengthQueuingStrategy;\n    new(init: QueuingStrategyInit): ByteLengthQueuingStrategy;\n};\n\n/** Provides a storage mechanism for Request / Response object pairs that are cached, for example as part of the ServiceWorker life cycle. Note that the Cache interface is exposed to windowed scopes as well as workers. You don\'t have to use it in conjunction with service workers, even though it is defined in the service worker spec. */\ninterface Cache {\n    add(request: RequestInfo): Promise<void>;\n    addAll(requests: RequestInfo[]): Promise<void>;\n    delete(request: RequestInfo, options?: CacheQueryOptions): Promise<boolean>;\n    keys(request?: RequestInfo, options?: CacheQueryOptions): Promise<ReadonlyArray<Request>>;\n    match(request: RequestInfo, options?: CacheQueryOptions): Promise<Response | undefined>;\n    matchAll(request?: RequestInfo, options?: CacheQueryOptions): Promise<ReadonlyArray<Response>>;\n    put(request: RequestInfo, response: Response): Promise<void>;\n}\n\ndeclare var Cache: {\n    prototype: Cache;\n    new(): Cache;\n};\n\n/** The storage for Cache objects. */\ninterface CacheStorage {\n    delete(cacheName: string): Promise<boolean>;\n    has(cacheName: string): Promise<boolean>;\n    keys(): Promise<string[]>;\n    match(request: RequestInfo, options?: MultiCacheQueryOptions): Promise<Response | undefined>;\n    open(cacheName: string): Promise<Cache>;\n}\n\ndeclare var CacheStorage: {\n    prototype: CacheStorage;\n    new(): CacheStorage;\n};\n\n/** An opaque object describing a gradient. It is returned by the methods CanvasRenderingContext2D.createLinearGradient() or CanvasRenderingContext2D.createRadialGradient(). */\ninterface CanvasGradient {\n    /**\n     * Adds a color stop with the given color to the gradient at the given offset. 0.0 is the offset at one end of the gradient, 1.0 is the offset at the other end.\n     *\n     * Throws an "IndexSizeError" DOMException if the offset is out of range. Throws a "SyntaxError" DOMException if the color cannot be parsed.\n     */\n    addColorStop(offset: number, color: string): void;\n}\n\ndeclare var CanvasGradient: {\n    prototype: CanvasGradient;\n    new(): CanvasGradient;\n};\n\ninterface CanvasPath {\n    arc(x: number, y: number, radius: number, startAngle: number, endAngle: number, counterclockwise?: boolean): void;\n    arcTo(x1: number, y1: number, x2: number, y2: number, radius: number): void;\n    bezierCurveTo(cp1x: number, cp1y: number, cp2x: number, cp2y: number, x: number, y: number): void;\n    closePath(): void;\n    ellipse(x: number, y: number, radiusX: number, radiusY: number, rotation: number, startAngle: number, endAngle: number, counterclockwise?: boolean): void;\n    lineTo(x: number, y: number): void;\n    moveTo(x: number, y: number): void;\n    quadraticCurveTo(cpx: number, cpy: number, x: number, y: number): void;\n    rect(x: number, y: number, w: number, h: number): void;\n}\n\n/** An opaque object describing a pattern, based on an image, a canvas, or a video, created by the CanvasRenderingContext2D.createPattern() method. */\ninterface CanvasPattern {\n    /**\n     * Sets the transformation matrix that will be used when rendering the pattern during a fill or stroke painting operation.\n     */\n    setTransform(transform?: DOMMatrix2DInit): void;\n}\n\ndeclare var CanvasPattern: {\n    prototype: CanvasPattern;\n    new(): CanvasPattern;\n};\n\n/** The Client interface represents an executable context such as a Worker, or a SharedWorker. Window clients are represented by the more-specific WindowClient. You can get Client/WindowClient objects from methods such as Clients.matchAll() and Clients.get(). */\ninterface Client {\n    readonly frameType: FrameType;\n    readonly id: string;\n    readonly type: ClientTypes;\n    readonly url: string;\n    postMessage(message: any, transfer: Transferable[]): void;\n    postMessage(message: any, options?: PostMessageOptions): void;\n}\n\ndeclare var Client: {\n    prototype: Client;\n    new(): Client;\n};\n\n/** Provides access to Client objects. Access it via self.clients within a service worker. */\ninterface Clients {\n    claim(): Promise<void>;\n    get(id: string): Promise<Client | undefined>;\n    matchAll<T extends ClientQueryOptions>(options?: T): Promise<ReadonlyArray<T["type"] extends "window" ? WindowClient : Client>>;\n    openWindow(url: string | URL): Promise<WindowClient | null>;\n}\n\ndeclare var Clients: {\n    prototype: Clients;\n    new(): Clients;\n};\n\n/** A CloseEvent is sent to clients using WebSockets when the connection is closed. This is delivered to the listener indicated by the WebSocket object\'s onclose attribute. */\ninterface CloseEvent extends Event {\n    /**\n     * Returns the WebSocket connection close code provided by the server.\n     */\n    readonly code: number;\n    /**\n     * Returns the WebSocket connection close reason provided by the server.\n     */\n    readonly reason: string;\n    /**\n     * Returns true if the connection closed cleanly; false otherwise.\n     */\n    readonly wasClean: boolean;\n}\n\ndeclare var CloseEvent: {\n    prototype: CloseEvent;\n    new(type: string, eventInitDict?: CloseEventInit): CloseEvent;\n};\n\n/** This Streams API interface provides a built-in byte length queuing strategy that can be used when constructing streams. */\ninterface CountQueuingStrategy extends QueuingStrategy {\n    readonly highWaterMark: number;\n    readonly size: QueuingStrategySize;\n}\n\ndeclare var CountQueuingStrategy: {\n    prototype: CountQueuingStrategy;\n    new(init: QueuingStrategyInit): CountQueuingStrategy;\n};\n\n/** Basic cryptography features available in the current context. It allows access to a cryptographically strong random number generator and to cryptographic primitives. */\ninterface Crypto {\n    readonly subtle: SubtleCrypto;\n    getRandomValues<T extends ArrayBufferView | null>(array: T): T;\n}\n\ndeclare var Crypto: {\n    prototype: Crypto;\n    new(): Crypto;\n};\n\n/** The CryptoKey dictionary of the Web Crypto API represents a cryptographic key. */\ninterface CryptoKey {\n    readonly algorithm: KeyAlgorithm;\n    readonly extractable: boolean;\n    readonly type: KeyType;\n    readonly usages: KeyUsage[];\n}\n\ndeclare var CryptoKey: {\n    prototype: CryptoKey;\n    new(): CryptoKey;\n};\n\ninterface CustomEvent<T = any> extends Event {\n    /**\n     * Returns any custom data event was created with. Typically used for synthetic events.\n     */\n    readonly detail: T;\n    /** @deprecated */\n    initCustomEvent(type: string, bubbles?: boolean, cancelable?: boolean, detail?: T): void;\n}\n\ndeclare var CustomEvent: {\n    prototype: CustomEvent;\n    new<T>(type: string, eventInitDict?: CustomEventInit<T>): CustomEvent<T>;\n};\n\n/** An abnormal event (called an exception) which occurs as a result of calling a method or accessing a property of a web API. */\ninterface DOMException {\n    readonly code: number;\n    readonly message: string;\n    readonly name: string;\n    readonly ABORT_ERR: number;\n    readonly DATA_CLONE_ERR: number;\n    readonly DOMSTRING_SIZE_ERR: number;\n    readonly HIERARCHY_REQUEST_ERR: number;\n    readonly INDEX_SIZE_ERR: number;\n    readonly INUSE_ATTRIBUTE_ERR: number;\n    readonly INVALID_ACCESS_ERR: number;\n    readonly INVALID_CHARACTER_ERR: number;\n    readonly INVALID_MODIFICATION_ERR: number;\n    readonly INVALID_NODE_TYPE_ERR: number;\n    readonly INVALID_STATE_ERR: number;\n    readonly NAMESPACE_ERR: number;\n    readonly NETWORK_ERR: number;\n    readonly NOT_FOUND_ERR: number;\n    readonly NOT_SUPPORTED_ERR: number;\n    readonly NO_DATA_ALLOWED_ERR: number;\n    readonly NO_MODIFICATION_ALLOWED_ERR: number;\n    readonly QUOTA_EXCEEDED_ERR: number;\n    readonly SECURITY_ERR: number;\n    readonly SYNTAX_ERR: number;\n    readonly TIMEOUT_ERR: number;\n    readonly TYPE_MISMATCH_ERR: number;\n    readonly URL_MISMATCH_ERR: number;\n    readonly VALIDATION_ERR: number;\n    readonly WRONG_DOCUMENT_ERR: number;\n}\n\ndeclare var DOMException: {\n    prototype: DOMException;\n    new(message?: string, name?: string): DOMException;\n    readonly ABORT_ERR: number;\n    readonly DATA_CLONE_ERR: number;\n    readonly DOMSTRING_SIZE_ERR: number;\n    readonly HIERARCHY_REQUEST_ERR: number;\n    readonly INDEX_SIZE_ERR: number;\n    readonly INUSE_ATTRIBUTE_ERR: number;\n    readonly INVALID_ACCESS_ERR: number;\n    readonly INVALID_CHARACTER_ERR: number;\n    readonly INVALID_MODIFICATION_ERR: number;\n    readonly INVALID_NODE_TYPE_ERR: number;\n    readonly INVALID_STATE_ERR: number;\n    readonly NAMESPACE_ERR: number;\n    readonly NETWORK_ERR: number;\n    readonly NOT_FOUND_ERR: number;\n    readonly NOT_SUPPORTED_ERR: number;\n    readonly NO_DATA_ALLOWED_ERR: number;\n    readonly NO_MODIFICATION_ALLOWED_ERR: number;\n    readonly QUOTA_EXCEEDED_ERR: number;\n    readonly SECURITY_ERR: number;\n    readonly SYNTAX_ERR: number;\n    readonly TIMEOUT_ERR: number;\n    readonly TYPE_MISMATCH_ERR: number;\n    readonly URL_MISMATCH_ERR: number;\n    readonly VALIDATION_ERR: number;\n    readonly WRONG_DOCUMENT_ERR: number;\n};\n\ninterface DOMMatrix extends DOMMatrixReadOnly {\n    a: number;\n    b: number;\n    c: number;\n    d: number;\n    e: number;\n    f: number;\n    m11: number;\n    m12: number;\n    m13: number;\n    m14: number;\n    m21: number;\n    m22: number;\n    m23: number;\n    m24: number;\n    m31: number;\n    m32: number;\n    m33: number;\n    m34: number;\n    m41: number;\n    m42: number;\n    m43: number;\n    m44: number;\n    invertSelf(): DOMMatrix;\n    multiplySelf(other?: DOMMatrixInit): DOMMatrix;\n    preMultiplySelf(other?: DOMMatrixInit): DOMMatrix;\n    rotateAxisAngleSelf(x?: number, y?: number, z?: number, angle?: number): DOMMatrix;\n    rotateFromVectorSelf(x?: number, y?: number): DOMMatrix;\n    rotateSelf(rotX?: number, rotY?: number, rotZ?: number): DOMMatrix;\n    scale3dSelf(scale?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\n    scaleSelf(scaleX?: number, scaleY?: number, scaleZ?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\n    skewXSelf(sx?: number): DOMMatrix;\n    skewYSelf(sy?: number): DOMMatrix;\n    translateSelf(tx?: number, ty?: number, tz?: number): DOMMatrix;\n}\n\ndeclare var DOMMatrix: {\n    prototype: DOMMatrix;\n    new(init?: string | number[]): DOMMatrix;\n    fromFloat32Array(array32: Float32Array): DOMMatrix;\n    fromFloat64Array(array64: Float64Array): DOMMatrix;\n    fromMatrix(other?: DOMMatrixInit): DOMMatrix;\n};\n\ninterface DOMMatrixReadOnly {\n    readonly a: number;\n    readonly b: number;\n    readonly c: number;\n    readonly d: number;\n    readonly e: number;\n    readonly f: number;\n    readonly is2D: boolean;\n    readonly isIdentity: boolean;\n    readonly m11: number;\n    readonly m12: number;\n    readonly m13: number;\n    readonly m14: number;\n    readonly m21: number;\n    readonly m22: number;\n    readonly m23: number;\n    readonly m24: number;\n    readonly m31: number;\n    readonly m32: number;\n    readonly m33: number;\n    readonly m34: number;\n    readonly m41: number;\n    readonly m42: number;\n    readonly m43: number;\n    readonly m44: number;\n    flipX(): DOMMatrix;\n    flipY(): DOMMatrix;\n    inverse(): DOMMatrix;\n    multiply(other?: DOMMatrixInit): DOMMatrix;\n    rotate(rotX?: number, rotY?: number, rotZ?: number): DOMMatrix;\n    rotateAxisAngle(x?: number, y?: number, z?: number, angle?: number): DOMMatrix;\n    rotateFromVector(x?: number, y?: number): DOMMatrix;\n    scale(scaleX?: number, scaleY?: number, scaleZ?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\n    scale3d(scale?: number, originX?: number, originY?: number, originZ?: number): DOMMatrix;\n    /** @deprecated */\n    scaleNonUniform(scaleX?: number, scaleY?: number): DOMMatrix;\n    skewX(sx?: number): DOMMatrix;\n    skewY(sy?: number): DOMMatrix;\n    toFloat32Array(): Float32Array;\n    toFloat64Array(): Float64Array;\n    toJSON(): any;\n    transformPoint(point?: DOMPointInit): DOMPoint;\n    translate(tx?: number, ty?: number, tz?: number): DOMMatrix;\n}\n\ndeclare var DOMMatrixReadOnly: {\n    prototype: DOMMatrixReadOnly;\n    new(init?: string | number[]): DOMMatrixReadOnly;\n    fromFloat32Array(array32: Float32Array): DOMMatrixReadOnly;\n    fromFloat64Array(array64: Float64Array): DOMMatrixReadOnly;\n    fromMatrix(other?: DOMMatrixInit): DOMMatrixReadOnly;\n};\n\ninterface DOMPoint extends DOMPointReadOnly {\n    w: number;\n    x: number;\n    y: number;\n    z: number;\n}\n\ndeclare var DOMPoint: {\n    prototype: DOMPoint;\n    new(x?: number, y?: number, z?: number, w?: number): DOMPoint;\n    fromPoint(other?: DOMPointInit): DOMPoint;\n};\n\ninterface DOMPointReadOnly {\n    readonly w: number;\n    readonly x: number;\n    readonly y: number;\n    readonly z: number;\n    matrixTransform(matrix?: DOMMatrixInit): DOMPoint;\n    toJSON(): any;\n}\n\ndeclare var DOMPointReadOnly: {\n    prototype: DOMPointReadOnly;\n    new(x?: number, y?: number, z?: number, w?: number): DOMPointReadOnly;\n    fromPoint(other?: DOMPointInit): DOMPointReadOnly;\n};\n\ninterface DOMQuad {\n    readonly p1: DOMPoint;\n    readonly p2: DOMPoint;\n    readonly p3: DOMPoint;\n    readonly p4: DOMPoint;\n    getBounds(): DOMRect;\n    toJSON(): any;\n}\n\ndeclare var DOMQuad: {\n    prototype: DOMQuad;\n    new(p1?: DOMPointInit, p2?: DOMPointInit, p3?: DOMPointInit, p4?: DOMPointInit): DOMQuad;\n    fromQuad(other?: DOMQuadInit): DOMQuad;\n    fromRect(other?: DOMRectInit): DOMQuad;\n};\n\ninterface DOMRect extends DOMRectReadOnly {\n    height: number;\n    width: number;\n    x: number;\n    y: number;\n}\n\ndeclare var DOMRect: {\n    prototype: DOMRect;\n    new(x?: number, y?: number, width?: number, height?: number): DOMRect;\n    fromRect(other?: DOMRectInit): DOMRect;\n};\n\ninterface DOMRectReadOnly {\n    readonly bottom: number;\n    readonly height: number;\n    readonly left: number;\n    readonly right: number;\n    readonly top: number;\n    readonly width: number;\n    readonly x: number;\n    readonly y: number;\n    toJSON(): any;\n}\n\ndeclare var DOMRectReadOnly: {\n    prototype: DOMRectReadOnly;\n    new(x?: number, y?: number, width?: number, height?: number): DOMRectReadOnly;\n    fromRect(other?: DOMRectInit): DOMRectReadOnly;\n};\n\n/** A type returned by some APIs which contains a list of DOMString (strings). */\ninterface DOMStringList {\n    /**\n     * Returns the number of strings in strings.\n     */\n    readonly length: number;\n    /**\n     * Returns true if strings contains string, and false otherwise.\n     */\n    contains(string: string): boolean;\n    /**\n     * Returns the string with index index from strings.\n     */\n    item(index: number): string | null;\n    [index: number]: string;\n}\n\ndeclare var DOMStringList: {\n    prototype: DOMStringList;\n    new(): DOMStringList;\n};\n\ninterface DedicatedWorkerGlobalScopeEventMap extends WorkerGlobalScopeEventMap {\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n}\n\n/** (the Worker global scope) is accessible through the self keyword. Some additional global functions, namespaces objects, and constructors, not typically associated with the worker global scope, but available on it, are listed in the JavaScript Reference. See also: Functions available to workers. */\ninterface DedicatedWorkerGlobalScope extends WorkerGlobalScope, AnimationFrameProvider {\n    /**\n     * Returns dedicatedWorkerGlobal\'s name, i.e. the value given to the Worker constructor. Primarily useful for debugging.\n     */\n    readonly name: string;\n    onmessage: ((this: DedicatedWorkerGlobalScope, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: DedicatedWorkerGlobalScope, ev: MessageEvent) => any) | null;\n    /**\n     * Aborts dedicatedWorkerGlobal.\n     */\n    close(): void;\n    /**\n     * Clones message and transmits it to the Worker object associated with dedicatedWorkerGlobal. transfer can be passed as a list of objects that are to be transferred rather than cloned.\n     */\n    postMessage(message: any, transfer: Transferable[]): void;\n    postMessage(message: any, options?: PostMessageOptions): void;\n    addEventListener<K extends keyof DedicatedWorkerGlobalScopeEventMap>(type: K, listener: (this: DedicatedWorkerGlobalScope, ev: DedicatedWorkerGlobalScopeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof DedicatedWorkerGlobalScopeEventMap>(type: K, listener: (this: DedicatedWorkerGlobalScope, ev: DedicatedWorkerGlobalScopeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var DedicatedWorkerGlobalScope: {\n    prototype: DedicatedWorkerGlobalScope;\n    new(): DedicatedWorkerGlobalScope;\n};\n\ninterface EXT_blend_minmax {\n    readonly MAX_EXT: GLenum;\n    readonly MIN_EXT: GLenum;\n}\n\ninterface EXT_color_buffer_float {\n}\n\ninterface EXT_color_buffer_half_float {\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT: GLenum;\n    readonly RGB16F_EXT: GLenum;\n    readonly RGBA16F_EXT: GLenum;\n    readonly UNSIGNED_NORMALIZED_EXT: GLenum;\n}\n\ninterface EXT_float_blend {\n}\n\n/** The EXT_frag_depth extension is part of the WebGL API and enables to set a depth value of a fragment from within the fragment shader. */\ninterface EXT_frag_depth {\n}\n\ninterface EXT_sRGB {\n    readonly FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT: GLenum;\n    readonly SRGB8_ALPHA8_EXT: GLenum;\n    readonly SRGB_ALPHA_EXT: GLenum;\n    readonly SRGB_EXT: GLenum;\n}\n\ninterface EXT_shader_texture_lod {\n}\n\ninterface EXT_texture_compression_rgtc {\n    readonly COMPRESSED_RED_GREEN_RGTC2_EXT: GLenum;\n    readonly COMPRESSED_RED_RGTC1_EXT: GLenum;\n    readonly COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT: GLenum;\n    readonly COMPRESSED_SIGNED_RED_RGTC1_EXT: GLenum;\n}\n\n/** The EXT_texture_filter_anisotropic extension is part of the WebGL API and exposes two constants for anisotropic filtering (AF). */\ninterface EXT_texture_filter_anisotropic {\n    readonly MAX_TEXTURE_MAX_ANISOTROPY_EXT: GLenum;\n    readonly TEXTURE_MAX_ANISOTROPY_EXT: GLenum;\n}\n\n/** Events providing information related to errors in scripts or in files. */\ninterface ErrorEvent extends Event {\n    readonly colno: number;\n    readonly error: any;\n    readonly filename: string;\n    readonly lineno: number;\n    readonly message: string;\n}\n\ndeclare var ErrorEvent: {\n    prototype: ErrorEvent;\n    new(type: string, eventInitDict?: ErrorEventInit): ErrorEvent;\n};\n\n/** An event which takes place in the DOM. */\ninterface Event {\n    /**\n     * Returns true or false depending on how event was initialized. True if event goes through its target\'s ancestors in reverse tree order, and false otherwise.\n     */\n    readonly bubbles: boolean;\n    cancelBubble: boolean;\n    /**\n     * Returns true or false depending on how event was initialized. Its return value does not always carry meaning, but true can indicate that part of the operation during which event was dispatched, can be canceled by invoking the preventDefault() method.\n     */\n    readonly cancelable: boolean;\n    /**\n     * Returns true or false depending on how event was initialized. True if event invokes listeners past a ShadowRoot node that is the root of its target, and false otherwise.\n     */\n    readonly composed: boolean;\n    /**\n     * Returns the object whose event listener\'s callback is currently being invoked.\n     */\n    readonly currentTarget: EventTarget | null;\n    /**\n     * Returns true if preventDefault() was invoked successfully to indicate cancelation, and false otherwise.\n     */\n    readonly defaultPrevented: boolean;\n    /**\n     * Returns the event\'s phase, which is one of NONE, CAPTURING_PHASE, AT_TARGET, and BUBBLING_PHASE.\n     */\n    readonly eventPhase: number;\n    /**\n     * Returns true if event was dispatched by the user agent, and false otherwise.\n     */\n    readonly isTrusted: boolean;\n    /** @deprecated */\n    returnValue: boolean;\n    /** @deprecated */\n    readonly srcElement: EventTarget | null;\n    /**\n     * Returns the object to which event is dispatched (its target).\n     */\n    readonly target: EventTarget | null;\n    /**\n     * Returns the event\'s timestamp as the number of milliseconds measured relative to the time origin.\n     */\n    readonly timeStamp: DOMHighResTimeStamp;\n    /**\n     * Returns the type of event, e.g. "click", "hashchange", or "submit".\n     */\n    readonly type: string;\n    /**\n     * Returns the invocation target objects of event\'s path (objects on which listeners will be invoked), except for any nodes in shadow trees of which the shadow root\'s mode is "closed" that are not reachable from event\'s currentTarget.\n     */\n    composedPath(): EventTarget[];\n    /** @deprecated */\n    initEvent(type: string, bubbles?: boolean, cancelable?: boolean): void;\n    /**\n     * If invoked when the cancelable attribute value is true, and while executing a listener for the event with passive set to false, signals to the operation that caused event to be dispatched that it needs to be canceled.\n     */\n    preventDefault(): void;\n    /**\n     * Invoking this method prevents event from reaching any registered event listeners after the current one finishes running and, when dispatched in a tree, also prevents event from reaching any other objects.\n     */\n    stopImmediatePropagation(): void;\n    /**\n     * When dispatched in a tree, invoking this method prevents event from reaching any objects other than the current object.\n     */\n    stopPropagation(): void;\n    readonly AT_TARGET: number;\n    readonly BUBBLING_PHASE: number;\n    readonly CAPTURING_PHASE: number;\n    readonly NONE: number;\n}\n\ndeclare var Event: {\n    prototype: Event;\n    new(type: string, eventInitDict?: EventInit): Event;\n    readonly AT_TARGET: number;\n    readonly BUBBLING_PHASE: number;\n    readonly CAPTURING_PHASE: number;\n    readonly NONE: number;\n};\n\ninterface EventListener {\n    (evt: Event): void;\n}\n\ninterface EventListenerObject {\n    handleEvent(object: Event): void;\n}\n\ninterface EventSourceEventMap {\n    "error": Event;\n    "message": MessageEvent;\n    "open": Event;\n}\n\ninterface EventSource extends EventTarget {\n    onerror: ((this: EventSource, ev: Event) => any) | null;\n    onmessage: ((this: EventSource, ev: MessageEvent) => any) | null;\n    onopen: ((this: EventSource, ev: Event) => any) | null;\n    /**\n     * Returns the state of this EventSource object\'s connection. It can have the values described below.\n     */\n    readonly readyState: number;\n    /**\n     * Returns the URL providing the event stream.\n     */\n    readonly url: string;\n    /**\n     * Returns true if the credentials mode for connection requests to the URL providing the event stream is set to "include", and false otherwise.\n     */\n    readonly withCredentials: boolean;\n    /**\n     * Aborts any instances of the fetch algorithm started for this EventSource object, and sets the readyState attribute to CLOSED.\n     */\n    close(): void;\n    readonly CLOSED: number;\n    readonly CONNECTING: number;\n    readonly OPEN: number;\n    addEventListener<K extends keyof EventSourceEventMap>(type: K, listener: (this: EventSource, ev: EventSourceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof EventSourceEventMap>(type: K, listener: (this: EventSource, ev: EventSourceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var EventSource: {\n    prototype: EventSource;\n    new(url: string | URL, eventSourceInitDict?: EventSourceInit): EventSource;\n    readonly CLOSED: number;\n    readonly CONNECTING: number;\n    readonly OPEN: number;\n};\n\n/** EventTarget is a DOM interface implemented by objects that can receive events and may have listeners for them. */\ninterface EventTarget {\n    /**\n     * Appends an event listener for events whose type attribute value is type. The callback argument sets the callback that will be invoked when the event is dispatched.\n     *\n     * The options argument sets listener-specific options. For compatibility this can be a boolean, in which case the method behaves exactly as if the value was specified as options\'s capture.\n     *\n     * When set to true, options\'s capture prevents callback from being invoked when the event\'s eventPhase attribute value is BUBBLING_PHASE. When false (or not present), callback will not be invoked when event\'s eventPhase attribute value is CAPTURING_PHASE. Either way, callback will be invoked if event\'s eventPhase attribute value is AT_TARGET.\n     *\n     * When set to true, options\'s passive indicates that the callback will not cancel the event by invoking preventDefault(). This is used to enable performance optimizations described in § 2.8 Observing event listeners.\n     *\n     * When set to true, options\'s once indicates that the callback will only be invoked once after which the event listener will be removed.\n     *\n     * If an AbortSignal is passed for options\'s signal, then the event listener will be removed when signal is aborted.\n     *\n     * The event listener is appended to target\'s event listener list and is not appended if it has the same type, callback, and capture.\n     */\n    addEventListener(type: string, callback: EventListenerOrEventListenerObject | null, options?: AddEventListenerOptions | boolean): void;\n    /**\n     * Dispatches a synthetic event event to target and returns true if either event\'s cancelable attribute value is false or its preventDefault() method was not invoked, and false otherwise.\n     */\n    dispatchEvent(event: Event): boolean;\n    /**\n     * Removes the event listener in target\'s event listener list with the same type, callback, and options.\n     */\n    removeEventListener(type: string, callback: EventListenerOrEventListenerObject | null, options?: EventListenerOptions | boolean): void;\n}\n\ndeclare var EventTarget: {\n    prototype: EventTarget;\n    new(): EventTarget;\n};\n\n/** Extends the lifetime of the install and activate events dispatched on the global scope as part of the service worker lifecycle. This ensures that any functional events (like FetchEvent) are not dispatched until it upgrades database schemas and deletes the outdated cache entries. */\ninterface ExtendableEvent extends Event {\n    waitUntil(f: any): void;\n}\n\ndeclare var ExtendableEvent: {\n    prototype: ExtendableEvent;\n    new(type: string, eventInitDict?: ExtendableEventInit): ExtendableEvent;\n};\n\n/** This ServiceWorker API interface represents the event object of a message event fired on a service worker (when a channel message is received on the ServiceWorkerGlobalScope from another context) — extends the lifetime of such events. */\ninterface ExtendableMessageEvent extends ExtendableEvent {\n    readonly data: any;\n    readonly lastEventId: string;\n    readonly origin: string;\n    readonly ports: ReadonlyArray<MessagePort>;\n    readonly source: Client | ServiceWorker | MessagePort | null;\n}\n\ndeclare var ExtendableMessageEvent: {\n    prototype: ExtendableMessageEvent;\n    new(type: string, eventInitDict?: ExtendableMessageEventInit): ExtendableMessageEvent;\n};\n\n/** This is the event type for fetch events dispatched on the service worker global scope. It contains information about the fetch, including the request and how the receiver will treat the response. It provides the event.respondWith() method, which allows us to provide a response to this fetch. */\ninterface FetchEvent extends ExtendableEvent {\n    readonly clientId: string;\n    readonly handled: Promise<undefined>;\n    readonly request: Request;\n    readonly resultingClientId: string;\n    respondWith(r: Response | PromiseLike<Response>): void;\n}\n\ndeclare var FetchEvent: {\n    prototype: FetchEvent;\n    new(type: string, eventInitDict: FetchEventInit): FetchEvent;\n};\n\n/** Provides information about files and allows JavaScript in a web page to access their content. */\ninterface File extends Blob {\n    readonly lastModified: number;\n    readonly name: string;\n    readonly webkitRelativePath: string;\n}\n\ndeclare var File: {\n    prototype: File;\n    new(fileBits: BlobPart[], fileName: string, options?: FilePropertyBag): File;\n};\n\n/** An object of this type is returned by the files property of the HTML <input> element; this lets you access the list of files selected with the <input type="file"> element. It\'s also used for a list of files dropped into web content when using the drag and drop API; see the DataTransfer object for details on this usage. */\ninterface FileList {\n    readonly length: number;\n    item(index: number): File | null;\n    [index: number]: File;\n}\n\ndeclare var FileList: {\n    prototype: FileList;\n    new(): FileList;\n};\n\ninterface FileReaderEventMap {\n    "abort": ProgressEvent<FileReader>;\n    "error": ProgressEvent<FileReader>;\n    "load": ProgressEvent<FileReader>;\n    "loadend": ProgressEvent<FileReader>;\n    "loadstart": ProgressEvent<FileReader>;\n    "progress": ProgressEvent<FileReader>;\n}\n\n/** Lets web applications asynchronously read the contents of files (or raw data buffers) stored on the user\'s computer, using File or Blob objects to specify the file or data to read. */\ninterface FileReader extends EventTarget {\n    readonly error: DOMException | null;\n    onabort: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onerror: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onload: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onloadend: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onloadstart: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    onprogress: ((this: FileReader, ev: ProgressEvent<FileReader>) => any) | null;\n    readonly readyState: number;\n    readonly result: string | ArrayBuffer | null;\n    abort(): void;\n    readAsArrayBuffer(blob: Blob): void;\n    readAsBinaryString(blob: Blob): void;\n    readAsDataURL(blob: Blob): void;\n    readAsText(blob: Blob, encoding?: string): void;\n    readonly DONE: number;\n    readonly EMPTY: number;\n    readonly LOADING: number;\n    addEventListener<K extends keyof FileReaderEventMap>(type: K, listener: (this: FileReader, ev: FileReaderEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof FileReaderEventMap>(type: K, listener: (this: FileReader, ev: FileReaderEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var FileReader: {\n    prototype: FileReader;\n    new(): FileReader;\n    readonly DONE: number;\n    readonly EMPTY: number;\n    readonly LOADING: number;\n};\n\n/** Allows to read File or Blob objects in a synchronous way. */\ninterface FileReaderSync {\n    readAsArrayBuffer(blob: Blob): ArrayBuffer;\n    /** @deprecated */\n    readAsBinaryString(blob: Blob): string;\n    readAsDataURL(blob: Blob): string;\n    readAsText(blob: Blob, encoding?: string): string;\n}\n\ndeclare var FileReaderSync: {\n    prototype: FileReaderSync;\n    new(): FileReaderSync;\n};\n\ninterface FontFace {\n    ascentOverride: string;\n    descentOverride: string;\n    display: string;\n    family: string;\n    featureSettings: string;\n    lineGapOverride: string;\n    readonly loaded: Promise<FontFace>;\n    readonly status: FontFaceLoadStatus;\n    stretch: string;\n    style: string;\n    unicodeRange: string;\n    variant: string;\n    variationSettings: string;\n    weight: string;\n    load(): Promise<FontFace>;\n}\n\ndeclare var FontFace: {\n    prototype: FontFace;\n    new(family: string, source: string | BinaryData, descriptors?: FontFaceDescriptors): FontFace;\n};\n\ninterface FontFaceSetEventMap {\n    "loading": Event;\n    "loadingdone": Event;\n    "loadingerror": Event;\n}\n\ninterface FontFaceSet extends EventTarget {\n    onloading: ((this: FontFaceSet, ev: Event) => any) | null;\n    onloadingdone: ((this: FontFaceSet, ev: Event) => any) | null;\n    onloadingerror: ((this: FontFaceSet, ev: Event) => any) | null;\n    readonly ready: Promise<FontFaceSet>;\n    readonly status: FontFaceSetLoadStatus;\n    check(font: string, text?: string): boolean;\n    load(font: string, text?: string): Promise<FontFace[]>;\n    forEach(callbackfn: (value: FontFace, key: FontFace, parent: FontFaceSet) => void, thisArg?: any): void;\n    addEventListener<K extends keyof FontFaceSetEventMap>(type: K, listener: (this: FontFaceSet, ev: FontFaceSetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof FontFaceSetEventMap>(type: K, listener: (this: FontFaceSet, ev: FontFaceSetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var FontFaceSet: {\n    prototype: FontFaceSet;\n    new(initialFaces: FontFace[]): FontFaceSet;\n};\n\ninterface FontFaceSetLoadEvent extends Event {\n    readonly fontfaces: ReadonlyArray<FontFace>;\n}\n\ndeclare var FontFaceSetLoadEvent: {\n    prototype: FontFaceSetLoadEvent;\n    new(type: string, eventInitDict?: FontFaceSetLoadEventInit): FontFaceSetLoadEvent;\n};\n\ninterface FontFaceSource {\n    readonly fonts: FontFaceSet;\n}\n\n/** Provides a way to easily construct a set of key/value pairs representing form fields and their values, which can then be easily sent using the XMLHttpRequest.send() method. It uses the same format a form would use if the encoding type were set to "multipart/form-data". */\ninterface FormData {\n    append(name: string, value: string | Blob, fileName?: string): void;\n    delete(name: string): void;\n    get(name: string): FormDataEntryValue | null;\n    getAll(name: string): FormDataEntryValue[];\n    has(name: string): boolean;\n    set(name: string, value: string | Blob, fileName?: string): void;\n    forEach(callbackfn: (value: FormDataEntryValue, key: string, parent: FormData) => void, thisArg?: any): void;\n}\n\ndeclare var FormData: {\n    prototype: FormData;\n    new(): FormData;\n};\n\ninterface GenericTransformStream {\n    readonly readable: ReadableStream;\n    readonly writable: WritableStream;\n}\n\n/** This Fetch API interface allows you to perform various actions on HTTP request and response headers. These actions include retrieving, setting, adding to, and removing. A Headers object has an associated header list, which is initially empty and consists of zero or more name and value pairs.  You can add to this using methods like append() (see Examples.) In all methods of this interface, header names are matched by case-insensitive byte sequence. */\ninterface Headers {\n    append(name: string, value: string): void;\n    delete(name: string): void;\n    get(name: string): string | null;\n    has(name: string): boolean;\n    set(name: string, value: string): void;\n    forEach(callbackfn: (value: string, key: string, parent: Headers) => void, thisArg?: any): void;\n}\n\ndeclare var Headers: {\n    prototype: Headers;\n    new(init?: HeadersInit): Headers;\n};\n\n/** This IndexedDB API interface represents a cursor for traversing or iterating over multiple records in a database. */\ninterface IDBCursor {\n    /**\n     * Returns the direction ("next", "nextunique", "prev" or "prevunique") of the cursor.\n     */\n    readonly direction: IDBCursorDirection;\n    /**\n     * Returns the key of the cursor. Throws a "InvalidStateError" DOMException if the cursor is advancing or is finished.\n     */\n    readonly key: IDBValidKey;\n    /**\n     * Returns the effective key of the cursor. Throws a "InvalidStateError" DOMException if the cursor is advancing or is finished.\n     */\n    readonly primaryKey: IDBValidKey;\n    readonly request: IDBRequest;\n    /**\n     * Returns the IDBObjectStore or IDBIndex the cursor was opened from.\n     */\n    readonly source: IDBObjectStore | IDBIndex;\n    /**\n     * Advances the cursor through the next count records in range.\n     */\n    advance(count: number): void;\n    /**\n     * Advances the cursor to the next record in range.\n     */\n    continue(key?: IDBValidKey): void;\n    /**\n     * Advances the cursor to the next record in range matching or after key and primaryKey. Throws an "InvalidAccessError" DOMException if the source is not an index.\n     */\n    continuePrimaryKey(key: IDBValidKey, primaryKey: IDBValidKey): void;\n    /**\n     * Delete the record pointed at by the cursor with a new value.\n     *\n     * If successful, request\'s result will be undefined.\n     */\n    delete(): IDBRequest<undefined>;\n    /**\n     * Updated the record pointed at by the cursor with a new value.\n     *\n     * Throws a "DataError" DOMException if the effective object store uses in-line keys and the key would have changed.\n     *\n     * If successful, request\'s result will be the record\'s key.\n     */\n    update(value: any): IDBRequest<IDBValidKey>;\n}\n\ndeclare var IDBCursor: {\n    prototype: IDBCursor;\n    new(): IDBCursor;\n};\n\n/** This IndexedDB API interface represents a cursor for traversing or iterating over multiple records in a database. It is the same as the IDBCursor, except that it includes the value property. */\ninterface IDBCursorWithValue extends IDBCursor {\n    /**\n     * Returns the cursor\'s current value.\n     */\n    readonly value: any;\n}\n\ndeclare var IDBCursorWithValue: {\n    prototype: IDBCursorWithValue;\n    new(): IDBCursorWithValue;\n};\n\ninterface IDBDatabaseEventMap {\n    "abort": Event;\n    "close": Event;\n    "error": Event;\n    "versionchange": IDBVersionChangeEvent;\n}\n\n/** This IndexedDB API interface provides a connection to a database; you can use an IDBDatabase object to open a transaction on your database then create, manipulate, and delete objects (data) in that database. The interface provides the only way to get and manage versions of the database. */\ninterface IDBDatabase extends EventTarget {\n    /**\n     * Returns the name of the database.\n     */\n    readonly name: string;\n    /**\n     * Returns a list of the names of object stores in the database.\n     */\n    readonly objectStoreNames: DOMStringList;\n    onabort: ((this: IDBDatabase, ev: Event) => any) | null;\n    onclose: ((this: IDBDatabase, ev: Event) => any) | null;\n    onerror: ((this: IDBDatabase, ev: Event) => any) | null;\n    onversionchange: ((this: IDBDatabase, ev: IDBVersionChangeEvent) => any) | null;\n    /**\n     * Returns the version of the database.\n     */\n    readonly version: number;\n    /**\n     * Closes the connection once all running transactions have finished.\n     */\n    close(): void;\n    /**\n     * Creates a new object store with the given name and options and returns a new IDBObjectStore.\n     *\n     * Throws a "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    createObjectStore(name: string, options?: IDBObjectStoreParameters): IDBObjectStore;\n    /**\n     * Deletes the object store with the given name.\n     *\n     * Throws a "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    deleteObjectStore(name: string): void;\n    /**\n     * Returns a new transaction with the given mode ("readonly" or "readwrite") and scope which can be a single object store name or an array of names.\n     */\n    transaction(storeNames: string | string[], mode?: IDBTransactionMode): IDBTransaction;\n    addEventListener<K extends keyof IDBDatabaseEventMap>(type: K, listener: (this: IDBDatabase, ev: IDBDatabaseEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof IDBDatabaseEventMap>(type: K, listener: (this: IDBDatabase, ev: IDBDatabaseEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var IDBDatabase: {\n    prototype: IDBDatabase;\n    new(): IDBDatabase;\n};\n\n/** In the following code snippet, we make a request to open a database, and include handlers for the success and error cases. For a full working example, see our To-do Notifications app (view example live.) */\ninterface IDBFactory {\n    /**\n     * Compares two values as keys. Returns -1 if key1 precedes key2, 1 if key2 precedes key1, and 0 if the keys are equal.\n     *\n     * Throws a "DataError" DOMException if either input is not a valid key.\n     */\n    cmp(first: any, second: any): number;\n    databases(): Promise<IDBDatabaseInfo[]>;\n    /**\n     * Attempts to delete the named database. If the database already exists and there are open connections that don\'t close in response to a versionchange event, the request will be blocked until all they close. If the request is successful request\'s result will be null.\n     */\n    deleteDatabase(name: string): IDBOpenDBRequest;\n    /**\n     * Attempts to open a connection to the named database with the current version, or 1 if it does not already exist. If the request is successful request\'s result will be the connection.\n     */\n    open(name: string, version?: number): IDBOpenDBRequest;\n}\n\ndeclare var IDBFactory: {\n    prototype: IDBFactory;\n    new(): IDBFactory;\n};\n\n/** IDBIndex interface of the IndexedDB API provides asynchronous access to an index in a database. An index is a kind of object store for looking up records in another object store, called the referenced object store. You use this interface to retrieve data. */\ninterface IDBIndex {\n    readonly keyPath: string | string[];\n    readonly multiEntry: boolean;\n    /**\n     * Returns the name of the index.\n     */\n    name: string;\n    /**\n     * Returns the IDBObjectStore the index belongs to.\n     */\n    readonly objectStore: IDBObjectStore;\n    readonly unique: boolean;\n    /**\n     * Retrieves the number of records matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the count.\n     */\n    count(query?: IDBValidKey | IDBKeyRange): IDBRequest<number>;\n    /**\n     * Retrieves the value of the first record matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the value, or undefined if there was no matching record.\n     */\n    get(query: IDBValidKey | IDBKeyRange): IDBRequest<any>;\n    /**\n     * Retrieves the values of the records matching the given key or key range in query (up to count if given).\n     *\n     * If successful, request\'s result will be an Array of the values.\n     */\n    getAll(query?: IDBValidKey | IDBKeyRange | null, count?: number): IDBRequest<any[]>;\n    /**\n     * Retrieves the keys of records matching the given key or key range in query (up to count if given).\n     *\n     * If successful, request\'s result will be an Array of the keys.\n     */\n    getAllKeys(query?: IDBValidKey | IDBKeyRange | null, count?: number): IDBRequest<IDBValidKey[]>;\n    /**\n     * Retrieves the key of the first record matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the key, or undefined if there was no matching record.\n     */\n    getKey(query: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey | undefined>;\n    /**\n     * Opens a cursor over the records matching query, ordered by direction. If query is null, all records in index are matched.\n     *\n     * If successful, request\'s result will be an IDBCursorWithValue, or null if there were no matching records.\n     */\n    openCursor(query?: IDBValidKey | IDBKeyRange | null, direction?: IDBCursorDirection): IDBRequest<IDBCursorWithValue | null>;\n    /**\n     * Opens a cursor with key only flag set over the records matching query, ordered by direction. If query is null, all records in index are matched.\n     *\n     * If successful, request\'s result will be an IDBCursor, or null if there were no matching records.\n     */\n    openKeyCursor(query?: IDBValidKey | IDBKeyRange | null, direction?: IDBCursorDirection): IDBRequest<IDBCursor | null>;\n}\n\ndeclare var IDBIndex: {\n    prototype: IDBIndex;\n    new(): IDBIndex;\n};\n\n/** A key range can be a single value or a range with upper and lower bounds or endpoints. If the key range has both upper and lower bounds, then it is bounded; if it has no bounds, it is unbounded. A bounded key range can either be open (the endpoints are excluded) or closed (the endpoints are included). To retrieve all keys within a certain range, you can use the following code constructs: */\ninterface IDBKeyRange {\n    /**\n     * Returns lower bound, or undefined if none.\n     */\n    readonly lower: any;\n    /**\n     * Returns true if the lower open flag is set, and false otherwise.\n     */\n    readonly lowerOpen: boolean;\n    /**\n     * Returns upper bound, or undefined if none.\n     */\n    readonly upper: any;\n    /**\n     * Returns true if the upper open flag is set, and false otherwise.\n     */\n    readonly upperOpen: boolean;\n    /**\n     * Returns true if key is included in the range, and false otherwise.\n     */\n    includes(key: any): boolean;\n}\n\ndeclare var IDBKeyRange: {\n    prototype: IDBKeyRange;\n    new(): IDBKeyRange;\n    /**\n     * Returns a new IDBKeyRange spanning from lower to upper. If lowerOpen is true, lower is not included in the range. If upperOpen is true, upper is not included in the range.\n     */\n    bound(lower: any, upper: any, lowerOpen?: boolean, upperOpen?: boolean): IDBKeyRange;\n    /**\n     * Returns a new IDBKeyRange starting at key with no upper bound. If open is true, key is not included in the range.\n     */\n    lowerBound(lower: any, open?: boolean): IDBKeyRange;\n    /**\n     * Returns a new IDBKeyRange spanning only key.\n     */\n    only(value: any): IDBKeyRange;\n    /**\n     * Returns a new IDBKeyRange with no lower bound and ending at key. If open is true, key is not included in the range.\n     */\n    upperBound(upper: any, open?: boolean): IDBKeyRange;\n};\n\n/** This example shows a variety of different uses of object stores, from updating the data structure with IDBObjectStore.createIndex inside an onupgradeneeded function, to adding a new item to our object store with IDBObjectStore.add. For a full working example, see our To-do Notifications app (view example live.) */\ninterface IDBObjectStore {\n    /**\n     * Returns true if the store has a key generator, and false otherwise.\n     */\n    readonly autoIncrement: boolean;\n    /**\n     * Returns a list of the names of indexes in the store.\n     */\n    readonly indexNames: DOMStringList;\n    /**\n     * Returns the key path of the store, or null if none.\n     */\n    readonly keyPath: string | string[];\n    /**\n     * Returns the name of the store.\n     */\n    name: string;\n    /**\n     * Returns the associated transaction.\n     */\n    readonly transaction: IDBTransaction;\n    /**\n     * Adds or updates a record in store with the given value and key.\n     *\n     * If the store uses in-line keys and key is specified a "DataError" DOMException will be thrown.\n     *\n     * If put() is used, any existing record with the key will be replaced. If add() is used, and if a record with the key already exists the request will fail, with request\'s error set to a "ConstraintError" DOMException.\n     *\n     * If successful, request\'s result will be the record\'s key.\n     */\n    add(value: any, key?: IDBValidKey): IDBRequest<IDBValidKey>;\n    /**\n     * Deletes all records in store.\n     *\n     * If successful, request\'s result will be undefined.\n     */\n    clear(): IDBRequest<undefined>;\n    /**\n     * Retrieves the number of records matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the count.\n     */\n    count(query?: IDBValidKey | IDBKeyRange): IDBRequest<number>;\n    /**\n     * Creates a new index in store with the given name, keyPath and options and returns a new IDBIndex. If the keyPath and options define constraints that cannot be satisfied with the data already in store the upgrade transaction will abort with a "ConstraintError" DOMException.\n     *\n     * Throws an "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    createIndex(name: string, keyPath: string | string[], options?: IDBIndexParameters): IDBIndex;\n    /**\n     * Deletes records in store with the given key or in the given key range in query.\n     *\n     * If successful, request\'s result will be undefined.\n     */\n    delete(query: IDBValidKey | IDBKeyRange): IDBRequest<undefined>;\n    /**\n     * Deletes the index in store with the given name.\n     *\n     * Throws an "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    deleteIndex(name: string): void;\n    /**\n     * Retrieves the value of the first record matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the value, or undefined if there was no matching record.\n     */\n    get(query: IDBValidKey | IDBKeyRange): IDBRequest<any>;\n    /**\n     * Retrieves the values of the records matching the given key or key range in query (up to count if given).\n     *\n     * If successful, request\'s result will be an Array of the values.\n     */\n    getAll(query?: IDBValidKey | IDBKeyRange | null, count?: number): IDBRequest<any[]>;\n    /**\n     * Retrieves the keys of records matching the given key or key range in query (up to count if given).\n     *\n     * If successful, request\'s result will be an Array of the keys.\n     */\n    getAllKeys(query?: IDBValidKey | IDBKeyRange | null, count?: number): IDBRequest<IDBValidKey[]>;\n    /**\n     * Retrieves the key of the first record matching the given key or key range in query.\n     *\n     * If successful, request\'s result will be the key, or undefined if there was no matching record.\n     */\n    getKey(query: IDBValidKey | IDBKeyRange): IDBRequest<IDBValidKey | undefined>;\n    index(name: string): IDBIndex;\n    /**\n     * Opens a cursor over the records matching query, ordered by direction. If query is null, all records in store are matched.\n     *\n     * If successful, request\'s result will be an IDBCursorWithValue pointing at the first matching record, or null if there were no matching records.\n     */\n    openCursor(query?: IDBValidKey | IDBKeyRange | null, direction?: IDBCursorDirection): IDBRequest<IDBCursorWithValue | null>;\n    /**\n     * Opens a cursor with key only flag set over the records matching query, ordered by direction. If query is null, all records in store are matched.\n     *\n     * If successful, request\'s result will be an IDBCursor pointing at the first matching record, or null if there were no matching records.\n     */\n    openKeyCursor(query?: IDBValidKey | IDBKeyRange | null, direction?: IDBCursorDirection): IDBRequest<IDBCursor | null>;\n    /**\n     * Adds or updates a record in store with the given value and key.\n     *\n     * If the store uses in-line keys and key is specified a "DataError" DOMException will be thrown.\n     *\n     * If put() is used, any existing record with the key will be replaced. If add() is used, and if a record with the key already exists the request will fail, with request\'s error set to a "ConstraintError" DOMException.\n     *\n     * If successful, request\'s result will be the record\'s key.\n     */\n    put(value: any, key?: IDBValidKey): IDBRequest<IDBValidKey>;\n}\n\ndeclare var IDBObjectStore: {\n    prototype: IDBObjectStore;\n    new(): IDBObjectStore;\n};\n\ninterface IDBOpenDBRequestEventMap extends IDBRequestEventMap {\n    "blocked": Event;\n    "upgradeneeded": IDBVersionChangeEvent;\n}\n\n/** Also inherits methods from its parents IDBRequest and EventTarget. */\ninterface IDBOpenDBRequest extends IDBRequest<IDBDatabase> {\n    onblocked: ((this: IDBOpenDBRequest, ev: Event) => any) | null;\n    onupgradeneeded: ((this: IDBOpenDBRequest, ev: IDBVersionChangeEvent) => any) | null;\n    addEventListener<K extends keyof IDBOpenDBRequestEventMap>(type: K, listener: (this: IDBOpenDBRequest, ev: IDBOpenDBRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof IDBOpenDBRequestEventMap>(type: K, listener: (this: IDBOpenDBRequest, ev: IDBOpenDBRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var IDBOpenDBRequest: {\n    prototype: IDBOpenDBRequest;\n    new(): IDBOpenDBRequest;\n};\n\ninterface IDBRequestEventMap {\n    "error": Event;\n    "success": Event;\n}\n\n/** The request object does not initially contain any information about the result of the operation, but once information becomes available, an event is fired on the request, and the information becomes available through the properties of the IDBRequest instance. */\ninterface IDBRequest<T = any> extends EventTarget {\n    /**\n     * When a request is completed, returns the error (a DOMException), or null if the request succeeded. Throws a "InvalidStateError" DOMException if the request is still pending.\n     */\n    readonly error: DOMException | null;\n    onerror: ((this: IDBRequest<T>, ev: Event) => any) | null;\n    onsuccess: ((this: IDBRequest<T>, ev: Event) => any) | null;\n    /**\n     * Returns "pending" until a request is complete, then returns "done".\n     */\n    readonly readyState: IDBRequestReadyState;\n    /**\n     * When a request is completed, returns the result, or undefined if the request failed. Throws a "InvalidStateError" DOMException if the request is still pending.\n     */\n    readonly result: T;\n    /**\n     * Returns the IDBObjectStore, IDBIndex, or IDBCursor the request was made against, or null if is was an open request.\n     */\n    readonly source: IDBObjectStore | IDBIndex | IDBCursor;\n    /**\n     * Returns the IDBTransaction the request was made within. If this as an open request, then it returns an upgrade transaction while it is running, or null otherwise.\n     */\n    readonly transaction: IDBTransaction | null;\n    addEventListener<K extends keyof IDBRequestEventMap>(type: K, listener: (this: IDBRequest<T>, ev: IDBRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof IDBRequestEventMap>(type: K, listener: (this: IDBRequest<T>, ev: IDBRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var IDBRequest: {\n    prototype: IDBRequest;\n    new(): IDBRequest;\n};\n\ninterface IDBTransactionEventMap {\n    "abort": Event;\n    "complete": Event;\n    "error": Event;\n}\n\ninterface IDBTransaction extends EventTarget {\n    /**\n     * Returns the transaction\'s connection.\n     */\n    readonly db: IDBDatabase;\n    /**\n     * If the transaction was aborted, returns the error (a DOMException) providing the reason.\n     */\n    readonly error: DOMException | null;\n    /**\n     * Returns the mode the transaction was created with ("readonly" or "readwrite"), or "versionchange" for an upgrade transaction.\n     */\n    readonly mode: IDBTransactionMode;\n    /**\n     * Returns a list of the names of object stores in the transaction\'s scope. For an upgrade transaction this is all object stores in the database.\n     */\n    readonly objectStoreNames: DOMStringList;\n    onabort: ((this: IDBTransaction, ev: Event) => any) | null;\n    oncomplete: ((this: IDBTransaction, ev: Event) => any) | null;\n    onerror: ((this: IDBTransaction, ev: Event) => any) | null;\n    /**\n     * Aborts the transaction. All pending requests will fail with a "AbortError" DOMException and all changes made to the database will be reverted.\n     */\n    abort(): void;\n    commit(): void;\n    /**\n     * Returns an IDBObjectStore in the transaction\'s scope.\n     */\n    objectStore(name: string): IDBObjectStore;\n    addEventListener<K extends keyof IDBTransactionEventMap>(type: K, listener: (this: IDBTransaction, ev: IDBTransactionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof IDBTransactionEventMap>(type: K, listener: (this: IDBTransaction, ev: IDBTransactionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var IDBTransaction: {\n    prototype: IDBTransaction;\n    new(): IDBTransaction;\n};\n\n/** This IndexedDB API interface indicates that the version of the database has changed, as the result of an IDBOpenDBRequest.onupgradeneeded event handler function. */\ninterface IDBVersionChangeEvent extends Event {\n    readonly newVersion: number | null;\n    readonly oldVersion: number;\n}\n\ndeclare var IDBVersionChangeEvent: {\n    prototype: IDBVersionChangeEvent;\n    new(type: string, eventInitDict?: IDBVersionChangeEventInit): IDBVersionChangeEvent;\n};\n\ninterface ImageBitmap {\n    /**\n     * Returns the intrinsic height of the image, in CSS pixels.\n     */\n    readonly height: number;\n    /**\n     * Returns the intrinsic width of the image, in CSS pixels.\n     */\n    readonly width: number;\n    /**\n     * Releases imageBitmap\'s underlying bitmap data.\n     */\n    close(): void;\n}\n\ndeclare var ImageBitmap: {\n    prototype: ImageBitmap;\n    new(): ImageBitmap;\n};\n\ninterface ImageBitmapRenderingContext {\n    /**\n     * Transfers the underlying bitmap data from imageBitmap to context, and the bitmap becomes the contents of the canvas element to which context is bound.\n     */\n    transferFromImageBitmap(bitmap: ImageBitmap | null): void;\n}\n\ndeclare var ImageBitmapRenderingContext: {\n    prototype: ImageBitmapRenderingContext;\n    new(): ImageBitmapRenderingContext;\n};\n\n/** The underlying pixel data of an area of a <canvas> element. It is created using the ImageData() constructor or creator methods on the CanvasRenderingContext2D object associated with a canvas: createImageData() and getImageData(). It can also be used to set a part of the canvas by using putImageData(). */\ninterface ImageData {\n    /**\n     * Returns the one-dimensional array containing the data in RGBA order, as integers in the range 0 to 255.\n     */\n    readonly data: Uint8ClampedArray;\n    /**\n     * Returns the actual dimensions of the data in the ImageData object, in pixels.\n     */\n    readonly height: number;\n    /**\n     * Returns the actual dimensions of the data in the ImageData object, in pixels.\n     */\n    readonly width: number;\n}\n\ndeclare var ImageData: {\n    prototype: ImageData;\n    new(sw: number, sh: number, settings?: ImageDataSettings): ImageData;\n    new(data: Uint8ClampedArray, sw: number, sh?: number, settings?: ImageDataSettings): ImageData;\n};\n\ninterface KHR_parallel_shader_compile {\n    readonly COMPLETION_STATUS_KHR: GLenum;\n}\n\ninterface MediaCapabilities {\n    decodingInfo(configuration: MediaDecodingConfiguration): Promise<MediaCapabilitiesDecodingInfo>;\n    encodingInfo(configuration: MediaEncodingConfiguration): Promise<MediaCapabilitiesEncodingInfo>;\n}\n\ndeclare var MediaCapabilities: {\n    prototype: MediaCapabilities;\n    new(): MediaCapabilities;\n};\n\n/** This Channel Messaging API interface allows us to create a new message channel and send data through it via its two MessagePort properties. */\ninterface MessageChannel {\n    /**\n     * Returns the first MessagePort object.\n     */\n    readonly port1: MessagePort;\n    /**\n     * Returns the second MessagePort object.\n     */\n    readonly port2: MessagePort;\n}\n\ndeclare var MessageChannel: {\n    prototype: MessageChannel;\n    new(): MessageChannel;\n};\n\n/** A message received by a target object. */\ninterface MessageEvent<T = any> extends Event {\n    /**\n     * Returns the data of the message.\n     */\n    readonly data: T;\n    /**\n     * Returns the last event ID string, for server-sent events.\n     */\n    readonly lastEventId: string;\n    /**\n     * Returns the origin of the message, for server-sent events and cross-document messaging.\n     */\n    readonly origin: string;\n    /**\n     * Returns the MessagePort array sent with the message, for cross-document messaging and channel messaging.\n     */\n    readonly ports: ReadonlyArray<MessagePort>;\n    /**\n     * Returns the WindowProxy of the source window, for cross-document messaging, and the MessagePort being attached, in the connect event fired at SharedWorkerGlobalScope objects.\n     */\n    readonly source: MessageEventSource | null;\n    /** @deprecated */\n    initMessageEvent(type: string, bubbles?: boolean, cancelable?: boolean, data?: any, origin?: string, lastEventId?: string, source?: MessageEventSource | null, ports?: MessagePort[]): void;\n}\n\ndeclare var MessageEvent: {\n    prototype: MessageEvent;\n    new<T>(type: string, eventInitDict?: MessageEventInit<T>): MessageEvent<T>;\n};\n\ninterface MessagePortEventMap {\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n}\n\n/** This Channel Messaging API interface represents one of the two ports of a MessageChannel, allowing messages to be sent from one port and listening out for them arriving at the other. */\ninterface MessagePort extends EventTarget {\n    onmessage: ((this: MessagePort, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: MessagePort, ev: MessageEvent) => any) | null;\n    /**\n     * Disconnects the port, so that it is no longer active.\n     */\n    close(): void;\n    /**\n     * Posts a message through the channel. Objects listed in transfer are transferred, not just cloned, meaning that they are no longer usable on the sending side.\n     *\n     * Throws a "DataCloneError" DOMException if transfer contains duplicate objects or port, or if message could not be cloned.\n     */\n    postMessage(message: any, transfer: Transferable[]): void;\n    postMessage(message: any, options?: PostMessageOptions): void;\n    /**\n     * Begins dispatching messages received on the port.\n     */\n    start(): void;\n    addEventListener<K extends keyof MessagePortEventMap>(type: K, listener: (this: MessagePort, ev: MessagePortEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof MessagePortEventMap>(type: K, listener: (this: MessagePort, ev: MessagePortEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var MessagePort: {\n    prototype: MessagePort;\n    new(): MessagePort;\n};\n\ninterface NavigatorConcurrentHardware {\n    readonly hardwareConcurrency: number;\n}\n\ninterface NavigatorID {\n    /** @deprecated */\n    readonly appCodeName: string;\n    /** @deprecated */\n    readonly appName: string;\n    /** @deprecated */\n    readonly appVersion: string;\n    /** @deprecated */\n    readonly platform: string;\n    /** @deprecated */\n    readonly product: string;\n    readonly userAgent: string;\n}\n\ninterface NavigatorLanguage {\n    readonly language: string;\n    readonly languages: ReadonlyArray<string>;\n}\n\ninterface NavigatorNetworkInformation {\n    readonly connection: NetworkInformation;\n}\n\ninterface NavigatorOnLine {\n    readonly onLine: boolean;\n}\n\ninterface NavigatorStorage {\n    readonly storage: StorageManager;\n}\n\ninterface NetworkInformation extends EventTarget {\n    readonly type: ConnectionType;\n}\n\ndeclare var NetworkInformation: {\n    prototype: NetworkInformation;\n    new(): NetworkInformation;\n};\n\ninterface NotificationEventMap {\n    "click": Event;\n    "close": Event;\n    "error": Event;\n    "show": Event;\n}\n\n/** This Notifications API interface is used to configure and display desktop notifications to the user. */\ninterface Notification extends EventTarget {\n    readonly body: string;\n    readonly data: any;\n    readonly dir: NotificationDirection;\n    readonly icon: string;\n    readonly lang: string;\n    onclick: ((this: Notification, ev: Event) => any) | null;\n    onclose: ((this: Notification, ev: Event) => any) | null;\n    onerror: ((this: Notification, ev: Event) => any) | null;\n    onshow: ((this: Notification, ev: Event) => any) | null;\n    readonly tag: string;\n    readonly title: string;\n    close(): void;\n    addEventListener<K extends keyof NotificationEventMap>(type: K, listener: (this: Notification, ev: NotificationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof NotificationEventMap>(type: K, listener: (this: Notification, ev: NotificationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var Notification: {\n    prototype: Notification;\n    new(title: string, options?: NotificationOptions): Notification;\n    readonly permission: NotificationPermission;\n};\n\n/** The parameter passed into the onnotificationclick handler, the NotificationEvent interface represents a notification click event that is dispatched on the ServiceWorkerGlobalScope of a ServiceWorker. */\ninterface NotificationEvent extends ExtendableEvent {\n    readonly action: string;\n    readonly notification: Notification;\n}\n\ndeclare var NotificationEvent: {\n    prototype: NotificationEvent;\n    new(type: string, eventInitDict: NotificationEventInit): NotificationEvent;\n};\n\n/** The OES_element_index_uint extension is part of the WebGL API and adds support for gl.UNSIGNED_INT types to WebGLRenderingContext.drawElements(). */\ninterface OES_element_index_uint {\n}\n\ninterface OES_fbo_render_mipmap {\n}\n\n/** The OES_standard_derivatives extension is part of the WebGL API and adds the GLSL derivative functions dFdx, dFdy, and fwidth. */\ninterface OES_standard_derivatives {\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT_OES: GLenum;\n}\n\n/** The OES_texture_float extension is part of the WebGL API and exposes floating-point pixel types for textures. */\ninterface OES_texture_float {\n}\n\n/** The OES_texture_float_linear extension is part of the WebGL API and allows linear filtering with floating-point pixel types for textures. */\ninterface OES_texture_float_linear {\n}\n\n/** The OES_texture_half_float extension is part of the WebGL API and adds texture formats with 16- (aka half float) and 32-bit floating-point components. */\ninterface OES_texture_half_float {\n    readonly HALF_FLOAT_OES: GLenum;\n}\n\n/** The OES_texture_half_float_linear extension is part of the WebGL API and allows linear filtering with half floating-point pixel types for textures. */\ninterface OES_texture_half_float_linear {\n}\n\ninterface OES_vertex_array_object {\n    bindVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES | null): void;\n    createVertexArrayOES(): WebGLVertexArrayObjectOES | null;\n    deleteVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES | null): void;\n    isVertexArrayOES(arrayObject: WebGLVertexArrayObjectOES | null): GLboolean;\n    readonly VERTEX_ARRAY_BINDING_OES: GLenum;\n}\n\ninterface OVR_multiview2 {\n    framebufferTextureMultiviewOVR(target: GLenum, attachment: GLenum, texture: WebGLTexture | null, level: GLint, baseViewIndex: GLint, numViews: GLsizei): void;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_BASE_VIEW_INDEX_OVR: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_NUM_VIEWS_OVR: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_VIEW_TARGETS_OVR: GLenum;\n    readonly MAX_VIEWS_OVR: GLenum;\n}\n\n/** @deprecated this is not available in most browsers */\ninterface OffscreenCanvas extends EventTarget {\n}\n\n/** This Canvas 2D API interface is used to declare a path that can then be used on a CanvasRenderingContext2D object. The path methods of the CanvasRenderingContext2D interface are also present on this interface, which gives you the convenience of being able to retain and replay your path whenever desired. */\ninterface Path2D extends CanvasPath {\n    /**\n     * Adds to the path the path given by the argument.\n     */\n    addPath(path: Path2D, transform?: DOMMatrix2DInit): void;\n}\n\ndeclare var Path2D: {\n    prototype: Path2D;\n    new(path?: Path2D | string): Path2D;\n};\n\ninterface PerformanceEventMap {\n    "resourcetimingbufferfull": Event;\n}\n\n/** Provides access to performance-related information for the current page. It\'s part of the High Resolution Time API, but is enhanced by the Performance Timeline API, the Navigation Timing API, the User Timing API, and the Resource Timing API. */\ninterface Performance extends EventTarget {\n    onresourcetimingbufferfull: ((this: Performance, ev: Event) => any) | null;\n    readonly timeOrigin: DOMHighResTimeStamp;\n    clearMarks(markName?: string): void;\n    clearMeasures(measureName?: string): void;\n    clearResourceTimings(): void;\n    getEntries(): PerformanceEntryList;\n    getEntriesByName(name: string, type?: string): PerformanceEntryList;\n    getEntriesByType(type: string): PerformanceEntryList;\n    mark(markName: string, markOptions?: PerformanceMarkOptions): PerformanceMark;\n    measure(measureName: string, startOrMeasureOptions?: string | PerformanceMeasureOptions, endMark?: string): PerformanceMeasure;\n    now(): DOMHighResTimeStamp;\n    setResourceTimingBufferSize(maxSize: number): void;\n    toJSON(): any;\n    addEventListener<K extends keyof PerformanceEventMap>(type: K, listener: (this: Performance, ev: PerformanceEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof PerformanceEventMap>(type: K, listener: (this: Performance, ev: PerformanceEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var Performance: {\n    prototype: Performance;\n    new(): Performance;\n};\n\n/** Encapsulates a single performance metric that is part of the performance timeline. A performance entry can be directly created by making a performance mark or measure (for example by calling the mark() method) at an explicit point in an application. Performance entries are also created in indirect ways such as loading a resource (such as an image). */\ninterface PerformanceEntry {\n    readonly duration: DOMHighResTimeStamp;\n    readonly entryType: string;\n    readonly name: string;\n    readonly startTime: DOMHighResTimeStamp;\n    toJSON(): any;\n}\n\ndeclare var PerformanceEntry: {\n    prototype: PerformanceEntry;\n    new(): PerformanceEntry;\n};\n\n/** PerformanceMark is an abstract interface for PerformanceEntry objects with an entryType of "mark". Entries of this type are created by calling performance.mark() to add a named DOMHighResTimeStamp (the mark) to the browser\'s performance timeline. */\ninterface PerformanceMark extends PerformanceEntry {\n    readonly detail: any;\n}\n\ndeclare var PerformanceMark: {\n    prototype: PerformanceMark;\n    new(markName: string, markOptions?: PerformanceMarkOptions): PerformanceMark;\n};\n\n/** PerformanceMeasure is an abstract interface for PerformanceEntry objects with an entryType of "measure". Entries of this type are created by calling performance.measure() to add a named DOMHighResTimeStamp (the measure) between two marks to the browser\'s performance timeline. */\ninterface PerformanceMeasure extends PerformanceEntry {\n    readonly detail: any;\n}\n\ndeclare var PerformanceMeasure: {\n    prototype: PerformanceMeasure;\n    new(): PerformanceMeasure;\n};\n\ninterface PerformanceObserver {\n    disconnect(): void;\n    observe(options?: PerformanceObserverInit): void;\n    takeRecords(): PerformanceEntryList;\n}\n\ndeclare var PerformanceObserver: {\n    prototype: PerformanceObserver;\n    new(callback: PerformanceObserverCallback): PerformanceObserver;\n    readonly supportedEntryTypes: ReadonlyArray<string>;\n};\n\ninterface PerformanceObserverEntryList {\n    getEntries(): PerformanceEntryList;\n    getEntriesByName(name: string, type?: string): PerformanceEntryList;\n    getEntriesByType(type: string): PerformanceEntryList;\n}\n\ndeclare var PerformanceObserverEntryList: {\n    prototype: PerformanceObserverEntryList;\n    new(): PerformanceObserverEntryList;\n};\n\n/** Enables retrieval and analysis of detailed network timing data regarding the loading of an application\'s resources. An application can use the timing metrics to determine, for example, the length of time it takes to fetch a specific resource, such as an XMLHttpRequest, <SVG>, image, or script. */\ninterface PerformanceResourceTiming extends PerformanceEntry {\n    readonly connectEnd: DOMHighResTimeStamp;\n    readonly connectStart: DOMHighResTimeStamp;\n    readonly decodedBodySize: number;\n    readonly domainLookupEnd: DOMHighResTimeStamp;\n    readonly domainLookupStart: DOMHighResTimeStamp;\n    readonly encodedBodySize: number;\n    readonly fetchStart: DOMHighResTimeStamp;\n    readonly initiatorType: string;\n    readonly nextHopProtocol: string;\n    readonly redirectEnd: DOMHighResTimeStamp;\n    readonly redirectStart: DOMHighResTimeStamp;\n    readonly requestStart: DOMHighResTimeStamp;\n    readonly responseEnd: DOMHighResTimeStamp;\n    readonly responseStart: DOMHighResTimeStamp;\n    readonly secureConnectionStart: DOMHighResTimeStamp;\n    readonly serverTiming: ReadonlyArray<PerformanceServerTiming>;\n    readonly transferSize: number;\n    readonly workerStart: DOMHighResTimeStamp;\n    toJSON(): any;\n}\n\ndeclare var PerformanceResourceTiming: {\n    prototype: PerformanceResourceTiming;\n    new(): PerformanceResourceTiming;\n};\n\ninterface PerformanceServerTiming {\n    readonly description: string;\n    readonly duration: DOMHighResTimeStamp;\n    readonly name: string;\n    toJSON(): any;\n}\n\ndeclare var PerformanceServerTiming: {\n    prototype: PerformanceServerTiming;\n    new(): PerformanceServerTiming;\n};\n\ninterface PermissionStatusEventMap {\n    "change": Event;\n}\n\ninterface PermissionStatus extends EventTarget {\n    onchange: ((this: PermissionStatus, ev: Event) => any) | null;\n    readonly state: PermissionState;\n    addEventListener<K extends keyof PermissionStatusEventMap>(type: K, listener: (this: PermissionStatus, ev: PermissionStatusEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof PermissionStatusEventMap>(type: K, listener: (this: PermissionStatus, ev: PermissionStatusEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var PermissionStatus: {\n    prototype: PermissionStatus;\n    new(): PermissionStatus;\n};\n\ninterface Permissions {\n    query(permissionDesc: PermissionDescriptor): Promise<PermissionStatus>;\n}\n\ndeclare var Permissions: {\n    prototype: Permissions;\n    new(): Permissions;\n};\n\n/** Events measuring progress of an underlying process, like an HTTP request (for an XMLHttpRequest, or the loading of the underlying resource of an <img>, <audio>, <video>, <style> or <link>). */\ninterface ProgressEvent<T extends EventTarget = EventTarget> extends Event {\n    readonly lengthComputable: boolean;\n    readonly loaded: number;\n    readonly target: T | null;\n    readonly total: number;\n}\n\ndeclare var ProgressEvent: {\n    prototype: ProgressEvent;\n    new(type: string, eventInitDict?: ProgressEventInit): ProgressEvent;\n};\n\ninterface PromiseRejectionEvent extends Event {\n    readonly promise: Promise<any>;\n    readonly reason: any;\n}\n\ndeclare var PromiseRejectionEvent: {\n    prototype: PromiseRejectionEvent;\n    new(type: string, eventInitDict: PromiseRejectionEventInit): PromiseRejectionEvent;\n};\n\n/** This Push API interface represents a push message that has been received. This event is sent to the global scope of a ServiceWorker. It contains the information sent from an application server to a PushSubscription. */\ninterface PushEvent extends ExtendableEvent {\n    readonly data: PushMessageData | null;\n}\n\ndeclare var PushEvent: {\n    prototype: PushEvent;\n    new(type: string, eventInitDict?: PushEventInit): PushEvent;\n};\n\n/** This Push API interface provides a way to receive notifications from third-party servers as well as request URLs for push notifications. */\ninterface PushManager {\n    getSubscription(): Promise<PushSubscription | null>;\n    permissionState(options?: PushSubscriptionOptionsInit): Promise<PushPermissionState>;\n    subscribe(options?: PushSubscriptionOptionsInit): Promise<PushSubscription>;\n}\n\ndeclare var PushManager: {\n    prototype: PushManager;\n    new(): PushManager;\n    readonly supportedContentEncodings: ReadonlyArray<string>;\n};\n\n/** This Push API interface provides methods which let you retrieve the push data sent by a server in various formats. */\ninterface PushMessageData {\n    arrayBuffer(): ArrayBuffer;\n    blob(): Blob;\n    json(): any;\n    text(): string;\n}\n\ndeclare var PushMessageData: {\n    prototype: PushMessageData;\n    new(): PushMessageData;\n};\n\n/** This Push API interface provides a subcription\'s URL endpoint and allows unsubscription from a push service. */\ninterface PushSubscription {\n    readonly endpoint: string;\n    readonly options: PushSubscriptionOptions;\n    getKey(name: PushEncryptionKeyName): ArrayBuffer | null;\n    toJSON(): PushSubscriptionJSON;\n    unsubscribe(): Promise<boolean>;\n}\n\ndeclare var PushSubscription: {\n    prototype: PushSubscription;\n    new(): PushSubscription;\n};\n\ninterface PushSubscriptionOptions {\n    readonly applicationServerKey: ArrayBuffer | null;\n}\n\ndeclare var PushSubscriptionOptions: {\n    prototype: PushSubscriptionOptions;\n    new(): PushSubscriptionOptions;\n};\n\n/** This Streams API interface represents a readable stream of byte data. The Fetch API offers a concrete instance of a ReadableStream through the body property of a Response object. */\ninterface ReadableStream<R = any> {\n    readonly locked: boolean;\n    cancel(reason?: any): Promise<void>;\n    getReader(): ReadableStreamDefaultReader<R>;\n    pipeThrough<T>(transform: ReadableWritablePair<T, R>, options?: StreamPipeOptions): ReadableStream<T>;\n    pipeTo(destination: WritableStream<R>, options?: StreamPipeOptions): Promise<void>;\n    tee(): [ReadableStream<R>, ReadableStream<R>];\n    forEach(callbackfn: (value: any, key: number, parent: ReadableStream<R>) => void, thisArg?: any): void;\n}\n\ndeclare var ReadableStream: {\n    prototype: ReadableStream;\n    new<R = any>(underlyingSource?: UnderlyingSource<R>, strategy?: QueuingStrategy<R>): ReadableStream<R>;\n};\n\ninterface ReadableStreamDefaultController<R = any> {\n    readonly desiredSize: number | null;\n    close(): void;\n    enqueue(chunk?: R): void;\n    error(e?: any): void;\n}\n\ndeclare var ReadableStreamDefaultController: {\n    prototype: ReadableStreamDefaultController;\n    new(): ReadableStreamDefaultController;\n};\n\ninterface ReadableStreamDefaultReader<R = any> extends ReadableStreamGenericReader {\n    read(): Promise<ReadableStreamDefaultReadResult<R>>;\n    releaseLock(): void;\n}\n\ndeclare var ReadableStreamDefaultReader: {\n    prototype: ReadableStreamDefaultReader;\n    new<R = any>(stream: ReadableStream<R>): ReadableStreamDefaultReader<R>;\n};\n\ninterface ReadableStreamGenericReader {\n    readonly closed: Promise<undefined>;\n    cancel(reason?: any): Promise<void>;\n}\n\n/** This Fetch API interface represents a resource request. */\ninterface Request extends Body {\n    /**\n     * Returns the cache mode associated with request, which is a string indicating how the request will interact with the browser\'s cache when fetching.\n     */\n    readonly cache: RequestCache;\n    /**\n     * Returns the credentials mode associated with request, which is a string indicating whether credentials will be sent with the request always, never, or only when sent to a same-origin URL.\n     */\n    readonly credentials: RequestCredentials;\n    /**\n     * Returns the kind of resource requested by request, e.g., "document" or "script".\n     */\n    readonly destination: RequestDestination;\n    /**\n     * Returns a Headers object consisting of the headers associated with request. Note that headers added in the network layer by the user agent will not be accounted for in this object, e.g., the "Host" header.\n     */\n    readonly headers: Headers;\n    /**\n     * Returns request\'s subresource integrity metadata, which is a cryptographic hash of the resource being fetched. Its value consists of multiple hashes separated by whitespace. [SRI]\n     */\n    readonly integrity: string;\n    /**\n     * Returns a boolean indicating whether or not request can outlive the global in which it was created.\n     */\n    readonly keepalive: boolean;\n    /**\n     * Returns request\'s HTTP method, which is "GET" by default.\n     */\n    readonly method: string;\n    /**\n     * Returns the mode associated with request, which is a string indicating whether the request will use CORS, or will be restricted to same-origin URLs.\n     */\n    readonly mode: RequestMode;\n    /**\n     * Returns the redirect mode associated with request, which is a string indicating how redirects for the request will be handled during fetching. A request will follow redirects by default.\n     */\n    readonly redirect: RequestRedirect;\n    /**\n     * Returns the referrer of request. Its value can be a same-origin URL if explicitly set in init, the empty string to indicate no referrer, and "about:client" when defaulting to the global\'s default. This is used during fetching to determine the value of the `Referer` header of the request being made.\n     */\n    readonly referrer: string;\n    /**\n     * Returns the referrer policy associated with request. This is used during fetching to compute the value of the request\'s referrer.\n     */\n    readonly referrerPolicy: ReferrerPolicy;\n    /**\n     * Returns the signal associated with request, which is an AbortSignal object indicating whether or not request has been aborted, and its abort event handler.\n     */\n    readonly signal: AbortSignal;\n    /**\n     * Returns the URL of request as a string.\n     */\n    readonly url: string;\n    clone(): Request;\n}\n\ndeclare var Request: {\n    prototype: Request;\n    new(input: RequestInfo, init?: RequestInit): Request;\n};\n\n/** This Fetch API interface represents the response to a request. */\ninterface Response extends Body {\n    readonly headers: Headers;\n    readonly ok: boolean;\n    readonly redirected: boolean;\n    readonly status: number;\n    readonly statusText: string;\n    readonly type: ResponseType;\n    readonly url: string;\n    clone(): Response;\n}\n\ndeclare var Response: {\n    prototype: Response;\n    new(body?: BodyInit | null, init?: ResponseInit): Response;\n    error(): Response;\n    redirect(url: string | URL, status?: number): Response;\n};\n\n/** Inherits from Event, and represents the event object of an event sent on a document or worker when its content security policy is violated. */\ninterface SecurityPolicyViolationEvent extends Event {\n    readonly blockedURI: string;\n    readonly columnNumber: number;\n    readonly disposition: SecurityPolicyViolationEventDisposition;\n    readonly documentURI: string;\n    readonly effectiveDirective: string;\n    readonly lineNumber: number;\n    readonly originalPolicy: string;\n    readonly referrer: string;\n    readonly sample: string;\n    readonly sourceFile: string;\n    readonly statusCode: number;\n    readonly violatedDirective: string;\n}\n\ndeclare var SecurityPolicyViolationEvent: {\n    prototype: SecurityPolicyViolationEvent;\n    new(type: string, eventInitDict?: SecurityPolicyViolationEventInit): SecurityPolicyViolationEvent;\n};\n\ninterface ServiceWorkerEventMap extends AbstractWorkerEventMap {\n    "statechange": Event;\n}\n\n/** This ServiceWorker API interface provides a reference to a service worker. Multiple browsing contexts (e.g. pages, workers, etc.) can be associated with the same service worker, each through a unique ServiceWorker object. */\ninterface ServiceWorker extends EventTarget, AbstractWorker {\n    onstatechange: ((this: ServiceWorker, ev: Event) => any) | null;\n    readonly scriptURL: string;\n    readonly state: ServiceWorkerState;\n    postMessage(message: any, transfer: Transferable[]): void;\n    postMessage(message: any, options?: PostMessageOptions): void;\n    addEventListener<K extends keyof ServiceWorkerEventMap>(type: K, listener: (this: ServiceWorker, ev: ServiceWorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ServiceWorkerEventMap>(type: K, listener: (this: ServiceWorker, ev: ServiceWorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var ServiceWorker: {\n    prototype: ServiceWorker;\n    new(): ServiceWorker;\n};\n\ninterface ServiceWorkerContainerEventMap {\n    "controllerchange": Event;\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n}\n\n/** The ServiceWorkerContainer interface of the ServiceWorker API provides an object representing the service worker as an overall unit in the network ecosystem, including facilities to register, unregister and update service workers, and access the state of service workers and their registrations. */\ninterface ServiceWorkerContainer extends EventTarget {\n    readonly controller: ServiceWorker | null;\n    oncontrollerchange: ((this: ServiceWorkerContainer, ev: Event) => any) | null;\n    onmessage: ((this: ServiceWorkerContainer, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: ServiceWorkerContainer, ev: MessageEvent) => any) | null;\n    readonly ready: Promise<ServiceWorkerRegistration>;\n    getRegistration(clientURL?: string | URL): Promise<ServiceWorkerRegistration | undefined>;\n    getRegistrations(): Promise<ReadonlyArray<ServiceWorkerRegistration>>;\n    register(scriptURL: string | URL, options?: RegistrationOptions): Promise<ServiceWorkerRegistration>;\n    startMessages(): void;\n    addEventListener<K extends keyof ServiceWorkerContainerEventMap>(type: K, listener: (this: ServiceWorkerContainer, ev: ServiceWorkerContainerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ServiceWorkerContainerEventMap>(type: K, listener: (this: ServiceWorkerContainer, ev: ServiceWorkerContainerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var ServiceWorkerContainer: {\n    prototype: ServiceWorkerContainer;\n    new(): ServiceWorkerContainer;\n};\n\ninterface ServiceWorkerGlobalScopeEventMap extends WorkerGlobalScopeEventMap {\n    "activate": ExtendableEvent;\n    "fetch": FetchEvent;\n    "install": ExtendableEvent;\n    "message": ExtendableMessageEvent;\n    "messageerror": MessageEvent;\n    "notificationclick": NotificationEvent;\n    "notificationclose": NotificationEvent;\n    "push": PushEvent;\n}\n\n/** This ServiceWorker API interface represents the global execution context of a service worker. */\ninterface ServiceWorkerGlobalScope extends WorkerGlobalScope {\n    readonly clients: Clients;\n    onactivate: ((this: ServiceWorkerGlobalScope, ev: ExtendableEvent) => any) | null;\n    onfetch: ((this: ServiceWorkerGlobalScope, ev: FetchEvent) => any) | null;\n    oninstall: ((this: ServiceWorkerGlobalScope, ev: ExtendableEvent) => any) | null;\n    onmessage: ((this: ServiceWorkerGlobalScope, ev: ExtendableMessageEvent) => any) | null;\n    onmessageerror: ((this: ServiceWorkerGlobalScope, ev: MessageEvent) => any) | null;\n    onnotificationclick: ((this: ServiceWorkerGlobalScope, ev: NotificationEvent) => any) | null;\n    onnotificationclose: ((this: ServiceWorkerGlobalScope, ev: NotificationEvent) => any) | null;\n    onpush: ((this: ServiceWorkerGlobalScope, ev: PushEvent) => any) | null;\n    readonly registration: ServiceWorkerRegistration;\n    skipWaiting(): Promise<void>;\n    addEventListener<K extends keyof ServiceWorkerGlobalScopeEventMap>(type: K, listener: (this: ServiceWorkerGlobalScope, ev: ServiceWorkerGlobalScopeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ServiceWorkerGlobalScopeEventMap>(type: K, listener: (this: ServiceWorkerGlobalScope, ev: ServiceWorkerGlobalScopeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var ServiceWorkerGlobalScope: {\n    prototype: ServiceWorkerGlobalScope;\n    new(): ServiceWorkerGlobalScope;\n};\n\ninterface ServiceWorkerRegistrationEventMap {\n    "updatefound": Event;\n}\n\n/** This ServiceWorker API interface represents the service worker registration. You register a service worker to control one or more pages that share the same origin. */\ninterface ServiceWorkerRegistration extends EventTarget {\n    readonly active: ServiceWorker | null;\n    readonly installing: ServiceWorker | null;\n    onupdatefound: ((this: ServiceWorkerRegistration, ev: Event) => any) | null;\n    readonly pushManager: PushManager;\n    readonly scope: string;\n    readonly updateViaCache: ServiceWorkerUpdateViaCache;\n    readonly waiting: ServiceWorker | null;\n    getNotifications(filter?: GetNotificationOptions): Promise<Notification[]>;\n    showNotification(title: string, options?: NotificationOptions): Promise<void>;\n    unregister(): Promise<boolean>;\n    update(): Promise<void>;\n    addEventListener<K extends keyof ServiceWorkerRegistrationEventMap>(type: K, listener: (this: ServiceWorkerRegistration, ev: ServiceWorkerRegistrationEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof ServiceWorkerRegistrationEventMap>(type: K, listener: (this: ServiceWorkerRegistration, ev: ServiceWorkerRegistrationEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var ServiceWorkerRegistration: {\n    prototype: ServiceWorkerRegistration;\n    new(): ServiceWorkerRegistration;\n};\n\ninterface SharedWorkerGlobalScopeEventMap extends WorkerGlobalScopeEventMap {\n    "connect": MessageEvent;\n}\n\ninterface SharedWorkerGlobalScope extends WorkerGlobalScope {\n    /**\n     * Returns sharedWorkerGlobal\'s name, i.e. the value given to the SharedWorker constructor. Multiple SharedWorker objects can correspond to the same shared worker (and SharedWorkerGlobalScope), by reusing the same name.\n     */\n    readonly name: string;\n    onconnect: ((this: SharedWorkerGlobalScope, ev: MessageEvent) => any) | null;\n    /**\n     * Aborts sharedWorkerGlobal.\n     */\n    close(): void;\n    addEventListener<K extends keyof SharedWorkerGlobalScopeEventMap>(type: K, listener: (this: SharedWorkerGlobalScope, ev: SharedWorkerGlobalScopeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof SharedWorkerGlobalScopeEventMap>(type: K, listener: (this: SharedWorkerGlobalScope, ev: SharedWorkerGlobalScopeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var SharedWorkerGlobalScope: {\n    prototype: SharedWorkerGlobalScope;\n    new(): SharedWorkerGlobalScope;\n};\n\ninterface StorageManager {\n    estimate(): Promise<StorageEstimate>;\n    persisted(): Promise<boolean>;\n}\n\ndeclare var StorageManager: {\n    prototype: StorageManager;\n    new(): StorageManager;\n};\n\n/** This Web Crypto API interface provides a number of low-level cryptographic functions. It is accessed via the Crypto.subtle properties available in a window context (via Window.crypto). */\ninterface SubtleCrypto {\n    decrypt(algorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams, key: CryptoKey, data: BufferSource): Promise<any>;\n    deriveBits(algorithm: AlgorithmIdentifier | EcdhKeyDeriveParams | HkdfParams | Pbkdf2Params, baseKey: CryptoKey, length: number): Promise<ArrayBuffer>;\n    deriveKey(algorithm: AlgorithmIdentifier | EcdhKeyDeriveParams | HkdfParams | Pbkdf2Params, baseKey: CryptoKey, derivedKeyType: AlgorithmIdentifier | AesDerivedKeyParams | HmacImportParams | HkdfParams | Pbkdf2Params, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    digest(algorithm: AlgorithmIdentifier, data: BufferSource): Promise<ArrayBuffer>;\n    encrypt(algorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams, key: CryptoKey, data: BufferSource): Promise<any>;\n    exportKey(format: "jwk", key: CryptoKey): Promise<JsonWebKey>;\n    exportKey(format: Exclude<KeyFormat, "jwk">, key: CryptoKey): Promise<ArrayBuffer>;\n    generateKey(algorithm: RsaHashedKeyGenParams | EcKeyGenParams, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKeyPair>;\n    generateKey(algorithm: AesKeyGenParams | HmacKeyGenParams | Pbkdf2Params, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    generateKey(algorithm: AlgorithmIdentifier, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKeyPair | CryptoKey>;\n    importKey(format: "jwk", keyData: JsonWebKey, algorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    importKey(format: Exclude<KeyFormat, "jwk">, keyData: BufferSource, algorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    sign(algorithm: AlgorithmIdentifier | RsaPssParams | EcdsaParams, key: CryptoKey, data: BufferSource): Promise<ArrayBuffer>;\n    unwrapKey(format: KeyFormat, wrappedKey: BufferSource, unwrappingKey: CryptoKey, unwrapAlgorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams, unwrappedKeyAlgorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    verify(algorithm: AlgorithmIdentifier | RsaPssParams | EcdsaParams, key: CryptoKey, signature: BufferSource, data: BufferSource): Promise<boolean>;\n    wrapKey(format: KeyFormat, key: CryptoKey, wrappingKey: CryptoKey, wrapAlgorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams): Promise<ArrayBuffer>;\n}\n\ndeclare var SubtleCrypto: {\n    prototype: SubtleCrypto;\n    new(): SubtleCrypto;\n};\n\n/** A decoder for a specific method, that is a specific character encoding, like utf-8, iso-8859-2, koi8, cp1261, gbk, etc. A decoder takes a stream of bytes as input and emits a stream of code points. For a more scalable, non-native library, see StringView – a C-like representation of strings based on typed arrays. */\ninterface TextDecoder extends TextDecoderCommon {\n    /**\n     * Returns the result of running encoding\'s decoder. The method can be invoked zero or more times with options\'s stream set to true, and then once without options\'s stream (or set to false), to process a fragmented input. If the invocation without options\'s stream (or set to false) has no input, it\'s clearest to omit both arguments.\n     *\n     * ```\n     * var string = "", decoder = new TextDecoder(encoding), buffer;\n     * while(buffer = next_chunk()) {\n     *   string += decoder.decode(buffer, {stream:true});\n     * }\n     * string += decoder.decode(); // end-of-queue\n     * ```\n     *\n     * If the error mode is "fatal" and encoding\'s decoder returns error, throws a TypeError.\n     */\n    decode(input?: BufferSource, options?: TextDecodeOptions): string;\n}\n\ndeclare var TextDecoder: {\n    prototype: TextDecoder;\n    new(label?: string, options?: TextDecoderOptions): TextDecoder;\n};\n\ninterface TextDecoderCommon {\n    /**\n     * Returns encoding\'s name, lowercased.\n     */\n    readonly encoding: string;\n    /**\n     * Returns true if error mode is "fatal", otherwise false.\n     */\n    readonly fatal: boolean;\n    /**\n     * Returns the value of ignore BOM.\n     */\n    readonly ignoreBOM: boolean;\n}\n\ninterface TextDecoderStream extends GenericTransformStream, TextDecoderCommon {\n    readonly readable: ReadableStream<string>;\n    readonly writable: WritableStream<BufferSource>;\n}\n\ndeclare var TextDecoderStream: {\n    prototype: TextDecoderStream;\n    new(label?: string, options?: TextDecoderOptions): TextDecoderStream;\n};\n\n/** TextEncoder takes a stream of code points as input and emits a stream of bytes. For a more scalable, non-native library, see StringView – a C-like representation of strings based on typed arrays. */\ninterface TextEncoder extends TextEncoderCommon {\n    /**\n     * Returns the result of running UTF-8\'s encoder.\n     */\n    encode(input?: string): Uint8Array;\n    /**\n     * Runs the UTF-8 encoder on source, stores the result of that operation into destination, and returns the progress made as an object wherein read is the number of converted code units of source and written is the number of bytes modified in destination.\n     */\n    encodeInto(source: string, destination: Uint8Array): TextEncoderEncodeIntoResult;\n}\n\ndeclare var TextEncoder: {\n    prototype: TextEncoder;\n    new(): TextEncoder;\n};\n\ninterface TextEncoderCommon {\n    /**\n     * Returns "utf-8".\n     */\n    readonly encoding: string;\n}\n\ninterface TextEncoderStream extends GenericTransformStream, TextEncoderCommon {\n    readonly readable: ReadableStream<Uint8Array>;\n    readonly writable: WritableStream<string>;\n}\n\ndeclare var TextEncoderStream: {\n    prototype: TextEncoderStream;\n    new(): TextEncoderStream;\n};\n\n/** The dimensions of a piece of text in the canvas, as created by the CanvasRenderingContext2D.measureText() method. */\ninterface TextMetrics {\n    /**\n     * Returns the measurement described below.\n     */\n    readonly actualBoundingBoxAscent: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly actualBoundingBoxDescent: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly actualBoundingBoxLeft: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly actualBoundingBoxRight: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly fontBoundingBoxAscent: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly fontBoundingBoxDescent: number;\n    /**\n     * Returns the measurement described below.\n     */\n    readonly width: number;\n}\n\ndeclare var TextMetrics: {\n    prototype: TextMetrics;\n    new(): TextMetrics;\n};\n\ninterface TransformStream<I = any, O = any> {\n    readonly readable: ReadableStream<O>;\n    readonly writable: WritableStream<I>;\n}\n\ndeclare var TransformStream: {\n    prototype: TransformStream;\n    new<I = any, O = any>(transformer?: Transformer<I, O>, writableStrategy?: QueuingStrategy<I>, readableStrategy?: QueuingStrategy<O>): TransformStream<I, O>;\n};\n\ninterface TransformStreamDefaultController<O = any> {\n    readonly desiredSize: number | null;\n    enqueue(chunk?: O): void;\n    error(reason?: any): void;\n    terminate(): void;\n}\n\ndeclare var TransformStreamDefaultController: {\n    prototype: TransformStreamDefaultController;\n    new(): TransformStreamDefaultController;\n};\n\n/** The URL interface represents an object providing static methods used for creating object URLs. */\ninterface URL {\n    hash: string;\n    host: string;\n    hostname: string;\n    href: string;\n    toString(): string;\n    readonly origin: string;\n    password: string;\n    pathname: string;\n    port: string;\n    protocol: string;\n    search: string;\n    readonly searchParams: URLSearchParams;\n    username: string;\n    toJSON(): string;\n}\n\ndeclare var URL: {\n    prototype: URL;\n    new(url: string | URL, base?: string | URL): URL;\n    createObjectURL(object: any): string;\n    revokeObjectURL(url: string): void;\n};\n\ninterface URLSearchParams {\n    /**\n     * Appends a specified key/value pair as a new search parameter.\n     */\n    append(name: string, value: string): void;\n    /**\n     * Deletes the given search parameter, and its associated value, from the list of all search parameters.\n     */\n    delete(name: string): void;\n    /**\n     * Returns the first value associated to the given search parameter.\n     */\n    get(name: string): string | null;\n    /**\n     * Returns all the values association with a given search parameter.\n     */\n    getAll(name: string): string[];\n    /**\n     * Returns a Boolean indicating if such a search parameter exists.\n     */\n    has(name: string): boolean;\n    /**\n     * Sets the value associated to a given search parameter to the given value. If there were several values, delete the others.\n     */\n    set(name: string, value: string): void;\n    sort(): void;\n    /**\n     * Returns a string containing a query string suitable for use in a URL. Does not include the question mark.\n     */\n    toString(): string;\n    forEach(callbackfn: (value: string, key: string, parent: URLSearchParams) => void, thisArg?: any): void;\n}\n\ndeclare var URLSearchParams: {\n    prototype: URLSearchParams;\n    new(init?: string[][] | Record<string, string> | string | URLSearchParams): URLSearchParams;\n    toString(): string;\n};\n\ninterface WEBGL_color_buffer_float {\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT: GLenum;\n    readonly RGBA32F_EXT: GLenum;\n    readonly UNSIGNED_NORMALIZED_EXT: GLenum;\n}\n\ninterface WEBGL_compressed_texture_astc {\n    getSupportedProfiles(): string[];\n    readonly COMPRESSED_RGBA_ASTC_10x10_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_10x5_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_10x6_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_10x8_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_12x10_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_12x12_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_4x4_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_5x4_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_5x5_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_6x5_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_6x6_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_8x5_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_8x6_KHR: GLenum;\n    readonly COMPRESSED_RGBA_ASTC_8x8_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x10_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x5_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x6_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_10x8_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_12x10_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_12x12_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_5x4_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_5x5_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_6x5_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_6x6_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x5_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x6_KHR: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ASTC_8x8_KHR: GLenum;\n}\n\ninterface WEBGL_compressed_texture_etc {\n    readonly COMPRESSED_R11_EAC: GLenum;\n    readonly COMPRESSED_RG11_EAC: GLenum;\n    readonly COMPRESSED_RGB8_ETC2: GLenum;\n    readonly COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2: GLenum;\n    readonly COMPRESSED_RGBA8_ETC2_EAC: GLenum;\n    readonly COMPRESSED_SIGNED_R11_EAC: GLenum;\n    readonly COMPRESSED_SIGNED_RG11_EAC: GLenum;\n    readonly COMPRESSED_SRGB8_ALPHA8_ETC2_EAC: GLenum;\n    readonly COMPRESSED_SRGB8_ETC2: GLenum;\n    readonly COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2: GLenum;\n}\n\ninterface WEBGL_compressed_texture_etc1 {\n    readonly COMPRESSED_RGB_ETC1_WEBGL: GLenum;\n}\n\ninterface WEBGL_compressed_texture_pvrtc {\n    readonly COMPRESSED_RGBA_PVRTC_2BPPV1_IMG: GLenum;\n    readonly COMPRESSED_RGBA_PVRTC_4BPPV1_IMG: GLenum;\n    readonly COMPRESSED_RGB_PVRTC_2BPPV1_IMG: GLenum;\n    readonly COMPRESSED_RGB_PVRTC_4BPPV1_IMG: GLenum;\n}\n\n/** The WEBGL_compressed_texture_s3tc extension is part of the WebGL API and exposes four S3TC compressed texture formats. */\ninterface WEBGL_compressed_texture_s3tc {\n    readonly COMPRESSED_RGBA_S3TC_DXT1_EXT: GLenum;\n    readonly COMPRESSED_RGBA_S3TC_DXT3_EXT: GLenum;\n    readonly COMPRESSED_RGBA_S3TC_DXT5_EXT: GLenum;\n    readonly COMPRESSED_RGB_S3TC_DXT1_EXT: GLenum;\n}\n\ninterface WEBGL_compressed_texture_s3tc_srgb {\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT: GLenum;\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT: GLenum;\n    readonly COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT: GLenum;\n    readonly COMPRESSED_SRGB_S3TC_DXT1_EXT: GLenum;\n}\n\n/** The WEBGL_debug_renderer_info extension is part of the WebGL API and exposes two constants with information about the graphics driver for debugging purposes. */\ninterface WEBGL_debug_renderer_info {\n    readonly UNMASKED_RENDERER_WEBGL: GLenum;\n    readonly UNMASKED_VENDOR_WEBGL: GLenum;\n}\n\ninterface WEBGL_debug_shaders {\n    getTranslatedShaderSource(shader: WebGLShader): string;\n}\n\n/** The WEBGL_depth_texture extension is part of the WebGL API and defines 2D depth and depth-stencil textures. */\ninterface WEBGL_depth_texture {\n    readonly UNSIGNED_INT_24_8_WEBGL: GLenum;\n}\n\ninterface WEBGL_draw_buffers {\n    drawBuffersWEBGL(buffers: GLenum[]): void;\n    readonly COLOR_ATTACHMENT0_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT10_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT11_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT12_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT13_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT14_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT15_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT1_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT2_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT3_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT4_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT5_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT6_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT7_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT8_WEBGL: GLenum;\n    readonly COLOR_ATTACHMENT9_WEBGL: GLenum;\n    readonly DRAW_BUFFER0_WEBGL: GLenum;\n    readonly DRAW_BUFFER10_WEBGL: GLenum;\n    readonly DRAW_BUFFER11_WEBGL: GLenum;\n    readonly DRAW_BUFFER12_WEBGL: GLenum;\n    readonly DRAW_BUFFER13_WEBGL: GLenum;\n    readonly DRAW_BUFFER14_WEBGL: GLenum;\n    readonly DRAW_BUFFER15_WEBGL: GLenum;\n    readonly DRAW_BUFFER1_WEBGL: GLenum;\n    readonly DRAW_BUFFER2_WEBGL: GLenum;\n    readonly DRAW_BUFFER3_WEBGL: GLenum;\n    readonly DRAW_BUFFER4_WEBGL: GLenum;\n    readonly DRAW_BUFFER5_WEBGL: GLenum;\n    readonly DRAW_BUFFER6_WEBGL: GLenum;\n    readonly DRAW_BUFFER7_WEBGL: GLenum;\n    readonly DRAW_BUFFER8_WEBGL: GLenum;\n    readonly DRAW_BUFFER9_WEBGL: GLenum;\n    readonly MAX_COLOR_ATTACHMENTS_WEBGL: GLenum;\n    readonly MAX_DRAW_BUFFERS_WEBGL: GLenum;\n}\n\ninterface WEBGL_lose_context {\n    loseContext(): void;\n    restoreContext(): void;\n}\n\ninterface WebGL2RenderingContext extends WebGL2RenderingContextBase, WebGL2RenderingContextOverloads, WebGLRenderingContextBase {\n}\n\ndeclare var WebGL2RenderingContext: {\n    prototype: WebGL2RenderingContext;\n    new(): WebGL2RenderingContext;\n    readonly ACTIVE_UNIFORM_BLOCKS: GLenum;\n    readonly ALREADY_SIGNALED: GLenum;\n    readonly ANY_SAMPLES_PASSED: GLenum;\n    readonly ANY_SAMPLES_PASSED_CONSERVATIVE: GLenum;\n    readonly COLOR: GLenum;\n    readonly COLOR_ATTACHMENT1: GLenum;\n    readonly COLOR_ATTACHMENT10: GLenum;\n    readonly COLOR_ATTACHMENT11: GLenum;\n    readonly COLOR_ATTACHMENT12: GLenum;\n    readonly COLOR_ATTACHMENT13: GLenum;\n    readonly COLOR_ATTACHMENT14: GLenum;\n    readonly COLOR_ATTACHMENT15: GLenum;\n    readonly COLOR_ATTACHMENT2: GLenum;\n    readonly COLOR_ATTACHMENT3: GLenum;\n    readonly COLOR_ATTACHMENT4: GLenum;\n    readonly COLOR_ATTACHMENT5: GLenum;\n    readonly COLOR_ATTACHMENT6: GLenum;\n    readonly COLOR_ATTACHMENT7: GLenum;\n    readonly COLOR_ATTACHMENT8: GLenum;\n    readonly COLOR_ATTACHMENT9: GLenum;\n    readonly COMPARE_REF_TO_TEXTURE: GLenum;\n    readonly CONDITION_SATISFIED: GLenum;\n    readonly COPY_READ_BUFFER: GLenum;\n    readonly COPY_READ_BUFFER_BINDING: GLenum;\n    readonly COPY_WRITE_BUFFER: GLenum;\n    readonly COPY_WRITE_BUFFER_BINDING: GLenum;\n    readonly CURRENT_QUERY: GLenum;\n    readonly DEPTH: GLenum;\n    readonly DEPTH24_STENCIL8: GLenum;\n    readonly DEPTH32F_STENCIL8: GLenum;\n    readonly DEPTH_COMPONENT24: GLenum;\n    readonly DEPTH_COMPONENT32F: GLenum;\n    readonly DRAW_BUFFER0: GLenum;\n    readonly DRAW_BUFFER1: GLenum;\n    readonly DRAW_BUFFER10: GLenum;\n    readonly DRAW_BUFFER11: GLenum;\n    readonly DRAW_BUFFER12: GLenum;\n    readonly DRAW_BUFFER13: GLenum;\n    readonly DRAW_BUFFER14: GLenum;\n    readonly DRAW_BUFFER15: GLenum;\n    readonly DRAW_BUFFER2: GLenum;\n    readonly DRAW_BUFFER3: GLenum;\n    readonly DRAW_BUFFER4: GLenum;\n    readonly DRAW_BUFFER5: GLenum;\n    readonly DRAW_BUFFER6: GLenum;\n    readonly DRAW_BUFFER7: GLenum;\n    readonly DRAW_BUFFER8: GLenum;\n    readonly DRAW_BUFFER9: GLenum;\n    readonly DRAW_FRAMEBUFFER: GLenum;\n    readonly DRAW_FRAMEBUFFER_BINDING: GLenum;\n    readonly DYNAMIC_COPY: GLenum;\n    readonly DYNAMIC_READ: GLenum;\n    readonly FLOAT_32_UNSIGNED_INT_24_8_REV: GLenum;\n    readonly FLOAT_MAT2x3: GLenum;\n    readonly FLOAT_MAT2x4: GLenum;\n    readonly FLOAT_MAT3x2: GLenum;\n    readonly FLOAT_MAT3x4: GLenum;\n    readonly FLOAT_MAT4x2: GLenum;\n    readonly FLOAT_MAT4x3: GLenum;\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_BLUE_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_GREEN_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_RED_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER: GLenum;\n    readonly FRAMEBUFFER_DEFAULT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MULTISAMPLE: GLenum;\n    readonly HALF_FLOAT: GLenum;\n    readonly INTERLEAVED_ATTRIBS: GLenum;\n    readonly INT_2_10_10_10_REV: GLenum;\n    readonly INT_SAMPLER_2D: GLenum;\n    readonly INT_SAMPLER_2D_ARRAY: GLenum;\n    readonly INT_SAMPLER_3D: GLenum;\n    readonly INT_SAMPLER_CUBE: GLenum;\n    readonly INVALID_INDEX: GLenum;\n    readonly MAX: GLenum;\n    readonly MAX_3D_TEXTURE_SIZE: GLenum;\n    readonly MAX_ARRAY_TEXTURE_LAYERS: GLenum;\n    readonly MAX_CLIENT_WAIT_TIMEOUT_WEBGL: GLenum;\n    readonly MAX_COLOR_ATTACHMENTS: GLenum;\n    readonly MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_COMBINED_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_DRAW_BUFFERS: GLenum;\n    readonly MAX_ELEMENTS_INDICES: GLenum;\n    readonly MAX_ELEMENTS_VERTICES: GLenum;\n    readonly MAX_ELEMENT_INDEX: GLenum;\n    readonly MAX_FRAGMENT_INPUT_COMPONENTS: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_PROGRAM_TEXEL_OFFSET: GLenum;\n    readonly MAX_SAMPLES: GLenum;\n    readonly MAX_SERVER_WAIT_TIMEOUT: GLenum;\n    readonly MAX_TEXTURE_LOD_BIAS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS: GLenum;\n    readonly MAX_UNIFORM_BLOCK_SIZE: GLenum;\n    readonly MAX_UNIFORM_BUFFER_BINDINGS: GLenum;\n    readonly MAX_VARYING_COMPONENTS: GLenum;\n    readonly MAX_VERTEX_OUTPUT_COMPONENTS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_COMPONENTS: GLenum;\n    readonly MIN: GLenum;\n    readonly MIN_PROGRAM_TEXEL_OFFSET: GLenum;\n    readonly OBJECT_TYPE: GLenum;\n    readonly PACK_ROW_LENGTH: GLenum;\n    readonly PACK_SKIP_PIXELS: GLenum;\n    readonly PACK_SKIP_ROWS: GLenum;\n    readonly PIXEL_PACK_BUFFER: GLenum;\n    readonly PIXEL_PACK_BUFFER_BINDING: GLenum;\n    readonly PIXEL_UNPACK_BUFFER: GLenum;\n    readonly PIXEL_UNPACK_BUFFER_BINDING: GLenum;\n    readonly QUERY_RESULT: GLenum;\n    readonly QUERY_RESULT_AVAILABLE: GLenum;\n    readonly R11F_G11F_B10F: GLenum;\n    readonly R16F: GLenum;\n    readonly R16I: GLenum;\n    readonly R16UI: GLenum;\n    readonly R32F: GLenum;\n    readonly R32I: GLenum;\n    readonly R32UI: GLenum;\n    readonly R8: GLenum;\n    readonly R8I: GLenum;\n    readonly R8UI: GLenum;\n    readonly R8_SNORM: GLenum;\n    readonly RASTERIZER_DISCARD: GLenum;\n    readonly READ_BUFFER: GLenum;\n    readonly READ_FRAMEBUFFER: GLenum;\n    readonly READ_FRAMEBUFFER_BINDING: GLenum;\n    readonly RED: GLenum;\n    readonly RED_INTEGER: GLenum;\n    readonly RENDERBUFFER_SAMPLES: GLenum;\n    readonly RG: GLenum;\n    readonly RG16F: GLenum;\n    readonly RG16I: GLenum;\n    readonly RG16UI: GLenum;\n    readonly RG32F: GLenum;\n    readonly RG32I: GLenum;\n    readonly RG32UI: GLenum;\n    readonly RG8: GLenum;\n    readonly RG8I: GLenum;\n    readonly RG8UI: GLenum;\n    readonly RG8_SNORM: GLenum;\n    readonly RGB10_A2: GLenum;\n    readonly RGB10_A2UI: GLenum;\n    readonly RGB16F: GLenum;\n    readonly RGB16I: GLenum;\n    readonly RGB16UI: GLenum;\n    readonly RGB32F: GLenum;\n    readonly RGB32I: GLenum;\n    readonly RGB32UI: GLenum;\n    readonly RGB8: GLenum;\n    readonly RGB8I: GLenum;\n    readonly RGB8UI: GLenum;\n    readonly RGB8_SNORM: GLenum;\n    readonly RGB9_E5: GLenum;\n    readonly RGBA16F: GLenum;\n    readonly RGBA16I: GLenum;\n    readonly RGBA16UI: GLenum;\n    readonly RGBA32F: GLenum;\n    readonly RGBA32I: GLenum;\n    readonly RGBA32UI: GLenum;\n    readonly RGBA8: GLenum;\n    readonly RGBA8I: GLenum;\n    readonly RGBA8UI: GLenum;\n    readonly RGBA8_SNORM: GLenum;\n    readonly RGBA_INTEGER: GLenum;\n    readonly RGB_INTEGER: GLenum;\n    readonly RG_INTEGER: GLenum;\n    readonly SAMPLER_2D_ARRAY: GLenum;\n    readonly SAMPLER_2D_ARRAY_SHADOW: GLenum;\n    readonly SAMPLER_2D_SHADOW: GLenum;\n    readonly SAMPLER_3D: GLenum;\n    readonly SAMPLER_BINDING: GLenum;\n    readonly SAMPLER_CUBE_SHADOW: GLenum;\n    readonly SEPARATE_ATTRIBS: GLenum;\n    readonly SIGNALED: GLenum;\n    readonly SIGNED_NORMALIZED: GLenum;\n    readonly SRGB: GLenum;\n    readonly SRGB8: GLenum;\n    readonly SRGB8_ALPHA8: GLenum;\n    readonly STATIC_COPY: GLenum;\n    readonly STATIC_READ: GLenum;\n    readonly STENCIL: GLenum;\n    readonly STREAM_COPY: GLenum;\n    readonly STREAM_READ: GLenum;\n    readonly SYNC_CONDITION: GLenum;\n    readonly SYNC_FENCE: GLenum;\n    readonly SYNC_FLAGS: GLenum;\n    readonly SYNC_FLUSH_COMMANDS_BIT: GLenum;\n    readonly SYNC_GPU_COMMANDS_COMPLETE: GLenum;\n    readonly SYNC_STATUS: GLenum;\n    readonly TEXTURE_2D_ARRAY: GLenum;\n    readonly TEXTURE_3D: GLenum;\n    readonly TEXTURE_BASE_LEVEL: GLenum;\n    readonly TEXTURE_BINDING_2D_ARRAY: GLenum;\n    readonly TEXTURE_BINDING_3D: GLenum;\n    readonly TEXTURE_COMPARE_FUNC: GLenum;\n    readonly TEXTURE_COMPARE_MODE: GLenum;\n    readonly TEXTURE_IMMUTABLE_FORMAT: GLenum;\n    readonly TEXTURE_IMMUTABLE_LEVELS: GLenum;\n    readonly TEXTURE_MAX_LEVEL: GLenum;\n    readonly TEXTURE_MAX_LOD: GLenum;\n    readonly TEXTURE_MIN_LOD: GLenum;\n    readonly TEXTURE_WRAP_R: GLenum;\n    readonly TIMEOUT_EXPIRED: GLenum;\n    readonly TIMEOUT_IGNORED: GLint64;\n    readonly TRANSFORM_FEEDBACK: GLenum;\n    readonly TRANSFORM_FEEDBACK_ACTIVE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BINDING: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_BINDING: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_MODE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_SIZE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_START: GLenum;\n    readonly TRANSFORM_FEEDBACK_PAUSED: GLenum;\n    readonly TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN: GLenum;\n    readonly TRANSFORM_FEEDBACK_VARYINGS: GLenum;\n    readonly UNIFORM_ARRAY_STRIDE: GLenum;\n    readonly UNIFORM_BLOCK_ACTIVE_UNIFORMS: GLenum;\n    readonly UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES: GLenum;\n    readonly UNIFORM_BLOCK_BINDING: GLenum;\n    readonly UNIFORM_BLOCK_DATA_SIZE: GLenum;\n    readonly UNIFORM_BLOCK_INDEX: GLenum;\n    readonly UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER: GLenum;\n    readonly UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER: GLenum;\n    readonly UNIFORM_BUFFER: GLenum;\n    readonly UNIFORM_BUFFER_BINDING: GLenum;\n    readonly UNIFORM_BUFFER_OFFSET_ALIGNMENT: GLenum;\n    readonly UNIFORM_BUFFER_SIZE: GLenum;\n    readonly UNIFORM_BUFFER_START: GLenum;\n    readonly UNIFORM_IS_ROW_MAJOR: GLenum;\n    readonly UNIFORM_MATRIX_STRIDE: GLenum;\n    readonly UNIFORM_OFFSET: GLenum;\n    readonly UNIFORM_SIZE: GLenum;\n    readonly UNIFORM_TYPE: GLenum;\n    readonly UNPACK_IMAGE_HEIGHT: GLenum;\n    readonly UNPACK_ROW_LENGTH: GLenum;\n    readonly UNPACK_SKIP_IMAGES: GLenum;\n    readonly UNPACK_SKIP_PIXELS: GLenum;\n    readonly UNPACK_SKIP_ROWS: GLenum;\n    readonly UNSIGNALED: GLenum;\n    readonly UNSIGNED_INT_10F_11F_11F_REV: GLenum;\n    readonly UNSIGNED_INT_24_8: GLenum;\n    readonly UNSIGNED_INT_2_10_10_10_REV: GLenum;\n    readonly UNSIGNED_INT_5_9_9_9_REV: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_2D: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_2D_ARRAY: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_3D: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_CUBE: GLenum;\n    readonly UNSIGNED_INT_VEC2: GLenum;\n    readonly UNSIGNED_INT_VEC3: GLenum;\n    readonly UNSIGNED_INT_VEC4: GLenum;\n    readonly UNSIGNED_NORMALIZED: GLenum;\n    readonly VERTEX_ARRAY_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_INTEGER: GLenum;\n    readonly WAIT_FAILED: GLenum;\n    readonly ACTIVE_ATTRIBUTES: GLenum;\n    readonly ACTIVE_TEXTURE: GLenum;\n    readonly ACTIVE_UNIFORMS: GLenum;\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\n    readonly ALPHA: GLenum;\n    readonly ALPHA_BITS: GLenum;\n    readonly ALWAYS: GLenum;\n    readonly ARRAY_BUFFER: GLenum;\n    readonly ARRAY_BUFFER_BINDING: GLenum;\n    readonly ATTACHED_SHADERS: GLenum;\n    readonly BACK: GLenum;\n    readonly BLEND: GLenum;\n    readonly BLEND_COLOR: GLenum;\n    readonly BLEND_DST_ALPHA: GLenum;\n    readonly BLEND_DST_RGB: GLenum;\n    readonly BLEND_EQUATION: GLenum;\n    readonly BLEND_EQUATION_ALPHA: GLenum;\n    readonly BLEND_EQUATION_RGB: GLenum;\n    readonly BLEND_SRC_ALPHA: GLenum;\n    readonly BLEND_SRC_RGB: GLenum;\n    readonly BLUE_BITS: GLenum;\n    readonly BOOL: GLenum;\n    readonly BOOL_VEC2: GLenum;\n    readonly BOOL_VEC3: GLenum;\n    readonly BOOL_VEC4: GLenum;\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\n    readonly BUFFER_SIZE: GLenum;\n    readonly BUFFER_USAGE: GLenum;\n    readonly BYTE: GLenum;\n    readonly CCW: GLenum;\n    readonly CLAMP_TO_EDGE: GLenum;\n    readonly COLOR_ATTACHMENT0: GLenum;\n    readonly COLOR_BUFFER_BIT: GLenum;\n    readonly COLOR_CLEAR_VALUE: GLenum;\n    readonly COLOR_WRITEMASK: GLenum;\n    readonly COMPILE_STATUS: GLenum;\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\n    readonly CONSTANT_ALPHA: GLenum;\n    readonly CONSTANT_COLOR: GLenum;\n    readonly CONTEXT_LOST_WEBGL: GLenum;\n    readonly CULL_FACE: GLenum;\n    readonly CULL_FACE_MODE: GLenum;\n    readonly CURRENT_PROGRAM: GLenum;\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\n    readonly CW: GLenum;\n    readonly DECR: GLenum;\n    readonly DECR_WRAP: GLenum;\n    readonly DELETE_STATUS: GLenum;\n    readonly DEPTH_ATTACHMENT: GLenum;\n    readonly DEPTH_BITS: GLenum;\n    readonly DEPTH_BUFFER_BIT: GLenum;\n    readonly DEPTH_CLEAR_VALUE: GLenum;\n    readonly DEPTH_COMPONENT: GLenum;\n    readonly DEPTH_COMPONENT16: GLenum;\n    readonly DEPTH_FUNC: GLenum;\n    readonly DEPTH_RANGE: GLenum;\n    readonly DEPTH_STENCIL: GLenum;\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\n    readonly DEPTH_TEST: GLenum;\n    readonly DEPTH_WRITEMASK: GLenum;\n    readonly DITHER: GLenum;\n    readonly DONT_CARE: GLenum;\n    readonly DST_ALPHA: GLenum;\n    readonly DST_COLOR: GLenum;\n    readonly DYNAMIC_DRAW: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\n    readonly EQUAL: GLenum;\n    readonly FASTEST: GLenum;\n    readonly FLOAT: GLenum;\n    readonly FLOAT_MAT2: GLenum;\n    readonly FLOAT_MAT3: GLenum;\n    readonly FLOAT_MAT4: GLenum;\n    readonly FLOAT_VEC2: GLenum;\n    readonly FLOAT_VEC3: GLenum;\n    readonly FLOAT_VEC4: GLenum;\n    readonly FRAGMENT_SHADER: GLenum;\n    readonly FRAMEBUFFER: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\n    readonly FRAMEBUFFER_BINDING: GLenum;\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\n    readonly FRONT: GLenum;\n    readonly FRONT_AND_BACK: GLenum;\n    readonly FRONT_FACE: GLenum;\n    readonly FUNC_ADD: GLenum;\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\n    readonly FUNC_SUBTRACT: GLenum;\n    readonly GENERATE_MIPMAP_HINT: GLenum;\n    readonly GEQUAL: GLenum;\n    readonly GREATER: GLenum;\n    readonly GREEN_BITS: GLenum;\n    readonly HIGH_FLOAT: GLenum;\n    readonly HIGH_INT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\n    readonly INCR: GLenum;\n    readonly INCR_WRAP: GLenum;\n    readonly INT: GLenum;\n    readonly INT_VEC2: GLenum;\n    readonly INT_VEC3: GLenum;\n    readonly INT_VEC4: GLenum;\n    readonly INVALID_ENUM: GLenum;\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\n    readonly INVALID_OPERATION: GLenum;\n    readonly INVALID_VALUE: GLenum;\n    readonly INVERT: GLenum;\n    readonly KEEP: GLenum;\n    readonly LEQUAL: GLenum;\n    readonly LESS: GLenum;\n    readonly LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\n    readonly LINES: GLenum;\n    readonly LINE_LOOP: GLenum;\n    readonly LINE_STRIP: GLenum;\n    readonly LINE_WIDTH: GLenum;\n    readonly LINK_STATUS: GLenum;\n    readonly LOW_FLOAT: GLenum;\n    readonly LOW_INT: GLenum;\n    readonly LUMINANCE: GLenum;\n    readonly LUMINANCE_ALPHA: GLenum;\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_TEXTURE_SIZE: GLenum;\n    readonly MAX_VARYING_VECTORS: GLenum;\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\n    readonly MAX_VIEWPORT_DIMS: GLenum;\n    readonly MEDIUM_FLOAT: GLenum;\n    readonly MEDIUM_INT: GLenum;\n    readonly MIRRORED_REPEAT: GLenum;\n    readonly NEAREST: GLenum;\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\n    readonly NEVER: GLenum;\n    readonly NICEST: GLenum;\n    readonly NONE: GLenum;\n    readonly NOTEQUAL: GLenum;\n    readonly NO_ERROR: GLenum;\n    readonly ONE: GLenum;\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\n    readonly ONE_MINUS_DST_COLOR: GLenum;\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\n    readonly OUT_OF_MEMORY: GLenum;\n    readonly PACK_ALIGNMENT: GLenum;\n    readonly POINTS: GLenum;\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\n    readonly POLYGON_OFFSET_FILL: GLenum;\n    readonly POLYGON_OFFSET_UNITS: GLenum;\n    readonly RED_BITS: GLenum;\n    readonly RENDERBUFFER: GLenum;\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\n    readonly RENDERBUFFER_BINDING: GLenum;\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\n    readonly RENDERBUFFER_HEIGHT: GLenum;\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\n    readonly RENDERBUFFER_WIDTH: GLenum;\n    readonly RENDERER: GLenum;\n    readonly REPEAT: GLenum;\n    readonly REPLACE: GLenum;\n    readonly RGB: GLenum;\n    readonly RGB565: GLenum;\n    readonly RGB5_A1: GLenum;\n    readonly RGBA: GLenum;\n    readonly RGBA4: GLenum;\n    readonly SAMPLER_2D: GLenum;\n    readonly SAMPLER_CUBE: GLenum;\n    readonly SAMPLES: GLenum;\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\n    readonly SAMPLE_BUFFERS: GLenum;\n    readonly SAMPLE_COVERAGE: GLenum;\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\n    readonly SCISSOR_BOX: GLenum;\n    readonly SCISSOR_TEST: GLenum;\n    readonly SHADER_TYPE: GLenum;\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\n    readonly SHORT: GLenum;\n    readonly SRC_ALPHA: GLenum;\n    readonly SRC_ALPHA_SATURATE: GLenum;\n    readonly SRC_COLOR: GLenum;\n    readonly STATIC_DRAW: GLenum;\n    readonly STENCIL_ATTACHMENT: GLenum;\n    readonly STENCIL_BACK_FAIL: GLenum;\n    readonly STENCIL_BACK_FUNC: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_BACK_REF: GLenum;\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\n    readonly STENCIL_BITS: GLenum;\n    readonly STENCIL_BUFFER_BIT: GLenum;\n    readonly STENCIL_CLEAR_VALUE: GLenum;\n    readonly STENCIL_FAIL: GLenum;\n    readonly STENCIL_FUNC: GLenum;\n    readonly STENCIL_INDEX8: GLenum;\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_REF: GLenum;\n    readonly STENCIL_TEST: GLenum;\n    readonly STENCIL_VALUE_MASK: GLenum;\n    readonly STENCIL_WRITEMASK: GLenum;\n    readonly STREAM_DRAW: GLenum;\n    readonly SUBPIXEL_BITS: GLenum;\n    readonly TEXTURE: GLenum;\n    readonly TEXTURE0: GLenum;\n    readonly TEXTURE1: GLenum;\n    readonly TEXTURE10: GLenum;\n    readonly TEXTURE11: GLenum;\n    readonly TEXTURE12: GLenum;\n    readonly TEXTURE13: GLenum;\n    readonly TEXTURE14: GLenum;\n    readonly TEXTURE15: GLenum;\n    readonly TEXTURE16: GLenum;\n    readonly TEXTURE17: GLenum;\n    readonly TEXTURE18: GLenum;\n    readonly TEXTURE19: GLenum;\n    readonly TEXTURE2: GLenum;\n    readonly TEXTURE20: GLenum;\n    readonly TEXTURE21: GLenum;\n    readonly TEXTURE22: GLenum;\n    readonly TEXTURE23: GLenum;\n    readonly TEXTURE24: GLenum;\n    readonly TEXTURE25: GLenum;\n    readonly TEXTURE26: GLenum;\n    readonly TEXTURE27: GLenum;\n    readonly TEXTURE28: GLenum;\n    readonly TEXTURE29: GLenum;\n    readonly TEXTURE3: GLenum;\n    readonly TEXTURE30: GLenum;\n    readonly TEXTURE31: GLenum;\n    readonly TEXTURE4: GLenum;\n    readonly TEXTURE5: GLenum;\n    readonly TEXTURE6: GLenum;\n    readonly TEXTURE7: GLenum;\n    readonly TEXTURE8: GLenum;\n    readonly TEXTURE9: GLenum;\n    readonly TEXTURE_2D: GLenum;\n    readonly TEXTURE_BINDING_2D: GLenum;\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\n    readonly TEXTURE_MAG_FILTER: GLenum;\n    readonly TEXTURE_MIN_FILTER: GLenum;\n    readonly TEXTURE_WRAP_S: GLenum;\n    readonly TEXTURE_WRAP_T: GLenum;\n    readonly TRIANGLES: GLenum;\n    readonly TRIANGLE_FAN: GLenum;\n    readonly TRIANGLE_STRIP: GLenum;\n    readonly UNPACK_ALIGNMENT: GLenum;\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\n    readonly UNSIGNED_BYTE: GLenum;\n    readonly UNSIGNED_INT: GLenum;\n    readonly UNSIGNED_SHORT: GLenum;\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\n    readonly VALIDATE_STATUS: GLenum;\n    readonly VENDOR: GLenum;\n    readonly VERSION: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\n    readonly VERTEX_SHADER: GLenum;\n    readonly VIEWPORT: GLenum;\n    readonly ZERO: GLenum;\n};\n\ninterface WebGL2RenderingContextBase {\n    beginQuery(target: GLenum, query: WebGLQuery): void;\n    beginTransformFeedback(primitiveMode: GLenum): void;\n    bindBufferBase(target: GLenum, index: GLuint, buffer: WebGLBuffer | null): void;\n    bindBufferRange(target: GLenum, index: GLuint, buffer: WebGLBuffer | null, offset: GLintptr, size: GLsizeiptr): void;\n    bindSampler(unit: GLuint, sampler: WebGLSampler | null): void;\n    bindTransformFeedback(target: GLenum, tf: WebGLTransformFeedback | null): void;\n    bindVertexArray(array: WebGLVertexArrayObject | null): void;\n    blitFramebuffer(srcX0: GLint, srcY0: GLint, srcX1: GLint, srcY1: GLint, dstX0: GLint, dstY0: GLint, dstX1: GLint, dstY1: GLint, mask: GLbitfield, filter: GLenum): void;\n    clearBufferfi(buffer: GLenum, drawbuffer: GLint, depth: GLfloat, stencil: GLint): void;\n    clearBufferfv(buffer: GLenum, drawbuffer: GLint, values: Float32List, srcOffset?: GLuint): void;\n    clearBufferiv(buffer: GLenum, drawbuffer: GLint, values: Int32List, srcOffset?: GLuint): void;\n    clearBufferuiv(buffer: GLenum, drawbuffer: GLint, values: Uint32List, srcOffset?: GLuint): void;\n    clientWaitSync(sync: WebGLSync, flags: GLbitfield, timeout: GLuint64): GLenum;\n    compressedTexImage3D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, imageSize: GLsizei, offset: GLintptr): void;\n    compressedTexImage3D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, srcData: ArrayBufferView, srcOffset?: GLuint, srcLengthOverride?: GLuint): void;\n    compressedTexSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, imageSize: GLsizei, offset: GLintptr): void;\n    compressedTexSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, srcData: ArrayBufferView, srcOffset?: GLuint, srcLengthOverride?: GLuint): void;\n    copyBufferSubData(readTarget: GLenum, writeTarget: GLenum, readOffset: GLintptr, writeOffset: GLintptr, size: GLsizeiptr): void;\n    copyTexSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    createQuery(): WebGLQuery | null;\n    createSampler(): WebGLSampler | null;\n    createTransformFeedback(): WebGLTransformFeedback | null;\n    createVertexArray(): WebGLVertexArrayObject | null;\n    deleteQuery(query: WebGLQuery | null): void;\n    deleteSampler(sampler: WebGLSampler | null): void;\n    deleteSync(sync: WebGLSync | null): void;\n    deleteTransformFeedback(tf: WebGLTransformFeedback | null): void;\n    deleteVertexArray(vertexArray: WebGLVertexArrayObject | null): void;\n    drawArraysInstanced(mode: GLenum, first: GLint, count: GLsizei, instanceCount: GLsizei): void;\n    drawBuffers(buffers: GLenum[]): void;\n    drawElementsInstanced(mode: GLenum, count: GLsizei, type: GLenum, offset: GLintptr, instanceCount: GLsizei): void;\n    drawRangeElements(mode: GLenum, start: GLuint, end: GLuint, count: GLsizei, type: GLenum, offset: GLintptr): void;\n    endQuery(target: GLenum): void;\n    endTransformFeedback(): void;\n    fenceSync(condition: GLenum, flags: GLbitfield): WebGLSync | null;\n    framebufferTextureLayer(target: GLenum, attachment: GLenum, texture: WebGLTexture | null, level: GLint, layer: GLint): void;\n    getActiveUniformBlockName(program: WebGLProgram, uniformBlockIndex: GLuint): string | null;\n    getActiveUniformBlockParameter(program: WebGLProgram, uniformBlockIndex: GLuint, pname: GLenum): any;\n    getActiveUniforms(program: WebGLProgram, uniformIndices: GLuint[], pname: GLenum): any;\n    getBufferSubData(target: GLenum, srcByteOffset: GLintptr, dstBuffer: ArrayBufferView, dstOffset?: GLuint, length?: GLuint): void;\n    getFragDataLocation(program: WebGLProgram, name: string): GLint;\n    getIndexedParameter(target: GLenum, index: GLuint): any;\n    getInternalformatParameter(target: GLenum, internalformat: GLenum, pname: GLenum): any;\n    getQuery(target: GLenum, pname: GLenum): WebGLQuery | null;\n    getQueryParameter(query: WebGLQuery, pname: GLenum): any;\n    getSamplerParameter(sampler: WebGLSampler, pname: GLenum): any;\n    getSyncParameter(sync: WebGLSync, pname: GLenum): any;\n    getTransformFeedbackVarying(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\n    getUniformBlockIndex(program: WebGLProgram, uniformBlockName: string): GLuint;\n    getUniformIndices(program: WebGLProgram, uniformNames: string[]): GLuint[] | null;\n    invalidateFramebuffer(target: GLenum, attachments: GLenum[]): void;\n    invalidateSubFramebuffer(target: GLenum, attachments: GLenum[], x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    isQuery(query: WebGLQuery | null): GLboolean;\n    isSampler(sampler: WebGLSampler | null): GLboolean;\n    isSync(sync: WebGLSync | null): GLboolean;\n    isTransformFeedback(tf: WebGLTransformFeedback | null): GLboolean;\n    isVertexArray(vertexArray: WebGLVertexArrayObject | null): GLboolean;\n    pauseTransformFeedback(): void;\n    readBuffer(src: GLenum): void;\n    renderbufferStorageMultisample(target: GLenum, samples: GLsizei, internalformat: GLenum, width: GLsizei, height: GLsizei): void;\n    resumeTransformFeedback(): void;\n    samplerParameterf(sampler: WebGLSampler, pname: GLenum, param: GLfloat): void;\n    samplerParameteri(sampler: WebGLSampler, pname: GLenum, param: GLint): void;\n    texImage3D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, format: GLenum, type: GLenum, pboOffset: GLintptr): void;\n    texImage3D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texImage3D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, format: GLenum, type: GLenum, srcData: ArrayBufferView | null): void;\n    texImage3D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, border: GLint, format: GLenum, type: GLenum, srcData: ArrayBufferView, srcOffset: GLuint): void;\n    texStorage2D(target: GLenum, levels: GLsizei, internalformat: GLenum, width: GLsizei, height: GLsizei): void;\n    texStorage3D(target: GLenum, levels: GLsizei, internalformat: GLenum, width: GLsizei, height: GLsizei, depth: GLsizei): void;\n    texSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, type: GLenum, pboOffset: GLintptr): void;\n    texSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texSubImage3D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, zoffset: GLint, width: GLsizei, height: GLsizei, depth: GLsizei, format: GLenum, type: GLenum, srcData: ArrayBufferView | null, srcOffset?: GLuint): void;\n    transformFeedbackVaryings(program: WebGLProgram, varyings: string[], bufferMode: GLenum): void;\n    uniform1ui(location: WebGLUniformLocation | null, v0: GLuint): void;\n    uniform1uiv(location: WebGLUniformLocation | null, data: Uint32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2ui(location: WebGLUniformLocation | null, v0: GLuint, v1: GLuint): void;\n    uniform2uiv(location: WebGLUniformLocation | null, data: Uint32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3ui(location: WebGLUniformLocation | null, v0: GLuint, v1: GLuint, v2: GLuint): void;\n    uniform3uiv(location: WebGLUniformLocation | null, data: Uint32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4ui(location: WebGLUniformLocation | null, v0: GLuint, v1: GLuint, v2: GLuint, v3: GLuint): void;\n    uniform4uiv(location: WebGLUniformLocation | null, data: Uint32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformBlockBinding(program: WebGLProgram, uniformBlockIndex: GLuint, uniformBlockBinding: GLuint): void;\n    uniformMatrix2x3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2x4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3x2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3x4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4x2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4x3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    vertexAttribDivisor(index: GLuint, divisor: GLuint): void;\n    vertexAttribI4i(index: GLuint, x: GLint, y: GLint, z: GLint, w: GLint): void;\n    vertexAttribI4iv(index: GLuint, values: Int32List): void;\n    vertexAttribI4ui(index: GLuint, x: GLuint, y: GLuint, z: GLuint, w: GLuint): void;\n    vertexAttribI4uiv(index: GLuint, values: Uint32List): void;\n    vertexAttribIPointer(index: GLuint, size: GLint, type: GLenum, stride: GLsizei, offset: GLintptr): void;\n    waitSync(sync: WebGLSync, flags: GLbitfield, timeout: GLint64): void;\n    readonly ACTIVE_UNIFORM_BLOCKS: GLenum;\n    readonly ALREADY_SIGNALED: GLenum;\n    readonly ANY_SAMPLES_PASSED: GLenum;\n    readonly ANY_SAMPLES_PASSED_CONSERVATIVE: GLenum;\n    readonly COLOR: GLenum;\n    readonly COLOR_ATTACHMENT1: GLenum;\n    readonly COLOR_ATTACHMENT10: GLenum;\n    readonly COLOR_ATTACHMENT11: GLenum;\n    readonly COLOR_ATTACHMENT12: GLenum;\n    readonly COLOR_ATTACHMENT13: GLenum;\n    readonly COLOR_ATTACHMENT14: GLenum;\n    readonly COLOR_ATTACHMENT15: GLenum;\n    readonly COLOR_ATTACHMENT2: GLenum;\n    readonly COLOR_ATTACHMENT3: GLenum;\n    readonly COLOR_ATTACHMENT4: GLenum;\n    readonly COLOR_ATTACHMENT5: GLenum;\n    readonly COLOR_ATTACHMENT6: GLenum;\n    readonly COLOR_ATTACHMENT7: GLenum;\n    readonly COLOR_ATTACHMENT8: GLenum;\n    readonly COLOR_ATTACHMENT9: GLenum;\n    readonly COMPARE_REF_TO_TEXTURE: GLenum;\n    readonly CONDITION_SATISFIED: GLenum;\n    readonly COPY_READ_BUFFER: GLenum;\n    readonly COPY_READ_BUFFER_BINDING: GLenum;\n    readonly COPY_WRITE_BUFFER: GLenum;\n    readonly COPY_WRITE_BUFFER_BINDING: GLenum;\n    readonly CURRENT_QUERY: GLenum;\n    readonly DEPTH: GLenum;\n    readonly DEPTH24_STENCIL8: GLenum;\n    readonly DEPTH32F_STENCIL8: GLenum;\n    readonly DEPTH_COMPONENT24: GLenum;\n    readonly DEPTH_COMPONENT32F: GLenum;\n    readonly DRAW_BUFFER0: GLenum;\n    readonly DRAW_BUFFER1: GLenum;\n    readonly DRAW_BUFFER10: GLenum;\n    readonly DRAW_BUFFER11: GLenum;\n    readonly DRAW_BUFFER12: GLenum;\n    readonly DRAW_BUFFER13: GLenum;\n    readonly DRAW_BUFFER14: GLenum;\n    readonly DRAW_BUFFER15: GLenum;\n    readonly DRAW_BUFFER2: GLenum;\n    readonly DRAW_BUFFER3: GLenum;\n    readonly DRAW_BUFFER4: GLenum;\n    readonly DRAW_BUFFER5: GLenum;\n    readonly DRAW_BUFFER6: GLenum;\n    readonly DRAW_BUFFER7: GLenum;\n    readonly DRAW_BUFFER8: GLenum;\n    readonly DRAW_BUFFER9: GLenum;\n    readonly DRAW_FRAMEBUFFER: GLenum;\n    readonly DRAW_FRAMEBUFFER_BINDING: GLenum;\n    readonly DYNAMIC_COPY: GLenum;\n    readonly DYNAMIC_READ: GLenum;\n    readonly FLOAT_32_UNSIGNED_INT_24_8_REV: GLenum;\n    readonly FLOAT_MAT2x3: GLenum;\n    readonly FLOAT_MAT2x4: GLenum;\n    readonly FLOAT_MAT3x2: GLenum;\n    readonly FLOAT_MAT3x4: GLenum;\n    readonly FLOAT_MAT4x2: GLenum;\n    readonly FLOAT_MAT4x3: GLenum;\n    readonly FRAGMENT_SHADER_DERIVATIVE_HINT: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_BLUE_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_GREEN_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_RED_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER: GLenum;\n    readonly FRAMEBUFFER_DEFAULT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MULTISAMPLE: GLenum;\n    readonly HALF_FLOAT: GLenum;\n    readonly INTERLEAVED_ATTRIBS: GLenum;\n    readonly INT_2_10_10_10_REV: GLenum;\n    readonly INT_SAMPLER_2D: GLenum;\n    readonly INT_SAMPLER_2D_ARRAY: GLenum;\n    readonly INT_SAMPLER_3D: GLenum;\n    readonly INT_SAMPLER_CUBE: GLenum;\n    readonly INVALID_INDEX: GLenum;\n    readonly MAX: GLenum;\n    readonly MAX_3D_TEXTURE_SIZE: GLenum;\n    readonly MAX_ARRAY_TEXTURE_LAYERS: GLenum;\n    readonly MAX_CLIENT_WAIT_TIMEOUT_WEBGL: GLenum;\n    readonly MAX_COLOR_ATTACHMENTS: GLenum;\n    readonly MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_COMBINED_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_DRAW_BUFFERS: GLenum;\n    readonly MAX_ELEMENTS_INDICES: GLenum;\n    readonly MAX_ELEMENTS_VERTICES: GLenum;\n    readonly MAX_ELEMENT_INDEX: GLenum;\n    readonly MAX_FRAGMENT_INPUT_COMPONENTS: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_COMPONENTS: GLenum;\n    readonly MAX_PROGRAM_TEXEL_OFFSET: GLenum;\n    readonly MAX_SAMPLES: GLenum;\n    readonly MAX_SERVER_WAIT_TIMEOUT: GLenum;\n    readonly MAX_TEXTURE_LOD_BIAS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS: GLenum;\n    readonly MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS: GLenum;\n    readonly MAX_UNIFORM_BLOCK_SIZE: GLenum;\n    readonly MAX_UNIFORM_BUFFER_BINDINGS: GLenum;\n    readonly MAX_VARYING_COMPONENTS: GLenum;\n    readonly MAX_VERTEX_OUTPUT_COMPONENTS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_BLOCKS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_COMPONENTS: GLenum;\n    readonly MIN: GLenum;\n    readonly MIN_PROGRAM_TEXEL_OFFSET: GLenum;\n    readonly OBJECT_TYPE: GLenum;\n    readonly PACK_ROW_LENGTH: GLenum;\n    readonly PACK_SKIP_PIXELS: GLenum;\n    readonly PACK_SKIP_ROWS: GLenum;\n    readonly PIXEL_PACK_BUFFER: GLenum;\n    readonly PIXEL_PACK_BUFFER_BINDING: GLenum;\n    readonly PIXEL_UNPACK_BUFFER: GLenum;\n    readonly PIXEL_UNPACK_BUFFER_BINDING: GLenum;\n    readonly QUERY_RESULT: GLenum;\n    readonly QUERY_RESULT_AVAILABLE: GLenum;\n    readonly R11F_G11F_B10F: GLenum;\n    readonly R16F: GLenum;\n    readonly R16I: GLenum;\n    readonly R16UI: GLenum;\n    readonly R32F: GLenum;\n    readonly R32I: GLenum;\n    readonly R32UI: GLenum;\n    readonly R8: GLenum;\n    readonly R8I: GLenum;\n    readonly R8UI: GLenum;\n    readonly R8_SNORM: GLenum;\n    readonly RASTERIZER_DISCARD: GLenum;\n    readonly READ_BUFFER: GLenum;\n    readonly READ_FRAMEBUFFER: GLenum;\n    readonly READ_FRAMEBUFFER_BINDING: GLenum;\n    readonly RED: GLenum;\n    readonly RED_INTEGER: GLenum;\n    readonly RENDERBUFFER_SAMPLES: GLenum;\n    readonly RG: GLenum;\n    readonly RG16F: GLenum;\n    readonly RG16I: GLenum;\n    readonly RG16UI: GLenum;\n    readonly RG32F: GLenum;\n    readonly RG32I: GLenum;\n    readonly RG32UI: GLenum;\n    readonly RG8: GLenum;\n    readonly RG8I: GLenum;\n    readonly RG8UI: GLenum;\n    readonly RG8_SNORM: GLenum;\n    readonly RGB10_A2: GLenum;\n    readonly RGB10_A2UI: GLenum;\n    readonly RGB16F: GLenum;\n    readonly RGB16I: GLenum;\n    readonly RGB16UI: GLenum;\n    readonly RGB32F: GLenum;\n    readonly RGB32I: GLenum;\n    readonly RGB32UI: GLenum;\n    readonly RGB8: GLenum;\n    readonly RGB8I: GLenum;\n    readonly RGB8UI: GLenum;\n    readonly RGB8_SNORM: GLenum;\n    readonly RGB9_E5: GLenum;\n    readonly RGBA16F: GLenum;\n    readonly RGBA16I: GLenum;\n    readonly RGBA16UI: GLenum;\n    readonly RGBA32F: GLenum;\n    readonly RGBA32I: GLenum;\n    readonly RGBA32UI: GLenum;\n    readonly RGBA8: GLenum;\n    readonly RGBA8I: GLenum;\n    readonly RGBA8UI: GLenum;\n    readonly RGBA8_SNORM: GLenum;\n    readonly RGBA_INTEGER: GLenum;\n    readonly RGB_INTEGER: GLenum;\n    readonly RG_INTEGER: GLenum;\n    readonly SAMPLER_2D_ARRAY: GLenum;\n    readonly SAMPLER_2D_ARRAY_SHADOW: GLenum;\n    readonly SAMPLER_2D_SHADOW: GLenum;\n    readonly SAMPLER_3D: GLenum;\n    readonly SAMPLER_BINDING: GLenum;\n    readonly SAMPLER_CUBE_SHADOW: GLenum;\n    readonly SEPARATE_ATTRIBS: GLenum;\n    readonly SIGNALED: GLenum;\n    readonly SIGNED_NORMALIZED: GLenum;\n    readonly SRGB: GLenum;\n    readonly SRGB8: GLenum;\n    readonly SRGB8_ALPHA8: GLenum;\n    readonly STATIC_COPY: GLenum;\n    readonly STATIC_READ: GLenum;\n    readonly STENCIL: GLenum;\n    readonly STREAM_COPY: GLenum;\n    readonly STREAM_READ: GLenum;\n    readonly SYNC_CONDITION: GLenum;\n    readonly SYNC_FENCE: GLenum;\n    readonly SYNC_FLAGS: GLenum;\n    readonly SYNC_FLUSH_COMMANDS_BIT: GLenum;\n    readonly SYNC_GPU_COMMANDS_COMPLETE: GLenum;\n    readonly SYNC_STATUS: GLenum;\n    readonly TEXTURE_2D_ARRAY: GLenum;\n    readonly TEXTURE_3D: GLenum;\n    readonly TEXTURE_BASE_LEVEL: GLenum;\n    readonly TEXTURE_BINDING_2D_ARRAY: GLenum;\n    readonly TEXTURE_BINDING_3D: GLenum;\n    readonly TEXTURE_COMPARE_FUNC: GLenum;\n    readonly TEXTURE_COMPARE_MODE: GLenum;\n    readonly TEXTURE_IMMUTABLE_FORMAT: GLenum;\n    readonly TEXTURE_IMMUTABLE_LEVELS: GLenum;\n    readonly TEXTURE_MAX_LEVEL: GLenum;\n    readonly TEXTURE_MAX_LOD: GLenum;\n    readonly TEXTURE_MIN_LOD: GLenum;\n    readonly TEXTURE_WRAP_R: GLenum;\n    readonly TIMEOUT_EXPIRED: GLenum;\n    readonly TIMEOUT_IGNORED: GLint64;\n    readonly TRANSFORM_FEEDBACK: GLenum;\n    readonly TRANSFORM_FEEDBACK_ACTIVE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BINDING: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_BINDING: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_MODE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_SIZE: GLenum;\n    readonly TRANSFORM_FEEDBACK_BUFFER_START: GLenum;\n    readonly TRANSFORM_FEEDBACK_PAUSED: GLenum;\n    readonly TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN: GLenum;\n    readonly TRANSFORM_FEEDBACK_VARYINGS: GLenum;\n    readonly UNIFORM_ARRAY_STRIDE: GLenum;\n    readonly UNIFORM_BLOCK_ACTIVE_UNIFORMS: GLenum;\n    readonly UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES: GLenum;\n    readonly UNIFORM_BLOCK_BINDING: GLenum;\n    readonly UNIFORM_BLOCK_DATA_SIZE: GLenum;\n    readonly UNIFORM_BLOCK_INDEX: GLenum;\n    readonly UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER: GLenum;\n    readonly UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER: GLenum;\n    readonly UNIFORM_BUFFER: GLenum;\n    readonly UNIFORM_BUFFER_BINDING: GLenum;\n    readonly UNIFORM_BUFFER_OFFSET_ALIGNMENT: GLenum;\n    readonly UNIFORM_BUFFER_SIZE: GLenum;\n    readonly UNIFORM_BUFFER_START: GLenum;\n    readonly UNIFORM_IS_ROW_MAJOR: GLenum;\n    readonly UNIFORM_MATRIX_STRIDE: GLenum;\n    readonly UNIFORM_OFFSET: GLenum;\n    readonly UNIFORM_SIZE: GLenum;\n    readonly UNIFORM_TYPE: GLenum;\n    readonly UNPACK_IMAGE_HEIGHT: GLenum;\n    readonly UNPACK_ROW_LENGTH: GLenum;\n    readonly UNPACK_SKIP_IMAGES: GLenum;\n    readonly UNPACK_SKIP_PIXELS: GLenum;\n    readonly UNPACK_SKIP_ROWS: GLenum;\n    readonly UNSIGNALED: GLenum;\n    readonly UNSIGNED_INT_10F_11F_11F_REV: GLenum;\n    readonly UNSIGNED_INT_24_8: GLenum;\n    readonly UNSIGNED_INT_2_10_10_10_REV: GLenum;\n    readonly UNSIGNED_INT_5_9_9_9_REV: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_2D: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_2D_ARRAY: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_3D: GLenum;\n    readonly UNSIGNED_INT_SAMPLER_CUBE: GLenum;\n    readonly UNSIGNED_INT_VEC2: GLenum;\n    readonly UNSIGNED_INT_VEC3: GLenum;\n    readonly UNSIGNED_INT_VEC4: GLenum;\n    readonly UNSIGNED_NORMALIZED: GLenum;\n    readonly VERTEX_ARRAY_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_DIVISOR: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_INTEGER: GLenum;\n    readonly WAIT_FAILED: GLenum;\n}\n\ninterface WebGL2RenderingContextOverloads {\n    bufferData(target: GLenum, size: GLsizeiptr, usage: GLenum): void;\n    bufferData(target: GLenum, srcData: BufferSource | null, usage: GLenum): void;\n    bufferData(target: GLenum, srcData: ArrayBufferView, usage: GLenum, srcOffset: GLuint, length?: GLuint): void;\n    bufferSubData(target: GLenum, dstByteOffset: GLintptr, srcData: BufferSource): void;\n    bufferSubData(target: GLenum, dstByteOffset: GLintptr, srcData: ArrayBufferView, srcOffset: GLuint, length?: GLuint): void;\n    compressedTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, border: GLint, imageSize: GLsizei, offset: GLintptr): void;\n    compressedTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, border: GLint, srcData: ArrayBufferView, srcOffset?: GLuint, srcLengthOverride?: GLuint): void;\n    compressedTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, imageSize: GLsizei, offset: GLintptr): void;\n    compressedTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, srcData: ArrayBufferView, srcOffset?: GLuint, srcLengthOverride?: GLuint): void;\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, dstData: ArrayBufferView | null): void;\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, offset: GLintptr): void;\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, dstData: ArrayBufferView, dstOffset: GLuint): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, pboOffset: GLintptr): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, srcData: ArrayBufferView, srcOffset: GLuint): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pboOffset: GLintptr): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, srcData: ArrayBufferView, srcOffset: GLuint): void;\n    uniform1fv(location: WebGLUniformLocation | null, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform1iv(location: WebGLUniformLocation | null, data: Int32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2fv(location: WebGLUniformLocation | null, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2iv(location: WebGLUniformLocation | null, data: Int32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3fv(location: WebGLUniformLocation | null, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3iv(location: WebGLUniformLocation | null, data: Int32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4fv(location: WebGLUniformLocation | null, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4iv(location: WebGLUniformLocation | null, data: Int32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Float32List, srcOffset?: GLuint, srcLength?: GLuint): void;\n}\n\n/** Part of the WebGL API and represents the information returned by calling the WebGLRenderingContext.getActiveAttrib() and WebGLRenderingContext.getActiveUniform() methods. */\ninterface WebGLActiveInfo {\n    readonly name: string;\n    readonly size: GLint;\n    readonly type: GLenum;\n}\n\ndeclare var WebGLActiveInfo: {\n    prototype: WebGLActiveInfo;\n    new(): WebGLActiveInfo;\n};\n\n/** Part of the WebGL API and represents an opaque buffer object storing data such as vertices or colors. */\ninterface WebGLBuffer {\n}\n\ndeclare var WebGLBuffer: {\n    prototype: WebGLBuffer;\n    new(): WebGLBuffer;\n};\n\n/** The WebContextEvent interface is part of the WebGL API and is an interface for an event that is generated in response to a status change to the WebGL rendering context. */\ninterface WebGLContextEvent extends Event {\n    readonly statusMessage: string;\n}\n\ndeclare var WebGLContextEvent: {\n    prototype: WebGLContextEvent;\n    new(type: string, eventInit?: WebGLContextEventInit): WebGLContextEvent;\n};\n\n/** Part of the WebGL API and represents a collection of buffers that serve as a rendering destination. */\ninterface WebGLFramebuffer {\n}\n\ndeclare var WebGLFramebuffer: {\n    prototype: WebGLFramebuffer;\n    new(): WebGLFramebuffer;\n};\n\n/** The WebGLProgram is part of the WebGL API and is a combination of two compiled WebGLShaders consisting of a vertex shader and a fragment shader (both written in GLSL). */\ninterface WebGLProgram {\n}\n\ndeclare var WebGLProgram: {\n    prototype: WebGLProgram;\n    new(): WebGLProgram;\n};\n\ninterface WebGLQuery {\n}\n\ndeclare var WebGLQuery: {\n    prototype: WebGLQuery;\n    new(): WebGLQuery;\n};\n\n/** Part of the WebGL API and represents a buffer that can contain an image, or can be source or target of an rendering operation. */\ninterface WebGLRenderbuffer {\n}\n\ndeclare var WebGLRenderbuffer: {\n    prototype: WebGLRenderbuffer;\n    new(): WebGLRenderbuffer;\n};\n\n/** Provides an interface to the OpenGL ES 2.0 graphics rendering context for the drawing surface of an HTML <canvas> element. */\ninterface WebGLRenderingContext extends WebGLRenderingContextBase, WebGLRenderingContextOverloads {\n}\n\ndeclare var WebGLRenderingContext: {\n    prototype: WebGLRenderingContext;\n    new(): WebGLRenderingContext;\n    readonly ACTIVE_ATTRIBUTES: GLenum;\n    readonly ACTIVE_TEXTURE: GLenum;\n    readonly ACTIVE_UNIFORMS: GLenum;\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\n    readonly ALPHA: GLenum;\n    readonly ALPHA_BITS: GLenum;\n    readonly ALWAYS: GLenum;\n    readonly ARRAY_BUFFER: GLenum;\n    readonly ARRAY_BUFFER_BINDING: GLenum;\n    readonly ATTACHED_SHADERS: GLenum;\n    readonly BACK: GLenum;\n    readonly BLEND: GLenum;\n    readonly BLEND_COLOR: GLenum;\n    readonly BLEND_DST_ALPHA: GLenum;\n    readonly BLEND_DST_RGB: GLenum;\n    readonly BLEND_EQUATION: GLenum;\n    readonly BLEND_EQUATION_ALPHA: GLenum;\n    readonly BLEND_EQUATION_RGB: GLenum;\n    readonly BLEND_SRC_ALPHA: GLenum;\n    readonly BLEND_SRC_RGB: GLenum;\n    readonly BLUE_BITS: GLenum;\n    readonly BOOL: GLenum;\n    readonly BOOL_VEC2: GLenum;\n    readonly BOOL_VEC3: GLenum;\n    readonly BOOL_VEC4: GLenum;\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\n    readonly BUFFER_SIZE: GLenum;\n    readonly BUFFER_USAGE: GLenum;\n    readonly BYTE: GLenum;\n    readonly CCW: GLenum;\n    readonly CLAMP_TO_EDGE: GLenum;\n    readonly COLOR_ATTACHMENT0: GLenum;\n    readonly COLOR_BUFFER_BIT: GLenum;\n    readonly COLOR_CLEAR_VALUE: GLenum;\n    readonly COLOR_WRITEMASK: GLenum;\n    readonly COMPILE_STATUS: GLenum;\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\n    readonly CONSTANT_ALPHA: GLenum;\n    readonly CONSTANT_COLOR: GLenum;\n    readonly CONTEXT_LOST_WEBGL: GLenum;\n    readonly CULL_FACE: GLenum;\n    readonly CULL_FACE_MODE: GLenum;\n    readonly CURRENT_PROGRAM: GLenum;\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\n    readonly CW: GLenum;\n    readonly DECR: GLenum;\n    readonly DECR_WRAP: GLenum;\n    readonly DELETE_STATUS: GLenum;\n    readonly DEPTH_ATTACHMENT: GLenum;\n    readonly DEPTH_BITS: GLenum;\n    readonly DEPTH_BUFFER_BIT: GLenum;\n    readonly DEPTH_CLEAR_VALUE: GLenum;\n    readonly DEPTH_COMPONENT: GLenum;\n    readonly DEPTH_COMPONENT16: GLenum;\n    readonly DEPTH_FUNC: GLenum;\n    readonly DEPTH_RANGE: GLenum;\n    readonly DEPTH_STENCIL: GLenum;\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\n    readonly DEPTH_TEST: GLenum;\n    readonly DEPTH_WRITEMASK: GLenum;\n    readonly DITHER: GLenum;\n    readonly DONT_CARE: GLenum;\n    readonly DST_ALPHA: GLenum;\n    readonly DST_COLOR: GLenum;\n    readonly DYNAMIC_DRAW: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\n    readonly EQUAL: GLenum;\n    readonly FASTEST: GLenum;\n    readonly FLOAT: GLenum;\n    readonly FLOAT_MAT2: GLenum;\n    readonly FLOAT_MAT3: GLenum;\n    readonly FLOAT_MAT4: GLenum;\n    readonly FLOAT_VEC2: GLenum;\n    readonly FLOAT_VEC3: GLenum;\n    readonly FLOAT_VEC4: GLenum;\n    readonly FRAGMENT_SHADER: GLenum;\n    readonly FRAMEBUFFER: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\n    readonly FRAMEBUFFER_BINDING: GLenum;\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\n    readonly FRONT: GLenum;\n    readonly FRONT_AND_BACK: GLenum;\n    readonly FRONT_FACE: GLenum;\n    readonly FUNC_ADD: GLenum;\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\n    readonly FUNC_SUBTRACT: GLenum;\n    readonly GENERATE_MIPMAP_HINT: GLenum;\n    readonly GEQUAL: GLenum;\n    readonly GREATER: GLenum;\n    readonly GREEN_BITS: GLenum;\n    readonly HIGH_FLOAT: GLenum;\n    readonly HIGH_INT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\n    readonly INCR: GLenum;\n    readonly INCR_WRAP: GLenum;\n    readonly INT: GLenum;\n    readonly INT_VEC2: GLenum;\n    readonly INT_VEC3: GLenum;\n    readonly INT_VEC4: GLenum;\n    readonly INVALID_ENUM: GLenum;\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\n    readonly INVALID_OPERATION: GLenum;\n    readonly INVALID_VALUE: GLenum;\n    readonly INVERT: GLenum;\n    readonly KEEP: GLenum;\n    readonly LEQUAL: GLenum;\n    readonly LESS: GLenum;\n    readonly LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\n    readonly LINES: GLenum;\n    readonly LINE_LOOP: GLenum;\n    readonly LINE_STRIP: GLenum;\n    readonly LINE_WIDTH: GLenum;\n    readonly LINK_STATUS: GLenum;\n    readonly LOW_FLOAT: GLenum;\n    readonly LOW_INT: GLenum;\n    readonly LUMINANCE: GLenum;\n    readonly LUMINANCE_ALPHA: GLenum;\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_TEXTURE_SIZE: GLenum;\n    readonly MAX_VARYING_VECTORS: GLenum;\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\n    readonly MAX_VIEWPORT_DIMS: GLenum;\n    readonly MEDIUM_FLOAT: GLenum;\n    readonly MEDIUM_INT: GLenum;\n    readonly MIRRORED_REPEAT: GLenum;\n    readonly NEAREST: GLenum;\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\n    readonly NEVER: GLenum;\n    readonly NICEST: GLenum;\n    readonly NONE: GLenum;\n    readonly NOTEQUAL: GLenum;\n    readonly NO_ERROR: GLenum;\n    readonly ONE: GLenum;\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\n    readonly ONE_MINUS_DST_COLOR: GLenum;\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\n    readonly OUT_OF_MEMORY: GLenum;\n    readonly PACK_ALIGNMENT: GLenum;\n    readonly POINTS: GLenum;\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\n    readonly POLYGON_OFFSET_FILL: GLenum;\n    readonly POLYGON_OFFSET_UNITS: GLenum;\n    readonly RED_BITS: GLenum;\n    readonly RENDERBUFFER: GLenum;\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\n    readonly RENDERBUFFER_BINDING: GLenum;\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\n    readonly RENDERBUFFER_HEIGHT: GLenum;\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\n    readonly RENDERBUFFER_WIDTH: GLenum;\n    readonly RENDERER: GLenum;\n    readonly REPEAT: GLenum;\n    readonly REPLACE: GLenum;\n    readonly RGB: GLenum;\n    readonly RGB565: GLenum;\n    readonly RGB5_A1: GLenum;\n    readonly RGBA: GLenum;\n    readonly RGBA4: GLenum;\n    readonly SAMPLER_2D: GLenum;\n    readonly SAMPLER_CUBE: GLenum;\n    readonly SAMPLES: GLenum;\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\n    readonly SAMPLE_BUFFERS: GLenum;\n    readonly SAMPLE_COVERAGE: GLenum;\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\n    readonly SCISSOR_BOX: GLenum;\n    readonly SCISSOR_TEST: GLenum;\n    readonly SHADER_TYPE: GLenum;\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\n    readonly SHORT: GLenum;\n    readonly SRC_ALPHA: GLenum;\n    readonly SRC_ALPHA_SATURATE: GLenum;\n    readonly SRC_COLOR: GLenum;\n    readonly STATIC_DRAW: GLenum;\n    readonly STENCIL_ATTACHMENT: GLenum;\n    readonly STENCIL_BACK_FAIL: GLenum;\n    readonly STENCIL_BACK_FUNC: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_BACK_REF: GLenum;\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\n    readonly STENCIL_BITS: GLenum;\n    readonly STENCIL_BUFFER_BIT: GLenum;\n    readonly STENCIL_CLEAR_VALUE: GLenum;\n    readonly STENCIL_FAIL: GLenum;\n    readonly STENCIL_FUNC: GLenum;\n    readonly STENCIL_INDEX8: GLenum;\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_REF: GLenum;\n    readonly STENCIL_TEST: GLenum;\n    readonly STENCIL_VALUE_MASK: GLenum;\n    readonly STENCIL_WRITEMASK: GLenum;\n    readonly STREAM_DRAW: GLenum;\n    readonly SUBPIXEL_BITS: GLenum;\n    readonly TEXTURE: GLenum;\n    readonly TEXTURE0: GLenum;\n    readonly TEXTURE1: GLenum;\n    readonly TEXTURE10: GLenum;\n    readonly TEXTURE11: GLenum;\n    readonly TEXTURE12: GLenum;\n    readonly TEXTURE13: GLenum;\n    readonly TEXTURE14: GLenum;\n    readonly TEXTURE15: GLenum;\n    readonly TEXTURE16: GLenum;\n    readonly TEXTURE17: GLenum;\n    readonly TEXTURE18: GLenum;\n    readonly TEXTURE19: GLenum;\n    readonly TEXTURE2: GLenum;\n    readonly TEXTURE20: GLenum;\n    readonly TEXTURE21: GLenum;\n    readonly TEXTURE22: GLenum;\n    readonly TEXTURE23: GLenum;\n    readonly TEXTURE24: GLenum;\n    readonly TEXTURE25: GLenum;\n    readonly TEXTURE26: GLenum;\n    readonly TEXTURE27: GLenum;\n    readonly TEXTURE28: GLenum;\n    readonly TEXTURE29: GLenum;\n    readonly TEXTURE3: GLenum;\n    readonly TEXTURE30: GLenum;\n    readonly TEXTURE31: GLenum;\n    readonly TEXTURE4: GLenum;\n    readonly TEXTURE5: GLenum;\n    readonly TEXTURE6: GLenum;\n    readonly TEXTURE7: GLenum;\n    readonly TEXTURE8: GLenum;\n    readonly TEXTURE9: GLenum;\n    readonly TEXTURE_2D: GLenum;\n    readonly TEXTURE_BINDING_2D: GLenum;\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\n    readonly TEXTURE_MAG_FILTER: GLenum;\n    readonly TEXTURE_MIN_FILTER: GLenum;\n    readonly TEXTURE_WRAP_S: GLenum;\n    readonly TEXTURE_WRAP_T: GLenum;\n    readonly TRIANGLES: GLenum;\n    readonly TRIANGLE_FAN: GLenum;\n    readonly TRIANGLE_STRIP: GLenum;\n    readonly UNPACK_ALIGNMENT: GLenum;\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\n    readonly UNSIGNED_BYTE: GLenum;\n    readonly UNSIGNED_INT: GLenum;\n    readonly UNSIGNED_SHORT: GLenum;\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\n    readonly VALIDATE_STATUS: GLenum;\n    readonly VENDOR: GLenum;\n    readonly VERSION: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\n    readonly VERTEX_SHADER: GLenum;\n    readonly VIEWPORT: GLenum;\n    readonly ZERO: GLenum;\n};\n\ninterface WebGLRenderingContextBase {\n    readonly drawingBufferHeight: GLsizei;\n    readonly drawingBufferWidth: GLsizei;\n    activeTexture(texture: GLenum): void;\n    attachShader(program: WebGLProgram, shader: WebGLShader): void;\n    bindAttribLocation(program: WebGLProgram, index: GLuint, name: string): void;\n    bindBuffer(target: GLenum, buffer: WebGLBuffer | null): void;\n    bindFramebuffer(target: GLenum, framebuffer: WebGLFramebuffer | null): void;\n    bindRenderbuffer(target: GLenum, renderbuffer: WebGLRenderbuffer | null): void;\n    bindTexture(target: GLenum, texture: WebGLTexture | null): void;\n    blendColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;\n    blendEquation(mode: GLenum): void;\n    blendEquationSeparate(modeRGB: GLenum, modeAlpha: GLenum): void;\n    blendFunc(sfactor: GLenum, dfactor: GLenum): void;\n    blendFuncSeparate(srcRGB: GLenum, dstRGB: GLenum, srcAlpha: GLenum, dstAlpha: GLenum): void;\n    checkFramebufferStatus(target: GLenum): GLenum;\n    clear(mask: GLbitfield): void;\n    clearColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;\n    clearDepth(depth: GLclampf): void;\n    clearStencil(s: GLint): void;\n    colorMask(red: GLboolean, green: GLboolean, blue: GLboolean, alpha: GLboolean): void;\n    compileShader(shader: WebGLShader): void;\n    copyTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, x: GLint, y: GLint, width: GLsizei, height: GLsizei, border: GLint): void;\n    copyTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    createBuffer(): WebGLBuffer | null;\n    createFramebuffer(): WebGLFramebuffer | null;\n    createProgram(): WebGLProgram | null;\n    createRenderbuffer(): WebGLRenderbuffer | null;\n    createShader(type: GLenum): WebGLShader | null;\n    createTexture(): WebGLTexture | null;\n    cullFace(mode: GLenum): void;\n    deleteBuffer(buffer: WebGLBuffer | null): void;\n    deleteFramebuffer(framebuffer: WebGLFramebuffer | null): void;\n    deleteProgram(program: WebGLProgram | null): void;\n    deleteRenderbuffer(renderbuffer: WebGLRenderbuffer | null): void;\n    deleteShader(shader: WebGLShader | null): void;\n    deleteTexture(texture: WebGLTexture | null): void;\n    depthFunc(func: GLenum): void;\n    depthMask(flag: GLboolean): void;\n    depthRange(zNear: GLclampf, zFar: GLclampf): void;\n    detachShader(program: WebGLProgram, shader: WebGLShader): void;\n    disable(cap: GLenum): void;\n    disableVertexAttribArray(index: GLuint): void;\n    drawArrays(mode: GLenum, first: GLint, count: GLsizei): void;\n    drawElements(mode: GLenum, count: GLsizei, type: GLenum, offset: GLintptr): void;\n    enable(cap: GLenum): void;\n    enableVertexAttribArray(index: GLuint): void;\n    finish(): void;\n    flush(): void;\n    framebufferRenderbuffer(target: GLenum, attachment: GLenum, renderbuffertarget: GLenum, renderbuffer: WebGLRenderbuffer | null): void;\n    framebufferTexture2D(target: GLenum, attachment: GLenum, textarget: GLenum, texture: WebGLTexture | null, level: GLint): void;\n    frontFace(mode: GLenum): void;\n    generateMipmap(target: GLenum): void;\n    getActiveAttrib(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\n    getActiveUniform(program: WebGLProgram, index: GLuint): WebGLActiveInfo | null;\n    getAttachedShaders(program: WebGLProgram): WebGLShader[] | null;\n    getAttribLocation(program: WebGLProgram, name: string): GLint;\n    getBufferParameter(target: GLenum, pname: GLenum): any;\n    getContextAttributes(): WebGLContextAttributes | null;\n    getError(): GLenum;\n    getExtension(extensionName: "EXT_blend_minmax"): EXT_blend_minmax | null;\n    getExtension(extensionName: "EXT_color_buffer_float"): EXT_color_buffer_float | null;\n    getExtension(extensionName: "EXT_color_buffer_half_float"): EXT_color_buffer_half_float | null;\n    getExtension(extensionName: "EXT_float_blend"): EXT_float_blend | null;\n    getExtension(extensionName: "EXT_texture_filter_anisotropic"): EXT_texture_filter_anisotropic | null;\n    getExtension(extensionName: "EXT_frag_depth"): EXT_frag_depth | null;\n    getExtension(extensionName: "EXT_shader_texture_lod"): EXT_shader_texture_lod | null;\n    getExtension(extensionName: "EXT_sRGB"): EXT_sRGB | null;\n    getExtension(extensionName: "KHR_parallel_shader_compile"): KHR_parallel_shader_compile | null;\n    getExtension(extensionName: "OES_vertex_array_object"): OES_vertex_array_object | null;\n    getExtension(extensionName: "OVR_multiview2"): OVR_multiview2 | null;\n    getExtension(extensionName: "WEBGL_color_buffer_float"): WEBGL_color_buffer_float | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_astc"): WEBGL_compressed_texture_astc | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_etc"): WEBGL_compressed_texture_etc | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_etc1"): WEBGL_compressed_texture_etc1 | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_pvrtc"): WEBGL_compressed_texture_pvrtc | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_s3tc_srgb"): WEBGL_compressed_texture_s3tc_srgb | null;\n    getExtension(extensionName: "WEBGL_debug_shaders"): WEBGL_debug_shaders | null;\n    getExtension(extensionName: "WEBGL_draw_buffers"): WEBGL_draw_buffers | null;\n    getExtension(extensionName: "WEBGL_lose_context"): WEBGL_lose_context | null;\n    getExtension(extensionName: "WEBGL_depth_texture"): WEBGL_depth_texture | null;\n    getExtension(extensionName: "WEBGL_debug_renderer_info"): WEBGL_debug_renderer_info | null;\n    getExtension(extensionName: "WEBGL_compressed_texture_s3tc"): WEBGL_compressed_texture_s3tc | null;\n    getExtension(extensionName: "OES_texture_half_float_linear"): OES_texture_half_float_linear | null;\n    getExtension(extensionName: "OES_texture_half_float"): OES_texture_half_float | null;\n    getExtension(extensionName: "OES_texture_float_linear"): OES_texture_float_linear | null;\n    getExtension(extensionName: "OES_texture_float"): OES_texture_float | null;\n    getExtension(extensionName: "OES_standard_derivatives"): OES_standard_derivatives | null;\n    getExtension(extensionName: "OES_element_index_uint"): OES_element_index_uint | null;\n    getExtension(extensionName: "ANGLE_instanced_arrays"): ANGLE_instanced_arrays | null;\n    getExtension(name: string): any;\n    getFramebufferAttachmentParameter(target: GLenum, attachment: GLenum, pname: GLenum): any;\n    getParameter(pname: GLenum): any;\n    getProgramInfoLog(program: WebGLProgram): string | null;\n    getProgramParameter(program: WebGLProgram, pname: GLenum): any;\n    getRenderbufferParameter(target: GLenum, pname: GLenum): any;\n    getShaderInfoLog(shader: WebGLShader): string | null;\n    getShaderParameter(shader: WebGLShader, pname: GLenum): any;\n    getShaderPrecisionFormat(shadertype: GLenum, precisiontype: GLenum): WebGLShaderPrecisionFormat | null;\n    getShaderSource(shader: WebGLShader): string | null;\n    getSupportedExtensions(): string[] | null;\n    getTexParameter(target: GLenum, pname: GLenum): any;\n    getUniform(program: WebGLProgram, location: WebGLUniformLocation): any;\n    getUniformLocation(program: WebGLProgram, name: string): WebGLUniformLocation | null;\n    getVertexAttrib(index: GLuint, pname: GLenum): any;\n    getVertexAttribOffset(index: GLuint, pname: GLenum): GLintptr;\n    hint(target: GLenum, mode: GLenum): void;\n    isBuffer(buffer: WebGLBuffer | null): GLboolean;\n    isContextLost(): boolean;\n    isEnabled(cap: GLenum): GLboolean;\n    isFramebuffer(framebuffer: WebGLFramebuffer | null): GLboolean;\n    isProgram(program: WebGLProgram | null): GLboolean;\n    isRenderbuffer(renderbuffer: WebGLRenderbuffer | null): GLboolean;\n    isShader(shader: WebGLShader | null): GLboolean;\n    isTexture(texture: WebGLTexture | null): GLboolean;\n    lineWidth(width: GLfloat): void;\n    linkProgram(program: WebGLProgram): void;\n    pixelStorei(pname: GLenum, param: GLint | GLboolean): void;\n    polygonOffset(factor: GLfloat, units: GLfloat): void;\n    renderbufferStorage(target: GLenum, internalformat: GLenum, width: GLsizei, height: GLsizei): void;\n    sampleCoverage(value: GLclampf, invert: GLboolean): void;\n    scissor(x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    shaderSource(shader: WebGLShader, source: string): void;\n    stencilFunc(func: GLenum, ref: GLint, mask: GLuint): void;\n    stencilFuncSeparate(face: GLenum, func: GLenum, ref: GLint, mask: GLuint): void;\n    stencilMask(mask: GLuint): void;\n    stencilMaskSeparate(face: GLenum, mask: GLuint): void;\n    stencilOp(fail: GLenum, zfail: GLenum, zpass: GLenum): void;\n    stencilOpSeparate(face: GLenum, fail: GLenum, zfail: GLenum, zpass: GLenum): void;\n    texParameterf(target: GLenum, pname: GLenum, param: GLfloat): void;\n    texParameteri(target: GLenum, pname: GLenum, param: GLint): void;\n    uniform1f(location: WebGLUniformLocation | null, x: GLfloat): void;\n    uniform1i(location: WebGLUniformLocation | null, x: GLint): void;\n    uniform2f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat): void;\n    uniform2i(location: WebGLUniformLocation | null, x: GLint, y: GLint): void;\n    uniform3f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat, z: GLfloat): void;\n    uniform3i(location: WebGLUniformLocation | null, x: GLint, y: GLint, z: GLint): void;\n    uniform4f(location: WebGLUniformLocation | null, x: GLfloat, y: GLfloat, z: GLfloat, w: GLfloat): void;\n    uniform4i(location: WebGLUniformLocation | null, x: GLint, y: GLint, z: GLint, w: GLint): void;\n    useProgram(program: WebGLProgram | null): void;\n    validateProgram(program: WebGLProgram): void;\n    vertexAttrib1f(index: GLuint, x: GLfloat): void;\n    vertexAttrib1fv(index: GLuint, values: Float32List): void;\n    vertexAttrib2f(index: GLuint, x: GLfloat, y: GLfloat): void;\n    vertexAttrib2fv(index: GLuint, values: Float32List): void;\n    vertexAttrib3f(index: GLuint, x: GLfloat, y: GLfloat, z: GLfloat): void;\n    vertexAttrib3fv(index: GLuint, values: Float32List): void;\n    vertexAttrib4f(index: GLuint, x: GLfloat, y: GLfloat, z: GLfloat, w: GLfloat): void;\n    vertexAttrib4fv(index: GLuint, values: Float32List): void;\n    vertexAttribPointer(index: GLuint, size: GLint, type: GLenum, normalized: GLboolean, stride: GLsizei, offset: GLintptr): void;\n    viewport(x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    readonly ACTIVE_ATTRIBUTES: GLenum;\n    readonly ACTIVE_TEXTURE: GLenum;\n    readonly ACTIVE_UNIFORMS: GLenum;\n    readonly ALIASED_LINE_WIDTH_RANGE: GLenum;\n    readonly ALIASED_POINT_SIZE_RANGE: GLenum;\n    readonly ALPHA: GLenum;\n    readonly ALPHA_BITS: GLenum;\n    readonly ALWAYS: GLenum;\n    readonly ARRAY_BUFFER: GLenum;\n    readonly ARRAY_BUFFER_BINDING: GLenum;\n    readonly ATTACHED_SHADERS: GLenum;\n    readonly BACK: GLenum;\n    readonly BLEND: GLenum;\n    readonly BLEND_COLOR: GLenum;\n    readonly BLEND_DST_ALPHA: GLenum;\n    readonly BLEND_DST_RGB: GLenum;\n    readonly BLEND_EQUATION: GLenum;\n    readonly BLEND_EQUATION_ALPHA: GLenum;\n    readonly BLEND_EQUATION_RGB: GLenum;\n    readonly BLEND_SRC_ALPHA: GLenum;\n    readonly BLEND_SRC_RGB: GLenum;\n    readonly BLUE_BITS: GLenum;\n    readonly BOOL: GLenum;\n    readonly BOOL_VEC2: GLenum;\n    readonly BOOL_VEC3: GLenum;\n    readonly BOOL_VEC4: GLenum;\n    readonly BROWSER_DEFAULT_WEBGL: GLenum;\n    readonly BUFFER_SIZE: GLenum;\n    readonly BUFFER_USAGE: GLenum;\n    readonly BYTE: GLenum;\n    readonly CCW: GLenum;\n    readonly CLAMP_TO_EDGE: GLenum;\n    readonly COLOR_ATTACHMENT0: GLenum;\n    readonly COLOR_BUFFER_BIT: GLenum;\n    readonly COLOR_CLEAR_VALUE: GLenum;\n    readonly COLOR_WRITEMASK: GLenum;\n    readonly COMPILE_STATUS: GLenum;\n    readonly COMPRESSED_TEXTURE_FORMATS: GLenum;\n    readonly CONSTANT_ALPHA: GLenum;\n    readonly CONSTANT_COLOR: GLenum;\n    readonly CONTEXT_LOST_WEBGL: GLenum;\n    readonly CULL_FACE: GLenum;\n    readonly CULL_FACE_MODE: GLenum;\n    readonly CURRENT_PROGRAM: GLenum;\n    readonly CURRENT_VERTEX_ATTRIB: GLenum;\n    readonly CW: GLenum;\n    readonly DECR: GLenum;\n    readonly DECR_WRAP: GLenum;\n    readonly DELETE_STATUS: GLenum;\n    readonly DEPTH_ATTACHMENT: GLenum;\n    readonly DEPTH_BITS: GLenum;\n    readonly DEPTH_BUFFER_BIT: GLenum;\n    readonly DEPTH_CLEAR_VALUE: GLenum;\n    readonly DEPTH_COMPONENT: GLenum;\n    readonly DEPTH_COMPONENT16: GLenum;\n    readonly DEPTH_FUNC: GLenum;\n    readonly DEPTH_RANGE: GLenum;\n    readonly DEPTH_STENCIL: GLenum;\n    readonly DEPTH_STENCIL_ATTACHMENT: GLenum;\n    readonly DEPTH_TEST: GLenum;\n    readonly DEPTH_WRITEMASK: GLenum;\n    readonly DITHER: GLenum;\n    readonly DONT_CARE: GLenum;\n    readonly DST_ALPHA: GLenum;\n    readonly DST_COLOR: GLenum;\n    readonly DYNAMIC_DRAW: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER: GLenum;\n    readonly ELEMENT_ARRAY_BUFFER_BINDING: GLenum;\n    readonly EQUAL: GLenum;\n    readonly FASTEST: GLenum;\n    readonly FLOAT: GLenum;\n    readonly FLOAT_MAT2: GLenum;\n    readonly FLOAT_MAT3: GLenum;\n    readonly FLOAT_MAT4: GLenum;\n    readonly FLOAT_VEC2: GLenum;\n    readonly FLOAT_VEC3: GLenum;\n    readonly FLOAT_VEC4: GLenum;\n    readonly FRAGMENT_SHADER: GLenum;\n    readonly FRAMEBUFFER: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: GLenum;\n    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: GLenum;\n    readonly FRAMEBUFFER_BINDING: GLenum;\n    readonly FRAMEBUFFER_COMPLETE: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_DIMENSIONS: GLenum;\n    readonly FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: GLenum;\n    readonly FRAMEBUFFER_UNSUPPORTED: GLenum;\n    readonly FRONT: GLenum;\n    readonly FRONT_AND_BACK: GLenum;\n    readonly FRONT_FACE: GLenum;\n    readonly FUNC_ADD: GLenum;\n    readonly FUNC_REVERSE_SUBTRACT: GLenum;\n    readonly FUNC_SUBTRACT: GLenum;\n    readonly GENERATE_MIPMAP_HINT: GLenum;\n    readonly GEQUAL: GLenum;\n    readonly GREATER: GLenum;\n    readonly GREEN_BITS: GLenum;\n    readonly HIGH_FLOAT: GLenum;\n    readonly HIGH_INT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_FORMAT: GLenum;\n    readonly IMPLEMENTATION_COLOR_READ_TYPE: GLenum;\n    readonly INCR: GLenum;\n    readonly INCR_WRAP: GLenum;\n    readonly INT: GLenum;\n    readonly INT_VEC2: GLenum;\n    readonly INT_VEC3: GLenum;\n    readonly INT_VEC4: GLenum;\n    readonly INVALID_ENUM: GLenum;\n    readonly INVALID_FRAMEBUFFER_OPERATION: GLenum;\n    readonly INVALID_OPERATION: GLenum;\n    readonly INVALID_VALUE: GLenum;\n    readonly INVERT: GLenum;\n    readonly KEEP: GLenum;\n    readonly LEQUAL: GLenum;\n    readonly LESS: GLenum;\n    readonly LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_LINEAR: GLenum;\n    readonly LINEAR_MIPMAP_NEAREST: GLenum;\n    readonly LINES: GLenum;\n    readonly LINE_LOOP: GLenum;\n    readonly LINE_STRIP: GLenum;\n    readonly LINE_WIDTH: GLenum;\n    readonly LINK_STATUS: GLenum;\n    readonly LOW_FLOAT: GLenum;\n    readonly LOW_INT: GLenum;\n    readonly LUMINANCE: GLenum;\n    readonly LUMINANCE_ALPHA: GLenum;\n    readonly MAX_COMBINED_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_CUBE_MAP_TEXTURE_SIZE: GLenum;\n    readonly MAX_FRAGMENT_UNIFORM_VECTORS: GLenum;\n    readonly MAX_RENDERBUFFER_SIZE: GLenum;\n    readonly MAX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_TEXTURE_SIZE: GLenum;\n    readonly MAX_VARYING_VECTORS: GLenum;\n    readonly MAX_VERTEX_ATTRIBS: GLenum;\n    readonly MAX_VERTEX_TEXTURE_IMAGE_UNITS: GLenum;\n    readonly MAX_VERTEX_UNIFORM_VECTORS: GLenum;\n    readonly MAX_VIEWPORT_DIMS: GLenum;\n    readonly MEDIUM_FLOAT: GLenum;\n    readonly MEDIUM_INT: GLenum;\n    readonly MIRRORED_REPEAT: GLenum;\n    readonly NEAREST: GLenum;\n    readonly NEAREST_MIPMAP_LINEAR: GLenum;\n    readonly NEAREST_MIPMAP_NEAREST: GLenum;\n    readonly NEVER: GLenum;\n    readonly NICEST: GLenum;\n    readonly NONE: GLenum;\n    readonly NOTEQUAL: GLenum;\n    readonly NO_ERROR: GLenum;\n    readonly ONE: GLenum;\n    readonly ONE_MINUS_CONSTANT_ALPHA: GLenum;\n    readonly ONE_MINUS_CONSTANT_COLOR: GLenum;\n    readonly ONE_MINUS_DST_ALPHA: GLenum;\n    readonly ONE_MINUS_DST_COLOR: GLenum;\n    readonly ONE_MINUS_SRC_ALPHA: GLenum;\n    readonly ONE_MINUS_SRC_COLOR: GLenum;\n    readonly OUT_OF_MEMORY: GLenum;\n    readonly PACK_ALIGNMENT: GLenum;\n    readonly POINTS: GLenum;\n    readonly POLYGON_OFFSET_FACTOR: GLenum;\n    readonly POLYGON_OFFSET_FILL: GLenum;\n    readonly POLYGON_OFFSET_UNITS: GLenum;\n    readonly RED_BITS: GLenum;\n    readonly RENDERBUFFER: GLenum;\n    readonly RENDERBUFFER_ALPHA_SIZE: GLenum;\n    readonly RENDERBUFFER_BINDING: GLenum;\n    readonly RENDERBUFFER_BLUE_SIZE: GLenum;\n    readonly RENDERBUFFER_DEPTH_SIZE: GLenum;\n    readonly RENDERBUFFER_GREEN_SIZE: GLenum;\n    readonly RENDERBUFFER_HEIGHT: GLenum;\n    readonly RENDERBUFFER_INTERNAL_FORMAT: GLenum;\n    readonly RENDERBUFFER_RED_SIZE: GLenum;\n    readonly RENDERBUFFER_STENCIL_SIZE: GLenum;\n    readonly RENDERBUFFER_WIDTH: GLenum;\n    readonly RENDERER: GLenum;\n    readonly REPEAT: GLenum;\n    readonly REPLACE: GLenum;\n    readonly RGB: GLenum;\n    readonly RGB565: GLenum;\n    readonly RGB5_A1: GLenum;\n    readonly RGBA: GLenum;\n    readonly RGBA4: GLenum;\n    readonly SAMPLER_2D: GLenum;\n    readonly SAMPLER_CUBE: GLenum;\n    readonly SAMPLES: GLenum;\n    readonly SAMPLE_ALPHA_TO_COVERAGE: GLenum;\n    readonly SAMPLE_BUFFERS: GLenum;\n    readonly SAMPLE_COVERAGE: GLenum;\n    readonly SAMPLE_COVERAGE_INVERT: GLenum;\n    readonly SAMPLE_COVERAGE_VALUE: GLenum;\n    readonly SCISSOR_BOX: GLenum;\n    readonly SCISSOR_TEST: GLenum;\n    readonly SHADER_TYPE: GLenum;\n    readonly SHADING_LANGUAGE_VERSION: GLenum;\n    readonly SHORT: GLenum;\n    readonly SRC_ALPHA: GLenum;\n    readonly SRC_ALPHA_SATURATE: GLenum;\n    readonly SRC_COLOR: GLenum;\n    readonly STATIC_DRAW: GLenum;\n    readonly STENCIL_ATTACHMENT: GLenum;\n    readonly STENCIL_BACK_FAIL: GLenum;\n    readonly STENCIL_BACK_FUNC: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_BACK_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_BACK_REF: GLenum;\n    readonly STENCIL_BACK_VALUE_MASK: GLenum;\n    readonly STENCIL_BACK_WRITEMASK: GLenum;\n    readonly STENCIL_BITS: GLenum;\n    readonly STENCIL_BUFFER_BIT: GLenum;\n    readonly STENCIL_CLEAR_VALUE: GLenum;\n    readonly STENCIL_FAIL: GLenum;\n    readonly STENCIL_FUNC: GLenum;\n    readonly STENCIL_INDEX8: GLenum;\n    readonly STENCIL_PASS_DEPTH_FAIL: GLenum;\n    readonly STENCIL_PASS_DEPTH_PASS: GLenum;\n    readonly STENCIL_REF: GLenum;\n    readonly STENCIL_TEST: GLenum;\n    readonly STENCIL_VALUE_MASK: GLenum;\n    readonly STENCIL_WRITEMASK: GLenum;\n    readonly STREAM_DRAW: GLenum;\n    readonly SUBPIXEL_BITS: GLenum;\n    readonly TEXTURE: GLenum;\n    readonly TEXTURE0: GLenum;\n    readonly TEXTURE1: GLenum;\n    readonly TEXTURE10: GLenum;\n    readonly TEXTURE11: GLenum;\n    readonly TEXTURE12: GLenum;\n    readonly TEXTURE13: GLenum;\n    readonly TEXTURE14: GLenum;\n    readonly TEXTURE15: GLenum;\n    readonly TEXTURE16: GLenum;\n    readonly TEXTURE17: GLenum;\n    readonly TEXTURE18: GLenum;\n    readonly TEXTURE19: GLenum;\n    readonly TEXTURE2: GLenum;\n    readonly TEXTURE20: GLenum;\n    readonly TEXTURE21: GLenum;\n    readonly TEXTURE22: GLenum;\n    readonly TEXTURE23: GLenum;\n    readonly TEXTURE24: GLenum;\n    readonly TEXTURE25: GLenum;\n    readonly TEXTURE26: GLenum;\n    readonly TEXTURE27: GLenum;\n    readonly TEXTURE28: GLenum;\n    readonly TEXTURE29: GLenum;\n    readonly TEXTURE3: GLenum;\n    readonly TEXTURE30: GLenum;\n    readonly TEXTURE31: GLenum;\n    readonly TEXTURE4: GLenum;\n    readonly TEXTURE5: GLenum;\n    readonly TEXTURE6: GLenum;\n    readonly TEXTURE7: GLenum;\n    readonly TEXTURE8: GLenum;\n    readonly TEXTURE9: GLenum;\n    readonly TEXTURE_2D: GLenum;\n    readonly TEXTURE_BINDING_2D: GLenum;\n    readonly TEXTURE_BINDING_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_NEGATIVE_Z: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_X: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Y: GLenum;\n    readonly TEXTURE_CUBE_MAP_POSITIVE_Z: GLenum;\n    readonly TEXTURE_MAG_FILTER: GLenum;\n    readonly TEXTURE_MIN_FILTER: GLenum;\n    readonly TEXTURE_WRAP_S: GLenum;\n    readonly TEXTURE_WRAP_T: GLenum;\n    readonly TRIANGLES: GLenum;\n    readonly TRIANGLE_FAN: GLenum;\n    readonly TRIANGLE_STRIP: GLenum;\n    readonly UNPACK_ALIGNMENT: GLenum;\n    readonly UNPACK_COLORSPACE_CONVERSION_WEBGL: GLenum;\n    readonly UNPACK_FLIP_Y_WEBGL: GLenum;\n    readonly UNPACK_PREMULTIPLY_ALPHA_WEBGL: GLenum;\n    readonly UNSIGNED_BYTE: GLenum;\n    readonly UNSIGNED_INT: GLenum;\n    readonly UNSIGNED_SHORT: GLenum;\n    readonly UNSIGNED_SHORT_4_4_4_4: GLenum;\n    readonly UNSIGNED_SHORT_5_5_5_1: GLenum;\n    readonly UNSIGNED_SHORT_5_6_5: GLenum;\n    readonly VALIDATE_STATUS: GLenum;\n    readonly VENDOR: GLenum;\n    readonly VERSION: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_ENABLED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_NORMALIZED: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_POINTER: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_SIZE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_STRIDE: GLenum;\n    readonly VERTEX_ATTRIB_ARRAY_TYPE: GLenum;\n    readonly VERTEX_SHADER: GLenum;\n    readonly VIEWPORT: GLenum;\n    readonly ZERO: GLenum;\n}\n\ninterface WebGLRenderingContextOverloads {\n    bufferData(target: GLenum, size: GLsizeiptr, usage: GLenum): void;\n    bufferData(target: GLenum, data: BufferSource | null, usage: GLenum): void;\n    bufferSubData(target: GLenum, offset: GLintptr, data: BufferSource): void;\n    compressedTexImage2D(target: GLenum, level: GLint, internalformat: GLenum, width: GLsizei, height: GLsizei, border: GLint, data: ArrayBufferView): void;\n    compressedTexSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, data: ArrayBufferView): void;\n    readPixels(x: GLint, y: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, width: GLsizei, height: GLsizei, border: GLint, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texImage2D(target: GLenum, level: GLint, internalformat: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, width: GLsizei, height: GLsizei, format: GLenum, type: GLenum, pixels: ArrayBufferView | null): void;\n    texSubImage2D(target: GLenum, level: GLint, xoffset: GLint, yoffset: GLint, format: GLenum, type: GLenum, source: TexImageSource): void;\n    uniform1fv(location: WebGLUniformLocation | null, v: Float32List): void;\n    uniform1iv(location: WebGLUniformLocation | null, v: Int32List): void;\n    uniform2fv(location: WebGLUniformLocation | null, v: Float32List): void;\n    uniform2iv(location: WebGLUniformLocation | null, v: Int32List): void;\n    uniform3fv(location: WebGLUniformLocation | null, v: Float32List): void;\n    uniform3iv(location: WebGLUniformLocation | null, v: Int32List): void;\n    uniform4fv(location: WebGLUniformLocation | null, v: Float32List): void;\n    uniform4iv(location: WebGLUniformLocation | null, v: Int32List): void;\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Float32List): void;\n}\n\ninterface WebGLSampler {\n}\n\ndeclare var WebGLSampler: {\n    prototype: WebGLSampler;\n    new(): WebGLSampler;\n};\n\n/** The WebGLShader is part of the WebGL API and can either be a vertex or a fragment shader. A WebGLProgram requires both types of shaders. */\ninterface WebGLShader {\n}\n\ndeclare var WebGLShader: {\n    prototype: WebGLShader;\n    new(): WebGLShader;\n};\n\n/** Part of the WebGL API and represents the information returned by calling the WebGLRenderingContext.getShaderPrecisionFormat() method. */\ninterface WebGLShaderPrecisionFormat {\n    readonly precision: GLint;\n    readonly rangeMax: GLint;\n    readonly rangeMin: GLint;\n}\n\ndeclare var WebGLShaderPrecisionFormat: {\n    prototype: WebGLShaderPrecisionFormat;\n    new(): WebGLShaderPrecisionFormat;\n};\n\ninterface WebGLSync {\n}\n\ndeclare var WebGLSync: {\n    prototype: WebGLSync;\n    new(): WebGLSync;\n};\n\n/** Part of the WebGL API and represents an opaque texture object providing storage and state for texturing operations. */\ninterface WebGLTexture {\n}\n\ndeclare var WebGLTexture: {\n    prototype: WebGLTexture;\n    new(): WebGLTexture;\n};\n\ninterface WebGLTransformFeedback {\n}\n\ndeclare var WebGLTransformFeedback: {\n    prototype: WebGLTransformFeedback;\n    new(): WebGLTransformFeedback;\n};\n\n/** Part of the WebGL API and represents the location of a uniform variable in a shader program. */\ninterface WebGLUniformLocation {\n}\n\ndeclare var WebGLUniformLocation: {\n    prototype: WebGLUniformLocation;\n    new(): WebGLUniformLocation;\n};\n\ninterface WebGLVertexArrayObject {\n}\n\ndeclare var WebGLVertexArrayObject: {\n    prototype: WebGLVertexArrayObject;\n    new(): WebGLVertexArrayObject;\n};\n\ninterface WebGLVertexArrayObjectOES {\n}\n\ninterface WebSocketEventMap {\n    "close": CloseEvent;\n    "error": Event;\n    "message": MessageEvent;\n    "open": Event;\n}\n\n/** Provides the API for creating and managing a WebSocket connection to a server, as well as for sending and receiving data on the connection. */\ninterface WebSocket extends EventTarget {\n    /**\n     * Returns a string that indicates how binary data from the WebSocket object is exposed to scripts:\n     *\n     * Can be set, to change how binary data is returned. The default is "blob".\n     */\n    binaryType: BinaryType;\n    /**\n     * Returns the number of bytes of application data (UTF-8 text and binary data) that have been queued using send() but not yet been transmitted to the network.\n     *\n     * If the WebSocket connection is closed, this attribute\'s value will only increase with each call to the send() method. (The number does not reset to zero once the connection closes.)\n     */\n    readonly bufferedAmount: number;\n    /**\n     * Returns the extensions selected by the server, if any.\n     */\n    readonly extensions: string;\n    onclose: ((this: WebSocket, ev: CloseEvent) => any) | null;\n    onerror: ((this: WebSocket, ev: Event) => any) | null;\n    onmessage: ((this: WebSocket, ev: MessageEvent) => any) | null;\n    onopen: ((this: WebSocket, ev: Event) => any) | null;\n    /**\n     * Returns the subprotocol selected by the server, if any. It can be used in conjunction with the array form of the constructor\'s second argument to perform subprotocol negotiation.\n     */\n    readonly protocol: string;\n    /**\n     * Returns the state of the WebSocket object\'s connection. It can have the values described below.\n     */\n    readonly readyState: number;\n    /**\n     * Returns the URL that was used to establish the WebSocket connection.\n     */\n    readonly url: string;\n    /**\n     * Closes the WebSocket connection, optionally using code as the the WebSocket connection close code and reason as the the WebSocket connection close reason.\n     */\n    close(code?: number, reason?: string): void;\n    /**\n     * Transmits data using the WebSocket connection. data can be a string, a Blob, an ArrayBuffer, or an ArrayBufferView.\n     */\n    send(data: string | ArrayBufferLike | Blob | ArrayBufferView): void;\n    readonly CLOSED: number;\n    readonly CLOSING: number;\n    readonly CONNECTING: number;\n    readonly OPEN: number;\n    addEventListener<K extends keyof WebSocketEventMap>(type: K, listener: (this: WebSocket, ev: WebSocketEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof WebSocketEventMap>(type: K, listener: (this: WebSocket, ev: WebSocketEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var WebSocket: {\n    prototype: WebSocket;\n    new(url: string | URL, protocols?: string | string[]): WebSocket;\n    readonly CLOSED: number;\n    readonly CLOSING: number;\n    readonly CONNECTING: number;\n    readonly OPEN: number;\n};\n\n/** This ServiceWorker API interface represents the scope of a service worker client that is a document in a browser context, controlled by an active worker. The service worker client independently selects and uses a service worker for its own loading and sub-resources. */\ninterface WindowClient extends Client {\n    readonly focused: boolean;\n    readonly visibilityState: VisibilityState;\n    focus(): Promise<WindowClient>;\n    navigate(url: string | URL): Promise<WindowClient | null>;\n}\n\ndeclare var WindowClient: {\n    prototype: WindowClient;\n    new(): WindowClient;\n};\n\ninterface WindowOrWorkerGlobalScope {\n    readonly caches: CacheStorage;\n    readonly crossOriginIsolated: boolean;\n    readonly crypto: Crypto;\n    readonly indexedDB: IDBFactory;\n    readonly isSecureContext: boolean;\n    readonly origin: string;\n    readonly performance: Performance;\n    atob(data: string): string;\n    btoa(data: string): string;\n    clearInterval(handle?: number): void;\n    clearTimeout(handle?: number): void;\n    createImageBitmap(image: ImageBitmapSource, options?: ImageBitmapOptions): Promise<ImageBitmap>;\n    createImageBitmap(image: ImageBitmapSource, sx: number, sy: number, sw: number, sh: number, options?: ImageBitmapOptions): Promise<ImageBitmap>;\n    fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\n    queueMicrotask(callback: VoidFunction): void;\n    setInterval(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\n    setTimeout(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\n}\n\ninterface WorkerEventMap extends AbstractWorkerEventMap {\n    "message": MessageEvent;\n    "messageerror": MessageEvent;\n}\n\n/** This Web Workers API interface represents a background task that can be easily created and can send messages back to its creator. Creating a worker is as simple as calling the Worker() constructor and specifying a script to be run in the worker thread. */\ninterface Worker extends EventTarget, AbstractWorker {\n    onmessage: ((this: Worker, ev: MessageEvent) => any) | null;\n    onmessageerror: ((this: Worker, ev: MessageEvent) => any) | null;\n    /**\n     * Clones message and transmits it to worker\'s global environment. transfer can be passed as a list of objects that are to be transferred rather than cloned.\n     */\n    postMessage(message: any, transfer: Transferable[]): void;\n    postMessage(message: any, options?: PostMessageOptions): void;\n    /**\n     * Aborts worker\'s associated global environment.\n     */\n    terminate(): void;\n    addEventListener<K extends keyof WorkerEventMap>(type: K, listener: (this: Worker, ev: WorkerEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof WorkerEventMap>(type: K, listener: (this: Worker, ev: WorkerEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var Worker: {\n    prototype: Worker;\n    new(scriptURL: string | URL, options?: WorkerOptions): Worker;\n};\n\ninterface WorkerGlobalScopeEventMap {\n    "error": ErrorEvent;\n    "languagechange": Event;\n    "offline": Event;\n    "online": Event;\n    "rejectionhandled": PromiseRejectionEvent;\n    "unhandledrejection": PromiseRejectionEvent;\n}\n\n/** This Web Workers API interface is an interface representing the scope of any worker. Workers have no browsing context; this scope contains the information usually conveyed by Window objects — in this case event handlers, the console or the associated WorkerNavigator object. Each WorkerGlobalScope has its own event loop. */\ninterface WorkerGlobalScope extends EventTarget, FontFaceSource, WindowOrWorkerGlobalScope {\n    /**\n     * Returns workerGlobal\'s WorkerLocation object.\n     */\n    readonly location: WorkerLocation;\n    /**\n     * Returns workerGlobal\'s WorkerNavigator object.\n     */\n    readonly navigator: WorkerNavigator;\n    onerror: ((this: WorkerGlobalScope, ev: ErrorEvent) => any) | null;\n    onlanguagechange: ((this: WorkerGlobalScope, ev: Event) => any) | null;\n    onoffline: ((this: WorkerGlobalScope, ev: Event) => any) | null;\n    ononline: ((this: WorkerGlobalScope, ev: Event) => any) | null;\n    onrejectionhandled: ((this: WorkerGlobalScope, ev: PromiseRejectionEvent) => any) | null;\n    onunhandledrejection: ((this: WorkerGlobalScope, ev: PromiseRejectionEvent) => any) | null;\n    /**\n     * Returns workerGlobal.\n     */\n    readonly self: WorkerGlobalScope & typeof globalThis;\n    /**\n     * Fetches each URL in urls, executes them one-by-one in the order they are passed, and then returns (or throws if something went amiss).\n     */\n    importScripts(...urls: (string | URL)[]): void;\n    addEventListener<K extends keyof WorkerGlobalScopeEventMap>(type: K, listener: (this: WorkerGlobalScope, ev: WorkerGlobalScopeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof WorkerGlobalScopeEventMap>(type: K, listener: (this: WorkerGlobalScope, ev: WorkerGlobalScopeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var WorkerGlobalScope: {\n    prototype: WorkerGlobalScope;\n    new(): WorkerGlobalScope;\n};\n\n/** The absolute location of the script executed by the Worker. Such an object is initialized for each worker and is available via the WorkerGlobalScope.location property obtained by calling self.location. */\ninterface WorkerLocation {\n    readonly hash: string;\n    readonly host: string;\n    readonly hostname: string;\n    readonly href: string;\n    toString(): string;\n    readonly origin: string;\n    readonly pathname: string;\n    readonly port: string;\n    readonly protocol: string;\n    readonly search: string;\n}\n\ndeclare var WorkerLocation: {\n    prototype: WorkerLocation;\n    new(): WorkerLocation;\n};\n\n/** A subset of the Navigator interface allowed to be accessed from a Worker. Such an object is initialized for each worker and is available via the WorkerGlobalScope.navigator property obtained by calling window.self.navigator. */\ninterface WorkerNavigator extends NavigatorConcurrentHardware, NavigatorID, NavigatorLanguage, NavigatorNetworkInformation, NavigatorOnLine, NavigatorStorage {\n    readonly mediaCapabilities: MediaCapabilities;\n}\n\ndeclare var WorkerNavigator: {\n    prototype: WorkerNavigator;\n    new(): WorkerNavigator;\n};\n\n/** This Streams API interface provides a standard abstraction for writing streaming data to a destination, known as a sink. This object comes with built-in backpressure and queuing. */\ninterface WritableStream<W = any> {\n    readonly locked: boolean;\n    abort(reason?: any): Promise<void>;\n    close(): Promise<void>;\n    getWriter(): WritableStreamDefaultWriter<W>;\n}\n\ndeclare var WritableStream: {\n    prototype: WritableStream;\n    new<W = any>(underlyingSink?: UnderlyingSink<W>, strategy?: QueuingStrategy<W>): WritableStream<W>;\n};\n\n/** This Streams API interface represents a controller allowing control of a WritableStream\'s state. When constructing a WritableStream, the underlying sink is given a corresponding WritableStreamDefaultController instance to manipulate. */\ninterface WritableStreamDefaultController {\n    error(e?: any): void;\n}\n\ndeclare var WritableStreamDefaultController: {\n    prototype: WritableStreamDefaultController;\n    new(): WritableStreamDefaultController;\n};\n\n/** This Streams API interface is the object returned by WritableStream.getWriter() and once created locks the < writer to the WritableStream ensuring that no other streams can write to the underlying sink. */\ninterface WritableStreamDefaultWriter<W = any> {\n    readonly closed: Promise<undefined>;\n    readonly desiredSize: number | null;\n    readonly ready: Promise<undefined>;\n    abort(reason?: any): Promise<void>;\n    close(): Promise<void>;\n    releaseLock(): void;\n    write(chunk?: W): Promise<void>;\n}\n\ndeclare var WritableStreamDefaultWriter: {\n    prototype: WritableStreamDefaultWriter;\n    new<W = any>(stream: WritableStream<W>): WritableStreamDefaultWriter<W>;\n};\n\ninterface XMLHttpRequestEventMap extends XMLHttpRequestEventTargetEventMap {\n    "readystatechange": Event;\n}\n\n/** Use XMLHttpRequest (XHR) objects to interact with servers. You can retrieve data from a URL without having to do a full page refresh. This enables a Web page to update just part of a page without disrupting what the user is doing. */\ninterface XMLHttpRequest extends XMLHttpRequestEventTarget {\n    onreadystatechange: ((this: XMLHttpRequest, ev: Event) => any) | null;\n    /**\n     * Returns client\'s state.\n     */\n    readonly readyState: number;\n    /**\n     * Returns the response body.\n     */\n    readonly response: any;\n    /**\n     * Returns response as text.\n     *\n     * Throws an "InvalidStateError" DOMException if responseType is not the empty string or "text".\n     */\n    readonly responseText: string;\n    /**\n     * Returns the response type.\n     *\n     * Can be set to change the response type. Values are: the empty string (default), "arraybuffer", "blob", "document", "json", and "text".\n     *\n     * When set: setting to "document" is ignored if current global object is not a Window object.\n     *\n     * When set: throws an "InvalidStateError" DOMException if state is loading or done.\n     *\n     * When set: throws an "InvalidAccessError" DOMException if the synchronous flag is set and current global object is a Window object.\n     */\n    responseType: XMLHttpRequestResponseType;\n    readonly responseURL: string;\n    readonly status: number;\n    readonly statusText: string;\n    /**\n     * Can be set to a time in milliseconds. When set to a non-zero value will cause fetching to terminate after the given time has passed. When the time has passed, the request has not yet completed, and this\'s synchronous flag is unset, a timeout event will then be dispatched, or a "TimeoutError" DOMException will be thrown otherwise (for the send() method).\n     *\n     * When set: throws an "InvalidAccessError" DOMException if the synchronous flag is set and current global object is a Window object.\n     */\n    timeout: number;\n    /**\n     * Returns the associated XMLHttpRequestUpload object. It can be used to gather transmission information when data is transferred to a server.\n     */\n    readonly upload: XMLHttpRequestUpload;\n    /**\n     * True when credentials are to be included in a cross-origin request. False when they are to be excluded in a cross-origin request and when cookies are to be ignored in its response. Initially false.\n     *\n     * When set: throws an "InvalidStateError" DOMException if state is not unsent or opened, or if the send() flag is set.\n     */\n    withCredentials: boolean;\n    /**\n     * Cancels any network activity.\n     */\n    abort(): void;\n    getAllResponseHeaders(): string;\n    getResponseHeader(name: string): string | null;\n    /**\n     * Sets the request method, request URL, and synchronous flag.\n     *\n     * Throws a "SyntaxError" DOMException if either method is not a valid method or url cannot be parsed.\n     *\n     * Throws a "SecurityError" DOMException if method is a case-insensitive match for `CONNECT`, `TRACE`, or `TRACK`.\n     *\n     * Throws an "InvalidAccessError" DOMException if async is false, current global object is a Window object, and the timeout attribute is not zero or the responseType attribute is not the empty string.\n     */\n    open(method: string, url: string | URL): void;\n    open(method: string, url: string | URL, async: boolean, username?: string | null, password?: string | null): void;\n    /**\n     * Acts as if the `Content-Type` header value for a response is mime. (It does not change the header.)\n     *\n     * Throws an "InvalidStateError" DOMException if state is loading or done.\n     */\n    overrideMimeType(mime: string): void;\n    /**\n     * Initiates the request. The body argument provides the request body, if any, and is ignored if the request method is GET or HEAD.\n     *\n     * Throws an "InvalidStateError" DOMException if either state is not opened or the send() flag is set.\n     */\n    send(body?: XMLHttpRequestBodyInit | null): void;\n    /**\n     * Combines a header in author request headers.\n     *\n     * Throws an "InvalidStateError" DOMException if either state is not opened or the send() flag is set.\n     *\n     * Throws a "SyntaxError" DOMException if name is not a header name or if value is not a header value.\n     */\n    setRequestHeader(name: string, value: string): void;\n    readonly DONE: number;\n    readonly HEADERS_RECEIVED: number;\n    readonly LOADING: number;\n    readonly OPENED: number;\n    readonly UNSENT: number;\n    addEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var XMLHttpRequest: {\n    prototype: XMLHttpRequest;\n    new(): XMLHttpRequest;\n    readonly DONE: number;\n    readonly HEADERS_RECEIVED: number;\n    readonly LOADING: number;\n    readonly OPENED: number;\n    readonly UNSENT: number;\n};\n\ninterface XMLHttpRequestEventTargetEventMap {\n    "abort": ProgressEvent<XMLHttpRequestEventTarget>;\n    "error": ProgressEvent<XMLHttpRequestEventTarget>;\n    "load": ProgressEvent<XMLHttpRequestEventTarget>;\n    "loadend": ProgressEvent<XMLHttpRequestEventTarget>;\n    "loadstart": ProgressEvent<XMLHttpRequestEventTarget>;\n    "progress": ProgressEvent<XMLHttpRequestEventTarget>;\n    "timeout": ProgressEvent<XMLHttpRequestEventTarget>;\n}\n\ninterface XMLHttpRequestEventTarget extends EventTarget {\n    onabort: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onerror: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onload: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onloadend: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onloadstart: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    onprogress: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    ontimeout: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;\n    addEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestEventTarget, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestEventTarget, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var XMLHttpRequestEventTarget: {\n    prototype: XMLHttpRequestEventTarget;\n    new(): XMLHttpRequestEventTarget;\n};\n\ninterface XMLHttpRequestUpload extends XMLHttpRequestEventTarget {\n    addEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestUpload, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\n    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\n    removeEventListener<K extends keyof XMLHttpRequestEventTargetEventMap>(type: K, listener: (this: XMLHttpRequestUpload, ev: XMLHttpRequestEventTargetEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\n    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\n}\n\ndeclare var XMLHttpRequestUpload: {\n    prototype: XMLHttpRequestUpload;\n    new(): XMLHttpRequestUpload;\n};\n\ninterface Console {\n    assert(condition?: boolean, ...data: any[]): void;\n    clear(): void;\n    count(label?: string): void;\n    countReset(label?: string): void;\n    debug(...data: any[]): void;\n    dir(item?: any, options?: any): void;\n    dirxml(...data: any[]): void;\n    error(...data: any[]): void;\n    group(...data: any[]): void;\n    groupCollapsed(...data: any[]): void;\n    groupEnd(): void;\n    info(...data: any[]): void;\n    log(...data: any[]): void;\n    table(tabularData?: any, properties?: string[]): void;\n    time(label?: string): void;\n    timeEnd(label?: string): void;\n    timeLog(label?: string, ...data: any[]): void;\n    timeStamp(label?: string): void;\n    trace(...data: any[]): void;\n    warn(...data: any[]): void;\n}\n\ndeclare var console: Console;\n\ndeclare namespace WebAssembly {\n    interface CompileError extends Error {\n    }\n\n    var CompileError: {\n        prototype: CompileError;\n        new(): CompileError;\n    };\n\n    interface Global {\n        value: any;\n        valueOf(): any;\n    }\n\n    var Global: {\n        prototype: Global;\n        new(descriptor: GlobalDescriptor, v?: any): Global;\n    };\n\n    interface Instance {\n        readonly exports: Exports;\n    }\n\n    var Instance: {\n        prototype: Instance;\n        new(module: Module, importObject?: Imports): Instance;\n    };\n\n    interface LinkError extends Error {\n    }\n\n    var LinkError: {\n        prototype: LinkError;\n        new(): LinkError;\n    };\n\n    interface Memory {\n        readonly buffer: ArrayBuffer;\n        grow(delta: number): number;\n    }\n\n    var Memory: {\n        prototype: Memory;\n        new(descriptor: MemoryDescriptor): Memory;\n    };\n\n    interface Module {\n    }\n\n    var Module: {\n        prototype: Module;\n        new(bytes: BufferSource): Module;\n        customSections(moduleObject: Module, sectionName: string): ArrayBuffer[];\n        exports(moduleObject: Module): ModuleExportDescriptor[];\n        imports(moduleObject: Module): ModuleImportDescriptor[];\n    };\n\n    interface RuntimeError extends Error {\n    }\n\n    var RuntimeError: {\n        prototype: RuntimeError;\n        new(): RuntimeError;\n    };\n\n    interface Table {\n        readonly length: number;\n        get(index: number): any;\n        grow(delta: number, value?: any): number;\n        set(index: number, value?: any): void;\n    }\n\n    var Table: {\n        prototype: Table;\n        new(descriptor: TableDescriptor, value?: any): Table;\n    };\n\n    interface GlobalDescriptor {\n        mutable?: boolean;\n        value: ValueType;\n    }\n\n    interface MemoryDescriptor {\n        initial: number;\n        maximum?: number;\n        shared?: boolean;\n    }\n\n    interface ModuleExportDescriptor {\n        kind: ImportExportKind;\n        name: string;\n    }\n\n    interface ModuleImportDescriptor {\n        kind: ImportExportKind;\n        module: string;\n        name: string;\n    }\n\n    interface TableDescriptor {\n        element: TableKind;\n        initial: number;\n        maximum?: number;\n    }\n\n    interface WebAssemblyInstantiatedSource {\n        instance: Instance;\n        module: Module;\n    }\n\n    type ImportExportKind = "function" | "global" | "memory" | "table";\n    type TableKind = "anyfunc" | "externref";\n    type ValueType = "anyfunc" | "externref" | "f32" | "f64" | "i32" | "i64";\n    type ExportValue = Function | Global | Memory | Table;\n    type Exports = Record<string, ExportValue>;\n    type ImportValue = ExportValue | number;\n    type Imports = Record<string, ModuleImports>;\n    type ModuleImports = Record<string, ImportValue>;\n    function compile(bytes: BufferSource): Promise<Module>;\n    function compileStreaming(source: Response | PromiseLike<Response>): Promise<Module>;\n    function instantiate(bytes: BufferSource, importObject?: Imports): Promise<WebAssemblyInstantiatedSource>;\n    function instantiate(moduleObject: Module, importObject?: Imports): Promise<Instance>;\n    function instantiateStreaming(source: Response | PromiseLike<Response>, importObject?: Imports): Promise<WebAssemblyInstantiatedSource>;\n    function validate(bytes: BufferSource): boolean;\n}\n\ninterface FrameRequestCallback {\n    (time: DOMHighResTimeStamp): void;\n}\n\ninterface OnErrorEventHandlerNonNull {\n    (event: Event | string, source?: string, lineno?: number, colno?: number, error?: Error): any;\n}\n\ninterface PerformanceObserverCallback {\n    (entries: PerformanceObserverEntryList, observer: PerformanceObserver): void;\n}\n\ninterface QueuingStrategySize<T = any> {\n    (chunk?: T): number;\n}\n\ninterface TransformerFlushCallback<O> {\n    (controller: TransformStreamDefaultController<O>): void | PromiseLike<void>;\n}\n\ninterface TransformerStartCallback<O> {\n    (controller: TransformStreamDefaultController<O>): any;\n}\n\ninterface TransformerTransformCallback<I, O> {\n    (chunk: I, controller: TransformStreamDefaultController<O>): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSinkAbortCallback {\n    (reason?: any): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSinkCloseCallback {\n    (): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSinkStartCallback {\n    (controller: WritableStreamDefaultController): any;\n}\n\ninterface UnderlyingSinkWriteCallback<W> {\n    (chunk: W, controller: WritableStreamDefaultController): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSourceCancelCallback {\n    (reason?: any): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSourcePullCallback<R> {\n    (controller: ReadableStreamController<R>): void | PromiseLike<void>;\n}\n\ninterface UnderlyingSourceStartCallback<R> {\n    (controller: ReadableStreamController<R>): any;\n}\n\ninterface VoidFunction {\n    (): void;\n}\n\n/**\n * Returns dedicatedWorkerGlobal\'s name, i.e. the value given to the Worker constructor. Primarily useful for debugging.\n */\ndeclare var name: string;\ndeclare var onmessage: ((this: DedicatedWorkerGlobalScope, ev: MessageEvent) => any) | null;\ndeclare var onmessageerror: ((this: DedicatedWorkerGlobalScope, ev: MessageEvent) => any) | null;\n/**\n * Aborts dedicatedWorkerGlobal.\n */\ndeclare function close(): void;\n/**\n * Clones message and transmits it to the Worker object associated with dedicatedWorkerGlobal. transfer can be passed as a list of objects that are to be transferred rather than cloned.\n */\ndeclare function postMessage(message: any, transfer: Transferable[]): void;\ndeclare function postMessage(message: any, options?: PostMessageOptions): void;\n/**\n * Dispatches a synthetic event event to target and returns true if either event\'s cancelable attribute value is false or its preventDefault() method was not invoked, and false otherwise.\n */\ndeclare function dispatchEvent(event: Event): boolean;\n/**\n * Returns workerGlobal\'s WorkerLocation object.\n */\ndeclare var location: WorkerLocation;\n/**\n * Returns workerGlobal\'s WorkerNavigator object.\n */\ndeclare var navigator: WorkerNavigator;\ndeclare var onerror: ((this: DedicatedWorkerGlobalScope, ev: ErrorEvent) => any) | null;\ndeclare var onlanguagechange: ((this: DedicatedWorkerGlobalScope, ev: Event) => any) | null;\ndeclare var onoffline: ((this: DedicatedWorkerGlobalScope, ev: Event) => any) | null;\ndeclare var ononline: ((this: DedicatedWorkerGlobalScope, ev: Event) => any) | null;\ndeclare var onrejectionhandled: ((this: DedicatedWorkerGlobalScope, ev: PromiseRejectionEvent) => any) | null;\ndeclare var onunhandledrejection: ((this: DedicatedWorkerGlobalScope, ev: PromiseRejectionEvent) => any) | null;\n/**\n * Returns workerGlobal.\n */\ndeclare var self: WorkerGlobalScope & typeof globalThis;\n/**\n * Fetches each URL in urls, executes them one-by-one in the order they are passed, and then returns (or throws if something went amiss).\n */\ndeclare function importScripts(...urls: (string | URL)[]): void;\n/**\n * Dispatches a synthetic event event to target and returns true if either event\'s cancelable attribute value is false or its preventDefault() method was not invoked, and false otherwise.\n */\ndeclare function dispatchEvent(event: Event): boolean;\ndeclare var fonts: FontFaceSet;\ndeclare var caches: CacheStorage;\ndeclare var crossOriginIsolated: boolean;\ndeclare var crypto: Crypto;\ndeclare var indexedDB: IDBFactory;\ndeclare var isSecureContext: boolean;\ndeclare var origin: string;\ndeclare var performance: Performance;\ndeclare function atob(data: string): string;\ndeclare function btoa(data: string): string;\ndeclare function clearInterval(handle?: number): void;\ndeclare function clearTimeout(handle?: number): void;\ndeclare function createImageBitmap(image: ImageBitmapSource, options?: ImageBitmapOptions): Promise<ImageBitmap>;\ndeclare function createImageBitmap(image: ImageBitmapSource, sx: number, sy: number, sw: number, sh: number, options?: ImageBitmapOptions): Promise<ImageBitmap>;\ndeclare function fetch(input: RequestInfo, init?: RequestInit): Promise<Response>;\ndeclare function queueMicrotask(callback: VoidFunction): void;\ndeclare function setInterval(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\ndeclare function setTimeout(handler: TimerHandler, timeout?: number, ...arguments: any[]): number;\ndeclare function cancelAnimationFrame(handle: number): void;\ndeclare function requestAnimationFrame(callback: FrameRequestCallback): number;\ndeclare function addEventListener<K extends keyof DedicatedWorkerGlobalScopeEventMap>(type: K, listener: (this: DedicatedWorkerGlobalScope, ev: DedicatedWorkerGlobalScopeEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;\ndeclare function addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;\ndeclare function removeEventListener<K extends keyof DedicatedWorkerGlobalScopeEventMap>(type: K, listener: (this: DedicatedWorkerGlobalScope, ev: DedicatedWorkerGlobalScopeEventMap[K]) => any, options?: boolean | EventListenerOptions): void;\ndeclare function removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;\ntype AlgorithmIdentifier = Algorithm | string;\ntype BigInteger = Uint8Array;\ntype BinaryData = ArrayBuffer | ArrayBufferView;\ntype BlobPart = BufferSource | Blob | string;\ntype BodyInit = ReadableStream | XMLHttpRequestBodyInit;\ntype BufferSource = ArrayBufferView | ArrayBuffer;\ntype CanvasImageSource = ImageBitmap | OffscreenCanvas;\ntype DOMHighResTimeStamp = number;\ntype DOMTimeStamp = number;\ntype EventListenerOrEventListenerObject = EventListener | EventListenerObject;\ntype Float32List = Float32Array | GLfloat[];\ntype FormDataEntryValue = File | string;\ntype GLbitfield = number;\ntype GLboolean = boolean;\ntype GLclampf = number;\ntype GLenum = number;\ntype GLfloat = number;\ntype GLint = number;\ntype GLint64 = number;\ntype GLintptr = number;\ntype GLsizei = number;\ntype GLsizeiptr = number;\ntype GLuint = number;\ntype GLuint64 = number;\ntype HashAlgorithmIdentifier = AlgorithmIdentifier;\ntype HeadersInit = string[][] | Record<string, string> | Headers;\ntype IDBValidKey = number | string | Date | BufferSource | IDBValidKey[];\ntype ImageBitmapSource = CanvasImageSource | Blob | ImageData;\ntype Int32List = Int32Array | GLint[];\ntype MessageEventSource = MessagePort | ServiceWorker;\ntype NamedCurve = string;\ntype OnErrorEventHandler = OnErrorEventHandlerNonNull | null;\ntype PerformanceEntryList = PerformanceEntry[];\ntype PushMessageDataInit = BufferSource | string;\ntype ReadableStreamController<T> = ReadableStreamDefaultController<T>;\ntype ReadableStreamDefaultReadResult<T> = ReadableStreamDefaultReadValueResult<T> | ReadableStreamDefaultReadDoneResult;\ntype ReadableStreamReader<T> = ReadableStreamDefaultReader<T>;\ntype RequestInfo = Request | string;\ntype TexImageSource = ImageBitmap | ImageData | OffscreenCanvas;\ntype TimerHandler = string | Function;\ntype Transferable = ArrayBuffer | MessagePort | ImageBitmap;\ntype Uint32List = Uint32Array | GLuint[];\ntype VibratePattern = number | number[];\ntype XMLHttpRequestBodyInit = Blob | BufferSource | FormData | URLSearchParams | string;\ntype BinaryType = "arraybuffer" | "blob";\ntype ClientTypes = "all" | "sharedworker" | "window" | "worker";\ntype ColorGamut = "p3" | "rec2020" | "srgb";\ntype ColorSpaceConversion = "default" | "none";\ntype ConnectionType = "bluetooth" | "cellular" | "ethernet" | "mixed" | "none" | "other" | "unknown" | "wifi";\ntype EndingType = "native" | "transparent";\ntype FontFaceLoadStatus = "error" | "loaded" | "loading" | "unloaded";\ntype FontFaceSetLoadStatus = "loaded" | "loading";\ntype FrameType = "auxiliary" | "nested" | "none" | "top-level";\ntype HdrMetadataType = "smpteSt2086" | "smpteSt2094-10" | "smpteSt2094-40";\ntype IDBCursorDirection = "next" | "nextunique" | "prev" | "prevunique";\ntype IDBRequestReadyState = "done" | "pending";\ntype IDBTransactionMode = "readonly" | "readwrite" | "versionchange";\ntype ImageOrientation = "flipY" | "none";\ntype KeyFormat = "jwk" | "pkcs8" | "raw" | "spki";\ntype KeyType = "private" | "public" | "secret";\ntype KeyUsage = "decrypt" | "deriveBits" | "deriveKey" | "encrypt" | "sign" | "unwrapKey" | "verify" | "wrapKey";\ntype MediaDecodingType = "file" | "media-source" | "webrtc";\ntype MediaEncodingType = "record" | "webrtc";\ntype NotificationDirection = "auto" | "ltr" | "rtl";\ntype NotificationPermission = "default" | "denied" | "granted";\ntype PermissionName = "gamepad" | "geolocation" | "notifications" | "persistent-storage" | "push" | "screen-wake-lock";\ntype PermissionState = "denied" | "granted" | "prompt";\ntype PredefinedColorSpace = "display-p3" | "srgb";\ntype PremultiplyAlpha = "default" | "none" | "premultiply";\ntype PushEncryptionKeyName = "auth" | "p256dh";\ntype PushPermissionState = "denied" | "granted" | "prompt";\ntype ReferrerPolicy = "" | "no-referrer" | "no-referrer-when-downgrade" | "origin" | "origin-when-cross-origin" | "same-origin" | "strict-origin" | "strict-origin-when-cross-origin" | "unsafe-url";\ntype RequestCache = "default" | "force-cache" | "no-cache" | "no-store" | "only-if-cached" | "reload";\ntype RequestCredentials = "include" | "omit" | "same-origin";\ntype RequestDestination = "" | "audio" | "audioworklet" | "document" | "embed" | "font" | "frame" | "iframe" | "image" | "manifest" | "object" | "paintworklet" | "report" | "script" | "sharedworker" | "style" | "track" | "video" | "worker" | "xslt";\ntype RequestMode = "cors" | "navigate" | "no-cors" | "same-origin";\ntype RequestRedirect = "error" | "follow" | "manual";\ntype ResizeQuality = "high" | "low" | "medium" | "pixelated";\ntype ResponseType = "basic" | "cors" | "default" | "error" | "opaque" | "opaqueredirect";\ntype SecurityPolicyViolationEventDisposition = "enforce" | "report";\ntype ServiceWorkerState = "activated" | "activating" | "installed" | "installing" | "parsed" | "redundant";\ntype ServiceWorkerUpdateViaCache = "all" | "imports" | "none";\ntype TransferFunction = "hlg" | "pq" | "srgb";\ntype VisibilityState = "hidden" | "visible";\ntype WebGLPowerPreference = "default" | "high-performance" | "low-power";\ntype WorkerType = "classic" | "module";\ntype XMLHttpRequestResponseType = "" | "arraybuffer" | "blob" | "document" | "json" | "text";\n',"lib.webworker.importscripts.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n\n/////////////////////////////\n/// WorkerGlobalScope APIs\n/////////////////////////////\n// These are only available in a Web Worker\ndeclare function importScripts(...urls: string[]): void;\n',"lib.webworker.iterable.d.ts":'/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\n\n\n\n/// <reference no-default-lib="true"/>\n\n\n/////////////////////////////\n/// Worker Iterable APIs\n/////////////////////////////\n\ninterface Cache {\n    addAll(requests: Iterable<RequestInfo>): Promise<void>;\n}\n\ninterface DOMStringList {\n    [Symbol.iterator](): IterableIterator<string>;\n}\n\ninterface FileList {\n    [Symbol.iterator](): IterableIterator<File>;\n}\n\ninterface FontFaceSet extends Set<FontFace> {\n}\n\ninterface FormData {\n    [Symbol.iterator](): IterableIterator<[string, FormDataEntryValue]>;\n    /**\n     * Returns an array of key, value pairs for every entry in the list.\n     */\n    entries(): IterableIterator<[string, FormDataEntryValue]>;\n    /**\n     * Returns a list of keys in the list.\n     */\n    keys(): IterableIterator<string>;\n    /**\n     * Returns a list of values in the list.\n     */\n    values(): IterableIterator<FormDataEntryValue>;\n}\n\ninterface Headers {\n    [Symbol.iterator](): IterableIterator<[string, string]>;\n    /**\n     * Returns an iterator allowing to go through all key/value pairs contained in this object.\n     */\n    entries(): IterableIterator<[string, string]>;\n    /**\n     * Returns an iterator allowing to go through all keys of the key/value pairs contained in this object.\n     */\n    keys(): IterableIterator<string>;\n    /**\n     * Returns an iterator allowing to go through all values of the key/value pairs contained in this object.\n     */\n    values(): IterableIterator<string>;\n}\n\ninterface IDBDatabase {\n    /**\n     * Returns a new transaction with the given mode ("readonly" or "readwrite") and scope which can be a single object store name or an array of names.\n     */\n    transaction(storeNames: string | Iterable<string>, mode?: IDBTransactionMode): IDBTransaction;\n}\n\ninterface IDBObjectStore {\n    /**\n     * Creates a new index in store with the given name, keyPath and options and returns a new IDBIndex. If the keyPath and options define constraints that cannot be satisfied with the data already in store the upgrade transaction will abort with a "ConstraintError" DOMException.\n     *\n     * Throws an "InvalidStateError" DOMException if not called within an upgrade transaction.\n     */\n    createIndex(name: string, keyPath: string | Iterable<string>, options?: IDBIndexParameters): IDBIndex;\n}\n\ninterface MessageEvent<T = any> {\n    /** @deprecated */\n    initMessageEvent(type: string, bubbles?: boolean, cancelable?: boolean, data?: any, origin?: string, lastEventId?: string, source?: MessageEventSource | null, ports?: Iterable<MessagePort>): void;\n}\n\ninterface ReadableStream<R = any> {\n    [Symbol.iterator](): IterableIterator<any>;\n    entries(): IterableIterator<[number, any]>;\n    keys(): IterableIterator<number>;\n    values(): IterableIterator<any>;\n}\n\ninterface SubtleCrypto {\n    deriveKey(algorithm: AlgorithmIdentifier | EcdhKeyDeriveParams | HkdfParams | Pbkdf2Params, baseKey: CryptoKey, derivedKeyType: AlgorithmIdentifier | AesDerivedKeyParams | HmacImportParams | HkdfParams | Pbkdf2Params, extractable: boolean, keyUsages: Iterable<KeyUsage>): Promise<CryptoKey>;\n    generateKey(algorithm: RsaHashedKeyGenParams | EcKeyGenParams, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKeyPair>;\n    generateKey(algorithm: AesKeyGenParams | HmacKeyGenParams | Pbkdf2Params, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    generateKey(algorithm: AlgorithmIdentifier, extractable: boolean, keyUsages: Iterable<KeyUsage>): Promise<CryptoKeyPair | CryptoKey>;\n    importKey(format: "jwk", keyData: JsonWebKey, algorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: KeyUsage[]): Promise<CryptoKey>;\n    importKey(format: Exclude<KeyFormat, "jwk">, keyData: BufferSource, algorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: Iterable<KeyUsage>): Promise<CryptoKey>;\n    unwrapKey(format: KeyFormat, wrappedKey: BufferSource, unwrappingKey: CryptoKey, unwrapAlgorithm: AlgorithmIdentifier | RsaOaepParams | AesCtrParams | AesCbcParams | AesGcmParams, unwrappedKeyAlgorithm: AlgorithmIdentifier | RsaHashedImportParams | EcKeyImportParams | HmacImportParams | AesKeyAlgorithm, extractable: boolean, keyUsages: Iterable<KeyUsage>): Promise<CryptoKey>;\n}\n\ninterface URLSearchParams {\n    [Symbol.iterator](): IterableIterator<[string, string]>;\n    /**\n     * Returns an array of key, value pairs for every entry in the search params.\n     */\n    entries(): IterableIterator<[string, string]>;\n    /**\n     * Returns a list of keys in the search params.\n     */\n    keys(): IterableIterator<string>;\n    /**\n     * Returns a list of values in the search params.\n     */\n    values(): IterableIterator<string>;\n}\n\ninterface WEBGL_draw_buffers {\n    drawBuffersWEBGL(buffers: Iterable<GLenum>): void;\n}\n\ninterface WebGL2RenderingContextBase {\n    clearBufferfv(buffer: GLenum, drawbuffer: GLint, values: Iterable<GLfloat>, srcOffset?: GLuint): void;\n    clearBufferiv(buffer: GLenum, drawbuffer: GLint, values: Iterable<GLint>, srcOffset?: GLuint): void;\n    clearBufferuiv(buffer: GLenum, drawbuffer: GLint, values: Iterable<GLuint>, srcOffset?: GLuint): void;\n    drawBuffers(buffers: Iterable<GLenum>): void;\n    getActiveUniforms(program: WebGLProgram, uniformIndices: Iterable<GLuint>, pname: GLenum): any;\n    getUniformIndices(program: WebGLProgram, uniformNames: Iterable<string>): Iterable<GLuint> | null;\n    invalidateFramebuffer(target: GLenum, attachments: Iterable<GLenum>): void;\n    invalidateSubFramebuffer(target: GLenum, attachments: Iterable<GLenum>, x: GLint, y: GLint, width: GLsizei, height: GLsizei): void;\n    transformFeedbackVaryings(program: WebGLProgram, varyings: Iterable<string>, bufferMode: GLenum): void;\n    uniform1uiv(location: WebGLUniformLocation | null, data: Iterable<GLuint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2uiv(location: WebGLUniformLocation | null, data: Iterable<GLuint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3uiv(location: WebGLUniformLocation | null, data: Iterable<GLuint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4uiv(location: WebGLUniformLocation | null, data: Iterable<GLuint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2x3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2x4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3x2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3x4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4x2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4x3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    vertexAttribI4iv(index: GLuint, values: Iterable<GLint>): void;\n    vertexAttribI4uiv(index: GLuint, values: Iterable<GLuint>): void;\n}\n\ninterface WebGL2RenderingContextOverloads {\n    uniform1fv(location: WebGLUniformLocation | null, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform1iv(location: WebGLUniformLocation | null, data: Iterable<GLint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2fv(location: WebGLUniformLocation | null, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform2iv(location: WebGLUniformLocation | null, data: Iterable<GLint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3fv(location: WebGLUniformLocation | null, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform3iv(location: WebGLUniformLocation | null, data: Iterable<GLint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4fv(location: WebGLUniformLocation | null, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniform4iv(location: WebGLUniformLocation | null, data: Iterable<GLint>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, data: Iterable<GLfloat>, srcOffset?: GLuint, srcLength?: GLuint): void;\n}\n\ninterface WebGLRenderingContextBase {\n    vertexAttrib1fv(index: GLuint, values: Iterable<GLfloat>): void;\n    vertexAttrib2fv(index: GLuint, values: Iterable<GLfloat>): void;\n    vertexAttrib3fv(index: GLuint, values: Iterable<GLfloat>): void;\n    vertexAttrib4fv(index: GLuint, values: Iterable<GLfloat>): void;\n}\n\ninterface WebGLRenderingContextOverloads {\n    uniform1fv(location: WebGLUniformLocation | null, v: Iterable<GLfloat>): void;\n    uniform1iv(location: WebGLUniformLocation | null, v: Iterable<GLint>): void;\n    uniform2fv(location: WebGLUniformLocation | null, v: Iterable<GLfloat>): void;\n    uniform2iv(location: WebGLUniformLocation | null, v: Iterable<GLint>): void;\n    uniform3fv(location: WebGLUniformLocation | null, v: Iterable<GLfloat>): void;\n    uniform3iv(location: WebGLUniformLocation | null, v: Iterable<GLint>): void;\n    uniform4fv(location: WebGLUniformLocation | null, v: Iterable<GLfloat>): void;\n    uniform4iv(location: WebGLUniformLocation | null, v: Iterable<GLint>): void;\n    uniformMatrix2fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Iterable<GLfloat>): void;\n    uniformMatrix3fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Iterable<GLfloat>): void;\n    uniformMatrix4fv(location: WebGLUniformLocation | null, transpose: GLboolean, value: Iterable<GLfloat>): void;\n}\n'},Jt=function(){return Jt=Object.assign||function(e){for(var n,t=1,r=arguments.length;t<r;t++)for(var i in n=arguments[t],n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i]);return e},Jt.apply(this,arguments)},qt=function(e,n,t,r){function i(e){return e instanceof t?e:new t(function(n){n(e)})}return new(t||(t=Promise))(function(t,a){function o(e){try{c(r.next(e))}catch(n){a(n)}}function s(e){try{c(r["throw"](e))}catch(n){a(n)}}function c(e){e.done?t(e.value):i(e.value).then(o,s)}c((r=r.apply(e,n||[])).next())})},Xt=function(e,n){var t,r,i,a,o={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return a={next:s(0),throw:s(1),return:s(2)},"function"===typeof Symbol&&(a[Symbol.iterator]=function(){return this}),a;function s(e){return function(n){return c([e,n])}}function c(a){if(t)throw new TypeError("Generator is already executing.");while(o)try{if(t=1,r&&(i=2&a[0]?r["return"]:a[0]?r["throw"]||((i=r["return"])&&i.call(r),0):r.next)&&!(i=i.call(r,a[1])).done)return i;switch(r=0,i&&(a=[2&a[0],i.value]),a[0]){case 0:case 1:i=a;break;case 4:return o.label++,{value:a[1],done:!1};case 5:o.label++,r=a[1],a=[0];continue;case 7:a=o.ops.pop(),o.trys.pop();continue;default:if(i=o.trys,!(i=i.length>0&&i[i.length-1])&&(6===a[0]||2===a[0])){o=0;continue}if(3===a[0]&&(!i||a[1]>i[0]&&a[1]<i[3])){o.label=a[1];break}if(6===a[0]&&o.label<i[1]){o.label=i[1],i=a;break}if(i&&o.label<i[2]){o.label=i[2],o.ops.push(a);break}i[2]&&o.ops.pop(),o.trys.pop();continue}a=n.call(e,o)}catch(s){a=[6,s],r=0}finally{t=i=0}if(5&a[0])throw a[1];return{value:a[0]?a[1]:void 0,done:!0}}};function Yt(e){return"string"===typeof e?!!/^file:\/\/\//.test(e)&&!!zt[e.substr(8)]:0===e.path.indexOf("/lib.")&&!!zt[e.path.slice(1)]}var Qt=function(){function e(e,n){this._extraLibs=Object.create(null),this._languageService=Wt["createLanguageService"](this),this._ctx=e,this._compilerOptions=n.compilerOptions,this._extraLibs=n.extraLibs,this._inlayHintsOptions=n.inlayHintsOptions}return e.prototype.getCompilationSettings=function(){return this._compilerOptions},e.prototype.getLanguageService=function(){return this._languageService},e.prototype.getExtraLibs=function(){return this._extraLibs},e.prototype.getScriptFileNames=function(){var e=this._ctx.getMirrorModels().map(function(e){return e.uri}),n=e.filter(function(e){return!Yt(e)}).map(function(e){return e.toString()});return n.concat(Object.keys(this._extraLibs))},e.prototype._getModel=function(e){for(var n=this._ctx.getMirrorModels(),t=0;t<n.length;t++)if(n[t].uri.toString()===e)return n[t];return null},e.prototype.getScriptVersion=function(e){var n=this._getModel(e);return n?n.version.toString():this.isDefaultLibFileName(e)?"1":e in this._extraLibs?String(this._extraLibs[e].version):""},e.prototype.getScriptText=function(e){return qt(this,void 0,void 0,function(){return Xt(this,function(n){return[2,this._getScriptText(e)]})})},e.prototype._getScriptText=function(e){var n,t=this._getModel(e),r="lib."+e+".d.ts";if(t)n=t.getValue();else if(e in zt)n=zt[e];else if(r in zt)n=zt[r];else{if(!(e in this._extraLibs))return;n=this._extraLibs[e].content}return n},e.prototype.getScriptSnapshot=function(e){var n=this._getScriptText(e);if(void 0!==n)return{getText:function(e,t){return n.substring(e,t)},getLength:function(){return n.length},getChangeRange:function(){}}},e.prototype.getScriptKind=function(e){var n=e.substr(e.lastIndexOf(".")+1);switch(n){case"ts":return Wt["ScriptKind"].TS;case"tsx":return Wt["ScriptKind"].TSX;case"js":return Wt["ScriptKind"].JS;case"jsx":return Wt["ScriptKind"].JSX;default:return this.getCompilationSettings().allowJs?Wt["ScriptKind"].JS:Wt["ScriptKind"].TS}},e.prototype.getCurrentDirectory=function(){return""},e.prototype.getDefaultLibFileName=function(e){switch(e.target){case 99:var n="lib.esnext.full.d.ts";if(n in zt||n in this._extraLibs)return n;case 7:case 6:case 5:case 4:case 3:case 2:default:var t="lib.es"+(2013+(e.target||99))+".full.d.ts";return t in zt||t in this._extraLibs?t:"lib.es6.d.ts";case 1:case 0:return"lib.d.ts"}},e.prototype.isDefaultLibFileName=function(e){return e===this.getDefaultLibFileName(this._compilerOptions)},e.prototype.getLibFiles=function(){return qt(this,void 0,void 0,function(){return Xt(this,function(e){return[2,zt]})})},e.clearFiles=function(e){for(var n=[],t=0,r=e;t<r.length;t++){var i=r[t],a=Jt({},i);if(a.file=a.file?{fileName:a.file.fileName}:void 0,i.relatedInformation){a.relatedInformation=[];for(var o=0,s=i.relatedInformation;o<s.length;o++){var c=s[o],l=Jt({},c);l.file=l.file?{fileName:l.file.fileName}:void 0,a.relatedInformation.push(l)}}n.push(a)}return n},e.prototype.getSyntacticDiagnostics=function(n){return qt(this,void 0,void 0,function(){var t;return Xt(this,function(r){return Yt(n)?[2,[]]:(t=this._languageService.getSyntacticDiagnostics(n),[2,e.clearFiles(t)])})})},e.prototype.getSemanticDiagnostics=function(n){return qt(this,void 0,void 0,function(){var t;return Xt(this,function(r){return Yt(n)?[2,[]]:(t=this._languageService.getSemanticDiagnostics(n),[2,e.clearFiles(t)])})})},e.prototype.getSuggestionDiagnostics=function(n){return qt(this,void 0,void 0,function(){var t;return Xt(this,function(r){return Yt(n)?[2,[]]:(t=this._languageService.getSuggestionDiagnostics(n),[2,e.clearFiles(t)])})})},e.prototype.getCompilerOptionsDiagnostics=function(n){return qt(this,void 0,void 0,function(){var t;return Xt(this,function(r){return Yt(n)?[2,[]]:(t=this._languageService.getCompilerOptionsDiagnostics(),[2,e.clearFiles(t)])})})},e.prototype.getCompletionsAtPosition=function(e,n){return qt(this,void 0,void 0,function(){return Xt(this,function(t){return Yt(e)?[2,void 0]:[2,this._languageService.getCompletionsAtPosition(e,n,void 0)]})})},e.prototype.getCompletionEntryDetails=function(e,n,t){return qt(this,void 0,void 0,function(){return Xt(this,function(r){return[2,this._languageService.getCompletionEntryDetails(e,n,t,void 0,void 0,void 0,void 0)]})})},e.prototype.getSignatureHelpItems=function(e,n,t){return qt(this,void 0,void 0,function(){return Xt(this,function(r){return Yt(e)?[2,void 0]:[2,this._languageService.getSignatureHelpItems(e,n,t)]})})},e.prototype.getQuickInfoAtPosition=function(e,n){return qt(this,void 0,void 0,function(){return Xt(this,function(t){return Yt(e)?[2,void 0]:[2,this._languageService.getQuickInfoAtPosition(e,n)]})})},e.prototype.getOccurrencesAtPosition=function(e,n){return qt(this,void 0,void 0,function(){return Xt(this,function(t){return Yt(e)?[2,void 0]:[2,this._languageService.getOccurrencesAtPosition(e,n)]})})},e.prototype.getDefinitionAtPosition=function(e,n){return qt(this,void 0,void 0,function(){return Xt(this,function(t){return Yt(e)?[2,void 0]:[2,this._languageService.getDefinitionAtPosition(e,n)]})})},e.prototype.getReferencesAtPosition=function(e,n){return qt(this,void 0,void 0,function(){return Xt(this,function(t){return Yt(e)?[2,void 0]:[2,this._languageService.getReferencesAtPosition(e,n)]})})},e.prototype.getNavigationBarItems=function(e){return qt(this,void 0,void 0,function(){return Xt(this,function(n){return Yt(e)?[2,[]]:[2,this._languageService.getNavigationBarItems(e)]})})},e.prototype.getFormattingEditsForDocument=function(e,n){return qt(this,void 0,void 0,function(){return Xt(this,function(t){return Yt(e)?[2,[]]:[2,this._languageService.getFormattingEditsForDocument(e,n)]})})},e.prototype.getFormattingEditsForRange=function(e,n,t,r){return qt(this,void 0,void 0,function(){return Xt(this,function(i){return Yt(e)?[2,[]]:[2,this._languageService.getFormattingEditsForRange(e,n,t,r)]})})},e.prototype.getFormattingEditsAfterKeystroke=function(e,n,t,r){return qt(this,void 0,void 0,function(){return Xt(this,function(i){return Yt(e)?[2,[]]:[2,this._languageService.getFormattingEditsAfterKeystroke(e,n,t,r)]})})},e.prototype.findRenameLocations=function(e,n,t,r,i){return qt(this,void 0,void 0,function(){return Xt(this,function(a){return Yt(e)?[2,void 0]:[2,this._languageService.findRenameLocations(e,n,t,r,i)]})})},e.prototype.getRenameInfo=function(e,n,t){return qt(this,void 0,void 0,function(){return Xt(this,function(r){return Yt(e)?[2,{canRename:!1,localizedErrorMessage:"Cannot rename in lib file"}]:[2,this._languageService.getRenameInfo(e,n,t)]})})},e.prototype.getEmitOutput=function(e){return qt(this,void 0,void 0,function(){return Xt(this,function(n){return Yt(e)?[2,{outputFiles:[],emitSkipped:!0}]:[2,this._languageService.getEmitOutput(e)]})})},e.prototype.getCodeFixesAtPosition=function(e,n,t,r,i){return qt(this,void 0,void 0,function(){var a;return Xt(this,function(o){if(Yt(e))return[2,[]];a={};try{return[2,this._languageService.getCodeFixesAtPosition(e,n,t,r,i,a)]}catch(s){return[2,[]]}return[2]})})},e.prototype.updateExtraLibs=function(e){return qt(this,void 0,void 0,function(){return Xt(this,function(n){return this._extraLibs=e,[2]})})},e.prototype.provideInlayHints=function(e,n,t){var r;return qt(this,void 0,void 0,function(){var i,a;return Xt(this,function(o){if(Yt(e))return[2,[]];i=null!==(r=this._inlayHintsOptions)&&void 0!==r?r:{},a={start:n,length:t-n};try{return[2,this._languageService.provideInlayHints(e,a,i)]}catch(s){return[2,[]]}return[2]})})},e}();self.onmessage=function(){Ht(function(e,n){return new Qt(e,n)})}},dd40:function(e,n){e.exports=function(e){if(!e.webpackPolyfill){var n=Object.create(e);n.children||(n.children=[]),Object.defineProperty(n,"loaded",{enumerable:!0,get:function(){return n.l}}),Object.defineProperty(n,"id",{enumerable:!0,get:function(){return n.i}}),Object.defineProperty(n,"exports",{enumerable:!0}),n.webpackPolyfill=1}return n}},f28c:function(e,n){var t,r,i=e.exports={};function a(){throw new Error("setTimeout has not been defined")}function o(){throw new Error("clearTimeout has not been defined")}function s(e){if(t===setTimeout)return setTimeout(e,0);if((t===a||!t)&&setTimeout)return t=setTimeout,setTimeout(e,0);try{return t(e,0)}catch(n){try{return t.call(null,e,0)}catch(n){return t.call(this,e,0)}}}function c(e){if(r===clearTimeout)return clearTimeout(e);if((r===o||!r)&&clearTimeout)return r=clearTimeout,clearTimeout(e);try{return r(e)}catch(n){try{return r.call(null,e)}catch(n){return r.call(this,e)}}}(function(){try{t="function"===typeof setTimeout?setTimeout:a}catch(e){t=a}try{r="function"===typeof clearTimeout?clearTimeout:o}catch(e){r=o}})();var l,u=[],d=!1,p=-1;function f(){d&&l&&(d=!1,l.length?u=l.concat(u):p=-1,u.length&&m())}function m(){if(!d){var e=s(f);d=!0;var n=u.length;while(n){l=u,u=[];while(++p<n)l&&l[p].run();p=-1,n=u.length}l=null,d=!1,c(e)}}function _(e,n){this.fun=e,this.array=n}function g(){}i.nextTick=function(e){var n=new Array(arguments.length-1);if(arguments.length>1)for(var t=1;t<arguments.length;t++)n[t-1]=arguments[t];u.push(new _(e,n)),1!==u.length||d||s(m)},_.prototype.run=function(){this.fun.apply(null,this.array)},i.title="browser",i.browser=!0,i.env={},i.argv=[],i.version="",i.versions={},i.on=g,i.addListener=g,i.once=g,i.off=g,i.removeListener=g,i.removeAllListeners=g,i.emit=g,i.prependListener=g,i.prependOnceListener=g,i.listeners=function(e){return[]},i.binding=function(e){throw new Error("process.binding is not supported")},i.cwd=function(){return"/"},i.chdir=function(e){throw new Error("process.chdir is not supported")},i.umask=function(){return 0}}});